/* vi:ts=4
 *-----------------------------------------------------------------
 *
 * Name: nfx-port.h
 *
 * RCS:  $Id: nfx-ports.h,v 1.6 2002/10/18 02:52:32 johnh Exp $
 *
 * Copyright (C) 2001, Agile Storage, Inc.
 *
 * Description: How to parse/create the port name strings
 *
 * Created by:	Greg Seibert
 *
 * Date Created: 11 Feb 2002
 *
 *-----------------------------------------------------------------
 */
#if !defined (__nfx_ports_h_)
#define __nfx_ports_h_

#include "nfx-defs.h"

typedef enum {
	bt_sp,				// Fibre Channel Storage Processor
	bt_fp,				// GbE File Processor
	bt_sc				// SSC
} boardType_t;

// If you don't have a slot # in your string, this is what you get

#define __SP_DEF_SLOT_	2
#define __FP_DEF_SLOT_	1
#define __SC_DEF_SLOT_	0

// These are the ANSI definitions of these library functions.
// These are now in sm-libc/libc.h but some SSC build targets
// cannot access the prototypes in libc.h so
// its necessary to keep these prototypes under !_LIBC_H

#if !defined(_STDLIB_H)			/* if we couldnt include exports/libc.h */
extern int sprintf(char *str, const char *format, ...);
extern int atoi(const char *nptr);
extern char *strcat(char *s, const char *append);
extern char *strncpy(char *dst, const char *src, unsigned int len);
extern int strncmp(const char *s1, const char *s2, unsigned int len);
#endif /* _STDLIB_H */

static __inline__ int 
formPortName (const char *nodeName, boardType_t board, int slotNum, 
              int portNum, char *name)
{
	char *str;

	if (!nodeName || !strlen (nodeName)) {
		nodeName = "local";
	}
	strcpy (name, nodeName);
	strcat (name, ":");

	switch (board) {
		case bt_sp:
			str = "sp";
			break;
		case bt_fp:
			str = "fp";
			break;
		case bt_sc:
			str = "sc";
			break;
		default:
			return 1;
	}

	strcat (name, str);
/*
	SSC portspecs don't have slot #s in them - 26 Feb
 */
	if (board != bt_sc) {
		if ((slotNum < NFX_SLOT_MIN) || (slotNum > NFX_SLOT_MAX)) {
			return 1;
		}

		sprintf (&name[strlen(name)], "%d.", slotNum);
	}

	if (portNum < 0) {
		return 1;
	}
	sprintf (&name[strlen(name)], "%d", portNum);

	return 0;
}

static __inline__ int 
getPortInfo (const char *name, char *nodeName, boardType_t *board, 
             int *slotNum, int *portNum)
{
	const char *str;

	for (str = name; *str && (*str != ':'); str++);

	if (*str && (str != name)) {
		strncpy (nodeName, name, str - name);
		nodeName[str - name] = '\0';
		str++;
	}
	else {
		str = name;
		strcpy (nodeName, "local");
	}

	if (!strncmp (str, "sp", 2)) {
		*board = bt_sp;
	}
	else if (!strncmp (str, "fp", 2)) {
		*board = bt_fp;
	}
	else if (!strncmp (str, "sc", 2)) {
		*board = bt_sc;
	}
	else {
		return 1;	// Bogus board type
	}
	str += 2;	// Past board name

	if (*str == '.') {
		switch (*board) {
			case bt_sp:
				*slotNum = __SP_DEF_SLOT_;
				break;
			case bt_fp:
				*slotNum = __FP_DEF_SLOT_;
				break;
			case bt_sc:
				*slotNum = __SC_DEF_SLOT_;
				break;
		}
		str++;
	}
/*
	SSC ports never have the '.' so we default it's slot #
 */
	else if (*board != bt_sc) {
		const char *dot;
		for (dot = str; *dot && (*dot != '.'); dot++);

		if (!(*dot)) {
			return 1;
		}
		*slotNum = atoi (str);
		str = dot + 1;
	}
	else {
		*slotNum = __SC_DEF_SLOT_;
	}

	if (!(*str)) {
		return 1;
	}

	if ((*slotNum < NFX_SLOT_MIN) || (*slotNum > NFX_SLOT_MAX)) {
		return 1;
	}

	*portNum = atoi (str);
	if (*portNum < 0) {
		return 1;
	}

	return 0;
}

#endif
