/*
 *-----------------------------------------------------------------
 *
 * Name:	nfx-types.h
 *
 * RCS:  $Id: nfx-types.h,v 1.25 2002/12/10 22:26:38 bobonfeline Exp $
 *
 * Copyright (C) 2001, Agile Storage Inc.
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	4/3/01
 *
 *-----------------------------------------------------------------
 */

#ifndef _NFX_TYPES_H
#define _NFX_TYPES_H

#ifndef _NFX_ERROR_H
#include "nfx-error.h"
#endif

#ifdef LANGUAGE_C

typedef long long			int64;
typedef unsigned long long	uint64;

NFX_ASSERT_STATIC(sizeof (uint64)   == 8);
NFX_ASSERT_STATIC(sizeof (int64)    == 8);

typedef int					int32;
typedef unsigned int		uint32;

NFX_ASSERT_STATIC(sizeof (int32)    == 4);
NFX_ASSERT_STATIC(sizeof (uint32)   == 4);

typedef short               int16;
typedef unsigned short      uint16;
typedef short				short16;
typedef unsigned short		ushort16;

NFX_ASSERT_STATIC(sizeof (int16)    == 2);
NFX_ASSERT_STATIC(sizeof (uint16)   == 2);
NFX_ASSERT_STATIC(sizeof (short16)  == 2);
NFX_ASSERT_STATIC(sizeof (ushort16) == 2);

typedef unsigned char		byte;
typedef char                int8;
typedef unsigned char       uint8;
typedef char				char8;
typedef unsigned char		uchar8;

NFX_ASSERT_STATIC(sizeof (byte)     == 1);
NFX_ASSERT_STATIC(sizeof (int8)     == 1);
NFX_ASSERT_STATIC(sizeof (uint8)    == 1);
NFX_ASSERT_STATIC(sizeof (char8)    == 1);
NFX_ASSERT_STATIC(sizeof (uchar8)   == 1);

typedef unsigned char		boolean;

typedef unsigned long long	nfx_cod_t;

NFX_ASSERT_STATIC(sizeof (nfx_cod_t) == 8);

/*-----------------------------------------------------------------------------
 * A common data structure for a cod description used in write req
 * and read resp
 */
typedef struct nfx_cod_entry_t
{
     nfx_cod_t           cod;      // Caller supplied user data
     uint32              offset;   // offset within the cod
     uint32              length;   // length of user data in the COD
}nfx_cod_entry_t;
#define NFX_MAX_COD_MSG		10     // Max number of cod's in a single
                                   // request/response message.


typedef struct enet_hdr {
	uchar8		DA[6];
	uchar8		SA[6];
	ushort16	type;
} __attribute__ ((packed)) enet_hdr_t;

typedef struct agile_hdr {
	ushort16	pad;
	uint32		dest_port;
	uint32		src_port;
#if defined(NETEEE_FRAGMENT)
	uint32		offset;			/* Offset of the fragment + more flag*/
	uint32		pad__;
#endif
} __attribute__ ((packed)) agile_hdr_t;

#if defined(NETEEE_FRAGMENT)
#define AGILEHDR_MF 0x80000000	/* If this bit is set in agile_hdr->offset,
								   more fragments follow */
#define AGILEHDR_ID_MASK 0x7ff00000 /* Sequence ID mask */
#define AGILEHDR_ID_SHIFT 24	/* Sequence ID shift */
#define AGILEHDR_ID_MAX (AGILEHDR_ID_MASK >> AGILEHDR_ID_SHIFT)
#endif


#ifdef SBM_REV2

typedef struct gmc_hdr {
	char8	nil[0];
} __attribute__ ((packed)) gmc_app_hdr_t;

#else

#ifdef USE_SAC_CHECKSUM
typedef struct gmc_hdr {
	uint32		control;		/* contains app_id for ACK msg */
	uint32		cod_len;		/* cod length, in bytes */
	union {
		nfx_cod_t	cod;		/* COD handle */
	} id;
	uint64		chksum;			/* using crc32_checksum() */
	uint32		tid;			/* transaction id, for EVM */
	uint32		resv;			/* make doubleword aligned in size  */
} __attribute__ ((packed)) gmc_app_hdr_t;
#else
typedef struct gmc_hdr {
	uint32		control;
	uint32		cod_len;
	union {
		nfx_cod_t	cod;
		uint32		tid;
	} id;
} __attribute__ ((packed)) gmc_app_hdr_t;
#endif

#endif

#if !defined(FS_STANDALONE)
/**
 * The high granularity real time clock.
 * Note that we assume that the time between which we read the RTC
 * is never longer than the time it takes to wrap.  Given that this
 * counter is incremented 400M times a second for an 800 MHz CPU,
 * this means we need only call some time function once every
 * 2^32 / 400M seconds = ~10 seconds.
 *
 * If the above cannot be guaranteed we will need an interrupt timer
 * that runs at least once every 10 seconds.
 */
#ifdef TIMER_USE_SB1250_TIMER
typedef union {
    uint64 val;
    struct {
        uint32 usec;
        uint32 ref_sec;
    } u;
} eeeRTC_t;
#else
typedef union {
    uint64 val;
    struct {
        uint32 low;
        uint32 high;
    } u;
} eeeRTC_t;
#endif

/** An integer type that is as long as a pointer.
  * This was in nfx-types.h but uintptr_t is defined
  * in very different ways and places depending on the build target.
  * This is the earliest place that it's really available. */

#if defined(NFX_MOD_SSC)	// Only valid for SSC compiles
#if defined(SSC_MGMT)
typedef unsigned long uintptr_t;
typedef long          intptr_t;
#else
#include <inttypes.h>		// For ANSI types
#endif
#elif defined(NFX_MOD_NFP)
typedef unsigned long uintptr_t;
typedef long          intptr_t;
#elif ((!defined(_SYS_TYPES_H_)) || defined(FREE_BSD))
typedef unsigned long uintptr_t;
typedef long          intptr_t;
#elif defined(__OpenBSD__) 
typedef unsigned long uintptr_t;
typedef long		  intptr_t;			
#endif
typedef uintptr_t     address_t;
#ifdef NFX_SPINLOCK_DEBUG
typedef struct spinlock {
	uint32		lock;
	ushort16	cpu_num;
	ushort16	lock_level;
	address_t	owner;
	int32		lock_start_time;
	uint64		accumulated_wait_time;
	uint32		num_got;
	uint32		num_waited_for;
	uint32		max_time_held;
	uint32		sl_magic;
	LIST_ENTRY(spinlock) sl_entry; 
} nfx_spinlock_t;
#else
typedef struct spinlock	{
	uint32 lock;
} nfx_spinlock_t;
#endif

#else /* !FS_STANDALONE */

/**
 * The simulated high granularity real time clock, we use usec here.
 */
typedef unsigned long long eeeRTC_t;

#include <pthread.h>
typedef struct nfx_spinlock {
    pthread_mutex_t mutex;
    uint32          magic;
} nfx_spinlock_t;

/** An integer type that is as long as a pointer. */
typedef unsigned long address_t;

#endif  /* !FS_STANDALONE */

NFX_ASSERT_STATIC(sizeof (address_t) == sizeof (void *));

/* make sure that the headers are a multiple of doubleword */
NFX_ASSERT_STATIC(((sizeof(enet_hdr_t) + sizeof(agile_hdr_t)) % 8) == 0);

#ifdef NFX_SMP
#define CPU_PRIVATE __attribute__ ((section ("cpu_private")))
#else
#define CPU_PRIVATE
#endif

#endif	/* LANGUAGE_C */

#endif	/* _NFX_TYPES_H */
