/*
 *-----------------------------------------------------------------
 *
 * Name:
 *
 * RCS:  $Id: mips.h,v 1.26 2002/10/25 21:23:17 janv Exp $
 *
 * Copyright (C) 2001, Agile Storage, Inc.
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	12/20/01
 *
 *-----------------------------------------------------------------
 */

#ifndef _MIPS_H
#define _MIPS_H

#ifdef LANGUAGE_ASSEMBLY
#define _ACAST32_
#else
#define _ACAST32_	(int)
#endif

/*------------------------------------------------------------------
 * Address Spaces
 *------------------------------------------------------------------*/
#define K0BASE      (_ACAST32_ 0x80000000)  /* kernel unmapped cached */
#define K0SIZE      0x20000000
#define K1BASE      (_ACAST32_ 0xa0000000)  /* kernel unmapped uncached */
#define K1SIZE      0x20000000
#define K2BASE      (_ACAST32_ 0xc0000000)  /* supervisor+ mapped */
#define K2SIZE      0x20000000
#define K3BASE      (_ACAST32_ 0xe0000000)  /* kernel mapped */
#define K3SIZE      0x20000000

/*------------------------------------------------------------------
 * Bit Field macros
 *------------------------------------------------------------------*/
/*
 * Make a mask for 1 bit at position 'n'
 */

#define _MM_MAKEMASK1(n) (1 << (n))

/*
 * Make a mask for 'v' bits at position 'n'
 */

#define _MM_MAKEMASK(v,n) (((1<<(v))-1) << (n))

/*
 * Make a value at 'v' at bit position 'n'
 */

#define _MM_MAKEVALUE(v,n) ((v) << (n))

/*
 * Retrieve a value from 'v' at bit position 'n' with 'm' mask bits
 */

#define _MM_GETVALUE(v,n,m) (((v) & (m)) >> (n))


/*------------------------------------------------------------------
 * Address Conversion macros
 *------------------------------------------------------------------*/
#define K_CALG_COH_EXCL1_NOL2	0
#define K_CALG_COH_SHRL1_NOL2	1
#define K_CALG_UNCACHED		    2
#define K_CALG_NONCOHERENT	    3
#define K_CALG_COH_EXCL		    4
#define K_CALG_COH_SHAREABLE	5
#define K_CALG_NOTUSED		    6
#define K_CALG_UNCACHED_ACCEL	7

#ifdef LANGUAGE_ASSEMBLY
#define PHYS_TO_K0(pa)  ((pa)|K0BASE)
#define PHYS_TO_K1(pa)  ((pa)|K1BASE)
#define K0_TO_PHYS(va)  ((va)&(K0SIZE-1))
#define K1_TO_PHYS(va)  ((va)&(K1SIZE-1))
#define K0_TO_K1(va)    ((va)|K1SIZE)
#define K1_TO_K0(va)    ((va)&~K1SIZE)
#define PA_TO_KVA0(pa)  PHYS_TO_K0(pa)
#define PA_TO_KVA1(pa)  PHYS_TO_K1(pa)
#define KVA_TO_PA(pa)   K1_TO_PHYS(pa)
#define KSEG0_BASE      K0BASE
#define KSEG1_BASE      K1BASE

#define PHYS_TO_XK1(p)              (_ACAST64_ (0xffffffffa0000000 | (p)))
#define XK1_TO_PHYS(p)              ((p) & (K1SIZE-1))
#define PHYS_TO_XKPHYS(cca,p) (_SB_MAKEMASK1(63) | (_SB_MAKE64(cca) << 59) | (p))
#define PHYS_TO_XKSEG_UNCACHED(p)   PHYS_TO_XKPHYS(K_CALG_UNCACHED,(p))
#define PHYS_TO_XKSEG_CACHED(p)     PHYS_TO_XKPHYS(K_CALG_COH_SHAREABLE,(p))
#define PHYS_TO_XKSEG_UNCACCEL(p)   PHYS_TO_XKPHYS(K_CALG_UNCACHED_ACCEL, (p))
#define XKPHYS_TO_PHYS(p)           ((p) & _SB_MAKEMASK(0,59))

#endif	/* LANGUAGE_ASSEMBLY */

#ifdef LANGUAGE_C
#define PHYS_TO_K0(pa)  ((pa)|K0BASE)
#define PHYS_TO_K1(pa)  ((pa)|K1BASE)
#define PA_TO_KVA0(pa)  PHYS_TO_K0(pa)
#define PA_TO_KVA1(pa)  PHYS_TO_K1(pa)
#define KVA_TO_PA(pa)   K1_TO_PHYS(pa)
#define K0_TO_PHYS(va)  ((unsigned long)(va)&(K0SIZE-1))
#define K1_TO_PHYS(va)  ((unsigned long)(va)&(K1SIZE-1))
#define K0_TO_K1(va)    ((unsigned long)(va)|K1SIZE)
#define K1_TO_K0(va)    ((unsigned long)(va)&~K1SIZE)
#define IS_K0SEG(x) ((x) >= K0BASE && (x) < K1BASE)
#define IS_K1SEG(x) ((x) >= K1BASE && (x) < K2BASE)
#define KVA0_TO_KVA1(v) (void *)K0_TO_K1(v)
#define KVA1_TO_KVA0(v) (void *)K1_TO_K0(v)
#define IS_KVA(v)       ((int)(v) < 0)
#define IS_KVA0(v)      ((((unsigned long)v) >> 29) == 0x4)
#define IS_KVA1(v)      ((((unsigned long)v) >> 29) == 0x5)
#define IS_KVA01(v)     ((((unsigned long)v) >> 30) == 0x2)

#define PHYS_TO_XK1(p)              (_ACAST64_ (0xffffffffa0000000 | (p)))
#define XK1_TO_PHYS(p)              ((p) & (K1SIZE-1))
#define PHYS_TO_XKPHYS(cca,p) (_SB_MAKEMASK1(63) | (_SB_MAKE64(cca) << 59) | (p))
#define PHYS_TO_XKSEG_UNCACHED(p)   PHYS_TO_XKPHYS(K_CALG_UNCACHED,(p))
#define PHYS_TO_XKSEG_CACHED(p)     PHYS_TO_XKPHYS(K_CALG_COH_SHAREABLE,(p))
#define PHYS_TO_XKSEG_UNCACCEL(p)   PHYS_TO_XKPHYS(K_CALG_UNCACHED_ACCEL, (p))
#define XKPHYS_TO_PHYS(p)           ((p) & _SB_MAKEMASK(0,59))

#endif

#ifndef LANAGUAGE_ASSEMBLY
#ifdef SB1250
#define mips_wbflush()  __asm__ __volatile__ ("sync" : : : "memory")
#endif
#endif

/*------------------------------------------------------------------
 * General Purpose Register Aliases
 *------------------------------------------------------------------*/
#ifdef LANGUAGE_ASSEMBLY
#define zero    $0
#define AT      $1      /* assembler temporaries */
#define v0      $2      /* value holders */
#define v1      $3
#define a0      $4      /* arguments */
#define a1      $5
#define a2      $6
#define a3      $7
#define t0      $8      /* temporaries */
#define t1      $9
#define t2      $10
#define t3      $11
#define t4      $12
#define t5      $13
#define t6      $14
#define t7      $15
#define s0      $16     /* saved registers */
#define s1      $17
#define s2      $18
#define s3      $19
#define s4      $20
#define s5      $21
#define s6      $22
#define s7      $23
#define t8      $24     /* temporaries */
#define t9      $25
#define k0      $26     /* kernel registers */
#define k1      $27
#define gp      $28     /* global pointer */
#define sp      $29     /* stack pointer */
#define s8      $30     /* saved register */
#define fp      $30     /* frame pointer (old usage) */
#define ra      $31     /* return address */
#endif	/* LANGUAGE_ASSEMBLY */

/*------------------------------------------------------------------
 * Coprocessor 0 Register Aliases
 *------------------------------------------------------------------*/
#ifdef LANGUAGE_ASSEMBLY
#ifdef R7000
#define C0_INX          $0      /* TLB Index */
#define C0_RAND         $1      /* TLB Random */
#define C0_TLBLO        $2      /* TLB EntryLo */
#define C0_TLBLO0       $2      /* TLB EntryLo0 */
#define C0_TLBLO1       $3      /* TLB EntryLo1 */
#define C0_CTEXT        $4      /* Context */
#define C0_PGMASK       $5      /* TLB PageMask */
#define C0_WIRED        $6      /* TLB Wired */
#define C0_INFO         $7
#define C0_BADADDR      $8      /* Bad Address */
#define C0_BADVADDR     $8      /* Bad Virtual Address */
#define C0_COUNT        $9      /* Count */
#define C0_TLBHI        $10     /* TLB EntryHi */
#define C0_COMPARE      $11     /* Compare */
#define C0_SR           $12     /* Processor Status */
#define C0_STATUS       $12     /* Processor Status */
#define C0_CAUSE        $13     /* Exception Cause */
#define C0_EPC          $14     /* Exception PC */
#define C0_PRID         $15     /* Processor Revision Indentifier */
#define C0_CONFIG       $16     /* Config */
#define C0_LLADDR       $17     /* LLAddr */
#define C0_WATCHLO      $18     /* WatchpointLo */
#define C0_WATCHHI      $19     /* WatchpointHi */
#define C0_XCTEXT       $20     /* XContext */
#define C0_XCONTEXT     $20     /* XContext */
#define C0_WATCHMASK    $24     /* RM7000 Watchmask */
#define C0_PERFCOUNT	$25
#define C0_ECC          $26     /* ECC */
#define C0_CACHEERR     $27     /* CacheErr */
#define C0_TAGLO        $28     /* TagLo */
#define C0_TAGHI        $29     /* TagHi */
#define C0_ERREPC  	      $30     /* ErrorEPC */
#define C0_ERRPC        $30     /* ErrorEPC */
#endif

#ifdef TX3927
#define C0_INX          $0      /* TLB Index */
#define C0_RAND         $1      /* TLB Random */
#define C0_TLBLO        $2      /* TLB EntryLo */
#define C0_CONFIG       $3      /* Config */
#define C0_CTEXT        $4      /* Context */
#define C0_PGMASK       $5      /* TLB PageMask */
#define C0_WIRED        $6      /* TLB Wired */
#define C0_CACHE        $7      /* cache lock function */
#define C0_BADADDR      $8      /* Bad Address */
#define C0_BADVADDR     $8      /* Bad Virtual Address */
#define C0_TLBHI        $10     /* TLB EntryHi */
#define C0_SR           $12     /* Processor Status */
#define C0_STATUS       $12     /* Processor Status */
#define C0_CAUSE        $13     /* Exception Cause */
#define C0_EPC          $14     /* Exception PC */
#define C0_PRID         $15     /* Processor Revision Indentifier */
#define C0_DEBUG        $16     /* debug exception control */
#define C0_DEPC         $17     /* PC for debug exception */
#define C0_TAGLO        $20     /* cache tag register */
#endif

#ifdef SB1250
#define C0_INX      $0      /* CP0: TLB Index */
#define C0_RAND     $1      /* CP0: TLB Random */
#define C0_TLBLO0   $2      /* CP0: TLB EntryLo0 */
#define C0_TLBLO    C0_TLBLO0   /* CP0: TLB EntryLo0 */
#define C0_TLBLO1   $3      /* CP0: TLB EntryLo1 */
#define C0_CTEXT    $4      /* CP0: Context */
#define C0_PGMASK   $5      /* CP0: TLB PageMask */
#define C0_WIRED    $6      /* CP0: TLB Wired */
#define C0_BADVADDR $8      /* CP0: Bad Virtual Address */
#define C0_COUNT    $9      /* CP0: Count */
#define C0_TLBHI    $10     /* CP0: TLB EntryHi */
#define C0_COMPARE  $11     /* CP0: Compare */
#define C0_SR       $12     /* CP0: Processor Status */
#define C0_STATUS   C0_SR       /* CP0: Processor Status */
#define C0_CAUSE    $13     /* CP0: Exception Cause */
#define C0_EPC      $14     /* CP0: Exception PC */
#define C0_PRID     $15     /* CP0: Processor Revision Indentifier */
#define C0_CONFIG   $16     /* CP0: Config */
#define C0_LLADDR   $17     /* CP0: LLAddr */
#define C0_WATCHLO  $18     /* CP0: WatchpointLo */
#define C0_WATCHHI  $19     /* CP0: WatchpointHi */
#define C0_XCTEXT   $20     /* CP0: XContext */
#define C0_ECC      $26     /* CP0: ECC */
#define C0_CACHEERR $27     /* CP0: CacheErr */
#define C0_TAGLO    $28     /* CP0: TagLo */
#define C0_TAGHI    $29     /* CP0: TagHi */
#define C0_ERREPC   $30     /* CP0: ErrorEPC */
#define C0_ERRPC   $30     /* CP0: ErrorEPC */
#endif

#ifdef R7000
#define C0C_IPLLO       $18
#define C0C_IPLHI       $19
#define C0C_ICR         $20
#define C0C_DERRADDR1   $26
#define C0C_DERRADDR2   $27
#endif
#endif	/* LANGUAGE_ASSEMBLY */

#ifdef LANGUAGE_C
#ifdef R7000
#define C0_INX          0      /* TLB Index */
#define C0_RAND         1      /* TLB Random */
#define C0_TLBLO        2      /* TLB EntryLo */
#define C0_TLBLO0       2      /* TLB EntryLo0 */
#define C0_TLBLO1       3      /* TLB EntryLo1 */
#define C0_CTEXT        4      /* Context */
#define C0_PGMASK       5      /* TLB PageMask */
#define C0_WIRED        6      /* TLB Wired */
#define C0_INFO         7
#define C0_BADADDR      8      /* Bad Address */
#define C0_BADVADDR     8      /* Bad Virtual Address */
#define C0_COUNT        9      /* Count */
#define C0_TLBHI        10     /* TLB EntryHi */
#define C0_COMPARE      11     /* Compare */
#define C0_SR           12     /* Processor Status */
#define C0_STATUS       12     /* Processor Status */
#define C0_CAUSE        13     /* Exception Cause */
#define C0_EPC          14     /* Exception PC */
#define C0_PRID         15     /* Processor Revision Indentifier */
#define C0_CONFIG       16     /* Config */
#define C0_LLADDR       17     /* LLAddr */
#define C0_WATCHLO      18     /* WatchpointLo */
#define C0_WATCHHI      19     /* WatchpointHi */
#define C0_XCTEXT       20     /* XContext */
#define C0_XCONTEXT     20     /* XContext */
#define C0_WATCHMASK    24     /* RM7000 Watchmask */
#define C0_PERFCOUNT	25
#define C0_ECC          26     /* ECC */
#define C0_CACHEERR     27     /* CacheErr */
#define C0_TAGLO        28     /* TagLo */
#define C0_TAGHI        29     /* TagHi */
#define C0_ERREPC       30     /* ErrorEPC */
#define C0_ERRPC       30     /* ErrorEPC */
#endif

#ifdef TX3927
#define C0_INX          0      /* TLB Index */
#define C0_RAND         1      /* TLB Random */
#define C0_TLBLO        2      /* TLB EntryLo */
#define C0_CONFIG       3      /* Config */
#define C0_CTEXT        4      /* Context */
#define C0_PGMASK       5      /* TLB PageMask */
#define C0_WIRED        6      /* TLB Wired */
#define C0_CACHE        7      /* cache lock function */
#define C0_BADADDR      8      /* Bad Address */
#define C0_BADVADDR     8      /* Bad Virtual Address */
#define C0_TLBHI        10     /* TLB EntryHi */
#define C0_SR           12     /* Processor Status */
#define C0_STATUS       12     /* Processor Status */
#define C0_CAUSE        13     /* Exception Cause */
#define C0_EPC          14     /* Exception PC */
#define C0_PRID         15     /* Processor Revision Indentifier */
#define C0_DEBUG        16     /* debug exception control */
#define C0_DEPC         17     /* PC for debug exception */
#define C0_TAGLO        20     /* cache tag register */
#endif

#ifdef SB1250
#define C0_INX      0       /* CP0: TLB Index */
#define C0_RAND     1       /* CP0: TLB Random */
#define C0_TLBLO0   2       /* CP0: TLB EntryLo0 */
#define C0_TLBLO    C0_TLBLO0   /* CP0: TLB EntryLo0 */
#define C0_TLBLO1   3       /* CP0: TLB EntryLo1 */
#define C0_CTEXT    4       /* CP0: Context */
#define C0_PGMASK   5       /* CP0: TLB PageMask */
#define C0_WIRED    6       /* CP0: TLB Wired */
#define C0_BADVADDR 8       /* CP0: Bad Virtual Address */
#define C0_COUNT    9       /* CP0: Count */
#define C0_TLBHI    10      /* CP0: TLB EntryHi */
#define C0_COMPARE  11      /* CP0: Compare */
#define C0_SR       12      /* CP0: Processor Status */
#define C0_STATUS   C0_SR       /* CP0: Processor Status */
#define C0_CAUSE    13      /* CP0: Exception Cause */
#define C0_EPC      14      /* CP0: Exception PC */
#define C0_PRID     15      /* CP0: Processor Revision Indentifier */
#define C0_CONFIG   16      /* CP0: Config */
#define C0_LLADDR   17      /* CP0: LLAddr */
#define C0_WATCHLO  18      /* CP0: WatchpointLo */
#define C0_WATCHHI  19      /* CP0: WatchpointHi */
#define C0_XCTEXT   20      /* CP0: XContext */
#define C0_ECC      26      /* CP0: ECC */
#define C0_CACHEERR 27      /* CP0: CacheErr */
#define C0_TAGLO    28      /* CP0: TagLo */
#define C0_TAGHI    29      /* CP0: TagHi */
#define C0_ERREPC   30      /* CP0: ErrorEPC */
#define C0_ERRPC   30      /* CP0: ErrorEPC */
#endif

#ifdef R7000
#define C0C_IPLLO       18
#define C0C_IPLHI       19
#define C0C_ICR         20
#define C0C_DERRADDR1   26
#define C0C_DERRADDR2   27
#endif
#endif	/* LANGUAGE_C */

/*------------------------------------------------------------------
 * Assembly file macros
 *------------------------------------------------------------------*/
#ifdef LANGUAGE_ASSEMBLY
#define LEAF(name) \
    .text; \
    .globl  name; \
    .ent    name; \
name:

#define XLEAF(name) \
    .text; \
    .globl  name; \
    .aent   name; \
name:

#define SLEAF(name) \
    .text; \
    .ent    name; \
name:

#define END(name) \
    .size name,.-name; \
    .end    name

#define SEND(name) END(name)

#define EXPORT(name) \
    .globl name; \
name:

#define EXPORTS(name,sz) \
    .globl name; \
name:

#define IMPORT(name, size) \
    .extern name,size

#define BSS(name,size) \
    .comm   name,size

#define COMM(name,size) \
    .comm   name,size

#define LBSS(name,size) \
    .lcomm  name,size

#define LCOMM(name,size) \
    .lcomm  name,size

#endif	/* LANGUAGE_ASSEMBLY */

/*------------------------------------------------------------------
 * Floating Point Control registers
 *------------------------------------------------------------------*/
#if defined(R7000) || defined(SB1250)
#ifdef LANGUAGE_ASSEMBLY
#define FPA_IRR         $0      /* Implementation/Revision register */
#define FPA_CSR         $31     /* Control/Status register */
#define C1_FRID          $0      /* Implementation/Revision register */
#define C1_CSR          $31     /* Control/Status register */
#endif

#ifdef LANGUAGE_C
#define FPA_IRR         0      /* Implementation/Revision register */
#define FPA_CSR         31     /* Control/Status register */
#define C1_FRID         0      /* Implementation/Revision register */
#define C1_CSR          31     /* Control/Status register */
#endif

#define CSR_C       0x00800000
#define CSR_EXC     0x0003f000
#define CSR_EE      0x00020000
#define CSR_EV      0x00010000
#define CSR_EZ      0x00008000
#define CSR_EO      0x00004000
#define CSR_EU      0x00002000
#define CSR_EI      0x00001000
#define CSR_TV      0x00000800
#define CSR_TZ      0x00000400
#define CSR_TO      0x00000200
#define CSR_TU      0x00000100
#define CSR_TI      0x00000080
#define CSR_SV      0x00000040
#define CSR_SZ      0x00000020
#define CSR_SO      0x00000010
#define CSR_SU      0x00000008
#define CSR_SI      0x00000004
#define CSR_RM      0x00000003
#endif	/* R7000 || SB1250 */

/*------------------------------------------------------------------
 * Status Register
 *------------------------------------------------------------------*/
#ifdef R7000
#define SR_IE       0x00000001  /* interrupt enable */
#define SR_EXL      0x00000002  /* exception level */
#define SR_ERL      0x00000004  /* error level */

#define SR_KSU_MASK 0x00000018  /* ksu mode mask */
#define SR_KSU_USER 0x00000010  /* user mode */
#define SR_KSU_SPVS 0x00000008  /* supervisor mode */
#define SR_KSU_KERN 0x00000000  /* kernel mode */

#define SR_UX       0x00000020  /* mips3 & xtlb in user mode */
#define SR_SX       0x00000040  /* mips3 & xtlb in supervisor mode */
#define SR_KX       0x00000080  /* xtlb in kernel mode */

/* interrupt mask bits */
#define SR_SINT0    0x00000100  /* enable s/w interrupt 0 */
#define SR_SINT1    0x00000200  /* enable s/w interrupt 1 */
#define SR_HINT0    0x00000400  /* enable h/w interrupt 1 */
#define SR_HINT1    0x00000800  /* enable h/w interrupt 2 */
#define SR_HINT2    0x00001000  /* enable h/w interrupt 3 */
#define SR_HINT3    0x00002000  /* enable h/w interrupt 4 */
#define SR_HINT4    0x00004000  /* enable h/w interrupt 5 */
#define SR_HINT5    0x00008000  /* enable h/w interrupt 6 */

/* alternative interrupt mask naming */
#define SR_IM0      0x00000100
#define SR_IM1      0x00000200
#define SR_IM2      0x00000400
#define SR_IM3      0x00000800
#define SR_IM4      0x00001000
#define SR_IM5      0x00002000
#define SR_IM6      0x00004000
#define SR_IM7      0x00008000

#define SR_IMASK    0x0000ff00

/* diagnostic field */
#define SR_DE       0x00010000  /* disable cache/ecc errors */
#define SR_CE       0x00020000  /* use ecc register */
#define SR_SR       0x00100000  /* soft reset occurred */
#define SR_BEV      0x00400000  /* boot exception vectors */

#define SR_RE       0x02000000  /* reverse endian (user mode) */
#define SR_FR       0x04000000  /* 64-bit fpu registers */
#define SR_CO       0x08000000  /* enable use cache flush */

#define SR_CU0      0x10000000  /* coprocessor 0 enable */
#define SR_CU1      0x20000000  /* coprocessor 1 enable */
#define SR_CU2      0x40000000  /* coprocessor 2 enable */
#define SR_XX       0x80000000  /* Mips IV ISA enable */
#endif

#ifdef TX3927
#define SR_CUMASK   0xf0000000  /* Coprocessor usable bits */
#define SR_CU3      0x80000000  /* Coprocessor 3 usable */
#define SR_CU2      0x40000000  /* coprocessor 2 usable */
#define SR_CU1      0x20000000  /* Coprocessor 1 usable */
#define SR_CU0      0x10000000  /* Coprocessor 0 usable */

#define SR_RE       0x02000000  /* Reverse Endian in user mode */
#define SR_BEV      0x00400000  /* Bootstrap Exception Vector */
#define SR_TS       0x00200000  /* TLB shutdown */

#define SR_NMI      0x00100000  /* generate an NMI */
#define SR_KUO      0x00000020  /* Kernel/User mode, old */
#define SR_IEO      0x00000010  /* Interrupt Enable, old */
#define SR_KUP      0x00000008  /* Kernel/User mode, previous */
#define SR_IEP      0x00000004  /* Interrupt Enable, previous */
#define SR_KUC      0x00000002  /* Kernel/User mode, current */
#define SR_IEC      0x00000001  /* Interrupt Enable, current */
#define SR_IE       SR_IEC	/* Match the rest please */

/* interrupt mask bits */
#define SR_SINT0    0x00000100  /* enable s/w interrupt 0 */
#define SR_SINT1    0x00000200  /* enable s/w interrupt 1 */
#define SR_HINT0    0x00000400  /* enable h/w interrupt 1 */
#define SR_HINT1    0x00000800  /* enable h/w interrupt 2 */
#define SR_HINT2    0x00001000  /* enable h/w interrupt 3 */
#define SR_HINT3    0x00002000  /* enable h/w interrupt 4 */
#define SR_HINT4    0x00004000  /* enable h/w interrupt 5 */
#define SR_HINT5    0x00008000  /* enable h/w interrupt 6 */

/* alternative interrupt mask naming */
#define SR_IM0      0x00000100
#define SR_IM1      0x00000200
#define SR_IM2      0x00000400
#define SR_IM3      0x00000800
#define SR_IM4      0x00001000
#define SR_IM5      0x00002000
#define SR_IM6      0x00004000
#define SR_IM7      0x00008000

#define SR_IMASK    0x0000ff00
#endif

#ifdef SB1250
#define SR_CUMASK 0xf0000000
#define SR_CU3    0x80000000
#define SR_CU2    0x40000000
#define SR_CU1    0x20000000
#define SR_CU0    0x10000000

#define SR_RP     0x08000000
#define SR_FR     0x04000000
#define SR_RE     0x02000000
#define SR_MX     0x01000000
#define SR_PX     0x00800000
#define SR_BEV    0x00400000
#define SR_TS     0x00200000
#define SR_SR     0x00100000
#define SR_NMI    0x00080000

#define SR_IMASK  0x0000ff00

#define SR_IM0      0x00000100
#define SR_IM1      0x00000200
#define SR_IM2      0x00000400
#define SR_IM3      0x00000800
#define SR_IM4      0x00001000
#define SR_IM5      0x00002000
#define SR_IM6      0x00004000
#define SR_IM7      0x00008000

#define SR_KX     0x00000080
#define SR_SX     0x00000040
#define SR_UX     0x00000020

#define SR_KSU_MASK 0x00000018  /* ksu mode mask */
#define SR_KSU_USER 0x00000010  /* user mode */
#define SR_KSU_SPVS 0x00000008  /* supervisor mode */
#define SR_KSU_KERN 0x00000000  /* kernel mode */

#define SR_ERL    0x00000004
#define SR_EXL    0x00000002
#define SR_IE     0x00000001
#endif

/*------------------------------------------------------------------
 * Config Register
 *------------------------------------------------------------------*/
#ifdef R7000
#define CFG_SC      0x80000000  /* Secondary Cache Present */
#define CFG_ECMASK  0x70000000  /* System Clock Ratio */
#define CFG_ECSHIFT 28
#define CFG_EPMASK  0x0f000000  /* Transmit data pattern */
#define CFG_EPD     0x00000000  /* D */
#define CFG_EPDDX   0x01000000  /* DDX */
#define CFG_EPDDXX  0x02000000  /* DDXX */
#define CFG_EPDXDX  0x03000000  /* DXDX */
#define CFG_EPDDXXX 0x04000000  /* DDXXX */
#define CFG_EPDDXXXX    0x05000000  /* DDXXXX */
#define CFG_EPDXXDXX    0x06000000  /* DXXDXX */
#define CFG_EPDDXXXXX   0x07000000  /* DDXXXXX */
#define CFG_EPDXXXDXXX  0x08000000  /* DXXXDXXX */
#define CFG_SBMASK  0x00c00000  /* Scache block size */
#define CFG_SBSHIFT 22
#define CFG_PIMASK  0x00300000  /* Processor ID */
#define CFG_PISHIFT 20
#define CFG_EWMASK  0x000c0000
#define CFG_EW64    0x00000000
#define CFG_TC      0x00020000  /* Tertiary Cache Present */
#define CFG_CLK     0x00010000  /* Half-integer clock multiplier */
#define CFG_BE      0x00008000  /* Big Endian */
#define CFG_EM      0x00004000  /* ECC Mode */
#define CFG_EB      0x00002000  /* Block ordering */
#define CFG_TE      0x00001000  /* Tertiary Cache Enable */
#define CFG_ICMASK  0x00000e00  /* Instruction cache size */
#define CFG_ICSHIFT 9
#define CFG_DCMASK  0x000001c0  /* Data cache size */
#define CFG_DCSHIFT 6
#define CFG_IB      0x00000020  /* Instruction cache block size */
#define CFG_DB      0x00000010  /* Data cache block size */
#define CFG_SE      0x00000008
#define CFG_K0MASK  0x00000007  /* KSEG0 coherency algorithm */
#endif

#ifdef TX3927
#define CFG_ICMASK      0x00380000      /* instr cache size */
#define CFG_ICSHIFT     19
#define CFG_DCMASK      0x00070000      /* data cache size */
#define CFG_DCSHIFT     16
#define CFG_CWFON       0x00004000      /* critical word first mode */
#define CFG_WBON        0x00000200      /* data cache mode */
#define CFG_RF          0x00000C00      /* reduced frequency */
#define CFG_DOZE        0x00000200
#define CFG_HALT        0x00000100
#define CFG_LOCK_CFG    0x00000080
#define CFG_ICE         0x00000020      /* instr cache enable */
#define CFG_DCE         0x00000010      /* data cache enable */
#define CFG_IRSIZE_MASK 0x0000000C      /* instr cache burst refill size */
#define CFG_IRSIZE_SHFIT    2
#define CFG_IRSIZE_4W       0
#define CFG_IRSIZE_8W       1
#define CFG_IRSIZE_16W      2
#define CFG_IRSIZE_32W      3
#define CFG_DRSIZE_MASK 0x00000003      /* data cache burst refill size */
#define CFG_DRSIZE_SHIFT    0
#define CFG_DRSIZE_4W       0
#define CFG_DRSIZE_8W       1
#define CFG_DRSIZE_16W      2
#define CFG_DRSIZE_32W      3
#endif

#ifdef SB1250
#define CFG_BE			0x00008000
#define CFG_K0MASK		0x00000007  /* KSEG0 coherency algorithm */

#define CFG_C_UNCACHED          2
#define CFG_C_NONCOHERENT       3
#define CFG_C_WBACK             3
#define CFG_C_COHERENTXCL       4
#define CFG_C_COHERENTXCLW      5
#define CFG_C_UNCACHEDACCEL	7

#endif

/*------------------------------------------------------------------
 * Cause Register
 *------------------------------------------------------------------*/
#ifdef R7000
#define CAUSE_BD        0x80000000  /* Branch Delay */
#define CAUSE_CEMASK    0x30000000  /* Coprocessor Error */
#define CAUSE_CESHIFT   28          /* Right justify CE  */
#define CAUSE_W2        0x04000000  /* watch 2 triggered */
#define CAUSE_W1        0x02000000  /* watch 1 triggered */
#define CAUSE_IV        0x01000000  /* interrupt vector enable */
#define CAUSE_IPSHIFT   8           /* Right justify IP  */
#define CAUSE_IPMASK    0x00ffff00  /* interrupt pending mask */
#define CAUSE_IP15     0x00800000
#define CAUSE_IP14     0x00400000
#define CAUSE_IP13     0x00200000
#define CAUSE_IP12     0x00100000
#define CAUSE_IP11     0x00080000
#define CAUSE_IP10     0x00040000
#define CAUSE_IP9      0x00020000
#define CAUSE_IP8      0x00010000
#define CAUSE_IP7      0x00008000
#define CAUSE_IP6      0x00004000
#define CAUSE_IP5      0x00002000
#define CAUSE_IP4      0x00001000
#define CAUSE_IP3      0x00000800
#define CAUSE_IP2      0x00000400
#define CAUSE_IP1      0x00000200
#define CAUSE_IP0      0x00000100
#define CAUSE_SW2       0x00000200  /*  (Software Interrupt 1) */
#define CAUSE_SW1       0x00000100  /*  (Software Interrupt 0) */
#define CAUSE_EXCMASK   0x0000007c  /* Exception Code */
#define CAUSE_EXCSHIFT  2           /* Right justify EXC */

#define CEXC_INT    (0 << 2)    /* External interrupt */
#define CEXC_MOD    (1 << 2)    /* TLB modification */
#define CEXC_TLBL   (2 << 2)    /* TLB miss (Load or Ifetch) */
#define CEXC_TLBS   (3 << 2)    /* TLB miss (Save) */
#define CEXC_ADEL   (4 << 2)    /* Address error (Load or Ifetch) */
#define CEXC_ADES   (5 << 2)    /* Address error (Save) */
#define CEXC_IBE    (6 << 2)    /* Bus error (Ifetch) */
#define CEXC_DBE    (7 << 2)    /* Bus error (data load or store) */
#define CEXC_SYS    (8 << 2)    /* System call */
#define CEXC_BP     (9 << 2)    /* Break point */
#define CEXC_RI     (10 << 2)   /* Reserved instruction */
#define CEXC_CPU    (11 << 2)   /* Coprocessor unusable */
#define CEXC_OVF    (12 << 2)   /* Arithmetic overflow */
#define CEXC_TRAP   (13 << 2)   /* Trap exception */
#define CEXC_FPE    (15 << 2)   /* Floating Point Exception */
#define CEXC_IWATCH (16 << 2)   /* Inst Watchpoint exception */
#define CEXC_DWATCH (23 << 2)   /* Data Watchpoint exception */
#define CEXC_CACHE  (30 << 2)   /* Fake Cache Error */
#endif

#ifdef TX3927
#define CAUSE_BD        0x80000000  /* Branch Delay */
#define CAUSE_CEMASK    0x30000000  /* Coprocessor Error */
#define CAUSE_CESHIFT   28          /* Right justify CE  */
#define CAUSE_IPMASK    0x0000ff00  /* Interrupt Pending */
#define CAUSE_IPSHIFT   8           /* Right justify IP  */
#define CAUSE_IP8       0x00008000  /*  (Intr5) */
#define CAUSE_IP7       0x00004000  /*  (Intr4) */
#define CAUSE_IP6       0x00002000  /*  (Intr3) */
#define CAUSE_IP5       0x00001000  /*  (Intr2) */
#define CAUSE_IP4       0x00000800  /*  (Intr1) */
#define CAUSE_IP3       0x00000400  /*  (Intr0) */
#define CAUSE_SW2       0x00000200  /*  (Software Interrupt 1) */
#define CAUSE_SW1       0x00000100  /*  (Software Interrupt 0) */
#define CAUSE_EXCMASK   0x0000007c  /* Exception Code */
#define CAUSE_EXCSHIFT  2           /* Right justify EXC */

#define CEXC_INT    (0 << 2)    /* External interrupt */
#define CEXC_MOD    (1 << 2)    /* TLB modification */
#define CEXC_TLBL   (2 << 2)    /* TLB miss (Load or Ifetch) */
#define CEXC_TLBS   (3 << 2)    /* TLB miss (Save) */
#define CEXC_ADEL   (4 << 2)    /* Address error (Load or Ifetch) */
#define CEXC_ADES   (5 << 2)    /* Address error (Save) */
#define CEXC_IBE    (6 << 2)    /* Bus error (Ifetch) */
#define CEXC_DBE    (7 << 2)    /* Bus error (data load or store) */
#define CEXC_SYS    (8 << 2)    /* System call */
#define CEXC_BP     (9 << 2)    /* Break point */
#define CEXC_RI     (10 << 2)   /* Reserved instruction */
#define CEXC_CPU    (11 << 2)   /* Coprocessor unusable */
#define CEXC_OVF    (12 << 2)   /* Arithmetic overflow */
#endif

#ifdef SB1250
#define CAUSE_BD        0x80000000  /* Branch Delay */
#define CAUSE_CEMASK    0x30000000  /* Coprocessor Error */
#define CAUSE_CESHIFT   28          /* Right justify CE  */

#define CAUSE_IV		0x00800000	/* special interrupt */
#define CAUSE_WP		0x00400000	/* watch interrupt deferred */
#define CAUSE_IPMASK    0x0000ff00  /* Interrupt Pending */
#define CAUSE_IPSHIFT   8           /* Right justify IP  */
#define CAUSE_IP7      0x00008000
#define CAUSE_IP6      0x00004000
#define CAUSE_IP5      0x00002000
#define CAUSE_IP4      0x00001000
#define CAUSE_IP3      0x00000800
#define CAUSE_IP2      0x00000400
#define CAUSE_IP1      0x00000200
#define CAUSE_IP0      0x00000100
#define CAUSE_SW2      0x00000200  /*  (Software Interrupt 1) */
#define CAUSE_SW1      0x00000100  /*  (Software Interrupt 0) */
#define CAUSE_EXCMASK   0x0000007c  /* Exception Code */
#define CAUSE_EXCSHIFT  2           /* Right justify EXC */

#define CEXC_INT    (0 << 2)    /* External interrupt */
#define CEXC_MOD    (1 << 2)    /* TLB modification */
#define CEXC_TLBL   (2 << 2)    /* TLB miss (Load or Ifetch) */
#define CEXC_TLBS   (3 << 2)    /* TLB miss (Save) */
#define CEXC_ADEL   (4 << 2)    /* Address error (Load or Ifetch) */
#define CEXC_ADES   (5 << 2)    /* Address error (Save) */
#define CEXC_IBE    (6 << 2)    /* Bus error (Ifetch) */
#define CEXC_DBE    (7 << 2)    /* Bus error (data load or store) */
#define CEXC_SYS    (8 << 2)    /* System call */
#define CEXC_BP     (9 << 2)    /* Break point */
#define CEXC_RI     (10 << 2)   /* Reserved instruction */
#define CEXC_CPU    (11 << 2)   /* Coprocessor unusable */
#define CEXC_OVF    (12 << 2)   /* Arithmetic overflow */
#define CEXC_TRAP   (13 << 2)   /* Trap exception */
#define CEXC_VCEI	(14 << 2)	/* Virtual Coherency Exception (I) */
#define CEXC_FPE    (15 << 2)   /* Floating Point Exception */
#define CEXC_CP2	(16 << 2)   /* Cp2 exception */
#define CEXC_WATCH	(23 << 2)   /* Watchpoint exception */
#define CEXC_CACHE  (30 << 2)   /* Fake Cache Error */
#define CEXC_VCED	(31 << 2)	/* Virtual Coherency Exception (D) */
#endif
/*------------------------------------------------------------------
 * Cache Tag Registers
 *------------------------------------------------------------------*/
#ifdef R7000
/*
 * Primary cache mode
 */
#define CFG_C_WTHRU_NOALLOC 0
#define CFG_C_WTHRU_ALLOC   1
#define CFG_C_UNCACHED      2
#define CFG_C_NONCOHERENT   3
#define CFG_C_WBACK     3
#define CFG_C_UNCACHED_NOBLOCK  6
#define CFG_C_BYPASS        7

/* Primary Cache TagLo */
#define TAGLO_PREDEC_MASK   0xfff00000  /* Icache predecode */
#define TAGLO_PREDEC_SHIFT  20
#define TAGLO_PSTATE_MASK   0x000000c0  /* Cache State */
#define TAGLO_PSTATE_SHIFT  6

/* Secondary Cache TagLo */
#define TAGLO_SSTATE_MASK   0x000001c0  /* Cache State */
#define TAGLO_SSTATE_SHIFT  6

/* Tertiary Cache TagLo */
#define TAGLO_TSTATE_MASK   0x000001c0  /* Cache State */
#define TAGLO_TSTATE_SHIFT  6

/* Common TagLo bits */
#define TAGLO_SP        0x00000020  /* State Parity */
#define TAGLO_B         0x00000008  /* Bypass */
#define TAGLO_L         0x00000004  /* Locked */
#define TAGLO_F         0x00000002  /* Fifo bit */
#define TAGLO_P         0x00000001  /* SP,B,P,L Parity */

/* State values for primary cache */
#define TAGLO_PSTATE_INVAL  0
#define TAGLO_PSTATE_PENDING    1
#define TAGLO_PSTATE_IVALID 2       /* InstrValid */
#define TAGLO_PSTATE_DVALID 3       /* DataValid */

/* State values for secondary cache */
#define TAGLO_SSTATE_INVAL  0
#define TAGLO_SSTATE_CLEAN  4
#define TAGLO_SSTATE_DIRTY  5
#define TAGLO_SSTATE_PENDING    6
#define TAGLO_SSTATE_DIRTY_DC   7

/* State values for tertiary cache */
#define TAGLO_TSTATE_INVAL  0
#define TAGLO_TSTATE_VALID  4

/*
 * RM7000 TagHi register
 */
#define TAGHI_PTAG_MASK     0xffffff00
#define TAGHI_STAG_MASK     0xfffff000
#define TAGHI_TTAG_MASK     0xffff8000
#define TAGHI_ADDRTOIDX(addr)   ((unsigned)(addr) >> 4)
#define TAGHI_IDXTOADDR(idx)    ((unsigned)(addr) << 4)
#endif

#ifdef SB1250

#define S_TAGLO_PTAG    8
#define M_TAGLO_PTAG    0xffffffffffffff00LL

#define TAGLO_PSTATE_SHIFT  6
#define TAGLO_PSTATE_MASK  0x000000c0
#define TAGLO_PSTATE_INVAL        0
#define TAGLO_PSTATE_SHARED       1
#define TAGLO_PSTATE_CLEAN_EXCL   2
#define TAGLO_PSTATE_DIRTY_EXCL   3

#define TAGLO_L		0x00000020
#define TAGLO_P		0x00000001

#endif

/*------------------------------------------------------------------
 * Info Register
 *------------------------------------------------------------------*/
#ifdef R7000
#define INFO_TL     0x20000000  /* Large TLB: 64 entries */
#define INFO_BM     0x10000000  /* Burst Mode Scache RAMS */
#define INFO_DSMASK 0x0c000000  /* Drive Strength */
#define INFO_DSSHIFT    26
#define INFO_NP     0x02000000  /* Non-Pendant bus mode */
#define INFO_WPMASK 0x01800000  /* Write Protocol */
#define INFO_WPSHIFT    23
#define INFO_DWMASK 0x007e0000  /* Dcache ways/sets */
#define INFO_DWSHIFT    17
#define INFO_IWMASK 0x0001f800  /* Icache ways/sets */
#define INFO_IWSHIFT    11
#define INFO_SWMASK 0x000007e0  /* Scache ways/sets */
#define INFO_SWSHIFT    5
#define INFO_SSMASK 0x0000001e  /* Scache size */
#define INFO_SSSHIFT    1
#define INFO_AE     0x00000001  /* Atomic Enable */
#endif

/*------------------------------------------------------------------
 * ECC Register
 *------------------------------------------------------------------*/
#ifdef R7000
#define ECC_SS          0x10000000  /* Set Select */
#define ECC_I           0x08000000  /* Enable Icache locking */
#define ECC_D           0x04000000  /* Enable Dcache locking */
#define ECC_S           0x02000000  /* Enable Scache locking */
#define ECC_PARMASK     0x000000ff  /* Parity bits */
#endif

/*------------------------------------------------------------------
 * CacheErr Register
 *------------------------------------------------------------------*/
#ifdef R7000
#define CACHEERR_ER     0x80000000
#define CACHEERR_TYPE	0x80000000  /* reference type: 0=Instr, 1=Data */
#define CACHEERR_EC     0x40000000
#define CACHEERR_LEVEL	0x40000000  /* cache level: 0=Primary, 1=Secondary */

#define CACHEERR_ED     0x20000000
#define CACHEERR_DATA	0x20000000  /* data field: 0=No error, 1=Error */

#define CACHEERR_ET     0x10000000
#define CACHEERR_TAG	0x10000000  /* tag field: 0=No error, 1=Error */
#define CACHEERR_EE     0x04000000
#define CACHEERR_BUS	0x04000000  /* error on bus: 0=No, 1=Yes */

#define CACHEERR_E0     0x00800000  /* Physaddr in DErrAddr1 */

#define CACHEERR_E1     0x00400000  /* Physaddr in DErrAddr2 */

#define CACHEERR_IDX        0x003ffff8  /* Physaddr 21:3 */
#endif

#ifdef SB1250
#define CACHEERR_ER     0x80000000
#define CACHEERR_TYPE	0x80000000  /* reference type: 0=Instr, 1=Data */
#define CACHEERR_EC     0x40000000
#define CACHEERR_LEVEL	0x40000000  /* cache level: 0=Primary, 1=Secondary */

#define CACHEERR_ED     0x20000000
#define CACHEERR_DATA	0x20000000  /* data field: 0=No error, 1=Error */

#define CACHEERR_ET     0x10000000
#define CACHEERR_TAG	0x10000000  /* tag field: 0=No error, 1=Error */

#define CACHEERR_EXTRQ	0x08000000	/* External request caused error */

#define CACHEERR_EE     0x04000000
#define CACHEERR_BUS	0x04000000  /* error on bus: 0=No, 1=Yes */

#define CACHEERR_ADATA	0x02000000	/* additional data */

#define CACHEERR_IDX	0x003fffff
#endif

/*------------------------------------------------------------------
 * TLB Registers
 *------------------------------------------------------------------*/
#ifdef R7000
#define NTLBENTRIES		64
#endif
#ifdef TX3927
#define NTLBENTRIES		32
#endif
#ifdef SB1250
#define NTLBENTRIES		64
#endif

#ifdef R7000
#define TLBHI_ADDR_MASK     0xFFFFE000
#define TLBLO_ADDR_MASK     0xFFFFF000
#define TLBLO_ADDR_SHIFT    6

#define TLBMASK_MASKMASK        0x01ffe000
#define TLBMASK_MASK4Kb         0x00000000
#define TLBMASK_MASK16kb        0x00006000
#define TLBMASK_MASK64kb        0x0001e000
#define TLBMASK_MASK256kb       0x0007e000
#define TLBMASK_MASK1Mb         0x001fe000
#define TLBMASK_MASK4Mb         0x007fe000
#define TLBMASK_MASK16Mb        0x01ffe000

#ifdef R9000
#define TLBMASK_MASK64Mb        0x07ffe000
#define TLBMASK_MASK256Mb       0x1fffe000
#endif

/* FIXME 64bit TLB entries */
#define TLBHI_ASIDMASK          0x000000ff
#define TLBHI_VPN2MASK          0xffffe000
#define TLBHI_4100_VPN2MASK     0xfffff800

#define TLBLO_G             0x00000001
#define TLBLO_V             0x00000002
#define TLBLO_D             0x00000004
#define TLBLO_CALGMASK      0x00000038
#define TLBLO_PFNMASK       0x3fffffc0

#define TLBLO_CALG_UNCACHED 0x00000010
#define TLBLO_CALG_WBACK    0x00000018
#endif

#ifdef TX3927
#define TLBMASK_MASKMASK        0x003ff000
#define TLBMASK_MASK4Kb         0x00000000
#define TLBMASK_MASK16kb        0x00003000
#define TLBMASK_MASK64kb        0x0000f000
#define TLBMASK_MASK256kb       0x0003f000
#define TLBMASK_MASK1Mb         0x000ff000
#define TLBMASK_MASK4Mb         0x003ff000

#define TLBLO_G                 0x00000100
#define TLBLO_V                 0x00000200
#define TLBLO_D                 0x00000400
#define TLBLO_N                 0x00000800

#define TLB_ENTLO_ADDR_MASK     0xfffff000
#define TLB_ENTLO_ADDR_SHIFT    0
#define TLB_ENTHI_ADDR_MASK     0xfffff000
#endif

#ifdef SB1250
#define TLBMASK_MASKMASK        0x01ffe000
#define TLBMASK_MASK4Kb         0x00000000
#define TLBMASK_MASK16kb        0x00006000
#define TLBMASK_MASK64kb        0x0001e000
#define TLBMASK_MASK256kb       0x0007e000
#define TLBMASK_MASK1Mb         0x001fe000
#define TLBMASK_MASK4Mb         0x007fe000
#define TLBMASK_MASK16Mb        0x01ffe000
#define TLBMASK_MASK64Mb        0x07ffe000

#define TLBLO_G             0x00000001
#define TLBLO_V             0x00000002
#define TLBLO_D             0x00000004
#define TLBLO_CALGMASK      0x00000038

#define TLBLO_CALG_UNCACHED			0x00000010
#define TLBLO_CALG_NONCOHERENT		0x00000018
#define TLBLO_CALG_COHERENT_EXCL	0x00000020
#define TLBLO_CALG_COHERENT_SHAREABLE	0x00000028
#define TLBLO_CALG_UNCACHED_ACCEL	0x00000038

#define TLBHI_ASIDMASK          0x000000ff
#define TLBHI_VPN2MASK          0xffffe000
#define TLBLO_PFNMASK       0x3fffffc0

#define TLBHI_ADDR_MASK		0x00000fffffffe000LL
#define TLBLO_ADDR_MASK		0x000000fffffff000LL
#define TLBLO_ADDR_SHIFT	6
#endif

/* R4000 Index bits */
#define TLBIDX_MASK 0x3f
#define TLBIDX_SHIFT    0

/* R4000 Random bits */
#define TLBRAND_MASK    0x3f
#define TLBRAND_SHIFT   0

/*------------------------------------------------------------------
 * Debug Registers
 *------------------------------------------------------------------*/
#ifdef R7000
/* RM7000 Watch1 and Watch2 registers */
#define WATCH_ST    0x8000000000000000LL    /* Watch Stores */
#define WATCH_LD    0x4000000000000000LL    /* Watch Loads */
#define WATCH_IN    0x2000000000000000LL    /* WatchIfetch */
#define WATCH_PA    0x0000000ffffffffcLL    /* Phyaddr 35:2 */

/* RM7000 WatchMask register */
#define WATCHMASK_MASK  0xfffffffc      /* Address mask */
#define WATCHMASK_W2    0x00000002      /* Apply to Watch2 */
#define WATCHMASK_W1    0x00000001      /* Apply to Watch1 */

/* RM7000 PerfCtrl register */
#define PERFCTRL_CE 0x00000400      /* Count Enable */
#define PERFCTRL_UM 0x00000200      /* Count in User Mode */
#define PERFCTRL_KM 0x00000100      /* Count in Kernel Mode */
#define PERFCTRL_CSF    0x0000001f      /* Counter Select */

/* RM7000 PerfCtrl counter select */
#define PERFCTRL_CS_PCLOCK  0x00        /* Clock cycles */
#define PERFCTRL_CS_TOTAL_ISSUE 0x01        /* Total insns issued */
#define PERFCTRL_CS_FP_ISSUE    0x02        /* FP insns issued */
#define PERFCTRL_CS_INT_ISSUE   0x03        /* Int insns issued */
#define PERFCTRL_CS_LOAD    0x04        /* Load insns issued */
#define PERFCTRL_CS_STORE   0x05        /* Store insns issued */
#define PERFCTRL_CS_DUAL_ISSUE  0x06        /* Dual issued pairs */
#define PERFCTRL_CS_BR_PREFETCH 0x07        /* Branch prefetches */
#define PERFCTRL_CS_TCMISS  0x08        /* T-cache misses */
#define PERFCTRL_CS_STALL   0x09        /* Stall cycles */
#define PERFCTRL_CS_SCMISS  0x0a        /* S-cache misses */
#define PERFCTRL_CS_ICMISS  0x0b        /* I-cache misses */
#define PERFCTRL_CS_DCMISS  0x0c        /* D-cache misses */
#define PERFCTRL_CS_DTLBMISS    0x0d        /* DTLB misses */
#define PERFCTRL_CS_ITLBMISS    0x0e        /* ITLB misses */
#define PERFCTRL_CS_IJTLBMISS   0x0f        /* JTLB insn misses */
#define PERFCTRL_CS_DJTLBMISS   0x10        /* JTLB data misses */
#define PERFCTRL_CS_BR_TAKEN    0x11        /* Branches taken */
#define PERFCTRL_CS_BR_ISSUE    0x12        /* Branches issued */
#if 1   /* errata */
#define PERFCTRL_CS_DC_WBACK    0x13        /* D-cache writebacks */
#else
#define PERFCTRL_CS_SC_WBACK    0x13        /* S-cache writebacks */
#define PERFCTRL_CS_DC_WBACK    0x14        /* D-cache writebacks */
#endif
#define PERFCTRL_CS_DC_STALL    0x15        /* D-cache miss stalls */
#define PERFCTRL_CS_MISS    0x16        /* Cache misses */
#define PERFCTRL_CS_FPU_STALL   0x17        /* FPU stalls */
/* Rev2.0 only from here */
#define PERFCTRL_CS_MULT_SLIP   0x18        /* Multiplier slip cycles */
#define PERFCTRL_CS_CP0_SLIP    0x19        /* Coprocessor 0 slip cycles */
#define PERFCTRL_CS_NBLD_SLIP   0x1a        /* NB-load slip cycles */
#define PERFCTRL_CS_WB_STALL    0x1b        /* Write buffer full stalls */
#define PERFCTRL_CS_CACHE_STALL 0x1c        /* Cache insn stalls */
#define PERFCTRL_CS_MULT_STALL  0x1d        /* Multuplier stalls */
#define PERFCTRL_CS_NBLD_STALL  0x1e        /* NB-load stalls */
#endif

#ifdef SB1250
#define WATCH_PA      0xfffffff8
#define WATCH_R       0x00000002
#define WATCH_W       0x00000001
#endif

/*------------------------------------------------------------------
 * Cacheops
 *------------------------------------------------------------------*/
#ifdef R7000
#define Index_Invalidate_I               0x00        /* 0       0 */
#define Index_Writeback_Inv_D            0x01        /* 0       1 */
#define Flash_Invalidate_T       0x02        /* 0       2 */
#define Index_Writeback_Inv_S            0x03        /* 0       3 */
/*----*/
#define Index_Load_Tag_I                 0x04        /* 1       0 */
#define Index_Load_Tag_D                 0x05        /* 1       1 */
#define Index_Load_Tag_T                 0x06        /* 1       2 */
#define Index_Load_Tag_S                 0x07        /* 1       3 */
/*----*/
#define Index_Store_Tag_I                0x08        /* 2       0 */
#define Index_Store_Tag_D                0x09        /* 2       1 */
#define Index_Store_Tag_T                0x0A        /* 2       2 */
#define Index_Store_Tag_S                0x0B        /* 2       3 */
/*----*/
#define Create_Dirty_Exc_D               0x0D        /* 3       1 */
/*----*/
#define Hit_Invalidate_I                 0x10        /* 4       0 */
#define Hit_Invalidate_D                 0x11        /* 4       1 */
#define Hit_Invalidate_S                 0x13        /* 4       3 */
/*----*/
#define Fill_I                           0x14        /* 5       0 */
#define Hit_Writeback_Inv_D              0x15        /* 5       1 */
#define Page_Invalidate_T                0x16        /* 5       2 */
#define Hit_Writeback_Inv_S              0x17        /* 5       3 */
/*----*/
#define Hit_Writeback_I                  0x18        /* 6       0 */
#define Hit_Writeback_D                  0x19        /* 6       1 */
#define Hit_Writeback_S                  0x1B        /* 6       3 */
#endif

#ifdef TX3927
#define Index_Invalidate_I                  0x0         /* 0    0 */
#define Index_Writeback_Inv_D               0x1         /* 0    1 */
#define Index_Invalidate_SI                 0x0
#define Index_Writeback_Inv_SD              0x1
#define Index_Load_Tag_I                    0x4         /* 1    0 */
#define Index_Load_Tag_D                    0x5         /* 1    1 */
#define Index_Load_Tag_SI                   0x4
#define Index_Load_Tag_SD                   0x5
#define Index_Store_Tag_I                   0x8         /* 2    0 */
#define Index_Store_Tag_D                   0x9         /* 2    1 */
#define Index_Store_Tag_SI                  0x8
#define Index_Store_Tag_SD                  0x9
#define Create_Dirty_Exc_D                  0xD         /* 3    1 */
#define Create_Dirty_Exc_SD                 0xD
#define Hit_Invalidate_I                    0x10        /* 4    0 */
#define Hit_Invalidate_D                    0x11        /* 4    1 */
#define Hit_Invalidate_SI                   0x10        /* 4    2 */
#define Hit_Invalidate_S                    0x11        /* 4    3 */
#define Fill_I                              0x14        /* 5    0 */
#define Hit_Writeback_Inv_D                 0x15        /* 5    1 */
#define Hit_Writeback_Inv_S                 0x15        /* 5    3 */
#define Hit_Writeback_I                     0x18        /* 6    0 */
#define Hit_Writeback_D                     0x19        /* 6    1 */
#define Hit_Writeback_S                     0x19        /* 6    3 */
#endif

#ifdef SB1250
#define Index_Invalidate_I               0x0         /* 0       0 */
#define Index_Writeback_Inv_D            0x1         /* 0       1 */
#define Index_Load_Tag_I                 0x4         /* 1       0 */
#define Index_Load_Tag_D                 0x5         /* 1       1 */
#define Index_Store_Tag_I                0x8         /* 2       0 */
#define Index_Store_Tag_D                0x9         /* 2       1 */
#define Hit_Invalidate_I                 0x10        /* 4       0 */
#define Hit_Invalidate_D                 0x11        /* 4       1 */
#define Hit_Writeback_Inv_D              0x15        /* 5       1 */
#define Hit_Writeback_D                  0x19        /* 6       1 */
#endif

/*------------------------------------------------------------------
 * Types
 *------------------------------------------------------------------*/
#ifdef LANGUAGE_C
/*
 * Standard types
 */
typedef unsigned long       reg32_t;    /* a 32-bit register */
typedef unsigned long long  reg64_t;    /* a 64-bit register */
#if __mips >= 3 || __mips64
typedef unsigned long long  reg_t;
typedef signed long long    sreg_t;
#else
typedef unsigned long       reg_t;
typedef signed long     sreg_t;
#endif
/*
 * Standard address types
 */
#if !defined(SSC_mips) && !defined(_KERNEL) && !defined(_MIPS_TYPES_H_)
typedef unsigned long       paddr_t;    /* a physical address */
typedef unsigned long       vaddr_t;    /* a virtual address */
#endif
#if __mips64
typedef unsigned long long  tlbhi_t;    /* the tlbhi field */
#else
typedef unsigned long       tlbhi_t;    /* the tlbhi field */
#endif
typedef unsigned long       tlblo_t;    /* the tlblo field */

#endif	/* LANGUAGE_C */

/*------------------------------------------------------------------
 * Access Macros
 *------------------------------------------------------------------*/
#ifdef LANGUAGE_C
#define _mips_nop() \
  __asm__ __volatile ("%(nop%)" : :)

#define _mips_sync() \
  __asm__ __volatile ("sync")

#define _mips_wait() \
  __asm__ __volatile ("wait")

#define _mips_get_reg(reg)  \
({                  \
  register reg32_t __r;     \
  __asm__ __volatile ("move %0,$%1" : "=r" (__r) : "JK" (reg));   \
  __r;              \
})

#define _mips_get_address_reg(reg)  \
({                  \
  register address_t __r;     \
  __asm__ __volatile ("move %0,$%1" : "=r" (__r) : "JK" (reg));   \
  __r;              \
})


#define crash_mfc0(reg) \
({ \
  register reg32_t __r; \
  __asm__ __volatile ("mfc0 %0,$%1" \
              : "=d" (__r) \
              : "JK" (reg)); \
  __r; \
})

#define get_ra()    \
({                  \
  register address_t __r;     \
  __asm__ __volatile ("move %0,$31" : "=r" (__r) : );   \
  __r;              \
})

#define _mips_mfc0(reg) \
({ \
  register reg32_t __r; \
  __asm__ __volatile ("mfc0 %0,$%1" \
              : "=d" (__r) \
              : "JK" (reg)); \
  __r; \
})

#define _mips_mtc0(reg, val) \
do { \
    __asm__ __volatile ("%(mtc0 %z0,$%1%)" \
            : \
            : "dJ" ((reg32_t)(val)), "JK" (reg) \
            : "memory");\
} while (0)


#if __mips64
#define _mips_dmfc0(reg) \
({ \
  register reg64_t __r; \
  __asm__ __volatile ("dmfc0 %0,$%1" \
              : "=d" (__r) \
              : "JK" (reg)); \
  __r; \
})

#define _mips_dmtc0(reg, val) \
do { \
    __asm__ __volatile ("%(dmtc0 %z0,$%1%)" \
            : \
            : "dJ" ((reg64_t)(val)), "JK" (reg) \
            : "memory");\
} while (0)

#else
/* handle registers in two parts */
#define _mips_dmfc0(reg) \
({ \
  register reg64_t __r, __v; \
  __asm__ __volatile ("dmfc0 %0,$%1" \
              : "=d" (__r) \
              : "JK" (reg)); \
  __asm__ ("dsll %L0,%1,32; dsra %L0,%L0,32; dsra %M0,%1,32" \
        : "=&d" (__v) \
        : "d" (__r)); \
  __v; \
})

#define _mips_dmtc0(reg, val) \
do { \
    register reg64_t __v = (val), __r; \
    register reg32_t __t; \
    __asm__ ("dsll %0,%M2,32; dsll %1,%L2,32; dsrl %1,32; or %0,%1" \
        : "=&d" (__r), "=&d" (__t) \
        : "d" (__v)); \
    __asm__ __volatile ("%(dmtc0 %z0,$%1%)" \
            : \
            : "dJ" (__r), "JK" (reg) \
            : "memory");\
} while (0)

#endif

/*
 * Define cpu-specific macros for accessing new RM7000 CP0 registers.
 */

#define _rm7k_cfc0(reg) \
({ \
  register reg32_t __r; \
  __asm__ __volatile ("cfc0 %0,$%1" \
              : "=d" (__r) \
                  : "JK" (reg)); \
  __r; \
})

#define _rm7k_ctc0(reg, val) \
do { \
    __asm__ __volatile ("%(ctc0 %z0,$%1%)" \
            : \
            : "dJ" ((reg32_t)(val)), "JK" (reg) \
            : "memory");\
} while (0)

#define _rm7k_cxc0(reg, val) \
({ \
    register reg32_t __o; \
    __o = _rm7k_cfc0 (reg); \
    _rm7k_ctc0 (reg, val); \
    __o; \
})

#define _rm7k_csc0(reg, set) \
({ \
    register reg32_t __o; \
    __o = _rm7k_cfc0 (reg); \
    _rm7k_ctc0 (reg, __o | (set)); \
    __o; \
})

#define _rm7k_ccc0(reg, clr) \
({ \
    register reg32_t __o; \
    __o = _rm7k_cfc0 (reg); \
    _rm7k_ctc0 (reg, __o & ~(clr)); \
    __o; \
})

#define _rm7k_ccsc0(reg, clr, set) \
({ \
    register reg32_t __o; \
    __o = _rm7k_cfc0 (reg); \
    _rm7k_ctc0 (reg, (__o & ~(clr)) | (set)); \
    __o; \
})

/* RM7000A Info register */
#define rm7k_getinfo()      _mips_mfc0(7)
#define rm7k_setinfo(v)     _mips_mtc0(7,v)

/* RM7000 PerfCtrl register */
#define rm7k_getperfctrl()  _mips_mfc0(22)
#define rm7k_setperfctrl(v) _mips_mtc0(22,v)
#define rm7k_xchperfctrl(v) _mips_mxc0(22,v)

/* RM7000 WatchMask register */
#define rm7k_getwatch1()    _mips_dmfc0(18)
#define rm7k_setwatch1(v)   _mips_dmtc0(18,v)

/* RM7000 WatchMask register */
#define rm7k_getwatch2()    _mips_dmfc0(19)
#define rm7k_setwatch2(v)   _mips_dmtc0(19,v)

/* RM7000 WatchMask register */
#define rm7k_getwatchmask() _mips_mfc0(24)
#define rm7k_setwatchmask(v)    _mips_mtc0(24,v)
#define rm7k_xchwatchmask(v)    _mips_mxc0(24,v)

/* RM7000 PerfCount register */
#define rm7k_getperfcount() _mips_mfc0(25)
#define rm7k_setperfcount(v)    _mips_mtc0(25,v)
#define rm7k_xchperfcount(v)    _mips_mxc0(25,v)

/* RM7000 IPLLo register */
#define rm7k_getipllo()     _rm7k_cfc0(18)
#define rm7k_setipllo(v)    _rm7k_ctc0(18,v)
#define rm7k_xchipllo(v)    _rm7k_cxc0(18,v)
#define rm7k_bcsipllo(c,s)  _rm7k_ccsc0(18,c,s)

/* RM7000 IPLHi register */
#define rm7k_getiplhi()     _rm7k_cfc0(19)
#define rm7k_setiplhi(v)    _rm7k_ctc0(19,v)
#define rm7k_xchiplhi(v)    _rm7k_cxc0(19,v)
#define rm7k_bcsiplhi(c,s)  _rm7k_ccsc0(19,c,s)

/* RM7000 IntCtrl register (NOTE they are NOT atomic operations) */
#define rm7k_geticr()       _rm7k_cfc0(20)
#define rm7k_seticr(v)      _rm7k_ctc0(20,v)
#define rm7k_xchicr(v)      _rm7k_cxc0(20,v)
#define rm7k_bicicr(clr)    _rm7k_ccc0(20,clr)
#define rm7k_bisicr(set)    _rm7k_csc0(20,set)
#define rm7k_bcsicr(c,s)    _rm7k_ccsc0(20,c,s)

/* RM7000 DerrAddr1 */
#define rm7k_getderraddr1() _rm7k_cfc0(26)

/* RM7000 DerrAddr1 */
#define rm7k_getderraddr2() _rm7k_cfc0(27)

/* R4000 CP0 Status register */
#define r4k_getsr()			_mips_mfc0(C0_SR)
#define r4k_setsr(v)		_mips_mtc0(C0_SR,v)

/* R4000 CP0 Context register */
#define r4k_getcontext()    _mips_mfc0(C0_CONTEXT)
#define r4k_setcontext(v)   _mips_mtc0(C0_CONTEXT,v)
#define r4k_xchcontext(v)   _mips_mxc0(C0_CONTEXT,v)

/* R4000 CP0 EntryHi register */
#define r4k_getentryhi()    _mips_mfc0(C0_ENTRYHI)
#define r4k_setentryhi(v)   _mips_mtc0(C0_ENTRYHI,v)
#define r4k_xchentryhi(v)   _mips_mxc0(C0_ENTRYHI,v)

/* R4000 CP0 EntryLo0 register */
#define r4k_getentrylo0()   _mips_mfc0(C0_ENTRYLO0)
#define r4k_setentrylo0(v)  _mips_mtc0(C0_ENTRYLO0,v)
#define r4k_xchentrylo0(v)  _mips_mxc0(C0_ENTRYLO0,v)

/* R4000 CP0 EntryLo1 register */
#define r4k_getentrylo1()   _mips_mfc0(C0_ENTRYLO1)
#define r4k_setentrylo1(v)  _mips_mtc0(C0_ENTRYLO1,v)
#define r4k_xchentrylo1(v)  _mips_mxc0(C0_ENTRYLO1,v)

/* R4000 CP0 PageMask register */
#define r4k_getpagemask()   _mips_mfc0(C0_PAGEMASK)
#define r4k_setpagemask(v)  _mips_mtc0(C0_PAGEMASK,v)
#define r4k_xchpagemask(v)  _mips_mxc0(C0_PAGEMASK,v)

/* R4000 CP0 Wired register */
#define r4k_getwired()      _mips_mfc0(C0_WIRED)
#define r4k_setwired(v)     _mips_mtc0(C0_WIRED,v)
#define r4k_xchwired(v)     _mips_mxc0(C0_WIRED,v)

#endif	/* LANGUAGE_C */

/*------------------------------------------------------------------
 * PRID
 *------------------------------------------------------------------*/
#define	PRID_R3900	0x22	/* Toshiba/Philips R3900 CPU	ISA I	*/
#define	PRID_RM7000	0x27	/* QED RM7000 CPU		ISA IV  */

#define	PRID_RM7000F	0x27	/* QED RM7000 FPU		ISA IV  */
#define PRID_RM9000	0x34    /* PMC-Sierra E9000 core (RM9k) ISA IV  */

#define PRID_R9K_CPU_SHIFT	24		/* RM9k CPU ID Shift  */
#define PRID_R9K_CPU_MASK	(7 << PRID_R9K_CPU_SHIFT)    /* CPU ID Mask  */


/*------------------------------------------------------------------
 * jump buf defs
 *------------------------------------------------------------------*/
/* defines for longjmp buffer */
#define JB_S0       0
#define JB_S1       1
#define JB_S2       2
#define JB_S3       3
#define JB_S4       4
#define JB_S5       5
#define JB_S6       6
#define JB_S7       7
#define JB_FP       8
#define JB_SP       9
#define JB_RA       10
#define JB_SIZ      11

#ifdef LANGUAGE_C
#if __mips >= 3
typedef long long jmp_buf[JB_SIZ];
#else
typedef int jmp_buf[JB_SIZ];
#endif
#endif

#ifdef SIBYTE
#define PREF_LOAD           0
#define PREF_STORE          1
#define PREF_LOAD_STREAMED  4
#define PREF_STORE_STREAMED 5
#define PREFETCH(_op, _base, _offset)	\
	__asm__ __volatile__("pref %0,%2(%1)" : : "i" (_op), "r" (_base), "i" (_offset))
#endif

#endif	/* !_MIPS_H */

