/*
 *-----------------------------------------------------------------
 *
 * Name:	st16552.h
 *
 * RCS:  $Id: st16552.h,v 1.2 2001/09/17 01:08:51 rick Exp $
 *
 * Copyright (C) 2001, Agile Storage, Inc.
 *
 * Description:	definitions for 16552 serial chip (Exar chip on Eval Board)
 *
 * Created by:	Rick Lund
 *
 * Date Created:	6/14/01
 *
 *-----------------------------------------------------------------
 */

#ifndef _ST16552_H
#define _ST16552_H

#define ST16552_CHANNEL_A_OFF	0x0
#define ST16552_CHANNEL_B_OFF	0x20

#ifdef LANGUAGE_C

/*
 * ST16552 registers
 */
typedef struct st16552_channel {
	uint32	rx_reg;
#define tx_reg	rx_reg
	uint32	ier;
	uint32	isr;
#define fifo	isr
	uint32	lcr;
	uint32	mcr;
	uint32	lsr;
	uint32	msr;
	uint32	scr;
} st16552_channel_t;

typedef struct st16552_dev {
	st16552_channel_t	chan_a;
	st16552_channel_t	chan_b;
} st16552_dev_t;

#endif	/* LANGUAGE_C */

#define ST16552_DATA_OFF	0x0
#define ST16552_IER_OFF		0x4
#define ST16552_ISR_OFF		0x8
#define ST16552_FIFO_OFF	0x8
#define ST16552_LCR_OFF		0xC
#define ST16552_MCR_OFF		0x10
#define ST16552_LSR_OFF		0x14
#define ST16552_MSR_OFF		0x18
#define ST16552_SCR_OFF		0x1C

/*
 * 16 bit baud rate divisor (LSB in DLL, MSB in DLM)
 */
#define ST16552_HZ	3686400
#define ST16552_BRTC(x)	(ST16552_HZ/ (16 * (x)))

/*
 * Interrupt Enable Bits
 */
#define IER_MODEM_STATUS	0x8
#define IER_RX_LINE			0x4
#define IER_TX				0x2
#define IER_RX				0x1

/*
 * FIFO Control Register Bits
 */
#define FIFO_ENABLE		0x01
#define FIFO_RX_RESET	0x02
#define FIFO_TX_RESET	0x04
#define FIFO_DMA_MODE	0x08
#define FIFO_TRIGGER_1	0x00
#define FIFO_TRIGGER_4	0x40
#define FIFO_TRIGGER_8	0x80
#define FIFO_TRIGGER_14	0xC0

/*
 * Interrupt Status Bits
 */
#define ISR_IMASK		0xF
#define ISR_RX_TIMEOUT	0xC
#define ISR_RLS			0x6
#define ISR_RXRDY		0x4
#define ISR_TXRDY		0x2
#define ISR_MSR			0x0
#define ISR_FIFO		0xC0

/*
 * Line Control Register Bits
 */
#define LCR_DIV_LATCH_ENABLE	0x80
#define LCR_SET_BREAK			0x40
#define LCR_SET_PARITY			0x20
#define LCR_EVEN_PARITY			0x10
#define LCR_ODD_PARITY			0x00
#define LCR_PARITY_ENABLE		0x08
#define LCR_STOP_BITS			0x04
#define LCR_DATA_BITS_8			0x03
#define LCR_DATA_BITS_7			0x02
#define LCR_DATA_BITS_6			0x01
#define LCR_DATA_BITS_5			0x00

/*
 * Modem Control Register Bits
 */
#define MCR_LOOPBACK		0x10
#define MCR_INT_ENABLE		0x08
#define MCR_RI				0x04
#define MCR_RTS				0x02
#define MCR_DTR				0x01

/*
 * Line Status Register Bits
 */
#define LSR_RX_FIFO			0x80
#define LSR_TSRE			0x40
#define LSR_TXRDY			0x20
#define LSR_BREAK			0x10
#define LSR_FRAME_ERROR		0x08
#define LSR_PARITY_ERROR	0x04
#define LSR_OVERRUN			0x02
#define LSR_RXRDY			0x01
#define LSR_RCV_MASK		0x1f

/*
 * Modem Status Register Bits
 */
#define MSR_DCD		0x80
#define MSR_RI		0x40
#define MSR_DSR		0x20
#define MSR_CTS		0x10
#define MSR_DDCD	0x08
#define MSR_TERI	0x04
#define MSR_DDSR	0x02
#define MSR_DCTS	0x01

#endif	/* _ST16552_H */

