#!/bin/sh
#   makerom -f FULL_PROM_TYPE <file.bin> <file.big> <file.version> COMP_START [<diag.big> <diag.version> COMP_START [<lmux.hex> <lmux.info> <luc.hex> <luc.info>]]
#
#       PROM_TYPE : r7k
#                   sibyte
#                   ssc_mgmt
#                   rprom_r7k
#                   rprom_sibyte
#
#       COMP_START : either the *.nm file to the *.big file or offset to where compression should start,
#                    (i.e. 0 ==> compress all) and "" ==> no compression
#
#	makerom -s PROM_TYPE <file.bin> <file.big> <file.version> [COMP_START]
#
#       PROM_TYPE : r7k
#                   sibyte
#                   rprom_r7k
#                   rprom_sibyte
#                   ssc_mgmt
#                   diag_r7k
#                   diag_sibyte
#                   lmux_hex
#                   luc_hex
#
#       COMP_START : either the *.nm file to the *.big file or offset to where compression should start,
#                    (i.e. 0 ==> compress all) and "" or not specifying this argument ==> no compression
# 
# run this script w/ the 'sh' cmd from the base of you nfx-tree and it assumes you have
# checked out ../prom_binaries and write access to /tftpboot
#


DESTDIR=/tftpboot/newproms
SIBYDIR=Install/PROM_SIBYTE/Images
SIBRDIR=Install/PROM_SIBYTE/Images
R7K_DIR=Install/PROM_R7000/Images
R7KRDIR=Install/PROM_R7000/Images
MGMTDIR=Install/DIAG_SSC_MGMT/Images
LMUXDIR=prom-binaries/fpga-images/LMUX
LUCDIR=prom-binaries/fpga-images/LUC
SIBDDIR=prom-binaries/diag-images/SIBYTE
R7KDDIR=prom-binaries/diag-images/R7000

BCSIBYDIR=Install/PROM_SIBYTE_BC/Images
BCSIBRDIR=Install/PROM_SIBYTE_BC/Images
R9K_DIR=Install/PROM_R9000/Images
R9KRDIR=Install/PROM_R9000/Images
BCSIBDDIR=prom-binaries/diag-images/SIBYTE_BC
R9KDDIR=prom-binaries/diag-images/R9000

rm -rf $DESTDIR

mkdir  $DESTDIR

MKROM='Tools/makerom -s'
$MKROM r7k          $DESTDIR/r7kboot.bin $R7K_DIR/bootrom.big $R7K_DIR/bootrom.version $R7K_DIR/bootrom.nm
$MKROM sibyte       $DESTDIR/sibboot.bin $SIBYDIR/bootrom.big $SIBYDIR/bootrom.version $SIBYDIR/bootrom.nm
$MKROM ssc_mgmt     $DESTDIR/mgtboot.bin $MGMTDIR/bootrom.big $MGMTDIR/bootrom.version
$MKROM lmux_hex     $DESTDIR/lmux.bin $LMUXDIR/lmux_sw.hex $LMUXDIR/revision.txt 0
$MKROM luc_hex      $DESTDIR/luc3.bin $LUCDIR/luc3_sw.hex $LUCDIR/revision.txt 0
$MKROM r9k          $DESTDIR/r9kboot.bin $R9K_DIR/bootrom.big $R9K_DIR/bootrom.version $R9K_DIR/bootrom.nm
$MKROM sibyte_bc    $DESTDIR/sibboot_bc.bin $BCSIBYDIR/bootrom.big $BCSIBYDIR/bootrom.version $BCSIBYDIR/bootrom.nm

MKROM='Tools/makerom -f'
$MKROM rprom_r7k    $DESTDIR/r7k.bin $R7KRDIR/bootrom.big $R7KRDIR/bootrom.version $R7KRDIR/bootrom.nm
$MKROM rprom_sibyte $DESTDIR/sibyte.bin $SIBRDIR/bootrom.big $SIBRDIR/bootrom.version $SIBRDIR/bootrom.nm
$MKROM r7k          $DESTDIR/r7kfimg.bin $R7K_DIR/bootrom.big $R7K_DIR/bootrom.version $R7K_DIR/bootrom.nm \
                                         $R7KDDIR/diag.big $R7KDDIR/diag.version $R7KDDIR/diag.nm
$MKROM sibyte       $DESTDIR/sibfimg.bin $SIBYDIR/bootrom.big $SIBYDIR/bootrom.version $SIBYDIR/bootrom.nm \
                                         $SIBDDIR/diag.big $SIBDDIR/diag.version $SIBDDIR/diag.nm \
                                         $LMUXDIR/lmux_sw.hex $LMUXDIR/revision.txt \
                                         $LUCDIR/luc3_sw.hex   $LUCDIR/revision.txt

$MKROM rprom_r9k    $DESTDIR/r9k.bin $R9KRDIR/bootrom.big $R9KRDIR/bootrom.version $R9KRDIR/bootrom.nm
$MKROM rprom_sibyte_bc $DESTDIR/sibyte_bc.bin $BCSIBRDIR/bootrom.big $BCSIBRDIR/bootrom.version $BCSIBRDIR/bootrom.nm
$MKROM r9k          $DESTDIR/r9kfimg.bin $R9K_DIR/bootrom.big $R9K_DIR/bootrom.version $R9K_DIR/bootrom.nm \
                                         $R9KDDIR/diag.big $R9KDDIR/diag.version $R9KDDIR/diag.nm
$MKROM sibyte_bc    $DESTDIR/sibfimg_bc.bin $BCSIBYDIR/bootrom.big $BCSIBYDIR/bootrom.version $BCSIBYDIR/bootrom.nm \
                                         $BCSIBDDIR/diag.big $BCSIBDDIR/diag.version $BCSIBDDIR/diag.nm \
                                         $LMUXDIR/lmux_sw.hex $LMUXDIR/revision.txt \
                                         $LUCDIR/luc3_sw.hex   $LUCDIR/revision.txt

ls -l $DESTDIR
