/*
 *-----------------------------------------------------------------
 *
 * Name:	promio.sx
 *
 * RCS:  $Id: promio.S,v 1.7 2002/10/17 21:32:42 johnh Exp $
 *
 * Copyright (C) 2001, Agile Storage, Inc.
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	6/16/01
 *
 *-----------------------------------------------------------------
 */

#ifdef __mips

#include <nfx-incl.h>
#include <sys/mips.h>
#include <exports/promio-api.h>

	.set	noreorder

/*-----------------------------------------------------------------
 * Name:	promio_prs
 *
 * Description:	Print a string on console
 *			registers used:
 *						a0 = ARG1 = string to print (string must end with 0)
 *						t2 = TEMP
 *
 *			Must be called with JMP_SUB level <= 4
 *
 *
 * Created by:	Rick Lund
 *
 * Date Created:	6/16/01
 *
 *-----------------------------------------------------------------
 */
	.globl	promio_prs
	.ent	promio_prs
promio_prs:
	move	t2,ARG1
1:
	lbu		ARG1,0(t2)
	nop
	beqz	ARG1,2f
	nop
	JMP_SUB5(promio_put_char)
	addu	t2,t2,1
	b		1b
	nop

2:
	RET_SUB

	.end	promio_prs

/*-----------------------------------------------------------------
 * Name:	promio_pnl
 *
 * Description:	Print <CR, LF> on console
 *			registers used:
 *						s5 = ra save in JMP_SUB5
 *
 *			Must be called with JMP_SUB level <= 4
 *
 * Created by:	Rick Lund
 *
 * Date Created:	6/16/01
 *
 *-----------------------------------------------------------------
 */
	.globl	promio_pnl
	.ent	promio_pnl
promio_pnl:
	li	ARG1,'\r'
	JMP_SUB5(promio_put_char)
	li	ARG1,'\n'
	JMP_SUB5(promio_put_char)

	RET_SUB

	.end	promio_pnl

/*-----------------------------------------------------------------
 * Name:	promio_prx
 *
 * Description:	Print N hex digits
 *			registers used:
 *						a0 = ARG1 = hex value to print
 *						a1 = ARG2 = number of nibbles to print
 *						t3 = TEMP
 *						t4 = TEMP
 *						t5 = TEMP
 *
 *			Must be called with JMP_SUB level <= 3
 *
 * Created by:	Rick Lund
 *
 * Date Created:	6/16/01
 *
 *-----------------------------------------------------------------
 */
	.globl	promio_prx
	.ent	promio_prx
promio_prx:
	move	t3,ARG1		# integer
	move	t4,ARG2		# number of nibbles to print

	/*
	 * Print '0x' prefix
	 */
	li	ARG1,'0'
	JMP_SUB5(promio_put_char)
	li	ARG1,'x'
	JMP_SUB5(promio_put_char)

	/*
	 * Get shift
	 */
	move	t5,t4
	subu	t5,t5,1
	sll		t5,t5,2

	/*
	 * print N nibbles
	 */
1:
	move	ARG1,t3
#if __mips >= 3
	dsrl	ARG1,ARG1,t5
#else
	srl		ARG1,ARG1,t5
#endif
	and		ARG1,0xf
	JMP_SUB4(promio_prx_nib)
	beqz	t5,2f
	nop
	subu	t5,t5,4
	b		1b
	nop

2:
	RET_SUB

	.end	promio_prx

/*-----------------------------------------------------------------
 * Name:	promio_prx_nib
 *
 * Description:	Print a hex nibble on console
 *			registered used:
 *						a0 = ARG1 = hex nibble to print
 *
 *			Must be called with JMP_SUB level <= 4
 *
 * Created by:	Rick Lund
 *
 * Date Created:	6/16/01
 *
 *-----------------------------------------------------------------
 */
	.globl	promio_prx_nib
	.ent	promio_prx_nib
promio_prx_nib:
	ble		ARG1,9,1f
	nop
	subu	ARG1,ARG1,10
	addu	ARG1,ARG1,'A'
	JMP_SUB5(promio_put_char)
	RET_SUB

1:
	addu	ARG1,ARG1,'0'
	JMP_SUB5(promio_put_char)
	RET_SUB

	.end	promio_prx_nib

/*-----------------------------------------------------------------
 * Name:	promio_readx
 *
 * Description:	Read a hex value from the console
 *
 *			Must be called with JMP_SUB level <= 2
 *
 * Created by:	Rick Lund
 *
 * Date Created:	6/16/01
 *
 *-----------------------------------------------------------------
 */
	.globl	promio_readx
	.ent	promio_readx
promio_readx:
	move	t3,zero
	move	t4,zero
	move	t5,ARG1

1:
	JMP_SUB3(promio_readx_nib)

	beq		RET,ESC_CODE,do_esc
	nop
	beq		RET,BS_CODE,do_bs
	nop
	beq		RET,CR_CODE,do_cr
	nop
do_hex:
	bge		t4,t5,1b
	nop
	move	ARG1,RET
	bge		RET,0xA,show_alpha
	nop
show_num:
	add		ARG1,ARG1,'0'
	b		2f
	nop
show_alpha:
	subu	ARG1,ARG1,0xA
	addu	ARG1,ARG1,'A'

2:
#if __mips >= 3
	dsll	t3,t3,4
#else
	sll		t3,t3,4
#endif
	or		t3,t3,RET
	add		t4,t4,1
	b		1b
	nop

do_esc:
#ifdef USE_NSDB
	j		nsdb
	nop
#endif

do_bs:
#if __mips >= 3
	dsrl	t3,t3,4
#else
	srl		t3,t3,4
#endif
	subu	t4,t4,1
	b		1b
	nop

do_cr:
	JMP_SUB4(promio_pnl)

	beqz	t4,none_read
	nop
	move	RET,t3
	b		1f
	nop
none_read:
	move	RET,SAVE_VAL
1:
	RET_SUB

	.end	promio_readx

/*-----------------------------------------------------------------
 * Name:	promio_readx_nib
 *
 * Description:
 *
 *			Must be called with JMP_SUB level <= 3
 *
 * Created by:	Rick Lund
 *
 * Date Created:	6/16/01
 *
 *-----------------------------------------------------------------
 */
	.globl	promio_readx_nib
	.ent	promio_readx_nib
promio_readx_nib:
	JMP_SUB5(promio_get_char)

chk_esc:
	bne		RET,ESC,chk_bs
	nop
	li		RET,ESC_CODE
	b		got_hex_nib
	nop
chk_bs:
	bne		RET,BS,chk_cr
	nop
	li		RET,BS_CODE
	b		got_hex_nib
	nop
chk_cr:
	bne		RET,CR,chk_hex
	nop
	li		RET,CR_CODE
	b		got_hex_nib
	nop
chk_hex:
	bgt		RET,'9',chk_upper_case
	nop
	sub		RET,RET,'0'
	bltz	RET,not_hex_nib
	nop
	b		got_hex_nib
	nop
chk_upper_case:
	blt		RET,'A',not_hex_nib
	nop
	bgt		RET,'F',chk_lower_case
	nop
	subu	RET,RET,'A'
	addu	RET,RET,0xA
	b		got_hex_nib
	nop
chk_lower_case:
	blt		RET,'a',not_hex_nib
	nop
	bgt		RET,'f',not_hex_nib
	nop
	subu	RET,RET,'a'
	addu	RET,RET,0xA
	b		got_hex_nib
	nop

not_hex_nib:
	li		ARG1,BS
	JMP_SUB5(promio_put_char)
	b		promio_readx_nib
	nop

got_hex_nib:
	RET_SUB

	.end	promio_readx_nib

#endif	/* __mips */

