/*
 *-----------------------------------------------------------------
 *
 * Name:	st16552-promio.sx
 *
 * RCS:  $Id: st16552-promio.S,v 1.10 2001/12/08 22:17:38 rick Exp $
 *
 * Copyright (C) 2001, Agile Storage, Inc.
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	6/8/01
 *
 *-----------------------------------------------------------------
 */

#ifdef __mips

#if defined(EVAL_BOARD) && defined(PROM_R7000)

#include <exports/hwMap.h>
#include <sys/st16552.h>
#include <sys/mips.h>
#include <exports/promio-api.h>

 
	.set	noreorder

	.text

/*-----------------------------------------------------------------
 * Name:	promio_init()
 *
 * Description:
 *			registers used: t1 = UART
 *							t0 = TEMP
 *							t2 = TEMP
 *							t3 = TEMP
 *
 * Created by:	Rick Lund
 *
 * Date Created:	6/16/01
 *
 *-----------------------------------------------------------------
 */
	.globl	promio_init
	.ent	promio_init
promio_init:

	la		UART,PHYS_TO_K1(HW_ST16552_0_BASE + ST16552_CHANNEL_A_OFF)

ser_init_channel:
	sw		zero,ST16552_IER_OFF(UART)

	li		t0,(FIFO_ENABLE | FIFO_RX_RESET | FIFO_TX_RESET | FIFO_TRIGGER_4)
	sb		t0,ST16552_FIFO_OFF(UART)

#if 0
	/*
	 * Short pause, while Rx and Tx come out of reset
	 */
	li		t0,0x7FFF
1:
	sub		t0,t0,1
	bgtz	t0,1b
	nop
#endif

#if 0
	/*
	 * Wait for transmitter to drain any characters
	 */
	li		t0,0x3FFF
	li		t2,LSR_TSRE
1:
	lw		t3,ST16552_LSR_OFF(UART)
	and		t3,t3,t2
	bnez	t3,1f
	nop
	subu	t0,1
	bnez	t0,1b
	nop
#endif

1:
	/*
	 * Initialize baud rate to 115200
	 */
/* li		t2,2		*/			/* 2 = 115200, 4 = 57600, 6 = 38400 */

	li		t2,LCR_DIV_LATCH_ENABLE
	sb		t2,ST16552_LCR_OFF(UART)

	li		t2,ST16552_BRTC(57600)
	sb		t2,ST16552_DATA_OFF(UART)

	srl		t2,8
	sb		t2,ST16552_IER_OFF(UART)

	li		t2,LCR_DATA_BITS_8
	sb		t2,ST16552_LCR_OFF(UART)

	li		t2,(MCR_DTR | MCR_RTS)
	sb		t2,ST16552_MCR_OFF(UART)

	sb		zero,ST16552_IER_OFF(UART)

	move	t2,UART
	la		UART,PHYS_TO_K1(HW_ST16552_0_BASE + ST16552_CHANNEL_B_OFF)
	bne		t2,UART,ser_init_channel
	nop

	j		ra
	nop

	.end	promio_init

/*-----------------------------------------------------------------
 * Name:	promio_put_char()
 *
 * Description:
 *			registers used: t1 = UART
 *							t0 = TEMP
 *							a0 = ARG1
 *
 * Created by:	Rick Lund
 *
 * Date Created:	6/16/01
 *
 *-----------------------------------------------------------------
 */
	.globl	promio_put_char
	.ent	promio_put_char
promio_put_char:
	la		UART,PHYS_TO_K1(HW_ST16552_0_BASE + ST16552_CHANNEL_B_OFF)
1:	lbu		TEMP,ST16552_LSR_OFF(UART)
	nop
	and		TEMP,TEMP,LSR_TXRDY
	beqz	TEMP,1b
	nop
	sb		ARG1,ST16552_DATA_OFF(UART)
	nop
	j		ra
	nop

	.end	promio_put_char

/*-----------------------------------------------------------------
 * Name:	promio_get_char()
 *
 * Description:
 *			registers used: t1 = UART
 *							t0 = TEMP
 *							v0 = RET
 *							a0 = ARG1
 *
 * Created by:	Rick Lund
 *
 * Date Created:	6/16/01
 *
 *-----------------------------------------------------------------
 */
	.globl	promio_get_char
	.ent	promio_get_char
promio_get_char:
	la		UART,PHYS_TO_K1(HW_ST16552_0_BASE + ST16552_CHANNEL_B_OFF)
1:	lbu		TEMP,ST16552_LSR_OFF(UART)
	nop
	and		TEMP,TEMP,LSR_RXRDY
	beqz	TEMP,1b
	nop
	lb		RET,ST16552_DATA_OFF(UART)
	nop

1:	lbu		TEMP,ST16552_LSR_OFF(UART)
	nop
	and		TEMP,TEMP,LSR_TXRDY
	beqz	TEMP,1b
	nop
	sb		RET,ST16552_DATA_OFF(UART)
	nop

	j		ra
	nop

	.end	promio_get_char

	.globl	promio_get_char_no_echo
	.ent	promio_get_char_no_echo
promio_get_char_no_echo:
	la		UART,PHYS_TO_K1(HW_ST16552_0_BASE + ST16552_CHANNEL_B_OFF)
1:	lbu		TEMP,ST16552_LSR_OFF(UART)
	nop
	and		TEMP,TEMP,LSR_RXRDY
	beqz	TEMP,1b
	nop
	lb		RET,ST16552_DATA_OFF(UART)
	nop

	j		ra
	nop

	.end	promio_get_char_no_echo

/*-----------------------------------------------------------------
 * Name:	promio_char_avail()
 *
 * Description:
 *			registers used: t1 = UART
 *							v0 = RET
 *
 * Created by:	Rick Lund
 *
 * Date Created:	6/16/01
 *
 *-----------------------------------------------------------------
 */
	.globl	promio_char_avail
	.ent	promio_char_avail
promio_char_avail:
	la		UART,PHYS_TO_K1(HW_ST16552_0_BASE + ST16552_CHANNEL_B_OFF)
	lbu		RET,ST16552_LSR_OFF(UART)
	nop
	and		RET,RET,LSR_RXRDY
	j		ra
	nop

	.end	promio_char_avail

/*-----------------------------------------------------------------
 * Name:	promio_tx_ready()
 *
 * Description:
 *			registers used:	t1 = UART
 *							v0 = RET
 *
 *			RET will be 0 or != 0 (NOT == 1, if transmitter is ready)
 *
 * Created by:	Rick Lund
 *
 * Date Created:	6/16/01
 *
 *-----------------------------------------------------------------
 */
	.globl	promio_tx_ready
	.ent	promio_tx_ready
promio_tx_ready:
	la		UART,PHYS_TO_K1(HW_ST16552_0_BASE + ST16552_CHANNEL_B_OFF)
	lbu		RET,ST16552_LSR_OFF(UART)
	nop
	and		RET,RET,LSR_TXRDY
	j		ra
	nop

	.end	promio_tx_ready

	.set	reorder

#endif	/* EVAL_BOARD && PROM_R7000 */
#endif	/* __mips */

