/*
 *-----------------------------------------------------------------
 *
 * Name:	nfx-init.S
 *
 * RCS:  $Id: nfx-init.S,v 1.27 2002/12/04 22:46:09 lipikab Exp $
 *
 * Copyright (C) 2001, Agile Storage, Inc.
 *
 * Description:	low level board initialization - PROM code
 *
 * Created by:	Rick Lund
 *
 * Date Created:	9/13/01
 *
 *-----------------------------------------------------------------
 */

#ifdef PROM

#include <nfx-incl.h>
#include <sys/mips.h>
#include <exports/hwMap.h>
#include <sys/gt64240r.h>
#include <sys/bmfpga.h>
#include <exports/promio-api.h>
#include <exports/gpp-defs.h>

	.text

	.globl	nfx_init

	.extern	nfx_init_done

	.set	noreorder

/*-----------------------------------------------------------------
 * Name:	nfx_init
 *
 * Description:
 *				galileo init
 *				uart init
 *				nsdb
 *
 * Created by:	Rick Lund
 *
 * Date Created:	9/13/01
 *
 *-----------------------------------------------------------------
 */
	.ent	nfx_init
nfx_init:

#ifdef AGILE_HARDWARE

	mfc0	t0,C0_CONFIG
	nop
	nop
	nop
	and		t0,CFG_PIMASK
	srl		t0,CFG_PISHIFT
	and		t0,1
	beq		t0,0,nfx_init_ssc			/* Are we the SSC? CFG_SI = [00] */
	nop
	beq		t0,1,nfx_init_fc			/* Are we the FC? CFG_SI = [01] */
	nop
	j		nfx_init_done				/* otherwise, unknown board, give up */
	nop

nfx_init_ssc:
	#Jump to DIAG partition-based on env variables:qt
	li		a0,0xA8
    	li		a1,0x88	
    	jal     	gt_read_spd_byte /* read the 136th byte */
	nop
	nop
    	move    	t0,v0
	beq		t0,0,1f
	nop
	beq		t0,0xff,1f
	nop
	j		0xbfcc0000
	nop

1:
	#Jump to DIAG partition-based on env variables:ft
	li		a0,0xA8
    	li		a1,0x89	
    	jal     	gt_read_spd_byte /* read the 137th byte */
	nop
	nop
    	move    	t0,v0
	beq		t0,0,1f
	nop
	beq		t0,0xff,1f
	nop
	j		0xbfcc0000
	nop

1:
	#Jump to DIAG partition-based on env variables:diag
	li		a0,0xA8
    	li		a1,0x8a	
    	jal     	gt_read_spd_byte /* read the 138th byte */
	nop
	nop
    	move    	t0,v0
	beq		t0,0,1f
	nop
	beq		t0,0xff,1f
	nop
	j		0xbfcc0000
	nop

1:
	/*
	 * If it's a warm reset, don't reinit windows, just return
	 */
	mfc0	t0,C0_SR
	nop
	nop
	nop
	and		t0,SR_SR		/* warm reset? */
	beqz	t0,1f
	nop
	j		nfx_init_done
	nop

1:
	la		a0,PHYS_TO_K1(HW_GALILEO_BASE)
	sw		t0,DEVICE_BANK3PARAMETERS(a0)
	nop

	/*
	 * move GT internal registers
	 */
	li	t0,HW_GALILEO_BASE
	srl	t0,20
	or	t0,0x01000000
	la	a0,0xb4000000
	sw	t0,0x68(a0)
	nop
	nop
	nop
	nop

    la      a0,PHYS_TO_K1(HW_GALILEO_BASE)
    lw      t0,CPU_CONFIGURATION(a0)
    or      t0,0x00020000
    sw      t0,CPU_CONFIGURATION(a0)  

	/* enable BMFPGA accesses */
	GT_WRITE_REG_ASS(DEVICE_BANK2PARAMETERS, 0xf226e87f)

    la      a0,PHYS_TO_K1(HW_GALILEO_BASE)
	li		t0,HW_BMFPGA_BASE
	srl		t0,20
	sw		t0,CS_2_LOW_DECODE_ADDRESS(a0)
	li		t0,HW_BMFPGA_BASE+0x001fffff
	srl		t0,20
	sw		t0,CS_2_HIGH_DECODE_ADDRESS(a0)

	#Jump to DIAG partition-based on Jmpr
	la		a0,PHYS_TO_K1(HW_BMFPGA_BASE)
	lw		t0,0(a0)
	and		t0,0x00002000
	srl		t0,13
	bne		t0,1,2f
	nop
	j		0xbfcc0000
	nop

2:
	la		a0,PHYS_TO_K1(HW_BMFPGA_BASE)
	lw		t0,BMFPGA_REG_BOARD_STATUS_OFF(a0)
	and		t0,BMFPGA_BOARD_STATUS_MEM_TYPE_MASK
	srl		t0,BMFPGA_BOARD_STATUS_MEM_TYPE_SHIFT
	beq		t0,BMFPGA_BOARD_STATUS_MEM_TYPE_UNREG,ssc_mem_unreg
	nop
ssc_mem_reg:
	GT_WRITE_REG_ASS(SDRAM_CONFIGURATION, 0xd8e20578)
	b		ssc_mem_config_done
	nop
ssc_mem_unreg:
	GT_WRITE_REG_ASS(SDRAM_CONFIGURATION, 0xd8e00578)
	b		ssc_mem_config_done
	nop

ssc_mem_config_done:
	la		a0,PHYS_TO_K1(HW_BMFPGA_BASE)
	lw		t0,BMFPGA_REG_BOARD_STATUS_OFF(a0)
	and		t0,BMFPGA_BOARD_STATUS_CLK_MODE_MASK
	srl		t0,BMFPGA_BOARD_STATUS_CLK_MODE_SHIFT
	beq		t0,BMFPGA_BOARD_STATUS_CLK_MODE_125MHz,ssc_mem_125
	nop
ssc_mem_100:
    GT_ORVAL_REG_ASS(SDRAM_CONFIGURATION, SDRAM_CONFIG_HEAVYLOAD)
	GT_WRITE_REG_ASS(SDRAM_TIMING_PARAMETERS, 0x0000252a)
	b		ssc_mem_timing_done
	nop
ssc_mem_125:
	GT_WRITE_REG_ASS(SDRAM_TIMING_PARAMETERS, 0x0000272a)
	b		ssc_mem_timing_done
	nop

ssc_mem_timing_done:
	GT_WRITE_REG_ASS(PCI_0ARBITER_CONTROL, 0x80000000)
	GT_WRITE_REG_ASS(PCI_1ARBITER_CONTROL, 0x80000000)
	GT_WRITE_REG_ASS(PCI_0TIMEOUT_RETRY, 0x0040ffff)
	GT_WRITE_REG_ASS(PCI_1TIMEOUT_RETRY, 0x0040ffff)

	GT_WRITE_REG_ASS(DEVICE_BANK0PARAMETERS, 0xf009e6f0)
	GT_WRITE_REG_ASS(DEVICE_BANK1PARAMETERS, 0x146FFFFF)
	GT_WRITE_REG_ASS(DEVICE_BANK3PARAMETERS, 0x1449e6f0)

	GT_WRITE_REG_ASS(PCI_0BASE_ADDRESS_REGISTERS_ENABLE, 0x00000007)
	GT_WRITE_REG_ASS(PCI_1BASE_ADDRESS_REGISTERS_ENABLE, 0x00000007)

	la		a0,PHYS_TO_K1(HW_GALILEO_BASE)

	li		t1,(14<<3)|(13<<7)|(4<<11)|(7<<14)|(4<<17)|0x14400000
	sw		t1,DEVICE_BOOT_BANK_PARAMETERS(a0)
	sw		t1,DEVICE_BANK3PARAMETERS(a0)

	li		t0,HW_RTC_BASE
	srl		t0,20
	sw		t0,CS_0_LOW_DECODE_ADDRESS(a0)
	li		t0,HW_RTC_BASE+0x001fffff
	srl		t0,20
	sw		t0,CS_0_HIGH_DECODE_ADDRESS(a0)

	li		t0,HW_BMFPGA_NMI_CLEAR
	srl		t0,20
	sw		t0,CS_1_LOW_DECODE_ADDRESS(a0)
	li		t0,HW_BMFPGA_NMI_CLEAR+0x001fffff
	srl		t0,20
	sw		t0,CS_1_HIGH_DECODE_ADDRESS(a0)

	li		t0,HW_BIG_BOOTROM_BASE
	srl		t0,20
	sw		t0,CS_3_LOW_DECODE_ADDRESS(a0)
	li		t0,HW_BIG_BOOTROM_BASE+0x001fffff
	srl		t0,20
	sw		t0,CS_3_HIGH_DECODE_ADDRESS(a0)

/*
	jal		promio_init
	nop
*/

	DEBUG_STR("nfx-init\r\n")

    /* Turn off PCI_0 I/O space */
	GT_WRITE_REG_ASS(PCI_0I_O_HIGH_DECODE_ADDRESS, 0)
	GT_WRITE_REG_ASS(PCI_0I_O_LOW_DECODE_ADDRESS, 0x01000100)

    /* Turn off PCI_1 I/O space */
	GT_WRITE_REG_ASS(PCI_1I_O_HIGH_DECODE_ADDRESS, 0)
	GT_WRITE_REG_ASS(PCI_1I_O_LOW_DECODE_ADDRESS, 0x01000100)

    /* Turn off PCI_0 Mem 0 */
	GT_WRITE_REG_ASS(PCI_0MEMORY0_HIGH_DECODE_ADDRESS, 0)
	GT_WRITE_REG_ASS(PCI_0MEMORY0_LOW_DECODE_ADDRESS, 0x01000100)

    /* Turn off PCI_0 Mem 1 */
	GT_WRITE_REG_ASS(PCI_0MEMORY1_HIGH_DECODE_ADDRESS, 0)
	GT_WRITE_REG_ASS(PCI_0MEMORY1_LOW_DECODE_ADDRESS, 0x01000100)

    /* Turn off PCI_0 Mem 2 */
	GT_WRITE_REG_ASS(PCI_0MEMORY2_HIGH_DECODE_ADDRESS, 0)
	GT_WRITE_REG_ASS(PCI_0MEMORY2_LOW_DECODE_ADDRESS, 0x01000100)

    /* Turn off PCI_0 Mem 3 */
	GT_WRITE_REG_ASS(PCI_0MEMORY3_HIGH_DECODE_ADDRESS, 0)
	GT_WRITE_REG_ASS(PCI_0MEMORY3_LOW_DECODE_ADDRESS, 0x01000100)

    /* Turn off PCI_1 Mem 0 */
	GT_WRITE_REG_ASS(PCI_1MEMORY0_HIGH_DECODE_ADDRESS, 0)
	GT_WRITE_REG_ASS(PCI_1MEMORY0_LOW_DECODE_ADDRESS, 0x01000100)

    /* Turn off PCI_1 Mem 1 */
	GT_WRITE_REG_ASS(PCI_1MEMORY1_HIGH_DECODE_ADDRESS, 0)
	GT_WRITE_REG_ASS(PCI_1MEMORY1_LOW_DECODE_ADDRESS, 0x01000100)

    /* Turn off PCI_1 Mem 2 */
	GT_WRITE_REG_ASS(PCI_1MEMORY2_HIGH_DECODE_ADDRESS, 0)
	GT_WRITE_REG_ASS(PCI_1MEMORY2_LOW_DECODE_ADDRESS, 0x01000100)

    /* Turn off PCI_1 Mem 3 */
	GT_WRITE_REG_ASS(PCI_1MEMORY3_HIGH_DECODE_ADDRESS, 0)
	GT_WRITE_REG_ASS(PCI_1MEMORY3_LOW_DECODE_ADDRESS, 0x01000100)

    /* Turn on PCI_0 Mem 0 */
	GT_WRITE_REG_ASS(PCI_0MEMORY0_LOW_DECODE_ADDRESS, 0x01000100)
	GT_WRITE_REG_ASS(PCI_0MEMORY0_HIGH_DECODE_ADDRESS, 0x0000013f)
    /* Turn on PCI_0 Mem 1 */
	GT_WRITE_REG_ASS(PCI_0MEMORY1_LOW_DECODE_ADDRESS, 0x01000160)
	GT_WRITE_REG_ASS(PCI_0MEMORY1_HIGH_DECODE_ADDRESS, 0x0000016f)
    /* Turn on PCI_0 Mem 2 */
	GT_WRITE_REG_ASS(PCI_0MEMORY2_LOW_DECODE_ADDRESS, 0x01000158)
	GT_WRITE_REG_ASS(PCI_0MEMORY2_HIGH_DECODE_ADDRESS, 0x0000015f)

    /* Turn on PCI_1 Mem 0 */
	GT_WRITE_REG_ASS(PCI_1MEMORY0_LOW_DECODE_ADDRESS, 0x01000180)
	GT_WRITE_REG_ASS(PCI_1MEMORY0_HIGH_DECODE_ADDRESS, 0x0000019f)

	/* Turn on PCI_0 I/O */
	GT_WRITE_REG_ASS(PCI_0I_O_LOW_DECODE_ADDRESS, 0x01000150)
	GT_WRITE_REG_ASS(PCI_0I_O_HIGH_DECODE_ADDRESS, 0x00000157)
	GT_WRITE_REG_ASS(PCI_0I_O_ADDRESS_REMAP, 0)

	/* Turn off PCI CS access */
	GT_WRITE_REG_ASS(PCI_0CS_0_BANK_SIZE, 0)
	GT_WRITE_REG_ASS(PCI_0CS_1_BANK_SIZE, 0)
	GT_WRITE_REG_ASS(PCI_0CS_2_BANK_SIZE, 0)
	GT_WRITE_REG_ASS(PCI_0CS_3_BANK_SIZE, 0)
	GT_WRITE_REG_ASS(PCI_1CS_0_BANK_SIZE, 0)
	GT_WRITE_REG_ASS(PCI_1CS_1_BANK_SIZE, 0)
	GT_WRITE_REG_ASS(PCI_1CS_2_BANK_SIZE, 0)
	GT_WRITE_REG_ASS(PCI_1CS_3_BANK_SIZE, 0)
	GT_WRITE_REG_ASS(PCI_0CS_BOOT_BANK_SIZE, 0)
	GT_WRITE_REG_ASS(PCI_1CS_BOOT_BANK_SIZE, 0)

	/* Enable PCI access to SCS0, CS2, P2P mem 0 (PCI_0 only) */
	GT_WRITE_REG_ASS(PCI_0BASE_ADDRESS_REGISTERS_ENABLE, 0xFFFFF5BE)
	GT_WRITE_REG_ASS(PCI_1BASE_ADDRESS_REGISTERS_ENABLE, 0xFFFFFDFE)
	
	/* Set up PCI_0 CFG space, function 0 */
	GT_WRITE_REG_ASS(PCI_0CONFIGURATION_ADDRESS, 0x80000004)
	GT_WRITE_REG_ASS(PCI_0CONFIGURATION_DATA_VIRTUAL_REGISTER, 0xF3A00146)
	GT_WRITE_REG_ASS(PCI_0CONFIGURATION_ADDRESS, 0x8000000c)
	GT_WRITE_REG_ASS(PCI_0CONFIGURATION_DATA_VIRTUAL_REGISTER, 0x00800404)
	GT_WRITE_REG_ASS(PCI_0CONFIGURATION_ADDRESS, 0x80000010)
	GT_WRITE_REG_ASS(PCI_0CONFIGURATION_DATA_VIRTUAL_REGISTER, 0x80000008)
	GT_WRITE_REG_ASS(PCI_0CONFIGURATION_ADDRESS, 0x80000014)
	GT_WRITE_REG_ASS(PCI_0CONFIGURATION_DATA_VIRTUAL_REGISTER, 0x00000008)
	GT_WRITE_REG_ASS(PCI_0CONFIGURATION_ADDRESS, 0x80000018)
	GT_WRITE_REG_ASS(PCI_0CONFIGURATION_DATA_VIRTUAL_REGISTER, 0x00000008)
	GT_WRITE_REG_ASS(PCI_0CONFIGURATION_ADDRESS, 0x8000001c)
	GT_WRITE_REG_ASS(PCI_0CONFIGURATION_DATA_VIRTUAL_REGISTER, 0x00000008)

	/* Set up PCI_1 CFG command register, function 0 */
	GT_WRITE_REG_ASS(PCI_1CONFIGURATION_ADDRESS, 0x80000004)
	GT_WRITE_REG_ASS(PCI_1CONFIGURATION_DATA_VIRTUAL_REGISTER, 0xF3A00146)
	GT_WRITE_REG_ASS(PCI_1CONFIGURATION_ADDRESS, 0x8000000c)
	GT_WRITE_REG_ASS(PCI_1CONFIGURATION_DATA_VIRTUAL_REGISTER, 0x00800404)
	GT_WRITE_REG_ASS(PCI_1CONFIGURATION_ADDRESS, 0x80000010)
	GT_WRITE_REG_ASS(PCI_1CONFIGURATION_DATA_VIRTUAL_REGISTER, 0x80000008)
	GT_WRITE_REG_ASS(PCI_1CONFIGURATION_ADDRESS, 0x80000014)
	GT_WRITE_REG_ASS(PCI_1CONFIGURATION_DATA_VIRTUAL_REGISTER, 0x00000008)
	GT_WRITE_REG_ASS(PCI_1CONFIGURATION_ADDRESS, 0x80000018)
	GT_WRITE_REG_ASS(PCI_1CONFIGURATION_DATA_VIRTUAL_REGISTER, 0x00000008)
	GT_WRITE_REG_ASS(PCI_1CONFIGURATION_ADDRESS, 0x8000001c)
	GT_WRITE_REG_ASS(PCI_1CONFIGURATION_DATA_VIRTUAL_REGISTER, 0x00000008)

	/* Set up PCI_0 CFG space, function 1 */
	GT_WRITE_REG_ASS(PCI_0CONFIGURATION_ADDRESS, 0x80000118)
	GT_WRITE_REG_ASS(PCI_0CONFIGURATION_DATA_VIRTUAL_REGISTER, 0xA0000000)

	/* Set up PCI_0 CFG space, function 2 */
	GT_WRITE_REG_ASS(PCI_0CONFIGURATION_ADDRESS, 0x80000210)
	GT_WRITE_REG_ASS(PCI_0CONFIGURATION_DATA_VIRTUAL_REGISTER, 0xB0000008)

	/* Set up PCI command regs */
	GT_WRITE_REG_ASS(PCI_0COMMAND, 0x01079341)
	GT_WRITE_REG_ASS(PCI_1COMMAND, 0x01079341)
	 
#if 0
	/* Set up PCI accesses to GT memory */
	GT_WRITE_REG_ASS(PCI_0SCS_0_BANK_SIZE, 0x0ffff000)
	GT_WRITE_REG_ASS(PCI_0SCS_0_BASE_ADDRESS_REMAP, 0x0)
	GT_WRITE_REG_ASS(PCI_1SCS_0_BANK_SIZE, 0x0ffff000)
	GT_WRITE_REG_ASS(PCI_1SCS_0_BASE_ADDRESS_REMAP, 0x0)
#endif

	/* PR# 3367 Set up PCI accesses to FLASH_ROM 
           To allow the Toshiba to access the prom verseion string
           stored in the unused location
         */
	GT_WRITE_REG_ASS(PCI_0CS_3_BANK_SIZE, 0x001ff000)
	GT_WRITE_REG_ASS(PCI_0CS_BOOT_BANK_SIZE, 0x001ff000)
	GT_WRITE_REG_ASS(PCI_0CS_3_BASE_ADDRESS_REMAP, 0x1f600000)
	GT_WRITE_REG_ASS(PCI_0CS_BOOTCS_BASE_ADDRESS_REMAP, 0x1fc00000)
	GT_WRITE_REG_ASS(PCI_0BASE_ADDRESS_REGISTERS_ENABLE, 0xfffff43e)

	/* Set up PCI accesses to CS2 (BMFPGA) */
	GT_WRITE_REG_ASS(PCI_0CS_2_BANK_SIZE, 0x001ff000)
	GT_WRITE_REG_ASS(PCI_0CS_2_BASE_ADDRESS_REMAP, 0x1f400000)

	/* Set up PCI0 Mem 1 Remap (for 5600 access) */
	GT_WRITE_REG_ASS(PCI_0MEMORY1_ADDRESS_REMAP, 0x00000a60)

	/* Set up P2P access from PCI_0 to PCI_1 */
	/* Reducing from 64MB to 48 MB */
	GT_WRITE_REG_ASS(PCI_0P2P_MEM0_BAR_SIZE, 0x02fff000)
	GT_WRITE_REG_ASS(PCI_0P2P_MEM0_BASE_ADDRESS_REMAP_LOW, 0x18000000)

	/* Set up GPP pins */
	GT_WRITE_REG_ASS(MPP_CONTROL0, 0x00007777)
	GT_WRITE_REG_ASS(MPP_CONTROL1, 0x88880000)
	GT_WRITE_REG_ASS(GPP_IO_CONTROL, 0xFFFF5FF5)
	
	GT_WRITE_REG_ASS(GPP_VALUE, 0x40000000)

	/*
	 * Set up Ethernet PHY addresses
	 */
	mfc0	t0,C0_CONFIG
	nop
	nop
	nop
	and		t0,CFG_PIMASK
	srl		t0,CFG_PISHIFT
	and		t0,2
	beq		t0,2,ssc_old			/* new rev or old rev? */
	nop
ssc_new:
	/* phy addrs = 0, 1, 2 */
	GT_WRITE_REG_ASS(ETHERNET_PHY_ADDRESS_REGISTER,0x820)
	b		1f
	nop
ssc_old:
	/* use defaults, phy addr = 4, 5, 6 */
1:
	j		nfx_init_done
	nop

nfx_init_fc:
	/*
	 * move GT internal registers
	 */
	li	t0,HW_GALILEO_BASE
	srl	t0,20
	or	t0,0x01000000
	la	a0,0xb4000000
	sw	t0,0x68(a0)
	nop
	nop
	nop
	nop

    la      a0,PHYS_TO_K1(HW_GALILEO_BASE)
    lw      t0,0(a0)
    or      t0,0x00020000
    sw      t0,0(a0)  

	li	a0,PHYS_TO_K1(HW_GALILEO_BASE)
	lw	t0,GPP_VALUE(a0)
	srl	t0,25
	and	t0,3
	beq	t0,0,fc_slice0
	nop
fc_slice_not0:
	/* Set up GPP pins */
	GT_WRITE_REG_ASS(MPP_CONTROL0, 0x00008877)
    GT_WRITE_REG_ASS(MPP_CONTROL1, 0x00000099)
    GT_WRITE_REG_ASS(MPP_CONTROL2, 0x00000000)
    GT_WRITE_REG_ASS(MPP_CONTROL3, 0x00000000)
    GT_WRITE_REG_ASS(GPP_IO_CONTROL, 0xF90787F5)   

	la		a0,PHYS_TO_K1(HW_GALILEO_BASE)

#ifdef USE_DIAG_LED
    /* set all the led bits to 0, off or RED */
	lw		t0,GPP_VALUE(a0)
	and		t0,~GPP_FC_DIAG_LED_ALL_MASK
	sw		t0,GPP_VALUE(a0)

    /* lite up the cpuok to green */
    GT_ORVAL_REG_ASS(GPP_VALUE, GPP_FC_DIAG_LED_CPUOK_MASK)
#endif

	li		t1,(14<<3)|(13<<7)|(4<<11)|(7<<14)|(4<<17)|0x14400000
	sw		t1,DEVICE_BOOT_BANK_PARAMETERS(a0)
	sw		t1,DEVICE_BANK3PARAMETERS(a0)

	li		t0,HW_BIG_BOOTROM_BASE
	srl		t0,20
	sw		t0,CS_3_LOW_DECODE_ADDRESS(a0)
	li		t0,HW_BIG_BOOTROM_BASE+0x001fffff
	srl		t0,20
	sw		t0,CS_3_HIGH_DECODE_ADDRESS(a0)

	la		a0,PHYS_TO_K1(HW_GALILEO_BASE)
	lw		t0,GPP_VALUE(a0)
	and		t0,GPP_FC_REV_MASK
	srl		t0,GPP_FC_REV_SHIFT
	beqz	t0,fc123_new_board
	nop
fc123_old_board:
	mfc0    t0,C0_CONFIG
	nop
	nop
	nop
	and     t0,CFG_PIMASK
	srl     t0,CFG_PISHIFT
	and     t0,2
	beq     t0,0,fc123_old_init_mem100
	nop

fc123_old_init_mem125:
	GT_WRITE_REG_ASS(SDRAM_CONFIGURATION, 0xd8c20578)
	GT_WRITE_REG_ASS(SDRAM_TIMING_PARAMETERS, 0x0000272a)
	b		fc123_mem_config_done
	nop
fc123_old_init_mem100:
	GT_WRITE_REG_ASS(SDRAM_CONFIGURATION, 0xd8c00578)
    GT_ORVAL_REG_ASS(SDRAM_CONFIGURATION, SDRAM_CONFIG_HEAVYLOAD)
	GT_WRITE_REG_ASS(SDRAM_TIMING_PARAMETERS, 0x0000252a)
	b		fc123_mem_config_done
	nop

fc123_new_board:
	/* based on jmpr, load from DIAG partition */
	la		a0,PHYS_TO_K1(HW_GALILEO_BASE)
	lw		t0,GPP_VALUE(a0)
	and		t0,0x00080000
	srl		t0,19
	bne		t0,1,2f
	nop
	j		0xbfcc0000
	nop

2:
	#Jump to DIAG partition-based on env variables:QT
	li		a0,0xAE
   	li		a1,0x88	
   	jal     	gt_read_spd_byte /* read the 136th byte */
	nop
	nop
   	move    	t0,v0
	beq		t0,0,2f
	nop
	beq		t0,0xff,2f
	nop
	j		0xbfcc0000
	nop

2:
	#Jump to DIAG partition-based on env variables:FT
	li		a0,0xAE
    	li		a1,0x89	
    	jal     	gt_read_spd_byte /* read the 137th byte */
	nop
	nop
    	move    	t0,v0
	beq		t0,0,2f
	nop
	beq		t0,0xff,2f
	nop
	j		0xbfcc0000
	nop

2:
	#Jump to DIAG partition-based on env variables:DIAG
	li		a0,0xAE
    	li		a1,0x8a	
    	jal     	gt_read_spd_byte /* read the 138th byte */
	nop
	nop
    	move    	t0,v0
	beq		t0,0,2f
	nop
	beq		t0,0xff,2f
	nop
	j		0xbfcc0000
	nop

2:
	la		a0,PHYS_TO_K1(HW_GALILEO_BASE)
	lw		t0,GPP_VALUE(a0)
	and		t0,GPP_FC123_MEM_TYPE_MASK
	srl		t0,GPP_FC123_MEM_TYPE_SHIFT
	beq		t0,GPP_FC123_MEM_TYPE_UNREG,fc123_mem_unreg
	nop
fc123_mem_reg:
	GT_WRITE_REG_ASS(SDRAM_CONFIGURATION, 0xd8c20578)
	b		fc123_new_board_cont
	nop
fc123_mem_unreg:
	GT_WRITE_REG_ASS(SDRAM_CONFIGURATION, 0xd8c00578)

fc123_new_board_cont:
	la		a0,PHYS_TO_K1(HW_GALILEO_BASE)
	lw		t0,GPP_VALUE(a0)
	and		t0,GPP_FC123_CLK_MODE_MASK
	srl		t0,GPP_FC123_CLK_MODE_SHIFT
	beq		t0,GPP_FC123_CLK_MODE_125MHz,fc123_new_init_mem125
	nop

fc123_new_init_mem100:
    GT_ORVAL_REG_ASS(SDRAM_CONFIGURATION, SDRAM_CONFIG_HEAVYLOAD)
	GT_WRITE_REG_ASS(SDRAM_TIMING_PARAMETERS, 0x0000252a)
	b		fc123_mem_config_done
	nop

fc123_new_init_mem125:
	GT_WRITE_REG_ASS(SDRAM_TIMING_PARAMETERS, 0x0000272a)

fc123_mem_config_done:
	b	fc_cont
	nop

fc_slice0:
	/* Set up GPP pins */
	GT_WRITE_REG_ASS(MPP_CONTROL0, 0x00888877)
    GT_WRITE_REG_ASS(MPP_CONTROL1, 0x00000099)
    GT_WRITE_REG_ASS(MPP_CONTROL2, 0x00000000)
    GT_WRITE_REG_ASS(MPP_CONTROL3, 0x00000000)
    GT_WRITE_REG_ASS(GPP_IO_CONTROL, 0xF80007D5)   

	la		a0,PHYS_TO_K1(HW_GALILEO_BASE)

#ifdef USE_DIAG_LED
    /* set all diag leds to off or RED */ 
	lw		t0,GPP_VALUE(a0)
	and		t0,~GPP_FC_DIAG_LED_ALL_MASK
	sw		t0,GPP_VALUE(a0)

    /* lite up the cpuok to green */
    GT_ORVAL_REG_ASS(GPP_VALUE, GPP_FC_DIAG_LED_CPUOK_MASK)
#endif

	li		t1,(14<<3)|(13<<7)|(4<<11)|(7<<14)|(4<<17)|0x14400000
	sw		t1,DEVICE_BOOT_BANK_PARAMETERS(a0)
	sw		t1,DEVICE_BANK3PARAMETERS(a0)

	li		t0,HW_BMFPGA_NMI_CLEAR
	srl		t0,20
	sw		t0,CS_1_LOW_DECODE_ADDRESS(a0)
	li		t0,HW_BMFPGA_NMI_CLEAR+0x001fffff
	srl		t0,20
	sw		t0,CS_1_HIGH_DECODE_ADDRESS(a0)

	li		t0,HW_BMFPGA_BASE
	srl		t0,20
	sw		t0,CS_2_LOW_DECODE_ADDRESS(a0)
	li		t0,HW_BMFPGA_BASE+0x001fffff
	srl		t0,20
	sw		t0,CS_2_HIGH_DECODE_ADDRESS(a0)

    /* config and init the gpp/ctl bits */
    /* set gpp init bits with all Front panel led at flt=red ok=off*/ 
	la		t0,PHYS_TO_K1(HW_BMFPGA_BASE)
    /* set dir 0=out 1=in */
    li      t1,0x0f000026
    sw      t1,BMFPGA_REG_GPP_CTL_DIR_OFF(t0)
    li      t1,0x0f00e1ff
    li      t2,(BMFPGA_CTRL_FP_LED_FLT_RED | BMFPGA_CTRL_FP_LED_OK_OFF)
    or      t1,t2
    sw      t1,BMFPGA_REG_GPP_CTL_VAL_OFF(t0)

	li		t0,HW_BIG_BOOTROM_BASE
	srl		t0,20
	sw		t0,CS_3_LOW_DECODE_ADDRESS(a0)
	li		t0,HW_BIG_BOOTROM_BASE+0x001fffff
	srl		t0,20
	sw		t0,CS_3_HIGH_DECODE_ADDRESS(a0)

	lw		t0,GPP_VALUE(a0)
	and		t0,GPP_FC_REV_MASK
	srl		t0,GPP_FC_REV_SHIFT
	beqz	t0,fc0_new_board
	nop
fc0_old_board:
	mfc0    t0,C0_CONFIG
	nop
	nop
	nop
	and     t0,CFG_PIMASK
	srl     t0,CFG_PISHIFT
	and     t0,2
	beq     t0,0,fc0_old_init_mem100
	nop

fc0_old_init_mem125:
	GT_WRITE_REG_ASS(SDRAM_CONFIGURATION, 0xd8c20578)
	GT_WRITE_REG_ASS(SDRAM_TIMING_PARAMETERS, 0x0000272a)
	b		fc0_mem_config_done
	nop
fc0_old_init_mem100:
	GT_WRITE_REG_ASS(SDRAM_CONFIGURATION, 0xd8c00578)
    GT_ORVAL_REG_ASS(SDRAM_CONFIGURATION, SDRAM_CONFIG_HEAVYLOAD)
	GT_WRITE_REG_ASS(SDRAM_TIMING_PARAMETERS, 0x0000252a)
	b		fc0_mem_config_done
	nop

fc0_new_board:
	#Jump to DIAG partition-based on Diag Jmpr
	la		a0,PHYS_TO_K1(HW_BMFPGA_BASE)
	lw		t0,0(a0)
	and		t0,0x00002000
	srl		t0,13
	bne		t0,1,2f
	nop
	j		0xbfcc0000
	nop

2:
	#Jump to DIAG partition-based on env variables:QT
	li		a0,0xAE
   	li		a1,0x88	
   	jal     	gt_read_spd_byte /* read the 136th byte */
	nop
	nop
   	move    	t0,v0
	beq		t0,0,2f
	nop
	beq		t0,0xff,2f
	nop
	j		0xbfcc0000
	nop

2:
	#Jump to DIAG partition-based on env variables:FT
	li		a0,0xAE
    	li		a1,0x89	
    	jal     	gt_read_spd_byte /* read the 137th byte */
	nop
	nop
    	move    	t0,v0
	beq		t0,0,2f
	nop
	beq		t0,0xff,2f
	nop
	j		0xbfcc0000
	nop

2:
	#Jump to DIAG partition-based on env variables:DIAG
	li		a0,0xAE
    	li		a1,0x8a	
    	jal     	gt_read_spd_byte /* read the 138th byte */
	nop
	nop
    	move    	t0,v0
	beq		t0,0,2f
	nop
	beq		t0,0xff,2f
	nop
	j		0xbfcc0000
	nop

2:
	la		a0,PHYS_TO_K1(HW_BMFPGA_BASE)
	lw		t0,BMFPGA_REG_BOARD_STATUS_OFF(a0)

	and		t0,BMFPGA_BOARD_STATUS_MEM_TYPE_MASK
	srl		t0,BMFPGA_BOARD_STATUS_MEM_TYPE_SHIFT
	beq		t0,BMFPGA_BOARD_STATUS_MEM_TYPE_UNREG,fc0_mem_unreg
	nop

fc0_mem_reg:
	GT_WRITE_REG_ASS(SDRAM_CONFIGURATION, 0xd8c20578)
	b		fc0_new_board_cont
	nop
fc0_mem_unreg:
	GT_WRITE_REG_ASS(SDRAM_CONFIGURATION, 0xd8c00578)

fc0_new_board_cont:
	la		a0,PHYS_TO_K1(HW_BMFPGA_BASE)
	lw		t0,BMFPGA_REG_BOARD_STATUS_OFF(a0)
	and		t0,BMFPGA_BOARD_STATUS_CLK_MODE_MASK
	srl		t0,BMFPGA_BOARD_STATUS_CLK_MODE_SHIFT
	beq		t0,BMFPGA_BOARD_STATUS_CLK_MODE_125MHz,fc0_new_init_mem125
	nop

fc0_new_init_mem100:
    GT_ORVAL_REG_ASS(SDRAM_CONFIGURATION, SDRAM_CONFIG_HEAVYLOAD)
	GT_WRITE_REG_ASS(SDRAM_TIMING_PARAMETERS, 0x0000252a)
	b		fc0_mem_config_done
	nop

fc0_new_init_mem125:
	GT_WRITE_REG_ASS(SDRAM_TIMING_PARAMETERS, 0x0000272a)

fc0_mem_config_done:
	b	fc_cont
	nop

fc_cont:
	GT_WRITE_REG_ASS(PCI_0ARBITER_CONTROL, 0x801f4000)
	GT_WRITE_REG_ASS(PCI_1ARBITER_CONTROL, 0x801ec000)
	GT_WRITE_REG_ASS(PCI_0TIMEOUT_RETRY, 0x0040ffff)
	GT_WRITE_REG_ASS(PCI_1TIMEOUT_RETRY, 0x0040ffff)

#if 0
	/* Enable ECC interrupt */
    GT_WRITE_REG_ASS(SDRAM_ECC_CONTROL, 0x00010000)
#endif

	 /* Enable Memory Prefetch. Frank Add 4-3-2002 */
    GT_WRITE_REG_ASS(PCI_0ACCESS_CONTROL_BASE_0_LOW, 0x01001000)
    GT_WRITE_REG_ASS(PCI_0ACCESS_CONTROL_TOP_0, 0x00000FFF)
    GT_WRITE_REG_ASS(PCI_1ACCESS_CONTROL_BASE_0_LOW, 0x01001000)
    GT_WRITE_REG_ASS(PCI_1ACCESS_CONTROL_TOP_0, 0x00000FFF)


    /* if heavyload bit is on skip aggressive prefetch */
    GT_REGV_READ_ASS(SDRAM_CONFIGURATION)
    and     v0,SDRAM_CONFIG_HEAVYLOAD
    beq     v0,SDRAM_CONFIG_HEAVYLOAD,no_aggressive_prefetch
    nop

	 /* Set SDRAM Crossbar control. Frank Add 8-20-2002 */
    GT_WRITE_REG_ASS(SDRAM_CROSS_BAR_CONTROL_LOW, 0x43765432)
    GT_WRITE_REG_ASS(SDRAM_CROSS_BAR_CONTROL_HIGH, 0x43765432)	
	
    GT_WRITE_REG_ASS(PCI_0ACCESS_CONTROL_BASE_0_LOW, 0x01271000)
    GT_WRITE_REG_ASS(PCI_1ACCESS_CONTROL_BASE_0_LOW, 0x01271000)

no_aggressive_prefetch:

	GT_WRITE_REG_ASS(DEVICE_BANK0PARAMETERS, 0x1FCFFFFF)
	GT_WRITE_REG_ASS(DEVICE_BANK1PARAMETERS, 0x146FFFFF)
	GT_WRITE_REG_ASS(DEVICE_BANK2PARAMETERS, 0xf226e87f)

	GT_WRITE_REG_ASS(PCI_0BASE_ADDRESS_REGISTERS_ENABLE, 0x00000007)
	GT_WRITE_REG_ASS(PCI_1BASE_ADDRESS_REGISTERS_ENABLE, 0x00000007)


/*
	jal		promio_init
	nop
*/

	DEBUG_STR("nfx-init\r\n")

    /* Turn off PCI_0 I/O space */
	GT_WRITE_REG_ASS(PCI_0I_O_HIGH_DECODE_ADDRESS, 0)
	GT_WRITE_REG_ASS(PCI_0I_O_LOW_DECODE_ADDRESS, 0x01000100)

    /* Turn off PCI_1 I/O space */
	GT_WRITE_REG_ASS(PCI_1I_O_HIGH_DECODE_ADDRESS, 0)
	GT_WRITE_REG_ASS(PCI_1I_O_LOW_DECODE_ADDRESS, 0x01000100)

    /* Turn off PCI_0 Mem 0 */
	GT_WRITE_REG_ASS(PCI_0MEMORY0_HIGH_DECODE_ADDRESS, 0)
	GT_WRITE_REG_ASS(PCI_0MEMORY0_LOW_DECODE_ADDRESS, 0x01000100)

    /* Turn off PCI_0 Mem 1 */
	GT_WRITE_REG_ASS(PCI_0MEMORY1_HIGH_DECODE_ADDRESS, 0)
	GT_WRITE_REG_ASS(PCI_0MEMORY1_LOW_DECODE_ADDRESS, 0x01000100)

    /* Turn off PCI_0 Mem 2 */
	GT_WRITE_REG_ASS(PCI_0MEMORY2_HIGH_DECODE_ADDRESS, 0)
	GT_WRITE_REG_ASS(PCI_0MEMORY2_LOW_DECODE_ADDRESS, 0x01000100)

    /* Turn off PCI_0 Mem 3 */
	GT_WRITE_REG_ASS(PCI_0MEMORY3_HIGH_DECODE_ADDRESS, 0)
	GT_WRITE_REG_ASS(PCI_0MEMORY3_LOW_DECODE_ADDRESS, 0x01000100)

    /* Turn off PCI_1 Mem 0 */
	GT_WRITE_REG_ASS(PCI_1MEMORY0_HIGH_DECODE_ADDRESS, 0)
	GT_WRITE_REG_ASS(PCI_1MEMORY0_LOW_DECODE_ADDRESS, 0x01000100)

    /* Turn off PCI_1 Mem 1 */
	GT_WRITE_REG_ASS(PCI_1MEMORY1_HIGH_DECODE_ADDRESS, 0)
	GT_WRITE_REG_ASS(PCI_1MEMORY1_LOW_DECODE_ADDRESS, 0x01000100)

    /* Turn off PCI_1 Mem 2 */
	GT_WRITE_REG_ASS(PCI_1MEMORY2_HIGH_DECODE_ADDRESS, 0)
	GT_WRITE_REG_ASS(PCI_1MEMORY2_LOW_DECODE_ADDRESS, 0x01000100)

    /* Turn off PCI_1 Mem 3 */
	GT_WRITE_REG_ASS(PCI_1MEMORY3_HIGH_DECODE_ADDRESS, 0)
	GT_WRITE_REG_ASS(PCI_1MEMORY3_LOW_DECODE_ADDRESS, 0x01000100)

    /* Turn on PCI_0 Mem 0 */
	GT_WRITE_REG_ASS(PCI_0MEMORY0_LOW_DECODE_ADDRESS, 0x01000300)
	GT_WRITE_REG_ASS(PCI_0MEMORY0_HIGH_DECODE_ADDRESS, 0x0000037f)
    /* Turn on PCI_1 Mem 0 */
	GT_WRITE_REG_ASS(PCI_1MEMORY0_LOW_DECODE_ADDRESS, 0x01000380)
	GT_WRITE_REG_ASS(PCI_1MEMORY0_HIGH_DECODE_ADDRESS, 0x000003ff)

	/* Turn off PCI CS access */
	GT_WRITE_REG_ASS(PCI_0CS_0_BANK_SIZE, 0)
	GT_WRITE_REG_ASS(PCI_0CS_1_BANK_SIZE, 0)
	GT_WRITE_REG_ASS(PCI_0CS_2_BANK_SIZE, 0)
	GT_WRITE_REG_ASS(PCI_0CS_3_BANK_SIZE, 0)
	GT_WRITE_REG_ASS(PCI_1CS_0_BANK_SIZE, 0)
	GT_WRITE_REG_ASS(PCI_1CS_1_BANK_SIZE, 0)
	GT_WRITE_REG_ASS(PCI_1CS_2_BANK_SIZE, 0)
	GT_WRITE_REG_ASS(PCI_1CS_3_BANK_SIZE, 0)
	GT_WRITE_REG_ASS(PCI_0CS_BOOT_BANK_SIZE, 0)
	GT_WRITE_REG_ASS(PCI_1CS_BOOT_BANK_SIZE, 0)

	/* Enable PCI access to SCS0 */
	GT_WRITE_REG_ASS(PCI_0BASE_ADDRESS_REGISTERS_ENABLE, 0xFFFFFDFE)
	GT_WRITE_REG_ASS(PCI_1BASE_ADDRESS_REGISTERS_ENABLE, 0xFFFFFDFE)
	
	/* Set up PCI command regs */
	GT_WRITE_REG_ASS(PCI_0COMMAND, 0x01079341)
	GT_WRITE_REG_ASS(PCI_1COMMAND, 0x01079341)

	/* Set up PCI_0 CFG space */
	GT_WRITE_REG_ASS(PCI_0CONFIGURATION_ADDRESS, 0x80000004)
	GT_WRITE_REG_ASS(PCI_0CONFIGURATION_DATA_VIRTUAL_REGISTER, 0xF3A00146)
	GT_WRITE_REG_ASS(PCI_0CONFIGURATION_ADDRESS, 0x8000000c)
	GT_WRITE_REG_ASS(PCI_0CONFIGURATION_DATA_VIRTUAL_REGISTER, 0x00800404)
	GT_WRITE_REG_ASS(PCI_0CONFIGURATION_ADDRESS, 0x80000010)
	GT_WRITE_REG_ASS(PCI_0CONFIGURATION_DATA_VIRTUAL_REGISTER, 0x00000008)
	GT_WRITE_REG_ASS(PCI_0CONFIGURATION_ADDRESS, 0x80000014)
	GT_WRITE_REG_ASS(PCI_0CONFIGURATION_DATA_VIRTUAL_REGISTER, 0x00000008)
	GT_WRITE_REG_ASS(PCI_0CONFIGURATION_ADDRESS, 0x80000018)
	GT_WRITE_REG_ASS(PCI_0CONFIGURATION_DATA_VIRTUAL_REGISTER, 0x00000008)
	GT_WRITE_REG_ASS(PCI_0CONFIGURATION_ADDRESS, 0x8000001c)
	GT_WRITE_REG_ASS(PCI_0CONFIGURATION_DATA_VIRTUAL_REGISTER, 0x00000008)

	/* Set up PCI_1 CFG command register */
	GT_WRITE_REG_ASS(PCI_1CONFIGURATION_ADDRESS, 0x80000004)
	GT_WRITE_REG_ASS(PCI_1CONFIGURATION_DATA_VIRTUAL_REGISTER, 0xF3A00146)
	GT_WRITE_REG_ASS(PCI_1CONFIGURATION_ADDRESS, 0x8000000c)
	GT_WRITE_REG_ASS(PCI_1CONFIGURATION_DATA_VIRTUAL_REGISTER, 0x00800404)
	GT_WRITE_REG_ASS(PCI_1CONFIGURATION_ADDRESS, 0x80000010)
	GT_WRITE_REG_ASS(PCI_1CONFIGURATION_DATA_VIRTUAL_REGISTER, 0x00000008)
	GT_WRITE_REG_ASS(PCI_1CONFIGURATION_ADDRESS, 0x80000014)
	GT_WRITE_REG_ASS(PCI_1CONFIGURATION_DATA_VIRTUAL_REGISTER, 0x00000008)
	GT_WRITE_REG_ASS(PCI_1CONFIGURATION_ADDRESS, 0x80000018)
	GT_WRITE_REG_ASS(PCI_1CONFIGURATION_DATA_VIRTUAL_REGISTER, 0x00000008)
	GT_WRITE_REG_ASS(PCI_1CONFIGURATION_ADDRESS, 0x8000001c)
	GT_WRITE_REG_ASS(PCI_1CONFIGURATION_DATA_VIRTUAL_REGISTER, 0x00000008)

#if 0
	/*
 	 * For FCNIM, set up PCI to SCS size = 256 MB
 	 */
    GT_WRITE_REG_ASS(PCI_0SCS_0_BANK_SIZE, 0x0FFFF000)
    GT_WRITE_REG_ASS(PCI_1SCS_0_BANK_SIZE, 0x0FFFF000)
#endif

#ifdef USE_DIAG_LED
    /* diag leds */
    GT_ORVAL_REG_ASS(GPP_VALUE, GPP_FC_DIAG_LED_ERR_MASK)
#endif

	j		nfx_init_done
	nop

#endif	/* AGILE_HARDWARE */


#ifdef EVAL_BOARD

	GT_WRITE_REG_ASS(DEVICE_BANK0PARAMETERS, 0x144FFFFF)
	GT_WRITE_REG_ASS(DEVICE_BANK1PARAMETERS, 0x146FFFFF)
	GT_WRITE_REG_ASS(DEVICE_BANK2PARAMETERS, 0x144FFFFF)
	GT_WRITE_REG_ASS(DEVICE_BANK3PARAMETERS, 0x167FFFFF)

	GT_WRITE_REG_ASS(PCI_0TIMEOUT_RETRY, 0x0000FFFF)
	GT_WRITE_REG_ASS(PCI_1TIMEOUT_RETRY, 0x0000FFFF)

	GT_WRITE_REG_ASS(PCI_0BASE_ADDRESS_REGISTERS_ENABLE, 0x00000007)
	GT_WRITE_REG_ASS(PCI_1BASE_ADDRESS_REGISTERS_ENABLE, 0x00000007)

	la		a0,0xBC808000		/* LED 0 */
	sb		zero,0(a0)

	la		a0,PHYS_TO_K1(HW_GALILEO_BASE)

	lw		t0,DEVICE_BOOT_BANK_PARAMETERS(a0)
	li		t1,(14<<3)|(13<<7)|(4<<11)|(7<<14)|(4<<17)|0x14400000
	and		t0,(3<<20)
	or		t1,t0
	li		t2,(1<<20)
	beqz	t0,bootis8
	sw		t1,DEVICE_BOOT_BANK_PARAMETERS(a0)

	li		t2,(0<<20)
bootis8:
	lw		t0,DEVICE_BANK3PARAMETERS(a0)
	and		t0,0x3d400000
	or		t0,t2
	li		t1,(14<<3)|(13<<7)|(4<<11)|(7<<14)|(7<<17)
	or		t0,t1
	lw		t0,DEVICE_BANK3PARAMETERS(a0)


	la		a0,0xBC80C000		/* LED 1 */
	sb		zero,0(a0)

	jal		promio_init
	nop

	DEBUG_STR("nfx-init\r\n")

    /* Turn off PCI_0 I/O space */
	GT_WRITE_REG_ASS(PCI_0I_O_HIGH_DECODE_ADDRESS, 0)
	GT_WRITE_REG_ASS(PCI_0I_O_LOW_DECODE_ADDRESS, 0x01000100)

    /* Turn off PCI_1 I/O space */
	GT_WRITE_REG_ASS(PCI_1I_O_HIGH_DECODE_ADDRESS, 0)
	GT_WRITE_REG_ASS(PCI_1I_O_LOW_DECODE_ADDRESS, 0x01000100)

    /* Turn off PCI_0 Mem 0 */
	GT_WRITE_REG_ASS(PCI_0MEMORY0_HIGH_DECODE_ADDRESS, 0)
	GT_WRITE_REG_ASS(PCI_0MEMORY0_LOW_DECODE_ADDRESS, 0x01000100)

    /* Turn off PCI_0 Mem 1 */
	GT_WRITE_REG_ASS(PCI_0MEMORY1_HIGH_DECODE_ADDRESS, 0)
	GT_WRITE_REG_ASS(PCI_0MEMORY1_LOW_DECODE_ADDRESS, 0x01000100)

    /* Turn off PCI_0 Mem 2 */
	GT_WRITE_REG_ASS(PCI_0MEMORY2_HIGH_DECODE_ADDRESS, 0)
	GT_WRITE_REG_ASS(PCI_0MEMORY2_LOW_DECODE_ADDRESS, 0x01000100)

    /* Turn off PCI_0 Mem 3 */
	GT_WRITE_REG_ASS(PCI_0MEMORY3_HIGH_DECODE_ADDRESS, 0)
	GT_WRITE_REG_ASS(PCI_0MEMORY3_LOW_DECODE_ADDRESS, 0x01000100)

    /* Turn off PCI_1 Mem 0 */
	GT_WRITE_REG_ASS(PCI_1MEMORY0_HIGH_DECODE_ADDRESS, 0)
	GT_WRITE_REG_ASS(PCI_1MEMORY0_LOW_DECODE_ADDRESS, 0x01000100)

    /* Turn off PCI_1 Mem 1 */
	GT_WRITE_REG_ASS(PCI_1MEMORY1_HIGH_DECODE_ADDRESS, 0)
	GT_WRITE_REG_ASS(PCI_1MEMORY1_LOW_DECODE_ADDRESS, 0x01000100)

    /* Turn off PCI_1 Mem 2 */
	GT_WRITE_REG_ASS(PCI_1MEMORY2_HIGH_DECODE_ADDRESS, 0)
	GT_WRITE_REG_ASS(PCI_1MEMORY2_LOW_DECODE_ADDRESS, 0x01000100)

    /* Turn off PCI_1 Mem 3 */
	GT_WRITE_REG_ASS(PCI_1MEMORY3_HIGH_DECODE_ADDRESS, 0)
	GT_WRITE_REG_ASS(PCI_1MEMORY3_LOW_DECODE_ADDRESS, 0x01000100)

    /* Turn on PCI_0 Mem 0 */
	GT_WRITE_REG_ASS(PCI_0MEMORY0_LOW_DECODE_ADDRESS, 0x01000120)
	GT_WRITE_REG_ASS(PCI_0MEMORY0_HIGH_DECODE_ADDRESS, 0x0000013f)

    /* Turn on PCI_1 Mem 0 */
	GT_WRITE_REG_ASS(PCI_1MEMORY0_LOW_DECODE_ADDRESS, 0x01000160)
	GT_WRITE_REG_ASS(PCI_1MEMORY0_HIGH_DECODE_ADDRESS, 0x0000017f)

	/* Turn off PCI CS access */
	GT_WRITE_REG_ASS(PCI_0CS_0_BANK_SIZE, 0)
	GT_WRITE_REG_ASS(PCI_0CS_1_BANK_SIZE, 0)
	GT_WRITE_REG_ASS(PCI_0CS_2_BANK_SIZE, 0)
	GT_WRITE_REG_ASS(PCI_0CS_3_BANK_SIZE, 0)
	GT_WRITE_REG_ASS(PCI_1CS_0_BANK_SIZE, 0)
	GT_WRITE_REG_ASS(PCI_1CS_1_BANK_SIZE, 0)
	GT_WRITE_REG_ASS(PCI_1CS_2_BANK_SIZE, 0)
	GT_WRITE_REG_ASS(PCI_1CS_3_BANK_SIZE, 0)
	GT_WRITE_REG_ASS(PCI_0CS_BOOT_BANK_SIZE, 0)
	GT_WRITE_REG_ASS(PCI_1CS_BOOT_BANK_SIZE, 0)

	/* Enable PCI access to SCS0 */
	GT_WRITE_REG_ASS(PCI_0BASE_ADDRESS_REGISTERS_ENABLE, 0xFFFFFDFC)
	GT_WRITE_REG_ASS(PCI_1BASE_ADDRESS_REGISTERS_ENABLE, 0xFFFFFDFC)
	
	/* Set up slave window sizes */
	GT_WRITE_REG_ASS(PCI_0SCS_0_BANK_SIZE, 0x01FFF000);
	GT_WRITE_REG_ASS(PCI_1SCS_0_BANK_SIZE, 0x01FFF000);
	
	/* Set up PCI_0 CFG space */
	GT_WRITE_REG_ASS(PCI_0CONFIGURATION_ADDRESS, 0x80000004)
	GT_WRITE_REG_ASS(PCI_0CONFIGURATION_DATA_VIRTUAL_REGISTER, 0xF3A00146)
	GT_WRITE_REG_ASS(PCI_0CONFIGURATION_ADDRESS, 0x8000000c)
	GT_WRITE_REG_ASS(PCI_0CONFIGURATION_DATA_VIRTUAL_REGISTER, 0x00800404)
	GT_WRITE_REG_ASS(PCI_0CONFIGURATION_ADDRESS, 0x80000010)
	GT_WRITE_REG_ASS(PCI_0CONFIGURATION_DATA_VIRTUAL_REGISTER, 0x00000008)
	GT_WRITE_REG_ASS(PCI_0CONFIGURATION_ADDRESS, 0x80000014)
	GT_WRITE_REG_ASS(PCI_0CONFIGURATION_DATA_VIRTUAL_REGISTER, 0x00000008)
	GT_WRITE_REG_ASS(PCI_0CONFIGURATION_ADDRESS, 0x80000018)
	GT_WRITE_REG_ASS(PCI_0CONFIGURATION_DATA_VIRTUAL_REGISTER, 0x00000008)
	GT_WRITE_REG_ASS(PCI_0CONFIGURATION_ADDRESS, 0x8000001c)
	GT_WRITE_REG_ASS(PCI_0CONFIGURATION_DATA_VIRTUAL_REGISTER, 0x00000008)

	/* Set up PCI_1 CFG command register */
	GT_WRITE_REG_ASS(PCI_1CONFIGURATION_ADDRESS, 0x80000004)
	GT_WRITE_REG_ASS(PCI_1CONFIGURATION_DATA_VIRTUAL_REGISTER, 0xF3A00146)
	GT_WRITE_REG_ASS(PCI_1CONFIGURATION_ADDRESS, 0x8000000c)
	GT_WRITE_REG_ASS(PCI_1CONFIGURATION_DATA_VIRTUAL_REGISTER, 0x00800404)
	GT_WRITE_REG_ASS(PCI_1CONFIGURATION_ADDRESS, 0x80000010)
	GT_WRITE_REG_ASS(PCI_1CONFIGURATION_DATA_VIRTUAL_REGISTER, 0x00000008)
	GT_WRITE_REG_ASS(PCI_1CONFIGURATION_ADDRESS, 0x80000014)
	GT_WRITE_REG_ASS(PCI_1CONFIGURATION_DATA_VIRTUAL_REGISTER, 0x00000008)
	GT_WRITE_REG_ASS(PCI_1CONFIGURATION_ADDRESS, 0x80000018)
	GT_WRITE_REG_ASS(PCI_1CONFIGURATION_DATA_VIRTUAL_REGISTER, 0x00000008)
	GT_WRITE_REG_ASS(PCI_1CONFIGURATION_ADDRESS, 0x8000001c)
	GT_WRITE_REG_ASS(PCI_1CONFIGURATION_DATA_VIRTUAL_REGISTER, 0x00000008)

	j		nfx_init_done
	nop

#endif	/* EVAL_BOARD */

	.end	nfx_init

#endif	/* PROM */
