/*
 *-----------------------------------------------------------------
 *
 * Name:	sbdreset.S
 *
 * RCS:  $Id: sbdreset.S,v 1.8 2002/09/30 22:56:09 johnh Exp $
 *
 * Copyright (C) 2001, Agile Storage, Inc.
 *
 * Description:	low level code for GT 64240 board
 *
 * Created by:	Rick Lund
 *
 * Date Created:	9/21/01
 *
 *-----------------------------------------------------------------
 */
#ifdef PROM

#include <nfx-incl.h>
#include <sys/mips.h>
#include <exports/hwMap.h>
#include "../prom-pmon/pmon.h"

#include <exports/promio-api.h>
#include "sbd.h"
#include <sys/gt64240r.h>
#include <exports/gpp-defs.h>


#if !__mips64 && __mips >= 3 && __mips != 32
	/* force 64-bit register support */
#undef __mips64	
#define __mips64 1	
	.set	gp64
#endif
	
#if RAL_XXX
#define DISPLAY(d0,d1,d2,d3)			\
	li	ARG1,d0;						\
	jal	promio_put_char;				\
	li	ARG1,d1;						\
	jal	promio_put_char;				\
	li	ARG1,d2;						\
	jal	promio_put_char;				\
	li	ARG1,d3;						\
	jal	promio_put_char
#else
#define DISPLAY(d0,d1,d2,d3)
#endif

#define MEG	0x100000		

	.globl	nfx_init_done
	.globl	nsdb_done

/*
 * Basic board initialisation, called straight from RESET.
 * It is only called if this program is built for ROM.  
 * It can use any registers except s8, k0 and k1.
 *
 * Note that s8 holds a "relocation factor" (see ../share/romlow.sx)
 * which must be added to any address before it is used.  This
 * is to support relocatable roms.
 */
	
#define msize		s4

LEAF(sbdreset)
	
	mfc0	t1,C0_CONFIG

#ifdef C0_WATCHLO
	mtc0	zero,C0_WATCHLO
	mtc0	zero,C0_WATCHHI
#endif

#ifdef CFG_K0MASK
	/* make KSEG0 cacheable */
	and	t1,~CFG_K0MASK
	or	t1,CFG_C_NONCOHERENT
#endif	
	
#ifdef C0_CALG
	/* R4650: reset default cache algorithm and watchpoints */
	li	t1,0x22233333
	mtc0	t1,C0_CALG
	mtc0	zero,C0_IWATCH
	mtc0	zero,C0_DWATCH
#endif
	
	/* Vr4300: software controls the endianness */
#if #endian(big)
	or	t1,CFG_BE	
#else
	and	t1,~CFG_BE	
#endif
	
#ifdef CFG_AD
	/* Vr4100: switch off accelerated data */
	and	t1,~CFG_AD
#endif
	
	/* set DDDD rate for CPUs that aren't hardware configured */
	and	t1,~CFG_EPMASK
	or	t1,CFG_EPD
	
	mtc0	t1,C0_CONFIG

	j	nfx_init
nfx_init_done:

	/* initialise tlb */
	DISPLAY ('I','T','L','B')
	jal	init_tlb

#if RAL_XXX
	jal	nsdb
nsdb_done:
	DEBUG_STR("back from nsdb")
#else
	jal	sizeram

    li      a0,0xa0000000
    addu    a1,a0,0x40000
#if __mips >= 3
1:
    sd      zero,0(a0)
    sd      zero,8(a0)
    sd      zero,16(a0)
    sd      zero,24(a0)
    addu    a0,32
    bne     a0,a1,1b
#else
1:
    sw      zero,0(a0)
    sw      zero,4(a0)
    sw      zero,8(a0)
    sw      zero,12(a0)
    addu    a0,16
    bne     a0,a1,1b
#endif

    jal     mips_init_cache
    nop

#ifdef USE_DECOMPRESS
	jal		meminit_decompression_data
	nop

	jal		meminit_decompression_bss
	nop

	jal		meminit_bss
	nop

#else
    jal     meminit_data
    nop

    jal     meminit_bss
    nop
#endif

#ifdef USE_DECOMPRESS
	la		a0,_fdata
	add		a0,0x1F
#else
	la		a0,_end
#endif
	and		a0,a0,0xFFFFFFE0
#ifdef AGILE_HARDWARE
#ifdef PROM_R7000
	/* Don't cleanup high area of memory on warm start */
	mfc0	t0,C0_SR				
	and		t0,SR_SR				
	bne		$0,t0,3f
	nop
	beq		msize,512,eccinit_512
	nop
	la		a1,0x90000000			/* 256 MB */
	b       4f
	nop
eccinit_512:
	and		a0,0x0FFFFFFF
	or		a0,0x40000000
	la		a1,0x60000000
	b		4f
	nop
3:  la		a1,0x8b000000			/* 180 MB */
4:			
#else
	la		a1,0x90000000			/* 256 MB */
#endif
#else
	la		a1,0x82000000			/* 32 MB */
#endif
	la		t0,ecc_init
	jalr	t0
	nop

    /* take care of lo mem area too */
	la		a0,0x80040000   /* 256k - 8 MB */
	la		a1,0x80800000

	la		t0,ecc_init
	jalr	t0
	nop

    jal     rm7k_size_cache         /* restore cache sizes */
    nop

#endif

#ifdef USE_DIAG_LED
    GT_ORVAL_REG_ASS(GPP_VALUE, GPP_FC_DIAG_LED_2LINK_MASK)
#endif

	mul	msize,MEG

	/* Panic if we've not got any memory! */
	bnez	msize,1f
	DISPLAY ('!','M','E','M')
2:	b	2b
	
1:
	
	DISPLAY ('C','A','C','H')
	jal	mips_size_cache

	/* store memory size */
	sd	msize,memorysize

	j	sbdreset_done
END(sbdreset)


	.lcomm	wbfltmp,4

LEAF(wbflush)
	sync
	la	t0,wbfltmp
	or	t0,K1BASE
	lw	zero,0(t0)
	j	ra
END(wbflush)


LEAF(sbddelay)
	mfc0	t0,C0_CONFIG
	li	t1,CACHEMISS+ROMCYCLE
	and	t0,CFG_K0MASK
	beq	t0,CFG_C_UNCACHED,1f
	and	t0,ra,0x20000000
	bnez	t0,1f
	li	t1,CACHECYCLE
1:	mul	a0,1000
	addu	a0,t1
	sll	t1,1
	divu	a0,t1
	subu	a0,48		# approx number of loops so far
	.set	noreorder	
	.set	nomacro
	nop
2:	bgtz	a0,2b
	subu	a0,1
	.set	macro
	.set	reorder
	j	ra
END(sbddelay)


LEAF(mips_cycle)
	.set	noreorder	
	.set	nomacro
1:	bgtz	a0,1b
	subu	a0,1
	.set	macro
	.set	reorder
	j	ra
END(mips_cycle)

#endif	/* PROM */

