/*
 *-----------------------------------------------------------------
 *
 * Name:	boot_i2c.S
 *
 * RCS:  $Id: $
 *
 * Copyright (C) 2004, OnStor Inc.
 *
 * Description:	low level GT 6444x (Discovery 3) i2C routines
 *
 * Created by:	Shaun Hui
 *
 * Date Created:03/18/04
 *
 *-----------------------------------------------------------------
 */
#ifdef PROM

#include <nfx-incl.h>
#include <sys/mips.h>

#include <exports/hwMap.h>
#include <sys/gt64440r.h>

#define GT_REGAD(offs)                  \
        la      v1, GT_BASE_ADDR+(offs)
                                                                                
#define GT_REGRD(offs)                  \
        lw      v0, GT_BASE_ADDR+(offs)
                                                                                
#define GT_REGWR(offs, value)           \
        li      v0, HTOLE32(value);     \
        sw      v0, GT_BASE_ADDR+(offs)
                                                                                
#define GT_REGSET(offs, value)          \
        lw      v0, GT_BASE_ADDR+(offs);\
        li      v1, HTOLE32(value);     \
        or      v0, v1;                 \
        sw      v0, GT_BASE_ADDR+(offs)
                                                                                
#define GT_REGCLR(offs, value)          \
        lw      v0, GT_BASE_ADDR+(offs);\
        li      v1, HTOLE32(~(value));  \
        and     v0, v1;                 \
        sw      v0, GT_BASE_ADDR+(offs)

#define I2C_INT_ENABLE  0x80
#define I2C_ENABLE      0x40
#define I2C_ACK         0x04
#define I2C_INT_FLAG    0x08
#define I2C_STOP_BIT    0x10
#define I2C_START_BIT   0x20
                                                                                
#define I2C_AMOD_RD     0x01
                                                                                
#define BUS_ERROR                               0x00
#define START_CONDITION_TRA                     0x08
#define RSTART_CONDITION_TRA                    0x10
#define ADDR_AND_WRITE_BIT_TRA_ACK_REC          0x18
#define ADDR_AND_READ_BIT_TRA_ACK_REC           0x40
#define SLAVE_REC_WRITE_DATA_ACK_TRA            0x28
#define MAS_REC_READ_DATA_ACK_NOT_TRA           0x58
                                                                                

        .text
        .align  2
        .set    noreorder

/*
 *  Wait for interrupt 
 */
wait_int:
        GT_REGRD(I2C_CONTROL)
        li      v1, HTOLE32(I2C_INT_FLAG)
        and     v0, v1
        beqz    v0, wait_int
        nop
                                                                                
        jr      ra
        nop

wait_stop:
        GT_REGRD(I2C_CONTROL)
        li      v1, HTOLE32(I2C_STOP_BIT)
        and     v0, v1
        beqz    v0, wait_stop
        nop
                                                                                
        jr      ra
        nop

wait_ack:
        GT_REGRD(I2C_STATUS_BAUDE_RATE)
        li      v1, HTOLE32(0xF8)
        beq     v0, v1, wait_ack
        nop
	
	jr	ra
	nop

/*
 *  I2C Master init.
 */
                                                                                
        .globl  boot_i2c_init
boot_i2c_init:
                                                                                
        GT_REGWR(I2C_SOFT_RESET, 0x0)
        GT_REGWR(I2C_STATUS_BAUDE_RATE, 0x24)
        GT_REGWR(I2C_CONTROL, I2C_ENABLE)
                                                                                
        jr      ra
        nop
                                                                                
/*
 * I2C Read byte from device. Use RANDOM READ protocol.
 */
                                                                                
        .globl  boot_i2c_read
boot_i2c_read:
                                                                                
        move    a3, ra                          /* Save return address */
                                                                                
        GT_REGWR(I2C_CONTROL, I2C_ENABLE|I2C_START_BIT)
                                                                                
        bal     wait_int
        nop
                                                                                
	move	v0, a0                         /* Send device address */
        GT_REGAD(I2C_DATA)
        sw      v0, 0(v1)			
        GT_REGWR(I2C_CONTROL, I2C_ENABLE)

        bal     wait_int
        nop
                                                                                
	bal	wait_ack
	nop

        li      v1, HTOLE32(ADDR_AND_WRITE_BIT_TRA_ACK_REC)
        li      a2, -1
        bne     v0, v1, boot_i2c_read_bad
        nop

	move	v0, a1                         /* Send byte address */
        GT_REGAD(I2C_DATA)
        sw      v0, 0(v1)
        GT_REGWR(I2C_CONTROL, I2C_ENABLE)

        bal     wait_int
        nop
                                                                                
	bal	wait_ack
	nop

        li      v1, HTOLE32(SLAVE_REC_WRITE_DATA_ACK_TRA)
        li      a2, -2
        bne     v0, v1, boot_i2c_read_bad
        nop

        GT_REGWR(I2C_CONTROL, I2C_ENABLE|I2C_START_BIT)
                                                                                
        bal     wait_int
        nop
                                                                                
	move	v0, a0               /* Send start + device address */
        ori     v0, 0x01
        GT_REGAD(I2C_DATA)
        sw      v0, 0(v1)			
        GT_REGWR(I2C_CONTROL, I2C_ENABLE)

        bal     wait_int
        nop
                                                                                
        bal     wait_ack
        nop
                                                                                
        li      v1, HTOLE32(ADDR_AND_READ_BIT_TRA_ACK_REC)
        li      a2, -3
        bne     v0, v1, boot_i2c_read_bad
        nop


        GT_REGWR(I2C_CONTROL, I2C_ENABLE)

        bal     wait_int
        nop
                                                                                
        bal     wait_ack
        nop
                                                                                

        li      v1, 0x50
        beq     v0, v1, boot_i2c_read_ok
        nop

        li      v1, 0x58
        beq     v0, v1, boot_i2c_read_ok
        nop

        li      a2, -4
        b       boot_i2c_read_bad
        nop

boot_i2c_read_ok:
        GT_REGRD(I2C_DATA)
        move    a2, v0

        GT_REGWR(I2C_CONTROL, I2C_ENABLE|I2C_STOP_BIT)

        bal     wait_stop
        nop
                                                                                
        move    v0, a2
        jr      a3

boot_i2c_read_bad:
        move    v0, a2
                                                                                
        jr      a3
        nop


#endif	/* PROM */

