/* vi:ts=4
 *-----------------------------------------------------------------
 *
 * Name:	sbd.c
 *
 * RCS:  $Id: sbd.c,v 1.21 2002/09/30 22:56:09 johnh Exp $
 *
 * Copyright (C) 2001, Agile Storage, Inc.
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	9/19/01
 *
 *-----------------------------------------------------------------
 */

#ifdef PROM

#include <nfx-target.h>
#include <sys/mips.h>
#include <exports/hwMap.h>
#include "../prom-pmon/pmon.h"
#include "../prom-pmon/ds1511.h"
#include <pmon/termio.h>
//#include "sbd.h"
#include <sys/mips.h>
#include <sys/led-api.h>
#include <sys/gt64440r.h>
#include <sys/r9k-regs.h>
#include <exports/i2c-api.h>

extern int	ns16550();
unsigned int get_busclock( void );
int isFC(void);
int isSSC(void);


ConfigEntry     ConfigTable[] =
{
	{0}
};


void
sbdpoll()
{
}

/* display a message (four chars packed into one word),
   for very low-level code. */
void
sbddisplay (unsigned long msg, unsigned int x)
{
}


#ifdef NOTUSED
/* display a long message on the display, scrolling if necessary. */
void
sbdmessage (int line, const char *msg)
{
}

/* display a message and then pause so that it can be read */
static void
display (const char *msg)
{
}

/* display a register name and its value, then pause */
static void
xdisplay (char *s, unsigned int x)
{
}
#endif

char *
sbddbgintr (unsigned int cr)
{
	return 0;
}

#ifdef NOTUSED
char *
sbdexception (epc, cause, ra, extra, exc)
	unsigned long   epc, cause, ra, extra;
	char *exc;
{
	
}

_fpstatesz ()
{
	return 0;
}

_fpstate ()
{
	return -1;
}

cop1 ()
{
	printf ("\ncop1 called\n");
	cliexit ();
}
#endif

int
getmachtype ( void )
{

	switch ((Prid >> 8) & 0xff) {
		case PRID_RM9000:
			return 9000;
		case PRID_RM7000:
			return 7000;
		default:
			return 0;
	}
}

#ifdef NOTUSED
#ifndef INET
/* gsignal() is called when a keyboard interrupt (^C) is detected */
gsignal (jmp_buf *jb, int sig)
{
	if (jb) longjmp (jb, 1);
}

netopen()	{return -1;}
netread()	{return -1;}
netwrite()	{return -1;}
netclose()	{return -1;}
netlseek()	{return -1;}
netioctl()	{return -1;}
netpoll()	{}


unsigned long __time (void)
{
#ifdef RTC
	return (sbd_gettime ());
#else
	return 0;
#endif
}
#endif
#endif

unsigned long pipefreq = 0;
int cpuspeed = 100;

/* SysClk to PipeClk ratio table */
struct ratio {char mult; char div;};
static const struct ratio ratios_rm7k[8] = {{2,1}, {3,1}, {4,1}, {5,2},
											{6,1}, {7,2}, {8,1}, {9,2}};

/* hand tune delay */
static void
probefreq(void)
{
	int i;
	unsigned long start, cnt, timeout, sec, cur;

	/* 
	 * Do the loop twice to make sure the RTC SECOND is synched.
	 */
	for (i=2; i != 0; i--)
	{
		/* 
		 * read RTC SECOND & start count
		 */
		sec = ds1511_read_reg(DS1511_SEC);
		start = get_count();
		timeout = 800000000;

		/* 
		 * Loop for 1 sec & check count.
		 */
		do
		{
			timeout--;
			cur = ds1511_read_reg(DS1511_SEC);
			
		} while (timeout !=0 && cur == sec );

		cnt = get_count() - start;
		if (timeout == 0)
			return;
	}

	/* 
	 * Multiple cnt by 2 since count register 
	 * is incremented at 1/2 pipeline freq.
	 */
	pipefreq = cnt * 2; 
}


unsigned long sbdpipefreq (void)
{
	if (pipefreq == 0)
		probefreq ();
	return pipefreq;
}

unsigned long sbdcpufreq (void)
{
	return get_busclock() * 1000000;
}

void
sbdmachinfo (void)
{
}

unsigned int
sbdenable (int machtype)
{
	unsigned int sr;

#ifdef FLASH
	{
		/* now is the chance to report any early eerom failures */
		extern const char *_sbd_envinit (void);
		const char * err = _sbd_envinit ();
		if (err)
			printf ("WARNING: flash environment: %s\n", err);
	}
#endif

#if RAL_XXX
	/* clear interrupt latches */
	icu->ctrl.clear = ~0;

	/* all external interrupts masked initially */
	imask_set (0);
#endif

#if RAL_XXX
	/* enable panic errors */
	sr = SR_IBIT7 | SR_IE;
#else
	sr = 0;
#endif

	sr = r4k_getsr();
#if 0
	(void) bis_sr (sr);
#endif
	return sr;
}


#define WDELAY()							\
{											\
	register int i;							\
	wbflush();								\
	for (i = DELAY; i != 0; i--) continue;	\
}

/*
 * Prototypes for functions called by sbddevinit.
 */
void	eee_init( int );
void	pci_init( void );
void	mbinit( void );
void	pkt_init( void );
void	luc_init( void );
void	tpl_init( void );
void	q_initQueue( void );
void	eee_timerInit( void );
void	timer_ticInit( void );
void	eee_initFragmentTimeout( void );
void	rconInit( void );
void	gt64440mii_initPci( void );
void	gt64440mii_initModule( void );
void	dp83816_initModule(void);
void	tftp_init( void );
void	domaininit( void );
void	luc_postInit( void );
void	seepGetIpAddr(char *);
void	luc_simStart_setIpAddr(char *);
void	luc_simStart_withArgs( int );
void	cmFanMonitorInit( void );
void	gt_console_init( void );
void	load_bc_init( void );

void
sbddevinit ()
{
	char			ipaddr[16];
    unsigned int    ipaddr_uint;

#ifdef AGILE_HARDWARE
	gt_mem_init();
#endif

	load_bc_init();

	eee_init(1);

	pci_init();

	mbinit();

	pkt_init();

	luc_init();

	tpl_init();

	eee_init(2);

	q_initQueue();

	eee_timerInit();

	timer_ticInit();

	eee_initFragmentTimeout();

	rconInit();

#ifdef AGILE_HARDWARE
	seepInit();
	seepGetIpAddr(ipaddr);
	getIpAddr(ipaddr, &ipaddr_uint);
	seepInitExtMacAddr(ipaddr_uint);
#endif

#ifndef PROM_R9000
#ifdef USE_NEW_BACKPLANE
	bp_init();
#endif
#endif

    pci_init( );

#ifdef AGILE_HARDWARE
	mem_map_gt_init();
	dp83816_initModule();
#else

    gt64440mii_initPci( );
                                                                                
	gt64440mii_initModule();
#endif

	tftp_init();

	domaininit();

	luc_postInit();

	seepGetIpAddr(ipaddr);
	luc_simStart_setIpAddr(ipaddr);
	luc_simStart_withArgs(2088);

	cmFanMonitorInit( );

	gt_console_init();
}

#ifdef AGILE_HARDWARE
int
gt_dll_table[] = {
(0x00000000), (0x00000001), (0x00000042), (0x00000083), 
(0x000000c4), (0x00000105), (0x00000146), (0x00000187), 
(0x000001c8), (0x00000209), (0x0000024a), (0x0000028b),
(0x000002cc), (0x0000030d), (0x0000034e), (0x0000038f), 
(0x000003d0), (0x00000411), (0x00000452), (0x00000493), 
(0x000004d4), (0x00000515), (0x00000556), (0x00000597),
(0x000005d8), (0x00000619), (0x0000065a), (0x0000069b), 
(0x000006dc), (0x0000071d), (0x0000075e), (0x0000079f), 
(0x000007e0), (0x00000821), (0x00000862), (0x000008a3),
(0x000008e4), (0x00000925), (0x00000966), (0x000009a7), 
(0x000009e8), (0x00000a29), (0x00000a6a), (0x00000aab), 
(0x00000aec), (0x00000b2d), (0x00000b6e), (0x00000baf),
(0x00000bf0), (0x00000c31), (0x00000c72), (0x00000cb3), 
(0x00000cf4), (0x00000d35), (0x00000d76), (0x00000db7), 
(0x00000df8), (0x00000e39), (0x00000e7a), (0x00000ebb),
(0x00000efc), (0x00000f3d), (0x00000f7e), (0x00000fbf)

};
                                                                                
typedef unsigned int uint32;

int roundup(int val, int base)
{
	return (((val % base) * 2 >= base) ? 1 : 0);
}

/*
 *------------------------------------------------------------------------------ * Function Name:       gt_set_sdram_timing
 *
 * Description:         Set up the GT64440 SDRAM timing based on SEEP value.
 *
 *------------------------------------------------------------------------------ */
void
gt_set_sdram_timing(char *i2c_buff )
{
	int		cycle_time;
	int		sdram_timing;
	int		t_rcd;
	int		t_rp;
	int		t_ras;
	int		t_rrd;
	int		t_rfc;
	int		t_wtr;
	int		t_wr;
	
	cycle_time = 1000/get_busclock();		/* cycle time in ns */

	t_rp  = (int)i2c_buff[27];
	t_rrd = (int)i2c_buff[28];
	t_rcd = (int)i2c_buff[29];
	t_ras = (int)i2c_buff[30];
	t_rfc = (int)i2c_buff[42];
	t_wtr = (int)1;
	t_wr  = (int)1;
	
	t_rcd = ((t_rcd/4) /cycle_time) + roundup(t_rcd/4, cycle_time) - 1;
	t_rp  = ((t_rp /4) /cycle_time) + roundup(t_rp/4 , cycle_time) - 1;
	t_rrd = ((t_rrd/4) /cycle_time) + roundup(t_rrd/4, cycle_time) - 1;
	t_ras = (t_ras /cycle_time) + roundup(t_ras, cycle_time) - 1;
	t_rfc = (t_rfc /cycle_time) + roundup(t_rfc, cycle_time) - 1;

	GT_WRITE(SDRAM_TIMING_LOW, ((t_rrd << 24) | (t_ras << 20) | (t_rp << 8) | 
							 	(t_rcd << 4)  | (t_wtr << 16) | (t_wr << 12)));
	GT_WRITE(SDRAM_TIMING_HIGH, t_rfc);

//	printf("SDRAM Timing Low =0x%08x\n", GT_READ(SDRAM_TIMING_LOW));
//	printf("SDRAM Timing High=0x%08x\n", GT_READ(SDRAM_TIMING_HIGH));
}

#define MEMSIZE_1G      0x40000000

/*
 *------------------------------------------------------------------------------ * Function Name:       gt_mem_init
 *
 * Description:         Initializes GT SDRAM controller
 *
 *------------------------------------------------------------------------------ */
gt_mem_init( void )
{
	int		rc, i, sr;
	int		mem_size;
	int		sdram_cfg;
	char	i2c_buff[48];
	char 	bus=0;

    /*
     * Read memory SEEP
     */
	rc = i2c( I2C_READ, bus, I2C_ADDR_GT_MEM_SEEP, 0, sizeof(i2c_buff), i2c_buff );
	if (rc != 0)
	{
		printf("%s: I2C Read failed!\n", __FUNCTION__);
		return;
	}

    /*
     * mem_size (MB) = #of banks * Module bank density * banks on device
     */
	mem_size = i2c_buff[5] * i2c_buff[31] * i2c_buff[17];
	memorysize = mem_size << 20;

    /*
     * Setup CS 0
     */
	GT_WRITE(SCS_0_BASE_ADDRESS, (HW_GT_MEM_0_BASE >> 16) );
	GT_WRITE(SCS_0_SIZE, ((GT_MEM_0_SIZE-1) >> 16) );

    /*
     * Setup CS 1 if 2nd bank is present.
     */
	if (i2c_buff[5] == 2)
	{
		GT_WRITE(SCS_1_BASE_ADDRESS, (HW_GT_MEM_1_BASE >> 16) );
		GT_WRITE(SCS_1_SIZE, ((GT_MEM_1_SIZE-1) >> 16) );
	}

	GT_WRITE(SRAM_ADDRESS, 0);

	for (i=0; i  < (sizeof(gt_dll_table)/sizeof(int)); i++)
	{
		GT_WRITE(SRAM_DATA0, gt_dll_table[i]);
	}

	GT_WRITE(DFCDL_CONFIGURATION0, 0x00300000);

	GT_WRITE(SDRAM_OPERATION, SDRAM_OP_CMD_NOP); 		/* Send NOP */
	rc = GT_READ(SDRAM_OPERATION);

    /*
     * Setup SDRAM Config, check if registered dimm
     */
    sdram_cfg = SDRAM_CFG_REFRESH_INTERVAL | SDRAM_CFG_FBSPLIT_DISABLE |
                SDRAM_CFG_FBVM_DISABLE     | SDRAM_CFG_TYPE_SDRAM      |
                SDRAM_CFG_ECC_ENABLED      | SDRAM_CFG_ECC_REPORTED    |
                SDRAM_CFG_DQS_X8_DEV       | SDRAM_CFG_RFMODE_1        |
                SDRAM_CFG_RDBUFF_0_MPSC    | SDRAM_CFG_RDBUFF_1_PCI1   |
                SDRAM_CFG_RDBUFF_0_GBE     | SDRAM_CFG_RDBUFF_1_IDMA   |
                SDRAM_CFG_RDBUFF_1_PCI0    |
                ((i2c_buff[21]== 0x26) ? SDRAM_CFG_REGISTERED : 0);
                                                                                

	GT_WRITE(SDRAM_CONFIGURATION, sdram_cfg);	/* registered dimm */

	GT_WRITE(SDRAM_OPEN_PAGES_CONTROL,0x00000000);
	GT_WRITE(DUNIT_CONTROL_LOW,(0xc5000500|GT_READ(DUNIT_CONTROL_LOW) & 0xff));

    /*
     * Set MODE register for:
     * CAS Latency 2.5, Burst Length 4
     */
	GT_WRITE(SDRAM_OPERATION, SDRAM_OP_CMD_SET_MODE);
	rc = GT_READ(SDRAM_OPERATION);

	GT_WRITE(SDRAM_MODE, SDRAM_MODE_BL_4 | SDRAM_MODE_CL_25);

	GT_WRITE(SDRAM_OPERATION, SDRAM_OP_CMD_SET_MODE);		/* set mode reg */
	rc = GT_READ(SDRAM_OPERATION);

    /*
     * Set Address control for 512M or 1G.
     */
    GT_WRITE(SDRAM_ADDRESS_CONTROL, SDRAM_ADDR_CTRL_ADDR_SEL_2 |
                                    SDRAM_ADDR_CTRL_DCFG_512M_OR_1G);
                                                                                
    /*
     * Set SDRAM timing
     */
	gt_set_sdram_timing( i2c_buff );

	/* 
	 * Enable DUNIT interrupt for 
	 * single bit, double bit ECC errors & 
	 * DUNIT internal parity error
	 */
	GT_WRITE(DUNIT_INTERRUPT_MASK,  DUNIT_INTERRUPT_SBIT_ERR |
									DUNIT_INTERRUPT_DBIT_ERR |
									DUNIT_INTERRUPT_DP_ERR);

	/* 
	 * Enable CPU interrupt 1 for DRAM ECC error
	 */
	GT_WRITE(CPU_INT_1_MASK_LOW,  INT_MASK_LOW_DRAM_ERR );

	/* 
	 * Init GT memory for ECC.
	 */
	printf("Initializing Shared Memory (%d MB) for ECC\n", mem_size);
	if (memorysize == MEMSIZE_1G)
		ecc_init( SW_RAM_BASE_CACHED, SW_RAM_BASE_CACHED+MEMSIZE_1G);
	else
		ecc_init( SW_RAM_BASE_CACHED, SW_RAM_BASE_CACHED+MEMSIZE_1G/2);

	/* 
	 * Enable R9K external Int. & HW INT 0
	 * GT CPU INT 1 -> HWINT 0.
	 */
	sr  = _mips_mfc0(C0_SR);
	sr |= SR_HINT0 | SR_IE;
	_mips_mtc0(C0_SR, sr);

}
#endif 

const char *
sbdgetname ()
{
#ifdef AGILE_HARDWARE
	if (isSSC())
		return "SSC";
	else if (isFC())
		return "FCNIM";
	else
		return "Unknown";
#else
	return "GT64440";
#endif
}


/* early low-level machine initialisation */
void
sbdmachinit ()
{
}

#ifndef RTC
long
sbd_gettime (void)
{
	return 0;
}
#endif


#ifndef FLASH
/*
 * dummy nvram handler for systems without nvram.
 */

char *
sbd_getenv (name)
	char *name;
{
	return 0;
}


int
sbd_setenv (char *name, char *value)
{
	return 1;
}


int
sbd_unsetenv (char *name)
{
	return 1;
}


void
sbd_mapenv (int (*func)(char *, char *))
{
}

void
sbd_flashinfo (void **buf, int *size)
{
	buf = 0;
	size = 0;
}


int
sbd_flashprogram (void *buf, int size)
{
	return 0;
}
#endif

unsigned long long
get_memorysize()
{
	return memorysize;
}

extern	unsigned int cpu_startvectors[];
extern	unsigned int cpu_start_spvals[];
extern	unsigned int stack;
typedef unsigned int u_int32_t;

int
r9k_cpu1_go( int argc, char *argv[] )
{
	uint32	vector;

	printf("argc = %d\n", argc);
	if (argc != 2)
		return -1;

	if (!get_rsa (&vector, argv[1]))
		return -1;

	cpu_startvectors[1] = vector;
	cpu_start_spvals[1] = ((uint32)&stack) + (STKSIZE >> 1) - 24;

	printf("startvectors=0x%x stack =%p \n", 
			cpu_startvectors[1], cpu_start_spvals[1] );
	RM9K_WRITE(R9K_UPENR, 1);			// enable write to RSTSTATUS 
	RM9K_WRITE(R9K_RSTSTATUS, 0);		// unreset CPU 1

	return 0;
}

/*
 *----------------------------------------------------------------------------
 * Function Name:       ecc_exception
 *
 * Description:         Handles ECC exceptions for R9K & GT memories.
 *
 *---------------------------------------------------------------------------
 */
void
ecc_exception(void)
{
    /*
     * Check for R9K ECC errors
     */
	if ( RM9K_READ(R9K_XECC) & (R9K_XECC_SECC | R9K_XECC_MECC))
	{
		printf("\nR9K ECC Exception  \n" );
		printf("XECC:    0x%08x\n", RM9K_READ(R9K_XECC));
		printf("XCAUFA0: 0x%08x\n", RM9K_READ(R9K_XCAUFA0));
		printf("XCAUFA1: 0x%08x\n", RM9K_READ(R9K_XCAUFA1));
		printf("SDRAM Write Err Addr Hi = 0x%08x\n", RM9K_READ(R9K_SDWEAH));
		printf("SDRAM Write Err Addr Lo = 0x%08x\n", RM9K_READ(R9K_SDWEAL));
		printf("SDRAM Error Address  Hi = 0x%08x\n", RM9K_READ(R9K_SDEAH));
		printf("SDRAM Error Address  Lo = 0x%08x\n", RM9K_READ(R9K_SDEAL));
	}

    /*
     * Check for GT64440 ECC errors
     */
    if (GT_READ(LOW_INTERRUPT_CAUSE_REGISTER) & INT_MASK_LOW_DRAM_ERR)
    {
        printf("\nGT ECC Exception  \n");
        printf("DUNIT Int Cause   = 0x%08x\n", GT_READ(DUNIT_INTERRUPT_CAUSE));
        printf("SDRAM Err Data Lo = 0x%08x\n", GT_READ(SDRAM_ERROR_DATA_LOW));
        printf("SDRAM Err Data Hi = 0x%08x\n", GT_READ(SDRAM_ERROR_DATA_HIGH));
        printf("SDRAM Err Address = 0x%08x\n",
                GT_READ(SDRAM_AND_DEVICE_ERROR_ADDRESS));
        printf("SDRAM Err Count   = 0x%08x\n",
                GT_READ(SDRAM_ECC_ERROR_COUNTER));
    }

    /*
     * Check for GT64440 WDE & WD NMI
     */
    if (GT_READ(LOW_INTERRUPT_CAUSE_REGISTER) &
        (INT_MASK_LOW_WDE | INT_MASK_LOW_WDNMI))
    {
        printf("\nCPU 1 Watchdog NMI Exception \n");
    }
                                                                                

}
#endif	/* PROM */

