/*
 *-----------------------------------------------------------------
 *
 * Name:	sbdreset.S
 *
 * RCS:  $Id: $
 *
 * Copyright (C) 2004, OnStor Inc.
 *
 * Description:	boot up code for Bobcat HW & R9000 eval board (Jaguar-ATX)
 *
 * Created by:	Shaun Hui
 *
 * Date Created:03/18/04
 *
 *-----------------------------------------------------------------
 */
#ifdef PROM

#include <nfx-incl.h>
#include <sys/mips.h>
#include <exports/hwMap.h>
#include <sys/gt64440r.h>
#include <sys/ns16550.h>
#include "serio-io.h"


#define OC_SRAM_BASE_CACHED	PHYS_TO_K0(HW_OC_SRAM_BASE)

#if !__mips64 && __mips >= 3 && __mips != 32
	/* force 64-bit register support */
#undef __mips64	
#define __mips64 1	
	.set	gp64
#endif
	
#define DISPLAY(d0,d1,d2,d3)

#define MEG	0x100000		

#define GTINIT(offset, value) \
        .word   GT_BASE_ADDR+(offset), value
                                                                                

	.globl	sdram_init_done
	.globl	cache_init_done

/*
 * Basic board initialisation, called straight from RESET.
 * It is only called if this program is built for ROM.  
 */

        .set    noreorder
                                                                                
        .globl  sbdreset
        .ent    sbdreset
sbdreset:

        la      s0, sbdreset            /* RA set from JAL in start  */
        subu    s0, ra, s0              /* s0 is now load vs. link offset */
        and     s0, 0xffff0000          /* Mask off lower bits */

        /*
         * Find CPU ID. If CPU 1, skip OCD
         * & GT related setups. Done by CPU 0.
         */
        mfc0    t4, C0_PRID
        srl     t4, 24
        bnez    t4, init_tlb
        nop
                                                                                

                                                                                
/*
 * We set LKM/B 5 to show the GT-64440 internal registers before we move them.
 */
                                                                                
        /* set LKM5 for 64KByte */
        li      t0, 0x00000f00
        sw      t0, 0x12c(s2)
        lw      t0, 0x12c(s2)
        sync
                                                                                
        /* set LKB5 for 0x1400_0000 base */
        li      t0, 0x01400001
        sw      t0, 0x128(s2)
        lw      t0, 0x128(s2)
        sync


/*
 *  Turn off all high decoders to avoid address conflicts.
 */
        la      s2, GT_BASE_ADDR_DEFAULT
#ifdef EVAL_BOARD
        li      t0, HTOLE32(0x0007fe0e)
#else
        li      t0, 0x00043a0c
#endif
        sw      t0, CPU_BASE_ADDRESS_ENABLE(s2)


/*
 *  Relocate the Galileo to HIGH memory.
 */
#ifdef EVAL_BOARD
        li      t0, HTOLE32((GT_BASE_ADDR >> 16) | 0x01000000)
#else
        li      t0, HTOLE32((HW_GALILEO_BASE >> 16) | 0x01000000)
#endif
        sw      t0, INTERNAL_SPACE_BASE(s2)
                                                                                
        li      t0, 0x100               /* Need a small delay here */
1:
        bnez    t0, 1b
        addiu   t0, -1

init_tlb:

#ifdef EVAL_BOARD
/*
 *  Clean out and initialize the TLB to map 0xc0000000 - 0xffffffff
 */
        jal     CPU_TLBClear
        nop
                                                                                
        li      a0, 0xc0000000	# virtual addr start
        li      a1, 0x40000000	# virtual mem size (1 GB)
        li      t0, 0xc0000000	# physical addr start
        li      t1, 0x10000000	# page size (256 MB)
        li      t2, TLBMASK_MASK256Mb	# page mask
        li      t3, TLBLO_CALG_UNCACHED	# write back
        jal     CPU_TLBInit
        nop
                                                                                
/*
 * Set LKM/B 5 to map entire GT-64340 I/O space at 0xc0000000 - 0xffffffff
 * This will also map in the GT-64340 at its new location.
 */
        la      s2, RM9K_BASE_ADDR
                                                                                
        /* set LKB5 for 0xc000_0000 base */
        li      t0, 0x0c000001
        sw      t0, 0x128(s2)
        lw      t0, 0x128(s2)
        sync
                                                                                
        /* set LKM5 for 1GByte */
        li      t0, 0x03ffff00
        sw      t0, 0x12c(s2)
        lw      t0, 0x12c(s2)
        sync
                                                                                
#else

/*
 *  Clear TLB. 
 */
        jal     CPU_TLBClear
        nop

/*
 * Setup TLB to map 256 of R9K Memory from phy addr 0x20000000 
 * for cached access.
 *
 * virt addr     phys addr      page    cache mode
 * 0x10000000    0x20000000     64MB    cached-writeback
 */
        li      a0, 0x10000000	# virtual addr start
        li      a1, 0x10000000	# virtual mem size (256 MB)
        li      t0, 0x20000000	# physical addr start
        li      t1, 0x04000000	# page size (64 MB)
        li      t2, TLBMASK_MASK64Mb	# page mask
        li      t3, TLBLO_CALG_WBACK	# write back
        jal     CPU_TLBInit
        nop


/*
 * Setup TLB to map 1GB of GT Memory from phy addr 0x40000000 
 * for cached access.
 *
 * virt addr     phys addr      page    cache mode
 * 0x40000000    0x40000000     256MB   cached-writeback
 * 0x50000000    0x50000000     256MB   cached-writeback
 * 0x60000000    0x60000000     256MB   cached-writeback
 * 0x70000000    0x70000000     256MB   cached-writeback
 */
        li      a0, 0x40000000	# virtual addr start
        li      a1, 0x40000000	# virtual mem size (1 GB)
        li      t0, 0x40000000	# physical addr start
        li      t1, 0x10000000	# page size (256 MB)
        li      t2, TLBMASK_MASK256Mb	# page mask
        li      t3, TLBLO_CALG_WBACK	# write back
        jal     CPU_TLBInit
        nop
                                                                                

/*
 * Setup TLB to map 1GB of GT Memory from 0x40000000 
 * for uncached access.
 *
 * virt addr     phys addr      page    cache mode
 * 0xc0000000    0x40000000     256MB   uncached
 * 0xd0000000    0x50000000     256MB   uncached
 * 0xe0000000    0x60000000     256MB   uncached
 * 0xf0000000    0x70000000     256MB   uncached
 */
        li      a0, 0xc0000000	# virtual addr start
        li      a1, 0x40000000	# virtual mem size
        li      t0, 0x40000000	# physical addr start
        li      t1, 0x10000000	# page size (256 MB)
        li      t2, TLBMASK_MASK256Mb	# page mask
        li      t3, TLBLO_CALG_UNCACHED	# uncached
        jal     CPU_TLBInit
        nop
                                                                                
		/* 
		 * Find CPU ID. If CPU 1, skip OCD
		 * registers setup. Done by CPU 0.
		 */
        mfc0    t4, C0_PRID
        srl     t4, 24
		bnez	t4, init_serio
		nop

/*
 * Set LKM/B 5 to map 1GB of GT Memory from phy addr 0x40000000 
 */

        la      s2, RM9K_BASE_ADDR
                                                                                
        /* set LKB5 for 0x4000_0000 base */
        li      t0, 0x04000001
        sw      t0, 0x128(s2)
        lw      t0, 0x128(s2)
        sync
                                                                                
        /* set LKM5 for 1GByte */
        li      t0, 0x03ffff00
        sw      t0, 0x12c(s2)
        lw      t0, 0x12c(s2)
        sync
                                                                                
        /* set LKM6 for 128MB */
        li      t0, 0x007fff00
        sw      t0, 0x134(s2)
        lw      t0, 0x134(s2)
        sync
                                                                                
        /* set LKB6 for 0x1800_0000 base */
        li      t0, 0x01800001
        sw      t0, 0x130(s2)
        lw      t0, 0x130(s2)
        sync
                                                                                
        /* set LKM13 for 8KB */
        li      t0, 0x00000100
        sw      t0, 0x16c(s2)
        lw      t0, 0x16c(s2)
        sync
                                                                                
        /* set LKB13 for Scratch RAM base @0x1000_4000 */
        li      t0, 0x01000401
        sw      t0, 0x168(s2)
        lw      t0, 0x168(s2)
        sync
                                                                                
        /* Init 8KB of Scratch RAM @0x1000_4000 */
        la      t0, 0xb0004000
        addu    t1, t0, 0x2000
1:
        addu    t0, 8
        bne     t1, t0, 1b
        sd      zero, -8(t0)
                                                                                
#endif
        la      s2, GT_BASE_ADDR        /* From now this is the GT base */
                                                                                
        /* Assume 150 MHz timing */
        bal     tgt_setpar150mhz
        nop
                                                                                
        lw      t0, CPU_CONFIG(s2)      /* Turn off  automatic retries */
        li      t1, HTOLE32(0x00020000)
        or      t0, t0, t1
        sw      t0, CPU_CONFIG(s2)
                                                                                
/*
 *  Set up I/O decoders to point correctly.
 */
        bal     2f      /* Load address to init table */
        nop
                                                                                
#ifdef EVAL_BOARD
        GTINIT(CS_0_BASE_ADDRESS, FPGA_BASE >> 16)
        GTINIT(CS_0_SIZE, (FPGA_SIZE - 1) >> 16)
        GTINIT(CS_1_BASE_ADDRESS, RTC_BASE >> 16)
        GTINIT(CS_1_SIZE, (RTC_SIZE - 1) >> 16)
        GTINIT(CS_2_BASE_ADDRESS, UART_BASE >> 16)
        GTINIT(CS_2_SIZE, (UART_SIZE - 1) >> 16)
        GTINIT(CS_3_BASE_ADDRESS, FLASH_BASE >> 16)
        GTINIT(CS_3_SIZE, (FLASH_SIZE - 1) >> 16)
                                                                                
        GTINIT(INTERNAL_SRAM_BASE, OC_SRAM_BASE >> 16)
                                                                                
        /* Serial ports in MV64340 */
        GTINIT(MAIN_ROUTING_REGISTER, 0x00000038)       /* Route both UARTS */
        GTINIT(RECEIVE_CLOCK_ROUTING_REGISTER, 0)
        GTINIT(TRANSMIT_CLOCK_ROUTING_REGISTER, 0)
        GTINIT(MPP_CONTROL1, 0x00002222)
#define GTBAUD ((GTSYSCLK / (32 * 9600) / 2) - 1)
        GTINIT(BRG0_CFG_REG, 0x00210000 + GTBAUD)
#undef GTBAUD
        GTINIT(BRG0_BAUD_TUNING_REG, 0x0)
        GTINIT(MPSC0_MAIN_CONFIGURATION_LOW, 0x04c4)
        GTINIT(MPSC0_MAIN_CONFIGURATION_HIGH, 0x06400600)
        GTINIT(MPSC0_PROTOCOL_CONFIGURATION, 0x00003000)
        GTINIT(CHANNEL0_COMMAND_REGISTER, 0x00008000)
        GTINIT(CHANNEL0_REGISTER4, 0x20000000)
        GTINIT(CHANNEL0_REGISTER5, 0x00009000)
                                                                                
        /* GPP for seven seg LED */
        GTINIT(GPP_IO_CONTROL, 0xff000000)
        GTINIT(GPP_LEVEL_CONTROL, 0xff000000)
        GTINIT(GPP_VALUE, 0x80000000)
#else

	/* Setup CS 0 - RTC */
        GTINIT(CS_0_SIZE, (RTC_SIZE - 1) >> 16)
        GTINIT(CS_0_BASE_ADDRESS, HW_RTC_BASE >> 16)

	/* Setup CS 1 - Reserved */
        GTINIT(CS_1_SIZE, (RESERVED_SIZE - 1) >> 16)
        GTINIT(CS_1_BASE_ADDRESS, HW_RESERVED_BASE >> 16)

	/* Setup CS 2 - BMFPGA */
        GTINIT(CS_2_SIZE, (BMFPGA_SIZE - 1) >> 16)
        GTINIT(CS_2_BASE_ADDRESS, HW_BMFPGA_BASE >> 16)

	/* Setup BOOTCS - Boot Prom */
        GTINIT(BOOTCS_BASE_ADDRESS,  (HW_PROM_BASE >> 16))
        GTINIT(BOOTCS_SIZE, (BOOT_SIZE-1) >> 16)

	/* Setup CS 3 - Non Boot Prom */
        GTINIT(CS_3_SIZE, (NON_BOOT_PROM_SIZE - 1) >> 16)
        GTINIT(CS_3_BASE_ADDRESS, HW_BIG_BOOTROM_BASE >> 16)

        GTINIT(INTERNAL_SRAM_BASE, HW_OC_SRAM_BASE >> 16)
                                                                                
        /* Serial ports in MV64440 */
        GTINIT(MAIN_ROUTING_REGISTER, 0x00000000)       /* Route both UARTS */
        GTINIT(RECEIVE_CLOCK_ROUTING_REGISTER, 0)
        GTINIT(TRANSMIT_CLOCK_ROUTING_REGISTER, 0)

        /* Setup GPP pins (GPIO, arbiter, uart etc) */
        GTINIT(MPP_CONTROL0, 0x00111111)
        GTINIT(MPP_CONTROL1, 0x33332222)
        GTINIT(MPP_CONTROL2, 0x00000011)
        GTINIT(MPP_CONTROL3, 0x00000000)

        GTINIT(BRG0_BAUD_TUNING_REG, 0x0)

	/* UART 0 */
        GTINIT(MPSC0_MAIN_CONFIGURATION_LOW, 0x04c4)
        GTINIT(MPSC0_MAIN_CONFIGURATION_HIGH, 0x06400600)
        GTINIT(MPSC0_PROTOCOL_CONFIGURATION, 0x00003000)

	/* UART 1 */
        GTINIT(MPSC1_MAIN_CONFIGURATION_LOW, 0x04c4)
        GTINIT(MPSC1_MAIN_CONFIGURATION_HIGH, 0x06400600)
        GTINIT(MPSC1_PROTOCOL_CONFIGURATION, 0x00003000)

        GTINIT(CHANNEL0_COMMAND_REGISTER, 0x00008000)
        GTINIT(CHANNEL0_REGISTER4, 0x20000000)
        GTINIT(CHANNEL0_REGISTER5, 0x00009000)
                                                                                
        /* GPP for LEDs */
        GTINIT(GPP_IO_CONTROL, 0xffe55515)
        GTINIT(GPP_LEVEL_CONTROL, 0x00000000) 
        GTINIT(GPP_VALUE, 0x08000000) /* Use LSI MAC for port status */
#endif
        /* end mark */
        .word   0, 0
1:
        sw      v1, 0(v0)
2:
        lw      v0, 0(ra)               /* Address */
        lw      v1, 4(ra)               /* Data */
        bnez    v0, 1b
        addiu   ra, 8
                                                                                
#ifndef EVAL_BOARD
/*
 * Read bit 15 of BMFPGA to determine 
 * if SYS CLK is 133MHz or 100MHz
 * Set UART baud rate to 57600 based on SYSCLK.
 */
        la      t0, BMFPGA_BASE
        lw      t1, 0(t0)
        andi    t1, 0x8000
	li	t0, 0x00210000
        bnez    t1, 1f
	nop
	ori	t0, ((GTSYSCLK_100MHZ / (32 * 57600) / 2) - 1)
	b	2f
	nop
1:
	ori	t0, ((GTSYSCLK_133MHZ  / (32 * 57600) / 2) - 1)
2:
        sw      t0, BRG0_CFG_REG(s2)
#endif

init_serio:
/*
 *  Init serial I/O for diagnostic output.
 */
        jal     initserial
        nop

        la      s0, sbdreset           /* RA set from JAL above! */
        subu    s0, ra, s0              /* s0 is now load vs. link offset */
        and     s0, 0xffff0000          /* Mask off lower bits */
                                                                                

        PRINTSTR("\r\nPMON CPU ")
        move    a0, t4
        jal     hexserial
        nop

        PRINTSTR(" Initializing. Standby...\r\n")
                                                                                
        PRINTSTR("EXCEPTIONPC=")
        mfc0    a0, C0_EPC
        jal     hexserial
        nop
                                                                                
        PRINTSTR(" CONFIG=")
        mfc0    a0, C0_CONFIG
        jal     hexserial
        nop
                                                                                
        PRINTSTR(" STATUS=")
        mfc0    a0, C0_SR
        jal     hexserial
        nop
        PRINTSTR("\r\n")
                                                                                
#ifdef  EVAL_BOARD
        TTYDBG("Remapping BOOT\r\n")
                                                                                
        bal     1f
        nop
        /* This code is copied to Marvell on chip ram and executed there */
ocramcode:
        li      v0, HTOLE32(BOOT_BASE >> 16)
        sw      v0, BOOTCS_BASE_ADDRESS(s2)
        li      v0, HTOLE32((BOOT_SIZE - 1) >> 16)
        sw      v0, BOOTCS_SIZE(s2)
        lw      v0, BOOTCS_SIZE(s2)     /* IOSYNC */
        li      v0, BOOT_BASE - 0xbfc00000
        addu    s0, v0
        li      v0, BOOT_BASE
        or      ra, v0
        jr      ra
        nop
ocramcodesz = . - ocramcode
                                                                                
1:
        la      v0, OC_SRAM_BASE
        li      v1, ocramcodesz
2:
        lw      t0, (ra)
        sw      t0, (v0)
        addiu   v1, -4
        addiu   v0, 4
        bgtz    v1, 2b
        addiu   ra, 4
                                                                                
        la      v0, OC_SRAM_BASE
        jalr    v0                      /* Execute remapping in Marvell ram */
        nop
#endif

        mfc0    t4, C0_PRID
        srl     t4, 24
		bnez	t4, in_ram
                                                                                
        TTYDBG("Setting up SDRAM controllers\r\n")

        jal     sdram_init
        nop

/*
 *  Clear out 1Mb of memory (maximum cache size * 4)
 */
                                                                                
sdram_init_done:
        TTYDBG("SDRAM Memory setup done.\r\n")

        li      v0, 0xb0000000          /* ra is set from previous bal */
        bgtu    v0, ra, in_ram          /* if pc is lower than rom space.. */
        nop
                                                                                
        TTYDBG("Clearing cache size memory...\r\n")
                                                                                
        la      t0, 0xa0000000
        addu    t1, t0, 1*1024*1024
1:
        addu    t0, 8
        bne     t1, t0, 1b
        sd      zero, -8(t0)
                                                                                

in_ram:
        jal     mips_init_cache
        nop

                                                                                
cache_init_done:
        TTYDBG("Cache init done.\r\n")

        mfc0    t4, C0_PRID
        srl     t4, 24
	bnez	t4, cpu1_init_done
	nop
                                                                                
	/* clear the R9K memory for ECC cksum */
        TTYDBG("Initializing ECC memory.\r\n")

        bal     1f
        nop

        /*
         * ECC init code to be copied to OCRAM
         */
ocramcode:
        /*
         * initialize memory parity for ECC
         */
3:      cache   Create_Dirty_Exc_D,0(a0)
        sd      zero,0(a0)
        sd      zero,8(a0)
        sd      zero,16(a0)
        sd      zero,24(a0)
        cache   Hit_Writeback_Inv_S,0(a0)
        addu    a0,32
        bne     a0,a1,3b
        nop
                                                                                
        j       ra
        nop
ocramcodesz = . - ocramcode
                                                                                
        /*
         * Copy ECC init code to GT on chip ram 
         * and execute from there
         */
1:
        la      v0, OC_SRAM_BASE_CACHED
        li      v1, ocramcodesz
2:
        lw      t0, (ra)
        sw      t0, (v0)
        addiu   v1, -4
        addiu   v0, 4
        bgtz    v1, 2b
        addiu   ra, 4
                                                                                
	li	a0, 0x80000000		/* clear 256 M from 0x00000000 */
	add	a1, a0, 0x10000000
        la      v0, OC_SRAM_BASE_CACHED
        jalr    v0                      /* Execute ecc_init in Marvell ram */
        nop

	li	a0, LOCAL_MEM_BASE	/* clear 256 M from 0x10000000 */
	add	a1, a0, LOCAL_MEM_BASE
        la      v0, OC_SRAM_BASE_CACHED
        jalr    v0                      /* Execute ecc_init in Marvell ram */
        nop

	jal	meminit_decompression_data
	nop
                                                                                
        TTYDBG("Decompression bss start.\r\n")
	jal	meminit_decompression_bss
	nop
                                                                                
        TTYDBG("Init bss.\r\n")
	jal	meminit_bss
	nop

        j       sbdreset_done

cpu1_init_done:
        PRINTSTR("CPU 1 init done. \r\n")
        PRINTSTR("Waiting for CPU 0 to set vector. \r\n")
		la		t0, cpu_startvectors
		or		t0, K1BASE
cpu1_wait:
		lw		t1, 4(t0)
		beqz	t1,	cpu1_wait
		nop
		
		la		t0, cpu_start_spvals
		or		t0, K1BASE
		lw		sp, 4(t0)

        PRINTSTR("Jumping to vector: ")
        move    a0, t1
        jal     hexserial
        nop
        PRINTSTR("\r\n")

		j		t1

        .end    sbdreset
                                                                                



LEAF(wbflush)
        nop
        sync
        j       ra
        nop
END(wbflush)


       .globl  tgt_setpar150mhz
tgt_setpar150mhz:
        move    a0, ra          /* Don't put in delay slot! */
        bal     do_table        /* Load address to init table */
        nop
                                                                                
#ifdef EVAL_BOARD
        /* Device CS0 - FPGA */
        GTINIT(DEVICE_BANK0PARAMETERS, \
                                GT_DEVPAR_TurnOff(3) |          \
                                GT_DEVPAR_AccToFirst(6) |       \
                                GT_DEVPAR_AccToNext(6)  |       \
                                GT_DEVPAR_ALEtoWr(3) |          \
                                GT_DEVPAR_WrActive(3) |         \
                                GT_DEVPAR_WrHigh(5) |           \
                                GT_DEVPAR_DevWidth8 |           \
                                GT_DEVPAR_Reserved)
                                                                                
        /* Device CS1 - NVRAM */
        GTINIT(DEVICE_BANK1PARAMETERS, \
                                GT_DEVPAR_TurnOff(3) |          \
                                GT_DEVPAR_AccToFirst(10) |      \
                                GT_DEVPAR_AccToNext(10) |       \
                                GT_DEVPAR_ALEtoWr(5) |          \
                                GT_DEVPAR_WrActive(6) |         \
                                GT_DEVPAR_WrHigh(5) |           \
                                GT_DEVPAR_DevWidth8 |           \
                                GT_DEVPAR_Reserved)
                                                                                
        /* Device CS2 - UART */
        GTINIT(DEVICE_BANK2PARAMETERS, \
                                GT_DEVPAR_TurnOff(4) |          \
                                GT_DEVPAR_AccToFirst(11) |      \
                                GT_DEVPAR_AccToNext(11) |       \
                                GT_DEVPAR_ALEtoWr(5) |          \
                                GT_DEVPAR_WrActive(6) |         \
                                GT_DEVPAR_WrHigh(5) |           \
                                GT_DEVPAR_DevWidth8 |           \
                                GT_DEVPAR_Reserved)
                                                                                
        /* Device CS3 - Flash Disk */
        GTINIT(DEVICE_BANK3PARAMETERS, \
                                GT_DEVPAR_TurnOff(2) |          \
                                GT_DEVPAR_AccToFirst(19) |      \
                                GT_DEVPAR_AccToNext(19) |       \
                                GT_DEVPAR_ALEtoWr(4) |          \
                                GT_DEVPAR_WrActive(8) |         \
                                GT_DEVPAR_WrHigh(2) |           \
                                GT_DEVPAR_DevWidth8 |           \
                                GT_DEVPAR_BaddrSkew0 |          \
                                GT_DEVPAR_DPEnDisable |         \
                                GT_DEVPAR_Reserved)
                                                                                
        /* Device BootCS - Boot PROM */
        GTINIT(DEVICE_BOOT_BANK_PARAMETERS, \
                                GT_DEVPAR_TurnOff(2) |          \
                                GT_DEVPAR_AccToFirst(19) |      \
                                GT_DEVPAR_AccToNext(19) |       \
                                GT_DEVPAR_ALEtoWr(4) |          \
                                GT_DEVPAR_WrActive(8) |         \
                                GT_DEVPAR_WrHigh(2) |           \
                                GT_DEVPAR_DevWidth8 |           \
                                GT_DEVPAR_BaddrSkew0 |          \
                                GT_DEVPAR_DPEnDisable |         \
                                GT_DEVPAR_Reserved)
#else
        /* Device CS0 - RTC */
        GTINIT(DEVICE_BANK0PARAMETERS, 0xf009e6f7)

        /* Device CS1 - NS UART (byte access) */
        GTINIT(DEVICE_BANK1PARAMETERS, 0x1449e6f0)

        /* Device CS2 - BM FPGA */
        GTINIT(DEVICE_BANK2PARAMETERS, 0xf226e87f)

        /* Device CS3 - Non Boot PROM */
        GTINIT(DEVICE_BANK3PARAMETERS, 0x1449e6f0)

        /* Device BootCS - Boot PROM */
        GTINIT(DEVICE_BOOT_BANK_PARAMETERS, 0x1449e6f0)

        /* Device Control */
        GTINIT(DEVICE_CONTROL, 0x0005ffff)

#endif
                                                                                
        /* end mark */
        .word   0, 0
                                                                                
1:
        sw      v1, 0(v0)
do_table:
        lw      v0, 0(ra)               /* Address */
        lw      v1, 4(ra)               /* Data */
        bnez    v0, 1b
        addiu   ra, 8
                                                                                
        jr      a0
        nop
                                                                                

#endif	/* PROM */

