/*  *********************************************************************
    *  SB1250 Board Support Package
    *  
    *  Board-specific initialization		File: SWARM_INIT.S
    *
    *  This module contains the assembly-language part of the init
    *  code for this board support package.  The routine
    *  "board_earlyinit" lives here.
    *  
    *  Author:  Mitch Lichtenberg (mpl@broadcom.com)
    *  
    *********************************************************************  
    *
    *  Copyright 2000,2001
    *  Broadcom Corporation. All rights reserved.
    *  
    *  This software is furnished under license and may be used and 
    *  copied only in accordance with the following terms and 
    *  conditions.  Subject to these conditions, you may download, 
    *  copy, install, use, modify and distribute modified or unmodified 
    *  copies of this software in source and/or binary form.  No title 
    *  or ownership is transferred hereby.
    *  
    *  1) Any source code used, modified or distributed must reproduce 
    *     and retain this copyright notice and list of conditions as 
    *     they appear in the source file.
    *  
    *  2) No right is granted to use any trade name, trademark, or 
    *     logo of Broadcom Corporation. Neither the "Broadcom 
    *     Corporation" name nor any trademark or logo of Broadcom 
    *     Corporation may be used to endorse or promote products 
    *     derived from this software without the prior written 
    *     permission of Broadcom Corporation.
    *  
    *  3) THIS SOFTWARE IS PROVIDED "AS-IS" AND ANY EXPRESS OR
    *     IMPLIED WARRANTIES, INCLUDING BUT NOT LIMITED TO, ANY IMPLIED 
    *     WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR 
    *     PURPOSE, OR NON-INFRINGEMENT ARE DISCLAIMED. IN NO EVENT 
    *     SHALL BROADCOM BE LIABLE FOR ANY DAMAGES WHATSOEVER, AND IN 
    *     PARTICULAR, BROADCOM SHALL NOT BE LIABLE FOR DIRECT, INDIRECT, 
    *     INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
    *     (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE 
    *     GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
    *     BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY 
    *     OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR 
    *     TORT (INCLUDING NEGLIGENCE OR OTHERWISE), EVEN IF ADVISED OF 
    *     THE POSSIBILITY OF SUCH DAMAGE.
    ********************************************************************* */


#include <sys/mips.h>
#include <sys/sb1250/sb1250_genbus.h>
#include <sys/sb1250/sb1250_regs.h>
#include <sys/sb1250/sb1250_scd.h>
#include <exports/hwMap.h>
#include "bsp_config.h"
#include "swarm.h"

		.text


#define K_EOT 0xFF
#define DRAMINFO(mcchan,smbusdev,rows,cols,banks,sides) \
      .byte mcchan,smbusdev,rows,cols,banks,sides,0,0

/*  *********************************************************************
    *  Macros
    ********************************************************************* */


#if CFG_RUNFROMKSEG0
#define CALLKSEG1(x)  la k0,x ; or k0,K1BASE ; jal k0
#else
#define CALLKSEG1(x)  jal x
#endif


/*  *********************************************************************
    *  BOARD_EARLYINIT()
    *  
    *  Initialize board registers.  This is the earliest 
    *  time the BSP gets control.  This routine cannot assume that
    *  memory is operational, and therefore all code in this routine
    *  must run from registers only.  The $ra register must not
    *  be modified, as it contains the return address.
    *
    *  This routine will be called from uncached space, before
    *  the caches are initialized.  If you want to make
    *  subroutine calls from here, you must use the CALLKSEG1 macro.
    *
    *  Among other things, this is where the GPIO registers get 
    *  programmed to make on-board LEDs function, or other startup
    *  that has to be done before anything will work.
    *  
    *  Input parameters: 
    *  	   nothing
    *  	   
    *  Return value:
    *  	   nothing
    ********************************************************************* */

LEAF(board_earlyinit)

	#
	# Reprogram the SCD to make sure UART0 is enabled.
	# Some CSWARM boards have the SER0 enable bit when
	# they're not supposed to, which switches the UART
	# into synchronous mode.  Kill off the SCD bit.
	# XXX this should be investigated in hardware, as 
	# XXX it is a strap option on the CPU.
	#

		li      t0,PHYS_TO_K1(A_SCD_SYSTEM_CFG)
		ld	t1,0(t0)
		dli	t2,~M_SYS_SER0_ENABLE
		and	t1,t1,t2
		sd	t1,0(t0)

#if defined(EVAL_BOARD)
       #
       # Configure the GPIOs
       #

		li	t0,PHYS_TO_K1(A_GPIO_DIRECTION)
		li	t1,GPIO_OUTPUT_MASK
		sd	t1,0(t0)

		li	t0,PHYS_TO_K1(A_GPIO_INT_TYPE)
		li	t1,GPIO_INTERRUPT_MASK
		sd	t1,0(t0)

       #
       # Turn on the diagnostic LED and turn off the sturgeon NMI
       #
		li	t0,PHYS_TO_K1(A_GPIO_PIN_SET)
		li	t1,M_GPIO_DEBUG_LED
		sd	t1,0(t0)

		li	t0,PHYS_TO_K1(A_GPIO_PIN_CLR)
		li	t1,M_GPIO_STURGEON_NMI
		sd	t1,0(t0)


       #
       # Configure the LEDs
       #     

		li	t0,PHYS_TO_K1(A_IO_EXT_CS_BASE(LEDS_CS))
		li	t1,LEDS_PHYS >> S_IO_ADDRBASE
		sd	t1,R_IO_EXT_START_ADDR(t0)

		li	t1,LEDS_SIZE-1	/* Needs to be 1 smaller, se UM for details */
		sd	t1,R_IO_EXT_MULT_SIZE(t0)

		li	t1,LEDS_TIMING0
		sd	t1,R_IO_EXT_TIME_CFG0(t0)

		li	t1,LEDS_TIMING1
		sd	t1,R_IO_EXT_TIME_CFG1(t0)

		li	t1,LEDS_CONFIG
		sd	t1,R_IO_EXT_CFG(t0)



       #
       # Configure the alternate boot ROM
       #     

		li	t0,PHYS_TO_K1(A_IO_EXT_CS_BASE(ALT_BOOTROM_CS))

		li	t1,ALT_BOOTROM_PHYS >> S_IO_ADDRBASE
		sd	t1,R_IO_EXT_START_ADDR(t0)

		li	t1,ALT_BOOTROM_SIZE-1
		sd	t1,R_IO_EXT_MULT_SIZE(t0)

		li	t1,ALT_BOOTROM_TIMING0
		sd	t1,R_IO_EXT_TIME_CFG0(t0)

		li	t1,ALT_BOOTROM_TIMING1
		sd	t1,R_IO_EXT_TIME_CFG1(t0)

		li	t1,ALT_BOOTROM_CONFIG
		sd	t1,R_IO_EXT_CFG(t0)



       #
       # Configure the IDE interface
       #     

		li	t0,PHYS_TO_K1(A_IO_EXT_CS_BASE(IDE_CS))

		li	t1,IDE_PHYS >> S_IO_ADDRBASE
		sd	t1,R_IO_EXT_START_ADDR(t0)

		li	t1,IDE_SIZE-1
		sd	t1,R_IO_EXT_MULT_SIZE(t0)

		li	t1,IDE_TIMING0
		sd	t1,R_IO_EXT_TIME_CFG0(t0)

		li	t1,IDE_TIMING1
		sd	t1,R_IO_EXT_TIME_CFG1(t0)

		li	t1,IDE_CONFIG
		sd	t1,R_IO_EXT_CFG(t0)


       #
       # Configure the PCMCIA
       #     

		li	t0,PHYS_TO_K1(A_IO_EXT_CS_BASE(PCMCIA_CS))

		li	t1,PCMCIA_PHYS >> S_IO_ADDRBASE
		sd	t1,R_IO_EXT_START_ADDR(t0)

		li	t1,PCMCIA_SIZE-1
		sd	t1,R_IO_EXT_MULT_SIZE(t0)

		li	t1,PCMCIA_TIMING0
		sd	t1,R_IO_EXT_TIME_CFG0(t0)

		li	t1,PCMCIA_TIMING1
		sd	t1,R_IO_EXT_TIME_CFG1(t0)

		li	t1,PCMCIA_CONFIG
		sd	t1,R_IO_EXT_CFG(t0)

#else

       #
       # Configure the Bootrom
       #     
		li	t0,PHYS_TO_K1(A_IO_EXT_CS_BASE(BOOTROM_CS))

		li	t1,BOOTROM_TIMING1
		sd	t1,R_IO_EXT_TIME_CFG1(t0)

       #
       # Configure the BMFPGA
       #     
		li	t0,PHYS_TO_K1(A_IO_EXT_CS_BASE(BMFPGA_CS))

		li	t1,BMFPGA_PHYS >> S_IO_ADDRBASE
		sd	t1,R_IO_EXT_START_ADDR(t0)

		li	t1,BMFPGA_SIZE-1
		sd	t1,R_IO_EXT_MULT_SIZE(t0)

		li	t1,BMFPGA_TIMING0
		sd	t1,R_IO_EXT_TIME_CFG0(t0)

		li	t1,BMFPGA_TIMING1
		sd	t1,R_IO_EXT_TIME_CFG1(t0)

		li	t1,BMFPGA_CONFIG
		sd	t1,R_IO_EXT_CFG(t0)


       #
       # Configure the LUC Registers
       #     
		li	t0,PHYS_TO_K1(A_IO_EXT_CS_BASE(LUC_REG_CS))

		li	t1,LUC_REG_PHYS >> S_IO_ADDRBASE
		sd	t1,R_IO_EXT_START_ADDR(t0)

		li	t1,LUC_REG_SIZE-1
		sd	t1,R_IO_EXT_MULT_SIZE(t0)

		li	t1,LUC_REG_TIMING0
		sd	t1,R_IO_EXT_TIME_CFG0(t0)

		li	t1,LUC_REG_TIMING1
		sd	t1,R_IO_EXT_TIME_CFG1(t0)

		li	t1,LUC_REG_CONFIG
		sd	t1,R_IO_EXT_CFG(t0)


/*
 * isCheetah ?
 */
		la		t0,PHYS_TO_K1(A_SCD_SYSTEM_CFG)
		ld		t0,0(t0)
		li		t1,0x00000000fc000000
		and		t0,t0,t1
		srl		t0,27
		and		t0,1
		beqz	t0,fpnim_init_not_cheetah

fpnim_init_cheetah:
/*
 *  is Cheetah, configure LSI_MAC_REG + BIG_BOOTROM + XILINX_PROGRAM
 */
       #
       # Configure the LSI_MAC_REG
       #     
		li	t0,PHYS_TO_K1(A_IO_EXT_CS_BASE(CHEETAH_LSI_MAC_REG_CS))

		li	t1,LSI_MAC_REG_PHYS >> S_IO_ADDRBASE
		sd	t1,R_IO_EXT_START_ADDR(t0)

		li	t1,LSI_MAC_REG_SIZE-1
		sd	t1,R_IO_EXT_MULT_SIZE(t0)

		li	t1,LSI_MAC_REG_TIMING0
		sd	t1,R_IO_EXT_TIME_CFG0(t0)

		li	t1,LSI_MAC_REG_TIMING1
		sd	t1,R_IO_EXT_TIME_CFG1(t0)

		li	t1,LSI_MAC_REG_CONFIG
		sd	t1,R_IO_EXT_CFG(t0)

       #
       # Configure the BIG_BOOTROM
       #     
		li	t0,PHYS_TO_K1(A_IO_EXT_CS_BASE(BIG_BOOTROM_CS))

		li	t1,BIG_BOOTROM_PHYS >> S_IO_ADDRBASE
		sd	t1,R_IO_EXT_START_ADDR(t0)

		li	t1,BIG_BOOTROM_SIZE-1
		sd	t1,R_IO_EXT_MULT_SIZE(t0)

		li	t1,BIG_BOOTROM_TIMING0
		sd	t1,R_IO_EXT_TIME_CFG0(t0)

		li	t1,BIG_BOOTROM_TIMING1
		sd	t1,R_IO_EXT_TIME_CFG1(t0)

		li	t1,BIG_BOOTROM_CONFIG
		sd	t1,R_IO_EXT_CFG(t0)

#ifndef PROM_SIBYTE_BC // 01/06/04: Disabled for now
	/* based on jmpr, load from DIAG partition */
		la	a0,0xb0061aa0
		lw	t0,0(a0)
		and	t0,0x00000002
		srl	t0,1
		bne	t0,1,prom_cont
		nop
		la	a0,0xbfcc0000
		j	a0
		nop
#endif
       #
       # Configure the XILINX_PROGRAM
       #     
prom_cont:
		li	t0,PHYS_TO_K1(A_IO_EXT_CS_BASE(XILINX_PROGRAM_CS))

		li	t1,XILINX_PROGRAM_PHYS >> S_IO_ADDRBASE
		sd	t1,R_IO_EXT_START_ADDR(t0)

		li	t1,XILINX_PROGRAM_SIZE-1
		sd	t1,R_IO_EXT_MULT_SIZE(t0)

		li	t1,XILINX_PROGRAM_TIMING0
		sd	t1,R_IO_EXT_TIME_CFG0(t0)

		li	t1,XILINX_PROGRAM_TIMING1
		sd	t1,R_IO_EXT_TIME_CFG1(t0)

		li	t1,XILINX_PROGRAM_CONFIG
		sd	t1,R_IO_EXT_CFG(t0)

		b	fpnim_init_cont

fpnim_init_not_cheetah:
/*
 * not Cheetah, configure LSI_MAC_REG
 */
       #
       # Configure the LSI_MAC_REG
       #     
		li	t0,PHYS_TO_K1(A_IO_EXT_CS_BASE(LSI_MAC_REG_CS))

		li	t1,LSI_MAC_REG_PHYS >> S_IO_ADDRBASE
		sd	t1,R_IO_EXT_START_ADDR(t0)

		li	t1,LSI_MAC_REG_SIZE-1
		sd	t1,R_IO_EXT_MULT_SIZE(t0)

		li	t1,LSI_MAC_REG_TIMING0
		sd	t1,R_IO_EXT_TIME_CFG0(t0)

		li	t1,LSI_MAC_REG_TIMING1
		sd	t1,R_IO_EXT_TIME_CFG1(t0)

		li	t1,LSI_MAC_REG_CONFIG
		sd	t1,R_IO_EXT_CFG(t0)

fpnim_init_cont:
		#
		# Set up Output Drive Control Registers
		#
		la	a0,PHYS_TO_K1(A_IO_DRIVE_0)
		li	t0,0x000077F7
		sw	t0,0(a0)

		la	a0,PHYS_TO_K1(A_IO_DRIVE_1)
		li	t0,0x00007777
		sw	t0,0(a0)

		la	a0,PHYS_TO_K1(A_IO_DRIVE_2)
		li	t0,0x0000FF77
		sw	t0,0(a0)

		la	a0,PHYS_TO_K1(A_IO_DRIVE_3)
		li	t0,0x0000FFFF
		sw	t0,0(a0)

#endif	/* EVAL_BOARD */

		j	ra

END(board_earlyinit)


/*  *********************************************************************
    *  BOARD_DRAMINFO
    *  
    *  Return the address of the DRAM information table
    *  
    *  Input parameters: 
    *  	   nothing
    *  	   
    *  Return value:
    *  	   v0 - DRAM info table, return 0 to use default table
    ********************************************************************* */

#if (CFG_EMBEDDED_PIC)
#define LOADREL(reg,label)			\
	.set noreorder ;			\
	bal  1f	       ;			\
	nop	       ;			\
1:	nop	       ;			\
	.set reorder   ;			\
	la   reg,label-1b ;			\
	add  reg,ra
#else
#define	LOADREL(reg,label)			\
	la   reg,label
#endif

LEAF(board_draminfo)

		move	t0,ra

#ifdef _HARDWIRED_MEMORY_TABLE
		LOADREL(v0,myinfo)
#else
		move	v0,zero		# auto configure
#endif

		move	ra,t0
		j	ra



myinfo:
	#       Chan SMBusAdr   Rows Cols Bnks ChpSel
        #       ---- ---------  ---- ---- ---- ------
	DRAMINFO(0,  0,         12,  8,   2,   0);
	DRAMINFO(K_EOT,0,0,0,0,0);

END(board_draminfo)


/*  *********************************************************************
    *  BOARD_SETLEDS(x)
    *  
    *  Set LEDs for boot-time progress indication.  Not used if
    *  the board does not have progress LEDs.  This routine
    *  must not call any other routines, since it may be invoked
    *  either from KSEG0 or KSEG1 and it may be invoked 
    *  whether or not the icache is operational.
    *  
    *  Input parameters: 
    *  	   a0 - LED value (8 bits per character, 4 characters)
    *  	   
    *  Return value:
    *  	   nothing
    *  
    *  Registers used:
    *  	   t0,t1,t2,t3
    ********************************************************************* */


#define LED_CHAR0	(32+8*3)
#define LED_CHAR1	(32+8*2)
#define LED_CHAR2	(32+8*1)
#define LED_CHAR3	(32+8*0)

LEAF(board_setleds)

#ifdef EVAL_BOARD
		li	t0,PHYS_TO_K1(LEDS_PHYS)

		rol	a0,a0,8
		and	t1,a0,0xFF
		sb	t1,LED_CHAR0(t0)

		rol	a0,a0,8
		and	t1,a0,0xFF
		sb	t1,LED_CHAR1(t0)

		rol	a0,a0,8
		and	t1,a0,0xFF
		sb	t1,LED_CHAR2(t0)

		rol	a0,a0,8
		and	t1,a0,0xFF
		sb	t1,LED_CHAR3(t0)
#else
#if 0
		move	t4,ra

		rol	a0,a0,8
		and	t5,a0,0xFF
		rol	a0,a0,8
		and	t6,a0,0xFF
		rol	a0,a0,8
		and	t7,a0,0xFF
		rol	a0,a0,8
		and	t8,a0,0xFF

		move	a0,t5
		la		t0,promio_put_char
		jalr	t0
		nop
		move	a0,t6
		la		t0,promio_put_char
		jalr	t0
		nop
		move	a0,t7
		la		t0,promio_put_char
		jalr	t0
		nop
		move	a0,t8
		la		t0,promio_put_char
		jalr	t0
		nop

		move	ra,t4
#endif
#endif	/* EVAL_BOARD */

		j	ra

END(board_setleds)

