/*
 *-----------------------------------------------------------------
 *
 * Name:	nfx-init.S
 *
 * RCS:  $Id: nfx-init.S,v 1.1.1.1 2001/12/31 19:35:32 rick Exp $
 *
 * Copyright (C) 2001, Agile Storage, Inc.
 *
 * Description:	low level board initialization - PROM code
 *
 * Created by:	Rick Lund
 *
 * Date Created:	12/20/01
 *
 *-----------------------------------------------------------------
 */

#ifdef PROM

#include <nfx-incl.h>
#include <sys/mips.h>
#include <exports/hwMap.h>
#include <sys/sb1250/sb1250_genbus.h>

	.text

	.globl	nfx_init

	.extern	nfx_init_done

	.set	noreorder

#define BMFPGA_SIZE     NUM64K(1*MB)    /* size of BMFPGA */

#define BMFPGA_TIMING0  V_IO_ALE_WIDTH(1) | \
                        V_IO_ALE_TO_CS(2) | \
                        V_IO_CS_WIDTH(24) | \
                        V_IO_RDY_SMPLE(3)

#define BMFPGA_TIMING1  V_IO_ALE_TO_WRITE(7) | \
                        V_IO_WRITE_WIDTH(15) | \
                        V_IO_IDLE_CYCLE(6) | \
                        V_IO_CS_TO_OE(0) | \
                        V_IO_OE_TO_CS(0)

#define BMFPGA_CONFIG   V_IO_WIDTH_SEL(K_IO_WIDTH_SEL_4) | \
                        V_IO_TIMEOUT(0x38)

#define LUC_REG_SIZE    NUM64K(1*MB)    /* size of LUC_REG */

#define LUC_REG_TIMING0 V_IO_ALE_WIDTH(1) | \
                        V_IO_ALE_TO_CS(2) | \
                        V_IO_CS_WIDTH(24) | \
                        V_IO_RDY_SMPLE(3)

#define LUC_REG_TIMING1 V_IO_ALE_TO_WRITE(7) | \
                        V_IO_WRITE_WIDTH(7) | \
                        V_IO_IDLE_CYCLE(6) | \
                        V_IO_CS_TO_OE(0) | \
                        V_IO_OE_TO_CS(0)

#define LUC_REG_CONFIG  V_IO_WIDTH_SEL(K_IO_WIDTH_SEL_4) | \
                        V_IO_TIMEOUT(0x38)

#define LSI_MAC_REG_SIZE    NUM64K(1*MB)    /* size of LSI_MAC_REG */

#define LSI_MAC_REG_TIMING0 V_IO_ALE_WIDTH(1) | \
                            V_IO_ALE_TO_CS(2) | \
                            V_IO_CS_WIDTH(24) | \
                            V_IO_RDY_SMPLE(3)

#define LSI_MAC_REG_TIMING1 V_IO_ALE_TO_WRITE(7) | \
                            V_IO_WRITE_WIDTH(7) | \
                            V_IO_IDLE_CYCLE(6) | \
                            V_IO_CS_TO_OE(0) | \
                            V_IO_OE_TO_CS(0)

#define LSI_MAC_REG_CONFIG  V_IO_WIDTH_SEL(K_IO_WIDTH_SEL_4) | \
                            V_IO_TIMEOUT(0x38)


/*-----------------------------------------------------------------
 * Name:	nfx_init
 *
 * Description:
 *				galileo init
 *				uart init
 *				nsdb
 *
 * Created by:	Rick Lund
 *
 * Date Created:	9/13/01
 *
 *-----------------------------------------------------------------
 */
	.ent	nfx_init
nfx_init:

#ifdef AGILE_HARDWARE

       #
       # Configure the BMFPGA
       #     
		li	t0,PHYS_TO_K1(A_IO_EXT_CS_BASE(BMFPGA_CS))

		li	t1,BMFPGA_PHYS >> S_IO_ADDRBASE
		sd	t1,R_IO_EXT_START_ADDR(t0)

		li	t1,BMFPGA_SIZE-1
		sd	t1,R_IO_EXT_MULT_SIZE(t0)

		li	t1,BMFPGA_TIMING0
		sd	t1,R_IO_EXT_TIME_CFG0(t0)

		li	t1,BMFPGA_TIMING1
		sd	t1,R_IO_EXT_TIME_CFG1(t0)

		li	t1,BMFPGA_CONFIG
		sd	t1,R_IO_EXT_CFG(t0)

       #
       # Configure the LUC Registers
       #     
		li	t0,PHYS_TO_K1(A_IO_EXT_CS_BASE(LUC_REG_CS))

		li	t1,LUC_REG_PHYS >> S_IO_ADDRBASE
		sd	t1,R_IO_EXT_START_ADDR(t0)

		li	t1,LUC_REG_SIZE-1
		sd	t1,R_IO_EXT_MULT_SIZE(t0)

		li	t1,LUC_REG_TIMING0
		sd	t1,R_IO_EXT_TIME_CFG0(t0)

		li	t1,LUC_REG_TIMING1
		sd	t1,R_IO_EXT_TIME_CFG1(t0)

		li	t1,LUC_REG_CONFIG
		sd	t1,R_IO_EXT_CFG(t0)

       #
       # Configure the LSI_MAC_REG
       #     
		li	t0,PHYS_TO_K1(A_IO_EXT_CS_BASE(LSI_MAC_REG_CS))

		li	t1,LSI_MAC_REG_PHYS >> S_IO_ADDRBASE
		sd	t1,R_IO_EXT_START_ADDR(t0)

		li	t1,LSI_MAC_REG_SIZE-1
		sd	t1,R_IO_EXT_MULT_SIZE(t0)

		li	t1,LSI_MAC_REG_TIMING0
		sd	t1,R_IO_EXT_TIME_CFG0(t0)

		li	t1,LSI_MAC_REG_TIMING1
		sd	t1,R_IO_EXT_TIME_CFG1(t0)

		li	t1,LSI_MAC_REG_CONFIG
		sd	t1,R_IO_EXT_CFG(t0)

		#
		# Set up Output Drive Control Registers
		#
		la	a0,PHYS_TO_K1(A_IO_DRIVE_0)
		li	t0,0x00007777
		sw	t0,0(a0)

		la	a0,PHYS_TO_K1(A_IO_DRIVE_1)
		li	t0,0x00007777
		sw	t0,0(a0)

		la	a0,PHYS_TO_K1(A_IO_DRIVE_2)
		li	t0,0x0000FF77
		sw	t0,0(a0)

		la	a0,PHYS_TO_K1(A_IO_DRIVE_3)
		li	t0,0x0000FFFF
		sw	t0,0(a0)

		j	nfx_init_done
		nop
#endif

	.end	nfx_init

#endif	/* PROM */
