/* vi:ts=4
 *-----------------------------------------------------------------
 *
 * Name:	sbd.c
 *
 * RCS:  $Id: sbd.c,v 1.22 2002/10/10 18:36:08 johnh Exp $
 *
 * Copyright (C) 2001, Agile Storage, Inc.
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	9/19/01
 *
 *-----------------------------------------------------------------
 */

#ifdef PROM

#include <nfx-target.h>
#include <sys/mips.h>
#include <exports/hwMap.h>
#include "../prom-pmon/pmon.h"
#include <pmon/termio.h>
#include "sbd.h"
#include <sys/mips.h>

extern int	ns16550();
extern void xlnx_download_sel(int, int);

ConfigEntry     ConfigTable[] =
{
#ifdef EVAL_BOARD
/*
	{(Addr)PHYS_TO_K1(HW_ST16552_1_BASE), 0, ns16550, 256, B57600},
	{(Addr)PHYS_TO_K1(HW_ST16552_0_BASE), 0, ns16550, 256, B57600},
*/
#else
	{0}
#endif
};


void
sbdpoll()
{
}

/* display a message (four chars packed into one word),
   for very low-level code. */
void
sbddisplay (unsigned long msg, unsigned int x)
{
	board_setleds(msg);
}


/* display a long message on the display, scrolling if necessary. */
void
sbdmessage (int line, const char *msg)
{
}

/* display a message and then pause so that it can be read */
static void
display (const char *msg)
{
}

/* display a register name and its value, then pause */
static void
xdisplay (char *s, unsigned int x)
{
}

char *
sbddbgintr (unsigned int cr)
{
	return 0;
}

char *
sbdexception (epc, cause, ra, extra, exc)
	unsigned long   epc, cause, ra, extra;
	char *exc;
{
}

_fpstatesz ()
{
	return 0;
}

_fpstate ()
{
	return -1;
}

cop1 ()
{
	printf ("\ncop1 called\n");
	cliexit ();
}

getmachtype ()
{
	return 1250;
}

#ifndef INET
/* gsignal() is called when a keyboard interrupt (^C) is detected */
gsignal (jmp_buf *jb, int sig)
{
	if (jb) longjmp (jb, 1);
}

netopen()	{return -1;}
netread()	{return -1;}
netwrite()	{return -1;}
netclose()	{return -1;}
netlseek()	{return -1;}
netioctl()	{return -1;}
netpoll()	{}


unsigned long __time (void)
{
#ifdef RTC
	return (sbd_gettime ());
#else
	return 0;
#endif
}
#endif

unsigned int pipefreq = 0;
int cpuspeed = 100;

/* hand tune delay */
static void
probefreq(void)
{
#ifdef RTC
	unsigned long start, cnt, ohead, timeout;
	int i;

	SBD_DISPLAY ("FREQ", CHKPNT_FREQ);

	pipefreq = 133333333;   /* default value */

	/* select interrupt register */
	outb (RTC_ADDR_PORT, RTC_INTR);

	/* run more than once to be sure that we're in the cache */
	for (i = 2; i != 0; i--) {
	start = get_count ();
	/* clear any pending rtc i/u */
	(void) inb (RTC_DATA_PORT);
	/* wait until we see an RTC i/u */
	timeout = 10000000 / RTC_HZ;
	while ((icu->irr.dev & ICU_DEV_RTC) == 0)
		if (--timeout == 0) goto done;
	cnt = get_count ();
	}
	cnt -= start;

	/* discover icu read overhead */

	for (i = 2; i != 0; i--) {
	start = get_count ();
	timeout = 10000000 / RTC_HZ;
	while ((icu->irr.dev & ICU_DEV_RTC) == 0)
		if (--timeout == 0) goto done;
	ohead = get_count ();
	}
	ohead -= start;
	ohead /= 2;         /* average */

	/* work out number of cpu ticks per sec, the cpu pipeline */
	pipefreq = cnt * RTC_HZ;
	ohead *= RTC_HZ;        /* XXX is this right? */
	/*printf ("(%d-%d->", pipefreq, ohead);*/
	pipefreq -= ohead;

	/* All R5000 processors clock the count register at 1/2 of PClock */
	pipefreq *= 2;

	/* round frequency to 3 decimal places */
	pipefreq += 500;
	pipefreq -= pipefreq % 1000;
	/*printf ("%d)", pipefreq);*/
#else
    int   us = 1000000;
    unsigned int  c1, c2;
    unsigned int  freq;

    c1 = _mips_mfc0(C0_COUNT);
    delay_us(us);
    c2 = _mips_mfc0(C0_COUNT);

//    printf("c1 = %d, c2 = %d, ", c1, c2);

//  printf("c2-c1 = %d, pipefreq = %d, ", c2-c1, (c2-c1)/us);
    freq = ((c2-c1)+500)/us;
//    printf("adjusted freq = %d\n", freq);

	pipefreq = freq * 1000000;
#endif
done:
	if (IS_KVA0 (&probefreq))
	cpuspeed = (pipefreq/2) / 1000000;
	else
	cpuspeed = 1;
}

unsigned int sbdpipefreq (void)
{
	if (pipefreq == 0)
		probefreq ();
	return pipefreq;
}

/* SysClk to PipeClk ratio table */
struct ratio {char mult; char div;};
static const struct ratio ratios_rm7k[8] = {{2,1}, {3,1}, {4,1}, {5,2},
											{6,1}, {7,2}, {8,1}, {9,2}};

unsigned int sbdcpufreq (void)
{
#ifdef SB1250
	return sbdpipefreq();
#else
	unsigned int ec = (Config & CFG_ECMASK) >> CFG_ECSHIFT;
	const struct ratio *ratio;

	switch (getmachtype ()) {
		case 1250:
		case 7000:
		case 7010:
			ratio = &ratios_rm7k[ec];
			break;
		default:
			printf("sbdcpufreq, invalid machtype, %d\n", getmachtype());
			break;
	}

	/* For PipeClk to SysClk reverse the ratio calculation */
	return sbdpipefreq () * ratio->div / ratio->mult;
#endif
}

void
sbdmachinfo (void)
{
    extern int mips_icache_size, mips_icache_linesize;
    extern int mips_dcache_size, mips_dcache_linesize;
    extern int mips_scache_size, mips_scache_linesize;
    extern int mips_tcache_size, mips_tcache_linesize;
    extern int mips_icache_ways, mips_dcache_ways;
    extern int mips_scache_ways, mips_tcache_ways;
    extern int mips_scache_split, mips_scache_discontig;

	mips_icache_size = 0x8000;
	mips_dcache_size = 0x8000;
	mips_scache_size = 0x40000;
	mips_tcache_size = 0;
	mips_icache_linesize = 32;
	mips_dcache_linesize = 32;
	mips_scache_linesize = 32;
	mips_tcache_linesize = 0;
	mips_icache_ways = 4;
	mips_dcache_ways = 4;
	mips_scache_ways = 4;
	mips_scache_split = 0;
	mips_scache_discontig = 0;
}

unsigned int
sbdenable (int machtype)
{
	unsigned int sr;

#ifdef FLASH
	{
		/* now is the chance to report any early eerom failures */
		extern const char *_sbd_envinit (void);
		const char * err = _sbd_envinit ();
		if (err)
			printf ("WARNING: flash environment: %s\n", err);
	}
#endif

#if RAL_XXX
	/* clear interrupt latches */
	icu->ctrl.clear = ~0;

	/* all external interrupts masked initially */
	imask_set (0);
#endif

#if RAL_XXX
	/* enable panic errors */
	sr = SR_IBIT7 | SR_IE;
#else
	sr = 0;
#endif

	(void) bis_sr (sr);
	return sr;
}


#define WDELAY()							\
{											\
	register int i;							\
	wbflush();								\
	for (i = DELAY; i != 0; i--) continue;	\
}

void
sbddevinit ()
{
	unsigned int	read;
	extern int warm_start;
	char	ipaddr[16];
	unsigned int ipaddr_uint;

	mem_map_tlb_init();

	eee_init(1);

/*
	pci_configure(0x5);
*/

	pci_init();

	if (isTXRX()) {
		/* make sure phys are out of reset */
		*(volatile int *)(int)0xbf400000 = 0xffffffff;
		delay_us(1000);
		*(volatile int *)(int)0xbf400000 = 0;
		delay_us(1000);
		mbinit();
		pkt_init();
		luc_init();
		tpl_init();
	}

	eee_init(2);

	q_initQueue();
	eee_timerInit();
	timer_ticInit();

	eee_initFragmentTimeout();

	rconInit();

    seepInit();

	if (isTXRX()) {
		seepGetIpAddr(ipaddr);
		getIpAddr(ipaddr, &ipaddr_uint);
		seepInitExtMacAddr(ipaddr_uint);

#ifndef PROM_SIBYTE_BC		
		dp83820_initModule();
		xlnx_download_sel(1, 1);  // load LMUX only
#endif		
		xlnx_download_sel(1, 2);  // load LUC only
	}

	bmc12500Eth_initModule();
	sb1250dm_initModule();

	if (isTXRX()) {
		tftp_init();
		domaininit();
		luc_postInit();

		seepGetIpAddr(ipaddr);
		luc_simStart_setIpAddr(ipaddr);
		luc_simStart_withArgs(2088);
	}

}


const char *
sbdgetname ()
{
	return "FP";
	return "TXRX";
}


/* early low-level machine initialisation */
void
sbdmachinit ()
{
}

#ifndef RTC
long
sbd_gettime (void)
{
	return 0;
}
#endif


#ifndef FLASH
/*
 * dummy nvram handler for systems without nvram.
 */

char *
sbd_getenv (name)
	char *name;
{
	return 0;
}


int
sbd_setenv (char *name, char *value)
{
	return 1;
}


int
sbd_unsetenv (char *name)
{
	return 1;
}


void
sbd_mapenv (int (*func)(char *, char *))
{
}

void
sbd_flashinfo (void **buf, int *size)
{
	buf = 0;
	size = 0;
}


int
sbd_flashprogram (void *buf, int size)
{
	return 0;
}
#endif

unsigned long long
get_memorysize()
{
	return memorysize;
}
#endif	/* PROM */

