#include "nfx-incl.h"


#include <sys/wait.h>
#include <sys/uio.h>
#include <readline/history.h>
#include <fcntl.h>

#include "ef-fence.h"

/**
 * Electric fence.
 */
#ifdef EFENCE
extern void *ef_malloc(size_t size);
extern void ef_free(void *ptr);

void fs_memInit(void)
{
    // initialize the electric fence heap corruption detector
}   

#endif


/**
 * Default memory allocator.
 */
#if 0
#if !defined(EFENCE)
void fs_memInit(void)
{
}
#endif
#endif


/**
 * Allocate memory.
 */
void *ef_eee_ramAlloc(uint32 numBytes)
{
#ifdef EFENCE
    void *ptr;
    
    printf ("Efence Alloc\n");
    ptr = ef_malloc(numBytes);
#else
    void *ptr = (void *)malloc(numBytes);
#endif

    //printf("allocing %p\n", ptr);

    return ptr;
}


/**
 * Simulator doesn't need to distinguish this from
 * normal malloc.
 */
void *
ef_eee_ramAllocAligned(uint32 numBytes)
{
    return ef_eee_ramAlloc(numBytes);
}


/**
 * Deallocate memory.  For some inane reason, this routine has a return code.
 */
int32
ef_eee_ramDealloc(void *ptr)
{
    //printf("freeing %p\n", ptr);

#ifdef EFENCE
    printf ("Efence Dealloc\n");
    ef_free(ptr);
#else
    free(ptr);
#endif

    return NFX_OK;
}
