/*
 *------------------------------------------------------------------------------
 * Copyright (C) 2001, Agile Storage, Inc.
 *
 * Name         : target.c
 *
 * RCS          :
 *
 * Description  : Action functions for the AM agent on the target.
 *
 * Created by   : Nitin Jain
 *
 * Date Created : 23-JAN-2002
 *
 * History      : Date       Name         Comments
 *                --------------------------------
 *                1/23/02    Nitin        created
 * 
 *------------------------------------------------------------------------------
 */
#ifndef SSC

#include "agent.h"
 
/*
 * Prototypes of local functions.
 *
 */

void am_post_event_abort(am_agCb_t *agtCb, char *msg);

/*
 * Local functions.
 */

/*
 *------------------------------------------------------------------------------
 * Name         : am_writeCOD_to_disk
 *
 * Description  :
 *
 * Created by   : Nitin Jain
 *
 *------------------------------------------------------------------------------
 */
static void am_writeCOD_to_disk (am_agCb_t *agtCb)
{
    char                *req = NULL;
    evm_IOReq_t         evmReq;
    am_rc_t             rc = NFX_OK;
    dm_rcvInd_t         *dmReq = NULL;
    am_data_msg_hdr_t   *amReq = NULL;
#ifndef INCLUDE_EVM_ON_TARGET
    evm_IOStat_t        evmRsp;
#endif
    uint64              appHdl = (uint64)agtCb;

    AM_DBG_PRINT(("am_writeCOD_to_disk..."));

    /*
     * EVM request. Check all the parameters!!!
     */
    req   = (char *)agtCb->msgQ->rcvMsg;
    dmReq = (dm_rcvInd_t *)req;
    amReq = (am_data_msg_hdr_t *)(req + sizeof(dm_rcvInd_t));

#ifdef INCLUDE_EVM_ON_TARGET
    bzero(&evmReq, sizeof(evm_IOReq_t));
    evmReq.reqType  = EVM_WRITE_REQ;
    evmReq.dataType = EVM_USER_DATA_REQ;
    memcpy (&(evmReq.id), &(agtCb->amVolId), sizeof(evm_volId_t));
    evmReq.offset   = amReq->blkOffSet;
    evmReq.reqSize  = amReq->blkCnt;         // # of blocks
    evmReq.wDataHandle.codHandle = dmReq->cod;
    evmReq.clientPrivate = appHdl;
    evmReq.codOffset = 0;

    AM_DBG_PRINT(("Target -> Write COD: blk: %d cod: %lld datasize: %d", 
			amReq->blkOffSet, (uint64)dmReq->cod, (uint32)dmReq->dataSize));
			
    rc = evm_IOReq(&evmReq, target_evmIOStat);
    M_POST_ABORT_EVENT(agtCb, "EVM IO Write Failed", rc);
#else
    bzero(&evmRsp, sizeof(evm_IOStat_t));
    evmRsp.status = EVM_STAT_NO_ERROR;
    evmRsp.clientPrivate = appHdl;
    target_evmIOStat(&evmRsp);
#endif

    AM_DBG_PRINT(("...am_writeCOD_to_disk"));
    return;
}



/*
 *------------------------------------------------------------------------------
 * Name         : am_free_queued_requests
 *
 * Description  : Free queued requests
 *
 * Created by   : Nitin Jain
 *
 *------------------------------------------------------------------------------
 */
void am_free_queued_requests(am_agCb_t *agtCb)
{
    am_rcv_msg_t  *nxt = NULL, *cur = NULL;

    AM_DBG_PRINT(("am_free_queued_requests..."));

    cur = agtCb->msgQ;
    while(cur)
    {
        nxt = cur->next;
        am_eee_ramDealloc(cur->rcvMsg);
        am_eee_ramDealloc(cur);
        cur = nxt;
    }
    agtCb->msgQ    = NULL;
    agtCb->lastMsg = NULL;

    AM_DBG_PRINT(("...am_free_queued_requests"));
    return;
}


/*
 * Global (action) functions.
 */

/*
 *------------------------------------------------------------------------------
 * Name         : target_create
 *
 * Description  : Create function for target state machine.
 *
 * Created by   : Nitin Jain
 *
 *------------------------------------------------------------------------------
 */
int32 target_create(esm_handle_t  handle,
                    int32         nextState,
                    esm_event_t   thisEvent,
                    void          *userInfo)
{
    AM_FCT_PRINT(("target_create..."));

    /*
     * Nothing to do here.
     */

    AM_DBG_PRINT(("...target_create"));
    return (nextState);
}


/*
 *------------------------------------------------------------------------------
 * Name         : target_bad_state
 *
 * Description  :
 *
 * Created by   : Nitin Jain
 *
 *------------------------------------------------------------------------------
 */
int32 target_bad_state (esm_handle_t  handle,
                        int32         nextState,
                        esm_event_t   thisEvent,
                        void          *userInfo)
{
    AM_ERR_PRINT(("target_bad_state..."));

    /*
     * Nothing to do here.
     */
    if (thisEvent.p1.u.p)
        am_eee_ramDealloc(thisEvent.p1.u.p);

    AM_ERR_PRINT(("...target_bad_state"));
    return (nextState);
}


/*
 *------------------------------------------------------------------------------
 * Name         : target_auth
 *
 * Description  :
 *
 * Created by   : Nitin Jain
 *
 *------------------------------------------------------------------------------
 */
int32 target_auth (esm_handle_t  handle,
                   int32         nextState,
                   esm_event_t   thisEvent,
                   void          *userInfo)
{
    esm_event_t     newEvent;
    am_rc_t         rc = 0;

    AM_DBG_PRINT(("target_auth..."));

    /*
     * We need to implement a new message/way to do the basic authentication
     * for a connect request from source!!!
     *
     * For now just generate authOK event.
     */

	/* 
	 * TBD see if source is allowed to mirror on our volumes
	 */

    bzero(&newEvent, sizeof(esm_event_t));
    newEvent.event    = TARGET_EVENT_AUTH_CONN_OK;
    newEvent.p1.u.dw  = 0;
    newEvent.p2.u.dw  = 0;

    AM_FCT_PRINT(("Posting TARGET_EVENT_AUTH_CONN_OK %d to %d SM", \
                       newEvent.event, handle));
    rc = esm_deliverEvent(handle, &newEvent);
    NFX_ASSERT(rc == NFX_OK);
    if (thisEvent.p1.u.p)
        am_eee_ramDealloc(thisEvent.p1.u.p);

    AM_DBG_PRINT(("...target_auth"));
    return (nextState);
}


/*
 *------------------------------------------------------------------------------
 * Name         : target_authOK
 *
 * Description  :
 *
 * Created by   : Nitin Jain
 *
 *------------------------------------------------------------------------------
 */
int32 target_authOK (esm_handle_t  handle,
                     int32         nextState,
                     esm_event_t   thisEvent,
                     void          *userInfo)
{
    am_agCb_t       *agtCb = (am_agCb_t *)userInfo;
    am_rc_t         rc = 0;
    dm_acceptRsp_t  dmRsp;
    uint64          appHdl = (uint64)userInfo;

    AM_DBG_PRINT(("target_authOK..."));

    bzero((void *)&dmRsp, sizeof(dm_acceptRsp_t));
    dmRsp.hdr.msgType = DM_ACCEPT_RSP;
    dmRsp.hdr.appHdl  = appHdl;                 // control block
    dmRsp.hdr.dmHdl   = agtCb->dmHdl;
    dmRsp.result      = NFX_OK;                 // datamover-api.h

    /* Send message to the data mover */
    rc = eee_sendMessage(&dmRsp, sizeof(dm_acceptRsp_t), datamoverAddr, amAddr);
    NFX_ASSERT(rc == NFX_OK);

    if (agtCb->msgQ != NULL)
    {
        /*
         * The last snap request the only thing that can be queued here.
         */
        char            *msg   = (char *)agtCb->msgQ->rcvMsg;
        am_msg_hdr_t    *amMsg = (am_msg_hdr_t *)(msg+sizeof(dm_rcvInd_t));
        fs_MsgAmLastSnapshotReq_t  fsReq;

        /*
         * TBD: Verify all the authentication information. Also check the
         * change in file system size that needs to be impletemented.
         */

        AM_FCT_PRINT(("Processing queued last snap request on target"));
        bzero (&fsReq, sizeof(fs_MsgAmLastSnapshotReq_t));
        fsReq.hdr.requestType   = FS_REQ_AM_LAST_SNAPSHOT;
        fsReq.hdr.len           = sizeof(fs_MsgAmLastSnapshotReq_t);
        fsReq.hdr.clientPrivate = appHdl;
        fsReq.cookie            = appHdl;
        memcpy(&(agtCb->amVolId), &(amMsg->volId), sizeof(evm_volId_t));
        memcpy(&(fsReq.volId), &(amMsg->volId), sizeof(evm_volId_t));

		AM_FCT_PRINT(("Sending FS_REQ_AM_LAST_SNAPSHOT to 0x%x\n", efsAddr));
        rc = eee_sendMessage(&fsReq, sizeof(fs_MsgAmLastSnapshotReq_t),
                             efsAddr, amAddr);
        NFX_ASSERT(rc == NFX_OK);
    }

    if (thisEvent.p1.u.p)
        am_eee_ramDealloc(thisEvent.p1.u.p);

    AM_DBG_PRINT(("...target_authOK"));
    return (nextState);
}


/*
 *------------------------------------------------------------------------------
 * Name         : target_snapQ
 *
 * Description  :
 *
 * Created by   : Nitin Jain
 *
 *------------------------------------------------------------------------------
 */
int32 target_snapQ (esm_handle_t  handle,
                    int32         nextState,
                    esm_event_t   thisEvent,
                    void          *userInfo)
{
    
    am_agCb_t       *agtCb = (am_agCb_t *)userInfo;
    char            *msg = (char *)thisEvent.p1.u.p;
    am_rcv_msg_t    *new = NULL;

    AM_DBG_PRINT(("target_snapQ..."));

    new = am_eee_ramAlloc(sizeof(am_rcv_msg_t));
    NFX_ASSERT(new != NULL);
    new->rcvMsg = msg;
    new->next   = NULL;

    /*
     * At this point we should only have this message in the queue
     * because the source is waiting for the last snap response
     */

    if (agtCb->msgQ == NULL)
        agtCb->msgQ = new;
    else
        agtCb->lastMsg->next = new;

    agtCb->lastMsg = new;

    AM_DBG_PRINT(("...target_snapQ"));
    return (nextState);
}


/*
 *------------------------------------------------------------------------------
 * Name         : target_getVolId
 *
 * Description  :
 *
 * Created by   : Pankaj Gupta
 *
 *------------------------------------------------------------------------------
 */
int32 target_getVolId (esm_handle_t  handle,
                       int32         nextState,
                       esm_event_t   thisEvent,
                       void          *userInfo)
{
    am_agCb_t       *agtCb = (am_agCb_t *)userInfo;
    char            *msg   = (char *)thisEvent.p1.u.p;
    am_getVolIdReq_t    *amMsg = (am_getVolIdReq_t *)(msg+sizeof(dm_rcvInd_t));
    am_rc_t         rc = 0;
	evm_MsgGetId_t	evmReq;
    uint64          appHdl = (uint64)userInfo;

    AM_DBG_PRINT(("target_getVolId..."));

    bzero (&evmReq, sizeof(evm_MsgGetId_t));
	evmReq.hdr.msgType = EVM_VOL_GET_ID;
	evmReq.hdr.clientPrivate = appHdl;
	strcpy(evmReq.name, amMsg->tgtFsysName);
    strcpy(agtCb->tgtFsysName, amMsg->tgtFsysName);

	AM_FCT_PRINT(("Req vol id for %s from SSC addr 0x%x",
				amMsg->tgtFsysName, evmSscAddr));

    rc = eee_sendMessage(&evmReq, sizeof(evm_MsgGetId_t),
                         evmSscAddr, amAddr);
    NFX_ASSERT(rc == NFX_OK);
    if (thisEvent.p1.u.p)
        am_eee_ramDealloc(thisEvent.p1.u.p);

    AM_DBG_PRINT(("...target_getVolId"));
    return (nextState);
}

/*
 *------------------------------------------------------------------------------
 * Name         : target_sendVolId
 *
 * Description  :
 *
 * Created by   : Pankaj Gupta
 *
 *------------------------------------------------------------------------------
 */
int32 target_sendVolId (esm_handle_t  handle,
                       int32         nextState,
                       esm_event_t   thisEvent,
                       void          *userInfo)
{
    am_agCb_t                   *agtCb = (am_agCb_t *)userInfo;
    evm_MsgStat_t				*evmRsp = NULL;
    am_rc_t                     rc;
    char                        *req = NULL;
    dm_sendReq_t                *dmReq = NULL;
    am_getVolIdRsp_t            *amMsg = NULL;
    uint32                      msgLen;
    uint64                      appHdl = (uint64)userInfo;

    AM_DBG_PRINT(("target_sendVolId..."));

    evmRsp = (evm_MsgStat_t *)thisEvent.p1.u.p;

    msgLen = sizeof(dm_sendReq_t) + sizeof(am_getVolIdRsp_t);
    req = (char *)am_eee_ramAlloc(msgLen);
    NFX_ASSERT(req != NULL);

    dmReq = (dm_sendReq_t *)req;
    dmReq->hdr.msgType = DM_SEND_REQ;
    dmReq->hdr.appHdl  = appHdl;
    dmReq->hdr.dmHdl   = agtCb->dmHdl;
    dmReq->dataSize    = 0;
    dmReq->cod         = 0;
    dmReq->offset      = 0;
    dmReq->appHdrSize  = sizeof(am_getVolIdRsp_t);

	/*
	 * Update info in control block
	 */
	memcpy(&agtCb->tgtVolId, &evmRsp->payload.lVolId, sizeof(evm_volId_t));

	/*
	 * prepare AM msg to be sent to source
	 */
    amMsg = (am_getVolIdRsp_t *)(req+sizeof(dm_sendReq_t));
    amMsg->hdr.msgType = AM_GET_VOLID_RSP;
	strcpy(amMsg->tgtFsysName, agtCb->tgtFsysName);
    memcpy(&amMsg->tgtVolId, &evmRsp->payload.lVolId, sizeof(evm_volId_t));

    AM_FCT_PRINT(("Target -> send vol id 0x%llx for vol %s to source",
			amMsg->tgtVolId.whole_thing, amMsg->tgtFsysName));
/*
*/

    if (agtCb->msgQ != NULL)
    {
        am_eee_ramDealloc(agtCb->msgQ->rcvMsg);
        am_eee_ramDealloc(agtCb->msgQ);
        agtCb->msgQ = NULL;
        agtCb->lastMsg = NULL;
    }

    rc = eee_sendMessage(req, msgLen, datamoverAddr, amAddr);
    am_eee_ramDealloc(req);
    NFX_ASSERT(rc == NFX_OK);
    if (thisEvent.p1.u.p)
        am_eee_ramDealloc(thisEvent.p1.u.p);

    AM_DBG_PRINT(("...target_sendVolId"));
    return (nextState);
}


/*
 *------------------------------------------------------------------------------
 * Name         : target_lastSnap
 *
 * Description  :
 *
 * Created by   : Nitin Jain
 *
 *------------------------------------------------------------------------------
 */
int32 target_lastSnap (esm_handle_t  handle,
                       int32         nextState,
                       esm_event_t   thisEvent,
                       void          *userInfo)
{
#ifdef INCLUDE_FS_ON_TARGET
    am_agCb_t       *agtCb = (am_agCb_t *)userInfo;
    char            *msg   = (char *)thisEvent.p1.u.p;
    am_msg_hdr_t    *amMsg = (am_msg_hdr_t *)(msg+sizeof(dm_rcvInd_t));
    am_rc_t         rc = 0;
    fs_MsgAmLastSnapshotReq_t  fsReq;
    uint64          appHdl = (uint64)userInfo;

    AM_DBG_PRINT(("target_lastSnap..."));

    /*
     * Verify all the authentication information. Also check the
     * change in file system size that needs to be impletemented.
     */

    bzero (&fsReq, sizeof(fs_MsgAmLastSnapshotReq_t));
    fsReq.hdr.requestType   = FS_REQ_AM_LAST_SNAPSHOT;
    fsReq.hdr.len           = sizeof(fs_MsgAmLastSnapshotReq_t);
    fsReq.hdr.clientPrivate = appHdl;
    fsReq.cookie            = appHdl;

	/*
	 * TBD: Find the target vol id based on Vol Name
	 */
    memcpy(&(agtCb->amVolId), &(amMsg->volId), sizeof(evm_volId_t));
    memcpy(&(fsReq.volId), &(amMsg->volId), sizeof(evm_volId_t));

	AM_FCT_PRINT(("Sending FS_REQ_AM_LAST_SNAPSHOT to 0x%x\n", efsAddr));
    rc = eee_sendMessage(&fsReq, sizeof(fs_MsgAmLastSnapshotReq_t),
                         efsAddr, amAddr);
    NFX_ASSERT(rc == NFX_OK);
    if (thisEvent.p1.u.p)
        am_eee_ramDealloc(thisEvent.p1.u.p);

    AM_DBG_PRINT(("...target_lastSnap"));
    return (nextState);
#else
    am_agCb_t       *agtCb = (am_agCb_t *)userInfo;
    am_rc_t         rc = 0;
    fs_MsgAmLastSnapshotResp_t  fsRsp;
    uint64          appHdl = (uint64)userInfo;
    char            *msg   = (char *)thisEvent.p1.u.p;
    dm_rcvInd_t     *dmRcv = (dm_rcvInd_t *)msg;
    am_msg_hdr_t    *amMsg = (am_msg_hdr_t *)(msg+sizeof(dm_rcvInd_t));

    AM_DBG_PRINT(("target_lastSnap..."));

    /*
     * Verify all the authentication information. Also check the
     * change in file system size that needs to be impletemented.
     */

    bzero (&fsRsp, sizeof(fs_MsgAmLastSnapshotResp_t));
    fsRsp.hdr.requestType   = FS_REQ_AM_LAST_SNAPSHOT;
    fsRsp.hdr.len           = sizeof(fs_MsgAmLastSnapshotResp_t);
    fsRsp.hdr.clientPrivate = appHdl;
    fsRsp.hdr.fsStatus      = FS_ERR_NONE;
    fsRsp.cookie            = appHdl;
    memcpy(&(agtCb->amVolId), &(amMsg->volId), sizeof(evm_volId_t));
    bzero(fsRsp.name, sizeof(am_snapName_t));

    rc = eee_sendMessage(&fsRsp,
                         sizeof(fs_MsgAmLastSnapshotResp_t),
                         amAddr,
                         amAddr);
    NFX_ASSERT(rc == NFX_OK);
    if (thisEvent.p1.u.p)
        am_eee_ramDealloc(thisEvent.p1.u.p);

    AM_DBG_PRINT(("...target_lastSnap"));
    return (nextState);
#endif
}


/*
 *------------------------------------------------------------------------------
 * Name         : target_sendSnap
 *
 * Description  :
 *
 * Created by   : Nitin Jain
 *
 *------------------------------------------------------------------------------
 */
int32 target_sendSnap (esm_handle_t  handle,
                       int32         nextState,
                       esm_event_t   thisEvent,
                       void          *userInfo)
{
    am_agCb_t                   *agtCb = (am_agCb_t *)userInfo;
    fs_MsgAmLastSnapshotResp_t  *efsRsp = NULL;
    am_rc_t                     rc;
    char                        *req = NULL;
    dm_sendReq_t                *dmReq = NULL;
    am_snap_msg_hdr_t           *amMsg = NULL;
    uint32                      msgLen;
    uint64                      appHdl = (uint64)userInfo;

    AM_DBG_PRINT(("target_sendSnap..."));

    efsRsp = (fs_MsgAmLastSnapshotResp_t *)thisEvent.p1.u.p;

    msgLen = sizeof(dm_sendReq_t) + sizeof(am_snap_msg_hdr_t);
    req = (char *)am_eee_ramAlloc(msgLen);
    NFX_ASSERT(req != NULL);

    dmReq = (dm_sendReq_t *)req;
    dmReq->hdr.msgType = DM_SEND_REQ;
    dmReq->hdr.appHdl  = appHdl;
    dmReq->hdr.dmHdl   = agtCb->dmHdl;
    dmReq->dataSize    = 0;
    dmReq->cod         = 0;
    dmReq->offset      = 0;
    dmReq->appHdrSize  = sizeof(am_snap_msg_hdr_t);

    amMsg = (am_snap_msg_hdr_t *)(req+sizeof(dm_sendReq_t));
    amMsg->am_msgType = AM_LAST_SNAP_RSP;
    memcpy(agtCb->lastSnap, efsRsp->name, sizeof(am_snapName_t));
    memcpy(amMsg->snapName, efsRsp->name, sizeof(am_snapName_t));
    memcpy(&(amMsg->volId), &(agtCb->amVolId), sizeof(evm_volId_t));

    AM_FCT_PRINT(("Target -> send last snap to source : %s", 
			(strlen(efsRsp->name) > 0) ? efsRsp->name : "None"));

    printf("Target -> send last snap to source : %s\n", 
			(strlen(efsRsp->name) > 0) ? efsRsp->name : "None");

    if (agtCb->msgQ != NULL)
    {
        am_eee_ramDealloc(agtCb->msgQ->rcvMsg);
        am_eee_ramDealloc(agtCb->msgQ);
        agtCb->msgQ = NULL;
        agtCb->lastMsg = NULL;
    }

    rc = eee_sendMessage(req, msgLen, datamoverAddr, amAddr);
    am_eee_ramDealloc(req);
    NFX_ASSERT(rc == NFX_OK);
    if (thisEvent.p1.u.p)
        am_eee_ramDealloc(thisEvent.p1.u.p);

    AM_DBG_PRINT(("...target_sendSnap"));
    return (nextState);
}


/*
 *------------------------------------------------------------------------------
 * Name         : target_instlSnap
 *
 * Description  :
 *
 * Created by   : Nitin Jain
 *
 *------------------------------------------------------------------------------
 */
int32 target_instlSnap (esm_handle_t  handle,
                        int32         nextState,
                        esm_event_t   thisEvent,
                        void          *userInfo)
{
    am_agCb_t                      *agtCb = (am_agCb_t *)userInfo;
    char                           *rcvMsg = (char *)thisEvent.p1.u.p;
    am_snap_msg_hdr_t              *amMsg = NULL;
    am_rc_t                        rc;
    char                           *msg = NULL;
    int32                          msgLen;
    uint32                         destAddr;
    uint64                         appHdl = (uint64)userInfo;

    AM_DBG_PRINT(("target_instlSnap..."));

    amMsg = (am_snap_msg_hdr_t *)(rcvMsg+sizeof(dm_rcvInd_t));
    if (amMsg->snapName != NULL)
    {
#ifdef INCLUDE_FS_ON_TARGET
        /*
         * Send request to EFS
         */

        fs_MsgAmDeltaInstallSetupReq_t *fsMsg = NULL;

        msgLen = sizeof(fs_MsgAmDeltaInstallSetupReq_t);
        msg = am_eee_ramAlloc(msgLen);
        NFX_ASSERT(msg != NULL);

        bzero(msg, msgLen);
        fsMsg = (fs_MsgAmDeltaInstallSetupReq_t *)msg;
        fsMsg->hdr.requestType   = FS_REQ_AM_DELTA_INSTALL_SETUP;
        fsMsg->hdr.len           = sizeof(fs_MsgAmDeltaInstallSetupReq_t);
        fsMsg->hdr.clientPrivate = appHdl;
        fsMsg->cookie            = appHdl;
        fsMsg->snapNum           = amMsg->snapNum;
        memcpy(agtCb->newSnap, amMsg->snapName, sizeof(am_snapName_t));
        memcpy(fsMsg->newName, amMsg->snapName, sizeof(am_snapName_t));
        memcpy(&(fsMsg->volId), &(amMsg->volId), sizeof(evm_volId_t));
        destAddr = efsAddr;
		AM_FCT_PRINT(("Sending FS_REQ_AM_DELTA_INSTALL_SETUP to 0x%x\n", destAddr));
        AM_FCT_PRINT(("Target -> install new snap : %s", amMsg->snapName));
        printf("Target -> install new snap : %s\n", amMsg->snapName);
#else
        /*
         * Send request to EFS
         */

        fs_MsgAmDeltaInstallSetupResp_t *fsMsg = NULL;

        msgLen = sizeof(fs_MsgAmDeltaInstallSetupResp_t);
        msg = am_eee_ramAlloc(msgLen);
        NFX_ASSERT(msg != NULL);

        bzero(msg, msgLen);
        fsMsg = (fs_MsgAmDeltaInstallSetupResp_t *)msg;
        fsMsg->hdr.requestType   = FS_REQ_AM_DELTA_INSTALL_SETUP;
        fsMsg->hdr.len           = sizeof(fs_MsgAmDeltaInstallSetupResp_t);
        fsMsg->hdr.clientPrivate = appHdl;
        fsMsg->hdr.fsStatus      = FS_ERR_NONE;
        fsMsg->cookie            = appHdl;
        destAddr = amAddr;
		AM_FCT_PRINT(("Sending FS_REQ_AM_DELTA_INSTALL_SETUP to 0x%x\n", destAddr));
#endif
    }
    else
    {
        /*
         * Send snap not OK
         */

        dm_sendReq_t    *dmReq;
        am_msg_hdr_t    *amRsp;

        msgLen = sizeof(dm_sendReq_t) + sizeof(am_msg_hdr_t);
        msg = am_eee_ramAlloc(msgLen);
        NFX_ASSERT(msg != NULL);

        dmReq = (dm_sendReq_t *)msg;
        dmReq->hdr.msgType = DM_SEND_REQ;
        dmReq->hdr.appHdl  = appHdl;
        dmReq->hdr.dmHdl   = agtCb->dmHdl;
        dmReq->dataSize    = 0;
        dmReq->cod         = 0;
        dmReq->offset      = 0;
        dmReq->appHdrSize  = sizeof(am_msg_hdr_t);

        amRsp = (am_msg_hdr_t *)(msg+sizeof(dm_sendReq_t));
        amRsp->am_msgType = AM_SET_SNAP_NOK;
        memcpy(&(amRsp->volId), &(agtCb->amVolId), sizeof(evm_volId_t));
        destAddr = datamoverAddr;
		AM_FCT_PRINT(("Sending AM_SET_SNAP_NOK to 0x%x\n", destAddr));
        AM_FCT_PRINT(("Target -> install new snap : NULL - error"));
    }
    rc = eee_sendMessage(msg, msgLen, destAddr, amAddr);
    NFX_ASSERT(rc == NFX_OK);
    am_eee_ramDealloc(msg);
    if (thisEvent.p1.u.p)
        am_eee_ramDealloc(thisEvent.p1.u.p);

    AM_DBG_PRINT(("...target_instlSnap"));
    return (nextState);
}


/*
 *------------------------------------------------------------------------------
 * Name         : target_instlRsp
 *
 * Description  :
 *
 * Created by   : Nitin Jain
 *
 *------------------------------------------------------------------------------
 */
int32 target_instlRsp (esm_handle_t  handle,
                       int32         nextState,
                       esm_event_t   thisEvent,
                       void          *userInfo)
{
    am_agCb_t       *agtCb = (am_agCb_t *)userInfo;
    char            *rsp = NULL;
    int32           msgLen;
    dm_sendReq_t    *dmReq = NULL;
    am_msg_hdr_t    *amRsp = NULL;
    am_rc_t         rc;
    uint64          appHdl = (uint64)userInfo;
    fs_MsgAmDeltaInstallSetupResp_t   *fsRsp = NULL;

    AM_DBG_PRINT(("target_instlRsp..."));

    fsRsp = (fs_MsgAmDeltaInstallSetupResp_t *)thisEvent.p1.u.p;

    msgLen = sizeof(dm_sendReq_t) + sizeof(am_msg_hdr_t);
    rsp = am_eee_ramAlloc(msgLen);
    NFX_ASSERT(rsp != NULL);

    dmReq = (dm_sendReq_t *)rsp;
    dmReq->hdr.msgType = DM_SEND_REQ;
    dmReq->hdr.appHdl  = appHdl;
    dmReq->hdr.dmHdl   = agtCb->dmHdl;
    dmReq->dataSize    = 0;
    dmReq->cod         = 0;
    dmReq->offset      = 0;
    dmReq->appHdrSize  = sizeof(am_msg_hdr_t);

    amRsp = (am_msg_hdr_t *)(rsp+sizeof(dm_sendReq_t));
    if (fsRsp->hdr.fsStatus == FS_ERR_NONE)
    {
        amRsp->am_msgType = AM_SET_SNAP_OK;
        AM_FCT_PRINT(("Target -> FS install snap rsp : OK"));
    }
    else
    {
        amRsp->am_msgType = AM_SET_SNAP_NOK;
        AM_ERR_PRINT(("Target -> FS install snap rsp : NOK"));
    }
    memcpy(&(amRsp->volId), &(agtCb->amVolId), sizeof(evm_volId_t));

    rc = eee_sendMessage(rsp, msgLen, datamoverAddr, amAddr);
    NFX_ASSERT(rc == NFX_OK);
    am_eee_ramDealloc(rsp);

    if (thisEvent.p1.u.p)
        am_eee_ramDealloc(thisEvent.p1.u.p);

    AM_DBG_PRINT(("...target_instlRsp"));
    return (nextState);
}

void target_processQEntry(esm_handle_t  handle, am_agCb_t *agtCb)
{
    am_rc_t         rc;
    am_intfMsg_t    *am_msg_type = NULL;
    char            *currMsg = (char *)agtCb->msgQ->rcvMsg;

    am_msg_type = (am_intfMsg_t *)(currMsg+sizeof(dm_rcvInd_t));

    switch (*am_msg_type)
    {
        case AM_LAST_SNAP_REQ:
        case AM_LAST_SNAP_RSP:
        case AM_SET_SNAP_REQ:
        case AM_SET_SNAP_OK:
        case AM_SET_SNAP_NOK:
        case AM_RCVD_DONE_OK:
        case AM_RCVD_DONE_NOK:
        {
            /*
             * This is error. What to do?
             * Idealy this should neven happen. Source waits for each
             * response before sending the next control message. Only
             * SEND message can be queued.
             */
            M_POST_ABORT_EVENT(agtCb, "Invalid request queued", NFX_ERR);
            break;
        }
        case AM_SEND_DATA_IND:
        {
            AM_DBG_PRINT(("Target -> msg Q : process next write req"));
            am_writeCOD_to_disk(agtCb);
            break;
        }
        case AM_SEND_DONE_IND:
        {
            esm_event_t     newEvent;
    		bzero(&newEvent, sizeof(esm_event_t));
            newEvent.event   = TARGET_EVENT_EOM_IND;
            newEvent.p1.u.dw = 0;
            newEvent.p2.u.dw = 0;
            AM_DBG_PRINT(("Target -> msg Q : process am-done message"));

			AM_DBG_PRINT(("Posting TARGET_EVENT_EOM_IND %d to %d SM", \
                       newEvent.event, handle));
            rc = esm_deliverEvent(handle, &newEvent);
            NFX_ASSERT(rc == NFX_OK);
            break;
        }
    }
}

/*
 *------------------------------------------------------------------------------
 * Name         : target_queueReq
 *
 * Description  :
 *
 * Created by   : Nitin Jain
 *
 *------------------------------------------------------------------------------
 */
int32 target_queueReq (esm_handle_t  handle,
                       int32         nextState,
                       esm_event_t   thisEvent,
                       void          *userInfo)
{
    am_agCb_t       *agtCb = (am_agCb_t *)userInfo;
    char            *msg = (char *)thisEvent.p1.u.p;
    am_rcv_msg_t    *new = NULL;
	uint32			emptyQueue = 0;

    AM_DBG_PRINT(("target_queueReq..."));

	

    new = am_eee_ramAlloc(sizeof(am_rcv_msg_t));
    NFX_ASSERT(new != NULL);
    new->rcvMsg = msg;
    new->next   = NULL;

	if (agtCb->lastMsg)
	{
    	agtCb->lastMsg->next = new;
    	agtCb->lastMsg = new;
	}
	else
	{
		emptyQueue = 1;
		agtCb->msgQ = new;
		agtCb->lastMsg = new;
	}

	/*
	 * We just queued a message. If the queue was
	 * empty we can process it right here
	 */
	if (emptyQueue)
        target_processQEntry(handle, agtCb);

    AM_DBG_PRINT(("...target_queueReq"));
    return (nextState);
}


/*
 *------------------------------------------------------------------------------
 * Name         : target_procQReq
 *
 * Description  :
 *
 * Created by   : Nitin Jain
 *
 *------------------------------------------------------------------------------
 */
int32 target_procQueue (esm_handle_t  handle,
                       int32         nextState,
                       esm_event_t   thisEvent,
                       void          *userInfo)
{
    am_agCb_t       *agtCb = (am_agCb_t *)userInfo;
    am_rcv_msg_t    *curr = NULL;

    AM_DBG_PRINT(("target_procQueue..."));

    /*
     * Skip the first request. There will always be atleast one request
     * to get to this procedure.
     */
    curr = agtCb->msgQ;
    agtCb->msgQ = curr->next;
    if (agtCb->msgQ == NULL)
        agtCb->lastMsg = NULL;
    am_eee_ramDealloc(curr->rcvMsg);
    am_eee_ramDealloc(curr);


	if (agtCb->msgQ != NULL)
		target_processQEntry(handle, agtCb);


    if (thisEvent.p1.u.p)
        am_eee_ramDealloc(thisEvent.p1.u.p);

    AM_DBG_PRINT(("...target_procQueue"));
    return (nextState);
}


/*
 *------------------------------------------------------------------------------
 * Name         : target_snapCommit
 *
 * Description  :
 *
 * Created by   : Nitin Jain
 *
 *------------------------------------------------------------------------------
 */
int32 target_snapCommit (esm_handle_t  handle,
                         int32         nextState,
                         esm_event_t   thisEvent,
                         void          *userInfo)
{
#ifdef INCLUDE_FS_ON_TARGET
    am_agCb_t                        *agtCb = (am_agCb_t *)userInfo;
    am_rc_t                          rc;
    uint64                           appHdl = (uint64)userInfo;
    fs_MsgAmDeltaInstallCommitReq_t  fsReq;

    AM_DBG_PRINT(("target_snapCommit..."));

    bzero(&fsReq, sizeof(fs_MsgAmDeltaInstallCommitReq_t));
    fsReq.hdr.requestType   = FS_REQ_AM_DELTA_INSTALL_COMMIT;
    fsReq.hdr.len           = sizeof(fs_MsgAmDeltaInstallCommitReq_t);
    fsReq.hdr.clientPrivate = appHdl;
    memcpy(&(fsReq.volId), &(agtCb->amVolId), sizeof(evm_volId_t));
    fsReq.cookie            = appHdl;

	AM_FCT_PRINT(("Sending FS_REQ_AM_DELTA_INSTALL_COMMIT to 0x%x\n", efsAddr));
    rc = eee_sendMessage(&fsReq, sizeof(fs_MsgAmDeltaInstallCommitReq_t),
                         efsAddr, amAddr);

    NFX_ASSERT(rc == NFX_OK);
    if (thisEvent.p1.u.p)
        am_eee_ramDealloc(thisEvent.p1.u.p);

    AM_DBG_PRINT(("...target_snapCommit"));
    return (nextState);
#else
    am_agCb_t                        *agtCb = (am_agCb_t *)userInfo;
    am_rc_t                          rc;
    uint64                           appHdl = (uint64)userInfo;
    fs_MsgAmDeltaInstallCommitResp_t fsRsp;

    AM_DBG_PRINT(("target_snapCommit..."));

    bzero(&fsRsp, sizeof(fs_MsgAmDeltaInstallCommitResp_t));
    fsRsp.hdr.requestType   = FS_REQ_AM_DELTA_INSTALL_COMMIT;
    fsRsp.hdr.len           = sizeof(fs_MsgAmDeltaInstallCommitResp_t);
    fsRsp.hdr.clientPrivate = appHdl;
    fsRsp.hdr.fsStatus      = FS_ERR_NONE;
    fsRsp.cookie            = appHdl;

    rc = eee_sendMessage(&fsRsp,
                         sizeof(fs_MsgAmDeltaInstallCommitResp_t),
                         amAddr,
                         amAddr);

    NFX_ASSERT(rc == NFX_OK);
    if (thisEvent.p1.u.p)
        am_eee_ramDealloc(thisEvent.p1.u.p);

    AM_DBG_PRINT(("...target_snapCommit"));
    return (nextState);
#endif
}


/*
 *------------------------------------------------------------------------------
 * Name         : target_sendDone
 *
 * Description  :
 *
 * Created by   : Nitin Jain
 *
 *------------------------------------------------------------------------------
 */
int32 target_sendDone (esm_handle_t  handle,
                       int32         nextState,
                       esm_event_t   thisEvent,
                       void          *userInfo)
{
    am_agCb_t           *agtCb = (am_agCb_t *)userInfo;
    dm_sendReq_t        *dmReq = NULL;
    char                *req = NULL;
    int32               rc = 0, msgLen;
    am_msg_hdr_t        *amMsg = NULL;
    int32               amMsgLen = sizeof(am_msg_hdr_t);
    int32               dmReqLen = sizeof(dm_sendReq_t);
    uint64              appHdl = (uint64)userInfo;

    AM_DBG_PRINT(("target_sendDone..."));

    msgLen = dmReqLen + amMsgLen;
    req = (char *)am_eee_ramAlloc(msgLen);
    NFX_ASSERT(req != NULL);

    dmReq = (dm_sendReq_t *)req;
    dmReq->hdr.msgType = DM_SEND_REQ;
    dmReq->hdr.appHdl  = appHdl;
    dmReq->hdr.dmHdl   = agtCb->dmHdl;

    dmReq->dataSize    = 0;
    dmReq->cod         = 0;
    dmReq->offset      = 0;
    dmReq->appHdrSize  = amMsgLen;

    amMsg = (am_msg_hdr_t *)(req+dmReqLen);
    amMsg->am_msgType  = AM_RCVD_DONE_OK;
    memcpy (&(amMsg->volId), &(agtCb->amVolId), sizeof(evm_volId_t));

    rc = eee_sendMessage(req, msgLen, datamoverAddr, amAddr);
    NFX_ASSERT(rc == NFX_OK);
    am_eee_ramDealloc(req);
    if (thisEvent.p1.u.p)
        am_eee_ramDealloc(thisEvent.p1.u.p);

    AM_DBG_PRINT(("...target_sendDone"));
    return (nextState);
}


/*
 *------------------------------------------------------------------------------
 * Name         : target_sendFail
 *
 * Description  :
 *
 * Created by   : Nitin Jain
 *
 *------------------------------------------------------------------------------
 */
int32 target_sendFail (esm_handle_t  handle,
                       int32         nextState,
                       esm_event_t   thisEvent,
                       void          *userInfo)
{
    am_agCb_t           *agtCb = (am_agCb_t *)userInfo;
    char                *req = NULL;
    dm_sendReq_t        *dmReq = NULL;
    am_msg_hdr_t        *amMsg = NULL;
    int32               dmReqLen = sizeof(dm_sendReq_t);
    int32               amMsgLen = sizeof(am_msg_hdr_t);
    int32               rc = 0, msgLen;
    uint64              appHdl = (uint64)userInfo;

    AM_DBG_PRINT(("target_sendFail..."));

    msgLen = dmReqLen + amMsgLen;
    req = (char *)am_eee_ramAlloc(msgLen);
    NFX_ASSERT(req != NULL);

    dmReq = (dm_sendReq_t *)req;
    dmReq->hdr.msgType = DM_SEND_REQ;
    dmReq->hdr.appHdl  = appHdl;
    dmReq->hdr.dmHdl   = agtCb->dmHdl;

    dmReq->dataSize    = 0;
    dmReq->cod         = 0;
    dmReq->offset      = 0;
    dmReq->appHdrSize  = amMsgLen;

    amMsg = (am_msg_hdr_t *)(req+dmReqLen);
    amMsg->am_msgType  = AM_RCVD_DONE_NOK;
    memcpy (&(amMsg->volId), &(agtCb->amVolId), sizeof(evm_volId_t));

    rc = eee_sendMessage(req, msgLen, datamoverAddr, amAddr);
    NFX_ASSERT(rc == NFX_OK);
    am_eee_ramDealloc(req);
    if (thisEvent.p1.u.p)
        am_eee_ramDealloc(thisEvent.p1.u.p);

    AM_DBG_PRINT(("...target_sendFail"));
    return (nextState);
}


#if 0
/*
 *------------------------------------------------------------------------------
 * Name         : target_nop
 *
 * Description  :
 *
 * Created by   : Nitin Jain
 *
 *------------------------------------------------------------------------------
 */
int32 target_nop (esm_handle_t  handle,
                  int32         nextState,
                  esm_event_t   thisEvent,
                  void          *userInfo)
{
    AM_DBG_PRINT(("target_nop..."));

    if (thisEvent.p1.u.p)
        am_eee_ramDealloc(thisEvent.p1.u.p);

    AM_DBG_PRINT(("...target_nop"));
    return (nextState);
}
#endif


/*
 *------------------------------------------------------------------------------
 * Name         : target_dmClose
 *
 * Description  :
 *
 * Created by   : Nitin Jain
 *
 *------------------------------------------------------------------------------
 */

int32 target_dmClose (esm_handle_t  handle,
                      int32         nextState,
                      esm_event_t   thisEvent,
                      void          *userInfo)
{
    am_agCb_t       *agtCb = (am_agCb_t *)userInfo;
    dm_closeReq_t   dmReq;
    am_rc_t         rc = 0;
    uint64          appHdl = (uint64)userInfo;

    AM_FCT_PRINT(("target_dmClose..."));

    bzero(&dmReq, sizeof(dm_closeReq_t));
    dmReq.hdr.msgType = DM_CLOSE_REQ;
    dmReq.hdr.appHdl  = appHdl;
    dmReq.hdr.dmHdl   = agtCb->dmHdl;
    
    rc = eee_sendMessage(&dmReq, sizeof(dm_closeReq_t), datamoverAddr, amAddr);
    NFX_ASSERT(rc == NFX_OK);
    if (thisEvent.p1.u.p)
        am_eee_ramDealloc(thisEvent.p1.u.p);

    AM_DBG_PRINT(("...target_dmClose"));
    return (nextState);
}


/*
 *------------------------------------------------------------------------------
 * Name         : target_kill
 *
 * Description  :
 *
 * Created by   : Nitin Jain
 *
 *------------------------------------------------------------------------------
 */

int32 target_kill (esm_handle_t  handle,
                   int32         nextState,
                   esm_event_t   thisEvent,
                   void          *userInfo)
{
    am_agCb_t       *agtCb = (am_agCb_t *)userInfo;
    am_rc_t   rc;

	AM_FCT_PRINT(("target_kill: Terminating target SM %d\n", handle));

    am_free_control_block(agtCb);

    if (thisEvent.p1.u.p)
        am_eee_ramDealloc(thisEvent.p1.u.p);

	printf("target_kill: Terminating target SM %d\n", handle);

    rc = target_sm_destroy(handle);
    if (rc != NFX_OK)
        AM_ERR_PRINT(("Target : Could not destroy sm handle %d", handle));

    return (nextState);
}


/*
 *------------------------------------------------------------------------------
 * Name         : target_err
 *
 * Description  :
 *
 * Created by   : Nitin Jain
 *
 *------------------------------------------------------------------------------
 */

int32 target_err (esm_handle_t  handle,
                  int32         nextState,
                  esm_event_t   thisEvent,
                  void          *userInfo)
{
    AM_ERR_PRINT(("target_err..."));

    if (thisEvent.p1.u.p)
        am_eee_ramDealloc(thisEvent.p1.u.p);

    AM_ERR_PRINT(("...target_err"));
    return (nextState);
}


/*----------------------------------------------------------------------------*/
#endif  /* SSC */
