/*
 *-----------------------------------------------------------------
 *
 * Name: anp_sscEmbed.c
 *
 * RCS:  $Id: anp-sscEmbed.c,v 1.2 2001/11/13 19:28:22 rick Exp $
 *
 * Copyright (C) 2001, Agile Storage, Inc.
 *
 * Description:	Embedded SSC anp operations
 *
 * Created by:	Rick Lund
 *
 * Date Created:	7/9/01
 *
 *-----------------------------------------------------------------
 */

#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <nfx-incl.h>
#include <exports/eee-api.h>
#include <exports/cm-api.h>
#include <netdb.h>
#include <exports/anpssc-api.h>

int32	ssc_anp_fd;
uchar8	rcv_buf[9000];

/*-----------------------------------------------------------------
 * Name:	anp_poll()
 *
 * Description:	EEE layer polling function to test for received messages.
 *
 * Created by:	Rick Lund
 *
 * Date Created:	7/9/01
 *
 *-----------------------------------------------------------------
 */
void
anp_poll(
	void	* cb,
	uint32	tref
)
{
	uint32				i;
	int32				n;
	uint32				toSlot, toCPU, toApp, toPlane;
	struct timeval		to;
	ans_ssc_msghdr_t	*hdr;
	uchar8				*buf;
	uchar8				app[80];
	uint32				fromCPU;
	uint32				fromSlot;
	uint32				dp, sp;
	uint32				port;

	for (i=0; i<256; i++) {
		to.tv_sec = 0;
		to.tv_usec = 0;
		if ((n=recvAgileMsg(ssc_anp_fd,
							rcv_buf,
							sizeof(rcv_buf),
							&to,
							app,
							&fromCPU,
							&fromSlot,0)) < 0) {
			perror("error from recvAgileMsg");
			//exit(1);
			continue;
		}

		if (!n) {
/*
			printf("recvAgileMsg timeout\n");
*/
		}
		else {
			hdr = (ans_ssc_msghdr_t *)rcv_buf;
			toCPU   = hdr->cpu;
			toSlot  = hdr->slot;
			toApp   = hdr->appid;
			toPlane = hdr->plane;

			port = eee_getApplicationId(app);
			if (port == EEE_ERR_INVALID_APP_NAME) {
				port = atoi(&app[5]);
			}

			printf("app = %s, port = %d\n", app, port);
			printf("[anp_poll] got msg at 0x%x destined for Slot %d, CPU %d, app %d, plane %d\n", rcv_buf, toSlot, toCPU, toApp, toPlane);

			buf = (uchar8 *)(++hdr);
			dp = PACK_UNICAST_PORT_ADDRESS(
									toSlot,
									toCPU,
									toApp,
									toPlane,
									NFX_COS_HIGH,
									0);
			sp = PACK_UNICAST_PORT_ADDRESS(
									nfxMySlotId,
									nfxMyCPUId,
									port,
									toPlane,
									NFX_COS_HIGH,
									0);
			eee_sendMessage(buf, n-sizeof(ans_ssc_msghdr_t), dp, sp);
		}
	}

}	/* anp_poll() */

/*-----------------------------------------------------------------
 * Name:	anp_sscInit()
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	7/9/01
 *
 *-----------------------------------------------------------------
 */
void
anp_sscInit()
{
	/*
	 * Register with anp library to receive packets from socket
	 */
	ssc_anp_fd = regAgileApp("ssc");
	if (ssc_anp_fd < 0)
		panic("Error registering SSC application name\n");

	eee_registerPollingFunc(EEE_HIGH_PRIORITY, anp_poll, NULL);

}	/* anp_sscInit() */
