/* vi:ts=4
 *-----------------------------------------------------------------
 *
 * Name: faux-eee.c
 *
 * RCS:  $Id: faux-eee.c,v 1.7 2002/01/29 22:16:22 bobonfrogger Exp $
 *
 * Copyright (C) 2001, Agile Storage, Inc.
 *
 * Description: Contains the functions to translate between app id
 *              and name using the UNIX services accessors. It's a
 *              "fake" eee.
 *
 * Created by:	Greg Seibert
 *
 * Date Created: 14 Nov 2001
 *
 *-----------------------------------------------------------------
 */
#include <netdb.h>

#include <nfx-incl.h>
#include "anpssc-api.h"
#include <exports/eee-error.h>

int32
eee_getApplicationId (uchar8 *name)
{
	struct servent *serv;

	setservent(0);
	serv = getservbyname (name, "udp");
//	printf("eee_getid: appl name %s, %x\n", name, serv);
	endservent();
	if (!serv) {
		return EEE_ERR_INVALID_APP_NAME;
	}
	return ntohs(serv->s_port) - AGILE_BASE;
}

uchar8 *
eee_getAppNameFromId (int32 id)
{
	struct servent *serv;

	setservent(0);
	serv = getservbyport (htons(id + AGILE_BASE), "udp");
	endservent();
	if (!serv) {
		return NULL;
	}
	return serv->s_name;
}

