#include <sys/types.h>
#include <stdio.h>
#include "anpssc-api.h"
#include <unistd.h>
#include <stdlib.h>
#include <sys/socket.h>

struct bufhdr
{
	uint32 slotId;
	uint32 cpuId;
	uint32 size;
	uint32 plane;
};

static void
usage(void)
{
	printf("usage: t pktlen slot cpu");
}


static int
xstrtol(char *str)
{
	char *endptr;
	int val = strtol(str, &endptr, 10);
	if (!(endptr != str && *endptr == 0)) {
		printf("%s is not a number\n", str);
		usage();
		exit(1);
	}
	return val;
}

extern int bindEEESocket(int s, int slot, int cpu, int app);

int
main(int argc, char **argv)
{
	int n;
	unsigned char buf[300000];
	unsigned char resp[300000];
	struct bufhdr *hdr;
	struct timeval to;
	int pktlen, init_pktlen;
	int slot, cpu;
	int i;
	int repeat;
	int cnt = 0;
	int s;
	int increment;
	int maxdiff;

	if (argc >= 4) {
		init_pktlen = pktlen = xstrtol(argv[1]);
		slot = xstrtol(argv[2]);
		cpu = xstrtol(argv[3]);
	}
	else {
		usage();
		exit(1);
	}

	if (argc == 5)
		repeat = 1;
	else
		repeat = 0;

	for (i = 0; i < pktlen + 20000; i++) {
		buf[i] = i & 0xff;
	}

	hdr = (struct bufhdr *)&buf[0];
	hdr->slotId = 0;
	hdr->cpuId = 0;
	hdr->size = pktlen;
	if (slot == 0 && cpu == 1)
		hdr->plane = 1;
	else
		hdr->plane = 2;

	to.tv_sec = 5;
	to.tv_usec = 0;

	//printf("send message length=%d\n", pktlen);

	s = socket(PF_EEE, SOCK_DGRAM, 0);
	if (s != 0) {
		if (!bindEEESocket(s, 0, 0, 0)) {
			perror("bindEEESocket\n");
			exit(1);
		}
		if (!setSocketOpt(s)) {
			perror("setSocketOpt");
			exit(1);
		}
	}

	increment = 1;
	maxdiff = pktlen / 2 < 10000 ? pktlen / 2 : 10000;
	do {
		hdr->size = pktlen;
		n = sendAgileMsg(s, buf, pktlen, "eee_test_simple", cpu, slot, 0,
						 &to, resp, sizeof(resp));
		if (n < 0) {
			perror("sendAM error");
		}
		else if (!n) {
			printf ("Response timed out\n");
			break;
		}
		else {
			//printf("receive responce length=%d\n", n);
			if (n != pktlen) {
				printf("size does not match: send %d receive %d\n",
					   pktlen, n);
				break;
			}
			else if (memcmp(buf, resp, pktlen)) {
				printf("Responce does not match pktlen=%d\n", pktlen);
				break;
			}
		}

		if (++cnt == 80) {
			cnt = 0;
			putchar('.');
			fflush(stdout);
		}
		pktlen+=increment;
		if (pktlen == init_pktlen + maxdiff)
			increment = -1;
		if (pktlen == init_pktlen - maxdiff)
			increment = 1;
	}
	while (repeat);

	return 0;
}
