/*
 *=============================================================================
 * File Name   :    bcm5600-cmic.h
 *
 *  RCS: $Id: bcm5600-cmic.h,v 1.6 2002/10/17 00:03:27 raster Exp $
 *
 *  Description:    Copied and modified from Broadcom
 *
 *  Created    :    09-04-2001
 *
 *  Author     :    Tony Wong
 *
 *  Copyright(c)  Agile Storage, Inc. 2001
 *=============================================================================
 */

#ifndef _BCM5600_CMIC_H
#define _BCM5600_CMIC_H

#include "bcm5600-schanmsg.h"

#if defined(PROM) || defined(FCNIM) || defined(FCNIM_TM)
#define BCM5600_PCI_BASE_ADDR	0x38000000
#define BCM5600_BUS_ID			8
#define BCM5600_DEV_ID			8
#else /* SSC */
#define BCM5600_PCI_BASE_ADDR	0x26000000
#define BCM5600_BUS_ID			0
#define BCM5600_DEV_ID			6
#endif

#define COPYNO_ALL              -1
#define COPYNO_NONE             -2

/*
 * S-Channel Message Buffer Registers (0x00 -> 0x4c).
 * Block where S-Channel messages are written to CMIC.
 */
#define CMIC_SCHAN_MESSAGE(word)	(0x00000000 + 4 * (word))

/* S-Channel Control Register */
#define CMIC_SCHAN_CTRL			0x00000050

/*
 * SCHAN_CONTROL: control bits
 *
 *  SC_xxx_SET and SC_xxx_CLR can be WRITTEN to CMIC_SCHAN_CTRL.
 *  SC_xxx_TST can be masked against values READ from CMIC_SCHAN_CTRL.
 */
#define SC_MSG_START_SET		(0x80|0)
#define SC_MSG_START_CLR		(0x00|0)
#define SC_MSG_START_TST		0x00000001

#define SC_MSG_DONE_SET			(0x80|1)
#define SC_MSG_DONE_CLR			(0x00|1)
#define SC_MSG_DONE_TST			0x00000002

#define SC_ARL_RDY_SET(msg)		(0x80|((msg)+2))   /* 0 <= msg <= 3 */
#define SC_ARL_RDY_CLR(msg)		(0x00|((msg)+2))
#define SC_ARL_RDY_TST(msg)		(0x00000004 << (msg))
#define SC_ARL_RDY_TST_ANY		(0x0000003c)

#define SC_ARL_DMA_EN_SET		(0x80|6)
#define SC_ARL_DMA_EN_CLR		(0x00|6)
#define SC_ARL_DMA_EN_TST		0x00000040

#define SC_ARL_DMA_DONE_SET		(0x80|7)
#define SC_ARL_DMA_DONE_CLR		(0x00|7)
#define SC_ARL_DMA_DONE_TST		0x00000080

#define SC_LINK_STAT_MSG_SET		(0x80|8)
#define SC_LINK_STAT_MSG_CLR		(0x00|8)
#define SC_LINK_STAT_MSG_TST		0x00000100

#define SC_PCI_FATAL_ERR_SET		(0x80|9)
#define SC_PCI_FATAL_ERR_CLR		(0x00|9)
#define SC_PCI_FATAL_ERR_TST		0x00000200

#define SC_PCI_PARITY_ERR_SET		(0x80|10)
#define SC_PCI_PARITY_ERR_CLR		(0x00|10)
#define SC_PCI_PARITY_ERR_TST		0x00000400

#define SC_ARL_MSG_RCV_OFF_SET		(0x80|11)
#define SC_ARL_MSG_RCV_OFF_CLR		(0x00|11)
#define SC_ARL_MSG_RCV_OFF_TST		0x00000800

#define SC_ARL_MSG_DROPPED_SET		(0x80|12)
#define SC_ARL_MSG_DROPPED_CLR		(0x00|12)
#define SC_ARL_MSG_DROPPED_TST		0x00001000

#define SC_ARL_DMA_XFER_DONE_SET	(0x80|13)
#define SC_ARL_DMA_XFER_DONE_CLR	(0x00|13)
#define SC_ARL_DMA_XFER_DONE_TST	0x00002000

#define SC_MIIM_SCAN_BUSY_TST		0x00004000

#define SC_GBP_OK_SET			(0x80|15)
#define SC_GBP_OK_CLR			(0x00|15)
#define SC_GBP_OK_TST			0x00008000

#define SC_MIIM_RD_START_SET		(0x80|16)
#define SC_MIIM_RD_START_CLR		(0x00|16)
#define SC_MIIM_RD_START_TST		0x00010000

#define SC_MIIM_WR_START_SET		(0x80|17)
#define SC_MIIM_WR_START_CLR		(0x00|17)
#define SC_MIIM_WR_START_TST		0x00020000

#define SC_MIIM_OP_DONE_SET		(0x80|18)
#define SC_MIIM_OP_DONE_CLR		(0x00|18)
#define SC_MIIM_OP_DONE_TST		0x00040000

#define SC_MIIM_LINK_SCAN_EN_SET	(0x80|19)
#define SC_MIIM_LINK_SCAN_EN_CLR	(0x00|19)
#define SC_MIIM_LINK_SCAN_EN_TST	0x00080000


/* ARL DMA Address Register */
#define CMIC_ARL_DMA_ADDR		0x00000054	/* 32-bit address */

/* ARL DMA Packet count Register */
#define CMIC_ARL_DMA_CNT		0x00000058	/* 16-bit counter */

/*
 * S-Channel Error Status Register.
 *
 * Use the SCERR_XXX macros to extract fields.  Fields are valid only if
 * VALID bit is on.  VALID bit is cleared on register read.
 */
#define CMIC_SCHAN_ERR			0x0000005C

#define SCERR_VALID(reg)		((reg) >> 31 & 0x01)
#define SCERR_DST_PORT(reg)		((reg) >> 23 & 0xff)
#define SCERR_SRC_PORT(reg)		((reg) >> 15 & 0xff)
#define SCERR_OP_CODE(reg)		((reg) >>  8 & 0x7f)
#define SCERR_ERR_CODE(reg)		((reg) >>  0 & 0x03)

/*
 * COS Port Status
 *
 * There is one Active Port register per class of service.
 * Each register contains a port bitmap of active ports (see PBMP_*).
 * There are 4 classes of service (0 <= cos <= 3), with room for 8 more.
 */

#define CMIC_COS_ENABLE(cos)		(0x00000060 + 4 * (cos))

/*
 * ARL Message Buffers. These messages are in the format schan_msg_t:
 *  If msg.header.opcode==ARL_INSERT_CMD_MSG, use msg.arlins union member.
 *  If msg.header.opcode==ARL_DELETE_CMD_MSG, use msg.arldel union member.
 * There are four buffers (0 <= num <= 3) and each is four words in size.
 */

#define NUM_ARL_MBUF			4

#define CMIC_ARL_MBUF(num)		(0x00000080 + 0x20 * (num))

/* DMA Control Register */
#define CMIC_DMA_CTRL			0x00000100

#define DC_SOC_TO_MEM(ch)		(0x00 << (8 * (ch))) /* 0 <= ch <= 3 */
#define DC_MEM_TO_SOC(ch)		(0x01 << (8 * (ch)))
#define DC_DIRECTION_MASK(ch)		(0x01 << (8 * (ch)))

#define DC_MOD_BITMAP(ch)		(0x00 << (8 * (ch))) /* 0 <= ch <= 3 */
#define DC_NO_MOD_BITMAP(ch)		(0x02 << (8 * (ch)))
#define DC_MOD_BITMAP_MASK(ch)		(0x02 << (8 * (ch)))

#define DC_NO_ABORT_DMA(ch)		(0x00 << (8 * (ch))) /* 0 <= ch <= 3 */
#define DC_ABORT_DMA(ch)		(0x04 << (8 * (ch)))
#define DC_ABORT_DMA_MASK(ch)		(0x04 << (8 * (ch)))

#define DC_INTR_ON_PKT(ch)		(0x00 << (8 * (ch))) /* 0 <= ch <= 3 */
#define DC_INTR_ON_DESC(ch)		(0x08 << (8 * (ch)))
#define DC_INTR_ON_MASK(ch)		(0x08 << (8 * (ch)))

#define DC_NO_DROP_TX(ch)		(0x00 << (8 * (ch))) /* 0 <= ch <= 3 */
#define DC_DROP_TX(ch)			(0x10 << (8 * (ch)))
#define DC_DROP_TX_MASK(ch)		(0x10 << (8 * (ch)))

#define	DC_CHAN_MASK(ch)		(0xff << (8 * (ch)))

/* DMA Status Register */
#define CMIC_DMA_STAT			0x00000104

/*
 * DMA_STAT: control bits
 *
 *  xxx_SET and xxx_CLR can be WRITTEN to CMIC_DMA_STAT
 *  xxx_TST can be masked against values read from CMIC_DMA_STAT.
 *  Argument required: 0 <= ch <= 3
 */
#define DS_DMA_EN_SET(ch)		(0x80|(ch))
#define DS_DMA_EN_CLR(ch)		(0x00|(ch))
#define DS_DMA_EN_TST(ch)		(0x00000001 << (ch))

#define DS_CHAIN_DONE_SET(ch)		(0x80|(4+(ch)))
#define DS_CHAIN_DONE_CLR(ch)		(0x00|(4+(ch)))
#define DS_CHAIN_DONE_TST(ch)		(0x00000010 << (ch))

#define DS_DESC_DONE_SET(ch)		(0x80|(8+(ch)))
#define DS_DESC_DONE_CLR(ch)		(0x00|(8+(ch)))
#define DS_DESC_DONE_TST(ch)		(0x00000100 << (ch))

#define DS_DMA_RESET			0x1000
#define _DS_DMA_RESET			DS_DMA_RESET

#define DS_STAT_DMA_DONE_SET		(0x80|13)
#define DS_STAT_DMA_DONE_CLR		(0x00|13)
#define DS_STAT_DMA_DONE_TST		0x2000

#define DS_STAT_DMA_ITER_DONE_SET	(0x80|14)
#define DS_STAT_DMA_ITER_DONE_CLR	(0x00|14)
#define DS_STAT_DMA_ITER_DONE_TST	0x4000

#define DS_STAT_DMA_ACTIVE		0x20000

#define DS_DMA_ACTIVE(ch)		(0x00040000 << (ch))

/* These actually return the DMA channel number of failure */
#define DS_PCI_PARITY_ERR(reg)		((reg) >> 24 & 3)
#define DS_PCI_FATAL_ERR(reg)		((reg) >> 29 & 3)

/*
 * CMIC Head Of Line (HOL) Status.
 * Contains a port bitmap (see PBMP_*) of ports that are
 * currently AVAILABLE (NOT blocking on HOL)
 */
#define CMIC_HOL_STAT			0x00000108

/* CMIC Configuration Register */
#define CMIC_CONFIG			0x0000010C

#define CC_RD_BRST_EN			0x00000001 /* enb PCI read bursts */
#define CC_WR_BRST_EN			0x00000002 /* enb PCI write bursts */
#define CC_BE_CHECK_EN			0x00000004 /* Big endian chk PCI Wr */
#define CC_MSTR_Q_MAX_EN		0x00000008 /* Queue 4 PCI mastr reqs */
#define CC_LINK_STAT_EN			0x00000010 /* enb linkstat autoupd */
#define CC_RESET_CPS			0x00000020 /* Drive CPS bus reset */
#define CC_ACT_LOW_INT			0x00000040 /* INTA# is active low */
#define CC_SCHAN_ABORT			0x00000080 /* abort S-Channel opern */
#define CC_STACK_ARCH_EN		0x00000100 /* (see databook) */
#define CC_UNTAG_EN			0x00000200 /* (see databook) */
#define CC_LE_DMA_EN			0x00000400 /* Little endian DMA fmt */
#define CC_I2C_EN			0x00000800 /* enb CPU access to I2C */
#define CC_LINK_SCAN_GIG		0x00001000 /* Also scan gig ports */
#define CC_DMA_GARBAGE_COLL_EN		0x00008000 /* Collect 'purge' pkts */

#define CC_ALN_OPN_EN			0x00002000 /* DMA Un-Align Xfer enable */
#define CC_PCI_RST_ON_INIT_EN		0x00010000
#define CC_DIS_TIME_STAMP_CTR		0x00020000
#define CC_SG_OPN_EN			0x00040000 /* DMA S/G enable */
#define CC_RLD_OPN_EN			0x00080000 /* DMA RLD enable */
#define CC_DIS_RLD_STAT_UPDATE		0x00100000
#define CC_STOP_AUTO_SCAN_ON_LCHG	0x00200000 /* LS stop in intr */
#define CC_ABORT_STAT_DMA		0x00400000

#define N_DMA_CHAN			4

/* CMIC DMA Memory Address Registers (4 base addresses for 0 <= ch <= 3) */
#define CMIC_DMA_DESC(ch)		(0x00000110 + 4 * (ch))

/* CMIC I2C Registers */
#define CMIC_I2C_SLAVE_ADDR		0x00000120
#define CMIC_I2C_DATA			0x00000124
#define CMIC_I2C_CTRL			0x00000128
#define CMIC_I2C_STAT			0x0000012C
#define CMIC_I2C_SLAVE_XADDR		0x00000130
#define CMIC_I2C_GP0			0x00000134
#define CMIC_I2C_GP1			0x00000138
#define CMIC_I2C_RESET			0x0000013C

/*
 * Link Status (UP/Down) Register
 * Contains a port bitmap (see PBMP_*) of ports whose links are up.
 */
#define CMIC_LINK_STAT			0x00000140

/* IRQ Register (RO) */
#define CMIC_IRQ_STAT			0x00000144

/* IRQ Mask Register (R/W) */
#define CMIC_IRQ_MASK			0x00000148

/* IRQ Mask bits */
#define IRQ_SCH_MSG_DONE		0x00000001 /* Bit 0 */
#define IRQ_ARL_MBUF			0x00000002
#define IRQ_ARL_MBUF_DROP		0x00000004
#define IRQ_GBP_FULL			0x00000008
#define IRQ_LINK_STAT_MOD		0x00000010 /* Bit 4 */
#define IRQ_ARL_DMA_XFER		0x00000020
#define IRQ_ARL_DMA_CNT0		0x00000040
#define IRQ_DESC_DONE(ch)		(0x00000080 << (2 * (ch)))
#define IRQ_CHAIN_DONE(ch)		(0x00000100 << (2 * (ch)))
#define IRQ_PCI_PARITY_ERR		0x00008000
#define IRQ_PCI_FATAL_ERR		0x00010000
#define IRQ_SCHAN_ERR			0x00020000
#define IRQ_I2C_INTR			0x00040000
#define IRQ_MIIM_OP_DONE		0x00080000 /* Bit 19 */
#define IRQ_STAT_ITER_DONE		0x00100000 /* Bit 20 */
#define IRQ_ALL				0x001fffff
#define SOC_NUM_IRQ			21

/* Memory Fault Register */
#define CMIC_MEM_FAIL			0x0000014C

/* Ingress Back Pressure Warning Register */
#define CMIC_IGBP_WARN			0x00000150

/* Ingress Back Pressure Discard Register */
#define CMIC_IGBP_DISCARD		0x00000154

/* MIIM Parameter Register */
#define CMIC_MIIM_PARAM			0x00000158

/* MIIM Read Data Register */
#define CMIC_MIIM_READ_DATA		0x0000015C

/* Scan Ports Register */
#define CMIC_SCAN_PORTS			0x00000160

/* Counter DMA address register */
#define CMIC_STAT_DMA_ADDR		0x00000164

/* Counter DMA setup register */
#define CMIC_STAT_DMA_SETUP		0x00000168

#define SDS_EN				0x80000000
#define SDS_ET				0x40000000
#define SDS_TIMERVAL_SHFT		16
#define SDS_TIMERVAL_MASK		(0x3fff << SDS_TIMERVAL_SHFT)
#define SDS_LASTREG_SHFT		8
#define SDS_LASTREG_MASK		(0xff << SDS_LASTREG_SHFT)
#define SDS_FIRSTREG_SHFT		0
#define SDS_FIRSTREG_MASK		(0xff << SDS_FIRSTREG_SHFT)

/* Counter DMA valid ports register */
#define CMIC_STAT_DMA_PORTS		0x0000016c

/*
 * Counter DMA current position register
 * Contains the internal address of the counter register currently
 * being scanned.
 */

#define CMIC_STAT_DMA_CURRENT		0x00000170

/* Endian selection register */
#define CMIC_ENDIAN_SELECT		0x00000174

#define ES_BIG_ENDIAN_PIO		0x01010101
#define ES_BIG_ENDIAN_DMA_PACKET	0x02020202
#define ES_BIG_ENDIAN_DMA_OTHER		0x04040404

/*
 * Define the number of microseconds or polls before an S-Channel or
 * MIIM operation times out and fails.  This value is different based on
 * platform.  If a timeout occurs, the operation is aborted (schan_reset).
 * S-Channel timeout should be fairly long because the S-Channel may be
 * receiving heavy use and the CPU receives low priority access to it.
 */

#ifdef INTERRUPT_DRIVEN_SCHAN
#   if defined(PLISIM)
#	define SCHAN_TIMEOUT		(20 * 1000000)	/* usec */
#   else
#	ifdef QUICKTURN
#	    define SCHAN_TIMEOUT	3000000		/* usec */
#	else
#	    define SCHAN_TIMEOUT	300000		/* usec */
#	endif
#   endif
#else /* !INTERRUPT_DRIVEN_SCHAN */
#   if defined(PLISIM)
#	define SCHAN_TIMEOUT		10		/* polls */
#   else
#	ifdef QUICKTURN
#	    define SCHAN_TIMEOUT	500		/* polls */
#	else
#	    define SCHAN_TIMEOUT	10000		/* polls */
#	endif
#   endif
#endif /* !INTERRUPT_DRIVEN_SCHAN */

#ifdef INTERRUPT_DRIVEN_MIIM
#   if defined(PLISIM)
#	define MIIM_TIMEOUT		(20 * 1000000)	/* usec */
#   else
#	ifdef QUICKTURN
#	    define MIIM_TIMEOUT		1000000		/* usec */
#	else
#	    define MIIM_TIMEOUT		100000		/* usec */
#	endif
#   endif
#else /* !INTERRUPT_DRIVEN_MIIM */
#   if defined(PLISIM)
#	define MIIM_TIMEOUT		10		/* polls */
#   else
#	ifdef QUICKTURN
#	    define MIIM_TIMEOUT		500		/* polls */
#	else
#	    define MIIM_TIMEOUT		10000		/* polls */
#	endif
#   endif
#endif /* !INTERRUPT_DRIVEN_MIIM */


typedef union {
    uint32	dmac_val;	/* Raw 32-bit value */
    struct {
#ifdef	BCM5600_LE_HOST
	uint32	_cnt:16,	/* Byte count to transfer */
	        :1,		/* Reserved (0) */
	        _rld:1,		/* Reload */
	        _sg:1,		/* Scatter Gather */
	        :2,		/* Reserved (0) */
		:1,		/* Unused (IPX data) */
		_ip:1,		/* Indicates IP switched packet */
		_obit:2,	/* DMA Rd: Ingress optimization bits (below) */
		_crc:2,		/* If 0, packet is assumed to have valid */
				/*   CRC and is sent out as is */
				/* If 1, packet is assumed to have no CRC */
				/*   dest. egress will append it unless */
				/*   egress is CPU itself (MMU loopback). */
				/* If 2. packet is assumed to have CRC */
				/*   which is ignored and regenerated by */
				/*   egress unless egress is CPU itself. */
		_jbit:1,	/* DMA Rd: Ingress indicates jumbo packet */
		_cos:3,		/* DMA Rd: Class of service from packet */
		_chain:1;	/* DMA Chaining */
#else /* BCM5600_BE_HOST */
	uint32	_chain:1,
		_cos:3,
		_jbit:1,
		_crc:2,
		_obit:2,
		_ip:1,
		:1,
	        :2,		/* Reserved (0) */
	        _sg:1,		/* Scatter Gather */
	        _rld:1,		/* Reload */
	        :1,		/* Reserved (0) */
	        _cnt:16;	/* Byte Count */
#endif /* BCM5600_BE_HOST */
    } _dmac;
} dmac_t;			/* Control bit definition */

#define	dmac_cnt	_dmac._cnt
#define	dmac_ipx	_dmac._ipx
#define	dmac_ip		_dmac._ip
#define	dmac_obit	_dmac._obit
#define	dmac_crc	_dmac._crc
#define	dmac_jbit	_dmac._jbit
#define	dmac_cos	_dmac._cos
#define	dmac_chain	_dmac._chain
#define	dmac_sg		_dmac._sg
#define	dmac_rld	_dmac._rld

#define DMAC_INIT(dp, len)	((dp).dmac_val = 0, (dp).dmac_cnt = (len))

#define DMAC_OBIT_NONE		0	/* Normal packet */
#define DMAC_OBIT_CBIT		1	/* Packet was sent to CPU as a */
					/*   result of the C bit being set */
					/*   in the default router table */
#define DMAC_OBIT_FTYPE		2	/* Frame type mismatch (unused) */
#define DMAC_OBIT_PURGE		3	/* Packet from GARBAGE_COLLECT */

#define	DMAC_CRC_LEAVE	0		/* Egress should leave packet alone */
#define	DMAC_CRC_REGEN	2		/* Egress should regenerate CRC */


/*
 * DMA status word #0: updated by DMA controller on SOC -> memory
 * master write over PCI.
 */
typedef union {
    uint32	dmas0_val;	/* Raw 32-bit value */
    struct {
#ifdef BCM5600_LE_HOST
	uint32	_cnt:16,/* Transferred byte count (SOC -> MEM) */
		:4,	/* Reserved */
		_sw:1,	/* Start of packet transferred from SOC ->MEM */
		_ow:2,	/* 0:unused, 1:C bit set, 2:Frame Mismatch,3:rsvp */
		_crc:2,	/* If 0, packet contains valid CRC. */
			/* If 1, packet has no CRC */
			/* If 2, packet has wrong CRC */
			/* The CPU is actually being asked to perform a */
			/*   CRC function as if it were any other egress */
		:1,	/* Reserved */
		_cosw:3,/* COS value for packet transferred from SOC ->MEM */
		_utw:1,	/* If 1, CMIC was asked to remove tag from packet */
			/*    (as if it were any other egress) */
		_ew:1,	/* Last packet or end of packet */
		_v:1;	/* Status word valid */
#else
	uint32	_v:1,
		_ew:1,
		_utw:1,
		_cosw:3,
		:1,
		_crc:2,
		_ow:2,
		_sw:1,
		:4,
		_cnt:16;
#endif
    } _dmas0;
} dmas0_t;

#define	dmas0_cnt	_dmas0._cnt
#define	dmas0_sw	_dmas0._sw
#define	dmas0_ow	_dmas0._ow
#define	dmas0_crc	_dmas0._crc
#define	dmas0_cosw	_dmas0._cosw
#define	dmas0_utw	_dmas0._utw
#define	dmas0_ew	_dmas0._ew
#define	dmas0_v		_dmas0._v

typedef union {
    uint32	dmas1_val;	/* Full Raw value */
    struct {
#ifdef BCM5600_LE_HOST
	uint32	_sp:6,	/* Source port (Ingress-Origin) for this packet */
		:2,	/* Reserved */
		_d:1,	/* DMA Done */
		:5,	/* Reserved */
		_op:18;	/* CPU opcode (tells what to do with packet) */
#else
	uint32	_op:18,
		:5,
		_d:1,
		:2,
		_sp:6;
#endif
    } _dmas1;
} dmas1_t;

#define	dmas1_sp	_dmas1._sp
#define	dmas1_op	_dmas1._op
#define	dmas1_d		_dmas1._d

/*
 * CPU Opcodes
 */

#define	DMAS1_OP_FM	0x00001		/* Filter match and one of the
					 * action bits is send to CPU */
#define	DMAS1_OP_VSL	0x00002		/* PTABLE cpu-learn bit=1 and src
					 * MAC is learned in ARL; or CML=1
					 * and src MAC lookup failure; or
					 * or VLAN ID not found in VTABLE;
					 * or VLAN ID = 0xfff */
#define	DMAS1_OP_SRS	0x00004		/* Source routing bit is set in
					 * source MAC address (bit 40) */
#define	DMAS1_OP_DLF	0x00008		/* DLF or L3 station movement */
#define DMAS1_OP_CF	0x00010		/* Control frame: packet is BPDU,
					 * GVRP, GMRP, or reserved addr */
#define DMAS1_OP_IPSW	0x00020		/* Packet needs to be IP-switched */
#define DMAS1_OP_IPINF	0x00080		/* IP Info bit: set if IP options
					 * are present or IP version not 4 */
#define DMAS1_OP_IPMC	0x00100		/* Set for class D multicast pkt
					 * with DA = 01 00 5e xx yy zz, where
					 * bit 7 of xx is also zero */
#define DMAS1_OP_TTL	0x00200		/* Set if TTL <= 0 after decrement,
					   or less than threshold for IPMC */
#define DMAS1_OP_BCA	0x00400		/* Set if broadcast packet */
#define DMAS1_OP_MCA	0x00800		/* Set if (non-IP) multicast packet */
#define DMAS1_OP_IGMP	0x01000		/* IGMP packet */

/*
 * Typedef: DMA Control Block (dcb)
 * Purpose: Maps the CMIC DMA Control block area.
 *
 * Note:  Byte-swapping is required when reading or writing the 6 words
 * found in this structure.  For this reason it is illegal to access the
 * structure members directly.  Instead, the DCB_SET_* and DCB_GET_*
 * macros defined below should be used.
 */
typedef struct dcb_s {
    uint32	_dcb_addr;		/* 32-bit physical address */
    dmac_t	_dcb_dmac;		/* DMA control word */
    pbmp_t	_dcb_portbmp;		/* Port map definitions */
    pbmp_t	_dcb_utagbmp;		/* CP Channel transfers */
    dmas0_t	_dcb_dmas0;		/* DMA Status 0 */
    dmas1_t	_dcb_dmas1;		/* DMA Status 1 */
} dcb_t;

#define DCB_SET_ADDR(dcb, addr) \
    ((dcb)->_dcb_addr = pci_swap(addr))
#define DCB_SET_DMAC(dcb, dmac) \
    ((dcb)->_dcb_dmac.dmac_val = pci_swap(dmac.dmac_val))
#define DCB_SET_PORTBMP(dcb, pbmp) \
    ((dcb)->_dcb_portbmp = pci_swap(pbmp))
#define DCB_SET_UTAGBMP(dcb, ubmp) \
    ((dcb)->_dcb_utagbmp = pci_swap(ubmp))
#define DCB_SET_DMAS0(dcb, dmas0) \
    ((dcb)->_dcb_dmas0.dmas0_val = pci_swap(dmas0.dmas0_val))
#define DCB_SET_DMAS1(dcb, dmas1) \
    ((dcb)->_dcb_dmas1.dmas1_val = pci_swap(dmas1.dmas1_val))

#define DCB_CLR_DMAS0(dcb) \
    ((dcb)->_dcb_dmas0.dmas0_val = 0)
#define DCB_CLR_DMAS1(dcb) \
    ((dcb)->_dcb_dmas1.dmas1_val = 0)

#define DCB_ADDR(dcb) \
    (pci_swap((dcb)->_dcb_addr))
#define DCB_GET_DMAC(dcb, dmac) \
    ((dmac).dmac_val = pci_swap((dcb)->_dcb_dmac.dmac_val))
#define DCB_PORTBMP(dcb) \
    (pci_swap((dcb)->_dcb_portbmp))
#define DCB_UTAGBMP(dcb) \
    (pci_swap((dcb)->_dcb_utagbmp))
#define DCB_GET_DMAS0(dcb, dmas0) \
    ((dmas0).dmas0_val = pci_swap((dcb)->_dcb_dmas0.dmas0_val))
#define DCB_GET_DMAS1(dcb, dmas1) \
    ((dmas1).dmas1_val = pci_swap((dcb)->_dcb_dmas1.dmas1_val))

#define DCB_DONE(dcb) \
    ((pci_swap((dcb)->_dcb_dmas1.dmas1_val) & 0x100) != 0)

/*
 * CMIC I/O API
 */

/* Initialize Slot Free Address Pool */
int cmic_init_sfap(int unit);

/* Get CMIC PCI register */
int cmic_pci_setreg(int unit, uint32 addr, uint32 data);

/* Set CMIC PCI register */
uint32 cmic_pci_getreg(int unit, uint32 addr);

/* Dump an S-Channel header and raw data words */
void cmic_dump_schan_msg(schan_msg_t *msg, int dwc);

/* Transact an S-Channel message to CMIC */
int cmic_schan_op(int unit, schan_msg_t* msg, int dwc_write, int dwc_read);

/* Read internal register */
int cmic_read_reg(int unit, uint32 addr, uint32 *data);

/* Write internal register */
int cmic_write_reg(int unit, uint32 addr, uint32 data);

/* Write internal register for a group of ports */
int cmic_write_reg_ports(int unit, soc_reg_t reg, pbmp_t pbmp,
			 uint32 value);

/* Write internal register for a block or group of blocks */
int cmic_write_reg_blocks(int unit, soc_reg_t reg, soc_block_t block,
			  uint32 value);

/* Write internal memories
 * NOTE: GBP/CBP memory is only accessible when in DEBUG mode.
 */
int cmic_write_memory(int unit,
		      soc_mem_t mem,
		      int copyno,		/* COPYNO_ALL allowed */
		      int index,
		      void *entry_data);

/* Read internal memories
 * NOTE: GBP/CBP memory is only accessible when in DEBUG mode.
 */
int cmic_read_memory(int unit,
		     soc_mem_t mem,
		     int copyno,
		     int index,
		     void *entry_data);

int cmic_debug_memory(int unit, int enable);

/* Write to PHY registers */
int cmic_write_miim(int unit, uint8 phy_id,
		    uint8 phy_reg_addr, uint16 phy_wr_data);

/* Read from PHY registers */
int cmic_read_miim(int unit, uint8 phy_id,
		   uint8 phy_reg_addr, uint16 *phy_rd_data);

/*
 * Enable (unmask) or disable (mask) a set of CMIC interrupts.
 * These routines should be used instead of manipulating CMIC_IRQ_MASK
 * directly, since a shadow value is kept.  The return value is the
 * previous mask (can pass mask of 0 to just get the current mask).
 */
uint32 cmic_irq_enable(int unit, uint32 mask);
uint32 cmic_irq_disable(int unit, uint32 mask);
void cmic_link_scan_pause(int unit);
void cmic_link_scan_enable(int unit);

#endif /* _BCM5600_CMIC_H */
