/*
 *=============================================================================
 * File Name   :    bcm5600-error.h
 *
 *  RCS: $Id: bcm5600-error.h,v 1.1.1.1 2001/12/20 04:53:01 tonyw Exp $
 *
 *  Description:    Copied and modified from Broadcom
 *
 *  Created    :    09-04-2001
 *
 *  Author     :    Tony Wong
 *
 *  Copyright(c)  Agile Storage, Inc. 2001
 *=============================================================================
 */

#ifndef BCM5600_ERROR_H
#define BCM5600_ERROR_H

/*
 * Defines: 	
 *	SOC_XXX
 * Purpose:	
 *	SOC API return values.
 * Notes:
 *	Some SOC API return values are returned to the BCM layer. Those
 *	that are also have a BCM_E_ value defined in bcm/error.h.
 */

#define SOC_E_NONE		 0
#define	SOC_E_SCHAN_TIMEOUT	-1
#define	SOC_E_MII_TIMEOUT	-2
#define	SOC_E_INTERNAL		-3
#define SOC_E_NOT_IMPL		-4
#define	SOC_E_AUTONEG_DISABLED	-5
#define	SOC_E_AUTONEG_BUSY	-6
#define	SOC_E_AUTONEG_ERROR	-7
#define	SOC_E_CALLOUT		-8
#define SOC_E_MEMORY		-9
#define SOC_E_ENTRY_NOT_FOUND	-10
#define	SOC_E_SW_ARL_NOT_FOUND	-11
#define	SOC_E_SW_ARL_FULL	-12
#define	SOC_E_UNIT		-13
#define	SOC_E_INIT		-14
#define	SOC_E_DMA_FAILED	-15
#define SOC_E_DMA_BUSY		-16
#define	SOC_E_DMA_NOT_FOUND	-17
#define SOC_E_DMA_DV_FULL	-18
#define SOC_E_PARAM		-19

#define	SOC_E_LIMIT		-19	/* Maximum abs value */

#define SOC_SUCCESS(rv)		((rv) >= 0)	/* Demonstrative only */
#define SOC_FAILURE(rv)		((rv) < 0)	/* Demonstrative only */

extern char *soc_errmsg(int);

/*
 * Macro:
 *	SOC_IF_ERROR_RETURN
 * Purpose:
 * 	Evaluate _op as an expression, and if an error, return.
 * Notes:
 *	This macro uses a do-while construct to maintain expected
 *	"C" blocking, and evaluates "_op" ONLY ONCE so it may be
 *	a function call that has side affects.
 */
#define SOC_IF_ERROR_RETURN(_op)	\
    do { int __rv__; if ((__rv__ = (_op)) < 0) return(__rv__); } while(0)

#define BCM_E				SOC_E_LIMIT

#define BCM_E_NONE                      SOC_E_NONE
#define BCM_E_SCHAN_TIMEOUT             SOC_E_SCHAN_TIMEOUT
#define BCM_E_MII_TIMEOUT               SOC_E_MII_TIMEOUT
#define BCM_E_INTERNAL                  SOC_E_INTERNAL
#define BCM_E_AUTONEG_DISABLED          SOC_E_AUTONEG_DISABLED
#define BCM_E_AUTONEG_BUSY              SOC_E_AUTONEG_BUSY
#define BCM_E_AUTONEG_ERROR             SOC_E_AUTONEG_ERROR
#define BCM_E_NOT_IMPL                  SOC_E_NOT_IMPL
#define BCM_E_MEMORY                    SOC_E_MEMORY
#define BCM_E_UNIT                      SOC_E_UNIT
#define BCM_E_INIT                      (BCM_E - 1)
#define BCM_E_PARAM                     (BCM_E - 2)
#define BCM_E_PORT                      (BCM_E - 3)
#define BCM_E_PORT_NOT_FOUND            (BCM_E - 4)
#define BCM_E_PORT_EXISTS               (BCM_E - 5)
#define BCM_E_VLAN_ID                   (BCM_E - 6)
#define BCM_E_VLAN_NOT_FOUND            (BCM_E - 7)
#define BCM_E_VLAN_EXISTS               (BCM_E - 8)
#define BCM_E_VLAN_MAX                  (BCM_E - 9)
#define BCM_E_VLAN_PBM_CONFLICT         (BCM_E - 10)
#define BCM_E_VLAN_IF_CONFLICT          (BCM_E - 11)
#define BCM_E_TRUNK_ID                  (BCM_E - 12)
#define BCM_E_TRUNK_MAX                 (BCM_E - 13)
#define BCM_E_TPORT_NOT_FOUND           (BCM_E - 14)
#define BCM_E_TPORT_EXISTS              (BCM_E - 15)
#define BCM_E_L2_NOT_FOUND              (BCM_E - 16)
#define BCM_E_L3_NOT_FOUND              (BCM_E - 17)
#define BCM_E_MASK_MAX                  (BCM_E - 18)
#define BCM_E_HANDLER_NOT_FOUND         (BCM_E - 19)
#define BCM_E_HANDLER_BUSY              (BCM_E - 20)
#define BCM_E_UNTAG_CONFLICT            (BCM_E - 21)
#define BCM_E_INVALID_INDEX             (BCM_E - 22)
#define BCM_E_DUP_INSERT                (BCM_E - 23)
#define BCM_E_MASK_TBL_FULL             (BCM_E - 24)
#define BCM_E_MCAST_NOT_FOUND           (BCM_E - 25)
#define BCM_E_FILTER_MASK_NOT_FOUND     (BCM_E - 26)
#define BCM_E_FILTER_RULE_NOT_FOUND     (BCM_E - 27)
#define BCM_E_L3INTF_TBL_FULL           (BCM_E - 28)
#define BCM_E_L3INTF_TBL_EMPTY          (BCM_E - 29)
#define BCM_E_L3_TBL_FULL               (BCM_E - 30)
#define BCM_E_L3_TBL_EMPTY              (BCM_E - 31)
#define BCM_E_DEFIP_TBL_FULL            (BCM_E - 32)
#define BCM_E_DEFIP_TBL_EMPTY           (BCM_E - 33)

#define BCM_IF_ERROR_RETURN(_op)	SOC_IF_ERROR_RETURN(_op)

#endif	/* !_BCM_ERROR_H */
