/*
 *=============================================================================
 * File Name   :    bcm5600-init.c
 *
 *  RCS: $Id: bcm5600-init.c,v 1.11 2002/10/09 00:59:31 tonyw Exp $
 *
 *  Description:	Copied and modified from Broadcom
 *
 *  Created    :    09-04-2001
 *
 *  Author     :    Tony Wong
 *
 *  Copyright(c)  Agile Storage, Inc. 2001
 *=============================================================================
 */

#include <nfx-incl.h>
#include <exports/eee-api.h>
#ifndef SSC_MGMT
#include <exports/hwMap.h>
#include <exports/pci-api.h>
#endif

#include "bcm5600-soctypes.h"
#include "bcm5600-cmic.h"
#include "bcm5600-schanmsg.h"
#include "bcm5600-error.h"

#define CHK(r)	if ((r) < 0) return -1;
#define MEGA(x) ((x) * 1024 * 1024)

int soc_reset_init(int);
int soc_rc(int);

extern int bcm_l2_init_5600(int);
extern int bcm5600_port_init(int);
extern void delay_us(uint32);

#if 0
extern bcm_vlan_init_5600(int unit);

void
bcm5600_poll()
{
    /* manage link states/status */
    bcm5600_linkscan_update(0);
}
#endif

/*
 * Function:
 *	bcm_init_5600
 * Purpose:
 * 	Reset the switch chip, re-initialize, and initialize the BCM API layer.
 * Parameters:
 *	unit - StrataSwitch unit #.
 * Returns:
 *	BCM_E_XXX
 */
int 
bcm_init_5600(int unit)
{
	uint32 rc = NFX_OK;

    soc_reset_init(unit);

    /*
     * soc_rc() resets the chip and pre-loads all the registers 
     * and tables with initial values for normal switching.  
     */
	SOC_IF_ERROR_RETURN(soc_rc(unit));
    SOC_IF_ERROR_RETURN(bcm5600_port_init(unit));
    SOC_IF_ERROR_RETURN(bcm_l2_init_5600(unit));
#if 0
    SOC_IF_ERROR_RETURN(bcm_vlan_init_5600(unit));

	cmic_link_scan_enable(0); /* Enable link status scan */

    rc = eee_registerPollingFunc(EEE_LOW_PRIORITY, (void *)bcm5600_poll, NULL);
    if (rc != NFX_OK)
        printf("bcm_init_5600: couldn't register polling function\n");
#endif
    return rc;
}

/* Reset and initialize StrataSwitch */

int soc_reset_init(int unit)
{
#if defined(PROM) || defined(FCNIM)
	uint32 cmdstatus;

	pci_cfg_read(BCM5600_BUS_ID, BCM5600_DEV_ID, 0, 0x04, &cmdstatus);
	cmdstatus |= 0x00000002; 
    pci_cfg_write(BCM5600_BUS_ID, BCM5600_DEV_ID,0,0x04, cmdstatus);
    pci_cfg_write(BCM5600_BUS_ID, BCM5600_DEV_ID,0,0x10, BCM5600_PCI_BASE_ADDR);
#endif

#ifndef BCM5600_LE_HOST
    cmic_pci_setreg(unit, CMIC_ENDIAN_SELECT, 
			  ES_BIG_ENDIAN_PIO | ES_BIG_ENDIAN_DMA_OTHER);
#endif

	cmic_pci_setreg(unit, CMIC_CONFIG, CC_RESET_CPS);

	delay_us(10000);
	
#ifndef BCM5600_LE_HOST
    cmic_pci_setreg(unit, CMIC_ENDIAN_SELECT, 
			  ES_BIG_ENDIAN_PIO | ES_BIG_ENDIAN_DMA_OTHER);
#endif
	return(SOC_E_NONE);
}

int soc_rc(int unit)
{
    uint32 val;
	uint32	port;

    if (READ_SFAPPOOLSIZEr(unit, &val) < 0) {
        printf("soc_rc: couldn't read slot free address pool size!\n");
        return SOC_E_SCHAN_TIMEOUT;
    }

    SET_SFAPPOOLSIZEr_SDRAMSIZE(val, 1); /* 0=2 banks, 1=4 banks */
    SET_SFAPPOOLSIZEr_SFAPPOOLSIZE(val, 0x1fd ); /* 32 MB */
    SET_SFAPPOOLSIZEr_GBPFULLSETPOINT(val, 0x0f);
    SET_SFAPPOOLSIZEr_GBPFULLCLEARPOINT(val, 0x0b);

	WRITE_SUM_LWMSr(unit, 0x804);
	for (port = 0; port < 26; port++) {
		WRITE_HOLBLOCKINGMARKERr(unit, port, 0xe000);
	}

    if (WRITE_SFAPPOOLSIZEr(unit, val) < 0) {
        printf("soc_rc: couldn't write slot free address pool size!\n");
        return SOC_E_SCHAN_TIMEOUT;
    }

    cmic_write_reg(unit, SDRAMCONTROLr_ADDR(), 0x000000fc);
    cmic_write_reg(unit, SDRAMCONTROL2r_ADDR(), 0x6ddf8400);
    cmic_init_sfap(unit);

//    printf("SFAP initialized\n");
    return SOC_E_NONE;
}

