/*
 *=============================================================================
 * File Name   :    bcm5600-link.c
 *
 *  RCS: $Id: bcm5600-link.c,v 1.1.1.1 2001/12/20 04:53:01 tonyw Exp $
 *
 *  Description: Copied and modified from Broadcom
 *
 *  This module contains routines for detecting changes in link state and
 *  performing the appropriate actions, such as reprogramming the MAC
 *  speed/duplex to correspond with the negotiated PHY speed/duplex.
 *
 *  Created    :    09-04-2001
 *
 *  Author     :    Tony Wong
 *
 *  Copyright(c)  Agile Storage, Inc. 2001
 *=============================================================================
 */
#include <nfx-incl.h>

#include "bcm5600-soctypes.h"
#include "bcm5600-cmic.h"
#include "bcm5600-schanmsg.h"
#include "bcm5600-error.h"
#include "bcm5600-port.h"
#include "bcm5600-ll.h"
#include "bcm5600-phy.h"

typedef struct ls_cntl_s {
    int			lc_init;	/* TRUE if initialized */
    pbmp_t		lc_pbm_hw;	/* Hardware link scan ports */
    pbmp_t		lc_pbm_sw;	/* Software link scan ports */
    pbmp_t		lc_pbm_last;/* Last scan value */
} ls_cntl_t;

ls_cntl_t	bcm5600_link_control[N_SOC_DEVICES];

#if 0
/*
 * Function: 	
 *	_bcm_linkscan_update_sw
 * Purpose:	
 *	Look at all the PHYs and check for link UP/DOWN.
 * Parameters:	
 *	unit - StrataSwitch unit #.
 * Returns:	
 *	Port bit map, 1 indicates link active, 0 indicates link NOT active.
 */
static	pbmp_t
_bcm_linkscan_update_sw(int unit)
{
    ls_cntl_t		*lc = &bcm5600_link_control[unit];
    int			up;
    int			rv;
    pbmp_t		ls;		/* Return value */
    soc_port_t		port;		/* Port # */
    
    ls = 0;

    PBMP_ITER(lc->lc_pbm_sw, port) {
		if ((rv = bcm5600_port_link_status(unit, port, &up)) != BCM_E_NONE) {
	        printf("LinkScan: Unit %d Port %d: Failed to recover link status\n",
		           unit, port + 1);
		} 
		else 
	    if (up) {
	   	    ls |= PBMP_PORT(port);
		}
    }
    return(ls);
}

/*
 * Function: 	
 *	_bcm_linkscan_update_hw
 * Purpose:	
 *	Get current link status for ports being scanned by hardware.
 * Parameters:	
 *	unit - StrataSwitch unit #.
 * Returns:	
 *	nothing.
 * Notes:	
 *	Returns with link scanning paused.
 */
static pbmp_t
_bcm_linkscan_update_hw(int unit)
{
    ls_cntl_t		*lc = &bcm5600_link_control[unit];

    return(cmic_pci_getreg(unit, CMIC_LINK_STAT) & lc->lc_pbm_hw);
}

/*
 * Function:
 *	bcm5600_linkscan_update
 * Purpose:
 *	Performs actual scan and update of all links that have changed.
 * Parameters:
 *	unit - StrataSwitch unit #
 * Returns:
 *	Nothing
 */
void
bcm5600_linkscan_update(int unit)
{
    ls_cntl_t		*lc = &bcm5600_link_control[unit];
    int				link;
    int				rv = 0;
    soc_port_t		port;
    pbmp_t			ls_current, ls_changed, ls_ports;
    bcm_port_info_t	info;

    ls_ports   = lc->lc_pbm_hw | lc->lc_pbm_sw;
    ls_current = _bcm_linkscan_update_sw(unit) | _bcm_linkscan_update_hw(unit);
    ls_changed = (ls_current ^ lc->lc_pbm_last) & ls_ports & PBMP_E_ALL;

    PBMP_ITER(ls_changed, port) {
		if ((rv = bcm5600_port_update(unit, port, &link)) != BCM_E_NONE) {
	    	if (rv == BCM_E_AUTONEG_BUSY) {
			/* This requires another scan */
				ls_current &= ~PBMP_PORT(port);
	    	} 
			else {
			    printf("LinkScan: Unit %d Port %d:bcm5600_port_update failed\n",
 					   unit, port + 1); 
			    continue;
	    	}
		} 
	    
		if ((rv = bcm5600_port_info_get(unit, port, &info)) != BCM_E_NONE) {
	        printf("LinkScan: Unit %d Port %d:bcm5600_port_info_get failed\n", 
		           unit, port + 1);
	        continue;
		}

		/* Display state changes if requested. */

		if (info.linkstatus) {
	    	printf("LinkScan: Unit %d Port %d: Link active:%dMb/s %s\n", unit, 
               port + 1, info.speed, info.duplex ? "Full-duplex":"Half-duplex");
		} else {	
	     	 printf("LinkScan: Unit %d Port %d: Link inactive\n", 
                    unit, port + 1);
		  }    
    }
    lc->lc_pbm_last = ls_current;
}

/*
 * Function:
 *  bcm5600_linkscan_port_set
 * Purpose:
 *  Set the current scanning mode for the specified port.
 * Parameters:
 *  unit - StrataSwitch Unit #.
 *  port - StrataSwitch Port #.
 *  mode - Net scan mode for specified port.
 * Returns:
 *  BCM_E_XXX
 */
int
bcm5600_linkscan_port_set(int unit, soc_port_t port, int mode)
{
    ls_cntl_t   *lc = &bcm5600_link_control[unit];
    int     rv = BCM_E_NONE;

    if (!PBMP_MEMBER(PBMP_E_ALL, port)) {
    return(BCM_E_PARAM);
    }

    if (lc->lc_pbm_hw & PBMP_PORT(port)) { /* Disable H/W link scan */
    	lc->lc_pbm_hw &= ~PBMP_PORT(port);
    	cmic_pci_setreg(unit, CMIC_SCAN_PORTS, lc->lc_pbm_hw);
    } 
	else 
	if (lc->lc_pbm_sw & PBMP_PORT(port)) {
    	lc->lc_pbm_sw &= ~PBMP_PORT(port);
    }

    switch(mode) {
    	case BCM_PORT_LINKSCAN_NONE:    /* Do nothing */
    		break;
    	case BCM_PORT_LINKSCAN_SW:
    		lc->lc_pbm_sw |= PBMP_PORT(port);
    		break;
    	case BCM_PORT_LINKSCAN_HW:
    		lc->lc_pbm_hw |= PBMP_PORT(port);
    		cmic_pci_setreg(unit, CMIC_SCAN_PORTS, lc->lc_pbm_hw);
    		break;
    	default:
    		rv = BCM_E_PARAM;
    		break;
    }
    return(rv);
}

/*
 * Function:
 *  bcm5600_linkscan_pbm_set
 * Purpose:
 *  Set the current scanning mode for the specified ports.
 * Parameters:
 *  unit - StrataSwitch Unit #.
 *  pbm  - Port bit map indicating port to set.
 *  mode - New scan mode for specified ports.
 * Returns:
 *  BCM_E_XXX
 */
int
bcm5600_linkscan_pbm_set(int unit, pbmp_t pbm, int mode)
{
    soc_port_t  port;
    int     rv;

    PBMP_ITER(pbm, port) {
    	if ((rv = bcm5600_linkscan_port_set(unit, port, mode)) < 0) {
        	return(rv);
    	}
    }
    return(BCM_E_NONE);
}
#endif

