/*
 *=============================================================================
 * File Name   :    bcm5600-port.c
 *
 *  RCS: $Id: bcm5600-port.c,v 1.8 2002/10/09 00:34:53 tonyw Exp $		Copied and modified from Broadcom
 *
 *  Description:
 *
 *  Created    :    09-04-2001
 *
 *  Author     :    Tony Wong
 *
 *  Copyright(c)  Agile Storage, Inc. 2001
 *=============================================================================
 */

#include <nfx-incl.h>

#include "bcm5600-soctypes.h"
#include "bcm5600-cmic.h"
#include "bcm5600-schanmsg.h"
#include "bcm5600-error.h"
#include "bcm5600-port.h"
#include "bcm5600-ll.h"
#include "bcm5600-phy.h"

/* diag flag for enabling debug ports */
extern boolean bcm5600_enable_debug_ports;

#define BCM_VLAN_IFILTER_OFF    0
#define BCM_VLAN_IFILTER_ON     1

#if 0
extern void bcm5600_vlan_default_get(int unit, vlan_data_t *vdata);
#endif

typedef struct port_cntl_s {
    int			pc_mask_index;
    pbmp_t		pc_autoforward; 	/* Auto-forward mask */
#if 0
    phy_driver_t	*pc_phy[NUM_PORT_5600];	/* Per port PHY driver */
#endif
    mac_driver_t	*pc_mac[NUM_PORT_5600];	/* Per port MAC driver */
} port_cntl_t;

/* Declare static functions */
static int _bcm_port_stp_set(int unit, soc_port_t port, int stp_state);
#if 0
static int _bcm_ptable_update_vlan_remove(int unit,
                                          vlan_id_t vid, pbmp_t pbmp);
static int _bcm_ptable_update_vlan_add(int unit, vlan_id_t vid,
                           pbmp_t pbmp, pbmp_t ubmp);
static int _bcm_ptable_update_vlan_destroy(int unit, vlan_id_t vid);
static int _bcm_port_vlan_disable(int unit, pbmp_t ports);
#endif

/*
 * Variable:	bcm_port_control
 * Purpose:	One entry for each SOC device containing port information
 *		for that device.  Use the PORT macro to access it.
 */
static port_cntl_t bcm_port_control[N_SOC_DEVICES];

#define PORT(unit)	bcm_port_control[unit]		/* Current port */

static int
_port_ptable_get(int unit, soc_port_t port, ptable_entry_t *pv)
/*
 * Function: _port_ptable_get
 * Purpose:
 *	Retrieve the PTABLE entry for the specified port.
 * Parameters:
 *	unit - StrataSwitch unit number.
 *	port - StrataSwitch port bit mask of ports to update.
 *	pv   - pointer to ptable entry to write.
 * Returns:
 *	BCM_E_NONE
 *	BCM_E_INTERNAL if table read failed.
 * Notes:
 *	Assumes PLOCK head for specified unit.
 */
{
    return(cmic_read_memory(unit, PTABLE_m, 0, port, pv));
}

static int
_port_ptable_set(int unit, soc_port_t port, ptable_entry_t *pv)
/*
 * Function:
 *	_port_ptable_set
 * Purpose:
 *	Set the ptable entry for the specified port.
 * Parameters:
 *	unit - StrataSwitch unit number.
 *	port - StrataSwitch port bit mask of ports to update.
 *	pv   - pointer to ptable entry to write.
 * Returns:
 *	BCM_E_NONE
 *	BCM_E_INTERNAL if table write failed.
 * Notes:
 *	Assumes PLOCK head for specified unit.
 */
{
    return(cmic_write_memory(unit, PTABLE_m, COPYNO_ALL, port, pv));
}

#if 0
/*
 * Function: bcm5600_port_vlan_disable
 * Purpose:
 *	Disable PVLAN by setting VID to VLAN_ID_INVALID
 *      All bitmaps are set to all
 * Parameters:
 *	unit  - StrataSwitch unit number.
 *	ports - StrataSwitch port bit mask of ports to update.
 * Returns:
 *	BCM_E_NONE
 *	BCM_E_INTERNAL if table read failed.
 * Notes:
 */
int
bcm5600_port_vlan_disable(int unit, pbmp_t ports)
{
    int rv;

    rv = _bcm_port_vlan_disable(unit, ports);
    return rv;
}

/*
 * Function: _bcm_port_vlan_disable
 * Purpose:
 *      Guts of bcm5600_port_vlan_disable.  See above
 *      Assumes locking at higher level
 */
static int
_bcm_port_vlan_disable(int unit, pbmp_t ports)
{
    ptable_entry_t pv;
    soc_port_t p;

    PBMP_ITER(ports, p) {

        SOC_IF_ERROR_RETURN(_port_ptable_get(unit, p, &pv));

        pv.vlan_tag = VLAN_ID_INVALID;
        pv.port_bitmap = PBMP_ALL;
        pv.ut_port_bitmap = PBMP_ALL;

        SOC_IF_ERROR_RETURN(_port_ptable_set(unit, p, &pv));
    }

    return BCM_E_NONE;
}
#endif

static int
_bcm_port_stp_set(int unit, soc_port_t port, int stp_state)
/*
 * Function:
 *	_bcm_port_stp_set
 * Purpose:
 *	Set the Spanning tree state for a port.
 * Parameters:
 *	unit - StrataSwitch unit number.
 *	port - StrataSwitch port number.
 *	stp_state - State to place port in, one of BCM_PORT_STP_xxx.
 * Returns:
 *	BCM_E_NONE
 *	BCM_E_INTERNAL
 *
 * Notes:
 *	This routine manipulates the EPC_LINK register. No other
 *	modules should modify this value.
 *
 *      Minor workaround for 5600_b0, 5680_a0 chips:
 *          Turn learning off for ports put into blocking, listening
 *          or disabled state.  Turn learning on for ports put into
 *          forwarding or learning state state.  This is implemented
 *          via the "learn_flag" variable.
 *
 */
{
    uint32		epc_link, epc_link_add=0, epc_link_del=0;
    int			rv = BCM_E_NONE;
    int			p_state;
    uint32              lrn_add=0, lrn_remove=0;

    /* Calculate EPC link, learn and PTABLE state values */

    switch(stp_state) {
    case BCM_PORT_STP_FORWARDING:
        p_state = PVP_STP_FORWARDING;
	epc_link_add = PBMP_PORT(port);
        lrn_add = BCM_PORT_LEARN_ON;
	break;
    case BCM_PORT_STP_LISTENING:
    case BCM_PORT_STP_BLOCKING:
        p_state = PVP_STP_BLOCKING;
	epc_link_del = PBMP_PORT(port);
        lrn_remove = BCM_PORT_LEARN_ON;
        break;
    case BCM_PORT_STP_LEARNING:
        p_state = PVP_STP_LEARNING;
	epc_link_del = PBMP_PORT(port);
        lrn_add = BCM_PORT_LEARN_ON;
        break;
    case BCM_PORT_STP_DISABLED:
        p_state = PVP_STP_DISABLED;
	epc_link_del = PBMP_PORT(port);
	lrn_remove = BCM_PORT_LEARN_ON;
	break;
    default:
	return (BCM_E_PARAM);
    }

    /*
     * Pause link scanning if running to avoid EPC_LINK update from S/W
     * and H/W at the same time.
     */
#if 0
	cmic_link_scan_pause(unit);
#endif
    /*
     * Read EPC_LINK from one of the PICs, and update it. If successful,
     * write out the new PTABLE entries.
     */
    if ((rv = READ_EPC_LINKr(unit, 0, &epc_link) >= 0)) {
	ptable_entry_t		pv;

	epc_link |= epc_link_add;
	epc_link &= ~epc_link_del;

	rv = cmic_write_reg_blocks(unit, EPC_LINKr, ANY_PIC, epc_link);
	if (rv >= 0) {
	    rv = _port_ptable_get(unit, port, &pv);
        }
        if (rv >= 0) {
            pv.sp_st = p_state;
            rv = _port_ptable_set(unit, port, &pv);
        }
    }
    if (rv >= 0) {
        rv = bcm5600_port_learn_modify(unit, port, lrn_add, lrn_remove);
    }
#if 0
	cmic_link_scan_enable(unit);
#endif
    return(rv);
}

int
bcm5600_port_monitor_ingress_set(int unit, soc_port_t port, int enable)
{
	int 				rv;
	ptable_entry_t		pv;

	rv = _port_ptable_get(unit, port, &pv);
	if (rv >= 0) {
		if (enable == 1)
			pv.mi = 1;
		else
			pv.mi = 0;
		rv = _port_ptable_set(unit, port, &pv);
	}
	return(BCM_E_NONE);
}

/* 
 * Function:
 *     bcm5600_port_enable_set
 * Purpose:
 *     Physically enable/disable the PHY on this port. 
 *     In the future, this may also disable the MAC TX/RX
 *     and or egress
 * Parameters:
 *	unit - StrataSwitch unit #.
 *	port - StrataSwitch port #.
 *	enable - TRUE, port is enabled, FALSE port is disabled.
 * Returns:
 *     BCM_E_XXXX
 */

int 
bcm5600_port_enable_set(int unit, soc_port_t port, int enable)
{
#if 0
    return(PHY_ENABLE_SET(PORT(unit).pc_phy[port], unit, port, enable));
#endif
	return(BCM_E_NONE);
}

/* 
 * Function:
 *     bcm5600_port_enable_get
 * Purpose:
 *     Gets the enable state as defined by bcm5600_port_enable_set()
 * Parameters:
 *	unit - StrataSwitch unit #.
 *	port - StrataSwitch port #.
 *	enable - (OUT) TRUE, port is enabled, FALSE port is disabled.
 * Returns:
 *     BCM_E_XXXX
 */
int 
bcm5600_port_enable_get(int unit, soc_port_t port, int *enable)
{
#if 0
    return(PHY_ENABLE_GET(PORT(unit).pc_phy[port], unit, port, enable));
#endif
	return(BCM_E_NONE);
}

/*
 * Function:
 *	bcm5600_port_egress_enable_set
 * Purpose:
 *	Enable/Disable the egress for the specified port.
 * Parameters:
 *	unit - StrataSwitch unit #.
 *	port - StrataSwitch port #.
 *	enable - Boolean: TRUE enable egress, FALSE disable egress.
 * Returns:
 *	BCM_E_XXXX
 */
int
bcm5600_port_egress_enable_set(int unit, soc_port_t port, int enable)
{
    int		rv;
    uint32	egr_enable, egr_mask, egr_addr;

#if NUM_FE_PORT_5600 > 0
    if (PBMP_MEMBER(PBMP_FE_ALL, port)) {
	egr_mask = 1 << (port % 8);
	egr_addr = EGR_ENABLEr_ADDR(port2block(port));
    } else 
#endif

#if NUM_GE_PORT_5600 > 0
    if (PBMP_MEMBER(PBMP_GE_ALL, port)) {
	egr_mask = 1;
	egr_addr = GEGR_ENABLEr_ADDR(port2block(port));
    } else 
#endif
    {
	return(BCM_E_PARAM);
    }

    rv = cmic_read_reg(unit, egr_addr, &egr_enable);
    if (rv >= 0) {
	if (enable) {
	    egr_enable |= egr_mask;
	} else {
	    egr_enable &= ~egr_mask;
	}
	rv = cmic_write_reg(unit, egr_addr, egr_enable);
    }
    return(rv);
}

/*
 * Function:
 *	bcm5600_port_egress_enable_get
 * Purpose:
 *	Retrieve the Enable/Disable state of an egress.
 * Parameters:
 *	unit - StrataSwitch unit #.
 *	port - StrataSwitch port #.
 *	enable - Boolean: TRUE enable egress, FALSE disable egress.
 * Returns:
 *	BCM_E_XXXX
 */
int
bcm5600_port_egress_enable_get(int unit, soc_port_t port, int *enable)
{
    uint32	egr_enable, egr_mask;

#if NUM_FE_PORT_5600 > 0
    if (PBMP_MEMBER(PBMP_FE_ALL, port)) {
	egr_mask = 1 << (port % 8);
	SOC_IF_ERROR_RETURN(READ_EGR_ENABLEr(unit, port2block(port),
					     &egr_enable));
    } else
#endif

#if NUM_GE_PORT_5600 > 0
    if (PBMP_MEMBER(PBMP_GE_ALL, port)) {
	SOC_IF_ERROR_RETURN(READ_GEGR_ENABLEr(unit, port2block(port),
					      &egr_enable));
	egr_mask = 1;
    } else 
#endif
    {
	return(BCM_E_PARAM);
    }

    *enable = (egr_enable & egr_mask) ? TRUE : FALSE;
    return(BCM_E_NONE);
}

/*
 * Function:
 *	_bcm_port_setup_mode
 * Purpose:
 *	Set initial operating mode for a port.
 * Parameters:
 *	unit - StrataSwitch unit #.
 *	port - StrataSwitch port #.
 *	enable - Boolean: TRUE enable egress, FALSE disable egress.
 * Returns:
 *	BCM_E_XXXX
 */
static int
_bcm_port_setup_mode(int unit, soc_port_t port)
{
    port_mode_t	pm;

    SOC_IF_ERROR_RETURN(bcm5600_port_ability_get(unit, port, &pm));

    /* If MII supported, enable it, otherwise use 10B */
#if 0
    SOC_IF_ERROR_RETURN(PHY_INTERFACE_SET(PORT(unit).pc_phy[port], unit, port,
					  pm & SOC_PM_MII ? 
					  TRUE : FALSE));
#endif
    SOC_IF_ERROR_RETURN(MAC_INTERFACE_SET(PORT(unit).pc_mac[port], unit, port, 1));
    return(BCM_E_NONE);
}

/*
 * Function:
 *	bcm5600_port_init
 * Purpose:
 *	Initialize the PORT interface layer for the specified SOC device.
 * Parameters:
 *	unit - StrataSwitch unit number.
 * Returns:
 *	BCM_E_NONE - success (or already initialized)
 *	BCM_E_INTERNAL- failed to write PTABLE entries
 *	BCM_E_MEMORY - failed to allocate required memory.
 * Notes:
 *	All ports set in forwarding state.
 *	PTABLE initialized.
 */
int
bcm5600_port_init(int unit)
{
    int				i, rv = BCM_E_NONE;
    vlan_data_t     vdata;
    soc_port_t      p;

    /* Initialize each port's PTABLE entry to cleared state */
    vdata.vlan_tag = VLAN_ID_INVALID;
    vdata.port_bitmap = PBMP_ALL;
    vdata.ut_port_bitmap = PBMP_ALL;
    bcm5600_ptable_init(unit, &vdata);
    
    /* Probe and initialize MAC and PHY drivers */
#if defined(PROM) || defined(FCNIM)
	/* FC-NIM */
	/* Enable FE ports 15-23 and GE port 24-25 */
	for (p = 15; p < 26; p++) {
		int	enable_port = TRUE;

        /* don't enable unused ports to Galileo */
		if ((bcm5600_enable_debug_ports == 0) && ((p >= 20) && (p <= 23)))
			continue;
#else
	/* SSC */
	/* Enable FE ports 0-4 and FE port connected to BSD Galileo(23) */
	for (p = 0; p <= 5; p++) {
		int	enable_port = TRUE;
		if (p == 5) /* temp hack to init port 23 also */
			p = 23;
#endif
		if (!PBMP_MEMBER(PBMP_E_ALL, p)) {
	    	continue;
		}
#if 0
		PORT(unit).pc_phy[p] = &phy_default_fe;
#endif
		PORT(unit).pc_mac[p] = soc_mac_probe(unit, p);

		if (/*PORT(unit).pc_phy[p] != NULL &&*/ PORT(unit).pc_mac[p] != NULL) {
#if 0
	    	if ((rv = PHY_INIT(PORT(unit).pc_phy[p], unit, p)) < 0) {
				printf("Warning: Unit %d Port %d: Failed to initialize PHY:\n",
				       unit, p + 1);
				enable_port = FALSE;
	    	}
#endif
	    	if ((rv = MAC_INIT(PORT(unit).pc_mac[p], unit, p)) < 0) {
				printf("Warning: Unit %d Port %d: Failed to initialize MAC: \n",
					   unit, p + 1);
				enable_port = FALSE;
	    	}
		} else {
	    	enable_port = FALSE;
		}
		if ((rv = _bcm_port_stp_set(unit, p, BCM_PORT_STP_FORWARDING)) < 0) {
	    	printf("Warning: Unit %d Port %d: _bcm_port_stp_set failed: \n",
		   		   unit, p + 1);
		}

		/* Set default speeds etc. */
	
		if (enable_port) {
	    	if ((rv = _bcm_port_setup_mode(unit, p)) < 0) {
				printf("Warning: Unit %d Port %d: Failed to set port mode:\n",
					   unit, p + 1);
	    	}
		}
		if ((rv = bcm5600_port_egress_enable_set(unit, p, enable_port)) < 0){
	    	printf("Warning: Unit %d Port %d: Egress Enable failed: \n",
		   		   unit, p + 1);
		}
    }

    PORT(unit).pc_autoforward = 1;

	if (isFC()) {
		/* disable unused FE ports on FC-NIM */
		for (i = 0; i <= 14; i++) {
			bcm5600_port_egress_enable_set(unit, i, 0);	
		}
		if (bcm5600_enable_debug_ports == 0) {
            for (i = 20; i <= 23; i++) {
                bcm5600_port_egress_enable_set(unit, i, 0);	
            }
		}
	}
	else
	if (isSSC()) {
		/* disable unused FE and GE ports on SSC */
		for (i = 5; i <= 22; i++) {
			bcm5600_port_egress_enable_set(unit, i, 0);	
		}
		bcm5600_port_egress_enable_set(unit, 24, 0);	
		bcm5600_port_egress_enable_set(unit, 25, 0);	
	}
//	printf("bcm5600_port_init done\n");
    return(BCM_E_NONE);
}

/*
 * Function:
 *	bcm5600_ptable_init(int unit, vlan_data_t *vdata)
 * Purpose:
 *	Initialize the ptable according to data in vdata
 * Parameters:
 *	unit - StrataSwitch unit number.
 *	vdata - Set the port based VLAN information for the ports in
 *              vdata->port_bitmap
 * Returns:
 *	BCM_E_NONE
 *	BCM_E_INTERNAL - internal error.
 */
int
bcm5600_ptable_init(int unit, vlan_data_t *vdata)
{
    soc_port_t p;
    ptable_entry_t	pv;

    memset(&pv, 0, sizeof (pv));

    pv.vlan_tag = vdata->vlan_tag;
    pv.port_bitmap = vdata->port_bitmap;
    pv.ut_port_bitmap = vdata->ut_port_bitmap;
    PBMP_ITER(vdata->port_bitmap, p) {
	if (PBMP_MEMBER(PBMP_CMIC, p)) {
	    pv.ptype = 3;
	} else if (PBMP_MEMBER(PBMP_FE_ALL, p)) {
	    pv.ptype = 1;
	} else if (PBMP_MEMBER(PBMP_GE_ALL, p)) {
	    pv.ptype = 2;
	} else {
	    printf("bcm5600_ptable_init: error - unknown port type\n");
	}
	SOC_IF_ERROR_RETURN(_port_ptable_set(unit, p, &pv));
    }
//	printf("ptable init done\n");
    return(BCM_E_NONE);
}

/*
 * Function:
 *	bcm_port_stp_set
 * Purpose:
 *	Set the Spanning tree state for a port.
 * Parameters:
 *	unit - StrataSwitch unit number.
 *	port - StrataSwitch port number.
 *	stp_state - State to place port in.
 * Returns:
 *	BCM_E_NONE
 *	BCM_E_INTERNAL
 *
 * Notes:
 *	This routine manipulates the EPC_LINK register. No other
 *	modules should modify this value.
 */
int
bcm5600_port_stp_set(int unit, soc_port_t port, int stp_state)
{
    int		rv;

    rv = _bcm_port_stp_set(unit, port, stp_state);
    return(rv);
}

/*
 * Function:
 *	bcm5600_port_stp_get
 * Purpose:
 *	Get the Spanning tree state for a port.
 * Parameters:
 *	unit - StrataSwitch unit number.
 *	port - StrataSwitch port number.
 *	stp_state - Pointer where state stored.
 * Returns:
 *	BCM_E_NONE
 *	BCM_E_INTERNAL
 */
int
bcm5600_port_stp_get(int unit, soc_port_t port, int *stp_state)
{
    int			rv;
    ptable_entry_t	pv;

    /* Determine PTABLE state values */

    if ((rv = _port_ptable_get(unit, port, &pv)) == BCM_E_NONE) {
	switch(pv.sp_st) {
	case PVP_STP_FORWARDING: *stp_state = BCM_PORT_STP_FORWARDING; break;
	case PVP_STP_BLOCKING: *stp_state = BCM_PORT_STP_BLOCKING;     break;
	case PVP_STP_LEARNING: *stp_state = BCM_PORT_STP_LEARNING;     break;
	case PVP_STP_DISABLED: *stp_state = BCM_PORT_STP_DISABLED;     break;
	default:
	    rv = BCM_E_INTERNAL;
	}
    }

    return(rv);
}

/*
 * Function:
 *	bcm5600_port_enable_bpdu
 * Purpose:
 *	Enable/Disable BPDUs on the specified port.
 * Parameters:
 *	unit - SOC unit #
 *	port - Port number (0 based)
 *	enable - TRUE to enable, FALSE to disable (reject bpdu).
 * Returns:
 *	BCM_E_NONE
 *	BCM_E_INTERNAL
 */
int
bcm5600_port_enable_bpdu(int unit, soc_port_t port, int enable)
{
    ptable_entry_t	pv;
    int			rv;

    if ((rv = _port_ptable_get(unit, port, &pv)) == BCM_E_NONE) {
	pv.bpdu = enable ? 0 : 1;
	rv = _port_ptable_set(unit, port, &pv);
    }

    return(rv);
}

/*
 * Function: bcm5600_port_autoneg_get
 * Purpose: Get the autonegotiation state of the port
 * Parameters:
 *	unit - StrataSwitch Unit #.
 *	port - StrataSwitch port #.
 *	autoneg - (OUT) Boolean value
 * Returns:
 *	BCM_E_NONE
 *	BCM_E_XXX
 */
int
bcm5600_port_autoneg_get(int unit, soc_port_t port, int *autoneg)
{
#if 0
    int	busy;

    return(PHY_AUTO_NEGOTIATE_GET(PORT(unit).pc_phy[port], unit, port,
				  autoneg, &busy));
#endif
	return 0;
}

/*
 * Function: bcm5600_port_autoneg_set
 * Purpose: Set the autonegotiation state for a given port
 * Parameters:
 *	unit - StrataSwitch Unit #.
 *	port - StrataSwitch port #.
 *	autoneg - Boolean value
 * Returns:
 *	BCM_E_NONE
 *	BCM_E_XXX
 */
int
bcm5600_port_autoneg_set(int unit, soc_port_t port, int autoneg)
{
#if 0
    int		rv;

    rv = PHY_AUTO_NEGOTIATE_SET(PORT(unit).pc_phy[port], unit, port, autoneg);

    return(rv);
#endif
	return 0;
}

/*
 * Function: bcm5600_port_autoforward_get
 * Purpose: Get the auto-forward state of the port
 * Parameters:
 *	unit - StrataSwitch Unit #.
 *	port - StrataSwitch port #.
 *	autofwd - (OUT) Boolean value
 * Returns:
 *	BCM_E_NONE
 *	BCM_E_XXX
 */
int
bcm5600_port_autoforward_get(int unit, soc_port_t port, int *autofwd)
{
    *autofwd = PBMP_MEMBER(PORT(unit).pc_autoforward, port);
    return(BCM_E_NONE);
}

/*
 * Function: bcm5600_port_autoforward_set
 * Purpose: Set the auto-forwarding state for a given port
 * Parameters:
 *	unit - StrataSwitch Unit #.
 *	port - StrataSwitch port #.
 *	autofwd - Boolean value
 * Returns:
 *	BCM_E_NONE
 *	BCM_E_XXX
 * Notes: If enabling Autoforward, STP state is immediately changed to the 
 *	correct value (DISABLED or FORWARDING)
 */
int
bcm5600_port_autoforward_set(int unit, soc_port_t port, int autofwd)
{
    int		rv;

    if (autofwd) {
	int	link;

	if (0 <=  (rv = bcm5600_port_link_status(unit, port, &link))) {
	    PORT(unit).pc_autoforward |= PBMP_PORT(port);
	    rv = _bcm_port_stp_set(unit, port, 
				   link ? 
				   BCM_PORT_STP_FORWARDING : 
				   BCM_PORT_STP_DISABLED);
	}
    } else {
	PORT(unit).pc_autoforward &= ~PBMP_PORT(port);
	rv = BCM_E_NONE;
    }
    return(rv);
}

/*
 * Function: bcm5600_port_speed_get
 * Purpose: Getting the spped of the port
 * Parameters:
 *	unit - StrataSwitch Unit #.
 *	port - StrataSwitch port #.
 *	speed - (OUT) Value in megabits/sec (10, 100, etc)
 * Returns:
 *	BCM_E_NONE
 *	BCM_E_XXX
 */
int
bcm5600_port_speed_get(int unit, soc_port_t port, int *speed)
{
#if 0
    return(PHY_SPEED_GET(PORT(unit).pc_phy[port], unit, port, speed));
#endif
	return 0;
}

/*
 * Function: 
 *	bcm5600_port_speed_set
 * Purpose: 
 *	Setting the speed for a given port
 * Parameters:
 *	unit - StrataSwitch Unit #.
 *	port - StrataSwitch port #.
 *	speed - Value in megabits/sec (10, 100, etc)
 * Returns:
 *	BCM_E_NONE
 *	BCM_E_XXX
 */
int
bcm5600_port_speed_set(int unit, soc_port_t port, int speed)
{
#if 0
    int		rv;

    rv = PHY_AUTO_NEGOTIATE_SET(PORT(unit).pc_phy[port], unit, port, FALSE);
    if (rv >= 0) {
	rv = PHY_SPEED_SET(PORT(unit).pc_phy[port], unit, port, speed);
    }
    if (rv >= 0) {
	rv = MAC_SPEED_SET(PORT(unit).pc_mac[port], unit, port, speed);
    }
    
    return(rv);
#endif
	return 0;
}

/*
 * Function: 
 *	bcm5600_port_loopback_set
 * Purpose: 
 *	Setting the speed for a given port
 * Parameters:
 *	unit - StrataSwitch Unit #.
 *	port - StrataSwitch port #.
 *	loopback - (OUT) one of:
 *		BCM_PORT_LOOPBACK_NONE
 *		BCM_PORT_LOOPBACK_MAC
 *		BCM_PORT_LOOPBACK_PHY
 * Returns:
 *	BCM_E_NONE
 *	BCM_E_XXX
 */
int
bcm5600_port_loopback_set(int unit, soc_port_t port, int loopback)
{
    int		error;
    int		rv = BCM_E_NONE;

    switch(loopback) {
    case BCM_PORT_LOOPBACK_NONE:
	error =  MAC_LOOPBACK_SET(PORT(unit).pc_mac[port], unit, port, FALSE);
#if 0
	error |= PHY_LOOPBACK_SET(PORT(unit).pc_phy[port], unit, port, FALSE);
#endif
	break;
    case BCM_PORT_LOOPBACK_MAC:
	error =  MAC_LOOPBACK_SET(PORT(unit).pc_mac[port], unit, port, TRUE);
#if 0
	error |= PHY_LOOPBACK_SET(PORT(unit).pc_phy[port], unit, port, FALSE);
#endif
	break;
    case BCM_PORT_LOOPBACK_PHY:
	error =  MAC_LOOPBACK_SET(PORT(unit).pc_mac[port], unit, port, FALSE);
#if 0
	error |= PHY_LOOPBACK_SET(PORT(unit).pc_phy[port], unit, port, TRUE);
#endif
	break;
    default:
	error = 0;
	rv = BCM_E_PARAM;
    }
    if (error < 0) {
	rv = BCM_E_INTERNAL;
    }

    return(rv);
}
/*
 * Function: 
 *	bcm5600_port_loopback_get
 * Purpose: 
 *	Recover the current loopback operation for the specified port.
 * Parameters:
 *	unit - StrataSwitch Unit #.
 *	port - StrataSwitch port #.
 *	loopback - (OUT) one of:
 *		BCM_PORT_LOOPBACK_NONE
 *		BCM_PORT_LOOPBACK_MAC
 *		BCM_PORT_LOOPBACK_PHY
 * Returns:
 *	BCM_E_NONE
 *	BCM_E_XXX
 */
int
bcm5600_port_loopback_get(int unit, soc_port_t port, int *loopback)
{
    int		rv = 0;
    int		phy_lb = 0;
	int		mac_lb = 0;

#if 0
    rv  = PHY_LOOPBACK_GET(PORT(unit).pc_phy[port], unit, port, &phy_lb);
#endif
    rv |= MAC_LOOPBACK_GET(PORT(unit).pc_mac[port], unit, port, &mac_lb);

    if (rv < 0) {
	return(BCM_E_INTERNAL);
    } else if (mac_lb) {
	*loopback = BCM_PORT_LOOPBACK_MAC;
    } else if (phy_lb) {
	*loopback = BCM_PORT_LOOPBACK_PHY;
    } else {
	*loopback = BCM_PORT_LOOPBACK_NONE;
    }

    return(BCM_E_NONE);
}

/*
 * Function: bcm5600_port_duplex_get
 * Purpose:  Get the port duplex settings
 * Parameters:
 *	unit - StrataSwitch Unit #.
 *	port - StrataSwitch port #.
 *      duplex - (OUT) Duplex setting, one of BCM_PORT_DUPLEX_xxx
 * Returns:
 *	BCM_E_NONE
 *	BCM_E_XXX
 */
int
bcm5600_port_duplex_get(int unit, soc_port_t port, int *duplex)
{
#if 0
    int	phy_duplex;

    if (PHY_DUPLEX_GET(PORT(unit).pc_phy[port], unit, port, &phy_duplex)) {
	return(BCM_E_INTERNAL);
    }

    *duplex = phy_duplex ? BCM_PORT_DUPLEX_FULL : BCM_PORT_DUPLEX_HALF;
#endif
    return(BCM_E_NONE);
}

/*
 * Function: bcm5600_port_duplex_set
 * Purpose:  Set the port duplex settings.
 * Parameters:
 *	unit - StrataSwitch Unit #.
 *	port - StrataSwitch port #.
 *      duplex - Duplex setting, one of BCM_PORT_DUPLEX_xxx
 * Returns:
 *	BCM_E_NONE
 *	BCM_E_XXX
 */
int
bcm5600_port_duplex_set(int unit, soc_port_t port, int duplex)
{
    int		rv = BCM_E_NONE;
#if 0
    rv =  PHY_AUTO_NEGOTIATE_SET(PORT(unit).pc_phy[port], unit, port, FALSE);
    if (rv >= 0) {
	rv = PHY_DUPLEX_SET(PORT(unit).pc_phy[port], unit, port, duplex);
    }
#endif
    if (rv >= 0) {
	rv = MAC_DUPLEX_SET(PORT(unit).pc_mac[port], unit, port, duplex);
    }

    return(rv ? BCM_E_INTERNAL : BCM_E_NONE);
}

/*
 * Function: bcm5600_port_pause_get
 * Purpose: 
 *	Get the source address for transmitted PAUSE frames.
 * Parameters:
 *	unit - StrataSwitch Unit #.
 *	port - StrataSwitch port #.
 *	pause_tx - (OUT) Boolean value
 *	pause_rx - (OUT) Boolean value
 * Returns:
 *	BCM_E_NONE
 *	BCM_E_XXX
 */
int
bcm5600_port_pause_get(int unit, soc_port_t port, int *pause_tx, int *pause_rx)
{
    int		rv;

    rv = MAC_PAUSE_GET(PORT(unit).pc_mac[port], unit, port, pause_tx, pause_rx);
    return(rv);
}

/*
 * Function: bcm5600_port_pause_set
 * Purpose: Set the pause state for a given port
 * Parameters:
 *	unit - StrataSwitch Unit #.
 *	port - StrataSwitch port #.
 *	pause_tx - Boolean value
 *	pause_rx - Boolean value
 * Returns:
 *	BCM_E_NONE
 *	BCM_E_XXX
 * Notes:
 * 	Symmetric pause requires the two "pause" values to be the same.
 */
int
bcm5600_port_pause_set(int unit, soc_port_t port, int pause_tx, int pause_rx)
{
    int		rv;

#if 0    
    rv = PHY_AUTO_NEGOTIATE_SET(PORT(unit).pc_phy[port], unit, port, FALSE);
    if (rv >= 0) 
#endif
	rv = MAC_PAUSE_SET(PORT(unit).pc_mac[port], unit, port, 
			   pause_tx, pause_rx);
    
    return(rv);
}

/*
 * Function: bcm5600_port_pause_addr_get
 * Purpose: 
 *	Get the source address for transmitted PAUSE frames.
 * Parameters:
 *	unit - StrataSwitch Unit #.
 *	port - StrataSwitch port #.
 *	pause_tx - (OUT) Boolean value
 *	pause_rx - (OUT) Boolean value
 *	mac - (OUT) MAC address sent with apuse frames.
 * Returns:
 *	BCM_E_NONE
 *	BCM_E_XXX
 */
int
bcm5600_port_pause_addr_get(int unit, soc_port_t port, mac_addr_t mac)
{
    int		rv;

    rv = MAC_PAUSE_ADDR_GET(PORT(unit).pc_mac[port], unit, port, mac);
    return(rv);
}

/*
 * Function: bcm5600_port_pause_addr_set
 * Purpose: Set the source address for transmistted PAUSE frames.
 * Parameters:
 *	unit - StrataSwitch Unit #.
 *	port - StrataSwitch port #.
 *	pause_tx - Boolean value
 *	pause_rx - Boolean value
 *	mac - station address used for pause frames. 
 * Returns:
 *	BCM_E_NONE
 *	BCM_E_XXX
 * Notes:
 * 	Symmetric pause requires the two "pause" values to be the same.
 */
int
bcm5600_port_pause_addr_set(int unit, soc_port_t port, mac_addr_t mac)
{
    int		rv;

    rv = MAC_PAUSE_ADDR_SET(PORT(unit).pc_mac[port], unit, port, mac);
    return(rv);
    
}

/*
 * Function: bcm5600_port_advert_get
 * Purpose:  
 *	Retrieve the local port advertisment for autonegotiation.
 * Parameters:
 *	unit - StrataSwitch Unit #.
 *	port - StrataSwitch port #.
 *	ability_mask - (OUT) Local advertisment.
 * Returns:
 *	BCM_E_NONE
 *	BCM_E_XXX
 */
int
bcm5600_port_advert_get(int unit, soc_port_t port, port_mode_t *ability_mask)
{
#if 0
    return(PHY_ADV_GET(PORT(unit).pc_phy[port], unit, port, ability_mask));
#endif
	return(BCM_E_NONE);
}

/*
 * Function: bcm5600_port_advert_set
 * Purpose:  
 *	Set the local port advertisment for autonegotiation.
 * Parameters:
 *	unit - StrataSwitch Unit #.
 *	port - StrataSwitch port #.
 *	ability_mask - Local advertisment.
 * Returns:
 *	BCM_E_NONE
 *	BCM_E_XXX
 * Notes:
 *	Does NOT restart autonegotiation.
 */
int
bcm5600_port_advert_set(int unit, soc_port_t port, port_mode_t ability_mask)
{
#if 0
    int		rv;
    rv = PHY_ADV_SET(PORT(unit).pc_phy[port], unit, port, ability_mask);
    return(rv);
#endif
	return 0;
}

/*
 * Function: bcm5600_port_remote_advert_get
 * Purpose:  
 *	Retrieve the local port advertisment for autonegotiation.
 * Parameters:
 *	unit - StrataSwitch Unit #.
 *	port - StrataSwitch port #.
 *	ability_mask - (OUT) Remote advertisement.
 * Returns:
 *	BCM_E_NONE
 *	BCM_E_XXX
 */
int
bcm5600_port_remote_advert_get(int unit, soc_port_t port, 
							   port_mode_t *ability_mask)
{
#if 0
    int			phy_autoneg, phy_busy;

    SOC_IF_ERROR_RETURN(PHY_AUTO_NEGOTIATE_GET(PORT(unit).pc_phy[port], unit, 
					       port, &phy_autoneg, 
					       &phy_busy));
    if (!phy_autoneg) {
	return(BCM_E_AUTONEG_DISABLED);
    } else if (phy_busy) {
	return(BCM_E_AUTONEG_BUSY);
    }

    return(PHY_ADV_REMOTE_GET(PORT(unit).pc_phy[port], unit, 
			      port, ability_mask));
#endif
	return 0;
}

/*
 * Function: bcm5600_port_ability_get
 * Purpose:  
 *	Retrieve the local port abilities.
 * Parameters:
 *	unit - StrataSwitch Unit #.
 *	port - StrataSwitch port #.
 *	ability_mask - (OUT) Mask of BCM_PORT_ABIL_ values indicating the 
 *		ability of the MAC/PHY.
 * Returns:
 *	BCM_E_NONE
 *	BCM_E_XXX
 */
int
bcm5600_port_ability_get(int unit, soc_port_t port, port_mode_t *ability_mask)
{
#if 0
    port_mode_t		mac_ability, phy_ability;

    SOC_IF_ERROR_RETURN(PHY_ABILITY_GET(PORT(unit).pc_phy[port], unit, port, 
					&phy_ability));
    SOC_IF_ERROR_RETURN(MAC_ABILITY_GET(PORT(unit).pc_mac[port], unit, port, 
					&mac_ability));

    *ability_mask  = mac_ability & phy_ability;
    *ability_mask |= mac_ability & SOC_PM_ABILITY_MAC;
    *ability_mask |= phy_ability & SOC_PM_ABILITY_PHY;
#endif
    return(BCM_E_NONE);
}    

#if 0
/*
 * Function: _bcm_port_update
 * Purpose:  
 *	Get port characteristics from PHY and program MAC to match.  If 
 *	Autoforwarding is enabled, the port is put into forwarding state
 *	when the link is detected. When link is lost, Port is put into 
 *	disabled state.
 * Parameters:
 *	unit - StrataSwitch Unit #.
 *	port - StrataSwitch port #.
 *      link - (OUT) true if link is active, false if link is inactive.
 *		(May be NULL, in which case it is ignored).
 * Returns:
 *	BCM_E_NONE
 *	BCM_E_XXX
 */
static int
_bcm_port_update(int unit, soc_port_t port, int *link)
{
    int		rv;
    int		an, an_done, l_link;	/* local AN/link value */
    int		duplex, speed;

    *link = FALSE;			/* Starting assumption */

    SOC_IF_ERROR_RETURN(PHY_LINK_GET(PORT(unit).pc_phy[port], unit, port, 
				     &l_link));
    SOC_IF_ERROR_RETURN(PHY_AUTO_NEGOTIATE_GET(PORT(unit).pc_phy[port], 
					       unit, port, &an, &an_done));
    if (l_link == 0 || (an && !an_done)) { /* No link */
        /* Callback to phy if function present */
        if (PORT(unit).pc_phy[port]->pd_linkdn_evt) {
            SOC_IF_ERROR_RETURN(PORT(unit).pc_phy[port]->
                                pd_linkdn_evt(unit, port));
        }
		if (PORT(unit).pc_autoforward & PBMP_PORT(port)) {
	    /*
	     * Only set the port to blocking state if autoforward is enabled
	     * for this port. 
	     */
	    	SOC_IF_ERROR_RETURN(_bcm_port_stp_set(unit, port, 
						  		BCM_PORT_STP_DISABLED));
		}
		return(an ? BCM_E_AUTONEG_BUSY : BCM_E_NONE);
    } 
	else {   /* Link up.  Phy callback if present */
        if (PORT(unit).pc_phy[port]->pd_linkup_evt) {
            SOC_IF_ERROR_RETURN(PORT(unit).pc_phy[port]->
                                pd_linkup_evt(unit, port));
        }
    }
    SOC_IF_ERROR_RETURN(PHY_DUPLEX_GET(PORT(unit).pc_phy[port], unit, 
				       port, &duplex));
    SOC_IF_ERROR_RETURN(PHY_SPEED_GET(PORT(unit).pc_phy[port], unit, 
				      port, &speed));
    SOC_IF_ERROR_RETURN(MAC_SPEED_SET(PORT(unit).pc_mac[port], unit, 
				      port, speed));
    SOC_IF_ERROR_RETURN(MAC_DUPLEX_SET(PORT(unit).pc_mac[port], unit, 
				       port, duplex));
    /*
     * If Autonegotiating, check the negotiated PAUSE values, and program
     * MACs accordingly.
     */
    if (an) {
	port_mode_t	r_advert, l_advert;
	int		tx_pause, rx_pause;

	SOC_IF_ERROR_RETURN(PHY_ADV_GET(PORT(unit).pc_phy[port], 
					unit, port, &l_advert));
	SOC_IF_ERROR_RETURN(PHY_ADV_REMOTE_GET(PORT(unit).pc_phy[port], 
					       unit, port, &r_advert));

	tx_pause = (l_advert & SOC_PM_PAUSE_TX) && 
	           (r_advert & SOC_PM_PAUSE_RX);
	rx_pause = (l_advert & SOC_PM_PAUSE_RX) && 
	           (r_advert & SOC_PM_PAUSE_TX);
	SOC_IF_ERROR_RETURN(MAC_PAUSE_SET(PORT(unit).pc_mac[port], 
					  unit, port, tx_pause, rx_pause));
    }
    /* Link is active, check if auto-forwarding enabled */

    if (PORT(unit).pc_autoforward & PBMP_PORT(port)) {
		rv = _bcm_port_stp_set(unit, port, BCM_PORT_STP_FORWARDING);
    } 
	else {
		rv = BCM_E_NONE;
    }
    if (link) *link = l_link;

    return(rv);
}

/*
 * Function: bcm5600_port_update
 * Purpose:  
 *	Get port characteristics from PHY and program MAC to match.  If 
 *	Autoforwarding is enabled, the port is put into forwarding state
 *	when the link is detected.
 * Parameters:
 *	unit - StrataSwitch Unit #.
 *	port - StrataSwitch port #.
 *      link - (OUT) true if link is active, false if link is inactive.
 *		(May be NULL, in which case it is ignored).
 * Returns:
 *	BCM_E_NONE
 *	BCM_E_XXX
 */
int
bcm5600_port_update(int unit, soc_port_t port, int *link)
{
    int		rv;

    rv = _bcm_port_update(unit, port, link);
    return(rv);
}
#endif

#if 0
/*
 * Function: 
 *	bcm5600_port_discard_get
 * Purpose:  
 *	Get port discard attributes for the specified port
 * Parameters:
 *	unit - StrataSwitch Unit #.
 *	port - StrataSwitch port #.
 *      mode - (OUT) Port discard mode, one of BCM_PORT_DISCARD_xxx
 * Returns:
 *	BCM_E_NONE
 *	BCM_E_XXX
 */
int
bcm5600_port_discard_get(int unit, soc_port_t port, int *mode)
{
    ptable_entry_t	pv;
    int			rv;

    if ((rv = _port_ptable_get(unit, port, &pv)) == BCM_E_NONE) {
	switch(pv.prt_dis) {
	case 0x00:	*mode = BCM_PORT_DISCARD_NONE;	break;
	case 0x10:	*mode = BCM_PORT_DISCARD_ALL;	break;
	case 0x20:	*mode = BCM_PORT_DISCARD_UNTAG;	break;
	default:	*mode = -1;
	}
    }

    return(rv);
}

/*
 * Function: 
 *	bcm5600_port_discard_set
 * Purpose: 
 *	Set port discard attributes for the specified port.
 * Parameters:
 *	unit - StrataSwitch Unit #.
 *	port - StrataSwitch port #.
 *      mode - Port discard mode, one of BCM_PORT_DISCARD_xxx
 * Returns:
 *	BCM_E_NONE
 *	BCM_E_XXX
 */
int
bcm5600_port_discard_set(int unit, soc_port_t port, int mode)
{
    ptable_entry_t	pv;
    int			rv;

    if ((rv = _port_ptable_get(unit, port, &pv)) == BCM_E_NONE) {
	switch(mode) {
	case BCM_PORT_DISCARD_NONE:	pv.prt_dis = 0x00;	break;
	case BCM_PORT_DISCARD_ALL:	pv.prt_dis = 0x10;	break;
	case BCM_PORT_DISCARD_UNTAG:	pv.prt_dis = 0x20;	break;
	default:
	    rv = BCM_E_PARAM;
	}
	if (rv == BCM_E_NONE) {
	    rv = _port_ptable_set(unit, port, &pv);
	}
    }

    return(rv);
}
#endif

/*
 * Function: bcm5600_port_learn_modify
 * Purpose:  Modify the port learn flags, adding add and removing
 *           remove flags.
 *
 * Parameters:
 *	unit - StrataSwitch Unit #.
 *	port - StrataSwitch port #.
 *	add  - Flags to set.
 *      remove - Flags to clear.
 * Returns:
 *	BCM_E_NONE
 *	BCM_E_XXX
 */
int
bcm5600_port_learn_modify(int unit, soc_port_t port, uint32 add, uint32 remove)
{
    uint32 flags;

    SOC_IF_ERROR_RETURN(bcm5600_port_learn_get(unit, port, &flags));
    flags |= add;
    flags &= ~remove;
    SOC_IF_ERROR_RETURN(bcm5600_port_learn_set(unit, port, flags));

    return BCM_E_NONE;
}


/*
 * Function: bcm5600_port_learn_get
 * Purpose:  Get the ARL hardware learning options for this port.
 *		This defines what the hardware will do when a packet
 *		is seen with an unknown address.
 * Parameters:
 *	unit - StrataSwitch Unit #.
 *	port - StrataSwitch port #.
 *	flags - (OUT) Logical OR of BCM_PORT_LEARN_xxx flags
 * Returns:
 *	BCM_E_NONE
 *	BCM_E_XXX
 */
int
bcm5600_port_learn_get(int unit, soc_port_t port, uint32 *flags)
{
    ptable_entry_t	pv;
    int			rv;

    if ((rv = _port_ptable_get(unit, port, &pv)) == BCM_E_NONE) {

	switch (pv.cml) {
	case PVP_CML_SWITCH:
	    *flags = (BCM_PORT_LEARN_ON |
		      BCM_PORT_LEARN_SWITCH |
		      (pv.cpu ? BCM_PORT_LEARN_CPU : 0));
	    break;
	case PVP_CML_CPU:
	    *flags = BCM_PORT_LEARN_CPU;
	    break;
	case PVP_CML_FORWARD:
	    *flags = BCM_PORT_LEARN_SWITCH;
	    break;
	case PVP_CML_DROP:
	    *flags = 0;
	    break;
	default:
	    break;
	}

    }

    return(rv);
}

/*
 * Function: bcm5600_port_learn_set
 * Purpose:  Set the ARL hardware learning options for this port.
 *		This defines what the hardware will do when a packet
 *		is seen with an unknown address.
 * Parameters:
 *	unit - StrataSwitch Unit #.
 *	port - StrataSwitch port #.
 *	flags - Logical OR of BCM_PORT_LEARN_xxx flags
 * Returns:
 *	BCM_E_NONE
 *	BCM_E_XXX
 */
int
bcm5600_port_learn_set(int unit, soc_port_t port, uint32 flags)
{
    ptable_entry_t	pv;
    int 		rv = BCM_E_NOT_IMPL;

    if ((rv = _port_ptable_get(unit, port, &pv)) >= 0) {

	pv.cpu = ((flags & BCM_PORT_LEARN_CPU) != 0);

	/* Use shortened names to handle each flag combination individually */

#define Lrn	BCM_PORT_LEARN_ON
#define Cpu	BCM_PORT_LEARN_CPU
#define Swc	BCM_PORT_LEARN_SWITCH

	switch (flags) {
	case ((!Lrn) | (!Cpu) | (!Swc)):
	    pv.cml = PVP_CML_DROP;
	    break;
	case ((!Lrn) | (!Cpu) | ( Swc)):
	    pv.cml = PVP_CML_FORWARD;
	    break;
	case ((!Lrn) | ( Cpu) | (!Swc)):
	    pv.cml = PVP_CML_CPU;
	    break;
	case ((!Lrn) | ( Cpu) | ( Swc)):
	    goto done;			/* Not implementable */
	case (( Lrn) | (!Cpu) | (!Swc)):
	    goto done;			/* Not implementable */
	case (( Lrn) | (!Cpu) | ( Swc)):
	    pv.cml = PVP_CML_SWITCH;	/* clearn clear */
	    break;
	case (( Lrn) | ( Cpu) | (!Swc)):
	    goto done;			/* Not implementable */
	case (( Lrn) | ( Cpu) | ( Swc)):
	    pv.cml = PVP_CML_SWITCH;	/* clearn set */
	    break;
	}

#undef Lrn
#undef Cpu
#undef Swc

	rv =  _port_ptable_set(unit, port, &pv);
    }

done:
    return(rv);
}

#if 0
/*
 * Function: bcm5600_port_ifilter_get
 *
 * Description:
 *	Return input filter mode for a port.
 *
 * Parameters:
 *      unit - StrataSwitch PCI device unit number (driver internal).
 *	port - Port number to operate on
 *	mode - (OUT) Filter mode, one of BCM_PORT_IFILTER_xxx
 *
 * Returns:
 *	BCM_E_NONE		Success.
 *	BCM_E_INTERNAL		Chip access failure.
 *
 * Notes:
 *	See also: bcm_vlan_ifilter_get
 */
int
bcm5600_port_ifilter_get(int unit, soc_port_t port, int *mode)
{
    uint32		pbmp;

    if (READ_EN_IFILTERr(unit, 0, &pbmp) < 0)
	return(BCM_E_INTERNAL);

    if (PBMP_MEMBER(pbmp, port))
	*mode = BCM_PORT_IFILTER_ON;
    else
	*mode = BCM_PORT_IFILTER_OFF;

    return(BCM_E_NONE);
}

/*
 * Function: bcm5600_port_ifilter_set
 *
 * Description:
 *	Set input filter mode for a port.
 *
 * Parameters:
 *	unit - StrataSwitch PCI device unit number (driver internal).
 *	port - Port number to operate on
 *	mode - BCM_PORT_IFILTER_ON or BCM_PORT_IFILTER_OFF.
 *
 * Returns:
 *	BCM_E_NONE		Success.
 *	BCM_E_INTERNAL		Chip access failure.
 *
 * Notes:
 *	When input filtering is turned on for a port, packets received
 *	on the port that do not match the port's VLAN classifications
 *	are discarded.
 *	See also: bcm_vlan_ifilter_set
 */
int
bcm5600_port_ifilter_set(int unit, soc_port_t port, int mode)
{
    uint32		pbmp;
    int			rv = BCM_E_NONE;

    if ((rv = READ_EN_IFILTERr(unit, 0, &pbmp)) >= 0) {
	if (mode) {
	    pbmp |= PBMP_PORT(port);
	} else {
	    pbmp &= ~PBMP_PORT(port);
	}
	rv = cmic_write_reg_blocks(unit, EN_IFILTERr, ANY_PIC, pbmp);
    }
    
    return(rv);
}
#endif

/*
 * Function:
 *	bcm5600_port_link_status
 * Purpose:
 *	Return current link up/down status
 * Parameters:
 *	unit - StrataSwitch Unit #.
 *	port - StrataSwitch port #.
 *	up - (OUT) Boolean value, FALSE for link down and TRUE for link up
 * Returns:
 *	BCM_E_NONE
 *	BCM_E_XXX
 */
int
bcm5600_port_link_status(int unit, soc_port_t port, int *up)
{
#if 0
    return(PHY_LINK_GET(PORT(unit).pc_phy[port], unit, port, up));
#endif
	return(BCM_E_NONE);
}

#if 0
/*
 * Function:
 *	bcm5600_port_pfm_set
 * Purpose:
 *	Set current port filtering mode (see port.h)
 * Parameters:
 *	unit - StrataSwitch Unit #.
 *	port - StrataSwitch port #.
 *      mode - mode for PFM bits (see port.h)
 *
 * Returns:
 *	BCM_E_NONE
 *	BCM_E_XXX
 */
int
bcm5600_port_pfm_set(int unit, soc_port_t port, int mode)
{
    uint32 	pfm;
    int		rv = BCM_E_NONE;

    if (! PBMP_MEMBER(PBMP_E_ALL, port))
	return (BCM_E_PORT);

    /*
     * The filter mode for the first 16 ports is in one register
     * and for the rest in another
     */
    if (port < 16) {
	if ((rv = READ_EPC_PFM0r(unit, 0, &pfm)) >= 0) {

	    /* Clear the correct bits */

	    pfm &= ~(3 << (port * 2));
	    pfm |= (mode & 3) << (port * 2);

	    rv = cmic_write_reg_blocks(unit, EPC_PFM0r, ANY_PIC, pfm);
	}
    } else {
	if ((rv = READ_EPC_PFM1r(unit, 0, &pfm)) >= 0) {
	    
	    /* Clear the correct bits */

	    pfm &= ~(3 << ((port - 16) * 2));
	    pfm |= (mode & 3) << ((port - 16) * 2);

	    rv = cmic_write_reg_blocks(unit, EPC_PFM1r, ANY_PIC, pfm);
	}
    }
    return(rv);
}

/*
 * Function:
 *	bcm5600_port_pfm_get
 * Purpose:
 *	Return current port filtering mode (see port.h)
 * Parameters:
 *	unit - StrataSwitch Unit #.
 *	port - StrataSwitch port #.
 *      mode - (OUT) mode read from StrataSwitch for PFM bits (see port.h)
 *
 * Returns:
 *	BCM_E_NONE
 *	BCM_E_XXX
 */
int
bcm5600_port_pfm_get(int unit, soc_port_t port, int *mode)
{
    uint32 pfm;

    if (! PBMP_MEMBER(PBMP_E_ALL, port))
	return(BCM_E_PORT);

    if (port < 16) {
	SOC_IF_ERROR_RETURN(READ_EPC_PFM0r(unit, 0, &pfm));
	*mode = (pfm >> (port * 2)) & 3;
    } else {
	SOC_IF_ERROR_RETURN(READ_EPC_PFM1r(unit, 0, &pfm));
	*mode = (pfm >> ((port - 16) * 2)) & 3;
    }

    return(BCM_E_NONE);
}
#endif

/*
 * Function:
 *	bcm5600_port_info_get
 *
 * Purpose:
 *	Get all available port parameters at once.
 *
 * Parameters:
 *	unit - StrataSwitch Unit #.
 *	port - StrataSwitch port #.
 *      info - (OUT) Structure to pass the information (see port.h)
 *
 * Returns:
 *	BCM_E_NONE
 *	BCM_E_XXX
 *
 * Notes:
 *	None.
 */
int
bcm5600_port_info_get(int unit, soc_port_t port, bcm_port_info_t *info)
{
    int			r;

    if ((r = bcm5600_port_link_status(unit, port, &info->linkstatus)) < 0) {
		printf("bcm5600_port_link_status failed\n");
		return(r);
    }

    if ((r = bcm5600_port_autoneg_get(unit, port, &info->autoneg)) < 0) {
		printf("bcm5600_port_autoneg_get failed\n");
		return(r);
    }

    if ((r = bcm5600_port_autoforward_get(unit, port, &info->autoforward)) < 0){
		printf("bcm5600_port_autoforward_get failed\n");
		return(r);
    }

    if ((r = bcm5600_port_speed_get(unit, port, &info->speed)) < 0) {
		if (BCM_E_AUTONEG_BUSY != r) {
	    	printf("bcm5600_port_speed_get failed\n");
	    	return(r);
		} else {
	    	info->speed = 0;
		}
    }

    if ((r = bcm5600_port_duplex_get(unit, port, &info->duplex)) < 0) {
		if (BCM_E_AUTONEG_BUSY != r) {
	    	printf("bcm5600_port_duplex_set failed\n");
	    	return(r);
		} else {
	    	info->duplex = 0;
		}
    }
    if ((r = bcm5600_port_pause_get(unit, port, &info->pause_tx, 
				&info->pause_rx)) < 0) {
		printf("bcm5600_port_pause_get failed\n");
		return(r);
    }

    if ((r = bcm5600_port_learn_get(unit, port, &info->learn)) < 0) {
		printf("bcm5600_port_learn_get failed\n");
		return(r);
    }
#if 0
    if ((r = bcm5600_port_discard_get(unit, port, &info->discard)) < 0) {
		printf("bcm5600_port_discard_get failed\n");
		return(r);
    }

    if ((r = bcm5600_port_ifilter_get(unit, port, &info->vlanfilter)) < 0) {
		printf("bcm5600_port_ifilter_get failed\n");
		return(r);
    }
#endif
    if ((r = bcm5600_port_stp_get(unit, port, &info->stp_state)) < 0) {
		printf("bcm5600_port_stp_get failed\n");
		return(r);
    }
#if 0
    if ((r = bcm5600_port_pfm_get(unit, port, &info->pfm)) < 0) {
		printf("bcm5600_port_pfm_get failed\n");
		return(r);
    }
#endif
    if ((r = bcm5600_port_loopback_get(unit, port, &info->loopback)) < 0) {
		printf("bcm5600_port_loopback_get failed\n");
		return(r);
    }
    return BCM_E_NONE;
}

/*
 * Function:
 *	bcm5600_port_info_set
 *
 * Purpose:
 *	Set all available port parameters at once.
 *
 * Parameters:
 *	unit - StrataSwitch Unit #.
 *	port - StrataSwitch port #.
 *      info - Structure to pass the information (see port.h)
 *
 * Returns:
 *	BCM_E_NONE
 *	BCM_E_XXX
 *
 * Notes:
 *	Does not set linkstatus.
 */

int
bcm5600_port_info_set(int unit, soc_port_t port, bcm_port_info_t *info)
{
    int			r;

    if ((r = bcm5600_port_pause_addr_set(unit, port, info->pause_mac)) < 0) {
	printf("bcm5600_port_pause_addr_set failed: \n");
	return(r);
    }

    if ((r = bcm5600_port_autoforward_set(unit, port, info->autoforward)) < 0) {
	printf("bcm5600_port_autoforward_set failed: \n");
	return(r);
    }
#if 0
    if ((r = bcm5600_linkscan_port_set(unit, port, info->linkscan)) < 0) {
    printf("bcm5600_linkscan_port_set failed: \n");
    return(r);
    }
#endif
    if ((r = bcm5600_port_learn_set(unit, port, info->learn)) < 0) {
	printf("bcm5600_port_learn_set failed: \n");
	return(r);
    }
#if 0
    if ((r = bcm5600_port_discard_set(unit, port, info->discard)) < 0) {
	printf("bcm5600_port_discard_set failed: \n");
	return(r);
    }

    if ((r = bcm5600_port_ifilter_set(unit, port, info->vlanfilter)) < 0) {
	printf("bcm5600_port_ifilter_set failed: \n");
	return(r);
    }
#endif
    if ((r = bcm5600_port_stp_set(unit, port, info->stp_state)) < 0) {
	printf("bcm5600_port_stp_set failed: \n");
	return(r);
    }
#if 0
    if ((r = bcm5600_port_pfm_set(unit, port, info->pfm)) < 0) {
	printf("bcm5600_port_pfm_set failed: \n");
	return(r);
    }
#endif
    if ((r = bcm5600_port_loopback_set(unit, port, info->loopback)) < 0) {
	printf("bcm5600_port_loopback_set failed: \n"); 
	return(r);
    }
    if ((r = bcm5600_port_autoneg_set(unit, port, FALSE)) < 0) {
	printf("bcm5600_port_autoneg_set failed: \n");
	return(r);
    }
    if (info->autoneg) {
	port_mode_t	mode;
	/*
	 * Set link sdvertisments from speed/duplex/pause.
	 */
	switch(info->speed) {
	case 0:				/* Set to highest value */
	    BCM_IF_ERROR_RETURN(bcm5600_port_ability_get(unit,  port, &mode));
	    mode &= BCM_PORT_ABIL_10MB_HD | BCM_PORT_ABIL_10MB_FD | 
		    BCM_PORT_ABIL_100MB_HD | BCM_PORT_ABIL_100MB_FD | 
		    BCM_PORT_ABIL_1000MB_HD | BCM_PORT_ABIL_1000MB_FD;
	    break;
	case 10:
	    if (info->duplex) {
		mode = BCM_PORT_ABIL_10MB_FD | BCM_PORT_ABIL_10MB_HD;
	    } else {
		mode = BCM_PORT_ABIL_10MB_HD;
	    }
	    break;
	case 100:
	    if (info->duplex) {
		mode = BCM_PORT_ABIL_100MB_FD | BCM_PORT_ABIL_100MB_HD;
	    } else {
		mode = BCM_PORT_ABIL_100MB_HD;
	    }
	    mode |= BCM_PORT_ABIL_10MB_HD | BCM_PORT_ABIL_10MB_FD;
	    break;
	case 1000:
	    if (info->duplex) {
		mode = BCM_PORT_ABIL_1000MB_FD | BCM_PORT_ABIL_1000MB_HD;
	    } else {
		mode = BCM_PORT_ABIL_1000MB_HD;
	    }
	    mode |= BCM_PORT_ABIL_10MB_HD | BCM_PORT_ABIL_10MB_FD;
	    mode |= BCM_PORT_ABIL_100MB_HD | BCM_PORT_ABIL_100MB_FD;
	    break;
	default:
	    printf("bcm5600_port_info_set: Invalid speed: %d\n",
		   info->speed);
	    return(BCM_E_PARAM);
	}
	if (info->pause_tx) mode |= BCM_PORT_ABIL_PAUSE_TX;
	if (info->pause_rx) mode |= BCM_PORT_ABIL_PAUSE_RX;

	if ((r = bcm5600_port_advert_set(unit, port, mode)) < 0) {
	    printf("bcm5600_port_advert_set failed: (0x%x): \n", mode);
	    return(r);
	}
	if ((r = bcm5600_port_autoneg_set(unit, port, info->autoneg)) < 0) {
	    printf("bcm5600_port_autoneg_set failed: \n");
	    return(r);
	}

    } else {
	/*
	 * If auto-negotiation not set, force the speeds.
	 */
	if ((r = bcm5600_port_speed_set(unit, port, info->speed)) < 0) {
	    printf("bcm5600_port_speed_set failed: \n");
	    return(r);
	}

	if ((r = bcm5600_port_duplex_set(unit, port, info->duplex)) < 0) {
	    printf("bcm5600_port_duplex_set failed: \n");
	    return(r);
	}

	if ((r = bcm5600_port_pause_set(unit, port, info->pause_tx, 
				    info->pause_rx)) < 0) {
	    printf("bcm5600_port_pause_set failed: \n");
	    return(r);
	}
    }

    return(BCM_E_NONE);
}

#if 0
/*
 * Function:
 *	bcm_ptable_update_vlan_destroy
 * Purpose:
 *      VLAN has been remove from system.  Update ptable.
 *      Set pvlan to "default vlan" for any ports that have
 *      the destroyed VID as their default (ptable) PVID.
 * Parameters:
 *	unit - StrataSwitch unit number.
 *	vid  - vlan destroyed
 * Returns:
 *	BCM_E_NONE
 *	BCM_E_INTERNAL - internal error.
 */
int
bcm_ptable_update_vlan_destroy(int unit, vlan_id_t vid)
{
    int rv;

    rv = _bcm_ptable_update_vlan_destroy(unit, vid);
    return rv;
}

/*
 * Function:
 *	_bcm_ptable_update_vlan_destroy
 *      Guts of bcm_ptable_update_vlan_destroy.  See above
 *
 *      Assumes locking at higher level.
 */
static int
_bcm_ptable_update_vlan_destroy(int unit, vlan_id_t vid)
{
    soc_port_t port;
    ptable_entry_t	pv;
    vlan_data_t vdata;

    bcm5600_vlan_default_get(unit, &vdata);
    PBMP_ITER(PBMP_ALL, port) {
        SOC_IF_ERROR_RETURN(_port_ptable_get(unit, port, &pv));
        if (pv.vlan_tag == vid) {
            pv.vlan_tag = vdata.vlan_tag;
            pv.port_bitmap = vdata.port_bitmap;
            pv.ut_port_bitmap = vdata.ut_port_bitmap;
            SOC_IF_ERROR_RETURN(_port_ptable_set(unit, port, &pv));
        }
    }

    return(SOC_E_NONE);
}

/*
 * Function:
 *	bcm_ptable_update_vlan_add
 * Purpose:
 *      Ports have been added to a VLAN.  Update the PTABLE with the
 *      new information for all ports.
 * Parameters:
 *	unit - StrataSwitch unit number.
 *	port - StrataSwitch port number.
 *	vid  - The VLAN id being changed.
 *      pbmp, ubmp  - bit maps of ports effected.
 *
 * Returns:
 *	BCM_E_NONE
 *	BCM_E_INTERNAL - internal error.
 */
int
bcm_ptable_update_vlan_add(int unit, vlan_id_t vid,
                           pbmp_t pbmp, pbmp_t ubmp)
{
    int rv;

    rv = _bcm_ptable_update_vlan_add(unit, vid, pbmp, ubmp);
    return rv;
}

/*
 * Function:
 *	_bcm_ptable_update_vlan_add
 *      Guts of bcm_ptable_update_vlan_add.  See above.
 *
 *      Assumes locking at higher level.
 *
 */
static int
_bcm_ptable_update_vlan_add(int unit, vlan_id_t vid,
                           pbmp_t pbmp, pbmp_t ubmp)
{
    soc_port_t port;
    ptable_entry_t	pv;

    PBMP_ITER(PBMP_ALL, port) {
        SOC_IF_ERROR_RETURN(_port_ptable_get(unit, port, &pv));
        if (pv.vlan_tag == vid) {
            pv.port_bitmap |= pbmp;
            pv.ut_port_bitmap &= ~pbmp;
            pv.ut_port_bitmap |= ubmp;
            SOC_IF_ERROR_RETURN(_port_ptable_set(unit, port, &pv));
        }
    }

    return(SOC_E_NONE);
}
#endif

#if 0
/*
 * Function:
 *	bcm_ptable_update_vlan_remove
 * Purpose:
 *      Ports have been removed from a VLAN.  Update the PTABLE with the
 *      new information for all ports.
 * Parameters:
 *	unit - StrataSwitch unit number.
 *	port - StrataSwitch port number.
 *	vid  - The VLAN id being changed.
 *      pbmp, ubmp  - bit maps of ports effected.
 *
 * Returns:
 *	BCM_E_NONE
 *	BCM_E_INTERNAL - internal error.
 */
int
bcm_ptable_update_vlan_remove(int unit, vlan_id_t vid, pbmp_t pbmp)
{
    int rv;
    rv = _bcm_ptable_update_vlan_remove(unit, vid, pbmp);
    return rv;
}

/*
 * Function:
 *	_bcm_ptable_update_vlan_remove
 *      Guts of bcm_ptable_update_vlan_remove.  See above.
 *
 *      Assumes locking at higher level.
 */
static int
_bcm_ptable_update_vlan_remove(int unit, vlan_id_t vid, pbmp_t pbmp)
{
    soc_port_t port;
    ptable_entry_t	pv;
    PBMP_ITER(PBMP_ALL, port) {
        SOC_IF_ERROR_RETURN(_port_ptable_get(unit, port, &pv));
        if (pv.vlan_tag == vid) {
            pv.port_bitmap &= ~pbmp;
            pv.ut_port_bitmap &= ~pbmp;
            SOC_IF_ERROR_RETURN(_port_ptable_set(unit, port, &pv));
        }
    }

    return(SOC_E_NONE);
}

char *
bcm_port_get_phy_drv_name(int unit, soc_port_t port)
{
        if (PORT(unit).pc_phy[port]->drv_name) {
            return(PORT(unit).pc_phy[port]->drv_name);
        } else {
            return "unknown driver";
        }
}
#endif

/* TODO implement me properly */

int 
bcm5600_port_reset(int unit, soc_port_t port)
{
  return(MAC_INIT(PORT(unit).pc_mac[port], unit, port));
}
