/*
 *=============================================================================
 * File Name   :    bcm5600-schanmsg.h
 *
 *  RCS: $Id: bcm5600-schanmsg.h,v 1.1.1.1 2001/12/20 04:53:01 tonyw Exp $
 *
 *  Description:    Copied and modified from Broadcom
 *
 *  Created    :    09-04-2001
 *
 *  Author     :    Tony Wong
 *
 *  Copyright(c)  Agile Storage, Inc. 2001
 *=============================================================================
 */

#ifndef _BCM5600_SCHANMSG_H
#define _BCM5600_SCHANMSG_H

#define BCM5600_LE_HOST
#define CMIC_SCHAN_WORDS	20	/* Size of CMIC message buffer */

/*
 * The endianness of the host is taken into account by the routines
 * which copy S-Channel messages into and out of PCI space as 32-bit
 * words.  Unfortunately, the compiler also switches bit field packing
 * order according to host endianness.  We must undo this "feature" by
 * giving the fields in both orders.
 */

typedef struct schan_header_t {
#if defined(BCM5600_LE_HOST)
    uint32 cpu:1,		/* Bit 0 */
	   cos:3,
	   ecode: 2,
	   ebit:1,
	   datalen:7,
	   srcblk:6,		/* See socregs.h: xxx_SCH_BLK_NUM */
	   dstblk:6,
	   opcode:6;
#else
    uint32 opcode:6,		/* Bits 31:26 */
	   dstblk:6,
	   srcblk:6,
	   datalen:7,
	   ebit:1,
	   ecode: 2,
	   cos:3,
	   cpu:1;
#endif
} schan_header_t;

/*
 * Individual SChannel message formats.
 * Different ways of peeking and poking at an SChannel message
 * packet.  Applicable message types are listed inside each structure.
 */

typedef struct schan_msg_plain_t {
    /* GBP Full Notification */
    /* GBP Available Notification */
    /* Write Memory Ack */
    /* Write Register Ack */
    /* ARL Insert Complete */
    /* ARL Delete Complete */
    /* Memory Failed Notification */
    /* Initialize CFAP (Cell FAP) */
    /* Initialize SFAP (Slot FAP) */
    /* Enter Debug Mode */
    /* Exit Debug Mode */
    schan_header_t header;
} schan_msg_plain_t;

typedef struct schan_msg_bitmap_t {
    /* Back Pressure Warning Status */
    /* Back Pressure Discard Status */
    /* Link Status Notification */
    /* COS Queue Status Notification */
    /* HOL Status Notification */
    schan_header_t header;
    uint32 bitmap;
} schan_msg_bitmap_t;

typedef struct schan_msg_readcmd_t {
    /* Read Memory Command */
    /* Read Register Command */
    schan_header_t header;
    uint32 address;
} schan_msg_readcmd_t;

typedef struct schan_msg_readresp_t {
    /* Read Memory Ack */
    /* Read Register Ack */
    schan_header_t header;
    uint32 data[CMIC_SCHAN_WORDS - 1];
} schan_msg_readresp_t;

typedef struct schan_msg_writecmd_t {
    /* Write Memory Command */
    /* Write Register Command */
    schan_header_t header;
    uint32 address;
    uint32 data[CMIC_SCHAN_WORDS - 2];
} schan_msg_writecmd_t;

typedef struct schan_msg_arlins_t {
    /* ARL Insert Command */
    /* (Also: ARL Message Buffer Format) */
    /* (Also: ARL DMA Message Format) */
    schan_header_t header;
    uint32 data[3];
} schan_msg_arlins_t;

typedef struct schan_msg_arldel_t {
    /* ARL Delete Command */
    schan_header_t header;
    uint32 data[2];
} schan_msg_arldel_t;

typedef struct schan_msg_arllkup_t {
    /* ARL Delete Command */
    schan_header_t header;
    uint32 address;
    uint32 data[2];
} schan_msg_arllkup_t;

/*
 * Union of all SChannel message types (use to declare all message buffers)
 *
 * When building messages, address the union according to packet type.
 * When writing to PCI, address data.dwords.
 * When writing to I2C, address data.bytes.
 */

#define schan_msg_clear(m)	((m)->header_dword = 0)

typedef union schan_msg_t {
    schan_header_t header;
    uint32 header_dword;
    schan_msg_plain_t plain;
    schan_msg_bitmap_t bitmap;
    schan_msg_readcmd_t readcmd;
    schan_msg_readresp_t readresp;
    schan_msg_writecmd_t writecmd;
    schan_msg_arlins_t arlins;
    schan_msg_arldel_t arldel;
    schan_msg_arllkup_t arllkup;
    uint32 dwords[CMIC_SCHAN_WORDS];
    uint8 bytes[sizeof(uint32) * CMIC_SCHAN_WORDS];
} schan_msg_t;

/*
 * SChannel Message Types
 */

#define BP_WARN_STATUS_MSG	      0x01
#define BP_DISCARD_STATUS_MSG	      0x02
#define COS_QSTAT_NOTIFY_MSG	      0x03
#define HOL_STAT_NOTIFY_MSG	      0x04
#define GBP_FULL_NOTIFY_MSG	      0x05
#define GBP_AVAIL_NOTIFY_MSG	      0x06
#define READ_MEMORY_CMD_MSG	      0x07
#define READ_MEMORY_ACK_MSG	      0x08
#define WRITE_MEMORY_CMD_MSG	      0x09
#define WRITE_MEMORY_ACK_MSG	      0x0a
#define READ_REGISTER_CMD_MSG	      0x0b
#define READ_REGISTER_ACK_MSG	      0x0c
#define WRITE_REGISTER_CMD_MSG	      0x0d
#define WRITE_REGISTER_ACK_MSG	      0x0e
#define ARL_INSERT_CMD_MSG	      0x0f
#define ARL_INSERT_DONE_MSG	      0x10
#define ARL_DELETE_CMD_MSG	      0x11
#define ARL_DELETE_DONE_MSG	      0x12
#define LINKSTAT_NOTIFY_MSG	      0x13
#define MEMORY_FAIL_NOTIFY	      0x14
#define INIT_CFAP_MSG		      0x15
#define INIT_SFAP_MSG		      0x16
#define ENTER_DEBUG_MODE_MSG	      0x17
#define EXIT_DEBUG_MODE_MSG	      0x18
#define ARL_LOOKUP_CMD_MSG	      0x19

#endif /* BCM5600_SCHANMSG_H */
