/*
 *=============================================================================
 * File Name   :    bcm5632-diag.c
 *
 *  RCS: $Id: bcm5632-diag.c,v 1.12 2002/11/07 01:23:07 rick Exp $
 *
 *  Description:	Diagnostics menu
 *
 *  Created    :    09-04-2001
 *
 *  Author     :    Tony Wong
 *
 *  Copyright(c)  Agile Storage, Inc. 2001
 *=============================================================================
 */

#include <nfx-incl.h>

#include "bcm5632-soctypes.h"
#include "bcm5632-cmic.h"
#include "bcm5632-error.h"
#include "bcm5632-port.h"
#include "bcm5632-l2.h"

extern int get_cmd(char8 *);
extern void watchdog_disable(void);
extern void watchdog_enable(int32);
extern int bcm_init_5632(int unit);
extern uint32 reg_addr_5632(soc_reg_t reg, ...);
extern int cmic_read_miim(int unit, uint8 phy_id, uint8 phy_reg_addr, uint16 *phy_rd_data);
extern int cmic_write_miim(int unit, uint8 phy_id, uint8 phy_reg_addr, uint16 phy_wr_data);
extern int _BCM5632_L2_management(int unit, soc_mem_t mem, void *key_data, void *entry_data, int index, int cmd);
extern int _BCM5632_table_management(int unit, soc_mem_t mem, void *key_data, int index, int cmd);

extern uint32 BCM5632_DEV_ID[BCM5632_NUM_DEV];
uint32 mib_buffer_5632[NUM_GE_PORT_5632][50]; 

/* PHY IDs for each port */
uint32 phy_map_5632[12] = 
{ 0x00, 0x01, 0x02, 0x03, 0x06, 0x07, 0x04, 0x05, 0x1c, 0x1d, 0x1e, 0x1f };

struct {
   char desc[28];
   int  bank;
   int  index;
} mib_table_5632[] = {  /* MIB counter access table */

  { "Rx bytes (lower word)   : ",	5, 2 },
  { "Rx bytes (upper word)   : ",	5, 3 },
  { "Rx unicast frames       : ",	0, 0 },
  { "Rx broadcast frames     : ",	0, 1 },
  { "Rx multicast frames     : ",	0, 2 },
  { "Rx MAC ctrl frames      : ",	0, 3 },
  { "Rx FCS errors           : ",	0, 4 },
  { "Rx align errors         : ",	0, 5 },
  { "Rx symbol errors        : ",	0, 6 },
  { "Rx 64 byte frames       : ",	2, 0 },
  { "Rx 65-127 byte frames   : ",	2, 1 },
  { "Rx 128-255 byte frames  : ",	2, 2 },
  { "Rx 256-511 byte frames  : ",	2, 3 },
  { "Rx 512-1023 byte frames : ",	2, 4 },
  { "Rx 1024-max byte frames : ",	2, 5 },
  { "Rx max-4095 byte frames : ",	2, 6 },
  { "Rx 4096-9216 byte frames: ",	2, 7 },
  { "Rx undersize frames     : ",	2, 8 },
  { "Rx oversize frames      : ",	2, 9 },
  { "Rx fragments            : ",  	2, 10 },
  { "Rx jabbers              : ",  	2, 11 },

  { "Tx bytes (lower word)   : ",	5, 0 },
  { "Tx bytes (upper word)   : ",	5, 1 },
  { "Tx unicast frames       : ",	1, 0 },
  { "Tx broadcast frames     : ",	1, 1 },
  { "Tx multicast frames     : ",	1, 2 },
  { "Tx MAC ctrl frames      : ",	1, 3 },
  { "Tx 64 byte frames       : ",	3, 0 },
  { "Tx 65-127 byte frames   : ",	3, 1 },
  { "Tx 128-255 byte frames  : ",	3, 2 },
  { "Tx 256-511 byte frames  : ",	3, 3 },
  { "Tx 512-1023 byte frames : ",	3, 4 },
  { "Tx 1024-max byte frames : ",	3, 5 },
  { "Tx max-4095 byte frames : ",	3, 6 },
  { "Tx 4096-9216 byte frames: ",	3, 7 },
  { "Tx no collision frames  : ",	6, 0 },
  { "Tx 1 collision frames   : ",	6, 1 },
  { "Tx 2-15 collision frames: ",	6, 2 },
  { "Tx excess clsn frames   : ",	6, 3 },
  { "Tx late collision frames: ",	6, 4 },

  { "Forwarded packets       : ",	4, 0 },
  { "Flooded packets         : ",	4, 1 },
  { "Filtered packets        : ",	4, 2 },
  { "Buffer full drops       : ",	4, 3 },
  { "Security violation drops: ",	4, 4 },
  { "VLAN drops              : ",	4, 5 },
  { "Storm/queue limit drops : ",	4, 6 },
  { "Tx timeout drops        : ",  10, 42},
  { "Rx storm drops          : ",  10, 55},
  { "End of table", -1, -1 } /* must mark end of table */
};

uint32
get_mib_index_5632(uint32 port, uint32 bank, uint32 index)
{
   uint32 entry, vlan;

   entry = (bank << 8) | (1 << 16); /* read one entry at a time */
   switch (bank) {
      case 0: entry |= port << 3;
              entry |= index << 0;
              break;
      case 1: entry |= port << 2;
              entry |= index << 0;
              break;
      case 2: entry |= port << 0;
              entry |= index << 4;
              break;
      case 3: entry |= port << 0;
              entry |= index << 4;
              break;
      case 4: entry |= port << 3;
              entry |= index << 0;
              break;
      case 5: entry |= port << 2;
              entry |= index << 0;
              break;
      case 6: entry |= port << 0;
              entry |= index << 4;
              break;
      case 7: entry |= port << 3;
              entry |= index << 0;
              break;
      case 8: entry |= port << 4;
              entry |= index << 0;
              break;
      case 9: vlan = port;
              entry |= vlan << 2;
              entry |= index << 0;
              break;
      case 10:entry |= index << 0;
              break;
      default:break;
   }
   return (entry);
}

int
diag5632_uiGetInput( char *msg )
{
   int sel;
   char cmd[80];

   printf("%s: ", msg);
   memset(cmd, 0, 80);
   get_cmd(cmd);
   sscanf(cmd, "%d", &sel);
   return sel;
} /* diag5632_uiGetInput */

int
diag5632_uiGetHex( char *msg )
{
   uint32 sel;
   char cmd[80];

   printf("%s: ", msg);
   memset(cmd, 0, 80);
   get_cmd(cmd);
   sscanf(cmd, "%x", &sel);
   return sel;
} /* diag5632_uiGetHex */


void
diag5632_showPCI(uint32 unit)
{
#if 0
	uint32 reg_val;

	printf("PCI registers\n");
   	printf("=============\n");
   	pci_cfg_read(BCM5632_BUS_ID, BCM5632_DEV_ID[unit], 0x00, &reg_val);
   	printf("Vendor ID = %0x\n", reg_val);

   	pci_cfg_read(BCM5632_BUS_ID, BCM5632_DEV_ID[unit], 0x04, &reg_val);
   	printf("Command/Status = %0x\n", reg_val);

   	pci_cfg_read(BCM5632_BUS_ID, BCM5632_DEV_ID[unit], 0x08, &reg_val);
   	printf("Class Code/Revision ID = %0x\n", reg_val);

   	pci_cfg_read(BCM5632_BUS_ID, BCM5632_DEV_ID[unit], 0x0c, &reg_val);
   	printf("Header/Latency/Cacheline Size = %0x\n", reg_val);

   	pci_cfg_read(BCM5632_BUS_ID, BCM5632_DEV_ID[unit], 0x10, &reg_val);
   	printf("Base Addr = %0x\n", reg_val);
#else
	printf("Not supported on Toshiba!\n");
#endif
}

void
diag5632_writePCI(uint32 unit)
{
#if 0
	uint32 reg_addr;
	uint32 reg_val;

	reg_addr = diag5632_uiGetHex("Enter reg addr(hex): ");

	reg_val = diag5632_uiGetHex("Enter reg value(hex): ");

	pci_cfg_write(BCM5632_BUS_ID, BCM5632_DEV_ID[unit], reg_addr, reg_val);
#else
	printf("Not supported on Toshiba!\n");
#endif
}

void
diag5632_readReg(uint32 unit)
{
   uint32 reg_addr;
   uint32 reg_val;

   reg_addr = diag5632_uiGetHex("Enter reg addr(hex): ");

   cmic_read_reg_5632(unit, reg_addr, &reg_val);

   printf("Value at %0x: %0x\n", reg_addr, reg_val);
}

void
diag5632_writeReg(uint32 unit)
{
   uint32 reg_addr;
   uint32 reg_val;

   reg_addr = diag5632_uiGetHex("Enter reg addr(hex): ");

   reg_val = diag5632_uiGetHex("Enter reg value(hex): ");

   cmic_write_reg_5632(unit, reg_addr, reg_val);
}

void 
diag5632_viewStatus(uint32 unit)
{
	uint32 port, data; 
#if 0
	uint32 loopback;
#endif
	for (port = 0; port < NUM_GE_PORT_5632; port++)
	{
		printf("Port #%d status: ", port);
#if 0 	/* if MIIM through 5632 */
		phy_ge_lb_get5632(unit, port, &loopback);
#endif
		cmic_read_miim(unit, phy_map_5632[port], 0x00, (uint16 *)&data);
		if ((data >> 14) & 0x1)
			printf("PHY loopback\n");
		else
		{
			READ_P_STATUSr(unit, port, &data);
          	if (GET_P_STATUSr_LinkFail(data))
          	{
              	printf("Fail\n");
              	SET_P_STATUSr_LinkFail(data,0);
              	WRITE_P_STATUSr(unit, port, data);
          	}
          	else
              	printf("Normal\n");
      	} 
	}
}

void
diag5632_viewMIB(uint32 unit)
{
   uint32 port, entry, data, i;

   /* 
    * Verify how to read MIB counters when documentation is available.
    */

   printf("MIB counters for switch #%d\n", unit);
   printf("==========================\n");
   port = diag5632_uiGetInput("Enter port number(0-11): ");
   if (port > 11)
   {
       printf("Invalid entry!\n");
       return;
   }
   i = 0;
   do {
      entry = get_mib_index_5632(port, mib_table_5632[i].bank,
                                 mib_table_5632[i].index);

      cmic_read_mib_5632(unit, entry, &data);
      printf("%s %d\n", mib_table_5632[i].desc, data-mib_buffer_5632[port][i]);
	  mib_buffer_5632[port][i] = data;

   } while (mib_table_5632[++i].index != -1);
}

void
diag5632_clearAllMIBS(uint32 unit)
{
   uint32 port, entry, data, i;

   printf("clearing MIB counters for switch #%d\n", unit);

   for (port = 0; port < 12; port++) {
      i = 0;
      do {
         entry = get_mib_index_5632(port, mib_table_5632[i].bank,
                                          mib_table_5632[i].index);

         cmic_read_mib_5632(unit, entry, &data);
         mib_buffer_5632[port][i] = data;

      } while (mib_table_5632[++i].index != -1);
   }
}

void
diag5632_viewVLAN(uint32 unit)
{
	uint32 num, index, start;
    uint32 raw[2];
	VLAN_m_entry_t entry;

   	start = diag5632_uiGetInput("Enter starting entry: ");
   	if (start >= VLAN_ENTRY_COUNT)
   	{
   	    printf("Invalid entry!\n");
   	    return;
   	}
   	num = diag5632_uiGetInput("Enter number of entries to display: ");
   	if ((num + start) > VLAN_ENTRY_COUNT)
   	{
		printf("Invalid entry!\n");
		return;
	}
	for (index = start; index <= start+num; index++) {
		_BCM5632_table_management(unit, VLAN_m, &raw[0], index, TABLE_READ);
		printf("index = %d, raw = %0x %0x - ", index, raw[0], raw[1]);
		buf2struct_VLAN_5632(&entry, &raw[0]);
		printf("Port map = %0x Port state = %0x\n", entry.PortMap, 
				entry.PortState);
	}
}

void
diag5632_viewARL(uint32 unit)
{
   uint32 num, index, start;
   ARL_5632_m_entry_t entry;
   uint32 raw[3];
   uint32 raw1[3];

   start = diag5632_uiGetInput("Enter starting entry: ");
   if (start > BCM5632_MAX_ARL_ENTRIES)
   {
       printf("Invalid entry!\n");
       return;
   }
   num = diag5632_uiGetInput("Enter number of entries to display: ");
   if ((num + start) > BCM5632_MAX_ARL_ENTRIES)
   {
       printf("Invalid entry!\n");
       return;
   }

   watchdog_disable();
   for (index = start; index < start+num; index++) {
   		if (_BCM5632_L2_management(unit, ARL_5632_m, &raw1, &raw, index, 
							       TABLE_READ) != SOC_E_NONE)
			printf("Failed to read ARL entry #%d\n", index);
		else {
			if (raw[2] != 0) {
				memset(&entry, 0, sizeof(ARL_5632_m_entry_t));
				buf2struct_ARL_5632(&entry, raw);
           	  	printf("ARL entry #%d, port #%d, ",index,entry.DestinationPort);
              	printf("MAC addr: %02x %02x %02x %02x %02x %02x ", 
						entry.MacAddr0, entry.MacAddr1, entry.MacAddr2, 
						entry.MacAddr3, entry.MacAddr4, entry.MacAddr5);
			  	if (entry.valid)
				 	printf("(valid) ");
			  	if (entry.Static)
				 	printf("(static) ");
				printf("VLAN = %d\n", entry.VLANId);
			}
		}
   }
#if 0
   for (index = start; index <= start+num; index++) {
       if (cmic_read_memory_5632(unit, ARL_5632_m, 0, ((1 << 16) | index), 
		   (ARL_5632_m_entry_t *)raw) < 0)
           printf("Failed to read ARL entry #%d\n", index);
       else
       {
		   buf2struct_ARL_5632(&entry, raw);
		   if (entry.VLANId != 0) {
           	  printf("ARL entry #%d, port #%d, ", index, entry.DestinationPort);
              printf("MAC addr: %0x %0x %0x %0x %0x %0x\n", entry.MacAddr0,
                     entry.MacAddr1, entry.MacAddr2, entry.MacAddr3,
                     entry.MacAddr4, entry.MacAddr5);
			  if (entry.valid)
				 printf("valid entry\n");
			  if (entry.Static)
				 printf("Static entry\n");
		   }
       }
	   printf("raw[0] = %0x, raw[1] = %0x, raw[2] = %0x\n",raw[0],raw[1],raw[2]);
   }
#endif
   watchdog_enable(8);
}

void
diag5632_setLoopback(unit)
{
   uint32 port, enable;

   enable = diag5632_uiGetInput("Enter loopback mode (1=Enable, 0=Disable): ");
   if (enable > 1)
   {
       printf("Invalid entry!\n");
       return;
   }
   port = diag5632_uiGetInput("Enter port number(0-11): ");
   if (port > 11)
   {
       printf("Invalid entry!\n");
       return;
   }
#if 0  /* if MIIM through 5632 */
   phy_ge_lb_set5632(unit, port, enable);
#endif
	if (enable == 1)
		cmic_write_miim(unit, phy_map_5632[port], 0x00, 0x5140);
	else
		cmic_write_miim(unit, phy_map_5632[port], 0x00, 0x1140);
}

void
diag5632_readMIIM(unit)
{
   uint32 dev, reg, data;

   dev = diag5632_uiGetHex("Enter dev addr(hex): ");
   dev &= 0x1f;
   reg = diag5632_uiGetHex("Enter reg addr(hex): ");
   reg &= 0x1f;
#if 0 /* if MIIM through 5632 */
   cmic_read_miim_5632(unit, dev, reg, (uint16 *)&data);
#else
   cmic_read_miim(unit, dev, reg, (uint16 *)&data);
#endif
   printf("Data read: %0x\n", data);
}

void
diag5632_writeMIIM(unit)
{
   uint32 dev, reg, data;

   dev = diag5632_uiGetHex("Enter dev addr(hex): ");
   dev &= 0x1f;
   reg = diag5632_uiGetHex("Enter reg addr(hex): ");
   reg &= 0x1f;
   data = diag5632_uiGetHex("Enter data(hex): ");
#if 0 /* if MIIM through 5632 */
   cmic_write_miim_5632(unit, dev, reg, data);
#else
	cmic_write_miim(unit, dev, reg, data);
#endif
}

void
diag5632_scanARL(unit)
{
	mac_addr_t mac;
	bcm_l2_addr_t l2addr;

	mac[0] = diag5632_uiGetHex("Enter MAC byte(hex): ");
	mac[1] = diag5632_uiGetHex("Enter MAC byte(hex): ");
	mac[2] = diag5632_uiGetHex("Enter MAC byte(hex): ");
	mac[3] = diag5632_uiGetHex("Enter MAC byte(hex): ");
	mac[4] = diag5632_uiGetHex("Enter MAC byte(hex): ");
	mac[5] = diag5632_uiGetHex("Enter MAC byte(hex): ");
	if (bcm_l2_addr_get_5632(unit, mac, 1, &l2addr) == BCM_E_NONE) 
		printf("MAC found: port = %d, vid = %d\n", l2addr.port, l2addr.vid);
	else
		printf("MAC not found\n");
}

void
struct2buf_VLAN_5632(VLAN_m_entry_t *ve, uint32 *raw)
{
	raw[1] = ve->PortState & 0x03ffffff;
	raw[1] |= (ve->OutputTagUntag & 0x3f) << 26;
	raw[0] = (ve->OutputTagUntag >> 6) & 0xff;
	raw[0] |= (ve->PortMap & 0x3fff) << 8;
}

void
buf2struct_VLAN_5632(VLAN_m_entry_t *ve, uint32 *raw)
{
	memset(ve, 0, sizeof(VLAN_m_entry_t));
	ve->PortState = raw[1] & 0x03ffffff;
	ve->OutputTagUntag = ((raw[1] >> 26) & 0x3f) | ((raw[0] & 0xff) << 6);
	ve->PortMap = (raw[0] >> 8) & 0x3fff;
}

void
struct2buf_ARL_5632(ARL_5632_m_entry_t *ae, uint32 *raw)
{
	raw[2] = ae->Static | (ae->valid << 1) | (ae->TimeStamp << 2) | 
			(ae->DestinationPort << 9) | (ae->VLANId << 15);
	raw[2] |= (ae->MacAddr5 << 27);
	raw[1] = (ae->MacAddr5 >> 5) | (ae->MacAddr4 << 3) | (ae->MacAddr3 << 11) |
			 (ae->MacAddr2 << 19) | (ae->MacAddr1 << 27);
	raw[0] = (ae->MacAddr1 >> 5) | (ae->MacAddr0 << 3);
/*
	printf("raw[0] = %0x, raw[1] = %0x, raw[2] = %0x\n", raw[0],raw[1],raw[2]);
*/
}

void buf2struct_ARL_5632(ARL_5632_m_entry_t *ae, uint32 *raw)
{
	memset(ae, 0, sizeof(ARL_5632_m_entry_t));
	ae->Static = raw[2] & 0x1;
	ae->valid = (raw[2] >> 1) & 0x1;
	ae->TimeStamp = (raw[2] >> 2) & 0x7f;
	ae->DestinationPort = (raw[2] >> 9) & 0x3f;
	ae->VLANId = (raw[2] >> 15) & 0xfff;
	ae->MacAddr5 = ((raw[2] >> 27) & 0x1f) | ((raw[1] & 0x7) << 5);
	ae->MacAddr4 = (raw[1] >> 3) & 0xff;
	ae->MacAddr3 = (raw[1] >> 11) & 0xff;
	ae->MacAddr2 = (raw[1] >> 19) & 0xff;
	ae->MacAddr1 = ((raw[1] >> 27) & 0x1f) | ((raw[0] & 0x7) << 5);
	ae->MacAddr0 = (raw[0] >> 3) & 0xff;
/*
	printf("mac addr: %0x %0x %0x %0x %0x %0x\n", ae->MacAddr0, ae->MacAddr1,
	ae->MacAddr2, ae->MacAddr3, ae->MacAddr4, ae->MacAddr5);
	printf("VLAN Id = %d, Static = %d, Port = %d\n", ae->VLANId, ae->Static,
	ae->DestinationPort);
*/
}

void
diag5632_test()
{
	ARL_5632_m_entry_t ae, ae1;
	uint32 raw[3];

	ae.MacAddr0 = diag5632_uiGetHex("Enter MAC byte(hex): ");
	ae.MacAddr1 = diag5632_uiGetHex("Enter MAC byte(hex): ");
	ae.MacAddr2 = diag5632_uiGetHex("Enter MAC byte(hex): ");
	ae.MacAddr3 = diag5632_uiGetHex("Enter MAC byte(hex): ");
	ae.MacAddr4 = diag5632_uiGetHex("Enter MAC byte(hex): ");
	ae.MacAddr5 = diag5632_uiGetHex("Enter MAC byte(hex): ");
	ae.VLANId = 1;
	ae.Static = 1;
	ae.DestinationPort = 7;
	struct2buf_ARL_5632(&ae, (uint32 *)raw);
	buf2struct_ARL_5632(&ae1, (uint32 *)raw);
#if 0	
	raw[2] = ae.Static | (ae.valid << 1) | (ae.TimeStamp << 2) | 
			(ae.DestinationPort << 9) | (ae.VLANId << 15);
	raw[2] |= (ae.MacAddr5 << 27);
	raw[1] = (ae.MacAddr5 >> 5) | (ae.MacAddr4 << 3) | (ae.MacAddr3 << 11) |
			 (ae.MacAddr2 << 19) | (ae.MacAddr1 << 27);
	raw[0] = (ae.MacAddr1 >> 5) | (ae.MacAddr0 << 3);
	printf("raw[0] = %0x, raw[1] = %0x, raw[2] = %0x\n", raw[0], raw[1], raw[2]);
	memset(&ae1, 0, sizeof(ARL_5632_m_entry_t));
	ae1.Static = raw[2] & 0x1;
	ae1.valid = (raw[2] >> 1) & 0x1;
	ae1.TimeStamp = (raw[2] >> 2) & 0x7f;
	ae1.DestinationPort = (raw[2] >> 9) & 0x3f;
	ae1.VLANId = (raw[2] >> 15) & 0xfff;
	ae1.MacAddr5 = ((raw[2] >> 27) & 0x1f) | ((raw[1] & 0x7) << 5);
	ae1.MacAddr4 = (raw[1] >> 3) & 0xff;
	ae1.MacAddr3 = (raw[1] >> 11) & 0xff;
	ae1.MacAddr2 = (raw[1] >> 19) & 0xff;
	ae1.MacAddr1 = ((raw[1] >> 27) & 0x1f) | ((raw[0] & 0x7) << 5);
	ae1.MacAddr0 = (raw[0] >> 3) & 0xff;
	printf("mac addr: %0x %0x %0x %0x %0x %0x\n", ae1.MacAddr0, ae1.MacAddr1,
	ae1.MacAddr2, ae1.MacAddr3, ae1.MacAddr4, ae1.MacAddr5);
	printf("VLAN Id = %d, Static = %d, Port = %d\n", ae1.VLANId, ae1.Static,
	ae1.DestinationPort);
#endif
}

int diag5632_enablePort(uint32 unit)
{
	uint32 port;

	port = diag5632_uiGetInput("Enter port number(0-11): ");
	/* set id and forwarding state */ 	
	WRITE_P_CONF_1r(unit, port, (port << 12) | 0xc); 
	return 0;
}

int diag5632_enableLoopbacks(uint32 unit)
{
	uint32 port, val;

	/* set id and forwarding state */ 
	for (port = 0; port < 12 ; port++)
	{	
		READ_P_CONF_1r(unit, port, &val);
		val |= (1<<9);
		WRITE_P_CONF_1r(unit, port, val); 
	}
	return 0;
}

int diag5632_disableLoopbacks(uint32 unit)
{
	uint32 port, val;

	/* set id and forwarding state */ 
	for (port = 0; port < 12 ; port++)
	{	
		READ_P_CONF_1r(unit, port, &val);
		val &= ~(1<<9);
		WRITE_P_CONF_1r(unit, port, val); 
	}
	return 0;
}

int diag5632_disablePort(uint32 unit)
{
	uint32 port;

	port = diag5632_uiGetInput("Enter port number(0-11): ");
	WRITE_P_CONF_1r(unit, port, 0x00); 
	return 0;
}

int bcm5632_bpStats(uint32 unit, uint32 port)
{
	uint32 data, entry, i = 0;
	
   	do {
		entry = get_mib_index_5632(port, mib_table_5632[i].bank,
							       mib_table_5632[i].index);
      	cmic_read_mib_5632(unit, entry, &data);
      	printf("%s %d\n",mib_table_5632[i].desc,data-mib_buffer_5632[port][i]);
	  	mib_buffer_5632[port][i] = data;

	} while (mib_table_5632[++i].index != -1);
	return 0;
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	3/2/02
 *
 *-----------------------------------------------------------------
 */
int diag5632_showPortConfigRegs(uint32 unit)
{
	uint32	reg_addr;
	uint32	port, pcr1, pcr2, pcr3;

	for (port=0; port<12; port++) {
		reg_addr = 0x2E + (port * 3);
		cmic_read_reg_5632(unit, reg_addr, &pcr1);
		reg_addr++;
		cmic_read_reg_5632(unit, reg_addr, &pcr2);
		reg_addr++;
		cmic_read_reg_5632(unit, reg_addr, &pcr3);
		printf("Port %2d: PCR1 = 0x%04x, PCR2 = 0x%04x, PCR3 = 0x%04x\n",
													port, pcr1, pcr2, pcr3);
	}
	return 0;
}

int diag5632_setMonitorPort(uint32 unit)
{
	uint16 port, in, out, which;
	uint16 reg_val = 0;

	port = diag5632_uiGetInput("Enter port to be monitored(0-11): ");
	if (port > 11) {
		printf("Invalid port entry!\n");
		return 0;
	}
	SET_PMON_CONFr_MdPID(reg_val, port);
	which = diag5632_uiGetInput("Enter type (0-ingress, 1-egress, 2-both: ");
	if ((which == 2) || (which == 0)) {
		in = diag5632_uiGetInput("Enter port to monitor ingress(0-11): ");
		if (in > 11) {
			printf("Invalid port entry!\n");
			return 0;
		}
		SET_PMON_CONFr_MgIPID(reg_val, in);
	}
	if ((which == 2) || (which == 1)) {
		out = diag5632_uiGetInput("Enter port to monitor egress(0-11): ");
		if (out > 11) {
			printf("Invalid port entry!\n");
			return 0;
		}
		SET_PMON_CONFr_MgOPID(reg_val, out);
	}
	WRITE_PMON_CONFr(0, 0, reg_val); 
	return 0;
}

int diag5632_monitorPort(uint32 unit)
{
	uint32 onoff, reg_val, rv;

    if ((rv = READ_SYS_CONF_1r(unit, 0, &reg_val)) == BCM_E_NONE) {
		onoff = diag5632_uiGetInput("Port ingress monitoring (0-off, 1-on): ");
		if (onoff > 1) {
			printf("Invalid entry!\n");
			return 0;
		}
		SET_SYS_CONF_1r_PinMon(reg_val, onoff);
		onoff = diag5632_uiGetInput("Port egress monitoring (0-off, 1-on): ");
		if (onoff > 1) {
			printf("Invalid entry!\n");
			return 0;
		}
		SET_SYS_CONF_1r_PoutMon(reg_val, onoff);
        WRITE_SYS_CONF_1r(unit, 0, reg_val);
	}
	return 0;
}

int diag5632_dumpRegs(uint32 unit)
{
	uint32 reg_addr, reg_val, i = 0;
	uint32 from, to;

	from = diag5632_uiGetHex("Enter start reg(hex): ");
	to = diag5632_uiGetHex("Enter end reg(hex): ");
	if ((from > 0x484) || (to > 0x484)) {
		printf("Out of range!\n");
		return 0;
	}
	printf("Register Dump:\n");

	for (reg_addr = from; reg_addr <= to; reg_addr++) {
		cmic_read_reg_5632(unit, reg_addr, &reg_val);
		printf("%04x:%08x ", reg_addr, reg_val);
		if (++i % 4 == 0)
			printf("\n");
	}
	printf("\n");
	return 0;
}

/*
 *------------------------------------------------------------------------------
 * Function Name:       bcm5632_uiShowDiagMenu
 *
 * Description:         Prints BCM5632 diagnostic menu items.
 *
 * Created by:          Tony Wong
 *
 * Data Created:        OCT 09, 2001
 *
 * Returns: NFX_OK
 *
 *------------------------------------------------------------------------------
 */
int
bcm5632_uiShowDiagMenu(uint32 unit)
{
   int sel;
   char cmd[80];

   sel = 0;
   printf("BCM 5632 Menu - Switch #%d\n", unit);
   printf("=========================\n");
   printf("1 : Read reg\n");
   printf("2 : Write reg\n");
   printf("3 : Show PCI regs\n");
   printf("4 : Write PCI reg\n");
   printf("5 : View link status\n");
   printf("6 : View MIB counters\n");
   printf("7 : View ARL table\n");
   printf("8 : Set port PHY loopback\n");
   printf("9 : Re-init BCM5632\n");
   printf("10: Read MIIM reg\n");
   printf("11: Write MIIM reg\n");
   printf("12: Scan ARL for MAC\n");
   printf("13: View VLAN table\n");
   printf("14: Enable port\n");
   printf("15: Disable port\n");
   printf("16: Clear all MIB counters\n");
   printf("17: Show All Port Configuration regs\n");
   printf("18: Set monitoring ports\n");
   printf("19: Enable/disable port monitoring\n");
   printf("20: Dump registers\n");
   printf("99: Quit\n");
   printf("Enter Selection: ");

   memset(cmd, 0, 80);
   get_cmd(cmd);
   sscanf(cmd, "%d", &sel);
   printf("\n");
   return sel;
} 

/*
 *------------------------------------------------------------------------------
 * Function Name:       bcm5632_uiDiag
 *
 * Description:         BCM 5632 UI entry function.
 *
 * Created by:          Tony Wong
 *
 * Data Created:        OCT 08, 2001
 *
 * Returns: NFX_OK
 *
 *------------------------------------------------------------------------------
 */
int
bcm5632_uiDiag()
{
   uint32 sel, unit;

   unit = diag5632_uiGetInput("Enter switch number(0-1): ");
   if (unit > BCM5632_NUM_DEV)
   {
       printf("Invalid entry!\n");
       return 0;
   }
   for(;;) {
      sel = bcm5632_uiShowDiagMenu(unit);
      switch(sel)
      {
        case 1: diag5632_readReg(unit);
                break;
        case 2: diag5632_writeReg(unit);
                break;
		case 3: diag5632_showPCI(unit);
				break;
		case 4: diag5632_writePCI(unit);
				break;
		case 5: diag5632_viewStatus(unit);
 				break;
		case 6: diag5632_viewMIB(unit);
 				break;
        case 7: diag5632_viewARL(unit);
                break;
        case 8: diag5632_setLoopback(unit);
                break;
		case 9: bcm_init_5632(unit);
				break;
		case 10:diag5632_readMIIM(unit);
				break;
		case 11:diag5632_writeMIIM(unit);
				break;
		case 12:diag5632_scanARL(unit);
				break;
		case 13:diag5632_viewVLAN(unit);
				break;
		case 14:diag5632_enablePort(unit);
				break;
		case 15:diag5632_disablePort(unit);
				break;
		case 16:diag5632_clearAllMIBS(unit);
				break;
		case 17:diag5632_showPortConfigRegs(unit);
				break;
		case 18:diag5632_setMonitorPort(unit);
				break;
		case 19:diag5632_monitorPort(unit);
				break;
		case 20:diag5632_dumpRegs(unit);
				break;
        case 99:
   				return(NFX_OK);
        default:
                break;
      }
   }
   return(NFX_OK);
}
