/*
 *=============================================================================
 * File Name   :    bcm5632-error.h
 *
 *  RCS: $Id: bcm5632-error.h,v 1.1.1.1 2001/12/22 04:14:58 tonyw Exp $
 *
 *  Description:    Copied and modified from Broadcom
 *
 *  Created    :    09-04-2001
 *
 *  Author     :    Tony Wong
 *
 *  Copyright(c)  Agile Storage, Inc. 2001
 *=============================================================================
 */
#ifndef _SOC_ERROR_H
#define _SOC_ERROR_H

/*
 * Defines: 	
 *	SOC_XXX
 * Purpose:	
 *	SOC API return values.
 * Notes:
 *	Some SOC API return values are returned to the BCM layer. Those
 *	that are also have a BCM_E_ value defined in bcm/error.h.
 */

#define SOC_E_NONE			 0
#define	SOC_E_MII_TIMEOUT	-1
#define	SOC_E_INTERNAL		-2
#define SOC_E_NOT_IMPL		-3
#define	SOC_E_AUTONEG_DISABLED	-4
#define	SOC_E_AUTONEG_BUSY	-5
#define	SOC_E_AUTONEG_ERROR	-6
#define SOC_E_MEMORY		-7
#define	SOC_E_INIT			-8
#define	SOC_E_DMA_FAILED	-9
#define SOC_E_DMA_BUSY		-10
#define	SOC_E_DMA_NOT_FOUND	-11
#define SOC_E_DMA_DV_FULL	-12
#define SOC_E_PARAM			-13
#define SOC_E_L2_NOT_FOUND	-14

#define	SOC_E_LIMIT			-14	/* Maximum abs value */

#define SOC_SUCCESS(rv)		((rv) >= 0)	/* Demonstrative only */
#define SOC_FAILURE(rv)		((rv) < 0)	/* Demonstrative only */

extern char *soc_errmsg(int);

/*
 * Macro:
 *	SOC_IF_ERROR_RETURN
 * Purpose:
 * 	Evaluate _op as an expression, and if an error, return.
 * Notes:
 *	This macro uses a do-while construct to maintain expected
 *	"C" blocking, and evaluates "_op" ONLY ONCE so it may be
 *	a function call that has side affects.
 */
#define SOC_IF_ERROR_RETURN(_op)	\
    do { int __rv__; if ((__rv__ = (_op)) < 0) return(__rv__); } while(0)

#define BCM_E                           SOC_E_LIMIT

#define BCM_E_NONE                      SOC_E_NONE
#define BCM_E_MII_TIMEOUT               SOC_E_MII_TIMEOUT
#define BCM_E_INTERNAL                  SOC_E_INTERNAL
#define BCM_E_NOT_IMPL                  SOC_E_NOT_IMPL
#define BCM_E_AUTONEG_DISABLED          SOC_E_AUTONEG_DISABLED
#define BCM_E_AUTONEG_BUSY              SOC_E_AUTONEG_BUSY
#define BCM_E_AUTONEG_ERROR             SOC_E_AUTONEG_ERROR
#define BCM_E_MEMORY                    SOC_E_MEMORY
#define BCM_E_INIT						SOC_E_INIT
#define BCM_E_PARAM                     SOC_E_PARAM
#define BCM_E_L2_NOT_FOUND				SOC_E_L2_NOT_FOUND
#define BCM_E_PORT						(BCM_E - 1)
#define BCM_E_NOT_VLAN_MEMBER			(BCM_E - 2)
#define BCM_E_VLAN_PBM_CONFLICT			(BCM_E - 3)

#define BCM_IF_ERROR_RETURN(_op)        SOC_IF_ERROR_RETURN(_op)

#endif	/* !_BCM_ERROR_H */
