/*
 *=============================================================================
 * File Name   :    bcm5632-l2.c
 *
 *  RCS: $Id: bcm5632-l2.c,v 1.3 2002/12/04 06:28:32 tonyw Exp $
 *
 *  Description:    Copied and modified from Broadcom
 *
 *  Created    :    09-04-2001
 *
 *  Author     :    Tony Wong
 *
 *  Copyright(c)  Agile Storage, Inc. 2001
 *=============================================================================
 */

#include <nfx-incl.h>

#include "bcm5632-soctypes.h"
#include "bcm5632-cmic.h"
#include "bcm5632-error.h"
#include "bcm5632-port.h"
#include "bcm5632-l2.h"
#include <exports/cm-api.h>

extern void Struct2Buf(soc_mem_t mem, unsigned char *from, unsigned char *to, int pack_to_32);
extern void Buf2Struct(soc_mem_t mem, unsigned char *from, unsigned char *to, int pack_to_32);
extern void dump_l2_addr (char *pfx, bcm_l2_addr_t *l2addr);
extern uint32 reg_addr_5632(soc_reg_t reg, ...);
extern int bcm5632_vlan_port_add(int unit, vlan_id_t vid, pbmp_t pbmp, pbmp_t ubmp);
extern int _BCM5632_L2_management(int unit, soc_mem_t mem, void *key_data, void *entry_data, int index, int cmd);

static ARL_5632_m_entry_t sme_arl;
static int bcm5632_system_type = 0;

void bcm_l2_addr_remove_5632(int, bcm_l2_addr_t *);

void bcm5632_l2_fix_addr(int action)	// 1 = add, 0 = remove
{
	bcm_l2_addr_t l2addr;
	mac_addr_t mac;

	/* static entries for FP backplane ports */

    mac[0] = 0x00;          // type
    mac[1] = 0x01;          // port
    mac[2] = NFX_NFP_FP_ANY;// cpu
    mac[3] = 0x01;          // slot
    mac[4] = 0x08;          // 0x08
    mac[5] = 0x08;          // 0x08
    bcm_l2_addr_init_5632(&l2addr, mac, 1);
    l2addr.port = 2;
    l2addr.flags = BCM_L2_STATIC;
	if (action)
	    bcm_l2_addr_add_5632(0, &l2addr);
	else
		bcm_l2_addr_remove_5632(0, &l2addr);

    mac[0] = 0x00;
    mac[1] = 0x01;
    mac[2] = NFX_NFP_FP1;
    mac[3] = 0x01;
    mac[4] = 0x08;
    mac[5] = 0x08;
    bcm_l2_addr_init_5632(&l2addr, mac, 1);
    l2addr.port = 2;
    l2addr.flags = BCM_L2_STATIC;
	if (action)
	    bcm_l2_addr_add_5632(0, &l2addr);
	else
		bcm_l2_addr_remove_5632(0, &l2addr);

    mac[0] = 0x00;
    mac[1] = 0x01;
    mac[2] = NFX_NFP_FP2;
    mac[3] = 0x01;
    mac[4] = 0x08;
    mac[5] = 0x08;
    bcm_l2_addr_init_5632(&l2addr, mac, 1);
    l2addr.port = 2;
    l2addr.flags = BCM_L2_STATIC;
	if (action)
	    bcm_l2_addr_add_5632(0, &l2addr);
	else
		bcm_l2_addr_remove_5632(0, &l2addr);

    mac[0] = 0x00;
    mac[1] = 0x02;
    mac[2] = NFX_NFP_FP_ANY;
    mac[3] = 0x01;
    mac[4] = 0x08;
    mac[5] = 0x08;
    bcm_l2_addr_init_5632(&l2addr, mac, 1);
    l2addr.port = 3;
    l2addr.flags = BCM_L2_STATIC;
	if (action)
	    bcm_l2_addr_add_5632(0, &l2addr);
	else
		bcm_l2_addr_remove_5632(0, &l2addr);

    mac[0] = 0x00;
    mac[1] = 0x02;
    mac[2] = NFX_NFP_FP1;
    mac[3] = 0x01;
    mac[4] = 0x08;
    mac[5] = 0x08;
    bcm_l2_addr_init_5632(&l2addr, mac, 1);
    l2addr.port = 3;
    l2addr.flags = BCM_L2_STATIC;
	if (action)
	    bcm_l2_addr_add_5632(0, &l2addr);
	else
		bcm_l2_addr_remove_5632(0, &l2addr);

    mac[0] = 0x00;
    mac[1] = 0x02;
    mac[2] = NFX_NFP_FP2;
    mac[3] = 0x01;
    mac[4] = 0x08;
    mac[5] = 0x08;
    bcm_l2_addr_init_5632(&l2addr, mac, 1);
    l2addr.port = 3;
    l2addr.flags = BCM_L2_STATIC;
	if (action)
	    bcm_l2_addr_add_5632(0, &l2addr);
	else
		bcm_l2_addr_remove_5632(0, &l2addr);

    mac[0] = 0x00;
    mac[1] = 0x04;
    mac[2] = NFX_NFP_FP_ANY;
    mac[3] = 0x01;
    mac[4] = 0x08;
    mac[5] = 0x08;
    bcm_l2_addr_init_5632(&l2addr, mac, 1);
    l2addr.port = 5;
    l2addr.flags = BCM_L2_STATIC;
	if (action)
	    bcm_l2_addr_add_5632(0, &l2addr);
	else
		bcm_l2_addr_remove_5632(0, &l2addr);

    mac[0] = 0x00;
    mac[1] = 0x04;
    mac[2] = NFX_NFP_FP1;
    mac[3] = 0x01;
    mac[4] = 0x08;
    mac[5] = 0x08;
    bcm_l2_addr_init_5632(&l2addr, mac, 1);
    l2addr.port = 5;
    l2addr.flags = BCM_L2_STATIC;
	if (action)
	    bcm_l2_addr_add_5632(0, &l2addr);
	else
		bcm_l2_addr_remove_5632(0, &l2addr);

    mac[0] = 0x00;
    mac[1] = 0x04;
    mac[2] = NFX_NFP_FP2;
    mac[3] = 0x01;
    mac[4] = 0x08;
    mac[5] = 0x08;
    bcm_l2_addr_init_5632(&l2addr, mac, 1);
    l2addr.port = 5;
    l2addr.flags = BCM_L2_STATIC;
	if (action)
	    bcm_l2_addr_add_5632(0, &l2addr);
	else
		bcm_l2_addr_remove_5632(0, &l2addr);

    mac[0] = 0x00;
    mac[1] = 0x05;
    mac[2] = NFX_NFP_FP_ANY;
    mac[3] = 0x01;
    mac[4] = 0x08;
    mac[5] = 0x08;
    bcm_l2_addr_init_5632(&l2addr, mac, 1);
    l2addr.port = 0;
    l2addr.flags = BCM_L2_STATIC;
	if (action)
	    bcm_l2_addr_add_5632(0, &l2addr);
	else
		bcm_l2_addr_remove_5632(0, &l2addr);

    mac[0] = 0x00;
    mac[1] = 0x05;
    mac[2] = NFX_NFP_FP1;
    mac[3] = 0x01;
    mac[4] = 0x08;
    mac[5] = 0x08;
    bcm_l2_addr_init_5632(&l2addr, mac, 1);
    l2addr.port = 0;
    l2addr.flags = BCM_L2_STATIC;
	if (action)
	    bcm_l2_addr_add_5632(0, &l2addr);
	else
		bcm_l2_addr_remove_5632(0, &l2addr);

    mac[0] = 0x00;
    mac[1] = 0x05;
    mac[2] = NFX_NFP_FP2;
    mac[3] = 0x01;
    mac[4] = 0x08;
    mac[5] = 0x08;
    bcm_l2_addr_init_5632(&l2addr, mac, 1);
    l2addr.port = 0;
    l2addr.flags = BCM_L2_STATIC;
	if (action)
	    bcm_l2_addr_add_5632(0, &l2addr);
	else
		bcm_l2_addr_remove_5632(0, &l2addr);

	/* static entries for SP-NIM backplane ports */

    mac[0] = 0x00;
    mac[1] = 0x01;
    mac[2] = NFX_FCNIM_CPUID0;
    mac[3] = 0x02;
    mac[4] = 0x08;
    mac[5] = 0x08;
    bcm_l2_addr_init_5632(&l2addr, mac, 1);
    l2addr.port = 11;
    l2addr.flags = BCM_L2_STATIC;
	if (action)
	    bcm_l2_addr_add_5632(0, &l2addr);
	else
		bcm_l2_addr_remove_5632(0, &l2addr);

    mac[0] = 0x00;
    mac[1] = 0x01;
    mac[2] = NFX_FCNIM_CPUID1;
    mac[3] = 0x02;
    mac[4] = 0x08;
    mac[5] = 0x08;
    bcm_l2_addr_init_5632(&l2addr, mac, 1);
    l2addr.port = 10;
    l2addr.flags = BCM_L2_STATIC;
	if (action)
	    bcm_l2_addr_add_5632(0, &l2addr);
	else
		bcm_l2_addr_remove_5632(0, &l2addr);

    mac[0] = 0x00;
    mac[1] = 0x01;
    mac[2] = NFX_FCNIM_CPUID2;
    mac[3] = 0x02;
    mac[4] = 0x08;
    mac[5] = 0x08;
    bcm_l2_addr_init_5632(&l2addr, mac, 1);
    l2addr.port = 9;
    l2addr.flags = BCM_L2_STATIC;
	if (action)
	    bcm_l2_addr_add_5632(0, &l2addr);
	else
		bcm_l2_addr_remove_5632(0, &l2addr);

    mac[0] = 0x00;
    mac[1] = 0x01;
    mac[2] = NFX_FCNIM_CPUID3;
    mac[3] = 0x02;
    mac[4] = 0x08;
    mac[5] = 0x08;
    bcm_l2_addr_init_5632(&l2addr, mac, 1);
    l2addr.port = 8;
    l2addr.flags = BCM_L2_STATIC;
	if (action)
	    bcm_l2_addr_add_5632(0, &l2addr);
	else
		bcm_l2_addr_remove_5632(0, &l2addr);
}

/*
 * Function: bcm_l2_init_5632
 *
 * Description:
 *	Perform required initializations to L2 table.
 *
 * Parameters:
 *	unit - StrataSwitch PCI device unit number (driver internal).
 *
 * Returns:
 *	BCM_E_XXX
 *
 * Notes:
 *	None.
 */

int bcm_l2_init_5632(int unit)
{
    /*
     * Insert a static ARL table entry via VLAN ID 0xfff and MAC address
     * 0xffffffffffff.  This is an errata that is required for proper
     * operation to continue when the ARL gets full.
     */
    ARL_5632_m_entry_t ae;
    int			rv;
    uint32 raw[3];

    sme_arl.MacAddr0 = ~0;
    sme_arl.MacAddr1 = ~0;
    sme_arl.MacAddr2 = ~0;
    sme_arl.MacAddr3 = ~0;
    sme_arl.MacAddr4 = ~0;
    sme_arl.MacAddr5 = ~0;
    sme_arl.VLANId = ~0;

    memcpy(&ae, &sme_arl, sizeof (ae));
    ae.Static = 1;
/*
    Struct2Buf(ARL_5632_m, (unsigned char *)&ae, (unsigned char *)raw, 1); 
*/
	struct2buf_ARL_5632(&ae, raw);
    if ((rv = cmic_insert_memory_5632(unit, ARL_5632_m, COPYNO_ALL, raw)) < 0) {
	return rv;
    }

	bcm5632_vlan_port_add(0, 1, PBMP_GE_ALL, PBMP_GE_ALL);
	bcm5632_vlan_port_add(0, 0, PBMP_GE_ALL, PBMP_GE_ALL);

    return SOC_E_NONE;
}

/*
 * Internal functions to translate between ARL_5632_m_entry_t and bcm_l2_addr_t
 */

void _bcm_l2_from_arl_5632(bcm_l2_addr_t *l2addr, ARL_5632_m_entry_t *arl_entry)
{
    ARL_5632_m_entry_t *p;

    /* fill l2addr base on arl_entry contents */
    l2addr->flags = 0;
    p= (ARL_5632_m_entry_t *)arl_entry;
    ARL_MAC_GET_5632(*p, l2addr->mac);
    l2addr->vid = p->VLANId;
    l2addr->port = p->DestinationPort;
    if (p->Static) {
	l2addr->flags |= BCM_L2_STATIC;
    }
}

void _bcm_l2_to_arl_5632(ARL_5632_m_entry_t *arl_entry, bcm_l2_addr_t *l2addr)
{
    ARL_5632_m_entry_t *p = (ARL_5632_m_entry_t *)arl_entry;

    /* clear struct */
    memset(p, 0, sizeof(ARL_5632_m_entry_t));

    /* set arl_entry fields base on l2addr contents */
    ARL_MAC_SET_5632(*p, l2addr->mac);
    p->VLANId = l2addr->vid; 
    p->DestinationPort = l2addr->port;
    if (l2addr->flags & BCM_L2_STATIC) {
	p->Static = 0x1;
    }
}

/*
 * Function: bcm_l2_addr_init_5632
 *
 * Description:
 *	Initialize a bcm_l2_addr_t to a specified MAC address and VLAN,
 *	while zeroing all other fields.
 *
 * Parameters:
 *	l2addr - Pointer to bcm_l2_addr_t
 *
 * Returns:
 *	Nothing.
 *
 * Notes:
 *	None.
 */

void bcm_l2_addr_init_5632(bcm_l2_addr_t *l2addr, mac_addr_t mac, vlan_id_t vid)
{
    memset(l2addr, 0, sizeof (*l2addr));
    memcpy(l2addr->mac, mac, sizeof (mac_addr_t));
    l2addr->vid = vid;
}

/*
 * Function: bcm_l2_addr_add_5632
 *
 * Description:
 *	Add a MAC address to the Switch Address Resolution Logic (ARL)
 *	port with the given VLAN ID and parameters.
 *
 * Parameters:
 *	unit - StrataSwitch PCI device unit number (driver internal).
 *	l2addr - Pointer to bcm_l2_addr_t containing all valid fields
 *
 * Returns:
 *	BCM_E_NONE		Success
 *	BCM_E_INTERNAL		Chip access failure
 *
 * Notes:
 */
int 
bcm_l2_addr_add_5632(int unit, bcm_l2_addr_t *l2addr)
{
    ARL_5632_m_entry_t	ae;
    uint32		raw[3];

    /* initialize */
    _bcm_l2_to_arl_5632((void *)&ae, l2addr);
    /* convert struct to buffer format */
/*
    Struct2Buf(ARL_5632_m, (unsigned char *)&ae, (unsigned char *)raw, 1);  
*/
	struct2buf_ARL_5632(&ae, raw);
    /* add entry to chip's L2 table */
    cmic_insert_memory_5632(unit, ARL_5632_m, COPYNO_ALL, raw);

    return BCM_E_NONE;
}

void
bcm_l2_addr_remove_5632(int unit, bcm_l2_addr_t *l2addr)
{
    ARL_5632_m_entry_t  ae;
    uint32      raw[3];

    /* initialize */
    _bcm_l2_to_arl_5632((void *)&ae, l2addr);
    /* convert struct to buffer format */
    struct2buf_ARL_5632(&ae, raw);
    _BCM5632_L2_management(unit, ARL_5632_m, raw, (void *)0, 0, TABLE_DELETE);
}

/*
 * Function: bcm_l2_addr_get_5632
 *
 * Description:
 *  Given a MAC address and VLAN ID, check if the entry is present
 *  in the L2 table, and if so, return all associated information.
 *
 * Parameters:
 *  unit - StrataSwitch PCI device unit number (driver internal).
 *  mac - input MAC address to search
 *  vid - input VLAN ID to search
 *  l2addr - Pointer to bcm_l2_addr_t structure to receive results
 *
 * Returns:
 *  BCM_E_NONE      Success (l2addr filled in)
 *  BCM_E_PARAM     Illegal parameter (NULL pointer)
 *  BCM_E_INTERNAL      Chip access failure
 *  BCM_E_L2_NOT_FOUND  Address not found (l2addr not filled in)
 *
 * Notes:
 *  None.
 */
int
bcm_l2_addr_get_5632(int unit, mac_addr_t mac, vlan_id_t vid, bcm_l2_addr_t *l2addr)
{
    /* for BCM5632, L2 search is base on MAC only */

    ARL_5632_m_entry_t ae, ae1;
    uint32 raw[3];
	uint32 raw1[3];
    int rv;

    /* initialize local storage */
    memset((char *)&ae, 0, sizeof(ARL_5632_m_entry_t));
    memset((char *)&ae1, 0, sizeof(ARL_5632_m_entry_t));
    memset((char *)&raw, 0, sizeof(uint32) * 3);
    memset((char *)&raw1, 0, sizeof(uint32) * 3);

    /* set in struct the MAC to search for ...*/
    ARL_MAC_SET_5632(ae, mac);
    ae.VLANId = vid;

    /* transform struct into a raw buffer ready for cmic_ ...*/
	struct2buf_ARL_5632(&ae, raw);
    /* search L2 ... */
/*
printf("raw = %0x %0x %0x\n", raw[0], raw[1], raw[2]);
*/
    rv = cmic_search_memory_5632(unit, ARL_5632_m, &raw, &raw1);
    /* return if error occurs in operation ... */
    SOC_IF_ERROR_RETURN(rv);
    /* return if not found ... */
    if (SOC_FAILURE(rv)) {
        return BCM_E_L2_NOT_FOUND;
    }

    /* transform cmic_ buffer back into struct ...*/
/*
printf("raw1 = %0x %0x %0x\n", raw1[0], raw1[1], raw1[2]);
*/
	buf2struct_ARL_5632(&ae1, raw1);
    if (ae1.VLANId != vid)
        return BCM_E_L2_NOT_FOUND;

    /* update bcm l2 struct ...*/
    _bcm_l2_from_arl_5632(l2addr, (ARL_5632_m_entry_t *)&ae1);
    return BCM_E_NONE;
}

/*
 * Function: bcm_l2_age_timer_set_5632
 *
 * Description:
 *	Set the age timer for all blocks.
 *	Setting the value to 0 disables the age timer.
 *
 * Parameters:
 *	unit - StrataSwitch PCI device unit number (driver internal).
 *	age_seconds - Age timer value in seconds
 *
 * Returns:
 *	BCM_E_NONE		Success
 *	BCM_E_INTERNAL		Chip access failure
 *
 * Notes:
 *	None.
 */
int
bcm_l2_age_timer_set_5632(int unit, int age_seconds)
{
    uint32	value;

    /* program max age */
    cmic_read_reg_5632(unit, CMIC_REG(SYS_CONF_2r), &value);
    SET_SYS_CONF_2r_MaxAge(value, age_seconds);
    cmic_write_reg_5632(unit, CMIC_REG(SYS_CONF_2r), value);

    /* disable aging if MaxAge is 0; otherwise, enable aging */
    cmic_read_reg_5632(unit, CMIC_REG(SYS_CONF_1r), &value);
    SET_SYS_CONF_1r_AgeEN(value, ((age_seconds) ? 1 : 0));
    cmic_write_reg_5632(unit, CMIC_REG(SYS_CONF_1r), value);

    return BCM_E_NONE;
}

/*
 * Function: bcm_l2_age_timer_get_5632
 *
 * Description:
 *	Returns the current age timer value.
 *	The value is 0 if aging is not enabled.
 *
 * Parameters:
 *	unit - StrataSwitch PCI device unit number (driver internal).
 *	age_seconds - Place to store returned age timer value in seconds
 *
 * Returns:
 *	BCM_E_NONE		Success
 *	BCM_E_INTERNAL		Chip access failure
 *
 * Notes:
 *	None.
 */
int
bcm_l2_age_timer_get_5632(int unit, int *age_seconds)
{
    uint32		value;

    cmic_read_reg_5632(unit, CMIC_REG(SYS_CONF_2r), &value);
    *age_seconds = GET_SYS_CONF_2r_MaxAge(value);

    return BCM_E_NONE;
}

void
bcm_update_5632()
{   int sys_type;

    sys_type = cmGetCardType(1);
    if (bcm5632_system_type != sys_type) {
        if (sys_type == CM_CARD_TYPE_CHEETAH)
            bcm5632_l2_fix_addr(1);
        else
            bcm5632_l2_fix_addr(0);
        bcm5632_system_type = sys_type;
    }
}

