/*
 *=============================================================================
 * File Name   :    bcm5632-l2.h
 *
 *  RCS: $Id: bcm5632-l2.h,v 1.1.1.1 2001/12/22 04:14:58 tonyw Exp $
 *
 *  Description:    Copied and modified from Broadcom
 *
 *  Created    :    09-04-2001
 *
 *  Author     :    Tony Wong
 *
 *  Copyright(c)  Agile Storage, Inc. 2001
 *=============================================================================
 */

#ifndef BCM5632_L2_H
#define BCM5632_L2_H

#define BCM5632_MAX_ARL_ENTRIES	32*1024
/*
 * Device-independent structure for representing an L2 address
 */

#define BCM_L2_COS_SRC_PRI	0x00000001  /* src COS has prio over dst COS */
#define	BCM_L2_DISCARD_SRC	0x00000002
#define	BCM_L2_DISCARD_DST	0x00000004
#define	BCM_L2_COPY_TO_CPU	0x00000008
#define	BCM_L2_L3LOOKUP		0x00000010
#define BCM_L2_STATIC		0x00000020
#define BCM_L2_HIT		0x00000040
#define BCM_L2_TRUNK_MEMBER	0x00000080

#define BCM_L2_RTAG_MAC_SRC	0x1
#define BCM_L2_RTAG_MAC_DST	0x2
#define BCM_L2_RTAG_MAC_SRCDST	0x3
#define BCM_L2_RTAG_IP_SRC	0x4
#define BCM_L2_RTAG_IP_DST	0x5
#define BCM_L2_RTAG_IP_SRCDST	0x6

typedef struct bcm_l2_addr_s {
    uint32		flags;			/* BCM_L2_XXX flags */
    mac_addr_t		mac;			/* 802.3 MAC address */
    vlan_id_t		vid;			/* VLAN identifier */
    int			port;			/* Zero-based port number */
    trunk_id_t		tgid;			/* Trunk group ID */
    int			rtag;			/* Trunk port select formula */
    cos_prio_t		cos_dst;		/* COS based on dst addr */
    cos_prio_t		cos_src;		/* COS based on src addr */
} bcm_l2_addr_t;

/*
 * L2 table initialization procedure (called from init.c)
 */

extern int bcm_l2_init_5632(int unit);

/*
 * Initialize a bcm_l2_addr_t to a specified MAC address and VLAN,
 * zeroing all other fields.
 */

extern void bcm_l2_addr_init_5632(bcm_l2_addr_t *l2addr,
			     mac_addr_t mac_addr, vlan_id_t vid);

/*
 * Add address to L2 table
 */

extern int bcm_l2_addr_add_5632(int unit, bcm_l2_addr_t *l2addr);

/*
 * Look up address in L2 table
 */

extern int bcm_l2_addr_get_5632(int unit, mac_addr_t mac_addr, vlan_id_t vid,
			   bcm_l2_addr_t *l2addr);

/*
 * Set L2 table aging time
 */

extern int bcm_l2_age_timer_set_5632(int unit, int age_seconds);
extern int bcm_l2_age_timer_get_5632(int unit, int *age_seconds);

/*
 * Routines for internal (diagnostics) use only
 */

void _bcm_l2_to_arl_5632(ARL_5632_m_entry_t *arl_entry, bcm_l2_addr_t *l2addr);

#endif	/* BCM5632_L2_H */
