/*
 *=============================================================================
 * File Name   :    bcm5632-port.c
 *
 *  RCS: $Id: bcm5632-port.c,v 1.5 2002/10/09 19:39:26 tonyw Exp $
 *
 *  Description:    Copied and modified from Broadcom
 *
 *  Created    :    09-04-2001
 *
 *  Author     :    Tony Wong
 *
 *  Copyright(c)  Agile Storage, Inc. 2001
 *=============================================================================
 */
#include <nfx-incl.h>
#include <exports/eee-api.h>
#include <exports/malloc-api.h>

#include "bcm5632-soctypes.h"
#include "bcm5632-cmic.h"
#include "bcm5632-socregs.h"
#include "bcm5632-error.h"
#include "bcm5632-port.h"
#include "bcm5632-phy.h"

typedef struct port_cntl_s {
    int				pc_inited;
    int				pc_mask_index;
    pbmp_t			pc_autoforward; 	/* Auto-forward mask */
} port_cntl_t;

/* Declare static functions */
static int _bcm_port_vlan_set(int unit, pbmp_t ports, vlan_id_t vid);
static int _bcm_port_stp_set(int unit, soc_port_t port, int stp_state);
static int _bcm_ptable_update_vlan_add(int unit, vlan_id_t vid,
                           pbmp_t pbmp, pbmp_t ubmp);
static int _bcm_port_vlan_disable(int unit, pbmp_t ports);

/*
 * Variable:	bcm_port_control
 * Purpose:	One entry for each SOC device containing port information
 *		for that device.  Use the PORT macro to access it.
 */
static port_cntl_t bcm_port_control[N_SOC_DEVICES];

#define PORT(unit)	bcm_port_control[unit]		/* Current port */

/*
 * Define:	PORT_INIT
 * Purpose:
 *	Causes a routine to return BCM_E_INIT if port is not yet initialized.
 */

#define PORT_INIT(unit) \
	if (! PORT(unit).pc_inited) { return (BCM_E_INIT); }

int is_tag_vlan_on(int unit);

extern uint32 reg_addr_5632(soc_reg_t reg, ...);
extern int mac_ge_ability5632(int unit, soc_port_t port, port_mode_t *mode);
extern int mac_ge_loopback_set5632(int unit, soc_port_t port, int loopback);
extern int mac_ge_loopback_get5632(int unit, soc_port_t port, int *loopback);
extern int bcm5632_linkscan_port_set(int unit, soc_port_t port, int mode);
extern int bcm5632_vlan_port_get(int unit, vlan_id_t vid, pbmp_t *pbmp, pbmp_t *ubmp);


static int
_port_ptable_get(int unit, soc_port_t port, ptable_entry_t *pv)
/*
 * Function: _port_ptable_get
 * Purpose:
 *	Retrieve the PTABLE entry for the spcified port.
 * Parameters:
 *	unit - StrataSwitch unit number.
 *	port - StrataSwitch port bit mask of ports to update.
 *	pv   - pointer to ptable entry to write.
 * Returns:
 *	BCM_E_NONE
 *	BCM_E_INTERNAL if table read failed.
 * Notes:
 *	Assumes PLOCK head for specified unit.
 */
{
	uint32     data;
	uint32		data1;
 	VLAN_m_entry_t entry;
	_u32 buf[MAX_MEM_WORDS];
    int rc;

	SOC_IF_ERROR_RETURN(READ_SYS_CONF_1r(unit, 0, &data));
	/* Vlan table */
	if ((is_tag_vlan_on(unit)) && GET_SYS_CONF_1r_Per_VLAN_ST(data)) { 
		rc = cmic_read_memory_5632(unit, VLAN_m, COPYNO_ALL, 
								  (unsigned int) pv -> vlan_tag, buf);
		if (rc != SOC_E_NONE) {
			return(BCM_E_INTERNAL);
		}
		Buf2Struct(VLAN_m, (unsigned char *)&buf[0],(unsigned char *)&entry, 1);
		pv -> ut_port_bitmap = ~ entry.OutputTagUntag; 
 	}

	SOC_IF_ERROR_RETURN(cmic_read_reg_5632(unit, 
						reg_addr_5632(P_CONF_1r, port), &data));
	pv->cml = GET_P_CONF_1r_PortST(data);
	if ((GET_P_CONF_1r_PortST(data) == 0x10 || 
		 GET_P_CONF_1r_PortST(data) == 0x11) && !GET_P_CONF_1r_LrnDis(data))
		pv->cml = 0x00;
	else if (GET_P_CONF_1r_PortST(data) == 0x11 && GET_P_CONF_1r_LrnDis(data))
		pv->cml = 0x02;
	else if (GET_P_CONF_1r_PortST(data) == 0x00)
		pv->cml = 0x03;
	else
		pv->cml = 0x01;

	SOC_IF_ERROR_RETURN(READ_PMON_CONFr(unit, 0, &data));
	if (GET_PMON_CONFr_MgIPID(data) == port)
		pv -> mi = 1;
	else
		pv -> mi = 0;
  
	SOC_IF_ERROR_RETURN(cmic_read_reg_5632(unit, 
						reg_addr_5632(P_VLAN_MAP_0r, port), &data));
	pv->port_bitmap = data;
	  
	SOC_IF_ERROR_RETURN(cmic_read_reg_5632(unit, 
						reg_addr_5632(P_CONF_3r, port), &data));	  
	switch (GET_P_CONF_3r_Speedselect_cfg(data)) {
		case 0:
		case 1:
			pv->ptype = 0x01; /* 10/100M */
		break;
		
		case 2:
			pv->ptype = 0x02; /* 1G */
		break;

		default:
		break;
	}

	SOC_IF_ERROR_RETURN(cmic_read_reg_5632(unit, 
						reg_addr_5632(P_VLAN_MAP_0r, port), &data));	
	pv->cpu = GET_P_VLAN_MAP_0r_CpuMap(data);
		  
	SOC_IF_ERROR_RETURN(cmic_read_reg_5632(unit, 
						CMIC_REG(TRK_SETUP_0r + pv ->tgid), &data));
	pv-> tgid = GET_TRK_SETUP_0r_Tsetup_remote(data);
	pv-> trunk = GET_TRK_SETUP_0r_Tsetup_local(data);
	  
	SOC_IF_ERROR_RETURN(cmic_read_reg_5632(unit,CMIC_REG(TRK_DIST_SELr),&data));
	pv->rtag = 0x00;
	if (GET_TRK_DIST_SELr_L2MacSA (data))
		pv->rtag = pv->rtag | 0x1;
	if (GET_TRK_DIST_SELr_L2MacDA (data))
		pv->rtag = pv->rtag | 0x2;
	if (GET_TRK_DIST_SELr_IPMacSA (data))
		pv->rtag = pv->rtag | 0x4;
	if (GET_TRK_DIST_SELr_IPMacDA (data))
		pv->rtag = pv->rtag | 0x5;
	if (GET_TRK_DIST_SELr_IPMacSA (data) && GET_TRK_DIST_SELr_IPMacDA (data))
		pv->rtag = 0x6;
	  
	SOC_IF_ERROR_RETURN(cmic_read_reg_5632(unit, 
						reg_addr_5632(P_CONF_1r, port), &data));
	if (!GET_P_CONF_1r_PortST(data))
		pv->prt_dis |= 0x10; /* set bit4 1*/
	else 
	if ((data1 = GET_P_CONF_1r_PortST(data)) == PVP_STP_FORWARDING)
		pv->prt_dis |= 0x20; /* set bit5 1*/
	else 
		pv->prt_dis &= 0x0f; /* bit4,5 =0*/;	
	
	 SOC_IF_ERROR_RETURN(cmic_read_reg_5632(unit, 
						 reg_addr_5632(P_CONF_1r, port), &data));
	 pv->sp_st = GET_P_CONF_1r_PortST(data);
	 return(SOC_E_NONE);	
}

static int
_port_ptable_set(int unit, soc_port_t port, ptable_entry_t *pv)
/*
 * Function:
 *	_port_ptable_set
 * Purpose:
 *	Set the ptable entry for the specified port.
 * Parameters:
 *	unit - StrataSwitch unit number.
 *	port - StrataSwitch port bit mask of ports to update.
 *	pv   - pointer to ptable entry to write.
 * Returns:
 *	BCM_E_NONE
 *	BCM_E_INTERNAL if table write failed.
 * Notes:
 *	Assumes PLOCK head for specified unit.
 */
{
	int	       p_state;
	uint32     data, speed_bit;
	uint32		mirror;

    VLAN_m_entry_t entry;
	_u32 buf[MAX_MEM_WORDS];
    int rv,rc;
              
	SOC_IF_ERROR_RETURN(READ_SYS_CONF_1r(unit, 0, &data));
	/* Vlan table */
	if ((is_tag_vlan_on(unit)) && GET_SYS_CONF_1r_Per_VLAN_ST(data)) { 
		rc = cmic_read_memory_5632(unit, VLAN_m, COPYNO_ALL, 
								  (unsigned int) pv -> vlan_tag, buf);
		if (rc != SOC_E_NONE) {
			return(BCM_E_INTERNAL);
		}
		Buf2Struct(VLAN_m, (unsigned char *)&buf[0],(unsigned char *)&entry, 1);
		entry.OutputTagUntag = (int) ~ (pv -> ut_port_bitmap); 
 
	    /* Update the entry for this VLAN.  */
		Struct2Buf(VLAN_m, (unsigned char *)&entry,(unsigned char *)&buf[0], 1);
		rc = cmic_write_memory_5632(unit, VLAN_m, COPYNO_ALL, 
								   (unsigned int) pv -> vlan_tag, buf);
		if (rc != SOC_E_NONE) {
			return(BCM_E_INTERNAL);
		}
	}
	else { /* port base */
	  SOC_IF_ERROR_RETURN(cmic_read_reg_5632(unit, 
						  reg_addr_5632(P_CONF_1r, port), &data));
	  switch(pv->cml) {
		case PVP_CML_SWITCH: /* 0 */
			p_state = PVP_STP_FORWARDING; /* set forward */
			SET_P_CONF_1r_PortST(data, p_state);
			p_state = 0; /* clear lrnDis */
			SET_P_CONF_1r_LrnDis(data, p_state);
		break;	
		
		case PVP_CML_CPU: /* 1 */
		break;

		case PVP_CML_FORWARD: /* 2 */
			p_state = PVP_STP_FORWARDING; /* set forward */
			SET_P_CONF_1r_PortST(data, p_state);
			p_state = 1; /* set lrnDis */
			SET_P_CONF_1r_LrnDis(data, p_state);
		break;

		case PVP_CML_DROP: /* 3 */
			p_state = PVP_STP_DISABLED; /* set port disable */
			SET_P_CONF_1r_PortST(data, p_state);
		break;
		
	    default:
		break;
	  }
	  rv = cmic_write_reg_5632(unit, reg_addr_5632(P_CONF_1r, port), data);

	  rv = cmic_write_reg_5632(unit, reg_addr_5632(P_VLAN_MAP_0r, port), 
							  (uint32) pv->port_bitmap);

	  SOC_IF_ERROR_RETURN(cmic_read_reg_5632(unit, 
						  reg_addr_5632(P_CONF_1r, port), &data));
	  if (pv->prt_dis & 0x10) { /* bit4 = 1*/
		p_state = PVP_STP_DISABLED; 
		SET_P_CONF_1r_PortST(data, p_state);
	  }
	  else if (pv->prt_dis & 0x20) {/* bit5 = 1*/
		p_state = PVP_STP_FORWARDING; /* 3 */
		SET_P_CONF_1r_PortST(data, p_state);
	  }
	  else ;
      rv = cmic_write_reg_5632(unit, reg_addr_5632(P_CONF_1r, port), data);		
	}
	
	SOC_IF_ERROR_RETURN(READ_PMON_CONFr(unit, 0, &mirror));
	if (pv -> mi) 
			SET_PMON_CONFr_MgIPID(mirror, GET_LO_WORD(port));
	SOC_IF_ERROR_RETURN(WRITE_PMON_CONFr(unit, 0, mirror));

	SOC_IF_ERROR_RETURN(cmic_read_reg_5632(unit, 
						reg_addr_5632(P_CONF_3r, port), &data));	  
	switch(pv->ptype) {
		case 0: /* reserve */
		break;	
		
		case 1: /* 10/100 */
			speed_bit = 1; /* 100 Mbps */
			SET_P_CONF_3r_Speedselect_cfg(data, speed_bit);
		break;

		case 2: 
			speed_bit = 2; /* 1G Mbps */
			SET_P_CONF_3r_Speedselect_cfg(data, speed_bit);
		break;

		case 3: /* CPU */
		break;
		
	    default:
		break;
	  }
	  rv = cmic_write_reg_5632(unit, reg_addr_5632(P_CONF_3r, port), data);

	  SOC_IF_ERROR_RETURN(cmic_read_reg_5632(unit, 
						  reg_addr_5632(P_VLAN_MAP_0r, port), &data));	
	  if (pv->cpu) {
		  p_state = 0x00;
	
	  }
	  else {
		  p_state = 0x01;
	  }
	  SET_P_VLAN_MAP_0r_CpuMap(data,p_state);
	  rv = cmic_write_reg_5632(unit, reg_addr_5632(P_VLAN_MAP_0r, port), data);
	  
	  SOC_IF_ERROR_RETURN(cmic_read_reg_5632(unit, 
						  CMIC_REG(TRK_SETUP_0r + pv ->tgid), &data));
	  SET_TRK_SETUP_0r_Tsetup_remote(data,pv-> tgid);
	  SET_TRK_SETUP_0r_Tsetup_local(data,pv-> trunk);
	  rv = cmic_write_reg_5632(unit, CMIC_REG(TRK_SETUP_0r+ pv -> tgid), data);


	  SOC_IF_ERROR_RETURN(cmic_read_reg_5632(unit, 
						  CMIC_REG(TRK_DIST_SELr), &data));
	  p_state = 0x1;
	  switch(pv->rtag) {
		case 1: 
			SET_TRK_DIST_SELr_L2MacSA (data, p_state);
		break;	
		
		case 2: 
			SET_TRK_DIST_SELr_L2MacDA (data, p_state);
		break;

		case 3:
			SET_TRK_DIST_SELr_L2MacSA (data, p_state);
			SET_TRK_DIST_SELr_L2MacDA (data, p_state);
		break;

		case 4:
			SET_TRK_DIST_SELr_IPMacSA (data, p_state);
		break;

		case 5:
			SET_TRK_DIST_SELr_IPMacDA (data, p_state);
		break;

		case 6:
			SET_TRK_DIST_SELr_IPMacSA (data, p_state);
			SET_TRK_DIST_SELr_IPMacSA (data, p_state);
		break;
		
	    default:
		break;
	  }
	  rv = cmic_write_reg_5632(unit, CMIC_REG(TRK_DIST_SELr), data);
	
	 SOC_IF_ERROR_RETURN(cmic_read_reg_5632(unit, 
						 reg_addr_5632(P_CONF_1r, port), &data));
	 p_state = pv->sp_st;
	 SET_P_CONF_1r_PortST(data, p_state);
	 rv = cmic_write_reg_5632(unit, reg_addr_5632(P_CONF_1r, port), data);
	 return(rv);	
}

/* 
 * Function:get_avlan_entry
 * Purpose:
 *      Return a specific vlan entry 
 * Parameters: 
 *	unit   - Switch unit number.
 *      vlanid - vlan id
 *      entry  - a pointer to a vlan entry buffer for return value.
 * Returns:
 *      B_E_NONE 
 *      B_E_INTERNAL 
 */
int
get_avlan_entry(int unit, int vlanid, VLAN_m_entry_t *entry)
{
    int rc, index, entry_dw;
    uint32  *list;

    entry_dw = BYTES2WORDS(get_mem_numbytes_5632(VLAN_m));
    list = (uint32 *)eee_ramAlloc(entry_dw * sizeof(uint32)); 
    if (list == NULL) {
       return BCM_E_MEMORY;
    }
    index = MAKE_UINT32(0, vlanid);
    rc = cmic_read_memory_5632(unit, VLAN_m, 0, index, list);
    if (rc != SOC_E_NONE) {
       eee_ramDealloc(list);
       return(rc);
    }
    Buf2Struct(VLAN_m, (unsigned char *)list, (unsigned char *)entry, 1);
    eee_ramDealloc(list);
    return(BCM_E_NONE);
}

/* 
 * Function:set_avlan_entry
 * Purpose:
 *      Set a vlan entry 
 * Parameters: 
 *	unit   - Switch unit number.
 *      vlanid - vlan id
 *      entry  - a pointer to a vlan entry buffer to set the vlan 
 * Returns:
 *      B_E_NONE 
 *      B_E_INTERNAL 
 */
int
set_avlan_entry(int unit, int vlanid, VLAN_m_entry_t *entry)
{
    _u32 buf[MAX_MEM_WORDS];
    int rc;

    Struct2Buf(VLAN_m, (unsigned char *)entry, (unsigned char *)&buf[0], 1);
    rc = cmic_write_memory_5632(unit, VLAN_m, COPYNO_ALL, vlanid, buf);
    if (rc != SOC_E_NONE) {
       return(BCM_E_INTERNAL);
    }
    return(BCM_E_NONE);
}

/* 
 * Function:is_vlan_member
 * Purpose:
 *      Return 0 or 1 
 * Parameters: 
 *	unit   - Switch unit number.
 *      vlanid - vlan id
 *      port   - a port nuber 
 * Returns:
 *      0 : The port is not a member of the vlan.
 *      1 : The port is a member of the vlan.
 *      BCM_E_INTERNAL
 */
int
is_vlan_member(int unit, int vlanid, soc_port_t port) 
{
    VLAN_m_entry_t  entry;
       
    SOC_IF_ERROR_RETURN(get_avlan_entry(unit, vlanid, &entry));
    if (PBMP_MEMBER(entry.PortMap, port)) {
       return(1);
    } else {
       return(0);
    }
}
/* 
 * Function:is_tag_vlan_on 
 * Purpose:
 *      Return if the 802.1q VLAN is on or not
 * Parameters: None
 *	unit - Switch unit number.
 * Returns:
 *      None zero if Qon is on 
 *      0 if Qon is off 
 */
int
is_tag_vlan_on(int unit)
{
    uint32  data;

    cmic_read_reg_5632(unit, CMIC_REG(SYS_CONF_3r), &data);
    return(GET_SYS_CONF_3r_Qon(data));
}

/*
 * Function: bcm5632_port_vlan_get
 * Purpose:
 *	Retrieve the PTABLE VID for the indicated port
 * Parameters:
 *	unit - StrataSwitch unit number.
 *	port - StrataSwitch port number of port to get info for
 *	vid  - pointer to vlan id for return
 * Returns:
 *	BCM_E_NONE
 *	BCM_E_INTERNAL if table read failed.
 */
int
bcm5632_port_vlan_get(int unit, soc_port_t port, vlan_data_t *vdata)
{
    ptable_entry_t pv;

    SOC_IF_ERROR_RETURN(_port_ptable_get(unit, port, &pv));
    vdata->vlan_tag = pv.vlan_tag;
    vdata->port_bitmap = pv.port_bitmap;
    vdata->ut_port_bitmap = pv.ut_port_bitmap;

    return BCM_E_NONE;
}

/*
 * Function: bcm5632_port_vlan_set
 * Purpose:
 *	Update the PTABLE VID and bitmaps for the indicated port
 * Parameters:
 *	unit  - StrataSwitch unit number.
 *	ports - StrataSwitch port bit mask of ports to update.
 *	vid   - pointer to vlan id for return
 * Returns:
 *	BCM_E_NONE
 *	BCM_E_VLAN_NOT_FOUND if vid not in VTABLE
 *	BCM_E_INTERNAL if table read failed.
 * Notes:
 *	Reads the vtable for the given vid.  Sets ptable entry
 *      with those bitmaps.
 *      Checks that the ports all belong to the indicated VLAN.
 */
int
bcm5632_port_vlan_set(int unit, pbmp_t ports, vlan_id_t vid)
{
    int rv;

    PORT_INIT(unit);
    rv = _bcm_port_vlan_set(unit, ports, vid);
    return rv;
}

/*
 * Function: _bcm_port_vlan_set
 * Purpose:
 *      Guts of bcm_port_vlan_set.  See above.
 *      Assumes locking at higher level
 */
static int
_bcm_port_vlan_set(int unit, pbmp_t ports, vlan_id_t vid)
{
    pbmp_t pbmp, ubmp;
    soc_port_t p;

    SOC_IF_ERROR_RETURN(bcm5632_vlan_port_get(unit, vid, &pbmp, &ubmp));

    /* All ports must belong to this VLAN. */
    if ((pbmp & ports) != ports) {
        return BCM_E_VLAN_PBM_CONFLICT;
    }
    PBMP_ITER(ports, p) {
        uint32 data;
 
        SOC_IF_ERROR_RETURN(cmic_read_reg_5632(unit, 
							reg_addr_5632(P_VLAN_MAP_0r,p), &data));
        /* bit 14-15 is Reserved, keep bit14-15, clear bit0-13  */
        /* pbmp only valid for bit0-13 */
        data = (data & 0xc000) | (pbmp & 0x3ff);

        SOC_IF_ERROR_RETURN(cmic_write_reg_5632(unit, 
							reg_addr_5632(P_VLAN_MAP_0r, p), data));
    }
    return BCM_E_NONE;
}

/*
 * Function: bcm5632_port_vlan_disable
 * Purpose:
 *	Disable PVLAN by setting VID to VLAN_ID_INVALID
 *      All bitmaps are set to all
 * Parameters:
 *	unit  - StrataSwitch unit number.
 *	ports - StrataSwitch port bit mask of ports to update.
 * Returns:
 *	BCM_E_NONE
 *	BCM_E_INTERNAL if table read failed.
 * Notes:
 */
int
bcm5632_port_vlan_disable(int unit, pbmp_t ports)
{
    int rv;

    PORT_INIT(unit);
    rv = _bcm_port_vlan_disable(unit, ports);
    return rv;
}

/*
 * Function: _bcm_port_vlan_disable
 * Purpose:
 *      Guts of bcm_port_vlan_disable.  See above
 *      Assumes locking at higher level
 */
static int
_bcm_port_vlan_disable(int unit, pbmp_t ports)
{
/* BCM5632 does not support individual port base vlan disable */
/* When disable, it will set the mode to tag vlan mode */
/* currently no routine to set the mode to port base vlan */

    uint32  data;
    cmic_read_reg_5632(unit, CMIC_REG(SYS_CONF_3r), &data);
    SET_SYS_CONF_3r_Qon(data, 1);
    SOC_IF_ERROR_RETURN(WRITE_SYS_CONF_3r(unit, 0, data));
    return BCM_E_NONE;
}

static int
_bcm_port_stp_set(int unit, soc_port_t port, int stp_state)
/*
 * Function:
 *	_bcm_port_stp_set
 * Purpose:
 *	Set the Spanning tree state for a port.
 * Parameters:
 *	unit - StrataSwitch unit number.
 *	port - StrataSwitch port number.
 *	stp_state - State to place port in, one of BCM_PORT_STP_xxx.
 * Returns:
 *	BCM_E_NONE
 *	BCM_E_INTERNAL
 *
 * Notes:
 *	This routine manipulates the EPC_LINK register. No other
 *	modules should modify this value.
 *
 *      Minor workaround for 5600_b0, 5680_a0 chips:
 *          Turn learning off for ports put into blocking, listening
 *          or disabled state.  Turn learning on for ports put into
 *          forwarding or learning state state.  This is implemented
 *          via the "learn_flag" variable.
 *
 */
{
    int	       rv = BCM_E_NONE;
    int	       p_state;
    uint32     data;

    switch(stp_state) {
    case BCM_PORT_STP_FORWARDING:
        p_state = PVP_STP_FORWARDING;
	break;
    case BCM_PORT_STP_LISTENING:
    case BCM_PORT_STP_BLOCKING:
        p_state = PVP_STP_BLOCKING;
        break;
    case BCM_PORT_STP_LEARNING:
        p_state = PVP_STP_LEARNING;
        break;
    case BCM_PORT_STP_DISABLED:
        p_state = PVP_STP_DISABLED;
	break;
    default:
	return (BCM_E_PARAM);
    }

    /* 
      if Qon is on, and Per VLAN spanning tree bit is enabled, then port state 
      is in vlan table Port State field, else port state is in port 
      configuration register.
     */
    SOC_IF_ERROR_RETURN(READ_SYS_CONF_1r(unit, 0, &data));
    if ((is_tag_vlan_on(unit)) && GET_SYS_CONF_1r_Per_VLAN_ST(data)) {
    /* per vlan port state, the higher 16 bit is the vlan id of the port,
	 * lower 16 bit is the port 
	 */
        VLAN_m_entry_t  entry;
        int vlanid = GET_HI_WORD(port);
       
        port = GET_LO_WORD(port);

        if (!is_vlan_member(unit, vlanid, port)) {
            return(BCM_E_NOT_VLAN_MEMBER);
        }
 
        SOC_IF_ERROR_RETURN(get_avlan_entry(unit, vlanid, &entry));
        SET_VLAN_STATE_ENTRY(entry,port,p_state);
        SOC_IF_ERROR_RETURN(set_avlan_entry(unit, vlanid, &entry));  
        rv = BCM_E_NONE;
    } 
	else {
        rv = cmic_read_reg_5632(unit, reg_addr_5632(P_CONF_1r,port), &data);
        if (rv >= 0) {
            SET_P_CONF_1r_PortST(data, p_state);
            rv = cmic_write_reg_5632(unit, reg_addr_5632(P_CONF_1r, port),data);
        }
    }
    return(rv);
}

/* 
 * Function:
 *     bcm5632_port_enable_set
 * Purpose:
 *     Physically enable/disable the PHY on this port. 
 *     In the future, this may also disable the MAC TX/RX
 *     and or egress
 * Parameters:
 *	unit - StrataSwitch unit #.
 *	port - StrataSwitch port #.
 *	enable - TRUE, port is enabled, FALSE port is disabled.
 * Returns:
 *     BCM_E_XXXX
 */

int 
bcm5632_port_enable_set(int unit, soc_port_t port, int enable)
{
    return(phy_ge_enable_set5632(unit, port, enable));
}

/* 
 * Function:
 *     bcm5632_port_enable_get
 * Purpose:
 *     Gets the enable state as defined by bcm5632_port_enable_set()
 * Parameters:
 *	unit - StrataSwitch unit #.
 *	port - StrataSwitch port #.
 *	enable - (OUT) TRUE, port is enabled, FALSE port is disabled.
 * Returns:
 *     BCM_E_XXXX
 */
int 
bcm5632_port_enable_get(int unit, soc_port_t port, int *enable)
{
    return(phy_ge_enable_get5632(unit, port, enable));
}

/*
 * Function:
 *	bcm5632_port_egress_enable_set
 * Purpose:
 *	Enable/Disable the egress for the specified port.
 * Parameters:
 *	unit - StrataSwitch unit #.
 *	port - StrataSwitch port #.
 *	enable - Boolean: TRUE enable egress, FALSE disable egress.
 * Returns:
 *	BCM_E_XXXX
 */
int
bcm5632_port_egress_enable_set(int unit, soc_port_t port, int enable)
{
   /* BCM5632 always enable the egress by chip */
   if (enable) {
      return(BCM_E_NONE);
   } else {
      return(BCM_E_NOT_IMPL);
   }
}

/*
 * Function:
 *	bcm5632_port_egress_enable_get
 * Purpose:
 *	Retrieve the Enable/Disable state of an egress.
 * Parameters:
 *	unit - StrataSwitch unit #.
 *	port - StrataSwitch port #.
 *	enable - Boolean: TRUE enable egress, FALSE disable egress.
 * Returns:
 *	BCM_E_XXXX
 */
int
bcm5632_port_egress_enable_get(int unit, soc_port_t port, int *enable)
{
   /* BCM5632 always enable the egress by chip */
    *enable = TRUE;
    return(BCM_E_NONE);
}

/*
 * Function:
 *	bcm5632_port_init
 * Purpose:
 *	Initialize the PORT interface layer for the specified SOC device.
 * Parameters:
 *	unit - StrataSwitch unit number.
 * Returns:
 *	BCM_E_NONE - success (or already initialized)
 *	BCM_E_INTERNAL- failed to write PTABLE entries
 *	BCM_E_MEMORY - failed to allocate required memory.
 * Notes:
 *	All ports set in disabled state.
 *	PTABLE initialized.
 */
int
bcm5632_port_init(int unit)
{
    soc_port_t          p;
	extern uint32 mib_buffer_5632[NUM_E_PORT_5632][50];
	int i;

    PBMP_ITER(PBMP_ALL, p) {
		if (!PBMP_MEMBER(PBMP_E_ALL, p)) {
	    	continue;
		}
#if 0  /* don't enable yet */
		WRITE_P_CONF_1r(unit, p, (p << 12) | 0xc); /* id and forwarding state */
#endif
#ifdef RAL
		WRITE_P_CONF_2r(unit, p, 0x00002000);
#else
		/*
		 * RAL - turn on pause packets, 64 bit preamble, 96 bit idle, PAUSE cap
		 */
		WRITE_P_CONF_2r(unit, p, 0x0000367f);
#endif
		WRITE_P_CONF_3r(unit, p, 0x00002863);
		for (i = 0; i < 50; i++)	/* clear port MIB stats buffer */
			mib_buffer_5632[p][i] = 0;
    }
    PORT(unit).pc_inited      = TRUE;
    PORT(unit).pc_autoforward = 0;

    return(BCM_E_NONE);
}

/*
 * Function:
 *	bcm5632_ptable_init(int unit, vlan_data_t *vdata)
 * Purpose:
 *	Initialize the vtable according to data in vdata
 * Parameters:
 *	unit - StrataSwitch unit number.
 *	vdata - Set the port based VLAN information for the ports in
 *              vdata->port_bitmap
 * Returns:
 *	BCM_E_NONE
 *	BCM_E_INTERNAL - internal error.
 */
int
bcm5632_ptable_init(int unit, vlan_data_t *vdata)
{
    soc_port_t p;
    ptable_entry_t	pv;
    PBMP_ITER(PBMP_E_ALL, p) {
	SOC_IF_ERROR_RETURN(_port_ptable_get(unit, p, &pv));
	/* bit 14-15 is Reserved, keep bit14-15, clear bit0-13  */
	/* pbmp only valid for bit0-13 */
	pv.port_bitmap = pv.port_bitmap & 0xc000;
	SOC_IF_ERROR_RETURN(_port_ptable_set(unit, p, &pv));
    }

    return(BCM_E_NONE);
}

/*
 * Function:
 *	bcm5632_port_stp_set
 * Purpose:
 *	Set the Spanning tree state for a port.
 * Parameters:
 *	unit - StrataSwitch unit number.
 *	port - StrataSwitch port number.
 *	stp_state - State to place port in.
 * Returns:
 *	BCM_E_NONE
 *	BCM_E_INTERNAL
 *
 * Notes:
 *	This routine manipulates the EPC_LINK register. No other
 *	modules should modify this value.
 */
int
bcm5632_port_stp_set(int unit, soc_port_t port, int stp_state)
{
    int		rv;

    PORT_INIT(unit);
    rv = _bcm_port_stp_set(unit, port, stp_state);
    return(rv);
}

/*
 * Function:
 *	bcm5632_port_stp_get
 * Purpose:
 *	Get the Spanning tree state for a port.
 * Parameters:
 *	unit - StrataSwitch unit number.
 *	port - StrataSwitch port number.
 *	stp_state - Pointer where state stored.
 * Returns:
 *	BCM_E_NONE
 *	BCM_E_INTERNAL
 */
int
bcm5632_port_stp_get(int unit, soc_port_t port, int *stp_state)
{
    int			rv;
    int        state;
    uint32     data;

    PORT_INIT(unit);

    /* 
      if Qon is on, and Per VLAN spanning tree bit is enabled, then port state 
      is in vlan table Port State field, else port state is in port 
      configuration register.
     */
     
    SOC_IF_ERROR_RETURN(READ_SYS_CONF_1r(unit, 0, &data));
    if ((is_tag_vlan_on(unit)) && GET_SYS_CONF_1r_Per_VLAN_ST(data)) {
    /* per vlan port state, the higher 16 bit is the vlan id of the port,
	 * lower 16 bit is the port 
	 */
        VLAN_m_entry_t  entry;
        int vlanid = GET_HI_WORD(port);
        
        port = GET_LO_WORD(port);
        SOC_IF_ERROR_RETURN(get_avlan_entry(unit, vlanid, &entry));
        state = GET_VLAN_STATE_ENTRY(entry, port);
    } else {
        SOC_IF_ERROR_RETURN(cmic_read_reg_5632(unit, 
							reg_addr_5632(P_CONF_1r,port), &data));
        state = GET_P_CONF_1r_PortST(data);
    }

    rv = BCM_E_NONE;
    switch(state) {
    case PVP_STP_FORWARDING: *stp_state = BCM_PORT_STP_FORWARDING; break;
    case PVP_STP_BLOCKING: *stp_state = BCM_PORT_STP_BLOCKING;     break;
    case PVP_STP_LEARNING: *stp_state = BCM_PORT_STP_LEARNING;     break;
    case PVP_STP_DISABLED: *stp_state = BCM_PORT_STP_DISABLED;     break;
    default:
	rv = BCM_E_INTERNAL;
    }
    return(rv);
}

/*
 * Function:
 *	bcm5632_port_enable_bpdu
 * Purpose:
 *	Enable/Disable BPDUs on the specified port.
 * Parameters:
 *	unit - SOC unit #
 *	port - Port number (0 based)
 *	enable - TRUE to enable, FALSE to disable (reject bpdu).
 * Returns:
 *	BCM_E_NONE
 *	BCM_E_INTERNAL
 */
int
bcm5632_port_enable_bpdu(int unit, soc_port_t port, int enable)
{
    uint32    data;
    int			rv;

    PORT_INIT(unit);

    if ((rv = READ_SYS_CONF_1r(unit, 0, &data)) == BCM_E_NONE) {
        if (enable) 
            SET_SYS_CONF_1r_STPEN(data, 1);
        else
            SET_SYS_CONF_1r_STPEN(data, 0);
        rv = WRITE_SYS_CONF_1r(unit, 0, data);
    }
    return(rv);
}

/*
 * Function: bcm5632_port_linkscan_set
 * Purpose: Set the linkscan state for a given port
 * Parameters:
 *	unit - StrataSwitch Unit #.
 *	port - StrataSwitch port #.
 *	linkscan - Linkscan value (None, S/W, H/W)
 * Returns:
 *	BCM_E_NONE
 *	BCM_E_XXX
 */
int
bcm5632_port_linkscan_set(int unit, soc_port_t port, int linkscan)
{
    return(bcm5632_linkscan_port_set(unit, port, linkscan));
}

/*
 * Function: bcm5632_port_autoneg_get
 * Purpose: Get the autonegotiation state of the port
 * Parameters:
 *	unit - StrataSwitch Unit #.
 *	port - StrataSwitch port #.
 *	autoneg - (OUT) Boolean value
 * Returns:
 *	BCM_E_NONE
 *	BCM_E_XXX
 */
int
bcm5632_port_autoneg_get(int unit, soc_port_t port, int *autoneg)
{
    int	busy;

    PORT_INIT(unit);
    return(phy_ge_an_get5632(unit, port, autoneg, &busy));
}

/*
 * Function: bcm5632_port_autoneg_set
 * Purpose: Set the autonegotiation state for a given port
 * Parameters:
 *	unit - StrataSwitch Unit #.
 *	port - StrataSwitch port #.
 *	autoneg - Boolean value
 * Returns:
 *	BCM_E_NONE
 *	BCM_E_XXX
 */
int
bcm5632_port_autoneg_set(int unit, soc_port_t port, int autoneg)
{
    int		rv;

    PORT_INIT(unit);
    rv = phy_ge_an_set5632(unit, port, autoneg);
    return(rv);
}

/*
 * Function: bcm5632_port_autoforward_get
 * Purpose: Get the auto-forward state of the port
 * Parameters:
 *	unit - StrataSwitch Unit #.
 *	port - StrataSwitch port #.
 *	autofwd - (OUT) Boolean value
 * Returns:
 *	BCM_E_NONE
 *	BCM_E_XXX
 */
int
bcm5632_port_autoforward_get(int unit, soc_port_t port, int *autofwd)
{
    PORT_INIT(unit);

    *autofwd = PBMP_MEMBER(PORT(unit).pc_autoforward, port);
    return(BCM_E_NONE);
}

/*
 * Function: bcm5632_port_autoforward_set
 * Purpose: Set the auto-forwarding state for a given port
 * Parameters:
 *	unit - StrataSwitch Unit #.
 *	port - StrataSwitch port #.
 *	autofwd - Boolean value
 * Returns:
 *	BCM_E_NONE
 *	BCM_E_XXX
 * Notes: If enabling Autoforward, STP state is immediately changed to the 
 *	correct value (DISABLED or FORWARDING)
 */
int
bcm5632_port_autoforward_set(int unit, soc_port_t port, int autofwd)
{
    int		rv;

    PORT_INIT(unit);
    if (autofwd) {
		int	link;

		if (0 <=  (rv = bcm5632_port_link_status(unit, port, &link))) {
	    	PORT(unit).pc_autoforward |= PBMP_PORT(port);
	    	rv = _bcm_port_stp_set(unit, port, 
				   link ? 
				   BCM_PORT_STP_FORWARDING : 
				   BCM_PORT_STP_DISABLED);
		}
   	} 
	else {
		PORT(unit).pc_autoforward &= ~PBMP_PORT(port);
		rv = BCM_E_NONE;
    }
    return(rv);
}

/*
 * Function: bcm5632_port_speed_get
 * Purpose: Getting the spped of the port
 * Parameters:
 *	unit - StrataSwitch Unit #.
 *	port - StrataSwitch port #.
 *	speed - (OUT) Value in megabits/sec (10, 100, etc)
 * Returns:
 *	BCM_E_NONE
 *	BCM_E_XXX
 */
int
bcm5632_port_speed_get(int unit, soc_port_t port, int *speed)
{
    PORT_INIT(unit);

    return(phy_ge_speed_get5632(unit, port, speed));
}

/*
 * Function: 
 *	bcm5632_port_speed_set
 * Purpose: 
 *	Setting the speed for a given port
 * Parameters:
 *	unit - StrataSwitch Unit #.
 *	port - StrataSwitch port #.
 *	speed - Value in megabits/sec (10, 100, etc)
 * Returns:
 *	BCM_E_NONE
 *	BCM_E_XXX
 */
int
bcm5632_port_speed_set(int unit, soc_port_t port, int speed)
{
    int		rv;

    PORT_INIT(unit);

    rv = phy_ge_an_set5632(unit, port, FALSE);
    if (rv >= 0) {
	rv = phy_ge_speed_set5632(unit, port, speed);
    }
    if (rv >= 0) {
	rv = mac_ge_speed_set5632(unit, port, speed);
    }
    
    return(rv);
}

/*
 * Function: 
 *	bcm5632_port_loopback_set
 * Purpose: 
 *	Setting the speed for a given port
 * Parameters:
 *	unit - StrataSwitch Unit #.
 *	port - StrataSwitch port #.
 *	loopback - (OUT) one of:
 *		BCM_PORT_LOOPBACK_NONE
 *		BCM_PORT_LOOPBACK_MAC
 *		BCM_PORT_LOOPBACK_PHY
 * Returns:
 *	BCM_E_NONE
 *	BCM_E_XXX
 */
int
bcm5632_port_loopback_set(int unit, soc_port_t port, int loopback)
{
    int		error;
    int		rv = BCM_E_NONE;

    PORT_INIT(unit);

    switch(loopback) {
    case BCM_PORT_LOOPBACK_NONE:
	error =  mac_ge_loopback_set5632(unit, port, FALSE);
	error |= phy_ge_lb_set5632(unit, port, FALSE);
	break;
    case BCM_PORT_LOOPBACK_MAC:
	error =  mac_ge_loopback_set5632(unit, port, TRUE);
	error |= phy_ge_lb_set5632(unit, port, FALSE);
	break;
    case BCM_PORT_LOOPBACK_PHY:
	error =  mac_ge_loopback_set5632(unit, port, FALSE);
	error |= phy_ge_lb_set5632(unit, port, TRUE);
	break;
    default:
	error = 0;
	rv = BCM_E_PARAM;
    }
    if (error < 0) {
	rv = BCM_E_INTERNAL;
    }
    return(rv);
}
/*
 * Function: 
 *	bcm5632_port_loopback_get
 * Purpose: 
 *	Recover the current loopback operation for the specified port.
 * Parameters:
 *	unit - StrataSwitch Unit #.
 *	port - StrataSwitch port #.
 *	loopback - (OUT) one of:
 *		BCM_PORT_LOOPBACK_NONE
 *		BCM_PORT_LOOPBACK_MAC
 *		BCM_PORT_LOOPBACK_PHY
 * Returns:
 *	BCM_E_NONE
 *	BCM_E_XXX
 */
int
bcm5632_port_loopback_get(int unit, soc_port_t port, int *loopback)
{
    int		rv;
    int		phy_lb, mac_lb;

    PORT_INIT(unit);

    rv  = phy_ge_lb_get5632(unit, port, &phy_lb);
    rv |= mac_ge_loopback_get5632(unit, port, &mac_lb);
    
    if (rv < 0) {
	return(BCM_E_INTERNAL);
    } else if (mac_lb) {
	*loopback = BCM_PORT_LOOPBACK_MAC;
    } else if (phy_lb) {
	*loopback = BCM_PORT_LOOPBACK_PHY;
    } else {
	*loopback = BCM_PORT_LOOPBACK_NONE;
    }
    return(BCM_E_NONE);
}

/*
 * Function: bcm5632_port_duplex_get
 * Purpose:  Get the port duplex settings
 * Parameters:
 *	unit - StrataSwitch Unit #.
 *	port - StrataSwitch port #.
 *      duplex - (OUT) Duplex setting, one of BCM_PORT_DUPLEX_xxx
 * Returns:
 *	BCM_E_NONE
 *	BCM_E_XXX
 */
int
bcm5632_port_duplex_get(int unit, soc_port_t port, int *duplex)
{
    int	phy_duplex;

    PORT_INIT(unit);

    if (phy_ge_duplex_get5632(unit, port, &phy_duplex)) {
	return(BCM_E_INTERNAL);
    }

    *duplex = phy_duplex ? BCM_PORT_DUPLEX_FULL : BCM_PORT_DUPLEX_HALF;
    return(BCM_E_NONE);
}

/*
 * Function: bcm5632_port_duplex_set
 * Purpose:  Set the port duplex settings.
 * Parameters:
 *	unit - StrataSwitch Unit #.
 *	port - StrataSwitch port #.
 *      duplex - Duplex setting, one of BCM_PORT_DUPLEX_xxx
 * Returns:
 *	BCM_E_NONE
 *	BCM_E_XXX
 */
int
bcm5632_port_duplex_set(int unit, soc_port_t port, int duplex)
{
    int		rv;

    PORT_INIT(unit);

    rv =  phy_ge_an_set5632(unit, port, FALSE);
    if (rv >= 0) {
		rv = phy_ge_duplex_set5632(unit, port, duplex);
    }
    if (rv >= 0) {
		rv = mac_ge_duplex_set5632(unit, port, duplex);
    }
    return(rv ? BCM_E_INTERNAL : BCM_E_NONE);
}

/*
 * Function: bcm5632_port_pause_get
 * Purpose: 
 *	Get the source address for transmitted PAUSE frames.
 * Parameters:
 *	unit - StrataSwitch Unit #.
 *	port - StrataSwitch port #.
 *	pause_tx - (OUT) Boolean value
 *	pause_rx - (OUT) Boolean value
 * Returns:
 *	BCM_E_NONE
 *	BCM_E_XXX
 */
int
bcm5632_port_pause_get(int unit, soc_port_t port, int *pause_tx, int *pause_rx)
{
    int		rv;

    PORT_INIT(unit);
    rv = mac_ge_pause_get5632(unit, port, pause_tx, pause_rx);
    return(rv);
}

/*
 * Function: bcm5632_port_pause_set
 * Purpose: Set the pause state for a given port
 * Parameters:
 *	unit - StrataSwitch Unit #.
 *	port - StrataSwitch port #.
 *	pause_tx - Boolean value
 *	pause_rx - Boolean value
 * Returns:
 *	BCM_E_NONE
 *	BCM_E_XXX
 * Notes:
 * 	Symmetric pause requires the two "pause" values to be the same.
 */
int
bcm5632_port_pause_set(int unit, soc_port_t port, int pause_tx, int pause_rx)
{
    int		rv;

    PORT_INIT(unit);
    
    rv = phy_ge_an_set5632(unit, port, FALSE);
    if (rv >= 0) {
	rv = mac_ge_pause_set5632(unit, port, pause_tx, pause_rx);
    }
    return(rv);
}

/*
 * Function: bcm5632_port_pause_addr_get
 * Purpose: 
 *	Get the source address for transmitted PAUSE frames.
 * Parameters:
 *	unit - StrataSwitch Unit #.
 *	port - StrataSwitch port #.
 *	pause_tx - (OUT) Boolean value
 *	pause_rx - (OUT) Boolean value
 *	mac - (OUT) MAC address sent with apuse frames.
 * Returns:
 *	BCM_E_NONE
 *	BCM_E_XXX
 */
int
bcm5632_port_pause_addr_get(int unit, soc_port_t port, mac_addr_t mac)
{
    int		rv;

    PORT_INIT(unit);
    rv = mac_ge_pause_addr_get5632(unit, port, mac);
    return(rv);
}

/*
 * Function: bcm5632_port_pause_addr_set
 * Purpose: Set the source address for transmistted PAUSE frames.
 * Parameters:
 *	unit - StrataSwitch Unit #.
 *	port - StrataSwitch port #.
 *	pause_tx - Boolean value
 *	pause_rx - Boolean value
 *	mac - station address used for pause frames. 
 * Returns:
 *	BCM_E_NONE
 *	BCM_E_XXX
 * Notes:
 * 	Symmetric pause requires the two "pause" values to be the same.
 */
int
bcm5632_port_pause_addr_set(int unit, soc_port_t port, mac_addr_t mac)
{
    int		rv;

    PORT_INIT(unit);
    rv = mac_ge_pause_addr_set5632(unit, port, mac);
    return(rv);
    
}

/*
 * Function: bcm5632_port_advert_get
 * Purpose:  
 *	Retrieve the local port advertisment for autonegotiation.
 * Parameters:
 *	unit - StrataSwitch Unit #.
 *	port - StrataSwitch port #.
 *	ability_mask - (OUT) Local advertisment.
 * Returns:
 *	BCM_E_NONE
 *	BCM_E_XXX
 */
int
bcm5632_port_advert_get(int unit, soc_port_t port, port_mode_t *ability_mask)
{
    PORT_INIT(unit);

    return(phy_ge_adv_local_get5632(unit, port, ability_mask));
}

/*
 * Function: bcm5632_port_advert_set
 * Purpose:  
 *	Set the local port advertisment for autonegotiation.
 * Parameters:
 *	unit - StrataSwitch Unit #.
 *	port - StrataSwitch port #.
 *	ability_mask - Local advertisment.
 * Returns:
 *	BCM_E_NONE
 *	BCM_E_XXX
 * Notes:
 *	Does NOT restart autonegotiation.
 */
int
bcm5632_port_advert_set(int unit, soc_port_t port, port_mode_t ability_mask)
{
    int		rv;

    PORT_INIT(unit);
    rv = phy_ge_adv_local_set5632(unit, port, ability_mask);
    return(rv);
}

/*
 * Function: bcm5632_port_remote_advert_get
 * Purpose:  
 *	Retrieve the local port advertisment for autonegotiation.
 * Parameters:
 *	unit - StrataSwitch Unit #.
 *	port - StrataSwitch port #.
 *	ability_mask - (OUT) Remote advertisement.
 * Returns:
 *	BCM_E_NONE
 *	BCM_E_XXX
 */
int
bcm5632_port_remote_advert_get(int unit, soc_port_t port, 
							   port_mode_t *ability_mask)
{
    int			phy_autoneg, phy_busy;

    PORT_INIT(unit);

    SOC_IF_ERROR_RETURN(phy_ge_an_get5632(unit, port, &phy_autoneg, &phy_busy));
    if (!phy_autoneg) {
		return(BCM_E_AUTONEG_DISABLED);
    } 
	else if (phy_busy) {
		return(BCM_E_AUTONEG_BUSY);
    }
    return(phy_ge_adv_remote_get5632(unit, port, ability_mask));
}

/*
 * Function: bcm5632_port_ability_get
 * Purpose:  
 *	Retrieve the local port abilities.
 * Parameters:
 *	unit - StrataSwitch Unit #.
 *	port - StrataSwitch port #.
 *	ability_mask - (OUT) Mask of BCM_PORT_ABIL_ values indicating the 
 *		ability of the MAC/PHY.
 * Returns:
 *	BCM_E_NONE
 *	BCM_E_XXX
 */
int
bcm5632_port_ability_get(int unit, soc_port_t port, port_mode_t *ability_mask)
{
    port_mode_t		mac_ability, phy_ability;

    SOC_IF_ERROR_RETURN(phy_ge_ability5632(unit, port, &phy_ability));
    SOC_IF_ERROR_RETURN(mac_ge_ability5632(unit, port, &mac_ability));

    *ability_mask  = mac_ability & phy_ability;
    *ability_mask |= mac_ability & SOC_PM_ABILITY_MAC;
    *ability_mask |= phy_ability & SOC_PM_ABILITY_PHY;
    return(BCM_E_NONE);
}    

/*
 * Function: _bcm_port_update
 * Purpose:  
 *	Get port characteristics from PHY and program MAC to match.  If 
 *	Autoforwarding is enabled, the port is put into forwarding state
 *	when the link is detected. When link is lost, Port is put into 
 *	disabled state.
 * Parameters:
 *	unit - StrataSwitch Unit #.
 *	port - StrataSwitch port #.
 *      link - (OUT) true if link is active, false if link is inactive.
 *		(May be NULL, in which case it is ignored).
 * Returns:
 *	BCM_E_NONE
 *	BCM_E_XXX
 */
static int
_bcm_port_update(int unit, soc_port_t port, int *link)
{
    int		rv;
    int		an, an_done, l_link;	/* local AN/link value */
    int		duplex, speed;

    *link = FALSE;			/* Starting assumption */

    SOC_IF_ERROR_RETURN(phy_ge_link_get5632(unit, port, &l_link));
    SOC_IF_ERROR_RETURN(phy_ge_an_get5632(unit, port, &an, &an_done));
    if (l_link == 0 || (an && !an_done)) { /* No link */
	if (PORT(unit).pc_autoforward & PBMP_PORT(port)) {
	    /*
	     * Only set the port to blocking state if autoforward is enabled
	     * for this port. 
	     */
	    SOC_IF_ERROR_RETURN(_bcm_port_stp_set(unit, port, 
						  BCM_PORT_STP_DISABLED));
	}
	return(an ? BCM_E_AUTONEG_BUSY : BCM_E_NONE);
    } 

    SOC_IF_ERROR_RETURN(phy_ge_duplex_get5632(unit, port, &duplex));
    SOC_IF_ERROR_RETURN(phy_ge_speed_get5632(unit, port, &speed));
    SOC_IF_ERROR_RETURN(mac_ge_speed_set5632(unit, port, speed));
    SOC_IF_ERROR_RETURN(mac_ge_duplex_set5632(unit, port, duplex));

    /*
     * If Autonegotiating, check the negotiated PAUSE values, and program
     * MACs accordingly.
     */
    if (an) {
	port_mode_t	r_advert, l_advert;
	int		tx_pause, rx_pause;

	SOC_IF_ERROR_RETURN(phy_ge_adv_local_get5632(unit, port, &l_advert));
	SOC_IF_ERROR_RETURN(phy_ge_adv_remote_get5632(unit, port, &r_advert));

	tx_pause = (l_advert & SOC_PM_PAUSE_TX) && 
	           (r_advert & SOC_PM_PAUSE_RX);
	rx_pause = (l_advert & SOC_PM_PAUSE_RX) && 
	           (r_advert & SOC_PM_PAUSE_TX);
	SOC_IF_ERROR_RETURN(mac_ge_pause_set5632(unit, port, tx_pause, rx_pause));
    }

    /* Link is active, check if auto-forwarding enabled */

    if (PORT(unit).pc_autoforward & PBMP_PORT(port)) {
	rv = _bcm_port_stp_set(unit, port, BCM_PORT_STP_FORWARDING);
    } else {
	rv = BCM_E_NONE;
    }
    if (link) *link = l_link;

    return(rv);
}

/*
 * Function: bcm5632_port_update
 * Purpose:  
 *	Get port characteristics from PHY and program MAC to match.  If 
 *	Autoforwarding is enabled, the port is put into forwarding state
 *	when the link is detected.
 * Parameters:
 *	unit - StrataSwitch Unit #.
 *	port - StrataSwitch port #.
 *      link - (OUT) true if link is active, false if link is inactive.
 *		(May be NULL, in which case it is ignored).
 * Returns:
 *	BCM_E_NONE
 *	BCM_E_XXX
 */
int
bcm5632_port_update(int unit, soc_port_t port, int *link)
{
    int		rv;

    PORT_INIT(unit);
    rv = _bcm_port_update(unit, port, link);
    return(rv);
}

/*
 * Function: 
 *	bcm5632_port_discard_get
 * Purpose:  
 *	Get port discard attributes for the specified port
 * Parameters:
 *	unit - StrataSwitch Unit #.
 *	port - StrataSwitch port #.
 *      mode - (OUT) Port discard mode, one of BCM_PORT_DISCARD_xxx
 * Returns:
 *	BCM_E_NONE
 *	BCM_E_XXX
 */
int
bcm5632_port_discard_get(int unit, soc_port_t port, int *mode)
{
    uint32  i, data;
    
    PORT_INIT(unit);
   
    /* port discard is in PTABLE of BCM5600, for compatibility reason
       we still use _port_ptable_get() to get the port discard information */ 

    SOC_IF_ERROR_RETURN(cmic_read_reg_5632(unit, reg_addr_5632(P_CONF_1r,port), &data));
    i = (GET_P_CONF_1r_Drop_untag(data)<<1) | (GET_P_CONF_1r_Drop_tag(data));
    switch(i) {
    case 0x00:	*mode = BCM_PORT_DISCARD_NONE;	break;
    case 0x01:	*mode = BCM_PORT_DISCARD_TAG;	break;
    case 0x02:	*mode = BCM_PORT_DISCARD_UNTAG;	break;
    case 0x03:	*mode = BCM_PORT_DISCARD_ALL;	break;
    default:    *mode = -1;
    }
    return(BCM_E_NONE);
}

/*
 * Function: 
 *	bcm5632_port_discard_set
 * Purpose: 
 *	Set port discard attributes for the specified port.
 * Parameters:
 *	unit - StrataSwitch Unit #.
 *	port - StrataSwitch port #.
 *      mode - Port discard mode, one of BCM_PORT_DISCARD_xxx
 * Returns:
 *	BCM_E_NONE
 *	BCM_E_XXX
 */
int
bcm5632_port_discard_set(int unit, soc_port_t port, int mode)
{
    int        rv;
    uint32     tag, untag, data;

    PORT_INIT(unit);

    rv = cmic_read_reg_5632(unit, reg_addr_5632(P_CONF_1r,port), &data);
    if (rv == BCM_E_NONE) {
		switch(mode) {
		case BCM_PORT_DISCARD_NONE:	tag = untag = 0;    break;
		case BCM_PORT_DISCARD_ALL:	tag = untag = 1;    break;
		case BCM_PORT_DISCARD_UNTAG:	tag = 0; untag = 1; break;
		case BCM_PORT_DISCARD_TAG:	tag = 1; untag = 0; break;
		default:
             tag = untag = 0; /* avoid compiling warning */
	    	 rv = BCM_E_PARAM;
			 break;
		}

		if (rv == BCM_E_NONE) {
        	SET_P_CONF_1r_Drop_untag(data, untag);
        	SET_P_CONF_1r_Drop_tag(data, tag);
        	rv = cmic_write_reg_5632(unit, reg_addr_5632(P_CONF_1r, port),data);
		}
    }
    return(rv);
}

/*
 * Function: bcm5632_port_learn_modify
 * Purpose:  Modify the port learn flags, adding add and removing
 *           remove flags.
 *
 * Parameters:
 *	unit - StrataSwitch Unit #.
 *	port - StrataSwitch port #.
 *	add  - Flags to set.
 *      remove - Flags to clear.
 * Returns:
 *	BCM_E_NONE
 *	BCM_E_XXX
 */
int
bcm5632_port_learn_modify(int unit, soc_port_t port, uint32 add, uint32 remove)
{
    uint32 flags;

    SOC_IF_ERROR_RETURN(bcm5632_port_learn_get(unit, port, &flags));
    flags |= add;
    flags &= ~remove;
    SOC_IF_ERROR_RETURN(bcm5632_port_learn_set(unit, port, flags));

    return BCM_E_NONE;
}


/*
 * Function: bcm5632_port_learn_get
 * Purpose:  Get the ARL hardware learning options for this port.
 *		This defines what the hardware will do when a packet
 *		is seen with an unknown address.
 * Parameters:
 *	unit - StrataSwitch Unit #.
 *	port - StrataSwitch port #.
 *	flags - (OUT) Logical OR of BCM_PORT_LEARN_xxx flags
 * Returns:
 *	BCM_E_NONE
 *	BCM_E_XXX
 */
int
bcm5632_port_learn_get(int unit, soc_port_t port, uint32 *flags)
{
    uint32     data;

    PORT_INIT(unit);
 
    SOC_IF_ERROR_RETURN(cmic_read_reg_5632(unit, reg_addr_5632(P_CONF_1r,port), &data));
    data = GET_P_CONF_1r_LrnDis(data);
    /* if data = 1 switch will not learn */
    *flags = data ? 0 : (BCM_PORT_LEARN_SWITCH|BCM_PORT_LEARN_ON);

    return(SOC_E_NONE);
}

/*
 * Function: bcm5632_port_learn_set
 * Purpose:  Set the ARL hardware learning options for this port.
 *		This defines what the hardware will do when a packet
 *		is seen with an unknown address.
 * Parameters:
 *	unit - StrataSwitch Unit #.
 *	port - StrataSwitch port #.
 *	flags - Logical OR of BCM_PORT_LEARN_xxx flags
 * Returns:
 *	BCM_E_NONE
 *	BCM_E_XXX
 */
int
bcm5632_port_learn_set(int unit, soc_port_t port, uint32 flags)
{
    int 		rv = BCM_E_NOT_IMPL;
    uint32     data, data2;

    PORT_INIT(unit);

    /* BCM5632 only has switch learn and switch not learn mode */
    /* either LEARN_ON or LEARN_SWITCH will turn on the learn bit */
    if ((flags & BCM_PORT_LEARN_ON) || (flags & BCM_PORT_LEARN_SWITCH)) {
        data = 0; /* source address will be learned by switch */
    } 
	else {
        data = 1; /* source address will not be learned by switch */
    }

    SOC_IF_ERROR_RETURN(cmic_read_reg_5632(unit, 
						reg_addr_5632(P_CONF_1r,port), &data2));
    SET_P_CONF_1r_LrnDis(data2, data);

    rv = cmic_write_reg_5632(unit, reg_addr_5632(P_CONF_1r, port), data2);
    return(rv);
}

/*
 * Function: bcm5632_port_ifilter_get
 *
 * Description:
 *	Return input filter mode for a port.
 *
 * Parameters:
 *      unit - StrataSwitch PCI device unit number (driver internal).
 *	port - Port number to operate on
 *	mode - (OUT) Filter mode, one of BCM_PORT_IFILTER_xxx
 *
 * Returns:
 *	BCM_E_NONE		Success.
 *	BCM_E_INTERNAL		Chip access failure.
 *
 * Notes:
 *	See also: bcm5632_vlan_ifilter_get
 */
int
bcm5632_port_ifilter_get(int unit, soc_port_t port, int *mode)
{
    uint32	pbmp;
    uint32 	reg_addr = 0;

    reg_addr = reg_addr_5632(P_CONF_1r, port);
    if (cmic_read_reg_5632(unit, reg_addr, &pbmp) < 0) {
       return(BCM_E_INTERNAL);
    }
    if (pbmp & 0x20) { /* bit5 of the register */
	*mode = BCM_PORT_IFILTER_ON;
    } else {
	*mode = BCM_PORT_IFILTER_OFF;
    }
    return(BCM_E_NONE);
}

/*
 * Function: bcm5632_port_ifilter_set
 *
 * Description:
 *	Set input filter mode for a port.
 *
 * Parameters:
 *	unit - StrataSwitch PCI device unit number (driver internal).
 *	port - Port number to operate on
 *	mode - BCM_PORT_IFILTER_ON or BCM_PORT_IFILTER_OFF.
 *
 * Returns:
 *	BCM_E_NONE		Success.
 *	BCM_E_INTERNAL		Chip access failure.
 *
 * Notes:
 *	When input filtering is turned on for a port, packets received
 *	on the port that do not match the port's VLAN classifications
 *	are discarded.
 *	See also: bcm5632_vlan_ifilter_set
 */
int
bcm5632_port_ifilter_set(int unit, soc_port_t port, int mode)
{
    uint32	pbmp;
    int		rv = BCM_E_NONE;
    uint32 	reg_addr = 0; 

    PORT_INIT(unit);
    reg_addr = reg_addr_5632(P_CONF_1r, port);
    if (cmic_read_reg_5632(unit, reg_addr, &pbmp) < 0) {
        return(BCM_E_INTERNAL);
    }
    if (mode) { /* turn on the filter */
       pbmp |= 0x20; /* bit5 of the register */
    } else {
       pbmp &= ~0x20; /* bit5 of the register */
    }

    if (cmic_write_reg_5632(unit, reg_addr, pbmp) < 0) {
       return(BCM_E_INTERNAL);
    }
    return(rv);
}

/*
 * Function:
 *	bcm5632_port_link_status
 * Purpose:
 *	Return current link up/down status
 * Parameters:
 *	unit - StrataSwitch Unit #.
 *	port - StrataSwitch port #.
 *	up - (OUT) Boolean value, FALSE for link down and TRUE for link up
 * Returns:
 *	BCM_E_NONE
 *	BCM_E_XXX
 */
int
bcm5632_port_link_status(int unit, soc_port_t port, int *up)
{
    PORT_INIT(unit);

    return(phy_ge_link_get5632(unit, port, up));
}

/*
 * Function:
 *	bcm5632_port_pfm_set
 * Purpose:
 *	Set current port filtering mode (see port.h)
 * Parameters:
 *	unit - StrataSwitch Unit #.
 *	port - StrataSwitch port #.
 *      mode - mode for PFM bits (see port.h)
 *
 * Returns:
 *	BCM_E_NONE
 *	BCM_E_XXX
 */
int
bcm5632_port_pfm_set(int unit, soc_port_t port, int mode)
{
/* BCM5632 always operate in one mode similiar to Port in Filtering Mode B */
/* To avoid port command return error, we always return none error */
    return(BCM_E_NONE);
}

/*
 * Function:
 *	bcm5632_port_pfm_get
 * Purpose:
 *	Return current port filtering mode (see port.h)
 * Parameters:
 *	unit - StrataSwitch Unit #.
 *	port - StrataSwitch port #.
 *      mode - (OUT) mode read from StrataSwitch for PFM bits (see port.h)
 *
 * Returns:
 *	BCM_E_NONE
 *	BCM_E_XXX
 */
int
bcm5632_port_pfm_get(int unit, soc_port_t port, int *mode)
{
    if (! PBMP_MEMBER(PBMP_E_ALL, port))
		return(BCM_E_PORT);
/* BCM5632 always operate in one mode similiar to Port in Filtering Mode B */
    *mode = 1;
    return(BCM_E_NONE);
}

/*
 * Function:
 *	bcm5632_port_untagged_priority_set
 * Purpose:
 *	Insert 802.3p priority tag for untagged packets coming on
 *      the give port. The priority can be -1 thru 7. If the priority
 *      is -1, the tag is not inserted. Otherwise a tag with the given
 *      priority will be inserted.
 * Parameters:
 *	unit      - StrataSwitch Unit #.
 *	port      - StrataSwitch port #.
 *      priority  - Priority to be set in 802.1p priority tag.
 * Returns:
 *	BCM_E_NONE
 *	BCM_E_XXX
 */
int
bcm5632_port_untagged_priority_set(int unit, soc_port_t port, int priority)
{
    uint32  data;

    SOC_IF_ERROR_RETURN(cmic_read_reg_5632(unit, 
						reg_addr_5632(DFT_PORT_PRIORITY_0r, port),&data));
    SET_DFT_PORT_PRIORITY_0r_Priority(data, priority);
    SOC_IF_ERROR_RETURN(cmic_write_reg_5632(unit, 
						reg_addr_5632(DFT_PORT_PRIORITY_0r, port), data));
    return(SOC_E_NONE);
}

/*
 * Function:
 *	bcm5632_port_untagged_priority_get
 * Purpose:
 *      Returns
 * Parameters:
 *	unit      - StrataSwitch Unit #.
 *	port      - StrataSwitch port #.
 *      priority  - pointer to an int in which priority value is returned.
 *                  if value returned is -1, it means the untagged packets
 *                  are not being prioritized. Otherwise, the returned value
 *                  will be between 0 and 7 inclusive.
 * Returns:
 *	BCM_E_NONE
 *	BCM_E_XXX
 */
int
bcm5632_port_untagged_priority_get(int unit, soc_port_t port, int *priority)
{
    uint32  data;
    SOC_IF_ERROR_RETURN(cmic_read_reg_5632(unit, 
						CMIC_REG(DFT_PORT_PRIORITY_0r+port),&data));
    *priority = GET_DFT_PORT_PRIORITY_0r_Priority(data);
    return(SOC_E_NONE);
}

/*
 * Function:
 *	bcm5632_port_info_get
 *
 * Purpose:
 *	Get all available port parameters at once.
 *
 * Parameters:
 *	unit - StrataSwitch Unit #.
 *	port - StrataSwitch port #.
 *      info - (OUT) Structure to pass the information (see port.h)
 *
 * Returns:
 *	BCM_E_NONE
 *	BCM_E_XXX
 *
 * Notes:
 *	None.
 */
int
bcm5632_port_info_get(int unit, soc_port_t port, bcm_port_info_t *info)
{
    int			r;

    if ((r = bcm5632_port_link_status(unit, port, &info->linkstatus)) < 0) {
	printf("bcm5632_port_link_status failed\n");
	return(r);
    }
    if ((r = bcm5632_port_autoneg_get(unit, port, &info->autoneg)) < 0) {
	printf("bcm5632_port_autoneg_get failed\n");
	return(r);
    }

    if ((r = bcm5632_port_autoforward_get(unit, port, &info->autoforward)) < 0) {
	printf("bcm5632_port_autoforward_get failed\n");
	return(r);
    }

    if ((r = bcm5632_port_speed_get(unit, port, &info->speed)) < 0) {
	if (BCM_E_AUTONEG_BUSY != r) {
	    printf("bcm5632_port_speed_get failed\n");
	    return(r);
	} else {
	    info->speed = 0;
	}
    }

    if ((r = bcm5632_port_duplex_get(unit, port, &info->duplex)) < 0) {
	if (BCM_E_AUTONEG_BUSY != r) {
	    printf("bcm5632_port_duplex_set failed\n");
	    return(r);
	} else {
	    info->duplex = 0;
	}
    }
    if ((r = bcm5632_port_pause_get(unit, port, &info->pause_tx, 
				&info->pause_rx)) < 0) {
	printf("bcm5632_port_pause_get failed\n");
	return(r);
    }

    if ((r = bcm5632_port_learn_get(unit, port, &info->learn)) < 0) {
	printf("bcm5632_port_learn_get failed\n");
	return(r);
    }

    if ((r = bcm5632_port_discard_get(unit, port, &info->discard)) < 0) {
	printf("bcm5632_port_discard_get failed\n");
	return(r);
    }

    if ((r = bcm5632_port_ifilter_get(unit, port, &info->vlanfilter)) < 0) {
	printf("bcm5632_port_ifilter_get failed\n");
	return(r);
    }
    if ((r = bcm5632_port_untagged_priority_get(unit, port,
					    &info->untagged_priority)) < 0) {
	printf("bcm5632_port_untagged_priority_get failed\n");
	return(r);
    }
    if ((r = bcm5632_port_stp_get(unit, port, &info->stp_state)) < 0) {
	printf("bcm5632_port_stp_get failed\n");
	return(r);
    }

    if ((r = bcm5632_port_pfm_get(unit, port, &info->pfm)) < 0) {
	printf("bcm5632_port_pfm_get failed\n");
	return(r);
    }
    if ((r = bcm5632_port_loopback_get(unit, port, &info->loopback)) < 0) {
	printf("bcm5632_port_loopback_get failed\n");
	return(r);
    }
    return BCM_E_NONE;
}

/*
 * Function:
 *	bcm5632_port_info_set
 *
 * Purpose:
 *	Set all available port parameters at once.
 *
 * Parameters:
 *	unit - StrataSwitch Unit #.
 *	port - StrataSwitch port #.
 *      info - Structure to pass the information (see port.h)
 *
 * Returns:
 *	BCM_E_NONE
 *	BCM_E_XXX
 *
 * Notes:
 *	Does not set linkstatus.
 */

int
bcm5632_port_info_set(int unit, soc_port_t port, bcm_port_info_t *info)
{
    int			r;

    if ((r = bcm5632_port_pause_addr_set(unit, port, info->pause_mac)) < 0) {
	printf("bcm5632_port_pause_addr_set failed\n");
	return(r);
    }
    
    if ((r = bcm5632_port_autoforward_set(unit, port, info->autoforward)) < 0) {
	printf("bcm5632_port_autoforward_set failed\n");
	return(r);
    }

    if ((r = bcm5632_port_linkscan_set(unit, port, info->linkscan)) < 0) {
	printf("bcm5632_port_linkscan_set failed\n");
	return(r);
    }

    if ((r = bcm5632_port_learn_set(unit, port, info->learn)) < 0) {
	printf("bcm5632_port_learn_set failed\n");
	return(r);
    }

    if ((r = bcm5632_port_discard_set(unit, port, info->discard)) < 0) {
	printf("bcm5632_port_discard_set failed\n");
	return(r);
    }

    if ((r = bcm5632_port_ifilter_set(unit, port, info->vlanfilter)) < 0) {
	printf("bcm5632_port_ifilter_set failed\n");
	return(r);
    }

    if ((r = bcm5632_port_untagged_priority_set(unit, port,
					    info->untagged_priority)) < 0) {
	printf("bcm5632_port_untagged_priority_set failed\n");
	return(r);
    }

    if ((r = bcm5632_port_stp_set(unit, port, info->stp_state)) < 0) {
	printf("bcm5632_port_stp_set failed\n");
	return(r);
    }

    if ((r = bcm5632_port_pfm_set(unit, port, info->pfm)) < 0) {
	printf("bcm5632_port_pfm_set failed\n");
	return(r);
    }

    if ((r = bcm5632_port_loopback_set(unit, port, info->loopback)) < 0) {
	printf("bcm5632_port_loopback_set failed\n");
	return(r);
    }
    if ((r = bcm5632_port_autoneg_set(unit, port, FALSE)) < 0) {
	printf("bcm5632_port_autoneg_set failed\n");
	return(r);
    }
    if (info->autoneg) {
	port_mode_t	mode;
	/*
	 * Set link sdvertisments from speed/duplex/pause.
	 */
	switch(info->speed) {
	case 0:				/* Set to highest value */
	    BCM_IF_ERROR_RETURN(bcm5632_port_ability_get(unit,  port, &mode));
	    mode &= BCM_PORT_ABIL_10MB_HD | BCM_PORT_ABIL_10MB_FD | 
		    BCM_PORT_ABIL_100MB_HD | BCM_PORT_ABIL_100MB_FD | 
		    BCM_PORT_ABIL_1000MB_HD | BCM_PORT_ABIL_1000MB_FD;
	    break;
	case 10:
	    if (info->duplex) {
		mode = BCM_PORT_ABIL_10MB_FD | BCM_PORT_ABIL_10MB_HD;
	    } else {
		mode = BCM_PORT_ABIL_10MB_HD;
	    }
	    break;
	case 100:
	    if (info->duplex) {
		mode = BCM_PORT_ABIL_100MB_FD | BCM_PORT_ABIL_100MB_HD;
	    } else {
		mode = BCM_PORT_ABIL_100MB_HD;
	    }
	    mode |= BCM_PORT_ABIL_10MB_HD | BCM_PORT_ABIL_10MB_FD;
	    break;
	case 1000:
	    if (info->duplex) {
		mode = BCM_PORT_ABIL_1000MB_FD | BCM_PORT_ABIL_1000MB_HD;
	    } else {
		mode = BCM_PORT_ABIL_1000MB_HD;
	    }
	    mode |= BCM_PORT_ABIL_10MB_HD | BCM_PORT_ABIL_10MB_FD;
	    mode |= BCM_PORT_ABIL_100MB_HD | BCM_PORT_ABIL_100MB_FD;
	    break;
	default:
	    printf("bcm5632_port_info_set: Invalid speed: %d\n", info->speed);
	    return(BCM_E_PARAM);
	}
	if (info->pause_tx) mode |= BCM_PORT_ABIL_PAUSE_TX;
	if (info->pause_rx) mode |= BCM_PORT_ABIL_PAUSE_RX;

	if ((r = bcm5632_port_advert_set(unit, port, mode)) < 0) {
	    printf("bcm5632_port_advert_set failed: (0x%x)\n", mode);
	    return(r);
	}
	if ((r = bcm5632_port_autoneg_set(unit, port, info->autoneg)) < 0) {
	    printf("bcm5632_port_autoneg_set failed\n");
	    return(r);
	}

    } else {
	/*
	 * If auto-negotiation not set, force the speeds.
	 */
	if ((r = bcm5632_port_speed_set(unit, port, info->speed)) < 0) {
	    printf("bcm5632_port_speed_set failed\n");
	    return(r);
	}

	if ((r = bcm5632_port_duplex_set(unit, port, info->duplex)) < 0) {
	    printf("bcm5632_port_duplex_set failed\n");
	    return(r);
	}

	if ((r = bcm5632_port_pause_set(unit, port, info->pause_tx, 
				    info->pause_rx)) < 0) {
	    printf("bcm5632_port_pause_set failed\n");
	    return(r);
	}
    }

    return(BCM_E_NONE);
}

/*
 * Function:
 *	bcm5632_ptable_update_vlan_add
 * Purpose:
 *      Ports have been added to a VLAN.  Update the PTABLE with the
 *      new information for all ports.
 * Parameters:
 *	unit - StrataSwitch unit number.
 *	port - StrataSwitch port number.
 *	vid  - The VLAN id being changed.
 *      pbmp, ubmp  - bit maps of ports effected.
 *
 * Returns:
 *	BCM_E_NONE
 *	BCM_E_INTERNAL - internal error.
 */
int
bcm5632_ptable_update_vlan_add(int unit, vlan_id_t vid,
                           pbmp_t pbmp, pbmp_t ubmp)
{
    int rv;

    PORT_INIT(unit);
    rv = _bcm_ptable_update_vlan_add(unit, vid, pbmp, ubmp);
    return rv;
}

/*
 * Function:
 *	_bcm_ptable_update_vlan_add
 *      Guts of bcm_ptable_update_vlan_add.  See above.
 *
 *      Assumes locking at higher level.
 *
 */
static int
_bcm_ptable_update_vlan_add(int unit, vlan_id_t vid,
                           pbmp_t pbmp, pbmp_t ubmp)
{
    soc_port_t port;
    ptable_entry_t	pv;

    PBMP_ITER(PBMP_ALL, port) {
        SOC_IF_ERROR_RETURN(_port_ptable_get(unit, port, &pv));
        if (pv.vlan_tag == vid) {
            pv.port_bitmap |= pbmp;
            pv.ut_port_bitmap &= ~pbmp;
            pv.ut_port_bitmap |= ubmp;
            SOC_IF_ERROR_RETURN(_port_ptable_set(unit, port, &pv));
        }
    }

    return(SOC_E_NONE);
}

int 
bcm5632_port_reset(int unit, soc_port_t port)
{
  return(mac_ge_init5632(unit, port));
}
