/*
 *=============================================================================
 * File Name   :    bcm5632-port.h
 *
 *  RCS: $Id: bcm5632-port.h,v 1.1.1.1 2001/12/22 04:14:58 tonyw Exp $
 *
 *  Description:
 *
 *  Created    :    09-04-2001
 *
 *  Author     :    Tony Wong
 *
 *  Copyright(c)  Agile Storage, Inc. 2001
 *=============================================================================
 */

#ifndef BCM5632_PORT_H
#define BCM5632_PORT_H

typedef uint32 port_mode_t;

typedef uchar8 mac_addr_t[6];

typedef struct vlan_data_s {
    vlan_id_t vlan_tag;
    pbmp_t port_bitmap;
    pbmp_t ut_port_bitmap;
} vlan_data_t;

/*
 * Ability mask
 *
 *   The following flags are used to indicate which set of capabilities
 *   are provided by a PHY when retrieving the ability of a port,
 *   setting or getting the local advertisement, or getting the remote
 *   advertisement.
 */

#define	BCM_PORT_ABIL_10MB_HD	SOC_PM_10MB_HD
#define	BCM_PORT_ABIL_10MB_FD	SOC_PM_10MB_FD
#define BCM_PORT_ABIL_100MB_HD	SOC_PM_100MB_HD
#define BCM_PORT_ABIL_100MB_FD	SOC_PM_100MB_FD
#define	BCM_PORT_ABIL_1000MB_HD	SOC_PM_1000MB_HD
#define	BCM_PORT_ABIL_1000MB_FD	SOC_PM_1000MB_FD
#define	BCM_PORT_ABIL_PAUSE_TX	SOC_PM_PAUSE_TX
#define	BCM_PORT_ABIL_PAUSE_RX	SOC_PM_PAUSE_RX
#define	BCM_PORT_ABIL_MII	SOC_PM_MII
#define	BCM_PORT_ABIL_10B	SOC_PM_10B
#define	BCM_PORT_ABIL_LB_MAC	SOC_PM_LB_MAC
#define	BCM_PORT_ABIL_LB_PHY	SOC_PM_LB_PHY

/* Module one-time initialization routine */

extern int bcm5632_port_init(int unit);

/* General routines on which most port routines are built */

extern int bcm5632_port_enable_set(int unit, soc_port_t port, int enable);
extern int bcm5632_port_enable_get(int unit, soc_port_t port, int *enable);

extern int bcm5632_port_egress_enable_set(int unit, soc_port_t port, int enable);
extern int bcm5632_port_egress_enable_get(int unit, soc_port_t port, int *enable);
extern int bcm5632_port_advert_set(int unit, soc_port_t port, 
			       port_mode_t ability_mask);
extern int bcm5632_port_advert_get(int unit, soc_port_t port,
			       port_mode_t *ability_mask);
extern int bcm5632_port_remote_advert_get(int unit, soc_port_t port, 
			       port_mode_t *ability_mask);
extern int bcm5632_port_ability_get(int unit, soc_port_t port, 
				port_mode_t *local_ability_mask);
extern int bcm5632_ptable_init(int unit, vlan_data_t *vdata);

/* pvlan functions */
extern int bcm5632_port_vlan_set(int unit, pbmp_t ports, vlan_id_t vid);
extern int bcm5632_port_vlan_get(int unit, soc_port_t port, vlan_data_t *vdata);
extern int bcm5632_port_vlan_disable(int unit, pbmp_t ports);
extern int bcm5632_port_vlan_update_bitmaps(int unit, pbmp_t ports);

extern int bcm5632_ptable_update_vlan_destroy(int unit, vlan_id_t vid);



#define	BCM_PORT_LINKSCAN_NONE	0
#define BCM_PORT_LINKSCAN_SW	1
#define	BCM_PORT_LINKSCAN_HW	2

extern int bcm5632_port_reset(int unit, soc_port_t port);

extern int bcm5632_port_linkscan_get(int unit, soc_port_t port, int *linkscan);
extern int bcm5632_port_linkscan_set(int unit, soc_port_t port, int linkscan);

extern int bcm5632_port_autoneg_get(int unit, soc_port_t port, int *autoneg);
extern int bcm5632_port_autoneg_set(int unit, soc_port_t port, int autoneg);

extern int bcm5632_port_autoforward_get(int unit, soc_port_t port, int *autofwd);
extern int bcm5632_port_autoforward_set(int unit, soc_port_t port, int autofwd);

extern int bcm5632_port_speed_get(int unit, soc_port_t port, int *speed);
extern int bcm5632_port_speed_set(int unit, soc_port_t port, int speed);

#define BCM_PORT_DUPLEX_HALF	0
#define BCM_PORT_DUPLEX_FULL	1

extern int bcm5632_port_duplex_get(int unit, soc_port_t port, int *duplex);
extern int bcm5632_port_duplex_set(int unit, soc_port_t port, int duplex);

extern int bcm5632_port_pause_get(int unit, soc_port_t port, 
			      int *pause_tx, int *pause_rx);
extern int bcm5632_port_pause_set(int unit, soc_port_t port, 
			      int pause_tx, int pause_rx);
extern int bcm5632_port_pause_addr_get(int unit, soc_port_t port, mac_addr_t);
extern int bcm5632_port_pause_addr_set(int unit, soc_port_t port, mac_addr_t);

extern int bcm5632_port_update(int unit, soc_port_t port, int *link);

/*
 * Flags used for loopback modes
 */

#define BCM_PORT_LOOPBACK_NONE		0
#define BCM_PORT_LOOPBACK_MAC		1
#define BCM_PORT_LOOPBACK_PHY		3

extern int bcm5632_port_loopback_set(int unit, soc_port_t port, int loopback);
extern int bcm5632_port_loopback_get(int unit, soc_port_t port, int *loopback);

/*
 * Port Spanning Tree States used in bcm_port_state_get() and set()
 */

#define BCM_PORT_STP_DISABLED     0x00	/* Disabled */
#define BCM_PORT_STP_LISTENING    0x01	/* BPDUs and NO Learning */
#define BCM_PORT_STP_BLOCKING     0x02	/* BPDUs Only, No learning */
#define BCM_PORT_STP_LEARNING     0x03	/* Learning Only, no forwarding */
#define BCM_PORT_STP_FORWARDING   0x04 	/* Normal operation */

extern int bcm5632_port_stp_get(int unit, soc_port_t port, int *state);
extern int bcm5632_port_stp_set(int unit, soc_port_t port, int state);

/*
 * Modes for discard
 */

#define BCM_PORT_DISCARD_NONE	0
#define BCM_PORT_DISCARD_ALL	1
#define BCM_PORT_DISCARD_UNTAG	2
#define BCM_PORT_DISCARD_TAG	3

extern int bcm5632_port_discard_get(int unit, soc_port_t port, int *mode);
extern int bcm5632_port_discard_set(int unit, soc_port_t port, int mode);

/*
 * Flags for learn
 *
 * This call takes boolean flags to turn on and off actions that should
 * be taken when a packet is received with an unknown address.  The set
 * call returns BCM_E_NOT_IMPL for flag combinations that are not
 * supported by the hardware.
 */

#define BCM_PORT_LEARN_ON		0x01
#define BCM_PORT_LEARN_CPU		0x02
#define BCM_PORT_LEARN_SWITCH		0x04

extern int bcm5632_port_learn_get(int unit, soc_port_t port, uint32 *flags);
extern int bcm5632_port_learn_set(int unit, soc_port_t port, uint32 flags);
int bcm5632_port_learn_modify(int unit, soc_port_t port,
                          uint32 add, uint32 remove);

extern int bcm5632_port_link_status(int unit, soc_port_t port, int *up);

/*
 * Modes for VLAN input filtering
 * (see also VLAN-based ifilter setting in vlan.h)
 */

#define BCM_PORT_IFILTER_OFF	0
#define BCM_PORT_IFILTER_ON	1

extern int bcm5632_port_ifilter_get(int unit, soc_port_t port, int *mode);
extern int bcm5632_port_ifilter_set(int unit, soc_port_t port, int mode);

/*
 * The FFP is used to set the default port priority
 */

extern int bcm5632_port_untagged_priority_set(int unit, soc_port_t port, 
					  int priority);

extern int bcm5632_port_untagged_priority_get(int unit, soc_port_t port, 
					  int* priority);

extern int bcm5632_port_enable_bpdu(int unit, soc_port_t port, int enable);

/*
 * Port Filter Modes
 */

#define	BCM_PORT_PFM_MODEA	0x00	/* DLF all MCAST packets */
#define	BCM_PORT_PFM_MODEB	0x01	/* DLF MCAST packets not in MARL */
#define	BCM_PORT_PFM_MODEC	0x02	/* Drop MCAST packets not in MARL */

extern int bcm5632_port_pfm_set(int unit, soc_port_t port, int mode);
extern int bcm5632_port_pfm_get(int unit, soc_port_t port, int *mode);

/*
 * Routines to set or get ALL available port parameters in one call.
 */

typedef struct bcm_port_info_s {
    int 	linkstatus;
    int		autoneg;
    int		autoforward;
    int		speed;
    int		duplex;
    int		pause_tx;
    int		pause_rx;
    mac_addr_t	pause_mac;
    int		linkscan;
    int		learn;
    int		discard;
    int		vlanfilter;
    int		untagged_priority;
    int		stp_state;
    int		pfm;
    int		loopback;
} bcm_port_info_t;

extern int bcm5632_port_info_get(int unit, soc_port_t port, bcm_port_info_t *info);
extern int bcm5632_port_info_set(int unit, soc_port_t port, bcm_port_info_t *info);
extern char *bcm5632_port_get_phy_drv_name(int unit, soc_port_t port);

#endif	/* BCM5632_PORT_H */
