/*  
 *=============================================================================
 * File Name   :    bmc12500-eth-tx.c
 *
 *  Description:    
 *
 *  Created    :    23 October 2001
 *
 *  Author     :    Frank Salzmann
 *
 *  RCS        :    $Id: bmc12500-eth-tx.c,v 1.68 2002/12/07 01:04:51 tonyw Exp $
 *
 *  Copyright(c)  Agile Storage, Inc. 2001
 *=============================================================================
 */

#include <nfx-incl.h>
#include "bmc12500-eth.h"
#include "bmc12500-eth-ip.h"
#include <exports/mem-api.h>
#include <exports/malloc-api.h>
#include <sys/sb1250/sb1250_regs.h>
#include <exports/cm-api.h>

#ifndef SIBYTE_BC
#define LMUX_DEBUG
#endif
#define LUC_DEBUG

uint32 lmux_txPkts[6] = { 0,0,0,0,0,0 };
uint32 lmux_txBytes[6] = { 0,0,0,0,0,0 };
uint32 luc_txPkts[4] = { 0,0,0,0 };
uint32 luc_txBytes[4] = { 0,0,0,0 };
uint32 bmc12500Eth_tx_pkt_trace = 0;
#ifndef PROM
extern uint32 sysDVTrunning;
extern uint32 dvtFirstPkt;
#endif
extern uint32 lmux_loopback_test;
extern uint32 luc_loopback_test;

char broadcast_addr[] = {0xff, 0xff, 0xff, 0xff, 0xff, 0xff,};
/* 
 * LSI port assignment based on destination SP-NIM slice(0-3), data/ctrl(0-1)
 * uint32 lmux_lsi_port[4][2] = { {0,2}, {1,2}, {3,5}, {4,5} };
 * the lmux_lsi_port[CPUID][DTYPE] is accessed via FC CPUID (NOT slice#)  
 */
uint32 lmux_lsi_port[4] = { 4,3,1,0 }; /* control plane is obsolete */

int get_lmux_lsi_mac_port(uint32 dest);
int bmc12500_is_oldPass();
int bmc12500Eth_addEeeHdrs(eee_desc_t*, uint32, uint32, bc1250EthCfg_t*);
int bmc12500Eth_txLinkPoll_drr(); 
int bmc12500Eth_txLinkPoll_rr(); 
int bmc1250_txPktCapture(char*, uint32, int32);
int bmc12500Eth_txCheck(eee_desc_t *);

extern int bp_setDestMacAddr(uint32, uint32, char*); 
extern void eee_descDisplay(eee_descPtr_t);
extern void dp_proxy_tx_cleanup(uchar8*);
extern int bmc12500Eth_printBuf(char*, int, int);
extern int luc_simQueueXmtCmpl(eee_desc_t*, uint32);

extern unsigned char bmc12500_lsi_macAddr[6][6];
extern uint32 bc1250_tx_opt[3];

#ifdef SPANS_PHYS_BLOCKS
extern int32 spans_dram_blocks( address_t       start, uint32          len);
#endif
extern void *   memcpy6(void *dst, const void *src, __SIZE_TYPE__ nbytes);

extern bobcat_test_pkt;

/*
 *------------------------------------------------------------------------------
 * Function Name:	bmc12500Eth_txUnicast
 *
 * Description: 	Transmits a unicast packet to the backplane.
 *
 * Created by:		Frank Salzmann
 *
 * Data Created:	October 23, 2001
 * 					
 * Returns: ?????
 *
 *------------------------------------------------------------------------------
 */
int
bmc12500Eth_txUnicast( uint32 dest, eee_desc_t *pktDesc )
{
	int i, priQ;
	int bufCnt;
	int pktLen;
	uint32 dest_slot;
	uint32 dest_cpu;
	bc1250EthCfg_t  *cfg;
	bc1250Eth_txCtrl_t        *txc;
	volatile bc1250_txdesc_t  *txCur;
	volatile bc1250Eth_regs_t *regs;

	//printf("bmc12500Eth_txunicast: enter, dest: %x\n", dest);
	//eee_descDisplay(pktDesc);
	
	//-- Figure out which ethernet interface to use. Based on 
	//-- the port component of the 'dest' function parameter.
	BC1250ETH_GET_TX_PORT(dest, cfg);

	//printf("port addr: %d\n", EEE_GET_PORT_ID(dest));
	if( cfg == 0 ) {
		printf("Invalid port addr: %d\n", EEE_GET_PORT_ID(dest));
		return( EEE_DROP_PACKET );
	}

	priQ = 0;	

	dest_slot = EEE_GET_SLOT_NUM(dest);
	dest_cpu = EEE_GET_DEST_CPU(dest);

#ifdef PROM
#ifdef SIBYTE_BC	
	if( isFP() && (!bobcat_test_pkt))
#else			
	if( isFP() && (!(lmux_loopback_test)) ) 
#endif			
	{
		bmc12500Eth_addEeeHdrs(pktDesc, dest_slot, dest_cpu, cfg);
	}	
#endif
	//-- Look at edesc
	//eee_descDisplay(pktDesc);

	regs  = &(cfg->regs);
	txc   = &(cfg->txc[priQ]);

	txCur = txc->txCur;
	pktLen = pktDesc->hdr.control & EDESC_TOTAL_LEN;
	bufCnt = (pktDesc->hdr.control & EDESC_BUFCNT_MASK) >> EDESC_BUFCNT_SHIFT;

	/* 
	 * Check for room on the transmit queue.
     	 * Make sure there is at least one free entry left so tx cleanup
    	 * pointer won't equal tx current pointer when queue is full. 
	 */
	if( bufCnt > (BC1250_MAC_TX_Q_LEN_MINUS_ONE - txc->qd)  ) {
		//-- No room on the tx queue.
		printf("Txq Full: bufCnt: %d, desc qd: %d, ", bufCnt, txc->qd);
		printf("dma_desc_cnt: %llx\n", *(regs->dma_cur_dscr_addr_tx[priQ]));	
		txc->qFull++;
		return( EEE_DROP_PACKET );
	}

	/* Update pkt Stats */
	txc->pkts++;
	txc->octets += pktLen;

#if defined(LMUX_DEBUG) || defined(LUC_DEBUG)
	if (isCheetah()) {
		if (isFP()) {
			if (lmux_loopback_test) {
				i = pktDesc->bd[1].buf[3];
				if (cfg->macId != 0)
					i += 3;
#ifdef NFX_SMP
			   	txc->lmux_txPkts[i] += 1;
				txc->lmux_txBytes[i] += pktLen;
#else
			   	lmux_txPkts[i] += 1;
				lmux_txBytes[i] += pktLen;
#endif
			}
			else {
			    i = get_lmux_lsi_mac_port(pktDesc->hdr.dest_port);
#ifdef NFX_SMP
				txc->lmux_txPkts[i] += 1;
				txc->lmux_txBytes[i] += pktLen;
#else
				lmux_txPkts[i] += 1;
				lmux_txBytes[i] += pktLen;
#endif
			}
		}
		else {
			i = (pktDesc->bd[1].buf[4] >> 4) & 0x03;
			luc_txPkts[i] += 1;
			luc_txBytes[i] += pktLen;
		}
	}
#endif

	//-- Mark first descriptor of a packet.
	txCur->dw1 = BC1250_DESC_STATUS_SOP; 

	if( bufCnt == 1 ) {

		if( bmc12500Eth_tx_pkt_trace ) {
			printf("Chip Tx Desc Cnt: %lld\n", *regs->dma_dscr_cnt_tx[0]);
			printf("pktDesc: %lx, tx buf cnt: %d, buf ptr: %lx, buf len: %d\n", 
				   (address_t)pktDesc, bufCnt,(address_t)pktDesc->bd[1].buf,
					pktDesc->bd[1].len);

			bmc12500Eth_printBuf((char *)(pktDesc->bd[1].buf), 
								pktDesc->bd[1].len, 1); 
		}
#ifdef SPANS_PHYS_BLOCKS
        if (mm_isMallocBuffer(pktDesc->bd[1].buf) &&
            (spans_dram_blocks((address_t)pktDesc->bd[1].buf, pktLen))) {
            uchar8  * tbuf;
            printf("bmc tx, malloc buffer spans physical block boundary\n");
            printf("converting buf at %p\n", pktDesc->bd[1].buf);
            
            NFX_ASSERT(FALSE);
    
            tbuf = eee_allocateBuffer(EEE_SHARED_POOL, EEE_BUF_ULTRA, NFX_COS_HIGH);
            memcpy(tbuf, pktDesc->bd[1].buf, pktLen);
            //-- Set buf pointer, buf size and SOP bit.
            BC1250_DESC_SET_WD_A_TX(txCur,
                                    (uint64)tbuf,
                                    EEE_BUFSIZE_ULTRA/32ll,
                                    BC1250_DESC_STATUS_SOP);
        }
        else {
#endif
        //-- Set buf pointer, buf size and SOP bit.
		BC1250_DESC_SET_WD_A_TX(txCur, 
								(uint64)pktDesc->bd[1].buf,
								EEE_BUFSIZE_ULTRA/32ll,
								BC1250_DESC_STATUS_SOP);
#ifdef SPANS_PHYS_BLOCKS
        }
#endif
		//-- Set packet size and tx option.
		BC1250_DESC_SET_WD_B_TX(txCur, (uint64)pktLen, bc1250_tx_opt[cfg->macId]);

#if BC1250ETH_TXQ_LIST
		//-- Stash pointer to edesc.
		txCur->edesc = (uint64)pktDesc;
#endif
		txc->qd++;

#if BC1250_TX_PKT_CAPTURE
		if( bc1250_pcb[cfg->macId].txCapEnabled )
			bmc1250_txPktCapture(pktDesc->bd[1].buf, pktDesc->bd[1].len, cfg->macId );
#endif

		if( ++txCur >= txc->txqEnd )
			txCur = txc->txqStart;

		*(regs->dma_dscr_cnt_tx[priQ]) = 1;	// Give tx desc to MAC.
	}
	else {
		uint64 firstLen;
		uint32 j = bufCnt-1;
    	uint32 idx, k, offset;

#if 0
		if( bmc12500Eth_txCheck(pktDesc) != NFX_OK )
			return EEE_DROP_PACKET;
#else
//////////////////////////////////////////////////////////////////////
// bmc12500Eth_txCheck

    if( bmc12500Eth_tx_pkt_trace ) {
        printf("bmc12500 tx check: buf Cnt: %d\n", bufCnt);
        eee_descDisplay(pktDesc);
    }

    if( pktDesc->hdr.control & EDESC_COMMAND_USE_FIRST_BUFFER )
        i=0;
    else
        i=1;

    offset = (address_t)pktDesc->bd[i].buf & 0x1f; // offset

    //-- First Buffer needs end on a cache line boundry.
    if( (address_t)((pktDesc->bd[i].buf) + (pktDesc->bd[i].len)) & 0x1F ) {
        printf("BCM12500: Tx Buf End Not Aligned Error: Buf: %lx, Len: %d\n",
               (address_t)pktDesc->bd[i].buf, pktDesc->bd[i].len);
        eee_descDisplay(pktDesc);
        return( EEE_DROP_PACKET );
    }

    if( (pktDesc->bd[i].len + offset) % 64 ) {
        printf("BCM12500: Tx Buf Len+Offset Not Modulo 64: Buf: %lx, Len: %d, Offset: %d\n",
               (address_t)pktDesc->bd[i].buf, pktDesc->bd[i].len, offset);
        eee_descDisplay(pktDesc);
        return( EEE_DROP_PACKET );
    }

    //-- Make sure buffers passed down are aligned on 64 byte boundry.
    idx = i+1; // Move one past the first buffer.
    for(k=1; k < bufCnt; idx++, k++ )  {
        if( (address_t)(pktDesc->bd[idx].buf) & 0x3f ) {
            printf("** BCM12500: Tx Buf Not aligned error: %lx, idx=%d\n",
                    (address_t)pktDesc->bd[idx].buf, idx);
            return( EEE_DROP_PACKET );
        }
    }

    if( bmc12500Eth_tx_pkt_trace ) {
        printf("bmc12500 tx check: pkt buf ok\n");
    }

///////////////////////////////////////////////////////////////////
#endif

		//-----------------------------------------------------
		//-- Setup descriptor for first buffer in the packet.
		//-----------------------------------------------------

		if( bmc12500Eth_tx_pkt_trace ) {
			printf("pktDesc: %lx, tx buf: %d, buf ptr: %lx, buf len: %d\n", 
				   (address_t)pktDesc, j, (address_t)pktDesc->bd[i].buf, 
					pktDesc->bd[i].len);

			bmc12500Eth_printBuf((char *)(pktDesc->bd[i].buf), 
					pktDesc->bd[i].len, 1); 
		}

#if BC1250_TX_PKT_CAPTURE
		if( bc1250_pcb[cfg->macId].txCapEnabled )
			bmc1250_txPktCapture(pktDesc->bd[i].buf, pktDesc->bd[i].len, cfg->macId );
#endif

		//-- Calculate length of first buffer in units of cacheline
		//-- size. Take into account the fact that the data start
		//-- may be offset from the start of the buffer.
		//		firstLen = ((uint64)pktDesc->bd[i].buf & 0x1F);
	    firstLen = pktDesc->bd[i].len + offset;
		firstLen = firstLen / 32ll;

#ifdef SPANS_PHYS_BLOCKS
        if (mm_isMallocBuffer(pktDesc->bd[i].buf) &&
            (spans_dram_blocks((address_t)pktDesc->bd[i].buf, firstLen))) {
            uchar8  * tbuf;
            printf("bmc tx, malloc buffer spans physical block boundary\n");
            printf("converting buf at %p\n", pktDesc->bd[i].buf);
    
            NFX_ASSERT(FALSE);
    
            tbuf = eee_allocateBuffer(EEE_SHARED_POOL, EEE_BUF_ULTRA, NFX_COS_HIGH);
            memcpy(tbuf, pktDesc->bd[i].buf, firstLen);
            //-- Set buf pointer, buf size and SOP bit.
            BC1250_DESC_SET_WD_A_TX(txCur, 
                                    (uint64)tbuf,
                                    firstLen,
                                    BC1250_DESC_STATUS_SOP);
        }
        else {
#endif
		//-- Set buf pointer, buf size and SOP bit.
		BC1250_DESC_SET_WD_A_TX(txCur, 
								(uint64)pktDesc->bd[i].buf,
								firstLen,
								BC1250_DESC_STATUS_SOP);
#ifdef SPANS_PHYS_BLOCKS
        }
#endif
		//-- Set packet size and tx option.
		BC1250_DESC_SET_WD_B_TX(txCur, (uint64)pktLen, bc1250_tx_opt[cfg->macId]);

		if( bmc12500Eth_tx_pkt_trace ) {
			printf("wd1: %llx\n wd2: %llx",txCur->dw1, txCur->dw2);
		}	

		txc->qd++;
		i++;
		if( ++txCur >= txc->txqEnd )
			txCur = txc->txqStart;

		while(j--) {

			if( bmc12500Eth_tx_pkt_trace ) {
				printf("tx buf: %d, buf ptr: %lx, buf len: %d\n", 
					   j, (address_t)pktDesc->bd[i].buf, pktDesc->bd[i].len);

				bmc12500Eth_printBuf((char *)(pktDesc->bd[i].buf), 
						pktDesc->bd[i].len, 0); 
			}


			//-- Set buf pointer, buf size and SOP bit.
#if BC1250ETH_TXQ_LIST
			if( j == 0 ) {
#ifdef SPANS_PHYS_BLOCKS
                if (mm_isMallocBuffer(pktDesc->bd[i].buf) &&
                    (spans_dram_blocks((address_t)pktDesc->bd[i].buf, pktDesc->bd[i].len))) {
                    uchar8  * tbuf;
                    printf("bmc tx, malloc buffer spans physical block boundary\n");
                    printf("converting buf at %p\n", pktDesc->bd[i].buf);
    
                    NFX_ASSERT(FALSE);
    
                    tbuf = eee_allocateBuffer(EEE_SHARED_POOL, EEE_BUF_ULTRA, NFX_COS_HIGH);
                    memcpy(tbuf, pktDesc->bd[i].buf, pktDesc->bd[i].len);
                    BC1250_DESC_SET_WD_A_TX(txCur, 
                                            (uint64)tbuf,
                                            //BC1250_DESC_BUF_SIZE,
                                            EEE_BUFSIZE_ULTRA/32ll,
                                            NULL);
                }
                else {
#endif
			    BC1250_DESC_SET_WD_A_TX(txCur, 
										(uint64)pktDesc->bd[i].buf,
										//BC1250_DESC_BUF_SIZE,
										EEE_BUFSIZE_ULTRA/32ll,
										NULL);
#ifdef SPANS_PHYS_BLOCKS
                }
#endif
				//-- Stash pointer to edesc in last desc of packet.
				txCur->edesc = (uint64)pktDesc;
			}
			else {
#ifdef SPANS_PHYS_BLOCKS
                if (mm_isMallocBuffer(pktDesc->bd[i].buf) &&
                    (spans_dram_blocks((address_t)pktDesc->bd[i].buf, pktDesc->bd[i].len))) {
                    uchar8  * tbuf;
                    printf("bmc tx, malloc buffer spans physical block boundary\n");
                    printf("converting buf at %p\n", pktDesc->bd[i].buf);
    
                    NFX_ASSERT(FALSE);
    
                    tbuf = eee_allocateBuffer(EEE_SHARED_POOL, EEE_BUF_ULTRA, NFX_COS_HIGH);
                    memcpy(tbuf, pktDesc->bd[i].buf, pktDesc->bd[i].len);
                    BC1250_DESC_SET_WD_A_TX(txCur, 
                                            (uint64)tbuf,
                                            (uint64)pktDesc->bd[i].len / 32ll,
                                            NULL);
                }
                else {
#endif
				BC1250_DESC_SET_WD_A_TX(txCur, 
										(uint64)pktDesc->bd[i].buf,
										(uint64)pktDesc->bd[i].len / 32ll,
										NULL);
#ifdef SPANS_PHYS_BLOCKS
                }
#endif
            }
#endif 
			i++;
			txc->qd++;

			if( ++txCur >= txc->txqEnd )
				txCur = txc->txqStart;
		}

		*(regs->dma_dscr_cnt_tx[priQ]) = bufCnt;	// Give pkt descs to MAC.
	}

	txc->txCur  = (bc1250_txdesc_t *)txCur;

#if BC1250ETH_TXQ_LIST

#else
	//-- Release the packet descriptor but don't free the buffers.
	eee_deallocateDesc(pktDesc);
#endif // BC1250ETH_TXQ_LIST


	return(EEE_FORWARDED_PACKET);
} // bmc12500Eth_txUnicast 

/*
 *------------------------------------------------------------------------------
 * Function Name:	bmc12500Eth_tx_raw
 *
 * Description: 	Transmits a buffer.
 *
 * Created by:		Frank Salzmann
 *
 * Data Created:	December 11, 2001
 * 					
 * Returns: ?????
 *
 *------------------------------------------------------------------------------
 */
int
bmc12500Eth_tx_raw(char *txbuf, uint32 txlen, uint32 dest)
{
    int i, priQ, lid;
    char                     *curPtr;
    bc1250EthCfg_t            *cfg;
    bc1250Eth_txCtrl_t        *txc;
    volatile bc1250_txdesc_t  *txCur;
    volatile bc1250Eth_regs_t *regs;
 
    //printf("bmc12500Eth_tx_raw: enter, dest: %x\n", dest);
 
    //-- Figure out which ethernet interface to use. Based on
    //-- the port component of the 'dest' function parameter.
    BC1250ETH_GET_TX_PORT(dest, cfg);
 
    //printf("port addr: %d\n", EEE_GET_PORT_ID(dest));
    if( cfg == 0 ) {
        printf("Invalid port addr: %d\n", EEE_GET_PORT_ID(dest));
        return( EEE_DROP_PACKET );
    }
#ifdef NFX_SMP
    // dedicate separate tx channel to each CPU to avoid contention
    if (cmGetMyCPUId() == NFX_NFP_FP1)
        priQ = 1;
    else
#endif
    priQ = 0;
 
    txc = &(cfg->txc[priQ]);

    //-- Add Backplane MAC Headers.
    if (isCheetah()) {
        i = get_lmux_lsi_mac_port(dest);
        lid = i % 3;
        txbuf -= 18;
        txlen += 18;
        curPtr = txbuf;
		curPtr += 3; /* skip past tx seq id and rsvd fields */
        *curPtr++ = lid;

        if (txlen < 64) 
            txlen = 64; /* LSI mac won't pass smaller packets */
#ifdef LMUX_DEBUG
#ifdef NFX_SMP
        txc->lmux_txPkts[i] += 1;
		txc->lmux_txBytes[i] += txlen;
#else
        lmux_txPkts[i] += 1;
		lmux_txBytes[i] += txlen;
#endif
#endif
    }
    else {
	    txbuf -= 14;
	    txlen += 14;
	    curPtr = txbuf;
	}

	bp_setDestMacAddr(dest, 1, curPtr);
#ifdef B1250_OLD_CODE 
    curPtr += 6;
#else
	curPtr += 12; /* skip setting source macAddr because it is not needed */
#endif
#ifdef B1250_OLD_CODE
    memcpy(curPtr, cfg->macAddr, 6); /* used to set source macAddr */
    curPtr += 6;
#endif
	*(uint16*)curPtr = EEE_ETH_TYPE;
    curPtr += 2;
 
    regs  = &(cfg->regs);
    txCur = txc->txCur;
    /*
     * Check for room on the transmit queue.
     * Make sure there is at least one free entry left so tx cleanup
     * pointer won't equal tx current pointer when queue is full. 
     */
    if( 1 > (BC1250_MAC_TX_Q_LEN_MINUS_ONE - txc->qd)  ) {
        //-- No room on the tx queue.
        printf("Txq Full: bufCnt: %d, desc qd: %d\n", 1, txc->qd);
        txc->qFull++;
        return( EEE_DROP_PACKET );
    }
    //-- Mark first descriptor of a packet.
    txCur->dw1 = BC1250_DESC_STATUS_SOP;
 
    if( bmc12500Eth_tx_pkt_trace ) {
        printf("Chip Tx Desc Cnt: %lld\n", (uint64)(*regs->dma_dscr_cnt_tx[0]));
        printf("Tx Raw: buf ptr: %lx, buf len: %d\n", (address_t)txbuf, txlen);
        bmc12500Eth_printBuf(txbuf, txlen, 0);
    }
    //-- Set buf pointer, buf size and SOP bit.
    BC1250_DESC_SET_WD_A_TX(txCur,
                            (uint64)txbuf,
                            EEE_BUFSIZE_ULTRA/32ll,
                            BC1250_DESC_STATUS_SOP);

    //-- Set packet size and tx option.
    BC1250_DESC_SET_WD_B_TX(txCur, (uint64)txlen, bc1250_tx_opt[cfg->macId]);

    //-- Tag descriptor as special 9k buffer.
    txCur->edesc = 0xfeabbaef;
 
    txc->qd++;

	if( ++txCur >= txc->txqEnd )
        txCur = txc->txqStart;
 
    *(regs->dma_dscr_cnt_tx[priQ]) = 1; // Give tx desc to MAC.
 
    txc->txCur  = (bc1250_txdesc_t *)txCur;
 
    return(EEE_FORWARDED_PACKET);
} // bmc12500Eth_tx_raw

/*
 *------------------------------------------------------------------------------
 * Function Name:	bmc12500Eth_txCleanupQ
 *
 * Description: 	Cleans up descriptors on a transmit queue.
 *
 * Created by:		Frank Salzmann
 *
 * Data Created:	October 26, 2001
 * 					
 * Returns: NFX_OK
 *
 *------------------------------------------------------------------------------
 */
int32 bmc_txMax=1024;
int32 bmc_txHWM CPU_PRIVATE;

int
bmc12500Eth_txCleanupQ( bc1250EthCfg_t *cfg, bc1250Eth_txCtrl_t *txc )
{
	uint32				     clean_cnt;
	uint32				     tx_cnt;
	uint32					 chip_descCnt;
    uint32                   priQ;
	bc1250Eth_regs_t         *regs;
#ifdef NFP_FP
	char                     *buf;
#endif
	eee_desc_t				 *edesc;
	volatile bc1250_txdesc_t *ccur;
	volatile bc1250_txdesc_t *tcur;

	ccur = txc->txCln;
	tcur = txc->txCur;

	regs  = &(cfg->regs);

	tx_cnt = 0;

    if (txc->txqId == 0)
        priQ = 0;
    else
        priQ = 1;

	chip_descCnt = *regs->dma_dscr_cnt_tx[priQ];
	
	//-- Sanity check counters.
	if( txc->qd < chip_descCnt ) {
		//printf("Err: Tx queue depth:%d  < chip desc cnt:%d\n", 
		//txc->qd, chip_descCnt);
		return 0;
	}

	clean_cnt = txc->qd - chip_descCnt;

	if( clean_cnt > bmc_txMax )
		clean_cnt = bmc_txMax;

	while( ccur != tcur && (clean_cnt--) ) {

		tx_cnt++;
		if( tx_cnt > bmc_txHWM )
			bmc_txHWM = tx_cnt;

#if BC1250ETH_TXQ_LIST
		edesc = (eee_desc_t *)ccur->edesc;
		if( edesc ) {
			if( cfg->link.loopback != lb_none )
				eee_freePacket(edesc);
			else
#if defined(PROM)
				eee_freePacket(edesc);
#else
			// On FP backplane look for special 9k buf.
			if( (address_t)edesc == 0xfeabbaef )  {    
				BC1250_DESC_GET_A_ADDR( ccur, (address_t)buf );
				if( buf )
					dp_proxy_tx_cleanup(buf); 
			}
			else
				eee_freePacket(edesc);
#endif // NFP_TXRX
			ccur->edesc = 0;
		}

#else	// BC1250ETH_TXQ_LIST
		
		BC1250_DESC_GET_A_ADDR( ccur, buf );
		if( buf )
			eee_deallocateBuffer( buf );
#endif // BC1250ETH_TXQ_LIST

		//-- Decrement number of descriptors on tx queue.
		txc->qd--;
		ccur->dw1 = 0;
		ccur->dw2 &= BC1250_DESC_NEXT_DESC;

		if( ++ccur >= txc->txqEnd )
			ccur = txc->txqStart;

	}
	txc->txCln = (bc1250_txdesc_t *)ccur;
	return(NFX_OK);
} // bmc12500Eth_txCleanupQ 

/*
 *------------------------------------------------------------------------------
 * Function Name:	dp83820_txCleanup
 *
 * Description: 	Polling function registered with the EEE layer and 
 * 					called to cleanup the transmit queues.
 *
 * Created by:		Frank Salzmann
 *
 * Data Created:	April 30, 2001 * * Returns: ?????
 *
 *------------------------------------------------------------------------------
 */
void
bmc12500Eth_txCleanPoll(void *cb, uint32 tlimit)
{
	bc1250EthCfg_t *cfg;
	int				i = 0;

#if defined(NFX_SMP)
#if defined(NFP_FP)
    if (!bc1250_FP2CPU_active) {
        if (cmGetMyCPUId() == NFX_NFP_FP2)
            return;
    }
    if (!bc1250_FP1CPU_active) {
        if (cmGetMyCPUId() == NFX_NFP_FP1)
            return;
    }
#else 
	if (!CM_IS_TXRX_NCPU)
		return;
#endif /* NFX_FP */
#endif /* NFX_SMP */

	bmc12500Eth_txLinkPoll_rr();

    cfg = (bc1250EthCfg_t *)cb;

#if defined(NFX_SMP) && defined(NFP_FP)
    if (cmGetMyCPUId() == NFX_NFP_FP2) {
        bmc12500Eth_txCleanupQ( cfg, &cfg->txc[0] );
    }
    else {
        bmc12500Eth_txCleanupQ( cfg, &cfg->txc[1] );
    }
#else
	for(i=0; i < BC1250_MAC_TX_Q_CNT; i++)
		bmc12500Eth_txCleanupQ( cfg, &cfg->txc[i] );
#endif
	return;

} // bmc12500Eth_txCleanup 

int
bmc12500Eth_addEeeHdrs( eee_desc_t *pktDesc, 
						uint32 slot, 
						uint32 cpu,
						bc1250EthCfg_t *cfg)
{
    int   pktLen = EEE_GET_PKT_LEN(pktDesc);
	int   offset;
	int   padLen = 0;
    int   bufCnt = EEE_GET_NUM_BUFS(pktDesc);
    int   hdrLen, i;
	int   fNewBuf = 0;
    char *hdrBuf;
    char *curPtr;
	char *myMacAddr;
	uint32 lbMode;
	uchar8	lid = 0;
	agile_hdr_t *eeeHdr;
	enet_hdr_t *ethHdr;
#ifdef NFP_FP
	scsi_msgHdr_t *scsiHdr;
#endif
 
    if (isCheetah()) {
        /* determine which LSI MAC to use and get its mac_addr */
        i = get_lmux_lsi_mac_port(pktDesc->hdr.dest_port);
#ifndef SIBYTE_BC			
        lid = i % 3;
#endif		
        myMacAddr = (char *)bmc12500_lsi_macAddr[i];
    }
    else {
		/* Garfield uses same MAC addresses as Cheetah, but has no 
         * LMUX or LSI macs 
		 */
        i = get_lmux_lsi_mac_port(pktDesc->hdr.dest_port);
        myMacAddr = (char *)bmc12500_lsi_macAddr[i];
	}
	lbMode    = cfg->link.loopback;
    //printf("bmc12500Eth_addEeeHdrs [Enter]\n");
 
	/* add 4-byte LMUX header for Cheetah */
#ifndef SIBYTE_BC			
	if (isCheetah())
		hdrLen = sizeof(enet_hdr_t) + sizeof(agile_hdr_t) + 4; 
	else
#endif			
		hdrLen = sizeof(enet_hdr_t) + sizeof(agile_hdr_t);

	// Check for room in the first buffer.
	hdrBuf = pktDesc->bd[1].buf;	
#if defined(EEE_NEW_BUFSIZE)
    offset = ((address_t)hdrBuf & 0x7F);
#else
	offset = ((address_t)hdrBuf & 0xFF);
#endif		

	if( !(pktDesc->bd[1].control & EDESC_BD_CMD_START_IN_USE)
		&& offset >= hdrLen) {	

    	pktDesc->bd[1].buf = (char *)((address_t)hdrBuf &= ~0xff);
		/* shift another 4 bytes for 8-byte alignment */
		if (isCheetah()) {
#ifndef SIBYTE_BC			
			padLen = offset - hdrLen - 4;
			hdrBuf += 4; 
			pktDesc->bd[1].buf += 4;
#else			
			padLen = offset - hdrLen;
#endif			
		}
		else {
			padLen = offset - hdrLen;
		}
    	pktDesc->bd[1].len += (hdrLen + padLen);
	}
	else {
		NFX_ASSERT(!(pktDesc->hdr.control & EDESC_COMMAND_USE_FIRST_BUFFER));
		hdrBuf = eee_allocateBuffer(EEE_SHARED_POOL, EEE_BUF_LARGE, 0);
		if( !hdrBuf ) {
			printf("addEeeHdrs: Error cannot allocate buffer.\n");
			return -1;
		}
        eee_attrAssignBuf(hdrBuf, EEE_ATTR_MAC_TX);
		// Offset 16 bytes into the buffer.
		//(address_t)hdrBuf |= 0x10;
		padLen = 64 - hdrLen;
		fNewBuf = 1;
	}
	
	curPtr = hdrBuf;
	//-- Insert LMUX hdr.
#ifndef SIBYTE_BC	
    if (isCheetah()) {
		curPtr += 3; /* skip past tx seq id and rsvd fields */
        *curPtr++ = lid;
    }
#endif	

    // Insert MAC Hdr
	//printf("curPtr: %x\n", curPtr);
	//printf("Slot: %d, Cpu: %d, Type: %d, Da: ", slot, cpu, type);
	ethHdr = (enet_hdr_t *)curPtr;
	eeeHdr = (agile_hdr_t *)(curPtr + sizeof(enet_hdr_t));

	if( lbMode == lb_none )
		bp_setDestMacAddr(pktDesc->hdr.dest_port, 1, ethHdr->DA);  
	else
   		memcpy6(ethHdr->DA, &broadcast_addr, 6);

    memcpy6(ethHdr->SA, myMacAddr, 6);

	ethHdr->type = EEE_ETH_TYPE;

	eeeHdr->pad = padLen;
#if defined(NETEEE_FRAGMENT)
	eeeHdr->offset = pktDesc->hdr.offset;
	eeeHdr->pad__ = 0;
#endif
	eeeHdr->dest_port = pktDesc->hdr.dest_port;
	eeeHdr->src_port = pktDesc->hdr.src_port;
 
   	pktLen += (hdrLen + padLen);

#ifdef NFP_FP
#if 1 // elongate tx packet to help prevent National rx overrun on SP-NIM
	scsiHdr = (scsi_msgHdr_t *)(curPtr + sizeof(enet_hdr_t) + 
				sizeof(agile_hdr_t) + padLen);
	if (scsiHdr->mid == SCSI_MID_READ_REQ)
		pktLen += 512;
#endif
#endif
    // Add header buffer to the edesc.
    //printf("Setup pktdest: buf: %x, len: %d\n", hdrBuf, hdrLen);
	if( fNewBuf ) {
    	pktDesc->bd[0].buf     = hdrBuf;
    	pktDesc->bd[0].len     = hdrLen + padLen;
    	pktDesc->bd[0].control = EDESC_BD_CMD_IN_USE | EDESC_BD_LARGE;
    	bufCnt ++;
    	pktDesc->hdr.control =
                    (pktLen & EDESC_TOTAL_LEN) |
                    ((bufCnt << EDESC_BUFCNT_SHIFT) & EDESC_BUFCNT_MASK) |
                    EDESC_COMMAND_USE_FIRST_BUFFER;
	}
	else {
    	pktDesc->hdr.control =
                    (pktLen & EDESC_TOTAL_LEN) |
                    ((bufCnt << EDESC_BUFCNT_SHIFT) & EDESC_BUFCNT_MASK);
	}
    //printf("Final PktLen: %d\n", pktLen);
 
    //printf("bmc12500Eth_addEeeHdrs [Leave]\n");
    return 0;
} // bmc12500Eth_addEeeHdrs                 

int
bmc12500Eth_txCheck( eee_desc_t *edesc )
{
	uint32 i, j, offset, bufCnt;

	//printf("txCheck: [start]\n");

	bufCnt = (edesc->hdr.control & EDESC_BUFCNT_MASK) >> EDESC_BUFCNT_SHIFT;

    if( bmc12500Eth_tx_pkt_trace ) {
        printf("bmc12500 tx check: buf Cnt: %d\n", bufCnt);
		eee_descDisplay(edesc);
    }

	if( edesc->hdr.control & EDESC_COMMAND_USE_FIRST_BUFFER )
		i=0;
	else
		i=1;

#if 0
	//-- Checked chained buffer count.
	if( bufCnt > 2 ) {
		printf("BCM12500: Tx Buf Err Chain more than 2 buffers. BufCnt: %d, pktLen: %d\n", 
			   bufCnt, edesc->hdr.control & EDESC_TOTAL_LEN);
		return( EEE_DROP_PACKET );
	}
#endif

#if 1
	offset = (address_t)edesc->bd[i].buf & 0x1f;

	//-- First Buffer needs end on a cache line boundry.
	if( (address_t)((edesc->bd[i].buf) + (edesc->bd[i].len)) & 0x1F ) {
		printf("BCM12500: Tx Buf End Not Aligned Error: Buf: %lx, Len: %d\n", 
			   (address_t)edesc->bd[i].buf, edesc->bd[i].len);
		eee_descDisplay(edesc);
		return( EEE_DROP_PACKET );
	}

	//-- Len+offset needs to be at least 64 bytes long and a module 64 size.
	//if( (edesc->bd[i].len + offset) < 64 ) {
	//	printf("BCM12500: Tx Buf Too Short Error: Buf: %lx, Len: %d\n", 
	//	  (address_t)edesc->bd[i].buf, edesc->bd[i].len);
	//	eee_descDisplay(edesc);
	//	return( EEE_DROP_PACKET );
	//	}
	// This check is already covered by the modulo check below

	if( (edesc->bd[i].len + offset) % 64 ) {
		printf("BCM12500: Tx Buf Len+Offset Not Modulo 64: Buf: %lx, Len: %d, Offset: %d\n", 
			   (address_t)edesc->bd[i].buf, edesc->bd[i].len, offset);
		eee_descDisplay(edesc);
		return( EEE_DROP_PACKET );
	}
#endif

	//-- Make sure buffers passed down are aligned on 
	//-- 64 byte boundry.
	i++; // Move one past the first buffer.
	for(j=1; j < bufCnt; i++, j++ )  {
		if( (address_t)(edesc->bd[i].buf) & 0x3f ) {
			printf("** BCM12500: Tx Buf Not aligned error: %lx, i=%d\n",
					(address_t)edesc->bd[i].buf, i);
			return( EEE_DROP_PACKET );
		}
	}

    if( bmc12500Eth_tx_pkt_trace ) {
        printf("bmc12500 tx check: pkt buf ok\n");
    }

	return NFX_OK;
} // bmc12500Eth_txCheck


int
bmc12500Eth_txLink_edesc(uint32 dest, eee_desc_t *edesc)
{
	int32 lid = 0;					// Link Id
	bc1250_ltc_t *ltc;

	ltc = bcLinkTxqs[lid];

	//-- Make sure there is room on the queue.
	if(ltc->qdCnt >= BC1250_LINK_TX_Q_LEN) {
		ltc->qFullErr++;
		return EEE_DROP_PACKET;
	}
/*	
	printf("Added pkt to link Q: %d\n", lid);
*/
	//-- Add packet to the link transmit queue.
	ltc->qAddCur->control = BC1250_LINK_TX_EDESC;
	ltc->qAddCur->dest    = dest;
	ltc->qAddCur->pktLen  = edesc->hdr.control & EDESC_TOTAL_LEN;
	ltc->qAddCur->o.edesc = edesc;

	//-- Finish up.
	if( ++ltc->qAddCur >= ltc->qEnd )
		ltc->qAddCur = ltc->qStart;

	ltc->qdCnt++;

	return EEE_FORWARDED_PACKET;
} // bmc12500Eth_txLink_edesc

int
bmc12500Eth_txLink_raw(char *txbuf, uint32 txlen, uint32 dest)
{
	int32 lid = 0;					// Link Id
	bc1250_ltc_t *ltc;

	//-- Get link trasmit control struct.
	ltc = bcLinkTxqs[lid];

	//-- Make sure there is room on the queue.
	if(ltc->qdCnt >= BC1250_LINK_TX_Q_LEN) {
		ltc->qFullErr++;
		return EEE_DROP_PACKET;
	}

	//-- Add packet to the link transmit queue.
	ltc->qAddCur->control = BC1250_LINK_TX_RAW;
	ltc->qAddCur->dest    = dest;
	ltc->qAddCur->pktLen  = txlen;
	ltc->qAddCur->o.txbuf = txbuf;

	//-- Finish up.
	if( ++ltc->qAddCur >= ltc->qEnd )
		ltc->qAddCur = ltc->qStart;

	ltc->qdCnt++;

	return EEE_FORWARDED_PACKET;
} // bmc12500Eth_txLink_raw

#if 0

// deficit round robin version for transmit queue processing
int
bmc12500Eth_txLinkPoll_drr() 
{
	int i;
	bc1250_ltc_t *ltc;
	uint64 zcycle_cnt, zdiff;
	int fContinue[BC1250_LINK_TX_Q_CNT];
	
	zcycle_cnt = *((volatile uint64 *)PHYS_TO_K1(0x10030000));
	
	zdiff = zcycle_cnt - bcLinkTxqs[0]->zcycle_cnt;
	bcLinkTxqs[0]->zdiff = zdiff;
	bcLinkTxqs[0]->zcycle_cnt = zcycle_cnt;
	bcLinkTxqs[0]->dCnt += ((zdiff * 2) / 5 );
	fContinue[0] = 1;

	zdiff = zcycle_cnt - bcLinkTxqs[1]->zcycle_cnt;
	bcLinkTxqs[1]->zdiff = zdiff;
	bcLinkTxqs[1]->zcycle_cnt = zcycle_cnt;
	bcLinkTxqs[1]->dCnt += ((zdiff * 2) / 5 );
	fContinue[1] = 1;

	zdiff = zcycle_cnt - bcLinkTxqs[2]->zcycle_cnt;
	bcLinkTxqs[2]->zdiff = zdiff;
	bcLinkTxqs[2]->zcycle_cnt = zcycle_cnt;
	bcLinkTxqs[2]->dCnt += ((zdiff * 2) / 5 );
	fContinue[2] = 1;

	zdiff = zcycle_cnt - bcLinkTxqs[3]->zcycle_cnt;
	bcLinkTxqs[3]->zdiff = zdiff;
	bcLinkTxqs[3]->zcycle_cnt = zcycle_cnt;
	bcLinkTxqs[3]->dCnt += ((zdiff * 2) / 5 );
	fContinue[3] = 1;

	while( fContinue[0] || fContinue[1] || fContinue[2] || fContinue[3] ) {

		for( i=0; i < BC1250_LINK_TX_Q_CNT; i++) {

			ltc = bcLinkTxqs[i];

			if( ltc->qdCnt  && (ltc->dCnt > 0) ) {
			
				//-- Make sure pkt len is less than current deficit cnt.
				if( ltc->qRemCur->pktLen > ltc->dCnt ) {
					fContinue[i] = 0;
				}
				else {

					//-- Queue the pkt for transmission.
					if( ltc->qRemCur->control == BC1250_LINK_TX_EDESC ) {
						bmc12500Eth_txUnicast( ltc->qRemCur->dest, 
											   ltc->qRemCur->o.edesc );
						ltc->qRemCur->o.edesc = 0;
					}
					else {
						bmc12500Eth_tx_raw( ltc->qRemCur->o.txbuf,
											ltc->qRemCur->pktLen,
											ltc->qRemCur->dest);
						ltc->qRemCur->o.txbuf = 0;
					}

					//-- Decrement the deficit count
					ltc->dCnt -= ltc->qRemCur->pktLen;

					ltc->qdTotal++;
					ltc->octets += ltc->qRemCur->pktLen;
					ltc->qRemCur->dest    = 0;
					ltc->qRemCur->control = 0;
					ltc->qRemCur->pktLen  = 0;

					if( ++ltc->qRemCur >= ltc->qEnd )
						ltc->qRemCur = ltc->qStart;	

					ltc->qdCnt--;
				}
			}
			else {
				fContinue[i] = 0;
			}

			//- If the tx queue was drained then reset the deficit count to zero.
			if( ltc->qdCnt == 0 )
				ltc->dCnt = 0;
		}
	} //- End While
	
	return NFX_OK;
} // bmc12500Eth_txLinkPoll_drr

#else

// deficit round robin version for tranmit queue processing
int
bmc12500Eth_txLinkPoll_drr() 
{
	int i;
	bc1250_ltc_t *ltc;
	uint64 zcycle_cnt, zdiff;
	

	for( i=0; i < BC1250_LINK_TX_Q_CNT; i++) {

		ltc = bcLinkTxqs[i];

		zcycle_cnt = *((volatile uint64 *)PHYS_TO_K1(0x10030000));
		
		zdiff = zcycle_cnt - ltc->zcycle_cnt;
		ltc->zdiff = zdiff;
		ltc->zcycle_cnt = zcycle_cnt;

		//ltc->dCnt += ((zdiff * 2) / 5 ); 
		ltc->dCnt += (zdiff >> 1); // Divide by 2, try to speed things up by eliminating multiply and divide op.

		while( ltc->qdCnt  && (ltc->dCnt > 0) ) {
			
			//-- Make sure pkt len is less than current deficit cnt.
			if( ltc->qRemCur->pktLen > ltc->dCnt )
				break;

			//-- Queue the pkt for transmission.
			if( ltc->qRemCur->control == BC1250_LINK_TX_EDESC ) {
				bmc12500Eth_txUnicast( ltc->qRemCur->dest, 
									   ltc->qRemCur->o.edesc );
				ltc->qRemCur->o.edesc = 0;
			}
			else {
				bmc12500Eth_tx_raw( ltc->qRemCur->o.txbuf,
									ltc->qRemCur->pktLen,
									ltc->qRemCur->dest);
				ltc->qRemCur->o.txbuf = 0;
			}

			//-- Decrement the deficit count
			ltc->dCnt -= ltc->qRemCur->pktLen;

			ltc->qdTotal++;
			ltc->octets += ltc->qRemCur->pktLen;
			ltc->qRemCur->dest    = 0;
			ltc->qRemCur->control = 0;
			ltc->qRemCur->pktLen  = 0;

			if( ++ltc->qRemCur >= ltc->qEnd )
				ltc->qRemCur = ltc->qStart;	

			ltc->qdCnt--;

		}

		//- If the tx queue was drained then reset the deficit count to zero.
		if( ltc->qdCnt == 0 )
			ltc->dCnt = 0;
	}
	
	return NFX_OK;
} // bmc12500Eth_txLinkPoll_drr

#endif

// plain round robin version for transmit queue processing 
int
bmc12500Eth_txLinkPoll_rr()
{
    int i;
    bc1250_ltc_t *ltc;

    for( i=0; i < BC1250_LINK_TX_Q_CNT; i++) {
#if defined(NFX_SMP) && defined(NFP_FP)
        if (cmGetMyCPUId() == NFX_NFP_FP1)
            ltc = bcLinkTxqs_2ndCPU[i];
        else
#endif
        ltc = bcLinkTxqs[i];

        if( ltc->qdCnt > 0 ) {

            //-- Decrement deficit count.
            //-- For now we will ignore this. Which gives us
            //-- a basic round robin aglorithim.
            // ltc->dCnt -= ltc->qRemCur->pktLen;
            // ltc->dCnt = 0;

            if( ltc->dCnt <= 0 ) {
                if( ltc->qRemCur->control == BC1250_LINK_TX_EDESC ) {
                    bmc12500Eth_txUnicast( ltc->qRemCur->dest,
                                           ltc->qRemCur->o.edesc );
                    ltc->qRemCur->o.edesc = 0;
                }
                else {
                    bmc12500Eth_tx_raw( ltc->qRemCur->o.txbuf,
                                        ltc->qRemCur->pktLen,
                                        ltc->qRemCur->dest);
                    ltc->qRemCur->o.txbuf = 0;
                }
                ltc->qdTotal++;
                ltc->octets += ltc->qRemCur->pktLen;
                ltc->qRemCur->dest    = 0;
                ltc->qRemCur->control = 0;
                ltc->qRemCur->pktLen  = 0;

                if( ++ltc->qRemCur >= ltc->qEnd )
                    ltc->qRemCur = ltc->qStart;
                ltc->qdCnt--;
            }
        }
    }
    return NFX_OK;
} // bmc12500Eth_txLinkPoll_rr


int
get_lmux_lsi_mac_port(uint32 dest)
{
    uint32 port, cpuid;

    cpuid = EEE_GET_DEST_CPU(dest);
    cpuid &= 0x03;
    port = lmux_lsi_port[cpuid];
    return(port);
}

int
bmc1250_txPktCapture( char *buf, uint32 len, int32 macId )
{
#ifdef BC1250_TX_PKT_CAPTURE  
	uint32 capLen;

	if( len > BC1250_BYTES_CAPTURE_MAX )
		capLen = BC1250_BYTES_CAPTURE_MAX;
	else
		capLen = len;

	memcpy(bc1250_pcb[macId].pcbe_tx[bc1250_pcb[macId].curEntry_tx].bytes, buf, capLen); 

	bc1250_pcb[macId].pcbe_tx[bc1250_pcb[macId].curEntry_tx].pktLen = len;
	bc1250_pcb[macId].pcbe_tx[bc1250_pcb[macId].curEntry_tx].capLen = capLen;
	bc1250_pcb[macId].pcbe_tx[bc1250_pcb[macId].curEntry_tx].bufPtr = buf;
	eee_rtc(&(bc1250_pcb[macId].pcbe_tx[bc1250_pcb[macId].curEntry_tx].t));

	if( ++bc1250_pcb[macId].curEntry_tx >= BC1250_CAPTURE_ENTRIES_MAX )
		bc1250_pcb[macId].curEntry_tx = 0;
#endif

	return 0;
} // bmc1250_txPktCapture


// return 1 for pass 1 sibyte
int
bmc12500_is_oldPass()
{
	uint64 *rev_reg;

	rev_reg = (uint64 *)PHYS_TO_K1(A_SCD_SYSTEM_REVISION);
	if (((*rev_reg >> 8) & 0xff) >= 4) // rev 4 and up is beyond pass 1
		return 0;
	else
		return 1;
}
