/*  
 *=============================================================================
 * File Name	: bmc12500-eth.c
 *
 *  Description	:    
 *
 *  Created		: 26 April 2001
 *
 *  Author		: Frank Salzmann
 *
 *  RCS			: $Id: bmc12500-eth.c,v 1.62 2002/12/07 01:02:22 tonyw Exp $
 *
 *  Copyright(c)  Agile Storage, Inc. 2001
 *=============================================================================
 */
#include <sys/mips.h>
#include "bmc12500-eth.h"
#include <exports/mem-api.h>
#include <exports/malloc-api.h>
#include <exports/cpu-api.h>
#include <exports/timer-api.h>

extern int bmc12500Eth_rx_pkt_trace;
typedef int32 funcptr();

int32 bmc12500Eth_linkPoll( int32 tmr,           
						  bc1250EthCfg_t *cfg,
						  void *user2,
						  void *user3,
						  void *user4 );

bc1250EthCfg_t *bcEthCfg[BC1250_MAC_CNT+1];

bc1250_ltc_t *bcLinkTxqs[BC1250_LINK_TX_Q_CNT];
#if defined(NFX_SMP) && defined(NFP_FP)
bc1250_ltc_t *bcLinkTxqs_2ndCPU[BC1250_LINK_TX_Q_CNT];
#endif

unsigned char bmc12500_lsi_macAddr[6][6];
int bmc12500_link_status[4] = { -1,-1,-1,-1 };

int bmc12500Eth_startMacFifo(bc1250EthCfg_t *);
int bmc12500Eth_initFifoRegs(bc1250EthCfg_t *); 
int bmc12500Eth_procLSReq(uint32, bc1250Eth_ls_reqMsg_t *);
int bmc12500Eth_initRmonRegs(bc1250EthCfg_t *); 
int bmc12500Eth_initQ_tx(bc1250_txdesc_t *, int);
int bmc12500Eth_fillRxQ(bc1250Eth_rxCtrl_t *, int); 
int bmc12500Eth_setMacSpeed(bc1250EthCfg_t *);
int bmc12500Eth_startMac(bc1250EthCfg_t *); 
int bmc12500Eth_initDmaRegs(bc1250EthCfg_t *); 
int bmc12500Eth_initMacRegs(bc1250EthCfg_t *); 
int bmc12500Eth_setMacDuplex(bc1250EthCfg_t *);
int bmc12500Eth_resetMac(bc1250EthCfg_t *); 
int bmc12500Eth_initPhy(bc1250EthCfg_t *);
int bmc12500Eth_initPktFifo(bc1250EthCfg_t *);
int bmc12500Eth_initMac(bc1250EthCfg_t *);
int bmc12500Eth_setAddr(bc1250EthCfg_t *); 
int bmc12500Eth_initLinkTxqs(bc1250_ltc_t **); 
int bmc12500Eth_setRegAddrs(bc1250EthCfg_t *, uint32);

extern int bmc12500Eth_txUnicast(uint32, eee_desc_t *);
extern int bmc12500Eth_txLink_edesc(uint32, eee_desc_t *);
extern int bmc12500Eth_diagLbPktReceive(eee_desc_t *);
extern int bmc12500Eth_mii_initPhy(bc1250EthCfg_t *);
extern int bmc12500Eth_mii_initPhyFifo();
extern int bmc12500Eth_mii_findPhy(bc1250EthCfg_t *);
extern int bmc12500Eth_mii_poll(bc1250EthCfg_t *);
extern int bmc12500_is_oldPass(void);
extern int lsi_get_linkstatus(uint16);
extern int lsi_init(void);
extern int lsi_gmac_wait(int);
extern void lsi_set_mac_addr(int, uchar8 *);
extern int32 bsd_ChangeLinkStatus(int, int);
extern void delay_us(uint32);
extern int bp_getMyMacAddr(uint32, uint32, uint32, uint32, char *);
extern uchar8 *seepGetMyExtMacAddr(uint32);

extern char eee_txrx_ext_mac[3][6];

//-- Packet capture buffer
bc1250_pcb_t bc1250_pcb[3];
uint32 bc1250_tx_opt[3]; /* transmit option flag for descriptor per macId */

/*
 *------------------------------------------------------------------------------
 * Function Name:	bmc12500Eth_initModule
 *
 * Description: 	Initializes the BMC12500 MAC Device Driver module.
 *
 * Created by:		Frank Salzmann
 *
 * Data Created:	October 05, 2001
 * 					
 * Returns: 
 *
 *------------------------------------------------------------------------------
 */
int
bmc12500Eth_initModule()
{
#if defined(SIBYTE)
	int i, j, rc, appId, lucappId, sacAppId;
	bc1250EthCfg_t *cfg;
	int	port = 0;
    volatile uint32 * luc_reset;
    uchar8 mac_addr[6];
    volatile uint64 * fp_bmfpga;
    volatile uint64 * fp_txrx_gpio;
    uint32 bobcat_mod_2280 = 0, rbmfpga = 0, rtxrxgpio = 0;
	
//	printf("bmc12500Eth_initModule: [Enter]\n");

// WTG Get the bits from the registers that tells us what model of
// bobcat we are..   bobcat_mod_2280 = 1 =  2280 in FIFO mode...
       if( isFP() ) {  // We got some work to do to find out how to init.
           // go read BM-FPGA through PCI bus offset 0 (Board Status Reg) and check bit 14
           // 0 = 2280    1 = 2220/40/60
           fp_bmfpga = (uint64 *)0x900000f81f400000;
           fp_txrx_gpio = (uint64 *)0x900000e010061aa0;
           rbmfpga = *fp_bmfpga;
           rtxrxgpio = *fp_txrx_gpio;
           printf("  - FP reading Bmfpga  and TXRX's GPIO pins\n");
           printf("    - BMFPGA  addr = %p data = %08x \n",fp_bmfpga,rbmfpga); 
           printf("    - TXRX GPIO addr = %p data = %08x \n",fp_txrx_gpio,rtxrxgpio); 
           if( (rbmfpga & 0x00004000)) { 
                printf(" FP  read model = 2220/40/60 from bmfpga\n");
                bobcat_mod_2280 = 0;
           }
           else {
               // now we need to read the TXRX GPIO reg bit 9 to see what mode the FMUX is in
               // 0 = GMII  (Ethernet)   1 = FIFO
               if( (rtxrxgpio & 0x00000200) ) {
                   printf(" FP  read model = 2280 in FIFO mode \n");
                   bobcat_mod_2280 = 1;
               }
               else {
                   printf(" FP  read model = 2280 in GMII mode \n");
                   bobcat_mod_2280 = 0;
               }
           }
       }
// WTG -----------------
	/*
	 *--------------------------------------
	 * Register with EEE and get app id
	 *--------------------------------------
	 */
	appId = eee_allocateApplicationId("bmc12500Eth");
	if( appId == NFX_ERR )
	{
		printf("[bmc12500_mac_initModule] Error Allocate app id failed.");
		return NFX_ERR;
	}

	rc = eee_registerReceiveFunc( appId,				// App Id	
								  bmc12500Eth_rxAppPkt, // Ucast Rx Func 
								  NULL);				// Mcast Rx Func

	if( rc == NFX_ERR ) 
		printf("[bmc12500Eth_initModule] Err: Cannot Register App Rx Func\n");

#ifdef BC1250ETH_NETWORK
	lucappId = eee_getApplicationId("lucRcv");
	if( lucappId == NFX_ERR )
	{
		printf("[bmc12500_mac_initModule] Err Get Luc AppId Failed");
		return NFX_ERR;
	}
#endif // BC1250ETH_NETWORK 

	//-- Initialize the pkt capture buffers.
	memset( bc1250_pcb, 0, sizeof(bc1250_pcb) );

	//-- Initialize the per link transmit queues.
	bmc12500Eth_initLinkTxqs(bcLinkTxqs);
#if defined(NFX_SMP) && defined(NFP_FP)
	bmc12500Eth_initLinkTxqs(bcLinkTxqs_2ndCPU);
#endif
#ifdef PROM
#ifndef SIBYTE_BC
//WTG  May need to put this back in for Bobcat 2280

    /* For proper reset sequence, wait for LMUX image to be loaded before 
     * continuing sibyte init. This is true by the time LSI regs can be 
     * accessed. 
	 */
    if (isCheetah() && isFP()) {
        lsi_gmac_wait(10000);       /* wait for 10 secs max */
    }
#endif
#endif

	for(i=0; i <= BC1250_MAC_CNT; i++ ) {
		/* For BOBCAT TXRX only */	
#ifdef SIBYTE_BC			
        if(isTXRX()) 
#endif		
        {	
            if ((i == 1) && isCheetah()) /* skip MAC 1 on Cheetah */
                continue;
            if ((i == 2) && !(isCheetah())) /* skip MAC 2 on Garfield */
                continue;
        }
       /* WTG  *************************************************************
       ** New model of bobcat will use FIFO mode for Eth 0, 1  and GigE 
       **  for Eth 2 
       ********************************************************************/
       if( isFP() )
           if(bobcat_mod_2280)
               if((i == 1) && isCheetah()) /* skip MAC 1 on Bobcat Model 2280 with smux */
                   continue;    // isCheetah() are straps/pull-ups on SiByte
       /********************************************************************/

		bcEthCfg[i] = cfg = (bc1250EthCfg_t *)eee_ramAlloc(sizeof(bc1250EthCfg_t));

		if( !cfg ) {
			printf("[bmc12500Eth_initModule] Err: Malloc Failed: bmcEthCfg.\n");
			return(NFX_ERR);
		}
		memset(cfg,0,sizeof(bc1250EthCfg_t));

		bmc12500Eth_setRegAddrs(cfg, i);
		cfg->appId = appId;

#ifdef BC1250ETH_NETWORK
		cfg->lucRcvAddr = PACK_UNICAST_PORT_ADDRESS(
										nfxMySlotId, 
										nfxMyCPUId,
										lucappId,
										EEE_USER_DATA,
										NFX_COS_HIGH, 
										0);

		//printf("Luc Rcv Id: %d, Addr: %x\n", lucappId, cfg->lucRcvAddr);
#endif // BC1250ETH_NETWORK 

		//--  Make sure bmc12500 ethernet interface is stopped.
		bmc12500Eth_resetMac( cfg );

        if (isCheetah()) {
            //--  Initialize physical interfaces
#ifdef SIBYTE_BC
            if ((i == 0) && isTXRX())
                bmc12500Eth_mii_initPhyFifo();

            //--  Initialize and start MAC.
            if(isTXRX()) {
                if( bmc12500Eth_initPktFifo(cfg) == NFX_ERR ) {
                    printf("BMC12500_MAC Initialize Module Failed\n");
                    return 0;
                }	
            // init mode determines transmit options
            // no modifications for packet fifo mode or 0x07
		bc1250_tx_opt[i] = BMC12500_ETH_TXOPT_NO_MOD;   // no modifications for packet fifo mode
            }

            if(isFP()) {
       /********************************************************************/
       /* We are Bobcat cuz of SIBYTE_BC and now check for model 2280
       **  which has a different set up from all others. Eth 0, 1 FIFO 
       **  and Eth 2 GigE                                                  */
       /********************************************************************/
                if (! (bobcat_mod_2280)) {    // Not 2280 init MAC's Eth mode

                    //--  Intialize physical interfaces
                    bmc12500Eth_initPhy(cfg);

                    //--  Initialize and start MAC.
                    if( bmc12500Eth_initMac(cfg) == NFX_ERR ) {
                        printf("BMC12500_MAC Initialize Module Failed\n");
                        return 0;
                    }
                    // init mode determines transmit options
		    // append CRC and pad for ethernet mode 0x03
		    bc1250_tx_opt[i] = BMC12500_ETH_TXOPT_APPEND_CRC_PAD; // append CRC and pad for ethernet mode
                }
                else {   // We are the FP and a bobcat 2280
                    switch (i) {  // Done on a per port basis
                        case 0: 
                        case 1: 
                            if( bmc12500Eth_initPktFifo(cfg) == NFX_ERR ) {
                                printf("BMC12500_MAC Initialize Module Failed\n");
                                return 0;
                            }	
            // init mode determines transmit options
            // no modifications for packet fifo mode or 0x07
		            bc1250_tx_opt[i] = BMC12500_ETH_TXOPT_NO_MOD; // no modifications for packet fifo mode
                            break;
                        
                        case 2: {
                            //--  Intialize physical interfaces / also structure set up?
                            bmc12500Eth_initPhy(cfg);
                            //--  Initialize and start MAC.
                            if( bmc12500Eth_initMac(cfg) == NFX_ERR ) {
                                printf("BMC12500_MAC Initialize Module Failed\n");
                                return 0;
                            }
                            // init mode determines transmit options
		            // append CRC and pad for ethernet mode 0x03
		            bc1250_tx_opt[i] = BMC12500_ETH_TXOPT_APPEND_CRC_PAD; // append CRC and pad for ethernet mode
                            break;
                        }
                        default: {
                            printf(" - BMC12500 Mac Initialization Failure\n");
                            return 0;
                        }
                    }
                }
            }
#else			
            if ((i == 0) && isTXRX())
                bmc12500Eth_mii_initPhyFifo();

            //--  Initialize and start MAC.
            if( bmc12500Eth_initPktFifo(cfg) == NFX_ERR ) {
                printf("BMC12500_MAC Initialize Module Failed\n");
                return 0;
            }
            // init mode determines transmit options
            // no modifications for packet fifo mode or 0x07
            bc1250_tx_opt[i] = BMC12500_ETH_TXOPT_NO_MOD; // no modifications for packet fifo mode
#endif			
        }
		//-- Initialize Local Mac Address
		if( isTXRX() ) {
#ifdef USE_TEMP_EXT_MAC_ADDR_SCHEME
//			printf("********* Getting mac addr from seep **************");
			if (!isCheetah()) {
				memcpy(cfg->macAddr, seepGetMyExtMacAddr(i), 6);
				bmc12500Eth_setAddr(cfg); 
			}
#else
			if (!isCheetah()) {
	    		memcpy(cfg->macAddr, &eee_txrx_ext_mac[i][0], 6);
				bmc12500Eth_setAddr(cfg); 
			}
#endif
		}

		//-- Create and start link timer.
		if (isCheetah() && isTXRX()) {
			cfg->link.timer = eee_timerCreate(BC1250_LINK_TIMER, 
											  EEE_TIMER_REPEAT);
			if(cfg->link.timer == TIMER_ERR_NO_TIMER_AVAILABLE) {
				printf("BC1250 ETH: Could not allocate link timer.\n");
			}
			else {
				eee_timerStart(cfg->link.timer,
							   (funcptr*)bmc12500Eth_linkPoll,           
							   cfg, NULL, NULL, NULL);
			}
		}

	    sacAppId = eee_getApplicationId("sac");
		//-- Create destination port address for SAC application.
		cfg->sacDestPort =  PACK_UNICAST_PORT_ADDRESS( nfxMySlotId, 
													   NFX_NFP_TXRX_ANY,
													   0,
													   EEE_SAC,
											           NFX_COS_HIGH, 
											           0);

		//-- Put link into normal mode.
		cfg->link.loopback = lb_none;
			
		//-- Mark MAC Id: 
		cfg->macId = i;

		//-- Setup local application address. 
#if defined(PROM)
		if( isFP() ) {
			if (i == 0)
				port = PROM_PORT_FP0;
			else 
			if (i == 1)
				port = PROM_PORT_FP1;
			else
				port = PROM_PORT_FP2;
		}
		else {
			if (i == 0)
				port = PROM_PORT_TXRX0;
			else 
			if (i == 1)
				port = PROM_PORT_TXRX1;
			else 
				port = PROM_PORT_TXRX2;
		}
#endif
		cfg->myPortAddr = PACK_UNICAST_PORT_ADDRESS( nfxMySlotId, 
													 nfxMyCPUId,
													 cfg->appId,
													 EEE_USER_DATA,
													 NFX_COS_HIGH, 
													 port);
		if (isCheetah()) {
			rc = eee_registerTransmitFunc( port,
										   bmc12500Eth_txLink_edesc,
										   bmc12500Eth_txLink_edesc,
										   9*1024);
		}
		else {
			rc = eee_registerTransmitFunc( port,
										   bmc12500Eth_txUnicast,
										   bmc12500Eth_txUnicast,
										   9*1024);
		}

		if( rc == NFX_ERR )
			printf("Tx Func Register Error: %x\n", rc);

		rc = eee_registerPollingFunc(EEE_HIGH_PRIORITY,bmc12500Eth_txCleanPoll,cfg);
		if( rc == EEE_ERR_POLLING_TABLE_FULL )
			printf("BMC12500: Err Cannot register txCleanPoll function. Tbl full\n");
		else
		if( rc == EEE_ERR_BAD_PARAM )
			printf("BMC12500: Err Cannot register txCleanPoll function. Invalid Priority\n");
/*
		else
			printf("BMC12500: txCleanPoll function registered. \n");
*/


		eee_registerPollingFunc(EEE_HIGH_PRIORITY,bmc12500Eth_rxPktPoll,cfg);

	}  // End of For loop 

// WTG
/******************************************************************************************************************
**** All this got moved up to right after initMode cuz it determins transmit options
**** and now we do it on a per macId  Eth(i) in the for loop above
	if (isCheetah()) {			// determine transmit options
#ifdef SIBYTE_BC
 // WTG   these are packet tras modifiers..  "pkt_mod" in UserMan
		if(isTXRX())	
		    bc1250_tx_opt = BMC12500_ETH_TXOPT_NO_MOD; 	// no modifications for packet fifo mode
		else if(isFP())	
		    bc1250_tx_opt = BMC12500_ETH_TXOPT_APPEND_CRC_PAD; // append CRC and pad for ethernet mode
#else		
		bc1250_tx_opt = 0x7;  	// no modifications for packet fifo mode
#endif		
	}
	else {
		bc1250_tx_opt = 0x3;  	// append CRC and pad for ethernet mode
	}
******************************************************************************************************************/

#if defined(NFP_FP)

	if (isCheetah()) {
		eee_registerDefaultTransmitFunc(EEE_CONTROL,   
									bmc12500Eth_txLink_edesc, 9*1024);
		eee_registerDefaultTransmitFunc(EEE_MGMT,      
									bmc12500Eth_txLink_edesc, 1500);
		eee_registerDefaultTransmitFunc(EEE_USER_DATA, 
									bmc12500Eth_txLink_edesc, 9*1024);
	}
	else {
		eee_registerDefaultTransmitFunc(EEE_CONTROL,
									bmc12500Eth_txUnicast, 9*1024);
		eee_registerDefaultTransmitFunc(EEE_MGMT,
									bmc12500Eth_txUnicast, 1500);
		eee_registerDefaultTransmitFunc(EEE_USER_DATA,
									bmc12500Eth_txUnicast, 9*1024);
	}

#elif defined(PROM)
	if( isFP() ) {
		eee_registerDefaultTransmitFunc(EEE_CONTROL,   
									bmc12500Eth_txUnicast, 9*1024);
		eee_registerDefaultTransmitFunc(EEE_MGMT, 	   
									bmc12500Eth_txUnicast, 1500);
		eee_registerDefaultTransmitFunc(EEE_USER_DATA, 
									bmc12500Eth_txUnicast, 9*1024);
	}
#endif
    if (isCheetah()) {
        if (isTXRX()) {

            luc_reset = (volatile uint32 *)0xffffffffbf0000acll;
            *luc_reset = 0;          /* unreset the LUC */
            delay_us(10);
            luc_reset = (volatile uint32 *)0xffffffffbf000154ll;/* little endian */
            *luc_reset = 0;
            delay_us(10);
            luc_reset = (volatile uint32 *)0xffffffffbf000000ll;
            *luc_reset = 0x00001061; /* hold in reset, bypass LUC CAM, since it doesn't work */

            lsi_init(); /* must re-init LSI MACs after LUC is up */

            *luc_reset = 0x00001060; /* take out of reset, bypass LUC CAM, since it doesn't work*/

            for (j = 0; j < 4; j++) {/* initialize MAC addrs */
#ifdef USE_TEMP_EXT_MAC_ADDR_SCHEME
                memcpy(mac_addr, (uchar8 *)seepGetMyExtMacAddr(j), 6);
                lsi_set_mac_addr(j, mac_addr);
#else
                memcpy(mac_addr, &eee_txrx_ext_mac[j][0], 6);
                lsi_set_mac_addr(j, mac_addr);
#endif
            }
        }
        else { /* FP */
#ifndef SIBYTE_BC
			lmux_reset(1);    /* hold lmux in reset */	
            lsi_init(); 
			lmux_reset(0);    /* take lmux out of reset */
            for (j = 0; j < 5; j++) {   /* set LSI MACs for EEE_USER_DATA */
                if (j == 2)             /* skip this MAC for EEE_CONTROL */
                    continue;
                bp_getMyMacAddr(nfxMySlotId,
                                nfxMyCPUId,
                                j+1,
                                EEE_USER_DATA,
                                mac_addr);
                lsi_set_mac_addr(j, mac_addr);
                memcpy(bmc12500_lsi_macAddr[j], mac_addr, 6);
            }
            bp_getMyMacAddr(nfxMySlotId,nfxMyCPUId, 3, EEE_CONTROL, mac_addr);
            lsi_set_mac_addr(2, mac_addr); /* 3rd MAC is for EEE_CONTROL */
            memcpy(bmc12500_lsi_macAddr[2], mac_addr, 6);
            bp_getMyMacAddr(nfxMySlotId,nfxMyCPUId, 6, EEE_CONTROL, mac_addr);
            lsi_set_mac_addr(5, mac_addr); /* 6th MAC is for EEE_CONTROL */
            memcpy(bmc12500_lsi_macAddr[5], mac_addr, 6);
#endif			
        }
    }
	else { // Set backplane source addresses for Garfield 
		   // Using same addressing scheme as Cheetah, but there are no LSI macs
		if (isFP()) {
            for (j = 0; j < 5; j++) {   
                if (j == 2)            
                    continue;
                bp_getMyMacAddr(nfxMySlotId,
                                nfxMyCPUId,
                                j+1,
                                EEE_USER_DATA,
                                mac_addr);
                memcpy(bmc12500_lsi_macAddr[j], mac_addr, 6);
            }
            bp_getMyMacAddr(nfxMySlotId,nfxMyCPUId, 3, EEE_CONTROL, mac_addr);
            memcpy(bmc12500_lsi_macAddr[2], mac_addr, 6);
            bp_getMyMacAddr(nfxMySlotId,nfxMyCPUId, 6, EEE_CONTROL, mac_addr);
            memcpy(bmc12500_lsi_macAddr[5], mac_addr, 6);

			bmc12500Eth_setAddr(bcEthCfg[0]); 
			bmc12500Eth_setAddr(bcEthCfg[1]); 
		}
	}
//	printf("bmc12500Eth_initModule: [Leave]\n");
#endif // SIBYTE

	return(NFX_OK);
} // bmc12500Eth_initModule 

/*
 *------------------------------------------------------------------------------
 * Function Name:	bmc12500Eth_setRegAddrs
 *
 * Description: 	Initializes the bmc12500_mac register addresses.
 *
 * Created by:		Frank Salzmann
 *
 * Data Created:	October 08, 2001
 * 					
 * Returns: 		NFX_OK
 *------------------------------------------------------------------------------
 */
int
bmc12500Eth_setRegAddrs( bc1250EthCfg_t *pCfg, uint32 mac )
{
	switch( mac ) {

		case 0:
			pCfg->regs.dma_config0_rx[0]    =(uint64 *)PHYS_TO_K1(0x10064800);
			pCfg->regs.dma_config0_rx[1]    =(uint64 *)PHYS_TO_K1(0x10064900);
			pCfg->regs.dma_config0_tx[0]    =(uint64 *)PHYS_TO_K1(0x10064C00);
			pCfg->regs.dma_config0_tx[1]    =(uint64 *)PHYS_TO_K1(0x10064D00);

			pCfg->regs.dma_config1_rx[0]    =(uint64 *)PHYS_TO_K1(0x10064808);
			pCfg->regs.dma_config1_rx[1]    =(uint64 *)PHYS_TO_K1(0x10064908);
			pCfg->regs.dma_config1_tx[0]    =(uint64 *)PHYS_TO_K1(0x10064C08);
			pCfg->regs.dma_config1_tx[1]    =(uint64 *)PHYS_TO_K1(0x10064D08);

			pCfg->regs.dma_dscr_base_rx[0]  =(uint64 *)PHYS_TO_K1(0x10064810);
			pCfg->regs.dma_dscr_base_rx[1]  =(uint64 *)PHYS_TO_K1(0x10064910);
			pCfg->regs.dma_dscr_base_tx[0]  =(uint64 *)PHYS_TO_K1(0x10064C10);
			pCfg->regs.dma_dscr_base_tx[1]  =(uint64 *)PHYS_TO_K1(0x10064D10);

			pCfg->regs.dma_asic_addr        =(uint64 *)PHYS_TO_K1(0x10064418);

			pCfg->regs.dma_dscr_cnt_rx[0]   =(uint64 *)PHYS_TO_K1(0x10064818);
			pCfg->regs.dma_dscr_cnt_rx[1]   =(uint64 *)PHYS_TO_K1(0x10064918);
			pCfg->regs.dma_dscr_cnt_tx[0]   =(uint64 *)PHYS_TO_K1(0x10064C18);
			pCfg->regs.dma_dscr_cnt_tx[1]   =(uint64 *)PHYS_TO_K1(0x10064D18);

			pCfg->regs.dma_cur_dscra_rx[0]  =(uint64 *)PHYS_TO_K1(0x10064820);
			pCfg->regs.dma_cur_dscra_rx[1]  =(uint64 *)PHYS_TO_K1(0x10064920);
			pCfg->regs.dma_cur_dscra_tx[0]  =(uint64 *)PHYS_TO_K1(0x10064C20);
			pCfg->regs.dma_cur_dscra_tx[1]  =(uint64 *)PHYS_TO_K1(0x10064D20);

			pCfg->regs.dma_cur_dscrb_rx[0]  =(uint64 *)PHYS_TO_K1(0x10064828);
			pCfg->regs.dma_cur_dscrb_rx[1]  =(uint64 *)PHYS_TO_K1(0x10064928);
			pCfg->regs.dma_cur_dscrb_tx[0]  =(uint64 *)PHYS_TO_K1(0x10064C28);
			pCfg->regs.dma_cur_dscrb_tx[1]  =(uint64 *)PHYS_TO_K1(0x10064D28);

			pCfg->regs.dma_cur_dscr_addr_rx[0] =(uint64 *)PHYS_TO_K1(0x10064830);
			pCfg->regs.dma_cur_dscr_addr_rx[1] =(uint64 *)PHYS_TO_K1(0x10064930);
			pCfg->regs.dma_cur_dscr_addr_tx[0] =(uint64 *)PHYS_TO_K1(0x10064C30);
			pCfg->regs.dma_cur_dscr_addr_tx[1] =(uint64 *)PHYS_TO_K1(0x10064D30);

			pCfg->regs.mac_cfg			  =(uint64 *)PHYS_TO_K1(0x10064100);
			pCfg->regs.mac_enable		  =(uint64 *)PHYS_TO_K1(0x10064400);
			pCfg->regs.mac_txd_ctl		  =(uint64 *)PHYS_TO_K1(0x10064420);
			pCfg->regs.mac_thrsh_cfg	  =(uint64 *)PHYS_TO_K1(0x10064108);
			pCfg->regs.mac_frame_cfg	  =(uint64 *)PHYS_TO_K1(0x10064118);
			pCfg->regs.mac_vlantag		  =(uint64 *)PHYS_TO_K1(0x10064110);
			pCfg->regs.mac_status		  =(uint64 *)PHYS_TO_K1(0x10064408);
			pCfg->regs.mac_debug_status	  =(uint64 *)PHYS_TO_K1(0x10064448);
			pCfg->regs.mac_int_mask		  =(uint64 *)PHYS_TO_K1(0x10064410);
			pCfg->regs.mac_fifo_ptrs	  =(uint64 *)PHYS_TO_K1(0x10064130);
			pCfg->regs.mac_eopcnt	  	  =(uint64 *)PHYS_TO_K1(0x10064120);
			pCfg->regs.mac_addr[0]		  =(uint64 *)PHYS_TO_K1(0x10064280);
			pCfg->regs.mac_addr[1]		  =(uint64 *)PHYS_TO_K1(0x10064288);
			pCfg->regs.mac_addr[2]		  =(uint64 *)PHYS_TO_K1(0x10064290);
			pCfg->regs.mac_addr[3]		  =(uint64 *)PHYS_TO_K1(0x10064298);
			pCfg->regs.mac_addr[4]		  =(uint64 *)PHYS_TO_K1(0x100642A0);
			pCfg->regs.mac_addr[5]		  =(uint64 *)PHYS_TO_K1(0x100642A8);
			pCfg->regs.mac_addr[6]		  =(uint64 *)PHYS_TO_K1(0x100642B0);
			pCfg->regs.mac_addr[7]		  =(uint64 *)PHYS_TO_K1(0x100642B8);
			pCfg->regs.mac_hash[0]		  =(uint64 *)PHYS_TO_K1(0x10064240);
			pCfg->regs.mac_hash[1]		  =(uint64 *)PHYS_TO_K1(0x10064248);
			pCfg->regs.mac_hash[2]		  =(uint64 *)PHYS_TO_K1(0x10064250);
			pCfg->regs.mac_hash[3]		  =(uint64 *)PHYS_TO_K1(0x10064258);
			pCfg->regs.mac_hash[4]		  =(uint64 *)PHYS_TO_K1(0x10064260);
			pCfg->regs.mac_hash[5]		  =(uint64 *)PHYS_TO_K1(0x10064268);
			pCfg->regs.mac_hash[6]		  =(uint64 *)PHYS_TO_K1(0x10064270);
			pCfg->regs.mac_hash[7]		  =(uint64 *)PHYS_TO_K1(0x10064278);
			pCfg->regs.mac_ethernet_addr  =(uint64 *)PHYS_TO_K1(0x10064208);
			pCfg->regs.mac_type_cfg		  =(uint64 *)PHYS_TO_K1(0x10064210);
			pCfg->regs.mac_adfilter_cfg	  =(uint64 *)PHYS_TO_K1(0x10064200);
			pCfg->regs.mac_chlo[0]		  =(uint64 *)PHYS_TO_K1(0x10064300);
			pCfg->regs.mac_chlo[1]		  =(uint64 *)PHYS_TO_K1(0x10064308);
			pCfg->regs.mac_chlo[2]		  =(uint64 *)PHYS_TO_K1(0x10064310);
			pCfg->regs.mac_chlo[3]		  =(uint64 *)PHYS_TO_K1(0x10064318);
			pCfg->regs.mac_chup[0]		  =(uint64 *)PHYS_TO_K1(0x10064320);
			pCfg->regs.mac_chup[1]		  =(uint64 *)PHYS_TO_K1(0x10064328);
			pCfg->regs.mac_chup[2]		  =(uint64 *)PHYS_TO_K1(0x10064330);
			pCfg->regs.mac_chup[3]		  =(uint64 *)PHYS_TO_K1(0x10064338);
			pCfg->regs.mac_mdio			  =(uint64 *)PHYS_TO_K1(0x10064428);
			pCfg->regs.rmon			      =(uint64 *)PHYS_TO_K1(0x10064000);
			break;

		case 1:
			pCfg->regs.dma_config0_rx[0]  =(uint64 *)PHYS_TO_K1(0x10065800);
			pCfg->regs.dma_config0_rx[1]  =(uint64 *)PHYS_TO_K1(0x10065900);
			pCfg->regs.dma_config0_tx[0]  =(uint64 *)PHYS_TO_K1(0x10065C00);
			pCfg->regs.dma_config0_tx[1]  =(uint64 *)PHYS_TO_K1(0x10065D00);

			pCfg->regs.dma_config1_rx[0]  =(uint64 *)PHYS_TO_K1(0x10065808);
			pCfg->regs.dma_config1_rx[1]  =(uint64 *)PHYS_TO_K1(0x10065908);
			pCfg->regs.dma_config1_tx[0]  =(uint64 *)PHYS_TO_K1(0x10065C08);
			pCfg->regs.dma_config1_tx[1]  =(uint64 *)PHYS_TO_K1(0x10065D08);

			pCfg->regs.dma_dscr_base_rx[0]  =(uint64 *)PHYS_TO_K1(0x10065810);
			pCfg->regs.dma_dscr_base_rx[1]  =(uint64 *)PHYS_TO_K1(0x10065910);
			pCfg->regs.dma_dscr_base_tx[0]  =(uint64 *)PHYS_TO_K1(0x10065C10);
			pCfg->regs.dma_dscr_base_tx[1]  =(uint64 *)PHYS_TO_K1(0x10065D10);

			pCfg->regs.dma_asic_addr        =(uint64 *)PHYS_TO_K1(0x10065418);

			pCfg->regs.dma_dscr_cnt_rx[0]   =(uint64 *)PHYS_TO_K1(0x10065818);
			pCfg->regs.dma_dscr_cnt_rx[1]   =(uint64 *)PHYS_TO_K1(0x10065918);
			pCfg->regs.dma_dscr_cnt_tx[0]   =(uint64 *)PHYS_TO_K1(0x10065C18);
			pCfg->regs.dma_dscr_cnt_tx[1]   =(uint64 *)PHYS_TO_K1(0x10065D18);

			pCfg->regs.dma_cur_dscra_rx[0]  =(uint64 *)PHYS_TO_K1(0x10065820);
			pCfg->regs.dma_cur_dscra_rx[1]  =(uint64 *)PHYS_TO_K1(0x10065920);
			pCfg->regs.dma_cur_dscra_tx[0]  =(uint64 *)PHYS_TO_K1(0x10065C20);
			pCfg->regs.dma_cur_dscra_tx[1]  =(uint64 *)PHYS_TO_K1(0x10065D20);

			pCfg->regs.dma_cur_dscrb_rx[0]  =(uint64 *)PHYS_TO_K1(0x10065828);
			pCfg->regs.dma_cur_dscrb_rx[1]  =(uint64 *)PHYS_TO_K1(0x10065928);
			pCfg->regs.dma_cur_dscrb_tx[0]  =(uint64 *)PHYS_TO_K1(0x10065C28);
			pCfg->regs.dma_cur_dscrb_tx[1]  =(uint64 *)PHYS_TO_K1(0x10065D28);

			pCfg->regs.dma_cur_dscr_addr_rx[0]=(uint64 *)PHYS_TO_K1(0x10065830);
			pCfg->regs.dma_cur_dscr_addr_rx[1]=(uint64 *)PHYS_TO_K1(0x10065930);
			pCfg->regs.dma_cur_dscr_addr_tx[0]=(uint64 *)PHYS_TO_K1(0x10065C30);
			pCfg->regs.dma_cur_dscr_addr_tx[1]=(uint64 *)PHYS_TO_K1(0x10065D30);

			pCfg->regs.mac_cfg			  =(uint64 *)PHYS_TO_K1(0x10065100);
			pCfg->regs.mac_enable		  =(uint64 *)PHYS_TO_K1(0x10065400);
			pCfg->regs.mac_txd_ctl		  =(uint64 *)PHYS_TO_K1(0x10065420);
			pCfg->regs.mac_thrsh_cfg	  =(uint64 *)PHYS_TO_K1(0x10065108);
			pCfg->regs.mac_frame_cfg	  =(uint64 *)PHYS_TO_K1(0x10065118);
			pCfg->regs.mac_vlantag		  =(uint64 *)PHYS_TO_K1(0x10065110);
			pCfg->regs.mac_status		  =(uint64 *)PHYS_TO_K1(0x10065408);
			pCfg->regs.mac_debug_status	  =(uint64 *)PHYS_TO_K1(0x10065448);
			pCfg->regs.mac_int_mask		  =(uint64 *)PHYS_TO_K1(0x10065410);
			pCfg->regs.mac_fifo_ptrs	  =(uint64 *)PHYS_TO_K1(0x10065130);
			pCfg->regs.mac_eopcnt	  	  =(uint64 *)PHYS_TO_K1(0x10065120);
			pCfg->regs.mac_addr[0]		  =(uint64 *)PHYS_TO_K1(0x10065280);
			pCfg->regs.mac_addr[1]		  =(uint64 *)PHYS_TO_K1(0x10065288);
			pCfg->regs.mac_addr[2]		  =(uint64 *)PHYS_TO_K1(0x10065290);
			pCfg->regs.mac_addr[3]		  =(uint64 *)PHYS_TO_K1(0x10065298);
			pCfg->regs.mac_addr[4]		  =(uint64 *)PHYS_TO_K1(0x100652A0);
			pCfg->regs.mac_addr[5]		  =(uint64 *)PHYS_TO_K1(0x100652A8);
			pCfg->regs.mac_addr[6]		  =(uint64 *)PHYS_TO_K1(0x100652B0);
			pCfg->regs.mac_addr[7]		  =(uint64 *)PHYS_TO_K1(0x100652B8);
			pCfg->regs.mac_hash[0]		  =(uint64 *)PHYS_TO_K1(0x10065240);
			pCfg->regs.mac_hash[1]		  =(uint64 *)PHYS_TO_K1(0x10065248);
			pCfg->regs.mac_hash[2]		  =(uint64 *)PHYS_TO_K1(0x10065250);
			pCfg->regs.mac_hash[3]		  =(uint64 *)PHYS_TO_K1(0x10065258);
			pCfg->regs.mac_hash[4]		  =(uint64 *)PHYS_TO_K1(0x10065260);
			pCfg->regs.mac_hash[5]		  =(uint64 *)PHYS_TO_K1(0x10065268);
			pCfg->regs.mac_hash[6]		  =(uint64 *)PHYS_TO_K1(0x10065270);
			pCfg->regs.mac_hash[7]		  =(uint64 *)PHYS_TO_K1(0x10065278);
			pCfg->regs.mac_ethernet_addr  =(uint64 *)PHYS_TO_K1(0x10065208);
			pCfg->regs.mac_type_cfg		  =(uint64 *)PHYS_TO_K1(0x10065210);
			pCfg->regs.mac_adfilter_cfg	  =(uint64 *)PHYS_TO_K1(0x10065200);
			pCfg->regs.mac_chlo[0]		  =(uint64 *)PHYS_TO_K1(0x10065300);
			pCfg->regs.mac_chlo[1]		  =(uint64 *)PHYS_TO_K1(0x10065308);
			pCfg->regs.mac_chlo[2]		  =(uint64 *)PHYS_TO_K1(0x10065310);
			pCfg->regs.mac_chlo[3]		  =(uint64 *)PHYS_TO_K1(0x10065318);
			pCfg->regs.mac_chup[0]		  =(uint64 *)PHYS_TO_K1(0x10065320);
			pCfg->regs.mac_chup[1]		  =(uint64 *)PHYS_TO_K1(0x10065328);
			pCfg->regs.mac_chup[2]		  =(uint64 *)PHYS_TO_K1(0x10065330);
			pCfg->regs.mac_chup[3]		  =(uint64 *)PHYS_TO_K1(0x10065338);
			pCfg->regs.mac_mdio			  =(uint64 *)PHYS_TO_K1(0x10065428);
			pCfg->regs.rmon			      =(uint64 *)PHYS_TO_K1(0x10065000);
			break;

		case 2:
			pCfg->regs.dma_config0_rx[0]  =(uint64 *)PHYS_TO_K1(0x10066800);
			pCfg->regs.dma_config0_rx[1]  =(uint64 *)PHYS_TO_K1(0x10066900);
			pCfg->regs.dma_config0_tx[0]  =(uint64 *)PHYS_TO_K1(0x10066C00);
			pCfg->regs.dma_config0_tx[1]  =(uint64 *)PHYS_TO_K1(0x10066D00);

			pCfg->regs.dma_config1_rx[0]  =(uint64 *)PHYS_TO_K1(0x10066808);
			pCfg->regs.dma_config1_rx[1]  =(uint64 *)PHYS_TO_K1(0x10066908);
			pCfg->regs.dma_config1_tx[0]  =(uint64 *)PHYS_TO_K1(0x10066C08);
			pCfg->regs.dma_config1_tx[1]  =(uint64 *)PHYS_TO_K1(0x10066D08);

			pCfg->regs.dma_dscr_base_rx[0]  =(uint64 *)PHYS_TO_K1(0x10066810);
			pCfg->regs.dma_dscr_base_rx[1]  =(uint64 *)PHYS_TO_K1(0x10066910);
			pCfg->regs.dma_dscr_base_tx[0]  =(uint64 *)PHYS_TO_K1(0x10066C10);
			pCfg->regs.dma_dscr_base_tx[1]  =(uint64 *)PHYS_TO_K1(0x10066D10);

			pCfg->regs.dma_asic_addr        =(uint64 *)PHYS_TO_K1(0x10066418);

			pCfg->regs.dma_dscr_cnt_rx[0]   =(uint64 *)PHYS_TO_K1(0x10066818);
			pCfg->regs.dma_dscr_cnt_rx[1]   =(uint64 *)PHYS_TO_K1(0x10066918);
			pCfg->regs.dma_dscr_cnt_tx[0]   =(uint64 *)PHYS_TO_K1(0x10066C18);
			pCfg->regs.dma_dscr_cnt_tx[1]   =(uint64 *)PHYS_TO_K1(0x10066D18);

			pCfg->regs.dma_cur_dscra_rx[0]  =(uint64 *)PHYS_TO_K1(0x10066820);
			pCfg->regs.dma_cur_dscra_rx[1]  =(uint64 *)PHYS_TO_K1(0x10066920);
			pCfg->regs.dma_cur_dscra_tx[0]  =(uint64 *)PHYS_TO_K1(0x10066C20);
			pCfg->regs.dma_cur_dscra_tx[1]  =(uint64 *)PHYS_TO_K1(0x10066D20);

			pCfg->regs.dma_cur_dscrb_rx[0]  =(uint64 *)PHYS_TO_K1(0x10066828);
			pCfg->regs.dma_cur_dscrb_rx[1]  =(uint64 *)PHYS_TO_K1(0x10066928);
			pCfg->regs.dma_cur_dscrb_tx[0]  =(uint64 *)PHYS_TO_K1(0x10066C28);
			pCfg->regs.dma_cur_dscrb_tx[1]  =(uint64 *)PHYS_TO_K1(0x10066D28);

			pCfg->regs.dma_cur_dscr_addr_rx[0]=(uint64 *)PHYS_TO_K1(0x10066830);
			pCfg->regs.dma_cur_dscr_addr_rx[1]=(uint64 *)PHYS_TO_K1(0x10066930);
			pCfg->regs.dma_cur_dscr_addr_tx[0]=(uint64 *)PHYS_TO_K1(0x10066C30);
			pCfg->regs.dma_cur_dscr_addr_tx[1]=(uint64 *)PHYS_TO_K1(0x10066D30);
			
			pCfg->regs.mac_cfg			  =(uint64 *)PHYS_TO_K1(0x10066100);
			pCfg->regs.mac_enable		  =(uint64 *)PHYS_TO_K1(0x10066400);
			pCfg->regs.mac_txd_ctl		  =(uint64 *)PHYS_TO_K1(0x10066420);
			pCfg->regs.mac_thrsh_cfg	  =(uint64 *)PHYS_TO_K1(0x10066108);
			pCfg->regs.mac_frame_cfg	  =(uint64 *)PHYS_TO_K1(0x10066118);
			pCfg->regs.mac_vlantag		  =(uint64 *)PHYS_TO_K1(0x10066110);
			pCfg->regs.mac_status		  =(uint64 *)PHYS_TO_K1(0x10066408);
			pCfg->regs.mac_debug_status	  =(uint64 *)PHYS_TO_K1(0x10066448);
			pCfg->regs.mac_int_mask		  =(uint64 *)PHYS_TO_K1(0x10066410);
			pCfg->regs.mac_fifo_ptrs	  =(uint64 *)PHYS_TO_K1(0x10066130);
			pCfg->regs.mac_eopcnt	  	  =(uint64 *)PHYS_TO_K1(0x10066120);
			pCfg->regs.mac_addr[0]		  =(uint64 *)PHYS_TO_K1(0x10066280);
			pCfg->regs.mac_addr[1]		  =(uint64 *)PHYS_TO_K1(0x10066288);
			pCfg->regs.mac_addr[2]		  =(uint64 *)PHYS_TO_K1(0x10066290);
			pCfg->regs.mac_addr[3]		  =(uint64 *)PHYS_TO_K1(0x10066298);
			pCfg->regs.mac_addr[4]		  =(uint64 *)PHYS_TO_K1(0x100662A0);
			pCfg->regs.mac_addr[5]		  =(uint64 *)PHYS_TO_K1(0x100662A8);
			pCfg->regs.mac_addr[6]		  =(uint64 *)PHYS_TO_K1(0x100662B0);
			pCfg->regs.mac_addr[7]		  =(uint64 *)PHYS_TO_K1(0x100662B8);
			pCfg->regs.mac_hash[0]		  =(uint64 *)PHYS_TO_K1(0x10066240);
			pCfg->regs.mac_hash[1]		  =(uint64 *)PHYS_TO_K1(0x10066248);
			pCfg->regs.mac_hash[2]		  =(uint64 *)PHYS_TO_K1(0x10066250);
			pCfg->regs.mac_hash[3]		  =(uint64 *)PHYS_TO_K1(0x10066258);
			pCfg->regs.mac_hash[4]		  =(uint64 *)PHYS_TO_K1(0x10066260);
			pCfg->regs.mac_hash[5]		  =(uint64 *)PHYS_TO_K1(0x10066268);
			pCfg->regs.mac_hash[6]		  =(uint64 *)PHYS_TO_K1(0x10066270);
			pCfg->regs.mac_hash[7]		  =(uint64 *)PHYS_TO_K1(0x10066278);
			pCfg->regs.mac_ethernet_addr  =(uint64 *)PHYS_TO_K1(0x10066208);
			pCfg->regs.mac_type_cfg		  =(uint64 *)PHYS_TO_K1(0x10066210);
			pCfg->regs.mac_adfilter_cfg	  =(uint64 *)PHYS_TO_K1(0x10066200);
			pCfg->regs.mac_chlo[0]		  =(uint64 *)PHYS_TO_K1(0x10066300);
			pCfg->regs.mac_chlo[1]		  =(uint64 *)PHYS_TO_K1(0x10066308);
			pCfg->regs.mac_chlo[2]		  =(uint64 *)PHYS_TO_K1(0x10066310);
			pCfg->regs.mac_chlo[3]		  =(uint64 *)PHYS_TO_K1(0x10066318);
			pCfg->regs.mac_chup[0]		  =(uint64 *)PHYS_TO_K1(0x10066320);
			pCfg->regs.mac_chup[1]		  =(uint64 *)PHYS_TO_K1(0x10066328);
			pCfg->regs.mac_chup[2]		  =(uint64 *)PHYS_TO_K1(0x10066330);
			pCfg->regs.mac_chup[3]		  =(uint64 *)PHYS_TO_K1(0x10066338);
			pCfg->regs.mac_mdio			  =(uint64 *)PHYS_TO_K1(0x10066428);
			pCfg->regs.rmon			      =(uint64 *)PHYS_TO_K1(0x10066000);
			break;
	}
	return NFX_OK;
} // bmc12500Eth_setupRegs
/*
 *------------------------------------------------------------------------------
 * Function Name:	bmc12500Eth_initPhy
 *
 * Description: 	Initializes the physical layer device attached to the
 *					BMC12500 MAC.
 *
 * Created by:		Frank Salzmann
 *
 * Data Created:	October 11, 2001
 * 					
 * Returns:			NFX_OK 
 *------------------------------------------------------------------------------
 */
int
bmc12500Eth_initPhy( bc1250EthCfg_t *cfg )
{
//	printf("[bmc12500Eth_initPhy] Enter\n");
#ifdef AGILE_HARDWARE
/*
	printf("[bmc12500Eth_initPhy] : release Phy from reset\n");
	// Bring PHY out of reset 
	*(volatile uint32 *)0xbf400000 = 0x04040404;
	delay_us(1000000);
*/
#endif

	// First job is to get the MII phy address.
	bmc12500Eth_mii_findPhy( cfg );

	// First job is to get the MII phy address.
	bmc12500Eth_mii_initPhy( cfg );

	// Next grab some link info from the phy.
	bmc12500Eth_mii_poll( cfg );

#if RAL_XXX
	//-- Hack required for the 5411 phy on the eval board
	bmc12500Eth_mii_song_and_dance(&cfg->regs);
#endif

//	printf("[bmc12500Eth_initPhy] Leave\n");
	return NFX_OK;
} // bmc12500Eth_initPhy

/*
 *------------------------------------------------------------------------------
 * Function Name:	bmc12500Eth_initMac
 *
 * Description: 	Initializes the bmc12500 MAC.
 *
 * Created by:		Frank Salzmann
 *
 * Data Created:	October 08, 2001
 * 					
 * Returns:			NFX_OK 
 *------------------------------------------------------------------------------
 */
int
bmc12500Eth_initMac( bc1250EthCfg_t *pbmcEthCfg )
{
	uint32           i;
	uint32           align_diff;
	uint32           size_rx;
	uint32           size_tx;
	char             *qstart;
	bc1250Eth_txCtrl_t *txc;
	bc1250Eth_rxCtrl_t *rxc;

//	printf("[bmc12500Eth_initMac] Enter\n");

	/*
	 * Allocate and align memory on 64-bit boundry
	 * for receive and transmit descriptor rings. 
	 */
	size_tx = sizeof(bc1250_txdesc_t) * BC1250_MAC_TX_Q_LEN;
	size_rx = sizeof(bc1250_rxdesc_t) * BC1250_MAC_RX_Q_LEN;


	/* Setup descriptor rings */
	for( i=0; i < BC1250_MAC_TX_Q_CNT; i++ )
	{
		txc = &pbmcEthCfg->txc[i];

		qstart = (uchar8 *)eee_ramAllocAligned(size_tx + 64);

		//printf("txq start: %p: %d bytes allocated\n", qstart, size_tx+64);

		align_diff = (address_t)qstart & 0x00000007;

		if( align_diff )
			align_diff = 8 - align_diff;
		qstart += align_diff;

		memset(qstart, 0, size_tx);
		txc->txqStart = (bc1250_txdesc_t *)qstart;
		txc->txCur = txc->txCln = txc->txqStart;
		txc->txqEnd = txc->txqStart + BC1250_MAC_TX_Q_LEN;
#if BC1250ETH_TXQ_LIST
		bmc12500Eth_initQ_tx( txc->txqStart, BC1250_MAC_TX_Q_LEN );
#endif
	}

	for( i=0; i < BC1250_MAC_RX_Q_CNT; i++ )
	{
		rxc = &pbmcEthCfg->rxc[i];
		rxc->rxqId = i;

#ifdef NFX_SMP
		NFX_SPIN_INIT(&rxc->lock, SPINLOCK_LEVEL_UNKNOWN);
#endif

#if defined(NFP_FP)
		if( i == 1 ) {
			if (sb1250_getRev() > SB1250_PASS1)
				rxc->bufPoolId   = EEE_TXRX_POOL;
			else
				rxc->bufPoolId   = EEE_SHARED_POOL;
			rxc->eeeBufSize  = EEE_BUF_LARGE;
			rxc->descBufSize = EEE_BUFSIZE_LARGE;
			rxc->bufOffset   = 0;
		}
		else {
			rxc->bufPoolId   = EEE_SHARED_POOL;
			rxc->eeeBufSize  = EEE_BUF_ULTRA;
#if defined(EEE_NEW_BUFSIZE)
			rxc->descBufSize = EEE_BUFSIZE_ULTRA - 64;
			rxc->bufOffset   = 64;
#else
			rxc->descBufSize = EEE_BUFSIZE_ULTRA-128;
			rxc->bufOffset   = 128;
#endif
		}
#else	// TxRx
		rxc->bufPoolId   = EEE_SHARED_POOL;
		rxc->eeeBufSize  = EEE_BUF_LARGE;
		rxc->descBufSize = EEE_BUFSIZE_LARGE;
		rxc->bufOffset   = 2;
#endif

		
		qstart = (uchar8 *)eee_ramAllocAligned(size_rx + 64);

		//printf("rxq start: %p: %d bytes allocated\n", qstart, size_rx+64);
		memset(qstart, 0, size_rx);
		align_diff = (address_t)qstart & 0x00000007;
		if( align_diff )
			align_diff = 8 - align_diff;
		qstart += align_diff;

		rxc->rxqStart = (bc1250_rxdesc_t *)(qstart);
		rxc->rxCur = rxc->rxqStart;
		rxc->rxqEnd = rxc->rxqStart + BC1250_MAC_RX_Q_LEN;
		rxc->avail = BC1250_MAC_RX_Q_LEN;
		bmc12500Eth_fillRxQ( rxc, BC1250_MAC_RX_Q_LEN );
	}

	// Initialize Ethernet Mac and Dma Registers.. 
	bmc12500Eth_initDmaRegs( pbmcEthCfg ); 

	bmc12500Eth_initMacRegs( pbmcEthCfg ); 

	bmc12500Eth_setMacSpeed( pbmcEthCfg );
	bmc12500Eth_setMacDuplex( pbmcEthCfg );

	bmc12500Eth_startMac( pbmcEthCfg );

//	printf("[bmc12500Eth_initMac] Leave\n");
	return( NFX_OK ); 
} // bmc12500Eth_initMac 
/*
 *------------------------------------------------------------------------------
 * Function Name:	bmc12500Eth_initDmaRegs
 *
 * Description: 	This function initializes BMC12500 Ethernet DMA 
 *					specific registers.
 *
 * Created by:		Frank Salzmann
 *
 * Data Created:	October 08, 2001
 * 					
 * Returns: 		NFX_OK
 *------------------------------------------------------------------------------
 */
int
bmc12500Eth_initDmaRegs(bc1250EthCfg_t *pbmcEthCfg) 
{
	int i;
	uint64 val;
	uint64 *reg;
	bc1250Eth_regs_t *regs;
	bmc12500_mac_dma_cr0_t macDmaCfg0;
	bmc12500_mac_dma_cr1_t cfg1;

//	printf("[bmc12500Eth_initDmaRegs] Enter\n");

	regs = &(pbmcEthCfg->regs);

	//-- Setup MAC Rx DMA Registers
	for( i=0; i < BC1250_MAC_RX_Q_CNT; i++) {

		//-- MAC DMA Rx Config 1 Registers
		reg = regs->dma_config1_rx[i];
		cfg1.wd = 0;
		cfg1.bits.hdr_cf_ena = 0;
		cfg1.bits.hdr_size   = 0;
        if (isCheetah()) {
            cfg1.bits.flow_ctl_en = 1;
        }
		bmc12500Eth_writeReg(reg, cfg1.wd);		

		//-- MAC DMA Rx Desc Queue Base Address Registers
		reg = regs->dma_dscr_base_rx[i];
#ifdef LARGE_MEMORY
		val = virt_to_phys((address_t)(pbmcEthCfg->rxc[i].rxqStart));
		val &= 0xFFFFFFFFFFFFFFF0;
#else
		val = ((address_t)(pbmcEthCfg->rxc[i].rxqStart) & ~V2PHYS_ADDR_MASK);
		val &= 0xFFFFFFFFF0;
#endif
		bmc12500Eth_writeReg(reg, val);

		//-- MAC DMA Configuration 0 Register.
		reg = regs->dma_config0_rx[i];
		macDmaCfg0.wd = 0;
		macDmaCfg0.bits.chain_sel = 0;
		macDmaCfg0.bits.ringsz    = BC1250_MAC_RX_Q_LEN;
        if (isCheetah()) {
            macDmaCfg0.bits.hi_watermk = 8;
            macDmaCfg0.bits.lo_watermk = 4;
        }
		bmc12500Eth_writeReg(reg, macDmaCfg0.wd);
	
	
		//-- MAC DMA Rx Desc Count Registers
		reg = regs->dma_dscr_cnt_rx[i];
		bmc12500Eth_writeReg(reg, BC1250_MAC_RX_Q_LEN);

	}

	//-- Setup MAC Tx DMA Registers
	for( i=0; i < BC1250_MAC_TX_Q_CNT; i++) {

		uint64 chip_cnt;
		uint64 zero_cnt;

		//-- MAC DMA Tx Config 1 Registers
		reg = regs->dma_config1_tx[i];
cfg1.wd = 0;
cfg1.bits.no_dscr_updt = 1;
		bmc12500Eth_writeReg(reg, cfg1.wd);		// Turn off for now.

		//-- Set Tx Desc Queue Base Addresses
		reg = regs->dma_dscr_base_tx[i];
#ifdef LARGE_MEMORY
		val = virt_to_phys((address_t)(pbmcEthCfg->txc[i].txqStart));
		val &= 0xFFFFFFFFFFFFFFF0;
#else
		val = ((address_t)(pbmcEthCfg->txc[i].txqStart) & ~V2PHYS_ADDR_MASK);
		val &= 0xFFFFFFFFF0;
#endif
		bmc12500Eth_writeReg(reg, val);

		//-- MAC DMA Configuration 0 Register.
		reg = regs->dma_config0_tx[i];
		macDmaCfg0.wd = 0;
#if BC1250ETH_TXQ_LIST
		macDmaCfg0.bits.chain_sel = 1;
#else
		macDmaCfg0.bits.ringsz    = BC1250_MAC_TX_Q_LEN;
#endif
		macDmaCfg0.bits.tbx_en    = 1;
		bmc12500Eth_writeReg(reg, macDmaCfg0.wd);

		//-- Clear MAC DMA Tx Desc Count Register
		reg = regs->dma_dscr_cnt_tx[i];
		chip_cnt = bmc12500Eth_readReg(reg);
		if( chip_cnt > 0 ) {
		    zero_cnt = (chip_cnt ^ 0xFFFF) + 1;
			printf("chip tx desc cnt: %lld\n", chip_cnt);
			zero_cnt &= 0x000000000000FFFF;
			printf("chip tx desc cnt: %lld\n", zero_cnt);
			bmc12500Eth_writeReg(reg, zero_cnt);
		}
	}
//	printf("[bmc12500Eth_initDmaRegs] Leave\n");
	return NFX_OK;
} // bmc12500Eth_initDmaRegs

/*
 *------------------------------------------------------------------------------
 * Function Name:	bmc12500Eth_initMacRegs
 *
 * Description: 	This function initializes BMC12500 Ethernet MAC 
 *					specific registers.
 *
 * Created by:		Frank Salzmann
 *
 * Data Created:	October 08, 2001
 * 					
 * Returns: 		NFX_OK
 *------------------------------------------------------------------------------
 */
int
bmc12500Eth_initMacRegs(bc1250EthCfg_t *cfg) 
{
	int i;
	bc1250Eth_regs_t *regs;

	//printf("[bmc12500Eth_initMacRegs] Enter\n");
	regs = &cfg->regs;

	{	//-- MAC Enable Register
		bmc12500Eth_writeReg(regs->mac_enable, 0); // Bring out of reset.
	}

	{	//-- MAC Transmit DMA Control Register
		bmc12500Eth_writeReg(regs->mac_txd_ctl, 0);
	}

	{	//-- MAC Fifo Threshold Register.
		bmc12500_mac_fifo_thresh_r_t macFifoThresh;
		macFifoThresh.wd = 0;
		macFifoThresh.bits.tx_wr_thresh = 8;	// From sample code.
		macFifoThresh.bits.tx_rd_thresh = 24;	// From sample code.
		macFifoThresh.bits.tx_rl_thresh = 4;	// From sample code.
		macFifoThresh.bits.rx_rd_thresh = 4;	// From sample code.
		macFifoThresh.bits.rx_rl_thresh = 8;	// From sample code.
		bmc12500Eth_writeReg(regs->mac_thrsh_cfg, macFifoThresh.wd);
	}

	{	//-- MAC Frame Configuration Register.
		bmc12500_macFrmCfgr_t macFrameCfg;
		macFrameCfg.wd = 0;
		macFrameCfg.bits.backoff_sel	= 1;
		macFrameCfg.bits.min_framesz	= 64;
		macFrameCfg.bits.max_framesz	= 9216;
		bmc12500Eth_writeReg(regs->mac_frame_cfg, macFrameCfg.wd);
	}

	{	//-- MAC VLAN Tag Register
		bmc12500Eth_writeReg(regs->mac_vlantag, 0);
	}

	{	//-- MAC Addr Exact Match Table
		for(i=0; i < 8; i++)
			bmc12500Eth_writeReg(regs->mac_addr[i], 0);
	}

	{	//-- MAC Addr Hash Table
		for(i=0; i < 8; i++)
			bmc12500Eth_writeReg(regs->mac_hash[i], 0);
	}

	{	//-- MAC DMA Channel Mapping Table
		for(i=0; i < 4; i++) {
#if defined(NFP_FP)
			bmc12500Eth_writeReg(regs->mac_chlo[i], 0xFFFFFFFFFFFFFFFEll);
			bmc12500Eth_writeReg(regs->mac_chup[i], 0xFFFFFFFFFFFFFFFEll);
#else
			bmc12500Eth_writeReg(regs->mac_chlo[i], 0);
			bmc12500Eth_writeReg(regs->mac_chup[i], 0);
#endif
		}
	}
	
	{	//-- MAC Rx Filter Register
#if defined(EVAL_BOARD) && defined(NFP_FP)
		bmc12500Eth_writeReg(regs->mac_adfilter_cfg, 0x08); // Turn off bcast in eval setup.
#else
		if( isFP() ) {
			bmc12500Eth_writeReg(regs->mac_adfilter_cfg, 0x21); // Turn on bcast in FP image.
		}	
		else
			bmc12500Eth_writeReg(regs->mac_adfilter_cfg, 0x38); // Bcast enabled for TXRX
#endif
	}

	{	//-- MAC Cfg Register.
		bmc12500_macCfgr_t macCfg;

		macCfg.wd = 0;
		macCfg.bits.tx_hold_sop_en 	= 1;
		macCfg.bits.retry_en       	= 1;
		macCfg.bits.ret_drpreq_en	= 0;		
		macCfg.bits.ret_ufl_en		= 0;
		macCfg.bits.burst_en		= 0;
		macCfg.bits.tx_pause_cnt	= 101;		// 16k slot times
		macCfg.bits.ap_stat_en		= 1;
		macCfg.bits.drp_errpkt_en   = 1;		// Drop pkt on any rx err.
		macCfg.bits.speed_sel       = 0;		// 100Mbps : For Eval purposes
		macCfg.bits.loopback_sel    = 0;		// Internal Loopback
		macCfg.bits.ss_en           = 1;		// Magic broadcom bit always set
		macCfg.bits.fast_sync		= 1;
		macCfg.bits.rx_ch_sel		= 26;		// Set offset in nibbles: byte position == 15
		
		bmc12500Eth_writeReg(regs->mac_cfg, macCfg.wd);
	}

	//-- Init RMON Regs
	bmc12500Eth_initRmonRegs( cfg );

	//printf("[bmc12500Eth_initMacRegs] Leave\n");
	return(NFX_OK);
} /* bmc12500Eth_initMacRegs */

/*
 *------------------------------------------------------------------------------
 * Function Name:	bmc12500Eth_initRmonRegs
 *
 * Description: 	This function initializes BMC12500 Ethernet RMON 
 *					specific registers.
 *
 * Created by:		Frank Salzmann
 *
 * Data Created:	November 01, 2001
 * 					
 * Returns: 		NFX_OK
 *------------------------------------------------------------------------------
 */
int
bmc12500Eth_initRmonRegs(bc1250EthCfg_t *pbmcEthCfg) 
{
	bc1250Eth_regs_t *regs;
	bc1250Eth_rmon_t *rm;

	regs = &pbmcEthCfg->regs;
	rm = (bc1250Eth_rmon_t *)regs->rmon;

	bmc12500Eth_writeReg(&rm->rx_byte_cnt, 0);
	bmc12500Eth_writeReg(&rm->rx_good_cnt, 0);
	bmc12500Eth_writeReg(&rm->rx_mcst_cnt, 0);
	bmc12500Eth_writeReg(&rm->rx_bcst_cnt, 0);
	bmc12500Eth_writeReg(&rm->rx_badp_cnt, 0);
	bmc12500Eth_writeReg(&rm->rx_runt_cnt, 0);
	bmc12500Eth_writeReg(&rm->rx_ovsz_cnt, 0);
	bmc12500Eth_writeReg(&rm->rx_fcse_cnt, 0);
	bmc12500Eth_writeReg(&rm->rx_lene_cnt, 0);
	bmc12500Eth_writeReg(&rm->rx_code_cnt, 0);
	bmc12500Eth_writeReg(&rm->rx_drbl_cnt, 0);
	bmc12500Eth_writeReg(&rm->tx_byte_cnt, 0);
	bmc12500Eth_writeReg(&rm->tx_good_cnt, 0);
	bmc12500Eth_writeReg(&rm->tx_badp_cnt, 0);
	bmc12500Eth_writeReg(&rm->tx_runt_cnt, 0);
	bmc12500Eth_writeReg(&rm->tx_ovsz_cnt, 0);
	bmc12500Eth_writeReg(&rm->tx_fcse_cnt, 0);
	bmc12500Eth_writeReg(&rm->tx_coll_cnt, 0);
	bmc12500Eth_writeReg(&rm->tx_lcol_cnt, 0);
	bmc12500Eth_writeReg(&rm->tx_xcol_cnt, 0);
	bmc12500Eth_writeReg(&rm->tx_abrt_cnt, 0);
	return(NFX_OK);
} // bmc12500Eth_initRmonRegs

/*
 *------------------------------------------------------------------------------
 * Function Name:	bmc12500Eth_startMac
 *
 * Description: 	Enables the BMC12500 Rx/Tx MAC and Rx/Tx DMA channels.
 *
 * Created by:		Frank Salzmann
 *
 * Data Created:	October 09, 2001
 * 					
 * Returns: 		NFX_OK
 *------------------------------------------------------------------------------
 */
int
bmc12500Eth_startMac(bc1250EthCfg_t *pbmcEthCfg) 
{
	bc1250Eth_regs_t   *regs;
	bmc12500_macEnar_t macEn;

//	printf("[bmc12500Eth_startMac] Enter\n");
	regs = &pbmcEthCfg->regs;

	//-- MAC Enable Register
	macEn.wd = bmc12500Eth_readReg(regs->mac_enable);
	macEn.bits.rxdma_en  = BC1250_MAC_RX_Q_CNT == 1 ? 1:3;	// Enable Channel 0 and 1
	macEn.bits.txdma_en  = 1;
	macEn.bits.mac_rx_en = 1;
	macEn.bits.mac_tx_en = 1;
	bmc12500Eth_writeReg(regs->mac_enable, macEn.wd); 

//	printf("[bmc12500Eth_startMac] Leave\n");
	return(NFX_OK);
} // bmc12500Eth_startMac 

/*
 *------------------------------------------------------------------------------
 * Function Name:	bmc12500Eth_stopMac
 *
 * Description: 	Stops the BMC12500 Ethernet MAC.
 *
 * Created by:		Frank Salzmann
 *
 * Data Created:	October 09, 2001
 * 					
 * Returns: 		NFX_OK
 *------------------------------------------------------------------------------
 */
int
bmc12500Eth_stopMac(bc1250EthCfg_t *pbmcEthCfg) 
{
	bc1250Eth_regs_t *regs;
	bmc12500_macEnar_t macEn;

//	printf("[bmc12500Eth_stopMac] Enter\n");
	regs = &pbmcEthCfg->regs;
    macEn.wd    = bmc12500Eth_readReg(regs->mac_enable);
	macEn.bits.mac_rx_en = 0;
	macEn.bits.mac_tx_en = 0;
	bmc12500Eth_writeReg(regs->mac_enable, macEn.wd); 
	
//	printf("[bmc12500Eth_stopMac] Leave\n");
	return(NFX_OK);
} // bmc12500Eth_stopMac 

/*
 *------------------------------------------------------------------------------
 * Function Name:	bmc12500Eth_resetMac
 *
 * Description: 	Resets the BMC12500 Ethernet MAC.
 *
 * Created by:		Frank Salzmann
 *
 * Data Created:	October 09, 2001
 * 					
 * Returns: 		NFX_OK
 *------------------------------------------------------------------------------
 */
int
bmc12500Eth_resetMac(bc1250EthCfg_t *pbmcEthCfg) 
{
	bc1250Eth_regs_t *regs;
	bmc12500_macEnar_t macEn;

//	printf("[bmc12500Eth_resetMac] Enter\n");
	regs = &pbmcEthCfg->regs;

	macEn.wd = 0;
	macEn.bits.p_reset = 1;
	bmc12500Eth_writeReg(regs->mac_enable, macEn.wd); 

//	printf("[bmc12500Eth_resetMac] Leave\n");
	return(NFX_OK);
} // bmc12500Eth_resetMac 

/*
 *------------------------------------------------------------------------------
 * Function Name:	bmc12500Eth_setMacSpeed
 *
 * Description: 	Sets up the MAC to the speed auto-detected by the
 *					physical layer.
 *
 * Created by:		Frank Salzmann
 *
 * Data Created:	October 11, 2001
 * 					
 * Returns: 		NFX_OK
 *------------------------------------------------------------------------------
 */
int 
bmc12500Eth_setMacSpeed(bc1250EthCfg_t *pbmcEthCfg)
{
    bmc12500_macCfgr_t    cfg;
    bmc12500_macFrmCfgr_t frmCfg;
	bc1250Eth_regs_t	  *r;

	r = &pbmcEthCfg->regs;

	//-- Read current register values 
    cfg.wd    = bmc12500Eth_readReg(r->mac_cfg);
    frmCfg.wd = bmc12500Eth_readReg(r->mac_frame_cfg);

	//-- Setup appropriate bits in cfg and frame registers.
    switch( pbmcEthCfg->link.speed ) 
	{
		case speed_10:
			frmCfg.bits.ifg_rx      = 18;
			frmCfg.bits.ifg_tx      = 18;
			frmCfg.bits.ifg_thresh  = 18;
			frmCfg.bits.slot_offset = 64;

			cfg.bits.burst_en		= 0;
			cfg.bits.speed_sel      = 0;		// 10Mbps
			break;

		case speed_100:
			frmCfg.bits.ifg_rx      = 18;
			frmCfg.bits.ifg_tx      = 18;
			frmCfg.bits.ifg_thresh  = 18;
			frmCfg.bits.slot_offset = 64;

			cfg.bits.burst_en		= 0;
			cfg.bits.speed_sel      = 1;		// 100Mbps
			break;

		case speed_1000:
			frmCfg.bits.ifg_rx      = 18;
			frmCfg.bits.ifg_tx      = 60;
			frmCfg.bits.ifg_thresh  = 18;
			frmCfg.bits.slot_offset = 512;

			cfg.bits.speed_sel      = 2;		// 1000Mbps
			cfg.bits.burst_en		= 1;
			break;

		default:
			return NFX_OK;
    }

    //-- Write bits back to Mac registers.
    bmc12500Eth_writeReg(r->mac_frame_cfg,frmCfg.wd);
    bmc12500Eth_writeReg(r->mac_cfg, cfg.wd);

	return NFX_OK;
} // bmc12500Eth_setMacSpeed

/*
 *------------------------------------------------------------------------------
 * Function Name:	bmc12500Eth_setMacDuplex
 *
 * Description: 	Sets up the MAC to the duplex auto-detected by the
 *					physical layer.
 *
 * Created by:		Frank Salzmann
 *
 * Data Created:	October 12, 2001
 * 					
 * Returns: 		NFX_OK
 *------------------------------------------------------------------------------
 */
int 
bmc12500Eth_setMacDuplex(bc1250EthCfg_t *pbmcEthCfg)
{
    bmc12500_macCfgr_t    cfg;
	bc1250Eth_regs_t	  *r;

	r = &pbmcEthCfg->regs;

	//-- Read current register values 
    cfg.wd = bmc12500Eth_readReg(r->mac_cfg);

	//-- Setup appropriate bits in cfg and frame registers.
    switch( pbmcEthCfg->link.duplex ) 
	{
		case half_duplex:
			switch( pbmcEthCfg->link.fc ) 
			{
				case fc_disabled:
					cfg.bits.fc_sel = 0;
					cfg.bits.hdx_en = 1;
					cfg.bits.fc_cmd = 0;
					break;

				case fc_collision:
					cfg.bits.fc_sel = 0;
					cfg.bits.hdx_en = 1;
					cfg.bits.fc_cmd = 1;
					break;

				case fc_carrier:
					cfg.bits.fc_sel = 0;
					cfg.bits.hdx_en = 1;
					cfg.bits.fc_cmd = 2;
					break;

				case fc_frame:		// not valid in half duplex
				default:			// invalid selection 
					return NFX_OK;
			}
			break;

		case full_duplex:
			switch( pbmcEthCfg->link.fc ) 
			{
				case fc_disabled:
					cfg.bits.fc_sel = 0;
					cfg.bits.hdx_en = 0;
					cfg.bits.fc_cmd = 0;
					break;

				case fc_frame:
					cfg.bits.fc_sel = 0;
					cfg.bits.hdx_en = 0;
					cfg.bits.fc_cmd = 0;
					break;

				case fc_collision:    // not valid in full duplex
				case fc_carrier:      // not valid in full duplex
				default:
					return NFX_OK;
			}
			break;
    }

    //-- Write bits back to Mac registers.
    bmc12500Eth_writeReg(r->mac_cfg, cfg.wd);

	return NFX_OK;
} // bmc12500Eth_setMacDuplex

char pause_addr[] = {0x01, 0x80, 0xC2, 0x00, 0x00, 0x01,};
/*
 *------------------------------------------------------------------------------
 * Function Name:	bmc12500Eth_setAddr
 *
 * Description: 	Program the local mac address.
 *
 * Created by:		Frank Salzmann
 *
 * Data Created:	October 12, 2001
 * 					
 * Returns: 		NFX_OK
 *------------------------------------------------------------------------------
 */
int
bmc12500Eth_setAddr(bc1250EthCfg_t *cfg) 
{
    int i, j, offset, idx = 0;
    uint64 addr = 0;
	unsigned char *addrBytes;
	bc1250Eth_regs_t *r;
	uchar8 mac_addr[6][6];

	/* 
     * Assign same backplane addresses used for Cheetah to Garfield, so that 
     * both systems work the same way without checking to do things differently 
     */
	if (isFP()) {
		r = &cfg->regs;
		if (cfg->macId == 0) 
			offset = 0;
		else // macId == 1, use different set of addresses
			offset = 3;

        for (i = 1; i <= 2; i++) {
            bp_getMyMacAddr(nfxMySlotId, NFX_NFP_FP1, i+offset,
                            EEE_USER_DATA, mac_addr[idx++]);
        }
        for (i = 1; i <= 2; i++) {
            bp_getMyMacAddr(nfxMySlotId, NFX_NFP_FP2, i+offset,
                            EEE_USER_DATA, mac_addr[idx++]);
        }
        for (i = 1; i <= 2; i++) {
            bp_getMyMacAddr(nfxMySlotId, NFX_NFP_FP_ANY, i+offset,
                            EEE_USER_DATA, mac_addr[idx++]);
        }

		for (j = 0; j < idx; j++) {
			addr = 0;
			addrBytes = mac_addr[j];
	   		// Pack the bytes into the register, with the first byte transmitted
	   		// in the lowest-order 8 bits of the register.
	   		for (i=0; i < 6; i++) {
				addr |= (((uint64) (*addrBytes)) << (i*8));
				addrBytes++;
	    	}
	    	bmc12500Eth_writeReg(r->mac_addr[j],addr);
		}

	}
	else { // TxRx
		addrBytes = cfg->macAddr;
		r = &cfg->regs;

	    // Pack the bytes into the register, with the first byte transmitted
	    // in the lowest-order 8 bits of the register.
	    for (i=0; i < 6; i++) {
			addr |= (((uint64) (*addrBytes)) << (i*8));
			addrBytes++;
	    }
	    bmc12500Eth_writeReg(r->mac_addr[0],addr);
	}

	addr = 0;
	addrBytes = pause_addr;
    for (i=0; i < 6; i++) {
		addr |= (((uint64) (*addrBytes)) << (i*8));
		addrBytes++;
    }
    bmc12500Eth_writeReg(r->mac_ethernet_addr,addr);

	return NFX_OK;
} /* bmc12500Eth_setAddr */

/*
 *------------------------------------------------------------------------------
 * Function Name:	bmc12500Eth_resetModule
 *
 * Description: 	Resets the module and devce.
 *
 * Created by:		Frank Salzmann
 *
 * Data Created:	April 26, 2001
 * 					
 * Returns: 
 *
 *------------------------------------------------------------------------------
 */
int
bmc12500Eth_resetModule(bc1250EthCfg_t *pbmcEthCfg) 
{
	return 0;
} /* bmc12500Eth_resetModule */

/*
 *------------------------------------------------------------------------------
 * Function Name:	bmc12500Eth_initQ
 *
 * Description: 	Initializes descriptor queue by carving up the 
 *					descriptor memory into individual descriptors and 
 *					linking them together.
 *
 * Created by:		Frank Salzmann
 *
 * Data Created:	October 09, 2001
 *------------------------------------------------------------------------------
 */
int
bmc12500Eth_initQ_tx(bc1250_txdesc_t *qstart, int qLen)
{
	int i;
	address_t val;
	bc1250_txdesc_t *desc;

	desc  = qstart;
	/* Convert to physical address space */
	for(i=1; i < qLen; i++, desc++) {
#ifdef LARGE_MEMORY
		val = virt_to_phys((address_t)(desc+1));
#else
		val = (address_t)((address_t)(desc+1) & ~V2PHYS_ADDR_MASK);
#endif
		desc->dw2 = val & BC1250_DESC_NEXT_DESC;
	}
#ifdef LARGE_MEMORY
	val = virt_to_phys((address_t)qstart);
#else
	val = (address_t)qstart & ~V2PHYS_ADDR_MASK;
#endif
	desc->dw2 = val & BC1250_DESC_NEXT_DESC;
	return(NFX_OK);
} // bmc12500Eth_initQ 

/*
 *------------------------------------------------------------------------------
 * Function Name:	bmc12500Eth_fillRxQ
 *
 * Description: 	Fills a receive queue with empty system data uffers.
 *
 * Created by:		Frank Salzmann
 *
 * Data Created:	April 26, 2001
 *------------------------------------------------------------------------------
 */
int
bmc12500Eth_fillRxQ( bc1250Eth_rxCtrl_t *rxc, int qLen ) 
{
	int i;
	void *pBuf;
	bc1250_rxdesc_t *d;

	//printf("[bmc12500Eth_fillRxQ] Enter\n");

	d = rxc->rxqStart;
	for(i=0; i < qLen; i++, d++)
	{
		pBuf = eee_allocateBuffer(rxc->bufPoolId, rxc->eeeBufSize, 0);
		if( !pBuf ) {
			printf("[bmc12500Eth_fillRxQ]: Err: Cannot alloc free buf.\n");
			return(NFX_ERR);
		}
        eee_attrAssignBuf(pBuf, EEE_ATTR_MAC_RX);
		BC1250_DESC_SET_A_ADDR(d, (uint64)pBuf, rxc->bufOffset);
		BC1250_DESC_SET_A_SIZE(d, rxc->descBufSize);

		pBuf = eee_allocateBuffer(rxc->bufPoolId, rxc->eeeBufSize, 0);
		if( !pBuf ) {
			printf("[bmc12500Eth_fillRxQ]: Err: Cannot alloc free buf.\n");
			return(NFX_ERR);
		}
        eee_attrAssignBuf(pBuf, EEE_ATTR_MAC_RX);
		BC1250_DESC_SET_B_ADDR(d, (uint64)pBuf);
		BC1250_DESC_SET_B_SIZE(d, rxc->descBufSize);
	}
	//printf("[bmc12500Eth_fillRxQ] Leave\n");

	return(NFX_OK);
} // bmc12500Eth_fillRxQ 

/*
 *------------------------------------------------------------------------------
 * Function Name:	bmc12500Eth_rxAppPkt
 *
 * Description: 	Receive packet sent to BMC12500 MAC mgmt layer.
 *
 * Created by:		Frank Salzmann
 *
 * Data Created:	May 12, 2001
 *------------------------------------------------------------------------------
 */
void
bmc12500Eth_rxAppPkt( eee_desc_t *edesc )
{

	uint32        *mid;
	uchar8 mid_a[4];
	void          *msg;
    eee_descBuf_t *bd;
	uint32 i=0;
 
    /*
     * Point to request message portion of data buffer.
     */
    if( !edesc ) {
        return;
    }
 
    bd  = &edesc->bd[1];
printf("bd->buf %x\n",bd->buf);	
    msg = (void *)(bd->buf);
    for(i=0;i<4;i++) {
        mid_a[i] = *(uchar8 *)msg;
	(uchar8 *)msg++;
    }	
printf("mid_a %2x %2x %2x %2x\n",mid_a[0],mid_a[1], mid_a[2], mid_a[3]);    
   mid = (uint32 *)&mid_a;
    //mid = *(uint32 *)msg;
printf("mid %4x\n",*(uint32 *)mid);	
    switch( *(uint32 *)mid )
    {
        case ETH_MID_LINK_STATUS_REQ:
			bmc12500Eth_procLSReq(edesc->hdr.src_port, 
								  (bc1250Eth_ls_reqMsg_t *)msg);
			eee_freePacket(edesc);
			break;

		default:
			bmc12500Eth_diagLbPktReceive(edesc);
	}

	return;
} /* bmc12500Eth_rxAppPkt */

int
bmc12500Eth_procLSReq( uint32 srcPort, bc1250Eth_ls_reqMsg_t *msg )
{

	return NFX_OK;
} // bmc12500Eth_procLSReq

/*
 *------------------------------------------------------------------------------
 * Function Name:	bmc12500Eth_writeReg
 *
 * Description: 	Writes 64-bit value to a device register.
 *
 * Created by:		Frank Salzmann
 *
 * Data Created:	October 08, 2001
 *------------------------------------------------------------------------------
 */
int
bmc12500Eth_writeReg(volatile uint64 *reg, uint64 val) 
{
#ifdef LINUX_TEST
#else
	*(uint64 *)(PHYS_TO_K1((address_t)reg)) = val;
#endif
	return NFX_OK;
} /* bmc12500Eth_writeReg */

/*
 *------------------------------------------------------------------------------
 * Function Name:	bmc12500Eth_readReg
 *
 * Description: 	Returns 64-bit value read from device register.
 *
 * Created by:		Frank Salzmann
 *
 * Data Created:	October 08, 2001
 *------------------------------------------------------------------------------
 */
uint64
bmc12500Eth_readReg(volatile uint64 *reg) 
{
	uint64 val;
#ifdef LINUX_TEST
	return 0;
#else
	//	printf("readReg: %016llx\n", reg);

	val = *(uint64 *)(PHYS_TO_K1((address_t)reg));
	return(val);
#endif
} /* bmc12500Eth_readReg */

/*
 *------------------------------------------------------------------------------
 * Function Name:	bmc12500Eth_linkPoll
 *
 * Description: 	Polls for ethernet link status and statistics. 
 *
 * Created by:		Frank Salzmann
 *
 * Data Created:	November 04, 2001
 *------------------------------------------------------------------------------
 */
int
bmc12500Eth_linkPoll( int32 tmr,                /* Timer Handle */
					  bc1250EthCfg_t *cfg,
					  void *user2,
					  void *user3,
					  void *user4 ) 
{
#ifdef NFP_TXRX
	int status, i;

	/* report link status change events */
	if (isCheetah()) {
        nfx_eventMsg_t msg;

		for (i = cfg->macId; i <= cfg->macId+1; i++) {
			status = lsi_get_linkstatus(i);
			if (status != bmc12500_link_status[i]) {
				bmc12500_link_status[i] = status;
				bsd_ChangeLinkStatus(i, status);
				/* notify event manager */
				bzero(&msg, sizeof(msg));
				msg.msgType = NFX_EVENT_POST;
				msg.eventType = NFX_EVENT_PORT;
				msg.param.portEvent.portAddr = PACK_UNICAST_PORT_ADDRESS(
											   nfxMySlotId,
											   nfxMyCPUId,
											   cfg->appId,
											   EEE_USER_DATA,
											   NFX_COS_HIGH,
											   i);
				if (status)
					msg.param.portEvent.state = EVENT_STATE_UP;
				else
					msg.param.portEvent.state = EVENT_STATE_DOWN;

				event_postEvent(&msg);
			}
		}
	}
#endif
#if defined(AGILE_HARDWARE)
	return NFX_OK;
#endif

	if( cfg->link.loopback == lb_int )
		return 0;

	//-- Look for changes in physical layer configuration.
	if( bmc12500Eth_mii_poll( cfg ) ) {
		bmc12500Eth_setMacSpeed( cfg );
		bmc12500Eth_setMacDuplex( cfg );
	}
	return(NFX_OK);
} // bmc12500Eth_linkPoll

/*
 *------------------------------------------------------------------------------
 * Function Name:	bmc12500Eth_setPromiscMode
 *
 * Description: 	Places Mac Rx into promiscuous mode.
 *
 * Created by:		Frank Salzmann
 *
 * Data Created:	November 04, 2001
 *------------------------------------------------------------------------------
 */
int
bmc12500Eth_setPromiscMode( bc1250EthCfg_t *cfg, int mode )
{
	uint64           val;
	bc1250Eth_regs_t *r;

	r = &cfg->regs;

	//-- MAC Rx Filter Register
	val = bmc12500Eth_readReg(r->mac_adfilter_cfg);
	if( mode == BC1250_SET )
		val |= 1;
	else
		val &= ~0x1ll;

	bmc12500Eth_writeReg(r->mac_adfilter_cfg, val);
	return(NFX_OK);
} 

/*
 *------------------------------------------------------------------------------
 * Function Name:	bmc12500Eth_initPktFifo
 *
 * Description: 	Initializes the bmc12500 packet FIFO interface.
 *
 * Created by:		Frank Salzmann
 *
 * Data Created:	February 23, 2002
 * 					
 * Returns:			NFX_OK 
 *------------------------------------------------------------------------------
 */
int
bmc12500Eth_initPktFifo( bc1250EthCfg_t *pbmcEthCfg )
{
	uint32           i;
	uint32           align_diff;
	uint32           size_rx;
	uint32           size_tx;
	uchar8           *qstart;
	bc1250Eth_txCtrl_t *txc;
	bc1250Eth_rxCtrl_t *rxc;

	//printf("[bmc12500Eth_initPktFifo] Enter\n");

	/*
	 * Allocate and align memory on 64-bit boundry
	 * for receive and transmit descriptor rings. 
	 */
	size_tx = sizeof(bc1250_txdesc_t) * BC1250_MAC_TX_Q_LEN;
	size_rx = sizeof(bc1250_rxdesc_t) * BC1250_MAC_RX_Q_LEN;

	/* Setup sibyte DMA descriptor rings */
	for( i=0; i < BC1250_MAC_TX_Q_CNT; i++ )
	{
		txc = &pbmcEthCfg->txc[i];
		txc->txqId = i;

		qstart = (uchar8 *)eee_ramAllocAligned(size_tx + 64);

		//printf("txq start: %x: %d bytes allocated\n", qstart, size_tx+64);

		align_diff = (address_t)qstart & 0x00000007;

		if( align_diff )
			align_diff = 8 - align_diff;
		qstart += align_diff;

		memset(qstart, 0, size_tx);
		txc->txqStart = (bc1250_txdesc_t *)qstart;
		txc->txCur = txc->txCln = txc->txqStart;
		txc->txqEnd = txc->txqStart + BC1250_MAC_TX_Q_LEN;
#if BC1250ETH_TXQ_LIST
		bmc12500Eth_initQ_tx( txc->txqStart, BC1250_MAC_TX_Q_LEN );
#endif
	}

	for( i=0; i < BC1250_MAC_RX_Q_CNT; i++ )
	{
		rxc = &pbmcEthCfg->rxc[i];
		rxc->rxqId = i;

#if defined(NFP_FP)

#ifdef NFX_SMP
		NFX_SPIN_INIT(&rxc->lock, SPINLOCK_LEVEL_UNKNOWN);
#endif
		if( i == 1 ) {
			rxc->bufPoolId   = EEE_TXRX_POOL;
			rxc->eeeBufSize  = EEE_BUF_LARGE;
			rxc->descBufSize = EEE_BUFSIZE_LARGE;
			rxc->bufOffset   = 0;
	
		}
		else {
			rxc->bufPoolId   = EEE_SHARED_POOL;
			rxc->eeeBufSize  = EEE_BUF_ULTRA;
			rxc->descBufSize = EEE_BUFSIZE_ULTRA;
#if defined(EEE_NEW_BUFSIZE)
			rxc->bufOffset   = 64;
#else
			rxc->bufOffset   = 128;
#endif
		}
#else	// TxRx
		rxc->bufPoolId   = EEE_SHARED_POOL;
		rxc->eeeBufSize  = EEE_BUF_LARGE;
		rxc->descBufSize = EEE_BUFSIZE_LARGE;
		rxc->bufOffset   = 0;
#endif

		
		qstart = (uchar8 *)eee_ramAllocAligned(size_rx + 64);

		//printf("rxq start: %x: %d bytes allocated\n", qstart, size_rx+64);
		memset(qstart, 0, size_rx);
		align_diff = (address_t)qstart & 0x00000007;
		if( align_diff )
			align_diff = 8 - align_diff;
		qstart += align_diff;

		rxc->rxqStart = (bc1250_rxdesc_t *)(qstart);
		rxc->rxCur = rxc->rxqStart;
		rxc->rxqEnd = rxc->rxqStart + BC1250_MAC_RX_Q_LEN;
		rxc->avail = BC1250_MAC_RX_Q_LEN;
		bmc12500Eth_fillRxQ( rxc, BC1250_MAC_RX_Q_LEN );
	}

	// Initialize Fifo and Dma Registers.. 
	bmc12500Eth_initDmaRegs( pbmcEthCfg ); 
	bmc12500Eth_initFifoRegs( pbmcEthCfg ); 
	bmc12500Eth_startMacFifo( pbmcEthCfg );

	//printf("[bmc12500Eth_initPktFifo] Leave\n");
	return( NFX_OK ); 
} // bmc12500Eth_initPktFifo 

/*
 *------------------------------------------------------------------------------
 * Function Name:	bmc12500Eth_initLinkTxqs
 *
 * Description: 	
 *
 * Created by:		Frank Salzmann
 *
 * Data Created:	February 25, 2002
 * 					
 * Returns: 		NFX_OK
 *------------------------------------------------------------------------------
 */
int
bmc12500Eth_initLinkTxqs(bc1250_ltc_t **txq) 
{
	uint32  i;
	uint32  qSize;
	uchar8 *qStart;
	bc1250_ltc_t *ltc;

	qSize = sizeof( bc1250_ld_t) * BC1250_LINK_TX_Q_LEN; 

	for(i=0; i <  BC1250_LINK_TX_Q_CNT; i++) {

		//-- Allocate control structure 
		ltc = (bc1250_ltc_t *)eee_ramAllocAligned(sizeof(bc1250_ltc_t));
		if( !ltc ) 
			panic("bmc12500Eth_initLinkTxqs: ramAlloc: Ctrl Struct: %d\n", i);
		
		memset(ltc, 0, sizeof(bc1250_ltc_t));

		txq[i] = ltc;

		//-- Allocate the link pre-transmit queue.
		qStart = (uchar8 *)eee_ramAllocAligned(qSize);
		if( !qStart )
			panic("bmc12500Eth_initLinkTxqs: ramAlloc: Q: %d\n", i);

		//printf("rxq start: %x: %d bytes allocated\n", qStart, qSize);
		memset(qStart, 0, qSize);
		ltc->qStart  = (bc1250_ld_t *)qStart;
		ltc->qEnd    = (bc1250_ld_t *)qStart + BC1250_LINK_TX_Q_LEN;
		ltc->qAddCur = (bc1250_ld_t *)qStart;
		ltc->qRemCur = (bc1250_ld_t *)qStart;
	}

	return NFX_OK;
} // bc1250Eth_initLinkTxqs

/*
 *------------------------------------------------------------------------------
 * Function Name:	bmc12500Eth_initFifoRegs
 *
 * Description: 	This function initializes BMC12500 Fifo interface
 *					specific registers.
 *
 * Created by:		Frank Salzmann
 *
 * Data Created:	February 23, 2002
 * 					
 * Returns: 		NFX_OK
 *------------------------------------------------------------------------------
 */
int
bmc12500Eth_initFifoRegs(bc1250EthCfg_t *cfg) 
{
	int i;
	bc1250Eth_regs_t *r;

//	printf("[bmc12500Eth_initFifoRegs] Enter\n");
	r = &cfg->regs;

	{	//-- MAC Enable Register
		bmc12500Eth_writeReg(r->mac_enable, 0); // Bring out of reset.
	}

	{	//-- MAC Transmit DMA Control Register
		bmc12500Eth_writeReg(r->mac_txd_ctl, 0x11);  
	}

	{	//-- MAC Fifo Threshold Register.
		bmc12500_mac_fifo_thresh_r_t macFifoThresh;
		macFifoThresh.wd = 0;
		macFifoThresh.bits.tx_wr_thresh = 8;	// From sample code.
		macFifoThresh.bits.tx_rd_thresh = 16;	// From sample code.
		macFifoThresh.bits.tx_rl_thresh = 8;	// From sample code.
		macFifoThresh.bits.rx_rd_thresh = 4;	// From sample code.
		macFifoThresh.bits.rx_rl_thresh = 8;	// From sample code.
		macFifoThresh.bits.enc_fc_thresh = 4;	// default
		bmc12500Eth_writeReg(r->mac_thrsh_cfg, macFifoThresh.wd);
	}

	{	//-- MAC Frame Configuration Register.
		bmc12500_macFrmCfgr_t macFrameCfg;
		macFrameCfg.wd = 0;
		macFrameCfg.bits.backoff_sel	= 1;
		macFrameCfg.bits.min_framesz	= 64;
		macFrameCfg.bits.max_framesz	= 9216;
		bmc12500Eth_writeReg(r->mac_frame_cfg, macFrameCfg.wd);
	}

	{	//-- MAC VLAN Tag Register
		bmc12500Eth_writeReg(r->mac_vlantag, 0);
	}

	{	//-- MAC Addr Exact Match Table
		for(i=0; i < 8; i++)
			bmc12500Eth_writeReg(r->mac_addr[i], 0);
	}

	{	//-- MAC Addr Hash Table
		for(i=0; i < 8; i++)
			bmc12500Eth_writeReg(r->mac_hash[i], 0);
	}

	{	//-- MAC DMA Channel Mapping Table
		for(i=0; i < 4; i++) {
#if defined(NFP_FP)
			bmc12500Eth_writeReg(r->mac_chlo[i], 0xFFFFFFFFFFFFFFFEll);
			bmc12500Eth_writeReg(r->mac_chup[i], 0xFFFFFFFFFFFFFFFEll);
#else
			bmc12500Eth_writeReg(r->mac_chlo[i], 0);
			bmc12500Eth_writeReg(r->mac_chup[i], 0);
#endif
		}
	}
	
	//-- MAC Rx Filter Register
	if( isFP() )
		bmc12500Eth_writeReg(r->mac_adfilter_cfg, 0x21); // Set to promiscuous mode
	else
		bmc12500Eth_writeReg(r->mac_adfilter_cfg, 0x21); // Set to promiscuous mode
	

	{	//-- MAC Cfg Register.
		bmc12500_macCfgr_t macCfg;

		macCfg.wd = 0;
		macCfg.bits.tx_hold_sop_en 	= 1;
		macCfg.bits.retry_en       	= 1;
		macCfg.bits.ret_drpreq_en	= 0;		
		macCfg.bits.ret_ufl_en		= 1;
		macCfg.bits.burst_en		= 0; 
		macCfg.bits.tx_pause_cnt	= 5;	// 16k slot times
		macCfg.bits.ap_stat_en		= 0;
		macCfg.bits.drp_errpkt_en   = 1;	// Drop pkt on any rx err.
		macCfg.bits.loopback_sel    = 0;	// Internal Loopback
		macCfg.bits.ss_en           = 1;	// Magic broadcom bit always set
		macCfg.bits.fast_sync		= 1;
		macCfg.bits.rx_ch_sel		= 42;	// Set offset in nibbles: byte position == 15
        macCfg.bits.fc_sel          = 0;    // don't force flow control on
        macCfg.bits.hdx_en          = 0;    // full duplex
        macCfg.bits.fc_cmd          = 0;
        macCfg.bits.bypass_sel      = 1;    // select FIFO mode
        macCfg.bits.bypass_ifg      = 15;   // interpacket gap
        macCfg.bits.bypass_16       = 1;    // interpacket gap
        macCfg.bits.bypass_cfg      = 1;    // select encoded FIFO mode
        macCfg.bits.bypass_fcs_chk  = 0;    // bypass CRC check
        macCfg.bits.tx_clk_edge 	= 0; 	// rising edge 
		
		bmc12500Eth_writeReg(r->mac_cfg, macCfg.wd);
	}

	//-- Init RMON Regs
	bmc12500Eth_initRmonRegs( cfg );

	//-- Set interface to operate at 1000Mb/s
    {
        bmc12500_macCfgr_t    cfg;
        bmc12500_macFrmCfgr_t frmCfg;

        //-- Read current register values
        cfg.wd    = bmc12500Eth_readReg(r->mac_cfg);
        frmCfg.wd = bmc12500Eth_readReg(r->mac_frame_cfg);

        frmCfg.bits.ifg_rx      = 18;
        frmCfg.bits.ifg_tx      = 60;
        frmCfg.bits.ifg_thresh  = 18;
        frmCfg.bits.slot_offset = 512;

        cfg.bits.speed_sel      = 2;        // 1000Mbps
        cfg.bits.burst_en       = 1;

        //-- Write back new values
        bmc12500Eth_writeReg(r->mac_frame_cfg,frmCfg.wd);
        bmc12500Eth_writeReg(r->mac_cfg, cfg.wd);
	}

//	printf("[bmc12500Eth_initFifoRegs] Leave\n");
	return(NFX_OK);
} /* bmc12500Eth_initFifoRegs */

/*
 *------------------------------------------------------------------------------ * Function Name:   bmc12500Eth_startMacFifo
 *
 * Description:     Enables the BMC12500 Rx/Tx MAC and Rx/Tx DMA channels.
 *
 * Created by:      Frank Salzmann
 *
 * Data Created:    October 09, 2001
 *
 * Returns:         NFX_OK
 *------------------------------------------------------------------------------ */
int
bmc12500Eth_startMacFifo(bc1250EthCfg_t *pbmcEthCfg)
{
    bc1250Eth_regs_t   *regs;
    bmc12500_macEnar_t macEn;

    printf("[bmc12500Eth_startMacFifo] Enter\n");
    regs = &pbmcEthCfg->regs;

    //-- MAC Enable Register
    macEn.wd = bmc12500Eth_readReg(regs->mac_enable);
    macEn.bits.rxdma_en  = BC1250_MAC_RX_Q_CNT == 1 ? 1:3;  // Enable Channel 0 and 1
    macEn.bits.txdma_en  = BC1250_MAC_TX_Q_CNT == 1 ? 1:3;
    macEn.bits.byp_rx_en = 1;
    macEn.bits.byp_tx_en = 1;
    bmc12500Eth_writeReg(regs->mac_enable, macEn.wd);

    printf("[bmc12500Eth_startMacFifo] Leave\n");
    return(NFX_OK);
} // bmc12500Eth_startMacFifo

