/* vi:ts=4  
 *-----------------------------------------------------------------
 *
 * Name: bsd-api.h
 *
 * RCS:  $Id: bsd-api.h,v 1.11 2002/10/30 19:45:07 swang Exp $
 *
 * Copyright (C) 2001, Agile Storage, Inc.
 *
 * Description: general header file to BSD TCP/IP 
 *
 * Created by:  Su Wang
 *
 * Date Created: 08/09/01
 *
 *-----------------------------------------------------------------
 */

#ifndef _BSD_API_H
#define _BSD_API_H

typedef uint32          ipa_t, *ipa_ptr_t;

struct rtentry;
struct ifnet;
struct socket;
struct mbuf;
struct sockaddr;
struct inpcb;

//#define debugPrintf(a)  printf a
#define debugPrintf(a)

#define uiPrintf      printf


extern char* bsd_ifnamebuf[];

/* type of input/output buffer to socket */
#define BSD_BUF_FLAT  1
#define BSD_BUF_DESCR 2

#define BSD_SINGLE       1
#define BSD_ALL       2

/* bsd universal action definition */
#define BSD_UP        1 
#define BSD_DOWN      2 
#define BSD_ADD       3
#define BSD_DEL       4
#define BSD_MOD		  5

/* IP protcols, mirrors in.h. do not change numbers they */
/* reflects the real protocol number*/
#define	BSD_IPPROTO_IP		0		/* dummy for IP */
#define	BSD_IPPROTO_ICMP	1		/* control message protocol */
#define	BSD_IPPROTO_IGMP	2		/* group mgmt protocol */
#define	BSD_IPPROTO_TCP		6		/* tcp */
#define	BSD_IPPROTO_UDP		17		/* user datagram protocol */

#define	BSD_IPPROTO_MAX		256

extern char prnt_ip[64];
extern char prnt_mac[64];
extern char prnt_dev[64];

/* utility functions */
ushort16 bsd_GetFreePort(ushort16);
char*	 bsdPrintPort(int, int, int, int);
void	 bsd_ShowPkt(char*, int);
ushort16 bsd_ChkSum(ushort16*, int);
int32    bsd_AdjBuffer(struct mbuf*, int);
int32    bsd_ModBuffer(struct mbuf*, uint32, char*, uint32);
int32    bsdMasklen(void*);
int32	 bsd_IsSupernet(uint32, uint32);
char*    bsdPrintIntf(struct ifnet*);
char*	 bsdPrintIntfWithVif(struct ifnet* ifp);
int32    bsdIpNtoS(ipa_ptr_t, char*, int);
int32    bsdIpStoN(char8* str, ipa_ptr_t addr);
char*    bsdPrintIpa(ipa_t);
int32    bsdMacStoN(char*, char*);
int32    bsdMacNtoS(char*, char*, int);
char*    bsdPrintMac(char*);
void	 bsdPortToString2(int, int, int, int, char*, int);
void     bsdPortToString(char*, char*);
char*    bsdPrintPort(int, int, int, int);
int		 bsdDTypeStoN(char*);
short16  bsd_getLinkId(struct inpcb*);
void     bsd_log_f(int32, char8*, ...);
int32	 bsd_FillIpmStatsMsg(int mtype, void *msg, void	*parm);
void     bsd_FillIpmMsg(int, void*, void*, void*);
void	 bsd_logErr(char*, int32, int32, char*);
int32	 bsd_setLinkId( void *in_pcb, int);
void	 bsd_SetPcbNHIp(void *inp, uint32 ip);
char*	 bsdPrintPortFromSSC(char	*port);
void	 bsd_InitApi();
uint32	 bsd_GetPcbNHIp(void *inp);
char*	bsd_GetPcbNHMac(void *inp);
#endif 
