#include <nfx-incl.h>
#include <exports/eee-api.h>
#include "../sm-bsd-sys/flags.h"
#include "../sm-bsd-sys/cdefs.h"
#include "../sm-bsd-sys/types.h"
#include "../sm-bsd-sys/errno.h"
#include "../sm-bsd-sys/mbuf.h"
#include "../sm-bsd-sys/domain.h"
#include "../sm-bsd-sys/protosw.h"
#include "../sm-bsd-sys/socket.h"
#include "../sm-bsd-sys/socketvar.h"
#include "../sm-bsd-sys/syslog.h"

#include "../sm-bsd-net/if.h"
#include "../sm-bsd-net/if_var.h"
#include "../sm-bsd-net/if_dl.h"
#include "../sm-bsd-net/route.h"

#include "../sm-bsd-netinet/in.h"
#include "../sm-bsd-netinet/in_var.h"
#include "../sm-bsd-netinet/ip.h"
#include "../sm-bsd-netinet/ip_var.h"
#include "../sm-bsd-netinet/udp.h"
#include "../sm-bsd-netinet/udp_var.h"
#include "../sm-bsd-netinet/tcp.h"
#include "../sm-bsd-netinet/tcp_timer.h"
#include "../sm-bsd-netinet/tcp_var.h"
#include "../sm-bsd-netinet/ip_icmp.h"
#include "../sm-bsd-netinet/ip_icmp.h"
#include "../sm-bsd-netinet/icmp_var.h"
#include "../sm-bsd-netinet/if_ether.h"

#include "exports/libc.h"
#include "exports/bsdnet-api.h"
#include "exports/bsdsock-api.h"
#include "../sm-ipm/ipm.h"
#include "bsd-arpsniff.h"

extern int tcp_debug_flag;

/* function declarations */
int32 do_dominit(int32 argc, char8 *argv[]);
int32 do_ipintfadd(int32 argc, char8 *argv[]);
int32 do_ipintfdel(int32 argc, char8 *argv[]);
int32 do_intfchg(int32 argc, char8 *argv[]);
int32 do_intfshow(int32 argc, char8 *argv[]);
int32 do_iprtadd(int32 argc, char8 *argv[]);
int32 do_iprtdel(int32 argc, char8 *argv[]);
int32 do_iprtshow(int32 argc, char8 *argv[]);
int32 do_arpadd(int32 argc, char8 *argv[]);
int32 do_arpdel(int32 argc, char8 *argv[]);
int32 do_arpshow(int32 argc, char8 *argv[]);
int32 do_sockshow(int32 argc, char8 *argv[]);
int32 do_ping(int32 argc, char8 *argv[]);
int32 do_showsrt(int32 argc, char8 *argv[]);
int32 do_showmbuf(int32 argc, char8 *argv[]);
int32 do_showipstats(int32 argc, char8 *argv[]);
int32 do_clearipstats(int32 argc, char8 *argv[]);
int32 do_showudpstats(int32 argc, char8 *argv[]);
int32 do_clearudpstats(int32 argc, char8 *argv[]);
int32 do_showtcpstats(int32 argc, char8 *argv[]);
int32 do_cleartcpstats(int32 argc, char8 *argv[]);
int32 do_showicmpstats(int32 argc, char8 *argv[]);
int32 do_clearicmpstats(int32 argc, char8 *argv[]);
int32 do_showarpstats(int32 argc, char8 *argv[]);
int32 do_cleararpstats(int32 argc, char8 *argv[]);
int32 do_showbsdmem(int32 argc, char8 *argv[]);
int32 do_tcptrace(int32 argc, char8 *argv[]);
int32 do_tcpshowconfig(int32 argc, char8 *argv[]);
#if defined(LINUX_TEST)
int32 do_ipsend(int32 argc, char8 *argv[]);
int32 do_socksend(int32 argc, char8 *argv[]);
int32 do_tcploop(int32 argc, char8 *argv[]);
int32 do_tcploopsend(int32 argc, char8 *argv[]);
#endif /* LINUX_TEST */
int32 do_sockadd(int32 argc, char8 *argv[]);
int32 do_sockdel(int32 argc, char8 *argv[]);

void bsd_ShowIpStats(struct ipstat *ips);
void bsd_ClearIpStats(struct ipstat *ips);
void bsd_ShowUdpStats(struct udpstat *ups);
void bsd_ClearUdpStats(struct udpstat *ups);
void bsd_ShowTcpStats(struct tcpstat *tps);
void bsd_ClearTcpStats(struct tcpstat *tps);
void bsd_ShowIcmpStats(struct icmpstat *cps);
void bsd_ClearIcmpStats(struct icmpstat *cps);
void bsd_ShowArpStats(struct arpstat *aps);
void bsd_ClearArpStats(struct arpstat *aps);
void bsd_ShowMemStats(void);

/*********************************************************************
 IP Related UI Commands
*********************************************************************/								      

#if defined(NFP_TXRX) || defined(PROM)
/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Su Wang
 *
 * Date Created:
 *
 *-----------------------------------------------------------------
 */
int32
do_bsd(int32 argc, char8 *argv[])
{

  
  if (argc == 1)
	goto printme;

  argc--;
  argv++;

  /* interface */
  if (strcmp(argv[0], "ip")==0)
	{
		argc--;
		argv++;

		if (!argc)
			goto print_ip;
		
		if (strcmp(argv[0], "add") == 0) {
			if (do_ipintfadd(argc, argv) == -1)
				goto print_ip;
		} else if (strcmp(argv[0], "del") == 0) {
			if (do_ipintfdel(argc, argv) == -1)
				goto print_ip;
		} else if (strcmp(argv[0], "set") == 0) {
			if (do_intfchg(argc, argv) == -1)
				goto print_ip;
		} else if (strcmp(argv[0], "show")==0) {
			if (do_intfshow(argc, argv)== -1)
				goto print_ip;
		} else {
print_ip:
			uiPrintf("bsd ip set  <intf> 0/1:         dis/enable intf\n");
			uiPrintf("bsd ip add  <intf> <ip> <mask>: add ip intf\n");
			uiPrintf("bsd ip show <intf>:             show intf\n");
			uiPrintf("bsd ip del  <intf> <ip>:        del. ip intf\n");
		}
	} /* route */
  else if (strcmp(argv[0], "rt")==0)
	{
		argc--;
		argv++;

		if (!argc)
			goto print_rt;
		
		if (strcmp(argv[0], "add") == 0) {
			if (do_iprtadd(argc, argv) == -1)
				goto print_rt;
		} else if (strcmp(argv[0], "del") == 0) {
			if (do_iprtdel(argc, argv) == -1)
				goto print_rt;
		} else if (strcmp(argv[0], "show") == 0) {
			if (do_iprtshow(argc, argv) == -1)
				goto print_rt;
		} else {
print_rt:
			uiPrintf("bsd rt add <ip> <mask> <gateway>: add static rt\n");
			uiPrintf("bsd rt del <ip> <mask> <gateway>: del static rt\n");
			uiPrintf("bsd rt show:                      show static rt\n");
		}
	} /* arp */
  else if (strcmp(argv[0], "arp")== 0)
	{
		argc--;
		argv++;
		
		if (!argc)
			goto print_arp;		

		if (strcmp(argv[0], "add") == 0) {
			if (do_arpadd(argc, argv) == -1)
				goto print_arp;
  		} else if (strcmp(argv[0], "del") == 0) {
			if (do_arpdel(argc, argv) == -1)
				goto print_arp;
		} else if (strcmp(argv[0], "show")== 0 ) {
			if (do_arpshow(argc, argv) == -1)
				goto print_arp;
		} else {
print_arp:
			uiPrintf("bsd arp add <ip> <mac>: add static arp \n");
			uiPrintf("bsd arp del <ip> :      delete arp \n");
			uiPrintf("bsd arp show:           show arp \n");
		}
	} /* socket */
  else if (strcmp(argv[0], "sock")==0)
	{
		argc--;
		argv++;

		if (!argc)
			goto print_sock;	
		
		if (strcmp(argv[0], "add") == 0) {
			if (do_sockadd(argc, argv) == -1)
				goto print_sock;
		} else if (strcmp(argv[0], "del") == 0) {
			if (do_sockdel(argc, argv) == -1)
				goto print_sock;
		} else if (strcmp(argv[0], "show")==0) {
			if (do_sockshow(argc, argv) == -1)
				goto print_sock;
		} else if (strcmp(argv[0], "tcptrace")==0) {
			if (do_tcptrace(argc, argv) == -1)
				goto print_sock;
		} else if (strcmp(argv[0], "showtcpcfg")==0) {
			if (do_tcpshowconfig(argc, argv) == -1)
				goto print_sock;
		
		} else {
print_sock:
			uiPrintf("bsd sock show <tcp/udp/all>: show socket config\n");
			uiPrintf("bsd sock add  server/client <proto><addr><port>: open a socket\n");
			uiPrintf("bsd sock del  <id>:          delete a socket\n");
			uiPrintf("bsd sock tcptrace 0/1:       disable, enable tcp trace\n");
			uiPrintf("bsd sock showtcpcfg <id>:    show tcp control blk config.\n");  
		}
	}
  else if (strcmp(argv[0], "ping")==0)
	{
		do_ping(argc, argv);
	} /* stats */
  else if (strcmp(argv[0], "stats")==0)
	{
		argc--;
		argv++;
		
		if (!argc)
			goto print_stats;

		if (strcmp(argv[0], "mbuf") == 0) {
			if (do_showmbuf(argc, argv) == -1)
				goto print_stats;
		}
		else if (strcmp(argv[0], "bsdmem")==0) {
			if (do_showbsdmem(argc, argv) == -1)
				goto print_stats;
		}
		else if (strcmp(argv[0], "ip") == 0) {
			if (do_showipstats(argc, argv) == -1)
				goto print_stats;
		}
		else if (strcmp(argv[0], "udp")==0) {
			if (do_showudpstats(argc, argv) == -1)
				goto print_stats;
		}
		else if (strcmp(argv[0], "tcp") == 0) {
			if (do_showtcpstats(argc, argv) == -1)
				goto print_stats;
		}
		else if (strcmp(argv[0], "icmp")==0) {
			if (do_showicmpstats(argc, argv) == -1)
				goto print_stats;
		}
		else if (strcmp(argv[0], "arp")==0) {
			if (do_showarpstats(argc, argv) == -1)
				goto print_stats;
		}
		else if (strcmp(argv[0], "clrip")==0) {
			if (do_clearipstats(argc, argv) == -1)
				goto print_stats;
		}
		else if (strcmp(argv[0], "clrudp")==0) {
			if (do_clearudpstats(argc, argv) == -1)
				goto print_stats;
		}
		else if (strcmp(argv[0], "clrtcp")==0) {
			if (do_cleartcpstats(argc, argv) == -1)
				goto print_stats;
		}
		else if (strcmp(argv[0], "clricmp")==0) {
			if (do_clearicmpstats(argc, argv) == -1)
				goto print_stats;
		}
		else if (strcmp(argv[0], "clrarp")==0) {
			if (do_cleararpstats(argc, argv) == -1)
				goto print_stats;
		}
		else {
print_stats:
			uiPrintf("bsd stats mbuf:      show mbuf usage\n");
			uiPrintf("bsd stats bsdmem:    show bsd mem allocation\n");
			uiPrintf("bsd stats ip:        show ip stats\n");
			uiPrintf("bsd stats udp:       show udp stats\n");
			uiPrintf("bsd stats tcp:       show tcp stats\n");
			uiPrintf("bsd stats icmp:      show icmp stats\n");
			uiPrintf("bsd stats arp:      show arp stats\n");
			uiPrintf("bsd stats clrip:     clear ip stats\n");
			uiPrintf("bsd stats clrudp:    clear udp stats\n");
			uiPrintf("bsd stats clrtcp:    clear tcp stats\n");
			uiPrintf("bsd stats clricmp:   clear icmp stats\n");
			uiPrintf("bsd stats clrarp:    clear arp stats\n");
		}
		
	}
  else if (strcmp(argv[0], "misc")==0) 
	{
		argc--;
		argv++;
		
		if (!argc)
			goto print_misc;

#if !defined(PROM)
		if (strcmp(argv[0], "ipmstat")==0) {
				ipm_PrintStats(&ipm_stats);
		}
		else if (strcmp(argv[0], "clearipm")==0) {
			ipm_ClearStats(&ipm_stats);
		}
		else if (strcmp(argv[0], "arpsniff")==0){
			if (argc == 2)
				arpsCfgPtr->enable = atoi(argv[1]);	
		}
		else if (strcmp(argv[0], "ipminfo")==0) {
			if (argc == 2)
				ipm_Updt_Trig = atoi(argv[1]);
		} else
#endif /* PROM */
#if defined(LINUX_TEST)
		if (strcmp(argv[0], "sendip")==0) {
			if (do_ipsend(argc, argv))
				goto print_misc;
		}
		else if (strcmp(argv[0], "sendsock")==0) {
			if (do_socksend(argc, argv))
				goto print_misc;
		}
		else if (strcmp(argv[0], "tcploop")==0) {
			do_tcploop(argc, argv);
		}
		else if (strcmp(argv[0], "tcpsend")==0) {
			do_tcploopsend(argc, argv);
		} else
#endif /* LINUX_TEST */
			{
print_misc:
#if !defined(PROM)
				uiPrintf("bsd misc ipmstat:      display ipm stats\n");
				uiPrintf("bsd misc clearipm:     clear ipm stats\n");
				uiPrintf("bsd misc ipminfo 0/1:  disable, enable ipm send info msg\n");				
				uiPrintf("bsd misc arpsniff 0/1: disable, enable arp sniffing\n");
#endif /* PROM */
#if defined(LINUX_TEST)
				uiPrintf("bsd misc tcploop server/clinet <ip> <port>: open tcp loopbck tst\n");
				uiPrintf("bsd misc tcpsend <num> <id>: send num pkt to loop bck tcp\n");
				uiPrintf("bsd misc sendip <proto><sa><da><sp><dp>: send ip pkt to upper layer\n");
				uiPrintf("bsd misc sendsock <id><da><dp> : send data to socket\n");
#endif /* LINUX_TEST */	
				uiPrintf("end of help\n");
			}
		
	}
  else if (strcmp(argv[0], "nttcp")==0) {
	  
		argc--;
		argv++;
		
		if (!argc)
			goto print_nttcp;

		if (strcmp(argv[0], "server")==0) 
			//	bsd_InitNttcpServer();
			;
		else {
print_nttcp:
			uiPrintf("bsd nttcp server: open nttcp server\n");
		}
  }
  else
	{
printme:
	  uiPrintf("bsd ping <da> <repeat> <size>: ping program\n");
	  uiPrintf("bsd ip:    ip related commands\n");
	  uiPrintf("bsd rt:    route related commands\n");
	  uiPrintf("bsd arp:   arp related commands\n");
	  uiPrintf("bsd sock:  socket related commands\n");
	  uiPrintf("bsd stats: stats related commands\n");
	  uiPrintf("bsd misc:  misc. commands\n");
	  uiPrintf("bsd nttcp: nttcp commands\n");
	}
  
  return 0;
}


/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Su Wang
 *
 * Date Created:
 *
 *-----------------------------------------------------------------
 */
int32
do_dominit(int32 argc, char8 *argv[])
{
  if (argc != 1)
	{
	  uiPrintf("bsd ipdom: enable ip domain\n");
	  return -1;
	}

//  domaininit();

  return 0;
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Su Wang
 *
 * Date Created:
 *
 *-----------------------------------------------------------------
 */
int32
do_ipintfadd(int32 argc, char8 *argv[])
{
  char nam_buf[8];
  uint32 ip, mask;

  if (argc != 4)
    return -1;

  strcpy(nam_buf, argv[1]);

  printf("ip = %s\n", argv[2]);
  if (bsdIpStoN(argv[2], &ip) == NFX_ERR)
    return -1;
  printf("after conversion %x \n", ntohl(ip));

  printf("mask = %s\n", argv[3]);
  if (bsdIpStoN(argv[3], &mask) == NFX_ERR)
    return -1; 
  printf("after conversion %x \n", ntohl(mask));

  if (bsd_CreateIpIntf(ip, mask, nam_buf, 0, 0) == NFX_ERR)
    {
	  uiPrintf("bsd addipintf <intf> <ip> <mask>: add ip intf\n");
      return -1;
    }

  return 0;
} 

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Su Wang
 *
 * Date Created:
 *
 *-----------------------------------------------------------------
 */
int32
do_ipintfdel(int32 argc, char8 *argv[])
{
  char nam_buf[8];
  uint32 ip;

  if (argc != 3)
    return -1;

	 
  strcpy(nam_buf, argv[1]);
  
  if (bsdIpStoN(argv[2], &ip) == NFX_ERR)
    return -1;
  
  if (bsd_DeleteIpIntf(ip, nam_buf, 0, 0) == NFX_ERR)
    {
      return -1;
    }

  return 0;  
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Su Wang
 *
 * Date Created:
 *
 *-----------------------------------------------------------------
 */
int32
do_intfchg(int32 argc, char8 *argv[])
{
  char nam_buf[8];
  uint32 state;

  if (argc != 3)
    return -1;

  strcpy(nam_buf, argv[1]);

  state = atoi(argv[2]);

  state = (state == 0) ? BSD_DOWN :  BSD_UP;

  if (bsd_ChangeStateIntf(nam_buf, state, 0, 0) == NFX_ERR)
    return -1;

  return 0;
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Su Wang
 *
 * Date Created:
 *
 *-----------------------------------------------------------------
 */
int32
do_intfshow(int32 argc, char8 *argv[])
{
  char nam_buf[8];

  if (argc > 2)
    return -1;

  if (argc == 2)
    {
      strcpy(nam_buf, argv[1]);

      bsd_ShowIntf(BSD_SINGLE, nam_buf, 0, 0);
    }
  else
    bsd_ShowIntf(BSD_ALL, 0, 0, 0);

  return 0;
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Su Wang
 *
 * Date Created:
 *
 *-----------------------------------------------------------------
 */
int32
do_iprtadd(int32 argc, char8 *argv[])
{
  uint32 ip, mask, gw;

    if (argc != 4)
    return -1;
  

  if (bsdIpStoN(argv[1], &ip) == NFX_ERR)
    return -1;

  if (bsdIpStoN(argv[2], &mask) == NFX_ERR)
    return -1; 

  if (bsdIpStoN(argv[3], &gw) == NFX_ERR)
    return -1; 

  if (bsd_ModRoute(ip, mask, gw, 0, BSD_ADD, 0, 0) == NFX_ERR)
    {
      return -1;
    }

  return 0;

}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Su Wang
 *
 * Date Created:
 *
 *-----------------------------------------------------------------
 */
int32
do_iprtdel(int32 argc, char8 *argv[])
{
  uint32 ip, mask, gw;

  if (argc != 4)
    return -1;
  

  if (bsdIpStoN(argv[1], &ip) == NFX_ERR)
    return -1;

  if (bsdIpStoN(argv[2], &mask) == NFX_ERR)
    return -1; 

  if (bsdIpStoN(argv[3], &gw) == NFX_ERR)
    return -1; 

  if (bsd_ModRoute(ip, mask, gw, 0, BSD_DEL, 0, 0) == NFX_ERR)
    {
      return -1;
    }

  return 0;  
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Su Wang
 *
 * Date Created:
 *
 *-----------------------------------------------------------------
 */
int32
do_iprtshow(int32 argc, char8 *argv[])
{
  uint32 ip = 0xffffffff, mask;

  if (!(argc == 1 || argc == 3))
    return -1;

  if (argc == 1)
    {
      bsd_ShowRoute(BSD_ALL, 0, 0, 0, 0,0);
    }
  else
    {
      if (bsdIpStoN(argv[1], &ip) == NFX_ERR)
	return -1;    

      if (bsdIpStoN(argv[2], &mask) == NFX_ERR)
	return -1;   

      bsd_ShowRoute(BSD_SINGLE, 0, ip, mask, 0, 0);
    }
  
  return 0;
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Su Wang
 *
 * Date Created:
 *
 *-----------------------------------------------------------------
 */
int32
do_arpadd(int32 argc, char8 *argv[])
{
  uint32 ip;
  char mac[6];

  if (argc != 3)
    return -1;

  if (bsdIpStoN(argv[1], &ip) == NFX_ERR)
    return -1;

  if (bsdMacStoN(argv[2], mac) == NFX_ERR)
    return -1;

  if (bsd_ModARP(ip, mac, BSD_ADD, 0,0) == NFX_ERR)
    return -1;

  return 0;
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Su Wang
 *
 * Date Created:
 *
 *-----------------------------------------------------------------
 */
int32
do_arpdel(int32 argc, char8 *argv[])
{
  uint32 ip;
  char mac[6];

  if (argc != 2)
    return -1;

  if (bsdIpStoN(argv[1], &ip) == NFX_ERR)
    return -1;


  if (bsd_ModARP(ip, mac, BSD_DEL, 0,0) == NFX_ERR)
    return -1;

  return 0;
}


/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Su Wang
 *
 * Date Created:
 *
 *-----------------------------------------------------------------
 */
int32
do_arpshow(int32 argc, char8 *argv[])
{

  if (argc != 1)
    return -1;


  bsd_ShowARP(0,0,0,0);
  printf("\n\n");
#if !defined(PROM)
  arpS_ShowL2ConnTbl();			/* also display connectivity tbl */
#endif

  return 0;
}

int32
do_sockshow(int32 argc, char8 *argv[])
{
  int what = 0;

  if (argc == 2)    {
      if (strcmp("udp", argv[1]) == 0 || strcmp("UDP", argv[1]) == 0)
		  what = 1;
      else if (strcmp("tcp", argv[1]) == 0 || strcmp("TCP", argv[1]) == 0)
		  what = 2;
      else if (strcmp("rip", argv[1]) == 0 || strcmp("RIP", argv[1]) == 0)
		  what = 3;
      else if (strcmp("all", argv[1]) == 0 || strcmp("ALL", argv[1]) == 0)
		  what = 4;
      else
		  return -1;

      bsd_ShowSock(what, 0, 0, 0);
  }
  else if (argc == 3)
	  bsd_ShowSock(0, (void*)atolx(argv[2]), 0, 0);
  else 
	  bsd_ShowSock(4, 0, 0, 0);	/* default show all sockets */

  return 0; 
}

int32 do_ping(int32 argc, char8 *argv[])
{
  uint32 ip;
  ushort16 size = 60, repeat =3;
  

  if (!(argc ==2 || argc == 4))
    return -1;

  if (bsdIpStoN(argv[1], &ip) == NFX_ERR)
    return -1;

  if (argc == 4)
    {
      repeat = atoi(argv[2]) & 0xffff;
      size  = atoi(argv[3]) & 0xffff;
    }

  bsd_Ping(ip, size, repeat);

  return 0;
}

int32 do_showsrt(int32 argc, char8 *argv[])
{
  if (argc != 1)
    return -1;
#if defined(BSD_SAVE_STATIC)
  bsd_ShowStaticRoute();
#endif /* BSD_SAVE_STATIC */
  return 0;  
}
int32 do_showmbuf(int32 argc, char8 *argv[])
{
  if (argc != 1)
    return -1;

  bsd_ShowMbufStats();
  return 0;
}


int32 do_showipstats(int32 argc, char8 *argv[])
{
  if (argc != 1)
	return -1;

  bsd_ShowIpStats(&ipstat);
  return 0;
}

int32 do_clearipstats(int32 argc, char8 *argv[])
{
  if (argc != 1)
	return -1;

  bsd_ClearIpStats(&ipstat);
  return 0;
}

int32 do_showudpstats(int32 argc, char8 *argv[])
{
  if (argc != 1)
	return -1;

  bsd_ShowUdpStats(&udpstat);
  return 0;  
}

int32 do_clearudpstats(int32 argc, char8 *argv[])
{
  if (argc != 1)
	return -1;

  bsd_ClearUdpStats(&udpstat);
  return 0;
}


int32 do_showtcpstats(int32 argc, char8 *argv[])
{
  if (argc != 1)
	return -1;

  bsd_ShowTcpStats(&tcpstat);
  return 0;
}

int32 do_cleartcpstats(int32 argc, char8 *argv[])
{
  if (argc != 1)
	return -1;

  bsd_ClearTcpStats(&tcpstat);
  return 0;
}

int32 do_showicmpstats(int32 argc, char8 *argv[])
{
  if (argc != 1)
	return -1;

  bsd_ShowIcmpStats(&icmpstat);
  return 0;
}

int32 do_clearicmpstats(int32 argc, char8 *argv[])
{
  if (argc != 1)
	return -1;

  bsd_ClearIcmpStats(&icmpstat);
  return 0;
}

int32 do_showarpstats(int32 argc, char8 *argv[])
{
  if (argc != 1)
	return -1;

  bsd_ShowArpStats(&arpstat);
  return 0;
}

int32 do_cleararpstats(int32 argc, char8 *argv[])
{
  if (argc != 1)
	return -1;

  bsd_ClearArpStats(&arpstat);
  return 0;
}

int32 do_showbsdmem(int32 argc, char8 *argv[])
{
  if (argc != 1)
	return -1;

  bsd_ShowMemStats();
  return 0;
}

int32 do_tcptrace(int32 argc, char8 *argv[])
{
  int state;

    if (argc != 2)
    return -1;

  state = atoi(argv[1]);

  if(state == 0) 
	tcp_debug_flag = 0;
  else
	tcp_debug_flag = 1;
  return 0;
}

int32 do_tcpshowconfig(int32 argc, char8 *argv[])

{
  if (argc != 2)
	return -1;

  bsd_ShowTcp((void*)atolx(argv[1]), NULL, NULL);

  return 0;
}	 


#if defined(LINUX_TEST)
int32 do_ipsend(int32 argc, char8 *argv[])
{
  uint32 sip, dip;

  if (argc == 6) 
    {
      if (bsdIpStoN(argv[2], &sip) == NFX_ERR)
	return -1;

      if (bsdIpStoN(argv[4], &dip) == NFX_ERR)
	return -1; 

      if (strcmp(argv[1], "udp") == 0 || strcmp(argv[1], "UDP") == 0)
	bsd_TstUDP(sip, dip, atoi(argv[3]), atoi(argv[5]));
    }
  else if (argc == 2)
    {
      if (strcmp(argv[1], "udp") == 0 || strcmp(argv[1], "UDP") == 0)
	bsd_TstUDP(0, 0, 0, 0);
    }	
  else
    return -1;

  return 0;
}

int32 do_socksend(int32 argc, char8 *argv[])
{
  void* id;

  if (argc < 2)
    return -1;

  id = (void*)atolx(argv[1]);

  if (argc == 2)
    bsd_TstSock(id, 0, 0);
  else if (argc == 4)
	{
      uint32 ip;
      if (bsdIpStoN(argv[2], &ip) == NFX_ERR)
		return -1;     

      bsd_TstSock(id, ip, atoi(argv[3]));
    }
  else
    return -1;

  return 0;
}


int32 do_tcploop(int32 argc, char8 *argv[])
{
  int type;
  uint32 ip;
  ushort16 port;

  if (argc != 4)
    return -1;

  if (strcmp("server", argv[1])== 0)
    type = BSD_SO_SERVER;
  else if (strcmp("client", argv[1]) == 0)
    type = BSD_SO_CLIENT;
  else
    return -1;

  if (bsdIpStoN(argv[2], &ip) == NFX_ERR)
    return -1;
  
  port = htons(atoi(argv[3]));
  bsd_SetupFastTcp(type, ip, port);

  return 0;

}
int32 do_tcploopsend(int32 argc, char8 *argv[])
{
  int num;
  void *pconn;

  if (argc != 3)
    return -1;

  num = atoi(argv[1]);
  pconn  = (void*)atolx(argv[2]);
  fast_send_1(pconn, num);
  
  return 0;

}
#endif /* LINUX_TEST */

int32 do_sockadd(int32 argc, char8 *argv[])
{
  int type;
  uint32 ip;
  ushort16 port;

  if (argc != 5)
    return -1;

  if (strcmp("server", argv[1])== 0)
    type = 1;
  else if (strcmp("client", argv[1]) == 0)
    type = 2;
  else
    return -1;

  if (bsdIpStoN(argv[3], &ip) == NFX_ERR)
    return -1;
  
  port = htons(atoi(argv[4]));
  bsd_AddSock(argv[2], type, ip, port);

  return 0;
}

int32 do_sockdel(int32 argc, char8 *argv[])
{
  void* id;

  if (argc != 2)
    return -1;

  id = (void*)atolx(argv[1]);
  bsd_DeleteSock(id);

  return 0;
}


/*******************************************************************
 End, IP related UI commands
*******************************************************************/
#endif /* NFP_TXRX */
