/* vi:ts=4
 *-----------------------------------------------------------------
 *
 * Name: bsd-nttcpsvr.c
 *
 * RCS:  $Id: bsd-nttcpsvr.c,v 1.2 2002/03/13 22:21:20 shaun Exp $
 *
 * Copyright (C) 2001, Agile Storage, Inc.
 *
 * Description: nttcpsvr.c
 *
 * Created by:  Su Wang
 *
 * Date Created: 02/22/2002
 *
 *-----------------------------------------------------------------
 */
#if !defined(PROM)
#include    <nfx-incl.h>
#include    <exports/bsd-api.h>
#include	<exports/eee-api.h>
#include    <exports/pkt-api.h>
#include    <exports/luc-api.h>
#include    <exports/tpl-api.h>
#include	"../sm-bsd-api/bsdrelay.h"

typedef struct _nttcp_cb_t
{
#define NTTCP_CLOSE_TRIG	0x1	/* close nttcp close trigger */
	uint32	flags;
	void	*ctrl_pConn;		/* pConn for control socket */
	void	*data_bind;			/* server bind for data socket */
	void	*data_pConn;		/* pConn for data socket */
	int		data_size;			/* rcvd data size thus far */
	uint32	start_time;			/* time when rcv data */
	int		proto;				/* connc. protocol */
} nttcp_cb_t;

typedef struct _opt 
{
	int		proto;
} opt_t;
opt_t	option;

static void
parse_option(pkt_desc_t	*pdesc)
{
	char	*str_ptr;
	
	str_ptr = (char*)pdesc->pHdr;
	
	/* limit string size to be 24 */
	*(str_ptr + 24) = '\0';
	if (strchr(str_ptr, 'u'))
		option.proto = BSD_IPPROTO_UDP;
	else
		option.proto = BSD_IPPROTO_TCP;

	return;
}


/*-----------------------------------------------------------------
 * Name:        nttcpS_Cleanup()
 *
 * Description: clean up connections. 
 *  
 * Created by : Su Wang
 *
 * Date Created: 02/25/2002
 *-----------------------------------------------------------------
 */
static void
nttcpS_Cleanup(nttcp_cb_t	*nttcp_cb_p)
{
	/* terminate control connection or data connection */
	if (nttcp_cb_p->ctrl_pConn)
		tpl_delConnReq(nttcp_cb_p->ctrl_pConn);
	
	if (nttcp_cb_p->data_pConn)
		tpl_delConnReq(nttcp_cb_p->data_pConn);
	
	/* terminate data server */
	if (nttcp_cb_p->data_bind) {
		tpl_unbindReq(nttcp_cb_p->data_bind);
		nttcp_cb_p->data_bind = 0;
	}
	
	/* ther is no more connection, we can remove control block */
	if (!(nttcp_cb_p->ctrl_pConn) && !(nttcp_cb_p->data_pConn))
		eee_ramDealloc(nttcp_cb_p);
	else
		/* must be tcp connections, clear close trigger 
		 * such that cleanup can be called again to 
		 * dealloc ctrl block
		 */
		nttcp_cb_p->flags &=  ~NTTCP_CLOSE_TRIG;
	

	return;
}

/*-----------------------------------------------------------------
 * Name:        nttcpS_Calculate()
 *
 * Description: calculate time used up. 
 *  
 * Created by : Su Wang
 *
 * Date Created: 02/22/2002
 *-----------------------------------------------------------------
 */
static void
nttcpS_Calculate(nttcp_cb_t *nttcp_cb_p)
{
	float		rt;
	char		*str_ptr;
	pkt_desc_t	*pdesc;

	
	
	rt = ((float)(eee_timerGetCurrTic() - nttcp_cb_p->start_time))/100;
	

	pdesc = TPL_ALLOC_PACKET(nttcp_cb_p->ctrl_pConn, 0);
	str_ptr = pdesc->pHdr;

	/* msg format:
	 * rcvbyte real-time cpu-time Mbits/S etc 
	 */
	sprintf(str_ptr, "1  %9d%8.2f%8.2f%12.4f\n", nttcp_cb_p->data_size,
			rt, 0, ((float)nttcp_cb_p->data_size)/rt);
	
	pdesc->hdrSize = strlen(str_ptr);

	tpl_xmtPkt(nttcp_cb_p->ctrl_pConn, pdesc);
}

/*-----------------------------------------------------------------
 * Name:        nttcpS_RcvDataPkt
 *
 * Description: data socket rec pkt. 
 *  
 * Created by : Su Wang
 *
 * Date Created: 02/22/2002
 *-----------------------------------------------------------------
 */
static int32
nttcpS_RcvDataPkt(void			*pAppHdl, 
				  pkt_desc_t	*pdesc)
{
	nttcp_cb_t		*nttcp_cb_p;

	/* increment rcved data size */
	nttcp_cb_p = (nttcp_cb_t*)pAppHdl;
	nttcp_cb_p->data_size += pdesc->hdrSize;

	/* for udp, check for end of connection */
	if ((nttcp_cb_p->proto == BSD_IPPROTO_UDP) &&
		(pdesc->hdrSize < 4)) {
		nttcpS_Calculate(nttcp_cb_p);
		nttcp_cb_p->flags |= NTTCP_CLOSE_TRIG;
		nttcpS_Cleanup(nttcp_cb_p);
	}
	
	return TPL_FREE_PKT;
}

/*-----------------------------------------------------------------
 * Name:        nttcpS_XmtCmpInd
 *
 * Description: transmit complete
 *  
 * Created by : Su Wang
 *
 * Date Created: 02/22/2002
 *-----------------------------------------------------------------
 */
static int32
nttcpS_XmtCmpInd(void		*pAppHdl, 
				 pkt_desc_t	*pdesc)
{
	return TPL_FREE_PKT;
}

/*-----------------------------------------------------------------
 * Name:        nttcpS_AddDataConn
 *
 * Description: a new data socket has established. 
 *  
 * Created by : Su Wang
 *
 * Date Created: 02/22/2002
 *-----------------------------------------------------------------
 */
static int32
nttcpS_AddDataConn( tpl_bindCb_t    *pBind,
					void            *pConnHdl, 
					void            **pAppHdl,
					uint32          raddr,
					ushort16        rport )
{
	nttcp_cb_t		*nttcp_cb_p;
	
	nttcp_cb_p = (nttcp_cb_t*)pBind->usrInfo;

	/* hook data pConn with nttcp_cb */
	nttcp_cb_p->data_pConn = pConnHdl;
	
	*pAppHdl = nttcp_cb_p;

	return NFX_OK;
}

/*-----------------------------------------------------------------
 * Name:        nttcpS_DelDataConn
 *
 * Description: delete data connection indicated by lower layer. 
 *  
 * Created by : Su Wang
 *
 * Date Created: 02/22/2002
 *-----------------------------------------------------------------
 */
static int32
nttcpS_DelDataConn( void            *pConnHdl, 
					void            *pAppHdl)
{
	nttcp_cb_t		*nttcp_cb_p;
	
	nttcp_cb_p = (nttcp_cb_t*)pAppHdl;

	/* calculate processing time */
	nttcpS_Calculate(nttcp_cb_p);
	
	
	/* clean up all associated conns. */
	nttcp_cb_p->data_pConn = 0;
	
	/* if cleanup has not been called, call it */
	if (!(nttcp_cb_p->flags & NTTCP_CLOSE_TRIG)) {
		nttcp_cb_p->flags |= NTTCP_CLOSE_TRIG;
		nttcpS_Cleanup(nttcp_cb_p);
	}
	
	return NFX_OK;
}

/*-----------------------------------------------------------------
 * Name:        nttcpS_RcvCtrlPkt
 *
 * Description: control socket rec pkt. 
 *  
 * Created by : Su Wang
 *
 * Date Created: 02/22/2002
 *-----------------------------------------------------------------
 */
static int32
nttcpS_RcvCtrlPkt(void		*pAppHdl, 
				 pkt_desc_t	*pdesc)

{
	ushort16		src_port;
	tpl_bindCb_t	bind_req, *bind_rspp;
	nttcp_cb_t		*nttcp_cb_p;
	char			*str_ptr;
	int				rc;
	

	/* get nttcp cb */
	nttcp_cb_p = (nttcp_cb_t*)pAppHdl;
		
	/* parsing options */
	parse_option(pdesc);
	
	/* get a local src port for data socket */
	src_port =  bsd_GetFreePort(option.proto);

	/* create data socket */
	bind_req.laddr = 0;
	bind_req.lport = src_port;
	bind_req.rport = 0;
	bind_req.raddr = 0;

	/* remember nttcp ctrl block, used to free various
	 * pConns.
	 */
	bind_req.usrInfo = (void*)nttcp_cb_p;

	bind_req.tplProt = option.proto;
	bind_req.ul_addConnInd = nttcpS_AddDataConn;
	bind_req.ul_delConnInd = nttcpS_DelDataConn;
	bind_req.ul_rcvPktInd  = nttcpS_RcvDataPkt;
    bind_req.ul_xmtCmpInd  = nttcpS_XmtCmpInd;	

	/* open data server socket */
	rc = tpl_bindReq(&bind_req, &bind_rspp, 0);
		
	if (rc != NFX_OK) {
		printf("nttcp: data server create failed\n");
		nttcp_cb_p->flags |= NTTCP_CLOSE_TRIG;
		nttcpS_Cleanup(nttcp_cb_p);
		return TPL_FREE_PKT;
	}

	/* save pBind data server socket */
	nttcp_cb_p->data_bind = bind_rspp;

	/* 
	 * send a response to client about the data port 
	 */
	
	/* alloc pkt from connection,reuse pdesc */
	pdesc = TPL_ALLOC_PACKET(nttcp_cb_p->ctrl_pConn, 0);

	/* cp rely back to client */
	str_ptr = (char*)pdesc->pHdr;
	sprintf(str_ptr, "dataport: %d\n", ntohs(src_port));
	
	pdesc->hdrSize = strlen(str_ptr);

	/* send it off */
	tpl_xmtPkt(nttcp_cb_p->ctrl_pConn, pdesc);

	/* free recved pkt */
	return TPL_FREE_PKT;
}


/*-----------------------------------------------------------------
 * Name:        nttcpS_AddCtrlConn
 *
 * Description: a new control socket has established. 
 *  
 * Created by : Su Wang
 *
 * Date Created: 02/22/2002
 *-----------------------------------------------------------------
 */
static int32
nttcpS_AddCtrlConn( tpl_bindCb_t    *pBind,
					void            *pConnHdl, 
					void            **pAppHdl,
					uint32          raddr,
					ushort16        rport )
{
	nttcp_cb_t		*nttcp_cb_p;
	
	nttcp_cb_p = (nttcp_cb_t*)eee_ramAlloc(sizeof(nttcp_cb_t));
	
	bzero(nttcp_cb_p, sizeof(nttcp_cb_t));
	
	/* remember ctrl pConn handle, so it can be deleted
	 * later
	 */
	nttcp_cb_p->ctrl_pConn = pConnHdl;

	*pAppHdl = (void*)nttcp_cb_p;
	
	return NFX_OK;
}

/*-----------------------------------------------------------------
 * Name:        nttcpS_DelCtrlConn
 *
 * Description: a new control socket has established.
 *  
 * Created by : Su Wang
 *
 * Date Created: 02/22/2002
 *-----------------------------------------------------------------
 */
static int32
nttcpS_DelCtrlConn( void            *pConnHdl, 
				 void            *pAppHdl)
{
	
	nttcp_cb_t	*nttcp_cb_p = (nttcp_cb_t*)pAppHdl;
	
	
	nttcp_cb_p->ctrl_pConn = 0;

	/* clean up connections */
	if (!(nttcp_cb_p->flags & NTTCP_CLOSE_TRIG)) {
		nttcp_cb_p->flags |= NTTCP_CLOSE_TRIG;
		nttcpS_Cleanup(nttcp_cb_p);
	}
	
	return NFX_OK;
}


/*-----------------------------------------------------------------
 * Name:        bsd_InitNttcpServer
 *
 * Description: Initiate nttcp server.
 *  
 * Created by : Su Wang
 *
 * Date Created: 02/22/2002
 *-----------------------------------------------------------------
 */
void
bsd_InitNttcpServer()
{
	tpl_bindCb_t	bind_req, *bind_rspp;
	int				rc;
	

	bind_req.laddr				= 0;
	bind_req.lport			= htons(BSD_WELLKNOWN_PORT_NTTCP);
	bind_req.raddr			= 0;
	bind_req.rport			= 0;
	bind_req.tplProt		= TPL_PROT_TCP;
	bind_req.ul_addConnInd	= nttcpS_AddCtrlConn; 
	bind_req.ul_delConnInd	= nttcpS_DelCtrlConn;
	bind_req.ul_rcvPktInd	= nttcpS_RcvCtrlPkt;
    bind_req.ul_xmtCmpInd	= nttcpS_XmtCmpInd;

	rc = tpl_bindReq(&bind_req, &bind_rspp, 0);
	
	if (rc != NFX_OK)
		printf("NttcpServier Init failed\n");

	return;
}
#endif /* PROM */
