/* vi:ts=4 
 *-----------------------------------------------------------------
 *
 * Name: bsdip-appl.c
 *
 * RCS:  $Id: bsdip-appl.c,v 1.5 2002/10/22 01:30:30 jong Exp $
 *
 * Copyright (C) 2001, Agile Storage, Inc.
 *
 * Description: Some basic TCP/IP application program.
 *
 * Created by:	Su Wang
 *
 * Date Created: 08/15/01
 *
 *-----------------------------------------------------------------
 */
#include <nfx-incl.h>
#include "../sm-bsd-sys/flags.h"
#include "../sm-bsd-sys/cdefs.h"
#include "../sm-bsd-sys/types.h"
#include "../sm-bsd-sys/errno.h"
#include "../sm-bsd-sys/macros.h"
#include "../sm-bsd-sys/mbuf.h"
#include "../sm-bsd-sys/socket.h"

#include "../sm-bsd-net/ethernet.h"
#include "../sm-bsd-net/route.h"
#include "../sm-bsd-netinet/in.h"
#include "../sm-bsd-netinet/ip.h"
#include "../sm-bsd-netinet/ip_icmp.h"
#include "../sm-bsd-netinet/icmp_var.h"

#include "exports/bsd-api.h"
#include "exports/bsdsock-api.h"
#include "exports/timer-api.h"
#include "exports/queue-api.h"

static ushort16  ping_sess_id = 0;

extern void* bsd_OpenRipClient(int, ipa_t, ipa_t, int, void*,
						connevent_calbck_t, rcvdata_calbck_t); 
/*
 *------------------------------------------------------------------
 * 
 * Name: send_ping
 *
 * Descriptions: Ping program 
 *
 *
 * Created By : Su Wang
 * 
 * Date Created: 8/15/01
 *---------------------------------------------------------------------
 */

static int32
send_ping(	 
		  uint32  ping_timer,	/* timer for this ping session */
		  void*  so,			/* socket to send */
		  void*  seq_rep,		/* combine seq and repeat, */
								/* because only 4 params allowd */
		  void*  ptr,			/* buf ptr */
		  void*  size_i			/* buf size */
		  )
{
  ushort16 seq_num = (address_t)seq_rep & 0xffff;
  ushort16 repeat = ((address_t)seq_rep >> 16) & 0xffff;
  struct icmp* icmp_ptr = (struct icmp*)ptr;
  uint32 size = ((address_t)size_i)&0xffffffff;

  if (seq_num == 0)				/* this is the ind. for close socket */
    {
clean_up:
      free(ptr, MT_DATA);
      if (eee_timerStop(ping_timer) == TIMER_ERR_INVALID_HANDLE)
		printf("Ping send: ping_timer %d stop failed\n", ping_timer);
      
      if (eee_timerDestroy(ping_timer) != NFX_OK)
		printf("Ping send: ping timer %d destroy failed\n", ping_timer);
      
      if (bsd_CloseSock(so) != NFX_OK)
		printf("Ping send: close socket %lx failed\n", (address_t)so);
      
	  return NFX_OK;      
    }
  
  icmp_ptr->icmp_seq = seq_num;
  /* get system time */
  *(uint32*)(((char*)icmp_ptr)+ICMP_MINLEN) = eee_timerGetCurrTic();
  
  icmp_ptr->icmp_cksum = 0;
  icmp_ptr->icmp_cksum = bsd_ChkSum((ushort16*)icmp_ptr, size);
  
  if (bsd_ChkSum((ushort16*)icmp_ptr, size))
	printf("bad chksum func\n");

  // printf("sent pkt at %d, timer %d\n",eee_timerGetCurrTic(), ping_timer);
  //bsd_ShowPkt(icmp_ptr, size);
  if (bsd_SendSock(so, 0, 0, icmp_ptr, BSD_BUF_FLAT, (uint32)size,  0) 
	  != NFX_OK)
	goto clean_up;
  
  if (seq_num >= repeat)		/* we are done all sends, but we cannot */
								/* cleanup yet. as we must wait for the */
								/* last reply */
    {
      seq_num = 0;
    }
  else							/* more to send, schedule the next ping */
    {
      seq_num++;
    }
     
  seq_rep = (void*)(((address_t)(seq_num | repeat << 16)) & 0xffffffff);
  if (eee_timerStart(ping_timer, send_ping,  
		     so, seq_rep, ptr, size_i) != NFX_OK)
    {
      printf("Ping send: ping timer %d cannot start\n", ping_timer);
      goto clean_up;
    }
  return NFX_OK;
}


/*
 *------------------------------------------------------------------------------
 * 
 * Name: rcv_ping
 *
 * Descriptions: Ping program 
 *
 *
 * Created By : Su Wang
 * 
 * Date Created: 8/15/01
 *------------------------------------------------------------------------------
 */
static void
rcv_ping(void* so1, /* rcv socket */
	 void* ptr, /* data ptr */
	 ipa_t sa, /* sender addr */
	 ushort16 sp, /* sender port if any */
	 uint32 size,
	 void* sess_id
	 )
{
  struct ip* ip_ptr;
  struct icmp* icmp_ptr;
  ushort16 hlen;
  uint32  rtt;

  ip_ptr = (struct ip*)ptr;
  hlen = (ip_ptr->ip_hl)<<2;

  icmp_ptr = (struct icmp*)(((char*)ptr) + hlen);

  // printf("print received pkt\n");
  // bsd_ShowPkt(icmp_ptr, (size-hlen));
  /* check if the packet is for us */
  if (icmp_ptr->icmp_type != ICMP_ECHOREPLY)
    return;

  if (icmp_ptr->icmp_id != (((address_t)sess_id) & 0xffff))
    return;

  rtt = eee_timerGetCurrTic() - *(uint32*)(((char*)icmp_ptr)+ICMP_MINLEN);
  
  if (bsd_ChkSum((ushort16*)icmp_ptr, (size - hlen)))
      printf("chksum error \n");

  uiPrintf("%d byte from %s: seq=%u, ttl=%d, rtt=%ums\n",
	   size, bsdPrintIpa(sa), icmp_ptr->icmp_seq, ip_ptr->ip_ttl, rtt*10);
  uiPrintf("\n");

  return;
}
	 
  
/*
 *------------------------------------------------------------------------------
 * 
 * Name: bsd_Ping
 *
 * Descriptions: Ping program 
 *
 *
 * Created By : Su Wang
 * 
 * Date Created: 8/15/01
 *------------------------------------------------------------------------------
 */
int32
bsd_Ping(ipa_t da, ushort16 size, ushort16 repeat)
{
  struct socket			*so;
  struct icmp			*icmp_ptr;
  int32					ping_timer;
  int					len;
  struct sockaddr_in	soaddr;
  struct rtentry		*rt;
  

  if (repeat == 0)
    {
      printf("Ping: 0 ping specified\n");
      return NFX_OK;
    }
  
  /*  if (size > ETHERMTU)
    {
      printf("Ping: size too large\n");
      return NFX_OK;
    }
  */

  /* Make sure destination is reachable  */
  bzero(&soaddr, sizeof(struct sockaddr_in));
  soaddr.sin_len = 16;
  soaddr.sin_family = AF_INET;
  soaddr.sin_addr.s_addr = da;
  if ((rt = rtalloc1((struct sockaddr*)&soaddr, 0, 0)) == 0)
	{
	  printf("Ping: no route to dest %s\n", bsdPrintIpa(da));
	  return NFX_OK;
	}
  else
	rt->rt_refcnt--;


  if ((so=(struct socket*)bsd_OpenRipClient
	   (IPPROTO_ICMP, 0, da, BSD_BUF_FLAT, 
		(void*)((address_t)ping_sess_id), 0, rcv_ping))== NULL)
    {
      printf("ping: socket to dest %s not found\n", bsdPrintIpa(da));
      return NFX_OK;
    }


  /* find ICMP packet length */
  len = max((size-sizeof(struct ip)), ICMP_MINLEN+4); 

  if ((icmp_ptr = (struct icmp*)malloc(len, MT_DATA, M_DONTWAIT))
      == NULL)
    {
      printf("Ping: buf alloc %d bytes failed\n", len);
      return NFX_OK;
    }

  icmp_ptr->icmp_type = ICMP_ECHO;
  icmp_ptr->icmp_code = 0;
  icmp_ptr->icmp_id = ping_sess_id;

  /* fill in rest with packet with some recognizable data pattern*/
  memset(((char*)icmp_ptr)+ICMP_MINLEN, 0x30, (len-ICMP_MINLEN));

  /* create a timer */
  if ((ping_timer = eee_timerCreate(nfx_time_tics, EEE_TIMER_ONCE))
      == TIMER_ERR_NO_TIMER_AVAILABLE)
    {
      printf("Ping: cannot allocate timer\n");
      free(icmp_ptr, MT_DATA);
      return NFX_OK;
    }

  
  send_ping(ping_timer, so, (void*)((address_t)(1| repeat << 16)), icmp_ptr, 
			(void*)((address_t)len));
  ping_sess_id++;
  return NFX_OK;				/* we do not deallocate icmp here, */
								/* instead it is deallocated in send_ping */
}
