/* vi:ts=4   
 *-----------------------------------------------------------------
 *
 * Name: bsdsock-api.h
 *
 * RCS:  $Id: bsdsock-api.h,v 1.12 2002/09/23 21:49:18 swang Exp $
 *
 * Copyright (C) 2001, Agile Storage, Inc.
 *
 * Description: provide socket interface for application access to TCP/IP stack
 *
 * Created by:	Su Wang
 *
 * Date Created: 07/16/01
 *
 *-----------------------------------------------------------------
 */

#ifndef _BSDSOCK_API_H
#define   _BSDSOCK_API_H

#include "exports/bsd-api.h"
#include "exports/queue-api.h"

/* indicate to application that a connection state has changed */
#define SO_CONN_UP_IND					1
#define SO_CONN_DOWN_IND				2
#define SO_CONN_RTCHG_IND				3
// #define SO_CONN_DOWNREQ_IND				4

/* indicate an action issued on application has completed */
#define SO_CONN_CLOSE_REQC  11



/* indicate an action issued on application has completed */
// #define SO_CONNECTION_CLOSE_CONFIRM  11

typedef	int	   (*connevent_calbck_t)
				(void*,			/* socket ptr */
				 void*,			/* pcb ptr */
				 int,			/* socket event type */
				 void**			/* handle assigned by application */
				);


typedef void	(*rcvdata_calbck_t)
				(void*,			/* socket ptr */
				 void*,			/* ptr to data */
				 uint32,		/* src addr, network order */
				 ushort16,		/* src port, network order */
				 uint32,		/* buf len */
				 void*			/* handle */
				 );

/*specify parameters of a socket to be created  */
/*all IPs and port numbers must be in network order */
typedef struct bsd_sarg 
{
  ipa_t			laddr;			/* local IP, mandatory for Raw IP server. */
								/* optional for other server and all clients */
  ipa_t			faddr;			/* remote IP, 0 for all servers, mandatory */
								/* for all clients */
  ushort16		lport;			/* local port, not applicable for RAW IP */
								/* must for TCP/UDP Server, optional for clts*/
  ushort16		fport;			/* remote port, not applicable for RAW IP */
								/* 0 all TCP/UDP server, must for their clts */
  void*			handle;			/* optional handle, application passed */
  int			bufType;		/* buf type application expect to rcv */
#define BSD_SO_NO_SPAWN		0x1	/* UDP/RIP svr dont want to spawn off conn.*/
#define BSD_SO_FASTPATH		0x2 /* socket for fast fast only */	
  uint32		flags;			/* flags controls behavior of the socket */
  connevent_calbck_t	cb;		/* conn.callbck when conn. state changes */
  rcvdata_calbck_t		db;		/* data callbck when data rcved */
} bsd_sarg_t, *bsd_sarg_ptr_t;


/******************************************************************
 * Name : bsd_CloseSock 
 *
 * Description: completely remove connection including socket structure.
 *
 * Created by: Su Wang
 * 
 * Date created: 07/16/01
 ******************************************************************/
int32							/* NFX_ERR, NFX_OK */		
bsd_CloseSock(void* so);

/******************************************************************
 * Name : bsd_AbortSock 
 *
 * Description: completely remove connection including socket structure,
 *				immidiately. 
 *				
 *
 * Created by: Su Wang
 * 
 * Date created: 07/16/01
 ******************************************************************/
int32							/* NFX_ERR, NFX_OK */		
bsd_AbortSock(void* so);

/***********************************************************************
 * Name : bsd_shutdown 
 *
 * Description: terminate read and/write part of connection.
 *
 * Created by: Su Wang
 *
 * Date created: 07/16/01 
 ************************************************************************/
int32							/* NFX_OK, NFX_ERR */
bsd_ShutSock(
			 void* so,
			 int   how			/* terminate incoming and/or outgoing */
								/* 0 read, 1 write, 2 both */			 
			 );

/******************************************************************
 * Name : bsd_OpenSock 
 *
 * Description: open a socket structure.
 *
 * Created by: Su Wang
 * 
 * Date created: 07/16/01
 ******************************************************************/
void*							/* NULL failed, ptr to socket */
bsd_OpenSock(
			 int			proto, /* see bsd-api.h */
#define	BSD_SO_CLIENT	1
#define BSD_SO_SERVER	2
			 int			type,  /* server or client */
			 bsd_sarg_ptr_t arg    /* socket creation arguments */
			 );

/*******************************************************************
 * 
 * Name : bsd_SetSoOption
 *
 * Description : set socket options.
 *
 * Created by : Su Wang
 *
 * Date created: 07/18/01
 ********************************************************************/
void
bsd_SetSoOption(void *so, 
				int flag,		/* same as flag is bsd_sarg */
				int value);		/* 0 unset, 1 set */

/*****************************************************************************
 * 
 * Name : bsd_SendSock
 *
 * Description : api to send traffic on to socket.  Assuming flat buffer only 
 *
 * Created by : Su Wang
 *
 * Date created: 07/18/01
 **************************************************************************/
int32							/* NFX_OK, NFX_ERR */
bsd_SendSock(
			 void*				so1, 
								/* socket to send traffic */
			 ipa_t				faddr, 
								/* remote ip address */
								/* needed if socket is RAW or UDP */
			 ipa_t				fport, 
								/* remote port address,*/
								/* may be needed for some UDP socket */
			 void*				data_ptr, 
								/* pointer to data */
			 int				type,  
								/* flat buffer, or pkt descr */
			 uint32				size,
								/* buf size */
			 int32				flags   
								/* MSG_OOB, MSG_EOF */
			 );

/**************************************************************************** 
 * Name : bsd_RcvSock
 *
 * Description: This function send data to upper 
 *              layer.
 *
 * Created by : Su Wang
 * 
 * Data created : 07/20/01 
 ****************************************************************************/
int32							/* to be determined     */
bsd_RcvSock(			
				  struct		socket* so, 
								/* receiving socket */ 
				  int			flags, 
								/* control flag indicating type of */
								/* messages in receiving queue */   
				  uint32		offset  
								/* IP-UDP-TCP header offset */ 
				  );


/**************************************************************************** 
 * Name : bsd_ShowSock
 *
 * Description: This function displays socket configuration 
 *
 *
 * Created by : Su Wang
 * 
 * Data created : 07/20/01 
 ****************************************************************************/
void bsd_ShowSock(
				   int			what,
				   void*		id, 
				   void*		param1,
				   void*		param2
		  );

/**************************************************************************** 
 * Name : bsd_MakeConnQ
 *
 * Description: build TCP/UDP connection update queue. 
 *
 *
 * Created by : Su Wang
 * 
 * Data created : 07/20/01 
 ****************************************************************************/
void
bsd_MakeConnQ(int	all);		/* want all connections */

void	bsd_ResetTcp(void);
int32	bsd_SendArp(void*);
int		bsd_DeleteSock(void*);
void	bsd_ShowTcp(void*, void*, void*);
int32   bsd_AddSock(char*, int, uint32, ushort16);
#endif /* _BSDSOCK_API_H */
