#include "stdlib.h"
#include "string.h"

#undef paddr_t
#undef vaddr_t
#include "exports/opt-api.h"
#include "exports/anpssc-api.h"
#include "exports/event-api.h"
#include "../../sm-event/event-as.h"

#include "aghelper.h"


int getAutoSupportConfig(autoSupport_t *ast)
{
int rc;
as_config_t asCfg;
event_configMsg_t eventCfgReq;
struct timeval tout = {10, 0};

	memset(&eventCfgReq, 0, sizeof(event_configMsg_t));
	eventCfgReq.msgType = NFX_EVENT_AS_CONFIG_VIEW_REQ;

	/* Send the request to eventd daemon on SSC */
	rc = sendAgileMsg(-1, &eventCfgReq, sizeof(event_configMsg_t),
						"eventd", NFX_SSC_UNIX, SSC_SLOT, MGMTPLANE,
						&tout, &asCfg, sizeof(as_config_t));

	if (rc != sizeof(as_config_t))
		return 0;

	ast->enabled = asCfg.enable;
	strcpy(ast->to, asCfg.to);
	strcpy(ast->noteto, asCfg.noteto);
	return 1;
}


int setAutoSupportConfig(autoSupport_t *ast)
{
int rc;
event_configMsg_t eventCfgReq;
struct timeval tout = {10, 0};

	memset(&eventCfgReq, 0, sizeof(event_configMsg_t));
	eventCfgReq.msgType = NFX_EVENT_AS_CONFIG_SET_REQ;

	strcpy(eventCfgReq.entry.option, OPTIONS_AUTOSUPPORT_ENABLE);
	if (ast->enabled)
		strcpy(eventCfgReq.entry.value, "on");
	else
		strcpy(eventCfgReq.entry.value, "off");

	rc = sendAgileMsg(-1, &eventCfgReq, sizeof(event_configMsg_t),
						"eventd", NFX_SSC_UNIX, SSC_SLOT, MGMTPLANE,
						&tout, NULL, 0);
	if (rc != NFX_OK)
		return 0;

	strcpy(eventCfgReq.entry.option, OPTIONS_AUTOSUPPORT_TO);
	strcpy(eventCfgReq.entry.value, ast->to);

	rc = sendAgileMsg(-1, &eventCfgReq, sizeof(event_configMsg_t),
						"eventd", NFX_SSC_UNIX, SSC_SLOT, MGMTPLANE,
						&tout, NULL, 0);
	if (rc != NFX_OK)
		return 0;

	strcpy(eventCfgReq.entry.option, OPTIONS_AUTOSUPPORT_NOTETO);
	strcpy(eventCfgReq.entry.value, ast->noteto);

	rc = sendAgileMsg(-1, &eventCfgReq, sizeof(event_configMsg_t),
						"eventd", NFX_SSC_UNIX, SSC_SLOT, MGMTPLANE,
						&tout, NULL, 0);
	if (rc != NFX_OK)
		return 0;

	return 1;
}


/* SY: ht_ag_ht_ag_get_cfg_val() and ht_ag_ht_ag_mod_cfg_val() are shameless */
/* copied from Greg's cfg_nfxnis.c                   */

char **ht_ag_ht_ag_get_cfg_val(char *key)
{
regmGetReq_t req;
uint64 buf[1024];
regmGetRsp_t *rsp = (regmGetRsp_t *) buf;
int rc;
struct timeval regmTo = {10, 0};
char **list;
int i;

	req.hdr.msgType = OPT_GET_OPTION_REQ;
	req.hdr.userInfo = 0;
	strcpy(req.getReq.option, key);

	rc = sendAgileMsg(-1, &req, sizeof(req),
	                   "regm", NFX_SSC_UNIX, SSC_SLOT, MGMTPLANE, &regmTo,
	                   buf, sizeof(buf));

	if (!rc)
		return NULL;	/* timeout */

	if (!rsp->getRsp.nbEntry)
		return NULL;	/* No matches */

	/* Create and populate the list of key/data pairs */
	i = sizeof(char *) * 2 * (rsp->getRsp.nbEntry + 1);
	list = malloc(i);
	memset(list, 0, i);

	for (i = 0; i < rsp->getRsp.nbEntry; i++) {
		list[2*i] = strdup(rsp->getRsp.entry[i].option);
		list[(2*i)+1] = strdup(rsp->getRsp.entry[i].value);
	}

	return list;
}

int ht_ag_ht_ag_mod_cfg_val(char *option, char *value, int del)
{
regmSetReq_t req;
regmSetRsp_t rsp;
struct timeval regmTo = {10, 0};
int rc;

	if (!option || !option[0])
		return 1;

	req.hdr.msgType = OPT_SET_OPTION_REQ;
	req.hdr.userInfo = 0;
	req.setReq.del = del;
	strcpy(req.setReq.entry.option, option);
	if (!del)
		strcpy(req.setReq.entry.value, value);

	rc = sendAgileMsg(-1, &req, sizeof(req),
	                   "regm", NFX_SSC_UNIX, SSC_SLOT, MGMTPLANE, &regmTo,
	                   &rsp, sizeof(rsp));

	if ((rc != sizeof(rsp)) || (rsp.setRsp.result != NFX_OK))
		return 1;
	else
		return 0;
}

int ht_ag_ht_ag_put_cfg_val(char *option, char *value)
{
	return ht_ag_ht_ag_mod_cfg_val(option, value, 0);
}

int ht_ag_del_cfg_obj(char *option, char *value)
{
	return ht_ag_ht_ag_mod_cfg_val(option, value, 1);
}


void ht_ag_ht_ag_free_cfg_val(char **list)
{
	char **str;

	if (!list)
		return;
	str = list;

	while (*str)
	{
		free(*str);
		str++;
	}
	free(list);
}


