#include <stdio.h>
#include <unistd.h>
#include <sys/types.h>
#include <signal.h>

#include <anpssc-api.h>
#include <nfxnis-api.h>

#include <ipmd-lib.h>

#include "httpdagent.h"
#include "bsdhelper.h"
#include "interface.h"
#include "syscfg.h"

int processRequest(char *, char *);

#define CHASSIS_CMD_PROG "/usr/local/agile/bin/chassis"



int main()
{
int fd, n;
char fromApp[128];
char inbuf[4096];
char outbuf[4096];
struct timeval tout = {60, 0};
int fcpu, fslot, fplane;

	init_kmem("/dev/kmem");

	fd = regAgileApp("httpdagent");
	if (fd < 0) {
		perror ("regAgileApp(): Unable to register application");
		exit(1);
	}

	/* Tell mcp we're ready to roll. */
	kill(getppid(), SIGUSR1);

	while (1) {
		n = recvAgileMsg (fd, inbuf, sizeof(inbuf), &tout,
							fromApp, &fcpu, &fslot, &fplane);
		if (n < 0) { 
			perror ("recvAgileMsg() failed. Exiting...");
			exit(1);
		}

		if (!n)
			continue;		/* nothing coming in... */

		/* received something here. Now check what it is */
		n = processRequest(inbuf, outbuf);
		if (n <= 0){
			/* Error or nothing to send back... */
			continue;
		}
		if (sendAgileMsg(-1, outbuf, n,
				fromApp, fcpu, fslot, fplane, NULL, 0, 0) < 0) {
		}
	}
}

int processRequest(char *inbuf, char *outbuf)
{
int rval;
haMessage_t *msg, *outmsg;

	rval = -1;
	msg = (haMessage_t *) inbuf;
	outmsg = (haMessage_t *) outbuf;
	outmsg->clientData	= msg->clientData;
	outmsg->type		= msg->type;

	switch (msg->type) {
	case HA_GETIFCOUNT:
		*((int *)outmsg->data) = InterfaceScanGetCount();
		outmsg->rspstatus = HARSP_OK;
		outmsg->datalen = sizeof(int);
		break;

	case HA_GETIFSTATS:
		{
		haIfStats_t *ifstat = NULL;
		int count, need;

			/* data buffer should contain interface index */
			if (msg->datalen != sizeof(haIfStats_t)) {
				outmsg->rspstatus = HARSP_INVALID_DATALEN;
				outmsg->datalen = 0;
				break;
			}
#if 0
			rval = getInterfaceStats(0, (haIfStats_t *) msg->data);
			count = 1;
			memset(ifstat->stats, 0, sizeof(ipm_ifstat_t));
#endif
			count = 1;
			ifstat = (haIfStats_t *) msg->data;
			rval = ipmd_GetIfStats(ifstat->if_name,
							(ipm_ifstat_t *) ifstat->stats,
							sizeof(ipm_ifstat_t), &count, &need);
			if ((rval == IPM_GET_ERR) || (rval == IPM_GET_NO_BUF)) {
				outmsg->datalen = 0;
				outmsg->rspstatus = HARSP_FAILED;
			}
			else {
				outmsg->datalen = sizeof(haIfStats_t);
				memcpy(outmsg->data, ifstat, sizeof(haIfStats_t));
				outmsg->rspstatus = HARSP_OK;
			}
		}
		break;

	case HA_ADDARP:
	case HA_DELARP:
		{
		haArpEntry_t *parpentry;

			if (msg->datalen != sizeof(haArpEntry_t)) {
				outmsg->rspstatus = HARSP_INVALID_DATALEN;
				outmsg->datalen = 0;
				break;
			}
			parpentry = (haArpEntry_t *) msg->data;

			if (msg->type == HA_ADDARP)
				rval = ipmd_AddArp(parpentry->ip, parpentry->bufmac, NULL, 0);
			else
				rval = ipmd_DelArp(parpentry->ip, NULL, 0);
			if (rval == IPM_GET_OK)
				outmsg->rspstatus = HARSP_OK;
			else
				outmsg->rspstatus = HARSP_FAILED;
			outmsg->datalen = 0;
		}
		break;

	case HA_GETSYSTEMCONFIG:
		{
		haSystemConfigType_t *getcfg;
			if (msg->datalen != sizeof(haSystemConfigType_t)) {
				outmsg->rspstatus = HARSP_INVALID_DATALEN;
				outmsg->datalen = 0;
				break;
			}
			getcfg = (haSystemConfigType_t *) msg->data;
			rval = getSystemConfig(getcfg->from, getcfg->step, getcfg->id,
									(haSystemConfig_t *) outmsg->data);
			if (rval == 1)
			{
				outmsg->datalen = sizeof(haSystemConfig_t);
				outmsg->rspstatus = HARSP_OK;
			}
			else
			{
				outmsg->datalen = 0;
				outmsg->rspstatus = HARSP_FAILED;
			}
		}
		break;

	case HA_SETSYSTEMCONFIG:
		{
		haSystemConfig_t *config;

			if (msg->datalen != sizeof(haSystemConfig_t)) {
				outmsg->rspstatus = HARSP_INVALID_DATALEN;
				outmsg->datalen = 0;
				break;
			}
			config = (haSystemConfig_t *) msg->data;
			rval = setSystemConfig(config->type.from, config->type.step,
											config->type.id, config->configs);
			if (rval == 1)
			{
				outmsg->datalen = sizeof(haSystemConfigType_t);
				outmsg->rspstatus = HARSP_OK;
			}
			else
			{
				outmsg->datalen = 0;
				outmsg->rspstatus = HARSP_FAILED;
			}
		}
		break;

	case HA_SETSYSTIME:
		{
			outmsg->datalen = 0;
			if (SetSystemTime(msg->data) < 0)
				outmsg->rspstatus = HARSP_FAILED;
			else
				outmsg->rspstatus = HARSP_OK;
		}
		break;

	case HA_VERIFYUSERPASS:
		{
			haUserPass_t *p = (haUserPass_t *) msg->data;

			outmsg->datalen = 0;
			if (VerifyUserPass(p->user, p->pass) < 0)
				outmsg->rspstatus = HARSP_FAILED;
			else
				outmsg->rspstatus = HARSP_OK;
		}
		break;

	case HA_UPDATENIS:
		{
			haUpdateNIS_t *p = (haUpdateNIS_t *) msg->data;

			outmsg->datalen = 0;
			if (nfxnis_addservice (nis_name,
					p->ip, p->mask, p->nisSvr, p->domain) == 0)
				outmsg->rspstatus = HARSP_OK;
			else
				outmsg->rspstatus = HARSP_FAILED;
		}
		break;

	case HA_REBOOT_SSC:
		execute_command("/sbin/reboot", 0, NULL, NULL);
		break;

	case HA_SYSTEM_OP:
		switch (*((u_int32_t *) msg->data))
		{
		case HA_SYS_OP_REBOOT:
			/* We need to dismount things before we */
			/* reset the embedded side so let's run */
			/* the shutdown script first.           */
			execute_command("/bin/sh", 1, "/etc/rc.shutdown", NULL);

			/* Reset the embedded boards */
			execute_command(CHASSIS_CMD_PROG, 3, "reboot", "slot", "1", NULL);
			execute_command(CHASSIS_CMD_PROG, 3, "reboot", "slot", "2", NULL);

			/* Now reboot us */
			execute_command("/sbin/reboot", 1, "-c", NULL);
			outmsg->rspstatus = HARSP_OK;
			outmsg->datalen = 0;
			break;

		case HA_SYS_OP_HALT:
			execute_command("/sbin/halt", 0, NULL, NULL);
			outmsg->rspstatus = HARSP_OK;
			outmsg->datalen = 0;
			break;
		}
		break;

	default:
		outmsg->rspstatus = HARSP_INVALID_REQCODE;
		outmsg->datalen = 0;
		break;
	}

	rval = HAMESSAGE_HEADER_SIZE + outmsg->datalen;
	return rval;
}

