/* httpdagent.h */

#ifndef _HTTPDAGENT_H
#define _HTTPDAGENT_H

#define APP_HTTPDAGENT		"httpdagent"

/* Message Type (request code) */
#define HA_GETIFCOUNT				1
#define HA_GETIFSTATS				2
#define HA_ADDARP					3
#define HA_DELARP					4
#define HA_GETSYSTEMCONFIG			5
#define HA_SETSYSTEMCONFIG			6
#define HA_SETSYSTIME				7
#define HA_VERIFYUSERPASS			8
#define HA_UPDATENIS				9
#define HA_REBOOT_SSC				10
#define HA_SYSTEM_OP				11

/* Response Status Code */
#define HARSP_OK					0
#define HARSP_INVALID_REQCODE		1
#define HARSP_INVALID_DATALEN		2
#define HARSP_FAILED				3

/* when a request requires no data, client */
/* can just use the first four fields of  */
/* the following message structure with    */
/* datalen field set to 0.                 */

typedef struct _haMessage {
	int		clientData;		/* client use only. opaque to httpdagent */
	int		type;			/* message type defined above */
	int		rspstatus;		/* response status code */
	int		datalen;		/* data length */
	char	data[0];		/* the actual data */
							/* data structure defined below */
} haMessage_t;

#define HAMESSAGE_HEADER_SIZE (4*sizeof(int))

/* 
 * Data structure is defined for each individual request code.
 */

/* HA_GETIFCOUNT - get the total number of interface
 * No data structure is defined for this request code
 * as response will contain number of interfaces in
 * data buffer and the size of this data is sizeof(int)
 * bytes.
 */

/* HA_GETIFSTATS - get the interface stats of specified index */
typedef struct _haIfStats {
	char if_name[64];
	char stats[256];		/* ipm_ifstat_t structure here */
} haIfStats_t;

/* HA_ADDARP and HA_DELARP - add or delete ARP */
typedef struct _haArpEntry {
	u_int32_t	ip;
	char 		bufmac[20];
} haArpEntry_t;

/* HA_GETSYSTEMCONFIG and HA_SETSYSTEMCONFIG - system config */
typedef struct _haSystemConfigType {
	int			from;		/* 1 - system config file, 0 - temp file */
	int			step;		/* 1 - 4 */
	u_int32_t   id;			/* client IP address */
} haSystemConfigType_t;

typedef struct _haSystemConfig {
	haSystemConfigType_t	type;
	char					configs[1536];	/* config data */
} haSystemConfig_t;

/* HA_SETSYSTIME - set system will pass "struct tm" in data */

/* HA_VERIFYUSERPASS */
typedef struct _haUserPass_t {
	char	user[128];
	char	pass[128];
} haUserPass_t;

/* HA_UPDATENIS */
typedef struct _haUpdateNIS_t {
	u_int32_t	ip;
	u_int32_t	mask;
	u_int32_t	nisSvr;
	char		domain[128];
} haUpdateNIS_t;

/* HA_SYSTEM_OP - will pass u_int32_t operation code defined below */
#define HA_SYS_OP_REBOOT	1
#define HA_SYS_OP_HALT		2


#endif
