/* agileifrpt.c */

#include <sys/types.h>
#include <stdlib.h>
#include <unistd.h>
#include <stdio.h>
#include <string.h>
#include <time.h>

#include <sys/socket.h>
#include <netinet/in.h>
#include <net/if.h>
#include <netinet/in_var.h>
#include <net/if_types.h>
#include <net/if_dl.h>
#include <fcntl.h>
#include <kvm.h>

#include <nfx-defs.h>
#include <nfx-types.h>
#include <cm-api.h>
#include <ipmd-lib.h>
#include "httpdagent.h"

#define KERNEL_LOC		"/dev/ksyms"
#define IFNET_SYMBOL	"ifnet"
#define IFADDR_SYMBOL	"in_ifaddr"

long auto_nlist_value(const char *string);
int auto_nlist(const char *string, char *var, int size);
int init_nlist (struct nlist *);
int klookup(unsigned long off, char   *target, int siz);
void init_kmem(const char *file);
int KNLookup(struct nlist nl[], int nl_which, char *buf, int s);

void InterfaceScanInit (void);
int InterfaceScanNext(int *, char *, struct ifnet *, struct in_ifaddr *);
int InterfaceGetEtherByKernelIndex(int Index, u_char *EtherAddr);
int InterfaceScanByKernelIndex(int, char *, struct ifnet *, struct in_ifaddr *);
int InterfaceScanGetCount(void);

int GetGbeSlots(int *slot, int *size, int *need);
int GetSlotNumber(int interface);
int GetSlotType(int slot);
int GetSscIfStats(int Index, haIfStats_t *ifstat);
int GetFpnimIfStats(int Index, haIfStats_t *ifstat);
int getInterfaceStats(int Index, haIfStats_t *ifstat);

/*
 * SUPPORTING FUNCTIONS (for interfaces)
 */


struct autonlist {
   char *symbol;
   struct nlist nl[2];
   struct autonlist *left, *right;
};
struct autonlist *nlists = 0;

long auto_nlist_value(const char *string)
{
struct autonlist **ptr, *it=0;
int cmp;

	if (string == 0)
		return 0;

	ptr = &nlists;
	while(*ptr != 0 && it == 0) {
		cmp = strcmp((*ptr)->symbol, string);
		if (cmp == 0)
			it = *ptr;
		else if (cmp < 0) {
			ptr = &((*ptr)->left);
		}
		else {
			ptr = &((*ptr)->right);
		}
	}

	if (*ptr == 0) {
		*ptr = (struct autonlist *) malloc(sizeof (struct autonlist));
		it = *ptr;
		it->left = 0;
		it->right = 0;
		it->symbol = (char *) malloc(strlen(string)+1);
		strcpy(it->symbol,string);
		/* allocate an extra byte for inclusion of a preceding '_' later */
		it->nl[0].n_name = (char *) malloc(strlen(string)+2);
		sprintf(it->nl[0].n_name,"_%s",string);
		it->nl[1].n_name = 0;
		if (init_nlist(it->nl) < 0)
			return -1;
		if (it->nl[0].n_type == 0) {
			strcpy(it->nl[0].n_name,string);
			if (init_nlist(it->nl) < 0)
				return -1;
		}
		if (it->nl[0].n_type == 0)
			return -1;
		else
			return (it->nl[0].n_value);
	}
	else
		return (it->nl[0].n_value);
}

int auto_nlist(const char *string, char *var, int size)
{
long result;
int ret;

	result = auto_nlist_value(string);
// fprintf(stderr, "auto_nlist_value(%s) result: %x\n", string, result);
	if (result != -1) {
		if (var != NULL) {
			ret = klookup(result, var, size);
			return ret;
		}
		else
			return 1;
	}
	else {
// fprintf(stderr, "auto_nlist_value failed\n");
	}

	return 0;
}
 
int init_nlist(struct nlist nl[])
{
int ret;
kvm_t *kernel;
char kvm_errbuf[4096];

	kernel = kvm_openfiles(KERNEL_LOC, NULL, NULL, O_RDONLY, kvm_errbuf);
	if (kernel == NULL) {
		return -1;
	}

	if ((ret = kvm_nlist(kernel, nl)) == -1) {
		return -1;
	}

	kvm_close(kernel);

#if 0
	for(ret = 0; nl[ret].n_name != NULL; ret++) {
		if (nl[ret].n_type == 0) {
		} else {
		}
	}
#endif

	return 0;
}

int KNLookup(struct nlist nl[], int nl_which, char *buf, int s)
{ 
struct nlist *nlp = &nl[nl_which];

	if (nlp->n_value == 0) {
		nlp->n_value = -1;	/* only one error message ... */
		return 0;
	}
	if (nlp->n_value == -1)
		return 0;

	return klookup(nlp->n_value, buf, s);
}

static kvm_t *kd;

void init_kmem(const char *file)
{
char err[128];

    kd = kvm_openfiles(NULL, NULL, NULL, O_RDONLY, err);
	return;
}

int klookup(unsigned long off, char   *target, int siz)
{
int result;
	if (kd == NULL) return 0;
	result = kvm_read(kd, off, target, siz);
	if (result != siz) {
		return -1;
	}
	return 1;
}

static int saveIndex = 0;
static char saveName[16];
static struct in_ifaddr savein_ifaddr;
static struct ifnet *ifnetaddr, saveifnet, *saveifnetaddr;

void InterfaceScanInit (void)
{
	auto_nlist(IFNET_SYMBOL, (char *)&ifnetaddr, sizeof(ifnetaddr));
	saveIndex=0;
}

#define ia_next ia_list.tqe_next
#define if_next if_list.tqe_next

int InterfaceScanNext(int *Index, char *Name,
			struct ifnet *Retifnet, struct in_ifaddr *Retin_ifaddr)
{
struct ifnet ifnet;
struct in_ifaddr *ia, in_ifaddr;
short has_ipaddr=0;

	while (ifnetaddr) {
		/* Get the "ifnet" structure and extract the device name */
		klookup((unsigned long)ifnetaddr, (char *)&ifnet, sizeof(ifnet));
		strncpy(saveName, ifnet.if_xname, sizeof(saveName));
		saveName[sizeof(saveName)-1] = '\0';

		if (1 || strcmp(saveName,"lo0") != 0) {  /* XXX */
			/* Try to find an address for this interface */

			auto_nlist(IFADDR_SYMBOL, (char *)&ia, sizeof(ia));
			while (ia) {
				klookup((unsigned long)ia ,  (char *)&in_ifaddr, sizeof(in_ifaddr));
				if (in_ifaddr.ia_ifp == ifnetaddr) {
					has_ipaddr = 1; /* this IF has IP-address */
					break;
				}
				ia = in_ifaddr.ia_next;
			}

			if (Index)
				*Index = ++saveIndex;
			if (Retifnet)
				*Retifnet = ifnet;
			if (Retin_ifaddr && has_ipaddr) /* assign the in_ifaddr only
											   if the IF has IP-address */
				*Retin_ifaddr = in_ifaddr;
			if (Name)
				strcpy(Name, saveName);

			saveifnet = ifnet;
			saveifnetaddr = ifnetaddr;
			savein_ifaddr = in_ifaddr;
			ifnetaddr = ifnet.if_next;

			return(1);	/* DONE */
		}
		ifnetaddr = ifnet.if_next;
	}
	return(-1);		/* EOF */
}


int InterfaceGetEtherByKernelIndex(int Index, u_char *EtherAddr)
{
	int i;
	struct arpcom {
	  char ac_enaddr[6];
	} arpcom;
        struct sockaddr_dl sadl;
        struct ifaddr ifaddr;
        u_long ifaddraddr;

        memset(&arpcom.ac_enaddr, 0, sizeof(arpcom.ac_enaddr));
        memset(EtherAddr, 0, sizeof(arpcom.ac_enaddr));

	if (saveIndex != Index) {	/* Optimization! */

	    InterfaceScanInit();

		while (InterfaceScanNext(&i, NULL, NULL, NULL) != 0) {
			if (i == Index)
				break;
	    }
	    if (i != Index) return(-1);     /* Error, doesn't exist */
	}

	/*
	 *  the arpcom structure is an extended ifnet structure which
	 *  contains the ethernet address.
	 */
#define if_addrlist if_addrlist.tqh_first
#define ifa_next    ifa_list.tqe_next

        ifaddraddr = (unsigned long)saveifnet.if_addrlist;
        while (ifaddraddr) {
          klookup(ifaddraddr, (char *)&ifaddr, sizeof ifaddr);
          klookup((unsigned long)ifaddr.ifa_addr, (char *)&sadl, sizeof sadl);
          if (sadl.sdl_family == AF_LINK &&
              (saveifnet.if_type == IFT_ETHER ||
               saveifnet.if_type == IFT_ISO88025 ||
               saveifnet.if_type == IFT_FDDI)) {
            memcpy(arpcom.ac_enaddr, sadl.sdl_data + sadl.sdl_nlen,
                   sizeof(arpcom.ac_enaddr));
          break;
          }
          ifaddraddr = (unsigned long)ifaddr.ifa_next;
        }

	if (strncmp("lo", saveName, 2) == 0) {
	    /*
	     *  Loopback doesn't have a HW addr, so return 00:00:00:00:00:00
	     */
	    memset(EtherAddr, 0, sizeof(arpcom.ac_enaddr));
	} else {
          memcpy( EtherAddr,(char *) &arpcom.ac_enaddr, sizeof (arpcom.ac_enaddr));
	}

#if 0
fprintf(stderr, "Found interface ethernet addr: %02x%02x%02x%02x%02x%02x\n",
(unsigned char) EtherAddr[0],
(unsigned char) EtherAddr[1],
(unsigned char) EtherAddr[2],
(unsigned char) EtherAddr[3],
(unsigned char) EtherAddr[4],
(unsigned char) EtherAddr[5]);
#endif

	return(0);	/* DONE */
}


int InterfaceScanByKernelIndex(int Index, char *Name,
				   struct ifnet *Retifnet, struct in_ifaddr *Retin_ifaddr)
{
	int i;

	InterfaceScanInit();
	while (InterfaceScanNext(&i, Name, Retifnet, Retin_ifaddr) >= 0) {
		if (i == Index) break;
	}
	if (i != Index)
		return(-1);	 /* Error, doesn't exist */
	return(0);	/* DONE */
}


int InterfaceScanGetCount(void)
{
	return 10;		/* E10 has 10 ports */
#if 0
int Interface_Count=0;

	InterfaceScanInit();
	while (InterfaceScanNext(NULL, NULL, NULL, NULL) >= 0) {
		Interface_Count++;
	}

	return(Interface_Count);
#endif
}


int GetGbeSlots(int *slot, int *size, int *need)
{
int i, count, ipmd_needed = 0;
ipm_if_t ipm_ifdata[64];
int sindex;
int ret_val;

	/* talk to IPMD to get all the GbE interfaces information */
	count = 64;
	ret_val = ipmd_GetIfByName(NULL, ipm_ifdata, sizeof(ipm_ifdata),
											&count, &ipmd_needed);
	if (ret_val == IPM_GET_NO_BUF) {
		/* should do something to increase the buffer size */
		return -1;
	}
	if (ret_val == IPM_GET_ERR) {
		return -1;
	}

	/* find out what slot is GbE NIM located */
	sindex = 0;
	for (i = 0; i < count; i++) {
		int j, newslot, exists;

		newslot = ipm_ifdata[i].if_slot;
		if (newslot > 1) {  /* slot 1 is SSC slot */
			exists = 0;
			for (j = 0; j < sindex; j++) {
				if (slot[j] == newslot) {
					exists = 1;
					break;
				}
			}
			if (exists == 0) {
				slot[sindex] = newslot;
				sindex++;
				if (sindex >= *size)
					break;		/* no more space available */
			}
		}
	}

	if (need) {
		int j, found;

		if ((sindex > 0) && (sindex < *size))
			*need = sindex;
		else {
			*need = 1;
			for (i = 1; i < count; i++) {
				found = 0;
				for (j = 0; j < i; j++) {
					if (ipm_ifdata[i].if_slot == ipm_ifdata[j].if_slot) {
						found = 1;
						break;
					}
				}
				if (found == 0)
					*need += 1;
			}
		}
	}

	*size = sindex;

	return 0;
}


int GetSlotNumber(int interface)
{
	/* slot 1: SSC slot with 2 ports */
	/* slot 2,3: can be either FC or FP NIM */
	/*           4 ports for either type    */
	if (interface <= 2)
		return 1;
	else
		return ((interface - 3)/4 + 2);
}


#define AGILE_CARD_SSC		1
#define AGILE_CARD_FCNIM	2
#define AGILE_CARD_FPNIM	3

int GetSlotType(int slot)
{
int gbeslots[32], numofgbeslots;
int i, cardtype;

	if (slot == 1) {
		/* this is SSC slot for E10 */
		cardtype = AGILE_CARD_SSC;
	}
	else {
		numofgbeslots = 32;
		if (GetGbeSlots(gbeslots, &numofgbeslots, NULL) < 0) {
			/* something's wrong, can't get GbE slot information */
			return NULL;
		}
		if (numofgbeslots == 0)
			return NULL;		/* something is wrong */

		cardtype = AGILE_CARD_FCNIM;
		for (i = 0; i < numofgbeslots; i++) {
			if (slot == gbeslots[i]) {
				cardtype = AGILE_CARD_FPNIM;
				break;
			}
		}
	}

	return cardtype;
}

int GetSscIfStats(int Index, haIfStats_t *ifstat)
{
/* use ipmd now... */
	return -1;
#if 0
int i, rval, found = 0;
char Name[40], searchName[40], PhysAddr[64];
struct ifnet retifnet;
struct in_ifaddr retin_ifaddr;

	if (ifstat == NULL)
		return 0;

	sprintf(searchName, "sc0.%d", Index);

	InterfaceScanInit();
	while (InterfaceScanNext(&i, Name, &retifnet, &retin_ifaddr) >= 0) {
		if (!strcmp(searchName, Name)) {
			found = 1;
			break;
		}
	}
	if (!found)
		return -1;

	/* Now lets get physical address */
	rval = InterfaceGetEtherByKernelIndex(i, PhysAddr);
	if (rval < 0) {
		/* unable to get physical address.             */
		/* still, we need to return the rest of stuffs */
		/* so we just zero out the PhysAddr space      */
		memset(PhysAddr, 0, 64);
	}

	/* seems like we got the data      */
	/* let's pack the data into buffer */

	ifstat->index = Index;
	strncpy(ifstat->name, Name, 40);
	memcpy(ifstat->physaddr, PhysAddr, 64);
	ifstat->type = retifnet.if_type;
	ifstat->mtu = retifnet.if_mtu;
	ifstat->speed = retifnet.if_baudrate;
	ifstat->admstatus = (retifnet.if_flags & IFF_UP ? 1 : 2);
	ifstat->opstatus = (retifnet.if_flags & IFF_RUNNING ? 1 : 2);
	ifstat->ibytes = retifnet.if_ibytes;
	ifstat->iucasts = retifnet.if_ipackets - retifnet.if_imcasts;
	ifstat->inucasts = retifnet.if_imcasts;
	ifstat->idiscards = retifnet.if_iqdrops;
	ifstat->ierrors = retifnet.if_ierrors;
	ifstat->iunknownproto = retifnet.if_noproto;
	ifstat->obytes = retifnet.if_obytes;
	ifstat->oucasts = retifnet.if_opackets - retifnet.if_omcasts;
	ifstat->onucasts = retifnet.if_omcasts;
	ifstat->odiscards = retifnet.if_snd.ifq_drops;
	ifstat->oerrors = retifnet.if_oerrors;
	ifstat->oqlen = retifnet.if_snd.ifq_len;

fprintf(stderr, "index: %d\n", ifstat->index);
fprintf(stderr, "Name: %s\n", ifstat->name);
fprintf(stderr, "ibytes: %d\n", ifstat->ibytes);

	return sizeof(haIfStats_t);
#endif
}

int GetFpnimIfStats(int Index, haIfStats_t *ifstat)
{
/* use ipmd now... */
	return -1;

#if 0
int rval, count;
char Name[40];
ipm_if_t ipmifdata;
ipm_ifstat_t ipmifstats;

	if (ifstat == NULL)
		return 0;

	/* Use IPMD API to get FPNIM interface stats */
	sprintf(Name, "eth:%d/%d", GetSlotNumber(Index), ((Index-3) % 4) + 1);
	count = 1;
	rval = ipmd_GetIfByName(Name, &ipmifdata, sizeof(ipm_if_t), &count, NULL);
	if (rval == IPM_GET_ERR)
		return -1;

	rval = ipmd_GetIfStats(Name, &ipmifstats, sizeof(ipm_ifstat_t), &count, NULL);
	if (rval == IPM_GET_ERR)
		return -1;

	/* got the ifstats from IPMD, now pass it back */
	ifstat->index = Index;
	strncpy(ifstat->name, ipmifdata.if_name, IPM_STR_SIZE);
	memcpy(ifstat->physaddr, ipmifdata.if_mac, 8);
	ifstat->type = ipmifstats.ifi_type;
	ifstat->mtu = ipmifstats.ifi_mtu;
	ifstat->speed = ipmifstats.ifi_baudrate;
	ifstat->admstatus = (ipmifdata.if_flags & IFF_UP ? 1 : 2);
	ifstat->opstatus = (ipmifdata.if_flags & IFF_RUNNING ? 1 : 2);
	ifstat->ibytes = ipmifstats.ifi_ibytes;
	ifstat->iucasts = ipmifstats.ifi_ipackets - ipmifstats.ifi_imcasts;
	ifstat->inucasts = ipmifstats.ifi_imcasts;
	ifstat->idiscards = ipmifstats.ifi_iqdrops;
	ifstat->ierrors = ipmifstats.ifi_ierrors;
	ifstat->iunknownproto = ipmifstats.ifi_noproto;
	ifstat->obytes = ipmifstats.ifi_obytes;
	ifstat->oucasts = ipmifstats.ifi_opackets - ipmifstats.ifi_omcasts;
	ifstat->onucasts = ipmifstats.ifi_omcasts;
	ifstat->odiscards = 0; /* not available in IPM */
	ifstat->oerrors = ipmifstats.ifi_oerrors;
	ifstat->oqlen = ipmifstats.ifi_oQlen;

	return sizeof(haIfStats_t);
#endif
}

int getInterfaceStats(int Index, haIfStats_t *ifstat)
{
int res, count, need;

	/* use ipmd to retrieve interface stats */
	/* Index parameter is no longer used.   */
	/* ifstat parameter should contain the  */
	/* interface name that are used to      */
	/* retrieve the interface stat from ipm */

	count = 1;
	memset(ifstat->stats, 0, sizeof(ipm_ifstat_t));

	res = ipmd_GetIfStats(ifstat->if_name,
					(ipm_ifstat_t *) ifstat->stats,
					sizeof(ipm_ifstat_t), &count, &need);
	if (res == IPM_GET_ERR) {
		return -1;
	}
	else if (res == IPM_GET_NO_BUF) {
		return -1;
	}
	else
	{
		return sizeof(haIfStats_t);
	}

#if 0
int rval;

fprintf(stderr, "getInterfaceStats: Index=%d\n", Index);

	switch (GetSlotType(GetSlotNumber(Index))) {

	case AGILE_CARD_SSC:
		/* SSC port: we'll need to use kernel image to get stats. */
		rval = GetSscIfStats(Index, ifstat);
fprintf(stderr, "getInterfaceStats: %d is SSC port, return %d\n", Index, rval);
		break;


	case AGILE_CARD_FCNIM:
		return -1;

	case AGILE_CARD_FPNIM:
		/* FP port: we'll need to use kernel image to get stats. */
		rval = GetFpnimIfStats(Index, ifstat);
fprintf(stderr, "getInterfaceStats: %d is FP NIM port, return %d\n", Index, rval);
		return -1;

	default:
		return -1;
	}

	return rval;
#endif
}

