/*  File Name  :    agileadmmain.c 
 *
 *  Description:    HTTPD action functions for administartor information
 *
 *  Created    :    5/20/2002
 *
 *  Author     :    Shyh-Pei Yen
 *
 *  Copyright(c)  Agile Storage, Inc. 2002
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <pwd.h>
#include <grp.h>
#include <utmp.h>

#include <sys/time.h>
#include <anpssc-api.h>

#include <nfx-defs.h>
#include <nfx-types.h>
#include <cm-api.h>

#include "agilepost.h"
#include "agileutil.h"

#define INVALID_TAG			0
#define ADMIN_USER			1
#define ADMIN_PRIV			2
#define NUM_OF_TAGS			2

static tagNameTable_t tagTable[NUM_OF_TAGS] = {
	{ADMIN_USER,	"username"},
	{ADMIN_PRIV,	"privilege"},
};



/* HERE ARE VARIABLES FOR ADMINISTRATOR INFORMATION */

typedef struct _admin_t
{
	char username[UT_NAMESIZE+1];
	char ondev[UT_LINESIZE+1];
	int privilege;
} admin_t;

static int dataCount;
static int dataCurr;
#define MAX_ADMIN_COUNT		30
static admin_t adminConfig[MAX_ADMIN_COUNT];

#define NUMPRIVS	5
static char *privNames[NUMPRIVS] =
{
	"View", 
	"Add", 
	"Change", 
	"Full", 
	"Diags"
};

static char *groupNames[NUMPRIVS] =
{
	"nfxview", 
	"nfxadd", 
	"nfxchang", 
	"nfxfull", 
	"nfxdiags"
};




/***********************************************************
 * routines to retrieve filer information thru anpssc
 **********************************************************/

static int getTableRowCount(void)
{
FILE *ufp;
struct utmp usr;
char uname[16];
struct passwd *pwd;
struct group *grp;
int i, count = 0;

	ufp = fopen(_PATH_UTMP, "r");
	if (ufp == NULL)
		return 0;

	count = 0;
	while (fread((char *)&usr, sizeof(usr), 1, ufp) == 1)
	{
		if (usr.ut_name[0] && usr.ut_line[0])
		{
			/* username */
			memset(uname, 0, sizeof(uname));
			strncpy(uname, usr.ut_name, UT_NAMESIZE);

			/* password, in this data strcuture we can find gid */
			pwd = getpwnam(uname);
			if (!pwd)
				continue;

			/* group name */
			grp = getgrgid(pwd->pw_gid);
			if (!grp)
				continue;

			for (i = 0; i < NUMPRIVS; i++)
			{
				if (!strcmp(grp->gr_name, groupNames[i]))
				{
					strncpy(adminConfig[count].username, usr.ut_name, UT_NAMESIZE);
					strncpy(adminConfig[count].ondev, usr.ut_line, UT_LINESIZE);
					adminConfig[count].privilege = i;
					count++;
					break;
				}
			}

			if (count >= MAX_ADMIN_COUNT)
				break;
		}
	}
	fclose(ufp);

	return count;
}



/***********************************************************
 *  Main routines to process tags
 **********************************************************/

static int getTagValue(char *tag, char *value)
{
int i, tagType;
	
	if ((dataCurr <= 0) || (dataCurr > dataCount))
		return -1;

	tagType = INVALID_TAG;
	for (i = 0; i < NUM_OF_TAGS; i++) {
		if (!strcmp(tag, tagTable[i].tagName)) {
			tagType = tagTable[i].tagType;
			break;
		}
	}
	if (tagType == INVALID_TAG)
		return -1;

	switch (tagType) 
	{
	case ADMIN_USER:
		sprintf(value, "%s", adminConfig[dataCurr-1].username);
		break;
	case ADMIN_PRIV:
		i = adminConfig[dataCurr-1].privilege;
		strcpy(value, privNames[i]);
		break;

	default:
		return -1;
	}

	return 1;
}


/*
 * actRptTagFunc()
 */
static int actRptTagFunc(tagCallbackParams_t *tp)
{
char *tag = tp->tag;
char *value = tp->value;

	if (!strcmp(tag, "TABLE_BEGIN"))
	{
		/* TABLE_BEGIN tag, need to see if there is any data */
		dataCount = getTableRowCount();
		if (dataCount > 0)
		{
			/* need to retrieve first table's data */
			dataCurr = 1;
			return 1;
		}

		dataCurr = 0;
		return -1;
	}
	else if (!strcmp(tag, "TABLE_END"))
	{
		/* TABLE_END tag, check if end of data */
		if ((dataCount <= 0) || (dataCurr <= 0) || (dataCurr >= dataCount))
			return -1;

		dataCurr++;
		return 1;
	}
	else
	{
		/* other tags */
		if (dataCount <= 0)
		{
			dataCount = getTableRowCount();
			if (dataCount <= 0)
				return -1;
		}

		if ((dataCurr < 1) || (dataCurr > dataCount))
			dataCurr = 1;

		return getTagValue(tag, value);
	}

	/* shouldn't reach here... */
	return -1;
}

int actionAdminMain(void *r, int count, char **argname, char **argval)
{
char newfname[128];

	memset(newfname, 0, sizeof(newfname));
	prepare_agile_template(r, newfname);

	/* initialize storage information variables */
	dataCount = 0;
	dataCurr = 0;

	agile_scan_and_send(r, actRptTagFunc);

	return 1;
}
