/*  File Name  :    agilebanner.c
 *
 *  Description:    HTTPD action functions for displaying banner.
 *
 *  Created    :    3/11/2002
 *
 *  Author     :    Tiffany Chiang
 *
 *  Copyright(c)  Agile Storage, Inc. 2001
 *
 */

#include <stdio.h>
#include <sys/types.h>
#include <unistd.h>
#include <anpssc-api.h>

#include "agilepost.h"
#include "agileutil.h"
#include "../agent/httpdagent.h"

#define INVALID_TAG			0
#define SYS_IP				1
#define SYS_HOST			2
#define NUM_OF_TAGS			2

static tagNameTable_t tagTable[NUM_OF_TAGS] = {
	{SYS_IP,		"system.ip"},
	{SYS_HOST,		"system.hostname"},
};


/* HERE ARE VARIABLES FOR NETWORK INFORMATION */
static char ip[20];
static char host[80];
static int getres = -1;



/***********************************************************
 *  Main routines to process tags
 **********************************************************/

static int getTagValue(char *tag, char *value)
{
	int i, tagType;
	
	if (getres < 0)
		return -1;

	tagType = INVALID_TAG;
	for (i = 0; i < NUM_OF_TAGS; i++) {
		if (!strcmp(tag, tagTable[i].tagName)) {
			tagType = tagTable[i].tagType;
			break;
		}
	}
	if (tagType == INVALID_TAG)
		return -1;

	switch (tagType) 
	{
	case SYS_IP:
		strcpy(value, ip);
		break;
	case SYS_HOST:
		strcpy(value, host);
		break;
	default:
		return -1;
	}

	return 1;
}


/*
 * actRptTagFunc()
 *	tag   - tag name
 *	value - replacement string for tag
 */
static int actRptTagFunc(tagCallbackParams_t *tp)
{
char *tag = tp->tag;
char *value = tp->value;

	return getTagValue(tag, value);
}

int actionBanner(void *r)
{
	/* initialize variables */
	GetSystemHostInfo(r, ip, host);
	getres = 1;

	prepare_agile_template(r, NULL);

	agile_scan_and_send(r, actRptTagFunc);

	return 1;
}
