/*  File Name  :    agilediskaddvolact.c 
 *
 *  Description:    HTTPD action functions for Add Volume from Disk Page.
 *
 *  Created    :    3/15/2002
 *
 *  Author     :    Tiffany Chiang
 *
 *  Copyright(c)  Agile Storage, Inc. 2001
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

// #include <sys/types.h>
// #include <unistd.h>
#include <sys/time.h>
#include <anpssc-api.h>

#include <nfx-defs.h>
#include <nfx-types.h>
#include <evm_com_defs.h>
#include <ea-api.h>

#include "agilepost.h"
#include "agileutil.h"


#define BYTES_IN_MB		(1024*1024)

#define PARAM_MAKE			1
#define PARAM_MODEL			2
#define PARAM_RAID			3
#define PARAM_NAME			4
#define PARAM_MINLUN		5
#define PARAM_MAXLUN		6
#define PARAM_HQUOTA		7
#define PARAM_AGROW			8
#define PARAM_MINLSEL		9
#define PARAM_MAXLSEL		10
#define PARAM_HQSEL			11
#define PARAM_SQSEL			12
#define PARAM_SQUOTA		13
#define PARAM_AGSEL			14
#define PARAM_AGTHR			15
#define NUM_OF_PARAMS		15

static paramNameTable_t paramTable[NUM_OF_PARAMS] = {
	{PARAM_MAKE,	"make"},
	{PARAM_MODEL,	"model"},
	{PARAM_RAID,	"raid"},
	{PARAM_NAME,	"name"},
	{PARAM_MINLUN,	"minlun"},
	{PARAM_MAXLUN,	"maxlun"},
	{PARAM_HQUOTA,	"hq"},
	{PARAM_AGROW,	"ag"},
	{PARAM_MINLSEL,	"minlunr"},
	{PARAM_MAXLSEL,	"maxlunr"},
	{PARAM_HQSEL,	"hqr"},
	{PARAM_SQSEL,	"sqr"},
	{PARAM_SQUOTA,	"sq"},
	{PARAM_AGSEL,	"agr"},
	{PARAM_AGTHR,	"agt"},
};


/* HERE ARE VARIABLES FOR VOLUME INFORMATION */
static char inMake[SDM_VENDOR_NAME_LEN+1] = "";
static char inModel[SDM_MODEL_NAME_LEN+1] = "";
static int inRaid = EVM_RAID_NONE;
static char inName[VOL_MAX_NAME+1];
static uint64 inMinLun = EVM_DONT_CARE;
static uint64 inMaxLun = EVM_DONT_CARE;
static uint64 inHQuota = 0;
static uint64 inAGrow = 0;
static int inMinLunSel;
static int inMaxLunSel;
static int inHQuotaSel;
#if 0
static int inSQuotaSel;
#endif
static uint32 inSQuota;
static int inAGrowSel;
static uint64 inAGThrehold;


/***********************************************************
 *  Main routines to handle Add Volume actions
 **********************************************************/

static int getInputParams(int count, char **argname, char **argval)
{
	int argind, paramType;

	for (argind = 0; argind < count; argind ++)
	{
		paramType = GetParamType(argname[argind], paramTable, NUM_OF_PARAMS);
		if (paramType == INVALID_PARAM)
			continue;

		switch (paramType) 
		{
		case PARAM_MAKE:
			strcpy(inMake, argval[argind]);
			break;
		case PARAM_MODEL:
			strcpy(inModel, argval[argind]);
			break;
		case PARAM_RAID:
			sscanf(argval[argind], "%d", &inRaid);
			break;
		case PARAM_NAME:
			strcpy(inName, argval[argind]);
			break;
		case PARAM_MINLUN:
			sscanf(argval[argind], "%lld", &inMinLun);
			if (inMinLun == 0)
				inMinLun = EVM_DONT_CARE;
			break;
		case PARAM_MAXLUN:
			sscanf(argval[argind], "%lld", &inMaxLun);
			if (inMaxLun == 0)
				inMaxLun = EVM_DONT_CARE;
			break;
		case PARAM_HQUOTA:
			sscanf(argval[argind], "%lld", &inHQuota);
			break;
		case PARAM_AGROW:
			sscanf(argval[argind], "%lld", &inAGrow);
			break;
		case PARAM_MINLSEL:
			sscanf(argval[argind], "%d", &inMinLunSel);
			break;
		case PARAM_MAXLSEL:
			sscanf(argval[argind], "%d", &inMaxLunSel);
			break;
		case PARAM_HQSEL:
			sscanf(argval[argind], "%d", &inHQuotaSel);
			break;
#if 0
		case PARAM_SQSEL:
			sscanf(argval[argind], "%d", &inSQuotaSel);
			break;
#endif
		case PARAM_AGSEL:
			sscanf(argval[argind], "%d", &inAGrowSel);
			break;
		case PARAM_SQUOTA:
			sscanf(argval[argind], "%d", &inSQuota);
			break;
		case PARAM_AGTHR:
			sscanf(argval[argind], "%lld", &inAGThrehold);
			break;
		}
	}

	return 0;
}

static int DoVolumeAdd()
{
    int					stat;
    ea_addFsysReq_t		req;
    ea_addFsysRsp_t		rsp;
	struct timeval		timeout = {90, 0};
    
    //Fill the request. 
    bzero((void *) &req, sizeof(ea_addFsysReq_t));
    req.hdr.msgType = EA_ADD_FSYS_REQ;
    strcpy(req.hdr.node, "\0");
	strcpy(req.fsysId.fsysName, inName);
    req.volSize = 0;
	if (inHQuotaSel == 1)
		req.maxSize = GetBlockfromMB(inHQuota);
	else
		req.maxSize = 0;
	req.softQuota = inSQuota;
	if (inAGrowSel == 1)
		req.minAutoGrow = GetBlockfromMB(inAGrow);
	else
		req.minAutoGrow = 0;
    req.highWaterMark = GetBlockfromMB(inAGThrehold);
	if (inMinLunSel == 1)
		req.attr.minLunSize = GetBlockfromMB(inMinLun);
	else
		req.attr.minLunSize = EVM_DONT_CARE;
	if (inMaxLunSel == 1)
		req.attr.maxLunSize = GetBlockfromMB(inMaxLun);
	else
		req.attr.maxLunSize = EVM_DONT_CARE;
	req.attr.num = 1;
	req.attr.raidLevel = inRaid;
	strcpy(req.attr.vendorInfo[0].vendor, inMake);
	strcpy(req.attr.vendorInfo[0].model, inModel);
	req.attr.flags = EVM_SAME_VENDOR | EVM_SAME_MODEL;
	req.nim = 1;

    // Send message to EA. 
    stat = sendAgileMsg(-1, &req, sizeof(req),
                        "ea", NFX_SSC_UNIX, SSC_SLOT, MGMTPLANE,
                        &timeout, &rsp, sizeof(ea_addFsysRsp_t));

    // Check response from EA. 
	if (stat < 0) 
		return -1;
	else if (!stat)
		return -1;

    if (rsp.hdr.msgType != EA_ADD_FSYS_RSP)
		return -1;
	if (rsp.result != NFX_OK)
		return -1;

	return 1;
}

static void HandleVolAction(char *newfname, int count, char **argname, char **argval)
{
	int res;

	getInputParams(count, argname, argval);
	res = DoVolumeAdd();
	if (res <= 0)
		strcpy(newfname, "error_diskaddvolume.html");
	else
		strcpy(newfname, "DiskAddVolumeRes.html");
}


/***********************************************************
 *  Main routines to process tags
 **********************************************************/

static int getTagValue(char *tag, char *value)
{
	return -1;
#if 0
	int i, tagType, priv, prev, next;
	uint64 size;
	
	tagType = INVALID_TAG;
	for (i = 0; i < NUM_OF_TAGS; i++) {
		if (!strcmp(tag, tagTable[i].tagName)) {
			tagType = tagTable[i].tagType;
			break;
		}
	}
	if (tagType == INVALID_TAG)
		return -1;

	switch (tagType) 
	{
	default:
		return -1;
	}

	return 1;
#endif
}


/*
 * actRptTagFunc()
 *	tag   - tag name
 *	value - replacement string for tag
 */
static int actRptTagFunc(tagCallbackParams_t *tp)
{
char *tag = tp->tag;
char *value = tp->value;

	if (!strcmp(tag, "TABLE_BEGIN")) {
		return -1;
	}
	else if (!strcmp(tag, "TABLE_END")) {
		return -1;
	}
	else {
		/* other tags */
		return getTagValue(tag, value);
	}

	/* shouldn't reach here... */
	return -1;
}

int actionDiskAddVolAct(void *r, int count, char **argname, char **argval)
{
	char newfname[128];

	memset(newfname, 0, sizeof(newfname));

	/* Handle Add Volume action */
	HandleVolAction(newfname, count, argname, argval);

	prepare_agile_template(r, newfname);

	agile_scan_and_send(r, actRptTagFunc);

	return 1;
}
