/*  File Name  :    agilefilerslot.c 
 *
 *  Description:    HTTPD action functions for filer slot information
 *
 *  Created    :    5/15/2002
 *
 *  Author     :    Shyh-Pei Yen
 *
 *  Copyright(c)  Agile Storage, Inc. 2001
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <sys/time.h>
#include <anpssc-api.h>

#include <nfx-defs.h>
#include <nfx-types.h>
#include <cm-api.h>

#include "agilepost.h"
#include "agileutil.h"
#include "../agent/httpdagent.h"

#define INVALID_TAG			0
#define SLOTCFG_SLOT		1
#define SLOTCFG_CPUNAME		2
#define SLOTCFG_CPU			3
#define SLOTCFG_SOFTVER		4
#define SLOTCFG_CPUSTATE	5
#define SLOTCFG_OP			6
#define NUM_OF_TAGS			6

static tagNameTable_t tagTable[NUM_OF_TAGS] = {
	{SLOTCFG_SLOT,		"slot"},
	{SLOTCFG_CPUNAME,	"cpuname"},
	{SLOTCFG_CPU,		"cpu"},
	{SLOTCFG_SOFTVER,	"softver"},
	{SLOTCFG_CPUSTATE,	"state"},
	{SLOTCFG_OP,		"op"},
};


#define PARAM_SLOT			1
#define PARAM_CPU			2
#define PARAM_ACT			3
#define NUM_OF_PARAMS		3

static paramNameTable_t paramTable[NUM_OF_PARAMS] = {
	{PARAM_SLOT,	"slot"},
	{PARAM_CPU,		"cpu"},
	{PARAM_ACT,		"act"},
};


/* HERE ARE VARIABLES FOR FILER SYSTEM INFORMATION */

typedef struct _slot_config_t
{
	int cpu;
	int	state;
	char prom_ver[CM_VER_STRING_LEN];
	char rt_ver[CM_VER_STRING_LEN];
} slot_config_t;

static int32 inSlot;
static int32 inCpu;
static char inAct[10];
static int dataCount;
static int dataCurr;

static int numCpus = 0;
static int boardType = -1;
static slot_config_t slotConfig[CM_NUM_CPUS];
static slot_config_t *pstData;

char *cpuName[CM_NUM_CARDTYPES][CM_NUM_CPUS] =
{
	{CM_CPU_TYPE_SSC_STR, CM_CPU_TYPE_SSC_MGMT_STR, "UNKNOWN2", "UNKNOWN3"},
	{CM_CPU_TYPE_TXRX0_STR, CM_CPU_TYPE_TXRX1_STR, CM_CPU_TYPE_FP0_STR, CM_CPU_TYPE_FP1_STR},
	{CM_CPU_TYPE_TXRX0_STR, CM_CPU_TYPE_TXRX1_STR, CM_CPU_TYPE_FP0_STR, CM_CPU_TYPE_FP1_STR},
	{CM_CPU_TYPE_FC0_STR, CM_CPU_TYPE_FC1_STR, CM_CPU_TYPE_FC2_STR, CM_CPU_TYPE_FC3_STR}
};

char *stateName[] =
{
	"UNKNOWN",
	"PROM_INIT",
	"PROM",
	"PROM_CRASH",
	"LOADING",
	"CTEXT_SWITCH",
	"RT_INIT",
	"UP",
	"DOWN",
	"REBOOT",
	"UNKNOWN",
	"UNKNOWN",
	"UNKNOWN",
	"UNKNOWN",
	"UNKNOWN",
	"UNKNOWN",
};


/***********************************************************
 *  Main routines to handle Reset/Enable/Disable Port actions
 **********************************************************/

static int getInputParams(int count, char **argname, char **argval)
{
	int argind, paramType;

	for (argind = 0; argind < count; argind ++)
	{
		paramType = GetParamType(argname[argind], paramTable, NUM_OF_PARAMS);
		if (paramType == INVALID_PARAM)
			continue;

		switch (paramType) 
		{
		case PARAM_SLOT:
			sscanf(argval[argind], "%d", &inSlot);
			break;
		case PARAM_CPU:
			sscanf(argval[argind], "%d", &inCpu);
			break;
		case PARAM_ACT:
			strcpy(inAct, argval[argind]);
			break;
		}
	}

	return 0;
}

static int DoRebootCpu(int slot, int cpu)
{
int stat;
chassisdMsgRebootReq_t req;
chassisdMsgRebootResp_t resp;
struct timeval to = { 5, 0 };

	memset(&resp, 0, sizeof(resp));

	if (slot >= CM_NUM_SLOTS)
		return -1;
	if (cpu >= CM_NUM_CPUS)
		return -1;

	if (slot == CM_CARD_TYPE_SSC)
	{
	haMessage_t req;
	haMessage_t *rsp;

		req.type = HA_REBOOT_SSC;
		req.datalen = 0;

		stat = sendAgileMsg(-1, &req, sizeof(req),
					"httpdagent", NFX_SSC_UNIX, SSC_SLOT, MGMTPLANE,
					&to, rsp, sizeof(haMessage_t));
		if ((stat <= 0) || (rsp->rspstatus != HARSP_OK))
			return -1;
	}
	else
	{
		req.hdr.mid = CHASSISD_REBOOT_REQ;
		req.reboot_type = CM_REBOOT_TYPE_SLICE;
		req.slot = slot;
		req.slice = cpu;

		stat = sendAgileMsg(-1, &req, sizeof(req),
				"cm", NFX_SSC_UNIX, SSC_SLOT, 0, &to, &resp, sizeof(resp));

		if ((stat <= 0) || (resp.rc != CM_REBOOT_OK))
			return -1;
	}

	return 1;
}

static int HandlePortAction(char *newfname, int count, char **argname, char **argval)
{
	int res;

	getInputParams(count, argname, argval);

	if (!strcmp(inAct, "reboot"))
	{
		res = DoRebootCpu(inSlot, inCpu);
		if (res <= 0)
			strcpy(newfname, "error_rebootcpu.html");
	}
	return 1;
}


/***********************************************************
 * routines to retrieve slot information thru anpssc
 **********************************************************/

static int GetSlotConfigInfo(int slot)
{
chassisdMsgInfoReq_t req;
chassisdMsgInfoResp_t resp;
chassisdMsgVersionReq_t ver_req;
chassisdMsgVersionResp_t ver_resp;
struct timeval to = { 5, 0 };
int stat, i;

	memset(slotConfig, 0, sizeof(slotConfig));

	memset(&req, 0, sizeof(req));
	memset(&resp, 0, sizeof(resp));
	memset(&ver_req, 0, sizeof(ver_req));
	memset(&ver_resp, 0, sizeof(ver_resp));

	req.hdr.mid = CHASSISD_GET_CHASSIS_INFO_REQ;

	stat = sendAgileMsg(-1, &req, sizeof(req),
						"cm", NFX_SSC_UNIX, SSC_SLOT, 0,
						&to, &resp, sizeof(resp));
	if (stat <= 0)
		return 0;

	/* we don't care if the request failed... */
	ver_req.hdr.mid = CHASSISD_GET_VERSION_REQ;
	stat = sendAgileMsg(-1, &ver_req, sizeof(ver_req),
						"cm", NFX_SSC_UNIX, SSC_SLOT, 0,
						&to, &ver_resp, sizeof(ver_resp));

	if (resp.present[slot])
	{
		numCpus = resp.num_cpus[slot];
		boardType = resp.board_type[slot];
		for (i = 0; i < numCpus; i++)
		{
			slotConfig[i].cpu = i;
			slotConfig[i].state = resp.cpu_state[slot][i];
			strcpy(slotConfig[i].prom_ver, ver_resp.prom_ver[slot][i]);
			strcpy(slotConfig[i].rt_ver, ver_resp.rt_ver[slot][i]);
		}
	}
	else
		numCpus = 0;

	return numCpus;
}

static int getTableRowCount(void)
{
	return GetSlotConfigInfo(inSlot);
}


/***********************************************************
 *  Main routines to process tags
 **********************************************************/

static int getTagValue(char *tag, char *value)
{
	int i, tagType;
	char tmpbuf[1024];
	
	tagType = INVALID_TAG;
	for (i = 0; i < NUM_OF_TAGS; i++) {
		if (!strcmp(tag, tagTable[i].tagName)) {
			tagType = tagTable[i].tagType;
			break;
		}
	}
	if (tagType == INVALID_TAG)
		return -1;

	if (tagType == SLOTCFG_SLOT)
	{
		sprintf(value, "%d", inSlot);
		return 1;
	}

	if ((dataCurr <= 0) || (dataCurr > dataCount))
		return -1;

	switch (tagType) 
	{
	case SLOTCFG_SLOT:
		sprintf(value, "%d", inSlot);
		break;
	case SLOTCFG_CPUNAME:
		if (dataCurr <= numCpus)
			strcpy(value, cpuName[boardType][pstData->cpu]);
		else
			strcpy(value, "UNKNOWN");
		break;
	case SLOTCFG_CPU:
		sprintf(value, "%d", dataCurr-1);
		break;
	case SLOTCFG_SOFTVER:
		value[0] = '\0';
		if (strlen(pstData->prom_ver)) {
			sprintf(tmpbuf, "%s<br>", pstData->prom_ver);
			strcat(value, tmpbuf);
		}
		if (strlen(pstData->rt_ver)) {
			sprintf(tmpbuf, "%s<br>", pstData->rt_ver);
			strcat(value, tmpbuf);
		}
		if (!value[0])
			strcpy(value, "&nbsp");
		break;
	case SLOTCFG_CPUSTATE:
		if (pstData->state <= 16)
			strcpy(value, stateName[pstData->state]);
		else
			strcpy(value, "UNKNOWN");
		break;
	case SLOTCFG_OP:
		/* SSC_MGMT, FP are not allowed to reboot single CPU */
		/* Only SSC, and FC are */
		if (((boardType == CM_CARD_TYPE_SSC) && (pstData->cpu == 0)) || /* SSC CPU */
			(boardType == CM_CARD_TYPE_FCNIM) )		/* FC */
		{
			sprintf(value, "<a href=\"/agilerequestprefix/FilerCpuReboot?act=reboot&slot=%d&cpu=%d&cpuname=%s\">Reset</a>", inSlot, dataCurr-1, cpuName[boardType][pstData->cpu]);
		}
		else
		{
			sprintf(value, "%s", "&nbsp");
		}
		break;

	default:
		return -1;
	}

	return 1;
}


/*
 * actRptTagFunc()
 *	tag   - tag name
 *	value - replacement string for tag
 */
static int actRptTagFunc(tagCallbackParams_t *tp)
{
char *tag = tp->tag;
char *value = tp->value;

	if (!strcmp(tag, "TABLE_BEGIN")) {
		/* TABLE_BEGIN tag, need to see if there is any data */
		dataCount = getTableRowCount();
		if (dataCount > 0) {
			/* need to retrieve first table's data */
			dataCurr = 1;
			pstData = &slotConfig[0];
			return 1;
		}

		dataCurr = 0;
		return -1;
	}
	else if (!strcmp(tag, "TABLE_END")) {
		/* TABLE_END tag, check if end of data */
		if ((dataCount <= 0) || (dataCurr <= 0) || (dataCurr >= dataCount))
			return -1;

		dataCurr++;
		pstData = &slotConfig[dataCurr-1];
		return 1;
	}
	else {
		/* other tags */
		if (dataCount <= 0) {
			dataCount = getTableRowCount();
			if (dataCount <= 0)
				return -1;
		}

		if ((dataCurr < 1) || (dataCurr > dataCount)) {
			dataCurr = 1;
			pstData = &slotConfig[0];
		}

		return getTagValue(tag, value);
	}

	/* shouldn't reach here... */
	return -1;
}

int actionFilerSlot(void *r, int count, char **argname, char **argval)
{
char newfname[128];

	memset(newfname, 0, sizeof(newfname));
	numCpus = 0;
	boardType = -1;
	inSlot = 0;
	inCpu = 0;
	inAct[0] = '\0';

	/* Handle reboot slot actions */
	if (count > 0)
		HandlePortAction(newfname, count, argname, argval);

	prepare_agile_template(r, newfname);

	/* initialize storage information variables */
	dataCount = 0;
	dataCurr = 0;

	agile_scan_and_send(r, actRptTagFunc);

	return 1;
}
