/*  File Name  :    agilefilersysop.c
 *
 *  Description:    HTTPD action functions for reboot/halt filer system
 *
 *  Created    :    7/19/2002
 *
 *  Author     :    Shyh-Pei Yen
 *
 *  Copyright(c)  Agile Storage, Inc. 2001
 *
 */

#include <sys/types.h>
#include <string.h>
#include <unistd.h>
#include <anpssc-api.h>

#include "agilepost.h"
#include "agileutil.h"


#define INVALID_TAG			0
#define FILER_FILERNAME		1
#define FILER_SYSOP			2
#define NUM_OF_TAGS			2

static tagNameTable_t tagTable[NUM_OF_TAGS] = {
	{FILER_FILERNAME,	"filername"},
	{FILER_SYSOP,		"sysop"},
};


#define PARAM_ACT			1
#define NUM_OF_PARAMS		1

static paramNameTable_t paramTable[NUM_OF_PARAMS] = {
	{PARAM_ACT,		"act"},
};

/* HERE ARE VARIABLES FOR NETWORK INFORMATION */
static char inAct[64];



/***********************************************************
 * routines to retrieve network information 
 **********************************************************/

static int getInputParams(int count, char **argname, char **argval)
{
	int argind, paramType;

	for (argind = 0; argind < count; argind ++)
	{
		paramType = GetParamType(argname[argind], paramTable, NUM_OF_PARAMS);
		if (paramType == INVALID_PARAM)
			continue;

		switch (paramType) 
		{
		case PARAM_ACT:
			strncpy(inAct, argval[argind], sizeof(inAct));
			break;
		}
	}

	return 0;
}


/***********************************************************
 *  Main routines to process tags
 **********************************************************/

static int getTagValue(char *tag, char *value)
{
	int i, tagType;
	
	tagType = INVALID_TAG;
	for (i = 0; i < NUM_OF_TAGS; i++) {
		if (!strcmp(tag, tagTable[i].tagName)) {
			tagType = tagTable[i].tagType;
			break;
		}
	}
	if (tagType == INVALID_TAG)
		return -1;

	switch (tagType) 
	{
	case FILER_FILERNAME:
		gethostname(value, 80);
		break;
	case FILER_SYSOP:
		if (strcmp(inAct, "halt") == 0)
			strcpy(value, "Halt");
		else
			strcpy(value, "Reboot");
		break;
	default:
		return -1;
	}

	return 1;
}


/*
 * actRptTagFunc()
 *	tag   - tag name
 *	value - replacement string for tag
 */
static int actRptTagFunc(tagCallbackParams_t *tp)
{
char *tag = tp->tag;
char *value = tp->value;

	if (!strcmp(tag, "TABLE_BEGIN"))
		return -1;
	else if (!strcmp(tag, "TABLE_END"))
		return -1;
	else
		return getTagValue(tag, value);

	/* shouldn't reach here... */
	return -1;
}

int actionFilerSysOp(void *r, int count, char **argname, char **argval)
{
	/* initialize network information variables */
	getInputParams(count, argname, argval);

	prepare_agile_template(r, NULL);

	agile_scan_and_send(r, actRptTagFunc);

	return 1;
}
