/*  File Name  :    agilefsetugquo.c 
 *
 *  Description:    HTTPD action functions for displaying/modifing FileSet group/user quota.
 *
 *  Created    :    3/21/2002
 *
 *  Author     :    Tiffany Chiang
 *
 *  Copyright(c)  Agile Storage, Inc. 2001
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <sys/types.h>
#include <unistd.h>
#include <anpssc-api.h>
#include <evm_com_defs.h>
#include <evm-api.h>
#include <fs-api.h>

#include "agilepost.h"
#include "agileutil.h"

#define INVALID_TAG			0
#define FS_VOLID			1
#define FS_VOLNAME			2
#define FS_PATH				3
#define FS_TYPE				4
#define FS_TYPESTR			5
#define FS_GUID				6
#define FS_ISOFT			7
#define FS_IHARD			8
#define FS_ITIME			9
#define FS_BSOFT			10
#define FS_BHARD			11
#define FS_BTIME			12
#define NUM_OF_TAGS			12

static tagNameTable_t tagTable[NUM_OF_TAGS] = {
	{FS_VOLID,		"args.vol_id"},
	{FS_VOLNAME,	"args.vol_name"},
	{FS_PATH,		"args.path"},
	{FS_TYPE,		"args.type"},
	{FS_TYPESTR,	"args.typestr"},
	{FS_GUID,		"fileset_entry.id"},
	{FS_ISOFT,		"fileset_entry.inodeSoftLimit"},
	{FS_IHARD,		"fileset_entry.inodeHardLimit"},
	{FS_ITIME,		"fileset_entry.inodeTimeLimit"},
	{FS_BSOFT,		"fileset_entry.blockSoftLimit"},
	{FS_BHARD,		"fileset_entry.blockHardLimit"},
	{FS_BTIME,		"fileset_entry.blockTimeLimit"},
};


#define PARAM_TYPE			1
#define PARAM_ACT			2
#define PARAM_VOLID			3
#define PARAM_VOLNAME		4
#define PARAM_FSPATH		5
#define PARAM_ISOFT			6
#define PARAM_IHARD			7
#define PARAM_ITIME			8
#define PARAM_BSOFT			9
#define PARAM_BHARD			10
#define PARAM_BTIME			11
#define PARAM_UGID			12
#define NUM_OF_PARAMS		12

static paramNameTable_t paramTable[NUM_OF_PARAMS] = {
	{PARAM_TYPE,	"type"},
	{PARAM_ACT,		"act"},
	{PARAM_VOLID,	"id"},
	{PARAM_VOLNAME,	"vn"},
	{PARAM_FSPATH,	"path"},
	{PARAM_ISOFT,	"isoft"},
	{PARAM_IHARD,	"ihard"},
	{PARAM_ITIME,	"itime"},
	{PARAM_BSOFT,	"bsoft"},
	{PARAM_BHARD,	"bhard"},
	{PARAM_BTIME,	"btime"},
	{PARAM_UGID,	"ugid"},
};


/* HERE ARE VARIABLES FOR FILESET INFORMATION */
#define MAX_NUM_OF_USG		10

static char inType[5];
static char inAct[5];
static uint64 inId;
static char inName[VOL_MAX_NAME+1];
static char inPath[FS_PATH_MAX+1];
static fs_QuotaBlk_t inData;

static int dataCount;
static int dataCurr;
static fs_QuotaBlk_t stData;
static fs_QuotaBlk_t *quoList;


/***********************************************************
 *  Main routines to handle Add/Delete/Qouta On/Off actions
 **********************************************************/

static int getInputParams(int count, char **argname, char **argval)
{
	int argind, paramType;

	for (argind = 0; argind < count; argind ++)
	{
		paramType = GetParamType(argname[argind], paramTable, NUM_OF_PARAMS);
		if (paramType == INVALID_PARAM)
			continue;

		switch (paramType) 
		{
		case PARAM_TYPE:
			strcpy(inType, argval[argind]);
			break;
		case PARAM_ACT:
			strcpy(inAct, argval[argind]);
			break;
		case PARAM_VOLID:
			sscanf(argval[argind], "%lld", &inId);
			break;
		case PARAM_VOLNAME:
			strcpy(inName, argval[argind]);
			break;
		case PARAM_FSPATH:
			strcpy(inPath, argval[argind]);
			break;
		case PARAM_ISOFT:
			sscanf(argval[argind], "%lld", &inData.inodeSoftLimit);
			break;
		case PARAM_IHARD:
			sscanf(argval[argind], "%lld", &inData.inodeHardLimit);
			break;
		case PARAM_ITIME:
			sscanf(argval[argind], "%d", &inData.inodeTimeLimit);
			break;
		case PARAM_BSOFT:
			sscanf(argval[argind], "%lld", &inData.blockSoftLimit);
			break;
		case PARAM_BHARD:
			sscanf(argval[argind], "%lld", &inData.blockHardLimit);
			break;
		case PARAM_BTIME:
			sscanf(argval[argind], "%d", &inData.blockTimeLimit);
			break;
		case PARAM_UGID:
			sscanf(argval[argind], "%lld", &inData.id);
			break;
		}
	}

	return 0;
}

static int DoQuotaMod()
{
    int						stat;
    fs_MsgSetQuotaReq_t		req;
    fs_MsgSetQuotaResp_t	rsp;
	struct timeval			timeout = {60, 0};
    
    /* Fill the request. */
    memset(&req, 0, sizeof(fs_MsgSetQuotaReq_t));

	req.msgHdr.requestType = FS_REQ_SET_QUOTA;
	req.fsid.whole_thing = inId;
	strcpy(req.path, inPath);
	req.numberLimits = 1;
	if (strcmp(inType, "u") == 0)
		inData.flag = FS_QUOTA_USER;
	else
		inData.flag = FS_QUOTA_GROUP;
	memcpy(&req.limits[0], &inData, sizeof(fs_QuotaBlk_t));

    /* Send message to FS. */
    stat = sendAgileMsg(-1, &req, sizeof(req),
                        "efs", NFX_NFP_FP2, 1, MGMTPLANE,
						&timeout, &rsp, sizeof(rsp));

    /* Check response from FS. */
	if (stat <= 0) 
		return -1;

	if (rsp.msgHdr.fsStatus != FS_ERR_NONE)
		return -1;
	if (rsp.responses[0].status != FS_ERR_NONE)
		return -1;

	return 1;
}

static int HandleFsAction(char *newfname, int count, char **argname, char **argval)
{
	int res;

	getInputParams(count, argname, argval);
	res = 1;
	if (!strcmp(inAct, "mod"))
	{
		res = DoQuotaMod();
		if (res <= 0)
			strcpy(newfname, "error_fssetugquota.html");
	}
	return res;
}


/***********************************************************
 * routines to get fileset information thru anpssc
 **********************************************************/

static int getTableRowCount(void)
{
    int						stat, i;
    char					respbuff[4096];
	fs_MsgReportQuotaReq_t	req;
	fs_MsgReportQuotaResp_t	*rsp;
	struct timeval			timeout = {60, 0};
	int						sindex = 0;

    /* Fill the request. */
    memset(&req, 0, sizeof(fs_MsgReportQuotaReq_t));

	req.msgHdr.requestType = FS_REQ_REPORT_QUOTA;
	req.fsid.whole_thing = inId;
	strcpy(req.path, inPath);
	if (strcmp(inType, "u") == 0)
		req.flag = FS_QUOTA_USER;
	else
		req.flag = FS_QUOTA_GROUP;
	req.numberEntries = MAX_NUM_OF_USG;

	while (1)
	{
		req.startIndex = sindex;

		/* Send message to FS. */
		stat = sendAgileMsg(-1, &req, sizeof(req),
					 "efs", NFX_NFP_FP2, 1, MGMTPLANE,
					 &timeout, respbuff, sizeof(respbuff));

		/* Check response from FS. */
		if (stat <= 0) 
			break;

		rsp = (fs_MsgReportQuotaResp_t *)respbuff;

		if (rsp->msgHdr.fsStatus != FS_ERR_NONE)
			break;
		if (rsp->numberResponses <= 0)
			break;

		if (quoList == NULL)
		{
			quoList = malloc(rsp->numberResponses * sizeof(fs_QuotaBlk_t));
			if (quoList == NULL)
				break;
		}
		else
		{
			fs_QuotaBlk_t *p;

			p = realloc(quoList,
					(sindex+rsp->numberResponses) * sizeof(fs_QuotaBlk_t));
			if (p)
				quoList = p;
			else
				break;
		}

		for (i = 0; i < rsp->numberResponses; i ++)
		{
			int j = i + sindex;
			quoList[j].id = rsp->responses[i].id;
			quoList[j].inodeSoftLimit = rsp->responses[i].inodeSoftLimit;
			quoList[j].inodeHardLimit = rsp->responses[i].inodeHardLimit;
			quoList[j].inodeTimeLimit = rsp->responses[i].inodeTimeLimit;
			quoList[j].blockSoftLimit = rsp->responses[i].blockSoftLimit;
			quoList[j].blockHardLimit = rsp->responses[i].blockHardLimit;
			quoList[j].blockTimeLimit = rsp->responses[i].blockTimeLimit;
		}

		sindex += rsp->numberResponses;
	}

	return sindex;
}

static int getTableRowData(int *index, fs_QuotaBlk_t *data)
{
	if (*index < 1 || *index > dataCount)
		return -1;
	if (quoList == NULL)
		return -1;

	memcpy(data, &quoList[(*index)-1], sizeof(fs_QuotaBlk_t));
	return 0;
}


/***********************************************************
 *  Main routines to process tags
 **********************************************************/

static int getTagValue(char *tag, char *value)
{
	int i, tagType;
	
	tagType = INVALID_TAG;
	for (i = 0; i < NUM_OF_TAGS; i++) {
		if (!strcmp(tag, tagTable[i].tagName)) {
			tagType = tagTable[i].tagType;
			break;
		}
	}
	if (tagType == INVALID_TAG)
		return -1;

	/* first for those tags don't need to have fset data */
	switch (tagType) 
	{
	case FS_VOLNAME:
		strcpy(value, inName);
		return 1;
	case FS_PATH:
		strcpy(value, inPath);
		return 1;
	case FS_TYPE:
		strcpy(value, inType);
		return 1;
	case FS_TYPESTR:
		if (strcmp(inType, "u") == 0)
			strcpy(value, "User");
		else
			strcpy(value, "Group");
		return 1;
	}

	/* for those tags required to have data */
	if ((dataCurr <= 0) || (dataCurr > dataCount))
		return -1;

	switch (tagType) 
	{
	case FS_VOLID:
		sprintf(value, "%lld", inId);
		break;
	case FS_GUID:
		sprintf(value, "%lld", stData.id);
		break;
	case FS_ISOFT:
		sprintf(value, "%lld", stData.inodeSoftLimit);
		break;
	case FS_IHARD:
		sprintf(value, "%lld", stData.inodeHardLimit);
		break;
	case FS_ITIME:
		sprintf(value, "%d", stData.inodeTimeLimit);
		break;
	case FS_BSOFT:
		sprintf(value, "%lld", stData.blockSoftLimit);
		break;
	case FS_BHARD:
		sprintf(value, "%lld", stData.blockHardLimit);
		break;
	case FS_BTIME:
		sprintf(value, "%d", stData.blockTimeLimit);
		break;
	default:
		return -1;
	}

	return 1;
}


/*
 * actRptTagFunc()
 *	tag   - tag name
 *	value - replacement string for tag
 */
static int actRptTagFunc(tagCallbackParams_t *tp)
{
char *tag = tp->tag;
char *value = tp->value;

	if (!strcmp(tag, "TABLE_BEGIN")) {
		/* TABLE_BEGIN tag, need to see if there is any data */
		dataCount = getTableRowCount();
		if (dataCount > 0) {
			/* need to retrieve first table's data */
			dataCurr = 1;
			if (getTableRowData(&dataCurr, &stData) >= 0)
				return 1;
		}

		dataCurr = 0;
		return -1;
	}
	else if (!strcmp(tag, "TABLE_END")) {
		/* TABLE_END tag, check if end of data */
		if ((dataCount <= 0) || (dataCurr <= 0) || (dataCurr >= dataCount))
			return -1;

		dataCurr++;
		if (getTableRowData(&dataCurr, &stData) < 0)
			return -1;
		else {
			return 1;
		}
	}
	else {
		/* other tags */
		return getTagValue(tag, value);
	}

	/* shouldn't reach here... */
	return -1;
}

int actionFsetUGQuo(void *r, int count, char **argname, char **argval)
{
	char newfname[128];

	newfname[0] = '\0';

	/* handle Modify Quota actions */
	HandleFsAction(newfname, count, argname, argval);

	prepare_agile_template(r, newfname);

	/* initialize fileset information variables */
	dataCount = 0;
	dataCurr = 0;
	quoList = NULL;

	agile_scan_and_send(r, actRptTagFunc);

	if (quoList != NULL)
	{
		free(quoList);
		quoList = NULL;
	}
	return 1;
}
