/*  File Name  :    agileifdel.c
 *
 *  Description:    HTTPD action functions for delete ip from interface.
 *
 *  Created    :    2/5/2002
 *
 *  Author     :    Tiffany Chiang
 *
 *  Copyright(c)  Agile Storage, Inc. 2001
 *
 */

#include <sys/types.h>
#include <unistd.h>
#include <anpssc-api.h>

#include <ipmd-lib.h>

#include "agilepost.h"
#include "agileutil.h"


#define INVALID_TAG			0
#define NWIF_NAME			1
#define NWIF_TYPE			2
#define NWIF_SLOT			3
#define NWIF_LINK			4
#define NWIF_IP				5
#define NUM_OF_TAGS			5

static tagNameTable_t tagTable[NUM_OF_TAGS] = {
	{NWIF_NAME,		"args.if_name"},
	{NWIF_IP,		"args.if_ip"},
	{NWIF_TYPE,		"args.if_type"},
	{NWIF_SLOT,		"args.if_slot"},
	{NWIF_LINK,		"args.if_link"}
};


#define PARAM_NAME			1
#define PARAM_TYPE			2
#define PARAM_SLOT			3
#define PARAM_LINK			4
#define PARAM_IP			5
#define NUM_OF_PARAMS		5

static paramNameTable_t paramTable[NUM_OF_PARAMS] = {
	{PARAM_NAME,	"name"},
	{PARAM_IP,		"ip"},
	{PARAM_TYPE,	"type"},
	{PARAM_SLOT,	"slot"},
	{PARAM_LINK,	"link"},
};


/* HERE ARE VARIABLES FOR NETWORK INFORMATION */
static char if_name[IPM_STR_SIZE];	/* interface name */
static char if_ip[20];
static char if_type[10];
static char if_slot[10];
static char if_link[10];


/***********************************************************
 * routines to retrieve network information 
 **********************************************************/

static int getInputParams(int count, char **argname, char **argval)
{
	int argind, paramType;

	for (argind = 0; argind < count; argind ++)
	{
		paramType = GetParamType(argname[argind], paramTable, NUM_OF_PARAMS);
		if (paramType == INVALID_PARAM)
			continue;

		switch (paramType) 
		{
		case PARAM_TYPE:
			strcpy(if_type, argval[argind]);
			break;
		case PARAM_SLOT:
			strcpy(if_slot, argval[argind]);
			break;
		case PARAM_LINK:
			strcpy(if_link, argval[argind]);
			break;
		case PARAM_IP:
			strcpy(if_ip, argval[argind]);
			break;
		}
	}

	GetInterfaceNameStr(if_name, if_type, if_slot, if_link);
	return 0;
}


/***********************************************************
 *  Main routines to process tags
 **********************************************************/

static int getTagValue(char *tag, char *value)
{
	int i, tagType;
	
	tagType = INVALID_TAG;
	for (i = 0; i < NUM_OF_TAGS; i++) {
		if (!strcmp(tag, tagTable[i].tagName)) {
			tagType = tagTable[i].tagType;
			break;
		}
	}
	if (tagType == INVALID_TAG)
		return -1;

	switch (tagType) 
	{
	case NWIF_NAME:
		strcpy(value, if_name);
		break;
	case NWIF_SLOT:
		strcpy(value, if_slot);
		break;
	case NWIF_LINK:
		strcpy(value, if_link);
		break;
	case NWIF_TYPE:
		strcpy(value, if_type);
		break;
	case NWIF_IP:
		strcpy(value, if_ip);
		break;
	default:
		return -1;
	}

	return 1;
}


/*
 * actRptTagFunc()
 *	tag   - tag name
 *	value - replacement string for tag
 */
static int actRptTagFunc(tagCallbackParams_t *tp)
{
char *tag = tp->tag;
char *value = tp->value;

	if (!strcmp(tag, "TABLE_BEGIN")) {
		return -1;
	}
	else if (!strcmp(tag, "TABLE_END")) {
		return -1;
	}
	else {
		return getTagValue(tag, value);
	}

	/* shouldn't reach here... */
	return -1;
}

int actionInterfaceDel(void *r, int count, char **argname, char **argval)
{
	/* initialize network information variables */
	getInputParams(count, argname, argval);

	prepare_agile_template(r, NULL);

	agile_scan_and_send(r, actRptTagFunc);

	return 1;
}
