/*  File Name  :    agileifmain.c
 *
 *  Description:    HTTPD action functions for network interface main page.
 *
 *  Created    :    3/3/2002
 *
 *  Author     :    Tiffany Chiang
 *
 *  Copyright(c)  Agile Storage, Inc. 2001
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <unistd.h>
#include <anpssc-api.h>

#include <sys/socket.h>
#include <net/if.h>
#include <netinet/in.h>
#undef MIN
#undef MAX
#include <arpa/inet.h>
#include <ipmd-lib.h>

#include "agilepost.h"
#include "agileutil.h"


#define INVALID_TAG			0
#define NWIF_NAME			1
#define NWIF_TYPE			2
#define NWIF_SLOT			3
#define NWIF_LINK			4
#define NWIF_POS			5
#define NWIF_STATUS			6
#define NWIF_IPLIST			7
#define NWIF_MASKLIST		8
#define NWIF_DELIPLIST		9
#define NWIF_MODIPSTATUS	10
#define NWIF_PRIVCOL		11
#define NUM_OF_TAGS			11

#define NUM_IF_ENTRIES		20
#define NUM_IP_ENTRIES		20

static tagNameTable_t tagTable[NUM_OF_TAGS] = {
	{NWIF_NAME,			"ipm_if_t.if_name"},
	{NWIF_TYPE,			"ipm_if_t.if_name.type"},
	{NWIF_SLOT,			"ipm_if_t.if_slot"},
	{NWIF_LINK,			"ipm_if_t.if_link"},
	{NWIF_POS,			"ipm_if_t.if_position"},
	{NWIF_STATUS,		"ipm_if_t.if_status"},
	{NWIF_IPLIST,		"ipm_ip_t.ip_addr.all"},
	{NWIF_MASKLIST,		"ipm_ip_t.ip_mask.all"},
	{NWIF_DELIPLIST,	"ipm_ip_t.del_ip.all"},
	{NWIF_MODIPSTATUS,	"ipm_ip_t.mod_ip.status"},
	{NWIF_PRIVCOL,		"priv.colspan"}
};


#define PARAM_ACT			1
#define PARAM_TYPE			2
#define PARAM_SLOT			3
#define PARAM_LINK			4
#define PARAM_IP			5
#define PARAM_MASK			6
#define PARAM_STATUS		7
#define NUM_OF_PARAMS		7

static paramNameTable_t paramTable[NUM_OF_PARAMS] = {
	{PARAM_ACT,		"act"},
	{PARAM_TYPE,	"type"},
	{PARAM_SLOT,	"slot"},
	{PARAM_LINK,	"link"},
	{PARAM_IP,		"ip"},
	{PARAM_MASK,	"mask"},
	{PARAM_STATUS,	"status"}
};

static char *strUp   = "Up";
static char *strDown = "Down";

static char bufact[5];
static char buftype[10];
static char bufslot[5];
static char buflink[5];
static char bufip[20];
static char bufmask[20];
static char bufstatus[10];


/* HERE ARE VARIABLES FOR INTERFACE INFORMATION */
static int dataCount;
static int dataCurr;
static int ipCount;
static ipm_if_t *stIfTable;
static ipm_ip_t *stIpTable;
static ipm_if_t stData;


/***********************************************************
 * routines to retrieve network information 
 **********************************************************/

static int getTableRowCount(void)
{
	int32	res;
	int count, need, i;

	/* Try to get IF Table with maxi. 20 entries */
	stIfTable = (ipm_if_t *) malloc(NUM_IF_ENTRIES*sizeof(ipm_if_t));
	if (stIfTable == NULL)
		return -1;

	res = ipmd_GetIfByName(NULL, stIfTable, NUM_IF_ENTRIES*sizeof(ipm_if_t), 
							&count, &need);
	if (res == IPM_GET_ERR)
		return -1;
	else if ((res == IPM_GET_NO_BUF) || (res == IPM_CMDERR_APPNOBUF))
	{
		/* Need a larger buffer */
		free(stIfTable);
		stIfTable = malloc(need);
		if (stIfTable == NULL)
			return -1;

		res = ipmd_GetIfByName(NULL, stIfTable, need, &count, &need);
		if (res == IPM_GET_ERR)
			return -1;
		else if ((res == IPM_GET_NO_BUF) || (res == IPM_CMDERR_APPNOBUF))
			return -1;
	}

	need = 0;
	for (i = 0; i < count; i++)
	{
	int keepinterface = 0;

		if (!strncmp(stIfTable[i].if_name, "sc", 2)) {
			if ((stIfTable[i].if_name[2] == '1') ||
			 	(stIfTable[i].if_name[2] == '2')) {
				keepinterface = 1;
			}
		}
		else if (!strncmp(stIfTable[i].if_name, "fp", 2)) {
			keepinterface = 1;
		}

		if (keepinterface) {
			if (i != need)
				memcpy(&stIfTable[need], &stIfTable[i], sizeof(ipm_if_t));
			need++;
		}
	}

	return need;

#if 0
	if (count < 0)
		return -1;

	return count;
#endif
}

static int getTableRowData(int *index, ipm_if_t *data)
{
	int32	res;
	int count, need;

	if (*index < 1 || *index > dataCount)
		return -1;
	if (stIfTable == NULL)
		return -1;

	memcpy(data, &stIfTable[(*index)-1], sizeof(ipm_if_t));

	/* get IP Table data */
	ipCount = 0;
	if (stIpTable != NULL)
		free(stIpTable);
	stIpTable = (ipm_ip_t *) malloc(NUM_IP_ENTRIES*sizeof(ipm_ip_t));
	if (stIpTable == NULL)
		return 0;
	res = ipmd_GetIpByName(stIfTable[(*index)-1].if_name, stIpTable, 
							NUM_IP_ENTRIES*sizeof(ipm_ip_t), &count, &need);
	if (res == IPM_GET_ERR)
		return 0;
	else if ((res == IPM_GET_NO_BUF) || (res == IPM_CMDERR_APPNOBUF))
	{
		/* Need a larger buffer */
		free(stIpTable);
		stIpTable = (ipm_ip_t *) malloc(need);
		if (stIpTable == NULL)
			return 0;

		res = ipmd_GetIpByName(stIfTable[(*index)-1].if_name, stIpTable, 
							need, &count, &need);
		if (res == IPM_GET_ERR)
			return 0;
		else if ((res == IPM_GET_NO_BUF) || (res == IPM_CMDERR_APPNOBUF))
			return 0;
	}

	ipCount = count;
	return 0;
}


/***********************************************************
 *  Main routines to handle Addif, Delif, Modif actions
 **********************************************************/

static int getInputParams(int count, char **argname, char **argval)
{
	int argind, paramType;

	for (argind = 0; argind < count; argind ++)
	{
		paramType = GetParamType(argname[argind], paramTable, NUM_OF_PARAMS);
		if (paramType == INVALID_PARAM)
			continue;

		switch (paramType) 
		{
		case PARAM_ACT:
			strcpy(bufact, argval[argind]);
			break;
		case PARAM_TYPE:
			strcpy(buftype, argval[argind]);
			break;
		case PARAM_SLOT:
			strcpy(bufslot, argval[argind]);
			break;
		case PARAM_LINK:
			strcpy(buflink, argval[argind]);
			break;
		case PARAM_IP:
			strcpy(bufip, argval[argind]);
			break;
		case PARAM_MASK:
			strcpy(bufmask, argval[argind]);
			break;
		case PARAM_STATUS:
			strcpy(bufstatus, argval[argind]);
			break;
		}
	}

	return 0;
}

static int HandleIfAction(char *newfname, int count, char **argname, char **argval)
{
	char ifname[IPM_STR_SIZE];
	int32 res;
	struct in_addr ipaddr, ipmask;

	getInputParams(count, argname, argval);
	GetInterfaceNameStr(ifname, buftype, bufslot, buflink);

	if (!strcmp(bufact, strAdd))
	{
		inet_aton(bufip, &ipaddr);
		inet_aton(bufmask, &ipmask);
		res = ipmd_AddIp(ifname,
				(uint32) ipaddr.s_addr, (uint32) ipmask.s_addr, NULL, 0);
		if (res == NFX_ERR)
			strcpy(newfname, "error_addif.html");
	}
	else if (!strcmp(bufact, strDel))
	{
		inet_aton(bufip, &ipaddr);
		res = ipmd_DelIp(ifname, (uint32) ipaddr.s_addr, NULL, 0);
		if (res == NFX_ERR)
			strcpy(newfname, "error_delif.html");
	}
	else if (!strcmp(bufact, strMod))
	{
		if (!strcmp(bufstatus, strUp))
		{
			res = ipmd_EnableIntf(ifname, NULL, 0);
			if (res == NFX_ERR)
				strcpy(newfname, "error_modif.html");
		}
		else
		{
			res = ipmd_DisableIntf(ifname, NULL, 0);
			if (res == NFX_ERR)
				strcpy(newfname, "error_modif.html");
		}
	}
	return 1;
}


/***********************************************************
 *  Main routines to process tags
 **********************************************************/

static int getTagValue(char *tag, char *value)
{
	int i, tagType, pri;
	char tempbuf[24], typestr[10], slotstr[10], linkstr[10];
	
	if ((dataCurr <= 0) || (dataCurr > dataCount))
		return -1;

	tagType = INVALID_TAG;
	for (i = 0; i < NUM_OF_TAGS; i++) {
		if (!strcmp(tag, tagTable[i].tagName)) {
			tagType = tagTable[i].tagType;
			break;
		}
	}
	if (tagType == INVALID_TAG)
		return -1;

	switch (tagType) 
	{
	case NWIF_NAME:
		strcpy(value, stData.if_name);
		break;
	case NWIF_SLOT:
		GetInterfaceSlotStr(stData.if_name, value);
		break;
	case NWIF_LINK:
		GetInterfaceLinkStr(stData.if_name, value);
		break;
	case NWIF_TYPE:
		GetInterfaceTypeStr(stData.if_name, value);
		break;
	case NWIF_STATUS:
		if ((stData.if_flags & IFF_UP) != 0)
			strcpy(value, strUp);
		else
			strcpy(value, strDown);
		break;
#if 0
	case NWIF_NOT_STATUS:
		if ((stData.if_flags & IFF_UP) != 0)
			strcpy(value, strDown);
		else
			strcpy(value, strUp);
		break;
#endif
	case NWIF_IPLIST:
		if (ipCount <= 0)
		{
			strcpy(value, "<tr><td class=\"tabledata\" nowrap>&nbsp;</td></tr>\n");
		}
		else
		{
			strcpy(value, "");
			for(i = 0; i < ipCount; i ++)
			{
				strcat(value, "<tr><td class=\"tabledata\" nowrap> ");
				GetIpAddressStr(stIpTable[i].ip_addr, tempbuf);
				strcat(value, tempbuf);
				strcat(value, " </td></tr>\n");
			}
		}
		break;
	case NWIF_MASKLIST:
		if (ipCount <= 0)
		{
			strcpy(value, "<tr><td class=\"tabledata\" nowrap>&nbsp;</td></tr>\n");
		}
		else
		{
			strcpy(value, "");
			for(i = 0; i < ipCount; i ++)
			{
				strcat(value, "<tr><td class=\"tabledata\" nowrap> ");
				GetIpAddressStr(stIpTable[i].ip_mask, tempbuf);
				strcat(value, tempbuf);
				strcat(value, " </td></tr>\n");
			}
		}
		break;
	case NWIF_DELIPLIST:
		if (ipCount <= 0)
		{
			strcpy(value, "<tr><td class=\"tabledata\" nowrap>&nbsp;</td></tr>\n");
		}
		else
		{
			memset(typestr, 0, 10);
			memset(slotstr, 0, 10);
			memset(linkstr, 0, 10);
			GetInterfaceStrs(stData.if_name, typestr, slotstr, linkstr);
			strcpy(value, "");
			for(i = 0; i < ipCount; i ++)
			{
				GetIpAddressStr(stIpTable[i].ip_addr, tempbuf);
				strcat(value, "<tr><td class=\"tabledata\" nowrap> <a href=\"/agilerequestprefix/InterfaceDel?type=");
				strcat(value, typestr);
				strcat(value, "&slot=");
				strcat(value, slotstr);
				strcat(value, "&link=");
				strcat(value, linkstr);
				strcat(value, "&ip=");
				strcat(value, tempbuf);
				strcat(value, "\">Delete</a></td></tr>\n");
			}
		}
		break;
	case NWIF_MODIPSTATUS:
		pri = GetUserPrivilege();
		/* Current status - up, modify status means bring it down - Priority FULL cmd */
		if ((stData.if_flags & IFF_UP) != 0)
		{
			if (pri < PRIV_FULL)
			{
				strcpy(value, "");
				return 1;
			}
		}
		memset(typestr, 0, 10);
		memset(slotstr, 0, 10);
		memset(linkstr, 0, 10);
		GetInterfaceStrs(stData.if_name, typestr, slotstr, linkstr);

		/* no enable or disable interface for ssc interfaces */
		if (!strcmp(typestr, "sc"))
		{
			strcpy(value, "&nbsp</a>");
			break;
		}

		strcpy(value, "<a href=\"/agilerequestprefix/InterfaceMod?type=");
		strcat(value, typestr);
		strcat(value, "&slot=");
		strcat(value, slotstr);
		strcat(value, "&link=");
		strcat(value, linkstr);
		strcat(value, "&status=");
		/* Current status - up, modify status means bring it down - Priority FULL cmd */
		if ((stData.if_flags & IFF_UP) != 0)
		{
			strcat(value, strDown);
			strcat(value, "\">");
			strcat(value, strDown);
			strcat(value, "</a>");
		}
		/* Current status - down, modify status means bring it up - Priority ADD cmd */
		else
		{
			strcat(value, strUp);
			strcat(value, "\">");
			strcat(value, strUp);
			strcat(value, "</a>");
		}
		break;
	case NWIF_POS:
		sprintf(value, "%d/%d", stData.if_slot, stData.if_link);
#if 0
		sprintf(value, "%d/%d/%d", stData.if_slot, stData.if_cpu, stData.if_link);
#endif
		break;
	case NWIF_PRIVCOL:
		pri = GetUserPrivilege();
		// colspan=2 - Add, Up
		if ((pri > PRIV_VIEW) && (pri < PRIV_FULL))
			strcpy(value, "2");
		// colspan=3 - Delete, Add, Up/Down
		else if (pri >= PRIV_FULL)
			strcpy(value, "3");
		break;
	default:
		return -1;
	}

	return 1;
}


/*
 * actRptTagFunc()
 *	tag   - tag name
 *	value - replacement string for tag
 */
static int actRptTagFunc(tagCallbackParams_t *tp)
{
char *tag = tp->tag;
char *value = tp->value;

	if (!strcmp(tag, "TABLE_BEGIN")) {
		/* TABLE_BEGIN tag, need to see if there is any data */
		dataCount = getTableRowCount();
		if (dataCount > 0) {
			/* need to retrieve first table's data */
			dataCurr = 1;
			if (getTableRowData(&dataCurr, &stData) >= 0)
				return 1;
		}

		dataCurr = 0;
		return -1;
	}
	else if (!strcmp(tag, "TABLE_END")) {
		/* TABLE_END tag, check if end of data */
		if ((dataCount <= 0) || (dataCurr <= 0) || (dataCurr >= dataCount))
			return -1;

		dataCurr++;
		if (getTableRowData(&dataCurr, &stData) < 0)
			return -1;
		else {
			return 1;
		}
	}
	else {
		/* other tags */
		if (dataCount <= 0) {
			dataCount = getTableRowCount();
			if (dataCount <= 0)
				return -1;
		}

		if ((dataCurr < 1) || (dataCurr > dataCount)) {
			dataCurr = 1;
			if (getTableRowData(&dataCurr, &stData) < 0) {
				return -1;
			}
		}

		return getTagValue(tag, value);
	}

	/* shouldn't reach here... */
	return -1;
}


int actionInterfaceMain(void *r, int count, char **argname, char **argval)
{
char newfname[128];

	memset(newfname, 0, sizeof(newfname));

	/* Handle Addif, Delif, Modif actions */
	if (count > 0)
		HandleIfAction(newfname, count, argname, argval);

	prepare_agile_template(r, newfname);

	/* initialize storage information variables */
	dataCount = 0;
	dataCurr = 0;
	ipCount = 0;
	stIfTable = NULL;
	stIpTable = NULL;

	agile_scan_and_send(r, actRptTagFunc);

	if (stIfTable != NULL)
		free(stIfTable);
	if (stIpTable != NULL)
		free(stIpTable);
	return 1;
}
