/*  File Name  :    agilelunreset.c
 *
 *  Description:    HTTPD action functions for reset lun.
 *
 *  Created    :    2/6/2002
 *
 *  Author     :    Tiffany Chiang
 *
 *  Copyright(c)  Agile Storage, Inc. 2001
 *
 */

#include <stdio.h>
#include <sys/types.h>
#include <unistd.h>
#undef MIN
#undef MAX
#include <anpssc-api.h>

#include "agilepost.h"
#include "agileutil.h"


#define INVALID_TAG			0
#define STG_WWNSTR			1
#define STG_WWN				2
#define STG_LUN				3
#define STG_TYPE			4
#define STG_RAID			5
#define STG_LINK			6
#define NUM_OF_TAGS			6

static tagNameTable_t tagTable[NUM_OF_TAGS] = {
	{STG_WWNSTR,	"args.wwnstr"},
	{STG_WWN,		"args.wwn"},
	{STG_LUN,		"args.lun"},
	{STG_TYPE,		"args.type"},
	{STG_RAID,		"args.rl"},
	{STG_LINK,		"nextpage"},
};


#define PARAM_WWN			1
#define PARAM_LUN			2
#define PARAM_RAID			3
#define PARAM_TYPE			4
#define NUM_OF_PARAMS		4

static paramNameTable_t paramTable[NUM_OF_PARAMS] = {
	{PARAM_WWN,		"wwn"},
	{PARAM_LUN,		"lun"},
	{PARAM_RAID,	"rl"},
	{PARAM_TYPE,	"type"},
};

/* HERE ARE VARIABLES FOR NETWORK INFORMATION */
static uint64 inWwn;
static uint32 inLun;
static int inRaid;
static char inType[10];


/***********************************************************
 * routines to retrieve lun information 
 **********************************************************/

static int getInputParams(int count, char **argname, char **argval)
{
	int argind, paramType;

	for (argind = 0; argind < count; argind ++)
	{
		paramType = GetParamType(argname[argind], paramTable, NUM_OF_PARAMS);
		if (paramType == INVALID_PARAM)
			continue;

		switch (paramType) 
		{
		case PARAM_WWN:
			sscanf(argval[argind], "%lld", &inWwn);
			break;
		case PARAM_LUN:
			sscanf(argval[argind], "%d", &inLun);
			break;
		case PARAM_RAID:
			sscanf(argval[argind], "%d", &inRaid);
			break;
		case PARAM_TYPE:
			strcpy(inType, argval[argind]);
			break;
		}
	}

	return 0;
}


/***********************************************************
 *  Main routines to process tags
 **********************************************************/

static int getTagValue(char *tag, char *value)
{
	int i, tagType;
	
	tagType = INVALID_TAG;
	for (i = 0; i < NUM_OF_TAGS; i++) {
		if (!strcmp(tag, tagTable[i].tagName)) {
			tagType = tagTable[i].tagType;
			break;
		}
	}
	if (tagType == INVALID_TAG)
		return -1;

	switch (tagType) 
	{
	case STG_WWNSTR:
		GetWwnStr(inWwn, value);
		break;
	case STG_WWN:
		sprintf(value, "%lld", inWwn);
		break;
	case STG_LUN:
		sprintf(value, "%d", inLun);
		break;
	case STG_TYPE:
		strcpy(value, inType);
		break;
	case STG_RAID:
		sprintf(value, "%d", inRaid);
		break;
	case STG_LINK:
		if (!strcmp(inType, "u") || !strcmp(inType, "f"))
			strcpy(value, "DiskLuns");
		else
			strcpy(value, "DiskFOLuns");
		break;
	default:
		return -1;
	}

	return 1;
}


/*
 * actRptTagFunc()
 *	tag   - tag name
 *	value - replacement string for tag
 */
static int actRptTagFunc(tagCallbackParams_t *tp)
{
char *tag = tp->tag;
char *value = tp->value;

	if (!strcmp(tag, "TABLE_BEGIN")) {
		return -1;
	}
	else if (!strcmp(tag, "TABLE_END")) {
		return -1;
	}
	else {
		return getTagValue(tag, value);
	}

	/* shouldn't reach here... */
	return -1;
}

int actionLunReset(void *r, int count, char **argname, char **argval)
{
	/* initialize lun information variables */
	getInputParams(count, argname, argval);

	prepare_agile_template(r, NULL);

	agile_scan_and_send(r, actRptTagFunc);

	return 1;
}
