/*  File Name  :    agilendmpasess.c
 *
 *  Description:    HTTPD action functions for ndmp main page (active sessions).
 *
 *  Created    :    4/3/2002
 *
 *  Author     :    Tiffany Chiang
 *
 *  Copyright(c)  Agile Storage, Inc. 2001
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <sys/types.h>
#include <unistd.h>

#include <anpssc-api.h>

#undef MIN
#undef MAX
#include <sys/socket.h>
#include <net/if.h>
#include <netinet/in.h>
#include <arpa/inet.h>

#include <ndmp-ui-api.h>

#include "agilepost.h"
#include "agileutil.h"


#define INVALID_TAG			0
#define NDMP_STATUS			1
#define NDMP_STATUSCHG		2
#define NDMP_STATUSBTN		3
#define NDMP_SESSID			4
#define NDMP_HOST			5
#define NDMP_HOSTSTR		6
#define NDMP_APPNAME		7
#define NUM_OF_TAGS			7

static tagNameTable_t tagTable[NUM_OF_TAGS] = {
	{NDMP_STATUS,		"ndmpentry.statusstr"},
	{NDMP_STATUSCHG,	"ndmpentry.status.updatepage"},
	{NDMP_STATUSBTN,	"ndmpentry.status.notbtn"},
	{NDMP_SESSID,		"ndmpsession.session_id"},
	{NDMP_HOST,			"ndmpsession.host_addr"},
	{NDMP_HOSTSTR,		"ndmpsession.host_str"},
	{NDMP_APPNAME,		"ndmpsession.app_name"},
};


#define PARAM_ACT			1
#define PARAM_SESSID		2
#define NUM_OF_PARAMS		2

static paramNameTable_t paramTable[NUM_OF_PARAMS] = {
	{PARAM_ACT,		"act"},
	{PARAM_SESSID,	"sid"},
};

static char bufact[10];
static int inId;


/* HERE ARE VARIABLES FOR NDMP INFORMATION */
typedef struct _export_entry_t
{
    int			id;
    uint32		host;
    char		app[80];
} actsess_entry_t;

static int ndmpStatus;
static int dataCount;
static int dataCurr;
static actsess_entry_t *stActSessTable;
static actsess_entry_t stData;


/***********************************************************
 * routines to retrieve ndmp information 
 **********************************************************/

static int getInputParams(int count, char **argname, char **argval)
{
	int argind, paramType;

	for (argind = 0; argind < count; argind ++)
	{
		paramType = GetParamType(argname[argind], paramTable, NUM_OF_PARAMS);
		if (paramType == INVALID_PARAM)
			continue;

		switch (paramType) 
		{
		case PARAM_ACT:
			strcpy(bufact, argval[argind]);
			break;
		case PARAM_SESSID:
			sscanf(argval[argind], "%d", &inId);
			break;
		}
	}

	return 0;
}

static int getSessionTable(ndmp_NFXGetTblRsp_t* rsp)
{
    ndmp_NFXGenericUIReq_t 	req;
	int32					rc;
    struct timeval			to;

    req.mid = NDMP_MID_UI_GET_SESS_TBL;
	memset(rsp, 0, sizeof(ndmp_NFXGetTblRsp_t));
    to.tv_sec  = 5;
    to.tv_usec = 0;

    /* Send to parent server to get table of sessions (children NDMP servers) */
    rc = sendAgileMsg(-1, &req, sizeof(ndmp_NFXGenericUIReq_t),
                        "ssc_ndmp", NFX_SSC_UNIX, SSC_SLOT, 0, &to,
                        rsp, sizeof(ndmp_NFXGetTblRsp_t)); 
    return rc;
}

static int getTableRowCount(void)
{
    ndmp_NFXGetTblRsp_t 	rsp;
	int32					rc;
	int						count, i, index;

	rc = getSessionTable(&rsp);
		return -1;

	/* Find number of active sessions */
    for (i = 0, count = 0; i < NDMP_MAX_SESS; i ++)
	{
		if (rsp.tbl[i].in_use == TRUE)
			count ++;
	}

	/* Copy to our table */
	stActSessTable = malloc(count * sizeof(actsess_entry_t));
	if (stActSessTable == NULL)
		return -1;

    for (i = 0, index = 0; i < NDMP_MAX_SESS; i ++)
	{
		if (rsp.tbl[i].in_use == TRUE)
		{
			stActSessTable[index].id   = i;
			stActSessTable[index].host = rsp.tbl[i].from.sin_addr.s_addr;
			strcpy(stActSessTable[index].app, rsp.tbl[i].app_name);
			index ++;
		}
	}
	
	return count;
}

static int getTableRowData(int *index, actsess_entry_t *data)
{
	if (*index < 1 || *index > dataCount)
		return -1;
	if (stActSessTable == NULL)
		return -1;

	memcpy(data, &stActSessTable[(*index)-1], sizeof(actsess_entry_t));
	return 0;
}

static void GetNdmpStatus()
{
	ndmp_NFXGenericUIReq_t	req;
	ndmp_NFXDisEnaRsp_t		rsp;
	int32					rc;
    struct timeval			to = {5, 0};

	ndmpStatus = -1;
    req.mid = NDMP_MID_UI_SHOW_STATE;
	memset(&rsp, 0, sizeof(ndmp_NFXDisEnaRsp_t));

    /* Send to parent server */
    rc = sendAgileMsg(-1, &req, sizeof(ndmp_NFXGenericUIReq_t),
                        "ssc_ndmp", NFX_SSC_UNIX, SSC_SLOT, 0, &to,
                        &rsp, sizeof(ndmp_NFXDisEnaRsp_t)); 
	if (rc <= 0)
		return;

	if (rsp.state == NDMP_ON)
		ndmpStatus = 1;
	else
		ndmpStatus = 0;
}


/***********************************************************
 *  Main routines to handle Del/DellAll/
 *					EnableNDMP/DisableNDMP actions
 **********************************************************/

static int DoDelSess(int sessId)
{
	ndmp_NFXDelSessReq_t 	c_req;
	ndmp_NFXDelSessRsp_t	c_rsp;
    ndmp_NFXGetTblRsp_t 	rsp;
	int32					rc;
	int						i, res;
    struct timeval			to = {5, 0};

	rc = getSessionTable(&rsp);
		return -1;

    /* Send deletion request to child NDMP server */
    c_req.mid = NDMP_MID_UI_DEL_SESS;
    if (sessId != -1)
    {
        if (!rsp.tbl[sessId].in_use)
            return 1;
		else 
		{
            /* Query a child NDMP server */
            c_req.sessId = sessId;
		    rc = sendAgileMsg(-1, &c_req, sizeof(ndmp_NFXDelSessReq_t),
				                rsp.tbl[sessId].app_name,
								NFX_SSC_UNIX, SSC_SLOT, 0, &to,
						        &c_rsp, sizeof(ndmp_NFXDelSessRsp_t)); 

            if ((rc <= 0) || (c_rsp.stat == NFX_ERR))
				return -1;
			return 1;
        }
    }  
	else 
	{
		res = 1;
        for (i = 0; i < NDMP_MAX_SESS; i++)
        {
            if (rsp.tbl[i].in_use == TRUE)
            {
                /* Query all active NDMP children servers ourselves */
                c_req.sessId = i;
			    rc = sendAgileMsg(-1, &c_req, sizeof(ndmp_NFXDelSessReq_t),
					                rsp.tbl[sessId].app_name,
									NFX_SSC_UNIX, SSC_SLOT, 0, &to,
							        &c_rsp, sizeof(ndmp_NFXDelSessRsp_t)); 
                if ((rc <= 0) || (c_rsp.stat == NFX_ERR))
					res = -1;
            }
        }
		if (res != 1)
			return -1;
		return 1;
    }
	return -1;
}

static int DoEnableNdmp(int status)
{
	ndmp_NFXGenericUIReq_t	req;
	ndmp_NFXDisEnaRsp_t		rsp;
	int32					rc;
    struct timeval			to;

	req.mid = (status == 1) ? NDMP_MID_UI_ENABLE_NDMP : NDMP_MID_UI_DISABLE_NDMP;
	memset(&rsp, 0, sizeof(ndmp_NFXDisEnaRsp_t));
    to.tv_sec  = 5;
    to.tv_usec = 0;

    rc = sendAgileMsg(-1, &req, sizeof(ndmp_NFXGenericUIReq_t),
                        "ssc_ndmp", NFX_SSC_UNIX, SSC_SLOT, 0, &to,
                        &rsp, sizeof(ndmp_NFXDisEnaRsp_t)); 
    if (rc <= 0)
		return -1;
	return 1;
}

static int HandleNdmpAction(char *newfname, int count, char **argname, char **argval)
{
	int res = 1;
	getInputParams(count, argname, argval);

	// DelAll
	if (!strcmp(bufact, "delall"))
	{
		res = DoDelSess(-1);
		if (res <= 0)
			strcpy(newfname, "error_delallndmp.html");
	}
	// Del
	else if (!strcmp(bufact, "del"))
	{
		res = DoDelSess(inId);
		if (res <= 0)
			strcpy(newfname, "error_delndmp.html");
	}
	// Enable
	else if (!strcmp(bufact, "en"))
	{
		res = DoEnableNdmp(1);
		if (res <= 0)
			strcpy(newfname, "error_enndmp.html");
	}
	// Disable
	else if (!strcmp(bufact, "dis"))
	{
		res = DoEnableNdmp(0);
		if (res <= 0)
			strcpy(newfname, "error_disndmp.html");
	}
	return res;
}


/***********************************************************
 *  Main routines to process tags
 **********************************************************/

static int getTagValue(char *tag, char *value)
{
	int i, tagType, pri;
	
	tagType = INVALID_TAG;
	for (i = 0; i < NUM_OF_TAGS; i++) {
		if (!strcmp(tag, tagTable[i].tagName)) {
			tagType = tagTable[i].tagType;
			break;
		}
	}
	if (tagType == INVALID_TAG)
		return -1;

	pri = GetUserPrivilege();
	switch (tagType) 
	{
	case NDMP_STATUS:
		if (ndmpStatus == 1)
			strcpy(value, "Enabled");
		else if (ndmpStatus == 0)
			strcpy(value, "Disabled");
		else
			strcpy(value, "Unknown");
		return 1;
	case NDMP_STATUSCHG:
		if (ndmpStatus != 1 && ndmpStatus != 0)
			strcpy(value, "&nbsp;");
		else if (ndmpStatus == 0)
		{
			if (pri >= PRIV_ADD)
				strcpy(value, "<a href=\"/agilerequestprefix/NdmpEnable\">");
			else
				strcpy(value, "&nbsp;");
		}
		else
		{
			if (pri >= PRIV_FULL)
				strcpy(value, "<a href=\"/agilerequestprefix/NdmpDisable\">");
			else
				strcpy(value, "&nbsp;");
		}
		return 1;
	case NDMP_STATUSBTN:
		if (ndmpStatus != 1 && ndmpStatus != 0)
			strcpy(value, "&nbsp;");
		else if (ndmpStatus == 0)
		{
			if (pri >= PRIV_ADD)
				strcpy(value, "<img src=\"/AgileView/images/button_enndmp.gif\" width=\"120\" height=\"20\" border=\"0\"></a>");
			else
				strcpy(value, "&nbsp;");
		}
		else
		{
			if (pri >= PRIV_FULL)
				strcpy(value, "<img src=\"/AgileView/images/button_disndmp.gif\" width=\"120\" height=\"20\" border=\"0\"></a>");
			else
				strcpy(value, "&nbsp;");
		}
		return 1;
	}

	if ((dataCurr <= 0) || (dataCurr > dataCount))
		return -1;

	switch (tagType) 
	{
	case NDMP_HOST:
		sprintf(value, "%u", stData.host);
		break;
	case NDMP_HOSTSTR:
		GetIpAddressStr(stData.host, value);
		break;
	case NDMP_SESSID:
		sprintf(value, "%d", stData.id);
		break;
	case NDMP_APPNAME:
		strcpy(value, stData.app);
		break;
	default:
		return -1;
	}

	return 1;
}


/*
 * actRptTagFunc()
 *	tag   - tag name
 *	value - replacement string for tag
 */
static int actRptTagFunc(tagCallbackParams_t *tp)
{
char *tag = tp->tag;
char *value = tp->value;

	if (!strcmp(tag, "TABLE_BEGIN")) {
		/* TABLE_BEGIN tag, need to see if there is any data */
		dataCount = getTableRowCount();
		if (dataCount > 0) {
			/* need to retrieve first table's data */
			dataCurr = 1;
			if (getTableRowData(&dataCurr, &stData) >= 0)
				return 1;
		}

		dataCurr = 0;
		return -1;
	}
	else if (!strcmp(tag, "TABLE_END")) {
		/* TABLE_END tag, check if end of data */
		if ((dataCount <= 0) || (dataCurr <= 0) || (dataCurr >= dataCount))
			return -1;

		dataCurr++;
		if (getTableRowData(&dataCurr, &stData) < 0)
			return -1;
		else {
			return 1;
		}
	}
	else {
		/* other tags */
		return getTagValue(tag, value);
	}

	/* shouldn't reach here... */
	return -1;
}


int actionNdmpActSess(void *r, int count, char **argname, char **argval)
{
	char newfname[128];

	memset(newfname, 0, sizeof(newfname));

	/* Handle Del/DellAll/EnableNDMP/DisableNDMP actions */
	if (count > 0)
		HandleNdmpAction(newfname, count, argname, argval);

	prepare_agile_template(r, newfname);

	/* initialize nfs information variables */
	dataCount = 0;
	dataCurr = 0;
	stActSessTable = NULL;
	GetNdmpStatus();

	agile_scan_and_send(r, actRptTagFunc);

	if (stActSessTable != NULL)
		free(stActSessTable);
	return 1;
}
