/*  File Name  :    agilendmpsessdetail.c 
 *
 *  Description:    HTTPD action functions for displaying ndmp active session details.
 *
 *  Created    :    4/4/2002
 *
 *  Author     :    Tiffany Chiang
 *
 *  Copyright(c)  Agile Storage, Inc. 2001
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <nfx-defs.h>
#include <nfx-types.h>
#include <eee-desc.h>
#include <anpssc-api.h>

#undef MIN
#undef MAX
#include <sys/socket.h>
#include <net/if.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <ndmp-ui-api.h>

#include "agilepost.h"
#include "agileutil.h"

#define INVALID_TAG			0
#define AS_SID				1
#define AS_AUTHORIZE		2
#define AS_VERSION			3
#define AS_AUTHTYPE			4
#define AS_CLNTADDR			5
#define AS_TAPESTATE		6
#define AS_TAPEALIAS		7
#define AS_TAPENAME			8
#define AS_TAPEMODE			9
#define AS_MCHSTATE			10
#define AS_MCHALIAS			11
#define AS_MCHNAME			12
#define AS_MOVSTATE			13
#define AS_MOVMODE			14
#define AS_MOVPAUR			15
#define AS_MOVHALR			16
#define AS_MOVRSIZE			17
#define AS_MOVRNUM			18
#define AS_MOVWRITE			19
#define AS_MOVSEEK			20
#define AS_MOVTOREAD		21
#define AS_MOVWDOFF			22
#define AS_MOVWDLEN			23
#define AS_MOVADDRTY		24
#define AS_DATASTATE		25
#define AS_DATAOP			26
#define AS_DATAHALR			27
#define AS_DATAPROC			28
#define AS_DATARDOFF		29
#define AS_DATARDLEN		30
#define AS_DATAADDRTY		31
#define AS_APPNAME			32
#define NUM_OF_TAGS			32

static tagNameTable_t tagTable[NUM_OF_TAGS] = {
	{AS_SID,		"args.sid"},
	{AS_APPNAME,	"args.app"},
	{AS_AUTHORIZE,	"sess_detail.authorize"},
	{AS_VERSION,	"sess_detail.version"},
	{AS_AUTHTYPE,	"sess_detail.authtype"},
	{AS_CLNTADDR,	"sess_detail.clientaddr"},
	{AS_TAPESTATE,	"sess_detail.tapestate"},
	{AS_TAPEALIAS,	"sess_detail.tapealias"},
	{AS_TAPENAME,	"sess_detail.tapephyname"},
	{AS_TAPEMODE,	"sess_detail.tapemode"},
	{AS_MCHSTATE,	"sess_detail.mchstate"},
	{AS_MCHALIAS,	"sess_detail.mchalias"},
	{AS_MCHNAME,	"sess_detail.mchphyname"},
	{AS_MOVSTATE,	"sess_detail.moverstate"},
	{AS_MOVMODE,	"sess_detail.movermode"},
	{AS_MOVPAUR,	"sess_detail.moverpaur"},
	{AS_MOVHALR,	"sess_detail.moverhalr"},
	{AS_MOVRSIZE,	"sess_detail.moverrecsize"},
	{AS_MOVRNUM,	"sess_detail.moverrecnum"},
	{AS_MOVWRITE,	"sess_detail.moverwrite"},
	{AS_MOVSEEK,	"sess_detail.moverseek"},
	{AS_MOVTOREAD,	"sess_detail.movertoread"},
	{AS_MOVWDOFF,	"sess_detail.moverwoff"},
	{AS_MOVWDLEN,	"sess_detail.moverwlen"},
	{AS_MOVADDRTY,	"sess_detail.moveraddrt"},
	{AS_DATASTATE,	"sess_detail.datastate"},
	{AS_DATAOP,		"sess_detail.dataoper"},
	{AS_DATAHALR,	"sess_detail.datahalr"},
	{AS_DATAPROC,	"sess_detail.dataproc"},
	{AS_DATARDOFF,	"sess_detail.dataroff"},
	{AS_DATARDLEN,	"sess_detail.datarlen"},
	{AS_DATAADDRTY,	"sess_detail.dataaddrt"},
};


#define PARAM_SID			1
#define PARAM_APPNAME		2
#define NUM_OF_PARAMS		2

static paramNameTable_t paramTable[NUM_OF_PARAMS] = {
	{PARAM_SID,		"sid"},
	{PARAM_APPNAME,	"app"},
};


/* HERE ARE VARIABLES FOR NDMP DETAIL INFORMATION */
static int	inId;
static char	inApp[80];
static int	res;
static ndmp_Session_t stData;


/***********************************************************
 * routines to retrieve ndmp information thru anpssc
 **********************************************************/

static int getInputParams(int count, char **argname, char **argval)
{
	int argind, paramType;

	for (argind = 0; argind < count; argind ++)
	{
		paramType = GetParamType(argname[argind], paramTable, NUM_OF_PARAMS);
		if (paramType == INVALID_PARAM)
			continue;

		switch (paramType) 
		{
		case PARAM_SID:
			inId = atoi(argval[argind]);
			break;
		case PARAM_APPNAME:
			strcpy(inApp, argval[argind]);
			break;
		}
	}

	return 0;
}

static void GetSessionData()
{
	ndmp_NFXShowReq_t 		req;
	ndmp_NFXShowChildRsp_t	rsp;
    struct timeval			to;
	int32					rc;

	res = -1;
	memset(&rsp, 0, sizeof(ndmp_NFXShowChildRsp_t));
	req.mid = NDMP_MID_UI_SHOW_DETAIL;
	req.sessId = inId;
    to.tv_sec  = 5;
    to.tv_usec = 0;

	/* Send request to child NDMP server */
    rc = sendAgileMsg(-1, &req, sizeof(ndmp_NFXShowReq_t),
                        inApp, NFX_SSC_UNIX, SSC_SLOT, 0, &to,
                        &rsp, sizeof(ndmp_NFXShowChildRsp_t)); 
	if (rc <= 0)
		return;
	if (rsp.stat != NFX_OK)
		return;

	memcpy(&stData, &rsp.sess, sizeof(ndmp_Session_t));
	res = 1;
}


/***********************************************************
 *  Main routines to process tags
 **********************************************************/

static int getTagValue(char *tag, char *value)
{
	int i, tagType;
	
	if (res <= 0)
		return -1;

	tagType = INVALID_TAG;
	for (i = 0; i < NUM_OF_TAGS; i ++) {
		if (!strcmp(tag, tagTable[i].tagName)) {
			tagType = tagTable[i].tagType;
			break;
		}
	}
	if (tagType == INVALID_TAG)
		return -1;

	switch (tagType) 
	{
	case AS_SID:
		sprintf(value, "%d", inId);
		break;
	case AS_APPNAME:
		strcpy(value, inApp);
		break;
	case AS_AUTHORIZE:
		if (stData.authed)
			strcpy(value, "Yes");
		else
			strcpy(value, "No");
		break;
	case AS_VERSION:
		if (stData.protoVer != -1)
			sprintf(value, "%d", stData.protoVer);
		else
			strcpy(value, "0");
		break;
	case AS_AUTHTYPE:
		if (stData.authed)
		{
			if (stData.auth_typ == AUTHNONE)
				strcpy(value, "None");
			else if (stData.auth_typ == AUTHTXT)
				strcpy(value, "Text");
			else if (stData.auth_typ == AUTHMD5)
				strcpy(value, "MD5");
			else
				strcpy(value, "Invalid");
		}
		else
			strcpy(value, "None");
		break;
	case AS_CLNTADDR:
		value = inet_ntoa(stData.host_ipaddr.sin_addr);
		break;
	case AS_TAPESTATE:
		if (stData.Tape.hdl)
			strcpy(value, "Open");
		else
			strcpy(value, "Closed");
		break;
	case AS_TAPEALIAS:
		if (stData.Tape.hdl)
			strcpy(value, stData.Tape.devname);
		else
			strcpy(value, "N/A");
		break;
	case AS_TAPENAME:
		if (stData.Tape.hdl)
			strcpy(value, stData.Tape.phys_devname);
		else
			strcpy(value, "N/A");
		break;
	case AS_TAPEMODE:
		if (stData.Tape.mode == TP_MODE_RD_WR)
			strcpy(value, "Read-Write");
		else if (stData.Tape.mode == TP_MODE_RD_ONLY)
			strcpy(value, "Read-Only");
		else
			strcpy(value, "None");
		break;
	case AS_MCHSTATE:
		if (stData.SCSI.hdl)
			strcpy(value, "Open");
		else
			strcpy(value, "Closed");
		break;
	case AS_MCHALIAS:
		if (stData.SCSI.hdl)
			strcpy(value, stData.SCSI.devname);
		else
			strcpy(value, "N/A");
		break;
	case AS_MCHNAME:
		if (stData.SCSI.hdl)
			strcpy(value, stData.SCSI.phys_devname);
		else
			strcpy(value, "N/A");
		break;
	case AS_MOVSTATE:
		if (stData.Mover.state == MOVER_ST_IDLE)
			strcpy(value, "Idle");
		else if (stData.Mover.state == MOVER_ST_LISTEN)
			strcpy(value, "Listen");
		else if (stData.Mover.state == MOVER_ST_ACTIVE)
			strcpy(value, "Active");
		else if (stData.Mover.state == MOVER_ST_PAUSED)
			strcpy(value, "Paused");
		else
			strcpy(value, "Halted");
		break;
	case AS_MOVMODE:
		if (stData.Mover.mode == MOVER_ST_IDLE)
			strcpy(value, "N/A");
		else if (stData.Mover.mode == MVR_MODE_WRITE)
			strcpy(value, "Write");
		else if (stData.Mover.mode == MVR_MODE_READ)
			strcpy(value, "Read");
		else
			strcpy(value, "None");
		break;
	case AS_MOVPAUR:
		if (stData.Mover.pause_reason == MOVER_PAUSE_NA)
			strcpy(value, "N/A");
		else if (stData.Mover.pause_reason == MOVER_PAUSE_EOM)
			strcpy(value, "EOM");
		else if (stData.Mover.pause_reason == MOVER_PAUSE_EOF)
			strcpy(value, "EOF");
		else if (stData.Mover.pause_reason == MOVER_PAUSE_SEEK)
			strcpy(value, "Seek");
		else if (stData.Mover.pause_reason == MOVER_PAUSE_MEDIA_ERROR)
			strcpy(value, "Media Error");
		else
			strcpy(value, "EOW");
		break;
	case AS_MOVHALR:
		if (stData.Mover.halt_reason == MOVER_HALT_NA)
			strcpy(value, "N/A");
		else if (stData.Mover.halt_reason == MOVER_HALT_CONNECT_CLOSED)
			strcpy(value, "Connection Closed");
		else if (stData.Mover.halt_reason == MOVER_HALT_ABORTED)
			strcpy(value, "Aborted");
		else if (stData.Mover.halt_reason == MOVER_HALT_INTERNAL_ERROR)
			strcpy(value, "Internal Error");
		else
			strcpy(value, "Connection Error");
		break;
	case AS_MOVRSIZE:
		sprintf(value, "%d", stData.Mover.recsiz);
		break;
	case AS_MOVRNUM:
		sprintf(value, "%d", stData.Mover.recnum);
		break;
	case AS_MOVWRITE:
		sprintf(value, "%lld", stData.Mover.data_written);
		break;
	case AS_MOVSEEK:
		sprintf(value, "%lld", stData.Mover.seek_pos);
		break;
	case AS_MOVTOREAD:
		sprintf(value, "%lld", stData.Mover.bytes_left_to_read);
		break;
	case AS_MOVWDOFF:
		sprintf(value, "%lld", stData.Mover.window_offset);
		break;
	case AS_MOVWDLEN:
		sprintf(value, "%lld", stData.Mover.window_len);
		break;
	case AS_MOVADDRTY:
		if (stData.Mover.state == MOVER_ST_IDLE)
			strcpy(value, "N/A");
		else if (stData.Mover.addr_type == ADDR_TYP_LOCAL)
			strcpy(value, "Local");
		else
			strcpy(value, "TCP");
		break;
	case AS_DATASTATE:
		if (stData.Data.state == DATA_ST_IDLE)
			strcpy(value, "Idle");
		else if (stData.Data.state == DATA_ST_LISTEN)
			strcpy(value, "Listen");
		else if (stData.Data.state == DATA_ST_CONNECTED)
			strcpy(value, "Connected");
		else if (stData.Data.state == DATA_ST_ACTIVE)
			strcpy(value, "Active");
		else
			strcpy(value, "Halted");
		break;
	case AS_DATAOP:
		if (stData.Data.op == DATA_OP_NOACTION)
			strcpy(value, "None");
		else if (stData.Data.op == DATA_OP_BACKUP)
			strcpy(value, "Backup");
		else
			strcpy(value, "Restore");
		break;
	case AS_DATAHALR:
		if (stData.Data.halt_reason == DATA_HALT_NA)
			strcpy(value, "N/A");
		else if (stData.Data.halt_reason == DATA_HALT_SUCCESSFUL)
			strcpy(value, "Successful");
		else if (stData.Data.halt_reason == DATA_HALT_ABORTED)
			strcpy(value, "Aborted");
		else if (stData.Data.halt_reason == DATA_HALT_INTERNAL_ERROR)
			strcpy(value, "Internal Error");
		else
			strcpy(value, "Media Error");
		break;
	case AS_DATAPROC:
		sprintf(value, "%lld", stData.Data.bytes_processed);
		break;
	case AS_DATARDOFF:
		sprintf(value, "%lld", stData.Data.read_offset);
		break;
	case AS_DATARDLEN:
		sprintf(value, "%lld", stData.Data.read_length);
		break;
	case AS_DATAADDRTY:
		if (stData.Data.state == DATA_ST_IDLE)
			strcpy(value, "N/A");
		else if (stData.Data.addr_type == ADDR_TYP_LOCAL)
			strcpy(value, "Local");
		else
			strcpy(value, "TCP");
		break;
	default:
		return -1;
	}

	if (value && (value[0] == '\0'))
		strcpy(value, "&nbsp");

	return 1;
}


/*
 * actRptTagFunc()
 *	tag   - tag name
 *	value - replacement string for tag
 */
static int actRptTagFunc(tagCallbackParams_t *tp)
{
char *tag = tp->tag;
char *value = tp->value;

	if (!strcmp(tag, "TABLE_BEGIN")) {
		return -1;
	}
	else if (!strcmp(tag, "TABLE_END")) {
		return -1;
	}
	else {
		/* other tags */
		return getTagValue(tag, value);
	}

	/* shouldn't reach here... */
	return -1;
}

int actionNdmpSessDetail(void *r, int count, char **argname, char **argval)
{
	/* initialize ndmp information variables */
	getInputParams(count, argname, argval);
	GetSessionData();

	prepare_agile_template(r, NULL);

	agile_scan_and_send(r, actRptTagFunc);

	return 1;
}
