/*  File Name  :    agilendmpsessstatus.c 
 *
 *  Description:    HTTPD action functions for displaying ndmp active session status.
 *
 *  Created    :    4/4/2002
 *
 *  Author     :    Tiffany Chiang
 *
 *  Copyright(c)  Agile Storage, Inc. 2001
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <nfx-defs.h>
#include <nfx-types.h>
#include <eee-desc.h>
#include <anpssc-api.h>

#undef MIN
#undef MAX
#include <sys/socket.h>
#include <net/if.h>
#include <netinet/in.h>
#include <arpa/inet.h>

#include <ndmp-ui-api.h>

#include "agilepost.h"
#include "agileutil.h"

#define INVALID_TAG			0
#define AS_SID				1
#define AS_APPNAME			2
#define AS_AUTHORIZE		3
#define AS_VERSION			4
#define AS_TAPESTATE		5
#define AS_MCHSTATE			6
#define AS_MOVSTATE			7
#define AS_DATASTATE		8
#define AS_DATAOP			9
#define NUM_OF_TAGS			9

static tagNameTable_t tagTable[NUM_OF_TAGS] = {
	{AS_SID,		"args.sid"},
	{AS_APPNAME,	"args.app"},
	{AS_AUTHORIZE,	"sess_detail.authorize"},
	{AS_VERSION,	"sess_detail.version"},
	{AS_TAPESTATE,	"sess_detail.tapestate"},
	{AS_MCHSTATE,	"sess_detail.mchstate"},
	{AS_MOVSTATE,	"sess_detail.moverstate"},
	{AS_DATASTATE,	"sess_detail.datastate"},
	{AS_DATAOP,		"sess_detail.dataoper"},
};


#define PARAM_SID			1
#define PARAM_APPNAME		2
#define NUM_OF_PARAMS		2

static paramNameTable_t paramTable[NUM_OF_PARAMS] = {
	{PARAM_SID,		"sid"},
	{PARAM_APPNAME,	"app"},
};


/* HERE ARE VARIABLES FOR NDMP DETAIL INFORMATION */
static int	inId;
static char	inApp[80];
static int	res;
static ndmp_Session_t stData;


/***********************************************************
 * routines to retrieve ndmp information thru anpssc
 **********************************************************/

static int getInputParams(int count, char **argname, char **argval)
{
	int argind, paramType;

	for (argind = 0; argind < count; argind ++)
	{
		paramType = GetParamType(argname[argind], paramTable, NUM_OF_PARAMS);
		if (paramType == INVALID_PARAM)
			continue;

		switch (paramType) 
		{
		case PARAM_SID:
			inId = atoi(argval[argind]);
			break;
		case PARAM_APPNAME:
			strcpy(inApp, argval[argind]);
			break;
		}
	}

	return 0;
}

static void GetSessionData()
{
	ndmp_NFXShowReq_t 		req;
	ndmp_NFXShowChildRsp_t	rsp;
    struct timeval			to;
	int32					rc;

	res = -1;
	memset(&rsp, 0, sizeof(ndmp_NFXShowChildRsp_t));
	req.mid = NDMP_MID_UI_SHOW_STATUS;
	req.sessId = inId;
    to.tv_sec  = 5;
    to.tv_usec = 0;

	/* Send request to child NDMP server */
    rc = sendAgileMsg(-1, &req, sizeof(ndmp_NFXShowReq_t),
                        inApp, NFX_SSC_UNIX, SSC_SLOT, 0, &to,
                        &rsp, sizeof(ndmp_NFXShowChildRsp_t)); 
	if (rc <= 0)
		return;
	if (rsp.stat != NFX_OK)
		return;

	memcpy(&stData, &rsp.sess, sizeof(ndmp_Session_t));
	res = 1;
}


/***********************************************************
 *  Main routines to process tags
 **********************************************************/

static int getTagValue(char *tag, char *value)
{
	int i, tagType;
	
	if (res <= 0)
		return -1;

	tagType = INVALID_TAG;
	for (i = 0; i < NUM_OF_TAGS; i ++) {
		if (!strcmp(tag, tagTable[i].tagName)) {
			tagType = tagTable[i].tagType;
			break;
		}
	}
	if (tagType == INVALID_TAG)
		return -1;

	switch (tagType) 
	{
	case AS_SID:
		sprintf(value, "%d", inId);
		break;
	case AS_APPNAME:
		strcpy(value, inApp);
		break;
	case AS_AUTHORIZE:
		if (stData.authed)
			strcpy(value, "Yes");
		else
			strcpy(value, "No");
		break;
	case AS_VERSION:
		if (stData.protoVer != -1)
			sprintf(value, "%d", stData.protoVer);
		else
			strcpy(value, "0");
		break;
	case AS_TAPESTATE:
		if (stData.Tape.hdl)
			strcpy(value, "Open");
		else
			strcpy(value, "Closed");
		break;
	case AS_MCHSTATE:
		if (stData.SCSI.hdl)
			strcpy(value, "Open");
		else
			strcpy(value, "Closed");
		break;
	case AS_MOVSTATE:
		if (stData.Mover.state == MOVER_ST_IDLE)
			strcpy(value, "Idle");
		else if (stData.Mover.state == MOVER_ST_LISTEN)
			strcpy(value, "Listen");
		else if (stData.Mover.state == MOVER_ST_ACTIVE)
			strcpy(value, "Active");
		else if (stData.Mover.state == MOVER_ST_PAUSED)
			strcpy(value, "Paused");
		else
			strcpy(value, "Halted");
		break;
	case AS_DATASTATE:
		if (stData.Data.state == DATA_ST_IDLE)
			strcpy(value, "Idle");
		else if (stData.Data.state == DATA_ST_LISTEN)
			strcpy(value, "Listen");
		else if (stData.Data.state == DATA_ST_CONNECTED)
			strcpy(value, "Connected");
		else if (stData.Data.state == DATA_ST_ACTIVE)
			strcpy(value, "Active");
		else
			strcpy(value, "Halted");
		break;
	case AS_DATAOP:
		if (stData.Data.op == DATA_OP_NOACTION)
			strcpy(value, "None");
		else if (stData.Data.op == DATA_OP_BACKUP)
			strcpy(value, "Backup");
		else
			strcpy(value, "Restore");
		break;
	default:
		return -1;
	}

	if (value && (value[0] == '\0'))
		strcpy(value, "&nbsp");

	return 1;
}


/*
 * actRptTagFunc()
 *	tag   - tag name
 *	value - replacement string for tag
 */
static int actRptTagFunc(tagCallbackParams_t *tp)
{
char *tag = tp->tag;
char *value = tp->value;

	if (!strcmp(tag, "TABLE_BEGIN")) {
		return -1;
	}
	else if (!strcmp(tag, "TABLE_END")) {
		return -1;
	}
	else {
		/* other tags */
		return getTagValue(tag, value);
	}

	/* shouldn't reach here... */
	return -1;
}

int actionNdmpSessStatus(void *r, int count, char **argname, char **argval)
{
	/* initialize ndmp information variables */
	getInputParams(count, argname, argval);
	GetSessionData();

	prepare_agile_template(r, NULL);

	agile_scan_and_send(r, actRptTagFunc);

	return 1;
}
