/*  File Name  :    agilenfsmodhost.c
 *
 *  Description:    HTTPD action functions for modify export/share host.
 *
 *  Created    :    3/25/2002
 *
 *  Author     :    Tiffany Chiang
 *
 *  Copyright(c)  Agile Storage, Inc. 2001
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <unistd.h>
#include <ctype.h>

#include <anpssc-api.h>

#include <evm_com_defs.h>
#include <evm-api.h>
#include <fs-api.h>
#include <rpc.h>
#include <mount.h>

#include "agilepost.h"
#include "agileutil.h"


#define INVALID_TAG			0
#define VOL_ID				1
#define VOL_NAME			2
#define PATH_NAME			3
#define PATH_IP				4
#define PATH_HOST			5
#define PATH_MASK			6
#define PATH_ROOT			7
#define PATH_ACCESS			8
#define PATH_PERMNO			9
#define PATH_PERMRO			10
#define PATH_PERMRW			11
#define PATH_RSQYES			12
#define PATH_RSQNO			13
#define NEXT_PAGE			14
#define NUM_OF_TAGS			14

static tagNameTable_t tagTable[NUM_OF_TAGS] = {
	{VOL_ID,		"args.vol_id"},
	{VOL_NAME,		"args.vol_name"},
	{PATH_NAME,		"args.path"},
	{PATH_IP,		"args.ip"},
	{PATH_HOST,		"args.host"},
	{PATH_MASK,		"args.mask"},
	{PATH_ROOT,		"pathhost.root"},
	{PATH_ACCESS,	"pathhost.access"},
	{PATH_PERMNO,	"pathhost.permno"},
	{PATH_PERMRO,	"pathhost.permro"},
	{PATH_PERMRW,	"pathhost.permrw"},
	{PATH_RSQYES,	"pathhost.rootsq_y"},
	{PATH_RSQNO,	"pathhost.rootsq_n"},
	{NEXT_PAGE,		"nextpage"},
};


#define PARAM_VOLID			1
#define PARAM_VOLNAME		2
#define PARAM_PATH			3
#define PARAM_IP			4
#define PARAM_HOST			5
#define PARAM_MASK			6
#define PARAM_TYPE			7
#define NUM_OF_PARAMS		7

static paramNameTable_t paramTable[NUM_OF_PARAMS] = {
	{PARAM_VOLID,	"id"},
	{PARAM_VOLNAME,	"vn"},
	{PARAM_PATH,	"path"},
	{PARAM_IP,		"ip"},
	{PARAM_HOST,	"host"},
	{PARAM_MASK,	"mask"},
	{PARAM_TYPE,	"type"},
};


/* HERE ARE VARIABLES FOR SHARE/EXPORT INFORMATION */
typedef struct _shareCharacter_t {
	int perm;
	int rootsq;
} shareCharacter_t;

static char bufpath[MAX_NAME_LEN+1];
static char bufvol[VOL_MAX_NAME+1];
static char bufhost[20];
static char bufmask[20];
static char inType[10];
static char inIP[32];
static uint64 inId;
static int getres;
shareCharacter_t data;


/***********************************************************
 * routines to retrieve export/share information 
 **********************************************************/

static int getInputParams(int count, char **argname, char **argval)
{
	int argind, paramType;

	for (argind = 0; argind < count; argind ++)
	{
		paramType = GetParamType(argname[argind], paramTable, NUM_OF_PARAMS);
		if (paramType == INVALID_PARAM)
			continue;

		switch (paramType) 
		{
		case PARAM_VOLID:
			sscanf(argval[argind], "%lld", &inId);
			break;
		case PARAM_VOLNAME:
			strcpy(bufvol, argval[argind]);
			break;
		case PARAM_PATH:
			strcpy(bufpath, argval[argind]);
			break;
		case PARAM_IP:
			strcpy(inIP, argval[argind]);
			break;
		case PARAM_HOST:
			strcpy(bufhost, argval[argind]);
			break;
		case PARAM_MASK:
			strcpy(bufmask, argval[argind]);
			break;
		case PARAM_TYPE:
			strcpy(inType, argval[argind]);
			break;
		}
	}

	return 0;
}

static void GetHostInfo()
{
	int					i, stat;
	char 				reqbuff[1024];
	char 				respbuff[4096];
	mnt_infoReq_t   	*req;
	mnt_infoResp_t		*rsp;
	mnt_infoEntry_t		*mtnInfo;
	uint32				sharedIP;
	struct timeval		timeout = {5, 0};

	getres = -1;

    /* Fill the request. */
	memset(reqbuff, 0, sizeof(reqbuff));
    req = (mnt_infoReq_t *)reqbuff;

	req->msgHdr.msgType = MNT_EXPORT_SHOW;
	req->volId.whole_thing = inId;

#if 0
/* need better API from Joseph */
	req->nameLen = strlen(bufpath);
	strcpy(req->pathName, bufpath);
	req->ipAddr = GetIpAddressFromStr(bufhost);
#endif

    /* Send message to FS. */
	stat = sendAgileMsg(-1, req, sizeof(mnt_infoReq_t),
                        "mountd", NFX_SSC_UNIX, SSC_SLOT, MGMTPLANE, &timeout,
                        respbuff, sizeof(respbuff));

    /* Check response from FS. */
	if (stat < 0) 
		return;
	else if (!stat)
		return;

    rsp = (mnt_infoResp_t *) respbuff;
	if (rsp->msgHdr.status != NFX_OK)
		return;

	sharedIP = GetIpAddressFromStr(inIP);
	mtnInfo = rsp->mtnInfoEntry;

	for (i = 0; i < rsp->numOfEntries; i++)
	{
		strncpy(reqbuff, mtnInfo->pathName, mtnInfo->pathNameLen);
		reqbuff[mtnInfo->pathNameLen] = '\0';

		if (!strcmp(reqbuff, &bufpath[1]) &&
			(mtnInfo->localIpAddr == sharedIP))
//			(ntohl(mtnInfo->localIpAddr) == sharedIP))
		{
			char *p;

			/* it's the path and shared local IP we are looking for */
			/* now we need to check whether this is the host we want */
			p = mtnInfo->pathName + mtnInfo->pathNameLen;
			strncpy(reqbuff, p, mtnInfo->exportInfoLen);
			reqbuff[mtnInfo->exportInfoLen]='\0';
			p = reqbuff;

			if (isdigit(*p))		/* start with digit, an IP addr */
			{
				while (isdigit(*p) || (*p == '.'))
					p++;
			}
			*p = '\0';

			/* now reqbuff should contain host address */
			if (reqbuff[0] == '\0')
				strcpy(reqbuff, "0.0.0.0");

			if (!strcmp(bufhost, reqbuff))
			{
				/* yup, this is the ONE */
				p++;	/* point to exportInfo */
				data.perm = GetNfsPermission(p);
				data.rootsq = GetNfsRootSquash(p);
				getres = 1;
				break;
			}
		}

		MNT_INFO_RESP_ADV_ENTRY_PTR(mtnInfo);
	}
	
	return;
}


/***********************************************************
 *  Main routines to process tags
 **********************************************************/

static int getTagValue(char *tag, char *value)
{
	int i, tagType;
	
	tagType = INVALID_TAG;
	for (i = 0; i < NUM_OF_TAGS; i++) {
		if (!strcmp(tag, tagTable[i].tagName)) {
			tagType = tagTable[i].tagType;
			break;
		}
	}
	if (tagType == INVALID_TAG)
		return -1;

	switch (tagType) 
	{
	case VOL_ID:
		sprintf(value, "%lld", inId);
		return 1;
	case VOL_NAME:
		strcpy(value, bufvol);
		return 1;
	case PATH_NAME:
		strcpy(value, bufpath);
		return 1;
	case PATH_IP:
		strcpy(value, inIP);
		return 1;
	case PATH_HOST:
		strcpy(value, bufhost);
		return 1;
	case PATH_MASK:
		strcpy(value, bufmask);
		return 1;
	}

	if (getres < 0)
		return -1;

	switch (tagType) 
	{
	case PATH_ROOT:
	case PATH_ACCESS:
	case PATH_PERMNO:
		break;
	case PATH_PERMRO:
		if (data.perm == 0)
			strcpy(value, "selected");
		break;
	case PATH_PERMRW:
		if (data.perm == 1)
			strcpy(value, "selected");
		break;
	case PATH_RSQYES:
		if (data.rootsq == 1)
			strcpy(value, "selected");
		break;
	case PATH_RSQNO:
		if (data.rootsq == 0)
			strcpy(value, "selected");
		break;
	case NEXT_PAGE:
		if (!strcmp(inType, "add"))
			strcpy(value, "NfsAdd");
		else
			strcpy(value, "NfsMod");
		break;
	default:
		return -1;
	}

	return 1;
}


/*
 * actRptTagFunc()
 *	tag   - tag name
 *	value - replacement string for tag
 */
static int actRptTagFunc(tagCallbackParams_t *tp)
{
char *tag = tp->tag;
char *value = tp->value;

	if (!strcmp(tag, "TABLE_BEGIN")) {
		return -1;
	}
	else if (!strcmp(tag, "TABLE_END")) {
		return -1;
	}
	else {
		return getTagValue(tag, value);
	}

	/* shouldn't reach here... */
	return -1;
}

int actionNfsModHost(void *r, int count, char **argname, char **argval)
{
	getInputParams(count, argname, argval);
	GetHostInfo();

	prepare_agile_template(r, NULL);

	agile_scan_and_send(r, actRptTagFunc);

	return 1;
}
