/*  File Name  :    agilensdel.c
 *
 *  Description:    HTTPD action functions for delete DNS, NIS service.
 *
 *  Created    :    329/2002
 *
 *  Author     :    Tiffany Chiang
 *
 *  Copyright(c)  Agile Storage, Inc. 2001
 *
 */

#include <stdio.h>
#include <unistd.h>
#include <sys/types.h>
#include <anpssc-api.h>

#include <ipmd-lib.h>

#include "agilepost.h"
#include "agileutil.h"


#define INVALID_TAG			0
#define NS_IP				1
#define NS_MASK				2
#define NUM_OF_TAGS			2

static tagNameTable_t tagTable[NUM_OF_TAGS] = {
	{NS_IP,			"args.ip"},
	{NS_MASK,		"args.mask"},
};


#define PARAM_IP			1
#define PARAM_MASK			2
#define NUM_OF_PARAMS		2

static paramNameTable_t paramTable[NUM_OF_PARAMS] = {
	{PARAM_IP,		"ip"},
	{PARAM_MASK,	"mask"},
};


/* HERE ARE VARIABLES FOR NIS,DNS INFORMATION */
static char bufip[20];
static char bufmask[20];


/***********************************************************
 * routines to retrieve network information 
 **********************************************************/

static int getInputParams(int count, char **argname, char **argval)
{
	int argind, paramType;
	uint32 data;

	for (argind = 0; argind < count; argind ++)
	{
		paramType = GetParamType(argname[argind], paramTable, NUM_OF_PARAMS);
		if (paramType == INVALID_PARAM)
			continue;

		switch (paramType) 
		{
		case PARAM_IP:
			sscanf(argval[argind], "%u", &data);
			GetIpAddressStr(data, bufip);
			break;
		case PARAM_MASK:
			sscanf(argval[argind], "%u", &data);
			GetIpAddressStr(data, bufmask);
			break;
		}
	}

	return 0;
}


/***********************************************************
 *  Main routines to process tags
 **********************************************************/

static int getTagValue(char *tag, char *value)
{
	int i, tagType;
	
	tagType = INVALID_TAG;
	for (i = 0; i < NUM_OF_TAGS; i++) {
		if (!strcmp(tag, tagTable[i].tagName)) {
			tagType = tagTable[i].tagType;
			break;
		}
	}
	if (tagType == INVALID_TAG)
		return -1;

	switch (tagType) 
	{
	case NS_IP:
		strcpy(value, bufip);
		break;
	case NS_MASK:
		strcpy(value, bufmask);
		break;
	default:
		return -1;
	}

	return 1;
}


/*
 * actRptTagFunc()
 *	tag   - tag name
 *	value - replacement string for tag
 */
static int actRptTagFunc(tagCallbackParams_t *tp)
{
char *tag = tp->tag;
char *value = tp->value;

	if (!strcmp(tag, "TABLE_BEGIN")) {
		return -1;
	}
	else if (!strcmp(tag, "TABLE_END")) {
		return -1;
	}
	else {
		return getTagValue(tag, value);
	}

	/* shouldn't reach here... */
	return -1;
}

int actionNsDel(void *r, int count, char **argname, char **argval)
{
	getInputParams(count, argname, argval);

	prepare_agile_template(r, NULL);

	agile_scan_and_send(r, actRptTagFunc);

	return 1;
}
