/*  File Name  :    agilensmain.c
 *
 *  Description:    HTTPD action functions for name service main page.
 *
 *  Created    :    3/29/2002
 *
 *  Author     :    Tiffany Chiang
 *
 *  Copyright(c)  Agile Storage, Inc. 2001
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <unistd.h>
#include <anpssc-api.h>

#include <nfxnis-api.h>

#include "agilepost.h"
#include "agileutil.h"
#include "../agent/httpdagent.h"


#define INVALID_TAG			0
#define NS_PRIVCOL			1
#define NS_NETSTR			2
#define NS_NET				3
#define NS_MASKSTR			4
#define NS_MASK				5
#define NS_DNSDOMAIN		6
#define NS_DNSSVRALL		7
#define NS_NISDOMAIN		8
#define NS_NISSVR			9
#define NS_MODPAGE			10
#define NS_DELETE			11
#define NUM_OF_TAGS			11

static tagNameTable_t tagTable[NUM_OF_TAGS] = {
	{NS_PRIVCOL,		"priv.colspan"},
	{NS_NETSTR,			"nfxnis_cfg_t.netstr"},
	{NS_NET,			"nfxnis_cfg_t.net"},
	{NS_MASKSTR,		"nfxnis_cfg_t.maskstr"},
	{NS_MASK,			"nfxnis_cfg_t.mask"},
	{NS_DNSDOMAIN,		"nfxnis_cfg_t.dnsdomain"},
	{NS_DNSSVRALL,		"nfxnis_cfg_t.dnsserver.all"},
	{NS_NISDOMAIN,		"nfxnis_cfg_t.nisdomain"},
	{NS_NISSVR,			"nfxnis_cfg_t.nisserver"},
	{NS_MODPAGE,		"nfxnis.modpage"},
	{NS_DELETE,			"nfxnis.delete"},
};


#define PARAM_ACT			1
#define PARAM_IP			2
#define PARAM_MASK			3
#define PARAM_DNS			4
#define PARAM_DNSDOMAIN		5
#define PARAM_DNSSVR1		6
#define PARAM_DNSSVR2		7
#define PARAM_DNSSVR3		8
#define PARAM_NIS			9
#define PARAM_NISDOMAIN		10
#define PARAM_NISSVR		11
#define NUM_OF_PARAMS		11

static paramNameTable_t paramTable[NUM_OF_PARAMS] = {
	{PARAM_ACT,		"act"},
	{PARAM_IP,		"ip"},
	{PARAM_MASK,	"mask"},
	{PARAM_DNS,		"dns"},
	{PARAM_DNSDOMAIN,"dnsdm"},
	{PARAM_DNSSVR1,	"dnssvr1"},
	{PARAM_DNSSVR2,	"dnssvr2"},
	{PARAM_DNSSVR3,	"dnssvr3"},
	{PARAM_NIS,		"nis"},
	{PARAM_NISDOMAIN,"nisdm"},
	{PARAM_NISSVR,	"nissvr"},
};

#define DOMAIN_NAME_LEN		64
#define MAX_NUM_ENTRIES		10

static char bufact[10];
static uint32 inIp;
static uint32 inMask;
static int inDns;
static char inDnsDomain[DOMAIN_NAME_LEN+1];
static uint32 inDnsSvr1;
static uint32 inDnsSvr2;
static uint32 inDnsSvr3;
static int inNis;
static char inNisDomain[128];
static uint32 inNisSvr;


/* HERE ARE VARIABLES FOR DNS,NIS INFORMATION */
typedef struct _nfxnis_entry_t
{
    uint32			ip;
    uint32			mask;
	char			dnsDomain[DOMAIN_NAME_LEN+1];
	int				numDnsServers;
	uint32			dnsServers[NFX_MAX_DNS_SERVERS_PER_DOMAIN];
	char			nisDomain[DOMAIN_NAME_LEN+1];
	int				numNisServers;
	uint32			nisServers[1];
} nfxnis_entry_t;

static int dataCount;
static int dataCurr;
static nfxnis_entry_t *stNameSvcTable;
static nfxnis_entry_t stData;


/***********************************************************
 * routines to retrieve name service information 
 **********************************************************/

static int getInputParams(int count, char **argname, char **argval)
{
	int argind, paramType;

	for (argind = 0; argind < count; argind ++)
	{
		paramType = GetParamType(argname[argind], paramTable, NUM_OF_PARAMS);
		if (paramType == INVALID_PARAM)
			continue;

		switch (paramType) 
		{
		case PARAM_ACT:
			strcpy(bufact, argval[argind]);
			break;
		case PARAM_IP:
			inIp = GetIpAddressFromStr(argval[argind]);
			break;
		case PARAM_MASK:
			inMask = GetIpAddressFromStr(argval[argind]);
			break;
		case PARAM_DNS:
			if (strcmp(argval[argind], "on") == 0)
				inDns = 1;
			else
				inDns = 0;
			break;
		case PARAM_DNSDOMAIN:
			strcpy(inDnsDomain, argval[argind]);
			break;
		case PARAM_DNSSVR1:
			inDnsSvr1 = GetIpAddressFromStr(argval[argind]);
			break;
		case PARAM_DNSSVR2:
			inDnsSvr2 = GetIpAddressFromStr(argval[argind]);
			break;
		case PARAM_DNSSVR3:
			inDnsSvr3 = GetIpAddressFromStr(argval[argind]);
			break;
		case PARAM_NIS:
			if (strcmp(argval[argind], "on") == 0)
				inNis = 1;
			else
				inNis = 0;
			break;
		case PARAM_NISDOMAIN:
			strcpy(inNisDomain, argval[argind]);
			break;
		case PARAM_NISSVR:
			inNisSvr = GetIpAddressFromStr(argval[argind]);
			break;
		}
	}

	return 0;
}

static void SetDefaultEntry()
{
	stNameSvcTable[0].ip   = 0;
	stNameSvcTable[0].mask = 0;
	strcpy(stNameSvcTable[0].dnsDomain, "\0");
	stNameSvcTable[0].numDnsServers = 0;
	strcpy(stNameSvcTable[0].nisDomain, "\0");
	stNameSvcTable[0].numNisServers = 0;
}

static void AddEntry(nfxnis_cfg_t* entry, int* count, int* alloc)
{
	int i, index;
	nfxnis_entry_t* newTable;

	// Find entry, first
	for (i = 0, index = -1; i < *count; i ++)
	{
		if (stNameSvcTable[i].ip == entry->net) 
		{
			index = i;
			break;
		}
	}

	// Entry not found
	if (index == -1)
	{
		if (*count >= *alloc)
		{
			newTable = realloc(stNameSvcTable,
					((*alloc)+MAX_NUM_ENTRIES)*sizeof(nfxnis_entry_t));
			if (newTable == NULL)
				return;
			stNameSvcTable = newTable;
			memset(&stNameSvcTable[*alloc], 0,
							MAX_NUM_ENTRIES * sizeof(nfxnis_entry_t));
			*alloc += MAX_NUM_ENTRIES;
		}
		index = *count;
		(*count)++;
	}
	if (index < 0)
		return;

	// Update the entry
	stNameSvcTable[index].ip = entry->net;
	stNameSvcTable[index].mask = entry->mask;
	if (entry->type == dns_name)
	{
		strcpy(stNameSvcTable[index].dnsDomain, entry->domain);
		stNameSvcTable[index].numDnsServers = entry->num_srvrs;

		for (i = 0; i < entry->num_srvrs; i ++)
			stNameSvcTable[index].dnsServers[i] = entry->srvrs[i];
		for (i = entry->num_srvrs; i < NFX_MAX_DNS_SERVERS_PER_DOMAIN; i ++)
			stNameSvcTable[index].dnsServers[i] = 0;
	}
	else if (entry->type == nis_name)
	{
		strcpy(stNameSvcTable[index].nisDomain, entry->domain);
		stNameSvcTable[index].numNisServers = entry->num_srvrs;
		stNameSvcTable[index].nisServers[0] = entry->srvrs[0];
	}
}

static int getTableRowCount(void)
{
	int i, alloccnt, ttlcnt;
	nfxnis_cfg_t **list;
	nfxnis_cfg_t *entry;

	// Allocate local memory
	stNameSvcTable = malloc(MAX_NUM_ENTRIES * sizeof(nfxnis_entry_t));
	if (stNameSvcTable == NULL)
		return -1;

	memset(stNameSvcTable, 0, (MAX_NUM_ENTRIES * sizeof(nfxnis_entry_t)));
	alloccnt = MAX_NUM_ENTRIES;

	// Set the default entry
	SetDefaultEntry();
	ttlcnt = 1;

	// Get dns,nis configurations
	list = nfxnis_get_cfg(0, 0);
	if (list == NULL || *list == NULL)
		return ttlcnt;

	// Copy the configuration to local memory
	for (i = 0; list[i] != NULL; i++)
	{
		entry = list[i];
		AddEntry(entry, &ttlcnt, &alloccnt);

		// Free memory allocated by nfxnis_api
		free(entry->domain);
		free(entry);
	}
	free(list);
	return ttlcnt;
}

static int getTableRowData(int *index, nfxnis_entry_t *data)
{
	if (*index < 1 || *index > dataCount)
		return -1;
	if (stNameSvcTable == NULL)
		return -1;

	memcpy(data, &stNameSvcTable[(*index)-1], sizeof(nfxnis_entry_t));
	return 0;
}


/***********************************************************
 *  Main routines to handle Add/Del/Mod/ModDef actions
 **********************************************************/

static int DoAddNameSvc(uint32 ip, uint32 mask)
{
	int res;

	if (inDns != 0)
	{
		res = nfxnis_addservice (dns_name, ip, mask, inDnsSvr1, inDnsDomain);
		if (res != 0)
			return -1;
		if (inDnsSvr2 != 0)
		{
			res = nfxnis_addservice (dns_name, ip, mask, inDnsSvr2, inDnsDomain);
			if (res != 0)
				return -1;
		}
		if (inDnsSvr3 != 0)
		{
			res = nfxnis_addservice (dns_name, ip, mask, inDnsSvr3, inDnsDomain);
			if (res != 0)
				return -1;
		}
	}

	if (inNis != 0)
	{
		char buf[512];
		char rspbuf[128];
		haMessage_t *req;
		haMessage_t *rsp;
		haUpdateNIS_t *pdata;
		int rc;
		struct timeval tval = {300, 0};

		/* add NIS service need to be in privilege mode. */
		/* so we'll have to let httpdagent to do it...   */

		rsp = (haMessage_t *) rspbuf;
		req = (haMessage_t *) buf;

		req->type = HA_UPDATENIS;
		req->datalen = sizeof(haUpdateNIS_t);
		pdata = (haUpdateNIS_t *) req->data;

		pdata->ip = ip;
		pdata->mask = mask;
		pdata->nisSvr = inNisSvr;
		strcpy(pdata->domain, inNisDomain);

	    rc = sendAgileMsg(-1, buf, HAMESSAGE_HEADER_SIZE + req->datalen,
                        "httpdagent", NFX_SSC_UNIX, SSC_SLOT, MGMTPLANE,
                        &tval, rspbuf, sizeof(rspbuf));
		if (rc <= 0)
			return -1;
		if (rsp->rspstatus != HARSP_OK)
			return -1;
#if 0
		res = nfxnis_addservice (nis_name, ip, mask, inNisSvr, inNisDomain);
		if (res != 0)
			return -1;
#endif
	}
	return 1;
}

static int DoDelNameSvc(uint32 ip, uint32 mask)
{
	int res;
	res = nfxnis_delservice(ip, mask);
	if (res == 0)
		return 1;
	return -1;
}

static int DoModNameSvc(uint32 ip, uint32 mask)
{
	int res, dodelete = 1;

	/* if modifying default, we need to know if */
	/* default entry exists. If not, we should  */
	/* not do delete first as it'll crash the   */
	/* thread...                                */
	if ((ip == 0) && (mask == 0))
	{
		int i;
		nfxnis_cfg_t **list;

		list = nfxnis_get_cfg(0, 0);
		if (list == NULL || *list == NULL)
			dodelete = 0;
		else
		{
			dodelete = 0;
			for (i = 0; list[i] != NULL; i ++)
			{
				if ((list[i]->net == 0) && (list[i]->mask == 0))
					dodelete = 1;
				// Free memory allocated by nfxnis_api
				free(list[i]->domain);
				free(list[i]);
			}
			free(list);
		}
	}

	if (dodelete)
	{
		res = DoDelNameSvc(ip, mask);
		if (res < 0)
			return -1;
	}

	return DoAddNameSvc(ip, mask);
}

static int HandleNameSvcAction(char *newfname, int count, char **argname, char **argval)
{
	int res = 1;

	getInputParams(count, argname, argval);

	// Add
	if (!strcmp(bufact, "add"))
	{
		res = DoAddNameSvc(inIp, inMask);
		if (res <= 0)
			strcpy(newfname, "error_addns.html");
	}
	// Del
	else if (!strcmp(bufact, "del"))
	{
		res = DoDelNameSvc(inIp, inMask);
		if (res <= 0)
			strcpy(newfname, "error_delns.html");
	}
	// Modify
	else if (!strcmp(bufact, "mod"))
	{
		res = DoModNameSvc(inIp, inMask);
		if (res <= 0)
			strcpy(newfname, "error_modns.html");
	}
	// Modify Default
	else if (!strcmp(bufact, "moddef"))
	{
		res = DoModNameSvc(0, 0);
		if (res <= 0)
			strcpy(newfname, "error_modnsdef.html");
	}
	return res;
}


/***********************************************************
 *  Main routines to process tags
 **********************************************************/

static int getTagValue(char *tag, char *value)
{
	int i, tagType, pri;
	char tempbuf[256];
	
	tagType = INVALID_TAG;
	for (i = 0; i < NUM_OF_TAGS; i++) {
		if (!strcmp(tag, tagTable[i].tagName)) {
			tagType = tagTable[i].tagType;
			break;
		}
	}
	if (tagType == INVALID_TAG)
		return -1;

	switch (tagType) 
	{
	case NS_PRIVCOL:
		pri = GetUserPrivilege();
		if (pri >= PRIV_FULL)	// del, mod
			strcpy(value, "2");
		else
			strcpy(value, "1");	// mod
		return 1;
	}

	if ((dataCurr <= 0) || (dataCurr > dataCount))
		return -1;

	switch (tagType) 
	{
	case NS_NETSTR:
		GetIpAddressStr(stData.ip, value);
		break;
	case NS_NET:
		sprintf(value, "%u", stData.ip);
		break;
	case NS_MASKSTR:
		GetIpAddressStr(stData.mask, value);
		break;
	case NS_MASK:
		sprintf(value, "%u", stData.mask);
		break;
	case NS_DNSDOMAIN:
		strcpy(value, stData.dnsDomain);
		if (!value[0])
			strcpy(value, "&nbsp;");
		break;
	case NS_DNSSVRALL:
		if (stData.numDnsServers <= 0)
		{
			strcpy(value, "<tr><td class=\"tabledata\" nowrap>&nbsp;</td></tr>\n");
		}
		else
		{
			strcpy(value, "");
			for(i = 0; i < stData.numDnsServers; i ++)
			{
				strcat(value, "<tr><td class=\"tabledata\" nowrap>");
				GetIpAddressStr(stData.dnsServers[i], tempbuf);
				strcat(value, tempbuf);
				strcat(value, "</td></tr>\n");
			}
		}
		break;
	case NS_NISDOMAIN:
		strcpy(value, stData.nisDomain);
		if (!value[0])
			strcpy(value, "&nbsp;");
		break;
	case NS_NISSVR:
		if (stData.numNisServers <= 0)
			strcpy(value, "&nbsp;");
		else
			GetIpAddressStr(stData.nisServers[0], value);
		break;
	case NS_MODPAGE:
		if (dataCurr == 1 && stData.ip == 0)
			strcpy(value, "NameSvcModDef");
		else
			strcpy(value, "NameSvcMod");
		break;
	case NS_DELETE:
		if (dataCurr == 1 && stData.ip == 0)
			strcpy(value, "&nbsp;");
		else
		{
			sprintf(value,
				"<a href=\"/agilerequestprefix/NameSvcDel?ip=%d&mask=%d\">Delete</a>",
				stData.ip, stData.mask);
#if 0
			strcpy(value, "<a href=\"/agilerequestprefix/NameSvcDel\">Delete</a>");
#endif
		}
		break;
	default:
		return -1;
	}

	return 1;
}


/*
 * actRptTagFunc()
 *	tag   - tag name
 *	value - replacement string for tag
 */
static int actRptTagFunc(tagCallbackParams_t *tp)
{
char *tag = tp->tag;
char *value = tp->value;

	if (!strcmp(tag, "TABLE_BEGIN")) {
		/* TABLE_BEGIN tag, need to see if there is any data */
		dataCount = getTableRowCount();
		if (dataCount > 0) {
			/* need to retrieve first table's data */
			dataCurr = 1;
			if (getTableRowData(&dataCurr, &stData) >= 0)
				return 1;
		}

		dataCurr = 0;
		return -1;
	}
	else if (!strcmp(tag, "TABLE_END")) {
		/* TABLE_END tag, check if end of data */
		if ((dataCount <= 0) || (dataCurr <= 0) || (dataCurr >= dataCount))
			return -1;

		dataCurr++;
		if (getTableRowData(&dataCurr, &stData) < 0)
			return -1;
		else {
			return 1;
		}
	}
	else {
		/* other tags */
		return getTagValue(tag, value);
	}

	/* shouldn't reach here... */
	return -1;
}


int actionNsMain(void *r, int count, char **argname, char **argval)
{
	char newfname[128];

	memset(newfname, 0, sizeof(newfname));
	inDnsDomain[0] = '\0';
	inNisDomain[0] = '\0';

	/* Handle Add/Del/Mod/ModDef actions */
	if (count > 0)
		HandleNameSvcAction(newfname, count, argname, argval);

	prepare_agile_template(r, newfname);

	/* initialize nfs information variables */
	dataCount = 0;
	dataCurr = 0;
	stNameSvcTable = NULL;

	agile_scan_and_send(r, actRptTagFunc);

	if (stNameSvcTable != NULL)
		free(stNameSvcTable);
	return 1;
}
