/*  File Name  :    agilenwdelarp.c
 *
 *  Description:    HTTPD action functions for delete ARP.
 *
 *  Created    :    2/6/2002
 *
 *  Author     :    Tiffany Chiang
 *
 *  Copyright(c)  Agile Storage, Inc. 2001
 *
 */

#include <sys/types.h>
#include <unistd.h>
#include <anpssc-api.h>

#include <ipmd-lib.h>

#include "agilepost.h"
#include "agileutil.h"


#define INVALID_TAG			0
#define NWRT_IP				1
#define NWRT_MAC			2
#define NUM_OF_TAGS			2

static tagNameTable_t tagTable[NUM_OF_TAGS] = {
	{NWRT_IP,		"args.ip"},
	{NWRT_MAC,		"args.mac"},
};


#define PARAM_IP			1
#define PARAM_MAC			2
#define NUM_OF_PARAMS		2

static paramNameTable_t paramTable[NUM_OF_PARAMS] = {
	{PARAM_IP,		"ip"},
	{PARAM_MAC,		"mac"},
};


/* HERE ARE VARIABLES FOR NETWORK INFORMATION */
static char bufip[20];
static char bufmac[20];


/***********************************************************
 * routines to retrieve network information 
 **********************************************************/

static int getInputParams(int count, char **argname, char **argval)
{
	int argind, paramType;

	for (argind = 0; argind < count; argind ++)
	{
		paramType = GetParamType(argname[argind], paramTable, NUM_OF_PARAMS);
		if (paramType == INVALID_PARAM)
			continue;

		switch (paramType) 
		{
		case PARAM_IP:
			strcpy(bufip, argval[argind]);
			break;
		case PARAM_MAC:
			strcpy(bufmac, argval[argind]);
			break;
		}
	}

	return 0;
}


/***********************************************************
 *  Main routines to process tags
 **********************************************************/

static int getTagValue(char *tag, char *value)
{
	int i, tagType;
	
	tagType = INVALID_TAG;
	for (i = 0; i < NUM_OF_TAGS; i++) {
		if (!strcmp(tag, tagTable[i].tagName)) {
			tagType = tagTable[i].tagType;
			break;
		}
	}
	if (tagType == INVALID_TAG)
		return -1;

	switch (tagType) 
	{
	case NWRT_IP:
		strcpy(value, bufip);
		break;
	case NWRT_MAC:
		strcpy(value, bufmac);
		break;
	default:
		return -1;
	}

	return 1;
}


/*
 * actRptTagFunc()
 *	tag   - tag name
 *	value - replacement string for tag
 */
static int actRptTagFunc(tagCallbackParams_t *tp)
{
char *tag = tp->tag;
char *value = tp->value;

	if (!strcmp(tag, "TABLE_BEGIN")) {
		return -1;
	}
	else if (!strcmp(tag, "TABLE_END")) {
		return -1;
	}
	else {
		return getTagValue(tag, value);
	}

	/* shouldn't reach here... */
	return -1;
}

int actionNwDelARP(void *r, int count, char **argname, char **argval)
{
	/* initialize network information variables */
	getInputParams(count, argname, argval);

	prepare_agile_template(r, NULL);

	agile_scan_and_send(r, actRptTagFunc);

	return 1;
}
