/*  File Name  :    agileportdetail.c 
 *
 *  Description:    HTTPD action functions for displaying storage port details.
 *
 *  Created    :    2/25/2002
 *
 *  Author     :    Tiffany Chiang
 *
 *  Copyright(c)  Agile Storage, Inc. 2001
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#if 0
#include <sys/types.h>
#include <unistd.h>
#endif

#include <nfx-defs.h>
#include <nfx-types.h>
#include <eee-desc.h>
#include <scsi-api.h>
#include <hal.h>
#include <sdm-api.h>
#include <anpssc-api.h>

#include "agilepost.h"
#include "agileutil.h"

#define INVALID_TAG			0
#define STGPT_ADPTNAME		1
#define STGPT_FWVER			2
#define STGPT_HWVER			3
#define STGPT_IFTYPE		4
#define STGPT_LNKSPEED		5
#define STGPT_MAXTAGS		6
#define STGPT_NODENAME		7
#define STGPT_PKTSIZE		8
#define STGPT_SRAMPRTY		9
#define STGPT_LNKSTATE		10
#define STGPT_SLOT			11
#define STGPT_PORT			12
#define NUM_OF_TAGS			12

static tagNameTable_t tagTable[NUM_OF_TAGS] = {
	{STGPT_ADPTNAME,"hal_adapter_info_t.name"},
	{STGPT_FWVER,	"hal_adapter_info_t.firmware_rev"},
	{STGPT_HWVER,	"hal_adapter_info_t.hardware_rev"},
	{STGPT_IFTYPE,	"hal_adapter_info_t.interface_type"},
	{STGPT_LNKSPEED,"hal_adapter_info_t.link_speed"},
	{STGPT_MAXTAGS,	"hal_adapter_info_t.max_tags_per_device"},
	{STGPT_NODENAME,"hal_adapter_info_t.fc_node_name"},
	{STGPT_PKTSIZE,	"hal_adapter_info_t.fc_packet_size"},
	{STGPT_SRAMPRTY,"hal_adapter_info_t.sram_parity"},
	{STGPT_LNKSTATE,"adap_connection_state_rsp.data"},
	{STGPT_SLOT,	"args.slot"},
	{STGPT_PORT,	"args.port"},
};


#define PARAM_SLOT			1
#define PARAM_PORT			2
#define NUM_OF_PARAMS		2

static paramNameTable_t paramTable[NUM_OF_PARAMS] = {
	{PARAM_SLOT,	"slot"},
	{PARAM_PORT,	"port"},
};


/* HERE ARE VARIABLES FOR STORAGE DEVICE DETAIL INFORMATION */
static int32 inSlot;
static int32 inPort;
static int res;
static hal_adapter_info_t stData;
static int32 linkState;


/***********************************************************
 * routines to retrieve storage information thru anpssc
 **********************************************************/

static int getInputParams(int count, char **argname, char **argval)
{
	int argind, paramType;

	for (argind = 0; argind < count; argind ++)
	{
		paramType = GetParamType(argname[argind], paramTable, NUM_OF_PARAMS);
		if (paramType == INVALID_PARAM)
			continue;

		switch (paramType) 
		{
		case PARAM_SLOT:
			inSlot = atoi(argval[argind]);
			break;
		case PARAM_PORT:
			inPort = atoi(argval[argind]);
			break;
		}
	}

	return 0;
}

static void GetPortData()
{
    sdm_ioctlReqMsg_t		msg;
	sdm_ioctlRspMsg_t *		adap_info_rsp = 0;
	sdm_ioctlRspMsg_t *		adap_connection_state_rsp = 0;
	sdm_ioctlRspMsg_t *		rsp;
	int						rsp_len;
    int						rc;
	struct timeval			timeout = {5, 0};

	res = -1;
	memset(&stData, 0, sizeof(stData));

	/*
	 * Get PORT/LINK INFO
	 */
	memset(&msg, 0, sizeof(sdm_ioctlReqMsg_t));
	msg.hdr.mid = SDM_MID_IOCTL_REQ;
	msg.code = HAL_IOCTL_COMMON_GET_ADAPTER_INFO;
	msg.slot = inSlot;
	msg.port = inPort;
	
	rsp_len = sizeof(sdm_ioctlRspMsg_t) + sizeof(hal_adapter_info_t);
	rsp = adap_info_rsp = malloc(rsp_len);
	if (rsp == NULL) 
	{
		goto done;
	}
	rc = sendAgileMsg(-1, &msg, sizeof(sdm_ioctlReqMsg_t),
					  "sdm", NFX_SSC_UNIX, SSC_SLOT, 0, &timeout,
					  rsp, rsp_len);
	if (rc <= 0 || 
		rsp->hdr.mid != SDM_MID_IOCTL_RSP ||
		rsp->eCode != SDM_OK) 
	{
		goto done;
	}
	if (adap_info_rsp->dataLen < sizeof(hal_adapter_info_t)) 
	{
		goto done;
	}

	memcpy(&stData, adap_info_rsp->data, sizeof(hal_adapter_info_t));

	/*
	 * Get PORT/LINK CONNECTION STATE
	 */
	memset(&msg, 0, sizeof(sdm_ioctlReqMsg_t));
	msg.hdr.mid = SDM_MID_IOCTL_REQ;
	msg.code = HAL_IOCTL_FC_CONNECTION_STATE;
	msg.slot = inSlot;
	msg.port = inPort;
	
	rsp_len = sizeof(sdm_ioctlRspMsg_t) + sizeof(uint32);
	rsp = adap_connection_state_rsp = malloc(rsp_len);
	if (rsp == NULL) 
	{
		goto done;
	}
	rc = sendAgileMsg(-1, &msg, sizeof(sdm_ioctlReqMsg_t),
					  "sdm", NFX_SSC_UNIX, SSC_SLOT, 0, &timeout,
					  rsp, rsp_len);

	if (rc <= 0 || 
		rsp->hdr.mid != SDM_MID_IOCTL_RSP ||
		rsp->eCode != SDM_OK) 
	{
		goto done;
	}
	if (adap_connection_state_rsp->dataLen < sizeof(uint32)) 
	{
		goto done;
	}
	// memcpy(adap_connection_state_rsp->data, &linkState, sizeof(uint32));
	memcpy(&linkState, adap_connection_state_rsp->data, sizeof(uint32));
	

 done:
	if (adap_info_rsp)
		free(adap_info_rsp);
	if (adap_connection_state_rsp)
		free(adap_connection_state_rsp);
	
	res = 1;
}


/***********************************************************
 *  Main routines to process tags
 **********************************************************/

static int getTagValue(char *tag, char *value)
{
	int i, tagType;
	
	if (res <= 0)
		return -1;

	tagType = INVALID_TAG;
	for (i = 0; i < NUM_OF_TAGS; i ++) {
		if (!strcmp(tag, tagTable[i].tagName)) {
			tagType = tagTable[i].tagType;
			break;
		}
	}
	if (tagType == INVALID_TAG)
		return -1;

	switch (tagType) 
	{
	case STGPT_ADPTNAME:
		strcpy(value, stData.name);
		break;
	case STGPT_FWVER:
		strcpy(value, stData.firmware_rev);
		break;
	case STGPT_HWVER:
		strcpy(value, stData.hardware_rev);
		break;
	case STGPT_IFTYPE:
		switch(stData.interface_type) 
		{
		case HAL_IF_FC_L_PORT:	    strcpy(value, "L_PORT  "); break;
		case HAL_IF_FC_NL_PORT:	    strcpy(value, "NL_PORT "); break;
		case HAL_IF_FC_N_PORT:	    strcpy(value, "N_PORT  "); break;
		case HAL_IF_FC_F_PORT:	    strcpy(value, "F_PORT  "); break;
		case HAL_IF_FC_FL_PORT:	    strcpy(value, "FL_PORT "); break;
		default:					strcpy(value, "Unknown "); break;
		}
		break;
	case STGPT_LNKSPEED:
		sprintf(value, "%d", stData.u.fc_data.link_speed);
		break;
	case STGPT_MAXTAGS:
		sprintf(value, "%d", stData.max_tags_per_device);
		break;
	case STGPT_NODENAME:
		GetWwnStr(stData.u.fc_data.fc_node_name.u.wwn, value);
		break;
	case STGPT_PKTSIZE:
		sprintf(value, "%d", stData.u.fc_data.fc_packet_size);
		break;
	case STGPT_SRAMPRTY:
		if (stData.u.fc_data.sram_parity)
			strcpy(value, "Yes");
		else
			strcpy(value, "No");
		break;
	case STGPT_LNKSTATE:
		switch(linkState) 
		{
		case HAL_IOCTL_FC_LINK_STATE_LINK_DOWN:		strcpy(value, "Down");     break;
		case HAL_IOCTL_FC_LINK_STATE_LINK_UP:		strcpy(value, "Up");       break;
		case HAL_IOCTL_FC_LINK_STATE_LINK_OFFLINE:	strcpy(value, "Disabled"); break;
		default:									strcpy(value, "Unknown");  break;
		}
		break;
	case STGPT_SLOT:
		sprintf(value, "%d", inSlot);
		break;
	case STGPT_PORT:
		sprintf(value, "%d", inPort);
		break;
	default:
		return -1;
	}

	if (value && (value[0] == '\0'))
		strcpy(value, "&nbsp");

	return 1;
}


/*
 * actRptTagFunc()
 *	tag   - tag name
 *	value - replacement string for tag
 */
static int actRptTagFunc(tagCallbackParams_t *tp)
{
char *tag = tp->tag;
char *value = tp->value;

	if (!strcmp(tag, "TABLE_BEGIN")) {
		return -1;
	}
	else if (!strcmp(tag, "TABLE_END")) {
		return -1;
	}
	else {
		/* other tags */
		return getTagValue(tag, value);
	}

	/* shouldn't reach here... */
	return -1;
}

int actionPortDetail(void *r, int count, char **argname, char **argval)
{
	/* initialize storage information variables */
	getInputParams(count, argname, argval);
	GetPortData();

	prepare_agile_template(r, NULL);

	agile_scan_and_send(r, actRptTagFunc);

	return 1;
}
