/*  File Name  :    agileportmain.c 
 *
 *  Description:    HTTPD action functions for storage port main page.
 *
 *  Created    :    2/25/2002
 *
 *  Author     :    Tiffany Chiang
 *
 *  Copyright(c)  Agile Storage, Inc. 2001
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

// #include <sys/types.h>
// #include <unistd.h>
#include <sys/time.h>
#include <anpssc-api.h>

#include <nfx-defs.h>
#include <nfx-types.h>
#include <hal.h>
#include <eee-desc.h>
//#include <cm-api.h>
#include <sdm-api.h>

#include "agilepost.h"
#include "agileutil.h"

#define INVALID_TAG			0
#define STGPORT_SLOT		1
#define STGPORT_PORT		2
#define STGPORT_TYPE		3
#define STGPORT_STATE		4
#define STGPORT_SP			5
#define STGPORT_ENABLE		6
#define STGPORT_DISABLE		7
#define NUM_OF_TAGS			7

static tagNameTable_t tagTable[NUM_OF_TAGS] = {
	{STGPORT_SLOT,		"fcnim.slot"},
	{STGPORT_PORT,		"fcnim.port"},
	{STGPORT_TYPE,		"fcnim.ptype"},
	{STGPORT_STATE,		"fcnim.state"},
	{STGPORT_SP,		"fcnim.slot_port"},
	{STGPORT_ENABLE,	"fcnim.enable"},
	{STGPORT_DISABLE,	"fcnim.disable"},
};


#define PARAM_SLOT			1
#define PARAM_PORT			2
#define PARAM_ACT			3
#define NUM_OF_PARAMS		3

static paramNameTable_t paramTable[NUM_OF_PARAMS] = {
	{PARAM_SLOT,	"slot"},
	{PARAM_PORT,	"port"},
	{PARAM_ACT,		"act"},
};


/* HERE ARE VARIABLES FOR STORAGE PORT LIST INFORMATION */
#define PORT_DESC_LEN		30
#define FCNIM_PORT_COUNT	4
#define FCNIM_SLOT_NUM		2

typedef struct _storage_port 
{
	int slot;
	int port;
	char desc[PORT_DESC_LEN];
	int state;
} storage_port_t;

static int32 inSlot;
static int32 inPort;
static char inAct[10];
static int dataCount;
static int dataCurr;
static int portCount;
// static int realPortCount;
static storage_port_t stData;
static storage_port_t *portList;

#define PORT_STATE_DOWN			0
#define PORT_STATE_UP			1
#define PORT_STATE_DISABLE		2
#define PORT_STATE_UNKNOWN		3

static char *portState_str[] = {
	"Down",
	"Up",
	"Disable",
	"Unknown",
};


/***********************************************************
 *  Main routines to handle Reset/Enable/Disable Port actions
 **********************************************************/

static int getInputParams(int count, char **argname, char **argval)
{
	int argind, paramType;

	for (argind = 0; argind < count; argind ++)
	{
		paramType = GetParamType(argname[argind], paramTable, NUM_OF_PARAMS);
		if (paramType == INVALID_PARAM)
			continue;

		switch (paramType) 
		{
		case PARAM_SLOT:
			sscanf(argval[argind], "%d", &inSlot);
			break;
		case PARAM_PORT:
			sscanf(argval[argind], "%d", &inPort);
			break;
		case PARAM_ACT:
			strcpy(inAct, argval[argind]);
			break;
		}
	}

	return 0;
}

int IsStoragePortAlive(int32 slot, int32 port)
{

    sdm_ioctlReqMsg_t		msg;
	sdm_ioctlRspMsg_t *		rsp;
	int						rsp_len;
    int						rc;
	struct timeval			to = {5, 0};

	/*
	 * "Ping" the port by getting the PORT/LINK INFO
	 */
	memset(&msg, 0, sizeof(sdm_ioctlReqMsg_t));
	msg.hdr.mid = SDM_MID_IOCTL_REQ;
	msg.code = HAL_IOCTL_COMMON_GET_ADAPTER_INFO;
	msg.slot = slot;
	msg.port = port;
	
	rsp_len = sizeof(sdm_ioctlRspMsg_t) + sizeof(hal_adapter_info_t);
	rsp = malloc(rsp_len);
	if (rsp == NULL) 
		return(0);

	rc = sendAgileMsg(-1, &msg, sizeof(sdm_ioctlReqMsg_t),
					  "sdm", NFX_SSC_UNIX, SSC_SLOT, 0, &to,
					  rsp, rsp_len);
	if (rc <= 0 || 
		rsp->hdr.mid != SDM_MID_IOCTL_RSP ||
		rsp->eCode != SDM_OK) 
	{
		free(rsp);
		return(0);
	}

	free(rsp);
	return(1);
}

static int DoResetPort()
{
    sdm_ioctlReqMsg_t		msg;
	sdm_ioctlRspMsg_t *		rsp;
    char					respbuff[1024];
    int						rc;
	struct timeval			timeout = {5, 0};

	if (!IsStoragePortAlive(inSlot, inPort))
		return -1;

	memset(&msg, 0, sizeof(sdm_ioctlReqMsg_t));
	msg.hdr.mid = SDM_MID_IOCTL_REQ;
	msg.code = HAL_IOCTL_COMMON_RESET_ADAPTER;
	msg.slot = inSlot;
	msg.port = inPort;
	
	rc = sendAgileMsg(-1, &msg, sizeof(sdm_ioctlReqMsg_t),
					  "sdm", NFX_SSC_UNIX, SSC_SLOT, 0, &timeout,
					  respbuff, sizeof(respbuff));
	if (rc <= 0)
		return -1;

	rsp = (sdm_ioctlRspMsg_t *)respbuff;

	if (rsp->hdr.mid != SDM_MID_IOCTL_RSP)
		return -1;
	else if (rsp->eCode != SDM_OK)
		return -1;

	return 1;
}

static int DoEnablePort()
{
    sdm_ioctlReqMsg_t		msg;
	sdm_ioctlRspMsg_t *		rsp;
    char					respbuff[1024];
    int						rc;
	struct timeval			timeout = {5, 0};

	if (!IsStoragePortAlive(inSlot, inPort))
		return -1;

	memset(&msg, 0, sizeof(sdm_ioctlReqMsg_t));
	msg.hdr.mid = SDM_MID_IOCTL_REQ;
	msg.code = HAL_IOCTL_COMMON_ENABLE_ADAPTER;
	msg.slot = inSlot;
	msg.port = inPort;
	
	rc = sendAgileMsg(-1, &msg, sizeof(sdm_ioctlReqMsg_t),
					  "sdm", NFX_SSC_UNIX, SSC_SLOT, 0, &timeout,
					  respbuff, sizeof(respbuff));
	if (rc <= 0)
		return -1;

	rsp = (sdm_ioctlRspMsg_t *)respbuff;

	if (rsp->hdr.mid != SDM_MID_IOCTL_RSP)
		return -1;
	else if (rsp->eCode != SDM_OK)
		return -1;

	return 1;
}

static int DoDisablePort()
{
    sdm_ioctlReqMsg_t		msg;
	sdm_ioctlRspMsg_t *		rsp;
    char					respbuff[1024];
    int						rc;
	struct timeval			timeout = {5, 0};

	if (!IsStoragePortAlive(inSlot, inPort))
		return -1;

	memset(&msg, 0, sizeof(sdm_ioctlReqMsg_t));
	msg.hdr.mid = SDM_MID_IOCTL_REQ;
	msg.code = HAL_IOCTL_COMMON_DISABLE_ADAPTER;
	msg.slot = inSlot;
	msg.port = inPort;
	
	rc = sendAgileMsg(-1, &msg, sizeof(sdm_ioctlReqMsg_t),
					  "sdm", NFX_SSC_UNIX, SSC_SLOT, 0, &timeout,
					  respbuff, sizeof(respbuff));
	if (rc <= 0)
		return -1;

	rsp = (sdm_ioctlRspMsg_t *)respbuff;

	if (rsp->hdr.mid != SDM_MID_IOCTL_RSP)
		return -1;
	else if (rsp->eCode != SDM_OK)
		return -1;

	return 1;
}

static int HandlePortAction(char *newfname, int count, char **argname, char **argval)
{
	int res;

	getInputParams(count, argname, argval);

	if (!strcmp(inAct, "reset"))
	{
		res = DoResetPort();
		if (res <= 0)
			strcpy(newfname, "error_resetstgport.html");
	}
	else if (!strcmp(inAct, "enable"))
	{
		res = DoEnablePort();
		if (res <= 0)
			strcpy(newfname, "error_enstgport.html");
	}
	else if (!strcmp(inAct, "disable"))
	{
		res = DoDisablePort();
		if (res <= 0)
			strcpy(newfname, "error_disstgport.html");
	}
	return 1;
}


/***********************************************************
 * routines to retrieve storage information thru anpssc
 **********************************************************/

static int GetPortState(int slot, int port)
{
    sdm_ioctlReqMsg_t		msg;
	sdm_ioctlRspMsg_t *		adap_connection_state_rsp = 0;
	sdm_ioctlRspMsg_t *		rsp;
	int						rsp_len;
    int						rc;
	uint32					connection_state;
	int						state;
	struct timeval			timeout = {2, 0};

	state = PORT_STATE_UNKNOWN;
	/*
	 * Get PORT/LINK CONNECTION STATE
	 */
	memset(&msg, 0, sizeof(sdm_ioctlReqMsg_t));
	msg.hdr.mid = SDM_MID_IOCTL_REQ;
	msg.code = HAL_IOCTL_FC_CONNECTION_STATE;
	msg.slot = slot;
	msg.port = port;
	
	rsp_len = sizeof(sdm_ioctlRspMsg_t) + sizeof(uint32);
	rsp = adap_connection_state_rsp = malloc(rsp_len);
	if (rsp == NULL)
		return state;

	rc = sendAgileMsg(-1, &msg, sizeof(sdm_ioctlReqMsg_t),
					  "sdm", NFX_SSC_UNIX, SSC_SLOT, 0, &timeout,
					  rsp, rsp_len);
	if (rc < 0)
		goto done;
	else if (rc == 0)
		goto done;

	if (rsp->hdr.mid != SDM_MID_IOCTL_RSP)
		goto done;
	else if (rsp->eCode != SDM_OK)
		goto done;

	if (adap_connection_state_rsp->dataLen < sizeof(uint32))
		goto done;

	memcpy(&connection_state, adap_connection_state_rsp->data, sizeof(uint32));
	
	switch(connection_state) 
	{
	case HAL_IOCTL_FC_LINK_STATE_LINK_DOWN:
		state = PORT_STATE_DOWN;
		break;
	case HAL_IOCTL_FC_LINK_STATE_LINK_UP:
		state = PORT_STATE_UP;
		break;
	case HAL_IOCTL_FC_LINK_STATE_LINK_OFFLINE:
		state = PORT_STATE_DISABLE;
		break;
	default:
		break;
	}

 done:
	if (adap_connection_state_rsp)
		free(adap_connection_state_rsp);
	
	return state;
}

static void GetPortList()
{
	int i;

	portCount = 0;
	// realPortCount = 0;

	portList = malloc(portCount * sizeof(storage_port_t));
	if (portList == NULL)
		return;

	portCount = FCNIM_PORT_COUNT;
	memset(portList, 0, portCount * sizeof(storage_port_t));

	for (i = 0; i < portCount; i++) 
	{
		portList[i].slot = FCNIM_SLOT_NUM;
		portList[i].port = i;
		strcpy(portList[i].desc, "Fibre Channel Port");
		portList[i].state = GetPortState(FCNIM_SLOT_NUM, i);
#if 0
		if (portList[i].state != PORT_STATE_UNKNOWN)
			realPortCount++;
#endif
	}
}

static int getTableRowCount(void)
{
	return FCNIM_PORT_COUNT;
	// return realPortCount;
}

static int getTableRowData(int index, storage_port_t *data)
{
	if ((index < 1) || (index > dataCount) || (portList == NULL))
		return -1;

	memcpy(data, &portList[index-1], sizeof(storage_port_t));
	return 1;
#if 0
int i, j;
	j = 0;
	for (i = 0; i < portCount; i++) {
		if (portList[i].state != PORT_STATE_UNKNOWN) {
			if (j == (*index-1)) {
				memcpy(data, &portList[i], sizeof(storage_port_t));
				return 0;
			}
			j++;
		}
	}
	// memcpy(data, &portList[*index-1], sizeof(storage_port_t));
	// return 0;

	return -1;		/* shouldn't reach here... */
#endif
}



/***********************************************************
 *  Main routines to process tags
 **********************************************************/

static int getTagValue(char *tag, char *value)
{
	int i, tagType;
	
	if ((dataCurr <= 0) || (dataCurr > dataCount))
		return -1;

	tagType = INVALID_TAG;
	for (i = 0; i < NUM_OF_TAGS; i++) {
		if (!strcmp(tag, tagTable[i].tagName)) {
			tagType = tagTable[i].tagType;
			break;
		}
	}
	if (tagType == INVALID_TAG)
		return -1;

	switch (tagType) 
	{
	case STGPORT_SLOT:
		sprintf(value, "%d", stData.slot);
		break;
	case STGPORT_PORT:
		sprintf(value, "%d", stData.port);
		break;
	case STGPORT_TYPE:
		strcpy(value, stData.desc);
		break;
	case STGPORT_STATE:
		if (stData.state != PORT_STATE_UNKNOWN)
			strcpy(value, portState_str[stData.state]);
		else
			strcpy(value, "Unknown");
		break;
	case STGPORT_SP:
		if (stData.state != PORT_STATE_UNKNOWN)
			sprintf(value,
				"<a href=\"/agilerequestprefix/PortDetail?slot=%d&port=%d\">%d/%d</a>",
				stData.slot, stData.port, stData.slot, stData.port);
		else
			sprintf(value, "%d/%d", stData.slot, stData.port);
		break;
	case STGPORT_ENABLE:
		if (stData.state != PORT_STATE_UNKNOWN)
			sprintf(value,
				"<a href=\"/agilerequestprefix/PortEnable?slot=%d&port=%d\">Enable</a>",
				stData.slot, stData.port);
		else
			sprintf(value, "&nbsp");
		break;
	case STGPORT_DISABLE:
		if (stData.state != PORT_STATE_UNKNOWN)
			sprintf(value,
				"<a href=\"/agilerequestprefix/PortDisable?slot=%d&port=%d\">Disable</a>",
				stData.slot, stData.port);
		else
			sprintf(value, "&nbsp");
		break;
	default:
		return -1;
	}

	return 1;
}


/*
 * actRptTagFunc()
 *	tag   - tag name
 *	value - replacement string for tag
 */
static int actRptTagFunc(tagCallbackParams_t *tp)
{
char *tag = tp->tag;
char *value = tp->value;

	if (!strcmp(tag, "TABLE_BEGIN")) {
		/* TABLE_BEGIN tag, need to see if there is any data */
		dataCount = getTableRowCount();
		if (dataCount > 0) {
			/* need to retrieve first table's data */
			dataCurr = 1;
			if (getTableRowData(dataCurr, &stData) >= 0)
				return 1;
		}

		dataCurr = 0;
		return -1;
	}
	else if (!strcmp(tag, "TABLE_END")) {
		/* TABLE_END tag, check if end of data */
		if ((dataCount <= 0) || (dataCurr <= 0) || (dataCurr >= dataCount))
			return -1;

		dataCurr++;
		if (getTableRowData(dataCurr, &stData) < 0)
			return -1;
		else
			return 1;
	}
	else {
		/* other tags */
		if (dataCount <= 0) {
			dataCount = getTableRowCount();
			if (dataCount <= 0)
				return -1;
		}

		if ((dataCurr < 1) || (dataCurr > dataCount)) {
			dataCurr = 1;
			if (getTableRowData(dataCurr, &stData) < 0)
				return -1;
		}

		return getTagValue(tag, value);
	}

	/* shouldn't reach here... */
	return -1;
}

int actionPortMain(void *r, int count, char **argname, char **argval)
{
char newfname[128];

	memset(newfname, 0, sizeof(newfname));

	/* Handle Reset/Enable/Disable Port actions */
	if (count > 0)
		HandlePortAction(newfname, count, argname, argval);

	prepare_agile_template(r, newfname);

	/* initialize storage information variables */
	dataCount = 0;
	dataCurr = 0;
	portList = NULL;
	GetPortList();

	agile_scan_and_send(r, actRptTagFunc);

	if (portList != NULL)
		free(portList);
	return 1;
}
