/* agilepost.c */

#include <unistd.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>

#include <time.h>
#include "httpd.h"
#include "http_request.h"
#include "http_log.h"
#include "http_protocol.h"
#include "http_main.h"

#include "agilepost.h"

#define MAX_ACTSTR_SIZE		32
#define MAX_TMPLSTR_SIZE	128

void convertUrlCharToSpecChar(char *s);

typedef struct _agileActions_t {
	int		actioncode;
	char	actionstr[MAX_ACTSTR_SIZE];
#if 0
/* template name removed. actionstr above imply the template name */
/* For example, if actionstr is "InterfaceReport", then the name  */
/* of the action template is "InterfaceReport.html"               */
	char	actiontmpl[MAX_TMPLSTR_SIZE];
#endif
} agileActions_t;


/* Action code definition */
#define ACT_INTERFACEMAIN		1	
#define ACT_INTERFACEDETAIL		2		
#define ACT_INTERFACEADD		3		
#define ACT_INTERFACEDEL		4		
#define ACT_INTERFACEMOD		5		
#define ACT_INTERFACESTAT		6		
#define ACT_ROUTEMAIN			7		
#define ACT_ROUTEADD			8
#define ACT_ROUTEDEL			9
#define ACT_LUNMAIN				10
#define ACT_LUNDETAIL			11
#define ACT_LUNSTAT				12
#define ACT_LUNLABEL			13
#define ACT_LUNRESET			14
#define ACT_DISKMAIN			15
#define ACT_DISKLSTLUN			16
#define ACT_DISKFOLUN			17
#define ACT_DISKVOLADD			18
#define ACT_DISKVOLADDACT		19
#define ACT_PORTMAIN			20
#define ACT_PORTDETAIL			21
#define ACT_PORTSTAT			22
#define ACT_PORTRESET			23
#define ACT_PORTENABLE			24
#define ACT_PORTDISABLE			25
#define ACT_NWSTATS				26		
#define ACT_NWSSCCONNECT		27		
#define ACT_NWFPCONNECT			28		
#define ACT_VOLMAIN				29
#define ACT_VOLDETAIL			30
#define ACT_VOLADD				31
#define ACT_VOLDEL				32
#define ACT_VOLMOD				33
#define ACT_SSHOTMAIN			34
#define ACT_SSHOTSCH			35
#define ACT_SSHOTADD			36
#define ACT_SSHOTDEL			37
#define ACT_NFSMAIN				38
#define ACT_NFSDEL				39
#define ACT_NFSADD				40
#define ACT_NFSMOD				41
#define ACT_NFSADDHOST			42
#define ACT_NFSDELHOST			43
#define ACT_NFSMODHOST			44
#define ACT_TAPEMAIN			45
#define ACT_TAPEDETAIL			46
#define ACT_TAPESTAT			47
#define ACT_TAPERESET			48
#define ACT_FSETMAIN			49
#define ACT_FSETADD				50
#define ACT_FSETDEL				51
#define ACT_FSETMOD				52
#define ACT_FSETFUSG			53
#define ACT_FSETUGUSG			54
#define ACT_FSETFQUO			55
#define ACT_FSETUGQUO			56
#define ACT_FSETDQUO			57
#define ACT_FSETFQUOMOD			58
#define ACT_FSETUGQUOMOD		59
#define ACT_FSETDQUOMOD			60
#define ACT_NETWORKARP			61		
#define ACT_NWMANARP			62		
#define ACT_NWADDARP			63		
#define ACT_NWDELARP			64
#define ACT_NSMAIN				65
#define	ACT_NSADD				66
#define ACT_NSMOD				67
#define ACT_NSMODDEF			68
#define ACT_NSDEL				69
#define ACT_NDMPACTSESS			70
#define ACT_NDMPSEDETAIL		71
#define ACT_NDMPSESTATUS		72
#define ACT_NDMPSEDEL			73
#define ACT_NDMPSEDELALL		74
#define ACT_NDMPEN				75
#define ACT_NDMPDIS				76
#define ACT_SYSWZ1				77
#define ACT_SYSWZ2				78
#define ACT_SYSWZ3				79
#define ACT_SYSWZ4				80
#define ACT_SYSWZ5				81
#define ACT_SYSWZSETTIME		82
#define ACT_SYSWZSAVE			83
#define ACT_FILERMAIN			84
#define ACT_FILERSYSOP			85
#define ACT_FILERSLOTREBOOT		86
#define ACT_FILERSLOT			87
#define ACT_FILERCPUREBOOT		88
#define ACT_ADMINMAIN			89
#define ACT_BANNER				90
#define ACT_MENU				91
#define ACT_LOGIN				92
#define TOTAL_ACTIONS			92		

#define ACT_INVALID				-1


int parse_args(char **, char *, char *);
int GetUserPrivilege();


/* Buffer for reading HTML template */
char buf[MAX_AGILETEMPLATE_SIZE];


static agileActions_t agileActions[TOTAL_ACTIONS+1] =
{
	{ACT_INTERFACEMAIN,		"InterfaceMain"},
	{ACT_INTERFACEDETAIL,	"InterfaceDetail"},
	{ACT_INTERFACEADD,		"InterfaceAdd"},
	{ACT_INTERFACEDEL,		"InterfaceDel"},
	{ACT_INTERFACEMOD,		"InterfaceMod"},
	{ACT_INTERFACESTAT,		"InterfaceStat"},
	{ACT_ROUTEMAIN,			"RouteMain"},			
	{ACT_ROUTEADD,			"RouteAdd"},	
	{ACT_ROUTEDEL,			"RouteDel"},
	{ACT_LUNMAIN,			"LunMain"},
	{ACT_LUNDETAIL,			"LunDetail"},
	{ACT_LUNSTAT,			"LunStat"},
	{ACT_LUNLABEL,			"LunLabel"},
	{ACT_LUNRESET,			"LunReset"},
	{ACT_DISKMAIN,			"DiskMain"},
	{ACT_DISKLSTLUN,		"DiskLuns"},
	{ACT_DISKFOLUN,			"DiskFOLuns"},
	{ACT_DISKVOLADD,		"DiskAddVol"},
	{ACT_DISKVOLADDACT,		"DiskAddVolAct"},
	{ACT_PORTMAIN,			"PortMain"},
	{ACT_PORTDETAIL,		"PortDetail"},
	{ACT_PORTSTAT,			"PortStat"},
	{ACT_PORTRESET,			"PortReset"},
	{ACT_PORTENABLE,		"PortEnable"},
	{ACT_PORTDISABLE,		"PortDisable"},
	{ACT_NWSTATS,			"NetworkStats"},
	{ACT_NWSSCCONNECT,		"NetworkConnSSC"},
	{ACT_NWFPCONNECT,		"NetworkConnFP"},
	{ACT_VOLMAIN,			"VolumeMain"},
	{ACT_VOLDETAIL,			"VolumeDetail"},
	{ACT_VOLADD,			"VolumeAdd"},
	{ACT_VOLDEL,			"VolumeDel"},
	{ACT_VOLMOD,			"VolumeMod"},
	{ACT_SSHOTMAIN,			"SnapshotMain"},
	{ACT_SSHOTSCH,			"SnapshotSchedule"},
	{ACT_SSHOTADD,			"SnapshotAdd"},
	{ACT_SSHOTDEL,			"SnapshotDel"},
	{ACT_NFSMAIN,			"NfsMain"},
	{ACT_NFSDEL,			"NfsDel"},
	{ACT_NFSADD,			"NfsAdd"},
	{ACT_NFSMOD,			"NfsMod"},
	{ACT_NFSADDHOST,		"NfsAddHost"},
	{ACT_NFSDELHOST,		"NfsDelHost"},
	{ACT_NFSMODHOST,		"NfsModHost"},
	{ACT_TAPEMAIN,			"TapeMain"},
	{ACT_TAPEDETAIL,		"TapeDetail"},
	{ACT_TAPESTAT,			"TapeStat"},
	{ACT_TAPERESET,			"TapeReset"},
	{ACT_FSETMAIN,			"FilesetMain"},
	{ACT_FSETADD,			"FilesetAdd"},
	{ACT_FSETDEL,			"FilesetDel"},
	{ACT_FSETMOD,			"FilesetMod"},
	{ACT_FSETFUSG,			"FilesetFUsg"},
	{ACT_FSETUGUSG,			"FilesetUGUsg"},
	{ACT_FSETFQUO,			"FilesetFQuo"},
	{ACT_FSETUGQUO,			"FilesetUGQuo"},
	{ACT_FSETDQUO,			"FilesetDQuo"},
	{ACT_FSETFQUOMOD,		"FilesetFQuoMod"},
	{ACT_FSETUGQUOMOD,		"FilesetUGQuoMod"},
	{ACT_FSETDQUOMOD,		"FilesetDQuoMod"},
	{ACT_NETWORKARP,		"NetworkARP"},
	{ACT_NWMANARP,			"NetworkManageArp"},
	{ACT_NWADDARP,			"NetworkAddArp"},
	{ACT_NWDELARP,			"NetworkDelArp"},
	{ACT_NSMAIN,			"NameSvcMain"},
	{ACT_NSADD,				"NameSvcAdd"},
	{ACT_NSMOD,				"NameSvcMod"},
	{ACT_NSMODDEF,			"NameSvcModDef"},
	{ACT_NSDEL,				"NameSvcDel"},
	{ACT_NDMPACTSESS,		"NdmpActSess"},
	{ACT_NDMPSEDETAIL,		"NdmpSessDetail"},
	{ACT_NDMPSESTATUS,		"NdmpSessStatus"},
	{ACT_NDMPSEDEL,			"NdmpSessDel"},
	{ACT_NDMPSEDELALL,		"NdmpSessDelAll"},
	{ACT_NDMPEN,			"NdmpEnable"},
	{ACT_NDMPDIS,			"NdmpDisable"},
	{ACT_SYSWZ1,			"SysSetWizard1"},
	{ACT_SYSWZ2,			"SysSetWizard2"},
	{ACT_SYSWZ3,			"SysSetWizard3"},
	{ACT_SYSWZ4,			"SysSetWizard4"},
	{ACT_SYSWZ5,			"SysSetWizard5"},
	{ACT_SYSWZSETTIME,		"SysSetWizSetTime"},
	{ACT_SYSWZSAVE,			"SysSetWizSaveRes"},
	{ACT_FILERMAIN,			"FilerMain"},
	{ACT_FILERSYSOP,		"FilerSysOp"},
	{ACT_FILERSLOTREBOOT,	"FilerSlotReboot"},
	{ACT_FILERSLOT,			"FilerSlot"},
	{ACT_FILERCPUREBOOT,	"FilerCpuReboot"},
	{ACT_ADMINMAIN,			"AdminMain"},
	{ACT_BANNER,			"Banner"},
	{ACT_MENU,				"Menu"},
	{ACT_LOGIN,				"Login"},
	{ACT_INVALID, ""}
};


int prepare_agile_template(void *req, char *newfname)
{
FILE *f;
int errstatus, rangestatus;
request_rec *r = (request_rec *) req;

	/* whether a new filename is necessary? */
	if (newfname && strlen(newfname)) {
		char *cp;

		cp = r->filename + strlen(r->filename) - 1;
		while ((*cp != '/') && (cp != r->filename)) cp--;
		strcpy(cp+1, newfname);
	}

	/* initialize the HTTP response */
	f = ap_pfopen(r->pool, r->filename, "r");
	if (f == NULL) {
		ap_log_rerror(APLOG_MARK, APLOG_ERR, r,
				"file permissions deny server access: %s", r->filename);
		return FORBIDDEN;
	}
	r->rspfhandle = f;

	ap_update_mtime(r, time(NULL));
	ap_set_last_modified(r);
	ap_set_etag(r);
	ap_table_setn(r->headers_out, "Accept-Ranges", "bytes");
	if ((errstatus = ap_meets_conditions(r)) != OK) {
		return errstatus;
	}

	rangestatus = ap_set_byterange(r);
	r->content_type = "text/html";

	ap_send_http_header(r);

	return OK;
}

int process_agile_getpost(void *req, char *parambuf)
{
int i, fnlen, rv, actcode, totalparams;
char namebuf[MAX_NUM_PARAMS][MAX_PARAMNAME_SIZE];
char valbuf[MAX_NUM_PARAMS][MAX_PARAMVAL_SIZE];
char *argname[MAX_NUM_PARAMS];
char *argval[MAX_NUM_PARAMS];
request_rec *r = (request_rec *) req;

// fprintf(stderr, "process_agile_getpost()\n");
// fprintf(stderr, "r->uri: %s\n", r->uri);
// fprintf(stderr, "r->args: %s\n", r->args);

	/* parse the parameter into param array */
	totalparams = 0;
	r->rspfhandle = NULL;

	if (r->args && *(r->args)) {
		char *cptr = r->args;

		for (i = 0; i < MAX_NUM_PARAMS; i++) {
			namebuf[i][0] = '\0';
			valbuf[i][0] = '\0';
			argname[i] = namebuf[i];
			argval[i]  = valbuf[i];
			if (parse_args(&cptr, argname[i], argval[i]) < 0)
				break;
			totalparams++;
		}
	}
	if (parambuf && parambuf[0]) {
		char *cptr = parambuf;

		for (i = totalparams; i < MAX_NUM_PARAMS; i++) {
			namebuf[i][0] = '\0';
			valbuf[i][0] = '\0';
			argname[i] = namebuf[i];
			argval[i]  = valbuf[i];
			if (parse_args(&cptr, argname[i], argval[i]) < 0)
				break;
			totalparams++;
		}
	}

	/* change URI into no agilerequestprefix prefix */
	strcpy(r->uri, r->uri + strlen(PREFIX_AGILEGET));

// fprintf(stderr, "before r->filename: %s\n", r->filename);
	actcode = ACT_INVALID;
	for (i = 0; i < TOTAL_ACTIONS; i++) {
		if (!strcmp(r->uri+1, agileActions[i].actionstr)) {

			actcode = agileActions[i].actioncode;

			/* fix the URI */
#if 0
			strcpy(r->uri+1, agileActions[i].actiontmpl);
#endif
			strcpy(r->uri+1, agileActions[i].actionstr);
			strcat(r->uri, ".html");

			/* fix the filename */
			fnlen = strlen(r->filename);
			r->filename[fnlen-strlen(PREFIX_AGILEGET)+1] = '\0';
#if 0
			strcat(r->filename, agileActions[i].actiontmpl);
#endif
			strcat(r->filename, agileActions[i].actionstr);
			strcat(r->filename, ".html");
			break;
		}
	}

	rv = NOT_IMPLEMENTED;
	if (actcode == ACT_INVALID)
		return rv;

	/* Now call the proper agile GET/POST function */
	/* The parameters (args) are now in argname[][] and argval[][] */
	/* array. All are in string format. If you need integer value  */
	switch (actcode) {
	case ACT_INTERFACEMAIN:				
		rv = actionInterfaceMain(r, totalparams, argname, argval);	
		break;								
	case ACT_INTERFACEDETAIL:				
		rv = actionInterfaceDetail(r, totalparams, argname, argval);	
		break;								
	case ACT_INTERFACEADD:				
		rv = actionInterfaceAdd(r, totalparams, argname, argval);	
		break;								
	case ACT_INTERFACEDEL:				
		rv = actionInterfaceDel(r, totalparams, argname, argval);	
		break;								
	case ACT_INTERFACEMOD:				
		rv = actionInterfaceMod(r, totalparams, argname, argval);	
		break;								
#if 0
	case ACT_INTERFACESTAT:				
		rv = actionInterfaceStat(r, totalparams, argname, argval);	
		break;								
#endif

	case ACT_ROUTEMAIN:					
		rv = actionRouteMain(r, totalparams, argname, argval);		
		break;								
	case ACT_ROUTEADD:					
		rv = actionRouteAdd(r, totalparams, argname, argval);			
		break;								
	case ACT_ROUTEDEL:					
		rv = actionRouteDel(r, totalparams, argname, argval);			
		break;								

	case ACT_LUNMAIN:				
		rv = actionLunMain(r, totalparams, argname, argval);		
		break;								
	case ACT_LUNDETAIL:				
		rv = actionLunDetail(r, totalparams, argname, argval);	
		break;								
	case ACT_LUNSTAT:				
		rv = actionLunStat(r, totalparams, argname, argval);	
		break;								
	case ACT_LUNLABEL:				
		rv = actionLunLabel(r, totalparams, argname, argval);	
		break;								
	case ACT_LUNRESET:					
		rv = actionLunReset(r, totalparams, argname, argval);	
		break;								

	case ACT_DISKMAIN:					
		rv = actionDiskMain(r, totalparams, argname, argval);	
		break;
	case ACT_DISKLSTLUN:					
		rv = actionDiskLuns(r, totalparams, argname, argval);	
		break;
	case ACT_DISKFOLUN:
		rv = actionDiskLuns(r, totalparams, argname, argval);	
		break;
	case ACT_DISKVOLADD:					
		rv = actionDiskAddVol(r, totalparams, argname, argval);	
		break;								
	case ACT_DISKVOLADDACT:					
		rv = actionDiskAddVolAct(r, totalparams, argname, argval);	
		break;

	case ACT_PORTMAIN:					
		rv = actionPortMain(r, totalparams, argname, argval);	
		break;								
	case ACT_PORTDETAIL:					
		rv = actionPortDetail(r, totalparams, argname, argval);	
		break;
	case ACT_PORTSTAT:					
		rv = actionPortStat(r, totalparams, argname, argval);	
		break;
	case ACT_PORTRESET:					
		rv = actionPortReset(r, totalparams, argname, argval);	
		break;
	case ACT_PORTENABLE:					
		rv = actionPortEnable(r, totalparams, argname, argval);	
		break;
	case ACT_PORTDISABLE:					
		rv = actionPortDisable(r, totalparams, argname, argval);	
		break;								

	case ACT_NWSTATS:
		rv = actionNwStat(r);			
		break;								
	case ACT_NWSSCCONNECT:					
		rv = actionNwConnSsc(r, totalparams, argname, argval);	
		break;								
	case ACT_NWFPCONNECT:					
		rv = actionNwConnFp(r, totalparams, argname, argval);		
		break;								

	case ACT_VOLMAIN:
		rv = actionVolMain(r, totalparams, argname, argval);		
		break;								
	case ACT_VOLDETAIL:
		rv = actionVolDetail(r, totalparams, argname, argval);		
		break;								
	case ACT_VOLADD:
		rv = actionVolAdd(r);		
		break;								
	case ACT_VOLDEL:
		rv = actionVolDelete(r, totalparams, argname, argval);		
		break;								
	case ACT_VOLMOD:
		rv = actionVolModify(r, totalparams, argname, argval);		
		break;								

	case ACT_SSHOTMAIN:
		rv = actionSshotMain(r, totalparams, argname, argval);		
		break;								
	case ACT_SSHOTADD:
		rv = actionSshotAdd(r, totalparams, argname, argval);		
		break;								
	case ACT_SSHOTDEL:
		rv = actionSshotDel(r, totalparams, argname, argval);		
		break;								

	case ACT_NFSMAIN:
		rv = actionNfsMain(r, totalparams, argname, argval);		
		break;			
	case ACT_NFSDEL:
		rv = actionNfsDel(r, totalparams, argname, argval);		
		break;
	case ACT_NFSADD:
	case ACT_NFSMOD:
		rv = actionNfsAdd(r, totalparams, argname, argval);		
		break;
	case ACT_NFSADDHOST:
		rv = actionNfsAddHost(r, totalparams, argname, argval);		
		break;
	case ACT_NFSDELHOST:
		rv = actionNfsDelHost(r, totalparams, argname, argval);		
		break;
	case ACT_NFSMODHOST:
		rv = actionNfsModHost(r, totalparams, argname, argval);		
		break;								

	case ACT_TAPEMAIN:
		rv = actionLunMain(r, totalparams, argname, argval);		
		break;		
	case ACT_TAPEDETAIL:
		rv = actionLunDetail(r, totalparams, argname, argval);		
		break;			
	case ACT_TAPESTAT:
		rv = actionLunStat(r, totalparams, argname, argval);		
		break;
	case ACT_TAPERESET:
		rv = actionLunReset(r, totalparams, argname, argval);		
		break;
		
	case ACT_FSETMAIN:
		rv = actionFsetMain(r, totalparams, argname, argval);		
		break;
	case ACT_FSETADD:
		rv = actionFsetAdd(r, totalparams, argname, argval);		
		break;
	case ACT_FSETDEL:
		rv = actionFsetDel(r, totalparams, argname, argval);		
		break;
	case ACT_FSETMOD:
		rv = actionFsetMod(r, totalparams, argname, argval);		
		break;
	case ACT_FSETFUSG:
		rv = actionFsetFUsg(r, totalparams, argname, argval);		
		break;
	case ACT_FSETUGUSG:
		rv = actionFsetUGUsg(r, totalparams, argname, argval);		
		break;
	case ACT_FSETFQUO:
		rv = actionFsetFQuo(r, totalparams, argname, argval);		
		break;
	case ACT_FSETUGQUO:
		rv = actionFsetUGQuo(r, totalparams, argname, argval);		
		break;
	case ACT_FSETDQUO:
		rv = actionFsetDQuo(r, totalparams, argname, argval);		
		break;
	case ACT_FSETFQUOMOD:
		rv = actionFsetFQuoMod(r, totalparams, argname, argval);		
		break;
	case ACT_FSETUGQUOMOD:
		rv = actionFsetUGQuoMod(r, totalparams, argname, argval);		
		break;
	case ACT_FSETDQUOMOD:
		rv = actionFsetDQuoMod(r, totalparams, argname, argval);		
		break;

	case ACT_NSMAIN:						
		rv = actionNsMain(r, totalparams, argname, argval);			
		break;								
	case ACT_NSADD:						
		rv = actionNsAdd(r, totalparams, argname, argval);			
		break;								
	case ACT_NSMOD:						
	case ACT_NSMODDEF:						
		rv = actionNsMod(r, totalparams, argname, argval);			
		break;								
	case ACT_NSDEL:						
		rv = actionNsDel(r, totalparams, argname, argval);			
		break;								

	case ACT_NDMPACTSESS:						
		rv = actionNdmpActSess(r, totalparams, argname, argval);			
		break;								
	case ACT_NDMPSEDETAIL:						
		rv = actionNdmpSessDetail(r, totalparams, argname, argval);			
		break;
	case ACT_NDMPSESTATUS:
		rv = actionNdmpSessStatus(r, totalparams, argname, argval);			
		break;		
	case ACT_NDMPSEDEL:						
	case ACT_NDMPSEDELALL:						
		rv = actionNdmpSessDel(r, totalparams, argname, argval);			
		break;								
	case ACT_NDMPEN:						
	case ACT_NDMPDIS:						
		rv = actionNdmpEnable(r);			
		break;								

	case ACT_NETWORKARP:					
		rv = actionNetworkARP(r);			
		break;								
	case ACT_NWMANARP:						
		rv = actionNwManARP(r, totalparams, argname, argval);			
		break;								
	case ACT_NWADDARP:						
		rv = actionNwAddARP(r, totalparams, argname, argval);			
		break;								
	case ACT_NWDELARP:						
		rv = actionNwDelARP(r, totalparams, argname, argval);			
		break;								

	case ACT_SYSWZ1:						
		rv = actionSysWizard1(r, totalparams, argname, argval);			
		break;								
	case ACT_SYSWZ2:						
		rv = actionSysWizard2(r, totalparams, argname, argval);			
		break;								
	case ACT_SYSWZ3:						
		rv = actionSysWizard3(r, totalparams, argname, argval);			
		break;								
	case ACT_SYSWZ4:						
		rv = actionSysWizard4(r, totalparams, argname, argval);			
		break;								
	case ACT_SYSWZ5:						
		rv = actionSysWizard5(r, totalparams, argname, argval);			
		break;
	case ACT_SYSWZSETTIME:				
		rv = actionSysWizardSetTime(r, totalparams, argname, argval);			
		break;
	case ACT_SYSWZSAVE:
		rv = actionSysWizardSaveRes(r);			
		break;

	case ACT_FILERMAIN:
		rv = actionFilerMain(r, totalparams, argname, argval);			
		break;
	case ACT_FILERSYSOP:
		rv = actionFilerSysOp(r, totalparams, argname, argval);			
		break;
	case ACT_FILERSLOTREBOOT:
		rv = actionFilerSlotReboot(r, totalparams, argname, argval);			
		break;
	case ACT_FILERSLOT:
		rv = actionFilerSlot(r, totalparams, argname, argval);			
		break;
	case ACT_FILERCPUREBOOT:
		rv = actionFilerCpuReboot(r, totalparams, argname, argval);			
		break;

	case ACT_ADMINMAIN:
		rv = actionAdminMain(r, totalparams, argname, argval);			
		break;

	case ACT_BANNER:
		rv = actionBanner(r);			
		break;

	case ACT_MENU:
		rv = actionMenu(r);			
		break;		

	case ACT_LOGIN:
		rv = actionLogin(r, totalparams, argname, argval);			
		break;		

	case ACT_INVALID:
	default:
		rv = NOT_IMPLEMENTED;
		break;
	}

	/* closing up the HTTP response */
	ap_kill_timeout(r);
	do {
		if (r->sent_bodyct)
			ap_bgetopt(r->connection->client, BO_BYTECT, &r->bytes_sent);
	} while (0);

	if (r->rspfhandle) {
		ap_pfclose(r->pool, r->rspfhandle);
		r->rspfhandle = NULL;
	}

	return rv;
}


/* find_tag()
 * strbuf  - buffer contains string data
 * offset  - search starting offset
 * strsize - end-of-buffer position
 * tagstr  - return the tag string here
 *
 * return: the starting of tag position (point to '<' character)
 */
static int find_tag(char *strbuf, int offset, int strsize, char *tagstr)
{
int i, j;

	if (tagstr)
		tagstr[0] = '\0';

	for (i = offset; i < strsize-1; i++) {
		if ((strbuf[i] == '<') && (strbuf[i+1] == '@')) {
			/* seems like a tag, now look for tag closure */
			for (j = i+2; j < strsize-1; j++) {
				if ((strbuf[j] == '@') && (strbuf[j+1] == '>')) {
					if (tagstr) {
						int taglen = j - i - 2;
						strncpy(tagstr, &strbuf[i+2], taglen);
						tagstr[taglen] = '\0';
					}
					return i;
				}
			}
			/* no tag end characters at all */
			return -1;
		}
	}

	/* not found */
	return -1;
}


int dump_to_client(void *req, char *databuf, int datalen)
{
request_rec *r = (request_rec *) req;
int o, n, w;

	o = 0;
	n = datalen;

	while ((n > 0) && !r->connection->aborted) {
		w = ap_bwrite(r->connection->client, &databuf[o], n);
		if (w > 0) {
			ap_reset_timeout(r); /* reset timeout after write */
			n -= w;
			o += w;
		}
		else if (w < 0) {
			if (!r->connection->aborted) {
				ap_log_rerror(APLOG_MARK, APLOG_INFO, r,
				 "client stopped connection before send body completed");
				ap_bsetflag(r->connection->client, B_EOUT, 1);
				r->connection->aborted = 1;
			}
			return -1;
		}
	}
	return o;
}


int GetUserPrivilege()
{
	/* we need to provide user privilege check... */
	return 4;
}


/* agile_scan_and_send()
 *   f         - template file stream
 *   r         - http request record
 *   callback  - callback function when found agile tag
 */
int agile_scan_and_send(void *req, tagCallbackFunc_t tagcb_func)
{
	char tagstr[MAX_TAGNAME_SIZE];
	char replace[MAX_REPLACESTR_LEN];
	long total_bytes_sent = 0;
	tagCallbackParams_t tagParams;
	int n, o, nr;
	int tablebegin, tableend;
	request_rec *r = (request_rec *) req;
	FILE *f = r->rspfhandle;

	ap_soft_timeout("send body", r);

	if (r->connection->aborted)
		goto scan_send_finished;

	/* read until we got something in buffer */
	while ((nr = fread(buf, sizeof(char), MAX_AGILETEMPLATE_SIZE, f)) < 1
		   && ferror(f) && errno == EINTR && !r->connection->aborted)
		continue;

	/* for now, we assume our buffer size is big   */
	/* enough for the template. So, we should have */
	/* end-of-file flag set after above fread().   */
	if ((n < 1) && !feof(f))
		goto scan_send_finished;

	tablebegin = -1;
	tableend = -1;
	o = 0;
	while (o < nr) {

		tagstr[0] = '\0';
		n = find_tag(buf, o, nr, tagstr);
		if (n < 0)
			n = nr; /* tag not found, dump the rest */

		/* dump whatever before tag to client */
		if (dump_to_client(req, &buf[o], n-o) < 0)
			goto scan_send_finished;
		else
			total_bytes_sent += (n-o);

		/* point o to next starting point */
		o = n + strlen(tagstr) + 4; /* strlen("<@@>") == 4 */

		/* no more tag? above dump_to_client() should */
		/* have output everything to client already.. */
		if (tagstr[0] == '\0')
			continue;
		
		/* A tag is found. Now, process the tag */

		if (!strncmp(tagstr, "TABLE_BEGIN", 11)) {
			/* A TABLE_BEGIN tag */
			tablebegin = o;
			/* just tell callback func that a TABLE_BEGIN */
			/* tag is found. No replacement is allowed    */
			memset(&tagParams, 0, sizeof(tagParams));
			tagParams.req = req;
			tagParams.valueSize = MAX_REPLACESTR_LEN;
			tagParams.tag = tagstr;
			if ((*tagcb_func)(&tagParams) < 0) {
				/* no more table row entry available */
				if (tableend != -1) {
					/* skip to TABLE_END */
					o = tableend;
					tableend = -1;
				}
				else {
					/* TABLE_END is not defined */
					/* there isn't any entry... */
					do {
						n = find_tag(buf, o, nr, tagstr);
						o = n + strlen(tagstr) + 4;
					} while ((o < nr) && strncmp(tagstr, "TABLE_END", 9));
				}
			}
		}
		else if (!strncmp(tagstr, "TABLE_END", 9)) {
			tableend = o;
			/* just tell callback func that a TABLE_END   */
			/* tag is found. No replacement is allowed    */
			/* Expecting to callback func return status,  */
			/* that is, whether to continue for the next  */
			/* round of entry row...                      */
			memset(&tagParams, 0, sizeof(tagParams));
			tagParams.req = req;
			tagParams.valueSize = MAX_REPLACESTR_LEN;
			tagParams.tag = tagstr;
			if ((*tagcb_func)(&tagParams) < 0) {
				/* no more row data now based on callback */
				/* func return value. So, no need to loop */
				/* back to TABLE_BEGIN.                   */
				/* no change to variable o (offset)       */
				tableend = -1;	// reset tableend value
			}
			else {
				/* loop back to TABLE_BEGIN */
				o = tablebegin;
			}
		}
		else if (!strncmp(tagstr, "PRIV_BEGIN", strlen("PRIV_BEGIN"))) {
			char *p = &tagstr[10];
			int input_priv;

			input_priv = tagstr[11] - '0';
			if (((*p == '>') && (GetUserPrivilege() <= input_priv)) ||
				((*p == '=') && (GetUserPrivilege() != input_priv)) ||
				((*p == '<') && (GetUserPrivilege() >= input_priv))) {
				/* no privilege, skip the whole privileged block */
				do {
					n = find_tag(buf, o, nr, tagstr);
					o = n + strlen(tagstr) + 4;
				} while ((o < nr) && strncmp(tagstr, "PRIV_END", 8));
			}
		}
		else {
			memset(&tagParams, 0, sizeof(tagParams));
			tagParams.req = req;
			tagParams.valueSize = MAX_REPLACESTR_LEN;
			tagParams.tag = tagstr;
			tagParams.value = replace;
			replace[0] = '\0';
			((*tagcb_func)(&tagParams));
			if (dump_to_client(req, replace, strlen(replace)) < 0) {
				/* client is closed. Stop dumping... */
				goto scan_send_finished;
			}
			else
			{
				total_bytes_sent +=
					(tagParams.bytesAlreadyDumped + strlen(replace));
			}
		}
	}

scan_send_finished:
	return total_bytes_sent;
}


/* parse_args()
 *   argpos    - IN: staring position
 *               OUT: next starting position or at the end of arg string
 *   argname   - the name of the arg parsed
 *   argvaule  - the value of the arg parsed
 */
int parse_args(char **argpos, char *argname, char *argvalue)
{
char *pSrc, *pDst;

	pSrc = *argpos;
	if (!pSrc || *pSrc == '\0')
		return -1;

	/* retrieve the arg name first */
	pDst = argname;
	while ((*pSrc != '\0') && (*pSrc != '=')) {
#if 0
/* no need to do this as we now do convertion below */
		/* check if this is %20 for blank character */
		if (*pSrc == '%') {
			if ( (*(pSrc+1) == '2') && (*(pSrc+2) == '0') ) {
				if (pDst && ((pDst - argname) < MAX_PARAMNAME_SIZE))
					*pDst++ = ' ';
				pSrc += 3;
				continue;
			}
		}
#endif
		if (pDst && ((pDst - argname) < MAX_PARAMNAME_SIZE))
			*pDst++ = *pSrc;
		pSrc++;
	}
	if (pDst)
		*pDst = '\0';
	convertUrlCharToSpecChar(argname);

	if (*pSrc != '=') {
		/* value missing? Set value to nothing... */
		if (argvalue)
			*argvalue = '\0';

		if (*pSrc == '&')
			pSrc++;		/* point to next arg */

		return 0;
	}

	/* now parse value part of arg */
	pSrc++;	/* skip '=' character */
	pDst = argvalue;
	while ((*pSrc != '\0') && (*pSrc != '&')) {
#if 0
/* no need to do this as we now do convertion below */
		/* check if this is %20 for blank character */
		if (*pSrc == '%') {
			if ( (*(pSrc+1) == '2') && (*(pSrc+2) == '0') ) {
				pSrc += 3;
				if (pDst && ((pDst - argname) < MAX_PARAMVAL_SIZE))
					*pDst++ = ' ';
				continue;
			}
		}
#endif
		if (pDst && ((pDst - argvalue) < MAX_PARAMVAL_SIZE))
			*pDst++ = *pSrc;
		pSrc++;
	}
	if (pDst)
		*pDst = '\0';
	convertUrlCharToSpecChar(argvalue);

	/* now we are done with arg name and value */
	if (*pSrc == '&')
		pSrc++;		/* point to next arg */

	*argpos = pSrc;
	return 0;
}


void GetSystemHostInfo(void *req, char *ip, char *host)
{
request_rec *r = (request_rec *) req;
conn_rec *conn = r->connection;

	strcpy(ip, inet_ntoa(conn->local_addr.sin_addr));
	gethostname(host, 80);
}

u_int32_t GetHttpClientIP(void *req)
{
request_rec *r = (request_rec *) req;
conn_rec *conn = r->connection;

	return (conn->remote_addr.sin_addr.s_addr);
}

