/*  File Name  :    agilesyswz3.c 
 *
 *  Description:    HTTPD action functions for System Setup Wizard step 3.
 *
 *  Created    :    4/9/2002
 *
 *  Author     :    Tiffany Chiang
 *
 *  Copyright(c)  Agile Storage, Inc. 2001
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <anpssc-api.h>
#include "../agent/httpdagent.h"
#include <nfxnis-api.h>

#include "agilepost.h"
#include "agileutil.h"

#define INVALID_TAG			0
#define SYS_DNSDOMAIN		1
#define SYS_DNSSVR1			2
#define SYS_DNSSVR2			3
#define SYS_DNSSVR3			4
#define SYS_NISDOMAIN		5
#define SYS_NISSVR			6
#define NUM_OF_TAGS			6

static tagNameTable_t tagTable[NUM_OF_TAGS] = {
	{SYS_DNSDOMAIN,	"system.dnsdomain"},
	{SYS_DNSSVR1,	"system.dnssvr1"},
	{SYS_DNSSVR2,	"system.dnssvr2"},
	{SYS_DNSSVR3,	"system.dnssvr3"},
	{SYS_NISDOMAIN,	"system.nisdomain"},
	{SYS_NISSVR,	"system.nissvr"},
};


#define PARAM_ACT			1
#define PARAM_AUTOSUPP		2
#define PARAM_TOEMAIL		3
#define PARAM_NOTEEMAIL		4
#define NUM_OF_PARAMS		4

static paramNameTable_t paramTable[NUM_OF_PARAMS] = {
	{PARAM_ACT,			"act"},
	{PARAM_AUTOSUPP,	"as"},
	{PARAM_TOEMAIL,		"toemail"},
	{PARAM_NOTEEMAIL,	"noteemail"},
};


/* HERE ARE VARIABLES FOR SYSTEM INFORMATION */
typedef struct _system_config_t
{
    char		dnsDomain[128];
    char		dnsServer1[20];
    char		dnsServer2[20];
    char		dnsServer3[20];
    char		nisDomain[128];
    char		nisServer[20];
} system_config_t;


static char inAct[10];
static char inAutoSupp[16];
static char inToEmail[256];
static char inNoteEmail[256];
static int	res;
static system_config_t stData;


/***********************************************************
 * routines to retrieve system information thru anpssc
 **********************************************************/

static int getInputParams(int count, char **argname, char **argval)
{
	int argind, paramType;

	memset(inAct, 0, 10);
	for (argind = 0; argind < count; argind ++)
	{
		paramType = GetParamType(argname[argind], paramTable, NUM_OF_PARAMS);
		if (paramType == INVALID_PARAM)
			continue;

		switch (paramType) 
		{
		case PARAM_ACT:
			strcpy(inAct, argval[argind]);
			break;
		case PARAM_AUTOSUPP:
			strcpy(inAutoSupp, argval[argind]);
			break;
		case PARAM_TOEMAIL:
			strcpy(inToEmail, argval[argind]);
			break;
		case PARAM_NOTEEMAIL:
			strcpy(inNoteEmail, argval[argind]);
			break;
		}
	}

	return 0;
}

static void GetDnsNisConfigs(char* buf)
{
	int from = 0;

	from = GetDelimiteredParam(from, buf, stData.dnsDomain);
	from = GetDelimiteredParam(from, buf, stData.dnsServer1);
	from = GetDelimiteredParam(from, buf, stData.dnsServer2);
	from = GetDelimiteredParam(from, buf, stData.dnsServer3);
	from = GetDelimiteredParam(from, buf, stData.nisDomain);
	from = GetDelimiteredParam(from, buf, stData.nisServer);
}

static void GetTempSystemConfig(void *r, int from)
{
	int rc;
	char reqbuf[256];
	char rspbuf[2048];
	haSystemConfigType_t	*cfgtype;
	haSystemConfig_t		*config;
    haMessage_t				*req;
    haMessage_t				*rsp;
	struct timeval			timeout = {15, 0};

	res = -1;

    memset(rspbuf, 0, sizeof(rspbuf));
	rsp = (haMessage_t *) rspbuf;

	// Get DNS and NIS info
    //...... Fill the request.
    memset(reqbuf, 0, sizeof(reqbuf));
	req = (haMessage_t *) reqbuf;
	cfgtype = (haSystemConfigType_t *) req->data;

	cfgtype->from = from;
	cfgtype->step = 3;
	cfgtype->id   = GetHttpClientIP(r);

    req->type = HA_GETSYSTEMCONFIG;
	req->datalen = sizeof(haSystemConfigType_t);

    //...... Send the request.
    rc = sendAgileMsg(-1, reqbuf, HAMESSAGE_HEADER_SIZE + req->datalen,
                        "httpdagent", NFX_SSC_UNIX, SSC_SLOT, MGMTPLANE,
                        &timeout, rspbuf, sizeof(rspbuf));
	if (rc <= 0)
		return;
	if ((rsp->rspstatus != HARSP_OK) || (rsp->datalen != sizeof(haSystemConfig_t)))
		return;

    //...... Save the request.
	config = (haSystemConfig_t *) rsp->data;
	GetDnsNisConfigs(config->configs);

	res = 1;
}

static void SaveSystemConfig(void *r)
{
	int rc;
	char reqbuf[2048];
	char rspbuf[2048];
	haSystemConfigType_t	*cfgtype;
	haSystemConfig_t		*config;
    haMessage_t				*req;
    haMessage_t				*rsp;
	struct timeval			timeout = {10, 0};

    memset(rspbuf, 0, sizeof(rspbuf));
	rsp = (haMessage_t *) rspbuf;

	// Save data from prev page - agile contact and local mail info
    //...... Fill the request.
    memset(reqbuf, 0, sizeof(reqbuf));
	req = (haMessage_t *) reqbuf;
	config = (haSystemConfig_t *) req->data;

    req->type = HA_SETSYSTEMCONFIG;
	req->datalen = sizeof(haSystemConfig_t);
	config->type.from = 0;
	config->type.step = 2;
	config->type.id   = GetHttpClientIP(r);

	sprintf(config->configs, "%s,%s,%s",
			inAutoSupp, inToEmail, inNoteEmail);

    //...... Send the request.
    rc = sendAgileMsg(-1, reqbuf, HAMESSAGE_HEADER_SIZE + req->datalen,
                        "httpdagent", NFX_SSC_UNIX, SSC_SLOT, MGMTPLANE,
                        &timeout, rspbuf, sizeof(rspbuf));

	cfgtype = (haSystemConfigType_t *) rsp->data;
}

static void GetAndSetSystemData(void *r)
{
	int from;
	memset(&stData, 0, sizeof(system_config_t));

	// prev action, get data from temp file that we just created
	if (!strcmp(inAct, "prev"))
	{
		from = 0;
		
		// Get System Configuration
		GetTempSystemConfig(r, from);
	}
	// next action, get data from perminate file, also save data for prev page
	// SY: changed to get data from temp file (always...)
	else
	{
		// from = 1;
		from = 0;	// always get from temp file...
		SaveSystemConfig(r);

		// Get System Configuration
		GetTempSystemConfig(r, from);
	}
}


/***********************************************************
 *  Main routines to process tags
 **********************************************************/

static int getTagValue(char *tag, char *value)
{
	int i, tagType;
	
	if (res <= 0)
		return -1;

	tagType = INVALID_TAG;
	for (i = 0; i < NUM_OF_TAGS; i ++) {
		if (!strcmp(tag, tagTable[i].tagName)) {
			tagType = tagTable[i].tagType;
			break;
		}
	}
	if (tagType == INVALID_TAG)
		return -1;

	switch (tagType) 
	{
	case SYS_DNSDOMAIN:
		strcpy(value, stData.dnsDomain);
		break;
	case SYS_DNSSVR1:
		strcpy(value, stData.dnsServer1);
		break;
	case SYS_DNSSVR2:
		strcpy(value, stData.dnsServer2);
		break;
	case SYS_DNSSVR3:
		strcpy(value, stData.dnsServer3);
		break;
	case SYS_NISDOMAIN:
		strcpy(value, stData.nisDomain);
		break;
	case SYS_NISSVR:
		strcpy(value, stData.nisServer);
		break;
	default:
		return -1;
	}

#if 0
	if (value && (value[0] == '\0'))
		strcpy(value, "&nbsp");
#endif

	return 1;
}


/*
 * actRptTagFunc()
 *	tag   - tag name
 *	value - replacement string for tag
 */
static int actRptTagFunc(tagCallbackParams_t *tp)
{
char *tag = tp->tag;
char *value = tp->value;

	if (!strcmp(tag, "TABLE_BEGIN")) {
		return -1;
	}
	else if (!strcmp(tag, "TABLE_END")) {
		return -1;
	}
	else {
		/* other tags */
		return getTagValue(tag, value);
	}

	/* shouldn't reach here... */
	return -1;
}

int actionSysWizard3(void *r, int count, char **argname, char **argval)
{
	/* initialize system information variables */
	getInputParams(count, argname, argval);
	GetAndSetSystemData(r);

	prepare_agile_template(r, NULL);

	agile_scan_and_send(r, actRptTagFunc);

	return 1;
}
