/*  File Name  :    agilesyswz4.c 
 *
 *  Description:    HTTPD action functions for System Setup Wizard step 4.
 *
 *  Created    :    4/9/2002
 *
 *  Author     :    Tiffany Chiang
 *
 *  Copyright(c)  Agile Storage, Inc. 2001
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <anpssc-api.h>
#include "../agent/httpdagent.h"
#include <ipmd-lib.h>

#include "agilepost.h"
#include "agileutil.h"

#define INVALID_TAG			0
#define SYS_MIFLIST			1
#define SYS_MIFIP			2
#define SYS_MIFMASK			3
#define SYS_CIF1LIST		4
#define SYS_CIF2LIST		5
#define SYS_CIF3LIST		6
#define SYS_CIF4LIST		7
#define SYS_CIF1IP			8
#define SYS_CIF2IP			9
#define SYS_CIF3IP			10
#define SYS_CIF4IP			11
#define SYS_CIF1MASK		12
#define SYS_CIF2MASK		13
#define SYS_CIF3MASK		14
#define SYS_CIF4MASK		15
#define SYS_DEFROUTE		16
#define NUM_OF_TAGS			16

static tagNameTable_t tagTable[NUM_OF_TAGS] = {
	{SYS_MIFLIST,	"system.mif.list"},
	{SYS_MIFIP,		"system.mifip"},
	{SYS_MIFMASK,	"system.mifmask"},
	{SYS_CIF1LIST,	"system.cif1.list"},
	{SYS_CIF1IP,	"system.cif1ip"},
	{SYS_CIF1MASK,	"system.cif1mask"},
	{SYS_CIF2LIST,	"system.cif2.list"},
	{SYS_CIF2IP,	"system.cif2ip"},
	{SYS_CIF2MASK,	"system.cif2mask"},
	{SYS_CIF3LIST,	"system.cif3.list"},
	{SYS_CIF3IP,	"system.cif3ip"},
	{SYS_CIF3MASK,	"system.cif3mask"},
	{SYS_CIF4LIST,	"system.cif4.list"},
	{SYS_CIF4IP,	"system.cif4ip"},
	{SYS_CIF4MASK,	"system.cif4mask"},
	{SYS_DEFROUTE,	"system.defroute"},
};


#define PARAM_ACT			1
#define PARAM_DNSDM			2
#define PARAM_DNSSVR1		3
#define PARAM_DNSSVR2		4
#define PARAM_DNSSVR3		5
#define PARAM_NISDM			6
#define PARAM_NISSVR		7
#define NUM_OF_PARAMS		7

static paramNameTable_t paramTable[NUM_OF_PARAMS] = {
	{PARAM_ACT,		"act"},
	{PARAM_DNSDM,	"dnsdm"},
	{PARAM_DNSSVR1,	"dnssvr1"},
	{PARAM_DNSSVR2,	"dnssvr2"},
	{PARAM_DNSSVR3,	"dnssvr3"},
	{PARAM_NISDM,	"nisdm"},
	{PARAM_NISSVR,	"nissvr"},
};


/* HERE ARE VARIABLES FOR SYSTEM INFORMATION */
typedef struct _system_config_t
{
    char		mif[20];
    char		mifip[16];
    char		mifmask[16];
    char		cif[4][20];
    char		cifip[4][16];
    char		cifmask[4][16];
    char		defroute[16];
} system_config_t;


static char inAct[10];
static char inDnsDomain[128];
static char inDnsServer1[16];
static char inDnsServer2[16];
static char inDnsServer3[16];
static char inNisDomain[128];
static char inNisServer[16];
static int	res;
static system_config_t stData;
static int ifCount;
static ipm_if_t *stIfTable;


/***********************************************************
 * routines to retrieve system information thru anpssc
 **********************************************************/

static int getInputParams(int count, char **argname, char **argval)
{
	int argind, paramType;

	memset(inAct, 0, 10);
	for (argind = 0; argind < count; argind ++)
	{
		paramType = GetParamType(argname[argind], paramTable, NUM_OF_PARAMS);
		if (paramType == INVALID_PARAM)
			continue;

		switch (paramType) 
		{
		case PARAM_ACT:
			strcpy(inAct, argval[argind]);
			break;
		case PARAM_DNSDM:
			strcpy(inDnsDomain, argval[argind]);
			break;
		case PARAM_DNSSVR1:
			strcpy(inDnsServer1, argval[argind]);
			break;
		case PARAM_DNSSVR2:
			strcpy(inDnsServer2, argval[argind]);
			break;
		case PARAM_DNSSVR3:
			strcpy(inDnsServer3, argval[argind]);
			break;
		case PARAM_NISDM:
			strcpy(inNisDomain, argval[argind]);
			break;
		case PARAM_NISSVR:
			strcpy(inNisServer, argval[argind]);
			break;
		}
	}

	return 0;
}

static void GetIpIfConfigs(char* buf)
{
	int i, from = 0;

	from = GetDelimiteredParam(from, buf, stData.mif);
	from = GetDelimiteredParam(from, buf, stData.mifip);
	from = GetDelimiteredParam(from, buf, stData.mifmask);
	for (i = 0; i < 4; i++)
	{
		from = GetDelimiteredParam(from, buf, stData.cif[i]);
		from = GetDelimiteredParam(from, buf, stData.cifip[i]);
		from = GetDelimiteredParam(from, buf, stData.cifmask[i]);
	}
	from = GetDelimiteredParam(from, buf, stData.defroute);
}

static void GetSystemConfig(void *r, int from)
{
	int rc;
	char reqbuf[256];
	char rspbuf[2048];
	haSystemConfigType_t	*cfgtype;
	haSystemConfig_t		*config;
    haMessage_t				*req;
    haMessage_t				*rsp;
	struct timeval			timeout = {10, 0};

	res = -1;

    memset(rspbuf, 0, sizeof(rspbuf));
	rsp = (haMessage_t *) rspbuf;

	// Get Agile contact and local e-mail info
    //...... Fill the request.
    memset(reqbuf, 0, sizeof(reqbuf));
	req = (haMessage_t *) reqbuf;
	cfgtype = (haSystemConfigType_t *) req->data;

	cfgtype->from = from;
	cfgtype->step = 4;
	cfgtype->id   = GetHttpClientIP(r);

    req->type = HA_GETSYSTEMCONFIG;
	req->datalen = sizeof(haSystemConfigType_t);

    //...... Send the request.
    rc = sendAgileMsg(-1, reqbuf, HAMESSAGE_HEADER_SIZE + req->datalen,
                        "httpdagent", NFX_SSC_UNIX, SSC_SLOT, MGMTPLANE,
                        &timeout, rspbuf, sizeof(rspbuf));
	if (rc <= 0)
		return;
	if ((rsp->rspstatus != HARSP_OK) || (rsp->datalen != sizeof(haSystemConfig_t)))
		return;

    //...... Save the request.
	config = (haSystemConfig_t *) rsp->data;
	GetIpIfConfigs(config->configs);

	res = 1;
}

static void SaveSystemConfig(void *r)
{
	int rc;
	char reqbuf[2048];
	char rspbuf[2048];
	haSystemConfigType_t	*cfgtype;
	haSystemConfig_t		*config;
    haMessage_t				*req;
    haMessage_t				*rsp;
	struct timeval			timeout = {10, 0};

    memset(rspbuf, 0, sizeof(rspbuf));
	rsp = (haMessage_t *) rspbuf;

	// Save data from prev page - admin and pwd (save to temp)
    //...... Fill the request.
    memset(reqbuf, 0, sizeof(reqbuf));
	req = (haMessage_t *) reqbuf;
	config = (haSystemConfig_t *) req->data;

    req->type = HA_SETSYSTEMCONFIG;
	req->datalen = sizeof(haSystemConfig_t);

	config->type.from = 0;
	config->type.step = 3;
	config->type.id   = GetHttpClientIP(r);

	sprintf(config->configs, "%s,%s,%s,%s,%s,%s",
			inDnsDomain, inDnsServer1, inDnsServer2, inDnsServer3,
			inNisDomain, inNisServer);

    //...... Send the request.
    rc = sendAgileMsg(-1, reqbuf, HAMESSAGE_HEADER_SIZE + req->datalen,
                        "httpdagent", NFX_SSC_UNIX, SSC_SLOT, MGMTPLANE,
                        &timeout, rspbuf, sizeof(rspbuf));

	cfgtype = (haSystemConfigType_t *) rsp->data;
}

static void GetAndSetSystemData(void *r)
{
	int from;
	memset(&stData, 0, sizeof(system_config_t));

	// prev action, get data from temp file that we just created
	if (!strcmp(inAct, "prev"))
	{
		from = 0;
	}
	// next action, get data from perminate file, also save data for prev page
	// SY: Changed to always get data from temp files...
	else
	{
		// from = 1;
		from = 0;
		SaveSystemConfig(r);
	}

	// Get System Configuration
	GetSystemConfig(r, from);
}

static void GetIfList()
{
	int32	res;
	int count, need;

	ifCount = 0;
	stIfTable = NULL;

	/* Try to get IF Table with maxi. 20 entries */
	stIfTable = (ipm_if_t *) malloc(20*sizeof(ipm_if_t));
	if (stIfTable == NULL)
		return;
	res = ipmd_GetIfByName(NULL, stIfTable, 20*sizeof(ipm_if_t), 
							&count, &need);
	if (res == IPM_GET_ERR)
		return;
	else if ((res == IPM_GET_NO_BUF) || (res == IPM_CMDERR_APPNOBUF))
	{
		/* Need a larger buffer */
		free(stIfTable);
		stIfTable = malloc(need);
		if (stIfTable == NULL)
			return;

		res = ipmd_GetIfByName(NULL, stIfTable, need, &count, &need);
		if (res == IPM_GET_ERR)
			return;
		else if ((res == IPM_GET_NO_BUF) || (res == IPM_CMDERR_APPNOBUF))
			return;
	}

	if (count < 0)
		return;

	ifCount = count;
}


/***********************************************************
 *  Main routines to process tags
 **********************************************************/

static int getTagValue(char *tag, char *value)
{
	int i, tagType, found;
	
	tagType = INVALID_TAG;
	for (i = 0; i < NUM_OF_TAGS; i ++) {
		if (!strcmp(tag, tagTable[i].tagName)) {
			tagType = tagTable[i].tagType;
			break;
		}
	}
	if (tagType == INVALID_TAG)
		return -1;

	switch (tagType) 
	{
	case SYS_MIFLIST:
		found = 0;
		strcpy(value, "");
		for (i = 0; i < ifCount; i ++)
		{
			/* only allow sc* interface */
			if (strncmp(stIfTable[i].if_name, "sc", 2))
				continue;

			strcat(value, "<option value=\"");
			strcat(value, stIfTable[i].if_name);
			if (strcmp(stIfTable[i].if_name, stData.mif) == 0)
			{
				strcat(value, "\" selected>");
				found = 1;
			}
			else 
				strcat(value, "\">");
			strcat(value, stIfTable[i].if_name);
			strcat(value, "</option>");
		}
		strcat(value, "<option value=\"no\"");
		if (found == 1)
			strcat(value, ">---</option>");
		else
			strcat(value, " selected>---</option>");
		return 1;
	case SYS_CIF1LIST:
	case SYS_CIF2LIST:
	case SYS_CIF3LIST:
	case SYS_CIF4LIST:
		found = 0;
		strcpy(value, "");
		for (i = 0; i < ifCount; i ++)
		{
			/* only allow fp* interface */
			if (strncmp(stIfTable[i].if_name, "fp", 2))
				continue;

			strcat(value, "<option value=\"");
			strcat(value, stIfTable[i].if_name);
			if (!strcmp(stIfTable[i].if_name, stData.cif[tagType-SYS_CIF1LIST]))
			{
				strcat(value, "\" selected>");
				found = 1;
			}
			else 
				strcat(value, "\">");
			strcat(value, stIfTable[i].if_name);
			strcat(value, "</option>");
		}
		strcat(value, "<option value=\"no\"");
		if (found == 1)
			strcat(value, ">---</option>");
		else
			strcat(value, " selected>---</option>");
		return 1;
	}

	if (res <= 0)
		return -1;

	switch (tagType) 
	{
	case SYS_MIFIP:
		strcpy(value, stData.mifip);
		break;
	case SYS_MIFMASK:
		strcpy(value, stData.mifmask);
		break;
	case SYS_CIF1IP:
	case SYS_CIF2IP:
	case SYS_CIF3IP:
	case SYS_CIF4IP:
		strcpy(value, stData.cifip[tagType - SYS_CIF1IP]);
		break;
	case SYS_CIF1MASK:
	case SYS_CIF2MASK:
	case SYS_CIF3MASK:
	case SYS_CIF4MASK:
		strcpy(value, stData.cifmask[tagType - SYS_CIF1MASK]);
		break;
	case SYS_DEFROUTE:
		strcpy(value, stData.defroute);
		break;
	default:
		return -1;
	}

#if 0
	if (value && (value[0] == '\0'))
		strcpy(value, "&nbsp");
#endif

	return 1;
}


/*
 * actRptTagFunc()
 *	tag   - tag name
 *	value - replacement string for tag
 */
static int actRptTagFunc(tagCallbackParams_t *tp)
{
char *tag = tp->tag;
char *value = tp->value;

	if (!strcmp(tag, "TABLE_BEGIN")) {
		return -1;
	}
	else if (!strcmp(tag, "TABLE_END")) {
		return -1;
	}
	else {
		/* other tags */
		return getTagValue(tag, value);
	}

	/* shouldn't reach here... */
	return -1;
}

int actionSysWizard4(void *r, int count, char **argname, char **argval)
{
	/* initialize system information variables */
	getInputParams(count, argname, argval);
	GetAndSetSystemData(r);
	GetIfList();

	prepare_agile_template(r, NULL);

	agile_scan_and_send(r, actRptTagFunc);

	if (stIfTable != NULL)
		free(stIfTable);
	return 1;
}
