/*  File Name  :    agilesyswztime.c 
 *
 *  Description:    HTTPD action functions for System Setup Wizard step 1 - set current date/time.
 *
 *  Created    :    4/8/2002
 *
 *  Author     :    Tiffany Chiang
 *
 *  Copyright(c)  Agile Storage, Inc. 2001
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <sys/time.h>
#include <time.h>

#include <anpssc-api.h>
#include "../agent/httpdagent.h"

#include "agilepost.h"
#include "agileutil.h"


#define PARAM_MONTH			1
#define PARAM_DAY			2
#define PARAM_YEAR			3
#define PARAM_HOUR			4
#define PARAM_MIN			5
#define NUM_OF_PARAMS		5

static paramNameTable_t paramTable[NUM_OF_PARAMS] = {
	{PARAM_MONTH,		"mon"},
	{PARAM_DAY,			"day"},
	{PARAM_YEAR,		"year"},
	{PARAM_HOUR,		"hour"},
	{PARAM_MIN,			"min"},
};


/* HERE ARE VARIABLES FOR SYSTEM INFORMATION */
static char inMonth[5];
static char inDay[5];
static char inYear[5];
static char inHour[5];
static char inMin[5];


/***********************************************************
 * routines to retrieve system information thru anpssc
 **********************************************************/

static int getInputParams(int count, char **argname, char **argval)
{
	int argind, paramType;

	for (argind = 0; argind < count; argind ++)
	{
		paramType = GetParamType(argname[argind], paramTable, NUM_OF_PARAMS);
		if (paramType == INVALID_PARAM)
			continue;

		switch (paramType) 
		{
		case PARAM_MONTH:
			strcpy(inMonth, argval[argind]);
			break;
		case PARAM_DAY:
			strcpy(inDay, argval[argind]);
			break;
		case PARAM_YEAR:
			strcpy(inYear, argval[argind]);
			break;
		case PARAM_HOUR:
			strcpy(inHour, argval[argind]);
			break;
		case PARAM_MIN:
			strcpy(inMin, argval[argind]);
			break;
		}
	}

	return 0;
}

static void SetDateTime(char* file)
{
	int 			rc, res = -1;
	char 			reqbuf[1024];
	char 			rspbuf[256];
    haMessage_t		*req;
    haMessage_t		*rsp;
	struct tm 		*tmval;
	struct timeval	tval = {15, 0};

	//set date, time......

    memset(rspbuf, 0, sizeof(rspbuf));
	rsp = (haMessage_t *) rspbuf;
    memset(reqbuf, 0, sizeof(reqbuf));
	req = (haMessage_t *) reqbuf;

	tmval = (struct tm *) req->data;
	tmval->tm_min  = atoi(inMin);
	tmval->tm_hour = atoi(inHour);
	tmval->tm_mday = atoi(inDay);
	tmval->tm_mon  = atoi(inMonth);
	tmval->tm_year = atoi(inYear) - 1900;

    req->type = HA_SETSYSTIME;
	req->datalen = sizeof(struct tm);

    rc = sendAgileMsg(-1, req, HAMESSAGE_HEADER_SIZE + req->datalen,
                        "httpdagent", NFX_SSC_UNIX, SSC_SLOT, MGMTPLANE,
                        &tval, rsp, sizeof(rspbuf));

	if (rc <= 0)
		res = -1;
	else if (rsp->rspstatus == HARSP_OK)
		res = 1;

	if (res <= 0)
		strcpy(file, "error_syswzsetdate.html");
}


/***********************************************************
 *  Main routines to process tags
 **********************************************************/

static int getTagValue(char *tag, char *value)
{
	return 1;
}


/*
 * actRptTagFunc()
 *	tag   - tag name
 *	value - replacement string for tag
 */
static int actRptTagFunc(tagCallbackParams_t *tp)
{
char *tag = tp->tag;
char *value = tp->value;

	if (!strcmp(tag, "TABLE_BEGIN")) {
		return -1;
	}
	else if (!strcmp(tag, "TABLE_END")) {
		return -1;
	}
	else {
		/* other tags */
		return getTagValue(tag, value);
	}

	/* shouldn't reach here... */
	return -1;
}

int actionSysWizardSetTime(void *r, int count, char **argname, char **argval)
{
	char newfname[128];

	memset(newfname, 0, sizeof(newfname));

	/* initialize system information variables */
	getInputParams(count, argname, argval);
	SetDateTime(newfname);

	prepare_agile_template(r, newfname);

	agile_scan_and_send(r, actRptTagFunc);

	return 1;
}
