/*  File Name  :    agileutil.c 
 *
 *  Description:    HTTPD utilities.
 *
 *  Created    :    1/31/2002
 *
 *  Author     :    Tiffany Chiang
 *
 *  Copyright(c)  Agile Storage, Inc. 2001
 *
 */

#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <ctype.h>

#include <nfx-types.h>

#if 0
#include "httpd.h"
#include "http_request.h"
#include "http_log.h"
#endif

#include "agileutil.h"

/* global variables */

char *strAdd = "add";
char *strMod = "mod";
char *strDel = "del";

/* export functions */

/* This only work with "," delimitered string */
/* return the next starting point, which can  */
/* be used to pass back in "from" parameter   */
int GetDelimiteredParam(int from, char *srcbuf, char *dstbuf)
{
int start = from;

	if (dstbuf)
		dstbuf[0] = '\0';

	if ((from < 0) || !srcbuf)		/* no buffer? */
		return 0;

	if (!srcbuf[from])		/* already end of buffer */
		return 0;

	while (srcbuf[from] && (srcbuf[from] != ','))
		from++;

	if (from > start)
	{
		strncpy(dstbuf, &srcbuf[start], from - start);
		dstbuf[from-start] = '\0';
	}

	if (srcbuf[from])
		from++;

	return from;
}

void GetWwnStr(uint64 wwn, char* buf)
{
	int i;
	char temp[16];

	sprintf(temp, "%016llx", wwn);
	for (i = 0; i < 8; i ++)
	{
		buf[i*3] = temp[i*2];
		buf[i*3+1] = temp[i*2+1];
		if (i < 7)
			buf[i*3+2] = ':';
		else
			buf[i*3+2] = '\0';
	}
}

void GetIpAddressStr(uint32 src, char* buf)
{
	unsigned char* ptr;
	
	ptr = (char *) &src;
	sprintf(buf, "%d.%d.%d.%d", ptr[0], ptr[1], ptr[2], ptr[3]);
}

uint32 GetIpAddressFromStr(char* buf)
{
	char seps[]   = ".";
	char tmpbuf[32];
	char* token;
	int i;
	unsigned char ptr[4];
	uint32 *p = (uint32 *) ptr;

	/* strtok will mess up the original string... */
	strcpy(tmpbuf, buf);

	i = 0;
	*p = 0;
	token = strtok(tmpbuf, seps);
	while(token != NULL)
	{
		ptr[i++] = atoi(token);
		token = strtok(NULL, seps);
	}
	return *p;
#if 0
	unsigned char ptr[4];
	uint32 *p = (uint32 *) ptr;
	
	sscanf(buf, "%d.%d.%d.%d",
		(int *) &ptr[0],
		(int *) &ptr[1],
		(int *) &ptr[2],
		(int *) &ptr[3]);

	return *p;
#endif
}

void GetMacAddressStr(char* mac, char* buf)
{
unsigned char *ptr;

	ptr = (unsigned char *) mac;
	sprintf(buf, "%02x:%02x:%02x:%02x:%02x:%02x",
					ptr[0], ptr[1], ptr[2], ptr[3], ptr[4], ptr[5]);
#if 0
	char temp[10];
	int i, len;
	strcpy(buf, "");
	for (i = 0; i < 6; i ++)
	{
		sprintf(temp, "%02x", mac[i]);
		if (strlen(temp) == 2)
		{
			strcat(buf, temp);
		}
		else
		{
			len = strlen(temp);
			strcat(buf, temp+len-2);
		}
		if (i < 5)
			strcat(buf, ":");
	}
#endif
}

void GetInterfaceStrs(uchar8* ifname, char* typebuf, char* slotbuf, char* linkbuf)
{
char *p, *pslot, *plink;

	strncpy(typebuf, ifname, 2);
	typebuf[2] = '\0';

	p = &ifname[2];
	pslot = slotbuf;
	plink = linkbuf;
	if (!strcmp(typebuf, "sc")) {
		strcpy(slotbuf, "0");
		strcpy(linkbuf, &ifname[2]);
	}
	else if (!strcmp(typebuf, "fc") || !strcmp(typebuf, "fp")) {
		while (*p && (*p != '.'))
			*pslot++ = *p++;
		*pslot = '\0';
		if (*p == '.') {
			p++;
			while (*p)
				*plink++ = *p++;
			*plink = '\0';
		}
		else
			strcpy(linkbuf, "na");
	}
	else {
		strcpy(slotbuf, "na");
		strcpy(linkbuf, "na");
	}
#if 0
	char seps[]   = ":/";
	char* buf1;
	char* buf2;

	buf1 = strstr(ifname, ":");
	buf2 = strstr(ifname, "/");
	if (buf1 == NULL && buf2 == NULL)
	{
		strcpy(typebuf, ifname);
		strcpy(slotbuf, "na");
		strcpy(linkbuf, "na");
	}
	else
	{
		typebuf = strtok(ifname, seps);
		slotbuf = strtok(NULL, seps);
		linkbuf = strtok(NULL, seps);
	}
#endif
}

void GetInterfaceTypeStr(uchar8* ifname, char* buf)
{
	strncpy(buf, ifname, 2);
	buf[2] = '\0';
#if 0
	char temp1[10];
	char temp2[10];
	GetInterfaceStrs(ifname, buf, temp1, temp2);
#endif
}

void GetInterfaceSlotStr(uchar8* ifname, char* buf)
{
	char temp1[10];
	char temp2[10];
	GetInterfaceStrs(ifname, temp1, buf, temp2);
}

void GetInterfaceLinkStr(uchar8* ifname, char* buf)
{
	char temp1[10];
	char temp2[10];
	GetInterfaceStrs(ifname, temp1, temp2, buf);
}

void GetInterfaceNameStr(char* ifname, char* typebuf, char* slotbuf, char* linkbuf)
{
	if (!strcmp(typebuf, "sc"))
		sprintf(ifname, "%s%s", typebuf, linkbuf);
	else
		sprintf(ifname, "%s%s.%s", typebuf, slotbuf, linkbuf);
#if 0
	strcpy(ifname, typebuf);
	if (!strcmp(slotbuf, "na"))
	{
		strcat(ifname, ":");
		strcat(ifname, slotbuf);
		strcat(ifname, "/");
		strcat(ifname, linkbuf);
	}
#endif
}

int GetParamType(char* paramName, paramNameTable_t* table, int count)
{
	int type, i;

	type = INVALID_PARAM;
	for (i = 0; i < count; i++) {
		if (!strcmp(paramName, table[i].paramName)) {
			type = table[i].paramType;
			break;
		}
	}
	return type;
}

#ifdef ISABELLE // code to delete
void GetVolTypeStr(evm_lVol_t type, char* buf)
{
	switch (type)
	{
	case EVM_VOL_PLAIN:		strcpy(buf, "Plain");	break;
	case EVM_VOL_RAID0:		strcpy(buf, "RAID 0");	break;
	case EVM_VOL_RAID1:		strcpy(buf, "RAID 1");	break;
	case EVM_VOL_RAID1n0:	strcpy(buf, "RAID 1+0");break;
	case EVM_VOL_RAID5:		strcpy(buf, "RAID 5");	break;
	default:				strcpy(buf, "Invalid");
	}
}
#endif // ISABELLE

void GetRaidLevelStr(evm_raid_t raid, char* buf)
{
	switch (raid)
	{
	case EVM_RAID0:			strcpy(buf, "RAID-0");	break;
	case EVM_RAID1:			strcpy(buf, "RAID-1");	break;
	case EVM_RAID01:		strcpy(buf, "RAID-1+0");break;
	case EVM_RAID3:			strcpy(buf, "RAID-3");	break;
	case EVM_RAID5:			strcpy(buf, "RAID-5");	break;
	case EVM_RAID_NONE:		strcpy(buf, "Plain");	break;
	case EVM_RAID_ANY:		strcpy(buf, "Any");		break;
	default:				strcpy(buf, "Invalid");
	}
}

evm_raid_t GetRaidLevelFromStr(char* buf)
{
	if (!strcmp(buf, "RAID-0"))
		return EVM_RAID0;
	else if (!strcmp(buf, "RAID-1"))
		return EVM_RAID1;
	else if (!strcmp(buf, "RAID-1+0"))
		return EVM_RAID01;
	else if (!strcmp(buf, "RAID-3"))
		return EVM_RAID3;
	else if (!strcmp(buf, "RAID-5"))
		return EVM_RAID5;
	else if (!strcmp(buf, "None"))
		return EVM_RAID_NONE;
	else if (!strcmp(buf, "Any"))
		return EVM_RAID_ANY;
	else
		return EVM_RAID_ANY;
}

void GetSystemIP(char* buf)
{
}

void GetSystemHostName(char* buf)
{
}

uint64 GetVolumeDispUsedSize(uint64 intSize)
{
	/* intSize is in 8k block size unit (8192) */
	/* so, mb = intSize * 8192 / (1024 * 1024) */
	uint64 mb = intSize / 128;
	return mb;
}

uint64 GetVolumeIntUsedSize(uint64 dispSize)
{
	/* intsize should be in 8k block size unit (8192) */
	/* so, intsize = dispsize(mb) * 1024 * 1024 / 8192 */
	uint64 intsize = dispSize * 128;
	return intsize;
}

uint64 GetVolumeDispCurrSize(uint64 intSize)
{
	return GetMBfromBlock(intSize);
}

uint64 GetVolumeIntCurrSize(uint64 dispSize)
{
	return GetBlockfromMB(dispSize);
}

uint64 GetMBfromBlock(uint64 block)
{
	/* one block is 512 bytes */
	/* so mb = block * 512 / (1024 * 1024) */
	return (block / 2048);
}

uint64 GetKBfromBlock(uint64 block)
{
	uint64 kb = block / 2;
	return kb;
}

uint64 GetBlockfromMB(uint64 mb)
{
	/* one MB equal to 1024 * 1024 / 512 blocks */
	return (mb * 2048);
}

uint64 GetBlockfromKB(uint64 kb)
{
	uint64 blk = kb * 2;
	return blk;
}

int GetNfsPermission(char* exportInfo)
{
	char seps[]   = "(), ";
	char *token;
	char tmpbuf[1024];

	/* strtok will mess up original buffer */
	strcpy(tmpbuf, exportInfo);

	token = strtok(tmpbuf, seps);
	while(token != NULL)
	{
		if (strcmp(token, "ro") == 0 || strcmp(token, "RO") == 0)
			return 0;
		else if (strcmp(token, "rw") == 0 || strcmp(token, "RW") == 0)
			return 1;
		token = strtok( NULL, seps );
	}
	return 0;
}

int GetNfsRootSquash(char* exportInfo)
{
	if (strstr(exportInfo, "no_root_squash"))
		return 0;
	else if (strstr(exportInfo, "root_squash"))
		return 1;

	return 0;
#if 0
	char seps[]   = "(), ";
	char *token;
	char tmpbuf[1024];

	/* strtok will mess up original buffer */
	strcpy(tmpbuf, exportInfo);

	token = strtok(tmpbuf, seps);
	while(token != NULL)
	{
		if (strcmp(token, "no_root_squash") == 0)
			return 0;
		else if (strcmp(token, "root_squash") == 0)
			return 1;
		token = strtok( NULL, seps );
	}
	return 0;
#endif
}

void removeTrailingBlank(char *s)
{
char *p;

	if (!s)
		return;

	p = s + (strlen(s) - 1);
	while (p >= s) {
		if (*p == ' ')
			*p = '\0';
		else
			break;
		p--;
	}
}


/* this is used only for parameter string encoding */
/* don't use this routine to convert the whole URL */
void convertSpecCharToUrlChar(char *s)
{
char *p, buf[256];

	if (!s)
		return;

	strcpy(buf, s);
	p = buf;
	while (*p) {
		if (isdigit(*p) || isalpha(*p) ||
			(*p == '_') || (*p == '-') || (*p == '.')) {
			*s++ = *p++;
		}
		else {
			sprintf(s, "%%%02X", *p++);
			s += 3;
		}
	}
	*s = '\0';
}

void convertUrlCharToSpecChar(char *s)
{
char buf[256];
char c, *p = buf;

	if (!s)
		return;

	strcpy(p, s);
	while (*p) {
		if (*p == '%') {
			p++;
			if ((*p >= 'A') && (*p <= 'F'))
				c = *p - 'A' + 10;
			else if ((*p >= 'a') && (*p <= 'f'))
				c = *p - 'a' + 10;
			else
				c = *p - '0';
			c = c << 4;

			p++;
			if ((*p >= 'A') && (*p <= 'F'))
				c += *p - 'A' + 10;
			else if ((*p >= 'a') && (*p <= 'f'))
				c += *p - 'a' + 10;
			else
				c += *p - '0';

			p++;
			*s++ = c;
		}
		else
			*s++ = *p++;
	}
	*s = '\0';
}
