/*  File Name  :	agilevolmain.c 
 *
 *  Description:	HTTPD action functions for main Volume Page.
 *
 *  Created	:	3/6/2002
 *
 *  Author	 :	Tiffany Chiang
 *
 *  Copyright(c)  Agile Storage, Inc. 2001
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
// #include <sys/types.h>
// #include <unistd.h>
#include <sys/time.h>
#include <anpssc-api.h>

#include <nfx-defs.h>
#include <nfx-types.h>
#include <evm-api.h>
#include <evm_com_defs.h>
#include <ea-api.h>

#include "agilepost.h"
#include "agileutil.h"

#define INVALID_TAG			0
#define VOL_COLSPAN			1
#define VOL_VOLID			2
#define VOL_VOLNAME			3
#define VOL_STATUS			4
#define VOL_FILESYS			5
#define VOL_USED			6
#define VOL_ALLOC			7
#define VOL_SOFTQ			8
#define VOL_HARDQ			9
#define VOL_MAKE			10
#define VOL_MODEL			11
#define VOL_RAID			12
#define NUM_OF_TAGS			12


static tagNameTable_t tagTable[NUM_OF_TAGS] = {
	{VOL_COLSPAN,	"priv.colspan"},
	{VOL_VOLID,		"ea_fsysCfg_t.volId"},
	{VOL_VOLNAME,	"ea_fsysCfg_t.fsysName"},
	{VOL_STATUS,	"ea_fsysCfg_t.onlineStatus"},
	{VOL_FILESYS,	"ea_fsysCfg_t.filesys"},
	{VOL_USED,		"ea_fsysCfg_t.usedSize"},
	{VOL_ALLOC,		"ea_fsysCfg_t.currSize"},
	{VOL_SOFTQ,		"ea_fsysCfg_t.softQuota"},
	{VOL_HARDQ,		"ea_fsysCfg_t.maxSize"},
	{VOL_MAKE,		"ea_fsysCfg_t.vendorInfo.make"},
	{VOL_MODEL,		"ea_fsysCfg_t.vendorInfo.model"},
	{VOL_RAID,		"ea_fsysCfg_t.raidLevel"},
};


#define PARAM_ACT			1
#define PARAM_NAME			2
#define PARAM_HQUOTA		3
#define PARAM_AGROW			4
#define PARAM_HIGHW			5
#define PARAM_MINLUN		6
#define PARAM_MAXLUN		7
#define PARAM_ID			8
#define PARAM_NEWNAME		9
#define PARAM_STGRADE		10
#define PARAM_HQSEL			11
#define PARAM_SQSEL			12
#define PARAM_SQUOTA		13
#define PARAM_AGSEL			14
#define PARAM_AGTHR			15
#define PARAM_RAID			16
#define PARAM_MINLSEL		17
#define PARAM_MAXLSEL		18
#define NUM_OF_PARAMS		18

static paramNameTable_t paramTable[NUM_OF_PARAMS] = {
	{PARAM_ACT,		"act"},
	{PARAM_NAME,	"name"},
	{PARAM_HQUOTA,	"hq"},
	{PARAM_AGROW,	"ag"},
	{PARAM_MINLUN,	"minlun"},
	{PARAM_MAXLUN,	"maxlun"},
	{PARAM_ID,		"id"},
	{PARAM_NEWNAME,	"newn"},
	{PARAM_STGRADE,	"stglist"},
	{PARAM_HQSEL,	"hqr"},
	{PARAM_SQSEL,	"sqr"},
	{PARAM_SQUOTA,	"sq"},
	{PARAM_AGSEL,	"agr"},
	{PARAM_AGTHR,	"agt"},
	{PARAM_RAID,	"raidlist"},
	{PARAM_MINLSEL,	"minlunr"},
	{PARAM_MAXLSEL,	"maxlunr"},
};


/* HERE ARE VARIABLES FOR VOLUME INFORMATION */
static int64 inId;
static char inAct[10];
static char inName[VOL_MAX_NAME+1];
static char inNewName[VOL_MAX_NAME+1];
static char inStgGrade[100];
static uint64 inHQuota;
static uint64 inAGrow;
static uint64 inMinLun;
static uint64 inMaxLun;
// New inputs for new Add/Modify Volume Pages
static int inHQuotaSel;
static int inSQuotaSel;
static uint64 inSQuota;
static int inAGrowSel;
static uint32 inAGThrehold;
static int inMinLunSel;
static int inMaxLunSel;
static char inRaid[10];

#define VOLS_PER_REQ		20

static int totalEntries;

static int dataCount;
static int dataCurr;
static ea_fsysCfg_t stData;
static ea_fsysCfg_t *volumeList;

static char *volState_str[] = {"Offline", "Online", "Unknown" };
static char *fs_str[] = {"No", "Yes" };
static char *vfsState_str[] = {"Unmounted", "Mounted" };


/***********************************************************
 *  Main routines to handle Add/Delete/Modify Volume actions
 **********************************************************/

static int getInputParams(int count, char **argname, char **argval)
{
	int argind, paramType;

	for (argind = 0; argind < count; argind ++)
	{
		paramType = GetParamType(argname[argind], paramTable, NUM_OF_PARAMS);
		if (paramType == INVALID_PARAM)
			continue;

		switch (paramType) 
		{
		case PARAM_ID:
			sscanf(argval[argind], "%lld", &inId);
			break;
		case PARAM_ACT:
			strcpy(inAct, argval[argind]);
			break;
		case PARAM_NAME:
			strcpy(inName, argval[argind]);
			break;
		case PARAM_HQUOTA:
			sscanf(argval[argind], "%lld", &inHQuota);
			break;
		case PARAM_AGROW:
			sscanf(argval[argind], "%lld", &inAGrow);
			break;
		case PARAM_MINLUN:
			sscanf(argval[argind], "%lld", &inMinLun);
			break;
		case PARAM_MAXLUN:
			sscanf(argval[argind], "%lld", &inMaxLun);
			break;
		case PARAM_NEWNAME:
			strcpy(inNewName, argval[argind]);
			break;
		case PARAM_STGRADE:
			strcpy(inStgGrade, argval[argind]);
			break;
		case PARAM_HQSEL:
			sscanf(argval[argind], "%d", &inHQuotaSel);
			break;
		case PARAM_SQSEL:
			sscanf(argval[argind], "%d", &inSQuotaSel);
			break;
		case PARAM_AGSEL:
			sscanf(argval[argind], "%d", &inAGrowSel);
			break;
		case PARAM_MINLSEL:
			sscanf(argval[argind], "%d", &inMinLunSel);
			break;
		case PARAM_MAXLSEL:
			sscanf(argval[argind], "%d", &inMaxLunSel);
			break;
		case PARAM_SQUOTA:
			sscanf(argval[argind], "%lld", &inSQuota);
			break;
		case PARAM_AGTHR:
			sscanf(argval[argind], "%d", &inAGThrehold);
			break;
		case PARAM_RAID:
			strcpy(inRaid, argval[argind]);
			break;
		}
	}

	return 0;
}

static int getTotalRowCount()
{
	int						stat;
	ea_numFsysReq_t			req;
	ea_numFsysRsp_t			rsp;
	struct timeval			timeout = {10, 0};
			
	//Fill the request. 
	bzero((void *) &req, sizeof(ea_numFsysReq_t));
	req.hdr.msgType = EA_NUM_FSYS_REQ;
	strcpy(req.hdr.node, "\0");

	// Send message to EA.

	stat = sendAgileMsg(-1, &req, sizeof(req),
						"ea", NFX_SSC_UNIX, SSC_SLOT, MGMTPLANE,
						&timeout, &rsp, sizeof(ea_numFsysRsp_t));

	// Check response from EA.
	if (stat <= 0)
		return -1;

	if ((rsp.hdr.msgType != EA_NUM_FSYS_RSP) ||
		(rsp.result != NFX_OK) ||
		(rsp.totNum < 0))
		return -1;

	return rsp.totNum;
}

static void getPageTable()
{
	int						stat, size, count;
	ea_getFsysReq_t			req;
	ea_getFsysRsp_t			*rsp;
	struct timeval			timeout = {60, 0};

	// get count
	totalEntries = getTotalRowCount();

	// allocate space for volume information
	if (volumeList != NULL)
		free(volumeList);
	volumeList = malloc(totalEntries * sizeof(ea_fsysCfg_t));
	if (volumeList == NULL)
		return;

	// allocate space for response...
	size = sizeof(ea_getFsysRsp_t) + (sizeof(ea_fsysCfg_t) * totalEntries);
	rsp = (ea_getFsysRsp_t *) malloc(size);
	if (rsp == NULL)
	{
		if (volumeList)
			free(volumeList);
		volumeList = NULL;
		return;
	}

	//Fill the request. 
	memset(&req, 0, sizeof(ea_getFsysReq_t));

	req.hdr.msgType = EA_GET_FSYS_REQ;
	req.reqType = EA_REQ_HAS_FSYS_INIT;
	req.num = VOLS_PER_REQ;
	req.fsysId[0].volId.whole_thing = 0; /* start from beginning */

	count = 0;
	while (1)
	{
		memset(rsp, 0, size);

		// Send message to EA. 
		stat = sendAgileMsg(-1, &req, sizeof(req),
						"ea", NFX_SSC_UNIX, SSC_SLOT, MGMTPLANE,
						&timeout, rsp, size);

		// Check response from EA. 
		if ((stat <= 0) || (rsp->result != NFX_OK))
			break;

		if ((count+rsp->fsysList.nb) > totalEntries)
			break;

		memcpy(&volumeList[count], rsp->fsysList.fsys,
							rsp->fsysList.nb * sizeof(ea_fsysCfg_t));
		count += rsp->fsysList.nb;

		if (rsp->fsysList.nb < VOLS_PER_REQ)
			break;

		req.fsysId[0].volId.whole_thing = 
						rsp->fsysList.fsys[rsp->fsysList.nb-1].fsysId.volId.whole_thing;
	}

	// reduce the total entries to whatever we got from EA
	totalEntries = count;

	if (rsp)
		free(rsp);
	return;
}


// following function are shamelessly copied from cmd_vol.c,
// originall written by Isabelle
#ifdef NEWVOLID
static evm_volId_t get_newVolId(void)
{
	char                    tmpbuf[48000];
	int                     rc;
	evm_MsgGetNewVolId_t    getReq;
	evm_MsgStat_t *         getRsp = (evm_MsgStat_t *) tmpbuf;
	struct timeval			timeout = {30, 0};

	memset(&getReq, 0, sizeof(evm_MsgGetNewVolId_t));
	memset(tmpbuf, 0, sizeof(tmpbuf));

	getReq.hdr.msgType = EVM_VOL_GET_NEWVOLID;

	rc = sendAgileMsg(-1, &getReq, sizeof(evm_MsgGetNewVolId_t),
                           "evm", NFX_SSC_UNIX, SSC_SLOT, MGMTPLANE,
                           &timeout, tmpbuf, sizeof(tmpbuf));

	if ((rc != sizeof(evm_MsgStat_t)) || (getRsp->stat != NFX_OK))
        getRsp->payload.lVolId.whole_thing = -1;

	return(getRsp->payload.lVolId);
}
#endif

static int DoVolumeDel()
{
	int						stat;
	ea_delFsysReq_t			req;
	ea_delFsysRsp_t			rsp;
			
	//Fill the request. 
	bzero((void *) &req, sizeof(ea_delFsysReq_t));
	req.hdr.msgType = EA_DEL_FSYS_REQ;
	strcpy(req.hdr.node, "\0");
	strcpy(req.fsysId.fsysName, inName);

	// Send message to EA. 
	stat = sendAgileMsg(-1, &req, sizeof(req),
						"ea", NFX_SSC_UNIX, SSC_SLOT, MGMTPLANE, NULL,
						&rsp, sizeof(ea_delFsysRsp_t));

	// Check response from EA. 
	if ((stat <= 0) ||
		(rsp.hdr.msgType != EA_DEL_FSYS_RSP) ||
		(rsp.result != NFX_OK))
		return -1;

	return 1;
}

static int DoVolumeMod()
{
	int						stat;
	char					reqbuf[1024];
	char					rspbuf[4096];
	ea_getDetailFsysReq_t	*req1 = (ea_getDetailFsysReq_t *) reqbuf;
	ea_getDetailFsysRsp_t	*rsp1 = (ea_getDetailFsysRsp_t *) rspbuf;
	ea_modFsysReq_t			*req2 = (ea_modFsysReq_t *) reqbuf;
	ea_modFsysRsp_t			*rsp2 = (ea_modFsysRsp_t *) rspbuf;
	struct timeval			timeout = {90, 0};

	// First, retrieve the volume information again
	// this is to compare which item been changed...
	memset(reqbuf, 0, sizeof(reqbuf));
	memset(rspbuf, 0, sizeof(rspbuf));

	req1->hdr.msgType = EA_GET_DETAIL_FSYS_REQ;
	strcpy(req1->hdr.node, "local");
	strcpy(req1->fsysId.fsysName, inName);

	// Send message to EA. 
	stat = sendAgileMsg(-1, req1, sizeof(ea_getDetailFsysReq_t),
						"ea", NFX_SSC_UNIX, SSC_SLOT, MGMTPLANE,
						&timeout, &rspbuf, sizeof(rspbuf));

	// Check response from EA. 
	if (stat <= 0) 
		return -1;

	rsp1 = (ea_getDetailFsysRsp_t *) rspbuf;
	if (rsp1->result != NFX_OK)
		return -1;

	// Now do the modify request
	// the original volume info is in rspbuf, so don't
	// mess up the data in there...
	memset(reqbuf, 0, sizeof(reqbuf));

	req2->hdr.msgType = EA_MOD_FSYS_REQ;
	req2->hdr.userInfo = 0;

	strcpy(req2->hdr.node, "\0");
	strcpy(req2->fsysId.fsysName, inName);

	req2->mask = 0;
	if (strlen(inNewName) > 0)
	{
		strcpy(req2->newFsysName, inNewName);
		SET_FSYS_NAME(req2->mask);
	}

	if (inHQuotaSel == 1)
		req2->hardQuota = GetBlockfromMB(inHQuota);
	else
		req2->hardQuota = 0;
	if (rsp1->fsysDetail.volCfg.maxSize != req2->hardQuota)
		SET_FSYS_HQUOTA(req2->mask);

	req2->softQuota = inSQuota;
	if (rsp1->fsysDetail.volCfg.softQuota != req2->softQuota)
		SET_FSYS_SQUOTA(req2->mask);

	if (inAGrowSel == 1)
		req2->minAutoGrow = GetBlockfromMB(inAGrow);
	else
		req2->minAutoGrow = 0;
	if (rsp1->fsysDetail.volCfg.minAutoGrow != req2->minAutoGrow)
		SET_FSYS_MINAUTOGROW(req2->mask);

	req2->highWaterMark = inAGThrehold;
	if (rsp1->fsysDetail.highWaterMark != req2->highWaterMark)
		SET_FSYS_HIGHWATERMARK(req2->mask);

	// now we don't need the original data, so let's
	// reset the buffer memory to 0...
	memset(rspbuf, 0, sizeof(rspbuf));

	// Send message to EA. 
	stat = sendAgileMsg(-1, req2, sizeof(ea_modFsysReq_t),
						"ea", NFX_SSC_UNIX, SSC_SLOT, MGMTPLANE,
						&timeout, rspbuf, sizeof(rspbuf));

	// Check response from EA. 
	if (stat < 0) 
		return -1;
	else if (!stat)
		return -1;

	if (rsp2->result != NFX_OK)
		return -1;

	return 1;
}

static int DoVolumeAdd()
{
	int					stat;
	ea_addFsysReq_t		req;
	ea_addFsysRsp_t		rsp;
	struct timeval		timeout = {90, 0};
	
	// Find Storage Grade Info.
	char buf1[128], buf2[128];

	if (!strcmp(inStgGrade, "any"))
	{
		buf1[0] = '\0';
		buf2[0] = '\0';
	}
	else
	{
		int from = 0;

		from = GetDelimiteredParam(from, inStgGrade, buf1);
		from = GetDelimiteredParam(from, inStgGrade, buf2);
	}

	//Fill the request. 
	memset((void *) &req, 0, sizeof(ea_addFsysReq_t));
	req.hdr.msgType = EA_ADD_FSYS_REQ;
	req.hdr.userInfo = 0;

#ifdef NEWVOLID
    req.fsysId.volId = get_newVolId();
    if (req.fsysId.volId.whole_thing == -1)
        return -1;
#endif

	strcpy(req.hdr.node, "");
	strcpy(req.fsysId.fsysName, inName);
	req.volSize = 0;
	if (inHQuotaSel == 1)
		req.maxSize = GetBlockfromMB(inHQuota);
	else
		req.maxSize = 0;
	req.softQuota = inSQuota;
	if (inAGrowSel == 1)
		req.minAutoGrow = GetBlockfromMB(inAGrow);
	else
		req.minAutoGrow = 0;
	req.highWaterMark = inAGThrehold;
	if (inMinLunSel == 1)
		req.attr.minLunSize = GetBlockfromMB(inMinLun);
	else
		req.attr.minLunSize = EVM_DONT_CARE;
	if (inMaxLunSel == 1)
		req.attr.maxLunSize = GetBlockfromMB(inMaxLun);
	else
		req.attr.maxLunSize = EVM_DONT_CARE;
	req.attr.num = 1;
	strcpy(req.attr.vendorInfo[0].vendor, buf1);
	if (buf1[0])
		req.attr.flags = EVM_SAME_VENDOR;
	strcpy(req.attr.vendorInfo[0].model, buf2);
	if (buf2[0])
		req.attr.flags = EVM_SAME_MODEL;
	req.attr.raidLevel = GetRaidLevelFromStr(inRaid);
	req.nim = 1;

	// Send message to EA. 
	stat = sendAgileMsg(-1, &req, sizeof(req),
						"ea", NFX_SSC_UNIX, SSC_SLOT, MGMTPLANE,
						&timeout, &rsp, sizeof(ea_addFsysRsp_t));

	// Check response from EA. 
	if (stat <= 0) 
		return -1;

	if (rsp.result != NFX_OK)
		return -1;
	return 1;
}

static int HandleVolAction(char *newfname, int count, char **argname, char **argval)
{
	int res;

	getInputParams(count, argname, argval);
	res = 1;
	if (!strcmp(inAct, "del"))
	{
		res = DoVolumeDel();
		if (res <= 0)
			strcpy(newfname, "error_deletevolume.html");
	}
	else if (!strcmp(inAct, "mod"))
	{
		res = DoVolumeMod();
		if (res <= 0)
			strcpy(newfname, "error_modifyvolume.html");
	}
	else if (!strcmp(inAct, "add"))
	{
		res = DoVolumeAdd();
		if (res <= 0)
			strcpy(newfname, "error_addvolume.html");
	}
	return res;
}

static void	HandleDisplayPage(char *newfname, int count, char **argname, char **argval)
{
	int res;

	if (count > 0)
	{
		getInputParams(count, argname, argval);
		res = HandleVolAction(newfname, count, argname, argval);
		if (res <= 0)
			return;
	}

	// get row data for page table
	getPageTable();
}


/***********************************************************
 * routines to volume information thru anpssc
 **********************************************************/

static int getTableRowData(int *index, ea_fsysCfg_t *data)
{
	if (*index < 1 || *index > dataCount)
		return -1;
	if (volumeList == NULL)
		return -1;

	memcpy(data, &volumeList[(*index)-1], sizeof(ea_fsysCfg_t));

	return 0;
}


/***********************************************************
 *  Main routines to process tags
 **********************************************************/

static int getTagValue(char *tag, char *value)
{
	int i, tagType, priv;
	uint64 size;

	tagType = INVALID_TAG;
	for (i = 0; i < NUM_OF_TAGS; i++) {
		if (!strcmp(tag, tagTable[i].tagName)) {
			tagType = tagTable[i].tagType;
			break;
		}
	}
	if (tagType == INVALID_TAG)
		return -1;

	switch (tagType) 
	{
	case VOL_COLSPAN:
		priv = GetUserPrivilege();
		if (priv >= PRIV_FULL)
			sprintf(value, "%d", 5);
		else if (priv == PRIV_CHANGE)
			sprintf(value, "%d", 4);
		else
			sprintf(value, "%d", 3);
		break;
	case VOL_VOLID:
		sprintf(value, "%lld", stData.fsysId.volId.whole_thing);
		break;
	case VOL_VOLNAME:
		strcpy(value, stData.fsysId.fsysName);
		break;
	case VOL_STATUS:
		strcpy(value, volState_str[stData.onlineStatus]);
		break;
	case VOL_FILESYS:
		if (stData.fsState)
			sprintf(value, "%s/%s", fs_str[1], vfsState_str[stData.mountStatus]);
		else
			sprintf(value, "%s/%s", fs_str[0], vfsState_str[stData.mountStatus]);
		break;
	case VOL_USED:
		size = GetVolumeDispUsedSize(stData.usedSize);
		sprintf(value, "%lld MB", size);
		break;
	case VOL_ALLOC:
		size = GetMBfromBlock(stData.currSize);
		sprintf(value, "%lld MB", size);
		break;
	case VOL_HARDQ:
		if (stData.maxSize)
		{
			size = GetMBfromBlock(stData.maxSize);
			sprintf(value, "%lld MB", size);
		}
		else
			strcpy(value, "No Limit");
		break;
	case VOL_SOFTQ:
		sprintf(value, "%d%%", stData.softQuota);
		break;
	case VOL_MAKE:
		strcpy(value, stData.vendorInfo[0].vendor);
		if (!value[0])
			strcpy(value, "Any");
		break;
	case VOL_MODEL:
		strcpy(value, stData.vendorInfo[0].model);
		if (!value[0])
			strcpy(value, "Any");
		break;
	case VOL_RAID:
		GetRaidLevelStr(stData.raidLevel, value);
		break;
	default:
		return -1;
	}

	return 1;
}


/*
 * actRptTagFunc()
 *	tag   - tag name
 *	value - replacement string for tag
 */
static int actRptTagFunc(tagCallbackParams_t *tp)
{
char *tag = tp->tag;
char *value = tp->value;

	if (!strcmp(tag, "TABLE_BEGIN"))
	{
		/* TABLE_BEGIN tag, need to see if there is any data */
		dataCount = totalEntries;
		if (dataCount > 0)
		{
			/* need to retrieve first table's data */
			dataCurr = 1;
			if (getTableRowData(&dataCurr, &stData) >= 0)
				return 1;
		}

		dataCurr = 0;
		return -1;
	}
	else if (!strcmp(tag, "TABLE_END"))
	{
		/* TABLE_END tag, check if end of data */
		if ((dataCount <= 0) || (dataCurr <= 0) || (dataCurr >= dataCount))
			return -1;

		dataCurr++;
		if (getTableRowData(&dataCurr, &stData) < 0)
			return -1;
		else
			return 1;
	}
	else 
	{
		/* other tags */
		return getTagValue(tag, value);
	}

	/* shouldn't reach here... */
	return -1;
}

int actionVolMain(void *r, int count, char **argname, char **argval)
{
	char newfname[128];

	memset(newfname, 0, sizeof(newfname));

	/* Find entries for the current displaying page */
	/* Also handle Add/Delete/Modify actions */
	volumeList = NULL;
	totalEntries = 0;
	HandleDisplayPage(newfname, count, argname, argval);

	prepare_agile_template(r, newfname);

	/* initialize volume information variables */
	dataCount = 0;
	dataCurr = 0;

	agile_scan_and_send(r, actRptTagFunc);

	if (volumeList != NULL)
		free(volumeList);
	volumeList = NULL;

	return 1;
}
