/*
 *  ICMP MIB group implementation - icmp.c
 *
 */

#include <config.h>
#include <sys/param.h>
#include <sys/socket.h>
#include <string.h>
#include <netinet/in.h>
#include <net/if.h>
#include <netinet/in_systm.h>
#include <netinet/ip.h>
#include <netinet/ip_icmp.h>
#include <netinet/icmp_var.h>
#include <sys/sysctl.h>

#include "tools.h"
#include "mibincl.h"
#include "util_funcs.h"
#include "system.h"
#include "asn1.h"
#include "snmp_debug.h"

#include "auto_nlist.h"

#include "icmp.h"
#include "sysORTable.h"

#include <ipmd-lib.h>
#include <snmpmsgapi.h>
#include "ipifhelp.h"

#define ICMP_STATS_CACHE_TIMEOUT	MIB_STATS_CACHE_TIMEOUT
marker_t icmp_stats_cache_marker = NULL;
// static struct timeval msg_timeout = {3, 0};  // We'll wait 3 sec

	/*********************
	 *
	 *  Kernel & interface information,
	 *   and internal forward declarations
	 *
	 *********************/


	/*********************
	 *
	 *  Initialisation & common implementation functions
	 *
	 *********************/


/* define the structure we're going to ask the agent to register our
   information at */
struct variable2 icmp_variables[] = {
	{ICMPINMSGS, ASN_COUNTER, RONLY, var_icmp, 1, {1}},
	{ICMPINERRORS, ASN_COUNTER, RONLY, var_icmp, 1, {2}},
	{ICMPINDESTUNREACHS, ASN_COUNTER, RONLY, var_icmp, 1, {3}},
	{ICMPINTIMEEXCDS, ASN_COUNTER, RONLY, var_icmp, 1, {4}},
	{ICMPINPARMPROBS, ASN_COUNTER, RONLY, var_icmp, 1, {5}},
	{ICMPINSRCQUENCHS, ASN_COUNTER, RONLY, var_icmp, 1, {6}},
	{ICMPINREDIRECTS, ASN_COUNTER, RONLY, var_icmp, 1, {7}},
	{ICMPINECHOS, ASN_COUNTER, RONLY, var_icmp, 1, {8}},
	{ICMPINECHOREPS, ASN_COUNTER, RONLY, var_icmp, 1, {9}},
	{ICMPINTIMESTAMPS, ASN_COUNTER, RONLY, var_icmp, 1, {10}},
	{ICMPINTIMESTAMPREPS, ASN_COUNTER, RONLY, var_icmp, 1, {11}},
	{ICMPINADDRMASKS, ASN_COUNTER, RONLY, var_icmp, 1, {12}},
	{ICMPINADDRMASKREPS, ASN_COUNTER, RONLY, var_icmp, 1, {13}},
	{ICMPOUTMSGS, ASN_COUNTER, RONLY, var_icmp, 1, {14}},
	{ICMPOUTERRORS, ASN_COUNTER, RONLY, var_icmp, 1, {15}},
	{ICMPOUTDESTUNREACHS, ASN_COUNTER, RONLY, var_icmp, 1, {16}},
	{ICMPOUTTIMEEXCDS, ASN_COUNTER, RONLY, var_icmp, 1, {17}},
	{ICMPOUTPARMPROBS, ASN_COUNTER, RONLY, var_icmp, 1, {18}},
	{ICMPOUTSRCQUENCHS, ASN_COUNTER, RONLY, var_icmp, 1, {19}},
	{ICMPOUTREDIRECTS, ASN_COUNTER, RONLY, var_icmp, 1, {20}},
	{ICMPOUTECHOS, ASN_COUNTER, RONLY, var_icmp, 1, {21}},
	{ICMPOUTECHOREPS, ASN_COUNTER, RONLY, var_icmp, 1, {22}},
	{ICMPOUTTIMESTAMPS, ASN_COUNTER, RONLY, var_icmp, 1, {23}},
	{ICMPOUTTIMESTAMPREPS, ASN_COUNTER, RONLY, var_icmp, 1, {24}},
	{ICMPOUTADDRMASKS, ASN_COUNTER, RONLY, var_icmp, 1, {25}},
	{ICMPOUTADDRMASKREPS, ASN_COUNTER, RONLY, var_icmp, 1, {26}}
};

/* Define the OID pointer to the top of the mib tree that we're
   registering underneath */
oid icmp_variables_oid[] = { SNMP_OID_MIB2,5 };
extern oid ip_module_oid[];
extern int ip_module_oid_len;
extern int ip_module_count;

void init_icmp(void)
{
	/* register ourselves with the agent to handle our mib tree */
	REGISTER_MIB("mibII/icmp", icmp_variables, variable2, icmp_variables_oid);

	if (++ip_module_count == 2)
		REGISTER_SYSOR_TABLE(ip_module_oid, ip_module_oid_len,
			"The MIB module for managing IP and ICMP implementations");

	auto_nlist(ICMPSTAT_SYMBOL,0,0);
}


	/*********************
	 *
	 *  System specific implementation functions
	 *
	 *********************/

long read_icmp_stat(ipm_icpstat_t *, int);


u_char *
var_icmp(struct variable *vp,
	 oid *name,
	 size_t *length,
	 int exact,
	 size_t *var_len,
	 WriteMethod **write_method)
{
	static ipm_icpstat_t icmpstat;
	static long ret_value;
	int i;

	if (header_generic(vp, name, length, exact, var_len, write_method) == MATCH_FAILED )
		return NULL;

	ret_value = read_icmp_stat(&icmpstat, vp->magic);
	if (ret_value < 0)
		return NULL;

	switch (vp->magic) {
	case ICMPINMSGS:
		long_return = icmpstat.icps_badcode + icmpstat.icps_tooshort +
						icmpstat.icps_checksum + icmpstat.icps_badlen;
		for (i = 0; i <= ICMP_MAXTYPE; i++)
			long_return += icmpstat.icps_inhist[i];
		return (u_char *)&long_return;
	case ICMPINERRORS:
		long_return = icmpstat.icps_badcode + icmpstat.icps_tooshort +
						icmpstat.icps_checksum + icmpstat.icps_badlen;
		return (u_char *)&long_return;
	case ICMPINDESTUNREACHS:
		long_return = icmpstat.icps_inhist[ICMP_UNREACH];
		return (u_char *)&long_return;
	case ICMPINTIMEEXCDS:
		long_return = icmpstat.icps_inhist[ICMP_TIMXCEED];
		return (u_char *)&long_return;
	case ICMPINPARMPROBS:
		long_return = icmpstat.icps_inhist[ICMP_PARAMPROB];
		return (u_char *)&long_return;
	case ICMPINSRCQUENCHS:
		long_return = icmpstat.icps_inhist[ICMP_SOURCEQUENCH];
		return (u_char *)&long_return;
	case ICMPINREDIRECTS:
		long_return = icmpstat.icps_inhist[ICMP_REDIRECT];
		return (u_char *)&long_return;
	case ICMPINECHOS:
		long_return = icmpstat.icps_inhist[ICMP_ECHO];
		return (u_char *)&long_return;
	case ICMPINECHOREPS:
		long_return = icmpstat.icps_inhist[ICMP_ECHOREPLY];
		return (u_char *)&long_return;
	case ICMPINTIMESTAMPS:
		long_return = icmpstat.icps_inhist[ICMP_TSTAMP];
		return (u_char *)&long_return;
	case ICMPINTIMESTAMPREPS:
		long_return = icmpstat.icps_inhist[ICMP_TSTAMPREPLY];
		return (u_char *)&long_return;
	case ICMPINADDRMASKS:
		long_return = icmpstat.icps_inhist[ICMP_MASKREQ];
		return (u_char *)&long_return;
	case ICMPINADDRMASKREPS:
		long_return = icmpstat.icps_inhist[ICMP_MASKREPLY];
		return (u_char *)&long_return;
	case ICMPOUTMSGS:
		long_return = icmpstat.icps_oldshort + icmpstat.icps_oldicmp;
		for (i = 0; i <= ICMP_MAXTYPE; i++)
			long_return += icmpstat.icps_outhist[i];
		return (u_char *)&long_return;
	case ICMPOUTERRORS:
		long_return = icmpstat.icps_oldshort + icmpstat.icps_oldicmp;
		return (u_char *)&long_return;
	case ICMPOUTDESTUNREACHS:
		long_return = icmpstat.icps_outhist[ICMP_UNREACH];
		return (u_char *)&long_return;
	case ICMPOUTTIMEEXCDS:
		long_return = icmpstat.icps_outhist[ICMP_TIMXCEED];
		return (u_char *)&long_return;
	case ICMPOUTPARMPROBS:
		long_return = icmpstat.icps_outhist[ICMP_PARAMPROB];
		return (u_char *)&long_return;
	case ICMPOUTSRCQUENCHS:
		long_return = icmpstat.icps_outhist[ICMP_SOURCEQUENCH];
		return (u_char *)&long_return;
	case ICMPOUTREDIRECTS:
		long_return = icmpstat.icps_outhist[ICMP_REDIRECT];
		return (u_char *)&long_return;
	case ICMPOUTECHOS:
		long_return = icmpstat.icps_outhist[ICMP_ECHO];
		return (u_char *)&long_return;
	case ICMPOUTECHOREPS:
		long_return = icmpstat.icps_outhist[ICMP_ECHOREPLY];
		return (u_char *)&long_return;
	case ICMPOUTTIMESTAMPS:
		long_return = icmpstat.icps_outhist[ICMP_TSTAMP];
		return (u_char *)&long_return;
	case ICMPOUTTIMESTAMPREPS:
		long_return = icmpstat.icps_outhist[ICMP_TSTAMPREPLY];
		return (u_char *)&long_return;
	case ICMPOUTADDRMASKS:
		long_return = icmpstat.icps_outhist[ICMP_MASKREQ];
		return (u_char *)&long_return;
	case ICMPOUTADDRMASKREPS:
		long_return = icmpstat.icps_outhist[ICMP_MASKREPLY];
		return (u_char *)&long_return;

	default:
		DEBUGMSGTL(("snmpd", "unknown sub-id %d in var_icmp\n", vp->magic));
	}

	return NULL;
}


	/*********************
	 *
	 *  Internal implementation functions
	 *
	 *********************/

long
read_icmp_stat(ipm_icpstat_t *icmpstat, int magic)
{
	long ret_value = 0;
	int gbeslots[32], numofgbeslots;
	ipm_icpstat_t tmpstat;
	struct icmpstat sscicmpstats;
	int count, needed, i;

	if (icmp_stats_cache_marker &&
		(!atime_ready(icmp_stats_cache_marker, ICMP_STATS_CACHE_TIMEOUT*1000)))
		return 0;

	if (icmp_stats_cache_marker )
		atime_setMarker( icmp_stats_cache_marker );
	else
		icmp_stats_cache_marker = atime_newMarker();

	/* first get ICMP stats on SSC ports */
	if (auto_nlist(ICMPSTAT_SYMBOL,
			(char *)&sscicmpstats, sizeof(struct icmpstat)) < 0) {
		ret_value = -1;
		goto readstat_finished;
	}

	/* now get ICMP stats from IPM for GbE ports */
	memset(icmpstat, 0, sizeof(ipm_icpstat_t));
	numofgbeslots = 32;
	if (IpInterface_Get_GbE_Slot(gbeslots, &numofgbeslots, NULL) >= 0) {
		int rval;

		/* for now, we assume we have only one GbE slot */
		count = 1;
		needed = 0;
		rval = ipmd_GetIcmpStats(gbeslots[0],
				&tmpstat, sizeof(ipm_icpstat_t), &count, &needed);
		if ((rval == IPM_GET_NO_BUF) || (rval == IPM_CMDERR_APPNOBUF)) {
			/* need to do something here.... */
		}
		else if (rval == IPM_GET_ERR) {
			/* something wrong here... */
		}
		else {
			/* assume there is only one GbE slot */
			/* if more than one, then we need to */
			/* add all the stats together. But   */
			/* we don't support that yet...	  */
			memcpy(icmpstat, &tmpstat, sizeof(ipm_icpstat_t));
		}
	}
	else {
		/* this might be buffer not big enough err */
		/* NEED TO HANDLE IT LATER... */
	}

	/* combine both SSC and GbE stats */
	icmpstat->icps_error      += sscicmpstats.icps_error;
	icmpstat->icps_oldshort   += sscicmpstats.icps_oldshort;
	icmpstat->icps_oldicmp    += sscicmpstats.icps_oldicmp;
	icmpstat->icps_badcode    += sscicmpstats.icps_badcode;
	icmpstat->icps_tooshort   += sscicmpstats.icps_tooshort;
	icmpstat->icps_checksum   += sscicmpstats.icps_checksum;
	icmpstat->icps_badlen     += sscicmpstats.icps_badlen;
	icmpstat->icps_reflect    += sscicmpstats.icps_reflect;
	icmpstat->icps_bmcastecho += sscicmpstats.icps_bmcastecho;
	for (i = 0; i <= ICMP_MAXTYPE; i++) {
		icmpstat->icps_inhist[i]  += sscicmpstats.icps_inhist[i];
		icmpstat->icps_outhist[i] += sscicmpstats.icps_outhist[i];
	}

readstat_finished:
	if (ret_value < 0) {
		free(icmp_stats_cache_marker);
		icmp_stats_cache_marker = NULL;
	}
	return ret_value;
}
