/*
 *  IP MIB group implementation - ip.c
 *
 */

#include <config.h>

#include <sys/param.h>
#include <sys/socket.h>

#include <string.h>
#include <sys/sysctl.h>
#include <net/if.h>
#include <netinet/in.h>
#include <netinet/in_systm.h>
#include <netinet/in_var.h>
#include <netinet/ip.h>
#include <netinet/ip_var.h>
#include <net/route.h>
#include <syslog.h>

#include "tools.h"
#include "kernel.h"

#include "mibincl.h"
#include "util_funcs.h"
#include "system.h"
#include "auto_nlist.h"

#include "ip.h"
#include "ipAddr.h"
#include "interfaces.h"
#include "sysORTable.h"

#include <ipmd-lib.h>
#include <snmpmsgapi.h>

#include "ipifhelp.h"

#define IP_STATS_CACHE_TIMEOUT	MIB_STATS_CACHE_TIMEOUT
marker_t ip_stats_cache_marker = NULL;

// static struct timeval msg_timeout = {3, 0};  // We'll wait 3 sec

	/*********************
	 *
	 *  Kernel & interface information,
	 *  and internal forward declarations
	 *
	 *********************/


	/*********************
	 *
	 *  Initialisation & common implementation functions
	 *
	 *********************/

extern void init_routes (void);


/* define the structure we're going to ask the agent to register our
   information at */
struct variable4 ip_variables[] = {
	{IPFORWARDING, ASN_INTEGER, RONLY, var_ip, 1, {1 }},
	{IPDEFAULTTTL, ASN_INTEGER, RONLY, var_ip, 1, {2 }},
	{IPINRECEIVES, ASN_COUNTER, RONLY, var_ip, 1, {3 }},
	{IPINHDRERRORS, ASN_COUNTER, RONLY, var_ip, 1, {4 }},
	{IPINADDRERRORS, ASN_COUNTER, RONLY, var_ip, 1, {5 }},
	{IPFORWDATAGRAMS, ASN_COUNTER, RONLY, var_ip, 1, {6 }},
	{IPINUNKNOWNPROTOS, ASN_COUNTER, RONLY, var_ip, 1, {7 }},
	{IPINDISCARDS, ASN_COUNTER, RONLY, var_ip, 1, {8 }},
	{IPINDELIVERS, ASN_COUNTER, RONLY, var_ip, 1, {9 }},
	{IPOUTREQUESTS, ASN_COUNTER, RONLY, var_ip, 1, {10 }},
	{IPOUTDISCARDS, ASN_COUNTER, RONLY, var_ip, 1, {11 }},
	{IPOUTNOROUTES, ASN_COUNTER, RONLY, var_ip, 1, {12 }},
	{IPREASMTIMEOUT, ASN_INTEGER, RONLY, var_ip, 1, {13 }},
	{IPREASMREQDS, ASN_COUNTER, RONLY, var_ip, 1, {14 }},
	{IPREASMOKS, ASN_COUNTER, RONLY, var_ip, 1, {15 }},
	{IPREASMFAILS, ASN_COUNTER, RONLY, var_ip, 1, {16 }},
	{IPFRAGOKS, ASN_COUNTER, RONLY, var_ip, 1, {17 }},
	{IPFRAGFAILS, ASN_COUNTER, RONLY, var_ip, 1, {18 }},
	{IPFRAGCREATES, ASN_COUNTER, RONLY, var_ip, 1, {19 }},
	{IPADADDR, ASN_IPADDRESS, RONLY, var_ipAddrEntry, 3, {20, 1, 1}},
	{IPADIFINDEX, ASN_INTEGER, RONLY, var_ipAddrEntry, 3, {20, 1, 2}},
	{IPADNETMASK, ASN_IPADDRESS, RONLY, var_ipAddrEntry, 3, {20, 1, 3}},
	{IPADBCASTADDR, ASN_INTEGER, RONLY, var_ipAddrEntry, 3, {20, 1, 4}},
	{IPADREASMMAX, ASN_INTEGER, RONLY, var_ipAddrEntry, 3, {20, 1, 5}},
	{IPROUTEDEST, ASN_IPADDRESS, RONLY, var_ipRouteEntry, 3, {21, 1, 1}},
	{IPROUTEIFINDEX, ASN_INTEGER, RONLY, var_ipRouteEntry, 3, {21, 1, 2}},
	{IPROUTEMETRIC1, ASN_INTEGER, RONLY, var_ipRouteEntry, 3, {21, 1, 3}},
	{IPROUTEMETRIC2, ASN_INTEGER, RONLY, var_ipRouteEntry, 3, {21, 1, 4}},
	{IPROUTEMETRIC3, ASN_INTEGER, RONLY, var_ipRouteEntry, 3, {21, 1, 5}},
	{IPROUTEMETRIC4, ASN_INTEGER, RONLY, var_ipRouteEntry, 3, {21, 1, 6}},
	{IPROUTENEXTHOP, ASN_IPADDRESS, RONLY, var_ipRouteEntry, 3, {21, 1, 7}},
	{IPROUTETYPE, ASN_INTEGER, RONLY, var_ipRouteEntry, 3, {21, 1, 8}},
	{IPROUTEPROTO, ASN_INTEGER, RONLY, var_ipRouteEntry, 3, {21, 1, 9}},
	{IPROUTEAGE, ASN_INTEGER, RONLY, var_ipRouteEntry, 3, {21, 1, 10}},
	{IPROUTEMASK, ASN_IPADDRESS, RONLY, var_ipRouteEntry, 3, {21, 1, 11}},
	{IPROUTEMETRIC5, ASN_INTEGER, RONLY, var_ipRouteEntry, 3, {21, 1, 12}},
	{IPROUTEINFO, ASN_OBJECT_ID, RONLY, var_ipRouteEntry, 3, {21, 1, 13}},
	{IPMEDIAIFINDEX, ASN_INTEGER, RONLY, var_atEntry, 3, {22, 1, 1}},
	{IPMEDIAPHYSADDRESS, ASN_OCTET_STR, RONLY, var_atEntry, 3, {22, 1, 2}},
	{IPMEDIANETADDRESS, ASN_IPADDRESS, RONLY, var_atEntry, 3, {22, 1, 3}},
	{IPMEDIATYPE, ASN_INTEGER, RONLY, var_atEntry, 3, {22, 1, 4}},
	{IPROUTEDISCARDS, ASN_COUNTER, RONLY, var_ip, 1, {23 }}
};

/* Define the OID pointer to the top of the mib tree that we're
   registering underneath, and the OID of the MIB module */
oid ip_variables_oid[] = { SNMP_OID_MIB2,4 };
oid ip_module_oid[]	= { SNMP_OID_MIB2,4 };
oid ip_module_oid_len  = sizeof(ip_module_oid) / sizeof( oid );
int ip_module_count	= 0;   /* Need to liaise with icmp.c */

void init_ip(void)
{
  /* register ourselves with the agent to handle our mib tree */
  REGISTER_MIB("mibII/ip", ip_variables, variable4, ip_variables_oid);
  if ( ++ip_module_count == 2 )
	  REGISTER_SYSOR_ENTRY( ip_module_oid,
		"The MIB module for managing IP and ICMP implementations");
}


	/*********************
	 *
	 *  System specific implementation functions
	 *
	 *********************/

long read_ip_stat(ipm_ipstat_t *, int);

u_char *
var_ip(struct variable *vp,
	oid *name,
	size_t *length,
	int exact,
	size_t *var_len,
	WriteMethod **write_method)
{
	static ipm_ipstat_t ipstat;
	static long ret_value;

	if (header_generic(vp, name, length, exact, var_len, write_method) == MATCH_FAILED )
	return NULL;

	if (vp->magic == IPFORWARDING) {
		ret_value = 2; /* fixed to not forwarding */
		return (u_char *) &ret_value;
	}

	ret_value = read_ip_stat(&ipstat, vp->magic);
	if (ret_value < 0)
		return NULL;

	switch (vp->magic) {
	case IPDEFAULTTTL:
		ret_value = ipstat.ips_defaultTTL;
		return (u_char *) &ret_value;
	case IPINRECEIVES:
		long_return = ipstat.ips_total;
		return (u_char *) &long_return;
	case IPINHDRERRORS:
		long_return = ipstat.ips_badsum + ipstat.ips_tooshort +
						ipstat.ips_toosmall + ipstat.ips_badhlen +
						ipstat.ips_badlen;
		return (u_char *) &long_return;
	case IPINADDRERRORS:
		long_return = ipstat.ips_cantforward;
		return (u_char *) &long_return;
	case IPFORWDATAGRAMS:
		long_return = ipstat.ips_forward;
		return (u_char *) &long_return;
	case IPINUNKNOWNPROTOS:
		long_return = ipstat.ips_noproto;
		return (u_char *) &long_return;
	case IPINDISCARDS:
		long_return = ipstat.ips_fragdropped;
		return (u_char *) &long_return;
	case IPINDELIVERS:
		long_return = ipstat.ips_delivered;
		return (u_char *) &long_return;
	case IPOUTREQUESTS:
		long_return = ipstat.ips_localout;
		return (u_char *) &long_return;
	case IPOUTDISCARDS:
		long_return = ipstat.ips_odropped;
		return (u_char *) &long_return;
	case IPOUTNOROUTES:
		long_return = ipstat.ips_noroute;
		return (u_char *) &long_return;
	case IPREASMTIMEOUT:
		long_return = IPFRAGTTL;
		return (u_char *) &long_return;
	case IPREASMREQDS:
		long_return = ipstat.ips_fragments;
		return (u_char *) &long_return;
	case IPREASMOKS:
		long_return = ipstat.ips_reassembled;
		return (u_char *) &long_return;
	case IPREASMFAILS:
		long_return = ipstat.ips_fragdropped + ipstat.ips_fragtimeout;
		return (u_char *) &long_return;
	case IPFRAGOKS:
		long_return = ipstat.ips_fragments -
						(ipstat.ips_fragdropped + ipstat.ips_fragtimeout);
		return (u_char *) &long_return;
	case IPFRAGFAILS:
		long_return = ipstat.ips_cantfrag;
		return (u_char *) &long_return;
	case IPFRAGCREATES:
		long_return = ipstat.ips_ofragments;
		return (u_char *) &long_return;
	case IPROUTEDISCARDS:
		long_return = ipstat.ips_noroute;
		return (u_char *) &long_return;

	default:
		DEBUGMSGTL(("snmpd", "unknown sub-id %d in var_ip\n", vp->magic));
	}
	return NULL;

}



	/*********************
	 *
	 *  Internal implementation functions
	 *
	 *********************/

long
read_ip_stat(ipm_ipstat_t *ipstat, int magic )
{
	long ret_value = -1;
	int gbeslots[32], numofgbeslots;
	ipm_ipstat_t tmpstat;
	struct ipstat sscipstat;
	int count, needed;

	if (ip_stats_cache_marker &&
		(!atime_ready( ip_stats_cache_marker, IP_STATS_CACHE_TIMEOUT*1000 )))
		return 0;

	if (ip_stats_cache_marker )
		atime_setMarker( ip_stats_cache_marker );
	else
		ip_stats_cache_marker = atime_newMarker();

	/* read stats for SSC ethernet ports */
	if (auto_nlist(IPSTAT_SYMBOL, (char *)&sscipstat, sizeof(struct ipstat)))
		ret_value = 0;
	else
		goto finished_readipstat;

	/* read stats for FP-NIM GbE ports */
	memset(ipstat, 0, sizeof(ipm_ipstat_t));
	numofgbeslots = 32;
	if (IpInterface_Get_GbE_Slot(gbeslots, &numofgbeslots, NULL) >= 0) {
		int rval;

		/* for now, we assume we have only one GbE slot */
		count = 1;
		needed = 0;
		rval = ipmd_GetIpStats(gbeslots[0],
				&tmpstat, sizeof(ipm_ipstat_t), &count, &needed);
		if ((rval == IPM_GET_NO_BUF) || (rval == IPM_CMDERR_APPNOBUF)) {
			/* need to do something here.... */
		}
		else if (rval == IPM_GET_ERR) {
			/* something wrong here... */
		}
		else {
			/* assume there is only one GbE slot */
			/* if more than one, then we need to */
			/* add all the stats together. But   */
			/* we don't support that yet...      */
			memcpy(ipstat, &tmpstat, sizeof(ipm_ipstat_t));
			rval = 0;
		}
	}

	/* now combine stats from SSC and FP-NIM */
	ipstat->ips_total        += sscipstat.ips_total;
	ipstat->ips_badsum       += sscipstat.ips_badsum;      
	ipstat->ips_tooshort     += sscipstat.ips_tooshort;
	ipstat->ips_toosmall     += sscipstat.ips_toosmall;
	ipstat->ips_badhlen      += sscipstat.ips_badhlen;
	ipstat->ips_badlen       += sscipstat.ips_badlen;
	ipstat->ips_fragments    += sscipstat.ips_fragments;
	ipstat->ips_fragdropped  += sscipstat.ips_fragdropped;
	ipstat->ips_fragtimeout  += sscipstat.ips_fragtimeout;
	ipstat->ips_forward      += sscipstat.ips_forward;
	ipstat->ips_cantforward  += sscipstat.ips_cantforward;
	ipstat->ips_redirectsent += sscipstat.ips_redirectsent;
	ipstat->ips_noproto      += sscipstat.ips_noproto;
	ipstat->ips_delivered    += sscipstat.ips_delivered;
	ipstat->ips_localout     += sscipstat.ips_localout;
	ipstat->ips_odropped     += sscipstat.ips_odropped;
	ipstat->ips_reassembled  += sscipstat.ips_reassembled;
	ipstat->ips_fragmented   += sscipstat.ips_fragmented;
	ipstat->ips_ofragments   += sscipstat.ips_ofragments;
	ipstat->ips_cantfrag     += sscipstat.ips_cantfrag;
	ipstat->ips_badoptions   += sscipstat.ips_badoptions;
	ipstat->ips_noroute      += sscipstat.ips_noroute;
	ipstat->ips_badvers      += sscipstat.ips_badvers;
	ipstat->ips_rawout       += sscipstat.ips_rawout;
	ipstat->ips_toolong      += sscipstat.ips_toolong;

finished_readipstat:
	if (ret_value < 0) {
		free(ip_stats_cache_marker);
		ip_stats_cache_marker = NULL;
	}
	return ret_value;
}
