/*
 *  IP MIB group implementation - ip.c
 *
 */

#include <config.h>

#include <sys/param.h>
#include <sys/socket.h>
#include <string.h>
#include <sys/types.h>
#include <sys/sysctl.h>
#include <netinet/in.h>
#include <net/if.h>
#include <net/if_dl.h>
#include <net/if_types.h>
#include <netinet/in_systm.h>
#include <netinet/in_var.h>
#include <netinet/ip.h>
#include <netinet/ip_var.h>
#include <net/route.h>
#include <syslog.h>

#include "kernel.h"
#include "mibincl.h"
#include "system.h"
#include "auto_nlist.h"
#include "ip.h"
#include "ipifhelp.h"
#include "sysORTable.h"


	/*********************
	 *
	 *  Kernel & interface information,
	 *   and internal forward declarations
	 *
	 *********************/


	/*********************
	 *
	 *  Internal implementation functions
	 *
	 *********************/


	/*********************
	 *
	 *  Initialisation & common implementation functions
	 *
	 *********************/

	/*********************
	 *
	 *  System specific implementation functions
	 *
	 *********************/

/*
  var_ipAddrEntry(...
  Arguments:
  vp      IN      - pointer to variable entry that points here
  name    IN/OUT  - IN/name requested, OUT/name found
  length  IN/OUT  - length of IN/OUT oid's 
  exact   IN      - TRUE if an exact match was requested
  var_len OUT     - length of variable or 0 if function returned
  write_method
*/

u_char *
var_ipAddrEntry(struct variable *vp,
		oid *name,
		size_t *length,
		int exact,
		size_t *var_len,
		WriteMethod **write_method)
{
	/*
	 * object identifier is of form:
	 * 1.3.6.1.2.1.4.20.1.?.A.B.C.D,  where A.B.C.D is IP address.
	 * IPADDR starts at offset 10.
	 */
	oid		lowest[14];
	oid		current[14], *op;
	u_char	*cp;
	int		lowinterface=0;
	short	interface;
	static struct in_ifaddr in_ifaddr;
	static struct in_ifaddr lowin_ifaddr;
	static struct ifnet ifnet;

	/* fill in object part of name for current (less sizeof instance part) */

	memcpy( (char *)current,(char *)vp->name, (int)vp->namelen * sizeof(oid));

	Interface_Scan_Init();
	for (;;) {

		if (Interface_Scan_Next(&interface, NULL, &ifnet, &in_ifaddr) == 0)
			break;
		cp = (u_char *)&(((struct sockaddr_in *) &(in_ifaddr.ia_addr))->sin_addr.s_addr);

		op = current + 10;
		*op++ = *cp++;
		*op++ = *cp++;
		*op++ = *cp++;
		*op++ = *cp++;
		if (exact){
			if (snmp_oid_compare(current, 14, name, *length) == 0){
				memcpy( (char *)lowest,(char *)current, 14 * sizeof(oid));
				lowinterface = interface;
				lowin_ifaddr = in_ifaddr;
				break;	/* no need to search further */
			}
		} else {
			if ((snmp_oid_compare(current, 14, name, *length) > 0) &&
			 (!lowinterface || (snmp_oid_compare(current, 14, lowest, 14) < 0))){
				/*
				 * if new one is greater than input and closer to input than
				 * previous lowest, save this one as the "next" one.
				 */
				lowinterface = interface;
				lowin_ifaddr = in_ifaddr;
				memcpy( (char *)lowest,(char *)current, 14 * sizeof(oid));
			}
		}
	}

	if (!lowinterface) return(NULL);
		memcpy( (char *)name,(char *)lowest, 14 * sizeof(oid));
	*length = 14;
	*write_method = 0;
	*var_len = sizeof(long_return);
	switch(vp->magic){
	case IPADADDR:
		return(u_char *) &((struct sockaddr_in *) &lowin_ifaddr.ia_addr)->sin_addr.s_addr;
	case IPADIFINDEX:
		long_return = lowinterface;
		return(u_char *) &long_return;
	case IPADNETMASK:
		long_return = lowin_ifaddr.ia_subnetmask;
		return(u_char *) &long_return;
	case IPADBCASTADDR:
		long_return = ntohl(((struct sockaddr_in *) &lowin_ifaddr.ia_broadaddr)->sin_addr.s_addr) & 1;
		return(u_char *) &long_return;	   
	case IPADREASMMAX:
		return NULL;
		long_return = -1;
		return(u_char *) &long_return;
	default:
		DEBUGMSGTL(("snmpd", "unknown sub-id %d in var_ipAddrEntry\n", vp->magic));
	}
	return NULL;
}


	/*********************
	 *
	 *  Internal implementation functions
	 *
	 *********************/

