/*
 *  UDP MIB group Table implementation - udpTable.c
 *
 */

#include <config.h>

#include <string.h>
#include <sys/types.h>
#include <sys/param.h>
#include <netinet/in.h>
#include <sys/socket.h>
#include <net/if.h>
#include <net/route.h>
#include <netinet/in_systm.h>
#include <netinet/ip.h>
#include <sys/queue.h>
#include <sys/socketvar.h>
#include <netinet/ip_var.h>
#include <netinet/in_pcb.h>
#include <netinet/udp.h>
#include <netinet/udp_var.h>

#include "kernel.h"
#include "system.h"
#include "asn1.h"
#include "snmp_debug.h"
#include "mibincl.h"
#include "auto_nlist.h"
#include "udp.h"
#include "udpTable.h"
#include "sysORTable.h"

#include <sys/sysctl.h>

#include <anpssc-api.h>
#include <ipmd-lib.h>


// static struct timeval msg_timeout = {3, 0};  // We'll wait 3 sec

	/*********************
	 *
	 *  Kernel & interface information,
	 *   and internal forward declarations
	 *
	 *********************/

	/*********************
	 *
	 *  Initialisation & common implementation functions
	 *
	 *********************/


	/*********************
	 *
	 *  System specific implementation functions
	 *
	 *********************/


u_char *
var_udpEntry(struct variable *vp,
	     oid *name,
	     size_t *length,
	     int exact,
	     size_t *var_len,
	     WriteMethod **write_method)
{
	static ipm_conn_t *udpconn = NULL, *nextconn = NULL;
    int i;
	uint32_t localIP, localport;
	unsigned char *ipaddr;

/* we dropped support for UDP Table for now */
return NULL;

	// IP address in network order
	ipaddr = (unsigned char *) &localIP;
	for (i = 0; i < 4; i++)
		ipaddr[i] = (unsigned char) name[10+i];

	localport = name[14];

	if (udpconn) {
		if ((udpconn->conn_lAddr != localIP) ||
			(udpconn->conn_lPort != localport)) {
			/* A different request came in, gotta search */
			/* from beginning again */
			udpconn = NULL;
		}
	}

	while (1) {
#if 0
		nextconn = ipmd_GetTcpNxtConn(udpconn);
#endif
		udpconn = nextconn;

		if (nextconn == NULL)
			return NULL;	/* no match */

		if (nextconn->conn_lAddr > localIP) {
			if (exact)
				return NULL; /* no match */
			else {
/* need to search SSC side of UDP table */
				break;
			}
		}
		if (nextconn->conn_lAddr < localIP)
			continue;

		if (nextconn->conn_lPort > localport) {
			if (exact)
				return NULL; /* no match */
			else
				break;	/* found */
		}
		if (nextconn->conn_lPort < localport)
			continue;

		if (exact)
			break;		/* found */
		else
			continue;	/* need next one */
	}

	if (udpconn == NULL)
		return NULL;

	localIP = udpconn->conn_lAddr;
	ipaddr = (unsigned char *) &localIP;
	for (i = 0; i < 4; i++)
		name[10+i] = (oid) ipaddr[i];
	name[14] = (oid) udpconn->conn_lPort;
	*length = vp->namelen + 5;
	*write_method = 0;
	*var_len = sizeof(long);
	switch (vp->magic) {
	case UDPLOCALADDRESS:
		long_return = udpconn->conn_lAddr;
		return (u_char *) &long_return;
	case UDPLOCALPORT:
		long_return = udpconn->conn_lPort;
		return (u_char *) &long_return;
	default:
		DEBUGMSGTL(("snmpd", "unknown sub-id %d in var_udpEntry\n", vp->magic));
	}
    return  NULL;
}

	/*********************
	 *
	 *  Internal implementation functions
	 *
	 *********************/

