/* snmpmsgapi.h */

#ifndef _SNMPMSGAPI_H_
#define _SNMPMSGAPI_H_

typedef enum {
	SAMSGTYPE_PORTSTATS = 1,
	SAMSGTYPE_ENABLEPORT,
	SAMSGTYPE_IPSTATS,
	SAMSGTYPE_ICMPSTATS,
	SAMSGTYPE_TCPSTATS,
	SAMSGTYPE_TCPCONN_ENTRY,
	SAMSGTYPE_UDPSTATS,
	SAMSGTYPE_UDP_ENTRY
} saMsgType_t;


typedef struct {
	saMsgType_t	msgType;		// message type
	int32_t		clientOpaque;	// client data
	int32_t		status;			// response status: 0-failed, 1-OK
#if 0
	char		msgBody[];		// message body
#endif
} saMessage_t;   /* sa stands for SNMP Agent */


/* Physicaly port stats for MIB-II     */
/* Use SAMSGTYPE_PORTSTATS for msgType */

typedef struct {
	saMessage_t hdr;
	uint32_t	iftype;			// iftype (6 - enet, 117 - GbE, 56 - FC)
	uint32_t	MTU;			// interface MTU
	uint32_t	speed;			// in bits/second
	uint32_t	physAddrSize;	// actual no. of bytes used in next field
	char		physAddr[32];	// physical address
	uint32_t	admStatus;		// administration status: 1 up 2 down 3 testing
	uint32_t	opStatus;		// curr operating status: 1 up 2 down 3 testing
	uint32_t	lastChange;		// up/down moment system clock
	uint64_t	iBytes;			// total number of bytes data inbound
	uint32_t	iUcastPkts;		// unicast pkts inbound
	uint32_t	iNUcastPkts;	// non-unicast pkts inbound
	uint32_t	iDiscards;		// discarded pkts inbound
	uint32_t	iErrors;		// error pkts inbound
	uint32_t	iUnknownProto;	// unknown protocol pkts inbound
	uint64_t	oBytes;			// total number of bytes data outbound
	uint32_t	oUcastPkts;		// unicast pkts outbound
	uint32_t	oNUcastPkts;	// non-unicast pkts outbound
	uint32_t	oDiscards;		// discarded pkts outbound
	uint32_t	oErrors;		// error pkts outbound
	uint32_t	oQueueLen;		// Length of output pkt queue (in pkts)
} msgPortStat_t;


/* Enable/disable physicaly port        */
/* Use SAMSGTYPE_ENABLEPORT for msgType */

typedef struct {
	saMessage_t hdr;
	uint32_t	enable;	/* 1: enable, 2: disable, 3: testing */
} msgEnablePort_t;


/* Card-level IP stats               */
/* Use SAMSGTYPE_IPSTATS for msgType */

typedef struct {
	saMessage_t hdr;
	uint32_t	defaultTTL;	// default time-to-live
	uint32_t	iPkts;	// IP pkts inbound
	uint32_t	iHdrErrors;	// IP pkts header error inbound
	uint32_t	iAddrErrors;	// IP pkts address error inbound
	uint32_t	iUnknownProto;	// IP pkts discarded for unknown proto in
	uint32_t	iDiscards;	// IP pkts discarded inbound
	uint32_t	iDelivers;	// IP pkts successfully delivered inbound
	uint32_t	oRequests;	// IP pkts requested for transmission outbound
	uint32_t	oDiscards;	// IP pkts discarded outbound
	uint32_t	reasmTimeout;	// time to wait for fragments to be received (in)
	uint32_t	reasmRequests;	// IP fragments received (in)
	uint32_t	reasmOKs;	// IP fragments successfully re-assembled (in)
	uint32_t	reasmFails;	// IP fragments failed re-assemble (in)
	uint32_t	fragOKs;	// IP fragmentation successfully done (out)
	uint32_t	fragFails;	// IP fragmentation failed (out)
	uint32_t	fragCreates;	// IP fragmentation generated (out)
} msgIpStats_t;


/* Card-level ICMP stats               */
/* Use SAMSGTYPE_ICMPSTATS for msgType */

typedef struct {
	saMessage_t hdr;
	uint32_t	iIcmps;	// ICMP packets received
	uint32_t	iErrors;	// ICMP error packets received
	uint32_t	iDestUnrchs;	// ICMP Dest Unreachable packets received
	uint32_t	iTimeExceeds;	// ICMP Time Exceeded packets received
	uint32_t	iParmProbs;	// ICMP Param Problem packets received
	uint32_t	iSrcQuenchs;	// ICMP Source Quench packets received
	uint32_t	iRedirects;	// ICMP Redirect packets received
	uint32_t	iEchos;	// ICMP Echo packets received
	uint32_t	iEchoReplys;	// ICMP Echo Reply packets received
	uint32_t	iTimestamps;	// ICMP Timestamp request packets received
	uint32_t	iTmstmpReps;	// ICMP Timestamp Reply packets received
	uint32_t	iAdrMsks;	// ICMP Address Mask request packets received
	uint32_t	iAdrMskReps;	// ICMP Address Mask Reply packets received
	uint32_t	oIcmps;	// ICMP packets attempted to send
	uint32_t	oErrors;	// ICMP error packets not sent
	uint32_t	oDestUnrchs;	// ICMP Dest Unreachable packets sent
	uint32_t	oTimeExceeds;	// ICMP Time Exceeded packets sent
	uint32_t	oParmProbs;	// ICMP Param Problem packets sent
	uint32_t	oSrcQuenchs;	// ICMP Source Quench packets sent
	uint32_t	oRedirects;	// ICMP Redirect packets sent
	uint32_t	oEchos;	// ICMP Echo packets sent
	uint32_t	oEchoReplys;	// ICMP Echo Reply packets sent
	uint32_t	oTimestamps;	// ICMP Timestamp request packets sent
	uint32_t	oTmstmpReps;	// ICMP Timestamp Reply packets sent
	uint32_t	oAdrMsks;	// ICMP Address Mask request packets sent
	uint32_t	oAdrMskReps;	// ICMP Address Mask Reply packets sent
} msgIcmpStats_t;


/* Card-level TCP stats               */
/* Use SAMSGTYPE_TCPSTATS for msgType */

typedef struct {
	saMessage_t hdr;
	uint32_t	activeOpens;	// no. of direct CLOSED to SYN-SENT transitions
	uint32_t	passiveOpens;	// no. of direct LISTEN to SYN-RCVD transitions
	uint32_t	attmpFails;	// no. of direct SYN-SENT/RCVD to CLOSED trans
	uint32_t	estabResets;	// no. of direct ESTABLISH/CLOSE-WAIT to CLOSED
	uint32_t	currEstabs;	// connections curr state is ESTABLISHED/CLOSE-WAIT
	uint32_t	iSegs;	// total segments received
	uint32_t	oSegs;	// total segments sent (not include reoSegs)
	uint32_t	reoSegs;	// total segments retransmitted
	uint32_t	iErrors;	// total segments received in error
	uint32_t	oRsts;	// total segments sent with RST flag
} msgTcpStats_t;


/* Card-level TCPCONN table                */
/* Use SAMSGTYPE_TCPCONN_ENTRY for msgType */

typedef enum {
	CONNSTATE_INVALID = 0,	// no entry available
	CONNSTATE_CLOSED,
	CONNSTATE_LISTEN,
	CONNSTATE_SYNSENT,
	CONNSTATE_SYNRCVD,
	CONNSTATE_ESTAB,
	CONNSTATE_FINWAIT1,
	CONNSTATE_FINWAIT2,
	CONNSTATE_CLOSEWAIT,
	CONNSTATE_LASTACK,
	CONNSTATE_CLOSING,
	CONNSTATE_TIMEWAIT,
	CONNSTATE_DELETETCB
} connState_t;

typedef struct {
	saMessage_t hdr;
	uint32_t	target;	// 0: this, 1: next
	uint32_t	ipLocal;	// connection's local IP address
	uint16_t	portLocal;	// connection's local IP port
	uint32_t	ipRemote;	// connection's remote IP address
	uint16_t	portRemote;	// connection's remote IP port
	connState_t state;	// connection's state
} msgTcpConn_t;


/* Card-level UDP stats               */
/* Use SAMSGTYPE_UDPSTATS for msgType */

typedef struct {
	saMessage_t hdr;
	uint32_t	iPackets;	// packets received
	uint32_t	iNoPorts;	// packets received with none listener port
	uint32_t	iErrors;	// packets not passed up (exclude iNoPorts)
	uint32_t	oPackets;	// packets sent
} msgUdpStats_t;


/* Card-level UDP table                */
/* Use SAMSGTYPE_UDP_ENTRY for msgType */

typedef struct {
	saMessage_t hdr;
	uint32_t	target;		// 0: this, 1: next
	uint32_t	ipLocal;	// Listener's IP address (return 0 if failed)
	uint32_t	portLocal;	// Listener's IP port number (return 0 if failed)
} msgUdpTable_t;

#endif /* _SNMPMSGAPI_H_ */

