/* vi:ts=4  
 *-----------------------------------------------------------------
 *
 * Name: flags.h
 *
 * RCS:  $Id: flags.h,v 1.18 2002/10/22 01:30:30 jong Exp $
 *
 * Copyright (C) 2001, Agile Storage, Inc.
 *
 * Description: This files contains port related flags, macros, and 
 *				declarations.
 *
 * Created by:	Su Wang
 *
 * Date Created: 08/09/01
 *
 *-----------------------------------------------------------------
 */

#define KERNEL		1
#define INET		1
#define BIG_ENDIAN	1
#define LITTLE_ENDIAN   2

#define BYTE_ORDER	LITTLE_ENDIAN

#define AG_EXTENSIONS
#define BSD_TEST
// #define TCPDEBUG


#include <exports/malloc-api.h>
#if !defined(PROM)
#include <exports/elog-api.h>
#endif /* PROM */
#include <exports/timer-api.h>
#include "../sm-bsd-sys/types.h"

/* su, put malloc definitions here. Not logical but most 
   convinient */
#define malloc(size, x, y) (void*)eee_ramAlloc(size)
#define free(ptr, x)         eee_ramDealloc((uchar8*)ptr)


#define nfx_time_tics	(1000)
#if defined(PROM)
#define  nfx_time_second (((uint32)(eee_timerGetCurrTic()/100) > 5)?(eee_timerGetCurrTic()/100):5)
#else
#define nfx_time_second ((eee_timerGetCurrTic()/100) + 1)   /* tic is incremented per 10 ms */
#endif /* PROM */
#define nfx_clk_freq	1000
#define	nfx_cur_pid		0

#define iptime()		(eee_timerGetCurrTic())


#define random()		eee_timerGetCurrTic() /* should be no major problem  */

/* error log and panic functions */ 
#define BSD_PANIC( )			{ *(char *)0 = 1;}
#define bsd_log					printf("File: %s, Line: %d\n", __FILE__,  __LINE__),\
								bsd_log_f
#define log					    bsd_log

#define BSD_ERR_MBUF_DUPF		(BSD_ERROR_BASE+30) /* our own err code */
#define BSD_ERR_MBUF_DUPA		(BSD_ERROR_BASE+31)
#define BSD_ERR_GEN_PANIC		(BSD_ERROR_BASE+32)
#define BSD_ERR_INTF			(BSD_ERROR_BASE+33)
#define BSD_ERR_IPRT			(BSD_ERROR_BASE+34)
#define BSD_ERR_SOCK			(BSD_ERROR_BASE+35)
#define BSD_ERR_TCP_TX			(BSD_ERROR_BASE+36)
#define BSD_ERR_TCP_RX			(BSD_ERROR_BASE+37)
#define BSD_ERR_RL_PKT			(BSD_ERROR_BASE+38)

#define BSD_WARN_INTF			(BSD_ERROR_BASE+40)
#define BSD_WARN_IPRT			(BSD_ERROR_BASE+41)
#define BSD_WARN_SOCK			(BSD_ERROR_BASE+42)
		
#define BSD_INFO_ARPT_START		(BSD_ERROR_BASE+60)


#define BSD_NOTICE_INTF			(BSD_ERROR_BASE+50)

#define BSD_LOG_LAST			(BSD_ERROR_BASE+0xFF)


#define bcmp	memcmp

/* need forward declaration to resolve a warning */
void * hashinit();

/* dummy function declarations */
int		splnet(void);
int		splimp(void);
void	splx( int s);
void	getmicrotime(struct timeval *tvp);
void	getmicrouptime(struct timeval *tvp);
int		tvtohz( struct timeval *tv);
void	funsetown(void *sigio);
int		wakeup(void);
int		wakeup_one(void *chan);
int		selwakeup(void *chan);
int		microtime (struct timeval*);
int		copyout(caddr_t dest, caddr_t src, int len);
int		copyin(const void *uaddr, void *kdaddr, size_t len);

