/*
 *-----------------------------------------------------------------
 *
 * Name:
 *
 * RCS:  $Id: chassisd-msg.c,v 1.8 2002/11/19 04:43:32 tonyw Exp $
 *
 * Copyright (C) 2002, Agile Storage, Inc.
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	4/18/02
 *
 *-----------------------------------------------------------------
 */

#ifdef SSC

#include <stdio.h>
#include <string.h>
#include <sys/socket.h>

#include <nfx-incl.h>
#include <exports/anpssc-api.h>
#include <exports/cm-api.h>
#include "cm.h"

int	chassisd_rxif;

#define CHASSISD_DEBUG_RX_MSG	0
#define CHASSISD_DEBUG_TX_MSG	0

int chassisd_msgInit(void);
int chassisd_sendMsg(void *, int, char *, uint32,
									uint32, struct timeval*, void *, uint32);
int cm_sendInfoReq(void);
int cm_sendVersionReq(int slot, int cpu);

int chassisd_rcvMsg(void);
int chassisd_procRcvMsg(char *, int, uchar8 *, uint32, uint32);
int chassisd_procRebootReq(chassisd_msgDesc_t * mdesc);
int chassisd_procInfoReq(chassisd_msgDesc_t * mdesc);
int chassisd_procVersionReq(chassisd_msgDesc_t * mdesc);
int chassisd_procPowerReq(chassisd_msgDesc_t * mdesc);
int chassisd_procTempReq(chassisd_msgDesc_t * mdesc);
int chassisd_procDebugReq(chassisd_msgDesc_t * mdesc);

int cm_procInfoResp(chassisd_msgDesc_t * mdesc);
int cm_procVersionResp(chassisd_msgDesc_t * mdesc);
int cm_procVersionReq(chassisd_msgDesc_t * mdesc);


/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	4/18/02
 *
 *-----------------------------------------------------------------
 */
int
chassisd_msgInit()
{
	chassisd_rxif = regAgileApp("cm");
	if (chassisd_rxif < 0) {
		perror("chassisd error registering for messages");
		exit(1);
	}

	return 0;
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	4/18/02
 *
 *-----------------------------------------------------------------
 */
int
chassisd_sendMsg(
	void	* msg,
	int		len,
	char	* app,
	uint32	to_cpu,
	uint32	to_slot,
	struct timeval	* to,
	void	* rsp_buf,
	uint32	rsp_len
)
{
	int	rc;

#if CHASSISD_DEBUG_TX_MSG
	printf("CHASSISD tx msg: %d bytes to app: %s, cpu: %d, slot: %d\n",
													len, app, to_cpu, to_slot);
#endif

	rc = sendAgileMsg(	chassisd_rxif,
						(char *)msg,
						len,
						app,
						to_cpu,
						to_slot,
						0,
						to,
						(char *)rsp_buf,
						rsp_len);

	if (rc < 0)
		printf("chassisd send msg error\n");

	return rc;
}

/*-----------------------------------------------------------------
 * Name: cm_sendInfoReq()
 *
 * Description: When chassisd starts, send info req message to Toshiba
 *				to retrieve current state of system
 *
 * Created by:	Rick Lund
 *
 * Date Created:	4/24/02
 *
 *-----------------------------------------------------------------
 */
int
cm_sendInfoReq()
{
	cmMsgInfoReq_t		req;
	cmMsgInfoResp_t		resp;
	chassisd_msgDesc_t	mdesc;
    struct timeval		to;
	int					rc;

	to.tv_sec = 0;
	to.tv_usec = 500;

	bzero(&resp, sizeof(resp));

	req.hdr.mid = CM_SET_INFO_REQ;
	req.hdr.srcSlot = 0;
	req.hdr.srcCpu = 0;
	req.hdr.srcApp = chassisd_rxif;
	req.hdr.srcPlane = MGMTPLANE;

	rc = chassisd_sendMsg(&req, sizeof(req), "cm",
							1, 0, &to,
							&resp, sizeof(resp));

	if (rc == sizeof(resp)) {
		if (resp.hdr.mid == CM_SET_INFO_RESP) {
			//printf("Got Info Response message\n");
			mdesc.msg = &resp;
			mdesc.len = rc;
			mdesc.srcCpu = 1;
			mdesc.srcSlot = 0;
			cm_procInfoResp(&mdesc);
		}
		else
			printf("Error in response from SET_INFO_REQ, mid = 0x%x\n",
																resp.hdr.mid);
	}

	return NFX_OK;
}

/*-----------------------------------------------------------------
 * Name: cm_sendVersionReq(slot, cpu)
 *
 * Description: When chassisd starts, send version req messages to all
 *				cpus to get their version strings
 *
 * Created by:	Rick Lund
 *
 * Date Created:	4/25/02
 *
 *-----------------------------------------------------------------
 */
int
cm_sendVersionReq(
	int		slot,
	int		cpu
)
{
	cmMsgVersionReq_t	req;
	chassisd_msgDesc_t	mdesc;
	int					rc;

	req.hdr.mid = CM_SET_VERSION_REQ;
	req.hdr.srcSlot = 0;
	req.hdr.srcCpu = 0;
	req.hdr.srcApp = chassisd_rxif;
	req.hdr.srcPlane = MGMTPLANE;

	rc = chassisd_sendMsg(&req, sizeof(req), "cm",
							cpu, slot, NULL,
							0, 0);

	return NFX_OK;
}

/*-----------------------------------------------------------------
 * Name: chassisd_rcvMsg()
 *
 * Description: Registered receive function for "cm" application on BSD
 *
 * Created by:	Rick Lund
 *
 * Date Created:	4/18/02
 *
 *-----------------------------------------------------------------
 */
int
chassisd_rcvMsg()
{
	int		bytes;
	uint32	srcCpu, srcSlot, srcPlane;
	char	srcApp[32];
	char	msg[2000];

	bytes = recvAgileMsg(	chassisd_rxif,
							&msg,
							sizeof(msg),
							NULL,
							srcApp,
							&srcCpu,
							&srcSlot,
							&srcPlane);

	if (bytes < 0)
		return 0;
	else if (bytes == 0)
		return 0;
	else {
#if CHASSISD_DEBUG_RX_MSG
		printf("CHASSISD rx msg: %d bytes from app: %s, cpu: %d, slot: %d\n",
											bytes, srcApp, srcCpu, srcSlot);
#endif
		chassisd_procRcvMsg(msg, bytes, srcApp, srcCpu, srcSlot);
	}

	return 0;
}

/*-----------------------------------------------------------------
 * Name: chassisd_procRcvMsg()
 *
 * Description: process the message received by chassisd
 *
 * Created by:	Rick Lund
 *
 * Date Created:	4/18/02
 *
 *-----------------------------------------------------------------
 */
int
chassisd_procRcvMsg(
	char	* msg,
	int		mlen,
	uchar8	* srcApp,
	uint32	srcCpu,
	uint32	srcSlot
)
{
	chassisd_msg_hdr_t	* hdr;
	uint32				mid;
	chassisd_msgDesc_t	mdesc;
	int slot, cpu;

	hdr = (chassisd_msg_hdr_t *)msg;

	mdesc.msg = msg;
	mdesc.len = mlen;
	mdesc.srcCpu = srcCpu;
	mdesc.srcSlot = srcSlot;
	memcpy(mdesc.srcApp, srcApp, 32);

	mid = hdr->mid;

	switch(mid) {
		case CHASSISD_REBOOT_REQ:
			if (mlen < sizeof(chassisdMsgRebootReq_t)) {
#if CHASSISD_DEBUG_RX_MSG
				printf("msg too small, REBOOT_REQ, mlen = %d\n", mlen);
#endif
			}
			else
				chassisd_procRebootReq(&mdesc);
			break;

		case CHASSISD_GET_CHASSIS_INFO_REQ:
			if (mlen < sizeof(chassisdMsgInfoReq_t)) {
#if CHASSISD_DEBUG_RX_MSG
				printf("msg too small, CHASSIS_INFO_REQ, mlen = %d\n", mlen);
#endif
			}
			else
				chassisd_procInfoReq(&mdesc);
			break;

		case CHASSISD_GET_VERSION_REQ:
		    for (slot=0; slot<CM_NUM_SLOTS; slot++) {
		        for (cpu=0; cpu<CM_NUM_CPUS; cpu++) {
		            cm_sendVersionReq(slot, cpu);
		        }
		    }

			if (mlen < sizeof(chassisdMsgVersionReq_t)) {
#if CHASSISD_DEBUG_RX_MSG
				printf("msg too small, GET_VERSION_REQ, mlen = %d\n", mlen);
#endif
			}
			else
				chassisd_procVersionReq(&mdesc);
			break;

		case CHASSISD_GET_POWER_REQ:
			if (mlen < sizeof(chassisdMsgPowerReq_t)) {
#if CHASSISD_DEBUG_RX_MSG
				printf("msg too small, GET_POWER_REQ, mlen = %d\n", mlen);
#endif
			}
			else
				chassisd_procPowerReq(&mdesc);
			break;

		case CHASSISD_GET_TEMP_REQ:
			if (mlen < sizeof(chassisdMsgTempReq_t)) {
#if CHASSISD_DEBUG_RX_MSG
				printf("msg too small, GET_TEMP_REQ, mlen = %d\n", mlen);
#endif
			}
			else
				chassisd_procTempReq(&mdesc);
			break;

		case CHASSISD_DEBUG_REQ:
			if (mlen < sizeof(chassisdMsgDebugReq_t)) {
#if CHASSISD_DEBUG_RX_MSG
				printf("msg too small, DEBUG_REQ, mlen = %d\n", mlen);
#endif
			}
			else
				chassisd_procDebugReq(&mdesc);
			break;

		case CM_SET_VERSION_REQ:
			if (mlen < sizeof(cmMsgVersionReq_t)) {
#if CHASSISD_DEBUG_RX_MSG
				printf("msg too small, CM_SET_VERSION_REQ, mlen = %d\n", mlen);
#endif
			}
			else
				cm_procVersionReq(&mdesc);
			break;

		case CM_SET_INFO_RESP:
			if (mlen < sizeof(cmMsgInfoResp_t)) {
#if CHASSISD_DEBUG_RX_MSG
				printf("msg too small, CM_SET_INFO_RESP, mlen = %d\n", mlen);
#endif
			}
			else
				cm_procInfoResp(&mdesc);
			break;

		case CM_SET_VERSION_RESP:
			if (mlen < sizeof(cmMsgVersionResp_t)) {
#if CHASSISD_DEBUG_RX_MSG
				printf("msg too small, CM_SET_VERSION_RESP, mlen = %d\n", mlen);
#endif
			}
			else
				cm_procVersionResp(&mdesc);
			break;

		default:
			printf("Invalid Message Received, mid = %d\n", mid);
	}

	return 0;

}

/*-----------------------------------------------------------------
 * Name: chassisd_procRebootReq()
 *
 * Description: process the RebootReq message sent by CLI/GUI to chassisd
 *
 * Created by:	Rick Lund
 *
 * Date Created:	4/18/02
 *
 *-----------------------------------------------------------------
 */
int
chassisd_procRebootReq(
	chassisd_msgDesc_t	* mdesc
)
{
	chassisdMsgRebootResp_t	resp;
	chassisdMsgRebootReq_t	* req;
	uint32	slot;
	uint32	cpu;

#if CHASSISD_DEBUG_RX_MSG
	printf("chassisd_procRebootReq: Enter\n");
#endif

	/*
	 * Send Reboot Request to toshiba
	 */
	chassisd_sendMsg(mdesc->msg, mdesc->len,
					"cm", NFX_SSC_MGMT, SSC_SLOT,
					NULL, 0, 0);

	/*
	 * Send Response back to requester
	 */
	resp.hdr.mid = CHASSISD_REBOOT_RESP;
	resp.rc = CM_REBOOT_OK;
	chassisd_sendMsg(&resp, sizeof(resp),
					mdesc->srcApp, mdesc->srcCpu, mdesc->srcSlot,
					NULL, 0, 0);

	/*
	 * Remove old version strings
	 */
	req = (chassisdMsgRebootReq_t *)(mdesc->msg);
	if (req->reboot_type == CM_REBOOT_TYPE_SLOT) {
		slot = req->slot;
		bzero(cm.slot[slot].bmfpga_ver, CM_VER_STRING_LEN);
		bzero(cm.slot[slot].luc_ver, CM_VER_STRING_LEN);
		bzero(cm.slot[slot].lmux_ver, CM_VER_STRING_LEN);
		bzero(cm.slot[slot].hw_ver, CM_VER_STRING_LEN);
		for (cpu = 0; cpu < CM_NUM_CPUS; cpu++) {
			bzero(cm.slot[slot].cpu[cpu].prom_ver, CM_VER_STRING_LEN);
			bzero(cm.slot[slot].cpu[cpu].rt_ver, CM_VER_STRING_LEN);
		}
	}
	else {
		slot = req->slot;
		cpu = req->slice;
		bzero(cm.slot[slot].cpu[cpu].prom_ver, CM_VER_STRING_LEN);
		bzero(cm.slot[slot].cpu[cpu].rt_ver, CM_VER_STRING_LEN);
	}

	return NFX_OK;
}

/*-----------------------------------------------------------------
 * Name: chassisd_procInfoReq()
 *
 * Description: process the InfoReq message sent by CLI/GUI to chassisd
 *				Returns info stored in the cm_t structure.
 *
 * Created by:	Rick Lund
 *
 * Date Created:	4/18/02
 *
 *-----------------------------------------------------------------
 */
int
chassisd_procInfoReq(
	chassisd_msgDesc_t	* mdesc
)
{
	chassisdMsgInfoResp_t	resp;
	int						slot, cpu;

#if CHASSISD_DEBUG_RX_MSG
	printf("chassisd_procInfoReq: Enter\n");
#endif

	/*
	 * Send Response back to requester
	 */
	bzero(&resp, sizeof(resp));

	resp.hdr.mid = CHASSISD_GET_CHASSIS_INFO_RESP;
	resp.chassis_type = 0;						/* TODO */
	for (slot=0; slot<CM_NUM_SLOTS;slot++) {
		if (cm.slot[slot].present) {
			resp.present[slot] = 1;
			resp.num_cpus[slot] = cm.slot[slot].num_cpus;
			resp.board_type[slot] = cm.slot[slot].type;
			for (cpu=0; cpu<cm.slot[slot].num_cpus; cpu++)
				resp.cpu_state[slot][cpu] = cm.slot[slot].cpu[cpu].state;
		}
		else {
			resp.present[slot] = 0;
			resp.board_type[slot] = 0;
			resp.num_cpus[slot] = 0;
			for (cpu=0; cpu<CM_NUM_CPUS; cpu++)
				resp.cpu_state[slot][cpu] = CM_SLICE_STATE_NOT_PRESENT;
		}
	}
	chassisd_sendMsg(&resp, sizeof(resp),
					mdesc->srcApp, mdesc->srcCpu, mdesc->srcSlot,
					NULL, 0, 0);

	return NFX_OK;
}

/*-----------------------------------------------------------------
 * Name: chassisd_procVersionReq()
 *
 * Description: process the VersionReq message sent by CLI/GUI to chassisd
 *				Returns info stored in the cm_t structure.
 *
 * Created by:	Rick Lund
 *
 * Date Created:	4/18/02
 *
 *-----------------------------------------------------------------
 */
int
chassisd_procVersionReq(
	chassisd_msgDesc_t	* mdesc
)
{
	chassisdMsgVersionResp_t	resp;
	int							slot, cpu;

#if CHASSISD_DEBUG_RX_MSG
	printf("chassisd_procVersionReq: Enter\n");
#endif

	/*
	 * Send Response back to requester
	 */
	bzero(&resp, sizeof(resp));

	resp.hdr.mid = CHASSISD_GET_VERSION_RESP;
	for (slot=0; slot<CM_NUM_SLOTS;slot++) {
		if (cm.slot[slot].present) {
			resp.present[slot] = 1;
			resp.board_type[slot] = cm.slot[slot].type;
			memcpy(resp.bmfpga_ver[slot], cm.slot[slot].bmfpga_ver,
														CM_VER_STRING_LEN);
			memcpy(resp.luc_ver[slot], cm.slot[slot].luc_ver,
														CM_VER_STRING_LEN);
			memcpy(resp.lmux_ver[slot], cm.slot[slot].lmux_ver,
														CM_VER_STRING_LEN);
			memcpy(resp.hw_ver[slot], cm.slot[slot].hw_ver,
														CM_VER_STRING_LEN);
//			for (cpu=0; cpu<cm.slot[slot].num_cpus; cpu++) {
			for (cpu=0; cpu<CM_NUM_CPUS; cpu++) {
				memcpy(resp.prom_ver[slot][cpu],cm.slot[slot].cpu[cpu].prom_ver,
														CM_VER_STRING_LEN);
				memcpy(resp.rt_ver[slot][cpu], cm.slot[slot].cpu[cpu].rt_ver,
														CM_VER_STRING_LEN);
			}
		}
	}
	chassisd_sendMsg(&resp, sizeof(resp),
					mdesc->srcApp, mdesc->srcCpu, mdesc->srcSlot,
					NULL, 0, 0);

	return NFX_OK;
}

/*-----------------------------------------------------------------
 * Name: chassisd_procPowerReq()
 *
 * Description: process the PowerReq message sent by CLI/GUI to chassisd
 *				Returns info stored in the cm_t structure.
 *
 * Created by:	Rick Lund
 *
 * Date Created:	4/18/02
 *
 *-----------------------------------------------------------------
 */
int
chassisd_procPowerReq(
	chassisd_msgDesc_t	* mdesc
)
{
	chassisdMsgPowerResp_t	resp;
	int						slot, cpu;

#if CHASSISD_DEBUG_RX_MSG
	printf("chassisd_procPowerReq: Enter\n");
#endif

	/*
	 * Send Response back to requester
	 */
	bzero(&resp, sizeof(resp));

	resp.hdr.mid = CHASSISD_GET_POWER_RESP;
	for (slot=0; slot<CM_NUM_SLOTS;slot++) {
		if (cm.slot[slot].present) {
			resp.present[slot] = 1;
			resp.valid[slot] = cm.slot[slot].power_valid;
			resp.power[slot] = cm.slot[slot].power;
		}
	}
	chassisd_sendMsg(&resp, sizeof(resp),
					mdesc->srcApp, mdesc->srcCpu, mdesc->srcSlot,
					NULL, 0, 0);

	return NFX_OK;
}

/*-----------------------------------------------------------------
 * Name: chassisd_procTempReq()
 *
 * Description: process the TempReq message sent by CLI/GUI to chassisd
 *				Returns info stored in the cm_t structure.
 *
 * Created by:	Rick Lund
 *
 * Date Created:	4/18/02
 *
 *-----------------------------------------------------------------
 */
int
chassisd_procTempReq(
	chassisd_msgDesc_t	* mdesc
)
{
	chassisdMsgTempResp_t	resp;
	int						slot, cpu;

#if CHASSISD_DEBUG_RX_MSG
	printf("chassisd_procTempReq: Enter\n");
#endif

	/*
	 * Send Response back to requester
	 */
	bzero(&resp, sizeof(resp));

	resp.hdr.mid = CHASSISD_GET_TEMP_RESP;
	for (slot=0; slot<CM_NUM_SLOTS;slot++) {
		if (cm.slot[slot].present) {
			resp.present[slot] = 1;
			resp.valid[slot] = cm.slot[slot].temp_valid;
			resp.temp[slot] = cm.slot[slot].temp;
		}
	}
	chassisd_sendMsg(&resp, sizeof(resp),
					mdesc->srcApp, mdesc->srcCpu, mdesc->srcSlot,
					NULL, 0, 0);

	return NFX_OK;
}

/*-----------------------------------------------------------------
 * Name:	cm_procVersionReq()
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	4/25/02
 *
 *-----------------------------------------------------------------
 */
int
cm_procVersionReq(
	chassisd_msgDesc_t	* mdesc
)
{
	cmMsgVersionReq_t	* buf = (cmMsgVersionReq_t *)(mdesc->msg);
	cm_msg_hdr_t		* hdr = (cm_msg_hdr_t *)buf;

#if CHASSISD_DEBUG_RX_MSG
	printf("cm_procVersionReq: Enter\n");
#endif

/*
	strncpy(cm.slot[hdr->srcSlot].cpu[hdr->srcCpu].prom_ver,
								 getPromImageVerString(), CM_VER_STRING_LEN);
*/

	return NFX_OK;
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	4/23/02
 *
 *-----------------------------------------------------------------
 */
int
cm_procInfoResp(
	chassisd_msgDesc_t	* mdesc
)
{
	cmMsgInfoResp_t	* buf = (cmMsgInfoResp_t *)(mdesc->msg);
	cm_msg_hdr_t	* hdr = (cm_msg_hdr_t *)buf;
	int				slot, cpu;

#if CHASSISD_DEBUG_RX_MSG
	printf("cm_procInfoResp: Enter\n");
#endif

	for (slot=0;slot<CM_NUM_SLOTS;slot++) {
		cm.slot[slot].present = buf->present[slot];
		cm.slot[slot].type = buf->board_type[slot];
		cm.slot[slot].temp = buf->temp[slot];
		cm.slot[slot].temp_valid = buf->temp_valid[slot];
		cm.slot[slot].power = buf->power[slot];
		cm.slot[slot].power_valid = buf->power_valid[slot];
		cm.slot[slot].num_cpus = buf->num_cpus[slot];
		for (cpu=0; cpu<buf->num_cpus[slot]; cpu++)
			cm.slot[slot].cpu[cpu].state = buf->cpu_state[slot][cpu];
	}

	return NFX_OK;
}

/*-----------------------------------------------------------------
 * Name: cm_procVersionResp()
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	4/21/02
 *
 *-----------------------------------------------------------------
 */
int
cm_procVersionResp(
	chassisd_msgDesc_t	* mdesc
)
{
	cmMsgVersionResp_t	* buf = (cmMsgVersionResp_t *)(mdesc->msg);
	cm_msg_hdr_t		* hdr = (cm_msg_hdr_t *)buf;

#if CHASSISD_DEBUG_RX_MSG
	printf("cm_procVersionResp: Enter\n");
#endif

	if (buf->which_ver == CM_VER_STRING_PROM) {
		strncpy(cm.slot[hdr->srcSlot].cpu[hdr->srcCpu].prom_ver,
											buf->ver, CM_VER_STRING_LEN);
	}
	else if (buf->which_ver == CM_VER_STRING_RT) {
		strncpy(cm.slot[hdr->srcSlot].cpu[hdr->srcCpu].rt_ver,
											buf->ver, CM_VER_STRING_LEN);
	}
	else if (buf->which_ver == CM_VER_STRING_BMFPGA) {
		strncpy(cm.slot[hdr->srcSlot].bmfpga_ver, buf->ver, CM_VER_STRING_LEN);
	}
	else if (buf->which_ver == CM_VER_STRING_LUC) {
		strncpy(cm.slot[hdr->srcSlot].luc_ver, buf->ver, CM_VER_STRING_LEN);
	}
	else if (buf->which_ver == CM_VER_STRING_LMUX) {
		strncpy(cm.slot[hdr->srcSlot].lmux_ver, buf->ver, CM_VER_STRING_LEN);
	}
	else if (buf->which_ver == CM_VER_STRING_HW) {
		strncpy(cm.slot[hdr->srcSlot].hw_ver, buf->ver, CM_VER_STRING_LEN);
	}

	return NFX_OK;
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	4/25/02
 *
 *-----------------------------------------------------------------
 */
int
chassisd_procDebugReq(
	chassisd_msgDesc_t	* mdesc
)
{
	chassisdMsgDebugResp_t	resp;

#if CHASSISD_DEBUG_RX_MSG
	printf("chassisd_procDebugReq: Enter\n");
#endif

	/*
	 * Send Response back to requester
	 */
	bzero(&resp, sizeof(resp));

	resp.hdr.mid = CHASSISD_DEBUG_RESP;

	memcpy(&(resp.cm), &cm, sizeof(cm_t));

	chassisd_sendMsg(&resp, sizeof(resp),
					mdesc->srcApp, mdesc->srcCpu, mdesc->srcSlot,
					NULL, 0, 0);

	return NFX_OK;
}
#endif	/* SSC */

