/*
 *-----------------------------------------------------------------
 *
 * Name:
 *
 * RCS:  $Id: chassisd.c,v 1.4 2002/04/30 20:46:47 rick Exp $
 *
 * Copyright (C) 2002, Agile Storage, Inc.
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	4/18/02
 *
 *-----------------------------------------------------------------
 */

#ifdef SSC

#include <stdio.h>
#include <sys/types.h>
#include <unistd.h>
#include <stdlib.h>
#include <signal.h>
#include <string.h>
#include <exports/anpssc-api.h>
#include <nfx-incl.h>
#include <exports/cm-api.h>
#include <exports/elog-api.h>
#include "cm.h"

cm_t	cm;

void chassisd_msgInit(void);
int chassisd_rcvMsg(void);
int	cm_sendInfoReq(void);
int	cm_sendVersionReq(uint32 slot, uint32 cpu);

static void     chassisd_sigTerm(int sig);
static void     chassisd_sigHup(int sig);

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	4/21/02
 *
 *-----------------------------------------------------------------
 */
int
main()
{
	int		slot, cpu;
	sigset_t	set;

	//-- Setup the signal handling.
	sigfillset(&set);
	sigdelset(&set, SIGALRM);
	sigdelset(&set, SIGTERM);
	sigdelset(&set, SIGHUP);
	sigdelset(&set, SIGCHLD);
	sigdelset(&set, SIGSEGV);
	sigdelset(&set, SIGBUS);
	sigprocmask(SIG_SETMASK, &set, NULL);

	//-- Set up our termination handler
	signal (SIGTERM, chassisd_sigTerm);
	signal (SIGHUP, chassisd_sigHup);

	bzero(&cm, sizeof(cm_t));

	chassisd_msgInit();

	/*
	 * Get current state of the chassis
	 */
	cm_sendInfoReq();

	/*
	 * Get version strings from everyone
	 */
	for (slot=0; slot<CM_NUM_SLOTS; slot++) {
		for (cpu=0; cpu<CM_NUM_CPUS; cpu++) {
			cm_sendVersionReq(slot, cpu);
		}
	}

	kill (getppid(), SIGUSR1);
	E_LOG (class_1, notice_s, eee_getApplicationId("cm"), 0, 0, 0,
						("CHASSISD: Agile Chassis Manager (c)2002: Started"));

	while (1)
		chassisd_rcvMsg();

	return 0;
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	4/25/02
 *
 *-----------------------------------------------------------------
 */
static void
chassisd_sigTerm(int sig)
{
	E_LOG (class_1, notice_s, eee_getApplicationId("cm"), 0, 0, 0,
				("CHASSISD: Agile Chassis Manager (c)2002: Terminated"));
	exit(0);
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	4/25/02
 *
 *-----------------------------------------------------------------
 */
static void
chassisd_sigHup(int sig)
{
	E_LOG (class_1, notice_s, eee_getApplicationId("cm"), 0, 0, 0,
				("CHASSISD: Agile Chassis Manager (c)2002: Re-Started"));
	exit(0);
}

#endif	/* SSC */

