/*
 *-----------------------------------------------------------------
 *
 * Name:	cm-api.h
 *
 * RCS:  $Id: cm-api.h,v 1.17 2002/12/04 01:46:14 rick Exp $
 *
 * Copyright (C) 2002, Agile Storage, Inc.
 *
 * Description: defines API messages for communication between various
 *				pieces of the ChassisManager.
 *
 * Created by:	Rick Lund
 *
 * Date Created:	4/15/02
 *
 *-----------------------------------------------------------------
 */

#ifndef _CM_API_H
#define _CM_API_H

#include "exports/eee-api.h"

void    cm_init(int32 pass);
void	cmInitCardType(uint32 type);
void    cmSetCardType(uint32 type);
void    cmSetCPUState(uint32 state);
void    cmSetCardTemp(uint32 temp);
void    cmSetCardPower(uint32 power);
int32   cmGetCardType(uint32 slot);
int32   cmGetCPUState(uint32 slot, uint32 cpu);
uchar8 *cmGetCpuString(uint32 slot_num, uint32 cpu_num);
int32   cmGetNumCpus(uint32 board_type); 
uint32  cmGetMySlotId(void);
uint32  cmGetMyCPUId(void);
uint32  cmGetMyCardType(void);
void    cmGpioCardStatePoll(void * cb, uint32 tref);
void    cm_rcvMsg(eee_descPtr_t	edesc);
void    cmSendVersionResp(int32	slot, int32	cpu);
void    cmSendInfoReq(void);
void    cmSendInfoResp(int32 slot, int32 cpu);
void    cmSendCpuEvent(uint32 slot, uint32 cpu, int	state);
void    cm_intrHandleStatusChange(uint32 sr, uint32 cause, address_t epc);
void    cm_intrHandleMessageChange(uint32 sr, uint32 cause, address_t epc);
void    cm_poll_bmfpga_intr(void *cb, uint32 tref);
void    cm_poll_module_present(void *cb, uint32 tref);
uchar8 *getPromImageVerString(void);

#ifdef R9000
void	cmInitFan( int32 ctrl_addr );
void	cmTempEna( int32 ctrl_addr );
#endif

#ifdef SSC_MGMT
void    cmRebootCard(uint32	slot_num);
void    cmRebootCPU(uint32 slot_num, uint32	cpu_num);
int32   cm_checkFanStatus(uint32, void * p1, void * p2, void * p3, void * p4);
void    cmSendInfoReq(void);
int		cm_checkFanFail(void);
int		cm_checkPowerFail(void);
void    sscm_led_set(int);
#endif

extern uint32		nfxMySlotId;	/* NFX_SLOT_MIN - NFX_SLOT_MAX */
extern uint32		nfxMyCoreId;	/* 0, 1, 2, 3 */
extern uint32		nfxMyCPUId;	    /* one of the CPU identifiers in nfx-defs.h */
extern uint32		nfxMyChassisId;
extern uint32		nfxSscSlotId;
extern uint32		cm_inited;
#ifdef R7000
extern uint32		cm_fc_slice0_state;
#endif

#ifdef SSC_MGMT
void cm_heartbeatSendInfo();
typedef struct {
	uint32	chassisdUp;
	uint32  attempts;
} chassis_heartbeat_t;
#endif

#if defined (NFP_TXRX) || (SSC_MGMT)
void cmProcHeartbeatInfo();
void cm_heartbeatInit();
void cm_Heartbeat(int timer);
void cm_heartbeat_timeout(int32 tmr_handle, void * usr_param0,
         void * usr_param1, void * usr_param2, void * usr_param3);
#endif

#define CM_NUM_CARDTYPES	4
#define CM_NUM_SLOTS		3
#define CM_NUM_CPUS			4

#define CM_SLICE_STATE_NOT_PRESENT			0
#define CM_SLICE_STATE_PROM_INIT			1
#define CM_SLICE_STATE_PROM_NO_AUTOBOOT		2
#define CM_SLICE_STATE_PROM_CRASHED			3
#define CM_SLICE_STATE_PROM_LOADING			4
#define CM_SLICE_STATE_SWITCH_TO_RUNTIME	5
#define CM_SLICE_STATE_RUNTIME_INIT			6
#define CM_SLICE_STATE_RUNTIME				7
#define CM_SLICE_STATE_RUNTIME_CRASHED		8
#define CM_SLICE_STATE_RUNTIME_REBOOT		9
#define CM_SLICE_STATE_RESET_WAIT           10

#define CM_SLICE_STATE_NOT_PRESENT_STR      "NOT PRESENT"
#define CM_SLICE_STATE_PROM_INIT_STR        "PROM_INIT"
#define CM_SLICE_STATE_PROM_NO_AUTOBOOT_STR "PROM"
#define CM_SLICE_STATE_PROM_CRASHED_STR     "PROM_CRASH"
#define CM_SLICE_STATE_PROM_LOADING_STR     "LOADING"
#define CM_SLICE_STATE_SWITCH_TO_RUNTIME_STR "CTEXT_SWITCH"
#define CM_SLICE_STATE_RUNTIME_INIT_STR     "RT_INIT"
#define CM_SLICE_STATE_RUNTIME_STR          "UP"
#define CM_SLICE_STATE_RUNTIME_CRASHED_STR  "DOWN"
#define CM_SLICE_STATE_RUNTIME_REBOOT_STR   "REBOOT"
#define CM_SLICE_STATE_RESET_WAIT_STR       "RESET_WAIT"

#define CM_CARD_TYPE_SSC		0
#define CM_CARD_TYPE_GARFIELD	1
#define CM_CARD_TYPE_CHEETAH	2
#define CM_CARD_TYPE_FCNIM		3

#define CM_CARD_TYPE_SSC_STR        "SSC"
#define CM_CARD_TYPE_GARFIELD_STR   "NFPNIM-A"
#define CM_CARD_TYPE_CHEETAH_STR    "NFPNIM-B"
#define CM_CARD_TYPE_FCNIM_STR      "FCNIM"

#define CM_CPU_TYPE_SSC_STR         "SSC"
#define CM_CPU_TYPE_SSC_MGMT_STR    "SSC_MGMT"
#define CM_CPU_TYPE_TXRX0_STR       "TXRX0"
#define CM_CPU_TYPE_TXRX1_STR       "TXRX1"
#define CM_CPU_TYPE_FP0_STR         "FP0"
#define CM_CPU_TYPE_FP1_STR         "FP1"
#define CM_CPU_TYPE_FC0_STR         "FC0"
#define CM_CPU_TYPE_FC1_STR         "FC1"
#define CM_CPU_TYPE_FC2_STR         "FC2"
#define CM_CPU_TYPE_FC3_STR         "FC3"

#define CM_VER_STRING_LEN		80
#define CM_VER_STRING_NUM		5
#define CM_VER_STRING_PROM		0
#define CM_VER_STRING_RT		1
#define CM_VER_STRING_BMFPGA	2
#define CM_VER_STRING_LUC		3
#define CM_VER_STRING_LMUX		4
#define CM_VER_STRING_HW		5

/* CpuId map cpuid to slice # */ 
#define CM_FC_CPUID2SLICE(_id)  (3 - (_id))
#define CM_FC_SLICE2CPUID(_sl)  (3 - (_sl))

#define CM_IS_FC_CTRL           (isFC() && (nfxMyCPUId == NFX_FCNIM_CTRL_CPU))
#define CM_IS_FC_CPUID3         (isFC() && (nfxMyCPUId == NFX_FCNIM_CPUID3))
#define CM_IS_FC_CPUID2         (isFC() && (nfxMyCPUId == NFX_FCNIM_CPUID2))
#define CM_IS_FC_CPUID1         (isFC() && (nfxMyCPUId == NFX_FCNIM_CPUID1))
#define CM_IS_FC_CPUID0         (isFC() && (nfxMyCPUId == NFX_FCNIM_CPUID0))
#define CM_IS_FC_CPUID012       (isFC() && (nfxMyCPUId != NFX_FCNIM_CPUID3))

#define CM_IS_TXRX_NCPU			(isTXRX() &&(nfxMyCPUId == NFX_NFP_TXRX1))

/*------------------------------------------------------------------
 * Message ID's
 *------------------------------------------------------------------*/

/*
 * CHASSISD_* messages are between cli/gui and chassisd daemon.
 * CM_* messages are between chassisd and embedded chassis management apps
 *
 * CHASSISD_*_REQ and CHASSISD_*_RESP messages are synchronous
 * CM_*_REQ and CM_*_RESP messages are asynchronous
 * CHASSISD_* and CM_* messages are asynchronous
 *
 * REQ messages to chassisd retrieve info stored in the local structures,
 * they do not get redistributed as messages to the other entities in the
 * system.
 *
 * embedded CM applications can send RESP messages without being originated
 * from a REQ message
 *
 */
typedef enum {
	CHASSISD_REBOOT_REQ = CM_MSG_BASE,
	CHASSISD_GET_CHASSIS_INFO_REQ,
	CHASSISD_GET_VERSION_REQ,
	CHASSISD_GET_POWER_REQ,
	CHASSISD_GET_TEMP_REQ,
	CHASSISD_DEBUG_REQ,
	CHASSISD_REBOOT_RESP,
	CHASSISD_GET_CHASSIS_INFO_RESP,
	CHASSISD_GET_VERSION_RESP,
	CHASSISD_GET_POWER_RESP,
	CHASSISD_GET_TEMP_RESP,
	CHASSISD_DEBUG_RESP,
	CM_SET_INFO_REQ,
	CM_SET_VERSION_REQ,
	CM_SET_POWER_REQ,
	CM_SET_TEMP_REQ,
	CM_SET_INFO_RESP,
	CM_SET_VERSION_RESP,
	CM_SET_POWER_RESP,
	CM_SET_TEMP_RESP,
	CM_SEND_HEARTBEAT_INFO,
	CM_SEND_HEARTBEAT_ACK
} cm_msgType_t;

/*------------------------------------------------------------------
 * chasssisd Messages (cli/gui <--> chassisd daemon)
 *------------------------------------------------------------------*/
typedef struct {
	uint32	mid;
} chassisd_msg_hdr_t;

/*----------------------- Request Messages -------------------------*/

typedef struct {
	chassisd_msg_hdr_t	hdr;
	uint32	reboot_type;
#define CM_REBOOT_TYPE_SLOT		0
#define CM_REBOOT_TYPE_SLICE	1
	uint32	slot;		/* 0, 1, 2 */
	uint32	slice;		/* 0, 1, 2, 3, or unused (for TYPE_SLOT) */
} chassisdMsgRebootReq_t;

typedef struct {
	chassisd_msg_hdr_t	hdr;
} chassisdMsgInfoReq_t;

typedef struct {
	chassisd_msg_hdr_t	hdr;
} chassisdMsgVersionReq_t;

typedef struct {
	chassisd_msg_hdr_t	hdr;
} chassisdMsgPowerReq_t;

typedef struct {
	chassisd_msg_hdr_t	hdr;
} chassisdMsgTempReq_t;

typedef struct {
	chassisd_msg_hdr_t	hdr;
} chassisdMsgDebugReq_t;

/*----------------------- Response Messages ------------------------*/

typedef struct {
	chassisd_msg_hdr_t	hdr;
	uint32				rc;
#define CM_REBOOT_OK			0
} chassisdMsgRebootResp_t;

typedef struct {
	chassisd_msg_hdr_t	hdr;
	uint32				chassis_type;
	uint32				present[CM_NUM_SLOTS];
	uint32				board_type[CM_NUM_SLOTS];
	uint32				num_cpus[CM_NUM_SLOTS];
	uint32				cpu_state[CM_NUM_SLOTS][CM_NUM_CPUS];
} chassisdMsgInfoResp_t;

typedef struct {
	chassisd_msg_hdr_t	hdr;
	uint32				present[CM_NUM_SLOTS];
	uint32				board_type[CM_NUM_SLOTS];
	uchar8				bmfpga_ver[CM_NUM_SLOTS][CM_VER_STRING_LEN];
	uchar8				luc_ver[CM_NUM_SLOTS][CM_VER_STRING_LEN];
	uchar8				lmux_ver[CM_NUM_SLOTS][CM_VER_STRING_LEN];
	uchar8				hw_ver[CM_NUM_SLOTS][CM_VER_STRING_LEN];
	uchar8				prom_ver[CM_NUM_SLOTS][CM_NUM_CPUS][CM_VER_STRING_LEN];
	uchar8				rt_ver[CM_NUM_SLOTS][CM_NUM_CPUS][CM_VER_STRING_LEN];
} chassisdMsgVersionResp_t;

typedef struct {
	chassisd_msg_hdr_t	hdr;
	uint32				present[CM_NUM_SLOTS];
	uint32				valid[CM_NUM_SLOTS];
	uint32				power[CM_NUM_SLOTS];
} chassisdMsgPowerResp_t;

typedef struct {
	chassisd_msg_hdr_t	hdr;
	uint32				present[CM_NUM_SLOTS];
	uint32				valid[CM_NUM_SLOTS];
	uint32				temp[CM_NUM_SLOTS];
} chassisdMsgTempResp_t;

/*------------------------------------------------------------------
 * CM Messages (daemon <--> embedded "cm" application)
 *------------------------------------------------------------------*/
typedef struct {
	uint32	mid;
	uint32	srcSlot;
	uint32	srcCpu;
	uint32	srcApp;
	uint32	srcPlane;
} cm_msg_hdr_t;

/*----------------------- Request Messages -------------------------*/

typedef struct {
	cm_msg_hdr_t	hdr;
} cmMsgInfoReq_t;

typedef struct {
	cm_msg_hdr_t	hdr;
} cmMsgVersionReq_t;

typedef struct {
	cm_msg_hdr_t	hdr;
} cmMsgPowerReq_t;

typedef struct {
	cm_msg_hdr_t	hdr;
} cmMsgTempReq_t;

/*----------------------- Response Messages ------------------------*/

typedef struct {
	cm_msg_hdr_t	hdr;
	uint32			chassis_type;
	uint32			present[CM_NUM_SLOTS];
	uint32			board_type[CM_NUM_SLOTS];
	uint32			temp_valid[CM_NUM_SLOTS];
	uint32			temp[CM_NUM_SLOTS];
	uint32			power_valid[CM_NUM_SLOTS];
	uint32			power[CM_NUM_SLOTS];
	uint32			num_cpus[CM_NUM_SLOTS];
	uint32			cpu_state[CM_NUM_SLOTS][CM_NUM_CPUS];
} cmMsgInfoResp_t;

typedef struct {
	cm_msg_hdr_t	hdr;
	uint32			which_ver;
	uchar8			ver[CM_VER_STRING_LEN];
} cmMsgVersionResp_t;

typedef struct {
	cm_msg_hdr_t	hdr;
	uint32			valid;
	uint32			power;
} cmMsgPowerResp_t;

typedef struct {
	cm_msg_hdr_t	hdr;
	uint32			valid;
	uint32			temp;
} cmMsgTempResp_t;

/*------------ Heartbeat Message ( Toshiba <-> SSC)--------------*/

typedef struct {
	cm_msg_hdr_t	hdr;
	uint32			chassis_type;
	uint32			present[CM_NUM_SLOTS];
	uint32			board_type[CM_NUM_SLOTS];
	uint32			num_cpus[CM_NUM_SLOTS];
	uint32			cpu_state[CM_NUM_SLOTS][CM_NUM_CPUS];
} cmHeartbeatMsgInfo_t;

typedef struct {
	cm_msg_hdr_t	hdr;
} cmHeartbeatAck_t;

/*------------------------------------------------------------------
 * BMFPGA Messages
 *------------------------------------------------------------------*/
typedef struct {
	uint32	mid;
} bmfpgaMsgInfoReq_t;

/*------------------------------------------------------------------
 * TXRX <--> FP Messages
 *------------------------------------------------------------------*/
typedef struct {
	uint32	mid;
	uint32	cpu;
	uint32	state;
} cmCpuStateMsg_t;

#endif	/* _CM_API_H */

