/*
 *-----------------------------------------------------------------
 *
 * Name:	cm-msg.c
 *
 * RCS:  $Id: cm-msg.c,v 1.32 2002/12/10 21:25:30 tonyw Exp $
 *
 * Copyright (C) 2002, Agile Storage, Inc.
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	4/18/02
 *
 *-----------------------------------------------------------------
 */

#ifndef SSC

#include <nfx-incl.h>
#include <sys/gt64240r.h>
#include <exports/hwMap.h>
#include <exports/eee-api.h>
#include <exports/cm-api.h>
#include <sys/bmfpga.h>
#include "cm.h"
#if !defined(PROM) && !defined(FCNIM_TM)
#include "exports/event-api.h"
#endif  /* !PROM && !FCNIM_TM */
#include <sys/led-api.h>

/* number of heartbeat attempts before SSC is considered "down"
 */
#define HB_FAIL_THRESHOLD 10

#ifdef SSC_MGMT
#include "exports/prom-part-api.h"
#include "exports/from-api.h"
#endif

#ifdef R7000
uint32	cm_fc_slice0_state;
#endif

uint32	cm_slot_status;

void cmSendVersionResp(int32 slot, int32 cpu);
void cmSendInfoResp(int32 slot, int32 cpu);
extern void cmGetCardTemp();

#if defined (NFP_TXRX)
extern int32 heartbeat_timer;
int32 eee_timerStop(uint32 tmr_handle);
#endif

#ifdef SSC_MGMT
chassis_heartbeat_t hb;
uchar8 * copySSCPromImageVerString(void);
uchar8	SSCImageVerString[CM_VER_STRING_LEN];
#endif

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	4/18/02
 *
 *-----------------------------------------------------------------
 */
void
cm_rcvMsg(
	eee_descPtr_t	edesc
)
{
	eee_descBufPtr_t	bd;
	cm_msg_hdr_t		* hdr;
	uint32				mid;
	ushort16			len;

	bd = &(edesc->bd[0]);
	if (!(edesc->hdr.control & EDESC_COMMAND_USE_FIRST_BUFFER))
		bd++;

	hdr = (cm_msg_hdr_t *)(bd->buf);
	len = bd->len;
	mid = hdr->mid;

	switch (mid) {
#ifdef SSC_MGMT
		case CHASSISD_REBOOT_REQ :
		{
			chassisdMsgRebootReq_t	* msg = (chassisdMsgRebootReq_t *)hdr;
			CM_DEBUG(("cm_rcvMsg, CHASSISD_REBOOT_REQ\n"));
			if (msg->reboot_type == CM_REBOOT_TYPE_SLOT) {
				printf("Rebooting slot %d\n", msg->slot);
				cmRebootCard(msg->slot);
			}
			else {
				printf("Rebooting slot %d, cpu %d\n", msg->slot, msg->slice);
				cmRebootCPU(msg->slot, msg->slice);
			}
			break;
		}

		case CM_SET_INFO_REQ :
			CM_DEBUG(("cm_rcvMsg, CM_SET_INFO_REQ\n"));
			hb.chassisdUp++; // to indicate chassisd is Up, so we can start sending to SSC
			hb.attempts=0;
			cmSendInfoResp(hdr->srcSlot, hdr->srcCpu);
			break;

		case CM_SEND_HEARTBEAT_ACK:
			CM_DEBUG(("cm_rcvMsg, CM_SEND_HEARTBEAT_ACK\n"));
			hb.chassisdUp = 1; // to indicate ack from SSC, so we can send another msg
			hb.attempts = 0;
            sscm_led_set(LED_FAULT_OFF);
            sscm_led_set(LED_OK_ON);
			break;
#endif

		case CM_SET_VERSION_REQ :
			CM_DEBUG(("cm_rcvMsg, CM_SET_VERSION_REQ\n"));
			cmSendVersionResp(hdr->srcSlot, hdr->srcCpu);
			break;

#ifdef NFP_TXRX
		case CM_SET_INFO_RESP :
		{
			cmCpuStateMsg_t	* msg = (cmCpuStateMsg_t *)hdr;
			CM_DEBUG(("cm_rcvMsg, CM_SET_INFO_RESP\n"));
			cm_slot_status &= ~( BMFPGA_SLOT_STATUS_SLICE_STATE_MASK <<
															(msg->cpu * 4));
			cm_slot_status |= (msg->state << (msg->cpu*4));
			bmfpgaSetCPUState(cm_slot_status);
			break;
		}
#endif

#if defined (NFP_TXRX) || defined (PROM_SIBYTE)
		case CM_SEND_HEARTBEAT_INFO:
		{
			CM_DEBUG(("cm_rcvMsg, CM_SEND_HEARTBEAT_INFO\n"));
#ifdef NFP_TXRX
			cmProcHeartbeatInfo();
#endif
			break;
		}
#endif

		case CHASSISD_GET_TEMP_REQ :
			CM_DEBUG(("cm_rcvMsg, CHASSISD_GET_TEMP_REQ\n"));
			cmGetCardTemp();
			break;

#if 0
#if defined(NFP_TXRX) || defined(FCNIM)
		case CM_GET_CHASSIS_INFO_REQ :
			{
			uint32	card_type = cmGetMyCardType();
			CM_DEBUG(("cm_rcvMsg, CHASSISD_GET_INFO_REQ\n"));
			cmSetCardType(card_type);
			cmSetCardPower();
			cmSetCardTemp();
			bmfpgaSetCPUState(cm_slot_status);
			}
			break;

		case CM_GET_POWER_REQ :
			CM_DEBUG(("cm_rcvMsg, CHASSISD_GET_POWER_REQ\n"));
			cmSetCardPower();
			break;

#endif
#endif

		default:
			printf("cm_rcvMsg, Invalid msg id, 0x%x\n", mid);
	}
	eee_freePacket(edesc);
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	4/18/02
 *
 *-----------------------------------------------------------------
 */
void
cmSendVersionResp(
	int32	slot,
	int32	cpu
)
{
	cmMsgVersionResp_t	msg;
	uchar8				* getPromImageVerString();
#ifdef NFP_TXRX
	extern void luc_getVerString(char *);
#endif
#ifdef NFP_FP
	extern void lmux_getVerString(char *);
#endif
	extern void seepGetHwRevision(char *);
	extern uchar8		bmfpga_str[];

	bzero(&msg, sizeof(cmMsgVersionResp_t));

	msg.hdr.mid = CM_SET_VERSION_RESP;
	msg.hdr.srcSlot = nfxMySlotId;
	msg.hdr.srcCpu = nfxMyCPUId;
	msg.hdr.srcApp = cm_appid;
	msg.hdr.srcPlane = EEE_MGMT;

	msg.which_ver = CM_VER_STRING_PROM;
	strcpy(msg.ver, getPromImageVerString());
#ifdef PROM_SIBYTE
	if (isTXRX())
#endif
	eee_sendMessage(&msg, sizeof(cmMsgVersionResp_t),
												cm_ssc_port, cm_src_port);

#ifndef PROM
    {
	extern uchar8		ImageVerString[];

	msg.which_ver = CM_VER_STRING_RT;
	strcpy(msg.ver, ImageVerString);

	eee_sendMessage(&msg, sizeof(cmMsgVersionResp_t),
												cm_ssc_port, cm_src_port);
	}
#endif

#ifdef SSC_MGMT
	/* 
	 * Send SSC prom version string to chassisd.   
	 */  
	msg.which_ver = CM_VER_STRING_PROM;
	strcpy(msg.ver, copySSCPromImageVerString());
	msg.hdr.srcSlot = nfxMySlotId;
	msg.hdr.srcCpu = NFX_SSC_UNIX;
	eee_sendMessage(&msg, sizeof(cmMsgVersionResp_t),
												cm_ssc_port, cm_src_port);
#endif
 	if (strlen(bmfpga_str)) {
		msg.which_ver = CM_VER_STRING_BMFPGA;
		strcpy(msg.ver, bmfpga_str);
		eee_sendMessage(&msg, sizeof(cmMsgVersionResp_t),
												cm_ssc_port, cm_src_port);
	}
#ifdef NFP_TXRX 
	msg.which_ver = CM_VER_STRING_LUC;
	luc_getVerString(msg.ver);
	eee_sendMessage(&msg, sizeof(cmMsgVersionResp_t),
												cm_ssc_port, cm_src_port);
#endif
#ifdef NFP_FP
	msg.which_ver = CM_VER_STRING_LMUX;
	lmux_getVerString(msg.ver);
	eee_sendMessage(&msg, sizeof(cmMsgVersionResp_t),
												cm_ssc_port, cm_src_port);
#endif
	msg.which_ver = CM_VER_STRING_HW;
	seepGetHwRevision(msg.ver);
	eee_sendMessage(&msg, sizeof(cmMsgVersionResp_t),
												cm_ssc_port, cm_src_port);
}

#ifdef SSC_MGMT
/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	4/25/02
 *
 *-----------------------------------------------------------------
 */
void
cmSendInfoResp(
	int32	slot,
	int32	cpu
)
{
	cmMsgInfoResp_t	msg;
/* 
 * This is a temp workaround for Garfield/Cheetah systems with 5632 switches. 
 * Cheetah systems require some static entries in the ARL table, but Garfield 
 * systems don't. The switch on the SSC has to dynamically reflect that.
 */
#ifdef SSC_MGMT
    extern void bcm_update_5632();

    bcm_update_5632();
#endif

	bzero(&msg, sizeof(cmMsgVersionResp_t));

	cmGetCardTemp();

	msg.hdr.mid = CM_SET_INFO_RESP;
	msg.hdr.srcSlot = nfxMySlotId;
	msg.hdr.srcCpu = nfxMyCPUId;
	msg.hdr.srcApp = cm_appid;
	msg.hdr.srcPlane = EEE_MGMT;

	msg.chassis_type = 0;				/* TODO */
	for (slot=0; slot<CM_NUM_SLOTS; slot++) {
		msg.present[slot] = cm.slot[slot].present;
		msg.board_type[slot] = cm.slot[slot].type;
		msg.temp_valid[slot] = cm.slot[slot].temp_valid;
		msg.temp[slot] = cm.slot[slot].temp;
		msg.power_valid[slot] = cm.slot[slot].power_valid;
		msg.power[slot] = cm.slot[slot].power;
		msg.num_cpus[slot] = cm.slot[slot].num_cpus;
		for (cpu=0; cpu<msg.num_cpus[slot]; cpu++)
			msg.cpu_state[slot][cpu] = cm.slot[slot].cpu[cpu].state;
	}

	eee_sendMessage(&msg, sizeof(cmMsgInfoResp_t), cm_ssc_port, cm_src_port);
}
#endif

#ifdef SSC_MGMT
/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	4/18/02
 *
 *-----------------------------------------------------------------
 */
void
cmSendInfoReq(void)
{
	cmMsgInfoReq_t	msg;
	int32			slot, cpu;

	msg.hdr.mid = CHASSISD_GET_CHASSIS_INFO_REQ;
	msg.hdr.srcSlot = nfxMySlotId;
	msg.hdr.srcCpu = nfxMyCPUId;
	msg.hdr.srcApp = cm_appid;
	msg.hdr.srcPlane = EEE_MGMT;

	for (slot = 0; slot < CM_NUM_SLOTS; slot++)
		for (cpu = 0; cpu < CM_NUM_CPUS; cpu++)
			bmfpga_sendMessage(slot, cpu, (uint32 *)&msg,
											sizeof(chassisdMsgInfoReq_t));
}

#ifndef PROM
/*------------------------------------------------------------------------------
 * Name         : cmSendCpuEvent
 *
 * Description  : Send CPU event
 *
 * Created by   : Pankaj Gupta
 *
 * Date Created : 29-MAY-2002
 *
 *------------------------------------------------------------------------------
 */

void
cmSendCpuEvent(uint32 slot, uint32 cpu, int	state)
{
	uint32				eventd_ssc_port;
    nfx_eventMsg_t      evtMsg;
    
    /* 
	 * Prepare event message
     */
    bzero((void *) &evtMsg, sizeof(nfx_eventMsg_t));
    evtMsg.msgType = NFX_EVENT_POST;
    evtMsg.needAck = FALSE;
    evtMsg.eventScope = NFX_EVENT_LOCAL;
    evtMsg.eventType = NFX_EVENT_CPU;
    sprintf(evtMsg.reason, "Slot %d CPU %d State: %s",
			slot, cpu, (state == EVENT_STATE_UP) ? "UP":"DOWN");
    evtMsg.param.cpuEvent.slot = slot;
    evtMsg.param.cpuEvent.cpu = cpu;
    evtMsg.param.cpuEvent.state = state;
   
   	/*
	 * EEE address of eventd on SSC Unix
	 */
	eventd_ssc_port = PACK_UNICAST_PORT_ADDRESS(0, NFX_SSC_UNIX,
								eee_getApplicationId("eventd"),
								EEE_MGMT, NFX_COS_HIGH, PORT_BEST_FIT);

    /* 
	 * Post event.
     */
	eee_sendMessage(&evtMsg, sizeof(nfx_eventMsg_t), 
							eventd_ssc_port, cm_src_port);

	printf("cmSendCpuEvent: Slot %d CPU %d State %s\n",
	  		slot, cpu, (state == EVENT_STATE_UP) ? "UP":"DOWN");

}
#endif  /* !PROM */
#endif

uint32	cm_update_cntr = 0;

#if defined(FCNIM) || defined(PROM_R7000)
/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	4/18/02
 *
 *-----------------------------------------------------------------
 */
void
cmGpioCardStatePoll(
	void	* cb,
	uint32	tref
)
{
	uint32	gpio_reg;
	uint32	fc_slice1_state, fc_slice2_state, fc_slice3_state;
	uint32	new_fc_slot_status;

	gpio_reg = *(volatile uint32 *)PHYS_TO_K1(HW_GALILEO_BASE+GPP_VALUE);
	fc_slice1_state = (gpio_reg & CM_FC1_GPIO_READ_STATE_MASK) >>
												CM_FC1_GPIO_READ_STATE_SHIFT;
	fc_slice2_state = (gpio_reg & CM_FC2_GPIO_READ_STATE_MASK) >>
												CM_FC2_GPIO_READ_STATE_SHIFT;
	fc_slice3_state = (gpio_reg & CM_FC3_GPIO_READ_STATE_MASK) >>
												CM_FC3_GPIO_READ_STATE_SHIFT;

    // clear all state fields from current copy of slot register value
	new_fc_slot_status = cm_slot_status & ~BMFPGA_SLOT_STATUS_SLICE0123_STATE_MASK;
	new_fc_slot_status |= (	(fc_slice3_state << 12) |
						(fc_slice2_state << 8) |
						(fc_slice1_state << 4) |
						cm_fc_slice0_state);

	if ((cm_slot_status != new_fc_slot_status) || (++cm_update_cntr >= 0x100000)) {

        //printf("cm_slot_status=%08x new_fc_slot_status=%08x  cm_update_cntr=0x%x\n", cm_slot_status&0xffff, new_fc_slot_status&0xffff, cm_update_cntr);

        /* we are FCNIM and FCSLICE==0 so we can do this */
        if (fc_slice3_state == CM_SLICE_STATE_RUNTIME &&
            fc_slice2_state == CM_SLICE_STATE_RUNTIME &&
            fc_slice1_state == CM_SLICE_STATE_RUNTIME &&
            cm_fc_slice0_state == CM_SLICE_STATE_RUNTIME) 
        {
            led_set(LED_FAULT_OFF);
            led_set(LED_OK_ON);
        }
        else {
            led_set(LED_FAULT_ON);
            led_set(LED_OK_OFF);
        }
		cm_slot_status = new_fc_slot_status;
		bmfpgaSetCPUState(cm_slot_status);
		cm_update_cntr = 0;
	}
}

#endif	/* FCNIM || PROM_R7000 */

#if defined(NFP_TXRX) || defined(PROM_SIBYTE)
/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	6/24/02
 *
 *-----------------------------------------------------------------
 */
void
cmGpioCardStatePoll(
	void	* cb,
	uint32	tref
)
{
	if (++cm_update_cntr >= 0x100000) {

        //printf("cm_slot_status=%08x cm_update_cntr=0x%x\n", cm_slot_status, cm_update_cntr);

        if ((cm_slot_status & 0xf) == CM_SLICE_STATE_RUNTIME &&
            (cm_slot_status>>4 & 0xf) == CM_SLICE_STATE_RUNTIME &&
            (cm_slot_status>>8 & 0xf) == CM_SLICE_STATE_RUNTIME &&
            (cm_slot_status>>12 & 0xf) == CM_SLICE_STATE_RUNTIME) 
        {
            led_set(LED_FAULT_OFF);
            led_set(LED_OK_ON);
        }
        else {
            led_set(LED_FAULT_ON);
            led_set(LED_OK_OFF);
        }
		bmfpgaSetCPUState(cm_slot_status);
		cm_update_cntr = 0;
	}
}

#endif	/* TXRX || PROM_SIBYTE */

#ifdef SSC_MGMT
/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	7/1/02
 *
 *-----------------------------------------------------------------
 */
int32
cm_checkFanStatus(
	uint32	tmr_handle,
	void * p1,
	void * p2,
	void * p3,
	void * p4
)
{
	uint32	gpp;

	gpp = *(volatile uint32 *)(HW_BMFPGA_BASE + 0x10);

	*(volatile uint32 *)(HW_GALILEO_BASE+GPP_VALUE) &= ~0x03000000;

	if (gpp & 0x00004003)
		*(volatile uint32 *)(HW_GALILEO_BASE+GPP_VALUE) |= 0x01000000;
	else
		*(volatile uint32 *)(HW_GALILEO_BASE+GPP_VALUE) |= 0x02000000;
	return NFX_OK;
}

/*-----------------------------------------------------------------
 * Name: cm_heartbeat_SendInfo()
 *
 * Description: every timeout, send msg to SSC with cpu status
 *
 * Created by:	lipika
 *
 * Date Created: 6/12/03
 *
 *-----------------------------------------------------------------
 */
void
cm_heartbeatSendInfo()
{
	cmHeartbeatMsgInfo_t	msg;
	int32			slot, cpu;

//printf("Enter cm_heartbeatSendInfo: \n");
	bzero(&msg, sizeof(cmHeartbeatMsgInfo_t));
	msg.hdr.mid = CM_SEND_HEARTBEAT_INFO;
	msg.hdr.srcSlot = nfxMySlotId;
	msg.hdr.srcCpu = nfxMyCPUId;
	msg.hdr.srcApp = cm_appid;
	msg.hdr.srcPlane = EEE_MGMT;

	msg.chassis_type = 0;				/* TODO */
	for (slot=0; slot<CM_NUM_SLOTS; slot++) {
		msg.present[slot] = cm.slot[slot].present;
		msg.board_type[slot] = cm.slot[slot].type;
		msg.num_cpus[slot] = cm.slot[slot].num_cpus;
		for (cpu=0; cpu<msg.num_cpus[slot]; cpu++)
			msg.cpu_state[slot][cpu] = cm.slot[slot].cpu[cpu].state;
	}

	eee_sendMessage(&msg, sizeof(cmMsgInfoResp_t), cm_ssc_port, cm_src_port);
    if (hb.attempts++ > HB_FAIL_THRESHOLD) {
        sscm_led_set(LED_FAULT_ON);
        sscm_led_set(LED_OK_OFF);
    }
}	
#endif

#ifdef NFP_TXRX
/*-----------------------------------------------------------------
 * Name: cmPromHeartbeatInfo()
 *
 * Description: Once you rcvd info from SSC, stop the local timer
 *
 * Created by:	lipika
 *
 * Date Created: 6/12/03
 *
 *-----------------------------------------------------------------
 */
void
cmProcHeartbeatInfo()
{
	int32 rct=0;
//printf("cmProcHeartbeatInfo\n");
	/* Stop the local timer */
	rct = eee_timerStop(heartbeat_timer);
	if(rct != NFX_OK)
		printf("Could not Stop heartbeat timer %x\n",rct);

	/* Need to process the info ?? */

	/* restart the timer once again */
	cm_Heartbeat(heartbeat_timer);
}

#endif

#ifdef SSC_MGMT

#define PROM_CS3_BASEADDR	    0X3b000000
#define PROM_BOOTCS_BASEADDR	0X3b800000

/*-----------------------------------------------------------------
 * Name: copySSCPromImageVerString()
 *
 * Description: copy the ssc prom image ver string from flash rom 
 *              to specified location. 
 *
 * Created by:	lipika
 *
 * Date Created: 8/12/03
 *
 *-----------------------------------------------------------------
 */
uchar8 *
copySSCPromImageVerString(void)
{
    volatile uint32 bmfpga_boardStatus = *(volatile uint32 *)(HW_BMFPGA_BASE + BMFPGA_REG_BOARD_STATUS_OFF);
	uchar8 *verPtr = (uchar8 *)&SSCImageVerString;
	volatile uchar8	* sp;
	int32			i;

	/* The SSC prom ver string is set only for Beta boards
	 * in the std flash ROM. Only BETA boards have the std
	 * flash ROM.
	 */  
	if (((bmfpga_boardStatus & BMFPGA_BOARD_STATUS_CLK_MODE_MASK) >>
					BMFPGA_BOARD_STATUS_CLK_MODE_SHIFT) ==
						BMFPGA_BOARD_STATUS_CLK_MODE_125MHz)
	{
			/* Beta Board */
		if (((bmfpga_boardStatus & BMFPGA_BOARD_STATUS_PROM_SEL_MASK) >>
					BMFPGA_BOARD_STATUS_PROM_SEL_SHIFT) == 1)
		{
				/* Booting from large prom, read from Large Prom */
				/* R7000 crashdump_offset + crashdump_max_size = 0x1fff80 */
				sp = (volatile uchar8 *)(PROM_BOOTCS_BASEADDR+0x1fff80);
		}		
		else
		{
				/* Booting from small prom, read from Large Prom */
				sp = (volatile uchar8 *)(PROM_CS3_BASEADDR+0x1fff80);
		}		

		/* copy bytes to fixed location */
		for (i=0; i<CM_VER_STRING_LEN; i++) {
			SSCImageVerString[i] = sp[i];
		}	
	}		
	//printf("%s\n",verPtr);
	return(verPtr);
}

#endif

#endif	/* !SSC */
