/*
 *-----------------------------------------------------------------
 *
 * Name:	cm.h
 *
 * RCS:  $Id: cm.h,v 1.9 2002/11/19 04:43:32 tonyw Exp $
 *
 * Copyright (C) 2002, Agile Storage, Inc.
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	4/15/02
 *
 *-----------------------------------------------------------------
 */

#ifndef _CM_H
#define _CM_H

#define CM_DEBUG(x)		if (cm_debug) printf x

#define CM_TEMP_CRITICAL	100
#define CM_TEMP_HIGH		85
#define CM_TEMP_NORMAL		50

/*
 * The cm_t structure is compiled and maintained by the chassisd daemon on
 * the SSC.
 *
 * Version info comes from direct MGMT plane messages from the other cards
 * in the system.  The rest of the info comes from communication with the
 * Toshiba CPU, which monitors BMFPGA info changes.
 */
typedef struct cm_cpu_struct {
	uchar8		prom_ver[CM_VER_STRING_LEN];
	uchar8		rt_ver[CM_VER_STRING_LEN];
	uint32		state;
} cm_cpu_t;

typedef struct cm_slot_struct {
	uint32		present;
	uint32		type;
	uint32		power_valid;
	uint32		power;
	uint32		temp_valid;
	uint32		temp;
	uint32		state;
	uchar8		bmfpga_ver[CM_VER_STRING_LEN];
	uchar8		luc_ver[CM_VER_STRING_LEN];
	uchar8		lmux_ver[CM_VER_STRING_LEN];
	uchar8		hw_ver[CM_VER_STRING_LEN];
	uint32		num_cpus;
	cm_cpu_t	cpu[CM_NUM_CPUS];
	uint32		control;			/* shadow copy of write-only reg */
} cm_slot_t;

typedef struct cm_struct {
	cm_slot_t	slot[CM_NUM_SLOTS];
} cm_t;

extern cm_t		cm;
extern uint32	cm_debug;

#define CM_FC1_GPIO_READ_STATE_MASK		0x00007800
#define CM_FC2_GPIO_READ_STATE_MASK		0x00078000
#define CM_FC3_GPIO_READ_STATE_MASK		0x00780000
#define CM_FC1_GPIO_READ_STATE_SHIFT		11
#define CM_FC2_GPIO_READ_STATE_SHIFT		15
#define CM_FC3_GPIO_READ_STATE_SHIFT		19
#define CM_FC_GPIO_WRITE_STATE_MASK		0x00078000
#define CM_FC_GPIO_WRITE_STATE_SHIFT		15

extern uint32	cm_ssc_port;
extern uint32	cm_txrx_port;
extern uint32	cm_fp_port;
extern uint32	cm_fc0_port;
extern uint32	cm_fc1_port;
extern uint32	cm_fc2_port;
extern uint32	cm_fc3_port;
extern uint32	cm_src_port;

extern int32	cm_appid;

extern uint32	cm_slot_status;

typedef struct {
	void	* msg;
	uint32	len;
	uint32	srcCpu;
	uint32	srcSlot;
	uchar8	srcApp[32];
} chassisd_msgDesc_t;

typedef struct {
    chassisd_msg_hdr_t  hdr;
	cm_t				cm;
} chassisdMsgDebugResp_t;

#endif	/* _CM_H */

