/* vi:ts=4
 *-----------------------------------------------------------------
 *
 * Copyright (C) 2001, Agile Storage Inc.
 *
 * RCS:  $Id: cifs-rpc-api.h,v 1.1.1.1 2002/04/23 17:47:36 laurentb Exp $
 * 
 * Name			: cifs-rpc-api.h
 *
 * Description	: CIFS DCE/RPC module api header
 * 
 * Created by	: Laurent Brard
 *
 * Date Created	: 12/09/2001
 *
 *-----------------------------------------------------------------
 */
#ifndef _CIFS_RPC_API_H
#define _CIFS_RPC_API_H

#ifdef FS_STANDALONE

#include "../sm-cifs-rpc/cifs-rpc-mem.h"
#include "../sm-cifs-rpc/cifs-rpc-encdec.h"
#include "../sm-cifs-rpc/cifs-rpc.h"
#include "../sm-cifs/cifs.h"
#include "../sm-cifs/cifs-string.h"

#else   //FS_STANDALONE

#include <exports/cifs-rpc-mem.h>
#include <exports/cifs-rpc-encdec.h>
#include <exports/cifs-rpc.h>
#include <exports/cifs.h>
#include <exports/cifs-string.h>

#endif  //FS_STANDALONE

// ************************ API functions *********************************


			
/*-----------------------------------------------------------------
 * Name         : cifsrpc_pipeIsSupported
 *
 * Description  : check whether a named pipe is supported
 *
 * Created by   : Laurent Brard
 *
 * Date Created : 12/12/2001
 *
 *-----------------------------------------------------------------
 */
int32 //index in pipe's list, -1 if not supported
cifsrpc_pipeIsSupported(uni16 *pipeName);
		

/*-----------------------------------------------------------------
 * Name			: cifsrpc_initPipe
 *
 * Description	: initialize an RPC pipe
 *
 * Created by	: Laurent Brard
 *
 * Date Created	: 12/11/2001
 *
 *-----------------------------------------------------------------
 */
int32 //NFX_OK or NFX_ERR
cifsrpc_initPipe(	cifsrpc_pipe *rpip, 
					uchar8		 pipeIndex,
					cifs_client  *clp,
                    cifs_session *ssp);

/*-----------------------------------------------------------------
 * Name         : cifsrpc_closePipe
 *
 * Description  : close an RPC pipe
 *
 * Created by   : Laurent Brard
 *
 * Date Created : 01/29/2002
 *
 *-----------------------------------------------------------------
 */
int32 //NFX_OK or NFX_ERR
cifsrpc_closePipe(cifsrpc_pipe *rpip);
		
/*-----------------------------------------------------------------
 * Name			: cifsrpc_writeData
 *
 * Description	: write on a RPC named pipe
 *
 * Created by	: Laurent Brard
 *
 * Date Created	: 12/09/2001
 *
 *-----------------------------------------------------------------
 */
int32	//Number of bytes written
cifsrpc_writeData(	cifsrpc_pipe	*rpip,
					uchar8	 		*datp,
					uint32			datCnt);

	
/*-----------------------------------------------------------------
 * Name			: cifsrpc_readData
 *
 * Description	: read from an RPC named pipe
 *
 * Created by	: Laurent Brard
 *
 * Date Created	: 12/09/2001
 *
 *-----------------------------------------------------------------
 */
int32	//Number of bytes read
cifsrpc_readData(	cifsrpc_pipe	*rpip,
					uchar8			**datpp,
					uint32			datCnt, 
					boolean			*moreData);




#endif	//_CIFS_RPC_API_H


