/* vi:ts=4
 *-----------------------------------------------------------------
 *
 * Copyright (C) 2002, Agile Storage Inc.
 * 
 * RCS:  $Id: cifs-rpc-ed-lsarpc.c,v 1.1.1.1 2002/04/23 17:47:36 laurentb Exp $
 *
 * Name			: cifs-rpc-ed-lsarpc.c
 *
 * Description	: CIFS DCE/RPC LSA services encode & decode functions. 
 *
 * Created by	: Laurent Brard
 *
 * Date Created	: 04/19/2002
 *
 *-----------------------------------------------------------------
 */

#ifdef FS_STANDALONE
#include "../sm-cifs/cifs-api.h"
#else
#include <exports/cifs-api.h>
#endif

#include "cifs-rpc.x"
#include "cifs-rpc-lsarpc.h"
#include "cifs-rpc-ed-lsarpc.h"
#include "cifs-rpc-ed-sec.h"


/*-----------------------------------------------------------------
 * Name         : cifsrpcEDlsaObjAttrs()
 *
 * Description  : LSA object attributes encode / decode 
 *
 * Created by   : Laurent Brard
 *
 * Date Created : 04/19/2002
 *
 *-----------------------------------------------------------------
 */
int32	//NFX_OK or NFX_ERR
cifsrpcEDlsaObjAttrs(cifsrpc_ed *edp,
					lsaObjAttrs *p, 
					uchar8		lv)
{
	lsaSecQos *p2 = &p->secQos;
		
	CIFS_PRINT(CIFS_DBGLV_DBG3, ("%slsaObjAttrs", cifs_spaces(lv)));
	lv++;
	if (cifsrpcEDalign   (edp)										!= NFX_OK
	 || cifsrpcEDuint32  (edp, &p->length,		"length    ",	lv)	!= NFX_OK
	 || cifsrpcEDuint32  (edp, &p->rootDirp,	"rootDirp  ",	lv)	!= NFX_OK
	 || cifsrpcEDuint32  (edp, &p->objNamep,	"objNamep  ",	lv)	!= NFX_OK
	 || cifsrpcEDuint32  (edp, &p->attributes,	"attributes",	lv)	!= NFX_OK
	 || cifsrpcEDuint32  (edp, &p->secDescp,	"secDescp  ",	lv)	!= NFX_OK
	 || cifsrpcEDuint32  (edp, &p->secQosp,		"secQosp   ",	lv)	!= NFX_OK)
		return (NFX_ERR);

	if (p->secQosp == 0)
		return (NFX_OK);

	CIFS_PRINT(CIFS_DBGLV_DBG3, ("%slsaSecQos", cifs_spaces(lv)));
	lv++;
	if (cifsrpcEDalign   (edp)										!= NFX_OK
	 || cifsrpcEDuint32  (edp, &p2->length,		"length      ",	lv)	!= NFX_OK
	 || cifsrpcEDushort16(edp, &p2->impersoLvl,	"impersoLvl  ",	lv) != NFX_OK
	 || cifsrpcEDuchar8  (edp, &p2->ctxTrackMod,"ctxTrackMode",	lv)	!= NFX_OK
	 || cifsrpcEDuchar8  (edp, &p2->effectiOnly,"effectivOnly",	lv)	!= NFX_OK)
		return (NFX_ERR);

	return (NFX_OK);
}


/*-----------------------------------------------------------------
 * Name         : cifsrpcEDlsaPolHnd()
 *
 * Description  : LSA policy handle encode / decode 
 *
 * Created by   : Laurent Brard
 *
 * Date Created : 04/19/2002
 *
 *-----------------------------------------------------------------
 */
int32	//NFX_OK or NFX_ERR
cifsrpcEDlsaPolHnd(	cifsrpc_ed *edp,
					cifsrpc_lsaPolHnd *polp, 
					uchar8 lv)
{
	CIFS_PRINT(CIFS_DBGLV_DBG3, ("%spolicyHandle", cifs_spaces(lv)));
	lv++;
	if (cifsrpcEDalign   (edp)							!= NFX_OK
	 || cifsrpcEDuint32  (edp, &polp->zero,			"zero   ",	lv)	!= NFX_OK
	 || cifsrpcEDuint32  (edp, &polp->uuid.d1,		"uuid d1",	lv)	!= NFX_OK
	 || cifsrpcEDushort16(edp, &polp->uuid.d2,		"uuid d2",	lv) != NFX_OK
	 || cifsrpcEDushort16(edp, &polp->uuid.d3,		"uuid d3",	lv) != NFX_OK
	 || cifsrpcEDuchar8Ar(edp, polp->uuid.d4, 2, 0,	"uuid d4",	lv)	!= NFX_OK
	 || cifsrpcEDuchar8Ar(edp, polp->uuid.d5, 6, 0,	"uuid d5",	lv)	!= NFX_OK)
		return (NFX_ERR);

	return (NFX_OK);
}









/*-----------------------------------------------------------------
 * Name         : cifsrpcEDlsaOpenPolicyCall()
 *
 * Description  : LSARPC lsaOpenPolicy function call encode / decode 
 *
 * Created by   : Laurent Brard
 *
 * Date Created : 04/19/2002
 *
 *-----------------------------------------------------------------
 */
int32	//NFX_OK or NFX_ERR
cifsrpcEDlsaOpenPolicyCall(	cifsrpc_ed *edp,
							lsaOpenPolicyCall *p, 
							uchar8 lv)
{
	CIFS_PRINT(CIFS_DBGLV_DBG3, ("%slsaOpenPolicyCall", cifs_spaces(lv)));
	lv++;
	if (cifsrpcEDuint32  (edp, &p->unknown,		"unknown    ",	lv)	!= NFX_OK
	 || cifsrpcEDushort16(edp, &p->systemName,	"systemName ",	lv) != NFX_OK
	 || cifsrpcEDlsaObjAttrs(edp, &p->attrs,					lv)	!= NFX_OK
	 || cifsrpcEDuint32(edp, &p->desiredAccess,	"desirAccess", lv)	!= NFX_OK)
		return (NFX_ERR);

	return (NFX_OK);
}


/*-----------------------------------------------------------------
 * Name         : cifsrpcEDlsaOpenPolicyResp()
 *
 * Description  : LSARPC lsaOpenPolicy response encode / decode 
 *
 * Created by   : Laurent Brard
 *
 * Date Created : 04/19/2002
 *
 *-----------------------------------------------------------------
 */
int32	//NFX_OK or NFX_ERR
cifsrpcEDlsaOpenPolicyResp(	cifsrpc_ed *edp,
							lsaOpenPolicyResp *p, 
							uchar8 lv)
{
	CIFS_PRINT(CIFS_DBGLV_DBG3, ("%slsaOpenPolicyXResp", cifs_spaces(lv)));
	lv++;
	if (cifsrpcEDalign    (edp)										!= NFX_OK
	 || cifsrpcEDlsaPolHnd(edp, &p->pol,						lv) != NFX_OK
	 || cifsrpcEDuint32   (edp, &p->returnValue,"returnValue",	lv) != NFX_OK)
		return (NFX_ERR);

	return (NFX_OK);
}


/*-----------------------------------------------------------------
 * Name         : cifsrpcEDlsaOpenPolicy2Call()
 *
 * Description  : LSARPC lsaOpenPolicy2 function call encode / decode 
 *
 * Created by   : Laurent Brard
 *
 * Date Created : 04/19/2002
 *
 *-----------------------------------------------------------------
 */
int32	//NFX_OK or NFX_ERR
cifsrpcEDlsaOpenPolicy2Call(	cifsrpc_ed *edp,
								lsaOpenPolicy2Call *p, 
								uchar8 lv)
{
	CIFS_PRINT(CIFS_DBGLV_DBG3, ("%slsaOpenPolicy2Call", cifs_spaces(lv)));
	lv++;
	if (cifsrpcEDuint32(edp, &p->unknown,		"unknown    ",	lv)	!= NFX_OK
	 || cifsrpcEDuniStruc(edp, &p->serverName,	"srvName",		lv) != NFX_OK
	 || cifsrpcEDlsaObjAttrs(edp, &p->attrs,					lv)	!= NFX_OK
	 || cifsrpcEDuint32(edp, &p->desiredAccess,	"desirAccess", lv)	!= NFX_OK)
		return (NFX_ERR);

	return (NFX_OK);
}


/*-----------------------------------------------------------------
 * Name         : cifsrpcEDlsaQueryInfoPolicyCall()
 *
 * Description  : LSARPC QueryInfoPolicy function call encode / decode 
 *
 * Created by   : Laurent Brard
 *
 * Date Created : 04/19/2002
 *
 *-----------------------------------------------------------------
 */
int32	//NFX_OK or NFX_ERR
cifsrpcEDlsaQueryInfoPolicyCall(cifsrpc_ed *edp,
								lsaQueryInfoPolicyCall *p, 
								uchar8 lv)
{
	CIFS_PRINT(CIFS_DBGLV_DBG3, ("%slsaQueryInfoPolicyCall", cifs_spaces(lv)));
	lv++;
	if (cifsrpcEDlsaPolHnd(edp, &p->pol,						lv) != NFX_OK
	 || cifsrpcEDushort16 (edp, &p->infoClass,	"infoClass",	lv) != NFX_OK)
		return (NFX_ERR);

	return (NFX_OK);
}



/*-----------------------------------------------------------------
 * Name         : cifsrpcEDlsaDomInfo()
 *
 * Description  : LSARPC domain info encode / decode 
 *
 * Created by   : Laurent Brard
 *
 * Date Created : 04/20/2002
 *
 *-----------------------------------------------------------------
 */
static int32	//NFX_OK or NFX_ERR
cifsrpcEDlsaDomInfo(cifsrpc_ed *edp, domInfo *p, uchar8 lv)
{
	CIFS_PRINT(CIFS_DBGLV_DBG3, ("%slsaDomInfo", cifs_spaces(lv)));
	lv++;
	if (cifsrpcEDalign   (edp)										!= NFX_OK
	 || cifsrpcEDushort16(edp, &p->uniLen,		"uniLen   ",	lv) != NFX_OK
	 || cifsrpcEDushort16(edp, &p->uniMaxLen,	"uniMaxLen",	lv) != NFX_OK
	 || cifsrpcEDuint32  (edp, &p->domNamep,	"domNamep ",	lv)	!= NFX_OK
	 || cifsrpcEDuint32  (edp, &p->domSidp,		"domSidp  ",	lv)	!= NFX_OK
	 || cifsrpcEDuniStruc(edp, &p->domName,		"domName",		lv) != NFX_OK
	 || cifsrpcEDalign	 (edp)										!= NFX_OK
	 || (p->domSidp != 0
	  && cifsrpcEDrpcSid (edp, &p->domSid,		"domSid",		lv) != NFX_OK))
		return (NFX_ERR);
				
	return (NFX_OK);
}

#define cifsrpcEDlsaDomInfo3	cifsrpcEDlsaDomInfo
#define cifsrpcEDlsaDomInfo5	cifsrpcEDlsaDomInfo


/*-----------------------------------------------------------------
 * Name         : cifsrpcEDlsaDomInfo2()
 *
 * Description  : LSARPC domain info class 2 encode / decode 
 *
 * Created by   : Laurent Brard
 *
 * Date Created : 04/20/2002
 *
 *-----------------------------------------------------------------
 */
static int32	//NFX_OK or NFX_ERR
cifsrpcEDlsaDomInfo2(cifsrpc_ed *edp, domInfo2 *p, uchar8 lv)
{
	CIFS_PRINT(CIFS_DBGLV_DBG3, ("%slsaDomInfo2", cifs_spaces(lv)));
	lv++;
	if (cifsrpcEDalign   (edp)										!= NFX_OK
	 || cifsrpcEDuint32  (edp, &p->auditEnabled,"auditEnabled", lv)	!= NFX_OK
	 || cifsrpcEDuint32  (edp, &p->count1,		"count1      ",	lv)	!= NFX_OK
	 || cifsrpcEDuint32  (edp, &p->count2,		"count2      ",	lv)	!= NFX_OK)
		return (NFX_ERR);
				
	//TODO
					
	return (NFX_OK);
}
	

/*-----------------------------------------------------------------
 * Name         : cifsrpcEDlsaDomInfo6()
 *
 * Description  : LSARPC domain info class 6 encode / decode 
 *
 * Created by   : Laurent Brard
 *
 * Date Created : 04/20/2002
 *
 *-----------------------------------------------------------------
 */
static int32	//NFX_OK or NFX_ERR
cifsrpcEDlsaDomInfo6(cifsrpc_ed *edp, domInfo6 *p, uchar8 lv)
{
	CIFS_PRINT(CIFS_DBGLV_DBG3, ("%slsaDomInfo6", cifs_spaces(lv)));
	lv++;
	if (cifsrpcEDushort16(edp, &p->serverRole,	"serverRole",	lv)	!= NFX_OK)
		return (NFX_ERR);
					
	return (NFX_OK);
}
	

/*-----------------------------------------------------------------
 * Name         : cifsrpcEDlsaQueryInfoPolicyResp()
 *
 * Description  : LSARPC QueryInfoPolicy response encode / decode 
 *
 * Created by   : Laurent Brard
 *
 * Date Created : 04/19/2002
 *
 *-----------------------------------------------------------------
 */
int32	//NFX_OK or NFX_ERR
cifsrpcEDlsaQueryInfoPolicyResp(cifsrpc_ed *edp,
								lsaQueryInfoPolicyResp *p, 
								uchar8 lv)
{
	CIFS_PRINT(CIFS_DBGLV_DBG3, ("%slsaQueryInfoPolicyResp", cifs_spaces(lv)));
	lv++;
	if (cifsrpcEDuint32  (edp, &p->infop,		"infop      ", lv)	!= NFX_OK)
		return (NFX_ERR);

	if (p->infop != 0)
	{
		if (cifsrpcEDushort16(edp, &p->infoClass, "infoClass  ", lv)!= NFX_OK	
		 || cifsrpcEDalign(edp)										!= NFX_OK)
			return (NFX_ERR);
		
		switch (p->infoClass) {
			case 2:
				if (cifsrpcEDlsaDomInfo2(edp, &p->info.cl2, lv)	!= NFX_OK)
					return (NFX_ERR);
				break;
			case 3:
				if (cifsrpcEDlsaDomInfo3(edp, &p->info.cl3, lv)	!= NFX_OK)
					return (NFX_ERR);
				break;
			case 5:
				if (cifsrpcEDlsaDomInfo5(edp, &p->info.cl5, lv)	!= NFX_OK)
					return (NFX_ERR);
				break;
			case 6:
				if (cifsrpcEDlsaDomInfo6(edp, &p->info.cl6, lv)	!= NFX_OK)
					return (NFX_ERR);
				break;
			default:
				CIFS_PRINT(CIFS_DBGLV_ERR, 
							("lsaQueryInfoPolicyResp: wrong info level"));
		}
	}

	if (cifsrpcEDalign (edp)										!= NFX_OK
	 || cifsrpcEDuint32(edp, &p->returnValue,	"returnValue", lv)	!= NFX_OK)
		return (NFX_ERR);
	return (NFX_OK);
}


/*-----------------------------------------------------------------
 * Name         : cifsrpcEDlsaCloseCall()
 *
 * Description  : LSARPC lsaClose function call encode / decode 
 *
 * Created by   : Laurent Brard
 *
 * Date Created : 04/19/2002
 *
 *-----------------------------------------------------------------
 */
int32	//NFX_OK or NFX_ERR
cifsrpcEDlsaCloseCall(	cifsrpc_ed *edp,
						lsaCloseCall *closep, 
						uchar8 lv)
{
	CIFS_PRINT(CIFS_DBGLV_DBG3, ("%slsaCloseCall", cifs_spaces(lv)));
	lv++;
	if (cifsrpcEDlsaPolHnd(edp, &closep->pol, lv) != NFX_OK)
		return (NFX_ERR);

	return (NFX_OK);
}


/*-----------------------------------------------------------------
 * Name         : cifsrpcEDlsaCloseResp()
 *
 * Description  : LSARPC lsaClose response encode / decode 
 *
 * Created by   : Laurent Brard
 *
 * Date Created : 04/19/2002
 *
 *-----------------------------------------------------------------
 */
int32	//NFX_OK or NFX_ERR
cifsrpcEDlsaCloseResp(	cifsrpc_ed *edp,
						lsaCloseResp *p, 
						uchar8 lv)
{
	CIFS_PRINT(CIFS_DBGLV_DBG3, ("%slsaCloseResp", cifs_spaces(lv)));
	lv++;
	if (cifsrpcEDlsaPolHnd(edp, &p->pol,						lv) != NFX_OK
	 || cifsrpcEDuint32  (edp, &p->returnValue,	"returnValue ", lv) != NFX_OK)
		return (NFX_ERR);

	return (NFX_OK);
}



#ifdef NERVERDEFINED

/*-----------------------------------------------------------------
 * Name         : ()
 *
 * Description  : LSARPC  function call encode / decode 
 *
 * Created by   : Laurent Brard
 *
 * Date Created : 04/19/2002
 *
 *-----------------------------------------------------------------
 */
int32	//NFX_OK or NFX_ERR
(	cifsrpc_ed *edp,
								 *p, 
								uchar8 lv)
{
	CIFS_PRINT(CIFS_DBGLV_DBG3, ("%s", cifs_spaces(lv)));
	lv++;
	if (cifsrpcEDalign   (edp)										!= NFX_OK
	 || cifsrpcEDuint32  (edp, &p->serverNamep,	"serverNamep",	lv)	!= NFX_OK
	 || cifsrpcEDuniStruc(edp, &p->serverName,	"srvName",		lv) != NFX_OK
	 || cifsrpcEDalign   (edp)										!= NFX_OK
	 || cifsrpcEDuint32  (edp, &p->level,		"level      ", lv)	!= NFX_OK)
		return (NFX_ERR);

	return (NFX_OK);
}


/*-----------------------------------------------------------------
 * Name         : ()
 *
 * Description  : LSARPC  response encode / decode 
 *
 * Created by   : Laurent Brard
 *
 * Date Created : 04/19/2002
 *
 *-----------------------------------------------------------------
 */
int32	//NFX_OK or NFX_ERR
(	cifsrpc_ed *edp,
								 *p, 
								uchar8 lv)
{
	CIFS_PRINT(CIFS_DBGLV_DBG3, ("%s", cifs_spaces(lv)));
	lv++;
	if (cifsrpcEDalign   (edp)										!= NFX_OK
	 || cifsrpcEDuint32  (edp, &p->returnValue,	"returnValue ", lv) != NFX_OK)
		return (NFX_ERR);

	return (NFX_OK);
}

#endif
