/* vi:ts=4
 *-----------------------------------------------------------------
 *
 * Copyright (C) 2002, Agile Storage Inc.
 * 
 * RCS:  $Id: cifs-rpc-ed-srvsvc.h,v 1.1.1.1 2002/04/23 17:47:36 laurentb Exp $
 *
 * Name			: cifs-rpc-ed-srvsvc.c
 *
 * Description	: CIFS DCE/RPC server services encode & decode functions. 
 *
 * Created by	: Laurent Brard
 *
 * Date Created	: 01/14/2002
 *
 *-----------------------------------------------------------------
 */

#ifndef _CIFS_RPC_ED_SRVSVC_H
#define _CIFS_RPC_ED_SRVSVC_H


/*-----------------------------------------------------------------
 * Name         : cifsrpcEDnetShareEnumCall()
 *
 * Description  : SRVSVC NetShareEnum function call encode / decode 
 *
 * Created by   : Laurent Brard
 *
 * Date Created : 01/14/2002
 *
 *-----------------------------------------------------------------
 */
int32	//NFX_OK or NFX_ERR
cifsrpcEDnetShareEnumCall(cifsrpc_ed *edp, netShareEnumCall *p, uchar8 lv);


/*-----------------------------------------------------------------
 * Name         : cifsrpcEDnetShareEnumResp()
 *
 * Description  : SRVSVC NetShareEnum function response encode / decode 
 *
 * Created by   : Laurent Brard
 *
 * Date Created : 01/14/2002
 *
 *-----------------------------------------------------------------
 */
int32	//NFX_OK or NFX_ERR
cifsrpcEDnetShareEnumResp(cifsrpc_ed *edp, netShareEnumResp *p, uchar8 lv);


/*-----------------------------------------------------------------
 * Name         : cifsrpcEDnetShareGetInfoCall()
 *
 * Description  : SRVSVC NetShareGetInfo function call encode / decode 
 *
 * Created by   : Laurent Brard
 *
 * Date Created : 01/18/2002
 *
 *-----------------------------------------------------------------
 */
int32	//NFX_OK or NFX_ERR
cifsrpcEDnetShareGetInfoCall(	cifsrpc_ed *edp,
								netShareGetInfoCall *p, 
								uchar8 lv);


/*-----------------------------------------------------------------
 * Name         : cifsrpcEDnetShareGetInfoResp()
 *
 * Description  : SRVSVC NetShareGetInfo function response encode / decode 
 *
 * Created by   : Laurent Brard
 *
 * Date Created : 01/18/2002
 *
 *-----------------------------------------------------------------
 */
int32	//NFX_OK or NFX_ERR
cifsrpcEDnetShareGetInfoResp(	cifsrpc_ed *edp,
								netShareGetInfoResp *p, 
								uchar8 lv);


/*-----------------------------------------------------------------
 * Name         : cifsrpcEDnetShareSetInfoCall()
 *
 * Description  : SRVSVC NetShareSetInfo function call encode / decode 
 *
 * Created by   : Laurent Brard
 *
 * Date Created : 01/18/2002
 *
 *-----------------------------------------------------------------
 */
int32	//NFX_OK or NFX_ERR
cifsrpcEDnetShareSetInfoCall(	cifsrpc_ed *edp,
								netShareSetInfoCall *p, 
								uchar8 lv);


/*-----------------------------------------------------------------
 * Name         : cifsrpcEDnetShareSetInfoResp()
 *
 * Description  : SRVSVC NetShareSetInfo function response encode / decode 
 *
 * Created by   : Laurent Brard
 *
 * Date Created : 01/18/2002
 *
 *-----------------------------------------------------------------
 */
int32	//NFX_OK or NFX_ERR
cifsrpcEDnetShareSetInfoResp(	cifsrpc_ed *edp,
								netShareSetInfoResp *p, 
								uchar8 lv);


/*-----------------------------------------------------------------
 * Name         : cifsrpcEDnetShareAddCall()
 *
 * Description  : SRVSVC NetShareAdd function call encode / decode 
 *
 * Created by   : Laurent Brard
 *
 * Date Created : 01/18/2002
 *
 *-----------------------------------------------------------------
 */
int32	//NFX_OK or NFX_ERR
cifsrpcEDnetShareAddCall(	cifsrpc_ed *edp,
								netShareAddCall *p, 
								uchar8 lv);


/*-----------------------------------------------------------------
 * Name         : cifsrpcEDnetShareAddResp()
 *
 * Description  : SRVSVC NetShareAdd function response encode / decode 
 *
 * Created by   : Laurent Brard
 *
 * Date Created : 01/18/2002
 *
 *-----------------------------------------------------------------
 */
int32	//NFX_OK or NFX_ERR
cifsrpcEDnetShareAddResp(	cifsrpc_ed *edp,
							netShareAddResp *p, 
							uchar8 lv);


/*-----------------------------------------------------------------
 * Name         : cifsrpcEDnetShareDelCall()
 *
 * Description  : SRVSVC NetShareDel function call encode / decode 
 *
 * Created by   : Laurent Brard
 *
 * Date Created : 01/18/2002
 *
 *-----------------------------------------------------------------
 */
int32	//NFX_OK or NFX_ERR
cifsrpcEDnetShareDelCall(	cifsrpc_ed *edp,
								netShareDelCall *p, 
								uchar8 lv);


/*-----------------------------------------------------------------
 * Name         : cifsrpcEDnetShareDelResp()
 *
 * Description  : SRVSVC NetShareDel function response encode / decode 
 *
 * Created by   : Laurent Brard
 *
 * Date Created : 01/18/2002
 *
 *-----------------------------------------------------------------
 */
int32	//NFX_OK or NFX_ERR
cifsrpcEDnetShareDelResp(	cifsrpc_ed *edp,
								netShareDelResp *p, 
								uchar8 lv);



/*-----------------------------------------------------------------
 * Name         : cifsrpcEDnetServerGetInfoCall()
 *
 * Description  : SRVSVC NetServerGetInfo function call encode / decode 
 *
 * Created by   : Laurent Brard
 *
 * Date Created : 01/17/2002
 *
 *-----------------------------------------------------------------
 */
int32	//NFX_OK or NFX_ERR
cifsrpcEDnetServerGetInfoCall(	cifsrpc_ed *edp,
								netServerGetInfoCall *p, 
								uchar8 lv);


/*-----------------------------------------------------------------
 * Name         : cifsrpcEDnetServerGetInfoResp()
 *
 * Description  : SRVSVC NetServerGetInfo function response encode / decode 
 *
 * Created by   : Laurent Brard
 *
 * Date Created : 01/17/2002
 *
 *-----------------------------------------------------------------
 */
int32	//NFX_OK or NFX_ERR
cifsrpcEDnetServerGetInfoResp(	cifsrpc_ed *edp,
								netServerGetInfoResp *p, 
								uchar8 lv);


/*-----------------------------------------------------------------
 * Name         : cifsrpcEDnetServerSetInfoCall()
 *
 * Description  : SRVSVC NetServerSetInfo function call encode / decode 
 *
 * Created by   : Laurent Brard
 *
 * Date Created : 01/17/2002
 *
 *-----------------------------------------------------------------
 */
int32	//NFX_OK or NFX_ERR
cifsrpcEDnetServerSetInfoCall(	cifsrpc_ed *edp,
								netServerSetInfoCall *p, 
								uchar8 lv);


/*-----------------------------------------------------------------
 * Name         : cifsrpcEDnetServerSetInfoResp()
 *
 * Description  : SRVSVC NetServerSetInfo function response encode / decode 
 *
 * Created by   : Laurent Brard
 *
 * Date Created : 01/17/2002
 *
 *-----------------------------------------------------------------
 */
int32	//NFX_OK or NFX_ERR
cifsrpcEDnetServerSetInfoResp(	cifsrpc_ed *edp,
								netServerSetInfoResp *p, 
								uchar8 lv);


#endif	//#ifndef _CIFS_RPC_ED_SRVSVC_H


