/* vi:ts=4
 *-----------------------------------------------------------------
 *
 * Copyright (C) 2002, Agile Storage Inc.
 *
 * RCS:  $Id: cifs-rpc-srvsvc.h,v 1.3 2002/12/10 18:22:27 laurentb Exp $
 *
 * Name			: cifs-rpc-srvsvc.h
 *
 * Description	: Contains CIFS DCE/RPC server services definitions.
 *
 * Created by	: Laurent Brard
 *
 * Date Created	: 01/11/2002
 *
 *-----------------------------------------------------------------*/
   
#ifndef _CIFS_RPC_SRVSVC_H
#define _CIFS_RPC_SRVSVC_H

//Server services operation numbers
#define SRVSVC_NETCONENUM			0x08
#define SRVSVC_NETFILEENUM			0x09
#define SRVSVC_NETSESSENUM			0x0c
#define SRVSVC_NETSHAREADD			0x0e
#define SRVSVC_NETSHARENUM			0x0f
#define SRVSVC_NETSHAREGETINFO		0x10
#define SRVSVC_NETSHARESETINFO		0x11
#define SRVSVC_NETSHAREDEL			0x12
#define SRVSVC_NETSRVGETINFO		0x15
#define SRVSVC_NETSRVSETINFO		0x16
#define SRVSVC_NETSHARENUMSTICKY	0x24


//SRVSVC Share information level 1
typedef struct shareInfoLv1
{
  uint32 netNamep;					//netname pointer
  uint32 type;						//share type   
  uint32 remarkp;					//comment pointer
  
  cifsrpc_uniStruc netName;			//netname string
  cifsrpc_uniStruc remark;			//comment string
}shareInfoLv1;

//SRVSVC Share information level 2
typedef struct shareInfoLv2
{
  uint32 netNamep;					//netname pointer
  uint32 type;						//share type   
  uint32 remarkp;					//comment pointer
  uint32 perms;						//permissions
  uint32 nbUsesMax;					//maximum uses
  uint32 nbUses;					//current uses
  uint32 pathp;						//path name pointer
  uint32 passwordp;					//password pointer
  
  cifsrpc_uniStruc netName;			//netname string
  cifsrpc_uniStruc remark;			//comment string
  cifsrpc_uniStruc path;			//local path string   
  cifsrpc_uniStruc password;		//password string   
}shareInfoLv2;

//SRVSVC Share information level 502
typedef struct shareInfoLv502
{
  uint32 netNamep;					//netname pointer
  uint32 type;						//share type   
  uint32 remarkp;					//comment pointer
  uint32 perms;						//permissions
  uint32 nbUsesMax;					//maximum uses
  uint32 nbUses;					//current uses
  uint32 pathp;						//path name pointer
  uint32 passwordp;					//password pointer
  uint32 secDescSize;				//security descriptor size
  uint32 secDescp;					//security descriptor pointer
  
  cifsrpc_uniStruc netName;			//netname string
  cifsrpc_uniStruc remark;			//comment string
  cifsrpc_uniStruc path;			//local path string   
  cifsrpc_uniStruc password;		//password string   
  uint32		   sdSize2;			//security descriptor size
  union
  { 
      sec_cifsSd sd;
      sec_cifsSd *sdp;
  }secDesc;							//security descriptor
}shareInfoLv502;

//SRVSVC Share information level 1501
typedef sec_cifsSdBuf shareInfoLv1501;	//security descriptor buffer


//SRVSVC Share enumeration information container   
typedef struct shareEnumInfoContainer
{
  uint32 level;
  uint32 switchValue;
  uint32 shareInfop;

  uint32 entriesRead;
  uint32 entriesp;
  uint32 nbEntries;

  union {
	shareInfoLv1	*lv1p;   
	shareInfoLv2	*lv2p;   
	shareInfoLv502	*lv502p;   
	void *info;
  }info;

}shareEnumInfoContainer;

//SRVSVC NetShareEnum function call   
typedef struct netShareEnumCall
{
  uint32					serverNamep;   
  cifsrpc_uniStruc			serverName;   
  shareEnumInfoContainer	shiContr;   
  uint32 					prefMaxLen;   
  cifsrpc_enumHandle		resumeHandle;
  boolean					showHidden;
}netShareEnumCall;

//SRVSVC NetShareEnum function response   
typedef struct netShareEnumResp
{
  shareEnumInfoContainer	shiContr;   
  uint32					totalEntries;   
  cifsrpc_enumHandle		resumeHandle;
  uint32					returnValue;
}netShareEnumResp;


//SRVSVC Share information  
typedef struct shareInfo
{
  uint32 switchValue;
  uint32 shareInfop;
  union {
	shareInfoLv1	lv1;   
	shareInfoLv2	lv2;   
	shareInfoLv502	lv502;   
	shareInfoLv1501	lv1501;   
  }info;
}shareInfo;

//SRVSVC NetShareGetInfo function call   
typedef struct netShareGetInfoCall
{
  uint32			svNamep;   
  cifsrpc_uniStruc	svName;   
  cifsrpc_uniStruc	shareName;   
  uint32			level;   
}netShareGetInfoCall;

//SRVSVC NetShareGetInfo function response
typedef struct netShareGetInfoResp
{
  shareInfo			shareInfo;
  uint32			returnValue;
}netShareGetInfoResp;

//SRVSVC NetShareSetInfo function call   
typedef struct netShareSetInfoCall
{
  uint32			svNamep;   
  cifsrpc_uniStruc	svName;   
  cifsrpc_uniStruc	shareName;   
  uint32			level;   
  shareInfo			shareInfo;
}netShareSetInfoCall;

//SRVSVC NetShareSetInfo function response   
typedef struct netShareSetInfoResp
{
  uint32	unknown;   
  uint32	returnValue;
}netShareSetInfoResp;

//SRVSVC NetShareAdd function call
typedef struct netShareAddCall
{
  uint32			svNamep;   
  cifsrpc_uniStruc	svName;   
  uint32			level;   
  shareInfo			shareInfo;
}netShareAddCall;

//SRVSVC NetShareAdd function response
typedef struct netShareAddResp
{
  uint32	unknown;   
  uint32	returnValue;
}netShareAddResp;

//SRVSVC NetShareDel function call
typedef struct netShareDelCall
{
  uint32			svNamep;   
  cifsrpc_uniStruc	svName;   
  cifsrpc_uniStruc	shareName;   
}netShareDelCall;

//SRVSVC NetShareDel function response
typedef struct netShareDelResp
{
  uint32	returnValue;
}netShareDelResp;







//SRVSVC Server information level 100
typedef struct serverInfoLv100
{
  uint32 platformId;				//platform Id   
  uint32 namep;						//name pointer
  cifsrpc_uniStruc name;			//name   
}serverInfoLv100;

//SRVSVC Server information level 101
typedef struct serverInfoLv101
{
  uint32 platformId;				//platform Id   
  uint32 namep;						//name pointer
  uint32 verMajor;					//version Major
  uint32 verMinor;					//version Minor
  uint32 type;						//server type
  uint32 commentp;					//comment pointer
  
  cifsrpc_uniStruc name;			//name   
  cifsrpc_uniStruc comment;			//comment
}serverInfoLv101;

//SRVSVC Server information level 102
typedef struct serverInfoLv102
{
  uint32 platformId;				//platform Id   
  uint32 namep;						//name pointer
  uint32 verMajor;					//version Major
  uint32 verMinor;					//version Minor
  uint32 type;						//server type
  uint32 commentp;					//comment pointer
  uint32 users;
  uint32 disc;
  uint32 hidden;
  uint32 announce;
  uint32 annDelta;
  uint32 licenses;
  uint32 usrPathp;					//user path pointer
  
  cifsrpc_uniStruc name;			//name   
  cifsrpc_uniStruc comment;			//comment
  cifsrpc_uniStruc usrPath;			//user path
}serverInfoLv102;

//SRVSVC Server information container   
typedef struct serverInfoContainer
{
  uint32 switchValue;   
  uint32 serverInfop;   
  union
  {
	serverInfoLv100 lv100;   
	serverInfoLv101 lv101;   
	serverInfoLv102 lv102;   
  }info;
}serverInfoContainer;

//SRVSVC NetServerGetInfo function call   
typedef struct netServerGetInfoCall
{
  uint32			serverNamep;   
  cifsrpc_uniStruc	serverName;   
  uint32			level;   
}netServerGetInfoCall;

//SRVSVC NetServerGetInfo function response
typedef struct netServerGetInfoResp
{
  serverInfoContainer	sviContr;
  uint32				returnValue;
}netServerGetInfoResp;

//SRVSVC NetServerSetInfo function call   
typedef struct netServerSetInfoCall
{
  uint32				serverNamep;   
  cifsrpc_uniStruc		serverName;   
  uint32				level;   
  serverInfoContainer	sviContr;
}netServerSetInfoCall;

//SRVSVC NetServerSetInfo function response   
typedef struct netServerSetInfoResp
{
  uint32	unknown;   
  uint32	returnValue;
}netServerSetInfoResp;



/*
#define MAX_SERVER_DISK_ENTRIES 15

typedef struct disk_info {
	uint32  unknown;
UNISTR3 disk_name;
} DISK_INFO;

typedef struct disk_enum_container {
	uint32 level;
	uint32 entries_read;
	uint32 unknown;
	uint32 disk_info_ptr;
	DISK_INFO disk_info[MAX_SERVER_DISK_ENTRIES];
} DISK_ENUM_CONTAINER;

typedef struct net_srv_disk_enum {
	uint32 ptr_srv_name;         // pointer (to server name?)   
	UNISTR2 uni_srv_name;        // server name   

	DISK_ENUM_CONTAINER disk_enum_ctr;

	uint32 preferred_len;        // preferred maximum length (0xffff ffff)   
	uint32 total_entries;        // total number of entries   
	ENUM_HND enum_hnd;
	NTSTATUS status;               // return status   
} SRV_Q_NET_DISK_ENUM, SRV_R_NET_DISK_ENUM;

typedef struct net_name_validate {
	uint32 ptr_srv_name;
	UNISTR2 uni_srv_name;
	UNISTR2 uni_name; //name to validate  
	uint32 type;
	uint32 flags;
	NTSTATUS status;
} SRV_Q_NET_NAME_VALIDATE, SRV_R_NET_NAME_VALIDATE;

// SESS_INFO_0 (pointers to level 0 session info strings)   
typedef struct ptr_sess_info0
{
	uint32 ptr_name; // pointer to name.   

} SESS_INFO_0;

// SESS_INFO_0_STR (level 0 session info strings)   
typedef struct str_sess_info0
{
	UNISTR2 uni_name; // unicode string of name   

} SESS_INFO_0_STR;

// oops - this is going to take up a *massive* amount of stack.   
// the UNISTR2s already have 1024 uint16 chars in them...   
#define MAX_SESS_ENTRIES 32

// SRV_SESS_INFO_0   
typedef struct srv_sess_info_0_info
{
	uint32 num_entries_read;                     // EntriesRead   
	uint32 ptr_sess_info;                       // Buffer   
	uint32 num_entries_read2;                    // EntriesRead   

	SESS_INFO_0     info_0    [MAX_SESS_ENTRIES]; // session entry pointers   
	SESS_INFO_0_STR info_0_str[MAX_SESS_ENTRIES]; // session entry strings   

} SRV_SESS_INFO_0;

// SESS_INFO_1 (pointers to level 1 session info strings)   
typedef struct ptr_sess_info1
{
	uint32 ptr_name; // pointer to name.   
	uint32 ptr_user; // pointer to user name.   

	uint32 num_opens;
	uint32 open_time;
	uint32 idle_time;
	uint32 user_flags;

} SESS_INFO_1;

// SESS_INFO_1_STR (level 1 session info strings)   
typedef struct str_sess_info1
{
	UNISTR2 uni_name; // unicode string of name   
	UNISTR2 uni_user; // unicode string of user   

} SESS_INFO_1_STR;

// SRV_SESS_INFO_1   
typedef struct srv_sess_info_1_info
{
	uint32 num_entries_read;                     // EntriesRead   
	uint32 ptr_sess_info;                       // Buffer   
	uint32 num_entries_read2;                    // EntriesRead   

	SESS_INFO_1     info_1    [MAX_SESS_ENTRIES]; // session entry pointers   
	SESS_INFO_1_STR info_1_str[MAX_SESS_ENTRIES]; // session entry strings   

} SRV_SESS_INFO_1;

// SRV_SESS_INFO_CTR   
typedef struct srv_sess_info_ctr_info
{
	uint32 switch_value;         // switch value   
	uint32 ptr_sess_ctr;       // pointer to sess info union   
	union
    {
		SRV_SESS_INFO_0 info0; // session info level 0   
		SRV_SESS_INFO_1 info1; // session info level 1   

    } sess;

} SRV_SESS_INFO_CTR;


// SRV_Q_NET_SESS_ENUM   
typedef struct q_net_sess_enum_info
{
	uint32 ptr_srv_name;         // pointer (to server name?)   
	UNISTR2 uni_srv_name;        // server name   

	uint32 ptr_qual_name;         // pointer (to qualifier name)   
	UNISTR2 uni_qual_name;        // qualifier name "\\qualifier"   

	uint32 sess_level;          // session level   

	SRV_SESS_INFO_CTR *ctr;

	uint32 preferred_len;        // preferred maximum length (0xffff ffff)   
	ENUM_HND enum_hnd;

} SRV_Q_NET_SESS_ENUM;

// SRV_R_NET_SESS_ENUM   
typedef struct r_net_sess_enum_info
{
	uint32 sess_level;          // share level   

	SRV_SESS_INFO_CTR *ctr;

	uint32 total_entries;                    // total number of entries   
	ENUM_HND enum_hnd;

	NTSTATUS status;               // return status   

} SRV_R_NET_SESS_ENUM;

// CONN_INFO_0 (pointers to level 0 connection info strings)   
typedef struct ptr_conn_info0
{
	uint32 id; // connection id.   

} CONN_INFO_0;

// oops - this is going to take up a *massive* amount of stack.   
// the UNISTR2s already have 1024 uint16 chars in them...   
#define MAX_CONN_ENTRIES 32

// SRV_CONN_INFO_0   
typedef struct srv_conn_info_0_info
{
	uint32 num_entries_read;                     // EntriesRead   
	uint32 ptr_conn_info;                       // Buffer   
	uint32 num_entries_read2;                    // EntriesRead   

	CONN_INFO_0     info_0    [MAX_CONN_ENTRIES]; // connection entry pointers   

} SRV_CONN_INFO_0;

// CONN_INFO_1 (pointers to level 1 connection info strings)   
typedef struct ptr_conn_info1
{
	uint32 id;   // connection id   
	uint32 type; // 0x3   
	uint32 num_opens;
	uint32 num_users;
	uint32 open_time;

	uint32 ptr_usr_name; // pointer to user name.   
	uint32 ptr_net_name; // pointer to network name (e.g IPC$).   

} CONN_INFO_1;

// CONN_INFO_1_STR (level 1 connection info strings)   
typedef struct str_conn_info1
{
	UNISTR2 uni_usr_name; // unicode string of user   
	UNISTR2 uni_net_name; // unicode string of name   

} CONN_INFO_1_STR;

// SRV_CONN_INFO_1   
typedef struct srv_conn_info_1_info
{
	uint32 num_entries_read;                     // EntriesRead   
	uint32 ptr_conn_info;                       // Buffer   
	uint32 num_entries_read2;                    // EntriesRead   

	CONN_INFO_1     info_1    [MAX_CONN_ENTRIES]; // connection entry pointers   
	CONN_INFO_1_STR info_1_str[MAX_CONN_ENTRIES]; // connection entry strings   

} SRV_CONN_INFO_1;

// SRV_CONN_INFO_CTR   
typedef struct srv_conn_info_ctr_info
{
	uint32 switch_value;         // switch value   
	uint32 ptr_conn_ctr;       // pointer to conn info union   
	union
    {
		SRV_CONN_INFO_0 info0; // connection info level 0   
		SRV_CONN_INFO_1 info1; // connection info level 1   

    } conn;

} SRV_CONN_INFO_CTR;


// SRV_Q_NET_CONN_ENUM   
typedef struct q_net_conn_enum_info
{
	uint32 ptr_srv_name;         // pointer (to server name)   
	UNISTR2 uni_srv_name;        // server name "\\server"   

	uint32 ptr_qual_name;         // pointer (to qualifier name)   
	UNISTR2 uni_qual_name;        // qualifier name "\\qualifier"   

	uint32 conn_level;          // connection level   

	SRV_CONN_INFO_CTR *ctr;

	uint32 preferred_len;        // preferred maximum length (0xffff ffff)   
	ENUM_HND enum_hnd;

} SRV_Q_NET_CONN_ENUM;

// SRV_R_NET_CONN_ENUM   
typedef struct r_net_conn_enum_info
{
	uint32 conn_level;          // share level   

	SRV_CONN_INFO_CTR *ctr;

	uint32 total_entries;                    // total number of entries   
	ENUM_HND enum_hnd;

	NTSTATUS status;               // return status   

} SRV_R_NET_CONN_ENUM;

// FILE_INFO_3 (level 3 file info strings)   
typedef struct file_info3_info
{
	uint32 id;            // file index   
	uint32 perms;         // file permissions. don't know what format   
	uint32 num_locks;     // file locks   
	uint32 ptr_path_name; // file name   
	uint32 ptr_user_name; // file owner   

} FILE_INFO_3;

// FILE_INFO_3_STR (level 3 file info strings)   
typedef struct str_file_info3_info
{
	UNISTR2 uni_path_name; // unicode string of file name   
	UNISTR2 uni_user_name; // unicode string of file owner.   

} FILE_INFO_3_STR;

// oops - this is going to take up a *massive* amount of stack.   
// the UNISTR2s already have 1024 uint16 chars in them...   
#define MAX_FILE_ENTRIES 32

// SRV_FILE_INFO_3   
typedef struct srv_file_info_3
{
	uint32 num_entries_read;                     // EntriesRead   
	uint32 ptr_file_info;                        // Buffer   

	uint32 num_entries_read2;                    // EntriesRead   

	FILE_INFO_3     info_3    [MAX_FILE_ENTRIES]; // file entry details   
	FILE_INFO_3_STR info_3_str[MAX_FILE_ENTRIES]; // file entry strings   

} SRV_FILE_INFO_3;

// SRV_FILE_INFO_CTR   
typedef struct srv_file_info_3_info
{
	uint32 switch_value;         // switch value   
	uint32 ptr_file_ctr;       // pointer to file info union   
	union
    {
		SRV_FILE_INFO_3 info3; // file info with 0 entries   

    } file;

} SRV_FILE_INFO_CTR;


// SRV_Q_NET_FILE_ENUM   
typedef struct q_net_file_enum_info
{
	uint32 ptr_srv_name;         // pointer (to server name?)   
	UNISTR2 uni_srv_name;        // server name   

	uint32 ptr_qual_name;         // pointer (to qualifier name)   
	UNISTR2 uni_qual_name;        // qualifier name "\\qualifier"   

	uint32 file_level;          // file level   

	SRV_FILE_INFO_CTR *ctr;

	uint32 preferred_len; // preferred maximum length (0xffff ffff)   
	ENUM_HND enum_hnd;

} SRV_Q_NET_FILE_ENUM;


// SRV_R_NET_FILE_ENUM   
typedef struct r_net_file_enum_info
{
	uint32 file_level;          // file level   

	SRV_FILE_INFO_CTR *ctr;

	uint32 total_entries;                    // total number of files   
	ENUM_HND enum_hnd;

	NTSTATUS status;        // return status   

} SRV_R_NET_FILE_ENUM;

// SRV_Q_NET_REMOTE_TOD   
typedef struct q_net_remote_tod
{
	uint32  ptr_srv_name;
	UNISTR2 uni_srv_name; // "\\server"   

} SRV_Q_NET_REMOTE_TOD;

// TIME_OF_DAY_INFO   
typedef struct time_of_day_info
{
	uint32	elapsedt;
	uint32	msecs;
	uint32	hours;
	uint32	mins;
	uint32	secs;
	uint32	hunds;
	uint32	zone;
	uint32	tintervals;
	uint32	day;
	uint32	month;
	uint32	year;
	uint32	weekday;
	
} TIME_OF_DAY_INFO;

// SRV_R_NET_REMOTE_TOD   
typedef struct r_net_remote_tod
{
	uint32 ptr_srv_tod;         // pointer to TOD   
	TIME_OF_DAY_INFO *tod;
	
	NTSTATUS status;               // return status   

} SRV_R_NET_REMOTE_TOD;

// SRV_Q_NET_FILE_QUERY_SECDESC   
typedef struct q_net_file_query_secdesc
{
	uint32  ptr_srv_name;
	UNISTR2 uni_srv_name;
	uint32  ptr_qual_name;
	UNISTR2 uni_qual_name;
	UNISTR2 uni_file_name;
	uint32  unknown1;
	uint32  unknown2;
	uint32  unknown3;
} SRV_Q_NET_FILE_QUERY_SECDESC;

// SRV_R_NET_FILE_QUERY_SECDESC   
typedef struct r_net_file_query_secdesc
{
	uint32 ptr_response;
	uint32 size_response;
	uint32 ptr_secdesc;
	uint32 size_secdesc;
	SEC_DESC *sec_desc;
	NTSTATUS status;
} SRV_R_NET_FILE_QUERY_SECDESC;

// SRV_Q_NET_FILE_SET_SECDESC   
typedef struct q_net_file_set_secdesc
{
	uint32  ptr_srv_name;
	UNISTR2 uni_srv_name;
	uint32  ptr_qual_name;
	UNISTR2 uni_qual_name;
	UNISTR2 uni_file_name;
	uint32  sec_info;
	uint32  size_set;
	uint32  ptr_secdesc;
	uint32  size_secdesc;
	SEC_DESC *sec_desc;
} SRV_Q_NET_FILE_SET_SECDESC;

// SRV_R_NET_FILE_SET_SECDESC   
typedef struct r_net_file_set_secdesc
{
	NTSTATUS status;
} SRV_R_NET_FILE_SET_SECDESC;
*/

//Server services api entry point function
int32 cifsrpc_srvsvc(cifsrpc_pipe *rpip);

#endif // #ifndef _CIFS_RPC_SRVSVC_H 

