/* vi:ts=4
 *-----------------------------------------------------------------
 *
 * Copyright (C) 2001, Agile Storage Inc.
 *
 * RCS:  $Id: cifs-cmd.c,v 1.1.1.1 2002/04/23 17:41:16 laurentb Exp $
 *
 * Name			:        cifs-cmd.c
 *                              
 * Description	: Contain CIFS UI command functions.
 *
 * Created by	: Laurent Brard 
 *
 * Date Created	: 11/17/2001
 *
 *-----------------------------------------------------------------
 */
#include "cifs-api.h"
#include "testcifs.h"

#ifdef FS_STANDALONE
int32 strequ(char8	* str1, char8	* str2)
{
	if (!str1 || !str2) return (0);
	while (*str1)
		if (*str1++ != *str2++)	return (0);
	if (*str2 == 0)	return (1);
	return (0);
}	/* strequ() */

#endif

int32 cifs_cmdShow  (int32 argc, char *argv[] );
int32 cifs_cmdTrace (int32 argc, char *argv[] );
int32 cifs_cmdDebug (int32 argc, char *argv[] );
int32 cifs_cmdDbgRap(int32 argc, char *argv[] );
int32 cifs_cmdDbgRpc(int32 argc, char *argv[] );
int32 cifs_cmdDbgT2 (int32 argc, char *argv[] );
int32 cifs_cmdDbgVol(int32 argc, char *argv[] );
int32 cifs_cmdDbgFos(int32 argc, char *argv[] );
int32 nb_cmdUnitTest(int32 argc, char *argv[] );

typedef struct
{
	int32   (*cmdFunc)( int32 argc, char *argv[] );
	char    *cmd;
	char    *helpTxt;
} cifs_cmdTbl_t;

cifs_cmdTbl_t    cifs_cmdTable[ ] =
{
  { cifs_cmdShow,  "show",	"cifs show statistics" },
  { cifs_cmdTrace, "trace",	"cifs trace on|off" },
  { cifs_cmdDebug, "d",		"cifs d i|nbss|neg|ss1|ss2|ss3|tc1|cre|clo|echo\n" },
  { cifs_cmdDbgRpc,"drpc",	"cifs drpc bind|she|she2|shes|shgi2|shgi502|"
							"shsi2|shsi1501|\n"
							"shadd|shdel|svgi|svgi2|svsi2|fie|sse|wksqi\n" },
  { cifs_cmdDbgRap,"drap",	"cifs drap shenum|shinfo|svinfo\n" },
  { cifs_cmdDbgT2, "dt2",	"cifs dt2 uu|ffq|ffp|fnq|fnp|fc|qfsi|qpi|qfi|spi|sfi\n"},
  { cifs_cmdDbgVol,"dvol",	"cifs dvol vol <thread id>|sh <thread id>\n"},
  { cifs_cmdDbgFos,"dfo",	"cifs dfo mkdir|ckpa|rmdir|cre|opdir|opfdir|open|"
		  					"rdx|wtx|wt|clo|delfile\n" },
#ifndef FS_STANDALONE							
  { nb_cmdUnitTest,"nb",	"cifs nb oreqque1|oreqque2|oreqque3|oreqrel|"
		  					"orspreg|orspque|orsprel|\n"
							"        ireqreg|ireqque|ireqrel|\n"
							"        irspreg|irspref|irspque|irsprel|irspwack|\n"
							"        ireqss|odata|idata|ika|oka|endss\n"
							"        odgm\n"}
#endif  //FS_STANDALONE							
};

#define CIFS_CMD_MAX	sizeof(cifs_cmdTable)/sizeof(cifs_cmdTbl_t)

/*-----------------------------------------------------------------
 * Name: cifs_cmdHelpAll
 *
 * Description: Display help-texts for all CIFS commands.
 *
 * Created by : Laurent Brard 
 *
 * Date Created: 11/17/2001
 *-----------------------------------------------------------------
 */
int32 cifs_cmdHelpAll(void)
{
	int32	i;

	printf("Usage:");
	for (i=0; i < CIFS_CMD_MAX; i++)
	{
		printf("\t%s\n", cifs_cmdTable[i].helpTxt);
	}
	return (0);
}

/*-----------------------------------------------------------------
 * Name: cifs_cmdHelpCmd
 *
 * Description: Display help-texts for an individual CIFS command.
 *
 * Created by : Laurent Brard 
 * 
 * Date Created: 11/17/2001
 *-----------------------------------------------------------------
 */
int32 cifs_cmdHelpCmd (char *pCmd)
{
	int32	i;

	for (i=0; i < CIFS_CMD_MAX; i++)
	{
		if (strequ(pCmd, cifs_cmdTable[i].cmd))
		{
			printf("Usage:");
			printf("\t%s\n", cifs_cmdTable[i].helpTxt);
			return 0;
		}
	}
	return (cifs_cmdHelpAll( ));

}


/*-----------------------------------------------------------------
 * Name: cifs_cmdHelp
 *
 * Description: Display CIFS help-texts.
 *
 *              cifs help [show | trace]
 *
 * Created by : Laurent Brard 
 *
 * Date Created: 11/17/2001
 *-----------------------------------------------------------------
 */
int32 cifs_cmdHelp ( int32 argc, char *argv[] )
{
	int32 i;

	if (argc < 3)
		return (cifs_cmdHelpAll());
	else
		return (cifs_cmdHelpCmd( argv[2] ));
}
/*-----------------------------------------------------------------
 * Name: cifs_cmdShow
 *
 * Description: Dispatch for cifs show commands.
 *
 *              cifs show [stat]
 * 
 * Created by : Laurent Brard 
 * 
 * Date Created: 11/17/2001
 *-----------------------------------------------------------------
 */
int32 cifs_cmdShow ( int32 argc, char *argv[] )
{
/*		
	if (argc < 3)
	{
		cifs_cmdHelpCmd("show");
		return (0);
	}

	if ( !(strncmp(argv[2], "stat", 4)))
	{
		return (cifs_displayStat());
	}
	else
		return (cifs_cmdHelpCmd("show"));
*/		
	return (0);
}
/*-----------------------------------------------------------------
 * Name: cifs_cmdTrace
 *
 * Description: Enables or disables CIFS tracing.
 *
 *              cifs trace <on|off>
 * 
 * Created by : Laurent Brard 
 * 
 * Date Created: 11/17/2001
 *-----------------------------------------------------------------
 */
int32 cifs_cmdTrace ( int32 argc, char *argv[] )
{
/*		
	if (argc < 3)
	{
		cifs_cmdHelpCmd("trace");
		return 0;
	}
	if (strequ(argv[2], "on"))	
	{
		trace_enable = 1;
		printf("CIFS trace has been turned on.\n");
	}
	else if (strequ(argv[2], "off"))
	{
		trace_enable = 0;
		printf("CIFS trace has been turned off\n");
	}
	else
		cifs_cmdHelpCmd("trace");
*/
	return 0;
}

/*-----------------------------------------------------------------
 * Name: cifs_cmdDebug
 *
 * Description: Enables or disables CIFS Debug.
 *
 *              cifs Debug <on|off>
 * 
 * Created by : Laurent Brard 
 * 
 * Date Created: 11/17/2001
 *-----------------------------------------------------------------
 */
int32 cifs_cmdDebug( int32 argc, char *argv[] )
{
	if (argc < 3)
	{
		cifs_cmdHelpCmd("d");
		return 0;
	}
//	if (strequ(argv[2], "init"))	
	if (strequ(argv[2], "i"))	
	{
		cifstest_init();
		printf("CIFS Debug init\n");
	}
	else if (strequ(argv[2], "nbss"))
	{
		cifstest_createSession();
		printf("CIFS Debug session created\n");
	}
	else if (strequ(argv[2], "neg"))
	{
		printf("negociation message sent\n");
		cifstest_negociate();
	}
	else if (strequ(argv[2], "ss1"))
	{
		printf("session setup 1 message sent\n");
		cifstest_sessionSetup1();
	}
	else if (strequ(argv[2], "ss2"))
	{
		printf("session setup 2 message sent\n");
		cifstest_sessionSetup2();
	}
	else if (strequ(argv[2], "ss3"))
	{
		printf("session setup 3 message sent\n");
		cifstest_sessionSetup3();
	}
	else if (strequ(argv[2], "tc1"))
    {
        printf("tree connect 1 message sent\n");
		cifstest_treeConnectNX1();
	}
	else if (strequ(argv[2], "cre"))
	{
		printf("create and X message sent\n");
		cifstest_createNX();
	}
	else if (strequ(argv[2], "clo"))
	{
		printf("closePipe message sent\n");
		cifstest_closePipe();
	}
	else if (strequ(argv[2], "echo"))
	{
		printf("Echo message sent\n");
		cifstest_echo();
	}
	else
		cifs_cmdHelpCmd("d");

	return 0;
}



/*-----------------------------------------------------------------
 * Name: cifs_cmdDbgRpc
 *
 * Description: send Rpc commands for debug.
 *
 * Created by : Laurent Brard 
 * 
 * Date Created: 01/18/2002
 *-----------------------------------------------------------------
 */
int32 cifs_cmdDbgRpc( int32 argc, char *argv[] )
{
	if (argc < 3)
	{
		cifs_cmdHelpCmd("drpc");
		return 0;
	}
	if (strequ(argv[2], "bind"))
	{
		printf("rpc bind message sent\n");
		cifstest_rpcBind();
	}
	else if (strequ(argv[2], "she"))
	{
		printf("netrShareEnum message sent\n");
		cifstest_netrShareEnum();
	}
	else if (strequ(argv[2], "she2"))
	{
		printf("netrShareEnum2 message sent\n");
		cifstest_netrShareEnum2();
	}
	else if (strequ(argv[2], "shes"))
	{
		printf("netrShareEnumSticky2 message sent\n");
		cifstest_netrShareEnumSticky2();
	}
	else if (strequ(argv[2], "shgi2"))
	{
		printf("netrShareGetInfo2 message sent\n");
		cifstest_netrShareGetInfo2();
	}
	else if (strequ(argv[2], "shgi502"))
	{
		printf("netrShareGetInfo502 message sent\n");
		cifstest_netrShareGetInfo502();
	}
	else if (strequ(argv[2], "shsi2"))
	{
		printf("netrShareSetInfo2 message sent\n");
		cifstest_netrShareSetInfo2();
	}
	else if (strequ(argv[2], "shsi1501"))
	{
		printf("netrShareSetInfo1501 message sent\n");
		cifstest_netrShareSetInfo1501();
	}
	else if (strequ(argv[2], "shadd"))
	{
		printf("netrShareAdd message sent\n");
		cifstest_netrShareAdd();
	}
	else if (strequ(argv[2], "shdel"))
	{
		printf("netrShareDel message sent\n");
		cifstest_netrShareDel();
	}
	else if (strequ(argv[2], "svgi"))
	{
		printf("netrServerGetInfo101 message sent\n");
		cifstest_netrServerGetInfo101();
	}
	else if (strequ(argv[2], "svgi2"))
	{
		printf("netrServerGetInfo102 message sent\n");
		cifstest_netrServerGetInfo102();
	}
	else if (strequ(argv[2], "svsi2"))
	{
		printf("netrServerSetInfo102 message sent\n");
		cifstest_netrServerSetInfo102();
	}
	else if (strequ(argv[2], "fie"))
	{
		printf("netrFileEnum message sent\n");
		cifstest_netrFileEnum();
	}
	else if (strequ(argv[2], "sse"))
	{
		printf("netrSessionEnum message sent\n");
		cifstest_netrSessionEnum();
	}
	else if (strequ(argv[2], "wksqi"))
	{
		printf("wkssvc Query Info message sent\n");
		cifstest_wkssvcQueryInfo100();
	}
	else
		cifs_cmdHelpCmd("drpc");

	return (0);
}							

/*-----------------------------------------------------------------
 * Name: cifs_cmdDbgRap
 *
 * Description: send Rap messages for debug.
 *
 * Created by : Laurent Brard 
 * 
 * Date Created: 01/18/2002
 *-----------------------------------------------------------------
 */
int32 cifs_cmdDbgRap( int32 argc, char *argv[] )
{
	if (argc < 3)
	{
		cifs_cmdHelpCmd("drap");
		return 0;
	}
	if (strequ(argv[2], "shenum"))
	{
		printf("RAP NetShareEnum message sent\n");
		cifstest_rapNetShareEnum();
	}
	else if (strequ(argv[2], "shinfo"))
	{
		printf("RAP NetShareGetInfo message sent\n");
		cifstest_rapNetShareGetInfo();
	}
	else if (strequ(argv[2], "svinfo"))
	{
		printf("RAP NetServerGetInfo message sent\n");
		cifstest_rapNetServerGetInfo();
	}
	else
		cifs_cmdHelpCmd("drap");

	return 0;
}

/*-----------------------------------------------------------------
 * Name: cifs_cmdDbgT2
 *
 * Description: send Transaction2 messages for debug.
 *
 * Created by : Laurent Brard 
 * 
 * Date Created: 01/18/2002
 *-----------------------------------------------------------------
 */
int32 cifs_cmdDbgT2( int32 argc, char *argv[] )
{
	if (argc < 3)
	{
		cifs_cmdHelpCmd("dt2");
		return 0;
	}
	if (strequ(argv[2], "uu"))
	{
		printf("Converting between Unicode and Utf-8 strings\n");
		cifstest_uniAndUtf8();
	}
	else if (strequ(argv[2], "ffq"))
	{
		printf("Transaction2 Find First request message sent\n");
		cifstest_t2FindFirstReq();
	}
	else if (strequ(argv[2], "ffp"))
	{
		printf("Transaction2 Find First response message sent\n");
		cifstest_t2FindFirstRsp();
	}
	else if (strequ(argv[2], "fnq"))
	{
		printf("Transaction2 Find Next request message sent\n");
		cifstest_t2FindNextReq();
	}
	else if (strequ(argv[2], "fnp"))
	{
		printf("Transaction2 Find Next response message sent\n");
		cifstest_t2FindNextRsp();
	}
	else if (strequ(argv[2], "fc"))
	{
		printf("Find close2 message sent\n");
		cifstest_t2FindClose();
	}
	else if (strequ(argv[2], "qfsi"))
	{
		printf("Transaction2 Query Filesystem Info message sent\n");
		cifstest_t2QueryFilesystemInfo();
	}
	else if (strequ(argv[2], "qpi"))
	{
		printf("Transaction2 Query Path Info message sent\n");
		cifstest_t2QueryPathInfo();
	}	
	else if (strequ(argv[2], "qfi"))
	{
		printf("Transaction2 Query File Info message sent\n");
		cifstest_t2QueryFileInfo();
	}	
	else if (strequ(argv[2], "spi"))
	{
		printf("Transaction2 Set Path Info message sent\n");
		cifstest_t2SetPathInfo();
	}	
	else if (strequ(argv[2], "sfi"))
	{
		printf("Transaction2 Set File Info message sent\n");
		cifstest_t2SetFileInfo();
	}	
	else
		cifs_cmdHelpCmd("dt2");

  return 0;
}

/*-----------------------------------------------------------------
 * Name: cifs_cmdDbgVol
 *
 * Description: send FS and EVM messages for getting share handle debug.
 *
 * Created by : Xiaobing Zhang 
 * 
 * Date Created: 03/15/2002
 *-----------------------------------------------------------------
 */
int32 cifs_cmdDbgVol( int32 argc, char *argv[] )
{
	esm_handle_t threadId;

	if (argc < 4)
	{
		cifs_cmdHelpCmd("dvol");
		return 0;
	}
/*
	threadId = (esm_handle_t) atoi(argv[3]);

	if (strequ(argv[2], "vol"))
	{
		printf("EVM Get Vol Id response message sent\n");
		cifs_simfs_evmVolId(threadId);
	}
	else if (strequ(argv[2], "sh"))
	{
		printf("FS Get Root response message sent\n");
		cifs_simfs_ROOT(threadId);
	}
	else
		cifs_cmdHelpCmd("dvol");
*/		
	return 0;
}


/*-----------------------------------------------------------------
 * Name: cifs_cmdDbgFos
 *
 * Description: debug file operations
 *
 * Created by : Laurent Brard 
 * 
 * Date Created: 03/20/2002
 *-----------------------------------------------------------------
 */
int32 cifs_cmdDbgFos( int32 argc, char *argv[] )
{
	if (argc < 3)
	{
		cifs_cmdHelpCmd("dfo");
		return 0;
	}
	if (strequ(argv[2], "ckpa"))	
	{
		printf("CIFS check path\n");
		cifstest_checkPath();
	}
	else if (strequ(argv[2], "mkdir"))
	{
		printf("CIFS create directory\n");
		cifstest_mkdir();
	}
	else if (strequ(argv[2], "rmdir"))
	{
		printf("CIFS delete directory\n");
		cifstest_rmdir();
	}
	else if (strequ(argv[2], "cre"))	
	{
		printf("CIFS NTcreateNX file\n");
		cifstest_NTCreateNX();
	}
	else if (strequ(argv[2], "opdir"))	
	{
		printf("CIFS NTcreateNX opendir\n");
		cifstest_openDir();
	}
	else if (strequ(argv[2], "opfdir"))	
	{
		printf("CIFS NTcreateNX open from dir handle\n");
		cifstest_openFromDirHandle();
	}
	else if (strequ(argv[2], "open"))	
	{
		printf("CIFS openNX\n");
		cifstest_openNX();
	}
	else if (strequ(argv[2], "clo"))	
	{
		printf("CIFS close file\n");
		cifstest_closeFile();
	}
	else if (strequ(argv[2], "rdx"))
	{
		printf("CIFS readNX file\n");
		cifstest_readNXFile();
	}
	else if (strequ(argv[2], "wtx"))
	{
		printf("CIFS writeNX file\n");
		cifstest_writeNXFile();
	}
	else if (strequ(argv[2], "wt"))
	{
		printf("CIFS write file\n");
		cifstest_writeFile();
	}
	else if (strequ(argv[2], "delfile"))
	{
		printf("CIFS delete file\n");
		cifstest_deleteFile();
	}
	else
		cifs_cmdHelpCmd("dfo");

	return (0);
}

/*-----------------------------------------------------------------
 * Name: cifs_cmd
 *
 * Description: Main dispatch for CIFS commands.  Called from
 *              command table.
 *
 *				cifs show statistics
 *				cifs trace on|off
 * Created by : Laurent Brard
 *
 * Date Created: 11/17/2001
 *-----------------------------------------------------------------
 */
int32 cifs_cmd ( int32 argc, char *argv[] )
{
	int32	i;

	if (argc < 2)
	{
		return (cifs_cmdHelp(argc, argv));
	}

	for (i=0; i < CIFS_CMD_MAX; i++)
	{
		if (strequ(argv[1], cifs_cmdTable[i].cmd))
		{
			return ((*cifs_cmdTable[i].cmdFunc)(argc, argv));
		}
	}
	return (cifs_cmdHelp(argc, argv));
}
