/* vi:ts=4
 *-----------------------------------------------------------------
 *
 * Copyright (C) 2001, Agile Storage Inc.
 * 
 * RCS:  $Id: cifs-err.c,v 1.1.1.1 2002/04/23 17:41:16 laurentb Exp $
 *
 * Name			: cifs-error.c
 *
 * Description	: Contains CIFS errors conversion functions. 
 *
 * Created by	: Laurent Brard
 *
 * Date Created	: 03/21/2002
 *
 *-----------------------------------------------------------------
 */

#include "cifs-api.h"
#include "cifs-err.h"

#define IFDBG(lvl)  if (lvl <= cifs_debugLevel)

#define CIFSTATUS_ERRMAX	(sizeof(cifstatus2Dos) / sizeof(cifs_dosError))

typedef struct cifs_dosError {
	uchar8		class;
	ushort16	code;
}cifs_dosError;

static cifs_dosError cifstatus2Dos[] =  { 
 { 0,		   0 },					// CIFSTATUS_OK
 { CIFS_ERRDOS, CIFSERR_general },	// CIFSTATUS_UNSUCCESSFUL
 { CIFS_ERRDOS, CIFSERR_badfunc },	// CIFSTATUS_NOT_IMPLEMENTED
 { CIFS_ERRDOS, 87 },				// CIFSTATUS_INVALID_INFO_CLASS
 { CIFS_ERRDOS, 24 },				// CIFSTATUS_INFO_LENGTH_MISMATCH
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_ACCESS_VIOLATION
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_IN_PAGE_ERROR
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_PAGEFILE_QUOTA
 { CIFS_ERRDOS, CIFSERR_badfid },	// CIFSTATUS_INVALID_HANDLE
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_BAD_INITIAL_STACK
 { CIFS_ERRDOS, 193 },				// CIFSTATUS_BAD_INITIAL_PC
 { CIFS_ERRDOS, 87 },				// CIFSTATUS_INVALID_CID
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_TIMER_NOT_CANCELED
 { CIFS_ERRDOS, 87 },				// CIFSTATUS_INVALID_PARAMETER
 { CIFS_ERRDOS, CIFSERR_badfile },	// CIFSTATUS_NO_SUCH_DEVICE
 { CIFS_ERRDOS, CIFSERR_badfile },	// CIFSTATUS_NO_SUCH_FILE
 { CIFS_ERRDOS, CIFSERR_badfunc },	// CIFSTATUS_INVALID_DEVICE_REQUEST
 { CIFS_ERRDOS, 38 },				// CIFSTATUS_END_OF_FILE
 { CIFS_ERRDOS, 34 },				// CIFSTATUS_WRONG_VOLUME
 { CIFS_ERRDOS, 21 },				// CIFSTATUS_NO_MEDIA_IN_DEVICE
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_UNRECOGNIZED_MEDIA
 { CIFS_ERRDOS, 27 },				// CIFSTATUS_NONEXISTENT_SECTOR
 { 0, 0 },   //Change to  { CIFS_ERRHRD,  CIFSERR_general }, ?
 { CIFS_ERRDOS, CIFSERR_nomem },	// CIFSTATUS_NO_MEMORY
 { CIFS_ERRDOS, 487 },				// CIFSTATUS_CONFLICTING_ADDRESSES
 { CIFS_ERRDOS, 487 },				// CIFSTATUS_NOT_MAPPED_VIEW
 { CIFS_ERRDOS, 87 },				// CIFSTATUS_UNABLE_TO_FREE_VM
 { CIFS_ERRDOS, 87 },				// CIFSTATUS_UNABLE_TO_DELETE_SECTION
 { CIFS_ERRDOS, 2142 },				// CIFSTATUS_INVALID_SYSTEM_SERVICE
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_ILLEGAL_INSTRUCTION
 { CIFS_ERRDOS, CIFSERR_noaccess },	// CIFSTATUS_INVALID_LOCK_SEQUENCE
 { CIFS_ERRDOS, CIFSERR_noaccess },	// CIFSTATUS_INVALID_VIEW_SIZE
 { CIFS_ERRDOS, 193 },				// CIFSTATUS_INVALID_FILE_FOR_SECTION
 { CIFS_ERRDOS, CIFSERR_noaccess },	// CIFSTATUS_ALREADY_COMMITTED
 { CIFS_ERRDOS, CIFSERR_noaccess },	// CIFSTATUS_ACCESS_DENIED
 { CIFS_ERRDOS, 111 },				// CIFSTATUS_BUFFER_TOO_SMALL
 { CIFS_ERRDOS, CIFSERR_badfid },	// CIFSTATUS_OBJECT_TYPE_MISMATCH
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_NONCONTINUABLE_EXCEPTION
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_INVALID_DISPOSITION
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_UNWIND
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_BAD_STACK
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_INVALID_UNWIND_TARGET
 { CIFS_ERRDOS, 158 },				// CIFSTATUS_NOT_LOCKED
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_PARITY_ERROR
 { CIFS_ERRDOS, 487 },				// CIFSTATUS_UNABLE_TO_DECOMMIT_VM
 { CIFS_ERRDOS, 487 },				// CIFSTATUS_NOT_COMMITTED
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_INVALID_PORT_ATTRIBUTES
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_PORT_MESSAGE_TOO_LONG
 { CIFS_ERRDOS, 87 },				// CIFSTATUS_INVALID_PARAMETER_MIX
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_INVALID_QUOTA_LOWER
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_DISK_CORRUPT_ERROR
 { CIFS_ERRDOS, CIFSERR_invalidname},// CIFSTATUS_OBJECT_NAME_INVALID
 { CIFS_ERRDOS, CIFSERR_badfile },	// CIFSTATUS_OBJECT_NAME_NOT_FOUND
 { CIFS_ERRDOS, 183 },				// CIFSTATUS_OBJECT_NAME_COLLISION
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_HANDLE_NOT_WAITABLE
 { CIFS_ERRDOS, CIFSERR_badfid },	// CIFSTATUS_PORT_DISCONNECTED
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_DEVICE_ALREADY_ATTACHED
 { CIFS_ERRDOS, 161 },				// CIFSTATUS_OBJECT_PATH_INVALID
 { CIFS_ERRDOS, CIFSERR_badpath },	// CIFSTATUS_OBJECT_PATH_NOT_FOUND
 { CIFS_ERRDOS, 161 },				// CIFSTATUS_OBJECT_PATH_SYNTAX_BAD
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_DATA_OVERRUN
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_DATA_LATE_ERROR
 { CIFS_ERRDOS, 23 },				// CIFSTATUS_DATA_ERROR
 { CIFS_ERRDOS, 23 },				// CIFSTATUS_CRC_ERROR
 { CIFS_ERRDOS, CIFSERR_nomem },	// CIFSTATUS_SECTION_TOO_BIG
 { CIFS_ERRDOS, CIFSERR_noaccess },	// CIFSTATUS_PORT_CONNECTION_REFUSED
 { CIFS_ERRDOS, CIFSERR_badfid },	// CIFSTATUS_INVALID_PORT_HANDLE
 { CIFS_ERRDOS, CIFSERR_badshare },	// CIFSTATUS_SHARING_VIOLATION
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_QUOTA_EXCEEDED
 { CIFS_ERRDOS, 87 },				// CIFSTATUS_INVALID_PAGE_PROTECTION
 { CIFS_ERRDOS, 288 },				// CIFSTATUS_MUTANT_NOT_OWNED
 { CIFS_ERRDOS, 298 },				// CIFSTATUS_SEMAPHORE_LIMIT_EXCEEDED
 { CIFS_ERRDOS, 87 },				// CIFSTATUS_PORT_ALREADY_SET
 { CIFS_ERRDOS, 87 },				// CIFSTATUS_SECTION_NOT_IMAGE
 { CIFS_ERRDOS, 156 },				// CIFSTATUS_SUSPEND_COUNT_EXCEEDED
 { CIFS_ERRDOS, CIFSERR_noaccess },	// CIFSTATUS_THREAD_IS_TERMINATING
 { CIFS_ERRDOS, 87 },				// CIFSTATUS_BAD_WORKING_SET_LIMIT
 { CIFS_ERRDOS, 87 },				// CIFSTATUS_INCOMPATIBLE_FILE_MAP
 { CIFS_ERRDOS, 87 },				// CIFSTATUS_SECTION_PROTECTION
 { CIFS_ERRDOS, 282 },				// CIFSTATUS_EAS_NOT_SUPPORTED
 { CIFS_ERRDOS, 255 },				// CIFSTATUS_EA_TOO_LARGE
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_NONEXISTENT_EA_ENTRY
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_NO_EAS_ON_FILE
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_EA_CORRUPT_ERROR
 { CIFS_ERRDOS, CIFSERR_lock },		// CIFSTATUS_FILE_LOCK_CONFLICT
 { CIFS_ERRDOS, CIFSERR_lock },		// CIFSTATUS_LOCK_NOT_GRANTED
 { CIFS_ERRDOS, CIFSERR_noaccess },	// CIFSTATUS_DELETE_PENDING
 { CIFS_ERRDOS, CIFSERR_unsup },	// CIFSTATUS_CTL_FILE_NOT_SUPPORTED
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_UNKNOWN_REVISION
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_REVISION_MISMATCH
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_INVALID_OWNER
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_INVALID_PRIMARY_GROUP
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_NO_IMPERSONATION_TOKEN
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_CANT_DISABLE_MANDATORY
 { CIFS_ERRDOS, 2215 },				// CIFSTATUS_NO_LOGON_SERVERS
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_NO_SUCH_LOGON_SESSION
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_NO_SUCH_PRIVILEGE
 { CIFS_ERRDOS, CIFSERR_noaccess },	// CIFSTATUS_PRIVILEGE_NOT_HELD
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_INVALID_ACCOUNT_NAME
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_USER_EXISTS
 { CIFS_ERRDOS, CIFSERR_noaccess },	// CIFSTATUS_NO_SUCH_USER
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_GROUP_EXISTS
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_NO_SUCH_GROUP
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_MEMBER_IN_GROUP
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_MEMBER_NOT_IN_GROUP
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_LAST_ADMIN
 { CIFS_ERRSRV, CIFSERR_badpw },	// CIFSTATUS_WRONG_PASSWORD
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_ILL_FORMED_PASSWORD
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_PASSWORD_RESTRICTION
 { CIFS_ERRDOS, CIFSERR_noaccess },	// CIFSTATUS_LOGON_FAILURE
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_ACCOUNT_RESTRICTION
 { CIFS_ERRSRV, 2241 },				// CIFSTATUS_INVALID_LOGON_HOURS
 { CIFS_ERRSRV, 2240 },				// CIFSTATUS_INVALID_WORKSTATION
 { CIFS_ERRSRV, 2242 },				// CIFSTATUS_PASSWORD_EXPIRED
 { CIFS_ERRSRV, 2239 },				// CIFSTATUS_ACCOUNT_DISABLED
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_NONE_MAPPED
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_TOO_MANY_LUIDS_REQUESTED
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_LUIDS_EXHAUSTED
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_INVALID_SUB_AUTHORITY
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_INVALID_ACL
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_INVALID_SID
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_INVALID_SECURITY_DESCR
 { CIFS_ERRDOS, 127 },				// CIFSTATUS_PROCEDURE_NOT_FOUND
 { CIFS_ERRDOS, 193 },				// CIFSTATUS_INVALID_IMAGE_FORMAT
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_NO_TOKEN
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_BAD_INHERITANCE_ACL
 { CIFS_ERRDOS, 158 },				// CIFSTATUS_RANGE_NOT_LOCKED
 { CIFS_ERRDOS, 112 },				// CIFSTATUS_DISK_FULL
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_SERVER_DISABLED
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_SERVER_NOT_DISABLED
 { CIFS_ERRDOS, 68 },				// CIFSTATUS_TOO_MANY_GUIDS_REQUESTED
 { CIFS_ERRDOS, 259 },				// CIFSTATUS_GUIDS_EXHAUSTED
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_INVALID_ID_AUTHORITY
 { CIFS_ERRDOS, 259 },				// CIFSTATUS_AGENTS_EXHAUSTED
 { CIFS_ERRDOS, 154 },				// CIFSTATUS_INVALID_VOLUME_LABEL
 { CIFS_ERRDOS, CIFSERR_reserved },	// CIFSTATUS_SECTION_NOT_EXTENDED
 { CIFS_ERRDOS, 487 },				// CIFSTATUS_NOT_MAPPED_DATA
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_RESOURCE_DATA_NOT_FOUND
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_RESOURCE_TYPE_NOT_FOUND
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_RESOURCE_NAME_NOT_FOUND
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_ARRAY_BOUNDS_EXCEEDED
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_FLOAT_DENORMAL_OPERAND
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_FLOAT_DIVIDE_BY_ZERO
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_FLOAT_INEXACT_RESULT
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_FLOAT_INVALID_OPERATION
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_FLOAT_OVERFLOW
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_FLOAT_STACK_CHECK
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_FLOAT_UNDERFLOW
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_INTEGER_DIVIDE_BY_ZERO
 { CIFS_ERRDOS, 534 },				// CIFSTATUS_INTEGER_OVERFLOW
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_PRIVILEGED_INSTRUCTION
 { CIFS_ERRDOS, CIFSERR_nomem },	// CIFSTATUS_TOO_MANY_PAGING_FILES
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_FILE_INVALID
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_ALLOTTED_SPACE_EXCEEDED
 { CIFS_ERRDOS, CIFSERR_nomem },	// CIFSTATUS_INSUFFICIENT_RESOURCES
 { CIFS_ERRDOS, CIFSERR_badpath },	// CIFSTATUS_DFS_EXIT_PATH_FOUND
 { CIFS_ERRDOS, 23 },				// CIFSTATUS_DEVICE_DATA_ERROR
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_DEVICE_NOT_CONNECTED
 { CIFS_ERRDOS, 21 },				// CIFSTATUS_DEVICE_POWER_FAILURE
 { CIFS_ERRDOS, 487 },				// CIFSTATUS_FREE_VM_NOT_AT_BASE
 { CIFS_ERRDOS, 487 },				// CIFSTATUS_MEMORY_NOT_ALLOCATED
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_WORKING_SET_QUOTA
 { CIFS_ERRDOS, 19 },				// CIFSTATUS_MEDIA_WRITE_PROTECTED
 { CIFS_ERRDOS, 21 },				// CIFSTATUS_DEVICE_NOT_READY
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_INVALID_GROUP_ATTRIBUTES
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_BAD_IMPERSONATION_LEVEL
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_CANT_OPEN_ANONYMOUS
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_BAD_VALIDATION_CLASS
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_BAD_TOKEN_TYPE
 { CIFS_ERRDOS, 87 },				// CIFSTATUS_BAD_MASTER_BOOT_RECORD
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_INSTRUCTION_MISALIGNMENT
 { CIFS_ERRDOS, CIFSERR_pipebusy },	// CIFSTATUS_INSTANCE_NOT_AVAILABLE
 { CIFS_ERRDOS, CIFSERR_pipebusy },	// CIFSTATUS_PIPE_NOT_AVAILABLE
 { CIFS_ERRDOS, CIFSERR_badpipe },	// CIFSTATUS_INVALID_PIPE_STATE
 { CIFS_ERRDOS, CIFSERR_pipebusy },	// CIFSTATUS_PIPE_BUSY
 { CIFS_ERRDOS, CIFSERR_badfunc },	// CIFSTATUS_ILLEGAL_FUNCTION
 { CIFS_ERRDOS, CIFSERR_notconnected},// CIFSTATUS_PIPE_DISCONNECTED
 { CIFS_ERRDOS, CIFSERR_pipeclosing},// CIFSTATUS_PIPE_CLOSING
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_PIPE_CONNECTED
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_PIPE_LISTENING
 { CIFS_ERRDOS, CIFSERR_badpipe },	// CIFSTATUS_INVALID_READ_MODE
 { CIFS_ERRDOS, 121 },				// CIFSTATUS_IO_TIMEOUT
 { CIFS_ERRDOS, 38 },				// CIFSTATUS_FILE_FORCED_CLOSED
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_PROFILING_NOT_STARTED
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_PROFILING_NOT_STOPPED
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_COULD_NOT_INTERPRET
 { CIFS_ERRDOS, CIFSERR_noaccess },	// CIFSTATUS_FILE_IS_A_DIRECTORY
 { CIFS_ERRDOS, CIFSERR_unsup },	// CIFSTATUS_NOT_SUPPORTED
 { CIFS_ERRDOS, 51 },				// CIFSTATUS_REMOTE_NOT_LISTENING
 { CIFS_ERRDOS, 52 },				// CIFSTATUS_DUPLICATE_NAME
 { CIFS_ERRDOS, 53 },				// CIFSTATUS_BAD_NETWORK_PATH
 { CIFS_ERRDOS, 54 },				// CIFSTATUS_NETWORK_BUSY
 { CIFS_ERRDOS, 55 },				// CIFSTATUS_DEVICE_DOES_NOT_EXIST
 { CIFS_ERRDOS, 56 },				// CIFSTATUS_TOO_MANY_COMMANDS
 { CIFS_ERRDOS, 57 },				// CIFSTATUS_ADAPTER_HARDWARE_ERROR
 { CIFS_ERRDOS, 58 },				// CIFSTATUS_INVALID_NETWORK_RESPONSE
 { CIFS_ERRDOS, 59 },				// CIFSTATUS_UNEXPECTED_NETWORK_ERROR
 { CIFS_ERRDOS, 60 },				// CIFSTATUS_BAD_REMOTE_ADAPTER
 { CIFS_ERRDOS, 61 },				// CIFSTATUS_PRINT_QUEUE_FULL
 { CIFS_ERRDOS, 62 },				// CIFSTATUS_NO_SPOOL_SPACE
 { CIFS_ERRDOS, 63 },				// CIFSTATUS_PRINT_CANCELLED
 { CIFS_ERRDOS, 64 },				// CIFSTATUS_NETWORK_NAME_DELETED
 { CIFS_ERRDOS, 65 },				// CIFSTATUS_NETWORK_ACCESS_DENIED
 { CIFS_ERRDOS, 66 },				// CIFSTATUS_BAD_DEVICE_TYPE
 { CIFS_ERRDOS, CIFSERR_nosuchshare},// CIFSTATUS_BAD_NETWORK_NAME
 { CIFS_ERRDOS, 68 },				// CIFSTATUS_TOO_MANY_NAMES
 { CIFS_ERRDOS, 69 },				// CIFSTATUS_TOO_MANY_SESSIONS
 { CIFS_ERRDOS, 70 },				// CIFSTATUS_SHARING_PAUSED
 { CIFS_ERRDOS, 71 },				// CIFSTATUS_REQUEST_NOT_ACCEPTED
 { CIFS_ERRDOS, 72 },				// CIFSTATUS_REDIRECTOR_PAUSED
 { CIFS_ERRDOS, 88 },				// CIFSTATUS_NET_WRITE_FAULT
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_PROFILING_AT_LIMIT
 { CIFS_ERRDOS, CIFSERR_diffdevice },// CIFSTATUS_NOT_SAME_DEVICE
 { CIFS_ERRDOS, CIFSERR_noaccess },	// CIFSTATUS_FILE_RENAMED
 { CIFS_ERRDOS, 240 },				// CIFSTATUS_VIRTUAL_CIRCUIT_CLOSED
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_NO_SECURITY_ON_OBJECT
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_CANT_WAIT
 { CIFS_ERRDOS, CIFSERR_pipeclosing},// CIFSTATUS_PIPE_EMPTY
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_CANT_ACCESS_DOMAIN_INFO
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_CANT_TERMINATE_SELF
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_INVALID_SERVER_STATE
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_INVALID_DOMAIN_STATE
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_INVALID_DOMAIN_ROLE
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_NO_SUCH_DOMAIN
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_DOMAIN_EXISTS
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_DOMAIN_LIMIT_EXCEEDED
 { CIFS_ERRDOS, 300 },				// CIFSTATUS_OPLOCK_NOT_GRANTED
 { CIFS_ERRDOS, 301 },				// CIFSTATUS_INVALID_OPLOCK_PROTOCOL
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_INTERNAL_DB_CORRUPTION
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_INTERNAL_ERROR
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_GENERIC_NOT_MAPPED
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_BAD_DESCRIPTOR_FORMAT
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_INVALID_USER_BUFFER
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_UNEXPECTED_IO_ERROR
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_UNEXPECTED_MM_CREATE_ERR
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_UNEXPECTED_MM_MAP_ERROR
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_UNEXPECTED_MM_EXTEND_ERR
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_NOT_LOGON_PROCESS
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_LOGON_SESSION_EXISTS
 { CIFS_ERRDOS, 87 },				// CIFSTATUS_INVALID_PARAMETER_1
 { CIFS_ERRDOS, 87 },				// CIFSTATUS_INVALID_PARAMETER_2
 { CIFS_ERRDOS, 87 },				// CIFSTATUS_INVALID_PARAMETER_3
 { CIFS_ERRDOS, 87 },				// CIFSTATUS_INVALID_PARAMETER_4
 { CIFS_ERRDOS, 87 },				// CIFSTATUS_INVALID_PARAMETER_5
 { CIFS_ERRDOS, 87 },				// CIFSTATUS_INVALID_PARAMETER_6
 { CIFS_ERRDOS, 87 },				// CIFSTATUS_INVALID_PARAMETER_7
 { CIFS_ERRDOS, 87 },				// CIFSTATUS_INVALID_PARAMETER_8
 { CIFS_ERRDOS, 87 },				// CIFSTATUS_INVALID_PARAMETER_9
 { CIFS_ERRDOS, 87 },				// CIFSTATUS_INVALID_PARAMETER_10
 { CIFS_ERRDOS, 87 },				// CIFSTATUS_INVALID_PARAMETER_11
 { CIFS_ERRDOS, 87 },				// CIFSTATUS_INVALID_PARAMETER_12
 { CIFS_ERRDOS, CIFSERR_badpath },	// CIFSTATUS_REDIRECTOR_NOT_STARTED
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_REDIRECTOR_STARTED
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_STACK_OVERFLOW
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_NO_SUCH_PACKAGE
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_BAD_FUNCTION_TABLE
 { CIFS_ERRDOS, 203 },
 { CIFS_ERRDOS, 145 },				// CIFSTATUS_DIRECTORY_NOT_EMPTY
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_FILE_CORRUPT_ERROR
 { CIFS_ERRDOS, 267 },				// CIFSTATUS_NOT_A_DIRECTORY
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_BAD_LOGON_SESSION_STATE
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_LOGON_SESSION_COLLISION
 { CIFS_ERRDOS, 206 },				// CIFSTATUS_NAME_TOO_LONG
 { CIFS_ERRDOS, 2401 },				// CIFSTATUS_FILES_OPEN
 { CIFS_ERRDOS, 2404 },				// CIFSTATUS_CONNECTION_IN_USE
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_MESSAGE_NOT_FOUND
 { CIFS_ERRDOS, CIFSERR_noaccess },	// CIFSTATUS_PROCESS_IS_TERMINATING
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_INVALID_LOGON_TYPE
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_NO_GUID_TRANSLATION
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_CANNOT_IMPERSONATE
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_IMAGE_ALREADY_LOADED
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_ABIOS_NOT_PRESENT
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_ABIOS_LID_NOT_EXIST
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_ABIOS_LID_ALREADY_OWNED
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_ABIOS_NOT_LID_OWNER
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_ABIOS_INVALID_COMMAND
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_ABIOS_INVALID_LID
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_ABIOS_SELECTOR_NOT_AVAILABLE
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_ABIOS_INVALID_SELECTOR
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_NO_LDT
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_INVALID_LDT_SIZE
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_INVALID_LDT_OFFSET
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_INVALID_LDT_DESCRIPTOR
 { CIFS_ERRDOS, 193 },				// CIFSTATUS_INVALID_IMAGE_NE_FORMAT
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_RXACT_INVALID_STATE
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_RXACT_COMMIT_FAILURE
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_MAPPED_FILE_SIZE_ZERO
 { CIFS_ERRDOS, CIFSERR_nofids },	// CIFSTATUS_TOO_MANY_OPENED_FILES
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_CANCELLED
 { CIFS_ERRDOS, CIFSERR_noaccess },	// CIFSTATUS_CANNOT_DELETE
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_INVALID_COMPUTER_NAME
 { CIFS_ERRDOS, CIFSERR_noaccess },	// CIFSTATUS_FILE_DELETED
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_SPECIAL_ACCOUNT
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_SPECIAL_GROUP
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_SPECIAL_USER
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_MEMBERS_PRIMARY_GROUP
 { CIFS_ERRDOS, CIFSERR_badfid },	// CIFSTATUS_FILE_CLOSED
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_TOO_MANY_THREADS
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_THREAD_NOT_IN_PROCESS
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_TOKEN_ALREADY_IN_USE
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_PAGEFILE_QUOTA_EXCEEDED
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_COMMITMENT_LIMIT
 { CIFS_ERRDOS, 193 },				// CIFSTATUS_INVALID_IMAGE_LE_FORMAT
 { CIFS_ERRDOS, 193 },				// CIFSTATUS_INVALID_IMAGE_NOT_MZ
 { CIFS_ERRDOS, 193 },				// CIFSTATUS_INVALID_IMAGE_PROTECT
 { CIFS_ERRDOS, 193 },				// CIFSTATUS_INVALID_IMAGE_WIN_16
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_LOGON_SERVER_CONFLICT
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_TIME_DIFFERENCE_AT_DC
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_SYNCHRONIZATION_REQUIRED
 { CIFS_ERRDOS, 126 },				// CIFSTATUS_DLL_NOT_FOUND
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_OPEN_FAILED
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_IO_PRIVILEGE_FAILED
 { CIFS_ERRDOS, 182 },				// CIFSTATUS_ORDINAL_NOT_FOUND
 { CIFS_ERRDOS, 127 },				// CIFSTATUS_ENTRYPOINT_NOT_FOUND
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_CONTROL_C_EXIT
 { CIFS_ERRDOS, 64 },				// CIFSTATUS_LOCAL_DISCONNECT
 { CIFS_ERRDOS, 64 },				// CIFSTATUS_REMOTE_DISCONNECT
 { CIFS_ERRDOS, 51 },				// CIFSTATUS_REMOTE_RESOURCES
 { CIFS_ERRDOS, 59 },				// CIFSTATUS_LINK_FAILED
 { CIFS_ERRDOS, 59 },				// CIFSTATUS_LINK_TIMEOUT
 { CIFS_ERRDOS, 59 },				// CIFSTATUS_INVALID_CONNECTION
 { CIFS_ERRDOS, 59 },				// CIFSTATUS_INVALID_ADDRESS
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_DLL_INIT_FAILED
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_MISSING_SYSTEMFILE
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_UNHANDLED_EXCEPTION
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_APP_INIT_FAILURE
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_PAGEFILE_CREATE_FAILED
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_NO_PAGEFILE
 { CIFS_ERRDOS, 124 },				// CIFSTATUS_INVALID_LEVEL
 { CIFS_ERRDOS, 86 },				// CIFSTATUS_WRONG_PASSWORD_CORE
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_ILLEGAL_FLOAT_CONTEXT
 { CIFS_ERRDOS, 109 },				// CIFSTATUS_PIPE_BROKEN
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_REGISTRY_CORRUPT
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_REGISTRY_IO_FAILED
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_NO_EVENT_PAIR
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_UNRECOGNIZED_VOLUME
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_SERIAL_NO_DEVICE_INITED
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_NO_SUCH_ALIAS
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_MEMBER_NOT_IN_ALIAS
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_MEMBER_IN_ALIAS
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_ALIAS_EXISTS
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_LOGON_NOT_GRANTED
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_TOO_MANY_SECRETS
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_SECRET_TOO_LONG
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_INTERNAL_DB_ERROR
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_FULLSCREEN_MODE
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_TOO_MANY_CONTEXT_IDS
 { CIFS_ERRDOS, CIFSERR_noaccess },	// CIFSTATUS_LOGON_TYPE_NOT_GRANTED
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_NOT_REGISTRY_FILE
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_NT_CROSS_ENCRYPTION_REQUIRED
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_DOMAIN_CTRLR_CONFIG_ERROR
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_FT_MISSING_MEMBER
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_ILL_FORMED_SERVICE_ENTRY
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_ILLEGAL_CHARACTER
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_UNMAPPABLE_CHARACTER
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_UNDEFINED_CHARACTER
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_FLOPPY_VOLUME
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_FLOPPY_ID_MARK_NOT_FOUND
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_FLOPPY_WRONG_CYLINDER
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_FLOPPY_UNKNOWN_ERROR
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_FLOPPY_BAD_REGISTERS
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_DISK_RECALIBRATE_FAILED
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_DISK_OPERATION_FAILED
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_DISK_RESET_FAILED
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_SHARED_IRQ_BUSY
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_FT_ORPHANING
 { CIFS_ERRHRD, CIFSERR_general },
 { CIFS_ERRHRD, CIFSERR_general },
 { CIFS_ERRHRD, CIFSERR_general },
 { CIFS_ERRHRD, CIFSERR_general },
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_PARTITION_FAILURE
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_INVALID_BLOCK_LENGTH
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_DEVICE_NOT_PARTITIONED
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_UNABLE_TO_LOCK_MEDIA
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_UNABLE_TO_UNLOAD_MEDIA
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_EOM_OVERFLOW
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_NO_MEDIA
 { CIFS_ERRHRD, CIFSERR_general },
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_NO_SUCH_MEMBER
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_INVALID_MEMBER
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_KEY_DELETED
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_NO_LOG_SPACE
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_TOO_MANY_SIDS
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_LM_CROSS_ENCRYPTION_REQUIRED
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_KEY_HAS_CHILDREN
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_CHILD_MUST_BE_VOLATILE
 { CIFS_ERRDOS, 87 },				// CIFSTATUS_DEVICE_CONFIGURATION_ERROR
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_DRIVER_INTERNAL_ERROR
 { CIFS_ERRDOS, 22 },				// CIFSTATUS_INVALID_DEVICE_STATE
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_IO_DEVICE_ERROR
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_DEVICE_PROTOCOL_ERROR
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_BACKUP_CONTROLLER
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_LOG_FILE_FULL
 { CIFS_ERRDOS, 19 },				// CIFSTATUS_TOO_LATE
 { CIFS_ERRDOS, CIFSERR_noaccess },	// CIFSTATUS_NO_TRUST_LSA_SECRET
 { CIFS_ERRDOS, CIFSERR_noaccess },	// CIFSTATUS_NO_TRUST_SAM_ACCOUNT
 { CIFS_ERRDOS, CIFSERR_noaccess },	// CIFSTATUS_TRUSTED_DOMAIN_FAILURE
 { CIFS_ERRDOS, CIFSERR_noaccess },	// CIFSTATUS_TRUSTED_RELATIONSHIP_FAILURE
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_EVENTLOG_FILE_CORRUPT
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_EVENTLOG_CANT_START
 { CIFS_ERRDOS, CIFSERR_noaccess },	// CIFSTATUS_TRUST_FAILURE
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_MUTANT_LIMIT_EXCEEDED
 { CIFS_ERRDOS, CIFSERR_invgroup },	// CIFSTATUS_NETLOGON_NOT_STARTED
 { CIFS_ERRSRV, 2239 },				// CIFSTATUS_ACCOUNT_EXPIRED
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_POSSIBLE_DEADLOCK
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_NETWORK_CREDENTIAL_CONFLICT
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_REMOTE_SESSION_LIMIT
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_EVENTLOG_FILE_CHANGED
 { CIFS_ERRDOS, CIFSERR_noaccess },	// CIFSTATUS_NOLOGON_INTERDOMAIN_TRUST_ACC
 { CIFS_ERRDOS, CIFSERR_noaccess },	// CIFSTATUS_NOLOGON_WORKSTATION_TRUST_ACC
 { CIFS_ERRDOS, CIFSERR_noaccess },	// CIFSTATUS_NOLOGON_SERVER_TRUST_ACCOUNT
 { CIFS_ERRDOS, CIFSERR_noaccess },	// CIFSTATUS_DOMAIN_TRUST_INCONSISTENT
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_FS_DRIVER_REQUIRED
 { CIFS_ERRHRD, CIFSERR_general },
 { CIFS_ERRHRD, CIFSERR_general },
 { CIFS_ERRHRD, CIFSERR_general },
 { CIFS_ERRHRD, CIFSERR_general },
 { CIFS_ERRHRD, CIFSERR_general },
 { CIFS_ERRHRD, CIFSERR_general },
 { CIFS_ERRHRD, CIFSERR_general },
 { CIFS_ERRHRD, CIFSERR_general },
 { CIFS_ERRHRD, CIFSERR_general },
 { CIFS_ERRHRD, CIFSERR_general },
 { CIFS_ERRHRD, CIFSERR_general },
 { CIFS_ERRHRD, CIFSERR_general },
 { CIFS_ERRHRD, CIFSERR_general },
 { CIFS_ERRHRD, CIFSERR_general },
 { CIFS_ERRHRD, CIFSERR_general },
 { CIFS_ERRHRD, CIFSERR_general },
 { CIFS_ERRHRD, CIFSERR_general },
 { CIFS_ERRHRD, CIFSERR_general },
 { CIFS_ERRHRD, CIFSERR_general },
 { CIFS_ERRHRD, CIFSERR_general },
 { CIFS_ERRHRD, CIFSERR_general },
 { CIFS_ERRHRD, CIFSERR_general },
 { CIFS_ERRHRD, CIFSERR_general },
 { CIFS_ERRHRD, CIFSERR_general },
 { CIFS_ERRHRD, CIFSERR_general },
 { CIFS_ERRHRD, CIFSERR_general },
 { CIFS_ERRHRD, CIFSERR_general },
 { CIFS_ERRHRD, CIFSERR_general },
 { CIFS_ERRHRD, CIFSERR_general },
 { CIFS_ERRHRD, CIFSERR_general },
 { CIFS_ERRHRD, CIFSERR_general },
 { CIFS_ERRHRD, CIFSERR_general },
 { CIFS_ERRHRD, CIFSERR_general },
 { CIFS_ERRHRD, CIFSERR_general },
 { CIFS_ERRHRD, CIFSERR_general },
 { CIFS_ERRHRD, CIFSERR_general },
 { CIFS_ERRHRD, CIFSERR_general },
 { CIFS_ERRHRD, CIFSERR_general },
 { CIFS_ERRHRD, CIFSERR_general },
 { CIFS_ERRHRD, CIFSERR_general },
 { CIFS_ERRHRD, CIFSERR_general },
 { CIFS_ERRHRD, CIFSERR_general },
 { CIFS_ERRHRD, CIFSERR_general },
 { CIFS_ERRHRD, CIFSERR_general },
 { CIFS_ERRHRD, CIFSERR_general },
 { CIFS_ERRHRD, CIFSERR_general },
 { CIFS_ERRHRD, CIFSERR_general },
 { CIFS_ERRHRD, CIFSERR_general },
 { CIFS_ERRHRD, CIFSERR_general },
 { CIFS_ERRHRD, CIFSERR_general },
 { CIFS_ERRHRD, CIFSERR_general },
 { CIFS_ERRHRD, CIFSERR_general },
 { CIFS_ERRHRD, CIFSERR_general },
 { CIFS_ERRHRD, CIFSERR_general },
 { CIFS_ERRHRD, CIFSERR_general },
 { CIFS_ERRHRD, CIFSERR_general },
 { CIFS_ERRHRD, CIFSERR_general },
 { CIFS_ERRHRD, CIFSERR_general },
 { CIFS_ERRHRD, CIFSERR_general },
 { CIFS_ERRHRD, CIFSERR_general },
 { CIFS_ERRHRD, CIFSERR_general },
 { CIFS_ERRHRD, CIFSERR_general },
 { CIFS_ERRHRD, CIFSERR_general },
 { CIFS_ERRHRD, CIFSERR_general },
 { CIFS_ERRHRD, CIFSERR_general },
 { CIFS_ERRHRD, CIFSERR_general },
 { CIFS_ERRHRD, CIFSERR_general },
 { CIFS_ERRHRD, CIFSERR_general },
 { CIFS_ERRHRD, CIFSERR_general },
 { CIFS_ERRHRD, CIFSERR_general },
 { CIFS_ERRHRD, CIFSERR_general },
 { CIFS_ERRHRD, CIFSERR_general },
 { CIFS_ERRHRD, CIFSERR_general },
 { CIFS_ERRHRD, CIFSERR_general },
 { CIFS_ERRHRD, CIFSERR_general },
 { CIFS_ERRHRD, CIFSERR_general },
 { CIFS_ERRHRD, CIFSERR_general },
 { CIFS_ERRHRD, CIFSERR_general },
 { CIFS_ERRHRD, CIFSERR_general },
 { CIFS_ERRHRD, CIFSERR_general },
 { CIFS_ERRHRD, CIFSERR_general },
 { CIFS_ERRHRD, CIFSERR_general },
 { CIFS_ERRHRD, CIFSERR_general },
 { CIFS_ERRHRD, CIFSERR_general },
 { CIFS_ERRHRD, CIFSERR_general },
 { CIFS_ERRHRD, CIFSERR_general },
 { CIFS_ERRHRD, CIFSERR_general },
 { CIFS_ERRHRD, CIFSERR_general },
 { CIFS_ERRHRD, CIFSERR_general },
 { CIFS_ERRHRD, CIFSERR_general },
 { CIFS_ERRHRD, CIFSERR_general },
 { CIFS_ERRHRD, CIFSERR_general },
 { CIFS_ERRHRD, CIFSERR_general },
 { CIFS_ERRHRD, CIFSERR_general },
 { CIFS_ERRHRD, CIFSERR_general },
 { CIFS_ERRHRD, CIFSERR_general },
 { CIFS_ERRHRD, CIFSERR_general },
 { CIFS_ERRHRD, CIFSERR_general },
 { CIFS_ERRHRD, CIFSERR_general },
 { CIFS_ERRHRD, CIFSERR_general },
 { CIFS_ERRHRD, CIFSERR_general },
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_NO_USER_SESSION_KEY
 { CIFS_ERRDOS, 59 },				// CIFSTATUS_USER_SESSION_DELETED
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_RESOURCE_LANG_NOT_FOUND
 { CIFS_ERRDOS, CIFSERR_nomem },	// CIFSTATUS_INSUFF_SERVER_RESOURCES
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_INVALID_BUFFER_SIZE
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_INVALID_ADDRESS_COMPONENT
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_INVALID_ADDRESS_WILDCARD
 { CIFS_ERRDOS, 68 },				// CIFSTATUS_TOO_MANY_ADDRESSES
 { CIFS_ERRDOS, 52 },				// CIFSTATUS_ADDRESS_ALREADY_EXISTS
 { CIFS_ERRDOS, 64 },				// CIFSTATUS_ADDRESS_CLOSED
 { CIFS_ERRDOS, 64 },				// CIFSTATUS_CONNECTION_DISCONNECTED
 { CIFS_ERRDOS, 64 },				// CIFSTATUS_CONNECTION_RESET
 { CIFS_ERRDOS, 68 },				// CIFSTATUS_TOO_MANY_NODES
 { CIFS_ERRDOS, 59 },				// CIFSTATUS_TRANSACTION_ABORTED
 { CIFS_ERRDOS, 59 },				// CIFSTATUS_TRANSACTION_TIMED_OUT
 { CIFS_ERRDOS, 59 },				// CIFSTATUS_TRANSACTION_NO_RELEASE
 { CIFS_ERRDOS, 59 },				// CIFSTATUS_TRANSACTION_NO_MATCH
 { CIFS_ERRDOS, 59 },				// CIFSTATUS_TRANSACTION_RESPONDED
 { CIFS_ERRDOS, 59 },				// CIFSTATUS_TRANSACTION_INVALID_ID
 { CIFS_ERRDOS, 59 },				// CIFSTATUS_TRANSACTION_INVALID_TYPE
 { CIFS_ERRDOS, CIFSERR_unsup },		// CIFSTATUS_NOT_SERVER_SESSION
 { CIFS_ERRDOS, CIFSERR_unsup },		// CIFSTATUS_NOT_CLIENT_SESSION
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_CANNOT_LOAD_REGISTRY_FILE
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_DEBUG_ATTACH_FAILED
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_SYSTEM_PROCESS_TERMINATED
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_DATA_NOT_ACCEPTED
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_NO_BROWSER_SERVERS_FOUND
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_VDM_HARD_ERROR
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_DRIVER_CANCEL_TIMEOUT
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_REPLY_MESSAGE_MISMATCH
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_MAPPED_ALIGNMENT
 { CIFS_ERRDOS, 193 },				// CIFSTATUS_IMAGE_CHECKSUM_MISMATCH
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_LOST_WRITEBEHIND_DATA
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_CLIENT_SERVER_PARAMETERS_INVAL
 { CIFS_ERRSRV, 2242 },				// CIFSTATUS_PASSWORD_MUST_CHANGE
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_NOT_FOUND
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_NOT_TINY_STREAM
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_RECOVERY_FAILURE
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_STACK_OVERFLOW_READ
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_FAIL_CHECK
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_DUPLICATE_OBJECTID
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_OBJECTID_EXISTS
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_CONVERT_TO_LARGE
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_RETRY
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_FOUND_OUT_OF_SCOPE
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_ALLOCATE_BUCKET
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_PROPSET_NOT_FOUND
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_MARSHALL_OVERFLOW
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_INVALID_VARIANT
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_DOMAIN_CONTROLLER_NOT_FOUND
 { CIFS_ERRDOS, CIFSERR_noaccess },	// CIFSTATUS_ACCOUNT_LOCKED_OUT
 { CIFS_ERRDOS, CIFSERR_badfid },	// CIFSTATUS_HANDLE_NOT_CLOSABLE
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_CONNECTION_REFUSED
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_GRACEFUL_DISCONNECT
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_ADDRESS_ALREADY_ASSOCIATED
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_ADDRESS_NOT_ASSOCIATED
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_CONNECTION_INVALID
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_CONNECTION_ACTIVE
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_NETWORK_UNREACHABLE
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_HOST_UNREACHABLE
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_PROTOCOL_UNREACHABLE
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_PORT_UNREACHABLE
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_REQUEST_ABORTED
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_CONNECTION_ABORTED
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_BAD_COMPRESSION_BUFFER
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_USER_MAPPED_FILE
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_AUDIT_FAILED
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_TIMER_RESOLUTION_NOT_SET
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_CONNECTION_COUNT_LIMIT
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_LOGIN_TIME_RESTRICTION
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_LOGIN_WKSTA_RESTRICTION
 { CIFS_ERRDOS, 193 },				// CIFSTATUS_IMAGE_MP_UP_MISMATCH
 { CIFS_ERRHRD, CIFSERR_general },
 { CIFS_ERRHRD, CIFSERR_general },
 { CIFS_ERRHRD, CIFSERR_general },
 { CIFS_ERRHRD, CIFSERR_general },
 { CIFS_ERRHRD, CIFSERR_general },
 { CIFS_ERRHRD, CIFSERR_general },
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_INSUFFICIENT_LOGON_INFO
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_BAD_DLL_ENTRYPOINT
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_BAD_SERVICE_ENTRYPOINT
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_LPC_REPLY_LOST
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_IP_ADDRESS_CONFLICT1
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_IP_ADDRESS_CONFLICT2
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_REGISTRY_QUOTA_LIMIT
 { CIFS_ERRSRV, CIFSERR_badtype },	// CIFSTATUS_PATH_NOT_COVERED
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_NO_CALLBACK_ACTIVE
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_LICENSE_QUOTA_EXCEEDED
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_PWD_TOO_SHORT
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_PWD_TOO_RECENT
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_PWD_HISTORY_CONFLICT
 { CIFS_ERRHRD, CIFSERR_general },
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_PLUGPLAY_NO_DEVICE
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_UNSUPPORTED_COMPRESSION
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_INVALID_HW_PROFILE
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_INVALID_PLUGPLAY_DEVICE_PATH
 { CIFS_ERRDOS, 182 },				// CIFSTATUS_DRIVER_ORDINAL_NOT_FOUND
 { CIFS_ERRDOS, 127 },				// CIFSTATUS_DRIVER_ENTRYPOINT_NOT_FOUND
 { CIFS_ERRDOS, 288 },				// CIFSTATUS_RESOURCE_NOT_OWNED
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_TOO_MANY_LINKS
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_QUOTA_LIST_INCONSISTENT
 { CIFS_ERRHRD, CIFSERR_general },	// CIFSTATUS_FILE_IS_OFFLINE
 { CIFS_ERRHRD, CIFSERR_general },
 { CIFS_ERRHRD, CIFSERR_general },
 { CIFS_ERRHRD, CIFSERR_general },
 { CIFS_ERRHRD, CIFSERR_general },
 { CIFS_ERRHRD, CIFSERR_general },
 { CIFS_ERRHRD, CIFSERR_general },
 { CIFS_ERRDOS, 21 },
 { CIFS_ERRHRD, CIFSERR_general },
 { CIFS_ERRHRD, CIFSERR_general },
 { CIFS_ERRHRD, CIFSERR_general },
 { CIFS_ERRHRD, CIFSERR_general },
 { CIFS_ERRHRD, CIFSERR_general },
 { CIFS_ERRHRD, CIFSERR_general },
 { CIFS_ERRHRD, CIFSERR_general },
 { CIFS_ERRHRD, CIFSERR_general },
 { CIFS_ERRHRD, CIFSERR_general },
 { CIFS_ERRHRD, CIFSERR_general },
 { CIFS_ERRHRD, CIFSERR_general },
 { CIFS_ERRHRD, CIFSERR_general },
 { CIFS_ERRHRD, CIFSERR_general },
 { CIFS_ERRHRD, CIFSERR_general },
 { CIFS_ERRHRD, CIFSERR_general },
 { CIFS_ERRHRD, CIFSERR_general },
 { CIFS_ERRHRD, CIFSERR_general },
 { CIFS_ERRHRD, CIFSERR_general },
 { CIFS_ERRDOS, 161 },
 { CIFS_ERRHRD, CIFSERR_general },
 { CIFS_ERRHRD, CIFSERR_general },
 { CIFS_ERRHRD, CIFSERR_general },
 { CIFS_ERRHRD, CIFSERR_general },
 { CIFS_ERRHRD, CIFSERR_general },
 { CIFS_ERRHRD, CIFSERR_general },
 { CIFS_ERRHRD, CIFSERR_general },
 { CIFS_ERRHRD, CIFSERR_general },
 { CIFS_ERRDOS, CIFSERR_noaccess },
 { CIFS_ERRDOS, CIFSERR_noaccess },
 { CIFS_ERRHRD, CIFSERR_general },
 { CIFS_ERRDOS, CIFSERR_noaccess },
 { CIFS_ERRDOS, CIFSERR_noaccess },
 { CIFS_ERRDOS, CIFSERR_noaccess },
 { CIFS_ERRDOS, CIFSERR_noaccess },
 { CIFS_ERRHRD, CIFSERR_general },
 { CIFS_ERRHRD, CIFSERR_general },
 { CIFS_ERRHRD, CIFSERR_general },
 { CIFS_ERRHRD, CIFSERR_general },
 { CIFS_ERRHRD, CIFSERR_general },
 { CIFS_ERRHRD, CIFSERR_general },
 { CIFS_ERRHRD, CIFSERR_general },
 { CIFS_ERRHRD, CIFSERR_general },
 { CIFS_ERRHRD, CIFSERR_general },
 { CIFS_ERRHRD, CIFSERR_general },
 { CIFS_ERRHRD, CIFSERR_general },
 { CIFS_ERRDOS, CIFSERR_badfunc },
};




/*-----------------------------------------------------------------
 * Name			: cifs_ntStatusToDosErrMapping 
 *
 * Description	: map NT status to Dos error.
 *
 * Created by	: Laurent Brard
 *
 * Date Created	: 03/21/2002
 *
 *-----------------------------------------------------------------
 */
void
cifs_ntStatusToDosErrMapping(uint32 ntstatus, uchar8 *classp, ushort16 *errp)
{
	uint32 statusClass = ntstatus & NT_SEVERITY_ERR;
	
printf("###sizTab=%x, sizItem=%x, CIFSTATUS_ERRMAX = %x\n",
		sizeof(cifstatus2Dos), sizeof(cifs_dosError), CIFSTATUS_ERRMAX);
	switch(statusClass) {
	
		case NT_SEVERITY_ERR:
			ntstatus &= ~NT_SEVERITY_ERR;
			if (ntstatus <= CIFSTATUS_ERRMAX)
			{
				*classp = cifstatus2Dos[ntstatus].class;
				*errp	= cifstatus2Dos[ntstatus].code;
			} else {
				*classp = CIFS_ERRHRD; *errp = CIFSERR_general;
			}
			break;
		
		case NT_SEVERITY_WARN:
			switch (ntstatus) {
				case CIFSTATUS_BUFFER_OVERFLOW:
				case CIFSTATUS_MORE_ENTRIES:
					*classp = CIFS_ERRDOS; *errp = 111; break;
				default:
					*classp = CIFS_ERRHRD; *errp = CIFSERR_general;
			}
			break;
	}
}


