/* vi:ts=4
 *-----------------------------------------------------------------
 *
 * Copyright (C) 2001, Agile Storage Inc.
 * 
 * RCS:  $Id: cifs-err.h,v 1.1.1.1 2002/04/23 17:41:16 laurentb Exp $
 *
 * Name			: cifs-err.h
 *
 * Description	: Contains CIFS errors definitions.
 *
 * Created by	: Laurent Brard
 *
 * Date Created	: 11/13/2001
 *
 *-----------------------------------------------------------------
 */
#ifndef _CIFS_ERR_H
#define _CIFS_ERR_H

//***************************** Dos Errors ***********************************	

//error classes
#define CIFS_SUCCESS		0
#define CIFS_ERRDOS			1  // DOS error
#define CIFS_ERRSRV			2  // CIFS server error
#define CIFS_ERRHRD			3  // Hardware error
#define CIFS_ERRCMD			255// Format error

// CIFS_SUCCESS error class - error codes */
#define CIFSERR_success 0 /* No error */

// CIFSERR_DOS error class - error codes
#define CIFSERR_badfunc		1  // Invalid function (or system call)
#define CIFSERR_badfile		2  // File not found (pathname error)
#define CIFSERR_badpath		3  // Directory not found
#define CIFSERR_nofids		4  // Too many open files
#define CIFSERR_noaccess	5  // Access denied
#define CIFSERR_badfid		6  // Invalid fid
#define CIFSERR_badmcb		7  // Memory control blocks destroyed
#define CIFSERR_nomem		8  // Out of memory
#define CIFSERR_badmem		9  // Invalid memory block address
#define CIFSERR_badenv		10 // Invalid environment
#define CIFSERR_badaccess	12 // Invalid open mode
#define CIFSERR_baddata		13 // Invalid data (from ioctl call)
#define CIFSERR_reserved	14 // Reserved
#define CIFSERR_baddrive	15 // Invalid drive
#define CIFSERR_remcd		16 // Attempt to delete current directory
#define CIFSERR_diffdevice	17 // rename/move across different filesystems
#define CIFSERR_nofiles		18 // no more files found in file search
#define CIFSERR_badshare	32 // Share mode on file conflict with open mode
#define CIFSERR_lock		33 // Lock request conflicts with existing lock
#define CIFSERR_unsup		50 // Request unsupported, returned by Win 95
#define CIFSERR_nonetaccess	65 // Network access denied
#define CIFSERR_noipc		66 // don't support ipc
#define CIFSERR_nosuchshare 67 // invalid share name
#define CIFSERR_filexists	80 // File already exists
#define CIFSERR_invalidpara 87
#define CIFSERR_cannotopen 	110 // Cannot open the file specified
#define CIFSERR_insufficientbuffer 122
#define CIFSERR_invalidname 123 // Invalid name
#define CIFSERR_unknownlvl	124
#define CIFSERR_notlocked 	158 // region is not locked by this locking context
#define CIFSERR_rename		183
#define CIFSERR_badpipe		230 // Named pipe invalid
#define CIFSERR_pipebusy	231 // All instances of pipe are busy
#define CIFSERR_pipeclosing 232 // named pipe close in progress
#define CIFSERR_notconnected 233// No process on other end of named pipe
#define CIFSERR_moredata	234 // More data to be returned
#define CIFSERR_nomoreitems 259
#define CIFSERR_baddirectory 267// Invalid directory name in a path
#define CIFSERR_easnotsupported 282 // Extended attributes
#define CIFSERR_buftoosmall	2123
#define CIFSERR_srvnotstarted	2114 // Server not started
#define CIFSERR_badtransconf	2141 // Bad transaction config
#define CIFSERR_unknownipc	2142
#define CIFSERR_invgroup	2455

// CIFSERR_SRV error class - error codes
#define CIFSERR_error		1  // Non specific error code
#define CIFSERR_badpw		2  // Bad password
#define CIFSERR_badtype		3  // reserved
#define CIFSERR_access		4  // No permissions to do the requested operation
#define CIFSERR_invtid		5  // tid invalid
#define CIFSERR_invnetname	6  // Invalid servername
#define CIFSERR_invdevice	7  // Invalid device
#define CIFSERR_smbcmd		64 // Unrecognised command
#define CIFSERR_srverror	65 // smb server internal error
#define CIFSERR_filespecs	67 // fid and pathname invalid combination
#define CIFSERR_badlink		68 // reserved
#define CIFSERR_badpermits	69 // Access specified for a file is not valid
#define CIFSERR_badpid		70 // reserved
#define CIFSERR_setattrmode 71 // attribute mode invalid
#define CIFSERR_timeout		88 // operation timed out
#define CIFSERR_noresource  89 // No resources currently available for request
#define CIFSERR_toomanyuids 90 // too many userids
#define CIFSERR_baduid		91 // bad userid
#define CIFSERR_useMPX		250// temporarily unable to use raw mode, use MPX
#define CIFSERR_useSTD		251// temporarily unable to use raw, use standard
#define CIFSERR_contMPX		252// resume MPX mode
#define CIFSERR_nosupport	0xFFFF

// CIFSERR_HRD error class - error codes
#define CIFSERR_nowrite		19 // read only media
#define CIFSERR_badunit		20 // Unknown device
#define CIFSERR_notready	21 // Drive not ready
#define CIFSERR_badcmd		22 // Unknown command
#define CIFSERR_data		23 // Data (CRC) error
#define CIFSERR_badreq		24 // Bad request structure length
#define CIFSERR_seek		25
#define CIFSERR_badmedia	26
#define CIFSERR_badsector	27
#define CIFSERR_nopaper		28
#define CIFSERR_write		29 // write fault
#define CIFSERR_read		30 // read fault
#define CIFSERR_general		31 // General hardware failure
#define CIFSERR_wrongdisk	34
#define CIFSERR_FCBunavail	35
#define CIFSERR_sharebufexc	36 // share buffer exceeded
#define CIFSERR_diskfull	39


//***************************** NT Status ***********************************	

#define NT_SEVERITY_SUCC	0x00000000
#define NT_SEVERITY_INFO	0x40000000
#define NT_SEVERITY_WARN	0x80000000
#define NT_SEVERITY_ERR		0xC0000000


//NT status codes. 
#define CIFSTATUS_BUFFER_OVERFLOW	(NT_SEVERITY_WARN | 0x0005)
#define CIFSTATUS_MORE_ENTRIES      (NT_SEVERITY_WARN | 0x0105)
#define CIFSTATUS_NOTIFY_ENUM_DIR	(NT_SEVERITY_WARN | 0x010C)

//NT error codes (from Netmon)
#define CIFSTATUS_OK							(NT_SEVERITY_SUCC | 0x0000)
#define CIFSTATUS_UNSUCCESSFUL					(NT_SEVERITY_ERR | 0x0001)
#define CIFSTATUS_NOT_IMPLEMENTED				(NT_SEVERITY_ERR | 0x0002)
#define CIFSTATUS_INVALID_INFO_CLASS			(NT_SEVERITY_ERR | 0x0003)
#define CIFSTATUS_INFO_LENGTH_MISMATCH			(NT_SEVERITY_ERR | 0x0004)
#define CIFSTATUS_ACCESS_VIOLATION				(NT_SEVERITY_ERR | 0x0005)
#define CIFSTATUS_IN_PAGE_ERROR					(NT_SEVERITY_ERR | 0x0006)
#define CIFSTATUS_PAGEFILE_QUOTA				(NT_SEVERITY_ERR | 0x0007)
#define CIFSTATUS_INVALID_HANDLE				(NT_SEVERITY_ERR | 0x0008)
#define CIFSTATUS_BAD_INITIAL_STACK				(NT_SEVERITY_ERR | 0x0009)
#define CIFSTATUS_BAD_INITIAL_PC				(NT_SEVERITY_ERR | 0x000a)
#define CIFSTATUS_INVALID_CID					(NT_SEVERITY_ERR | 0x000b)
#define CIFSTATUS_TIMER_NOT_CANCELED			(NT_SEVERITY_ERR | 0x000c)
#define CIFSTATUS_INVALID_PARAMETER				(NT_SEVERITY_ERR | 0x000d)
#define CIFSTATUS_NO_SUCH_DEVICE				(NT_SEVERITY_ERR | 0x000e)
#define CIFSTATUS_NO_SUCH_FILE					(NT_SEVERITY_ERR | 0x000f)
#define CIFSTATUS_INVALID_DEVICE_REQUEST		(NT_SEVERITY_ERR | 0x0010)
#define CIFSTATUS_END_OF_FILE					(NT_SEVERITY_ERR | 0x0011)
#define CIFSTATUS_WRONG_VOLUME					(NT_SEVERITY_ERR | 0x0012)
#define CIFSTATUS_NO_MEDIA_IN_DEVICE			(NT_SEVERITY_ERR | 0x0013)
#define CIFSTATUS_UNRECOGNIZED_MEDIA			(NT_SEVERITY_ERR | 0x0014)
#define CIFSTATUS_NONEXISTENT_SECTOR			(NT_SEVERITY_ERR | 0x0015)
#define CIFSTATUS_MORE_PROCESSING_REQUIRED		(NT_SEVERITY_ERR | 0x0016)
#define CIFSTATUS_NO_MEMORY						(NT_SEVERITY_ERR | 0x0017)
#define CIFSTATUS_CONFLICTING_ADDRESSES			(NT_SEVERITY_ERR | 0x0018)
#define CIFSTATUS_NOT_MAPPED_VIEW				(NT_SEVERITY_ERR | 0x0019)
#define CIFSTATUS_UNABLE_TO_FREE_VM				(NT_SEVERITY_WARN | 0x001a)
#define CIFSTATUS_UNABLE_TO_DELETE_SECTION		(NT_SEVERITY_ERR | 0x001b)
#define CIFSTATUS_INVALID_SYSTEM_SERVICE		(NT_SEVERITY_ERR | 0x001c)
#define CIFSTATUS_ILLEGAL_INSTRUCTION			(NT_SEVERITY_ERR | 0x001d)
#define CIFSTATUS_INVALID_LOCK_SEQUENCE			(NT_SEVERITY_ERR | 0x001e)
#define CIFSTATUS_INVALID_VIEW_SIZE				(NT_SEVERITY_ERR | 0x001f)
#define CIFSTATUS_INVALID_FILE_FOR_SECTION		(NT_SEVERITY_ERR | 0x0020)
#define CIFSTATUS_ALREADY_COMMITTED				(NT_SEVERITY_ERR | 0x0021)
#define CIFSTATUS_ACCESS_DENIED					(NT_SEVERITY_ERR | 0x0022)
#define CIFSTATUS_BUFFER_TOO_SMALL				(NT_SEVERITY_ERR | 0x0023)
#define CIFSTATUS_OBJECT_TYPE_MISMATCH			(NT_SEVERITY_ERR | 0x0024)
#define CIFSTATUS_NONCONTINUABLE_EXCEPTION		(NT_SEVERITY_ERR | 0x0025)
#define CIFSTATUS_INVALID_DISPOSITION			(NT_SEVERITY_ERR | 0x0026)
#define CIFSTATUS_UNWIND						(NT_SEVERITY_ERR | 0x0027)
#define CIFSTATUS_BAD_STACK						(NT_SEVERITY_ERR | 0x0028)
#define CIFSTATUS_INVALID_UNWIND_TARGET			(NT_SEVERITY_ERR | 0x0029)
#define CIFSTATUS_NOT_LOCKED					(NT_SEVERITY_ERR | 0x002a)
#define CIFSTATUS_PARITY_ERROR					(NT_SEVERITY_ERR | 0x002b)
#define CIFSTATUS_UNABLE_TO_DECOMMIT_VM			(NT_SEVERITY_ERR | 0x002c)
#define CIFSTATUS_NOT_COMMITTED					(NT_SEVERITY_ERR | 0x002d)
#define CIFSTATUS_INVALID_PORT_ATTRIBUTES		(NT_SEVERITY_ERR | 0x002e)
#define CIFSTATUS_PORT_MESSAGE_TOO_LONG			(NT_SEVERITY_ERR | 0x002f)
#define CIFSTATUS_INVALID_PARAMETER_MIX			(NT_SEVERITY_ERR | 0x0030)
#define CIFSTATUS_INVALID_QUOTA_LOWER			(NT_SEVERITY_ERR | 0x0031)
#define CIFSTATUS_DISK_CORRUPT_ERROR			(NT_SEVERITY_ERR | 0x0032)
#define CIFSTATUS_OBJECT_NAME_INVALID			(NT_SEVERITY_ERR | 0x0033)
#define CIFSTATUS_OBJECT_NAME_NOT_FOUND			(NT_SEVERITY_ERR | 0x0034)
#define CIFSTATUS_OBJECT_NAME_COLLISION			(NT_SEVERITY_ERR | 0x0035)
#define CIFSTATUS_HANDLE_NOT_WAITABLE			(NT_SEVERITY_ERR | 0x0036)
#define CIFSTATUS_PORT_DISCONNECTED				(NT_SEVERITY_ERR | 0x0037)
#define CIFSTATUS_DEVICE_ALREADY_ATTACHED		(NT_SEVERITY_ERR | 0x0038)
#define CIFSTATUS_OBJECT_PATH_INVALID			(NT_SEVERITY_ERR | 0x0039)
#define CIFSTATUS_OBJECT_PATH_NOT_FOUND			(NT_SEVERITY_ERR | 0x003a)
#define CIFSTATUS_OBJECT_PATH_SYNTAX_BAD		(NT_SEVERITY_ERR | 0x003b)
#define CIFSTATUS_DATA_OVERRUN					(NT_SEVERITY_ERR | 0x003c)
#define CIFSTATUS_DATA_LATE_ERROR				(NT_SEVERITY_ERR | 0x003d)
#define CIFSTATUS_DATA_ERROR					(NT_SEVERITY_ERR | 0x003e)
#define CIFSTATUS_CRC_ERROR						(NT_SEVERITY_ERR | 0x003f)
#define CIFSTATUS_SECTION_TOO_BIG				(NT_SEVERITY_ERR | 0x0040)
#define CIFSTATUS_PORT_CONNECTION_REFUSED		(NT_SEVERITY_ERR | 0x0041)
#define CIFSTATUS_INVALID_PORT_HANDLE			(NT_SEVERITY_ERR | 0x0042)
#define CIFSTATUS_SHARING_VIOLATION				(NT_SEVERITY_ERR | 0x0043)
#define CIFSTATUS_QUOTA_EXCEEDED				(NT_SEVERITY_ERR | 0x0044)
#define CIFSTATUS_INVALID_PAGE_PROTECTION		(NT_SEVERITY_ERR | 0x0045)
#define CIFSTATUS_MUTANT_NOT_OWNED				(NT_SEVERITY_ERR | 0x0046)
#define CIFSTATUS_SEMAPHORE_LIMIT_EXCEEDED		(NT_SEVERITY_ERR | 0x0047)
#define CIFSTATUS_PORT_ALREADY_SET				(NT_SEVERITY_ERR | 0x0048)
#define CIFSTATUS_SECTION_NOT_IMAGE				(NT_SEVERITY_ERR | 0x0049)
#define CIFSTATUS_SUSPEND_COUNT_EXCEEDED		(NT_SEVERITY_ERR | 0x004a)
#define CIFSTATUS_THREAD_IS_TERMINATING			(NT_SEVERITY_ERR | 0x004b)
#define CIFSTATUS_BAD_WORKING_SET_LIMIT			(NT_SEVERITY_ERR | 0x004c)
#define CIFSTATUS_INCOMPATIBLE_FILE_MAP			(NT_SEVERITY_ERR | 0x004d)
#define CIFSTATUS_SECTION_PROTECTION			(NT_SEVERITY_ERR | 0x004e)
#define CIFSTATUS_EAS_NOT_SUPPORTED				(NT_SEVERITY_ERR | 0x004f)
#define CIFSTATUS_EA_TOO_LARGE					(NT_SEVERITY_ERR | 0x0050)
#define CIFSTATUS_NONEXISTENT_EA_ENTRY			(NT_SEVERITY_ERR | 0x0051)
#define CIFSTATUS_NO_EAS_ON_FILE				(NT_SEVERITY_ERR | 0x0052)
#define CIFSTATUS_EA_CORRUPT_ERROR				(NT_SEVERITY_ERR | 0x0053)
#define CIFSTATUS_FILE_LOCK_CONFLICT			(NT_SEVERITY_ERR | 0x0054)
#define CIFSTATUS_LOCK_NOT_GRANTED				(NT_SEVERITY_ERR | 0x0055)
#define CIFSTATUS_DELETE_PENDING				(NT_SEVERITY_ERR | 0x0056)
#define CIFSTATUS_CTL_FILE_NOT_SUPPORTED		(NT_SEVERITY_ERR | 0x0057)
#define CIFSTATUS_UNKNOWN_REVISION				(NT_SEVERITY_ERR | 0x0058)
#define CIFSTATUS_REVISION_MISMATCH				(NT_SEVERITY_ERR | 0x0059)
#define CIFSTATUS_INVALID_OWNER					(NT_SEVERITY_ERR | 0x005a)
#define CIFSTATUS_INVALID_PRIMARY_GROUP			(NT_SEVERITY_ERR | 0x005b)
#define CIFSTATUS_NO_IMPERSONATION_TOKEN		(NT_SEVERITY_ERR | 0x005c)
#define CIFSTATUS_CANT_DISABLE_MANDATORY		(NT_SEVERITY_ERR | 0x005d)
#define CIFSTATUS_NO_LOGON_SERVERS				(NT_SEVERITY_ERR | 0x005e)
#define CIFSTATUS_NO_SUCH_LOGON_SESSION			(NT_SEVERITY_ERR | 0x005f)
#define CIFSTATUS_NO_SUCH_PRIVILEGE				(NT_SEVERITY_ERR | 0x0060)
#define CIFSTATUS_PRIVILEGE_NOT_HELD			(NT_SEVERITY_ERR | 0x0061)
#define CIFSTATUS_INVALID_ACCOUNT_NAME			(NT_SEVERITY_ERR | 0x0062)
#define CIFSTATUS_USER_EXISTS					(NT_SEVERITY_ERR | 0x0063)
#define CIFSTATUS_NO_SUCH_USER					(NT_SEVERITY_ERR | 0x0064)
#define CIFSTATUS_GROUP_EXISTS					(NT_SEVERITY_ERR | 0x0065)
#define CIFSTATUS_NO_SUCH_GROUP					(NT_SEVERITY_ERR | 0x0066)
#define CIFSTATUS_MEMBER_IN_GROUP				(NT_SEVERITY_ERR | 0x0067)
#define CIFSTATUS_MEMBER_NOT_IN_GROUP			(NT_SEVERITY_ERR | 0x0068)
#define CIFSTATUS_LAST_ADMIN					(NT_SEVERITY_ERR | 0x0069)
#define CIFSTATUS_WRONG_PASSWORD				(NT_SEVERITY_ERR | 0x006a)
#define CIFSTATUS_ILL_FORMED_PASSWORD			(NT_SEVERITY_ERR | 0x006b)
#define CIFSTATUS_PASSWORD_RESTRICTION			(NT_SEVERITY_ERR | 0x006c)
#define CIFSTATUS_LOGON_FAILURE					(NT_SEVERITY_ERR | 0x006d)
#define CIFSTATUS_ACCOUNT_RESTRICTION			(NT_SEVERITY_ERR | 0x006e)
#define CIFSTATUS_INVALID_LOGON_HOURS			(NT_SEVERITY_ERR | 0x006f)
#define CIFSTATUS_INVALID_WORKSTATION			(NT_SEVERITY_ERR | 0x0070)
#define CIFSTATUS_PASSWORD_EXPIRED				(NT_SEVERITY_ERR | 0x0071)
#define CIFSTATUS_ACCOUNT_DISABLED				(NT_SEVERITY_ERR | 0x0072)
#define CIFSTATUS_NONE_MAPPED					(NT_SEVERITY_ERR | 0x0073)
#define CIFSTATUS_TOO_MANY_LUIDS_REQUESTED		(NT_SEVERITY_ERR | 0x0074)
#define CIFSTATUS_LUIDS_EXHAUSTED				(NT_SEVERITY_ERR | 0x0075)
#define CIFSTATUS_INVALID_SUB_AUTHORITY			(NT_SEVERITY_ERR | 0x0076)
#define CIFSTATUS_INVALID_ACL					(NT_SEVERITY_ERR | 0x0077)
#define CIFSTATUS_INVALID_SID					(NT_SEVERITY_ERR | 0x0078)
#define CIFSTATUS_INVALID_SECURITY_DESCR		(NT_SEVERITY_ERR | 0x0079)
#define CIFSTATUS_PROCEDURE_NOT_FOUND			(NT_SEVERITY_ERR | 0x007a)
#define CIFSTATUS_INVALID_IMAGE_FORMAT			(NT_SEVERITY_ERR | 0x007b)
#define CIFSTATUS_NO_TOKEN						(NT_SEVERITY_ERR | 0x007c)
#define CIFSTATUS_BAD_INHERITANCE_ACL			(NT_SEVERITY_ERR | 0x007d)
#define CIFSTATUS_RANGE_NOT_LOCKED				(NT_SEVERITY_ERR | 0x007e)
#define CIFSTATUS_DISK_FULL						(NT_SEVERITY_ERR | 0x007f)
#define CIFSTATUS_SERVER_DISABLED				(NT_SEVERITY_ERR | 0x0080)
#define CIFSTATUS_SERVER_NOT_DISABLED			(NT_SEVERITY_ERR | 0x0081)
#define CIFSTATUS_TOO_MANY_GUIDS_REQUESTED		(NT_SEVERITY_ERR | 0x0082)
#define CIFSTATUS_GUIDS_EXHAUSTED				(NT_SEVERITY_ERR | 0x0083)
#define CIFSTATUS_INVALID_ID_AUTHORITY			(NT_SEVERITY_ERR | 0x0084)
#define CIFSTATUS_AGENTS_EXHAUSTED				(NT_SEVERITY_ERR | 0x0085)
#define CIFSTATUS_INVALID_VOLUME_LABEL			(NT_SEVERITY_ERR | 0x0086)
#define CIFSTATUS_SECTION_NOT_EXTENDED			(NT_SEVERITY_ERR | 0x0087)
#define CIFSTATUS_NOT_MAPPED_DATA				(NT_SEVERITY_ERR | 0x0088)
#define CIFSTATUS_RESOURCE_DATA_NOT_FOUND		(NT_SEVERITY_ERR | 0x0089)
#define CIFSTATUS_RESOURCE_TYPE_NOT_FOUND		(NT_SEVERITY_ERR | 0x008a)
#define CIFSTATUS_RESOURCE_NAME_NOT_FOUND		(NT_SEVERITY_ERR | 0x008b)
#define CIFSTATUS_ARRAY_BOUNDS_EXCEEDED			(NT_SEVERITY_ERR | 0x008c)
#define CIFSTATUS_FLOAT_DENORMAL_OPERAND		(NT_SEVERITY_ERR | 0x008d)
#define CIFSTATUS_FLOAT_DIVIDE_BY_ZERO			(NT_SEVERITY_ERR | 0x008e)
#define CIFSTATUS_FLOAT_INEXACT_RESULT			(NT_SEVERITY_ERR | 0x008f)
#define CIFSTATUS_FLOAT_INVALID_OPERATION		(NT_SEVERITY_ERR | 0x0090)
#define CIFSTATUS_FLOAT_OVERFLOW				(NT_SEVERITY_ERR | 0x0091)
#define CIFSTATUS_FLOAT_STACK_CHECK				(NT_SEVERITY_ERR | 0x0092)
#define CIFSTATUS_FLOAT_UNDERFLOW				(NT_SEVERITY_ERR | 0x0093)
#define CIFSTATUS_INTEGER_DIVIDE_BY_ZERO		(NT_SEVERITY_ERR | 0x0094)
#define CIFSTATUS_INTEGER_OVERFLOW				(NT_SEVERITY_ERR | 0x0095)
#define CIFSTATUS_PRIVILEGED_INSTRUCTION		(NT_SEVERITY_ERR | 0x0096)
#define CIFSTATUS_TOO_MANY_PAGING_FILES			(NT_SEVERITY_ERR | 0x0097)
#define CIFSTATUS_FILE_INVALID					(NT_SEVERITY_ERR | 0x0098)
#define CIFSTATUS_ALLOTTED_SPACE_EXCEEDED		(NT_SEVERITY_ERR | 0x0099)
#define CIFSTATUS_INSUFFICIENT_RESOURCES		(NT_SEVERITY_ERR | 0x009a)
#define CIFSTATUS_DFS_EXIT_PATH_FOUND			(NT_SEVERITY_ERR | 0x009b)
#define CIFSTATUS_DEVICE_DATA_ERROR				(NT_SEVERITY_ERR | 0x009c)
#define CIFSTATUS_DEVICE_NOT_CONNECTED			(NT_SEVERITY_ERR | 0x009d)
#define CIFSTATUS_DEVICE_POWER_FAILURE			(NT_SEVERITY_ERR | 0x009e)
#define CIFSTATUS_FREE_VM_NOT_AT_BASE			(NT_SEVERITY_ERR | 0x009f)
#define CIFSTATUS_MEMORY_NOT_ALLOCATED			(NT_SEVERITY_ERR | 0x00a0)
#define CIFSTATUS_WORKING_SET_QUOTA				(NT_SEVERITY_ERR | 0x00a1)
#define CIFSTATUS_MEDIA_WRITE_PROTECTED			(NT_SEVERITY_ERR | 0x00a2)
#define CIFSTATUS_DEVICE_NOT_READY				(NT_SEVERITY_ERR | 0x00a3)
#define CIFSTATUS_INVALID_GROUP_ATTRIBUTES		(NT_SEVERITY_ERR | 0x00a4)
#define CIFSTATUS_BAD_IMPERSONATION_LEVEL		(NT_SEVERITY_ERR | 0x00a5)
#define CIFSTATUS_CANT_OPEN_ANONYMOUS			(NT_SEVERITY_ERR | 0x00a6)
#define CIFSTATUS_BAD_VALIDATION_CLASS			(NT_SEVERITY_ERR | 0x00a7)
#define CIFSTATUS_BAD_TOKEN_TYPE				(NT_SEVERITY_ERR | 0x00a8)
#define CIFSTATUS_BAD_MASTER_BOOT_RECORD		(NT_SEVERITY_ERR | 0x00a9)
#define CIFSTATUS_INSTRUCTION_MISALIGNMENT		(NT_SEVERITY_ERR | 0x00aa)
#define CIFSTATUS_INSTANCE_NOT_AVAILABLE		(NT_SEVERITY_ERR | 0x00ab)
#define CIFSTATUS_PIPE_NOT_AVAILABLE			(NT_SEVERITY_ERR | 0x00ac)
#define CIFSTATUS_INVALID_PIPE_STATE			(NT_SEVERITY_ERR | 0x00ad)
#define CIFSTATUS_PIPE_BUSY						(NT_SEVERITY_ERR | 0x00ae)
#define CIFSTATUS_ILLEGAL_FUNCTION				(NT_SEVERITY_ERR | 0x00af)
#define CIFSTATUS_PIPE_DISCONNECTED				(NT_SEVERITY_ERR | 0x00b0)
#define CIFSTATUS_PIPE_CLOSING					(NT_SEVERITY_ERR | 0x00b1)
#define CIFSTATUS_PIPE_CONNECTED				(NT_SEVERITY_ERR | 0x00b2)
#define CIFSTATUS_PIPE_LISTENING				(NT_SEVERITY_ERR | 0x00b3)
#define CIFSTATUS_INVALID_READ_MODE				(NT_SEVERITY_ERR | 0x00b4)
#define CIFSTATUS_IO_TIMEOUT					(NT_SEVERITY_ERR | 0x00b5)
#define CIFSTATUS_FILE_FORCED_CLOSED			(NT_SEVERITY_ERR | 0x00b6)
#define CIFSTATUS_PROFILING_NOT_STARTED			(NT_SEVERITY_ERR | 0x00b7)
#define CIFSTATUS_PROFILING_NOT_STOPPED			(NT_SEVERITY_ERR | 0x00b8)
#define CIFSTATUS_COULD_NOT_INTERPRET			(NT_SEVERITY_ERR | 0x00b9)
#define CIFSTATUS_FILE_IS_A_DIRECTORY			(NT_SEVERITY_ERR | 0x00ba)
#define CIFSTATUS_NOT_SUPPORTED					(NT_SEVERITY_ERR | 0x00bb)
#define CIFSTATUS_REMOTE_NOT_LISTENING			(NT_SEVERITY_ERR | 0x00bc)
#define CIFSTATUS_DUPLICATE_NAME				(NT_SEVERITY_ERR | 0x00bd)
#define CIFSTATUS_BAD_NETWORK_PATH				(NT_SEVERITY_ERR | 0x00be)
#define CIFSTATUS_NETWORK_BUSY					(NT_SEVERITY_ERR | 0x00bf)
#define CIFSTATUS_DEVICE_DOES_NOT_EXIST			(NT_SEVERITY_ERR | 0x00c0)
#define CIFSTATUS_TOO_MANY_COMMANDS				(NT_SEVERITY_ERR | 0x00c1)
#define CIFSTATUS_ADAPTER_HARDWARE_ERROR		(NT_SEVERITY_ERR | 0x00c2)
#define CIFSTATUS_INVALID_NETWORK_RESPONSE		(NT_SEVERITY_ERR | 0x00c3)
#define CIFSTATUS_UNEXPECTED_NETWORK_ERROR		(NT_SEVERITY_ERR | 0x00c4)
#define CIFSTATUS_BAD_REMOTE_ADAPTER			(NT_SEVERITY_ERR | 0x00c5)
#define CIFSTATUS_PRINT_QUEUE_FULL				(NT_SEVERITY_ERR | 0x00c6)
#define CIFSTATUS_NO_SPOOL_SPACE				(NT_SEVERITY_ERR | 0x00c7)
#define CIFSTATUS_PRINT_CANCELLED				(NT_SEVERITY_ERR | 0x00c8)
#define CIFSTATUS_NETWORK_NAME_DELETED			(NT_SEVERITY_ERR | 0x00c9)
#define CIFSTATUS_NETWORK_ACCESS_DENIED			(NT_SEVERITY_ERR | 0x00ca)
#define CIFSTATUS_BAD_DEVICE_TYPE				(NT_SEVERITY_ERR | 0x00cb)
#define CIFSTATUS_BAD_NETWORK_NAME				(NT_SEVERITY_ERR | 0x00cc)
#define CIFSTATUS_TOO_MANY_NAMES				(NT_SEVERITY_ERR | 0x00cd)
#define CIFSTATUS_TOO_MANY_SESSIONS				(NT_SEVERITY_ERR | 0x00ce)
#define CIFSTATUS_SHARING_PAUSED				(NT_SEVERITY_ERR | 0x00cf)
#define CIFSTATUS_REQUEST_NOT_ACCEPTED			(NT_SEVERITY_ERR | 0x00d0)
#define CIFSTATUS_REDIRECTOR_PAUSED				(NT_SEVERITY_ERR | 0x00d1)
#define CIFSTATUS_NET_WRITE_FAULT				(NT_SEVERITY_ERR | 0x00d2)
#define CIFSTATUS_PROFILING_AT_LIMIT			(NT_SEVERITY_ERR | 0x00d3)
#define CIFSTATUS_NOT_SAME_DEVICE				(NT_SEVERITY_ERR | 0x00d4)
#define CIFSTATUS_FILE_RENAMED					(NT_SEVERITY_ERR | 0x00d5)
#define CIFSTATUS_VIRTUAL_CIRCUIT_CLOSED		(NT_SEVERITY_ERR | 0x00d6)
#define CIFSTATUS_NO_SECURITY_ON_OBJECT			(NT_SEVERITY_ERR | 0x00d7)
#define CIFSTATUS_CANT_WAIT						(NT_SEVERITY_ERR | 0x00d8)
#define CIFSTATUS_PIPE_EMPTY					(NT_SEVERITY_ERR | 0x00d9)
#define CIFSTATUS_CANT_ACCESS_DOMAIN_INFO		(NT_SEVERITY_ERR | 0x00da)
#define CIFSTATUS_CANT_TERMINATE_SELF			(NT_SEVERITY_ERR | 0x00db)
#define CIFSTATUS_INVALID_SERVER_STATE			(NT_SEVERITY_ERR | 0x00dc)
#define CIFSTATUS_INVALID_DOMAIN_STATE			(NT_SEVERITY_ERR | 0x00dd)
#define CIFSTATUS_INVALID_DOMAIN_ROLE			(NT_SEVERITY_ERR | 0x00de)
#define CIFSTATUS_NO_SUCH_DOMAIN				(NT_SEVERITY_ERR | 0x00df)
#define CIFSTATUS_DOMAIN_EXISTS					(NT_SEVERITY_ERR | 0x00e0)
#define CIFSTATUS_DOMAIN_LIMIT_EXCEEDED			(NT_SEVERITY_ERR | 0x00e1)
#define CIFSTATUS_OPLOCK_NOT_GRANTED			(NT_SEVERITY_ERR | 0x00e2)
#define CIFSTATUS_INVALID_OPLOCK_PROTOCOL		(NT_SEVERITY_ERR | 0x00e3)
#define CIFSTATUS_INTERNAL_DB_CORRUPTION		(NT_SEVERITY_ERR | 0x00e4)
#define CIFSTATUS_INTERNAL_ERROR				(NT_SEVERITY_ERR | 0x00e5)
#define CIFSTATUS_GENERIC_NOT_MAPPED			(NT_SEVERITY_ERR | 0x00e6)
#define CIFSTATUS_BAD_DESCRIPTOR_FORMAT			(NT_SEVERITY_ERR | 0x00e7)
#define CIFSTATUS_INVALID_USER_BUFFER			(NT_SEVERITY_ERR | 0x00e8)
#define CIFSTATUS_UNEXPECTED_IO_ERROR			(NT_SEVERITY_ERR | 0x00e9)
#define CIFSTATUS_UNEXPECTED_MM_CREATE_ERR		(NT_SEVERITY_ERR | 0x00ea)
#define CIFSTATUS_UNEXPECTED_MM_MAP_ERROR		(NT_SEVERITY_ERR | 0x00eb)
#define CIFSTATUS_UNEXPECTED_MM_EXTEND_ERR		(NT_SEVERITY_ERR | 0x00ec)
#define CIFSTATUS_NOT_LOGON_PROCESS				(NT_SEVERITY_ERR | 0x00ed)
#define CIFSTATUS_LOGON_SESSION_EXISTS			(NT_SEVERITY_ERR | 0x00ee)
#define CIFSTATUS_INVALID_PARAMETER_1			(NT_SEVERITY_ERR | 0x00ef)
#define CIFSTATUS_INVALID_PARAMETER_2			(NT_SEVERITY_ERR | 0x00f0)
#define CIFSTATUS_INVALID_PARAMETER_3			(NT_SEVERITY_ERR | 0x00f1)
#define CIFSTATUS_INVALID_PARAMETER_4			(NT_SEVERITY_ERR | 0x00f2)
#define CIFSTATUS_INVALID_PARAMETER_5			(NT_SEVERITY_ERR | 0x00f3)
#define CIFSTATUS_INVALID_PARAMETER_6			(NT_SEVERITY_ERR | 0x00f4)
#define CIFSTATUS_INVALID_PARAMETER_7			(NT_SEVERITY_ERR | 0x00f5)
#define CIFSTATUS_INVALID_PARAMETER_8			(NT_SEVERITY_ERR | 0x00f6)
#define CIFSTATUS_INVALID_PARAMETER_9			(NT_SEVERITY_ERR | 0x00f7)
#define CIFSTATUS_INVALID_PARAMETER_10			(NT_SEVERITY_ERR | 0x00f8)
#define CIFSTATUS_INVALID_PARAMETER_11			(NT_SEVERITY_ERR | 0x00f9)
#define CIFSTATUS_INVALID_PARAMETER_12			(NT_SEVERITY_ERR | 0x00fa)
#define CIFSTATUS_REDIRECTOR_NOT_STARTED		(NT_SEVERITY_ERR | 0x00fb)
#define CIFSTATUS_REDIRECTOR_STARTED			(NT_SEVERITY_ERR | 0x00fc)
#define CIFSTATUS_STACK_OVERFLOW				(NT_SEVERITY_ERR | 0x00fd)
#define CIFSTATUS_NO_SUCH_PACKAGE				(NT_SEVERITY_ERR | 0x00fe)
#define CIFSTATUS_BAD_FUNCTION_TABLE			(NT_SEVERITY_ERR | 0x00ff)
#define CIFSTATUS_DIRECTORY_NOT_EMPTY			(NT_SEVERITY_ERR | 0x0101)
#define CIFSTATUS_FILE_CORRUPT_ERROR			(NT_SEVERITY_ERR | 0x0102)
#define CIFSTATUS_NOT_A_DIRECTORY				(NT_SEVERITY_ERR | 0x0103)
#define CIFSTATUS_BAD_LOGON_SESSION_STATE		(NT_SEVERITY_ERR | 0x0104)
#define CIFSTATUS_LOGON_SESSION_COLLISION		(NT_SEVERITY_ERR | 0x0105)
#define CIFSTATUS_NAME_TOO_LONG					(NT_SEVERITY_ERR | 0x0106)
#define CIFSTATUS_FILES_OPEN					(NT_SEVERITY_ERR | 0x0107)
#define CIFSTATUS_CONNECTION_IN_USE				(NT_SEVERITY_ERR | 0x0108)
#define CIFSTATUS_MESSAGE_NOT_FOUND				(NT_SEVERITY_ERR | 0x0109)
#define CIFSTATUS_PROCESS_IS_TERMINATING		(NT_SEVERITY_ERR | 0x010a)
#define CIFSTATUS_INVALID_LOGON_TYPE			(NT_SEVERITY_ERR | 0x010b)
#define CIFSTATUS_NO_GUID_TRANSLATION			(NT_SEVERITY_ERR | 0x010c)
#define CIFSTATUS_CANNOT_IMPERSONATE			(NT_SEVERITY_ERR | 0x010d)
#define CIFSTATUS_IMAGE_ALREADY_LOADED			(NT_SEVERITY_ERR | 0x010e)
#define CIFSTATUS_ABIOS_NOT_PRESENT				(NT_SEVERITY_ERR | 0x010f)
#define CIFSTATUS_ABIOS_LID_NOT_EXIST			(NT_SEVERITY_ERR | 0x0110)
#define CIFSTATUS_ABIOS_LID_ALREADY_OWNED		(NT_SEVERITY_ERR | 0x0111)
#define CIFSTATUS_ABIOS_NOT_LID_OWNER			(NT_SEVERITY_ERR | 0x0112)
#define CIFSTATUS_ABIOS_INVALID_COMMAND			(NT_SEVERITY_ERR | 0x0113)
#define CIFSTATUS_ABIOS_INVALID_LID				(NT_SEVERITY_ERR | 0x0114)
#define CIFSTATUS_ABIOS_SELECTOR_NOT_AVAILABLE	(NT_SEVERITY_ERR | 0x0115)
#define CIFSTATUS_ABIOS_INVALID_SELECTOR		(NT_SEVERITY_ERR | 0x0116)
#define CIFSTATUS_NO_LDT						(NT_SEVERITY_ERR | 0x0117)
#define CIFSTATUS_INVALID_LDT_SIZE				(NT_SEVERITY_ERR | 0x0118)
#define CIFSTATUS_INVALID_LDT_OFFSET			(NT_SEVERITY_ERR | 0x0119)
#define CIFSTATUS_INVALID_LDT_DESCRIPTOR		(NT_SEVERITY_ERR | 0x011a)
#define CIFSTATUS_INVALID_IMAGE_NE_FORMAT		(NT_SEVERITY_ERR | 0x011b)
#define CIFSTATUS_RXACT_INVALID_STATE			(NT_SEVERITY_ERR | 0x011c)
#define CIFSTATUS_RXACT_COMMIT_FAILURE			(NT_SEVERITY_ERR | 0x011d)
#define CIFSTATUS_MAPPED_FILE_SIZE_ZERO			(NT_SEVERITY_ERR | 0x011e)
#define CIFSTATUS_TOO_MANY_OPENED_FILES			(NT_SEVERITY_ERR | 0x011f)
#define CIFSTATUS_CANCELLED						(NT_SEVERITY_ERR | 0x0120)
#define CIFSTATUS_CANNOT_DELETE					(NT_SEVERITY_ERR | 0x0121)
#define CIFSTATUS_INVALID_COMPUTER_NAME			(NT_SEVERITY_ERR | 0x0122)
#define CIFSTATUS_FILE_DELETED					(NT_SEVERITY_ERR | 0x0123)
#define CIFSTATUS_SPECIAL_ACCOUNT				(NT_SEVERITY_ERR | 0x0124)
#define CIFSTATUS_SPECIAL_GROUP					(NT_SEVERITY_ERR | 0x0125)
#define CIFSTATUS_SPECIAL_USER					(NT_SEVERITY_ERR | 0x0126)
#define CIFSTATUS_MEMBERS_PRIMARY_GROUP			(NT_SEVERITY_ERR | 0x0127)
#define CIFSTATUS_FILE_CLOSED					(NT_SEVERITY_ERR | 0x0128)
#define CIFSTATUS_TOO_MANY_THREADS				(NT_SEVERITY_ERR | 0x0129)
#define CIFSTATUS_THREAD_NOT_IN_PROCESS			(NT_SEVERITY_ERR | 0x012a)
#define CIFSTATUS_TOKEN_ALREADY_IN_USE			(NT_SEVERITY_ERR | 0x012b)
#define CIFSTATUS_PAGEFILE_QUOTA_EXCEEDED		(NT_SEVERITY_ERR | 0x012c)
#define CIFSTATUS_COMMITMENT_LIMIT				(NT_SEVERITY_ERR | 0x012d)
#define CIFSTATUS_INVALID_IMAGE_LE_FORMAT		(NT_SEVERITY_ERR | 0x012e)
#define CIFSTATUS_INVALID_IMAGE_NOT_MZ			(NT_SEVERITY_ERR | 0x012f)
#define CIFSTATUS_INVALID_IMAGE_PROTECT			(NT_SEVERITY_ERR | 0x0130)
#define CIFSTATUS_INVALID_IMAGE_WIN_16			(NT_SEVERITY_ERR | 0x0131)
#define CIFSTATUS_LOGON_SERVER_CONFLICT			(NT_SEVERITY_ERR | 0x0132)
#define CIFSTATUS_TIME_DIFFERENCE_AT_DC			(NT_SEVERITY_ERR | 0x0133)
#define CIFSTATUS_SYNCHRONIZATION_REQUIRED		(NT_SEVERITY_ERR | 0x0134)
#define CIFSTATUS_DLL_NOT_FOUND					(NT_SEVERITY_ERR | 0x0135)
#define CIFSTATUS_OPEN_FAILED					(NT_SEVERITY_ERR | 0x0136)
#define CIFSTATUS_IO_PRIVILEGE_FAILED			(NT_SEVERITY_ERR | 0x0137)
#define CIFSTATUS_ORDINAL_NOT_FOUND				(NT_SEVERITY_ERR | 0x0138)
#define CIFSTATUS_ENTRYPOINT_NOT_FOUND			(NT_SEVERITY_ERR | 0x0139)
#define CIFSTATUS_CONTROL_C_EXIT				(NT_SEVERITY_ERR | 0x013a)
#define CIFSTATUS_LOCAL_DISCONNECT				(NT_SEVERITY_ERR | 0x013b)
#define CIFSTATUS_REMOTE_DISCONNECT				(NT_SEVERITY_ERR | 0x013c)
#define CIFSTATUS_REMOTE_RESOURCES				(NT_SEVERITY_ERR | 0x013d)
#define CIFSTATUS_LINK_FAILED					(NT_SEVERITY_ERR | 0x013e)
#define CIFSTATUS_LINK_TIMEOUT					(NT_SEVERITY_ERR | 0x013f)
#define CIFSTATUS_INVALID_CONNECTION			(NT_SEVERITY_ERR | 0x0140)
#define CIFSTATUS_INVALID_ADDRESS				(NT_SEVERITY_ERR | 0x0141)
#define CIFSTATUS_DLL_INIT_FAILED				(NT_SEVERITY_ERR | 0x0142)
#define CIFSTATUS_MISSING_SYSTEMFILE			(NT_SEVERITY_ERR | 0x0143)
#define CIFSTATUS_UNHANDLED_EXCEPTION			(NT_SEVERITY_ERR | 0x0144)
#define CIFSTATUS_APP_INIT_FAILURE				(NT_SEVERITY_ERR | 0x0145)
#define CIFSTATUS_PAGEFILE_CREATE_FAILED		(NT_SEVERITY_ERR | 0x0146)
#define CIFSTATUS_NO_PAGEFILE					(NT_SEVERITY_ERR | 0x0147)
#define CIFSTATUS_INVALID_LEVEL					(NT_SEVERITY_ERR | 0x0148)
#define CIFSTATUS_WRONG_PASSWORD_CORE			(NT_SEVERITY_ERR | 0x0149)
#define CIFSTATUS_ILLEGAL_FLOAT_CONTEXT			(NT_SEVERITY_ERR | 0x014a)
#define CIFSTATUS_PIPE_BROKEN					(NT_SEVERITY_ERR | 0x014b)
#define CIFSTATUS_REGISTRY_CORRUPT				(NT_SEVERITY_ERR | 0x014c)
#define CIFSTATUS_REGISTRY_IO_FAILED			(NT_SEVERITY_ERR | 0x014d)
#define CIFSTATUS_NO_EVENT_PAIR					(NT_SEVERITY_ERR | 0x014e)
#define CIFSTATUS_UNRECOGNIZED_VOLUME			(NT_SEVERITY_ERR | 0x014f)
#define CIFSTATUS_SERIAL_NO_DEVICE_INITED		(NT_SEVERITY_ERR | 0x0150)
#define CIFSTATUS_NO_SUCH_ALIAS					(NT_SEVERITY_ERR | 0x0151)
#define CIFSTATUS_MEMBER_NOT_IN_ALIAS			(NT_SEVERITY_ERR | 0x0152)
#define CIFSTATUS_MEMBER_IN_ALIAS				(NT_SEVERITY_ERR | 0x0153)
#define CIFSTATUS_ALIAS_EXISTS					(NT_SEVERITY_ERR | 0x0154)
#define CIFSTATUS_LOGON_NOT_GRANTED				(NT_SEVERITY_ERR | 0x0155)
#define CIFSTATUS_TOO_MANY_SECRETS				(NT_SEVERITY_ERR | 0x0156)
#define CIFSTATUS_SECRET_TOO_LONG				(NT_SEVERITY_ERR | 0x0157)
#define CIFSTATUS_INTERNAL_DB_ERROR				(NT_SEVERITY_ERR | 0x0158)
#define CIFSTATUS_FULLSCREEN_MODE				(NT_SEVERITY_ERR | 0x0159)
#define CIFSTATUS_TOO_MANY_CONTEXT_IDS			(NT_SEVERITY_ERR | 0x015a)
#define CIFSTATUS_LOGON_TYPE_NOT_GRANTED		(NT_SEVERITY_ERR | 0x015b)
#define CIFSTATUS_NOT_REGISTRY_FILE				(NT_SEVERITY_ERR | 0x015c)
#define CIFSTATUS_NT_CROSS_ENCRYPTION_REQUIRED	(NT_SEVERITY_ERR | 0x015d)
#define CIFSTATUS_DOMAIN_CTRLR_CONFIG_ERROR		(NT_SEVERITY_ERR | 0x015e)
#define CIFSTATUS_FT_MISSING_MEMBER				(NT_SEVERITY_ERR | 0x015f)
#define CIFSTATUS_ILL_FORMED_SERVICE_ENTRY		(NT_SEVERITY_ERR | 0x0160)
#define CIFSTATUS_ILLEGAL_CHARACTER				(NT_SEVERITY_ERR | 0x0161)
#define CIFSTATUS_UNMAPPABLE_CHARACTER			(NT_SEVERITY_ERR | 0x0162)
#define CIFSTATUS_UNDEFINED_CHARACTER			(NT_SEVERITY_ERR | 0x0163)
#define CIFSTATUS_FLOPPY_VOLUME					(NT_SEVERITY_ERR | 0x0164)
#define CIFSTATUS_FLOPPY_ID_MARK_NOT_FOUND		(NT_SEVERITY_ERR | 0x0165)
#define CIFSTATUS_FLOPPY_WRONG_CYLINDER			(NT_SEVERITY_ERR | 0x0166)
#define CIFSTATUS_FLOPPY_UNKNOWN_ERROR			(NT_SEVERITY_ERR | 0x0167)
#define CIFSTATUS_FLOPPY_BAD_REGISTERS			(NT_SEVERITY_ERR | 0x0168)
#define CIFSTATUS_DISK_RECALIBRATE_FAILED		(NT_SEVERITY_ERR | 0x0169)
#define CIFSTATUS_DISK_OPERATION_FAILED			(NT_SEVERITY_ERR | 0x016a)
#define CIFSTATUS_DISK_RESET_FAILED				(NT_SEVERITY_ERR | 0x016b)
#define CIFSTATUS_SHARED_IRQ_BUSY				(NT_SEVERITY_ERR | 0x016c)
#define CIFSTATUS_FT_ORPHANING					(NT_SEVERITY_ERR | 0x016d)
#define CIFSTATUS_PARTITION_FAILURE				(NT_SEVERITY_ERR | 0x0172)
#define CIFSTATUS_INVALID_BLOCK_LENGTH			(NT_SEVERITY_ERR | 0x0173)
#define CIFSTATUS_DEVICE_NOT_PARTITIONED		(NT_SEVERITY_ERR | 0x0174)
#define CIFSTATUS_UNABLE_TO_LOCK_MEDIA			(NT_SEVERITY_ERR | 0x0175)
#define CIFSTATUS_UNABLE_TO_UNLOAD_MEDIA		(NT_SEVERITY_ERR | 0x0176)
#define CIFSTATUS_EOM_OVERFLOW					(NT_SEVERITY_ERR | 0x0177)
#define CIFSTATUS_NO_MEDIA						(NT_SEVERITY_ERR | 0x0178)
#define CIFSTATUS_NO_SUCH_MEMBER				(NT_SEVERITY_ERR | 0x017a)
#define CIFSTATUS_INVALID_MEMBER				(NT_SEVERITY_ERR | 0x017b)
#define CIFSTATUS_KEY_DELETED					(NT_SEVERITY_ERR | 0x017c)
#define CIFSTATUS_NO_LOG_SPACE					(NT_SEVERITY_ERR | 0x017d)
#define CIFSTATUS_TOO_MANY_SIDS					(NT_SEVERITY_ERR | 0x017e)
#define CIFSTATUS_LM_CROSS_ENCRYPTION_REQUIRED	(NT_SEVERITY_ERR | 0x017f)
#define CIFSTATUS_KEY_HAS_CHILDREN				(NT_SEVERITY_ERR | 0x0180)
#define CIFSTATUS_CHILD_MUST_BE_VOLATILE		(NT_SEVERITY_ERR | 0x0181)
#define CIFSTATUS_DEVICE_CONFIGURATION_ERROR	(NT_SEVERITY_ERR | 0x0182)
#define CIFSTATUS_DRIVER_INTERNAL_ERROR			(NT_SEVERITY_ERR | 0x0183)
#define CIFSTATUS_INVALID_DEVICE_STATE			(NT_SEVERITY_ERR | 0x0184)
#define CIFSTATUS_IO_DEVICE_ERROR				(NT_SEVERITY_ERR | 0x0185)
#define CIFSTATUS_DEVICE_PROTOCOL_ERROR			(NT_SEVERITY_ERR | 0x0186)
#define CIFSTATUS_BACKUP_CONTROLLER				(NT_SEVERITY_ERR | 0x0187)
#define CIFSTATUS_LOG_FILE_FULL					(NT_SEVERITY_ERR | 0x0188)
#define CIFSTATUS_TOO_LATE						(NT_SEVERITY_ERR | 0x0189)
#define CIFSTATUS_NO_TRUST_LSA_SECRET			(NT_SEVERITY_ERR | 0x018a)
#define CIFSTATUS_NO_TRUST_SAM_ACCOUNT			(NT_SEVERITY_ERR | 0x018b)
#define CIFSTATUS_TRUSTED_DOMAIN_FAILURE		(NT_SEVERITY_ERR | 0x018c)
#define CIFSTATUS_TRUSTED_RELATIONSHIP_FAILURE	(NT_SEVERITY_ERR | 0x018d)
#define CIFSTATUS_EVENTLOG_FILE_CORRUPT			(NT_SEVERITY_ERR | 0x018e)
#define CIFSTATUS_EVENTLOG_CANT_START			(NT_SEVERITY_ERR | 0x018f)
#define CIFSTATUS_TRUST_FAILURE					(NT_SEVERITY_ERR | 0x0190)
#define CIFSTATUS_MUTANT_LIMIT_EXCEEDED			(NT_SEVERITY_ERR | 0x0191)
#define CIFSTATUS_NETLOGON_NOT_STARTED			(NT_SEVERITY_ERR | 0x0192)
#define CIFSTATUS_ACCOUNT_EXPIRED				(NT_SEVERITY_ERR | 0x0193)
#define CIFSTATUS_POSSIBLE_DEADLOCK				(NT_SEVERITY_ERR | 0x0194)
#define CIFSTATUS_NETWORK_CREDENTIAL_CONFLICT	(NT_SEVERITY_ERR | 0x0195)
#define CIFSTATUS_REMOTE_SESSION_LIMIT			(NT_SEVERITY_ERR | 0x0196)
#define CIFSTATUS_EVENTLOG_FILE_CHANGED			(NT_SEVERITY_ERR | 0x0197)
#define CIFSTATUS_NOLOGON_INTERDOMAIN_TRUST_ACC	(NT_SEVERITY_ERR | 0x0198)
#define CIFSTATUS_NOLOGON_WORKSTATION_TRUST_ACC	(NT_SEVERITY_ERR | 0x0199)
#define CIFSTATUS_NOLOGON_SERVER_TRUST_ACC		(NT_SEVERITY_ERR | 0x019a)
#define CIFSTATUS_DOMAIN_TRUST_INCONSISTENT		(NT_SEVERITY_ERR | 0x019b)
#define CIFSTATUS_FS_DRIVER_REQUIRED			(NT_SEVERITY_ERR | 0x019c)
#define CIFSTATUS_NO_USER_SESSION_KEY			(NT_SEVERITY_ERR | 0x0202)
#define CIFSTATUS_USER_SESSION_DELETED			(NT_SEVERITY_ERR | 0x0203)
#define CIFSTATUS_RESOURCE_LANG_NOT_FOUND		(NT_SEVERITY_ERR | 0x0204)
#define CIFSTATUS_INSUFF_SERVER_RESOURCES		(NT_SEVERITY_ERR | 0x0205)
#define CIFSTATUS_INVALID_BUFFER_SIZE			(NT_SEVERITY_ERR | 0x0206)
#define CIFSTATUS_INVALID_ADDRESS_COMPONENT		(NT_SEVERITY_ERR | 0x0207)
#define CIFSTATUS_INVALID_ADDRESS_WILDCARD		(NT_SEVERITY_ERR | 0x0208)
#define CIFSTATUS_TOO_MANY_ADDRESSES			(NT_SEVERITY_ERR | 0x0209)
#define CIFSTATUS_ADDRESS_ALREADY_EXISTS		(NT_SEVERITY_ERR | 0x020a)
#define CIFSTATUS_ADDRESS_CLOSED				(NT_SEVERITY_ERR | 0x020b)
#define CIFSTATUS_CONNECTION_DISCONNECTED		(NT_SEVERITY_ERR | 0x020c)
#define CIFSTATUS_CONNECTION_RESET				(NT_SEVERITY_ERR | 0x020d)
#define CIFSTATUS_TOO_MANY_NODES				(NT_SEVERITY_ERR | 0x020e)
#define CIFSTATUS_TRANSACTION_ABORTED			(NT_SEVERITY_ERR | 0x020f)
#define CIFSTATUS_TRANSACTION_TIMED_OUT			(NT_SEVERITY_ERR | 0x0210)
#define CIFSTATUS_TRANSACTION_NO_RELEASE		(NT_SEVERITY_ERR | 0x0211)
#define CIFSTATUS_TRANSACTION_NO_MATCH			(NT_SEVERITY_ERR | 0x0212)
#define CIFSTATUS_TRANSACTION_RESPONDED			(NT_SEVERITY_ERR | 0x0213)
#define CIFSTATUS_TRANSACTION_INVALID_ID		(NT_SEVERITY_ERR | 0x0214)
#define CIFSTATUS_TRANSACTION_INVALID_TYPE		(NT_SEVERITY_ERR | 0x0215)
#define CIFSTATUS_NOT_SERVER_SESSION			(NT_SEVERITY_ERR | 0x0216)
#define CIFSTATUS_NOT_CLIENT_SESSION			(NT_SEVERITY_ERR | 0x0217)
#define CIFSTATUS_CANNOT_LOAD_REGISTRY_FILE		(NT_SEVERITY_ERR | 0x0218)
#define CIFSTATUS_DEBUG_ATTACH_FAILED			(NT_SEVERITY_ERR | 0x0219)
#define CIFSTATUS_SYSTEM_PROCESS_TERMINATED		(NT_SEVERITY_ERR | 0x021a)
#define CIFSTATUS_DATA_NOT_ACCEPTED				(NT_SEVERITY_ERR | 0x021b)
#define CIFSTATUS_NO_BROWSER_SERVERS_FOUND		(NT_SEVERITY_ERR | 0x021c)
#define CIFSTATUS_VDM_HARD_ERROR				(NT_SEVERITY_ERR | 0x021d)
#define CIFSTATUS_DRIVER_CANCEL_TIMEOUT			(NT_SEVERITY_ERR | 0x021e)
#define CIFSTATUS_REPLY_MESSAGE_MISMATCH		(NT_SEVERITY_ERR | 0x021f)
#define CIFSTATUS_MAPPED_ALIGNMENT				(NT_SEVERITY_ERR | 0x0220)
#define CIFSTATUS_IMAGE_CHECKSUM_MISMATCH		(NT_SEVERITY_ERR | 0x0221)
#define CIFSTATUS_LOST_WRITEBEHIND_DATA			(NT_SEVERITY_ERR | 0x0222)
#define CIFSTATUS_CLIENT_SERVER_PARAMS_INVALID	(NT_SEVERITY_ERR | 0x0223)
#define CIFSTATUS_PASSWORD_MUST_CHANGE			(NT_SEVERITY_ERR | 0x0224)
#define CIFSTATUS_NOT_FOUND						(NT_SEVERITY_ERR | 0x0225)
#define CIFSTATUS_NOT_TINY_STREAM				(NT_SEVERITY_ERR | 0x0226)
#define CIFSTATUS_RECOVERY_FAILURE				(NT_SEVERITY_ERR | 0x0227)
#define CIFSTATUS_STACK_OVERFLOW_READ			(NT_SEVERITY_ERR | 0x0228)
#define CIFSTATUS_FAIL_CHECK					(NT_SEVERITY_ERR | 0x0229)
#define CIFSTATUS_DUPLICATE_OBJECTID			(NT_SEVERITY_ERR | 0x022a)
#define CIFSTATUS_OBJECTID_EXISTS				(NT_SEVERITY_ERR | 0x022b)
#define CIFSTATUS_CONVERT_TO_LARGE				(NT_SEVERITY_ERR | 0x022c)
#define CIFSTATUS_RETRY							(NT_SEVERITY_ERR | 0x022d)
#define CIFSTATUS_FOUND_OUT_OF_SCOPE			(NT_SEVERITY_ERR | 0x022e)
#define CIFSTATUS_ALLOCATE_BUCKET				(NT_SEVERITY_ERR | 0x022f)
#define CIFSTATUS_PROPSET_NOT_FOUND				(NT_SEVERITY_ERR | 0x0230)
#define CIFSTATUS_MARSHALL_OVERFLOW				(NT_SEVERITY_ERR | 0x0231)
#define CIFSTATUS_INVALID_VARIANT				(NT_SEVERITY_ERR | 0x0232)
#define CIFSTATUS_DOMAIN_CONTROLLER_NOT_FOUND	(NT_SEVERITY_ERR | 0x0233)
#define CIFSTATUS_ACCOUNT_LOCKED_OUT			(NT_SEVERITY_ERR | 0x0234)
#define CIFSTATUS_HANDLE_NOT_CLOSABLE			(NT_SEVERITY_ERR | 0x0235)
#define CIFSTATUS_CONNECTION_REFUSED			(NT_SEVERITY_ERR | 0x0236)
#define CIFSTATUS_GRACEFUL_DISCONNECT			(NT_SEVERITY_ERR | 0x0237)
#define CIFSTATUS_ADDRESS_ALREADY_ASSOCIATED	(NT_SEVERITY_ERR | 0x0238)
#define CIFSTATUS_ADDRESS_NOT_ASSOCIATED		(NT_SEVERITY_ERR | 0x0239)
#define CIFSTATUS_CONNECTION_INVALID			(NT_SEVERITY_ERR | 0x023a)
#define CIFSTATUS_CONNECTION_ACTIVE				(NT_SEVERITY_ERR | 0x023b)
#define CIFSTATUS_NETWORK_UNREACHABLE			(NT_SEVERITY_ERR | 0x023c)
#define CIFSTATUS_HOST_UNREACHABLE				(NT_SEVERITY_ERR | 0x023d)
#define CIFSTATUS_PROTOCOL_UNREACHABLE			(NT_SEVERITY_ERR | 0x023e)
#define CIFSTATUS_PORT_UNREACHABLE				(NT_SEVERITY_ERR | 0x023f)
#define CIFSTATUS_REQUEST_ABORTED				(NT_SEVERITY_ERR | 0x0240)
#define CIFSTATUS_CONNECTION_ABORTED			(NT_SEVERITY_ERR | 0x0241)
#define CIFSTATUS_BAD_COMPRESSION_BUFFER		(NT_SEVERITY_ERR | 0x0242)
#define CIFSTATUS_USER_MAPPED_FILE				(NT_SEVERITY_ERR | 0x0243)
#define CIFSTATUS_AUDIT_FAILED					(NT_SEVERITY_ERR | 0x0244)
#define CIFSTATUS_TIMER_RESOLUTION_NOT_SET		(NT_SEVERITY_ERR | 0x0245)
#define CIFSTATUS_CONNECTION_COUNT_LIMIT		(NT_SEVERITY_ERR | 0x0246)
#define CIFSTATUS_LOGIN_TIME_RESTRICTION		(NT_SEVERITY_ERR | 0x0247)
#define CIFSTATUS_LOGIN_WKSTA_RESTRICTION		(NT_SEVERITY_ERR | 0x0248)
#define CIFSTATUS_IMAGE_MP_UP_MISMATCH			(NT_SEVERITY_ERR | 0x0249)
#define CIFSTATUS_INSUFFICIENT_LOGON_INFO		(NT_SEVERITY_ERR | 0x0250)
#define CIFSTATUS_BAD_DLL_ENTRYPOINT			(NT_SEVERITY_ERR | 0x0251)
#define CIFSTATUS_BAD_SERVICE_ENTRYPOINT		(NT_SEVERITY_ERR | 0x0252)
#define CIFSTATUS_LPC_REPLY_LOST				(NT_SEVERITY_ERR | 0x0253)
#define CIFSTATUS_IP_ADDRESS_CONFLICT1			(NT_SEVERITY_ERR | 0x0254)
#define CIFSTATUS_IP_ADDRESS_CONFLICT2			(NT_SEVERITY_ERR | 0x0255)
#define CIFSTATUS_REGISTRY_QUOTA_LIMIT			(NT_SEVERITY_ERR | 0x0256)
#define CIFSTATUS_PATH_NOT_COVERED				(NT_SEVERITY_ERR | 0x0257)
#define CIFSTATUS_NO_CALLBACK_ACTIVE			(NT_SEVERITY_ERR | 0x0258)
#define CIFSTATUS_LICENSE_QUOTA_EXCEEDED		(NT_SEVERITY_ERR | 0x0259)
#define CIFSTATUS_PWD_TOO_SHORT					(NT_SEVERITY_ERR | 0x025a)
#define CIFSTATUS_PWD_TOO_RECENT				(NT_SEVERITY_ERR | 0x025b)
#define CIFSTATUS_PWD_HISTORY_CONFLICT			(NT_SEVERITY_ERR | 0x025c)
#define CIFSTATUS_PLUGPLAY_NO_DEVICE			(NT_SEVERITY_ERR | 0x025e)
#define CIFSTATUS_UNSUPPORTED_COMPRESSION		(NT_SEVERITY_ERR | 0x025f)
#define CIFSTATUS_INVALID_HW_PROFILE			(NT_SEVERITY_ERR | 0x0260)
#define CIFSTATUS_INVALID_PLUGPLAY_DEVICE_PATH	(NT_SEVERITY_ERR | 0x0261)
#define CIFSTATUS_DRIVER_ORDINAL_NOT_FOUND		(NT_SEVERITY_ERR | 0x0262)
#define CIFSTATUS_DRIVER_ENTRYPOINT_NOT_FOUND	(NT_SEVERITY_ERR | 0x0263)
#define CIFSTATUS_RESOURCE_NOT_OWNED			(NT_SEVERITY_ERR | 0x0264)
#define CIFSTATUS_TOO_MANY_LINKS				(NT_SEVERITY_ERR | 0x0265)
#define CIFSTATUS_QUOTA_LIST_INCONSISTENT		(NT_SEVERITY_ERR | 0x0266)
#define CIFSTATUS_FILE_IS_OFFLINE				(NT_SEVERITY_ERR | 0x0267)

//************************** Internal errors ********************************

#define CIFS_IERR_BAD_STR		(CIFS_ERROR_BASE - 1)
#define CIFS_IERR_STR_TOO_LONG	(CIFS_ERROR_BASE - 2)


//***************************************************************************

void
cifs_ntStatusToDosErrMapping(uint32 ntstatus, uchar8 *classp, ushort16 *errp);





#endif // #ifndef _CIFS_ERR_H

