/* vi:ts=4
 *-----------------------------------------------------------------
 *
 * Copyright (C) 2001, Agile Storage Inc.
 * 
 * RCS:  $Id: cifs-filereqs.h,v 1.1.1.1 2002/04/23 17:41:16 laurentb Exp $
 *
 * Name			: cifs-filereqs.h
 *
 * Description	: CIFS file requests to fs and fssf functions header.
 *
 * Created by	: Laurent Brard
 *
 * Date Created	: 03/19/2002
 *
 *-----------------------------------------------------------------
 */
#ifndef _CIFS_FILEREQS_H
#define _CIFS_FILEREQS_H


uint32	//CIFSTATUS
cifs_fsMCLReq(	cifs_operation  *opp,
				fs_Handle 	*startDirp,	//the directory to start from
				uchar8 		*smbp,		//source SMB message pointer
				uni16		*path,		//path
				fs_AttrType	attrType);	//attributes to request


uint32	//CIFSTATUS
cifs_fsMCLReqFromMsg(	cifs_operation  *opp,
						fs_Handle *startDirp,	//the directory to start from
						uchar8 *smbp,       	//source SMB message pointer
						uchar8 *spath,			//source path
						uni16  *elem,			//uni16 last element
						fs_AttrType attrType);	//attributes to request


uint32	//CIFSTATUS
cifs_fsMkdirReq(cifs_operation	*opp,
				uchar8			*smbp,       	//source SMB message pointer
				fs_Handle		*parentHandlep,	//handle of parent directory
				uni16			*dir,			//directory to create
				fs_Attr			*dirAttr,		//attributes of directory
				fs_AttrType		attrType);		//attributes to request back


uint32	//CIFSTATUS
cifs_fsReaddirplusReq(cifs_operation	*opp,
					 cifs_search	*searchp,
					 fs_AttrType	attrType,
					 uint32			maxBytes);


//**************************************************************************


uint32	//CIFSTATUS
cifs_fssfRemoveReq(	cifs_operation	*opp,
					uchar8			*smbp,     		//source SMB message pointer
					fs_Handle		*parentHandlep,	//handle of parent directory
					uni16			*file);			//file/directory to remove


uint32  //CIFSTATUS 
cifs_fssfCloseReq(  cifs_operation  *opp,
                    fssf_stateid_t  stateid);		//statid to close

uint32  //CIFSTATUS
cifs_fssfSetattrByIdReq(cifs_operation      *opp,
                        fssf_stateid_t      stateid,
                        fssf_attr_t         *attrp);



uint32	//CIFSTATUS
cifs_fssfGetAttrReqByFid(cifs_operation	*opp,
						cifs_fileOpen	*fop,
						uint64			bitmap);

uint32	//CIFSTATUS
cifs_fssfReadReq(cifs_operation	*opp,
				 cifs_fileOpen	*fop,
				 uint64			offset,
				 uint32			maxCnt);


//**************************************************************************


cifs_fileOpen *	//NULL if failed
cifs_openFile(	cifs_operation		*opp,
				fs_Handle			*parentHandlep,	//handle of parent directory
				uni16				*file,			//file to open
				fssf_action_t		openAction,
				fssf_reqAccess_t	reqAccess,		//see fssf-api.h
				fssf_shareMode_t	shareMode,
				uint32				openFlags,
				fs_Attr				*attrp,			//attrs of existing file if
													//exist, if not attrs for
													//the file to create.
													//if open succeeds, filled
													//with attrs of the open
													//file
				uint32				*createAction); //return fssf action



int32	//CIFSTATUS
cifs_readFile(	cifs_operation	*opp,
				cifs_fileOpen	*fop,
				uint64			offset,		// Offset from start of file
				uint32			maxCnt,		// Max number of bytes to read
				uint32			*dataCnt);	// Number of bytes read

int32	//CIFSTATUS 
cifs_writeFile(	cifs_operation	*opp,
				cifs_fileOpen	*fop,
				uchar8			*bp,			// Source data to be written
				uint64			offset,			// Offset from start of file
				uint32			dataLen,		// Number of bytes to write
				ushort16		writeMode,		// Write mode
				uint32			*writtenDataLen);	// Number of bytes written


#endif // #ifndef _CIFS_FILEREQS_H 

