/* vi:ts=4
 *-----------------------------------------------------------------
 *
 * Copyright (C) 2001, Agile Storage Inc.
 * 
 * RCS:  $Id: cifs-filetools.c,v 1.3 2002/12/10 18:22:20 laurentb Exp $
 *
 * Name			: cifs-filetools.c
 *
 * Description	: Contains CIFS file tools.
 *
 * Created by	: Laurent Brard
 *
 * Date Created	: 03/19/2002
 *
 *-----------------------------------------------------------------
 */

#include "cifs-api.h"
#include "cifs-err.h"
#include "cifs-string.h"
#include "cifs-utils.h"
#include "cifs-unicode.h"
#include "cifs-encdec.h"
#include "cifs-intmsg.h"
#include "cifs-filetools.h"

#define IFDBG(lvl)  if (lvl <= cifs_debugLevel)

//Unicode strings
//uni16 unistr_slash[] = { '/', 0 };
//uni16 unistr_dotSlash[] = { '.', '/', 0 };
//uni16 unistr_dotDotSlash[] = { '.', '.', '/', 0 };
uni16 unistr_backslash[] = { '\\', 0 };
uni16 unistr_dotBackslash[] = { '.', '\\', 0 };
uni16 unistr_dotDotBackslash[] = { '.', '.', '\\', 0 };
uni16 unistr_dotDot[] = { '.', '.', 0 };
uni16 unistr_com[] = { 'C', 'O', 'M', 0 };
uni16 unistr_dll[] = { 'D', 'L', 'L', 0 };
uni16 unistr_exe[] = { 'E', 'X', 'E', 0 };
uni16 unistr_sym[] = { 'S', 'Y', 'M', 0 };


fssf_action_t createDispo2fssfAction[SEC_FILE_CREATEDISPO_MAX] = {
	{	FSSF_IFEXIST_OVERWRITE,	FSSF_IFNOTEXIST_CREATE	},	//SUPERSEDE
	{	FSSF_IFEXIST_OPEN,		FSSF_IFNOTEXIST_FAIL	},	//OPEN
	{	FSSF_IFEXIST_FAIL,		FSSF_IFNOTEXIST_CREATE	},	//CREATE
	{	FSSF_IFEXIST_OPEN,		FSSF_IFNOTEXIST_CREATE	},	//OPEN_IF
	{	FSSF_IFEXIST_OVERWRITE,	FSSF_IFNOTEXIST_FAIL	},	//OVERWRITE
	{	FSSF_IFEXIST_OVERWRITE,	FSSF_IFNOTEXIST_CREATE	}	//OVERWRITE_IF
};

sec_accessMask	openMode2desiredAccess[16] = {
	SEC_GENR_FILE_READ,							//Read
	SEC_GENR_FILE_WRITE,							//Write
	SEC_GENR_FILE_READ|SEC_GENR_FILE_WRITE,		//Read & Write
	SEC_GENR_FILE_EXECUTE,							//execute
	-1,	-1, -1, -1, -1, -1, -1, -1, -1, -1, -1,		//Illegal
	SEC_GENR_FILE_ALL								//FCB
};

fssf_shareMode_t openMode2shareAccess[8] = {
	FSSF_SHARE_RW,									//DOS compatibility mode
	FSSF_ACCESS_NONE,								//Deny All
	FSSF_ACCESS_READ,								//Deny Write
	FSSF_ACCESS_WRITE,								//Deny Read
	FSSF_SHARE_RW,									//Deny None
	FSSF_ACCESS_NONE,								//Illegal
	FSSF_ACCESS_NONE,								//Illegal
	FSSF_ACCESS_NONE,								//FCB mode
};

uint16 openFunc2openActionIfExists[4] = {
	FSSF_IFEXIST_FAIL,
	FSSF_IFEXIST_OPEN,
	FSSF_IFEXIST_OVERWRITE,
	FSSF_IFEXIST_FAIL
};

uint32 fssfAction2createAction[8] = {
	0,
	SEC_FILE_WAS_OPENED,
	SEC_FILE_WAS_CREATED,
	SEC_FILE_WAS_CREATED,
	SEC_FILE_WAS_OVERWRITTEN,
	SEC_FILE_WAS_OVERWRITTEN,
	SEC_FILE_WAS_OVERWRITTEN,
	SEC_FILE_WAS_OVERWRITTEN
};



/*-----------------------------------------------------------------------------
 * Name			: cifs_decodeFileHandle
 *
 * Description	: Decode fs_Handle
 *
 * Created by	: Xiaobing Zhang
 *
 * Date Created	: 03/22/2002
 *-----------------------------------------------------------------------------
 */
void
cifs_decodeFileHandle(fs_Handle *fsHandle)
{
	printf("\tFile handle: volId    = 0x%llx\n", fsHandle->volId.whole_thing);
	printf("\t             inum     = 0x%llx\n", fsHandle->inum);
	printf("\t             gen      = 0x%x\n", fsHandle->gen);
	printf("\t             mkfsTime = 0x%x\n", fsHandle->mkfsTime);
	printf("\t             mntPnt   = 0x%llx\n", fsHandle->mntPnt);
}



/*-----------------------------------------------------------------
 * Name			: cifs_getUserAccessToFile
 *
 * Description	: Get access that a user has to the file
 *
 * Created by	: Xiaobing Zhang
 *
 * Date Created	: 03/25/2002
 *-----------------------------------------------------------------
 */
void
cifs_getUserAccessToFile(cifs_operation *opp,
						uchar8			*access,	//Access flag upon return
						fs_Attr			*fsAttrp)
{
	sec_cifsSd			*sdp;	//File security descriptor
	sec_cifsAccessToken	*atp;	//User access token
	sec_accessMask		accGiven;

	CIFS_PRINT(CIFS_DBGLV_WARN, (">>> cifs_getUserAccessToFile is not completely implemented yet!!!"));

	//TODO:
	//sdp =
	//atp =

	cifs_secCheckAccess(sdp, atp, SEC_MAXIMUM_ALLOWED_ACCESS, &accGiven);

	//TODO
} /* cifs_getUserAccessToFile() */

/*-----------------------------------------------------------------
 * Name			: cifs_getFileNameByFid
 *
 * Description	: Get a file's name given SMB volatile file handle
 *
 * Created by	: Xiaobing Zhang
 *
 * Date Created	: 03/25/2002
 *-----------------------------------------------------------------
 */
int32			//CIFSTATUS
cifs_getFileNameByFid(cifs_operation *opp,
						 ushort16		fid,
						 uni16			*fileName)
{
	cifs_fileOpen	*fop;

	fop = cifs_findFileOpenByFid(opp->siup, fid);
	if (fop != NULL)
	{
		unistrcpy(fileName, fop->pathName);
		return (CIFSTATUS_OK);
	}

	return (CIFSTATUS_UNSUCCESSFUL);
} /* cifs_getFileNameByFid() */

/*-----------------------------------------------------------------
 * Name			: cifs_fsGetAttrByType
 *
 * Description	: Get a file's attribute from FS given attribute type
 *
 * Created by	: Xiaobing Zhang
 *
 * Date Created	: 03/27/2002
 *-----------------------------------------------------------------
 */
fs_Attr *
cifs_fsGetAttrByType(fs_Attr *attrp, int32 numAttrs, int32 searchAttrType)
{
	int32		attrIndex;
	fs_Attr		*currAttrp;

	if ((numAttrs > CIFS_MAX_NUM_OF_ATTR) || (attrp == NULL))
		return (NULL);

	attrIndex = 0;
	currAttrp = attrp;
	while (attrIndex < numAttrs)
	{
		if (currAttrp->attrType == searchAttrType)
			return (currAttrp);
		attrIndex += 1;
		currAttrp += 1;
	}
	return (NULL);
} /* cifs_fsGetAttrByType() */

/*-----------------------------------------------------------------
 * Name			: cifs_putShortName
 *
 * Description	: Put short (8.3) name a message buffer
 *
 * Created by	: Xiaobing Zhang
 *
 * Date Created	: 03/28/2002
 *-----------------------------------------------------------------
 */
int32			//Number of bytes filled, or -1 if there is error
cifs_putShortName(uchar8	*dstp,	//Destination string in UNICODE or ASCII
				  uni16		*srcp,	//Source string in UNICODE
				  int32		maxLen)	//Max len in bytes of destination string
{
	//This feature is currently not supported. Put 0 in the buffer.
	int32	i;

	for(i = 0; i < maxLen; i++)
	{
		PCAR(dstp, i, 0);
	}
	return (maxLen);
} /* cifs_putShortName() */

/*-----------------------------------------------------------------
 * Name			: cifs_writeMode2StableHow
 *
 * Description	: Convert write mode to filesystem's stableHow
 *
 * Created by	: Xiaobing Zhang
 *
 * Date Created	: 04/15/2002
 *
 *-----------------------------------------------------------------
 */
fs_WriteStableHow
cifs_writeMode2StableHow(ushort16	writeMode)
{
	if (writeMode & CIFS_WRITEMODE_WRITE_THROUGH)
		return (FS_WRITE_STABLE_DATA_SYNC);
	else
		return (FS_WRITE_STABLE_UNSTABLE);
}


//369 years * (365.25 days * 24 hours = 8766) * 3600 seconds - corrections
#define TIME_DIFF_1601_1970	((369.0*8766*3600) - (3.0*24*3600 + 6.0*3600))

/*-----------------------------------------------------------------
 * Name			: cifs_fs2LongTime
 *
 * Description	: Convert time from fs_Time format to TIME format
 * 				  Note: TIME is specified in units of 100ns
 *
 * Created by	: Laurent Brard
 *
 * Date Created	: 04/04/2002
 *-----------------------------------------------------------------
 */
int64
cifs_fs2LongTime(fs_Time *fsTimep)
{
	int64	longTime;
	
	if (fsTimep->secs == 0)
		return (0);

	if (fsTimep->secs == 0xffffffff)
		return (-1);
	
	longTime	= fsTimep->secs;
	//TODO take into account Daylight savings adjustment
	longTime += TIME_DIFF_1601_1970;
	longTime *= 1000000;
	longTime += fsTimep->usecs;
	longTime *= 10;

	return (longTime);
}


/*-----------------------------------------------------------------
 * Name			: cifs_fillLongTime
 *
 * Description	: fill TIME field in SMB message from fs_Time value
 *
 * Created by	: Laurent Brard
 *
 * Date Created	: 04/04/2002
 *-----------------------------------------------------------------
 */
void
cifs_fillLongTime(	uchar8	*smbTimep,
					fs_Time	*fsTimep)
{
	int64	longTime;
	
	longTime = cifs_fs2LongTime(fsTimep);

	PUTLE8S(smbTimep, 0, longTime);
}


/*-----------------------------------------------------------------
 * Name			: cifs_fs2SmbDateTime
 *
 * Description	: Convert time from fs_Time format to SMB_DATE and 
 * 				  SMB_TIME format
 *
 * Created by	: Laurent Brard
 *
 * Date Created	: 04/04/2002
 *-----------------------------------------------------------------
 */
void
cifs_fs2SmbDateTime(uchar8	*msgBuf,		//Ptr to SMB time
					fs_Time	*fsTimep)
{
	//TODO....
	CIFS_PRINT(CIFS_DBGLV_WARN, (">>> cifs_fs2SmbDateTime is not implemented !!!"));
	PUTLE4(msgBuf, 0, 0x78563412);
} /* cifs_fs2SmbDateTime() */


/*-----------------------------------------------------------------
 * Name			: cifs_fillSmbFileAttr
 *
 * Description	: Fill SMB file attributes
 *
 * Created by	: Laurent Brard
 *
 * Date Created	: 04/04/2002
 *-----------------------------------------------------------------
 */
void
cifs_fillSmbFileAttr(uchar8		*smbAttrsp,
					fs_Attr		*fsAttrp,
					boolean		extAttributes)
{
	uint32		attrs;

	attrs = (fsAttrp->simpleAttrs & FS_INODE_SATTR_CIFSMASK);
	if (fsAttrp->type == FS_FTYPE_DIR)	//Make sure that directory bit is set
		attrs |= SEC_FILE_ATTRIBUTE_DIRECTORY;
	if (attrs == 0)
		attrs |= SEC_FILE_ATTRIBUTE_NORMAL;
	
	if (extAttributes)
		PUTLE4(smbAttrsp, 0, attrs);
	else
		PUTLE2(smbAttrsp, 0, attrs);
}


/*-----------------------------------------------------------------
 * Name			: cifs_extractPathLastElement
 *
 * Description	: extract the last element of the path
 * 				  if removeIt is TRUE, remove last element from path
 *
 * Created by	: Laurent Brard
 *
 * Date Created	: 03/20/2002
 *
 *-----------------------------------------------------------------
 */
uint32	//CIFSTATUS
cifs_extractPathLastElement(uni16 *path,
							uni16 *lastElem,
							uint32 lastElemSize,
							boolean removeIt)
{
	uni16 *p;
	uint32 rc, len;

	rc = CIFSTATUS_OK;
	p = unistrrchr(path, '\\', &len);
	if (p == NULL)
	{	//Only one element in the path
		if (lastElemSize > len)
			unistrcpy(lastElem, path);
		else {
			*lastElem = 0;
			rc = CIFSTATUS_NAME_TOO_LONG;
		}
		if (removeIt)
			*path = 0;
	} else {
		 if (lastElemSize > len + (path - p))
			unistrcpy(lastElem, p+1);
		 else {
			*lastElem = 0;
			rc = CIFSTATUS_NAME_TOO_LONG;
		}
		if (removeIt)
			*p = 0;
	}
	return (rc);
}

/*-----------------------------------------------------------------
 * Name			: cifs_containsWildcard
 *
 * Description	: return TRUE if the string contains wildcard
 * 				  NOTE: can be used with dos codepage or utf8 strings
 *
 * Created by	: Laurent Brard
 *
 * Date Created	: 03/21/2002
 *
 *-----------------------------------------------------------------
 */
boolean
cifs_containsWildcard(char *s)
{
	char c;

	while((c = *s++))
	{
		switch (c)
		{
			case '*': case '?': case '<': case '>': case '"':
				return (TRUE);
		}
	}
	return (FALSE);
}

/*-----------------------------------------------------------------
 * Name			: cifs_uniContainsWildcard
 *
 * Description	: return TRUE if the string contains wildcard
 *
 * Created by	: Laurent Brard
 *
 * Date Created	: 03/19/2002
 *
 *-----------------------------------------------------------------
 */
boolean
cifs_uniContainsWildcard(uni16 *s)
{
	uni16 c;

	while((c = *s++))
	{
		switch (c)
		{
			case '*': case '?': case '<': case '>': case '"':
				return (TRUE);
		}
	}
	return (FALSE);
}


/*-----------------------------------------------------------------
 * Name			: cifs_uniIsExecutable
 *
 * Description	: return TRUE if the file is a Windows executable
 *
 * Created by	: Laurent Brard
 *
 * Date Created	: 04/02/2002
 *
 *-----------------------------------------------------------------
 */
boolean
cifs_uniIsExecutable(uni16 *file)
{

	file = unistrrchr(file, '.', NULL);
	if (file)
	{
		file++;
		if (unistrcasecmp(file, unistr_com) == 0
		 || unistrcasecmp(file, unistr_dll) == 0
		 || unistrcasecmp(file, unistr_exe) == 0
		 || unistrcasecmp(file, unistr_sym) == 0)
			return (TRUE);
	}
	
	return (FALSE);
}


/*-----------------------------------------------------------------
 * Name			: cifs_checkPathSyntax
 *
 * Description	: return TRUE if the string has a valid path syntax
 *
 * Created by	: 
 *
 * Date Created	: 03/23/2002
 *
 *-----------------------------------------------------------------
 */
int32			//CIFSTATUS
cifs_checkPathSyntax(uni16 *s)
{
	//TODO....
	CIFS_PRINT(CIFS_DBGLV_WARN, (">>> cifs_checkPathSyntax is not implemented !!!"));	

	return (CIFSTATUS_OK);
}


/*-----------------------------------------------------------------
 * Name			: cifs_Smb2UniPath
 *
 * Description	: convert an SMB path name to an unicode path name 
 * 				  in accordance with internal path rules
 *
 * Created by	: Laurent Brard
 *
 * Date Created	: 03/26/2002
 *
 *-----------------------------------------------------------------
 */
uint32	//CIFSTATUS
cifs_Smb2UniPath(	uchar8 *smbp,		//source SMB message pointer
					uchar8 *spath,		//source path
					uni16  *dpath)		//destination path
{
	uint32 rc;
		
	if (!cifs_getSmbUniStr(dpath, spath, CIFS_S_PATHSTR, smbp))
		return (CIFSTATUS_NAME_TOO_LONG);
		
	IFDBG(CIFS_DBGLV_DBG3) {
		uniprintf("cifs_Smb2InternalPath original path:\n", dpath);
		printf("\n");
	}
	
	//We have a unicode path relative to the share, clean

	if (unistrstr(dpath, unistr_dotDotBackslash) != NULL)
	{
		CIFS_PRINT(CIFS_DBGLV_ERR, ("cifs_Smb2InternalPath: forbidden path"));
		return (CIFSTATUS_OBJECT_PATH_INVALID);
	}

	cutString(dpath, unistr_dotBackslash, NULL);
//Try like this and see if OK
//    cutString(dpath, unistr_backslash, unistr_backslash);
		
	IFDBG(CIFS_DBGLV_DBG3) {
		uniprintf("after cleaning:", dpath);
		printf("\n");
	}

	return (CIFSTATUS_OK);
}


/*-----------------------------------------------------------------
 * Name			: cifs_waitResp
 *
 * Description	: wait for a response from another module
 *
 * Created by	: Laurent Brard
 *
 * Date Created	: 03/20/2002
 *
 *-----------------------------------------------------------------
 */
uint32  //CIFSTATUS_OK if event requested, event received otherwise
cifs_waitResp(uint32	 	expectedEvent,
				void		**parpp,	//pointer on the msg response pointer
				eee_desc_t	**eDescpp,	//pointer on the eee desc pointer
				boolean 	takeCareOfKillEvent,
				uint32		timeout)
{
	esm_event_t evt;

	thread_yield(&evt, timeout);

	IFDBG(CIFS_DBGLV_DBG2)
	{
		do {
			if (evt.event >= FS_MSG_BASE
			 && evt.event < FS_MSG_BASE + FS_REQ_MAX) 
			{
				CIFS_PRINT(CIFS_DBGLV_DBG2, 
						("cifs_waitResp: rec FS event %d, fsStatus %d",
						evt.event, ((fs_MsgRespHdr*)(evt.p2.u.p))->fsStatus));
				break;
			}
			if (evt.event >= FSSF_MSG_BASE
			 && evt.event < FSSF_MSG_BASE + 40)
			{
				CIFS_PRINT(CIFS_DBGLV_DBG2, 
						("cifs_waitResp: rec FSSF event %d, status %d",
						evt.event, ((fssf_closeRsp_t*)(evt.p2.u.p))->status));
				break;
			}
			CIFS_PRINT(CIFS_DBGLV_DBG2, 
						("cifs_waitResp: rec event %d", evt.event));
		}while(0);
	}
	
	if (takeCareOfKillEvent)
		cifs_treatKillEvent(&evt);
	
	if (evt.event == expectedEvent)
	{
		*eDescpp = (eee_desc_t*)evt.p1.u.p;
		*parpp	 = evt.p2.u.p;
		return (CIFSTATUS_OK);
	}

	//This should never happen, see if needed
	if (evt.event != THREAD_EVENT_TIMEOUT || evt.event != CIFS_THREADEVT_KILL)
		eee_freePacket((eee_desc_t*)evt.p1.u.p);

	return(evt.event);
}


/*-----------------------------------------------------------------
 * Name			: cifs_fsToCifsErrMapping
 *
 * Description	: convert FS error codes to NT Status and Dos errors
 *
 * Created by	: Laurent Brard
 *
 * Date Created	: 03/21/2002
 *
 *-----------------------------------------------------------------
 */
uint32	//CIFSTATUS
cifs_fsToCifsErrMapping(fs_Status fsStatus, uchar8 *classp, ushort16 *errp)
{
	
	switch (fsStatus)	{
		case FS_ERR_NONE:		return (CIFSTATUS_OK);
		case FS_ERR_NOENT:		return (CIFSTATUS_NO_SUCH_FILE);
		case FS_ERR_PERM:
		case FS_ERR_ROFS:
		case FS_ERR_ACCESS:		return (CIFSTATUS_ACCESS_DENIED);
		case FS_ERR_BADPATH:	return (CIFSTATUS_OBJECT_PATH_NOT_FOUND);
		case FS_ERR_EXIST:		*classp = CIFS_ERRDOS;
								*errp   = CIFSERR_filexists;
								return (CIFSTATUS_ACCESS_DENIED);
		case FS_ERR_XDEV:		return (CIFSTATUS_NOT_SAME_DEVICE);
		case FS_ERR_INVAL:		return (CIFSTATUS_INVALID_HANDLE);
		case FS_ERR_NOSPC:
		case FS_ERR_DQUOT:		return (CIFSTATUS_DISK_FULL);
		case FS_ERR_NAMETOOLONG:return (CIFSTATUS_NAME_TOO_LONG);
		case FS_ERR_NOTEMPTY:	return (CIFSTATUS_DIRECTORY_NOT_EMPTY);
		case FS_ERR_BADHANDLE:	
		case FS_ERR_STALE:
								return (CIFSTATUS_OBJECT_PATH_NOT_FOUND);
		case FSSF_ERR_SHARE_DENIED:
								return (CIFSTATUS_SHARING_VIOLATION);
		case FSSF_ERR_RESOURCE:	return (CIFSTATUS_NO_MEMORY);

		default:				return (CIFSTATUS_UNSUCCESSFUL); 
	}
	
	/*
{ ENOTDIR, ERRDOS, ERRbadpath, NT_STATUS_NOT_A_DIRECTORY },
{ EIO, ERRHRD, ERRgeneral, NT_STATUS_IO_DEVICE_ERROR },
{ EBADF, ERRSRV, ERRsrverror, NT_STATUS_INVALID_HANDLE },
{ EINVAL, ERRSRV, ERRsrverror, NT_STATUS_INVALID_HANDLE },
{ ENFILE, ERRDOS, ERRnofids, NT_STATUS_TOO_MANY_OPENED_FILES },
{ EMFILE, ERRDOS, ERRnofids, NT_STATUS_TOO_MANY_OPENED_FILES },
 */
}

/*-----------------------------------------------------------------
 * Name			: cifs_returnFsError
 *
 * Description	: set the SMB error field from FS error codes
 *
 * Created by	: Laurent Brard
 *
 * Date Created	: 03/21/2002
 *
 *-----------------------------------------------------------------
 */
void
cifs_returnFsError(cifs_operation *opp, fs_Status fsStatus)
{
	uchar8 class = 0;
	ushort16 err = 0;
	uint32 NTstatus;
			
	if (fsStatus != FS_ERR_NONE)	
	{
		NTstatus = cifs_fsToCifsErrMapping(fsStatus, &class, &err);
		cifs_errResp(opp, class, err, NTstatus);
	}
}




/*-----------------------------------------------------------------
 * Name			: cifs_buildFssfClientId
 *
 * Description	: build the Fssf ClientId associated with this operation
 *
 * Created by	: Laurent Brard
 *
 * Date Created	: 03/31/2002
 *
 *-----------------------------------------------------------------
 */
fssf_clientid_t
cifs_buildFssfClientId(cifs_operation *opp)
{
	fssf_clientid_t clientId;
	
	clientId = opp->pid;
	clientId <<= 32;
	clientId += (address_t)opp->clp;

	return (clientId);
}



