/* vi:ts=4
 *-----------------------------------------------------------------
 *
 * Copyright (C) 2001, Agile Storage Inc.
 *
 * RCS:  $Id: cifs-rap.h,v 1.1.1.1 2002/04/23 17:41:16 laurentb Exp $
 *
 * Name			: cifs-rap.h
 *
 * Description	: Contains CIFS server API request (RAP) related definitions. 
 *
 * Created by	: Xiaobing Zhang
 *
 * Date Created	: 01/03/2002
 *
 *-----------------------------------------------------------------
 */
#ifndef _CIFS_RAP_H
#define _CIFS_RAP_H

#define CIFS_RAP_S_BUF					4096

// MSRAP command type
#define CIFS_RAP_CMD_NETSHAREENUM		0
#define CIFS_RAP_CMD_NETSHAREGETINFO	1
#define CIFS_RAP_CMD_NETSERVERGETINFO	13

// Share permission
#define CIFS_RAP_SHARE_ACCESS_READ		0x01 // Read & execute from resource
#define CIFS_RAP_SHARE_ACCESS_WRITE		0x02 // write data to resource
#define CIFS_RAP_SHARE_ACCESS_CREATE	0x04 // Create an instance of resource
#define CIFS_RAP_SHARE_ACCESS_EXEC		0x08 // Execute from resource
#define CIFS_RAP_SHARE_ACCESS_DELETE	0x10 // Delete resource
#define CIFS_RAP_SHARE_ACCESS_ATRIB		0x20 // Modify resource attribute
#define CIFS_RAP_SHARE_ACCESS_PERM		0x40 // Change permissions on resource
#define CIFS_RAP_SHARE_ACCESS_ALL		0x7F // All of the above permissions

// Function prototypes
int32 cifs_rapCheckShareInfoLevel(ushort16 level, char *datDescP);

int32 cifs_rapCheckServerInfoLevel(ushort16 level, char *datDescP);

int32 cifs_rapGetShareInfoSize(	cifs_share *shareP,
								ushort16 level,
								uint32 *fixedLen,
								uint32 *strLen);

int32 cifs_rapFillShareInfo(cifs_share *shareP,
							cifs_server *serverP,
							ushort16 level,
							char **baseP,
							char **fixedBufP,
							uint32 *totalFixedLen,
							char **strBufP,
							uint32 *totalStrLen);

int32 cifs_rapNetShareEnum(	cifs_operation *opP,
							uchar8 *parP,
							uchar8 *datP,
							ushort16 pbsCnt,
							ushort16 dbsCnt,
							char **oparP,
							uint32 *oparLen,
							char **odatP,
							uint32 *odatLen);

int32 cifs_rapNetShareGetInfo(cifs_operation *opP,
								uchar8 *parP,
								uchar8 *datP,
								ushort16 pbsCnt,
								ushort16 dbsCnt,
								char **oparP,
								uint32 *oparLen,
								char **odatP,
								uint32 *odatLen);
int32 cifs_rapNetServerGetInfo(	cifs_operation *opP,
								uchar8 *parP,
								uchar8 *datP,
								ushort16 pbsCnt,
								ushort16 dbsCnt,
								char **oparP,
								uint32 *oparLen,
								char **odatP,
								uint32 *odatLen);

int32 cifs_msrap(cifs_operation *opp,
				uchar8 *parp,
				uchar8 *datp,
				ushort16 pbsCnt,
				ushort16 dbsCnt);

#endif /* _CIFS_RAP_H */

