/* vi:ts=4
 *-----------------------------------------------------------------
 *
 * Copyright (C) 2001, Agile Storage Inc.
 * 
 * RCS:  $Id: cifs-sec.h,v 1.4 2002/12/10 18:22:20 laurentb Exp $
 *
 * Name			: cifs-sec.h
 *
 * Description	: Contains CIFS security definitions.
 *
 * Created by	: Laurent Brard
 *
 * Date Created	: 01/21/2002
 *
 *-----------------------------------------------------------------
 */
#ifndef _CIFS_SEC_H
#define _CIFS_SEC_H

#include <exports/sec-cifstruct.h>


//******************************** Security API *****************************


void cifs_secSidSprintf(char *str, sec_cifsSid *sidp);


void cifs_secSidPrintf(sec_cifsSid *sidp);


void
cifs_secFreeSecDesc(sec_cifsSd *sdp, boolean dynSd);


sec_cifsAce *	//The cloned ACEs
cifs_secAcesClone(sec_cifsAcl *aclp);


int32    //NFX_OK or NFX_ERR
cifs_secSecDescCopy(sec_cifsSd *dsdp, sec_cifsSd *ssdp);


sec_cifsSd *	//The cloned security descriptor
cifs_secSecDescClone(sec_cifsSd *sdp);


void
cifs_secSidCpy(sec_cifsSid *dsidp, sec_cifsSid *ssidp);


int32	//NFX_OK or NFX_ERR 
cifs_secMergeSidAndRid(sec_cifsSid *dsidp, sec_cifsSid *ssidp, uint32 rid);


boolean	//TRUE if found 
cifs_secFindRidInCreds(cifsauthCred_t *credsp, uint32 rid);


int32 	//0 if equal
cifs_secSidCmp(sec_cifsSid *sidp1, sec_cifsSid *sidp2);


int32 	//0 if equal
cifs_secAceCmp(sec_cifsAce *acep1, sec_cifsAce *acep2);


int32 	//0 if equal
cifs_secAclCmp(sec_cifsAcl *aclp1, sec_cifsAcl *aclp2);


int32 	//0 if equal
cifs_secSecDescCmp(sec_cifsSd *sdp1, sec_cifsSd *sdp2);


void
cifs_secMapGenericAccess(sec_accessMask *accessp, sec_accessGenericMap *mapp);


void
cifs_secSDMapGenericAccessBits(sec_cifsSd *sdp, sec_accessGenericMap *mapp);


sec_cifsAccessToken *	//Access token created
cifs_secCreateAccessToken(cifsauthCred_t *credsp);

void
cifs_secDeleteAccessToken(sec_cifsAccessToken *atp);
		

uint32	//CIFSTATUS
cifs_secCheckAccess(sec_cifsSd *sdp,
					sec_cifsAccessToken *atp, 
					sec_accessMask accRequested,
					sec_accessMask *accGivenp);


#define cifs_secCheckShareAccess(siup, accReq)			\
	(((siup->access & accReq)==accReq) ? CIFSTATUS_OK : CIFSTATUS_ACCESS_DENIED)
				

			
#endif // #ifndef _CIFS_SEC_H 

