/* vi:ts=4
 *-----------------------------------------------------------------
 *
 * Copyright (C) 2001, Agile Storage Inc.
 * 
 * RCS:  $Id: cifs-transac2.h,v 1.1.1.1 2002/04/23 17:41:16 laurentb Exp $
 *
 * Name			: cifs-transac2.h
 *
 * Description	: Contains SMB_COM_TRANSACTION2 related definitions.
 *
 * Created by	: Xiaobing Zhang
 *
 * Date Created	: 01/30/2002
 *
 *-----------------------------------------------------------------
 */
#ifndef _CIFS_TRANSAC2_H
#define _CIFS_TRANSAC2_H

#define CIFS_S_TRANS2_SUBCMD			0x12	// # of Transacion2 subcommands

// SMB_COM_TRANSACTION2 subcommand codes
#define CIFS_TRANS2_OPEN2				0x00
#define CIFS_TRANS2_FIND_FIRST2			0x01
#define CIFS_TRANS2_FIND_NEXT2			0x02
#define CIFS_TRANS2_QUERY_FS_INFO		0x03
#define CIFS_TRANS2_SET_FS_INFO			0x04	// Currently reserved
#define CIFS_TRANS2_QUERY_PATH_INFO		0x05
#define CIFS_TRANS2_SET_PATH_INFO		0x06
#define CIFS_TRANS2_QUERY_FILE_INFO		0x07
#define CIFS_TRANS2_SET_FILE_INFO		0x08
#define CIFS_TRANS2_FSCTL				0x09	// Not implemented by NT server
#define CIFS_TRANS2_IOCTL2				0x0A	// Not implemented by NT server
#define CIFS_TRANS2_FIND_NOTIFY_FIRST	0x0B	// Not implemented by NT server
#define CIFS_TRANS2_FIND_NOTIFY_NEXT	0x0C	// Not implemented by NT server
#define CIFS_TRANS2_CREATE_DIR			0x0D
#define CIFS_TRANS2_SESSION_SETUP		0x0E
#define CIFS_TRANS2_GET_DFS_REFERRAL	0x10
#define CIFS_TRANS2_REPORT_DFS_INCONSISTENCY	0x11

// Data structures
typedef struct cifs_transac2 {
	uint32			flags;
#define CIFS_TRANS2_FLAG_VALID          0x0001
#define CIFS_TRANS2_FLAG_CLOSE_TID      0x0002
#define CIFS_TRANS2_FLAG_NO_ANSWER      0x0004
#define CIFS_TRANS2_FLAG_MULTI_MSG      0x0008
#define CIFS_TRANS2_FLAG_LAST_MSG       0x0010
#define CIFS_TRANS2_FIND_FIRST			0x0020
#define CIFS_TRANS2_FIND_NEXT			0x0040
	ushort16		subCmd;
	//ushort16		fid;			// SMB file handle
	cifs_fileOpen	*fop;			// NULL if there is no related fileOpen 
	ushort16		infoLevel;
	uint32			maxReplySize;
	cifs_search		*searchp;		// Valid for 'find first' or 'find next'
	uchar8			*pathNamep;		// Pointer to path name in req message

	ushort16		maxParaCnt;		// Max param bytes in transaction answer
	ushort16		maxDataCnt;		// Max data bytes in transaction answer
	ushort16		maxSetupCnt;	// Max setup words in transaction answer
	ushort16		tpbsCnt;		// Total # of param bytes being sent
	ushort16		tdbsCnt;		// Total # of data bytes being sent
	ushort16		rpbsCnt;		// Remaining # of param bytes being sent
	ushort16		rdbsCnt;		// Remaining # of data bytes being sent
	ushort16		swsCnt;			// # of setup words being sent
	ushort16		*swp;			// Pointer to setup words being sent
	uchar8			*parap;			// Pointer to paremeters being sent
	uchar8			*datap;			// Pointer to data being sent
	uchar8			*internalDatap;	// Pointer to buffer that store data to
									// be sent in multiple messages
} cifs_transac2;

// Function declarations
boolean cifstrans2_replyInit(cifs_operation *opp, cifs_transac2 *transacp);

void cifstrans2_reply(cifs_operation *opp, cifs_transac2 *transacp);

int32 cifstrans2_open2(cifs_operation	*opp,
						cifs_transac2	*transacp,
						uchar8			*parp,
						ushort16		pbsCnt,
						uchar8			*datp,
						ushort16		dbsCnt);

int32 cifstrans2_replyOpen2(cifs_operation	*opp,
							cifs_transac2	*transacp /* ... */);

int32 cifstrans2_findFirst2(cifs_operation	*opp,
							cifs_transac2	*transacp,
							uchar8			*parp,
							ushort16		pbsCnt,
							uchar8			*datp,
							ushort16		dbsCnt);

int32 cifstrans2_findNext2(cifs_operation	*opp,
							cifs_transac2	*transacp,
							uchar8			*parp,
							ushort16		pbsCnt,
							uchar8			*datp,
							ushort16		dbsCnt);

int32 cifstrans2_replyFind(cifs_operation			*opp,
							cifs_transac2			*transacp,
							fs_Attr					*fsAttrp,
							fs_MsgReadDirPlusResp	*dirInfop,
							uni16					*resumeFileName,
							boolean					*eos);

int32 cifstrans2_queryFsInfo(cifs_operation	*opp,
							cifs_transac2	*transacp,
							uchar8			*parp,
							ushort16		pbsCnt,
							uchar8			*datp,
							ushort16		dbsCnt);

int32 cifstrans2_replyQueryFsInfo(cifs_operation	*opp,
								  cifs_transac2		*transacp /* ... */);

int32 cifstrans2_queryPathInfo(cifs_operation	*opp,
								cifs_transac2	*transacp,
								uchar8			*parp,
								ushort16		pbsCnt,
								uchar8			*datp,
								ushort16		dbsCnt);

int32 cifstrans2_queryFileInfo(cifs_operation	*opp,
								cifs_transac2	*transacp,
								uchar8			*parp,
								ushort16		pbsCnt,
								uchar8			*datp,
								ushort16		dbsCnt);

int32 cifstrans2_replyQueryPathFileInfo(cifs_operation	*opp,
										cifs_transac2	*transacp,
										fs_Attr			*fsAttrp);

int32 cifstrans2_setPathInfo(cifs_operation	*opp,
							cifs_transac2	*transacp,
							uchar8			*parp,
							ushort16		pbsCnt,
							uchar8			*datp,
							ushort16		dbsCnt);

int32 cifstrans2_replySetPathInfo(cifs_operation	*opp,
								  cifs_transac2		*transacp /* ... */);

int32 cifstrans2_setFileInfo(cifs_operation	*opp,
							cifs_transac2	*transacp,
							uchar8			*parp,
							ushort16		pbsCnt,
							uchar8			*datp,
							ushort16		dbsCnt);

int32 cifstrans2_replySetFileInfo(cifs_operation	*opp,
								  cifs_transac2		*transacp /* ... */);

#endif /* #ifndef _CIFS_TRANSAC2_H */

