/* vi:ts=4
 *-----------------------------------------------------------------
 *
 * Copyright (C) 2001, Agile Storage Inc.
 * 
 * RCS:  $Id: cifs-unicode.h,v 1.1.1.1 2002/04/23 17:41:16 laurentb Exp $
 *
 * Name			: cifs-unicode.h
 *
 * Description	: Contains CIFS unicode functions.
 *
 * Created by	: Laurent Brard
 *
 * Date Created	: 11/07/2001
 *
 *-----------------------------------------------------------------
 */

#ifndef _CIFS_UNICODE_H
#define _CIFS_UNICODE_H

#define	UNICHR_BACKSLASH		0x5C
#define	UNICHR_SLASH			0x2F

#define	unisizeof(obj)	(sizeof(obj)/sizeof(uni16))

//Initialize Dos codepage to unicode and unicode to dos conversion tables		
void cifs_initDosUnicodeMapping(void);		

//Convert a string in current dos codepage to a MS unicode string
//If zeroEnd=TRUE, adds zero character at the end of the destination string
uchar8*	//pointer on address after the SOURCE string ending 0, NULL if error
strDos2Uni(uni16 *dstp, const char *srcp, ushort16 maxDestLen, boolean zeroEnd);

//Convert a string in unicode to a current dos codepage string
uchar8*	//pointer on address after the DESTINATION string ending 0, NULL if err
strUni2Dos(char *dstp, uni16 *srcp, ushort16 maxDestLen);

//unicode strcpy
uni16 *unistrcpy(uni16 *dstp, uni16 *srcp);

//unicode strncpy
uni16 *unistrncpy(uni16 *dstp, uni16 *srcp, ushort16 n);
		
//unicode strlen
ushort16 unistrlen(uni16 *s);

//size in bytes of a unicode string including ending 0
ushort16 unistrsize(uni16 *s);

//return a ptr to c in s, return NUL if not found
uni16 *unistrchr(uni16 *s, uni16 c);

//unicode strstr
uni16 *unistrstr(uni16 *s, uni16 *subs);

//string comparison, return < 0, 0, or > 0
int32 unistrcmp(uni16 *s1, uni16 *s2);

//string comparison of at most n characters, return < 0, 0, or > 0
int32 unistrncmp(uni16 *s1, uni16 *s2, ushort16 n);

//case insensitive string comparison, return < 0, 0, or > 0
int32 unistrcasecmp(uni16 *s1, uni16 *s2);

//case insensitive string comp. of at most n characters. Return < 0, 0, or > 0
int32 unistrncasecmp(uni16 *s1, uni16 *s2, ushort16 n);

//replace all occurence of a character in the string by another
void uniCharReplace(uni16 *s, uni16 charToReplace, uni16 replacementChar);

//returns a pointer to the last occurrence of the character c in the string s
//Give the string length back if requested
uni16 *unistrrchr(uni16 *s, uni16 c, uint32 *slen);

//print a unicode string with an optional comment in front
void uniprintf(char *comment, uni16* unis);


int32			// NFX_OK or CIFS internal error
strUtf82Uni(uni16		*destp,		// UNICODE string
			const char	*srcp,		// UTF-8 string
			uint32		srcLen,		// Number of bytes of UTF-8 string
			uint32		*dstLen,	// Number of chars of UNICODE string
			uint32		maxDstLen,	// Maximum number of bytes of UNICODE string
			boolean		zeroEnd);	// Put 0 at the end of UNICODE string


int32			// NFX_OK or CIFS internal error
strUni2Utf8(uchar8		*destp,		// UTF-8 string
			uni16		*srcp,		// UNICODE string
			uint32		srcLen,		// UNICODE string length (in characters)
			uint32		*dstLen,	// Number of bytes of UTF-8 string
			uint32		maxDstLen);	// Maximum number of bytes of UTF-8 string

#endif //_CIFS_UNICODE_H



