/* vi:ts=4
 *-----------------------------------------------------------------
 *
 * Copyright (C) 2001, Agile Storage Inc.
 * 
 * RCS:  $Id: cifs.h,v 1.3 2002/12/10 18:22:20 laurentb Exp $
 *
 * Name			: cifs.h
 *
 * Description	: Contains CIFS general definitions.
 *
 * Created by	: Laurent Brard
 *
 * Date Created	: 10/31/2001
 *
 *-----------------------------------------------------------------
 */
#ifndef _CIFS_H
#define _CIFS_H


// Release version of product that CIFS software is part of
#define CIFS_OS					"EEE"
#define CIFS_LANMAN_TYPE		"NFX 2.0"
#define CIFS_VERSION_MAJOR		2
#define CIFS_VERSION_MINOR		0

//SMB commands
#define SMB_COM_MKDIR			0x00	// create directory
#define SMB_COM_RMDIR			0x01	// delete directory
#define SMB_COM_OPEN			0x02	// open file
#define SMB_COM_CREATE			0x03	// create file
#define SMB_COM_CLOSE			0x04	// close file
#define SMB_COM_FLUSH			0x05	// flush file
#define SMB_COM_DELETE			0x06	// delete file
#define SMB_COM_RENAME			0x07	// rename file
#define SMB_COM_QUERYINFO		0x08	// get file information
#define SMB_COM_SETINFO			0x09	// set file information
#define SMB_COM_READ			0x0A	// read from file
#define SMB_COM_WRITE			0x0B	// write to file
#define SMB_COM_LOCKBYTERANGE	0x0C	// lock byte range
#define SMB_COM_UNLOCKBYTERANGE	0x0D	// unlock byte range
#define SMB_COM_CREATETEMP		0x0E	// create temporary file
#define SMB_COM_CREATENEW		0x0F	// make new file
#define SMB_COM_CHECKDIRECTORY	0x10	// check directory path
#define SMB_COM_PROCESSEXIT		0x11	// process exit
#define SMB_COM_SEEK			0x12	// seek
#define SMB_COM_LOCKANDREAD		0x13	// lock a byte range and read
#define SMB_COM_WRITEANDUNLOCK	0x14	// unlock a range then write
#define SMB_COM_READRAW			0x1A	// read block raw
#define SMB_COM_READMPX			0x1B	// read block multiplexed
#define SMB_COM_READMPXSEC		0x1C	// read block multiplexed secondary
#define SMB_COM_WRITERAW		0x1D	// write block raw
#define SMB_COM_WRITEMPX		0x1E	// write block multiplexed
#define SMB_COM_WRITEMPXSEC		0x1F	// write block multiplexed secondary
#define SMB_COM_WRITECOMPLETE	0x20	// write complete response
#define SMB_COM_QUERYSERVER		0x21	// query server
#define SMB_COM_SETINFO2		0x22	// set file information level 2
#define SMB_COM_GETINFO2		0x23	// get file information level 2
#define SMB_COM_LOCKINGANDX		0x24	// locking and X
#define SMB_COM_TRANSACTION		0x25	// transaction
#define SMB_COM_TRANSACTIONSEC	0x26	// transaction (sec. request/response)
#define SMB_COM_IOCTL			0x27	// ioctl
#define SMB_COM_IOCTLSEC		0x28	// ioctl (secondary request/response)
#define SMB_COM_COPY			0x29	// copy
#define SMB_COM_MOVE			0x2A	// move
#define SMB_COM_ECHO			0x2B	// echo
#define SMB_COM_WRITEANDCLOSE	0x2C	// write in file and close it
#define SMB_COM_OPENANDX		0x2D	// open and X
#define SMB_COM_READANDX		0x2E	// read and X
#define SMB_COM_WRITEANDX		0x2F	// write and X
#define CMS_COM_NEWFILESIZE		0x30	// new file size
#define CMS_COM_CLOSEANDTREEDIS	0x31	// close file and tree disconnect
#define SMB_COM_TRANSACTION2	0x32	// transaction2 functions
#define SMB_COM_TRANSACTION2SEC	0x33	// transaction2 functions, sec. cmd
#define SMB_COM_FINDCLOSE2		0x34	// close a search operation
#define SMB_COM_FINDNOTIFCLOSE	0x35	// close TRANSACT2_FINDNOTIFYFIRST
#define SMB_COM_TREECONNECT		0x70	// tree connect
#define SMB_COM_TREEDISCONNECT	0x71	// tree disconnect
#define SMB_COM_NEGOTIATE		0x72	// negotiate protocol
#define SMB_COM_SESSETUPANDX	0x73	// Session Setup & X
#define SMB_COM_LOGOFFANDX		0x74	// user logoff
#define SMB_COM_TREECONNECTANDX	0x75	// tree connect and X
#define SMB_COM_QUERYINFODISK	0x80	// get disk attributes
#define SMB_COM_SEARCH			0x81	// search directory
#define SMB_COM_FIND			0x82	// find first
#define SMB_COM_FINDUNIQUE		0x83	// find unique
#define SMB_COM_FINDCLOSE		0x84	// find close
#define SMB_COM_NTTRANSACT		0xA0	// NT transaction
#define SMB_COM_NTTRANSACTSEC	0xA1	// NT transaction secondary
#define SMB_COM_NTCREATEANDX	0xA2	// NT create and X
#define SMB_COM_NTCANCEL		0xA4	// NT cancel
#define SMB_COM_INVALID			0xFE	// invalid command

//SMB Flags
#define SMB_FLGS_RESPONSE		0x80
#define SMB_FLGS_PATHCASELESS	0x08
#define SMB_FLGS_CANONICPATH	0x10

//SMB Flags2
#define SMB_FLG2_LONG_NAMES		0x0001
#define SMB_FLG2_EXTENDATT		0x0002
#define SMB_FLG2_EXTENDSEC		0x0800
#define SMB_FLG2_ERR_STATUS		0x4000
#define SMB_FLG2_UNICODE		0x8000

//Capabilities
#define CAP_RAW_MODE            0x0001
#define CAP_MPX_MODE            0x0002
#define CAP_UNICODE             0x0004
#define CAP_LARGE_FILES         0x0008
#define CAP_NT_SMBS             0x0010
#define CAP_RPC_REMOTE_APIS     0x0020
#define CAP_STATUS32            0x0040
#define CAP_LEVEL_II_OPLOCKS    0x0080
#define CAP_LOCK_AND_READ       0x0100
#define CAP_NT_FIND             0x0200
#define CAP_DFS                 0x1000
#define CAP_W2K_SMBS            0x2000
#define CAP_LARGE_READX         0x4000
#define CAP_LARGE_WRITEX        0x8000
#define CAP_EXTENDED_SECURITY   0x80000000

//Security Mode
#define CIFS_SEC_USER			0x1
#define CIFS_SEC_ENCRYPTPWD		0x2

//Share optional support (Tree connect and X)
#define SMB_SUPPORT_SEARCH_BITS	0x0001
#define SMB_SHARE_IN_DFS		0x0002

// Share type
#define CIFS_SHARE_TYPE_DISK	0   // Disk
#define CIFS_SHARE_TYPE_PRINTQ	1   // Printer queue
#define CIFS_SHARE_TYPE_DEVICE	2   // Communication device
#define CIFS_SHARE_TYPE_IPC		3   // Inter process communication (IPC)
#define CIFS_SHARE_HIDDEN		0x80000000 

//Flags field
#define CIFS_REQUEST_OPLOCK			0x2
#define CIFS_REQUEST_BATCH_OPLOCK	0x4
#define CIFS_OPEN_DIRECTORY			0x8

//ShareAccess field
#define SEC_FILE_SHARE_NONE		0x0
#define SEC_FILE_SHARE_READ		0x1
#define SEC_FILE_SHARE_WRITE		0x2
#define SEC_FILE_SHARE_DELETE		0x4

//FileAttributesField
#define SEC_FILE_ATTRIBUTE_READONLY	FS_INODE_SATTR_READ_ONLY	
#define SEC_FILE_ATTRIBUTE_HIDDEN		FS_INODE_SATTR_HIDDEN
#define SEC_FILE_ATTRIBUTE_SYSTEM		FS_INODE_SATTR_SYSTEM
#define SEC_FILE_ATTRIBUTE_DIRECTORY	FS_INODE_SATTR_DIR
#define SEC_FILE_ATTRIBUTE_ARCHIVE		FS_INODE_SATTR_ARCH
#define SEC_FILE_ATTRIBUTE_NORMAL		FS_INODE_SATTR_NORMAL
#define SEC_FILE_ATTRIBUTE_TEMPORARY	FS_INODE_SATTR_TEMP
#define SEC_FILE_ATTRIBUTE_COMPRESSED	FS_INODE_SATTR_COMPRES
#define SEC_FILE_ATTRIBUTE_MASK		FS_INODE_SATTR_CIFSMASK
/*
//Flags - combined with attributes
#define SEC_FILE_FLAG_WRITE_THROUGH    0x80000000L
#define SEC_FILE_FLAG_NO_BUFFERING     0x20000000L
#define SEC_FILE_FLAG_RANDOM_ACCESS    0x10000000L
#define SEC_FILE_FLAG_SEQUENTIAL_SCAN  0x08000000L
#define SEC_FILE_FLAG_DELETE_ON_CLOSE  0x04000000L
#define SEC_FILE_FLAG_BACKUP			0x02000000L
#define SEC_FILE_FLAG_POSIX_SEMANTICS  0x01000000L
*/
//CreateDisposition field
#define SEC_FILE_SUPERSEDE				0
#define SEC_FILE_OPEN					1
#define SEC_FILE_CREATE				2
#define SEC_FILE_OPEN_IF				3
#define SEC_FILE_OVERWRITE				4
#define SEC_FILE_OVERWRITE_IF			5
#define SEC_FILE_CREATEDISPO_MAX		6

//CreateOptions field
#define SEC_FILE_DIRECTORY_FILE		0x0001
#define SEC_FILE_WRITE_THROUGH			0x0002
#define SEC_FILE_SEQUENTIAL_ONLY		0x0004
#define SEC_FILE_NON_DIRECTORY_FILE	0x0040
#define SEC_FILE_NO_EA_KNOWLEDGE		0x0200
#define SEC_FILE_EIGHT_DOT_THREE_ONLY	0x0400
#define SEC_FILE_RANDOM_ACCESS			0x0800
#define SEC_FILE_DELETE_ON_CLOSE		0x1000

//Responses when opening a file
#define SEC_FILE_WAS_OPENED			1
#define SEC_FILE_WAS_CREATED			2
#define SEC_FILE_WAS_OVERWRITTEN		3

//File type flags
#define SEC_FILE_TYPE_DISK				0
#define SEC_FILE_TYPE_BYTE_MODE_PIPE	1
#define SEC_FILE_TYPE_MSG_MODE_PIPE	2
#define SEC_FILE_TYPE_PRINTER			3
#define SEC_FILE_TYPE_COMM_DEVICE		4
#define SEC_FILE_TYPE_UNKNOWN			0xFFFF

//Write mode
#define CIFS_WRITEMODE_WRITE_THROUGH	0x0001

//Flag for NT transact rename call
#define RENAME_REPLACE_IF_EXISTS 1

//Filesystem Attributes
#define SEC_FILE_CASE_SENSITIVE_SEARCH		0x0001
#define SEC_FILE_CASE_PRESERVED_NAMES		0x0002
#define SEC_FILE_UNICODE_ON_DISK			0x0004
#define SEC_FILE_PERSISTENT_ACLS			0x0008
#define SEC_FILE_FILE_COMPRESSION			0x0010
#define SEC_FILE_VOLUME_QUOTAS				0x0020
#define SEC_FILE_DEVICE_IS_MOUNTED			0x0020
#define SEC_FILE_VOLUME_IS_COMPRESSED		0x8000

//ChangeNotify flags
#define SEC_FILE_NOTIFY_CHANGE_FILE        0x001
#define SEC_FILE_NOTIFY_CHANGE_DIR_NAME    0x002
#define SEC_FILE_NOTIFY_CHANGE_ATTRIBUTES  0x004
#define SEC_FILE_NOTIFY_CHANGE_SIZE        0x008
#define SEC_FILE_NOTIFY_CHANGE_LAST_WRITE  0x010
#define SEC_FILE_NOTIFY_CHANGE_LAST_ACCESS 0x020
#define SEC_FILE_NOTIFY_CHANGE_CREATION    0x040
#define SEC_FILE_NOTIFY_CHANGE_EA          0x080
#define SEC_FILE_NOTIFY_CHANGE_SECURITY    0x100
#define SEC_FILE_NOTIFY_CHANGE_FILE_NAME   0x200

//File system information levels
#define CIFS_FSINFOLV_ALLOCATION			1
#define CIFS_FSINFOLV_VOLUME				2
#define CIFS_FSINFOLV_QUERY_VOLUME			0x102
#define CIFS_FSINFOLV_QUERY_SIZE			0x103
#define CIFS_FSINFOLV_QUERY_DEVICE			0x104
#define CIFS_FSINFOLV_QUERY_ATTR			0x105

//File information levels
#define CIFS_INFOLV_STANDARD				1
#define CIFS_INFOLV_QUERY_EA_SIZE			2
#define CIFS_INFOLV_QUERY_EAS_FROM_LIST		3
#define CIFS_INFOLV_QUERY_ALL_EAS			4
#define CIFS_INFOLV_IS_NAME_VALID			6

#define CIFS_INFOLV_QUERY_FIEL_BASIC		0x101
#define CIFS_INFOLV_QUERY_FILE_STANDARD		0x102
#define CIFS_INFOLV_QUERY_FILE_EA			0x103
#define CIFS_INFOLV_QUERY_FILE_NAME			0x104
#define CIFS_INFOLV_QUERY_FILE_ALL			0x107
#define CIFS_INFOLV_QUERY_FILE_ALT_NAME		0x108
#define CIFS_INFOLV_QUERY_FILE_STREAM		0x109
#define CIFS_INFOLV_QUERY_FILE_COMPRESSION	0x10B

#define CIFS_INFOLV_FIND_FILE_DIR			0x101
#define CIFS_INFOLV_FIND_FILE_FULL_DIR		0x102
#define CIFS_INFOLV_FIND_FILE_NAMES			0x103
#define CIFS_INFOLV_FIND_FILE_BOTH_DIR		0x104
#define CIFS_INFOLV_FIND_FILE_UNIX			0x202

#define CIFS_INFOLV_SET_BASIC				0x101
#define CIFS_INFOLV_SET_DISPOSITION			0x102
#define CIFS_INFOLV_SET_ALLOCATION			0x103
#define CIFS_INFOLV_SET_END_OF_FILE			0x104
#define CIFS_INFOLV_SET_UNIX_BASIC			0x200
#define CIFS_INFOLV_SET_UNIX_LINK			0x201
#define CIFS_INFOLV_SET_UNIX_HLINK			0x203

//CIFS server types		
#define CIFS_SRV_TYPE_WORKSTATION			0x00000001
#define CIFS_SRV_TYPE_SERVER				0x00000002
#define CIFS_SRV_TYPE_SQLSERVER				0x00000004
#define CIFS_SRV_TYPE_DOMAIN_CTRL			0x00000008
#define CIFS_SRV_TYPE_DOMAIN_BAKCTRL		0x00000010
#define CIFS_SRV_TYPE_TIME_SOURCE			0x00000020
#define CIFS_SRV_TYPE_AFP					0x00000040
#define CIFS_SRV_TYPE_NOVELL				0x00000080
#define CIFS_SRV_TYPE_DOMAIN_MEMBER			0x00000100
#define CIFS_SRV_TYPE_PRINTQ_SERVER			0x00000200
#define CIFS_SRV_TYPE_DIALIN_SERVER			0x00000400
#define CIFS_SRV_TYPE_SERVER_UNIX			0x00000800
#define CIFS_SRV_TYPE_NT					0x00001000
#define CIFS_SRV_TYPE_WFW					0x00002000
#define CIFS_SRV_TYPE_SERVER_MFPN			0x00004000
#define CIFS_SRV_TYPE_SERVER_NT				0x00008000
#define CIFS_SRV_TYPE_POTENTIAL_BROWSER		0x00010000
#define CIFS_SRV_TYPE_BACKUP_BROWSER		0x00020000
#define CIFS_SRV_TYPE_MASTER_BROWSER		0x00040000
#define CIFS_SRV_TYPE_DOMAIN_MASTER			0x00080000
#define CIFS_SRV_TYPE_SERVER_OSF			0x00100000
#define CIFS_SRV_TYPE_SERVER_VMS			0x00200000
#define CIFS_SRV_TYPE_WIN95_PLUS			0x00400000
#define CIFS_SRV_TYPE_DFS_SERVER			0x00800000
#define CIFS_SRV_TYPE_ALTERNATE_XPORT		0x20000000  
#define CIFS_SRV_TYPE_LOCAL_LIST_ONLY		0x40000000  
#define CIFS_SRV_TYPE_DOMAIN_ENUM			0x80000000

#define CIFS_SERVER_TYPE (CIFS_SRV_TYPE_SERVER | CIFS_SRV_TYPE_SERVER_NT)		
#define CIFS_SERVER_VERSION_MAJOR	4	//We act as Windows NT 4.0 for now
#define CIFS_SERVER_VERSION_MINOR	0


//Maximum number of CIFS objects
/*
#define CIFS_N_SERVERS		255			//Maximum number of CIFS servers 
#define	CIFS_N_CLIENTS		4096		//Maximum number of CIFS clients
*/
#define CIFS_N_SERVERS		2			//Maximum number of CIFS servers 
#define	CIFS_N_CLIENTS		2			//Maximum number of CIFS clients

#define CIFS_N_SHARES		(CIFS_N_SERVERS*8)
#define CIFS_N_SESSIONS		(CIFS_N_CLIENTS*2)
#define CIFS_N_SHARESINUSE	(CIFS_N_CLIENTS*3)
#define CIFS_N_OPERATIONS	(CIFS_N_CLIENTS*5)
#define CIFS_N_PIPES		(CIFS_N_CLIENTS*8)
#define CIFS_N_FILESOPEN	(CIFS_N_CLIENTS*16)
#define CIFS_N_SEARCHES		(CIFS_N_CLIENTS)

#define CIFS_N_MPXREQS			50		//Max nb of multiplexed requests
#define CIFS_N_VCS				1		//Max nb of virtual circuits
#define CIFS_N_SHAREINITS		1//10	//Max nb of simultaneous share inits

#define CIFS_N_ANYSIZE_ARRAY	1

#define	CIFS_N_FS_RETRIES	3		//Max nb of FS send retries
#define	CIFS_T_FS_RETRIES	2		//time in seconds between FS send retries

//NB: All C string lengths are mentioned without the ending 0 character 
#define CIFS_S_ENCRYPTKEY		8
#define	CIFS_S_SERVERCOMMENT	48
#define CIFS_S_SHARENAME		12
#define	CIFS_S_SHARECOMMENT		256
#define CIFS_S_PATHSTR			(FS_PATH_MAX-1)	
#define CIFS_S_FILESTR			(FS_NAME_MAX-1)
#define CIFS_S_PASSWD			14
#define CIFS_S_ENCRYPTPWD		24
#define CIFS_S_USERNAME			63
#define CIFS_S_DOMAINNAME		63
#define CIFS_S_PIPENAME			6
#define CIFS_S_BUFFER			(PKT_BUFF_SIZE - 64)

#define CIFS_S_MINFRAGMENT		128		//Minimum fragment size
//Ensure that the first msg fragment has always at least that amount of data
//IF NOT we defragment.


// Linked definitions pre-definitions
struct cifs_client;
struct cifs_session;
struct cifs_shareInUse;
struct cifs_operation;

/**********************************************************************
  TODO: ADD STATISTICS COUNTERS AT SERVER LEVEL 
      - transmit / receive
	  - message type counter
	  - any possible errors
 **********************************************************************/
/*********************************************************************
   TODO: PADDING WILL HAVE TO BE DONE ONCE STRUCTURES ARE STABILIZED     
 *********************************************************************/

/*
NB: The following structures are managed by the RX CPU.
    TX CPU only has read access to them.
*/

// ****************** Server Configuration Definitions ******************** 

// CIFS server configuration 
typedef struct cifs_serverCfg {
  uint32 ipAddr;						// ip address 
  uint32 subnetMask;					// subnet mask 
  uint32 winsSrvIpAddr;					// WINS server IP address. 
										// 0 if not configured 
  uint32 options;						// server options (other than capabi.)
#define CIFS_SVCFG_ENCRYPTPWD	0x00000001
/* 
define oplocksEnable
define rawMode ???
*/
  uint64 capabilities;					// server capabilities (negociate cmd)
 
  char	domainName[NB_S_NAME+1];		// domain we are part of 
  uni16 uniDomainName[NB_S_NAME+1];		// the above domain name in unicode
  char	name[NB_S_NAME+1];				// our NetBIOS name 
  uni16	uniName[NB_S_NAME+1];			// our name in unicode
  uni16	comment[CIFS_S_SERVERCOMMENT+1];
  char	trustAccountPwd[CIFS_S_PASSWD+1];// workstation trust account passwd 
} cifs_serverCfg;


// CIFS share configuration 
typedef struct cifs_shareCfg {
  uni16		name[CIFS_S_SHARENAME+1]; 
  uni16		comment[CIFS_S_SHARECOMMENT+1]; 
  uni16		pathName[CIFS_S_PATHSTR+1];		// encode in utf8 ? 
  uint32	flags;
#define CIFS_SHCFG_IPC			0x00000001	//IPC share
#define CIFS_SHCFG_ADMIN		0x00000002	//ADMIN share
#define CIFS_SHCFG_ISADMIN		0x00000003	//Is administrative (IPC or ADMIN)

  // Security descriptor 
  // If NULL, assume full access at share level
  sec_cifsSd	*secDescp;
  
} cifs_shareCfg;


// **************** Server instances related definitions ****************** 

typedef struct cifs_share {
  cifs_shareCfg		cfg;				// Share configuration 
  fs_Handle			handle;				// File handle of the share
  uint32			useCnt;				// # of current connections to the share

  esm_handle_t		threadId;			// id of the associated thread

  DLL_ENTRY(struct cifs_share) entry;
} cifs_share;

typedef struct cifs_server {
  cifs_serverCfg	cfg;				// Server configuration 
//  uchar8			serverId;			// Used internally by CIFS module
  uint32			shareCnt;			// Number of shares in the share list
  DLL_HDR(cifs_share) shares;			// Server's list of shares 
  nb_srvCB_t		*transportCBp;		// Lower layer server control block 
  uint32			flags;				
#define CIFS_SERVER_NAMEREGISTERED		0x00000001  
#define CIFS_SERVER_SHARESINITIALIZED   0x00000002
#define CIFS_SERVER_INITCOMPLETIONMASK  0x00000003

  DLL_HDR(struct cifs_client) clients;	// Clients currently connected 
  
  DLL_ENTRY(struct cifs_server) entry;
} cifs_server;


// **************** Client sessions related definitions ******************* 

/* A cifs_client structure is associated with each TCP connection 
   It is created during protocol negotiation. 

   A cifs_session structure is created when a user completes a session setup
   procedure with the server. It is identified via a unique Uid which is 
   created by the server.
   There may be multiple sessions created on one client.

   A cifs_shareInUse structure is created when a user connects to a new
   share. It is identified via a unique Tid which is created by the server. 
   A user's session can be connected to multiple shares.
*/


// CIFS client platforms 
typedef enum {
  CIFS_CLIENT_UNKNOWN, CIFS_CLIENT_WIN9x, CIFS_CLIENT_WINNT, CIFS_CLIENT_WIN2K 
} cifs_platform;

// CIFS dialects 
typedef enum {
  CIFS_DIALECT_NT1, CIFS_DIALECT_LAN2, CIFS_DIALECT_LAN1, CIFS_DIALECT_CORE2,
  CIFS_DIALECT_CORE1 
} cifs_dialect;

// Share types 
typedef enum { CIFS_SHARE_IPC, CIFS_SHARE_DISK } cifs_shareType;


// ***************************** Client ***********************************
typedef struct cifs_client {
  nb_ssnCB_t	*transportCBp;			// Lower layer client control block
  cifs_server	*serverp;				// Associated CIFS server 
  ushort16		clientId;				// Used internally by CIFS module
		  
//uint32		clientIpAddr;			// Client's IP address 
//char		 	clientName[NB_S_NAME+1];// Client's name 
  cifs_platform	clientPlatform;			// Client's platform 
  cifs_dialect	dialect;				// Negociated dialect 
  uint64		capabilities;			// remote capabilities 
  ushort16		maxBufSize;				// remote maximum buffer size 
//  uint16		maxBufSize;				// local maximum buffer size 
//  uint32		maxRawSize;				// local maximum raw buffer size 
  uint32		sessionKey;				// Always 0 ? 
  ushort16		uidCnt;					// UID counter
  ushort16		tidCnt;					// TID counter
  uchar8 		encryptKey[CIFS_S_ENCRYPTKEY]; // Used for challenge/Resp 
  DLL_HDR(struct cifs_session) sessions;// Sessions set up on this connection
  DLL_HDR(struct cifs_operation) waitOutBufOps;// ops waiting for an outg buffer

  DLL_ENTRY(struct cifs_client) entry;
} cifs_client;


// ***************************** Session **********************************
typedef struct cifs_session {
  uint32 options;						// session options 
#define CIFS_SESS_ENCRYPTPWD	0x00000001
#define CIFS_SESS_ANONYMOUS		0x00000002
#define CIFS_SESS_GUEST			0x00000004
  ushort16	uid;						// associated user id (smb header) 
//  ushort16	maxMpxReqs;					// max simultanuous requests 
  uint16	caseInsenPwdLen;			// case insentitive account password 
  uint16	caseSenPwdLen;				// case sensitive password (Unicode)

  cifsauthCred_t 	 userCreds;			// user credentials, from DC
  sec_cifsAccessToken *accessTokenp;		// user access token
  
  DLL_HDR(struct cifs_shareInUse) sharesInUse;// share connected by treeCon SMB 
  DLL_HDR(struct cifs_operation) pendingOps;// pending operations
 
  DLL_ENTRY(struct cifs_session) entry; 
  
  uni16		user[CIFS_S_USERNAME+1];	// user connected, empty if anonymous 
  uni16		domain[CIFS_S_DOMAINNAME+1];// user domain
  uchar8	caseInsenPwd[CIFS_S_ENCRYPTPWD+1];
  uchar8	caseSenPwd[CIFS_S_ENCRYPTPWD+1];
} cifs_session;


// **************************** fileOpen **********************************
typedef struct cifs_fileOpen {
//  uint32		pid;					// process id:pid+pidHigh (smb header)
  ushort16			fid;				// SMB file handle 
  ushort16			flags;	
#define	CIFS_FOP_ISDIR		0x0001
  fssf_stateid_t	stateid;			// internal file handle
  uni16				*pathName;
  DLL_ENTRY(struct cifs_fileOpen) entry; 
} cifs_fileOpen;


// ***************************** search ***********************************
typedef struct cifs_search {
  ushort16	findHandle;					// associated smb find handle 
  fs_Handle	parentHandle;				// parent handle
  uint64	cookieVerif;				// Obtained in fs_ReadDirPlusResp.
  										// Init value is 0.
  ushort16	count;						// max num of entries to return 
  ushort16	flags;						// additional info 
#define CIFS_FIND_CLOSE_AFTER_THIS	0x0001
#define CIFS_FIND_CLOSE_IF_END		0x0002
#define CIFS_FIND_RET_RESUME_KEY	0x0004
#define CIFS_FIND_FROM_PREV_END		0x0008
#define CIFS_FIND_BACKUP_INTENT		0x0010
  ushort16	attributes;					// Archive, Directory, System, Hidden
  ushort16	infoLevel;					// information level 
  uint32	cookie;						// number of files found so far ???
										//  (by analyzing trace) 
  uni16		pattern[CIFS_S_FILESTR+1];	// Search pattern used for filtering
										// It is last component in path name
										// in UNICODE format
  uni16		lastFileName[CIFS_S_FILESTR+1];	// File name of last entry in the
											// find response 
  DLL_ENTRY(struct cifs_search) entry; 
} cifs_search;


// ******************************* pipe ***********************************
typedef struct cifs_pipe {
  ushort16	fid;						// associated file id
  uint32    maxDataBytes;               // Max data bytes in transaction answer
  struct cifs_operation *opp;			// current operation 	  
  union {
	cifsrpc_pipe	rpc;				//DCE/RPC pipe
//	cifs_rapPipe	rap;				//RAP pipe - NOT USED TODAY
  }u;

  DLL_ENTRY(struct cifs_pipe) entry; 
}cifs_pipe;


// ***************************** shareInUse *******************************
typedef struct cifs_shareDisk {
  DLL_HDR(cifs_fileOpen)	openFiles;	// open files on this share 
  DLL_HDR(cifs_search)		searches;	// current searches 
}cifs_shareDisk;

typedef struct cifs_sharePipe {
  DLL_HDR(cifs_pipe) openPipes;			// open pipes in this ipc share
}cifs_sharePipe;

typedef struct cifs_shareInUse {
  ushort16			tid;				// associated tree id (smb header) 
  ushort16			flags;
#define CIFS_SIU_ISIPC		0x8000		// is an IPC share
/*
#define CIFS_SIU_RDAC		0x0001  	// read access allowed
#define CIFS_SIU_WRAC		0x0002  	// write access allowed
#define CIFS_SIU_FULLAC		0x0003	  	// full access allowed
*/
  sec_accessMask	access;				// share access granted
  
  cifs_share		*sharep;			// share configuration and statistics
  union {
	cifs_shareDisk	disk;
	cifs_sharePipe	pipe;
  }u;									  
  ushort16			fidCnt;				// fid counter
  DLL_ENTRY(struct cifs_shareInUse) entry; 
} cifs_shareInUse;


// ****************************** Operation ******************************
typedef struct {
  pkt_desc_t        *tplPktDescp;		// TPL packet descriptor pointer 
  uchar8			*smbp;				// SMB message header pointer 
  uint32			smbLen;				// SMB message length
  uint32			pad;
  uchar8			*cmdTypep;			// SMB command type pointer 
  uchar8			*cmdp;				// SMB command pointer 
} cifs_cmdDesc;

typedef enum 
{ cifs_opstatus_idle,
  cifs_opstatus_waitOutBuffer,
  cifs_opstatus_waitOplock,
  cifs_opstatus_processing,		// operation is being processed
  cifs_opstatus_sendPending,	// partial response sent, wait for available 
  								// xmt buf from tpl to send remaining response
  cifs_opstatus_sending,
  cifs_opstatus_lastSent,
  cifs_opstatus_canceled		// operation data structure is no longer valid
} cifs_opstatus;

typedef struct cifs_operation {
  cifs_cmdDesc		reqMsg;				// request message descriptor 
  cifs_cmdDesc		respMsg;			// response message descriptor 
  uint32			pid;				// process id:pid+pidHigh (smb header)
  ushort16			mid;				// multiplex id (smb header)
  ushort16			status;				// cifs_opstatus 
  ushort16			flags;
#define CIFS_OPFLAG_MORE_TO_SEND	0x01
#define CIFS_OPFLAG_READ_REQ		0x02
  cifs_client		*clp;				// associated client 
  cifs_session		*ssp;				// associated user's session 
  cifs_shareInUse	*siup;				// associated connected share 
  esm_handle_t		threadId;			// id of the associated thread
  uint64			fsPrivateForCod;	// Used in TX_COMPLETE message to FS

  DLL_ENTRY(struct cifs_operation) entry;
} cifs_operation;


// ************************** Global definitions ************************ 

// CIFS object Lists 
typedef struct cifs_objLists {
  nfp_objList	serverList;
  nfp_objList	shareList;		
  nfp_objList	clientList;		
  nfp_objList	sessionList;		
  nfp_objList	shareInUseList;		
  nfp_objList	operationList;		
  nfp_objList	pipeList;
  nfp_objList	fileOpenList;		
  nfp_objList	searchList;		
} cifs_objLists;

#define cifs_allocServer() \
		(cifs_server*)nfp_allocObject(&cifs_objectsp->serverList, TRUE)
#define cifs_allocShare() \
		(cifs_share*)nfp_allocObject(&cifs_objectsp->shareList, TRUE)
#define cifs_allocClient() \
		(cifs_client*)nfp_allocObject(&cifs_objectsp->clientList, TRUE)
#define cifs_allocSession() \
		(cifs_session*)nfp_allocObject(&cifs_objectsp->sessionList, TRUE)
#define cifs_allocShareInUse() \
		(cifs_shareInUse*)nfp_allocObject(&cifs_objectsp->shareInUseList, TRUE)
#define cifs_allocOperation() \
		(cifs_operation*)nfp_allocObject(&cifs_objectsp->operationList, TRUE)
#define cifs_allocPipe() \
		(cifs_pipe*)nfp_allocObject(&cifs_objectsp->pipeList, FALSE)
#define cifs_allocFileOpen() \
		(cifs_fileOpen*)nfp_allocObject(&cifs_objectsp->fileOpenList, TRUE)
#define cifs_allocSearch() \
		(cifs_search*)nfp_allocObject(&cifs_objectsp->searchList, TRUE)

#define cifs_freeServer(s)	nfp_deallocObject(&cifs_objectsp->serverList, s)
#define cifs_freeShare(s)	nfp_deallocObject(&cifs_objectsp->shareList, s)
#define cifs_freeClient(c)	nfp_deallocObject(&cifs_objectsp->clientList, c)
#define cifs_freeSession(s)	nfp_deallocObject(&cifs_objectsp->sessionList, s)
#define cifs_freeShareInUse(s) \
		nfp_deallocObject(&cifs_objectsp->shareInUseList, s)
#define cifs_freeOperation(o)  \
		nfp_deallocObject(&cifs_objectsp->operationList, o)
#define cifs_freePipe(p)	nfp_deallocObject(&cifs_objectsp->pipeList, p)
#define cifs_freeFileOpen(f) nfp_deallocObject(&cifs_objectsp->fileOpenList, f)
#define cifs_freeSearch(s)	nfp_deallocObject(&cifs_objectsp->searchList, s)
		
#define cifs_validateServer(s) \
		nfp_validateObject(&cifs_objectsp->serverList, s)		
#define cifs_validateShare(s) \
		nfp_validateObject(&cifs_objectsp->shareList, s)		
#define cifs_validateClient(c) \
		nfp_validateObject(&cifs_objectsp->clientList, c)		
#define cifs_validateSession(s) \
		nfp_validateObject(&cifs_objectsp->sessionList, s)		
#define cifs_validateShareInUse(s) \
		nfp_validateObject(&cifs_objectsp->shareInUseList, s)		
#define cifs_validateOperation(o) \
		nfp_validateObject(&cifs_objectsp->operationList, o)		
#define cifs_validatePipe(p) \
		nfp_validateObject(&cifs_objectsp->pipeList, p)		
#define cifs_validateFileOpen(f) \
		nfp_validateObject(&cifs_objectsp->fileOpenList, f)		
#define cifs_validateSearch(s) \
		nfp_validateObject(&cifs_objectsp->searchList, s)		
		

// CIFS global configuration 
typedef struct cifs_globalCfg {
  int32				eee_cifsAppId;		// CIFS server EEE application ID 
  //int32				eee_cifsauthAppId;	// cifsauth EEE application ID
  //int32				eee_efsAppId;		// Filesystem EEE application ID
  //int32				eee_evmAppId;		// Volume manager EEE application ID
  //int32				eee_fssfAppId;		// FS stateful EEE application ID
  uint32			mgmtSrcPort;		// Management source port
  uint32			ctrlSrcPort;		// Control source port
  uint32			cifsauthDstPort;	// cifsaut destination port
  uint32			evmDstPort;			// Volume manager destination port
  uint32			efsDstPort;			// FS destination port
  uint32			fssfDstPort;		// FSSF destination port
  uchar8			serverCounter;		// Server counter
  DLL_HDR(cifs_server) servers;			// List of CIFS servers
//  cifs_server		*serversp;			// List of CIFS servers 
}cifs_globalCfg;

extern cifs_globalCfg	*cifs_globalCfgp;
extern cifs_objLists	*cifs_objectsp;
extern uchar8			cifs_debugLevel;
extern uint32			cifs_elog_map[7];

//Debug traces levels
#define CIFS_DBGLV_ERR      1
#define CIFS_DBGLV_WARN     2
#define CIFS_DBGLV_INFO     3
#define CIFS_DBGLV_DBG      4
#define CIFS_DBGLV_DBG2     5
#define CIFS_DBGLV_DBG3     6

#define CIFS_PRINT(_lvl_, _par_)                                              \
    if (_lvl_ <= cifs_debugLevel)                                             \
        E_LOG(class_1, cifs_elog_map[_lvl_], cifs_globalCfgp->eee_cifsAppId,  \
              0, 0, 0, _par_)

//CIFS Threads common events
enum  {
	CIFS_THREADEVT_KILL	= CIFS_MSG_BASE + 200,
	CIFS_THREADEVT_BUF_AVAILABLE
}cifs_internalEvents;
		
#define cifs_treatKillEvent(evtp)	\
		if ((evtp)->event == CIFS_THREADEVT_KILL) thread_exit()


// ************************** Memory allocation ************************ 

#ifdef FS_STANDALONE
#define eee_ramAlloc(_size_) fs_ramAlloc(FS_RAM_CIFS_MSG, _size_)
		
#define eee_ramAllocZero(_size_)                                              \
	{	fs_ramAlloc(FS_RAM_CIFS_MSG, _size_);                                 \
		bzero(_p_, _size_);													  \
	}
	
#define eee_ramDealloc(_p_) fs_ramDealloc(FS_RAM_CIFS_MSG, _p_)
	
#endif // FS_STANDALONE

#define CIFS_RAM_ALLOC(_p_, _type_, _size_)                                   \
	{																		  \
    _p_ = (_type_ *) eee_ramAlloc(_size_);                                    \
    if (_p_ == NULL)                                                          \
    {                                                                         \
        CIFS_PRINT(CIFS_DBGLV_ERR, ("eee_ramAlloc() failed"));                \
        NFX_ASSERT(TRUE);                                                     \
    }																		  \
    }

#define CIFS_RAM_ALLOC_ZERO(_p_, _type_, _size_)                              \
	{																		  \
    _p_ = (_type_ *) eee_ramAlloc(_size_);                                    \
    if (_p_ == NULL)                                                          \
    {                                                                         \
        CIFS_PRINT(CIFS_DBGLV_ERR, ("eee_ramAllocZero() failed"));            \
        NFX_ASSERT(TRUE);                                                     \
    }																		  \
	bzero(_p_, _size_);														  \
	}


#define CIFS_RAM_ALLOC_ALIGNED(_p_, _type_, _size_)                           \
	{																		  \
    _p_ = (_type_ *) eee_ramAllocAligned(_size_);                             \
    if (_p_ == NULL)                                                          \
    {                                                                         \
        CIFS_PRINT(CIFS_DBGLV_ERR, ("eee_ramAllocAligned() failed"));         \
        NFX_ASSERT(TRUE);                                                     \
    }																		  \
    }

#define CIFS_RAM_ALLOC_ALIGNED_ZERO(_p_, _type_, _size_)                      \
	{																		  \
    _p_ = (_type_ *) eee_ramAllocAligned(_size_); 	                          \
    if (_p_ == NULL)                                                          \
    {                                                                         \
        CIFS_PRINT(CIFS_DBGLV_ERR, ("eee_ramAllocAlignedZero() failed"));     \
        NFX_ASSERT(TRUE);                                                     \
    }																		  \
	bzero(_p_, _size_);														  \
    }

#define CIFS_RAM_DEALLOC(_p_)                                                 \
	{																		  \
    if (_p_ != NULL)                                                          \
    {                                                                         \
        eee_ramDealloc(_p_);                                                  \
        _p_ = NULL;                                                           \
    }																		  \
    }


#endif // #ifndef _CIFS_H 

