/*
 *-----------------------------------------------------------------
 *
 * Name:	cod-api.h
 *
 * RCS:  $Id: cod-api.h,v 1.1.1.1 2001/08/06 23:42:12 rick Exp $
 *
 * Copyright (C) 2001, Agile Storage, Inc.
 *
 * Description:	COD creation routines
 *
 * Created by:	Rick Lund
 *
 * Date Created:	7/30/01
 *
 *-----------------------------------------------------------------
 */

#ifndef _COD_API_H
#define _COD_API_H

#include <exports/cm-api.h>

extern uint64	cod_cntr;

/*
 * A COD in the NFX system is represented by a 64-bit value.  The upper nibble
 * contains slot info.  The next nibble is an identifier for the entity that
 * created the COD,  The next nibble is for chassis number.  Then there is
 * a 52-bit incrementing counter.  That's enough bits to guarantee that we
 * won't wrap for lots of years.
 *
 *
 *        60     56      52                                               0
 *  +------+------+-------+-----------------------------------------------+
 *  |      |      |       |                                               |
 *  | slot |  cpu | chass |                  52 bit counter               |
 *  |      |      |       |                                               |
 *  +------+------+-------+-----------------------------------------------+
 *
 */

#define COD_CREATOR_MASK	0xFFF0000000000000
#define COD_SLOT_SHIFT		60
#define COD_CPU_SHIFT		56
#define COD_CHASSIS_SHIFT	52

/*-----------------------------------------------------------------
 * Name:	cod_init()
 *
 * Description:	Initialize COD counter
 *
 * Created by:	Rick Lund
 *
 * Date Created:	7/30/01
 *
 *-----------------------------------------------------------------
 */
void cod_init();

/*-----------------------------------------------------------------
 * Name:	COD_GET_NEXT()
 *
 * Description:	Returns the next COD handle for this slot and CPU.
 *
 * Created by:	Rick Lund
 *
 * Date Created:	7/30/01
 *
 *-----------------------------------------------------------------
 */
#define COD_GET_NEXT()											\
({																\
	uint64	_cod;												\
																\
    if (cod_cntr++ & COD_CREATOR_MASK)							\
        cod_cntr = 0;											\
																\
	_cod = ((uint64)nfxMySlotId << COD_SLOT_SHIFT) |			\
			((uint64)nfxMyCPUId << COD_CPU_SHIFT) |				\
			((uint64)nfxMyChassisId << COD_CHASSIS_SHIFT);		\
																\
	_cod |= cod_cntr;											\
																\
	_cod;														\
});

#endif	/* _COD_API_H */

