/* =============================================================================
 * File Name  :	bmc12500-eth-diag.c
 *
 * Description:    
 *
 * Created    :	29 October 2001
 *
 * Author     :	Frank Salzmann
 *
 * Copyright(c)  Agile Storage, Inc. 2001
 *
 * RCS        : $Id: bmc12500-eth-diag.c,v 1.1.1.1 2002/06/11 16:44:28 maximk Exp $
 * =============================================================================
 */
#include "bmc12500-eth.h"
extern char bc1250Eth_addr[3][6];

typedef struct bc1250Eth_test {

	uint32 portUnderTest;
	uint32 testState;				
#define BC_TEST_DONE		0
#define BC_TEST_INPROGRESS	1
#define BC_TEST_VERIFY		2
#define BC_TEST_CONTINUE	3
#define BC_TEST_END			4

	uint32 testMode;		
#define BC_TEST_INT			1
#define BC_TEST_EXT			2
#define BC_TEST_PHY			3

	uint32 testPktCnt;
	uint32 testPktSize;
	uint32 testDestAddr;
	uchar8 *testBuffer;
	uint32 fVerify;

	uint32 rxPktCnt;;
	uint32 rxErrPktCnt;
	uint32 rxByteCnt;
	uint32 txPktCnt;
	uint32 txByteCnt;
} bc1250Eth_test_t;

static bc1250Eth_test_t bc1250Eth_testControl;
#ifndef PROM
/* Extern variables for system DVT */
extern uint32	sysDVTrunning; /* used to add eee Hdr for front plane pkts fr txrx */
#endif

/*
 *------------------------------------------------------------------------------
 * Function Name:	bmc12500Eth_diagSetLbMode
 *
 * Description: 	Cleans up descriptors on a transmit queue.
 *
 * Created by:		Frank Salzmann
 *
 * Data Created:	April 30, 2001
 * 					
 * Returns: ?????
 *
 *------------------------------------------------------------------------------
 */
int
bmc12500Eth_diagSetLbMode( bc1250EthCfg_t *cfg, uint32 mode )
{
	//-- Bug in the SB1250 does not allow internal mac loopback
	//-- mode to work.
#if 0

	bmc12500_macCfgr_t macCfg;
	bc1250Eth_regs_t *regs;
	bmc12500_macFrmCfgr_t frmCfg;

	regs = &cfg->regs;

	macCfg.wd = bmc12500Eth_readReg(regs->mac_cfg);
	frmCfg.wd = bmc12500Eth_readReg(regs->mac_frame_cfg);	

	/*
	 * Set/Reset LB Mode
	 */
	if( mode == BC1250_MAC_LB_ENABLE ) {
		printf("BC1250 Internal Loopback Set\n");
		macCfg.bits.loopback_sel    = 1;		// Enable Int Loopback
#if 0
		macCfg.bits.speed_sel       = 10;		// Set to 1000mb/s
		macCfg.bits.hdx_en			= 0;		// Set to full duplex
		macCfg.bits.fc_sel          = 1;
		macCfg.bits.fc_cmd          = 1;
		macCfg.bits.burst_en        = 1;
		frmCfg.bits.slot_offset     = 512;
#endif

		cfg->link.loopback = lb_int;
		bmc12500Eth_setPromiscMode( cfg, BC1250_SET);
	}
	else {
		printf("BC1250 Internal Loopback Cleared\n");
		macCfg.bits.loopback_sel    = 0;		// Disable Int Loopback
		cfg->link.loopback          = lb_none;
		bmc12500Eth_setPromiscMode( cfg, BC1250_CLEAR);
	}

	bmc12500Eth_writeReg(regs->mac_frame_cfg,frmCfg.wd);
	bmc12500Eth_writeReg(regs->mac_cfg, macCfg.wd);

#endif
	return(NFX_OK);
} // bmc12500Eth_diagSetLbMode 

/*
 *------------------------------------------------------------------------------
 * Function Name:	bmc12500Eth_diagLBTest
 *
 * Description: 	Runs loopback test 
 *
 * Created by:		Frank Salzmann
 *
 * Data Created:	April 30, 2001
 * 					
 * Returns: ?????
 *
 *------------------------------------------------------------------------------
 */
int
bmc12500Eth_diagLBTest(bc1250EthCfg_t *cfg)
{
	int32  i, rc;
	int32  pktCnt;
	int32  pktSize;
	int32  testMode;
	uint32 destAddr;
	uint32 dstPort;
	uchar8 *bufptr, *buf;
	uint32 txport;
	uint32 destCpu;
	uint32 val;
	bc1250Eth_regs_t *regs;

	pktCnt = bmc12500Eth_uiGetInput("Enter Pkt Count: ");
	if( pktCnt <= 0 ) {
		printf("Zero Pkt Count. Terminating Loopback Test\n"); 
		return(NFX_OK);
	}

	pktSize = bmc12500Eth_uiGetInput("Enter Pkt Size: ");
	if((pktSize < 64) || (pktSize > BC1250_MAX_PKT_SIZE)) {
		printf("Invalid Pkt Size: %d. Terminating Loopback Test\n", pktSize); 
		return(NFX_OK);
	}

	testMode = bmc12500Eth_uiGetInput("Enter Test Mode [INT:1 | EXT:2 | PHY:3]: ");
	if((testMode < 1) || (testMode > 3)) {
		printf("Invalid Test Mode: %d. Terminating Loopback Test\n", testMode); 
		return(NFX_OK);
	}


	if( bc1250Eth_testControl.testState == BC_TEST_INPROGRESS ) {
		printf("Terminating Previous Test:\n");
		if( bc1250Eth_testControl.testBuffer) 
			eee_ramDealloc(bc1250Eth_testControl.testBuffer);
	} 

	memset(&bc1250Eth_testControl,0,sizeof(bc1250Eth_testControl));

	bc1250Eth_testControl.portUnderTest   = cfg->macId; 
	bc1250Eth_testControl.testState   = BC_TEST_INPROGRESS; 
	bc1250Eth_testControl.testMode    = testMode;
	bc1250Eth_testControl.testPktCnt  = pktCnt;
	bc1250Eth_testControl.testPktSize = pktSize;
	bc1250Eth_testControl.fVerify	  = 1;
	/*
	 * Allocate and initialize test data.
	 */
	buf = bufptr = (uchar8 *)eee_ramAlloc(pktSize);
	if( !bufptr ) {
		printf("Test data buffer allocation failed.\n");
		return(NFX_OK);
	}
	bc1250Eth_testControl.testBuffer = bufptr;

	//-- Set mac into promis mode.
	regs = &cfg->regs;
	val = bmc12500Eth_readReg(regs->mac_adfilter_cfg);
	val |= 1; // 
	bmc12500Eth_writeReg(regs->mac_adfilter_cfg, val);

	switch(testMode) {

		case BC_TEST_INT:
			bmc12500Eth_diagSetLbMode(cfg, BC1250_MAC_LB_ENABLE);
			cfg->link.loopback = lb_int;
			dstPort = 0;
			break;

		case BC_TEST_EXT:
			dstPort = bmc12500Eth_uiGetInput("Enter Destination Port[0|1]: ");
			if((dstPort != 0) && (dstPort != 1 )) {
				printf("Invalid Dest Port: %d.\n", dstPort); 
				eee_ramDealloc(buf);
				return(NFX_OK);
			}
			cfg->link.loopback = lb_ext;
			bcEthCfg[dstPort]->link.loopback = lb_ext;
			break;

		case BC_TEST_PHY:
			dstPort = 0;
			cfg->link.loopback = lb_phy;
			break;
	}

	//-- Initialize test packet data.
	for(i=0; i < pktSize; i++ )
		*bufptr++ = i & 0xFF;

	/*
	 * Send first set of loop back packets.
	 */
#ifdef NFP_TXRX
	destAddr = PACK_UNICAST_PORT_ADDRESS(
								 nfxMySlotId,
								 nfxMyCPUId,
								 cfg->appId,
								 EEE_USER_DATA,
								 NFX_COS_HIGH, 
								 NFP_PORT_TXRX_ETH0);
#elif defined(NFP_FP)
	if( cfg->macId == 0 )
		txport = NFP_PORT_FP_ETH0;
	else
		txport = NFP_PORT_FP_ETH1;

	if( dstPort == 0 )
		destCpu = NFX_NFP_FP1;
	else
		destCpu = NFX_NFP_FP2;

	destAddr = PACK_UNICAST_PORT_ADDRESS(
                                 nfxMySlotId,  
                                 destCpu,
                                 cfg->appId,
                                 EEE_USER_DATA,
                                 NFX_COS_HIGH,
                                 txport);        
#elif defined(PROM)
	if( isFP() ) {
		if( cfg->macId == 0 )
			txport = PROM_PORT_FP0;
		else
			txport = PROM_PORT_FP1;

		if( dstPort == 0 )
			destCpu = NFX_NFP_FP2; // change this when we bring up the 2nd core
		else
			destCpu = NFX_NFP_FP2; 
	}
	else {
		if( cfg->macId == 0 )
			txport = PROM_PORT_TXRX0;
		else
			txport = PROM_PORT_TXRX1;

		if( dstPort == 0 )
			destCpu = NFX_NFP_TXRX1;
		else
			destCpu = NFX_NFP_TXRX1; // change this when we bring up the 2nd core
	}

	destAddr = PACK_UNICAST_PORT_ADDRESS(
								 nfxMySlotId,
								 destCpu,
								 cfg->appId,
								 EEE_USER_DATA,
								 NFX_COS_HIGH, 
								 txport);
#endif

	bc1250Eth_testControl.testDestAddr = destAddr;

	for(i=0; i < 1; i++)
		rc = eee_sendMessage(buf, pktSize, destAddr, destAddr);

	bc1250Eth_testControl.txPktCnt += 1;
	bc1250Eth_testControl.txByteCnt += (1 * pktSize);
	return(NFX_OK);
} // bmc12500Eth_diagLBTest 

/*
 *------------------------------------------------------------------------------
 * Function Name:	bmc12500Eth_diagLbPktReceive
 *
 * Description: 	Receives Loop back packet 
 *
 * Created by:		Frank Salzmann
 *
 * Data Created:	November 05, 2001
 * 					
 * Returns: ?????
 *
 *------------------------------------------------------------------------------
 */
int
bmc12500Eth_diagLbPktReceive( eee_desc_t *edesc )
{
	int32				rc;
	int32 				pktLen;
	int32 				bufCnt;
	uint32 				val;
	bc1250Eth_test_t 	*dptc;
	bc1250EthCfg_t      *cfg;
	bc1250Eth_regs_t *regs;

	dptc = &bc1250Eth_testControl;
	cfg = bcEthCfg[dptc->portUnderTest];

	pktLen = edesc->hdr.control & EDESC_TOTAL_LEN;
	bufCnt = ((edesc->hdr.control & EDESC_BUFCNT_MASK) >> EDESC_BUFCNT_SHIFT);

	while(1) {
		switch( dptc->testState )
		{
			case BC_TEST_INPROGRESS:
				dptc->rxPktCnt++; 
				dptc->rxByteCnt += pktLen;

				if( dptc->fVerify )
					dptc->testState = BC_TEST_VERIFY;
				else {
					if( dptc->rxPktCnt >= dptc->testPktCnt)
						dptc->testState = BC_TEST_END;
					else
						dptc->testState = BC_TEST_CONTINUE;
				}
				break;

			case BC_TEST_VERIFY:
				rc = bmc12500Eth_diagLbVerifyData(edesc->bd[1].buf, 
												  edesc->bd[1].len, 
												  dptc->testBuffer);
				if( dptc->rxPktCnt >= dptc->testPktCnt)
					dptc->testState = BC_TEST_END;
				else
					dptc->testState = rc;
				break;

			case BC_TEST_CONTINUE:
				dptc->txPktCnt++; 
				dptc->txByteCnt += pktLen;
				edesc->hdr.dest_port = dptc->testDestAddr;
				eee_forwardPacket(edesc);
				dptc->testState = BC_TEST_INPROGRESS;
				return(NFX_OK);
				
			case BC_TEST_END:
				if(dptc->testBuffer)
					eee_ramDealloc(dptc->testBuffer);

				if(dptc->testMode == BC_TEST_INT)
					bmc12500Eth_diagSetLbMode(bcEthCfg[0], BC1250_MAC_LB_DISABLE);

				//-- Set mac into promis mode.
				regs = &cfg->regs;
				val = bmc12500Eth_readReg(regs->mac_adfilter_cfg);
				val &= ~1;  
				bmc12500Eth_writeReg(regs->mac_adfilter_cfg, val);

				bmc12500Eth_diagShowLbTestResults(dptc);
				dptc->testState = BC_TEST_DONE;
				break;

			case BC_TEST_DONE:
				eee_freePacket(edesc);
				return(NFX_OK);
		}		
	}
	return(NFX_OK);
} // bmc12500Eth_diagLbPktReceive

/*
 *------------------------------------------------------------------------------
 * Function Name:	bmc12500Eth_diagLbVerifyData
 *
 * Description: 	Verifies the integrety of loop back data 
 *
 * Created by:		Frank Salzmann
 *
 * Data Created:	May 14, 2001
 * 					
 * Returns: ?????
 *
 *------------------------------------------------------------------------------
 */
int
bmc12500Eth_diagLbVerifyData( uchar8 *bufptr, int32 len, uchar8 *tbufptr )
{
	int32  i;

	if( !bufptr || !tbufptr )
		return(NFX_ERR);

	for(i=0; i < len; i++) {
		if( *bufptr != *tbufptr ) {
			printf("LB Data Error: TestVal=%02x, ActualVal=%02x, Addr=%x\n",
				   *tbufptr, *bufptr, bufptr);
			return(BC_TEST_END);
		}
		bufptr++;
		tbufptr++;
	}
	return(BC_TEST_CONTINUE);
} // bmc12500Eth_diagLbVerifyData 



/*
 *------------------------------------------------------------------------------
 * Function Name:	bmc12500Eth_diagTxPkt
 *
 * Description: 	Transmits a test packet.
 *
 * Created by:		Frank Salzmann
 *
 * Data Created:	June 25, 2001
 * 					
 * Returns: ?????
 *
 *------------------------------------------------------------------------------
 */
int
bmc12500Eth_diagTxPkt( bc1250EthCfg_t *dp )
{
	int32  i,j,k;
	int32  bufCnt;
	int32  bufLen;
	int32  lastBufLen;
	int32  pktCnt;
	int32  pktSize;
	int32  testMode;
	uint32 destAddr;
	uchar8 *bufptr;
	uchar8 *p_uncached;
	eee_desc_t *edesc;
	eee_descHdr_t *hdr;
	int32   out_port;

	pktCnt = bmc12500Eth_uiGetInput("Enter Pkt Count: ");
	if( pktCnt <= 0 ) {
		printf("Zero Pkt Count. Terminating Loopback Test\n"); 
		return(NFX_OK);
	}

	pktSize = bmc12500Eth_uiGetInput("Enter Pkt Size: ");
	if((pktSize < 64) || (pktSize > BC1250_MAX_PKT_SIZE)) {
		printf("Invalid Pkt Size: %d. Terminating Loopback Test\n", pktSize); 
		return(NFX_OK);
	}

	if( !(pktSize % 2048) ) {
		bufCnt = (pktSize / 2048);
		lastBufLen = 2048;
	}
	else {
		bufCnt = (pktSize / 2048) + 1;
		lastBufLen = pktSize % 2048;
	}

	printf("BufCnt: %d, LastBufLen: %d\n", bufCnt, lastBufLen);

	for(i=0; i < pktCnt; i++) {

		edesc = eee_allocateDesc(EEE_SHARED_POOL, NFX_COS_HIGH);
		memset(edesc,0, sizeof(eee_desc_t));
		hdr = (eee_descHdr_t *)edesc;
		hdr->control = (bufCnt << EDESC_BUFCNT_SHIFT) | pktSize;

		for(k=1; k <= bufCnt; k++) {
			if(k == bufCnt)
				bufLen = lastBufLen;
			else
				bufLen = 2048;


			bufptr = eee_allocateBuffer(EEE_SHARED_POOL, EEE_BUF_LARGE, NFX_COS_HIGH);

			if( !bufptr ) {
				printf("Test data buffer allocation failed.\n");
				return(NFX_OK);
			}

			p_uncached = (uchar8 *)((address_t)bufptr | 0x00000000);
			for(j=0; j < 14; j++ )
				p_uncached[j] = 0xff;		// Setup Broadcast MAC DA
			for(j=14; j < bufLen-14; j++ )
				p_uncached[j] = (j & 0xff);

			// bmc12500Eth_printBuf(p_uncached, bufLen);

			edesc->hdr.src_port =  0xbbccddee;
#ifdef NFP_TXRX
			edesc->hdr.dest_port = PACK_UNICAST_PORT_ADDRESS(
														 nfxMySlotId,
														 nfxMyCPUId,
														 dp->appId,
														 EEE_USER_DATA,
														 NFX_COS_HIGH, 
														 NFP_PORT_TXRX_ETH0);
#elif defined(NFP_FP)
			edesc->hdr.dest_port = PACK_UNICAST_PORT_ADDRESS(
														 nfxMySlotId,
														 nfxMyCPUId,
														 dp->appId,
														 EEE_USER_DATA,
														 NFX_COS_HIGH, 
														 NFP_PORT_FP_ETH0);
#elif defined(PROM)
			if( isTXRX() ) {
				if( dp->macId == 0 )
					out_port = PROM_PORT_TXRX0;
				else
				if( dp->macId == 1 )
					out_port = PROM_PORT_TXRX1;
				else {
					printf("bmc12500_diagTxPkt: Invalid Mac Id: %d\n", dp->macId );
					return(0);
				}
			}
			else {
				if( dp->macId == 0 )
					out_port = PROM_PORT_FP0;
				else
				if( dp->macId == 1 )
					out_port = PROM_PORT_FP1;
				else {
					printf("bmc12500_diagTxPkt: Invalid Mac Id: %d\n", dp->macId );
					return(0);
				}
			}
			edesc->hdr.dest_port = PACK_UNICAST_PORT_ADDRESS(
														 nfxMySlotId,
														 nfxMyCPUId,
														 dp->appId,
														 EEE_USER_DATA,
														 NFX_COS_HIGH, 
														 out_port);
#endif

			edesc->bd[k].buf = bufptr;
			edesc->bd[k].len = bufLen;
			edesc->bd[k].control = EDESC_BD_CMD_IN_USE | EDESC_BD_LARGE;
		}
		bmc12500Eth_txUnicast( edesc->hdr.dest_port,edesc );
	}

	return(NFX_OK);
} // bmc12500Eth_diagTxPkt

int
bmc12500Eth_diagTxLuc( bc1250EthCfg_t *dp )
{
    int32  i,j,k;
    int32  bufCnt;
    int32  bufLen;
    int32  lastBufLen;
    int32  pktCnt;
    int32  pktSize;
    int32  testMode;
    uint32 destAddr;
    uchar8 *bufptr;
    uchar8 *p_uncached;
    eee_desc_t *edesc;
    eee_descHdr_t *hdr;
    int32   out_port;
	uchar8 junkbuf[BC1250_MAX_PKT_SIZE];

    uchar8 fill_byte;
    luc_tx_hdr_t luc_hdr;
    uint16 udp_len;
    uint16 ip_len;
    uint32 junk_idx;
    uint32 index = 0;
    int linkid;
	extern uint32 luc_loopback_test;

	if (!isCheetah()) {
		printf("bmc12500Eth_diagTxLuc: Only available on Cheetah!\n");
		return(NFX_OK);
	}
	if (!isTXRX()) {
		printf("bmc12500Eth_diagTxLuc: Only available on TXRX!\n");
		return(NFX_OK);
	}
    pktCnt = bmc12500Eth_uiGetInput("Enter Pkt Count: ");
    if( pktCnt <= 0 || pktCnt >= BC1250_MAC_TX_Q_LEN ) {
        printf("Invalid Pkt Count. Terminating Loopback Test\n");
        return(NFX_OK);
    }
    pktSize = bmc12500Eth_uiGetInput("Enter Pkt Size: ");

    fill_byte = 0;

    linkid = bmc12500Eth_uiGetInput("Enter link id: ");

    for (i = 0; i < BC1250_MAX_PKT_SIZE; i++)
        junkbuf[i] = fill_byte++;

    /* fill LUC header = 20 bytes */
    luc_hdr.field.seq_num = 0;
    luc_hdr.field.pkt_len = 0;
    luc_hdr.field.link_id = linkid;
    luc_hdr.field.control = 3; /* generate both checksums */
    luc_hdr.field.protocol = 0x11;
    luc_hdr.field.tcp_udp_len = pktSize - 20 - 14 - 20;
    luc_hdr.field.src_ip = 0x99999999;
    luc_hdr.field.dest_ip = 0x66666666;
    luc_hdr.field.l3_offset = 14;
    luc_hdr.field.l4_offset = 34;
    luc_hdr.field.tcp_udp_csum = 40;
    luc_hdr.field.tcp_udp_csum = 40;
    luc_hdr.field.ip_csum = 24;

    for (i = 19; i >= 0; i--)
        junkbuf[index++] = luc_hdr.data[i];

    /* fill MAC header = 14 bytes */
    for (i = 0; i < 6; i++)
        junkbuf[index++] = 0xff;
    for (i = 6; i < 12; i++)
        junkbuf[index++] = 0x00;
    junkbuf[index++] = 0x08;
    junkbuf[index++] = 0x00;

    ip_len = pktSize - 14 - 20;

    /* fill IP header = 20 bytes */
    junkbuf[index++] = 0x45;
    junkbuf[index++] = 0x00;
    junkbuf[index++] = (ip_len >> 8) & 0xff ; /* length */
    junkbuf[index++] = (ip_len & 0xff);
    junkbuf[index++] = 0x00; /* id */
    junkbuf[index++] = 0x00; /* id */
    junkbuf[index++] = 0x40; /* flag */
    junkbuf[index++] = 0x00;
    junkbuf[index++] = 0xff;
    junkbuf[index++] = 0x11;
    junkbuf[index++] = 0x00; /* checksum */
    junkbuf[index++] = 0x00; /* checksum */
    junkbuf[index++] = 0x99; /* src ip */
    junkbuf[index++] = 0x99;
    junkbuf[index++] = 0x99;
    junkbuf[index++] = 0x99;
    junkbuf[index++] = 0x66; /* dest ip */
    junkbuf[index++] = 0x66;
    junkbuf[index++] = 0x66;
    junkbuf[index++] = 0x66;

    /* fill UDP header = 8 bytes */
    junkbuf[index++] = 0x11; /* src port */
    junkbuf[index++] = 0x11;
    junkbuf[index++] = 0x22; /* dest port */
    junkbuf[index++] = 0x22;
    udp_len = pktSize - 14 - 20 -20 - 8;
    junkbuf[index++] = (udp_len >> 8) & 0xff;
    junkbuf[index++] = (udp_len & 0xff);
    junkbuf[index++] = 0x00; /* checksum */
    junkbuf[index++] = 0x00; /* checksum */

    if((pktSize < 60) || (pktSize > BC1250_MAX_PKT_SIZE)) {
        printf("Invalid Pkt Size: %d. Terminating Loopback Test\n", pktSize);
        return(NFX_OK);
    }
    if( !(pktSize % 2048) ) {
        bufCnt = (pktSize / 2048);
        lastBufLen = 2048;
    }
    else {
        bufCnt = (pktSize / 2048) + 1;
        lastBufLen = pktSize % 2048;
    }

    printf("BufCnt: %d, LastBufLen: %d\n", bufCnt, lastBufLen);

    for(i=0; i < pktCnt; i++) {

        edesc = eee_allocateDesc(EEE_SHARED_POOL, NFX_COS_HIGH);
        memset(edesc,0, sizeof(eee_desc_t));
        hdr = (eee_descHdr_t *)edesc;
        hdr->control = (bufCnt << EDESC_BUFCNT_SHIFT) | pktSize;
        junk_idx = 0;

        for(k=1; k <= bufCnt; k++) {
            if(k == bufCnt)
                bufLen = lastBufLen;
            else
                bufLen = 2048;

            bufptr = eee_allocateBuffer(EEE_SHARED_POOL, EEE_BUF_LARGE, NFX_COS_HIGH);
            if( !bufptr ) {
                printf("Test data buffer allocation failed.\n");
                return(NFX_OK);
            }
            p_uncached = (uchar8 *)((address_t)bufptr | 0x00000000);
            for (j=0; j < bufLen; j++)
                p_uncached[j] = junkbuf[junk_idx++];
            edesc->hdr.src_port =  0xbbccddee;
#ifdef NFP_TXRX
			if (dp->macId == 0)
				out_port = NFP_PORT_TXRX_ETH0;
			else
				out_port = NFP_PORT_TXRX_ETH2;
#elif defined(NFP_FP)
			if (dp->macId == 0)
				out_port = NFP_PORT_FP_ETH0;
			else
				out_port = NFP_PORT_FP_ETH2;
#elif defined(PROM)
            if( isTXRX() ) {
                if( dp->macId == 0 )
                    out_port = PROM_PORT_TXRX0;
                else 
                    out_port = PROM_PORT_TXRX2;
            }
            else {
                if( dp->macId == 0 )
                    out_port = PROM_PORT_FP0;
                else
                    out_port = PROM_PORT_FP2;
            }
#endif
            edesc->hdr.dest_port = PACK_UNICAST_PORT_ADDRESS(
                                                         nfxMySlotId,
                                                         nfxMyCPUId,
                                                         dp->appId,
                                                         EEE_USER_DATA,
                                                         NFX_COS_HIGH,
                                                         out_port);
            edesc->bd[k].buf = bufptr;
            edesc->bd[k].len = bufLen;
            edesc->bd[k].control = EDESC_BD_CMD_IN_USE | EDESC_BD_LARGE;
        }
		luc_loopback_test = 1;
        bmc12500Eth_txUnicast( edesc->hdr.dest_port,edesc );
    }
    return(NFX_OK);
} // bmc12500Eth_diagTxLuc

int
bmc12500Eth_diagTxLmux( bc1250EthCfg_t *dp )
{
    int32  i,j,k;
    int32  bufCnt;
    int32  bufLen;
    int32  lastBufLen;
    int32  pktCnt;
    int32  pktSize;
    int32  testMode;
    uint32 destAddr;
    uchar8 *bufptr;
    uchar8 *p_uncached;
    eee_desc_t *edesc;
    eee_descHdr_t *hdr;
    int32   out_port;
    int32   junk_idx;

    uchar8 fill_byte;
    int32 linkid;
	uchar8 junkbuf[BC1250_MAX_PKT_SIZE];
	extern uint32 lmux_loopback_test;

	if (!isCheetah()) {
		printf("bmc12500Eth_diagTxLmux: Only available on Cheetah!\n");
		return(NFX_OK);
	}
	if (!isFP()) {
		printf("bmc12500Eth_diagTxLmux: Only available on FP!\n");
		return(NFX_OK);
	}
    pktCnt = bmc12500Eth_uiGetInput("Enter Pkt Count: ");
    if( pktCnt <= 0 || pktCnt >= BC1250_MAC_TX_Q_LEN ) {
        printf("Invalid Pkt Count. Terminating Loopback Test\n");
        return(NFX_OK);
    }
    pktSize = bmc12500Eth_uiGetInput("Enter Pkt Size: ");
    linkid = bmc12500Eth_uiGetInput("Enter link id: ");
    fill_byte = 0;

    for (i = 0; i < 20; i++) {
        junkbuf[i] = 0;
    }

    for (i = 20; i < BC1250_MAX_PKT_SIZE; i++)
        junkbuf[i] = fill_byte++;

    junkbuf[3] = linkid;

    if((pktSize < 64) || (pktSize > BC1250_MAX_PKT_SIZE)) {
        printf("Invalid Pkt Size: %d. Terminating Loopback Test\n", pktSize);
        return(NFX_OK);
    }
    if( !(pktSize % 2048) ) {
        bufCnt = (pktSize / 2048);
        lastBufLen = 2048;
    }
    else {
        bufCnt = (pktSize / 2048) + 1;
        lastBufLen = pktSize % 2048;
    }
    printf("BufCnt: %d, LastBufLen: %d\n", bufCnt, lastBufLen);
    for(i=0; i < pktCnt; i++) {

        edesc = eee_allocateDesc(EEE_SHARED_POOL, NFX_COS_HIGH);
        memset(edesc,0, sizeof(eee_desc_t));
        hdr = (eee_descHdr_t *)edesc;
        hdr->control = (bufCnt << EDESC_BUFCNT_SHIFT) | pktSize;
        junk_idx = 0;

        for(k=1; k <= bufCnt; k++) {
            if(k == bufCnt)
                bufLen = lastBufLen;
            else
                bufLen = 2048;

            bufptr = eee_allocateBuffer(EEE_SHARED_POOL, EEE_BUF_LARGE, NFX_COS_HIGH);
            if( !bufptr ) {
                printf("Test data buffer allocation failed.\n");
                return(NFX_OK);
            }
            p_uncached = (uchar8 *)((address_t)bufptr | 0x00000000);
#if 1
            for (j=0; j < bufLen; j++)
                p_uncached[j] = junkbuf[junk_idx++];
#endif
#if 1
            for(j=4; j < 18; j++ )
                p_uncached[j] = 0xff;       // Setup Broadcast MAC DA
#endif
            // bmc12500Eth_printBuf(p_uncached, bufLen);
            edesc->hdr.src_port =  0xbbccddee;
#ifdef NFP_TXRX
			if (dp->macId == 0)
				out_port = NFP_PORT_TXRX_ETH0;
			else
				out_port = NFP_PORT_TXRX_ETH2;
#elif defined(NFP_FP)
			if (dp->macId == 0)
				out_port = NFP_PORT_FP_ETH0;
			else
				out_port = NFP_PORT_FP_ETH2;
#elif defined(PROM)
            if( isTXRX() ) {
                if( dp->macId == 0 )
                    out_port = PROM_PORT_TXRX0;
                else 
                    out_port = PROM_PORT_TXRX2;
            }
            else {
                if( dp->macId == 0 )
                    out_port = PROM_PORT_FP0;
                else
                    out_port = PROM_PORT_FP2;
            }
#endif
            edesc->hdr.dest_port = PACK_UNICAST_PORT_ADDRESS(
                                                         nfxMySlotId,
                                                         nfxMyCPUId,
                                                         dp->appId,
                                                         EEE_USER_DATA,
                                                         NFX_COS_HIGH,
                                                         out_port);
            edesc->bd[k].buf = bufptr;
            edesc->bd[k].len = bufLen;
            edesc->bd[k].control = EDESC_BD_CMD_IN_USE | EDESC_BD_LARGE;
        }
		lmux_loopback_test = 1;
        bmc12500Eth_txUnicast( edesc->hdr.dest_port,edesc );
    }
    return(NFX_OK);
} // bmc12500Eth_diagTxLmux

/*
 *------------------------------------------------------------------------------
 * Function Name:	bmc12500Eth_diagRxEna
 *
 * Description: 	Sets Tx Enable Bit
 *
 * Created by:		Frank Salzmann
 *
 * Data Created:	June 25, 2001
 * 					
 * Returns: ?????
 *
 *------------------------------------------------------------------------------
 */
int
bmc12500Eth_diagRxEna( bc1250EthCfg_t *dp )
{

	return(NFX_OK);
} // bmc12500Eth_diagRxEna

char *bcTestStates[] = {"TEST_DONE",
						"TEST_INPROGRESS",
						"TEST_VERIFY",
						"TEST_CONTINUE",
						"TEST_END"
};

char *bcTestMode[] = {"INT LOOPBACK",
					  "EXT LOOPBACK",
					  "PHY LOOPBACK"
};

int
bmc12500Eth_diagViewTestStats(bc1250EthCfg_t *dp)
{
	bc1250Eth_test_t *t;

	t = &bc1250Eth_testControl;

	bmc12500Eth_diagShowLbTestResults(t);
	return(NFX_OK);
} // bmc12500Eth_diagViewTestStats

/*
 *------------------------------------------------------------------------------
 * Function Name:	bmc12500Eth_diagLbVerifyData
 *
 * Description: 	Verifies the integrety of loop back data 
 *
 * Created by:		Frank Salzmann
 *
 * Data Created:	May 14, 2001
 * 					
 * Returns: ?????
 *
 *------------------------------------------------------------------------------
 */
int
bmc12500Eth_diagShowLbTestResults(bc1250Eth_test_t *t)
{
	printf("BMC12500ETH LOOPBACK TEST RESULTS\n");
	printf("=============================\n\n");
	printf("Test State        : %s\n", bcTestStates[t->testState]);
	printf("Test Mode         : %s\n", bcTestMode[t->testMode - 1]);
	printf("Test Packet Cnt   : %d\n", t->testPktCnt);
	printf("Test Packet Size  : %d\n", t->testPktSize);
	printf("Test Verify Flag  : %d\n", t->fVerify);
	printf("Actual Rx Pkt Cnt : %d\n", t->rxPktCnt);
	printf("Actual Rx Byte Cnt: %d\n", t->rxByteCnt);
	printf("Actual Rx Err Cnt : %d\n", t->rxErrPktCnt);
	printf("Actual Tx Pkt Cnt : %d\n", t->txPktCnt);
	printf("Actual Tx Byte Cnt: %d\n", t->txByteCnt);
	printf("\n");
	return(NFX_OK);
} // bmc12500Eth_diagShowLbTestResults 

void
dvt_set_mac_promisc_mode()
{	
	bc1250EthCfg_t * fr_cfg0 = bcEthCfg[0];
	bc1250EthCfg_t * fr_cfg1 = bcEthCfg[1];
	bc1250Eth_regs_t *regs;
	uint32 val;

	//Need To do this to allow loopback pkts to be received 
 	//-- Set mac into promis mode.
	regs = &fr_cfg0->regs;
	val = bmc12500Eth_readReg(regs->mac_adfilter_cfg);
	val |= 1; //
	bmc12500Eth_writeReg(regs->mac_adfilter_cfg, val);
	regs = &fr_cfg1->regs;
	val = bmc12500Eth_readReg(regs->mac_adfilter_cfg);
	val |= 1; //
	bmc12500Eth_writeReg(regs->mac_adfilter_cfg, val);

}

void
dvt_Reset_mac_promisc_mode()
{	
		bc1250EthCfg_t * fr_cfg0 = bcEthCfg[0];
		bc1250EthCfg_t * fr_cfg1 = bcEthCfg[1];
		bc1250Eth_regs_t *regs;
		uint32 val;
		//-- ReSet mac from promis mode.
		regs = &fr_cfg0->regs;
		val = bmc12500Eth_readReg(regs->mac_adfilter_cfg);
		val &= 0; // After test is over
		bmc12500Eth_writeReg(regs->mac_adfilter_cfg, val);
		regs = &fr_cfg1->regs;
		val = bmc12500Eth_readReg(regs->mac_adfilter_cfg);
		val &= 0; //
		bmc12500Eth_writeReg(regs->mac_adfilter_cfg, val);
}	

#ifndef PROM
void
sys_dvt_do_frontplane()
{
	uint32	pkts, pkt_size, n_bytes, destport, val, i;
	uchar8 * bufp;
	bc1250EthCfg_t * fr_cfg0 = bcEthCfg[0];
	bc1250EthCfg_t * fr_cfg1 = bcEthCfg[1];

	if (isTXRX())
	{
			fr_cfg0->link.loopback = lb_ext;
			fr_cfg1->link.loopback = lb_ext;
			sysDVTrunning = 1; /* Used to add eee hdr for front plane pkts */
			//-- Set mac into promis mode.
			dvt_set_mac_promisc_mode();
	}
}
#endif
