/* =============================================================================
 * File Name  :	bmc12500-eth-ui.c
 *
 * Description:    
 *
 * Created    :	08 May 2001
 *
 * Author     :	Frank Salzmann
 *
 * Copyright(c)  Agile Storage, Inc. 2001
 * =============================================================================
 */
#include "bmc12500-eth.h"

#define LMUX_DEBUG
#define LUC_DEBUG
#ifdef LMUX_DEBUG
extern uint32 lmux_txPkts[6];
extern uint32 lmux_txBytes[6];
extern uint32 lmux_rxPkts[6];
extern uint32 lmux_rxBytes[6];
#endif
#ifdef LUC_DEBUG
extern uint32 luc_txPkts[4];
extern uint32 luc_txBytes[4];
extern uint32 luc_rxPkts[4];
extern uint32 luc_rxBytes[4];
#endif
uint32 lmux_loopback_test = 0;
uint32 luc_loopback_test = 0;
uint32 bc1250_diagCPU = NFX_NFP_FP2;  // user-selected CPU for diag
#if defined(NFX_SMP) && defined(NFP_FP)
uint32 bc1250_FP2CPU_active = 1;
uint32 bc1250_FP1CPU_active = 1;
uint32 dpp_tx_cleanup_lockcnt[2] = {0,0};
uint32 dpp_tx_cleanup_unlockcnt[2] = {0,0};
uint32 dpp_rx_lockcnt[2] = {0,0};
uint32 dpp_rx_unlockcnt[2] = {0,0};
uint32 dpp_app_rx_lockcnt[2] = {0,0};
uint32 dpp_app_rx_unlockcnt[2] = {0,0};
extern int32 bmc_txHWM CPU_PRIVATE;
extern int32 bmc_rxHWM CPU_PRIVATE;
#endif

void luc_show_regs(void);
void luc_clear_stats(void);
void luc_show_stats(void);
void luc_show_rev(void);
void luc_getVerString(char *);
void lmux_getVerString(char *);
void lmux_show_rev(void);
void lmux_show_stats(void);
void lmux_clear_stats(void);
void bmc12500Eth_clear_stats(uint32);
uint32 lsi_get_txPktCnt(int);
uint32 lsi_get_txByteCnt(int);
uint32 lsi_get_rxPktCnt(int);
uint32 lsi_get_rxByteCnt(int);
int bmc12500Eth_uiGetInput(char *);
int bmc12500Eth_showTxCapBuf(bc1250EthCfg_t *);
int bmc12500Eth_uiSetTxCapBuf(bc1250EthCfg_t *); 
int bmc12500Eth_uiSetRxCapBuf(bc1250EthCfg_t *); 
int bmc12500Eth_showTxCapBuf(bc1250EthCfg_t *);
int bmc12500Eth_showRxCapBuf(bc1250EthCfg_t *);
int bmc12500Eth_uiShowLinkTxStats(void);
int bmc12500Eth_uiShowLinkTxq(void);
int bmc12500Eth_uiShowMacCfgReg(bc1250EthCfg_t *);
int bmc12500Eth_uiWriteValue(bc1250EthCfg_t *);
int bmc12500Eth_uiReadValue(bc1250EthCfg_t *);
int bmc12500Eth_uiSetPktTrace(bc1250EthCfg_t *);
int bmc12500Eth_uiShowRxq(bc1250EthCfg_t *);
int bmc12500Eth_uiShowTxq(bc1250EthCfg_t *);
int bmc12500Eth_uiConfigLBMode( bc1250EthCfg_t *);
int bmc12500Eth_uiShowRegs(bc1250EthCfg_t *);
int bmc12500Eth_uiViewStats(bc1250EthCfg_t *);
int bmc12500Eth_uiViewCfg(bc1250EthCfg_t *);
int bmc12500Eth_uiDiagMenu(bc1250EthCfg_t *);

extern void diag_lsi_clearStats(int);
extern int get_cmd(char8 *);
extern int bmc12500Eth_initRmonRegs(bc1250EthCfg_t *);
extern int bmc12500Eth_setPromiscMode(bc1250EthCfg_t *, int);
extern int bmc12500Eth_mii_phyLoopback(bc1250EthCfg_t *, int);
extern int bmc12500Eth_diagSetLbMode(bc1250EthCfg_t *, uint32);
extern int bmc12500Eth_printBuf(char*, int, int);
extern int bmc12500Eth_diagTxLuc(bc1250EthCfg_t *);
extern int bmc12500Eth_diagTxLmux(bc1250EthCfg_t *);
extern int bmc12500Eth_diagTxPkt(bc1250EthCfg_t *);
extern int bmc12500Eth_mii_viewPhyRegs(bc1250EthCfg_t *);
extern int bmc12500Eth_mii_readPhyReg(bc1250EthCfg_t *);
extern int bmc12500Eth_mii_writePhyRegs(bc1250EthCfg_t *);
extern int bmc12500Eth_diagViewTestStats(bc1250EthCfg_t *);
extern int bmc12500Eth_diagLBTest(bc1250EthCfg_t *);
extern int bmc12500Eth_diagRxEna( bc1250EthCfg_t *);

/*
 *------------------------------------------------------------------------------
 * Function Name:	bmc12500Eth_uiDiag
 *
 * Description: 	BMC12500ETH UI entry function.
 *
 * Created by:		Frank Salzmann
 *
 * Data Created:	May 08, 2001
 * 					
 * Returns: NFX_OK
 *
 *------------------------------------------------------------------------------
 */
int
bmc12500Eth_uiDiag()
{
	uint32 mac;

	mac = bmc12500Eth_uiGetInput("Which MAC:[0|1|2]: ");

	if( (mac != 0) && (mac != 1) && (mac != 2) )
		return 0;

    if ((mac == 2) && (!isCheetah())) {
        printf("Only MAC 0 and 1 available!\n");
        return 0;
    }
	
    if ((mac == 1) && (isCheetah())) {
        printf("Only MAC 0 and 2 available!\n");
        return 0;
    }

	bmc12500Eth_uiDiagMenu( bcEthCfg[mac] ); // First ethernet mac for now.
	lmux_loopback_test = 0;
	luc_loopback_test = 0;
	return(NFX_OK);
} /* bmc12500Eth_uiDiag */

/*
 *------------------------------------------------------------------------------
 * Function Name:	bmc12500Eth_uiShowDiagMenu
 *
 * Description: 	Prints bmc12500Eth diagnostic menu items.
 *
 * Created by:		Frank Salzmann
 *
 * Data Created:	May 09, 2001
 * 					
 * Returns: NFX_OK
 *
 *------------------------------------------------------------------------------
 */
int
bmc12500Eth_uiShowDiagMenu()
{
	int sel;
	char cmd[80];

	sel = 0;
	printf("1 : Show Driver Cfg       ");
	printf("2 : Show Driver Stats     ");
	printf("3 : Show OP Regs\n");
	printf("4 : Set/Clear LB Mode     ");
	printf("5 : Show RX Desc Q        ");
	printf("6 : Show TX Desc Q\n");
	printf("7 : Transmit Packet       ");
	printf("8 : Receive Enable        ");
	printf("9 : Run LB Test\n");
	printf("10: View Test Results     ");
	printf("11: Ena/Dis Pkt Trace     ");
	printf("12: Write Value\n");
	printf("13: Read Value            ");
	printf("14: Clear Rmon Regs       ");
	printf("15: Show Mac Cfg Reg\n");
	printf("16: Show Phy Regs         ");
	printf("17: Write Phy Reg         ");
	printf("18: Read Phy Reg\n");
	printf("19: Show Link Txq Stats   ");
	printf("20: Show Link Txqs        ");
	printf("21: Show Rx Capture Buf \n");
	printf("22: Ena/Dis Rx Capture    ");
	printf("23: Show Tx Capture Buf   ");
	printf("24: Ena/Dis Tx Capture  \n");
	if (isCheetah()) {
	printf("25: Transmit LMUX packet  ");
	printf("26: Transmit LUC packet   ");
	printf("27: Show LUC revision   \n");
	printf("28: Show LUC stats        ");
	printf("29: Clear LUC stats       ");
	printf("30: Show LMUX stats     \n");
	printf("31: Clear LMUX stats      ");
	printf("32: Show LUC registers    ");
	printf("33: Show LMUX revision  \n");
#if defined(NFX_SMP) && defined(NFP_FP)
	printf("34: Select diag CPU       ");
	printf("35: Enable/disable CPU    ");
	printf("36: Display SMP stats	\n");
	printf("37: Clear SMP stats     \n");
#endif
	}
	printf("999: Quit\n");
	printf("Enter Selection: ");
#ifdef LINUX_TEST
	fflush(0);
#endif
	memset(cmd, 0, 80);
	get_cmd(cmd);
	sscanf(cmd, "%d", &sel);
	printf("\n");
	return sel;
} /* bmc12500Eth_uiShowDiagMenu */

/*
 *------------------------------------------------------------------------------
 * Function Name:	bmc12500Eth_uiGetInput	
 *
 * Description: 	Gets console input.
 *
 * Created by:		Frank Salzmann
 *
 * Data Created:	May 09, 2001
 * 					
 * Returns: NFX_OK
 *
 *------------------------------------------------------------------------------
 */
int
bmc12500Eth_uiGetInput( char *msg )
{
	int sel;
	char cmd[80];

	printf("%s: ", msg);
#ifdef LINUX_TEST
	fflush(0);
#else
	memset(cmd, 0, 80);
	get_cmd(cmd);
	sscanf(cmd, "%d", &sel);
#endif
	return sel; 
} /* bmc12500Eth_uiGetInput */

int
bmc12500Eth_uiGetHex( char *msg )
{
    int sel;
    char cmd[80];

    printf("%s: ", msg);
#ifdef LINUX_TEST
    fflush(0);
#else
    memset(cmd, 0, 80);
    get_cmd(cmd);
    sscanf(cmd, "%x", &sel);
#endif
    return sel;
} /* bmc12500Eth_uiGetHex */

/*
 *------------------------------------------------------------------------------
 * Function Name:	bmc12500Eth_uiDiagMenu	
 *
 * Description: 	Runs the BMC12500_ETH diagnostic menu.
 *
 * Created by:		Frank Salzmann
 *
 * Data Created:	May 09, 2001
 * 					
 * Returns: NFX_OK
 *
 *------------------------------------------------------------------------------
 */
int
bmc12500Eth_uiDiagMenu(bc1250EthCfg_t *cfg)
{
	int sel;
#if defined(NFX_SMP) && defined(NFP_FP)
	int i, j;
#endif
	for(;;) {
		printf("BMC12500 ETH: %d\n", cfg->macId);
		printf("================\n");
		sel = bmc12500Eth_uiShowDiagMenu();
		switch(sel)
		{
			case 1:
				bmc12500Eth_uiViewCfg(cfg);
				break;
			case 2:
				bmc12500Eth_uiViewStats(cfg);
				break;
			case 3:
				bmc12500Eth_uiShowRegs(cfg);
				break;
			case 4:
				bmc12500Eth_uiConfigLBMode(cfg);
				break;
			case 5:
				bmc12500Eth_uiShowRxq(cfg);
				break;
			case 6:
				bmc12500Eth_uiShowTxq(cfg);
				break;
			case 7:
				bmc12500Eth_diagTxPkt(cfg);
				break;
			case 8:
				bmc12500Eth_diagRxEna(cfg);
				break;
			case 9:
				bmc12500Eth_diagLBTest(cfg);
				break;
			case 10:
				bmc12500Eth_diagViewTestStats(cfg);
				break;
			case 11:
				bmc12500Eth_uiSetPktTrace(cfg);
				break;
			case 12:
				bmc12500Eth_uiWriteValue(cfg);
				break;
			case 13:
				bmc12500Eth_uiReadValue(cfg);
				break;
			case 14:
				bmc12500Eth_clear_stats(cfg->macId);
				break;
			case 15:
				bmc12500Eth_uiShowMacCfgReg(cfg);
				break;
			case 16:
				bmc12500Eth_mii_viewPhyRegs(cfg);
				break;
			case 17:
				bmc12500Eth_mii_writePhyRegs(cfg);
				break;
			case 18:
				bmc12500Eth_mii_readPhyReg(cfg);
				break;
			case 19:
				bmc12500Eth_uiShowLinkTxStats();
				break;
			case 20:
				bmc12500Eth_uiShowLinkTxq();
				break;
			case 21:
				bmc12500Eth_showRxCapBuf(cfg);
				break;
			case 22:
				bmc12500Eth_uiSetRxCapBuf(cfg);
				break;
			case 23:
				bmc12500Eth_showTxCapBuf(cfg);
				break;
			case 24:
				bmc12500Eth_uiSetTxCapBuf(cfg);
				break;
			case 25:
				bmc12500Eth_diagTxLmux(cfg);
				break;
			case 26:
				bmc12500Eth_diagTxLuc(cfg);
				break;
			case 27:
				luc_show_rev();
				break;
			case 28:
				luc_show_stats();
				break;
			case 29:
				luc_clear_stats();
				break;
#ifdef LMUX_DEBUG
			case 30:
				lmux_show_stats();
				break;
			case 31:
				lmux_clear_stats();
				break;
#endif
			case 32:
				luc_show_regs();
				break;
			case 33:
				lmux_show_rev();
				break;
#if defined(NFX_SMP) && defined(NFP_FP)
			case 34:
				i = bmc12500Eth_uiGetInput("Select diag CPU (0 or 1): "); 
				if (i == 0)
					bc1250_diagCPU = NFX_NFP_FP2;  
				else
					bc1250_diagCPU = NFX_NFP_FP1;  
				break;
			case 35:
				i = bmc12500Eth_uiGetInput("Enable(1)/disable(0): "); 
				if (i) {
					i = bmc12500Eth_uiGetInput("CPU to enable (0 or 1): "); 
					if (i == 0)
						bc1250_FP2CPU_active = 1;
					else
						bc1250_FP1CPU_active = 1;
				}
				else {
					i = bmc12500Eth_uiGetInput("CPU to disable (0 or 1): "); 
					if (i == 0)
						bc1250_FP2CPU_active = 0;
					else
						bc1250_FP1CPU_active = 0;
				}
				printf("CPU 0: ");
				if (bc1250_FP2CPU_active)
					printf("enabled\n");
				else
					printf("disabled\n");
				printf("CPU 1: ");
				if (bc1250_FP1CPU_active)
					printf("enabled\n");
				else
					printf("disabled\n");
				break;
			case 36:
				j = 0;
				for (i = 1; i >= 0; i--) {
					printf("CPU %d -\n", j++);
					printf("Cfg 0, rx 0 lock: %d  unlock: %d\n",
						bcEthCfg[0]->rxc[0].lock_cnt[i], 
						bcEthCfg[0]->rxc[0].unlock_cnt[i]);
					printf("Cfg 0, rx 1 lock: %d  unlock: %d\n",
						bcEthCfg[0]->rxc[1].lock_cnt[i],
						bcEthCfg[0]->rxc[1].unlock_cnt[i]);
					printf("Cfg 2, rx 0 lock: %d  unlock: %d\n",
						bcEthCfg[2]->rxc[0].lock_cnt[i],
						bcEthCfg[2]->rxc[0].unlock_cnt[i]);
					printf("Cfg 2, rx 1 lock: %d  unlock: %d\n",
						bcEthCfg[2]->rxc[1].lock_cnt[i],
						bcEthCfg[2]->rxc[1].unlock_cnt[i]);
					printf("DP proxy app rx lock: %d  unlock %d\n",
						dpp_app_rx_lockcnt[i], dpp_app_rx_unlockcnt[i]);
					printf("DP proxy rx lock: %d  unlock %d\n",
						dpp_rx_lockcnt[i], dpp_rx_unlockcnt[i]);
					printf("DP proxy tx cleanup lock: %d  unlock: %d\n", 
						dpp_tx_cleanup_lockcnt[i], dpp_tx_cleanup_unlockcnt[i]);
					printf("Tx HWM = %d  Rx HWM = %d\n", bmc_txHWM, bmc_rxHWM);
					printf("\n");
				}
				break;
			case 37:
				for (i = 0; i < 2; i++) {
					bcEthCfg[0]->rxc[0].lock_cnt[i] = 0;
					bcEthCfg[0]->rxc[1].lock_cnt[i] = 0;
					bcEthCfg[2]->rxc[0].lock_cnt[i] = 0;
					bcEthCfg[2]->rxc[1].lock_cnt[i] = 0;
					bcEthCfg[0]->rxc[0].unlock_cnt[i] = 0;
					bcEthCfg[0]->rxc[1].unlock_cnt[i] = 0;
					bcEthCfg[2]->rxc[0].unlock_cnt[i] = 0;
					bcEthCfg[2]->rxc[1].unlock_cnt[i] = 0;
					dpp_app_rx_lockcnt[i] = 0;
					dpp_app_rx_unlockcnt[i] = 0;
					dpp_rx_lockcnt[i] = 0;
					dpp_rx_unlockcnt[i] = 0;
					dpp_tx_cleanup_lockcnt[i] = 0;
					dpp_tx_cleanup_unlockcnt[i] = 0;
				}
				break;
#endif
			case 999:
				return 0;
			default:
				break;
		}
	} 
	return(NFX_OK);
} // bmc12500Eth_uiDiagMenu 

/*
 *------------------------------------------------------------------------------
 * Function Name:	bmc12500Eth_uiShowAllRegs
 *
 * Description: 	Prints all of the chip registers.
 *
 * Created by:		Frank Salzmann
 *
 * Data Created:	May 08, 2001
 * 					
 * Returns: NFX_OK
 *
 *------------------------------------------------------------------------------
 */
int
bmc12500Eth_uiShowRegs( bc1250EthCfg_t *cfg )
{
	int				i;
	bc1250Eth_regs_t *r;

	r = &cfg->regs;
	printf("BMC12500ETH OPERATIONAL REGISTER DUMP: [Slot:%02d -- CPU:%02d]\n", 
		   nfxMySlotId,
		   nfxMyCPUId);
	printf("========================================================== \n\n");
	printf("DMA CONFIG0 RX 0      :%016llx    ", bmc12500Eth_readReg(r->dma_config0_rx[0]));
	printf("DMA CONFIG0 RX 1      :%016llx\n", bmc12500Eth_readReg(r->dma_config0_rx[1]));
	printf("DMA CONFIG1 RX 0      :%016llx    ", bmc12500Eth_readReg(r->dma_config1_rx[0]));
	printf("DMA CONFIG1 RX 1      :%016llx\n", bmc12500Eth_readReg(r->dma_config1_rx[1]));

	printf("DMA CONFIG0 TX 0      :%016llx    ", bmc12500Eth_readReg(r->dma_config0_tx[0]));
	printf("DMA CONFIG0 TX 1      :%016llx\n", bmc12500Eth_readReg(r->dma_config0_tx[1]));
	printf("DMA CONFIG1 TX 0      :%016llx    ", bmc12500Eth_readReg(r->dma_config1_tx[0]));
	printf("DMA CONFIG1 TX 1      :%016llx\n", bmc12500Eth_readReg(r->dma_config1_tx[1]));
	printf("----------------------:\n");
	printf("DMA DESC BASE RX 0    :%016llx    ", bmc12500Eth_readReg(r->dma_dscr_base_rx[0]));
	printf("DMA DESC BASE RX 1    :%016llx\n", bmc12500Eth_readReg(r->dma_dscr_base_rx[1]));
	printf("DMA DESC CNT RX 0     :%016llx    ", bmc12500Eth_readReg(r->dma_dscr_cnt_rx[0]));
	printf("DMA DESC CNT RX 1     :%016llx\n", bmc12500Eth_readReg(r->dma_dscr_cnt_rx[1]));

	printf("DMA CUR DESC addr RX 0:%016llx    ", bmc12500Eth_readReg(r->dma_cur_dscr_addr_rx[0]));
	printf("DMA CUR DESC addr RX 1:%016llx\n", bmc12500Eth_readReg(r->dma_cur_dscr_addr_rx[1]));
	printf("DMA CUR DESC_A RX 0   :%016llx    ", bmc12500Eth_readReg(r->dma_cur_dscra_rx[0]));
	printf("DMA CUR DESC_A RX 1   :%016llx\n", bmc12500Eth_readReg(r->dma_cur_dscra_rx[1]));
	printf("DMA CUR DESC_B RX 0   :%016llx    ", bmc12500Eth_readReg(r->dma_cur_dscrb_rx[0]));
	printf("DMA CUR DESC_B RX 1   :%016llx\n", bmc12500Eth_readReg(r->dma_cur_dscrb_rx[1]));
	printf("----------------------:\n");
	printf("DMA DESC BASE TX 0    :%016llx    ", bmc12500Eth_readReg(r->dma_dscr_base_tx[0]));
	printf("DMA DESC BASE TX 1    :%016llx\n", bmc12500Eth_readReg(r->dma_dscr_base_tx[1]));
	printf("DMA DESC CNT TX 0     :%016llx    ", bmc12500Eth_readReg(r->dma_dscr_cnt_tx[0]));
	printf("DMA DESC CNT TX 1     :%016llx\n", bmc12500Eth_readReg(r->dma_dscr_cnt_tx[1]));

	printf("DMA CUR DESC addr TX 0:%016llx    ", bmc12500Eth_readReg(r->dma_cur_dscr_addr_tx[0]));
	printf("DMA CUR DESC addr TX 1:%016llx\n", bmc12500Eth_readReg(r->dma_cur_dscr_addr_tx[1]));
	printf("DMA CUR DESC_A TX 0   :%016llx    ", bmc12500Eth_readReg(r->dma_cur_dscra_tx[0]));
	printf("DMA CUR DESC_A TX 1   :%016llx\n", bmc12500Eth_readReg(r->dma_cur_dscra_tx[1]));
	printf("DMA CUR DESC_B TX 0   :%016llx    ", bmc12500Eth_readReg(r->dma_cur_dscrb_tx[0]));
	printf("DMA CUR DESC_B TX 1   :%016llx\n", bmc12500Eth_readReg(r->dma_cur_dscrb_tx[1]));

	printf("\n");
	printf("DMA ASIC addr         :%016llx\n", bmc12500Eth_readReg(r->dma_asic_addr));
	printf("Mac Cfg               :%016llx\n", bmc12500Eth_readReg(r->mac_cfg));
	printf("Mac Enable            :%016llx\n", bmc12500Eth_readReg(r->mac_enable));
	printf("Mac txd ctl           :%016llx\n", bmc12500Eth_readReg(r->mac_txd_ctl));
	printf("Mac thrsh cfg         :%016llx\n", bmc12500Eth_readReg(r->mac_thrsh_cfg));
	printf("Mac frame cfg         :%016llx\n", bmc12500Eth_readReg(r->mac_frame_cfg));
	printf("Mac vlan tag          :%016llx\n", bmc12500Eth_readReg(r->mac_vlantag));
	printf("Mac status            :%016llx\n", bmc12500Eth_readReg(r->mac_status));
	printf("Mac debug status      :%016llx\n", bmc12500Eth_readReg(r->mac_debug_status));
	printf("Mac int mask          :%016llx\n", bmc12500Eth_readReg(r->mac_int_mask));
	printf("Mac fifo ptrs         :%016llx\n", bmc12500Eth_readReg(r->mac_fifo_ptrs));
	printf("Mac eopcnt            :%016llx\n", bmc12500Eth_readReg(r->mac_eopcnt));
	for(i=0;i<8;i++)
	printf("Mac addr %d            :%016llx\n", i, bmc12500Eth_readReg(r->mac_addr[i]));
	for(i=0;i<8;i++)
	printf("Mac hash %d            :%016llx\n", i, bmc12500Eth_readReg(r->mac_hash[i]));
	printf("Mac ethernet addr     :%016llx\n", bmc12500Eth_readReg(r->mac_ethernet_addr));
	printf("Mac type cfg          :%016llx\n", bmc12500Eth_readReg(r->mac_type_cfg));
	printf("Mac adfilter cfg      :%016llx\n", bmc12500Eth_readReg(r->mac_adfilter_cfg));
	for(i=0;i<4;i++)
	printf("Mac chlo %d            :%016llx\n", i, bmc12500Eth_readReg(r->mac_chlo[i]));
	for(i=0;i<4;i++)
	printf("Mac chup %d            :%016llx\n", i, bmc12500Eth_readReg(r->mac_chup[i]));
	printf("Mac mdio              :%016llx\n", bmc12500Eth_readReg(r->mac_mdio));
	printf("\n");
	return(NFX_OK);
} // bmc12500Eth_uiShowAllRegs

/*
 *------------------------------------------------------------------------------
 * Function Name:	bmc12500Eth_uiViewStats	
 *
 * Description: 	Display device/driver statistics.
 *
 * Created by:		Frank Salzmann
 *
 * Data Created:	May 09, 2001
 * 					
 * Returns: NFX_OK
 *
 *------------------------------------------------------------------------------
 */
int
bmc12500Eth_uiViewStats(bc1250EthCfg_t *cfg)
{
	bc1250Eth_regs_t *regs;
	bc1250Eth_rmon_t *rm;

	regs = &cfg->regs;
	rm = (bc1250Eth_rmon_t *)regs->rmon;

	printf("BMC12500ETH STATISTICS: [Slot:%02d -- CPU:%02d]\n", 
		   nfxMySlotId,
		   nfxMyCPUId);
	printf("=========================================== \n\n");
	printf("Rx Octets            : %016lld\n", bmc12500Eth_readReg(&rm->rx_byte_cnt));
	printf("Rx Good Packets      : %016lld\n", bmc12500Eth_readReg(&rm->rx_good_cnt));
	printf("Rx Multicast         : %016lld\n", bmc12500Eth_readReg(&rm->rx_mcst_cnt));
	printf("Rx Broadcast         : %016lld\n", bmc12500Eth_readReg(&rm->rx_bcst_cnt));
	printf("Rx Bad Packets       : %016lld\n", bmc12500Eth_readReg(&rm->rx_badp_cnt));
	printf("Rx Runt Errors       : %016lld\n", bmc12500Eth_readReg(&rm->rx_runt_cnt));
	printf("Rx Over Sized Errors : %016lld\n", bmc12500Eth_readReg(&rm->rx_ovsz_cnt));
	printf("Rx FCS Errors        : %016lld\n", bmc12500Eth_readReg(&rm->rx_fcse_cnt));
	printf("Rx Length Errors     : %016lld\n", bmc12500Eth_readReg(&rm->rx_lene_cnt));
	printf("Rx Code Errors       : %016lld\n", bmc12500Eth_readReg(&rm->rx_code_cnt));
	printf("Rx Dribble Packets   : %016lld\n", bmc12500Eth_readReg(&rm->rx_drbl_cnt));
    printf("Rx Bad Ch0 (manual)  : %016d\n", cfg->rxc[0].rxBadErr);
	printf("Rx No Buffer Ch0     : %016d\n", cfg->rxc[0].noBufErr);
	printf("Rx No Pkt Desc Ch0   : %016d\n", cfg->rxc[0].noDescErr);
	printf("Rx Low Descriptor Ch0: %016d\n", cfg->rxc[0].lowDesc);
	if (BC1250_MAC_RX_Q_CNT == 2) {
    	printf("Rx Bad Ch1 (manual)  : %016d\n", cfg->rxc[1].rxBadErr);
		printf("Rx No Buffer Ch1     : %016d\n", cfg->rxc[1].noBufErr);
		printf("Rx No Pkt Desc Ch1   : %016d\n", cfg->rxc[1].noDescErr);
		printf("Rx Low Descriptor Ch1: %016d\n", cfg->rxc[1].lowDesc);
	}
	printf("\n");
	printf("Tx Octets            : %016lld\n", bmc12500Eth_readReg(&rm->tx_byte_cnt));
	printf("Tx Good Packets      : %016lld\n", bmc12500Eth_readReg(&rm->tx_good_cnt));
	printf("Tx Bad Packets       : %016lld\n", bmc12500Eth_readReg(&rm->tx_badp_cnt));
	printf("Tx Runt Errors       : %016lld\n", bmc12500Eth_readReg(&rm->tx_runt_cnt));
	printf("Tx Over Sized Errors : %016lld\n", bmc12500Eth_readReg(&rm->tx_ovsz_cnt));
	printf("Tx FCS Errors        : %016lld\n", bmc12500Eth_readReg(&rm->tx_fcse_cnt));
	printf("Tx Collisions        : %016lld\n", bmc12500Eth_readReg(&rm->tx_coll_cnt));
	printf("Tx Late Collision    : %016lld\n", bmc12500Eth_readReg(&rm->tx_lcol_cnt));
	printf("Tx Excess Collision  : %016lld\n", bmc12500Eth_readReg(&rm->tx_xcol_cnt));
	printf("Tx Abort             : %016lld\n", bmc12500Eth_readReg(&rm->tx_abrt_cnt));
	printf("Tx Q Full Ch0        : %016d\n", cfg->txc[0].qFull);
	if (BC1250_MAC_TX_Q_CNT == 2) {
		printf("Tx Q Full Ch1        : %016d\n", cfg->txc[1].qFull);
	}
	printf("\n");
	return(NFX_OK);
} // bmc12500Eth_uiViewStats 

char *bc_link_speed_str[] = {"10Mbps-","100Mbps-","1000Mbps-","unknown"};
char *bc_link_duplex_str[] = {"hdx", "fdx", ""};
char *bc_link_fc_str[] = {"disabled", "frame", "collision", "carrier","unknown"};
char *bc_link_lb_str[] = {"none", "internal", "phy", "external"};

/*
 *------------------------------------------------------------------------------
 * Function Name:	bmc12500Eth_uiViewCfg
 *
 * Description: 	Display device/driver configuration.
 *
 * Created by:		Frank Salzmann
 *
 * Data Created:	June 20, 2001
 * 					
 * Returns: NFX_OK
 *
 *------------------------------------------------------------------------------
 */
int
bmc12500Eth_uiViewCfg(bc1250EthCfg_t *cfg)
{
	uint32 i;
	bc1250_ltc_t *ltc;

	printf("BCM12500 ETHERNET CONFIGURATION: [Slot:%02d -- CPU:%02d -- PORT:%02d]\n", 
		   nfxMySlotId, nfxMyCPUId, cfg->macId);
	printf("=============== General Config ============== \n\n");
	printf("Application Id             : %08d\n", cfg->appId);
	printf("EEE Port Address           : %08x\n", cfg->myPortAddr);
	printf("\n");
	printf("=============== Link Control ================ \n\n");
	printf("Link State:                : %s\n", 
		   cfg->link.linkstat == ETHER_SPEED_UNKNOWN ? "down":"up");
	printf("Link Speed:                : %s%s\n", bc_link_speed_str[cfg->link.speed],
		                                           bc_link_duplex_str[cfg->link.duplex]);
	printf("Link Flow Control Mode     : %s\n", bc_link_fc_str[cfg->link.fc]);
	printf("Link Loopback Mode         : %s\n", bc_link_lb_str[cfg->link.loopback]);

	printf("\n");
	printf("=============== Transmit Control ============== \n\n");
	for(i=0; i < BC1250_MAC_TX_Q_CNT; i++) {
		printf("TxQ %d Start                : %p\n", i, cfg->txc[i].txqStart);
		printf("TxQ %d End                  : %p\n", i, cfg->txc[i].txqEnd);
		printf("TxQ %d Current Addr         : %p\n", i, cfg->txc[i].txCur);
		printf("TxQ %d Cleanup Addr         : %p\n", i, cfg->txc[i].txCln);
		printf("TxQ %d Desc Q'd             : %08d\n", i, cfg->txc[i].qd);
	}
	printf("\n");
	printf("=============== Receive Control ============== \n\n");
	for(i=0; i < BC1250_MAC_RX_Q_CNT; i++) {
		printf("Rx Desc Pool %d Base Addr   : %p\n", i, cfg->rxQ[i]);
		printf("RxQ %d Start                : %p\n", i, cfg->rxc[i].rxqStart);
		printf("RxQ %d End                  : %p\n", i, cfg->rxc[i].rxqEnd);
		printf("RxQ %d Current Addr         : %p\n", i, cfg->rxc[i].rxCur);
	}
	if (isCheetah()) {
		printf("\n");
		printf("=============== Link Tx Q Control ============== \n\n");
		for(i=0; i < BC1250_LINK_TX_Q_CNT; i++) {
			ltc = bcLinkTxqs[i];
			if( !ltc )
				continue;
			printf("Link TxQ %d Start           : %p\n", i, ltc->qStart);
			printf("Link TxQ %d End             : %p\n", i, ltc->qEnd);
			printf("Link TxQ %d addCur          : %p\n", i, ltc->qAddCur);
			printf("Link TxQ %d remCur          : %p\n", i, ltc->qRemCur);
		}
#if defined(NFX_SMP) && defined(NFP_FP)
		printf("For 2nd CPU:\n");
		for(i=0; i < BC1250_LINK_TX_Q_CNT; i++) {
			ltc = bcLinkTxqs_2ndCPU[i];
			if( !ltc )
				continue;
			printf("Link TxQ %d Start           : %p\n", i, ltc->qStart);
			printf("Link TxQ %d End             : %p\n", i, ltc->qEnd);
			printf("Link TxQ %d addCur          : %p\n", i, ltc->qAddCur);
			printf("Link TxQ %d remCur          : %p\n", i, ltc->qRemCur);
		}
#endif
	}
	printf("\n");
	return(NFX_OK);
} // bmc12500Eth_uiViewCfg 

/*
 *------------------------------------------------------------------------------
 * Function Name:	bmc12500Eth_uiSetLoopbackMode
 *
 * Description: 	Allows user to enable/disable device loopback.
 *
 * Created by:		Frank Salzmann
 *
 * Data Created:	May 08, 2001
 * 					
 * Returns: NFX_OK
 *
 *------------------------------------------------------------------------------
 */
int
bmc12500Eth_uiConfigLBMode( bc1250EthCfg_t *cfg )
{
	int32 mode;

	if (isCheetah()) {
		printf("Not supported!\n");
		return(NFX_OK);
	}
	mode = bmc12500Eth_uiGetInput("Set Loopback Mode [OFF:1 | INT:2 | PHY:3 | EXT:4]: \n"); 
	if((mode != 1) && (mode != 2) && (mode !=3) && mode != 4) { 
		printf("Invalid Mode: %d. Exiting....\n", mode);
        return(NFX_OK);
    }    
	switch(mode) {

		case 1:
			bmc12500Eth_diagSetLbMode(cfg, BC1250_MAC_LB_DISABLE); 
			bmc12500Eth_mii_phyLoopback(cfg, BC1250_CLEAR);
			bmc12500Eth_setPromiscMode( cfg, BC1250_CLEAR);
			cfg->link.loopback = lb_none;
			break;
		case 2:
			bmc12500Eth_diagSetLbMode(cfg, BC1250_MAC_LB_ENABLE); 
			break;
		case 3:
			bmc12500Eth_mii_phyLoopback(cfg, BC1250_SET);
			bmc12500Eth_setPromiscMode( cfg, BC1250_SET);
			cfg->link.loopback = lb_phy;
			break;
		case 4:
			cfg->link.loopback = lb_ext;
			break;
	}

	return(NFX_OK);
} /* bmc12500Eth_uiConfigLBMode */

/*
 *------------------------------------------------------------------------------
 * Function Name:	bmc12500Eth_uiShowRxq
 *
 * Description: 	Dump the chip receive descriptor queue.
 *
 * Created by:		Frank Salzmann
 *
 * Data Created:	June 22, 2001
 * 					
 * Returns: NFX_OK
 *
 *------------------------------------------------------------------------------
 */
int
bmc12500Eth_uiShowRxq( bc1250EthCfg_t *dp)
{
	int i, quit, q;
	volatile bc1250_rxdesc_t *descp;

	q = bmc12500Eth_uiGetInput("Enter Q Number [0|1]: \n"); 
	if((q != 0) && (q != 1) ) { 
		printf("Invalid Queue: %d. Exiting....\n", q);
        return(NFX_OK);
    }    
	if ((q == 1) && (BC1250_MAC_RX_Q_CNT < 2)) {
		printf("Queue not available\n");
		return(NFX_OK);
	}
	descp = dp->rxc[q].rxqStart;

	printf("BMC12500ETH Receive Descriptor Queue Dump\n");
	printf("=====================================\n");
	for(i=0; i < BC1250_MAC_RX_Q_LEN; i++) { 
		printf("%03d  Adr: %lx  Word_1: %016llx  Word_2: %016llx  ", i, 
				(address_t)descp,descp->dw1, descp->dw2);
		printf("Sz: %lld\n", BC1250_GET_PKT_SIZE(descp));
		descp++;
		if( !(i % 25) && i != 0 ) {
			quit = bmc12500Eth_uiGetInput("Quit [99]: ");
			if( quit == 99 )
				return 0;
		}
	}
	printf("\n");
	return 0;
} // bmc12500Eth_uiShowRxq 

/*
 *------------------------------------------------------------------------------
 * Function Name:	bmc12500Eth_uiShowTxq
 *
 * Description: 	Dump the chip transmit descriptor queue.
 *
 * Created by:		Frank Salzmann
 *
 * Data Created:	June 22, 2001
 * 					
 * Returns: NFX_OK
 *
 *------------------------------------------------------------------------------
 */
int
bmc12500Eth_uiShowTxq( bc1250EthCfg_t *dp)
{
	int i, quit, q;
	volatile bc1250_txdesc_t *descp;

	q = bmc12500Eth_uiGetInput("Enter Q Number [0|1]: \n"); 
	if((q != 0) && (q != 1) ) { 
		printf("Invalid Queue: %d. Exiting....\n", q);
        return(NFX_OK);
    }    
	if ((q == 1) && (BC1250_MAC_TX_Q_CNT < 2)) {
		printf("Queue not available\n");
		return(NFX_OK);
	}
	descp = dp->txc[q].txqStart;

	printf("BMC12500ETH Transmit Descriptor Queue Dump\n");
	printf("======================================\n");
	for(i=0; i < BC1250_MAC_TX_Q_LEN; i++) {
		printf("%03d   Word_1: %016llx  Word_2: %016llx  ", i, descp->dw1, descp->dw2 );
#if BC1250ETH_TXQ_LIST
		printf("edesc: %llx, pktSz: %lld\n", descp->edesc, 
			   BC1250_GET_PKT_SIZE(descp));
#else
		printf("pktSz: %d\n", BC1250_GET_PKT_SIZE(descp));
#endif	
		descp++;
		if( !(i % 25) && i != 0 ) {
			quit = bmc12500Eth_uiGetInput("Quit [99]: ");
			if( quit == 99 )
				return 0;
		}
	}
	printf("\n");
	return 0;
} // bmc12500Eth_uiShowTxq 

/*
 *-----------------------------------------------------------------------------
 * Function Name:   bmc12500Eth_uiSetPktTrace
 *
 * Description:     Enables/Disables packet tracing in bmc12500Eth driver.
 *
 * Created by:      Frank Salzmann
 *
 * Data Created:    August 08, 2001
 *
 * Returns: NFX_OK
 *
 *-----------------------------------------------------------------------------
 */
int
bmc12500Eth_uiSetPktTrace(bc1250EthCfg_t *dp)
{
	uint32        sel;
	extern uint32 bmc12500Eth_rx_pkt_trace;
	extern uint32 bmc12500Eth_tx_pkt_trace;

	printf("Rx Pkt Trace: %s, Tx Pkt Trace: %s\n",
					(bmc12500Eth_rx_pkt_trace == 0) ? "DISABLED" : "ENABLED",
					(bmc12500Eth_tx_pkt_trace == 0) ? "DISABLED" : "ENABLED");
    sel = bmc12500Eth_uiGetInput("Rx Trace Dis/Ena [0|1], Tx Trace Dis/Ena [2|3]: ");
	switch(sel) {
	
		case 0:
			bmc12500Eth_rx_pkt_trace = 0;
			break;
		case 1:
			bmc12500Eth_rx_pkt_trace = 1;
			break;
		case 2:
			bmc12500Eth_tx_pkt_trace = 0;
			break;
		case 3:
			bmc12500Eth_tx_pkt_trace = 1;
			break;
		default: 
			printf("Invalid Entry: %d\n", sel);
	}
	printf("Rx Pkt Trace: %s, Tx Pkt Trace: %s\n",
					(bmc12500Eth_rx_pkt_trace == 0) ? "DISABLED" : "ENABLED",
					(bmc12500Eth_tx_pkt_trace == 0) ? "DISABLED" : "ENABLED");
    printf("\n");
    return 0;
} // bmc12500Eth_uiSetPktTrace

/*
 *------------------------------------------------------------------------------
 * Function Name:	bmc12500Eth_uiWriteValue
 *
 * Description: 	Write a 64-bit value into memory or register.
 *
 * Created by:		Frank Salzmann
 *
 * Data Created:	November 06, 2001
 * 					
 * Returns: NFX_OK
 *
 *------------------------------------------------------------------------------
 */
int
bmc12500Eth_uiWriteValue( bc1250EthCfg_t *cfg )
{
	uint64 *addr;
	uint64 value;
	uint64 dummy;

	dummy = bmc12500Eth_uiGetInput("Address: ");
	addr = (uint64 *)dummy;
	if((addr == 0) || ((uint64)addr & 0xf)) { 
		printf("Invalid Address: %lx\n", (address_t)addr);
        return(NFX_OK);
    }    
	value = bmc12500Eth_uiGetInput("Value: ");
	
	bmc12500Eth_writeReg(addr, value);

	return(NFX_OK);
} /* bmc12500Eth_uiWriteValue */

/*
 *------------------------------------------------------------------------------
 * Function Name:	bmc12500Eth_uiReadValue
 *
 * Description: 	Read a 64-bit value from memory or register.
 *
 * Created by:		Frank Salzmann
 *
 * Data Created:	November 06, 2001
 * 					
 * Returns: NFX_OK
 *
 *------------------------------------------------------------------------------
 */
int
bmc12500Eth_uiReadValue( bc1250EthCfg_t *cfg )
{
	uint64 *addr;
	uint64 value;
	uint64 dummy;

	dummy = bmc12500Eth_uiGetInput("Address: ");
	addr = (uint64 *)dummy;
	if((addr == 0) || ((uint64)addr & 0xf)) { 
		printf("Invalid Address: %lx\n", (address_t)addr);
        return(NFX_OK);
    }    
	
	value = bmc12500Eth_readReg(addr);
	printf("Addr: %lx, Value: %016llx\n", (address_t)addr, value);

	return(NFX_OK);
} /* bmc12500Eth_uiReadValue */
/*
 *------------------------------------------------------------------------------
 * Function Name:	bmc12500Eth_uiShowMacCfgReg
 *
 * Description: 	Displays detailed view ofthe Mac CFG Reg bits.
 *
 * Created by:		Frank Salzmann
 *
 * Data Created:	November 06, 2001
 * 					
 * Returns: NFX_OK
 *
 *------------------------------------------------------------------------------
 */
int
bmc12500Eth_uiShowMacCfgReg( bc1250EthCfg_t *cfg )
{
	bc1250Eth_regs_t  *r;
	bmc12500_macCfgr_t macCfg;

	r = &cfg->regs;
	
	macCfg.wd = bmc12500Eth_readReg(r->mac_cfg);
	printf("Reg Value        : %016llx\n", macCfg.wd);
	printf("ss_tmode         : %x\n", macCfg.bits.ss_tmode);         
    printf("tx_hold_sop_en   : %x\n", macCfg.bits.tx_hold_sop_en);           
    printf("retry_en         : %x\n", macCfg.bits.retry_en);            
    printf("ret_drpreq_en    : %x\n", macCfg.bits.ret_drpreq_en);     
    printf("ret_ufl_en       : %x\n", macCfg.bits.ret_ufl_en   ); 
    printf("burst_en         : %x\n", macCfg.bits.burst_en  ); 
    printf("tx_pause_cnt     : %x\n", macCfg.bits.tx_pause_cnt);   
    printf("ap_stat_en       : %x\n", macCfg.bits.ap_stat_en  ); 
    printf("drp_errpkt_en    : %x\n", macCfg.bits.drp_errpkt_en);   
    printf("drp_fcserrpkt_en : %x\n", macCfg.bits.drp_fcserrpkt_en);  
    printf("drp_codeerrpkt_en: %x\n", macCfg.bits.drp_codeerrpkt_en);   
    printf("drp_drblerrpkt_en: %x\n", macCfg.bits.drp_drblerrpkt_en ); 
    printf("drp_rntpkt_en    : %x\n", macCfg.bits.drp_rntpkt_en  ); 
    printf("drp_oszpkt_en    : %x\n", macCfg.bits.drp_oszpkt_en  ); 
    printf("drp_lenerrpkt_en : %x\n", macCfg.bits.drp_lenerrpkt_en);   
    printf("bypass_sel       : %x\n", macCfg.bits.bypass_sel   ); 
    printf("hdx_en           : %x\n", macCfg.bits.hdx_en  ); 
    printf("speed_sel        : %x\n", macCfg.bits.speed_sel);   
    printf("tx_clk_edge      : %x\n", macCfg.bits.tx_clk_edge);   
    printf("loopback_sel     : %x\n", macCfg.bits.loopback_sel);  
    printf("fast_sync        : %x\n", macCfg.bits.fast_sync  ); 
    printf("ss_en            : %x\n", macCfg.bits.ss_en  ); 
    printf("bypass_cfg       : %x\n", macCfg.bits.bypass_cfg );  
    printf("bypass_16        : %x\n", macCfg.bits.bypass_16  ); 
    printf("bypass_fcs_chk   : %x\n", macCfg.bits.bypass_fcs_chk);   
    printf("bypass_ifg       : %x\n", macCfg.bits.bypass_ifg   ); 
    printf("fc_sel           : %x\n", macCfg.bits.fc_sel   ); 
    printf("fc_cmd           : %x\n", macCfg.bits.fc_cmd   ); 
    printf("rx_ch_sel        : %x\n", macCfg.bits.rx_ch_sel);   
	return(NFX_OK);
} // bmc12500Eth_uiShowMacCfgReg 

/*
 *------------------------------------------------------------------------------ 
 * Function Name:   bmc12500Eth_bpStats
 *
 * Description:     Displays port stats.
 *
 * Created by:      Frank Salzmann
 *
 * Data Created:    November 06, 2001
 *
 * Returns: NFX_OK
 *
 *------------------------------------------------------------------------------ */
int
bmc12500Eth_bpStats(uint32 port)
{
    bc1250EthCfg_t *cfg;
    bc1250Eth_regs_t *regs;
    bc1250Eth_rmon_t *rm;

    cfg = bcEthCfg[port];
    if (!cfg)
        return(NFX_ERR);
    regs = &cfg->regs;
    rm = (bc1250Eth_rmon_t *)regs->rmon;
	if (!isCheetah()) {
    printf("Rx Octets            : %016lld\n", bmc12500Eth_readReg(&rm->rx_byte_cnt));
    printf("Rx Good Packets      : %016lld\n", bmc12500Eth_readReg(&rm->rx_good_cnt));
    printf("Rx Multicast         : %016lld\n", bmc12500Eth_readReg(&rm->rx_mcst_cnt));
    printf("Rx Broadcast         : %016lld\n", bmc12500Eth_readReg(&rm->rx_bcst_cnt));
	}
    printf("Rx Bad Packets       : %016lld\n", bmc12500Eth_readReg(&rm->rx_badp_cnt));
    printf("Rx Runt Errors       : %016lld\n", bmc12500Eth_readReg(&rm->rx_runt_cnt));
    printf("Rx Over Sized Errors : %016lld\n", bmc12500Eth_readReg(&rm->rx_ovsz_cnt));
    printf("Rx FCS Errors        : %016lld\n", bmc12500Eth_readReg(&rm->rx_fcse_cnt));
    printf("Rx Length Errors     : %016lld\n", bmc12500Eth_readReg(&rm->rx_lene_cnt));
    printf("Rx Code Errors       : %016lld\n", bmc12500Eth_readReg(&rm->rx_code_cnt));
    printf("Rx Dribble Packets   : %016lld\n", bmc12500Eth_readReg(&rm->rx_drbl_cnt));
    printf("Rx Bad Ch0 (manual)  : %08d\n", cfg->rxc[0].rxBadErr);
	printf("Rx No Buffer Ch0     : %08d\n", cfg->rxc[0].noBufErr);
	printf("Rx No Pkt Desc Ch0   : %08d\n", cfg->rxc[0].noDescErr);
	printf("Rx Low Descriptor Ch0: %08d\n", cfg->rxc[0].lowDesc);
	if (BC1250_MAC_RX_Q_CNT == 2) {
    	printf("Rx Bad Ch1 (manual)  : %08d\n", cfg->rxc[1].rxBadErr);
		printf("Rx No Buffer Ch1     : %08d\n", cfg->rxc[1].noBufErr);
		printf("Rx No Pkt Desc Ch1   : %08d\n", cfg->rxc[1].noDescErr);
		printf("Rx Low Descriptor Ch1: %08d\n", cfg->rxc[1].lowDesc);
	}
	printf("\n");
	if (!isCheetah()) {
    printf("Tx Octets            : %016lld\n", bmc12500Eth_readReg(&rm->tx_byte_cnt));
    printf("Tx Good Packets      : %016lld\n", bmc12500Eth_readReg(&rm->tx_good_cnt));
	}
    printf("Tx Bad Packets       : %016lld\n", bmc12500Eth_readReg(&rm->tx_badp_cnt));
    printf("Tx Runt Errors       : %016lld\n", bmc12500Eth_readReg(&rm->tx_runt_cnt));
    printf("Tx Over Sized Errors : %016lld\n", bmc12500Eth_readReg(&rm->tx_ovsz_cnt));
    printf("Tx FCS Errors        : %016lld\n", bmc12500Eth_readReg(&rm->tx_fcse_cnt));
    printf("Tx Collisions        : %016lld\n", bmc12500Eth_readReg(&rm->tx_coll_cnt));
    printf("Tx Late Collision    : %016lld\n", bmc12500Eth_readReg(&rm->tx_lcol_cnt));
    printf("Tx Excess Collision  : %016lld\n", bmc12500Eth_readReg(&rm->tx_xcol_cnt));
    printf("Tx Abort             : %016lld\n", bmc12500Eth_readReg(&rm->tx_abrt_cnt));
	printf("Tx Q Full Ch0        : %08d\n", cfg->txc[0].qFull);
	if (BC1250_MAC_TX_Q_CNT == 2) {
		printf("Tx Q Full Ch1        : %08d\n", cfg->txc[1].qFull);
	}
	printf("\n");
	return(NFX_OK);
}

void
bmc12500Eth_clear_stats(uint32 port)
{
    bc1250EthCfg_t *cfg;
	int i;

    cfg = bcEthCfg[port];
    if (!cfg)
        return;

	bmc12500Eth_initRmonRegs(cfg);
	for(i=0; i < BC1250_MAC_RX_Q_CNT; i++) {
    	cfg->rxc[i].rxBadErr = 0;
		cfg->rxc[i].noBufErr = 0;
		cfg->rxc[i].noDescErr = 0;
	}
	for(i=0; i < BC1250_MAC_TX_Q_CNT; i++) {
		cfg->txc[i].qFull = 0;
	}
}

/*
 *------------------------------------------------------------------------------ 
 * Function Name:   bmc12500Eth_linkTxStats
 *
 * Description:     Displays link txq stats.
 *
 * Created by:      Frank Salzmann
 *
 * Data Created:    February 28, 2002
 *
 * Returns: NFX_OK
 *------------------------------------------------------------------------------ */
int
bmc12500Eth_uiShowLinkTxStats()
{
	uint32 i;
	bc1250_ltc_t *ltc;

	for(i=0; i < BC1250_LINK_TX_Q_CNT; i++ ) {
		ltc = bcLinkTxqs[i];
		printf("Link %d Tx Q Stats:\n", i);
		printf("-------------------\n");
		printf("Tx Pkt Cnt    : %08d   ", ltc->qdTotal);
		printf("Tx Byte Cnt: %08d   ", ltc->octets);
		printf("Q Full Cnt: %08d\n", ltc->qFullErr);
		printf("Qd Current Cnt: %08d   ", ltc->qdCnt);
		printf("Deficit Cnt: %08d\n", ltc->dCnt);
		printf("Zdiff         : %lld   \n", ltc->zdiff);
		printf("\n");
	}
#if defined(NFX_SMP) && defined(NFP_FP)
	printf("For 2nd CPU:\n");
	for(i=0; i < BC1250_LINK_TX_Q_CNT; i++ ) {
		ltc = bcLinkTxqs_2ndCPU[i];
		printf("Link %d Tx Q Stats:\n", i);
		printf("-------------------\n");
		printf("Tx Pkt Cnt    : %08d   ", ltc->qdTotal);
		printf("Tx Byte Cnt: %08d   ", ltc->octets);
		printf("Q Full Cnt: %08d\n", ltc->qFullErr);
		printf("Qd Current Cnt: %08d   ", ltc->qdCnt);
		printf("Deficit Cnt: %08d\n", ltc->dCnt);
		printf("Zdiff         : %lld   \n", ltc->zdiff);
		printf("\n");
	}
#endif
	return NFX_OK;
} // bmc12500_uiShowLinkTxStats

/*
 *------------------------------------------------------------------------------
 * Function Name:	bmc12500Eth_uiShowLinkTxq
 *
 * Description: 	Dump the link transmit descriptor queue.
 *
 * Created by:		Frank Salzmann
 *
 * Data Created:	June 22, 2001
 * 					
 * Returns: NFX_OK
 *------------------------------------------------------------------------------
 */
int
bmc12500Eth_uiShowLinkTxq()
{
	volatile bc1250_ld_t *descp;
	int i, quit, q;

#if defined(NFX_SMP) && defined(NFP_FP)
	i = bmc12500Eth_uiGetInput("Enter which CPU's queue [0|1]: \n");
	if ((i != 0) && (i != 1)) {
		printf("Invalid entry!\n");
		return(NFX_OK);
	}
#endif
	q = bmc12500Eth_uiGetInput("Enter Q Number [0|1|2|3]: \n"); 
	if((q < 0) || (q > 3) ) { 
		printf("Invalid Queue: %d. Exiting....\n", q);
        return(NFX_OK);
    }    

#if defined(NFX_SMP) && defined(NFP_FP)
	if (i == 1)
		descp = bcLinkTxqs_2ndCPU[q]->qStart;
	else
#endif
	descp = bcLinkTxqs[q]->qStart;

	printf("BMC12500ETH Link Tx Descriptor Queue Dump\n");
	printf("=========================================\n");
	for(i=0; i < BC1250_LINK_TX_Q_LEN; i++) {
		printf("%03d: ctrl: %08x  pktSz: %08d  dest: %08x  edesc: %lx\n", 
			   i, 
			   descp->control, 
			   descp->pktLen,
			   descp->dest,
			   (address_t)descp->o.edesc);

		descp++;
		if( !(i % 25) && i != 0 ) {
			quit = bmc12500Eth_uiGetInput("Quit [99]: ");
			if( quit == 99 )
				return 0;
		}
	}
	printf("\n");
	return 0;
} // bmc12500Eth_uiShowLinkTxq 

void
luc_show_rev()
{
    uint32 *luc_sw_rev;
    uint32 *luc_hw_rev;
    uint32 reg_val, high, low;

	if (!isCheetah()) {
		printf("luc_show_rev: Only available on Cheetah!\n");
		return;
	}
	if (!isTXRX()) {
		printf("luc_show_rev: Only available on TXRX!\n");
		return;
	}
    luc_sw_rev = (uint32 *)LUC_SW_REV_REG;
    luc_hw_rev = (uint32 *)LUC_HW_REV_REG;
    reg_val = *luc_sw_rev;
    high = reg_val >> 24;
    low = reg_val & 0x00ffffff;
    reg_val = *luc_hw_rev;
    printf("LUC revision: %d.%d.%d\n", high, low, reg_val);
}

void luc_getVerString(char *str)
{
    uint32 *luc_sw_rev;
    uint32 *luc_hw_rev;
    uint32 reg_val, high, low;

	if (!isTXRX()) {
		str = NULL;
		return;
	}
    luc_sw_rev = (uint32 *)LUC_SW_REV_REG;
    luc_hw_rev = (uint32 *)LUC_HW_REV_REG;
    reg_val = *luc_sw_rev;
    high = reg_val >> 24;
    low = reg_val & 0x00ffffff;
    reg_val = *luc_hw_rev;
    sprintf(str, "LUC revision: %d.%d.%d", high, low, reg_val);
}

void
luc_show_stats()
{
    uint32 *luc_reg0;
    uint32 *luc_reg1;
    uint32 *luc_reg2;
    uint32 *luc_reg3;
    uint64 count, count1;
	uint32 i;

	if (!isCheetah()) {
		printf("luc_show_stats: Only available on Cheetah!\n");
		return;
	}
	if (!isTXRX()) {
		printf("luc_show_stats: Only available on TXRX!\n");
		return;
	}

    printf("LUC statistics\n");
    printf("==============\n");

    luc_reg0 = (uint32 *)LUC_FROM_SIBYTE0_REG;
    printf("Pkts from 1st SIBYTE: %d\n", *luc_reg0);

    luc_reg0 = (uint32 *)LUC_FROM_SIBYTES0_LO_REG;
    luc_reg1 = (uint32 *)LUC_FROM_SIBYTES0_HI_REG;
    count = *luc_reg1;
    count = count << 32;
    count |= *luc_reg0;
    printf("Bytes from 1st SIBYTE: %lld\n", count);

    luc_reg0 = (uint32 *)LUC_TO_SIBYTE0_REG;
    printf("Pkts to 1st SIBYTE: %d\n", *luc_reg0);

    luc_reg0 = (uint32 *)LUC_TO_SIBYTES0_LO_REG;
    luc_reg1 = (uint32 *)LUC_TO_SIBYTES0_HI_REG;
    count = *luc_reg1;
    count = count << 32;
    count |= *luc_reg0;
    printf("Bytes to 1st SIBYTE: %lld\n", count);
    printf("\n");

    luc_reg0 = (uint32 *)LUC_FROM_SIBYTE1_REG;
    printf("Pkts from 2nd SIBYTE: %d\n", *luc_reg0);

    luc_reg0 = (uint32 *)LUC_FROM_SIBYTES1_LO_REG;
    luc_reg1 = (uint32 *)LUC_FROM_SIBYTES1_HI_REG;
    count = *luc_reg1;
    count = count << 32;
    count |= *luc_reg0;
    printf("Bytes from 2nd SIBYTE: %lld\n", count);
    luc_reg0 = (uint32 *)LUC_TO_SIBYTE1_REG;
    printf("Pkts to 2nd SIBYTE: %d\n", *luc_reg0);

    luc_reg0 = (uint32 *)LUC_TO_SIBYTES1_LO_REG;
    luc_reg1 = (uint32 *)LUC_TO_SIBYTES1_HI_REG;
    count = *luc_reg1;
    count = count << 32;
    count |= *luc_reg0;
    printf("Bytes to 2nd SIBYTE: %lld\n", count);
    printf("\n");

    luc_reg0 = (uint32 *)LUC_INGRESS0_REG;
    printf("Pkts from LSI 0: %d\n", *luc_reg0);

    luc_reg0 = (uint32 *)LUC_RXBYTES0_LO_REG;
    luc_reg1 = (uint32 *)LUC_RXBYTES0_HI_REG;
    count = *luc_reg1;
    count = count << 32;
    count |= *luc_reg0;
    printf("Bytes from LSI 0: %lld\n", count);

    luc_reg0 = (uint32 *)LUC_EGRESS0_REG;
    printf("Pkts to LSI 0: %d\n", *luc_reg0);

    luc_reg0 = (uint32 *)LUC_TXBYTES0_LO_REG;
    luc_reg1 = (uint32 *)LUC_TXBYTES0_HI_REG;
    count = *luc_reg1;
    count = count << 32;
    count |= *luc_reg0;
    printf("Bytes to LSI 0: %lld\n", count);
    printf("\n");

    luc_reg1 = (uint32 *)LUC_INGRESS1_REG;
    printf("Pkts from LSI 1: %d\n", *luc_reg1);

    luc_reg0 = (uint32 *)LUC_RXBYTES1_LO_REG;
    luc_reg1 = (uint32 *)LUC_RXBYTES1_HI_REG;
    count = *luc_reg1;
    count = count << 32;
    count |= *luc_reg0;
    printf("Bytes from LSI 1: %lld\n", count);

    luc_reg1 = (uint32 *)LUC_EGRESS1_REG;
    printf("Pkts to LSI 1: %d\n", *luc_reg1);

    luc_reg0 = (uint32 *)LUC_TXBYTES1_LO_REG;
    luc_reg1 = (uint32 *)LUC_TXBYTES1_HI_REG;
    count = *luc_reg1;
    count = count << 32;
    count |= *luc_reg0;
    printf("Bytes to LSI 1: %lld\n", count);
    printf("\n");

    luc_reg2 = (uint32 *)LUC_INGRESS2_REG;
    printf("Pkts from LSI 2: %d\n", *luc_reg2);

    luc_reg0 = (uint32 *)LUC_RXBYTES2_LO_REG;
    luc_reg1 = (uint32 *)LUC_RXBYTES2_HI_REG;
    count = *luc_reg1;
    count = count << 32;
    count |= *luc_reg0;
    printf("Bytes from LSI 2: %lld\n", count);

    luc_reg1 = (uint32 *)LUC_EGRESS2_REG;
    printf("Pkts to LSI 2: %d\n", *luc_reg1);

    luc_reg0 = (uint32 *)LUC_TXBYTES2_LO_REG;
    luc_reg1 = (uint32 *)LUC_TXBYTES2_HI_REG;
    count = *luc_reg1;
    count = count << 32;
    count |= *luc_reg0;
    printf("Bytes to LSI 2: %lld\n", count);
    printf("\n");

    luc_reg3 = (uint32 *)LUC_INGRESS3_REG;
    printf("Pkts from LSI 3: %d\n", *luc_reg3);

    luc_reg0 = (uint32 *)LUC_RXBYTES3_LO_REG;
    luc_reg1 = (uint32 *)LUC_RXBYTES3_HI_REG;
    count = *luc_reg1;
    count = count << 32;
    count |= *luc_reg0;
    printf("Bytes from LSI 3: %lld\n", count);

    luc_reg3 = (uint32 *)LUC_EGRESS3_REG;
    printf("Pkts to LSI 3: %d\n", *luc_reg3);

    luc_reg0 = (uint32 *)LUC_TXBYTES3_LO_REG;
    luc_reg1 = (uint32 *)LUC_TXBYTES3_HI_REG;
    count = *luc_reg1;
    count = count << 32;
    count |= *luc_reg0;
    printf("Bytes to LSI 3: %lld\n", count);
	printf("\n");
#ifdef LUC_DEBUG
    for (i = 0; i < 4; i++) {
    	printf("LUC Tx Pkts[%d]: %u  LSI: %u\n", i, 
				luc_txPkts[i], lsi_get_txPktCnt(i));
        printf("LUC Tx Bytes[%d]: %u  LSI: %u\n", i, 
				luc_txBytes[i], lsi_get_txByteCnt(i));
		count = luc_rxPkts[i];
		count1 = lsi_get_rxPktCnt(i);
        printf("LUC Rx Pkts[%d]: %lld  LSI: %lld\n", i, count, count1);
        printf("LUC Rx Bytes[%d]: %u  LSI: %u\n", i, 
				luc_rxBytes[i], lsi_get_rxByteCnt(i));
	}
	printf("\n");
#endif
}

void
luc_clear_stats()
{
    uint32 *luc_reg;
	int i;

	if (!isCheetah()) {
		printf("luc_clear_stats: Only available on Cheetah!\n");
		return;
	}
	if (!isTXRX()) {
		printf("luc_clear_stats: Only available on TXRX!\n");
		return;
	}
    luc_reg = (uint32 *)LUC_CNTCLR0_REG;
    *luc_reg = 0xffffffff;
    luc_reg = (uint32 *)LUC_CNTCLR1_REG;
    *luc_reg = 0xffffffff;

#ifdef LUC_DEBUG
    for (i = 0; i < 4; i++) {
		luc_txPkts[i] = 0;
		luc_txBytes[i] = 0;
		luc_rxPkts[i] = 0;
		luc_rxBytes[i] = 0;
		diag_lsi_clearStats(i);
	}
#endif
}

void
luc_show_regs()
{
    uint32 *luc_reg0;

	if (!isCheetah()) {
		printf("luc_show_regs: Only available on Cheetah!\n");
		return;
	}
	if (!isTXRX()) {
		printf("luc_show_regs: Only available on TXRX!\n");
		return;
	}

    printf("LUC registers\n");
    printf("=============\n");

    luc_reg0 = (uint32 *)LUC_TX0_ERRORS;
    printf("TX0 sequence errors: %d, length errors: %d\n", 
			*luc_reg0 >> 16, *luc_reg0 & 0xffff);
    luc_reg0 = (uint32 *)LUC_TX1_ERRORS;
    printf("TX1 sequence errors: %d, length errors: %d\n", 
			*luc_reg0 >> 16, *luc_reg0 & 0xffff);
    luc_reg0 = (uint32 *)LUC_TX2_ERRORS;
    printf("TX2 sequence errors: %d, length errors: %d\n", 
			*luc_reg0 >> 16, *luc_reg0 & 0xffff);
    luc_reg0 = (uint32 *)LUC_TX3_ERRORS;
    printf("TX3 sequence errors: %d, length errors: %d\n", 
			*luc_reg0 >> 16, *luc_reg0 & 0xffff);
    luc_reg0 = (uint32 *)LUC_TO_SIBYTE_EOP;
	printf("EOP errors to 1st sibyte: %d, 2nd sibyte: %d\n", 
			*luc_reg0 & 0xffff, *luc_reg0 >> 16);
    luc_reg0 = (uint32 *)LUC_FROM_SIBYTE_EOP;
	printf("EOP errors from 1st sibyte: %d, 2nd sibyte: %d\n", 
			*luc_reg0 & 0xffff, *luc_reg0 >> 16);
    luc_reg0 = (uint32 *)LUC_ECC_ERRORS;
    printf("ECC correctable errors: %d, uncorrectable: %d\n", 
			*luc_reg0 & 0xffff, *luc_reg0 >> 16);

    luc_reg0 = (uint32 *)LUC_TX0_TCP_LEN;
    printf("TX0 TCP/UDP len: %d\n", *luc_reg0);
    luc_reg0 = (uint32 *)LUC_TX1_TCP_LEN;
    printf("TX1 TCP/UDP len: %d\n", *luc_reg0);
    luc_reg0 = (uint32 *)LUC_TX2_TCP_LEN;
    printf("TX2 TCP/UDP len: %d\n", *luc_reg0);
    luc_reg0 = (uint32 *)LUC_TX3_TCP_LEN;
    printf("TX3 TCP/UDP len: %d\n", *luc_reg0);

    luc_reg0 = (uint32 *)LUC_TX0_IPF_DIAG;
    printf("TX0 IPF diag (hex): %08x\n", *luc_reg0);
    luc_reg0 = (uint32 *)LUC_TX1_IPF_DIAG;
    printf("TX1 IPF diag (hex): %08x\n", *luc_reg0);
    luc_reg0 = (uint32 *)LUC_TX2_IPF_DIAG;
    printf("TX2 IPF diag (hex): %08x\n", *luc_reg0);
    luc_reg0 = (uint32 *)LUC_TX3_IPF_DIAG;
    printf("TX3 IPF diag (hex): %08x\n", *luc_reg0);
	printf("\n");
}

void
lmux_show_stats()
{
	int i; 
#ifdef NFX_SMP
	int tmp, tmp1, tmp2;
#endif
	if (!isFP()) {
		printf("LMUX stats only available on FP!\n");
		return;
	}
    for (i = 0; i < 6; i++) {
#ifdef NFX_SMP
    	tmp = bcEthCfg[0]->txc[0].lmux_txPkts[i];
    	tmp += bcEthCfg[0]->txc[1].lmux_txPkts[i];
    	tmp += bcEthCfg[2]->txc[0].lmux_txPkts[i];
    	tmp += bcEthCfg[2]->txc[1].lmux_txPkts[i];
		printf("LMUX Tx Pkts[%d]: %d  LSI: %d\n", i,tmp, lsi_get_txPktCnt(i));

    	tmp = bcEthCfg[0]->txc[0].lmux_txBytes[i];
    	tmp += bcEthCfg[0]->txc[1].lmux_txBytes[i];
    	tmp += bcEthCfg[2]->txc[0].lmux_txBytes[i];
    	tmp += bcEthCfg[2]->txc[1].lmux_txBytes[i];
		printf("LMUX Tx Bytes[%d]: %d  LSI: %d\n", i,tmp, lsi_get_txByteCnt(i));
#else
		printf("LMUX Tx Pkts[%d]: %d  LSI: %d\n", i, 
				lmux_txPkts[i], lsi_get_txPktCnt(i));
		printf("LMUX Tx Bytes[%d]: %d  LSI: %d\n", i, 
				lmux_txBytes[i], lsi_get_txByteCnt(i));
#endif
#ifdef NFX_SMP 
		tmp1 = lsi_get_rxPktCnt(i);
    	tmp2 = bcEthCfg[0]->rxc[0].lmux_rxPkts[i];
    	tmp2 += bcEthCfg[0]->rxc[1].lmux_rxPkts[i];
    	tmp2 += bcEthCfg[2]->rxc[0].lmux_rxPkts[i];
    	tmp2 += bcEthCfg[2]->rxc[1].lmux_rxPkts[i];
        printf("LMUX Rx Pkts[%d]: %d  LSI: %d\n", i, tmp2, tmp1);

    	tmp = bcEthCfg[0]->rxc[0].lmux_rxBytes[i];
    	tmp += bcEthCfg[0]->rxc[1].lmux_rxBytes[i];
    	tmp += bcEthCfg[2]->rxc[0].lmux_rxBytes[i];
    	tmp += bcEthCfg[2]->rxc[1].lmux_rxBytes[i];
        printf("LMUX Rx Bytes[%d]: %d  LSI: %d\n", i,tmp, lsi_get_rxByteCnt(i));
#else
        printf("LMUX Rx Pkts[%d]: %d  LSI: %d\n", i, 
				lmux_rxPkts[i], lsi_get_rxPktCnt(i));
        printf("LMUX Rx Bytes[%d]: %d  LSI: %d\n", i, 
				lmux_rxBytes[i], lsi_get_rxByteCnt(i));
#endif
    }
}
void
lmux_clear_stats()
{
	int i;

	if (!isFP()) {
		printf("LMUX stats only available on FP!\n");
		return;
	}
    for (i = 0; i < 6; i++) {
#ifdef NFX_SMP 
		bcEthCfg[0]->txc[0].lmux_txPkts[i] = 0;
		bcEthCfg[0]->txc[1].lmux_txPkts[i] = 0;
		bcEthCfg[2]->txc[0].lmux_txPkts[i] = 0;
		bcEthCfg[2]->txc[1].lmux_txPkts[i] = 0;
		bcEthCfg[0]->txc[0].lmux_txBytes[i] = 0;
		bcEthCfg[0]->txc[1].lmux_txBytes[i] = 0;
		bcEthCfg[2]->txc[0].lmux_txBytes[i] = 0;
		bcEthCfg[2]->txc[1].lmux_txBytes[i] = 0;

		bcEthCfg[0]->rxc[0].lmux_rxPkts[i] = 0;
		bcEthCfg[0]->rxc[1].lmux_rxPkts[i] = 0;
		bcEthCfg[2]->rxc[0].lmux_rxPkts[i] = 0;
		bcEthCfg[2]->rxc[1].lmux_rxPkts[i] = 0;
		bcEthCfg[0]->rxc[0].lmux_rxBytes[i] = 0;
		bcEthCfg[0]->rxc[1].lmux_rxBytes[i] = 0;
		bcEthCfg[2]->rxc[0].lmux_rxBytes[i] = 0;
		bcEthCfg[2]->rxc[1].lmux_rxBytes[i] = 0;
#else
		lmux_txPkts[i] = 0;
		lmux_txBytes[i] = 0;
		lmux_rxPkts[i] = 0;
		lmux_rxBytes[i] = 0;
#endif
		diag_lsi_clearStats(i);
	}
}

void
lmux_show_rev()
{
    extern uint32 lmux_rev[3];
	extern uint32 lmux_get_rev;

	if (!isCheetah()) {
		printf("luc_show_rev: Only available on Cheetah!\n");
		return;
	}
	if (!isFP()) {
		printf("lmux_show_rev: Only available on FP!\n");
		return;
	}
    printf("LMUX revision: %d.%d.%d\n", lmux_rev[0], lmux_rev[1], lmux_rev[2]);
	lmux_get_rev = 1;
}

void lmux_getVerString(char *str)
{
    extern uint32 lmux_rev[3];
	extern uint32 lmux_get_rev;

	if (!isFP()) {
		str = NULL;
		return;
	}
    sprintf(str, "LMUX revision: %d.%d.%d", 
			lmux_rev[0], lmux_rev[1], lmux_rev[2]);
	lmux_get_rev = 1;
}

//-----------------------------------------------------------------------------
// Function Name:   bmc12500Eth_showRxCapBuf
//
// Description:     Displays receive packet capture buffer.
//
// Created by:      Frank Salzmann
//
// Data Created:    July 17, 2002
//
// Returns: NFX_OK
//-----------------------------------------------------------------------------
int
bmc12500Eth_showRxCapBuf(bc1250EthCfg_t *cfg)
{ 
#ifdef BC1250_RX_PKT_CAPTURE
    int i, j, quit;
	char *timestr;
	bc1250_pcbe_t *pcbe;
 
	printf("Rx Capture Buffer:\n");
	printf("==================\n");
	printf("Status       : %s\n", 
		   (bc1250_pcb[cfg->macId].rxCapEnabled == 0) ? "DISABLED" : "ENABLED");
	printf("Current Entry: %d\n",bc1250_pcb[cfg->macId].curEntry_rx); 
	printf("\n");
	bmc12500Eth_uiGetInput("Press Enter To Continue");

	j = bc1250_pcb[cfg->macId].curEntry_rx - 1;
    for(i=0; i < BC1250_CAPTURE_ENTRIES_MAX; i++ ) {

		pcbe = &(bc1250_pcb[cfg->macId].pcbe_rx[j]);

		timestr = eee_humandate((uint32)eee_rtcToSec(&pcbe->t));
        printf("\n=== Time: %s === PktSize: %d === (Entry: %d)\n", timestr, pcbe->pktLen, j);
		printf("=== Rx Buf: %lx\n", (address_t)pcbe->bufPtr);
        bmc12500Eth_printBuf( pcbe->bytes, pcbe->capLen, 1);

		if( --j <= 0 )
			j = BC1250_CAPTURE_ENTRIES_MAX-1;

        if( !(i % 5) && i != 0 ) {
            quit = bmc12500Eth_uiGetInput("Quit [99]: ");
            if( quit == 99 )
                return 0;
        }
    }
#else
	printf("Rx capture buf not compiled into this build.\n");
#endif
	return 0;
} // bmc12500Eth_showRxCapBuf

//-----------------------------------------------------------------------------
// Function Name:   bmc12500Eth_uiSetRxCapBuf
//
// Description:     Enables/Disables packet capturing in bmc12500Eth driver.
//
// Created by:      Frank Salzmann
//
// Data Created:    July 17, 2002
//
// Returns: NFX_OK
//-----------------------------------------------------------------------------
int
bmc12500Eth_uiSetRxCapBuf(bc1250EthCfg_t *cfg) 
{
	uint32        sel;

	printf("Rx Pkt Capture Buffer: %s\n",
					(bc1250_pcb[cfg->macId].rxCapEnabled == 0) ? "DISABLED" : "ENABLED");
    sel = bmc12500Eth_uiGetInput("Rx Trace Dis/Ena/Clear [0|1|2]: ");
	switch(sel) {
		case 0:
			bc1250_pcb[cfg->macId].rxCapEnabled = 0;
			break;
		case 1:
			bc1250_pcb[cfg->macId].rxCapEnabled = 1;
			break;
		case 2:
			bc1250_pcb[cfg->macId].curEntry_rx = 0;
			memset(bc1250_pcb[cfg->macId].pcbe_rx, 0, sizeof(bc1250_pcb[cfg->macId].pcbe_rx));
			break;
		default: 
			printf("Invalid Entry: %d\n", sel);
	}
	printf("Rx Pkt Capture Buffer: %s\n",
					(bc1250_pcb[cfg->macId].rxCapEnabled == 0) ? "DISABLED" : "ENABLED");
    printf("\n");

	return NFX_OK;
} // bmc12500Eth_uiRxSetCapBuf

//-----------------------------------------------------------------------------
// Function Name:   bmc12500Eth_showTxCapBuf
//
// Description:     Displays transmit packet capture buffer.
//
// Created by:      Frank Salzmann
//
// Data Created:    July 17, 2002
//
// Returns: NFX_OK
//-----------------------------------------------------------------------------
int
bmc12500Eth_showTxCapBuf(bc1250EthCfg_t *cfg)
{ 
#ifdef BC1250_TX_PKT_CAPTURE
    int i, j, quit;
	char *timestr;
	bc1250_pcbe_t *pcbe;

	printf("Tx Capture Buffer:\n");
	printf("==================\n");
	printf("Status       : %s\n", 
		   (bc1250_pcb[cfg->macId].txCapEnabled == 0) ? "DISABLED" : "ENABLED");
	printf("Current Entry: %d\n",bc1250_pcb[cfg->macId].curEntry_tx); 
	printf("\n");
	bmc12500Eth_uiGetInput("Press Enter To Continue");

	j = bc1250_pcb[cfg->macId].curEntry_tx - 1;
    for(i=0; i < BC1250_CAPTURE_ENTRIES_MAX; i++ ) {

		pcbe = &(bc1250_pcb[cfg->macId].pcbe_tx[j]);

		timestr = eee_humandate((uint32)eee_rtcToSec(&pcbe->t));
        printf("\n=== Time: %s === PktSize: %d === (Entry: %d)\n", timestr, pcbe->pktLen, j);
		printf("=== Rx Buf: %lx\n", (address_t)pcbe->bufPtr);
        bmc12500Eth_printBuf( pcbe->bytes, pcbe->capLen, 1);

		if( --j <= 0 )
			j = BC1250_CAPTURE_ENTRIES_MAX-1;

        if( !(i % 5) && i != 0 ) {
            quit = bmc12500Eth_uiGetInput("Quit [99]: ");
            if( quit == 99 )
                return 0;
        }
    }
#else
	printf("Rx capture buf not compiled into this build.\n");
#endif
	return 0;
} // bmc12500Eth_showTxCapBuf

//-----------------------------------------------------------------------------
// Function Name:   bmc12500Eth_uiSetTxCapBuf
//
// Description:     Enables/Disables packet capturing in bmc12500Eth driver.
//
// Created by:      Frank Salzmann
//
// Data Created:    July 17, 2002
//
// Returns: NFX_OK
//-----------------------------------------------------------------------------
int
bmc12500Eth_uiSetTxCapBuf(bc1250EthCfg_t *cfg) 
{
	uint32        sel;

	printf("Tx Pkt Capture Buffer: %s\n",
					(bc1250_pcb[cfg->macId].txCapEnabled == 0) ? "DISABLED" : "ENABLED");
    sel = bmc12500Eth_uiGetInput("Tx Trace Dis/Ena/Clear [0|1|2]: ");
	switch(sel) {
		case 0:
			bc1250_pcb[cfg->macId].txCapEnabled = 0;
			break;
		case 1:
			bc1250_pcb[cfg->macId].txCapEnabled = 1;
			break;
		case 2:
			bc1250_pcb[cfg->macId].curEntry_tx = 0;
			memset(bc1250_pcb[cfg->macId].pcbe_tx, 0, sizeof(bc1250_pcb[cfg->macId].pcbe_tx));
			break;
		default: 
			printf("Invalid Entry: %d\n", sel);
	}
	printf("Tx Pkt Capture Buffer: %s\n",
					(bc1250_pcb[cfg->macId].txCapEnabled == 0) ? "DISABLED" : "ENABLED");
    printf("\n");

	return NFX_OK;
} // bmc12500Eth_uiTxSetCapBuf
