/*
 *-----------------------------------------------------------------
 *
 * Name: crash.c
 *
 * RCS:  $Id: crash.c,v 1.17 2002/10/24 21:12:38 isabellec Exp $
 *
 * Copyright (C) 2002, Agile Storage, Inc.
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	06/07/2002
 *
 *-----------------------------------------------------------------
 */



#include <nfx-incl.h>
#include <sys/mips.h>
#include <sys/amd29f040.h>
#include <exports/crashdump-api.h>
#include <exports/timer-api.h>
#include <exports/eee-api.h>
#include "../sm-eee/eee.h"
#include <exports/malloc-api.h>
#include "crash.h"
#include <exports/evm-api.h>
#include <exports/scsi-api.h>
#include <exports/scsi-msg.h>
#include <exports/opt-api.h>
#include <exports/sdm-api.h>

#include <exports/esm-api.h>
#include <exports/thread-api.h>

#include <sys/exec_elf.h>

#include <exports/file-api.h>
#include <exports/elog-api.h>

#include <exports/pte-api.h>

extern uint64 core_size(void);

void savecore(void);

#undef DEBUG
#if defined(DEBUG)
#define debug(x) printf x
#else
#define debug(x)
#endif

/* timeout after 30 seconds */
#define CRASH_TIMEOUT 30

extern void eee_descDisplay(eee_desc_t *desc);


extern uint32 bmc12500Eth_tx_pkt_trace;
extern uint32 bmc12500Eth_rx_pkt_trace;

extern int crash_bmc12500Eth_initModule(void);
extern void crash_bmc12500Eth_rx(void);
extern int crash_bmc12500Eth_txUnicast(uint32 dest, eee_desc_t *desc);
extern void crash_bmc12500Eth_txClean(void);


extern uint32 nfxMySlotId;
extern uint32 nfxMyCPUId;

extern int sdma_handleToPath(uint32 handle, scsi_path_t *path, uint32 *scsi_port);

static int outstanding_ios;
static int write_error;
static int nsend;

extern int coreauto;

struct core_file
{
	int32 th;
	int app;
	evm_volId_t vol_id;
	scsi_id_t scsi_id;
	int32 strg_handle;
	uint32 startBlk;
	int64 block_size;
	uint64 pos;
	scsi_path_t path;
	uint32 scsi_dest_port;
	uint32 src_port;
	int opened;
	int seqid;
	int got_storage;
	int opened_storage;
	char savedir[256 + 1];
	char volname[VOL_MAX_NAME];
	int (*open)(struct core_file *, char *filename);
	int (*write)(struct core_file *, uchar8 *, uint64);
	int (*seek)(struct core_file *, uint64);
	int (*close)(struct core_file *);
};


static struct core_file core;


static void savecore_thread(esm_handle_t handle, void *userInfo);

static uint64
core_blocks(void)
{
	return core_size() / 512;
}


/*-----------------------------------------------------------------
 * Name :	crash_fp_open()
 *
 * Description: Open core file on FP.
 *
 * Created by:	Maxim Kozlovsky
 *
 * Date Created:	06/10/02
 *
 *-----------------------------------------------------------------
 */
static int
crash_fp_open(struct core_file *file, char *filename)
{
	if (!file->opened) {
		if (core.opened_storage && crash_bmc12500Eth_initModule() == NFX_OK) {
			bmc12500Eth_tx_pkt_trace = 0;
			bmc12500Eth_rx_pkt_trace = 0;
			if (sdma_handleToPath(core.strg_handle, &core.path,
								  &core.scsi_dest_port) == NFX_OK) {
				file->opened = 1;
			}
		}
	}
	return file->opened ? NFX_OK : NFX_ERR;
}


static void
crash_fp_drain_ios(int count)
{
    eeeRTC_t time_start;
    eeeGMT_t duration;

    eee_rtcStart(&time_start);
	while (outstanding_ios > count && !write_error) {
		crash_bmc12500Eth_rx();
        eee_rtcDurationTV(&time_start, &duration);
		if (duration.sec > CRASH_TIMEOUT) {
			printf("Core write timed out\n");
			write_error = 1;
		}
	}
}


/*-----------------------------------------------------------------
 * Name :	crash_fp_close()
 *
 * Description: Close core file. Wait for all outstanding I/Os to complete
 *
 * Created by:	Maxim Kozlovsky
 *
 * Date Created:	06/10/02
 *
 *-----------------------------------------------------------------
 */
static int
crash_fp_close(struct core_file *file)
{
	file->opened = 0;
	crash_fp_drain_ios(0);
	return write_error ? NFX_ERR : NFX_OK;
}


/*-----------------------------------------------------------------
 * Name :	crash_fp_seek()
 *
 * Description: Seek to the specified position
 *
 * Created by:	Maxim Kozlovsky
 *
 * Date Created:	06/10/02
 *
 *-----------------------------------------------------------------
 */
static int
crash_fp_seek(struct core_file *file, uint64 pos)
{
	debug(("crash_fp_seek:%llx\n", pos));
	NFX_ASSERT((pos & (1024 - 1)) == 0);
	file->pos = pos;
	return NFX_OK;
}


/*-----------------------------------------------------------------
 * Name :	crash_fp_write()
 *
 * Description: write length bytes from buffer data
 *
 * Created by:	Maxim Kozlovsky
 *
 * Date Created:	06/10/02
 *
 *-----------------------------------------------------------------
 */
static int
crash_fp_write(struct core_file *file, uchar8 *data, uint64 length)
{
	scsi_writeReqMsg_t *msg;
	uint64 remain = length;
	uchar8 *data_ptr = data;
	int error = 0;
	uint32 writelen;
	uint32 msglen;
		
	debug(("crash_fp_write:%llx %p %llx\n", length, data, core.pos));
	
	NFX_ASSERT((length & (1024 - 1)) == 0);
	
  	while (!error && !write_error && remain > 0
 		   && core.pos < (core.block_size * 512 - 8 * 1024)) {
		eee_descPtr_t desc = eee_crash_allocateDesc(0, 0);
		uchar8 *buf;
		if (desc == NULL) {
			debug(("crash_fp_write: can not allocate descriptor\n"));
			error = 1;
		}
		if (!error) {
			buf = eee_crash_allocateBuffer(0, 0, 0);
			if (buf == NULL) {
				debug(("crash_fp_write: can not allocate buffer\n"));
				error = 1;
			}
			buf += 128;
		}
		if (!error) {
			/* Writes can not span multiple 8k regions */
			uint64 align = core.pos & (8 * 1024 - 1);
			writelen = MIN(remain, 8 * 1024 - align);
			msglen = writelen + sizeof(scsi_writeReqMsg_t);
			desc->hdr.src_port = core.src_port;
			desc->hdr.dest_port = core.scsi_dest_port;
			desc->hdr.control = 0;
			EEE_SET_PKT_LEN(desc, msglen);
			EEE_SET_NUM_BUFS(desc, 1);
			desc->bd[1].len = msglen;
			desc->bd[1].control = EEE_BUF_ULTRA | EDESC_BD_CMD_IN_USE;
			desc->bd[1].buf = buf - sizeof(scsi_writeReqMsg_t);
			msg = (scsi_writeReqMsg_t *)(buf - sizeof(scsi_writeReqMsg_t));
			msg->hdr.mid = SCSI_MID_WRITE_REQ;
			msg->hdr.cHandle = core.seqid++;
			msg->hdr.path = core.path;
			msg->transSeqId = 0;
			msg->blkOffset = core.pos / 512 + core.startBlk;
			msg->blkCnt = writelen / 512;
			msg->checkSum = 0;
			memcpy(buf, data_ptr, writelen);
			if (nsend++ == 40) {
				crash_bmc12500Eth_txClean();
				crash_bmc12500Eth_rx();
				nsend = 0;
			}
			if (outstanding_ios > 70) {
				crash_fp_drain_ios(48);
			}
			error = crash_bmc12500Eth_txUnicast(core.scsi_dest_port, desc);
            if (error) {
                debug(("crash_bmc12500Eth_txUnicast failure\n"));
            }
			++outstanding_ios;
			data_ptr += writelen;
			core.pos += writelen;
			remain -= writelen;
		}
	}

	return error || write_error ? NFX_ERR : NFX_OK;
}


/*-----------------------------------------------------------------
 * Name :	core_get_strg()
 *
 * Description: Get storage handle from EVM
 *
 * Created by:	Maxim Kozlovsky
 *
 * Date Created:	06/10/02
 *
 *-----------------------------------------------------------------
 */
static int
core_get_strg(void)
{
	int got_response = 0;
	int rc = NFX_ERR;
	while (!got_response) {
		esm_event_t event;
		evm_MsgGetStrg_t msg;
		uint32 dp;
		msg.hdr.msgType = EVM_GET_LV_STRG;
		msg.hdr.len = sizeof(msg);
		msg.hdr.clientPrivate = esm_getCurrHandle();
		msg.id = core.vol_id;
		dp = PACK_UNICAST_PORT_ADDRESS(0, 0, eee_getApplicationId("evm"),
									   EEE_MGMT, NFX_COS_HIGH, 0);
		if (eee_sendMessage(&msg, sizeof(msg), dp, core.src_port) != NFX_OK) {
			debug(("can not send EVM_GET_LV_STRG msg\n"));
			return NFX_ERR;
		}

		thread_yield(&event, 200);
		if (event.event == EVM_LV_STRG_CFG) {
			eee_descPtr_t desc = event.p1.u.p;
			evm_MsgLVStrg_t *rsp = (evm_MsgLVStrg_t *)edesc_firstBuf(desc)->buf;
			got_response = 1;
            if (rsp->nbStrg == 1) {
                evm_strgConfig_t *cfg = &(rsp->strg[0].au);
				if (cfg->auSize < core_blocks()) {
					E_LOG(class_1, LOG_WARNING, core.app,
						  0, 0, 0, ("core volume is too small: "
									"blocks=%lld size =%lld",
									core_blocks(), cfg->auSize));
				}
				core.scsi_id = cfg->strgId;
				core.strg_handle = cfg->openHandle;
				core.startBlk = cfg->startBlk;
				core.block_size = cfg->auSize;
				core.got_storage = TRUE;
				debug(("got vol params: scsi=%llx handle=%x start=%d size=%llx\n",
					   core.scsi_id, core.strg_handle, core.startBlk, core.block_size));
				rc = NFX_OK;
			}
			eee_freePacket(desc);
		}
	}
	return rc;
}


/*-----------------------------------------------------------------
 * Name :	core_open_strg()
 *
 * Description: Open the core device with SDM
 *
 * Created by:	Maxim Kozlovsky
 *
 * Date Created:	10/01/02
 *
 *-----------------------------------------------------------------
 */
static int
core_open_strg(void)
{
	int got_response = 0;
	int rc = NFX_ERR;
	while (!got_response) {
		esm_event_t event;
		sdm_openReqMsg_t msg;
		uint32 dp;
		msg.hdr.mid = SDM_MID_DEV_OPEN_REQ;
		msg.hdr.chandle = esm_getCurrHandle();
		msg.id = core.scsi_id;
		msg.slot = 1;
		msg.flags = 0;
		dp = PACK_UNICAST_PORT_ADDRESS(0, 0, eee_getApplicationId("sdm"),
									   EEE_MGMT, NFX_COS_HIGH, 0);
		if (eee_sendMessage(&msg, sizeof(msg), dp, core.src_port) != NFX_OK) {
			debug(("can not send EVM_GET_LV_STRG msg\n"));
			return NFX_ERR;
		}

		thread_yield(&event, 200);
		if (event.event == SDM_MID_DEV_OPEN_RSP) {
			eee_descPtr_t desc = event.p1.u.p;
			sdm_openRspMsg_t *rsp =
			(sdm_openRspMsg_t *)edesc_firstBuf(desc)->buf;
            if (rsp->rCode == SDM_OK) {
				got_response = 1;
				core.strg_handle = rsp->handle;
				core.opened_storage = TRUE;
				debug(("opened storage: handle = %x\n", core.strg_handle));	
				if (sdma_handleToPath(core.strg_handle, &core.path,
								  &core.scsi_dest_port) == NFX_OK) {
					/* printf("path = %x\n", core.path); */
				}
				rc = NFX_OK;
			}
			else if (rsp->rCode == SDM_TO_MANY_OPENS) {
				/* Retry after EVM is done with the volume */
				debug(("core_open_strg: retry open\n"));
				thread_yield(NULL, 100);
			}
			else {
				got_response = 1;
				E_LOG(class_1, LOG_WARNING, core.app,
					  0, 0, 0, ("can not open core volume: %d", rsp->rCode));
			}
			eee_freePacket(desc);
		}
	}
	return rc;
}


/*-----------------------------------------------------------------
 * Name :	core_getvolid()
 *
 * Description: Get crash volume id from EVM
 *
 * Created by:	Maxim Kozlovsky
 *
 * Date Created:	06/10/02
 *
 *-----------------------------------------------------------------
 */
static int
core_getvolid(void)
{
	int got_response = 0;
	int rc = NFX_OK;
	while (!got_response) {
		esm_event_t event;
		evm_MsgGetId_t msg;
		uint32 dp;
		msg.hdr.msgType = EVM_VOL_GET_ID;
		msg.hdr.len = sizeof(msg);
		msg.hdr.clientPrivate = esm_getCurrHandle();
		memset(msg.name, 0, sizeof(msg.name));
		strcpy(msg.name, core.volname);
		dp = PACK_UNICAST_PORT_ADDRESS(0, 0, eee_getApplicationId("evm"),
									   EEE_MGMT, NFX_COS_HIGH, 0);
		if (eee_sendMessage(&msg, sizeof(msg), dp, core.src_port) != NFX_OK) {
			debug(("can not send EVM_VOL_GET_ID msg\n"));
			return NFX_ERR;
		}

		thread_yield(&event, 200);
		if (event.event == EVM_MSG_STAT) {
			eee_descPtr_t desc = event.p1.u.p;
			evm_MsgStat_t *rsp = (evm_MsgStat_t *)edesc_firstBuf(desc)->buf;
			if (rsp->stat == NFX_OK) {
				core.vol_id = rsp->payload.lVolId;
				debug(("core.vol_id = %llx\n", rsp->payload.lVolId.whole_thing));
				got_response = 1;
			}
			else {
				thread_yield(&event, 200);
			}
			eee_freePacket(desc);
		}
	}
	return rc;
}


/*-----------------------------------------------------------------
 * Name :	core_getcrashname()
 *
 * Description: Get crash volume name from registry manager
 *
 * Created by:	Maxim Kozlovsky
 *
 * Date Created:	06/10/02
 *
 *-----------------------------------------------------------------
 */
static int
core_getcrashname()
{
	int got_response = 0;
	int rc = NFX_ERR;
	while (!got_response) {
		regmGetReq_t msg;
		esm_event_t event;
		uint32 dp;
		msg.hdr.msgType = OPT_GET_OPTION_REQ;
		msg.hdr.userInfo = esm_getCurrHandle();
		strcpy(msg.getReq.option, "option.corevolume");
		dp = PACK_UNICAST_PORT_ADDRESS(0, 0, eee_getApplicationId("regm"),
									   EEE_MGMT, NFX_COS_HIGH, 0);
		if (eee_sendMessage(&msg, sizeof(msg), dp, core.src_port) != NFX_OK) {
			debug(("can not send OPT_GET_OPTION_REQ msg\n"));
			return NFX_ERR;
		}

		thread_yield(&event, 200);
		if (event.event == OPT_GET_OPTION_RSP) {
			eee_descPtr_t desc = event.p1.u.p;
			regmGetRsp_t *rsp = (regmGetRsp_t *)edesc_firstBuf(desc)->buf;
			got_response = 1;
			if (rsp->getRsp.nbEntry == 1) {
				strcpy(core.volname, rsp->getRsp.entry[0].value);
				debug(("core_getcrashname: %s\n", core.volname));
				rc = NFX_OK;
			}
			else {
				debug(("core_getcrashname: no core volume assigned\n"));
			}
			eee_freePacket(desc);
		}
	}
	return rc;
}


/*-----------------------------------------------------------------
 * Name :	core_getcrashauto()
 *
 * Description: get auto core dump settings from registry manager
 *
 * Created by:	Maxim Kozlovsky
 *
 * Date Created:	06/10/02
 *
 *-----------------------------------------------------------------
 */
static int
core_getcrashauto()
{
	int got_response = 0;
	int rc = NFX_OK;
	while (!got_response) {
		regmGetReq_t msg;
		esm_event_t event;
		uint32 dp;
		msg.hdr.msgType = OPT_GET_OPTION_REQ;
		msg.hdr.userInfo = esm_getCurrHandle();
		strcpy(msg.getReq.option, "option.coredump");
		dp = PACK_UNICAST_PORT_ADDRESS(0, 0, eee_getApplicationId("regm"),
									   EEE_MGMT, NFX_COS_HIGH, 0);
		if (eee_sendMessage(&msg, sizeof(msg), dp, core.src_port) != NFX_OK) {
			debug(("can not send OPT_GET_OPTION_REQ msg\n"));
			return NFX_ERR;
		}

		thread_yield(&event, 200);
		if (event.event == OPT_GET_OPTION_RSP) {
			eee_descPtr_t desc = event.p1.u.p;
			regmGetRsp_t *rsp = (regmGetRsp_t *)edesc_firstBuf(desc)->buf;
			got_response = 1;
			if (rsp->getRsp.nbEntry == 1) {
				coreauto = 1;
			}
			else {
				coreauto = 0;
			}
			debug(("core_getcrashauto: %d\n", coreauto));
			eee_freePacket(desc);
		}
	}
	return rc;
}


/*-----------------------------------------------------------------
 * Name :	core_get_coredir()
 *
 * Description: Get the directory to save the core dump
 *
 * Created by:	Maxim Kozlovsky
 *
 * Date Created:	06/13/02
 *
 *-----------------------------------------------------------------
 */
static int
core_get_coredir()
{
	int got_response = 0;
	int rc = NFX_ERR;
	while (!got_response) {
		regmGetReq_t msg;
		esm_event_t event;
		uint32 dp;
		msg.hdr.msgType = OPT_GET_OPTION_REQ;
		msg.hdr.userInfo = esm_getCurrHandle();
		strcpy(msg.getReq.option, "option.coresave");
		dp = PACK_UNICAST_PORT_ADDRESS(0, 0, eee_getApplicationId("regm"),
									   EEE_MGMT, NFX_COS_HIGH, 0);
		if (eee_sendMessage(&msg, sizeof(msg), dp, core.src_port) != NFX_OK) {
			debug(("can not send OPT_GET_OPTION_REQ msg\n"));
			return NFX_ERR;
		}

		thread_yield(&event, 200);
		if (event.event == OPT_GET_OPTION_RSP) {
			eee_descPtr_t desc = event.p1.u.p;
			regmGetRsp_t *rsp = (regmGetRsp_t *)edesc_firstBuf(desc)->buf;
			got_response = 1;
			if (rsp->getRsp.nbEntry == 1) {
				strcpy(core.savedir, rsp->getRsp.entry[0].value);
				debug(("core_get_coredir: %s\n", core.savedir));
				rc = NFX_OK;
			}
			eee_freePacket(desc);
		}
	}
	return rc;
}


/*-----------------------------------------------------------------
 * Name :	core_rcv()
 *
 * Description:  coredump application receive function
 *
 * Created by:	Maxim Kozlovsky
 *
 * Date Created:	06/10/02
 *
 *-----------------------------------------------------------------
 */
static void
core_rcv(eee_descPtr_t desc)
{
	esm_event_t ev;
	uint32 thr_handle;
	uchar8 *buf = edesc_firstBuf(desc)->buf;
	ev.p1.u.p = desc;
	ev.event = *(uint32 *)edesc_firstBuf(desc)->buf;
	switch (ev.event) {
	case OPT_GET_OPTION_RSP:
		thr_handle = ((regmGetRsp_t *)buf)->hdr.userInfo;
		break;
	case EVM_LV_STRG_CFG:
	case EVM_MSG_STAT:
		thr_handle = ((evm_msgHdr_t *)buf)->clientPrivate;
		break;
	case SDM_MID_DEV_OPEN_RSP:
		thr_handle = ((sdm_openRspMsg_t *)buf)->hdr.chandle;
		break;
	case SCSI_MID_WRITE_RSP:
	case SCSI_MID_READ_RSP:
		thr_handle = ((scsi_msgHdr_t *)buf)->cHandle;
		break;
	default:
		eee_freePacket(desc);
		return;
	}
	if (thread_event(thr_handle, &ev) != NFX_OK) {
		debug(("core_rcv: thread event failed\n"));
		eee_freePacket(desc);
	}
}	


/*-----------------------------------------------------------------
 * Name :	init_core_file()
 *
 * Description: Initialize the core file, timeout, and send initial message
 * to registry manager to obtain the core volume name
 *
 * Created by:	Maxim Kozlovsky
 *
 * Date Created:	06/10/02
 *
 *-----------------------------------------------------------------
 */
static void
init_core_file(void)
{
    esm_handle_t handle;
	int coredump_app = eee_getApplicationId("coredump");
	if (coredump_app == EEE_ERR_INVALID_APP_NAME) {
		panic("Can not initialize coredump application\n");
	}

	core.app = coredump_app;
	core.src_port = PACK_UNICAST_PORT_ADDRESS(nfxMySlotId, nfxMyCPUId,
											  core.app, EEE_USER_DATA,
											  NFX_COS_HIGH, 0);
	core.opened = 0;
	core.pos = 0;
	core.open = crash_fp_open;
	core.close = crash_fp_close;
	core.write = crash_fp_write;
	core.seek = crash_fp_seek;

	if (eee_registerReceiveFunc(core.app, core_rcv, core_rcv) != NFX_OK) {
		panic("Can not register core dump app\n");
	}

	handle = thread_create(NULL);
	thread_start(&handle, savecore_thread);
}


/*-----------------------------------------------------------------
 * Name :	core_open()
 *
 * Description: open the core file
 *
 * Created by:	Maxim Kozlovsky
 *
 * Date Created:	06/10/02
 *
 *-----------------------------------------------------------------
 */
struct core_file *
core_open()
{
	int rc;
	rc = core.open(&core, NULL);
	return rc == NFX_OK ? &core : NULL;
}


/*-----------------------------------------------------------------
 * Name :	core_file_seek()
 *
 * Description: seek in the core file
 *
 * Created by:	Maxim Kozlovsky
 *
 * Date Created:	06/10/02
 *
 *-----------------------------------------------------------------
 */
int
core_file_seek(struct core_file *file, uint64 pos)
{
	file->pos = pos;
	return file->seek(file, pos);
}


/*-----------------------------------------------------------------
 * Name :	core_file_write()
 *
 * Description: write to a core file
 *
 * Created by:	Maxim Kozlovsky
 *
 * Date Created:	06/10/02
 *
 *-----------------------------------------------------------------
 */
int
core_file_write(struct core_file *file, uchar8 *data, uint64 length)
{
	return file->write(file, data, length);
}


/*-----------------------------------------------------------------
 * Name :	core_file_close()
 *
 * Description: close a core file
 *
 * Created by:	Maxim Kozlovsky
 *
 * Date Created:	06/10/02
 *
 *-----------------------------------------------------------------
 */
int
core_file_close(struct core_file *file)
{
	return file->close(file);
}

void
crashdump_test_sendpacket(void)
{
	eee_desc_t *desc = eee_crash_allocateDesc(0, 0);
	int i;
	uint32 *bufhdr;
	uint32 size = 1024;
	int rc;

	debug(("> crashdump test sendpacket\n"));

	crash_bmc12500Eth_txClean();
	
	if (desc == NULL) {
		debug(("crashdump_test_sendpacket: can not allocate descriptor\n"));
		return;
	}
	desc->hdr.control = 0;
	EEE_SET_NUM_BUFS(desc, 1);
	EEE_SET_PKT_LEN(desc, size);
	desc->bd[1].control = EEE_BUF_LARGE | EDESC_BD_CMD_IN_USE;
	desc->bd[1].len = size;
	desc->bd[1].buf = eee_crash_allocateBuffer(0, 0, 0);
	if (desc->bd[1].buf == NULL) {
		debug(("crashdump_test_sendpacket: can not allocate buffer\n"));
		return;
	}
	desc->bd[1].buf += 128;
	for (i = 0; i < size; ++i) {
		desc->bd[1].buf[i] = i & 0xff;
	}
	bufhdr = (uint32 *)desc->bd[1].buf;
	*bufhdr++ = nfxMySlotId;
	*bufhdr++ = nfxMyCPUId;
	*bufhdr++ = size;
	*bufhdr++ = EEE_USER_DATA;
	desc->hdr.dest_port = PACK_UNICAST_PORT_ADDRESS(2, 0, 1, EEE_USER_DATA,
													NFX_COS_HIGH, 0);
	desc->hdr.src_port = PACK_UNICAST_PORT_ADDRESS(nfxMySlotId, nfxMyCPUId,
												   50, EEE_USER_DATA, NFX_COS_HIGH, 0);
	desc->hdr.offset = 0;
	desc->hdr.attr = 0;
	desc->hdr.next = 0;
	rc = crash_bmc12500Eth_txUnicast(desc->hdr.dest_port, desc);

	debug(("< crashdump_test_sendpacket: %d\n", rc));
}


void
crashdump_test_receivepacket(void)
{
	debug(("> crashdump_test_receivepacket\n"));
	crash_bmc12500Eth_rx();
	debug(("< crashdump_test_receivepacket\n"));
}


/*-----------------------------------------------------------------
 * Name :	core_read_block()
 *
 * Description: Read 8*1024 bytes from core volume and return the
 * descriptor containing the data.
 *
 * Created by:	Maxim Kozlovsky
 *
 * Date Created:	06/13/02
 *
 *-----------------------------------------------------------------
 */
int
core_read_block(int block_offset, eee_desc_t **descptr)
{
	scsi_readReqMsg_t msg;
	scsi_readRspMsg_t *rsp;
	esm_event_t event;
	eee_desc_t *desc;
	uint32 dp;
	
	msg.hdr.mid = SCSI_MID_READ_REQ;
	msg.hdr.cHandle = esm_getCurrHandle();
	if (sdma_handleToPath(core.strg_handle, &msg.hdr.path, &dp) != NFX_OK) {
		debug(("core_read_block: no path to crash device\n"));
		return NFX_ERR;
	}
	msg.transSeqId = 0;
	msg.blkOffset = core.startBlk + block_offset;
	msg.blkCnt = 16;
	msg.dataDest = 0;
	if (eee_sendMessage(&msg, sizeof(msg), dp, core.src_port) != NFX_OK) {
		debug(("core_read_block: can not send scsi read\n"));
		return NFX_ERR;
	}

	thread_yield(&event, 100);

	desc = event.p1.u.p;

	rsp = (scsi_readRspMsg_t *)desc->bd[1].buf;
	if (rsp->hdr.mid != SCSI_MID_READ_RSP) {
		debug(("core_read_block: invalid read rsp\n"));
		eee_freePacket(desc);
		return NFX_ERR;
	}
	if (rsp->eCode != NFX_OK) {
		debug(("core_read_block: failed to read block %x\n", block_offset));
		eee_freePacket(desc);
		return NFX_ERR;
	}
	*descptr = desc;
	return NFX_OK;
}


int
core_zero_block(int block_offset)
{
	int rc = NFX_ERR;
	int msglen = sizeof(scsi_writeReqMsg_t) + 1024;
	char *data = eee_ramAlloc(msglen);
	scsi_writeReqMsg_t *msg = (scsi_writeReqMsg_t *)data;
	scsi_writeRspMsg_t *rsp;
	esm_event_t event;
	eee_descPtr_t desc;
	uint32 dp;
	if (data != NULL) {
		msg->hdr.mid = SCSI_MID_WRITE_REQ;
		msg->hdr.cHandle = esm_getCurrHandle();
		if (sdma_handleToPath(core.strg_handle, &msg->hdr.path, &dp) != NFX_OK) {
			debug(("core_read_block: no path to crash device\n"));
			return NFX_ERR;
		}
		msg->transSeqId = 0;
		msg->blkOffset = core.startBlk + block_offset;
		msg->blkCnt = 2;
		msg->checkSum = 0;
		bzero(data + sizeof(scsi_writeReqMsg_t), 1024);
		eee_sendMessage(data, msglen, dp, core.src_port);
		thread_yield(&event, 100);

		desc = event.p1.u.p;

		rsp = (scsi_writeRspMsg_t *)desc->bd[1].buf;
		if (rsp->hdr.mid != SCSI_MID_WRITE_RSP) {
			debug(("core_zero_block: invalid write rsp\n"));
		}
		if (rsp->eCode != NFX_OK) {
			debug(("core_zero_block: failed to write block %x\n", block_offset));
		}
		eee_freePacket(desc);
		eee_ramDealloc(data);
		rc = NFX_OK;
	}
	return rc;
}


void
rename_core()
{
	char orgcore[256];
	char newcore[256];
	int fileno;
	int found_place;

	sprintf(orgcore, "%s/fp.core", core.savedir);

	fileno = 1;
	found_place = 0;
	while (!found_place) {
		int32 fh;
		sprintf(newcore, "%s/fp.core.%d", core.savedir, fileno);
		fh = file_open(newcore);
		if (fh == NFX_ERR) {
			found_place = 1;
		}
		else {
			file_close(fh);
			++fileno;
		}
	}

	if (found_place) {
		file_rename(orgcore, newcore);
	}
}


/*-----------------------------------------------------------------
 * Name :	savecore_thread()
 *
 * Description: core thread function. Obtains the core volume parameters
 * for crashdump and saves the existing coredump.
 *
 * Created by:	Maxim Kozlovsky
 *
 * Date Created:	06/13/02
 *
 *-----------------------------------------------------------------
 */
static void
savecore_thread(esm_handle_t handle, void *userInfo)
{
	eee_descPtr_t desc;
	uchar8 *data;
	Elf64_Ehdr *ehdr;
	uint32 fh;
	char file_name[300];
	int i;
	int retries = 20;
	char *msgbuf;
	uint64 core_size;

	debug(("> savecore_thread\n"));

	if (core_getcrashname() == NFX_ERR) {
		debug(("can not get crash volume name\n"));
		return;
	}

	if (core_getcrashauto() == NFX_ERR) {
		debug(("can not get crash auto settings\n"));
		return;
	}

	if (core_getvolid() == NFX_ERR) {
		debug(("can not get crash volume id\n"));
		return;
	}

	if (core_get_strg() == NFX_ERR) {
		debug(("can not get crash volume storage\n"));
		return;
	}

	if (core_open_strg() == NFX_ERR) {
		debug(("can not open crash volume storage\n"));
		return;
	}

	if (core_get_coredir() == NFX_ERR) {
		debug(("core save directory is not assigned\n"));
		return;
	}

	if (core_read_block(0, &desc) == NFX_ERR) {
		debug(("failed to read block 0\n"));
		return;
	}

	data = desc->bd[1].buf + sizeof(scsi_readRspMsg_t);
	ehdr = (Elf64_Ehdr *)data;

	if (!(ehdr->e_ident[EI_MAG0] == ELFMAG0
		  && ehdr->e_ident[EI_MAG1] == ELFMAG1
		  && ehdr->e_ident[EI_MAG2] == ELFMAG2
		  && ehdr->e_ident[EI_MAG3] == ELFMAG3)) {
		E_LOG(class_1, LOG_INFO, core.app, 0, 0, 0, ("no core dump found"));
		return;
	}

	strcpy(file_name, core.savedir);
	strcat(file_name, "/fp.core");
	E_LOG(class_1, LOG_INFO, core.app, 0, 0, 0, ("saving core dump to %s", file_name));

	fh = NFX_ERR;
	while (retries-- && fh == NFX_ERR) {
		fh = file_create(file_name, 0444);
		thread_sleep(NULL, 30000);
	}
	
	if (fh == NFX_ERR) {
		debug(("can not open %s\n", file_name));
		return;
	}

	msgbuf = eee_ramAlloc(sizeof(file_WriteHeader) + FILE_BLOCK_SIZE);
	if (msgbuf == NULL) {
		return;
	}
	
	debug(("opened core file fh=%d\n", fh));
	if (file_write_use_prealloc(fh, data, 512 * 16, msgbuf) != 512 * 16) {
		debug(("savecore: write 0 failed\n"));
		eee_ramDealloc(msgbuf);
		return;
	}
	eee_freePacket(desc);
	desc = NULL;

	i = 16;
	core_size = MIN(core_blocks(), core.block_size - 16);
	while (i < core_size) {
		if (core_read_block(i, &desc) == NFX_ERR) {
			debug(("savecore thread - can not read block %d\n", i));
			eee_ramDealloc(msgbuf);
			return;
		}
		if ((i % (100 * 16)) == 0)
			debug(("."));
		NFX_ASSERT(EEE_GET_NUM_BUFS(desc) == 1);
		data = desc->bd[1].buf + sizeof(scsi_readRspMsg_t);
		if (file_write_use_prealloc(fh, data, 512 * 16, msgbuf) != 512 * 16) {
			debug(("savecore : write %d failed\n", i));
			eee_ramDealloc(msgbuf);
			return;
		}

		eee_freePacket(desc);
		i += 16;
	}

	eee_ramDealloc(msgbuf);

	file_close(fh);

	core_zero_block(0);

	rename_core();

	debug(("saved core successfully\n"));
}


void
savecore()
{
    esm_handle_t handle;
	handle = thread_create(NULL);
    thread_start(&handle, rename_core);
}


static uchar8 *malloc_mem_start;
static uchar8 *malloc_mem_end;
static uchar8 *malloc_mem_ptr;

/*-----------------------------------------------------------------
 * Name :	init_malloc()
 *
 * Description: Preallocate malloc pool to use after crash
 *
 * Created by:	Maxim Kozlovsky
 *
 * Date Created:	06/10/02
 *
 *-----------------------------------------------------------------
 */
static void
init_malloc()
{
	debug(("> init_malloc\n"));
	malloc_mem_start = eee_ramAlloc(CRASH_MALLOC_SIZE);
	if (malloc_mem_start == NULL) {
		panic("Can not allocate crash malloc memory\n");
	}
	malloc_mem_end = malloc_mem_start + CRASH_MALLOC_SIZE;
	malloc_mem_ptr = malloc_mem_start;
	debug(("< init_malloc\n"));
}


static uchar8 *buflist;
static int freebufs;

/*-----------------------------------------------------------------
 * Name :	init_buffers()
 *
 * Description: Preallocate buffers pool to use after crash
 *
 * Created by:	Maxim Kozlovsky
 *
 * Date Created:	06/10/02
 *
 *-----------------------------------------------------------------
 */
static void
init_buffers()
{
	int i;
	debug(("> init_buffers\n"));
	for (i = 0; i < CRASH_BUFFERS; ++i) {
		uchar8 *buf = eee_allocateBuffer(eee.my_pool_id, EEE_BUF_ULTRA, NFX_COS_HIGH);
		if (buf == NULL) {
			panic("Failed to allocate crash buffers pool\n");
		}
		eee_crash_deallocateBuffer(buf);
	}
	debug(("< init_buffers\n"));
}


static eee_desc_t *desclist;
static int freedescs;

/*-----------------------------------------------------------------
 * Name :	init_descriptors()
 *
 * Description: Preallocate descriptors pool to use after crash
 *
 * Created by:	Maxim Kozlovsky
 *
 * Date Created:	06/10/02
 *
 *-----------------------------------------------------------------
 */
static void
init_descriptors()
{
	int i;
	debug(("> init_descriptors\n"));
	for (i = 0; i < CRASH_DESCRIPTORS; ++i) {
		eee_desc_t *desc = eee_allocateDesc(eee.my_pool_id, NFX_COS_HIGH);
		if (desc == NULL) {
			panic("Failed to allocate crash descriptors pool\n");
		}
		eee_crash_deallocateDesc(desc);
	}
	debug(("< init_descriptors\n"));
}


/*-----------------------------------------------------------------
 * Name :	crash_init()
 *
 * Description: Initialize crashdump module
 *
 * Created by:	Maxim Kozlovsky
 *
 * Date Created:	06/10/02
 *
 *-----------------------------------------------------------------
 */
void
crash_init()
{
	debug(("> crash_init\n"));
	init_core_file();
	init_malloc();
	init_buffers();
	init_descriptors();
	debug(("< crash_init\n"));
}


/*-----------------------------------------------------------------
 * Name :	eee_crash_allocateDesc()
 *
 * Description: Allocate descriptor after crash
 *
 * Created by:	Maxim Kozlovsky
 *
 * Date Created:	06/10/02
 *
 *-----------------------------------------------------------------
 */
eee_desc_t *
eee_crash_allocateDesc(uint32 pool, uint32 cos)
{
	eee_desc_t *desc = desclist;
	if (desc != NULL) {
		desclist = desc->hdr.next;
		desc->hdr.next = NULL;
		--freedescs;
	}
	return desc;
}


/*-----------------------------------------------------------------
 * Name :	eee_crash_deallocateDesc()
 *
 * Description: Deallocate descriptor after crash
 *
 * Created by:	Maxim Kozlovsky
 *
 * Date Created:	06/10/02
 *
 *-----------------------------------------------------------------
 */
void
eee_crash_deallocateDesc(eee_desc_t *desc)
{
	desc->hdr.next = desclist;
	desclist = desc;
	++freedescs;
}


/*-----------------------------------------------------------------
 * Name :	eee_crash_ramAlloc()
 *
 * Description: allocate ram after crash
 *
 * Created by:	Maxim Kozlovsky
 *
 * Date Created:	06/10/02
 *
 *-----------------------------------------------------------------
 */
void *
eee_crash_ramAlloc(uint32 size)
{
	return eee_crash_ramAllocAligned(size);
}

/*-----------------------------------------------------------------
 * Name :	eee_crash_ramAllocAligned()
 *
 * Description: allocate aligned ram after crash
 *
 * Created by:	Maxim Kozlovsky
 *
 * Date Created:	06/10/02
 *
 *-----------------------------------------------------------------
 */
void *
eee_crash_ramAllocAligned(uint32 size)
{
	void *optr = malloc_mem_ptr;
	
	malloc_mem_ptr += size;
	malloc_mem_ptr = (void *)(((uint64)malloc_mem_ptr + CRASH_MALLOC_ALIGN - 1)
							  & ~(CRASH_MALLOC_ALIGN - 1));
	return malloc_mem_ptr > malloc_mem_end ? NULL : optr;
}


/*-----------------------------------------------------------------
 * Name :	eee_crash_allocateBuffer()
 *
 * Description: allocate a buffer after crash
 *
 * Created by:	Maxim Kozlovsky
 *
 * Date Created:	06/10/02
 *
 *-----------------------------------------------------------------
 */
void *
eee_crash_allocateBuffer(uint32 pool, uint32 size, uint32 cos)
{
	uchar8 *buf = buflist;
	if (buf != NULL) {
		buflist = *(uchar8 **)buflist;
		--freebufs;
	}
	return buf;
}


/*-----------------------------------------------------------------
 * Name :	eee_crash_deallocateBuffer()
 *
 * Description: Deallocate buffer after crash
 *
 * Created by:	Maxim Kozlovsky
 *
 * Date Created:	06/10/02
 *
 *-----------------------------------------------------------------
 */
void
eee_crash_deallocateBuffer(uchar8 *buf)
{
	uchar8 *realbuf = (uchar8 *)((address_t)buf & ~0xFF);
	*(uchar8 **)realbuf = buflist;
	buflist = realbuf;
	++freebufs;
}


/*-----------------------------------------------------------------
 * Name :	eee_crash_freePacket()
 *
 * Description: free packet after crash
 *
 * Created by:	Maxim Kozlovsky
 *
 * Date Created:	06/10/02
 *
 *-----------------------------------------------------------------
 */
void
eee_crash_freePacket(eee_desc_t *pktdesc)
{
	eee_desc_t *desc = pktdesc;
	eee_descBuf_t *bd = edesc_firstBuf(desc);
	while (bd != NULL) {
		eee_crash_deallocateBuffer(bd->buf);
		bd = eee_nextBuf(&desc, bd);
	}
	eee_crash_deallocateDesc(pktdesc);
}


/*-----------------------------------------------------------------
 * Name :	eee_crash_receivePacket()
 *
 * Description: Called by the driver to process received packets after crash
 *
 * Created by:	Maxim Kozlovsky
 *
 * Date Created:	06/10/02
 *
 *-----------------------------------------------------------------
 */
void
eee_crash_receivePacket(eee_desc_t *desc)
{
	if (EEE_GET_SLOT_NUM(desc->hdr.dest_port) == nfxMySlotId
		&& EEE_GET_DEST_CPU(desc->hdr.dest_port) == nfxMyCPUId
		&& EEE_GET_APP_ID(desc->hdr.dest_port) == core.app) {
		scsi_writeRspMsg_t *msg = (scsi_writeRspMsg_t *)desc->bd[1].buf;
		if (msg->hdr.mid != SCSI_MID_WRITE_RSP) {
			printf("Invalid scsi write response mid. slot=%d cpu=%d\n",
				   EEE_GET_SLOT_NUM(desc->hdr.src_port),
				   EEE_GET_DEST_CPU(desc->hdr.src_port));
		}
		else {
			if (msg->eCode != NFX_OK) {
				printf("scsi write failed: %d\n", msg->eCode);
				write_error = 1;
			}
			--outstanding_ios;
		}
	}
	eee_crash_freePacket(desc);
}
