/*------------------------------------------------------------------------------
 * Copyright (C) 2001, Agile Storage, Inc.
 * RCS          :  $Id: datamover-common.c,v 1.2 2002/09/25 22:17:53 pankajg Exp $
 *
 * Name         : am-datamover-common.c
 *
 * Description  : Agile Mirror (AM) data mover common code.
 *
 * Created by   : Isabelle Cnudde
 *
 * Date Created : 16-NOV-2001
 *
 *------------------------------------------------------------------------------
 */

#include "nfx-incl.h"
#include <exports/eee-api.h>

#include "datamover.h"


#ifdef LINUX_TEST
#include <stdio.h>
#include <errno.h>
#include <fcntl.h>
#include <sys/socket.h>
#include <netinet/ip.h>
#include <netinet/udp.h>
#endif // ISABELLE

/*------------------------------------------------------------------------------
 * Global functions.
 *------------------------------------------------------------------------------
 */


/*------------------------------------------------------------------------------
 * Name         : dm_sendMessage
 *
 * Description  : Send a message.
 *                Special handling when source = destination.
 *
 * Created by   : Isabelle Cnudde
 *
 * Date Created : 20-NOV-2001
 *
 *------------------------------------------------------------------------------
 */

int32
dm_sendMessage(void *    src,
               uint32    num_bytes,
               uint32    dest_port,
               uint32    source_port)
    
{
    int32 rc = NFX_OK;

	DM_DBG_PRINT(("dm_sendMessage() dest_port 0x%x source_port 0x%x\n",
					dest_port, source_port));

    if (dest_port != source_port)
    {
        rc = eee_sendMessage(src,
                             num_bytes,
                             dest_port,
                             source_port);
    }
    else
    {
        eee_desc_t *    eDescp;
        char *          eBuffp;
        
        /* Get a EEE Buffer.
         */
        eBuffp = eee_allocateBuffer(EEE_SHARED_POOL, EEE_BUF_LARGE, 0);
        if (eBuffp == NULL)
        {
            return(NFX_ERR);
        }
        memcpy((void *) eBuffp,
               (const void *) src,
               num_bytes);
        
        /* Get a EEE Descriptor.
         */
        eDescp = eee_allocateDesc(EEE_SHARED_POOL, NFX_COS_HIGH);
        if (eDescp == NULL)
        {
            return(NFX_ERR);
        }        
        bzero(eDescp, sizeof(eee_desc_t));
        
        /* Setup EDesc info.
         */
        eDescp->bd[0].control = 0;
        eDescp->bd[1].buf     = eBuffp;
        eDescp->bd[1].len     = (ushort16) num_bytes;
        eDescp->bd[1].control = EDESC_BD_CMD_IN_USE | EDESC_BD_LARGE;
        eDescp->hdr.dest_port = dest_port;
        eDescp->hdr.src_port  = source_port;
        eDescp->hdr.control   = (num_bytes & EDESC_TOTAL_LEN) |
            (( 1 << EDESC_BUFCNT_SHIFT) & EDESC_BUFCNT_MASK);
        
        eee_receivePacket(eDescp);
        return(NFX_OK);
    }
    
    return(rc);
}

#ifdef LINUX_TEST
/*------------------------------------------------------------------------------
 * Name         : am_dm_openSock
 *
 * Description  : Open linux sockets in simulated env.
 * For UDP only!!!
 *
 * Created by   : Isabelle Cnudde
 *
 * Date Created : 16-NOV-2001
 *
 *------------------------------------------------------------------------------
 */

int
dm_openSock(uint32   laddr,  // network byte order
            ushort16 lport)  // network byte order
    
{
	int					sock;
    struct sockaddr_in  sock_addr;

	DM_DBG_PRINT(("> dm_openSock\n"));

    /* Get TFTP UDP socket.
     */
    sock = socket(PF_INET, SOCK_DGRAM, IPPROTO_UDP);
    if (sock < 0)
    {
        printf("can't open UDP socket: %s\n", sys_errlist[errno]);
        return -1;
    }

    /* Make it non-blocking.
     */
    if (fcntl(sock, F_SETFL, FNDELAY) < 0)
    {
        printf("can't fcntl socket: %s\n", sys_errlist[errno]);
        return -1;
    }

    /* Bind the socket.
     */
    bzero( (char *) &sock_addr, sizeof(sock_addr) );
    sock_addr.sin_family      = AF_INET;
    sock_addr.sin_addr.s_addr = laddr;
    sock_addr.sin_port        = lport;

    if (bind(sock, (struct sockaddr *)&sock_addr, sizeof(sock_addr)) < 0)
    {
        printf("can't bind socket: %s\n", errno, sys_errlist[errno]);
        return -1;
    }

    return(sock);
}
#endif // LINUX_TEST

/*----------------------------------------------------------------------------*/
