/*------------------------------------------------------------------------------
 * Copyright (C) 2001, Agile Storage, Inc.
 * RCS          :  $Id: datamover.h,v 1.7 2002/10/02 00:18:00 pankajg Exp $
 *
 * Name         : datamover.h
 *
 * Description  : Main header file of the data mover (DM).
 *
 * Created by   : Isabelle Cnudde
 *
 * Date Created : 04-OCT-2001
 * Re-designed  : 30-NOV-2001
 *
 *------------------------------------------------------------------------------
 */
#ifndef _DATAMOVER_H_
#define _DATAMOVER_H_

/*------------------------------------------------------------------------------
 * Include files
 *------------------------------------------------------------------------------
 */

#include "nfx-incl.h"

#include "../sm-bsd-sys/types.h"
#include "../sm-bsd-netinet/in.h"

#include "exports/elog-api.h"
#include "exports/eee-api.h"
#include "exports/esm-api.h"
#include "exports/tpl-api.h"
#include "exports/pkt.h"
#include "exports/luc-api.h"

#include "datamover-api.h"
#include "datamover-sm.h"

/*------------------------------------------------------------------------------
 * Constants and macros.
 *------------------------------------------------------------------------------
 */

#define DM_DBG_PRINT(_par_) E_LOG(class_1, LOG_DEBUG, dmAppId, 0, 0, 0, _par_)
#define DM_ERR_PRINT(_par_) E_LOG(class_1, LOG_ERR, dmAppId, 0, 0, 0, _par_)
#define DM_FCT_PRINT(_par_) E_LOG(class_1, LOG_INFO, dmAppId, 0, 0, 0, _par_)

/*------------------------------------------------------------------------------
 * Global variables.
 *------------------------------------------------------------------------------
 */

/* My own application id and EEE address port.
 */
extern int32    dmAppId;
extern uint32   dmAddr;

extern ushort16 srcPort;

#ifdef LINUX_TEST
extern luc_user_t  lucUserTab[MAX_LUC_USER];

#ifdef ISABELLE // code to delete
// added for luc-sim.c !!!
ushort16 amServerPort;
ushort16 amClientPort;
int      amServerSock;
int      amClientSock;
#endif // ISABELLE

#endif // LINUX_TEST

/*------------------------------------------------------------------------------
 * Type definitions.
 *------------------------------------------------------------------------------
 */

#define DM_SERVER_CB    1
#define DM_INSTANCE_CB  2

/* Control block of a data mover server.
 */
typedef struct
{
    ushort16    type;           // DM_SERVER_CB
    
    uint64      appHdl;         // handle to AM agent
    uint32      appAddr;        // EEE address of the AM agent
    void *      tplBindCb;      // handle for TPL
    
    ushort16    protocol;       // protocol TCP/UDP
    uint64      lclAddr;        // local IP @ (source)
    uint64    	lclPort;        // local port (source)
    
#ifdef LINUX_TEST
    int         lucUserIndex;   // index in the LUC's user table
#endif // LINUX_TEST
} dm_server_t;

/* Control block of the data mover.
 */
typedef struct
{
    ushort16    type;           // DM_INSTANCE_CB
    
    uint64      dmHdl;          // handle to AM data mover (itself)
    uint64      appHdl;         // handle to AM agent
    uint32      appAddr;        // EEE address of the AM agent
    void *      tplBindCb;      // handle for TPL
    void *      tplConnId;      // handle for the connection in TPL
    
    boolean     isClient;       // TRUE: client, FALSE: server
    ushort16    protocol;       // protocol TCP/UDP
    uint64      lclAddr;        // local IP @ (source)
    uint64	    lclPort;        // local port (source)
    uint64      rmtAddr;        // remote IP @ (target)
    uint64	    rmtPort;        // remote port (target)

    uint32      nbPktSntOk;     // number of packet sent successfully
    uint32      nbPktSntNok;    // number of packet sent unsuccessfully
    uint32      nbPktRcv;       // number of packet received
    
#ifdef LINUX_TEST
    int         lucUserIndex;   // index in the LUC's user table
#endif // LINUX_TEST
} dm_cb_t;

/* Header of the data packet sent from the source to the target.
 */
typedef struct
{
	ushort16	appHdrSize;		// user application header size
	boolean     isCOD;          // is there data?
    //char        appHdr[1];      // application header
} dm_pktHdr_t;

/*------------------------------------------------------------------------------
 * Prototypes of global functions.
 *------------------------------------------------------------------------------
 */

void
dm_init(void);

void
dm_cleanup(void);

void
dm_rcvMsg(eee_descPtr_t  edesc);

void
dm_rcvMcastMsg(eee_descPtr_t  edesc);

int32
dm_addConnInd(tpl_bindCb_t *    pBind,
              void *            pConnId,
              void **           ppAppHdl,
              //uint32            status, - not yet implemented in TPL
              uint32            raddr,
              ushort16          rport);

int32
dm_delConnInd(void *    pAppHdl,
              void *    pConnId);

int32
dm_rcvPktInd(void *         pAppHdl,
             pkt_desc_t *   pDesc);

int32
dm_xmtCmpInd(void *         pAppHdl,
             pkt_desc_t *   pDesc);

//am-datamover-common.c
int32
dm_sendMessage(void *    src,
               uint32    num_bytes,
               uint32    dest_port,
               uint32    source_port);

#ifdef LINUX_TEST
int
dm_openSock(uint32   laddr,
            ushort16 lport);
#endif // LINUX_TEST

/*----------------------------------------------------------------------------*/

#endif // _DATAMOVER_H_
