/*
 *-----------------------------------------------------------------
 *
 * Name:	dp-proxy-init.c
 *
 * RCS:  $Id: dp-proxy-init.c,v 1.6 2002/12/09 23:56:55 rick Exp $
 *
 * Copyright (C) 2001, Agile Storage, Inc.
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	12/10/01
 *
 *-----------------------------------------------------------------
 */

#include <nfx-incl.h>
#include <exports/eee-api.h>
#include <exports/cm-api.h>
#include "dp-proxy.h"

#ifdef NFP_FP
nfx_spinlock_t dpp_rq_lock;
nfx_spinlock_t dpp_rxdq_lock;
#endif

void			** dpp_rq_start;
void			** dpp_rq_end;
void			** dpp_rq_curr;

dp_proxy_rxd_t	* dpp_rxdq_start;
dp_proxy_rxd_t	* dpp_rxdq_end;
dp_proxy_rxd_t	* dpp_rxdq_curr;

#ifdef NFP_FP
uchar8			* dpp_buf_base;
uchar8			* dpp_buf_end;
#endif

/*-----------------------------------------------------------------
 * Name:	dp_proxy_init()
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	12/10/01
 *
 *-----------------------------------------------------------------
 */
void
dp_proxy_init(
	uint32	pass
)
{
	int32	dpp_app;

	if (pass == 1) {
		printf("[dp_proxy_init]: pass 1\n");
#ifdef NFP_FP
		dpp_buf_base = (uchar8 *)eee_allocateBlock
									(DP_PROXY_NUM_BUFS * DP_PROXY_BUF_SIZE);
		dpp_buf_end = dpp_buf_base + (DP_PROXY_NUM_BUFS * DP_PROXY_BUF_SIZE);
//printf("dpp_buf_base at %p (%p), buf_end at %p (%p)\n", dpp_buf_base, virt_to_phys(dpp_buf_base), dpp_buf_end, virt_to_phys(dpp_buf_end));
		dpp_rxdq_start = (dp_proxy_rxd_t *)eee_allocateBlock
								(DP_PROXY_NUM_BUFS * sizeof(dp_proxy_rxd_t));
		if (spans_dram_blocks((address_t)dpp_rxdq_start,
								DP_PROXY_NUM_BUFS * sizeof(dp_proxy_rxd_t)))
			panic("dpp rxd spans physical dram blocks\n");
		dpp_rxdq_end = dpp_rxdq_start + DP_PROXY_NUM_BUFS;
//printf("dpp_rxdq_start at %p, rxdq_end at %p\n", dpp_rxdq_start, dpp_rxdq_end);
		dpp_rxdq_curr = dpp_rxdq_start;
		memset(dpp_rxdq_start, 0, DP_PROXY_NUM_BUFS * sizeof(dp_proxy_rxd_t));

		dpp_rq_start = dpp_rq_end = dpp_rq_curr = 0;
#endif	/* NFP_FP */

#ifdef NFP_TXRX
#ifdef VARIABLE_DPP_QUEUE
		dpp_rq_start = (void **)eee_allocateBlock
										(DP_PROXY_MAX_BUFS * sizeof(void *));
		dpp_rq_end = dpp_rq_start + DP_PROXY_MAX_BUFS;
		dpp_rq_curr = dpp_rq_start;
		memset(dpp_rq_start, 0, DP_PROXY_MAX_BUFS * sizeof(void *));
#else
		dpp_rq_start = (void **)eee_allocateBlock
										(DP_PROXY_NUM_BUFS * sizeof(void *));
		dpp_rq_end = dpp_rq_start + DP_PROXY_NUM_BUFS;
		dpp_rq_curr = dpp_rq_start;
		memset(dpp_rq_start, 0, DP_PROXY_NUM_BUFS * sizeof(void *));
#endif

		dpp_rxdq_start = dpp_rxdq_end = dpp_rxdq_curr = 0;
#endif	/* NFP_TXRX */

	}
	else {
		dp_proxy_init_msg_t	msg;
		uint32	dp;

		dpp_app = eee_getApplicationId("dp_proxy");

		printf("[dp_proxy_init]: pass 2\n");

		msg.cmd = DP_PROXY_CMD_INIT;
#ifdef NFP_TXRX
		msg.base = dpp_rq_start;
		msg.end = dpp_rq_end;
#ifdef VARIABLE_DPP_QUEUE
		msg.num_bufs = DP_PROXY_MAX_BUFS;
#endif
		dp = PACK_UNICAST_PORT_ADDRESS(nfxMySlotId,
										NFX_NFP_FP_ANY, 
										dpp_app,
										EEE_MGMT,
										NFX_COS_HIGH,
										PORT_BEST_FIT);
#endif

#ifdef NFP_FP
		NFX_SPIN_INIT(&dpp_rq_lock, SPINLOCK_LEVEL_UNKNOWN);
		NFX_SPIN_INIT(&dpp_rxdq_lock, SPINLOCK_LEVEL_UNKNOWN);

		msg.base = dpp_rxdq_start;
		msg.end = dpp_rxdq_end;

#ifdef VARIABLE_DPP_QUEUE
		msg.num_bufs = DP_PROXY_NUM_BUFS;
#endif

		dp = PACK_UNICAST_PORT_ADDRESS(nfxMySlotId,
										NFX_NFP_TXRX_ANY, 
										dpp_app,
										EEE_MGMT,
										NFX_COS_HIGH,
										PORT_BEST_FIT);

		eee_registerPollingFunc(EEE_HIGH_PRIORITY, dp_proxy_rx, NULL);
#endif
		eee_registerReceiveFunc(dpp_app, dp_proxy_app_rx, dp_proxy_app_rx);
printf("DP_PROXY: sendMessage\n");
		eee_sendMessage(&msg, sizeof(dp_proxy_init_msg_t), dp, 0);
	}

}	/* dp_proxy_init() */

