/*
 *-----------------------------------------------------------------
 *
 * Name:	dp-proxy.c
 *
 * RCS:  $Id: dp-proxy.c,v 1.24 2002/12/09 23:56:55 rick Exp $
 *
 * Copyright (C) 2001, Agile Storage, Inc.
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	12/10/01
 *
 *-----------------------------------------------------------------
 */

#include <nfx-incl.h>
#include <exports/eee-api.h>
#ifdef NFP_TXRX
#include <exports/sb1250-dm-api.h>
#endif
#include <exports/malloc-api.h>
#include <exports/bmc12500-eth.h>
#include "dp-proxy.h"

dpp_stats_t	dpp_stats;

#ifdef NFP_FP
extern nfx_spinlock_t dpp_rq_lock;
extern nfx_spinlock_t dpp_rxdq_lock;
#endif

/*-----------------------------------------------------------------
 * Name:	dp_proxy_app_rx()
 *
 * Description: Application receive function for messages destined to
 *				app, "dp_proxy".
 *
 * Created by:	Rick Lund
 *
 * Date Created:	12/10/01
 *
 *-----------------------------------------------------------------
 */
void
dp_proxy_app_rx(
	eee_descPtr_t	edesc
)
{
	dp_proxy_init_msg_t	* msg;
#ifdef NFP_FP
	uchar8				* ptr;
	extern int32 spans_dram_blocks( address_t, uint32);
#endif
	msg = (dp_proxy_init_msg_t *)edesc->bd[1].buf;

	if (msg->cmd != DP_PROXY_CMD_INIT)
		panic("Invalid cmd to dp_proxy_app_rx, edesc = 0x%x\n", edesc);

#ifdef NFP_FP
	NFX_SPIN_LOCK(&dpp_rq_lock);

	dpp_rq_start = msg->base;
#ifdef VARIABLE_DPP_QUEUE
	dpp_rq_end = (void *)((address_t *)msg->base + DP_PROXY_NUM_BUFS);
#else
	dpp_rq_end = msg->end;
#endif
	dpp_rq_curr = msg->base;

	/*
	 * Add all buffers to resource queue in TXRX memory
	 */
	for (ptr = dpp_buf_base; ptr < dpp_buf_end; ptr += DP_PROXY_BUF_SIZE) {
		if (spans_dram_blocks((address_t)ptr, DP_PROXY_BUF_SIZE)) {
			printf("Unusable DPP proxy buffer, %p\n", ptr);
			continue;
		}
		*dpp_rq_curr = ptr;
		dpp_rq_curr++;
		if (dpp_rq_curr == dpp_rq_end)
			dpp_rq_curr = dpp_rq_start;
	}

	NFX_SPIN_UNLOCK(&dpp_rq_lock);

#endif	/* NFP_FP */

#ifdef NFP_TXRX
	dpp_rxdq_start= msg->base;
	dpp_rxdq_end = msg->end;
	dpp_rxdq_curr = msg->base;

#ifdef VARIABLE_DPP_QUEUE
	dpp_rq_end = dpp_rq_start + msg->num_bufs;
#endif
#endif	/* NFP_TXRX */

	eee_freePacket(edesc);

}	/* dp_proxy_app_rx() */

#ifdef NFP_FP
/*-----------------------------------------------------------------
 * Name:	dp_proxy_rx()
 *
 * Description: Polling function running on FP, which checks the receive
 *				descriptor queue and enqueues packets to MAC driver.
 *
 * Created by:	Rick Lund
 *
 * Date Created:	12/10/01
 *
 *-----------------------------------------------------------------
 */
void
dp_proxy_rx(
	void	* cb,
	uint32	tref
)
{
	int			  rc;
	agile_hdr_t	* hdr;
	uint32		x;
    int32         retries;

	if (!NFX_SPIN_TRYLOCK(&dpp_rxdq_lock))
		return;

	while (1) {
		if ((x = dpp_rxdq_curr->control) & DP_PROXY_RXD_FP_OWN) {
			dpp_stats.rx++;
            retries = 0;
			if (dpp_rxdq_curr->buf == 0) {
				dpp_stats.badbuforsomething++;
				//printf("dpp_rxdq_curr = %p\n", dpp_rxdq_curr);
				//printf("buf = %p\n", dpp_rxdq_curr->buf);
				//printf("len = 0x%x\n", dpp_rxdq_curr->length);
				//printf("control = 0x%x(x = 0x%x)\n",dpp_rxdq_curr->control,x);
                if (++retries == 1000)
                {
                    printf("dp_proxy_rx: retries exhausted\n");
                    break;
                }
			}
			//printf("DPP: found good data at desc, %p\n", dpp_rxdq_curr);
			//printf("DPP: bufptr = %p, length = 0x%x\n",
			//						dpp_rxdq_curr->buf, dpp_rxdq_curr->length);

			/*
			 * Queue to driver
			 */
			hdr = (agile_hdr_t *)(dpp_rxdq_curr->buf);

			if (dpp_rxdq_curr->length < 36)
				dpp_rxdq_curr->length = 36;
			rc = bmc12500Eth_tx_raw( dpp_rxdq_curr->buf, 
								dpp_rxdq_curr->length,
							    hdr->dest_port);
			if( rc == EEE_DROP_PACKET )
				dp_proxy_tx_cleanup(dpp_rxdq_curr->buf);

			dpp_rxdq_curr->buf = 0;
			dpp_rxdq_curr->length = 0;
			dpp_rxdq_curr->control = 0;
			if (++dpp_rxdq_curr == dpp_rxdq_end)
				dpp_rxdq_curr = dpp_rxdq_start;
		}
		else
			break;
	}

	NFX_SPIN_UNLOCK(&dpp_rxdq_lock);

}	/* dp_proxy_rx() */

#endif	/* NFP_FP */

#ifdef NFP_TXRX
/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	8/22/02
 *
 *-----------------------------------------------------------------
 */
int32
dp_proxy_tx_sac_data(
	eee_descPtr_t	edesc,
	uint32			cod_len,
	uint32			cod_offset,
	uchar8			* msg,
	uint32			msglen,
	uint32			dp,
	uint32			sp
)
{
	uchar8			* bufstart, * dest;
	dp_proxy_rxd_t	* rxd;
	dp_proxy_rxd_t	*rxd_buf;
	uint32			tot_len;
	uchar8			* bptr;
	uint32			dp_proxy_offset;
	eee_descBuf_t	* bd;
	uint32			len;
	uint32			num_bufs;
	extern uint64	ht_bytes_tx;

	/*
	 * Get destination buffer address
	 */
	if ((dest = *dpp_rq_curr)) {
		*dpp_rq_curr = 0;
		if (++dpp_rq_curr == dpp_rq_end)
			dpp_rq_curr = dpp_rq_start;
	}
	else {
		printf("No FP dp_proxy resources available\n");
		dpp_stats.no_buf++;
		return -1;
	}

    if (isCheetah()) {
        dp_proxy_offset = DP_PROXY_OFFSET + 4; /* make room for LMUX header */
    }
    else {
        dp_proxy_offset = DP_PROXY_OFFSET;
    }
	/*
	 * Get rx desc entry address
	 */
	rxd = dpp_rxdq_curr;
	if (++dpp_rxdq_curr == dpp_rxdq_end)
		dpp_rxdq_curr = dpp_rxdq_start;
	
	/*
	 * copy data to destination buffer
	 */
	tot_len = cod_len + msglen + sizeof(agile_hdr_t);
	bufstart = dest + dp_proxy_offset;

//printf("dp_proxy_tx_sac_data: dest = %p, bufstart = %p, tot_len = 0x%x\n", dest, bufstart, tot_len);

	if (tot_len > DP_PROXY_BUF_SIZE - dp_proxy_offset)
		panic("DPP: Invalid buffer length, 0x%x\n", tot_len);

//printf("dp = 0x%x, sp = 0x%x\n", dp, sp);
	((agile_hdr_t *)bufstart)->dest_port = dp;
	((agile_hdr_t *)bufstart)->src_port = sp;


	bptr = bufstart+sizeof(agile_hdr_t);

	memcpy(bptr, msg, msglen);
	bptr += msglen;

	while (1) {
		num_bufs = EEE_GET_NUM_BUFS(edesc);
	
		if (edesc->hdr.control & EDESC_COMMAND_USE_FIRST_BUFFER)
			bd = &(edesc->bd[0]);
		else
			bd = &(edesc->bd[1]);
	
		while ((num_bufs) && (cod_offset)) {
			if (cod_offset >= bd->len) {
				cod_offset -= bd->len;
				bd++;
				num_bufs--;
			}
			else
				break;
		}
		if (num_bufs == 0)
			edesc = edesc->hdr.next;
		else if (cod_offset < bd->len)
			break;
		else
			edesc = edesc->hdr.next;
	}
	
	while (cod_len) {
		if (cod_len + cod_offset > bd->len)
			len = bd->len - cod_offset;
		else
			len = cod_len;

//printf("memcpy 0x%x bytes from %p to %p\n", len, bd->buf+cod_offset, bptr);
//        memcpy(bptr, bd->buf+cod_offset, len);
		sb1250dm_movData_buf(SB1250DM_CHNL_SAC_TO_SNIM,
                                len,
                                bd->buf+cod_offset,
                                bptr,
                                0);

		cod_offset = 0;
		cod_len -= len;
		bptr += len;
		bd++;
		num_bufs--;
		if (cod_len) {
			if (num_bufs == 0) {
				edesc = edesc->hdr.next;
				num_bufs = EEE_GET_NUM_BUFS(edesc);
	
				if (edesc->hdr.control & EDESC_COMMAND_USE_FIRST_BUFFER)
					bd = &(edesc->bd[0]);
				else
					bd = &(edesc->bd[1]);
			}
		}
	}

	/*
	 * add entry to receive descriptor queue
	 */
	rxd_buf = eee_ramAlloc(sizeof(dp_proxy_rxd_t));
	rxd_buf->length  = tot_len;
	rxd_buf->buf     = bufstart;
	rxd_buf->control = DP_PROXY_RXD_FP_OWN;
//printf("memcpy 0x%x bytes from %p to %p\n", sizeof(dp_proxy_rxd_t), rxd_buf, rxd);
//	memcpy( rxd, rxd_buf, sizeof(dp_proxy_rxd_t));
	sb1250dm_movData_buf( SB1250DM_CHNL_SAC_TO_SNIM,
                            sizeof(dp_proxy_rxd_t),
                            (char *)rxd_buf,
                            (char *)rxd,
                            (void *)1);

	dpp_stats.tx++;

	ht_bytes_tx += tot_len + sizeof(dp_proxy_rxd_t);

	return NFX_OK;

}	/* dp_proxy_tx_sac_data() */

#ifdef USE_DP_PROXY
/*-----------------------------------------------------------------
 * Name:	dp_proxy_tx_flat_buffer()
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	12/10/01
 *
 *-----------------------------------------------------------------
 */
int32
dp_proxy_tx_flat_buffer(
	uchar8	* buf,
	uint32	num_bytes,
	uint32	dp,
	uint32	sp
)
{
	uchar8			* bufstart, * dest;
	dp_proxy_rxd_t	* rxd;
	dp_proxy_rxd_t	*rxd_buf;
	uint32			tot_len;
	uint32			dp_proxy_offset;
	extern uint64	ht_bytes_tx;

	/*
	 * Get destination buffer address
	 */
	if ((dest = *dpp_rq_curr)) {
		*dpp_rq_curr = 0;
		if (++dpp_rq_curr == dpp_rq_end)
			dpp_rq_curr = dpp_rq_start;
	}
	else {
		printf("No FP dp_proxy resources available\n");
		dpp_stats.no_buf++;
		return -1;
	}

    if (isCheetah()) {
        dp_proxy_offset = DP_PROXY_OFFSET + 4; /* make room for LMUX header */
    }
    else {
        dp_proxy_offset = DP_PROXY_OFFSET;
    }
	/*
	 * Get rx desc entry address
	 */
	rxd = dpp_rxdq_curr;
	if (++dpp_rxdq_curr == dpp_rxdq_end)
		dpp_rxdq_curr = dpp_rxdq_start;
	
	/*
	 * copy data to destination buffer
	 */
	tot_len = num_bytes + sizeof(agile_hdr_t);
	bufstart = dest + dp_proxy_offset;

	if (tot_len > DP_PROXY_BUF_SIZE - dp_proxy_offset)
		panic("DPP: Invalid buffer length, 0x%x\n", tot_len);

	((agile_hdr_t *)bufstart)->dest_port = dp;
	((agile_hdr_t *)bufstart)->src_port = sp;
	sb1250dm_movData_buf(	1,
							num_bytes,
							buf,
							bufstart+sizeof(agile_hdr_t),
							0);

	/*
	 * add entry to receive descriptor queue
	 */
	rxd_buf = eee_ramAlloc(sizeof(dp_proxy_rxd_t));
	rxd_buf->length  = tot_len;
	rxd_buf->buf     = bufstart;
	rxd_buf->control = DP_PROXY_RXD_FP_OWN;
	sb1250dm_movData_buf(	1,
							sizeof(dp_proxy_rxd_t),
							(char *)rxd_buf,
							(char *)rxd,
							(void *)0);

	dpp_stats.tx++;

	ht_bytes_tx += num_bytes + sizeof(dp_proxy_rxd_t);

	return NFX_OK;

}	/* dp_proxy_tx_flat_buffer() */

void
dp_proxy_cb(char *buf, int len) 
{
		eee_ramDealloc(buf);
} //dp_proxy_cb

#endif	/* USE_DP_PROXY */

#endif	/* NFP_TXRX */

#ifdef NFP_FP
/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	12/10/01
 *
 *-----------------------------------------------------------------
 */
void
dp_proxy_tx_cleanup(
	uchar8	* bufptr
)
{
	(address_t)bufptr &= ~0x1F;

	//printf("dp_proxy_tx_cleanup, bufptr = %p\n", bufptr);

	NFX_SPIN_LOCK(&dpp_rq_lock); 

	*dpp_rq_curr = bufptr;
	if (++dpp_rq_curr == dpp_rq_end)
		dpp_rq_curr = dpp_rq_start;

	dpp_stats.rq_replenish++;

	NFX_SPIN_UNLOCK(&dpp_rq_lock);

}	/* dp_proxy_tx_cleanup() */

#endif	/* NFP_FP */

