/*
 *-----------------------------------------------------------------
 *
 * Name:	dp-proxy.h
 *
 * RCS:  $Id: dp-proxy.h,v 1.8 2002/10/18 18:10:11 rick Exp $
 *
 * Copyright (C) 2001, Agile Storage, Inc.
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	12/10/01
 *
 *-----------------------------------------------------------------
 */

#ifndef _DP_PROXY_H
#define _DP_PROXY_H

#ifdef LARGE_MEMORY
#define VARIABLE_DPP_QUEUE
#endif

#include <exports/mem-api.h>

#ifdef LINUX_TEST
#define DP_PROXY_NUM_BUFS		16
#else
#ifdef VARIABLE_DPP_QUEUE
#define DP_PROXY_NUM_BUFS		(memorysize_MB)	/* amount of FP memory */
#define DP_PROXY_MAX_BUFS		4096
#else
#define DP_PROXY_NUM_BUFS		256
#endif
#endif	

#define DP_PROXY_BUF_SIZE		9216

#ifdef BC1250_USE_PF
#define DP_PROXY_OFFSET			22		// Leave extra room for lmux hdr.
#else
#define DP_PROXY_OFFSET			14
#endif

/*
 * Ring structure in TXRX memory for buffer addresses that are available
 * in FP memory.
 *
 * Written by FP, read by TXRX.
 */
typedef struct dp_proxy_rq {
	void	** start;
	void	** end;
	void	** curr;
} dp_proxy_rq_t;

/*
 * 'Receive descriptor' in FP memory which should be written by data mover
 * after moving data from TXRX to FP.
 *
 * There shall be one rxd entry per proxy buffer, although not a one-to-one
 * static correspondence.
 *
 * Written by TXRX (datamover or CPU), read by FP.
 *
 * This structure should be <= 32 bytes.
 */
typedef struct dp_proxy_rxd {
	uint32	control;				/* ownership */
	uint32	length;					/* in bytes */
	void	* buf;					/* start of data */
#ifndef __long64
	void	* res;
#endif
	uint32	res1[4];
} dp_proxy_rxd_t;

#define DP_PROXY_RXD_FP_OWN		0x80000000

/*
 * Initialization message for exchanging structure pointers
 */
typedef struct dpp_msg {
	uint32	cmd;
	void	* base;
	void	* end;
#ifdef VARIABLE_DPP_QUEUE
	uint32	num_bufs;
#endif
} dp_proxy_init_msg_t;

#define DP_PROXY_CMD_INIT		1

/*
 * Statistics
 */
typedef struct dpp_stat {
	uint32	tx;
	uint32	rx;
	uint32  badbuforsomething;
	uint32	rq_replenish;
	uint32	no_buf;
} dpp_stats_t;

/*------------------------------------------------------------------
 * Variable declarations
 *------------------------------------------------------------------*/
extern void			** dpp_rq_start;
extern void			** dpp_rq_end;
extern void			** dpp_rq_curr;

extern dp_proxy_rxd_t	* dpp_rxdq_start;
extern dp_proxy_rxd_t	* dpp_rxdq_end;
extern dp_proxy_rxd_t	* dpp_rxdq_curr;

#ifdef NFP_FP
extern uchar8		* dpp_buf_base;
extern uchar8		* dpp_buf_end;
#endif

extern dpp_stats_t	dpp_stats;

/*------------------------------------------------------------------
 * Function prototypes
 *------------------------------------------------------------------*/
extern void dp_proxy_init(uint32 pass);
extern void dp_proxy_app_rx(eee_descPtr_t edesc);
extern void dp_proxy_rx(void * cb, uint32 tref);
extern void dp_proxy_tx(eee_descPtr_t edesc);
extern void dp_proxy_tx_cleanup(uchar8 * bufptr);
extern int32 dp_proxy_tx_flat_buffer(
    uchar8  * buf,
    uint32  num_bytes,
    uint32  dp,
    uint32  sp
);
extern int32 dp_proxy_tx_sac_data(
    eee_descPtr_t   edesc,
    uint32          cod_len,
    uint32          cod_offset,
    uchar8          * msg,
    uint32          msglen,
    uint32          dp,
    uint32          sp
);


#endif	/* _DP_PROXY_H */

