/*
 *=============================================================================
 * File Name   :    dp83816-tx.c
 *
 *  Description:    Transmit routines for DP83816 (NS 10/100 Ethernet MAC)
 *                  Ported from sm-dp83820 module.
 *
 *  Created    :    05/24/04
 *
 *  Author     :    Shaun Hui
 *
 *  Copyright(c)  2004, OnStor Inc
 *=============================================================================
 */

#include <nfx-incl.h>
#include <exports/eee-api.h>
#include <exports/mem-api.h>
#include <sys/cache.h>
#include "exports/debug.h"
#include "dp83816.h"

uint32 *dp83816_cr;

uint32 dp83816_tx_pkt_trace = 0;

extern int dp83816_printBuf(char *, int, int);


/*
 *------------------------------------------------------------------------------
 * Function Name:	dp83816_txUnicast
 *
 * Description: 	Transmits a unicast packet to the backplane.
 *
 * Created by:		Frank Salzmann
 *
 * Data Created:	April 30, 2001
 * 					
 * Returns: ?????
 *
 *------------------------------------------------------------------------------
 */
int
dp83816_txUnicast( uint32 dest, eee_desc_t *pktDesc )
{
	int 		i, priQ;
	int 		bufCnt;
	int 		pktLen;
	int 		remotePort;
	int 		port;
	uint32 		cmdsts;
	dp83816_cfg_t 				*dpCfg;
	volatile dp83816_txCtrl_t 	*txc;
	volatile dp83816_desc_t 	*txCur;

	
	remotePort = 1;

	priQ = 0;		/* TODO: Get this value from the descriptor */
	port = EEE_GET_PORT_ID(dest);
	dpCfg = dp83816;

	if (port != PROM_PORT_NET0)
		dpCfg++; 

	txc = &dpCfg->txc[priQ];
	txCur = (volatile dp83816_desc_t *)txc->txCur;

	pktLen = pktDesc->hdr.control & EDESC_TOTAL_LEN;
	bufCnt = (pktDesc->hdr.control & EDESC_BUFCNT_MASK) >> EDESC_BUFCNT_SHIFT;
	
	/* 
	 * Check for room on the transmit queue.
	 */
	if( bufCnt > txc->avail-1 ) {
		printf("tx: no room\n");
		/*
		 * Call transmit cleanup to try and free up some tx descritors.
		 */
		//dp83816_txCleanupQ( txc );
		if( bufCnt > txc->avail-1 ) {
			txc->qFull++;
			CACHE_FLUSH_PACKET(pktDesc);
			return( EEE_DROP_PACKET );
		}
	}

	if( dp83816_tx_pkt_trace ) {
		printf("bufCnt: %d, txCur: %lx\n", bufCnt, (address_t)txCur);
	}

	i = 1;
	if( bufCnt == 1 ) {
		uint32 pktLen;
#if DP83816_TX_SAFEMODE
		/* Make sure driver owns descriptor. */
		if( txCur->cmdsts & DP83816_CMDSTS_OWN ) {
			printf("tx: q full\n");
			dpCfg->txc[priQ].qFull++;
			CACHE_FLUSH_PACKET(pktDesc);
			return( EEE_DROP_PACKET );
		}
#endif /* DP83816_TX_SAFEMODE */
		txCur->cmdsts  = 0;
		txCur->pktDesc = pktDesc;

		if( dp83816_tx_pkt_trace ) {
			CACHE_FLUSH_PACKET(pktDesc);
			printf("pktDesc: %lx, tx buf cnt: %d, buf ptr: %lx, buf len: %d\n", 
			   	(address_t)pktDesc, bufCnt, (address_t)pktDesc->bd[1].buf, 
				pktDesc->bd[1].len);

			dp83816_printBuf((char *)pktDesc->bd[1].buf, pktDesc->bd[1].len, 1); 

		}

		txCur->bufptr = (uchar8 *)virt_to_phys((address_t)pktDesc->bd[1].buf);
		pktLen = pktDesc->bd[1].len;

		cmdsts = (DP83816_CMDSTS_OWN   |
				  DP83816_CMDSTS_INTR  |
				  (DP83816_CMDSTS_LEN & pktLen));

		//-- FSDEBUG CACHE_FLUSH_PACKET(pktDesc);
		CACHE_FLUSH(pktDesc->bd[1].buf, pktDesc->bd[1].buf + 256);    	

		txCur->cmdsts  = cmdsts;

		txc->avail--;
		txc->qd++;
		if( ++txCur >= txc->txqEnd )
			txCur = txc->txqStart;
	}
	else {
		if( pktDesc->hdr.control & EDESC_COMMAND_USE_FIRST_BUFFER )
		    i=0;
    	else
		    i=1;  

		while( bufCnt-- ) {
#if DP83816_TX_SAFEMODE
			/* Make sure driver owns descriptor. */
			if( txCur->cmdsts & DP83816_CMDSTS_OWN ) {
				printf("tx: q full\n");
				txc->qFull++;
				return( EEE_DROP_PACKET );
			}
#endif /* DP83816_TX_SAFEMODE */

			txCur->cmdsts  = 0;

			if( dp83816_tx_pkt_trace ) {
				CACHE_FLUSH_PACKET(pktDesc);
				printf("pktDesc: %lx, tx buf cnt: %d, buf ptr: %lx, buf len: %d\n", 
					   (address_t)pktDesc, bufCnt, (address_t)pktDesc->bd[i].buf, pktDesc->bd[i].len);
				if(bufCnt==
						(pktDesc->hdr.control&EDESC_BUFCNT_MASK)>>EDESC_BUFCNT_SHIFT)
					dp83816_printBuf(
							(char *)((uint32)pktDesc->bd[i].buf ),
							pktDesc->bd[i].len, 1);
				else
					dp83816_printBuf( 
							(char *)((uint32)pktDesc->bd[i].buf),
							pktDesc->bd[i].len, 0);
			}

			txCur->bufptr = (uchar8 *)virt_to_phys((address_t)pktDesc->bd[i].buf);

			if( bufCnt ) {
			    cmdsts = DP83816_CMDSTS_OWN	| DP83816_CMDSTS_MORE	|
					    (DP83816_CMDSTS_LEN & pktDesc->bd[i].len);	
			}
			else {
				txCur->pktDesc = pktDesc;
				cmdsts = DP83816_CMDSTS_OWN	| 
					(DP83816_CMDSTS_LEN & pktDesc->bd[i].len);	
				CACHE_FLUSH_PACKET(pktDesc);
			}

			//-- Pass ownership to the DP83816
			txCur->cmdsts = cmdsts;

			i++;
			txc->avail--;
			txc->qd++;

			if( ++txCur >= txc->txqEnd )
				txCur = txc->txqStart;
		}
	}
	// Strobe the tx enable bit.
	dp83816_writeReg(&dpCfg->regs->cr, 0x00000001);
	txc->txCur  = (dp83816_desc_t *)txCur;
	txc->pkts++;
	txc->octets += pktLen;

	return(EEE_FORWARDED_PACKET);
} /* dp83816_txUnicast */

/*
 *------------------------------------------------------------------------------
 * Function Name:	dp83816_txCleanupQ
 *
 * Description: 	Cleans up descriptors on a transmit queue.
 *
 * Created by:		Frank Salzmann
 *
 * Data Created:	April 30, 2001
 * 					
 * Returns: ?????
 *
 *------------------------------------------------------------------------------
 */
int
dp83816_txCleanupQ( dp83816_txCtrl_t *txc )
{
	uint32			        cmdsts;
	uint32				    bufCnt;
	uint32					f_noPkt = 1;
	volatile dp83816_desc_t *ccur;
	volatile dp83816_desc_t *tcur;
	eee_desc_t *edesc = NULL;
	uint32 retransmit = 0;

	ccur = txc->txCln;
	tcur = txc->txCur;


	cmdsts = ccur->cmdsts;
	while( ccur != tcur && ((cmdsts & DP83816_CMDSTS_OWN) == 0) ) {

		f_noPkt = 0;

		/*
		 * Look for the last descriptor in a chain. The cmdsts word
		 * in this descriptor contains the transmit completion
		 * status for the packet.
		 */
		if( (cmdsts & DP83816_CMDSTS_MORE) == 0 ) {
			// printf("txCleanupQ: cmdsts = 0x%08x\n", cmdsts);
			edesc  = ccur->pktDesc;
			bufCnt =  EEE_GET_NUM_BUFS( edesc );

			if (!(cmdsts & DP83816_CMDSTS_OK)) {
				if( cmdsts & DP83816_CMDSTS_TXA ) {
					txc->txaErr++;
					retransmit = 1;
				}
				if( cmdsts & DP83816_CMDSTS_TFU )
					txc->tfuErr++;
				if( cmdsts & DP83816_CMDSTS_CRS )
					txc->crsErr++;
				if( cmdsts & DP83816_CMDSTS_TD )
					txc->tdErr++;
				if( cmdsts & DP83816_CMDSTS_ED )
					txc->edErr++;
				if( cmdsts & DP83816_CMDSTS_OWC )
					txc->owcErr++;
				if( cmdsts & DP83816_CMDSTS_EC )
					txc->ecErr++;
			}

			if (retransmit == 0)
				eee_freePacket(edesc);     

			ccur->pktDesc = 0;		

			txc->avail+=bufCnt;
			txc->unqd+=bufCnt;
		}

		// printf("TxClean: Ptr: %x\n", ccur);		 MYDEBUG
		ccur->cmdsts = 0;
		ccur->bufptr = 0;			

		if( ++ccur >= txc->txqEnd )
			ccur = txc->txqStart;

		if (retransmit) // resume with clean-up next time
			break;

		cmdsts = ccur->cmdsts;
	}

	txc->txCln = (dp83816_desc_t *)ccur;

	if (retransmit) {
		edesc->hdr.control |= EDESC_COMMAND_REUSE_HEADER;
		dp83816_txUnicast( edesc->hdr.dest_port, edesc );
	}

	return(NFX_OK);
} /* dp83816_txCleanupQ */

/*
 *------------------------------------------------------------------------------
 * Function Name:	dp83816_txCleanup
 *
 * Description: 	Polling function registered with the EEE layer and 
 * 					called to cleanup the transmit queues.
 *
 * Created by:		Frank Salzmann
 *
 * Data Created:	April 30, 2001
 * 					
 * Returns: ?????
 *
 *------------------------------------------------------------------------------
 */
void
dp83816_txCleanPoll(void *cb, uint32 tlimit)
{
	int 			i;
	int 			port;
	dp83816_cfg_t	*dpCfg;
	
	for(i=0; i < DP83816_TX_Q_CNT; i++)
		for (port=0, dpCfg=dp83816; port < DP83816_NUM_PORT; port++, dpCfg++)
			dp83816_txCleanupQ( &dpCfg->txc[i] );
	return;
} /* dp83816_txCleanup */

