/*
 *=============================================================================
 * File Name   :    dp83816-ui.c
 *
 *  Description:    Debug UI routines for DP83816 (NS 10/100 Ethernet MAC)
 *                  Ported from sm-dp83820 module.
 *
 *  Created    :    05/24/04
 *
 *  Author     :    Shaun Hui
 *
 *  Copyright(c)  2004, OnStor Inc
 *=============================================================================
 */
#include "dp83816.h"

extern int dp_rxHWM;

extern int dp83816_printBuf(char *, int, int);
extern int dp83816_diagSetLbMode(dp83816_cfg_t *, uint32);
extern int dp83816_readReg(volatile uint32 *);
extern int dp83816_diagTxPkt(dp83816_cfg_t *);
extern int dp83816_diagTxEna(dp83816_cfg_t *);
extern int dp83816_diagRxEna(dp83816_cfg_t *);
extern int dp83816_diagLBTest(dp83816_cfg_t *);
extern int dp83816_diagViewTestStats(dp83816_cfg_t *);
extern int get_cmd(char8 *);

int dp83816_uiClearStats(dp83816_cfg_t *);
int dp83816_uiSetPktTrace(dp83816_cfg_t *);
int dp83816_uiShowRxq(dp83816_cfg_t *);
int dp83816_uiShowTxq(dp83816_cfg_t *);
int dp83816_uiConfigLBMode(dp83816_cfg_t *);
int dp83816_uiShowOpRegs(dp83816_cfg_t *);
int dp83816_uiViewCfg(dp83816_cfg_t *);
int dp83816_uiViewStats(dp83816_cfg_t *);
int dp83816_uiShowPciRegs(dp83816_cfg_t *);
int dp83816_uiDiagMenu(int port);
int dp83816_uiGetInput( char *msg );

/*
 *------------------------------------------------------------------------------
 * Function Name:	dp83816_uiDiag
 *
 * Description: 	DP83816 UI entry function.
 *
 * Created by:		Frank Salzmann
 *
 * Data Created:	May 08, 2001
 * 					
 * Returns: NFX_OK
 *
 *------------------------------------------------------------------------------
 */
int
dp83816_uiDiag()
{
	int	port;

	port = dp83816_uiGetInput("Enter port number [0 | 1]: \n");
	if((port < 0) || (port > 1)) {
		printf("Invalid port: %d. Exiting....\n", port);
		return(NFX_OK);
	}
	
	dp83816_uiDiagMenu( port );
	return(NFX_OK);
} /* dp83816_uiDiag */

/*
 *------------------------------------------------------------------------------
 * Function Name:	dp83816_uiShowDiagMenu
 *
 * Description: 	Prints DP83816 diagnostic menu items.
 *
 * Created by:		Frank Salzmann
 *
 * Data Created:	May 09, 2001
 * 					
 * Returns: NFX_OK
 *
 *------------------------------------------------------------------------------
 */
int
dp83816_uiShowDiagMenu(int port)
{
	int sel;
	char cmd[80];

	sel = 0;
	printf("\n============= Port %d DP83816 MENU =============\n", port);
	printf("1  : Show Driver Cfg        ");
	printf("2  : Show Driver Stats\n");
	printf("3  : Show Op Regs           ");
	printf("4  : Show PCI Regs\n");
	printf("5  : Set/Clear LB Mode      ");
	printf("6  : Show RX Desc Q\n");
	printf("7  : Show TX Desc Q         ");
	printf("8  : Transmit Packet\n");
	printf("9  : Transmit Enable        ");
	printf("10 : Receive Enable\n");
	printf("11 : Run LB Test            ");
	printf("12 : View Test Results\n");
	printf("13 : Ena/Dis Pkt Trace      ");
	printf("14 : Clear Stats\n");
	printf("Enter Selection [99:quit]:  ");

	memset(cmd, 0, 80);
	get_cmd(cmd);
	sscanf(cmd, "%d", &sel);
	printf("\n");
	return sel;
} /* dp83816_uiShowDiagMenu */

/*
 *------------------------------------------------------------------------------
 * Function Name:	dp83816_uiGetInput	
 *
 * Description: 	Gets console input.
 *
 * Created by:		Frank Salzmann
 *
 * Data Created:	May 09, 2001
 * 					
 * Returns: NFX_OK
 *
 *------------------------------------------------------------------------------
 */
int
dp83816_uiGetInput( char *msg )
{
	int sel;
	char cmd[80];

	printf("%s: ", msg);
	memset(cmd, 0, 80);
	get_cmd(cmd);
	sscanf(cmd, "%d", &sel);
	return sel; 
} /* dp83816_uiGetInput */
/*
 *------------------------------------------------------------------------------
 * Function Name:	dp83816_uiDiagMenu	
 *
 * Description: 	Runs the DP83816 diagnostic menu.
 *
 * Created by:		Frank Salzmann
 *
 * Data Created:	May 09, 2001
 * 					
 * Returns: NFX_OK
 *
 *------------------------------------------------------------------------------
 */
int
dp83816_uiDiagMenu(int port)
{
	int sel ;
	dp83816_cfg_t *dp;

	dp = dp83816+port;
	for(;;) {
		sel = dp83816_uiShowDiagMenu( port );
		switch(sel)
		{
			case 1:
				dp83816_uiViewCfg(dp);
				break;
			case 2:
				dp83816_uiViewStats(dp);
				break;
			case 3:
				dp83816_uiShowOpRegs(dp);
				break;
			case 4:
				dp83816_uiShowPciRegs(dp);
				break;
			case 5:
				dp83816_uiConfigLBMode(dp);
				break;
			case 6:
				dp83816_uiShowRxq(dp);
				break;
			case 7:
				dp83816_uiShowTxq(dp);
				break;
			case 8:
				dp83816_diagTxPkt(dp);
				break;
			case 9:
				dp83816_diagTxEna(dp);
				break;
			case 10:
				dp83816_diagRxEna(dp);
				break;
			case 11:
				dp83816_diagLBTest(dp);
				break;
			case 12:
				dp83816_diagViewTestStats(dp);
				break;
			case 13:
				dp83816_uiSetPktTrace(dp);
				break;
			case 14:
				dp83816_uiClearStats(dp);
				break;
			case 99:
			case 999:
				return 0;
			default:
				break;
		}
	} 
	return(NFX_OK);
} /* dp83816_uiDiagMenu */

/*
 *------------------------------------------------------------------------------
 * Function Name:	dp83816_uiShowAllRegs
 *
 * Description: 	Prints all of the chip registers.
 *
 * Created by:		Frank Salzmann
 *
 * Data Created:	May 08, 2001
 * 					
 * Returns: NFX_OK
 *
 *------------------------------------------------------------------------------
 */
int
dp83816_uiShowOpRegs( dp83816_cfg_t *dp )
{
	dp83816_regs_t *r;

	r = dp->regs;
	printf("DP83816 OPERATIONAL REGISTER DUMP: [Slot:%02d -- CPU:%02d]\n", 
		   nfxMySlotId,
		   nfxMyCPUId);
	printf("========================================================== \n\n");
	printf("CMD                     :%08x\n", dp83816_readReg(&r->cr));
	printf("CFG                     :%08x\n", dp83816_readReg(&r->cfg));
	printf("EEProm Access           :%08x\n", dp83816_readReg(&r->mear));
	printf("PCI Test Control        :%08x\n", dp83816_readReg(&r->ptscr));
	printf("Interrupt Status        :%08x\n", dp83816_readReg(&r->isr));
	printf("Interrupt Mask          :%08x\n", dp83816_readReg(&r->imr));
	printf("Interrupt Enable        :%08x\n", dp83816_readReg(&r->ier));
	printf("Interrupt Holdoff       :%08x\n", dp83816_readReg(&r->ihr));
	printf("Tx Descriptor Ptr       :%08x\n", dp83816_readReg(&r->txdp));
	printf("Tx Config               :%08x\n", dp83816_readReg(&r->txcfg));
	printf("Rx Descriptor Ptr       :%08x\n", dp83816_readReg(&r->rxdp));
	printf("Rx Config               :%08x\n", dp83816_readReg(&r->rxcfg));
	printf("Wake Ctrl/Status        :%08x\n", dp83816_readReg(&r->wcsr));
	printf("Pause Ctrl/Status       :%08x\n", dp83816_readReg(&r->pcr));
	printf("Rx Filter/Match Ctrl    :%08x\n", dp83816_readReg(&r->rfcr));
	printf("Rx Filter/Match Data    :%08x\n", dp83816_readReg(&r->rfdr));
	printf("Boot ROM Address        :%08x\n", dp83816_readReg(&r->brar));
	printf("Boot ROM Data           :%08x\n", dp83816_readReg(&r->brdr));
	printf("Silicon Revision        :%08x\n", dp83816_readReg(&r->srr));
	printf("MIB Ctrl                :%08x\n", dp83816_readReg(&r->mibc));
	printf("MIB0 RXErroredPkts      :%08x\n", dp83816_readReg(&r->mib[0]));
	printf("MIB1 RXFCSErrors        :%08x\n", dp83816_readReg(&r->mib[1]));
	printf("MIB2 RxFifoOverrun      :%08x\n", dp83816_readReg(&r->mib[2]));
	printf("MIB3 RXFAErrors         :%08x\n", dp83816_readReg(&r->mib[3]));
	printf("MIB4 RXSymbolErrors     :%08x\n", dp83816_readReg(&r->mib[4]));
	printf("MIB5 RXBig              :%08x\n", dp83816_readReg(&r->mib[5]));
	printf("MIB6 RXIRLErrors        :%08x\n", dp83816_readReg(&r->mib[6]));
	printf("MIB7 RXBadOpcodes       :%08x\n", dp83816_readReg(&r->mib[7]));
	printf("MIB8 RxPauseFrames      :%08x\n", dp83816_readReg(&r->mib[8]));
	printf("MIB9 TxPauseFrames      :%08x\n", dp83816_readReg(&r->mib[9]));
	printf("MIB10 TXSQEErrors       :%08x\n", dp83816_readReg(&r->mib[10]));
	printf("Clock Ctrl/Status       :%08x\n", dp83816_readReg(&r->ccsr));

	printf("PHY: BMCR               :%08x\n", dp83816_readReg(&r->bmcr));
	printf("PHY: BMSR               :%08x\n", dp83816_readReg(&r->bmsr));
	printf("PHY: ID 1               :%08x\n", dp83816_readReg(&r->phyidr1));
	printf("PHY: ID 2               :%08x\n", dp83816_readReg(&r->phyidr2));
	printf("PHY: ANAR               :%08x\n", dp83816_readReg(&r->anar));
	printf("PHY: ANLPAR             :%08x\n", dp83816_readReg(&r->anlpar));
	printf("PHY: ANER               :%08x\n", dp83816_readReg(&r->aner));
	printf("PHY: ANNPTR             :%08x\n", dp83816_readReg(&r->annptr));
	printf("PHY: STATUS             :%08x\n", dp83816_readReg(&r->physts));
	printf("PHY: MICR               :%08x\n", dp83816_readReg(&r->micr));
	printf("PHY: MISR               :%08x\n", dp83816_readReg(&r->misr));
	printf("PHY: FCSR               :%08x\n", dp83816_readReg(&r->fcsr));
	printf("PHY: RECR               :%08x\n", dp83816_readReg(&r->recr));
	printf("PHY: PCSR               :%08x\n", dp83816_readReg(&r->pcsr));
	printf("PHY: CR                 :%08x\n", dp83816_readReg(&r->phycr));
	printf("\n");
	return(NFX_OK);
} /* dp83816_uiShowAllRegs */

/*
 *------------------------------------------------------------------------------
 * Function Name:	dp83816_uiShowAllRegs
 *
 * Description: 	Prints all of the chip registers.
 *
 * Created by:		Frank Salzmann
 *
 * Data Created:	May 08, 2001
 * 					
 * Returns: NFX_OK
 *
 *------------------------------------------------------------------------------
 */
int
dp83816_uiShowPciRegs( dp83816_cfg_t *dp )
{
#ifdef XXXX
	dp83816_regs_t *r;

	r = dp->regs;
	printf("DP83816 PCI REGISTER DUMP: [Slot:%02d -- CPU:%02d]\n", 
		   nfxMySlotId,
		   nfxMyCPUId);
	printf("================================================== \n\n");
	printf("Config Id                   :%08x\n", 
					dp83816_readReg(&r->pciCfgRegs[0]));
	printf("Config Cmd and Status       :%08x\n", 
			dp83816_readReg(&r->pciCfgRegs[1]));
	printf("Config Revision Id          :%08x\n", 
			dp83816_readReg(&r->pciCfgRegs[2]));
	printf("Config Latency Timer        :%08x\n", 
			dp83816_readReg(&r->pciCfgRegs[3]));
	printf("Config IO Base Address      :%08x\n", 
			dp83816_readReg(&r->pciCfgRegs[4]));
	printf("Config Memory Address Lo    :%08x\n", 
			dp83816_readReg(&r->pciCfgRegs[5]));
	printf("Config Memory Address Hi    :%08x\n", 
			dp83816_readReg(&r->pciCfgRegs[6]));
	printf("Config Subsystem Id         :%08x\n", 
			dp83816_readReg(&r->pciCfgRegs[11]));
	printf("Boot ROM Config             :%08x\n", 
			dp83816_readReg(&r->pciCfgRegs[12]));
	printf("Capabilities Pointer        :%08x\n", 
			dp83816_readReg(&r->pciCfgRegs[13]));
	printf("Config Interrupt Select     :%08x\n", 
			dp83816_readReg(&r->pciCfgRegs[15]));
	printf("Power Mgmt Capabilities     :%08x\n", 
			dp83816_readReg(&r->pciCfgRegs[16]));
	printf("Power Mgmt Ctrl/Status      :%08x\n", 
			dp83816_readReg(&r->pciCfgRegs[17]));
	printf("\n");
#endif
	return(NFX_OK);
} /* dp83816_uiShowPciRegs */

/*
 *------------------------------------------------------------------------------
 * Function Name:	dp83816_uiViewStats	
 *
 * Description: 	Display device/driver statistics.
 *
 * Created by:		Frank Salzmann
 *
 * Data Created:	May 09, 2001
 * 					
 * Returns: NFX_OK
 *
 *------------------------------------------------------------------------------
 */
int
dp83816_uiViewStats(dp83816_cfg_t *dp)
{
	uint32 i;
	dp83816_regs_t *r;

	r = dp->regs;

	printf("DP83816 STATISTICS: [Slot:%02d -- CPU:%02d]\n", 
		   nfxMySlotId,
		   nfxMyCPUId);
	printf("================== Receive Q Statistics ================= \n\n");
	for(i=0; i < DP83816_RX_Q_CNT; i++) {
		printf("RxQ %d avail       : %8u       ", i, dp->rxc[i].avail);
		printf("RxQ %d pkts        : %8u\n", i, dp->rxc[i].pkts);
		printf("RxQ %d octets      : %8u       ", i, dp->rxc[i].octets);
		printf("RxQ %d rxo err     : %8u\n", i, dp->rxc[i].rxoErr);
		printf("RxQ %d rxa err     : %8u       ", i, dp->rxc[i].rxaErr);
		printf("RxQ %d no buf err  : %8u\n", i, dp->rxc[i].noBufErr);
		printf("RxQ %d no desc err : %8u       ", i, dp->rxc[i].noDescErr);
		printf("RxQ %d too long err: %8u\n", i, dp->rxc[i].tooLongErr);
		printf("RxQ %d runt err    : %8u       ", i, dp->rxc[i].runtErr);
		printf("RxQ %d is err      : %8u\n", i, dp->rxc[i].isErr);
		printf("RxQ %d crc err     : %8u       ", i, dp->rxc[i].crcErr);
		printf("RxQ %d fa err      : %8u\n", i, dp->rxc[i].faErr);
		printf("RxQ %d lbp err     : %8u       ", i, dp->rxc[i].lbpErr);
		printf("RxQ %d col err     : %8u\n", i, dp->rxc[i].colErr);
	}
	printf("HWM Rx desc Qd    : %8u\n", dp_rxHWM);
	printf("\n");
	printf("================== Transmit Q Statistics ================= \n\n");
	for(i=0; i < DP83816_TX_Q_CNT; i++) {
		printf("TxQ %d avail       : %8u       ", i, dp->txc[i].avail);
		printf("TxQ %d pkts        : %8u\n", i, dp->txc[i].pkts);
		printf("TxQ %d octets      : %8u       ", i, dp->txc[i].octets);
		printf("TxQ %d qFull err   : %8u\n", i, dp->txc[i].qFull);
		printf("TxQ %d txa err     : %8u       ", i, dp->txc[i].txaErr);
		printf("TxQ %d tfu err     : %8u\n", i, dp->txc[i].tfuErr);
		printf("TxQ %d crs err     : %8u       ", i, dp->txc[i].crsErr);
		printf("TxQ %d td err      : %8u\n", i, dp->txc[i].tdErr);
		printf("TxQ %d ed err      : %8u       ", i, dp->txc[i].edErr);
		printf("TxQ %d owc err     : %8u\n", i, dp->txc[i].owcErr);
		printf("TxQ %d ec err      : %8u       ", i, dp->txc[i].ecErr);
		printf("TxQ %d bufs Qd     : %8u\n", i, dp->txc[i].qd);
		printf("TxQ %d bufs un Qd  : %8u\n", i, dp->txc[i].unqd);
	}
	printf("\n");
	printf("================== DP83816 MIB Counters ================= \n\n");
	printf("MIB0 RXErroredPkts: %8u       ", dp83816_readReg(&r->mib[0]));
	printf("MIB1 RXFCSErrors  : %8u\n", dp83816_readReg(&r->mib[1]));
	printf("MIB2 RxFifoOverrun: %8u       ", dp83816_readReg(&r->mib[2]));
	printf("MIB3 RXFAErrors   : %8u\n", dp83816_readReg(&r->mib[3]));
	printf("MIB4 RXSymbolErrs : %8u       ", dp83816_readReg(&r->mib[4]));
	printf("MIB5 RXBig        : %8u\n", dp83816_readReg(&r->mib[5]));
	printf("MIB6 RXIRLErrors  : %8u       ", dp83816_readReg(&r->mib[6]));
	printf("MIB7 RXBadOpcodes : %8u\n", dp83816_readReg(&r->mib[7]));
	printf("MIB8 RxPauseFrames: %8u       ", dp83816_readReg(&r->mib[8]));
	printf("MIB9 TxPauseFrames: %8u\n", dp83816_readReg(&r->mib[9]));
	printf("MIB10 TXSQEErrors : %8u\n", dp83816_readReg(&r->mib[10]));
	printf("\n");
	return(NFX_OK);
} /* dp83816_uiViewStats */

char *dp_link_speed_str[] = {"10Mbps-","100Mbps-","unknown"};
char *dp_link_duplex_str[] = {"hdx", "fdx", ""};
char *dp_link_lb_str[] = {"none", "internal", "phy", "external"};

/*
 *------------------------------------------------------------------------------
 * Function Name:	dp83816_uiViewCfg
 *
 * Description: 	Display device/driver configuration.
 *
 * Created by:		Frank Salzmann
 *
 * Data Created:	June 20, 2001
 * 					
 * Returns: NFX_OK
 *
 *------------------------------------------------------------------------------
 */
int
dp83816_uiViewCfg(dp83816_cfg_t *dp)
{
	uint32 i;

	dp83816_cfgr_t cfg;

	cfg.wd = dp83816_readReg(&dp->regs->cfg);
	printf("DP83816 CONFIGURATION: [Slot:%02d -- CPU:%02d]\n", 
		   nfxMySlotId,
		   nfxMyCPUId);
	printf("=============== General Config ============== \n\n");
	printf("Application Id             : %08d\n", dp->appId);
	printf("Base Addr                  : %x\n", dp->baseAddr);
	printf("Phy Addr                   : %08x\n", dp->phyAddr);
	printf("\n");
	printf("=============== Link Control ================ \n\n");
	printf("Link State:                : %s\n", cfg.bits.lnksts ? "up":"down");
	printf("Link Speed:                : %s%s\n", 
							dp_link_speed_str[cfg.bits.speed100],
							dp_link_duplex_str[cfg.bits.fdup]);
	printf("Link Loopback Mode         : %s\n", 
							dp_link_lb_str[dp->link.loopback]);
	printf("\n");
	printf("=============== Transmit Control ============== \n\n");
	for(i=0; i < DP83816_TX_Q_CNT; i++) {
		printf("TxQ %d Start                : %p\n", i, dp->txc[i].txqStart);
		printf("TxQ %d End                  : %p\n", i, dp->txc[i].txqEnd);
		printf("TxQ %d Current Addr         : %p\n", i, dp->txc[i].txCur);
		printf("TxQ %d Cleanup Addr         : %p\n", i, dp->txc[i].txCln);
	}
	printf("\n");
	printf("=============== Receive Control ============== \n\n");
	for(i=0; i < DP83816_RX_Q_CNT; i++) {
		printf("Rx Desc Pool %d Base Addr   : %p\n", i, dp->rxQ[i]);
		printf("RxQ %d Start                : %p\n", i, dp->rxc[i].rxqStart);
		printf("RxQ %d End                  : %p\n", i, dp->rxc[i].rxqEnd);
		printf("RxQ %d Current Addr         : %p\n", i, dp->rxc[i].rxCur);
		printf("RxQ %d Buffer Count         : %08d\n", i, dp->rxc[i].bufCnt);
	}
	printf("\n");
	return(NFX_OK);
} /* dp83816_uiViewCfg */

/*
 *------------------------------------------------------------------------------
 * Function Name:	dp83816_uiSetLoopbackMode
 *
 * Description: 	Allows user to enable/disable device loopback.
 *
 * Created by:		Frank Salzmann
 *
 * Data Created:	May 08, 2001
 * 					
 * Returns: NFX_OK
 *
 *------------------------------------------------------------------------------
 */
int
dp83816_uiConfigLBMode( dp83816_cfg_t *dp )
{
	int32 mode;

	mode = dp83816_uiGetInput("Set Loopback Mode [ON:1 | OFF:2]: \n");
	if((mode != 1) && (mode != 2)) {
        printf("Invalid Mode: %d. Exiting....\n", mode);
        return(NFX_OK);
    }    
	if( mode == 1 )
		dp83816_diagSetLbMode(dp83816, DP83816_LB_ENABLE); 
	else
		dp83816_diagSetLbMode(dp83816, DP83816_LB_DISABLE); 

	return(NFX_OK);
} /* dp83816_uiConfigLBMode */

/*
 *------------------------------------------------------------------------------
 * Function Name:	dp83816_uiShowRxq
 *
 * Description: 	Dump the chip receive descriptor queue.
 *
 * Created by:		Frank Salzmann
 *
 * Data Created:	June 22, 2001
 * 					
 * Returns: NFX_OK
 *
 *------------------------------------------------------------------------------
 */
int
dp83816_uiShowRxq( dp83816_cfg_t *dp)
{
	int i, quit;
	volatile dp83816_desc_t *descp;

	descp = dp->rxc[0].rxqStart;

	printf("DP83816 Receive Descriptor Queue Dump\n");
	printf("=====================================\n");
	for(i=0; i < DP83816_RX_Q_LEN; i++) {
		printf("%03d   Link: %p   Buf: %p   Cmdsts: %08x   Edesc: %p\n",
			      i, descp->link, descp->bufptr, descp->cmdsts, descp->pktDesc);
		descp++;
		if( !(i % 25) && i != 0 ) {
			quit = dp83816_uiGetInput("Quit [99]: ");
			if( quit == 99 )
				return 0;
		}
	}
	printf("\n");
	return 0;
} /* dp83816_uiShowRxq */

/*
 *------------------------------------------------------------------------------
 * Function Name:	dp83816_uiShowTxq
 *
 * Description: 	Dump the chip transmit descriptor queue.
 *
 * Created by:		Frank Salzmann
 *
 * Data Created:	June 22, 2001
 * 					
 * Returns: NFX_OK
 *
 *------------------------------------------------------------------------------
 */
int
dp83816_uiShowTxq( dp83816_cfg_t *dp)
{
	int i, quit;
	volatile dp83816_desc_t *descp;

	descp = dp->txc[0].txqStart;

	printf("DP83816 Transmit Descriptor Queue Dump\n");
	printf("======================================\n");
	for(i=0; i < DP83816_TX_Q_LEN; i++) {
		printf("%03d   Link: %p   Buf: %p   Cmdsts: %08x   Edesc: %p\n",
			      i, descp->link, descp->bufptr, descp->cmdsts, descp->pktDesc);
		descp++;
		if( !(i % 25) && i != 0 ) {
			quit = dp83816_uiGetInput("Quit [99]: ");
			if( quit == 99 )
				return 0;
		}
	}
	printf("\n");
	return 0;
} /* dp83816_uiShowTxq */

/*
 *-----------------------------------------------------------------------------
 * Function Name:   dp83816_uiSetPktTrace
 *
 * Description:     Enables/Disables packet tracing in dp83816 driver.
 *
 * Created by:      Frank Salzmann
 *
 * Data Created:    August 08, 2001
 *
 * Returns: NFX_OK
 *
 *-----------------------------------------------------------------------------
 */
int
dp83816_uiSetPktTrace(dp83816_cfg_t *dp)
{
	uint32        sel;
	extern uint32 dp83816_rx_pkt_trace;
	extern uint32 dp83816_tx_pkt_trace;

	printf("Rx Pkt Trace: %s, Tx Pkt Trace: %s\n",
					(dp83816_rx_pkt_trace == 0) ? "DISABLED" : "ENABLED",
					(dp83816_tx_pkt_trace == 0) ? "DISABLED" : "ENABLED");
    sel = dp83816_uiGetInput("Rx Trace Dis/Ena [0|1], Tx Trace Dis/Ena [2|3]: ");
	switch(sel) {
	
		case 0:
			dp83816_rx_pkt_trace = 0;
			break;
		case 1:
			dp83816_rx_pkt_trace = 1;
			break;
		case 2:
			dp83816_tx_pkt_trace = 0;
			break;
		case 3:
			dp83816_tx_pkt_trace = 1;
			break;
		default: 
			printf("Invalid Entry: %d\n", sel);
	}
	printf("Rx Pkt Trace: %s, Tx Pkt Trace: %s\n",
					(dp83816_rx_pkt_trace == 0) ? "DISABLED" : "ENABLED",
					(dp83816_tx_pkt_trace == 0) ? "DISABLED" : "ENABLED");
    printf("\n");
    return 0;
} /* dp83816_uiSetPktTrace */

int
dp83816_uiClearStats(dp83816_cfg_t *dp)
{
	int i;
	dp83816_regs_t *r;

	//-- Clear chip MIB register counters.
	r = dp->regs;
	dp83816_writeReg(&r->mibc, 0x4);

	//-- Clear device driver software counters.
	for(i=0; i < DP83816_RX_Q_CNT; i++) {
		dp->rxc[i].pkts	    = 0;
		dp->rxc[i].octets	= 0;
		dp->rxc[i].rxoErr	= 0;
		dp->rxc[i].rxaErr	= 0;
		dp->rxc[i].noBufErr	= 0;
		dp->rxc[i].noDescErr= 0;
		dp->rxc[i].tooLongErr= 0;
		dp->rxc[i].runtErr	= 0;
		dp->rxc[i].isErr	= 0;
		dp->rxc[i].crcErr	= 0;
		dp->rxc[i].faErr	= 0;
		dp->rxc[i].lbpErr	= 0;
		dp->rxc[i].colErr	= 0;
	}
	for(i=0; i < DP83816_TX_Q_CNT; i++) {
		dp->txc[i].pkts	    = 0;
		dp->txc[i].octets	= 0;
		dp->txc[i].qFull	= 0;
		dp->txc[i].txaErr	= 0;
		dp->txc[i].tfuErr	= 0;
		dp->txc[i].crsErr	= 0;
		dp->txc[i].tdErr	= 0;
		dp->txc[i].edErr	= 0;
		dp->txc[i].owcErr	= 0;
		dp->txc[i].ecErr	= 0;
		dp->txc[i].qd	    = 0;
		dp->txc[i].unqd	    = 0;
	}
	dp_rxHWM = 0;
	return 0;
} /* dp83816_uiClearStats */

