/*
 *=============================================================================
 * File Name   :    dp83816.h
 *
 *  Description:    Control data structures for DP83816 (NS 10/100 Ethernet MAC)
 *                  Ported from sm-dp83820 module.
 *
 *  Created    :    05/24/04
 *
 *  Author     :    Shaun Hui
 *
 *  Copyright(c)  2004, OnStor Inc
 *=============================================================================
 */
#define NFX_DROP_PACKET			-1

#include <nfx-incl.h>
#include <exports/eee-api.h>
#include <exports/cm-api.h>
#include <exports/timer-api.h>
#include "dp83816-regs.h"
#include <exports/hwMap.h>

#ifndef DP83816_H_INCLUDED
#define DP83816_H_INCLUDED

#define DP83816_BASE_ADDR		DP83816_0_BASE
#define DP83816_PORT_OFFSET		(DP83816_1_BASE - DP83816_0_BASE)
#define DP83816_NUM_PORT		2
#define DP83816_BUSID			PCI1_SEC_BUS
#define DP83816_DEVID			PCI1_DP0_DEV

#define DP83816_TX_SAFEMODE		1
#define DP83816_RX_SAFEMODE		1



#define DP83816_TX_Q_CNT	  	1
#define DP83816_TX_Q_LEN		512

#define DP83816_RX_Q_CNT	  	1
#define DP83816_RX_Q_LEN		512




#define DP83816_BUFFER_SIZE		EEE_BUF_LARGE
#define DP83816_RX_BUF_SIZE		2048

#define DP83816_LB_ENABLE		1
#define DP83816_LB_DISABLE		2

#define DP83816_LINK_TIMER		3000		// 3 second link poll timer.

typedef unsigned long long dword;

/*
 *----------------------------
 * Descriptor Format
 *----------------------------
 */
typedef struct _dp83816_desc {
	struct _dp83816_desc *link;
	uint32 				 cmdsts;
	uchar8 				 *bufptr;
	eee_desc_t 			 *pktDesc;
} dp83816_desc_t;


/*
 *--------------------------------
 * cmdsts Common Bit definitions 
 *--------------------------------
 */
#define DP83816_CMDSTS_OWN	0x80000000
#define DP83816_CMDSTS_MORE	0x40000000
#define DP83816_CMDSTS_INTR	0x20000000
#define DP83816_CMDSTS_ECRC	0x10000000
#define DP83816_CMDSTS_OK	0x08000000
#define DP83816_CMDSTS_LEN	0x0000FFFF

/*
 *--------------------------------
 * cmdsts Transmit Bit definitions 
 *--------------------------------
 */
#define DP83816_CMDSTS_TXA	0x04000000
#define DP83816_CMDSTS_TFU	0x02000000
#define DP83816_CMDSTS_CRS	0x01000000
#define DP83816_CMDSTS_TD	0x00800000
#define DP83816_CMDSTS_ED	0x00400000
#define DP83816_CMDSTS_OWC	0x00200000
#define DP83816_CMDSTS_EC	0x00100000
#define DP83816_CMDSTS_CCNT	0x000F0000

/*
 *--------------------------------
 * cmdsts Receive Bit definitions 
 *--------------------------------
 */
#define DP83816_CMDSTS_RXA	0x04000000
#define DP83816_CMDSTS_RXO	0x02000000
#define DP83816_CMDSTS_DEST	0x01800000
#define DP83816_CMDSTS_LONG	0x00400000
#define DP83816_CMDSTS_RUNT	0x00200000
#define DP83816_CMDSTS_ISE	0x00100000
#define DP83816_CMDSTS_CRCE	0x00080000
#define DP83816_CMDSTS_FAE	0x00040000
#define DP83816_CMDSTS_LBP	0x00020000
#define DP83816_CMDSTS_COL	0x00010000

#define DP83816_CMDSTS_RX_ERRBITS (DP83816_CMDSTS_RXA  | \
								   DP83816_CMDSTS_RXO  | \
								   DP83816_CMDSTS_LONG | \
								   DP83816_CMDSTS_RUNT | \
								   DP83816_CMDSTS_ISE  | \
								   DP83816_CMDSTS_CRCE | \
								   DP83816_CMDSTS_FAE  | \
								   DP83816_CMDSTS_COL )
									

typedef struct _dp83816_stats {
	uint32 rxPkts;
	uint32 rxOctets;
	uint32 txPkts;
	uint32 txOctets;
	uint32 rxCrc;
	uint32 rxNoBuf;
	uint32 txQFull;
} dp83816_stats_t;

typedef struct _dp83816_txCtrl {
	dp83816_desc_t	*txqStart;
	dp83816_desc_t	*txqEnd;
	dp83816_desc_t	*txCur;
	dp83816_desc_t	*txCln;
	int32			avail;
	uint32			qd;
	uint32          unqd;
	uint32			pkts;
	uint32			octets;
	uint32			qFull;
	uint32			txaErr;
	uint32			tfuErr;
	uint32			crsErr;
	uint32			tdErr;
	uint32			edErr;
	uint32			owcErr;
	uint32			ecErr;
} dp83816_txCtrl_t;

typedef struct _dp83816_rxBuf_cache {
	uchar8 *ptr;
	uint32 len;
} dp83816_bc_t;

typedef struct _dp83816_rxCtrl {
	dp83816_desc_t	*rxqStart;
	dp83816_desc_t	*rxqEnd;
	dp83816_desc_t	*rxCur;
	dp83816_bc_t	bc[EEE_MAX_BUFS];
	uint32			bufCnt;
	uint32          avail;
	uint32			pkts;
	uint32			octets;
	uint32			rxoErr;
	uint32			rxaErr;
	uint32 			noBufErr;
	uint32 			noDescErr;
	uint32			tooLongErr;
	uint32			runtErr;
	uint32			isErr;
	uint32			crcErr;
	uint32			faErr;
	uint32			lbpErr;
	uint32			colErr;
} dp83816_rxCtrl_t;

#define ETHER_SPEED_UNKNOWN	0	
#define ETHER_SPEED_10HDX	1	/* 10MB hdx and fdx */
#define ETHER_SPEED_10FDX	2
#define ETHER_SPEED_100HDX	3	/* 100MB hdx and fdx */
#define ETHER_SPEED_100FDX	4
#define ETHER_SPEED_1000HDX	5	/* 1000MB hdx and fdx */
#define ETHER_SPEED_1000FDX	6

typedef enum { 
	speed_10, 
	speed_100, 
	speed_1000,
   	speed_unknown,	
} dp_speed_t;

typedef enum { 
	half_duplex,
	full_duplex,
	unknown_duplex,
} dp_duplex_t;

typedef enum { 
	fc_disabled, 
	fc_frame,
	fc_collision, 
	fc_carrier,
   	fc_unknown,	
} dp_fc_t;

typedef enum { 
	lb_none, 
	lb_int,
	lb_phy,
	lb_ext
} dp_loop_t;

typedef struct _dp83816_link {
	dp_fc_t     fc;
	uint32	 	timer;
	dp_speed_t  speed;
	dp_duplex_t duplex;
	uint32   	linkstat;
	dp_loop_t	loopback;
} dp83816_link_t;

typedef struct _dp83816_phy {

	uint32	addr;
	union {
		struct {
			uint16 bmsr;
			uint16 bmcr;
			uint16 k1stsr;
			uint16 anlpar;
		}bmc5411;
	}p;
} dp83816_phy_t;

// DP83816 Configuration Structure
typedef struct _dp83816_cfg {
	char			    macAddr[6];
	char			    macAddr_fp[6];
	uint32				appId;
	uint32				macId;
	uint32				baseAddr;
	uint32				phyAddr;
	uint32				ipAppPort;
	uint32				loopAddr;		/* System address of loopback app */
	dp83816_regs_t		*regs;
	dp83816_desc_t		*rxQ[DP83816_RX_Q_CNT];

	dp83816_txCtrl_t	txc[DP83816_TX_Q_CNT];
	dp83816_rxCtrl_t	rxc[DP83816_RX_Q_CNT];

	dp83816_stats_t		stats;
	dp83816_link_t		link;
	dp83816_phy_t		phy;
} dp83816_cfg_t;

//-- Special build of SP_NIM to allow it to act as
//-- a packet sniffer on the control and data planes.

#ifdef FCNIM
// #define DP83816_SNIFFER		1
#endif

#define DP83816_BYTES_SNIF_MAX		9000
#define DP83816_SNIF_ENTRIES_MAX	2048
typedef struct _dp83816_pkt_snifbuf {
	eeeRTC_t t;
	uint32   bufLen;
	uint32   capLen;
	uint32   cmdsts;
	uchar8   *buf;
} dp83816_psb_t;

extern dp83816_psb_t dp83816_rx_psb[DP83816_SNIF_ENTRIES_MAX];
extern uint32 dp83816_curRxSnifEntry;
extern uint32 dp83816_snifferOn;

//-- Pkt tracing built into the runtime 
#ifdef FCNIM
#define DP83816_TX_PKT_CAPTURE      1
#define DP83816_RX_PKT_CAPTURE      1
#endif

#define DP83816_TRACE_ENTRIES_MAX	1024	
#define DP83816_BYTES_TRACE_MAX     128

typedef struct _dp83816_pkt_tracebuf {
	eeeRTC_t t;
	uint32   pktLen;
	uint32   capLen;
	uint32   cmdsts;
	char    *orgBuf;
	uchar8   bytes[DP83816_BYTES_TRACE_MAX];
} dp83816_ptb_t;


typedef struct _dp83816_pkt_capture {
	uint32 txEna;
	uint32 maxTxBytesCap;
	uint32 txCapFilter;

	uint32 rxEna;
	uint32 maxRxBytesCap;
	uint32 rxCapFilter;
} dp83816_pc_t;

extern dp83816_ptb_t dp83816_tx_ptb[DP83816_TRACE_ENTRIES_MAX];
extern dp83816_ptb_t dp83816_rx_ptb[DP83816_TRACE_ENTRIES_MAX];
extern uint32 dp83816_curTxTraceEntry;
extern uint32 dp83816_curRxTraceEntry;
extern dp83816_pc_t dp83816_pc;

/*
 * Macros to convert virtual address to phy and back again.
 */
#if defined(EVAL_BOARD) || defined(AGILE_HARDWARE)
#ifndef FC_USE_NEW_MEM_MAP
#define V2PHYS_ADDR_MASK	0xA0000000
#define DP83816_2UNCHACHED(_x_)	((_x_) |= 0x20000000)
#endif
#else
#define V2PHYS_ADDR_MASK	0x00000000
#define DP83816_2UNCHACHED(_x_)	((_x_)) 
#endif	/* EVAL_BOARD || AGILE_HARDWARE */

/*
 * External Variable Declarations
 */
dp83816_cfg_t *dp83816;

/*
 * Function Prototypes
 */
void dp83816_rxPktPoll(void *, uint32);
void dp83816_txCleanPoll(void *, uint32);
void dp83816_rxAppPkt(eee_desc_t *);
int dp83816_writeReg(volatile uint32 *, uint32);


#endif /* DP83816_H_INCLUDED */
