/* =============================================================================
 * File Name  :	dp83820-diag.c
 *
 * Description:    
 *
 * Created    :	08 May 2001
 *
 * Author     :	Frank Salzmann
 *
 * Copyright(c)  Agile Storage, Inc. 2001
 * =============================================================================
 */
#include "dp83820.h"
#include <exports/malloc-api.h>

typedef struct dp83820_test {
	uint32 testState;				
#define DP_TEST_DONE		0
#define DP_TEST_INPROGRESS	1
#define DP_TEST_VERIFY		2
#define DP_TEST_CONTINUE	3
#define DP_TEST_END			4
	uint32 testMode;		
#define DP_TEST_INT			1
#define DP_TEST_EXT			2
#define DP_TEST_GMC			3
	uint32 startPktSize;
	uint32 endPktSize;
	uint32 testPktCnt;
	uint32 testPktSize;
	uint32 testDestAddr;
	uint32 testStep;
	uchar8 *testBuffer;
	uint32 fVerify;

	uint32 rxPktCnt;;
	uint32 rxErrPktCnt;
	uint32 rxByteCnt;
	uint32 txPktCnt;
	uint32 txByteCnt;
} dp83820_test_t;

static dp83820_test_t dp83820_testControl;

int dp83820_diagShowLbTestResults(dp83820_test_t *);
int dp83820_diagNewPkt(char *, uint32, uint32);
int dp83820_diagLbVerifyData(uchar8 *, int32, uchar8 *);
int dp83820_diagTxEna(dp83820_cfg_t *);
void dp83820_get_stats(uint32 *);

extern int dp83820_txUnicast(uint32, eee_desc_t *);
extern int dp83820_uiGetInput(char *);
extern int dp83820_readReg(volatile uint32 *);

/*
 *------------------------------------------------------------------------------
 * Function Name:	dp83820_diagSetLbMode
 *
 * Description: 	Cleans up descriptors on a transmit queue.
 *
 * Created by:		Frank Salzmann
 *
 * Data Created:	April 30, 2001
 * 					
 * Returns: ?????
 *
 *------------------------------------------------------------------------------
 */
int
dp83820_diagSetLbMode( dp83820_cfg_t *dp, uint32 mode )
{
	uint32 val;
	dp83820_cr_t cr;
	volatile dp83820_regs_t *r;
	dp83820_rxCtrl_t *rxc;

	r = dp->regs;

	/*
	 * First disable Rx and Tx
	 */
	cr.wd = 0;
	cr.bits.rxd = 1;
	cr.bits.txd = 1;
	dp83820_writeReg(&r->cr, cr.wd);
		
	/*
	 * Set/Reset LB Mode
	 */
	val = dp83820_readReg(&r->txcfg); 
	if( mode == DP83820_LB_ENABLE )
		val |= 0x20000000;
	else
		val &= ~0x20000000;
	dp83820_writeReg(&r->txcfg, val);
	
	/*
	 * For some reason the dp83820 increments it's receive pointer when
	 * rx is disabled/enabled, so we need to adjust the CPU's current 
	 * rx pointer as well.
	 */
	rxc = &dp->rxc[0];
	if( ++(rxc->rxCur) >= rxc->rxqEnd )
            rxc->rxCur = rxc->rxqStart;
	/*
	 * Enable Rx and Tx
	 */
	cr.wd = 0;
	cr.bits.rxe = 1;
	dp83820_writeReg(&r->cr, cr.wd);

	return(NFX_OK);
} /* dp83820_diagSetLbMode */

/*
 *------------------------------------------------------------------------------
 * Function Name:	dp83820_diagLBTest
 *
 * Description: 	Runs loopback test 
 *
 * Created by:		Frank Salzmann
 *
 * Data Created:	April 30, 2001
 * 					
 * Returns: ?????
 *
 *------------------------------------------------------------------------------
 */
int
dp83820_diagLBTest(dp83820_cfg_t *dp)
{
	int32  i, rc;
	int32  pktCnt;
	int32  step;
	int32  pktSize1, pktSize2;
	int32  testMode;
	int32  verify;
	uint32 destAddr;
	uchar8 *bufptr;

	pktCnt = dp83820_uiGetInput("Enter Pkt Count: ");
	if( pktCnt <= 0 ) {
		printf("Zero Pkt Count. Terminating Loopback Test\n"); 
		return(NFX_OK);
	}

	pktSize1 = dp83820_uiGetInput("Enter Start Pkt Size: ");
	if((pktSize1 < 64) || (pktSize1 > 9000)) {
		printf("Invalid Pkt Size: %d. Terminating Loopback Test\n", pktSize1); 
		return(NFX_OK);
	}

	pktSize2 = dp83820_uiGetInput("Enter End Pkt Size: ");
	if((pktSize2 < 64) || (pktSize2 > 9000)) {
		printf("Invalid Pkt Size: %d. Terminating Loopback Test\n", pktSize2); 
		return(NFX_OK);
	}

	step = dp83820_uiGetInput("Enter Step: ");

	verify = dp83820_uiGetInput("Verify [0:Off | 1:ON]: ");

	testMode = dp83820_uiGetInput("Enter Test Mode [INT:1 | EXT:2]: ");
	if((testMode < 1) || (testMode > 3)) {
		printf("Invalid Test Mode: %d. Terminating Loopback Test\n", testMode); 
		return(NFX_OK);
	}

	memset(&dp83820_testControl,0,sizeof(dp83820_testControl));

	dp83820_testControl.testState   = DP_TEST_INPROGRESS; 
	dp83820_testControl.testMode    = testMode;
	dp83820_testControl.testPktCnt  = pktCnt;
	dp83820_testControl.testPktSize = pktSize1;
	dp83820_testControl.startPktSize = pktSize1;
	dp83820_testControl.endPktSize = pktSize2;
	dp83820_testControl.testStep    = step;
	dp83820_testControl.fVerify		= verify;
	/*
	 * Allocate and initialize test data.
	 */
	bufptr = (uchar8 *)eee_ramAlloc(pktSize2);
	if( !bufptr ) {
		printf("Test data buffer allocation failed.\n");
		return(NFX_OK);
	}
	dp83820_testControl.testBuffer = bufptr;

#if 1
	if(testMode == DP_TEST_INT)
		dp83820_diagSetLbMode(dp83820, DP83820_LB_ENABLE);
#endif

	// printf("bufptr: %x, pktSize: %d\n", bufptr, pktSize);
	

#if 1
	for(i=0; i < pktSize2; i++ )
		bufptr[i] = i & 0xFF;
#endif

	/*
	 * Send first set of loop back packets.
	 */
#if defined(NFX_MOD_NFP)
	destAddr = PACK_UNICAST_PORT_ADDRESS(
								 nfxMySlotId,
								 nfxMyCPUId,
								 dp83820->appId,
								 EEE_MGMT,
								 NFX_COS_HIGH, 
								NFP_PORT_MGMT);
#elif defined(NFX_MOD_FCNIM)
	destAddr = PACK_UNICAST_PORT_ADDRESS(
								 nfxMySlotId,
								 nfxMyCPUId,
								 dp83820->appId,
								 EEE_USER_DATA,
								 NFX_COS_HIGH, 
								 FCNIM_PORT_BP_DATA);
#elif defined(PROM)
	destAddr = PACK_UNICAST_PORT_ADDRESS(
								 nfxMySlotId,
								 nfxMyCPUId,
								 dp83820->appId,
								 EEE_MGMT,
								 NFX_COS_HIGH, 
								PROM_PORT_MGMT);
#endif

	dp83820_testControl.testDestAddr = destAddr;

	for(i=0; i < 1; i++)
		rc = eee_sendMessage(bufptr, pktSize1, destAddr, destAddr);

	dp83820_diagTxEna(dp83820); 

	dp83820_testControl.txPktCnt += 1;
	dp83820_testControl.txByteCnt += (1 * pktSize1);
	return(NFX_OK);
} /* dp83820_diagLBTest */

/*
 *------------------------------------------------------------------------------
 * Function Name:	dp83820_diagLbPktReceive
 *
 * Description: 	Receives Loop back packet 
 *
 * Created by:		Frank Salzmann
 *
 * Data Created:	April 30, 2001
 * 					
 * Returns: ?????
 *
 *------------------------------------------------------------------------------
 */
int
dp83820_diagLbPktReceive( eee_desc_t *edesc )
{
	int32			rc;
	int32 			pktLen;
	int32 			bufCnt;
	dp83820_test_t 	*dptc;

	dptc = &dp83820_testControl;
	pktLen = edesc->hdr.control & EDESC_TOTAL_LEN;
	bufCnt = ((edesc->hdr.control & EDESC_BUFCNT_MASK) >> EDESC_BUFCNT_SHIFT);

	while(1) {
		switch( dptc->testState )
		{
			case DP_TEST_INPROGRESS:
				dptc->rxPktCnt++; 
				dptc->rxByteCnt += pktLen;

				if( dptc->fVerify )
					dptc->testState = DP_TEST_VERIFY;
				else {
					if( dptc->rxPktCnt >= dptc->testPktCnt)
						dptc->testState = DP_TEST_END;
					else
						dptc->testState = DP_TEST_CONTINUE;
				}
				break;

			case DP_TEST_VERIFY:
				rc = dp83820_diagLbVerifyData(edesc->bd[1].buf, 
											  edesc->bd[1].len, 
											  dptc->testBuffer);
				if( dptc->rxPktCnt >= dptc->testPktCnt)
					dptc->testState = DP_TEST_END;
				else
					dptc->testState = rc;
				break;

			case DP_TEST_CONTINUE:
				dptc->txPktCnt++; 
				dptc->txByteCnt += pktLen;

				if(dptc->testStep > 0 ) {
					dptc->testPktSize += dptc->testStep;
					if( dptc->testPktSize > dptc->endPktSize)
						dptc->testPktSize = dptc->startPktSize;

					eee_freePacket(edesc);

					dp83820_diagNewPkt( dptc->testBuffer,
										dptc->testPktSize, 
										dptc->testDestAddr);
				}
				else {
					edesc->hdr.dest_port = dptc->testDestAddr;
					eee_forwardPacket(edesc);
				}
				dptc->testState = DP_TEST_INPROGRESS;
				return(NFX_OK);
				
			case DP_TEST_END:
				if(dptc->testBuffer)
					eee_ramDealloc(dptc->testBuffer);

				if(dptc->testMode == DP_TEST_INT)
					dp83820_diagSetLbMode(dp83820, DP83820_LB_DISABLE);

				dp83820_diagShowLbTestResults(dptc);
				dptc->testState = DP_TEST_DONE;
				break;

			case DP_TEST_DONE:
				eee_freePacket(edesc);
				return(NFX_OK);
		}		
	}
	return(NFX_OK);
} /* dp83820_diagLbPktReceive */

/*
 *------------------------------------------------------------------------------
 * Function Name:	dp83820_diagLbVerifyData
 *
 * Description: 	Verifies the integrety of loop back data 
 *
 * Created by:		Frank Salzmann
 *
 * Data Created:	May 14, 2001
 * 					
 * Returns: ?????
 *
 *------------------------------------------------------------------------------
 */
int
dp83820_diagLbVerifyData( uchar8 *bufptr, int32 len, uchar8 *tbufptr )
{
	int32  i;

	if( !bufptr || !tbufptr )
		return(NFX_ERR);

	for(i=0; i < len; i++) {
		if( *bufptr != *tbufptr ) {
			printf("LB Data Error: TestVal=%02x, ActualVal=%02x, Addr=%lx\n",
				   *tbufptr, *bufptr, (address_t)bufptr);
			return(DP_TEST_END);
		}
		bufptr++;
		tbufptr++;
	}
	return(DP_TEST_CONTINUE);
} /* dp83820_diagLbVerifyData */

/*
 *------------------------------------------------------------------------------
 * Function Name:	dp83820_diagNewPkt
 *
 * Description: 	
 *
 * Created by:		Frank Salzmann
 *
 * Data Created:	Dec 03, 2001
 * 					
 * Returns: ?????
 *
 *------------------------------------------------------------------------------
 */
int
dp83820_diagNewPkt(char *bufptr, uint32 pktSize, uint32 destAddr)
{
	uint32 i;
	
	for(i=0; i < pktSize; i++ )
        bufptr[i] = i & 0xFF;     

	eee_sendMessage(bufptr, pktSize, destAddr, destAddr);
	return 0;
} // dp83820_diagNewPkt


/*
 *------------------------------------------------------------------------------
 * Function Name:	dp83820_diagTxPkt
 *
 * Description: 	Transmits a test packet.
 *
 * Created by:		Frank Salzmann
 *
 * Data Created:	June 25, 2001
 * 					
 * Returns: ?????
 *
 *------------------------------------------------------------------------------
 */
int
dp83820_diagTxPkt( dp83820_cfg_t *dp )
{
	int32  i,j,k;
	int32  bufCnt;
	int32  bufLen;
	int32  lastBufLen;
	int32  pktCnt;
	int32  pktSize;
	uchar8 *bufptr;
	eee_desc_t *edesc;
	eee_descHdr_t *hdr;
	dp83820_regs_t *regs;

	regs = dp83820->regs;

	pktCnt = dp83820_uiGetInput("Enter Pkt Count: ");
	if( pktCnt <= 0 ) {
		printf("Zero Pkt Count. Terminating Loopback Test\n"); 
		return(NFX_OK);
	}

	pktSize = dp83820_uiGetInput("Enter Pkt Size: ");
	if((pktSize < 64) || (pktSize > 9100)) {
		printf("Invalid Pkt Size: %d. Terminating Loopback Test\n", pktSize); 
		return(NFX_OK);
	}

	if( !(pktSize % 2048) ) {
		bufCnt = (pktSize / 2048);
		lastBufLen = 2048;
	}
	else {
		bufCnt = (pktSize / 2048) + 1;
		lastBufLen = pktSize % 2048;
	}

	printf("BufCnt: %d, LastBufLen: %d\n", bufCnt, lastBufLen);

	for(i=0; i < pktCnt; i++) {

		edesc = eee_allocateDesc(EEE_SHARED_POOL, NFX_COS_HIGH);
		memset(edesc,0, sizeof(eee_desc_t));
		hdr = (eee_descHdr_t *)edesc;
		hdr->control = (bufCnt << EDESC_BUFCNT_SHIFT) | pktSize;

		for(k=1; k <= bufCnt; k++) {
			if(k == bufCnt)
				bufLen = lastBufLen;
			else
				bufLen = 2048;

			bufptr = eee_allocateBuffer(EEE_SHARED_POOL, EEE_BUF_LARGE, NFX_COS_HIGH);
			if( !bufptr ) {
				printf("Test data buffer allocation failed.\n");
				return(NFX_OK);
			}

			for(j=0; j < bufLen; j++ ) {
				bufptr[j] = 0xff;
			}

			// dp83820_printBuf(p_uncached, bufLen);
			edesc->hdr.src_port =  dp83820->loopAddr;
			edesc->hdr.dest_port = dp83820->loopAddr;

			edesc->bd[k].buf = bufptr;
			edesc->bd[k].len = bufLen;
			edesc->bd[k].control = EDESC_BD_CMD_IN_USE | EDESC_BD_LARGE;
		}
		dp83820_txUnicast( 0,edesc );
	}
	return(NFX_OK);
} /* dp83820_diagTxPkt */

/*
 *------------------------------------------------------------------------------
 * Function Name:	dp83820_diagTxEna
 *
 * Description: 	Sets Tx Enable Bit
 *
 * Created by:		Frank Salzmann
 *
 * Data Created:	June 25, 2001
 * 					
 * Returns: ?????
 *
 *------------------------------------------------------------------------------
 */
int
dp83820_diagTxEna( dp83820_cfg_t *dp )
{
	dp83820_cr_t   cr;
	dp83820_regs_t *regs;

	regs = dp83820->regs;

	cr.wd = 0;
	cr.bits.txe = 1;
	dp83820_writeReg(&regs->cr, cr.wd);

	return(NFX_OK);
} /* dp83820_diagTxEna */

/*
 *------------------------------------------------------------------------------
 * Function Name:	dp83820_diagRxEna
 *
 * Description: 	Sets Tx Enable Bit
 *
 * Created by:		Frank Salzmann
 *
 * Data Created:	June 25, 2001
 * 					
 * Returns: ?????
 *
 *------------------------------------------------------------------------------
 */
int
dp83820_diagRxEna( dp83820_cfg_t *dp )
{
	dp83820_cr_t   cr;
	dp83820_regs_t *regs;

	regs = dp83820->regs;

	cr.wd = dp83820_readReg(&regs->cr);
	cr.bits.rxe = 1;
	dp83820_writeReg(&regs->cr, cr.wd);

	return(NFX_OK);
} /* dp83820_diagRxEna */

char *dpTestStates[] = {"TEST_DONE",
						"TEST_INPROGRESS",
						"TEST_VERIFY",
						"TEST_CONTINUE",
						"TEST_END"
};

char *dpTestMode[] = {"INTERNAL LOOPBACK",
					  "EXTERNAL LOOPBACK",
					  "GMC LOOPBACK"
};

int
dp83820_diagViewTestStats(dp83820_cfg_t *dp)
{
	dp83820_test_t *t;

	t = &dp83820_testControl;

	printf("DP83820 LOOPBACK TEST RESULTS\n");
	printf("=============================\n\n");
	printf("Test State        : %s\n", dpTestStates[t->testState]);
	printf("Test Mode         : %s\n", dpTestMode[t->testMode - 1]);
	printf("Test Packet Cnt   : %d\n", t->testPktCnt);
	printf("Test Packet Size  : %d\n", t->testPktSize);
	printf("Test Verify Flag  : %d\n", t->fVerify);
	printf("Actual Rx Pkt Cnt : %d\n", t->rxPktCnt);
	printf("Actual Rx Byte Cnt: %d\n", t->rxByteCnt);
	printf("Actual Rx Err Cnt : %d\n", t->rxErrPktCnt);
	printf("Actual Tx Pkt Cnt : %d\n", t->txPktCnt);
	printf("Actual Tx Byte Cnt: %d\n", t->txByteCnt);
	printf("\n");
	return 0;
} /* dp83820_diagViewTestStats */

/*
 *------------------------------------------------------------------------------
 * Function Name:	dp83820_diagShowLbTestResults
 *
 * Description: 	
 *
 * Created by:		Frank Salzmann
 *
 * Data Created:	May 14, 2001
 * 					
 * Returns: ?????
 *
 *------------------------------------------------------------------------------
 */
int
dp83820_diagShowLbTestResults(dp83820_test_t *t)
{
	printf("DP83820 LOOPBACK TEST RESULTS\n");
	printf("=============================\n\n");
	printf("Test State        : %s\n", dpTestStates[t->testState]);
	printf("Test Mode         : %s\n", dpTestMode[t->testMode - 1]);
	printf("Test Packet Cnt   : %d\n", t->testPktCnt);
	printf("Test Packet Size  : %d\n", t->testPktSize);
	printf("Test Verify Flag  : %d\n", t->fVerify);
	printf("Actual Rx Pkt Cnt : %d\n", t->rxPktCnt);
	printf("Actual Rx Byte Cnt: %d\n", t->rxByteCnt);
	printf("Actual Rx Err Cnt : %d\n", t->rxErrPktCnt);
	printf("Actual Tx Pkt Cnt : %d\n", t->txPktCnt);
	printf("Actual Tx Byte Cnt: %d\n", t->txByteCnt);
	printf("\n");
	return(NFX_OK);
} /* dp83820_diagShowLbTestResults */

/*
 *  This routine can be polled to gather stats for SNMP. All counters are
 *  cleared after each call. Counters returned are sequential as listed
 *  below. Caller must pass pointer to buffer with space for 18 32-bit values.
 */ 
void
dp83820_get_stats(uint32 *stat_buf)
{
	dp83820_cfg_t *dp = dp83820;
	dp83820_regs_t *regs;

	regs = dp->regs;

	*stat_buf++ = dp83820_readReg(&regs->mib[0]); // rx errors
	*stat_buf++ = dp83820_readReg(&regs->mib[1]); // rx fcs errors
	*stat_buf++ = dp83820_readReg(&regs->mib[2]); // rx overruns
	*stat_buf++ = dp83820_readReg(&regs->mib[3]); // rx frame alignment errors
	*stat_buf++ = dp83820_readReg(&regs->mib[4]); // rx symbol errors
	*stat_buf++ = dp83820_readReg(&regs->mib[6]); // rx in range length errors

	dp83820_writeReg(&regs->mibc, 0x4); // clear reg counters 

	*stat_buf++ = dp->rxc[0].pkts;		// rx packets
	*stat_buf++ = dp->rxc[0].octets;	// rx bytes
	*stat_buf++ = dp->rxc[0].rxaErr;	// rx aborts
	*stat_buf++ = dp->rxc[0].noBufErr;	// rx no buffers
	*stat_buf++ = dp->rxc[0].noDescErr;	// rx no EEE descriptors
	*stat_buf++ = dp->rxc[0].runtErr;	// rx runt packets 
	*stat_buf++ = dp->rxc[0].crcErr;	// rx crc errors 
	*stat_buf++ = dp->txc[0].pkts;		// tx packets
	*stat_buf++ = dp->txc[0].octets;	// tx bytes
	*stat_buf++ = dp->txc[0].qFull;		// tx queue full drops
	*stat_buf++ = dp->txc[0].txaErr;	// tx aborts
	*stat_buf++ = dp->txc[0].tfuErr;	// tx underruns

	dp->rxc[0].pkts = 0;
	dp->rxc[0].octets = 0;
	dp->rxc[0].rxaErr = 0;	
	dp->rxc[0].noBufErr = 0;
	dp->rxc[0].noDescErr = 0;
	dp->rxc[0].runtErr = 0;
	dp->rxc[0].crcErr = 0;
	dp->txc[0].pkts = 0;
	dp->txc[0].octets = 0;
	dp->txc[0].qFull = 0;	
	dp->txc[0].txaErr = 0;
	dp->txc[0].tfuErr = 0;
}

