/*  
 * File Name   :    dp83820.c
 *
 *  Description:    
 *
 *  Created    :    26 April 2001
 *
 *  Author     :    Frank Salzmann
 *
 *  Copyright(c)  Agile Storage, Inc. 2001
 */

/*
 *------------------------------------------------------------------------------
 * Function Name:	dp83820_pciInit
 *
 * Description: 	Initializes the DP83820 PCI Config Registers. Called by
 *					the PCI module when the PCI ID for this chip is found.
 *
 * Created by:		Frank Salzmann
 *
 * Data Created:	May 15, 2001
 * 					
 * Returns: 
 *
 *------------------------------------------------------------------------------
 */
#include "dp83820.h"
#include <exports/pci-api.h>

#define DP83820_PCI_DEVID	0x0022100B

int
dp83820_pciInit( int busId, int devId )
{
	uint32 value;
	uint32 offset;
	
	//printf("[dp83820_pciInit] Enter: busId: %d, devId: %d\n", busId, devId);

#ifndef LINUX_TEST

#if defined(EVAL_BOARD) || defined(AGILE_HARDWARE)
	/*
	 * Read Config Reg and See if national board is present.
	 */
	offset = 0;
	value  = 0;
	pci_cfg_read(busId, devId, 0, offset, &value);
	if( value != DP83820_PCI_DEVID ) {
		printf("DP83820 PCI Card Not Found\n");
		return(NFX_ERR);
	}
#if 0
	else
		printf("DP83820 PCI Card Found.\n");
#endif
#endif  // EVAL_BOARD || AGILE_HARDWARE

	/* 
	 * PCI Cmd & Status Reg 
	 */
	offset = 0x4;
	//value  = 0x00000006;
	value  = 0x00000146;
	pci_cfg_write(busId, devId, 0, offset, value);
	
	/* 
	 * PCI Latency Reg 
	 */
	offset = 0xC;
	value  = 0x0000ff04;
	pci_cfg_write(busId, devId, 0, offset, value);

	/* 
	 * PCI Memory Base Address Reg 
	 */
	offset = 0x14;
	value  = DP83820_BASE_ADDR;
	//printf("Setting BaseAddr: %x\n", DP83820_BASE_ADDR);
	pci_cfg_write(busId, devId, 0, offset, value);

	//printf("[dp83820_pciInit] Leave\n");
	return(NFX_OK);
#else
	return(NFX_ERR);
#endif /* NOT LINUX_TEST */


} /* dp83820_pciInit */
