/*  
 *=============================================================================
 * File Name   :    dp83820-regs.h
 *
 *  Description:    
 *
 *  Created    :    09 April 2001
 *
 *  Author     :    Frank Salzmann
 *
 *  Copyright(c)  Agile Storage, Inc. 2001
 *=============================================================================
 */
#ifndef DP83820_REGS_H_INCLUDED
#define DP83820_REGS_H_INCLUDED

typedef struct _dp83820_regs
{
	uint32 cr;					/* 0x00:		Cmd Reg						*/
	uint32 cfg;					/* 0x04:		Config Reg					*/
	uint32 mear;				/* 0x08:		EEProm Access Reg			*/
	uint32 ptscr;				/* 0x0C:		PCI Test Ctrl Reg			*/
	uint32 isr;					/* 0x10:		Interrupt Status Reg		*/
	uint32 imr;					/* 0x14:		Interrupt Mask Reg			*/
	uint32 ier;					/* 0x18:		Interrupt Enable Reg		*/
	uint32 ihr;					/* 0x1C:		Interrupt Holdoff Reg		*/
	uint32 txdp;				/* 0x20:		Tx Desc Pointer Reg			*/
	uint32 txdp_hi;				/* 0x24:		Tx Desc Ptr Hi Dword Reg	*/
	uint32 txcfg;				/* 0x28:		Tx Config Reg				*/
	uint32 gpior;				/* 0x2c:		Gen Purpose I/O Ctrl Reg	*/
	uint32 rxdp;				/* 0x30:		Rx Desc Pointer Reg			*/
	uint32 rxdp_hi;				/* 0x34:		Rx Desc Ptr Hi Dword Reg	*/
	uint32 rxcfg;				/* 0x38:		Rx Config Reg				*/
	uint32 pqcr;				/* 0x3C:		Priority Que'ing Ctrl Reg	*/
	uint32 wcsr;				/* 0x40:		Wake on LAN ctrl/status Reg	*/
	uint32 pcr;					/* 0x44:		Pause Ctrl/Status Reg		*/
	uint32 rfcr;				/* 0x48:		Rx Filter/Match Ctrl Reg	*/
	uint32 rfdr;				/* 0x4C:		Rx Filter/Match Data Reg	*/
	uint32 brar;				/* 0x50:		Boot ROM Address			*/
	uint32 brdr;				/* 0x54:		Boot ROM Data				*/
	uint32 srr;					/* 0x58:		Silicon Revision Reg		*/
	uint32 mibc;				/* 0x5C:		MIB Ctrl Reg				*/
	uint32 mib[11];				/* 0x60-0x88:	MIB Data Regs				*/
	uint32 resv1[5];			/* 0x8C-0x9C:	Reserved					*/
	uint32 txdp1;				/* 0xA0:		Tx Desc Ptr Priority 1 Reg	*/
	uint32 txdp2;				/* 0xA4:		Tx Desc Ptr Priority 2 Reg	*/
	uint32 txdp3;				/* 0xA8:		Tx Desc Ptr Priority 3 Reg	*/
	uint32 resv2;				/* 0xAC:		Reserved					*/
	uint32 rxdp1;				/* 0xB0:		Rx Desc Ptr Priority 1 Reg	*/
	uint32 rxdp2;				/* 0xB4:		Rx Desc Ptr Priority 2 Reg	*/
	uint32 rxdp3;				/* 0xB8:		Rx Desc Ptr Priority 3 Reg	*/
	uint32 vrcr;				/* 0xBC:		VLAN/IP Rx Ctrl Reg			*/
	uint32 vtcr;				/* 0xC0:		VLAN/IP Tx Ctrl Reg			*/
	uint32 vdr;					/* 0xC4:		VLAN Data Reg				*/
	uint32 resv3;				/* 0xC8:		Reserved					*/
	uint32 ccsr;				/* 0xCC:		Clk Run Ctrl/Status Reg		*/
	uint32 resv4[4];			/* 0xD0-0xDC:	Reserved					*/
	uint32 tbicr;				/* 0xE0:		TBI Ctrl Reg				*/
	uint32 tbisr;				/* 0xE4:		TBI Status Reg				*/
	uint32 tanar;				/* 0xE8:		TBI Auto-Neg Ad Reg			*/
	uint32 tanlpar;				/* 0xEC:		TBI Auto-Neg Link Pt Ab Reg	*/
	uint32 taner;				/* 0xF0:		TBI Auto-Neg Exp Reg		*/
	uint32 tesr;				/* 0xF4:		TBI Ext Status Reg			*/
	uint32 resv5[2];			/* 0xF8-0xFC:	Reserved					*/
	uint32 opAlias[64];			/* 0x100-0x1FC:	Mem Map Alias for Op Regs	*/
	uint32 pciCfgRegs[64];		/* 0x200-0x2FC: PCI Config Regs				*/
	uint32 pciAlias[64];		/* 0x300-0x3FC:	Mem Map Alias of PCI Regs	*/
} dp83820_regs_t;

/*
 *--------------------------------
 * DP83820 COMMAND REGISTER
 *--------------------------------
 */
typedef union _dp83820_cr
{
	uint32 wd;
	struct {
		uint32 txe    :1;
		uint32 txd	  :1;
        uint32 rxe    :1;
		uint32 rxd    :1;
		uint32 txr    :1;
        uint32 rxr    :1;
		uint32 unused2:1;
		uint32 swi	  :1;
		uint32 rst    :1;
		uint32 txpri  :4;
		uint32 rxpri  :4;
		uint32 unused1:15;
	} bits;
} dp83820_cr_t;

/*
 *--------------------------------
 * DP83820 CONFIG REGISTER
 *--------------------------------
 */
typedef union _dp83820_cfgr
{
	uint32 wd;
	struct {
		uint32 bem      :1;
		uint32 ext_125  :1;
		uint32 brom_dis :1;
		uint32 pesel    :1;
		uint32 exd      :1;
		uint32 pow      :1;
		uint32 sb       :1;
		uint32 reqalg   :1;
		uint32 extsts_en:1;
		uint32 phy_dis  :1;
		uint32 phy_rst  :1;
		uint32 m64addr  :1;
		uint32 data64_en:1;
		uint32 pci64_det:1;
		uint32 t64addr  :1;
		uint32 mwi_dis  :1;
		uint32 mrm_dis  :1;
		uint32 tmrtest  :1;
		uint32 pint_ctl :3;
		uint32 unused3  :1;
		uint32 mode_1000:1;
		uint32 unused2  :1;
		uint32 tbi_en   :1;
		uint32 unused1  :3;
		uint32 dupsts   :1;
		uint32 spdsts   :2;
		uint32 lnksts   :1;
	} bits;
} dp83820_cfgr_t;

/*
 *--------------------------------
 * DP83820 RECEIVE CONFIG REGISTER
 *--------------------------------
 */
typedef union _dp83820_rxcfgr
{
	uint32 wd;
	struct {
		uint32 unused1  :1;
	    uint32 drth     :5;		
		uint32 unused2  :14;
		uint32 mxdma	:3;
		uint32 unused3  :3;
		uint32 airl		:1;
		uint32 alp		:1;
		uint32 rx_fd	:1;
		uint32 stripcrc :1;
		uint32 arp      :1;
		uint32 aep      :1;
	}bits;
} dp83820_rxcfgr_t;

/*
 *--------------------------------
 * DP83820 TRANSMIT CONFIG REGISTER
 *--------------------------------
 */
typedef union _dp83820_txcfgr
{
	uint32 wd;
	struct {
	    uint32 drth     :8;		
		uint32 flth     :8;
		uint32 unused1  :3;
		uint32 brst_dis :1;
		uint32 mxdma    :3;
		uint32 ecretry  :1;
		uint32 unused2  :4;
		uint32 atp      :1;
		uint32 mlb      :1;
		uint32 hbi      :1;
		uint32 csi      :1;
	}bits;
} dp83820_txcfgr_t;

/*
 *----------------------------------------
 * DP83820 RECEIVE FILTER CONTROL REGISTER
 *----------------------------------------
 */
typedef union _dp83820_rfcr
{
	uint32 wd;
	struct {
		uint32 rfaddr  :10;
		uint32 unused  :9;
		uint32 ulm     :1;
		uint32 uhen    :1;
		uint32 mhen    :1;
		uint32 aarp    :1;
		uint32 apat    :4;
		uint32 apm     :1;
		uint32 aau     :1;
		uint32 aam     :1;
		uint32 aab     :1;
		uint32 rfen    :1;
	}bits;
} dp83820_rfcr_t;

/*
 *----------------------------------------
 * DP83820 PAUSE CONTROL REGISTER
 *----------------------------------------
 */
typedef union _dp83820_pcr
{
	uint32 wd;
	struct {
		uint32 pause_cnt	:16;
		uint32 resv1		:1;
		uint32 ps_tx		:1;
		uint32 ps_fflo		:2;
		uint32 ps_ffhi		:2;
		uint32 ps_stlo		:2;
		uint32 ps_sthi		:2;
		uint32 resv2		:1;
		uint32 ps_rcvd		:1;
		uint32 ps_act		:1;
		uint32 ps_da		:1;
		uint32 ps_mcast		:1;
		uint32 psen			:1;
	}bits;
} dp83820_pcr_t;

#endif /* DP83820_REGS_H_INCLUDED */
