/*  
 *=============================================================================
 * File Name  :    dp83820-rx.c
 *
 * Description:    
 *
 * Created    :    30 April 2001
 *
 * Author     :    Frank Salzmann
 *
 * Copyright(c)  Agile Storage, Inc. 2001
 *=============================================================================
 */

#include "dp83820.h"
#include <exports/mem-api.h>

int dp83820_rx_pkt_trace = 0;
int dp_rxHWM = 0;

int dp83820_rxBufCapture(char *, uint32);
int dp83820_decodeScsi(char *);
int dp83820_printBuf(char *, int, int);

/*
 *------------------------------------------------------------------------------
 * Function Name:	dp83820_rxPacket
 *
 * Description: 	Transmits a unicast packet to the backplane.
 *
 * Created by:		Frank Salzmann
 *
 * Data Created:	April 30, 2001
 * 					
 * Returns: ?????
 *
 *------------------------------------------------------------------------------
 */
int
dp83820_rxPacket( dp83820_rxCtrl_t *rxc , dp83820_cfg_t *dp, uint32 loopCnt )
{
	int 			i, diff;
	address_t		tmp;
	uint32			fError;
	uint32 			cmdsts;
	uint32 			pktLen;
	uchar8 			*newBuf;
	eee_desc_t     	*eDesc = NULL;
	dp83820_desc_t 	*rxCur;
	int 			hdrLen;
	uint32			slot, cpu;
	int				maxloopCnt = loopCnt;

	rxCur = rxc->rxCur;
	cmdsts = rxCur->cmdsts;
	while( (cmdsts & DP83820_CMDSTS_OWN) && loopCnt-- ) {

		/*
		 * Allocate a new receive buffer and add to rx queue.
		 */
		newBuf = eee_allocateBuffer(EEE_SHARED_POOL, DP83820_BUFFER_SIZE,0);
		if( !newBuf ) {
			rxc->noBufErr++;
			dp->stats.rxNoBuf++;
			break;			/* Wait until more buffers are available */
		}

		/*
		 * Allocate EEE packet descriptor.
		 */
		if(((cmdsts & DP83820_CMDSTS_MORE)==0)&&(cmdsts & DP83820_CMDSTS_OK)) {
			eDesc = eee_allocateDesc(EEE_SHARED_POOL, NFX_COS_HIGH);
			if( !eDesc ) {
				eee_deallocateBuffer(newBuf);
				rxc->noDescErr++;
				break;		/* Wait until more descriptors are available */
			}
			// Mark first buffer as unused.
			eDesc->bd[0].control = 0;
		}
		else
			eDesc = NULL;

		/*
		 * Store buffer pointers in buffer cache for now.
		 */
#ifdef SIBYTE
		tmp = PHYS_TO_VIRT((address_t)rxCur->bufptr);
#else
#ifdef FC_USE_NEW_MEM_MAP
		tmp = phys_to_virt_cached((address_t)rxCur->bufptr);
#else
		tmp = (address_t)(rxCur->bufptr);
		tmp |= 0x80000000;
#endif
#endif
		rxc->bc[rxc->bufCnt].ptr = (uchar8 *)tmp;
		rxc->bc[rxc->bufCnt].len = cmdsts & DP83820_CMDSTS_LEN;
		rxc->bufCnt++;

		//-- Rx buf capture.
		if( dp83820_pc.rxEna ) {	
			dp83820_rxBufCapture((char *)tmp, cmdsts);
		}

#if 0
		if( dp83820_rx_pkt_trace ) {
			printf("dp83820 Rx: Bufptr: %x, BufLen: %d, cmdsts: %x\n",
			tmp, (cmdsts & DP83820_CMDSTS_LEN), cmdsts);
			dp83820_printBuf( tmp, (cmdsts & DP83820_CMDSTS_LEN), 1); 
		}
#endif

		/*
		 * Replenish buffer and transfer descriptor ownership 
		 * back to the chip.
		 */
#ifdef SIBYTE
		rxCur->bufptr = (uchar8 *)VIRT_TO_PHYS((address_t)newBuf);
#else
#ifdef FC_USE_NEW_MEM_MAP
		rxCur->bufptr = (uchar8 *)virt_to_phys((address_t)newBuf);
#else
		tmp = (address_t)(newBuf) & ~0xA0000000;
		rxCur->bufptr = (uchar8 *)tmp;
#endif
#endif
		rxCur->cmdsts = DP83820_RX_BUF_SIZE & DP83820_CMDSTS_LEN;

		/* 
		 * Look for the last buffer in the packet 
		 */
		if( (cmdsts & DP83820_CMDSTS_MORE) == 0 ) {
			if((cmdsts & DP83820_CMDSTS_OK) && !(cmdsts & DP83820_CMDSTS_RX_ERRBITS)) {
				fError = 0;
				pktLen = 0;
				for(i=1; i <= rxc->bufCnt; i++) {

					if( dp83820_rx_pkt_trace ) {
						printf("dp83820 Rx: Bufptr: %lx, BufLen: %d\n",
							   (address_t)rxc->bc[i-1].ptr, rxc->bc[i-1].len);
					}
					
					eDesc->bd[i].buf = rxc->bc[i-1].ptr;
					eDesc->bd[i].len = rxc->bc[i-1].len;
					eDesc->bd[i].control = EDESC_BD_CMD_IN_USE
						| EDESC_BD_LARGE;
					pktLen += rxc->bc[i-1].len;
				}
#if defined(PROM)
				//-- Look at the ethernet type field to see
				//-- if this is an IP packet. If so then
				//-- pass it up to the LUC receive function.
				if (((enet_hdr_t *)eDesc->bd[1].buf)->type != EEE_ETH_TYPE ) {
					eDesc->hdr.dest_port = dp83820->ipAppPort;
					eDesc->hdr.src_port  = dp83820->loopAddr;
					hdrLen = 0;
				}
				else
#endif // PROM
				{
					/* Used as backplane driver */
					hdrLen = eee_removeHeader(eDesc);
					pktLen -= hdrLen;
				}

				if( dp83820_rx_pkt_trace ) {
					printf("RxPkt: Len: %d, buf: %lx, ", pktLen,
						   (address_t)(eDesc->bd[1].buf - hdrLen));
					printf("dest_port: %x, ", eDesc->hdr.dest_port);
					printf("src_port: %x\n", eDesc->hdr.src_port);
					dp83820_printBuf( eDesc->bd[1].buf - hdrLen,
									  eDesc->bd[1].len + hdrLen, 1); 
				}
				eDesc->hdr.control = 
					(pktLen & EDESC_TOTAL_LEN) |
					((rxc->bufCnt << EDESC_BUFCNT_SHIFT) & EDESC_BUFCNT_MASK);

#if 1
				slot = EEE_GET_SLOT_NUM(eDesc->hdr.src_port);
				cpu = EEE_GET_DEST_CPU(eDesc->hdr.src_port);
				rxc->rxpkts[slot][cpu]++;
#endif
				rxc->pkts++;
				rxc->octets += pktLen;
				dp->stats.rxPkts++;
				dp->stats.rxOctets += pktLen;

				/*
				 * Queue the packet to the EEE layer.
				 */
				eee_receivePacket(eDesc);
				
			}
			else {    /* Errors found */

				printf("Rx Error Detected. Dropping Packet: cmdsts: %x\n", cmdsts);
				fError = 1;
				if( cmdsts & DP83820_CMDSTS_RXA ) {
					if( cmdsts & DP83820_CMDSTS_RXO )
						rxc->rxoErr++;
					else
						rxc->rxaErr++;
				}
				if( cmdsts & DP83820_CMDSTS_LONG ) 
					rxc->tooLongErr++;
				if( cmdsts & DP83820_CMDSTS_RUNT )
					rxc->runtErr++;
				if( cmdsts & DP83820_CMDSTS_ISE )
					rxc->isErr++;
				if( cmdsts & DP83820_CMDSTS_CRCE ) {
					rxc->crcErr++;
					dp->stats.rxCrc++;
				}
				if( cmdsts & DP83820_CMDSTS_FAE )
					rxc->faErr++;
				if( cmdsts & DP83820_CMDSTS_LBP )
					rxc->lbpErr++;
				if( cmdsts & DP83820_CMDSTS_IRL )
					rxc->irlErr++;

				if (eDesc)
					eee_deallocateDesc(eDesc);
			}

			/*
			 * Clear buffer cache.
			 */
			for(i=0; i < rxc->bufCnt; i++) {
				if(fError && rxc->bc[i].ptr)
					eee_deallocateBuffer(rxc->bc[i].ptr);
				rxc->bc[i].ptr = 0;
				rxc->bc[i].len = 0;
			}
			rxc->bufCnt = 0;
		}
		if( ++rxCur >= rxc->rxqEnd )
			rxCur = rxc->rxqStart;

		cmdsts = rxCur->cmdsts;
	}
	rxc->rxCur = rxCur;
	diff = maxloopCnt - loopCnt;
	if (diff > dp_rxHWM) // high water mark for rx descriptors processed
		dp_rxHWM = diff;
	return(NFX_OK);
} /* dp83820_rxPacket */

//------------------------------------------------------------------------------
// Function Name:	dp83820_rxPacketSniffer
//
// Description: 	Captures all packets in a circular buffer.
//
// Created by:		Frank Salzmann
//
// Data Created:	June 07, 2002
//------------------------------------------------------------------------------
int
dp83820_rxPacketSniffer( dp83820_rxCtrl_t *rxc , dp83820_cfg_t *dp, uint32 loopCnt )
{
#ifdef DP83820_SNIFFER
	address_t		tmp;
	uint32 			cmdsts;
	uchar8 			*newBuf, *cpyBuf, *oldBuf;
	dp83820_desc_t 	*rxCur;
	uint32			bufLen, capLen;

	rxCur = rxc->rxCur;
	cmdsts = rxCur->cmdsts;
	while( (cmdsts & DP83820_CMDSTS_OWN) && loopCnt-- ) {

		//-- Allocate a new receive buffer and add to rx queue.
		newBuf = eee_allocateBuffer(EEE_SHARED_POOL, DP83820_BUFFER_SIZE,0);
		if( !newBuf ) {
			rxc->noBufErr++;
			dp->stats.rxNoBuf++;
			break;			/* Wait until more buffers are available */
		}

#ifdef SIBYTE
		tmp = PHYS_TO_VIRT((address_t)rxCur->bufptr);
#else
#ifdef FC_USE_NEW_MEM_MAP
		tmp = phys_to_virt_cached(rxCur->bufptr);
#else
		tmp = (address_t)(rxCur->bufptr);
		tmp |= 0x80000000;
#endif
#endif
		cpyBuf = (uchar8 *)tmp;
		bufLen = cmdsts & DP83820_CMDSTS_LEN;
		
		dp->stats.rxOctets += bufLen;
		rxc->octets += bufLen;

		if( dp83820_rx_pkt_trace ) {
			printf("dp83820 Rx: Bufptr: %x, BufLen: %d, cmdsts: %x\n",
			cpyBuf, bufLen, cmdsts);
			dp83820_printBuf( cpyBuf, bufLen, 0); 
		}

		//-- Replenish buffer and transfer descriptor ownership 
		//-- back to the chip.
#ifdef SIBYTE
		rxCur->bufptr = VIRT_TO_PHYS((address_t)newBuf);
#else
#ifdef FC_USE_NEW_MEM_MAP
		rxCur->bufptr = virt_to_phys(newBuf);
#else
		tmp = (address_t)(newBuf) & ~0xA0000000;
		rxCur->bufptr = (uchar8 *)tmp;
#endif
#endif
		rxCur->cmdsts = DP83820_RX_BUF_SIZE & DP83820_CMDSTS_LEN;

		//-- Look for the last buffer in the packet 
		if( (cmdsts & DP83820_CMDSTS_MORE) == 0 ) {
			if( cmdsts & DP83820_CMDSTS_OK ) {
				rxc->pkts++;
				dp->stats.rxPkts++;
			}
			else {    // Errors found 

				printf("Rx Error Detected. Dropping Packet: cmdsts: %x\n", cmdsts);
				if( cmdsts & DP83820_CMDSTS_RXA ) {
					if( cmdsts & DP83820_CMDSTS_RXO )
						rxc->rxoErr++;
					else
						rxc->rxaErr++;
				}
				if( cmdsts & DP83820_CMDSTS_LONG ) 
					rxc->tooLongErr++;
				if( cmdsts & DP83820_CMDSTS_RUNT )
					rxc->runtErr++;
				if( cmdsts & DP83820_CMDSTS_ISE )
					rxc->isErr++;
				if( cmdsts & DP83820_CMDSTS_CRCE ) {
					rxc->crcErr++;
					dp->stats.rxCrc++;
				}
				if( cmdsts & DP83820_CMDSTS_FAE )
					rxc->faErr++;
				if( cmdsts & DP83820_CMDSTS_LBP )
					rxc->lbpErr++;
				if( cmdsts & DP83820_CMDSTS_IRL )
					rxc->irlErr++;
			}

		}

		//
		//-- Capture incoming buffer
		//
		if( dp83820_snifferOn ) {
			//-- Figure out how many bytes to copy from pkt.
			if( bufLen > DP83820_BYTES_SNIF_MAX )
				capLen = DP83820_BYTES_SNIF_MAX;
			else
				capLen = bufLen;

			//-- Save len information and timestamp pkt
			dp83820_rx_psb[dp83820_curRxSnifEntry].capLen = capLen;
			dp83820_rx_psb[dp83820_curRxSnifEntry].bufLen = bufLen;
			dp83820_rx_psb[dp83820_curRxSnifEntry].cmdsts = cmdsts;
			eee_rtc(&(dp83820_rx_psb[dp83820_curRxSnifEntry].t));
		
			//-- Copy data from buffer
			//capBuf = dp83820_rx_psb[dp83820_curRxSnifEntry].bytes; 
			//memcpy(capBuf, cpyBuf, capLen);
			oldBuf = dp83820_rx_psb[dp83820_curRxSnifEntry].buf;
			dp83820_rx_psb[dp83820_curRxSnifEntry].buf = cpyBuf;

			//-- Free buffer
			if( oldBuf )
				eee_deallocateBuffer(oldBuf);

			if( ++dp83820_curRxSnifEntry >= DP83820_SNIF_ENTRIES_MAX )
				dp83820_curRxSnifEntry = 0;	
		}
		else {
			if(cpyBuf)
				eee_deallocateBuffer(cpyBuf);
		}

		//-- Get next buffer
		if( ++rxCur >= rxc->rxqEnd )
			rxCur = rxc->rxqStart;

		cmdsts = rxCur->cmdsts;
	}
	rxc->rxCur = rxCur;
#endif // DP83820_SNIFFER
	return(NFX_OK);
} // dp83820_rxPacketSniffer

/*
 *------------------------------------------------------------------------------
 * Function Name:	dp83820_rxPktPoll
 *
 * Description: 	Called by the eee layer and in turn calls dp83820_rxPacket
 *					for each receive priority queue to collect packet buffers
 *					and pass them up to the eee layer.
 *
 * Created by:		Frank Salzmann
 *
 * Data Created:	May 2, 2001
 * 					
 * Returns: NFX_OK
 *
 *------------------------------------------------------------------------------
 */
void
dp83820_rxPktPoll( void *cb, uint32 tlimit )
{
	int i;
	int32 loopCnt;
	
#if defined(NFX_SMP) && defined(NFP_TXRX)
	if (!CM_IS_TXRX_NCPU)
		return;
#endif /* NFX_SMP, TXRX */
	loopCnt = 256;
#if DP83820_SNIFFER
	for(i=0; i < DP83820_RX_Q_CNT; i++) {
		if(CM_IS_FC_CPUID0)     // cpuid 0 is fc slice 3 on slot 2
			dp83820_rxPacketSniffer( &dp83820->rxc[i], dp83820, loopCnt );
		else
			dp83820_rxPacket( &dp83820->rxc[i], dp83820, loopCnt );
	}
#else
	for(i=0; i < DP83820_RX_Q_CNT; i++) {
			dp83820_rxPacket( &dp83820->rxc[i], dp83820, loopCnt );
	}
#endif
	return;
} /* dp83820_rxPktPoll */


static int
dp83820_decodeHdr( char *data )
{
	uint32 tmp;

	printf("\n");
	printf("MAC DA  : %02x:%02x:%02x:%02x:%02x:%02x\n", 
		   data[0]&0xff, data[1]&0xff, data[2]&0xff, 
		   data[3]&0xff, data[4]&0xff, data[5]&0xff);
	printf("MAC SA  : %02x:%02x:%02x:%02x:%02x:%02x\n", 
		   data[6]&0xff, data[7]&0xff, data[8]&0xff, 
		   data[9]&0xff, data[10]&0xff, data[11]&0xff);

	tmp = (*(uint32 *)(data+12)) & 0xFFFF;
	printf("TYPE    : %04x\n", tmp);

	if((tmp == EEE_ETH_TYPE) || (tmp==EEE_ETH_TYPE_SAC)) {		// Only if Agile Protocol type.
		tmp = *(uint32 *)(data+16);
		printf("EEE DEST: %08x, App: %d:%s, CPU: %d, Slot: %d, Dtype: %d\n", 
			   tmp,
			   EEE_GET_APP_ID(tmp),
			   eee_getAppNameFromId(EEE_GET_APP_ID(tmp)),
			   EEE_GET_DEST_CPU(tmp),
			   EEE_GET_SLOT_NUM(tmp),
			   EEE_GET_DTYPE(tmp));

		tmp = *(uint32 *)(data+20);
		printf("EEE SRC : %08x, App: %d:%s, CPU: %d, Slot: %d, Dtype: %d\n", 
			   tmp,
			   EEE_GET_APP_ID(tmp),
			   eee_getAppNameFromId(EEE_GET_APP_ID(tmp)),
			   EEE_GET_DEST_CPU(tmp),
			   EEE_GET_SLOT_NUM(tmp),
			   EEE_GET_DTYPE(tmp));

		dp83820_decodeScsi(data);
	}

	printf("\n............HEX DUMP.................\n");
	return 0;
} // dp83820_decodeHdr

int
dp83820_decodeScsi(char *data)
{
	uint32 hdrLen;
    uint32 tmp;
    uint32 *msg;
	agile_hdr_t *eeeHdr = (agile_hdr_t *)(data + sizeof(enet_hdr_t)); 

	hdrLen = sizeof(enet_hdr_t) + sizeof(agile_hdr_t) + eeeHdr->pad; 
    msg = (uint32 *)(data+hdrLen);
 
    tmp = *msg;

	if( tmp == 0x0400 || tmp == 0x0401 || tmp == 0x0403 || tmp == 0x0402 ) {
        printf("\n............SCSI HDR.................\n\n");
        if( tmp == 0x0401 )
            printf("SCSI READ RSP : ");
        else
		if( tmp == 0x0403 )
            printf("SCSI WRITE RSP: ");
		else
		if( tmp == 0x0400 )
            printf("SCSI READ REQ: ");
        else
		if( tmp == 0x0402 )
            printf("SCSI WRITE REQ: ");
 
        printf("CHANDLE: %x, LBA: %d, LBCNT: %d\n",
               *(msg+1), *(msg+5), *(msg+6));
    }
    return 0;
} // dp83820_decodeScsi

int
dp83820_printBuf(char *buf, int len, int fDecode)
{
	int i, j;
    char ch;
	char *data;
 
	data = buf;
	if( fDecode ) {
		dp83820_decodeHdr(data);
	}

#if 0
	if( len > 64 )
		len = 64;
#endif

    for(i=0; i < len; i+=16) {
        printf("\n%04x:\t", i);
        for( j=i; j < (i+16); j++ )
            printf("%02x ", data[j] & 0xff);
        printf("    ");
        for( j=i; j < (i+16); j++ ) {
            ch = data[j] & 0xff;
            if( (ch > 0x20) && (ch < 0x7F))
                printf("%c", ch);
            else
            if( ch == 0 )
                printf("%c",'.');
            else
                printf("%c",'_');
        }
    }
    printf("\n\n");                      
	return 0;
} // dp83820_printBuf

int
dp83820_rxBufCapture(char *buf, uint32 cmdsts)
{
#ifdef DP83820_RX_PKT_CAPTURE  
	char   *capBuf;
	char   *cpyBuf;
	uint32 capLen;
	uint32 bufLen;

	if( !buf ) {
		printf("dp83820:  Error capture null buf ptr.\n");
		//-- Save len information and timestamp pkt
		dp83820_rx_ptb[dp83820_curRxTraceEntry].capLen = 0;
		dp83820_rx_ptb[dp83820_curRxTraceEntry].pktLen = 0;
		dp83820_rx_ptb[dp83820_curRxTraceEntry].cmdsts = cmdsts;
		dp83820_rx_ptb[dp83820_curRxTraceEntry].orgBuf = 0;
		eee_rtc(&(dp83820_rx_ptb[dp83820_curRxTraceEntry].t));
	}
	else {
		bufLen = cmdsts & DP83820_CMDSTS_LEN;
		cpyBuf = (char *)((uint32)buf | 0x20000000); //-- Convert to un-cached address
		
		//-- Figure out how many bytes to copy from pkt.
		if( bufLen > DP83820_BYTES_TRACE_MAX )
			capLen = DP83820_BYTES_TRACE_MAX;
		else
			capLen = bufLen;

		//-- Save len information and timestamp pkt
		dp83820_rx_ptb[dp83820_curRxTraceEntry].capLen = capLen;
		dp83820_rx_ptb[dp83820_curRxTraceEntry].pktLen = bufLen;
		dp83820_rx_ptb[dp83820_curRxTraceEntry].cmdsts = cmdsts;
		dp83820_rx_ptb[dp83820_curRxTraceEntry].orgBuf = buf;
		eee_rtc(&(dp83820_rx_ptb[dp83820_curRxTraceEntry].t));

		//-- Copy data from buffer
		capBuf = dp83820_rx_ptb[dp83820_curRxTraceEntry].bytes; 
		memcpy(capBuf, cpyBuf, capLen);
	}

	if( ++dp83820_curRxTraceEntry >= DP83820_TRACE_ENTRIES_MAX )
		dp83820_curRxTraceEntry = 0;
#endif

	return 0;
} // dp83820_rxBufCapture
