/*  
 *=============================================================================
 * File Name  :    dp83820-sim.c
 *
 * Description:    
 *
 * Created    :    30 April 2001
 *
 * Author     :    Frank Salzmann
 *
 * Copyright(c)  Agile Storage, Inc. 2001
 *=============================================================================
 */

#include "dp83820.h"

typedef struct _dp83820_sim_rx {
	dp83820_desc_t *start;
	dp83820_desc_t *end;
	dp83820_desc_t *cur;
} dpSim_rx_t;

typedef struct _dp83820_sim_tx {
	dp83820_desc_t *start;
	dp83820_desc_t *end;
	dp83820_desc_t *cur;
} dpSim_tx_t;

dpSim_rx_t sim_rx;
dpSim_tx_t sim_tx;

int
dp83820_sim_init( dp83820_cfg_t *dp )
{
	sim_rx.start = dp->rxc[0].rxqStart;
	sim_rx.cur   = dp->rxc[0].rxqStart;
	sim_rx.end   = dp->rxc[0].rxqEnd;

	sim_tx.start = dp->txc[0].txqStart;
	sim_tx.cur   = dp->txc[0].txqStart;
	sim_tx.end   = dp->txc[0].txqEnd;
	return 0;
} /* dp83820_sim_init */

/*
 *------------------------------------------------------------------------------
 * Function Name:	dp83820_sim_rx
 *
 * Description: 	Simulates the DP83820 packet receive function.
 *
 * Created by:		Frank Salzmann
 *
 * Data Created:	June 21, 2001
 * 					
 *------------------------------------------------------------------------------
 */
int
dp83820_sim_rx(char *data, int len)
{
	int  i;
	char *db;
	uint32 cmdsts;
	dp83820_desc_t *cur;


	cur = sim_rx.cur;
	/* Check for ownership */
	cmdsts = cur->cmdsts;
	if( cmdsts & DP83820_CMDSTS_OWN ) {
		printf("dp82830_sim_rx: Rxq full\n");
		return 1;
	}	

    /* Copy data out of message */
	db = cur->bufptr;
	if( !db ) {
		printf("dp82830_sim_rx: Null Buf Ptr\n");
		return 1;
	}
    for(i=0; i < len; i++)
        *db++ = *data++;

	/* Setup cmd status word */
	cmdsts = DP83820_CMDSTS_OWN | DP83820_CMDSTS_OK;
	cmdsts |= (len & DP83820_CMDSTS_LEN);
	cur->cmdsts = cmdsts;
	
	return(NFX_OK);
} /* dp83820_rxPacket */


