/*  
 *=============================================================================
 * File Name   :    dp83820.c
 *
 *  Description:    
 *
 *  Created    :    26 April 2001
 *
 *  Author     :    Frank Salzmann
 *
 *  Copyright(c)  Agile Storage, Inc. 2001
 *=============================================================================
 */
#include "dp83820.h"
#include <exports/mem-api.h>
#include <exports/malloc-api.h>

typedef int32 funcptr();
dp83820_cfg_t *dp83820;

extern uint32 *dp83820_cr;
uint32 pcitmp;
int32 dp83820_txUnicast(uint32, eee_desc_t *);
int dp83820_linkPoll( int32 tmr,           
					  dp83820_cfg_t *cfg,
					  void *user2,
					  void *user3,
					  void *user4 );
int dp83820_setMacSpeed(dp83820_cfg_t *);
int dp83820_readReg(volatile uint32 *); 
int dp83820_fillRxQ(dp83820_desc_t *, int); 
int dp83820_swap(uint32);
int dp83820_initQ(dp83820_desc_t *, int);
int dp83820_initDevice(void);
int dp83820_resetDevice(dp83820_cfg_t *); 
int dp83820_initRegs(dp83820_cfg_t *); 

extern int dp83820_mii_poll(dp83820_cfg_t *);
extern int dp83820_diagLbPktReceive(eee_desc_t *);
extern int dp83820_pciInit(int, int);
extern address_t cached_to_uncached(address_t);
extern int bp_getMyMacAddr(uint32, uint32, uint32, uint32, char *);
extern uint32 hs_read32(uint64);
extern void hs_write32(uint64, uint32);
extern void *eee_mgmtAlloc(uint32);

dp83820_pc_t dp83820_pc;
uint32 dp83820_curTxTraceEntry = 0;
uint32 dp83820_curRxTraceEntry = 0;
dp83820_ptb_t dp83820_tx_ptb[DP83820_TRACE_ENTRIES_MAX];
dp83820_ptb_t dp83820_rx_ptb[DP83820_TRACE_ENTRIES_MAX];

uint32 dp83820_snifferOn = 1;
uint32 dp83820_curRxSnifEntry = 0;
dp83820_psb_t dp83820_rx_psb[DP83820_SNIF_ENTRIES_MAX];

char *dp83820_txBufWatch[DP83820_TX_Q_LEN];

/*
 *------------------------------------------------------------------------------
 * Function Name:	dp83820_initModule
 *
 * Description: 	Initializes the DP83820 device driver module.
 *
 * Created by:		Frank Salzmann
 *
 * Data Created:	April 26, 2001
 * 					
 * Returns: 
 *
 *------------------------------------------------------------------------------
 */
int
dp83820_initModule()
{
	int rc;
	
	dp83820 = (dp83820_cfg_t *)eee_ramAlloc(sizeof(dp83820_cfg_t));
	if(!dp83820) {
		printf("[dp83820_initModule] Error: Malloc Failed: dp83820_cfg_t.\n");
		return(NFX_ERR);
	}
	memset(dp83820,0,sizeof(dp83820_cfg_t));
	
#ifdef LINUX_TEST
	dp83820_sim_init( dp83820 );
#endif

	memset( &dp83820_pc, 0, sizeof(dp83820_pc) );
	memset( dp83820_tx_ptb, 0, sizeof(dp83820_tx_ptb) );
	memset( dp83820_rx_ptb, 0, sizeof(dp83820_rx_ptb) );
	memset( dp83820_rx_psb, 0, sizeof(dp83820_rx_psb) );
	memset( dp83820_txBufWatch, 0, sizeof(dp83820_txBufWatch));

	/*
	 *--------------------------------------
	 * Register with EEE and get app id
	 *--------------------------------------
	 */
	rc = eee_allocateApplicationId("dp83820");
	if( rc == NFX_ERR )
	{
		printf("[dp83820_initModule] Error Allocate app id failed.");
		return NFX_ERR;
	}
	dp83820->appId = rc;

#if defined(NFX_MOD_NFP)
	dp83820->loopAddr = PACK_UNICAST_PORT_ADDRESS( nfxMySlotId, 
												   nfxMyCPUId,
												   dp83820->appId,
												   EEE_MGMT,
												   NFX_COS_HIGH, 
												   NFP_PORT_MGMT);
	bp_getMyMacAddr(nfxMySlotId, 
					nfxMyCPUId,
					1,
					EEE_MGMT,
					dp83820->macAddr);

	bp_getMyMacAddr(nfxMySlotId, 
					3, 				// Hopefully this is the FP cpu id.
					1,
					EEE_MGMT,
					dp83820->macAddr_fp);

#elif defined(NFX_MOD_FCNIM)
	dp83820->loopAddr = PACK_UNICAST_PORT_ADDRESS( nfxMySlotId, 
												   nfxMyCPUId,
												   dp83820->appId,
												   EEE_USER_DATA,
												   NFX_COS_HIGH, 
												   FCNIM_PORT_BP_DATA);
	bp_getMyMacAddr(nfxMySlotId, 
					nfxMyCPUId,
					1,
					EEE_USER_DATA,
					dp83820->macAddr);
#elif defined(PROM)

	//-- Used in prom only on the TXRX/FP board.
	rc = eee_allocateApplicationId("lucRcv");
	if( rc == NFX_ERR ) {
		printf("[dp83820_initModule] Error: Allocate IP app id failed.");
		return NFX_ERR;
	}
	dp83820->ipAppPort = PACK_UNICAST_PORT_ADDRESS( nfxMySlotId,
                                                     nfxMyCPUId,
                                                     rc,
                                                     EEE_MGMT,
                                                     NFX_COS_HIGH,
                                                     PROM_PORT_MGMT);

	dp83820->loopAddr = PACK_UNICAST_PORT_ADDRESS( nfxMySlotId, 
												   nfxMyCPUId,
												   dp83820->appId,
												   EEE_MGMT,
												   NFX_COS_HIGH, 
												   PROM_PORT_MGMT);
	bp_getMyMacAddr(nfxMySlotId, 
					nfxMyCPUId,
					1,
					EEE_MGMT,
					dp83820->macAddr);
#ifdef DIAG_R7000 
	dp83820->loopAddr = PACK_UNICAST_PORT_ADDRESS( nfxMySlotId, 
												   nfxMyCPUId,
												   dp83820->appId,
												   EEE_USER_DATA,
												   NFX_COS_HIGH, 
												   PROM_PORT_NET0);
	bp_getMyMacAddr(nfxMySlotId, 
					nfxMyCPUId,
					1,
					EEE_USER_DATA,
					dp83820->macAddr);

#endif	
#endif

	//-- Initialize the National DP83820 Mac
	if( dp83820_initDevice() == NFX_ERR ) {
		printf("DP83820 Initialize Module Failed\n");
		return 0;
	}

	rc = eee_registerReceiveFunc( dp83820->appId,	/* App Id		 */
								  dp83820_rxAppPkt,	/* Ucast Rx Func */
								  NULL);			/* Mcast Rx Func */
	if( rc == NFX_ERR )
	{
		printf("[dp83820_initModule] Error: registerReceiveFunc\n");
		return NFX_ERR;
	}

	eee_registerPollingFunc( EEE_HIGH_PRIORITY, dp83820_rxPktPoll, 0 );
	eee_registerPollingFunc( EEE_HIGH_PRIORITY, dp83820_txCleanPoll, 0 );

	//printf("Calling eee_registerTransmitFunc\n");

#if defined(NFX_MOD_NFP)
	rc = eee_registerTransmitFunc( NFP_PORT_MGMT,
								   dp83820_txUnicast,
								   dp83820_txUnicast,
								   1500);

	eee_registerDefaultTransmitFunc( EEE_MGMT,
									 dp83820_txUnicast,
									 1500);
#elif defined(PROM)
#if defined DIAG_R7000 

	rc = eee_registerTransmitFunc( PROM_PORT_NET0,
								   dp83820_txUnicast,
								   dp83820_txUnicast,
								   1500);
	rc = eee_registerTransmitFunc( EEE_USER_DATA,
								   dp83820_txUnicast,
								   dp83820_txUnicast,
								   9*1024);

	eee_registerDefaultTransmitFunc( EEE_USER_DATA,
									 dp83820_txUnicast,
									 9*1024);
#else
	/* Sibyte PROM */
	rc = eee_registerTransmitFunc( PROM_PORT_MGMT,
								   dp83820_txUnicast,
								   dp83820_txUnicast,
								   1500);

	eee_registerDefaultTransmitFunc( EEE_MGMT,
									 dp83820_txUnicast,
									 1500);
#endif	
#elif defined(NFX_MOD_FCNIM)
	rc = eee_registerTransmitFunc( FCNIM_PORT_BP_DATA,
								   dp83820_txUnicast,
								   dp83820_txUnicast,
								   9*1024);

	eee_registerDefaultTransmitFunc( EEE_USER_DATA,
									 dp83820_txUnicast,
									 9*1024);
	eee_registerDefaultTransmitFunc( EEE_SAC,
									 dp83820_txUnicast,
									 9*1024);

#ifdef EVAL_BOARD
	eee_registerDefaultTransmitFunc( EEE_CONTROL,
									 dp83820_txUnicast,
									 1500);
	eee_registerDefaultTransmitFunc( EEE_MGMT,
									 dp83820_txUnicast,
									 1500);
	rc = eee_registerTransmitFunc( FCNIM_PORT_BP_MGMT,
								   dp83820_txUnicast,
								   dp83820_txUnicast,
								   9*1024);
#endif

#endif
	if( rc == NFX_ERR )
		printf("Tx Func Register Error: %x\n", rc);

	return(NFX_OK);
} /* dp83820_initModule */


/*
 *------------------------------------------------------------------------------
 * Function Name:	dp83820_initDevice
 *
 * Description: 	Initializes the DP83820 device.
 *
 * Created by:		Frank Salzmann
 *
 * Data Created:	April 26, 2001
 * 					
 * Returns: 
 *
 *------------------------------------------------------------------------------
 */
int
dp83820_initDevice()
{
	uint32           i, rc;
	uint32           align_diff;
	uint32           size_rx;
	uint32           size_tx;
	uchar8           *qstart;
	dp83820_txCtrl_t *txc;
	dp83820_rxCtrl_t *rxc;

	//printf("[dp83820_initDevice] Enter\n");

#ifdef SIBYTE 
	dp83820->baseAddr = DP83820_BASE_ADDR;
	dp83820->regs 	 = (dp83820_regs_t *)(DP83820_BASE_ADDR);
#else

#if defined(AGILE_HARDWARE)			// R7K Based 
	dp83820->baseAddr = DP83820_BASE_ADDR;
	dp83820->regs 	 = (dp83820_regs_t *)(DP83820_BASE_ADDR);
#endif // AGILE_HARDWARE

#ifdef EVAL_BOARD
	dp83820->baseAddr = DP83820_BASE_ADDR | 0xB0000000;
	dp83820->regs 	 = (dp83820_regs_t *)(DP83820_BASE_ADDR | 0xB0000000);
#endif // EVAL_BOARD

#endif // SIBYTE

#ifndef LINUX_TEST
	rc = dp83820_pciInit(DP83820_BUSID, DP83820_DEVID);
	if( rc == NFX_ERR )
		return NFX_ERR;
#endif

	/*
	 * Allocate and align memory on 64-bit boundry
	 * for receive and transmit descriptor rings. 
	 */
	size_tx = sizeof(dp83820_desc_t) * DP83820_TX_Q_LEN;
	size_rx = sizeof(dp83820_desc_t) * DP83820_RX_Q_LEN;


	/* Setup descriptor rings */
	for( i=0; i < DP83820_TX_Q_CNT; i++ )
	{
		txc = &dp83820->txc[i];

#ifdef EEE_BUF_MGMT
		qstart = (uchar8 *)eee_mgmtAlloc(size_tx + 64);
#else
		qstart = (uchar8 *)eee_ramAllocAligned(size_tx + 64);
#endif

		//printf("txq start: %x: %d bytes allocated\n", qstart, size_tx+64);
		/* Convert to un-cached memory address */
#ifndef SIBYTE
#ifdef FC_USE_NEW_MEM_MAP
		qstart = (uchar8 *)cached_to_uncached((address_t)qstart);
#else
		DP83820_2UNCHACHED((address_t)qstart);
#endif
#endif // SIBYTE
		align_diff = (address_t)qstart & 0x00000007;

		if( align_diff )
			align_diff = 8 - align_diff;
		qstart += align_diff;

		memset(qstart, 0, size_tx);
		txc->txqStart = (dp83820_desc_t *)qstart;
		txc->txCur = txc->txCln = txc->txqStart;
		txc->txqEnd = txc->txqStart + DP83820_TX_Q_LEN;
		txc->avail = DP83820_TX_Q_LEN;
		dp83820_initQ( txc->txqStart, DP83820_TX_Q_LEN );
	}

	for( i=0; i < DP83820_RX_Q_CNT; i++ )
	{
		rxc = &dp83820->rxc[i];

#ifdef EEE_BUF_MGMT
		qstart = (uchar8 *)eee_mgmtAlloc(size_tx + 64);
#else
		qstart = (uchar8 *)eee_ramAllocAligned(size_rx + 64);
#endif

		//printf("rxq start: %x: %d bytes allocated\n", qstart, size_rx+64);
		/* Convert to un-cached memory address */
#ifndef SIBYTE
#ifdef FC_USE_NEW_MEM_MAP
		qstart = (uchar8 *)cached_to_uncached((address_t)qstart);
#else
		DP83820_2UNCHACHED((address_t)qstart);
#endif
#endif // SIBYTE
		memset(qstart, 0, size_rx);

		align_diff = (address_t)qstart & 0x00000007;
		if( align_diff )
			align_diff = 8 - align_diff;
		qstart += align_diff;

		rxc->rxqStart = (dp83820_desc_t *)(qstart);
		rxc->rxCur = rxc->rxqStart;
		rxc->rxqEnd = rxc->rxqStart + DP83820_RX_Q_LEN;
		rxc->avail = DP83820_RX_Q_LEN;
		dp83820_initQ( rxc->rxqStart, DP83820_RX_Q_LEN );
		dp83820_fillRxQ( rxc->rxqStart, DP83820_RX_Q_LEN);
	}

	/*
	 * Initialize the device.
	 */
	dp83820_resetDevice( dp83820 );
	dp83820_initRegs( dp83820 ); 

#ifdef EVAL_BOARD
	// Initialize the phy.
	dp83820_getPhyAddr(dp83820); 
	dp83820_phyInit(dp83820);
#endif


	//-- Create and start link timer.
	dp83820->link.timer = eee_timerCreate(DP83820_LINK_TIMER, 
										  EEE_TIMER_REPEAT);
	if(dp83820->link.timer == TIMER_ERR_NO_TIMER_AVAILABLE) {
		printf("DP83820: Could not allocate link timer.\n");
	}
	else {
		eee_timerStart(dp83820->link.timer,
					   (funcptr*)dp83820_linkPoll,           
					   dp83820, NULL, NULL, NULL);
	}

	//-- Put link into normal mode.
	dp83820->link.loopback = lb_none;
			
	//-- Mark MAC Id: 
	dp83820->macId = i;

	//printf("[dp83820_initDevice] Leave\n");
	return( NFX_OK ); 
} /* dp83820_initDevice */ 

/*
 *------------------------------------------------------------------------------
 * Function Name:	dp83820_initRegs
 *
 * Description: 	Initializes the DP83820 registers.
 *
 * Created by:		Frank Salzmann
 *
 * Data Created:	April 26, 2001
 * 					
 * Returns: 
 *
 *------------------------------------------------------------------------------
 */
int
dp83820_initRegs(dp83820_cfg_t *dp83820) 
{
	int i;
	ushort16  *addr;
	uint32 *reg;
	uint32 val;
	dp83820_regs_t *regs;
	dp83820_cr_t cr;
	dp83820_cfgr_t cfg;
	dp83820_rxcfgr_t rxcfg;
	dp83820_txcfgr_t txcfg;
	dp83820_rfcr_t rfcr;
	dp83820_pcr_t pcr;

	//printf("[dp83820_initRegs] Enter\n");
	regs = dp83820->regs;

	/* 
	 * Setup Config Register.
	 */
	cfg.wd = 0;
#if defined(FCNIM)
	cfg.bits.mode_1000 = 1;
	cfg.bits.data64_en = 1;
	cfg.bits.reqalg    = 0;
	cfg.bits.pesel     = 0;
	cfg.bits.brom_dis  = 1;
	cfg.bits.ext_125   = 1;
	cfg.bits.bem       = 0;
#elif defined (DIAG_R7000)	
	cfg.bits.mode_1000 = 1;
	cfg.bits.data64_en = 1;
	cfg.bits.reqalg    = 0;
	cfg.bits.pesel     = 1;
	cfg.bits.brom_dis  = 1;
	cfg.bits.ext_125   = 1;
	cfg.bits.bem       = 0;
#else		// NFX_MOD_NFP Runtime && Prom
	cfg.bits.mode_1000 = 0;
	cfg.bits.data64_en = 0;
	cfg.bits.reqalg    = 0;
	cfg.bits.pesel     = 1;
	cfg.bits.brom_dis  = 1;
	cfg.bits.ext_125   = 0;
	cfg.bits.bem       = 0;
#ifdef __long64
	cfg.bits.m64addr   = 1;
#endif
#endif // FCNIM
	dp83820_writeReg(&regs->cfg, cfg.wd);

	// Setup global pointer to control reg. Used later on by the transmit code.
	dp83820_cr = &regs->cr;

	/* 
	 * Disable Rx and Tx 
	 */
	cr.wd = 0;
	cr.bits.txd = 1;
	cr.bits.rxd = 1;
	dp83820_writeReg(&regs->cr, cr.wd);

	/*
	 * Setup Receive Config Register.
	 */
	rxcfg.wd = dp83820_readReg(&regs->rxcfg);
	rxcfg.bits.rx_fd    = 1;
	rxcfg.bits.alp      = 1;
	rxcfg.bits.airl     = 1;
	rxcfg.bits.stripcrc = 1;
	rxcfg.bits.mxdma    = 7; 	// 512 bytes
	rxcfg.bits.drth     = 31;	// Drain threshhold: 256 bytes.
	dp83820_writeReg(&regs->rxcfg, rxcfg.wd);

	/* 
	 * Setup Unicast Address register.
	 */
	addr = (ushort16 *)dp83820->macAddr;
	dp83820_writeReg(&regs->rfcr, 0);
	dp83820_writeReg(&regs->rfdr, *addr);
	addr++;
	dp83820_writeReg(&regs->rfcr, 2);
	dp83820_writeReg(&regs->rfdr, *addr);
	addr++;
	dp83820_writeReg(&regs->rfcr, 4);
	dp83820_writeReg(&regs->rfdr, *addr);
#if defined(NFX_MOD_NFP)
	addr = (ushort16 *)dp83820->macAddr_fp;
	dp83820_writeReg(&regs->rfcr, 0x0006); // Setup pattern match byte cnt
	dp83820_writeReg(&regs->rfdr, 0x0003); // Cnt 0: Match 3 bytes. cnt1 = 0
	dp83820_writeReg(&regs->rfcr, 0x0008); // Setup pattern match byte cnt
	dp83820_writeReg(&regs->rfdr, 0x0000); // Cnt 2 & 3: Match 0 bytes. 

	dp83820_writeReg(&regs->rfcr, 0x0200); // Write dst bytes
	dp83820_writeReg(&regs->rfdr, *addr);
	addr++;
	dp83820_writeReg(&regs->rfcr, 0x0202); // Write dst bytes
	dp83820_writeReg(&regs->rfdr, *addr);
	addr++;
	dp83820_writeReg(&regs->rfcr, 0x0204); // Write dst bytes
	dp83820_writeReg(&regs->rfdr, *addr);
#endif // NFP_FP
	
	//-- Setup Pause Control Register
	pcr.wd = 0xe3bc0c00;
#if 0
	pcr.wd = 0;
	pcr.bits.pause_cnt	= 0xfff0;
	dp83820_writeReg(&regs->pcr, pcr.wd);
	pcr.bits.psen		= 1;		// Enable Pause Receive
	pcr.bits.ps_fflo	= 3;		// Less than 4k in fifo
	pcr.bits.ps_ffhi	= 3;		// Room for >8k in fifo
//	pcr.bits.ps_stlo	= 3;		// Less than 4 pkt in fifo
//	pcr.bits.ps_sthi	= 3;		// Room for >8 pkt avail
	pcr.bits.ps_stlo	= 0;		// Less than 4 pkt in fifo
	pcr.bits.ps_sthi	= 0;		// Room for >8 pkt avail
	pcr.bits.ps_mcast	= 1;
	pcr.bits.ps_da		= 1;
#endif
	dp83820_writeReg(&regs->pcr, pcr.wd);

	/*
	 * Setup Receive Filter/Match Register.
	 */
	rfcr.wd = 0;
	if(CM_IS_FC_CPUID0) {   // cpuid 0 is fc slice 3 on slot 2
#ifdef DP83820_SNIFFER
		rfcr.bits.aab  = 1;			// Allow all broadcast packets
		rfcr.bits.aam  = 1;			// Allow all multicast packets
		rfcr.bits.aau  = 1;			// Allow all unicast packets
		rfcr.bits.apm  = 1;			// Accept on perfect match
		rfcr.bits.apat = 0x01;		// Accept on pattern match
#else
		rfcr.bits.aab  = 0;			// Allow all broadcast packets
		rfcr.bits.aam  = 0;			// Allow all multicast packets
		rfcr.bits.aau  = 0;			// Allow all unicast packets
		rfcr.bits.apm  = 1;			// Accept on perfect match
		rfcr.bits.apat = 0x01;		// Accept on pattern match
#endif
	}
	else {
		rfcr.bits.aab  = 0;			// Allow all broadcast packets
		rfcr.bits.aam  = 0;			// Allow all multicast packets
		rfcr.bits.aau  = 0;			// Allow all unicast packets
		rfcr.bits.apm  = 1;			// Accept on perfect match
		rfcr.bits.apat = 0x01;		// Accept on pattern match
	}

#ifdef PROM
    // always enable bcast in Prom
    rfcr.bits.aab  = 1;			    // Allow all broadcast packets
#endif

	dp83820_writeReg(&regs->rfcr, rfcr.wd);
	rfcr.bits.rfen = 1;			// Receive enable.
	dp83820_writeReg(&regs->rfcr, rfcr.wd);

	/*
	 * Initialize MIB Control Register.
	 */
	reg = &(regs->mibc);
	val = 0;
	dp83820_writeReg(reg, val);

	/*
	 * Initialize Pointer to Rx Desc Queue
	 */
	reg = &(regs->rxdp);
#ifdef LARGE_MEMORY
	val = virt_to_phys((address_t)(dp83820->rxc[0].rxqStart));
#else
#ifdef FC_USE_NEW_MEM_MAP
	val = virt_to_phys((address_t)(dp83820->rxc[0].rxqStart));
#else
	val = ((address_t)(dp83820->rxc[0].rxqStart) & ~V2PHYS_ADDR_MASK);
#endif
#endif
	dp83820_writeReg(reg, val);

	reg = &(regs->rxdp1);
	for( i=1; i < DP83820_RX_Q_CNT; i++, reg++ ) {
#ifdef LARGE_MEMORY
		val = virt_to_phys((address_t)(dp83820->rxc[i].rxqStart));
#else
#ifdef FC_USE_NEW_MEM_MAP
		val = virt_to_phys((address_t)(dp83820->rxc[i].rxqStart));
#else
		val = ((address_t)(dp83820->rxc[i].rxqStart) & ~V2PHYS_ADDR_MASK);
#endif
#endif
		val &= 0xFFFFFFF8;
		dp83820_writeReg(reg, val);
	}
	
	/*
	 * Initialize Pointer to Tx Desc Queue
	 */
	reg = &(regs->txdp);
#ifdef LARGE_MEMORY
	val = virt_to_phys((address_t)(dp83820->txc[0].txqStart));
#else
#ifdef FC_USE_NEW_MEM_MAP
	val = virt_to_phys((address_t)(dp83820->txc[0].txqStart));
#else
	val = ((address_t)(dp83820->txc[0].txqStart) & ~V2PHYS_ADDR_MASK);
#endif
#endif
	dp83820_writeReg(reg, val);

	reg = &(regs->txdp1);
	for( i=1; i < DP83820_TX_Q_CNT; i++, reg++ ) {
#ifdef LARGE_MEMORY
		val = virt_to_phys((address_t)(dp83820->txc[i].txqStart));
#else
#ifdef FC_USE_NEW_MEM_MAP
		val = virt_to_phys((address_t)(dp83820->txc[i].txqStart));
#else
		val = ((address_t)(dp83820->txc[i].txqStart) & ~V2PHYS_ADDR_MASK);
#endif
#endif
		val &= 0xFFFFFFF8;
		dp83820_writeReg(reg, val);
	}

	//-- Initialize the TX CFG register.
	//val = 0xD00001E0;
#if 0
	txcfg.wd = 0;
	txcfg.bits.csi   = 1;
	txcfg.bits.hbi   = 1;
	txcfg.bits.atp   = 1;		// Automatic transmit padding.
	txcfg.bits.mxdma = 3;		// 32 bytes
	if( isTXRX() )
		txcfg.bits.flth  = 2;
	else
		txcfg.bits.flth  = 1;
	txcfg.bits.drth  = 0xFC;	// 8064 bytes
#else
	txcfg.wd = 0;
	txcfg.bits.csi   = 1;
	txcfg.bits.hbi   = 1;
	txcfg.bits.atp   = 1;		// Automatic transmit padding.
	txcfg.bits.mxdma = 5;		// 256 bytes
	txcfg.bits.drth  = 0xD0;	 
	if( isTXRX() )
		txcfg.bits.flth  = 2;
	else
		txcfg.bits.flth  = 0x29;
#endif

	dp83820_writeReg(&regs->txcfg, txcfg.wd);

	/*
	 * Set MAC loopback mode. ---- DEBUG 
	 */
#if 0	
	val = dp83820_readReg(&regs->txcfg); 
	val |= 0x20000000;
	dp83820_writeReg(&regs->txcfg, val);
#endif	

	/* 
	 * Enable Rx 
	 */
	cr.wd = 0;
	cr.bits.rxe = 1;
	dp83820_writeReg(&regs->cr, cr.wd);

	//printf("[dp83820_initRegs] Leave\n");
	return(NFX_OK);
} /* dp83820_initRegs */

/*
 *------------------------------------------------------------------------------
 * Function Name:	dp83820_resetDevice
 *
 * Description: 	Resets the device.
 *
 * Created by:		Frank Salzmann
 *
 * Data Created:	April 26, 2001
 * 					
 * Returns: 
 *
 *------------------------------------------------------------------------------
 */
int
dp83820_resetDevice(dp83820_cfg_t *dp83820) 
{
	int i;
	volatile dp83820_regs_t *regs;
	dp83820_cr_t cr;

	//printf("[dp83820_resetDevice] Enter\n");
	regs = dp83820->regs;

	cr.wd  = 0;
	cr.bits.rst = 1;
	cr.bits.rxr = 1;
	cr.bits.txr = 1;
	dp83820_writeReg((volatile uint32 *)&regs->cr, cr.wd);
	for(i=0; i < 10000; i++) {
		cr.wd = dp83820_readReg((volatile uint32 *)&regs->cr);
		if( cr.bits.rst == 0 ) {
			//printf("reset done: %08x\n", cr.wd);
			break;
		}
	}
	if (i == 10000) {
		panic("dp83820_resetDevice: chip reset failed!\n");
	}
	//printf("[dp83820_resetDevice] Leave\n");
	return(NFX_OK);
} /* dp83820_resetDevice */

/*
 *------------------------------------------------------------------------------
 * Function Name:	dp83820_resetModule
 *
 * Description: 	Resets the module and devce.
 *
 * Created by:		Frank Salzmann
 *
 * Data Created:	April 26, 2001
 * 					
 * Returns: 
 *
 *------------------------------------------------------------------------------
 */
int
dp83820_resetModule(dp83820_cfg_t *dp83820) 
{
	return 0;
} /* dp83820_resetModule */

/*
 *------------------------------------------------------------------------------
 * Function Name:	dp83820_initQ
 *
 * Description: 	Initializes descriptor queue by carving up the 
 *					descriptor memory into individual descriptors and 
 *					linking them together.
 *
 * Created by:		Frank Salzmann
 *
 * Data Created:	April 26, 2001
 *------------------------------------------------------------------------------
 */
int
dp83820_initQ(dp83820_desc_t *qstart, int qLen)
{
	int i;
	uint32 val;
	dp83820_desc_t *d;

	d  = qstart;
	/* Convert to physical address space */
	for(i=1; i < qLen; i++, d++) {
#ifdef LARGE_MEMORY
		val = virt_to_phys((address_t)(d+1));
#else
#ifdef FC_USE_NEW_MEM_MAP
		val = virt_to_phys((address_t)(d+1));
#else
		val = (address_t)((address_t)(d+1) & ~V2PHYS_ADDR_MASK);
#endif
#endif
		val = dp83820_swap(val);
		d->link = (dp83820_desc_t *)((address_t)val);
	}
#ifdef LARGE_MEMORY
	val = virt_to_phys((address_t)qstart);
#else
#ifdef FC_USE_NEW_MEM_MAP
	val = virt_to_phys((address_t)qstart);
#else
	val = (address_t)qstart & ~V2PHYS_ADDR_MASK;
#endif
#endif
	val = dp83820_swap(val);
	d->link = (dp83820_desc_t *)((address_t)val);
	return(NFX_OK);
} /* dp83820_initQ */

/*
 *------------------------------------------------------------------------------
 * Function Name:	dp83820_fillRxQ
 *
 * Description: 	Fills a receive queue with empty system data uffers.
 *
 * Created by:		Frank Salzmann
 *
 * Data Created:	April 26, 2001
 *------------------------------------------------------------------------------
 */
int
dp83820_fillRxQ(dp83820_desc_t *rxQ, int qLen) 
{
	int i;
	char *pBuf;
	uint32 val;
	dp83820_desc_t *d;

	//printf("[dp83820_fillRxQ] Enter\n");
	d = rxQ;
	for(i=0; i < qLen; i++, d++)
	{
		pBuf = eee_allocateBuffer(EEE_SHARED_POOL, DP83820_BUFFER_SIZE, 0);
		if( !pBuf ) {
			printf("[dp83820_fillRxQ]: Error: Cannot allocate free buffer\n");
			return(NFX_ERR);
		}
		//		(uint32)pBuf |= 0x20000000;        /* Convert to un-cached addr */
#ifdef SIBYTE
		pBuf = (char *)VIRT_TO_PHYS((address_t)pBuf);
#else
#ifdef FC_USE_NEW_MEM_MAP
		pBuf = (char *)virt_to_phys((address_t)pBuf);
#else
		(address_t)pBuf &= ~V2PHYS_ADDR_MASK;         /* Convert to physical addr  */
#endif
#endif
		val = (address_t)pBuf;
		val = dp83820_swap(val);
		d->bufptr = (uchar8 *)((address_t)val);
		val = dp83820_swap((DP83820_RX_BUF_SIZE & DP83820_CMDSTS_LEN));  
		d->cmdsts = val | DP83820_CMDSTS_MORE;
	}
	//printf("[dp83820_fillRxQ] Leave\n");
	return(NFX_OK);
} /* dp83820_fillRxQ */


/*
 *------------------------------------------------------------------------------
 * Function Name:	dp83820_writeReg
 *
 * Description: 	Writes 32-bit value to a device register.
 *
 * Created by:		Frank Salzmann
 *
 * Data Created:	April 30, 2001
 *------------------------------------------------------------------------------
 */
int
dp83820_writeReg(volatile uint32 *reg, uint32 val) 
{
#ifdef SIBYTE
	uint64 addr;
#endif
#ifdef LINUX_TEST
#else
#ifdef SIBYTE
	addr = 0x9000000000000000 + (uint64)reg;
	hs_write32(addr, val);
#else		
	*reg = val;
#endif

#endif
	return NFX_OK;
} /* dp83820_writeReg */
/*
 *------------------------------------------------------------------------------
 * Function Name:	dp83820_readReg
 *
 * Description: 	Returns 32-bit value read from device register.
 *
 * Created by:		Frank Salzmann
 *
 * Data Created:	April 30, 2001
 *------------------------------------------------------------------------------
 */
int
dp83820_readReg(volatile uint32 *reg) 
{
	uint32 val;
#ifdef SIBYTE
	uint64 addr;
#endif
#ifdef LINUX_TEST
	return 0;
#else

#ifdef SIBYTE
	addr = 0x9000000000000000 + (uint64)reg;
	val = hs_read32(addr);
#else		
	val = *reg;
#endif // SIBYTE
	return(val);
#endif
} /* dp83820_readReg */

int
dp83820_swap( uint32 tmp )
{
	uint32 val;
#if 0
	val = ((tmp & 0xFF000000) >> 24) | ((tmp & 0x00FF0000) >> 8 ) |
		  ((tmp & 0x0000FF00) << 8 ) | ((tmp & 0x000000FF) << 24);
#else
	val = tmp;
#endif
	return(val);
} /* dp83820_swap */

/*
 *------------------------------------------------------------------------------
 * Function Name:	dp83820_rxAppPkt
 *
 * Description: 	Receive packet sent to DP83820 mgmt layer.
 *
 * Created by:		Frank Salzmann
 *
 * Data Created:	May 12, 2001
 *------------------------------------------------------------------------------
 */
void
dp83820_rxAppPkt( eee_desc_t *edesc )
{
	dp83820_diagLbPktReceive(edesc);
	return;
} /* dp83820_rxAppPkt */

int
dp83820_wait( int msec )	
{

	return 0;
} /* dp83820_wait */

/*
 *------------------------------------------------------------------------------
 * Function Name:	dp83820_linkPoll
 *
 * Description: 	Polls for ethernet link status and statistics. 
 *
 * Created by:		Frank Salzmann
 *
 * Data Created:	November 04, 2001
 *------------------------------------------------------------------------------
 */
int
dp83820_linkPoll( int32 tmr,                /* Timer Handle */
				  dp83820_cfg_t *cfg,
				  void *user2,
				  void *user3,
				  void *user4 ) 
{
#if defined(AGILE_HARDWARE)  || defined(PROM)
	return 0; // Don't poll phy on real hardware.
#endif // AGILE_HARDWARE

	if( cfg->link.loopback == lb_int )
		return 0;

	//-- Look for changes in physical layer configuration.
	if( dp83820_mii_poll( cfg ) ) {
		dp83820_setMacSpeed( cfg );
	}
	return(NFX_OK);
} // dp83820_linkPoll
/*
 *------------------------------------------------------------------------------
 * Function Name:	dp83820_setMacSpeed
 *
 * Description: 	Sets up the MAC to the speed auto-detected by the
 *					physical layer.
 *
 * Created by:		Frank Salzmann
 *
 * Data Created:	October 11, 2001
 * 					
 * Returns: 		NFX_OK
 *------------------------------------------------------------------------------
 */
int 
dp83820_setMacSpeed(dp83820_cfg_t *dp)
{
	dp83820_cfgr_t cfg;
	dp83820_regs_t *r;
	dp83820_rxcfgr_t rxcfg;

	return 0; // FSDEBUG

	r = dp->regs;

	//-- Read current register values 
    cfg.wd    = dp83820_readReg(&r->cfg);
    rxcfg.wd  = dp83820_readReg(&r->rxcfg);

	//-- Setup appropriate bits in cfg register.
    switch( dp->link.speed ) 
	{
		case speed_10:
		case speed_100:
			cfg.bits.mode_1000		= 0;
			break;

		case speed_1000:
			cfg.bits.mode_1000		= 1;
			break;

		default:
			return NFX_OK;
    }
	
	//-- If Phy FDX set then set the MAC rx-fdx bit.
	if( cfg.bits.dupsts == 0 ) {
		rxcfg.bits.rx_fd = 1;
		dp83820_writeReg(&r->txcfg, 0xC00001E0);
	}
	else {
		dp83820_writeReg(&r->txcfg, 0x000001E0);
		rxcfg.bits.rx_fd = 0;
	}

    //-- Write bits back to Mac registers.
    dp83820_writeReg(&r->cfg, cfg.wd);
    dp83820_writeReg(&r->rxcfg, rxcfg.wd);

	return NFX_OK;
} // dp83820_setMacSpeed
