/*-----------------------------------------------------------------
 *
 * Name: ds1511-wd.c
 *
 * Copyright (C) 2004, OnStor Inc.
 *
 * Description: DS1511 Watchdog utility routines
 *
 * Created by:	Shaun Hui
 *
 * Date Created: 08/04/2004
 *
 *-----------------------------------------------------------------
 */


#include <nfx-incl.h>
#include <exports/ds1511.h>

extern uint32   watchdog_state;

void
ds1511_init(void)
{
	uchar8	reg;

	// Enable burst mode & power
	reg = DS1511_CONTROL_B_BME | DS1511_CONTROL_B_TPE;
	DS1511_WRITE_REG(DS1511_CONTROL_B, reg);

	// Clear WD counters
	DS1511_WRITE_REG(DS1511_WATCHDOG_SEC, 0);
	DS1511_WRITE_REG(DS1511_WATCHDOG_MSEC, 0);

	// Clear all flags in control A
	reg  = DS1511_READ_REG(DS1511_CONTROL_A);
	reg &= ~(DS1511_CONTROL_A_TDF|DS1511_CONTROL_A_KSF|
			 DS1511_CONTROL_A_WDF|DS1511_CONTROL_A_IRQF );
	DS1511_WRITE_REG(DS1511_CONTROL_A, reg);
                                                                                
	// Enable transfer 
	reg  = DS1511_READ_REG(DS1511_CONTROL_B);
	reg |= DS1511_CONTROL_B_TE;
	DS1511_WRITE_REG(DS1511_CONTROL_B, reg);

}

#ifdef PROM_R9000

void
wd_enable( uint32  nsec )
{

	if (!watchdog_state)
	{
		uchar8	reg;

		// Read flags from control A reg
		// Turn off WDF & IRQF bits so that
		// NMI is not generated when WD is re-enabled

		reg  = DS1511_READ_REG(DS1511_CONTROL_A);
		reg &= ~(DS1511_CONTROL_A_WDF | DS1511_CONTROL_A_IRQF);
		DS1511_WRITE_REG(DS1511_CONTROL_A, reg);

		// Write WD timeout value
		// Upper 4 bits for 10 seconds
		// Lower 4 bits for  1 seconds

		reg = ((nsec/10 & 0x0F) << 4 | (nsec % 10));
		DS1511_WRITE_REG(DS1511_WATCHDOG_SEC, reg);

		// Enable WD.

		reg = DS1511_READ_REG(DS1511_CONTROL_B) | DS1511_CONTROL_B_WDE;
		DS1511_WRITE_REG(DS1511_CONTROL_B, reg);
	}
}

void
wd_disable( void )
{
	if (watchdog_state)
	{
		uchar8	reg;

		// Turn off WDE bit

		reg = DS1511_READ_REG(DS1511_CONTROL_B) & ~DS1511_CONTROL_B_WDE;
		DS1511_WRITE_REG(DS1511_CONTROL_B, reg);
	}
}

void
wd_kick( void )
{
	if (watchdog_state)
	{
		uchar8	reg;

		// Kick WD by writing to WD sec reg. 
		reg = DS1511_READ_REG(DS1511_WATCHDOG_SEC);
		DS1511_WRITE_REG(DS1511_WATCHDOG_SEC, reg);
	}
}

void
wd_init(void)
{

	wd_disable();

	// Clear WD sec & msec registers.

	DS1511_WRITE_REG(DS1511_WATCHDOG_MSEC, 0);
	DS1511_WRITE_REG(DS1511_WATCHDOG_SEC, 0);
}
#endif
