/*
 *-----------------------------------------------------------------
 *
 * Name:	eee-desc.c
 *
 * RCS:  $Id: eee-desc.c,v 1.75 2002/12/07 00:02:01 rick Exp $
 *
 * Copyright (C) 2001, Agile Storage, Inc.
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:
 *
 *-----------------------------------------------------------------
 */

#define __EEE_DESC_COMPILE

#include <nfx-incl.h>
#include <exports/eee-api.h>
#include <exports/cm-api.h>
#include "eee-validate.h"
#include <sys/machine-dep.h>
#include <exports/malloc-api.h>
#include <exports/mem-api.h>


/*-----------------------------------------------------------------
 * Name :	eee_getBufferAllocAddr()
 *
 * Description: Return the address of function which allocated the
 * buffer.
 *
 * Created by:	Maxim Kozlovsky
 *
 * Date Created:	09/06/02
 *
 *-----------------------------------------------------------------
 */
address_t
eee_getBufferAllocAddr(
	void	*bufPtr		/* address within buffer */
)
{
#if defined(EEE_VALIDATE)
	uint32	i, j;
	address_t	buf_base;

	for (i=0; i<EEE_NUM_MEM_POOLS; i++) {
		for (j=0; j<EEE_NUM_BUF_SIZES; j++) {
			if ((bufPtr >= eee.buf_base[i][j]) &&
											(bufPtr < eee.buf_end[i][j])) {
				buf_base = EEE_BUF_BASE(bufPtr, i, j);
				if ((i == 0) || (i == eee.my_pool_id - SHARED_POOL_OFFSET)) {
					uint32 index =
						((uint32)buf_base
						 - (uint32)(address_t)eee.buf_base[i][j])
						/ eee.buf_size[j];
					uint32 vpool = i == 0 ? 0 : 1;
					return eee_buf_validate[vpool][j][index].alloc;
				}
				else {
					/* buffer from other CPU */
					return 0;
				}
			}
		}
	}

	return 0;
#else
	return 0;
#endif
}	/* eee_deallocateBuffer() */


#define REMOTE_SHARED_POOL (EEE_REMOTE_POOL - SHARED_POOL_OFFSET)

/*-----------------------------------------------------------------
 * Name:	eee_deallocateBuffer(void * bufPtr)
 *
 * Description:	Return buffer to free pool
 *
 *				If this is one of my buffers, then add it back onto
 *				either the local pool's free stack, or the shared pool's
 *				free stack, depending on the address.
 *				If it is not one of mine, then deliver it to the rightful
 *				owner.
 *
 * Created by:	Rick Lund
 *
 * Date Created:	4/9/01
 *
 *-----------------------------------------------------------------
 */
#if defined(EEE_VALIDATE)
void
eee_deallocateBufferRA(
	void	*bufPtr,		/* address within buffer to be released */
	address_t	ra
)
#else
void
eee_deallocateBuffer(void *bufPtr)
#endif
{
#if defined(FCNIM) && (EEE_NUM_SM_POOLS == 0)
    test_eee_deallocateBuffer(bufPtr);
#else

	address_t	buf_base;
	int size __attribute__((unused));

#if defined (NFP_FP) || defined (FEATURE_DIAG_PROM)
	if (mm_isMallocBuffer(bufPtr)) {
		eee_ramDealloc(bufPtr);
		return;
	}
#endif

#ifdef EEE_ATTR
	if (!(eee_attrOkToFreeBuf(bufPtr))) {
		eee_attrDisplay(bufPtr);
		panic("eee_attr failed in deallocateBufferRA, buf = %p, ra = %p\n", bufPtr, ra);
	}
	eee_attrSetBufValidFreeAttr(bufPtr, 0, 0, 0, 0);
#endif

#if defined(NFP_FP) || defined(NFP_TXRX)
	if (!eee_isRemoteAddr((address_t)bufPtr))
#endif
	{
		eee_stack_pool_cntl_t *pool;

#if defined(EEE_BUF_MGMT)
		if (bufPtr < eee.buf_end[EEE_MY_SHARED_POOL][EEE_BUF_MGMT]) {
			buf_base = (address_t)bufPtr -
				(uint32)(address_t)bufPtr % EEE_BUFSIZE_MGMT;
			pool = &eee.buf_pool[EEE_MY_SHARED_POOL][EEE_BUF_MGMT].stack;
			size = EEE_BUF_MGMT;
			CACHE_INVALIDATE_STRUCT(buf_base, EEE_BUFSIZE_MGMT);
		}
		else
#endif
		if (bufPtr < eee.buf_end[EEE_MY_SHARED_POOL][EEE_BUF_SMALL]) {
			buf_base = (address_t)bufPtr & ~(EEE_BUFSIZE_SMALL - 1);
			pool = &eee.buf_pool[EEE_MY_SHARED_POOL][EEE_BUF_SMALL].stack;
			size = EEE_BUF_SMALL;
			CACHE_INVALIDATE_STRUCT(buf_base, EEE_BUFSIZE_SMALL);
		}
		else if (bufPtr < eee.buf_end[EEE_MY_SHARED_POOL][EEE_BUF_LARGE]) {
			buf_base = (address_t)bufPtr -
				(uint32)(address_t)bufPtr % EEE_BUFSIZE_LARGE;
			pool = &eee.buf_pool[EEE_MY_SHARED_POOL][EEE_BUF_LARGE].stack;
			size = EEE_BUF_LARGE;
			CACHE_INVALIDATE_STRUCT(buf_base, EEE_BUFSIZE_LARGE);
		}
		else if (bufPtr < eee.buf_end[EEE_MY_SHARED_POOL][EEE_BUF_ULTRA]) {
			buf_base = (address_t)bufPtr -
				(uint32)(address_t)bufPtr % EEE_BUFSIZE_ULTRA;
			pool = &eee.buf_pool[EEE_MY_SHARED_POOL][EEE_BUF_ULTRA].stack;
			size = EEE_BUF_ULTRA;
			CACHE_INVALIDATE_STRUCT(buf_base, EEE_BUFSIZE_ULTRA);
		}
		else {
			*(volatile uint64 *)0 = 0;
			return;
		}

#if defined(EEE_VALIDATE)
		if (buf_base < (address_t)eee.buf_base[EEE_MY_SHARED_POOL][size]) {
			panic("invalid buf %p (%p %p) sz %d\n",
				  buf_base,
				  eee.buf_base[EEE_MY_SHARED_POOL][size],
				  eee.buf_end[EEE_MY_SHARED_POOL][size],
				  size);
		}
		EEE_VALIDATE_FREE_BUF(buf_base, EEE_MY_SHARED_POOL, size, ra);
#endif
		NFX_SPIN_LOCK(&eee_bufpool_spin);
		*(--pool->top) = buf_base;
		++pool->num_avail;

		eee_attrAssignBuf((void *)buf_base, EEE_ATTR_FREE);
		NFX_SPIN_UNLOCK(&eee_bufpool_spin);
	}
#if defined(NFP_FP) || defined(NFP_TXRX)
	else {
#if defined(EEE_BUF_MGMT)
		if (bufPtr < eee.buf_end[REMOTE_SHARED_POOL][EEE_BUF_MGMT]) {
			buf_base = (address_t)bufPtr -
				(uint32)(address_t)bufPtr % EEE_BUFSIZE_MGMT;
			size = EEE_BUF_MGMT;
			CACHE_INVALIDATE_STRUCT(buf_base, EEE_BUFSIZE_MGMT);
		}
		else
#endif			
		if (bufPtr < eee.buf_end[REMOTE_SHARED_POOL][EEE_BUF_SMALL]) {
			buf_base = (address_t)bufPtr & ~(EEE_BUFSIZE_SMALL - 1);
			size = EEE_BUF_SMALL;
			CACHE_INVALIDATE_STRUCT(buf_base, EEE_BUFSIZE_SMALL);
		}
		else if (bufPtr < eee.buf_end[REMOTE_SHARED_POOL][EEE_BUF_LARGE]) {
			buf_base = (address_t)bufPtr -
				(uint32)(address_t)bufPtr % EEE_BUFSIZE_LARGE;
			size = EEE_BUF_LARGE;
			CACHE_INVALIDATE_STRUCT(buf_base, EEE_BUFSIZE_LARGE);
		}
		else if (bufPtr < eee.buf_end[REMOTE_SHARED_POOL][EEE_BUF_ULTRA]) {
			buf_base = (address_t)bufPtr -
				(uint32)(address_t)bufPtr % EEE_BUFSIZE_ULTRA;
			size = EEE_BUF_ULTRA;
			CACHE_INVALIDATE_STRUCT(buf_base, EEE_BUFSIZE_ULTRA);
		}
		else {
			*(volatile uint64 *)0 = 0;
			return;
		}

#if defined(EEE_VALIDATE)
		if (buf_base < (address_t)eee.buf_base[REMOTE_SHARED_POOL][size]) {
			panic("invalid buf %p (%p %p) sz %d\n",
				  buf_base,
				  eee.buf_base[REMOTE_SHARED_POOL][size],
				  eee.buf_end[REMOTE_SHARED_POOL][size],
				  size);
		}
#endif
		ATOMIC_INC_INT(&eee_stats->fwd_free_buf[EEE_REMOTE_QUEUE_INDEX], 1);
		CACHE_INVALIDATE_STRUCT(buf_base, eee.buf_size[j]);

		eee_attrAssignBuf((void *)buf_base, EEE_ATTR_FWD_TO_FREE);

		NFX_SPIN_LOCK(&eee_fwdqueue_spin);
		if (FWD_PACKET(buf_base | EEE_FREE_BUF
					   | (size << EEE_FWD_BUF_SIZE_SHIFT),
					   EEE_REMOTE_QUEUE_INDEX) != EEE_FORWARDED_PACKET) {
#if defined(EEE_VALIDATE)			
			printf("No forwarding entry available to free buf, "
				   "buf = %lx, ra = %lx\n", buf_base, ra);
#endif			
		}
		NFX_SPIN_UNLOCK(&eee_fwdqueue_spin);
	}
#endif	
#endif
}	/* eee_deallocateBuffer() */


/*-----------------------------------------------------------------
 * Name:	eee_deallocateDesc(eee_descPtr_t edesc)
 *
 * Description:	Releases an EEE descriptor back into the free pool.
 *
 *				If this is one of my descriptors, then add it back onto
 *				either the local pool's free stack, or the shared pool's
 *				free stack, depending on the address.
 *				If it is not one of mine, then deliver it to the rightful
 *				owner.
 *
 * Created by:	Rick Lund
 *
 * Date Created:	4/9/01
 *
 *-----------------------------------------------------------------
 */
#if defined(EEE_VALIDATE)
void
eee_deallocateDescRA(
	eee_descPtr_t	edesc,		/* pointer to descriptor to free */
	address_t		ra
)
#else
void
eee_deallocateDesc(eee_descPtr_t edesc)
#endif	
{
	(address_t)edesc &= ~EDESC_ADDR_MASK;

#ifdef EEE_ATTR
	if (!(eee_attrOkToFreeDesc(edesc))) {
		eee_attrDisplay(edesc);
		panic("eee_attr failed in deallocateDescRA, edesc = %p, ra = %p\n", edesc, ra);
	}
	eee_attrSetDescValidFreeAttr(edesc, 0, 0, 0, 0);
#endif

#if defined(NFP_FP) || defined(NFP_TXRX)
	if (eee_isLocalAddr((address_t)edesc))
#endif
	{
		eee_stack_pool_cntl_t *pool = &eee.desc_pool[EEE_MY_SHARED_POOL].stack;
#ifdef EEE_VALIDATE
		if (edesc < eee.desc_base[EEE_MY_SHARED_POOL]
			|| edesc >= eee.desc_end[EEE_MY_SHARED_POOL]) {
			panic("descriptor %p is not shared descriptor (%p %p)\n",
				  edesc, eee.desc_base[EEE_MY_SHARED_POOL],
				  eee.desc_end[EEE_MY_SHARED_POOL]);
		}
		EEE_VALIDATE_FREE_DESC(edesc, EEE_MY_SHARED_POOL, ra);
#endif
		edesc->hdr.next = 0;
		edesc->hdr.offset = 0;
		
		NFX_SPIN_LOCK(&eee_descpool_spin);

		*(--pool->top) = (address_t)edesc;
		++pool->num_avail;

#ifdef EEE_ATTR
		eee_attrAssignDesc(edesc, EEE_ATTR_FREE);
#endif
		NFX_SPIN_UNLOCK(&eee_descpool_spin);
	}
#if defined(NFP_FP) || defined(NFP_TXRX)
	else if (eee_isRemoteCachedAddr((address_t)edesc)) {
		__asm__("cache %0,0(%1)" : : "i"(Hit_Invalidate_D),"r"(edesc));
		__asm__("cache %0,32(%1)" : : "i"(Hit_Invalidate_D),"r"(edesc));
		__asm__("cache %0,64(%1)" : : "i"(Hit_Invalidate_D),"r"(edesc));
		__asm__("cache %0,96(%1)" : : "i"(Hit_Invalidate_D),"r"(edesc));
		edesc = (eee_descPtr_t)
			(((address_t)edesc - REMOTE_CACHED_MEM) + REMOTE_MEM);
		goto free_uncached;
	}
	else 
	{
		int rc __attribute__((unused));
	free_uncached:
		ATOMIC_INC_INT(&eee_stats->fwd_free_desc[EEE_REMOTE_QUEUE_INDEX], 1);
#ifdef EEE_ATTR
		eee_attrAssignDesc(edesc, EEE_ATTR_FWD_TO_FREE);
#endif
		NFX_SPIN_LOCK(&eee_fwdqueue_spin);
		rc = FWD_PACKET((address_t)edesc | EEE_FREE_DESC,
						EEE_REMOTE_QUEUE_INDEX);
		NFX_SPIN_UNLOCK(&eee_fwdqueue_spin);
#if defined(EEE_VALIDATE)
		if (edesc < eee.desc_base[EEE_REMOTE_POOL - SHARED_POOL_OFFSET]
			|| edesc >= eee.desc_end[EEE_REMOTE_POOL - SHARED_POOL_OFFSET]) {
			panic("desciptor %p is not remote descriptor (%p %p)\n",
				  edesc, eee.desc_base[EEE_REMOTE_POOL - SHARED_POOL_OFFSET],
				  eee.desc_end[EEE_REMOTE_POOL - SHARED_POOL_OFFSET]);
		}
		if (rc != EEE_FORWARDED_PACKET) {
			printf("No forwarding entry available to free desc\n");
		}
#endif
	}
#endif	
}	/* eee_deallocateDesc() */


#if defined(EEE_VALIDATE)
/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:
 *
 *-----------------------------------------------------------------
 */
void
eee_deallocateDesc(
	eee_descPtr_t	edesc		/* pointer to descriptor to free */
)
{
	address_t	ra;

	ra = (address_t)get_ra();

	eee_deallocateDescRA(edesc, ra);
}
#endif


/*-----------------------------------------------------------------
 * Name:	eee_freePacket(eee_descPtr_t edesc)
 *
 * Description:	Free an EEE descriptor and the associated buffers
 *
 * Created by:	Rick Lund
 *
 * Date Created:	4/9/01
 *
 *-----------------------------------------------------------------
 */
void
eee_freePacket(
	eee_descPtr_t	edesc		/* pointer to packet to free */
)
{
	eee_descBuf_t	* bd;
	uint32			num_bufs, i;
	uint32			free_rout_ind;
	uint32			cont;

	while (edesc != NULL) {
		eee_descPtr_t next_desc = eee_nextDesc(edesc);
		eee_unlinkNextDesc(edesc);
		cont = edesc->hdr.control;
		free_rout_ind = (cont & EDESC_FREE_ROUTINE_INDEX) >>
			EDESC_FREE_ROUTINE_SHIFT;
		if (free_rout_ind)
			(*eee.free_routine[free_rout_ind-1])(edesc);

		if (!(cont & EDESC_PKT_NO_DEALLOC)) {
			bd = &(edesc->bd[0]);
			num_bufs = EEE_GET_NUM_BUFS(edesc);

			if (!(edesc->hdr.control & EDESC_COMMAND_USE_FIRST_BUFFER))
				bd++;

			for (i=0; i<num_bufs; i++, bd++) {
				if ((bd->control & EDESC_BD_CMD_IN_USE) &&
					!(bd->control & EDESC_BD_CMD_NO_DEALLOC))
					eee_deallocateBufferRA(bd->buf, get_ra());
			}
			eee_deallocateDescRA(edesc, get_ra());
		}
		edesc = next_desc;
	}
}	/* eee_freePacket() */


/*-----------------------------------------------------------------
 * Name :	eee_nextBuf()
 *
 * Description: Move the pointers to the next buffer. The new descriptor
 * is stored in the location specified by the argument edesc.
 *
 * Returns: new buffer descriptor or NULL is this was the last buffer
 * descriptor.
 *
 * Created by:	Maxim Kozlovsky
 *
 * Date Created:	04/24/02
 *
 *-----------------------------------------------------------------
 */
eee_descBuf_t *
eee_nextBuf(eee_descPtr_t *edesc, eee_descBuf_t *bd)
{
	NFX_ASSERT(edesc != NULL);
	NFX_ASSERT(&(*edesc)->bd[edesc_firstBufferIdx(*edesc)] <= bd
			   && bd < &(*edesc)->bd[EEE_GET_NUM_BUFS(*edesc) +
									edesc_firstBufferIdx(*edesc)]);
	++bd;
	if (bd == &(*edesc)->bd[EEE_GET_NUM_BUFS(*edesc) +
						   edesc_firstBufferIdx(*edesc)]) {
		*edesc = eee_nextDesc(*edesc);
		bd = (*edesc) != NULL ?
			&(*edesc)->bd[edesc_firstBufferIdx(*edesc)] : NULL;
	}
	return bd;
}


/*-----------------------------------------------------------------
 * Name :	eee_copyPacketDataFrom()
 *
 * Description: Copy num_bytes of descriptor data from the given
 * location to a local buffer. Location is specified by buffer index
 * and pointer into the buffer memory.
 *
 * Returns: number of bytes copied.
 *
 * Created by:	Maxim Kozlovsky
 *
 * Date Created:	04/24/02
 *
 *----------------------------------------------------------------- */
int
eee_copyPacketDataFrom(
	void *dest,
	eee_descPtr_t edesc,
	int num_bytes,
	int bufidx,
	char *bufmem
	)
{
	uchar8 *dp = dest;
	int copied;
	int buflen;
	eee_descBuf_t *bd;
	
	/* Copy first (maybe partial) buffer */
	bd = &edesc->bd[bufidx];
	buflen = MIN(num_bytes, eee_remainBufLen(bd, bufmem));
	memcpy(dest, bufmem, buflen);
	dp += buflen;
	copied = buflen;
	bd = eee_nextBuf(&edesc, bd);
	while (bd != NULL && copied < num_bytes) {
		buflen = bd->len;
		if (copied + buflen >= num_bytes) {
			memcpy(dp, bd->buf, num_bytes - copied);
			copied = num_bytes;
		}
		else {
			memcpy(dp, bd->buf, buflen);
			copied += buflen;
			dp += buflen;
			bd = eee_nextBuf(&edesc, bd);
		}
	}

	return copied;
}


/*-----------------------------------------------------------------
 * Name :	eee_verifyCopyArgs()
 *
 * Description: verify the arguments to the copy function and update
 * the number of bytes to copy according to the size of the packet.
 *
 * Returns: NFX_OK if arguments are correct, otherwise
 * EEE_ERR_PACKET_COPY_FAILED.
 *
 * Created by:	Maxim Kozlovsky
 *
 * Date Created:	04/24/02
 *
 *-----------------------------------------------------------------
 */
static int
eee_verifyCopyArgs(void *dest, eee_descPtr_t edesc, uint32 offset,
				   uint32 *num_bytes, int truncate_ok)
{
	int pktlen = EEE_GET_PKT_LEN(edesc);
	int buf_size;
	if (offset >= pktlen) {
		return EEE_ERR_PACKET_COPY_FAILED;
	}
	if (offset + *num_bytes > pktlen) {
		if (truncate_ok) {
			*num_bytes = pktlen - offset;
		}
		else {
			return EEE_ERR_PACKET_COPY_FAILED;
		}
	}
	/*
	 * If the destination buffer is a malloc'd buffer, then verify size
	 */
	if (mm_isMallocBuffer(dest)) {
		if (!(mm_isSizeOkay(dest, *num_bytes, &buf_size))) {
			panic("copyPacketToLocalBuffer, num_bytes = %d, buf = %p,"
				  "buf_size = %d\n", *num_bytes, dest, buf_size);
			return EEE_ERR_PACKET_COPY_FAILED;
		}
	}
	return NFX_OK;
}


/*-----------------------------------------------------------------
 * Name :	eee_copyPacketData()
 *
 * Description: Copies bytes from a system buffer starting from the
 * given offset into a contiguous memory area.
 *
 * Created by:	Maxim Kozlovsky
 *
 * Date Created:	04/23/02
 *
 *-----------------------------------------------------------------
 */
int32							/* returns number of bytes copied,
								 or EEE_ERR_PACKET_COPY_FAILED */
eee_copyPacketData(
	void *dest,					/* destination address of local buffer */
	eee_descPtr_t edesc,		/* source descriptor */
	uint32 offset,				/* offset in the descriptor */
	uint32 num_bytes,			/* bytes to copy */
	uint32 truncate_ok			/* if non-zero, allow truncation */
	)
{
	int32 rv;
	if ((rv = eee_verifyCopyArgs(dest, edesc, offset,
									&num_bytes, truncate_ok)) == NFX_OK) {
		int bufidx;
		uchar8 *copyfrom;
		eee_skipBufsFromStart(&edesc, offset, &bufidx, &copyfrom);
		rv = eee_copyPacketDataFrom(dest, edesc, num_bytes, bufidx, copyfrom);
	}
	return rv;
}

/*-----------------------------------------------------------------
 * Name:	eee_copyPacketToLocalBuffer(void			*dest,
 *										eee_descPtr_t	edesc,
 *										uint32			num_bytes,
 *										uint32			truncate_ok)
 *
 * Description:	Copies bytes from a system buffer into a flat memory
 *				area.
 *
 * Created by:	Rick Lund
 *
 * Date Created:	4/9/01
 *
 *-----------------------------------------------------------------
 */
int32				/* returns number of bytes copied, or
					 * EEE_ERR_PACKET_COPY_FAILED
					 */
eee_copyPacketToLocalBuffer(
	void			*dest,		/* address of local buffer into which
								 * the buffer data will be copied
								 */
	eee_descPtr_t	edesc,		/* source descriptor from which to copy
								 * buffer data
								 */
	uint32			num_bytes,	/* max bytes to copy */
	uint32			truncate_ok	/* whether or not to allow truncation
								 * of packet data when copying num_bytes
								 * to local buffer.  If truncation is not
								 * allowed, but packet is larger than
								 * num_bytes, then return error code
								 */
)
{
	int rv;
	if ((rv = eee_verifyCopyArgs(dest, edesc, 0, &num_bytes,
								 truncate_ok)) == NFX_OK) {
		rv = eee_copyPacketDataFrom(dest, edesc, num_bytes,
				  edesc_firstBufferIdx(edesc),
								  edesc->bd[edesc_firstBufferIdx(edesc)].buf);
	}
	return rv;
}	/* eee_copyPacketToLocalBuffer() */

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:
 *
 *-----------------------------------------------------------------
 */
#if defined(EEE_VALIDATE)
void
eee_deallocateBuffer(
	void	*bufPtr			/* address within buffer to be released */
)
{
	address_t	ra;

	ra = (address_t)get_ra();

	eee_deallocateBufferRA(bufPtr, ra);

}
#endif

/*-----------------------------------------------------------------
 * Name:    eee_isUltraBuf()
 *
 * Description: From an address within a buffer, figure out if its
 *              an ULTRA buf and return its base address.
 *
 * Created by:  Bob Fozard
 *
 * Date Created:    2/20/02
 *
 *-----------------------------------------------------------------
 */
boolean
eee_isUltraBuf(void *bufptr, void **buf_base)
{
    int32   pool_id;

    for (pool_id = 0; pool_id < EEE_NUM_MEM_POOLS; pool_id++)
    {
        if (bufptr >= eee.buf_base[pool_id][EEE_BUF_ULTRA] &&
            bufptr < eee.buf_end[pool_id][EEE_BUF_ULTRA])
        {
            *buf_base = (void *) EEE_BUF_BASE(bufptr, pool_id, EEE_BUF_ULTRA);
            return TRUE;
        }
    }
    return FALSE;
}

uint32
eee_getAvailableNbrOfDesc()
{
    uint32  mem;

	GET_MY_MEMPOOL_INDEX(mem);
    return (eee.desc_pool[mem].stack.num_avail);
}

uint32
eee_getLWMBufs(
	uint32	buf_size		/* EEE_BUF_SMALL
							 * EEE_BUF_LARGE
							 * EEE_BUF_ULTRA
							 */
)
{
    return (eee_stats->low_watermark_buf[eee.my_pool_id - SHARED_POOL_OFFSET][buf_size]);
}

#ifdef NFX_MOD_NFP
uint64
eee_getTotalSizeOfLargeBufsOnTxrx()
{
	uint64	num;

	num = (address_t)eee.buf_end[EEE_TXRX_POOL-SHARED_POOL_OFFSET][EEE_BUF_LARGE] -
		((address_t)eee.buf_base[EEE_TXRX_POOL-SHARED_POOL_OFFSET][EEE_BUF_LARGE]);

	return num;
}
#endif

/*-----------------------------------------------------------------
 * Name:	eee_initDeallocateSharedBuffers()
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	5/11/01
 *
 *-----------------------------------------------------------------
 */
void
eee_initDeallocateSharedBuffers(
	uint32	pass
)
{
	uchar8	* bufptr, * trash;
	uint32	pool_id, i;
	uint32	wd_kick_ctr = 0;

	EEE_PRINTF(("[eee_initDeallocateSharedBuffers], pass = %d\n", pass));

	pool_id = eee.my_pool_id - SHARED_POOL_OFFSET;

	for (i=0; i<EEE_NUM_BUF_SIZES; i++) {
		if (eee.sm_num_buf[i]) {
if (pass == 0) {
#ifdef EEE_BUF_MGMT
			if (i == EEE_BUF_MGMT)
				bufptr = (uchar8 *)eee.mgmt_space;
			else {
				trash = (uchar8 *)eee_allocateAlignedBlock(eee.buf_size[i]);
				bufptr = (uchar8 *)eee_allocateBlock(
									(eee.sm_num_buf[i] + 1) * eee.buf_size[i]);
				bufptr += eee.buf_size[i] -
					(uint32)(address_t)bufptr % eee.buf_size[i];
			}
#else
			trash = (uchar8 *)eee_allocateAlignedBlock(eee.buf_size[i]);
EEE_PRINTF(("initDeallocatesharedBuffers, trash = %p\n", trash));
			bufptr = (uchar8 *)eee_allocateBlock(
									(eee.sm_num_buf[i] + 1) * eee.buf_size[i]);
			bufptr += eee.buf_size[i] - (uint32)(address_t)bufptr % eee.buf_size[i];
#endif
			eee.buf_base[pool_id][i] = bufptr;
			eee.buf_end[pool_id][i] = bufptr +
								(eee.sm_num_buf[i] * eee.buf_size[i]); 
EEE_PRINTF(("EEE.BUF_BASE[%d][%d] = %p\n", pool_id, i, eee.buf_base[pool_id][i]));
EEE_PRINTF(("EEE.BUF_END[%d][%d] = %p\n", pool_id, i, eee.buf_end[pool_id][i]));
}
else if (pass == 1) {

			for (bufptr = eee.buf_base[pool_id][i];
				 (address_t)bufptr <
					 (address_t)(uchar8 *)eee.buf_end[pool_id][i];
				 (address_t)bufptr += eee.buf_size[i]) {
				if ((wd_kick_ctr++ & 0xFFF) == 0)
					watchdog_kick();
#ifdef SIBYTE
if (spans_dram_blocks((address_t)bufptr, eee.buf_size[i])) {
printf("unusable buffer, %p, phys_base = %lx, phys_end = %lx\n", bufptr,
virt_to_phys((address_t)bufptr), virt_to_phys((address_t)bufptr+eee.buf_size[i]));
continue;
}
#endif
				eee_deallocateBuffer(bufptr);
			}
}
		}
	}

	EEE_PRINTF(("[eee_initDeallocateSharedBuffers], leaving\n"));
}	/* eee_initDeallocateSharedBuffers() */

/*-----------------------------------------------------------------
 * Name:	eee_initDeallocateSharedDesc()
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:
 *
 *-----------------------------------------------------------------
 */
void
eee_initDeallocateSharedDesc(
	uint32	pass
)
{
	eee_descPtr_t	edesc;
	uint32			pool_id;

	EEE_PRINTF(("[eee_initDeallocateSharedDesc] pass = %d\n", pass));

	pool_id = eee.my_pool_id - SHARED_POOL_OFFSET;

if (pass == 0) {
	eee.desc_base[pool_id] = (eee_descPtr_t)eee_allocateBlock(
									eee.sm_num_desc * sizeof(eee_desc_t) + 1);
	eee.desc_base[pool_id] = (eee_descPtr_t)
		(((address_t)eee.desc_base[pool_id] + sizeof(eee_desc_t) - 1)
		 & ~(sizeof(eee_desc_t) - 1));
	eee.desc_end[pool_id] = eee.desc_base[pool_id] + eee.sm_num_desc;
EEE_PRINTF(("EEE.DESC_BASE[%d] = %p\n", pool_id, eee.desc_base[pool_id]));
EEE_PRINTF(("EEE.DESC_END[%d] = %p\n", pool_id, eee.desc_end[pool_id]));
	 
}
else if (pass == 1) {

	watchdog_kick();
	for (edesc = eee.desc_base[pool_id];
									edesc < eee.desc_end[pool_id]; edesc++) {
		eee_deallocateDesc(edesc);
	}
}

	EEE_PRINTF(("[eee_initDeallocateSharedDesc], leaving\n"));

}	/* eee_initDeallocateSharedDesc() */

/*-----------------------------------------------------------------
 * Name:	eee_initTailPointers()
 *
 * Description:	Write initial values into tail pointers in remote memory
 *				pools
 *
 * Created by:	Rick Lund
 *
 * Date Created:	5/16/01
 *
 *-----------------------------------------------------------------
 */
void
eee_initTailPointers()
{
	uint32	i, j, my_index;

	GET_MY_MEMPOOL_INDEX(my_index);

	EEE_PRINTF(("[eee_initTailPointers]\n"));
	for (i=EEE_NUM_LM_POOLS; i<EEE_NUM_MEM_POOLS; i++) {
		if (i == my_index)
			continue;
		if (eee.shmem_avail[i-EEE_NUM_LM_POOLS]) {
			EEE_PRINTF(("writing %p to %p\n", eee.desc_pool[i].queue.end-1,
						eee.desc_pool[i].queue.tail_ptr));
			*(eee.desc_pool[i].queue.tail_ptr) = eee.desc_pool[i].queue.end-1;

			for (j=0; j<EEE_NUM_BUF_SIZES; j++) {
				if (eee.buf_pool[i][j].queue.head == eee.buf_pool[i][j].queue.end) {
				EEE_PRINTF(("writing 0 to %p\n", eee.buf_pool[i][j].queue.tail_ptr));
				*(eee.buf_pool[i][j].queue.tail_ptr) = 0;
}
else {
				EEE_PRINTF(("writing %p to %p\n",
							eee.buf_pool[i][j].queue.end-1,
							eee.buf_pool[i][j].queue.tail_ptr));
				*(eee.buf_pool[i][j].queue.tail_ptr) = 
											eee.buf_pool[i][j].queue.end-1;
}
			}
		}
		else
			printf("shmem pool %d unavailable\n", i-EEE_NUM_LM_POOLS);
	}
	for (i=0; i<EEE_NUM_FWD_QUEUES; i++) {
		if (i == eee.my_fwd_id)
			continue;
#if 0
		if (eee.shmem_avail[eee_index[2][i] - EEE_NUM_LM_POOLS]) {
#else
		if (eee.shmem_avail[eee_index[2][i]]) {
#endif
			EEE_PRINTF(("I'm now writing %p to %p\n",
					   eee.rcv_queue[i].rcv_end-1,
					   eee.rcv_queue[i].rcv_tail_ptr));
			*(eee.rcv_queue[i].rcv_tail_ptr) = eee.rcv_queue[i].rcv_end-1;
		}
	}

}	/* eee_initTailPointers() */

/*-----------------------------------------------------------------
 * Name:	eee_descDisplay()
 *
 * Description:	
 *
 * Created by:	Rick Lund
 *
 * Date Created:	12/11/01
 *
 *-----------------------------------------------------------------
 */
void
eee_descDisplay(
	eee_descPtr_t	edesc
)
{
	printf("edesc at 0x%p\n", edesc);
	printf("      control   = 0x%08x\n", edesc->hdr.control);
	printf("      dest_port = 0x%08x\n", edesc->hdr.dest_port);
	printf("      src_port  = 0x%08x\n", edesc->hdr.src_port);
	printf("      attr      = %p\n", (void *)edesc->hdr.attr);
	printf("      next      = %p\n", edesc->hdr.next);
	printf("      offset    = %x\n", edesc->hdr.offset);
	printf("      bd[0]     = 0x%04x 0x%04x %p\n", edesc->bd[0].control, edesc->bd[0].len, edesc->bd[0].buf);
	printf("      bd[1]     = 0x%04x 0x%04x %p\n", edesc->bd[1].control, edesc->bd[1].len, edesc->bd[1].buf);
	printf("      bd[2]     = 0x%04x 0x%04x %p\n", edesc->bd[2].control, edesc->bd[2].len, edesc->bd[2].buf);
	printf("      bd[3]     = 0x%04x 0x%04x %p\n", edesc->bd[3].control, edesc->bd[3].len, edesc->bd[3].buf);
}	/* eee_descDisplay() */

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	3/2/02
 *
 *-----------------------------------------------------------------
 */
int32
eee_getBufSize(
	int32	buf_type		/* number of bytes in this buffer type */
)
{
	return (eee.buf_size[buf_type]);
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	3/2/02
 *
 *-----------------------------------------------------------------
 */
int32					/* EEE_BUF_SMALL,
						 *	EEE_BUF_LARGE,
						 *	EEE_BUF_ULTRA,
						 *	-1 on error
						 */
eee_getBufType(
	void	* _bufptr
)
{
	int32   _pool,_size;

	for (_pool=0; _pool<EEE_NUM_MEM_POOLS; _pool++) {
		for (_size=0; _size<EEE_NUM_BUF_SIZES; _size++) {
			if ((_bufptr >= eee.buf_base[_pool][_size]) &&
									(_bufptr < eee.buf_end[_pool][_size]))
				return _size;
		}
	}

	return -1;
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	3/2/02
 *
 *-----------------------------------------------------------------
 */
void *					/* buffer base address or -1 */
eee_getBufBase(
	void	* buf
)
{
	int32   _pool,_size;

	for (_pool=0; _pool<EEE_NUM_MEM_POOLS; _pool++) {
		for (_size=0; _size<EEE_NUM_BUF_SIZES; _size++) {
			if ((buf >= eee.buf_base[_pool][_size]) &&
									(buf < eee.buf_end[_pool][_size])) {
				if (_size == EEE_BUF_SMALL)
					return (void *)((address_t)(buf) &
											eee.buf_mask[EEE_BUF_SMALL]);
				else if (_size == EEE_BUF_LARGE)
				{
					address_t	base;
					uint32		index;

					base = (address_t)eee.buf_base[(_pool)][EEE_BUF_LARGE];
					index = (uint32)((address_t)(buf) - base) / (EEE_BUFSIZE_LARGE);
					return (void *)(base + (index * EEE_BUFSIZE_LARGE));
				}
#ifdef EEE_BUF_MGMT
				else if (_size == EEE_BUF_MGMT)
				{
					address_t	base;
					uint32		index;

					base = (address_t)eee.buf_base[(_pool)][EEE_BUF_MGMT];
					index = (uint32)((address_t)(buf) - base) / (EEE_BUFSIZE_MGMT);
					return (void *)(base + (index * EEE_BUFSIZE_MGMT));
				}
#endif
				else {
					address_t	base;
					uint32		index;

					base = (address_t)eee.buf_base[(_pool)][EEE_BUF_ULTRA];
					index = (uint32)((address_t)(buf) - base) / (EEE_BUFSIZE_ULTRA);
					return (void *)(base + (index * EEE_BUFSIZE_ULTRA));
				}
			}
		}
	}

	return (void *)-1;
}

int32
spans_dram_blocks(
	address_t	start,
	uint32		len
)
{
#ifdef SIBYTE
	if ((virt_to_phys(start+len - 1) - virt_to_phys(start)) == (len - 1))
		return 0;
	else
		return 1;
#else
	return 0;
#endif
}


address_t get_pc()
{
	return get_ra();
}

uint32
eee_getPktNumBufs(
	eee_descPtr_t	ed
)
{
	uint32  nb = 0;
	while (ed) {
		nb += EEE_GET_NUM_BUFS(ed);
		ed = ed->hdr.next;
	}
	return nb;
}

