/*
 *-----------------------------------------------------------------
 *
 * Name:	eee.h
 *
 * RCS:  $Id: eee.h,v 1.57 2002/12/07 00:02:01 rick Exp $
 *
 * Copyright (C) 2001, Agile Storage, Inc.
 *
 * Description:	Local EEE structure and config structure
 *
 * Created by:	Rick Lund
 *
 * Date Created:	4/10/01
 *
 *-----------------------------------------------------------------
 */

#ifndef _EEE_H
#define _EEE_H

#define EEE_NEW_RATIO

// #define EEE_VALIDATE

#if defined(EEE_DEBUG)
#define EEE_PRINTF(x) printf x
#else
#define EEE_PRINTF(x)
#endif

#if defined(SSC) || defined(SSC_MGMT) || defined(PROM)
#else

#ifdef NFP_FP
// #define EEE_POLLING_BUCKET 1
//#define EEE_POLLING_PERF   1
#endif // NFP_FP
#ifdef NFP_TXRX
// #define EEE_POLLING_BUCKET 1
//#define EEE_POLLING_PERF   1
#endif // NFP_TXRX
#ifdef FCNIM
// #define EEE_POLLING_BUCKET 1
#define EEE_POLLING_PERF   1
#endif // FCNIM

#endif  // SSC


#ifdef EEE_POLLING_PERF
#define EEE_POLLING_HIST_START_AFTER_N_SEC     5
#define EEE_POLLING_HIST                       10
extern int64 eeePollingCounterMin;
extern int64 eeePollingCounterMax;
extern int64 eeePollingCountersHist[EEE_POLLING_HIST];
extern int32 eeePollingCounterIndex;
#endif

/*------------------------------------------------------------------*
 *
 *	Local Polling Interface structures
 *
 *------------------------------------------------------------------*/

#define EEE_MAX_POLL_HIGH_ROUTINES		32
#define EEE_MAX_POLL_LOW_ROUTINES		32
#define EEE_NBR_POLLING_BUCKETS         6

struct eee_poll_cb {
	void	(*func)();
	void	*cb;
	uint32	num;
#ifdef EEE_POLLING_BUCKET
	uint32  bucket[EEE_NBR_POLLING_BUCKETS];
#endif
};

typedef struct polling {
	struct eee_poll_cb high[EEE_MAX_POLL_HIGH_ROUTINES];
	struct eee_poll_cb low[EEE_MAX_POLL_LOW_ROUTINES];
#ifdef EEE_POLLING_BUCKET
    eeeGMT_t clearTime;
#endif
} eee_pfunc_t;


#define EEE_HIGH_PRIORITY_IND	0x4000
#define EEE_LOW_PRIORITY_IND	0x0

/*------------------------------------------------------------------*
 *
 *	Local Receive Interface structures
 *
 *------------------------------------------------------------------*/

#define EEE_MAX_APP_CLIENTS		65

typedef struct rcv {
	void	(*uni[EEE_MAX_APP_CLIENTS])();
	void	(*multi[EEE_MAX_APP_CLIENTS])();
} eee_rcv_t;

/*------------------------------------------------------------------*
 *
 *	Forwarding Interface structures
 *
 *------------------------------------------------------------------*/

typedef struct eee_port {
	eee_transmitFunc_t func;
	uint32	dropped;
	uint32	forwarded;
	int mtu;
} eee_port_t;

typedef struct drv {
	eee_port_t uni[EDESC2_PORT_NUM];
	eee_port_t multi[EDESC2_PORT_NUM];
	eee_port_t dtype_default[EDESC2_DATA_TYPE_NUM];
	uint32	no_uni;
	uint32	no_multi;
} eee_drv_t;

/*------------------------------------------------------------------*
 *
 * Memory sizing definitions
 *
 *------------------------------------------------------------------*/
#if defined(NFP_TXRX) || defined(NFP_FP)

	#define EEE_NUM_SM_POOLS		2
	#define EEE_NUM_LM_POOLS		1
	#define EEE_NUM_IPC_QUEUES		0		/* num CPUs */
	#define EEE_NUM_FWD_QUEUES		2
	#define EEE_NUM_CPUS			6		/* 4 CPUs + TXRX_ANY + FP_ANY */
extern uint32	eee_index[3][EEE_NUM_CPUS];

	#if defined(NFP_TXRX)

		/*
		 * These are per 16 MB of Shared Memory
		 */
		/* no TXRX -> TXRX packets */
		#define RQ_TXRX_SMALL_BUF		0
		#define RQ_TXRX_LARGE_BUF		0
		#define RQ_TXRX_ULTRA_BUF		0
#ifdef EEE_BUF_MGMT
		#define RQ_TXRX_MGMT_BUF		0
#endif
		#define RQ_TXRX_DESC			0
		/* for TXRX -> FP packets */
		#define RQ_FP_SMALL_BUF			0x40
		#define RQ_FP_LARGE_BUF			0x40
		#define RQ_FP_ULTRA_BUF			0
#ifdef EEE_BUF_MGMT
		#define RQ_FP_MGMT_BUF			0
#endif
		#define RQ_FP_DESC				0x80	/* SMALL + LARGE + ULTRA */

#ifdef EEE_NEW_RATIO
		#define SMALL_RATIO				10
		#define LARGE_RATIO				500
		#define ULTRA_RATIO				1
		#define DESC_RATIO				(SMALL_RATIO + LARGE_RATIO + ULTRA_RATIO)
		#define FWD_RATIO				(DESC_RATIO)

#ifdef EEE_ATTR
		#define ALLOC_DIVISOR           ((SMALL_RATIO * EEE_BUFSIZE_SMALL) + \
                                        (LARGE_RATIO * EEE_BUFSIZE_LARGE) + \
                                        (ULTRA_RATIO * EEE_BUFSIZE_ULTRA) + \
                                        (DESC_RATIO * sizeof(eee_desc_t)) + \
                                        (FWD_RATIO * sizeof(address_t)) + \
                                        ((SMALL_RATIO +     \
                                            LARGE_RATIO +   \
                                            ULTRA_RATIO +   \
                                            DESC_RATIO) * sizeof(address_t)) + \
                                        ((SMALL_RATIO +     \
                                            LARGE_RATIO +   \
                                            ULTRA_RATIO +   \
                                            DESC_RATIO) * sizeof(eee_attr_t))   \
                                        )
#else
		#define ALLOC_DIVISOR           ((SMALL_RATIO * EEE_BUFSIZE_SMALL) + \
                                        (LARGE_RATIO * EEE_BUFSIZE_LARGE) + \
                                        (ULTRA_RATIO * EEE_BUFSIZE_ULTRA) + \
                                        (DESC_RATIO * sizeof(eee_desc_t)) + \
                                        (FWD_RATIO * sizeof(address_t)) + \
                                        ((SMALL_RATIO +     \
                                            LARGE_RATIO +   \
                                            ULTRA_RATIO +   \
                                            DESC_RATIO) * sizeof(address_t)) \
                                        )
#endif	/* EEE_ATTR */

#else
		/*
		 * These are per 8 small buffers
		 */
		#define SMALL_TO_LARGE_RATIO	200		/* S = L */
		#define SMALL_TO_ULTRA_RATIO    1  		/* U = 0 */
		#define DESC_TO_SMALL_RATIO		209		/* D = S + L + U */
		#define FWD_TO_SMALL_RATIO		209		/* F = D */
//		#define SMALL_TO_LARGE_RATIO	400		/* S = L */
//		#define SMALL_TO_ULTRA_RATIO    1  		/* U = 0 */
//		#define DESC_TO_SMALL_RATIO		409		/* D = S + L + U */
//		#define FWD_TO_SMALL_RATIO		409		/* F = D */

#endif	/* EEE_NEW_RATIO */

		/*
		 * These are per 512 forwarding entries
		 */
		#define FWD_TXRX_TXRX			0
		#define FWD_TXRX_FP				512
#if 0
		#define FWD_TX_TX				0
		#define FWD_TX_RX				64
		#define FWD_TX_FP1				192
		#define FWD_TX_FP2				192
		#define FWD_RX_TX				64
		#define FWD_RX_RX				0
		#define FWD_RX_FP1				192
		#define FWD_RX_FP2				192
#endif

	#elif defined(NFP_FP)

		/*
		 * These are per 16 MB of Shared Memory
		 */
		/* for FP -> TXRX packets */
		#define RQ_TXRX_SMALL_BUF		0x40
		#define RQ_TXRX_LARGE_BUF		0xc0
		#define RQ_TXRX_ULTRA_BUF		0
#ifdef EEE_BUF_MGMT
		#define RQ_TXRX_MGMT_BUF		0x4
#endif
		#define RQ_TXRX_DESC			0x80	/* SMALL + LARGE + ULTRA */
		/* no FP -> FP packets */
		#define RQ_FP_SMALL_BUF			0
		#define RQ_FP_LARGE_BUF			0
		#define RQ_FP_ULTRA_BUF			0
#ifdef EEE_BUF_MGMT
		#define RQ_FP_MGMT_BUF			0
#endif
		#define RQ_FP_DESC				0

#ifdef EEE_NEW_RATIO
		#define SMALL_RATIO             1
		#define LARGE_RATIO             1
		#define ULTRA_RATIO             5
		#define DESC_RATIO              (SMALL_RATIO + LARGE_RATIO + ULTRA_RATIO)
		#define FWD_RATIO               (DESC_RATIO)

#ifdef EEE_ATTR
		#define ALLOC_DIVISOR           ((SMALL_RATIO * EEE_BUFSIZE_SMALL) + \
                                        (LARGE_RATIO * EEE_BUFSIZE_LARGE) + \
                                        (ULTRA_RATIO * EEE_BUFSIZE_ULTRA) + \
                                        (DESC_RATIO * sizeof(eee_desc_t)) + \
                                        (FWD_RATIO * sizeof(address_t)) + \
                                        ((SMALL_RATIO +     \
                                            LARGE_RATIO +   \
                                            ULTRA_RATIO +   \
                                            DESC_RATIO) * sizeof(address_t)) + \
                                        ((SMALL_RATIO +     \
                                            LARGE_RATIO +   \
                                            ULTRA_RATIO +   \
                                            DESC_RATIO) * sizeof(eee_attr_t))   \
                                        )
#else
		#define ALLOC_DIVISOR           ((SMALL_RATIO * EEE_BUFSIZE_SMALL) + \
                                        (LARGE_RATIO * EEE_BUFSIZE_LARGE) + \
                                        (ULTRA_RATIO * EEE_BUFSIZE_ULTRA) + \
                                        (DESC_RATIO * sizeof(eee_desc_t)) + \
                                        (FWD_RATIO * sizeof(address_t)) + \
                                        ((SMALL_RATIO +     \
                                            LARGE_RATIO +   \
                                            ULTRA_RATIO +   \
                                            DESC_RATIO) * sizeof(address_t)) \
                                        )
#endif	/* EEE_ATTR */

#else
		/*
		 * These are per 8 small buffers
		 */
		#define SMALL_TO_LARGE_RATIO	8		/* S = L */
#ifdef LINUX_TEST
		#define SMALL_TO_ULTRA_RATIO	0		/* U = 0 */
		#define DESC_TO_SMALL_RATIO		16		/* D = S + L + U */
		#define FWD_TO_SMALL_RATIO		16		/* F = D */
#else
		#define SMALL_TO_ULTRA_RATIO	40		/* U = 0 per 8 small, 80 ultra*/
		#define DESC_TO_SMALL_RATIO		56		/* D = S + L + U */
		#define FWD_TO_SMALL_RATIO		56		/* F = D */
//		#define SMALL_TO_ULTRA_RATIO	80		/* U = 0 per 8 small, 80 ultra*/
//		#define DESC_TO_SMALL_RATIO		96		/* D = S + L + U */
//		#define FWD_TO_SMALL_RATIO		96		/* F = D */
#endif
#endif	/* EEE_NEW_RATIO */

		/*
		 * These are per 512 forwarding entries
		 */
		#define FWD_FP_TXRX				512
		#define FWD_FP_FP				0
#if 0
		#define FWD_FP1_TX				64
		#define FWD_FP1_RX				256
		#define FWD_FP1_FP1				0
		#define FWD_FP1_FP2				64
		#define FWD_FP2_TX				64
		#define FWD_FP2_RX				256
		#define FWD_FP2_FP1				64
		#define FWD_FP2_FP2				0
#endif

	#endif

#elif defined(FCNIM) || defined(FCNIM_TM) || defined(PROM)

	#define EEE_NUM_SM_POOLS		1
	#define EEE_NUM_LM_POOLS		1
	#define EEE_NUM_IPC_QUEUES		0		/* no IPC on FCNIM */
	#define EEE_NUM_FWD_QUEUES		0
	#define EEE_NUM_CPUS			4
extern uint32	eee_index[3][EEE_NUM_CPUS];

#ifdef EEE_NEW_RATIO
#ifdef PROM
	#define SMALL_RATIO             1
	#define LARGE_RATIO             1
	#define ULTRA_RATIO             0
	#define DESC_RATIO              2
	#define FWD_RATIO               0
#else
	#define SMALL_RATIO             1
	#define LARGE_RATIO             2
	#define ULTRA_RATIO             4
	#define DESC_RATIO              6
	#define FWD_RATIO               0
#endif

#ifdef EEE_ATTR
	#define ALLOC_DIVISOR           ((SMALL_RATIO * EEE_BUFSIZE_SMALL) + \
                                    (LARGE_RATIO * EEE_BUFSIZE_LARGE) + \
                                    (ULTRA_RATIO * EEE_BUFSIZE_ULTRA) + \
                                    (DESC_RATIO * sizeof(eee_desc_t)) + \
                                    (FWD_RATIO * sizeof(address_t)) + \
                                    ((SMALL_RATIO +     \
                                            LARGE_RATIO +   \
                                            ULTRA_RATIO +   \
                                            DESC_RATIO) * sizeof(address_t)) + \
                                    ((SMALL_RATIO +     \
                                            LARGE_RATIO +   \
                                            ULTRA_RATIO +   \
                                            DESC_RATIO) * sizeof(eee_attr_t))   \
                                    )
#else
	#define ALLOC_DIVISOR           ((SMALL_RATIO * EEE_BUFSIZE_SMALL) + \
                                    (LARGE_RATIO * EEE_BUFSIZE_LARGE) + \
                                    (ULTRA_RATIO * EEE_BUFSIZE_ULTRA) + \
                                    (DESC_RATIO * sizeof(eee_desc_t)) + \
                                    (FWD_RATIO * sizeof(address_t)) + \
                                    ((SMALL_RATIO +     \
                                            LARGE_RATIO +   \
                                            ULTRA_RATIO +   \
                                            DESC_RATIO) * sizeof(address_t)) \
                                    )
#endif  /* EEE_ATTR */

#else

	/*
	 * These are per 8 small buffers
	 */
#if defined(PROM) || defined(LINUX_TEST)
	#define SMALL_TO_LARGE_RATIO	8		/* S = L */
	#define SMALL_TO_ULTRA_RATIO	0		/* U = 0 */
	#define DESC_TO_SMALL_RATIO		16		/* D = S + L + U */
#else
	#define SMALL_TO_LARGE_RATIO	16		/* S = L */
	#define SMALL_TO_ULTRA_RATIO	32		/* U = 0 */
	#define DESC_TO_SMALL_RATIO		48		/* D = S + L + U */
#endif
	#define FWD_TO_SMALL_RATIO		0		/* F = D */

#endif	/* EEE_NEW_RATIO */

#elif defined(SSC) || defined(SSC_MGMT) || defined(NFX_MOD_SSC)

	#define EEE_NUM_SM_POOLS		1
	#define EEE_NUM_LM_POOLS		1
	#define EEE_NUM_IPC_QUEUES		0		/* num CPUs */
	#define EEE_NUM_FWD_QUEUES		0		/* num CPUs */
	#define EEE_NUM_CPUS			2
extern uint32	eee_index[3][EEE_NUM_CPUS];

	#if defined(SSC)

		/*
		 * These are per 16 MB of Shared Memory
		 */
		/* no SSC->SSC packets */
		#define RQ_SSC_SMALL_BUF		0
		#define RQ_SSC_LARGE_BUF		0
		#define RQ_SSC_ULTRA_BUF		0
		#define RQ_SSC_DESC				0
		/* for SSC->405 packets */
		#define RQ_405_SMALL_BUF		0x400
		#define RQ_405_LARGE_BUF		0x400
		#define RQ_405_ULTRA_BUF		0
		#define RQ_405_DESC				0x800	/* SMALL + LARGE + ULTRA */

		/*
		 * These are per 8 small buffers
		 */
		#define SMALL_TO_LARGE_RATIO	8		/* S = L */
		#define SMALL_TO_ULTRA_RATIO	0		/* U = 0 */
		#define DESC_TO_SMALL_RATIO		16		/* D = S + L + U */
		#define FWD_TO_SMALL_RATIO		16		/* F = D */

		/*
		 * These are per 512 forwarding entries
		 */
		#define FWD_SSC_SSC				0
		#define FWD_SSC_405				512		/* all entries are in receive
												 * queue from SSC_MGMT
												 */

	#elif defined(SSC_MGMT)

		/*
		 * These are per 16 MB of Shared Memory
		 */
		/* for 405->SSC packets */
		#define RQ_SSC_SMALL_BUF		0x400
		#define RQ_SSC_LARGE_BUF		0x400
		#define RQ_SSC_ULTRA_BUF		0
		#define RQ_SSC_DESC				0x800	/* SMALL + LARGE + ULTRA */
		/* no 405->405 packets */
		#define RQ_405_SMALL_BUF		0
		#define RQ_405_LARGE_BUF		0
		#define RQ_405_ULTRA_BUF		0
		#define RQ_405_DESC				0

#ifdef EEE_NEW_RATIO
		/*
		 * These are per each small buffer
		 */
		#define SMALL_RATIO             1
		#define LARGE_RATIO             1
		#define ULTRA_RATIO             0
		#define DESC_RATIO              2
		#define FWD_RATIO               0

#ifdef EEE_ATTR
		#define ALLOC_DIVISOR		((SMALL_RATIO * EEE_BUFSIZE_SMALL) + \
									(LARGE_RATIO * EEE_BUFSIZE_LARGE) + \
									(ULTRA_RATIO * EEE_BUFSIZE_ULTRA) + \
									(DESC_RATIO * sizeof(eee_desc_t)) + \
									(FWD_RATIO * sizeof(address_t)) + \
									((SMALL_RATIO +     \
											LARGE_RATIO +   \
											ULTRA_RATIO +   \
											DESC_RATIO) * sizeof(address_t)) + \
									 ((SMALL_RATIO +     \
											LARGE_RATIO +   \
											ULTRA_RATIO +   \
											DESC_RATIO) * sizeof(eee_attr_t))   \
									)
#else
		#define ALLOC_DIVISOR		((SMALL_RATIO * EEE_BUFSIZE_SMALL) + \
									(LARGE_RATIO * EEE_BUFSIZE_LARGE) + \
									(ULTRA_RATIO * EEE_BUFSIZE_ULTRA) + \
									(DESC_RATIO * sizeof(eee_desc_t)) + \
									(FWD_RATIO * sizeof(address_t)) + \
									((SMALL_RATIO +     \
											LARGE_RATIO +   \
											ULTRA_RATIO +   \
											DESC_RATIO) * sizeof(address_t)) \
									)
#endif  /* EEE_ATTR */

#else
		/*
		 * These are per 8 small buffers
		 */
		#define SMALL_TO_LARGE_RATIO	8		/* S = L */
		#define SMALL_TO_ULTRA_RATIO	0		/* U = 0 */
		#define DESC_TO_SMALL_RATIO		16		/* D = S + L + U */
		#define FWD_TO_SMALL_RATIO		16		/* F = D */

#endif	/* EEE_NEW_RATIO */

		/*
		 * These are per 512 forwarding entries
		 */
		#define FWD_405_SSC				512		/* all entries are in receive
												 * queue from SSC_405
												 */
		#define FWD_405_405				0

	#endif

#else
	// Just some bogus values so this header file can be included in always
	#define EEE_NUM_SM_POOLS		1
	#define EEE_NUM_LM_POOLS		1
	#define EEE_NUM_IPC_QUEUES		0
	#define EEE_NUM_FWD_QUEUES		1
	#define EEE_NUM_CPUS			1
extern uint32	eee_index[3][EEE_NUM_CPUS];
#endif

#define EEE_NUM_MEM_POOLS	(EEE_NUM_LM_POOLS+EEE_NUM_SM_POOLS)

/*------------------------------------------------------------------*/

#define GET_MY_MEMPOOL_INDEX(_index)		(_index) = eee_index[0][nfxMyCPUId]
#define GET_MY_IPC_INDEX(_index)			(_index) = eee_index[1][nfxMyCPUId]
#define GET_MY_FWD_INDEX(_index)			(_index) = eee_index[2][nfxMyCPUId]
#define GET_FWD_INDEX_FROM_CPU_ID(_c_id)			\
({													\
	uint32	_r;										\
													\
	_r = eee_index[2][_c_id];						\
	_r;												\
});

#define FWD_QUEUE_FROM_MEM_POOL(_index)				\
({													\
	uint32	_r = 0xFFFFFFFF;						\
	uint32	_i;										\
													\
	for (_i=0; _i<EEE_NUM_CPUS; _i++) {				\
		if (eee_index[0][_i] == (_index)) {			\
			_r = eee_index[2][_i];					\
			break;									\
		}											\
	}												\
	if (_i == EEE_NUM_CPUS)  						\
		printf("Error finding fwd queue index\n");	\
	_r;												\
});

#define GET_MEMPOOL_INDEX_FROM_CPU_ID(_c_id)		\
({													\
	uint32	_r;										\
													\
	_r = SHARED_POOL_OFFSET + eee_index[0][_c_id];	\
});

/*------------------------------------------------------------------*
 *
 * Control structures for resource allocation
 *
 *------------------------------------------------------------------*/
typedef struct stack_pool {
	address_t	* top;
	uint32		num_avail;
} eee_stack_pool_cntl_t;

typedef struct queue_pool {
	address_t	* start;
	address_t	* end;
	address_t	* head;
	address_t	** tail_ptr;
} eee_queue_pool_cntl_t;

typedef union pool_cntl {
	eee_stack_pool_cntl_t	stack;
	eee_queue_pool_cntl_t	queue;
} eee_pool_cntl_t;

typedef struct mem_pool {
	uint32		* curr_addr;
	uint32		bytes_avail;
} eee_mem_pool_cntl_t;

/*------------------------------------------------------------------*
 *
 *	Statistics
 *
 *------------------------------------------------------------------*/
typedef struct eee_stats {

	uint32	no_buf_avail[EEE_NUM_MEM_POOLS][EEE_NUM_BUF_SIZES];
	uint32	no_desc_avail[EEE_NUM_MEM_POOLS];

	uint32	low_watermark_buf[EEE_NUM_MEM_POOLS][EEE_NUM_BUF_SIZES];
	uint32	low_watermark_desc[EEE_NUM_MEM_POOLS];

	uint32	ipc_no_space[EEE_NUM_IPC_QUEUES];
	uint32	ipc_msg_too_big[EEE_NUM_IPC_QUEUES];
	uint32	ipc_nack[EEE_IPC_NUM_CLIENTS];
	uint32	ipc_ok[EEE_IPC_NUM_CLIENTS];

	uint32	no_fwd_avail[EEE_NUM_FWD_QUEUES];

	uint32	packet_too_big;

	uint32	invalid_desc_addr;
	uint32	invalid_buf_addr;

	uint32	invalid_app_id;

	uint32	rq_desc_allocated[EEE_NUM_MEM_POOLS];
	uint32	rq_buf_allocated[EEE_NUM_MEM_POOLS][EEE_NUM_BUF_SIZES];

	uint32	fwd_free_pkt[EEE_NUM_FWD_QUEUES];
	uint32	fwd_free_buf[EEE_NUM_FWD_QUEUES];
	uint32	fwd_free_desc[EEE_NUM_FWD_QUEUES];
/*
	uint32	fwd_pkts_forwarded[EEE_NUM_FWD_QUEUES];
	uint32	fwd_pkts_remote[NFX_NUM_SLOTS_PER_CHASSIS];
*/

	uint32	fwd_pkts[NFX_NUM_SLOTS_PER_CHASSIS][NFX_MAX_CPUS_PER_SLOT];
	uint32	app_rcv_pkts[EEE_MAX_APP_CLIENTS];

#if defined(NFP_TXRX)
	uint64  txrx_mgmt_copied;
#endif
} eee_stats_t;



struct eee_rcv_queue
{
	nfx_spinlock_t  rcv_spin;
	eee_descPtr_t	* rcv_start;
	eee_descPtr_t	* rcv_end;
	eee_descPtr_t	* rcv_head;
	eee_descPtr_t	** rcv_tail_ptr;
	uint32			rcv_cnt;
	uint32 			rcv_max;
	uint32			rcv_hwm;
	uint32			rcv_free_pkt;
	uint32			rcv_free_buf;
	uint32			rcv_free_desc;
	uint32			rcv_pkts_forwarded;
};

/*------------------------------------------------------------------*
 *
 *	 Config structure
 *
 *------------------------------------------------------------------*/
typedef struct eee_config {

	uint32	init_done;

	address_t	shmem_base;
	address_t	wka;

	uint32	my_pool_id;
	uint32	my_fwd_id;

	uint32	buf_size[EEE_NUM_BUF_SIZES];

	address_t	buf_mask[EEE_NUM_BUF_SIZES];

	uint32	buf_offset;

	uint32	sm_size;
#ifndef EEE_NEW_RATIO
	uint32	sm_for_desc_and_buf;
	uint32	bytes_per_small_buf;
#else
	uint32	num_alloc_pages;
#endif

	uint32	sm_num_buf[EEE_NUM_BUF_SIZES];
	uint32	sm_num_desc;
	uint32	sm_num_fwd;					/* total number of fwd rcv q entries */

#ifdef EEE_BUF_MGMT
	address_t	mgmt_space;	/* base addr of region for pci accesses on sibyte */
	uint32		mgmt_size_buf;	/* for buffers */
	uint32		mgmt_size_cont;	/* for other structures (descriptors) */
#endif
	uint32		mgmt_stack_size;

	/*
	 * Memory pools for allocation of resources
	 */
	eee_mem_pool_cntl_t		sm_pool;		/* shared memory pool */
	eee_mem_pool_cntl_t		lm_pool;		/* local memory pool */
#ifdef EEE_BUF_MGMT
	eee_mem_pool_cntl_t		mgmt_pool;		/* shared memory pool */
#endif

	/*
	 * Is the memory pool available (did the CPU put it's RQ entries to the
	 * WKA during init?
	 */
	uint32	shmem_avail[EEE_NUM_SM_POOLS];

	/*
	 * Buffer address ranges
	 */
	void	* buf_base[EEE_NUM_MEM_POOLS][EEE_NUM_BUF_SIZES];
	void	* buf_end[EEE_NUM_MEM_POOLS][EEE_NUM_BUF_SIZES];

	/*
	 * Descriptor address ranges
	 */
	eee_descPtr_t	desc_base[EEE_NUM_MEM_POOLS];
	eee_descPtr_t	desc_end[EEE_NUM_MEM_POOLS];

	/*------------------------------------------------------------------*
	 *
	 * Structures allocated in my memory pool
	 *
	 *------------------------------------------------------------------*/

	/*
	 * Free Descriptor Pools
	 */
	eee_pool_cntl_t		desc_pool[EEE_NUM_MEM_POOLS];

	/*
	 * Free Buffer Pools
	 */
	eee_pool_cntl_t		buf_pool[EEE_NUM_MEM_POOLS][EEE_NUM_BUF_SIZES];

	/*
	 * traditional IPC queues
	 */
	eee_ipcEntry_t	* ipc_rcv_start[EEE_NUM_IPC_QUEUES];
	eee_ipcEntry_t	* ipc_rcv_end[EEE_NUM_IPC_QUEUES];
	volatile eee_ipcEntry_t	* ipc_rcv_curr[EEE_NUM_IPC_QUEUES];

#ifdef EEE_ATTR
	/*
	 * Attribute structures (for descriptors and buffers)
	 */
	eee_attr_t		* desc_attr[EEE_NUM_MEM_POOLS];
	eee_attr_t		* buf_attr[EEE_NUM_MEM_POOLS][EEE_NUM_BUF_SIZES];
#endif

	/*
	 * Forwarding (IPC) queues
	 */
	struct eee_rcv_queue rcv_queue[EEE_NUM_FWD_QUEUES];

	/*------------------------------------------------------------------*
	 *
	 * Structures allocated from other CPU's memory pools.  At init time,
	 * this CPU will need to read these addresses from the structures at
	 * the Well Known Address.
	 *
	 *------------------------------------------------------------------*/

	eee_queue_pool_cntl_t	far_desc_pool[EEE_NUM_MEM_POOLS];
	eee_queue_pool_cntl_t	far_buf_pool[EEE_NUM_MEM_POOLS][EEE_NUM_BUF_SIZES];
	eee_ipcEntry_t	* ipc_send_start[EEE_NUM_IPC_QUEUES];
	eee_ipcEntry_t	* ipc_send_end[EEE_NUM_IPC_QUEUES];
	eee_ipcEntry_t	* ipc_send_curr[EEE_NUM_IPC_QUEUES];
	eee_descPtr_t	* fwd_start[EEE_NUM_FWD_QUEUES];
	eee_descPtr_t	* fwd_end[EEE_NUM_FWD_QUEUES];
	eee_descPtr_t	* fwd_head[EEE_NUM_FWD_QUEUES];
	eee_descPtr_t	** fwd_tail_ptr[EEE_NUM_FWD_QUEUES];
	uint32			fwd_cnt[EEE_NUM_FWD_QUEUES];

	/*
	 * Application Ids
	 */
	uint32	app_id;

	/*
	 * Driver and application interfaces
	 */
	eee_rcv_t		rcv;
	eee_pfunc_t		pfunc;
	
	eee_drv_t		drv;

	/*
	 * registered freeing routines
	 */
	void			(*free_routine[EEE_MAX_FREE_ROUTINES])(eee_descPtr_t);

	/*
	 * Statistics and error counters
	 */
	eee_stats_t		stats;

	/*
	 * Test stuff
	 */
	uint32			test_dvt_running[3];	/* EEE_USER_DATA, CONTROL, MGMT */
	uint32			simple_test_app_id;
	uint32			dvt_test_app_id;
	uint32			anp_test_app_id;

	uint32			test_start_valid[NFX_NUM_SLOTS_PER_CHASSIS]
													[NFX_MAX_CPUS_PER_SLOT][3];
	uint32			test_dvt_valid[NFX_NUM_SLOTS_PER_CHASSIS]
													[NFX_MAX_CPUS_PER_SLOT][3];
	uint32			test_dvt_my_sent[NFX_NUM_SLOTS_PER_CHASSIS]
													[NFX_MAX_CPUS_PER_SLOT][3];
	uint32			test_dvt_my_rcvd[NFX_NUM_SLOTS_PER_CHASSIS]
													[NFX_MAX_CPUS_PER_SLOT][3];
	uint32			test_dvt_other_sent[NFX_NUM_SLOTS_PER_CHASSIS]
													[NFX_MAX_CPUS_PER_SLOT][3];
	uint32			test_dvt_other_rcvd[NFX_NUM_SLOTS_PER_CHASSIS]
													[NFX_MAX_CPUS_PER_SLOT][3];
	uint32			test_dvt_send_err[3];
	uint32			test_dvt_rcv_err[3];
} eee_config_t;

extern eee_config_t		eee;
extern eee_stats_t		* eee_stats;
extern nfx_spinlock_t   eee_bufpool_spin;
extern nfx_spinlock_t	eee_descpool_spin;
extern nfx_spinlock_t	eee_fwdqueue_spin;

#ifdef NFP_TXRX
#define EEE_MY_POOL_ID EEE_TXRX_POOL
#endif

#ifdef NFP_FP
#define EEE_MY_POOL_ID EEE_FP_POOL
#endif

#ifdef SSC
#define EEE_MY_POOL_ID EEE_SSC_7000_POOL
#endif

#ifdef SSC_MGMT
#define EEE_MY_POOL_ID EEE_SSC_MGMT_POOL
#endif

#if defined(FCNIM) || defined(FCNIM_TM)
#define EEE_MY_POOL_ID EEE_FC_POOL
#endif

#ifdef PROM
#define EEE_MY_POOL_ID EEE_PROM_POOL
#endif

#define EEE_MY_SHARED_POOL (EEE_MY_POOL_ID - SHARED_POOL_OFFSET)

#if defined(NFP_FP)
#define EEE_REMOTE_QUEUE_INDEX 0
#elif defined(NFP_TXRX)
#define EEE_REMOTE_QUEUE_INDEX 1
#endif

#if defined(NFP_FP)
#define LOCAL_MEM 0x4000000000LL
#define REMOTE_MEM 0x2000000000LL
#define REMOTE_CACHED_MEM 0x6000000000LL
#elif defined(NFP_TXRX)
#define LOCAL_MEM 0x2000000000LL
#define REMOTE_MEM 0x4000000000LL
#define REMOTE_CACHED_MEM 0x7000000000LL
#endif

extern struct eee_rcv_queue *eee_my_receive_queue(void);

#endif	/* _EEE_H */

