/*------------------------------------------------------------------------------
 * Copyright (C) 2001, Agile Storage, Inc.
 *
 * Name         : el-tcp-common.h
 *
 * Description  : Eagle Link (EL).
 * Eagle Link is the interface between Eagle Vision and the NFX.
 * In a first instance, the interface will be TCP/IP.
 * Eagle Vision will be the TCP server, whereas NFXs will act as TCP client.
 *
 * Created by   : Isabelle Cnudde
 *
 * Date Created : 22-OCT-2001
 *
 *------------------------------------------------------------------------------
 */
#ifndef _EL_TCP_COMMON_H_
#define _EL_TCP_COMMON_H_

#include <sys/types.h>
#include <signal.h>
#include <unistd.h>

/*------------------------------------------------------------------------------
 * Macros and constants.
 *------------------------------------------------------------------------------
 */

#define EL_ASSERT(_expr)                                      \
    if (!(_expr))                                             \
    {                                                         \
        printf("%s:%d: ASSERT FAILED: %s in %s\n",            \
            __FILE__, __LINE__, #_expr, __FUNCTION__);        \
        kill(getpid(), SIGTERM);                              \
    }

/*------------------------------------------------------------------------------
 * Prototype of global functions.
 *------------------------------------------------------------------------------
 */

int
el_read(int    fd,
        char * ptr,
        int    maxlen);

int
el_write(int    fd,
         char * ptr,
         int    nbytes);

/*----------------------------------------------------------------------------*/

#endif // _EL_TCP_COMMON_H_
