/*
 * Name:  elog-embedded.c
 *
 * RCS:  $Id: elog-embedded.c,v 1.12 2002/11/27 00:27:31 bob Exp $
 *
 * Copyright (C) 2001, Agile Storage Inc.
 *
 * Description:  Eagle Logging
 *
 * Created By:  Jan Vanhoutte
 *              Pankaj Gupta
 *
 * Date Created:  8/20/2001
 */


// Not used on the SSC
#if !defined (SSC)


// Just as a temporary hack:
// In standalone mode, the linux lib are available 
// The eee (TX/RX) functions are not.
// Use the next lines, to be able to keep building 
// the FS standalone
#ifdef FS_STANDALONE
#define STANDALONE
#endif

#if defined(STANDALONE)
#include <sys/types.h>
#include <sys/time.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
// #include <assert.h>
#include <unistd.h>
#include <time.h>
#endif

#include "nfx-incl.h"

#if defined(STANDALONE)
extern address_t get_ra();
#endif

#include "exports/eee-api.h"
#include "exports/malloc-api.h"
#include "exports/cm-api.h"
#include <exports/elog-api.h>
#include "elog-def.h"

#if defined(AGILE_HARDWARE)
#include "exports/timer-api.h"
#endif

/* 
 * external variables 
 */

/* 
 * These parameters are updated if any corresponding
 * changes are made for elog configuration on the SSC
 */
int elog_enabled = ELOG_ENABLED ;
int elog_severity = LOG_INFO ;
int elog_show_local = ELOG_LOCAL_STDOUT ;

elog_appl_data_t elog_appl_data [ELOG_MAX_APP] ;

/* 
 * internal variables
 */


// static int elog_show_date = ELOG_DISABLED ;
static int elog_show_date = ELOG_SHOW_DATE_MSEC ;

static int elog_format = ELOG_ONE_LINE ;
// static int elog_format = ELOG_MULTIPLE_LINE ;

// static int elog_show_line_file = ELOG_ENABLED ;
static int elog_show_line_file = ELOG_DISABLED ;

// static int elog_show_version_string = ELOG_DISABLED ;

// static int elog_show_sequence = ELOG_ENABLED ;
static int elog_show_sequence = ELOG_DISABLED ;

static int elog_show_severity = ELOG_DISABLED ;
static int elog_show_app_name = ELOG_DISABLED ;

static int elog_sequence = 1 ;


#define ELOG_STRING_MAX    1024


static int32 elog_applicationId = -1;
static int32 e_log_applicationId = -1;


/*
 * prototypes
 */

int
e_log_configure(e_log_configure_msg_t *msg, int size);

/*
 * Todo: get these from libc....
 */

#if defined(STANDALONE)
#else
int32
vsprintf(
	char8		* d,
	const char8	* s,
	va_list		ap
    );

int32
sprintf(char *buf, const char *fmt, ...);
#endif



/*-----------------------------------------------------------------------------
 * elog_receiveEEEMessage
 *
 * Description:  Handle a received EEE message for the elog.
 * 
 * Created By:  Jan Vanhoutte
 *
 * Date Created:  9/10/2001
 */

#if defined (STANDALONE)
#else

void
elog_receiveEEEMessage(
    eee_desc_t   *edesc)
{
    uint32  srcAddr;
    int32   i, rc, msgLen, numBufs, bufType;
    char    *msgPtr;
#define SYSBUF_TYPE_MALLOC    1
#define SYSBUF_TYPE_SHMEM     2

    srcAddr = edesc->hdr.src_port;
    // TODO - should be a macro for EEE_GET_NUM_BUFS_IN_EDESC
    numBufs = (edesc->hdr.control & EDESC_BUFCNT_MASK) >> EDESC_BUFCNT_SHIFT;
    msgLen = 0;
    for (i = 0; i < numBufs; i++)
    {
        msgLen += edesc->bd[i+1].len;  // TODO - first buffer not used???
    }               
    if (numBufs == 1)
    {
        // Serve the request from the system buffer.
        bufType = SYSBUF_TYPE_SHMEM;
        msgPtr = edesc->bd[1].buf;  // TODO - first buffer not used???

        // Done with the descriptor.
        eee_deallocateDesc(edesc);
    }
    else
    {
        // Copy the multi-buffer message to a
        // contiguous malloc buffer and free the
        // received packet.
        bufType = SYSBUF_TYPE_MALLOC;
        msgPtr = eee_ramAlloc(msgLen);
        rc = eee_copyPacketToLocalBuffer(   msgPtr,
                                            edesc,
                                            msgLen,
                                            0);
        if (rc != msgLen)
        {
            E_LOG(class_1, LOG_ERR, elog_applicationId, 0, 0, 0, 
                  ("e-log eee_copyPacketToLocalBuffer failed"));
            // panic("");
        }

        // Done with the packet.
        eee_freePacket(edesc);
    }

    e_log_configure((e_log_configure_msg_t *)msgPtr, msgLen);


    if (bufType == SYSBUF_TYPE_MALLOC)
        eee_ramDealloc(msgPtr);
    else
    // if (bufType == SYSBUF_TYPE_SHMEM)
        eee_deallocateBuffer(msgPtr);

}
#endif  // STANDALONE

/*-----------------------------------------------------------------------------
 * e_log_init
 *
 * Description:  initialize the e-log module.
 * 
 * Created By:  Jan Vanhoutte
 *
 * Date Created:  9/10/2001
 */

int
e_log_init(void)
{
    int loop ;
#if !defined(STANDALONE)
    int rc;
#endif

    for (loop = 0 ; loop < ELOG_MAX_APP; loop++)
    {
        elog_appl_data[loop].severity = LOG_DEBUG ;
        // elog_appl_data[loop].bitmap = 0xFFFFFFFF ;
        elog_appl_data[loop].bitmap = 0x0 ;
    }

#if defined(STANDALONE)
    // as a small hack, value is not that important in standalone mode
    elog_applicationId = 63;
    e_log_applicationId = 23;
#else

    // Obtain the application id of the elog (to send the log messages).
    elog_applicationId = eee_allocateApplicationId("elog");
    if (elog_applicationId == EEE_ERR_INVALID_APP_NAME)
    {
        E_LOG(class_1, LOG_ERR, elog_applicationId, 0, 0, 0, 
             ("e-log does not find applicationId for elog"));
        panic("e-log does not find applicationId for elog");
    }

    // Obtain our application id to receive config messages.
    e_log_applicationId = eee_allocateApplicationId("e_log");
    if (e_log_applicationId == EEE_ERR_INVALID_APP_NAME)
    {
        E_LOG(class_1, LOG_ERR, elog_applicationId, 0, 0, 0, 
              ("e-log does not find applicationId for e_log"));
        panic("");
    }

    // Register receive message function with eee.
    rc = eee_registerReceiveFunc(e_log_applicationId,
                                 elog_receiveEEEMessage,
                                 elog_receiveEEEMessage);
    if (rc != NFX_OK)
    {
        E_LOG(class_1, LOG_ERR, e_log_applicationId, 0, 0, 0, 
              ("e-log eee_registerReceiveFunc failed"));
        // panic("");
    }
#endif

    return (NFX_OK);
}


void
e_log_get_config()
{
	int cmd;
	int rc;

#if defined(FCNIM) || defined(NFP_TXRX)
	cmd = ELOG_CONFIG_GET_REQ;
	rc = eee_sendMessage(&cmd, sizeof(cmd),
						 PACK_UNICAST_PORT_ADDRESS(NFX_SLOTTYPE_SSC,
												   NFX_SSC_UNIX,
												   elog_applicationId,
												   EEE_MGMT,
												   NFX_COS_HIGH,
												   0),
						 PACK_UNICAST_PORT_ADDRESS(nfxMySlotId,
												   nfxMyCPUId,
												   e_log_applicationId,
												   EEE_MGMT,
												   NFX_COS_HIGH,
												   0));
	if (rc != NFX_OK)
		printf("e_log_get_config: eee_sendMessage failed\n");
#endif
#if defined(NFP_TXRX)
	rc = eee_sendMessage(&cmd, sizeof(cmd),
						 PACK_UNICAST_PORT_ADDRESS(NFX_SLOTTYPE_SSC,
												   NFX_SSC_UNIX,
												   elog_applicationId,
												   EEE_MGMT,
												   NFX_COS_HIGH,
												   0),
						 PACK_UNICAST_PORT_ADDRESS(nfxMySlotId,
												   NFX_NFP_FP_ANY,
												   e_log_applicationId,
												   EEE_MGMT,
												   NFX_COS_HIGH,
												   0));
	if (rc != NFX_OK)
		printf("e_log_get_config: eee_sendMessage failed\n");
#endif
}


char *
e_log_var_pa (char *format, ...)
{
    int index = 0 ;
    char *the_msg ;
    va_list ap;

    the_msg =  (char *)eee_ramAlloc(ELOG_STRING_MAX);

    va_start(ap, format);
    if (format != NULL && format[0] != '\0') {
      index += vsnprintf (the_msg, ELOG_STRING_MAX, format, ap);
    }
    va_end(ap);

	return (the_msg);
}


typedef struct elog_IncoreTune
{
    int  absMin;             // the absolute min value needed to accept req
    int  relMin;             // the relative (%) min value needed to accept req
} elog_IncoreTune;

elog_IncoreTune elog_incoreTune [8] = {
    {   2,  0},   // LOG_EMERG
    {  10,  0},   // LOG_ALERT
	{  20,  0},   // LOG_CRIT
	{  30,  0},   // LOG_ERR
	{  40,  1},   // LOG_WARNING
	{  50,  1},   // LOG_NOTICE
	{  60,  1},   // LOG_INFO
	{  80,  3},   // LOG_DEBUG
} ;

int
e_log (log_class_t class,
       log_severity_t severity,
       uint32 app_id,
       uint32 app_type,
       uint32 cookie,
       uint32 code,
       int line,
       char* file,
       char *string)
{
    int rc;
    char log_string[ELOG_STRING_MAX] = "" ;
    int log_size = 0 ;
  
#if defined(STANDALONE)
#else
    elog_eventNtfy_t *snd_msg ;
    uint32 msg_size = 0;
    uint32 str_size = 0;
    uint32 dest;
    uint32 src;
#endif

    if (severity > elog_severity)
      return (NFX_OK);
    
    if (app_id < 0)
      return (NFX_ERR);
    
    if ((app_id < ELOG_MAX_APP) && 
        (severity > elog_appl_data[app_id].severity))
      return (NFX_OK);

    if ((app_id < ELOG_MAX_APP) && 
        (severity > LOG_INFO))
      {
        if ((elog_appl_data[app_id].bitmap & (1 << app_type)) == 0)
          return (NFX_OK);
      }

    elog_sequence++;

    if (elog_show_severity)
    {
        switch (severity)
        {
        case LOG_DEBUG:
            log_size += sprintf (&log_string[log_size], "DEBUG   ");
            break;
        case LOG_INFO:
            log_size += sprintf (&log_string[log_size], "INFO    ");
            break;
        case LOG_NOTICE:
            log_size += sprintf (&log_string[log_size], "NOTICE  ");
            break;
        case LOG_WARNING:
            log_size += sprintf (&log_string[log_size], "WARNING ");
            break;
        case LOG_ERR:
            log_size += sprintf (&log_string[log_size], "ERROR   ");
            break;
        case LOG_CRIT:
            log_size += sprintf (&log_string[log_size], "CRITIC  ");
            break;
        case LOG_ALERT:
            log_size += sprintf (&log_string[log_size], "ALERT   ");
            break;
        case LOG_EMERG:
            log_size += sprintf (&log_string[log_size], "EMERGENCY ");
            break;
        }
    }

    if (elog_show_app_name)
    {
#if defined(STANDALONE)
        log_size += sprintf (&log_string[log_size], "AppId: %d (%d) ", app_id, app_type);
#else
        log_size += sprintf (&log_string[log_size], "AppId: %d (%d) ", app_id, app_type);
        // log_size += sprintf (&log_string[log_size], "%s (%d.%d) ", 
        //                     eee_getAppNameFromId(app_id), app_id, app_type);
        // log_size += sprintf (&log_string[log_size], "%s ", eee_getAppNameFromId(app_id));
#endif
    }
    
    if (elog_show_date)
    {
        uint32 tv_sec = 1013000000;
        uint32 tv_usec = 0;
        
#if defined(STANDALONE)
        {
            struct timeval  now;
            
            gettimeofday (&now, NULL);

            tv_sec = now.tv_sec;
            tv_usec = now.tv_usec;
        }
#endif
#if defined(AGILE_HARDWARE)
        {
            eeeGMT_t now;
        
            eee_TimeGMT (&now);

            tv_sec = now.sec;
            tv_usec = now.usec;
        }
#endif

        
#if defined(STANDALONE)
        strcpy (&(log_string[log_size]), ctime((const time_t*)&tv_sec));
        log_size = strlen(log_string) ;
        log_string[log_size - 1] = ' ' ; // remove the \n 
#endif
#if defined(AGILE_HARDWARE)
        strcpy (&(log_string[log_size]), eee_humandate_short(tv_sec));
        log_size = strlen(log_string) ;
#endif
        if (elog_show_date == ELOG_SHOW_DATE_MSEC)
          log_size += sprintf (&log_string[log_size], " (%03d msec) ", tv_usec / 1000);
        if (elog_show_date == ELOG_SHOW_DATE_USEC)
          log_size += sprintf (&log_string[log_size], " (%06d usec) ", tv_usec);
        
        if (elog_format == ELOG_MULTIPLE_LINE)
        {
			log_size += sprintf (&log_string[log_size], "\n    ");
        }
    }
    
    if (elog_show_line_file)
      {
        log_size += sprintf (&log_string[log_size], "line: %d, file %s ", line, file);
        
        if (elog_format == ELOG_MULTIPLE_LINE)
          {
			log_size += sprintf (&log_string[log_size], "\n    ");
          }
      }
    
    if (elog_show_sequence)
      {
        log_size += sprintf (&log_string[log_size], "nbr: %d ", elog_sequence);
        
        if (elog_format == ELOG_MULTIPLE_LINE)
          {
			log_size += sprintf (&log_string[log_size], "\n    ");
          }
      }
    
    if ((elog_show_local == ELOG_LOCAL_STDOUT) ||
        (elog_show_local == ELOG_ONLY_LOCAL_STDOUT))
    {
        if (log_size)
            printf ("%s", log_string);

        if (string)
            printf ("%s\n", string);
        else
            printf ("\n");
    }
    else if (elog_show_local == ELOG_LOCAL_STDERR)
    {
#if defined(FS_STANDALONE)
        if (log_size)
            fprintf (stderr, "%s", log_string);

        if (string)
            fprintf (stderr, "%s\n", string);
        else
            fprintf (stderr, "\n");
#endif
    }
    
    rc = NFX_OK;
    
    if (elog_show_local == ELOG_ONLY_LOCAL_STDOUT)
    {
        // no need to send to SSC
        return (rc);
    }
    

#if defined(STANDALONE)
#else
    
    msg_size = sizeof (elog_eventNtfy_t) + log_size ;
    if (string)
        str_size = strlen(string);
    msg_size += str_size ;

    // prefent overload
    // Todo: dimensioning 
//    if (msg_size < EEE_BUFSIZE_SMALL)                  // eee.buf_size[EEE_BUF_SMALL]
    {
        if (eee_getAvailableNbrOfBufs(EEE_BUF_SMALL) < 
            (elog_incoreTune[severity].absMin + 
             (eee_getTotalNbrOfBufs(EEE_BUF_SMALL) / 100 * elog_incoreTune[severity].relMin)))
        {
            elog_show_sequence = ELOG_ENABLED;
            return (rc);
        }
    }
//    else
    {
        if (eee_getAvailableNbrOfBufs(EEE_BUF_LARGE) < 
            (elog_incoreTune[severity].absMin + 
             (eee_getTotalNbrOfBufs(EEE_BUF_LARGE) / 100 * elog_incoreTune[severity].relMin)))
        {
            elog_show_sequence = ELOG_ENABLED;
            return (rc);
        }
    }
    
    // Todo: if (msg_size >= ELOG_STRING_MAX)
    
    snd_msg =  (elog_eventNtfy_t *)eee_ramAlloc(msg_size);
    
    snd_msg->msgType = ELOG_MSG_NOTIFY;
    
    snd_msg->class = class;
    snd_msg->severity = severity;
    snd_msg->appId = app_id;
    snd_msg->appType = app_type;
    snd_msg->cookie = cookie;
    snd_msg->code = code;
    if (log_size)
      strcpy (snd_msg->amsg, log_string);
    else
      snd_msg->amsg[0] = 0 ;

    if (string)
      strcpy (&(snd_msg->amsg[log_size]), string);
    
    dest = PACK_UNICAST_PORT_ADDRESS (
								NFX_SLOTTYPE_SSC,
								NFX_SSC_UNIX,
								elog_applicationId,
								EEE_MGMT,
								NFX_COS_HIGH,
								0);
    src = PACK_UNICAST_PORT_ADDRESS (
								nfxMySlotId,
								nfxMyCPUId,
								app_id,
								EEE_MGMT,
								NFX_COS_HIGH,
								0);

    rc = eee_sendMessage (snd_msg, msg_size, dest, src);
    
    eee_ramDealloc (snd_msg);
    
#endif
    
    return (rc);
}



int
e_log_configure(e_log_configure_msg_t *msg, int size)
{
    int tmp;

    if (size < sizeof(e_log_configure_msg_t))
        return (NFX_ERR);

    switch (msg->command)
    {
    case ELOG_CONFIGURE_ENABLE:
        if ((msg->value_1 == ELOG_DISABLED) || (msg->value_1 == ELOG_ENABLED))
		{
            elog_enabled = msg->value_1;
			printf("Elog state is now %s\n", 
								elog_enabled?"Enabled":"Disabled");
		}
        break;
        
    case ELOG_CONFIGURE_SEVERITY:
        if ((msg->value_1 >= LOG_EMERG) && (msg->value_1 <= LOG_DEBUG))
		{
			printf("Set Elog severity to %d, was %d\n",
				   msg->value_1, elog_severity);
            elog_severity = msg->value_1;
		}
        break;

    case ELOG_CONFIGURE_APP_DEFAULT_SEVERITY:
        if ((msg->value_1 >= LOG_EMERG) && (msg->value_1 <= LOG_DEBUG))
            for (tmp = 0 ; tmp  < ELOG_MAX_APP ; tmp++)
                elog_appl_data[tmp].severity  = msg->value_1;
        break;

    case ELOG_CONFIGURE_APP_DEFAULT_BITMAP:
        for (tmp = 0 ; tmp  < ELOG_MAX_APP ; tmp++)
            elog_appl_data[tmp].bitmap  = msg->value_1;
        break;

    case ELOG_CONFIGURE_APP_SEVERITY:
        if (((msg->value_2 >= LOG_EMERG) && (msg->value_2 <= LOG_DEBUG)) &&
            (msg->value_1 >= 0) && (msg->value_1 < ELOG_MAX_APP))
            elog_appl_data[msg->value_1].severity  = msg->value_2;
        break;
        
    case ELOG_CONFIGURE_APP_SET_BITMAP:
        if ((msg->value_1 >= 0) && (msg->value_1 < ELOG_MAX_APP))
            elog_appl_data[msg->value_1].bitmap  |= msg->value_2;
        break;
        
    case ELOG_CONFIGURE_APP_CLEAR_BITMAP:
        if ((msg->value_1 >= 0) && (msg->value_1 < ELOG_MAX_APP))
            elog_appl_data[msg->value_1].bitmap  &= ~msg->value_2;
        break;
        
    case ELOG_CONFIGURE_SHOW_DATE:
        elog_show_date = msg->value_1;
        break;
        
    case ELOG_CONFIGURE_FORMAT:
        if (msg->value_1 == ELOG_ONE_LINE)
            elog_format = msg->value_1;
        else
        if (msg->value_1 == ELOG_MULTIPLE_LINE)
            elog_format = msg->value_1;
        else
        if (msg->value_1 == ELOG_FORMAT_MIN_INFO)
        {
            elog_format = ELOG_ONE_LINE;
            elog_show_date = ELOG_DISABLED;
            elog_show_line_file = ELOG_DISABLED;
            elog_show_sequence = ELOG_DISABLED;
            elog_show_severity = ELOG_DISABLED;
            elog_show_app_name = ELOG_DISABLED;
        }
        else
        if (msg->value_1 == ELOG_FORMAT_MAX_INFO)
        {
            elog_format = ELOG_MULTIPLE_LINE;
            elog_show_date = ELOG_SHOW_DATE_USEC;
            elog_show_line_file = ELOG_ENABLED;
            elog_show_sequence = ELOG_ENABLED;
            elog_show_severity = ELOG_ENABLED;
            elog_show_app_name = ELOG_ENABLED;
        }
        break;

    case ELOG_CONFIGURE_SHOW_LINE:
        if ((msg->value_1 == ELOG_DISABLED) || (msg->value_1 == ELOG_ENABLED))
            elog_show_line_file = msg->value_1;
        break;

    case ELOG_CONFIGURE_SHOW_SEQ:
        if ((msg->value_1 == ELOG_DISABLED) || (msg->value_1 == ELOG_ENABLED))
            elog_show_sequence = msg->value_1;
        break;

    case ELOG_CONFIGURE_SHOW_SEVERITY:
        if ((msg->value_1 == ELOG_DISABLED) || (msg->value_1 == ELOG_ENABLED))
            elog_show_severity = msg->value_1;
        break;
 
    case ELOG_CONFIGURE_SHOW_APP_NAME:
        if ((msg->value_1 == ELOG_DISABLED) || (msg->value_1 == ELOG_ENABLED))
            elog_show_app_name = msg->value_1;
        break;

    case ELOG_CONFIGURE_SHOW_LOCAL:
        if ((msg->value_1 >= ELOG_DISABLED) || 
            (msg->value_1 <= ELOG_ONLY_LOCAL_STDOUT))
		{
            elog_show_local = msg->value_1;
			printf("Elog local show is now %s\n", 
								elog_show_local?"Enabled":"Disabled");
		}
        break;

	case ELOG_CONFIGURE_INIT:
		elog_enabled = msg->value_1;
		elog_severity = msg->value_2;
		break;

    case ELOG_CONFIGURE_TEST:
        if ((msg->value_1 >= LOG_EMERG) && (msg->value_1 <= LOG_DEBUG))
            E_LOG(class_1, msg->value_1, e_log_applicationId, 0, 0, 0, 
                  ("e-log test msg, level %d", msg->value_1));
        break;

   }


    E_LOG(class_1, LOG_INFO, e_log_applicationId, 0, 0, 0, 
          ("e-log configure OK, cmd %d %d %d", msg->command, msg->value_1, msg->value_2));
    
    return (NFX_OK);
}

#endif // (SSC)
