/*
 *-----------------------------------------------------------------
 *
 * Name:
 *
 * RCS:  $Id: esm-stack.c,v 1.7 2002/11/22 20:27:50 maximk Exp $
 *
 * Copyright (C) 2002, Agile Storage, Inc.
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	6/3/02
 *
 *-----------------------------------------------------------------
 */

#include <nfx-incl.h>

#if defined(USE_THREAD_STACKS) || defined(USE_BSS_THREAD_STACKS)

#ifdef USE_THREAD_STACKS

#include <exports/esm-api.h>
#include <exports/pte-api.h>
#include <exports/eee-api.h>

#elif defined(USE_BSS_THREAD_STACKS)

#include <exports/esm-api.h>
#include <exports/queue-api.h>
#include "esm.h"

uchar8	bss_threadstack[ESM_NUM_STACKS][ESM_STACK_SIZE];

#endif

address_t	threadstack_stack[ESM_NUM_STACKS];

struct ts_control {
	uint32		num_avail;
	address_t	* top;
} threadstack;

struct ts_stats {
	uint32		alloc;
	uint32		freed;
} threadstack_stats;

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	6/5/02
 *
 *-----------------------------------------------------------------
 */
address_t
esm_stackAlloc()
{
	address_t	ptr;

	if (threadstack.num_avail == 0)
		return 0;

    NFX_SPIN_LOCK(&esm_spinlock);
	threadstack.num_avail--;
	ptr = *threadstack.top++;
	threadstack_stats.alloc++;
    NFX_SPIN_UNLOCK(&esm_spinlock);

    //	printf("esm_stackAlloc, ptr = %lx\n", ptr);

	return (ptr);

}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	6/5/02
 *
 *-----------------------------------------------------------------
 */
void
esm_stackDealloc(
	address_t	ptr
)
{
	address_t	stackbase;

	if (threadstack.num_avail == ESM_NUM_STACKS) {
		address_t	ra;
		ra = get_ra();
		panic("Threadstack full, ptr = %lx, ra = %lx\n", ptr, ra);
	}

    NFX_SPIN_LOCK(&esm_spinlock);
#ifdef USE_THREAD_STACKS
	stackbase = ptr & ~(PTE_ENTRY_SIZE-1);
#else
	stackbase = ptr;
#endif
	//printf("esm_stackDealloc, ptr = %lx, stackbase = %lx\n", ptr, stackbase);

	*(--threadstack.top) = stackbase;
	threadstack.num_avail++;

	threadstack_stats.freed++;
    NFX_SPIN_UNLOCK(&esm_spinlock);
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	6/5/02
 *
 *-----------------------------------------------------------------
 */
void
esm_stackViewStats()
{
	printf("\n");
	printf("ESM thread stacks total = %d\n", ESM_NUM_STACKS);
	printf("ESM thread stacks avail = %d\n", threadstack.num_avail);
	printf("ESM thread stacks allocated = %d\n", threadstack_stats.alloc);
	printf("ESM thread stacks freed     = %d\n", threadstack_stats.freed);
	printf("\n");
}

#ifdef USE_THREAD_STACKS
extern uint64 crashdump_reserved_stack;
#endif

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	6/5/02
 *
 *-----------------------------------------------------------------
 */
void
esm_stackInit()
{
	uint32		i;

	printf("esm_stackInit(): Enter\n");

	threadstack.top = &(threadstack_stack[ESM_NUM_STACKS]);
	threadstack.num_avail = 0;

#ifdef USE_THREAD_STACKS
	{
	address_t	ptr;
	ptr = 0x5000000000;
	for (i=0; i<ESM_NUM_STACKS; i++) {
		esm_stackDealloc(ptr);
		ptr += PTE_ENTRY_SIZE;	/* 8MB so that the xtlb refill handler works */
	}
	}
#endif

#ifdef USE_BSS_THREAD_STACKS
	for (i=0; i<ESM_NUM_STACKS; i++) {
		esm_stackDealloc((address_t)bss_threadstack[i]);
	}
#endif

	bzero(&threadstack_stats, sizeof(struct ts_stats));
	
#ifdef USE_THREAD_STACKS
	crashdump_reserved_stack = esm_stackAlloc();
#endif

	printf("esm_stackInit(): Leave\n");
}

#endif	/* USE_THREAD_STACKS || USE_BSS_THREAD_STACKS */

