/* vi:ts=4
 *-----------------------------------------------------------------
 *
 * Name: event-as-cron.c
 *
 * RCS:  $Id: event-as-cron.c,v 1.1 2002/09/18 23:41:28 pankajg Exp $
 *
 * Copyright (C) 2001, Agile Storage, Inc.
 *
 * Description: Used by cron job to request autosupport report 
 *              generation
 *
 * Created by:    Pankaj Gupta
 *
 * Date Created:
 *
 *-----------------------------------------------------------------
 */

#include <stdio.h>      // fprintf(), stderr
#include <stdlib.h>     // exit()
#include <string.h>     // memset()
#include <unistd.h>     // close()

#include <sys/time.h>
#include <sys/socket.h>
#include <signal.h>
#include <fcntl.h>
#include <netinet/in.h>
#include <errno.h>

#include "exports/elog-api.h"

#include "exports/anpssc-api.h"
#include "exports/event-api.h"

/*
 * Constants.
 */

// Wait for Agent to respond (10s)
static struct timeval waitEventdTo = {10, 0};

/*
 *------------------------------------------------------------------------------
 * Name         : main
 *
 * Description  : Trigger an auto support report
 *
 * Created by   : Pankaj Gupta 
 *
 * Date Created : 06-NOV-2001
 *------------------------------------------------------------------------------
 */

int32 main(int argc, char **argv)
{
	int                 rsplen;
	int					myappid;
	event_configMsg_t   eventCfgReq;
	event_configMsg_t   eventCfgRsp;

	myappid = eee_getApplicationId (EVENT_SERVICE_NAME);

	EVENT_DBG_PRINT(("EVENT Autosupport Cron: Request autosupport report"));

	bzero((void *) &eventCfgReq, sizeof(event_configMsg_t));
	bzero((void *) &eventCfgRsp, sizeof(event_configMsg_t));

	eventCfgReq.msgType = NFX_EVENT_AS_GENERATE_REPORT;

	/*
	 * Send the request to eventd daemon on SSC
	 */
	rsplen = sendAgileMsg(-1, &eventCfgReq, sizeof(event_configMsg_t),
				"eventd", NFX_SSC_UNIX, SSC_SLOT, MGMTPLANE,
					&waitEventdTo, &eventCfgRsp, sizeof(event_configMsg_t));

	if (rsplen != sizeof(event_configMsg_t))
		return NFX_ERR;

    return NFX_OK;
}
