/*------------------------------------------------------------------------------
 * Copyright (C) 2001, Agile Storage, Inc.
 *
 * Name         : event-as-trigger.c
 *
 * Description  :
 * Trigger automatically autosupport.
 * Usually done once a week.
 *
 * Created by   : Isabelle Cnudde
 *				  Pankaj Gupta
 *
 * Date Created : 13-SEP-2001
 *
 *------------------------------------------------------------------------------
 */

#include "event-as.h"
#include "exports/anpssc-api.h"

/*------------------------------------------------------------------------------
 * Prototype of local functions.
 *------------------------------------------------------------------------------
 */

static void
ems_sendAutosupportTrigger(void);

/*------------------------------------------------------------------------------
 * Local functions.
 *------------------------------------------------------------------------------
 */

/*------------------------------------------------------------------------------
 * Name         : ems_sendAutosupportTrigger()
 *
 * Description  : Send an 'autosupport trigger' to EMS.
 *
 * Created by   : Isabelle Cnudde
 *				  Pankaj Gupta
 *
 * Date Created : 13-SEP-2001
 *
 *------------------------------------------------------------------------------
 */

static void
ems_sendAutosupportTrigger(void)
    
{
    ems_msgType_t   reqType = EMS_AUTOSUPPORT_REQ;
    ems_msg_t       req;

    EVENT_DBG_PRINT("> ems_sendAutosupportTrigger()\n");

    /* Fill the request.
     */
    bzero((void *) &req.buf, sizeof(ems_msgHdr_t));    
    memcpy((void *) &req.buf, (void *) &reqType, sizeof(ems_msgType_t));
    req.len = sizeof(ems_msgHdr_t);

    /* Send the request to the registry manager located on the SSC.
     */
    if (sendAgileMsg(-1,
                     req.buf, req.len,
                     "ems", NFX_SSC_UNIX, SSC_SLOT, MGMTPLANE,
                     0, NULL, NULL) != 0)
    {
        perror("sendAgileMsg() failed");
    }
    
    EVENT_DBG_PRINT("< ems_sendAutosupportTrigger()\n");
    return;
}

/*------------------------------------------------------------------------------
 * Global functions.
 *------------------------------------------------------------------------------
 */

/*------------------------------------------------------------------------------
 * Name         : main
 *
 * Description  : XXX
 *
 * Created by   : Isabelle Cnudde
 *				  Pankaj Gupta
 *
 * Date Created : 13-SEP-2001
 *
 *------------------------------------------------------------------------------
 */

int
main(void)
    
{
    EVENT_DBG_PRINT("> main()\n");
    
    /*  Trigger EMS autosupport.
     */
    ems_sendAutosupportTrigger();

    EVENT_DBG_PRINT("< main()\n");
    return(NFX_OK);
}

/*----------------------------------------------------------------------------*/
