/*
 *-----------------------------------------------------------------
 *
 * Name:	event-mgr-ui.c
 *
 * RCS:  $Id: event-mgr-ui.c,v 1.6 2002/06/12 18:24:46 pankajg Exp $
 *
 * Copyright (C) 2002, Agile Storage, Inc.
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	5/13/02
 *
 *-----------------------------------------------------------------
 */

#ifndef SSC

#include "nfx-incl.h"
#include "exports/event-api.h"
#include "exports/cm-api.h"

uchar8	* event_name_string[] = {
	"NODE",
	"PCC",
	"SCC",
	"CARD",
	"CPU",
	"VOLUME",
	"MOUNT",
	"IP IF",
	"PORT",
};

extern void (*eventClientList[NFX_EVENT_MAX+1][MAX_CLIENT_PER_EVENT])();

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	5/12/02
 *
 *-----------------------------------------------------------------
 */
uchar8 *
event_mgr_get_event_name(
	int32	eventId
)
{
	if (eventId <= NFX_EVENT_MAX)
		return event_name_string[eventId];
	else
		return "UNKNOWN";
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	5/12/02
 *
 *-----------------------------------------------------------------
 */
void
event_mgr_ui_client_func(
	nfx_eventMsg_t	* mptr
)
{
	printf("EVENT client, event = %d\n", mptr->eventType);

	switch(mptr->eventType) {
		case NFX_EVENT_NODE:
			break;
		case NFX_EVENT_PCC:
			break;
		case NFX_EVENT_SCC:
			break;
		case NFX_EVENT_CARD:
			printf("slot = %d\n", mptr->param.cardEvent.slot);
			printf("state = %d\n", mptr->param.cardEvent.state);
			printf("msg = %s\n", mptr->reason);
			break;
		case NFX_EVENT_CPU:
			printf("slot = %d\n", mptr->param.cpuEvent.slot);
			printf("cpu = %d\n", mptr->param.cpuEvent.cpu);
			printf("state = %d\n", mptr->param.cpuEvent.state);
			printf("msg = %s\n", mptr->reason);
			break;
		case NFX_EVENT_VOLUME:
			break;
		case NFX_EVENT_NFS:
			break;
		case NFX_EVENT_IP_IF:
			break;
		case NFX_EVENT_PORT:
			break;
		default:
			break;
	}
}

/*-----------------------------------------------------------------
 * Name:	event_mgr_ui()
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	5/13/02
 *
 *-----------------------------------------------------------------
 */
int32
event_mgr_ui(
	int32	argc,
	uchar8	* argv[]
)
{
	int32			eventId;
	nfx_eventMsg_t	msg;
	uint32			i, j;

	memset(&msg, 0, sizeof(nfx_eventMsg_t));

	if (!(strncmp(argv[1], "po", 2))) {
		/* post new event */
		if (argc != 3)
			return -1;
		eventId = atoi(argv[2]);
		switch(eventId) {
			case NFX_EVENT_CARD:
				msg.eventType = NFX_EVENT_CARD;
				strcpy(msg.reason, "Card Event");
				msg.param.cardEvent.slot = nfxMySlotId;
#ifdef NFX_MOD_NFP
				msg.param.cardEvent.cardType = NFX_CARDTYPE_NFP;
#elif defined(NFX_MOD_FCNIM)
				msg.param.cardEvent.cardType = NFX_CARDTYPE_FCNIM;
#elif defined(NFX_MOD_SSC)
				msg.param.cardEvent.cardType = NFX_CARDTYPE_SSC;
#endif
				msg.param.cardEvent.state = EVENT_STATE_UP;
				break;
			case NFX_EVENT_CPU:
				msg.eventType = NFX_EVENT_CPU;
				strcpy(msg.reason, "CPU Event");
				msg.param.cpuEvent.slot = nfxMySlotId;
				msg.param.cpuEvent.cpu = nfxMyCPUId;
				msg.param.cpuEvent.state = EVENT_STATE_UP;
				break;
			default:
				msg.eventType = eventId;
				break;
		}
		event_postEvent(&msg);
	}
	else if (!(strncmp(argv[1], "reg", 3))) {
		/* register reception of event */
		if (argc != 3)
			return -1;
		eventId = atoi(argv[2]);
		event_registerClient(eventId, event_mgr_ui_client_func);
	}
	else if (!(strncmp(argv[1], "li", 2))) {
		/* list event types */
		for (i=NFX_EVENT_MIN; i<=NFX_EVENT_MAX; i++)
			printf("%d : %s\n", i, event_mgr_get_event_name(i));
	}
	else if (!(strncmp(argv[1], "stat", 3))) {
		/* display event statistics */
		printf("   event      posted    received\n");
		printf("--------------------------------\n");
		for (i=NFX_EVENT_MIN; i<=NFX_EVENT_MAX; i++)
			printf("%8s%12d%12d\n", event_mgr_get_event_name(i),
									event_stats[i].posted,
									event_stats[i].rcvd);
		printf("--------------------------------\n\n");
		printf("   event              client    delivered\n");
		printf("-----------------------------------------\n");
		for (i=NFX_EVENT_MIN; i<=NFX_EVENT_MAX; i++) {
			printf("%8s", event_mgr_get_event_name(i));
			if (eventClientList[i][0]) {
				for (j=0;j < MAX_CLIENT_PER_EVENT; j++) {
					if (eventClientList[i][j]) {
						if (j != 0)
							printf("        ");
						printf("%20p", eventClientList[i][j]);
						printf("%13d", event_client_stats[i][j]);
						printf("\n");
					}
				}
			}
			else
				printf("\n");
		}
		printf("-----------------------------------------\n\n");
	}
	else if (!(strncmp(argv[1], "debug", 2))) {
		if (argc != 3)
			printf("event debug <on|off>\n");
		else if (!(strncmp(argv[2], "on", 2)))
			event_mgr_debug = 1;
		else if (!(strncmp(argv[2], "off", 3)))
			event_mgr_debug = 0;
		else
			printf("event debug <on|off>\n");
	}
	else
		return -1;

	return 0;
}

#endif	/* !SSC */

