/*
 *-----------------------------------------------------------------
 *
 * Name:	event-mgr.c
 *
 * RCS:  $Id: event-mgr.c,v 1.8 2002/10/08 03:16:18 johnh Exp $
 *
 * Copyright (C) 2002, Agile Storage, Inc.
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	5/12/02
 *
 *-----------------------------------------------------------------
 */

#ifndef SSC

#include "nfx-incl.h"
#include "exports/event-api.h"
#include "exports/eee-api.h"
#include "exports/cm-api.h"

int32	event_mgr_id;

void	(*eventClientList[NFX_EVENT_MAX+1][MAX_CLIENT_PER_EVENT])();

event_mgr_stats_t	event_stats[NFX_EVENT_MAX+1];
uint32				event_client_stats[NFX_EVENT_MAX+1][MAX_CLIENT_PER_EVENT];

uchar8	* event_mgr_get_event_name(int32 eventId);

int32	event_mgr_debug = 0;

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	5/12/02
 *
 *-----------------------------------------------------------------
 */
void
event_registerClient(
	int32	eventId,
	void	(*rcv_func)(nfx_eventMsg_t	* mptr)
)
{
	uint32	i;

	NFX_ASSERT(eventId > NFX_EVENT_MIN);
	NFX_ASSERT(eventId <= NFX_EVENT_MAX);

	for (i=0; i<MAX_CLIENT_PER_EVENT; i++) {
		if (eventClientList[eventId][i] == 0) {
			eventClientList[eventId][i] = rcv_func;
			return;
		}
	}

	panic("Too Many clients for event %d\n", eventId);
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	5/12/02
 *
 *-----------------------------------------------------------------
 */
void
event_deliverToApp(
	nfx_eventMsg_t	* mptr
)
{
	int32	eventId;
	uint32	i;

	eventId = mptr->eventType;

	for (i=0; i<MAX_CLIENT_PER_EVENT; i++) {
		if (eventClientList[eventId][i]) {
			event_client_stats[eventId][i]++;
			(*eventClientList[eventId][i])(mptr);
		}
	}
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	5/12/02
 *
 *-----------------------------------------------------------------
 */
void
event_distributeEvent(
	uint32	slot,
	uint32	cpu,
	uint32	sp,
	nfx_eventMsg_t	* mptr
)
{
	uint32	dp;

	dp = PACK_UNICAST_PORT_ADDRESS(slot,
								cpu,
								event_mgr_id,
								EEE_MGMT,
								NFX_COS_HIGH,
								PORT_BEST_FIT);

	eee_sendMessage(mptr, sizeof(nfx_eventMsg_t), dp, sp);
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	5/12/02
 *
 *-----------------------------------------------------------------
 */
void
event_postEvent(
	nfx_eventMsg_t	* mptr
)
{
	uint32	sp;

	EVENT_MGR_DEBUG(("event_postEvent, event = %d\n", mptr->eventType));

	NFX_ASSERT(mptr->eventType > NFX_EVENT_MIN);
	NFX_ASSERT(mptr->eventType <= NFX_EVENT_MAX);

	event_stats[mptr->eventType].posted++;

	mptr->msgType = NFX_EVENT_POST;

	/*
	 * send to other slots/cpus
	 */
	sp = PACK_UNICAST_PORT_ADDRESS(nfxMySlotId,
								nfxMyCPUId,
								event_mgr_id,
								EEE_MGMT,
								NFX_COS_HIGH,
								PORT_BEST_FIT);
	
    /* SSC */
	event_distributeEvent(NFX_SLOTTYPE_SSC, NFX_SSC_UNIX, sp, mptr);
    /* TXRX */
	if (!isTXRX())
		event_distributeEvent(NFX_SLOTTYPE_NFP, NFX_NFP_TXRX1, sp, mptr);
    /* FP */
	if (!isFP())
		event_distributeEvent(NFX_SLOTTYPE_NFP, NFX_NFP_FP2, sp, mptr);
    /* FC CPU 3 is Slice FC0 */
	if (!CM_IS_FC_CPUID3)
		event_distributeEvent(NFX_SLOTTYPE_FC, NFX_FCNIM_CPUID3, sp, mptr);
    /* FC CPU 2 is Slice FC1 */
	if (!CM_IS_FC_CPUID2)
		event_distributeEvent(NFX_SLOTTYPE_FC, NFX_FCNIM_CPUID2, sp, mptr);
    /* FC CPU 1 is Slice FC2 */
	if (!CM_IS_FC_CPUID1)
		event_distributeEvent(NFX_SLOTTYPE_FC, NFX_FCNIM_CPUID1, sp, mptr);
    /* FC CPU 0 is Slice FC3 */
	if (!CM_IS_FC_CPUID0)
		event_distributeEvent(NFX_SLOTTYPE_FC, NFX_FCNIM_CPUID0, sp, mptr);

	/*
	 * send to apps on my CPU
	 */
	event_deliverToApp(mptr);
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	5/12/02
 *
 *-----------------------------------------------------------------
 */
void
event_mgr_rcv(
	eee_descPtr_t	edesc
)
{
	nfx_eventMsg_t	* mptr;
	eee_descBuf_t *eBufPtr;

	eBufPtr = edesc_firstBuf(edesc);
	mptr = (nfx_eventMsg_t *)eBufPtr->buf;

	EVENT_MGR_DEBUG(("event_mgr_rcv, event = %s, edesc = %p\n",
							event_mgr_get_event_name(mptr->eventType), edesc));

	event_stats[mptr->eventType].rcvd++;

	event_deliverToApp(mptr);

    eee_freePacket(edesc);
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	5/12/02
 *
 *-----------------------------------------------------------------
 */
void
event_init()
{
	event_mgr_id = eee_allocateApplicationId(EVENT_SERVICE_NAME);
	eee_registerReceiveFunc(event_mgr_id, event_mgr_rcv, event_mgr_rcv);

	bzero(eventClientList, sizeof(eventClientList));
	bzero(event_stats, sizeof(event_stats));
	bzero(event_client_stats, sizeof(event_client_stats));
}

#endif
