/*------------------------------------------------------------------------------
 * Copyright (C) 2001, Agile Storage, Inc.
 *
 * Name         : evm-lv.h
 *
 * Description  : Volume Manager (EVM) - Logical Volume management
 *
 * Created by   : Isabelle Cnudde
 *
 * Date Created : 19-DEC-2001
 *
 *------------------------------------------------------------------------------
 */

#ifndef _EVM_LV_H_
#define _EVM_LV_H_

#include "evm-srvr-includes.h"
#include "nfx-incl.h"
#include "exports/event-api.h"

/*------------------------------------------------------------------------------
 * Global variables.
 *------------------------------------------------------------------------------
 */

extern nfx_spinlock_t   evmVolLock;
extern address_t        evm_volHash;

/*------------------------------------------------------------------------------
 * Function prototypes.
 *------------------------------------------------------------------------------
 */

/* Initialization.
 */
int32
evm_lv_init(void);

/* Takeover operations.
 */
void
evm_startLookingForTakeover(void);

void
evm_stopLookingForTakeover(void);

int32
evm_tkoClockTick(uint32   tmr_handle,
                 void *  usr_param1,
                 void *  usr_param2,
                 void *  usr_param3,
                 void *  usr_param4);

void
evm_lv_writeMgrAreaDone(evm_IOStat_t * stat);

void
evm_lv_readTkoAreaDone(evm_IOStat_t * stat);

/* Volume operations.
 */
int32
evm_createVol(evm_MsgCreateLV_t * cmsg);

int32
evm_addLun(evm_MsgAddStrg_t * amsg);

int32
evm_offlineLV(evm_volId_t   id,
              boolean       doLock);

uint32
evm_getListVol(evm_volId_t *    ids);

boolean
evm_lvExists (evm_volId_t   id,
              evm_vol_t **  vol);

evm_vol_t *
evm_getVol(evm_volId_t id);

/* IO mapping.
 */
evm_lun_t*
evm_getPhys(uint64      lpos,
            int         blks,
            evm_vol_t * lv,
            uint32 *    pos,
            uint32 *    nblks,
            boolean     accessPrivate);

/* Misc. utilities.
 */
void
evm_printVolErr(void);

void
evm_resetVolErr(void);

/*----------------------------------------------------------------------------*/

#endif // _EVM_LV_H_
