/*------------------------------------------------------------------------------
 * RCS:  $Id: evm-perf.c,v 1.13 2002/10/22 01:30:32 jong Exp $
 * Copyright (C) 2002, Agile Storage, Inc.
 *
 * Name		 : evm-perf.c
 *
 * Description  : 
 *
 * Created by   : Lan Chin
 *				Isabelle Cnudde
 *              Shyh-Pei Yen
 *
 * Date Created : 25-JUN-2002
 *
 *-----------------------------------------------------------------
 */

#include "nfx-incl.h"
#include "exports/hash-api.h"
#include "exports/timer-api.h"
#include "evm-lv.h"
#include "evm-msg.h"
#include "evm-debug.h"

#ifdef SY_IO_PERFORMANCE_TEST

/* this skip the whole performance testing code if not defined... */

#undef USE_NEW_COD_EVERYTIME


/*------------------------------------------------------------------------------
 * Type definition.
 *------------------------------------------------------------------------------
 */

typedef struct
{
	int32		key;	// hash key
	int32		vidx;	// volume index for this test (always start from 0)
	int32		id;		// test id
	evm_pReq_t	type;	// request type
	evm_IOReq_t	req;
	char		*buf;	// write data
	int32		len;	// nbytes of data
	ushort16	chksum;	// calculated write checksum
	eeeGMT_t	ts;		// timestamp at request sent
} evm_perf_cb_t;

/*------------------------------------------------------------------------------
 * Global variables.
 *------------------------------------------------------------------------------
 */

static int32   evm_srvr_test_init = 0;

#define EVM_SRVR_TEST_HASH_BUCKETS  255

static address_t	evm_srvr_test_hash;
static int32		evm_srvr_test_seqno = 1;

#define EVM_MAX_NUM_OF_LVS		10

#define EVM_PERF_FIXED		0
#define EVM_PERF_SEQ		1
#define EVM_PERF_SKIPSEQ	2
#define EVM_PERF_RAND		3
#define EVM_PERF_LOOPBACK	4


/* command line parameters */
static uint32			totalvols;
static int32			perf_type;		/* m_write/m_read/u_write/u_read */
#if 0
static int32			perf_respt;		/* for checking respone time only */
#endif
static evm_volOffset_t	perf_offset;	/* LBA offset */
static int32			perf_nBlks;		/* total blocks to read/write */
static int32			perf_opoffset;	/* fixed|skip|seq|rand */
static uint32			perf_nreqs;		/* total requests to send out for each volume */
static uint32			perf_Qlen;		/* keep request queue to this length for each volume */
static evm_vol_t        *perf_lvp[EVM_MAX_NUM_OF_LVS];	/* LUN volume array */
static uint32			perf_fill;
static uint32			perf_doRspTime = 0;		/* include queue depth */
static uint32			perf_rspTimeSize = 1000;

/* stats keeper */
static uint32		*rsptime = NULL;
static uint32		*rsptime1 = NULL;
static uint32		*scsiQdepth = NULL;
static uint32		nextid[EVM_MAX_NUM_OF_LVS];
static eeeGMT_t		perf_before_reqdump;
static eeeGMT_t		perf_after_reqdump;
static eeeGMT_t		perf_all_finished;
static uint32		totalrsps[EVM_MAX_NUM_OF_LVS];


/*------------------------------------------------------------------------------
 * Local function prototypes.
 *------------------------------------------------------------------------------
 */

static uint32
evm_srvr_test_hashFunc(uint32 key);

static int32
evm_srvr_test_hashCompare(evm_perf_cb_t *cb, int32	compare_key);

static void
evm_srvr_test_hashRelease(evm_perf_cb_t *cb);

static evm_perf_cb_t *
evm_createPerfCb(void);

static void
evm_deletePerfCb(evm_perf_cb_t *cb);

static ushort16
evm_doChksum(ushort16 *buf, int32 len);

static void
evm_readComplete(evm_IOStat_t *stat);

static int32
evm_doNextRead(evm_perf_cb_t *cb);

static int32
evm_doRead(int32				vidx,
			int32				n,
			evm_pReq_t			type,
			evm_vol_t		    *lv,
			evm_volOffset_t		offset,
			evm_volSize_t		size,
			boolean				doUltra);

static void
evm_writeComplete(evm_IOStat_t *stat);

static int32
evm_doNextWrite(evm_perf_cb_t *cb);

static int32
evm_doWrite(int32			vidx,
			int32			n,
			evm_pReq_t		type,
			evm_vol_t	    *lv,
			evm_volOffset_t	offset,
			evm_volSize_t	size,
			boolean			doUltra);


/*-----------------------------------------------------------------
 * Name:		mysqrtf
 * Description: square root of a float type
 *              This routine is copied somewhere on the internet...
 * Created by : Shyh-Pei Yen
 * Date Created: 7/29/02
 *-----------------------------------------------------------------
 */

static float
mysqrtf(float x)
{
float eplison = 0.0001;
float guess = 1.0;
float a;

	if (x < 0)
		return -1;

	while (1)
	{
		a = guess * guess;
		if (a > x)
			a = a - x;
		else
			a = x - a;

		if (a < eplison)
			return guess;

		guess = (x/guess + guess) / 2.0;
	}

	/* shouldn't reach here */
	return -1;
	
} /* mysqrtf */

/*-----------------------------------------------------------------
 * Name:		evm_srvr_test_hashFunc
 *
 * Description: hash function for the general hash routine
 *			  (see hash-api.h)
 *
 *			  called to return the key or hash bucket
 *			  for the test hash
 *
 * Created by : Lan Chin
 *
 * Date Created: 9/28/01
 *-----------------------------------------------------------------
 */

static uint32
evm_srvr_test_hashFunc(uint32 key)
	
{
	return (key % EVM_SRVR_TEST_HASH_BUCKETS);
} /* evm_srvr_test_hashFunc */

/*-----------------------------------------------------------------
 * Name:		evm_srvr_test_hashCompare
 *
 * Description: compare function for the general hash routine
 *			  (see hash-api.h)
 *
 *			  called to check for a key match for the test hash
 *
 * Created by : Lan Chin
 *
 * Date Created: 9/28/01
 *-----------------------------------------------------------------
 */

static int32
evm_srvr_test_hashCompare(evm_perf_cb_t *cb, // data stored in hash
						 int32 compare_key)  // data to compare with
{

	if (cb->key == compare_key)
	{
		/* return 0 for a match */
		return 0;
	}

	return 1;

} /* evm_srvr_test_hashCompare */

/*-----------------------------------------------------------------
 * Name:		evm_srvr_test_hashRelease
 *
 * Description: release function for the general hash routine
 *			  (see hash-api.h)
 *
 * Note:		only called by hash module if the entire
 *			  hash table is destoryed !! 
 *			  will NOT be called for a hash_delete()
 *
 *			  called to deallocate memory (if necc) for a node
 *			  in the IO request hash
 *
 * Created by : Lan Chin
 *
 * Date Created: 9/28/01
 *-----------------------------------------------------------------
 */

static void
evm_srvr_test_hashRelease(evm_perf_cb_t * cb)
	
{
	/*
	 * we have to deallocate the write buffer as well
	 * since evm will set eee control to EDESC_BD_CMD_NO_DEALLOC
	 */
	if (cb->buf)
	{
		void *bufBase;

		if (eee_isUltraBuf(cb->buf, &bufBase))
		{
			EVM_DEALLOC_BUF(cb->buf);
		}
		else
		{
			EVM_RAM_DEALLOC(cb->buf);
		}
	}
	EVM_RAM_DEALLOC(cb);

	return;

} /* evm_srvr_test_hashRelease */

/*-----------------------------------------------------------------
 * Name:		evm_createPerfCb
 *
 * Description: called to create a test control block
 *
 * Created by : Lan Chin
 *
 * Date Created: 9/28/01
 *-----------------------------------------------------------------
 */

static evm_perf_cb_t *
evm_createPerfCb(void)
	
{
	int32		   rc;
	evm_perf_cb_t   *cb;

	EVM_RAM_ALLOC_ZERO(cb, evm_perf_cb_t, sizeof(evm_perf_cb_t));
	if (cb == NULL)
		return NULL;

	cb->key = evm_srvr_test_seqno++;
	/* a signature for evm-io to send qdepth response */
	cb->req.clientPrivate = cb->key;
	rc = hash_insert(evm_srvr_test_hash, (void *) ((uintptr_t)cb->key),
					 (void *) ((uintptr_t)cb));
	if (rc == NFX_ERR)
	{
		evm_srvr_test_hashRelease(cb);
		return NULL;
	}

	return cb;

} /* evm_createPerfCb */

/*-----------------------------------------------------------------
 * Name:		evm_deletePerfCb
 *
 * Description: called to delete a test control block
 *
 * Created by : Lan Chin
 *
 * Date Created: 9/28/01
 *-----------------------------------------------------------------
 */

static void
evm_deletePerfCb(evm_perf_cb_t * cb)
	
{

	hash_delete(evm_srvr_test_hash, (void *) ((uintptr_t)cb->key),
				(void *) ((uintptr_t)cb->key));
	evm_srvr_test_hashRelease(cb);

} /* evm_deletePerfCb */

/*-----------------------------------------------------------------
 * Name:		evm_doChksum
 *
 * Description: called to calculate checksum
 *			  based on standard TCP/IP checksum algorithm
 *
 * Created by : Lan Chin
 *
 * Date Created: 9/28/01
 *-----------------------------------------------------------------
 */

static ushort16
evm_doChksum(ushort16 * buf,
			 int32	  len)
	
{
	ushort16	sum=0;
	int32	   n=len;

	while (n--)
		sum += *buf++;

	while (sum > 0xffff)
		sum = (sum & 0xffff) + (sum >> 16l);

	sum = (~sum & 0xffff);
	return sum;

} /* evm_doChksum */

/*-----------------------------------------------------------------
 * Name:		evm_readComplete
 *
 * Description: test completion routine for evm_IOReq Read
 *
 * Created by : Lan Chin
 *
 * Date Created: 9/28/01
 *-----------------------------------------------------------------
 */

static void
evm_readComplete(evm_IOStat_t * stat)
{
int32 i, vidx, ridx, rsptm;
int32 key;
ushort16 chksum;
uchar8 *bufp;
evm_perf_cb_t *cb;
eeeGMT_t now;

	/* GET timestamp value here */
	eee_TimeGMT(&now);

	key = stat->clientPrivate;
	cb = (evm_perf_cb_t *) hash_lookup(evm_srvr_test_hash,
									   (void *) ((uintptr_t)key), 
									   (void *) ((uintptr_t)key));
	if (cb == NULL)
	{
		printf("\nEVM PERF: read cb ERROR, cannot find perf cb for key %d\n", key);
		return;
	}

	/*
	 * if read ok, calculate checksum and make sure
	 * it matches with the write
	 */
	if (stat->status != EVM_STAT_NO_ERROR)
	{
		printf("EVM PERF[%d]: read request failed: %d\n", cb->id, stat->status);
		evm_deletePerfCb(cb);
		return;
	}

	if (cb->type == u_read)
	{
		// printf("EVM PERF[%d]: read OK, free COD\n", cb->id);
		evm_sendFreeCodMsg(stat->rDataHandle.codHandle);   // free read COD
	}

	vidx = cb->vidx;

	if (perf_doRspTime)
	{
		ridx = vidx * perf_rspTimeSize + totalrsps[vidx];
		rsptime[ridx] = now.usec;
		rsptime1[ridx] = now.sec;
		// scsiQdepth[ridx] = stat->reserved[0];
	}

	totalrsps[vidx]++;
	if (totalrsps[vidx] >= perf_nreqs)
	{
		printf("EVM performance test for vol[%d] done\n", vidx);
		eee_TimeGMT(&perf_all_finished);
		evm_deletePerfCb(cb);
		return;
	}

	/* issue next read request */
	if (nextid[vidx] < perf_nreqs)
	{
		if (evm_doNextRead(cb) == NFX_ERR)
		{
			printf("EVM PERF[%d]: can't do read for offset %llx nBlks %lld\n",
				cb->id, cb->req.offset, cb->req.reqSize);
			evm_deletePerfCb(cb);
		}
	}

	return;

} /* evm_readComplete */


/*-----------------------------------------------------------------
 * Name:		evm_doNextRead
 *
 * Description: called to test EVM srvr meta or user read IO
 *
 * Created by : Shyh-Pei Yen
 *
 * Date Created: 8/14/02
 *-----------------------------------------------------------------
 */

static int32
evm_doNextRead(evm_perf_cb_t *cb)
{
	int32		   rc;

	cb->id = nextid[cb->vidx]++;

	if (perf_opoffset == EVM_PERF_SEQ)
		cb->req.offset = perf_offset + (cb->id * cb->req.reqSize);
	else if (perf_opoffset == EVM_PERF_SKIPSEQ)
	{
		if (perf_nBlks == 16)
			cb->req.offset = perf_offset + (cb->id * 32);
		else
			cb->req.offset = perf_offset + (cb->id * 16);
		// cb->req.offset = perf_offset + (cb->id * (cb->req.reqSize+2));
	}
	/* no need to handle loopback's offset as it doesn't matter */

	eee_TimeGMT(&cb->ts);

	rc = evm_IOReq(&cb->req, evm_readComplete);
	if (rc == NFX_ERR)
	{
		printf("EVM PERF[%d]: evmIOReq/read failed\n", cb->id);
		return NFX_ERR;
	}

	return NFX_OK;

} /* evm_doNextRead */


/*-----------------------------------------------------------------
 * Name:		evm_doRead
 *
 * Description: called to unit test EVM srvr meta or user read IO
 *
 * Created by : Lan Chin
 *
 * Date Created: 9/28/01
 *-----------------------------------------------------------------
 */

static int32
evm_doRead(int32				vidx,
			int32				n,
			evm_pReq_t			type,
			evm_vol_t		    *lv,
			evm_volOffset_t		offset,
			evm_volSize_t		size,
			boolean				doUltra)
{
	int32 rc;
	evm_perf_cb_t *cb;
	char *buf;

	cb = evm_createPerfCb();
	if (cb == NULL)
		return NFX_ERR;

	cb->type = type;
	cb->req.reqType = EVM_READ_REQ;
	cb->req.dataType = (type == m_read) ? EVM_META_DATA_REQ : EVM_USER_DATA_REQ;
	cb->req.id = lv->id;
	cb->req.offset = offset;
	cb->req.reqSize = size;
	cb->id = n;
	cb->vidx = vidx;
	cb->len = size * EVM_LBLK_SIZE;

	if (type == m_read)
	{
		EVM_RAM_ALLOC_ALIGNED(buf, char, cb->len);
		if (buf == NULL)
		{
			printf("\nEVM PERF META IO[%d]: no memory for read buffer\n", n);
			evm_deletePerfCb(cb);
			return NFX_ERR;
		}
		cb->buf = buf;
		cb->req.wDataHandle.dataPtr = buf;
	}
	else	/* u_read */
	{
		/* do nothing as expecting a COD handle (SAC buffer) */
		/* should returned to us */
	}

	/* put timestamp into cb */
	eee_TimeGMT(&cb->ts);

	rc = evm_IOReq(&cb->req, evm_readComplete);
	if (rc == NFX_ERR)
	{
		printf("\nEVM PERF[%d]: evmIOReq/read failed\n", cb->id);
		return NFX_ERR;
	}

	return NFX_OK;

} /* evm_doRead */


/*-----------------------------------------------------------------
 * Name:		evm_perfDumpStats()
 *
 * Description: Dump statistics
 *
 * Created by : Shyh-Pei Yen
 *
 * Date Created: 6/26/02
 *-----------------------------------------------------------------
 */
static void
evm_perfDumpStats(int32 dumpT, int32 dumpQ)
{
	int32 i, j, idx, baseidx;
	int32 maxq[EVM_MAX_NUM_OF_LVS];
	int32 minq[EVM_MAX_NUM_OF_LVS];
	uint64 sum;
	float avg, sd, delta;
#if 0
	int32 totaltm[EVM_MAX_NUM_OF_LVS];
	char line[256];
	char buf[16];
#endif

	/* Dump SCSI queue depth detail information if requested */
	if (dumpQ && perf_doRspTime)
	{
		printf("==========================================\n");
		printf("Queue Depth detail dump:\n");
		printf("==========================================\n");

		for (j=0; j<totalvols; j++)
		{
			baseidx = j * perf_rspTimeSize;
			for (i=0; i<perf_nreqs; i++)
			{
				printf("vol[%02d] req[%05d]: %d\n", j, i, scsiQdepth[baseidx+i]);
			}
		}

		printf("==========================================\n\n");
	}

	/* Dump timestamp detail information if requested */
	if (dumpT && perf_doRspTime)
	{
		printf("==========================================\n");
		printf("Receive timestamp detail dump:\n");
		printf("==========================================\n");

		for (j=0; j<totalvols; j++)
		{
			baseidx = j * perf_rspTimeSize;
			for (i=0; i<perf_nreqs; i++)
			{
				idx = baseidx + i;
				printf("vol[%02d] req[%04d]: %d:%06d\n",
						j, i, rsptime1[idx], rsptime[idx]);
			}
		}
		printf("==========================================\n\n");
	}


	if (perf_doRspTime)
	{
		/* print out the max/min/avg/SD queue depth */
		printf("==========================================\n");
		printf("Queue Depth Summary:\n");
		printf("==========================================\n");

		for (j=0; j<totalvols; j++)
		{
			maxq[j] = 0;
			minq[j] = 0x7fffffff;
		}

		for (j=0; j<totalvols; j++)
		{
			baseidx = j * perf_rspTimeSize;

			sum = 0;
			for (i=0; i<perf_nreqs; i++)
			{
				idx = baseidx + i;

				/* min */
				if (minq[j] > scsiQdepth[idx])
					minq[j] = scsiQdepth[idx];

				/* max */
				if (maxq[j] < scsiQdepth[idx])
					maxq[j] = scsiQdepth[idx];

				sum += scsiQdepth[idx];
			}
			avg = (float) sum / perf_nreqs;

			sum = 0;
			for (i=0; i<perf_nreqs; i++)
			{
				idx = baseidx + i;

				if (avg >= scsiQdepth[idx])
					delta = avg - scsiQdepth[idx];
				else
					delta = scsiQdepth[idx] - avg;
				sum += (delta * delta);
			}
			sd = mysqrtf((float) sum / perf_nreqs);

			printf("  vol[%d] min=%04d max=%04d avg=%.2f sd=%.2f\n",
					j, minq[j], maxq[j], avg, sd);
			
		}
		printf("==========================================\n\n");


		/* print out the max/min/avg/SD receive time delta */
		printf("==========================================\n");
		printf("Rsp rcv timestamp delta Summary:\n");
		printf("==========================================\n");

		for (j=0; j<totalvols; j++)
		{
		}

		for (j=0; j<totalvols; j++)
		{
			uint32 diff, mindiff, maxdiff;

			baseidx = j * perf_rspTimeSize;

			mindiff = 0x7fffffff;
			maxdiff = 0;

			sum = 0;
			for (i=1; i<perf_nreqs; i++)
			{
				idx = baseidx + i;

				/* this is to find the earliest and latest timestamp */
				/* for calculating the requests per second later     */
				diff = (rsptime1[idx] - rsptime1[idx-1]) * USEC_PER_SEC +
									rsptime[idx] - rsptime[idx-1];

				sum += diff;

				if (mindiff > diff)
					mindiff = diff;

				if (maxdiff < diff)
					maxdiff = diff;
			}

			avg = (float) sum / (perf_nreqs - 1);

			sum = 0;
			for (i=1; i<perf_nreqs; i++)
			{
				idx = baseidx + i;

				diff = (rsptime1[idx] - rsptime1[idx-1]) * USEC_PER_SEC +
									rsptime[idx] - rsptime[idx-1];
				if (avg >= diff)
					delta = avg - diff;
				else
					delta = diff - avg;

				sum += (delta * delta);
			}
			sd = mysqrtf((float) sum / (perf_nreqs - 1));

			printf("  vol[%d] min=%08d max=%08d avg=%.2f sd=%.2f\n",
					j, mindiff, maxdiff, avg, sd);
		}

		for (j=0; j<totalvols; j++)
		{
			idx = j * perf_rspTimeSize;

			printf("earliest rsp tmstamp for vol[%02d] -   %d:%06d\n",
				j, rsptime1[0], rsptime[0]);
			printf("  latest rsp tmstamp for vol[%02d] -   %d:%06d\n",
				j, rsptime1[perf_nreqs-1], rsptime[perf_nreqs-1]);
			printf("               delta for vol[%02d] -   %d\n", j,
				(rsptime1[perf_nreqs-1] - rsptime1[0]) * USEC_PER_SEC
				+ rsptime[perf_nreqs-1] - rsptime[0]);
		}
	}
	
	/* Summary part is always dumped */
	printf("\n");
	printf("timestamp before queue up requests - %d:%06d\n",
			perf_before_reqdump.sec, perf_before_reqdump.usec);
	printf("timestamp after  queue up requests - %d:%06d\n",
			perf_after_reqdump.sec, perf_after_reqdump.usec);
	printf("                    queue up delta - %d\n",
			(perf_after_reqdump.sec - perf_before_reqdump.sec) * USEC_PER_SEC
			+ perf_after_reqdump.usec - perf_before_reqdump.usec);

	printf("timestamp after all resp received  - %d:%06d\n",
			perf_all_finished.sec, perf_all_finished.usec);
	printf("    from 1st req to last rsp delta - %d\n",
			(perf_all_finished.sec - perf_before_reqdump.sec) * USEC_PER_SEC
			+ perf_all_finished.usec - perf_before_reqdump.usec);
	{
		uint64 x, y;

		x = perf_nreqs * totalvols;
		x *= USEC_PER_SEC;
		y = ((perf_all_finished.sec - perf_before_reqdump.sec) * USEC_PER_SEC
						+ perf_all_finished.usec - perf_before_reqdump.usec);
		if (y)
			printf("Overall requests per second - %lld\n", x / y);
		else
			printf("Overall requests per second - invalid stats\n");
	}

#if 0
int32 maxtm[EVM_MAX_NUM_OF_LVS];
int32 mintm[EVM_MAX_NUM_OF_LVS];

	if (perf_Qlen == 1)
	{
		/* this is for queue depth at 1 - response time stats */
		for (j=0; j<totalvols; j++)
		{
			maxtm[j] = 0;
			mintm[j] = 0x7fffffff;
			totaltm[j] = 0;
		}
		for (i=0; i<perf_nreqs; i++)
		{
			line[0] = '\0';
			for (j=0; j<totalvols; j++)
			{
				sprintf(buf, "%6d", rsptime[j][i]);
				strcat(line, buf);
				strcat(line, "  ");
				if (rsptime[j][i] > maxtm[j])
					maxtm[j] = rsptime[j][i];
				if (rsptime[j][i] < mintm[j])
					mintm[j] = rsptime[j][i];
				totaltm[j] += rsptime[j][i];
			}
			printf("EVMReq[%04d]: %s\n", i, line);
		}
	}

	if (perf_Qlen == 1)
	{
		line[0] = '\0';
		for (j=0; j<totalvols; j++)
		{
			sprintf(buf, "%6d", maxtm[j]);
			strcat(line, buf);
			strcat(line, "  ");
		}
		printf("max rsp time: %s\n", line);

		line[0] = '\0';
		for (j=0; j<totalvols; j++)
		{
			sprintf(buf, "%6d", mintm[j]);
			strcat(line, buf);
			strcat(line, "  ");
		}
		printf("min rsp time: %s\n", line);

		line[0] = '\0';
		for (j=0; j<totalvols; j++)
		{
			sprintf(buf, "%6d", totaltm[j]/perf_nreqs);
			strcat(line, buf);
			strcat(line, "  ");
		}
		printf("avg rsp time: %s\n", line);
	}
#endif

	printf("================================================\n");

	return;

} /* evm_perfDumpStats() */


/*-----------------------------------------------------------------
 * Name:		evm_writeComplete
 *
 * Description: test completion routine for evm_IOReq
 *
 * Created by : Lan Chin
 *
 * Date Created: 9/28/01
 *-----------------------------------------------------------------
 */

static void
evm_writeComplete(evm_IOStat_t * stat)
{
int32 i, vidx, ridx, rsptm;
int32 key;
evm_perf_cb_t *cb;
eeeGMT_t now;

	/* GET timestamp value here */
	eee_TimeGMT(&now);

	key = stat->clientPrivate;
	cb = (evm_perf_cb_t *) hash_lookup(evm_srvr_test_hash,
									   (void *) ((uintptr_t)key),
									   (void *) ((uintptr_t)key));
	if (cb == NULL)
	{
		printf("\nEVM PERF: write cb ERROR, cannot find perf cb for key %d\n", key);
		return;
	}

	if (stat->status != EVM_STAT_NO_ERROR)
	{
		printf("EVM PERF[%d]: write request failed: %d\n", cb->id, stat->status);
		if (cb->type == u_write)
		{
			evm_sendFreeCodMsg(cb->req.wDataHandle.codHandle);
			cb->req.wDataHandle.codHandle = NULL;
		}
		evm_deletePerfCb(cb);
		return;
	}

	vidx = cb->vidx;
#if 0
	if (perf_Qlen == 1)
	{
		/* if queue depth is 1, then we'd like to know the */
		/* response time as this is same as synchronized   */

		/* calculate response time from timestamp */
		rsptm = 0;
		if (now.sec > cb->ts.sec)
			rsptm = (now.sec - cb->ts.sec) * USEC_PER_SEC;

		rsptime[vidx][cb->id] = rsptm + now.usec - cb->ts.usec;
	}
	else
#endif
	if (perf_doRspTime)
	{
		/* if queue depth is more than one, then we'd like to know */
		/* the # of reqs per seconds instead of the response time  */
		ridx = vidx * perf_rspTimeSize + totalrsps[vidx];
		rsptime[ridx] = now.usec;
		rsptime1[ridx] = now.sec;
		// scsiQdepth[ridx] = stat->reserved[0];
	}

	totalrsps[vidx]++;

#ifdef USE_NEW_COD_EVERYTIME
	/* free the current user write COD */
	if (cb->type == u_write)
	{
        evm_sendFreeCodMsg(cb->req.wDataHandle.codHandle);
        cb->req.wDataHandle.codHandle = NULL;
	}
#endif

	if (totalrsps[vidx] >= perf_nreqs)
	{
		printf("EVM performance test for vol[%d] done\n", vidx);
		eee_TimeGMT(&perf_all_finished);
		evm_deletePerfCb(cb);
		return;
	}

	/* issue next write request */
	if (nextid[vidx] < perf_nreqs)
	{
		if (evm_doNextWrite(cb) == NFX_ERR)
		{
			printf("\nERROR[%d]: cannot do subsequent metawr for offset %llx nBlks %lld\n",
				cb->id, cb->req.offset, cb->req.reqSize);
		}
	}
	else
	{
		/* since we already sent enough requests */
		/* we'll no long need to use this cb     */
		if ((cb->type == u_write) && cb->req.wDataHandle.codHandle)
		{
			evm_sendFreeCodMsg(cb->req.wDataHandle.codHandle);
			cb->req.wDataHandle.codHandle = NULL;
		}
		evm_deletePerfCb(cb);
	}

	return;

} /* evm_writeComplete */


/*-----------------------------------------------------------------
 * Name:		evm_doNextWrite
 *
 * Description: called to test EVM srvr meta or user write IO
 *
 * Created by : Shyh-Pei Yen
 *
 * Date Created: 6/25/02
 *-----------------------------------------------------------------
 */

static int32
evm_doNextWrite(evm_perf_cb_t *cb)
{
	int32		   rc;

	cb->id = nextid[cb->vidx]++;

	if (perf_opoffset == EVM_PERF_SEQ)
		cb->req.offset = perf_offset + (cb->id * cb->req.reqSize);
	else if (perf_opoffset == EVM_PERF_SKIPSEQ)
	{
		if (perf_nBlks == 16)
			cb->req.offset = perf_offset + (cb->id * 32);
		else
			cb->req.offset = perf_offset + (cb->id * 16);
		// cb->req.offset = perf_offset + (cb->id * (cb->req.reqSize+2));
	}
	/* no need to handle loopback's offset as it doesn't matter */

#ifdef USE_NEW_COD_EVERYTIME
	if (cb->type == u_write)
	{
		/* allocate another COD for next write */
		rc = evm_sendCreateCodMsg(&cb->req.wDataHandle.codHandle,
								  cb->key, cb->req.reqSize, perf_fill);
		if (rc == NFX_ERR)
		{
			/* callback evm_writeComplete deletes cb if err */
			printf("EVM perf userwr: can't create cod for next write\n");
			return NFX_ERR;
		}
	}
#endif

	eee_TimeGMT(&cb->ts);

	rc = evm_IOReq(&cb->req, evm_writeComplete);
	if (rc == NFX_ERR)
	{
		printf("EVM TEST IO[%d]: evmIOReq/WRITE FAILED\n", cb->id);
		return NFX_ERR;
	}

	return NFX_OK;

} /* evm_doNextWrite */


/*-----------------------------------------------------------------
 * Name:		evm_doWrite
 *
 * Description: called to unit test EVM srvr meta or user WRITE
 *
 * Created by : Lan Chin
 *
 * Date Created: 9/28/01
 *-----------------------------------------------------------------
 */

static int32
evm_doWrite(int32				vidx,
			int32				n,
			evm_pReq_t			type,
			evm_vol_t	   	    *lv,
			evm_volOffset_t		offset,
			evm_volSize_t		size,
			boolean				doUltra)
{
	int32		   rc;
	evm_perf_cb_t   *cb;
	char			*buf;

	cb = evm_createPerfCb();
	if (cb == NULL)
		return NFX_ERR;

	cb->type = type;
	cb->req.reqType = EVM_WRITE_REQ;
	cb->req.dataType = (type == m_write) ? EVM_META_DATA_REQ : EVM_USER_DATA_REQ;
	cb->req.id = lv->id;
	cb->req.offset = offset;
	cb->req.reqSize = size;
	if (perf_opoffset == EVM_PERF_LOOPBACK)
	{
		cb->req.attributes = SCSI_MSG_ATTR_LOOPBACK;
	}
	cb->id = n;
	cb->vidx = vidx;
	cb->len = size * EVM_LBLK_SIZE;
	perf_fill = (cb->key % EVM_SRVR_TEST_HASH_BUCKETS);

	/* meta write need a local buffer, not the user write */
	/* as the buffer for user write is in TXRX???         */
	if (type == m_write)
	{
		if (doUltra)
		{
			NFX_ASSERT(EEE_BUFSIZE_ULTRA > 8192);
			if (cb->len > 8192)
			{
				printf("evmadmin: truncating to max ultra buf is 8192\n");
				cb->len = 8192;
			}
			buf = eee_allocateBuffer(EEE_SHARED_POOL, EEE_BUF_ULTRA, NFX_COS_HIGH);
			// Preserve the unused leader for packet headers.
			buf += EEE_BUFSIZE_ULTRA - 8192;
		}
		else
		{
			EVM_RAM_ALLOC_ALIGNED(buf, char, cb->len);
		}
		if (buf == NULL)
		{
			printf("\nEVM perf metawr[%d]: no mem for write buf\n", n);
			evm_deletePerfCb(cb);
			return NFX_ERR;
		}
		memset((void *) buf, perf_fill, cb->len);
		{
			int i;
			//printf("len = %d\n", cb->len);
			for (i = 0; i < cb->len; i++)
			{
				buf[i] = i & 0xff;
			} // for
		}
		cb->chksum = evm_doChksum((ushort16 *) buf, cb->len);
		cb->buf = buf;

		cb->req.wDataHandle.dataPtr = buf;
	}
	else
	{
		/*
		 * user write, allocate COD and perf_fill with pattern
		 */
		cb->buf = NULL;

		rc = evm_sendCreateCodMsg(&cb->req.wDataHandle.codHandle,
								  cb->key, size, perf_fill);
		if (rc == NFX_ERR)
		{
			/* callback evm_writeComplete deletes cb if err */
			printf("\nEVM perf userwr[%d]: can't create cod\n", n);
			return NFX_ERR;
		}
	}

	/* put timestamp into cb */
	eee_TimeGMT(&cb->ts);

	rc = evm_IOReq(&cb->req, evm_writeComplete);
	if (rc == NFX_ERR)
	{
		/* callback evm_writeComplete deletes cb if err */
		printf("\nEVM PERF IO[%d]: evmIOReq/WRITE FAILED\n", n);
		return NFX_ERR;
	}

	return NFX_OK;

} /* evm_doWrite */


/*-----------------------------------------------------------------
 * Name:		evm_perf_usage
 *-----------------------------------------------------------------
 */
static void
evm_perf_usage()
{
	printf("USAGE: evmadmin perf_evm {allocmem|deallocmem} [totalreqs]\n");
	printf("       evmadmin perf_evm {metawr|userwr|userrd} {fixed|seq|skip|loopback} <offset> <nsectors> <totalreqs> <num-of-vol> <Qnreqs>\n");
	printf("       evmadmin perf_evm dump [time] [qdepth]\n");
	return;
}


/*-----------------------------------------------------------------
 * Name:		evm_perf
 *
 * Description: called to performance test EVM srvr IO
 * Created by : Lan Chin
 *				Shyh-Pei Yen
 * Date Created: 6/26/02
 *-----------------------------------------------------------------
 */

int32
evm_perf(int32 argc, char8 *argv[])
{
	int32			rc, argi;
	int32			i, j;
	int32			lvToIdx;
	evm_vol_t       *lv;
	boolean			doUltra = FALSE;

	if (evm_srvr_test_init == 0)
	{
		evm_srvr_test_init = 1;
		evm_srvr_test_hash = hash_create(EVM_SRVR_TEST_HASH_BUCKETS,
										 evm_srvr_test_hashFunc,
										 evm_srvr_test_hashCompare,
										 evm_srvr_test_hashRelease);
	}

	/* check number of arguments */
	if (argc < 2)
	{
		evm_perf_usage();
		return NFX_ERR;
	}

	/* NOW PARSE THE PARAMETERS */
	argi = 1;

	/* ==================================== */
	/* what type of operation? meta write   */
	/* or meta read or user write or user   */
	/* read?                                */
	if (strcmp(argv[argi], "dump") == 0)
	{
		int32 dumpTime = 0;
		int32 dumpQdepth = 0;

		for (i = 2; i < argc; i++)
		{
			if (strcmp(argv[i], "time") == 0)
				dumpTime = 1;
			else if (strcmp(argv[i], "qdepth") == 0)
				dumpQdepth = 1;
		}
		evm_perfDumpStats(dumpTime, dumpQdepth);

		return NFX_OK;
	}
	else if (strcmp(argv[argi], "allocmem") == 0)
	{
		if (perf_doRspTime)
		{
			printf("EVM PERF: memory for array already allocated\n");
			return NFX_OK;
		}

		if (argc < 3)
			perf_rspTimeSize = 1000;	/* default size to 1000 */
		else
			perf_rspTimeSize = atoi(argv[2]);

    	EVM_RAM_ALLOC(rsptime, uint32,
				(EVM_MAX_NUM_OF_LVS * perf_rspTimeSize * sizeof(uint32)));
    	if (rsptime == NULL)
		{
			printf("EVM PERF: unable to allocate memory for rsptime array\n");
			return NFX_ERR;
		}

    	EVM_RAM_ALLOC(rsptime1, uint32,
				(EVM_MAX_NUM_OF_LVS * perf_rspTimeSize * sizeof(uint32)));
    	if (rsptime1 == NULL)
		{
			printf("EVM PERF: unable to allocate memory for rsptime1 array\n");
			EVM_RAM_DEALLOC(rsptime);
			return NFX_ERR;
		}

    	EVM_RAM_ALLOC(scsiQdepth, uint32,
				(EVM_MAX_NUM_OF_LVS * perf_rspTimeSize * sizeof(uint32)));
    	if (scsiQdepth == NULL)
		{
			printf("EVM PERF: unable to allocate memory for scsiQdepth array\n");
			EVM_RAM_DEALLOC(rsptime);
			EVM_RAM_DEALLOC(rsptime1);
			return NFX_ERR;
		}
		perf_doRspTime = 1;
		printf("EVM PERF: successfully allocated memory (size %d)\n", perf_rspTimeSize);
		return NFX_OK;
	}
	else if (strcmp(argv[argi], "deallocmem") == 0)
	{
		if (perf_doRspTime)
		{
			EVM_RAM_DEALLOC(rsptime);
			EVM_RAM_DEALLOC(rsptime1);
			EVM_RAM_DEALLOC(scsiQdepth);
			perf_doRspTime = 0;
			printf("EVM PERF: memory de-allocated...\n");
		}
		else
			printf("EVM PERF: no allocated memory to be de-allocated\n");
		return NFX_OK;
	}
	else if (strcmp(argv[argi], "metawr") == 0)
	{
		/* check number of arguments */
		if (argc < 8)
		{
			evm_perf_usage();
			return NFX_ERR;
		}

		perf_type = m_write;
		doUltra = FALSE;
	}
	else if (strcmp(argv[argi], "userwr") == 0)
	{
		/* check number of arguments */
		if (argc < 8)
		{
			evm_perf_usage();
			return NFX_ERR;
		}

		perf_type = u_write;
		doUltra = FALSE;
	}
	else if (strcmp(argv[argi], "userrd") == 0)
	{
		/* check number of arguments */
		if (argc < 8)
		{
			evm_perf_usage();
			return NFX_ERR;
		}

		perf_type = u_read;
		doUltra = FALSE;
	}
	else
	{
		evm_perf_usage();
		return NFX_ERR;
	}
	argi++;

	/* initialized variables and counters */
	for (i=0; i<EVM_MAX_NUM_OF_LVS; i++)
	{
		perf_lvp[i] = 0;
		if (perf_doRspTime)
		{
			for (j=0; j<perf_rspTimeSize; j++)
			{
				uint32 idx = i * perf_rspTimeSize + j;
				rsptime[idx] = 0;
				rsptime1[idx] = 0;
				scsiQdepth[idx] = 0;
			}
		}
		nextid[i] = 0;
		totalrsps[i] = 0;
	}
	totalvols = 0;

	/* ==================================== */
	/* is it a fixed or sequential or       */
	/* random offset for each read/write?   */
	if (strcmp(argv[argi], "fixed") == 0)
		perf_opoffset = EVM_PERF_FIXED;
	else if (strcmp(argv[argi], "seq") == 0)
		perf_opoffset = EVM_PERF_SEQ;
#if 0
/* not yet supported */
	else if (strcmp(argv[argi], "rand") == 0)
		perf_opoffset = EVM_PERF_RAND;
#endif
	else if (strcmp(argv[argi], "skip") == 0)
		perf_opoffset = EVM_PERF_SKIPSEQ;
	else if (strcmp(argv[argi], "loopback") == 0)
		perf_opoffset = EVM_PERF_LOOPBACK;
	else
	{
		evm_perf_usage();
		return NFX_ERR;
	}
	argi++;

	/* ==================================== */
	/* get volume offset and number of      */
	/* sectors to be read/write             */
	perf_offset = (atol(argv[argi++]) / 16) * 16;	/* round to multiply of 16 */
	perf_nBlks = atol(argv[argi++]);
	if (perf_nBlks % 2)
	{
		printf("\nERROR: scsi checksum reqires even number block io\n");
		return NFX_ERR;
	}
	else if (perf_nBlks > 16)
	{
		printf("\nERROR: number of blocks greater than 16\n");
		return NFX_ERR;
	}

	/* ==================================== */
	/* total requests to send out           */
	perf_nreqs = atoi(argv[argi++]);
	if (perf_doRspTime && (perf_nreqs > perf_rspTimeSize))
	{
		printf("EVM num of reqs specified(%d) exceeds allocated limit(%d), truncated\n",
			perf_nreqs, perf_rspTimeSize);
		perf_nreqs = perf_rspTimeSize;
	}

	/* ==================================== */
	/* get the volumes (from and to indexes */
	lvToIdx = atoi(argv[argi++]);
	if (lvToIdx <= 0)
	{
		evm_perf_usage();
		return NFX_ERR;
	}

	NFX_SPIN_LOCK(&evm_lv_lock);
    DLL_ITERATE(lv, &evm_volList, entry)
	{
		if (lv == NULL)
			break;
		if (lvToIdx > 0)
			lvToIdx--;
		else
			break;
		perf_lvp[totalvols++] = lv;
	} // DLL_ITERATE
	NFX_SPIN_UNLOCK(&evm_lv_lock);
	if (lvToIdx > 0)
	{
		printf("No. of volumes specified exceeds max %d volumes this test supported.\n",
			EVM_MAX_NUM_OF_LVS);
	}

	/* no volume? */
	if (perf_lvp[0] == NULL)
	{
		printf("No logical volume found...\n");
		return NFX_ERR;
	}

	/* ==================================== */
	/* how many requests to sent to queue   */
	/* (maintain that queue length until    */
	/* all requests are sent)               */
	perf_Qlen = atol(argv[argi++]);

	if (perf_nreqs < perf_Qlen)
		perf_Qlen = perf_nreqs;

	if ((perf_type == m_write) || (perf_type == u_write))
	{
		eee_TimeGMT(&perf_before_reqdump);

		for (i=0; i<totalvols; i++)
		{
			evm_volOffset_t	offset1 = perf_offset;

			printf("evmadmin write performance test: volID=%llx nsectors=%d totalreqs=%d offset=%lld Qnreqs=%d\n",
					perf_lvp[i]->id.whole_thing, perf_nBlks, perf_nreqs, offset1, perf_Qlen);
			nextid[i] = perf_Qlen;
			for (j=0; j<perf_Qlen; j++)
			{
				rc = evm_doWrite(i, j, perf_type, perf_lvp[i], offset1, perf_nBlks, doUltra);
				if (rc == NFX_ERR)
				{
					printf("\nERROR[%d]: cannot do write for vol %d offset %llx nBlks %d\n",
							rc, i, offset1, perf_nBlks);
				}
				if (perf_opoffset == EVM_PERF_SEQ)
					offset1 += perf_nBlks;
				if (perf_opoffset == EVM_PERF_SKIPSEQ)
				{
					if (perf_nBlks == 16)
						offset1 += 32; /* to avoid spanning error on scsi */
					else
						offset1 += 16; /* to avoid spanning error on scsi */
					// offset1 += perf_nBlks + 2;
				}
			}
		}

		eee_TimeGMT(&perf_after_reqdump);
	}
	else if ((perf_type == m_read) || (perf_type == u_read))
	{
		eee_TimeGMT(&perf_before_reqdump);

		for (i=0; i<totalvols; i++)
		{
			evm_volOffset_t	offset1 = perf_offset;

			printf("evmadmin read performance test: volID=%llx nsectors=%d totalreqs=%d offset=%lld Qnreqs=%d\n",
					perf_lvp[i]->id.whole_thing, perf_nBlks, perf_nreqs, offset1, perf_Qlen);
			nextid[i] = perf_Qlen;
			for (j=0; j<perf_Qlen; j++)
			{
				rc = evm_doRead(i, j, perf_type, perf_lvp[i], offset1, perf_nBlks, doUltra);
				if (rc == NFX_ERR)
				{
					printf("\nERROR[%d]: cannot do read for vol %d offset %llx nBlks %ld\n",
							rc, i, offset1, perf_nBlks);
				}
				if (perf_opoffset == EVM_PERF_SEQ)
					offset1 += perf_nBlks;
				if (perf_opoffset == EVM_PERF_SKIPSEQ)
				{
					if (perf_nBlks == 16)
						offset1 += 32; /* to avoid spanning error on scsi */
					else
						offset1 += 16; /* to avoid spanning error on scsi */
					// offset1 += perf_nBlks + 2;
				}
			}
		}

		eee_TimeGMT(&perf_after_reqdump);
	}

	return NFX_OK;

} /* evm_perf */

/*----------------------------------------------------------------------------*/

#endif // SY_IO_PERFORMANCE_TEST
