/*------------------------------------------------------------------------------
 * RCS:  $Id: evm-api.h,v 1.76 2002/09/28 22:28:40 isabellec Exp $
 * Copyright (C) 2001, Agile Storage, Inc.
 *
 * Name         : evm-api.h
 *
 * Description  : Volume Manager (EVM) API
 *
 * Created by   : Greg Seibert
 *                Lan Chin
 *                Isabelle Cnudde
 *
 * Date Created : 26-APR-2001
 *
 *------------------------------------------------------------------------------
 */

#ifndef _EVM_API_H_
#define _EVM_API_H_

#include "exports/evm_com_defs.h"
#include "exports/evm-srvr-api.h"

#define NEWVOLID

/*------------------------------------------------------------------------------
 * EVM message header.
 *------------------------------------------------------------------------------
 */

typedef	enum
{
    // The External Messages
    EVM_VOL_CREATE = EVM_MSG_BASE,	// evm_MsgVolCreate_t
    EVM_VOL_GET_ATTRIB,				// evm_MsgVolGetAttrib_t
    EVM_VOL_CONFIG,					// evm_MsgVolCreate_t
    EVM_VOL_GET_ID,					// evm_MsgGetId_t
    EVM_VOL_GET_NEWVOLID,           // evm_MsgGetNewVolId_t
    EVM_VOL_CHECK_VOLNAME,          // evm_MsgCheckVolName_t
    EVM_VOL_ONLINE,					// evm_MsgVolOnline_t
    EVM_VOL_OFFLINE,				// evm_MsgVolOffline_t
    EVM_VOL_DESTROY,				// evm_MsgVolDestroy_t
    EVM_VOL_RENAME,					// evm_MsgVolRename_t
    EVM_STAT_REQ,                   // evm_MsgStatReq_t
    EVM_STAT_RSP,					// evm_MsgStatRsp_t
    EVM_VOL_LIST,					// evm_MsgVolList_t
    EVM_SET_LV_OPTS,				// evm_MsgSetOpt_t
    EVM_VOL_GROW,					// evm_MsgVolStrg_t
    EVM_GET_LV_STRG,				// evm_MsgGetStrg_t
    EVM_LV_STRG_CFG,				// evm_MsgLVStrg_t
    EVM_VOL_TAKEOVER,               // evm_MsgVolTakeover_t
    EVM_VOL_GET_NAME_LIST,          // evm_MsgVolGetNameList_t      
    EVM_VOL_NAME_LIST,              // evm_MsgVolNameList_t
    EVM_STRG_GET_ATTRIB,			// evm_MsgStrg_Get_Attrib_t
    EVM_STRG_VOL_CFG,				// evm_MsgStrgVolCfg_t
    EVM_STRG_EVT,					// evm_MsgStrgEvt_t
    EVM_STRG_LIST,					// evm_MsgStrgList_t
    EVM_STRG_GET_ID,				// evm_MsgGetStrgId_t
    
    // Intra-EVM Messages
    EVM_LVACT_CREATELV,				// evm_MsgCreateLV_t
    EVM_LVACT_ADDSTRG,				// evm_MsgAddStrg_t
    EVM_LVACT_REMSTRG,				// evm_MsgRemStrg_t
    EVM_LVACT_ONLINELV,				// evm_MsgOnLinLV_t
    EVM_LVACT_OFFLINELV,			// evm_MsgOffLinLV_t
    EVM_LVACT_DESTRLV,				// evm_MsgDestLV_t
    EVM_CNTRL_LVFULL,				// evm_MsgLVFull_t
    EVM_CNTRL_SYNC,                 // evm_MsgSync_t
    EVM_LVACT_VOLLIST_REQ,			// evm_MsgReqVolList_t
    EVM_LVACT_VOLLIST_RSP,			// evm_MsgVolList_t
    
    // Status Message
    EVM_MSG_STAT,					// evm_MsgStat_t
    
    // Configuration notification
    EVM_CONFIG_NTFY,                // evm_configNtfy_t

    // Event sent by file system when full
    EVM_FSYS_FULL,

    // Interface with SPM
    EVM_GET_LUN_REQ,
    EVM_GET_LUN_RSP,
    EVM_LABEL_LUN_REQ,
    EVM_LABEL_LUN_RSP,
    EVM_UNLABEL_LUN_REQ,
    EVM_UNLABEL_LUN_RSP,
    EVM_RESCAN_LUN_REQ,
    EVM_MOD_LUN_CLUSTER_REQ,
    EVM_MOD_LUN_CLUSTER_RSP,
    
    EVM_READ_QUORUM_REQ,            // evm_MsgReadQuorumReq_t
    EVM_READ_QUORUM_RSP,            // evm_MsgReadQuorumRsp_t
    EVM_WRITE_QUORUM_REQ,           // evm_MsgWriteQuorumReq_t
    EVM_WRITE_QUORUM_RSP,           // evm_MsgWriteQuorumRsp_t
    
    EVM_READ_MIRROR_REQ,            // evm_MsgReadMirrorReq_t
    EVM_READ_MIRROR_RSP,            // evm_MsgReadMirrorRsp_t
    EVM_WRITE_MIRROR_REQ,           // evm_MsgWriteMirrorReq_t
    EVM_WRITE_MIRROR_RSP,           // evm_MsgWriteMirrorRsp_t
    
    EVM_DUMMY
} evm_msgType_t;					// All the message types

typedef	struct
{
    evm_msgType_t	msgType;		// What this message is
    uint32          len;            // length of the total message
    uint64			clientPrivate;	// Opaque to EVM.
} evm_msgHdr_t;

/*------------------------------------------------------------------------------
 * EVM message status.
 *------------------------------------------------------------------------------
 */

// Note: In the case that there is a error description string, it will be
// magically appended to this message. Don't care? Just ignore it.
typedef	struct
{
    evm_msgHdr_t		hdr;		// EVM_MSG_STAT
#define EVM_OK                  NFX_OK
#define EVM_ERR                 NFX_ERR
#define EVM_VOL_ALREADY_EXIST   -2
#define EVM_VOL_ALREADY_ONLINE  -3
#define EVM_VOL_ALREADY_OFFLINE -4
#define EVM_VOL_GROW_PENDING	-5
    uint32				stat;		// NFX_* codes
    union {
        evm_volId_t		lVolId;		// This LV
        int32			sPoolId;	// or Storage Pool
        scsi_id_t		sId;		// or this physical volume
    } payload;
} evm_MsgStat_t;

/*------------------------------------------------------------------------------
 * Logical volume messages.
 *------------------------------------------------------------------------------
 */

// Tell me about this logical volume
typedef	struct
{
    evm_msgHdr_t		hdr;    // EVM_VOL_GET_ATTRIB
    evm_volId_t			id;		// What I want -1 gets you EVM_VOL_LIST
} evm_MsgVolGetAttrib_t;

// Here's the list of logical volumes I know about
typedef struct
{
    evm_msgHdr_t		hdr;		// EVM_VOL_LIST
    uint32				num;		// How many
    evm_volId_t			ids[MAX_IDS];
} evm_MsgVolList_t;

// Tell me about all existing volumes
typedef	struct
{
    evm_msgHdr_t		hdr;		// EVM_VOL_GET_NAME_LIST
} evm_MsgVolGetNameList_t;

// Here's the list of volumes with their id and name
typedef struct
{
    evm_msgHdr_t		hdr;		// EVM_VOL_NAME_LIST
    uint32				num;		// How many
    struct {
        evm_volId_t     id;
        char            name[VOL_MAX_NAME];
    } s[1];
} evm_MsgVolNameList_t;

// Get LV id for this name
typedef struct
{
    evm_msgHdr_t		hdr;		// EVM_VOL_GET_ID
    char 				name[VOL_MAX_NAME];
} evm_MsgGetId_t;

// Create a new logical volume
typedef	struct
{
    evm_msgHdr_t		hdr;		// EVM_VOL_CREATE/CONFIG
    evm_VolConfig_t		config;		// Config of new logical volume
#define EVM_VOL_READY       0               // Ready for online
#define EVM_VOL_NOT_READY   1               // Missing LUNs
    uint32              state;      // State of logical volume
    uint32              nbLun;
    char                lunList[1][SDM_DEV_NAME_LEN+8];
} evm_MsgVolCreate_t;

// Put this logical volume online
typedef	struct
{
    evm_msgHdr_t    hdr;		// EVM_VOL_ONLINE
    evm_volId_t		id;			// volume to online
    uint32			nim;		// on which FP-NIM
    uint32          checkFlag;  // TRUE: check if nbdy is managing the volume
                                // before turning it online, FALSE: no check
} evm_MsgVolOnline_t;

// Take this logical volume offline
typedef	struct
{
    evm_msgHdr_t		hdr;		// EVM_VOL_OFFLINE
    evm_volId_t			id;			// This LV
    uint32				nim;		// from which NIM
} evm_MsgVolOffline_t;

// Destroy this logical volume
typedef	struct
{
    evm_msgHdr_t		hdr;		// EVM_VOL_DESTROY
    evm_volId_t			id;			// This LV
} evm_MsgVolDestroy_t;

// Rename this logical volume
typedef	struct
{
    evm_msgHdr_t		hdr;		// EVM_VOL_RENAME
    evm_volId_t			id;			// This LV
    char				newName[VOL_MAX_NAME];
} evm_MsgVolRename_t;

// Add this specific storage to this logical volume
typedef	struct
{
    evm_msgHdr_t		hdr;		// EVM_VOL_GROW
    evm_volId_t			id;			// To this logical volume
    uint32              nbLun;
    char                lunList[1][SDM_DEV_NAME_LEN+8];
} evm_MsgVolStrg_t;

/* EVM server statistics.
 */
typedef	struct
{
    evm_msgHdr_t		hdr;		// EVM_STAT_REQ
    boolean             reset;      // reset statistics after reading them
} evm_MsgStatReq_t;

typedef	struct
{
    evm_msgHdr_t        hdr;        // EVM_STAT_RSP
    evm_stats_t         stats;      // EVM server statistics
} evm_MsgStatRsp_t;

/* Request the list of volumes currently served by a FP-NIM.
 */
typedef struct
{
    evm_msgHdr_t        hdr;        // EVM_LVACT_VOLLIST_REQ
} evm_MsgReqVolList_t;

typedef struct
{
    evm_msgHdr_t		hdr;		// EVM_LVACT_VOLLIST_RSP
    uint32				nbVol;		// number of volumes
    evm_volId_t			ids[1];     // list of volumes
} evm_MsgRspVolList_t;

// Change this LV's config to these values
typedef struct
{
    evm_msgHdr_t	hdr;            // EVM_SET_LV_OPTS
    evm_volId_t		id;             // For this LV
    uint64			maxSize;        // How big can it get
    uint64			minAutoGrow;	// Growth increment (blocks)
    uint32			softQuota;
    evm_volType_t   type;
} evm_MsgSetOpt_t;

// Tell me what storage has been assigned to this logical volume
typedef struct
{
    evm_msgHdr_t	hdr;        // EVM_GET_LV_STRG
    evm_volId_t		id;			// volume id
} evm_MsgGetStrg_t;

typedef struct
{
    evm_msgHdr_t	hdr;		// EVM_LV_STRG_CFG
    evm_volId_t		id;			// volume id
    uint32          nbStrg;     // number of LUNs composing the volume
    evm_lvStrg_t	strg[0];    // list of LUNs composing the volume
} evm_MsgLVStrg_t;

/* Takeover a volume.
 */
typedef struct
{
    evm_msgHdr_t	hdr;    // EVM_VOL_TAKEOVER
    evm_volId_t		id;     // volume to take over
    uint32          nim;    // NIM to online it
    uint32          delay;  // number of seconds after which EVM gives up
                            // trying to takeover the volume (0: forever)
} evm_MsgVolTakeover_t;

/*------------------------------------------------------------------------------
 * Physical storage messages.
 *----------------------------------------------------------------------------*/

// Tell me about this particular physical storage volume
typedef	struct
{
    evm_msgHdr_t		hdr;    // EVM_STRG_GET_ATTRIB
    scsi_id_t			id;		// for this physical vol -1 gets you EVM_STRG_LIST
} evm_MsgStrg_Get_Attrib_t;

// Response to EVM_STRG_GET_ATTRIB
typedef	struct
{
    evm_msgHdr_t		hdr;		// EVM_STRG_VOL_CFG
    scsi_id_t			devId;		// Here is where it is
    evm_StrgAttr_t		volAttr;	// and what it is
} evm_MsgStrgVolCfg_t;
#define MSVC_SIZE(x) (sizeof(evm_MsgStrgVolCfg_t) + (((x)->volAttr.nallocs - 1) * sizeof(alloc_t)))

// Physical Storage just did this
typedef struct
{
    evm_msgHdr_t        hdr;		// EVM_STRG_EVT
    scsi_id_t			devId;		// This LUN
    int					online;
} evm_MsgStrgEvt_t;

// Give me the scsi_id for this storage name
typedef struct
{
    evm_msgHdr_t		hdr;		// EVM_STRG_GET_ID
    char				name[VOL_MAX_NAME]; // for this
} evm_MsgGetStrgId_t;

/* Notification sent by the EVM server when it had to put off-line a volume.
 */
typedef struct
{
    evm_msgHdr_t    hdr;            // EVM_OFFLINE_EVT
    evm_volId_t     volId;          // volume id
#define EVM_REASON_IO_FAILED    10  // IO request failed after retries
#define EVM_REASON_MGR_CONFLICT 11  // Manager area is written by sbdy else
#define EVM_REASON_TAKEOVER     12  // Another node is going to takeover
    uint32          reason;         // why the volume has been put offline
    uint32          pad;
} evm_offlineInd_t;

/* Get a new volume id.
 */
typedef struct
{
    evm_msgHdr_t    hdr;            // EVM_VOL_GET_NEWVOLID
} evm_MsgGetNewVolId_t;

/* Check the uniqueness of a volume name.
 */
typedef struct
{
    evm_msgHdr_t    hdr;            // EVM_VOL_CHECK_VOLNAME
    char			name[VOL_MAX_NAME];
} evm_MsgCheckVolName_t;

// List of physical storage ids we know about
typedef struct
{
    evm_msgHdr_t		hdr;		// EVM_STRG_LIST
    int					num;		// How many of
    scsi_id_t			ids[MAX_PSTRG]; // these
} evm_MsgStrgList_t;

/* Configuration notification.
 */
typedef struct
{ 
    evm_msgHdr_t    hdr;        // EVM_CONFIG_NTFY
    struct
    { 
        char        option[OPT_MAX_OPTION_LEN];
        char        value[MAX_VALUE_LEN];
    } entry;
} evm_configNtfy_t;

/*------------------------------------------------------------------------------
 * Interface with SPM
 *------------------------------------------------------------------------------
 */

/* Get info on a specific LUN.
 */
typedef struct
{
    evm_msgHdr_t    hdr;
	scsi_id_t       id;                             // SCSI id
} evm_MsgGetLunReq_t;                               // EVM_GET_LUN_REQ

typedef struct
{
    evm_msgHdr_t    hdr;
    int32           result;                         // NFX_OK / NFX_ERR
	scsi_id_t       id;                             // SCSI id
	uint64          wwn;                            // WWN
	uint16          lun;                            // LUN
    evm_raid_t      raidLevel;                      // RAID level
	char            name[SDM_DEV_NAME_LEN+1];       // ASCII name set by user
	char            vendor[SDM_VENDOR_NAME_LEN+1];  // Device vendor string
	char            model[SDM_MODEL_NAME_LEN+1];    // Device model string
	uint64          size;                           // LUN size (bytes)
	evm_lunState_t  state;                          // LUN state
    evm_volId_t     volId;                          // volume id
    char            cluster[EVM_MAX_NODE_NAME];     // cluster name
} evm_MsgGetLunRsp_t;                               // EVM_GET_LUN_RSP

/* Agilize a foreign LUN.
 */
typedef struct
{
    evm_msgHdr_t    hdr;
    char            cluster[EVM_MAX_NODE_NAME];     // cluster name
    int32           idCnt;
	scsi_id_t       id[1];                          // SCSI id
} evm_MsgLabelLunReq_t;                             // EVM_LABEL_LUN_REQ

typedef struct
{
    evm_msgHdr_t    hdr;
    int32           result;                         // NFX_OK / NFX_ERR
} evm_MsgLabelLunRsp_t;                             // EVM_LABEL_LUN_RSP

/* Remove agile label from a LUN.
 */
typedef struct
{
    evm_msgHdr_t    hdr;
    int32           idCnt;
	scsi_id_t       id[1];                          // SCSI id
} evm_MsgUnlabelLunReq_t;                           // EVM_UNLABEL_LUN_REQ

typedef struct
{
    evm_msgHdr_t    hdr;
    int32           result;                         // NFX_OK / NFX_ERR
} evm_MsgUnlabelLunRsp_t;                           // EVM_UNLABEL_LUN_RSP

/* SPM notifies EVM that a LUN's state has been modified.
 * (cluster feature)
 */
typedef struct
{
    evm_msgHdr_t    hdr;
    scsi_id_t       id;                             // SCSI id
} evm_MsgRescanLunReq_t;                            // EVM_RESCAN_LUN_REQ

/* Modify the cluster to which a LUN belongs.
 */
typedef struct
{
    evm_msgHdr_t    hdr;
    char            cluster[EVM_MAX_NODE_NAME];     // cluster name
    int32           idCnt;
	scsi_id_t       id[1];                          // SCSI id
} evm_MsgModLunClusterReq_t;                        // EVM_MOD_LUN_CLUSTER_REQ

typedef struct
{
    evm_msgHdr_t    hdr;
    int32           result;                         // NFX_OK / NFX_ERR
} evm_MsgModLunClusterRsp_t;                        // EVM_MOD_LUN_CLUSTER_RSP

/*------------------------------------------------------------------------------
 * Interface to access the quorum disk.
 *------------------------------------------------------------------------------
 */

typedef struct
{
    evm_msgHdr_t    hdr;
    scsi_id_t       lunId;              // SCSI id of the quorum disk
} evm_MsgReadQuorumReq_t;               // EVM_READ_QUORUM_REQ

typedef struct
{
    evm_msgHdr_t    hdr;
    int32           result;             // NFX_OK / NFX_ERR
    evm_qArea_t     quorum;             // quorum data read
} evm_MsgReadQuorumRsp_t;               // EVM_READ_QUORUM_RSP

typedef struct
{
    evm_msgHdr_t    hdr;
    scsi_id_t       lunId;              // SCSI id of the quorum disk
    evm_qArea_t     quorum;             // quorum data to write
} evm_MsgWriteQuorumReq_t;              // EVM_WRITE_QUORUM_REQ

typedef struct
{
    evm_msgHdr_t    hdr;
    int32           result;             // NFX_OK / NFX_ERR
} evm_MsgWriteQuorumRsp_t;              // EVM_WRITE_QUORUM_RSP

/*------------------------------------------------------------------------------
 * Interface to access the mirror disk.
 *------------------------------------------------------------------------------
 */

typedef struct
{
    evm_msgHdr_t        hdr;
    evm_volId_t         volId;          // volume id.
} evm_MsgReadMirrorReq_t;               // EVM_READ_MIRROR_REQ

typedef struct
{
    evm_msgHdr_t        hdr;
    int32               result;         // NFX_OK / NFX_ERR
    evm_mirrorArea_t    mirror;         // mirror data read
} evm_MsgReadMirrorRsp_t;               // EVM_READ_MIRROR_RSP

typedef struct
{
    evm_msgHdr_t        hdr;
    evm_volId_t         volId;          // volume id.
    evm_mirrorArea_t    mirror;         // mirror data to write
} evm_MsgWriteMirrorReq_t;              // EVM_WRITE_MIRROR_REQ

typedef struct
{
    evm_msgHdr_t        hdr;
    int32               result;         // NFX_OK / NFX_ERR
} evm_MsgWriteMirrorRsp_t;              // EVM_WRITE_MIRROR_RSP

/*----------------------------------------------------------------------------*/

#endif // _EVM_API_H_
