/*------------------------------------------------------------------------------
 * RCS:  $Id: evm-srvr-api.h,v 1.2 2002/12/10 17:27:34 bob Exp $
 * Copyright (C) 2001, Agile Storage, Inc.
 *
 * Name         : evm-srvr-api.h
 *
 * Description  : Volume Manager (EVM) server API
 *
 * Created by   : Isabelle Cnudde
 *
 * Date Created : 27-SEP-2002
 *
 *------------------------------------------------------------------------------
 */

#ifndef _EVM_SRVR_API_H_
#define _EVM_SRVR_API_H_

#include "exports/evm_com_defs.h"

/*------------------------------------------------------------------------------
 * Types definition.
 *------------------------------------------------------------------------------
 */

typedef	enum
{
    EVM_READ_REQ,           // read 
    EVM_WRITE_REQ           // write
} evm_IO_t;                 // request type

typedef	enum
{
    EVM_USER_DATA_REQ = 0,  // user
    EVM_META_DATA_REQ = 1,  // meta
    EVM_DATA_TYPE_MAX = 2
} evm_req_t;                // data type

typedef	struct
{
    evm_IO_t		reqType;		// Type of request
    evm_req_t		dataType;		// Type of data
    evm_volId_t		id;				// to this logical volume
    evm_volOffset_t	offset;			// at this block position
    evm_volSize_t	reqSize;		// how many blocks
	uint64          attributes;     // see SCSI_MSG_ATTR_* in scsi-msg.h
    evm_handle_t	wDataHandle;	// if writing, here is the data
    uint64			clientPrivate;	// returned with call back
    int32			codOffset;		// offset into cod for writes
}  evm_IOReq_t;

typedef	struct
{
    evm_handle_t    rDataHandle;	// The the read data is here
    int32           status;         // status of this request
#define	EVM_STAT_NO_ERROR	 0      // Success
#define	EVM_STAT_INVALID	-1      // Invalid parameter(s) in request
#define EVM_STAT_REJECT	    -2      // Request rejected
#define	EVM_STAT_TIMEOUT	-3      // No response from SCSI layers
#define	EVM_STAT_DEV_ERROR	-4      // SCSI error
#define EVM_STAT_SYS_ERR	-5      // System error (memory...)
#define EVM_STAT_MSG_ERR	-6      // Error detected on a received message
    int32           devErrorCode;	// As returned by SCSI
    uint64          clientPrivate;	// As client passed to us in req
    uint32			reserved[4];	// reserved for internal use
}  evm_IOStat_t;

/*------------------------------------------------------------------------------
 * Functions prototype.
 *------------------------------------------------------------------------------
 */

extern int evm_IOReq(evm_IOReq_t *req, 
                     void (*completion_callback)(evm_IOStat_t *stat));

int32
evm_IOMap(evm_volId_t       volId,          // input
          uint64            volOffset,      // input
          uint32            volLen,         // input (nb 512bytes blocks)
          scsi_id_t *       lunId,          // output
          int32 *           lunHandle,      // output
          uint32 *          lunOffset,      // output
          uint32 *          lunLen);        // output (nb 512bytes blocks)

/*----------------------------------------------------------------------------*/

#endif // _EVM_SRVR_API_H_
