/*------------------------------------------------------------------------------
 * RCS:  $Id: evm_events.c,v 1.13 2002/08/01 17:56:08 sandrineb Exp $
 * Copyright (C) 2001, Agile Storage, Inc.
 *
 * Name         : evm_event.c
 *
 * Description  : Contains the functions that support sending event messages.
 *
 * Created by   : Greg Seibert
 *                Isabelle Cnudde
 *
 * Date Created : 25-AUG-2001
 *
 *------------------------------------------------------------------------------
 */

#include <unistd.h>
#include <string.h>

#include "nfx-incl.h"
#include "exports/anpssc-api.h"
#include "evm-api.h"
#include "evm_loc_defs.h"
#include "evm.h"

/*------------------------------------------------------------------------------
 * Name         : evm_sendEvt
 *
 * Description  : Send an event.
 *
 * Created by   : Isabelle Cnudde
 *
 * Date Created : 29-MAY-2002
 *
 *------------------------------------------------------------------------------
 */

void
evm_sendEvt(uint32      event,
            evm_volId_t volId,
            int			state)
    
{
    nfx_eventMsg_t      evtMsg;
    evm_VolConfig_t     volCfg;
    
    EVM_DBG_PRINT(("evm_sendEvt(event %d volId 0x%llx)",
                   event, volId.whole_thing));
    
    /* Get volume name.
     */
    get_lv_cfg_byid(volId, &volCfg, NULL);
    
    /* Fill event.
     */
    bzero((void *) &evtMsg, sizeof(nfx_eventMsg_t));
    evtMsg.msgType = NFX_EVENT_POST;
    //evtMsg.nodeName;
    evtMsg.needAck = FALSE;
    //evtMsg.status;
    evtMsg.eventScope = NFX_EVENT_LOCAL;
    evtMsg.eventType = NFX_EVENT_VOLUME;
    //eventType.reason;
    evtMsg.param.volEvent.event = event;
    evtMsg.param.volEvent.volId = volId;
    evtMsg.param.volEvent.state = state;
    strcpy(evtMsg.param.volEvent.volName, volCfg.name);
    
    /* Send event.
     */
	sendAgileMsg(-1, &evtMsg, sizeof(nfx_eventMsg_t),
                 "eventd", NFX_SSC_UNIX, SSC_SLOT, MGMTPLANE,
                 NULL, NULL, 0);

    return;
}

/*----------------------------------------------------------------------------*/

