/*
 *-----------------------------------------------------------------
 *
 * Name:
 *
 * RCS:  $Id: crashdump.c,v 1.15 2002/11/26 22:29:17 rick Exp $
 *
 * Copyright (C) 2002, Agile Storage, Inc.
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	4/7/02
 *
 *-----------------------------------------------------------------
 */

#include <nfx-incl.h>
#include <sys/mips.h>
#include <sys/amd29f040.h>
#include <exports/from-api.h>
#include <exports/crashdump-api.h>
#include <exports/cm-api.h>
#include "crashdump.h"

crashdump_t	saved_cd;
crashdump_t	pending_cd CPU_PRIVATE;
crashdump_t	* new_cd CPU_PRIVATE;
address_t	exc_nmi;

int crashdump_init_done CPU_PRIVATE;

extern address_t	stack_addr;
/* #define STKSIZE	0x3000 */

extern nfx_spinlock_t	prom_lock;

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	4/7/02
 *
 *-----------------------------------------------------------------
 */
uint32
crashdump_checksum(
	crashdump_t	* cptr
)
{
	uint32	sum = 0;
	uchar8	* bptr;

	for (bptr=(uchar8 *)cptr; bptr < (uchar8 *)&(cptr->checksum); bptr++)
		sum += *bptr++;

	return sum;
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	4/7/02
 *
 *-----------------------------------------------------------------
 */
void
crashdump_init()
{
	extern uchar8 ImageVerString[];
	extern int exc_nmi_handler();

	crashdump_t	* cptr;

    /*
     * check crashdump space
     */
    if (sizeof(crashdump_t) > CRASHDUMP_MAX_SIZE) {
        printf("\nWARNING: size of crashdump_t 0x%x exceeded reserved space 0x%x\n",
                (uint32) sizeof(crashdump_t), (uint32) CRASHDUMP_MAX_SIZE);
        return;
    }

	new_cd = &pending_cd;
	cptr = new_cd;

	bzero(cptr, sizeof(crashdump_t));

	cptr->signature = CRASHDUMP_SIGNATURE;
	cptr->format = CRASHDUMP_FORMAT;
#ifdef PROM
	strncpy(cptr->prom_version, ImageVerString, CRASHDUMP_STRING_LEN);
	cptr->flags |= CRASHDUMP_FLAGS_PROM_VER;
	cptr->flags |= CRASHDUMP_FLAGS_SOURCE_PROM;
#else
	strncpy(cptr->prom_version, getPromImageVerString(), CRASHDUMP_STRING_LEN);
	cptr->flags |= CRASHDUMP_FLAGS_PROM_VER;
	strncpy(cptr->rt_version, ImageVerString, CRASHDUMP_STRING_LEN);
	cptr->flags |= CRASHDUMP_FLAGS_RT_VER;
	cptr->flags |= CRASHDUMP_FLAGS_SOURCE_RT;
	if (crashdump_getTimeStamp(cptr->start_time))
		cptr->flags |= CRASHDUMP_FLAGS_START_TIME;
#endif
	set_nmi_exception_handler((address_t)exc_nmi_handler);

	exc_crashdump_set_stacktop(stack_addr+STKSIZE);

	cptr->checksum = crashdump_checksum(cptr);
    crashdump_init_done = 1;
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	4/7/02
 *
 *-----------------------------------------------------------------
 */
int32
crashdump_get(
	crashdump_t	* cptr
)
{
	from_readbytes(FROM_CRASHDUMP_OFFSET, (void *)cptr, sizeof(crashdump_t));

	return 1;
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	4/7/02
 *
 *-----------------------------------------------------------------
 */
int32
crashdump_valid(
	crashdump_t	* cptr
)
{
	if (cptr->signature == CRASHDUMP_SIGNATURE) {
		if (cptr->checksum == crashdump_checksum(cptr))
			return 1;
		else
			printf("Invalid crashdump checksum\n");
	}

	return 0;
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	4/7/02
 *
 *-----------------------------------------------------------------
 */
void
crashdump_save(
	crashdump_t	* cptr
)
{
    if (!crashdump_init_done) {
        printf("crashdump not enabled, cannot save\n");
        return;
    }
	printf("writing crashdump\n");
    from_programbytes(FROM_CRASHDUMP_OFFSET, (void *)cptr, sizeof(crashdump_t));
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	4/7/02
 *
 *-----------------------------------------------------------------
 */
void
crashdump_save_regs(
	crashdump_t	* cptr
)
{
	uint32	i;

	for (i=0; i<NREGS; i++)
		cptr->regs[i] = EXC_DBGREG[i];

	cptr->flags |= CRASHDUMP_FLAGS_REGS;
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	4/7/02
 *
 *-----------------------------------------------------------------
 */
void
crashdump_save_stack(
	crashdump_t	* cptr
)
{
	address_t	sp;
	uint32		i;

	cptr->stacksize = 0;
	sp = EXC_DBGREG[R_SP];
	cptr->sp = sp;
	if (!(sp & 3)) {
		for (i=0; i<CRASHDUMP_STACK_LEN; i++) {
			if (sp >= cptr->stacktop) {
				break;
			}
			else {
				cptr->stack[i] = *(uint32 *)sp;
				sp += 4;
				cptr->stacksize++;
			}
		}
	}

	cptr->flags |= CRASHDUMP_FLAGS_STACK;
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	4/7/02
 *
 *-----------------------------------------------------------------
 */
int32
crashdump_getTimeStamp(
	uchar8	* time_str
)
{

	return 0;
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	4/7/02
 *
 *-----------------------------------------------------------------
 */
static uint32
label_center(
    uchar8  * buf_p,
	uchar8	* str,
	int32	width
)
{
    uchar8 * bp = buf_p;
	int32 len = strlen(str);
	int32 isp = (width - len) / 2;
	int32 i;

	for (i = isp; i > 0; i--)
		bp += snprintf (bp, exc_fblen(bp), " ");
	bp += snprintf (bp, exc_fblen(bp), "%s", str);
	for (i = width - (isp + len); i > 0; i--)
		bp += snprintf (bp, exc_fblen(bp), " ");
    return((uint32)(bp - buf_p));
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	4/7/02
 *
 *-----------------------------------------------------------------
 */
uint32
crashdump_displayRegs(
	crashdump_t	*   cptr,
    uint8 *         buf_p
)
{
	uint32	    regno, i, per_col, width;
    uint8 *     bp;

#if __mips >= 3
	width = 16;
	per_col = 4;
#else
	width = 8;
	per_col = 8;
#endif

    bp = buf_p;

	for (regno=0; regno < NREGS; regno += per_col) {
		bp += snprintf(bp, exc_fblen(bp), "    ");
		for (i = 0; i < per_col; i++) {
			if (regno + i >= NREGS)
				break;
			if (regno < 32) {
				bp += snprintf(bp, exc_fblen(bp), " ");
                bp += label_center(bp, gpr_regs[regno + i], width);
			}
			else if (regno < NONFPREGS) {
				bp += snprintf(bp, exc_fblen(bp), " ");
                bp += label_center(bp, cp0_regs[regno-32 + i], width);
			}
		}
		bp += snprintf(bp, exc_fblen(bp), "\n");

		if (regno < 32)
            bp += snprintf(bp, exc_fblen(bp), "$%02d-", regno);
		else
			bp += snprintf(bp, exc_fblen(bp), "    ");
		for (i = 0; i < per_col; i++) {
			if (regno + i >= NREGS)
				break;
#if __mips >= 3
			bp += snprintf(bp, exc_fblen(bp), " %016llx", cptr->regs[regno + i]);
#else
			bp += snprintf(bp, exc_fblen(bp), " %08x", cptr->regs[regno + i]);
#endif
		}
		bp += snprintf(bp, exc_fblen(bp), "\n");
	}
    return((uint32) (bp - buf_p));
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	4/7/02
 *
 *-----------------------------------------------------------------
 */
int32
crashdump_more(void)
{
	uchar8	cmd[80];

	printf("Press <Enter> to continue, q<Enter> to quit ");
	get_input(cmd);
	if (strlen(cmd))
		return 1;
	else
		return 0;
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	4/7/02
 *
 *-----------------------------------------------------------------
 */
uint32
crashdump_displayRawStack(
	crashdump_t	*   cptr,
    uint8 *         buf_p,
    boolean         iflag
)
{
	uint32 *    saved_stack = (uint32 *)cptr->stack;
	uint32 *    sp = (uint32 *)cptr->sp;
	uint32	    i;
    uint8 *     bp;

    bp = buf_p;

	for (i=0; i<cptr->stacksize; i++) {
        if (iflag) {
            printf("    %p = 0x%x\n", sp++, *saved_stack++);
            if ((i & 0xF) == 0)
                if (crashdump_more())
                    return(0);
        }
        else {
            bp += snprintf(bp, exc_fblen(bp), "    %p = 0x%x\n", sp++, *saved_stack++);
        }
	}
    return((uint32) (bp - buf_p));
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	4/7/02
 *
 *-----------------------------------------------------------------
 */
uint32
crashdump_backtrace(
	crashdump_t	*   cptr,
    uint8 *         buf_p
)
{
	extern uchar8 ImageVerString[];
    uint8 *     bp;

    bp = buf_p;

	if (!strcmp(cptr->rt_version, ImageVerString))
		bp += format_stacktrace(cptr, bp);
	else
		bp += snprintf(bp, exc_fblen(bp), "\nMismatch in version string, skipping stacktrace\n\n");
    return((uint32) (bp - buf_p));

}

#if defined(PROM) || defined(SSC_MGMT)
/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	4/7/02
 *
 *-----------------------------------------------------------------
 */
crashdump_t *
get_pending_crashdump(void)
{
	return &pending_cd;
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	4/8/02
 *
 *-----------------------------------------------------------------
 */
void
set_nmi_exception_handler(
	address_t	addr
)
{
	exc_nmi = addr;
}
#endif

