/*
 *-----------------------------------------------------------------
 *
 * Name:	debug.c
 *
 * RCS:  $Id: debug.c,v 1.11 2002/11/05 00:44:05 rick Exp $
 *
 * Copyright (C) 2001, Agile Storage, Inc.
 *
 * Description:	GDB's debug stub
 *
 * Created by:	Rick Lund
 *
 * Date Created:	10/6/01
 *
 *-----------------------------------------------------------------
 */

#ifndef PROM

/*
 * Messages are transmitted over an RS232 serial link, and are of the
 * form:
 
 *      ^V type+len sequence# data checkSum
 
 * These messages are transmitted by the routine putpkt and received by
 * getpkt.
 */

#include <nfx-incl.h>
#include <sys/mips.h>
#include "except-api.h"
#include "except.h"
#include "debug.h"
#include <exports/mem-api.h>
#include <exports/serio-api.h>
#include <exports/cm-api.h>
#include <exports/esm-api.h>

/* All packets start with a SYN */
#define SYN			'\026'

/* MIPS SPP signal numbers */
#define	SPP_SIGTRAP		5
#define	SPP_SIGINT		2
#define	SPP_SIGSEGV		11
#define	SPP_SIGBUS		10
#define	SPP_SIGILL		4
#define	SPP_SIGFPE		8
#define	SPP_SIGTERM		15

#define WSTOPPED		0177
#define WSETSTOP(stat, sig)	*(stat) = ((sig) << 8) | WSTOPPED
#define WSETEXIT(stat,ret,sig)	*(stat) = ((ret) << 8) | (sig)

#ifndef EFAULT
#define EFAULT			14
#define EINVAL			22
#endif

#define DATA	0x00
#define ACKPKT	0x20
#define nextseq(x)		(((x) + 1) & 63)

#if defined(RTC) || defined(INET)
#define TIMEOUT 5	/* timeout in seconds */
#else
#define TIMEOUT 3000000	/* timeout in loops */
#endif

int			txseq;
int			rxseq;
int			lastwasget;
int			clifd;
int			hostfd;
int			vflag;
int			Vflag;
jmp_buf		pktstart;
jmp_buf		timeout;
int			newgdb;

#define MAXMSG	80

#if RAL_XXX
extern char		clientcmd[LINESZ];
extern char		* clientav[MAX_AC];
extern int		clientac;
#endif

#if RAL_XXX
extern jmp_buf	gobuf;
extern int		gobufvalid;
#endif

int32	trace_mode CPU_PRIVATE;
Bps		Bpt[MAX_AC];	/* user break points                    */
Bps		BptTmp;			/* tmp bpt used for continue cmd        */
Bps		BptTrc;			/* bpt for tracing                      */
Bps		BptTrcb;		/* bpt for tracing through branches     */


static int
#ifdef __GNUC__
inline
#endif
readc (char *msg, unsigned int state);

int do_req (int ac, char *av[]);
void dbg_init();
void dbgmode ();
void putmsg (char * msg);
int getmsg (char * msg);
void putpkt (int seq, char * msg);
int getpkt (int *seq, char * msg);

int32 argvize (char8   * av[], char8   * s);
void signextend( address_t   * addr);



#undef SBD_DISPLAY
#define SBD_DISPLAY(msg, n)	printf("%s\n", msg)

/*
 * RAL - my functions since real read/write/open don't exist
 */
void dbg_read (int port, char *ch, int len);
void dbg_write (int port, char *buf, int len);
void dbg_open (char *hostport, int core_id, int * hostfd, int * clifd);
void dbg_ioctl (int port, int cmd, int *tbuf);
void dbg_close (int port);


int	gdb_init;
int	gdb_init_continue;

uint64  currThreadId;
cpureg_t currThreadReg[NREGS];

volatile uint32 inDebugger = 0;

int32	runtime_exception CPU_PRIVATE;

int32 (*dbg_put_char_func)();
int32 (*dbg_get_char_func)();
int32 (*dbg_char_avail_func)();
int32 (*cli_put_char_func)();
int32 (*cli_get_char_func)();
int32 (*cli_char_avail_func)();
/*************************************************************
 *  debug(ac,av)
 *      The 'debug' command
 */
int32
debug (ac, av)
	int		ac;
	char	*av[];
{
	char			*hostport = NULL;
	int				sflag;
#if RAL_XXX
	extern int		optind;
	extern char		*optarg;
#endif
	int				c, arg, uart_rate = 57600;
	
#if RAL_XXX
	strcpy (clientcmd, av[0]);
	strcat (clientcmd, " ");
#endif
	
/*
	hostport = getenv ("dbgport");
	if (!hostport)
		hostport = getenv ("hostport");
*/
	
	vflag = 0;
	Vflag = 0;
	sflag = 0;
	
#if RAL_XXX
	optind = 0;
	while ((c = getopt (ac, av, "vVsh:")) != EOF)
		switch (c) {
			case 'v':
				vflag++; break;
			case 'V':
				Vflag++; break;
			case 's':
				sflag++; break;
			case 'h':
				hostport = optarg; break;
			default:
				return (-1);
		}
	
	while (optind < ac) {
		strcat (clientcmd, av[optind++]);
		strcat (clientcmd, " ");
	}
#else
	for (arg = 1;  arg < ac; arg++)
	{
		//printf("checking arg %d %s\n", arg, av[arg]);

		if (!strcmp(av[arg], "-v")) {
			//printf("setting vflag\n");
			vflag++;
		}
		else if (!strcmp(av[arg], "-V")) {
			//printf("setting Vflag\n");
			Vflag++;
		}
		else {
			uart_rate = atoi(av[arg]);
			//printf("setting uart_rate to %d\n", uart_rate);
		}
	}

	if (uart_rate != 57600 &&
        uart_rate != 38400 &&
        uart_rate != 19200 &&
        uart_rate != 9600)
	{
		printf("invalid uart_rate %d\n", uart_rate);
		return -1;
	}
#endif
	
/*
	if (Vflag)
		printf ("hostport=%s\n", hostport);
*/
	dbg_open (hostport, nfxMyCoreId, &hostfd, &clifd);
	if (hostfd == -1) {
		printf ("can't open %s\n", hostport);
		return (1);
	}
	if (Vflag)
		printf ("clifd = %d\n, hostfd = %d\n", clifd, hostfd);
	
	watchdog_disable();

	while (1) {
		char	c;
		int		rx_rdy;
		dbg_ioctl (hostfd, FIONREAD, &rx_rdy);
		if (rx_rdy)
			dbg_read (hostfd, &c, 1);
		else
			break;
	}

#if 0
#ifdef SIBYTE
	{
	int32 sb1250_uart( uint32  op, uint32  chan, int32   data);

    // Quick&dirty fix to get the uart for 2nd port
    // initialized with rts/cts flow control.
    sb1250_uart(OP_INIT, 1, uart_rate);
	}
#endif
#endif

	/* wait for "start" signal, this make initialisation much quicker */
	SBD_DISPLAY ("WAIT", CHKPNT_WAIT);

    inDebugger = nfxMyCPUId;

	do {
		c = readc (NULL, -1);
		if (c < 0 && Vflag)
			(*cli_put_char_func)('.');
		if (c == GDB_QUIT) {
			printf("quitting\n");
		    inDebugger = 0;
			return 0;
		}
	} while (c != '\r');
	dbg_write (hostfd, "\r", 1);
	
	txseq = rxseq = lastwasget = 0;
	
#if RAL_XXX
	if (!sflag) {
		clientac = argvize (clientav, clientcmd);
		/* set client's a0 (argc) & a1 (argv) and stack */
		EXC_DBGREG[R_SP] = clienttos ();
		initstack (clientac, clientav, 1);
		closelst (2);
		clrhndlrs ();
		EXC_DBGREG[R_STATUS] = initial_sr;
#ifdef Icr
		/* RM7000 interrupt extensions */
		Icr = IplLo = IplHi = 0;
#endif
		sbdenable (getmachtype ());  /* set up i/u hardware */
#ifdef FLOATINGPT
		Fcr = 0;		/* clear outstanding exceptions / enables */
#endif
	}
#endif
	
	dbg_init();

	clrbpts ();			/* clear user breakpoints */
	
	SBD_DISPLAY ("RDBG", CHKPNT_RDBG);
	
	gdb_init = 1;
	gdb_init_continue = 0;

	dbgmode ();

    inDebugger = 0;

	return 0;
}

const char * const wpflags[] = {"",  "w",  "r", "rw", "x", "xw", "xr", "xrw"};

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	4/8/02
 *
 *-----------------------------------------------------------------
 */
void
dbg_init()
{
	uint32	i;

	for (i=0; i<MAX_BPT; i++)
		Bpt[i].addr = NO_BPT;
	BptTmp.addr = NO_BPT;
	BptTrc.addr = NO_BPT;
	BptTrcb.addr = NO_BPT;
	trace_mode = TRACE_NO;

	if (runtime_exception == EXCEPTION_NONE) {
		printf("!runtime_exception, R_EPC = 0\n");
		EXC_DBGREG[R_EPC] = 0;
	}
	else
#ifdef __long64
		printf("dbg_init, R_EPC = %p\n", (void *)EXC_DBGREG[R_EPC]);
#else
#if __mips >= 3
		printf("dbg_init, R_EPC = 0x%llx\n", EXC_DBGREG[R_EPC]);
#else
		printf("dbg_init, R_EPC = 0x%lx\n", EXC_DBGREG[R_EPC]);
#endif
#endif
}

/*************************************************************
 *  dbgmode(type) enter dbx mode
 */
void
dbgmode ()
{
	char            rxstr[MAXMSG], *rxarg[8];
	int             req, sig, stat, ac, exited;
#if RAL_XXX
	int		    hit = 0; 
	vaddr_t	    hitaddr = 0;
	size_t	    hitlen = 0;
#endif
	
	watchdog_disable();
    currThreadId = 0;

#if RAL_XXX
	if (setjmp (&gobuf) == 0) {
		/* normal */
		gobufvalid = 1;
		exited = 0;
	}
	else {
		/* program called cliexit() */
		gobufvalid = 0;
		exited = 1;
	}
#else
	exited = 0;
#endif
	
	switch (trace_mode) {
		case TRACE_DS:
			req = 's';
			break;
		case TRACE_DC:
			req = 'c';
			break;
		default:
			req = 'b';
			break;
	}
	
#if RAL_XXX
	if (trace_mode != TRACE_NO)
		hit = _mips_watchpoint_hit (&hitaddr, &hitlen);
#endif
	
	sig = SPP_SIGTRAP;
//printf("dbgmode, req = %c, ", req);
	if (req != 'b') {
		switch (EXC_DBGREG[R_CAUSE] & CAUSE_EXCMASK) {
			case CEXC_TLBL:
			case CEXC_TLBS:
				sig = SPP_SIGSEGV;
				break;
			case CEXC_MOD:
			case CEXC_ADEL:
			case CEXC_ADES:
			case CEXC_IBE:
			case CEXC_DBE:
				sig = SPP_SIGBUS;
				break;
			case CEXC_BP:
				sig = SPP_SIGTRAP;
				break;
			case CEXC_SYS:
			case CEXC_RI:
			case CEXC_CPU:
				sig = SPP_SIGILL;
				break;
			case CEXC_OVF:
				sig = SPP_SIGFPE; /* !! */
				break;
#ifdef R4000
			case CEXC_TRAP:
#ifndef SB1250
			case CEXC_IWATCH:
			case CEXC_DWATCH:
#endif
				sig = SPP_SIGTRAP;
				break;
			case CEXC_FPE:
				sig = SPP_SIGFPE;
				break;
#if RAL_XXX
			case CEXC_VCEI:
			case CEXC_VCED:
				sig = SPP_SIGTERM;
				break;
#endif
#endif /* R4000 */
		}
	}
//printf("sig = 0x%x\n", sig);

	if (exited)
		WSETEXIT (&stat, 0, 0);
	else
		WSETSTOP (&stat, sig);
	
	sprintf (rxstr, "0x1 %c 0x0 0x%x", req, stat);
	if (newgdb) {
		/* add pc, s8, sp */
#if defined(SSC_MGMT)
		sprintf (rxstr + strlen (rxstr), " 0x%lx 0x%lx 0x%lx",
#else
		sprintf (rxstr + strlen (rxstr), " 0x%llx 0x%llx 0x%llx",
#endif
											EXC_DBGREG[R_EPC],
											EXC_DBGREG[R_FP],
											EXC_DBGREG[R_SP]);

#if RAL_XXX
		if (hit)
			/* add wpaddr, wpflags, wplen */
			sprintf (rxstr + strlen (rxstr), " 0x%x %s%d", 
											hitaddr, wpflags[hit & 7], hitlen);
#endif
	}
	
	putmsg (rxstr);
	for (;;) {
		int	quit;
		quit = getmsg (rxstr);
		if (quit) {
			printf("quitting debugger\n");
			if (!gdb_init)
				debug_exit();
			break;
		}
		ac = argvize (rxarg, rxstr);
		quit = do_req (ac, rxarg);
		if (quit) {
			if (!gdb_init)
				debug_exit();
			break;
		}
		if (gdb_init_continue) {
			gdb_init_continue = 0;
			break;
		}
	}
    currThreadId = 0;

    inDebugger = 0;

}

/*************************************************************
 *  putmsg(msg)  send msg, including wait for the ACK
 */
void
putmsg (msg)
     char           *msg;
{
	int             type, ts, ns;
	
	ns = nextseq (txseq);
	
resend:
	/* send the message */
	putpkt (txseq, msg);
	
	/* wait for other end to send acknowledge, or a timeout */
	for (;;) {
		if ((type = getpkt (&ts, 0)) == ACKPKT && ts == ns) 
			break;
	
		if (vflag && type >= 0)
			printf("PutMsg: got type=%s seq=%d, wanted type=ACK seq=%d\r\n",
									(type == ACKPKT) ? "ACK" : "DATA", ts, ns);
	
		if (type == GDB_QUIT) {
			printf("putmsg: Quit\n");
		}
	
		if (type == -2) {
			/* timeout:the other end may have lost our last Rx ACK, resend it */
			if (lastwasget)
				putpkt (rxseq, 0);
			goto resend;
		}
	
		/* ACK for previous packet: resend current packet */
		if (type == ACKPKT && ts == txseq)
			goto resend;
	}
	
	txseq = ns;
	lastwasget = 0;
	if (Vflag)
		printf ("\r\n");
}

/*************************************************************
 *  getmsg(msg)  get msg, including send the ACK
 */
int
getmsg (msg)
     char           *msg;
{
	int             type, ts;
	
	/* wait for a data packet */
	for (;;) {
		if ((type = getpkt (&ts, msg)) == DATA && ts == rxseq)
			break;
	
		if (vflag && type >= 0) {
			printf("GetMsg: got type=%s seq=%d msg=<%s>, ",
								(type == ACKPKT) ? "ACK" : "DATA", ts, msg);
			printf("wanted type=DATA seq=%d\r\n", rxseq);
		}
	
		if (type == GDB_QUIT) {
			printf("getmsg: Quit\n");
			return 1;
		}

		/* checksum/timeout error: reacknowledge last rx to force resend */
		if (type < 0)
			putpkt (rxseq, 0);
	
		/* quietly ignore other packets */
	}
	
	/* acknowledge this packet */
	rxseq = nextseq (rxseq);
	putpkt (rxseq, 0);
	lastwasget = 1;

	return 0;
}

/*************************************************************
 *  putpkt(seq,msg) send a packet, if msg == 0, type = ACK
 */
void
putpkt (seq, msg)
	int             seq;	/* sequence number to be sent */
	char           *msg;	/* message to be sent */
{
	int             len, type, type_len, csum;
	char            tmp[80], *t = tmp;
	
	if (Vflag)
		printf ("PutPkt: seq=%d msg=<%s>\r\n", seq, msg ? msg : "ACK");
	
	if (msg) {
		type = DATA;
		len = strlen (msg);
	} else {
		type = ACKPKT;
		len = 0;
	}
	
	type_len = type | (len >> 6);
	len &= 0x3f;
	csum = 0;
	
#define addc(c) (*t++ = (c), csum += (c))
	
	/* header */
	*t++ = SYN;
	addc (type_len + '@');
	addc (len + '@');
	addc (seq + '@');
	
	/* message */
	if (msg)
		for (; *msg; msg++)
			addc (*msg);
	
#undef addc
	
	/* checksum */
	*t++ = ((csum >> 12) & 0x3f) + '@';
	*t++ = ((csum >> 6) & 0x3f) + '@';
	*t++ = (csum & 0x3f) + '@';
	/**t++ = '\n';*/
	
	dbg_write (hostfd, tmp, t - tmp);
}


#define STATE_START	0
#define STATE_TYPLEN1	1
#define STATE_LEN2	2
#define STATE_SEQ	3
#define STATE_MSG	4
#define STATE_CSUM1	5
#define STATE_CSUM2	6
#define STATE_CSUM3	7
#define STATE_END	8

static const char * const statenames[] = {
    "START", "TYPLEN1", "LEN2", "SEQ", "MSG", "CSUM1", "CSUM2", "CSUM3", "END"
};

/*************************************************************
 *  readc(msg, state)
 */
static 
#ifdef __GNUC__
inline
#endif
int
readc (msg, state)
	char           *msg;
	unsigned int    state;
{
	char	ch;
	int		dbg_rx_rdy;
	

	dbg_ioctl (hostfd, FIONREAD, &dbg_rx_rdy);
	if (dbg_rx_rdy <= 0 && (state > STATE_START || !msg)) {
		/* in packet or waiting for an ACK: timeout the transfer */
		int i = TIMEOUT;
	
		do {
			int	cli_rx_rdy;
			dbg_ioctl(clifd, FIONREAD, &cli_rx_rdy);
			if (cli_rx_rdy) {
				dbg_read (clifd, &ch, 1);
				if (ch == CNTRL('E'))
					return (GDB_QUIT);
			}
			if (i-- <= 0) 
				return (-1);
			dbg_ioctl (hostfd, FIONREAD, &dbg_rx_rdy);
		} while (dbg_rx_rdy <= 0);
	}
	
	dbg_read (hostfd, &ch, 1);
	if (ch == CNTRL('E'))
		return GDB_QUIT;
	return (ch);
}


/*************************************************************
 *  getpkt(seq,msg) get a packet as a string, returns type
 */
int
getpkt (seq, msg)
	int            *seq;	/* received sequence number */
	char           *msg;	/* destination for message */
{
	unsigned int	state;
	int				type = 0;
	char			xmsg[MAXMSG], *rmsg;
	int				len = 0, rlen = 0;
	int				csum = 0, rsum = 0;
	int				ch;
	

	if (!(rmsg = msg))
		rmsg = xmsg;
	
	state = STATE_START;

	while (state != STATE_END) {
	
		if ((ch = readc (msg, state)) < 0) {
			if (ch == GDB_QUIT)
				return (GDB_QUIT);
			/* timeout: return error to getmsg/putmsg */
			if (vflag)
				printf ("GetPkt: state %s, timeout\r\n", statenames[state]);
			return (-2);
		}
	
		if (ch == SYN) {
			/* restart packet on a SYN */
			if (state != STATE_START && vflag)
				printf ("GetPkt: state %s, restart\r\n", statenames[state]);
			state = STATE_TYPLEN1;
			csum = 0;
			rsum = 0;
			rlen = 0;
			continue;
		}
	
		switch (state) {
			case STATE_START:
				/* waiting for SYN */
				break;
	
			case STATE_TYPLEN1:
				if (ch < '@')
					goto badchar;
				rsum += ch;
				type = ch & ACKPKT;
				len = ch & 0x1f;
				state++;
				break;
	
			case STATE_LEN2:
				if (ch < '@')
					goto badchar;
				rsum += ch;
				len = (len << 6) + (ch - '@');
				if (len >= MAXMSG) {
					if (vflag)
						printf ("GetPkt: bad length %d\r\n", len);
					return (-1);
				}
				state++;
				break;
	
			case STATE_SEQ:
				if (ch < '@')
					goto badchar;
				rsum += ch;
				*seq = ch - '@';
				state += (len == 0) ? 2 : 1;
				break;
	
			case STATE_MSG:
				rsum += ch;
				rmsg[rlen] = ch;
				if (++rlen == len)
					state++;
				break;
	
			case STATE_CSUM1:
			case STATE_CSUM2:
			case STATE_CSUM3:
				if (ch < '@')
					goto badchar;
				csum = (csum << 6) + (ch - '@');
				state++;
				break;
		}
	}
	
	if ((rsum & 0x3ffff) != csum) {
		if (vflag)
			printf ("GetPkt: csum error tx=%x rx=%x\r\n", csum, rsum);
		return (-1);
	}
	
	rmsg[rlen] = '\0';
	
	if (Vflag)
		printf("GetPkt: seq=%d type=%s msg=<%s>\r\n", *seq,
									(type == ACKPKT) ? "ACK" : "DATA", rmsg);
	
	return (type);
	
badchar:
	if (vflag)
		printf ("GetPkt: state %s, bad char=0x%x\r\n", statenames[state], ch);

	return (-1);
}



static sreg_t *
mapdbgreg (adr, put)
    unsigned long adr;
    int put;
{
    if (adr < 32) {
		adr = R_ZERO + adr;
#ifdef FLOATINGPT
#if RAL_XXX
    } else if (adr < 64) {
		adr = R_F0 + adr - 32;
#if __mips >= 3
		if (!(EXC_DBGREG[R_STATUS] & SR_FR))
		    adr &= ~1;		/* odd registers "don't exist" */
#endif
#endif
#endif
    } else {
		switch (adr) {
			case 96:
				if (runtime_exception == EXCEPTION_NMI)
				    adr = R_ERREPC;
				else
				    adr = R_EPC;
			    break;
			case 97:
			    adr = R_CAUSE;
			    break;
			case 98:
			    adr = R_HI;
			    break;
			case 99:
			    adr = R_LO;
			    break;
#ifdef FLOATINGPT
			case 100:
#if RAL_XXX
			    adr = R_FCR;
#else
			    adr = R_ZERO;
#endif
			    break;		/* fcsr */
			case 101:
			    adr = R_ZERO;
			    break;		/* feir */
#endif
			default:
			    adr = R_ZERO;
			    break;
		}
    }

    if (currThreadId == 0 || put)
        return &EXC_DBGREG[adr];
    else
        return &currThreadReg[adr];
}


static sreg_t
getdbgreg (adr, size)
    unsigned long adr;
    int size;
{
    sreg_t *rp = mapdbgreg (adr, 0);

#if defined(FLOATINGPT) && __mips >= 3
    if (adr >= 32 && adr < 64 && size == 4) {
		/* pick correct half of 64-bit register */
		unsigned int *frp = (unsigned int *)rp;
#if MIPSEB
		adr = (adr & 1) ^ 1;	/* register order != memory order */
#else
		adr = (adr & 1);
#endif
		return frp[adr];
    }
#endif

    return *rp;
}


static sreg_t
putdbgreg (adr, size, val)
    unsigned long adr;
    int size;
    sreg_t val;
{
    sreg_t *rp = mapdbgreg (adr, 1);
    sreg_t oval;

//printf("putdbgreg, adr = %p, val = 0x%x\n", adr, size);

#if defined(FLOATINGPT) && __mips >= 3
    if (adr >= 32 && adr < 64 && size == 4) {
		/* pick correct half of 64-bit register */
		unsigned int *frp = (unsigned int *)rp;
#if MIPSEB
		adr = (adr & 1) ^ 1;	/* register order != memory order */
#else
		adr = (adr & 1);
#endif
		oval = frp[adr];
		frp[adr] = val;
		return oval;
    }
#endif

    oval = *rp;
    if (adr != 0)
		*rp = val;
    return oval;
}

#ifdef PROM
/*
 * The prom doesn't (YET) have access to runtime
 * thread state, so these just return 0 now.
 */
uint32
esm_gdbGetNextThreadId(uint32 tid)
{
    return 0;
}

uint32
esm_gdbIsThreadAlive(uint32 tid)
{
    return 0;
}

void
esm_gdbGetRegs(cpu_reg_t *ptr, int num, uint32 tid)
{
}
#endif // PROM


/*************************************************************
 *  do_req(ac,av) handle a dbx request
 *      av[0]=pid av[1]=req av[2]=addr av[3]=data
 */
int
do_req (ac, av)
	int             ac;
	char           *av[];
{
	unsigned long	adr;
	uint32			val32 = 0;
	sreg_t			*rp = 0;
	sreg_t			dat;
	int				i;
	int				err = 0;
	char			msg[80];

	if (ac < 4) {
		if (vflag) {
			printf ("ac=%d: ", ac);
			for (i = 0; i < ac; i++)
				printf ("av[%d]=%s ", i, av[i]);
			printf ("\r\n");
		}
		goto badpkt;
	}

	if (av[1][1])
		goto badpkt;

#ifdef __long64
	llatob ((uint64 *)&adr, av[2], 0);
	signextend(&adr);
#else
	atob ((uint32 *)&adr, av[2], 0);
#endif
#if __mips >= 3
	llatob (&dat, av[3], 0);
#else
	atob ((uint32 *)&dat, av[3], 0);
#endif

//printf("do_req, av[1][0] = %c, adr = %p, dat = %llx\n", av[1][0], (void *)adr, dat);
	switch (av[1][0]) {
		case 'i':			/* read Ispace */
		case 'd':			/* read Dspace */
#if RAL_XXX
			if (adr >= K0BASE && adr < K2BASE && (adr & 3) == 0)
				val32 = load_word (adr);
			else {
				err = -1;
				val32 = EFAULT;
			}
#else
//printf("response: adr %p = 0x%x\n", adr, load_int(adr));
			if (is_valid_addr(adr))
				val32 = load_int (adr);
			else {
printf("bad addr, adr = %p\n", (void *)adr);
//				err = -1;
//				val32 = EFAULT;
				val32 = -1;
			}
#endif
			sprintf (msg, "%s %s 0x%x 0x%x", av[0], av[1], err, val32);
			putmsg (msg);
			break;

		case 'h':			/* read half-word */
#if RAL_XXX
			if (adr >= K0BASE && adr < K2BASE && (adr & 1) == 0)
				val32 = load_half (adr);
			else {
				err = -1;
				val32 = EFAULT;
			}
#else
			if (is_valid_addr(adr))
				val32 = load_half (adr);
			else {
				err = -1;
				val32 = EFAULT;
			}
#endif
			sprintf (msg, "%s %s 0x%x 0x%x", av[0], av[1], err, val32);
			putmsg (msg);
			break;

		case 'p':			/* read byte */
#if RAL_XXX
			if (adr >= K0BASE && adr < K2BASE)
				val32 = load_byte (adr);
			else {
				err = -1;
				val32 = EFAULT;
			}
#else
			if (is_valid_addr(adr))
				val32 = load_byte (adr);
			else {
				err = -1;
				val32 = EFAULT;
			}
#endif
			sprintf (msg, "%s %s 0x%x 0x%x", av[0], av[1], err, val32);
			putmsg (msg);
			break;

#if __mips >= 3
		case 't':			/* read single 64-bit reg */
//printf("response: reg %d = 0x%lx\n", adr, getdbgreg(adr, 8));
			sprintf (msg, "%s %s 0x0 0x%llx", av[0], av[1], getdbgreg (adr, 8));
			putmsg (msg);
			break;
#endif

		case 'r':			/* read reg */
			val32 = getdbgreg (adr, 4);
//printf("response: reg %d = 0x%lx\n", adr, val32);
			sprintf (msg, "%s %s 0x0 0x%x", av[0], av[1], val32);
			putmsg (msg);
			break;

#if __mips >= 3
		case 'u':			/* read multiple 64-bit regs */
			for (i = 0; i < 32; i++) {
				if (adr & 1) {
                    if (currThreadId == 0)
                        rp = &EXC_DBGREG[i];
                    else
                        rp = &currThreadReg[i];
                }
				sprintf (msg, "%s %s 0x0 0x%llx", av[0], av[1], *rp);
				putmsg (msg);
				adr >>= 1;
			}
			break;
#endif

		case 'g':			/* read multiple regs */
			for (i = 0; i < 32; i++) {
				if (adr & 1) {
                    if (currThreadId == 0)
                        rp = &EXC_DBGREG[i];
                    else
                        rp = &currThreadReg[i];
                }
				val32 = *rp;
				sprintf (msg, "%s %s 0x0 0x%x", av[0], av[1], val32);
				putmsg (msg);
				adr >>= 1;
			}
			break;

		case 'I':			/* write Ispace */
		case 'D':			/* write Dspace */
			//printf("write Ispace or Dspace\n");
			//printf("adr = %p, dat = 0x%x\n", adr, dat);
#if RAL_XXX
			if (adr >= K0BASE && adr < K2BASE && (adr & 3) == 0) {
				val32 = load_word (adr);
				store_word (adr, dat);
			} else {
				err = -1;
				val32 = EFAULT;
			}
#else
			if (is_valid_addr(adr)) {
				val32 = load_int (adr);
				RAL_store_int (adr, dat);
			} else {
				err = -1;
				val32 = EFAULT;
			}
#endif
			sprintf (msg, "%s %s 0x%x 0x%x", av[0], av[1], err, val32);
			putmsg (msg);
			break;

		case 'H':			/* write half-word */
			//printf("write half-word, adr = %p, dat = 0x%x\n", adr, dat);
#if RAL_XXX
			if (adr >= K0BASE && adr < K2BASE && (adr & 1) == 0) {
				val32 = load_byte (adr);
				store_half (adr, dat);
			} else {
				err = -1;
				val32 = EFAULT;
			}
#else
			if (is_valid_addr(adr)) {
				val32 = load_half (adr);
				store_half (adr, dat);
			} else {
				err = -1;
				val32 = EFAULT;
			}
#endif
			sprintf (msg, "%s %s 0x%x 0x%x", av[0], av[1], err, val32);
			putmsg (msg);
			break;

		case 'P':			/* write byte */
#if RAL_XXX
			if (adr >= K0BASE && adr < K2BASE) {
				val32 = load_byte (adr);
				store_byte (adr, dat);
			} else {
				err = -1;
				val32 = EFAULT;
			}
#else
			if (is_valid_addr(adr)) {
				val32 = load_byte (adr);
				store_byte (adr, dat);
			} else {
				err = -1;
				val32 = EFAULT;
			}
#endif
			sprintf (msg, "%s %s 0x%x 0x%x", av[0], av[1], err, val32);
			putmsg (msg);
			break;

#if __mips >= 3
		case 'T':			/* write 64-bit reg */
			{
			sprintf (msg, "%s %s 0x0 0x%llx", av[0], av[1], 
			putdbgreg (adr, 8, dat));
			putmsg (msg);
			}
			break;
#endif

		case 'R':			/* write reg */
			val32 = putdbgreg (adr, 4, (long)dat);
			sprintf (msg, "%s %s 0x0 0x%x", av[0], av[1], val32);
			putmsg (msg);
			break;

		case 's':			/* step */
			//printf("step, adr = %p\n", adr);
			/* optional addr */
			if (adr != 1)
				EXC_DBGREG[R_EPC] = adr;
			sstep ();
			break;

		case 'c':			/* continue */
			printf("continue, adr = %p\n", (void *)adr);
			/* optional addr */
			if (adr != 1 && currThreadId == 0)
				EXC_DBGREG[R_EPC] = adr;
            currThreadId = 0;
#if 0
			trace_mode = TRACE_DC;
			_go ();
#else
			if (gdb_init) {
				gdb_init = 0;
				gdb_init_continue = 1;
				godebug ();
				return 0;
			}
			godebug ();
#endif
			break;

#ifndef SSC_MGMT
		case 'x':			/* exit */
            currThreadId = 0;
			sprintf (msg, "%s %s 0x0 0x0", av[0], av[1]);
//			putmsg (msg);
			trace_mode = TRACE_NO;
#if RAL_XXX
			gobufvalid = 0;
#endif
			newgdb = 0;
			dbg_close (hostfd);
			clrbpts ();
			printf ("Exiting remote debug mode\r\n");
#if RAL_XXX
			cliexit ();
#else
			debug_exit();
			return 1;
#endif
			break;
#endif

		case 'B':	/* breakpoint set */
		case 'b':	/* breakpoint clear */
			if (ac < 5)
				goto badpkt;
			switch (av[4][0]) {
#if RAL_XXX
				case 'n':	/* return number of watchpoints */
					i = _mips_watchpoint_howmany ();
					sprintf (msg, "%s %s 0x0 0x%x", av[0], av[1], i < 0 ? 0 : i);
					break;
				case 'q':	/* return watchpoint capabilities */
					i = _mips_watchpoint_capabilities (adr);
					sprintf (msg, "%s %s 0x0 0x%x", av[0], av[1], i);
					break;
#endif
				case 'u':
					if (av[1][0] == 'B')
						goto badpkt;
					/* clear all breakpoints */
					clrbpts ();
					sprintf (msg, "%s %s 0x0 0x0", av[0], av[1]);
					newgdb = 1;
					break;
#if RAL_XXX
				case 'x':
					i = MIPS_WATCHPOINT_X;
					goto dobpt;
				case 'w':
					i = MIPS_WATCHPOINT_W;
					goto dobpt;
				case 'r':
					i = MIPS_WATCHPOINT_R;
					goto dobpt;
				case 'a':
					i = MIPS_WATCHPOINT_R | MIPS_WATCHPOINT_W;
					goto dobpt;
#endif
				case 'f':
					//printf("f: ");
					i = 0;
#if RAL_XXX
dobpt:
					//printf("dobpt: ");
#endif
					if (av[1][0] == 'B') {
						//printf("B, adr = %p, dat = 0x%llx\n", adr, dat);
						if (!setbpt (i, adr, (long)dat))
							goto error;
					}
					else {
						//printf("not B, adr = %p, dat = 0x%llx\n", adr, dat);
						if (!clrbpt (i, adr, (long)dat))
							goto error;
					}
					sprintf (msg, "%s %s 0x0 0x0", av[0], av[1]);
					break;
				default:
					goto badpkt;
			}
			putmsg (msg);
			break;

		case 'M': { /* Agile thread message */
            uint32  tid;

#if __mips >= 3
            tid = dat & 0x00000000ffffffff;
#else
            tid = dat;
#endif

			switch (adr) {
                case 'i':	/* return thread id */
                    tid = esm_gdbGetNextThreadId(tid);
                    sprintf (msg, "%s %s 0x%x 0x%x", av[0], av[1], err, tid);
                    putmsg (msg);
                    break;
                case 'a':	/* return thread alive */
                    tid = esm_gdbIsThreadAlive(tid);
                    sprintf (msg, "%s %s 0x%x 0x%x", av[0], av[1], err, tid);
                    putmsg (msg);
                    break;
                case 's':	/* set thread context */
                    if (tid == 0) {
                        currThreadId = 0;
                    }
                    else if (esm_gdbIsThreadAlive(tid)) {
                        currThreadId = tid;
                        esm_gdbGetRegs(currThreadReg, NREGS, tid);
                    }
                    sprintf (msg, "%s %s 0x%x 0x%x", av[0], av[1], err, tid);
                    putmsg (msg);
                    break;
                default:
                    if (vflag) {
                        printf("unknown agile thread request %c\n",
                               (int)adr);
                    }
					goto badpkt;
            }
            break;
        }

		default:
badpkt:
			if (vflag)
				printf ("unknown request type '%s %s %s %s'\r\n",
												av[0], av[1], av[2], av[3]);
error:
			sprintf (msg, "%s %s 0x%x 0x%x", av[0], av[1], -1, EINVAL);
			putmsg (msg);
			break;
	}

	return 0;
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	1/03/02
 *
 *-----------------------------------------------------------------
 */
void
dbg_read (
	int		port,
	char	*ch,
	int		len
)
{
	int		rx_rdy;
	char	c;

	if (port == hostfd) {
		while (1) {
			dbg_ioctl (hostfd, FIONREAD, &rx_rdy);
			if (!rx_rdy) {
				dbg_ioctl(clifd, FIONREAD, &rx_rdy);
				if (rx_rdy) {
					c = (*cli_get_char_func)();
					if (c == CNTRL('E')) {
						*ch = CNTRL('E');
						return;
					}
				}
			}
			else {
				*ch = (*dbg_get_char_func)();
				break;
			}
		}
	}
	else {
		*ch = (*cli_get_char_func)();
	}
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	1/08/02
 *
 *-----------------------------------------------------------------
 */
void
dbg_write (
	int		port,
	char	*buf,
	int		len
)
{
	int	i;

	if (port == hostfd) {
		for (i=0; i<len; i++) {
			(*dbg_put_char_func)(buf[i]);
		}
	}
	else {
		for (i=0; i<len; i++) {
			(*cli_put_char_func)(buf[i]);
		}
	}
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	1/08/02
 *
 *-----------------------------------------------------------------
 */
void
dbg_ioctl (
	int		port,
	int		cmd,
	int		*tbuf
)
{
	if (cmd == FIONREAD) {
		if (port == hostfd)
			*tbuf = (*dbg_char_avail_func)();
		else
			*tbuf = (*cli_char_avail_func)();
	}
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description: This function should return which uart port should be used
 *				for GDB communication with host.
 *
 * Created by:	Rick Lund
 *
 * Date Created:	1/08/02
 *
 *-----------------------------------------------------------------
 */
void
dbg_open (
	char	* hostport,
	int		core_id,
	int		* hostfd,
	int		* clifd
)
{
	if (isFC() || isSSC()) {
		cli_char_avail_func = char_avail;
		cli_get_char_func = getchar;
		cli_put_char_func = putchar;
		dbg_char_avail_func = alt_tty_char_avail;
		dbg_get_char_func = alt_tty_getchar;
		dbg_put_char_func = alt_tty_putchar;
		*hostfd = 1;
		*clifd = 0;
	}
	else if (isTXRX() || isFP()) {
#ifdef NFX_SMP
		if (core_id == 0) {
			cli_char_avail_func = char_avail;
			cli_get_char_func = getchar;
			cli_put_char_func = putchar;
			dbg_char_avail_func = alt_tty_char_avail;
			dbg_get_char_func = alt_tty_getchar;
			dbg_put_char_func = alt_tty_putchar;
			*hostfd = 1;
			*clifd = 0;
		}
		else {
			cli_char_avail_func = char_avail;
			cli_get_char_func = getchar;
			cli_put_char_func = putchar;
			dbg_char_avail_func = alt_tty_char_avail;
			dbg_get_char_func = alt_tty_getchar;
			dbg_put_char_func = alt_tty_putchar;
			*hostfd = 0;
			*clifd = 1;
		}
#else
		cli_char_avail_func = char_avail;
		cli_get_char_func = getchar;
		cli_put_char_func = putchar;
		dbg_char_avail_func = alt_tty_char_avail;
		dbg_get_char_func = alt_tty_getchar;
		dbg_put_char_func = alt_tty_putchar;
		*hostfd = 1;
		*clifd = 0;
#endif
	}
	else {
		/*
		 * -1 = error
		 */
		*hostfd = -1;
		*clifd = -1;
	}
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	4/8/02
 *
 *-----------------------------------------------------------------
 */
void
dbg_close(
	int	port
)
{

}

void    mips_flush_cache ();
void    mips_flush_dcache ();
void    mips_clean_cache ();

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	4/8/02
 *
 *-----------------------------------------------------------------
 */
void
flush_cache (type, adr)
     int             type;
     address_t       adr;
{

	switch (type) {
		case ICACHE:
#if defined(R3000)
				r3k_iflush ();
#elif defined(R4000)
				mips_flush_cache ();    /* flush (and writeback) everything! */
#endif
				break;

		case DCACHE:
#if defined(R3000)
				r3k_dflush ();
#elif defined(R4000)
				mips_flush_dcache ();
#endif
				break;

		case IADDR:
#if defined(R3000)
				r3k_iaflush (adr);
#elif defined(R4000)
				mips_clean_cache (adr, 1);
#endif
				break;
	}
}

void
store_dword (adr, v)
	address_t			adr;
	unsigned long long	v;
{
	*(unsigned long long *) adr = v;
	flush_cache (IADDR, adr);
}

void
store_word (adr, v)
	address_t		adr;
	unsigned long	v;
{
	*(unsigned long *) adr = v;
	flush_cache (IADDR, adr);
}

void
store_int (adr, v)
	address_t	adr;
	int			v;
{
	*(int *) adr = v;
	flush_cache (IADDR, adr);
}

void
store_half (adr, v)
	address_t	adr;
	ushort16	v;
{
	*(ushort16 *) adr = v;
	flush_cache (IADDR, adr);
}

void
store_byte (adr, v)
	address_t	adr;
	byte		v;
{
	*(byte *)adr = v;
	flush_cache (IADDR, adr);
}

#endif
