/*
 *-----------------------------------------------------------------
 *
 * Name:	exc-llhandler.S
 *
 * RCS:  $Id: exc-llhandler.S,v 1.24 2002/12/03 22:00:11 maximk Exp $
 *
 * Copyright (C) 2002, Agile Storage, Inc.
 *
 * Description:	Low Level exception handlers, jumping to real routines
 *
 * Created by:	Rick Lund
 *
 * Date Created:	3/17/02
 *
 *-----------------------------------------------------------------
 */

#include <nfx-incl.h>
#include <sys/mips.h>
#ifdef SIBYTE
#include <exports/pte-api.h>
#endif
#include "except-api.h"
#include "except.h"

#define EXCEPT_NEW_STACK

	.set noreorder
	.set noat

	.comm	xtlb_refill_count,4
	.comm	xtlb_refill_start,4
	.comm	xtlb_refill_time,4

	.text

#if defined(R4000)
LEAF(_tlb_refill_exception)
	la	k0,tlb_refill_exception
	jr	k0
	nop
END(_tlb_refill_exception)
EXPORT(_tlb_refill_end)

LEAF(_xtlb_refill_exception)
	la	k0,xtlb_refill_exception
	jr	k0
	nop
END(_xtlb_refill_exception)
EXPORT(_xtlb_refill_end)

LEAF(_cache_error_exception)
	la	k0,cache_error_exception
	jr	k0
	nop
END(_cache_error_exception)
EXPORT(_cache_error_end)

LEAF(_general_exception)
	la	k0,general_exception
	jr	k0
	nop
END(_general_exception)
EXPORT(_general_end)

LEAF(_interrupt_exception)
	la	k0,intr_entry
	jr	k0
	nop
END(_interrupt_exception)
EXPORT(_interrupt_end)

#elif defined(TX3927)

LEAF(_utlb_refill_exception)
	la	k0,utlb_refill_exception
	jr	k0
	nop
END(_utlb_refill_exception)
EXPORT(_utlb_refill_end)

LEAF(_general_exception)
	la	k0,general_exception
	jr	k0
	nop
END(_general_exception)
EXPORT(_general_end)

#endif

	.set at
	.set reorder

#if defined(R4000)
/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	4/4/02
 *
 *-----------------------------------------------------------------
 */
LEAF(tlb_refill_exception)

#ifdef EXCEPTIONS_IN_KSEG0

	j	general_exception

#else

	la	k0,0xbfc00200
	jr	k0

#endif	/* EXCEPTIONS_IN_KSEG0 */

END(tlb_refill_exception)

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	4/4/02
 *
 *-----------------------------------------------------------------
 */
LEAF(xtlb_refill_exception)

#ifdef EXCEPTIONS_IN_KSEG0

#ifdef LARGE_MEMORY
	.set	noat
	.set	noreorder

	la		k0,xtlb_refill_count
	lw		k1,0(k0)
	addiu	k1,k1,1
	sw		k1,0(k0)

	la		k0,xtlb_refill_start
	mfc0	k1,C0_COUNT
	sw		k1,0(k0)

	dmfc0	k0,C0_XCTEXT
	dsrl	k0,k0,0x1b
	andi	k0,k0,0xf		# k0 = region (bits 39:36)
#if (PTE_MAX_MEMORY <= 0x100000000)
	beqz	k0,xtlb_refill_error
#endif

#ifdef USE_THREAD_STACKS
	nop
	li		k1,5
	bne		k0,k1,xtlb_refill_doit	# thread stacks at 0x5000000000
	nop

	dmfc0	k1,C0_XCTEXT			# verify bits [2x:1x] are 0 (see pte-api.h)
	dsrl	k1,k1,PTE_THREAD_NUM_XCTEXT_SHIFT 
	and		k1,k1,PTE_THREAD_CHECK_XCTEXT_MASK
	beqz	k1,xtlb_refill_doit
	nop
	b		xtlb_refill_error
	nop

xtlb_refill_doit:
#endif

#ifdef TLB_VERIFY_4_GB_REGION_MAX
	dmfc0	k1,C0_XCTEXT
	dsrl	k1,k1,0x17
	and		k1,k1,0xf				# k1 = bits 35:32 - verify these are 0
	bnez	k1,xtlb_refill_error	# 4 GB max/region (see pte-api.h)
	nop
#endif

	dsll	k0,k0,3
	la		k1,mmu_tptr
	daddu	k0,k1,k0
	ld		k0,0(k0)		# k0 = mmu_tptr[region]

	dmfc0	k1,C0_XCTEXT
	dsrl	k1,k1,PTE_ENTRY_XCTEXT_SHIFT
	and		k1,k1,PTE_ENTRY_XCTEXT_MASK	# k1 = index

	dsll	k1,k1,5
	daddu	k0,k0,k1		# k0 = tptr[index]

	ld		k1,0(k0)
	dmtc0	k1,C0_TLBLO0
	ld		k1,8(k0)
	lw		k0,16(k0)
	dmtc0	k1,C0_TLBLO1
	mtc0	k0,C0_PGMASK
	tlbwr
	nop

	mfc0	k1,C0_COUNT
	la		k0,xtlb_refill_start
	lw		k0,0(k0)
	subu	k1,k0
	la		k0,xtlb_refill_time
	lw		k0,0(k0)
	addu	k0,k1
	la		k1,xtlb_refill_time
	sw		k0,0(k1)

	eret
	nop

xtlb_refill_error:
	j	general_exception
	nop

	.set	at
	.set	reorder

#else
	j	general_exception
#endif

#else

	la	k0,0xbfc00200
	jr	k0

#endif	/* EXCEPTIONS_IN_KSEG0 */

END(xtlb_refill_exception)

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	4/4/02
 *
 *-----------------------------------------------------------------
 */
LEAF(cache_error_exception)

#ifdef EXCEPTIONS_IN_KSEG0

#ifdef SIBYTE
	# look for signature of spurious CErr
	mfc0	k1,$26,0
	lui		k0,0x4000
	bne		k0,k1,real_cerr
	mfc0	k1,$27,1
	lui		k0,0xffe0
	and		k1,k0,k1
	lui		k0,0x0200
	bne		k0,k1,real_cerr
	# clear/unlock the registers
	mtc0	zero,$26,0
	mtc0	zero,$27,1

	lui	k0,0x8070
	lw	k1,0(k0)
	add	k1,1
	sw	k1,0(k0)

	eret
real_cerr:
#endif

	j	general_exception

#else

	la	k0,0xbfc00300
	jr	k0

#endif	/* EXCEPTIONS_IN_KSEG0 */

END(cache_error_exception)

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	3/19/02
 *
 *-----------------------------------------------------------------
 */
LEAF(general_exception)
	.set noat

#ifdef EXCEPTIONS_IN_KSEG0

	la		k0,EXC_DBGREG
	SREG(k1,R_K1TMP,k0)

#if 0
/*
 * RAL - not supported yet
 */
	# see if we have a user defined handler
	.set noreorder
	mfc0	k0,C0_CAUSE
	.set reorder
	la		k1,hndlrtbl
	and		k0,CAUSE_EXCMASK
	addu    k1,k0		# calc table entry addr
	move    k0,zero
	lw		k1,(k1)		# get contents of table entry
	beq		k1,zero,no_handler
	lw		k0,4(k1)	# user routine addr
	la		k1,EXC_DBGREG
	LREG(k1,R_K1TMP,k1)	# restore k1
	.set noreorder
	j		k0			# jump to user handler
	move	k0,zero
#endif
	.set reorder

no_handler:
	la	k1,EXC_DBGREG
	SREG(t0,R_T0,k1)		/* $8  (t0) */
	move	t0,k1			# now use t0 as pointer
	LREG(k1,R_K1TMP,t0)
	SREG(AT,R_AT,t0)		/* $1  (AT) */
	.set at
	SREG(v0,R_V0,t0)		/* $2  (v0) */
	SREG(v1,R_V1,t0)		/* $3  (v1) */
	SREG(a0,R_A0,t0)		/* $4  (a0) */
	SREG(a1,R_A1,t0)		/* $5  (a1) */
	SREG(a2,R_A2,t0)		/* $6  (a2) */
	SREG(a3,R_A3,t0)		/* $7  (a3) */
	SREG(t1,R_T1,t0)		/* $9  (t1) */
	SREG(t2,R_T2,t0)		/* $10 (t2) */
	SREG(t3,R_T3,t0)		/* $11 (t3) */
	SREG(t4,R_T4,t0)		/* $12 (t4) */
	SREG(t5,R_T5,t0)		/* $13 (t5) */
	SREG(t6,R_T6,t0)		/* $14 (t6) */
	SREG(t7,R_T7,t0)		/* $15 (t7) */
	SREG(s0,R_S0,t0)		/* $16 (s0) */
	SREG(s1,R_S1,t0)		/* $17 (s1) */
	SREG(s2,R_S2,t0)		/* $18 (s2) */
	SREG(s3,R_S3,t0)		/* $19 (s3) */
	SREG(s4,R_S4,t0)		/* $20 (s4) */
	SREG(s5,R_S5,t0)		/* $21 (s5) */
	SREG(s6,R_S6,t0)		/* $22 (s6) */
	SREG(s7,R_S7,t0)		/* $23 (s7) */
	SREG(t8,R_T8,t0)		/* $24 (t8) */
	SREG(t9,R_T9,t0)		/* $25 (t9) */
					/* $26 (k0) */
	SREG(k1,R_K1,t0)		/* $27 (k1) */
	SREG(gp,R_GP,t0)		/* $28 (gp) */
	SREG(sp,R_SP,t0)		/* $29 (sp) */
	SREG(s8,R_FP,t0)		/* $30 (s8) */
	SREG(ra,R_RA,t0)		/* $31 (ra) */

	mfhi	t1
	mflo	t2
	SREG(t1,R_HI,t0)		/* Hi */
	SREG(t2,R_LO,t0)		/* Lo */

	.set noreorder

#ifdef R7000
	mfc0	t1,C0_INX
	SREG(t1,R_INDEX,t0)			/* Index */
	mfc0	t1,C0_RAND
	SREG(t1,R_RANDOM,t0)		/* Random */
	MFC0	t1,C0_TLBLO
	SREG(t1,R_ENTRYLO,t0)		/* TLB EntryLo */
	MFC0	t1,C0_TLBLO1
	SREG(t1,R_ENTRYLO1,t0)		/* TLB EntryLo1 */
	MFC0	t1,C0_CTEXT
	SREG(t1,R_CONTEXT,t0)		/* Context */
	mfc0	t1,C0_PGMASK
	SREG(t1,R_PGMASK,t0)		/* TLB PageMask */
	mfc0	t1,C0_WIRED
	SREG(t1,R_WIRED,t0)			/* TLB Wired */
	MFC0	t1,C0_BADVADDR
	SREG(t1,R_BADVA,t0)			/* BadVA */
	MFC0	t1,C0_TLBHI
	SREG(t1,R_ENTRYHI,t0)		/* TLB EntryHi */
	mfc0	t1,C0_PRID
	SREG(t1,R_PRID,t0)			/* Prid */
	mfc0	t1,C0_CONFIG
	SREG(t1,R_CONFIG,t0)		/* Config */
	mfc0	t1,C0_LLADDR
	SREG(t1,R_LLADDR,t0)		/* LLAddr */
	MFC0	t1,C0_WATCHLO
	SREG(t1,R_WATCHLO,t0)		/* WatchLo */
	MFC0	t1,C0_WATCHHI
	SREG(t1,R_WATCHHI,t0)		/* WatchHi */
	MFC0	t1,C0_XCTEXT
	SREG(t1,R_XCONTEXT,t0)		/* XContext */
	mfc0	t1,C0_WATCHMASK
	SREG(t1,R_WATCHMASK,t0)		/* WatchLo */
	mfc0	t1,C0_ECC
	SREG(t1,R_ECC,t0)			/* ECC */
	mfc0	t1,C0_CACHEERR
	SREG(t1,R_CACHERR,t0)		/* CacheErr */
	mfc0	t1,C0_TAGLO
	SREG(t1,R_TAGLO,t0)			/* Taglo */
	mfc0	t1,C0_TAGHI
	SREG(t1,R_TAGHI,t0)			/* TagHi */
	MFC0	t1,C0_ERREPC
	SREG(t1,R_ERREPC,t0)		/* ErrorEPC */

	cfc0	t1,C0C_ICR
	SREG(t1,R_ICR,t0)			/* ICR */
	cfc0	t1,C0C_IPLLO
	SREG(t1,R_IPLLO,t0)			/* IPLLo */
	cfc0	t1,C0C_IPLHI
	SREG(t1,R_IPLHI,t0)			/* IPLHi */

	MTC0	zero,C0_WATCHLO
	MTC0	zero,C0_WATCHHI
	ctc0	zero,C0C_ICR
	ctc0	zero,C0C_IPLLO
	ctc0	zero,C0C_IPLHI
#endif

#ifdef SB1250
	mfc0	t1,C0_INX
	SREG(t1,R_INDEX,t0)			/* Index */
	mfc0	t1,C0_RAND
	SREG(t1,R_RANDOM,t0)		/* Random */
	MFC0	t1,C0_TLBLO
	SREG(t1,R_ENTRYLO,t0)		/* EntryLo */
	MFC0	t1,C0_TLBLO1
	SREG(t1,R_ENTRYLO1,t0)		/* TLB EntryLo1 */
	MFC0	t1,C0_CTEXT
	SREG(t1,R_CONTEXT,t0)		/* Context */
	mfc0	t1,C0_PGMASK
	SREG(t1,R_PGMASK,t0)		/* TLB PageMask */
	mfc0	t1,C0_WIRED
	SREG(t1,R_WIRED,t0)			/* TLB Wired */
	MFC0	t1,C0_BADVADDR
	SREG(t1,R_BADVA,t0)			/* BadVA */
	MFC0	t1,C0_TLBHI
	SREG(t1,R_ENTRYHI,t0)		/* EntryHi */
	mfc0	t1,C0_PRID
	SREG(t1,R_PRID,t0)			/* Prid */
	mfc0	t1,C0_CONFIG
	SREG(t1,R_CONFIG,t0)		/* Config */
	MFC0	t1,C0_LLADDR
	SREG(t1,R_LLADDR,t0)		/* LLAddr */
	MFC0	t1,C0_WATCHLO
	SREG(t1,R_WATCHLO,t0)		/* WatchLo */
	MFC0	t1,C0_WATCHHI
	SREG(t1,R_WATCHHI,t0)		/* WatchHi */
	MFC0	t1,C0_XCTEXT
	SREG(t1,R_XCONTEXT,t0)		/* XContext */
	mfc0	t1,C0_ECC
	SREG(t1,R_ECC,t0)			/* ECC */
	mfc0	t1,C0_CACHEERR
	SREG(t1,R_CACHERR,t0)		/* CacheErr */
	mfc0	t1,C0_TAGLO
	SREG(t1,R_TAGLO,t0)			/* Taglo */
	mfc0	t1,C0_TAGHI
	SREG(t1,R_TAGHI,t0)			/* TagHi */
	MFC0	t1,C0_ERREPC
	SREG(t1,R_ERREPC,t0)		/* ErrorEPC */

	MTC0	zero,C0_WATCHLO
	MTC0	zero,C0_WATCHHI
#endif

	mfc0	a0,C0_CAUSE
	MFC0	a1,C0_EPC
	mfc0	t3,C0_SR
	SREG(a0,R_CAUSE,t0)			/* Cause */
	SREG(a1,R_EPC,t0)			/* EPC */
	mtc0	zero,C0_CAUSE		/* Clear pending watchpoint */
	
#ifdef R4000
	# save the SR value but clear EXL bit to
	# reflect state before the exception
	and		t3,~SR_EXL		# switch off EXL bit
#else
	# save the SR value but fixup the KU & IE bits to 
	# reflect their state before the exception
	li		t1,0x3f			# KU&IE mask
	and		t2,t3,t1		# extract KU&IE bits
	not		t1				# invert the mask
	and		t3,t1			# clear KU&IE field
	srl		t2,2			# fixup KU&IE field
	or		t3,t2			# insert the KU&IE field
#endif
	SREG(t3,R_STATUS,t0)	# save Status


	# switch us back to the correct kernel/interrupt mode, but with
	# all external interrupts masked out.
#	lw		t3,initial_sr
	and		t3,~SR_IMASK
	mtc0	t3,C0_SR
	.set reorder

	move	a2,ra
#ifdef EXCEPT_NEW_STACK
	la		sp,exc_stack+0x2000
#else
subu	sp,128
#endif
	la	t0,exception
	j	t0

#else

	la	k0,0xbfc00380
	jr	k0

#endif	/* EXCEPTIONS_IN_KSEG0 */

END(general_exception)

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	3/19/02
 *
 *-----------------------------------------------------------------
 */
LEAF(exception_return)

	la	gp,EXC_DBGREG

	LREG(v0,R_V0,gp)		/* $2  (v0) */
	LREG(v1,R_V1,gp)		/* $3  (v1) */
	LREG(a0,R_A0,gp)		/* $4  (a0) */
	LREG(a1,R_A1,gp)		/* $5  (a1) */
	LREG(a2,R_A2,gp)		/* $6  (a2) */
	LREG(a3,R_A3,gp)		/* $7  (a3) */

	LREG(t3,R_T3,gp)		/* $11 (t3) */
	LREG(t4,R_T4,gp)		/* $12 (t4) */
	LREG(t5,R_T5,gp)		/* $13 (t5) */
	LREG(t6,R_T6,gp)		/* $14 (t6) */
	LREG(t7,R_T7,gp)		/* $15 (t7) */
	LREG(s0,R_S0,gp)		/* $16 (s0) */
	LREG(s1,R_S1,gp)		/* $17 (s1) */
	LREG(s2,R_S2,gp)		/* $18 (s2) */
	LREG(s3,R_S3,gp)		/* $19 (s3) */
	LREG(s4,R_S4,gp)		/* $20 (s4) */
	LREG(s5,R_S5,gp)		/* $21 (s5) */
	LREG(s6,R_S6,gp)		/* $22 (s6) */
	LREG(s7,R_S7,gp)		/* $23 (s7) */
	LREG(t8,R_T8,gp)		/* $24 (t8) */
	LREG(t9,R_T9,gp)		/* $25 (t9) */

	LREG(sp,R_SP,gp)		/* $29 (sp) */
	LREG(s8,R_FP,gp)		/* $30 (s8) */
	LREG(ra,R_RA,gp)		/* $31 (ra) */

	LREG(t0,R_HI,gp)		/* Hi */
	LREG(t1,R_LO,gp)		/* Lo */
	mthi	t0
	mtlo	t1

	.set	noat
	.set	noreorder

#ifdef R4000
	mtc0	zero,C0_SR		# set IMASK & IE = 0 before setting EXL
	LREG(t0,R_STATUS,gp)	# get Status
	or		t0,SR_EXL		# set EXL
	nop
#else
	mtc0	zero,C0_SR		# clear IEC (and everything else)
	# interrupts now off (k0 & k1 safe)
	LREG(t1,R_STATUS,gp)	# get Status
	li		AT,0x3f			# KU&IE mask
	and		t0,t1,AT		# get KU&IE field
    not		AT				# invert mask
	and		t1,AT			# clear KU&IE field
	sll		t0,2			# fixup KU&IE field
	or		t0,t1			# insert KU&IE field
#endif
	mtc0	t0,C0_SR

#ifdef R7000
	LREG(t1,R_ICR,gp)
	LREG(AT,R_WATCHMASK,gp)
	ctc0	t1,C0C_ICR
	LREG(t1,R_IPLLO,gp)
	LREG(AT,R_IPLHI,gp)
	ctc0	t1,C0C_IPLLO
	ctc0	AT,C0C_IPLHI

    LREG(t0,R_ENTRYHI,gp)       /* EntryHi */
    LREG(t1,R_CONTEXT,gp)       /* Context */
    LREG(AT,R_INDEX,gp)     /* Index */
    MTC0    t0,C0_TLBHI
    MTC0    t1,C0_CTEXT
    mtc0    AT,C0_INX
    LREG(t0,R_ENTRYLO,gp)       /* EntryLo */
    LREG(t1,R_XCONTEXT,gp)      /* XContext */
    MTC0    t0,C0_TLBLO
    MTC0    t1,C0_XCTEXT

	LREG(t0,R_WATCHLO,gp)       /* WatchpointLo */
	LREG(t1,R_WATCHHI,gp)       /* WatchpointHi */
	MTC0    t0,C0_WATCHLO
	MTC0    t1,C0_WATCHHI

	LREG(t0,R_ENTRYLO1,gp)      /* TLB EntryLo1 */
	LREG(t1,R_PGMASK,gp)        /* TLB PageMask */
	LREG(AT,R_WIRED,gp)     /* TLB Wired */
	MTC0    t0,C0_TLBLO1
	mtc0    t1,C0_PGMASK
	mtc0    AT,C0_WIRED
	LREG(t0,R_ECC,gp)       /* ECC */
	LREG(t1,R_TAGLO,gp)     /* TagLo */
	LREG(AT,R_TAGHI,gp)     /* TagHi */
	mtc0    t0,C0_ECC
	mtc0    t1,C0_TAGLO
	mtc0    AT,C0_TAGHI

	LREG(t2,R_EPC,gp)       /* EPC */
	LREG(AT,R_CAUSE,gp)     /* Cause - For Sw field only */
	MTC0    t2,C0_EPC
	mtc0    AT,C0_CAUSE
	LREG(AT,R_AT,gp)        /* $1  (AT) */
	LREG(t0,R_T0,gp)        /* $8  (t0) */
	LREG(t1,R_T1,gp)        /* $9  (t1) */
	LREG(t2,R_T2,gp)        /* $10 (t2) */
	LREG(k1,R_K1,gp)        /* $27 (k1) */
	LREG(gp,R_GP,gp)        /* $28 (gp) */

	eret
	nop

#endif

#ifdef SB1250
	LREG(t0,R_ENTRYHI,gp)       /* EntryHi */
	LREG(t1,R_CONTEXT,gp)       /* Context */
	LREG(AT,R_INDEX,gp)     /* Index */
	MTC0    t0,C0_TLBHI
	MTC0    t1,C0_CTEXT
	mtc0    AT,C0_INX
	LREG(t0,R_ENTRYLO,gp)       /* EntryLo */
	LREG(t1,R_XCONTEXT,gp)      /* XContext */
	MTC0    t0,C0_TLBLO
	MTC0    t1,C0_XCTEXT

	LREG(t0,R_WATCHLO,gp)       /* WatchpointLo */
	LREG(t1,R_WATCHHI,gp)       /* WatchpointHi */
	MTC0    t0,C0_WATCHLO
	MTC0    t1,C0_WATCHHI

	LREG(t0,R_ENTRYLO1,gp)      /* TLB EntryLo1 */
	LREG(t1,R_PGMASK,gp)        /* TLB PageMask */
	LREG(AT,R_WIRED,gp)     /* TLB Wired */
	MTC0    t0,C0_TLBLO1
	mtc0    t1,C0_PGMASK
	mtc0    AT,C0_WIRED
	LREG(t0,R_ECC,gp)       /* ECC */
	LREG(t1,R_TAGLO,gp)     /* TagLo */
	LREG(AT,R_TAGHI,gp)     /* TagHi */
	mtc0    t0,C0_ECC
	mtc0    t1,C0_TAGLO
	mtc0    AT,C0_TAGHI

	LREG(t2,R_EPC,gp)       /* EPC */
	LREG(AT,R_CAUSE,gp)     /* Cause - For Sw field only */
	MTC0    t2,C0_EPC
	mtc0    AT,C0_CAUSE
	LREG(AT,R_AT,gp)        /* $1  (AT) */
	LREG(t0,R_T0,gp)        /* $8  (t0) */
	LREG(t1,R_T1,gp)        /* $9  (t1) */
	LREG(t2,R_T2,gp)        /* $10 (t2) */
	LREG(k1,R_K1,gp)        /* $27 (k1) */
	LREG(gp,R_GP,gp)        /* $28 (gp) */

	eret
	nop
#endif

END(exception_return)

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	5/6/02
 *
 *-----------------------------------------------------------------
 */
LEAF(panic)
	.set noat

#ifdef EXCEPTIONS_IN_KSEG0

	la		k0,EXC_DBGREG
	SREG(k1,R_K1TMP,k0)

	.set reorder

	la	k1,EXC_DBGREG
	SREG(t0,R_T0,k1)		/* $8  (t0) */
	move	t0,k1			# now use t0 as pointer
	LREG(k1,R_K1TMP,t0)
	SREG(AT,R_AT,t0)		/* $1  (AT) */
	.set at
	SREG(v0,R_V0,t0)		/* $2  (v0) */
	SREG(v1,R_V1,t0)		/* $3  (v1) */
	SREG(a0,R_A0,t0)		/* $4  (a0) */
	SREG(a1,R_A1,t0)		/* $5  (a1) */
	SREG(a2,R_A2,t0)		/* $6  (a2) */
	SREG(a3,R_A3,t0)		/* $7  (a3) */
	SREG(t1,R_T1,t0)		/* $9  (t1) */
	SREG(t2,R_T2,t0)		/* $10 (t2) */
	SREG(t3,R_T3,t0)		/* $11 (t3) */
	SREG(t4,R_T4,t0)		/* $12 (t4) */
	SREG(t5,R_T5,t0)		/* $13 (t5) */
	SREG(t6,R_T6,t0)		/* $14 (t6) */
	SREG(t7,R_T7,t0)		/* $15 (t7) */
	SREG(s0,R_S0,t0)		/* $16 (s0) */
	SREG(s1,R_S1,t0)		/* $17 (s1) */
	SREG(s2,R_S2,t0)		/* $18 (s2) */
	SREG(s3,R_S3,t0)		/* $19 (s3) */
	SREG(s4,R_S4,t0)		/* $20 (s4) */
	SREG(s5,R_S5,t0)		/* $21 (s5) */
	SREG(s6,R_S6,t0)		/* $22 (s6) */
	SREG(s7,R_S7,t0)		/* $23 (s7) */
	SREG(t8,R_T8,t0)		/* $24 (t8) */
	SREG(t9,R_T9,t0)		/* $25 (t9) */
					/* $26 (k0) */
	SREG(k1,R_K1,t0)		/* $27 (k1) */
	SREG(gp,R_GP,t0)		/* $28 (gp) */
	SREG(sp,R_SP,t0)		/* $29 (sp) */
	SREG(s8,R_FP,t0)		/* $30 (s8) */
	SREG(ra,R_RA,t0)		/* $31 (ra) */

	mfhi	t1
	mflo	t2
	SREG(t1,R_HI,t0)		/* Hi */
	SREG(t2,R_LO,t0)		/* Lo */

	.set noreorder

#ifdef R7000
	mfc0	t1,C0_INX
	SREG(t1,R_INDEX,t0)			/* Index */
	mfc0	t1,C0_RAND
	SREG(t1,R_RANDOM,t0)		/* Random */
	MFC0	t1,C0_TLBLO
	SREG(t1,R_ENTRYLO,t0)		/* TLB EntryLo */
	MFC0	t1,C0_TLBLO1
	SREG(t1,R_ENTRYLO1,t0)		/* TLB EntryLo1 */
	MFC0	t1,C0_CTEXT
	SREG(t1,R_CONTEXT,t0)		/* Context */
	mfc0	t1,C0_PGMASK
	SREG(t1,R_PGMASK,t0)		/* TLB PageMask */
	mfc0	t1,C0_WIRED
	SREG(t1,R_WIRED,t0)			/* TLB Wired */
	MFC0	t1,C0_BADVADDR
	SREG(t1,R_BADVA,t0)			/* BadVA */
	MFC0	t1,C0_TLBHI
	SREG(t1,R_ENTRYHI,t0)		/* TLB EntryHi */
	mfc0	t1,C0_PRID
	SREG(t1,R_PRID,t0)			/* Prid */
	mfc0	t1,C0_CONFIG
	SREG(t1,R_CONFIG,t0)		/* Config */
	mfc0	t1,C0_LLADDR
	SREG(t1,R_LLADDR,t0)		/* LLAddr */
	MFC0	t1,C0_WATCHLO
	SREG(t1,R_WATCHLO,t0)		/* WatchLo */
	MFC0	t1,C0_WATCHHI
	SREG(t1,R_WATCHHI,t0)		/* WatchHi */
	MFC0	t1,C0_XCTEXT
	SREG(t1,R_XCONTEXT,t0)		/* XContext */
	mfc0	t1,C0_WATCHMASK
	SREG(t1,R_WATCHMASK,t0)		/* WatchLo */
	mfc0	t1,C0_ECC
	SREG(t1,R_ECC,t0)			/* ECC */
	mfc0	t1,C0_CACHEERR
	SREG(t1,R_CACHERR,t0)		/* CacheErr */
	mfc0	t1,C0_TAGLO
	SREG(t1,R_TAGLO,t0)			/* Taglo */
	mfc0	t1,C0_TAGHI
	SREG(t1,R_TAGHI,t0)			/* TagHi */
	MFC0	t1,C0_ERREPC
	SREG(t1,R_ERREPC,t0)		/* ErrorEPC */

	cfc0	t1,C0C_ICR
	SREG(t1,R_ICR,t0)			/* ICR */
	cfc0	t1,C0C_IPLLO
	SREG(t1,R_IPLLO,t0)			/* IPLLo */
	cfc0	t1,C0C_IPLHI
	SREG(t1,R_IPLHI,t0)			/* IPLHi */

#endif

#ifdef SB1250
	mfc0	t1,C0_INX
	SREG(t1,R_INDEX,t0)			/* Index */
	mfc0	t1,C0_RAND
	SREG(t1,R_RANDOM,t0)		/* Random */
	MFC0	t1,C0_TLBLO
	SREG(t1,R_ENTRYLO,t0)		/* EntryLo */
	MFC0	t1,C0_TLBLO1
	SREG(t1,R_ENTRYLO1,t0)		/* TLB EntryLo1 */
	MFC0	t1,C0_CTEXT
	SREG(t1,R_CONTEXT,t0)		/* Context */
	mfc0	t1,C0_PGMASK
	SREG(t1,R_PGMASK,t0)		/* TLB PageMask */
	mfc0	t1,C0_WIRED
	SREG(t1,R_WIRED,t0)			/* TLB Wired */
	MFC0	t1,C0_BADVADDR
	SREG(t1,R_BADVA,t0)			/* BadVA */
	MFC0	t1,C0_TLBHI
	SREG(t1,R_ENTRYHI,t0)		/* EntryHi */
	mfc0	t1,C0_PRID
	SREG(t1,R_PRID,t0)			/* Prid */
	mfc0	t1,C0_CONFIG
	SREG(t1,R_CONFIG,t0)		/* Config */
	MFC0	t1,C0_LLADDR
	SREG(t1,R_LLADDR,t0)		/* LLAddr */
	MFC0	t1,C0_WATCHLO
	SREG(t1,R_WATCHLO,t0)		/* WatchLo */
	MFC0	t1,C0_WATCHHI
	SREG(t1,R_WATCHHI,t0)		/* WatchHi */
	MFC0	t1,C0_XCTEXT
	SREG(t1,R_XCONTEXT,t0)		/* XContext */
	mfc0	t1,C0_ECC
	SREG(t1,R_ECC,t0)			/* ECC */
	mfc0	t1,C0_CACHEERR
	SREG(t1,R_CACHERR,t0)		/* CacheErr */
	mfc0	t1,C0_TAGLO
	SREG(t1,R_TAGLO,t0)			/* Taglo */
	mfc0	t1,C0_TAGHI
	SREG(t1,R_TAGHI,t0)			/* TagHi */
	MFC0	t1,C0_ERREPC
	SREG(t1,R_ERREPC,t0)		/* ErrorEPC */

#endif

	mfc0	t1,C0_CAUSE
	move	t2,ra
	mfc0	t3,C0_SR
	SREG(t1,R_CAUSE,t0)			/* Cause */
	SREG(t2,R_EPC,t0)			/* EPC */
	SREG(t3,R_STATUS,t0)	# save Status


	.set reorder

	la	t0,panic_service		/* panic_service(panic_str) */
	jr	t0

#else

	la	t0,panic_service
	jr	t0

#endif	/* EXCEPTIONS_IN_KSEG0 */

END(panic)

#elif defined(TX3927)

	.set noreorder

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	4/4/02
 *
 *-----------------------------------------------------------------
 */
LEAF(utlb_refill_exception)
	j	general_exception
END(utlb_refill_exception)

LEAF(general_exception)
	.set	noat

	mfc0	k0,C0_CAUSE
	nop
	nop
	nop
	and		k0,CAUSE_EXCMASK
	beq		k0,CEXC_INT,intr_entry
	nop

	la	k1,EXC_DBGREG
	SREG(t0,R_T0,k1)		/* $8  (t0) */
	move	t0,k1			# now use t0 as pointer
	LREG(k1,R_K1TMP,t0)
	SREG(AT,R_AT,t0)		/* $1  (AT) */
	.set at
	SREG(v0,R_V0,t0)		/* $2  (v0) */
	SREG(v1,R_V1,t0)		/* $3  (v1) */
	SREG(a0,R_A0,t0)		/* $4  (a0) */
	SREG(a1,R_A1,t0)		/* $5  (a1) */
	SREG(a2,R_A2,t0)		/* $6  (a2) */
	SREG(a3,R_A3,t0)		/* $7  (a3) */
	SREG(t1,R_T1,t0)		/* $9  (t1) */
	SREG(t2,R_T2,t0)		/* $10 (t2) */
	SREG(t3,R_T3,t0)		/* $11 (t3) */
	SREG(t4,R_T4,t0)		/* $12 (t4) */
	SREG(t5,R_T5,t0)		/* $13 (t5) */
	SREG(t6,R_T6,t0)		/* $14 (t6) */
	SREG(t7,R_T7,t0)		/* $15 (t7) */
	SREG(s0,R_S0,t0)		/* $16 (s0) */
	SREG(s1,R_S1,t0)		/* $17 (s1) */
	SREG(s2,R_S2,t0)		/* $18 (s2) */
	SREG(s3,R_S3,t0)		/* $19 (s3) */
	SREG(s4,R_S4,t0)		/* $20 (s4) */
	SREG(s5,R_S5,t0)		/* $21 (s5) */
	SREG(s6,R_S6,t0)		/* $22 (s6) */
	SREG(s7,R_S7,t0)		/* $23 (s7) */
	SREG(t8,R_T8,t0)		/* $24 (t8) */
	SREG(t9,R_T9,t0)		/* $25 (t9) */
					/* $26 (k0) */
	SREG(k1,R_K1,t0)		/* $27 (k1) */
	SREG(gp,R_GP,t0)		/* $28 (gp) */
	SREG(sp,R_SP,t0)		/* $29 (sp) */
	SREG(s8,R_FP,t0)		/* $30 (s8) */
	SREG(ra,R_RA,t0)		/* $31 (ra) */

	mfhi	t1
	mflo	t2
	SREG(t1,R_HI,t0)		/* Hi */
	SREG(t2,R_LO,t0)		/* Lo */

	
	mfc0	t1,C0_INX
	SREG(t1,R_INDEX,t0)			/* Index */
	mfc0	t1,C0_RAND
	SREG(t1,R_RANDOM,t0)		/* Random */
	mfc0	t1,C0_TLBLO
	SREG(t1,R_ENTRYLO,t0)		/* EntryLo */
	mfc0	t1,C0_CONFIG
	SREG(t1,R_CONFIG,t0)		/* Config */
	MFC0	t1,C0_CTEXT
	SREG(t1,R_CONTEXT,t0)		/* Context */
	mfc0	t1,C0_PGMASK
	SREG(t1,R_PGMASK,t0)		/* TLB PageMask */
	mfc0	t1,C0_WIRED
	SREG(t1,R_WIRED,t0)			/* TLB Wired */
	mfc0	t1,C0_CACHE
	SREG(t1,R_CACHE,t0)			/* Cache */
	MFC0	t1,C0_BADVADDR
	SREG(t1,R_BADVA,t0)			/* BadVA */
	mfc0	t1,C0_TLBHI
	SREG(t1,R_ENTRYHI,t0)		/* EntryHi */
	mfc0	t1,C0_STATUS
	SREG(t1,R_STATUS,t0)		/* Status */
	mfc0	t1,C0_CAUSE
	SREG(t1,R_CAUSE,t0)			/* Cause */
	mfc0	t1,C0_EPC
	SREG(t1,R_EPC,t0)			/* EPC */
	mfc0	t1,C0_PRID
	SREG(t1,R_PRID,t0)			/* Prid */
	mfc0	t1,C0_DEBUG
	SREG(t1,R_DEBUG,t0)			/* Debug */
	mfc0	t1,C0_DEPC
	SREG(t1,R_DEPC,t0)			/* DEPC */
	mfc0	t1,C0_TAGLO
	SREG(t1,R_TAGLO,t0)			/* Taglo */

	j		exception_tx3927

END(general_exception)

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	4/4/02
 *
 *-----------------------------------------------------------------
 */
LEAF(exception_return)
	.set	noat

	mfc0	k0,C0_EPC
	nop
	nop
	nop
	nop
	j		k0
	rfe
	nop

#ifdef TX3927

	LREG(t1,R_CAUSE,gp)     /* Cause - For Sw field only */
	LREG(t0,R_T0,gp)        /* $8  (t0) */
	LREG(t2,R_T2,gp)        /* $10 (t2) */
	mtc0    t1,C0_CAUSE
	LREG(AT,R_AT,gp)        /* $1  (AT) */
	LREG(t1,R_T1,gp)        /* $8  (t0) */
	LREG(k0,R_EPC,gp)       /* EPC */
	LREG(k1,R_K1,gp)        /* $27 (k1) */
	LREG(gp,R_GP,gp)        /* $28 (gp) */

	j	k0
	rfe

#endif

END(exception_return)

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	5/6/02
 *
 *-----------------------------------------------------------------
 */
LEAF(panic)
	.set	noat

	la	k1,EXC_DBGREG
	SREG(t0,R_T0,k1)		/* $8  (t0) */
	move	t0,k1			# now use t0 as pointer
	LREG(k1,R_K1TMP,t0)
	SREG(AT,R_AT,t0)		/* $1  (AT) */
	.set at
	SREG(v0,R_V0,t0)		/* $2  (v0) */
	SREG(v1,R_V1,t0)		/* $3  (v1) */
	SREG(a0,R_A0,t0)		/* $4  (a0) */
	SREG(a1,R_A1,t0)		/* $5  (a1) */
	SREG(a2,R_A2,t0)		/* $6  (a2) */
	SREG(a3,R_A3,t0)		/* $7  (a3) */
	SREG(t1,R_T1,t0)		/* $9  (t1) */
	SREG(t2,R_T2,t0)		/* $10 (t2) */
	SREG(t3,R_T3,t0)		/* $11 (t3) */
	SREG(t4,R_T4,t0)		/* $12 (t4) */
	SREG(t5,R_T5,t0)		/* $13 (t5) */
	SREG(t6,R_T6,t0)		/* $14 (t6) */
	SREG(t7,R_T7,t0)		/* $15 (t7) */
	SREG(s0,R_S0,t0)		/* $16 (s0) */
	SREG(s1,R_S1,t0)		/* $17 (s1) */
	SREG(s2,R_S2,t0)		/* $18 (s2) */
	SREG(s3,R_S3,t0)		/* $19 (s3) */
	SREG(s4,R_S4,t0)		/* $20 (s4) */
	SREG(s5,R_S5,t0)		/* $21 (s5) */
	SREG(s6,R_S6,t0)		/* $22 (s6) */
	SREG(s7,R_S7,t0)		/* $23 (s7) */
	SREG(t8,R_T8,t0)		/* $24 (t8) */
	SREG(t9,R_T9,t0)		/* $25 (t9) */
					/* $26 (k0) */
	SREG(k1,R_K1,t0)		/* $27 (k1) */
	SREG(gp,R_GP,t0)		/* $28 (gp) */
	SREG(sp,R_SP,t0)		/* $29 (sp) */
	SREG(s8,R_FP,t0)		/* $30 (s8) */
	SREG(ra,R_RA,t0)		/* $31 (ra) */

	mfhi	t1
	mflo	t2
	SREG(t1,R_HI,t0)		/* Hi */
	SREG(t2,R_LO,t0)		/* Lo */

	
	mfc0	t1,C0_INX
	SREG(t1,R_INDEX,t0)			/* Index */
	mfc0	t1,C0_RAND
	SREG(t1,R_RANDOM,t0)		/* Random */
	mfc0	t1,C0_TLBLO
	SREG(t1,R_ENTRYLO,t0)		/* EntryLo */
	mfc0	t1,C0_CONFIG
	SREG(t1,R_CONFIG,t0)		/* Config */
	MFC0	t1,C0_CTEXT
	SREG(t1,R_CONTEXT,t0)		/* Context */
	mfc0	t1,C0_PGMASK
	SREG(t1,R_PGMASK,t0)		/* TLB PageMask */
	mfc0	t1,C0_WIRED
	SREG(t1,R_WIRED,t0)			/* TLB Wired */
	mfc0	t1,C0_CACHE
	SREG(t1,R_CACHE,t0)			/* Cache */
	MFC0	t1,C0_BADVADDR
	SREG(t1,R_BADVA,t0)			/* BadVA */
	mfc0	t1,C0_TLBHI
	SREG(t1,R_ENTRYHI,t0)		/* EntryHi */
	mfc0	t1,C0_STATUS
	SREG(t1,R_STATUS,t0)		/* Status */
	mfc0	t1,C0_CAUSE
	SREG(t1,R_CAUSE,t0)			/* Cause */
	mfc0	t1,C0_EPC
	SREG(t1,R_EPC,t0)			/* EPC */
	mfc0	t1,C0_PRID
	SREG(t1,R_PRID,t0)			/* Prid */
	mfc0	t1,C0_DEBUG
	SREG(t1,R_DEBUG,t0)			/* Debug */
	mfc0	t1,C0_DEPC
	SREG(t1,R_DEPC,t0)			/* DEPC */
	mfc0	t1,C0_TAGLO
	SREG(t1,R_TAGLO,t0)			/* Taglo */

	move	t1,ra
	SREG(t1,R_EPC,t0)			/* EPC */

	j		panic_service

END(panic)

#endif

