/*
 *-----------------------------------------------------------------
 *
 * Name:	intr_entry.S
 *
 * RCS:  $Id: intr_entry.S,v 1.13 2002/12/02 18:45:07 rick Exp $
 *
 * Copyright (C) 2002, Agile Storage, Inc.
 *
 * Description:	Interrupt entry point for handler functions
 *
 * Created by:	Rick Lund
 *
 * Date Created:	2/28/02
 *
 *-----------------------------------------------------------------
 */

#include <nfx-incl.h>
#include <sys/mips.h>
#include "except.h"

#define INTR_NEW_STACK

#define STACK_RES_SIZE	(REGSZ*4)

#define _XGPR_INDEX(x)	(STACK_RES_SIZE+(REGSZ*(x)))
#define _XCP0_INDEX(x)	(STACK_RES_SIZE+(REGSZ*INT_NUM_GPR)+(REGSZ*(x)))
#define _XFP_INDEX(x)	(STACK_RES_SIZE+(REGSZ*(INT_NUM_GPR+INT_NUM_CP0))+(REGSZ*(x)))

#define INT_STACK_SIZE	STACK_RES_SIZE+(REGSZ*(INT_NUM_GPR+INT_NUM_CP0))

#define INT_NUM_GPR		34
#define INT_GPR_ZERO	_XGPR_INDEX(0)
#define INT_GPR_AT		_XGPR_INDEX(1)
#define INT_GPR_V0		_XGPR_INDEX(2)
#define INT_GPR_V1		_XGPR_INDEX(3)
#define INT_GPR_A0		_XGPR_INDEX(4)
#define INT_GPR_A1		_XGPR_INDEX(5)
#define INT_GPR_A2		_XGPR_INDEX(6)
#define INT_GPR_A3		_XGPR_INDEX(7)
#define INT_GPR_T0		_XGPR_INDEX(8)
#define INT_GPR_T1		_XGPR_INDEX(9)
#define INT_GPR_T2		_XGPR_INDEX(10)
#define INT_GPR_T3		_XGPR_INDEX(11)
#define INT_GPR_T4		_XGPR_INDEX(12)
#define INT_GPR_T5		_XGPR_INDEX(13)
#define INT_GPR_T6		_XGPR_INDEX(14)
#define INT_GPR_T7		_XGPR_INDEX(15)
#define INT_GPR_S0		_XGPR_INDEX(16)
#define INT_GPR_S1		_XGPR_INDEX(17)
#define INT_GPR_S2		_XGPR_INDEX(18)
#define INT_GPR_S3		_XGPR_INDEX(19)
#define INT_GPR_S4		_XGPR_INDEX(20)
#define INT_GPR_S5		_XGPR_INDEX(21)
#define INT_GPR_S6		_XGPR_INDEX(22)
#define INT_GPR_S7		_XGPR_INDEX(23)
#define INT_GPR_T8		_XGPR_INDEX(24)
#define INT_GPR_T9		_XGPR_INDEX(25)
#define INT_GPR_K0		_XGPR_INDEX(26)
#define INT_GPG_K1		_XGPR_INDEX(27)
#define INT_GPR_GP		_XGPR_INDEX(28)
#define INT_GPR_SP		_XGPR_INDEX(29)
#define INT_GPR_FP		_XGPR_INDEX(30)
#define INT_GPR_RA		_XGPR_INDEX(31)
#define INT_GPR_LO		_XGPR_INDEX(32)
#define INT_GPR_HI		_XGPR_INDEX(33)

#define INT_NUM_CP0		3
#define INT_CP0_SR		_XCP0_INDEX(0)
#define INT_CP0_CAUSE	_XCP0_INDEX(1)
#define INT_CP0_EPC		_XCP0_INDEX(2)

#define INT_NUM_FP		34
#define INT_FP_F0		_XFP_INDEX(0)
#define INT_FP_F1		_XFP_INDEX(1)
#define INT_FP_F2		_XFP_INDEX(2)
#define INT_FP_F3		_XFP_INDEX(3)
#define INT_FP_F4		_XFP_INDEX(4)
#define INT_FP_F5		_XFP_INDEX(5)
#define INT_FP_F6		_XFP_INDEX(6)
#define INT_FP_F7		_XFP_INDEX(7)
#define INT_FP_F8		_XFP_INDEX(8)
#define INT_FP_F9		_XFP_INDEX(9)
#define INT_FP_F10		_XFP_INDEX(10)
#define INT_FP_F11		_XFP_INDEX(11)
#define INT_FP_F12		_XFP_INDEX(12)
#define INT_FP_F13		_XFP_INDEX(13)
#define INT_FP_F14		_XFP_INDEX(14)
#define INT_FP_F15		_XFP_INDEX(15)
#define INT_FP_F16		_XFP_INDEX(16)
#define INT_FP_F17		_XFP_INDEX(17)
#define INT_FP_F18		_XFP_INDEX(18)
#define INT_FP_F19		_XFP_INDEX(19)
#define INT_FP_F20		_XFP_INDEX(20)
#define INT_FP_F21		_XFP_INDEX(21)
#define INT_FP_F22		_XFP_INDEX(22)
#define INT_FP_F23		_XFP_INDEX(23)
#define INT_FP_F24		_XFP_INDEX(24)
#define INT_FP_F25		_XFP_INDEX(25)
#define INT_FP_F26		_XFP_INDEX(26)
#define INT_FP_F27		_XFP_INDEX(27)
#define INT_FP_F28		_XFP_INDEX(28)
#define INT_FP_F29		_XFP_INDEX(29)
#define INT_FP_F30		_XFP_INDEX(30)
#define INT_FP_F31		_XFP_INDEX(31)
#define INT_FP_FCR		_XFP_INDEX(32)
#define INT_FP_FID		_XFP_INDEX(33)

	.set noreorder
	.set noat

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:	interrupt handler entry, save registers and call
 *				exc_interrupt(status, cause, epc)
 *
 * Created by:	Rick Lund
 *
 * Date Created:	2/28/02
 *
 *-----------------------------------------------------------------
 */
	.globl	intr_entry
	.ent	intr_entry
intr_entry:

#ifdef INTR_NEW_STACK
	la		k1,intr_stack+0x2000
#else
	subu	k1,sp,INT_STACK_SIZE
#endif
/*
	dsrl	k1,3
	dsll	k1,3
*/

	sr	AT,INT_GPR_AT(k1)
	sr	v0,INT_GPR_V0(k1)
	sr	v1,INT_GPR_V1(k1)
	sr	a0,INT_GPR_A0(k1)
	sr	a1,INT_GPR_A1(k1)
	sr	a2,INT_GPR_A2(k1)
	sr	a3,INT_GPR_A3(k1)
	sr	t0,INT_GPR_T0(k1)
	sr	t1,INT_GPR_T1(k1)
	sr	t2,INT_GPR_T2(k1)
	sr	t3,INT_GPR_T3(k1)
	sr	t4,INT_GPR_T4(k1)
	sr	t5,INT_GPR_T5(k1)
	sr	t6,INT_GPR_T6(k1)
	sr	t7,INT_GPR_T7(k1)
	sr	t8,INT_GPR_T8(k1)
	sr	t9,INT_GPR_T9(k1)
	sr	gp,INT_GPR_GP(k1)
	sr	sp,INT_GPR_SP(k1)
	sr	fp,INT_GPR_FP(k1)
	sr	ra,INT_GPR_RA(k1)

	mfc0	a0,C0_SR
	
#if !defined(TX3927)
	li		t1,~(SR_EXL|SR_IE)
	and		t0,a0,t1
	mtc0	t0,C0_SR
#endif		
	
	mfc0	a1,C0_CAUSE
	MFC0	a2,C0_EPC
	mflo	t0
	mfhi	t1
	sr	a0,INT_CP0_SR(k1)
	sr	a1,INT_CP0_CAUSE(k1)
	sr	a2,INT_CP0_EPC(k1)
	sr	t0,INT_GPR_LO(k1)
	sr	t1,INT_GPR_HI(k1)

	move	sp,k1

	la		a3,intr_mode
	li		t0,1
	sw		t0,0(a3)

	jal		exc_interrupt		# exc_interrupt(status, cause, epc)
	nop

	la		a3,intr_mode
	sw		zero,0(a3)

	move	k1,sp

#if !defined(TX3927)
	lr a0,INT_CP0_SR(k1)
	lr a1,INT_CP0_EPC(k1)
	mtc0 a0,C0_SR
	mtc0 a1,C0_EPC
#endif	

	lr	AT,INT_GPR_AT(k1)
	lr	v0,INT_GPR_V0(k1)
	lr	v1,INT_GPR_V1(k1)
	lr	a0,INT_GPR_A0(k1)
	lr	a1,INT_GPR_A1(k1)
	lr	a2,INT_GPR_A2(k1)
	lr	a3,INT_GPR_A3(k1)

	lr	t2,INT_GPR_T2(k1)
	lr	t3,INT_GPR_T3(k1)
	lr	t4,INT_GPR_T4(k1)
	lr	t5,INT_GPR_T5(k1)
	lr	t6,INT_GPR_T6(k1)
	lr	t7,INT_GPR_T7(k1)
	lr	t8,INT_GPR_T8(k1)
	lr	t9,INT_GPR_T9(k1)
	lr	gp,INT_GPR_GP(k1)
	lr	sp,INT_GPR_SP(k1)
	lr	fp,INT_GPR_FP(k1)
	lr	ra,INT_GPR_RA(k1)

	lr	t0,INT_GPR_LO(k1)
	lr	t1,INT_GPR_HI(k1)
	mtlo	t0
	mthi	t1

	lr	t0,INT_GPR_T0(k1)
	lr	t1,INT_GPR_T1(k1)

#ifndef TX3927
	eret
	nop
#else
	lr	k0,INT_CP0_EPC(k1)
	nop
	nop
	nop
	nop
	j		k0
	rfe
	nop
#endif

	.end	intr_entry

	.set	at
	.set	reorder

