#ifndef BTRACE_H
#define BTRACE_H
/*
 * btrace.h -- debugging utility that provides printings of the backtrace of
 * the current function.
 *
 * originally: btrace.h,v 1.3 2000/10/13 00:00:59
 *
 * Borrowed for sm-fs
 *
 * $Id: btrace.h,v 1.3 2001/12/03 22:24:09 haahr Exp $
 */

#include <execinfo.h>
#include <sys/types.h>

/**
 * Size of backtraces to generate.
 */
u_int btrace_level;

/**
 * initializes the global structures that btrace needs
 *
 * The function btrace_init() initializes the global symbol table reading
 * all the symbols related to functions present in the executable
 * image. It will not work if the executable has been stripped.  If the
 * environment variable BTRACE_LEVEL is set it will specify the depth of
 * the backtrace.
 *
 * This init routine does not need to be called explicitly, but it depends
 * on malloc() still being usable the first time btrace_print() or
 * btrace_print_signal() is called.  If you're worried about heap corruption,
 * it may be best to call this when the program starts.
 *
 * Returns: nothing, if something goes wrong it simply abort()s.
 */
extern void btrace_init(void);

/**
 * prints a backtrace of function names and addresses
 *
 * The function btrace_print() prints the trace of functions calling the
 * current function in a similar fashion as gdb's backtrace command.
 *
 * Returns: nothing, if something goes wrong you'd better wave a dead chicken
 * over it.
 */
extern void btrace_print(void);


/**
 * prints a backtrace of function names and addresses, starting from
 * the point that a signal was delivered.
 *
 * The btrace_print_signal() function is similar to btrace_print(), but
 * can be invoked from a signal handler to report the point which the
 * signal occurred, rather than printing that the signal handler itself
 * was on the stack.  The two arguments are the user context (the third
 * argument to a sa_sigaction function) and the handler function address.
 */
extern void btrace_print_signal(void *context, void *handler);


/**
 * Print the function name or hex address for the specified PC into the
 * specified buffer.  Returns the number of bytes written.
 */
extern int btrace_sprint_function(char *buf, void *pc);


/**
 * Print the source file name and line number for the specified PC if
 * known to the specified buffer, or do nothing if the line is not known.
 * Returns the number of bytes written.
 */
extern int btrace_sprint_source_line(char *buf, void *pc);

#endif /* !BTRACE_H */
