/*
 * Copyright (C) 2001, Agile Storage Inc.
 *
 * $Id: buf.c,v 1.31 2002/01/30 15:51:00 bob Exp $
 */

#if !defined(SSC)

#include "btree.h"

fs_Buf buf_table[40000];
uint64 total_blks = 1;
int nbufs;

int32 fs_applicationId;
uint32 nfxMySlotId;

elog_appl_data_t elog_appl_data[ELOG_MAX_APP];
int32 elog_enabled, elog_severity;

#define NINODES 300000
#define MAX_FILE_SIZE 64		/* max file size in buffers */

struct {
	fs_Inode inode;
	fs_Buf **bufs;
} bt_inode_table[NINODES];

int max_inodes = 1;

void fs_bufWrite(fs_Context *context, fs_Buf *buf, int32 offset, int32 len, uint32 flags, uchar8 *bitmap)
{
#if 0
	fs_Buf *shadow_buf = &shadow_buf_table[buf - buf_table];
	memcpy(&shadow_buf->u.data[offset], &buf->u.data[offset], len);
#endif
}
void fs_inodeWrite(fs_Context *context, fs_Inode *inode, uint32 flags)
{
}

void fs_bufRelease(fs_Context *context, fs_Buf *buf)
{
	assert(buf->lock.mode == FS_LOCK_MODE_SHARE);
	buf->lock.mode = 0;
}

void fs_bufReleaseExclusive(fs_Context *context, fs_Buf *buf)
{
	assert(buf->lock.mode == FS_LOCK_MODE_EXCL);
	buf->lock.mode = 0;
}

void fs_inodeRelease(fs_Context *context, fs_Inode *inode)
{
	assert(inode->lock.mode == FS_LOCK_MODE_SHARE);
	inode->lock.mode = 0;
}

void fs_inodeReleaseExclusive(fs_Context *context, fs_Inode *inode)
{
	assert(inode->lock.mode == FS_LOCK_MODE_EXCL);
	inode->lock.mode = 0;
}

#define NEW(x) new(sizeof (x))

void *
new(int x)
{
	void *p = malloc(x);
	assert(p != NULL);
	bzero(p, x);
	return p;
}

void *
fs_ramAlloc(int32 what, int32 nbytes)
{
	return new(nbytes);
}

int32
eee_ramDealloc(void *p)
{
	return NFX_OK;
}

fs_Buf *
fs_bufGetBlock(fs_Context *context, fs_Inode *inode, fs_Block block, int32 mode, uint32 option)
{
	uint64 blknum = block.block;
	fs_Buf *buf;
	
	assert(blknum > 0);

	if (option & BUF_GET_OPT_NEW) {
		buf = &buf_table[nbufs++];
		buf->u.data = new(PAGE_SIZE);
		buf->block.block = total_blks++;
		buf->lock.mode = (mode == BUF_GET_MODE_EXCL) ?
			FS_LOCK_MODE_EXCL : FS_LOCK_MODE_SHARE;
		return buf;
	} else {
		assert(buf_table[blknum-1].block.block == blknum);
		buf = &buf_table[blknum-1];
		if (buf->lock.mode != 0) {
			assert(buf->lock.mode == FS_LOCK_MODE_EXCL);
		}
		buf->lock.mode = (mode == BUF_GET_MODE_EXCL) ?
			FS_LOCK_MODE_EXCL : FS_LOCK_MODE_SHARE;
		return buf;
	}
}

fs_Inode *
fs_inodeGet(fs_Context *context, int32 snapnum, uint64 inum, fs_InodeMode mode, uint32 option)
{
	if (option & INO_GET_OPT_NEW) {
		fs_Inode *inode = &bt_inode_table[max_inodes].inode;
	
		assert(max_inodes < NINODES);
	
		inode->dinode = NEW(fs_Dinode);
		bt_inode_table[max_inodes].bufs = new(MAX_FILE_SIZE * sizeof (fs_Buf *));
	
		inode->inum = max_inodes++;
		inode->dinode->size = 0;
		inode->lock.mode = (mode == INO_GET_MODE_EXCL) ?
			FS_LOCK_MODE_EXCL : FS_LOCK_MODE_SHARE;
		return inode;
	} else {
		fs_Inode *inode = &bt_inode_table[inum].inode;
		assert(inum > 0 && inum < NINODES);
		if (inode->lock.mode != 0) {
			assert(inode->lock.mode == FS_LOCK_MODE_EXCL);
		}
		inode->lock.mode = (mode == INO_GET_MODE_EXCL) ?
			FS_LOCK_MODE_EXCL : FS_LOCK_MODE_SHARE;
		return inode;
	}
}

fs_Buf *                       
fs_bufGetPage(fs_Context *context, fs_Inode *inode, uint64 offset, int32 mode, uint32 option)
{
	fs_Buf **bufp = bt_inode_table[inode->inum].bufs;
	fs_Buf *buf;
	int i;

	if (offset*PAGE_SIZE >= inode->dinode->size)
		inode->dinode->size = offset*PAGE_SIZE;

	for (i = 0; ; i++, --offset) {
		if (bufp[i] == NULL)
			bufp[i] = B_CREATE(inode);
		if (offset == 0)
			break;
	}

	buf = bufp[i];

	if (buf->lock.mode != 0) {
		assert(buf->lock.mode == FS_LOCK_MODE_EXCL);
	}
	buf->lock.mode = (mode == BUF_GET_MODE_EXCL) ?
		FS_LOCK_MODE_EXCL : FS_LOCK_MODE_SHARE;

	return buf;
}


#if 0
void
buf_dump()
{
	int i;
	for (i = 0; i < nbufs; i++)
		if (buf_table[i].type == DIRLEAF) {
			/*fprintf(stderr, "leaf %d:\n", buf_table[i].extent.start);*/
			fs_dirleaf_read(buf_table[i].blknum);
		}
}
#endif

void
fs_trace(fs_Context *context, int32 level, char *fmt, ...)
{
}

void
fs_bufFree(fs_Context *context, fs_Buf *buf, int option)
{
	assert(&buf_table[buf->block.block-1] == buf);
	free(buf->u.data);
	buf->u.data = NULL;
}

fs_Inode *
fs_allocateFileInode(
    fs_Context *context,
    fs_Inode *directory,
    uint16 mode,
    uint32 uid,
    uint32 gid,
    uint32 filesetid)
{
	return fs_inodeGet(context, 0, 0, 0, INO_GET_OPT_NEW);
}

void
fs_deallocateInode(fs_Context *context, fs_Inode *inode)
{
}

void
fs_deallocateExtent(fs_Context *context, fs_Extent extent)
{
}

void
fs_truncateFile(fs_Context *context, fs_Inode *inode, uint64 newsize)
{
}

void
fs_inodeRemoveFromCache(fs_Context *context, fs_Inode *inode, fs_LogSummary *logSummary, boolean isReplay)
{
}

void
fs_eek_mark(char *msg, uint8 *bitmap, uint64 bitnum)
{
}

void
fs_eek_increment_linkcount(fs_Context *context, uint64 inum)
{
}

noreturn fs_abort() {
	abort();
}

noreturn fs_panic(char *fmt,...) {
	abort();
}

void fs_trace_elog(
    fs_Context *context,
    int32      level,
    int        line,
    char*      file,
    char       *traceString)
{
}

char *
e_log_var_pa (char *format, ...)
{
	return 0;
}

void fs_bufCopyOnWriteMetadata(
    fs_Context *context,
	fs_Inode   *inode,
    fs_Buf     *buf)
{
}

boolean fs_bufIsCopyOnWrite(
    fs_Context      *context,
    fs_Buf          *buf)
{
	return FALSE;
}

boolean
fs_lockOwnedExclusive(
	fs_Context *context,
	fs_Lock *lock)
{
	return TRUE;
}

fs_RefByte fs_refGetByte(
    fs_Context *context,
    fs_Block block)
{
	fs_RefByte r;
	return r;
}

unsigned fs_refbyteSnapCount(
    fs_RefByte refbyte)
{
	return 0;
}


#endif // SSC
