/*
 * Copyright (C) 2001, Agile Storage Inc.
 *
 * $Id: emap-debug.c,v 1.8 2002/09/20 18:51:58 bob Exp $
 */

#if !defined(SSC)

#include "btree.h"

static int length = 10;

/*
 * Populate an inode with 10,000 bogus extents, all adjacent.
 */
void
fs_emapdebug_populate(fs_Context *context, fs_Inode *inode, int count)
{
	fs_Extent e;
	uint64 offset = count * length;

	printf("O%d ", (int)offset);
#if defined(FS_STANDALONE)
	fflush(stdout);
#endif

	e = fs_makeExtent(length, fs_makeBlock(offset));
	fs_emap_add(context, inode, offset, e);
}

void
fs_emapdebug_search(fs_Context *context, fs_Inode *inode, int count)
{
	uint64 offset = count * length + (random() % length);
	fs_Emappair p = fs_emap_lookup(context, inode, offset);
	printf("offset = %lld\n", offset);
	printf("offset - offset%%length = %lld\n", offset - (offset%length));
	printf("p.offset = %lld\n", p.offset);
	FS_ASSERT(p.offset == offset - (offset % length));
}

void
fs_emapdebug(fs_Context *context)
{
	static int state = 0;
	static int count = 0;
	static int count_max = 10000;
	static int created;
	fs_Inode *inode;

	if (!created) {
		created = 1;
		inode = fs_inodeInstantiateMetanode(context, 0, 31);
		fs_emaproot_init(context, inode);
	} else {
		inode = I_GET_EXCL(0, 31);
	}

	switch (state) {
	default:
		FS_ASSERT(state == 0);
		/*NOTREACHED*/
	case 0:
		fs_emapdebug_populate(context, inode, count);
		break;
	case 1:
		fs_emapdebug_search(context, inode, count);
		break;
	}

	count++;
	if (count == count_max) {
		count = 0;
		state++;
	}

	inode->dinode.u.data[0] = 4; /* XXX: bogus touch */
    fs_inodeWrite(context, inode, INO_WRITE_LOG);
}
#endif // SSC
