/*
 * Name:  fs-cod-rev2.h
 *
 * RCS:  $Id: fs-cod-rev2.h,v 1.2 2002/08/24 02:34:08 bob Exp $
 *
 * Description:  Functions for communicating cod manipulations
 *               with revision 2 of the SAC Buffer Manager (SBM).
 * 
 * Date Created:  8/19/2002
 */

#if !defined(FS_COD_REV2_H)
#define FS_COD_REV2_H


/**
 * A special pre-initialized cod filled with '0's.
 *
 * This cod should never be freed.
 */
extern nfx_cod_t fs_nullCOD;


/**
 * Split a cod into two cods at the specified offset.
 */
extern int32
fs_cod_split (
    nfx_cod_t   src_COD,
    uint32      src_offset,
    nfx_cod_t   dest_COD);


/**
 * Merge the contents of src_COD into dst_COD.  After the
 * merge, src_COD is freed by the sbm.
 */
extern int32
fs_cod_merge (
    nfx_cod_t   src_COD,
    uint32      src_len,
    uint32      src_offset,
    nfx_cod_t   dst_COD,
    uint32      dst_offset);


/**
 * Create a zero-filled cod with the specified handle
 * and length.
 */
extern int32
fs_cod_create_null (
    nfx_cod_t   COD,
    uint32      len,
    uint32      offset);


/**
 * Fill the specified cod with 0's for the specified
 * offset and length.  Updates the COD pointer with a
 * new cod handle, the previous one is freed by the sbm.
 */ 
extern int32
fs_cod_fill_null (
    nfx_cod_t   *COD,
    uint32      len,
    uint32      offset);


/**
 * Create a new cod handle.
 */
extern nfx_cod_t 
fs_cod_get_new_cod();


/**
 * Free the specified cod.
 */
extern void
fs_cod_free_cod(
    nfx_cod_t COD);


/**
 * Initialize this module.
 */
extern void
fs_cod_init();

#endif
