/*
 * Name:  fs-debug.h
 *
 * RCS:  $Id: fs-debug.h,v 1.42 2002/11/22 21:32:32 maximk Exp $
 *
 * Description:  Filesystem debug message interface.
 * 
 * Created By:  Bob Fozard
 *
 * Date Created:  3/20/2001
 */

#if !defined(FS_DEBUG_H)
#define FS_DEBUG_H

#include "fs-vol-log.h"

/* export some debug functions and var
 */

#if defined(FS_DEBUG_CHECK_CRASH)
extern int32 fs_debugCrashEnabled ;
extern void fs_debugCrash();

void fs_debugCheckCrash (fs_Context *context);
#endif

/*-----------------------------------------------------------------------------
 * snapDiag req
 */
typedef struct fs_MsgSnapdiagReq
{
    fs_MsgReqHdr    hdr;
    evm_volId_t     volId;
#define FS_SNAP_DIAG_GET_REFBYTES   1
#define FS_SNAP_DIAG_GET_SNAPBITS   2
#define FS_SNAP_DIAG_GET_STATS      3
#define FS_SNAP_DIAG_FORCE_CRASH    4
    uint16          cmd;                //
    uint16          phase;              // Phase to crash if FS_SNAP_DIAG_FORCE_CRASH
    uint16          nameLen;            // Length of snapDirName in bytes NOT
                                        // counting NULL terminator
    char            snapDirName[0];     // NULL terminated snapshot dir name up
                                        // to FS_NAME_MAX

}  fs_MsgSnapdiagReq;
typedef struct fs_MsgSnapdiagResp
{
    fs_MsgRespHdr       hdr;
    uint16              cmd;
    fs_SnapStats        stats;          // only if cmd is FS_SNAP_DIAG_GET_STATS
}  fs_MsgSnapdiagResp;
NFX_ASSERT_STATIC(sizeof(fs_MsgSnapdiagResp) <= FS_SHELL_RESP_MAX);

/*-----------------------------------------------------------------------------
 * cachestats req
 */
typedef struct fs_MsgCachestatsReq
{
    fs_MsgReqHdr        hdr;
    evm_volId_t         volId;
    boolean             isClear;
    boolean             isVerbose;
}  fs_MsgCachestatsReq;
/*
 * cachestats resp
 */
typedef struct fs_CacheStats
{
    uint32              size;
    uint32              used;
    uint32              dirty;
    uint32              misses;
    uint32              hits;
    uint32              cows;
#ifdef FS_DYN_CACHE_SIZE
    uint32              LWM;
#endif
} fs_CacheStats;

typedef struct fs_MsgCachestatsResp
{
    fs_MsgRespHdr       hdr;
    evm_volId_t         volId;
    fs_CacheStats       bufCacheStats[FS_BUFLIST_MAX][FS_BUF_CACHE_ITYPE_MAX];
#ifdef FS_DEBUG_BUF_CNT_MISS_TYPE
    uint32              bufMissCntType[FS_BUFLIST_MAX][FS_MAX_META_INODE];
    uint32              bufUseCntType[FS_BUFLIST_MAX][FS_MAX_META_INODE]; 
#endif
    fs_CacheStats       inodeCacheStats[1];
#ifdef FS_DEBUG_INODE_CNT_MISS_TYPE
    uint32              missCntType[FS_MAX_META_INODE];
#endif
    uint32              diskWritebackType[FS_MAX_META_INODE];
    uint32              transWriteType[FS_MAX_META_INODE];
    uint32              bufWbCount[FS_BUFLIST_MAX][FS_WB_WAKEUP_MAX];
    uint32              userBufWbMaxPerLoop;
    uint32              metaBufWbMaxPerLoop;
    uint32              inodeCleanerCount[FS_INODE_CLEANER_MAX];
    uint32              inodeCleanerMaxPerLoop;
    uint32              maxReadsOutstanding;
    uint32              maxWritesOutstanding;
    uint32              currReadsOutstanding;
    uint32              currWritesOutstanding;
    uint32              totalMetaReads;
    uint32              totalMetaReadAheads;
    uint32              totalMetaWrites;
    uint32              totalUserReads;
    uint32              totalUserReadAheads;
    uint32              totalUserWrites;
    uint32              totalLogReads;
    uint32              totalLogWrites;
    uint32              freeMem;

#ifdef FS_INODE_CHECK_HASH_DEPTH
    uint32              inodeHashMax;
    uint32              inodeHashUsedEntries;
    uint32              inodeHashTotalEntries;
    uint32              inodeHashMaxDepth;
    uint32              nbrLookups;
    uint32              nbrScanned;
#endif

#ifdef FS_BUF_BLK_CHECK_HASH_DEPTH
    uint32              UserBufBlkHashMax;
    uint32              UserBufBlkHashUsedEntries;
    uint32              UserBufBlkHashTotalEntries;
    uint32              UserBufBlkHashMaxDepth;
    uint32              MetaBufBlkHashMax;
    uint32              MetaBufBlkHashUsedEntries;
    uint32              MetaBufBlkHashTotalEntries;
    uint32              MetaBufBlkHashMaxDepth;
#endif

#ifdef FS_BUF_PAGE_CHECK_HASH_DEPTH
    uint32              UserBufPageHashMax;
    uint32              UserBufPageHashUsedEntries;
    uint32              UserBufPageHashTotalEntries;
    uint32              UserBufPageHashMaxDepth;
    uint32              MetaBufPageHashMax;
    uint32              MetaBufPageHashUsedEntries;
    uint32              MetaBufPageHashTotalEntries;
    uint32              MetaBufPageHashMaxDepth;
#endif

#ifdef FS_BUF_INUM_CHECK_HASH_DEPTH
    uint32              UserBufInumHashMax;
    uint32              UserBufInumHashUsedEntries;
    uint32              UserBufInumHashTotalEntries;
    uint32              UserBufInumHashMaxDepth;
    uint32              MetaBufInumHashMax;
    uint32              MetaBufInumHashUsedEntries;
    uint32              MetaBufInumHashTotalEntries;
    uint32              MetaBufInumHashMaxDepth;
#endif

    boolean             isVerbose;

}  fs_MsgCachestatsResp;
NFX_ASSERT_STATIC(sizeof(fs_MsgCachestatsResp) <= FS_SHELL_RESP_MAX);


/*-----------------------------------------------------------------------------
 * dnlcstats req
 */
typedef struct fs_MsgDnlcstatsReq
{
    fs_MsgReqHdr        hdr;
    evm_volId_t         volId;
    boolean             isClear;
    boolean             isFlush;
}  fs_MsgDnlcstatsReq;
/*
 * dnlcstats resp
 */
typedef struct fs_MsgDnlcstatsResp
{
    fs_MsgRespHdr       hdr;
    evm_volId_t         volId;
    fs_DnlcStats        stats;
    int32               usedBuckets;
    int32               maxDepth;
} fs_MsgDnlcstatsResp;
NFX_ASSERT_STATIC(sizeof(fs_MsgDnlcstatsResp) <= FS_SHELL_RESP_MAX);


/*-----------------------------------------------------------------------------
 * memstats req
 */
typedef struct fs_MsgMemstatsReq
{
    fs_MsgReqHdr        hdr;
    evm_volId_t         volId;

}  fs_MsgMemstatsReq;
/*
 * memstats resp
 */
typedef struct fs_MsgMemstatsResp
{
    fs_MsgRespHdr       hdr;
    uint32              ramStats[FS_RAM_STATS_HOW_MAX][FS_RAM_TYPE_MAX];

#ifndef FS_STANDALONE
	uint64		total_flex_size;
	uint64      total_flex_used;
	uint64		total_stack_size;
	uint64      total_stack_used;
	uint64		total_cont_size;
	uint64      total_cont_used;

#define FS_EEE_NBR_BUFS_IN_MEM_RESP  4    //   EEE_BUF_SMALL, EEE_BUF_LARGE, EEE_BUF_ULTRA (+ 1 alignment)
	uint32	    eeeBufs[FS_EEE_NBR_BUFS_IN_MEM_RESP];
	uint32	    eeeAvailableBufs[FS_EEE_NBR_BUFS_IN_MEM_RESP];
	uint32	    eeeLWMBufs[FS_EEE_NBR_BUFS_IN_MEM_RESP];

	uint32	    eeeTxrxBufs[FS_EEE_NBR_BUFS_IN_MEM_RESP];
	uint32	    eeeTxrxAvailableBufs[FS_EEE_NBR_BUFS_IN_MEM_RESP];
	uint32	    eeeTxrxLWMBufs[FS_EEE_NBR_BUFS_IN_MEM_RESP];
#endif

}  fs_MsgMemstatsResp;
NFX_ASSERT_STATIC(sizeof(fs_MsgMemstatsResp) <= FS_SHELL_RESP_MAX);

/*-----------------------------------------------------------------------------
 * reqstats req
 */
typedef struct fs_MsgReqstatsReq
{
    fs_MsgReqHdr        hdr;
    evm_volId_t         volId;
    boolean             isClear;
    boolean             isVerbose;
}  fs_MsgReqstatsReq;
/*
 * reqstats resp
 */
typedef struct fs_MsgReqstatsResp
{
    fs_MsgRespHdr       hdr; 
    evm_volId_t         volId;
    boolean             isVerbose;
    fs_RecvReqStats     rcvd;
    fs_ReplyStats       reply;
}  fs_MsgReqstatsResp;
NFX_ASSERT_STATIC(sizeof(fs_MsgReqstatsResp) <= FS_SHELL_RESP_MAX);

/*-----------------------------------------------------------------------------
 * logstats req
 */
typedef struct fs_MsgLogstatsReq
{
    fs_MsgReqHdr        hdr;
    evm_volId_t         volId;
    boolean             isClear;
    boolean             isVerbose;
}  fs_MsgLogstatsReq;
/*
 * logstats resp
 */
typedef struct fs_MsgLogstatsResp
{
    fs_MsgRespHdr       hdr;

    fs_LogDevType       logDevType;         // int volid, ext vol, nvram, etc
    uint32              logSize;            // Size of external device
    uint32              head;
    uint32              currLen;
    uint32              tail;
    boolean             isVerbose;

    fs_LogStats         logStats;           // log statistics

    union
    {
        fs_vl_stats        vollog;
    } u;

}  fs_MsgLogstatsResp;
NFX_ASSERT_STATIC(sizeof(fs_MsgLogstatsResp) <= FS_SHELL_RESP_MAX);

/*-----------------------------------------------------------------------------
 * nfsstats req
 */
typedef struct fs_MsgNfsstatsReq
{
    fs_MsgReqHdr        hdr;
    evm_volId_t         volId;
    boolean             isVerbose;
    boolean             isClear;

}  fs_MsgNfsstatsReq;

typedef struct fs_MsgNfsstatsResp
{
    fs_MsgRespHdr       hdr; 
    evm_volId_t         volId;
    boolean             isVerbose;
    uint32              currentActiveReqs;
    fs_NfsStats         nfsStats;
    fs_YieldStats       yieldStats;
    fs_RecvNfsReqStats  recvNfsReqStats;
	uint64              count_per_usec;
}  fs_MsgNfsstatsResp;
NFX_ASSERT_STATIC(sizeof(fs_MsgNfsstatsResp) <= FS_SHELL_RESP_MAX);

/*-----------------------------------------------------------------------------
 * errstats req
 */
typedef struct fs_MsgErrstatsReq
{
    fs_MsgReqHdr        hdr;
    evm_volId_t         volId;
    boolean             isClear;
}  fs_MsgErrstatsReq;

typedef struct fs_MsgErrstatsResp
{
    fs_MsgRespHdr       hdr;
    fs_ErrStats         errStats;
}  fs_MsgErrstatsResp;
NFX_ASSERT_STATIC(sizeof(fs_MsgErrstatsResp) <= FS_SHELL_RESP_MAX);

/*-----------------------------------------------------------------------------
 * yieldstats req
 */
typedef struct fs_MsgYieldstatsReq
{
    fs_MsgReqHdr        hdr;
    evm_volId_t         volId;
    boolean             isClear;
    boolean             isAverage;
    boolean             showIO;
    boolean             showRestore;
    boolean             showDaemon;
}  fs_MsgYieldstatsReq;

typedef struct fs_MsgYieldstatsResp
{
    fs_MsgRespHdr       hdr;
    evm_volId_t         volId;
    fs_YieldStats       yieldStats;
    fs_RecvNfsReqStats  recvNfsReqStats;
    uint64	            ht_bytes_tx;
	uint64				ht_bytes_rx;
    uint32              currentActiveReqs;
    uint32              maxActiveReqs;
    uint32              maxNbrContexts;
    int32               nbrOngoingTrans;
    int32               maxOngoingTrans;
    boolean             isAverage;
    boolean             showIO;
    boolean             showRestore;
    boolean             showDaemon;
	uint64              count_per_usec;
}  fs_MsgYieldstatsResp;
NFX_ASSERT_STATIC(sizeof(fs_MsgYieldstatsResp) <= FS_SHELL_RESP_MAX);



/*-----------------------------------------------------------------------------
 * lockstats req
 */
typedef struct fs_LockStatBuf
{
    int32 metaBufInumPage[FS_MAX_META_INODE];
    int32 metaBufInumOther[FS_MAX_META_INODE];
    int32 userBufInumPage;
    int32 userBufInumOther;

} fs_LockStatBuf;
extern fs_LockStatBuf fs_lockStatBuf;

typedef struct fs_MsgLockstatsReq
{
    fs_MsgReqHdr        hdr;
    evm_volId_t         volId;
    boolean             isClear;
    boolean             isAverage;
    int32               lockReqType;

}  fs_MsgLockstatsReq;

typedef struct fs_MsgLockstatsResp
{
    fs_MsgRespHdr       hdr;
    evm_volId_t         volId;
    int32               lockReqType;
    int32               maxSharedLockListSize;
    union {
#define FS_LOCKSTATS_MAX    16
        eeeRTC_t    waitTime[FS_LOCKSTATS_MAX];
        uint32      waitStats[FS_LOCK_TYPE_MAX][FS_LOCK_OWNER_MAX];
        fs_LockStatBuf lockStatBuf;
    } u;
	uint64              count_per_usec;

}  fs_MsgLockstatsResp;
NFX_ASSERT_STATIC(sizeof(fs_MsgLockstatsResp) <= FS_SHELL_RESP_MAX);


/*-----------------------------------------------------------------------------
 * context req
 */
typedef struct fs_MsgContextReq
{
    fs_MsgReqHdr        hdr;
    evm_volId_t         volId;
    esm_handle_t        handle;

}  fs_MsgContextReq;
/*
 * context resp
 */
typedef struct fs_MsgContextResp
{
    fs_MsgRespHdr       hdr; 
    // this is too big for the management plane 
    // fs_Context          context;

}  fs_MsgContextResp;
NFX_ASSERT_STATIC(sizeof(fs_MsgContextResp) <= FS_SHELL_RESP_MAX);

extern void
fs_dumpBuf(
    fs_Buf *buf
);

/*-----------------------------------------------------------------------------
 * logdata req
 */
typedef struct fs_MsgLogdataReq
{
    fs_MsgReqHdr        hdr;
    evm_volId_t         volId;
    int32               nbrEntries;
    int32               offset;
	int32				criteria;
	uint64				value;

}  fs_MsgLogdataReq;
/*
 * logdata resp
 */
typedef struct fs_MsgLogdataResp
{
    fs_MsgRespHdr       hdr;
#define FS_LOGDATA_MAX_SIZE  1380
    char                data[FS_LOGDATA_MAX_SIZE];

}  fs_MsgLogdataResp;
NFX_ASSERT_STATIC(sizeof(fs_MsgLogdataResp) <= FS_SHELL_RESP_MAX);

/*-----------------------------------------------------------------------------
 * logtest req
 */
#define FS_LOG_TEST_PHASE1  1
#define FS_LOG_TEST_PHASE2  2
#define FS_LOG_TEST_PHASE3  3
#define FS_LOG_TEST_PHASE4  4

typedef struct fs_MsgLogtestReq
{
    fs_MsgReqHdr        hdr;
    evm_volId_t         volId;
#define FS_LOG_TEST_FORCE_CRASH 1
#define FS_LOG_TEST_REPLAY      2
#define FS_LOG_TEST_CRASH_NOW   3
    int32               type;
    int32               crashPhase;
    int32               n;
}  fs_MsgLogtestReq;

typedef struct fs_MsgLogtestResp
{
    fs_MsgRespHdr       hdr;
} fs_MsgLogtestResp;

/*-----------------------------------------------------------------------------
 * crash req
 */
typedef struct fs_MsgCrashReq
{
    fs_MsgReqHdr        hdr;
    evm_volId_t         volId;
    int32               type;
    int32               count;
}  fs_MsgCrashReq;
/*
 * crash resp (will not come back)
 */
typedef struct fs_MsgCrashResp
{
    fs_MsgRespHdr       hdr;
}  fs_MsgCrashResp;
NFX_ASSERT_STATIC(sizeof(fs_MsgCrashResp) <= FS_SHELL_RESP_MAX);


/*
 *  set/read incore tune parameters
 */

typedef struct {
    fs_MsgReqHdr        hdr;        // message header
    int32               fieldId;
    uint64              value;
} fs_MsgIncoreTuneReq;

typedef struct {
    fs_MsgRespHdr       hdr;        // message header
    fs_IncoreTune       incore;
	uint64              count_per_usec;
} fs_MsgIncoreTuneResp;
NFX_ASSERT_STATIC(sizeof(fs_MsgIncoreTuneResp) <= FS_SHELL_RESP_MAX);


#endif // FS_DEBUG_H
