/*
 * Name:  fs-dirhash.h
 *
 * Copyright (C) 2002, Agile Storage Inc.
 *
 * RCS:  $Id: fs-dirhash.h,v 1.1 2002/11/26 23:55:39 bob Exp $
 *
 * Description:  API for dirhash cache.
 *
 */

#if !defined(FS_DIRHASH_H)
#define FS_DIRHASH_H


/**
 * Lookup the specified entry in the specified directory.
 * If the table doesn't exist, return UNKNOWN, otherwise
 * return FOUND or NOT_FOUND.
 */
enum {
    FS_DIRHASH_FOUND,
    FS_DIRHASH_NOT_FOUND,
    FS_DIRHASH_UNKNOWN
};
extern int32
fs_dirhash_lookup(
    fs_Context  *context,
    fs_Inode    *inode,
    fs_DirSearchCb *cb);
  
/**
 * Insert the specified entry into the dirhash cache.
 */
extern void
fs_dirhash_insert(
    fs_Context  *context,
    fs_Inode    *inode,
    uint64      page,
    int         index,
    uint32      hash);

/**
 * Remove the specified entry from the dirhash cache.
 */
extern void
fs_dirhash_remove(
    fs_Context  *context,
    fs_Inode    *inode,
    uint64      page,
    int         index,
    uint32      hash);

/**
 * Iterate over the directory visiting each
 * entry with a matching hash with iter_fun().
 */
extern boolean
fs_dirhash_iterate(
    fs_Context      *context,
    fs_Inode        *inode,
    fs_DirSearchCb  *cb,
    void            (*iter_fun)(
            fs_Inode    *inode,
            fs_Dirent   *dirent,
            fs_DirSearchCb  *cb));

/**
 * Initialize the dirhash cache.
 */
extern void
fs_dirhash_cache_init();

/**
 * Flush the dirhash cache on an rmdir.
 */
extern void
fs_dirhash_destroy_directory(
    fs_Context  *context,
    fs_Inode    *inode);


/**
 * Flush the dirhash cache for the specified filesystem.
 */
extern void
fs_dirhash_destroy_volume(
    fs_FileSystem   *fileSystem);


/**
 * Flush the dirhash cache for the specified filesystem,snapnum.
 */
extern void
fs_dirhash_destroy_snapnum(
    fs_FileSystem   *fileSystem,
    int32           snapnum);


#endif // FS_DIRHASH_H
