/*
 * Copyright (C) 2001, Agile Storage Inc.
 *
 * $Id: fs-eek-query.c,v 1.13 2002/10/30 01:40:25 bob Exp $
 */

#if !defined(SSC)

#include "fs-includes.h"

int
fs_eek_vquery(fs_Context *context, char *fmt, va_list ap)
{
	int32 rc;
#if defined(FS_STANDALONE)
	int32 cpuId;
#endif
	fs_MsgEekResp *resp;
	fs_MsgEekReq *req;
	esm_event_t event;
	int q;

	context->respHdr->fsStatus = NFX_OK;

	resp = (fs_MsgEekResp *)context->respHdr;
	resp->has_query = TRUE;
    resp->has_status = FALSE;
	vsprintf(resp->msg, fmt, ap);
	resp->private = context->smHandle;
	// resp->hdr.requestType = FS_REQ_EEKCONT;

#if defined(FS_STANDALONE)
	cpuId = (int32) pthread_getspecific(fs_simKeyCpuId);
	resp = fs_ramAlloc(FS_RAM_CORE_MISC, sizeof *resp);
	memcpy(resp, context->respHdr, sizeof *resp);
	rc = fs_simSendMsg(cpuId, context->reqHdr->src, (char *)resp,
			  context->respSize);
#else
	// TODO - check if system buffer is used for response (vs. malloc
	// buffer), use the appropriate send function (this currently
	// assumes a malloc'd buffer is pointed to by respHdr).
	rc = eee_sendMessage(context->respHdr,
			     context->respSize,
			     context->srcAddr,
			     fs_packedPortAddr(nfxMyCPUId, EEE_CONTROL));
#endif
	if (rc != NFX_OK) {
		fs_error(context, FS_EEE_ERR, "eek query");
	}

    fs_yield(context, &event, FS_YIELD_TIME_FOREVER, FS_YIELD_TYPE_ADMIN);
	assert(event.event == FS_THREAD_EVENT_EEKCONT);

	/*
	 * XXX: hack: second and subsequent queries communicate
	 * with the EEKCONT message type.
	 */
	context->respHdr->requestType = FS_REQ_EEKCONT;

	req = event.p1.u.p;
	q = req->query_reply;

	context->respHdr->requestType = req->hdr.requestType;
	context->respHdr->clientPrivate = req->hdr.clientPrivate;
	context->respHdr->clientId  = req->hdr.clientId;

	if (event.p2.u.w & FS_CONTEXT_SHARED_REQUEST)
	{
		eee_deallocateBuffer(req);
	}
	else
	{
		fs_ramDealloc(FS_RAM_REQ_MSG, req); 
	}

	switch (q) {
	default: assert(0);
	case FS_EEK_QUERY_NO:
	case FS_EEK_QUERY_YES:
	case FS_EEK_QUERY_QUIT:
        return q;
	}

	/* NOTREACHED */
}

/**
 * Handle the EEKCONT notification.
 */
int32
fs_eekContInstantiate(const fs_ReqTypeTbl *reqType,
					  const fs_MsgReqHdr *reqHdr,
					  int32 bufType, uint32 srcAddr)
{
    fs_Context *context;
	fs_MsgEekReq *req;
	esm_handle_t handle;
	esm_event_t event;
	int32 rc;

	req = (fs_MsgEekReq *) reqHdr;
	handle = req->private;
	FS_TRACE(NULL, FS_TRACE_REQ, ("smHandle 0x%llx eekcont msg rcvd",
				      (uint64)((uintptr_t)handle)));
	event.event = FS_THREAD_EVENT_EEKCONT;
	event.p1.u.p = req;
	event.p2.u.w = bufType;

    // EEK thread needs to return response using this src address:
    //
    // SSC        FP
    // ---        --
    //  EEK     -> port x EEK thread
    //          <- port x EEK query
    //  EEK_CONT-> port y for this thread
    //          <- EEK rsp needs to be sent back on port y and not x
    //

    context = (fs_Context *) esm_getUserInfo(handle);
	if (context == NULL)
	{
		printf("eek_cont: eek context went away!\n");
		return NFX_ERR;
	}
    assert(context->reqHdr->requestType == FS_REQ_EEK);
    context->srcAddr = srcAddr;

	rc = thread_event(handle, &event);
	if (rc != NFX_OK) {
		fs_error(NULL, FS_EVENT_DELIVERY_FAIL, "eek cont event");
	}

	return NFX_OK;
}

#endif // SSC
