/*
 * Copyright (C) 2001, Agile Storage Inc.
 *
 * $Id: fs-eek-standalone.c,v 1.25 2002/02/13 19:46:35 byron Exp $
 */

#if !defined(SSC)

#include "fs-includes.h"
#include "btrace.h"

#include <sys/wait.h>
#include <sys/uio.h>
#include <readline/readline.h>
#include <readline/history.h>
#include <fcntl.h>


int volume_fd;

fs_Buf buf_table[40000];
uint64 total_blks = 1;
int nbufs;

int32 fs_applicationId;
uint32 nfxMySlotId;

int32 elog_enabled, elog_severity;

#define NINODES 300000
#define MAX_FILE_SIZE 64		/* max file size in buffers */

struct {
	fs_Inode inode;
	fs_Buf **bufs;
} bt_inode_table[NINODES];

int max_inodes = 1;

void fs_bufWrite(fs_Context *context, fs_Buf *buf, int32 offset, int32 len, uint32 flags, void *bitmap)
{
#if 0
	fs_Buf *shadow_buf = &shadow_buf_table[buf - buf_table];
	memcpy(&shadow_buf->u.data[offset], &buf->u.data[offset], len);
#endif
}
void fs_inodeWrite(fs_Context *context, fs_Inode *inode, uint32 flags)
{
}

#define NEW(x) new(sizeof (x))

void *
new(int x)
{
	void *p = malloc(x);
	assert(p != NULL);
	bzero(p, x);
	return p;
}

void *
fs_ramAlloc(int32 what, int32 nbytes)
{
	return new(nbytes);
}

void
fs_ramDealloc(int32 what, void *p)
{
}

int32
eee_ramDealloc(void *p)
{
	return 0;
}

int32
esm_deliverEvent(
    uint32      handle,
    esm_event_t *event)
{
        return NFX_OK;
}

void
esm_yield(
	esm_event_t	*eventp,
	int32		waitSecs)
{
} 

fs_Buf *
fs_bufGetBlock(fs_Context *context, fs_Inode *inode, fs_Block block, int32 mode, uint32 option)
{
	fs_Buf *buf = NEW(fs_Buf);
	char *data = new(PAGE_SIZE);
	uint64 blknum = fs_blockNumber(block);
	int64 r;

	if (option & INO_GET_OPT_NEW) {
		FS_ASSERT("get new buf" == NULL);
	}

	// printf("get block %d\n", (int)blknum);

	r = lseek64(volume_fd, blknum * PAGE_SIZE, 0);
	if (r != blknum * PAGE_SIZE) {
		perror("lseek: volume_fd");
		exit(1);
	}
	r = read(volume_fd, data, PAGE_SIZE);
	if (r != PAGE_SIZE) {
		perror("read: volume_fd");
		exit(1);
	}

	buf->u.data = data;

	return buf;
}

fs_Inode *
fs_inodeGet(fs_Context *context, int32 snapnum, uint64 inum, fs_InodeMode mode, uint32 option)
{
	fs_Inode *inode;
	fs_Buf *buf;

	if (option & INO_GET_OPT_NEW) {
		FS_ASSERT("get new inode" == NULL);
	}

	if (inum < FS_MAX_RESERVED_INODE) {
		buf = B_GET_SHARED(NULL, fs_firstInodeBlock(snapnum));
	}
        else {
		fs_Inode *inodes = I_GET_SHARED(snapnum, FS_INODE_INODE);
		buf = O_GET_SHARED(inodes, inum / FS_INODES_PER_PAGE);
		fs_inodeRelease(context,inodes);
	}

	if (buf == NULL
	 || buf->u.dinode->dinode[inum % FS_INODES_PER_PAGE].mode == 0x6c6c
	 || buf->u.dinode->dinode[inum % FS_INODES_PER_PAGE].mode == 0)
		return NULL;

	inode = NEW(fs_Inode);
	inode->dinode = NEW(fs_Dinode);
	inode->inum = inum;
	inode->snapnum = snapnum;
	bcopy(&buf->u.dinode->dinode[inum % FS_INODES_PER_PAGE], inode->dinode, sizeof (fs_Dinode));
	fs_bufRelease(context, buf);

	// printf("getting inode %lld\n", inode->inum);
	
	return inode;
}

fs_Buf *                       
fs_bufGetPage(fs_Context *context, fs_Inode *inode, uint64 offset, int32 mode, uint32 option)
{
	fs_Block block = fs_emap_lookup_block(context, inode, offset);
	if (!fs_blockIsValid(block))
		return NULL;
	return fs_bufGetBlock(context, inode, block, 0, 0);
}

void
fs_trace(fs_Context *context, int32 level, char *fmt, ...)
{
}

void
fs_bufRelease(fs_Context *context, fs_Buf *buf)
{
	free(buf->u.data);
	free(buf);
}

boolean
fs_inodeBtreeWasModified(
    fs_Context *context,
    fs_Inode *inode)
{
        return FALSE;
}

void
fs_inodeRelease(fs_Context *context, fs_Inode *inode)
{
	// printf("freeing inode %lld\n", inode->inum);
	free(inode->dinode);
	free(inode);
}

void
fs_inodeReleaseExclusive(fs_Context *context, fs_Inode *inode)
{
	free(inode->dinode);
	free(inode);
}

void
fs_bufReleaseExclusive(fs_Context *context, fs_Buf *buf)
{
	free(buf->u.data);
	free(buf);
}

void
fs_lockExclusive(fs_Context *context, fs_Lock *lock, nfx_spinlock_t *spin)
{
}

void
fs_unlockExclusive(fs_Context *context, fs_Lock *lock)
{
}

void
fs_lockShared(fs_Context *context, fs_Lock *lock, nfx_spinlock_t *spin)
{
}

void
fs_unlockShared(fs_Context *context, fs_Lock *lock)
{
}

void
fs_lockDemote(fs_Context *context, fs_Lock *lock, fs_LockMode newMode)
{
}

boolean
fs_lockHeldExclusive(fs_Context *context, fs_Lock *lock)
{
        return FALSE;
}

boolean
fs_lockOwnedExclusive(fs_Context *context, fs_Lock *lock)
{
        return FALSE;
}

void
fs_bufFree(fs_Context *context, fs_Buf *buf, int option)
{
	assert(0);
}

fs_Inode *
fs_allocateFileInode(fs_Context *context, fs_Inode *dir_inode,
    uint16 mode,
    uint32 uid,
    uint32 gid,
    uint32 filesetid)
{
	return fs_inodeGet(context, 0, 0, 0, INO_GET_OPT_NEW);
}

void
fs_deallocateInode(fs_Context *context, fs_Inode *inode)
{
}

void
fs_deallocateExtent(fs_Context *context, fs_Extent extent)
{
}

void
fs_truncateFile(fs_Context *context, fs_Inode *inode, uint64 newsize)
{
}

void
fs_inodeRemoveFromCache(fs_Context *context, fs_Inode *inode, fs_LogSummary *logSummary, boolean isReplay)
{
}

boolean fs_blockIsCopyOnWrite(fs_Context *context, fs_Block block)
{
        return FALSE;
}

boolean fs_bufIsCopyOnWrite(fs_Context *context, fs_Buf *buf)
{
        return FALSE;
}

void fs_bufCopyOnWriteMetadata(
    fs_Context *context,
    fs_Inode   *inode,
    fs_Buf     *buf)
{
}

boolean fs_bufWasModified(
    fs_Context  *context,
    fs_Buf      *buf)
{
        return FALSE;
}

boolean fs_inodeWasModified(
    fs_Context *context,
    fs_Inode *inode)
{
        return FALSE;
}

void fs_bufDirty(
    fs_Context *context,
    fs_Buf *buf)
{
}

noreturn fs_abort() {
	abort();
}

noreturn fs_panic(char *fmt,...) {
	abort();
}

noreturn fs_error(fs_Context *contexdt, int code, char *fmt,...) {
	abort();
}

void fs_trace_elog(
    fs_Context *context,
    int32      level,
    int        line,
    char*      file,
    char       *traceString)
{
}

char *
e_log_var_pa(char *format, ...)
{
	return 0;
}

void fs_deallocateBlock(fs_Context *context, fs_Block block)
{
}

elog_appl_data_t elog_appl_data[ELOG_MAX_APP];

static void btrace_signal_handler(int sig, siginfo_t *siginfo, void *context)
{
    btrace_print_signal(context, btrace_signal_handler);
    abort();
}

boolean fs_eek_query(fs_Context *context, char *fmt, ...)
{
	char buf[256];
	va_list ap;

	va_start(ap, fmt);
	vprintf(fmt, ap);
	va_end(ap);
	putchar(' ');
	fgets(buf, sizeof buf, stdin);
	return tolower(*buf) == 'y';
}

int
main(int argc, char *argv[])
{
	fs_Context c;
	fs_Context *context = &c;
        int i;
        struct sigaction sa;

        static int signals[] = { SIGSEGV, SIGBUS, SIGILL, SIGFPE, 0 };
    
        memset(&sa, 0, sizeof(struct sigaction));
        sa.sa_sigaction = btrace_signal_handler;
        sa.sa_flags = SA_SIGINFO;

        for (i = 0; signals[i] != 0; i++) {
                int sig = signals[i];
                if (sigaction(sig, &sa, NULL) == -1) {
                        char buf[BUFSIZ];
                        sprintf(buf, "sigaction(%d \"%s\")", sig,
                                sys_siglist[sig]);
                        perror(buf);
                        exit(1);
                }
        }

	if (argc != 2) {
		fprintf(stderr, "usage: eek volume\n");
		exit(1);
	}

	fs_bt_dispatch_init();

	volume_fd = open64(argv[1], 0); // readonly for now
	fs_eek(context);

	return 0;
}

#endif // SSC
