typedef struct fs_Eek fs_Eek;
typedef struct fs_Eek_linkcounts fs_Eek_linkcounts;
typedef struct fs_Eek_parent_linkcounts fs_Eek_parent_linkcounts;

struct fs_Eek {
    fs_Lock *lock;
    fs_Eek_linkcounts *linkcounts; 
    fs_Eek_parent_linkcounts *parent_linkcounts; 
    int passOneErrors;  // number of errors encountered during the walk it phase
	int errcount;       // number of errors encountered
    int fixcount;       // number of errors fixed
    int freecount;      // number of blocks free'd
                        // (used to determine if buffer cache needs to be flushed)
	byte inUse;		    // Eek is running; turn off logging
    byte defaultReply;  // always yes or no
	byte interactive;   // Eek is in interactive mode, ie modifications can be made
	byte sendToSsc;     // TRUE for message exchange with the SSC
	byte squelchSscPrintf;

	int nthreads;	// number of threads to spawn in mark_blocks
	int phase;	// phase to perform instead of entire eek
};

struct fs_Eek_linkcounts {
        fs_Eek_linkcounts *next;
        uint64 inum;
	uint64 parent;
        uint32 count;
	boolean visited;	// visited by fs_eek_compare_used_inodes()
};

typedef enum {
	FS_EEK_MARK_NODES = 1,
	FS_EEK_MARK_EXTENTS = 2,
	FS_EEK_MARK_NODES_AND_EXTENTS = 3, // HACK: 3 = 1|2,
} fs_Eekmark;

int fs_eek(fs_Context *context); /* {NFX_OK|NFS_ERR} */
int fs_eek_ref_vs_buddy(fs_Context *context); /* {NFX_OK|NFS_ERR} */
boolean fs_eek_mark(fs_Context *context, char *msg, uint8 *bitmap, uint64 bitnum, ...);
void fs_eek_unmark(fs_Context *context, uint8 *bitmap, uint64 bitnum);
void fs_eek_increment_linkcount(fs_Context *context, uint64 parent, uint64 inum);
void fs_eek_scavenge_parent_linkcounts(fs_Context *context, uint32 snapnum, boolean fixIt);
boolean fs_eek_query(fs_Context *context, char *fmt, ...);
int fs_eek_vquery(fs_Context *context, char *fmt, va_list ap);
void fs_eek_mark_dirty_and_cow(fs_Context *context, fs_Buf *buf);
boolean fs_eek_meta_inode(fs_Context *context, uint64 inum, int snapnum);
void fs_eek_print_status(fs_Context *context, boolean printDate, char *fmt, ...);
uint32 fs_eek_find_parent_linkcount(fs_Context *context, uint64 parent, uint64 inum);

