/*
 * Name:  fs-errstr.c
 *
 * RCS:  $Id: fs-errstr.c,v 1.4 2002/12/04 19:25:55 lanc Exp $
 *
 * Description: fs error codes to error strings conversion
 * 
 * Created By:  Laurent Brard
 *
 * Date Created:  09/10/2002
 */
 
#include "fs-includes.h"

char *fs_errCodeToString(fs_Status status)
{
        
    switch(status)  {
        case FS_ERR_BADPATH:
            return("bad path");
        case FS_ERR_GENERIC:
            return("generic error");
        case FS_ERR_NONE:
            return("no error");
        case FS_ERR_PERM:
            return("permission denied");
        case FS_ERR_NOENT:
            return("does not exist");
        case FS_ERR_IO:
            return("I/O error occured");
        case FS_ERR_ACCESS:
            return("access denied");
        case FS_ERR_EXIST:
            return("already exists");
        case FS_ERR_NOTDIR:
            return("not a directory");
        case FS_ERR_ISDIR:
            return("is a directory");
        case FS_ERR_INVAL:
            return("invalid request");
        case FS_ERR_FBIG:
            return("file too big / too many objects");
        case FS_ERR_NOSPC:
            return("no space left on device");
        case FS_ERR_ROFS:
            return("read-only file system");
        case FS_ERR_NAMETOOLONG:
            return("name too long");
        case FS_ERR_NOTEMPTY:
            return("not empty");
        case FS_ERR_DQUOT:
            return("disk quota exceeded");
        case FS_ERR_STALE:
            return("stale handle");
        case FS_ERR_BADHANDLE:
            return("bad handle");
        case FS_ERR_NOT_SYNC:
            return("not sync");
        case FS_ERR_BAD_COOKIE:
            return("bad cookie");
        case FS_ERR_NOTSUPP:
            return("not supported");
        case FS_ERR_TOOSMALL:
            return("too small");
        case FS_ERR_SERVERFAULT:
            return("server fault");
        case FS_ERR_BADTYPE:
            return("bad type");
        case FS_ERR_BUSY:
            return("busy");
        case FS_ERR_NOT_MOUNTED:
            return("volume not mounted");
        case FS_ERR_ALREADY_MOUNTED:
            return("volume already mounted");
        case FS_ERR_NO_RESOURCE:
            return("not enough resources");
        case FS_IN_PROGRESS:
            return("command still in progress");
        case FS_ERR_SNAP_EXISTS:
            return("snapshot already exists");
        case FS_ERR_SNAP_NO_EXIST:
            return("snapshot does not exist");
        case FS_ERR_SNAP_MAX:
            return("maximum number of snapshots already exist");
        case FS_ERR_SNAP_INVAL_NAME:
            return("cannot used reserved names (hourly., daily., weekly.) for snapshot operation");
        case FS_ERR_SNAP_NOT_ENABLED:
            return("snapshot operation not allowed");
        case FS_ERR_SNAP_PIN:
            return("cannot remove a pinned snapshot");
        default:
            return("no string for this error");
    }
}    
        


