/*
 * Name		: fs-incrdump.c
 * RCS		: $Id: fs-incrdump.c,v 1.79 2002/11/20 01:05:14 rahulk Exp $
 * Description	: Dump implementation
 * Created by	: Rahul Khona
 * Date Created	: 8/10/2001
 */

#include "fs-includes.h"
#include "fs-dump.h"
#include "fs-ndmp-api.h"
#include <exports/ndmp-fs-api.h>

#if !defined(SSC)

#define D   printf

#define testSpecialDirectories(dirp)    \
    {                               \
        if (strcmp(dirp->name, ".") == 0                                \
         || strcmp(dirp->name, "..") == 0                               \
         || strcmp(dirp->name, ".snapshot") == 0)                       \
        {                                                               \
            FS_READDIR_ADV_ENTRY_PTR(dirp);                             \
            continue;                                                   \
        }                                                               \
    }


PRIVATE fs_Status
fs_findCommonPath(
    fs_Context          *context,
    fs_DumpContext_t    *dumpContext,
    fs_Inode            *inode,
    char                **commonPath,
    int                 *isTmp);

PRIVATE void fs_getNewPath(fs_Context *, fs_DumpContext_t *, fs_Inode *,
                           char **, int *, int);
PRIVATE fs_Status fs_getAdjustedOldPath(fs_Context *, fs_DumpContext_t *,
                                        fs_Inode *, fs_Inode *, char **,
                                        int *, int *);
PRIVATE fs_Status fs_getName(fs_Context *, fs_DumpContext_t *, fs_Inode *,
                             fs_Inode *, char **, int *, uint64 *);
PRIVATE fs_Status fs_doRename(fs_Context *, fs_DumpContext_t *, fs_Inode *,
                              fs_Inode *, fs_Inode *, char *, char *, int);
PRIVATE fs_Status fs_doUnlink(fs_Context *, fs_DumpContext_t *, fs_Inode *,
                              fs_Inode *, char *, int, int);
PRIVATE fs_Status fs_doLink(fs_Context *, fs_DumpContext_t *, fs_Inode *,
                              fs_Inode *, char *, int);
PRIVATE fs_Status fs_removeUnwantedNodes(fs_Context *, fs_DumpContext_t *,
                                         fs_Inode *, fs_Inode *, char *, int);
PRIVATE fs_Status fs_dumpRename(fs_Context *, fs_DumpContext_t *, fs_Inode *,
                                char *, char *, int, int, int);
PRIVATE fs_Status fs_dumpRemove(fs_Context *, fs_DumpContext_t *, fs_Inode *,
                                char *, int);
PRIVATE fs_Status fs_dumpCreate(fs_Context *, fs_DumpContext_t *, fs_Inode *,
                                char *, int, int, int);
PRIVATE fs_Status fs_dumpLink(fs_Context *, fs_DumpContext_t *, fs_Inode *,
                                char *, int, int);
PRIVATE fs_Status fs_dumpName(fs_Context *, fs_DumpContext_t *, fs_Inode *,
                                char *, int);
PRIVATE fs_Status fs_dumpNonContainerInode(fs_Context *, fs_DumpContext_t *,
                                           fs_Inode *, fs_Inode *, char *,
                                           int, int);
PRIVATE fs_Status fs_dumpRegExtents(fs_Context *, fs_DumpContext_t *,
                                    fs_Inode *);
PRIVATE fs_Status fs_dumpAttr(fs_Context *, fs_DumpContext_t *, fs_Inode *,
                              fs_Inode *, char *, int);
PRIVATE fs_Status fs_walkTree(fs_Context *, fs_DumpContext_t *,
                              fs_Inode *, fs_Inumber , char *, int, int *);
PRIVATE fs_Status
fs_sendIncrFileHistory(fs_Context *context,
                       fs_DumpContext_t *dumpContext,
                       fs_Inode *rootInode,
                       int *sentParentHistory,
                       int *sentHistory,
                       uint64   *block,
                       uint16   *boff);


/*
 * For a given inode, get the path to it from the root in the
 * new snapshot
 */

PRIVATE void
fs_getNewPath(
    fs_Context          *context,
    fs_DumpContext_t    *dumpContext,
    fs_Inode            *inode,
    char                **path,
    int                 *isTmp,
    int                 isUnlink
)
{

    fs_Status           status = FS_ERR_NONE;
    fs_Inode            *parent = NULL, *t = inode;
    fs_Inumber          parentInum;
    char                *cp, *tp = *path;
    uint16              snapnum = dumpContext->snapnum;
    int                 len;

    if (isUnlink) {
        snapnum = inode->snapnum;
    }

    /*
     * If have been using the cpu since a long time, give others a chance
     */

    fs_yieldPeriodic(context);
    FS_TRACE(context, FS_TRACE_DUMP_RESTORE, ("In getNewPath"));
    FS_TRACE(context, FS_TRACE_DUMP_RESTORE, ("In getNewPath - inod is %lld",
                                      inode->inum));
    FS_TRACE(context, FS_TRACE_DUMP_RESTORE, ("getNewPath: path = %s", tp));

    /*
     * If this is a root, return /
     */

    if (inode->inum == FS_ROOT_INODE) {
        tp--;
        *tp = '/';
        goto done;
    }
    while (status == FS_ERR_NONE) {

        /*
         * Tricky. Here we are checking whether an ancestor is
         * temp or not. We don't check for the full path being temp
         * because we don't support hard links to directories
         */

        if (fs_isInodeTemp(context, dumpContext, t)) {
            *isTmp = 1;
        }

        /*
         * Find the parent of the inode in new path
         */
        status = fs_findParent(context, snapnum, t->inum, &parentInum, NULL);
        if (status != FS_ERR_NONE) {
            if (t != inode) {
                fs_inodeRelease(context, t);
            }
            continue;
        }

        FS_TRACE(context, FS_TRACE_DUMP_RESTORE,
            ("getNewPath: inum = %lld parentInum = %lld",
             t->inum, parentInum));
        parent = I_GET_SHARED(snapnum, parentInum);
        if (parent == NULL) {
            FS_TRACE(context, FS_TRACE_DUMP_RESTORE,
                     ("getNewPath - parent == null"));
            FS_ASSERT(t->inum == FS_ROOT_INODE);
            if (t != inode) {
                fs_inodeRelease(context, t);
            }
             tp--;
             *tp = '/';
            FS_TRACE(context, FS_TRACE_DUMP_RESTORE,
                      ("getNewPath: path = %s", tp));
            break;
        }
        status = fs_getName(context, dumpContext, parent, t, &cp, &len, NULL);
        if (t != inode) {
            fs_inodeRelease(context, t);
        }

        
        FS_TRACE(context, FS_TRACE_DUMP_RESTORE,
            ("getNewPath: inum = %lld parentInum = %lld name %s",
                t->inum, parentInum, cp));

        FS_ASSERT(status == FS_ERR_NONE);
        if (cp) {
            tp--;
            *tp = '/';
            tp -= len;
            strncpy(tp, cp, len);
            FS_TRACE(context, FS_TRACE_DUMP_RESTORE,
                      ("getNewPath: path = %s", tp));
            fs_ramDealloc(FS_RAM_DUMP_NAME, cp);
        }
        t = parent;
        parent = NULL;
        if (parentInum == FS_ROOT_INODE) {
            if (t != inode) {
                fs_inodeRelease(context, t);
            }
            tp--;
            *tp = '/';
            break;
        }

    }

    if (parent) {
        fs_inodeRelease(context, parent);
    }

done:
    *path = tp;
    FS_TRACE(context, FS_TRACE_DUMP_RESTORE,
              ("getNewPath: returning %s for inum %lld", tp, inode->inum));
    FS_TRACE(context, FS_TRACE_DUMP_RESTORE,
             ("getNewPath: end - path = %s", *path));
    return;
}

/*
 * Adjust the path of the parent in the source tree before issuing
 * a rename or remove
 *
 * The way incremnetal dump works is that we take a source tree and
 * generate file system operations which would be applied by rename
 * in sequence to transform the tree to the target. At every stage
 * when we issue an operation to transform the old tree to the new
 * tree, we have to be consider the fact that pervious transforms
 * that we have dumped would have brought the tree during restore 
 * stage to an intermediate step. If the intermediate step had the
 * the object that we want to rename in a differnet place, then
 * the move/remove operation should be targeted to this stage for 
 * e.g.
 *
 * Consider following source and destination trees
 *
 *   a
 *   |__ b
 *       |__ c
 *
 * Target tree is 
 *
 *   c
 *   |__a
 *   |__b
 *
 * The dump operations would generate following message in the order shown
 * in this comment
 * (i) mv /a/b/c /
 * (ii) mv /a /c/
 * (iii) mv /a/b /c/ - Note at this point the intermediate tree would 
 * look as follows
 *
 *  c
 *  |_a
 *    |_b
 *
 * The mv /a/b would thus fail. We need to adjust the path to 'b' appropriately
 * before issuing the move message
 *
 * (iii)' should be mv /c/a/b /c/ 
 *
 * The idea is that for a given pathname in the source tree, find the
 * closest ancestor to the last component, including the last component
 * that has been renamed. Replace the path including that ancestor with
 * the new path
 */

PRIVATE fs_Status
fs_getAdjustedOldPath(
    fs_Context          *context,
    fs_DumpContext_t    *dumpContext,
    fs_Inode            *inode,
    fs_Inode            *parent,
    char                **path,
    int                 *pathLen,
    int                 *isTmp
)
{
    char                *buf;
    char                *cp, *np;
    fs_Status           status = FS_ERR_NONE;
    fs_Inode            *t = NULL, *p, *i;
    fs_Inumber          parentInum = parent->inum;
    int                 len;

    FS_TRACE(context, FS_TRACE_DUMP_RESTORE,
             ("In getAdjustedOldPath parent %lld inode %lld", inode->inum,
             parent->inum));

    /*
     * If have been using the cpu since a long time, give others a chance
     */

    fs_yieldPeriodic(context);
    *path = NULL;
    buf = fs_ramAlloc(FS_RAM_DUMP_NAME, FS_MAXPATHLEN + 1);
    FS_ASSERT(buf != NULL);
    cp = &buf[FS_MAXPATHLEN + 1];
    *cp = '\0';

    /*
     * Get yet another shared lock on the inodes, so that
     * lock accounting is easier to implement
     */

    i = I_GET_SHARED(dumpContext->prevSnapNum, inode->inum);
    p = I_GET_SHARED(dumpContext->prevSnapNum, parent->inum);
    status = fs_getName(context, dumpContext, parent, inode, &np, &len, NULL);
    if (status != FS_ERR_NONE) {
        fs_inodeRelease(context, p);
        fs_inodeRelease(context, i);
        fs_ramDealloc(FS_RAM_DUMP_NAME, buf);
        return status;
    }
    cp -= len;
    cp -= 1;
    *cp = '/';
    strncpy((cp + 1), np, len);
    fs_ramDealloc(FS_RAM_DUMP_NAME, np);

    // RAHUL - Handle the parent immediately
    if (fs_isInodeRenamed(context, dumpContext, p)) {
            FS_TRACE(context, FS_TRACE_DUMP_RESTORE,
                     ("In getAdjustedOldPath immediate parent is renamed"));
            fs_getNewPath(context, dumpContext, p, &cp, isTmp, 0);
            fs_inodeRelease(context, i);
            fs_inodeRelease(context, p);
            np = fs_ramAlloc(FS_RAM_DUMP_NAME, strlen(cp) + 1);
            strcpy(np, cp);
            *path = np;
            FS_TRACE(context, FS_TRACE_DUMP_RESTORE,
                     ("In getAdjustedOldPath path after renamed parent %s",
                       *path));
            fs_ramDealloc(FS_RAM_DUMP_NAME, buf);
            return status;
    }

    while (status == FS_ERR_NONE && parentInum != FS_ROOT_INODE) {
        status = fs_findParent(context, dumpContext->prevSnapNum, p->inum,
                               &parentInum, NULL);
        if (status != FS_ERR_NONE) {
            fs_inodeRelease(context, p);
            fs_inodeRelease(context, i);
            fs_ramDealloc(FS_RAM_DUMP_NAME, buf);
            return status;
        }
        t = I_GET_SHARED(dumpContext->prevSnapNum, parentInum);

        fs_inodeRelease(context, i);
        i = p;
        p = t;
        status = fs_getName(context, dumpContext, p, i, &np, &len, NULL);
        if (status != FS_ERR_NONE) {
            fs_inodeRelease(context, p);
            fs_inodeRelease(context, i);
            fs_ramDealloc(FS_RAM_DUMP_NAME, buf);
            return status;
        }

        /*
         * Copy the name in the buffer
         */
        cp -= len;
        cp -= 1;
        *cp = '/';
        strncpy((cp + 1), np, len);
        fs_ramDealloc(FS_RAM_DUMP_NAME, np);

        /*
         * If the parent is renamed get its new path
         */

#if 1
        // RAHUL
        if (fs_isInodeRenamed(context, dumpContext, p)) {
            fs_getNewPath(context, dumpContext, p, &cp, isTmp, 0);
            fs_inodeRelease(context, i);
            fs_inodeRelease(context, p);
            np = fs_ramAlloc(FS_RAM_DUMP_NAME, strlen(cp) + 1);
            strcpy(np, cp);
            *path = np;
            FS_TRACE(context, FS_TRACE_DUMP_RESTORE,
                     ("In getAdjustedOldPath path after renamed ancestor %s",
                       *path));
            fs_ramDealloc(FS_RAM_DUMP_NAME, buf);
            return status;
        }
#endif
    }

    fs_inodeRelease(context, p);
    fs_inodeRelease(context, i);
    *pathLen = strlen(cp);
    np = fs_ramAlloc(FS_RAM_DUMP_NAME, *pathLen + 1);
    strcpy(np, cp);
    *path = np;
    fs_ramDealloc(FS_RAM_DUMP_NAME, buf);
    return status;
}

/*
 * Get the name of the inode in the dir, starting search at a given
 * offset in the dir. Hint is to be used only while searching for
 * harlinks in a directory and the scan should be performed starting from
 * beging to end of dir
 */

PRIVATE fs_Status
fs_getName(
    fs_Context          *context,
    fs_DumpContext_t    *dumpContext,
    fs_Inode            *parent,
    fs_Inode            *inode,
    char                **name,
    int                 *nameLen,
    uint64              *hint
)
{
    char                buf[2 * FS_MAXPATHLEN];
    fs_DirEntry         *dirents = NULL;
    fs_Status           status = FS_ERR_NONE;
    uint64              cookie = 0;
    int                 eof = 0;
    uint32              numEntries = 0;
    uint32              i;
    int                 found = 0;
    char                *cp;

    /*
     * If have been using the cpu since a long time, give others a chance
     */

    fs_yieldPeriodic(context);
    *name = NULL;
    if (hint) {
        cookie = *hint;
    }

    FS_TRACE(context, FS_TRACE_DUMP_RESTORE,
             ("Parent Inode - %lld - snap %lld",
                                     parent->inum, (uint64)parent->snapnum));
    FS_TRACE(context, FS_TRACE_DUMP_RESTORE,
              ("In GetName - cookie %lld", cookie));
    while (!eof && status == FS_ERR_NONE) {
        dirents = (fs_DirEntry *)buf;
        fs_dir_readdir(context, parent, dirents, 2 * FS_MAXPATHLEN,
                                cookie, FALSE, &numEntries, &eof);
        for (i = 0;  i < numEntries; i++) {
            testSpecialDirectories(dirents);
            cookie = dirents->cookie;
            FS_TRACE(context, FS_TRACE_DUMP_RESTORE, ("fs_getName: %s",
                                              dirents->name));
            if (dirents->inode == inode->inum) {
                found++;
                break;
            }
            FS_READDIR_ADV_ENTRY_PTR(dirents);
        }
        if (found) {
            break;
        }
    }

    if (!found) {
        FS_TRACE(context, FS_TRACE_DUMP_RESTORE,
              ("could not find entry for %lld",
                                         inode->inum));
        FS_TRACE(context, FS_TRACE_DUMP_RESTORE,
              ("parent is %lld", parent->inum));
        return FS_ERR_NOENT;
    }

    FS_TRACE(context, FS_TRACE_DUMP_RESTORE,
              ("Allocating %lld", (uint64)dirents->nameLen));
    cp = fs_ramAlloc(FS_RAM_DUMP_NAME, dirents->nameLen);
    FS_TRACE(context, FS_TRACE_DUMP_RESTORE, ("Allocated %lld",
                                     (uint64)dirents->nameLen));
    strcpy(cp, dirents->name);
    //Optimization, remove following when tested
    //strncpy(cp, dirents->name, dirents->nameLen);
    //cp[dirents->nameLen] = '\0';
    if (hint) {
        *hint = cookie;
    }
    *name = cp;
    *nameLen = dirents->nameLen - 1;    //remove ending '\0' from length
    return status;
}

PRIVATE fs_Status
fs_doRename(
    fs_Context          *context,
    fs_DumpContext_t    *dumpContext,
    fs_Inode            *inode,
    fs_Inode            *newParent,
    fs_Inode            *oldParent,
    char                *newName,
    char                *oldName,   //NOT USED !!!
    int                 newNameLen
)
{

    int                 sameParent = 0;
    int                 newlyCreatedParent;
    fs_Inode            *shadowParent = NULL;
    fs_Inode            *searchInode;
    fs_Inumber          collisionInum = 0;
    fs_Inode            *collisionSourceInode;
    fs_Inode            *collisionTargetInode;
    char                *path;
    int                 isTmp = 0;
    int                 pathLen;
    fs_Status           status = FS_ERR_NONE;

    /*
     * If have been using the cpu since a long time, give others a chance
     */

    fs_yieldPeriodic(context);
    FS_TRACE(context, FS_TRACE_DUMP_RESTORE,
        ("fs_doRename: newparent %lld oldparent %lld",
         newParent->inum, oldParent->inum));
    FS_TRACE(context, FS_TRACE_DUMP_RESTORE,
        ("fs_doRename: newName %s oldName %s", newName, oldName));

    /*
     * Determine whether the parent is still the same
     */

    if (newParent->inum == oldParent->inum &&
        newParent->dinode.genNum == oldParent->dinode.genNum) {
        sameParent = 1;
    } else {
        /*
         * Determine whether the new parent was created after the
         * previous snapshot
         */

        shadowParent = I_GET_SHARED(dumpContext->prevSnapNum, newParent->inum);
        if (shadowParent == NULL) {
            newlyCreatedParent = 1;
        } else if (shadowParent->dinode.genNum != newParent->dinode.genNum ||
            shadowParent->dinode.attrs == 0) {
            newlyCreatedParent = 1;
            fs_inodeRelease(context, shadowParent);
            shadowParent = NULL;
        }
    }

    /*
     * Detemrine whether the new name existed and is not yet renamed or
     * removed. If there is such a name collision, then the inode has
     * to be moved to a temporary location. After restore has processed
     * all the records, it will move the temporary objects back to their
     * final locations
     */

    if (sameParent) {
        searchInode = oldParent;
    } else {
        searchInode = shadowParent;
    }

    if (searchInode != NULL) {
        collisionInum = fs_dir_lookupent(context, searchInode, newName, 
                                            newNameLen);
    }

    if (collisionInum != 0) {
            FS_TRACE(context, FS_TRACE_DUMP_RESTORE,
                     ("fs_doRename: Collision inum = %lld\n", collisionInum));
        /*
         * There is potentially a name collision in the new parent.
         * Determine whether the inode has been removed in the target tree
         * or it has already been renamed. If so, then we can perform
         * a move to the target location otherwise we have to move it
         * to a temporary location
         */

        collisionSourceInode = I_GET_SHARED(dumpContext->prevSnapNum,
                                            collisionInum);
        collisionTargetInode = I_GET_SHARED(dumpContext->snapnum,
                                            collisionInum);

		if (collisionTargetInode == NULL ||
			(collisionTargetInode->dinode.genNum !=
			 collisionSourceInode->dinode.genNum) ||
			collisionTargetInode->dinode.attrs == 0) {

            /*
             * Collision inode is removed in the target. We can
             * freely move it to the new location. We will be removing
             * objects only after this pass, so we need to move
             * to temp. If we decide to break removes into 2 passes
             * and the removal is already performed, then we can
             * rename in place.
             */

        } else {

            /*
             * Determine if the collision inode is already renamed
             */

            if (FS_I_IS_DIR(collisionTargetInode->dinode.attrs)) {
#if 0
                if (fs_isInodeRenamed(context, dumpContext,
                    collisionTargetInode)) {
                    collisionInum = 0;
                }
#endif
            } else {

                /*
                 * We cannot determine whether the target inode's remaning
                 * is already processed at this point or not, so play it
                 * safe and move it to temporary directory till end of
                 * restore
                 */
            }
        }
		if (collisionTargetInode) {
				fs_inodeRelease(context, collisionTargetInode);
		}
        fs_inodeRelease(context, collisionSourceInode);
    }

    /*
     * Now get the adjusted source path for rename. If the closest
     * ancestor to the target inode is already moved, then the
     * intermediate tree during restore would have this inode in the
     * new location. We need to determine if that is the case and if
     * so, then the rename operation that is to be generated has to
     * have an adjusted source path
     */

    status = fs_getAdjustedOldPath(context, dumpContext, inode,
                                       oldParent, &path, &pathLen, &isTmp);

    if (status == FS_ERR_NONE) {
        int doTemp = (collisionInum != 0 || isTmp);
        status = fs_dumpRename(context, dumpContext, inode, path, 
                                dumpContext->currentName, pathLen, 
                                strlen(dumpContext->currentName), doTemp);
                                //(collisionInum != 0  || isTmp)? 1 : 0);
        //if (doTemp) {
        fs_markInodeRenamed(context, dumpContext, inode, doTemp);
        //}
        fs_dumpName(context, dumpContext, inode, dumpContext->currentName,
                    doTemp);

    }

    if (shadowParent != NULL) {
        fs_inodeRelease(context, shadowParent);
    }

    if (path != NULL) {
        fs_ramDealloc(FS_RAM_DUMP_NAME, path);
    }

    return status;
}

/*
 * Unlink a directory entry. We are walking the source, tree, so
 * we need to adjust the path, before we can issue an unlink request
 */

PRIVATE fs_Status
fs_doUnlink(
    fs_Context          *context,
    fs_DumpContext_t    *dumpContext,
    fs_Inode            *parent,
    fs_Inode            *inode,
    char                *name,
    int                 nameLen,
    int                 force
)
{
    char            buf[FS_MAXPATHLEN + 1];
    int             isTmp = 0;
    char            *cp;
    fs_Inode        *newParent, *collisionInode;
    fs_Inumber      collisionInum;
    fs_Status       status = FS_ERR_NONE;

    /*
     * If have been using the cpu since a long time, give others a chance
     */

    FS_TRACE(context, FS_TRACE_DUMP_RESTORE, ("In fs_doUnlink for %s", name));
    fs_yieldPeriodic(context);
//    FS_TRACE(context, FS_TRACE_DUMP_RESTORE, ("In fs_doUnlink"));

    buf[FS_MAXPATHLEN + 1] = '\0';
    cp = &buf[FS_MAXPATHLEN + 1];
    cp -= nameLen;
    strncpy(cp, name, nameLen);

    if (force && inode->dinode.linkCnt == 1) {
        /*
         * Find the new path to its parent. If parent is removed, we would
         * not be here
         */

        FS_TRACE(context, FS_TRACE_DUMP_RESTORE,
                 ("Calling remove because link count is 1"));
        fs_getNewPath(context, dumpContext, parent, &cp, &isTmp, 1);
        status = fs_dumpRemove(context, dumpContext, inode, cp, isTmp);
        return status;
    }

    /*
     * The uncommon case of link. We need to determine whether the
     * link needs to be removed or not. The way to detect that is
     * to look into the parent in the new snapshot, if the link does not
     * exist remove it.
     *
     * If the parent does not exist in the new tree, then definitely
     * the link has to be removed
     */

    newParent = I_GET_SHARED(dumpContext->snapnum, parent->inum);

    if (newParent &&
		newParent->dinode.genNum == parent->dinode.genNum) {

        /*
         * Parent exists
         */
        collisionInum = fs_dir_lookupent(context, newParent, name, nameLen);
        if (collisionInum == 0) {
            /*
             * Definately the entry needs to be removed
             */
            fs_getNewPath(context, dumpContext, parent, &cp, &isTmp, 1);
            fs_inodeRelease(context, newParent);
            FS_TRACE(context, FS_TRACE_DUMP_RESTORE,
                     ("Calling remove because collisionInum is 0"));
            status = fs_dumpRemove(context, dumpContext, inode, cp, isTmp);
            return status;
        }

        collisionInode = I_GET_SHARED(dumpContext->snapnum, collisionInum);
        if (inode->inum == collisionInum && 
            collisionInode->dinode.genNum == inode->dinode.genNum) {

            /*
             * Entry still exists, we don't need to unlink
             */

            fs_inodeRelease(context, collisionInode);
            fs_inodeRelease(context, newParent);

            /*
             * If this link is already dumped, at it existed, then
             * we would have linked it in the temp tree to a new inode
             * remove the link.
             */

            fs_getNewPath(context, dumpContext, parent, &cp, &isTmp, 1);

            if (fs_isInodeLinked(context, dumpContext, inode, cp)) {
                status = fs_dumpRemove(context, dumpContext, inode, cp, isTmp);
            }

            return status;
        }

        FS_TRACE(context, FS_TRACE_DUMP_RESTORE,
            ("proceeding to unlink %s c-gennum %d i-gennum %d",
            name, collisionInode->dinode.genNum, inode->dinode.genNum));
        FS_TRACE(context, FS_TRACE_DUMP_RESTORE,
            ("proceeding to unlink %s c-inum %lld i-inum %lld",
            name, collisionInode->inum, inode->inum));
        /*
         * We need to unlink.
         */
        fs_inodeRelease(context, collisionInode);
        fs_inodeRelease(context, newParent);
        fs_getNewPath(context, dumpContext, parent, &cp, &isTmp, 1);
        status = fs_dumpRemove(context, dumpContext, inode, cp, isTmp);
        return status;
    } else {
        FS_TRACE(context, FS_TRACE_DUMP_RESTORE,
                 ("Calling remove because parent is removed"));
        fs_getNewPath(context, dumpContext, parent, &cp, &isTmp, 1);
        status = fs_dumpRemove(context, dumpContext, inode, cp, isTmp);
	    if (newParent) {
			fs_inodeRelease(context, newParent);
	    }
        return status;
    }

    /*
     * RESTORE IS NOT DOING THE REMOVE 
     * Parent no longer exists as well. Since we are walking down
     * the source tree now, if the parent is removed, we would not
     * be here as we would have dumped a message to remove parent and
     * all its children
     */

	if (newParent) {
			fs_inodeRelease(context, newParent);
	}
    return status;
}

/*
 * Create a link. If the file is newer than the snapshot and is not
 * yet dumped, dump creation record, otherwise dump link records.
 * If there are any name collisions because an entry in the directory
 * existed with the same name (it was either removed or renamed before
 * this entry was created), determine, if the entry can be created now
 * or has to be defered and created in temporary directory
 */

PRIVATE fs_Status
fs_doLink(
    fs_Context          *context,
    fs_DumpContext_t    *dumpContext,
    fs_Inode            *parent,
    fs_Inode            *inode,
    char                *name,
    int                 nameLen
)
{

    fs_Inode            *shadowParent = NULL;
    fs_Inumber          collisionInum = 0;
    fs_Inode            *collisionInode;
    char                buf[FS_MAXPATHLEN + 1];
    char                *cp;
    int                 isTmp = 0, isTmpOld = 0, len;
    fs_Status           status = FS_ERR_NONE;
    char                *commonPath = NULL;

    FS_ASSERT(strlen(name) == nameLen);
    /*
     * If have been using the cpu since a long time, give others a chance
     */

    fs_yieldPeriodic(context);
    FS_TRACE(context, FS_TRACE_DUMP_RESTORE,
               ("Link %s to %lld", name, inode->inum));

    status = fs_findCommonPath(context, dumpContext, inode, &commonPath,
                                &isTmpOld);
    if (status != FS_ERR_NONE && status != FS_ERR_NOENT) {
        if (commonPath) {
            fs_ramDealloc(FS_RAM_DUMP_NAME, commonPath);
            return status;
        }
    }
    buf[FS_MAXPATHLEN + 1] = '\0';
    cp = &buf[FS_MAXPATHLEN + 1];
    cp -= nameLen;
    strncpy(cp, name, nameLen);

    /*
     * Determine whether there is a collision in this directory
     */

    shadowParent = I_GET_SHARED(dumpContext->prevSnapNum, parent->inum);

    if (!shadowParent ||
		shadowParent->dinode.genNum != parent->dinode.genNum) {

        FS_TRACE(context, FS_TRACE_DUMP_RESTORE, ("In link step 1 for %s", name));
        /*
         * Parent was created after the previous snapshot. Since the
         * name already exists, there isn't any collision. Either dump
         * link creation or object creation record directly
         */
        fs_getNewPath(context, dumpContext, parent, &cp, &isTmp, 0);
        len = strlen(cp);
        if (!fs_isInodeCreated(context, dumpContext, inode)) {

            if (commonPath) {
                fs_markInodeCreated(context, dumpContext, inode, commonPath, 
                                    strlen(commonPath), isTmp);
                status = fs_dumpLink(context, dumpContext, inode, cp, len,
                                    isTmp);
                fs_markLinkCreated(context, dumpContext, inode, cp, len, isTmp);
                //fs_markInodeCreated(context, dumpContext, inode, cp, isTmp);
                if (shadowParent) {
                        fs_inodeRelease(context, shadowParent);
                }
                fs_ramDealloc(FS_RAM_DUMP_NAME, commonPath);
                commonPath = NULL;
            } else {
                status = fs_dumpCreate(context, dumpContext, inode, cp, len,
                                       isTmp, (inode->dinode.fsetNum !=
                                               parent->dinode.fsetNum));
                fs_markInodeCreated(context, dumpContext, inode, cp, len,
                                    isTmp);
                if (shadowParent) {
                        fs_inodeRelease(context, shadowParent);
                }
            }
        } else {
            status = fs_dumpLink(context, dumpContext, inode, cp, len, isTmp);
            fs_markLinkCreated(context, dumpContext, inode, cp, len, isTmp);
            //fs_markInodeCreated(context, dumpContext, inode, cp, isTmp);
			if (shadowParent) {
					fs_inodeRelease(context, shadowParent);
			}
        }
        return status;
    }

    /*
     * Parent existed before the previous dump as well. We need to
     * determine if this link is new or not
     */
    collisionInum = fs_dir_lookupent(context, shadowParent, name, nameLen);

    if (collisionInum == 0) {

        FS_TRACE(context, FS_TRACE_DUMP_RESTORE, 
                 ("In link step 2 for %s", name));
        /*
         * Nothing by the same name existed in this directory.
         * Create the link
         */
        fs_getNewPath(context, dumpContext, parent, &cp, &isTmp, 0);
        len = strlen(cp);
#if 1
        if (!fs_isInodeCreated(context, dumpContext, inode)) {
            if (commonPath) {
                fs_markInodeCreated(context, dumpContext, inode,
                                    commonPath, strlen(commonPath), isTmp);
                status = fs_dumpLink(context, dumpContext, inode, cp, len,
                                    isTmp);
                fs_markLinkCreated(context, dumpContext, inode, cp, len, isTmp);
                //fs_markInodeCreated(context, dumpContext, inode, cp, isTmp);
                if (shadowParent) {
                        fs_inodeRelease(context, shadowParent);
                }
                fs_ramDealloc(FS_RAM_DUMP_NAME, commonPath);
                commonPath = NULL;

            } else {
                status = fs_dumpCreate(context, dumpContext, inode, cp, len,
                                        isTmp, (inode->dinode.fsetNum !=
                                               parent->dinode.fsetNum));
                fs_markInodeCreated(context, dumpContext, inode, cp, len, 
                                    isTmp);
                fs_inodeRelease(context, shadowParent);
            }
        } else {
            status = fs_dumpLink(context, dumpContext, inode, cp, len, isTmp);
            fs_markLinkCreated(context, dumpContext, inode, cp, len, isTmp);
            fs_inodeRelease(context, shadowParent);
        }
#else
            status = fs_dumpLink(context, dumpContext, inode, cp, len, isTmp);
            fs_markLinkCreated(context, dumpContext, inode, cp, len, isTmp);
            fs_inodeRelease(context, shadowParent);
#endif
        return status;
    }
    fs_inodeRelease(context, shadowParent);
	shadowParent = NULL;

    /*
     * There is a name collision, determine whether we are refering to
     * the same inode or not
     */

    collisionInode = I_GET_SHARED(dumpContext->prevSnapNum, collisionInum);
    if (collisionInode->inum == inode->inum &&
        collisionInode->dinode.genNum == inode->dinode.genNum) {

        FS_TRACE(context, FS_TRACE_DUMP_RESTORE, ("In link step 3 for %s", name));
        /*
         * Link already existed, we don't need to do anything. Just mark
         * the incore structure that this inode is seen
         */
        fs_inodeRelease(context, collisionInode);

        if (inode->dinode.linkCnt > 1) {

            /*
             * We could be called to create this inode since its ctime
             * was modified and don't need any action
             */

#if 0
            /*
             * On the contrary, if the inode is already created, link to a
             * temporary path. removeUnwantedInodes would ensure that we
             * remove the original path and relink
             */
            isTmp = 1;
            fs_getNewPath(context, dumpContext, parent, &cp, &isTmp, 0);
            if (!fs_isInodeCreated(context, dumpContext, inode)) {

                status = fs_dumpCreate(context, dumpContext, inode, cp, isTmp,
                                           (inode->dinode.fsetNum !=
                                           parent->dinode.fsetNum));
                fs_markInodeCreated(context, dumpContext, inode, cp, isTmp);
            } else {
                status = fs_dumpLink(context, dumpContext, inode, cp, isTmp);
                fs_markLinkCreated(context, dumpContext, inode, cp, isTmp);
            }
#endif

        }
        return status;
    }

    /*
     * There is collsion of names in this directory. Determine if the
     * previous inode is already processed
     */

#if 0
    if (fs_isInodeRenamed(context, dumpContext, collisionInode)) {
        FS_TRACE(context, FS_TRACE_DUMP_RESTORE, ("In link step 4 for %s", name));
        fs_inodeRelease(context, collisionInode);

        /*
         * We can create the link to the final place
         */

        fs_getNewPath(context, dumpContext, parent, &cp, &isTmp, 0);
        if (!fs_isInodeCreated(context, dumpContext, inode)) {

            status = fs_dumpCreate(context, dumpContext, inode, cp, isTmp,
                                   (inode->dinode.fsetNum !=
                                   parent->dinode.fsetNum));
            fs_markInodeCreated(context, dumpContext, inode, cp, isTmp);
        } else {
            status = fs_dumpLink(context, dumpContext, inode, cp, isTmp);
            fs_markLinkCreated(context, dumpContext, inode, cp, isTmp);
        }
        return status;
    }
#endif

    fs_inodeRelease(context, collisionInode);
    /*
     * Now we know for sure that there is a collision and
     * we need to create the link in temporary dir first
     */

    isTmp = 1;
    FS_TRACE(context, FS_TRACE_DUMP_RESTORE, ("In link step 5 for %s", name));
    fs_getNewPath(context, dumpContext, parent, &cp, &isTmp, 0);
    len = strlen(cp);
    if (!fs_isInodeCreated(context, dumpContext, inode)) {
            if (commonPath) {
                fs_markInodeCreated(context, dumpContext, inode,
                                    commonPath, strlen(commonPath), isTmp);
                status = fs_dumpLink(context, dumpContext, inode, cp, len,
                                    isTmp);
                fs_markLinkCreated(context, dumpContext, inode, cp, len, isTmp);
                //fs_markInodeCreated(context, dumpContext, inode, cp, isTmp);
                if (shadowParent) {
                        fs_inodeRelease(context, shadowParent);
                }
                fs_ramDealloc(FS_RAM_DUMP_NAME, commonPath);
                commonPath = NULL;
            } else {

                status = fs_dumpCreate(context, dumpContext, inode, cp, len,
                                        isTmp, (inode->dinode.fsetNum !=
                                                parent->dinode.fsetNum));
                fs_markInodeCreated(context, dumpContext, inode, cp, len, 
                                    isTmp);
            }
    } else {
        status = fs_dumpLink(context, dumpContext, inode, cp, len, isTmp);
        fs_markLinkCreated(context, dumpContext, inode, cp, len, isTmp);
    }

    if (commonPath) {
        fs_ramDealloc(FS_RAM_DUMP_NAME, commonPath);
    }
    return status;
}

/*
 * Walk down the target tree and generate any change ops required
 * for the node in question
 */

typedef struct fs_walkTreeContext_t    {
    DLL_ENTRY(struct fs_walkTreeContext_t)  list;
    fs_Inode    *parent;
    fs_Inumber  rootInum;
    fs_Inode    *rootInode;
    char        *name;
    int         nameLen;
    int         descent;
    int			*sentParentHistory;
    int         sentHistory;
    
    fs_DirEntry *dirents;
    int         eof;
    uint32      numEntries;
    uint32      i;
    uint64      cookie;
    int64       block;
    uint16      boff;                 
    char        *cp;
    char        buf[FS_MAXPATHLEN];
}fs_walkTreeContext;    

PRIVATE fs_Status
fs_walkTree(
    fs_Context          *context,
    fs_DumpContext_t    *dumpContext,
    fs_Inode            *parent,
    fs_Inumber          rootInum,            // root of the dir subtree
    char                *name,
    int                 descent,
    int					*sentParentHistory
)
{
    DLL_HDR(fs_walkTreeContext) walkTreeContexts;
    fs_walkTreeContext *ctx, *ctx2;
    fs_Inode            *oldRootInode;
    fs_Inumber          parentInum;
    fs_Inumber          oldParentInum;
    fs_Inode            *parentInode;
    fs_Inode            *oldParentInode;
    char                *oldName, *newName;
    int                 namespaceop, newNameLen, oldNameLen;
    fs_inodeAttr_t      *inodeAttr;
    fs_Status           status = FS_ERR_NONE;
    boolean             goingDown;
/*
elog_enabled = TRUE;
elog_severity = LOG_DEBUG;
elog_appl_data[fs_applicationId].severity = LOG_DEBUG;
elog_appl_data[fs_applicationId].bitmap |= (1 << FS_TRACE_DUMP_RESTORE);
*/
    //Initialize the contexts list
    DLL_INIT(&walkTreeContexts);
    
    //Create initial context
    ctx = fs_ramAlloc(FS_RAM_DUMP_DIRENT, sizeof(fs_walkTreeContext));
    ctx->parent             = parent;
    ctx->rootInum           = rootInum;
    ctx->name               = name;
    ctx->nameLen            = strlen(name);
    ctx->cp                 = dumpContext->currentName;
    ctx->cp                 += strlen(dumpContext->currentName);
    ctx->descent            = descent;
    ctx->eof                = 0;
    ctx->cookie             = 0;
    ctx->sentParentHistory  = sentParentHistory;
    ctx->sentHistory        = 0;
    ctx->block = 0;     // Initialize for the root inode
    ctx->boff = 0;      // Initialize for the root inode
    
    do {
        goingDown = FALSE;
        parentInode = NULL;
        oldParentInode = NULL;
        namespaceop = 1;
        
        /*
         * If have been using the cpu since a long time, give others a chance
         */

        fs_yieldPeriodic(context);
        FS_TRACE(context, FS_TRACE_DUMP_RESTORE, ("WalkTree for %s",ctx->name));
        FS_TRACE(context, FS_TRACE_DUMP_RESTORE,
                 ("rootinum = %lld name = %s descent = %d",
                 ctx->rootInum, ctx->name, ctx->descent));

        ctx->rootInode = I_GET_SHARED(dumpContext->snapnum, ctx->rootInum);
        FS_ASSERT(ctx->rootInode != NULL);

        FS_TRACE(context, FS_TRACE_DUMP_RESTORE,
                ("name %s ctime %u mtime %u snaptime %u",
                ctx->name, ctx->rootInode->dinode.ctime.secs, 
                ctx->rootInode->dinode.mtime.secs, 
                dumpContext->prevDumpTime.secs));
        if ((ctx->rootInode->dinode.linkCnt > 1)
         && !(FS_I_IS_DIR(ctx->rootInode->dinode.attrs))
         && ((FS_TIME_GT(ctx->rootInode->dinode.ctime, 
                         dumpContext->prevDumpTime))
          || (FS_TIME_GT(ctx->rootInode->dinode.mtime, 
                         dumpContext->prevDumpTime))))
        {

            /*
             * Ensure that we push any appropriate records for any ancestors
             * that were modified and also send the filehistory for all the
             * components in the path
             */

            status = fs_sendIncrFileHistory(context,
                                            dumpContext, 
                                            ctx->rootInode,
                                            ctx->sentParentHistory,
                                            &ctx->sentHistory,
                                            &ctx->block, &ctx->boff);
            if (status != FS_ERR_NONE) {
                goto goUp;
            }
            status = fs_doLink(context, dumpContext, ctx->parent, 
                                ctx->rootInode, ctx->name, ctx->nameLen);
            if (status != FS_ERR_NONE) {
                goto goUp;
            }
            status = fs_dumpNonContainerInode(context, dumpContext, ctx->parent,
                                              ctx->rootInode, ctx->name, 
                                              ctx->nameLen, 1);
            goto goUp;
        }

        if (((FS_TIME_GT(ctx->rootInode->dinode.ctime, 
                         dumpContext->prevDumpTime))
         || (FS_TIME_GT(ctx->rootInode->dinode.mtime,
                         dumpContext->prevDumpTime)))
         || (parent == NULL
         || (FS_TIME_GT(parent->dinode.mtime, dumpContext->prevDumpTime))
         || (FS_TIME_GT(parent->dinode.ctime, dumpContext->prevDumpTime))))
        {
            if (ctx->rootInum == FS_ROOT_INODE) {
                goto dumpRoot;
            }
            
            /*
             * Inode has been modified, either entries have been added
             * or this has been renamed or for non container objects file
             * is modified
             */

            oldRootInode = I_GET_SHARED(dumpContext->prevSnapNum, 
                                        ctx->rootInum);

            if (oldRootInode == NULL
             || (ctx->rootInode->dinode.genNum != oldRootInode->dinode.genNum
             || (oldRootInode->dinode.attrs == 0)))
            {
        
                /*
                 * This inode has been recreated and the older was removed
                 */

                FS_TRACE(context, FS_TRACE_DUMP_RESTORE,
                         ("walk Tree calling doLink for %s",
                            dumpContext->currentName));
                /*
                 * Ensure that we push any appropriate records for any ancestors
                 * that were modified and also send the filehistory for all the
                 * components in the path
                 */

                status = fs_sendIncrFileHistory(context,
                                                dumpContext,
                                                ctx->rootInode,
                                                ctx->sentParentHistory,
                                                &ctx->sentHistory,
                                                &ctx->block, &ctx->boff);
                if (status != FS_ERR_NONE) {
                    if (oldRootInode) {
                        fs_inodeRelease(context, oldRootInode);
                    }
                    goto goUp;
                }
                status = fs_doLink(context, dumpContext, ctx->parent, 
                                    ctx->rootInode, ctx->name, ctx->nameLen);
                if (status != FS_ERR_NONE) {
                    if (oldRootInode) {
                        fs_inodeRelease(context, oldRootInode);
                    }
                    goto goUp;
                }

                if (oldRootInode) {
                    fs_inodeRelease(context, oldRootInode);
                }
                goto descent;
            }

            FS_TRACE(context, FS_TRACE_DUMP_RESTORE,
                     ("old gen = %u new genum = %u",
                      oldRootInode->dinode.genNum,
                      ctx->rootInode->dinode.genNum));
            status = fs_findParent(context, dumpContext->snapnum, ctx->rootInum,
                                   &parentInum, NULL);

            if (status != FS_ERR_NONE) {
                fs_inodeRelease(context, oldRootInode);
                goto goUp;
            }

            FS_TRACE(context, FS_TRACE_DUMP_RESTORE,
                     ("walktree: parentInum = %lld",
                                             parentInum));
            status = fs_findParent(context, dumpContext->prevSnapNum, 
                                    ctx->rootInum, &oldParentInum, NULL);
            if (status != FS_ERR_NONE) {
                fs_inodeRelease(context, oldRootInode);
                goto goUp;
            }

            FS_TRACE(context, FS_TRACE_DUMP_RESTORE,
                    ("walktree: oldparentInum = %lld",
                                             oldParentInum));

            FS_ASSERT(parentInum != 0);
            FS_ASSERT(oldParentInum != 0);
            parentInode = I_GET_SHARED(dumpContext->snapnum, parentInum);
            oldParentInode = I_GET_SHARED(dumpContext->prevSnapNum,
                                          oldParentInum);

            /*
             * If the inode is renamed do the rename operation
             */
            status = fs_getName(context, dumpContext, parentInode, 
                                ctx->rootInode, &newName, &newNameLen, NULL);
            if (status != FS_ERR_NONE) {
                if (newName) {
                    fs_ramDealloc(FS_RAM_DUMP_NAME, newName);
                }
                fs_inodeRelease(context, parentInode);
                fs_inodeRelease(context, oldParentInode);
                fs_inodeRelease(context, oldRootInode);
                goto goUp;
            }
            status = fs_getName(context, dumpContext, oldParentInode, 
                                ctx->rootInode, &oldName, &oldNameLen, NULL);
            if (status != FS_ERR_NONE) {
                if (oldName) {
                    fs_ramDealloc(FS_RAM_DUMP_NAME, oldName);
                }
                fs_ramDealloc(FS_RAM_DUMP_NAME, newName);
                fs_inodeRelease(context, parentInode);
                fs_inodeRelease(context, oldParentInode);
                fs_inodeRelease(context, oldRootInode);
                goto goUp;
            }

            if (parentInum != oldParentInum ||
                parentInode->dinode.genNum != oldParentInode->dinode.genNum ||
                (strcmp(newName, oldName) != 0)) {

                status = fs_doRename(context, dumpContext, ctx->rootInode, 
                                     parentInode, oldParentInode, 
                                     newName, oldName, newNameLen);

                if (status != FS_ERR_NONE) {
                    fs_ramDealloc(FS_RAM_DUMP_NAME, newName);
                    fs_ramDealloc(FS_RAM_DUMP_NAME, oldName);
                    fs_inodeRelease(context, parentInode);
                    fs_inodeRelease(context, oldParentInode);
                    fs_inodeRelease(context, oldRootInode);
                    goto goUp;
                }
            }  else {

                /*
                 * This inode has not had any name space changes since last dump
                 */
                namespaceop = 0;
            }

            fs_ramDealloc(FS_RAM_DUMP_NAME, newName);
            fs_ramDealloc(FS_RAM_DUMP_NAME, oldName);
            fs_inodeRelease(context, parentInode);
            fs_inodeRelease(context, oldParentInode);
            fs_inodeRelease(context, oldRootInode);
        } else {
            FS_TRACE(context, FS_TRACE_DUMP_RESTORE,
                     ("walk Tree %s not modified", ctx->name));
            namespaceop = 0;
            FS_TRACE(context, FS_TRACE_DUMP_RESTORE,
                     ("Inode %lld not backedup ", ctx->rootInum));
            FS_TRACE(context, FS_TRACE_DUMP_RESTORE,
                    ("Inode mtime %d ctime %d snaptime %d",
                     ctx->rootInode->dinode.ctime.secs,
                     ctx->rootInode->dinode.mtime.secs,
                     dumpContext->prevDumpTime.secs));
            FS_TRACE(context, FS_TRACE_DUMP_RESTORE,
                    ("Parent mtime %d ctime %d snaptime %d",
                     ctx->parent->dinode.ctime.secs,
                     ctx->parent->dinode.mtime.secs,
                     dumpContext->prevDumpTime.secs));
        }

descent:

        if (!FS_I_IS_DIR(ctx->rootInode->dinode.attrs)
         && ((FS_TIME_GT(ctx->rootInode->dinode.ctime, 
                         dumpContext->prevDumpTime))
          || (FS_TIME_GT(ctx->rootInode->dinode.mtime,
                         dumpContext->prevDumpTime))
          || namespaceop))
        {
            /*
             * Ensure that we push any appropriate records for any ancestors
             * that were modified and also send the filehistory for all the
             * components in the path
             */

            status = fs_sendIncrFileHistory(context,
                                            dumpContext,
                                            ctx->rootInode,
                                            ctx->sentParentHistory,
                                            &ctx->sentHistory,
                                            &ctx->block, &ctx->boff);
            /*
             * If the inode was modified or file changed. Dump a name
             * operation irrespective of whether we did a name change
             * operation on it or not. We are walking off the new tree
             * so always the new name is sent
             */

            status = fs_dumpNonContainerInode(context, dumpContext, ctx->parent,
                                              ctx->rootInode, ctx->name, 
                                              ctx->nameLen, 1);
            goto goUp;
        }

        /*
         * If required dump the attr changes as well
         */

dumpRoot:
        if (FS_I_IS_DIR(ctx->rootInode->dinode.attrs)
         && ((FS_TIME_GT(ctx->rootInode->dinode.ctime,
                         dumpContext->prevDumpTime)))
         && !namespaceop)
        {
            FS_TRACE(context, FS_TRACE_DUMP_RESTORE,
                    ("walk Tree dumping attr for %s",dumpContext->currentName));

            /*
             * Ensure that we push any appropriate records for any ancestors
             * that were modified and also send the filehistory for all the
             * components in the path
             */

            fs_sendIncrFileHistory(context, dumpContext, ctx->rootInode,
                                   ctx->sentParentHistory, &ctx->sentHistory,
                                   &ctx->block, &ctx->boff);

            status = fs_dumpAttr(context, dumpContext, parentInode, 
                                    ctx->rootInode, ctx->name, ctx->nameLen);
            if (status != FS_ERR_NONE) {
                goto goUp;
            }
        }

        /*
         * Push the parent's information on the stack
         */
        dumpContext->stackIndex++;
        inodeAttr = &dumpContext->parentStack[dumpContext->stackIndex];
        inodeAttr->inum = ctx->rootInode->inum;
        inodeAttr->doneHistory = ctx->sentHistory;
        inodeAttr->block = ctx->block;
        inodeAttr->boff = ctx->boff;

        FS_ASSERT(ctx->cp ==
                  &dumpContext->currentName[strlen(dumpContext->currentName)]);
        *(ctx->cp++) = '/';
        FS_TRACE(context, FS_TRACE_DUMP_RESTORE, ("Walking tree for %lld",
                                          ctx->rootInode->inum));
        ctx->cookie = 0;
        while (!ctx->eof && ctx->descent
            && FS_I_IS_DIR(ctx->rootInode->dinode.attrs))
        {
            ctx->dirents = (fs_DirEntry *)ctx->buf;
            fs_dir_readdir(context, ctx->rootInode, ctx->dirents, FS_MAXPATHLEN,
                           ctx->cookie, FALSE, &ctx->numEntries, &ctx->eof);
            for (ctx->i = 0; ctx->i < ctx->numEntries; ctx->i++) {
                testSpecialDirectories(ctx->dirents);
                ctx->cookie = ctx->dirents->cookie;

                //Make sure current name would not go beyond FS_MAXPATHLEN
                if ((ctx->cp - dumpContext->currentName) 
                    + ctx->dirents->nameLen >= FS_MAXPATHLEN)   {
                    status = FS_ERR_NAMETOOLONG;
                    goto goUp;
                }
                
                strcpy(ctx->cp, ctx->dirents->name);
                //Optimization... to be tested then remove following
/*              strncpy(ctx->cp, ctx->dirents->name, ctx->dirents->nameLen);
                ctx->cp[ctx->dirents->nameLen] = '\0';*/
                FS_TRACE(context, FS_TRACE_DUMP_RESTORE,
                    ("WalkTree for %s", dumpContext->currentName));

                /*
                 * Push the parent's information on the stack
                 */
                ctx2 = fs_ramAlloc(FS_RAM_DUMP_DIRENT, 
                                    sizeof(fs_walkTreeContext));
                ctx2->parent            = ctx->rootInode;
                ctx2->rootInum          = ctx->dirents->inode;
                ctx2->name              = ctx->cp;
                ctx2->nameLen           = ctx->dirents->nameLen - 1;
                ctx2->cp                = ctx->cp + ctx2->nameLen;
                ctx2->descent           = dumpContext->recursiveDescent;
                ctx2->eof               = 0;
                ctx2->cookie            = 0;
                ctx2->sentParentHistory = &ctx->sentHistory;
                ctx2->sentHistory       = 0;
                DLL_ISOLATE_ELEM(ctx, list);
                DLL_INSERT_TAIL(&walkTreeContexts, ctx, list);
                FS_TRACE(context, FS_TRACE_DUMP_RESTORE, 
                         ("going down from %s to %s", ctx->name, ctx2->name));

                ctx = ctx2;
                FS_ASSERT(strlen(ctx->name) == ctx->nameLen);
                //We are now in the context of the new dir
                goingDown = TRUE;
                break;

                
goUp:           //Finish cleaning 
                fs_inodeRelease(context, ctx->rootInode);

                //Retrieve upper context
                ctx2 = ctx;
                DLL_REMOVE_TAIL(&walkTreeContexts, ctx, list);
                FS_TRACE(context, FS_TRACE_DUMP_RESTORE, 
                         ("going up from %s back to %s", ctx2->name,
                          (ctx!=NULL && ctx!=DLL_NIL) ? ctx->name : "[END]"));
                
                //Free previous context
                fs_ramDealloc(FS_RAM_DUMP_DIRENT, ctx2);
                
                if (ctx == NULL || ctx == DLL_NIL)  {
                    //We are done, return here
                    return status;        
                }
                
#if 0
                /*
                 * Restore the stack back
                 */

                inodeAttr = &dumpContext->parentStack[dumpContext->stackIndex];
                inodeAttr->doneHistory = ctx->sentHistory;
                dumpContext->stackIndex--;
#endif
                if (status != FS_ERR_NONE) {
                    FS_TRACE(context, FS_TRACE_DUMP_RESTORE,
                             ("returning status %d", status));
                    goto goUp;
                }
                FS_READDIR_ADV_ENTRY_PTR(ctx->dirents);
            }
            if (goingDown) break;
        }
        if (goingDown) continue;
        /*
         * Restore the stack back
         */

        inodeAttr = &dumpContext->parentStack[dumpContext->stackIndex];
        inodeAttr->doneHistory = ctx->sentHistory;
        dumpContext->stackIndex--;

        status = FS_ERR_NONE;
        goto goUp;

    } while (1);        
}


/*
 * Remove inodes that have been removed in the new snapshot
 * NB: We don't maintain currentName in the dumpContext because it is not used
 */

typedef struct fs_rmUnwantedNodesContext_t    {
    DLL_ENTRY(struct fs_rmUnwantedNodesContext_t)  list;
    fs_Inode    *parent;
    fs_Inode    *rootInode;
    char        *name;
    int         nameLen;
    int         descent;
    
    fs_DirEntry *dirents;
    int         eof;
    uint32      numEntries;
    uint32      i;
    uint64      cookie;
//    char        *cp;
    char        buf[FS_MAXPATHLEN];
}fs_rmUnwantedNodesContext;    

PRIVATE fs_Status
fs_removeUnwantedNodes(
    fs_Context          *context,
    fs_DumpContext_t    *dumpContext,
    fs_Inode            *parent,
    fs_Inode            *rootInode,
    char                *name,
    int                 descent
)
{
    DLL_HDR(fs_rmUnwantedNodesContext) rmUnwantedNodesContexts;
    fs_rmUnwantedNodesContext *ctx, *ctx2;
    fs_Inode    *newInode;
    fs_Status   status = FS_ERR_NONE;
    boolean     goingDown;

    //Initialize the contexts list
    DLL_INIT(&rmUnwantedNodesContexts);
    
    //Create initial context
    ctx = fs_ramAlloc(FS_RAM_DUMP_DIRENT, sizeof(fs_rmUnwantedNodesContext));
    ctx->parent     = parent;
    ctx->rootInode  = rootInode;
    ctx->name       = name;
    ctx->nameLen    = strlen(name);
    ctx->descent    = descent;
    ctx->eof        = 0;
    ctx->cookie     = 0;
    
    do {
        goingDown = FALSE;
       
        /*
         * If have been using the cpu since a long time, give others a chance
         */

        fs_yieldPeriodic(context);
        FS_TRACE(context, FS_TRACE_DUMP_RESTORE,
                 ("In fs_removeUnwantedNodes %s", ctx->name));

        if (ctx->rootInode->inum == FS_ROOT_INODE ||
            FS_I_IS_DIR(ctx->rootInode->dinode.attrs)) {
            goto walk;
        }
remove_inode:
        newInode = I_GET_SHARED(dumpContext->snapnum, ctx->rootInode->inum);
        if (newInode == NULL || newInode->dinode.attrs == 0 ||
            (newInode->dinode.genNum != ctx->rootInode->dinode.genNum)) {
    
            /*
             * This inode is removed
             */
    
            FS_TRACE(context, FS_TRACE_DUMP_RESTORE,
                    ("Calling unlink 1 for inode %lld name %s",
                      ctx->rootInode->inum, ctx->name));

            FS_TRACE(context, FS_TRACE_DUMP_RESTORE, ("Calling unlink 1"));
            status = fs_doUnlink(context, dumpContext, ctx->parent, 
                                    ctx->rootInode, ctx->name, ctx->nameLen, 1);
            if (newInode) {
                fs_inodeRelease(context, newInode);
            }
            goto goUp;
        }

        /*
         * If this inode has links, determine if we need to issue an
         * unlink
         */

        if ((newInode->dinode.linkCnt > 1 ||
             newInode->dinode.linkCnt != ctx->rootInode->dinode.linkCnt) && 
            !FS_I_IS_DIR(newInode->dinode.attrs)) {
            FS_TRACE(context, FS_TRACE_DUMP_RESTORE,
                    ("Calling unlink 2 for inode %lld name %s",
                      ctx->rootInode->inum, ctx->name));
            FS_TRACE(context, FS_TRACE_DUMP_RESTORE, ("Calling unlink 2"));
            status = fs_doUnlink(context, dumpContext, ctx->parent, 
                                    ctx->rootInode, ctx->name, ctx->nameLen, 0);
            fs_inodeRelease(context, newInode);
    
            /*
             * If this was a directory, we will assume that restore removes
             * entries recursively from it, as we have already performed
             * renames, so all the entries that are pending in this dir would
             * be removed anyway. This helps us in avoiding walking the tree
             * in order and potentially we would need to generate lesser
             * number of scans and messages for restore
             */
            
            goto goUp;
        }
        fs_inodeRelease(context, newInode);

        goto goUp;
        
walk:
        //ctx->cp = &dumpContext->currentName[strlen(dumpContext->currentName)];
        while (!ctx->eof && ctx->descent) {
            ctx->dirents = (fs_DirEntry *)ctx->buf;
            fs_dir_readdir(context, ctx->rootInode, ctx->dirents, FS_MAXPATHLEN,
                            ctx->cookie, FALSE, &ctx->numEntries, &ctx->eof);
            for (ctx->i = 0; ctx->i < ctx->numEntries; ctx->i++) {
                testSpecialDirectories(ctx->dirents);
                ctx->cookie = ctx->dirents->cookie;
                //*ctx->cp++ = '/';
                //strncpy(ctx->cp, ctx->dirents->name, ctx->dirents->nameLen);
                //ctx->cp[ctx->dirents->nameLen] = '\0';

                ctx2 = fs_ramAlloc(FS_RAM_DUMP_DIRENT, 
                                    sizeof(fs_rmUnwantedNodesContext));
                ctx2->parent    = ctx->rootInode;
                ctx2->rootInode = I_GET_SHARED(dumpContext->prevSnapNum,
                                               ctx->dirents->inode);
                ctx2->name      = ctx->dirents->name;
                ctx2->nameLen   = ctx->dirents->nameLen - 1; //remove end '\0'
                ctx2->descent   = dumpContext->recursiveDescent;
                ctx2->eof       = 0;
                ctx2->cookie    = 0;
                DLL_ISOLATE_ELEM(ctx, list);
                DLL_INSERT_TAIL(&rmUnwantedNodesContexts, ctx, list);
                FS_TRACE(context, FS_TRACE_DUMP_RESTORE, 
                         ("going down from %s to %s", ctx->name, ctx2->name));

                ctx = ctx2;
                //We are now in the context of the new dir
                goingDown = TRUE;
                break;


goUp:           //Finish cleaning 
                fs_inodeRelease(context, ctx->rootInode);

                //Retrieve upper context
                ctx2 = ctx;
                DLL_REMOVE_TAIL(&rmUnwantedNodesContexts, ctx, list);
                FS_TRACE(context, FS_TRACE_DUMP_RESTORE, 
                         ("going up from %s back to %s", ctx2->name,
                          (ctx!=NULL && ctx!=DLL_NIL) ? ctx->name : "[END]"));
                
                //Free previous context
                fs_ramDealloc(FS_RAM_DUMP_DIRENT, ctx2);
                
                if (ctx == NULL || ctx == DLL_NIL)  {
                    //We are done, return here
                    return status;        
                }
                
                //ctx->cp--;
                //*(ctx->cp) = '\0';
                if (status != FS_ERR_NONE) {
                    goto goUp;
                }
                FS_READDIR_ADV_ENTRY_PTR(ctx->dirents);
            }
            if (goingDown) break;
        }
        if (goingDown) continue;

        /*
         * If this directory was removed, handle that
         */

        if (ctx->rootInode->inum != FS_ROOT_INODE) {
            goto remove_inode;
        }
        
        goto goUp;
        
    } while (1);
}

/*
 * Incore state management routines
 */

/*
 * Create an entry for this inode in links hash table and mark it
 * as created
 */

EXPORTED void
fs_markInodeCreated(
    fs_Context          *context,
    fs_DumpContext_t    *dumpContext,
    fs_Inode            *inode,
    char                *path,
    int                 pathLen,
    int                 temp
)
{
    uint32              hash = (uint32)(inode->inum % 127);
    fs_dumpRenameLink_t *entry; 

    if (inode->dinode.linkCnt == 1 && !temp) {
        return;
    }

    /*
     * If have been using the cpu since a long time, give others a chance
     */

    fs_yieldPeriodic(context);
    FS_TRACE(context, FS_TRACE_DUMP_RESTORE, ("In fs_markInodeCreated"));

    entry = (fs_dumpRenameLink_t *)
            fs_ramAlloc(FS_RAM_DUMP_RENAMELINK,
                        sizeof(fs_dumpRenameLink_t) + pathLen);
    FS_ASSERT(entry != NULL);
    entry->inum = inode->inum;
    entry->flags = FS_DUMP_INO_CREATED;
    if (temp) {
        entry->flags |= FS_DUMP_INO_TEMP;
    }
	DLL_ISOLATE_ELEM(entry, link);
    DLL_INSERT_TAIL(&dumpContext->links[hash], entry, link);
    FS_TRACE(context, FS_TRACE_DUMP_RESTORE, ("markInode created path %s\n",
             path));
    strcpy(entry->path, path);
    entry->pathLen = pathLen;
    return;
}

/*
 * Mark that entry is linked
 */

EXPORTED void
fs_markLinkCreated(
    fs_Context          *context,
    fs_DumpContext_t    *dumpContext,
    fs_Inode            *inode,
    char                *path,
    int                 pathLen,
    int                 temp
)
{
    uint32              hash = (uint32)(inode->inum % 127);
    fs_dumpRenameLink_t *entry; 

    /*
     * If have been using the cpu since a long time, give others a chance
     */

    fs_yieldPeriodic(context);
    FS_TRACE(context, FS_TRACE_DUMP_RESTORE, ("In fs_markLinkCreated"));

    entry = (fs_dumpRenameLink_t *)
            fs_ramAlloc(FS_RAM_DUMP_RENAMELINK,
                        sizeof(fs_dumpRenameLink_t) + pathLen);

    FS_ASSERT(entry != NULL);
    entry->inum = inode->inum;
    entry->flags = FS_DUMP_INO_LINKED;
    if (temp) {
        entry->flags |= FS_DUMP_INO_TEMP;
    }
	DLL_ISOLATE_ELEM(entry, link);
    DLL_INSERT_TAIL(&dumpContext->links[hash], entry, link);
    strcpy(entry->path, path);
    entry->pathLen = pathLen;
    return;
}

/*
 * Is this inode created. It is used only for inodes with links
 */

EXPORTED int
fs_isInodeCreated(
    fs_Context          *context,
    fs_DumpContext_t    *dumpContext,
    fs_Inode            *inode
)
{
    uint32              hash = (uint32)(inode->inum % 127);
    fs_dumpRenameLink_t *entry;

    /*
     * If have been using the cpu since a long time, give others a chance
     */

    fs_yieldPeriodic(context);
    FS_TRACE(context, FS_TRACE_DUMP_RESTORE, ("In fs_isInodeCreated"));
    DLL_ITERATE(entry, &dumpContext->links[hash], link) {
        if (entry == DLL_NIL) {
            FS_ASSERT(0);
        }
        if (entry->inum == inode->inum) {
            return (entry->flags & FS_DUMP_INO_CREATED);
        }
    }

    return 0;
}

EXPORTED int
fs_isInodeLinked(
    fs_Context          *context,
    fs_DumpContext_t    *dumpContext,
    fs_Inode            *inode,
    char                *path
)
{
    uint32              hash = (uint32)(inode->inum % 127);
    fs_dumpRenameLink_t *entry;

    /*
     * If have been using the cpu since a long time, give others a chance
     */

    fs_yieldPeriodic(context);
    FS_TRACE(context, FS_TRACE_DUMP_RESTORE, ("In fs_isInodeCreated"));
    DLL_ITERATE(entry, &dumpContext->links[hash], link) {
        if (entry == DLL_NIL) {
            FS_ASSERT(0);
        }
        if (entry->inum == inode->inum && strcmp(entry->path, path) == 0) {
            return (entry->flags & FS_DUMP_INO_LINKED);
        }
    }

    return 0;
}

EXPORTED char *
fs_getLinkPath(
    fs_Context          *context,
    fs_DumpContext_t    *dumpContext,
    fs_Inode            *inode,
    int                 *pathLen
)
{
    uint32              hash = (uint32)(inode->inum % 127);
    fs_dumpRenameLink_t *entry;

    /*
     * If have been using the cpu since a long time, give others a chance
     */

    fs_yieldPeriodic(context);
    FS_TRACE(context, FS_TRACE_DUMP_RESTORE, ("In fs_getLinkPath"));
    DLL_ITERATE(entry, &dumpContext->links[hash], link) {
        if (entry == DLL_NIL) {
            FS_ASSERT(0);
        }
        if (entry->inum == inode->inum && (entry->flags & FS_DUMP_INO_CREATED))
        {
            *pathLen = entry->pathLen;
            return entry->path;
        }
    }

    FS_ASSERT(0);
    return NULL;
}


EXPORTED void
fs_markInodeRenamed(
    fs_Context          *context,
    fs_DumpContext_t    *dumpContext,
    fs_Inode            *inode,
    int                 temp
)
{
    uint32              hash = (uint32)(inode->inum % 1023);
    fs_dumpRenameLink_t *entry;

    /*
     * If have been using the cpu since a long time, give others a chance
     */

    fs_yieldPeriodic(context);
    FS_TRACE(context, FS_TRACE_DUMP_RESTORE,
     ("In fs_markInodeRenamed for inum %lld temp %d", inode->inum, temp));
    entry = (fs_dumpRenameLink_t *)fs_ramAlloc(FS_RAM_DUMP_RENAMELINK,
                                               sizeof(fs_dumpRenameLink_t));

    FS_ASSERT(entry != NULL);
    entry->inum = inode->inum;
    entry->path[0] = '\0';
    entry->flags = FS_DUMP_INO_RENAMED;
    if (temp) {
        entry->flags |= FS_DUMP_INO_TEMP;
    }
	DLL_ISOLATE_ELEM(entry, link);
    DLL_INSERT_TAIL(&dumpContext->renames[hash], entry, link);
    return;
}



/*
 * Is this inode renamed.
 */

EXPORTED int
fs_isInodeRenamed(
    fs_Context          *context,
    fs_DumpContext_t    *dumpContext,
    fs_Inode            *inode
)
{
    uint32              hash = (uint32)(inode->inum % 1023);
    fs_dumpRenameLink_t *entry; 

    /*
     * If have been using the cpu since a long time, give others a chance
     */

    fs_yieldPeriodic(context);
    FS_TRACE(context, FS_TRACE_DUMP_RESTORE,
            ("In fs_isInodeRenamed for %lld", inode->inum));
    DLL_ITERATE(entry, &dumpContext->renames[hash], link) {
        if (entry == DLL_NIL) {
            FS_ASSERT(0);
        }
        if (entry->inum == inode->inum) {
            FS_TRACE(context, FS_TRACE_DUMP_RESTORE,
                            ("inInodeRenamed entry inum %lld flags %x",
                             entry->inum, entry->flags));
            return (entry->flags & FS_DUMP_INO_RENAMED);
        }
    }

    return 0;
}

/*
 * Check whether this path to the inode is temp
 */

EXPORTED int
fs_isInodePathTemp(
    fs_Context          *context,
    fs_DumpContext_t    *dumpContext,
    fs_Inode            *inode,
    char                *path
)
{
    uint32              hash = (uint32)(inode->inum % 1023);
    fs_dumpRenameLink_t *entry;

    /*
     * If have been using the cpu since a long time, give others a chance
     */

    fs_yieldPeriodic(context);
    FS_TRACE(context, FS_TRACE_DUMP_RESTORE, ("In fs_isInodeTemp"));
    DLL_ITERATE(entry, &dumpContext->renames[hash], link) {
        if (entry == DLL_NIL) {
            FS_ASSERT(0);
        }
#if 0
        PUT THAT BACK WHEN entry->path IS FILLED IN markInodeRenamed
        if ((entry->inum == inode->inum) &&
            (strcmp(entry->path, path) == 0)) {
#endif
        if (entry->inum == inode->inum) {
            if (entry->flags & FS_DUMP_INO_TEMP) {
                    return 1;
            }
#if 0
            return (entry->flags & FS_DUMP_INO_TEMP);
#endif
        }
    }
    hash = (uint32)(inode->inum % 127);
    DLL_ITERATE(entry, &dumpContext->links[hash], link) {
        if (entry == DLL_NIL) {
            FS_ASSERT(0);
        }
        if (entry->inum == inode->inum &&
            (strcmp(entry->path, path) == 0)) {
            if (entry->flags & FS_DUMP_INO_TEMP) {
                    return 1;
            }
        }
    }


    return 0;
}

EXPORTED int
fs_isInodeTemp(
    fs_Context          *context,
    fs_DumpContext_t    *dumpContext,
    fs_Inode            *inode
)
{
    uint32              hash = (uint32)(inode->inum % 1023);
    fs_dumpRenameLink_t *entry;

    /*
     * If have been using the cpu since a long time, give others a chance
     */

    fs_yieldPeriodic(context);
    FS_TRACE(context, FS_TRACE_DUMP_RESTORE, ("In fs_isInodeTemp"));
    DLL_ITERATE(entry, &dumpContext->renames[hash], link) {
        if (entry == DLL_NIL) {
            FS_ASSERT(0);
        }
        if (entry->inum == inode->inum) {
//            return (entry->flags & FS_DUMP_INO_TEMP);
            if (entry->flags & FS_DUMP_INO_TEMP) {
                return 1;
            }
        }
    }
    hash = (uint32)(inode->inum % 127);
    DLL_ITERATE(entry, &dumpContext->links[hash], link) {
        if (entry == DLL_NIL) {
            FS_ASSERT(0);
        }
        if (entry->inum == inode->inum) {
            //return (entry->flags & FS_DUMP_INO_TEMP);
            if (entry->flags & FS_DUMP_INO_TEMP) {
                return 1;
            }
        }
    }


    return 0;
}

/*
 * Dump a rename record to tape
 */

PRIVATE fs_Status
fs_dumpRename(
    fs_Context          *context,
    fs_DumpContext_t    *dumpContext,
    fs_Inode            *inode,
    char                *oldPath,
    char                *newPath,
    int                 oldPathLen,
    int                 newPathLen,
    int                 isTmp
)
{
    int     totLen = oldPathLen + newPathLen + 2;
    int     availLen;
    uint32  nHeaders = 1;
    fs_DumpOperation_t  *obj;
    char                *cp, *tp;
    fs_Status           status;

    /*
     * If have been using the cpu since a long time, give others a chance
     */

    fs_yieldPeriodic(context);
    FS_TRACE(context, FS_TRACE_DUMP_RESTORE,
            ("In fs_dumpRename old Path %s - new path %s isTmp %d",
             oldPath, newPath, isTmp));

    totLen += (FS_DR_TBSIZE - sizeof (fs_DumpOperation_t));
    while (totLen > 0) {
        nHeaders++;
        totLen -= FS_DR_TBSIZE;
    }

    totLen = oldPathLen + newPathLen + 2;
    obj = &dumpContext->record.u.operation;
    obj->header.headerId = dumpContext->currentId++;
    obj->header.headerSubId = 0;
    obj->len = totLen;
    obj->sourceNameLen = oldPathLen;
    obj->targetNameLen = newPathLen;
    obj->header.dataExtents = 0;
    obj->header.attr.uid = inode->sd->u.nfs.uid;
    obj->header.attr.gid = inode->sd->u.nfs.gid;
    obj->header.attr.sizeDev.size = inode->dinode.size;
    obj->header.attr.ctime = inode->dinode.ctime;
    obj->header.attr.mtime = inode->dinode.mtime;
    obj->header.attr.atime = inode->dinode.atime;
    obj->header.attr.mode = (inode->sd->u.nfs.mode & FS_IMODE_MASK) |
                             (inode->dinode.attrs & FS_IATR_MASK);
    obj->header.attr.simpleAttrs = inode->dinode.userAttr;
    obj->header.op = FS_DR_RENAME;
    if (isTmp) {
        obj->header.flags = FS_DR_TEMP_NAME;
    }

    /*
     * Now copy both the names to the buffer. First copy old name
     * and then the new name. Both the names are null terminated
     */

    totLen = oldPathLen + 1;
    cp = oldPath;
    tp = &dumpContext->record.u.dummy[sizeof (fs_DumpOperation_t)];
    availLen = &dumpContext->record.u.dummy[FS_DR_TBSIZE] - tp;
    FS_TRACE(context, FS_TRACE_DUMP_RESTORE,
            ("dumpRename:dumped %s in header", cp));
    if (totLen < availLen) {
        memcpy(tp, cp, totLen);
        tp += totLen;
        availLen -= totLen;
    } else {
        memcpy(tp, cp, availLen);
        totLen -= availLen;
        while (totLen > 0)  {
            status = fs_writeTape(context, dumpContext);
            if (status != FS_ERR_NONE) {
                obj->header.flags = 0;
                return status;
            }
            tp = dumpContext->record.u.dummy;
            cp += availLen;
            availLen = FS_DR_TBSIZE;
            if (totLen < availLen) {
                memcpy(tp, cp, totLen);
                totLen = 0;
                tp += totLen;
                availLen -= totLen;
            } else {
                memcpy(tp, cp, availLen);
                totLen -= availLen;
            }
        }
    }

    totLen = newPathLen + 1;
    cp = newPath;
    FS_TRACE(context, FS_TRACE_DUMP_RESTORE,
            ("dumpRename:dumped %s in header", cp));
    if (totLen < availLen) {
        memcpy(tp, cp, totLen);
        tp += totLen;
        availLen -= totLen;
    } else {
        memcpy(tp, cp, availLen);
        totLen -= availLen;
        while (totLen > 0)  {
            status = fs_writeTape(context, dumpContext);
            if (status != FS_ERR_NONE) {
                obj->header.flags = 0;
                return status;
            }
            tp = dumpContext->record.u.dummy;
            cp += availLen;
            availLen = FS_DR_TBSIZE;
            if (totLen < availLen) {
                memcpy(tp, cp, totLen);
                totLen = 0;
                tp += totLen;
                availLen -= totLen;
            } else {
                memcpy(tp, cp, availLen);
                totLen -= availLen;
            }
        }
    }

#if 0    
    if (totLen < availLen) {
        bzero(tp + totLen, availLen - totLen);
        memcpy(tp, cp, totLen);
        totLen = 0;
    } else {
        memcpy(tp, cp, availLen);
        totLen -= availLen;
    }
    while (totLen > 0) {
        status = fs_writeTape(context, dumpContext);
        if (status != FS_ERR_NONE) {
            obj->header.flags = 0;
            return status;
        }
        tp = dumpContext->record.u.dummy;
        cp += availLen;
        if (totLen < availLen) {
            bzero(tp + totLen, availLen - totLen);
            memcpy(tp, cp, totLen);
            totLen = 0;
        } else {
            memcpy(tp, cp, availLen);
            totLen -= availLen;
        }
        tp += availLen;
        availLen = FS_DR_TBSIZE;
    }


    /*
     * Move the pointer forward for the next name
     */

    tp += strlen(tp) + 2;

    /*
     * If we go past the available buffer, start with new one
     */

    if (tp >= &dumpContext->record.u.dummy[FS_DR_TBSIZE]) {
        tp = NULL;
    }
    totLen = newPathLen + 1;
    availLen = &dumpContext->record.u.dummy[FS_DR_TBSIZE] - tp;
    cp = newPath;
    strncpy(tp, cp, availLen);
    FS_TRACE(context, FS_TRACE_DUMP_RESTORE,
            ("dumpRename:dumped %s in header", cp));
    FS_TRACE(context, FS_TRACE_DUMP_RESTORE,
             ("dumpRename:tp %s in header", tp));
    totLen -= availLen;
    while (totLen > 0) {
        fs_writeTape(context, dumpContext);
        tp = dumpContext->record.u.dummy;
        bzero(tp, FS_DR_TBSIZE);
        cp += availLen;
        bcopy(tp, cp, totLen);  //BUG !!!
        tp += totLen;
        totLen -= availLen;
        availLen = FS_DR_TBSIZE;
    }
#endif
    
    status = fs_writeTape(context, dumpContext);
    obj->header.flags = 0;
    return status;
}

/*
 * Dump a remove record to tape
 */

PRIVATE fs_Status
fs_dumpRemove(
    fs_Context          *context,
    fs_DumpContext_t    *dumpContext,
    fs_Inode            *inode,
    char                *newPath,
    int                 isTmp
)
{
    int     newPathLen = strlen(newPath);
    int     totLen = newPathLen + 1;
    int     availLen;
    uint32  nHeaders = 1;
    fs_DumpOperation_t  *obj;
    char                *cp, *tp;
    fs_Status           status;

    /*
     * If have been using the cpu since a long time, give others a chance
     */

    fs_yieldPeriodic(context);

    FS_TRACE(context, FS_TRACE_DUMP_RESTORE,
             ("In fs_dumpRemove - %s", newPath));
    totLen += (FS_DR_TBSIZE - sizeof (fs_DumpOperation_t));
    while (totLen > 0) {
        nHeaders++;
        totLen -= FS_DR_TBSIZE;
    }

    totLen = newPathLen + 1;
    obj = &dumpContext->record.u.operation;
    obj->header.headerId = dumpContext->currentId++;
    obj->header.headerSubId = 0;
    obj->len =  totLen;
    obj->sourceNameLen = newPathLen;
    obj->header.dataExtents = 0;
    obj->header.attr.uid = inode->sd->u.nfs.uid;
    obj->header.attr.gid = inode->sd->u.nfs.gid;
    obj->header.attr.sizeDev.size = inode->dinode.size;
    obj->header.attr.ctime = inode->dinode.ctime;
    obj->header.attr.mtime = inode->dinode.mtime;
    obj->header.attr.atime = inode->dinode.atime;
    obj->header.attr.mode = (inode->sd->u.nfs.mode & FS_IMODE_MASK) |
                             (inode->dinode.attrs & FS_IATR_MASK);
    obj->header.attr.simpleAttrs = inode->dinode.userAttr;
    if (FS_I_IS_DIR(inode->dinode.attrs)) {
        obj->header.op = FS_DR_REMOVE_DIR;
    } else {
        obj->header.op = FS_DR_REMOVE;
    }
    if (isTmp) {
        obj->header.flags = FS_DR_TEMP_NAME;
    }

    /*
     * Now copy both the names to the buffer. First copy old name
     * and then the new name. Both the names are null terminated
     */

    totLen = newPathLen + 1;
    cp = newPath;
    tp = &dumpContext->record.u.dummy[sizeof (fs_DumpOperation_t)];
    availLen = &dumpContext->record.u.dummy[FS_DR_TBSIZE] - tp;
    FS_TRACE(context, FS_TRACE_DUMP_RESTORE,
            ("dumpRemove:dumped %s in header", cp));
    if (totLen < availLen) {
        memcpy(tp, cp, totLen);
        tp += totLen;
        availLen -= totLen;
    } else {
        memcpy(tp, cp, availLen);
        totLen -= availLen;
        while (totLen > 0)  {
            status = fs_writeTape(context, dumpContext);
            if (status != FS_ERR_NONE) {
                obj->header.flags = 0;
                return status;
            }
            tp = dumpContext->record.u.dummy;
            cp += availLen;
            availLen = FS_DR_TBSIZE;
            if (totLen < availLen) {
                memcpy(tp, cp, totLen);
                totLen = 0;
                tp += totLen;
                availLen -= totLen;
            } else {
                memcpy(tp, cp, availLen);
                totLen -= availLen;
            }
        }
    }

#if 0    
    strncpy(tp, cp, availLen);
    tp += availLen;
    totLen -= availLen;
    while (totLen > 0) {
        status = fs_writeTape(context, dumpContext);
        if (status != FS_ERR_NONE) {
            obj->header.flags = 0;
            return status;
        }
        tp = dumpContext->record.u.dummy;
        bzero(tp, FS_DR_TBSIZE);
        cp += availLen;
        bcopy(tp, cp, totLen);
        tp += totLen;
        totLen -= availLen;
        availLen = FS_DR_TBSIZE;
    }
#endif    
    status = fs_writeTape(context, dumpContext);
    obj->header.flags = 0;
    return status;
}

/*
 * Dump a remove record to tape
 */

PRIVATE fs_Status
fs_dumpCreate(
    fs_Context          *context,
    fs_DumpContext_t    *dumpContext,
    fs_Inode            *inode,
    char                *newPath,
    int                 newPathLen,
    int                 isTmp,
    int                 isFset
)
{
    int     totLen = newPathLen + 1;
    int     availLen;
    uint32  nHeaders = 1;
    fs_DumpOperation_t  *obj;
    char                *cp, *tp;
    fs_Status           status;

    /*
     * If have been using the cpu since a long time, give others a chance
     */

    fs_yieldPeriodic(context);

    FS_TRACE(context, FS_TRACE_DUMP_RESTORE,
             ("dumpCreate: creating %s is tmp%d",  newPath, isTmp));
    FS_TRACE(context, FS_TRACE_DUMP_RESTORE, ("In fs_dumpCreate"));
    totLen += (FS_DR_TBSIZE - sizeof (fs_DumpOperation_t));
    while (totLen > 0) {
        nHeaders++;
        totLen -= FS_DR_TBSIZE;
    }

    totLen = newPathLen + 1;
    obj = &dumpContext->record.u.operation;
    obj->header.headerId = dumpContext->currentId++;
    obj->header.headerSubId = 0;
    obj->len =  totLen;
    obj->sourceNameLen = newPathLen;
    obj->header.dataExtents = 0;
    obj->header.attr.uid = inode->sd->u.nfs.uid;
    obj->header.attr.gid = inode->sd->u.nfs.gid;
    obj->header.attr.sizeDev.size = inode->dinode.size;
    obj->header.attr.ctime = inode->dinode.ctime;
    obj->header.attr.mtime = inode->dinode.mtime;
    obj->header.attr.atime = inode->dinode.atime;
    obj->header.attr.mode = (inode->sd->u.nfs.mode & FS_IMODE_MASK) |
                             (inode->dinode.attrs & FS_IATR_MASK);
    obj->header.attr.simpleAttrs = inode->dinode.userAttr;
    if (FS_I_IS_DIR(inode->dinode.attrs)) {
        if (!isFset) {
            obj->header.op = FS_DR_CREATE_DIR;
        } else {
            obj->header.op = FS_DR_CREATE_FSET;
        }
    } else if (FS_I_IS_REG(inode->dinode.attrs)) {
        obj->header.op = FS_DR_CREATE_FILE;
    } else if (FS_I_IS_SYML(inode->dinode.attrs)) {
        FS_TRACE(context, FS_TRACE_DUMP_RESTORE,
              ("Creating symlink for %s", newPath));
        FS_TRACE(context, FS_TRACE_DUMP_RESTORE,
              ("Creating symlink size %lld", inode->dinode.size));
        obj->header.op = FS_DR_CREATE_SYML;
    } else if (FS_I_IS_BLK(inode->dinode.attrs)) {
        obj->header.op = FS_DR_CREATE_BLOCK;
        obj->header.attr.sizeDev.devNum.major = inode->dinode.u.devNum.major;
        obj->header.attr.sizeDev.devNum.minor = inode->dinode.u.devNum.minor;
    } else if (FS_I_IS_CHR(inode->dinode.attrs)) {
        obj->header.op = FS_DR_CREATE_CHAR;
        obj->header.attr.sizeDev.devNum.major = inode->dinode.u.devNum.major;
        obj->header.attr.sizeDev.devNum.minor = inode->dinode.u.devNum.minor;
    } else if (FS_I_IS_SOCK(inode->dinode.attrs)) {
        obj->header.op = FS_DR_CREATE_SOCK;
        obj->header.attr.sizeDev.size = 0;
    } else if (FS_I_IS_FIFO(inode->dinode.attrs)) {
        obj->header.op = FS_DR_CREATE_FIFO;
        obj->header.attr.sizeDev.size = 0;
    }
    if (isTmp) {
        FS_TRACE(context, FS_TRACE_DUMP_RESTORE,
              ("Dump create to temp"));
        obj->header.flags = FS_DR_TEMP_NAME;
    }

    /*
     * Now copy both the names to the buffer. First copy old name
     * and then the new name. Both the names are null terminated
     */

    totLen = newPathLen + 1;
    cp = newPath;
    tp = &dumpContext->record.u.dummy[sizeof (fs_DumpOperation_t)];
    availLen = &dumpContext->record.u.dummy[FS_DR_TBSIZE] - tp;
    FS_TRACE(context, FS_TRACE_DUMP_RESTORE,
              ("dumpCreate:dumped %s in header", cp));
    if (totLen < availLen) {
        memcpy(tp, cp, totLen);
        tp += totLen;
        availLen -= totLen;
    } else {
        memcpy(tp, cp, availLen);
        totLen -= availLen;
        while (totLen > 0)  {
            status = fs_writeTape(context, dumpContext);
            if (status != FS_ERR_NONE) {
                obj->header.flags = 0;
                return status;
            }
            tp = dumpContext->record.u.dummy;
            cp += availLen;
            availLen = FS_DR_TBSIZE;
            if (totLen < availLen) {
                memcpy(tp, cp, totLen);
                totLen = 0;
                tp += totLen;
                availLen -= totLen;
            } else {
                memcpy(tp, cp, availLen);
                totLen -= availLen;
            }
        }
    }
#if 0    
    strncpy(tp, cp, availLen);
    tp += availLen;
    totLen -= availLen;
    while (totLen > 0) {
        status = fs_writeTape(context, dumpContext);
        if (status != FS_ERR_NONE) {
            obj->header.flags = 0;
            return status;
        }
        tp = dumpContext->record.u.dummy;
        bzero(tp, FS_DR_TBSIZE);
        cp += availLen;
        bcopy(tp, cp, totLen);
        tp += totLen;
        totLen -= availLen;
        availLen = FS_DR_TBSIZE;
    }
#endif    
    status = fs_writeTape(context, dumpContext);
    obj->header.flags = 0;
    return status;
}

/*
 * Dump a link
 */
PRIVATE fs_Status
fs_dumpLink(
    fs_Context          *context,
    fs_DumpContext_t    *dumpContext,
    fs_Inode            *inode,
    char                *newPath,
    int                 newPathLen,
    int                 isTmp
)
{
    int     totLen = newPathLen + 1;
    int     availLen;
    uint32  nHeaders = 1;
    fs_DumpOperation_t  *obj;
    char                *cp, *tp;
    fs_Status           status;
    int                 oldPathLen;
    char                *oldPath;
   
    oldPath = fs_getLinkPath(context, dumpContext, inode, &oldPathLen);

    /*
     * If have been using the cpu since a long time, give others a chance
     */

    fs_yieldPeriodic(context);
    FS_ASSERT(oldPath != NULL);
    FS_TRACE(context, FS_TRACE_DUMP_RESTORE,
              ("fs_dumpLink: oldPath %s newPath %s", oldPath, newPath));
    FS_TRACE(context, FS_TRACE_DUMP_RESTORE,
              ("In fs_dumpLink"));
    totLen += (FS_DR_TBSIZE - sizeof (fs_DumpOperation_t));
    while (totLen > 0) {
        nHeaders++;
        totLen -= FS_DR_TBSIZE;
    }

    totLen = oldPathLen + newPathLen + 2;
    obj = &dumpContext->record.u.operation;
    obj->header.headerId = dumpContext->currentId++;
    obj->header.headerSubId = 0;
    obj->len =  totLen;
    obj->sourceNameLen = oldPathLen;
    obj->targetNameLen = newPathLen;
    obj->header.dataExtents = 0;
    obj->header.attr.uid = inode->sd->u.nfs.uid;
    obj->header.attr.gid = inode->sd->u.nfs.gid;
    obj->header.attr.sizeDev.size = inode->dinode.size;
    obj->header.attr.ctime = inode->dinode.ctime;
    obj->header.attr.mtime = inode->dinode.mtime;
    obj->header.attr.atime = inode->dinode.atime;
    obj->header.attr.mode = (inode->sd->u.nfs.mode & FS_IMODE_MASK) |
                             (inode->dinode.attrs & FS_IATR_MASK);
    obj->header.attr.simpleAttrs = inode->dinode.userAttr;
    obj->header.op = FS_DR_CREATE_LINK;
    obj->header.flags = 0;
    if (isTmp) {
        obj->header.flags = FS_DR_TEMP_NAME;
    }

    if (fs_isInodePathTemp(context, dumpContext,inode, oldPath)) {
        obj->header.flags |= FS_DR_LINK_TEMP;
    }

    /*
     * Now copy both the names to the buffer. First copy old name
     * and then the new name. Both the names are null terminated
     */

    totLen = oldPathLen + 1;
    cp = oldPath;
    tp = &dumpContext->record.u.dummy[sizeof (fs_DumpOperation_t)];
    availLen = &dumpContext->record.u.dummy[FS_DR_TBSIZE] - tp;
    FS_TRACE(context, FS_TRACE_DUMP_RESTORE,
              ("dumpLink:dumped %s in header", cp));
    if (totLen < availLen) {
        memcpy(tp, cp, totLen);
        tp += totLen;
        availLen -= totLen;
    } else {
        memcpy(tp, cp, availLen);
        totLen -= availLen;
        while (totLen > 0)  {
            status = fs_writeTape(context, dumpContext);
            if (status != FS_ERR_NONE) {
                obj->header.flags = 0;
                return status;
            }
            tp = dumpContext->record.u.dummy;
            cp += availLen;
            availLen = FS_DR_TBSIZE;
            if (totLen < availLen) {
                memcpy(tp, cp, totLen);
                totLen = 0;
                tp += totLen;
                availLen -= totLen;
            } else {
                memcpy(tp, cp, availLen);
                totLen -= availLen;
            }
        }
    }

    totLen = newPathLen + 1;
    cp = newPath;
    FS_TRACE(context, FS_TRACE_DUMP_RESTORE,
            ("dumpRename:dumped %s in header", cp));
    if (totLen < availLen) {
        memcpy(tp, cp, totLen);
        tp += totLen;
        availLen -= totLen;
    } else {
        memcpy(tp, cp, availLen);
        totLen -= availLen;
        while (totLen > 0)  {
            status = fs_writeTape(context, dumpContext);
            if (status != FS_ERR_NONE) {
                obj->header.flags = 0;
                return status;
            }
            tp = dumpContext->record.u.dummy;
            cp += availLen;
            availLen = FS_DR_TBSIZE;
            if (totLen < availLen) {
                memcpy(tp, cp, totLen);
                totLen = 0;
                tp += totLen;
                availLen -= totLen;
            } else {
                memcpy(tp, cp, availLen);
                totLen -= availLen;
            }
        }
    }

#if 0    
    strncpy(tp, cp, availLen);
    totLen -= availLen;
    while (availLen < totLen) {
        status = fs_writeTape(context, dumpContext);
        if (status != FS_ERR_NONE) {
            obj->header.flags = 0;
            return status;
        }
        tp = dumpContext->record.u.dummy;
        bzero(tp, FS_DR_TBSIZE);
        cp += availLen;
        memcpy(tp, cp, totLen);
        tp += totLen;
        totLen -= availLen;
        availLen = FS_DR_TBSIZE;
    }

    tp += strlen(tp) + 2;

    /*
     * If we go past the available buffer, start with new one
     */

    if (tp >= &dumpContext->record.u.dummy[FS_DR_TBSIZE]) {
        tp = NULL;
    }
    totLen = newPathLen + 1;
    availLen = &dumpContext->record.u.dummy[FS_DR_TBSIZE] - tp;
    cp = newPath;
    strncpy(tp, cp, availLen);
    FS_TRACE(context, FS_TRACE_DUMP_RESTORE,
              ("dumpLink:dumped %s in header", cp));
    totLen -= availLen;
    while (availLen < totLen) {
        fs_writeTape(context, dumpContext);
        tp = dumpContext->record.u.dummy;
        bzero(tp, FS_DR_TBSIZE);
        cp += availLen;
        memcpy(tp, cp, totLen);
        tp += totLen;
        totLen -= availLen;
        availLen = FS_DR_TBSIZE;
    }
#endif    
    status = fs_writeTape(context, dumpContext);
    obj->header.flags = 0;
    return status;
}

/*
 * Dump a name
 */
PRIVATE fs_Status
fs_dumpName(
    fs_Context          *context,
    fs_DumpContext_t    *dumpContext,
    fs_Inode            *inode,
    char                *newPath,
    int                 isTmp
)
{
    int     newPathLen = strlen(newPath);
    int     totLen = newPathLen + 1;
    int     availLen;
    uint32  nHeaders = 1;
    fs_DumpOperation_t  *obj;
    char                *cp, *tp;
    fs_Status           status;

    /*
     * If have been using the cpu since a long time, give others a chance
     */

    fs_yieldPeriodic(context);

#ifdef FS_DEBUG
    if (FS_I_IS_SYML(inode->dinode.attrs)) {
        FS_TRACE(context, FS_TRACE_DUMP_RESTORE,
              ("Dumping name for symlink %s", newPath));
    }
#endif
    FS_TRACE(context, FS_TRACE_DUMP_RESTORE,
              ("In fs_dumpName for %s is tmpp %d", newPath, isTmp));
    totLen -= (FS_DR_TBSIZE - sizeof (fs_DumpOperation_t));
    while (totLen > 0) {
        nHeaders++;
        totLen -= FS_DR_TBSIZE;
    }

    totLen = newPathLen + 1;
    obj = &dumpContext->record.u.operation;
    obj->header.headerId = dumpContext->currentId++;
    obj->header.headerSubId = 0;
    obj->len =  totLen;
    obj->sourceNameLen =  newPathLen;
    obj->header.dataExtents = 0;
    obj->header.attr.uid = inode->sd->u.nfs.uid;
    obj->header.attr.gid = inode->sd->u.nfs.gid;
    obj->header.attr.sizeDev.size = inode->dinode.size;
    obj->header.attr.ctime = inode->dinode.ctime;
    obj->header.attr.mtime = inode->dinode.mtime;
    obj->header.attr.atime = inode->dinode.atime;
    //obj->header.attr.mode = inode->sd->u.nfs.mode & FS_IMODE_MASK;
    obj->header.attr.mode = (inode->dinode.attrs &
                                 (FS_IFMT_MASK|FS_IATR_MASK)) |
                                               inode->sd->u.nfs.mode;
    obj->header.attr.simpleAttrs = inode->dinode.userAttr;

    if (FS_I_IS_CHR(inode->dinode.attrs) ||
        FS_I_IS_BLK(inode->dinode.attrs)) {
        obj->header.attr.sizeDev.devNum.major = inode->dinode.u.devNum.major;
        obj->header.attr.sizeDev.devNum.minor = inode->dinode.u.devNum.minor;
    }

    obj->header.op = FS_DR_NAME_ONLY;
    if (isTmp) {
        FS_TRACE(context, FS_TRACE_DUMP_RESTORE,
              ("Dump to temp"));
        obj->header.flags = FS_DR_TEMP_NAME;
    }

    FS_TRACE(context, FS_TRACE_DUMP_RESTORE,
              ("dumping name with id %lld for %s",
              obj->header.headerId, newPath));

    /* TODO RAHUL: check this comment. sounds wrong
     * Now copy both the names to the buffer. First copy old name
     * and then the new name. Both the names are null terminated
     */

    totLen = newPathLen + 1;
    cp = newPath;
    tp = &dumpContext->record.u.dummy[sizeof (fs_DumpOperation_t)];
    availLen = &dumpContext->record.u.dummy[FS_DR_TBSIZE] - tp;
    FS_TRACE(context, FS_TRACE_DUMP_RESTORE,
              ("dumpName:dumped %s in header", cp));
    if (totLen < availLen) {
        memcpy(tp, cp, totLen);
        tp += totLen;
        availLen -= totLen;
    } else {
        memcpy(tp, cp, availLen);
        totLen -= availLen;
        while (totLen > 0)  {
            status = fs_writeTape(context, dumpContext);
            if (status != FS_ERR_NONE) {
                obj->header.flags = 0;
                return status;
            }
            tp = dumpContext->record.u.dummy;
            cp += availLen;
            availLen = FS_DR_TBSIZE;
            if (totLen < availLen) {
                memcpy(tp, cp, totLen);
                totLen = 0;
                tp += totLen;
                availLen -= totLen;
            } else {
                memcpy(tp, cp, availLen);
                totLen -= availLen;
            }
        }
    }
#if 0    
    strncpy(tp, cp, availLen);
    tp += availLen;
    totLen -= availLen;
    while (totLen > 0) {
        status = fs_writeTape(context, dumpContext);
        if (status != FS_ERR_NONE) {
            obj->header.flags = 0;
            return status;
        }
        tp = dumpContext->record.u.dummy;
        bzero(tp, FS_DR_TBSIZE);
        cp += availLen;
        bcopy(tp, cp, availLen);
        totLen -= availLen;
        tp += availLen;
        availLen = FS_DR_TBSIZE;
    }
#endif    
    status = fs_writeTape(context, dumpContext);
    if (!fs_isInodeLinked(context, dumpContext, inode, newPath)) {
        fs_markInodeCreated(context, dumpContext, inode, newPath, newPathLen,
                            isTmp);
    }
    obj->header.flags = 0;
    return status;
}

PRIVATE fs_Status
fs_dumpNonContainerInode(
    fs_Context              *context,
    fs_DumpContext_t        *dumpContext,
    fs_Inode                *parent,
    fs_Inode                *inode,
    char                    *name,
    int                     nameLen,
    int                     dumpName
)
{
    char                buf[FS_MAXPATHLEN + 1];
    char                *cp;
    int                 isTmp = 0;
    fs_Status           status = FS_ERR_NONE;

    /*
     * If have been using the cpu since a long time, give others a chance
     */

    fs_yieldPeriodic(context);

    FS_TRACE(context, FS_TRACE_DUMP_RESTORE,
              ("Dump Non Container Node %s", name));
    FS_TRACE(context, FS_TRACE_DUMP_RESTORE,
              ("Dump Non Container inum %lld snapnum %d", inode->inum,
                inode->snapnum));
    if (dumpName) {
        buf[FS_MAXPATHLEN + 1] = '\0';
        cp = &buf[FS_MAXPATHLEN + 1];
        cp -= nameLen;
        strncpy(cp, name, nameLen);
        fs_getNewPath(context, dumpContext, parent, &cp, &isTmp, 0);
        status = fs_dumpName(context, dumpContext, inode, cp,
                             isTmp  | fs_isInodePathTemp(context, dumpContext,
                                                     inode, cp));
        if (status != FS_ERR_NONE) {
            return status;
        }

    }

    status = fs_dumpRegExtents(context, dumpContext, inode);
    dumpContext->currentId++;
    return status;
}

PRIVATE fs_Status
fs_dumpRegExtents(
    fs_Context          *context,
    fs_DumpContext_t    *dumpContext,
    fs_Inode            *inode
)
{
    fs_DumpDataExtension_t      *dataObj;
    fs_Status                   status;
    uint32                      maxNumExtents = 0;
    uint32                      numExtents = 0;
    uint64                      curOffset = 0, start;
    fs_Emappair                 p;
    fs_Extent                   extent;
    uint64                      len;
    fs_Block                    block;
    fs_DumpExtent_t             *drExtent, *sdrExtent;
    uint64                      curId;
    uint32                      objSize;

    /*
     * If have been using the cpu since a long time, give others a chance
     */

    fs_yieldPeriodic(context);

    FS_TRACE(context, FS_TRACE_DUMP_RESTORE, ("In fs_dumpRegExtents - %s",
             dumpContext->currentName));
    curId = dumpContext->currentId;
    FS_TRACE(context, FS_TRACE_DUMP_RESTORE,
             ("dumping extents with id %lld ", curId));

    /*
     * If there are more blocks to dump, then create extention objects
     */

    numExtents = 0;
    dataObj = &dumpContext->record.u.extension;
    dataObj->header.headerId = curId;
    dataObj->header.headerSubId++;
    dataObj->header.op = FS_DR_DATA;
    objSize = sizeof (fs_DumpDataExtension_t);

    /*
     * If this is a symlink, create 1 extent
     */

    if (FS_I_IS_SYML(inode->dinode.attrs)) {
        char        dataBuf[FS_DR_TBSIZE];

        /*
         * If the symlink is stored in the inode, read it off the inode
         * else read it as if it is a regular file
         */

        FS_TRACE(context, FS_TRACE_DUMP_RESTORE,
            ("Dumping Reg extents for symlink via inline data for %s",
            dumpContext->currentName));
        if (inode->dinode.size <= FS_INODE_INLINE_LEN) {
            numExtents = 1;
            drExtent = dataObj->extents;
            drExtent->offset = 0;
            drExtent->len = 1;
            dataObj->header.dataExtents = numExtents;
            /*
             * First write the object record
             */

            status = fs_writeTape(context, dumpContext);
            if (status != FS_ERR_NONE) {
                return status;
            }

            /*
             * Now write the data to tape
             */
            memcpy(dataBuf, inode->dinode.u.data, inode->dinode.size);
            dataBuf[inode->dinode.size] = '\0';
            FS_TRACE(context, FS_TRACE_DUMP_RESTORE,
                ("Writing symlink %s to %s", dumpContext->currentName,
               dataBuf));
            status = fs_writeDataTape(context, dumpContext, dataBuf);
            return status;
        }
    }
more:
    if (curOffset >= inode->dinode.size) {
        return FS_ERR_NONE;
    }
    maxNumExtents = FS_DR_MAXEXTENTS;
    drExtent = dataObj->extents;
    sdrExtent = drExtent;
    while (curOffset < inode->dinode.size && numExtents < maxNumExtents) {
        p = fs_emap_lookup(context, inode, curOffset >> FS_BLOCK_SHIFT);
		if (fs_extentLength(p.extent) == 0) {
            curOffset += FS_BLOCK_SIZE;
            continue;
        }
        extent = p.extent;
        len = fs_extentLength(extent);
        block = fs_extentStartBlock(extent);
        if (numExtents < maxNumExtents) {
            drExtent->offset = curOffset;
            /*
             * Store extent length in tape record multiples
             */

            len = MIN(len << FS_BLOCK_SHIFT, inode->dinode.size - curOffset);
            len = (len + FS_DR_TBSIZE - 1) & (~(FS_DR_TBSIZE - 1));
            drExtent->len = (len >> FS_DR_TBSHIFT);
            FS_TRACE(context, FS_TRACE_DUMP_RESTORE,
          ("Number of blocks %lld", len * (FS_BLOCK_SIZE / FS_DR_TBSIZE)));
            numExtents++;
            drExtent++;
            curOffset += len;
        }
    }
    dataObj->header.dataExtents = numExtents;
    /*
     * First write the object record
     */

    FS_TRACE(context, FS_TRACE_DUMP_RESTORE,
             ("Dumping %d extents for id %lld", numExtents, curId));
    status = fs_writeTape(context, dumpContext);
    if (status != FS_ERR_NONE) {
        return status;
    }

    status = fs_dumpBlocks(context, dumpContext, inode, start, sdrExtent,
                           numExtents);
    if (status != FS_ERR_NONE) {
        return status;
    }

    dataObj->header.headerSubId++;
    numExtents = 0;
    goto more;
}

EXPORTED fs_Status
fs_incrementalDump(
    fs_Context          *context,
    fs_DumpContext_t    *dumpContext
)
{
    fs_Status           status;
    fs_Inode            *rootInode;
	int					sentHistory = 1;
/*
elog_enabled = TRUE;
elog_severity = LOG_DEBUG;
elog_appl_data[fs_applicationId].severity = LOG_DEBUG;
elog_appl_data[fs_applicationId].bitmap |= (1 << FS_TRACE_DUMP_RESTORE);
*/
    /*
     * If have been using the cpu since a long time, give others a chance
     */

    fs_yieldPeriodic(context);
    FS_ASSERT(dumpContext->files == 0);

    FS_TRACE(context, FS_TRACE_DUMP_RESTORE, ("In fs_incrementalDump"));
    status = fs_writeStartRecord(context, dumpContext, FS_DR_INCR, FS_DR_START,
                                 dumpContext->dumpTime,
                                 dumpContext->prevDumpTime);
    dumpContext->currentId++;
    rootInode = I_GET_SHARED(dumpContext->prevSnapNum, FS_ROOT_INODE);
    dumpContext->currentName[0] = '\0';
#if 0
	fs_ndmpSendDumpFileHistory(context, dumpContext, "/", FS_ROOT_INODE,
                               rootInode);
#endif

    /*
     * Push the parent inode on the stack
     */

    status = fs_walkTree(context, dumpContext, NULL, rootInode->inum, "/", 1,
                         &sentHistory);
    if (status != FS_ERR_NONE) {
        FS_TRACE(context, FS_TRACE_DUMP_RESTORE,
              ("walkTree failed with %d", status));
        fs_inodeRelease(context, rootInode);
        return status;
    }
    FS_ASSERT(fs_lockHeldShare(context, &rootInode->lock));
    fs_inodeRelease(context, rootInode);
    rootInode = I_GET_SHARED(dumpContext->prevSnapNum, FS_ROOT_INODE);
    dumpContext->currentName[0] = '\0';
    status = fs_removeUnwantedNodes(context, dumpContext, NULL,
                                    rootInode, "/", 1);

    dumpContext->currentId++;
    status = fs_writeStartRecord(context, dumpContext, FS_DR_INCR, FS_DR_END,
                                 dumpContext->dumpTime,
                                 dumpContext->prevDumpTime);
    FS_TRACE(context, FS_TRACE_DUMP_RESTORE, ("completed tree full dump "));
    fs_walkHistoryNodes(context, dumpContext, fs_fileHistoryProcessDirs);
    fs_ndmpFlushDirHistory(context, dumpContext);
    fs_walkHistoryNodes(context, dumpContext, fs_fileHistoryProcessNodes);
    fs_ndmpFlushNodeHistory(context, dumpContext);
#if 0
    fs_walkHistoryNodes(context, dumpContext, fs_fileHistoryProcessNodes);
    fs_ndmpFlushNodeHistory(context, dumpContext);
#endif
    return status;
}

PRIVATE fs_Status
fs_dumpAttr(
    fs_Context              *context,
    fs_DumpContext_t        *dumpContext,
    fs_Inode                *parent,
    fs_Inode                *inode,
    char                    *name,
    int                     nameLen
)
{
    char                buf[FS_MAXPATHLEN + 1];
    char                *cp;
    int                 isTmp = 0;
    fs_Status           status = FS_ERR_NONE;

    /*
     * If have been using the cpu since a long time, give others a chance
     */

    fs_yieldPeriodic(context);

    FS_TRACE(context, FS_TRACE_DUMP_RESTORE, ("In fs_dumpAttr"));
    buf[FS_MAXPATHLEN + 1] = '\0';
    cp = &buf[FS_MAXPATHLEN + 1];
    cp -= nameLen;
    strcpy(cp, name);
    //optimization, remove when tested
    //strncpy(cp, name, nameLen);
    if (parent != NULL) {
        fs_getNewPath(context, dumpContext, parent, &cp, &isTmp, 0);
    } else {
        FS_ASSERT(inode->inum == FS_ROOT_INODE);
    }
    status = fs_dumpName(context, dumpContext, inode, cp,
                         isTmp | fs_isInodePathTemp(context, dumpContext,
                                                inode, cp));
    return status;
}

/*
 * Do filehistory for incremental dump. We might need to send name attr
 * for parents if required
 */

PRIVATE fs_Status
fs_sendIncrFileHistory(fs_Context *context,
                       fs_DumpContext_t *dumpContext,
                       fs_Inode *rootInode,
                       int *sentParentHistory,
                       int *sentHistory,
                       uint64   *block,
                       uint16   *boff)
{
    int32           stackPtr = 0;
    fs_inodeAttr_t  *inodeAttr, *parentAttr = NULL;
    char            *c = dumpContext->currentName; 
    char            *name = c;
    fs_Inode        *inode;
    fs_Inode        *parent;
    fs_Status       status = FS_ERR_NONE;
    int             nameLen;

    if (dumpContext->abort) {
        return FS_ERR_IO;
    }

    /*
     * If have been using the cpu since a long time, give others a chance
     */

    fs_yieldPeriodic(context);

#ifdef FS_DEBUG
    FS_TRACE(context, FS_TRACE_DUMP_RESTORE,
              ("IncrFileHistory - printing inums for %s", dumpContext->currentName));
    for (stackPtr = 0; stackPtr < dumpContext->stackIndex;
         stackPtr++) {
        FS_TRACE(context, FS_TRACE_DUMP_RESTORE,
                 ("inum %lld", dumpContext->parentStack[stackPtr].inum));
    }
    stackPtr = 0;
#endif
    while (stackPtr <= dumpContext->stackIndex) {

        inodeAttr = &dumpContext->parentStack[stackPtr];
        /*
         * Check to see, if we need to push a name record on tape
         * for this parent
         */

        c = strchr(c, '/');

        if (inodeAttr->doneHistory == 0) {

            /*
             * Send the dump record and also send fileHistory for
             * this object
             */

            inode = I_GET_SHARED(dumpContext->snapnum, inodeAttr->inum);
            FS_ASSERT(FS_I_IS_DIR(inode->dinode.attrs));
            if (inode->inum != FS_ROOT_INODE) {
                FS_ASSERT(name != NULL);
                parent = I_GET_SHARED(dumpContext->snapnum, 
                                      parentAttr->inum);
                if (c) {
                    *c = '\0';
                }
                FS_TRACE(context, FS_TRACE_DUMP_RESTORE,
                    ("FileHist:calling dumpAttr for %s inum %lld",
                  name, inode->inum));
                nameLen = strlen(name);
                status = fs_dumpAttr(context, dumpContext, parent, inode,
                                     name, nameLen);
                fs_ndmpAddNode(context, dumpContext, parentAttr->block,
                                parentAttr->boff, inode, parent->inum,
                                name, nameLen, &inodeAttr->block, 
                                &inodeAttr->boff);
                fs_inodeRelease(context, parent);
                inodeAttr->doneHistory = 1;

                if (c) {
                    *c = '/';
                }
            } else {
                parent = NULL;
                fs_ndmpAddNode(context, dumpContext, 0, 0,
                               inode, FS_ROOT_INODE, ".", 1, &inodeAttr->block,
                               //inode, 0, "", &inodeAttr->block,
                               &inodeAttr->boff);
                inodeAttr->doneHistory = 1;
            }

            fs_inodeRelease(context, inode);
            if (status != FS_ERR_NONE) {
                return status;
            }
             
             *sentParentHistory = 1;
        }
        stackPtr++;
        parentAttr = inodeAttr;

        name = c + 1;
        c = name;
    }

    /*
     * Send the file history for root
     */
    if (!(*sentHistory)) {
        fs_ndmpAddNode(context, dumpContext, parentAttr->block,
                        parentAttr->boff, rootInode, parentAttr->inum,
                        name, strlen(name), block, boff);
        *sentHistory = 1;
    }
    return FS_ERR_NONE;
}

fs_Status
fs_findCommonPath(
    fs_Context          *context,
    fs_DumpContext_t    *dumpContext,
    fs_Inode            *inode,
    char                **commonPath,
    int                 *isTmp)
{
    fs_Inode    *shadowInode;
    fs_Inumber  oldParentInumber =  0, newInum;
    fs_Inode    *oldParent, *newParent;
    uint64      oldCookie =  0;
    fs_Status   status = FS_ERR_NONE;
    char        *path, *cp, *tp;
    int         len;

    *commonPath = NULL;
    shadowInode = I_GET_SHARED(dumpContext->prevSnapNum, inode->inum);
    if (shadowInode == NULL) {
        //D("findCommonPath: returnin noent as did not exist in old snapshot\n");
        return FS_ERR_NOENT;
    }

    if (shadowInode->dinode.genNum != inode->dinode.genNum) {
        fs_inodeRelease(context, shadowInode);
        //D("findCommonPath: returning noent as exist as different in old snapshot\n");
        return FS_ERR_NOENT;
    }

    FS_ASSERT(path != NULL);
next_old_parent:
    fs_findParent(context, dumpContext->prevSnapNum, shadowInode->inum,
                  &oldParentInumber, &oldCookie);
    FS_ASSERT(oldParentInumber != 0);

    oldParent = I_GET_SHARED(dumpContext->prevSnapNum, oldParentInumber);
    newParent = I_GET_SHARED(dumpContext->prevSnapNum, oldParentInumber);

    if (newParent == NULL ||
        newParent->dinode.genNum != oldParent->dinode.genNum) {

        /*
         * This parent no longer exists in the new snapshot, so
         * this cannot be part of the common path
         */

        if (newParent) {
            fs_inodeRelease(context, newParent);
        }
        goto next_old_parent;
    }
    status = fs_getName(context, dumpContext, oldParent, shadowInode, &tp, 
                        &len, NULL);
    if (status != FS_ERR_NONE) {
            if (tp) {
                fs_ramDealloc(FS_RAM_DUMP_NAME, tp);
                fs_inodeRelease(context, oldParent);
                fs_inodeRelease(context, newParent);
                fs_inodeRelease(context, shadowInode);
                return status;
            }
    }

    /*
     * Check if the name exists in new parent as well
     */

    newInum = fs_dir_lookupent(context, newParent, tp, len);
    if (newInum != inode->inum) {

        /*
         * The entry does not exist in the new parent, look further
         */

        fs_inodeRelease(context, newParent);
        fs_inodeRelease(context, oldParent);
        goto next_old_parent;
    }

    fs_inodeRelease(context, newParent);
    
    if (tp) {
        path = fs_ramAlloc(FS_RAM_DUMP_NAME, FS_MAXPATHLEN + 1);
        //strcpy(&path[FS_MAXPATHLEN + 1] - strlen(tp) - 2, "/");
        strcpy(&path[FS_MAXPATHLEN + 1] - len - 1, tp);
    } else {
        fs_inodeRelease(context, shadowInode);
        fs_inodeRelease(context, oldParent);
        D("findCommonPath: returning noent as getName did not return path\n");
        return FS_ERR_NOENT;
    }
    cp = &path[FS_MAXPATHLEN + 1 - len - 1];
    fs_ramDealloc(FS_RAM_DUMP_NAME, tp);
    fs_getNewPath(context, dumpContext, oldParent, &cp, isTmp, 0);
    memmove(path, cp, strlen(cp) + 1);
    *commonPath = path;
    fs_inodeRelease(context, oldParent);
    fs_inodeRelease(context, shadowInode);
    D("findCommonPath: returning %s\n", *commonPath);
    return status;
}
#endif  //SSC
