/*
 * Name:  fs-log-private.h
 *
 * RCS:  $Id: fs-log-private.h,v 1.1 2002/04/19 01:22:50 bobonfrogger Exp $
 *
 * Description:   Private definitions between fs-log.c and fs-log-replay.c
 * 
 * Created By:  Bob Fozard
 *
 * Date Created:  3/20/2001
 */


#if !defined(FS_LOG_PRIVATE_H)
#define FS_LOG_PRIVATE_H


/**
 * Private stuff between fs-log.c and fs-log-replay.c
 */


/**
 * Record the specified dependency type in the specified
 * object for the specified log record.
 */
void
fs_logAddDep(
    fs_Context          *context,
    fs_LogRecHdr        *recHdr,
    fs_LogObjectSumm    *objSumm,
    boolean             addUndo);

/**
 * Scan through the entire log for the specified action.
 */
int32
fs_logScan(
    fs_Context          *context,
    fs_LogFsRoot        *logRoot,
    fs_LogReplay        *replay,
    fs_LogAction        action);

/**
 * Unlock inodes at the end of a transaction.
 */
void
fs_logCommitInodes(
    fs_Context      *context,
    fs_LogSummary   *logSummary);


/**
 * Commit the summmary at the end of a transaction.
 */
int32
fs_logCommitSummary(
    fs_Context      *context,
    fs_LogFsRoot    *logRoot,
    fs_LogSummary   *logSummary);


/**
 * Return the size of the specified log record type.
 */
uint32
fs_logRecSize(
    fs_Context *context,
    int recType);


/**
 * Called to get size of a fs_LogRecord
 */
int32
fs_logGetRecLen(
    fs_Context      *context,
    fs_LogRecord    *record);


/**
 * Caclculate the checksum over the log record passed in
 * the specified vector.  A valid record header must be
 * available in the first vector entry.
 */
uint32
fs_logRecordCalcCksum(
	fs_Context	*context,
	int32		numVec,
	uchar8		*datap[],
	int32		lenp[]);


/**
 * Activate the affected log segments if they are
 * not already activated.
 */
void
fs_logMarkActive(
    fs_Context          *context,
    fs_LogObjectSumm    *objSumm);


#endif /* FS_LOG_PRIVATE_H */
