/*
 * Name:  fs-nvram.c
 *
 * RCS:  $Id: fs-nvram.h,v 1.8 2002/06/21 21:15:43 janv Exp $
 *
 * Description:  The file system specific functions for nvram
 *
 * Created By:  Jan Vanhoutte
 *
 * Date Created:  11/10/2001
 */

#if !defined _FS_NVRAM_H_
#define _FS_NVRAM_H_

/*-----------------------------------------------------------------------------
 * fs_logReadNvramDevHdr
 *
 * Description: Called to read the device header
 *              and also to verify the cksum, version, magic etc
 *              return NULL if not found.
 *
 * Created By:  Jan Vanhoutte
 *
 * Date Created:  9/20/2001
 */

extern int32
fs_logReadNvramDevHdr(
    fs_LogFsRoot    *logRoot,
    fs_LogDevHdr    *devHdr) ;


/*-----------------------------------------------------------------------------
 * fs_logNvramWriteDevHdr
 *
 * Description: Called to write the device header
 *              and also update the cksum, version, magic etc
 *
 * Created By:  Jan Vanhoutte
 *
 * Date Created:  9/20/2001
 */

extern int32
fs_logNvramWriteDevHdr(
    fs_Context      *context,
    int32           operation,
    uint32          count);


/*-----------------------------------------------------------------------------
 * fs_logReadNvram
 *
 * Description: Called to read something in the NVRAM
 *              No path of return, if it fails
 *
 * Created By:  Jan Vanhoutte
 *
 * Date Created:  9/20/2001
 */

extern void
fs_logReadNvram(
    fs_LogFsRoot    *logRoot,
    uchar8          *data,
    uint32          offset,
    uint32          size) ;

/*-----------------------------------------------------------------------------
 * fs_logNvramWrite
 *
 * Description: Called to write something in the NVRAM
 *              No path of return, if it fails
 *
 * Created By:  Jan Vanhoutte
 *
 * Date Created:  9/20/2001
 */

extern void
fs_logNvramWrite(
    fs_Context          *context,
    uint32              *offset,
    uchar8              *datap[LOG_IOVEC_MAX],
    uint32              lenp[LOG_IOVEC_MAX],
    fs_LogStableWrite   stable,
    uint64              inum);

/*-----------------------------------------------------------------------------
 * fs_logInitNvram
 *
 * Description: Called to find/allocate and initialize a log device.
 *
 * Created By:  Bob Fozard
 *
 * Date Created:  3/20/2001
 */

extern int32
fs_logNvramInitDevice(
    fs_Context      *context);


extern int32
fs_logNvramCommitWrite(
    fs_Context          *context,
    fs_Inode            *inode,
    fs_WriteStableHow   *stableHow);

extern int32
fs_logNvramRecordCksum(
    fs_Context      *context,
    fs_LogRecHdr    *recordHdr,
    int32           recordHdrLen,
    void            *data,
    int32           dataLen);

/*-----------------------------------------------------------------------------
 * fs_logNvramReadInit, fs_logNvramRead, fs_logNvramReadDone, fs_logNvramVerifyCksum
 *
 * Description: NVRAM read routines
 *
 * Created By:  Lan Chin
 *
 * Date Created:  2/22/2002
 */

extern int32
fs_logNvramReadInit(
    fs_Context  *context,
    fs_LogInfo  *info);

extern int32
fs_logNvramRead(
    fs_Context  *context,
    uchar8      **datap,
    uint32      offset,
    uint32      *bytesRead,
    uint32      tail);

extern void
fs_logNvramReadDone(
    fs_Context  *context,
    uchar8      *datap);

extern int32
fs_logNvramVerifyCksum(
    fs_Context  *context,
    uint32      *data,
    uint32      length);

extern int32
fs_logNvramIsEmpty(
    fs_Context  *context);

#endif //  _FS_NVRAM_H_

