/*
 * Name		: fs-restore.h
 * RCS		: $Id: fs-restore.h,v 1.17 2002/11/05 22:40:05 laurentb Exp $
 * Description	: Restore implementation
 * Created by	: Rahul Khona
 * Date Created	: 7/11/2001
 */

#ifndef _FS_RESTORE_H_
#define _FS_RESTORE_H_

#include "fs-dumprestore.h"

#define	FS_MAX_TAPE_READ_PENDING	12
typedef struct fs_RestoreIoQueue	fs_RestoreIoQueue_t;
struct fs_MsgRestoreIODoneReq;

struct fs_RestoreIoQueue {
		DLL_ENTRY(fs_RestoreIoQueue_t)	list;
		uint64							sequence;
		struct fs_MsgRestoreIODoneReq	*fsReq;
};

typedef struct fs_RestoreNameList fs_RestoreNameList_t;
struct fs_RestoreNameList {
    DLL_ENTRY(fs_RestoreNameList_t)    link;
    char                            *name;
    char                            *newPath;
    uint16                          nameLen;
    uint16                          newPathLen;
	int							    restored;
};

typedef struct fs_RestoreContext fs_RestoreContext_t;
struct fs_RestoreContext {
    DLL_ENTRY(fs_RestoreContext_t)    restoreList;   // hangs of fileSystem
    DLL_HDR(fs_RestoreNameList_t)   nameList;
    uint32                   cookie;    // ndmp cookie
    fs_DumpHeader_t         record;
    char                    currentName[FS_MAXPATHLEN + 1];
    char                    *cwd;
    fs_Inumber              cwdInum;
    uint32                  iobufLen;
    uint32                  aborted;
//    char                    *iobuf;
    int                     recursiveDescent;
    int                     extract;
    fs_Inumber              tempInodeNum;
    int                     isSourceTemp;
    int                     isTargetTemp;
    char                    sourceBuf[FS_MAXPATHLEN + 1];
    char                    targetBuf[FS_MAXPATHLEN + 1];
    uint16                  sourceBufLen;
    uint16                  targetBufLen;
    fs_DRAttr_t             attr;
    uint64                  currentId;
    esm_handle_t            smHandle;               // handle of thread doing
                                                    // dump
    int                     abort;                  // abort dump
    uint32                  curOffset;              // current io offset
	uint64					ioSequence;
	uint64					neSequence;		// next expected sequence
#ifndef FS_STANDALONE
	int8					pendingIo;
	uint8					ioWait;
#else
    int                     pendingIo;
    int                     ioWait;
#endif
    int                     maxPendingIo;
	nfx_spinlock_t			spinLock;
    eeeRTC_t                totTapeTime;
    uint32                  totTapeIo;
    uint32                  totIoWaits;
    uint64                  numBytes;
	DLL_HDR(fs_RestoreIoQueue_t)	ioList;
#ifdef FS_STANDALONE
    int             fd;                     // file descriptor where restore
                                            // reads its input
#endif  /* FS_STANDALONE */
    char                    iobuf[1];
};


//Structure for non recursive restoreFullTree function
typedef struct fs_restoreFullTreeContext	fs_restoreFullTreeContext_t;
struct fs_restoreFullTreeContext	{
	//Attached to restoreFullTree function
	DLL_ENTRY(fs_restoreFullTreeContext_t) list;
    fs_DRAttr_t             attr;
	fs_Inode				**root;
    fs_Inode                *inode;
	char					*buf;
	fs_DRAttr_t				*dirAttr;
    fs_Inumber              rootInum;
};


#if 0
extern fs_Status
fs_readDataTape(
    fs_Context          *context,
    fs_RestoreContext_t    *restoreContext,
    char                *buf,
    uint32              requestSize,
    uint32              *ioSize
);
#endif
extern fs_Status
fs_readDataTape(
    fs_Context          *context,
    fs_RestoreContext_t    *restoreContext,
    char                *buf
);
extern fs_Status
fs_readTape(
    fs_Context          *context,
    fs_RestoreContext_t    *restoreContext
);
extern fs_Status
fs_restorePushBackRead(
    fs_Context          *context,
    fs_RestoreContext_t *restoreContext
);
extern fs_Status
fs_restore(
    fs_Context          *context,
    fs_RestoreContext_t    *restoreContext
);
extern fs_Status
fs_fullRestore(
    fs_Context          *context,
    fs_RestoreContext_t *restoreContext
);
extern int
fs_isRestored(
    fs_Context          *context,
    fs_RestoreContext_t  *restoreContext,
    char                *path
);
extern int
fs_shouldIgnore(
    fs_Context          *context,
    fs_RestoreContext_t  *restoreContext,
    char                *buf,
    int                 bufLen,
    char                **newPath
);
extern fs_Status
fs_incrRestore(
    fs_Context              *context,
    fs_RestoreContext_t     *restoreContext
);
extern fs_Status
fs_decomposePath(
    fs_Context          *context,
    fs_RestoreContext_t *restoreContext,
    char                *pathName,
    fs_Inumber          *parent,
    fs_Inumber          *inum,
    char                **name,
    uint16              *nameLen,
    int                 temp
);
extern fs_Status fs_restoreWriteFile(fs_Context *, fs_RestoreContext_t *,
                                      fs_Inode **, char *, uint64, uint64);
extern void fs_cleanRestoreContext(fs_Context *, fs_RestoreContext_t *, boolean);
extern int fs_shouldExtract(fs_Context *, fs_RestoreContext_t *, char *, int);
#endif  /* _FS_RESTORE_H_ */
