/*
 * Name:  fs-vol-exer.h
 *
 * RCS:  $Id: fs-vol-exer.h,v 1.11 2002/10/22 01:30:35 jong Exp $
 *
 * Description:  Volume exerciser message interface.
 * 
 * Created By:  Bob Fozard
 */

#if !defined(FS_VOL_EXER_H)
#define FS_VOL_EXER_H


enum {
    FS_VOL_EXER_ACTION_READ     = EVM_READ_REQ,
    FS_VOL_EXER_ACTION_WRITE    = EVM_WRITE_REQ,
    FS_VOL_EXER_ACTION_NONE     = MAX(EVM_READ_REQ, EVM_WRITE_REQ) + 1,
    FS_VOL_EXER_ACTION_MAX      = FS_VOL_EXER_ACTION_NONE,
};


#define RANDOM_STATE_SIZE 256

typedef struct fs_VolExerReadWrite
{
    uint32              totalPendingCount;
    uint32              totalDoneCount;
    uint32              seqPendingCount;
    uint32              seqDoneCount;
    uint64              nextOffset;
    int32               nextNumReqs;
    uchar8              randomState[RANDOM_STATE_SIZE];
    struct random_data  randData;

} fs_VolExerReadWrite;


/**
 * An instance of a volume exerciser.
 */
typedef struct fs_VolExer fs_VolExer;
struct fs_VolExer
{
    esm_handle_t            handle;
    evm_volId_t             volId;
    nfx_cod_t               writeCod;
    uint32                  state;
    uint32                  genNumErrors;
    uint32                  dataErrors;
    uint64                  numBlksTotal;   // Total 8K blocks in volume
    uint64                  numBlksRandom;  // Space starting at 0 for random
                                            // IO (ie. the filesystem area).
    uint64                  numBlksSequential; // Space starting at
                                            // numBlksRandom for sequential
                                            // IO (ie. the log)
    uint64                  currSequentialROffset;  // in FS_BLOCK_SIZE blks
    uint64                  currSequentialWOffset;  // in FS_BLOCK_SIZE blks
    uint64                  seqReadDone;
    uint64                  seqWriteDone;
    uint64                  ioPendingCount; // Count of outstanding IO reqs.
    uint64                  blksRandMask;
    uint64                  attrib;
    uint32                  sequentialIoSize;
    uint32                  load;           // Max IO's outstanding.
    uint32                  seed;           // Seed for random generators.
    uint32                  ioPending;
    uint32                  loadType;       // RANDOM, SEQUENTIAL, BOTH
    uint32                  ioType;         // RW, RO, WO
    fs_VolExerReadWrite     rw[FS_VOL_EXER_ACTION_MAX];
    uchar8                  *reqState;
    uchar8                  actionRandomState[RANDOM_STATE_SIZE];
    struct random_data      actionRandData;
    DLL_ENTRY(fs_VolExer)   dllEntry;
#define FS_TIME_STATS_BUCKETS_MAX 8
    uint32                  respTimeRead[FS_TIME_STATS_BUCKETS_MAX]; //use same buckets as esm.
    eeeRTC_t                respTimeMaxRead;    
    uint32                  ioQDepthMaxRead;     // Max pending requests 
    eeeRTC_t                respTimeMinRead;    
    uint32                  ioQDepthMinRead;     // Min pending requests.
    eeeRTC_t                esmTimeMaxRead;
    eeeRTC_t                esmTimeMinRead;

    uint32                  respTimeWrite[FS_TIME_STATS_BUCKETS_MAX]; //use same buckets as esm.
    eeeRTC_t                respTimeMaxWrite;    
    uint32                  ioQDepthMaxWrite;     // Max pending requests 
    eeeRTC_t                respTimeMinWrite;    
    uint32                  ioQDepthMinWrite;     // Min pending requests.
    eeeRTC_t                esmTimeMaxWrite;
    eeeRTC_t                esmTimeMinWrite;

    eeeRTC_t                totalRespTimeRead;
    eeeRTC_t                totalRespTimeWrite;

    eeeRTC_t                startTime;
    boolean                 isUser;
};

/**
 * volExer request types.
 */
enum {
    FS_VOL_EXER_START       = 1,
    FS_VOL_EXER_STOP        = 2,
    FS_VOL_EXER_STAT        = 3,
};


/**
 * volExer load types.
 */
enum {
    FS_VOL_EXER_TYPE_RANDOM     = 1,
    FS_VOL_EXER_TYPE_SEQUENTIAL = 2,
    FS_VOL_EXER_TYPE_BOTH       = 3,
};


/**
 * volExer io types.
 */
enum {
    FS_VOL_EXER_IOTYPE_RW   = 1,
    FS_VOL_EXER_IOTYPE_RO   = 2,
    FS_VOL_EXER_IOTYPE_WO   = 3,
};

/**
 * volExer req.
 */
typedef struct fs_MsgVolExerReq
{
    fs_MsgReqHdr        hdr;
    evm_volId_t         volId;
    uint64              numBlks;
    uint64              numBlksSequential;
    uint64              sequentialIoSize;

    uint32              what;       // START, STOP, STAT
    uint32              load;

    uint32              seed;
    uint32              loadType;   // RANDOM, SEQUENTIAL, BOTH

    uint32              ioType;     // RW, RO, WO
    boolean             isUser;

    uint64              attrib;     // type of evem io, eg. loopback, no reorder etc.
} fs_MsgVolExerReq;


/** 
 * volExer resp.
 */
typedef struct fs_MsgVolExerResp
{
    fs_MsgReqHdr        hdr;
    evm_volId_t         volId;
    uint32              what;
    uint32              load;

    uint64              numBlks;
    uint64              numBlksSequential;

    uint32              seed;
    uint32              sequentialIoSize;

    uint32              loadType;
    uint32              ioType;

    uint32              readPending;
    uint32              readDone;

    uint32              writePending;
    uint32              writeDone;

    uint32              genNumErrors;
    uint32              dataErrors;

    uint32              avgLatencyRandom;
    uint32              avgLatencySequential;

    uint32              avgTputRandom;
    uint32              avgTputSequential;

    uint32              respTimeRead[FS_TIME_STATS_BUCKETS_MAX]; //use same buckets as esm.
    uint32              respTimeWrite[FS_TIME_STATS_BUCKETS_MAX]; //use same buckets as esm.

    uint32              ioQDepthMaxRead;     // Queue depth at the time we hit max resp time.
    uint32              ioQDepthMaxWrite;     // Queue depth at the time we hit max resp time.

    eeeRTC_t            respTimeMaxRead;
    eeeRTC_t            respTimeMaxWrite;

    eeeRTC_t            respTimeMinRead;
    eeeRTC_t            respTimeMinWrite;

    uint32              ioQDepthMinRead;     // Queue depth at the time we hit min resp time.
    uint32              ioQDepthMinWrite;     // Queue depth at the time we hit min resp time.

    eeeRTC_t            esmTimeMaxRead;      
    eeeRTC_t            esmTimeMaxWrite;      

    eeeRTC_t            esmTimeMinRead;      
    eeeRTC_t            esmTimeMinWrite;      

    eeeRTC_t            totalRespTimeRead;      
    eeeRTC_t            totalRespTimeWrite;      

    eeeRTC_t            runTime;
    boolean             isUser;
	uint64              count_per_usec; 
} fs_MsgVolExerResp;

NFX_ASSERT_STATIC(sizeof(fs_MsgVolExerResp) <= FS_SHELL_RESP_MAX);

typedef DLL_HDR(fs_VolExer) fs_VolExerDll;

extern fs_VolExerDll fs_volExerDllHdr;

#endif // FS_VOL_EXER_H
