/*
 * Name:  fsdb-context.h
 *
 * RCS:  $Id: fsdb-context.h,v 1.3 2002/11/26 22:54:00 bob Exp $
 *
 * Description:  Definitions for FS debugger related stuff.
 *
 * Created By:  Jan Vanhoutte
 *
 * Date Created:  5/20/2002
 */

#if !defined(FSDB_CONTEXT_H)
#define FSDB_CONTEXT_H

#if defined(FS_STANDALONE)
#else
#include <nfx-incl.h>
#include <nfx-types.h>
#endif

#include "fs-includes.h"


#if defined(FS_STANDALONE)
#else
#include <exports/evm-api.h>
#endif

typedef enum {
    // also used as menu entries ( => 0 == return)
    // update also fsdb_dataTypeString
    FSDB_DATA_TYPE_NO_DATA = 0,
    FSDB_DATA_TYPE_UNKNOWN = 1,
    FSDB_DATA_TYPE_SUPER,
    FSDB_DATA_TYPE_D_INODE,
    FSDB_DATA_TYPE_REF_BYTE,
    FSDB_DATA_TYPE_BTREE,
    FSDB_DATA_TYPE_DIR_LEAF,
    FSDB_DATA_TYPE_DIR_LEAF_COMPACT,
    FSDB_DATA_TYPE_EMAP_LEAF,
    FSDB_DATA_TYPE_FILE_SETS,
    FSDB_DATA_TYPE_BUDDY,
    FSDB_DATA_TYPE_FULL_INODE_BLOCK,
    FSDB_DATA_TYPE_DIR_IN_INODE_BLOCK,
    FSDB_DATA_TYPE_VOL_LOG_BUF,
    FSDB_DATA_TYPE_D_INODE_ONE_ENTRY,
//    FSDB_DATA_TYPE_QUOTA,
//    FSDB_DATA_TYPE_VOL_LOG_BUFFERS,
//    FSDB_DATA_TYPE_VOL_LOG_RECORDS,
    FSDB_DATA_TYPE_MAX
} fsdb_DataType;

typedef struct fsdb_LunInfo
{
    uint32      sdmHandle;
#define SDM_INVALID_HANDLE      0

    uint64      scsiId;		// Phys Storage volume
    int64		size;		// in bytes
    uint32		startBlk;
    uint32      nbr;

    uint32      sdmState;

    int64       wwn;
    int32       lun;
} fsdb_LunInfo;

#define SDM_BLOCK_SIZE          512
#define FSDB_MAX_NBR_LUN        128    // Todo: make dyn

typedef struct fsdb_VolumeInfo
{
    uint32                ready;

    char                  mountName[FS_NAME_MAX + 1];
    evm_volId_t           volId;
    uint64                currSize;
    uint32                evmState;
    uint32                evmOnLine;
    uint32                fsState;

    uint32                mkfsTime;                      // 0 means initialized

    int32                 nbrLuns;
    fsdb_LunInfo          lunInfo[FSDB_MAX_NBR_LUN];

    uint32                dataAvailable;
    fsdb_DataType         dataType;
    uint64                offset;
    char                  data[FS_BLOCK_SIZE];
} fsdb_VolumeInfo;


typedef enum {
    FSDB_SET_DUMP_DEFAULT,
    FSDB_SET_DUMP_LOCAL,
    FSDB_SHOW_DUMP,
} fsdb_SetDumpAction;

typedef enum {
    FSDB_DUMP_LEVEL_MINIMUM,
    FSDB_DUMP_LEVEL_MEDIUM,
    FSDB_DUMP_LEVEL_MAXIMUM,
    FSDB_DUMP_LEVEL_MAX
} fsdb_DumpLevel;

typedef enum {
    FSDB_DUMP_1_BYTE,
    FSDB_DUMP_2_BYTES,
    FSDB_DUMP_4_BYTES,
    FSDB_DUMP_8_BYTES,
    FSDB_DUMP_BYTES_MAX
} fsdb_DumpWordSize;

typedef enum {
    FSDB_DUMP_HEX,
    FSDB_DUMP_DEC,
    FSDB_DUMP_OCT,
    FSDB_DUMP_BIN,
    FSDB_DUMP_FORMAT_MAX
} fsdb_DumpFormat;

typedef struct fsdb_DumpFlags
{
    fsdb_DumpLevel          level;
    fsdb_DumpWordSize       size;
    fsdb_DumpFormat         format;
    uint32                  showBuddyBits;
    char                    typeStr[4];

    int32                   wordSize;
    int32                   radix;
} fsdb_DumpFlags;


extern int32    // sdmHandle
fsdb_sdm_ui_getSdmHandle( 
    int64 wwn, 
    int32 lun 
    );

extern int32   // NFX_OK, with data, else NFX_ERR
fsdb_sdm_disk_read(
    uint32     handle, 
    uint64     offset,        // offset on the LUN
    uint32     byteCount,     // nbr of bytes to read (multiple of 1024 (2 sdm blocks)
    char       *data          // mem location to copy data (min byte_count size)
    );

extern int32   // NFX_OK, with data, else NFX_ERR
fsdb_sdm_disk_write(
    uint32     handle, 
    uint64     offset,        // offset on the LUN
    uint32     byteCount,     // nbr of bytes to write (multiple of 1024 (2 sdm blocks)
    char       *data          // mem location to copy data (min byte_count size)
    );


typedef enum {
    FSDB_LUNLABEL_NO_ACTION,
    FSDB_LUNLABEL_WRITE_MANAGEMENT,
    FSDB_LUNLABEL_WRITE_TAKEOVER,
} fsdb_LunLabelAction;


extern int32
fsdb_evm_read_label(
    fsdb_VolumeInfo      *info,
    int32                lunNbr,
    fsdb_LunLabelAction  action);


extern int32 
fsdb_get_volid(
    char *id_str, 
    evm_volId_t *volId);


extern int32
fsdb_get_evm_vol_cfg(
    evm_volId_t          volId,
    fsdb_VolumeInfo      *info);



#endif //  FSDB_CONTEXT_H



