/*------------------------------------------------------------------------------
 * RCS:  $Id: fssf-debug.c,v 1.3 2002/03/20 04:35:10 isabellec Exp $
 * Copyright (C) 2001, Agile Storage, Inc.
 *
 * Name         : fssf-debug.c
 *
 * Description  : File System State-Full (FSSF) - debug functions
 *
 * Created by   : Isabelle Cnudde
 *
 * Date Created : 10-MAR-2002
 *
 *------------------------------------------------------------------------------
 */

#include "fssf.h"

/*------------------------------------------------------------------------------
 * Local function prototypes.
 *------------------------------------------------------------------------------
 */

static void
fssf_displayStats(void);

static int32
fssf_adminStatCmd(int32      argc,
                  char8 *    argv[]);

/*------------------------------------------------------------------------------
 * Name         : fssf_displayStats
 *
 * Description  : fssfadmin UI command to show or clear FSSF global statistics
 *
 * Created by   : Isabelle
 *
 * Date Created : 19-MAR-2002
 *------------------------------------------------------------------------------
 */

static void
fssf_displayStats(void)
    
{
    NFX_SPIN_LOCK(&fssf_stats_lock);
    printf("Client requests/responses:\n");
    printf("\tNULL ...................................... %lld/%lld\n",
           fssfStats.nNullReq, fssfStats.nNullRsp);
    printf("\tCLOSE ..................................... %lld/%lld\n",
           fssfStats.nCloseReq, fssfStats.nCloseRsp);
    printf("\tCREATE .................................... %lld/%lld\n",
           fssfStats.nCreateReq, fssfStats.nCreateRsp);
    printf("\tLOCK ...................................... %lld/%lld\n",
           fssfStats.nLockReq, fssfStats.nLockRsp);
    printf("\tOPEN ...................................... %lld/%lld\n",
           fssfStats.nOpenReq, fssfStats.nOpenRsp);
    printf("\tREAD ...................................... %lld/%lld\n",
           fssfStats.nReadReq, fssfStats.nReadRsp);
    printf("\tREMOVE .................................... %lld/%lld\n",
           fssfStats.nRemoveReq, fssfStats.nRemoveRsp);
    printf("\tSETATTR ................................... %lld/%lld\n",
           fssfStats.nSetattrReq, fssfStats.nSetattrRsp);
    printf("\tUNLOCK .................................... %lld/%lld\n",
           fssfStats.nUnlockReq, fssfStats.nUnlockRsp);
    printf("\tWRITE ..................................... %lld/%lld\n",
           fssfStats.nWriteReq, fssfStats.nWriteRsp);
    printf("FS requests/responses:\n");
    printf("\tCREATE .................................... %lld/%lld\n",
           fssfStats.nFsCreateReq, fssfStats.nFsCreateReq);
    printf("\tLOOKUP .................................... %lld/%lld\n",
           fssfStats.nFsLookupReq, fssfStats.nFsLookupReq);
    printf("Memory:\n");
    printf("\tAlloc/Dealloc ............................. %lld/%lld\n",
           fssfStats.nAlloc, fssfStats.nDealloc);
    NFX_SPIN_UNLOCK(&fssf_stats_lock);
}

/*------------------------------------------------------------------------------
 * Name         : fssf_adminStatCmd
 *
 * Description  : fssfadmin UI command to show or clear FSSF global statistics
 *
 * Created by   : Isabelle
 *
 * Date Created : 19-MAR-2002
 *------------------------------------------------------------------------------
 */

static int32
fssf_adminStatCmd(int32      argc,
                  char8 *    argv[])
    
{
    if (argc != 3)
    {
        printf("USAGE: fssfadmin stat {show|clear}\n");
        return NFX_ERR;
    }
    
    if (!strcmp(argv[2], "show"))
    {
        fssf_displayStats();
    }
    else if (!strcmp(argv[2], "clear"))
    {
        NFX_SPIN_LOCK(&fssf_stats_lock);
        bzero((void *) &fssfStats, sizeof(fssf_stats_t));
        printf("Done.\n");
        NFX_SPIN_UNLOCK(&fssf_stats_lock);
    }
    else
    {
        printf("USAGE: fssfadmin stat {show|clear}\n");
        return NFX_ERR;
    }
    
    return NFX_OK;
}

/*------------------------------------------------------------------------------
 * Name         : fssf_admin
 *
 * Description  : FSSF UI commands (fssfmadmin) see sm-ui/cmdtable.c
 *
 * Created by   : Isabelle Cnudde
 *
 * Date Created : 19-MAR-2002
 *------------------------------------------------------------------------------
 */

int32
fssf_admin(int32    argc,
           char8 *  argv[])
    
{
    int32   rc;
    char8 * cmd;

    if (argc < 2)
    {
        return NFX_ERR;
    }
    cmd = argv[1];
    if (strcmp(cmd, "stat") == 0)
    {
        rc = fssf_adminStatCmd(argc, argv);
    }
    else
    {
        rc = NFX_ERR;
    }
    
    return(rc);
}

/*----------------------------------------------------------------------------*/
