/*------------------------------------------------------------------------------
 * RCS:  $Id: fssf-lock.c,v 1.1 2002/03/27 00:09:48 isabellec Exp $
 * Copyright (C) 2001, Agile Storage, Inc.
 *
 * Name         : fssf-lock.c
 *
 * Description  : File System State-Full (FSSF)
 *                Operations on the lockTab.
 *
 * Created by   : Isabelle Cnudde
 *
 * Date Created : 26-MAR-2002
 *
 *------------------------------------------------------------------------------
 */

#include "fssf.h"
#include "fssf-hash.h"

/*------------------------------------------------------------------------------
 * Local function prototypes.
 *------------------------------------------------------------------------------
 */

static void
fssf_growLockTab(int growSize);

/*------------------------------------------------------------------------------
 * Local functions.
 *------------------------------------------------------------------------------
 */

/*------------------------------------------------------------------------------
 * Name         : fssf_growLockTab
 *
 * Description  : Grow the lock states table.
 *
 * Created by   : Isabelle Cnudde
 *
 * Date Created : 10-MAR-2002
 *
 *------------------------------------------------------------------------------
 */

static void
fssf_growLockTab(int growSize)

{
    int             i;
    fssf_lock_t *  tab = NULL;
    fssf_lock_t ** ptab = NULL;
    
    NFX_SPIN_LOCK(&fssf_tab_lock);
    tab = (fssf_lock_t *) fssf_ramAllocZero(growSize * sizeof(fssf_lock_t ));
    ptab = (fssf_lock_t **) fssf_ramAllocZero((lockTab.size + growSize) *
                                              sizeof(fssf_lock_t *));
    for (i = 0; i < lockTab.size; i++)
    {
        ptab[i] = lockTab.tab[i];
    } // for
    for (i = lockTab.size; i < (lockTab.size + growSize); i++)
    {
        ptab[i] = &tab[i - lockTab.size];
        DLL_INSERT_TAIL(&(lockTab.freeList), ptab[i], freeLock);
    } // for
    fssf_ramDealloc(lockTab.tab);
    lockTab.tab = ptab;
    lockTab.size = lockTab.size + growSize;
    NFX_SPIN_UNLOCK(&fssf_tab_lock);
    return;
}

/*------------------------------------------------------------------------------
 * Global functions.
 *------------------------------------------------------------------------------
 */

/*------------------------------------------------------------------------------
 * Name         : fssf_initLockTab
 *
 * Description  : Initialize the lock states table.
 *
 * Created by   : Isabelle Cnudde
 *
 * Date Created : 10-MAR-2002
 *
 *------------------------------------------------------------------------------
 */

void
fssf_initLockTab(int initSize)

{
    int             i;
    fssf_lock_t *  tab = NULL;
    fssf_lock_t ** ptab = NULL;
        
    NFX_SPIN_LOCK(&fssf_tab_lock);
    tab = (fssf_lock_t *) fssf_ramAllocZero(initSize * sizeof(fssf_lock_t ));
    ptab = (fssf_lock_t **) fssf_ramAllocZero(initSize * sizeof(fssf_lock_t *));
    DLL_INIT(&(lockTab.freeList));
    for (i = 0; i < initSize; i++)
    {
        ptab[i] = &tab[i];
        DLL_INSERT_TAIL(&(lockTab.freeList), ptab[i], freeLock);
    } // for
    lockTab.size = initSize;
    lockTab.tab = ptab;
    NFX_SPIN_UNLOCK(&fssf_tab_lock);
    return;
}

/*----------------------------------------------------------------------------*/
