/*------------------------------------------------------------------------------
 * RCS:  $Id: fssf-utils.c,v 1.7 2002/03/27 00:09:48 isabellec Exp $
 * Copyright (C) 2001, Agile Storage, Inc.
 *
 * Name         : fssf-utils.c
 *
 * Description  : File System State-Full (FSSF)
 *                Utilities.
 *
 * Created by   : Isabelle Cnudde
 *
 * Date Created : 10-MAR-2002
 *
 *------------------------------------------------------------------------------
 */

#include "fssf.h"
#include "fssf-hash.h"

/*------------------------------------------------------------------------------
 * Global functions.
 *------------------------------------------------------------------------------
 */

/*------------------------------------------------------------------------------
 * Name         : fssf_ramAlloc
 *
 * Description  : Allocate memory.
 *
 * Created by   : Isabelle Cnudde
 *
 * Date Created : 26-MAR-2002
 *
 *------------------------------------------------------------------------------
 */

inline void *
fssf_ramAlloc(uint32 size)

{
    void * p = NULL;
    
#ifndef FS_STANDALONE
    p = eee_ramAlloc(size);
#else
    p = fs_ramAlloc(FS_RAM_REQ_MSG, size);
#endif // FS_STANDALONE
    FSSF_STAT_ALLOC;
    if (p == NULL)
    {
        FSSF_ERR_PRINT(("FSSF ERROR: eee_ramAlloc() failed"));
        NFX_ASSERT(TRUE);
    }
    return(p);
}

/*------------------------------------------------------------------------------
 * Name         : fssf_ramAllocZero
 *
 * Description  : Allocate memory and reset it.
 *
 * Created by   : Isabelle Cnudde
 *
 * Date Created : 26-MAR-2002
 *
 *------------------------------------------------------------------------------
 */

inline void *
fssf_ramAllocZero(uint32 size)

{
    void * p = NULL;
    
#ifndef FS_STANDALONE
    p = eee_ramAllocZero(size);
#else
    p = fs_ramAlloc(FS_RAM_REQ_MSG, size);
    bzero(p, size);
#endif // FS_STANDALONE
    FSSF_STAT_ALLOC;
    if (p == NULL)
    {
        FSSF_ERR_PRINT(("FSSF ERROR: eee_ramAllocZero() failed"));
        NFX_ASSERT(TRUE);
    }
    return(p);
}

/*------------------------------------------------------------------------------
 * Name         : fssf_ramDealloc
 *
 * Description  : Deallocate memory.
 *
 * Created by   : Isabelle Cnudde
 *
 * Date Created : 26-MAR-2002
 *
 *------------------------------------------------------------------------------
 */

inline void
fssf_ramDealloc(void * p)

{
    if (p != NULL)
    {
#ifndef FS_STANDALONE
        eee_ramDealloc(p);
#else
        fs_ramDealloc(FS_RAM_REQ_MSG, p);
#endif // FS_STANDALONE
        FSSF_STAT_DEALLOC;
        p = NULL;
    }
    return;
}

/*------------------------------------------------------------------------------
 * Name         : fssf_sendMsg
 *
 * Description  : Send a message.
 *
 * Created by   : Isabelle Cnudde
 *
 * Date Created : 26-MAR-2002
 *
 *------------------------------------------------------------------------------
 */

inline int32
fssf_sendMsg(uint32     src,
             uint32     dest,
             void	*   buf,
             uint32     len)

{
    int32   rc = NFX_OK;

#ifndef FS_STANDALONE
    rc = eee_sendMessage(buf, len, dest, src);
#else
    if (dest == fsPort)
    {
        rc = fs_simSendMsg(FS_SIM_MSG_ADDR_FSSF,
                           (int32) pthread_getspecific(fs_simKeyCpuId),
                           (char *) buf, len);
    }
    else
    {
        rc = fs_simSendMsg(FS_SIM_MSG_ADDR_FSSF,
                           FS_SIM_MSG_ADDR_SSC,
                           (char *) buf, len);
    }        
#endif // FS_STANDALONE
    
    return(rc);
}

/*----------------------------------------------------------------------------*/
