/*------------------------------------------------------------------------------
 * RCS:  $Id: fssf.h,v 1.16 2002/03/27 00:09:48 isabellec Exp $
 * Copyright (C) 2001, Agile Storage, Inc.
 *
 * Name         : fssf.h
 *
 * Description  : File System State-Full (FSSF) - common header file
 *
 * Created by   : Isabelle Cnudde
 *
 * Date Created : 04-MAR-2002
 *
 *------------------------------------------------------------------------------
 */

#ifndef _FSSF_H_
#define _FSSF_H_

/*------------------------------------------------------------------------------
 * Header files.
 *------------------------------------------------------------------------------
 */

#ifndef FS_STANDALONE
#include "nfx-incl.h"
#include "exports/eee-api.h"
#include "exports/malloc-api.h"
#include "exports/thread-api.h"
#include "exports/cm-api.h"
#else
#include "../sm-fs/fs-includes.h"
#endif

#include "fssf-api.h"
#include "fssf-struct.h"

/*------------------------------------------------------------------------------
 * Global variables.
 *------------------------------------------------------------------------------
 */

/* Spinlock for the global variables.
 */
extern nfx_spinlock_t  fssf_stats_lock;
extern nfx_spinlock_t  fssf_tab_lock;

/* FSSF application id.
 */
extern int32           fssfAppId;
extern uint32          fssfPort;
extern uint32          fsPort;

/* FSSF statistics.
 */
extern fssf_stats_t    fssfStats;

/* Open file(s).
 * Hash table.
 */
extern uint32          fileTab;

/* Client with open file(s).
 * Hash table
 */
extern uint32          ownerTab;

/* Open state(s).
 * Indexed table.
 */
extern fssf_openTab_t  openTab;

/* Lock state(s).
 * Indexed table.
 */
extern fssf_lockTab_t  lockTab;

/*------------------------------------------------------------------------------
 * Macro definitions.
 *------------------------------------------------------------------------------
 */

#define FSSF_MAX_BUF_SIZE   1024
#define FSSF_YIELD_TIMEOUT  5       // 5 seconds

/* Debug printouts
 * ---------------
 */
#ifndef NO_SUCH_DEF
#define FSSF_DBG_PRINT(_par_) printf _par_; printf("\n")
#define FSSF_ERR_PRINT(_par_) printf _par_; printf("\n")
#define FSSF_FCT_PRINT(_par_) printf _par_; printf("\n")
#else
#define FSSF_DBG_PRINT(_par_) E_LOG(class_1, LOG_DEBUG, fssfAppId, 0,0,0, _par_)
#define FSSF_ERR_PRINT(_par_) E_LOG(class_1, LOG_ERR  , fssfAppId, 0,0,0, _par_)
#define FSSF_FCT_PRINT(_par_) E_LOG(class_1, LOG_INFO , fssfAppId, 0,0,0, _par_)
#endif // NO_SUCH_DEF

/* Statistics
 * ----------
 */
#define FSSF_DO_STATS

#ifdef  FSSF_DO_STATS
#define FSSF_STAT_NULLREQ                   \
        NFX_SPIN_LOCK(&fssf_stats_lock);    \
        fssfStats.nNullReq++;               \
        NFX_SPIN_UNLOCK(&fssf_stats_lock);
#define FSSF_STAT_NULLRSP                   \
        NFX_SPIN_LOCK(&fssf_stats_lock);    \
        fssfStats.nNullRsp++;               \
        NFX_SPIN_UNLOCK(&fssf_stats_lock);
#define FSSF_STAT_CLOSEREQ                  \
        NFX_SPIN_LOCK(&fssf_stats_lock);    \
        fssfStats.nCloseReq++;              \
        NFX_SPIN_UNLOCK(&fssf_stats_lock);
#define FSSF_STAT_CLOSERSP                  \
        NFX_SPIN_LOCK(&fssf_stats_lock);    \
        fssfStats.nCloseRsp++;              \
        NFX_SPIN_UNLOCK(&fssf_stats_lock);
#define FSSF_STAT_CREATEREQ                 \
        NFX_SPIN_LOCK(&fssf_stats_lock);    \
        fssfStats.nCreateReq++;             \
        NFX_SPIN_UNLOCK(&fssf_stats_lock);
#define FSSF_STAT_CREATERSP                 \
        NFX_SPIN_LOCK(&fssf_stats_lock);    \
        fssfStats.nCreateRsp++;             \
        NFX_SPIN_UNLOCK(&fssf_stats_lock);
#define FSSF_STAT_GETATTRREQ                \
        NFX_SPIN_LOCK(&fssf_stats_lock);    \
        fssfStats.nGetattrReq++;            \
        NFX_SPIN_UNLOCK(&fssf_stats_lock);
#define FSSF_STAT_GETATTRRSP                \
        NFX_SPIN_LOCK(&fssf_stats_lock);    \
        fssfStats.nGetattrRsp++;            \
        NFX_SPIN_UNLOCK(&fssf_stats_lock);
#define FSSF_STAT_LOCKREQ                   \
        NFX_SPIN_LOCK(&fssf_stats_lock);    \
        fssfStats.nLockReq++;               \
        NFX_SPIN_UNLOCK(&fssf_stats_lock);
#define FSSF_STAT_LOCKRSP                   \
        NFX_SPIN_LOCK(&fssf_stats_lock);    \
        fssfStats.nLockRsp++;               \
        NFX_SPIN_UNLOCK(&fssf_stats_lock);
#define FSSF_STAT_OPENREQ                   \
        NFX_SPIN_LOCK(&fssf_stats_lock);    \
        fssfStats.nOpenReq++;               \
        NFX_SPIN_UNLOCK(&fssf_stats_lock);
#define FSSF_STAT_OPENRSP                   \
        NFX_SPIN_LOCK(&fssf_stats_lock);    \
        fssfStats.nOpenRsp++;               \
        NFX_SPIN_UNLOCK(&fssf_stats_lock);
#define FSSF_STAT_READREQ                   \
        NFX_SPIN_LOCK(&fssf_stats_lock);    \
        fssfStats.nReadReq++;               \
        NFX_SPIN_UNLOCK(&fssf_stats_lock);
#define FSSF_STAT_READRSP                   \
        NFX_SPIN_LOCK(&fssf_stats_lock);    \
        fssfStats.nReadRsp++;               \
        NFX_SPIN_UNLOCK(&fssf_stats_lock);
#define FSSF_STAT_REMOVEREQ                 \
        NFX_SPIN_LOCK(&fssf_stats_lock);    \
        fssfStats.nRemoveReq++;             \
        NFX_SPIN_UNLOCK(&fssf_stats_lock);
#define FSSF_STAT_REMOVERSP                 \
        NFX_SPIN_LOCK(&fssf_stats_lock);    \
        fssfStats.nRemoveRsp++;             \
        NFX_SPIN_UNLOCK(&fssf_stats_lock);
#define FSSF_STAT_SETATTRREQ                \
        NFX_SPIN_LOCK(&fssf_stats_lock);    \
        fssfStats.nSetattrReq++;            \
        NFX_SPIN_UNLOCK(&fssf_stats_lock);
#define FSSF_STAT_SETATTRRSP                \
        NFX_SPIN_LOCK(&fssf_stats_lock);    \
        fssfStats.nSetattrRsp++;            \
        NFX_SPIN_UNLOCK(&fssf_stats_lock);
#define FSSF_STAT_UNLOCKREQ                 \
        NFX_SPIN_LOCK(&fssf_stats_lock);    \
        fssfStats.nUnlockReq++;             \
        NFX_SPIN_UNLOCK(&fssf_stats_lock);
#define FSSF_STAT_UNLOCKRSP                 \
        NFX_SPIN_LOCK(&fssf_stats_lock);    \
        fssfStats.nUnlockRsp++;             \
        NFX_SPIN_UNLOCK(&fssf_stats_lock);
#define FSSF_STAT_WRITEREQ                  \
        NFX_SPIN_LOCK(&fssf_stats_lock);    \
        fssfStats.nWriteReq++;              \
        NFX_SPIN_UNLOCK(&fssf_stats_lock);
#define FSSF_STAT_WRITERSP                  \
        NFX_SPIN_LOCK(&fssf_stats_lock);    \
        fssfStats.nWriteRsp++;              \
        NFX_SPIN_UNLOCK(&fssf_stats_lock);

#define FSSF_STAT_FSCREATEREQ               \
        NFX_SPIN_LOCK(&fssf_stats_lock);    \
        fssfStats.nFsCreateReq++;           \
        NFX_SPIN_UNLOCK(&fssf_stats_lock);
#define FSSF_STAT_FSCREATERSP               \
        NFX_SPIN_LOCK(&fssf_stats_lock);    \
        fssfStats.nFsCreateRsp++;           \
        NFX_SPIN_UNLOCK(&fssf_stats_lock);
#define FSSF_STAT_FSLOOKUPREQ               \
        NFX_SPIN_LOCK(&fssf_stats_lock);    \
        fssfStats.nFsLookupReq++;           \
        NFX_SPIN_UNLOCK(&fssf_stats_lock);
#define FSSF_STAT_FSLOOKUPRSP               \
        NFX_SPIN_LOCK(&fssf_stats_lock);    \
        fssfStats.nFsLookupRsp++;           \
        NFX_SPIN_UNLOCK(&fssf_stats_lock);

#define FSSF_STAT_ALLOC                     \
        NFX_SPIN_LOCK(&fssf_stats_lock);    \
        fssfStats.nAlloc++;                 \
        NFX_SPIN_UNLOCK(&fssf_stats_lock);
#define FSSF_STAT_DEALLOC                   \
        NFX_SPIN_LOCK(&fssf_stats_lock);    \
        fssfStats.nDealloc++;               \
        NFX_SPIN_UNLOCK(&fssf_stats_lock);
#else
#define FSSF_STAT_NULLREQ
#define FSSF_STAT_NULLRSP
#define FSSF_STAT_CLOSEREQ
#define FSSF_STAT_CLOSERSP
#define FSSF_STAT_CREATEREQ
#define FSSF_STAT_CREATERSP
#define FSSF_STAT_GETATTRREQ
#define FSSF_STAT_GETATTRRSP
#define FSSF_STAT_LOCKREQ
#define FSSF_STAT_LOCKRSP
#define FSSF_STAT_OPENREQ
#define FSSF_STAT_OPENRSP
#define FSSF_STAT_READREQ
#define FSSF_STAT_READRSP
#define FSSF_STAT_REMOVEREQ
#define FSSF_STAT_REMOVERSP
#define FSSF_STAT_SETATTRREQ
#define FSSF_STAT_SETATTRRSP
#define FSSF_STAT_UNLOCKREQ
#define FSSF_STAT_UNLOCKRSP
#define FSSF_STAT_WRITEREQ
#define FSSF_STAT_WRITERSP

#define FSSF_STAT_FSCREATEREQ
#define FSSF_STAT_FSCREATERSP
#define FSSF_STAT_FSLOOKUPREQ
#define FSSF_STAT_FSLOOKUPRSP

#define FSSF_STAT_ALLOC
#define FSSF_STAT_DEALLOC
#endif // FSSF_DO_STATS

/*------------------------------------------------------------------------------
 * Type definition.
 *------------------------------------------------------------------------------
 */

/*------------------------------------------------------------------------------
 * Function prototypes.
 *------------------------------------------------------------------------------
 */

/* Defined in fssf.c
 * -----------------
 */
void
fssf_processCloseReq(esm_handle_t   handle,
                     void *         userInfo);
void
fssf_processCreateReq(fssf_createReq_t * req);

void
fssf_processGetattrReq(esm_handle_t handle,
                       void *       userInfo);

void
fssf_processOpenReq(esm_handle_t    handle,
                    void *          userInfo);

void
fssf_processLockReq(fssf_lockReq_t * req);

void
fssf_processReadReq(esm_handle_t    handle,
                    void *          userInfo);

void
fssf_processRemoveReq(fssf_removeReq_t * req);

void
fssf_processSetattrReq(fssf_setattrReq_t * req);

void
fssf_processUnlockReq(fssf_unlockReq_t * req);

void
fssf_processWriteReq(fssf_writeReq_t * req);

/* Defined in fssf-file.c
 * ----------------------
 */
void
fssf_initFileTab(int initSize);

uint32
fssf_file_hashFunc(const void * key);

int32
fssf_file_hashCompare(const void *  key1,
                      const void *  key2);

void
fssf_file_hashDestroy(void *    key,
                      void *    obj);

void
fssf_getFile(fs_Handle *        fileHandle, // input
             fssf_share_t **    file);      // output

fssf_status_t
fssf_addNewFile(fs_Handle *     fileHandle, // input
                fssf_share_t ** newFile);   // output

void
fssf_addStateInFile(fssf_state_t *  state,  // input
                    fssf_share_t *  file);  // input / output

fssf_status_t
fssf_delStateFromFile(fssf_state_t *    statee);    // input / output

/* Defined in fssf-owner.c
 * -----------------------
 */
void
fssf_initOwnerTab(int initSize);

uint32
fssf_owner_hashFunc(const void * key);

int32
fssf_owner_hashCompare(const void * key1,
                       const void * key2);

void
fssf_owner_hashDestroy(void *   key,
                       void *   obj);

void
fssf_getOwner(fssf_clientid_t   id,     // input
              fssf_owner_t **   owner); // ouput

fssf_status_t
fssf_addNewOwner(fssf_clientid_t    id,     // input
                 fssf_owner_t **    owner); // ouput

void
fssf_addStateInOwner(fssf_state_t * state,  // input
                     fssf_owner_t * owner); // input / output

fssf_status_t
fssf_delStateFromOwner(fssf_state_t *   state); // input / output

/* Defined in fssf-open.c
 * ----------------------
 */
void
fssf_initOpenTab(int initSize);

void
fssf_addOpenState(fssf_state_t **   state); // output

void
fssf_delOpenState(fssf_state_t **   state); // input/output

fssf_status_t
fssf_getState(fssf_stateid_t *  stateid,    // input
              fssf_state_t **   openState); // output

/* Defined in fssf-lock.c
 * ----------------------
 */
void
fssf_initLockTab(int initSize);

/* Defined in fssf-fs.c
 * --------------------
 */
/**/
fssf_status_t
fssf_mcl(fssf_cb_t *    cb,         // input
         fs_Handle *    objHandle,  // output
         fs_Attr *      objAttr);   // output

fssf_status_t
fssf_createFile(fssf_cb_t * cb,         // input
                fs_Handle * dirHandle,  // input
                fs_Handle * fileHandle, // output
                fs_Attr *   fileAttr);  // output

/* Defined in fssf-utils.c
 * -----------------------
 */
inline void *
fssf_ramAlloc(uint32 size);

inline void *
fssf_ramAllocZero(uint32 size);

inline void
fssf_ramDealloc(void * p);

inline int32
fssf_sendMsg(uint32     src,
             uint32     dest,
             void	*   buf,
             uint32     len);

/*----------------------------------------------------------------------------*/
#endif // _FSSF_H_
