

/* More subroutines needed by GCC output code on some machines.  */
/* Compile this one with gcc.  */
/* Copyright (C) 1989, 92-98, 1999 Free Software Foundation, Inc.

This file is part of GNU CC.

GNU CC is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2, or (at your option)
any later version.

GNU CC is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with GNU CC; see the file COPYING.  If not, write to
the Free Software Foundation, 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA.  */

/* As a special exception, if you link this library with other files,
   some of which are compiled with GCC, to produce an executable,
   this library does not by itself cause the resulting executable
   to be covered by the GNU General Public License.
   This exception does not however invalidate any other reasons why
   the executable file might be covered by the GNU General Public License.  */

#include <nfx-incl.h>
#include "ssc-stdio.h"
#include "gcov-io.h"
#include <exports/thread-api.h>
#include <exports/malloc-api.h>

#define EOF -1

typedef void (*func_ptr)();

extern func_ptr __CTOR_LIST__[];
extern func_ptr __CTOR_END__[];

void
do_global_ctors()
{
	func_ptr *p;
	for (p = __CTOR_END__ - 1; *p != (func_ptr) -1LL; p--) {
		(*p) ();
	}
}

/* Structure emitted by -a  */
struct bb
{
  long zero_word;
  const char *filename;
  long *counts;
  long ncounts;
  struct bb *next;
  const unsigned long *addresses;

  /* Older GCC's did not emit these fields.  */
  long nwords;
  const char **functions;
  const long *line_nums;
  const char **filenames;
  char *flags;
};


static struct bb *bb_head;


#ifndef MACHINE_STATE_SAVE
#define MACHINE_STATE_SAVE(ID)
#endif
#ifndef MACHINE_STATE_RESTORE
#define MACHINE_STATE_RESTORE(ID)
#endif

#include <exports/libc.h>

void
__bb_init_func (struct bb *blocks)
{
  /* User is supposed to check whether the first word is non-0,
     but just in case....  */

	if (blocks->zero_word) {
		printf("blocks->zero_word is not 0\n");
	}

#ifdef ON_EXIT
  /* Initialize destructor.  */
  if (!bb_head)
    ON_EXIT (__bb_exit_func, 0);
#endif

  /* Set up linked list.  */
  blocks->zero_word = 1;
  blocks->next = bb_head;
  bb_head = blocks;
}



int32
gcov_reset_counts(void)
{
	struct bb *ptr;
	for (ptr = bb_head; ptr != (struct bb *) 0; ptr = ptr->next) {
		int i;
		for (i = 0; i < ptr->ncounts; ++i) {
			ptr->counts[i] = 0;
		}
	}
	return 0;
}


static void
gcov_write_thread(esm_handle_t handle, void *userInfo)
{
  ssc_FILE *da_file;
#if 0  
  ssc_FILE *file;
  long time_value;
#endif  
  int i;
  char *output_prefix = userInfo;

  if (bb_head == 0) {
	  printf("no gcov data available\n");
	  return;
  }

  i = strlen (bb_head->filename) - 3;

  if (!strcmp (bb_head->filename+i, ".da"))
    {
      /* Must be -fprofile-arcs not -a.
	 Dump data in a form that gcov expects.  */

      struct bb *ptr;

      for (ptr = bb_head; ptr != (struct bb *) 0; ptr = ptr->next)
	{
	  /* If the file exists, and the number of counts in it is the same,
	     then merge them in.  */
		char filename[200];

		strcpy(filename, output_prefix);
		if (strstr(ptr->filename, "/code")) {
			strcat(filename, strstr(ptr->filename, "/code"));

		printf("gcov: writing %s\n", filename);

	  if ((da_file = ssc_fopen (filename, "r")) != 0)
	    {
	      long n_counts = 0;
	      
	      if (__read_long (&n_counts, da_file, 8) != 0)
		{
			printf ("arc profiling: Can't read output file %s.\n",
					ptr->filename);
		  continue;
		}

	      if (n_counts == ptr->ncounts)
		{
		  int i;

		  for (i = 0; i < n_counts; i++)
		    {
		      long v = 0;

		      if (__read_long (&v, da_file, 8) != 0)
			{
				printf ("arc profiling: Can't read output file %s.\n",
						ptr->filename);
				break;
			}
		      ptr->counts[i] += v;
		    }
		}

	      if (ssc_fclose (da_file) == EOF)
			  printf ("arc profiling: Error closing output file %s.\n",
					  ptr->filename);
	    }
	  if ((da_file = ssc_fopen (filename, "w")) == 0)
	    {
			printf("arc profiling: Can't open output file %s.\n",
				   ptr->filename);
			continue;
	    }

	  /* ??? Should first write a header to the file.  Preferably, a 4 byte
	     magic number, 4 bytes containing the time the program was
	     compiled, 4 bytes containing the last modification time of the
	     source file, and 4 bytes indicating the compiler options used.

	     That way we can easily verify that the proper source/executable/
	     data file combination is being used from gcov.  */

	  if (__write_long (ptr->ncounts, da_file, 8) != 0)
	    {
	      
			printf("arc profiling: Error writing output file %s.\n",
				   ptr->filename);
	    }
	  else
	    {
	      int j;
	      long *count_ptr = ptr->counts;
	      int ret = 0;
	      for (j = ptr->ncounts; j > 0; j--)
		{
		  if (__write_long (*count_ptr, da_file, 8) != 0)
		    {
		      ret=1;
		      break;
		    }
		  count_ptr++;
		}
	      if (ret)
			  printf ("arc profiling: Error writing output file %s.\n",
					  ptr->filename);
	    }
	  
	  if (ssc_fclose (da_file) == EOF)
		  printf("arc profiling: Error closing output file %s.\n",
				 ptr->filename);
	}
		}

	  printf("gcov: done\n");
	  eee_ramDealloc(output_prefix);
      return;
    }
#if 0
  /* Must be basic block profiling.  Emit a human readable output file.  */

  file = ssc_fopen ("bb.out", "a");

  if (!file)
    perror ("bb.out");

  else
    {
      struct bb *ptr;

      /* This is somewhat type incorrect, but it avoids worrying about
	 exactly where time.h is included from.  It should be ok unless
	 a void * differs from other pointer formats, or if sizeof (long)
	 is < sizeof (time_t).  It would be nice if we could assume the
	 use of rationale standards here.  */

      time ((void *) &time_value);
      fprintf (file, "Basic block profiling finished on %s\n", ctime ((void *) &time_value));

      /* We check the length field explicitly in order to allow compatibility
	 with older GCC's which did not provide it.  */

      for (ptr = bb_head; ptr != (struct bb *) 0; ptr = ptr->next)
	{
	  int i;
	  int func_p	= (ptr->nwords >= sizeof (struct bb)
			   && ptr->nwords <= 1000
			   && ptr->functions);
	  int line_p	= (func_p && ptr->line_nums);
	  int file_p	= (func_p && ptr->filenames);
	  int addr_p	= (ptr->addresses != 0);
	  long ncounts	= ptr->ncounts;
	  long cnt_max  = 0;
	  long line_max = 0;
	  long addr_max = 0;
	  int file_len	= 0;
	  int func_len	= 0;
	  int blk_len	= num_digits (ncounts, 10);
	  int cnt_len;
	  int line_len;
	  int addr_len;

	  fprintf (file, "File %s, %ld basic blocks \n\n",
		   ptr->filename, ncounts);

	  /* Get max values for each field.  */
	  for (i = 0; i < ncounts; i++)
	    {
	      const char *p;
	      int len;

	      if (cnt_max < ptr->counts[i])
		cnt_max = ptr->counts[i];

	      if (addr_p && addr_max < ptr->addresses[i])
		addr_max = ptr->addresses[i];

	      if (line_p && line_max < ptr->line_nums[i])
		line_max = ptr->line_nums[i];

	      if (func_p)
		{
		  p = (ptr->functions[i]) ? (ptr->functions[i]) : "<none>";
		  len = strlen (p);
		  if (func_len < len)
		    func_len = len;
		}

	      if (file_p)
		{
		  p = (ptr->filenames[i]) ? (ptr->filenames[i]) : "<none>";
		  len = strlen (p);
		  if (file_len < len)
		    file_len = len;
		}
	    }

	  addr_len = num_digits (addr_max, 16);
	  cnt_len  = num_digits (cnt_max, 10);
	  line_len = num_digits (line_max, 10);

	  /* Now print out the basic block information.  */
	  for (i = 0; i < ncounts; i++)
	    {
	      fprintf (file,
		       "    Block #%*d: executed %*ld time(s)",
		       blk_len, i+1,
		       cnt_len, ptr->counts[i]);

	      if (addr_p)
		fprintf (file, " address= 0x%.*lx", addr_len,
			 ptr->addresses[i]);

	      if (func_p)
		fprintf (file, " function= %-*s", func_len,
			 (ptr->functions[i]) ? ptr->functions[i] : "<none>");

	      if (line_p)
		fprintf (file, " line= %*ld", line_len, ptr->line_nums[i]);

	      if (file_p)
		fprintf (file, " file= %s",
			 (ptr->filenames[i]) ? ptr->filenames[i] : "<none>");

	      fprintf (file, "\n");
	    }

	  fprintf (file, "\n");
	  fflush (file);
	}

      fprintf (file, "\n\n");
      ssc_fclose (file);
    }
#endif
}



int32
gcov_dump (int argc, char **argv)
{
    esm_handle_t handle;
	char *arg_prefix = argc < 2 ? "/mnt" : argv[1];
	char *output_prefix = eee_ramAlloc(strlen(arg_prefix) + 1);
	strcpy(output_prefix, arg_prefix);
	handle = thread_create(output_prefix);
	thread_start(&handle, gcov_write_thread);
	return 0;
}
