/*
 *-----------------------------------------------------------------
 *
 * Name:	gmc-queue.c
 *
 * RCS:  $Id: gmc-queue.c,v 1.3 2002/02/04 18:27:20 rick Exp $
 *
 * Copyright (C) 2001, Agile Storage, Inc.
 *
 * Description:	Emulation routines for GMC
 *
 * Created by:	Rick Lund
 *
 * Date Created:	4/5/01
 *
 *-----------------------------------------------------------------
 */

#include <nfx-incl.h>
#ifndef SBM_REMOVE_GMC
#include "../sm-sbm/sbm-gmc.h"
#include "../sm-sbm/sbm.h"
#include "gmc.h"

/*-----------------------------------------------------------------
 * Name: uint32 gmc_get_ifq_entry()
 *
 * Description:	the GMC gets a free HMT index from its IFQ
 *
 * Created by:	Rick Lund
 *
 * Date Created:	4/5/01
 *
 *-----------------------------------------------------------------
 */
uint32
gmc_get_ifq_entry()
{
	uint32	fi;

	if ((uint32)*gmc->ifq_cntl.head & IFQ_GMC_OWN) {
		fi = (uint32)*gmc->ifq_cntl.head & ~IFQ_GMC_OWN;
		*gmc->ifq_cntl.head = 0;

		if (!((uint32)gmc->ifq_cntl.head & 0x1F))
			*(gmc_regs->ifq_credit_addr) = (uint32)gmc->ifq_cntl.head;

		if (++gmc->ifq_cntl.head == gmc->ifq_cntl.end)
			gmc->ifq_cntl.head = gmc->ifq_cntl.start;


		GMC_TRACE(("GMC : GOT IFQ entry, 0x%x\n", fi));

		return fi;
	}
	else {
		printf("GMC : No IFQ entry available\n");
		return 0xFFFFFFFF;
	}
}

/*-----------------------------------------------------------------
 * Name: uint32 gmc_get_fbq_entry()
 *
 * Description:	the GMC gets a Free Buffer Index from its local
 *				FBQ.
 *
 * Created by:	Rick Lund
 *
 * Date Created:	4/5/01
 *
 *-----------------------------------------------------------------
 */
uint32
gmc_get_fbq_entry()
{
	uint32	fi;

	if ((uint32)*gmc->fbq_cntl.head & IFQ_GMC_OWN) {
		fi = (uint32)*gmc->fbq_cntl.head & ~IFQ_GMC_OWN;
		*gmc->fbq_cntl.head = 0;

		if (!((uint32)gmc->fbq_cntl.head & 0x1F))
			*(gmc_regs->fbq_credit_addr) = (uint32)gmc->fbq_cntl.head;

		if (++gmc->fbq_cntl.head == gmc->fbq_cntl.end)
			gmc->fbq_cntl.head = gmc->fbq_cntl.start;


		GMC_TRACE(("GMC : GOT FBQ entry, 0x%x\n", fi));

		return fi;
	}
	else {
		printf("GMC : No FBQ entry available\n");
		return 0xFFFFFFFF;
	}
}

#endif	/* !SBM_REMOVE_GMC */
