/*
 *-----------------------------------------------------------------
 *
 * Name:	gmc-rx.c
 *
 * RCS:  $Id: gmc-rx.c,v 1.6 2002/02/04 18:28:47 rick Exp $
 *
 * Copyright (C) 2001, Agile Storage, Inc.
 *
 * Description:	Emulation routines for the GMC
 *
 * Created by:	Rick Lund
 *
 * Date Created:	4/5/01
 *
 *-----------------------------------------------------------------
 */

#include <nfx-incl.h>
#ifndef SBM_REMOVE_GMC
#include "../sm-sbm/sbm-gmc.h"
#include "../sm-sbm/sbm.h"
#include "gmc.h"

/*-----------------------------------------------------------------
 * Name: gmc_receive(nfx_cod_t COD, uint32 len)
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	4/5/01
 *
 *-----------------------------------------------------------------
 */
gmc_receive(nfx_cod_t COD, uint32 len)
{
	sbm_dsokqEntry_t	* qptr;
	uint32				index;
	uint32				control;
	uint32				fb[10];
	uint32				num_blocks;
	sbm_hmtEntry_t		* hmt;
	sbm_bdlEntry_t		* bdl;
	uint32				blen;
	uint32				i;

	qptr = gmc->dsokq.curr;

	if (qptr->control & DSOKQ_CONT_OWN) {
		panic("DSOKQ entry at 0x%x already in use\n", qptr);
		return;
	}

	index = gmc_get_ifq_entry();
	hmt = (sbm_hmtEntry_t *)(gmc_regs->hmt_base_addr + (index*8));

	if (index == 0xFFFFFFFF) {
		panic("no IFQ entry available\n");
		return;
	}

	control = DSOKQ_CONT_OWN;
	num_blocks = ((len / gmc->regs.gmc_sac_block_size) + 1);
	control |= num_blocks;

	hmt->COD_len = len;
	for (i=0; i<num_blocks; i++) {
		if ((fb[i] = gmc_get_fbq_entry()) == 0xFFFFFFFF) {
			uint32	j;
			printf("GMC : No available free SAC block in FBQ\n");
			for (j=0; j<i; j++)
				sbm_freeFBQEntry(fb[j]);
			sbm_freeIFQEntry(index);
			return;
		}
	}
	for (i=0; i<num_blocks; i++) {
		bdl = (sbm_bdlEntry_t *)(gmc_regs->bdl_base_addr+(fb[i]*8));
		bdl->offset = 0;
		if (len > gmc_regs->gmc_sac_block_size) {
			blen = gmc_regs->gmc_sac_block_size;
			len -= blen;
		}
		else
			blen = len;
		bdl->len = blen;
		bdl->next_bdl_index = fb[i+1] | BDL_ENTRY_IN_USE;
	}
	bdl->next_bdl_index = BDL_EOC;
	hmt->bdl_index = fb[0] | HMT_ENTRY_IN_USE;
	
	qptr->status.ack.HMT_index = index;
	qptr->status.ack.COD = COD;
	qptr->control = control;

	GMC_TRACE(("GMC : 0x%08x:  0x%08x\n", qptr, qptr->status.ack.HMT_index));
	GMC_TRACE(("GMC :              0x%016llx\n", qptr->status.ack.COD));
	GMC_TRACE(("GMC :              0x%08x\n", qptr->control));

	if (++gmc->dsokq.curr == gmc->dsokq.end)
		gmc->dsokq.curr = gmc->dsokq.start;	
}

/*-----------------------------------------------------------------
 * Name:	gmc_sacify_rcv()
 *
 * Description:	Registered receive application function for "sac" app.
 *				For testing, a file is chopped up and sent to this function,
 *				running on the MCPU.  The "gmc emulator" should store the
 *				payload data into the SAC, and post the proper DSOKQ entry.
 *
 * Created by:	Rick Lund
 *
 * Date Created:	7/30/01
 *
 *-----------------------------------------------------------------
 */
void
gmc_sacify_rcv(
	eee_descPtr_t	edesc
)
{
	sbm_dsokqEntry_t	* qptr;
	uint32				index;
	uint32				control;
	uint32				num_bufs, i;
	eee_descBuf_t		* bd;
	uint32				fb[EEE_MAX_BUFS];
	uint32				tot_len, cod_len, bd_len, hdr_len;
	sbm_hmtEntry_t		* hmt;
	sbm_bdlEntry_t		* bdl;
	uchar8				* sac_block_addr;
	nfx_cod_t			COD;
	gmc_app_hdr_t		* gmc_app_hdr;
	uchar8				* src;
	uint32				dest_app;

	GMC_TRACE(("gmc_sacify_rcv, edesc = 0x%x\n", edesc));

#ifdef FIX_HDR_STUFF
	hdr_len = sizeof(gmc_app_hdr_t);
#else
	hdr_len = sizeof(enet_hdr_t) + sizeof(agile_hdr_t) + sizeof(gmc_app_hdr_t);
#endif

	tot_len = edesc->hdr.control & EDESC_TOTAL_LEN;
	cod_len = tot_len - hdr_len;

	GMC_TRACE(("    length = %d\n", tot_len));
	num_bufs = (edesc->hdr.control & EDESC_BUFCNT_MASK) >> EDESC_BUFCNT_SHIFT;
	if (edesc->hdr.control & EDESC_COMMAND_USE_FIRST_BUFFER)
		bd = &(edesc->bd[0]);
	else
		bd = &(edesc->bd[1]);
		
	qptr = gmc->dsokq.curr;

	if (qptr->control & DSOKQ_CONT_OWN) {
		panic("DSOKQ entry at 0x%x already in use\n", qptr);
		goto sacify_rcv_done;
	}

	index = gmc_get_ifq_entry();
	hmt = (sbm_hmtEntry_t *)(gmc_regs->hmt_base_addr + (index*8));

	if (index == 0xFFFFFFFF) {
		panic("no IFQ entry available\n");
		goto sacify_rcv_done;
	}

	for (i=0; i<num_bufs; i++) {
		if ((fb[i] = gmc_get_fbq_entry()) == 0xFFFFFFFF) {
			uint32	j;
			printf("GMC : No available free SAC block in FBQ\n");
			for (j=0; j<i; j++)
				sbm_freeFBQEntry(fb[j]);
			sbm_freeIFQEntry(index);
			goto sacify_rcv_done;
		}
	}

	for (i=0; i<num_bufs; i++) {
		bdl = (sbm_bdlEntry_t *)(gmc_regs->bdl_base_addr+(fb[i]*8));
		bdl->offset = 0;

		if (i==0) {
#ifdef FIX_HDR_STUFF
			gmc_app_hdr = (gmc_app_hdr_t *)(bd->buf);
#else
			gmc_app_hdr = (gmc_app_hdr_t *)(bd->buf +
								sizeof(enet_hdr_t) + sizeof(agile_hdr_t));
#endif
			bd_len = bd->len - hdr_len;
			src = bd->buf + hdr_len;
		}
		else {
			bd_len = bd->len;
			src = bd->buf;
		}

		GMC_TRACE(("    buffer at 0x%x, length %d\n", src, bd_len));

		if (bd_len > gmc_regs->gmc_sac_block_size) {
			panic("buffer length > sac block size\n");
		}
		sac_block_addr = gmc->sac + (gmc_regs->gmc_sac_block_size * fb[i]);
        GMC_TRACE(("bdl index = %d, sac_block at 0x%x\n", fb[i], sac_block_addr));
		memcpy(sac_block_addr, src, bd_len);
		bdl->len = bd_len;
		bdl->next_bdl_index = fb[i+1] | BDL_ENTRY_IN_USE;

		bd++;
	}

GMC_TRACE(("gmc_app_hdr at 0x%x\n", gmc_app_hdr));
GMC_TRACE(("cod = 0x%llx\n", gmc_app_hdr->id.cod));
GMC_TRACE(("control = 0x%x\n", gmc_app_hdr->control));
	COD = gmc_app_hdr->id.cod;
	dest_app = gmc_app_hdr->control & 0xFF;

	bdl->next_bdl_index = BDL_EOC;
	hmt->COD_len = cod_len;
	hmt->bdl_index = fb[0] | HMT_ENTRY_IN_USE;
	
GMC_TRACE(("qptr = 0x%x\n", qptr));
	qptr->status.ack.HMT_index = index;
	qptr->status.ack.COD = COD;
	qptr->control = num_bufs |
					(dest_app << DSOKQ_CONT_APP_SHIFT) |
					DSOKQ_CONT_OWN;

	if (++gmc->dsokq.curr == gmc->dsokq.end)
		gmc->dsokq.curr = gmc->dsokq.start;	

sacify_rcv_done:
	eee_freePacket(edesc);

}	/* gmc_sacify_rcv() */

#endif	/* !SBM_REMOVE_GMC */
