/*
 *=============================================================================
 * File Name   :    gt64240mii-hash.c
 *
 *  Description:
 *
 *  Created    :    08-03-2001
 *
 *  Author     :    Tony Wong
 *
 *  Copyright(c)  Agile Storage, Inc. 2001
 *=============================================================================
 */
#include "gt64240mii.h"
#include "gt64240mii-hash.h"

#define HashMode 0
#define HashSize 1
#define HashDefMode 0
     
/*****************************************************************************
* 
* int32 gt64240_addHashTableEntry(uint32 port, uint32 macH, uint32 macL,
*                                 uint32 rd, uint32 skip)
* 
* This function will add an entry to the address table.
* depends on the hash mode and hash size that was initialized.
* Inputs
* port - ETHERNET port number.
* macH - the 2 most significant bytes of the MAC address.
* macL - the 4 least significant bytes of the MAC address.
* skip - if 1 , skip this address.
* rd - the RD field in the address table.
* Outputs
* address table entry is added.
* true if success.
* false if fail to make the assignment.
*/
int32 gt64240_addHashTableEntry(uint32 port,uint32 macH, uint32 macL, uint32 rd,
						        uint32 skip)
{
uint32 addressHighValue;
uint32 addressLowValue;
void* entryNumber;
uint32 hashBase = (uint32)gt64240mii->port[port].hash_tbl; 
uint32 addressLowRead;
uint32 addressHighRead;
int32 i;

	//printf("addHashTableEntry, port=%d, macH=0x%x, macL=0x%x, rd = 0x%x\n",
	//													port, macH, macL, rd);
    entryNumber = (void*)(hashBase + MAC_ENTRY_SIZE*gt64240_hashFunc(macH, macL, HashSize, HashMode));
	
	addressLowValue = VALID | (skip<<SKIP_BIT) | (rd<<2) |
         (((macH>>8)&0xf)<<3) |
         (((macH>>12)&0xf)<<7) |
		(((macH>>0)&0xf)<<11) | (((macH>>4)&0xf)<<15) | 
		(((macL>>24)&0xf)<<19) | (((macL>>28)&0xf)<<23) |
         (((macL>>16)&0xf)<<27) |
		((((macL>>20)&0x1)<<31));

	addressHighValue = ((macL>>21)&0x7) | (((macL>>8)&0xf)<<3) |
         (((macL>>12)&0xf)<<7) | 
		(((macL>>0)&0xf)<<11) | (((macL>>4)&0xf)<<15);

    /* find a free place */
    for(i = 0 ; i < HOP_NUMBER ; i++)
    {
        addressLowRead = *(unsigned int*)(entryNumber+i*MAC_ENTRY_SIZE);
        if((!(addressLowRead & VALID))/* || (addressLowRead & SKIP)*/)
        {
            entryNumber = entryNumber+i*MAC_ENTRY_SIZE;
            break;
        }
        else /* if the address is the same locate it at the same position */
        {
            addressHighRead = *(unsigned int*)(entryNumber+i*MAC_ENTRY_SIZE+4);
            if(((addressLowRead>>3)&0x1fffffff)==
               ((addressLowValue>>3)&0x1fffffff)
                && ((addressHighRead)==(addressHighValue)))
            {
                printf("addressHighRead = 0x%x addressHighValue = 0x%x\n",
                       addressHighRead,addressHighValue);
                entryNumber = entryNumber+i*MAC_ENTRY_SIZE;
                break;
            }
        }
    }

    if(i == HOP_NUMBER)
    {
        printf("The address table section is full\n");
        return 0;
    }

    /* write the address to the address table */
    *(uint32 *)(entryNumber+4) = addressHighValue;
    *(uint32 *)(entryNumber) = addressLowValue;

	return 1;
}


/*****************************************************************************
* 
* int32 gt64240_hashFunc(uint32 macH, uint32 macL, int32 hash_size, 
*                         int32 hash_mode)
* 
* This function will calculate the hash function of the address.
* depends on the hash mode and hash size.
* Inputs
* macH - the 2 most significant bytes of the MAC address.
* macL - the 4 least significant bytes of the MAC address.
* hashMode - hash mode 0 or hash mode 1.
* hashSize - the size of the hash table (0=8K ,1=1/2K).
* Outputs
* return the caculated entry.
* true if success.
* false if fail to make the assignment.
*/
int32 gt64240_hashFunc(uint32 macH, uint32 macL, int32 hash_size,
                        int32 hash_mode)
{
uint32 hashResult;
uint32 ethernetAddH;
uint32 ethernetAddL;
uint32 ethernetAdd0;
uint32 ethernetAdd1;
uint32 ethernetAdd2;
uint32 ethernetAdd3;
uint32 ethernetAddHSwapped = 0;
uint32 ethernetAddLSwapped = 0;

	ethernetAddH = NIBBLE_SWAPPING_16_BIT(macH);
	ethernetAddL = NIBBLE_SWAPPING_32_BIT(macL);

	ethernetAddHSwapped = GT_NIBBLE(ethernetAddH&0xf)+
        ((GT_NIBBLE((ethernetAddH>>4)&0xf))<<4)+
		((GT_NIBBLE((ethernetAddH>>8)&0xf))<<8)+
        ((GT_NIBBLE((ethernetAddH>>12)&0xf))<<12);

	ethernetAddLSwapped = GT_NIBBLE(ethernetAddL&0xf)+
        ((GT_NIBBLE((ethernetAddL>>4)&0xf))<<4)+
		((GT_NIBBLE((ethernetAddL>>8)&0xf))<<8)+
        ((GT_NIBBLE((ethernetAddL>>12)&0xf))<<12)+
		((GT_NIBBLE((ethernetAddL>>16)&0xf))<<16)+
        ((GT_NIBBLE((ethernetAddL>>20)&0xf))<<20)+
		((GT_NIBBLE((ethernetAddL>>24)&0xf))<<24)+
        ((GT_NIBBLE((ethernetAddL>>28)&0xf))<<28);

	ethernetAddH = ethernetAddHSwapped;
	ethernetAddL = ethernetAddLSwapped;

	if(hash_mode == 0)
	{
		ethernetAdd0 = (ethernetAddL>>2) & 0x3f;
		ethernetAdd1 = (ethernetAddL & 0x3) | ((ethernetAddL>>8) & 0x7f)<<2;
		ethernetAdd2 = (ethernetAddL>>15) & 0x1ff;
		ethernetAdd3 = ((ethernetAddL>>24) & 0xff) | ((ethernetAddH & 0x1)<<8);
	}
	else
	{
		ethernetAdd0 = FLIP_6_BITS((ethernetAddL) & 0x3f);
		ethernetAdd1 = FLIP_9_BITS(((ethernetAddL>>6) & 0x1ff));
		ethernetAdd2 = FLIP_9_BITS((ethernetAddL>>15) & 0x1ff);
		ethernetAdd3 = FLIP_9_BITS((((ethernetAddL>>24) & 0xff) | 
                                    ((ethernetAddH & 0x1)<<8)));
	}

	hashResult = (ethernetAdd0<<9) | (ethernetAdd1^ethernetAdd2^ethernetAdd3);

	if(hash_size == 0)
	{
		hashResult = hashResult & 0xffff;
	}
	else
	{
		hashResult = hashResult & 0x7ff;
	}
	return hashResult;
}


/*****************************************************************************
* 
* int32 gt64240_scanHashTable(int32 port)
* 
* This function will scan and print all the address table valid entries.
* Inputs
* port - ETHERNET port number.
* true if success.
* false if fail to make the assignment.
*/
int32 gt64240_scanHashTable(int32 port)
{
uint32 entry=0;
char8 dummyBuffer[100];

    while(gt64240_findFirstHashEntry(port,&entry,dummyBuffer) != 0);

    return 1;
}

/*****************************************************************************
* 
* int32 gt64240_findFirstHashEntry(int32 port,uint32 *startEntry)
* 
* This function will scan and print the first address table valid entry.
* Inputs
* port - ETHERNET port number.
* true if success.
* false if fail to make the assignment.
*/
int32 gt64240_findFirstHashEntry(int32 port,uint32 *startEntry,
								 char8* resultBuffer)
{
uint32 currentEntry;
uint32 hashLengh[GT_MII_MAX_PORTS];
uint32 addressLowRead;
uint32 addressHighRead;
uint32 addressLowValue;
uint32 addressHighValue;
uint32 endTable;
int32 hashSize;
uint32 firstEntry;

    hashLengh[0] = EIGHT_K; 
    hashLengh[1] = HALF_K;

    hashSize = HashSize;
    endTable = (uint32)(gt64240mii->port[port].hash_tbl+hashLengh[hashSize]*MAC_ENTRY_SIZE);
    firstEntry = *startEntry;

    currentEntry = (uint32)(gt64240mii->port[port].hash_tbl+((firstEntry)*MAC_ENTRY_SIZE));
    while(currentEntry < endTable)
    {
        addressLowRead = *(unsigned int*)(currentEntry);
        if(addressLowRead & VALID)
        {
            addressHighRead = *(unsigned int*)(currentEntry+4);
            addressLowValue = (((addressHighRead>>11)&0xf)<<0) | 
                (((addressHighRead>>15)&0xf)<<4) |
                (((addressHighRead>>3)&0xf)<<8) | 
                (((addressHighRead>>7)&0xf)<<12) |
                (((addressLowRead>>27)&0xf)<<16) | 
                (((addressHighRead>>0)&0x7)<<21) |
                (((addressLowRead>>31)&0x1)<<20) | 
                (((addressLowRead>>19)&0xf)<<24) |
                (((addressLowRead>>23)&0xf)<<28);

            addressHighValue = (((addressLowRead>>11)&0xf)<<0) |
                 (((addressLowRead>>15)&0xf)<<4) |
                (((addressLowRead>>3)&0xf)<<8) |
                 (((addressLowRead>>7)&0xf)<<12);

            printf("valid address %04x%08x was found in entry # 0x%x SKIP=%d RD=%d\n",
                   addressHighValue,addressLowValue,firstEntry,
                   (addressLowRead>>1)&0x1,(addressLowRead>>2)&0x1);

            sprintf(resultBuffer,"%x!!!%02x!!!%02x!!!%02x!!!%02x !!!%02x!!!%02x!!!%x!!!%x!!!%x",firstEntry*8,
                    (addressHighValue>>8)&0xff,
                    (addressHighValue>>0)&0xff,(addressLowValue>>24)&0xff,
                    (addressLowValue>>16)&0xff,
                    (addressLowValue>>8)&0xff,(addressLowValue>>0)&0xff,
                    ((addressLowRead>>2)&0x1),((addressLowRead>>1)&0x1)
                    ,VALID);
                    
            firstEntry++;
            *startEntry = firstEntry;
            return 1;
        }
        currentEntry += MAC_ENTRY_SIZE;
        firstEntry ++;
    }

    return 0;
}

/*****************************************************************************
* 
* int32 gt64240_HashTableClear(int port)
* 
* This function will clear the address table  
* Inputs
* port - ETHERNET port number.
* Outputs
* address table is clear.
* true if success.
* false if fail to make the assignment.
*/
int32 gt64240_HashTableClear(int32 port)        
{
uint32 hashLengh[GT_MII_MAX_PORTS];

    hashLengh[0] = EIGHT_K; 
    hashLengh[1] = HALF_K;

    memset((void*)gt64240mii->port[port].hash_tbl,0,
           hashLengh[HashSize]*MAC_ENTRY_SIZE);

    return 1;
}

