/*
 *=============================================================================
 * File Name   :    gt64240mii-hash.h
 *
 *  Description:
 *
 *  Created    :    08-02-2001
 *
 *  Author     :    Tony Wong
 *
 *  Copyright(c)  Agile Storage, Inc. 2001
 *=============================================================================
 */

#ifndef GT64240_HASH_H_INCLUDED
#define GT64240_HASH_H_INCLUDED

#define ADDRESS_TABLE_ALIGNMENT             0x8
#define MAC_ENTRY_SIZE                      8
#define MAC_ADDRESS_STRING_SIZE             12
#define _8K_TABLE                           0

#define HASH_DEFUALT_MODE                   14
#define HASH_MODE                           13
#define HASH_SIZE                           12
#define PROMISCUOUS_MODE                    0

#define _8K_TABLE                           0
#define VALID								1
#define SKIP                                1<<1
#define SKIP_BIT                            1
#define HOP_NUMBER                          12
#define EIGHT_K                             0x8000
#define HALF_K                              0x8000/16

#define MAX_NUMBER_OF_ADDRESSES_TO_STORE    1000

#define NIBBLE_SWAPPING_16_BIT(X) (((X&0xf)<<4) | ((X&0xf0)>>4) | ((X&0xf00)<<4) | ((X&0xf000)>>4))
        
#define NIBBLE_SWAPPING_32_BIT(X) (((X&0xf)<<4) | ((X&0xf0)>>4) | ((X&0xf00)<<4) | ((X&0xf000)>>4) | ((X&0xf0000)<<4) | ((X&0xf00000)>>4) | ((X&0xf000000)<<4) | ((X&0xf0000000)>>4))
         
#define GT_NIBBLE(X) ((X&0x1)<<3)+((X&0x2)<<1)+((X&0x4)>>1)+((X&0x8)>>3)

#define FLIP_6_BITS(X) (((X&0x1)<<5)+((X&0x2)<<3)+((X&0x4)<<1)+ ((X&0x8)>>1)+((X&0x10)>>3)+((X&0x20)>>5))
        
#define FLIP_9_BITS(X) (((X&0x1)<<8)+((X&0x2)<<6)+((X&0x4)<<4)+ ((X&0x8)<<2)+((X&0x10)<<0)+((X&0x20)>>2)+((X&0x40)>>4)+ ((X&0x80)>>6)+((X&0x100)>>8))

typedef struct AddressTablestore
{
    unsigned int valid:1;
    unsigned int skip:1;
    unsigned int rd:1;
    unsigned int macH:16;
    unsigned int macL;
	unsigned int rsvd:12;

} ADDRESS_TABLE_STORE;

int32 gt64240_hashFunc(uint32 macH, uint32 macL, int32 hash_size, 
					   int32 hash_mode);
int32 gt64240_addHashTableEntry(uint32 port,uint32 macH, uint32 macL,
                                uint32 rd, uint32 skip);
int32 gt64240_scanHashTable(int port);
int32 gt64240_findFirstHashEntry(int32 port, uint32 *startEntry, 
								 char8 *resultBuffer);
int32 gt64240_HashTableClear(int32 port);

#endif /* GT64240_HASH_H_INCLUDED */
