/*  
 *=============================================================================
 * File Name   :    gt64240mii-regs.h
 *
 *  Description:    
 *
 *  Created    :    08-02-2001
 *
 *  Author     :    Tony Wong
 *
 *  Copyright(c)  Agile Storage, Inc. 2001
 *=============================================================================
 */
#ifndef GT64240MII_REGS_H_INCLUDED
#define GT64240MII_REGS_H_INCLUDED

#define	GT_MII_PORT_BASE	0x2000
#define	GT_MII_PHY_ADDR		0x2000
#define	GT_MII_SMI_REG		0x2010

#define GT_MII_MAX_PORTS	3
#define	GT_MII_PORT_OFFSET	0x0400

/* port register offsets */

#define GT_MII_PCR		0x0000
#define GT_MII_PCXR		0x0008
#define GT_MII_CMR		0x0010
#define GT_MII_PSR		0x0018
#define GT_MII_SPR		0x0020
#define GT_MII_HTPR		0x0028
#define GT_MII_FCSAL		0x0030
#define GT_MII_FCSAH		0x0038
#define GT_MII_SDCR		0x0040
#define GT_MII_SDCMR		0x0048
#define GT_MII_ICR		0x0050
#define GT_MII_IMR		0x0058

#define GT_MII_FRDP0		0x0080
#define GT_MII_FRDP1		0x0084
#define GT_MII_FRDP2		0x0088
#define GT_MII_FRDP3		0x008c

#define GT_MII_CRDP0		0x00a0
#define GT_MII_CRDP1		0x00a4
#define GT_MII_CRDP2		0x00a8
#define GT_MII_CRDP3		0x00ac

#define GT_MII_CTDP0		0x00e0
#define GT_MII_CTDP1		0x00e4

/* statistics counters */

#define GT_MII_OK_RXBYTES	0x0100
#define GT_MII_TXBYTES 	  	0x0104
#define GT_MII_OK_RXFRAMES	0x0108
#define GT_MII_TXFRAMES	  	0x010c
#define GT_MII_ALL_RXBYTES	0x0110
#define GT_MII_ALL_RXFRAMES	0x0114
#define GT_MII_OK_RXBCAST	0x0118
#define GT_MII_OK_RXMCAST	0x011c
#define GT_MII_CRC_ERRS		0x0120
#define GT_MII_TOO_LARGE	0x0124
#define GT_MII_FRAGMENTS	0x0128
#define GT_MII_JABBERS		0x012c
#define GT_MII_COLLISIONS	0x0130
#define GT_MII_LATE_COLLS	0x0134
#define GT_MII_64_BYTES		0x0138
#define GT_MII_65_127		0x013c
#define GT_MII_128_255		0x0140
#define GT_MII_256_511		0x0144
#define GT_MII_512_1023		0x0148
#define GT_MII_1024_MAX		0x014c
#define GT_MII_RX_ERRS		0x0150
#define GT_MII_FRAME_DROPS	0x0154
#define GT_MII_TX_MCAST		0x0158
#define GT_MII_TX_BCAST		0x015c
#define GT_MII_TOO_SMALL	0x0160

#define GT_MII_LB_NONE		0
#define GT_MII_LB_INTERNAL	1
#define GT_MII_LB_EXTERNAL	2

/*
 *----------------------------------
 * Descriptor cmdsts Bit definitions
 *----------------------------------
 */
#define GT_MII_CMDSTS_OWN       0x80000000 /* Tx/Rx ownership */
#define GT_MII_CMDSTS_INTR      0x00800000 /* Tx/Rx enable interrupt */
#define GT_MII_CMDSTS_1ST       0x00020000 /* Tx/Rx 1st buffer */
#define GT_MII_CMDSTS_LAST      0x00010000 /* Tx/Rx last buffer */
#define GT_MII_CMDSTS_ERRS      0x00008000 /* Tx/Rx error summary */

#define GT_MII_CMDSTS_GENCRC    0x00400000 /* Tx generate CRC */
#define GT_MII_CMDSTS_TXCOL     0x00000200 /* Tx collision */
#define GT_MII_CMDSTS_PAD       0x00040000 /* Tx zero pad */
#define GT_MII_CMDSTS_RLERR     0x00000100 /* Tx re-Tx limit */
#define GT_MII_CMDSTS_SHORT     0x00000100 /* Rx short frame */
#define GT_MII_CMDSTS_LONG      0x00000080 /* Rx length error */
#define GT_MII_CMDSTS_OVR       0x00000040 /* Rx overrun */
#define GT_MII_CMDSTS_UDR       0x00000040 /* Tx underrun */
#define GT_MII_CMDSTS_LCERR     0x00000020 /* Tx late collision */
#define GT_MII_CMDSTS_RXCOL     0x00000010 /* Rx collision */
#define GT_MII_CMDSTS_ERRCRC    0x00000001 /* Rx CRC error */

typedef union _gt64240mii_pcr
{
	uint32 wd;
#if #endian(little)
	struct {
		uint32 pm:1;	/* Promiscuous mode */
		uint32 rbm:1;	/* Reject broadcast mode */
		uint32 pbf:1;	/* Pass bad frames */
		uint32 rsvd2:4;	/* Reserved */
		uint32 en:1;	/* Enable */
		uint32 lpbk:2;	/* Loopback mode */
		uint32 fc:1;	/* Force collision */
		uint32 rsvd1:1;	/* Reserved */
		uint32 hs:1;	/* Hash size */
		uint32 hm:1;	/* Hash mode */
		uint32 hdm:1;	/* Hash default mode */
		uint32 hd:1;	/* Duplex mode */
		uint32 rsvd:15;	/* Reserved */
		uint32 accs:1;	/* Accelerate slot time */
	} bits;
#else
	struct {
		uint32 accs:1;	/* Accelerate slot time */
		uint32 rsvd:15;	/* Reserved */
		uint32 hd:1;	/* Duplex mode */
		uint32 hdm:1;	/* Hash default mode */
		uint32 hm:1;	/* Hash mode */
		uint32 hs:1;	/* Hash size */
		uint32 rsvd1:1;	/* Reserved */
		uint32 fc:1;	/* Force collision */
		uint32 lpbk:2;	/* Loopback mode */
		uint32 en:1;	/* Enable */
		uint32 rsvd2:4;	/* Reserved */
		uint32 pbf:1;	/* Pass bad frames */
		uint32 rbm:1;	/* Reject broadcast mode */
		uint32 pm:1;	/* Promiscuous mode */
	} bits;
#endif
} gt64240mii_pcr_t;

typedef union _gt64240mii_pcxr
{
	uint32 wd;
#if #endian(little)
	struct {
		uint32 igmp:1;	/* IGMP capture enable */
		uint32 span:1;	/* Spanning tree capture enable */
		uint32 par:1;	/* Partition enable */
		uint32 priotx:3;/* Default priority for Tx */
		uint32 priorx:2;/* Default priority for Rx */
		uint32 priorx_ov:1;/* Override priority for Rx */
		uint32 dplxen:1;/* Duplex auto-negotiate enable */
		uint32 fctlen:1;/* Flow ctrl auto-negotiate enable */
		uint32 flp:1;	/* Force link pass */
		uint32 fctl:1;	/* Flow ctrl mode */
		uint32 rsvd2:1;	/* Reserved */
		uint32 mfl:2;	/* Max frame length */
		uint32 mibclrmode:1;/* MIB counter clear mode */
		uint32 rsvd1:1;	/* Reserved */
		uint32 speed:1;	/* Port speed */
		uint32 speeden:1;/* Speed auto-negotiate enable */
		uint32 rmiien:1;/* RMII enable */
		uint32 dscpen:1;/* DSCP enable */
		uint32 rsvd:10;	/* Reserved */
	} bits;
#else
	struct {
		uint32 rsvd:10;	/* Reserved */
		uint32 dscpen:1;/* DSCP enable */
		uint32 rmiien:1;/* RMII enable */
		uint32 speeden:1;/* Speed auto-negotiate enable */
		uint32 speed:1;	/* Port speed */
		uint32 rsvd1:1;	/* Reserved */
		uint32 mibclrmode:1;/* MIB counter clear mode */
		uint32 mfl:2;	/* Max frame length */
		uint32 rsvd2:1;	/* Reserved */
		uint32 fctl:1;	/* Flow ctrl mode */
		uint32 flp:1;	/* Force link pass */
		uint32 fctlen:1;/* Flow ctrl auto-negotiate enable */
		uint32 dplxen:1;/* Duplex auto-negotiate enable */
		uint32 priorx_ov:1;/* Override priority for Rx */
		uint32 priorx:2;/* Default priority for Rx */
		uint32 priotx:3;/* Default priority for Tx */
		uint32 par:1;	/* Partition enable */
		uint32 span:1;	/* Spanning tree capture enable */
		uint32 igmp:1;	/* IGMP capture enable */
	} bits;
#endif
} gt64240mii_pcxr_t;

typedef union _gt64240mii_psr
{
	uint32 wd;
#if #endian(little)
	struct {
		uint32 speed:1;	/* Port speed */
		uint32 duplex:1;/* Port duplex mode */
		uint32 fctl:1;	/* Flow ctrl mode */
		uint32 link:1;	/* Link status */
		uint32 pause:1;	/* Indicate flow ctrl disabled */
		uint32 txlow:1;	/* Tx low priority status */
		uint32 txhigh:1;/* Tx high priority status */
		uint32 txinprog:1;/* Tx in progress */
		uint32 rsvd:24;	/* Reserved */
	} bits;
#else
	struct {
		uint32 rsvd:24;	/* Reserved */
		uint32 txinprog:1;/* Tx in progress */
		uint32 txhigh:1;/* Tx high priority status */
		uint32 txlow:1;	/* Tx low priority status */
		uint32 pause:1;	/* Indicate flow ctrl disabled */
		uint32 link:1;	/* Link status */
		uint32 fctl:1;	/* Flow ctrl mode */
		uint32 duplex:1;/* Port duplex mode */
		uint32 speed:1;	/* Port speed */
	} bits;
#endif
} gt64240mii_psr_t;

typedef union _gt64240mii_sdcr
{
	uint32 wd;
#if #endian(little)
	struct {
		uint32 rsvd2:2;	/* Reserved */
		uint32 rc:4;	/* Retransmit count */
		uint32 blmr:1;	/* Big/little endian Rx mode */
		uint32 blmt:1;	/* Big/little endian Tx mode */
		uint32 povr:1;	/* PCI override */
		uint32 rifb:1;	/* Rec interrupt on frame boundary */
		uint32 rsvd1:2;	/* Reserved */
		uint32 bsz:2;	/* Burst size */
		uint32 rsvd:18;	/* Reserved */
	} bits;
#else
	struct {
		uint32 rsvd:18;	/* Reserved */
		uint32 bsz:2;	/* Burst size */
		uint32 rsvd1:2;	/* Reserved */
		uint32 rifb:1;	/* Rec interrupt on frame boundary */
		uint32 povr:1;	/* PCI override */
		uint32 blmt:1;	/* Big/little endian Tx mode */
		uint32 blmr:1;	/* Big/little endian Rx mode */
		uint32 rc:4;	/* Retransmit count */
		uint32 rsvd2:2;	/* Reserved */
	} bits;
#endif
} gt64240mii_sdcr_t;

typedef union _gt64240mii_sdcmr
{
	uint32 wd;
#if #endian(little)
	struct {
		uint32 rsvd3:7;	/* Reserved */
		uint32 erd:1;	/* Enable Rx DMA */
		uint32 rsvd2:7;	/* Reserved */
		uint32 ar:1;	/* Abort Rx */
		uint32 stdh:1;	/* Stop Tx high priority queue */
		uint32 stdl:1;	/* Stop Tx low priority queue */
		uint32 rsvd1:5;	/* Reserved */
		uint32 txdh:1;	/* Start Tx high priority queue */
		uint32 txdl:1;	/* Start Tx low priority queue */
		uint32 rsvd:6;	/* Reserved */
		uint32 at:1;	/* Abort Tx */
	} bits;
#else
	struct {
		uint32 at:1;	/* Abort Tx */
		uint32 rsvd:6;	/* Reserved */
		uint32 txdl:1;	/* Start Tx low priority queue */
		uint32 txdh:1;	/* Start Tx high priority queue */
		uint32 rsvd1:5;	/* Reserved */
		uint32 stdl:1;	/* Stop Tx low priority queue */
		uint32 stdh:1;	/* Stop Tx high priority queue */
		uint32 ar:1;	/* Abort Rx */
		uint32 rsvd2:7;	/* Reserved */
		uint32 erd:1;	/* Enable Rx DMA */
		uint32 rsvd3:7;	/* Reserved */
	} bits;
#endif
} gt64240mii_sdcmr_t;

typedef union _gt64240mii_icr
{
	uint32 wd;
#if #endian(little)
	struct {
		uint32 rxbuffer:1;	/* Rx buffer return indication */
		uint32 rsvd5:1;		/* Reserved */
		uint32 txbuffhigh:1;	/* Tx buff for high priority queue */
		uint32 txbufflow:1;	/* Tx buff for low priority queue */
		uint32 rsvd4:2;		/* Reserved */
		uint32 txendhigh;	/* Tx end high priority queue */
		uint32 txendlow;	/* Tx end low priority queue */
		uint32 rxerror:1;	/* Rx resource error */
		uint32 rsvd3:1;		/* Reserved */
		uint32 txerrhigh:1;	/* Tx resource err high priority queue*/
		uint32 txerrlow:1;	/* Tx resource err low priority queue */
		uint32 rxovr:1;		/* Rx overrun */
		uint32 txudr:1;		/* Tx underrun */
		uint32 rsvd2:2;		/* Reserved */
		uint32 rxbuffq0:1;	/* Rx buffer return priority queue 0 */
		uint32 rxbuffq1:1;	/* Rx buffer return priority queue 1 */
		uint32 rxbuffq2:1;	/* Rx buffer return priority queue 2 */
		uint32 rxbuffq3:1;	/* Rx buffer return priority queue 3 */
		uint32 rxerrorq0:1;	/* Rx resource err priority queue 0 */
		uint32 rxerrorq1:1;	/* Rx resource err priority queue 1 */
		uint32 rxerrorq2:1;	/* Rx resource err priority queue 2 */
		uint32 rxerrorq3:1;	/* Rx resource err priority queue 3 */
		uint32 rsvd1:4;		/* Reserved */
		uint32 miiphystc:1;	/* MII PHY status change */
		uint32 smidone:1;	/* SMI command done */
		uint32 rsvd:1;		/* Reserved */
		uint32 etherintsum;	/* Ethernet interrupt summary */
	} bits;
#else
	struct {
		uint32 etherintsum;	/* Ethernet interrupt summary */
		uint32 rsvd:1;		/* Reserved */
		uint32 smidone:1;	/* SMI command done */
		uint32 miiphystc:1;	/* MII PHY status change */
		uint32 rsvd1:4;		/* Reserved */
		uint32 rxerrorq3:1;	/* Rx resource err priority queue 3 */
		uint32 rxerrorq2:1;	/* Rx resource err priority queue 2 */
		uint32 rxerrorq1:1;	/* Rx resource err priority queue 1 */
		uint32 rxerrorq0:1;	/* Rx resource err priority queue 0 */
		uint32 rxbuffq3:1;	/* Rx buffer return priority queue 3 */
		uint32 rxbuffq2:1;	/* Rx buffer return priority queue 2 */
		uint32 rxbuffq1:1;	/* Rx buffer return priority queue 1 */
		uint32 rxbuffq0:1;	/* Rx buffer return priority queue 0 */
		uint32 rsvd2:2;		/* Reserved */
		uint32 txudr:1;		/* Tx underrun */
		uint32 rxovr:1;		/* Rx overrun */
		uint32 txerrlow:1;	/* Tx resource err low priority queue */
		uint32 txerrhigh:1;	/* Tx resource err high priority queue*/
		uint32 rsvd3:1;		/* Reserved */
		uint32 rxerror:1;	/* Rx resource error */
		uint32 txendlow;	/* Tx end low priority queue */
		uint32 txendhigh;	/* Tx end high priority queue */
		uint32 rsvd4:2;		/* Reserved */
		uint32 txbufflow:1;	/* Tx buff for low priority queue */
		uint32 txbuffhigh:1;	/* Tx buff for high priority queue */
		uint32 rsvd5:1;		/* Reserved */
		uint32 rxbuffer:1;	/* Rx buffer return indication */
	} bits;
#endif
} gt64240mii_icr_t;

#endif /* GT64240MII_REGS_H_INCLUDED */
