/*
 *=============================================================================
 * File Name   :    gt64240mii.c
 *
 *  Description:    
 *
 *  Created    :    08-03-2001
 *
 *  Author     :    Tony Wong
 *
 *  Copyright(c)  Agile Storage, Inc. 2001
 *=============================================================================
 */
#include <nfx-incl.h>

#include <exports/hwMap.h>
#include <exports/mem-api.h>
#include "gt64240mii.h"
#include <exports/malloc-api.h>

gt64240mii_cfg_t *gt64240mii;
gt64240mii_desc_list_t txPktDesc[GT_MII_MAX_PORTS];


int32 gt64240mii_txUnicast(uint32, eee_desc_t *);
int gt64240mii_initPortRegs(uint32); 
int gt64240mii_fillRxQ(gt64240mii_rxdesc_t *, int); 
int gt64240mii_rxinitQ(gt64240mii_rxdesc_t *, int);
int gt64240mii_txinitQ(gt64240mii_txdesc_t *, int);
int gt64240mii_initDevice(void);

extern int gt_mii_diagLbPktReceive(eee_desc_t *);
extern int bp_getMyMacAddr(uint32, uint32, uint32, uint32, char *);
extern int32 gt64240_addHashTableEntry(uint32 port, uint32 macH, uint32 macL, 
										uint32 rd, uint32 skip);
extern uchar8 *seepGetMyExtMacAddr(uint32);
extern int gt_mii_diagPortEna(uint32);
extern void gt64240mii_clearStats(uint32);
extern address_t cached_to_uncached(address_t);

extern char eee_ssc_ext_mac[][6];
extern char eee_sp_ext_mac[][6];

/*
 *------------------------------------------------------------------------------
 * Function Name:	gt64240mii_initModule
 *
 * Description: 	Initializes the GT64240 MII device driver module.
 *
 * Created by:		Tony Wong
 *
 * Data Created:	08-03-2001
 * 					
 * Returns: 
 *
 *------------------------------------------------------------------------------
 */
int
gt64240mii_initModule()
{
   int rc;
	
   gt64240mii = (gt64240mii_cfg_t *)eee_ramAlloc(sizeof(gt64240mii_cfg_t));
   if(!gt64240mii) {
      printf("[gt64240mii_initModule] Error: Malloc Failed: gt64240mii_cfg_t.\n");
      return(NFX_ERR);
   }
   memset(gt64240mii,0,sizeof(gt64240mii_cfg_t));

   /*
    *--------------------------------------
    * Register with EEE and get app id
    *--------------------------------------
    */
   rc = eee_allocateApplicationId("gt64240_mii");
   if( rc == NFX_ERR )
   {
      printf("[gt64240mii_initModule] Error: Allocate app id failed.");
      return NFX_ERR;
   }
   gt64240mii->appId = rc;

#ifdef NFX_MOD_SSC
   gt64240mii->loopAddr = PACK_UNICAST_PORT_ADDRESS( nfxMySlotId, 
  					                                 nfxMyCPUId, 
					                                 gt64240mii->appId,
  					                                 EEE_USER_DATA,
					                                 NFX_COS_HIGH,
													 SSC_PORT_BP_MGMT);

   rc = eee_allocateApplicationId("lucRcv");
   if( rc == NFX_ERR )
   {
      printf("[gt64240mii_initModule] Error: Allocate IP app id failed.");
      return NFX_ERR;
   }
   gt64240mii->ipAppPort = PACK_UNICAST_PORT_ADDRESS( nfxMySlotId, 
  													  nfxMyCPUId, 
													  rc,
  													  EEE_USER_DATA,
				  									  NFX_COS_HIGH,
													  SSC_PORT_BP_MGMT);

   rc = eee_registerReceiveFunc(gt64240mii->appId,  /* App Id */
								gt64240mii_rxAppPkt,/* Ucast Rx Func */
								NULL);	 	        /* Mcast Rx Func */
   if( rc == NFX_ERR )
   {
      printf("[gt64240mii_initModule] Error: registerReceiveFunc\n");
      return NFX_ERR;
   }
   eee_registerPollingFunc( EEE_HIGH_PRIORITY, gt64240mii_rxPktPoll, 0 );
   eee_registerPollingFunc( EEE_HIGH_PRIORITY, gt64240mii_txCleanPoll, 0 );

   eee_registerTransmitFunc(SSC_PORT_BP_MGMT, 
			                gt64240mii_txUnicast,
			                gt64240mii_txUnicast,
							1500);
#elif defined(PROM)
   gt64240mii->loopAddr = PACK_UNICAST_PORT_ADDRESS( nfxMySlotId, 
  					                                 nfxMyCPUId, 
					                                 gt64240mii->appId,
  					                                 EEE_USER_DATA,
					                                 NFX_COS_HIGH,
													 PROM_PORT_NET0);

   gt64240mii->loopAddrBP = PACK_UNICAST_PORT_ADDRESS( nfxMySlotId, 
  					                                 nfxMyCPUId, 
					                                 gt64240mii->appId,
  					                                 EEE_USER_DATA,
					                                 NFX_COS_HIGH,
													 PROM_PORT_MGMT);

   rc = eee_allocateApplicationId("lucRcv");
   if( rc == NFX_ERR )
   {
      printf("[gt64240mii_initModule] Error: Allocate IP app id failed.");
      return NFX_ERR;
   }
   gt64240mii->ipAppPort = PACK_UNICAST_PORT_ADDRESS( nfxMySlotId, 
  													  nfxMyCPUId, 
													  rc,
  													  EEE_USER_DATA,
				  									  NFX_COS_HIGH,
													  PROM_PORT_NET0);

   rc = eee_registerReceiveFunc(gt64240mii->appId,  /* App Id */
								gt64240mii_rxAppPkt,/* Ucast Rx Func */
								NULL);	 	        /* Mcast Rx Func */
   if( rc == NFX_ERR )
   {
      printf("[gt64240mii_initModule] Error: registerReceiveFunc\n");
      return NFX_ERR;
   }
   eee_registerPollingFunc( EEE_HIGH_PRIORITY, gt64240mii_rxPktPoll, 0 );
   eee_registerPollingFunc( EEE_HIGH_PRIORITY, gt64240mii_txCleanPoll, 0 );

   eee_registerTransmitFunc(PROM_PORT_MGMT, 
			                gt64240mii_txUnicast,
			                gt64240mii_txUnicast,
							1500);
   eee_registerTransmitFunc(PROM_PORT_NET0,
			                gt64240mii_txUnicast,
			                gt64240mii_txUnicast,
							1500);
#ifdef DIAG_R7000   
   eee_registerTransmitFunc( EEE_MGMT,
							 gt64240mii_txUnicast,
                             gt64240mii_txUnicast,
							 1500);
   eee_registerTransmitFunc( EEE_CONTROL,
							 gt64240mii_txUnicast,
                             gt64240mii_txUnicast,
							 1500);
#endif   
   eee_registerDefaultTransmitFunc( EEE_MGMT,
                                    gt64240mii_txUnicast, 9*1024);
   eee_registerDefaultTransmitFunc( EEE_CONTROL,
                                    gt64240mii_txUnicast, 1500);
   eee_registerDefaultTransmitFunc( EEE_USER_DATA,
                                    gt64240mii_txUnicast, 1500);

#elif defined(NFX_MOD_FCNIM) && defined(AGILE_HARDWARE)
 
   gt64240mii->loopAddr = PACK_UNICAST_PORT_ADDRESS( nfxMySlotId,
                                                     nfxMyCPUId,
                                                     gt64240mii->appId,
                                                     EEE_CONTROL,
                                                     NFX_COS_HIGH,
                                                     FCNIM_PORT_BP_MGMT);     

	rc = eee_registerReceiveFunc(gt64240mii->appId,  /* App Id */
                                gt64240mii_rxAppPkt,/* Ucast Rx Func */
                                NULL);              /* Mcast Rx Func */
   if( rc == NFX_ERR )
   {
      printf("[gt64240mii_initModule] Error: registerReceiveFunc\n");
      return NFX_ERR;
   }
   eee_registerPollingFunc( EEE_HIGH_PRIORITY, gt64240mii_rxPktPoll, 0 );
   eee_registerPollingFunc( EEE_HIGH_PRIORITY, gt64240mii_txCleanPoll, 0 );

   eee_registerDefaultTransmitFunc( EEE_CONTROL,
                                    gt64240mii_txUnicast,
									1500);

   eee_registerDefaultTransmitFunc( EEE_MGMT,
                                    gt64240mii_txUnicast,
									1500);
 
  eee_registerTransmitFunc( FCNIM_PORT_BP_CONTROL,
				gt64240mii_txUnicast,
				gt64240mii_txUnicast,         
				1500);         
  eee_registerTransmitFunc( FCNIM_PORT_BP_MGMT,
				gt64240mii_txUnicast,
				gt64240mii_txUnicast,
				1500);         
#endif // NFX_MOD_SSC

   if( gt64240mii_initDevice() == NFX_ERR ) {
	printf("GT64240 MII Initialize Module Failed\n");
	return NFX_ERR;
   }

   gt_mii_diagPortEna(0);		// Enable Port 1.
   return(NFX_OK);

} /* gt64240mii_initModule */


/*
 *------------------------------------------------------------------------------
 * Function Name:	gt64240mii_initDevice
 *
 * Description: 	Initializes the GT64240 device.
 *
 * Created by:		Tony Wong
 *
 * Data Created:	08-03-2001
 * 					
 * Returns: 
 *
 *------------------------------------------------------------------------------
 */
int
gt64240mii_initDevice()
{
   uint32 i, port;
   uint32 align_diff;
   uint32 size_rx;
   uint32 size_tx;
   uchar8 *qstart;
   uchar8 *hstart;
   gt64240mii_txCtrl_t *txc;
   gt64240mii_rxCtrl_t *rxc;
   uint32 *reg_addr;

   //printf("[gt64240mii_initDevice] Enter\n");

   gt64240mii->baseAddr = GT64240MII_BASE_ADDR | 0xB0000000;
   reg_addr = (uint32 *)gt64240mii->baseAddr;
   *reg_addr = 0; /* clear PHY ID register */

   size_tx = sizeof(gt64240mii_txdesc_t) * GT_MII_TX_Q_LEN;
   size_rx = sizeof(gt64240mii_rxdesc_t) * GT_MII_RX_Q_LEN;

   for (port = 0; port < GT_MII_MAX_PORTS; port++)
   {
      gt64240mii->port[port].reg_offset = (port+1) * GT_MII_PORT_OFFSET;
 
      txPktDesc[port].index = 0;
      txPktDesc[port].outdex = 0;
      for (i = 0; i < GT_MII_TX_Q_LEN; i++)
         txPktDesc[port].pktDesc[i] = NULL;

      /*
       * Allocate and align memory on 128-bit boundary
       * for receive and transmit descriptor rings. 
       *
       * Setup descriptor rings. 
       */
      for( i=0; i < GT_MII_TX_Q_CNT; i++ )
      {
 	 txc = &(gt64240mii->port[port].txc[i]);

  	 qstart = (uchar8 *)eee_ramAlloc(size_tx + 16);
         if(!qstart) {
            printf("[gt64240mii_initDevice] Error: Malloc Failed:Tx desc.\n");
            return(NFX_ERR);
         }
	 //printf("txq start: %x\n", qstart);
	 /* Convert to un-cached memory address */
#ifdef FC_USE_NEW_MEM_MAP
	 qstart = (uchar8 *)cached_to_uncached((address_t)qstart);
#else
	 GT64240MII_2UNCACHED((uint32)qstart);
#endif
	 align_diff = (uint32)qstart & 0x0000000f;
	 if( align_diff )
	   align_diff = 16 - align_diff;
	 qstart += align_diff;
	 memset(qstart, 0, size_tx);

	 txc->txqStart = (gt64240mii_txdesc_t *)qstart;
	 txc->txCur = txc->txCln = txc->txqStart;
	 txc->txqEnd = txc->txqStart + GT_MII_TX_Q_LEN;
	 txc->avail = GT_MII_TX_Q_LEN;
	 gt64240mii_txinitQ( txc->txqStart, GT_MII_TX_Q_LEN );
      }

      for( i=0; i < GT_MII_RX_Q_CNT; i++ )
      {
	 rxc = &(gt64240mii->port[port].rxc[i]);

	 qstart = (uchar8 *)eee_ramAlloc(size_rx + 16);
         if(!qstart) {
            printf("[gt64240mii_initDevice] Error: Malloc Failed:Rx desc.\n");
            return(NFX_ERR);
         }
 	 /* Convert to un-cached memory address */
#ifdef FC_USE_NEW_MEM_MAP
	 qstart = (uchar8 *)cached_to_uncached((address_t)qstart);
#else
	 GT64240MII_2UNCACHED((uint32)qstart);
#endif
	 align_diff = (uint32)qstart & 0x0000000f;
	 if( align_diff )
	   align_diff = 16 - align_diff;
	 qstart += align_diff;
	 memset(qstart, 0, size_rx);

	 rxc->rxqStart = (gt64240mii_rxdesc_t *)qstart;
	 rxc->rxCur = rxc->rxqStart;
	 rxc->rxqEnd = rxc->rxqStart + GT_MII_RX_Q_LEN;
	 rxc->avail = GT_MII_RX_Q_LEN;
         gt64240mii->port[port].rxQ[i] = rxc->rxqStart;
	 gt64240mii_rxinitQ( rxc->rxqStart, GT_MII_RX_Q_LEN );
	 gt64240mii_fillRxQ( rxc->rxqStart, GT_MII_RX_Q_LEN );
      }

      /* allocate Hash table */
      hstart = (uchar8 *)eee_ramAlloc(GT_MII_HASH_TBL_SIZE + 8);
      if(!hstart) {
         printf("[gt64240mii_initDevice] Error: Malloc Failed:Hash table.\n");
         return(NFX_ERR);
      }

#ifdef FC_USE_NEW_MEM_MAP
	 hstart = (uchar8 *)cached_to_uncached((address_t)hstart);
#else
      GT64240MII_2UNCACHED((uint32)hstart);
#endif
      align_diff = (uint32)hstart & 0x00000007;
      if( align_diff )
         align_diff = 8 - align_diff;
      hstart += align_diff;
      memset(hstart, 0, GT_MII_HASH_TBL_SIZE);
      gt64240mii->port[port].hash_tbl = hstart;
#ifdef FC_USE_NEW_MEM_MAP
		i = (uint32)virt_to_phys((address_t)hstart);
#else
      i = ((uint32)hstart & ~V2PHYS_ADDR_MASK);
#endif
      gt_mii_writePortReg(port, GT_MII_HTPR, i);
     
#ifndef SSC_MGMT
   //-- Enter station address into hash table.
	if (isFC()) {
		uint32	mac1, mac2;
#ifndef USE_TEMP_EXT_MAC_ADDR_SCHEME
		extern char eee_sp_mac_ext[4][6];
#endif

		if (port == 0) {		//-- DEBUG PORT, Needs an external MAC address.
#ifdef USE_TEMP_EXT_MAC_ADDR_SCHEME
			uchar8 * macaddr;
			macaddr = (uchar8 *)seepGetMyExtMacAddr(nfxMyCPUId);
			mac1 = ((macaddr[0] << 8) |
					(macaddr[1]));
			mac2 = ((macaddr[2] << 24) |
					(macaddr[3] << 16) |
					(macaddr[4] << 8) |
					(macaddr[5]));
//printf("mac1 = 0x%x, mac2 = 0x%x\n", mac1, mac2);

			memcpy(gt64240mii->port[0].macAddr, 
				   macaddr, 6);
#else

			mac1 = ((eee_sp_ext_mac[nfxMyCPUId][0] << 8) |
					(eee_sp_ext_mac[nfxMyCPUId][1]));
			mac2 = ((eee_sp_ext_mac[nfxMyCPUId][2] << 24) |
					(eee_sp_ext_mac[nfxMyCPUId][3] << 16) |
					(eee_sp_ext_mac[nfxMyCPUId][4] << 8) |
					(eee_sp_ext_mac[nfxMyCPUId][5]));
			
			memcpy(gt64240mii->port[0].macAddr, 
				   &eee_sp_ext_mac[nfxMyCPUId][0], 6);
#endif
		}
		else if (port == 1) {		//-- MGMT PORT

       	 	bp_getMyMacAddr( nfxMySlotId,
                             nfxMyCPUId,
                             1,
                             EEE_MGMT,
                             gt64240mii->port[port].macAddr);

			mac1 = ((gt64240mii->port[port].macAddr[0] << 8) |
					(gt64240mii->port[port].macAddr[1]));
			mac2 = ((gt64240mii->port[port].macAddr[2] << 24) |
					(gt64240mii->port[port].macAddr[3] << 16) |
					(gt64240mii->port[port].macAddr[4] << 8) |
					(gt64240mii->port[port].macAddr[5]));
		}
		else if (port == 2) {		//-- CONTROL PORT
       	 	bp_getMyMacAddr( nfxMySlotId,
                             nfxMyCPUId,
                             1,
                             EEE_CONTROL,
                             gt64240mii->port[port].macAddr);

			mac1 = ((gt64240mii->port[port].macAddr[0] << 8) |
					(gt64240mii->port[port].macAddr[1]));
			mac2 = ((gt64240mii->port[port].macAddr[2] << 24) |
					(gt64240mii->port[port].macAddr[3] << 16) |
					(gt64240mii->port[port].macAddr[4] << 8) |
					(gt64240mii->port[port].macAddr[5]));
		}
		else
			break;

		gt64240_addHashTableEntry(port, mac1, mac2, 1, 0);
	}

	if (isSSC()) {
		uint32	mac1, mac2;
#ifndef USE_TEMP_EXT_MAC_ADDR_SCHEME
		extern char eee_ssc_ext_mac[2][6];
#endif

		if (port == 0) {		//-- Backplane MGMT Port
      	bp_getMyMacAddr( nfxMySlotId,
                          nfxMyCPUId,
                          1,
                          EEE_MGMT,
                          gt64240mii->port[port].macAddr);

			mac1 = ((gt64240mii->port[port].macAddr[0] << 8) |
					(gt64240mii->port[port].macAddr[1]));
			mac2 = ((gt64240mii->port[port].macAddr[2] << 24) |
					(gt64240mii->port[port].macAddr[3] << 16) |
					(gt64240mii->port[port].macAddr[4] << 8) |
					(gt64240mii->port[port].macAddr[5]));

		}
#ifdef USE_TEMP_EXT_MAC_ADDR_SCHEME
		else if (port == 1) {		//-- External Port 1
			uchar8 * macaddr;
			macaddr = (uchar8 *)seepGetMyExtMacAddr(0);
			mac1 = ((macaddr[0] << 8) |
					(macaddr[1]));
			mac2 = ((macaddr[2] << 24) |
					(macaddr[3] << 16) |
					(macaddr[4] << 8) |
					(macaddr[5]));
//printf("mac1 = 0x%x, mac2 = 0x%x\n", mac1, mac2);
		}
		else if (port == 2) {		//-- External Port 2
			uchar8 * macaddr;
			macaddr = (uchar8 *)seepGetMyExtMacAddr(1);
			mac1 = ((macaddr[0] << 8) |
					(macaddr[1]));
			mac2 = ((macaddr[2] << 24) |
					(macaddr[3] << 16) |
					(macaddr[4] << 8) |
					(macaddr[5]));
//printf("mac1 = 0x%x, mac2 = 0x%x\n", mac1, mac2);
		}
#else
		else if (port == 1) {		//-- External Port 1
			mac1 = ((eee_ssc_ext_mac[0][0] << 8) |
					(eee_ssc_ext_mac[0][1]));
			mac2 = ((eee_ssc_ext_mac[0][2] << 24) |
					(eee_ssc_ext_mac[0][3] << 16) |
					(eee_ssc_ext_mac[0][4] << 8) |
					(eee_ssc_ext_mac[0][5]));
		}
		else if (port == 2) {		//-- External Port 2
			mac1 = ((eee_ssc_ext_mac[1][0] << 8) |
					(eee_ssc_ext_mac[1][1]));
			mac2 = ((eee_ssc_ext_mac[1][2] << 24) |
					(eee_ssc_ext_mac[1][3] << 16) |
					(eee_ssc_ext_mac[1][4] << 8) |
					(eee_ssc_ext_mac[1][5]));
		}
#endif
		else
			break;

		gt64240_addHashTableEntry(port, mac1, mac2, 1, 0);
	}
#endif	/* !SSC_MGMT */

	gt64240mii_initPortRegs(port); 
	gt64240mii_clearStats(port);
   } /* for each port */

   //printf("[gt64240mii_initDevice] Leave\n");
   return( NFX_OK ); 

} /* gt64240mii_initDevice */ 

/*
 *------------------------------------------------------------------------------
 * Function Name:	gt64240mii_initPortRegs
 *
 * Description: 	Initializes the GT64240 MII registers.
 *
 * Created by:		Tony Wong
 *
 * Data Created:	08-02-2001
 * 					
 * Returns: NFX_OK 
 *
 *------------------------------------------------------------------------------
 */
int
gt64240mii_initPortRegs(uint32 port) 
{
   uint32 val;

   //printf("[gt64240mii_initPortRegs] Enter\n");

   gt64240mii->port[port].pcr.wd = 0;
   gt64240mii->port[port].pcr.bits.pm = 0;/* turn off promiscuous mode for now */
   gt64240mii->port[port].pcr.bits.hs = 1; /* hash table size = 16k */
   gt64240mii->port[port].pcr.bits.rbm = 0; /* Reject broadcasts*/

   gt_mii_writePortReg(port, GT_MII_PCR, gt64240mii->port[port].pcr.wd);

   gt64240mii->port[port].pcxr.wd = 0;
   gt64240mii->port[port].pcxr.bits.mfl = 3; /* max frame length = 64K bytes */
   gt64240mii->port[port].pcxr.bits.rmiien = 1;	/* 0=MII, 1=RMII  */
   gt64240mii->port[port].pcxr.bits.priorx = 3;	/* Rx highest priority queue */
   gt64240mii->port[port].pcxr.bits.priorx_ov = 1;/* Fix all Rx to high queue */
   gt64240mii->port[port].pcxr.bits.priotx = 7;	/* Tx highest priority queue */
   gt64240mii->port[port].pcxr.bits.fctl = 1; /* Pause enable */
   gt_mii_writePortReg(port, GT_MII_PCXR, gt64240mii->port[port].pcxr.wd);

   gt64240mii->port[port].sdcr.wd = 0;
   gt64240mii->port[port].sdcr.bits.rc = 3;    /* retransmit limit */
   gt64240mii->port[port].sdcr.bits.rifb = 1;  /* interrupt on frame boundary */
#if #endian(big)
   gt64240mii->port[port].sdcr.bits.blmt = 0;  /* Tx: 0=Big 1=Little Endian */
   gt64240mii->port[port].sdcr.bits.blmr = 0;  /* Rx: 0=Big 1=Little Endian */
#else
   gt64240mii->port[port].sdcr.bits.blmt = 1;  /* Tx: 0=Big 1=Little Endian */
   gt64240mii->port[port].sdcr.bits.blmr = 1;  /* Rx: 0=Big 1=Little Endian */
#endif
   gt64240mii->port[port].sdcr.bits.bsz = 2;  /* Max burst = 4 64-bit words */
   gt_mii_writePortReg(port, GT_MII_SDCR, gt64240mii->port[port].sdcr.wd);

   /*
    * Initialize Pointer for Rx Desc Queue
    */
#ifdef FC_USE_NEW_MEM_MAP
   val = (uint32)(virt_to_phys((address_t)gt64240mii->port[port].rxc[0].rxqStart));
#else
   val = ((uint32)(gt64240mii->port[port].rxc[0].rxqStart) & ~V2PHYS_ADDR_MASK);
#endif
   gt_mii_writePortReg(port, GT_MII_FRDP3, val);
   gt_mii_writePortReg(port, GT_MII_CRDP3, val);

   /*
    * Initialize Pointer for Tx Desc Queue
    */
#ifdef FC_USE_NEW_MEM_MAP
   val = (uint32)(virt_to_phys((address_t)gt64240mii->port[port].txc[0].txqStart));
#else
   val = ((uint32)(gt64240mii->port[port].txc[0].txqStart) & ~V2PHYS_ADDR_MASK);
#endif
   gt_mii_writePortReg(port, GT_MII_CTDP1, val);

   /* 
    * Enable Rx and Tx 
    */
   gt64240mii->port[port].pcr.bits.en = 1;
   gt_mii_writePortReg(port, GT_MII_PCR, gt64240mii->port[port].pcr.wd);

   gt64240mii->port[port].sdcmr.wd = 0;
   gt64240mii->port[port].sdcmr.bits.erd = 1;  /* Rx: 0=Disable 1=Enable DMA */
   gt64240mii->port[port].sdcmr.bits.txdh = 0; /* Tx: 1=Start DMA high  queue */
   gt_mii_writePortReg(port, GT_MII_SDCMR, gt64240mii->port[port].sdcmr.wd);

   //printf("[gt64240mii_initPortRegs] Leave\n");
   return(NFX_OK);
} /* gt64240mii_initPortRegs */

/*
 *------------------------------------------------------------------------------
 * Function Name:	gt64240mii_rxinitQ
 *
 * Description: 	Initializes descriptor queue by carving up the 
 *			descriptor memory into individual descriptors and 
 *			linking them together.
 *
 * Created by:		Tony Wong
 *
 * Data Created:	08-03-2001
 *------------------------------------------------------------------------------
 */
int
gt64240mii_rxinitQ(gt64240mii_rxdesc_t *qstart, int qLen)
{
   int i;
   uint32 val;
   gt64240mii_rxdesc_t *d;

   d  = qstart;
   /* Convert to physical address space */
   for(i=1; i < qLen; i++, d++) {
#ifdef FC_USE_NEW_MEM_MAP
      val = (uint32)((address_t)virt_to_phys((uint32)(d+1)));
#else
      val = (uint32)((uint32)(d+1) & ~V2PHYS_ADDR_MASK);
#endif
      val = gt64240mii_noswap(val);
      d->link = (gt64240mii_rxdesc_t *)val;
   }
#ifdef FC_USE_NEW_MEM_MAP
   val = (uint32)virt_to_phys((address_t)qstart);
#else
   val = (uint32)qstart & ~V2PHYS_ADDR_MASK;
#endif
   val = gt64240mii_noswap(val);
   d->link = (gt64240mii_rxdesc_t *)val;
   return(NFX_OK);

} /* gt64240mii_rxinitQ */

/*
 *------------------------------------------------------------------------------
 * Function Name:	gt64240mii_txinitQ
 *
 * Description: 	Initializes descriptor queue by carving up the 
 *			descriptor memory into individual descriptors and 
 *			linking them together.
 *
 * Created by:		Tony Wong
 *
 * Data Created:	08-03-2001
 *------------------------------------------------------------------------------
 */
int
gt64240mii_txinitQ(gt64240mii_txdesc_t *qstart, int qLen)
{
   int i;
   uint32 val;
   gt64240mii_txdesc_t *d;

   d  = qstart;
   /* Convert to physical address space */
   for(i=1; i < qLen; i++, d++) {
#ifdef FC_USE_NEW_MEM_MAP
      val = (uint32)((address_t)virt_to_phys((uint32)(d+1)));
#else
      val = (uint32)((uint32)(d+1) & ~V2PHYS_ADDR_MASK);
#endif
      val = gt64240mii_noswap(val);
      d->link = (gt64240mii_txdesc_t *)val;
   }
#ifdef FC_USE_NEW_MEM_MAP
   val = (uint32)virt_to_phys((address_t)qstart);
#else
   val = (uint32)qstart & ~V2PHYS_ADDR_MASK;
#endif
   val = gt64240mii_noswap(val);
   d->link = (gt64240mii_txdesc_t *)val;
   return(NFX_OK);

} /* gt64240mii_txinitQ */

/*
 *------------------------------------------------------------------------------
 * Function Name:	gt64240mii_fillRxQ
 *
 * Description: 	Fills a receive queue with empty system data buffers.
 *
 * Created by:		Tony Wong
 *
 * Data Created:	08-03-2001
 *------------------------------------------------------------------------------
 */
int
gt64240mii_fillRxQ(gt64240mii_rxdesc_t *rxQ, int qLen) 
{
   int i;
   char *pBuf;
   uint32 val;
   gt64240mii_rxdesc_t *d;

   //printf("[gt64240mii_fillRxQ] Enter\n");
   d = rxQ;

   for(i=0; i < qLen; i++, d++)
   {
      pBuf = eee_allocateBuffer(EEE_SHARED_POOL, EEE_BUF_LARGE, 0);
#ifdef FC_USE_NEW_MEM_MAP
      pBuf = (char *)virt_to_phys((address_t)pBuf);
#else
      (uint32)pBuf &= ~V2PHYS_ADDR_MASK; /* Convert to physical addr  */
#endif
      val = (uint32)pBuf;
      val = gt64240mii_noswap(val);
      d->bufptr = (uchar8 *)val;
      val = gt64240mii_noswap((GT_MII_RX_BUF_SIZE << 16));  
      d->bufsize_bytecount = val;
      d->cmdsts = gt64240mii_noswap(GT_MII_CMDSTS_OWN | GT_MII_CMDSTS_INTR);
   }
   //printf("[gt64240mii_fillRxQ] Leave\n");
   return(NFX_OK);

} /* gt64240mii_fillRxQ */

/*
 *------------------------------------------------------------------------------
 * Function Name:	gt_mii_writePortReg
 *
 * Description: 	Writes to a port register.
 *
 * Created by:		Tony Wong
 *
 * Data Created:	08-02-2001
 *------------------------------------------------------------------------------
 */
int
gt_mii_writePortReg(uint32 port, uint32 reg, uint32 val) 
{
#if GT64240_MII_BIG_ENDIAN
   uint32 tmp;
#endif
   volatile uint32 *reg_addr;

   reg_addr = (uint32 *)(gt64240mii->baseAddr + 
                        gt64240mii->port[port].reg_offset + reg);
#if GT64240_MII_BIG_ENDIAN
   /* Need to byte swap */
   tmp = ((val & 0xFF000000) >> 24) | ((val & 0x00FF0000) >> 8 ) |
	  ((val & 0x0000FF00) << 8 ) | ((val & 0x000000FF) << 24);
   *reg_addr = tmp;
#else
   *reg_addr = val;
#endif
   return NFX_OK;
} /* gt_mii_writePortReg */

int
gt_mii_writePortReg_noswap(uint32 port, uint32 reg, uint32 val) 
{
   volatile uint32 *reg_addr;

   reg_addr = (uint32 *)(gt64240mii->baseAddr + 
                        gt64240mii->port[port].reg_offset + reg);
   *reg_addr = val;
   return NFX_OK;
} /* gt_mii_writePortReg_noswap */

/*
 *------------------------------------------------------------------------------
 * Function Name:	gt_mii_readPortReg
 *
 * Description: 	Returns 32-bit value read from port register.
 *
 * Created by:		Tony Wong
 *
 * Data Created:	08-02-2001
 *------------------------------------------------------------------------------
 */
int
gt_mii_readPortReg(uint32 port, uint32 reg) 
{
#if GT64240_MII_BIG_ENDIAN
   uint32 val;
#endif
	uint32 tmp;
   volatile uint32 *reg_addr;

   reg_addr = (uint32 *)(gt64240mii->baseAddr + 
                        gt64240mii->port[port].reg_offset + reg);
   tmp = *reg_addr;
#if GT64240_MII_BIG_ENDIAN
   val = ((tmp & 0xFF000000) >> 24) | ((tmp & 0x00FF0000) >> 8 ) |
	  ((tmp & 0x0000FF00) << 8 ) | ((tmp & 0x000000FF) << 24);
   return(val);
#else
   return(tmp);
#endif
} /* gt_mii_readPortReg */

int
gt_mii_readPortReg_noswap(uint32 port, uint32 reg) 
{
   uint32 tmp;
   volatile uint32 *reg_addr;

   reg_addr = (uint32 *)(gt64240mii->baseAddr + 
                        gt64240mii->port[port].reg_offset + reg);
   tmp = *reg_addr;
   return(tmp);
} /* gt_mii_readPortReg_noswap */

uint32
gt64240mii_swap( uint32 tmp )
{
   uint32 val;
#if GT64240_MII_BIG_ENDIAN
   val = ((tmp & 0xFF000000) >> 24) | ((tmp & 0x00FF0000) >> 8 ) |
	  ((tmp & 0x0000FF00) << 8 ) | ((tmp & 0x000000FF) << 24);
#else
   val = tmp;
#endif
   return(val);
} /* gt64240mii_swap */

uint32
gt64240mii_noswap( uint32 tmp )
{
   uint32 val;
   val = tmp;
   return(val);
} /* gt64240mii_swap */

/*
 *------------------------------------------------------------------------------
 * Function Name:	gt64240mii_rxAppPkt
 *
 * Description: 	Receive packet sent to GT64240 MII mgmt layer.
 *
 * Created by:		Tony Wong
 *
 * Data Created:	08-03-2001
 *------------------------------------------------------------------------------
 */
void
gt64240mii_rxAppPkt( eee_desc_t *edesc )
{
   gt_mii_diagLbPktReceive(edesc);
   return;
} /* gt64240mii_rxAppPkt */

