/*  
 *=============================================================================
 * File Name   :    gt64240mii.h
 *
 *  Description:    
 *
 *  Created    :    08-02-2001
 *
 *  Author     :    Tony Wong
 *
 *  Copyright(c)  Agile Storage, Inc. 2001
 *=============================================================================
 */

#define NFX_DROP_PACKET		-1

#include <nfx-incl.h>
#include <exports/eee-api.h>
#include <exports/cm-api.h>
#include "gt64240mii-regs.h"

#ifndef GT64240MII_H_INCLUDED
#define GT64240MII_H_INCLUDED

#if defined(SSC_MGMT) || defined(LINUX_TEST)
#define GT64240MII_BASE_ADDR	(HW_GALILEO_BASE + 0x2000)
#else
#define GT64240MII_BASE_ADDR	PHYS_TO_K1(HW_GALILEO_BASE + 0x2000)
#endif

#if #endian(little)
#define GT64240_MII_BIG_ENDIAN  0
#else
#define GT64240_MII_BIG_ENDIAN  1
#endif

#define GT_MII_TX_SAFEMODE	1
#define GT_MII_RX_SAFEMODE	1


#ifdef PROM
#define GT_MII_TX_Q_CNT	  	1
#define GT_MII_TX_Q_LEN		256

#define GT_MII_RX_Q_CNT	  	1
#define GT_MII_RX_Q_LEN		256
#else
#define GT_MII_TX_Q_CNT	  	1
#define GT_MII_TX_Q_LEN		2048

#define GT_MII_RX_Q_CNT	  	1
#define GT_MII_RX_Q_LEN		2048
#endif

#define GT_MII_RX_BUF_SIZE	2048

#define GT_MII_HASH_TBL_SIZE	16*1024

typedef unsigned long long dword;

/* list of eee descriptors to free after usage */

typedef struct _gt_desc_list {
	uint32		index;
	uint32		outdex;
	eee_desc_t	*pktDesc[GT_MII_TX_Q_LEN];
} gt64240mii_desc_list_t;

/*
 *----------------------------
 * Descriptor Format
 *----------------------------
 */
#if #endian(big) || defined(LINUX_TEST) /* this doesn't match data sheet, but it works */
typedef struct _gt64240mii_txdesc {
	uint32				bytecnt_rsvd;
	uint32 				cmdsts;
	struct _gt64240mii_txdesc	*link;
	uchar8 				*bufptr;
} gt64240mii_txdesc_t;

typedef struct _gt64240mii_rxdesc {
	uint32				bufsize_bytecount;
	uint32 				cmdsts;
	struct _gt64240mii_rxdesc	*link;
	uchar8 				*bufptr;
} gt64240mii_rxdesc_t;
#endif
#if #endian(little) /* this matches data sheet, but it doesn't work */
typedef struct _gt64240mii_txdesc {
	uint32 				cmdsts;
	uint32				bytecnt_rsvd;
	uchar8 				*bufptr;
	struct _gt64240mii_txdesc	*link;
} gt64240mii_txdesc_t;

typedef struct _gt64240mii_rxdesc {
	uint32 				cmdsts;
	uint32				bufsize_bytecount;
	uchar8 				*bufptr;
	struct _gt64240mii_rxdesc	*link;
} gt64240mii_rxdesc_t;
#endif 

typedef struct _gt64240mii_stats {
	uint32 rxPkts;
	uint32 rxOctets;
	uint32 txPkts;
	uint32 txOctets;
	uint32 rxCrc;
	uint32 rxNoBuf;
	uint32 txQFull;
} gt64240mii_stats_t;

typedef struct _gt64240mii_txCtrl {
	gt64240mii_txdesc_t	*txqStart;
	gt64240mii_txdesc_t	*txqEnd;
	gt64240mii_txdesc_t	*txCur;
	gt64240mii_txdesc_t	*txCln;
	uint32  avail;
	uint32  qd;
	uint32  unqd;
	uint32  pkts;
	uint32  octets;
	uint32	qFull;
	uint32	collErr;
	uint32	relimErr;
	uint32	udrErr;
	uint32	lcolErr;
} gt64240mii_txCtrl_t;

typedef struct _gt64240mii_rxBuf_cache {
	uchar8 *ptr;
	uint32 len;
} gt64240mii_bc_t;

typedef struct _gt64240mii_rxCtrl {
	gt64240mii_rxdesc_t	*rxqStart;
	gt64240mii_rxdesc_t	*rxqEnd;
	gt64240mii_rxdesc_t	*rxCur;
	gt64240mii_bc_t		bc[EEE_MAX_BUFS];
	uint32  first_cmdsts;
	uint32  loopback;
	uint32	bufCnt;
	uint32  avail;
	uint32	pkts;
	uint32	octets;
	uint32	shortErr;
	uint32	longErr;
	uint32	ovrErr;
	uint32	collErr;
	uint32	crcErr;
	uint32 	noBufErr;
	uint32 	noDescErr;
} gt64240mii_rxCtrl_t;

typedef struct _gt64240mii_cfg {
	uint32	appId;
	uint32	baseAddr;
	uint32	phyAddr;
	uint32	loopAddr;/* System address of loopback app */
	uint32	loopAddrBP;
	uint32	ipAppPort;	/* packed port address */
	struct {
		char    macAddr[6];
		uint32	tx_trace;
		uint32	rx_trace;
		gt64240mii_rxdesc_t	*rxQ[GT_MII_RX_Q_CNT];
		gt64240mii_txCtrl_t	txc[GT_MII_TX_Q_CNT];
		gt64240mii_rxCtrl_t	rxc[GT_MII_RX_Q_CNT];
		gt64240mii_stats_t	stats;
		gt64240mii_pcr_t	pcr;
		gt64240mii_pcxr_t	pcxr;
		gt64240mii_sdcr_t	sdcr;
		gt64240mii_sdcmr_t	sdcmr;
		uint32			reg_offset;
		uchar8			*hash_tbl;
	} port[GT_MII_MAX_PORTS];
} gt64240mii_cfg_t;

/*
 * Macros to convert virtual address to phy and back again.
 */
#if defined(EVAL_BOARD) || defined(AGILE_HARDWARE)
#ifndef FC_USE_NEW_MEM_MAP
#define V2PHYS_ADDR_MASK	0xA0000000
#define GT64240MII_2UNCACHED(_x_)	((_x_) |= 0x20000000)
#endif
#else
#define V2PHYS_ADDR_MASK	0x00000000
#define GT64240MII_2UNCACHED(_x_)	((_x_)) 
#endif	/* EVAL_BOARD || AGILE_HARDWARE */

/*
 * External Variable Declarations
 */
extern gt64240mii_cfg_t *gt64240mii;
extern gt64240mii_desc_list_t txPktDesc[GT_MII_MAX_PORTS];

/*
 * Function Prototypes
 */
void gt64240mii_rxPktPoll(void *, uint32);
void gt64240mii_txCleanPoll(void *, uint32);
void gt64240mii_rxAppPkt(eee_desc_t *);
int gt_mii_writePortReg(uint32, uint32, uint32);
int gt_mii_readPortReg(uint32, uint32);
uint32 gt64240mii_swap(uint32);
int gt_mii_writePortReg_noswap(uint32, uint32, uint32);
int gt_mii_readPortReg_noswap(uint32, uint32);
uint32 gt64240mii_noswap(uint32);

#endif /* GT64240MII_H_INCLUDED */
