/*
 *-----------------------------------------------------------------
 *
 * Name:
 *
 * RCS:  $Id: delay.c,v 1.6 2002/10/22 01:30:37 jong Exp $
 *
 * Copyright (C) 2001, Agile Storage, Inc.
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	11/07/01
 *
 *-----------------------------------------------------------------
 */

#include <nfx-incl.h>
#include <sys/gt64240r.h>
#include <exports/hwMap.h>

/*-----------------------------------------------------------------
 * Name:	delay_us
 *
 * Description:	blocking delay loop which uses the internal gerneral purpose
 *				timer.  Counts down every TCLK.
 *
 *				parameter is in microseconds.
 *
 *				Uses GP timer 0.
 *
 * Created by:	Rick Lund
 *
 * Date Created:	10/30/01
 *
 *-----------------------------------------------------------------
 */
void
delay_us(
	uint32	us			/* how many microseconds to pause */
)
{
	extern uint32 get_busclock(void);
	uint32	tclk_cnt = get_busclock() * us;
	uint32	d;

	volatile uint32	* tptr_value = (volatile uint32 *)
								PHYS_TO_K1(HW_GALILEO_BASE + TIMER_COUNTER0);
	volatile uint32	* tptr_control = (volatile uint32 *)
						PHYS_TO_K1(HW_GALILEO_BASE + TIMER_COUNTER_0_3_CONTROL);
	volatile uint32	* tptr_imask = (volatile uint32 *)
				PHYS_TO_K1(HW_GALILEO_BASE + TIMER_COUNTER_0_3_INTERRUPT_MASK);
#if 0
	volatile uint32	* tptr_icause = (volatile uint32 *)
				PHYS_TO_K1(HW_GALILEO_BASE + TIMER_COUNTER_0_3_INTERRUPT_CAUSE);
#endif

	*tptr_value = tclk_cnt;
	*tptr_imask &= ~1;
	*tptr_control &= ~0xFF;
	*tptr_control |= 0x1;		/* counter mode | enable */

	while (1) {
		d = *tptr_value;
		if ((d == 0) || (d == tclk_cnt))
			break;
	}
	*tptr_control &= ~0xFF;
}

