/*
 *-----------------------------------------------------------------
 *
 * Name:
 *
 * RCS:  $Id: mem-api.c,v 1.6 2002/11/07 01:18:51 rick Exp $
 *
 * Copyright (C) 2002, Agile Storage, Inc.
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	7/17/02
 *
 *-----------------------------------------------------------------
 */

#include <nfx-incl.h>
#include <exports/hwMap.h>
#include <exports/mem-api.h>

#ifndef PROM
/* for proms the memorysize variables is declared in main.c */
uint64	memorysize = 0;
#endif
uint32	memorysize_MB = 0;
uint32	pcisize = 0;

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	7/22/02
 *
 *-----------------------------------------------------------------
 */
void
mem_map_init()
{

#ifdef FC_USE_NEW_MEM_MAP
	extern void mem_map_tlb_init();
	extern void mem_map_gt_init();

	mem_map_tlb_init();

	mem_map_gt_init();
#endif

}

/*-----------------------------------------------------------------
 * Name:	is_valid_addr(adr)
 *
 * Description: returns 0 or 1
 *
 * Created by:	Rick Lund
 *
 * Date Created:	7/17/02
 *
 *-----------------------------------------------------------------
 */
int32
is_valid_addr(
	address_t	adr
)
{
	if ((adr >= K0BASE) && (adr < K2BASE)) 
		return 1;

#ifdef FC_USE_NEW_MEM_MAP
	/* DRAM - cached */
	if ((adr >= SW_RAM_BASE_CACHED) && (adr < SW_RAM_BASE_CACHED + memorysize))
		return 1;
	/* DRAM - uncached */
	if ((adr >= SW_RAM_BASE_UNCACHED) && (adr < SW_RAM_BASE_UNCACHED + memorysize))
		return 1;
	
	/* PCI spaces */
	if ((adr >= SW_PCI_BASE) && (adr < SW_PCI_BASE + pcisize))
		return 1;
#else
	/* PCI spaces */
	if ((adr >= 0x60000000) && (adr < 0x74000000))
		return 1;
#endif

	return 0;
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	7/18/02
 *
 *-----------------------------------------------------------------
 */
address_t
virt_to_phys(
	address_t	virt
)
{
	address_t	phys;

	if ((virt >= K0BASE) && (virt < K2BASE))
		phys = KVA_TO_PA(virt);
#ifdef FC_USE_NEW_MEM_MAP
	else if ((virt >= 0x40000000) && (virt < 0x50000000))
		phys = virt - 0x40000000;
	else if ((virt >= 0x50000000) && (virt < 0x60000000))
		phys = virt - 0x30000000;

	else if ((virt >= 0x60000000) && (virt < 0x70000000))
		phys = virt - 0x60000000;
	else if ((virt >= 0x70000000) && (virt < 0x80000000))
		phys = virt - 0x50000000;
#endif

	else {
		address_t   ra;

		ra = (address_t)get_ra();
		panic("Invalid virt, %p, ra = %p\n", virt, ra);
		phys = 0;
	}

	return phys;
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	7/18/02
 *
 *-----------------------------------------------------------------
 */
address_t
phys_to_virt_cached(
	address_t	phys
)
{
	address_t	virt;

	if ((phys >= 0x0) && (phys < 0x10000000))
		virt = 0x40000000 + phys;
	else if ((phys >= 0x20000000) && (phys < 0x30000000))
		virt = 0x30000000 + phys;
	else {
		address_t   ra;

		ra = (address_t)get_ra();
		panic("Invalid virt, %p, ra = %p\n", phys, ra);
		virt = 0;
	}

	return virt;
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	7/18/02
 *
 *-----------------------------------------------------------------
 */
address_t
phys_to_virt_uncached(
	address_t	phys
)
{
	address_t	virt;

	if ((phys >= 0x0) && (phys < 0x10000000))
		virt = 0x60000000 + phys;
	else if ((phys >= 0x20000000) && (phys < 0x30000000))
		virt = 0x50000000 + phys;
	else {
		address_t   ra;

		ra = (address_t)get_ra();
		panic("Invalid virt, %p, ra = %p\n", phys, ra);
		virt = 0;
	}

	return virt;
}

#ifndef PROM
/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	7/1/02
 *
 *-----------------------------------------------------------------
 */
address_t
cached_to_uncached(
	address_t	cached
)
{
	address_t	ra;

	if ((cached >= K0BASE) && (cached < K1BASE))
		return K0_TO_K1(cached);

#ifdef FC_USE_NEW_MEM_MAP
	if ((cached >= SW_RAM_BASE_CACHED) &&
							(cached < SW_RAM_BASE_CACHED + memorysize))
		return (cached - SW_RAM_BASE_CACHED + SW_RAM_BASE_UNCACHED);
#endif

	ra = (address_t)get_ra();
	panic("Invalid cached addr, %p, ra = %p\n", cached, ra);

	return 0;
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	7/1/02
 *
 *-----------------------------------------------------------------
 */
address_t
uncached_to_cached(
	address_t	uncached
)
{
	address_t	ra;

	if ((uncached >= K1BASE) && (uncached < K2BASE))
		return K1_TO_K0(uncached);

#ifdef FC_USE_NEW_MEM_MAP
	if ((uncached >= SW_RAM_BASE_UNCACHED) &&
							(uncached < SW_RAM_BASE_UNCACHED + memorysize))
		return (uncached - SW_RAM_BASE_UNCACHED + SW_RAM_BASE_CACHED);
#endif

	ra = (address_t)get_ra();
	panic("Invalid uncached addr, %p, ra = %p\n", uncached, ra);

	return 0;
}

#endif /* !PROM */
