/*
 *-----------------------------------------------------------------
 *
 * Name:        gt64440mii-reg.h
 *
 * RCS:  $Id: $
 *
 * Copyright (C) 2004, OnStor Inc.
 *
 * Description: Ethernet port register definition file for GT64440.
 *
 * Created by:  Shaun Hui
 *
 * Date Created: 04/13/04
 *
 *-----------------------------------------------------------------
 */
#ifndef GT64440MII_REGS_H_INCLUDED
#define GT64440MII_REGS_H_INCLUDED


#define GT_MII_MAX_PORTS	3
#define GT_MII_LB_NONE		0
#define GT_MII_LB_INTERNAL	1
#define GT_MII_LB_EXTERNAL	2

/*----------------------------------
 * Bit definitions of Serial Control Reg 
 *----------------------------------
 */
#define PSCR_PORTEN             (1 << 0)
#define PSCR_FORCE_LINK_PASS    (1 << 1)
#define PSCR_DIS_AN_DUPLEX      (1 << 2)
#define PSCR_DIS_AN_FC          (1 << 3)
#define PSCR_PAUSE_ADV          (1 << 4)
#define PSCR_FORCE_FC_MODE(n)   (((n) & 3) << 5)
#define PSCR_FORCE_BP_MODE(n)   (((n) & 3) << 7)
#define PSCR_DIS_FORCE_LINK_FAIL        (1 << 10)
#define PSCR_RETR_FOREVER       (1 << 11)
#define PSCR_DIS_AN_SPEED       (1 << 13)
#define PSCR_DTE_ADVERT         (1 << 14)
#define PSCR_AN_BYPASS_EN       (1 << 15)
#define PSCR_RESTART_AN         (1 << 16)
#define PSCR_MRU(n)             (((n) & 7) << 17)
#define PSCR_SET_FULLDX         (1 << 21)
#define PSCR_SET_FC_EN          (1 << 22)
#define PSCR_SET_GMII_SPEED     (1 << 23)
#define PSCR_SET_MII_SPEED      (1 << 24)
                                                                                
#define PSCR_DEFAULT_SETTING    \
    (PSCR_FORCE_LINK_PASS | PSCR_DIS_AN_FC | PSCR_PAUSE_ADV | 0x200 | \
     PSCR_DIS_FORCE_LINK_FAIL | PSCR_MRU(2) | PSCR_SET_FULLDX | PSCR_SET_FC_EN)
                                                                                
/*----------------------------------
 * SMI Commands
 *----------------------------------
 */
#define MII_BMCR        0x00    /* Basic mode control register (rw) */
#define BMCR_RESET      0x8000  /* reset */
#define BMCR_LOOP       0x4000  /* loopback */
#define BMCR_S100       0x2000  /* speed (10/100) select */
#define BMCR_AUTOEN     0x1000  /* autonegotiation enable */
#define BMCR_PDOWN      0x0800  /* power down */
#define BMCR_ISO        0x0400  /* isolate */
#define BMCR_STARTNEG   0x0200  /* restart autonegotiation */
#define BMCR_FDX        0x0100  /* Set duplex mode */
#define BMCR_CTEST      0x0080  /* collision test */

/*----------------------------------
 * Bit definitions of SMI Reg 
 *----------------------------------
 */
#define SMI_PHYAD_0(n)  (((n) & 0x1f) << 0)
#define SMI_PHYAD_1(n)  (((n) & 0x1f) << 5)
#define SMI_PHYAD_2(n)  (((n) & 0x1f) << 10)
                                                                                
#define SMI_DATA(n)     (((n) & 0xffff) << 0)
#define SMI_PHYAD(n)    (((n) & 0x1f) << 16)
#define SMI_REGAD(n)    (((n) & 0x1f) << 21)
#define SMI_READ        (1 << 26)
#define SMI_WRITE       (0 << 26)
#define SMI_READVALID   (1 << 27)
#define SMI_BUSY        (1 << 28)
                                                                                
/*----------------------------------
 * Bit definitions of Port Config Reg 
 *----------------------------------
 */
#define PCR_UPM         1
#define PCR_RXQ(n)      (((n) & 7) << 1)
#define PCR_RXQARP(n)   (((n) & 7) << 4)
#define PCR_RB          (1 << 7)
#define PCR_RBIP        (1 << 8)
#define PCR_RARP        (1 << 9)
#define PCR_AMNOTXES    (1 << 12)
#define PCR_TCP_CAPEN   (1 << 14)
#define PCR_UDP_CAPEN   (1 << 15)
#define PCR_TCPQ(n)     (((n) & 7) << 16)
#define PCR_UDPQ(n)     (((n) & 7) << 19)
#define PCR_BPDUQ(n)    (((n) & 7) << 22)
                                                                                
#define PCR_DEFAULT_SETTING \
    (PCR_RXQ(0) | PCR_RXQARP(0) | PCR_TCPQ(0) | PCR_UDPQ(0) | PCR_BPDUQ(0))
                                                                                
/*----------------------------------
 * Bit definitions of Port Config Extend Reg 
 *----------------------------------
 */
#define PCXR_SPAN       (1 << 1)
#define PCXR_PAREN      (1 << 2)
                                                                                
#define PCXR_DEFAULT_SETTING    0
                                                                                
/*----------------------------------
 * Bit definitions of SDMA Config Reg
 *----------------------------------
 */
#define SDCR_RIFB       (1 << 0)
#define SDCR_RXBSZ(n)   (((n) & 7) << 1)
#define SDCR_BLMR       (1 << 4)
#define SDCR_BLMT       (1 << 5)
#define SDCR_SWAPMODE   (1 << 6)
#define SDCR_IGP_INT_RX(n) (((n) & 0x3fff) << 8)
#define SDCR_TXBSZ(n)   (((n) & 7) << 22)
                                                                                
#define SDCR_BLMRT      (SDCR_BLMR | SDCR_BLMT)
#define SDCR_DEFAULT_SETTING \
    (SDCR_RXBSZ(4) | SDCR_IGP_INT_RX(0) | SDCR_TXBSZ(4))
                                                                                

/*----------------------------------
 * Bit definitions of Interrupt Cause Reg 
 *----------------------------------
 */
#define ICR_RXBUFFER    (1 << 0)
#define ICR_EXTEND      (1 << 1)
#define ICR_RXBUFFERQ(n) ((1 << ((n) & 7)) << 2)
#define ICR_RXERROR     (1 << 10)
#define ICR_RXERRORQ(n) ((1 << ((n) & 7)) << 11)
#define ICR_RTXEND      (0xff << 19)
#define ICR_TXEND(n)    ((1 << ((n) & 7)) << 19)
#define ICR_ETHERINTSUM (1 << 31)
                                                                                
/*----------------------------------
 * Extended Interrupt Cause Reg Bits
 *----------------------------------
 */
#define XICR_TXBUFF(n)  ((1 << ((n) & 7)) << 0)
#define XICR_TXERROR(n) ((1 << ((n) & 7)) << 8)
#define XICR_PHYSTC     (1 << 16)
#define XICR_RXOVR      (1 << 18)
#define XICR_TXUDR      (1 << 19)
#define XICR_LINKCAHNGE (1 << 20)
#define XICR_PARTITION  (1 << 21)
#define XICR_AUTONEGDONE (1 << 22)
#define XICR_INTERNAL_ADDR_ERROR (1 << 23)
#define XICR_ETHER_INT_SUM (1 << 31)
                                                                                

/*----------------------------------
 * TX Descriptor cmdsts Bit definitions
 *----------------------------------
 */
#define TX_O            (1<<31)       // own bit
#define TX_AM           (1<<30)       // auto mode
#define TX_EI           (1<<23)       // enable interrupt
#define TX_GC           (1<<22)       // generate crc
#define TX_F            (1<<21)       // first 
#define TX_L            (1<<20)       // last
#define TX_P            (1<<19)       // pad
#define TX_GIPCHK       (1<<18)       // generate IPv4 checksum
#define TX_GL4CHK       (1<<17)       // generate TCP/UDP checksum
#define TX_L4TYPE       (1<<16)       // TCP/UDP
#define TX_VLAN         (1<<15)       // VLAN Tagged
#define TX_IPV4HDLEN    (0x0f<<11)    // IP V4 header length
#define TX_L4CHK_MODE   (1<<10)       // checksum calc. mode
#define TX_LLC_SNAP     (1<<9)        // LLC SNAP
#define TX_EC           (3<<1)        // error coding
#define TX_EC_RL        (2<<1)        // excessive collision
#define TX_EC_UR        (1<<1)        // underrun
#define TX_EC_LC        (0<<1)        // late collision
#define TX_ES           (1<<0)        // error summary
                                                                                

/*----------------------------------
 * RX Descriptor cmdsts Bit definitions
 *----------------------------------
 */
#define RX_O            (1<<31)       // own bit
#define RX_L4CHKOK      (1<<30)       // layer 4 cksum ok
#define RX_EI           (1<<29)       // enable interrupt
#define RX_U            (1<<28)       // unknown DA
#define RX_F            (1<<27)       // first
#define RX_L            (1<<26)       // last
#define RX_IPHEADOK     (1<<25)       // Ip header ok
#define RX_L3IP         (1<<24)       // frame type IPV4
#define RX_LAYER2EV2    (1<<23)       // layer 2 is ethernet v2
#define RX_LAYER4       (3<<21)       // layer 4 protocol
#define RX_BPDU         (1<<20)       // BPDU
#define RX_VLAN         (1<<19)       // VLAN tagged
#define RX_L4CHK        (0xffff<<3)   // calculated TCP/UDP cksum
#define RX_EC           (3<<1)        // error coding
#define RX_EC_RSRC      (3<<1)        // no desc. resource
#define RX_EC_LONG      (2<<1)        // frame too long
#define RX_EC_OVR       (1<<1)        // overrun
#define RX_EC_CRC       (0<<1)        // crc error
#define RX_ES           1             // error summary





#endif /* GT64440MII_REGS_H_INCLUDED */
