/*
 *----------------------------------------------------------------------------
 *
 * Name:         gt64440mii-tx.c
 *
 * Copyright (C) 2004, OnStor, Inc.
 *
 * Description : Contains Disco3 Ethernet Driver TX routines.
 *               Ported from sm-gt64240-mii module.
 *
 * Created by  : Shaun Hui
 *
 * Date Created: 04/13/04
 *
 *-----------------------------------------------------------------
 */

#ifndef SIBYTE_BC

#include "gt64440mii.h"
#include <exports/mem-api.h>

extern int gt64440_printBuf(char *, int, int);
extern void *eee_getBufBase(void *);
extern int32 eee_getBufSize(int32);
extern int32 eee_getBufType(void *);

int gt64440mii_txCleanupQ(uint32, gt64440mii_txCtrl_t *);
int isBackplane_packet(int);
int isIp_packet(eee_desc_t *);

extern uint32 gt_mii_test_port;
extern uint32 gt_mii_test;

#define GT_FLUSH 1

/*
 *------------------------------------------------------------------------------
 * Function Name:	gt64440mii_txUnicast
 *
 * Description: 	Transmits a unicast packet to the backplane.
 *
 * Created by:		Tony Wong
 *
 * Data Created:	08-06-2001
 * 					
 * Returns: ?????
 *
 *------------------------------------------------------------------------------
 */
int
gt64440mii_txUnicast( uint32 dest, eee_desc_t *pktDesc )
{
   int first = 1;
   int i;
#ifdef FEATURE_DIAG_PROM			
   uchar8 *tmp;
#endif
   int bufCnt;
   int pktLen;
   gt64440mii_txCtrl_t *txc;
   gt64440mii_txdesc_t *txCur;
   gt64440mii_txdesc_t *txTop = NULL;
   int port;
#ifndef PROM
	int	remotePort;
#endif
	uint32	dtype, dest_slot;
#ifdef FEATURE_DIAG_PROM			
	uint32 mdk_pkt=0;
#endif
	port = EEE_GET_PORT_ID(pktDesc->hdr.dest_port);
	dtype = EEE_GET_DTYPE(pktDesc->hdr.dest_port);
	dest_slot = EEE_GET_SLOT_NUM(pktDesc->hdr.dest_port);
#if defined(PROM)
	if (isSSC()) {
		if (port == PROM_PORT_NET0)
			port = 1;
		else if (port == PROM_PORT_NET1)
			port = 2;
		else if (port == PROM_PORT_MGMT)
			port = 0;
		else if (port == PORT_BEST_FIT) {
			if (dtype == EEE_MGMT)
				port = 0;
			else if (dtype == EEE_USER_DATA)
				port = 1;
			else
				port = 2;
		}
		else {
			panic("Invalid port address, dest_port = %x\n", pktDesc->hdr.dest_port);
			return( EEE_DROP_PACKET );
		}
#if defined(R9000)
		port = 0;
#endif
	}
	else if (isFC()) {
		if ((port == PROM_PORT_NET0) || (port == PROM_PORT_NET1))
			port = 0;
		else if (port == PROM_PORT_MGMT)
#ifdef FEATURE_DIAG_PROM			
		{	
			uchar8 *mark;
      		mark = tmp = pktDesc->bd[1].buf;
			if( (*tmp++ == 'b') && (*tmp++ == 'c') && (*tmp++ == 'm') )
			{
					mdk_pkt = 1;
					mark = tmp;
					if(*tmp++ == '0')
						port = 0;
					else {
						tmp = mark;
						if(*tmp++ == '2') 
							port = 2;
						else
							port = 1;	
					}
			/*printf("rcvd mdk pkt for port %d\n",port);*/
			}
			else
				port = 1;
		}
#else		
			port = 1;
#endif			
		else if (port == PORT_BEST_FIT) {
			if (dtype == EEE_MGMT)
				port = 1;
			else if (dtype == EEE_USER_DATA)
				port = 0;
			else
				port = 2;
		}
		else {
			panic("Invalid port address, dest_port = %x\n", pktDesc->hdr.dest_port);
			return( EEE_DROP_PACKET );
		}
	}
	else {
		panic("Invalid card type\n");
	}

	if ((gt_mii_test) || (isBackplane_packet(port) && (!isIp_packet(pktDesc)))) {
		// Add Mac and Agile Backplane header.
		eee_addHeader(pktDesc, 1, gt64440mii->port[port].macAddr); 
	}
#ifdef FEATURE_DIAG_PROM			
		// Add Mac and Agile Backplane header.
	if((port == 0) && (mdk_pkt)) {
		eee_addHeader(pktDesc, 1, gt64440mii->port[port].macAddr); 
	}		
#endif	

#else //-- This  must be the SP_NIM Runtime
	if (isFC()) {
		if (port == FCNIM_PORT_BP_MGMT)
			port = 1;
		else
		if (port == FCNIM_PORT_BP_CONTROL)
			port = 2;
		else 
		if (port == PORT_BEST_FIT) {
			if (dtype == EEE_MGMT)
				port = 1;
			else
			{
				if (gt_mii_test) {
					if (dtype == EEE_USER_DATA)
						port = 0;
					else
						port = 2;
				}
				else
					port = 2;
			}
		}
		else {
			panic("Invalid port address, dest_port = %x\n", pktDesc->hdr.dest_port);
			return( EEE_DROP_PACKET );
		}
	}
	else {
		panic("Invalid card type\n");
	}
	
	if (dest_slot == 1) {
		switch (nfxMyCPUId) {
            case 0:
                if (dtype == EEE_CONTROL)
                    remotePort = 3;
                else
                    remotePort = 1;
                break;
            case 1:
                if (dtype == EEE_CONTROL)
                    remotePort = 3;
                else
                    remotePort = 1;
                break;
            case 2:
                if (dtype == EEE_CONTROL)
                    remotePort = 6;
                else
                    remotePort = 1;
                break;
            case 3:
                if (dtype == EEE_CONTROL)
                    remotePort = 6;
                else
                    remotePort = 1;
                break;
            default:
				printf("gt64440mii_txUnicast: invalid nfxMyCPUId=%d\n",
						nfxMyCPUId);
				remotePort = 0;
                    break;
		}	
	}
	else
		remotePort = 1;
	//-- Add Mac and Agile Backplane header.
	if (eee_addHeader( pktDesc, remotePort,
					   gt64440mii->port[port].macAddr ) != NFX_OK) {
		return EEE_DROP_PACKET;
	}

#endif // PROM

   pktLen = pktDesc->hdr.control & EDESC_TOTAL_LEN;
   bufCnt = (pktDesc->hdr.control & EDESC_BUFCNT_MASK) >> EDESC_BUFCNT_SHIFT;
	
   txc = &(gt64440mii->port[port].txc[0]);

   /* printf("gt64440mii_txunicast: enter\n"); */

   /* 
    * Check for room on the transmit queue.
    */
   if( bufCnt > txc->avail-1 ) {
      /* printf("tx: no room\n"); */
      /*
       * Call transmit cleanup to try and free up some tx descriptors.
       */
      gt64440mii_txCleanupQ( port, txc );
      if( bufCnt > txc->avail-1 ) {
          txc->qFull++;
#if GT_FLUSH
	  CACHE_FLUSH_PACKET(pktDesc);
#endif
          return( EEE_DROP_PACKET );
      }
   }
   txCur = txc->txCur;

	if (gt64440mii->port[port].tx_trace)
		printf("TX BUF:\n");

   i = 1;
   if( bufCnt == 1 ) {

#if GT_MII_TX_SAFEMODE
      /* Make sure driver owns descriptor. */
      if( txCur->cmdsts & TX_O ) {
         gt64440mii->port[port].txc[0].qFull++;
#if GT_FLUSH
	 CACHE_FLUSH_PACKET(pktDesc);
#endif
         return( EEE_DROP_PACKET );
      }
#endif /* GT_MII_TX_SAFEMODE */

      txPktDesc[port].pktDesc[txPktDesc[port].index] = pktDesc;
      if (++txPktDesc[port].index >= GT_MII_TX_Q_LEN)
         txPktDesc[port].index = 0;

		if (gt64440mii->port[port].tx_trace)
			gt64440_printBuf( (char *)(uint32)pktDesc->bd[1].buf,
												pktDesc->bd[1].len, 0);

#ifdef FC_USE_NEW_MEM_MAP
      txCur->bufptr = (uchar8 *)virt_to_phys((address_t)(pktDesc->bd[1].buf));
#else
      txCur->bufptr = (uchar8 *)((uint32)(pktDesc->bd[1].buf) & ~0xA0000000);
#endif
      txCur->bytecnt_rsvd = (pktDesc->bd[1].len << 16);

#if GT_FLUSH
      CACHE_FLUSH_PACKET(pktDesc);
#endif
      txCur->cmdsts = (TX_O | TX_F | TX_L | TX_P | TX_GC | TX_EI );

      GT_WRITE(ETH_TRANSMIT_QUEUE_COMMAND_REG(port), 1);

//   gt64440mii->port[port].sdcmr.bits.txdh = 1; /* enable Tx DMA */
//   gt_mii_writePortReg(port, GT_MII_SDCMR, gt64440mii->port[port].sdcmr.wd);

      if( ++txCur >= txc->txqEnd )
         txCur = txc->txqStart;
   }
   else 
   {
      if( pktDesc->hdr.control & EDESC_COMMAND_USE_FIRST_BUFFER )
	      i=0;
	  else
	      i=1;  

	//printf("DestPort: %x\n", pktDesc->hdr.dest_port);
	while( bufCnt-- ) {
#if GT_MII_TX_SAFEMODE
		/* Make sure driver owns descriptor. */
		if( txCur->cmdsts & TX_O ) {
	   		txc->qFull++;
#if GT_FLUSH
			CACHE_FLUSH_PACKET(pktDesc);
#endif
	   		return( EEE_DROP_PACKET );
		}
#endif /* GT_MII_TX_SAFEMODE */

#if 0
		{
		int xlen;
		printf("TX: Buf: %x, Len: %d\n", pktDesc->bd[i].buf, pktDesc->bd[i].len);
		xlen = pktDesc->bd[i].len;
		if(xlen > 64)
			xlen = 64;
		dp83820_printBuf( pktDesc->bd[i].buf, xlen, 1);
		}
#endif

		if (gt64440mii->port[port].tx_trace)
			gt64440_printBuf( (char *)(uint32)pktDesc->bd[i].buf,
													pktDesc->bd[i].len, 0);

#ifdef FC_USE_NEW_MEM_MAP
		txCur->bufptr = (uchar8 *)virt_to_phys((address_t)(pktDesc->bd[i].buf));
#else
		txCur->bufptr = (uchar8 *)((uint32)(pktDesc->bd[i].buf) & ~0xA0000000);
#endif
    	txCur->bytecnt_rsvd = (pktDesc->bd[i].len << 16);
		if (first)
		{   
			txTop = txCur; /* save this to transfer ownership later */
	   		first = 0;
    	}
		else
		if( bufCnt ) {
	   		txCur->cmdsts = (TX_O | TX_P); 
		}
		else  // Process last data buffer in packet.
		{
    		txPktDesc[port].pktDesc[txPktDesc[port].index] = pktDesc;
       		if (++txPktDesc[port].index >= GT_MII_TX_Q_LEN)
       	 		txPktDesc[port].index = 0;

#if GT_FLUSH
       	    CACHE_FLUSH_PACKET(pktDesc);
#endif
	    	txCur->cmdsts = (TX_O | TX_GC | TX_P | TX_EI | TX_L );
		}

		//printf("bufPtr: %x, byteCnt: %x, cmdsts: %x\n", txCur->bufptr, 
		//												txCur->bytecnt_rsvd,
		//												txCur->cmdsts);
		i++;
		if( ++txCur >= txc->txqEnd )
	   		txCur = txc->txqStart;
	} /* while */
    i--;
    /* transfer ownership for top of descriptor chain here */
    txTop->cmdsts = (TX_O | TX_P | TX_F);

      GT_WRITE(ETH_TRANSMIT_QUEUE_COMMAND_REG(port), 1);

	} /* else */

   txc->txCur = txCur;
   txc->avail -= i;
   txc->qd += i;
   txc->pkts++;
   txc->octets += pktLen;

   return(NFX_OK);

} /* gt64440mii_txUnicast */

/*
 *------------------------------------------------------------------------------
 * Function Name:	gt64440mii_txCleanupQ
 *
 * Description: 	Cleans up descriptors on a transmit queue.
 *
 * Created by:		Tony Wong
 *
 * Data Created:	08-06-2001
 * 					
 * Returns: ?????
 *
 *------------------------------------------------------------------------------
 */
int
gt64440mii_txCleanupQ( uint32 port, gt64440mii_txCtrl_t *txc )
{
   uint32 cmdsts;
   volatile gt64440mii_txdesc_t *ccur;
   volatile gt64440mii_txdesc_t *tcur;
   eee_desc_t *edesc;

   ccur = txc->txCln;
   tcur = txc->txCur;

   cmdsts = ccur->cmdsts;
   while( ccur != tcur && ((cmdsts & TX_O) == 0) ) {
      /*
       * Look for the last descriptor in a chain. The cmdsts word
       * in this descriptor contains the transmit completion
       * status for the packet.
       */
	if( (cmdsts & TX_L) ) {

           edesc = txPktDesc[port].pktDesc[txPktDesc[port].outdex];

	   /* debug 
	   if (edesc != NULL) {
	      printf("bufptr: %x\n", edesc->bd[1].buf);
	      printf("buflen: %x\n", edesc->bd[1].len);
	      printf("ctrl:   %x\n", edesc->hdr.control);
           }
	   debug */
			
#ifndef PROM
		/*
		 * The application shouldn't be dealing with IP packets
		 */
	   eee_freePacket(edesc);
#else
		/*
		 * The prom will deal with IP packets during tftp download
		 */
   		eee_freePacket(edesc);
#endif


           txPktDesc[port].pktDesc[txPktDesc[port].outdex] = NULL;

           if (++txPktDesc[port].outdex >= GT_MII_TX_Q_LEN)
              txPktDesc[port].outdex = 0;
			
			/*
			 * Check Error Summary for TX errors
			 */
		   if( cmdsts & TX_ES )
		   {
				if( (cmdsts & TX_EC) == TX_EC_RL)
					txc->relimErr++;
				else if( (cmdsts & TX_EC) == TX_EC_UR)
					txc->udrErr++;
				else if( (cmdsts & TX_EC) == TX_EC_LC)
					txc->lcolErr++;
		   }
        } /* if last */
        ccur->cmdsts = 0;
        ccur->bufptr = 0;
        txc->avail++;
        txc->unqd++;

        if( ++ccur >= txc->txqEnd )
           ccur = txc->txqStart;

        cmdsts = ccur->cmdsts;
   } /* while */

   (volatile gt64440mii_txdesc_t *)txc->txCln = 
									(volatile gt64440mii_txdesc_t *)ccur;
   return(NFX_OK);

} /* gt64440mii_txCleanupQ */

/*
 *------------------------------------------------------------------------------
 * Function Name:	gt64440mii_txCleanPoll
 *
 * Description: 	Polling function registered with the EEE layer and 
 * 			called to cleanup the transmit queues.
 *
 * Created by:		Tony Wong
 *
 * Data Created:	08-06-2001
 * 					
 * Returns: ?????
 *
 *------------------------------------------------------------------------------
 */
void
gt64440mii_txCleanPoll(void *cb, uint32 tlimit)
{
   int i, port;

   for (port = 0; port < GT_MII_MAX_PORTS; port++)
   {
      for(i=0; i < GT_MII_TX_Q_CNT; i++)
         gt64440mii_txCleanupQ( port, &gt64440mii->port[port].txc[i] );
   }
   return;

} /* gt64440mii_txCleanup */


int
gt64440_printBuf(char *buf, int len, int fDecode)
{
    int i, j;
    char ch;
    char *data;

    data = buf;
/*
    if( fDecode ) {
        gt64440_decodeHdr(data);
    }
*/

    for(i=0; i < len; i+=16) {
        printf("\n%04x:\t", i);
        for( j=i; j < (i+16); j++ )
            printf("%02x ", data[j] & 0xff);
        printf("    ");
        for( j=i; j < (i+16); j++ ) {
            ch = data[j] & 0xff;
            if( (ch > 0x20) && (ch < 0x7F))
                printf("%c", ch);
            else
            if( ch == 0 )
                printf("%c",'.');
            else
                printf("%c",'_');
        }
    }
    printf("\n\n");
    return 0;
} // dp83820_printBuf

int
isBackplane_packet(
	int	port				/* 0, 1, 2 */
)
{
	if (isSSC() && ((port == 1) || (port == 2)))
		return 0;

	if (isFC() && (port == 0))
		return 0;

	return 1;
}

int
isIp_packet(
	eee_desc_t	* pktDesc
)
{
	uchar8	* bptr;

	bptr = pktDesc->bd[1].buf;
	//-- Look at the ethernet type field to see
	//-- if this is an IP packet.
	if( (*(ushort16 *)(bptr+12) == 0x0008) ||
							(*(ushort16 *)(bptr+12) == 0x0608) )
		return 1;
	else
		return 0;
}
#endif
