/*
 *-----------------------------------------------------------------
 *
 * Name:        gt64440mii.h
 *
 * RCS:  $Id: $
 *
 * Copyright (C) 2004, OnStor Inc.
 *
 * Description: Ethernet port register definition file for GT64440. 
 *
 * Created by:  Shaun Hui
 *
 * Date Created: 04/13/04
 *
 *-----------------------------------------------------------------
 */
#ifndef GT64440MII_H_INCLUDED
#define GT64440MII_H_INCLUDED

#include <sys/gt64440r.h>
#include "gt64440mii-regs.h"

#ifdef SIBYTE_BC
#include <sys/sb1250/sb1250_defs.h>

#else

#define NFX_DROP_PACKET		-1

#include <nfx-incl.h>
#include <exports/eee-api.h>
#include <exports/cm-api.h>

#define GT64440MII_BASE_ADDR	(GT_BASE_ADDR + 0x2000)

#if #endian(little)
#define GT64440_MII_BIG_ENDIAN  0
#else
#define GT64440_MII_BIG_ENDIAN  1
#endif

#define GT_MII_TX_SAFEMODE	1
#define GT_MII_RX_SAFEMODE	1

/*
 * PHY 0-2 SMI address
 */
#define GT_MII_PHY0     0x8
#define GT_MII_PHY1     0x9
#define GT_MII_PHY2     0xa

extern char phytab[];

#define GT_MII_GET_PHYID(port)	(phytab[port])

#ifdef PROM
#define GT_MII_TX_Q_CNT	  	1
#define GT_MII_TX_Q_LEN		256

#define GT_MII_RX_Q_CNT	  	1
#define GT_MII_RX_Q_LEN		256
#else
#define GT_MII_TX_Q_CNT	  	1
#define GT_MII_TX_Q_LEN		2048

#define GT_MII_RX_Q_CNT	  	1
#define GT_MII_RX_Q_LEN		2048
#endif

#define GT_MII_RX_BUF_SIZE	2048

#define GT_MII_HASH_TBL_SIZE	16*1024


typedef unsigned long long dword;

/* list of eee descriptors to free after usage */

typedef struct _gt_desc_list {
	uint32		index;
	uint32		outdex;
	eee_desc_t	*pktDesc[GT_MII_TX_Q_LEN];
} gt64440mii_desc_list_t;

/*
 *----------------------------
 * Descriptor Format
 *----------------------------
 */
#ifdef NOTUSED
#if #endian(big) || defined(LINUX_TEST) /* this doesn't match data sheet, but it works */
typedef struct _gt64440mii_txdesc {
	uint32				bytecnt_rsvd;
	uint32 				cmdsts;
	struct _gt64440mii_txdesc	*link;
	uchar8 				*bufptr;
} gt64440mii_txdesc_t;

typedef struct _gt64440mii_rxdesc {
	uint32				bufsize_bytecount;
	uint32 				cmdsts;
	struct _gt64440mii_rxdesc	*link;
	uchar8 				*bufptr;
} gt64440mii_rxdesc_t;
#endif
#endif
#if #endian(little) /* this matches data sheet, but it doesn't work */
#endif 
typedef struct _gt64440mii_txdesc {
	uint32 				cmdsts;
	uint32				bytecnt_rsvd;
	uchar8 				*bufptr;
	struct _gt64440mii_txdesc	*link;
} gt64440mii_txdesc_t;

typedef struct _gt64440mii_rxdesc {
	uint32 				cmdsts;
	uint32				bufsize_bytecount;
	uchar8 				*bufptr;
	struct _gt64440mii_rxdesc	*link;
} gt64440mii_rxdesc_t;

typedef struct _gt64440mii_stats {
	uint32 rxPkts;
	uint32 rxOctets;
	uint32 txPkts;
	uint32 txOctets;
	uint32 rxCrc;
	uint32 rxNoBuf;
	uint32 txQFull;
} gt64440mii_stats_t;

typedef struct _gt64440mii_txCtrl {
	gt64440mii_txdesc_t	*txqStart;
	gt64440mii_txdesc_t	*txqEnd;
	gt64440mii_txdesc_t	*txCur;
	gt64440mii_txdesc_t	*txCln;
	uint32  avail;
	uint32  qd;
	uint32  unqd;
	uint32  pkts;
	uint32  octets;
	uint32	qFull;
	uint32	collErr;
	uint32	relimErr;
	uint32	udrErr;
	uint32	lcolErr;
} gt64440mii_txCtrl_t;

typedef struct _gt64440mii_rxBuf_cache {
	uchar8 *ptr;
	uint32 len;
} gt64440mii_bc_t;

typedef struct _gt64440mii_rxCtrl {
	gt64440mii_rxdesc_t	*rxqStart;
	gt64440mii_rxdesc_t	*rxqEnd;
	gt64440mii_rxdesc_t	*rxCur;
	gt64440mii_bc_t		bc[EEE_MAX_BUFS];
	uint32  first_cmdsts;
	uint32  loopback;
	uint32	bufCnt;
	uint32  avail;
	uint32	pkts;
	uint32	octets;
	uint32	shortErr;
	uint32	longErr;
	uint32	ovrErr;
	uint32	collErr;
	uint32	crcErr;
	uint32 	noBufErr;
	uint32 	noDescErr;
} gt64440mii_rxCtrl_t;

typedef struct _gt64440mii_cfg {
	uint32	appId;
	uint32	baseAddr;
	uint32	phyAddr;
	uint32	loopAddr;/* System address of loopback app */
	uint32	loopAddrBP;
	uint32	ipAppPort;	/* packed port address */
	struct {
		char    macAddr[6];
		uint32	tx_trace;
		uint32	rx_trace;
		gt64440mii_rxdesc_t	*rxQ[GT_MII_RX_Q_CNT];
		gt64440mii_txCtrl_t	txc[GT_MII_TX_Q_CNT];
		gt64440mii_rxCtrl_t	rxc[GT_MII_RX_Q_CNT];
		gt64440mii_stats_t	stats;
		uint32			reg_offset;
		uchar8			*hash_tbl;
	} port[GT_MII_MAX_PORTS];
} gt64440mii_cfg_t;

/*
 * Macros to convert virtual address to phy and back again.
 */
#if defined(EVAL_BOARD) || defined(AGILE_HARDWARE)
#ifndef FC_USE_NEW_MEM_MAP
#define V2PHYS_ADDR_MASK	0xA0000000
#define GT64440MII_2UNCACHED(_x_)	((_x_) |= 0x20000000)
#endif
#else
#define V2PHYS_ADDR_MASK	0x00000000
#define GT64440MII_2UNCACHED(_x_)	((_x_)) 
#endif	/* EVAL_BOARD || AGILE_HARDWARE */

/*
 * External Variable Declarations
 */
extern gt64440mii_cfg_t *gt64440mii;
extern gt64440mii_desc_list_t txPktDesc[GT_MII_MAX_PORTS];

/*
 * Function Prototypes
 */
void gt64440mii_rxPktPoll(void *, uint32);
void gt64440mii_txCleanPoll(void *, uint32);
void gt64440mii_rxAppPkt(eee_desc_t *);
int gt_mii_writePortReg(uint32, uint32, uint32);
int gt_mii_readPortReg(uint32, uint32);
uint32 gt64440mii_swap(uint32);
int gt_mii_writePortReg_noswap(uint32, uint32, uint32);
int gt_mii_readPortReg_noswap(uint32, uint32);
uint32 gt64440mii_noswap(uint32);

void gt_mii_rxDis(uint32 port);
void gt_mii_rxEna(uint32 port);
void gt_mii_txDis(uint32 port);
void gt_mii_txEna(uint32 port);
void gt_mii_portDis(uint32 port);
void gt_mii_portEna(uint32 port);
void gt64440mii_clearStats(uint32);


int32 gt_mii_writeSMI(uint32 phy, uint32 reg, uint32 value, uint32 mask);
int32 gt_mii_readSMI(uint32 phy, uint32 reg, uint32 *value);

#endif /* !SIBYTE_BC */
#endif /* GT64440MII_H_INCLUDED */
