/*
 *-----------------------------------------------------------------
 *
 * Name: ecc.c
 *
 * RCS:  $Id: $
 *
 * Copyright (C) 2004, OnStor Inc.,
 *
 * Description: ECC exception hanlder. Ported from sm-gt64240/ecc.c
 *
 * Created by:	Shaun Hui
 *
 * Date Created: 05/05/04
 *
 *-----------------------------------------------------------------
 */

#include <nfx-incl.h>
#include <exports/hwMap.h>
#include <sys/mips.h>
#include <sys/gt64440r.h>
#include <exports/except-api.h>

#include <sys/r9k-regs.h>
typedef unsigned int u_int32_t;



/*-----------------------------------------------------------------
 * Name: 		ecc_error_exc
 *
 * Description: Dumps ECC error related register values.
 *
 *-----------------------------------------------------------------
 */
void
ecc_error_exc(
	uint32	sr,
	uint32	cause,
	address_t	epc
)
{
	uint32	errdatahigh, errdatalow, eccrx, ecccalc, erraddr, ecccount;

	printf("\nECC Error Exception, SR = 0x%x, EPC = %p\n\n", sr, (void *)epc);

	errdatahigh = GT_READ( SDRAM_ERROR_DATA_HIGH );
	errdatalow  = GT_READ( SDRAM_ERROR_DATA_LOW );
	eccrx       = GT_READ( SDRAM_RECEIVED_ECC );
	ecccalc     = GT_READ( SDRAM_CALCULATED_ECC );
	erraddr     = GT_READ( SDRAM_AND_DEVICE_ERROR_ADDRESS );
	ecccount    = GT_READ( SDRAM_ECC_ERROR_COUNTER );

#if 0
    printf("SDRAM Error Data = 0x%x 0x%x\n", *(volatile uint32 *)0xb4000480, *(volatile uint32 *)0xb4000484);
    printf("SDRAM Error Address = %p\n", erraddr);
    printf("SDRAM ECC Calculated = 0x%x, Received = 0x%x\n", errcalc, errrx);
    printf("SDRAM ECC Control = 0x%x\n", *(volatile uint32 *)0xb4000494);
    printf("SDRAM ECC Counter = 0x%x\n", *(volatile uint32 *)0xb4000498);

    printf("Main Interrupt Cause Low = 0x%x\n", *(volatile uint32 *)0xb4000c18);
    printf("CPU Interrupt Mask Low = 0x%x\n", *(volatile uint32 *)0xb4000c1c);
#endif

	GT_WRITE( SDRAM_ECC_CONTROL, 0 );
	GT_WRITE( SDRAM_AND_DEVICE_ERROR_ADDRESS, 0 );

	panic("ECC EXC, 0x%x, 0x%x, 0x%x, 0x%x, 0x%x, 0x%x\n", 
		errdatahigh, errdatalow, eccrx, ecccalc, erraddr, ecccount);
}

/*-----------------------------------------------------------------
 * Name: 		ecc_exception_init
 *
 * Description: Initialize ECC exception related registers.
 *
 *-----------------------------------------------------------------
 */
void
ecc_exception_init()
{
	uint32	t;

	printf("GT ECC exception handler Initialized\n");

	/* clear exisiting ECC error interrupts */
	GT_WRITE( SDRAM_AND_DEVICE_ERROR_ADDRESS, 0);
	GT_WRITE( DUNIT_INTERRUPT_MASK, 0);

#if 0
	/* enable ECC interrupt */
	sr = _mips_mfc0(C0_SR);
	sr |= (SR_IE | SR_HINT0);		/* Int0 is from galileo */
	_mips_mtc0(C0_SR, sr);
#endif

	GT_WRITE( SDRAM_ECC_ERROR_COUNTER, 0);

	/* Enable single bit, double bit & data path */
	GT_WRITE( DUNIT_INTERRUPT_MASK, DUNIT_INTERRUPT_SBIT_ERR | 
					DUNIT_INTERRUPT_DBIT_ERR | 
					DUNIT_INTERRUPT_DP_ERR); 
	GT_WRITE( SDRAM_ECC_CONTROL, 0x00000200);
	t = GT_READ( SDRAM_AND_DEVICE_ERROR_ADDRESS );

#if 0
	gtint_intr_registerHandler(2, ecc_error_exc);
#endif
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description: stub for command ui to ecc test
 *
 *-----------------------------------------------------------------
 */

int32
do_ecc(
	int32	ac,
	char8 * av[]
)
{
    uint32   errdatahigh, errdatalow, eccrx, ecccalc, erraddr, ecccount, eccctrl;
    uint32   sr, cause;

    if( strncmp(av[1], "start",2) == 0) {
        printf(" -- Work Inprogress \n");

    }
    if( strncmp(av[1], "view",2) == 0) {

        printf(" -- Work Inprogress \n");
        sr = _mips_mfc0(C0_SR);
        cause = _mips_mfc0(C0_CAUSE);

        printf("\nGT ECC Error Info, SR = 0x%x, CAUSE = 0x%x\n\n", sr, cause);

        errdatahigh = GT_READ( SDRAM_ERROR_DATA_HIGH );
        errdatalow  = GT_READ( SDRAM_ERROR_DATA_LOW );
        eccrx       = GT_READ( SDRAM_RECEIVED_ECC );
        ecccalc     = GT_READ( SDRAM_CALCULATED_ECC );
        erraddr     = GT_READ( SDRAM_AND_DEVICE_ERROR_ADDRESS );
        ecccount    = GT_READ( SDRAM_ECC_ERROR_COUNTER );
        eccctrl     = GT_READ( SDRAM_ECC_CONTROL);

        printf("GT SDRAM Error Address       = 0x%08x\n", erraddr);
        printf("GT SDRAM Error Data     High = 0x%08x   Low = 0x%08x\n", errdatahigh, errdatalow);
        printf("GT SDRAM ECC Calculated      = 0x%08x, Received = 0x%x\n", ecccalc, eccrx);
        printf("GT SDRAM ECC Control         = 0x%08x\n", eccctrl);
        printf("GT SDRAM ECC Counter         = 0x%08x\n", ecccount);
        printf("GT SDRAM DBit Err Count CS0  = 0x%08x\n",
                    GT_READ(SDRAM_CS0_DOUBLE_BIT_ERROR_COUNTER));
        printf("GT SDRAM SBit Err Count CS1  = 0x%08x\n",
                    GT_READ(SDRAM_CS1_SINGLE_BIT_ERROR_COUNTER));
        printf("GT SDRAM DBit Err Count CS1  = 0x%08x\n",
                    GT_READ(SDRAM_CS1_DOUBLE_BIT_ERROR_COUNTER));
        printf("GT SDRAM SBit Err Count CS2  = 0x%08x\n",
                    GT_READ(SDRAM_CS2_SINGLE_BIT_ERROR_COUNTER));
        printf("GT SDRAM DBit Err Count CS2  = 0x%08x\n",
                    GT_READ(SDRAM_CS2_DOUBLE_BIT_ERROR_COUNTER));
        printf("GT SDRAM SBit Err Count CS3  = 0x%08x\n",
                    GT_READ(SDRAM_CS3_SINGLE_BIT_ERROR_COUNTER));
        printf("GT SDRAM DBit Err Count CS3  = 0x%08x\n",
                    GT_READ(SDRAM_CS3_DOUBLE_BIT_ERROR_COUNTER));


//        printf("Main Interrupt Cause Low = 0x%x\n", *(volatile uint32 *)0xb4000c18);
//        printf("CPU Interrupt Mask Low = 0x%x\n", *(volatile uint32 *)0xb4000c1c);

        printf("\nR9K ECC Info  \n" );
        printf("XECC: Status / Control              0x%08x\n", RM9K_READ(R9K_XECC));
        printf("XCAUFA0: Extended Cause Fault Grp 0 0x%08x\n", RM9K_READ(R9K_XCAUFA0));
        printf("XCAUFA1: Extended Cause Fault Grp 1 0x%08x\n", RM9K_READ(R9K_XCAUFA1));
        printf("R9K SDRAM Write Err Addr Hi =       0x%08x\n", RM9K_READ(R9K_SDWEAH));
        printf("R9K SDRAM Write Err Addr Lo =       0x%08x\n", RM9K_READ(R9K_SDWEAL));
        printf("R9K SDRAM Error Address  Hi =       0x%08x\n", RM9K_READ(R9K_SDEAH));
        printf("R9K SDRAM Error Address  Lo =       0x%08x\n", RM9K_READ(R9K_SDEAL));
        if ( RM9K_READ(R9K_XECC) & (R9K_XECC_SECC)) {
            printf("R9K ECC Single Bit Error\n");
        }
        if ( RM9K_READ(R9K_XECC) & (R9K_XECC_MECC)) {
            printf("R9K ECC Multi Bit Error\n");
        }

    }
    else {
        printf("not implemented\n");
    }
    return NFX_OK;
}
