/*
 *-----------------------------------------------------------------
 *
 * Name:		led.c
 *
 * RCS:  $Id: $
 *
 * Copyright (C) 2004, OnStor Inc.
 *
 * Description:	Contain routines to set leds. 
 *				Ported from sm-gt64240.
 *
 * Created by:	Shaun Hui
 *
 * Date Created:04/18/04
 *
 *-----------------------------------------------------------------
 */

#include <nfx-incl.h>
#include <sys/mips.h>
#include <sys/gt64440r.h>
#include <exports/hwMap.h>
#include <sys/led-api.h>
#include <sys/bmfpga.h>
#include <exports/cm-api.h>
#include "gpp-defs.h"

#define GPP_LED_FAULT_MASK	0x01800000
#define GPP_LED_FAULT_OFF	(0 << 23)
#define GPP_LED_FAULT_RED	(1 << 23)
#define GPP_LED_FAULT_GREEN	(2 << 23)
#define GPP_LED_FAULT_AMBER	(3 << 23)

#define GPP_LED_OK_MASK		0x00600000
#define GPP_LED_OK_OFF		(0 << 21)
#define GPP_LED_OK_RED		(1 << 21)
#define GPP_LED_OK_GREEN	(2 << 21)
#define GPP_LED_OK_AMBER	(3 << 21)

/*-----------------------------------------------------------------
 * Name:		led_set
 *
 * Description: Set led on/off & color etc.
 *
 *-----------------------------------------------------------------
 */
void
led_set( int	led_state)
{
	volatile uint32	gpp_value;

	gpp_value = GT_READ(GPP_VALUE);

	switch(led_state)
	{
		case	LED_OK_RED:
				gpp_value &= ~GPP_LED_OK_MASK;
				gpp_value |= GPP_LED_OK_RED;
				break;

		case	LED_OK_AMBER:
				gpp_value &= ~GPP_LED_OK_MASK;
				gpp_value |= GPP_LED_OK_AMBER;
				break;

		case	LED_OK_ON:
				gpp_value &= ~GPP_LED_OK_MASK;
				gpp_value |= GPP_LED_OK_GREEN;
				break;

		case	LED_OK_OFF:
				gpp_value &= ~GPP_LED_OK_MASK;
				gpp_value |= GPP_LED_OK_OFF;
				break;

		case	LED_FAULT_GREEN:
				gpp_value &= ~GPP_LED_FAULT_MASK;
				gpp_value |= GPP_LED_FAULT_GREEN;
				break;

		case	LED_FAULT_AMBER:
				gpp_value &= ~GPP_LED_FAULT_MASK;
				gpp_value |= GPP_LED_FAULT_AMBER;
				break;

		case	LED_FAULT_ON:
				gpp_value &= ~GPP_LED_FAULT_MASK;
				gpp_value |= GPP_LED_FAULT_RED;
				break;

		case	LED_FAULT_OFF:
				gpp_value &= ~GPP_LED_FAULT_MASK;
				gpp_value |= GPP_LED_FAULT_OFF;
				break;
	}
	GT_WRITE(GPP_VALUE, gpp_value);
}

