/*
 *-----------------------------------------------------------------------------
 *
 * Name:	hal.h
 *
 * RCS:		$Id: hal.h,v 1.15 2002/11/07 01:17:59 rick Exp $
 *
 * Copyright (C) 2001, Agile Storage Inc.
 *
 * Description:	Host-Adapter-Layer Generic Interface definitions
 *
 * Created by:	Radek Aster
 *
 * Date Created: 5/11/01
 *
 *-----------------------------------------------------------------------------
 */

#ifndef __HAL_H__
#define __HAL_H__

#include <nfx-incl.h>
#include <exports/eee-api.h>
#include <exports/scsi-api.h>
#include <exports/wwn.h>

#if 1				/* We assume 32-bit pointers */
typedef uint32 ps_uint_t;
#endif

#define HAL_ALIGN_TO(_p_, _a_) \
	((void *)(((ps_uint_t)(_p_) + ((_a_)-1)) &~ ((_a_)-1)))

/*
 * SCSI Task management control block
 */
typedef struct scsi_task_mgmt {
	ushort16	hal_handle;			/* HAL device handle */
	byte		lun;				/* Logical Unit, if applicable */
	void *		handle;				/* Upper Layer Protocol Handle */
	void (*callback)(struct scsi_task_mgmt *);	/* Completion callback function */
	uint32		status;				/* Return status code */
	uint32		command;			/* Which TMO to execute */

	struct scsi_task_mgmt *next;
	struct scsi_task_mgmt *prev;

	/* For HAL driver private use */
	void (*priv_callback)();
} scsi_task_mgmt_t;

/*
 * SCSI task management command definition
 */
#define SCSI_TMO_CLEAR_ACA	1
#define SCSI_TMO_TARGET_RESET	2
#define SCSI_TMO_CLEAR_TASKSET	3
#define SCSI_TMO_ABORT_TASKSET	4

/*
 * CT descriptor
 */
#define	CT_ADDRESS_MS		0xFFFFFA
#define	CT_ADDRESS_NS		0xFFFFFC

#define CT_STATUS_MASK		0x00000FFF
#define CT_STATUS_OK		0
#define CT_STATUS_ERR		1
#define CT_STATUS_INVALID_PARM	2
#define CT_STATUS_ILLEGAL_OP	4
#define CT_STATUS_TIMEOUT	5
#define CT_STATUS_RESP_VALID	0x00001000
#define CT_STATUS_RESP_MASK	0xFFFF0000
#define CT_STATUS_RESP_SHIFT	16
#define CT_STATUS_RESP_FS_RJT	0x8001
#define CT_STATUS_RESP_FS_ACC	0x8002

#define CT_DESC_STATE_WAITING	0
#define CT_DESC_STATE_BUSY	1
#define CT_DESC_STATE_TIMED_OUT	2

typedef struct ct_desc {
	void *			handle;		/* Upper Layer Protocol Handle */
	void (*callback)(struct ct_desc *);	/* Completion callback function */
	uint32			server_addr;	/* NS or MS */
	uint32			status;		/* Return status */
	eee_desc_t *		e_desc;		/* 
						 * CT request  -- e_desc->bd[1] 
						 * CT response -- e_desc->bd[2]
						 */
	struct ct_desc *	next;
	struct ct_desc *	prev;

	/* For HAL driver private use */
	byte			state;
	byte			internal;
	int32			timeout_count;
	int32			retry_count;
	eeeRTC_t		first_issued_timestamp;
	void (*priv_callback)(struct ct_desc *);
} ct_desc_t;

typedef ushort16 hal_dev_handle_t;

typedef struct hal_dispatch_entry {
	uint32	hal_valid;			/* This entry valid */
	int	hal_bus_id;
	int	hal_dev_id;
	void *	hal_handle;			/* Pointer to driver specific */
						/* cookie */
	int32	(*hal_scsi_send)		/* SCSI delivery func */
			(scsi_desc_t *);	
	int32	(*hal_scsi_task_mgmt_send)	/* SCSI task mgmt delivery */
			(scsi_task_mgmt_t *);	/* func	*/
	int32	(*hal_ct_send)			/* CT delivery func */
			(ct_desc_t *);
	int32	(*hal_ioctl)			/* IOCTL func */	
			(uint32 ioctl_code, ...);
} hal_dispatch_entry_t;

/*
 * hal_ioctl function definitions : common functions 
 */
#define HAL_IOCTL_COMMON_BASE			(0x1000)
#define HAL_IOCTL_COMMON_DRIVER_VERBOSE_LEVEL	(HAL_IOCTL_COMMON_BASE + 1)
#define HAL_IOCTL_COMMON_SET_DRIVER_OPTIONS	(HAL_IOCTL_COMMON_BASE + 2)
#define HAL_IOCTL_COMMON_GET_INTERFACE_TYPE	(HAL_IOCTL_COMMON_BASE + 3)
#define HAL_IOCTL_COMMON_GET_ADAPTER_INFO	(HAL_IOCTL_COMMON_BASE + 4)
#define HAL_IOCTL_COMMON_RESET_ADAPTER		(HAL_IOCTL_COMMON_BASE + 5)
#define HAL_IOCTL_COMMON_DISABLE_ADAPTER	(HAL_IOCTL_COMMON_BASE + 6)
#define HAL_IOCTL_COMMON_ENABLE_ADAPTER		(HAL_IOCTL_COMMON_BASE + 7)
#define HAL_IOCTL_COMMON_RESET_DEVICE		(HAL_IOCTL_COMMON_BASE + 8)
#define HAL_IOCTL_COMMON_DELAY_DEVICE		(HAL_IOCTL_COMMON_BASE + 9)
#define HAL_IOCTL_COMMON_INVALIDATE_DEVICE	(HAL_IOCTL_COMMON_BASE + 10)
#define HAL_IOCTL_COMMON_GET_DEVICE_MAP		(HAL_IOCTL_COMMON_BASE + 11)
#define HAL_IOCTL_COMMON_DEV_LOOKUP		(HAL_IOCTL_COMMON_BASE + 12)
#define HAL_IOCTL_COMMON_RESCAN			(HAL_IOCTL_COMMON_BASE + 13)
#define HAL_IOCTL_COMMON_KILL_WRITES		(HAL_IOCTL_COMMON_BASE + 14)

/*
 * hal_ioctl function definitions : FC only functions 
 */
#define HAL_IOCTL_FC_BASE			(0x2000)
#define HAL_IOCTL_FC_GET_DEVICE_LINK_STATS	(HAL_IOCTL_FC_BASE + 1)
#define HAL_IOCTL_FC_CLR_DEVICE_LINK_STATS	(HAL_IOCTL_FC_BASE + 2)
#define HAL_IOCTL_FC_GET_ADAPTER_LINK_STATS	(HAL_IOCTL_FC_BASE + 3)
#define HAL_IOCTL_FC_CLR_ADAPTER_LINK_STATS	(HAL_IOCTL_FC_BASE + 4)
#define HAL_IOCTL_FC_LIP			(HAL_IOCTL_FC_BASE + 5)
#define HAL_IOCTL_FC_LOOP_PORT_BYPASS		(HAL_IOCTL_FC_BASE + 6)
#define HAL_IOCTL_FC_LOOP_PORT_ENABLE		(HAL_IOCTL_FC_BASE + 7)
#define HAL_IOCTL_FC_LOOP_PORT_ENABLE_ALL	(HAL_IOCTL_FC_BASE + 8)
#define HAL_IOCTL_FC_CONNECTION_STATE		(HAL_IOCTL_FC_BASE + 9)
#define HAL_IOCTL_FC_TOPOLOGY			(HAL_IOCTL_FC_BASE + 10)

/*
 * IOCTL usage:
 *	hal_ioctl(HAL_IOCTL_COMMON_DRIVER_VERBOSE_LEVEL, uint32 verbose_level);
 *	hal_ioctl(HAL_IOCTL_COMMON_SET_DRIVER_OPTIONS, uint32 driver_options);
 *      hal_ioctl(HAL_IOCTL_COMMON_GET_INTERFACE_TYPE, uint32 *interface_type);
 *      hal_ioctl(HAL_IOCTL_COMMON_GET_ADAPTER_INFO, hal_adapter_info_t *adapter_info);
 *      hal_ioctl(HAL_IOCTL_COMMON_RESET_ADAPTER);
 *      hal_ioctl(HAL_IOCTL_COMMON_DISABLE_ADAPTER);
 *      hal_ioctl(HAL_IOCTL_COMMON_ENABLE_ADAPTER);
 *      hal_ioctl(HAL_IOCTL_COMMON_RESET_ADAPTER);
 *      hal_ioctl(HAL_IOCTL_COMMON_RESET_DEVICE, hal_dev_handle_t hal_dev_handle);
 *      hal_ioctl(HAL_IOCTL_COMMON_DELAY_DEVICE, hal_dev_handle_t hal_dev_handle, uint32 pause_seconds);
 *      hal_ioctl(HAL_IOCTL_COMMON_INVALIDATE_DEVICE, hal_dev_handle_t hal_dev_handle);
 *	hal_ioctl(HAL_IOCTL_COMMON_GET_DEVICE_MAP, wwn_t channel_wwn, hal_device_map_t *device_map);
 *	hal_ioctl(HAL_IOCTL_COMMON_DEV_LOOKUP, hal_dev_handle_t dev_handle, hal_dev_lookup_t *dev_lookup);
 *      hal_ioctl(HAL_IOCTL_COMMON_RESCAN);
 *      hal_ioctl(HAL_IOCTL_COMMON_KILL_WRITES, hal_dev_handle_t hal_dev_handle, uint32 lun);
 *
 *      hal_ioctl(HAL_IOCTL_FC_GET_ADAPTER_LINK_STATS, hal_fc_link_stats_t *adapter_link_stats);
 *      hal_ioctl(HAL_IOCTL_FC_CLR_ADAPTER_LINK_STATS);
 *      hal_ioctl(HAL_IOCTL_FC_GET_DEVICE_LINK_STATS, hal_dev_handle_t hal_dev_handle, hal_fc_link_stats_t *device_link_stats);
 *      hal_ioctl(HAL_IOCTL_FC_CLR_DEVICE_LINK_STATS, hal_dev_handle_t hal_dev_handle);
 *      hal_ioctl(HAL_IOCTL_FC_LIP, wwn_t channel_wwn, byte lip_byte3, byte lip_byte4);
 *      hal_ioctl(HAL_IOCTL_FC_LOOP_PORT_BYPASS, wwn_t dev_wwn);
 *      hal_ioctl(HAL_IOCTL_FC_LOOP_PORT_ENABLE, wwn_t dev_wwn);
 *      hal_ioctl(HAL_IOCTL_FC_LOOP_PORT_ENABLE_ALL, wwn_t channel_wwn);
 *      hal_ioctl(HAL_IOCTL_FC_CONNECTION_STATE, uint32 *connection_state);
 *      hal_ioctl(HAL_IOCTL_FC_TOPOLOGY, void *topology, uint32 *topology_len);
 */

/*
 * Returned by 
 *     hal_ioctl(HAL_IOCTL_COMMON_GET_DEVICE_MAP, hal_device_map_t *);
 * if interface is HAL_IF_FC_[L/NL]_PORT 
 */
typedef struct {
	byte   hostid;
	byte   port_count;
	byte   port_map[126];
} hal_device_map_t;

/*
 * Returned by 
 *     hal_ioctl(HAL_IOCTL_COMMON_DEV_LOOKUP, hal_dev_lookup_t *);
 */
typedef struct {
	uint32			transport_type;	/* SCSI_LLNOTIFY_FC | SCSI_LLNOTIFY_IP */
	hal_dev_handle_t	dev_handle;	/* Dev handle used by lower layer driver */
	wwn_t			dev_WWN;
	wwn_t			channel_WWN;
	char *			channel_Name;
} hal_dev_lookup_t;

/*
 * Returned by 
 *     hal_ioctl(HAL_IOCTL_COMMON_GET_INTERFACE_TYPE, uint32 *);
 */
#define HAL_IF_FC_L_PORT			1	/* FC loop, directly attached */
#define HAL_IF_FC_NL_PORT			2	/* FC loop, switch attached */
#define HAL_IF_FC_N_PORT			3	/* FC point-to-point, switch attached */
#define HAL_IF_FC_F_PORT			4	/* A switch F-port */
#define HAL_IF_FC_FL_PORT			5	/* A switch FL-port */
#define HAL_IF_ISCSI				6	/* iSCSI connection */

/*
 * Returned by 
 *     hal_ioctl(HAL_IOCTL_COMMON_ADAPTER_INFO, hal_adapter_info_t *); 
 */
#define HAL_ADAPTER_INFO_NAME_LEN	32
#define HAL_ADAPTER_INFO_FIRMWARE_LEN	8
#define HAL_ADAPTER_INFO_HARDWARE_LEN	8
typedef struct {
	char   name[HAL_ADAPTER_INFO_NAME_LEN];
	char   firmware_rev[HAL_ADAPTER_INFO_FIRMWARE_LEN];
	char   hardware_rev[HAL_ADAPTER_INFO_HARDWARE_LEN];
	uint32 interface_type;
	uint32 max_tags_per_device;
	union {
		struct {
		} iscsi_data;
		struct {
			uint32	link_speed;
			wwn_t	fc_node_name;
			uint32	fc_port_id;
			uint32	fc_state_change_generation_num;
			uint32	fc_packet_size;	/* in bytes */
			byte	sram_parity;
		} fc_data;
	} u;
} hal_adapter_info_t;

/*
 * Returned by 
 *     hal_ioctl(HAL_IOCTL_FC_GET_[DEVICE|ADAPTER]_LINK_STATS, hal_fc_link_stats_t *);
 */

typedef struct {
	uint32	link_failure;
	uint32  loss_of_sync;
	uint32	loss_of_signal;
	uint32	prim_proto_error;
	uint32	invalid_xmit_words;
	uint32	invalid_crc;

	uint64	frames_out;
	uint64  frames_in;
	uint32	transport_errors;
	uint32	underrun_cnt;

	uint32	read_6_cmd_cnt;
	uint32	read_10_cmd_cnt;
	uint32	write_6_cmd_cnt;
	uint32	write_10_cmd_cnt;
	uint32	other_cmd_cnt;
    uint32  pad;

    uint32  userReadFp;
    uint32  userReadSsc;
    uint32  metaReadFp;
    uint32  metaReadSsc;
    uint32  writeFp;
    uint32  writeSsc;

    uint64  readUserBytes_ssc;
 	uint64  readUserBytes_fp;
 	uint64  writeUserBytes_ssc;
 	uint64  writeUserBytes_fp;
 	uint64  readMetaBytes_ssc;
 	uint64  readMetaBytes_fp;

    eeeRTC_t  totalTimeUserRead;
    eeeRTC_t  totalTimeMetaRead;
    eeeRTC_t  totalTimeUserWrite;
    eeeRTC_t  totalTimeMetaWrite;
    eeeRTC_t  fcTimeUserRead;
    eeeRTC_t  fcTimeMetaRead;
    eeeRTC_t  fcTimeUserWrite;
    eeeRTC_t  fcTimeMetaWrite;
	uint64    count_per_usec;     // RTC conversion factor
    
    uint32  bucketSize1;
    uint32  bucketSize2;
    uint32  bucketSize3;
    uint32  bucketSize4;
    uint32  bucketSize5;
    uint32  bucketSize6;

    scsi_timeTab_t totReadDataRspBuckets;
    scsi_timeTab_t fcReadDataRspBuckets;
    scsi_timeTab_t totWriteRspBuckets;
    scsi_timeTab_t fcWriteRspBuckets;

} hal_fc_link_stats_t;

/*
 * Returned by 
 *     hal_ioctl(HAL_IOCTL_FC_CONNECTION_STATE, uint32 *);
 */
#define HAL_IOCTL_FC_LINK_STATE_LINK_DOWN		1
#define HAL_IOCTL_FC_LINK_STATE_LINK_UP			2
#define HAL_IOCTL_FC_LINK_STATE_LINK_OFFLINE		3

#define HAL_IOCTL_FC_TOPOLOGY_MAX_LEN			1024

hal_dispatch_entry_t *hal_dt_entry_new(void);
hal_dispatch_entry_t *hal_dt_entry_get(void);

#endif /* __HAL_H__ */
