/*
 *-----------------------------------------------------------------------------
 *
 * Name:	wwn.c
 *
 * RCS:		$Id: wwn.c,v 1.2 2001/07/13 00:26:06 raster Exp $
 *
 * Copyright (C) 2001, Agile Storage Inc.
 *
 * Description:	WWN related code
 *
 * Created by:	Radek Aster
 *
 * Date Created: 6/8/01
 *
 *-----------------------------------------------------------------------------
 */

#include "wwn.h"

wwn_t null_wwn = {{ (uint64)0 }};

char *
wwn_to_str (wwn_t *wwn , char *callers_buf)
{
	static int	endianness_known = 0;
	static int	wwn_little_endian = 0;
	static char	static_buf[WWN_STR_LEN];
	char *		buf;
	char *		p;
        int		i;
        uint64		num;

	WWN_SET(&num, wwn);

	if (!endianness_known++) {
		wwn_t temp_wwn;

		temp_wwn.u.wwn = 0x1;
		wwn_little_endian = (temp_wwn.u.octets[0] == 0x1);
	}

	if (callers_buf)
		buf = callers_buf;
	else
		buf = static_buf;

        /* 
	 * Little endian is the order we want since the least significant byte
         * is printed first
         */
        for (i = 0; i < sizeof(uint64); i++) {
                /* Upper half of the byte needs to be printed first */
                if (wwn_little_endian) {
                        buf[i*2] = ((num >> (i * 8 + 4)) & 0xf) + '0';
                        buf[i*2+1] = ((num >> (i * 8)) & 0xf) + '0';
                } else {
                        buf[i*2] = ((num >> ((sizeof(uint64) - i - 1) * 8 + 4)) & 0xf) + '0';
                        buf[i*2+1] = ((num >> ((sizeof(uint64) - i - 1) * 8)) & 0xf) + '0';
                }
                if (buf[i*2] > '9')
                        buf[i*2] = buf[i*2] - '9' - 1 + 'a';
                if (buf[i*2+1] > '9')
                        buf[i*2+1] = buf[i*2+1] - '9' - 1 + 'a';
        }
        buf[i*2] = '\0';

	return(buf);
}
