/*
 *-----------------------------------------------------------------
 *
 * Name:	hash.c
 *
 * RCS:  $Id: hash.c,v 1.4 2002/11/22 23:22:48 maximk Exp $
 *
 * Copyright (C) 2001, Agile Storage Inc.
 *
 * Description:	hash table support functions
 *
 * Created by:	Rick Lund
 *
 * Date Created:	4/3/01
 *
 *-----------------------------------------------------------------
 */

#include <nfx-incl.h>
#include <exports/hash-api.h>
#include "hash.h"
#include <exports/malloc-api.h>

/*-----------------------------------------------------------------
 * Name:	hash_getNode()
 *
 * Description:	Allocate new hash node
 *
 * Created by:	Rick Lund
 *
 * Date Created:	4/3/01
 *
 *-----------------------------------------------------------------
 */
ht_item_t *
hash_getNode()
{
	return ((ht_item_t *)eee_ramAlloc(sizeof(ht_item_t)));

}	/* hash_getNode */

/*-----------------------------------------------------------------
 * Name:	hash_freeNode(np)
 *
 * Description: Deallocates node
 *
 * Created by:	Rick Lund
 *
 * Date Created:	4/3/01
 *
 *-----------------------------------------------------------------
 */
void
hash_freeNode(
	ht_item_t	* np
)
{
	eee_ramDealloc(np);

}	/* hash_freeNode */

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:
 *
 *-----------------------------------------------------------------
 */
void
hash_display(
	address_t	htp
)
{
	uint32	i;
	ht_item_t	* np;

	printf("ht at %p\n", (void *)htp);
	printf("  ht array at %p(%p)\n", ((ht_t *)htp)->head,
												&(((ht_t *)htp)->head[0]));
	NFX_SPIN_LOCK(&((ht_t *)htp)->ht_spin);
	for (i=0; i< ((ht_t *)htp)->num_buckets; i++) {
		printf("%p  ", ((ht_t *)htp)->head[i]);
		for (np = ((ht_t *)htp)->head[i]; np != NULL; np = np->next) {
			printf("%p  ", np->data);
		}
		printf("\n");
	}
	NFX_SPIN_UNLOCK(&((ht_t *)htp)->ht_spin);

}	/* hash_display */

