/*
 *-----------------------------------------------------------------
 *
 * Name:
 *
 * RCS:  $Id: hwMap.h,v 1.15 2002/10/17 00:24:30 rick Exp $
 *
 * Copyright (C) 2001, Agile Storage, Inc.
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	2/1/02
 *
 *-----------------------------------------------------------------
 */

#ifndef _HWMAP_H
#define _HWMAP_H

/*------------------------------------------------------------------*
 *
 *	LINUX_TEST definitions
 *
 *------------------------------------------------------------------*/

#if defined(LINUX_TEST)

#define HW_GALILEO_BASE		0


/*------------------------------------------------------------------*
 *
 *	AGILE HARDWARE definitions
 *
 *------------------------------------------------------------------*/

#elif defined(AGILE_HARDWARE) || defined(_KERNEL)

#define HW_MAX_PROM_NUM     2

#if defined (FEATURE_AUTOLOAD_RUNTIME) && (defined (SIBYTE_BC) || defined (R9000))
#define AUTOLOAD_FLAG_SIGNATURE	0x1a2b3c4d
#endif

#if defined(PROM_R9000)
/*  Shaun 05/07/04
 *  Memory mapping for Bobcat HW.
 */
                                                                                
#define IO_OFFSET               0x00000000
#define PHYS_TO_IOSPACE(x)      (PHYS_TO_K1(x) + IO_OFFSET)
                                                                                
/* 
 * Offset for Exception vector on CPU 0/1
 */
#define CPU0_EXCEPT_OFFSET		0x0400
#define CPU1_EXCEPT_OFFSET		0x0800


/* 
 * Physical Address Memory Map
 */
#define HW_RAM_BASE         0
#define HW_R9K_BASE			0x10000000		/* R9K OCD reg base */
#define HW_CF_CTRL_IO_BASE	0x18000000      /* Compact flash ctrl IO. base */
#define HW_CF_CTRL_MEM_BASE	0x18800000      /* Compact flash ctrl mem. base */
#define HW_DP83816_0_BASE  	0x19000000      /* SSC 10/100 Ethernet port 0*/
#define HW_DP83816_1_BASE  	0x19800000      /* SSC 10/100 Ethernet port 1*/
#define HW_TXRX_SHMEM_BASE  0x1a000000      /* TX RX Shared Mem. base */
#define HW_FP_SHMEM_BASE  	0x1b000000      /* FP Shared Mem. base */
#define HW_GALILEO_BASE     0x1c000000      /* GT 64440 register base */
#define HW_OC_SRAM_BASE     0x1c040000      /* GT 64440 sram base */
#define HW_QLOGIC_0_BASE    0x1d000000      /* Port 0 QLogic register base */
#define HW_QLOGIC_1_BASE    0x1d001000      /* Port 1 QLogic register base */
#define HW_TXRX_MBOX0_BASE  0x1e000000      /* TX RX Mailbox 0 base */
#define HW_TXRX_MBOX1_BASE  0x1e010000      /* TX RX Mailbox 1 base */
#define HW_FP_MBOX0_BASE    0x1e020000      /* FP Mailbox 0 base */
#define HW_FP_MBOX1_BASE    0x1e030000      /* FP Mailbox 1 base */
#define HW_RTC_BASE         0x1f000000      /* Real Time clock */
#define HW_RESERVED_BASE    0x1f200000      /* Reserved */
#define HW_NS16550_BASE     0x1f200000      /* NS16550 UART base */
#define HW_BMFPGA_NMI_CLEAR 0x1f200000      /* CS 1 */
#define HW_BMFPGA_BASE      0x1f400000      /* BM-FPGA */
#define HW_BOOTROM_BASE 	0x1f800000	    /* BOOT PROM */
#define HW_BIG_BOOTROM_BASE HW_BOOTROM_BASE /* Non-boot PROM */
#define HW_PROM_BASE        0x1fc00000
#define HW_LOCAL_MEM_BASE   0x20000000		/* Mem. attached to R9K */
#define HW_GT_MEM_0_BASE	0x40000000		/* Mem. attached to GT */
#define HW_GT_MEM_1_BASE	0x60000000		/* Mem. attached to GT */

/*
 * Virtual  Address Memory Map
 */
#define RM9K_BASE_ADDR      	PHYS_TO_K1(HW_R9K_BASE)
#define RTC_BASE                PHYS_TO_K1(HW_RTC_BASE)
#define NS16550_BASE            PHYS_TO_K1(HW_NS16550_BASE)
#define BMFPGA_BASE             PHYS_TO_K1(HW_BMFPGA_BASE)
#define NON_BOOTROM_BASE        PHYS_TO_K1(HW_BIG_BOOTROM_BASE)
#define BOOT_BASE               PHYS_TO_K1(HW_BOOTROM_BASE)
#define OC_SRAM_BASE            PHYS_TO_K1(HW_OC_SRAM_BASE)
#define DP83816_0_BASE          PHYS_TO_K1(HW_DP83816_0_BASE)
#define DP83816_1_BASE          PHYS_TO_K1(HW_DP83816_1_BASE)
#define SW_RAM_BASE_CACHED		HW_GT_MEM_0_BASE
#define SW_RAM_BASE_UNCACHED	(HW_GT_MEM_0_BASE +  0x80000000)
#define LOCAL_MEM_BASE			(HW_LOCAL_MEM_BASE - 0x10000000)


#define MBOX_SIZE               0x00200000		/*   2 MB */
#define RTC_SIZE                0x00200000		/*   2 MB */
#define RESERVED_SIZE           0x00200000		/*   2 MB */
#define NS16550_SIZE            0x00200000		/*   2 MB */
#define BMFPGA_SIZE             0x00200000		/*   2 MB */
#define NON_BOOT_PROM_SIZE      0x00400000		/*   4 MB */
#define BOOT_SIZE               0x00400000		/*   4 MB */
#define OC_SRAM_SIZE            0x00040000		/* 256 KB */
#define GT_MEM_0_SIZE           0x20000000		/* 512 MB */
#define GT_MEM_1_SIZE           0x20000000		/* 512 MB */
#define QLOGIC_MEM_SIZE         0x01000000		/*  16 MB */
#define CF_CTRL_IO_SIZE         0x00800000		/*   8 MB */
#define CF_CTRL_MEM_SIZE        0x00800000		/*   8 MB */
#define TXRX_SHMEM_SIZE         0x02000000		/*  32 MB */
#define DP83816_SIZE            0x01000000		/*  16 MB */
#define LOCAL_MEM_SIZE			0x10000000		/* 256 MB */
#define SW_RAM_SIZE				0x40000000		/*   1 GB */
                                                                                
/*
 *  PCIX Bus & Device IDs
 */
#define PCIX_0_PRI_BUS		0xFF			/* PCIX 0 primary bus id */
#define PCIX_0_GT_DEV		0x1F			/* GT64440 on PCIX 0 pri bus*/

/*
 *  PCI Bus IDs
 */
#define PCI0_PRI_BUS		0x00			/* PCI0 primary bus id */
#define PCI1_PRI_BUS		0x08			/* PCI1 primary bus id */
#define PCI1_SEC_BUS		0x09			/* PCI1 secondary bus id */

/*
 *  PCI Device IDs 
 */
#define PCI0_QLOGIC_DEV		0x00			/* QLogic on PCI0 pri bus*/
#define PCI0_GT_DEV			0x01			/* GT64440 on PCI0 pri bus*/

#define PCI1_TXRX_DEV		0x00			/* TXRX on PCI1 pri bus*/
#define PCI1_FP_DEV			0x01			/* FP on PCI1 pri bus*/
#define PCI1_BR_DEV			0x02			/* PCI Bridge on PCI1 pri bus*/
#define PCI1_GT_DEV			0x03			/* GT64440 on PCI1 pri bus*/

#define PCI1_DP0_DEV		0x00			/* 1st DP83816 on PCI1 2nd Bus*/
#define PCI1_DP1_DEV		0x01			/* 2nd DP83816 on PCI1 2nd Bus*/
#define PCI1_CF_DEV			0x02			/* CF Controller on PCI1 2nd Bus*/

/*
 *  PCI Bus allocation
 */
#define PCI0_MEM_SPACE_BASE     0x1d000000
#define PCI0_MEM_SPACE_SIZE     0x01000000		/* 16 MB */
#define PCI0_IO_SPACE_BASE      0xe0000000
#define PCI0_IO_SPACE_SIZE      0x08000000
                                                                                
#define PCI1_MEM_SPACE_BASE     0xd0000000
#define PCI1_MEM_SPACE_SIZE     0x10000000
#define PCI1_IO_SPACE_BASE      0xe8000000
#define PCI1_IO_SPACE_SIZE      0x08000000
                                                                                

#define PCI2_MEM_SPACE_BASE     0x18000000
#define PCI2_MEM_SPACE_SIZE     0x01000000
#define PCI2_IO_SPACE_BASE      0x19000000
#define PCI2_IO_SPACE_SIZE      0x01000000
                                                                                
#define ISA_IO_BASE             PCI_IO_BASE
                                                                                
/*
 *  Address for I2C Devices
 */
#define I2C_ADDR_FAN_CTRL_0     0x96        /* FAN Controller for fans 1-3 */
#define I2C_ADDR_FAN_CTRL_1     0x90        /* FAN Controller for fans 4-6 */
#define I2C_ADDR_TEMP_SENSOR    0x9A        /* Temperature sensor */
#define I2C_ADDR_PMC_MEM_SEEP   0xA8        /* PMC (R9K) Memory SEEP */
#define I2C_ADDR_GT_MEM_SEEP    0xAA        /* GT64440 Memory SEEP */
                                                                                

/*
 *  Device module discouart defs.
 */
#define GTSYSCLK_100MHZ        100000000       /* 100 MHz Sysclock */
#define GTSYSCLK_133MHZ        133000000       /* 133 MHz Sysclock */
                                                                                
#define RM9K_WRITE(ofs, data) \
   *(volatile u_int32_t *)(RM9K_BASE_ADDR+ofs) = data
#define RM9K_READ(ofs) \
   *(volatile u_int32_t *)(RM9K_BASE_ADDR+ofs)
                                                                                
#define BMFPGA_WRITE(offset, data) \
                *(volatile uint32 *)(BMFPGA_BASE+offset) = data
                                                                                
#define BMFPGA_READ(offset) \
                *(volatile uint32 *)(BMFPGA_BASE+offset)

#define SETLEDS(a,b,c,d)

#elif defined(NFX_MOD_FCNIM) || (defined(PROM) && defined(R7000)) || defined(_KERNEL)

#define HW_RAM_BASE         0

#define HW_GALILEO_BASE     0x14000000      /* from 0x14 */

#define HW_PROM_BASE        0x1fc00000

#define HW_RTC_BASE         0x1f000000      /* CS 0 */
#define HW_BMFPGA_NMI_CLEAR 0x1f200000      /* CS 1 */
#define HW_BMFPGA_BASE      0x1f400000      /* CS 2 */

#ifdef FC_USE_NEW_MEM_MAP
#define SW_PCI_BASE				0x30000000
#define SW_RAM_BASE_CACHED		0x40000000
#define SW_RAM_BASE_UNCACHED	0x60000000
#endif

#define HW_BIG_BOOTROM_BASE	0x1f600000		/* CS 3 */

#define SETLEDS(a,b,c,d)

#endif	/* NFX_MOD_FCNIM */

/*------------------------------------------------------------------*/

#if defined(NFX_MOD_NFP) || (defined(PROM) && defined(SIBYTE))

#define HW_RAM_BASE 0

#define HW_LSI_MAC_REGS_BASE	0x1e000000		/* 8 MB */
#define HW_BIG_BOOTROM_BASE		0x1ec00000		/* 4 MB */
#define HW_LUC_REGS_BASE		0x1f000000		/* 1 MB */
#define HW_XILINX_PROGRAM_BASE	0x1f200000		/* 1 MB */
#define HW_BMFPGA_BASE			0x1f400000		/* 1 MB */
#define HW_PROM_BASE			0x1fc00000		/* 1 MB */

#ifdef PROM_SIBYTE_BC	
#define HW_GALILEO_BASE     0x1c000000      /* GT 64440 register base */
#define HW_SB1250_PCISPACE_UPPER	0xf800000000   /* Defined in specs */	
#define GT_BASE_ADDR_FROM_TXRX  PHYS_TO_XKSEG_UNCACHED(HW_SB1250_PCISPACE_UPPER|HW_GALILEO_BASE)	
#define HW_GT_MEM_0_BASE	0x40000000		/* Mem. attached to GT */
#define HW_GT_MEM_0_BASE_FROM_TXRX  PHYS_TO_XKSEG_UNCACHED(HW_SB1250_PCISPACE_UPPER|HW_GT_MEM_0_BASE)	
#define HW_OC_SRAM_BASE     0x1c040000      /* GT 64440 sram base */
#define HW_OC_SRAM_BASE_FROM_TXRX  PHYS_TO_XKSEG_UNCACHED(HW_SB1250_PCISPACE_UPPER|HW_OC_SRAM_BASE)	
#endif	

#define HW_PMC8354_ADDR	    0x10	
#define LMUX_VALID_REG_MAX	0x84

#include <sys/mips.h>

#ifdef NFP_TXRX
#define LOCAL_SHARED_MEMORY_VIRT_BASE   0x20000000
#define LOCAL_SHARED_MEMORY_VIRT_END    0x24000000
#define LOCAL_SHARED_MEMORY_SIZE        0x04000000
#define LOCAL_SHARED_MEMORY_PHYS_BASE   0x08000000
#define LOCAL_SHARED_MEMORY_PHYS_END    0x0C000000

#define REMOTE_SHARED_MEMORY_VIRT_BASE  0x40000000
#define REMOTE_SHARED_MEMORY_VIRT_END   0x47000000
#define REMOTE_SHARED_MEMORY_SIZE       0x07000000
#define REMOTE_SHARED_MEMORY_PHYS_BASE  0x000000E008000000
#define REMOTE_SHARED_MEMORY_PHYS_END   0x000000E00F000000
#endif

#ifdef NFP_FP
#define LOCAL_SHARED_MEMORY_VIRT_BASE   0x40000000
#define LOCAL_SHARED_MEMORY_VIRT_END    0x47000000
#define LOCAL_SHARED_MEMORY_SIZE        0x07000000
#define LOCAL_SHARED_MEMORY_PHYS_BASE   0x08000000
#define LOCAL_SHARED_MEMORY_PHYS_END    0x0F000000

#define REMOTE_SHARED_MEMORY_VIRT_BASE  0x20000000
#define REMOTE_SHARED_MEMORY_VIRT_END   0x24000000
#define REMOTE_SHARED_MEMORY_SIZE       0x04000000
#define REMOTE_SHARED_MEMORY_PHYS_BASE  0x000000E008000000
#define REMOTE_SHARED_MEMORY_PHYS_END   0x000000E00C000000
#endif

#ifndef PROM

#ifdef LARGE_MEMORY
#define VIRT_TO_PHYS(_v)	virt_to_phys(_v)
#define PHYS_TO_VIRT(_p)	phys_to_virt(_p)
#else
#define VIRT_TO_PHYS(_v)                                                    \
({                                                                          \
    uint64  _p;                                                             \
    uint64  _t;                                                             \
                                                                            \
    _t = _v & 0x000000FFFFFFFFFFll;                                         \
                                                                            \
    if (((_t) >= LOCAL_SHARED_MEMORY_VIRT_BASE) &&                          \
                                    ((_t) < LOCAL_SHARED_MEMORY_VIRT_END))  \
        _p = (_t) - LOCAL_SHARED_MEMORY_VIRT_BASE +                         \
                                    LOCAL_SHARED_MEMORY_PHYS_BASE;          \
    else                                                                    \
    if (((_t) >= REMOTE_SHARED_MEMORY_VIRT_BASE) &&                         \
                                    ((_t) < REMOTE_SHARED_MEMORY_VIRT_END)) \
        _p = (_t) - REMOTE_SHARED_MEMORY_VIRT_BASE +                        \
                                    REMOTE_SHARED_MEMORY_PHYS_BASE;         \
    else                                                                    \
    if ((_t >= 0x000000FF80000000ll) && (_t < 0x000000FFA0000000ll))        \
        _p = _t - 0x000000FF80000000ll;                                     \
    else {                                                                  \
        panic("Invalid _t = 0x%llx\n", (_t)); _p = 0x0;                     \
    }                                                                       \
    _p;                                                                     \
})

#define PHYS_TO_VIRT(_p)                                                    \
({                                                                          \
    address_t  _v;                                                          \
                                                                            \
    if (((_p) >= LOCAL_SHARED_MEMORY_PHYS_BASE) &&                          \
                                    ((_p) < LOCAL_SHARED_MEMORY_PHYS_END))  \
        _v = (_p) - LOCAL_SHARED_MEMORY_PHYS_BASE +                         \
                                    LOCAL_SHARED_MEMORY_VIRT_BASE;          \
    else                                                                    \
    if (((_p) >= REMOTE_SHARED_MEMORY_PHYS_BASE) &&                         \
                                    ((_p) < REMOTE_SHARED_MEMORY_PHYS_END)) \
        _v = (_p) - REMOTE_SHARED_MEMORY_PHYS_BASE +                        \
                                    REMOTE_SHARED_MEMORY_VIRT_BASE;         \
    else                                                                    \
    if (((_p) >= 0) && ((_p) < 0x1FFFFFFF))                                 \
        _v = (_p) | K0BASE;                                                 \
    else                                                                    \
        panic("Invalid _p = 0x%llx\n", (_p));                               \
    _v;                                                                     \
})
#endif

#else

#define VIRT_TO_PHYS(_v)                    \
({                                          \
    uint64  _p;                             \
                                            \
    _p = (_v) & 0x000000001FFFFFFFll;       \
    _p;                                     \
})

#define PHYS_TO_VIRT(_p)                    \
({                                          \
    address_t  _v;                          \
                                            \
    _v = (_p) | K0BASE;                     \
    _v;                                     \
})

#endif

#define SETLEDS(a,b,c,d)

#define NFP_TXRX_GPIO_DIR               0xcc00
#define NFP_FP_GPIO_DIR                 0xff00

#ifdef LANGUAGE_C
#define NFP_SYSTEM_CFG_MASK             0x00000000fc000000ll
#else
#define NFP_SYSTEM_CFG_MASK             0x00000000fc000000
#endif
#define NFP_SYSTEM_CFG_SHIFT            26
#define NFP_SYSTEM_CFG_CHEETAH_SHIFT    27
#define NFP_SYSTEM_CFG_CHEETAH          1
#define NFP_SYSTEM_CFG_FP_SHIFT         28
#define NFP_SYSTEM_CFG_FP               1
#define NFP_SYSTEM_CFG_TXRX             3

#endif	/* NFX_MOD_NFP */

/*------------------------------------------------------------------*/

#if defined(SSC_MGMT) || defined(PROM_SSC_MGMT)

#define HW_PROM_BASE        0x1fc00000
#define HW_BIG_BOOTROM_BASE		(-1)		/* none, place holder*/
/*
 * this is tlb mapped to 0xa4000000
 */
#define HW_GALILEO_BASE     0x24000000

#define HW_BMFPGA_BASE      0x20000000      /* CS 2 */

#endif	/* SSC_MGMT */

/*------------------------------------------------------------------*/
#ifdef NFP_FP
#define isTXRX()	0
#define isFP()		1
#elif defined(NFP_TXRX)
#define isTXRX()	1
#define isFP()		0
#endif
/*------------------------------------------------------------------*
 *
 *	EVAL_BOARD definitions
 *
 *------------------------------------------------------------------*/

#elif defined(EVAL_BOARD)

#define HW_MAX_PROM_NUM     1

#if defined(NFX_MOD_FCNIM) || (defined(PROM) && defined(R7000) && !defined(R9000))

#define HW_RAM_BASE         0

#define HW_GALILEO_BASE     0x14000000      /* from 0x14 */

#define HW_ST16552_0_BASE   0x1d000000
#define HW_ST16552_1_BASE   0x1d000020

#define HW_PROM_BASE        0x1fc00000
#define HW_BIG_BOOTROM_BASE		(-1)		/* none, place holder*/

#define SETLEDS(a,b,c,d)

#endif	/* R7000 */

/*------------------------------------------------------------------*/

#if defined(NFX_MOD_NFP) || (defined(PROM) && defined(SIBYTE))

#define HW_RAM_BASE 0
#define HW_PROM_BASE    0x1fc00000
#define HW_BIG_BOOTROM_BASE		(-1)		/* none, place holder*/


#ifdef NFP_TXRX
#define LOCAL_SHARED_MEMORY_VIRT_BASE   0x20000000
#define LOCAL_SHARED_MEMORY_VIRT_END    0x24000000
#define LOCAL_SHARED_MEMORY_SIZE        0x04000000
#define LOCAL_SHARED_MEMORY_PHYS_BASE   0x08000000
#define LOCAL_SHARED_MEMORY_PHYS_END    0x0C000000

#define REMOTE_SHARED_MEMORY_VIRT_BASE  0x40000000
#define REMOTE_SHARED_MEMORY_VIRT_END   0x47000000
#define REMOTE_SHARED_MEMORY_SIZE       0x07000000
#define REMOTE_SHARED_MEMORY_PHYS_BASE  0x000000E008000000
#define REMOTE_SHARED_MEMORY_PHYS_END   0x000000E00F000000
#endif

#ifdef NFP_FP
#define LOCAL_SHARED_MEMORY_VIRT_BASE   0x40000000
#define LOCAL_SHARED_MEMORY_VIRT_END    0x47000000
#define LOCAL_SHARED_MEMORY_SIZE        0x07000000
#define LOCAL_SHARED_MEMORY_PHYS_BASE   0x08000000
#define LOCAL_SHARED_MEMORY_PHYS_END    0x0F000000

#define REMOTE_SHARED_MEMORY_VIRT_BASE  0x20000000
#define REMOTE_SHARED_MEMORY_VIRT_END   0x24000000
#define REMOTE_SHARED_MEMORY_SIZE       0x04000000
#define REMOTE_SHARED_MEMORY_PHYS_BASE  0x000000E008000000
#define REMOTE_SHARED_MEMORY_PHYS_END   0x000000E00C000000
#endif

#ifndef PROM
#define VIRT_TO_PHYS(_v)                                                    \
({                                                                          \
    uint64  _p;                                                             \
    uint64  _t;                                                             \
                                                                            \
    _t = _v & 0x000000FFFFFFFFFFll;                                         \
                                                                            \
    if (((_t) >= LOCAL_SHARED_MEMORY_VIRT_BASE) &&                          \
                                    ((_t) < LOCAL_SHARED_MEMORY_VIRT_END))  \
        _p = (_t) - LOCAL_SHARED_MEMORY_VIRT_BASE +                         \
                                    LOCAL_SHARED_MEMORY_PHYS_BASE;          \
    else                                                                    \
    if (((_t) >= REMOTE_SHARED_MEMORY_VIRT_BASE) &&                         \
                                    ((_t) < REMOTE_SHARED_MEMORY_VIRT_END)) \
        _p = (_t) - REMOTE_SHARED_MEMORY_VIRT_BASE +                        \
                                    REMOTE_SHARED_MEMORY_PHYS_BASE;         \
    else                                                                    \
        panic("Invalid _t = 0x%llx\n", (_t));                               \
    _p;                                                                     \
})

#define PHYS_TO_VIRT(_p)                                                    \
({                                                                          \
    uint32  _v;                                                             \
                                                                            \
    if (((_p) >= LOCAL_SHARED_MEMORY_PHYS_BASE) &&                          \
                                    ((_p) < LOCAL_SHARED_MEMORY_PHYS_END))  \
        _v = (_p) - LOCAL_SHARED_MEMORY_PHYS_BASE +                         \
                                    LOCAL_SHARED_MEMORY_VIRT_BASE;          \
    else                                                                    \
    if (((_p) >= REMOTE_SHARED_MEMORY_PHYS_BASE) &&                         \
                                    ((_p) < REMOTE_SHARED_MEMORY_PHYS_END)) \
        _v = (_p) - REMOTE_SHARED_MEMORY_PHYS_BASE +                        \
                                    REMOTE_SHARED_MEMORY_VIRT_BASE;         \
    else                                                                    \
        panic("Invalid _p = 0x%llx\n", (_p));                               \
    _v;                                                                     \
})

#else

#define VIRT_TO_PHYS(_v)                    \
({                                          \
    uint64  _p;                             \
                                            \
    _p = (_v) & 0x000000001FFFFFFFll;       \
    _p;                                     \
})

#define PHYS_TO_VIRT(_p)                    \
({                                          \
    uint32  _v;                             \
                                            \
    _v = (_p) | 0x80000000;                 \
    _v;                                     \
})

#endif

#define SETLEDS(a,b,c,d)                     \
       li     a0,(((a)<<24)|((b)<<16)|((c)<<8)|(d)) ;    \
       jal    board_setleds ;


#endif	/* NFX_MOD_NFP */

#if defined(PROM_R9000)
/*  
 *  Memory mapping for Jaguar-ATX (Disco 2) eval board.
 */
                                                                                
#define IO_OFFSET               0x40000000
#define PHYS_TO_IOSPACE(x)      (PHYS_TO_K1(x) + IO_OFFSET)
                                                                                
/*
 *  CS space mapping.
 */
#define FPGA_BASE               PHYS_TO_IOSPACE(0x1c000000)
#define FPGA_SIZE               0x00800000
#define RTC_BASE                PHYS_TO_IOSPACE(0x1c800000)
#define RTC_SIZE                0x00800000
#define UART_BASE               PHYS_TO_IOSPACE(0x1d000000)
#define UART_SIZE               0x01000000
#define FLASH_BASE              PHYS_TO_IOSPACE(0x1f000000)
#define FLASH_SIZE              0x00c00000
#define BOOT_BASE               PHYS_TO_IOSPACE(0x1fc00000)
#define BOOT_SIZE               0x00400000
                                                                                
/*
 * Values taken from R7000 to make compilation work
 */
#define HW_RAM_BASE         0
#define HW_GALILEO_BASE     (0x14000000+IO_OFFSET)      /* from 0x14 */
#define HW_PROM_BASE        0x1fc00000
#define HW_RTC_BASE         0x1f000000      /* CS 0 */
#define HW_BMFPGA_NMI_CLEAR 0x1f200000      /* CS 1 */
#define HW_BMFPGA_BASE      0x1f400000      /* CS 2 */
#define HW_BIG_BOOTROM_BASE 0x1f600000      /* CS 3 */
                                                                                
/*
 * Internal SRAM mapping
 */
                                                                                
#define OC_SRAM_BASE            PHYS_TO_IOSPACE(0x1e000000)
#define OC_SRAM_SIZE            0x00040000
                                                                                
/*
 *  PCI Bus allocation
 */
#define PCI0_MEM_SPACE_BASE     0xc0000000
#define PCI0_MEM_SPACE_SIZE     0x10000000
#define PCI0_IO_SPACE_BASE      0xe0000000
#define PCI0_IO_SPACE_SIZE      0x08000000
                                                                                
#define PCI1_MEM_SPACE_BASE     0xd0000000
#define PCI1_MEM_SPACE_SIZE     0x10000000
#define PCI1_IO_SPACE_BASE      0xe8000000
#define PCI1_IO_SPACE_SIZE      0x08000000
                                                                                
                                                                                
#define PCI2_MEM_SPACE_BASE     0x18000000
#define PCI2_MEM_SPACE_SIZE     0x01000000
#define PCI2_IO_SPACE_BASE      0x19000000
#define PCI2_IO_SPACE_SIZE      0x01000000
                                                                                
#define ISA_IO_BASE             PCI_IO_BASE
                                                                                
/*
 *  Timekeeper/NVRAM (M48T37)
 */
#define NVRAM_BASE_ADDR         RTC_BASE
#define NVRAM_SIZE              (0x7ff0)
#define NVRAM_BASE              NVRAM_BASE_ADDR
#define NVRAM_FIRST_ENV         (NVRAM_BASE + 0x0000)
#define NVRAM_LAST_ENV          (NVRAM_BASE + 0x77ff)
#define NVRAM_SIZE_ENV          (NVRAM_LAST_ENV - NVRAM_FIRST_ENV + 1)
#define NVRAM_VXWORKS           (NVRAM_BASE + NVRAM_SIZE - 512)
#define NVRAM_VXWORKS_DEFAULT \
"dc(0,0)host:/usr/vw/config/ev64340/vxWorks h=90.0.0.3 e=90.0.0.50 u=target"
#define NVRAM_ENETADDR          (NVRAM_BASE + 0x7cf2)
                                                                                
#define M48T37_BASE             (NVRAM_BASE_ADDR + NVRAM_SIZE)
                                                                                
/*
 *  Device module duart I/O ports.
 */
#define COM1_BASE_ADDR  (UART_BASE + 0x20)      /* Com 1 */
#define COM2_BASE_ADDR  (UART_BASE + 0x00)      /* Com 2 */
                                                                                
#define NS16550HZ       20000000
                                                                                
/*
 *  Device module discouart defs.
 */
#define GTSYSCLK        100000000       /* Sysclock HZ */
                                                                                
#define RM9K_BASE_ADDR          PHYS_TO_K1(0x1b000000)
#define RM9K_WRITE(ofs, data) \
   *(volatile u_int32_t *)(RM9K_BASE_ADDR+ofs) = data
#define RM9K_READ(ofs) \
   *(volatile u_int32_t *)(RM9K_BASE_ADDR+ofs)
                                                                                
/*
 *  Define how to test for non-dmaable address range.
 */
                                                                                
#define IS_DMA_ABLE(addr) \
        (addr >= 0x90000000)
                                                                                
#define SETLEDS(a,b,c,d)
                                                                                
#endif /* PROM_R9000 */

#endif	/* EVAL_BOARD */

#endif	/* _HWMAP_H */

