/*
 *-----------------------------------------------------------------
 *
 * Name:	i2c.c
 *
 * RCS:  $Id: i2c.c,v 1.7 2002/11/07 01:17:17 rick Exp $
 *
 * Copyright (C) 2001, Agile Storage, Inc.
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	10/30/01
 *
 *-----------------------------------------------------------------
 */

#include <nfx-incl.h>
#include <sys/mips.h>
#include <sys/sb1250/sb1250_regs.h>
#include <sys/sb1250/sb1250_smbus.h>

#include <exports/hwMap.h>
#include <exports/i2c-api.h>

#ifdef SIBYTE_BC
#include <sys/gt64440r.h>
#endif

typedef unsigned long long uint64_t;
typedef unsigned long sbport_t;

#define CFG_DRAM_SMBUS_CHANNEL	0

int sb1250_smbus_readbyte(sbport_t base,unsigned int dev,unsigned int offset);
int sb1250_smbus_writebyte(sbport_t base,unsigned int dev,unsigned int offset,char data);

#if defined (SIBYTE) && !defined (SIBYTE_BC)
uchar8	* i2c_reg_string[] = {
	"SLAVE ADDR", 
	"DATA",
	"CONTROL",
	"STATUS",
	"EXT SLAVE ADDR",
	"unused",
	"unused",
	"SOFT RESET"
};
#endif

/*-----------------------------------------------------------------
 * Name:	i2c()
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	10/30/01
 *
 *-----------------------------------------------------------------
 */
int32
sb1250_i2c(
	uint32	op,
	uchar8	bus,
	uchar8	i2c_addr,
	uchar8	byte_addr,
	uint32	len,
	uchar8	* buf
)
{
	uint32	i;
	unsigned long base;
	int32 res;
	uint32 offset = byte_addr;
	uint32 dev = i2c_addr;
	
	base = PHYS_TO_K1(A_SMB_BASE(bus));
	
	switch (op) {
		case I2C_READ:
			
 		 for (i=0; i<len; i++) {

			res = sb1250_smbus_readbyte(base,dev,offset+i);
			if (res < 0)
				return(-1);
			else
				buf[i] = res;
		 }

			break;

		case I2C_WRITE:
			for (i=0; i<len; i++) {

				res = sb1250_smbus_writebyte(base,dev,offset+i,buf[i]);

			   	if (res < 0) {
					return res;
				}

				delay_us(10000);		/* I2C protocol = 10 ms delay */
			}

			break;

		default:
			printf("bad i2c op, %d\n", op);
	}

	return 0;

}	/* i2c() */

