/* vi:ts=4
 *-----------------------------------------------------------------
 *
 * Name: ipm-api.c
 *
 * RCS:  $Id: ipm-api.c,v 1.11 2002/10/30 19:45:10 swang Exp $
 *
 * Copyright (C) 2001, Agile Storage, Inc.
 *
 * Description: ipm apis on txrx. 
 *
 * Created by:	Su Wang
 *
 * Date Created: 10/11/2001
 *
 *-----------------------------------------------------------------
 */
#include <nfx-incl.h>
#include "exports/eee-api.h"
#include "exports/queue-api.h"
#include "exports/ipm-api.h"
#include "exports/cm-api.h"
#include "exports/malloc-api.h"
#include "ipm.h"
#include "exports/bsd-api.h"


/* 
 *-----------------------------------------------------------------
 *
 * Name: ipm_SendMsg
 *
 * Description: send contrl msg to another card, client has msg
 *				ownership, should free msg buffer after send.
 *
 * Created by:	Su Wang
 *
 * Date Created: 10/16/2001
 *
 *-----------------------------------------------------------------
 */
int32							/* NFX_OK, NFX_ERR */
ipm_SendMsg(int		slot,
			int		cpu,
			void	*msg,		/* ptr to real msg */
			uint32	size,		/* size of msg */
			int		bcast
	)
{

	uint32		saddr;
	uint32		daddr;

	
	if (size > IPM_MSG_MAX_SIZE)
		return NFX_ERR;
  
	/* check for bcast */
	if (bcast) {
			/* no support to bcast yet, send to ssc */
		bcast = 0;
		slot = nfxSscSlotId;
		cpu  = NFX_SSC_UNIX;
	}

	/* tmp fix */
	if (slot > EDESC2_SLOT_NUM|| slot < 0 ) {
//		printf("nfxSscSlotId incorrect %d\n", nfxSscSlotId);
		slot = 0;
	}
	
	daddr = PACK_UNICAST_PORT_ADDRESS(
		slot,
		cpu,
		ipm_appid,
		EEE_MGMT,
		NFX_COS_HIGH,
		0 );
  saddr = ipm_eee_src;

//  printf("msg send to slot %d, cpu %d, app %d dest %x \n", slot, cpu, ipm_appid, daddr);
//  bsd_ShowPkt((char*)msg, size);	
  return  eee_sendMessage((char*)msg,
						  size,
						  daddr, saddr);	
}

/* 
 *-----------------------------------------------------------------
 *
 * Name: ipm_SendMsgByApp
 *
 * Description: same  as ipmd_SendMsg, except from application
 *
 * Created by:	Su Wang
 *
 * Date Created: 10/11/2002
 *
 *-----------------------------------------------------------------
 */
int32							/* NFX_OK, NFX_ERR */
ipm_SendMsgByApp(int		mtype,
				 int		slot,
				 int		cpu,
				 void		*msg,		/* ptr to real msg */
				 uint32		in_size, /* size of msg */
				 int		bcast
	)
{

	uint32			saddr;
	uint32			daddr;
	ipm_mhdr_t		*ipm_hdrp = NULL;
	int				size;
	
	size = in_size + sizeof(ipm_mhdr_t);
	if (size > IPM_MSG_MAX_SIZE)
		return NFX_ERR;
	ipm_hdrp = (ipm_mhdr_t*)IPM_ALLOC(size);	

	ipm_hdrp->ipmh_mtype = mtype;
	ipm_hdrp->ipmh_size	= size;
	ipm_hdrp->ipmh_flags = 0;
	
	memcpy((ipm_hdrp+1), msg, in_size);
	
	/* check for bcast */
	if (bcast) {
			/* no support to bcast yet, send to ssc */
		bcast = 0;
		slot = nfxSscSlotId;
		cpu  = NFX_SSC_UNIX;
	}

	/* tmp fix */
	if (slot > EDESC2_SLOT_NUM|| slot < 0 ) {
		slot = 0;
	}
	
	daddr = PACK_UNICAST_PORT_ADDRESS(
		slot,
		cpu,
		ipm_appid,
		EEE_MGMT,
		NFX_COS_HIGH,
		0 );
	saddr = ipm_eee_src;

	eee_sendMessage((char*)ipm_hdrp,
					size,
					daddr, saddr);	

	IPM_FREE(ipm_hdrp);
	return NFX_OK;
}

/* 
 *-----------------------------------------------------------------
 *
 * Name: ipm_SendHdr
 *
 * Created by:	Su Wang
 *
 * Date Created: 06/03/2001
 *
 *-----------------------------------------------------------------
 */
int32					/* NFX_OK, NFX_ERR */
ipm_SendHdr(int		mtype,
			int		slot,
			int		cpu,
			int		bcast)
{
	int			size;
	int32		rc = NFX_OK;
	ipm_mhdr_t	*ipm_hdrp = 0;

	size = sizeof(ipm_mhdr_t);
	
	ipm_hdrp = (ipm_mhdr_t*)IPM_ALLOC(size);

	if (!ipm_hdrp)	{
	  ipm_stats.err_fwd++;
	  IPM_LOGERR(IPM_ERR_ALLOC);
	  rc = NFX_ERR;
	  goto out;
	}

	  /* fill in msg hdr */
	if (ipm_FillIpmMsgHdr(ipm_hdrp, mtype, size) != NFX_OK) {
		ipm_stats.err_fwd++;
		IPM_LOGERR(IPM_ERR_ALLOC);
		rc = NFX_ERR;
		goto out;
	}

	 /* send msg */
	if (ipm_SendMsg(slot, cpu, ipm_hdrp, size, bcast)
		== NFX_OK)
		ipm_stats.dev_fwd++;
	else {
		ipm_stats.err_fwd++;
		rc = NFX_ERR;
	}

out:
	if (ipm_hdrp)
		IPM_FREE(ipm_hdrp);
 
  return rc;
	
}

/* 
 *-----------------------------------------------------------------
 *
 * Name: ipm_SendIntf
 *
 * Description: This function send a interface device. This func is
 *				called when 1) a new interf is detected 2)a request
 *				for a existing interf is detected. 3) a intf has
 *				changed its operational state. The func will be
 *				available only on txrx, because ipm on ssc is an
 *				application which do not have access to kernel directly.
 *
 * Created by:	Su Wang
 *
 * Date Created: 10/11/2001
 *
 *-----------------------------------------------------------------
 */
static int32					/* NFX_OK, NFX_ERR */
ipm_SendIntf(int		mtype,
			 void		*ifp, 
			 int		slot,
			 int		cpu,
			 int		bcast)
{
  int			size;
  ipm_mhdr_t	*ipm_hdrp = 0;
  ipm_mintf_t	*ipm_intfp;
  int			rc = NFX_OK;

  
  size = sizeof(ipm_mhdr_t) + sizeof(ipm_mintf_t);
  ipm_hdrp = (ipm_mhdr_t*)IPM_ALLOC(size);
  
  if (!ipm_hdrp)
	{
	  ipm_stats.err_fwd++;
	  IPM_LOGERR(IPM_ERR_ALLOC);
	  rc = NFX_ERR;
	  goto out;
	}

  /* fill in msg hdr */
  if (ipm_FillIpmMsgHdr(ipm_hdrp, mtype, size) != NFX_OK)
	{
	  ipm_stats.err_fwd++;
	  IPM_LOGERR(IPM_ERR_ALLOC);
	  rc = NFX_ERR;
	  goto out;
	}

  /* fill in interface */
  ipm_intfp = (ipm_mintf_t*)(ipm_hdrp+1);
  bsd_FillIpmMsg(mtype, (void*)ipm_intfp, ifp, 0);
  
  /* send msg */
  if (ipm_SendMsg(slot, cpu, ipm_hdrp, size, bcast)
	  == NFX_OK)
	  ipm_stats.dev_fwd++;
  else {
	  ipm_stats.err_fwd++;
	  rc = NFX_ERR;
  }

 out:
  if (ipm_hdrp)
	  IPM_FREE(ipm_hdrp);
 
  return rc;
}

/* 
 *-----------------------------------------------------------------
 *
 * Name: ipm_NewIntf
 *
 * Description: This function send a newly detected interface device
 *              to other cards. This func is
 *				available only on txrx, because ipm on ssc is an
 *				application which do not have access to kernel directly.
 *
 * Created by:	Su Wang
 *
 * Date Created: 10/11/2001
 *
 *-----------------------------------------------------------------
 */
int32							/* NFX_OK, NFX_ERR */
ipm_NewIntf( void		*ifp,	/* intf to send */
			 int		slot,	/* where to */
			 int		cpu,
			 int		bcast)	/* bcast ?? */
{
  return ipm_SendIntf(IPM_MSG_INTF_NEW, ifp, slot, cpu, bcast);
}


/* 
 *-----------------------------------------------------------------
 *
 * Name: ipm_RespIntf
 *
 * Description: This function send a requested interface device
 *              to other cards. This func is
 *				available only on txrx, because ipm on ssc is an
 *				application which do not have access to kernel directly.
 *
 * Created by:	Su Wang
 *
 * Date Created: 10/11/2001
 *
 *-----------------------------------------------------------------
 */
int32							/* NFX_OK, NFX_ERR */
ipm_RespIntf( void		*ifp,	/* intf to send */
			 int		slot,	/* where to */
			 int		cpu,
			 int		bcast)	/* bcast ?? */
{
  return ipm_SendIntf(IPM_MSG_INTF_RESP, ifp, slot, cpu, bcast);
}

extern int ipm_Updt_Trig;
/*
 *-----------------------------------------------------------------
 *
 * Name: ipm_UpdateConn
 *
 * Description: update connection table on ssc. 
 *
 * Created by:	Su Wang
 *
 * Date Created: 12/14/2001
 *
 *-----------------------------------------------------------------
 */
static int32					/* NFX_OK, NFX_ERR */
ipm_UpdateConn( int				proto, /* IPM_IP_PROTO_TCP, IPM_IP_PROTO_UDP */
				int				cmd, /* IPM_ADD, IPM_DEL, IPM_CHGE_STATE */
				uint32			laddr,
				ushort16		lport,
				uint32			raddr,
				ushort16		rport,
				uint32			state
				)
{
  ipm_mconn_t		*ipm_connp = NULL;
  ipm_updtconn_t	**connQ;
  int				new=1;
  int				hashIdx;
  ipm_updtconn_t	*ipmUdpConnPtr;
  uint32			*connCntPtr;

  /* stats update not turned on */ 
  if (!ipm_Updt_Trig)
	return NFX_OK;

  if (proto == IPM_IP_PROTO_TCP) {
	  connCntPtr = &(ipm_stats.connTcpNum);
	  connQ = ipm_TcpConnQ;
  }
  else {
	  connCntPtr = &(ipm_stats.connUdpNum);
	  connQ = ipm_UdpConnQ;  
  }
  
  /* traverse the bucket, looking for existing conn. */
  hashIdx = IPM_HASH(raddr, rport);
  for (ipmUdpConnPtr = connQ[hashIdx]; ipmUdpConnPtr; 
	   ipmUdpConnPtr = ipmUdpConnPtr->next) {
	  ipm_connp = &(ipmUdpConnPtr->conn);
	  if ((ipm_connp->conn.conn_lAddr == laddr) &&
		  (ipm_connp->conn.conn_lPort == lport) &&
		  (ipm_connp->conn.conn_rPort == rport) &&
		  (ipm_connp->conn.conn_rAddr == raddr)) {
		  /* exsiting conn. found */
		  new = 0;
		  break;
	  }
  }
  
  if (new) {
	  
	  if ((ipmUdpConnPtr = IPM_ALLOC(sizeof(ipm_updtconn_t))) == NULL) {
		  IPM_LOGERR(IPM_ERR_ALLOC);
		  return NFX_ERR;
	  }
	  
	  ipm_connp = &(ipmUdpConnPtr->conn);
	
	  /* fill in content */
	  ipm_connp->conn.conn_lAddr = laddr;
	  ipm_connp->conn.conn_lPort = lport; /* the first two bytes has port num */
	  ipm_connp->conn.conn_rAddr = raddr;
	  ipm_connp->conn.conn_rPort = rport; /* the first two bytes has port num */

	  /* link */
	  ipmUdpConnPtr->next = connQ[hashIdx];	
	  connQ[hashIdx] = ipmUdpConnPtr;
	
	  (*connCntPtr)++;
  }

  /* update cmd and state for new and existing conn. */
  ipm_connp->cmd = cmd;
  ipm_connp->conn.conn_state = state;

  return NFX_OK;
}

/*
 *-----------------------------------------------------------------
 *
 * Name: ipm_AddUdpConn
 *
 * Description:
 *
 * Created by:	Su Wang
 *
 * Date Created: 12/14/2001
 *
 *-----------------------------------------------------------------
 */
int32 
ipm_AddUdpConn(	uint32			laddr,
				ushort16		lport,
				uint32			raddr,
				ushort16		rport)
{
  return (ipm_UpdateConn(IPM_IP_PROTO_UDP, IPM_ADD,
						 laddr, lport, raddr, rport, 0));
}


/*
 *-----------------------------------------------------------------
 *
 * Name: ipm_DelUdpConn
 *
 * Description:
 *
 * Created by:	Su Wang
 *
 * Date Created: 12/14/2001
 *
 *-----------------------------------------------------------------
 */
int32 
ipm_DelUdpConn(	uint32			laddr,
				ushort16		lport,
				uint32			raddr,
				ushort16		rport)
{
  return (ipm_UpdateConn(IPM_IP_PROTO_UDP, IPM_DEL,
						 laddr, lport, raddr, rport, 0));
}

/*
 *-----------------------------------------------------------------
 *
 * Name: ipm_AddTcpConn
 *
 * Description:
 *
 * Created by:	Su Wang
 *
 * Date Created: 12/14/2001
 *
 *-----------------------------------------------------------------
 */
int32 
ipm_AddTcpConn(	uint32			laddr,
				ushort16		lport,
				uint32			raddr,
				ushort16		rport,
				uint32			state)
{
  return (ipm_UpdateConn(IPM_IP_PROTO_TCP, IPM_ADD,
						 laddr, lport, raddr, rport, state));
}
 
/*
 *-----------------------------------------------------------------
 *
 * Name: ipm_DelTcpConn
 *
 * Description:
 *
 * Created by:	Su Wang
 *
 * Date Created: 12/14/2001
 *
 *-----------------------------------------------------------------
 */
int32 
ipm_DelTcpConn(	uint32			laddr,
				ushort16		lport,
				uint32			raddr,
				ushort16		rport)
{
  return (ipm_UpdateConn(IPM_IP_PROTO_TCP, IPM_DEL,
						 laddr, lport, raddr, rport, 0));
}
 
/*
 *-----------------------------------------------------------------
 *
 * Name: ipm_ChgeStateTcpConn
 *
 * Description:
 *
 * Created by:	Su Wang
 *
 * Date Created: 12/14/2001
 *
 *-----------------------------------------------------------------
 */
int32 
ipm_ChgeStateTcpConn(uint32			laddr,
					 ushort16		lport,
					 uint32			raddr,
					 ushort16		rport,
					 uint32			state)
{
  return (ipm_UpdateConn(IPM_IP_PROTO_TCP, IPM_CHGE_STATE,
						 laddr, lport, raddr, rport, state));
}
