/* vi:ts=4
 *-----------------------------------------------------------------
 *
 * Name: ipm-api.h
 *
 * RCS:  $Id: ipm-api.h,v 1.11 2002/10/30 19:45:10 swang Exp $
 *
 * Copyright (C) 2001, Agile Storage, Inc.
 *
 * Description: IP interface managment API.
 *
 * Created by:	Su Wang
 *
 * Date Created: 10/11/2001
 *
 *-----------------------------------------------------------------
 */
#ifndef _IPM_API_H
#define _IPM_API_H

/* intf type, these are shared by txrx, ssc kernel and ssc apps */
#define IPM_INTF_TYPE_MIN	 1
#define IPM_INTF_TYPE_ETH	 1	/* loopbck */
#define IPM_INTF_TYPE_SSC	 2	/* ethernt */
#define IPM_INTF_TYPE_MAX	 2		

#define IPM_LINK_MAX		 4	

#if defined(NFP_TXRX)
/* 
 *-----------------------------------------------------------------
 *
 * Name: ipm_SendMsg
 *
 * Description: send contrl msg to another card, client has msg
 *				ownership, should free msg buffer after send.
 *
 * Created by:	Su Wang
 *
 * Date Created: 10/16/2001
 *
 *-----------------------------------------------------------------
 */
int32							/* NFX_OK, NFX_ERR */
ipm_SendMsg(int		slot,
			int		cpu,
			void	*msg,		/* ptr to real msg */
			uint32	size,		/* size of msg */
			int		bcast);


int32							/* NFX_OK, NFX_ERR */
ipm_SendMsgByApp(int		mtype,
				 int		slot,
				 int		cpu,
				 void		*msg,		/* ptr to real msg */
				 uint32		in_size, /* size of msg */
				 int		bcast);

/* 
 *-----------------------------------------------------------------
 *
 * Name: ipm_SendHdr
 *
 * Created by:	Su Wang
 *
 * Date Created: 06/03/2001
 *
 *-----------------------------------------------------------------
 */
int32					/* NFX_OK, NFX_ERR */
ipm_SendHdr(int		mtype,
			int		slot,
			int		cpu,
			int		bcast);

/* 
 *-----------------------------------------------------------------
 *
 * Name: ipm_NewIntf
 *
 * Description: This function send a newly detected interface device
 *              to other cards. This func is
 *				available only on txrx, because ipm on ssc is an
 *				application which do not have access to kernel directly.
 *
 * Created by:	Su Wang
 *
 * Date Created: 10/11/2001
 *
 *-----------------------------------------------------------------
 */
int32							/* NFX_OK, NFX_ERR */
ipm_NewIntf( void		*ifp,	/* intf to send */
			 int		slot,	/* where to */
			 int		cpu,
			 int		bcast);	/* bcast ?? */

/* 
 *-----------------------------------------------------------------
 *
 * Name: ipm_RespIntf
 *
 * Description: This function send a requested interface device
 *              to other cards. This func is
 *				available only on txrx, because ipm on ssc is an
 *				application which do not have access to kernel directly.
 *
 * Created by:	Su Wang
 *
 * Date Created: 10/11/2001
 *
 *-----------------------------------------------------------------
 */
int32							/* NFX_OK, NFX_ERR */
ipm_RespIntf( void		*ifp,	/* intf to send */
			 int		slot,	/* where to */
			 int		cpu,
			 int		bcast);	/* bcast ?? */

/*
 *-----------------------------------------------------------------
 *
 * Name: ipm_AddUdpConn
 *
 * Description:
 *
 * Created by:	Su Wang
 *
 * Date Created: 12/14/2001
 *
 *-----------------------------------------------------------------
 */
int32 
ipm_AddUdpConn(	uint32			laddr,
				ushort16		lport,
				uint32			raddr,
				ushort16		rport);


/*
 *-----------------------------------------------------------------
 *
 * Name: ipm_DelUdpConn
 *
 * Description:
 *
 * Created by:	Su Wang
 *
 * Date Created: 12/14/2001
 *
 *-----------------------------------------------------------------
 */
int32 
ipm_DelUdpConn(	uint32			laddr,
				ushort16		lport,
				uint32			raddr,
				ushort16		rport);

/*
 *-----------------------------------------------------------------
 *
 * Name: ipm_AddTcpConn
 *
 * Description:
 *
 * Created by:	Su Wang
 *
 * Date Created: 12/14/2001
 *
 *-----------------------------------------------------------------
 */
int32 
ipm_AddTcpConn(	uint32			laddr,
				ushort16		lport,
				uint32			raddr,
				ushort16		rport,
				uint32			state);


/*
 *-----------------------------------------------------------------
 *
 * Name: ipm_DelTcpConn
 *
 * Description:
 *
 * Created by:	Su Wang
 *
 * Date Created: 12/14/2001
 *
 *-----------------------------------------------------------------
 */
int32 
ipm_DelTcpConn(	uint32			laddr,
				ushort16		lport,
				uint32			raddr,
				ushort16		rport);


/*
 *-----------------------------------------------------------------
 *
 * Name: ipm_ChgeStateTcpConn
 *
 * Description:
 *
 * Created by:	Su Wang
 *
 * Date Created: 12/14/2001
 *
 *-----------------------------------------------------------------
 */
int32 
ipm_ChgeStateTcpConn(uint32			laddr,
					 ushort16		lport,
					 uint32			raddr,
					 ushort16		rport,
					 uint32			state);

#endif /* NFP_TXRX */

#endif /* _IPM_API_H */
