/* vi:ts=4
 *-----------------------------------------------------------------
 *
 * Name: ipm.c
 *
 * RCS:  $Id: ipm.c,v 1.24 2002/10/31 19:06:22 janv Exp $
 *
 * Copyright (C) 2001, Agile Storage, Inc.
 *
 * Description: 
 *
 * Created by:	Su Wang
 *
 * Date Created: 10/11/2001
 *
 *-----------------------------------------------------------------
 */
#include <nfx-incl.h>
#include "exports/queue-api.h"
#include "exports/eee-api.h"
#include "exports/cm-api.h"
#include "exports/timer-api.h"
#include "exports/bsdnet-api.h"
#include "exports/ipm-api.h"
#include "exports/malloc-api.h"
#include "ipm.h"
#include "ipm-api.h"
#include "exports/bsdnet-api.h"
#include "exports/bsdsock-api.h"
#include "exports/bmc12500-eth-api.h"
#include "../sm-bsd-api/bsd-arpsniff.h"
#include "../sm-bsd-api/bsd-vif.h"

extern int32 luc_sendintf(int32, char**);

static void  ipm_ProcessMsg(eee_descPtr_t epdesc);
static int32 ipm_Timer(uint32, void*, void*, void*, void*);
//static int32 ipm_ProcReqIfMsg(ipm_mintf_t *ipmp, int slot, int cpu);
static void  ipm_SendStatsUpdate();
static void  ipm_SendConnUpdate();
static int32 ipm_GetL2ConnEntry(uint32, uint32*, ipm_mhdr_t**);

/* ipm msg statis */
ipm_stats_t		ipm_stats;
int				ipm_appid=0;
uint32			ipm_eee_src;

/* connection update queue */
ipm_updtconn_t*	ipm_TcpConnQ[IPM_HASH_SIZE];
ipm_updtconn_t*	ipm_UdpConnQ[IPM_HASH_SIZE];

/* trigger to send ipm updates */
int				ipm_Updt_Trig = 1;
int32			ipm_TimerHandle;
	
/* 
 *-----------------------------------------------------------------
 *
 * Name: ipm_Init
 *
 * Description: Init Ip management.
 *
 * Created by:	Su Wang
 *
 * Date Created: 10/11/2001
 *
 *-----------------------------------------------------------------
 */
void
ipm_Init()
{



	/* register with EEE */
	ipm_appid = eee_allocateApplicationId("ipm");	
	if (ipm_appid == EEE_ERR_INVALID_APP_NAME)
		 panic("ipm_init: invalid app name \n");

	if (eee_registerReceiveFunc(ipm_appid, ipm_ProcessMsg, ipm_ProcessMsg)
		!= NFX_OK)
		panic("ipm_init: duplicated registeration with EEE\n");

	/* initiaze eee src address */
	ipm_eee_src = PACK_UNICAST_PORT_ADDRESS(
		nfxMySlotId,
		nfxMyCPUId,
		ipm_appid,
		EEE_MGMT,
		NFX_COS_HIGH,
		0);
	
	ipm_TimerHandle = eee_timerCreate(10000, EEE_TIMER_REPEAT);
 
	if (ipm_TimerHandle == TIMER_ERR_NO_TIMER_AVAILABLE)
		panic("ipm: create timer  failed\n");
	
	if (eee_timerStart(ipm_TimerHandle, ipm_Timer, NULL,
					   NULL, NULL, NULL) != NFX_OK)
		panic("ipm: start timer  failed\n");
	
	/* reset ipm stats */
	bzero(&ipm_stats, sizeof(ipm_stats));

	/* reset conn queues */
	bzero(ipm_TcpConnQ, sizeof(ipm_TcpConnQ));
	bzero(ipm_TcpConnQ, sizeof(ipm_TcpConnQ));

#if defined(ARP_SNIFFING)
	/* arp sniffing can be seen as part of ipm */
	arpS_Init();
#endif /* ARP_SNIFFING */

	/* send restart to ssc */
	ipm_SendHdr(IPM_MSG_RESTART, nfxSscSlotId, NFX_SSC_UNIX, 0);

	return;
}

/* 
 *-----------------------------------------------------------------
 *
 * Name: ipm_Timer
 *
 * Description: 
 *
 * Created by:	Su Wang
 *
 * Date Created: 10/19/2001
 *
 *-----------------------------------------------------------------
 */
static int32
ipm_Timer(uint32 handle, void* parm1, void* parm2, void* parm3, void* parm4)
{

	if (ipm_Updt_Trig) {
		/* send statis update */
		ipm_SendStatsUpdate();
		
		if (parm1)				/* ipmd restarted, send everything */
			bsd_MakeConnQ(1);
		else
			bsd_MakeConnQ(0);	/* regular update */
	  
		/* send connection update */
		ipm_SendConnUpdate();
	}

#if defined(ARP_SNIFFING)
	/* call arp sniffing timer function */
	arpS_Timer();
#endif /* ARP_SNIFFING */
	
	return NFX_OK;
}

/* 
 *-----------------------------------------------------------------
 *
 * Name: ipm_ProcessMsg
 *
 * Description: process ipm messages on txrx.
 *
 * Created by:	Su Wang
 *
 * Date Created: 10/11/2001
 *
 *-----------------------------------------------------------------
 */
static void
ipm_ProcessMsg(eee_descPtr_t epdesc)
{
  ipm_mhdr_t	*ipm_hdrp=0, *ipm_newHdrp;
  ipm_mintf_t	*ipm_intfp = 0;
  ipm_mip_t		*ipm_ipp;
  ipm_mrt_t		*ipm_rtp;
  ipm_marp_t	*ipm_arpp;
  char			*ipm_cur;		/* current ptr */
  int			mtype;
  int			dtype;
  int			slot;
  int			cpu;
  int			link = 0;
  char			*device = NULL;
  uint32		eee_dst = epdesc->hdr.src_port; /* remember where to send reply to */
  int			msgsize;
  int			rc = NFX_ERR;
  int			reply = 0;
#if defined(LINK_AGG)
  char			*vifName = NULL;
  void			*linkList = NULL;
  int			linkCnt = 0;
  ipm_mvif_t	*ipm_vifp = NULL;
  void			*ifp;
#endif /* LINK_AGG */

  msgsize = EEE_GET_PKT_LEN(epdesc);

  if (msgsize > IPM_MSG_MAX_SIZE) {
	  IPM_LOGERR(IPM_ERR_RCV);	
	  goto out;
  }
	

  ipm_hdrp = (ipm_mhdr_t*)eee_ramAlloc(msgsize);

  if (!ipm_hdrp) {
	  IPM_LOGERR(IPM_ERR_RCV);
	  goto out;
  }

  msgsize = eee_copyPacketToLocalBuffer((void*)ipm_hdrp,
									 epdesc,
									 msgsize,
									 0); 
   

  /* is size correct */
  if ((msgsize == EEE_ERR_PACKET_COPY_FAILED) ||
	  (msgsize < ipm_hdrp->ipmh_size)) {
	  IPM_LOGERR(IPM_ERR_RCV);
	  goto out;
  }
  
  /* msgsize could be larger */
  msgsize = ipm_hdrp->ipmh_size;
  
  /* retrive action */
  mtype = ipm_hdrp->ipmh_mtype;

  ipm_cur = (char*)(ipm_hdrp+1);
  if ((mtype == IPM_MSG_INTF_ENA) ||
	  (mtype == IPM_MSG_INTF_DIS) ||
	  (mtype == IPM_MSG_IP_DEL)   ||
	  (mtype == IPM_MSG_IP_ADD)   ||
	  (mtype == IPM_MSG_INTF_SETMTU) ||
	  (mtype == IPM_MSG_INTF_AUTOEN) ||
	  (mtype == IPM_MSG_INTF_AUTODIS) ||
	  (mtype == IPM_MSG_INTF_REQ))
	{
	  /* retrieve device interface that needed action */
	  ipm_intfp = (ipm_mintf_t*)ipm_cur;
	  dtype = ipm_intfp->if_dtype;
	  slot  = ipm_intfp->if_slot;
	  cpu   = ipm_intfp->if_cpu;
	  link  = ipm_intfp->if_link;
	  device = (char*)bsdPrintPort(slot, cpu, link, dtype);
	  ipm_cur = (char*)(ipm_intfp + 1);
	}

#if defined(LINK_AGG)
  if (IPM_VIF_MSGS(mtype)) {
	  
	  ipm_stats.adm_rcv++;

	  ipm_vifp = (ipm_mvif_t*)ipm_cur;
	  
	  /* retrive vif name */
	  vifName = ipm_vifp->vif_name;
	  
	  /* retrive link info */
	  linkCnt = ipm_vifp->vif_linkCnt;
	  if (linkCnt) {
		  address_t		*linkPtr;
		  int			i;
		  
		  linkList = eee_ramAlloc(sizeof(address_t)*linkCnt);
		  linkPtr = (address_t*)linkList;
		  for (i = 0; i < linkCnt; i++) {
			  device = bsdPrintPortFromSSC(ipm_vifp->vif_links[i]);
			  if ((*linkPtr = (address_t)bsd_FindIntfByName(device)) == NULL) {
				  rc = NFX_ERR;
				  /* no need to ack back any delelete cmds */
				  if (mtype == IPM_MSG_VIF_RMLNK)
					  reply = 0;
				  else
					  reply = 1;
				  
				  goto reply;
			  }
			  linkPtr ++;
		  }
	  }
  }
  
#endif /* LINK_AGG */

  switch(mtype)
	{
	  /* not used */
	case IPM_MSG_INTF_NEW:
	case IPM_MSG_INTF_RESP:
	  ipm_stats.dev_rcv++;
	  break;
	  
	case IPM_MSG_INTF_ENA:
	  ipm_stats.adm_rcv++;
	  if ((rc=bsd_AttachIf(ipm_intfp)) == NFX_OK)
		rc = bsd_ChangeStateIntf(device,
								 BSD_UP, 0, 0);
	  reply = 1;
	  break;

	case IPM_MSG_INTF_DIS:
	  ipm_stats.adm_rcv++;
	  if ((rc = bsd_AttachIf(ipm_intfp)) == NFX_OK)
		  rc = bsd_ChangeStateIntf(device,
								   BSD_DOWN, 0, 0);
	  reply = 1;
	  break;
  
	case IPM_MSG_INTF_SETMTU:
	  ipm_stats.adm_rcv++;
	  rc = bmc12500Eth_set_maxPktSize(link, ipm_intfp->if_mtusize);
	  if (rc == NFX_OK) {
		  rc = bsd_SetMtuIntf(device, ipm_intfp->if_mtusize);
		  // printf("mtu size %u to link %u set\n", 
		  //		 ipm_intfp->if_mtusize, link);
	  }
	  if (!(ipm_hdrp->ipmh_flags & IPM_MSG_CMD_NREPLY))
		  reply = 1;
	  break;

	case IPM_MSG_INTF_AUTOEN:
		ipm_stats.adm_rcv++;
		// printf("try toauto enable  to link %u set\n", link);
		rc = bmc12500Eth_set_autoneg(link, 1);
		// if (rc == NFX_OK) {
		//	printf("auto enable  to link %u set\n", link);
		// }
		if (!(ipm_hdrp->ipmh_flags & IPM_MSG_CMD_NREPLY))
			reply = 1;
		break;

	case IPM_MSG_INTF_AUTODIS:
		ipm_stats.adm_rcv++;
		// printf("try to auto disable  to link %u set\n", link);
		rc = bmc12500Eth_set_autoneg(link, 0);
		//if (rc == NFX_OK) {
		//	printf("auto disable  to link %u set\n", link);
		// }
		if (!(ipm_hdrp->ipmh_flags & IPM_MSG_CMD_NREPLY))
			reply = 1;
		break;		

	case IPM_MSG_IP_DEL:
	  ipm_stats.adm_rcv++;
	  ipm_ipp = (ipm_mip_t*)ipm_cur;
	  if ((rc = bsd_AttachIf(ipm_intfp)) == NFX_OK)
		  bsd_DeleteIpIntf(ipm_ipp->ip_addr, 
						   device, 0,0);
	  rc = NFX_OK;
	  reply = 1;
	  break;

	case IPM_MSG_IP_ADD:
	  ipm_stats.adm_rcv++;
	  ipm_ipp = (ipm_mip_t*)ipm_cur;
	  if ((rc = bsd_AttachIf(ipm_intfp)) == NFX_OK) {
		  void	*ifp;
		  /* remove old IP config */
		  if ((ifp = bsd_FindIntfByIp(ipm_ipp->ip_addr))) {
			  bsd_DeleteIpByIntf(ipm_ipp->ip_addr, ifp);
		  }
		  rc = bsd_CreateIpIntf(ipm_ipp->ip_addr, ipm_ipp->ip_mask,
								device, 0,0);
	  }
	  
	  if (ipm_hdrp->ipmh_flags & IPM_MSG_CMD_NREPLY)
		  reply = 0;
	  else
		  reply = 1;

	  break;

	case IPM_MSG_RT_ADD:
	  ipm_stats.adm_rcv++;
	  ipm_rtp = (ipm_mrt_t*)ipm_cur;

	  /* remove old route */
	  if (bsd_FindRt(ipm_rtp->rt_addr, 
					 ipm_rtp->rt_mask,ipm_rtp->rt_nhop,IPRT_ALL)) {
		  bsd_ModRoute(ipm_rtp->rt_addr, ipm_rtp->rt_mask,
					   ipm_rtp->rt_nhop, 0,
					   BSD_DEL, 0, 0);
	  }
	  rc = bsd_ModRoute(ipm_rtp->rt_addr, ipm_rtp->rt_mask,
						ipm_rtp->rt_nhop, 0,
						BSD_ADD, 0, 0);
	  reply = 1;
	  break;

	case IPM_MSG_RT_DEL:
	  ipm_stats.adm_rcv++;
	  ipm_rtp = (ipm_mrt_t*)ipm_cur;

	  /* remove route only if found */
	  if (bsd_FindRt(ipm_rtp->rt_addr, 
					 ipm_rtp->rt_mask,ipm_rtp->rt_nhop, IPRT_STATIC)) 
		  bsd_ModRoute(ipm_rtp->rt_addr, ipm_rtp->rt_mask,
					   ipm_rtp->rt_nhop, 0,
					   BSD_DEL, 0, 0);
	  rc = NFX_OK;
	  reply = 1;
	  break;

	case IPM_MSG_ARP_ADD:
	  ipm_stats.adm_rcv++;
	  ipm_arpp = (ipm_marp_t*)ipm_cur;
	  rc = bsd_ModARP(ipm_arpp->arp_addr, ipm_arpp->arp_mac, 
					  BSD_ADD, 0, 0);
	  reply = 1;
	  break;

	case IPM_MSG_ARP_DEL:
	  ipm_stats.adm_rcv++;
	  ipm_arpp = (ipm_marp_t*)ipm_cur;
	  bsd_ModARP(ipm_arpp->arp_addr, ipm_arpp->arp_mac, 
				 BSD_DEL, 0, 0);
	  rc = NFX_OK;
	  reply = 1;
	  break;

	case IPM_MSG_SHUTDOWN:
		bsd_ResetTcp();
		bsd_ChangeStateIntfAll(BSD_DOWN);
		rc = NFX_OK;
		reply = 1;
		break;

	case IPM_MSG_RESTART:
		bsd_ChangeStateIntfAll(BSD_UP);
		/* send interface info */
		luc_sendintf(0, NULL);
		
		/* send update now */
		ipm_Timer(ipm_TimerHandle, (void*)1, NULL, NULL, NULL);
		rc = NFX_OK;
		if (ipm_hdrp->ipmh_flags & IPM_MSG_CMD_NREPLY)
			reply = 0;
		else
			reply = 1;
		break;

	case  IPM_MSG_INTF_REQ: 
	{
		void	*ifp = 0;

		// printf("retrive %s info\n", device);
		ifp = bsd_FindIntfByName((char*)device);
		
		/* fill interface info */
		if (ifp) {
			bsd_FillIpmMsg(IPM_MSG_INTF_RESP, (ipm_hdrp+1), ifp, 0);
			rc = NFX_OK;
		}
		else
			rc = NFX_ERR;
		
		reply =1;
		break;
	}

#if defined(ARP_SNIFFING)
	case IPM_MSG_L2CONN_ENT_REQ:
	{
		ipm_mstat_req_t	*ipm_mreqp = (ipm_mstat_req_t*)ipm_cur;
		
		rc = ipm_GetL2ConnEntry(ipm_mreqp->req.ip, &msgsize, &ipm_newHdrp);
		if (rc == NFX_OK) {
			/* ipm_newHdrp will replace ipm_hdrp when sending,
			 * ipm_newHdrp is allocated inside arpS_GetL2ConnEntry
			 */
			eee_ramDealloc(ipm_hdrp);
			ipm_hdrp = ipm_newHdrp;
		}
		reply = 1;
		break;
	}
#endif /* ARP_SNIFFING */

#if defined(LINK_AGG)
	case IPM_MSG_VIF_CREAT:
		rc = bsd_AddVif(vifName, ipm_vifp->vif_flags, linkList, linkCnt);
		reply = 1;
		break;
		
	case IPM_MSG_VIF_DEL:
		rc = bsd_DelVif(vifName);
		reply = 0;
		break;
		
	case IPM_MSG_VIF_ADDLNK:
		rc = bsd_AddVifLink(vifName, linkList, linkCnt);
		reply = 1;
		break;
		
	case IPM_MSG_VIF_RMLNK:
		rc = bsd_RmVifLink(vifName, linkList, linkCnt);
		reply = 0;
		break;
		
	case IPM_MSG_VIP_ADD:
		rc = bsd_AddVifIP(vifName, ipm_vifp->vif_ip, ipm_vifp->vif_mask);
		reply = 1;
		break;
		
	case IPM_MSG_VIP_DEL:
		rc = bsd_DelVifIP(vifName, ipm_vifp->vif_ip);
		reply = 0;
		break;
		
	case IPM_MSG_VIF_PREF:
		reply = 1;
		device = bsdPrintPortFromSSC(ipm_vifp->vif_prefLink);
		if ((ifp = bsd_FindIntfByName(device)) == NULL) {
			rc = NFX_ERR;
			break;
		}
		rc = bsd_PrefLinkVif(vifName, ifp);
		break;
#endif /* LINK_AGG */	

	default:
	  ipm_stats.err_rcv++;
	  IPM_LOGERR(IPM_ERR_RCV);
	}

reply:
  /* we need to send a reply bck to sender */
  if (reply) {
	  if (rc == NFX_OK)
		  ipm_hdrp->ipmh_flags |= IPM_MSG_CMD_OK;
	  else
		  ipm_hdrp->ipmh_flags &= ~IPM_MSG_CMD_OK;


//	  printf("send reply to dest %x\n", eee_dst);
//	  bsd_ShowPkt((char*)ipm_hdrp, msgsize);

	  if (eee_sendMessage((char*)ipm_hdrp, msgsize,
						  eee_dst, ipm_eee_src) != NFX_OK)
		  ipm_stats.err_fwd++;
  }
  

out:
  eee_freePacket(epdesc);
  if (ipm_hdrp)
	eee_ramDealloc(ipm_hdrp);

#if defined(LINK_AGG)
  if (linkList)
	  eee_ramDealloc(linkList);
#endif /* LINK_AGG */  
  return;
}

/* 
 *-----------------------------------------------------------------
 *
 * Name: ipm_SendStatsUpdate
 *
 * Description: send statis update to ssc
 *
 * Created by:	Su Wang
 *
 * Date Created: 12/14/2001
 *
 *-----------------------------------------------------------------
 */
static void
ipm_SendStatsUpdate()
{
  void			*ifp = 0;
  int			size;
  ipm_mhdr_t	*ipm_hdrp = 0;
  void			*statsp;

  /* allocate single buffer to send all updates */
  size = IPM_MSG_MAX_SIZE;
  
  
  ipm_hdrp = (ipm_mhdr_t*)IPM_ALLOC(size);
  
  if (!ipm_hdrp)
	{
	  IPM_LOGERR(IPM_ERR_ALLOC);
	  return;
	}
  bzero(ipm_hdrp, size);
  
  statsp = (void*)(ipm_hdrp+1);

  /* send IPM_MSG_INTF_STATS  */
  size = sizeof(ipm_mhdr_t) + sizeof(ipm_ifstat_t);
  (void)ipm_FillIpmMsgHdr(ipm_hdrp, IPM_MSG_INTF_STATS, size);
  for (ifp = bsd_GetNextIntf(NULL); ifp; ifp = bsd_GetNextIntf(ifp))
	{
	  if (bsd_FillIpmStatsMsg(IPM_MSG_INTF_STATS, (void*)statsp, (void*)ifp)
		  != NFX_OK)
		{
		  IPM_LOGERR(IPM_ERR_SEND);
		  goto out;
		}

	  if (ipm_SendMsg(nfxSscSlotId, NFX_SSC_UNIX, (void*)ipm_hdrp, size, 0)
		  != NFX_OK)
		  ipm_stats.err_fwd++;
	  else
		  ipm_stats.info_fwd++;
	}


  /* send IPM_MSG_IP_STATS */
  size = sizeof(ipm_mhdr_t) + sizeof(ipm_ipstat_t);
  (void)ipm_FillIpmMsgHdr(ipm_hdrp, IPM_MSG_IP_STATS, size);
  if (bsd_FillIpmStatsMsg(IPM_MSG_IP_STATS, (void*)statsp, (void*)NULL)
	  != NFX_OK)
	{
	  IPM_LOGERR(IPM_ERR_SEND);
	  goto out;
	}

  if (ipm_SendMsg(nfxSscSlotId, NFX_SSC_UNIX, (void*)ipm_hdrp, size, 0)
	  != NFX_OK)
	ipm_stats.err_fwd++;
  else
	ipm_stats.info_fwd++;

  /* send IPM_MSG_ICMP_STATS */
  size = sizeof(ipm_mhdr_t) + sizeof(ipm_icpstat_t);
  (void)ipm_FillIpmMsgHdr(ipm_hdrp, IPM_MSG_ICMP_STATS, size);
  if (bsd_FillIpmStatsMsg(IPM_MSG_ICMP_STATS, (void*)statsp, (void*)NULL)
	  != NFX_OK)
	{
	  IPM_LOGERR(IPM_ERR_SEND);
	  goto out;
	}

  if (ipm_SendMsg(nfxSscSlotId, NFX_SSC_UNIX, (void*)ipm_hdrp, size, 0)
	  != NFX_OK)
	ipm_stats.err_fwd++;
  else
	ipm_stats.info_fwd++;
   
  /* send IPM_MSG_TCP_STATS */
  size = sizeof(ipm_mhdr_t) + sizeof(ipm_tcpstat_t);
  (void)ipm_FillIpmMsgHdr(ipm_hdrp, IPM_MSG_TCP_STATS, size);
  if (bsd_FillIpmStatsMsg(IPM_MSG_TCP_STATS, (void*)statsp, (void*)NULL)
	  != NFX_OK)
	{
	  IPM_LOGERR(IPM_ERR_SEND);
	  goto out;
	}

  if (ipm_SendMsg(nfxSscSlotId, NFX_SSC_UNIX, (void*)ipm_hdrp, size, 0)
	  != NFX_OK)
	ipm_stats.err_fwd++;
  else
	ipm_stats.info_fwd++;

  /* send IPM_MSG_UDP_STATS */
  size = sizeof(ipm_mhdr_t) + sizeof(ipm_udpstat_t);
  (void)ipm_FillIpmMsgHdr(ipm_hdrp, IPM_MSG_UDP_STATS, size);
  if (bsd_FillIpmStatsMsg(IPM_MSG_UDP_STATS, (void*)statsp, (void*)NULL)
	  != NFX_OK)
	{
	  IPM_LOGERR(IPM_ERR_SEND);
	  goto out;
	}

  if (ipm_SendMsg(nfxSscSlotId,  NFX_SSC_UNIX, (void*)ipm_hdrp, size, 0)
	  != NFX_OK)
	ipm_stats.err_fwd++;
  else
	ipm_stats.info_fwd++;

  
 out:
  if (ipm_hdrp)
	  IPM_FREE(ipm_hdrp);
  return;
}

/* 
 *-----------------------------------------------------------------
 *
 * Name: ipm_SendConnUpdate
 *
 * Description: send connection update to ssc
 *
 * Created by:	Su Wang
 *
 * Date Created: 12/14/2001
 *
 *-----------------------------------------------------------------
 */
static void
ipm_SendConnUpdate()
{
  ipm_mconn_t	*ipm_connp;
  ipm_mchdr_t	*ipm_mchdrp;
  ipm_mhdr_t	*ipm_hdrp = 0;
  int			msgsize, size, i;
  int			count;
  int			mtype;		
  int			hashIdx = 0;
  uint32		*connCntPtr;
  ipm_updtconn_t	**connQ;
  ipm_updtconn_t	*ipmq_connp, *ipmq_tmp;

  
  msgsize = IPM_MSG_MAX_SIZE;
  
  connQ = ipm_TcpConnQ;
  mtype = IPM_MSG_TCP_CONN;
  connCntPtr = &(ipm_stats.connTcpNum);
  

  /* allocate single buffer to send all updates */
  ipm_hdrp = IPM_ALLOC(msgsize);
  if (!ipm_hdrp) {
	  IPM_LOGERR(IPM_ERR_ALLOC);
	  return;
  }

nextqueue:  
  /* find the first conn update */
  for (i=0; i<IPM_HASH_SIZE; i++) {
	  if ((ipmq_connp = connQ[i])) {
		  hashIdx = i;
		  break;
	  }
  }
  
sendmore:
  size = sizeof(ipm_mhdr_t) + sizeof(ipm_mchdr_t);
  count = 0;
  ipm_mchdrp = (ipm_mchdr_t*)(ipm_hdrp+1);
  ipm_connp = (ipm_mconn_t*)(ipm_mchdrp+1);

  while(ipmq_connp) {
	  if (size + sizeof(ipm_mconn_t) > msgsize)
		  break;

	  if (!(*connCntPtr)) {
		  /* updt queue and stats does not match,
		   * clear hash table, go out 
		   */
		  IPM_LOGERR(IPM_ERR_QUEUE);
		  bzero(connQ, IPM_HASH_SIZE*sizeof(ipm_updtconn_t*));
		  goto out;
	  }
		  
	  memcpy(ipm_connp, &(ipmq_connp->conn), sizeof(ipm_mconn_t));
	  
	  /* free current conn */
	  ipmq_tmp = ipmq_connp->next;
	  IPM_FREE(ipmq_connp);
	  (*connCntPtr)--;
	  ipmq_connp = ipmq_tmp;
	  
	  /* find nxt conn on queue */
	  if (!ipmq_connp) {
		  /* we have traversed the list on current 
		   * bucket. Needs to find next valid ipmq_connp.
		   */
		  connQ[i] = 0;	/* clear the bucket */
		  for (i=(hashIdx + 1); i<IPM_HASH_SIZE; i++) {
			  if ((ipmq_connp = connQ[i])) {
				  hashIdx = i;
				  break;
			  }
		  }
	  }

	  ipm_connp++;
	  size += sizeof(ipm_mconn_t);
	  count++;
  }

  /* send out msg */
  ipm_mchdrp->count = count;
  (void)ipm_FillIpmMsgHdr(ipm_hdrp, mtype, size);
  if (ipm_SendMsg(nfxSscSlotId, NFX_SSC_UNIX, (void*)ipm_hdrp, size, 0)
	  != NFX_OK)
	  ipm_stats.err_fwd++;
  else
	  ipm_stats.info_fwd++;

  /* more to send */
  if (ipmq_connp)
	  goto sendmore;

  /* chk queue and stats match */
  if (*connCntPtr) 
	IPM_LOGERR(IPM_ERR_QUEUE);
	
out:
  if (connQ == ipm_TcpConnQ)
	{
	  /* process UDP queue */
	  connQ = ipm_UdpConnQ;
	  connCntPtr = &(ipm_stats.connUdpNum);
	  mtype = IPM_MSG_UDP_CONN;
	  goto nextqueue;
	}

  if (ipm_hdrp)
	IPM_FREE(ipm_hdrp);

  return;
}

/* 
 *-----------------------------------------------------------------
 *
 * Name: ipm_GetL2ConnEntry
 *
 * Description: get requested entry from layer2 connection table
 *
 * Created by:	Su Wang
 *
 * Date Created: 05/28/2001
 *
 *-----------------------------------------------------------------
 */
static int32							/* NFX_OK, NFX_ERR */
ipm_GetL2ConnEntry(uint32		ip,
				   uint32		*msgsize,
				   ipm_mhdr_t	**ret)
{
	int				count;
	ipm_mhdr_t		*ipm_hdrp;
	ipm_mstat_req_t	*ipm_reqp;
	ipm_arp_t		*ipm_arpp;
	l2conn_ent_t	*connEntPtr;
	int				i;
	
	if (ip)		{				/* want just one entry */
		connEntPtr = arpS_FindEntry(ip);
		if (connEntPtr)
			count = 1;
		else 
			count = 0;
	}
	else {						/* want whole table */
		connEntPtr = arpsCfgPtr->l2ConnTblHead;
		count = arpsCfgPtr->count;
	}
	
	/* calculate size of return msg and alloc */
	*msgsize = sizeof(ipm_mhdr_t) + sizeof(ipm_mstat_req_t) +
		count*sizeof(ipm_arp_t);
	
	ipm_hdrp = (ipm_mhdr_t*)eee_ramAlloc(*msgsize);
	if (!ipm_hdrp)
		return NFX_ERR;
	bzero(ipm_hdrp, *msgsize);
	
	/* setup headers */
	ipm_hdrp->ipmh_mtype = IPM_MSG_L2CONN_ENT_REQ;
	ipm_hdrp->ipmh_size = *msgsize;
	ipm_hdrp->ipmh_flags = 0;
	
	ipm_reqp = (ipm_mstat_req_t*)(ipm_hdrp+1);
	ipm_reqp->count = count;
	ipm_reqp->req.ip = ip;
	

	/* copy connection table entries */
	ipm_arpp = (ipm_arp_t*)(ipm_reqp+1);
	for (i=0; ((i<count) && connEntPtr); i++) {
		ipm_arpp->arp_addr = connEntPtr->l2ConnEnt_ip;
		memcpy(ipm_arpp->arp_ifname, connEntPtr->l2ConnEnt_ifname, ARPS_STR_SIZE);
		
		ipm_arpp++;
		connEntPtr = connEntPtr->l2ConnEnt_next;
	}
	
	*ret = ipm_hdrp;
	return NFX_OK;
}
