/* vi:ts=4
 *-----------------------------------------------------------------
 *
 * Name: ipm.h
 *
 * RCS:  $Id: ipm.h,v 1.19 2002/10/24 18:05:59 swang Exp $
 *
 * Copyright (C) 2001, Agile Storage, Inc.
 *
 * Description:
 *
 * Created by:	Su Wang
 *
 * Date Created: 10/11/2001
 *
 *-----------------------------------------------------------------
 */

#ifndef _IPM_H
#define _IPM_H

#include "../sm-ipmd-lib/ipmd-lib.h"
#if !defined(PROM)
#include "exports/elog-api.h"
#endif /* PROM */

/* misc. definition */
extern	uint32	ipm_eee_src;
extern  int		ipm_appid;

#define IPM_MSG_MAX_SIZE		1024 /* max. msg size rcved by ipmd */
#define IPM_STATSMEM_SIZE		1024*1024 /* stats memory space size */
#define IPM_MAX_CONN_ONCHASSIS	4096*4 /* max connection allowed */

#define IPM_IP_PROTO_TCP		6
#define IPM_IP_PROTO_UDP		17

#define ROUNDUP(a) (1 + (((a) - 1) | (sizeof(uint32) - 1)))

#define GET_CPU_FROM_ULINK(_link)				(0)
#define GET_LINK_FROM_ULINK(_link)				(_link)
#define GET_ULINK_FROM_CPULINK(_cpu, _link)		(_link)

/*
 * IPM msg types and structures
 */
/* device msg */
#define IPM_MSG_INTF_REQ		IPMD_MSG_BASE+1 /* request intf (layer2) info */
#define IPM_MSG_INTF_RESP		IPMD_MSG_BASE+2 /* respnose intf(layer2) info */
#define IPM_MSG_INTF_NEW		IPMD_MSG_BASE+3	/* new interface detected */
#define IPM_MSG_INTF_UP			IPMD_MSG_BASE+4 /* interface op state up */
#define IPM_MSG_INTF_DOWN		IPMD_MSG_BASE+5	/* interface op state down */

/* adminstrative msg */
#define IPM_MSG_INTF_ENA		IPMD_MSG_BASE+7 /* enable intf state  */
#define IPM_MSG_INTF_DIS		IPMD_MSG_BASE+8 /* disable intf state */

#define IPM_MSG_IP_ADD			IPMD_MSG_BASE+11 /* add ip intf  */
#define IPM_MSG_IP_DEL			IPMD_MSG_BASE+12 /* del ip intf */

#define IPM_MSG_VIF_CREAT		IPMD_MSG_BASE+13 /* create vif */
#define IPM_MSG_VIF_DEL			IPMD_MSG_BASE+14 /* delete vif */
#define IPM_MSG_VIF_ADDLNK		IPMD_MSG_BASE+15 /* add link to vif */
#define IPM_MSG_VIF_RMLNK		IPMD_MSG_BASE+16 /* remove link from vif */
#define IPM_MSG_VIP_ADD			IPMD_MSG_BASE+17 /* add IP to vif */
#define IPM_MSG_VIP_DEL			IPMD_MSG_BASE+18 /* remove ip from vif */
#define IPM_MSG_VIF_PREF		IPMD_MSG_BASE+19 /* specify prefer link */
#define IPM_MSG_VIF_EVNT		IPMD_MSG_BASE+20 /* event from txrx to ssc */


#define IPM_MSG_RT_ADD			IPMD_MSG_BASE+21 /* add route */
#define IPM_MSG_RT_DEL			IPMD_MSG_BASE+22 /* del route */

#define IPM_MSG_ARP_ADD			IPMD_MSG_BASE+31 /* add arp */
#define IPM_MSG_ARP_DEL			IPMD_MSG_BASE+32 /* del arp */

#define IPM_MSG_INTF_SETMTU		IPMD_MSG_BASE+33 /* set port mtu */
#define IPM_MSG_INTF_AUTOEN		IPMD_MSG_BASE+34 /* auto. negociate on */
#define IPM_MSG_INTF_AUTODIS	IPMD_MSG_BASE+35 /* auto. negociate off */

#define IPM_MSG_RESTART			IPMD_MSG_BASE+49 /* restart ipmd */
#define IPM_MSG_SHUTDOWN		IPMD_MSG_BASE+50 /* shutdown IPMD, remove shared memroy */


/* information msg */
#define IPM_MSG_INTF_STATS		IPMD_MSG_BASE+51 /* intf stats */
#define IPM_MSG_IP_STATS		IPMD_MSG_BASE+52 /* card ip stats */
#define IPM_MSG_TCP_STATS		IPMD_MSG_BASE+53 /* card tcp stats */
#define IPM_MSG_UDP_STATS		IPMD_MSG_BASE+54 /* card udp stats */
#define IPM_MSG_ICMP_STATS		IPMD_MSG_BASE+55 /* card icmp stats */
#define IPM_MSG_TCP_CONN		IPMD_MSG_BASE+56 /* TCP connection table */
#define IPM_MSG_UDP_CONN		IPMD_MSG_BASE+57 /* UDP connection table */

/* request for system info */
#define IPM_MSG_INTF_STATS_REQ	IPMD_MSG_BASE+61 /* intf stats */
#define IPM_MSG_IP_STATS_REQ	IPMD_MSG_BASE+62 /* card ip stats */
#define IPM_MSG_TCP_STATS_REQ	IPMD_MSG_BASE+63 /* card tcp stats */
#define IPM_MSG_UDP_STATS_REQ	IPMD_MSG_BASE+64 /* card udp stats */
#define IPM_MSG_ICMP_STATS_REQ	IPMD_MSG_BASE+65 /* card icmp stats */
#define IPM_MSG_TCP_CTBL_REQ    IPMD_MSG_BASE+66 /* request for tcp conn. table */
#define IPM_MSG_UDP_CTBL_REQ    IPMD_MSG_BASE+67 /* request for udp conn. table */
#define IPM_MSG_TCP_NXTC_REQ	IPMD_MSG_BASE+68 /* request for next tcp conn.*/
#define IPM_MSG_UDP_NXTC_REQ    IPMD_MSG_BASE+69 /* request for next udp conn. */
#define IPM_MSG_TCP_CONN_REQ	IPMD_MSG_BASE+70 /* request for tcp conn */
#define IPM_MSG_UDP_CONN_REQ    IPMD_MSG_BASE+71 /* request for udp conn */
#define IPM_MSG_TCP_KCTBL_REQ   IPMD_MSG_BASE+72 /* request for tcp conn. table from kern */
#define IPM_MSG_UDP_KCTBL_REQ   IPMD_MSG_BASE+73 /* request for udp conn. table from kern */
#define IPM_MSG_IFTBL_REQ		IPMD_MSG_BASE+74 /* request for interface table */
#define IPM_MSG_IPTBL_REQ		IPMD_MSG_BASE+75 /* request for IP table */
#define IPM_MSG_RTTBL_REQ		IPMD_MSG_BASE+76 /* request for RT table */ 
#define IPM_MSG_ARPTBL_REQ		IPMD_MSG_BASE+77 /* request for RT table */
#define IPM_MSG_ROOT_REQ		IPMD_MSG_BASE+78 /* request to stats mem root struct etc */
#define IPM_MSG_L2CONN_ENT_REQ  IPMD_MSG_BASE+79 /* request for lay2 connection table entry */
#define IPM_MSG_VIFTBL_REQ		IPMD_MSG_BASE+80 /* request for virtual interface */

/* 
 *-----------------------------------------------------------------
 * IPM message format:
 * 
 * IPM_MSG_INTF_XXX:
 *		ipm_mhdr_t + ipm_mintf_t
 *
 * IPM_MSG_IP_XXX:
 *		ipm_mhdr_t + ipm_mintf_t + ipm_mip_t
 *
 * IPM_MSG_RT_XXX:
 *		ipm_mhdr_t + ipm_mrt_t
 *
 * IPM_MSG_ARP_XXX:
 *		ipm_mhdr_t + ipm_marp_t
 *
 * IPM_MSG_XXX_STATS:
 *		ipm_mhdr_t + ipm_xxxstat_t
 * 
 * IPM_MSG_XXX_CONN:
 *		ipm_mhdr_t + ipm_mchdr_t + count*ipm_conn_t
 *
 * IPM_MSG_XXX_STATS_REQ:
 *		ipm_mhdr_t + ipm_mstat_req_t
 *
 * IPM_MSG_VIF_XXX:
 *		ipm_mhdr_t + ipm_mvif_t
 *-----------------------------------------------------------------
 */

typedef struct {
  uint32		ipmh_mtype;		/* msg type, */
  uint32		ipmh_size;		/* size */
#define IPM_MSG_CMD_OK		0x1		
#define IPM_MSG_CMD_NOBUF	0x2	/* ret buf is not large enough */
#define IPM_MSG_CMD_NREPLY	0x4	/* cmd need no rely */
#define IPM_MSG_CMD_NOINTF	0x8
#define IPM_MSG_CMD_NORT	0x10
#define IPM_MSG_CMD_DUPRT	0x20
#define IPM_MSG_CMD_TIMEOUT	0x40
#define IPM_MSG_CMD_DUPIP	0x80
#define IPM_MSG_CMD_NOIP	0x100
#define IPM_MSG_CMD_NOGW	0x200

#define IPM_MSG_CMD_NOVIF	0x400 /* vif not found */
#define IPM_MSG_CMD_DUPVIF	0x800 /* dup vif found */
#define IPM_MSG_CMD_HASIP_VIF	0x1000 /* vif has ip, err when del vif */
#define IPM_MSG_CMD_ACTLINK_VIF 0x2000 /* link is act link, err when rm link */
#define IPM_MSG_CMD_BADMODE_VIF	0x4000 /* bad mode to perform cmd */
#define IPM_MSG_CMD_NOLINK_VIF	0x8000 /* vif has no link, err when add ip */
#define IPM_MSG_CMD_MANYVIF		0x10000	/* too many vifs */

  uint32		ipmh_flags;		/* for now use to indicate a cmd result */
} ipm_mhdr_t;

typedef struct {				/* msg to send intf info */
  ushort16		if_dtype;		/* device type */
  ushort16		if_slot;
  ushort16		if_cpu;		
  ushort16		if_link;		/* link num. with in slot */
  ushort16		if_flags;		/* state flags */
  ushort16		if_dhdrlen;		/* device specific hdr len */
  ushort16		if_daddrlen;	/* device specific addr len */
  ushort16		if_mtusize;		/* interface mtu */
  char			if_mac[8];		/* 32 bits aligned */
} ipm_mintf_t;

#if defined(LINK_AGG)
typedef ipm_vif_t	ipm_mvif_t;
#endif /* LINK_AGG */

typedef struct {				/* msg to send ip info */
  uint32		ip_addr;
  uint32		ip_mask;	
} ipm_mip_t;	

typedef struct {				/* msg to send rt info */
  uint32		rt_addr;
  uint32		rt_mask;
  uint32		rt_nhop;
} ipm_mrt_t;

typedef struct {				/* msg to send arp info */
  uint32		arp_addr;
  char			arp_mac[8];		/* 32 bits aligned */
} ipm_marp_t;

typedef struct 
{
	int				count;		/* how many record is being cpied */
	union 
	{
		char		ifname[IPM_STR_SIZE];
		uint32		slot;
		ipm_conn_t	conn;		/* index==>conn.laddr, incount==>conn.faddr */
		uint32		ip;			/* used for requesting to lay2 connection table */
	} req;
} ipm_mstat_req_t;


/* statis msg types are defined in ipmd-lib.h because the stats structure */
/* need to be understood by ipmd-lib.h application */

/* conn msg type */
typedef struct {
  uint32		count;
} ipm_mchdr_t;

typedef struct {
#define	IPM_ADD			1		/* conn. added */
#define IPM_DEL			2		/* conn. removed */
#define IPM_CHGE_STATE   3		/* conn. state changed */
  uint32		cmd;
  ipm_conn_t	conn;
} ipm_mconn_t;


/* 
 * memory allocation/deallocation macro 
 */
#if defined(NFP_TXRX)
#define IPM_ALLOC(_size)		(void*)eee_ramAlloc(_size)	
#define IPM_FREE(_ptr)			eee_ramDealloc(_ptr)	
#else
#define IPM_ALLOC(_size)		(void*)malloc(_size)	
#define IPM_FREE(_ptr)			free(_ptr)	


/* special allocation for send/rcv msg through NCM */
#define IPM_NCM_HDR				(sizeof(ncmMsg_t) - 8)
//#define IPM_NCM_HDR				0

#define IPM_ALLOC_NCM(_ptr, _size)	\
{									\
	_ptr = (void*)(((char*)malloc(IPM_NCM_HDR + _size)) + IPM_NCM_HDR);	\
	if ((uint32)_ptr == IPM_NCM_HDR)		\
		_ptr = NULL;				\
} \

#define IPM_FREE_NCM(_ptr)			\
(free((((char*)_ptr) - IPM_NCM_HDR)))	
#endif /* NFP_TXRX */

/* 
 * ipm errors, use part of bsd space 
 */
#if defined(PROM)
#define IPM_LOGERR( code )      printf("ipm error: file %s, line %d\n", __FILE__, __LINE__)
#else
#define _IPM_ERR_STRING_        ("ipm error: file %s, line %d\n", __FILE__, __LINE__)
#define IPM_LOGERR( code )		E_LOG(class_1, LOG_ERR, ipm_appid, 0, 0, 0, _IPM_ERR_STRING_)
#endif /* PROM */

#if defined(NFP_TXRX)
#define IPM_PANIC( )			{ *(char *)0 = 1; }
#else
#define IPM_PANIC( )
#endif /* NFP_TXRX */ 

#define IPM_ERROR_BASE			(BSD_ERROR_BASE + 0xE0)
#define IPM_ERR_ALLOC			(IPM_ERROR_BASE + 1)
#define IPM_ERR_QUEUE			(IPM_ERROR_BASE + 2)
#define IPM_ERR_MTYPE			(IPM_ERROR_BASE + 3)
#define IPM_ERR_SEND			(IPM_ERROR_BASE + 4)
#define IPM_ERR_RCV				(IPM_ERROR_BASE + 5)
#define IPM_ERR_KERN			(IPM_ERROR_BASE + 6)
#define IPM_ERR_STATS			(IPM_ERROR_BASE + 7)
#define IPM_ERR_BUF				(IPM_ERROR_BASE + 8)
#define IPM_ERR_VIF				(IPM_ERROR_BASE + 9)

/* 
 * ipm stats 
 */
typedef struct {
  uint32		adm_fwd;		/* adminstrative msg stats */
  uint32		adm_rcv;
  uint32		dev_fwd;		/* device msg stats */
  uint32		dev_rcv;
  uint32		info_fwd;		/* info msg stats */
  uint32		info_rcv;
  uint32		req_rcv;
  uint32		err_fwd;		/* err stats */
  uint32		err_rcv;
  uint32		connTcpNum;		/* num. of tcp conn updt queued */
  uint32		connUdpNum;		/* num. of udp conn updt queued */	
} ipm_stats_t;
extern ipm_stats_t		ipm_stats;

#if defined(NFP_TXRX)
/* connection update queue on txrx */
#define IPM_HASH_SIZE		256
#define IPM_HASH(addr, port)	\
	(((addr&0xff) + ((addr>>8)&0xff) + ((addr>>16)&0xff) + ((addr>>24)&0xff) +	\
	  (port&0xff) + ((port>>8)&0xff)) & 0xff)

/*  internal conn type */
typedef struct udptconn {
  struct udptconn	*next;	
  ipm_mconn_t		conn;
} ipm_updtconn_t;

extern ipm_updtconn_t*	ipm_TcpConnQ[];
extern ipm_updtconn_t*	ipm_UdpConnQ[];

extern int				ipm_Updt_Trig;
#endif /* NFX_TXRX */

/* utility functions */
int32 ipm_FillIpmMsgHdr(ipm_mhdr_t*, uint32, int);
int32 ipm_FillIpmMsgIp(ipm_mip_t*, uint32, uint32);
int32 ipm_FillIpmMsgRt(ipm_mrt_t*, uint32, uint32, uint32);
int32 ipm_FillIpmMsgArp(ipm_marp_t*, uint32, char*, int);
void  ipm_LogErr(char*, int32, int32);
void  ipm_PrintStats(ipm_stats_t*);
void  ipm_ClearStats(ipm_stats_t*);
void  bsd_ShowPkt(char*, int);

#if defined(SSC)
int32 ipmd_ParseIfName(char*, int*, int*, int*);
int32 ipmd_FillIpmMsgIntfId(ipm_mintf_t*, char*);
int32 ipmd_SendMsgByNcm(int, void*, uint32, char*, char*, void*, int);
int32 ipmd_SendMsg(void*, uint32,int, int, char*, void*, int, int, int);
#endif /* SSC */



/*
 * Virtual link related constants
 */


#endif /* _IPM_H */
