/* vi:ts=4
 *-----------------------------------------------------------------
 *
 * Name: ipmd-stats.c
 *
 * RCS:  $Id: ipmd-kern.c,v 1.15 2002/10/29 01:26:53 swang Exp $
 *
 * Copyright (C) 2001, Agile Storage, Inc.
 *
 * Description: api interact with bsd kernel 
 *
 * Created by:	Su Wang
 *
 * Date Created: 02/12/2002
 *
 *-----------------------------------------------------------------
 */
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/socket.h>
#include <sys/param.h>
#include <sys/ioctl.h>
#include <sys/sysctl.h>
#include <net/if.h>
#include <net/route.h>
#include <netinet/in.h>
#include <netinet/if_ether.h>
#include <arpa/inet.h>
#include <sys/types.h>
#include <sys/shm.h> 
#include <sys/ipc.h>
#include <sys/types.h>
#include <sys/wait.h>    
#if defined(SSC_x86) || defined(SSC_mips)
#include <net/if_dl.h>
#include <netinet/in_systm.h>
#include <netinet/ip.h>
#include <netinet/in_pcb.h>
#include <netinet/ip_var.h>
#include <netinet/ip_icmp.h>
#include <netinet/icmp_var.h>
#include <netinet/tcp.h>
#include <netinet/tcp_seq.h>
#include <netinet/tcp_fsm.h>
#include <netinet/tcp_timer.h>
#include <netinet/tcp_var.h>
#include <netinet/udp.h>
#include <netinet/udp_var.h>
#include <fcntl.h>
#include <kvm.h>
#include <limits.h>
#endif /* SSC_x86 SSC_mips */

#include <nfx-error.h>
#include <nfx-types.h>
#include <nfx-defs.h>
#include <nfx-ports.h>
#include <exports/anpssc-api.h>
#include <exports/ipmd-lib.h>
#include <exports/ipm-api.h>
#include "../sm-ipm/ipm.h"
#include "ipmd-kern.h"
#include "ipmd-vif.h"

#include <exports/opt-api.h>

#define IPM_STR_SIZE	24	
#define IWANTIP			1
#define IWANTIF			2
#define IWANTRT			3
#define IWANTARP		4
#define IWANTIF_TXRX	5

#define min(a,b) ((a) <= (b) ? (a) : (b))
#define max(a,b) ((a) >= (b) ? (a) : (b))
static char*	if_indextoname1(unsigned int, char*);

/* 
 *-----------------------------------------------------------------
 *
 * Name: ipmd_AddKernIp
 *
 * Description: directly interact to kernel to add an IP intf.
 *
 * Created by:	Su Wang
 *
 * Date Created: 11/28/2001
 *
 *-----------------------------------------------------------------
 */
int32					/* NFX_OK, NFX_ERR */
ipmd_AddKernIp(char		*ifname,
			   uint32	ip,
			   uint32	mask
			   )
{
  int	pid;
  char  ip_str[IPM_STR_SIZE];
  char	mask_str[IPM_STR_SIZE];
  int	status;


  if (!ifname)
	return NFX_ERR;
  strncpy(ip_str, inet_ntoa(*((struct in_addr*)&ip)), IPM_STR_SIZE);
  strncpy(mask_str, inet_ntoa(*((struct in_addr*)&mask)), IPM_STR_SIZE);
  if ((pid = fork()) == 0) {
	  /* child process do exec */
	  execl("/sbin/ifconfig",
			"ifconfig", ifname, "alias", ip_str,  "netmask",  mask_str, 0);
	  perror("execl add ip failed");
	  exit(1);	  
  }
  else {
	  waitpid(pid, &status, 0); 
  }
  
  return NFX_OK;
}

/* 
 *-----------------------------------------------------------------
 *
 * Name: ipmd_DelKernIp
 *
 * Description: directly interact to kernel to delete an IP intf.
 *
 * Created by:	Su Wang
 *
 * Date Created: 11/28/2001
 *
 *-----------------------------------------------------------------
 */
int32							/* NFX_OK, NFX_ERR */
ipmd_DelKernIp(char		*ifname,
			   uint32	 ip
			   )
{
  int	pid;
  char	ip_str[24];
  int	status;

  if (!ifname)
	return NFX_ERR;

  strncpy(ip_str, inet_ntoa((*(struct in_addr*)&ip)), IPM_STR_SIZE);
  if ((pid = fork()) == 0)
	{
	  /* child process do exec */
	  execl("/sbin/ifconfig",
			"ifconfig", ifname, "-alias", ip_str, 0);
	  perror("execl del ip failed");
	  exit(1);
	}
  else
	  waitpid(pid, &status, 0); 

  return NFX_OK;  
}


/* 
 *-----------------------------------------------------------------
 *
 * Name: ipmd_AddKernRt
 *
 * Description: directly interact to kernel to add a rt..
 *
 * Created by:	Su Wang
 *
 * Date Created: 11/28/2001
 *
 *-----------------------------------------------------------------
 */
int32					/* NFX_OK, NFX_ERR */
ipmd_AddKernRt(uint32	ip,
			   uint32	mask,
			   uint32	nhop)
{
  int			pid;
  char			ip_str[IPM_STR_SIZE];
  char			mask_str[IPM_STR_SIZE];
  char			nhop_str[IPM_STR_SIZE];
  int			status;

  strncpy(ip_str, inet_ntoa(*(struct in_addr*)&ip), IPM_STR_SIZE);
  strncpy(mask_str, inet_ntoa(*(struct in_addr*)&mask), IPM_STR_SIZE);
  strncpy(nhop_str, inet_ntoa(*(struct in_addr*)&nhop), IPM_STR_SIZE);

  if ((pid = fork()) == 0)
	{
	  if (ntohl(mask) & 1)	/* host */
		execl("/sbin/route",
			  "route", "add", ip_str, nhop_str, 0);
	  else
		execl("/sbin/route",
			  "route", "add", ip_str, "-netmask", mask_str, nhop_str, 0);
	  perror("execl rt add failed");
	  exit(1);
	}
  else
	{
	  if (!ip && !mask)	/* It's the default route */
		{
			modify_readonly_file("/etc/mygate", string_to_file, nhop_str);
		}
	  waitpid(pid, &status, 0); 
	}

  return NFX_OK;
}

/* 
 *-----------------------------------------------------------------
 *
 * Name: ipmd_DelKernRt
 *
 * Description: directly interact to kernel to delete a rt..
 *
 * Created by:	Su Wang
 *
 * Date Created: 11/28/2001
 *
 *-----------------------------------------------------------------
 */
int32							/* NFX_OK, NFX_ERR */
ipmd_DelKernRt(uint32	ip,
			   uint32	mask,
			   uint32	nhop)
{
  int			pid;
  char			ip_str[IPM_STR_SIZE];
  char			mask_str[IPM_STR_SIZE];
  char			nhop_str[IPM_STR_SIZE];
  int			status;

  strncpy(ip_str, inet_ntoa(*(struct in_addr*)&ip), IPM_STR_SIZE);
  strncpy(mask_str, inet_ntoa(*(struct in_addr*)&mask), IPM_STR_SIZE);
  strncpy(nhop_str, inet_ntoa(*(struct in_addr*)&nhop), IPM_STR_SIZE);  

  if ((pid = fork()) == 0)
	{
	  /* child process do exec */
	  if (ntohl(mask) & 1)	/* host */
		execl("/sbin/route",
			  "route", "delete", ip_str, nhop_str, 0);
	  else
		execl("/sbin/route",
			  "route", "delete", ip_str, "-netmask", mask_str, nhop_str, 0);
	  perror("execl rt delete failed");
	  exit(1);
	}
  else
	{
	  if (!ip && !mask)	/* It's the default route */
		{
			modify_readonly_file("/etc/mygate", unlink, NULL);
		}
	  waitpid(pid, &status, 0); 
	}

  return NFX_OK;
}


/* 
 *-----------------------------------------------------------------
 *
 * Name: ipmd_AddKernArp
 *
 * Description: directly interact to kernel to add an arp .
 *
 * Created by:	Su Wang
 *
 * Date Created: 11/28/2001
 *
 *-----------------------------------------------------------------
 */
int32							/* NFX_OK, NFX_ERR */
ipmd_AddKernArp(uint32	ip,
				char	*mac
				)
{
  int	pid;
  char	ip_str[IPM_STR_SIZE];
  char  mac_str[IPM_STR_SIZE];
  int	status;

  strncpy(ip_str, inet_ntoa(*(struct in_addr*)&ip), IPM_STR_SIZE);
#if defined(SSC_x86) || defined(SSC_mips)
  strncpy(mac_str, ether_ntoa((struct ether_addr*)mac), IPM_STR_SIZE);
#else
  sprintf(mac_str, "%02X:%02X:%02X:%02X:%02X:%02X", 0xff&mac[0], 0xff&mac[1], 
		  0xff&mac[2], 0xff&mac[3], 0xff&mac[4], 0xff&mac[5]);
#endif /* SSC_x86 SSC_mips */
  
	
  if ((pid = fork()) == 0)
	{
		execl("/usr/sbin/arp",
			  "arp","-s",  ip_str, mac_str, 0);
		perror("execl arp add failed");
		exit(1);
	}
  else
	  waitpid(pid, &status, 0); 

  return NFX_OK;
}

/* 
 *-----------------------------------------------------------------
 *
 * Name: ipmd_DelKernArp
 *
 * Description: directly interact to kernel to delete an arp .
 *
 * Created by:	Su Wang
 *
 * Date Created: 11/28/2001
 *
 *-----------------------------------------------------------------
 */
int32							/* NFX_OK, NFX_ERR */
ipmd_DelKernArp(uint32 ip)
{
   int	pid;
   char ip_str[IPM_STR_SIZE];
   int	status;

   strncpy(ip_str, inet_ntoa(*(struct in_addr*)&ip), IPM_STR_SIZE);
  if ((pid = fork()) == 0)
	{
		  execl("/usr/sbin/arp",
				"arp","-d",  ip_str,0);
		  perror("execl arp delete failed");
		  exit(1);
	}
  else
	  waitpid(pid, &status, 0); 

  return NFX_OK;
}

/* 
 *-----------------------------------------------------------------
 *
 * Name: ipmd_EnableKernIntf
 *
 * Description: directly interact to kernel to enable a device interface.
 *
 * Created by:	Su Wang
 *
 * Date Created: 11/28/2001
 *
 *-----------------------------------------------------------------
 */
int32							/* NFX_OK, NFX_ERR */
ipmd_EnableKernIntf(char	*ifname)
{
  int	pid;
  int	status;

  if (!ifname)
	return NFX_ERR;

  if ((pid = fork()) == 0)
	{
	  /* child process do exec */
	  execl("/sbin/ifconfig",
			"ifconfig", ifname, "up", 0);

	  perror("execl enable intf failed");
	  exit(1);
	}
  else
	  waitpid(pid, &status, 0); 

  return NFX_OK;
}

/* 
 *-----------------------------------------------------------------
 *
 * Name: ipmd_DisableKernIntf
 *
 * Description: directly interact to kernel to disable a device interface.
 *
 * Created by:	Su Wang
 *
 * Date Created: 11/28/2001
 *
 *-----------------------------------------------------------------
 */
int32							/* NFX_OK, NFX_ERR */
ipmd_DisableKernIntf(char	*ifname)
{
  int	pid;
  int	status;

  if (!ifname)
	return NFX_ERR;

  if ((pid = fork()) == 0)
	{
	  /* child process do exec */
	  
	  execl("/sbin/ifconfig",
			"ifconfig", ifname, "down", 0);

	  perror("execl enable intf failed");
	  exit(1);
	}
  else
	  waitpid(pid, &status, 0); 

  return NFX_OK;
}

#if defined(SSC_x86) || defined(SSC_mips)
static int32
get_table(kvm_t*		kvmd, 
		  uint32		offset,
		  int			which,
		  char			*buf,
		  int			size,
		  int			*count,
		  int			*needed,
		  void			*extra)
{
	struct inpcbtable	table;
	struct inpcb		*head, *next, *prev;
	struct inpcb		inpcb;
	struct	tcpcb		tcpcb;
	ipm_conn_t			*ipm_connp;
	

	*count = 0;
	*needed = 0;
	
	/* get head of in_pcb table */
	if (kvm_read(kvmd, offset, &table, sizeof(table)) != sizeof(table)) {
		IPM_LOGERR(IPM_ERR_KERN);
		return IPM_GET_ERR;
	}
	prev = head = 
		(struct inpcb *)&((struct inpcbtable *)offset)->inpt_queue.cqh_first;
	next = table.inpt_queue.cqh_first;
	

	/* loop through all pcb blocks */
	while (next != head) {

		/* get each pcb */
		if (kvm_read(kvmd, (uint32)next, (char *)&inpcb, sizeof(inpcb)) 
			!= sizeof(inpcb)) {
			IPM_LOGERR(IPM_ERR_KERN);
			return IPM_GET_ERR;
		}
			
		if (inpcb.inp_queue.cqe_prev != prev) {
			printf("get_table: prev_pcb, next_pcb not matching\n");
			return IPM_GET_OK;
		}

		prev = next;
		next = inpcb.inp_queue.cqe_next;
		
		/* copy to buffers */
		if (size < sizeof(ipm_conn_t)) {
			*needed = (*count)*2*sizeof(ipm_conn_t); /* no more buffer space */
			return IPM_GET_NO_BUF;
		}
		
		ipm_connp = (ipm_conn_t*)buf;
		ipm_connp->conn_lAddr = inpcb.inp_laddr.s_addr;
		ipm_connp->conn_lPort = inpcb.inp_lport;
		ipm_connp->conn_rAddr = inpcb.inp_faddr.s_addr;
		ipm_connp->conn_rPort = inpcb.inp_fport;
		
		/* if tcp, also getting its state */
		if (which == TCB_TABLE) {
			if (kvm_read(kvmd, (uint32)inpcb.inp_ppcb, (char*)&tcpcb, sizeof(tcpcb))
						 != sizeof(tcpcb)) 
			{
				IPM_LOGERR(IPM_ERR_KERN);
				return IPM_GET_ERR;
			}
				
			ipm_connp->conn_state = tcpcb.t_state;
		}

		/* advance buf ptr */
		buf += sizeof(ipm_conn_t);
		size -= sizeof(ipm_conn_t);
		(*count)++;
	}

	return IPM_GET_OK;
}

static int32
get_stats(kvm_t*		kvmd, 
		  uint32		offset,
		  int			which,
		  char			*buf,
		  int			size,
		  int			*count,
		  int			*needed,
		  void			*extra)
{
	struct tcpstat		tcpstat;
	struct udpstat		udpstat;
	struct ipstat		ipstat;
	struct icmpstat		icmpstat;
	ipm_tcpstat_t		*ipm_tcpstatp;
	ipm_udpstat_t		*ipm_udpstatp;
	ipm_ipstat_t		*ipm_ipstatp;
	ipm_icpstat_t		*ipm_icmpstatp;
	int					rec_size;
	char				*rec_ptr;

	*count = 0;
	*needed = 0;
	
	/* intialize for various stat get */
	switch (which) {
	case TCP_STAT:
		rec_size	= sizeof(struct tcpstat);
		rec_ptr		= (char*)&tcpstat;
		ipm_tcpstatp= (ipm_tcpstat_t*)buf;
		break;
		
	case UDP_STAT:
		rec_size	= sizeof(struct udpstat);
		rec_ptr		= (char*)&udpstat;
		ipm_udpstatp= (ipm_udpstat_t*)buf;
		break;	
	
	case IP_STAT:
		rec_size	= sizeof(struct ipstat);
		rec_ptr		= (char*)&ipstat;
		ipm_ipstatp= (ipm_ipstat_t*)buf;
		break;	
		
	case ICMP_STAT:
		rec_size	= sizeof(struct icmpstat);
		rec_ptr		= (char*)&icmpstat;
		ipm_icmpstatp= (ipm_icpstat_t*)buf;
		break;	
		
	default:
		return IPM_GET_ERR;
	}
	
	
	if (size <= rec_size) {
		*needed = rec_size;
		return IPM_GET_NO_BUF;
	}
	
	/* get stats */
	if (kvm_read(kvmd, (uint32)offset, rec_ptr, rec_size)
		== -1) {
		IPM_LOGERR(IPM_ERR_KERN);
		return IPM_GET_ERR;	
	}


	switch (which) {
	case TCP_STAT:
		ipm_tcpstatp->tcps_card			= 0;
		ipm_tcpstatp->tcps_connattempt	= tcpstat.tcps_connattempt;
		ipm_tcpstatp->tcps_accepts		= tcpstat.tcps_accepts;
		ipm_tcpstatp->tcps_connects		= tcpstat.tcps_connects;
		ipm_tcpstatp->tcps_drops		= tcpstat.tcps_drops;
		ipm_tcpstatp->tcps_conndrops	= tcpstat.tcps_conndrops;
		ipm_tcpstatp->tcps_closed		= tcpstat.tcps_closed;
		ipm_tcpstatp->tcps_segstimed	= tcpstat.tcps_segstimed;
		ipm_tcpstatp->tcps_rttupdated	= tcpstat.tcps_rttupdated;
		ipm_tcpstatp->tcps_delack		= tcpstat.tcps_delack;
		ipm_tcpstatp->tcps_timeoutdrop	= tcpstat.tcps_timeoutdrop;
		ipm_tcpstatp->tcps_rexmttimeo	= tcpstat.tcps_rexmttimeo;
		ipm_tcpstatp->tcps_persisttimeo = tcpstat.tcps_persisttimeo;
		ipm_tcpstatp->tcps_keeptimeo	= tcpstat.tcps_keeptimeo;
		ipm_tcpstatp->tcps_keepprobe	= tcpstat.tcps_keepprobe;
		ipm_tcpstatp->tcps_keepdrops	= tcpstat.tcps_keepdrops;
		ipm_tcpstatp->tcps_rcvtotal		= tcpstat.tcps_rcvtotal;
		ipm_tcpstatp->tcps_rcvpack		= tcpstat.tcps_rcvpack;
		ipm_tcpstatp->tcps_rcvbyte		= tcpstat.tcps_rcvbyte;
		ipm_tcpstatp->tcps_rcvbadsum	= tcpstat.tcps_rcvbadsum;
		ipm_tcpstatp->tcps_rcvbadoff	= tcpstat.tcps_rcvbadoff;
		ipm_tcpstatp->tcps_rcvshort		= tcpstat.tcps_rcvshort;
		ipm_tcpstatp->tcps_rcvduppack	= tcpstat.tcps_rcvduppack;
		ipm_tcpstatp->tcps_rcvdupbyte	= tcpstat.tcps_rcvdupbyte;
		ipm_tcpstatp->tcps_rcvpartduppack	= tcpstat.tcps_rcvpartduppack;
		ipm_tcpstatp->tcps_rcvpartdupbyte	= tcpstat.tcps_rcvpartdupbyte;
		ipm_tcpstatp->tcps_rcvoopack	= tcpstat.tcps_rcvoopack;
		ipm_tcpstatp->tcps_rcvoobyte	= tcpstat.tcps_rcvoobyte;
		ipm_tcpstatp->tcps_rcvpackafterwin	= tcpstat.tcps_rcvpackafterwin;
		ipm_tcpstatp->tcps_rcvbyteafterwin	= tcpstat.tcps_rcvbyteafterwin;
		ipm_tcpstatp->tcps_rcvafterclose	= tcpstat.tcps_rcvafterclose;
		ipm_tcpstatp->tcps_rcvwinprobe	= 	tcpstat.tcps_rcvwinprobe;
		ipm_tcpstatp->tcps_rcvdupack	= tcpstat.tcps_rcvdupack;
		ipm_tcpstatp->tcps_rcvacktoomuch	= tcpstat.tcps_rcvacktoomuch;
		ipm_tcpstatp->tcps_rcvackpack	= tcpstat.tcps_rcvackpack;
		ipm_tcpstatp->tcps_rcvackbyte	= tcpstat.tcps_rcvackbyte;
		ipm_tcpstatp->tcps_rcvwinupd	= tcpstat.tcps_rcvwinupd;
		ipm_tcpstatp->tcps_pawsdrop		= tcpstat.tcps_pawsdrop;
		ipm_tcpstatp->tcps_predack		= tcpstat.tcps_predack;
		ipm_tcpstatp->tcps_preddat		= tcpstat.tcps_preddat;
		ipm_tcpstatp->tcps_persistdrop	= tcpstat.tcps_persistdrop;
		ipm_tcpstatp->tcps_badsyn		= tcpstat.tcps_badsyn;
		break;
		
	case UDP_STAT:
		ipm_udpstatp->udps_card			= 0;
		ipm_udpstatp->udps_ipackets		= udpstat.udps_ipackets;
		ipm_udpstatp->udps_hdrops		= udpstat.udps_hdrops;
		ipm_udpstatp->udps_badsum		= udpstat.udps_badsum;
		ipm_udpstatp->udps_badlen		= udpstat.udps_badlen;
		ipm_udpstatp->udps_noport		= udpstat.udps_noport;
		ipm_udpstatp->udps_noportbcast	= udpstat.udps_noportbcast;
		ipm_udpstatp->udps_fullsock		= udpstat.udps_fullsock;
		//	ipm_udpstatp->udps_pcbcachemiss = udpstat.udps_pcbcachemiss;
		ipm_udpstatp->udps_pcbhashmiss  = udpstat.udps_pcbhashmiss;
		ipm_udpstatp->udps_opackets		= udpstat.udps_opackets;
		break;
		
	case IP_STAT:
		ipm_ipstatp->ips_card			= 0;
		ipm_ipstatp->ips_defaultTTL		= 16;
		ipm_ipstatp->ips_total			= ipstat.ips_total;
		ipm_ipstatp->ips_badsum			= ipstat.ips_badsum;
		ipm_ipstatp->ips_tooshort		= ipstat.ips_tooshort;
		ipm_ipstatp->ips_toosmall		= ipstat.ips_toosmall;
		ipm_ipstatp->ips_badhlen		= ipstat.ips_badhlen;
		ipm_ipstatp->ips_badlen			= ipstat.ips_badlen;
		ipm_ipstatp->ips_fragments		= ipstat.ips_fragments;
		ipm_ipstatp->ips_fragdropped	= ipstat.ips_fragdropped;
		ipm_ipstatp->ips_fragtimeout	= ipstat.ips_fragtimeout;
		ipm_ipstatp->ips_forward		= ipstat.ips_forward;
		ipm_ipstatp->ips_cantforward	= ipstat.ips_cantforward;
		ipm_ipstatp->ips_redirectsent	= ipstat.ips_redirectsent;
		ipm_ipstatp->ips_noproto		= ipstat.ips_noproto;
		ipm_ipstatp->ips_delivered		= ipstat.ips_delivered;
		ipm_ipstatp->ips_localout		= ipstat.ips_localout;
		ipm_ipstatp->ips_odropped		= ipstat.ips_odropped;
		ipm_ipstatp->ips_reassembled	= ipstat.ips_reassembled;
		ipm_ipstatp->ips_fragmented		= ipstat.ips_fragmented;
		ipm_ipstatp->ips_ofragments		= ipstat.ips_ofragments;
		ipm_ipstatp->ips_cantfrag		= ipstat.ips_cantfrag;
		ipm_ipstatp->ips_badoptions		= ipstat.ips_badoptions;
		ipm_ipstatp->ips_noroute		= ipstat.ips_noroute;
		ipm_ipstatp->ips_badvers		= ipstat.ips_badvers;
		ipm_ipstatp->ips_rawout			= ipstat.ips_rawout;
		break;

	case ICMP_STAT:
		ipm_icmpstatp->icps_card		= 0;
		ipm_icmpstatp->icps_error		= icmpstat.icps_error;
		ipm_icmpstatp->icps_oldshort	= icmpstat.icps_oldshort;
		memcpy(ipm_icmpstatp->icps_outhist, icmpstat.icps_outhist, 
			   sizeof(uint32)*(ICMP_MAXTYPE+1));
		ipm_icmpstatp->icps_badcode		= icmpstat.icps_badcode;
		ipm_icmpstatp->icps_tooshort	= icmpstat.icps_tooshort;
		ipm_icmpstatp->icps_checksum	= icmpstat.icps_checksum;
		ipm_icmpstatp->icps_badlen		= icmpstat.icps_badlen;
		ipm_icmpstatp->icps_reflect		= icmpstat.icps_reflect;
		memcpy(ipm_icmpstatp->icps_inhist, icmpstat.icps_inhist,
			   sizeof(uint32)*(ICMP_MAXTYPE+1));
		break;

	default:
		return IPM_GET_ERR;
	} /* switch */
	
	(*count) = 1;

	return IPM_GET_OK;
}

static int32
get_ifstat(kvm_t*		kvmd, 
		   uint32		offset,
		   int			which,
		   char			*buf,
		   int			size,
		   int			*count,
		   int			*needed,
		   void			*extra)
{
	struct ifnet_head	ifhead;	/* TAILQ_HEAD */
	struct ifnet		ifnet;
	struct ifnet		*ifnetaddr;
	int					slot, type, link;
	ipm_ifstat_t		*ipm_ifstatp;
	char				*ifname = (char*)extra;

	*count = 0;
	*needed = 0;

//	printf("ifstat 1 called kvmd %d\n", kvmd);

	if (kvm_read(kvmd, offset, (char*)&ifhead, sizeof(ifhead))
		== -1) {
		IPM_LOGERR(IPM_ERR_KERN);
		return IPM_GET_ERR;	
	}

//	printf("ifstat 2 called kvmd %d\n", kvmd);

	ifnetaddr = ifhead.tqh_first;
	while (ifnetaddr) {
		
		/* retrive ifnet struct in chain */
		if (kvm_read(kvmd, (uint32)ifnetaddr, (char*)&ifnet, sizeof(ifnet))
			== -1) {
			IPM_LOGERR(IPM_ERR_KERN);
			return IPM_GET_ERR;	
		}
		ifnetaddr = ifnet.if_list.tqe_next;
			
		ifnet.if_xname[IFNAMSIZ - 1] = '\0';	/* sanity */
		ipmd_ParseIfName(ifnet.if_xname, &type, &slot, &link);
		if (slot != 0)
			continue;			/* ignore any interface that is not local */
		
		/* interface name not matching */
		if (ifname && (strcmp(ifname, ifnet.if_xname) != 0))
			continue;

		
		/* cpy to buffer */
		if (size < sizeof(ipm_ifstat_t)) {
			*needed = (*count)*2*sizeof(ipm_ifstat_t); /* no more buffer space */
			return IPM_GET_NO_BUF;
		}			
		ipm_ifstatp = (ipm_ifstat_t*)buf;
		memcpy(ipm_ifstatp->ifi_ifname, ifnet.if_xname, IFNAMSIZ);
		ipm_ifstatp->ifi_speed	= 100000000;
		ipm_ifstatp->ifi_oQlen	= ifnet.if_snd.ifq_len;
		
		ipm_ifstatp->ifi_type		= ifnet.if_type;
  	    ipm_ifstatp->ifi_addrlen	= ifnet.if_addrlen;
  	    ipm_ifstatp->ifi_hdrlen		= ifnet.if_hdrlen;
		ipm_ifstatp->ifi_mtu		= ifnet.if_mtu;
		ipm_ifstatp->ifi_metric		= ifnet.if_metric;
		ipm_ifstatp->ifi_baudrate	= ifnet.if_baudrate;
		ipm_ifstatp->ifi_ipackets	= ifnet.if_ipackets;
		ipm_ifstatp->ifi_ierrors	= ifnet.if_ierrors;
		ipm_ifstatp->ifi_opackets	= ifnet.if_opackets;
		ipm_ifstatp->ifi_oerrors	= ifnet.if_oerrors;
		ipm_ifstatp->ifi_collisions = ifnet.if_collisions;
		ipm_ifstatp->ifi_ibytes		= ifnet.if_ibytes;
		ipm_ifstatp->ifi_obytes		= ifnet.if_obytes;
		ipm_ifstatp->ifi_imcasts	= ifnet.if_imcasts;
		ipm_ifstatp->ifi_omcasts	= ifnet.if_omcasts;
		ipm_ifstatp->ifi_iqdrops	= ifnet.if_iqdrops;
		ipm_ifstatp->ifi_noproto	= ifnet.if_noproto;
		*((struct timeval*)&(ipm_ifstatp->ifi_lastchange_sec)) =
			ifnet.if_lastchange;

		/* advance buf */
		buf += sizeof(ipm_ifstat_t);
		size -= sizeof(ipm_ifstat_t);
		(*count)++;
		
		/* we have already found the interface */
		if (ifname)
			break;
	} /* while */

	return IPM_GET_OK;
}


const struct ipmd_krec {
	char		*name;
	int			(*funcp)(kvm_t*, uint32, int, char*, int, int*, int*, void*);
} krec[] = {
	{ "_tcbtable",		get_table },
	{ "_tcpstat",		get_stats },
	{ "_udbtable",		get_table },
	{ "_udpstat",		get_stats },
	{ "_icmpstat",		get_stats },
	{ "_ifnet",         get_ifstat }, 
	{ "_ipstat",		get_stats },
};

/* 
 *-----------------------------------------------------------------
 *
 * Name: ipmd_OpenKernMem
 *
 * Description: open kernel memory to read ssc tcp/udp stats
 *
 * Created by:	Su Wang
 *
 * Date Created: 01/09/2002
 *
 *-----------------------------------------------------------------
 */
static void*					/* descriptor, NULL */
ipmd_OpenKernMem() 
{
	char		buf[_POSIX2_LINE_MAX];
	kvm_t		*kvmd;

	/* open kernel */
	if ((kvmd = (void*)kvm_openfiles(NULL, NULL, NULL, O_RDONLY, buf)) == NULL) {
		printf("ipmd_OpenKernShm: kvm_open: %s\n", buf);
		return NULL;
	}
	
	return kvmd;
}

/* 
 *-----------------------------------------------------------------
 *
 * Name: ipmd_GetKernMem
 *
 * Description: get kernel memory content.
 *
 * Created by:	Su Wang
 *
 * Date Created: 01/09/2002
 *
 *-----------------------------------------------------------------
 */
static int32					/* IPM_GET_ERR,IPM_GET_OK, IPM_GET_NO_BUF */
ipmd_GetKernMem(void	*kvmd,	/* desc to kern shmem */
				int		id,		/* record id */
				char	*buf,
				int		size,
				int		*count,
				int		*need,
				void	*extra)
{
	struct nlist		nl[2];
	uint32				offset;
	int					rc;

	/* sanity checks */
	if ((kvmd == NULL) ||
		(buf == NULL))
		IPM_LOGERR(IPM_ERR_KERN);
	
	
	nl[0].n_name = krec[id].name;
	nl[0].n_value = 0;
	nl[1].n_name = "";
	
	if (kvm_nlist((kvm_t*)kvmd, nl) < 0 || nl[0].n_type == 0) {
		IPM_LOGERR(IPM_ERR_KERN);
		return NFX_ERR;
	}
	
	/* get offset to kernel shared memory */
	if ( (offset = nl[0].n_value) == 0) {
		printf("ipmd_GetKernMem: offset is zero\n");
		return IPM_GET_ERR;
	}
	
	rc = (*krec[id].funcp)((kvm_t*)kvmd, offset, id, buf, size, count, need, extra);
	kvm_close(kvmd);
	return rc;
}

/* 
 *-----------------------------------------------------------------
 *
 * Name: ipmd_GetFromSSC
 *
 * Description: getting information from bsd kernel space on ssc 
 *
 * Created by:	Su Wang
 *
 * Date Created: 01/11/2001
 *
 *-----------------------------------------------------------------
 */
static int32					/* IPM_GET_ERR,IPM_GET_OK, IPM_GET_NO_BUF  */
ipmd_GetFromSSC(int		id,
				char	*buf,
				int		size,
				int		*count,
				int		*needed,
				void	*extra)
{
	void		*kvmd;
	
	if ((kvmd = ipmd_OpenKernMem()) == NULL)
		return IPM_GET_ERR;
	
	return ipmd_GetKernMem(kvmd, id, buf, size, count, needed, extra);
}

/*-----------------------------------------------------------------
 *
 * Name: ipmd_GetKernIfStats
 *
 * Description: get stats associated with a device from kern. 
 *
 * Created by:	Su Wang
 *
 * Date Created: 12/12/2001
 *
 *-----------------------------------------------------------------
 */
int32							/* IPM_GET_OK, IPM_GET_ERR, IPM_GET_NO_BUF */
ipmd_GetKernIfStats(char*			ifname,	
					ipm_ifstat_t	*ifstat, /* usr provided buf */
					int				bufsize, /* buffer size */
					int				*count,	/* number of record in ifstat */
					int				*needed) /* buffer size needed */
	
{
	return ipmd_GetFromSSC(IF_STAT, (char*)ifstat, 
						   bufsize, count, needed, (void*)ifname);
}

/*-----------------------------------------------------------------
 *
 * Name: ipmd_GetKernIpStats
 *
 * Description: get ip stats from kernel. 
 *
 * Created by:	Su Wang
 *
 * Date Created: 10/23/2001
 *
 *-----------------------------------------------------------------
 */
int32							/* IPM_GET_OK, IPM_GET_ERR, IPM_GET_NO_BUF */
ipmd_GetKernIpStats(void			*ipstat, 
					int				bufsize, /* buffer size */
					int				*count,	/* number of record in ipstat */
					int				*needed)	/* buff size needed */
{
	return ipmd_GetFromSSC(IP_STAT, 
						   (char*)ipstat, bufsize, count, needed, NULL);
}

/*-----------------------------------------------------------------
 *
 * Name: ipmd_GetKernIcmpStats
 *
 * Description: get icmp stats from kernel
 *
 * Created by:	Su Wang
 *
 * Date Created: 10/23/2001
 *
 *-----------------------------------------------------------------
 */
int32							/* IPM_GET_OK, IPM_GET_ERR, IPM_GET_NO_BUF */
ipmd_GetKernIcmpStats(void				*icmpstat, 
					  int				bufsize, /* buffer size */
					  int				*count,	/* number of record */
					  int			    *needed) /* buff size needed */

{
	return ipmd_GetFromSSC(ICMP_STAT, 
						   (char*)icmpstat, bufsize, count, needed, NULL);
}

/*-----------------------------------------------------------------
 *
 * Name: ipmd_GetKernTcpStats
 *
 * Description: get Tcp stats from kernel.
 *
 * Created by:	Su Wang
 *
 * Date Created: 10/23/2001
 *
 *-----------------------------------------------------------------
 */
int32							/* IPM_GET_OK, IPM_GET_ERR, IPM_GET_NO_BUF */
ipmd_GetKernTcpStats(void			*tcpstat, 
					 int			bufsize,
					 int			*count,/* number of record */
					 int			*needed) /* buff size needed */
{
	return ipmd_GetFromSSC(TCP_STAT, 
						   (char*)tcpstat, bufsize, count, needed, NULL);
}

/*-----------------------------------------------------------------
 *
 * Name: ipmd_GetKernUdpStats
 *
 * Description: get Udp stats from kernel
 *
 * Created by:	Su Wang
 *
 * Date Created: 10/23/2001
 *
 *-----------------------------------------------------------------
 */
int32							/* IPM_GET_OK, IPM_GET_ERR, IPM_GET_NO_BUF */
ipmd_GetKernUdpStats(void			*udpstat, 
					 int			bufsize,
					 int			*count,/* number of record */
					 int			*needed) /* buff size needed */
{
	return ipmd_GetFromSSC(UDP_STAT, 
						   (char*)udpstat, bufsize, count, needed, NULL);
}

/*-----------------------------------------------------------------
 *
 * Name: ipmd_GetKernTcpConn
 *
 * Description: get tcp conn table from kernel.
 *
 * Created by:	Su Wang
 *
 * Date Created: 01/11/2002
 *
 *-----------------------------------------------------------------
 */
int32							/* IPM_GET_OK, IPM_GET_ERR, IPM_GET_NO_BUF */
ipmd_GetKernTcpConn(ipm_conn_t	*tcpconn,
					int			bufsize,
					int			*count,	/* number of record */
					int			*needed) /* buff size needed */

{
	return ipmd_GetFromSSC(TCB_TABLE, 
						   (char*)tcpconn, bufsize, count, needed, NULL);
}

/*-----------------------------------------------------------------
 *
 * Name: ipmd_GetKernUdpConn
 *
 * Description: get udp conn table from kernel.
 *
 * Created by:	Su Wang
 *
 * Date Created: 01/11/2002
 *
 *-----------------------------------------------------------------
 */
int32							/* IPM_GET_OK, IPM_GET_ERR, IPM_GET_NO_BUF */
ipmd_GetKernUdpConn(ipm_conn_t	*udpconn,
					int			bufsize,
					int			*count,	/* number of record */
					int			*needed)	/* buff size needed */
{
	return ipmd_GetFromSSC(UDB_TABLE, 
						   (char*)udpconn, bufsize, count, needed, NULL);
}
	
#endif /* SSC_x86 SSC_mips */

/* 
 *-----------------------------------------------------------------
 *
 * Name: ipmd_GetIfInfo
 *
 * Description: core function to get Interface and IP info.		
 * Created by:	Su Wang
 *
 * Date Created: 11/14/2001
 *
 *-----------------------------------------------------------------
 */
static int32					/* IPM_GET_ERR,IPM_GET_NO_BUF, IPM_GET_OK */
ipmd_GetIfInfo(int			what,
			   char			*device,
			   void			*buf,	
			   int			bufsize,	
			   int			*count,	
			   int			*need)
{
#if defined(SSC_x86) || defined(SSC_mips)
								/* in general works, but some versions of linux */
								/* not recognize SA_LEN */

  struct ifreq			*ifrp;	/* if request ptr */
  struct ifreq			ifrcpy;
  struct ifconf			ifc;	/* if config request ptr */
  int					len = 24*sizeof(struct ifreq);
								/* 6 interfaces, with (mac/ip) address = 12 */
  int					sock = 0;
  char					*inbuf = 0;	/* my own buffer ptr */
  ipm_ip_t				*ipm_ipp;
  ipm_if_t				*ipm_ifp;
  char					*tptr;
  int					rc = IPM_GET_OK;
  char					lastname[16];
  int					ifr_size; /* ifreq struct size */
  int					ipm_size; /* ipm_if_t, ipm_ip_t size */
  int					type, link, cpu, slot;
  // ipm_auxIntf_t			*ipmd_auxIntfPtr = 0;

  /* assume buf is enough */
  *need = 0;
  *count = 0;

  if (what == IWANTIP)
	ipm_size = sizeof(ipm_ip_t);
  else
	ipm_size = sizeof(ipm_if_t);

  /* first get a sock ptr */
  sock = socket(AF_INET, SOCK_DGRAM, 0);
  if (sock == 0)
	{
	  IPM_LOGERR(IPM_ERR_ALLOC);
	  rc = IPM_GET_ERR;
	  goto out;
	}

  
  /* recursively getting ifconfig. Kernel does not tell */
  /* us whether buff is enough we need to test it ourselves */
  	while (1) {
		ifc.ifc_len = len;
		ifc.ifc_buf = inbuf = malloc(len);

		if (inbuf == NULL)
		  {
			IPM_LOGERR(IPM_ERR_ALLOC);
			rc = IPM_GET_ERR;
			goto out;		
		  }

		if (ioctl(sock, SIOCGIFCONF, &ifc) < 0)
		  {
			IPM_LOGERR(IPM_ERR_KERN);
			rc = IPM_GET_ERR;
			goto out;		
		  }

		/* we are fine with current buf size */
		/* get out we have all the info */
		if (ifc.ifc_len + sizeof(struct ifreq) < len)
			break;

		/* buffer is lacking, */
		/* try again with doubled buffer size */
		free(inbuf);
		inbuf = 0;
		len *= 2;
	}

	/* remember last process interface */
	bzero(lastname, sizeof(lastname));
	for ( tptr = inbuf; tptr < inbuf+ifc.ifc_len;)
	  {

		/* setup the pointers, readying copy */
		ifrp	= (struct ifreq*)tptr;;
		ipm_ipp = (ipm_ip_t*)buf;
		ipm_ifp = (ipm_if_t*)buf;
		
		/* find the ptr to next entry in ifconf */
		/* by calucate current entry size */
		ifr_size = ifrp->ifr_addr.sa_len;

		if (ifr_size < sizeof(ifrp->ifr_addr))
		  ifr_size = sizeof(ifrp->ifr_addr);

		ifr_size += sizeof(ifrp->ifr_name);
		tptr += ifr_size;

		/* OK we have an ifreq entry, */
		/* process it */


		/* application want specific device, */
		/* device not matching */
		if (device && strcmp(device, ifrp->ifr_name))
		  continue;

		if (what == IWANTIP)
		  {
			/* address is not IP family address */
			if (ifrp->ifr_addr.sa_family != AF_INET)
			  continue;
		  }
		else					/* IWANTIF */
		  {
			/* unique link address for each interface */
			if (ifrp->ifr_addr.sa_family != AF_LINK)
			  continue;

			/* interface processed */
			if (strcpy(lastname, ifrp->ifr_name) == 0)
			  continue;

			/* is this interface valid */
			if (ipmd_ParseIfName(ifrp->ifr_name, &type, &slot, &link) != NFX_OK)
				continue;
		  }
		
		/* save interface for comparison next iteration */
		strcpy(lastname, ifrp->ifr_name);
		
	
		/* we found an entry that application want */
		
		/* is there enough buf space to proceed */
		if ((bufsize < ipm_size) || !buf)
		  {
			/* guess required amount */
			*need = (ifc.ifc_len/sizeof(struct ifreq))*
			  ipm_size;
			rc = IPM_GET_NO_BUF; 
			break;
		  }

		/* ready for next entry in input buf*/
		bufsize -= ipm_size;
		buf += ipm_size;
		(*count)++;

		/* getting info */
		if (what == IWANTIP)
		  {
			/* device name */
			strcpy(ipm_ipp->ip_ifname, ifrp->ifr_name);
#if defined(LINK_AGG)
			{
				ipm_vif_t	*vifp;
				if ((vifp = ipmd_FindVifByLink(ipm_ipp->ip_ifname)))
					strcpy(ipm_ipp->ip_vifName, vifp->vif_name);
				else
					bzero(ipm_ipp->ip_vifName, IPM_STR_SIZE);
			}
#endif /* LINK_AGG */

			/* ip */
			ipm_ipp->ip_addr =  
			  ((struct sockaddr_in*)&(ifrp->ifr_addr))->sin_addr.s_addr;
			
			/* mask */
			ifrcpy = *ifrp;
			if (ioctl(sock, SIOCGIFNETMASK, &ifrcpy) < 0)
				/* we will ignore the err */
				IPM_LOGERR(IPM_ERR_KERN);
			else
				ipm_ipp->ip_mask = 
				  ((struct sockaddr_in*)&(ifrcpy.ifr_addr))->sin_addr.s_addr;
		  }
		else if ((what == IWANTIF_TXRX) &&
				 strncmp(ifrp->ifr_name, "sc", 2)) {

			/* if getting single GibE interface info,
			 * retrive from txrx 
			 */
			ipm_mhdr_t	*ipm_mhdrp = 0;
			ipm_mintf_t	*ipm_mintfp;
			boardType_t btype;
			char		nodeName[48];
			
			ipm_mhdrp = malloc(sizeof(ipm_mhdr_t) + sizeof(ipm_mintf_t));
			if (!ipm_mhdrp) {
				IPM_LOGERR(IPM_ERR_BUF);
				rc = IPM_GET_ERR;
				goto out;
			}
			
			/* fill in msg hdr */
			ipm_mhdrp->ipmh_mtype =  IPM_MSG_INTF_REQ;
			ipm_mhdrp->ipmh_size = sizeof(ipm_mhdr_t) + sizeof(ipm_mintf_t);
			ipm_mhdrp->ipmh_flags = 0;
			
			/* fill in interface request info */
			if (getPortInfo (ifrp->ifr_name, nodeName, &btype, &slot, &link)) {
				rc = IPM_GET_ERR;
				goto done;
			}
			cpu = GET_CPU_FROM_ULINK(link);
			link = GET_LINK_FROM_ULINK(link);
			
			ipm_mintfp = (ipm_mintf_t*)(ipm_mhdrp+1);
			ipm_mintfp->if_slot = slot;
			ipm_mintfp->if_link  = link;
			ipm_mintfp->if_cpu  =  cpu;
			ipm_mintfp->if_dtype = (ushort16)btype;
			
			/* send to txrx for reply */
			if (ipmd_SendMsg(ipm_mhdrp, ipm_mhdrp->ipmh_size, slot, cpu, "ipm", 
							 ipm_mhdrp, ipm_mhdrp->ipmh_size, 0, -1) < 0) {
				rc = IPM_GET_ERR;
				goto done;
			}
			
			if (!(ipm_mhdrp->ipmh_flags & IPM_MSG_CMD_OK)) {
				rc = IPM_GET_ERR;
				goto done;
			}
			
			/* we have a possitive reply, parsing */
			strcpy(ipm_ifp->if_name, ifrp->ifr_name);

#if defined(LINK_AGG)
			{
				ipm_vif_t	*vifp;
				if ((vifp = ipmd_FindVifByLink(ipm_ifp->if_name)))
					strcpy(ipm_ifp->if_vifName, vifp->vif_name);
				else
					bzero(ipm_ifp->if_vifName, IPM_STR_SIZE);
			}
#endif /* LINK_AGG */

			memcpy(ipm_ifp->if_mac, ipm_mintfp->if_mac, 6);
			ipm_ifp->if_flags = ipm_mintfp->if_flags;

			/* reset intf state on ssc to ensure consistency */
			if (ipm_ifp->if_flags & IFF_UP)
				ipmd_EnableKernIntf(ipm_ifp->if_name);
			else
				ipmd_DisableKernIntf(ipm_ifp->if_name);

			ipm_ifp->if_auxFlags = 0;
			if (ipm_mintfp->if_flags & IFF_RUNNING)
				ipm_ifp->if_auxFlags |= IPM_AUXIF_RUNNING;
			
			if (ipm_mintfp->if_flags & IFF_LINK0)
				ipm_ifp->if_auxFlags |= IPM_AUXIF_AUTO;
			
			ipm_ifp->if_mtusize = ipm_mintfp->if_mtusize;

			
done:
			if (ipm_mhdrp)
				free (ipm_mhdrp);

			if (rc != NFX_OK) {
				rc = NFX_OK;
				(*count)--;
				goto out;
			}
			
			if (device)
				goto out;
		} 
		else {				/* WANTIF */
			struct sockaddr_dl	*sdl;

			/* device name */
			strcpy(ipm_ifp->if_name, ifrp->ifr_name);
#if defined(LINK_AGG)
			{
				ipm_vif_t	*vifp;
				if ((vifp = ipmd_FindVifByLink(ipm_ifp->if_name)))
					strcpy(ipm_ifp->if_vifName, vifp->vif_name);
				else
					bzero(ipm_ifp->if_vifName, IPM_STR_SIZE);
			}
#endif /* LINK_AGG */

			/* mac */
			sdl = (struct sockaddr_dl*)&ifrp->ifr_addr;
			memcpy(ipm_ifp->if_mac, LLADDR(sdl), 6);


			bzero(&ifrcpy, sizeof(ifrcpy));
			strcpy(ifrcpy.ifr_name, ifrp->ifr_name);
			strcpy(ipm_ifp->if_name, ifrp->ifr_name);
			
			/* mtu */
			if (ioctl(sock, SIOCGIFMTU, &ifrcpy)<0)
			  IPM_LOGERR(IPM_ERR_KERN);
			else
			  ipm_ifp->if_mtusize = ifrcpy.ifr_mtu;
			
			/* flags */
			if (ioctl(sock, SIOCGIFFLAGS, &ifrcpy)<0)
			  IPM_LOGERR(IPM_ERR_KERN);
			else
			  ipm_ifp->if_flags = ifrcpy.ifr_flags;

			/* fill in rest of structure */
			cpu = GET_CPU_FROM_ULINK(link);
			link = GET_LINK_FROM_ULINK(link);
			ipm_ifp->if_slot = slot;
			ipm_ifp->if_cpu  = cpu;
			ipm_ifp->if_link = link;

			/* assume sc always running */
			ipm_ifp->if_auxFlags = IPM_AUXIF_RUNNING;
			
#if 0
			/* if interface is GigE, use auxIntf */
			if (strncmp(ifrp->ifr_name, "sc", 2)) {
				ipmd_auxIntfPtr = IPM_GET_AUXINTF(1, link);
				ipm_ifp->if_mtusize = ipmd_auxIntfPtr->ipm_auxMtu;
				ipm_ifp->if_auxFlags = ipmd_auxIntfPtr->ipm_auxFlags;
			}
			else
				/* Todo, for now set operational state always on */
				ipm_ifp->if_auxFlags |= IPM_AUXIF_RUNNING;
#endif 
			/* we have found the device, get out */
			if (device)
			  goto out;
		  }
	  }
out:
	if (inbuf)
	  free(inbuf);
	
	if (sock)
	  close(sock);

// printf("record # %d, size %d, need %d bytes\n", *count, ipm_size, *need); 
	return rc;
#else
	return IPM_GET_OK;			/* tmp test */
#endif /* SSC_x86 SSC_mips */
}

/* 
 *-----------------------------------------------------------------
 *
 * Name: ipmd_GetKernIpByName
 *
 * Description: get IP interface associated with a device from kern. 
 *				if device is NULL, cpy all ip interface into
 *				buff in consecutive ipm_if_t.
 *
 * Created by:	Su Wang
 *
 * Date Created: 10/23/2001
 *
 *-----------------------------------------------------------------
 */
int32							/* IPM_GET_ERR,IPM_GET_NO_BUF, IPM_GET_OK */
ipmd_GetKernIpByName(char			*device,
					 ipm_ip_t		*buf,	
					 int			bufsize,	
					 int			*count,	
					 int			*need)

{
   return ipmd_GetIfInfo(IWANTIP, device, buf, bufsize, count, need);
}

/* 
 *-----------------------------------------------------------------
 *
 * Name: ipmd_GetKernFindIp
 *
 * Description: chk ip already exist
 *
 * Created by:	Su Wang
 *
 * Date Created: 04/05/2001
 *
 *-----------------------------------------------------------------
 */
int32							/* NFX_OK, NFX_ERR */
ipmd_GetKernFindIp(uint32	ip,
				   uint32	mask)
{
	ipm_ip_t	*ipBuf, *start;
	int			size = sizeof(ipm_ip_t)*128; /* max 128 ip interface on system */
	int			count, need, i, rc, ret = NFX_ERR;
	uint32		inMask;
	
try_again:
	start = ipBuf = (ipm_ip_t*)IPM_ALLOC(size);
	
	rc = ipmd_GetKernIpByName(NULL, ipBuf, size, &count, &need);
	
	/* not enough buf try again */
	if (rc == IPM_GET_NO_BUF) {
		size *= 2;
		size = max(size, need);
		IPM_FREE(ipBuf);
		goto try_again;
	}
	
	if (rc != IPM_GET_OK) {
		ret = NFX_ERR;
		goto free;
	}
	
	/* get must be ok */
	for (i = 0; i < count; i++) {
		inMask = mask & ipBuf->ip_mask;
		if ((inMask & ip) == (inMask & ipBuf->ip_addr)) {
			ret = NFX_OK;
			goto free;
		}
		ipBuf++;
	}

free:
	if (start)
		IPM_FREE(start);
	
	return ret;
}

/* 
 *-----------------------------------------------------------------
 *
 * Name: ipmd_GetKernFindIpExact
 *
 * Description: chk exact ip already exist
 *
 * Created by:	Su Wang
 *
 * Date Created: 04/05/2001
 *
 *-----------------------------------------------------------------
 */
int32							/* NFX_OK, NFX_ERR */
ipmd_GetKernFindIpExact(uint32	ip,
						uint32	mask,
						char*	intf)
{
	ipm_ip_t	*ipBuf, *start;
	int			size = sizeof(ipm_ip_t)*128; /* max 128 ip interface on system */
	int			count, need, i, rc, ret = NFX_ERR;
	
try_again:
	start = ipBuf = (ipm_ip_t*)IPM_ALLOC(size);
	
	rc = ipmd_GetKernIpByName(NULL, ipBuf, size, &count, &need);
	
	/* not enough buf try again */
	if (rc == IPM_GET_NO_BUF) {
		size *= 2;
		size = max(size, need);
		IPM_FREE(ipBuf);
		goto try_again;
	}
	
	if (rc != IPM_GET_OK) {
		ret = NFX_ERR;
		goto free;
	}
	
	/* get must be ok */
	for (i = 0; i < count; i++) {
		if ((ip == ipBuf->ip_addr) &&
			(mask == ipBuf->ip_mask) &&
			(strcmp(intf, ipBuf->ip_ifname) == 0)) {
			ret = NFX_OK;
			goto free;
		}
		ipBuf++;
	}

free:
	if (start)
		IPM_FREE(start);
	
	return ret;
}

/* 
 *-----------------------------------------------------------------
 *
 * Name: ipmd_GetKernIpByIp
 *
 * Description: chk exact ip already exist
 *
 * Created by:	Su Wang
 *
 * Date Created: 05/03/2002
 *
 *-----------------------------------------------------------------
 */
int32							/* IPM_GET_ERR, IPM_GET_OK, IPM_GET_NOBUF */
ipmd_GetKernIpByIp(uint32	ip,
				   int		exact,
				   ipm_ip_t	*buf,
				   int		bufsize,
				   int		*in_count,
				   int		*in_need)
{
	ipm_ip_t	*ipBuf, *start;
	int			size = sizeof(ipm_ip_t)*128; /* max 128 ip interface on system */
	int			count, need, i, rc, match = 0, ret = IPM_GET_ERR;

	try_again:
	start = ipBuf = (ipm_ip_t*)IPM_ALLOC(size);
	
	rc = ipmd_GetKernIpByName(NULL, ipBuf, size, &count, &need);
	
	/* not enough buf try again */
	if (rc == IPM_GET_NO_BUF) {
		size *= 2;
		size = max(size, need);
		IPM_FREE(ipBuf);
		goto try_again;
	}
	
	if (rc != IPM_GET_OK) {
		ret = IPM_GET_ERR;
		goto free;
	}

	/* get must be ok */
	for (i = 0; i < count; i++) {
		if ((ip & ipBuf->ip_mask) == 
			(ipBuf->ip_addr & ipBuf->ip_mask)) {
			if (!exact) 		/* subsumed by ip interface */
				match = 1;
			else if (ip == ipBuf->ip_addr) /* exact match */
				match = 1;
			
			if (match) {
				memcpy(buf, ipBuf, sizeof(ipm_ip_t));
				ret = IPM_GET_OK;
				goto free;
			}
		}
		ipBuf++;
	}

free:
	if (start)
		IPM_FREE(start);
	
	return ret;
}

/* 
 *-----------------------------------------------------------------
 *
 * Name: ipmd_GetKernIfByName
 *
 * Description: get device if configuration from kernel, 
 *				if device = NULL,
 *				all device interface are copied
 *
 * Created by:	Su Wang
 *
 * Date Created: 10/23/2001
 *
 *-----------------------------------------------------------------
 */
int32							/* IPM_GET_ERR, IPM_GET_OK, IPM_GET_NO_BUF */
ipmd_GetKernIfByName(char			*device,
					 ipm_if_t		*buf,	
					 int			bufsize,	
					 int			*count,	
					 int			*need)	
{
  return ipmd_GetIfInfo(IWANTIF, device, buf, bufsize, count, need);
}

/* 
 *-----------------------------------------------------------------
 *
 * Name: ipmd_GetKernIfByNameTxRx
 *
 * Description: same as ipmd_GetKernIfByName,
 *				except info is retrived directly
 *				from txrx when single GibE is given
 *
 * Created by:	Su Wang
 *
 * Date Created: 10/23/2001
 *
 *-----------------------------------------------------------------
 */
int32							/* IPM_GET_ERR, IPM_GET_OK, IPM_GET_NO_BUF */
ipmd_GetKernIfByNameTxRx(char			*device,
						 ipm_if_t		*buf,	
						 int			bufsize,	
						 int			*count,	
						 int			*need)	
{
  return ipmd_GetIfInfo(IWANTIF_TXRX, device, buf, bufsize, count, need);
}
/* 
 *-----------------------------------------------------------------
 *
 * Name: ipmd_GetRtInfo
 *
 * Description: core function to retrive arp/rt info. 
 *
 * Created by:	Su Wang
 *
 * Date Created: 10/23/2001
 *
 *-----------------------------------------------------------------
 */
static int32					/* IPM_GET_OK, IPM_GET_ERR, IPM_GET_NO_BUF */
ipmd_GetRtInfo(int		what,
			  void		*buf,	/* buff to cpy to, */
			  int		bufsize,/* size of buff */
			  int		*count,	/* how many being cpyed*/
			  int		*need			  
			  )
{
#if defined(SSC_x86) || defined(SSC_mips)

  struct rt_msghdr		*rtm;
  ipm_rt_t				*ipm_rtp;
  ipm_arp_t				*ipm_arpp;
  int					mib[6];
  int					in_need;
  char					*inbuf = 0;
  char					*tptr;
  int					incount = 0;
  struct sockaddr		*sa;
  struct sockaddr_inarp	*sina;
  struct sockaddr_dl	*sdl;
  int					rc = IPM_GET_OK;
  int					esize;	/* sock addr size */
  int					ipm_size; /* ipm_rt_t, ipm_arp_t size */
  char					ifname[IPM_STR_SIZE];


  mib[0] = CTL_NET;
  mib[1] = AF_ROUTE;
  mib[2] = 0;
  mib[3] = AF_INET;
  if (what == IWANTRT)  
	{
	  mib[4] = NET_RT_DUMP;
	  mib[5] = 0;
	  ipm_size = sizeof(ipm_rt_t);
	}
  else							/* IWANTARP */
	{
	  mib[4] = NET_RT_FLAGS;
	  mib[5] = RTF_LLINFO;
	  ipm_size = sizeof(ipm_arp_t);
	}

  *need = 0;
  *count = 0;

  /* estimate table size */
  if (sysctl(mib, 6, NULL, &in_need, NULL, 0) < 0)
	{
	  IPM_LOGERR(IPM_ERR_KERN);
	  rc = IPM_GET_ERR;
	  goto out;
	}

  if (in_need <= 0)
	{
	  /* nothing in the routing table */
	  *count = 0;
	  *need = 0;
	  rc = IPM_GET_OK;
	  goto out;
	}

  inbuf = malloc(in_need);
  if (inbuf == NULL)
	{
	  IPM_LOGERR(IPM_ERR_ALLOC);
	  rc = IPM_GET_ERR;
	  goto out;		  
	}

  /* getting table from kernel */
  if (sysctl(mib, 6, inbuf, &in_need, NULL, 0) < 0)
	{
	  IPM_LOGERR(IPM_ERR_KERN);
	  rc = IPM_GET_ERR;
	  goto out;
	}

  /* run through the table, if application give */
  /* enough buffer */
  for (tptr = inbuf; tptr < inbuf+in_need; tptr+= rtm->rtm_msglen)
	{
	  rtm = (struct rt_msghdr*)tptr;
	  incount++;
	}

  if (incount*ipm_size > bufsize)
	{
	  *count = 0;
	  *need = incount*ipm_size;
	  rc = IPM_GET_NO_BUF;
	  goto out;
	}


  /* ready to process each entry */
  for (tptr = inbuf; tptr < inbuf+in_need; tptr+= rtm->rtm_msglen)
	{

	  rtm = (struct rt_msghdr*)tptr;
	  
	  if (what == IWANTRT)
		{
		  sa = (struct sockaddr*)(rtm + 1);
		  ipm_rtp = (ipm_rt_t*)buf;

		  /* we only want to process IPv4 routes, exclude arp */
		  if ((sa->sa_family != AF_INET)		||
			  (rtm->rtm_flags & RTF_LLINFO))
			continue;
		  
		  if (rtm->rtm_addrs & RTA_DST)
			{ 
			  /* ip */
			  ipm_rtp->rt_addr = ((struct sockaddr_in*)sa)->sin_addr.s_addr;
			  
			}
		  else
			continue;

		  esize = max(sa->sa_len, sizeof(struct sockaddr));
		  sa = (struct sockaddr*)(((char*)sa) + esize);

		  if (rtm->rtm_addrs & RTA_GATEWAY)
			{
			  /* gateway */
			  if (sa->sa_family == AF_LINK)
				{
				  /* route is a directly connected network */
				  /* give 0 value to next hop */
				  ipm_rtp->rt_nhop = 0; 
				}
			  else if (sa->sa_family == AF_INET)
				{
				  /* normal route */
				  ipm_rtp->rt_nhop = 
					((struct sockaddr_in*)sa)->sin_addr.s_addr;
				}
			  else 
				{
				  /* address not recognized, skip to next entry */
				  continue;
				}
			}
		  else
			continue;
		  
		  esize = max(sa->sa_len, sizeof(struct sockaddr));
		  sa = (struct sockaddr*)(((char*)sa) + esize);	  
		  
		  if (rtm->rtm_addrs & RTA_NETMASK)
			{
			  /* netmask */
			  if (sa->sa_len)
				ipm_rtp->rt_mask = 
				  ((struct sockaddr_in*)sa)->sin_addr.s_addr;
			  else
				/* default route sa_len == 0 */
				ipm_rtp->rt_mask = 0;
			}
		  
		  /* flags */
		  ipm_rtp->rt_flags = rtm->rtm_flags;
		  
		  /* is route a route */
		  if (rtm->rtm_flags & RTF_HOST)
			  ipm_rtp->rt_mask = 0xffffffff;

		  /* interface */
	      // printf("interface idex %d\n", rtm->rtm_index);
		  if (if_indextoname1(rtm->rtm_index, ifname))
			strcpy(ipm_rtp->rt_ifname, ifname);
		  else
			IPM_LOGERR(IPM_ERR_KERN);

#if defined(LINK_AGG)
			{
				ipm_vif_t	*vifp;
				if ((vifp = ipmd_FindVifByLink(ipm_rtp->rt_ifname)))
					strcpy(ipm_rtp->rt_vifName, vifp->vif_name);
				else
					bzero(ipm_rtp->rt_vifName, IPM_STR_SIZE);
			}
#endif /* LINK_AGG */

		  /* Su, 09/06/02 
		   * don't show any routes related to lo
		   */
		  if (strncmp("lo", ifname, 2) == 0)
			  continue;
		}
	  else						/* IWANTARP */
		{
		  sina = (struct sockaddr_inarp *)(rtm + 1);
		  sdl = (struct sockaddr_dl *)(sina + 1);

		  ipm_arpp = (ipm_arp_t*)buf;

		  
		  /* not an arp entry */
		  if (!(rtm->rtm_flags & RTF_LLINFO))
			continue;

		  /* mac not resolved */
//		  if (sdl->sdl_alen == 0)
//		    continue;

		  /* ip */
		  ipm_arpp->arp_addr = sina->sin_addr.s_addr;

		  /* mac */
		  memcpy(ipm_arpp->arp_mac, LLADDR(sdl), 6);

		  /* age */
		  ipm_arpp->arp_age = rtm->rtm_rmx.rmx_expire;
		  

		  /* interface */
		  if (if_indextoname1(rtm->rtm_index, ifname))
			strcpy(ipm_arpp->arp_ifname, ifname);
		  else
			IPM_LOGERR(IPM_ERR_KERN);
		  
#if defined(LINK_AGG)
			{
				ipm_vif_t	*vifp;
				if ((vifp = ipmd_FindVifByLink(ipm_arpp->arp_ifname)))
					strcpy(ipm_arpp->arp_vifName, vifp->vif_name);
				else
					bzero(ipm_arpp->arp_vifName, IPM_STR_SIZE);
			}
#endif /* LINK_AGG */

		}

	  
	  /* advance buf ptr, counter */
	  buf += ipm_size;
	  (*count)++;
	}
  
 out:
	if (inbuf)
	  free(inbuf);

	return rc;
#else
	return IPM_GET_ERR;
#endif /* SSC_x86 SSC_mips */
}

/* 
 *-----------------------------------------------------------------
 *
 * Name: ipmd_GetKernRtTbl
 *
 * Description: get entire route table in buf. 
 *
 * Created by:	Su Wang
 *
 * Date Created: 10/23/2001
 *
 *-----------------------------------------------------------------
 */
int32							/* IPM_GET_OK, IPM_GET_ERR, IPM_GET_NO_BUF */
ipmd_GetKernRtTbl(ipm_rt_t	*buf,	/* buff to cpy to, */
				  int		bufsize,/* size of buff */
				  int		*count,	/* how many being cpyed*/
				  int		*need)			  
{
  return ipmd_GetRtInfo(IWANTRT, (void*)buf, bufsize, count, need); 
}

/* 
 *-----------------------------------------------------------------
 *
 * Name: ipmd_GetKernArpTblD
 *
 * Description: get entire ARP table in buf. 
 *
 * Created by:	Su Wang
 *
 * Date Created: 10/23/2001
 *
 *-----------------------------------------------------------------
 */
int32							/* IPM_GET_OK, IPM_GET_ERR, IPM_NO_BUF */
ipmd_GetKernArpTbl(ipm_arp_t *buf,	/* buff to cpy to, */
				   int		bufsize,/* size of buff */
				   int		*count,	/* how many being cpyed*/
				   int		*need)			  

{
  return ipmd_GetRtInfo(IWANTARP, (void*)buf, bufsize, count, need); 
}

extern int		ipmd_ssc_slot;
extern int		ipmd_ssc_cpu;
extern int		ipmd_txrx_slot;
extern int		ipmd_txrx_cpu;
/* 
 *-----------------------------------------------------------------
 *
 * Name: ipmd_ReplaceIp
 *
 * Description: This function is called whenver a ip is removed
 *				This function tries to establish another route
 *				if given ip has been removed
 *
 * Created by:	Su Wang
 *
 * Date Created: 10/16/2001
 *
 *-----------------------------------------------------------------
 */
void
ipmd_ReplaceIp(uint32 ip_addr)
{
	ipm_ip_t	ipm_ip;
	int			count =0, need=0;
	int			rc;
	ipm_mhdr_t	*ipm_hdrp = NULL;
	ipm_mintf_t	*ipm_intfp;
	ipm_mip_t	*ipm_ipp;
	int			msgSize;

	rc = ipmd_GetKernIpByIp(ip_addr, 0, 
							&ipm_ip, sizeof(ipm_ip), &count, &need);
	if (rc == IPM_GET_OK) {
		char	nodeName[48];
		boardType_t   btype;
		int			slot, link;

		rc = ipmd_AddKernIp(ipm_ip.ip_ifname, 
							ipm_ip.ip_addr, ipm_ip.ip_mask);

		if ((rc == NFX_OK) && strncmp(ipm_ip.ip_ifname, "sc", 2)) {
			  
			/* re-format new add intf msg */
			rc = getPortInfo (ipm_ip.ip_ifname, nodeName, &btype,
							  &slot, &link);
			
			if (rc) {
				printf("get port info failed\n");
				return;
			}
			

			/* send add IP to txrx */
			msgSize = sizeof(ipm_mhdr_t) + sizeof(ipm_mintf_t) + 
				sizeof(ipm_mip_t);
			ipm_hdrp = (ipm_mhdr_t*)IPM_ALLOC(msgSize);
			ipm_intfp = (ipm_mintf_t*)(ipm_hdrp + 1);
			ipm_ipp = (ipm_mip_t*)(ipm_intfp + 1);
			
			ipm_hdrp->ipmh_mtype = IPM_MSG_IP_ADD;
			ipm_hdrp->ipmh_size = msgSize;
			ipm_hdrp->ipmh_flags =IPM_MSG_CMD_NREPLY;
			
			ipm_intfp->if_dtype = (ushort16)btype;
			ipm_intfp->if_slot  = slot;
			ipm_intfp->if_link  = link;
			ipm_intfp->if_cpu   = ipmd_txrx_cpu;
			
			
			ipm_ipp->ip_addr = ipm_ip.ip_addr;
			ipm_ipp->ip_mask = ipm_ip.ip_mask;
			
			/* no need to get reply, because this msg is not initiazed by usr */
			if (ipmd_SendMsg(ipm_hdrp, ipm_hdrp->ipmh_size, ipmd_ssc_slot, ipmd_ssc_cpu, "ipm", 
							 NULL, 0, 1, -1) >= 0)
				ipm_stats.adm_fwd++;
			else
				ipm_stats.err_fwd++;  
		}
	}

	if (ipm_hdrp)
		IPM_FREE(ipm_hdrp);
	
	return;
}


#if defined(SSC_x86) || defined(SSC_mips)
/* tmp use until lib func is fixed */
static char *
if_indextoname1(unsigned int index, char *name)
{
        int     i, fd = -1, extra, len = 0;
        struct ifconf ifconf;
        char    lastname[IFNAMSIZ], iname[IFNAMSIZ], *retname = NULL, *inbuf;
        struct sockaddr *sa;
        void    *p;

        ifconf.ifc_buf = 0;


        if ((fd = socket(PF_INET, SOCK_DGRAM, 0)) < 0)
                goto ret;

        /*
         * Try ifc_len == 0 hack first, to get the actual length.
         * If that fails, revert to a loop which grows the ifc_buf
         * until it is sufficiently large.
         */
        extra = sizeof(struct ifreq);
        while (1) {
                ifconf.ifc_len = len;
                if (ioctl(fd, SIOCGIFCONF, (void *) &ifconf) == -1 &&
                    ifconf.ifc_buf)
                        goto ret;
                if (ifconf.ifc_buf &&
                    ifconf.ifc_len + extra < len)
                        break;
                if (ifconf.ifc_buf) {
                        if (len == 0)
                                len = 4096;
                        ifconf.ifc_len = len *= 2;
                } else {
                        len = ifconf.ifc_len;
                        extra = 0;
                }
                inbuf = realloc(ifconf.ifc_buf, ifconf.ifc_len);
                if (inbuf == NULL)
                        goto ret;
                ifconf.ifc_buf = inbuf;
     }

        i = 0;
        p = ifconf.ifc_buf;
        len = ifconf.ifc_len;
        lastname[0] = 0;
        lastname[sizeof(lastname)-1] = 0;
        iname[0] = 0;

		//	if (index == 21)
		// printf("config len is %d\n\n",ifconf.ifc_len);
		while (len > 0) {
			if (index == 21)
				// printf("curr len is %d\n", len);
                if (len < (IFNAMSIZ + sizeof(struct sockaddr)))
                        goto ret;
                if (strncmp(lastname, p, IFNAMSIZ)) {
                        if (i == index)
                                memcpy(iname, lastname, sizeof(iname));
                        strlcpy(lastname, p, sizeof(lastname));
                        i++;
                }
                len -= IFNAMSIZ;
                p += IFNAMSIZ;
                sa = p;

                if (sa->sa_family == AF_LINK) {
                        struct sockaddr_dl *sd = p;

                        if (sd->sdl_index == index) {
                                strlcpy(name, lastname, IFNAMSIZ);
                                retname = name;
                                goto ret;
                        }
                }

                if (len < sa->sa_len)
                        goto ret;
				if (sa->sa_len > sizeof(struct sockaddr)) {
					len -= sa->sa_len;
					p += sa->sa_len;
				} else {
					len -= sizeof(struct sockaddr);
					p += sizeof(struct sockaddr);
				}
        }

        if (i == index)
                strlcpy(iname, lastname, sizeof(iname));

        if (iname[0]) {
                strlcpy(name, iname, IFNAMSIZ);
                retname = name;
        }            
ret:
        if (fd != -1)
                close(fd);
        if (ifconf.ifc_buf)
                free(ifconf.ifc_buf);
        return (retname);
}
#endif /* SSC_x86 || SSC_mips */  
