/* vi:ts=4
 *-----------------------------------------------------------------
 *
 * Name: ipmd.c
 *
 * RCS:  $Id: ipmd.c,v 1.40 2002/11/15 04:04:58 shaun Exp $
 *
 * Copyright (C) 2001, Agile Storage, Inc.
 *
 * Description: ipm processing on ssc 
 *
 * Created by:	Su Wang
 *
 * Date Created: 10/11/2001
 *
 *-----------------------------------------------------------------
 */
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/errno.h>
#include <sys/socket.h>
#include <net/if.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netinet/if_ether.h>
#include <sys/types.h>
#include <sys/wait.h>
#if defined(SSC_x86) || defined(SSC_mips)
#include <neteee/eee.h>
#include <neteee/eee_if.h>
#endif /* SSC_x86 SSC_mips */
#include <sys/types.h>
#include <signal.h>

#include "nfx-error.h"
#include "nfx-types.h"
#include "nfx-defs.h"
#include "nfx-ports.h"
#include "exports/anpssc-api.h"
#include "exports/ipm-api.h"
#include "exports/evm-api.h"
#include "exports/ncm-api.h"
#include "exports/event-api.h"
#include "exports/opt-api.h"
#include "exports/event-api.h"
#include "../sm-ipm/ipm.h"
#include "ipmd-stats.h"
#include "ipmd-kern.h"
#include "ipmd-vif.h"

#define	 IPM_PRINTF(a)

/* msg process functions */
static void  ipmd_ProcessMsgFromSSC(void*, int, char*);
static void  ipmd_ProcessMsg(void*, int, uint32, uint32, char*);	
static int32 ipmd_ProcRespIfMsg(ipm_mhdr_t*, int, int);
static void  ipmd_GetIfStatsD(void*, char*);
static void  ipmd_GetCardStatsD(void*, char*);
static void  ipmd_GetConnD(void*, char*);
static void  ipmd_GetConfigD(void*, char*);
static void	 ipmd_Init();
static void  ipmd_SendCcm(ipm_mhdr_t*);
static void  ipmd_ProcessEvent(void*);

/* ipm stats */
ipm_stats_t		ipm_stats;

/* application id for IPM */
int				ipm_appid = 0;
int				ipm_sockid = 0;

/* on/off switch */
int				ipmd_active = 1;

/* allow large buffer for stats retrival */
uint64	resp_buf[IPM_SSC_BUFSIZE];

/* use static ssc slot and cpu id for now */
int		ipmd_ssc_slot = 0;
int		ipmd_ssc_cpu  = 0;
int		ipmd_txrx_slot = 1;
int		ipmd_txrx_cpu = 0;



/*
 *-----------------------------------------------------------------
 *
 * Name : ipmd_ProcessEvent
 *
 * Description: process msg from event manager
 *
 * Created by: Su Wang
 *
 * Data Created: 07/22/02
 *
 *-----------------------------------------------------------------
 */
static void
ipmd_ProcessEvent(void*	msg)
{
	nfx_eventMsg_t		*eventMsgPtr;
	nfx_eventType_t		type;
	ipm_ip_t			*ipm_ipp;
	int					rc, i;
	int					count =0, need=0;
	char				buf[64], *ifName;
	int				slot, cpu, state;
	
	eventMsgPtr = (nfx_eventMsg_t*)msg;
	type = eventMsgPtr->eventType;

	
	switch(type) {
	case NFX_EVENT_CPU:
		slot = eventMsgPtr->param.cpuEvent.slot;
		cpu = eventMsgPtr->param.cpuEvent.cpu;
		state = eventMsgPtr->param.cpuEvent.state;

		IPM_PRINTF(("event %d, state = %d, slot %d, cpu %d\n", 
					type, state, slot, cpu));
		
		/* removfe all interfaces on ssc at fp down event */
		if ((slot == NFX_SLOTTYPE_NFP) && (cpu == NFX_NFP_TXRX1) &&
		    (state == EVENT_STATE_DOWN)) {
			
			ipm_ipp = (ipm_ip_t*)resp_buf;
			rc = ipmd_GetKernIpByName(NULL, ipm_ipp, IPM_SSC_BUFSIZE*8, 
						  &count, &need);
			if (rc != IPM_GET_OK) {
				IPM_LOGERR(IPM_ERR_KERN);
				break;
			}
			
			formPortName(NULL,bt_fp, slot, 0, buf);
			ifName = ((char*)buf) + 6;
						 
			for (i = 0; i < count; i++, ipm_ipp++) {
				/* Remove ip interfaces, compare only
				 * the first three characters, linkid is
				 * variable
				 */
				if (strncmp(ipm_ipp->ip_ifname, ifName, 3) == 0) {
					IPM_PRINTF(("remove if %s address %x\n", ipm_ipp->ip_ifname,
								ipm_ipp->ip_addr));
					ipmd_DelKernIp(ipm_ipp->ip_ifname, ipm_ipp->ip_addr);
				}
			}
			
		}

#if	defined(LINK_AGG)
		/* restore all the virtual interface */
		if ((slot == NFX_SLOTTYPE_NFP) && (cpu == NFX_NFP_TXRX1) &&
		    (state == EVENT_STATE_UP)) {
			ipmd_InitVif();
		}
#endif	/* LINK_AGG */
		break;
		
	default:
	}
	
	return;
}

/*
 *-----------------------------------------------------------------
 *
 * Name : ipmd_GetIfStatsD
 *
 * Description: daemon retrive requested stats and return to 
 *				requestor.  
 *
 * Created by: Su Wang
 *
 * Data Created: 02/13/01
 *
 *-----------------------------------------------------------------
 */
static void 
ipmd_GetIfStatsD(void *msg, char *app)
{
	int				msgsize;
	int				bufsize = IPM_SSC_BUFSIZE*sizeof(uint64);
	int				count = 0, need = 0;
	ipm_ifstat_t	*ipm_ifstatp;
	ipm_mhdr_t		*ipm_hdrp = (ipm_mhdr_t*)msg;
	ipm_mstat_req_t *ipm_stat_reqp = (ipm_mstat_req_t*)(ipm_hdrp + 1);
	char			*ifname;
	int				slot, link, type;
	int				rc;
	
	/* copy back the request */
	msgsize= sizeof(ipm_mhdr_t) + sizeof(ipm_mstat_req_t);
	memcpy((char*)resp_buf, (char*)msg, msgsize);
	
	
	/* start of copy buffer */
	ipm_ifstatp = (ipm_ifstat_t*)(((char*)resp_buf) + msgsize);
	
	/* calculate remaining buffer size */
	bufsize -= sizeof(ipm_mhdr_t) + sizeof(ipm_mstat_req_t);

	/* is interface specified */
	if (ipm_stat_reqp->req.ifname[0] == '\0')
		ifname = 0;
	else
		ifname = ipm_stat_reqp->req.ifname;
	
	/* if name valid */
	if (ifname && 
		(ipmd_ParseIfName(ifname, &type, &slot, &link) != NFX_OK)) {

		//printf("ifname %s no good\n", ifname);
		/* unrecoginzed intf, reply err already */
		rc = IPM_GET_ERR;
		goto out;
	}
	
	/*
	 * retrive from kernel or database 
	 */
	if (ifname)	{				/* request for single device */
#if defined(SSC_x86) || defined(SSC_mips)
		if (slot == ipmd_ssc_slot) /* get device from kernel */
			rc = ipmd_GetKernIfStats(ifname, ipm_ifstatp, 
									 bufsize, &count, &need);
		else					/* get from stats mem  */
#endif /* SSC_x86, SSC_mips */
			rc = ipmd_GetTxRxIfStats(ifname, ipm_ifstatp, 
									 bufsize, &count, &need);

		if (rc != IPM_GET_OK) {
			count = 0;
			goto out;
		}
		
		msgsize += sizeof(ipm_ifstat_t);
	}
	else {						/* request for all device */
		
#if defined(SSC_x86) || defined(SSC_mips)
		/* get stats from kernel */
		rc = ipmd_GetKernIfStats(NULL, ipm_ifstatp, 
								 bufsize, &count, &need);
		
		if (rc != IPM_GET_OK) {
			count = 0;
			goto out;
		}
		
		/* get from kernel successful, advance cp buf ptr etc.*/
		ipm_ifstatp += count;
		bufsize -= count*sizeof(ipm_ifstat_t);
#endif /* SSC_x86, SSC_mips */
		
		/* get stats from stats mem */
		rc = ipmd_GetTxRxIfStats(NULL, ipm_ifstatp, 
								 bufsize, &count, &need);

		if (rc != IPM_GET_OK) {
			count = 0;
			goto out;
		}
		
		/* get from stats mem successful, calculate msgsize,
		 * note count is accumulative from previous
		 * ipmd_GetKernIfStats()
		 */
		msgsize  += count*sizeof(ipm_ifstat_t);
	}

	/* send reply back */
out:
	/* update msg size */
	ipm_hdrp = (ipm_mhdr_t*)resp_buf;
	ipm_hdrp->ipmh_size = msgsize;

	/* update num. of record copied */
	ipm_stat_reqp = (ipm_mstat_req_t*)(ipm_hdrp + 1);
	ipm_stat_reqp->count = count;	
	
	if (rc == IPM_GET_ERR) {
		ipm_hdrp->ipmh_flags &= ~(IPM_MSG_CMD_OK|IPM_MSG_CMD_NOBUF);
	}
	else if (rc == IPM_GET_NO_BUF) {
		ipm_hdrp->ipmh_flags &= ~IPM_MSG_CMD_OK;
		ipm_hdrp->ipmh_flags |= IPM_MSG_CMD_NOBUF;
		IPM_LOGERR(IPM_ERR_BUF);
	}
	else						/* get successful */
		ipm_hdrp->ipmh_flags |= IPM_MSG_CMD_OK;

	ipmd_SendMsg((void*)ipm_hdrp, msgsize, ipmd_ssc_slot, 
				 ipmd_ssc_cpu, app, NULL, 0, 0, ipm_sockid);
	
	return;
}


/*
 *-----------------------------------------------------------------
 *
 * Name : ipmd_GetCardStatsD
 *
 * Description: daemon retrive requested stats and return to 
 *				requestor.  
 *
 * Created by: Su Wang
 *
 * Data Created: 7/25/01
 *
 *-----------------------------------------------------------------
 */
static void 
ipmd_GetCardStatsD(void *msg, char *app)
{
	int				msgsize;
	int				bufsize = IPM_SSC_BUFSIZE*sizeof(uint64);
	int				count = 0, need = 0;
	ipm_mhdr_t		*ipm_hdrp = (ipm_mhdr_t*)msg;
	ipm_mstat_req_t *ipm_stat_reqp = (ipm_mstat_req_t*)(ipm_hdrp + 1);
	int				slot;
	int				rc;	
	int				mtype;
	void*			stats_ptr;
	int				statsize;
	int32			(*txrx_func)(int32, void*, int, int*, int*);
	int32			(*kern_func)(void*, int, int*, int*);


	/* copy back the request */
	msgsize= sizeof(ipm_mhdr_t) + sizeof(ipm_mstat_req_t);
	memcpy((char*)resp_buf, (char*)msg, msgsize);
	
	/* calculate remaining buffer size */
	bufsize -= sizeof(ipm_mhdr_t) + sizeof(ipm_mstat_req_t);

	/* which slot requestor what stats from */
	slot = ipm_stat_reqp->req.slot;
	
	/* which function to call */
	mtype = ipm_hdrp->ipmh_mtype;
	switch (mtype) {
	case IPM_MSG_IP_STATS_REQ:
		statsize = sizeof(ipm_icpstat_t);
		txrx_func = ipmd_GetTxRxIpStats;
#if defined(SSC_x86) || defined(SSC_mips)
		kern_func = ipmd_GetKernIpStats;
#endif /* SSC_x86, SSC_mips */
		break;

	case IPM_MSG_ICMP_STATS_REQ:
		statsize = sizeof(ipm_ipstat_t);
		txrx_func = ipmd_GetTxRxIcmpStats;
#if defined(SSC_x86) || defined(SSC_mips)
		kern_func = ipmd_GetKernIcmpStats;
#endif /* SSC_x86, SSC_mips */
		break;

	case IPM_MSG_TCP_STATS_REQ:
		statsize = sizeof(ipm_tcpstat_t);
		txrx_func = ipmd_GetTxRxTcpStats;
#if defined(SSC_x86) || defined(SSC_mips)
		kern_func = ipmd_GetKernTcpStats;
#endif /* SSC_x86, SSC_mips */
		break;

	case IPM_MSG_UDP_STATS_REQ:
		statsize = sizeof(ipm_udpstat_t);
		txrx_func = ipmd_GetTxRxUdpStats;
#if defined(SSC_x86) || defined(SSC_mips)
		kern_func = ipmd_GetKernUdpStats;
#endif /* SSC_x86, SSC_mips */
		break;
		
	default:
		IPM_LOGERR(IPM_ERR_MTYPE);
		rc = IPM_GET_ERR;
		goto out;
	}
	

	/* 
	 * retrive stats from kernel and database */
	stats_ptr = (void*)(((char*)resp_buf) + msgsize);
	if (slot == -1) {		/* get all stats from all card */
#if defined(SSC_x86) || defined(SSC_mips)
		rc = (*kern_func)(stats_ptr, bufsize, &count, &need);
		
		if (rc != IPM_GET_OK) {
			count = 0;
			goto out;
		}
		
		stats_ptr = ((char*)stats_ptr) + count*statsize;
		bufsize -= count*statsize;
#endif /* SSC_x86, SSC_mips */

		rc = (*txrx_func)(slot, stats_ptr, bufsize, &count, &need);
		
		if (rc != IPM_GET_OK) {
			count = 0;
			goto out;
		}
			
		msgsize += count*statsize;
	}
	else {					/* single card */
#if defined(SSC_x86) || defined(SSC_mips)
		if (slot == ipmd_ssc_slot)
			rc = (*kern_func)(stats_ptr, bufsize, &count, &need);
		else
#endif /* SSC_x86, SSC_mips */
			rc = (*txrx_func)(slot, stats_ptr, bufsize, &count, &need);

		if (rc != IPM_GET_OK) {
			count = 0;
			goto out;
		}
		
		msgsize += count*statsize;
	}

		
	/* send reply back */
out:
	/* update return msg size */
	ipm_hdrp = (ipm_mhdr_t*)resp_buf;
	ipm_hdrp->ipmh_size = msgsize;

	/* update num. of record copied */
	ipm_stat_reqp = (ipm_mstat_req_t*)(ipm_hdrp + 1);
	ipm_stat_reqp->count = count;
	
	if (rc == IPM_GET_ERR) {
		ipm_hdrp->ipmh_flags &= ~(IPM_MSG_CMD_OK|IPM_MSG_CMD_NOBUF);
	}
	else if (rc == IPM_GET_NO_BUF) {
		ipm_hdrp->ipmh_flags &= ~IPM_MSG_CMD_OK;
		ipm_hdrp->ipmh_flags |= IPM_MSG_CMD_NOBUF;
		IPM_LOGERR(IPM_ERR_BUF);
	}
	else						/* get successful */
		ipm_hdrp->ipmh_flags |= IPM_MSG_CMD_OK;

	ipmd_SendMsg((void*)ipm_hdrp, msgsize, ipmd_ssc_slot, 
				 ipmd_ssc_cpu, app, NULL, 0, 0, ipm_sockid);
	
	return;	
}

/*
 *-----------------------------------------------------------------
 *
 * Name : ipmd_GetConnD
 *
 * Description: daemon retrive requested stats and return to 
 *				requestor.  
 *
 * Created by: Su Wang
 *
 * Data Created: 02/13/01
 *
 *-----------------------------------------------------------------
 */
static void 
ipmd_GetConnD(void *msg, char *app)
{
	int				msgsize;
	int				bufsize = IPM_SSC_BUFSIZE*sizeof(uint64);
	int				count = 0, need = 0;
	ipm_mhdr_t		*ipm_hdrp = (ipm_mhdr_t*)msg;
	ipm_mstat_req_t *ipm_stat_reqp = (ipm_mstat_req_t*)(ipm_hdrp + 1);
	int				rc;	
	int				mtype;
	ipm_conn_t		*connp, *in_connp;
	
	int				index, incount;
	

	/* copy back the request */
	msgsize= sizeof(ipm_mhdr_t) + sizeof(ipm_mstat_req_t);
	memcpy((char*)resp_buf, (char*)msg, msgsize);
	
	/* calculate remaining buffer size */
	bufsize -= sizeof(ipm_mhdr_t) + sizeof(ipm_mstat_req_t);

	/* get destintion buffer */
	connp = (ipm_conn_t*)(((char*)resp_buf) + msgsize);

	mtype = ipm_hdrp->ipmh_mtype;
	switch (mtype) {
	case IPM_MSG_TCP_CTBL_REQ:
		index = (int)ipm_stat_reqp->req.conn.conn_lAddr;
		incount = (int)ipm_stat_reqp->req.conn.conn_rAddr;

		rc = ipmd_GetTxRxTcpConnTable(connp, bufsize, 
									  index, incount, &count, &need);
		
		if (rc == IPM_GET_ERR) {
			count = 0;
			goto out;
		}

		msgsize += count*sizeof(ipm_conn_t);
		break;
		
	case IPM_MSG_UDP_CTBL_REQ:
		index = (int)ipm_stat_reqp->req.conn.conn_lAddr;
		incount = (int)ipm_stat_reqp->req.conn.conn_rAddr;

		rc = ipmd_GetTxRxUdpConnTable(connp, bufsize, 
									  index, incount, &count, &need);

		if (rc == IPM_GET_ERR) {
			count = 0;
			goto out;
		}
		
		msgsize += count*sizeof(ipm_conn_t);
		break;	

	case IPM_MSG_TCP_NXTC_REQ:
		if (ipm_stat_reqp->count >= 0) /* ptr given */
			in_connp = &(ipm_stat_reqp->req.conn);
		else
			in_connp = NULL;
		
		rc = ipmd_GetTxRxTcpNxtConn(in_connp, connp);
		if (rc != IPM_GET_OK)
			goto out;
		
		count = 1;
		msgsize += sizeof(ipm_conn_t);
		break;
		

	case IPM_MSG_UDP_NXTC_REQ:
		if (ipm_stat_reqp->count >= 0) /* ptr given */
			in_connp = &(ipm_stat_reqp->req.conn);
		else
			in_connp = NULL;

		rc = ipmd_GetTxRxUdpNxtConn(in_connp, connp);
		if (rc != IPM_GET_OK)
			goto out;
		
		count = 1;
		msgsize += sizeof(ipm_conn_t);
		break;		

	case IPM_MSG_TCP_CONN_REQ:
		in_connp = &(ipm_stat_reqp->req.conn);
		rc = ipmd_GetTxRxTcpConn(in_connp, connp);
		if (rc != IPM_GET_OK)
			goto out;
		
		count = 1;
		msgsize += sizeof(ipm_conn_t);
		break;
		
	case IPM_MSG_UDP_CONN_REQ:
		in_connp = &(ipm_stat_reqp->req.conn);
		rc = ipmd_GetTxRxUdpConn(in_connp, connp);
		if (rc != IPM_GET_OK)
			goto out;
		
		count = 1;
		msgsize += sizeof(ipm_conn_t);
		break;		

#if defined(SSC_x86) || defined(SSC_mips)		
	case IPM_MSG_TCP_KCTBL_REQ:
		rc = ipmd_GetKernTcpConn(connp, bufsize, &count, &need);
		
		if (rc != IPM_GET_OK) {
			count = 0;
			goto out;
		}
		
		msgsize += count*sizeof(ipm_conn_t);
		break;

	case IPM_MSG_UDP_KCTBL_REQ:
		rc = ipmd_GetKernUdpConn(connp, bufsize, &count, &need);
		
		if (rc != IPM_GET_OK) {
			count = 0;
			goto out;
		}
		
		msgsize += count*sizeof(ipm_conn_t);
		break;
#endif /* SSC_x86, SSC_mips */

	default:
		IPM_LOGERR(IPM_ERR_MTYPE);
		rc = IPM_GET_ERR;
		goto out;
	}
	
	/* send reply back */
out:
	/* update return msg size */
	ipm_hdrp = (ipm_mhdr_t*)resp_buf;
	ipm_hdrp->ipmh_size = msgsize;

	/* update num. of record copied */
	ipm_stat_reqp = (ipm_mstat_req_t*)(ipm_hdrp + 1);
	ipm_stat_reqp->count = count;

	if (rc == IPM_GET_ERR) {
		ipm_hdrp->ipmh_flags &= ~(IPM_MSG_CMD_OK|IPM_MSG_CMD_NOBUF);
	}
	else if (rc == IPM_GET_NO_BUF) {
		ipm_hdrp->ipmh_flags &= ~IPM_MSG_CMD_OK;
		ipm_hdrp->ipmh_flags |= IPM_MSG_CMD_NOBUF;
		IPM_LOGERR(IPM_ERR_BUF);
	}
	else						/* get successful */
		ipm_hdrp->ipmh_flags |= IPM_MSG_CMD_OK;

	ipmd_SendMsg((void*)ipm_hdrp, msgsize, ipmd_ssc_slot, 
				 ipmd_ssc_cpu, app, NULL, 0, 0, ipm_sockid);
	
	return;		
}


/*
 *-----------------------------------------------------------------
 *
 * Name : ipmd_GetConfigD
 *
 * Description: daemon retrive requested stats and return to 
 *				requestor.  
 *
 * Created by: Su Wang
 *
 * Data Created: 02/13/01
 *
 *-----------------------------------------------------------------
 */
static void 
ipmd_GetConfigD(void *msg, char *app)
{
	int				msgsize;
	int				bufsize = IPM_SSC_BUFSIZE*sizeof(uint64);
	int				count = 0, need = 0;
	ipm_mhdr_t		*ipm_hdrp = (ipm_mhdr_t*)msg;
	ipm_mstat_req_t *ipm_stat_reqp = (ipm_mstat_req_t*)(ipm_hdrp + 1);
	char			*ifname = NULL;
	ipm_if_t		*ipm_ifp = 0;
	ipm_ip_t		*ipm_ipp = 0;
	ipm_rt_t		*ipm_rtp = 0;
	ipm_arp_t		*ipm_arpp = 0;
	int				mtype, slot, type,link;
	int				rc;

	
	/* copy back the request */
	msgsize= sizeof(ipm_mhdr_t) + sizeof(ipm_mstat_req_t);
	memcpy((char*)resp_buf, (char*)msg, msgsize);
	
	/* calculate remaining buffer size */
	bufsize -= sizeof(ipm_mhdr_t) + sizeof(ipm_mstat_req_t);	

	/* get msg type */
	mtype = ipm_hdrp->ipmh_mtype;
	if ((mtype == IPM_MSG_IFTBL_REQ)||(mtype == IPM_MSG_IPTBL_REQ)) {
		
		if (ipm_stat_reqp->req.ifname[0] != '\0')
			ifname = ipm_stat_reqp->req.ifname;

		if (ifname && 
			(ipmd_ParseIfName(ifname, &type, &slot, &link) != NFX_OK)) {

			/* unrecoginzed intf, reply err already */
			rc = IPM_GET_ERR;
			goto out;
		}		
	}

	/* make sure ifname is valid */
	switch(mtype) {
	case IPM_MSG_IFTBL_REQ:
		ipm_ifp = (ipm_if_t*)(((char*)resp_buf) + msgsize);
		rc = ipmd_GetKernIfByNameTxRx(ifname, ipm_ifp, bufsize, &count, &need);
		if (rc != IPM_GET_OK) {
			count = 0;
			goto out;
		}
		
		msgsize += count*sizeof(ipm_if_t);
		break;
		
	case IPM_MSG_IPTBL_REQ:
		ipm_ipp = (ipm_ip_t*)(((char*)resp_buf) + msgsize);
		rc = ipmd_GetKernIpByName(ifname, ipm_ipp, bufsize, &count, &need);
		if (rc != IPM_GET_OK) {
			count = 0;
			goto out;
		}
		
		msgsize += count*sizeof(ipm_ip_t);
		break;
		
	case IPM_MSG_RTTBL_REQ:
		ipm_rtp = (ipm_rt_t*)(((char*)resp_buf) + msgsize);
		rc = ipmd_GetKernRtTbl(ipm_rtp, bufsize, &count, &need);
		if (rc != IPM_GET_OK) {
			count = 0;
			goto out;
		}
		
		msgsize += count*sizeof(ipm_rt_t);
		break;

	case IPM_MSG_ARPTBL_REQ:
		ipm_arpp = (ipm_arp_t*)(((char*)resp_buf) + msgsize);
		rc = ipmd_GetKernArpTbl(ipm_arpp, bufsize, &count, &need);
		if (rc != IPM_GET_OK) {
			count = 0;
			goto out;
		}
		
		msgsize += count*sizeof(ipm_arp_t);
		break;

	case IPM_MSG_ROOT_REQ:		/* debuging msg */
	{
		char	*lstr =  ((char*)resp_buf) + msgsize;
		int		lsize;
		int		size;
		
		ipmd_ChkStatsMem(lstr);

		/* give error string 256 bytes */
		lstr += 256;
		lsize = 256;
		
		/* cpy root structure */
		size = sizeof(ipm_root_t) + 
			sizeof(ipm_card_t)*NFX_NUM_SLOTS_PER_CHASSIS;
		lsize += size;
		memcpy(lstr, ipmd_stats_root, size);
		lstr += size;
		
		/* cpy ipm stats */
		size = sizeof(ipm_stats_t);
		lsize += size;
		memcpy(lstr, &ipm_stats, size);
		lstr += size;
		
		rc = IPM_GET_OK;
		msgsize += lsize;
		count = 1;
	}
	break;
	
	default:
		IPM_LOGERR(IPM_ERR_MTYPE);
		rc = IPM_GET_ERR;
		goto out;
	}

	/* send reply back */
 out:
	/* update return msg size */
	ipm_hdrp = (ipm_mhdr_t*)resp_buf;
	ipm_hdrp->ipmh_size = msgsize;

	/* update num. of record copied */
	ipm_stat_reqp = (ipm_mstat_req_t*)(ipm_hdrp + 1);
	ipm_stat_reqp->count = count;
	
	if (rc == IPM_GET_ERR) {
		ipm_hdrp->ipmh_flags &= ~(IPM_MSG_CMD_OK|IPM_MSG_CMD_NOBUF);
	}
	else if (rc == IPM_GET_NO_BUF) {
		ipm_hdrp->ipmh_flags &= ~IPM_MSG_CMD_OK;
		ipm_hdrp->ipmh_flags |= IPM_MSG_CMD_NOBUF;
		IPM_LOGERR(IPM_ERR_BUF);
	}
	else						/* get successful */
		ipm_hdrp->ipmh_flags |= IPM_MSG_CMD_OK;

	if (ipmd_SendMsg((void*)ipm_hdrp, msgsize, ipmd_ssc_slot, 
					 ipmd_ssc_cpu, app, NULL, 0, 0, ipm_sockid) < 0) {
		IPM_PRINTF(("ipmd_GetConfigD: get failed\n"));
	}
	
	return;	
}

ipm_auxIntf_t	ipmd_auxIntf[IPM_LINK_MAX];
/* 
 *-----------------------------------------------------------------
 *
 * Name: ipmd_Init
 *
 * Description: Initialize IPM daemon
 *
 * Created by:	Su Wang
 *
 * Date Created: 10/11/2001
 *
 *-----------------------------------------------------------------
 */
static void
ipmd_Init()
{
	ipm_mhdr_t	*ipm_hdrp;
	int			pid;
	int			status;

	if ((ipmd_active == 1) && (ipm_sockid == 0)) {
		ipm_sockid = regAgileApp("ipm");
		if (ipm_sockid == -1) {
			printf("ipm_Init: invalid app name \n");
			exit(0);
		}
		
		ipm_appid = eee_getApplicationId("ipm"); 
		IPM_PRINTF(("ipmd_registered id = %d\n", ipm_sockid));
	}

	// printf("ippid is %d\n", ipm_sockid);
	
	/* initialize auxiliary structure */
	bzero(ipmd_auxIntf, sizeof(ipmd_auxIntf));

	/* reset stats counter */
	bzero(&ipm_stats, sizeof(ipm_stats));

	/* initiate stats mem */
	if (ipmd_InitStatsMem() != NFX_OK) {
		printf("ipm: initate share mem failed\n");
		exit(0);
	}
   

	/* remap ssc port to save some for txrx */
	if ((pid = fork()) == 0) {
	  /* child process do exec */
	  close (0); close (1); close (2);
	  execl("/sbin/sysctl",
			"sysctl", "-w", "net.inet.ip.portlast=48151", 0);
	  perror("ipmd_init: port remap failed");
	  exit(1);	  
	}
	else {
	  waitpid(pid, &status, 0); 
	}

	ipmd_active = 1;

	/* send msg to txrx about restart */
	ipm_hdrp = IPM_ALLOC(sizeof(ipm_mhdr_t)); 

	/* fill in the header */
	(void)ipm_FillIpmMsgHdr(ipm_hdrp, IPM_MSG_RESTART, sizeof(ipm_mhdr_t));
	ipm_hdrp->ipmh_flags |= IPM_MSG_CMD_NREPLY;
	
	/* send restart message to txrx */
	if (ipmd_SendMsg(ipm_hdrp, ipm_hdrp->ipmh_size, ipmd_ssc_slot, ipmd_ssc_cpu, "ipm", 
					 NULL, 0, 1, ipm_sockid) >= 0) 
		ipm_stats.adm_fwd++;
	else
		ipm_stats.err_fwd++;

	IPM_FREE(ipm_hdrp);

	/* initialize vif */
	ipmd_InitVif();
	
	return;
}

/* 
 *-----------------------------------------------------------------
 *
 * Name: ipmd_ProcessMsg
 *
 * Description: process ipm messages from another card. 
 *
 * Created by:	Su Wang
 *
 * Date Created: 10/11/2001
 *
 *-----------------------------------------------------------------
 */
static void
ipmd_ProcessMsg(void		*msg,	
				int			msgsize,
				uint32		scpu,
				uint32		sslot,
				char		*sapp	
			   )
{
  ipm_mhdr_t	*ipm_hdrp  = (ipm_mhdr_t*)msg;
  // ipm_mintf_t	*ipm_intfp = (ipm_mintf_t*)(ipm_hdrp+1);
  void			*ipm_statp = (ipm_mintf_t*)(ipm_hdrp+1); 
  ipm_mchdr_t	*ipm_connhdrp = (ipm_mchdr_t*)(ipm_hdrp+1);
  int			mtype;

  if (!ipmd_active)
	return;

  /* retrieve action */
  mtype = ipm_hdrp->ipmh_mtype;

  /* currently only IPM_MSG_INTF_ADD is sent by TXRX, */
  /* in the event of startup */
  switch(mtype)
	{
	case IPM_MSG_INTF_NEW:
	case IPM_MSG_INTF_RESP:
	  ipmd_ProcRespIfMsg(ipm_hdrp, sslot, scpu);
	  ipm_stats.dev_rcv++;
	  break;

	case IPM_MSG_INTF_STATS:
	case IPM_MSG_IP_STATS:
	case IPM_MSG_ICMP_STATS:
	case IPM_MSG_TCP_STATS:
	case IPM_MSG_UDP_STATS:
	  if (ipmd_ProcStatMsg(mtype, ipm_statp, sslot) == NFX_OK)
		ipm_stats.info_rcv++;
	  else
		ipm_stats.err_rcv++;
	  break;

	case IPM_MSG_TCP_CONN:
	case IPM_MSG_UDP_CONN:
	  {
		ipm_mconn_t		*ipm_connp = (ipm_mconn_t*)(ipm_connhdrp+1);
		if (ipmd_ProcConnMsg(mtype, sslot, ipm_connhdrp->count, ipm_connp)
			== NFX_OK)
		  ipm_stats.info_rcv++;
		else
		  ipm_stats.err_rcv++; 
		break;
	  }

	case IPM_MSG_RESTART:
		/* if txrx has restarted, we need to reset stats 
		 * mem. space.
		 */
		ipmd_InitStatsMem();
		break;

	case IPM_MSG_VIF_EVNT:
		ipmd_ProcessVifTxrx(mtype,msg);
		break;
		
	default:
		IPM_LOGERR(IPM_ERR_MTYPE);
		ipm_stats.err_rcv++;
	}
  
  return;
}

#define HOST_FILE_ROOT_NAME "/etc/hostname."

static void
create_hostname_file (char *ifname, uint32 ipaddr, uint32 mask)
{
	char *file_name;
	struct in_addr addr;
	char *addr_str;
	char *msk_str;
	char *inet_str;
	char ourname[256];

	asprintf(&file_name, "%s%s", HOST_FILE_ROOT_NAME, ifname);

	addr.s_addr = ipaddr;
	addr_str = strdup(inet_ntoa (addr));
	addr.s_addr = mask;
	msk_str = strdup(inet_ntoa (addr));
	asprintf(&inet_str, "inet %s %s", addr_str, msk_str);

	if (modify_readonly_file(file_name, string_to_file, inet_str) != 0) {
		fprintf(stderr, "failed to write %s: %s\n",
				file_name, strerror(errno));
	}

	free (file_name);
	free (addr_str);
	free (msk_str);
	free (inet_str);

	gethostname (ourname, sizeof(ourname));
	if (modify_readonly_file("/etc/hosts", redo_hosts, ourname) != 0) {
		fprintf(stderr, "failed to write /etc/hosts: %s\n", strerror(errno));
	}
}

static void
whack_hostname_file (char *ifname)
{
	char *file_name;
	char ourname[256];

	asprintf(&file_name, "%s%s", HOST_FILE_ROOT_NAME, ifname);
	modify_readonly_file(file_name, unlink, NULL);
	free (file_name);

	gethostname (ourname, sizeof(ourname));
	if (modify_readonly_file("/etc/hosts", redo_hosts, ourname) != 0) {
		fprintf(stderr, "failed to write /etc/hosts: %s\n", strerror(errno));
	}
}


/*-----------------------------------------------------------------
 *
 * Name: ipmd_ProcessMsgFromSSC
 *
 * Description: process ipm messages from modules on SSC. 
 *
 * Created by:	Su Wang
 *
 * Date Created: 10/11/2001
 *
 *-----------------------------------------------------------------
 */
static void
ipmd_ProcessMsgFromSSC(void		*msg,
					   int		msgsize,
					   char		*app
					   )
{
  ipm_mhdr_t	*ipm_hdrp = (ipm_mhdr_t*)msg;
  ipm_mintf_t	*ipm_intfp;
  ipm_mip_t		*ipm_ipp;
  ipm_mrt_t		*ipm_rtp;
  ipm_marp_t	*ipm_arpp;
  int			slot;
  int			cpu;
  int			link;
  char			ifbuf[IPM_STR_SIZE];
  char			*ifname = 0;
  ipm_if_t		ipm_if;
  ipm_ip_t		ipm_ip;
  int			count=0, need = 0;
  int32			mtype = ipm_hdrp->ipmh_mtype;
  int			reply = 1, rc = NFX_OK;
  int			is_ssc_mgmt;
  int			local = 0;
  uint32		gw;
  int			send_more = 0;
  
//  ipm_auxIntf_t *ipmd_auxIntfPtr = 0;
  char	value[64];
  char	option[64];

  if ((!ipmd_active) && (mtype != IPM_MSG_RESTART))
	return;


  IPM_PRINTF(("msg rcved, mtype = %u\n", mtype));	

  /* process event */
  if (mtype == NFX_EVENT_POST) {
	  ipmd_ProcessEvent((void*)msg);
	  return;
  }
  
  /* handle virtual interface in different function
   * too messy here
   */
  if (IPMD_VIF_PROC_SSC(mtype)) {
	  ipmd_ProcessVif(mtype, msg, app);
	  return;
  }

  /* make sure interface exist */
  if ((mtype == IPM_MSG_IP_ADD) ||
	  (mtype == IPM_MSG_IP_DEL) ||
	  (mtype == IPM_MSG_INTF_ENA) ||
	  (mtype == IPM_MSG_INTF_DIS) ||
	  (mtype == IPM_MSG_INTF_SETMTU) ||
	  (mtype == IPM_MSG_INTF_AUTOEN) ||
	  (mtype == IPM_MSG_INTF_AUTODIS)) {
	  
	  ipm_intfp = (ipm_mintf_t*)(ipm_hdrp + 1);
	  slot = ipm_intfp->if_slot;
	  cpu  = ipm_intfp->if_cpu;
	  link = ipm_intfp->if_link;

	  link =  GET_ULINK_FROM_CPULINK(cpu, link);

	  formPortName(NULL, (boardType_t)ipm_intfp->if_dtype, slot, link, ifbuf);
	  /* skip "local:" */
	  ifname = (char*)((char*)ifbuf + 6);

	  if ((strcmp (ifname, "sc0") == 0) || /* bp interface */
		  (ipmd_GetKernIfByName(ifname, &ipm_if, sizeof(ipm_if), &count, &need)
		  == IPM_GET_ERR || count == 0)) {
		  /* no interface found, send reply bck already */
		  ipm_hdrp->ipmh_flags |= IPM_MSG_CMD_NOINTF;
		  rc = NFX_ERR;
		  goto fail;
	  }


	  /* we need to fill in the intf content */
	  ipm_intfp->if_flags = ipm_if.if_flags;
	  ipm_intfp->if_dhdrlen = 14; /* fixed value, */
	  ipm_intfp->if_daddrlen = 6; /* we might have to get from kernel */
	  memcpy(ipm_intfp->if_mac, ipm_if.if_mac, 6);

	  /* no need to set mtu, unless requested */
	  /* IPM_MSG_INTF_SETMT already has updated mtusize */
	  //if (mtype != IPM_MSG_INTF_SETMTU)
	  // ipm_intfp->if_mtusize = ipm_if.if_mtusize;

	  /* locality of interface */
	  if (slot == ipmd_ssc_slot) 
		  local = 1;
  }

  /* chk the locality of command destination
   * if route/arp/interface is local to ssc, 
   * no need to send to txrx.
   */ 
  if ((mtype == IPM_MSG_RT_ADD) || 
	  (mtype == IPM_MSG_ARP_ADD)) {
	  
	  char			node[24];
	  boardType_t	btype;

	  if ((mtype == IPM_MSG_RT_ADD) ||
		  (mtype == IPM_MSG_RT_DEL)) {
		  ipm_rtp = (ipm_mrt_t*)(ipm_hdrp+1);
		  gw = ipm_rtp->rt_nhop;
	  }
	  else {
		  ipm_arpp = (ipm_marp_t*)(ipm_hdrp+1);
		  gw = ipm_arpp->arp_addr;
	  }
	
	  /* find interface that associated with route or arp */
	  rc = ipmd_GetKernIpByIp(gw, 0, &ipm_ip, sizeof(ipm_ip), &count, &need);
	  
	  /* if no interface associated, assume destination
	   * is local
	   */
	  if (rc == IPM_GET_OK) {
		  
		  /* invalid interface */
		  if (getPortInfo(ipm_ip.ip_ifname,  node, &btype, &slot, &link)) {
			  IPM_LOGERR(IPM_ERR_KERN);
			  rc = NFX_ERR;
			  goto fail;
		  }
	  
		  /* chk for locality */
		  if (slot == ipmd_ssc_slot)
			  local = 1;
	  }
	  else if (mtype == IPM_MSG_RT_ADD) {
		  /* if gw not valid, return err */
		  ipm_hdrp->ipmh_flags |= IPM_MSG_CMD_NOGW;
		  rc = NFX_ERR;
		  goto fail;
	  }
  }


/*
	Special processing for the SSC management interfaces "sc1" and "sc2"
	because we need to create/delete the /etc/hostname.<if> files since
	those are used by BSD at boot up.  
 */
  is_ssc_mgmt = ifname && (!strcmp (ifname, "sc1") || !strcmp (ifname, "sc2"));

  /* intialize flags */
  ipm_hdrp->ipmh_flags = IPM_MSG_CMD_OK;

  switch (mtype)
	{
	case IPM_MSG_IP_ADD:
		ipm_ipp = (ipm_mip_t*)(ipm_intfp+1);

		/* don't allow 0 address added, this sometime happens 
		 * I don't know how it could happen.
		 */
		if (!ipm_ipp->ip_addr || !ipm_ipp->ip_mask) {
			rc = NFX_ERR;
			break;
		}

#if defined(LINK_AGG)
	  /* cannot add ip to a link in virtual interface */
	  {
		  ipm_vif_t	*vifp;
		  if ((vifp = ipmd_FindVifByLink(ifname))) {
			  ipm_hdrp->ipmh_flags |= IPM_MSG_CMD_NOINTF;
			  rc = NFX_ERR;
			  break; 
		  }
	  }
#endif /* LINK_AGG */

		/* make sure no duplicated ip address */
		rc = ipmd_GetKernIpByIp(ipm_ipp->ip_addr, 1, 
								&ipm_ip, sizeof(ipm_ip), &count, &need);
		if (rc == IPM_GET_OK) {
			/* same IP address assigned to another interface */
			if (strcmp(ifname, ipm_ip.ip_ifname)) {
				ipm_hdrp->ipmh_flags |= IPM_MSG_CMD_DUPIP;
				rc = NFX_ERR;
				break;
			}
		}
		
		rc = ipmd_AddKernIp(ifname, ipm_ipp->ip_addr, ipm_ipp->ip_mask);
		ipm_stats.adm_rcv++;

		if (is_ssc_mgmt) {
			create_hostname_file (ifname, ipm_ipp->ip_addr, ipm_ipp->ip_mask);
		}

		send_more = 1;
		break;

	case IPM_MSG_IP_DEL:
		ipm_ipp = (ipm_mip_t*)(ipm_intfp+1);
		rc = ipmd_GetKernIpByIp(ipm_ipp->ip_addr, 1, 
								&ipm_ip, sizeof(ipm_ip), &count, &need);
		if (rc != IPM_GET_OK ||
			strcmp(ifname, ipm_ip.ip_ifname)) {		/* ip does not exist */
			ipm_hdrp->ipmh_flags &= ~IPM_MSG_CMD_OK;
			ipm_hdrp->ipmh_flags |= IPM_MSG_CMD_NOIP;
			rc = NFX_OK;		/* follow through to txrx, there maybe ip on txrx */
			break;
		}

#if defined(LINK_AGG)
		/* cannot remove ip from a link in virtual interface */
		{
			ipm_vif_t	*vifp;
			if ((vifp = ipmd_FindVifByLink(ifname))) {
				ipm_hdrp->ipmh_flags |= IPM_MSG_CMD_NOINTF;
				rc = NFX_ERR;
				break; 
			}
		}
#endif /* LINK_AGG */

		/* AR 1304, remove arp entries 
		 * contained by this interface.
		 */  
		count = need = 0;
		rc = ipmd_GetKernArpTbl((ipm_arp_t*)resp_buf, IPM_SSC_BUFSIZE*8, &count, &need);
		if (rc == IPM_GET_OK) {
			int			i;
			ipm_arp_t	*ipm_arp;
			
			ipm_arp = (ipm_arp_t*)resp_buf;
		
			for (i=0; i<count; i++) {
				if ((strcmp(ipm_ip.ip_ifname, ipm_arp->arp_ifname)==0) &&
					((ipm_ip.ip_addr & ipm_ip.ip_mask) == 
					 (ipm_arp->arp_addr & ipm_ip.ip_mask))) {
					ipmd_DelKernArp(ipm_arp->arp_addr);
				}
				
				ipm_arp++;
			}
		}
		else
			IPM_LOGERR(IPM_ERR_KERN);
			
		rc = ipmd_DelKernIp(ifname, ipm_ipp->ip_addr);
		ipm_stats.adm_rcv++;

		if (is_ssc_mgmt) {
			whack_hostname_file (ifname);
		}

		break;
		
	case IPM_MSG_INTF_ENA:
		sprintf(option, "%s.%s", OPTIONS_IPM_INTF_STATE, ifname);
		sprintf(value, "%s", "ENABLE");
		if (setRegistryOption(option, value) != NFX_OK) 
			IPM_LOGERR(IPM_ERR_SEND);
		
		rc = ipmd_EnableKernIntf(ifname);
		ipm_stats.adm_rcv++;
		break;

	case IPM_MSG_INTF_DIS:
		sprintf(option, "%s.%s", OPTIONS_IPM_INTF_STATE, ifname);
		sprintf(value, "%s", "DISABLE");
		if (setRegistryOption(option, value) != NFX_OK) 
			IPM_LOGERR(IPM_ERR_SEND);

		rc = ipmd_DisableKernIntf(ifname);
		ipm_stats.adm_rcv++;
		break;

	case IPM_MSG_INTF_SETMTU:
	case IPM_MSG_INTF_AUTOEN:
	case IPM_MSG_INTF_AUTODIS:
		ipm_stats.adm_rcv++;
		if (slot == ipmd_ssc_slot) /* we don't want to change ssc port */
			rc = NFX_ERR;
		else {
	
			/* save configuration to agile.conf */
			if (mtype == IPM_MSG_INTF_SETMTU) {
				sprintf(option, "%s.%s", OPTIONS_IPM_INTF_MTU, ifname);
				sprintf(value, "%d", ipm_intfp->if_mtusize);
			}
			else if (mtype == IPM_MSG_INTF_AUTOEN) {
				sprintf(option, "%s.%s", OPTIONS_IPM_INTF_AUTONEG, ifname);
				sprintf(value, "%s", "ENABLE");
			}
			else {
				sprintf(option, "%s.%s", OPTIONS_IPM_INTF_AUTONEG, ifname);
				sprintf(value, "%s", "DISABLE");
			}

			/* failed to save */
			if (setRegistryOption(option, value) != NFX_OK) 
				IPM_LOGERR(IPM_ERR_SEND);
			
			/* GibE info will be retrived directly from txrx */
#if 0
			ipmd_auxIntfPtr = IPM_GET_AUXINTF(1, link);
			if (mtype == IPM_MSG_INTF_SETMTU)
				ipmd_auxIntfPtr->ipm_auxMtu = ipm_intfp->if_mtusize;
			else if (mtype == IPM_MSG_INTF_AUTOEN)
				ipmd_auxIntfPtr->ipm_auxFlags |= IPM_AUXIF_AUTO;
			else
				ipmd_auxIntfPtr->ipm_auxFlags &= ~IPM_AUXIF_AUTO;
#endif 
		}
		break;
		
	case IPM_MSG_ARP_ADD:
		ipm_arpp = (ipm_marp_t*)(ipm_hdrp+1);
		rc = ipmd_AddKernArp(ipm_arpp->arp_addr, ipm_arpp->arp_mac);
		ipm_stats.adm_rcv++;
		break;

	case IPM_MSG_ARP_DEL:
		ipm_arpp = (ipm_marp_t*)(ipm_hdrp+1);
		rc = ipmd_DelKernArp(ipm_arpp->arp_addr);
		ipm_stats.adm_rcv++;
		break;
		
	case IPM_MSG_RT_ADD:
		ipm_rtp = (ipm_mrt_t*)(ipm_hdrp+1);

		/* Su, 09/07/02.
		 * If route already exist, return err
		 */
		rc = ipmd_GetKernRtTbl((ipm_rt_t*)resp_buf, IPM_SSC_BUFSIZE*8, &count, &need);
		if (rc == IPM_GET_OK) {
			int			i;
			ipm_rt_t	*tmp_rtp;
			
			tmp_rtp = (ipm_rt_t*)resp_buf;
			for (i=0; i<count; i++, tmp_rtp++) 
				if ((tmp_rtp->rt_addr == ipm_rtp->rt_addr) &&
					(tmp_rtp->rt_mask == ipm_rtp->rt_mask)) {
					if (tmp_rtp->rt_nhop != ipm_rtp->rt_nhop) {
						ipm_hdrp->ipmh_flags &= ~IPM_MSG_CMD_OK;
						ipm_hdrp->ipmh_flags |= IPM_MSG_CMD_DUPRT;
						goto reply;	/* not relay !!! */
					}
					else {		/* same gw */
						ipm_hdrp->ipmh_flags |= IPM_MSG_CMD_OK;
						send_more = 1;
						goto relay;
					}
				}
		}
		else
			IPM_LOGERR(IPM_ERR_KERN);	

		rc = ipmd_AddKernRt(ipm_rtp->rt_addr, ipm_rtp->rt_mask, ipm_rtp->rt_nhop);
		ipm_stats.adm_rcv++;
		send_more = 1;
		break;

	case IPM_MSG_RT_DEL:
		ipm_rtp = (ipm_mrt_t*)(ipm_hdrp+1);

		/* Su, 09/16/02.
		 * If route does not exist, do nothing
		 */
		rc = ipmd_GetKernRtTbl((ipm_rt_t*)resp_buf, IPM_SSC_BUFSIZE*8, &count, &need);
		if (rc == IPM_GET_OK) {
			int			i;
			ipm_rt_t	*tmp_rtp;
			int			found = 0;
			
			tmp_rtp = (ipm_rt_t*)resp_buf;
			for (i=0; i<count; i++, tmp_rtp++) {
				if ((tmp_rtp->rt_addr == ipm_rtp->rt_addr) &&
					(tmp_rtp->rt_mask == ipm_rtp->rt_mask)) {
			
					if (tmp_rtp->rt_nhop == 0) { /* route sprout from IP if */
						ipm_hdrp->ipmh_flags &= ~IPM_MSG_CMD_OK;
						ipm_hdrp->ipmh_flags |= IPM_MSG_CMD_NORT;
						rc = NFX_OK;
						goto relay;
					}
                    /* Shaun 11/14/02
                     * Fixed #2737
                     * For default route, check next hop
                     */
                    if ((tmp_rtp->rt_addr == 0) &&
                        (tmp_rtp->rt_mask == 0)) {

                        if (tmp_rtp->rt_nhop == ipm_rtp->rt_nhop) {
                            found = 1;
                            break;
                        }
                    }
                    else {
                        found = 1;
                        break;
                    }

                }
            }

			if (!found) {
				ipm_hdrp->ipmh_flags &= ~IPM_MSG_CMD_OK;
				ipm_hdrp->ipmh_flags |= IPM_MSG_CMD_NORT;
				rc = NFX_OK;
				goto relay;		/* give to txrx */
			}
		}
		else
			IPM_LOGERR(IPM_ERR_KERN);	

		rc = ipmd_DelKernRt(ipm_rtp->rt_addr, ipm_rtp->rt_mask, ipm_rtp->rt_nhop);
		ipm_stats.adm_rcv++;
		break;

	case IPM_MSG_SHUTDOWN:
		rc = NFX_OK;
		ipmd_active = 0;
		break;

	case IPM_MSG_RESTART:
		// IPM_PRINTF("restart rcved \n");
	  ipmd_Init();
	  ipmd_active = 1;
	  ipm_hdrp->ipmh_flags |= IPM_MSG_CMD_OK;
	  goto reply;

	case IPM_MSG_INTF_STATS_REQ:
		ipmd_GetIfStatsD(msg, app);
		return;
		
	case IPM_MSG_IP_STATS_REQ:
	case IPM_MSG_TCP_STATS_REQ:
	case IPM_MSG_UDP_STATS_REQ:
	case IPM_MSG_ICMP_STATS_REQ:
		ipm_stats.req_rcv++;
		ipmd_GetCardStatsD(msg, app);
		return;
		
	case IPM_MSG_TCP_CONN_REQ:
	case IPM_MSG_UDP_CONN_REQ:
	case IPM_MSG_TCP_CTBL_REQ:
	case IPM_MSG_UDP_CTBL_REQ:
	case IPM_MSG_TCP_NXTC_REQ:
	case IPM_MSG_UDP_NXTC_REQ:
	case IPM_MSG_TCP_KCTBL_REQ:
	case IPM_MSG_UDP_KCTBL_REQ:
		ipm_stats.req_rcv++;
		ipmd_GetConnD(msg, app);
		return;

	case IPM_MSG_IFTBL_REQ:
	case IPM_MSG_IPTBL_REQ:
	case IPM_MSG_RTTBL_REQ:
	case IPM_MSG_ARPTBL_REQ:
	case IPM_MSG_ROOT_REQ:
		ipm_stats.req_rcv++;
		ipmd_GetConfigD(msg, app);
		return;

	case IPM_MSG_L2CONN_ENT_REQ:
		ipm_stats.req_rcv++;	/* forward to txrx without delay */
		break;
		
	default:
	  IPM_LOGERR(IPM_ERR_MTYPE);
	  ipm_stats.err_rcv++;
	  return;
	}

fail:
  /* if kernel cmd failed, don't bother relay to txrx */  
  if (rc != NFX_OK) {
	  ipm_hdrp->ipmh_flags &= ~(IPM_MSG_CMD_OK|IPM_MSG_CMD_NOBUF);
	  ipmd_SendMsg(ipm_hdrp, ipm_hdrp->ipmh_size, ipmd_ssc_slot, ipmd_ssc_cpu, 
				   app, NULL, 0, 0, ipm_sockid);
	  return;
  }

  /* need to send to txrx ? */
relay:
  if (!local) {

	  IPM_PRINTF(("relay msg from ssc to txrx \n"));
	  
	  /* we want to brocast configuration info */
	  /* no need to allocate another buffer for receiving */
	  if (ipmd_SendMsg(ipm_hdrp, ipm_hdrp->ipmh_size, ipmd_ssc_slot, ipmd_ssc_cpu, "ipm", 
					   ipm_hdrp, IPM_MSG_MAX_SIZE, 1, -1) > 0) {
		  ipm_stats.adm_fwd++;
	  }
	  else {
		  ipm_stats.err_fwd++;

		  /* reply not received, send again */
		  ipm_hdrp->ipmh_flags &= ~IPM_MSG_CMD_OK;
		  ipm_hdrp->ipmh_flags |= IPM_MSG_CMD_TIMEOUT;
		  if (send_more--) 
			  goto relay;
		  
	  }
  }


  
reply:
  /* need to reply back to requestor */
  if (reply) {
	  IPM_PRINTF(("send reply msg to app %s\n", app));

	  /* send notification to ccm */
	  ipmd_SendCcm(ipm_hdrp);
	  
	  ipmd_SendMsg(ipm_hdrp, ipm_hdrp->ipmh_size, ipmd_ssc_slot , ipmd_ssc_cpu, app, 
				   NULL, 0, 0, ipm_sockid);

  }

  /* if deleting IP interface, 
   * and there still IP interface belongs to
   * same address range, add this interface to
   * re-create route entry 
   */

  if ((mtype == IPM_MSG_IP_DEL) &&
	  (ipm_hdrp->ipmh_flags & IPM_MSG_CMD_OK)) {
	  ipmd_ReplaceIp(ipm_ipp->ip_addr);
  }
  
  return;
}

/* 
 *-----------------------------------------------------------------
 *
 * Name: ipmd_ProcRespIfMsg
 *
 * Description: 
 *
 * Created by:	Su Wang
 *
 * Date Created: 10/11/2001
 *
 *-----------------------------------------------------------------
 */
static int32
ipmd_ProcRespIfMsg(ipm_mhdr_t	*ipm_hdrp,
				   int			sslot,
				   int			scpu)
{
  char			*ifname;
  char			buf[24];
  ipm_if_t		ipm_if;
  int			count = 0, need=0;
  /* user interface */ 
  int			link;
  // ipm_auxIntf_t *auxIntfPtr;
  char			option[64];
  char			value[64];
  ipm_mintf_t	*ipm_intfp = (ipm_mintf_t*)(ipm_hdrp+1);

  link = GET_ULINK_FROM_CPULINK(ipm_intfp->if_cpu, ipm_intfp->if_link);
  
  formPortName(NULL, (boardType_t)ipm_intfp->if_dtype, ipm_intfp->if_slot, link,
			   buf);
  /* skip "local:" */
  ifname = (char*)buf+6;
  
  IPM_PRINTF(("new intf msg rcvd %s\n", ifname));
	  
#if defined(SSC_x86) || defined(SSC_mips)  
  if (ipmd_GetKernIfByName(ifname, &ipm_if, sizeof(ipm_if), &count, &need)
	  == IPM_GET_ERR || 
	  count == 0)
	{
	  /* interface not found, add to kernel */
	  
         struct eee_attachreq req;
          int rc;
 
          req.addr.seee_slot = ipm_intfp->if_slot;
          req.addr.seee_cpu = ipm_intfp->if_cpu;
          req.link = link;
          strcpy(req.name, ifname);
          memcpy(req.ether_addr, ipm_intfp->if_mac, 6);
		  IPM_PRINTF(("new interface %s detected \n", ifname));
          rc = ethereee_ifattach(&req);
		  
	}
	else
		IPM_PRINTF(("interface %s already exist, count =%d\n", ipm_if.if_name, count));
#endif /* SSC_x86 SSC_mips */

  /* enable interface by default */
  ipmd_EnableKernIntf(ifname);

  /* 
   * retrieve interface configuration,
   * and use the same input buffer to 
   * send to txrx.
   */
  /* don't want a reply ??
   * would take too long.
   */
  ipm_hdrp->ipmh_flags = IPM_MSG_CMD_NREPLY;

  /* send to set mtu */
  sprintf(option, "%s.%s", OPTIONS_IPM_INTF_MTU, ifname);
  if (getRegistryOption(option, value) == NFX_OK) {
	  ipm_intfp->if_mtusize = atoi(value);
	  ipm_hdrp->ipmh_mtype = IPM_MSG_INTF_SETMTU;
	  
	  IPM_PRINTF(("intf %s mtu size restore is %d\n",
	  		 ifname, ipm_intfp->if_mtusize));
	  if (ipmd_SendMsg(ipm_hdrp, ipm_hdrp->ipmh_size, ipm_intfp->if_slot, ipm_intfp->if_cpu,
					   "ipm", NULL, 0, 0, -1) < 0)
		  IPM_LOGERR(IPM_ERR_SEND);
  }

  /* sen to set interface state */  
  sprintf(option, "%s.%s", OPTIONS_IPM_INTF_STATE, ifname);
  if (getRegistryOption(option, value) == NFX_OK) {
	  if (strncmp(value, "EN", 2) == 0) {
		  ipm_hdrp->ipmh_mtype = IPM_MSG_INTF_ENA;
		  ipmd_EnableKernIntf(ifname);
	  }
	  else {
		  ipm_hdrp->ipmh_mtype = IPM_MSG_INTF_DIS;
		  ipmd_DisableKernIntf(ifname);
	  }
	  


	  IPM_PRINTF(("intf %s state %s\n",
			 ifname, 
			 (ipm_hdrp->ipmh_mtype == IPM_MSG_INTF_ENA)?"enabled":"disabled"));
	  
	  if (ipmd_SendMsg(ipm_hdrp, ipm_hdrp->ipmh_size, ipm_intfp->if_slot, ipm_intfp->if_cpu,
					   "ipm", NULL, 0, 0, -1) < 0)
		  IPM_LOGERR(IPM_ERR_SEND);  
  }

  /* send to set auto negotiate */
  sprintf(option, "%s.%s", OPTIONS_IPM_INTF_AUTONEG, ifname);
  if (getRegistryOption(option, value) == NFX_OK) {
	  if (strncmp(value, "EN", 2) == 0) 
		  ipm_hdrp->ipmh_mtype = IPM_MSG_INTF_AUTOEN;
	  else 
		  ipm_hdrp->ipmh_mtype = IPM_MSG_INTF_AUTODIS;

	  IPM_PRINTF(("intf %s auto negotiation %s\n",
			 ifname, 
			 (ipm_hdrp->ipmh_mtype == IPM_MSG_INTF_AUTOEN)?"enabled":"disabled"));
	  
	  if (ipmd_SendMsg(ipm_hdrp, ipm_hdrp->ipmh_size, ipm_intfp->if_slot, ipm_intfp->if_cpu,
					   "ipm", NULL, 0, 0, -1) < 0)
		  IPM_LOGERR(IPM_ERR_SEND);  
  }
  
  /* GibE info will be retrived directly from txrx */
#if 0
  /* update auxIntf info */
  auxIntfPtr = IPM_GET_AUXINTF(1, link);
  auxIntfPtr->ipm_auxMtu = ipm_intfp->if_mtusize;
  
  /* operational state */
  if ((ipm_intfp->if_flags & IFF_RUNNING))
	  auxIntfPtr->ipm_auxFlags = IPM_AUXIF_RUNNING;

  /* autonegotiates */
  if  ((ipm_intfp->if_flags & IFF_LINK0))
	  auxIntfPtr->ipm_auxFlags |= IPM_AUXIF_AUTO;
#endif 

  return NFX_OK;
}


/* 
 *-----------------------------------------------------------------
 *
 * Name: ipmd_SendCcm
 *
 * Description:  send configuration to ccm.
 *
 * Created by:	Su Wang
 *
 * Date Created: 3/15/2002
 *
 *-----------------------------------------------------------------
 */
static void
ipmd_SendCcm(ipm_mhdr_t	*ipm_hdrp)
{
	int32				mtype = ipm_hdrp->ipmh_mtype;
	int32				flags = ipm_hdrp->ipmh_flags;
	ipm_mintf_t			*ipm_intfp;
	ipm_mip_t			*ipm_ipp;
	ipm_mrt_t			*ipm_rtp;
	
	nfx_eventMsg_t		evt_msg;

	/* if configuration not ok, don't send */
	if (!(flags & IPM_MSG_CMD_OK))
		return;
	
	/* post the event to eventd */

	memset (&evt_msg, 0, sizeof(evt_msg));
	evt_msg.msgType = NFX_EVENT_POST;
	gethostname (evt_msg.nodeName, sizeof(evt_msg.nodeName));

	switch (mtype) {
	case IPM_MSG_IP_ADD:
	case IPM_MSG_IP_DEL:
		/* get old msg ptr */
		ipm_intfp = (ipm_mintf_t*)(ipm_hdrp + 1);
		ipm_ipp   = (ipm_mip_t*)(ipm_intfp + 1);

		evt_msg.eventType = NFX_EVENT_IP_IF;

		/* fill in msgs */
		formPortName(evt_msg.nodeName, (boardType_t)ipm_intfp->if_dtype, 
					 ipm_intfp->if_slot, ipm_intfp->if_link, 
		             evt_msg.param.ipIfEvent.portName);
		IPM_PRINTF(("port name is %s\n", evt_msg.param.ipIfEvent.portName));
		
		evt_msg.param.ipIfEvent.ipa = ipm_ipp->ip_addr;
		evt_msg.param.ipIfEvent.mask = ipm_ipp->ip_mask;
		if (mtype ==  IPM_MSG_IP_DEL)
			evt_msg.param.ipIfEvent.state = EVENT_STATE_REMOVE;
		else
			evt_msg.param.ipIfEvent.state = EVENT_STATE_UP;
		break;
		
	case IPM_MSG_INTF_ENA:
	case IPM_MSG_INTF_DIS:
		/* get old msg ptr */
		ipm_intfp = (ipm_mintf_t*)(ipm_hdrp + 1);

		evt_msg.eventType = NFX_EVENT_IP_IF;

		/* fill in msgs */
		formPortName(evt_msg.nodeName, (boardType_t)ipm_intfp->if_dtype, 
					 ipm_intfp->if_slot, ipm_intfp->if_link, 
		             evt_msg.param.ipIfEvent.portName);
		IPM_PRINTF(("port name is %s\n", evt_msg.param.ipIfEvent.portName));

		if (mtype == IPM_MSG_INTF_ENA)
			evt_msg.param.ipIfEvent.state = EVENT_STATE_UP;
		else
			evt_msg.param.ipIfEvent.state = EVENT_STATE_DOWN;
		break;
		
	case IPM_MSG_RT_ADD:
	case IPM_MSG_RT_DEL:	
		ipm_rtp = (ipm_mrt_t*)(ipm_hdrp + 1);

		evt_msg.eventType = NFX_EVENT_ROUTE;

		/* fill in msgs */
		evt_msg.param.ipRteEvent.destNet = ipm_rtp->rt_addr;
		evt_msg.param.ipRteEvent.destMask = ipm_rtp->rt_mask;
		evt_msg.param.ipRteEvent.gateWay = ipm_rtp->rt_nhop;
		
		if (mtype == IPM_MSG_RT_DEL)
			evt_msg.param.ipRteEvent.state = EVENT_STATE_REMOVE;
		else
			evt_msg.param.ipRteEvent.state = EVENT_STATE_UP;
		break;
		
	default:
		return;
	}
	
	sendAgileMsg (ipm_sockid, &evt_msg, sizeof(evt_msg),
	              "eventd", NFX_SSC_UNIX, SSC_SLOT, MGMTPLANE,
	              NULL, NULL, 0);
		  
	return;
}

/* 
 *-----------------------------------------------------------------
 *
 * Name: main
 *
 * Description: ipm main function.
 *
 * Created by:	Su Wang
 *
 * Date Created: 10/16/2001
 *
 *-----------------------------------------------------------------
 */
int
main()
{
	uint32		msgbuf[IPM_MSG_MAX_SIZE/4+1];
	ipm_mhdr_t	*ipm_mhdrp = (ipm_mhdr_t*)msgbuf;
	char		app[16];
	uint32		cpu;
	uint32		slot;
	uint32		plane;
	int			rcvbyte;

	ipmd_Init();
	
	kill (getppid(), SIGUSR1);	// Tell MCP we're ready

	while (1) {
		rcvbyte = recvAgileMsg(ipm_sockid,
							   msgbuf,
							   IPM_MSG_MAX_SIZE,
							   0,
							   app,
							   &cpu, 
							   &slot,
							   &plane);
		
		/* check size making sense */
		if (rcvbyte == -1)  {
			IPM_PRINTF(("msg size not matching rcvbyte %d, ipmsize %d\n",
					   rcvbyte,ipm_mhdrp->ipmh_size));
			continue;
		}

		/* rcv size could be larger */
		rcvbyte = ipm_mhdrp->ipmh_size;
		
		/* 
		 * may be we should consider using fork(), if incoming messages 
		 * too heavy.
		 */
	   
		/* msg received from one of ipm msg from txrx */
		if (app && strcmp(app, "ipm") == 0) {
		
	
			/* if msg from the same cpu, something messed up */
			if ((slot == SSC_SLOT) && (cpu == NFX_SSC_UNIX)) {
				IPM_PRINTF(("msg rcved at its origination\n"));
				continue;
			}

			IPM_PRINTF(("msg rcvd from txrx slot %d\n", slot));
			//	bsd_ShowPkt((char*)ipm_mhdrp, rcvbyte);

			ipmd_ProcessMsg(ipm_mhdrp, rcvbyte, cpu, slot, app);
		}
		else  { /* msg from other application on ssc */
			
			IPM_PRINTF(("msg rcvd from app %s\n", app));
			// bsd_ShowPkt((char*)ipm_mhdrp, rcvbyte);
			
			ipmd_ProcessMsgFromSSC(ipm_mhdrp, rcvbyte, app);
		}
	}
	
	return 1;
	
}

