/*
 *------------------------------------------------------------------------------
 *
 * Name:	ispfc_ct.h
 *
 * RCS:		$Id: ispfc_ct.h,v 1.2 2001/07/13 00:24:19 raster Exp $
 *
 * Copyright (C) 2001, Agile Storage Inc.
 *
 * Description:	QLogic 2300 FC HAL driver CT (common transport) definitions
 *
 * Created by:	Radek Aster
 *
 * Date Created: 6/20/01
 *------------------------------------------------------------------------------
 */

#ifndef __ISPFC_CT_H__
#define __ISPFC_CT_H__

typedef struct {
	byte		revision;
	byte		in_id[3];
	byte		fs_type;
	byte		fs_subtype;
	byte		options;
	byte		rsvd1;
	ushort16	command_code;
	ushort16	maximum_resp_size;   /* Max. response data length in 32-bit words */
	byte		rsvd2[4];
} ispfc_ct_req_hdr_t;

typedef struct {
	byte		revision;
	byte		in_id[3];
	byte		fs_type;
	byte		fs_subtype;
	byte		options;
	byte		rsvd1;
	ushort16	response_code;
	ushort16	residual_size;
	byte		rsvd2;
	byte		reason_code;
	byte		explanation_code;
	byte		vendor_unique;
} ispfc_ct_resp_hdr_t;

typedef struct {
	/* CT request header */
	ispfc_ct_req_hdr_t	header;

	union {
		/* Name Server queries */
		/* **** Command definition for GPN_ID query **** */	
		struct {	
			byte	resvd;
			byte	portid[3];	  /* PortID being queried */
		} GPN_ID;

		/* **** Command definition for GNN_ID query **** */	
		struct {	
			byte	resvd;
			byte	portid[3];	  /* PortID being queried */
		} GNN_ID;

		/* **** Command definition for GFPN_ID query **** */	
		struct {	
			byte	resvd;
			byte	portid[3];	  /* PortID being queried */
		} GFPN_ID;

		/* **** Command definition for GID_FT query **** */	
		struct {	
			byte	resvd[3];
			byte	fc_4_type_code;	  /* FC_4 type code */
		} GID_FT;

		/* **** Command definition for RFT_ID query **** */	
		struct {	
			byte	resvd;
			byte	portid[3];	  /* PortID being registered */
			byte	fc_4_types[32];	  /* FC_4 type bitmap */
		} RFT_ID;

		/* **** Command definition for RSPN_ID query **** */	
		struct {	
			byte	resvd;
			byte	portid[3];	  /* PortID being registered */
			byte	str_length;
			byte	string[255];	  /* Symbolic name */
		} RSPN_ID;

		/* Management Server queries */
		/* **** Command definition for GFN query **** */
		struct {
			byte	ie_name[8];
		} GFN;

		/* **** Command definition for GIELN query **** */
		struct {
			byte	ie_name[8];
		} GIELN;

		/* **** Command definition for GPL query **** */
		struct {
			byte	ie_name[8];
		} GPL;

		/* **** Command definition for GPPN query **** */
		struct {
			byte	port_name[8];
		} GPPN;

		/* **** Command definition for GAPNL query **** */
		struct {
			byte	port_name[8];
		} GAPNL;
	} u;
} ispfc_ct_req_t;

#define MAX_PORTIDS		256
typedef struct {
	uint32	control_byte:8, portid:24;
} ispfc_ct_resp_GID_FT_t;

#define MAX_IES			16
typedef struct {
	byte  ie_name[8];
	byte  reserved[3];
	byte  ie_type;
} ispfc_ct_resp_ie_t;

#define MAX_PORTS		32
typedef struct {
	byte  port_name[8];
	byte  reserved[3];
	byte  port_type;
} ispfc_ct_resp_port_t;

#define MAX_ATTACHED_PORTS	20 /* XXX This needs to be 120 */
typedef struct {
	byte  port_name[8];
	byte  reserved[2];
	byte  port_flags;
	byte  port_type;
} ispfc_ct_resp_attached_port_t;

#define PORT_TYPE_N	0x01
#define PORT_TYPE_NL	0x02
#define PORT_TYPE_F	0x81
#define PORT_TYPE_FL	0x82
#define PORT_TYPE_E	0x84
#define PORT_TYPE_B	0x85

typedef struct {
	byte  len;
	byte  name[255];
} ispfc_ct_resp_IELN_t;

typedef struct {
	/* CT response header */
	ispfc_ct_resp_hdr_t	header;

	/* Subcommand payload */
	union {
		/* Name Server queries */
		struct {
			/* **** Subcommand definition for GID_FT response **** */
			ispfc_ct_resp_GID_FT_t	list[MAX_PORTIDS];
		} GID_FT;
		struct {
			/* **** Subcommand definition for GPN_ID response **** */
			byte			node_name[8];
		} GPN_ID;
		struct {
			/* **** Subcommand definition for GNN_ID response **** */
			byte			node_name[8];
		} GNN_ID;
		struct {
			/* **** Subcommand definition for GFPN_ID response **** */				       
			byte			node_name[8];
		} GFPN_ID;

		/* Management Server queries */
		struct {
			/* **** Subcommand definition for GIEL response **** */
			uint32			ie_count;
			ispfc_ct_resp_ie_t	ie[MAX_IES];
		} GIEL;
		struct {
			/* **** Subcommand definition for GFN response **** */
			byte			fabric_name[8];
		} GFN;
		struct {
			/* **** Subcommand definition for GIELN response **** */
			ispfc_ct_resp_IELN_t	ie_logical_name;
		} GIELN;
		struct {
			/* **** Subcommand definition for GPL response **** */
			uint32			port_count;
			ispfc_ct_resp_port_t	port[MAX_PORTS];
		} GPL;
		struct {
			/* **** Subcommand definition for GPPN response **** */
			uint32			port_number;
		} GPPN;
		struct {
			/* **** Subcommand definition for GAPNL response **** */
			uint32		   port_count;
			ispfc_ct_resp_attached_port_t attached_port[MAX_ATTACHED_PORTS];
		} GAPNL;
	} u;
} ispfc_ct_resp_t;

/*
 * CT Name Server command definitions
 */
#define CT_NS_CMD_GPN_ID			0x0112
#define CT_NS_CMD_GNN_ID			0x0113
#define CT_NS_CMD_GFPN_ID			0x011c
#define CT_NS_CMD_GID_FT			0x0171
#define CT_NS_CMD_RFT_ID			0x0217
#define CT_NS_CMD_RSPN_ID			0x0218

/*
 * CT Management Server command definitions
 */
#define CT_MS_CMD_GIEL				0x0101
#define CT_MS_CMD_GFN				0x0114
#define CT_MS_CMD_GIELN				0x0115
#define CT_MS_CMD_GPL				0x0118
#define CT_MS_CMD_GPPN				0x0122
#define CT_MS_CMD_GAPNL				0x0124

/*
 * FS_type codes
 */
#define FS_TYPE_NS				0xfc
#define FS_TYPE_MS				0xfa

/*
 * FS_subtype codes
 */
#define FS_SUB_TYPE_NAME_SERVICE		0x02
#define FS_SUB_TYPE_FABRIC_CONFIG_SERVICE	0x01

/* 
 * FC-4 type codes
 */
#define FC_4_TYPE_FCP				0x08

/*
 * CT response codes
 */
#define SNS_CT_FS_RJT				0x8001
#define SNS_CT_FS_ACC				0x8002

/*
 * CT reason codes
 */
#define SNS_CT_RSN_INVALID_COMMAND_CODE		0x01
#define SNS_CT_RSN_INVALID_VERSION_LEVEL	0x02
#define SNS_CT_RSN_LOGICAL_ERROR		0x03
#define SNS_CT_RSN_INVALID_IU_SIZE		0x04
#define SNS_CT_RSN_LOGICAL_BUSY			0x05
#define SNS_CT_RSN_PROTOCOL_ERROR		0x07
#define SNS_CT_RSN_UNABLE_TO_PERFORM		0x09
#define SNS_CT_RSN_UNSUPPORTED_COMMAND		0x0b

/*
 * External interface
 */
int32 ispfc_ct_send(ct_desc_t *);
int32 ispfc_ct_dispatch_desc(ispfc_cb_t *, ct_desc_t *);
void  ispfc_dispose_of_ct_desc(ispfc_cb_t *, ct_desc_t *, int);
int32 ispfc_ct_desc_start_handler(ispfc_cb_t *, uint32);

/*
 * Internal interfaces 
 */
int32 ispfc_ns_GID_FT(ispfc_cb_t *, ct_desc_t *);
int32 ispfc_ns_GNN_ID(ispfc_cb_t *, uint32, ct_desc_t *);
int32 ispfc_ns_GPN_ID(ispfc_cb_t *, uint32, ct_desc_t *);
int32 ispfc_ns_GFPN_ID(ispfc_cb_t *, uint32, ct_desc_t *);
int32 ispfc_ns_RFT_ID(ispfc_cb_t *, uint32, byte, ct_desc_t *);
int32 ispfc_ns_RSPN_ID(ispfc_cb_t *, uint32, byte, char *, ct_desc_t *);

int32 ispfc_ms_GIEL(ispfc_cb_t *, ct_desc_t *);
int32 ispfc_ms_GFN(ispfc_cb_t *, wwn_t *, ct_desc_t *);
int32 ispfc_ms_GIELN(ispfc_cb_t *, wwn_t *, ct_desc_t *);
int32 ispfc_ms_GPL(ispfc_cb_t *, wwn_t *, ct_desc_t *);
int32 ispfc_ms_GPPN(ispfc_cb_t *, wwn_t *, ct_desc_t *);
int32 ispfc_ms_GAPNL(ispfc_cb_t *, wwn_t *, ct_desc_t *);

#endif
