/*
 *------------------------------------------------------------------------------
 *
 * Name:	ispfc_device.h
 *
 * RCS:		$Id: ispfc_device.h,v 1.8 2002/10/10 22:36:32 raster Exp $
 *
 * Copyright (C) 2001, Agile Storage Inc.
 *
 * Description:	QLogic 2300 FC HAL driver device definitions.
 *
 * Created by:	Radek Aster
 *
 * Date Created: 6/6/01
 *
 *------------------------------------------------------------------------------
 */

#ifndef __ISPFC_DEVICE_H__
#define __ISPFC_DEVICE_H__

#define FOR_ALL_DEVS(_p_) \
        { \
                ispfc_dev_t *_next_; \
                if ((dev = cb->dev_list)) { \
                        do { \
                                _next_ = dev->next; \
                                { _p_ } \
                                dev = _next_; \
                        } while (dev != cb->dev_list); \
                } \
        }

typedef struct {
	uint32	dev_disc_state;
	uint32	dev_disc_retry;
	uint32	dev_disc_deferred_retry;
	uint32	dev_disc_generic_counter_a;
	uint32	dev_disc_generic_counter_b;
	uint32	dev_disc_timer_handle;
	union {
		struct {
			ispfc_position_map_t	fcal_map;
		} loop;
		struct {
			uint32			peer_id[4];
		} point_to_point;
		struct {
			ct_desc_t		ctd;
			ispfc_ct_req_t		ct_req;
			ispfc_ct_resp_t		ct_GID_FT_resp;
			ispfc_ct_resp_t		ct_GNN_ID_resp;
			ispfc_ct_resp_t		ct_GPN_ID_resp;
			ispfc_ct_resp_t		ct_GFPN_ID_resp;
			ispfc_ct_resp_t		ct_GIEL_resp;
			ispfc_ct_resp_t		ct_GFN_resp;
			ispfc_ct_resp_t		ct_GIELN_resp;
			ispfc_ct_resp_t		ct_GPL_resp;
			ispfc_ct_resp_t		ct_GPPN_resp;
			ispfc_ct_resp_t		ct_GAPNL_resp;
		} fabric;
	} u;
	void (*dev_disc_callback)(ispfc_cb_t *, void *, int32);
	void *	dev_disc_cookie;
} ispfc_dev_discovery_state_t;

#define fcal_map u.loop.fcal_map
#define peer_id  u.point_to_point.peer_id

#define ct_req u.fabric.ct_req
#define ct_GID_FT_resp u.fabric.ct_GID_FT_resp
#define ct_GNN_ID_resp u.fabric.ct_GNN_ID_resp
#define ct_GPN_ID_resp u.fabric.ct_GPN_ID_resp
#define ct_GFPN_ID_resp u.fabric.ct_GFPN_ID_resp
#define ct_GIEL_resp u.fabric.ct_GIEL_resp
#define ct_GFN_resp u.fabric.ct_GFN_resp
#define ct_GIELN_resp u.fabric.ct_GIELN_resp
#define ct_GPL_resp u.fabric.ct_GPL_resp
#define ct_GPPN_resp u.fabric.ct_GPPN_resp
#define ct_GAPNL_resp u.fabric.ct_GAPNL_resp

/*
 * Reestablish connection codes
 */
#define REESTABLISH_STATUS_SUCCESS NFX_OK
#define REESTABLISH_STATUS_FAILED  NFX_ERR

void  ispfc_reestablish_connection(ispfc_cb_t *, void (*)(ispfc_cb_t *, void *, int32), void *);
int32 ispfc_device_management_handler(ispfc_cb_t *);
int32 ispfc_timeout_handler(int32, ispfc_cb_t *);

ispfc_dev_t * ispfc_device_new(ispfc_cb_t *, uint32, channel_name_t *);
void          ispfc_device_free(ispfc_cb_t *, ispfc_dev_t *);
int	      ispfc_device_register(ispfc_cb_t *, ispfc_dev_t *);
int	      ispfc_device_deregister(ispfc_cb_t *, ispfc_dev_t *);
int32	      ispfc_device_pause(ispfc_cb_t *, ispfc_dev_t *, int);
int32	      ispfc_device_login(ispfc_cb_t *, ispfc_dev_t *);
int32	      ispfc_device_invalidate(ispfc_cb_t *, ispfc_dev_t *);

ispfc_dev_t * ispfc_device_lookup_by_phys_dev(ispfc_cb_t *, uint32);
ispfc_dev_t * ispfc_device_lookup_by_channel_and_loopid(ispfc_cb_t *, wwn_t, int);
ispfc_dev_t * ispfc_device_lookup_by_portname(ispfc_cb_t *, wwn_t);
ispfc_dev_t * ispfc_device_lookup_by_handle(ispfc_cb_t *, uint32);

void	      ispfc_queue_desc_on_dev_busy_queue(ispfc_cb_t *, ispfc_dev_t *, scsi_desc_t *);
void	      ispfc_queue_desc_on_dev_waiting_queue(ispfc_cb_t *, ispfc_dev_t *, scsi_desc_t *);
void	      ispfc_dequeue_desc_off_dev_busy_queue(ispfc_cb_t *, ispfc_dev_t *, scsi_desc_t *);
scsi_desc_t * ispfc_dequeue_desc_off_dev_waiting_queue(ispfc_cb_t *, ispfc_dev_t *);

void	      ispfc_queue_tmo_on_dev_waiting_queue(ispfc_cb_t *, ispfc_dev_t *, scsi_task_mgmt_t *);
scsi_task_mgmt_t * ispfc_dequeue_tmo_off_dev_waiting_queue(ispfc_cb_t *, ispfc_dev_t *);

void          ispfc_dev_abort_taskset(ispfc_cb_t *, ispfc_dev_t *, byte);

void	      ispfc_dump_devs(ispfc_cb_t *);

#endif
