#include "ispfc_private.h" 
#include <exports/pci-api.h>

uint32 VA_TO_IO32(void *addr)
{
	return((uint32)addr & 0x1FFFFFFF);
}

void
eee_delay(uint32 usecs)
{
}

uint32
pci_configRead32(int bus_id, int dev_id, int offset)
{
	uint32 value;
#if defined(ISPFC_SIM)
	value = 0;
#else
	VERIFY((offset % 4) == 0);
	pci_cfg_read(bus_id, dev_id, 0, offset, &value);
#endif
	return(value);
}

ushort16
pci_configRead16(int bus_id, int dev_id, int offset)
{
	uint32 value;

#if defined(ISPFC_SIM)
	value = 0;
#else
	VERIFY((offset % 2) == 0);
	pci_cfg_read(bus_id, dev_id, 0, offset & 0xFC, &value);
#endif
	switch (offset & 0x3) {
	case 0:
		return((ushort16)value);
	case 2:
		return((ushort16)(value >> 16));
	default:
		VERIFY(0);
	}

	return(NFX_OK);
}

byte
pci_configRead8(int bus_id, int dev_id, int offset)
{
	uint32 value;

#if defined(ISPFC_SIM)
	value = 0;
#else
	pci_cfg_read(bus_id, dev_id, 0, offset & 0xFC, &value);
#endif
	switch(offset & 0x03) {
	case 0:
		return((byte)value);
	case 1:
		return((byte)(value >> 8));
	case 2:
		return((byte)(value >> 16));
	case 3:
		return((byte)(value >> 24));
	default:
		VERIFY(0);
	}

	return(NFX_OK);
}

void
pci_configWrite32(int bus_id, int dev_id, int offset, uint32 value)
{
#if !defined(ISPFC_SIM)
	VERIFY((offset % 4) == 0);
	pci_cfg_write(bus_id, dev_id, 0, offset, value);
#endif
}

void
pci_configWrite16(int bus_id, int dev_id, int offset, ushort16 value)
{
#if !defined(ISPFC_SIM)
	uint32 val1;
	uint32 val2;

	VERIFY((offset % 2) == 0);
	pci_cfg_read(bus_id, dev_id, 0, offset & 0xFC, &val1);
	switch (offset & 0x3) {
	case 0:
		val2 = value;
		val2 = (val2 & 0x0000FFFF) | (val1 & 0xFFFF0000);
		break;
	case 2:
		val2 = value;
		val2 = ((val2 << 16) & 0xFFFF0000) | (val1 & 0x0000FFFF);
		break;
	default:
		VERIFY(0);
	}
	pci_cfg_write(bus_id, dev_id, 0, offset, val2);
#endif /* !defined(ISPFC_SIM) */
}

void
pci_configWrite8(int bus_id, int dev_id, int offset, byte value)
{
#if !defined(ISPFC_SIM)
	uint32 val1;
	uint32 val2;

	pci_cfg_read(bus_id, dev_id, 0, offset & 0xFC, &val1);
	switch (offset & 0x3) {
	case 0:
		val2 = value;
		val2 = ((val2 << 0) & 0x000000FF) | (val1 & 0xFFFFFF00);
		break;
	case 1:
		val2 = value;
		val2 = ((val2 << 8) & 0x0000FF00) | (val1 & 0xFFFF00FF);
		break;
	case 2:
		val2 = value;
		val2 = ((val2 << 16) & 0x00FF0000) | (val1 & 0xFF00FFFF);
		break;
	case 3:
		val2 = value;
		val2 = ((val2 << 25) & 0xFF000000) | (val1 & 0x00FFFFFF);
		break;
	default:
		VERIFY(0);
	}
	pci_cfg_write(bus_id, dev_id, 0, offset, val2);
#endif /* !defined(ISPFC_SIM) */
}
