/*
 *------------------------------------------------------------------------------
 *
 * Name:	ispfc_hash.h
 *
 * RCS:		$Id: ispfc_hash.h,v 1.2 2001/07/13 00:24:19 raster Exp $
 *
 * Copyright (C) 2001, Agile Storage Inc.
 *
 * Description:	QLogic 2300 FC HAL driver hash definitions.
 *
 * Created by:	Radek Aster
 *
 * Date Created: 6/6/01
 *
 *------------------------------------------------------------------------------
 */

#ifndef __ISPFC_HASH_H__
#define __ISPFC_HASH_H__

typedef uint32 ispfc_hash_key_t;
typedef void * ispfc_hash_obj_t;

#define ISPFC_EFFICIENCY_HIST_SIZE	128

#define ISPFC_HASH_TYPE1	1	/* 128 hash buckets */
#define ISPFC_HASH_TYPE2	2	/* 1024 hash buckets */
#define ISPFC_HASH_TYPE3	3	/* ??? */

typedef struct ispfc_hash_entry {
	ispfc_hash_key_t	      key;
	ispfc_hash_obj_t	      object;
	struct ispfc_hash_entry *     next;
	struct ispfc_hash_entry *     prev;
} ispfc_hash_entry_t;

typedef struct {
#ifdef DEBUG
	int			ispfc_efficiency_histogram[ISPFC_EFFICIENCY_HIST_SIZE+1];
#endif
	uint32			(*ispfc_hash_func)(ispfc_hash_key_t);
	int			ispfc_hash_table_size;
	ispfc_hash_entry_t **	ispfc_hash_buckets;
} ispfc_hash_t;

ispfc_hash_t *	 ispfc_hash_table_alloc(uint32);
void		 ispfc_hash_table_free(ispfc_hash_t *);
int		 ispfc_hash_table_obj_insert(ispfc_hash_t *, ispfc_hash_key_t, ispfc_hash_obj_t);
void		 ispfc_hash_table_obj_delete(ispfc_hash_t *, ispfc_hash_key_t, ispfc_hash_obj_t);
ispfc_hash_obj_t ispfc_hash_table_obj_find(ispfc_hash_t *, ispfc_hash_key_t);
void             ispfc_hash_table_dump(ispfc_hash_t *);

#endif /* __ISPFC_HASH_H__ */
