/*
 *------------------------------------------------------------------------------
 *
 * Name:	ispfc_mbox.h
 *
 * RCS:		$Id: ispfc_mbox.h,v 1.9 2002/10/25 22:24:36 raster Exp $
 *
 * Copyright (C) 2001, Agile Storage Inc.
 *
 * Description:	QLogic 2300 FC HAL driver mailbox interface definitions.
 *
 * Created by:	Radek Aster
 *
 * Date Created: 5/24/01
 *
 *------------------------------------------------------------------------------
 */

#ifndef __ISPFC_MBOX_H__
#define __ISPFC_MBOX_H__

typedef struct {
	uint32		mbox_state;
	int32		mbox_timer_handle;
	void (*mbox_callback)(ispfc_cb_t *, void *, int32);
	void *		mbox_cookie;
} mbox_cmd_sm_t;

/*
 * Mailbox state
 */
#define ISPFC_MBOX_STATE_FREE			0
#define ISPFC_MBOX_STATE_BUSY			1

/*
 * Mailbox completion codes
 */
#define MBOX_CMD_STATUS_SUCCESS		NFX_OK
#define MBOX_CMD_STATUS_FAILED		(NFX_ERR-1)
#define MBOX_CMD_STATUS_TIMEOUT		(NFX_ERR-2)

int	ispfc_mbox_start(ispfc_cb_t *, void (*)(ispfc_cb_t *, void *, int32), void *);
int	ispfc_mbox_is_busy(ispfc_cb_t *);
void    ispfc_mbox_abort(ispfc_cb_t *);
void	ispfc_mbox_done(ispfc_cb_t *);

/* Mailbox register status */
#define ISPFC_CS_MASK			0xf000
#define ISPFC_CS_MBOX_STATUS		0x4000  /* Mailbox completion status */
#define ISPFC_CS_AEN_STATUS		0x8000  /* AEN status */

/* Mailbox Command Completion Status Codes */
#define ISPFC_CS_MBOX_COMP		0x4000	/* Command Complete */
#define ISPFC_CS_MBOX_INVALID		0x4001	/* Invalid Command */
#define ISPFC_CS_MBOX_HI_ERROR		0x4002	/* Host interface error */
#define ISPFC_CS_MBOX_FAILED		0x4003	/* Test failed */
#define ISPFC_CS_MBOX_CMD		0x4005	/* Command Error */
#define ISPFC_CS_MBOX_CMD_PARAM		0x4006	/* Command parameter error */
#define ISPFC_CS_MBOX_PORT_ID_USED	0x4007	/* Port ID used */
#define ISPFC_CS_MBOX_LOOP_ID_USED	0x4008	/* Loop ID used */
#define ISPFC_CS_MBOX_ALL_IDS_IN_USE	0x4009	/* All IDs in use */
#define ISPFC_CS_MBOX_NOT_LOGGED_IN	0x400A	/* FFFFFC not logged in */

/* Asynchronous Event Status Codes */
#define ISPFC_CS_SYSTEM_ERROR		0x8002	/* Fatal system error */
#define ISPFC_CS_REQ_XFR_ERROR		0x8003	/* Request transfer error */
#define ISPFC_CS_RSP_XFR_ERROR		0x8004	/* Response transfer error */
#define ISPFC_CS_LIP_OCCURRED		0x8010	/* A lip is done */
#define ISPFC_CS_LOOP_UP		0x8011	/* Loop transitioned to up */
#define ISPFC_CS_LOOP_DOWN		0x8012	/* Loop transitioned to down */
#define ISPFC_CS_LIP_RESET		0x8013	/* LIP Reset was sent out. */
#define ISPFC_CS_PDB_CHANGED		0x8014	/* Port Database changed. */
#define ISPFC_CS_NSDB_CHANGED		0x8015	/* Name Server Database changed. */
#define ISPFC_CS_LIP_F8_RECVD		0x8016	/* LIP F8 recvd (loss of sync)  */
#define ISPFC_CS_LIP_FAILURE		0x8017	/* LIP failure reported */
#define ISPFC_CS_CMD_COMPLETE		0x8020	/* Fast-post command complete */
#define ISPFC_CS_PNT_TO_PNT_CONNECTION	0x8030	/* Made a pnt-to-pnt connection */

/* Mailbox command codes */
#define ISPFC_NOP			0x00	/* No operation */
#define ISPFC_LOAD_RAM			0x01	/* Load RAM */
#define ISPFC_EXEC_FIRM			0x02	/* Execute firmware */ 
#define ISPFC_DUMP_RAM			0x03	/* Dump RAM */
#define ISPFC_WRITE_RAM			0x04	/* Write RAM word */
#define ISPFC_READ_RAM			0x05	/* Read RAM word */
#define ISPFC_REG_TEST			0x06	/* Mailbox register test */
#define ISPFC_VERIFY_CKSUM		0x07	/* Verify checksum */
#define ISPFC_INIT_SRAM			0x08	/* Initialize SRAM (in ROM) */
#define ISPFC_ABOUT_FIRMWARE		0x08	/* About firmware */
#define ISPFC_LOAD_RISC			0x09	/* Load risc ram */
#define ISPFC_DUMP_RISC			0x0a	/* Dump risc ram */
#define ISPFC_LOAD_SRAM_EXT		0x0b	/* Load risc ram extended (in ROM) */
#define ISPFC_DUMP_SRAM_EXT		0x0c	/* Dump risc ram extended (in ROM) */
#define ISPFC_INIT_SRAM_EXT		0x0e	/* Initialize SRAM extended (in ROM) */
#define ISPFC_CKSUM_FIRMWARE		0x0e	/* Checksum firmware */
#define ISPFC_INIT_REQ_QUEUE		0x10	/* Initialize request queue */
#define ISPFC_INIT_RSP_QUEUE		0x11	/* Initialize response queue */
#define ISPFC_EXECUTE_IOCB		0x12	/* Execute I/O command block */
#define ISPFC_STOP_FIRMWARE		0x14	/* Stop firmware */
#define ISPFC_ABORT			0x15	/* Abort specified IOCB */
#define ISPFC_ABORT_DEVICE		0x16	/* Abort device */
#define ISPFC_ABORT_TARGET		0x17	/* Abort target */
#define ISPFC_RESET			0x18	/* Reset all devices on loop */
#define ISPFC_ABORT_QUEUE		0x1c	/* Abort queue */
#define ISPFC_GET_QUEUE_STATUS		0x1d	/* Get device queue status */
#define ISPFC_GET_FIRMWARE_STATUS	0x1f	/* Get firmware status */
#define ISPFC_GET_LOOP_ID		0x20	/* Get initiator LOOP ID */
#define ISPFC_GET_RETRY_COUNT		0x22	/* Get retry count */
#define ISPFC_GET_FW_OPTIONS		0x28	/* Get firmware options */
#define ISPFC_GET_PORT_QUEUE_PARAMS	0x29	/* Get device queue parameters  */
#define ISPFC_SET_RETRY_COUNT		0x32	/* Set retry count */
#define ISPFC_SET_FW_OPTIONS		0x38	/* Set firmware options */
#define ISPFC_SET_PORT_QUEUE_PARAMS	0x39	/* Set port queue parameters */
#define ISPFC_LOOP_PORT_BYPASS		0x40	/* Loop port bypass */
#define ISPFC_LOOP_PORT_ENABLE		0x41	/* Loop port enable */
#define ISPFC_NON_PARTICIPATING		0x43    /* Request non-participating mode */
#define ISPFC_DIAG_LOOP_BACK		0x45	/* Diag loop back */
#define ISPFC_ENHANCED_GET_PORT_DB	0x47	/* Enhanced get port database */
#define ISPFC_RESET_LINK_ERROR_STATUS	0x52	/* Reset link error status */
#define ISPFC_EXECUTE_COMMAND		0x54	/* Priority execute of command	*/
#define ISPFC_3RD_PARTY_PROCESS_LOGOUT	0x55	/* Third party process logout */
#define ISPFC_PORT_LOGOUT		0x56	/* Port logout */
#define ISPFC_DATA_RATE			0x5d	/* Data rate */
#define ISPFC_INITIALIZE_FW		0x60	/* Initialize firmware */
#define ISPFC_GET_ICB			0x61	/* Get initialization control block */
#define ISPFC_INITIATE_LIP		0x62	/* Start a loop initialization */
#define ISPFC_GET_FCAL_POSITION_MAP	0x63	/* Get FC-AL loop map */
#define ISPFC_GET_PORT_DB		0x64	/* Get the port database */
#define ISPFC_CLEAR_ACA			0x65	/* Clear the auto-contigent allegiance */
#define ISPFC_TARGET_RESET	   	0x66	/* Reset a target */
#define ISPFC_CLEAR_TASK_SET		0x67	/* Clear task set */
#define ISPFC_ABORT_TASK_SET		0x68	/* Abort task set */
#define ISPFC_GET_FW_STATE		0x69	/* Get the firmware state */
#define ISPFC_GET_PORT_NAME		0x6a	/* Get port name for loop ID */
#define ISPFC_GET_LINK_STATUS		0x6b	/* Get link status */
#define ISPFC_LIP_RESET			0x6c	/* Initiate a LIP reset */
#define ISPFC_LIP_FOLLOWED_BY_LOGIN	0x72	/* LIP followed up login */
#define ISPFC_PORT_LOGIN		0x74	/* [Re]Initiate PLOGI/PRLI */
#define ISPFC_SET_VENDOR_ID		0x76	/* Set vendor ID */
#define ISPFC_GET_ID_LIST		0x7c	/* Get list of logged in ports */
#define ISPFC_LUN_RESET			0x7e	/* Task management LUN reset command */

/* Additional mailbox command codes for fabric support */
#define ISPFC_GET_PORT_ID		0x20	/* Get fabric assigned initiator port ID */
#define ISPFC_SEND_SNS			0x6e	/* Query fabric name server */
#define ISPFC_LOGIN_FABRIC_PORT		0x6f	/* Login fabric port */
#define ISPFC_SEND_CHANGE_REQUEST	0x70	/* Register for Change Notify */
#define ISPFC_LOGOUT_FABRIC_PORT	0x71	/* Logout fabric port */
#define ISPFC_GET_NAME_LIST		0x75	/* Get port/node name list */
#define ISPFC_SEND_LFA			0x7d	/* Send Remote loop management command */

/* Firmware option definitions for ISPFC_SET_FW_OPTIONS mailbox command */
#define FO_LIP_F8_AEN			0x0001	/* Generate 0x8016 AEN for each received LIP(F8) primitive */
#define FO_LIP_RESET_AEN		0x0002	/* Generate 0x8013 AEN for each received LIP(reset) primitive */
#define FO_NO_LIP_F7_SWITCHING		0x0010	/* Don't switch to LIP(F7) transmitting LIP(F7) after initiating non-LIP(F7) init */
#define FO_LIP_IMMEDIATELY_AFTER_LOS	0x0020	/* Initiate LIP(F7) immediately after Loss-of-Sync */
#define FO_NO_GPIO_CONTROL		0x0040	/* Disable firmware use of GPIO pins */
#define FO_LIP_ERROR_AEN		0x0080	/* Generate 0x8017 AEN for errors encountered during LIP */

/* Return values for ISPFC_GET_FW_STATE mailbox command */
#define ISPFC_WAIT_CONFIG		0x0000    /* ISP waiting for init firm */
#define ISPFC_WAIT_LIP			0x0001    /* ISP is taking part in LIP */
#define ISPFC_WAIT_LOGIN		0x0002    /* ISP is performing login */
#define ISPFC_STATE_READY		0x0003    /* ISP ready for SCSI */
#define ISPFC_WAIT_LOS_REINIT		0x0004    /* LOS detected -- reinitting */

/* Returned by ISPFC_GET_FCAL_POSITION_MAP mailbox command */
typedef struct fcal_position_map {
	byte port_count;
        byte port_map[255];
} ispfc_position_map_t;

/* Returned by ISPFC_GET_PORT_DB mailbox command */
typedef struct {
        byte		options;
        byte		control;
        byte		master_state;
        byte		slave_state;
        uint32		hard_address;
        uint32		port_id;
        byte		nodename[8];
        byte		portname[8];
        ushort16	execution_throttle;
        ushort16	execution_count;
        byte		retry_count;
        byte		_rsrvd0;
        ushort16	resource_allocation;
        ushort16	current_allocation;
        ushort16	queue_head;
        ushort16	queue_tail;
        ushort16	xmit_list_next;
        ushort16	xmit_list_prev;
        ushort16	common_features;
        ushort16	total_concurrent_sequences;
        ushort16	RO_by_category;
        ushort16	recipient_control_flags;
        ushort16	recv_data_size;
        ushort16	concurrent_sequences;
        ushort16	open_sequences_per_exchange;
        ushort16	_rsrvd1;
        ushort16	_rsrvd2;
        ushort16	stop_queue_head;
        ushort16	stop_queue_tail;
        ushort16	port_retry_timer;
        ushort16	next_sequence_id;
        ushort16	frame_count;
        ushort16	PRLI_payload_len;
        ushort16	PRLI_service_params_0;
        ushort16	PRLI_service_params_3;
        ushort16	loop_id;
        ushort16	ext_lun_info_list_pointer;
        ushort16	ext_lun_info_stop_pointer;
	ushort16	pad[6];
} ispfc_port_database_t;

/* Port Database options definitions */
#define HARD_ADDR_VALID         0x0002
/* Port Database master_state definitions */
#define PORT_LOGGED_IN          6
#define PORT_UNAVAILABLE        7

/* Payload and response definitions for ISPFC_SEND_LFA mailbox command */
typedef struct {
	ushort16	lfa_response_buf_len;
	ushort16	_reserved1_;
	uint32		lfa_response_buf_lo32;
	uint32		lfa_response_buf_hi32;
	ushort16	lfa_command_len;
	ushort16	_reserved2_;
	uint32		lfa_fl_port;
	ushort16	lfa_command_code;
	ushort16	_reserved3_;
	union {
		struct {
			ushort16	lfa_linit_init_function;
			byte		lfa_linit_lip_byte_4;
			byte		lfa_linit_lip_byte_3;
		} linit;
		struct {
			ushort16	_reserved4_;
			ushort16	lfa_lpc_port_control_function;
			ushort16	lfa_lpc_lpb_alpa_bitmap[8];
			ushort16	lfa_lpc_lpe_alpa_bitmap[8];
		} lpc;
	} u;
} ispfc_lfa_cb_t;

#define LFA_COMMAND_CODE_LINIT			0x7000
#define LFA_COMMAND_CODE_LPC			0x7100
#define LFA_COMMAND_CODE_LSTS			0x7200

#define LFA_LINIT_INIT_FUNCTION_NORMAL		0
#define LFA_LINIT_INIT_FUNCTION_FORCE_LOGIN	1

#define LFA_LPC_PORT_CONTROL_FUNCTION_NORMAL	1
#define LFA_LPC_PORT_CONTROL_FUNCTION_INIT	2

typedef struct {
	uint32		lfa_response_code;
	union {
		struct {
			byte	_reserved1_[3];
			byte	lfa_linit_status;
		} linit;
		struct {
			byte	_reserved1_[3];
			byte	lfa_lpc_status;
		} lpc;
		struct {
			byte	_reserved1_;
			byte	lfa_lsts_failed_receiver;
			byte	lfa_lsts_compliance_level;
			byte	lfa_lsts_loop_state;
			byte	lfa_lsts_public_loop_bitmap[16];
			byte	lfa_lsts_private_loop_bitmap[16];
			byte	lfa_lsts_position_map[128];
		} lsts;
	} u;
} ispfc_lfa_resp_t;

#define LFA_RESPONSE_CODE_LS_ACC		0x02000000
#define LFA_RESPONSE_CODE_LS_RJT		0x01000000

#define LFA_LINIT_STATUS_SUCCESS		1
#define LFA_LINIT_STATUS_FAILURE		2

#define LFA_LPC_STATUS_SUCCESS			1
#define LFA_LPC_STATUS_FAILURE			2

#define LFA_LSTS_LOOP_STATE_ONLINE		1
#define LFA_LSTS_LOOP_STATE_LOOP_FAILURE	2
#define LFA_LSTS_LOOP_STATE_INIT_FAILURE	3
#define LFA_LSTS_LOOP_STATE_INITIALIZING	4

#endif /* __ISPFC_MBOX_H__ */
