/*
 *------------------------------------------------------------------------------
 *
 * Name:	ispfc_nvram.h
 *
 * RCS:		$Id: ispfc_nvram.h,v 1.2 2001/07/13 00:24:19 raster Exp $
 *
 * Copyright (C) 2001, Agile Storage Inc.
 *
 * Description:	QLogic 2300 FC HAL driver NVRAM definitions.
 *
 * Created by:	Radek Aster
 *
 * Date Created: 6/05/01
 *
 *------------------------------------------------------------------------------
 */

#ifndef __ISPFC_NVRAM_H__
#define __ISPFC_NVRAM_H__

/*
 * Data bit definitions.
 */
#define BIT_0   0x1
#define BIT_1   0x2
#define BIT_2   0x4
#define BIT_3   0x8
#define BIT_4   0x10
#define BIT_5   0x20
#define BIT_6   0x40
#define BIT_7   0x80
#define BIT_8   0x100
#define BIT_9   0x200
#define BIT_10  0x400
#define BIT_11  0x800
#define BIT_12  0x1000
#define BIT_13  0x2000
#define BIT_14  0x4000
#define BIT_15  0x8000
#define BIT_16  0x10000
#define BIT_17  0x20000
#define BIT_18  0x40000
#define BIT_19  0x80000
#define BIT_20  0x100000
#define BIT_21  0x200000
#define BIT_22  0x400000
#define BIT_23  0x800000
#define BIT_24  0x1000000
#define BIT_25  0x2000000
#define BIT_26  0x4000000
#define BIT_27  0x8000000
#define BIT_28  0x10000000
#define BIT_29  0x20000000
#define BIT_30  0x40000000
#define BIT_31  0x80000000

/*
 * NVRAM Command values.
 */
#define NV_START_BIT	BIT_2
#define NV_WRITE_OP	(BIT_26+BIT_24)
#define NV_READ_OP	(BIT_26+BIT_25)
#define NV_ERASE_OP	(BIT_26+BIT_25+BIT_24)
#define NV_MASK_OP	(BIT_26+BIT_25+BIT_24)

#define NV_DELAY_COUNT		10
#define NV_BYTE_COUNT		256
#define NV_MINIMUM_VERSION	1

/*
 * ISP2300 NVRAM structure definitions. 
 */
typedef struct
{
	/* NVRAM header */
	byte     id[4];					/* 0 */
	byte     nvram_version;				/* 4 */ 
	byte     reserved_0;				/* 5 */

	/* NVRAM RISC parameter block */
	byte     parameter_block_version;		/* 6 */
	byte     reserved_1;
	struct {					/* 8 */
		byte enable_hard_loop_id          :1;	
		byte enable_fairness              :1;
		byte enable_full_duplex           :1;
		byte enable_fast_posting          :1;
		byte enable_target_mode           :1;
		byte disable_initiator_mode       :1;
		byte enable_adisc                 :1;
		byte enable_lun_response          :1;
		byte enable_port_update_event     :1;
		byte disable_initial_lip          :1;
		byte enable_decending_soft_assign :1;
		byte previous_assigned_addressing :1;
		byte enable_stop_q_on_full        :1;
		byte enable_full_login_on_lip     :1;
		byte enable_name_change           :1;
		byte expanded_ifwcb               :1;
	} firmware_options;

	ushort16 frame_payload_size;			/* 10 */
	ushort16 max_iocb_allocation;			/* 12 */
	ushort16 execution_throttle;			/* 14 */
	byte     retry_count;				/* 16 */
	byte     retry_delay;				/* 17 */
	byte     port_name[8];				/* 18 */
	ushort16 adapter_hard_loop_id;			/* 26 */
	byte     inquiry_data;				/* 28 */
	byte     login_timeout;				/* 29 */
	byte     node_name[8];				/* 30 */

	/* Expanded RISC parameter block */
	struct {					/* 38 */
		byte operation_mode               :4;
		byte connection_options           :3;
		byte nonpart_if_hard_addr_failed  :1;
		byte enable_class2                :1;
		byte enable_ack0                  :1;
		byte unused_10                    :1;
		byte unused_11                    :1;
		byte enable_fc_tape               :1;
		byte enable_fc_confirm            :1;
		byte enable_command_reference_num :1;
	} additional_firmware_options;

	byte     response_accum_timer;			/* 40 */
	byte     interrupt_delay_timer;			/* 41 */
	ushort16 reserved_2[14];			/* 42 */

	/* NVRAM host parameter block */
	struct {					/* 70 */
		byte unused_0                :1;
		byte disable_bios            :1;
		byte disable_luns            :1;
		byte enable_selectable_boot  :1;
		byte disable_risc_code_load  :1;
		byte set_cache_line_size_1   :1;
		byte pci_parity_disable      :1;
		byte enable_extended_logging :1;
		byte enable_64bit_addressing :1;
		byte enable_lip_reset        :1;
		byte enable_lip_full_login   :1;
		byte enable_target_reset     :1;
		byte enable_database_storage :1;
		byte unused_13               :1;
		byte unused_14               :1;
		byte unused_15               :1;
	} host_p;

	byte     boot_node_name[8];			/* 72 */
	byte     boot_lun_number;			/* 80 */
	byte     reset_delay;				/* 81 */
	byte     port_down_retry_count;			/* 82 */
	byte     reserved_3;				/* 83 */

	ushort16 maximum_luns_per_target;		/* 84 */

	ushort16 reserved_6[7];				/* 86 */

	ushort16 reserved_7[25];			/* 100 */

	ushort16 reserved_8[25];			/* 150 */

	ushort16 reserved_9[16];			/* 200 */

	/* NVRAM adapter parameter block */
	struct {					/* 232 */
	        byte	 external_GBIC	    :1;
		byte	 RISC_ram_parity    :1;
		byte	 buffer_plus_module :1;
		byte	 unused_3	    :1;
		byte	 unused_4	    :1;
		byte	 unused_5	    :1;
		byte	 unused_6	    :1;
		byte	 unused_7	    :1;
		byte	 unused_8	    :1;
		byte	 unused_9	    :1;
		byte	 unused_10	    :1;
		byte	 unused_11	    :1;
		byte	 unused_12	    :1;
		byte	 unused_13	    :1;
		byte	 unused_14	    :1;
		byte	 unused_15	    :1;

		ushort16 unused1;
		ushort16 unused2;
		ushort16 unused3;
	} adapter_p;

	ushort16 reserved_10[2];			/* 240 */

	ushort16 subsystem_vendor_id;			/* 244 */

	ushort16 reserved_11;				/* 246 */

	ushort16 subsystem_device_id;			/* 248 */

	ushort16 reserved_12[2];			/* 250 */
	byte     reserved_13;				/* 254 */
	byte     checksum;				/* 255 */
} nvram2300_t;

ushort16 ispfc_get_nvram_word(ispfc_cb_t *, ushort16);
int      ispfc_nvram_config(ispfc_cb_t *);

#endif
