/*
 *-----------------------------------------------------------------
 *
 * Name:	atob.c
 *
 * RCS:  $Id: atob.c,v 1.2 2002/11/22 19:44:54 maximk Exp $
 *
 * Copyright (C) 2001, Agile Storage, Inc.
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	5/31/01
 *
 *-----------------------------------------------------------------
 */

#include <nfx-incl.h>

/*-----------------------------------------------------------------
 * Name:	_getbase()
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	5/31/01
 *
 *-----------------------------------------------------------------
 */
char8 *
_getbase (
	char8	* p,
	int32	* basep
)
{
	if (p[0] == '0') {
		switch (p[1]) {
				case 'x':
						*basep = 16;
						break;
				case 't': case 'n':
						*basep = 10;
						break;
				case 'o':
						*basep = 8;
						break;
				default:
						*basep = 10;
						return (p);
		}
		return (p + 2);
	}
	*basep = 10;

	return (p);

}	/* _getbase() */

/*-----------------------------------------------------------------
 * Name:	_atob()
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	5/31/01
 *
 *-----------------------------------------------------------------
 */
int32
_atob (
#if __mips >= 3
	uint64	* vp,
#else
	uint32	* vp,
#endif
char8	* p,
int32	base
)
{
#if __mips >= 3
	uint64	value, v1, v2;
#else
	uint32	value, v1, v2;
#endif
	char8	* q, tmp[20];
	int32	digit;

	if (p[0] == '0' && (p[1] == 'x' || p[1] == 'X')) {
		base = 16;
		p += 2;
	}

	if ((base == 16) && ((q = strchr (p, '.')) != 0)) {
		if (q - p > sizeof(tmp) - 1)
			return (0);
		strncpy (tmp, p, q - p);
		tmp[q - p] = '\0';
		if (!_atob (&v1, tmp, 16))
			return (0);
		q++;
		if (strchr (q, '.'))
			return (0);
		if (!_atob (&v2, q, 16))
			return (0);
		*vp = (v1 << 16) + v2;
		return (1);
	}

	value = *vp = 0;
	for (; *p; p++) {
		value *= base;
		if (*p >= '0' && *p <= '9')
			digit = *p - '0';
		else if (*p >= 'a' && *p <= 'f')
			digit = *p - 'a' + 10;
		else if (*p >= 'A' && *p <= 'F')
			digit = *p - 'A' + 10;
		else
			return (0);
		if (digit >= base)
			return (0);
		value += digit;
	}

	*vp = value;

	return (1);

}	/* _atob() */

/*-----------------------------------------------------------------
 * Name:	atob()
 *
 * Description: converts p to binary result in vp, rtn 1 on success
 *
 * Created by:	Rick Lund
 *
 * Date Created:	5/31/01
 *
 *-----------------------------------------------------------------
 */
int32
atob (
	uint32	* vp,
	char8	* p,
	int32	base
)
{
#if __mips >= 3
	uint64	v;
#else
	uint32	v;
#endif

	if (base == 0)
		p = _getbase (p, &base);

	if (_atob (&v, p, base)) {
		*vp = v;
		return (1);
	}
	return (0);

}	/* atob() */

#if __mips >= 3
/*-----------------------------------------------------------------
 * Name:	llatob()
 *
 * Description:	converts p to binary result in vp, rtn 1 on success
 *
 * Created by:	Rick Lund
 *
 * Date Created:	5/31/01
 *
 *-----------------------------------------------------------------
 */
int
llatob (
	uint64	* vp,
	char8	* p,
	int32	base
)
{
	if (base == 0)
		p = _getbase (p, &base);

	return _atob (vp, p, base);
}	/* llatob() */
#endif

/*-----------------------------------------------------------------
 * Name:	btoa()
 *
 * Description: converts value to ascii, result in dst
 *
 * Created by:	Rick Lund
 *
 * Date Created:	5/31/01
 *
 *-----------------------------------------------------------------
 */
char8 *
btoa (
	char8	* dst,
	uint32	value,
	int32	base
)
{
	char8	buf[34], digit = 0;
	int32	i, j, rem, neg;

	if (value == 0) {
		dst[0] = '0';
		dst[1] = 0;
		return (dst);
	}
	neg = 0;
	if (base == -10) {
		base = 10;
		if (value & (1L << 31)) {
			value = (~value) + 1;
			neg = 1;
		}
	}
	for (i = 0; value != 0; i++) {
		rem = value % base;
		value /= base;
		if (rem >= 0 && rem <= 9)
			digit = rem + '0';
		else if (rem >= 10 && rem <= 36)
			digit = (rem - 10) + 'a';
		buf[i] = digit;
	}
	buf[i] = 0;
	if (neg)
		strcat (buf, "-");

	/* reverse the string */
	for (i = 0, j = strlen (buf) - 1; j >= 0; i++, j--)
		dst[i] = buf[j];
	dst[i] = 0;

	return (dst);

}	/* btoa() */

#if __mips >= 3
/*-----------------------------------------------------------------
 * Name:	llbtoa()
 *
 * Description: converts value to ascii, result in dst
 *
 * Created by:	Rick Lund
 *
 * Date Created:	5/31/01
 *
 *-----------------------------------------------------------------
 */
char8 *
llbtoa (
	char8	* dst,
	uint64	value,
	int32	base
)
{
	char8	buf[66], digit = 0;
	int32	i, j, rem, neg;

	if (value == 0) {
		dst[0] = '0';
		dst[1] = 0;
		return (dst);
	}
	neg = 0;
	if (base == -10) {
		base = 10;
		if (value & (1LL << 63)) {
			value = (~value) + 1;
			neg = 1;
		}
	}
	for (i = 0; value != 0; i++) {
		rem = value % base;
		value /= base;
		if (rem >= 0 && rem <= 9)
			digit = rem + '0';
		else if (rem >= 10 && rem <= 36)
			digit = (rem - 10) + 'a';
		buf[i] = digit;
	}
	buf[i] = 0;
	if (neg)
		strcat (buf, "-");

	/* reverse the string */
	for (i = 0, j = strlen (buf) - 1; j >= 0; i++, j--)
		dst[i] = buf[j];
	dst[i] = 0;

	return (dst);

}	/* llbtoa() */
#endif

/*-----------------------------------------------------------------
 * Name:	gethex()
 *
 * Description:	convert n hex digits from p to binary, result in vp, 
 *				rtn 1 on success
 *
 * Created by:	Rick Lund
 *
 * Date Created:	5/31/01
 *
 *-----------------------------------------------------------------
 */
int32
gethex (
	uint32	* vp,
	char8	* p,
	int32	n
)
{
	uint32	v;
	int32	digit;

	for (v = 0; n > 0; n--) {
		if (*p == 0)
			return (0);
		v <<= 4;
		if (*p >= '0' && *p <= '9')
			digit = *p - '0';
		else if (*p >= 'a' && *p <= 'f')
			digit = *p - 'a' + 10;
		else if (*p >= 'A' && *p <= 'F')
			digit = *p - 'A' + 10;
		else
			return (0);
		v |= digit;
		p++;
	}
	*vp = v;

	return (1);

}	/* gethex() */

