/*
 *-----------------------------------------------------------------
 *
 * Name:	ht_memcpy.S
 *
 * RCS:  $Id: ht_memcpy.S,v 1.4 2002/10/18 17:51:34 rick Exp $
 *
 * Copyright (C) 2002, Agile Storage, Inc.
 *
 * Description:	void ht_memcpy_ass(dst, src, clines)
 *
 *				Assumes that dst and src and aligned on 32 byte (cacheline)
 *				boundary.  Clines is number of cache lines that include
 *				total number of bytes.  That is, if you really wanted to
 *				copy 36 bytes, clines would be 2.
 *
 *				Assumes that the src address is a HT cached address.
 *
 *				Invalidates src address after each line is copied.
 *
 *				No return value.
 *
 * Created by:	Rick Lund
 *
 * Date Created:	9/9/02
 *
 *-----------------------------------------------------------------
 */

#include <nfx-incl.h>
#include <sys/mips.h>

#ifdef SIBYTE

	.set noreorder
	.set noat

	.text

	.globl	ht_memcpy_fast
	.ent	ht_memcpy_fast
ht_memcpy_fast:
	daddiu	sp,sp,-64
	sd		s0,56(sp)
	sd		s1,48(sp)
	sd		s2,40(sp)
	sd		s3,32(sp)
	sd		s4,24(sp)
	sd		s5,16(sp)
	sd		s6,8(sp)
	sd		s7,0(sp)

ht_memcpy_loop:
	beq		a2,zero,do_ht_done
	slti	t7,a2,4
	beqz	t7,do_ht_memcpy_4
	slti	t7,a2,2
	beqz	t7,do_ht_memcpy_2

do_ht_memcpy_1:
	ld	t0,0x00(a1)
	ld	t1,0x08(a1)
	ld	t2,0x10(a1)
	ld	t3,0x18(a1)

	sd	t0,0x00(a0)
	sd	t1,0x08(a0)
	sd	t2,0x10(a0)
	sd	t3,0x18(a0)

	cache	Hit_Invalidate_D,0x00(a1)

	b	do_ht_done
	nop

do_ht_memcpy_2:
	ld	t0,0x00(a1)
	ld	t1,0x08(a1)
	ld	t2,0x10(a1)
	ld	t3,0x18(a1)
	ld	t4,0x20(a1)
	ld	t5,0x28(a1)
	ld	t6,0x30(a1)
	ld	t7,0x38(a1)

	sd	t0,0x00(a0)
	sd	t1,0x08(a0)
	sd	t2,0x10(a0)
	sd	t3,0x18(a0)
	sd	t4,0x20(a0)
	sd	t5,0x28(a0)
	sd	t6,0x30(a0)
	sd	t7,0x38(a0)

	cache	Hit_Invalidate_D,0x00(a1)
	cache	Hit_Invalidate_D,0x20(a1)

	daddiu	a1,0x40
	addiu	a2,-2
	b		ht_memcpy_loop
	daddiu	a0,0x40

do_ht_memcpy_4:
	ld	t0,0x00(a1)
	ld	t1,0x08(a1)
	ld	t2,0x10(a1)
	ld	t3,0x18(a1)
	ld	t4,0x20(a1)
	ld	t5,0x28(a1)
	ld	t6,0x30(a1)
	ld	t7,0x38(a1)
	ld	s0,0x40(a1)
	ld	s1,0x48(a1)
	ld	s2,0x50(a1)
	ld	s3,0x58(a1)
	ld	s4,0x60(a1)
	ld	s5,0x68(a1)
	ld	s6,0x70(a1)
	ld	s7,0x78(a1)

	sd	t0,0x00(a0)
	sd	t1,0x08(a0)
	sd	t2,0x10(a0)
	sd	t3,0x18(a0)
	sd	t4,0x20(a0)
	sd	t5,0x28(a0)
	sd	t6,0x30(a0)
	sd	t7,0x38(a0)
	sd	s0,0x40(a0)
	sd	s1,0x48(a0)
	sd	s2,0x50(a0)
	sd	s3,0x58(a0)
	sd	s4,0x60(a0)
	sd	s5,0x68(a0)
	sd	s6,0x70(a0)
	sd	s7,0x78(a0)

	cache	Hit_Invalidate_D,0x00(a1)
	cache	Hit_Invalidate_D,0x20(a1)
	cache	Hit_Invalidate_D,0x40(a1)
	cache	Hit_Invalidate_D,0x60(a1)

	daddiu	a1,0x80
	addiu	a2,-4
	b		ht_memcpy_loop
	daddiu	a0,0x80
	
do_ht_done:
	ld		s0,56(sp)
	ld		s1,48(sp)
	ld		s2,40(sp)
	ld		s3,32(sp)
	ld		s4,24(sp)
	ld		s5,16(sp)
	ld		s6,8(sp)
	ld		s7,0(sp)
	j		ra
	daddiu	sp,sp,64

	.end	ht_memcpy_fast

/*-----------------------------------------------------------------
 * Name:	void local_memcpy_ass(dst, src, clines)
 *
 * Description:	Just like ht_memcpy_ass, but copies from local address to
 *				local address using unrolled loop.
 *
 *				Number of cache lines (not number of bytes) is a parameter.
 *				Assumes that dst can be overwritten with clines worth of data.
 *				Src and dest addresses do not have to be cacheline aligned.
 *
 *				No return value.
 *
 * Created by:	Rick Lund
 *
 * Date Created:	9/24/02
 *
 *-----------------------------------------------------------------
 */
	.globl	local_memcpy_ass
	.ent	local_memcpy_ass
local_memcpy_ass:
	daddiu	sp,sp,-64
	sd		s0,56(sp)
	sd		s1,48(sp)
	sd		s2,40(sp)
	sd		s3,32(sp)
	sd		s4,24(sp)
	sd		s5,16(sp)
	sd		s6,8(sp)
	sd		s7,0(sp)

local_memcpy_loop:
	beq		a2,zero,do_local_done
	slti	t7,a2,4
	beqz	t7,do_local_memcpy_4
	slti	t7,a2,2
	beqz	t7,do_local_memcpy_2

do_local_memcpy_1:
	ld	t0,0x00(a1)
	ld	t1,0x08(a1)
	ld	t2,0x10(a1)
	ld	t3,0x18(a1)

	sd	t0,0x00(a0)
	sd	t1,0x08(a0)
	sd	t2,0x10(a0)
	sd	t3,0x18(a0)

	b	do_local_done
	nop

do_local_memcpy_2:
	ld	t0,0x00(a1)
	ld	t1,0x08(a1)
	ld	t2,0x10(a1)
	ld	t3,0x18(a1)
	ld	t4,0x20(a1)
	ld	t5,0x28(a1)
	ld	t6,0x30(a1)
	ld	t7,0x38(a1)

	sd	t0,0x00(a0)
	sd	t1,0x08(a0)
	sd	t2,0x10(a0)
	sd	t3,0x18(a0)
	sd	t4,0x20(a0)
	sd	t5,0x28(a0)
	sd	t6,0x30(a0)
	sd	t7,0x38(a0)

	daddiu	a1,0x40
	addiu	a2,-2
	b		local_memcpy_loop
	daddiu	a0,0x40

do_local_memcpy_4:
	ld	t0,0x00(a1)
	ld	t1,0x08(a1)
	ld	t2,0x10(a1)
	ld	t3,0x18(a1)
	ld	t4,0x20(a1)
	ld	t5,0x28(a1)
	ld	t6,0x30(a1)
	ld	t7,0x38(a1)
	ld	s0,0x40(a1)
	ld	s1,0x48(a1)
	ld	s2,0x50(a1)
	ld	s3,0x58(a1)
	ld	s4,0x60(a1)
	ld	s5,0x68(a1)
	ld	s6,0x70(a1)
	ld	s7,0x78(a1)

	sd	t0,0x00(a0)
	sd	t1,0x08(a0)
	sd	t2,0x10(a0)
	sd	t3,0x18(a0)
	sd	t4,0x20(a0)
	sd	t5,0x28(a0)
	sd	t6,0x30(a0)
	sd	t7,0x38(a0)
	sd	s0,0x40(a0)
	sd	s1,0x48(a0)
	sd	s2,0x50(a0)
	sd	s3,0x58(a0)
	sd	s4,0x60(a0)
	sd	s5,0x68(a0)
	sd	s6,0x70(a0)
	sd	s7,0x78(a0)

	daddiu	a1,0x80
	addiu	a2,-4
	b		local_memcpy_loop
	daddiu	a0,0x80
	
do_local_done:
	ld		s0,56(sp)
	ld		s1,48(sp)
	ld		s2,40(sp)
	ld		s3,32(sp)
	ld		s4,24(sp)
	ld		s5,16(sp)
	ld		s6,8(sp)
	ld		s7,0(sp)
	j		ra
	daddiu	sp,sp,64

	.end	local_memcpy_ass


#endif	/* SIBYTE */

