/*
 *-----------------------------------------------------------------
 *
 * Name:	socket-drv.c
 *
 * RCS:  $Id: socket-drv.c,v 1.9 2001/12/06 02:55:17 rick Exp $
 *
 * Copyright (C) 2001, Agile Storage, Inc.
 *
 * Description:	For communication between simulated cards in the Linux Test
 *
 * Created by:	Frank Salzmann
 *
 * Date Created:	7/31/01
 *
 *-----------------------------------------------------------------
 */

#include <nfx-incl.h>

#ifdef LINUX_TEST
#ifndef SHMEM

#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#include <stdio.h>
#include <unistd.h>
#include <string.h>
// #include <stdlib.h>
#include <sys/ipc.h>
#include <sys/sem.h>
#include <inttypes.h>
#include <sys/types.h>
#include <sys/time.h>
#include <errno.h>

#include <exports/cm-api.h>
#include <exports/eee-api.h>
#include "../sm-eee/eee.h"
#include "../sm-eee/eee-platform.h"


int eee_backplane_sock = 0;
/*-----------------------------------------------------------------
 * Name:	eee_linuxSockDrvInit()
 *
 * Description:	Initialize the SSC socket backplane interface.
 *
 * Created by:	Frank Salzmann
 *
 * Date Created:	7/31/01
 *
 *-----------------------------------------------------------------
 */
int
eee_linuxSockDrvInit()
{

	char           *servName;
    struct servent *serv;
    struct protoent *prot;
    struct hostent *host;
    int sock;
	int             i, send_rc;
	char            tmpbuf[64];
	int32			tmplen = 64;
	struct sockaddr_in eee_sockAddr;

	// printf(" ** Linux Sock Drv Init  [ enter ] ** \n");

    // Is this a proper "service"?
    setservent(0);

#ifdef NFP_FP 
	servName = "fp_backplane";
#elif defined(NFP_TXRX)
	servName = "txrx_backplane";
#elif defined(SSC)
	servName = "ssc_backplane";
#endif

    serv = getservbyname (servName, "udp");
    endservent();
    if (!serv) {
        perror ("regAgileApp: Invalid application");
        return -1;
    }
/*
    Create the socket and bind it to the requested port.
 */
    setprotoent(0);
    prot = getprotobyname ("udp");
    endprotoent();
    if (!prot) {
        perror ("eee_linuxSockDrvInit: getprotobyname failed");
        return -1;
    }

    sock = socket (PF_INET, SOCK_DGRAM, 0 /* prot->p_proto */);
    if (sock < 0) {
        perror("eee_linuxSockDrvInit: socket error");
        return -1;
    }

    memset (&eee_sockAddr, 0, sizeof(eee_sockAddr));
    eee_sockAddr.sin_family = AF_INET;
    eee_sockAddr.sin_port   = (serv->s_port);


    host = gethostbyname("dhcp-addr");
    if (!host) {
        perror ("eee_linuxSockDrvInit: gethostbyname failed");
        return -1;
    }
    memcpy (&eee_sockAddr.sin_addr, host->h_addr, host->h_length);

	// printf("port: %x: %d,  ", eee_sockAddr.sin_port, eee_sockAddr.sin_port);
	// printf("addr: %x\n", eee_sockAddr.sin_addr.s_addr);

	if(bind(sock, (struct sockaddr *)&eee_sockAddr, sizeof(eee_sockAddr)) < 0)
	{
        perror ("eee_linuxSockDrvInit: bind error");
        close(sock);
        return -1;
    }

	eee_backplane_sock = sock;

	// printf(" ** Linux Sock Drv Init: Sock: %d  [ Leave ] ** \n", 
	//		   eee_backplane_sock);

	return 0;
} // eee_linuxSockDrvInit

/*-----------------------------------------------------------------
 * Name:	eee_linuxSockFwdRx()
 *
 * Description:	Polling function to receive packets from linux socke.  
 *
 * Created by:	Frank Salzmann
 *
 * Date Created:	7/31/01
 *
 *-----------------------------------------------------------------
 */
void
eee_linuxSockFwdRx( void *cb, uint32 userInfo )
{
	int				   rc;
	fd_set             fds;
	int				   msgLen;
	char               rxBuf[2048];
	char			   *d_buf;
	uint32			   slot, cpu;
	socklen_t          fromLen;
	eee_desc_t	   	   *edesc;
	eee_descBuf_t	   *bd;
	struct timeval     to;
	struct sockaddr_in from;

	slot = nfxMySlotId;
	cpu = nfxMyCPUId;

	fromLen = sizeof( from );
	memset(&from, 0, fromLen);

    // Wait for a message. Use select() if they specify a timeout.
    // Otherwise, we just use recvfrom() and block until a message arrives.
	to.tv_sec  = 0;
	to.tv_usec = 100;

	FD_ZERO (&fds);
	FD_SET (eee_backplane_sock, &fds);
	if (select (eee_backplane_sock+1, &fds, NULL, NULL, &to) <= 0) {
		return;
	}

	msgLen = recvfrom( eee_backplane_sock, 
					   rxBuf, 
					   2048,
					   0,
					   &from, 
					   &fromLen);
	// No data received.
	if( msgLen == -1 )
		return;

	edesc = (eee_desc_t *)eee_allocateDesc(EEE_SHARED_POOL, NFX_COS_HIGH);
	if( !edesc ) {
		printf("eee_linuxSockFwdRx: Error: Cannot alloc edesc.\n");
		return;
	}
	memset(edesc, 0, sizeof(eee_desc_t));

	d_buf = eee_allocateBuffer( EEE_SHARED_POOL, EEE_BUF_LARGE, 0);
	if(!d_buf) {
		printf("eee_linuxSockFwdRx: Error: Cannot alloc buffer.\n");
		return;
	}
	memcpy(d_buf, &rxBuf[2], msgLen-2);

	// Extract port information from the Agile Hdr.
	edesc->hdr.dest_port = htonl(*(uint32 *)(&d_buf[0]));
	edesc->hdr.src_port  = htonl(*(uint32 *)(&d_buf[4]));

#if 0
	printf("msg len  : %d bytes, ", msgLen);
	printf("dest_port: %x, ", edesc->hdr.dest_port);
	printf("src_port : %x\n", edesc->hdr.src_port);
#endif 



	edesc->bd[1].buf = d_buf + 8;
	edesc->bd[1].len = msgLen - 10;
	edesc->bd[1].control = EDESC_BD_CMD_IN_USE | EDESC_BD_LARGE;
	edesc->hdr.control = (msgLen-10) & EDESC_TOTAL_LEN | 
		                 ((1 << EDESC_BUFCNT_SHIFT) & EDESC_BUFCNT_MASK);

/*
	rc = eee_forwardPacket(edesc);
*/
	eee_receivePacket(edesc);

#if 0 // No longer need to do this since it is now taken care of in eee_sendToApp
	if( rc == EEE_FORWARDED_PACKET )
		eee_freePacket(edesc);
#endif 

	return;
}	// eee_linuxSockFwdRx

/*-----------------------------------------------------------------
 * Name:	eee_linuxSockFwdPktUni()
 *
 * Description:	transmit function for Linux simulation environment for
 *				packets that will ultimately be transmitted over backplane
 *				switch fabric.
 *
 * Created by:	Frank Salzmann
 *
 * Date Created:	7/31/01
 *
 *-----------------------------------------------------------------
 */
int32
eee_linuxSockFwdPktUni(uint32 dest_port, eee_desc_t *edesc)
{
	char            *msg, *msgPtr;
	uint32          pktLen;
	uint32			slot, cpu, num_bufs;
	int				send_rc, i;
	int             port;
	eee_descBuf_t	* bd;
    struct servent *serv;
    struct protoent *prot;
    struct hostent *host;
	int32			rc = EEE_FORWARDED_PACKET;
	
    int sock;
	struct sockaddr_in sin;

	// printf("eee_linuxSockFwdPktUni\n");
/*
    Is this a proper "service"?
 */
    setservent(0);
    serv = getservbyname ("spnim_backplane", "udp");
    endservent();
    if (!serv) {
        perror ("regAgileApp: Invalid application");
        return -1;
    }
/*
    Create the socket and bind it to the requested port.
 */
    setprotoent(0);
    prot = getprotobyname ("udp");
    endprotoent();
    if (!prot) {
        perror ("eee_linuxSockDrvInit: getprotobyname failed");
        return -1;
    }

    sock = socket (PF_INET, SOCK_DGRAM, prot->p_proto);
    if (sock < 0) {
        perror("eee_linuxSockFwdPktUni: socket error");
        return -1;
    }

    memset (&sin, 0, sizeof(sin));
    sin.sin_family = AF_INET;
    sin.sin_port = serv->s_port;

    host = gethostbyname("sp-nim200");
    if (!host) {
        perror ("eee_linuxSockFwdPktUni: gethostbyname failed");
        return -1;
    }
    memcpy (&sin.sin_addr, host->h_addr, host->h_length);

	// Now setup the message being sent to the SP_NIM.

	slot = EEE_GET_SLOT_NUM(dest_port);
	cpu = EEE_GET_DEST_CPU(dest_port);

	pktLen = edesc->hdr.control & EDESC_TOTAL_LEN;
	// Make room for agile header. Plus 2-bytes of padding.
	pktLen += (8 + 2);

	msg = (char *)malloc(pktLen);
	if( !msg ) {
		printf("Could not allocate msg space.\n");
		return -1;
	}
	*(int32 *)(msg+2) = htonl(dest_port);
	*(int32 *)(msg+6) = htonl(edesc->hdr.src_port);
	
	bd = &(edesc->bd[0]);
	if (!(edesc->hdr.control & EDESC_COMMAND_USE_FIRST_BUFFER))
		bd++;

	num_bufs = (edesc->hdr.control & EDESC_BUFCNT_MASK) >> EDESC_BUFCNT_SHIFT;
	msgPtr = msg+10;
	for(i=0; i < num_bufs; i++) {
		memcpy(msgPtr, bd->buf, bd->len);
		msgPtr += bd->len;
		bd++;
	}

	send_rc = sendto( sock, 
					  msg, 
					  pktLen, 
					  0, 
					  (struct sockaddr *)&sin, 
					  sizeof(struct sockaddr)); 

	if( send_rc != pktLen ) {
		printf("ErrNum: %x ", errno);
		perror("eee_linuxSockFwdPktUni: sendto error:");
	}

	close(sock);
	free(msg);
	eee_freePacket(edesc);

	return rc;
}	// eee_linuxSockFwdPktUni


/*-----------------------------------------------------------------
 * Name:	eee_linuxSockFwdPktMulti()
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	6/28/01
 *
 *-----------------------------------------------------------------
 */
int32
eee_linuxSockFwdPktMulti(
	uint32		dest_port,
	eee_desc_t	* edesc
)
{
	printf("Remote Multi, edesc = 0x%x\n", edesc);
	printf("NOT IMPLEMENTED!!! Packet Dropped!!!\n");
	eee_freePacket(edesc);
}	// eee_linuxSockFwdPktMulti

#endif	/* !SHMEM */
#endif	/* LINUX_TEST */

