/*
 *-----------------------------------------------------------------
 *
 * Name:        load-bobcat-diag-menu.c
 *
 * RCS:  $Id: $
 *
 * Copyright (C) 2005, OnStor Inc.
 *
 * Description: Bobcat platform related autodiag code.
 *
 * Created by:  Warren Gale
 *
 * Date Created: 5/23/05
 *
 *-----------------------------------------------------------------
 */
#if defined(PROM_R9000) || defined(PROM_SIBYTE_BC)

#include <nfx-incl.h>
#include <exports/env-api.h>
#include <exports/seep-api.h>
#if defined(PROM_SIBYTE_BC)
#include <sys/sb1250/sb1250_defs.h>
#endif
#include "load-bobcat.h"
#include "load-bobcat-diag.h"


extern load_cb_t *load_cb_p; 

extern load_bc_tbl_t  load_bc_tbl[];

extern uint32 load_bc_set_mod_state();
extern uint32 load_bc_get_ssc_state();
extern uint32 load_bc_get_mod_state();
extern uint32 load_bc_set_ssc_state();

extern int get_input();
extern void eee_poll();

/*-----------------------------------------------------------------
 * Code that run on Sibyte PROMs only (TXRX & FP)
 *-----------------------------------------------------------------
 */
#ifdef PROM_SIBYTE_BC
#if 0
void
ad_printMenu()
{
  if(isFP()) {
    printf("===================================================================\n");
    printf(" FP Tests\n");
    printf("   01  I2C read tests              02  Hyper Transport DRAM test \n");
    printf("   03  FP to SSC Shared Mem test   04  DRAM tests \n");
  }
  if(isTXRX()) {
    printf("===================================================================\n");
    printf(" TXRX Tests\n");
    printf("   05  I2C read tests              06  LUC tests\n");
    printf("   07  LSI tests                   08  GigE tests\n");
    printf("   09  Hyper Transport DRAM test   10  TXRX to SSC Shared Mem test\n");
    printf("   11  DRAM tests \n");
  }
  printf("\n\n");
  printf(" Type [q] to Quit -- [m] for Menu --  or enter test number. \n");
}
#endif

/*-----------------------------------------------------
 * WTG  Add diag Menu states and report status
 *-----------------------------------------------------
 */
int
ad_module_menu()
{
  int32	mod_id, status, test2run, tickcnt;
  // int shortMem = 1; // WTG debug 
  int quitDiags = 0;
  int shortMem = 0; //  default setting.  Do All (Long) Memory tests 
  int local = 0, cached = 0, cacheMiss = 0;

    load_cb_p = (load_cb_t *)(HW_OC_SRAM_BASE_FROM_TXRX + LOAD_BC_SRAM_OFFSET);

    mod_id = isTXRX() ? LOAD_BC_TXRX_ID : LOAD_BC_FP_ID;

    load_bc_set_mod_state(mod_id, LOAD_BC_STATE_DIAG_INIT);

    printf("  -- wait ssc diags Menu.\n");

    printf("\n %s is listening and waiting for SSC diag commands...",load_bc_tbl[mod_id].mod_name);

    for (;;) {
        // wait for the SSC to be ready 
        while (load_bc_get_ssc_state(mod_id) != LOAD_BC_STATE_DIAG_EXECUTE)
        {
            delay_us(100000);
            eee_poll(10);   // clears xmit and rec Ques :)  (GigE loop tests)
            if (char_avail())
            {
                if (getchar() == CNTRL('e'))
                {
                    printf(" Diag Menu stopped.\n");
                    load_bc_set_mod_state(mod_id, LOAD_BC_STATE_DIAG_STOPPED);
                    return(1); // 1 = error since we stopped it
                }
            }
        }

        printf("   -- Get Cmd message\n");
        // Get command from mod_state
        test2run = load_bc_get_mod_state(mod_id);

        // Run auto diags and set correct state
        load_bc_set_mod_state(mod_id, LOAD_BC_STATE_DIAG_RUNNING);
        printf("   -- set mod state running\n");
        // ran into the problem of where the FP /TXRX ran the test and posted status
        // before SSC was able to set the state back to MENU.. Would get out of sync..
        tickcnt = 0;
        while (load_bc_get_ssc_state(mod_id) != LOAD_BC_STATE_DIAG_MENU)
        {
            // we wait for SSC to get out of execute mode so mod state 
            // doesn't see scc still in execute mode.
            delay_us(50000);
            if( tickcnt++ > 40 ) {
                printf("\n  -- Time Out -- No ACK from SSC\n");
                break;   // time out......
            }
        }
        if( tickcnt > 40 ){
            continue;  // go get command. don't do the rest. goto top of for(;;) loop
        }
        // execute test
        switch ( test2run) {
            // FP Tests
            case LOAD_BC_DIAG_TEST_00: {
                status = ad_i2c_tst();
                break;
            }
            case LOAD_BC_DIAG_TEST_01: {
                status = ad_hypermem_tst();
                break;
            }
            case LOAD_BC_DIAG_TEST_02: {
                status = ad_shmem_tst();
                break;
            }
            case LOAD_BC_DIAG_TEST_03: {
                status = ad_dram_tst( shortMem, cacheMiss);
                break;
            }
            case LOAD_BC_DIAG_TEST_31: {
                cacheMiss = 1;
                status = ad_dram_tst( shortMem, cacheMiss);
                break;
            }
            // FP memtest 60 - 63 
            case LOAD_BC_DIAG_TEST_60: {
                local = 1;
                cached = 1;
                status = ad_dram_tst2( local, cached, 0);
                break;
            }
            case LOAD_BC_DIAG_TEST_61: {
                local = 1;
                cached = 0;
                status = ad_dram_tst2( local, cached, 0);
                break;
            }
            case LOAD_BC_DIAG_TEST_62: {
                local = 0;
                cached = 1;
                status = ad_dram_tst2( local, cached, 0);
                break;
            }
            case LOAD_BC_DIAG_TEST_63: {
                local = 0;
                cached = 0;
                status = ad_dram_tst2( local, cached, 0);
                break;
            }
            // TXRX tests
            case LOAD_BC_DIAG_TEST_04: {
                status = ad_i2c_tst();
                break;
            }
            case LOAD_BC_DIAG_TEST_05: {
                status = ad_luc_tst();
                break;
            }
            case LOAD_BC_DIAG_TEST_06: {
                status = ad_lsi_tst();
                break;
            }
            case LOAD_BC_DIAG_TEST_07: {
                status = ad_gige_tst();
                eee_poll(100);  // clears the TXC Que  :) (embedded in getchar() )
                break;
            }
            case LOAD_BC_DIAG_TEST_08: {
                status = ad_hypermem_tst();
                break;
            }
            case LOAD_BC_DIAG_TEST_09: {
                status = ad_shmem_tst();
                break;
            }
            case LOAD_BC_DIAG_TEST_10: {
                status = ad_dram_tst( shortMem, cacheMiss);
                break;
            }
            case LOAD_BC_DIAG_TEST_32: {
                cacheMiss = 1;
                status = ad_dram_tst( shortMem, cacheMiss);
                break;
            }
            // utility set up GigE ports
            case LOAD_BC_DIAG_TEST_50: {
                status = ad_gige_setup();
                break;
            }
            // Other Utilities 
            case LOAD_BC_DIAG_TEST_51: 
            case LOAD_BC_DIAG_TEST_53: {
                status = ad_ena_sbe_exc( 1); // enable Single Bit Error Exception
                break;
            }
            case LOAD_BC_DIAG_TEST_52: 
            case LOAD_BC_DIAG_TEST_54: {
                status = ad_ena_sbe_exc( 0); // disable Single Bit Error Exception
                break;
            }
            
            // TXRX memtest 64 - 67 
            case LOAD_BC_DIAG_TEST_64: {
                local = 1;
                cached = 1;
                status = ad_dram_tst2( local, cached, 0);
                break;
            }
            case LOAD_BC_DIAG_TEST_65: {
                local = 1;
                cached = 0;
                status = ad_dram_tst2( local, cached, 0);
                break;
            }
            case LOAD_BC_DIAG_TEST_66: {
                local = 0;
                cached = 1;
                status = ad_dram_tst2( local, cached, 0);
                break;
            }
            case LOAD_BC_DIAG_TEST_67: {
                local = 0;
                cached = 0;
                status = ad_dram_tst2( local, cached, 0);
                break;
            }
            case LOAD_BC_DIAG_TEST_98: {
                shortMem = 0; // default   do Long Memory Tests
                status = 0;
                break;
            }
            case LOAD_BC_DIAG_TEST_99: {
                shortMem = 1; // debug   do Short Memory Tests
                status = 0;
                break;
            }
            case LOAD_BC_DIAG_TEST_100: {
                quitDiags = 1; // Quit Command so we return to what ever invoked us
                status = 0;
                break;
            }
            default: {
                printf("\n Unknown Command message [%08X] \n",test2run);
                continue;  // go to top wait for next cmd
            }

        }
	
        if( status)   // Status xxxx.p001 =  [xxxx = test] [p] = port 001 = error
            // save the test number and send it back with msg 
            load_bc_set_mod_state(mod_id, ((status & 0xFFFFFFF0)|LOAD_BC_STATE_DIAG_FAIL));
        else
            load_bc_set_mod_state(mod_id, LOAD_BC_STATE_DIAG_PASS);

        printf("  - Diag test post status.\n");
		
        printf(" Test Done.\n");

        // This will get us out of waiting for next comand
        if( quitDiags ) {
            return( status);   // go back to PROM promt / Autoload 
        }

    }  // end of for(;;) 

    return(status);  // 0 = Pass
}
#endif

/*-----------------------------------------------------------------
 * Code that runs on R9000 PROM only (SSC) 
 *-----------------------------------------------------------------
 */
#ifdef PROM_R9000

// Time out values waiting for a test to finish (in seconds)
#define SHORT_T_OUT   5  
#define MED_T_OUT     30
#define LONG_T_OUT    (60*8)
#define FRICKEN_LONG_T_OUT    (60*45)

int stopOnError;
// =====================================================================
// WTG  Diag work here 
// =====================================================================
// for testing
// shortMem = 1 if diag env is set

void 
ad_printMenu( shortMem) 
{
    printf("===================================================================\n");
    printf(" FP Tests\n");
    printf("   01  I2C read tests              02  Hyper Transport DRAM test \n");
    printf("   03  FP to SSC Shared Mem test   04  DRAM tests \n");
    printf("   31  DRAM Cache Mis test(Long)\n");
    printf("   51  Enable SBE Exceptions       52  Disable SBE Exceptions\n");
    printf("   60  DRAM memtest local cached   61  DRAM memtest local uncached \n");
    printf("   62  DRAM memtest remote cached  63  DRAM memtest remote uncached \n");
    printf("===================================================================\n");
    printf(" TXRX Tests\n");
    printf("   05  I2C read tests              06  LUC tests\n");
    printf("   07  LSI tests                   08  GigE tests\n");
    printf("   09  Hyper Transport DRAM test   10  TXRX to SSC Shared Mem test\n");
    printf("   11  DRAM tests                  32  DRAM Cache Mis test(Long)\n");
    printf("   53  Enable SBE Exceptions       54  Disable SBE Exceptions\n");
    printf("   64  DRAM memtest local cached   65  DRAM memtest local uncached \n");
    printf("   66  DRAM memtest remote cached  67  DRAM memtest remote uncached \n");
    printf("===================================================================\n");
    printf(" SSC Pre Tests\n");
    printf("   12  PCI read tests(Pre)         13  Pwr Fan Temperature Status\n");
    printf("   14  I2C read testsi(Pre)        15  SERDES Reset Test\n");
    printf("   16  R9K Scratch Mem Test        17  Gt Scratch Mem Test\n");
    printf("   18  SSC to TXRX image Mem Test  19  SSC to FP image Mem Test\n");
    printf("   20  Gt Main Mem Test [cached]\n");
    printf("===================================================================\n");
    printf(" SSC Tests\n");
    printf("   21  PCI read tests(Post)        22  I2C read tests(Post)\n");
    printf("   23  R9K Mem Test [cached]       24  R9K Mem Test [kseg0]\n");
    printf("   25  R9K Mem Test [kseg1]        26  Gt Main Mem Test [uncached]\n");
    printf("   27  BMFPGA Test                 28  RTC Test\n");
    printf("   29  CF Test                     30  ETH Test\n");
    printf("   70  GT  memtest local cached    71  GT  memtest local uncached \n");
    printf("   72  R9K memtest local cached    73  R9K memtest local uncached \n");
    printf("===================================================================\n");
    printf(" Utilities \n");
    printf("   50  Setup GigE Ports [Fiber/Copper] auto-detected \n");
    printf("   55  Compact Flash Read files [wd0a and wd1a] \n");
    printf("\n\n");
    if (shortMem)
       printf("    --- -- Debug Mode -- ---  Short Mem Tests -- ---\n");
    printf("  Stop On Error is %s  - Type [s] to change\n",(stopOnError == 0) ? "Off" : "On");
    printf(" Type [q] to Quit -- [m] for Menu --  or enter test number. \n");
}

//-------------------------------------------------------------------------
// wait for completion and print Status 
//-------------------------------------------------------------------------
int
ad_wait4done( int mod_id, int i, uint32 timeOut) 
{ 
  int status = 0;
  uint32 tickcnt = 0;
    // Now check for diag status on modulel  SSC, TXRX, or FP
    if( i == 0) { // i has loop count .. I only want to print the first time in loop
        printf(" Checking for Test Done.\n");
        printf("Type ctrl-e to stop Waiting for Diags done on Module.\n");
    }

    // Wait for module to be in NOT RUNNING State
    while ((load_bc_get_mod_state(mod_id) == LOAD_BC_STATE_DIAG_RUNNING) ||
             (load_bc_get_mod_state(mod_id) == 0) ||   // The initial state (hasen't got that far yet)
             (load_bc_get_mod_state(mod_id) == LOAD_BC_STATE_DIAG_INIT))
    {
        delay_us(1000000);
        tickcnt++;
        if (char_avail())
        {
            if (getchar() == CNTRL('e'))
            {
                printf(" Diags stopped on %s...\n",load_bc_tbl[mod_id].mod_name);
                load_bc_set_mod_state(mod_id, LOAD_BC_STATE_DIAG_STOPPED);
            }
        }
        if( tickcnt > timeOut ) {
            printf("  -Diag test Timeout on %s...\n",load_bc_tbl[mod_id].mod_name);
            printf("    tickcnt %d timeOut %d \n",tickcnt,timeOut);
            load_bc_set_mod_state(mod_id, LOAD_BC_STATE_DIAG_TIMEOUT);
            break;
        }
    }

    // Print Status of module diag test results 
    printf(" Diags for %4s: ",load_bc_tbl[mod_id].mod_name);
    if( load_bc_get_mod_state(mod_id) == LOAD_BC_STATE_DIAG_PASS)
        printf("  PASS");
    // Only an Error gets it's test number passed back with it's state
    //   Test number is upper 1/2 of State ie xxxx.psss  [xxxx] = test number [p] = port [sss] = state
    else if( (load_bc_get_mod_state(mod_id) & 0x0000000F) == LOAD_BC_STATE_DIAG_FAIL)  {
        printf("  FAILED");
        // Call Error decode function 
        autoDiagErrorPrint( load_bc_get_mod_state(mod_id)); // Send the 32 bits of status
        status = 1;  // report back an error 
    }
    else if( load_bc_get_mod_state(mod_id) == LOAD_BC_STATE_DIAG_TIMEOUT)  {
        printf("  TIME-OUT");
        status = 1;  // report back an error 
    }
    else if( load_bc_get_mod_state(mod_id) == LOAD_BC_STATE_DIAG_STOPPED) {
        printf("  STOPPED");
        status = 1;  // report back an error 
    }
    else if( load_bc_get_mod_state(mod_id) == LOAD_BC_STATE_PROM_INIT) {
        printf("  AUTOLOAD INIT");
        status = 1;  // report back an error 
    }
    else if( load_bc_get_mod_state(mod_id) == LOAD_BC_STATE_DIAG_PROM_PROMPT) {
        printf("  PROM PROMPT");
        status = 1;  // report back an error 
    }
    else if( load_bc_get_mod_state(mod_id) == LOAD_BC_STATE_DIAG_PROM_CRASHED) {
        printf("  DEBUG PROMPT");
        status = 1;  // report back an error 
    }
    else {
        printf("  UNKNOWN");
	status = 1;  // report back an error
    }
    // If looping test give them a loop count  1 = no print so anything above will print cnt
    if(i > 1) {
        printf("  -LoopCnt = %d\n",(i)+1);
    }
    else {
        printf("\n");
    }
    return(status);
}

/***********************************************************
* Go send execute message and wait for status 
*  return AD_TIMEOUT  if no response
*         1 = status error 
*         0 = All Ok
***********************************************************/
int 
ad_execute_wait( int mod_id, int i, uint32 timeOut)
{
  int tickcnt;
  int rc = 0;

    // send execute but if I'm SSC I already have executed test.
    if(!(mod_id == LOAD_BC_SSC_ID)) {

        tickcnt = 0;
        load_bc_set_ssc_state(mod_id, LOAD_BC_STATE_DIAG_EXECUTE);
        // wait for mod to go running
        // when mod goes to running it got the execute message
        while (load_bc_get_mod_state( mod_id) != LOAD_BC_STATE_DIAG_RUNNING)
        {
            delay_us(50000);
            if( tickcnt++ > 40 ) {
                printf("\n  -- Time Out -- No response from %s \n",load_bc_tbl[mod_id].mod_name);
                // this is to allow me to stop and re-start a module without getting out of sync
                load_bc_set_ssc_state(mod_id, LOAD_BC_STATE_DIAG_MENU); // so mod waits 
                return(AD_TIMEOUT); // don't loop on no response
            }
        }
                
        load_bc_set_ssc_state(mod_id, LOAD_BC_STATE_DIAG_MENU); // so mod waits after running cmd
    }

    // wait for completion and  check status of test(s)
    rc = ad_wait4done( mod_id, i, timeOut);  // i has loop count for print messages in wait4done

    return(rc);
}
/*---------------------------------------------------------
 * WTG  Add auto diag states, run tests, and report status
 *---------------------------------------------------------
 */
int
ad_ssc_menu()
{
  int32 mod_id;
  int32 rc = 0;
  int32 testnum = 0;
  int32 loopcnt, i;
  uchar8  cmd[80];
  uint32 timeOut = 0; 

  int gtmem = 0, cached = 0, local = 0;

  // int shortMem = 1; // WTG debug only
  int shortMem = 0; //  default setting.  Do All (Long) Memory tests 
     stopOnError = 1;  // Set up default state for Error

    // Set up SSC state in all mod Id's, so TXRX, and FP start listening .
    for (mod_id = LOAD_BC_MIN_ID; mod_id < LOAD_BC_MAX_ID; mod_id++)
    {
        load_bc_set_ssc_state(mod_id, LOAD_BC_STATE_DIAG_INIT);
    }

    // Set up FC state since FC CPU is in reset .
    load_bc_set_mod_state(LOAD_BC_FC_ID, LOAD_BC_STATE_DIAG_PASS);
    load_bc_set_mod_state(LOAD_BC_SSC_ID, LOAD_BC_STATE_DIAG_RUNNING);

    // print menu 1 time uless user wants it again 
    ad_printMenu(shortMem);

    for(;;) {     // Stay in loop until user exits
//  if cmd is bad the swith's default comes back here
        timeOut = 2; 
        loopcnt = 1;
        printf(" diag menu cmd: ");
        get_input( cmd );
        if ((cmd[0] == 'q') || (cmd[0] == 'Q')) {
            printf("\n Send Exit Diag Menu Cmd Msg\n");
            // send msg to both uP's
            testnum = LOAD_BC_DIAG_TEST_100;  // Quit command
            mod_id = LOAD_BC_FP_ID;
            load_bc_set_mod_state(mod_id, testnum);
            rc = ad_execute_wait( mod_id, 1, timeOut); // 1 = loop count(don't care) / no prints
            delay_us(100000);
                
            mod_id = LOAD_BC_TXRX_ID;
            load_bc_set_mod_state(mod_id, testnum);
            rc = ad_execute_wait( mod_id, 1, timeOut); // 1 = loop count(don't care) / no prints
            printf("\n Exit Diag Menu.\n");
            // Need to set state so DIAG_INIT is not there.. serio looks for it
            for (mod_id = LOAD_BC_MIN_ID; mod_id < LOAD_BC_MAX_ID; mod_id++)
            {
                load_bc_set_ssc_state(mod_id, LOAD_BC_STATE_DIAG_STOPPED);
            }

            return (0);  // exit
        }

        if ((cmd[0] == 'm') || (cmd[0] == 'M') || (cmd[0] == 'h') || (cmd[0] == 'H')) {
            ad_printMenu(shortMem); // reprint menu
            continue;
        }

        if ((cmd[0] == 's') || (cmd[0] == 'S')) {
            printf(" STOP on Error = %s \n",(stopOnError == 0) ? "Off" : "On");
            printf("  - set STOP on Error to  [0] = Off   [1] = On \n");
            get_input(cmd);
            if( cmd[0] == '0')
                stopOnError = 0;
            if( cmd[0] == '1')
                stopOnError = 1;
            ad_printMenu(shortMem); // reprint menu
            continue;
        }
        // for debugging   do short memory tests
        if ( cmd[0] == 'd') { // toggle it
           if( shortMem )  {
               shortMem = 0;
               testnum = LOAD_BC_DIAG_TEST_98;
           }
           else  {
               shortMem = 1;
               testnum = LOAD_BC_DIAG_TEST_99;
           }
        }
 
        if (cmd[0] == '\0') {
           continue;   // nothing entered 
        }

        if( !(cmd[0] == 'd')) {
            testnum = atoi(cmd);  // returns a "0" if some other char

            printf("  loop count: ");
            get_input( cmd );
            if( cmd[0] != '\0')
               loopcnt = atoi(cmd);
        }
        printf(" -debug cmd = %d loop count %d \n",testnum,loopcnt);

        for( i=0; i < loopcnt; i++ ) {
 
            switch( testnum) {
                // FP Tests
                case 1: {
                    mod_id = LOAD_BC_FP_ID;
                    // set up message
                    load_bc_set_mod_state(mod_id, LOAD_BC_DIAG_TEST_00);
                    timeOut = SHORT_T_OUT;
                    break;
                }
                case 2: {
                    mod_id = LOAD_BC_FP_ID;
                    load_bc_set_mod_state(mod_id, LOAD_BC_DIAG_TEST_01);
                    timeOut = MED_T_OUT;
                    break;
                }
                case 3: {
                    mod_id = LOAD_BC_FP_ID;
                    load_bc_set_mod_state(mod_id, LOAD_BC_DIAG_TEST_02);
                    timeOut = MED_T_OUT;
                    break;
                }
                case 4: {
                    mod_id = LOAD_BC_FP_ID;
                    load_bc_set_mod_state(mod_id, LOAD_BC_DIAG_TEST_03);
                    timeOut = LONG_T_OUT;
                    break;
                }
                // TXRX Tests
                case 5: {
                    mod_id = LOAD_BC_TXRX_ID;
                    load_bc_set_mod_state(mod_id, LOAD_BC_DIAG_TEST_04);
                    timeOut = SHORT_T_OUT;
                    break;
                }
                case 6: {
                    mod_id = LOAD_BC_TXRX_ID;
                    load_bc_set_mod_state(mod_id, LOAD_BC_DIAG_TEST_05);
                    timeOut = MED_T_OUT;
                    break;
                }
                case 7: {
                    mod_id = LOAD_BC_TXRX_ID;
                    load_bc_set_mod_state(mod_id, LOAD_BC_DIAG_TEST_06);
                    timeOut = MED_T_OUT;
                    break;
                }
                case 8: {
                    mod_id = LOAD_BC_TXRX_ID;
                    load_bc_set_mod_state(mod_id, LOAD_BC_DIAG_TEST_07);
                    timeOut = MED_T_OUT;
                    break;
                }
                case 9: {
                    mod_id = LOAD_BC_TXRX_ID;
                    load_bc_set_mod_state(mod_id, LOAD_BC_DIAG_TEST_08);
                    timeOut = MED_T_OUT;
                    break;
                }
                case 10: {
                    mod_id = LOAD_BC_TXRX_ID;
                    load_bc_set_mod_state(mod_id, LOAD_BC_DIAG_TEST_09);
                    timeOut = MED_T_OUT;
                    break;
                }
                case 11: {
                    mod_id = LOAD_BC_TXRX_ID;
                    load_bc_set_mod_state(mod_id, LOAD_BC_DIAG_TEST_10);
                    timeOut = LONG_T_OUT;
                    break;
                }
                // SSC pre tests
                case 12: {
                    mod_id = LOAD_BC_SSC_ID;
                    load_bc_set_mod_state(mod_id, LOAD_BC_DIAG_TEST_11);
                    rc = ad_pci_tst();
                    break;
                }
                case 13: {
                    mod_id = LOAD_BC_SSC_ID;
                    load_bc_set_mod_state(mod_id, LOAD_BC_DIAG_TEST_12);
                    rc = ad_pwr_fan_tst();
                    break;
                }
                case 14: {
                    mod_id = LOAD_BC_SSC_ID;
                    load_bc_set_mod_state(mod_id, LOAD_BC_DIAG_TEST_13);
                    rc = ad_ssc_i2c_tst();
                    break;
                }
                case 15: {
                    mod_id = LOAD_BC_SSC_ID;
                    load_bc_set_mod_state(mod_id, LOAD_BC_DIAG_TEST_14);
                    rc = ad_serdes_tst();
                    break;
                }
                case 16: {
                    mod_id = LOAD_BC_SSC_ID;
                    load_bc_set_mod_state(mod_id, LOAD_BC_DIAG_TEST_15);
                    rc = ad_r9k_scratch_mem_tst();
                    break;
                }
                case 17: {
                    mod_id = LOAD_BC_SSC_ID;
                    load_bc_set_mod_state(mod_id, LOAD_BC_DIAG_TEST_16);
                    rc = ad_gt_scratch_mem_tst();
                    break;
                }
                case 18: {
                    mod_id = LOAD_BC_SSC_ID;
                    load_bc_set_mod_state(mod_id, LOAD_BC_DIAG_TEST_17);
                    rc = ad_ssc_txrx_image_mem_tst();
                    break;
                }
                case 19: {
                    mod_id = LOAD_BC_SSC_ID;
                    load_bc_set_mod_state(mod_id, LOAD_BC_DIAG_TEST_18);
                    rc = ad_ssc_fp_image_mem_tst();
                    break;
                }
                case 20: {
                    mod_id = LOAD_BC_SSC_ID;
                    load_bc_set_mod_state(mod_id, LOAD_BC_DIAG_TEST_19);
                    ad_gt_mem_tst( shortMem);
                    break;
                }
                // SSC Tests
                case 21: {
                    mod_id = LOAD_BC_SSC_ID;
                    load_bc_set_mod_state(mod_id, LOAD_BC_DIAG_TEST_20);
                    rc = ad_pci2_tst();
                    break;
                }
                case 22: {
                    mod_id = LOAD_BC_SSC_ID;
                    load_bc_set_mod_state(mod_id, LOAD_BC_DIAG_TEST_21);
                    rc = ad_ssc_i2c2_tst();
                    break;
                }
                case 23: {
                    mod_id = LOAD_BC_SSC_ID;
                    load_bc_set_mod_state(mod_id, LOAD_BC_DIAG_TEST_22);
                    rc = ad_r9k_cache_mem_tst( shortMem);
                    break;
                }
                case 24: {
                    mod_id = LOAD_BC_SSC_ID;
                    load_bc_set_mod_state(mod_id, LOAD_BC_DIAG_TEST_23);
                    rc = ad_r9k_kseg0_mem_tst( shortMem);
                    break;
                }
                case 25: {
                    mod_id = LOAD_BC_SSC_ID;
                    load_bc_set_mod_state(mod_id, LOAD_BC_DIAG_TEST_24);
                    rc = ad_r9k_kseg1_mem_tst( shortMem);
                    break;
                }
                case 26: {
                    mod_id = LOAD_BC_SSC_ID;
                    load_bc_set_mod_state(mod_id, LOAD_BC_DIAG_TEST_25);
                    rc = ad_gt_uncached_mem_tst( shortMem);
                    break;
                }
                case 27: {
                    mod_id = LOAD_BC_SSC_ID;
                    load_bc_set_mod_state(mod_id, LOAD_BC_DIAG_TEST_26);
                    rc = ad_bmfpga_tst();
                    break;
                }
                case 28: {
                    mod_id = LOAD_BC_SSC_ID;
                    load_bc_set_mod_state(mod_id, LOAD_BC_DIAG_TEST_27);
                    rc = ad_rtc_tst();
                    break;
                }
                case 29: {
                    mod_id = LOAD_BC_SSC_ID;
                    load_bc_set_mod_state(mod_id, LOAD_BC_DIAG_TEST_28);
                    rc = ad_cf_tst();
                    break;
                }
                case 30: {
                    mod_id = LOAD_BC_SSC_ID;
                    load_bc_set_mod_state(mod_id, LOAD_BC_DIAG_TEST_29);
                    rc = ad_eth_tst();
                    break;
                }
                case 70: {
                    mod_id = LOAD_BC_SSC_ID;
                    load_bc_set_mod_state(mod_id, LOAD_BC_DIAG_TEST_70);
                    gtmem = 1;
                    local = 1;
                    cached = 1;
                    rc = ad_dram_tst2( local, cached, gtmem);
                    break;
                }
                case 71: {
                    mod_id = LOAD_BC_SSC_ID;
                    load_bc_set_mod_state(mod_id, LOAD_BC_DIAG_TEST_71);
                    gtmem = 1;
                    local = 1;
                    cached = 0;
                    rc = ad_dram_tst2( local, cached, gtmem);
                    break;
                }
                case 72: {
                    mod_id = LOAD_BC_SSC_ID;
                    load_bc_set_mod_state(mod_id, LOAD_BC_DIAG_TEST_72);
                    gtmem = 0;
                    local = 1;
                    cached = 1;
                    rc = ad_dram_tst2( local, cached, gtmem);
                    break;
                }
                case 73: {
                    mod_id = LOAD_BC_SSC_ID;
                    load_bc_set_mod_state(mod_id, LOAD_BC_DIAG_TEST_73);
                    gtmem = 0;
                    local = 1;
                    cached = 0;
                    rc = ad_dram_tst2( local, cached, gtmem);
                    break;
                }

                // Utilities  go here 
                case 50: { // this is to set up GigE ports Fiber/Copper
                    mod_id = LOAD_BC_TXRX_ID;
                    load_bc_set_mod_state(mod_id, LOAD_BC_DIAG_TEST_50);
                    timeOut = SHORT_T_OUT;
                    break;
                }

                case 51: { // this is to enable SBE ECC exceptions
                    mod_id = LOAD_BC_FP_ID;
                    load_bc_set_mod_state(mod_id, LOAD_BC_DIAG_TEST_51);
                    timeOut = SHORT_T_OUT;
                    break;
                }

                case 52: { // this is to set disable SBE ECC exceptions
                    mod_id = LOAD_BC_FP_ID;
                    load_bc_set_mod_state(mod_id, LOAD_BC_DIAG_TEST_52);
                    timeOut = SHORT_T_OUT;
                    break;
                }

                case 53: { // this is to enable SBE ECC exceptions
                    mod_id = LOAD_BC_TXRX_ID;
                    load_bc_set_mod_state(mod_id, LOAD_BC_DIAG_TEST_53);
                    timeOut = SHORT_T_OUT;
                    break;
                }

                case 54: { // this is to disable SBE ECC exceptions
                    mod_id = LOAD_BC_TXRX_ID;
                    load_bc_set_mod_state(mod_id, LOAD_BC_DIAG_TEST_54);
                    timeOut = SHORT_T_OUT;
                    break;
                }

                case 55: { // this reads all runtime images (not started) from CF
                    mod_id = LOAD_BC_SSC_ID;
                    load_bc_set_mod_state(mod_id, LOAD_BC_DIAG_TEST_55);
                    rc = ad_read_cf_files();
                    break;
                }

                // special memeory tests --
                case 31: { //dram cache mis test (very long test)
                    mod_id = LOAD_BC_FP_ID;
                    load_bc_set_mod_state(mod_id, LOAD_BC_DIAG_TEST_31);
                    timeOut = FRICKEN_LONG_T_OUT;
                    break;
                }
                case 32: { //dram cache mis test (very long test)
                    mod_id = LOAD_BC_TXRX_ID;
                    load_bc_set_mod_state(mod_id, LOAD_BC_DIAG_TEST_32);
                    timeOut = FRICKEN_LONG_T_OUT;
                    break;
                }
                //  Random read write memory tests
                case 60: {
                    mod_id = LOAD_BC_FP_ID;
                    load_bc_set_mod_state(mod_id, LOAD_BC_DIAG_TEST_60);
                    timeOut = LONG_T_OUT;
                    break;
                }
                case 61: {
                    mod_id = LOAD_BC_FP_ID;
                    load_bc_set_mod_state(mod_id, LOAD_BC_DIAG_TEST_61);
                    timeOut = LONG_T_OUT;
                    break;
                }
                case 62: {
                    mod_id = LOAD_BC_FP_ID;
                    load_bc_set_mod_state(mod_id, LOAD_BC_DIAG_TEST_62);
                    timeOut = LONG_T_OUT;
                    break;
                }
                case 63: {
                    mod_id = LOAD_BC_FP_ID;
                    load_bc_set_mod_state(mod_id, LOAD_BC_DIAG_TEST_63);
                    timeOut = LONG_T_OUT;
                    break;
                }
                case 64: {
                    mod_id = LOAD_BC_TXRX_ID;
                    load_bc_set_mod_state(mod_id, LOAD_BC_DIAG_TEST_64);
                    timeOut = LONG_T_OUT;
                    break;
                }
                case 65: {
                    mod_id = LOAD_BC_TXRX_ID;
                    load_bc_set_mod_state(mod_id, LOAD_BC_DIAG_TEST_65);
                    timeOut = LONG_T_OUT;
                    break;
                }
                case 66: {
                    mod_id = LOAD_BC_TXRX_ID;
                    load_bc_set_mod_state(mod_id, LOAD_BC_DIAG_TEST_66);
                    timeOut = LONG_T_OUT;
                    break;
                }
                case 67: {
                    mod_id = LOAD_BC_TXRX_ID;
                    load_bc_set_mod_state(mod_id, LOAD_BC_DIAG_TEST_67);
                    timeOut = LONG_T_OUT;
                    break;
                }

                // special case for debug mode - shortMem tests --
                case LOAD_BC_DIAG_TEST_98: 
                case LOAD_BC_DIAG_TEST_99: 
                    // send msg to both uP's
                    mod_id = LOAD_BC_FP_ID;
                    i = loopcnt; // don't loop on This cmd
                    load_bc_set_mod_state(mod_id, testnum);
                    timeOut = SHORT_T_OUT;
                    rc = ad_execute_wait( mod_id, i, timeOut);
                    delay_us(100000);
                    // careful  I just let it fall through to execute the last msg
                    mod_id = LOAD_BC_TXRX_ID;
                    i = loopcnt; // don't loop on This cmd
                    load_bc_set_mod_state(mod_id, testnum);
                    timeOut = SHORT_T_OUT;
                    break;  // let the normal break fall through to do the 2nd execute

                default: {
                    printf(" Unknown Command [%d]\n",testnum);
                    i = loopcnt; // don't loop on bad cmd
                    memset(cmd, 0, 80); // clear cmd string
                    continue;  // Skips the rest and looks for input again
                }

            } //end of switch

            if(mod_id == LOAD_BC_SSC_ID) { // If I'm the SSC I must set my own status in mod state...
                if( rc)  // if ssc test errored 
                    load_bc_set_mod_state(LOAD_BC_SSC_ID, ((rc & 0xFFFFFFF0)|LOAD_BC_STATE_DIAG_FAIL));
                else
                    load_bc_set_mod_state(LOAD_BC_SSC_ID, LOAD_BC_STATE_DIAG_PASS);
            }

            // go send execute msg and wait for status
            rc = ad_execute_wait( mod_id, i, timeOut);

            if( rc == AD_TIMEOUT ){
                i = loopcnt; // no more loops
                continue;  // go get command. don't do the rest. goto top of for(;;) loop
            }

            if(rc ) { // got an error
                if( stopOnError) // stop on error ?
                    i = loopcnt; // no more loops
            } 

        } // end of for(loopcnt) 

    } // end of for(;;) loop

    return (0);
}


#endif

//WTG  add auto diag calls
int
ad_menu()
{
	watchdog_disable();
#ifdef PROM_SIBYTE_BC
	return(ad_module_menu());
#endif
#ifdef PROM_R9000
	return(ad_ssc_menu());
#endif
	watchdog_restart();
}

/***************************************************************
* Called from Prom menu. 
* Allow running of the Menu from SSC.
*  the TXRX / FP will wait for messages from SSC to start a test
****************************************************************/
int
do_adMenu( int32  argc, uchar8  * argv[] )
{

  int rc = 0;

    if( argc > 1) {
        return -1;
    }
   rc = ad_menu();
   return(rc);
}

#endif	// PROM_R9000 || PROM_SIBYTE_BC
