/*
 *-----------------------------------------------------------------
 *
 * Name:        load-bobcat-diag.h
 *
 * RCS:  $Id: $
 *
 * Copyright (C) 2005, OnStor Inc.
 *
 * Description: Definitions for Bobcat platform related auto diag code.
 *
 * Created by:  Warren Gale
 *
 * Date Created: 4/20/05
 *
 *-----------------------------------------------------------------
 */
#ifndef _LOAD_BC_DIAG_H
#define _LOAD_BC_DIAG_H

#define AUTO_DIAG_TEST_ONE 0x00010000
#define MEG (1024*1024)
#define BC_PCB_MODEL_LEN (12+1)

//   Diag Test status and error return defs
/*******************************************************************
* 0x XXXX XXXX
*    |||| |  |
*    |||| |  Error Bit 0 = 1
*    |||| This is Port Num Error 1 = port 0
*    ||||                        2 = port 1
*    ||||                        4 = port 2
*    ||||                        8 = port 3
*    ||xx = Test Number returned
*    gg = Test Group Number returned
*
********************************************************************/
#define AUTO_DIAG_TEST_ERROR	0x00000001

#define DIAG_PORT_0_ERROR       0x00001000	// Bits that are ored in with Diag return status
#define DIAG_PORT_1_ERROR       0x00002000 
#define DIAG_PORT_2_ERROR       0x00004000
#define DIAG_PORT_3_ERROR       0x00008000


/*************************************************************************
*  More Diag Stuff 
*     Test Group Number defines 
*       Test Number defines
*         See above for port defines
*   
*************************************************************************/
#define DIAG_TEST_GROUP_INFO	0x01000000
#define DIAG_TEST_GROUP_PWRFAN	0x02000000
#define DIAG_TEST_GROUP_PCI	0x03000000
#define DIAG_TEST_GROUP_I2C	0x04000000
#define DIAG_TEST_GROUP_DEV	0x05000000
#define DIAG_TEST_GROUP_MEM	0x06000000

// Group Info test Numbers 
#define DIAG_TEST_UNKNOWN_MOD 	0x00010000
// Group Power/Fan test Numbers 
#define DIAG_TEST_PWR0		0x00010000
#define DIAG_TEST_PWR1		0x00020000
#define DIAG_TEST_FAN0		0x00030000
#define DIAG_TEST_FAN1		0x00040000
#define DIAG_TEST_FAN_SPEED	0x00050000
// PCI test Numbers 
#define DIAG_TEST_GT_PCI	0x00010000
#define DIAG_TEST_TXRX_PCI	0x00020000
#define DIAG_TEST_FP_PCI	0x00030000
#define DIAG_TEST_QLOGIC_PCI0	0x00040000
#define DIAG_TEST_QLOGIC_PCI1	0x00050000
#define DIAG_TEST_BRIDGE_PCI	0x00060000
#define DIAG_TEST_MAC0_PCI	0x00070000
#define DIAG_TEST_MAC1_PCI	0x00080000
#define DIAG_TEST_CF_PCI	0x00090000
// I2C test Numbers 
#define DIAG_TEST_BOARD_SEEP	0x00010000
#define DIAG_TEST_R9K_DRAM	0x00020000
#define DIAG_TEST_GT_DRAM	0x00030000
#define DIAG_TEST_DRAM0_SEEP	0x00040000
#define DIAG_TEST_DRAM1_SEEP	0x00050000
#define DIAG_TEST_DRAM1_EXTRA	0x00060000
#define DIAG_TEST_DRAM1_MISS	0x00070000
#define DIAG_TEST_DRAM2_SEEP	0x00080000
#define DIAG_TEST_DRAM2_EXTRA	0x00090000
#define DIAG_TEST_DRAM2_MISS	0x000a0000
#define DIAG_TEST_DRAM3_SEEP	0x000b0000
#define DIAG_TEST_DRAM3_EXTRA	0x000c0000
#define DIAG_TEST_DRAM3_MISS	0x000d0000
#define DIAG_TEST_MUX_READ	0x000e0000
#define DIAG_TEST_FAN0_READ	0x000f0000
#define DIAG_TEST_FAN1_READ	0x00100000
#define DIAG_TEST_TEMPSEN_READ	0x00110000
#define DIAG_TEST_MODEL_JUMPER	0x00120000
// Device test Numbers 
#define DIAG_TEST_BMFPGA_REV	0x00010000
#define DIAG_TEST_BMFPGA_REG	0x00020000
#define DIAG_TEST_BMFPGA_STAT	0x00030000
#define DIAG_TEST_CF_INIT	0x00040000
#define DIAG_TEST_CF_DEV_OK	0x00050000
#define DIAG_TEST_ETH_LINK	0x00060000
#define DIAG_TEST_LUC_REV	0x00070000
#define DIAG_TEST_LUC_PHY_ID	0x00080000
#define DIAG_TEST_LSI_REG	0x00090000
#define DIAG_TEST_GIGE_LUCSETUP	0x000a0000
#define DIAG_TEST_GIGE_PKT	0x000b0000
#define DIAG_TEST_GIGE_PKT_ECNT	0x000c0000
#define DIAG_TEST_SERDES_LB	0x000d0000
#define DIAG_TEST_RTC_REG	0x000e0000
#define DIAG_TEST_GIGE_AD_SFP	0x000f0000

// Mem test Numbers 
#define DIAG_TEST_MEM_HT	0x00010000
#define DIAG_TEST_MEM_TXRX_SSC	0x00020000
#define DIAG_TEST_MEM_SSC_TXRX	0x00030000
#define DIAG_TEST_MEM_FP_SSC	0x00040000
#define DIAG_TEST_MEM_SSC_FP	0x00050000
#define DIAG_TEST_MEM_DRAM	0x00060000
#define DIAG_TEST_MEM_GT_MAIN	0x00070000
#define DIAG_TEST_MEM_R9K_MAIN	0x00080000
#define DIAG_TEST_MEM_R9K_SCRATCH     0x00090000
#define DIAG_TEST_MEM_GT_SCRATCH      0x000a0000
#define DIAG_TEST_MEM_DRAM_SIZE	      0x000b0000
#define DIAG_TEST_MEM_FP_DRAM0        0x000c0000
#define DIAG_TEST_MEM_FP_DRAM1        0x000d0000
#define DIAG_TEST_MEM_FP_DRAM2        0x000e0000
#define DIAG_TEST_MEM_FP_DRAM3        0x000f0000
#define DIAG_TEST_MEM_TXRX_DRAM0      0x00100000
#define DIAG_TEST_MEM_TXRX_DRAM1      0x00110000
#define DIAG_TEST_MEM_TXRX_DRAM2      0x00120000
#define DIAG_TEST_MEM_TXRX_DRAM3      0x00130000
#define DIAG_TEST_MEM_L2_ECC_ERROR    0x00140000
#define DIAG_TEST_MEM_ECC_ERROR       0x00150000
#define DIAG_TEST_MEM_DRAM_ERROR      0x00160000

// Auto detect SFP's
// dont use 1 cuz it's the error return
#define AD_FIBER	4
#define AD_COPPER	5
#define AD_TIMEOUT	6

// Menu Test numbers
#define LOAD_BC_DIAG_TEST_00  0x00000100
#define LOAD_BC_DIAG_TEST_01  0x00000200
#define LOAD_BC_DIAG_TEST_02  0x00000300
#define LOAD_BC_DIAG_TEST_03  0x00000400
#define LOAD_BC_DIAG_TEST_04  0x00000500
#define LOAD_BC_DIAG_TEST_05  0x00000600
#define LOAD_BC_DIAG_TEST_06  0x00000700
#define LOAD_BC_DIAG_TEST_07  0x00000800
#define LOAD_BC_DIAG_TEST_08  0x00000900
#define LOAD_BC_DIAG_TEST_09  0x00000a00
// SSC Menu ///
#define LOAD_BC_DIAG_TEST_10  0x00000c00
#define LOAD_BC_DIAG_TEST_11  0x00000d00
#define LOAD_BC_DIAG_TEST_12  0x00000e00
#define LOAD_BC_DIAG_TEST_13  0x00000f00
#define LOAD_BC_DIAG_TEST_14  0x00001000
#define LOAD_BC_DIAG_TEST_15  0x00001100
#define LOAD_BC_DIAG_TEST_16  0x00001200
#define LOAD_BC_DIAG_TEST_17  0x00001300
#define LOAD_BC_DIAG_TEST_18  0x00001400
#define LOAD_BC_DIAG_TEST_19  0x00001500
#define LOAD_BC_DIAG_TEST_20  0x00001600
#define LOAD_BC_DIAG_TEST_21  0x00001700
#define LOAD_BC_DIAG_TEST_22  0x00001800
#define LOAD_BC_DIAG_TEST_23  0x00001900
#define LOAD_BC_DIAG_TEST_24  0x00001a00
#define LOAD_BC_DIAG_TEST_25  0x00001b00
#define LOAD_BC_DIAG_TEST_26  0x00001c00
#define LOAD_BC_DIAG_TEST_27  0x00001d00
#define LOAD_BC_DIAG_TEST_28  0x00001e00
#define LOAD_BC_DIAG_TEST_29  0x00001f00
// Utilities 
//  -- GigE Fiber/Copper set up
#define LOAD_BC_DIAG_TEST_50  0x00003400
// ena/dis able SBE ECC Exceptions
#define LOAD_BC_DIAG_TEST_51  0x00003000
#define LOAD_BC_DIAG_TEST_52  0x00003100
#define LOAD_BC_DIAG_TEST_53  0x00003200
#define LOAD_BC_DIAG_TEST_54  0x00003300
// CF read files 
#define LOAD_BC_DIAG_TEST_55  0x00003500

//New Dram Tests ECC and readwrite
#define LOAD_BC_DIAG_TEST_60  0x00004000
#define LOAD_BC_DIAG_TEST_61  0x00004100
#define LOAD_BC_DIAG_TEST_62  0x00004200
#define LOAD_BC_DIAG_TEST_63  0x00004300
#define LOAD_BC_DIAG_TEST_64  0x00004400
#define LOAD_BC_DIAG_TEST_65  0x00004500
#define LOAD_BC_DIAG_TEST_66  0x00004600
#define LOAD_BC_DIAG_TEST_67  0x00004700
#define LOAD_BC_DIAG_TEST_70  0x00004800
#define LOAD_BC_DIAG_TEST_71  0x00004900
#define LOAD_BC_DIAG_TEST_72  0x00004A00
#define LOAD_BC_DIAG_TEST_73  0x00004B00
// Dram Cache Mis Test
#define LOAD_BC_DIAG_TEST_31  0x00004C00
#define LOAD_BC_DIAG_TEST_32  0x00004D00

// Debug -- Short Memory tests
#define LOAD_BC_DIAG_TEST_98  0x00006300
#define LOAD_BC_DIAG_TEST_99  0x00006400
// Quit Cmd
#define LOAD_BC_DIAG_TEST_100 0x00006500

#ifdef PROM_SIBYTE_BC
// Memory test defines
#define DRAM_ECC_L2_ERR   2
#define DRAM_ECC_MEM_ERR  3
// this will hold the 1st error on the DRAM test for diags 
//  to determine which DIMM is bad.
extern volatile unsigned long long diagDramErrorAddr;
extern unsigned long dramEccErr;
#endif

extern int char_avail(); // ??
extern int domt();  // prom-pmon/memtst.c
extern int dram_memtst();  // prom-pmon/memtst.c
extern int diag_pcmcia_socket_check();  // sm-pmondisk/pcmcia.c
extern int is_cmFanSpeedLow(); // sm-chassis/cm-fan.c
extern int is_cmFanSpeedHigh(); // sm-chassis/cm-fan.c
//extern int diag_get_drams_from_model_info(); // sm-laoder/load-bobcat-diag-util.c
extern int diag_get_model_info(); // sm-laoder/load-bobcat-diag-util.c
extern int ad_fan_ctrl_tach_check(); // sm-laoder/load-bobcat-diag-util.c
extern int ad_detect_sfp_all(); // sm-laoder/load-bobcat-diag-util.c
extern int ad_gige_setup();  // sm-laoder/load-bobcat-diag-util.c
extern int ad_ena_sbe_exc(int ena);  // sm-laoder/load-bobcat-diag-util.c

extern unsigned int runSSC_preDiags( int shortMem); // sm-laoder/load-bobcat-diag.c
extern unsigned int runSSC_AutoDiags( int shortMem);  // sm-laoder/load-bobcat-diag.c
extern unsigned int runAutoDiags( int shortMem); // sm-laoder/load-bobcat-diag.c
extern unsigned int runSSC_preDiags( int shortMem); 
extern unsigned int runSSC_AutoDiags( int shortMem);


extern void autoDiagErrorPrint( unsigned int autoTestError); // load-bobcat-diag-err.c

// functions in file //sm-laoder/load-bobcat-diag-tests.c
extern int ad_i2c_tst();
extern int ad_luc_tst();
extern int ad_lsi_tst();
extern int ad_gige_tst();
extern int ad_hypermem_tst();
extern int ad_shmem_tst();
extern int ad_dram_tst( int shortMem, int cacheMiss);
extern int ad_dram_tst2( int local, int cached, int gtmem);

// SSC Pre Tests
extern int ad_pci_tst();
extern int ad_pwr_fan_tst();
extern int ad_ssc_i2c_tst();
extern int ad_serdes_tst();
extern int ad_r9k_scratch_mem_tst();
extern int ad_gt_scratch_mem_tst();
extern int ad_ssc_txrx_image_mem_tst();
extern int ad_ssc_fp_image_mem_tst();
extern int ad_gt_mem_tst(int shortMem);

// SSC Tests
extern int ad_pci2_tst();
extern int ad_ssc_i2c2_tst();
extern int ad_r9k_cache_mem_tst(int shortMem);
extern int ad_r9k_kseg0_mem_tst(int shortMem);
extern int ad_r9k_kseg1_mem_tst(int shortMem);
extern int ad_gt_uncached_mem_tst(int shortMem);
extern int ad_bmfpga_tst();
extern int ad_rtc_tst();
extern int ad_cf_tst();
extern int ad_eth_tst();
extern int ad_read_cf_files();


extern int gt_mii_readSMI();    // ./sm-gt64440-mii/gt64440mii.c

extern void diag_lsi_clearStats(int);  // ./sm-lsi8104/lsi8104.c
extern void luc_clear_stats(void);  // ./sm-bmc12500-eth/bmc12500-eth-ui.c

extern int auto_bmc12500Eth_diagTxLuc(); // ./sm-bmc12500-eth/bmc12500-eth-diag.c
extern int do_bmt_rw_local_cached(int local, int cached, int gtmem); // ./prom-pmon/memtst.c

extern int load_bc_load_mod_image(int32 mod_id, char *dev_str); //code/sm-loader/load-bobcat.c

#endif	// _LOAD_BC_DIAG_H
