/*
 *-----------------------------------------------------------------
 *
 * Name:
 *
 * RCS:  $Id: load-toshiba.c,v 1.5 2002/09/30 22:56:10 johnh Exp $
 *
 * Copyright (C) 2001, Agile Storage, Inc.
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	1/14/02
 *
 *-----------------------------------------------------------------
 */

#include <nfx-incl.h>
#include <exports/i2c-api.h>
#include <exports/mem-api.h>
#include "load.h"

#ifdef PROM_R7000

#define TOSHIBA_BB_VER	1
#define TOSHIBA_BB_SIG	0x19283746
#define TOSHIBA_JB_SIG	0x46372819

#define TOSHIBA_JUMP_BLOCK_OFFSET	sizeof(toshiba_boot_block_t)
#define TOSHIBA_ENTRY_OFFSET		0x8000

typedef struct tosh_boot {
	uint32	signature;
	uint32	ver;
	uint32	jump_block_offset;
	uint32	toshiba_memsize;
	uint32	toshiba_num_banks;
	uint32	toshiba_sdccr0;
	uint32	toshiba_sdccr1;
} toshiba_boot_block_t;

typedef struct tosh_jump {
	uint32	signature;
	uint32	ssc_memsize;
	uint32	ssc_rev;
	uint32	ssc_msg_base;
	uint32	ssc_msg_len;
	uint32	toshiba_entry_offset;
	uint32	toshiba_memsize;
} toshiba_jump_block_t;

uint32	toshiba_memorysize;
uint32	toshiba_num_banks;
uint32	toshiba_sdccr0;
uint32	toshiba_sdccr1;

static void load_get_toshiba_mem_config(void);

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:	Should fill in:
 *						 toshiba_memorysize
 *						 toshiba_num_banks
 *						 toshiba_sdccr0
 *						 toshiba_sdccr1
 *
 * Created by:	Rick Lund
 *
 * Date Created:	1/14/02
 *
 *-----------------------------------------------------------------
 */
static void
load_get_toshiba_mem_config(void)
{
	uchar8	rows, columns, num_banks, sdram_banks, bank_density;
	uint32	total_mem;
	uchar8	bus=0;

	i2c(I2C_READ, bus, 0xAE, 3, 1, &rows);
	i2c(I2C_READ, bus, 0xAE, 4, 1, &columns);
	i2c(I2C_READ, bus, 0xAE, 5, 1, &num_banks);
	i2c(I2C_READ, bus, 0xAE, 17, 1, &sdram_banks);
	i2c(I2C_READ, bus, 0xAE, 31, 1, &bank_density);

	total_mem = num_banks * sdram_banks * bank_density;
#if 0
	printf("TOSHIBA MEM CONFIG:\n");
	printf("     rows                = %d\n", rows);
	printf("     columns             = %d\n", columns);
	printf("     # banks             = %d\n", num_banks);
	printf("     SDRAM banks         = %d\n", sdram_banks);
	printf("     Module Bank Density = %d\n", bank_density);
	printf("     Total Mem           = 0x%x\n", total_mem);
#endif

	toshiba_num_banks = num_banks;

	if (total_mem == 64) {
		if (num_banks == 1) {
			toshiba_sdccr0 = 0x000303ec;
			toshiba_sdccr1 = 0;
		}
		else {
			toshiba_sdccr0 = 0x000301ea;
			toshiba_sdccr1 = 0x020301ea;
		}
	}
	else if (total_mem == 128) {
		toshiba_sdccr0 = 0x000303ec;
		toshiba_sdccr1 = 0x040303ec;
	}
	else
		printf("Unsupported Toshiba memory size, 0x%x\n", total_mem);

	toshiba_memorysize = total_mem * 0x100000;

#if 0
	printf("     SDCCR0              = 0x%08x\n", toshiba_sdccr0);
	printf("     SDCCR1              = 0x%08x\n", toshiba_sdccr1);
#endif
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:	Initialize boot block with parameters for Toshiba booting
 *
 * Created by:	Rick Lund
 *
 * Date Created:	1/14/02
 *
 *-----------------------------------------------------------------
 */
void
load_set_toshiba_boot_block(void)
{
	toshiba_boot_block_t	* bb = (toshiba_boot_block_t *)0xb0000000;
	toshiba_jump_block_t	* jb = (toshiba_jump_block_t *)
									(0xb0000000 + TOSHIBA_JUMP_BLOCK_OFFSET);

	bzero(jb, sizeof(toshiba_jump_block_t));

	load_get_toshiba_mem_config();

	bb->ver = TOSHIBA_BB_VER;

	bb->jump_block_offset = TOSHIBA_JUMP_BLOCK_OFFSET;
	bb->toshiba_memsize = toshiba_memorysize;
	bb->toshiba_num_banks = toshiba_num_banks;
	bb->toshiba_sdccr0 = toshiba_sdccr0;
	bb->toshiba_sdccr1 = toshiba_sdccr1;

	bb->signature = TOSHIBA_BB_SIG;

	delay_us(10000);
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	1/14/02
 *
 *-----------------------------------------------------------------
 */
void
load_set_toshiba_jump_block(void)
{
	toshiba_jump_block_t	* jb = (toshiba_jump_block_t *)
									(0xb0000000 + TOSHIBA_JUMP_BLOCK_OFFSET);

	jb->ssc_memsize = memorysize;
	if (isOldSSC())
		jb->ssc_rev = 0;
	else
		jb->ssc_rev = 1;
	jb->ssc_msg_base = 0;						/* TODO */
	jb->ssc_msg_len = 0;						/* TODO */
	jb->toshiba_entry_offset = TOSHIBA_ENTRY_OFFSET;
	jb->toshiba_memsize = toshiba_memorysize;

	jb->signature = TOSHIBA_JB_SIG;
}

#endif	/* PROM_R7000 */

