/*
 *-----------------------------------------------------------------
 *
 * Name:	load-util.c
 *
 * RCS:  $Id: load-util.c,v 1.5 2002/09/28 00:07:55 johnh Exp $
 *
 * Copyright (C) 2001, Agile Storage, Inc.
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	10/14/01
 *
 *-----------------------------------------------------------------
 */

#include <nfx-incl.h>
#include "loadfn.h"
#include "load.h"

uchar8	save_line[2000];
uchar8	load_line[512];
uint32	save_len;

uint32	load_tot_len;
int32	load_dltype;
uint32	load_recno;

/*-----------------------------------------------------------------
 * Name:	load_get_format()
 *
 * Description: returns format of file:
 *					-1 = undetermined
 *					0 = SREC
 *					1 = LSI
 *
 * Created by:	Rick Lund
 *
 * Date Created:	10/14/01
 *
 *-----------------------------------------------------------------
 */
int32
load_get_format(
	uchar8	* line
)
{
	if (load_dltype == -1) {
		if (line[0] == '/')
			load_dltype = 1;
		else if (line[0] == 'S')
			load_dltype = 0;
	}
	
	return load_dltype;
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	10/14/01
 *
 *-----------------------------------------------------------------
 */
void
load_add_line(
	uchar8	* buf,
	uint32	size
)
{
	if (save_len + size >= 2000)
		printf("save_len too big, %d+%d\n", save_len, size);

	memcpy(&(save_line[save_len]), buf, size);
	save_len += size;
}

/*-----------------------------------------------------------------
 * Name:	load_get_line()
 *
 * Description:	returns ascii line, terminated with '\0',
 *				or NULL, if a whole line is not available.
 *
 * Created by:	Rick Lund
 *
 * Date Created:	10/14/01
 *
 *-----------------------------------------------------------------
 */
uchar8 *
load_get_line(void)
{
	uint32	i, j;

	i = 0;
	
	for (i=0; i<save_len; i++) {
		if (save_line[i] == '\n') {
			memcpy(&(load_line[0]), &(save_line[0]), i);
			load_line[i] = '\0';
			for (j=0,i++;i<save_len;i++,j++)
				save_line[j] = save_line[i];
			save_len = j;
			return &(load_line[0]);
		}
	}
	return NULL;
}

/*-----------------------------------------------------------------
 * Name:	load_load_line()
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	10/14/01
 *
 *-----------------------------------------------------------------
 */
int32
load_load_line(
	uchar8	* lp,
	int32	format
)
{
	uint32	len;
	uint32	loaded_len;
	int32	flags;
	int32	s;

	flags = 0;

	load_recno++;
/*
	printf("load_load_line, lp %p, ", lp);
*/

	if (format == -1)
		format = load_get_format(lp);
	if (format == -1) {
		printf("Error loading line, invalid file format\n");
		return -1;
	}
	len = strlen(lp);
/*
	printf("len = %d", len);
*/
	if (format == 1) {
/*
		printf(", format = LSI\n");
*/
		s = dl_fastload(lp, len, &loaded_len, flags);
	}
	else if (format == 0) {
/*
		printf(", format = SREC\n");
*/
		s = dl_fastload(lp, len, &loaded_len, flags);
	}

	load_tot_len += loaded_len;
	if (s == DL_DONE) {
		printf("s == DL_DONE\n");
	}
	else if (s != DL_CONT) {
		printf ("line %d: %s\n", load_recno, dl_err (s));
		printf ("%s\n", lp);
		return -1;
	}

	return NFX_OK;
}

