/*
 *-----------------------------------------------------------------
 *
 * Name:	sload.c
 *
 * RCS:  $Id: sload.c,v 1.3 2002/03/11 18:56:15 rick Exp $
 *
 * Copyright (C) 2001, Agile Storage, Inc.
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	10/6/01
 *
 *-----------------------------------------------------------------
 */

#ifdef PROM

#include <nfx-incl.h>
#include <sys/mips.h>
#include <pmon/stdio.h>
#include <pmon/termio.h>
#include <pmon/string.h>
#include <pmon/signal.h>
#include "loadfn.h"
#include "load.h"

/* control character used for download */
#define ETX		CNTRL('c')
#define ACK		CNTRL('f')
#define NAK		CNTRL('u')
#define XON		CNTRL('q')
#define XOFF	CNTRL('s')

/* dlproto values */
#define NONE		0
#define XONXOFF		1
#define ETXACK		2

/* dlecho values */
#define OFF			0
#define ON			1
#define LFEED		2

/* flush_cache types */
#define ICACHE		0
#define DCACHE		1
#define IADDR		2

/*********************** GALILEO ***********************/
/*********************** START   ***********************/

/********************************************************************
* getcDirect - Gets a character from the user, if there is no character pending
*              it waits for it.
*            - The function access the serial port directly.
*
*
* INPUT:    N/A
* OUTPUT:   N/A
* RETURNS:  The character received from the user
*********************************************************************/

uchar8 getcDirect()
{
	uchar8	c, * cp;
	cp = (uchar8 *)0xbd000020;
	while(1) {
		if(*(cp+0x14) & 0x01) {
			c = *(uchar8 *)cp;
			return c;
		}
		cp++;	/* Useless code for avoiding compilers optimizations problems */
		cp--;
	}
}

/********************************************************************
* putcDirect - Sends a character to the console,if the console is busy it
*              waits until it IDLE.
*            - The function access the serial port directly.
*
*
* INPUTS:   char c - the charecter which send to the port.
* OUTPUT:   N/A
* RETURN:   N/A
*********************************************************************/

void putcDirect(uchar8 c)
{

	uchar8	temp;
	do {
		temp = *(uchar8 *)0xbd000008;
		temp = temp & 0x4;
		if (temp == 0)
			break;
	} while(1);

	*(uchar8 *)0xbd000020 = c;

	return;
}

uint32	csum = 0;

#define __ 255
static const uchar8 hextab[256] = {
	__, __, __, __, __, __, __, __, __, __, __, __, __, __, __, __,
	__, __, __, __, __, __, __, __, __, __, __, __, __, __, __, __,
	__, __, __, __, __, __, __, __, __, __, __, __, __, __, __, __,
	0,  1,  2,  3,  4,  5,  6,  7,  8,  9, __, __, __, __, __, __,
	__, 10, 11, 12, 13, 14, 15, __, __, __, __, __, __, __, __, __,
	__, __, __, __, __, __, __, __, __, __, __, __, __, __, __, __,
	__, 10, 11, 12, 13, 14, 15, __, __, __, __, __, __, __, __, __,
	__, __, __, __, __, __, __, __, __, __, __, __, __, __, __, __,
	__, __, __, __, __, __, __, __, __, __, __, __, __, __, __, __,
	__, __, __, __, __, __, __, __, __, __, __, __, __, __, __, __,
	__, __, __, __, __, __, __, __, __, __, __, __, __, __, __, __,
	__, __, __, __, __, __, __, __, __, __, __, __, __, __, __, __,
	__, __, __, __, __, __, __, __, __, __, __, __, __, __, __, __,
	__, __, __, __, __, __, __, __, __, __, __, __, __, __, __, __,
	__, __, __, __, __, __, __, __, __, __, __, __, __, __, __, __,
	__, __, __, __, __, __, __, __, __, __, __, __, __, __, __, __,
};
#undef __


uchar8 ascii_to_bin(uchar8 c)
{
	return hextab[c];
}

get_pair()
{
	uchar8 byte;

	byte  = ascii_to_bin(getcDirect()) << 4;
	byte |= ascii_to_bin(getcDirect());
	csum += byte;

	return(byte);
}

int32 galileo_dl()
{
	register	length, address,save_csum;
	int32		i, first, done, eof, reccount , type, client_pc,counter;
	int32		src,dbl_length;
	uchar8		* buffptr, databuff[258], tempo, c;
	uchar8		display_counter = 0, display_char = '.';
	register	chunks, leftovers, putter, bytes_per_chunk;

	printf("fast srecord download ...\n");

	bytes_per_chunk = 16 ;

	reccount = 1;

	for (first = 1, done = 0; !done; first = 0, reccount++) {
		while(getcDirect() != 'S') continue;
			csum = 0;
			type = getcDirect();
			length = get_pair();
			if (length < 0 || length >= 256) {
				printf("record %d: invalid length\n", reccount);
				return 0;
			}
			length--;
			switch (type) {
				case '0':
					while (length-- > 0)
						get_pair();
					break;
				case '3':
					address = 0;
					for (i = 0; i < 4; i++) {
						address <<= 8;
						address |= get_pair();
						length--;
					}
					if (address == -1) {
						eof = 1;
						continue;
					}
					buffptr = &databuff[0] ;
					dbl_length = length << 1;
					chunks = dbl_length / bytes_per_chunk ;
					leftovers = dbl_length % bytes_per_chunk ;
					putter = bytes_per_chunk >> 1;
					while (chunks--) {
						for(i = 0 ; i <  bytes_per_chunk ; i++)
							databuff[i] = getcDirect();
						src = i = 0 ;
						while (i++ < putter) {
							tempo = (ascii_to_bin(databuff[src++])<<4) |
												ascii_to_bin(databuff[src++]);
							csum += tempo ;
							*(char *)address++ = tempo;
						}
					}
					if (leftovers) {
						putter = leftovers/2 ;
						for(i = 0 ; i <  leftovers ; i++)
							databuff[i] = getcDirect();
						src = i = 0 ;
						while (i++ < putter) {
							tempo = (ascii_to_bin(databuff[src++]) <<4) |
												ascii_to_bin(databuff[src++]);
							csum += tempo ;
							*(char8 *)address++ = tempo;
						}
					}
					break;

				case '7':
					address = 0;
					for (i = 0; i < 4; i++) {
						address <<= 8;
						address |= get_pair();
						length--;
					}
					if (address == -1) {
						eof = 1;
						continue;
					}
					client_pc = address;
					if (length)
						printf("record %d: type 7 record bad\n",reccount);
					done = 1;
					break;

				default:
					printf("record %d: unknown record type, %c\n",
															reccount, type);
					break;
			}

		save_csum = (~csum) & 0xff;
		if ((csum = get_pair()) < 0) {
			eof = 1;
			continue;
		}
		if (csum != save_csum) {
			printf("record %d: checksum error, calculated 0x%x, received 0x%x\n",
													reccount, save_csum, csum);
		}
		else {
			if(display_counter % 50 == 0) {
				putcDirect(display_char);
				display_counter = 0;
			}
			display_counter++;
		}
	}

	--reccount;

	if (done)
		printf("\a\a\a\nDone: %d records, initial pc: %p\n",
														reccount, client_pc);
	dl_entry = client_pc;

	return 1;
}

/*********************** GALILEO ***********************/
/*********************** END     ***********************/

extern char8	* heaptop;

static jmp_buf	ldjmpb;		/* non-local goto jump buffer */

#ifdef INET
static void
ldintr (int32 dummy)
{
	sigsetmask (0);
	longjmp (ldjmpb, 1);
}
#endif


/*************************************************************
 *  load(ac,av)
 */
sload (ac, av)
	int32	ac;
	char8	*av[];
{
	FILE			*fp;
	struct termio	otbuf, tbuf;
	char8			*hostport, *cmdstr;
	int32			dlecho, dlproto, dltype, count, n;
	int32			len, err, i, j, s, tot, recno, flags, istty, isnet;
	char8			recbuf[DLREC], *ep, *baudrate;
	int32			errs[DL_MAX];
	uint32			offset;
#ifdef FLASH
	void			*flashbuf;
	int32			flashsize;
#endif
	extern int32	optind;
	extern char8	*optarg;
#ifdef INET
	sig_t			ointr;
#else
	jmp_buf			*ointr;
#endif
	int32			intr;
	int32			c;
#ifdef INET
	int32			ospl;
#endif
	int32			use_galileo = 0;
	
	baudrate	= 0;
	count		= 0;
	cmdstr		= 0;
	hostport	= 0;
	flags		= 0;
	istty		= 0;
	isnet		= 0;
	dlecho		= OFF;
	dlproto		= NONE;
	offset		= 0;
	
	optind		= 0;
	while ((c = getopt (ac, av, "gsbeatifnyvo:c:u:h:")) != EOF)
		switch (c) {
			case 'g':
				use_galileo = 1; break;
			case 's':
				flags |= SFLAG; break;
			case 'b':
				flags |= BFLAG; break;
			case 'e':
				flags |= EFLAG; break;
			case 'a':
				flags |= AFLAG; break;
			case 't':
				flags |= TFLAG; break;
			case 'i':
				flags |= IFLAG; break;
#ifdef FLASH
			case 'f':
				flags |= FFLAG; break;
			case 'w':
				flags |= WFLAG; break;
#endif
			case 'v':
				flags |= VFLAG; break;
			case 'o':
				if (!get_rsa (&dl_offset, optarg))
					return (-1);
				break;
			case 'c':
				cmdstr = optarg; break;
			case 'h':
				hostport = optarg; break;
			case 'u':
				baudrate = optarg; break;
			default:
				return (-1);
		}
	
	if (optind < ac && !hostport)
		hostport = av[optind++];
	if (optind < ac)
		return (-1);
	
#ifdef FLASH
	if (flags & FFLAG) {
		sbd_flashinfo (&flashbuf, &flashsize);
		if (flashsize == 0) {
			printf ("No flash on this target\n");
			return 0;
		}
		/* any loaded program will be trashed... */
		flags &= ~(SFLAG | BFLAG | EFLAG);
		flags |= NFLAG;		/* don't bother with symbols */
	
		/*
		 * Override any offset given on command line.
		 * Addresses should be 0 based, so load them into the flash buffer.
		 */
		offset = (unsigned long) flashbuf;
	}
#endif
	
	dl_initialise (offset, flags);
	
	if (!hostport)
		hostport = getenv ("hostport");
	fp = fopen (hostport, "r");
	if (!fp) {
		printf ("can't open %s\n", hostport);
		return (1);
	}
	dltype = -1;		/* dltype is set by 1st char of 1st record */
	
	if (ioctl (fileno (fp), TCGETA, &tbuf) >= 0) {
		istty = 1;
	
		otbuf = tbuf;
	
		dlproto = matchenv ("dlproto");
		dlecho = matchenv ("dlecho");
		if (!baudrate)
			baudrate = getenv ("hostbaud");
	
		if (baudrate) {
			const char8 *err;
			if (err = parsettymode (baudrate, &tbuf)) {
				printf ("load: %s: %s\n", err, baudrate);
				return (-1);
			}
		}
	
		tbuf.c_iflag &= ~IXOFF;		/* we do input flow control */
		tbuf.c_lflag &= ~ISIG;		/* signal handling disabled */
	
		if (dlecho != ON)
			tbuf.c_lflag &= ~ECHO;	/* no automatic echo */
	
		/* allow LF or ETX as line terminator */
		tbuf.c_cc[VEOL] = '\n';
		tbuf.c_cc[VEOL2] = ETX;
	
		printf ("Downloading from %s, ^C to abort\n", hostport);
		ioctl (fileno (fp), TCSETAF, &tbuf);
	
		if (cmdstr) {
			fputs (cmdstr, fp);
			putc ('\r', fp);
		}
	
#ifdef INET
		/* spot a network device because it doesn't support GETINTR */
		if (ioctl (fileno (fp), GETINTR, &ointr) < 0)
			isnet = 1;
#endif
	}
	else {
		printf ("Downloading from %s, ^C to abort\n", hostport);
	}
	
	err = intr = tot = recno = 0;
	bzero (errs, sizeof (errs));
	
#ifdef INET
	if (istty && !isnet)
		/* block network polling during serial download */
		ospl = splhigh ();
#endif
	
	if (setjmp (ldjmpb)) {
		intr = 1;
#ifdef INET
		if (istty && !isnet)
			/* re-enable network polling */
			splx (ospl);
#endif
		goto done;
	}
	
#ifdef INET
	ointr = signal (SIGINT, ldintr);
#else
	ioctl (STDIN, GETINTR, &ointr);
	ioctl (STDIN, SETINTR, ldjmpb);
#endif
	
	/*********************** GALILEO ***********************/
	/* Added by galileo to download faster through the serial - Gil */
	if(use_galileo)
	{
		galileo_dl();
		goto done;
	}
	/***************** Galileo - up to here ****************/
	/*********************** GALILEO ***********************/
	
	for (;;) {
#if RAL_XXX
		scandevs ();
#endif
	
		/* read one line */
		if (istty) {
#if RAL_XXX
			n = fread (recbuf, 1, DLREC - 1, fp);
#else
			char8	ch;
			n = 0;
			while (1) {
				ch = promio_get_char_no_echo();
	
				if ((ch == ETX) || (ch == '\n'))
					break;
				else
					recbuf[n++] = ch;
			}
#endif
			if (n < 0) {
				err++;
				errs[DL_IOERR]++;
				break;
			}
			if (n == 0)
				continue;
		} else {
			if (!fgets (recbuf, DLREC - 1, fp))
				break;
			n = strlen (recbuf);
		}
	
		if (dlproto == XONXOFF) {
			putc (XOFF, fp);
		}
#if RAL_XXX
		scandevs ();
#endif
	
#ifdef INET
		if (istty && !isnet) {
			/* now safe to poll network */
			splx (ospl);
			scandevs ();
		}
#endif
	
		++recno;
		if (!(recno & 0x1F))
			putchar('.');
	
		for (ep = recbuf + n; ep > recbuf; ep--) 
			if (!isspace (ep[-1]) && ep[-1] != ETX)
				break;
	
		if (ep > recbuf) {
			*ep = '\0';	/* terminate line */
	
			n = ep-recbuf;
	
			if (dltype == -1) {
				char8	c = recbuf[0];
				if (c == '/')
					dltype = 1;
				else if (c == 'S')
					dltype = 0;
				else {
					err++;
					errs[DL_BADCHAR]++;
					if (flags & VFLAG) {
						printf ("unknown record type '%c' (0x%x)\n",
						isprint(c) ? c : '?', c & 0xff);
					}
					break;
				}
			}
	
			if (dltype == 1)
				s = dl_fastload (recbuf, n, &len, flags);
			else if (dltype == 0)
				s = dl_s3load (recbuf, n, &len, flags);
	
			tot += len;
			if (s == DL_DONE) {			/* 1 */
				break;
			}
			if (s != DL_CONT) {			/* 0 */
				err++;
				errs[s]++;
				if (flags & VFLAG) {
					printf ("line %d: %s\n", recno, dl_err (s));
					printf ("%s\n", recbuf);
				}
				if (dlproto == ETXACK)
					break;
			}
		}
	
#ifdef INET
		if (istty && !isnet)
			/* block network polling again */
			ospl = splhigh ();
#endif
	
		if (dlproto == XONXOFF)
			putc (XON, fp);
		else if (dlproto == ETXACK)
			putc (ACK, fp);
		if (dlecho == LFEED)
			putc ('\n', fp);
	}
	
	done:
#ifdef INET
	signal (SIGINT, ointr);
#else
	ioctl (STDIN, SETINTR, ointr);
#endif
	
	if (dlproto == XONXOFF)
		putc (XON, fp);
	else if (dlproto == ETXACK)
		putc (err ? NAK : ACK, fp);
	if (dlecho == LFEED)
		putc ('\n', fp);
	if (istty)			/* restore tty state */
		ioctl (fileno (fp), TCSETAF, &otbuf);
	fclose (fp);
	
	if (err != 0 || intr) {
		printf ("\n%d error%s%s\n", err, (err == 1) ? "" : "s", 
												intr ? " (interrupted)" : "");
	for (i = 0; i < DL_MAX; i++)
		if (errs[i])
			printf ("   %3d %s\n", errs[i], dl_err(i));
		printf ("total = 0x%x bytes\n", tot);
		return (1);
	}
	
	printf ("\ntotal = 0x%x bytes\n", tot);
	
	if (!(flags & (FFLAG|YFLAG))) {
		printf ("Entry Address  = %p\n", dl_entry);
		flush_cache (ICACHE);
		flush_cache (DCACHE);
		Epc = dl_entry;
		if (!(flags & SFLAG))
			dl_setloadsyms ();
	}
	
#ifdef FLASH
	if (flags & FFLAG) {
		extern long dl_minaddr;
		extern long dl_maxaddr;
		/* default assume same endianness */
#if #endian(big)
		int bigend = 1;
#else
		int bigend = 0;
#endif
		if (flags & WFLAG)
			/* code endianness is the opposite current prom */
			bigend = !bigend;
		sbd_flashprogram ((void *)dl_minaddr,				/* address */
							dl_maxaddr - dl_minaddr,		/* size */
							dl_minaddr - (long)flashbuf,	/* offset */
							bigend);
	}
#endif
	
	return (0);
}

#endif	/* PROM */

