/* vi:ts=4
 *
 *-----------------------------------------------------------------
 *
 * Name:			lock-comp.h
 *
 * RCS:  $Id: $
 *
 * Copyright (C) 2002, ClariStor Inc.
 *
 * Description: Comparision definitions and function prototypes.
 *
 * Created by:		Xiaobing Zhang
 *
 * Date Created:	12/03/2002
 *
 *-----------------------------------------------------------------
 */
#ifndef _LOCK_COMP_H
#define _LOCK_COMP_H

/*
 * Search mask
 *
 * It is used to determine two byte range locks match or not.
 *
 * If none of the following bits of a search mask is set, the two locks
 * match if their ranges overlap.
 */
#define LOCK_SEARCH_SAME_RANGE      0x0001  /*Lock ranges are the same*/
#define LOCK_SEARCH_DIFF_RANGE      0x0002  /*Lock ranges are not same*/
#define LOCK_SEARCH_SAME_OWNER      0x0004  /*Lock owners are the same*/
#define LOCK_SEARCH_DIFF_OWNER      0x0008  /*Lock owners are different*/
#define LOCK_SEARCH_MANDATORY_LOCK  0x0010  /*Search for mandatory lock*/
#define LOCK_SEARCH_NONZERO_LEN     0x0020  /*Search for lock whose range length
											  in non-zero*/
#define LOCK_SEARCH_IN_SHARED_LIST  0x0040  /*Look at each lock in the linked
											  list of a node in a shared lock
											  tree*/
#define LOCK_SEARCH_SAME_FID        0x0080  /*Lock FIDs are the same*/
#define LOCK_SEARCH_SAME_KEY        0x0100  /*Lock keys are the same*/
#define LOCK_SEARCH_SAME_IPADDR     0x0200  /*Lock owners' IP addresses are
											  the same*/
#define LOCK_SEARCH_EXCL_BOTH_0LEN  0x0400  /*Search result will be 0 if
                                              two 0-length exclusive locks
                                              are compared*/

/*
 * Default comparison result if the lock ranges overlap
 * but the criteria is not satisfied.
 */
#define LOCK_SPECIAL_COMP_RESULT  1 /*Please DO NOT change the value*/

/* Definition of conflict rule array size */
#define LOCK_CONFLICT_RULE_ARRAY_S_PROTOCOL     2
#define LOCK_CONFLICT_RULE_ARRAY_S_ADDLOCKTYPE  2
#define LOCK_CONFLICT_RULE_ARRAY_S_TREETYPE     2
#define LOCK_CONFLICT_RULE_ARRAY_S_OWNER        2
#define LOCK_CONFLICT_RULE_ARRAY_S_BLOCK        2

/* The conflict rule array */
extern
uint8 lock_conflictRule[LOCK_CONFLICT_RULE_ARRAY_S_PROTOCOL]\
					   [LOCK_CONFLICT_RULE_ARRAY_S_ADDLOCKTYPE]\
					   [LOCK_CONFLICT_RULE_ARRAY_S_TREETYPE]\
					   [LOCK_CONFLICT_RULE_ARRAY_S_OWNER]\
					   [LOCK_CONFLICT_RULE_ARRAY_S_BLOCK];

/*--------------------
 * Function prototypes
 *--------------------
 */
int8
lock_lockInfoComp(vs_mem_index_t	inLockInfoIdx,
				  void				*inLockInfop,
				  vs_mem_index_t	treeLockInfoIdx,
				  uint8				inLockOwnerType,
			   	  uint8				treeLockOwnerType,
				  uint64			inLockOffset,
				  uint64			treeLockOffset,
				  uint64			inLockLen,
				  uint64			treeLockLen,
				  uint16			searchMask,
				  int8				specialCompResult);

int8
lock_twoLockInfoCompFunc(vs_mem_index_t	lockInfoIdx1,
						 void			*lockInfop1,
						 uint8			lockOwnerType1,
						 vs_mem_index_t	lockInfoIdx2,
						 uint16			searchMask,
						 int8			specialCompResult);

int8
lock_sharedTwoNodesCompFunc(
				  lock_shNode_t	*inShNodep,
				  lock_shNode_t *treeShNodep,
				  uint16        searchMask,
				  int8			specialCompResult);
		
int8
lock_sharedLockInfoAndNodeCompFunc(
				  void				*inLockInfop,
				  uint8				inLockOwnerType,
				  lock_shNode_t		*treeShNodep,
				  uint16			searchMask,
				  int8				specialCompResult,
				  lock_shLock_t		**foundShLockpp,
				  vs_mem_index_t	*prevShLockIdxp);

int8
lock_exclCompFunc(lock_exLock_t	*inExLockp,
				  void			*inLockInfop,
				  uint8			inLockOwnerType,
				  boolean       useNode,
				  lock_exLock_t *treeExLockp,
				  uint16        searchMask,
				  int8			specialCompResult);

int8
lock_sharedCompFunc(
				  lock_shNode_t	*inShNodep,
				  void			*inLockInfop,
				  uint8			inLockOwnerType,
				  boolean       useNode,
				  lock_shNode_t *treeShNodep,
				  uint16        searchMask,
				  int8			specialCompResult);

int8
lock_compFunc(void         *inNodep,
			  void         *inLockInfop,
			  uint8        inLockOwnerType,
			  boolean      useNode,
			  void         *treeNodep,
			  uint16       searchMask,
			  int8         specialCompResult,
			  uint8        treeType);

void
lock_checkUnlockedRange(lock_nlmLockInfo_t	*lockInfop,
						lock_nlmLockInfo_t	*unlockInfop,
						uint64				*lowerOffsetp,
						uint64				*lowerLenp,
						uint64				*upperOffsetp,
						uint64				*upperLenp);

void
lock_conflictRuleInit(void);

int32
lock_checkConflict(lock_fileLock_t	*fileLockp,
				   void				*lockInfop,
				   uint8			lockOwnerType,
				   uint8			lockType,
				   boolean			blockIfConflict,
				   vs_mem_index_t	*confLockIdxp,
				   int32			*exclResultp,
				   int32			*sharResultp);

void
lock_accessRuleInit(void);

int32
lock_checkAccess(lock_fileLock_t		*fileLockp,
				 lock_cifsLockInfo_t	*cifsLockInfop,
				 boolean				writeAccess);

#endif /*_LOCK_COMP_H*/

