/* vi:ts=4
 *-----------------------------------------------------------------
 *
 * Name:         lock-hash.h
 *
 * RCS:  $Id: $
 *
 * Copyright (C) 2002, ClariStor Inc.
 *
 * Description:  Byte range lock clients and owners hashing definitions
 *               and function prototypes.
 *
 * Created by:   Xiaobing Zhang
 *
 * Date Created: 01/14/2003
 *
 *-----------------------------------------------------------------
 */
#ifndef _LOCK_HASH_H
#define _LOCK_HASH_H

/*
 * Hashing key for CIFS owner
 */
typedef struct lock_htCifsOwnerKey {
	uint16		tid;			/*CIFS owner Tid*/
	uint16		fid;			/*CIFS owner Fid*/
	uint16		pid;			/*CIFS owner Pid*/
} lock_htCifsOwnerKey_t;

/*
 * Hashing key for NLM client
 */
typedef struct lock_htNlmClientKey {
	ip_index_t	clientIpAddr;	/*NLM client IP address*/
} lock_htNlmClientKey_t;

/*
 * Hashing key for NLM owner
 */
typedef struct lock_htNlmOwnerKey {
	ip_index_t	clientIpAddr;	/*NLM client IP address*/
	int32		pid;			/*NLM owner Pid*/
} lock_htNlmOwnerKey_t;


/*--------------------
 * Function prototypes
 *--------------------
 */
uint32
lock_cifsLockOwnerHashFunc(void	*keyp);

int32
lock_cifsLockOwnerHashCompare(vs_mem_index_t	htOwnerIdx,
							  vs_mem_index_t	searchOwnerIdx);

void
lock_cifsLockOwnerHashRelease(vs_mem_index_t	htOwnerIdx);

uint32
lock_nlmLockClientHashFunc(void	*keyp);

int32
lock_nlmLockClientHashCompare(vs_mem_index_t	htClientIdx,
							  vs_mem_index_t	searchClientIdx);

void
lock_nlmLockClientHashRelease(vs_mem_index_t	htClientIdx);

uint32
lock_nlmLockOwnerHashFunc(void	*keyp);

int32
lock_nlmLockOwnerHashCompare(vs_mem_index_t	htOwnerIdx,
							 vs_mem_index_t	searchOwnerIdx);

void
lock_nlmLockOwnerHashRelease(vs_mem_index_t	htOwnerIdx);

lock_cifsLockOwner_t *
lock_htLookupCifsLockOwner(uint16			tid,
						   uint16			fid,
						   uint16			pid,
						   vs_mem_index_t	searchOwnerIdx,
						   vs_mem_index_t	*foundOwnerIdxp);

int32
lock_htInsertCifsLockOwner(uint16			tid,
						   uint16			fid,
						   uint16			pid,
						   vs_mem_index_t	ownerIdx);

int32
lock_htDeleteCifsLockOwner(uint16			tid,
						   uint16			fid,
						   uint16			pid,
						   vs_mem_index_t	ownerIdx);

lock_nlmLockClient_t *
lock_htLookupNlmLockClient(ip_index_t		clientIpAddr,
						   vs_mem_index_t	searchClientIdx,
						   vs_mem_index_t	*foundClientIdxp);

int32
lock_htInsertNlmLockClient(ip_index_t		clientIpAddr,
						   vs_mem_index_t	clientIdx);

int32
lock_htDeleteNlmLockClient(ip_index_t		clientIpAddr,
						   vs_mem_index_t	clientIdx);

lock_nlmLockOwner_t *
lock_htLookupNlmLockOwner(ip_index_t		clientIpAddr,
						  int32				pid,
						  vs_mem_index_t	searchOwnerIdx,
						  vs_mem_index_t	*foundOwnerIdxp);

int32
lock_htInsertNlmLockOwner(ip_index_t		clientIpAddr,
						  int32				pid,
						  vs_mem_index_t	ownerIdx);

int32
lock_htDeleteNlmLockOwner(vs_mem_index_t	clientIdx,
						  uint16			pid,
						  vs_mem_index_t	ownerIdx);

#endif /* _LOCK_HASH_H */
