/* vi:ts=4
 *-----------------------------------------------------------------
 *
 * Name:         lock-nlmCluster.c
 *
 * RCS:  $Id: $
 *
 * Copyright (C) 2002, ClariStor Inc.
 *
 * Description:  Functions related to cluster NLM support.
 *
 * Created by:   Xiaobing Zhang
 *
 * Date Created: 01/09/2003
 *
 *-----------------------------------------------------------------
 */

#include "lock-incls.h"

/*-----------------------------------------------------------------
 * Name:         lock_nlmClusSaveLock
 *
 * Description:
 *
 *     If cluster NLM is enabled, call cluster NLM API to save the lock
 *     on the stable storage.
 *
 * Arguments:
 *
 *     lockInfoIdx - Index to a lock_nlmLockInfo_t structure.
 *                   This lock is to be saved on the stable storage.
 *
 *     lockInfop - Pointer to the lock_nlmLockInfo_t structure.
 *
 *     lockType - The type of the lock (exclusive or shared).
 *     
 *     lockState - The state of the lock (granted or pending).
 *
 * Return Value:
 *
 *     NFX_OK - If succeed.
 *
 *     NFX_ERR - If failed.
 *
 * Created by:   Xiaobing Zhang
 *
 * Date Created: 01/09/2003
 *
 *-----------------------------------------------------------------
 */
int32
lock_nlmClusSaveLock(vs_mem_index_t		lockInfoIdx,
					 lock_nlmLockInfo_t	*lockInfop,
					 uint8				lockType, /*LOCK_TYPE_EXCL or LOCK_TYPE_SHAR*/
					 uint8				lockState) /*LOCK_STATE_GRANTED or LOCK_STATE_BLOCKED*/
{
	int32	rc = NFX_OK;

	/*TODO:
	 * Check if cluster NLM is enabled or not;
	 * if not enabled,
	 * {
	 *     return (NFX_OK);
	 * }
	 *
	 * rc = nlm_clusRemoveLockReq();
	 *
	 */
	
	return (rc);
} /* lock_nlmClusSaveLock() */

/*-----------------------------------------------------------------
 * Name:         lock_nlmClusRemoveLock
 *
 * Description:
 *
 *     If cluster NLM is enabled, call cluster NLM API to remove the lock
 *     from the stable storage.
 *
 * Arguments:
 *
 *     lockInfoIdx - Index to the lock_nlmLockInfo_t structure.
 *                   This lock is to be removed from the stable storage.
 *
 *     lockInfop - Pointer to the lock_nlmLockInfo_t structure.
 *
 *     delete - This is output.
 *              If the value is TRUE, it means either the cluster NLM is
 *              disabled or the lock record is not on the stable storage,
 *              so this won't trigger the I/O operation and we can go ahead
 *              to delete the lock from memory. 
 *              If the value is FALSE, we need to wait till function
 *              lock_nlmSaveLockCmpInd() is called.
 *
 * Return Value:
 *
 *     NFX_OK - If succeed.
 *
 *     NFX_ERR - If failed.
 *
 * Created by:   Xiaobing Zhang
 *
 * Date Created: 01/09/2003
 *
 *-----------------------------------------------------------------
 */
int32
lock_nlmClusRemoveLock(vs_mem_index_t		lockInfoIdx,
					   lock_nlmLockInfo_t	*lockInfop,
					   boolean				*delete)
{
	int32	rc = NFX_OK;

	*delete = TRUE;

	/*TODO:
	 * Check if cluster NLM is enabled or not;
	 * if not enabled,
	 * {
	 *     *delete = TRUE;
	 *     return (NFX_OK);
	 * }
	 *
	 * // Check if the lock record is stored in the stable storage or not.
	 * if (lockInfop->idxInRecFile == 0)
	 * {
	 *     // The record is not in the stable storage.
	 *     *delete = TRUE;
	 *     return (NFX_OK);
	 * }
	 *
	 * // The record is in the stable storage.
	 * *delete = FALSE;
	 * rc = nlm_clusRemoveLockReq();
	 *
	 */
	
	return (rc);
} /* lock_nlmClusRemoveLock() */

/*-----------------------------------------------------------------
 * Name:         lock_nlmClusUpdateLock
 *
 * Description:
 *
 *     If cluster NLM is enabled, call cluster NLM API to update the lock
 *     info on the stable storage. The lock record is already stored on
 *     the stable storage, and its state or lock range has been changed.
 *
 * Arguments:
 *
 *     lockInfoIdx - Index to the lock_nlmLockInfo_t structure.
 *                   It contains new info of the lock.
 *
 *     lockInfop - Pointer to the lock_nlmLockInfo_t structure.
 *
 *     lockState - The state of the lock (granted or pending).
 *
 * Return Value:
 *
 *     NFX_OK - If succeed.
 *
 *     NFX_ERR - If failed.
 *
 * Created by:   Xiaobing Zhang
 *
 * Date Created: 01/09/2003
 *
 *-----------------------------------------------------------------
 */
int32
lock_nlmClusUpdateLock(vs_mem_index_t		lockInfoIdx,
					   lock_nlmLockInfo_t	*lockInfop,
					   uint8				lockState)
{
	int32	rc = NFX_OK;

	/*TODO:
	 * Check if cluster NLM is enabled or not;
	 * if not enabled,
	 * {
	 *     return (NFX_OK);
	 * }
	 *
	 * rc = nlm_clusUpdateLockReq();
	 *
	 */
	
	return (rc);
} /* lock_nlmClusUpdateLock() */

