/* vi:ts=4
 *-----------------------------------------------------------------
 *
 * Name: luc-if.c
 *
 * RCS:  $Id: luc-if.c,v 1.29 2002/10/29 01:18:29 swang Exp $
 *
 * Copyright (C) 2001, Agile Storage, Inc.
 *
 * Description: LUC code to interface with BSD if routines.
 *
 * Created by:	Shaun Hui
 *
 * Date Created: 06/01/2001
 *
 *-----------------------------------------------------------------
 */
#include <nfx-incl.h>

#include <exports/pkt-api.h>
#include <exports/timer-api.h>
#include <exports/luc-api.h>

#include "../sm-bsd-sys/flags.h"
#include "../sm-bsd-sys/cdefs.h"
#include "../sm-bsd-sys/types.h"
#include "../sm-bsd-sys/mbuf.h"
#include "../sm-bsd-sys/socket.h"
#include "../sm-bsd-sys/sockio.h"


#include "../sm-bsd-net/if.h"
#include "../sm-bsd-net/if_dl.h"
#include "../sm-bsd-net/if_types.h"
#include "../sm-bsd-net/if_arp.h"
#include "../sm-bsd-net/route.h"

#include "../sm-bsd-netinet/in.h"
#include "../sm-bsd-netinet/in_var.h"
#include "../sm-bsd-netinet/if_ether.h"
#include "exports/bsdnet-api.h"
#include "exports/ipm-api.h"
#include "exports/cm-api.h"
#if defined(SIBYTE)
#include "exports/bmc12500-eth-api.h"
#endif /* SIBYTE */
#include "../sm-bsd-api/bsd-vif.h"


typedef struct
{
	struct arpcom	arpcom;
	int32			link;
	int32			tmr_id;
	uint32			mode;
}	luc_if_t;


static char *ifName[] = {"fp0", "fp1", "fp2"};

char *
luc_ifGetName(int32 link)
{
	if ((link < 0) || (link >= lucMaxLinkNum))
		return NULL;

	return ifName[link];
}

luc_if_t	luc_if[LUC_MAX_NUM_LINK_CHE];

int luc_ifIoctl( struct ifnet *ifp, u_long cmd, caddr_t ia)
{
    short16 link;
    uint32  ipaddr;

    ipaddr = IA_SIN(ia)->sin_addr.s_addr;
    link   = ifp->if_unit;

    switch ( cmd )
    {
		/*-------------------------------
		 * Add/Set IP IF
		 *-------------------------------
		 */
        case    SIOCAIFADDR:	
        case    SIOCSIFADDR:
                luc_ipIfAddEntry( link, ipaddr);
                break;

		/*-------------------------------
		 * Delete IP IF
		 *-------------------------------
		 */
        case    SIOCDIFADDR:	
                luc_ipIfDelEntry( link, ipaddr );
                break;
    }


    return 0;
}


void luc_ifStart( struct ifnet *ifp )
{
}

void luc_ifInit( void *info )
{
}

void luc_ifWatchdog( struct ifnet *ifp )
{
	//printf("luc_ifWatchdog =%x\n", ifp);
}

int
bsd_XmtPkt( struct ifnet 	*ifp, 
			struct mbuf 	*mb, 
			struct sockaddr *dst,
			struct rtentry  *rt);
int
luc_ifOutput(struct ifnet 	*ifp, 
			struct mbuf 	*mb, 
			struct sockaddr *dst,
			struct rtentry  *rt)
{
	return 0;
}

void luc_ifAttach( luc_if_t	*luc_ifp)
{

	struct ifnet		*ifp;
	struct ifaddr		*ifa;
	struct sockaddr_dl	*sdl;
	byte				*pMac;

	ifp = &luc_ifp->arpcom.ac_if;
	ifp->if_softc   = luc_ifp;
	ifp->if_type    = IFT_ETHER;
	ifp->if_addrlen = 6;
	ifp->if_hdrlen  = 14;
	ifp->if_slot    = nfxMySlotId;
	ifp->if_cpu     = nfxMyCPUId;
	ifp->if_unit    = luc_ifp->link;
	ifp->if_name    = "fp";
	ifp->if_vif     = NULL; 
#if defined(SIBYTE)
	ifp->if_mtu     = bmc12500Eth_get_maxPktSize(luc_ifp->link);
#else
	ifp->if_mtu     =1500;
#endif /* SIBYTE */
	ifp->if_flags   = IFF_BROADCAST | IFF_MULTICAST | IFF_RUNNING | IFF_UP;
	ifp->if_ioctl   = luc_ifIoctl;
	ifp->if_output  = bsd_XmtPkt;
	ifp->if_start   = luc_ifStart;
	ifp->if_timer   = 2;
	ifp->if_watchdog= luc_ifWatchdog;
	ifp->if_init    = luc_ifInit;
	ifp->if_baudrate= 1000*1000000;
	ifp->if_snd.ifq_maxlen = 128;
	
	/*-------------------------------
	 * Copy mac address of link.
	 *-------------------------------
	 */
	pMac = luc_getMacAddr( luc_ifp->link );
	bcopy(pMac, (char *)luc_ifp->arpcom.ac_enaddr, 6);

	if_attach(ifp);

	ifa = ifnet_addrs[ifp->if_index - 1];
	if (ifa == 0) {
		printf("ether_ifattach: no lladdr!\n");
		return;
	}

	sdl = (struct sockaddr_dl *)ifa->ifa_addr;
	sdl->sdl_type = IFT_ETHER;
	sdl->sdl_alen = ifp->if_addrlen;
	bcopy(((struct arpcom *)ifp)->ac_enaddr, LLADDR(sdl), ifp->if_addrlen);
}

int32 luc_ifTimerHandler(uint32 tmr, 
						void 	*info1, 
						void 	*info2, 
						void 	*info3, 
						void	*info4)
{
	luc_if_t		*luc_ifp;
	struct ifnet	*ifp;

 	//printf("luc_ifTimerHandler %x\n", info1);

	luc_ifp = info1;
	ifp	= &luc_ifp->arpcom.ac_if;

	if ((ifp->if_timer != 0) && (--ifp->if_timer == 0))
	{
		if (ifp->if_watchdog)
			(*ifp->if_watchdog)(ifp);
	}

	eee_timerStart(luc_ifp->tmr_id, luc_ifTimerHandler, luc_ifp, 0, 0, 0);
	return 0;
}

void luc_ifTimerInit(luc_if_t *luc_ifp)
{
	luc_ifp->tmr_id = eee_timerCreate( 5000 /* 5sec */, EEE_TIMER_ONCE );
	//printf("luc_ifTimerInit=%x \n", luc_ifp->tmr_id);

	if (luc_ifp->tmr_id == TIMER_ERR_NO_TIMER_AVAILABLE)
	{
		extern void luc_panic(void);
		printf("Timer Create error!!\n");
		luc_panic( );
	}

	eee_timerStart(luc_ifp->tmr_id, luc_ifTimerHandler, luc_ifp, 0, 0, 0);
}

void luc_initIf( void )
{
	luc_if_t	*luc_ifp;
	int32		link;
	extern void ifinit __P((void *));
	extern void  loopattach __P((void *));

    for (link=0; link < lucMaxLinkNum; link++)
	{
		luc_ifp = &luc_if[link];
		luc_ifp->link = link;

        luc_ifAttach( luc_ifp ) ;
		luc_ifTimerInit( luc_ifp);

#if !defined(PROM)
		/* announce interface detected */

		ipm_NewIntf((void*)&(luc_ifp->arpcom), 0, 0, 1);

#endif /* PROM */
	}
	/* aatach loop back interface */
	loopattach( 0 );
	ifinit(NULL);
}

void luc_ifSlowPath( int32 link, pkt_desc_t *pDesc )
{
	int32 rc;

	switch (ntohs(pDesc->protocol))
	{ 
		case	ETHERTYPE_IP:
				rc =bsd_RcvIpPkt(pDesc, &luc_if[link].arpcom.ac_if, 0, 0);
				break;

		case	ETHERTYPE_ARP:
				rc =bsd_RcvArpPkt(pDesc, &luc_if[link].arpcom.ac_if, 0, 0);
				break;

		default:
				rc = PKT_FREE_PKT;
				break;
	}

	if (rc == PKT_FREE_PKT)
		PKT_FREE_PACKET(pDesc);
}

int32 luc_sendintf(int32 argc, char *argv[])
{
  luc_if_t	*luc_ifp;
  int			link;

  for (link=0; link < lucMaxLinkNum; link++)
	{
	  luc_ifp = &luc_if[link];
#if !defined(PROM)
	  ipm_NewIntf((void*)&(luc_ifp->arpcom), 0, 0, 1);
#endif
	}
  return 1;
}

byte *seepGetMyExtMacAddr(int32 link);

uchar8 luc_mac_addr[6];

byte *luc_getMacAddr( int32 link )
{
#ifdef PROM

	if (link == LUC_BP_LINK_ID)
		bp_getMyMacAddr(nfxMySlotId, nfxMyCPUId, 1, EEE_MGMT, luc_mac_addr);
	else {
		if (isFC())
			memcpy(luc_mac_addr, seepGetMyExtMacAddr(nfxMyCPUId), 6);
		else
			memcpy(luc_mac_addr, seepGetMyExtMacAddr(link), 6);
	}

	return luc_mac_addr;
#else
	return bsd_GetIntfMacVif((void*)(&(luc_if[link].arpcom)));
	
//	return (seepGetMyExtMacAddr(link));
#endif

}

/*
 *----------------------------------------------------------------
 * 
 * Name :  luc_getActLALink
 *
 * Description: return active link of link aggregation vif, else
 *				return input. 
 *				 				
 *
 * Created by: Su Wang
 *
 * Data Created: 10/28/02
 *
 *---------------------------------------------------------------
 */
int32
luc_getActLALink(link) 
{
	struct ifnet	*ifp = NULL;

#if defined(LINK_AGG)
	ifp = bsd_VifLAGetActLink((void*)(&(luc_if[link].arpcom)));
#endif /* LINK_AGG */

	if (!ifp)
		return link;
	else
		return ifp->if_unit;
}

/*
 *----------------------------------------------------------------
 * 
 * Name :  luc_getUpLALink
 *
 * Description: return up link of link aggregation vif, else
 *				return input
 *
 * Created by: Su Wang
 *
 * Data Created: 7/25/01
 *
 *---------------------------------------------------------------
 */
int32
luc_getUpLALink(link) 
{
	struct ifnet	*ifp = NULL;

#if defined(LINK_AGG)
	ifp = bsd_VifLAGetUpLink((void*)(&(luc_if[link].arpcom)));
#endif /* LINK_AGG */

	if (!ifp)
		return link;
	else
		return ifp->if_unit;
}

/*
 *----------------------------------------------------------------
 * 
 * Name :  luc_getIntfFromLink
 *
 * Description: convert link to struct ifnet*
 *
 * Created by: Su Wang
 *
 * Data Created: 7/25/01
 *
 *---------------------------------------------------------------
 */
void*
luc_getIntfFromLink(int32 link)
{
	return (void*)&(luc_if[link].arpcom);
	
}
