/* vi:ts=4
 *-----------------------------------------------------------------
 *
 * Name: luc-init.c
 *
 * RCS:  $Id: luc-init.c,v 1.25 2002/10/22 01:30:38 jong Exp $
 *
 * Copyright (C) 2001, Agile Storage, Inc.
 *
 * Description:
 *
 * Created by:	Shaun Hui
 *
 * Date Created: 05/01/2001
 *
 *-----------------------------------------------------------------
 */
#include <nfx-incl.h>

#include <exports/eee-api.h>
#include <exports/malloc-api.h>
#include <exports/cm-api.h>
#include <exports/pkt-api.h>
#include <exports/luc-api.h>
#ifndef PROM
#include <exports/sb1250-dm-api.h>
#endif

int32 luc_idx;

luc_cfgBlk_t	*pLucCfg;
/* Su, 04/14/02, define max link at run time */
int lucMaxLinkNum;

void  luc_rcvPkt( eee_desc_t   *eDesc);
void  luc_rcvDllPkt( eee_desc_t   *eDesc);

void luc_xmtCmp (eee_desc_t   *eDescp);


/*-----------------------------------------------------------------
 * Name: luc_initRing
 *
 * Description: Generic routine to allocate memeory for a ring.
 *
 *-----------------------------------------------------------------
 */
int32 luc_initRing( luc_ring_t 	*pRing, 	/* ring pointer */
					int32		memType,	/* local or ctrl mem. */
					int32 		descSize, 	/* size of descriptor */
					int32 		maxDesc)	/* max. # of desc. */
{
	int32		ringSize;
	void		*pMem;

	ringSize  = descSize * maxDesc; 
	
	/*-------------------------------
	 * Allocate ring mem.
	 *-------------------------------
	 */
	pMem = eee_ramAlloc( ringSize );
	if (!pMem)
		return( LUC_ERR_ALLOC_MEM );

	/*-------------------------------
	 * Clear memory.
	 *-------------------------------
	 */
	memset(pMem, 0, ringSize);

	/*-------------------------------
	 * Setup ring pointers.
	 *-------------------------------
	 */
	pRing->startAddr = pMem;
	pRing->endAddr   = pMem + ringSize - descSize;
	pRing->rdAddr    = pMem;
	pRing->wrAddr    = pMem;

	return (NFX_OK);
}


/*-----------------------------------------------------------------
 * Name: luc_initTCR
 *
 * Description: Routine to allocate Transmit Complete Ring. 
 *
 *-----------------------------------------------------------------
 */
#ifdef LINUX_TEST
int32 luc_initTCR( luc_cfgBlk_t *pCfg )
{
	int32		rc;

	/*-------------------------------
	 * Allocate Transmit Complete Ring 
	 * in CPU local memory.
	 *-------------------------------
	 */
	rc = luc_initRing(  &pCfg->tcr,
						LUC_MEM_LOCAL, 
						sizeof(luc_txCmplDesc_t),  
						LUC_MAX_DESC_TCR);
							
	/*-------------------------------
	 * Panic if init failed.
	 *-------------------------------
	 */
	if (rc != NFX_OK)
		luc_panic( );

	return (NFX_OK);
}
#endif



/*-----------------------------------------------------------------
 * Name: luc_init
 *
 * Description: LUC initialization routine.
 *
 * Created by:	Shaun Hui	
 *
 * Date Created: 05/01/01
 *-----------------------------------------------------------------
 */
int32 luc_init( void )
{
	int32			i;
	extern void luc_panic(void);

	pLucCfg = (luc_cfgBlk_t *) eee_allocateBlock( sizeof(luc_cfgBlk_t) );
	if (!pLucCfg)
		luc_panic();
	memset(pLucCfg, 0, sizeof(luc_cfgBlk_t));

#ifdef LINUX_TEST
	luc_initTCR( pLucCfg );	/* Init Tx Complete Ring */
#endif

	/* Su, 04/14/02, set lucMaxLinkNum */
#if defined(PROM)
	lucMaxLinkNum = LUC_MAX_NUM_LINK;
#else
	if (isCheetah())
		lucMaxLinkNum = LUC_MAX_NUM_LINK_CHE;
	else
		lucMaxLinkNum = LUC_MAX_NUM_LINK_GAR;
#endif /* PROM */

	luc_camInit( );			/* Init LUC CAM */


	return NFX_OK;
}


/*-----------------------------------------------------------------
 * Name: luc_initPoll
 *
 * Description: Perform EEE registration with LUC polling functions.
 *
 * Created by:	Shaun Hui	
 *
 * Date Created: 08/09/01
 *-----------------------------------------------------------------
 */
int32 luc_initPoll( void )
{
	luc_cfgBlk_t	*pCfg;
	int32			link;
	int32			port;
	int32			rc;

	pCfg = pLucCfg;

	/*-------------------------------
	 * Register Tx Complete Ring Poll.
	 *-------------------------------
	 */
#ifdef LINUX_TEST
	rc = eee_registerPollingFunc(EEE_HIGH_PRIORITY, luc_pollTCR, &pCfg->tcr);
	if (rc < 0)
		luc_panic( );
#endif

	/*-------------------------------
	 * Get the App. IDs that LUC 
	 * interacts with constantly.
	 *-------------------------------
	 */
	pLucCfg->rcvAppId = eee_getApplicationId("lucRcv");
#ifndef PROM
	pLucCfg->drvAppId = eee_getApplicationId("lucDrv");
#else
#ifdef R9000
#ifdef AGILE_HARDWARE
	pLucCfg->drvAppId = eee_getApplicationId("dp83816");
#else
	pLucCfg->drvAppId = eee_getApplicationId("gt64440_mii");
#endif
#else
	pLucCfg->drvAppId = eee_getApplicationId("gt64240_mii");
#endif
#endif
	pLucCfg->gmcAppId = eee_getApplicationId("lucGmc");
	pLucCfg->sacAppId = eee_getApplicationId("sac");
	pLucCfg->efsAppId = eee_getApplicationId("efs");
	pLucCfg->sbmAppId = eee_getApplicationId("sbm");

	/*-------------------------------
	 * Setup port for rx Data from driver.
	 *-------------------------------
	 */
    pLucCfg->rcvPort = PACK_UNICAST_PORT_ADDRESS(
										nfxMySlotId,
										nfxMyCPUId,
										pLucCfg->rcvAppId,
										EEE_USER_DATA,
										NFX_COS_HIGH,
										0	);
	/*-------------------------------
	 * Setup port for tx Data to driver.
	 *-------------------------------
	 */
#ifdef PROM
	pLucCfg->drvPort[LUC_BP_LINK_ID] = PACK_UNICAST_PORT_ADDRESS(
										nfxMySlotId,
										nfxMyCPUId,
										pLucCfg->drvAppId,
										EEE_USER_DATA,
										NFX_COS_HIGH,
										PROM_PORT_MGMT);
#ifdef SIBYTE
	pLucCfg->drvPort[0] = PACK_UNICAST_PORT_ADDRESS(
										nfxMySlotId,
										nfxMyCPUId,
										pLucCfg->drvAppId,
										EEE_USER_DATA,
										NFX_COS_HIGH,
										PROM_PORT_TXRX0);

#else
	pLucCfg->drvPort[0] = PACK_UNICAST_PORT_ADDRESS(
										nfxMySlotId,
										nfxMyCPUId,
										pLucCfg->drvAppId,
										EEE_USER_DATA,
										NFX_COS_HIGH,
										PROM_PORT_NET0);
#endif
#else
	/* su, 04/14/02, map link to port for cheeta port */
	if (isCheetah()) {
		/* link0, link1 map to NFP_PORT_TXRX_ETH0;
		 * link2, link3 map to NFP_PORT_TXRX_ETH2;
		 * NFP_PORT_TXRX_ETH1 unused
		 */
		for (link = 0; link < 2; link++)
			pLucCfg->drvPort[link] = PACK_UNICAST_PORT_ADDRESS(
				nfxMySlotId,
				nfxMyCPUId,
				pLucCfg->drvAppId,
				EEE_USER_DATA,
				NFX_COS_HIGH,
				NFP_PORT_TXRX_ETH0	);
		
		for (link = 2; link < 4; link++)
			pLucCfg->drvPort[link] = PACK_UNICAST_PORT_ADDRESS(
				nfxMySlotId,
				nfxMyCPUId,
				pLucCfg->drvAppId,
				EEE_USER_DATA,
				NFX_COS_HIGH,
				NFP_PORT_TXRX_ETH2	);
		
	}
	else {
		for (link=0, port = NFP_PORT_TXRX_ETH0; 
			 link < lucMaxLinkNum; 
			 link++, port++)
			{
				pLucCfg->drvPort[link] = PACK_UNICAST_PORT_ADDRESS(
					nfxMySlotId,
					nfxMyCPUId,
					pLucCfg->drvAppId,
					EEE_USER_DATA,
					NFX_COS_HIGH,
					port	);
			}
	}
#endif

	/*-------------------------------
	 * Setup port for rx Data from GMC.
	 *-------------------------------
	 */
    pLucCfg->gmcPort = PACK_UNICAST_PORT_ADDRESS(
										nfxMySlotId,
										nfxMyCPUId,
										pLucCfg->gmcAppId,
										EEE_USER_DATA,
										NFX_COS_HIGH,
										0	);

	/*-------------------------------
	 * Setup port for tx Data to GMC.
	 *-------------------------------
	 */
	pLucCfg->sacPort = PACK_UNICAST_PORT_ADDRESS(
										nfxMySlotId,
										NFX_NFP_TXRX_ANY,
										pLucCfg->sacAppId,
										EEE_USER_DATA,
										NFX_COS_HIGH,
										0	);

	/*-------------------------------
	 * Setup port for ack Data to EFS.
	 *-------------------------------
	 */
    pLucCfg->efsPort = PACK_UNICAST_PORT_ADDRESS(
										nfxMySlotId,
										nfxMyCPUId,
										pLucCfg->efsAppId,
										EEE_USER_DATA,
										NFX_COS_HIGH,
										0	);

	/*-------------------------------
	 * Setup port for req Data from SBM.
	 *-------------------------------
	 */
    pLucCfg->sbmPort = PACK_UNICAST_PORT_ADDRESS(
										nfxMySlotId,
										NFX_NFP_TXRX_ANY,
										pLucCfg->sbmAppId,
										EEE_CONTROL,
										NFX_COS_HIGH,
										0	);

	/*-------------------------------
	 * Register function to receive
	 * data from network.
	 *-------------------------------
	 */
	rc    = eee_registerReceiveFunc(pLucCfg->rcvAppId, 
									luc_rcvPkt, 
									luc_rcvPkt);
	if (rc < 0)
		luc_panic( );

#ifdef PROM
	/*-------------------------------
	 * Register tx cleanup func.
	 *-------------------------------
	 */
	pLucCfg->xmtCmpId = eee_registerCleanupRoutine( luc_xmtCmp );
	if (pLucCfg->xmtCmpId < 0)
		luc_panic( );
#endif

	return NFX_OK;
}

#ifdef PROM
/*-----------------------------------------------------------------
 * Name: luc_initBpIf
 *
 * Description: Create IP IF for for loading off SSC via backplane.
 *
 * Created by:	Shaun Hui	
 *
 * Date Created: 02/11/02
 *-----------------------------------------------------------------
 */
int32 luc_initBpIf( void )
{   
    uint32          ipMask;
	uint32			bpAddr;
	char			*device;

	/*-------------------------------
	 * Our BP IP address starts with
	 * 192.168.192.x
	 *-------------------------------
	 */
	device = luc_ifGetName(LUC_BP_LINK_ID);
	bpAddr = bp_getIpAddr();
    ipMask = LUC_BP_NETMASK;

//    printf("%s bpAddr=0x%x ipMask=0x%x name=%s\n", 
//		__FUNCTION__,  bpAddr, ipMask, device);   

	/*-------------------------------
	 * Bp uses interface device "luc2"
	 *-------------------------------
	 */
    if (bsd_CreateIpIntf(htonl(bpAddr), htonl(ipMask), device, 0, 0) != NFX_OK) 
	{
        printf("%s failed to create IP IF\n", __FUNCTION__);
	}

    return 0;
}
#endif


int32 luc_postInit( )
{
	extern void luc_ReassemInit(void);
	extern void luc_initIf(void);

#ifndef PROM
    int32 luc_xmtGmcDataReqCB(uchar8 * buf, uint32 len, void * cookie);
#endif

	luc_initPoll( );
	luc_initIf() ;			/* Init BSD If structures */
    luc_initIpIf( );        /* Init IP If structures */
	luc_ReassemInit();

#ifdef PROM
	luc_initBpIf( );		/* Init Backplane IP IF */
#endif

#ifndef PROM
    sb1250dm_openChnl(SB1250DM_CHNL_SAC_TO_LUC, luc_xmtGmcDataReqCB);
#endif
	return NFX_OK;
}
