/* vi:ts=4
 *-----------------------------------------------------------------
 *
 * Name: luc-ipif.h
 *
 * RCS:  $Id: luc-ipif.h,v 1.1 2001/12/12 21:24:30 shaun Exp $
 *
 * Copyright (C) 2001, Agile Storage, Inc.
 *
 * Description: Contains LUC IP interfaces related data structures
 *				and definitions.
 *
 * Created by:  Shaun Hui
 *
 * Date Created: 12/10/2001
 *
 *-----------------------------------------------------------------
 */

#ifndef _LUC_IPIF_H
#define _LUC_IPIF_H


/*-------------------------------
 * LUC IP IF structure.
 *-------------------------------
 */
typedef struct luc_ipIf
{   
    short16             inUse;		/* in use flag */
    short16             link;		/* link # */
    uint32              ipaddr;     /* local ip if address */
    void                *ia;        /* ptr to in_ifaddr in bsd code */
    struct luc_ipIf     *next;		/* chain to next ipIf_t of same hash */
} luc_ipIf_t;

/*-----------------------------------------------------------------
 * Name:        luc_ipIfFindEntry
 *
 * Description: Find an IP Interface given link & IP address.
 *
 * Created by:  Shaun Hui
 *  
 * Date Created: 12/10/2001
 *-----------------------------------------------------------------
 */ 
luc_ipIf_t * luc_ipIfFindEntry( short16 link, uint32 ipaddr);


/*-----------------------------------------------------------------
 * Name:        luc_ipIfAddEntry
 *
 * Description: Add an IP Interface given link & IP address.
 *
 * Created by:  Shaun Hui
 *  
 * Date Created: 12/10/2001
 *-----------------------------------------------------------------
 */ 
luc_ipIf_t * luc_ipIfAddEntry( short16 link, uint32 ipaddr);


/*-----------------------------------------------------------------
 * Name:        luc_ipIfDelEntry
 *
 * Description: Delete an IP Interface given link & IP address.
 *
 * Created by:  Shaun Hui
 *  
 * Date Created: 12/10/2001
 *-----------------------------------------------------------------
 */ 
int32 luc_ipIfDelEntry(short16 link, uint32 ipaddr);

/*-----------------------------------------------------------------
 * Name:        luc_initIpIf
 *
 * Description: Initialize IP IF related data structures.
 *
 * Created by:  Shaun Hui
 *  
 * Date Created: 12/10/2001
 *-----------------------------------------------------------------
 */ 
void luc_initIpIf( void );


/*-----------------------------------------------------------------
 * Name:        luc_showIpIfCfg
 *
 * Description: Shows IP Interface table & hash table entries.
 *
 * Created by:  Shaun Hui
 *  
 * Date Created: 12/10/2001
 *-----------------------------------------------------------------
 */ 
int32 luc_showIpIfCfg( void );
#endif
