/* vi:ts=4
 *-----------------------------------------------------------------
 *
 * Name: luc.h
 *
 * RCS:  $Id: luc.h,v 1.29 2002/11/20 18:03:55 swang Exp $
 *
 * Copyright (C) 2001, Agile Storage, Inc.
 *
 * Description: Contains Look Up Classifier related data structure
 *              definitions.
 *
 * Created by:	Shaun Hui
 *
 * Date Created: 5/01/2001
 *
 *-----------------------------------------------------------------
 */

#ifndef _LUC_H
#define _LUC_H

#include <exports/eee-api.h>

#define LUC_ERR_ALLOC_MEM		-100001

#define LUC_HDR_BUFF_SIZE		(PKT_BUFF_SIZE)

#ifdef 	PROM

#define LUC_MAX_NUM_LINK		3					 /* luc0, luc1 &  BP */
#define LUC_BP_LINK_ID			(LUC_MAX_NUM_LINK-1) /* last link for BP */
#define LUC_BP_SUBNET			0xC0A8C000      	 /* 192.168.192.0 */
#define LUC_BP_NETMASK			0xFFFFFF00		

#endif

#define LUC_MAX_NUM_LINK_GAR	2
#define LUC_MAX_NUM_LINK_CHE    4


#define LUC_MAX_DESC_TCR		1024
//#define LUC_MAX_FRAG_CB     	32
#define LUC_MAX_FRAG_CB     	45	

#define LUC_MEM_LOCAL			0
#define LUC_MEM_CTRL			1

#define LUC_TCR_LOOPCOUNT		128


/* #define LUC_DBG_COD			1 */

/*------------------------------------
 * LUC Tx Complete Descriptor Structure
 *------------------------------------
 */
typedef struct
{
    uint32		status;			/* tx complete status */
    void		*pDesc;			/* ptr to Pkt Desc   */
} luc_txCmplDesc_t;


/*------------------------------------
 * Generic LUC ring structure
 *------------------------------------
 */
typedef struct
{
    void		*startAddr;
    void		*endAddr;
    void		*rdAddr;
    void		*wrAddr;

} luc_ring_t;

/*------------------------------------
 * LUC Rx/Tx Statistics
 *------------------------------------
 */
typedef struct
{
	uint32	mcpuErr;
	uint32	noEeeDesc;
	uint32	noPktDesc;
	uint32	noBuffErr;
	uint32	badPktDesc;
	uint32	mcastCnt;
	uint32	pktCnt;
	uint32	byteCnt;
	uint32	discards;
	uint32	slowPktCnt;
	uint32	fastPktCnt;
	uint32	gmcPktCnt;
	uint32	gmcPktFree;
	uint32  verifyErr;			/* pkt verfication */
	uint32	cntCodBound;		/* pkt across cod boundary */
	
	
	/* hardware detected err */
	uint32  hwBadIpVer;			/* bad ver*/
	uint32  hwBadIpCsum;		/* bad ip hdr csum */
	uint32  hwBadTplCsum;		/* bad tcp/udp csum */
	uint32  hwBadIpHlen;		/* bad ip hlen */
	uint32  hwBadTcpHlen;		/* bad tcp hlen */
	uint32  hwErr;				/* unrecognized err */
	uint32  hwBadClass;			/* hardware detected bad classification */
	uint32  linkErr;			/* bad link */
	uint32	reaChkSumErr;		/* reasembly chksum err */
} luc_stats_t;

/*------------------------------------
 * LUC link cfg/info.
 *------------------------------------
 */
typedef struct
{
	uint32			state;
	char			macAddr[8];
	luc_stats_t		rxStats;
	luc_stats_t		txStats;
} luc_linkCfg_t;

/*------------------------------------
 * LUC Configuration Ctrl Blk.
 *------------------------------------
 */
typedef struct
{
	uint32			rcvPort;						/* LUC RX port */
	uint32			rcvAppId;						/* LUC RX app id. */
	uint32			drvPort[LUC_MAX_NUM_LINK_CHE];  /* Driver port */
	uint32			drvAppId;						/* Driver app id  */
	uint32			gmcPort;						/* GMC to LUC RX port */
	uint32			gmcAppId;						/* GMC to LUC RX app id */
	uint32			sacPort;		
	uint32			sacAppId;		
	uint32			efsPort;		
	uint32			efsAppId;		
	uint32			sbmPort;		
	uint32			sbmAppId;		
	int32			xmtCmpId;		
	luc_linkCfg_t	linkCfg[LUC_MAX_NUM_LINK_CHE];
	luc_ring_t		tcr;
	int				rxVerify;
    uint32          allocCnt;
	uint32			sbmSndSeq;						/* next seq num. for send to sbm */
	uint32			sbmAckSeq;						/* next seq num expected from sbm */
	
} luc_cfgBlk_t;

/*------------------------------------
 * LUC Fragment  Ctrl Blk.
 *------------------------------------
 */
typedef struct
{
	nfx_cod_t   cod;
    int32       codLength;
    int32       codOffset;	
} luc_cod_t;


typedef struct
{
    int32       ipOffset;
	int32		fcbCodNum;
	luc_cod_t	fcbCodEntry[NFX_MAX_COD_MSG];
} luc_fragCb_t;


extern luc_cfgBlk_t	*pLucCfg;
extern int lucMaxLinkNum;
void luc_pollTxCmp( );

void *luc_allocLocalMem( int32 size );
void *luc_allocCtrlMem( int32 size );


void luc_logError( int32 err_code );
void luc_abort( int32 err_code );

#ifdef SBM_REV2

int32 luc_sndGmcData(pkt_desc_t		*pDesc,
                    uchar8      	*pData,
                    int32       	dataSize,
                    int32       	dataOffset,
                    int32       	padSize,
                    nfx_cod_entry_t	*pCod);

#else

int32 luc_sndGmcData(pkt_desc_t  	*pDesc,
                    uchar8      	*pData,
                    int32       	dataSize,
                    nfx_cod_entry_t	*pCod);
#endif

int32 luc_simCamFind(int32, uint32, uint32, ushort16, ushort16);
void  luc_ifSlowPath( int32, pkt_desc_t*);
int32 luc_getActLALink(int32 link);
int32 luc_getUpLALink(int32 link);
void* luc_getIntfFromLink(int32 link);

#ifdef LINUX_TEST

/* Array of LUC's users.
 * Use by data mover to add some flexibility to LUC.
 */

#define MAX_LUC_USER    50
typedef struct
{
    boolean     inUse;
    ushort16    port;   // network byte order
    int         sock;
} luc_user_t;
luc_user_t  lucUserTab[MAX_LUC_USER];

void luc_initLucUserTable(void);
int  luc_getFreeLucUserIndex(void);
void luc_freeLucUserIndex(int index);
#endif // LINUX_TEST


#endif /* _LUC_H */
