/*
 *-----------------------------------------------------------------
 *
 * Name:	mdk-run.c
 *
 * RCS:  $Id: mdk-run.c,v 1.9 2002/08/28 22:17:52 lipikaathome Exp $
 *
 * Copyright (C) 2001, Agile Storage, Inc.
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	1/29/02
 *
 *-----------------------------------------------------------------
 */

#include <nfx-incl.h>
#include <exports/mdk-api.h>
#include "tests/mdk-mem.h"

#ifdef DIAG_SIBYTE
extern uint32 sb_conflict;
#elif defined (DIAG_R7000)
extern uint32 fc_conflict;
#endif

#define CNTRL(x)        ((x) & 0x1f)
uint32 testPassed = 0, testFailed = 0, testTimeout = 0;
extern mdk_loopback_t mdk_testState[6];
extern uint64 memorysize;
#define MEG (1024*1024)

/*-----------------------------------------------------------------
 * Name:	mdk_run_package_init()
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	1/14/02
 *
 *-----------------------------------------------------------------
 */
int32
mdk_run_package_init(
	mdk_package_t	** pkgPtr
)
{
	mdk_init_params(pkgPtr);

	if ((*pkgPtr)->packageInit)
		(*(*pkgPtr)->packageInit)();

	return 0;
}

uint32
mdk_show_results(
		uint32 testresult,
		uint32 i
)
{
#ifdef DIAG_SIBYTE
		if (sb_conflict)
		{
			if(!isCheetah()) {
				switch (i) {
					case 0:
						printf("\n\t\tSIBYTE MAC %d :",i);
						break;
					case 1:	
						printf("\t\tSIBYTE MAC %d :",i);
						break;
					case 2:
						printf("\t\tDP83820	:");
						break;
					default:
						break;
				}
			}
			else {	
				/* In a cheetah system, TXRX has 4 ext ports + 1 bp port
				 * and FP has 6 bp ports
				 */ 	
				if(isTXRX()) {
					if(i == 0)
						printf("\n");	
					if(i<4)
						printf("\t\tLINK %d :",i);
					if (i == 4)
						printf("\t\tDP83820	:");
				}
				else if(isFP()) {	
					if(i == 0)	
						printf("\n");	
					if(i<6)
						printf("\t\tLINK %d :",i);
				}		
			}	
		}		
#elif defined (DIAG_R7000)
		if (fc_conflict)
		{
			switch (i) {
				case 0:
					printf("\n\t\tGalileo MAC 2:");
					break;
				case 1:	
					printf("\t\tGalileo MAC 1:");
					break;
				case 2:
					printf("\t\tDP83820	:");
					break;
				default:
					break;
			}
		}		
#endif		
		if (testresult == RET_PASSED){
			printf("\b \t\tPASS\n");
			testPassed ++;
		}	
		else if (testresult == RET_SKIP_TEST) {
			printf("\b \t\tSKIPPED\n");
		}	
		else if (testresult == RET_DEV_UP) {
			printf("\b \t\t%s is UP!\n",tp->devName);
		}	
		else if (testresult == RET_DEV_DOWN) {
			if((isTXRX()) || (isFP())) 
			{	
				if((strncmp(tp->devName,"LSI",3)) == 0)	
					printf("\b \t\t%s is DOWN! Attach Loopback cables.\n",tp->devName);
				else
					printf("\b \t\t%s is DOWN!\n",tp->devName);
			}	
			else
				printf("\b \t\t%s is DOWN!\n",tp->devName);
		}	
		else if (testresult == RET_FAILED) {
			printf("\b \t\tFAILED\n");
			/* printf("%s  FAILED :Not within expected range.\n",tp->testName);*/
			testFailed++;
		}	
		else if(testresult == ERR_TIMEOUT) {
			printf("\b \t\tFAILED(T.O)\n");
			/* printf("%s  FAILED due to timeout.\n",tp->testName);*/
			testTimeout++;
		}	
		else if(testresult == ERR_LPBKDATA) {
			printf("\b \t\tFAILED(DE)\n");
			testFailed++;
		}	
		else if (testresult == RET_SKIPPED) {
		/*	printf("Loopback tests. Results will be displayed once test ends.\n");*/
		}
		else if(testresult >= RET_TEST_ERR_BASE) {
			printf("\b \t\tFAILED\n");
			testFailed++;
		}
		else {
			printf("\b \t\tUNKNOWN %d\n", testresult);
		}
		return(testresult);
}


/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	1/29/02
 *
 *-----------------------------------------------------------------
 */
void
mdk_run_test(
	mdk_package_t		** pkgPtr,
	mdk_test_t			* testPtr
)
{
	uchar8	buf[LINE_WIDTH];
	uint32 i,rc;
	uchar8 * tmp;

	tmp = "MEM";
	sprintf(buf, "%s:%s", (*pkgPtr)->menuName, testPtr->testName);
	printf("%-30s", buf);
	if((strcmp(testPtr->testName, "Full DRAM Test")) == 0) 
		printf("\b\b\b\b\b\b\b\b\b\b (MemSize %dMb) ",memorysize/MEG);
#if 0	
#ifdef DIAG_SIBYTE
	if(sb_conflict)
		//printf("\n");
#elif defined (DIAG_R7000)	
	if(fc_conflict)
		//printf("\n");
#endif
#endif		
	tp->menuName = (*pkgPtr)->menuName;
	tp->testName = testPtr->testName;

	tp->testresult = 0;
	(*testPtr->test)(tp);

	if(tp->testresult == RET_SKIPPED) {
#ifdef DIAG_SIBYTE
		if(sb_conflict)
#elif defined (DIAG_R7000)			
		if(fc_conflict)
#endif			
	   	{
			/* make sure all tests are completed */
			if(isFP()) {
				if(!isCheetah()) {
					while (((mdk_testState[1].state) != TEST_DONE) || 
						((mdk_testState[0].state) != TEST_DONE))	
					{	
						eee_poll();
					}
				}
				else {
					while (((mdk_testState[0].state) != TEST_DONE) || 
						((mdk_testState[1].state) != TEST_DONE) ||
						((mdk_testState[2].state) != TEST_DONE) ||	
						((mdk_testState[3].state) != TEST_DONE) ||	
						((mdk_testState[4].state) != TEST_DONE) ||	
						((mdk_testState[5].state) != TEST_DONE)) 
					{	
						eee_poll();
					}
				}		
			}
			else if(isTXRX()) {
				if(!isCheetah()) {	
					while (((mdk_testState[2].state) != TEST_DONE) || 
						((mdk_testState[1].state) != TEST_DONE) || 
						((mdk_testState[0].state) != TEST_DONE))	
					{	
						eee_poll();
					}
				}
				else {
					while (((mdk_testState[4].state) != TEST_DONE) || 
						((mdk_testState[3].state) != TEST_DONE) || 
						((mdk_testState[2].state) != TEST_DONE) || 
						((mdk_testState[1].state) != TEST_DONE) || 
						((mdk_testState[0].state) != TEST_DONE))	
					{	
						eee_poll();
					}
				}		
			}
			else {
				while (((mdk_testState[2].state) != TEST_DONE) || 
					((mdk_testState[1].state) != TEST_DONE) || 
					((mdk_testState[0].state) != TEST_DONE))	
				{	
					eee_poll();
				}
			}	
		}
		else {	
			while(!test_done)
				eee_poll();
		}	
	}	

	if(tp->resultCnt == 1) {
		rc = mdk_show_results(tp->testresult,0);
		if(rc  >= RET_TEST_ERR_BASE) 
			mdk_log_error(pkgPtr,0xFFFF0000);
	}
	else if(tp->resultCnt > 1) {
		uint32 err = 0;	

		/* For mem tests if any test fails, print failed */
		if((strcmp(tp->menuName, tmp)) == 0) {
			for(i=0;i<tp->resultCnt;i++) {
				if(tp->result[i].testresult >= RET_TEST_ERR_BASE)
					err++;	
			}		
			if(err)
				tp->testresult = RET_TEST_ERR_BASE;
			else
				tp->testresult = RET_PASSED;
			rc = mdk_show_results(tp->testresult,0);
			/* log for individual test though */
			for(i=0;i<tp->resultCnt;i++) {
				tp->testresult = tp->result[i].testresult;
				if( tp->testresult >= RET_TEST_ERR_BASE)
					mdk_log_error(pkgPtr,i);
			}
		}	
		else {
			for(i=0;i<tp->resultCnt;i++) {
				tp->testresult = tp->result[i].testresult;
				rc = mdk_show_results(tp->testresult,i);
				if(rc  >= RET_TEST_ERR_BASE) 
					mdk_log_error(pkgPtr,i);
			}
		}	
		tp->resultCnt = 1;
	}		
	/* Reset the conflict test flag here */
#ifdef DIAG_SIBYTE
	if (sb_conflict)
		sb_conflict = 0;
#elif defined (DIAG_R7000)
	if (fc_conflict)
		fc_conflict = 0;
#endif	
	/* Allow next test to proceed */
	testPtr->test_complt = 1;
	test_done = 0;
}


/*-----------------------------------------------------------------
 * Name:	mdk_run_accept()
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	1/14/02
 *
 *-----------------------------------------------------------------
 */
int32
mdk_run_sub_accept(
	mdk_package_t	** pkgPtr,
	uchar8			* req,		/* tests */
	uint32			reqnum,		/* how many in list are valid */
	uint32			num_basic,
	uint32			num_restr
)
{
	mdk_test_t	* test;
	uint32		i,j;
	mdk_test_t	* prv_test;

	if(mdkp->num_loops) {
			for(j=0;j<mdkp->num_loops;j++) {
				for (i=0; i<reqnum; i++) {
					test = mdk_sub_get_test(pkgPtr, num_basic, num_restr, req[i]);
					test->test_complt = 0;
					if(i==0)
						prv_test = 0;
					else
						prv_test = prv_test;
					if(prv_test) {
						while(!(prv_test->test_complt))
							eee_poll();
					}	
					mdk_run_test(pkgPtr, test);
					prv_test = test;
					if (char_avail()) 
					{
						if (getchar() == CNTRL('e'))
							return (-1);
					}
				}
			}
	}
	else {
		for (i=0; i<reqnum; i++) {
				test = mdk_sub_get_test(pkgPtr, num_basic, num_restr, req[i]);
				/* Run the next test only when the prv one has ended */
				test->test_complt = 0;
				if(i==0)
					prv_test = 0;
				else
					prv_test = prv_test;
				if(prv_test) {
					while(!(prv_test->test_complt))
							eee_poll();
				}	
				mdk_run_test(pkgPtr, test);
				prv_test = test;
				if (char_avail()) 
				{
					if (getchar() == CNTRL('e'))
						return (-1);
				}
		}	
	}	

	return 0;
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	1/30/02
 *
 *-----------------------------------------------------------------
 */
int32
mdk_run_top_accept(
	uchar8			* req_pkgs,	/* packages */
	uint32			reqnum		/* how many in list are valid */
)
{
	mdk_package_t	** pkgPtr;
	uchar8			req_tests[MDK_MAX_TESTS_PER_PACKAGE];
	uint32			num_tests_todo;
	uint32			i;
	uint32			num_basic, num_restr;
	int32			rc;

	for (i=0; i<reqnum; i++) {
		pkgPtr = &(mdkp->pkg[req_pkgs[i]]);

		mdk_run_package_init(pkgPtr);
		mdk_get_num_tests(pkgPtr, &num_basic, &num_restr);
		num_tests_todo = mdk_get_all_tests(pkgPtr, req_tests);
		rc = mdk_run_sub_accept( pkgPtr,
							req_tests,
							num_tests_todo,
							num_basic,
							num_restr);
		if (char_avail()) 
		{
			if (getchar() == CNTRL('e'))
				return 0;
		}
		if(rc < 0)
			return (-1);	
	}

	return 0;
}

