/* vi:ts=4
 *-----------------------------------------------------------------
 *
 * Name:
 *
 * RCS:  $Id: mdk-common-util.c,v 1.7 2002/08/07 20:37:02 lipikaathome Exp $
 *
 * Copyright (C) 2001, Agile Storage, Inc.
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	1/29/02
 *
 *-----------------------------------------------------------------
 */

#include <nfx-incl.h>
#include <exports/mdk-api.h>
#include "mdk-mem.h"
#include "../../sm-seep/seep-api.h"
#include "../../sm-seep/seep.h"
#if defined (DIAG_SIBYTE) || defined (DIAG_R7000)
#include <exports/i2c-api.h>
#endif

extern uint32 nfxMyCPUId;
int32 from_programbytes(int32 offset, char8 * src_buf, uint32 len);
int32 from_readbytes(int32 offset, uchar8 * dest_buf, uint32 len);

seepPCB_t test_seep = {
	"1111",
	"222222222",
	"333333333",
	"40404040",
	"FFFF",
	"D0D0",
	"999999999",
	"7F7F7F7F",
	"55555"
};		

#define	SSC_I2C_ADDR	0xA8
#define	FC_I2C_ADDR		0xA6  	/* Board Seep */
#define FC_SLICE_SEEP_ADDR	0xAE	/* Slice Seep for beta boards */
#define	FP_I2C_ADDR		0x50


int32
seep_test(
	mdk_test_params_t	* tp
)
{
	seepPCB_t orig_seep, ret_seep;
	uint32 rcw, rcr;

	/* Store current SEEP config to local structure */

	if((isTXRX()) || (isFP()))
		i2c(I2C_READ , FP_I2C_ADDR, 0, sizeof(seepPCB_t), (uchar8*)&orig_seep);
	else if ((isFC()) && (nfxMyCPUId == 0))
		i2c(I2C_READ , FC_I2C_ADDR, 0, sizeof(seepPCB_t), (uchar8*)&orig_seep);
	else if ((isFC()) && (nfxMyCPUId != 0)) {
		tp->testresult = RET_SKIP_TEST;
		return (tp->testresult);
	}	
	else if (isSSC())
		i2c(I2C_READ , SSC_I2C_ADDR, 0, sizeof(seepPCB_t), (uchar8*)&orig_seep);

	if (tp->param[TP_VERBOSE] == 1) {
		printf("\nOriginal SEEP info:\n");
		printf("%s\n",orig_seep.signature);
		printf("%s\n",orig_seep.model_num);
		printf("%s\n",orig_seep.assembly_num);
		printf("%s\n",orig_seep.serial_num);
		printf("%s\n",orig_seep.revision);
		printf("%s\n",orig_seep.deviation);
		printf("%s\n",orig_seep.ip_addr);
		printf("%s\n",orig_seep.ip_mask);
		printf("%s\n",orig_seep.misc_info);
	}
	delay_us(10000);

	if (tp->param[TP_VERBOSE] == 1) {
		printf("Test SEEP :\n");
		printf("%s\n",test_seep.signature);
		printf("%s\n",test_seep.model_num);
		printf("%s\n",test_seep.assembly_num);
		printf("%s\n",test_seep.serial_num);
		printf("%s\n",test_seep.revision);
		printf("%s\n",test_seep.deviation);
		printf("%s\n",test_seep.ip_addr);
		printf("%s\n",test_seep.ip_mask);
		printf("%s\n",test_seep.misc_info);
	}
	delay_us(10000);

	if((isTXRX()) || (isFP()))
		rcw = i2c(I2C_WRITE , FP_I2C_ADDR, 0, sizeof(seepPCB_t), (uchar8*)&test_seep);
	else if ((isFC()) && (nfxMyCPUId == 0))
		rcw = i2c(I2C_WRITE , FC_I2C_ADDR, 0, sizeof(seepPCB_t), (uchar8*)&test_seep);
	else if (isSSC())
		rcw = i2c(I2C_WRITE , SSC_I2C_ADDR, 0, sizeof(seepPCB_t), (uchar8*)&test_seep);

	if(rcw)
		return(tp->testresult = ERR_WRITE_FAIL);

	if((isTXRX()) || (isFP()))
		rcr = i2c(I2C_READ , FP_I2C_ADDR, 0, sizeof(seepPCB_t), (uchar8*)&ret_seep);
	else if ((isFC()) && (nfxMyCPUId == 0))
		rcr = i2c(I2C_READ , FC_I2C_ADDR, 0, sizeof(seepPCB_t), (uchar8*)&ret_seep);
	else if (isSSC())
		rcr = i2c(I2C_READ , SSC_I2C_ADDR, 0, sizeof(seepPCB_t), (uchar8*)&ret_seep);

	if (tp->param[TP_VERBOSE] == 1) {
		printf("\nRetrieved SEEP info:\n");
		printf("%s\n",ret_seep.signature);
		printf("%s\n",ret_seep.model_num);
		printf("%s\n",ret_seep.assembly_num);
		printf("%s\n",ret_seep.serial_num);
		printf("%s\n",ret_seep.revision);
		printf("%s\n",ret_seep.deviation);
		printf("%s\n",ret_seep.ip_addr);
		printf("%s\n",ret_seep.ip_mask);
		printf("%s\n",ret_seep.misc_info);
	}
	delay_us(10000);
	if( memcmp(&test_seep, &ret_seep,sizeof(seepPCB_t)) == 0) {
		tp->testresult = RET_PASSED;
	}	
	else
		tp->testresult = ERR_UNEXPECTED_RESULT;

	/* restore original seep values */
	if((isTXRX()) || (isFP()))
		rcw = i2c(I2C_WRITE , FP_I2C_ADDR, 0, sizeof(seepPCB_t), (uchar8*)&orig_seep);
	else if ((isFC()) && (nfxMyCPUId == 0))
		rcw = i2c(I2C_WRITE , FC_I2C_ADDR, 0, sizeof(seepPCB_t), (uchar8*)&orig_seep);
	else if (isSSC())
		rcw = i2c(I2C_WRITE , SSC_I2C_ADDR, 0, sizeof(seepPCB_t), (uchar8*)&orig_seep);
	if(rcw)
		return(tp->testresult = ERR_WRITE_FAIL);

	return(tp->testresult);
	
}	

#ifdef DIAG_R7000
int32
slice_seep_test(
	mdk_test_params_t	* tp
)
{
	seepPCB_t orig_seep, ret_seep;
	uint32 rcw, rcr;
	uint32 i;

	/* For Beta boards ONLY !!! */

	if(isFC()) {	
		/* Store current SEEP config to local structure */
		i2c(I2C_READ , FC_SLICE_SEEP_ADDR, 0, sizeof(seepPCB_t), (uchar8*)&orig_seep);

		if (tp->param[TP_VERBOSE] == 1) {
			printf("\nOriginal SEEP info:\n");
			printf("%s\n",orig_seep.signature);
			printf("%s\n",orig_seep.model_num);
			printf("%s\n",orig_seep.assembly_num);
			printf("%s\n",orig_seep.serial_num);
			printf("%s\n",orig_seep.revision);
			printf("%s\n",orig_seep.deviation);
			printf("%s\n",orig_seep.ip_addr);
			printf("%s\n",orig_seep.ip_mask);
			printf("%s\n",orig_seep.misc_info);
		}
		delay_us(10000);

		if (tp->param[TP_VERBOSE] == 1) {
			printf("Test SEEP :\n");
			printf("%s\n",test_seep.signature);
			printf("%s\n",test_seep.model_num);
			printf("%s\n",test_seep.assembly_num);
			printf("%s\n",test_seep.serial_num);
			printf("%s\n",test_seep.revision);
			printf("%s\n",test_seep.deviation);
			printf("%s\n",test_seep.ip_addr);
			printf("%s\n",test_seep.ip_mask);
			printf("%s\n",test_seep.misc_info);
		}
		delay_us(10000);

		rcw = i2c(I2C_WRITE , FC_SLICE_SEEP_ADDR, 0, sizeof(seepPCB_t), (uchar8*)&test_seep);

		if(rcw)
			return(tp->testresult = ERR_WRITE_FAIL);

		rcr = i2c(I2C_READ , FC_SLICE_SEEP_ADDR, 0, sizeof(seepPCB_t), (uchar8*)&ret_seep);

		if (tp->param[TP_VERBOSE] == 1) {
			printf("\nRetrieved SEEP info:\n");
			printf("%s\n",ret_seep.signature);
			printf("%s\n",ret_seep.model_num);
			printf("%s\n",ret_seep.assembly_num);
			printf("%s\n",ret_seep.serial_num);
			printf("%s\n",ret_seep.revision);
			printf("%s\n",ret_seep.deviation);
			printf("%s\n",ret_seep.ip_addr);
			printf("%s\n",ret_seep.ip_mask);
			printf("%s\n",ret_seep.misc_info);
		}
		delay_us(10000);
		if( memcmp(&test_seep, &ret_seep, sizeof(seepPCB_t)) == 0) {
			tp->testresult = RET_PASSED;
		}	
		else
			tp->testresult = ERR_UNEXPECTED_RESULT;

		/* restore original seep values */
		rcw = i2c(I2C_WRITE , FC_SLICE_SEEP_ADDR, 0, sizeof(seepPCB_t), (uchar8*)&orig_seep);
		if(rcw)
			return(tp->testresult = ERR_WRITE_FAIL);
	}
	else
		tp->testresult = RET_SKIP_TEST;
	return(tp->testresult);
	
}	
#endif

domt_walking_ones(unsigned long addr, unsigned long size, mdk_test_params_t *tp)
{
	int             i, j, temp;
	volatile unsigned int    w, *p, r;

	for (p = (uint32 *)addr, i = 0; i < size; i += 4, p++) {
		w = 1;
		for (j = 0; j < 32; j++) {
	    	*p = w;
	    	r = *p;
	    	if (r != w) {
				(int)tp->result[1].addr[i+j] = p;
		       	(int)tp->result[1].exptd[i+j] = w;
				(int)tp->result[1].rcvd[i+j] = r;	
				if (tp->param[TP_VERBOSE] == 1) {
					printf ("\b\nerror: addr=%08x read=%08x expected=%08x  ",
								p, r, w);
				}
				tp->testresult = tp->result[1].testresult = ERR_DATA_FAIL_1;
				if(!(tp->param[TP_CONT_DESP_ERR] )) 
					return(tp->testresult);
	    	}
	    	w <<= 1;
	    	dotik (1, 0);
		}
	}
	if(tp->testresult != ERR_DATA_FAIL_1)
		tp->testresult = tp->result[1].testresult = RET_PASSED;
	return(tp->testresult);
}	

int32
domt_walking_zeros(unsigned long addr, unsigned long size, mdk_test_params_t *tp)
{
	int             i, j, temp;
	volatile unsigned int    w,val, *p, r;

	for (p = (uint32 *)addr, i = 0; i < size; i += 4, p++) {
		w = 1;
		for (j = 0; j < 32; j++) {
            val = ~w;
	    	*p = val;
	    	r = *p;
	    	if (r != val) {
			(int)tp->result[0].addr[i+j] = p;
	       	(int)tp->result[0].exptd[i+j] = val;
			(int)tp->result[0].rcvd[i+j] = r;	
			if (tp->param[TP_VERBOSE] == 1) {
				printf ("\b\nerror: addr=%08x read=%08x expected=%08x  ",
						p, r, val);
			}
			tp->testresult = tp->result[0].testresult = ERR_DATA_FAIL_0;
			if(!(tp->param[TP_CONT_DESP_ERR] ))
				return(tp->testresult);
 			}	
	    	w <<= 1;
	    	dotik (1, 0);
		}
	}
	if(tp->testresult != ERR_DATA_FAIL_0)
		tp->testresult = tp->result[0].testresult = RET_PASSED;
	return(tp->testresult);
}	

int32
domt_address(unsigned long addr, unsigned long size, mdk_test_params_t *tp)
{
	int             i;
	volatile unsigned int    *p, r;

	/* store the address in each address */
   	for (p = (uint32 *)addr, i = 0; i < size; i += 4, p++) {
		*p = (unsigned int)p;
		dotik (1, 0);
   	}

	/* check that each address contains its address */
   	for (p = (uint32 *)addr, i = 0; i < size; i += 4, p++) {
		r = *p;
		if (r != (unsigned int)p) {
			(int)tp->result[2].addr[i] = p;
	        (int) tp->result[2].exptd[i] = p;
			(int)tp->result[2].rcvd[i] = r;	
			if (tp->param[TP_VERBOSE] == 1) {
	    		printf ("\b\nerror: addr=%08x read=%08x expected=%08x  ", p, r, p);
			}
			tp->testresult = tp->result[2].testresult = ERR_ADDR_FAIL;
			if(!(tp->param[TP_CONT_DESP_ERR] ))
				return;
		}
		dotik (1, 0);
   	}
	if(tp->testresult != ERR_ADDR_FAIL)
		tp->testresult = tp->result[2].testresult = RET_PASSED;
	return(tp->testresult);
}	

do_mt_bogus(int a,int b, mdk_test_params_t *tp)
{
	printf("mt_bogus!!!\n");
	tp->result[1].addr[0] = 0x11111111;
        tp->result[1].exptd[0] = 1;
	tp->result[1].rcvd[0] = 2;	
	tp->result[1].addr[1] = 0x22222222;
        tp->result[1].exptd[1] = 3;
	tp->result[1].rcvd[1] = 4;
	tp->result[1].addr[2] = 0x33333333;
        tp->result[1].exptd[2] = 5;
	tp->result[1].rcvd[2] = 6;
	tp->testresult = tp->result[1].testresult = ERR_DATA_FAIL_1;
	return(tp->testresult);
}	

do_mt_0_bogus(int a,int b, mdk_test_params_t *tp)
{
	printf("mt_bogus_0 !!!\n");
	tp->result[0].addr[0] = 0x99999999;
        tp->result[0].exptd[0] = 1;
	tp->result[0].rcvd[0] = 2;	
	tp->result[0].addr[1] = 0x88888888;
        tp->result[0].exptd[1] = 3;
	tp->result[0].rcvd[1] = 4;
	tp->result[0].addr[2] = 0x77777777;
        tp->result[0].exptd[2] = 5;
	tp->result[0].rcvd[2] = 6;
	tp->testresult = tp->result[0].testresult = ERR_DATA_FAIL_0;
	return(tp->testresult);
}	

do_mt_add_bogus(int a,int b, mdk_test_params_t *tp)
{
	printf("mt_add_bogus!!!\n");
	tp->result[2].addr[0] = 0xaaaaaaaa;
        tp->result[2].exptd[0] = 7;
	tp->result[2].rcvd[0] = 8;	
	tp->result[2].addr[1] = 0xbbbbbbbb;
        tp->result[2].exptd[1] = 9;
	tp->result[2].rcvd[1] = 0xa;	
	tp->result[2].addr[2] = 0xcccccccc;
        tp->result[2].exptd[2] = 0xb;
	tp->result[2].rcvd[2] = 8;	
	tp->result[2].addr[3] = 0xdddddddd;
        tp->result[2].exptd[3] = 9;
	tp->result[2].rcvd[3] = 8;	
	tp->testresult = tp->result[2].testresult = ERR_ADDR_FAIL;
	return(tp->testresult);
}	

#if 0
#ifdef DIAG_SIBYTE
int32
bigBootrom_test(
	mdk_test_params_t	* tp
)
{
    	seep_t	ret_seep;

	if(isFP())
		return (tp->testresult = RET_SKIP_FP);

	from_programbytes(0xbec00000, (uchar8*)&test_seep, sizeof(seep_t));

	if (tp->param[TP_VERBOSE] == 1) {
		printf("Test SEEP contents:\n");
		printf("%s\n",test_seep.name);
		printf("%s\n",test_seep.part_num);
		printf("%s\n",test_seep.revision);
		printf("%s\n",test_seep.deviation);
		printf("%s\n",test_seep.power);
		printf("%s\n",test_seep.serial_num);
		printf("%s\n",test_seep.ip_addr);
	}
	
	delay_us(10000);

	from_readbytes(0xbec00000,(uchar8*)&ret_seep, sizeof(seep_t));

	if (tp->param[TP_VERBOSE] == 1) {
		printf("Retrieved SEEP contents:\n");
		printf("%s\n",ret_seep.name);
		printf("%s\n",ret_seep.part_num);
		printf("%s\n",ret_seep.revision);
		printf("%s\n",ret_seep.deviation);
		printf("%s\n",ret_seep.power);
		printf("%s\n",ret_seep.serial_num);
		printf("%s\n",ret_seep.ip_addr);
	}	

	delay_us(10000);

	if( memcmp(&test_seep, &ret_seep, sizeof(seep_t)) == 0) {
		tp->testresult = RET_PASSED;
	}	
	else
		tp->testresult = ERR_UNEXPECTED_RESULT;
}		
#endif
#endif

