/* vi:ts=4
 *-----------------------------------------------------------------
 *
 * Name:
 *
 * RCS:  $Id: mdk-gmac.c,v 1.7 2002/07/27 01:36:16 lipikaathome Exp $
 *
 * Copyright (C) 2001, Agile Storage, Inc.
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	1/29/02
 *
 *-----------------------------------------------------------------
 */

#include <nfx-incl.h>
#include <exports/mdk-api.h>
#include "mdk-mem.h"

#if defined (DIAG_SIBYTE) || defined (DIAG_R7000)
#include "mdk-dp83820.h"

/* Copied some register defines from dp83820-mii.c to here */
#define	BMCR	0x00 	/* Basic Mode Control (rw) */
#define	BMSR	0x01	/* Basic Mode Status (ro) */
#define	PHYIDR1			0x02
#define	PHYIDR2			0x03
#define MIIread 		0x60000000l
#define	MIIwrite		0x50020000l
#define MDDIR           0x00000020      /* MII management data direction    */
#define MDC             0x00000040      /* MII management data clock        */
#define MDIO            0x00000010      /* MII management data input/output */


/* Some definations also used in device driver  */
extern dp83820_cfg_t *dp83820;
extern uint32 dp83820_rx_pkt_trace;
extern uint32 dp83820_tx_pkt_trace;
extern int32 dp83820_txUnicast(uint32, eee_desc_t *);

/* Variables used in MDK only */
extern uint32 conflict_timeout;
mdk_dp83820_test_t  mdk_dp83820_testControl;


/* Global Variables */
#ifdef DIAG_SIBYTE
extern uint32 sb_conflict;
extern int32 sb_lpbk_timeout(int32 tmr_handle, void * usr_param0, void* usr_param1, void* usr_param2,void* usr_param3);
#elif defined (DIAG_R7000)
int32 fc_lpbk_timeout(int32 tmr_handle, void * usr_param0, void* usr_param1, void* usr_param2,void* usr_param3);
extern uint32 fc_conflict;
#endif

extern mdk_loopback_t mdk_testState[6];

extern uint32	time_handle_dp83820;

int mdk_dp83820_diagSetLbMode( dp83820_cfg_t *dp, uint32 mode );
int mdk_dp83820_diagLBTest(dp83820_cfg_t *dp,mdk_test_params_t *tp,uint32 testMode);
int mdk_dp83820_diagShowLbTestResults(mdk_dp83820_test_t *t);

uint32 gmac_test_started = 0;
uint32 dpprvRxPktCnt;
uint32 dpconsecutivePktLoss;

void
gmac_pkg_init()
{
	tp->resultCnt = 1;
	tp->param[TP_LBPKT_CNT] = 10000;
	if(!time_handle_dp83820)
		time_handle_dp83820 = eee_timerCreate(MDK_INTERPKTDELAY, EEE_TIMER_ONCE);
}

/* MAIN TEST FUNCTIONS */

int32
gmac_intlb_test(
	mdk_test_params_t	* tp
)
{
	uint32 testMode;

	if(isFP())
	{
		return (tp->testresult = RET_SKIP_FP);
	}	

	testMode = TEST_INT;
	switch (tp->param[TP_PKT_TRACE]) {
		case 0:
			dp83820_rx_pkt_trace= 0;
			dp83820_tx_pkt_trace= 0;
			break;
		case 1:
			dp83820_rx_pkt_trace = 1;
			dp83820_tx_pkt_trace= 0;
			break;
		case 2:
			dp83820_rx_pkt_trace= 0;
			dp83820_tx_pkt_trace= 1;
			break;
		case 3:
			dp83820_rx_pkt_trace = 1;
			dp83820_tx_pkt_trace= 1;
			break;
		default: 
			printf("Invalid Entry: %d\n", tp->param[TP_PKT_TRACE]);
	}

	mdk_dp83820_diagSetLbMode(dp83820, DP83820_LB_ENABLE);
	mdk_dp83820_diagLBTest(dp83820, tp,testMode);
	
	tp->testresult = RET_SKIPPED;
	return (tp->testresult);
}

#ifdef DIAG_SIBYTE	
int32
gmac_extlb_test(
	mdk_test_params_t	* tp
)
{
	uint32 testMode;

	if(isFP())
	{
		return (tp->testresult = RET_SKIP_FP);
	}	

	testMode = TEST_EXT; /* 2 */ 
	switch (tp->param[TP_PKT_TRACE]) {
		case 0:
			dp83820_rx_pkt_trace= 0;
			dp83820_tx_pkt_trace= 0;
			break;
		case 1:
			dp83820_rx_pkt_trace = 1;
			dp83820_tx_pkt_trace= 0;
			break;
		case 2:
			dp83820_rx_pkt_trace= 0;
			dp83820_tx_pkt_trace= 1;
			break;
		case 3:
			dp83820_rx_pkt_trace = 1;
			dp83820_tx_pkt_trace= 1;
			break;
		default: 
			printf("Invalid Entry: %d\n", tp->param[TP_PKT_TRACE]);
	}
	mdk_dp83820_diagLBTest(dp83820, tp,testMode);
	if(sb_conflict) {
		if(isCheetah())	
			tp->result[4].testresult = RET_SKIPPED;
		else
			tp->result[2].testresult = RET_SKIPPED;
	}
	tp->testresult = RET_SKIPPED;
	return(tp->testresult);
}


int32
gmac_5208_phylb_test(
	mdk_test_params_t	* tp
)
{
	uint32 phyData;
	uint32 testMode;
	
	if(isFP())
	{
		return (tp->testresult = RET_SKIP_FP);
	}	

	/* Set the BC 5208 PHYS in loopback mode */
	testMode = TEST_PHY; /* Phys */
	dp83820->phyAddr = 0x3;
	phyData = dp83820_readMiiReg(dp83820, BMCR);
	phyData |= 0x4000;
	dp83820_writeMiiReg(dp83820, BMCR, phyData);

	switch (tp->param[TP_PKT_TRACE]) {
		case 0:
			dp83820_rx_pkt_trace= 0;
			dp83820_tx_pkt_trace= 0;
			break;
		case 1:
			dp83820_rx_pkt_trace = 1;
			dp83820_tx_pkt_trace= 0;
			break;
		case 2:
			dp83820_rx_pkt_trace= 0;
			dp83820_tx_pkt_trace= 1;
			break;
		case 3:
			dp83820_rx_pkt_trace = 1;
			dp83820_tx_pkt_trace= 1;
			break;
		default: 
			printf("Invalid Entry: %d\n", tp->param[TP_PKT_TRACE]);
	}
	/* Run loopback test */
	mdk_dp83820_diagLBTest(dp83820, tp, testMode);
	tp->testresult = RET_SKIPPED;
	return(tp->testresult);
}
#endif	

void
gmac_loopbk_results(
	mdk_test_params_t	* tp
)
{
	mdk_dp83820_test_t 	*dptc;

	dptc = &mdk_dp83820_testControl;
	mdk_dp83820_diagShowLbTestResults(dptc);

	return;
}

int
gmac_set_lbMode(
	mdk_test_params_t	* tp
)
{
	if(isFP())
	{
		return (tp->testresult = RET_SKIP_FP);
	}	

	mdk_dp83820_diagSetLbMode(dp83820, DP83820_LB_ENABLE);
	return(tp->testresult = RET_PASSED);
}

int
gmac_dis_lbMode(
	mdk_test_params_t	* tp
)
{
	if(isFP())
	{
		return (tp->testresult = RET_SKIP_FP);
	}	

	mdk_dp83820_diagSetLbMode(dp83820, DP83820_LB_DISABLE);
	return(tp->testresult = RET_PASSED);
}


#if defined (DIAG_SIBYTE) || defined (DIAG_R7000)
/*
 *------------------------------------------------------------------------------
 * Function Name:	dp83820_diagLBTest
 *
 * Description: 	Runs loopback test 
 *
 * Created by:		Frank Salzmann
 *
 * Data Created:	April 30, 2001
 * 					
 * Returns: ?????
 *
 *------------------------------------------------------------------------------
 */
int
mdk_dp83820_diagLBTest(dp83820_cfg_t *dp,mdk_test_params_t *tp,uint32 testMode)
{
	int32  i, rc;
	int32  pktCnt;
	int32  step;
	int32  pktSize1, pktSize2;
	int32  verify;
	uint32 destAddr = 0;
	uchar8 *bufptr;
    uchar8 *tmp;
	uint32	rct;

	if( mdk_dp83820_testControl.testState == TEST_INPROGRESS ) 
	{
		//printf("Terminating Previous Test:\n");
		if( mdk_dp83820_testControl.testBuffer) 
			eee_ramDealloc(mdk_dp83820_testControl.testBuffer);
	} 
	if(tp->param[TP_LBPKT_CNT])
		pktCnt = tp->param[TP_LBPKT_CNT];
	else
		pktCnt = 1; 
	pktSize1 = 64;
	pktSize2 = 64; 
	step = 1; 
	verify = 1; 

	gmac_test_started = 1; 
	memset(&mdk_dp83820_testControl,0,sizeof(mdk_dp83820_testControl));
#ifdef DIAG_SIBYTE
	if(isCheetah()) {
		memset(&mdk_testState[4],0,sizeof(mdk_loopback_t));
		mdk_testState[4].state = TEST_INPROGRESS;
	}	
	else 
#endif			
	{
		memset(&mdk_testState[2],0,sizeof(mdk_loopback_t));
		mdk_testState[2].state = TEST_INPROGRESS;
	}	
	mdk_dp83820_testControl.testState   = TEST_INPROGRESS; 
	mdk_dp83820_testControl.testMode    = testMode;
	mdk_dp83820_testControl.testPktCnt  = pktCnt;
	mdk_dp83820_testControl.testPktSize = pktSize1;
	mdk_dp83820_testControl.startPktSize = pktSize1;
	mdk_dp83820_testControl.endPktSize = pktSize2;
	mdk_dp83820_testControl.testStep    = step;
	mdk_dp83820_testControl.fVerify	= verify;
	/*
	 * Allocate and initialize test data.
	 */
	bufptr = (uchar8 *)eee_ramAlloc(pktSize2);
	tmp = bufptr;
	if( !bufptr ) {
		printf("Test data buffer allocation failed.\n");
		return(NFX_OK);
	}
	mdk_dp83820_testControl.testBuffer = bufptr;
#ifdef DIAG_SIBYTE	
	rct = eee_timerStart(time_handle_dp83820, sb_lpbk_timeout,0,0,0,0);
	mdk_dp83820_testControl.timer = time_handle_dp83820;
#elif defined (DIAG_R7000)	
	rct = eee_timerStart(time_handle_dp83820, fc_lpbk_timeout,0,0,0,0);
	mdk_dp83820_testControl.timer = time_handle_dp83820;
#endif	

	 //printf("bufptr: %x, pktSize: %d\n", bufptr, pktSize);
	

#if 1
	/* This marks the test packet so that we can call the 
	 * appropriate receive function
	 */
	bufptr[0] = '0';
	bufptr[1] = 'd';
	bufptr[2] = 'p';
	bufptr[3] = '8';
	tmp+=4;
	*(uint32 *)tmp = mdk_dp83820_testControl.txPktCnt; /* mark pkt with seq no */
	
	for(i=8; i < (pktSize2-8); i++ )
		bufptr[i] = i & 0xFF;
#endif


	/*
	 * Send first set of loop back packets.
	 */
#if defined(PROM)
	if(isFC()) {

	rc = eee_registerTransmitFunc( EEE_USER_DATA,
								   dp83820_txUnicast,
								   dp83820_txUnicast,
								   9 * 1024);

	eee_registerDefaultTransmitFunc( EEE_USER_DATA,
									 dp83820_txUnicast,
									 9 * 1024);
	destAddr = PACK_UNICAST_PORT_ADDRESS(
								 nfxMySlotId,
								 nfxMyCPUId,
								 mdk_appId,
								 EEE_USER_DATA,
								 NFX_COS_HIGH, 
								 FCNIM_PORT_BP_DATA);
	}
	else if(isTXRX()) {
	destAddr = PACK_UNICAST_PORT_ADDRESS(
								 nfxMySlotId,
								 nfxMyCPUId,
								 mdk_appId,
								 EEE_MGMT,
								 NFX_COS_HIGH, 
								PROM_PORT_MGMT);
	}
#endif

	mdk_dp83820_testControl.testDestAddr = destAddr;

	for(i=0; i < 1; i++)
		rc = eee_sendMessage(bufptr, pktSize1, destAddr, destAddr);

	dp83820_diagTxEna(dp83820); 

	mdk_dp83820_testControl.txPktCnt += 1;
	mdk_dp83820_testControl.txByteCnt += (1 * pktSize1);

	return(NFX_OK);
} /* mdk_dp83820_diagLBTest */

/*
 *------------------------------------------------------------------------------
 * Function Name:	mdk_dp83820_diagRemakeLBTest
 *
 * Description: 	Remakes the test packet.
 *
 * Created by:		lipika	
 *
 * Data Created:	jul19 20002	
 * 					
 * Returns: ?????
 *
 *------------------------------------------------------------------------------
 */
int
mdk_dp83820_diagRemakeLbPkt()
{
   int32  pktSize = 64;
   uint32 destAddr=0;
   uchar8 *bufptr;
   uchar8 *tmp;
   uint32 rc, i, rct;

	dpprvRxPktCnt = mdk_dp83820_testControl.rxPktCnt;

	if( mdk_dp83820_testControl.testState == TEST_INPROGRESS ) 
	{
		if( mdk_dp83820_testControl.testBuffer) 
			eee_ramDealloc(mdk_dp83820_testControl.testBuffer);
	} 

	pktSize = 64;

	/*
	 * Allocate and initialize test data.
	 */
	bufptr = (uchar8 *)eee_ramAlloc(pktSize);
	if( !bufptr ) {
		printf("Test data buffer allocation failed.\n");
		return(NFX_OK);
	}
	tmp = bufptr;
	mdk_dp83820_testControl.testBuffer = bufptr;

#ifdef DIAG_SIBYTE	
	rct = eee_timerStart(time_handle_dp83820, sb_lpbk_timeout,0,0,0,0);
	mdk_dp83820_testControl.timer = time_handle_dp83820;
#elif defined (DIAG_R7000)	
	rct = eee_timerStart(time_handle_dp83820, fc_lpbk_timeout,0,0,0,0);
	mdk_dp83820_testControl.timer = time_handle_dp83820;
#endif	

	/* This marks the test packet so that we can call the 
	 * appropriate receive function
	 */
	bufptr[0] = '0';
	bufptr[1] = 'd';
	bufptr[2] = 'p';
	bufptr[3] = '8';
	tmp+=4;
	*(uint32 *)tmp= mdk_dp83820_testControl.txPktCnt; /* mark pkt with seq no */
	for(i=8; i < (pktSize-8); i++ )
		bufptr[i] = i & 0xFF;

	destAddr = mdk_dp83820_testControl.testDestAddr;

	for(i=0; i < 1; i++)
		rc = eee_sendMessage(bufptr, pktSize, destAddr, destAddr);

	mdk_dp83820_testControl.txPktCnt += 1;
	mdk_dp83820_testControl.txByteCnt += (1 * pktSize);

	return(NFX_OK);
}

int
mdk_dp83820_diagNewPkt(char *bufptr, uint32 pktSize, uint32 destAddr)
{
	uint32 i;
	
	/* This marks the test packet so that we can call the 
	 * appropriate receive function
	 */
	bufptr[0] = '0';
	bufptr[1] = 'd';
	bufptr[2] = 'p';
	bufptr[3] = '8';
	for(i=4; i < (pktSize-4); i++ )
        	bufptr[i] = i & 0xFF;     

	eee_sendMessage(bufptr, pktSize, destAddr, destAddr);

} // mdk_dp83820_diagNewPkt


int
mdk_dp83820_diagSetLbMode( dp83820_cfg_t *dp, uint32 mode )
{
	uint32 val;
	dp83820_cr_t cr;
	volatile dp83820_regs_t *r;
	dp83820_rxCtrl_t *rxc;

	r = dp->regs;

	/*
	 * First disable Rx and Tx
	 */
	cr.wd = 0;
	cr.bits.rxd = 1;
	cr.bits.txd = 1;
	dp83820_writeReg(&r->cr, cr.wd);
		
	/*
	 * Set/Reset LB Mode
	 */
	val = dp83820_readReg(&r->txcfg); 
	if( mode == DP83820_LB_ENABLE )
		val |= 0x20000000;
	else
		val &= ~0x20000000;
	dp83820_writeReg(&r->txcfg, val);
	
	/*
	 * For some reason the dp83820 increments it's receive pointer when
	 * rx is disabled/enabled, so we need to adjust the CPU's current 
	 * rx pointer as well.
	 */
	rxc = &dp->rxc[0];
	if( ++(rxc->rxCur) >= rxc->rxqEnd )
            rxc->rxCur = rxc->rxqStart;
	/*
	 * Enable Rx and Tx
	 */
	cr.wd = 0;
	cr.bits.rxe = 1;
	dp83820_writeReg(&r->cr, cr.wd);

	return(NFX_OK);
} /* mdk_dp83820_diagSetLbMode */

int
mdk_dp83820_setPromisMode( dp83820_cfg_t *dp)
{
	uint32 val;
	dp83820_cr_t cr;
	volatile dp83820_regs_t *r;
	dp83820_rxCtrl_t *rxc;
	dp83820_rfcr_t rfcr;
	dp83820_regs_t *regs;

	regs = dp->regs;

	/* 
	 * Disable Rx 
	 */
	cr.wd = 0;
	cr.bits.rxd = 1;
	dp83820_writeReg(&regs->cr, cr.wd);
	/*
	 * Setup Receive Filter/Match Register.
	 */
	rfcr.wd = 0;
	rfcr.bits.aab  = 1;			// Allow all broadcast packets
	rfcr.bits.aam  = 1;			// Allow all multicast packets
	rfcr.bits.aau  = 1;			// Allow all unicast packets
	rfcr.bits.apm  = 0;			// Accept on perfect match
	rfcr.bits.apat = 0x00;		// Accept on pattern match
	dp83820_writeReg(&regs->rfcr, rfcr.wd);
	rfcr.bits.rfen = 1;			// Receive enable.
	dp83820_writeReg(&regs->rfcr, rfcr.wd);
	return(NFX_OK);
}

int
mdk_dp83820_diagShowLbTestResults(mdk_dp83820_test_t *t)
{
		char *mdkTestStates[] = {"",
								"TEST_DONE",
								"TEST_INPROGRESS",
								"TEST_VERIFY",
								"TEST_CONTINUE",
								"TEST_END"
		};

	printf("\nDIAG DP83820 LOOPBACK TEST \n");
	printf("======================================\n\n");
	printf("Test State        : %s\n", mdkTestStates[t->testState]);
	printf("Test Packet Cnt   : %d\n", t->testPktCnt);
	printf("Test Packet Size  : %d\n", t->testPktSize);
	printf("Test Verify Flag  : %d\n", t->fVerify);
	printf("Actual Rx Pkt Cnt : %d\n", t->rxPktCnt);
	printf("Actual Rx Byte Cnt: %d\n", t->rxByteCnt);
	printf("Actual Rx Err Cnt : %d\n", t->rxErrPktCnt);
	printf("Actual Tx Pkt Cnt : %d\n", t->txPktCnt);
	printf("Actual Tx Byte Cnt: %d\n", t->txByteCnt);
	printf("\n");
	return(tp->testresult = RET_PASSED);
} /* mdk_dp83820_diagShowLbTestResults */

int
mdk_dp83820_diagLbVerifyData( uchar8 *bufptr, int32 len, uchar8 *tbufptr )
{
	int32  i;

	if( !bufptr || !tbufptr )
		return(NFX_ERR);

	bufptr+=8;
	tbufptr+=8; /* skip over control information */
	for(i=0; i < len-8; i++) {
		if( *bufptr != *tbufptr ) {
			(int)tp->result[0].addr[i] = bufptr;
			(int)tp->result[0].exptd[i] = *tbufptr;
			(int)tp->result[0].rcvd[i] = *bufptr;	
			if (tp->param[TP_VERBOSE] == 1) {
				printf("DP83820: LB Data Error: TestVal=%02x, ActualVal=%02x, Addr=%x\n",
				   *tbufptr, *bufptr, bufptr);
			}	
			tp->testresult = ERR_LPBKDATA;
			return(TEST_END);
		}
		bufptr++;
		tbufptr++;
	}
	return(TEST_CONTINUE);
} /* mdk_dp83820_diagLbVerifyData */

int
mdk_dp83820_diagLbPktReceive( eee_desc_t *edesc )
{
	int32			rc;
	int32 			pktLen;
	int32 			bufCnt;
	mdk_dp83820_test_t 	*dptc;
	static uint32 num=0, n=0;
	uint32 			timeout;
	uchar8	*buf;
	uint32	seqNo;

	dptc = &mdk_dp83820_testControl;
	pktLen = edesc->hdr.control & EDESC_TOTAL_LEN;
	bufCnt = ((edesc->hdr.control & EDESC_BUFCNT_MASK) >> EDESC_BUFCNT_SHIFT);
	buf = edesc->bd[1].buf;
	seqNo = *((uint32 *)buf+1);

#ifdef DIAG_SIBYTE
	if(isCheetah())
		mdk_testState[4].state = dptc->testState;
	else
#endif			
		mdk_testState[2].state = dptc->testState;

	/* Rcvd packet after we've timed out */
#ifdef DIAG_SIBYTE
	if(isCheetah())
			timeout = mdk_testState[4].timeout;
	else
#endif			
			timeout = mdk_testState[2].timeout;

	if(timeout)
	{
#ifdef DIAG_R7000			
		if(!fc_conflict)
#elif defined (DIAG_SIBYTE)				
		if(!sb_conflict)		
#endif				
		{		
			dptc->testState = TEST_END;
		}	
	}

	/* Rcvd pkt even before starting test !! 
	if(!gmac_test_started) {
		printf("!!!!!!! gmac_test_started!!!\n");	
		eee_freePacket(edesc);
		return;
	}
*/
	while(1) {
		switch( dptc->testState )
		{
			case TEST_INPROGRESS:
				dptc->rxPktCnt++; 
				dptc->rxByteCnt += pktLen;

				if( dptc->fVerify )
					dptc->testState = TEST_VERIFY;
				else {
#ifdef DIAG_SIBYTE	
					if(sb_conflict)
#elif defined (DIAG_R7000)	
					if(fc_conflict) 
#endif							
					{
						if((tp->param[TP_LB_DURATION]) > 0) {
							if(!conflict_timeout)
								dptc->testState = TEST_CONTINUE;
							else {
								dptc->testState = TEST_END;
							} 
						}
						else {
							if( dptc->rxPktCnt >= dptc->testPktCnt) {
								dptc->testState = TEST_END;
							}	
							else
								dptc->testState = TEST_CONTINUE;
						}
					}	
					else 
					{		
						if( dptc->rxPktCnt >= dptc->testPktCnt)
							dptc->testState = TEST_END;
						else
							dptc->testState = TEST_CONTINUE;
					}
				}	
				break;

			case TEST_VERIFY:
				rc = mdk_dp83820_diagLbVerifyData(edesc->bd[1].buf, 
											  edesc->bd[1].len, 
											  dptc->testBuffer);
#ifdef DIAG_SIBYTE	
				if(sb_conflict)
#elif defined (DIAG_R7000)	
				if(fc_conflict) 
#endif							
				{
						if((tp->param[TP_LB_DURATION]) > 0) {
#if 0
							num++;
							if(num > n*100) {
								printf("%d\n",num);
								n++;
							}
#endif
							if(!conflict_timeout)
								dptc->testState = rc;
							else {
								dptc->testState = TEST_END;
							}
						}
				}		
				else
				{
						if( dptc->rxPktCnt >= dptc->testPktCnt) {
							dptc->testState = TEST_END;
						}	
						else
							dptc->testState = rc;
				}	
				break;

			case TEST_CONTINUE:
				dptc->txPktCnt++; 
				dptc->txByteCnt += pktLen;

				/* Stop the timer since we've rcvd a pkt,
				 * Restart the timer when we send the next pkt
				 */ 
				eee_timerStop(dptc->timer) ;
				if(dptc->testStep > 0 ) 
				{
					dptc->testPktSize += dptc->testStep;
					if( dptc->testPktSize > dptc->endPktSize)
						dptc->testPktSize = dptc->startPktSize;

					eee_freePacket(edesc);
#ifdef DIAG_SIBYTE	
					eee_timerStart(dptc->timer, sb_lpbk_timeout,0,0,0,0);
#elif defined (DIAG_R7000)	
					eee_timerStart(dptc->timer, fc_lpbk_timeout,0,0,0,0);
#endif					
					rc = mdk_dp83820_diagNewPkt( dptc->testBuffer,
										dptc->testPktSize, 
										dptc->testDestAddr);
				}
				else
			   	{
					edesc->hdr.dest_port = dptc->testDestAddr;
#ifdef DIAG_SIBYTE	
					eee_timerStart(dptc->timer, sb_lpbk_timeout,0,0,0,0);
#elif defined (DIAG_R7000)	
					eee_timerStart(dptc->timer, fc_lpbk_timeout,0,0,0,0);
#endif					
					rc = eee_forwardPacket(edesc);
					if(rc != EEE_FORWARDED_PACKET)
						tp->testresult = ERR_PKT_FWD;
				}
				dptc->testState = TEST_INPROGRESS;
				return(NFX_OK);
				
			case TEST_END:
				if(dptc->testBuffer)
					eee_ramDealloc(dptc->testBuffer);

				if(dptc->testMode == TEST_INT)
					mdk_dp83820_diagSetLbMode(dp83820, DP83820_LB_DISABLE);

#ifdef DIAG_SIBYTE	
				if(isTXRX()) {
					uint32 phyData;

					if(dptc->testMode == TEST_PHY) {
						dp83820->phyAddr = 0x3;
						phyData = dp83820_readMiiReg(dp83820, BMCR);
						phyData &= ~0x4000;
						dp83820_writeMiiReg(dp83820, BMCR, phyData);
					}
				}	
#endif						
				dptc->testState = TEST_DONE;
				if(!timeout)
						eee_timerStop(dptc->timer) ;
				break;

			case TEST_DONE:
				if(tp->testresult != ERR_LPBKDATA) {
					if(timeout) 
						tp->testresult = ERR_TIMEOUT;
					else
						tp->testresult = RET_PASSED;
				}
#ifdef DIAG_SIBYTE	
				if(sb_conflict)
#elif defined (DIAG_R7000)				
				if(fc_conflict)
#endif						
				{		
#ifdef DIAG_SIBYTE	
					if(isCheetah())	
						tp->result[4].testresult = tp->testresult;
					else
#endif							
						tp->result[2].testresult = tp->testresult;
				}	
#ifdef DIAG_SIBYTE	
				if(isCheetah())
					mdk_testState[4].state = TEST_DONE;
				else
#endif						
					mdk_testState[2].state = TEST_DONE;
				test_done = 1;
				gmac_test_started = 0;
				eee_freePacket(edesc);
				if (tp->param[TP_VERBOSE] == 1) 
					mdk_dp83820_diagShowLbTestResults(dptc);
				return(NFX_OK);
		}		
	}
	return(NFX_OK);
} /* mdk_dp83820_diagLbPktReceive */
#endif

/* National GMAC tests on FCNIM board */

#ifdef DIAG_R7000
int32
gt_gmac_phylb(
	mdk_test_params_t	* tp
)
{
	uint32 phyData;
	uint32 testMode;
	uint32 *reg_addr;
	uint32 phy_id;

	//mdk_dp83820_setPromisMode(dp83820);
	if((isFC()) && (nfxMyCPUId == 0)) {
		reg_addr = 0x0; /* Control register */
		phyData = 0x5140; /* bit 14 control register */
		gt_mii_writeSMI(0x18, reg_addr, phyData);
	}

	switch (tp->param[TP_PKT_TRACE]) {
		case 0:
			dp83820_rx_pkt_trace= 0;
			dp83820_tx_pkt_trace= 0;
			break;
		case 1:
			dp83820_rx_pkt_trace = 1;
			dp83820_tx_pkt_trace= 0;
			break;
		case 2:
			dp83820_rx_pkt_trace= 0;
			dp83820_tx_pkt_trace= 1;
			break;
		case 3:
			dp83820_rx_pkt_trace = 1;
			dp83820_tx_pkt_trace= 1;
			break;
		default: 
			printf("Invalid Entry: %d\n", tp->param[TP_PKT_TRACE]);
	}
	testMode = TEST_PHY; /* Phys */
	mdk_dp83820_diagLBTest(dp83820, tp, testMode);
	tp->testresult = RET_SKIPPED;
	return(tp->testresult);
}

int32
gmac_set_all_phylb(
	mdk_test_params_t	* tp
)
{
	uint32 phyData, tmp;
	uint32 testMode;
	uint32 *reg_addr;
	uint32 phy_id;
	if((isFC()) && (nfxMyCPUId == 0)) {
		reg_addr = 0x0; /* Control register */
		phyData = 0x5140; /* bit 14 control register */
		gt_mii_writeSMI(0x18, reg_addr, phyData);
		delay_us(1000);
		gt_mii_writeSMI(0x19, reg_addr, phyData);
		delay_us(1000);
		gt_mii_writeSMI(0x1A, reg_addr, phyData);
		delay_us(1000);
		gt_mii_writeSMI(0x1B, reg_addr, phyData);
		delay_us(1000);

		if (tp->param[TP_VERBOSE] == 1) {
			gt_mii_readSMI(0x18, reg_addr, &tmp); 
			printf("0x18 %04x\n",tmp);
			gt_mii_readSMI(0x19, reg_addr, &tmp); 
			printf("0x19 %04x\n",tmp);
			gt_mii_readSMI(0x1a, reg_addr, &tmp); 
			printf("0x1a %04x\n",tmp);
			gt_mii_readSMI(0x1b, reg_addr, &tmp); 
			printf("0x1b %04x\n",tmp);
		}
	tp->testresult = RET_PASSED;
	}		
	else 
		tp->testresult = RET_SKIP_TEST;

	return(tp->testresult);
}

int32
gt_gmac_extlb_test(
	mdk_test_params_t	* tp
)
{
	uint32 testMode;

	testMode = TEST_EXT; /* 2 */ 
	//mdk_dp83820_setPromisMode(dp83820);
	switch (tp->param[TP_PKT_TRACE]) {
		case 0:
			dp83820_rx_pkt_trace= 0;
			dp83820_tx_pkt_trace= 0;
			break;
		case 1:
			dp83820_rx_pkt_trace = 1;
			dp83820_tx_pkt_trace= 0;
			break;
		case 2:
			dp83820_rx_pkt_trace= 0;
			dp83820_tx_pkt_trace= 1;
			break;
		case 3:
			dp83820_rx_pkt_trace = 1;
			dp83820_tx_pkt_trace= 1;
			break;
		default: 
			printf("Invalid Entry: %d\n", tp->param[TP_PKT_TRACE]);
	}
	mdk_dp83820_diagLBTest(dp83820, tp,testMode);
	if(fc_conflict)
		tp->result[2].testresult = RET_SKIPPED;
	tp->testresult = RET_SKIPPED;
	return(tp->testresult);
}

/*
 *------------------------------------------------------------------------------
 * Function Name: 	gmac_reset_all_phylb
 *
 * Description: 	Turns off the loopback mode of the driver	
 *
 * Created by:		lipika	
 *
 * Data Created:	Mar 29 2002	
 * 					
 * Returns: ?????
 *
 *------------------------------------------------------------------------------
 */
void
gmac_reset_all_phylb(
	mdk_test_params_t	* tp
)	
{		
   /* Need to reset the loopback mode */
	uint32 phyData;
	uint32 testMode;
	uint32 *reg_addr;
	uint32 phy_id;
	if((isFC()) && (nfxMyCPUId == 0)) {
		reg_addr = 0x0; /* Control register */
		phyData = 0x1140; /* bit 14 control register */
		gt_mii_writeSMI(0x18, reg_addr, phyData);
		gt_mii_writeSMI(0x19, reg_addr, phyData);
		gt_mii_writeSMI(0x1A, reg_addr, phyData);
		gt_mii_writeSMI(0x1B, reg_addr, phyData);
		tp->testresult = RET_PASSED;
	}		
	else
		tp->testresult = RET_SKIP_TEST;

	return (tp->testresult);
}

int32
fc_lpbk_timeout(int32 tmr_handle,
		 void * usr_param0,
		 void * usr_param1,
		 void * usr_param2,
		 void * usr_param3)
{
	uint32 testresult;
	mdk_dp83820_test_t 	*dptc;

	dptc = &mdk_dp83820_testControl;
	gmac_test_started = 0;

#if 0	
   	if ((fc_conflict) && (dptc->rxPktCnt > 0) && ((dptc->txPktCnt - dptc->rxPktCnt) == 1)) 
	{		
		/* Test Passed if we've received some packets.
		 * some pkts will be trasmitted  but not rcvd 
		 * because we've stopped the test b4 receiving them
		 */ 
		//if(fc_conflict) {
				tp->result[2].testresult = RET_PASSED;
		//}
		tp->testresult = RET_PASSED;	
	}	
	else {
#endif			
	//}		
	/* force the state to done because time's up */
	if(tmr_handle == time_handle_dp83820) {
		//printf("dp timeout");
		if(!fc_conflict) {
			if(dptc->txPktCnt <= dptc->testPktCnt) {
				tp->testresult = ERR_TIMEOUT;
				if(dpprvRxPktCnt == dptc->rxPktCnt) {
					dpconsecutivePktLoss++;	
				}	
				else
					dpconsecutivePktLoss = 0;	
				if(dpconsecutivePktLoss > 20)
				{
					/* If we drop 50 consecutive packets,
				 	* declare test as failed
				 	*/ 	
				}		
				else {
					mdk_dp83820_diagRemakeLbPkt();
					return(NFX_OK);
				}
			}	
		}	
		else if(!conflict_timeout) {
			tp->testresult = ERR_TIMEOUT;
			tp->result[2].testresult = tp->testresult;
			mdk_dp83820_diagRemakeLbPkt();
			return(NFX_OK);
		}		
		mdk_testState[2].timeout = 1;

		if(!fc_conflict) {
			mdk_testState[2].state = TEST_DONE;
			test_done = 1;
		}
		else if(conflict_timeout) {
			mdk_testState[2].state = TEST_DONE;
			test_done = 1;
		}
		
		tp->testresult = ERR_TIMEOUT;
		if(fc_conflict) {
				tp->result[2].testresult = tp->testresult;
		}
	}		
	//if(!fc_conflict)
	//	mdk_show_results(tp->testresult,0);
	return(NFX_OK);
}
	
#endif	


int32
gmac_pkg_err(
	mdk_log_entry_t	* err
)
{

	switch (err->testresult) {
		case ERR_TIMEOUT:
			if (tp->param[TP_VERBOSE] == 1) {
				printf("Loopback test timeout\n");
			}	
				break;
		case ERR_PKT_FWD:
			if (tp->param[TP_VERBOSE] == 1) {
				printf("Error during packet FWD\n");
			}	
				break;
		case ERR_PKTDROP:
			if (tp->param[TP_VERBOSE] == 1) 
				printf("\nSome pkts have been dropped.\n");
			break;
		case ERR_LPBKDATA:
			if (tp->param[TP_VERBOSE] == 1) 
				printf("\nSome pkts have been dropped.\n");
			break;
		default:
				//printf("Unknown error, testresult = 0x%x\n", err->testresult);
				break;
	}

	return 0;
}


#endif
