/* vi:ts=4
 *-----------------------------------------------------------------
 *
 * Name:
 *
 * RCS:  $Id: mdk-gt.c,v 1.5 2002/07/27 00:54:45 lipikaathome Exp $
 *
 * Copyright (C) 2001, Agile Storage, Inc.
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	1/29/02
 *
 *-----------------------------------------------------------------
 */

#include <nfx-incl.h>
#include <exports/mdk-api.h>
#if defined (DIAG_R7000)
#include <exports/hwMap.h>
#include "mdk-mem.h"
#include <exports/i2c-api.h>
#include <exports/gt64240mii.h>
//#include "../../sm-bcm5600/bcm5600-ll.h"
#include <exports/timer-api.h>

extern uint32 nfxMyCPUId;
reginfo_t fpga_reg[] = {
	//{"r",0x18, 0x0f00e1d9,0,0}, 	
	//{"r",0x1C, 0x0026, 0,0},
	{"w",0x1e0,0xf0f0f0f0,0xf0f0f0f0,0}, //turned off for now
	{NULL, NULL, NULL,NULL,NULL},
};
#define GT_FPGA_REG_MAX		1

reginfo_t mdio_8353[] = {
	{"r",0x2, 0x00000027,0,0}, 	
	{"r",0x3, 0x0000a831, 0,0},
	{"w",0x0, 0x0, 0x00005140,0},
	{NULL, NULL, NULL,NULL,NULL},
};
#define MDIO_8353_REG_MAX	3

reginfo_t mdio_5228[] = {
	{"r",0x2, 0x00000040,0,0}, 	
	{"r",0x3, 0x000061d4, 0,0},
	{"w",0x0, 0x0, 0x00006100,0},
	{NULL, NULL, NULL,NULL,NULL},
};
#define MDIO_5228_REG_MAX	3
sb_smbus_dev_t fc_dev_0[] = {
	{"BCM 5600",0x88}, 
	{"Board Temp",0x92}, 
	{"Board SEEP",0xa6},
	{NULL,NULL},
};			 
#define I2C_DEV_SLICE0	3

sb_smbus_dev_t fc_dev[] = {
	{"DRAM",0xac}, 
	//{"Slice SEEP",0xae}, /* Only for beta boards */
	/*{"SFP OPTIC",0xa2}, */
	{NULL,NULL},
};			 
#define FC_I2C_DEV_MAX		1	/* For beta boards */ 

sb_smbus_dev_t ssc_dev[] = {
	{"DRAM",0xac}, 
	{"Board SEEP",0xa8}, 
	{"Temp Sensor",0xaa},
	{"Toshiba Memory",0xae},
	{NULL,NULL},
};			 
#define SSC_I2C_DEV_MAX		4	/* For beta boards */ 

/* Global definations */
extern uint32 mdk_gt_mii_test_port;
extern gt64240mii_cfg_t *gt64240mii;
uint32 mac_lpbk, phy_5228_lpbk, phy_8353_lpbk;
uint32 set_phy_5228_lpbk;
extern uint32 port0_added_hash;
uint32 testMode;
extern uint32 fc_conflict;
static uint32 createdGTtimer;	

/* Max ports for TXRX: 4 + 1
 * Max ports for FP : 6
 * Max ports for FC : 4
 */ 
mdk_loopback_t mdk_testState[6];

uint32	time_handle_dp83820;
uint32	time_handle_port0;
uint32	time_handle_port1;
uint32	time_handle_port2;
uint32	time_handle_port3;
uint32	time_handle_port4;
uint32	time_handle_port5;

void
gt_pkg_init()
{
	tp->resultCnt = 1;
	tp->param[TP_LBPKT_CNT] = 10000;
#ifdef DIAG_R7000
	if(!time_handle_port0) 
		time_handle_port0 = eee_timerCreate(MDK_INTERPKTDELAY, EEE_TIMER_ONCE);
	if(!time_handle_port1) 
		time_handle_port1 = eee_timerCreate(MDK_INTERPKTDELAY, EEE_TIMER_ONCE);
	if(!time_handle_port2) 
		time_handle_port2 = eee_timerCreate(MDK_INTERPKTDELAY, EEE_TIMER_ONCE);
#endif	
}

int32
gt_reg_test(
	mdk_test_params_t	* tp
)
{
	tp->testresult = RET_PASSED;
	return (tp->testresult);
}

int32
gt_mdio_bc5228(
	mdk_test_params_t	* tp
)
{
    uint32 *reg_addr;
	uint32 phy_id;
	uint32 rdata, sdata, odata;
	uint32 i,j;
	uchar8 * test;

	if(isFC() && (nfxMyCPUId != 0)) {
			tp->testresult = RET_SKIP_TEST;
			return (tp->testresult);
	}
	else {
			test = "r";
			phy_id = 0x10;
			if (tp->param[TP_VERBOSE] == 1) {
					printf("\nBCM 5228:  Read/Write through MDIO\n");
					printf("=========================================================\n");
					printf("Register	Result	Expected	Received\n");
			}
		
			for(i=0;i<MDIO_5228_REG_MAX;i++)
			{
				if(!(strcmp((mdio_5228[i].rw),test))) {
					gt_mii_readSMI(phy_id, mdio_5228[i].reg, &rdata); 
					if((rdata & 0x0000ffff) == mdio_5228[i].expected)
						mdio_5228[i].res = 1;
					else {	
						mdio_5228[i].res = 0;	
						tp->result[0].addr[i] = mdio_5228[i].reg;
						tp->result[0].exptd[i] = mdio_5228[i].expected ;
						tp->result[0].rcvd[i] = rdata;	
						tp->testresult = ERR_READ_FAIL;
						return ;
					}
					if (tp->param[TP_VERBOSE] == 1) {
						printf("%08x	%s	%08x	%08x\n",mdio_5228[i].reg, 
								(mdio_5228[i].res == 1)? "PASS":"FAIL" , 
								mdio_5228[i].expected,rdata);}
				}
				else {
					gt_mii_readSMI(phy_id, mdio_5228[i].reg, &sdata); 
					odata = sdata;
					sdata |= mdio_5228[i].wbits;
					gt_mii_writeSMI(phy_id, mdio_5228[i].reg, sdata);
					gt_mii_readSMI(phy_id, mdio_5228[i].reg, &rdata); 
					if(rdata  == sdata) {
						mdio_5228[i].res = 1;
						/* restore orig data */
						gt_mii_writeSMI(phy_id, mdio_5228[i].reg, odata);
					}	
					else {	
						mdio_5228[i].res = 0;	
						tp->result[0].addr[i] = mdio_5228[i].reg;
						mdio_5228[i].expected = sdata;;
						tp->result[0].exptd[i] = mdio_5228[i].expected ;
						tp->result[0].rcvd[i] = rdata;	
						tp->testresult = ERR_WRITE_FAIL;
						return ;
					}	
					if (tp->param[TP_VERBOSE] == 1) {
						printf("%08x	%s	%08x	%08x\n",mdio_5228[i].reg, 
								(mdio_5228[i].res == 1)? "PASS":"FAIL" , 
								sdata,rdata);}
				}	
			}		
	}		
	tp->testresult = RET_PASSED;
	return (tp->testresult);
}

int32
gt_mdio_8353_1(
	mdk_test_params_t	* tp
)
{
    uint32 *reg_addr;
	uint32 phy_id;
	uint32 rdata, sdata, odata;
	uint32 i,j;
	uchar8 * test;

	if(isFC() && (nfxMyCPUId != 0)) {
			tp->testresult = RET_SKIP_TEST;
			return (tp->testresult);
	}
	else {
			test = "r";
			phy_id = 0x0a;

			if (tp->param[TP_VERBOSE] == 1) {
					printf("\nPMC 8353 :  Read/Write through MDIO(phy id %08x)\n",phy_id);
					printf("=========================================================\n");
					printf("Register	Result	Expected	Received\n");
			}
			for(i=0;i<MDIO_8353_REG_MAX;i++)
			{
				if(!(strcmp((mdio_8353[i].rw),test))) {
					gt_mii_readSMI(phy_id, mdio_8353[i].reg, &rdata); 
					if((rdata & 0x0000ffff) == mdio_8353[i].expected)
						mdio_8353[i].res = 1;
					else {	
						mdio_8353[i].res = 0;	
						tp->result[0].addr[i] = mdio_8353[i].reg;
						tp->result[0].exptd[i] = mdio_8353[i].expected ;
						tp->result[0].rcvd[i] = rdata;	
						tp->testresult = ERR_READ_FAIL;
						return ;
					}
					if (tp->param[TP_VERBOSE] == 1) {
						printf("%08x	%s	%08x	%08x\n",mdio_8353[i].reg, 
								(mdio_8353[i].res == 1)? "PASS":"FAIL" , 
								mdio_8353[i].expected,rdata);}
				}
				else {
					gt_mii_readSMI(phy_id, mdio_8353[i].reg, &sdata); 
					odata = sdata;
					sdata |= mdio_8353[i].wbits;
					gt_mii_writeSMI(phy_id, mdio_8353[i].reg, sdata);
					gt_mii_readSMI(phy_id, mdio_8353[i].reg, &rdata); 
					if(rdata  == sdata) {
					mdio_8353[i].res = 1;
						/* restore orig data */
						gt_mii_writeSMI(phy_id, mdio_8353[i].reg, odata);
					}	
					else {	
						mdio_8353[i].res = 0;	
						tp->result[0].addr[i] = mdio_8353[i].reg;
						mdio_8353[i].expected = sdata;;
						tp->result[0].exptd[i] = mdio_8353[i].expected ;
						tp->result[0].rcvd[i] = rdata;	
						tp->testresult = ERR_WRITE_FAIL;
						return ;
					}	
					if (tp->param[TP_VERBOSE] == 1) {
						printf("%08x	%s	%08x	%08x\n",mdio_8353[i].reg, 
								(mdio_8353[i].res == 1)? "PASS":"FAIL" , 
								sdata,rdata);}
				}	
			}
	}		
	tp->testresult = RET_PASSED;
	return (tp->testresult);
}


int32
gt_mdio_8353_2(
	mdk_test_params_t	* tp
)
{
    uint32 *reg_addr;
	uint32 phy_id;
	uint32 rdata, sdata, odata;
	uint32 i,j;
	uchar8 * test;

	if(isFC() && (nfxMyCPUId != 0)) {
			tp->testresult = RET_SKIP_TEST;
			return (tp->testresult);
	}
	else {
			test = "r";
			phy_id = 0x18;

			if (tp->param[TP_VERBOSE] == 1) {
					printf("\nPMC 8353 :  Read/Write through MDIO(phy id %08x)\n",phy_id);
					printf("=========================================================\n");
					printf("Register	Result	Expected	Received\n");
			}
			for(i=0;i<MDIO_8353_REG_MAX;i++)
			{
				if(!(strcmp((mdio_8353[i].rw),test))) {
					gt_mii_readSMI(phy_id, mdio_8353[i].reg, &rdata); 
					if((rdata & 0x0000ffff) == mdio_8353[i].expected)
						mdio_8353[i].res = 1;
					else {	
						mdio_8353[i].res = 0;	
						tp->result[j].addr[i] = mdio_8353[i].reg;
						tp->result[j].exptd[i] = mdio_8353[i].expected ;
						tp->result[j].rcvd[i] = rdata;	
						tp->testresult = ERR_READ_FAIL;
						return ;
					}
					if (tp->param[TP_VERBOSE] == 1) {
						printf("%08x	%s	%08x	%08x\n",mdio_8353[i].reg, 
								(mdio_8353[i].res == 1)? "PASS":"FAIL" , 
								mdio_8353[i].expected,rdata);}
					}
					else {
						gt_mii_readSMI(phy_id, mdio_8353[i].reg, &sdata); 
						odata = sdata;
						sdata |= mdio_8353[i].wbits;
						gt_mii_writeSMI(phy_id, mdio_8353[i].reg, sdata);
						gt_mii_readSMI(phy_id, mdio_8353[i].reg, &rdata); 
						if(rdata  == sdata) {
							mdio_8353[i].res = 1;
							/* restore orig data */
							gt_mii_writeSMI(phy_id, mdio_8353[i].reg, odata);
						}	
						else {	
							mdio_8353[i].res = 0;	
							tp->result[j].addr[i] = mdio_8353[i].reg;
							mdio_8353[i].expected = sdata;;
							tp->result[j].exptd[i] = mdio_8353[i].expected ;
							tp->result[j].rcvd[i] = rdata;	
							tp->testresult = ERR_WRITE_FAIL;
							return ;
						}	
						if (tp->param[TP_VERBOSE] == 1) {
							printf("%08x	%s	%08x	%08x\n",mdio_8353[i].reg, 
									(mdio_8353[i].res == 1)? "PASS":"FAIL" , 
									sdata,rdata);}
					}	
				}
	}
	tp->testresult = RET_PASSED;
	return (tp->testresult);
}

int32
gt_i2c_test(
	mdk_test_params_t	* tp
)
{
	int32 rc;
	uchar8   buf[4];
	uchar8   i2c_addr, byte_addr;
	uint32    i, j=0,len;

#ifdef DIAG_R7000
	if(isSSC()) {
	}		
	else if(isFC()) {
		/* Only for Slice 0 */	
		if(nfxMyCPUId == 0) {	
				for (i=0; i<I2C_DEV_SLICE0; i++)
				{
					i2c_addr = fc_dev_0[i].dev_addr; /* addr of device */
					byte_addr = 0x0 ; /* offset within dev reg */
					len = 1;
					rc = i2c(I2C_READ, i2c_addr, byte_addr, len, buf);
					if (rc < 0) {
						tp->result[0].addr[j] = i2c_addr;
						tp->result[0].exptd[j] = 0;
						tp->result[0].rcvd[j] = 0;	
						if (tp->param[TP_VERBOSE] == 1) {
							printf("Error in I2C_READ at dev %s %02x, rc = 0x%x\n", fc_dev_0[i].dev_str,fc_dev_0[i].dev_addr, rc);
						}	
						j++;
						tp->testresult = ERR_I2CADDR_FAIL;
						//return (tp->testresult);
					}
					else
						if (tp->param[TP_VERBOSE] == 1) {
							printf("Dev %s at %04x passed.\n", fc_dev_0[i].dev_str, fc_dev_0[i].dev_addr);		
						}	
				}
		}
		/* DRAM & SFP for all slices */
		for (i=0; i<FC_I2C_DEV_MAX; i++)
		{
			i2c_addr = fc_dev[i].dev_addr; /* addr of device */
			byte_addr = 0x0 ; /* offset within dev reg */
			len = 1;
			rc = i2c(I2C_READ, i2c_addr, byte_addr, len, buf);
			if (rc < 0) {
				tp->result[0].addr[j] = i2c_addr;
				tp->result[0].exptd[j] = 0;
				tp->result[0].rcvd[j] = 0;	
					if (tp->param[TP_VERBOSE] == 1) {
					printf("Error in I2C_READ at dev %s %02x, rc = 0x%x\n", fc_dev[i].dev_str,fc_dev[i].dev_addr, rc);
					}	
				tp->testresult = ERR_I2CADDR_FAIL;
				j++;
				//return (tp->testresult);
			}
			else
				if (tp->param[TP_VERBOSE] == 1) {
					printf("Dev %s at %04x passed.\n", fc_dev[i].dev_str, fc_dev[i].dev_addr);		
				}	
		}
	}		
#endif	
	if(tp->testresult != ERR_I2CADDR_FAIL)
		tp->testresult = RET_PASSED;

	return (tp->testresult);
}

int32	
verify_temp(uchar8 add)
{
	char8 board_temp;
	uchar8   buf[4];
	int32 rc;
	int32 result;

	/* For DALLAS chip*/
	
	i2c(I2C_READ, add, START_CONV, 1, buf);
	delay_us(300000);
	rc = i2c(I2C_READ, add, READ_TEMP, 1, buf);
	if (rc < 0)  {
		if (tp->param[TP_VERBOSE] == 1) 
			printf("Error in I2C_READ, rc = 0x%x\n",rc);
	}
	board_temp = buf[0]; /* since len = 1 */
	if((board_temp < 5) || (board_temp > 45))
	{
		result = ERR_UNEXPECTED_RESULT;
		if (tp->param[TP_VERBOSE] == 1) 
			printf("Board Temp %04x FAILED\n",board_temp);
	}
	else{
		result = RET_PASSED;
		if (tp->param[TP_VERBOSE] == 1) 
			printf("Board Temp %d PASSED\n",board_temp);
	}
	return(result);
}


int32
gt_temp_test(
	mdk_test_params_t	* tp
)
{
	uint32 rc;
#ifdef DIAG_R7000
	/* Get temperature for each temp device on board */
	if(isSSC()) {
		rc = verify_temp(0xaa); //ssc_dev
	}		
	else if(isFC()) {
		if(nfxMyCPUId == 0)
			rc = verify_temp(0x92);
		else
			rc = tp->testresult = RET_SKIP_TEST;
	}	
#endif	
	tp->testresult = rc;
	return(tp->testresult);
}	

int32
gt_fpga_test(
	mdk_test_params_t	* tp
)
{
	uint32 i, j=0;
	uint32 sval, rval;
	uint32 regAddr;
	uchar8 * test;
	
	test = "r";

	if (tp->param[TP_VERBOSE] == 1) {
			printf("\n");
	}		
	if((isFC()) && (nfxMyCPUId))
		return(tp->testresult = RET_SKIP_TEST);
 
	if((isFC()) && (nfxMyCPUId == 0)) {
		for (i=0; i < GT_FPGA_REG_MAX;i++){
			regAddr = fpga_reg[i].reg;
			if(!(strcmp((fpga_reg[i].rw),test))) {
				rval = *(volatile unsigned int *)PHYS_TO_K1(HW_BMFPGA_BASE + regAddr);
				delay_us(1000);
				if(fpga_reg[i].expected == rval) {
					fpga_reg[i].res = 1;			
				}
				else {
					fpga_reg[i].res = 0;
					tp->result[0].addr[j] = regAddr;
					tp->result[0].exptd[j] = fpga_reg[i].expected;
					tp->result[0].rcvd[j] = rval;	
					tp->testresult = ERR_DATA_FAIL;
					j++;
					//return (tp->testresult);
				}	
			}
			else /* write */
			{	
					/* HAVE NOT TESTED THIS YET !!! */
				sval = *(volatile unsigned int *)PHYS_TO_K1(HW_BMFPGA_BASE + regAddr);
				delay_us(1000);
				*(volatile unsigned int *)PHYS_TO_K1(
						HW_BMFPGA_BASE + regAddr)= fpga_reg[i].wbits ;
				rval = *(volatile unsigned int *)PHYS_TO_K1(HW_BMFPGA_BASE + regAddr);
				delay_us(1000);
				if(rval == fpga_reg[i].expected)
					fpga_reg[i].res = 1;
				else {
					fpga_reg[i].res = 0;
					tp->result[0].addr[j] = regAddr;
					tp->result[0].exptd[j] = fpga_reg[i].expected;
					tp->result[0].rcvd[j] = rval;	
					tp->testresult = ERR_DATA_FAIL;
					//return(tp->testresult);
					j++;
				}	
				/* reprogram orig value */
				*(volatile unsigned int *)PHYS_TO_K1(
						HW_BMFPGA_BASE + regAddr)= sval;
				delay_us(1000);
			}	
			if (tp->param[TP_VERBOSE] == 1) {
			printf("reg:%08x	%s	exptd:%08x	rcvd:%08x\n",fpga_reg[i].reg, 
							(fpga_reg[i].res == 1)? "PASS":"FAIL", 
							fpga_reg[i].expected,rval);}
		}
	}	
	if(tp->testresult != ERR_DATA_FAIL)
		tp->testresult = RET_PASSED;
	return (tp->testresult);
}

int32
gt_int_test(
	mdk_test_params_t	* tp
)
{
	tp->testresult = RET_PASSED;
	return (tp->testresult);
}

/* LOOPBACK TESTS */

int32
gt_5228_phy_lb(
	mdk_test_params_t	* tp
)
{
    uint32 *reg_addr;
	uint32 phy_id = 0;
	uint32 port = 0;
	uint32 data, tmp;
	uint32 msg, cpu;

	if(isFC()) {
			/* Able to set port into loopback from slice 0 only */
		if(nfxMyCPUId == 0) {
			mdk_gt_mii_test_port = port;

			reg_addr = 0x0; /* Control register */
			data = 0x6100; /* bit 14 control register */
			gt_mii_writeSMI(0x10, reg_addr, data);
			delay_us(10000);
#if 0
			/* do this from the "set" fn */
			gt_mii_writeSMI(0x11, reg_addr, data);
			delay_us(10000);
			gt_mii_writeSMI(0x12, reg_addr, data);
			delay_us(10000);
			gt_mii_writeSMI(0x13, reg_addr, data);
			delay_us(10000);
			/* msg = MDK_FCNIM_SETLPBK; */
			/* gt_mii_readSMI(phy_id, reg_addr, &tmp); */
#endif
		}
		switch (tp->param[TP_PKT_TRACE]) {
			case 0:
				gt64240mii->port[port].rx_trace = 0;
				gt64240mii->port[port].tx_trace = 0;
				break;
			case 1:
				gt64240mii->port[port].rx_trace = 1;
				gt64240mii->port[port].tx_trace = 0;
				break;
			case 2:
				gt64240mii->port[port].rx_trace = 0;
				gt64240mii->port[port].tx_trace = 1;
				break;
			case 3:
				gt64240mii->port[port].rx_trace = 1;
				gt64240mii->port[port].tx_trace = 1;
				break;
			default: 
				printf("Invalid Entry: %d\n", tp->param[TP_PKT_TRACE]);
		}			
		port = 1; /* for Management Plane */
		phy_5228_lpbk = 1;
	}
	testMode = GT_LB_TEST_PHY;
	mdk_gt_mii_diagLBTest(port, testMode);
	tp->testresult = RET_SKIPPED;
	return(tp->testresult);
}	

int32
gt_5228_reset_lpbk(
	mdk_test_params_t	* tp
)
{
	if((isFC()) && (nfxMyCPUId == 0)) {
		gt_reset_loopback(0);
		tp->testresult = RET_PASSED;
	}
	else
		tp->testresult =  RET_SKIP_TEST;	

	set_phy_5228_lpbk = 0;
	return(tp->testresult);
}

int32
gt_5228_set_lpbk(
	mdk_test_params_t	* tp
)
{
    uint32 *reg_addr;
	uint32 data, msg;

		/* Able to set port into loopback from slice 0 only */
	if((isFC()) && (nfxMyCPUId == 0)) {
			reg_addr = 0x0; /* Control register */
			data = 0x6100; /* bit 14 control register */
			gt_mii_writeSMI(0x10, reg_addr, data);
			delay_us(10000);
			gt_mii_writeSMI(0x11, reg_addr, data);
			delay_us(10000);
			gt_mii_writeSMI(0x12, reg_addr, data);
			delay_us(10000);
			gt_mii_writeSMI(0x13, reg_addr, data);
			delay_us(10000);
		tp->testresult = RET_PASSED;
	}
	else
		tp->testresult =  RET_SKIP_TEST;	

	set_phy_5228_lpbk = 1;
	return(tp->testresult);
}

int32
gt_5228_ext_lb(
	mdk_test_params_t	* tp
)
{
	uint32 port;

	port = 1; /* for Management Plane */
	switch (tp->param[TP_PKT_TRACE]) {
		case 0:
			gt64240mii->port[port].rx_trace = 0;
			gt64240mii->port[port].tx_trace = 0;
			break;
		case 1:
			gt64240mii->port[port].rx_trace = 1;
			gt64240mii->port[port].tx_trace = 0;
			break;
		case 2:
			gt64240mii->port[port].rx_trace = 0;
			gt64240mii->port[port].tx_trace = 1;
			break;
		case 3:
			gt64240mii->port[port].rx_trace = 1;
			gt64240mii->port[port].tx_trace = 1;
			break;
		default: 
			printf("Invalid Entry: %d\n", tp->param[TP_PKT_TRACE]);
	}			
	mdk_gt_mii_test_port = port;
	testMode = GT_LB_TEST_EXT;
	mdk_gt_mii_diagLBTest(port, testMode);
	if(fc_conflict)
		tp->result[port].testresult = RET_SKIPPED;

	tp->testresult = RET_SKIPPED;

	return(tp->testresult);
}	

/* Loopbacks to MAC and 8353 PHY */
int32
gt_5600_ext_lb0(
	mdk_test_params_t	* tp
)
{
	uint32 port;
    uint32 mac1, mac2;

	//if((isFC()) && (nfxMyCPUId))
	//	return(tp->testresult = RET_SKIP_TEST);

	mdk_gt_mii_test_port = port = 0;

	if(!port0_added_hash) {
			mac1 = 0x0808;
			if(nfxMyCPUId == 0)
				mac2 = 0x02000100; 
			if(nfxMyCPUId == 1)
				mac2 = 0x02010100; 
			if(nfxMyCPUId == 2)
				mac2 = 0x02020100; 
			if(nfxMyCPUId == 3)
				mac2 = 0x02030100; 
			gt64240_addHashTableEntry(port, mac1, mac2, 1, 0);
			delay_us(1000);
			port0_added_hash = 1;
	}	
	switch (tp->param[TP_PKT_TRACE]) {
		case 0:
			gt64240mii->port[port].rx_trace = 0;
			gt64240mii->port[port].tx_trace = 0;
			break;
		case 1:
			gt64240mii->port[port].rx_trace = 1;
			gt64240mii->port[port].tx_trace = 0;
			break;
		case 2:
			gt64240mii->port[port].rx_trace = 0;
			gt64240mii->port[port].tx_trace = 1;
			break;
		case 3:
			gt64240mii->port[port].rx_trace = 1;
			gt64240mii->port[port].tx_trace = 1;
			break;
		default: 
			printf("Invalid Entry: %d\n", tp->param[TP_PKT_TRACE]);
	}			
	testMode = GT_LB_TEST_EXT;
	mdk_gt_mii_diagLBTest(port,testMode);
	tp->testresult = RET_SKIPPED;
	return(tp->testresult);
}	

int32
gt_5600_ext_lb2(
	mdk_test_params_t	* tp
)
{
	uint32 port;

	mdk_gt_mii_test_port = port = 2;
	switch (tp->param[TP_PKT_TRACE]) {
		case 0:
			gt64240mii->port[port].rx_trace = 0;
			gt64240mii->port[port].tx_trace = 0;
			break;
		case 1:
			gt64240mii->port[port].rx_trace = 1;
			gt64240mii->port[port].tx_trace = 0;
			break;
		case 2:
			gt64240mii->port[port].rx_trace = 0;
			gt64240mii->port[port].tx_trace = 1;
			break;
		case 3:
			gt64240mii->port[port].rx_trace = 1;
			gt64240mii->port[port].tx_trace = 1;
			break;
		default: 
			printf("Invalid Entry: %d\n", tp->param[TP_PKT_TRACE]);
	}			
	testMode = GT_LB_TEST_EXT;
	mdk_gt_mii_diagLBTest(port,testMode);
	if(fc_conflict)
		tp->result[0].testresult = RET_SKIPPED;
	tp->testresult = RET_SKIPPED;

	return(tp->testresult);
}	


int32
gt_pkg_err(
	mdk_log_entry_t	* err
)
{
	uint32 num1 = 0;
	uint32 i;
	switch (err->testresult) {
		case ERR_I2CADDR_FAIL:
			if (tp->param[TP_VERBOSE] == 1) {
				printf("GT I2C Test Failure : \n");
				printf("-------------------------------\n");
			    printf("addr	\n");
				while (err->result[0].addr[num1++]);
				for (i=0;i<(num1);i++) {
					printf("0x%x	 \n", err->result[0].addr[i]);
				}
			}
				break;
		case ERR_DATA_FAIL:
			if (tp->param[TP_VERBOSE] == 1) {
				printf("Register Test Failure : \n");
				printf("-------------------------------\n");
			    printf("addr	expected	got \n");
				while (err->result[0].addr[num1++]);
				for (i=0;i<(num1);i++) {
				printf("0x%x	 0x%x	0x%x\n",
					err->result[0].addr[i], err->result[0].exptd[i], err->result[0].rcvd[i]);}
			}
				break;
		case ERR_READ_FAIL:
			if (tp->param[TP_VERBOSE] == 1) {
				printf("\nPMC 8353 :  Read through MDIO\n");
				printf("=========================================================\n");
				printf("Register	Result	Expected	Received\n");
				while (err->result[0].addr[num1++]);
				for (i=0;i<(num1);i++) {
					printf("%08x	%s	%08x	%08x\n", 
					err->result[0].addr[i], err->result[0].exptd[i], err->result[0].rcvd[i]);}
			}
				break;
		case ERR_WRITE_FAIL:
			if (tp->param[TP_VERBOSE] == 1) {
				printf("\nPMC 8353 : Write through MDIO\n");
				printf("=========================================================\n");
				printf("Register	Result	Expected	Received\n");
				while (err->result[0].addr[num1++]);
				for (i=0;i<(num1);i++) {
					printf("%08x	%08x	%08x\n", 
					err->result[0].addr[i],  err->result[0].exptd[i], err->result[0].rcvd[i]);}
			}
				break;
		case ERR_TIMEOUT:
			if (tp->param[TP_VERBOSE] == 1) 
				printf("\nLoopback Test Timeout\n");
			break;

		case ERR_UNEXPECTED_RESULT:
			if (tp->param[TP_VERBOSE] == 1) 
				printf("\nResult not within expected limits.\n");
			break;
		case ERR_PKTDROP:
			if (tp->param[TP_VERBOSE] == 1) 
				printf("\nSome pkts have been dropped.\n");
			break;
		case ERR_LPBKDATA:
			if (tp->param[TP_VERBOSE] == 1) 
				printf("\nData corruption.\n");
			break;
		default:
				printf("Unknown error, testresult = 0x%x\n", err->testresult);
				break;
	}

	return 0;
}

#endif
