/* vi:ts=4
 *-----------------------------------------------------------------
 *
 * Name:
 *
 * RCS:  $Id: mdk-luc.c,v 1.2 2002/08/28 22:21:38 lipikaathome Exp $
 *
 * Copyright (C) 2001, Agile Storage, Inc.
 *
 * Description:
 *
 * Created by:	lipika	
 *
 * Date Created:	8/20/02
 *
 *-----------------------------------------------------------------
 */

#include <nfx-incl.h>
#include <exports/mdk-api.h>
#include "mdk-mem.h"
#if defined (DIAG_SIBYTE)
#include <exports/timer-api.h>
#include "../../sm-lsi8104/lsi8104.h"

extern uint32	time_handle_dp83820;
extern uint32	time_handle_port0;
extern uint32	time_handle_port1;
extern uint32	time_handle_port2;
extern uint32	time_handle_port3;
extern uint32	time_handle_port4;
extern uint32	time_handle_port5;


#define LUC_CMD_REG_DEFAULT 0x00000060
int32 sb_phy_lb_test(mdk_test_params_t	* tp, uint32 port, uint32 testMode);

/******************** FUNCTIONS ***********************************/

int32
luc_pkg_init()
{
	if(isFP())
		return;	
	tp->resultCnt = 1;
	tp->param[TP_LBPKT_CNT] = 10000;
	tp->param[TP_PKT_TRACE] = 0;
	if(!time_handle_port0)
		time_handle_port0 = eee_timerCreate(MDK_INTERPKTDELAY, EEE_TIMER_ONCE);
	if(!time_handle_port1)
		time_handle_port1 = eee_timerCreate(MDK_INTERPKTDELAY, EEE_TIMER_ONCE);
	if(!time_handle_port2)
		time_handle_port2 = eee_timerCreate(MDK_INTERPKTDELAY, EEE_TIMER_ONCE);
	if(!time_handle_port3)
		time_handle_port3 = eee_timerCreate(MDK_INTERPKTDELAY, EEE_TIMER_ONCE);
	if(!time_handle_port4)
		time_handle_port4 = eee_timerCreate(MDK_INTERPKTDELAY, EEE_TIMER_ONCE);
	if(!time_handle_port5)
		time_handle_port5 = eee_timerCreate(MDK_INTERPKTDELAY, EEE_TIMER_ONCE);
}

/* MAIN TEST FUNCTIONS */

int32 
sb_lucisUp_test(mdk_test_params_t *tp)
{
	volatile unsigned int *ndmp_reg;	
	uint32 o_data;

	if(isFP())
		return(tp->testresult = RET_SKIP_TEST);

	ndmp_reg = (unsigned int*)0xffffffffbf000000ll; /* Status reg*/ 
	o_data = *ndmp_reg;
	if(o_data == LUC_CMD_REG_DEFAULT)
		tp->testresult = RET_DEV_UP;
	else
		tp->testresult = RET_DEV_DOWN;
	tp->devName = "LUC";

	return(tp->testresult);
}

int32 
sb_luc_Regtest(mdk_test_params_t *tp)
{
	volatile unsigned int *ndmp_reg;	
	uint32 o_data, r_data;
	uint32 j = 0;
	uint32 data1 = 0xffffffff;
	uint32 	data2 = 0x50505050;

	if(isFP())
		return(tp->testresult = RET_SKIP_TEST);

	ndmp_reg = (unsigned int*)0xffffffffbf000080ll; /* read */ 
	o_data = *ndmp_reg;
	*ndmp_reg = data1;
	r_data = *ndmp_reg; 
	if(r_data != data1) {
		tp->testresult = ERR_DATA_FAIL;
		tp->result[0].addr[j] = ndmp_reg;
		tp->result[0].exptd[j] = data1;
		tp->result[0].rcvd[j] = r_data;	
		j++;
	}	
	else
		tp->testresult = RET_PASSED;

	if(tp->testresult == RET_PASSED) {		
		*ndmp_reg = data2;
		r_data = *ndmp_reg; 
		if(r_data != data2) {
			tp->testresult = ERR_DATA_FAIL;
			tp->result[0].addr[j] = ndmp_reg;
			tp->result[0].exptd[j] = data2;
			tp->result[0].rcvd[j] = r_data;	
		}	
		else
			tp->testresult = RET_PASSED;
	}			
	*ndmp_reg = o_data;
	return(tp->testresult);
}		

/* LOOPBACKS */
int32
sb_luc_int_lb_test_port0(
	mdk_test_params_t	* tp
)
{
	volatile unsigned int *ndmp_reg;	
	uint32 o_data;
	uint32 port;
	uint32 data1 = 0x00000002;
	uint32 testMode;

	if(isFP())
		return(tp->testresult = RET_SKIP_TEST);

	ndmp_reg = (unsigned int*)0xffffffffbf000000ll; /* read */ 
	o_data = *ndmp_reg;
	if(o_data != LUC_CMD_REG_DEFAULT)
		*ndmp_reg = LUC_CMD_REG_DEFAULT;

	*ndmp_reg = o_data|data1;

	port = 0;
	testMode = TEST_LUC; /* 3 */
	sb_phy_lb_test(tp,port,testMode);
	return(tp->testresult = RET_SKIPPED);
}

int32
sb_luc_int_lb_test_port1(
	mdk_test_params_t	* tp
)
{
	volatile unsigned int *ndmp_reg;	
	uint32 o_data;
	uint32 port;
	uint32 data1 = 0x00000004;
	uint32 testMode;

	if(isFP())
		return(tp->testresult = RET_SKIP_TEST);

	ndmp_reg = (unsigned int*)0xffffffffbf000000ll; /* read */ 
	o_data = *ndmp_reg;
	if(o_data != LUC_CMD_REG_DEFAULT)
		*ndmp_reg = LUC_CMD_REG_DEFAULT;
	*ndmp_reg = o_data|data1;

	port = 1;
	testMode = TEST_LUC; /* 3 */
	sb_phy_lb_test(tp,port,testMode);
	return(tp->testresult = RET_SKIPPED);
}

int32
sb_luc_int_lb_test_port2(
	mdk_test_params_t	* tp
)
{
	volatile unsigned int *ndmp_reg;	
	uint32 o_data;
	uint32 port;
	uint32 data1 = 0x00000006;
	uint32 testMode;

	if(isFP())
		return(tp->testresult = RET_SKIP_TEST);

	ndmp_reg = (unsigned int*)0xffffffffbf000000ll; /* read */ 
	o_data = *ndmp_reg;
	if(o_data != LUC_CMD_REG_DEFAULT)
		*ndmp_reg = LUC_CMD_REG_DEFAULT;
	*ndmp_reg = o_data|data1;

	port = 2;
	testMode = TEST_LUC; /* 3 */
	sb_phy_lb_test(tp,port,testMode);
	return(tp->testresult = RET_SKIPPED);
}

int32
sb_luc_int_lb_test_port3(
	mdk_test_params_t	* tp
)
{
	volatile unsigned int *ndmp_reg;	
	uint32 o_data;
	uint32 port;
	uint32 data1 = 0x00000070;
	uint32 testMode;

	if(isFP())
		return(tp->testresult = RET_SKIP_TEST);

	ndmp_reg = (unsigned int*)0xffffffffbf000000ll; /* read */ 
	o_data = *ndmp_reg;
	if(o_data != LUC_CMD_REG_DEFAULT)
		*ndmp_reg = LUC_CMD_REG_DEFAULT;
	*ndmp_reg = o_data|data1;

	port = 3;
	testMode = TEST_LUC; /* 3 */
	sb_phy_lb_test(tp,port,testMode);
	return(tp->testresult = RET_SKIPPED);
}

int32
sb_reset_luc_int_lpbk(
	mdk_test_params_t	* tp
)
{
	volatile unsigned int *ndmp_reg;	
	uint32 port;

	if(isFP())
		return(tp->testresult = RET_SKIP_TEST);

	ndmp_reg = (unsigned int*)0xffffffffbf000000ll; /* read */ 
	*ndmp_reg = LUC_CMD_REG_DEFAULT;  // This might change!!
	return(tp->testresult = RET_PASSED);
}

int32
luc_pkg_err(
	mdk_log_entry_t	* err
)
{

	uint32 i;
	uint32 num1 = 0;
	uint32 num2 = 0;

	switch (err->testresult) {
		case ERR_DATA_FAIL:
			if (tp->param[TP_VERBOSE] == 1) {
				printf("LUC Register Test Failure : \n");
				printf("-------------------------------\n");
			        printf("addr	expected	got \n");
				while (err->result[0].addr[num1++]);
				for (i=0;i<(num1 - 1);i++) {
				printf("0x%x	 0x%x		0x%x\n",
					err->result[0].addr[i], err->result[0].exptd[i], err->result[0].rcvd[i]);}
			}
			break;	
		case ERR_TIMEOUT:
			if (tp->param[TP_VERBOSE] == 1) {
				printf("Loopback test timeout\n");
			}	
				break;
		case ERR_UNEXPECTED_RESULT:
			if (tp->param[TP_VERBOSE] == 1) 
				printf("\nResult not within expected limits.\n");
			break;
		case ERR_PKTDROP:
			if (tp->param[TP_VERBOSE] == 1) 
				printf("\nSome pkts have been dropped.\n");
			break;
		case ERR_LPBKDATA:
			if (tp->param[TP_VERBOSE] == 1) 
				printf("\nData corruption.\n");
			break;
		default:
				printf("Unknown error, testresult = 0x%x\n", err->testresult);
				break;
	}

	return 0;
}
#endif
