/* vi:ts=4
 *-----------------------------------------------------------------
 *
 * Name:
 *
 * RCS:  $Id: mdk-mem-r7k.c,v 1.6 2002/07/27 00:50:46 lipikaathome Exp $
 *
 * Copyright (C) 2001, Agile Storage, Inc.
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	1/14/02
 *
 *-----------------------------------------------------------------
 */

#include <nfx-incl.h>
#include <exports/mdk-api.h>
#include "mdk-mem.h"
#if defined (DIAG_R7000)
uint64 memorysize;
#define MEG (1024*1024)

typedef struct _mem_tst_addr_t {
	unsigned long s_addr;
	unsigned long size;
} mem_tst_addr_t;

mem_tst_addr_t gt_256[] = {
	{PHYS_TO_K1(0x03000000),0xD000000}, 
	{NULL, NULL}
};

mem_tst_addr_t gt_mt[] = {
	{PHYS_TO_K1(0x03000000),0xD000000}, 
	{0x20000000,0x10000000}, 
	{0x60000000,0x14000000}, 
	{NULL, NULL}
};


int32
mem_pkg_init()
{
	tp->resultCnt = 1;
}


int32 
gt_mem_test(
	volatile mdk_test_params_t       * tp
)
{
	/* This tests the memory attached to the Galileo */
#ifdef DIAG_R7000	
	uint32 i;
	int rc;
	uint32 mem;
	mem_tst_addr_t *ram;
	uint32 tlb_changed=0;
	uint32 num;

	mem = memorysize / MEG;
	if (tp->param[TP_VERBOSE] == 1) 
		printf("Mem size %d",mem);
	tp->resultCnt = 0;
	if(isFC()) {
		if (mem <= 256) {
			num = 1;
			ram = gt_256;
		}	
		else { 
			fc_init_tlb();
			tlb_changed = 1;
			num = 3;
			ram = gt_mt;
		}		
		for (i=0;i<num; i++) {
			if (tp->param[TP_VERBOSE] == 1) {
			printf("\nTesting %08x to %08x (size %08x) ", ram[i].s_addr, ram[i].s_addr +  ram[i].size, ram[i].size);	
			//printf("Memory test running..  ");
			}
			rc = domt_walking_zeros(ram[i].s_addr, 0x100, tp);
			tp->resultCnt = 1;
			if(rc == RET_PASSED) {
				rc = domt_walking_ones(ram[i].s_addr, 0x100, tp);
				tp->resultCnt = 2;
			}	
			if(rc == RET_PASSED) {
				domt_address(ram[i].s_addr, ram[i].size,tp);
				tp->resultCnt = 3;
			}	
		}
	if(tlb_changed)
		init_tlb();
	}
#endif
	return;
}

int32
atmel_flash_test(
	mdk_test_params_t	* tp
)
{
	uint32	i, j;

	tp->resultCnt = 0;

	for (i=0; i<256; i++) {
		for (j=1; j<0x100; j <<= 1) {
			*(uint32 *)(0xbf000000 + (i*4)) = j;
			if ((*(uint32 *)(0xbf000000 + (i*4)) & 0xff) != j) {
				if (tp->param[TP_VERBOSE] == 1) {
					printf("error, offset 0x%x, wrote 0x%x, read 0x%x\n",
								i, j, *(uint32 *)(0xbf000000 + (i*4)) & 0xff);
					}
				tp->result[0].addr[i] = i;
				tp->result[0].exptd[i] = j;
				tp->result[0].rcvd[i] = (*(uint32 *)(0xbf000000 + (i*4)) & 0xff);	
				tp->result[0].testresult = ERR_DATA_FAIL;
			}	
			else
				tp->result[0].testresult = RET_PASSED;
		}
	}
	if (tp->param[TP_VERBOSE] == 1) 
		printf("Address Test\n");

	for (i=0; i<256; i++) 
	{
		if (tp->param[TP_VERBOSE] == 1) 
			printf("writing 0x%x to 0x%x\n", i, (uint32 *)(0xbf000000 + (i*4)));
		
		*(uint32 *)(0xbf000000 + (i*4)) = i;
		
		if (tp->param[TP_VERBOSE] == 1) 
			printf("reading addr 0x%x\n", (uint32 *)(0xbf000000 + (i*4)));

		if ((*(uint32 *)(0xbf000000 + (i*4)) & 0xff) != i) {
			if (tp->param[TP_VERBOSE] == 1) 
				printf("error, offset 0x%x, wrote 0x%x, read 0x%x\n",
								i, i, *(uint32 *)(0xbf000000 + (i*4)) & 0xff);
		
			tp->result[1].addr[i] = i;
			tp->result[1].exptd[i] = i;
			tp->result[1].rcvd[i] = (*(uint32 *)(0xbf000000 + (i*4)) & 0xff);	
			tp->result[1].testresult = ERR_ADDR_FAIL;

			if(!(tp->param[TP_CONT_DESP_ERR] )) 
				break;
		}
		else
				tp->result[1].testresult = RET_PASSED;
	}		
	/*
	for (i=0; i<256; i++) {
		if (tp->param[TP_VERBOSE] == 1) 
			printf("reading addr 0x%x\n", (uint32 *)(0xbf000000 + (i*4)));
		if ((*(uint32 *)(0xbf000000 + (i*4)) & 0xff) != i) {
			if (tp->param[TP_VERBOSE] == 1) 
				printf("error, offset 0x%x, wrote 0x%x, read 0x%x\n",
								i, i, *(uint32 *)(0xbf000000 + (i*4)) & 0xff);
			tp->result[1].addr[i] = i;
			tp->result[1].exptd[i] = i;
			tp->result[1].rcvd[i] = (*(uint32 *)(0xbf000000 + (i*4)) & 0xff);	
			tp->result[1].testresult = ERR_DATA_FAIL;
			if(!(tp->param[TP_CONT_DESP_ERR] )) 
				break;
		}
		else
				tp->result[1].testresult = RET_PASSED;
	}*/

	tp->resultCnt =2; /* Running 2 separate tests */
	return ;
}

int32 
gt_restr_mem_test(
	volatile mdk_test_params_t       * tp
)
{
	/* This tests the memory attached to the Galileo */
#ifdef DIAG_R7000	
	uint32 i;
	int rc;
	uint32 mem;
	mem_tst_addr_t *ram;
	uint32 tlb_changed=0;
	uint32 num;

	mem = memorysize / MEG;
	if (tp->param[TP_VERBOSE] == 1) 
		printf("Mem size %d",mem);
	tp->resultCnt = 0;
	if(isFC()) {
			if (mem <= 256) {
				num = 1;
				ram = gt_256;
			}	
			else { 
				fc_init_tlb();
				tlb_changed = 1;
				num = 3;
				ram = gt_mt;
			}		
		for (i=0;i<num; i++) {
			if (tp->param[TP_VERBOSE] == 1) {
			printf("\nTesting %08x to %08x (size %08x) ", ram[i].s_addr, ram[i].s_addr +  ram[i].size, ram[i].size);	
			//printf("Memory test running..  ");
			}
			rc = domt_walking_ones(ram[i].s_addr, ram[i].size, tp);
			tp->resultCnt = 1;
			if(rc == RET_PASSED) {
				rc = domt_walking_zeros(ram[i].s_addr, ram[i].size, tp);
				tp->resultCnt = 2;
			}	
			if(rc == RET_PASSED) {
				rc = domt_address(ram[i].s_addr, ram[i].size, tp);
				tp->resultCnt = 3;
			}	
		}
	if(tlb_changed)
		init_tlb();
	}
#endif
	return;
}


int32
gen_rmem_test(
	mdk_test_params_t       * tp
)
{
#ifdef DIAG_R7000	
	uint32 i;
	int rc;
	int32 s_addr;
	int32 size;
	uchar8	* line;

	tp->resultCnt = 0;
	printf("\nPlease enter starting address:\n");
	line = (uchar8 *)mdk_get_line(MDK_CMD_LINESIZE);
	if (strlen(line)) 
		s_addr =  atox(line);

	printf("Please enter size of mem to be tested:\n");
	line = (uchar8 *)mdk_get_line(MDK_CMD_LINESIZE);
	if (strlen(line)) 
		size =  atox(line);

	printf("\nTesting %08x to %08x ", s_addr, s_addr + size);	
	//printf("Memory test running..  ");
	if(tp->param[TP_WALKING_ONES] == 1) {
		printf("Walking ones\n");	
		domt_walking_ones(s_addr, size,tp);
		tp->resultCnt ++;
	}	
	else if(tp->param[TP_WALKING_ZEROS] == 1) {
		printf("Walking zeros\n");	
		domt_walking_zeros(s_addr, size,tp);
		tp->resultCnt ++;
	}	
	else if(tp->param[TP_ADDRESS] == 1) {
		printf("Address test\n");	
		domt_address(s_addr, size,tp);
		tp->resultCnt ++;
	}	
#else	
	tp->testresult = RET_PASSED;
#endif	
	return ;
}

fc_init_tlb()
{
#ifdef DIAG_R7000	
#endif
}


int32
mem_pkg_err(
	mdk_log_entry_t	* err
)
{
	uint32 i;
	uint32 num1 = 0;
	uint32 num2 = 0;
	switch (err->testresult) {
		case ERR_DATA_FAIL :
			if (tp->param[TP_VERBOSE] == 1) {
				printf("\nData Test Failure : \n");
				printf("--------------------------------------\n");
			        printf("addr	expected	got \n");
				while (err->result[0].addr[num1++]);
				for (i=0;i<(num1);i++) {
				printf("0x%x	 0x%x		0x%x\n",
					err->result[0].addr[i], err->result[0].exptd[i], err->result[0].rcvd[i]);}
			}	
			break;

		case ERR_ADDR_FAIL :		
			if (tp->param[TP_VERBOSE] == 1) {
				printf("\nAddress Test Failure : \n");
				printf("--------------------------------------\n");
			        printf("addr	expected	got \n");
				while (err->result[1].addr[num2++]);
				for (i=0;i<(num2);i++) {
				printf("0x%x	 0x%x		0x%x\n",
					err->result[1].addr[i], err->result[1].exptd[i], err->result[1].rcvd[i]);}
			}	
			break;

		default:
			printf("Unknown error, testresult = 0x%x\n", err->testresult);

	}		
	return 0;
}

#endif
