/* vi:ts=4
 *-----------------------------------------------------------------
 *
 * Name:        rpc.h
 *
 * RCS:  $Id: rpc.h,v 1.8 2002/09/13 00:19:20 joseph Exp $
 *
 * Copyright (C) 2001, Agile Storage Inc.
 *
 * Description: RPC  definitions
 *
 * Created by:  Joseph Vu
 *
 * Date Created:        5/18/01
 *
 *-----------------------------------------------------------------
 */

#define	RPC_PORTMAP_PROG_NUM	100000
#define	RPC_NFS_PROG_NUM		100003	
#define	RPC_MOUNT_PROG_NUM		100005	
#define	RPC_NLM_PROG_NUM		100021
#define	RPC_NSM_PROG_NUM		100024
#define	RPC_NFS_ACL_PROG_NUM	100227

#define	RPC_PORTMAPPER_WELLKNOWN_PORT	111

#define RPC_REPLY_MSG_ACCEPTED  0
#define RPC_REPLY_MSG_DENIED    1
#define RPC_REPLY_RPC_MISMATCH  0
#define RPC_REPLY_AUTH_ERROR    1

#define RPC_REPLY_SUCCESS   0
#define RPC_REPLY_PROG_UNAVAIL  1
#define RPC_REPLY_PROG_MISMATCH 2
#define RPC_REPLY_PROC_UNAVAIL  3
#define RPC_REPLY_GARBAGE_ARGS  4
#define RPC_REPLY_SYSTEM_ERR    5

#define RPC_AUTH_NONE       0       /*NULL authentication*/
#define PRC_AUTH_SYS        1       /*Same as AUTH_UNIX: UID, GID, groups.*/
#define RPC_AUTH_SHORT      2       /*Token authenticator for AUTH_SYS*/
#define RPC_AUTH_DH         3       /*Deffie-Hellman exchange of DES key*/
#define RPC_AUTH_KERB4      4       /*Kerberos ver 4 exchange of DES key*/
#define RPC_RSA             5
#define RPCSEC_GSS          6       /*Generic sercurity services API*/

#define	PRC_UID_ROOT		0
//#define RPC_UID_NOBODY		99
#define RPC_UID_NOBODY		65534	
#define RPC_VER2    2       		/*RPC Version number*/

#define	RPC_PORTMAPPER_VER2		2	/*portmapper program ver 2*/
#define	RPC_PORTMAPPER_GETPORT	3	/*portmapper getport proc*/
/* 
 *RPC Reply Status
 */
#define NFS_RPC_MISMATCH    0   	/*RPC version mismatch*/
#define NFS_RPC_AUTH_ERROR  1   	/*authentication errror*/
   
/* 
 *RPC Accept status
 */
#define NFS_RPC_OK              0   /*Remote procdedure was invoked successfully*/
#define NFS_RPC_PROG_UNAVAIL    1   /*The server does not recognize program number*/
#define NFS_RPC_PROG_MISMATCH   2   /*Bad program number*/
#define NFS_RPC_PROC_UNAVAIL    3   /*Bad procedure number*/
#define NFS_RPC_PROG_BAD_ARGS   4   /*The argument to the remote procedure cannot be XDR
                                     *decode.
                                     */


typedef struct rpc_authSysInfo
{
	uint32          authtype;       /* auth type */
	uint32          authlen;        /* auth length */
#ifdef	NOTUSED
	uint32      	stamp;
	uint32      	hostNameLen;
	char 			hostName[FS_NAME_MAX];
#endif	/*NOTUSED*/
	uint32			userId;
	uint32			groupId;
    uint32 			auxGidCount;
#define NFS_AUTH_SYS_GID_COUNT  16
	uint32			auxGids[NFS_AUTH_SYS_GID_COUNT];
} rpc_authSysInfo_t;

/*
 * RPC headers
 */
typedef struct rpcCallHdr
{
        uint32                  rpc_xid;		/* request transaction id */
        int32                   rpc_direction;  /* call direction (0) */
#define RPC_CALL        0               		/*call message*/
#define RPC_REPLY       1               		/*reply message*/
        uint32                  rpc_rpcVer;     /* rpc version (2) */
        uint32                  rpc_prog;       /* program */
        uint32                  progVer;     	/*program version */
        uint32                  rpc_proc;       /* procedure */
		union
		{
			rpc_authSysInfo_t	authSys;
		}authFlavor;
} rpcCallHdr_t;

typedef struct rpc_authInfo
{
	uint32          authtype;       /* auth type */
	uint32          authlen;        /* auth length */
} rpc_authInfo_t;

typedef struct rpcReplyHdr
{
        uint32          rpc_xid;        /* request transaction id */
        int32           rpc_direction;  /* call direction (1) */
        int32           rpc_status;     /* accept status (0: accepted) */
        union {
                uint32  rpu_errno;
                struct {
                        rpc_authInfo_t	rok_auth;
                        uint32          rok_status;
                } rpu_rok;
        } rp_u;
} rpcReplyHdr_t;

typedef struct rpc_stats
{
	uint64 rxReplySuccess;
	uint64 rxReplyAuthErr;
	uint64 rxReplyProgMismatchErr;
	uint64 rxReplyProcUnavailErr;
	uint64 rxReplyGarArgErr;
	uint64 rxReplySysErr;
	uint64 rxReplyGenErr;
	uint64 rxReplyRpcMismatchErr;
	uint64 rxInvalidReplyStatErr;
#define	RPC_AUTH_MIN_ERR				0
#define	RPC_AUTH_OK						0
#define	RPC_AUTH_BADCRED 				1
#define	RPC_AUTH_REJECTEDCRED			2
#define	RPC_AUTH_BADVERF				3
#define	RPC_AUTH_REJECTEDVERF			4
#define	RPC_AUTH_TOOWEAK				5
#define	RPC_AUTH_INVALIDRESP			6
#define	RPC_AUTH_FAILED					7
#define	RPC_AUTH_KERB_GENERIC			8
#define	RPC_AUTH_TIMEEXPIRE				9
#define	RPC_AUTH_TKT_FILE				10
#define	RPC_AUTH_DECODE					11
#define	RPC_AUTH_NET_ADDR				12
#define	RPC_RPCSEC_GSS_CREDPROBLEM		13
#define	RPC_RPCSEC_GSS_CTXPROBLEM		14
#define	PRC_AUTH_MAX_ERR				RPC_RPCSEC_GSS_CTXPROBLEM
	uint64 rxAuthErrType[PRC_AUTH_MAX_ERR];
} rpc_stats_t;

typedef struct rpc_globalCfg
{

	rpc_stats_t		stats;
} rpc_globalCfg_t;

/*Global variables*/
extern rpc_globalCfg_t	*rpc_globalCfgp;

/*
 *Prototypes
 */
rpcCallHdr_t	*rpc_processAuth(rpcCallHdr_t	*pktRpcCallHdrp, uint32 *rpcHdrSize);
