/* vi:ts=4
 *-----------------------------------------------------------------
 *
 * Name:        ndmp-agile-tbl.c
 *
 * RCS:  $Id: 
 *
 * Copyright (C) 2001, Agile Storage Inc.
 *
 * Description:
 *
 * Created by: Anh Vu
 *
 * Date Created:        01/27/02
 *
 *-----------------------------------------------------------------
 */
#include <nfx-incl.h>
#include <nfx-types.h>
#include <exports/opt-api.h> 
#include "ndmp.h"
#include "ndmp-ssc.h"
#include "ndmp-fs-api.h" 
#include <exports/tape-api.h> 
#include <exports/event-api.h>



/* Possible Agile msgs received on SSC side, from other processes
 * within SSC or from FP.
 */

/* Each message received is associated w/ a function to process.
 * Each table entry contains:
    - msgid          	(which message is it?) 
	- msgfunc  			(func to process) 
    - msg_name;      	(For debug msgs) 
	- idx to stat table (for debugging)
 */

ndmpAgileMsgEnt_t ndmpAgileMsgTbl[] =
{
	/* UI requests sent by UI process */
	{
		NDMP_MID_UI_SHOW_STATISTICS, ndmp_Child_ProcShowStatisticReq, "NDMP_MID_UI_SHOW_STATISTICS",
		UI_SHOW_STATISTICS_REQ_RCVD
	},
	{
		NDMP_MID_UI_SHOW_STATUS, ndmp_Child_ProcShowStatusReq, "NDMP_MID_UI_SHOW_STATUS",
		UI_SHOW_STATUS_REQ_RCVD
	},
	{
		NDMP_MID_UI_SHOW_DETAIL, ndmp_Child_ProcShowStatusReq, "NDMP_MID_UI_SHOW_DETAIL",
		UI_SHOW_DETAIL_REQ_RCVD
	},
	{
		NDMP_MID_UI_DEL_SESS, ndmp_Child_ProcDelSessReq, "NDMP_MID_UI_DEL_SESS",
		UI_DEL_SESS_REQ_RCVD
	},
	{
		NDMP_MID_UI_SET_DBG_LEVEL, ndmp_Child_ProcSetDbgLevel, "NDMP_MID_UI_SET_DBG_LEVEL",
		UI_CHLD_SET_DBG_LEVEL_REQ_RCVD
	},
	{
		NDMP_MID_UI_SET_TRACE_ON_OFF, ndmp_Child_ProcSetTraceFlag, "NDMP_MID_UI_SET_TRACE_ON_OFF",
		UI_CHLD_SET_TRACE_FLAG_REQ_RCVD
	},

	/* Response from FP for UI request */
	{
		NDMP_MID_UI_DM_GET_STATE, ndmp_procUIDMGetStateRsp, "NDMP_MID_UI_DM_GET_STATE",
		UI_DM_GET_STATE_REQ_RCVD
	},
	
#ifdef NFX_SMP
	/* Response from FP for miscellaneous request */
	{
		NDMP_MID_CLOSE_SESSION_RSP, ndmp_procCloseSessRsp, "NDMP_MID_CLOSE_SESSION_RSP",
		FP_CLOSE_SESS_RSP_RCVD
	},
#endif
	/***** Data & Mover responses *****/
	{
		NDMP_MID_MOVER_LISTEN, ndmp_procDM_Rsp, "NDMP_MID_MOVER_LISTEN",
		FP_MOVER_LISTEN_RSP_RCVD
	},
	{
		NDMP_MID_MOVER_CONNECT, ndmp_procDM_Rsp, "NDMP_MID_MOVER_CONNECT",
		FP_MOVER_CONN_RSP_RCVD
	},
	{
		NDMP_MID_MOVER_GET_STATE, ndmp_procDM_Rsp, "NDMP_MID_MOVER_GET_STATE",
		FP_MOVER_GET_STATE_RSP_RCVD
	},
	{
		NDMP_MID_MOVER_CONTINUE, ndmp_procDM_Rsp, "NDMP_MID_MOVER_CONTINUE",
		FP_MOVER_CONT_RSP_RCVD
	},
	{
		NDMP_MID_MOVER_ABORT, ndmp_procDM_Rsp, "NDMP_MID_MOVER_ABORT",
		FP_MOVER_ABORT_RSP_RCVD
	},
	{
		NDMP_MID_MOVER_CLOSE, ndmp_procDM_Rsp, "NDMP_MID_MOVER_CLOSE",
		FP_MOVER_CLOSE_RSP_RCVD
	},
	{
		NDMP_MID_MOVER_STOP, ndmp_procDM_Rsp, "NDMP_MID_MOVER_STOP",
		FP_MOVER_STOP_RSP_RCVD
	},
	{
		NDMP_MID_MOVER_SET_WINDOW, ndmp_procDM_Rsp, "NDMP_MID_SET_WINDOW",
		FP_MOVER_SET_WDW_RSP_RCVD
	},
	{
		NDMP_MID_MOVER_READ, ndmp_procDM_Rsp, "NDMP_MID_MOVER_READ",
		FP_MOVER_READ_RSP_RCVD
	},
	{
		NDMP_MID_MOVER_SET_RECORD_SIZE, ndmp_procDM_Rsp, "NDMP_MID_MOVER_SET_RECORD_SIZE",
		FP_MOVER_SET_RECSIZ_RSP_RCVD
	},
	{
		NDMP_MID_DATA_LISTEN, ndmp_procDM_Rsp, "NDMP_MID_DATA_LISTEN",
		FP_DATA_LISTEN_RSP_RCVD
	},
	{
		NDMP_MID_DATA_CONNECT, ndmp_procDM_Rsp, "NDMP_MID_DATA_CONNECT",
		FP_DATA_CONN_RSP_RCVD
	},
	{
		NDMP_MID_DATA_START_BACKUP, ndmp_procDM_Rsp, "NDMP_MID_DATA_START_BACKUP",
		FP_DATA_START_BKUP_RSP_RCVD
	},
	{
		NDMP_MID_DATA_START_RECOVER, ndmp_procDM_Rsp, "NDMP_MID_DATA_START_RECOVER",
		FP_DATA_START_RST_RSP_RCVD
	},
	{
		NDMP_MID_DATA_START_RECOVER_RSP, ndmp_procStartRecv_Rsp, "NDMP_MID_DATA_START_RECOVER_RSP",
		FP_DATA_START_RST_RSP_RCVD
	},
	{
		NDMP_MID_DATA_SEND_NAME_RSP, ndmp_procStartRecv_Rsp, "NDMP_MID_DATA_SEND_NAME_RSP",
		FP_DATA_SEND_NAME_RSP_RCVD
	},
	{
		NDMP_MID_DATA_GET_STATE, ndmp_procDM_Rsp, "NDMP_MID_DATA_GET_STATE",
		FP_DATA_GET_STATE_RSP_RCVD
	},
	{
		NDMP_MID_DATA_GET_ENV, ndmp_procDataGetEnvRsp, "NDMP_MID_DATA_GET_ENV",
		FP_DATA_GET_ENV_RSP_RCVD
	},
	{
		NDMP_MID_DATA_ABORT, ndmp_procDM_Rsp, "NDMP_MID_DATA_ABORT",
		FP_DATA_ABORT_RSP_RCVD
	},
	{
		NDMP_MID_DATA_STOP, ndmp_procDM_Rsp, "NDMP_MID_DATA_STOP",
		FP_DATA_STOP_RSP_RCVD
	},

	/* Configuration interface commands */
	{
		NDMP_MID_NOTIFY_DATA_HALTED, ndmp_procNotifHaltedMsg, "NDMP_MID_NOTIFY_DATA_HALTED",
		FP_NOTIFY_DATA_HALTED_REQ_RCVD
	},
	{
		NDMP_MID_NOTIFY_MOVER_HALTED, ndmp_procNotifHaltedMsg, "NDMP_MID_NOTIFY_MOVER_HALTED",
		FP_NOTIFY_MOVER_HALTED_REQ_RCVD
	},
	{
		NDMP_MID_NOTIFY_DATA_READ, ndmp_procNotifDM_Rsp, "NDMP_MID_NOTIFY_DATA_READ",
		FP_NOTIFY_DATA_RD_REQ_RCVD
	},
	{
		NDMP_MID_NOTIFY_MOVER_PAUSED, ndmp_procNotifDM_Rsp, "NDMP_MID_NOTIFY_MOVER_PAUSED",
		FP_NOTIFY_MOVER_PAUSED_REQ_RCVD
	},
	{
		NDMP_MID_LOG_FILE, ndmp_procLogFile, "NDMP_MID_LOG_FILE",
		FP_LOG_FILE_REQ_RCVD
	},
	{
		NDMP_MID_LOG_MESSAGE, ndmp_procLogMsg, "NDMP_MID_LOG_MESSAGE",
		FP_LOG_MSG_REQ_RCVD
	},
	{
		NDMP_MID_FH_ADD_DIR, ndmp_procFileHistMsg, "NDMP_MID_FH_ADD_DIR",
		FP_FH_ADD_DIR_REQ_RCVD
	},
	{
		NDMP_MID_FH_ADD_NODE, ndmp_procFileHistMsg, "NDMP_MID_FH_ADD_NODE",
		FP_FH_ADD_NODE_REQ_RCVD
	},
	{
		NDMP_MID_TAPE_SPACE_REQ, ndmp_sendTapeSpaceReq, "NDMP_MID_TAPE_SPACE_REQ",
		FP_TAPE_SPACE_REQ_RCVD
	},
	{
		/* Process responses from the Registry Manager */ 
		OPT_GET_OPTION_RSP, ndmp_procConnSrvrAuthRsp, "OPT_GET_OPTION_RSP",
		RM_OPT_GET_OPTION_RSP_RCVD
	},
	{
		NDMP_MID_SET_DUMPDATE_REQ, ndmp_setDumpDate, "NDMP_MID_SET_DUMPDATE_REQ",
		FP_SET_DMPDATE_REQ_RCVD

	},
    {
		NDMP_MID_UPDATE_ENV_REQ, ndmp_updateEnvReq, "NDMP_MID_UPDATE_ENV_REQ",
		FP_UPD_ENV_REQ_RCVD
    },
#if 0
	{
		/* Process responses from the EVM  */
		EVM_VOL_LIST, ndmp_procVolLstRsp, "EVM_VOL_LIST"
	},
	/* Process responses from NDMP on NNIM for FS */
	{
		FS_REQ_FSSTAT_VOL, ndmp_procFSInfoRsp, "FS_REQ_FSSTAT_VOL"
	},
	{
		TAPE_MID_GET_DEVLIST_RSP, ndmp_procTapeGetDevLstRsp, "TAPE_MID_GET_DEVLIST_RSP"
	},
#endif
	{
		TAPE_MID_OPEN_RSP, ndmp_procTapeOpenRsp, "TAPE_MID_OPEN_RSP", TAPE_OPEN_RSP_RCVD
	},
	{
		TAPE_MID_CLOSE_RSP, ndmp_procTapeCloseRsp, "TAPE_MID_CLOSE_RSP", TAPE_CLOSE_RSP_RCVD
	},
	{
		TAPE_MID_SPACE_RSP, ndmp_procTapeSpaceRsp, "TAPE_MID_SPACE_RSP", TAPE_SPACE_RSP_RCVD
	},
	{
		TAPE_MID_WRITE_FM_RSP, ndmp_procTapeWrFMRsp, "TAPE_MID_WRITE_FM_RSP", TAPE_WR_FM_RSP_RCVD
	},
	{
		TAPE_MID_GET_STATE_RSP, ndmp_procTapeGetStateRsp, "TAPE_MID_GET_STATE_RSP", TAPE_GET_STATE_RSP_RCVD
	},
	{
		TAPE_MID_LOAD_RSP, ndmp_procTapeLoadRsp, "TAPE_MID_LOAD_RSP", TAPE_LOAD_RSP_RCVD
	},
	{
		TAPE_MID_READ_RSP, ndmp_procTapeReadRsp, "TAPE_MID_READ_RSP", TAPE_RD_RSP_RCVD
	},
	{
		TAPE_MID_WRITE_RSP, ndmp_procTapeWriteRsp, "TAPE_MID_WRITE_RSP", TAPE_WR_RSP_RCVD
	},
	{
		TAPE_MID_PASSTHRU_RSP, ndmp_procTapePassThruRsp, "TAPE_MID_PASSTHRU_RSP", TAPE_PASSTHRU_RSP_RCVD
	},

	{
		MC_MID_GET_DEVLIST_RSP, ndmp_procScsiGetDevLstRsp, "MC_MID_GET_DEVLIST_RSP", MC_GET_DEVLST_RSP_RCVD
	},
	{
		MC_MID_OPEN_RSP, ndmp_procScsiOpenRsp, "MC_MID_OPEN_RSP", MC_OPEN_RSP_RCVD
	},
	{
		MC_MID_CLOSE_RSP, ndmp_procScsiCloseRsp, "MC_MID_CLOSE_RSP", MC_CLOSE_RSP_RCVD
	},
	{
		MC_MID_PASSTHRU_RSP, ndmp_procScsiPassThruRsp, "MC_MID_PASSTHRU_RSP", MC_PASSTHRU_RSP_RCVD
	},
	{
		0, 0, 0, 0
	}
};

ndmpParentUIMsgEnt_t ndmpParentUIMsgTbl[] =
{
	/* UI commands from ndmp-ui process */
	{
		NDMP_MID_UI_ENABLE_NDMP, ndmp_procEnaDisNdmpReq, "NDMP_MID_UI_ENABLE_NDMP",
		UI_ENABLE_REQ_RCVD
	},
	{
		NDMP_MID_UI_DISABLE_NDMP, ndmp_procEnaDisNdmpReq, "NDMP_MID_UI_DISABLE_NDMP",
		UI_DISABLE_REQ_RCVD
	},
	{
		NDMP_MID_UI_SHOW_STATE, ndmp_procShowStateReq, "NDMP_MID_UI_SHOW_STATE", 
		UI_SHOW_STATE_REQ_RCVD
	},
	{
		NDMP_MID_UI_SHOW_PAR_STATS, ndmp_procShowStatisticReq, "NDMP_MID_UI_SHOW_PAR_STATS", 
		UI_SHOW_PAR_STATISTICS_REQ_RCVD
	},
	{
		NDMP_MID_UI_CLEAR_PAR_STATS, ndmp_procClearStatisticReq, "NDMP_MID_UI_CLEAR_PAR_STATS", 
		UI_CLEAR_PAR_STATISTICS_REQ_RCVD
	},
	{
		NDMP_MID_UI_GET_SESS_TBL, ndmp_procGetSessTblReq, "NDMP_MID_UI_GET_SESS_TBL",
		UI_GET_SESSTBL_REQ_RCVD
	},
	{
		NDMP_MID_UI_SET_VERSION, ndmp_procSetVerReq, "NDMP_MID_UI_SET_VERSION", 
		UI_SET_VER_REQ_RCVD
	},
	{
		NDMP_MID_UI_SET_DBG_LEVEL, ndmp_procSetDbgLevel, "NDMP_MID_UI_SET_DBG_LEVEL",
		UI_PAR_SET_DBG_LEVEL_REQ_RCVD
	},
	{
		NDMP_MID_UI_SET_TRACE_ON_OFF, ndmp_procSetTraceFlag, "NDMP_MID_UI_SET_TRACE_ON_OFF",
		UI_SET_TRACE_FLAG_REQ_RCVD
	},
	{
		NDMP_MID_UI_SET_TRACE_FILES, ndmp_procSetTraceFiles, "NDMP_MID_UI_SET_TRACE_FILES",
		UI_SET_TRACE_FILES_REQ_RCVD
	},
	{
		NDMP_MID_UI_SET_RESTORE_MODE, ndmp_procSetRestoreMode, "NDMP_MID_UI_SET_RESTORE_MODE",
		UI_SET_RESTORE_MODE_REQ_RCVD
	},

	/* Msg from child session reporting to Parent its Agile appId */
	{
		NDMP_MID_CHILD_REPORT, ndmp_procChildReport, "NDMP_MID_CHILD_REPORT",
		CHILD_RPT_REQ_RCVD
	},
	{
		NDMP_MID_CHILD_SND_STATS, ndmp_procChildSndStats, "NDMP_MID_CHILD_SND_STATS",
		CHILD_SND_STATS_REQ_RCVD
	},
    /* Receive events from eventd */
    {
        NFX_EVENT_POST, ndmp_handleNfxEvent, "NFX_EVENT_POST",
		EVENT_POST_REQ_RCVD
    },
	{
		0, 0, 0, 0
	}
};

