/* vi:ts=4
 *-----------------------------------------------------------------
 *
 * Name:        ndmp-comm-util.c
 *
 * RCS:  $Id: ndmp-comm-util.c,v 1.8 2002/11/01 19:45:39 anhv Exp $
 *
 * Copyright (C) 2001, Agile Storage Inc.
 *
 * Description:		Contain utility funcs used by NDMP server
 *					on NNIM and SSC .
 *
 * Created by: Anh Vu
 *
 * Date Created:        10/31/01
 *
 *-----------------------------------------------------------------
 */
#include <nfx-incl.h>
#include <nfx-types.h>
#include "ndmp.h"


/*
 *------------------------------------------------------------------------------
 * Function Name:  ndmp_getEnvVar
 *
 * Description:		Get an environment variable.  given the environment
 *					variable name, it will return the environment variable's
 *					value via env_value.
 *
 * Created by:      Anh Vu
 *
 * Data Created:    9/17/01
 *
 * Returns:
 *------------------------------------------------------------------------------
 */
char8 *
ndmp_getEnvVar(char *env_name,
                ndmp_pval *env_lst,
                int env_lst_len)
{
    int32   i;

    for (i = 0; i< env_lst_len; i++)
    {
        if (!strcmp(env_name, env_lst[i].name))
        {
            return (env_lst[i].value);
        }
    }
    return NULL;
}      
/*
 *------------------------------------------------------------------------------
 * Function Name:   decode_data_halt_reas
 *
 * Description:		
 *
 * Created by:      Anh Vu
 *
 * Data Created:    10/10/02
 *
 * Returns:
 *------------------------------------------------------------------------------
 */
char8 *
decode_data_halt_reas(int reas)
{
	switch (reas)
	{
		case NDMP_DATA_HALT_NA: return "NA";
		case NDMP_DATA_HALT_SUCCESSFUL: return "Successful";
		case NDMP_DATA_HALT_ABORTED: return "Aborted";
		case NDMP_DATA_HALT_INTERNAL_ERROR: return "Internal Err";
		case NDMP_DATA_HALT_CONNECT_ERROR:	return "Connect Error";
		default:	return "Unknown";
	}
}
/*
 *------------------------------------------------------------------------------
 * Function Name:   decode_mover_halt_reas
 *
 * Description:		
 *
 * Created by:      Anh Vu
 *
 * Data Created:    10/11/02
 *
 * Returns:
 *------------------------------------------------------------------------------
 */
char8 *
decode_mover_halt_reas(int reas)
{
	switch (reas)
	{
		case NDMP_MOVER_HALT_NA: return "NA";
		case NDMP_MOVER_HALT_CONNECT_CLOSED: return "Connect Closed";
		case NDMP_MOVER_HALT_ABORTED: return "Aborted";
		case NDMP_MOVER_HALT_CONNECT_ERROR: return "Connect Err";
		case NDMP_MOVER_HALT_INTERNAL_ERROR:	return "Internal Error";
		default:	return "Unknown";
	}
}
/*
 *------------------------------------------------------------------------------
 * Function Name:   decode_mover_pause_reas
 *
 * Description:		
 *
 * Created by:      Anh Vu
 *
 * Data Created:    10/11/02
 *
 * Returns:
 *------------------------------------------------------------------------------
 */
char8 *
decode_mover_pause_reas(int reas)
{
	switch (reas)
	{
		case NDMP_MOVER_PAUSE_NA: return "NA";
		case NDMP_MOVER_PAUSE_EOM: return "EOM";
		case NDMP_MOVER_PAUSE_EOF: return "EOF";
		case NDMP_MOVER_PAUSE_SEEK: return "Seek";
		case NDMP_MOVER_PAUSE_MEDIA_ERROR:	return "Media Error";
		case NDMP_MOVER_PAUSE_EOW:	return "End of Window";
		default:	return "Unknown";
	}
}

