/* vi:ts=4
 *-----------------------------------------------------------------
 *
 * Name:        ndmp-connect.c
 *
 * RCS:  $Id: ndmp-connect.c,v 1.12 2002/08/28 23:40:18 anhv Exp $
 *
 * Copyright (C) 2001, Agile Storage Inc.
 *
 * Description:
 *
 * Created by: Anh Vu
 *
 * Date Created:        8/20/01
 *
 *-----------------------------------------------------------------
 */
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <nfx-types.h>
#include <nfx-incl.h>
#include <exports/eee-api.h>
#include <exports/elog-api.h>
#include <exports/anpssc-api.h>
#include <exports/ndmp-auth-api.h>
#include <exports/opt-api.h>
#include <exports/crypt-api.h>
#include "ndmp.h"
#include "ndmp-ssc.h"
#include "ndmp-debug.h"
extern char* AgileMsgbuf[];
char8	tbuf[256];

/*-----------------------------------------------------------------
 * Name:  ndmp_procConnOpenReq
 *
 * Description:	
 * 	
 * Created by:  Anh Vu
 *
 * Date Created:        8/20/01
 *
 *-----------------------------------------------------------------
 */
int
ndmp_procConnOpenReq(ndmpSess_t	*sess)
{
	ndmp_conn_open_req	*req;
	ndmp_conn_open_rsp	rsp;


	memset((void*)&rsp, 0, sizeof(rsp));

	req = (ndmp_conn_open_req *) sess->msgRcvd.content;

	NDMP_DBG_PRINTF(D_INFO, ("ndmp_ConnectOpen: protocol_version:%d.\n", req->protoVer));

	if (sess->protoVer != -1)  /* if protocol has been negotiated */	
	{
		rsp.error = (req->protoVer != NDMP_VER_2)?
				NDMP_ILLEGAL_STATE_ERR: NDMP_NO_ERR;
	} else {
		rsp.error = ((req->protoVer != NDMP_VER_3) &&
						(req->protoVer != NDMP_VER_2))? 
				NDMP_ILLEGAL_ARGS_ERR: NDMP_NO_ERR;
	}	
	/* Set the protocol version.  */
	if (rsp.error == NDMP_NO_ERR)
	{
		sess->protoVer =  req->protoVer;
	}
    /* Send Client a respond */
    return (ndmp_rspClient(sess, &rsp));

} /* ndmp_procConnOpenReq */


/*-----------------------------------------------------------------
 * Name: ndmp_procConnClientAuthReq 
 *
 * Description:	 Client is trying to authenticate itself by
 *				proving its identity to us.  
 * 	
 * Created by:  Anh Vu
 *
 * Date Created:        8/20/01
 *
 *-----------------------------------------------------------------
 */
int
ndmp_procConnClientAuthReq(ndmpSess_t	*sess)
{
	ndmp_conn_client_auth_req	*req;
	ndmp_conn_client_auth_rsp	rsp;
	ndmp_NFXReq_t				ndmp_req;	/* Msg to Mover/Data on NNIM */
	int32						auth_typ, rc;

	req = (ndmp_conn_client_auth_req *) sess->msgRcvd.content;
	auth_typ = req->auth_data.authTyp;

	NDMP_DBG_PRINTF(D_INFO, ("ndmp_procConnClientAut:  authTyp-> %s.\n",
		  auth_typ == NDMP_AUTHNONE ? "None" :
		  (auth_typ == NDMP_AUTHTXT) ? "Text" :
		   (auth_typ == NDMP_AUTHMD5) ? "MD5" : "Invalid"));

	memset((void*)&rsp, 0, sizeof(rsp));
	
	/* Begin authentication */

	rsp.error = NDMP_NO_ERR;		/* default */

	switch (req->auth_data.authTyp)
	{
		case NDMP_AUTHNONE:
		{
			break;
		}

		case NDMP_AUTHTXT:
		{
			ndmp_AuthTxt	*auth =
				&req->auth_data.ndmp_AuthData_u.authTxt;
			
			/*
			 * Call ndmp-auth API to validate username and password 
			 * Assuming we have some UI for user to add their username
		   	 * and encrypted password which will call Gilbert's 2nd API to
			 * enter these into a special file to be used to validate later.
			 */
			if (ndmp_authenticate(NDMP_AUTH_TEXT, auth->authId, auth->authPswd, 
								NULL) != TRUE)
			{
				NDMP_DBG_PRINTF(D_INFO, ("Failed TXT auth for user: %s.\n", auth->authId));
				E_LOG(class_1, err_s, eee_getApplicationId("ssc_ndmp"), 0, 0, 0, 
							("Failed TXT auth for user: %s.\n", auth->authId)); 
				rsp.error = NDMP_NOT_AUTHORIZED_ERR;
				break;
			}
			break;
		}
		
		case NDMP_AUTH_MD5:
		{
			ndmp_AuthMd5	*auth = 
				&req->auth_data.ndmp_AuthData_u.authMd5;
			
			/* Get our password for this user so we can calculate the Digest value */
			if (ndmp_authenticate(NDMP_AUTH_MD5, auth->authId, auth->authDigest, 
									sess->challenge) != TRUE)
			{	
				NDMP_DBG_PRINTF(D_INFO, ("Failed MD5 auth for user: %s.\n", auth->authId));
				E_LOG(class_1, err_s, eee_getApplicationId("ssc_ndmp"), 0, 0, 0, 
					("Failed MD5 auth for user: %s.\n", auth->authId)); 
				rsp.error = NDMP_NOT_AUTHORIZED_ERR;
				break;
			}
			break;
		}
		
		default:
		{
			/* don't support this type of auth */
			NDMP_DBG_PRINTF(D_INFO, ("Invalid auth type received %d", req->auth_data.authTyp));
			E_LOG(class_1, err_s, eee_getApplicationId("ssc_ndmp"), 0, 0, 0,
					("Invalid auth type received %d", req->auth_data.authTyp));
			rsp.error = NDMP_ILLEGAL_ARGS_ERR;
		}
	}
	
	sess->authed = (rsp.error == NDMP_NO_ERR)? TRUE : FALSE;
	sess->auth_typ = (rsp.error == NDMP_NO_ERR)? 
				req->auth_data.authTyp : NDMP_AUTHNONE;
    /* Send Client a respond */
    rc = (ndmp_rspClient(sess, &rsp));

	/* Send msg to NNIM to creates state machines for MOVER & DATA services there */
	if (sess->authed)
	{
	    memset((void*)&ndmp_req, 0, sizeof(ndmp_req));
    	ndmp_req.hdr.mid        = NDMP_MID_NEW_SESSION;
    	ndmp_req.hdr.sessId     = sess->sessId;
#ifdef NFX_SMP
		ndmp_req.hdr.seqId     = sess->xmt2fp_seqId;
		sess->xmt2fp_seqId ++;
		ndmp_req.hdr.idx        = sess->sess_idx;
#endif
       	NDMP_DBG_PRINTF(D_INFO, ("ndmp_procConnClientAuthReq:  SessId "
				"sent to FP 0x%x\n", ndmp_req.hdr.sessId));
    	ndmp_req.userInfo     	= sess->protoVer;
    	ndmp_req.content[0]    	= sess->sess_idx;

	    if (sendAgileMsg (sess->Agilesock, &ndmp_req, sizeof(ndmp_req),
                              "ndmp",       /* NDMP on the N-NIM */
                               NFX_NFP_FP2, 1, MGMTPLANE, /* cpu, slot, plane */
                               NULL, NULL, 0) < 0)
    	{
        	NDMP_DBG_PRINTF(D_INFO, ("ndmp_procConnClientAuthReq:  Agile "
				"message send to CLOSE tape failed\n"));
        	return NFX_ERR;
    	}
	}
	return rc;

} /* ndmp_procConnClientAuthReq */

/*-----------------------------------------------------------------
 * Name:  ndmp_procConnSrvrAuthReq
 *
 * Description:	  Process request to authenticate server to client.
 * 	
 * Created by:  Anh Vu
 *
 * Date Created:        8/20/01
 *
 *-----------------------------------------------------------------
 */
int
ndmp_procConnSrvrAuthReq(ndmpSess_t *sess)
{
	ndmp_conn_srvr_auth_req	*req;
	ndmp_conn_srvr_auth_rsp	rsp;
	void					*bufp = NULL;
	char8					reqbuf[1024];
	opt_msgHdr_t			opt_hdr;
	opt_getOptionReq_t 		get_opt;
	int						reqAuthTyp, buflen;
	

	req = (ndmp_conn_srvr_auth_req *) sess->msgRcvd.content;
	reqAuthTyp = req->client_attr.authTyp;

	NDMP_DBG_PRINTF(D_INFO, ("ndmp_ConnectServerAuth: authTyp:%s.\n",
		  req->client_attr.authTyp == NDMP_AUTHNONE ? "None" :
		  (req->client_attr.authTyp == NDMP_AUTHTXT ? "Text" :
		   (req->client_attr.authTyp == NDMP_AUTHMD5 ? "MD5" :
		   "Invalid"))));


	memset((void*)&rsp, 0, sizeof(rsp));
	rsp.error = NDMP_NO_ERR;
	rsp.srvr_result.authTyp = req->client_attr.authTyp;

	/* 
	 * If it's Text or MD5, we go fetch our own id and password first
	 */
	if ((reqAuthTyp==NDMP_AUTHTXT) || (reqAuthTyp==NDMP_AUTHMD5))
	{
		bufp = &reqbuf;
		memset(bufp, 0, sizeof(reqbuf));
		opt_hdr.msgType = OPT_GET_OPTION_REQ;
		strcpy(get_opt.option, "options.ndmp.auth.server.*");	
		memcpy(bufp, (void *) &opt_hdr, sizeof(opt_msgHdr_t));
		memcpy(bufp + sizeof(opt_msgHdr_t), (void *) &get_opt, sizeof(opt_getOptionRsp_t));
		buflen = sizeof(opt_msgHdr_t) + sizeof(opt_getOptionRsp_t);

		/* Send the request to the registry manager located on the SSC.  */
    	if (sendAgileMsg(sess->Agilesock,
                           reqbuf, buflen,
                           "regm", 0, 0, 0,
                           NULL,
                           NULL, 0) < 0)
		{
			rsp.error = NDMP_UNDEFINED_ERR;
			NDMP_DBG_PRINTF(D_INFO, ("Message send failed\n"));
		} else {
			sess->ndmp_ssc_chld_msgStats[RM_OPT_GET_OPTION_REQ_SNT]++;
			return NFX_OK;
		}
	}
	    /* Send Client a respond */
    return (ndmp_rspClient(sess, &rsp));

} /* ndmp_procConnSrvrAuthReq */

/*-----------------------------------------------------------------
 * Name:  ndmp_procConnSrvrAuthRsp
 *
 * Description:   Process request to authenticate server to client.
 *
 * Created by:  Anh Vu
 *
 * Date Created:        8/20/01
 *
 *-----------------------------------------------------------------
 */
int32
ndmp_procConnSrvrAuthRsp(ndmpSess_t *sess, char8 *rspbuf, ndmp_src_t *src)
{ 
	
	ndmp_conn_srvr_auth_req	*req;
	ndmp_conn_srvr_auth_rsp	rsp;
	opt_entry_t 			*entry;
	char8					id[32];	/* client's id max len */
	char8  					encodedPasswd[CRYPT_MAX_PASSWD_LEN], 
							pswd[CRYPT_MAX_PASSWD_LEN];  /* 64 */
	int						reqAuthTyp;
	opt_msgHdr_t *      	hdr;
	opt_getOptionRsp_t *	getrsp;


	req = (ndmp_conn_srvr_auth_req *) sess->msgRcvd.content;
	reqAuthTyp = req->client_attr.authTyp;

	memset((void*)&rsp, 0, sizeof(rsp));
	rsp.error = NDMP_NO_ERR;
	rsp.srvr_result.authTyp = reqAuthTyp;

	/* Assuming only one entry for server id & password 
	 * Skip nbEntry which is the len of variable-len array of entries.
	 * Take first entry.
	 */
	hdr = (opt_msgHdr_t *)rspbuf;
	getrsp = (opt_getOptionRsp_t *)(hdr + 1);
	if (getrsp->nbEntry < 1)
	{
		NDMP_DBG_PRINTF(D_INFO, ("ndmp_procConnSrvrAuthRsp:  no entry in Registry Mgr for server to authenticate\n"));
		rsp.error = NDMP_UNDEFINED_ERR;
    	return (ndmp_rspClient(sess, &rsp));
	}
	entry = &getrsp->entry[0];
	if (!entry || !entry->option || ! entry->value)
	{
		NDMP_DBG_PRINTF(D_INFO, ("ndmp_procConnSrvrAuthRsp:  entry values returned by Registry Mgr is NULL.\n"));
		rsp.error = NDMP_UNDEFINED_ERR;
    	return (ndmp_rspClient(sess, &rsp));
	}
	NDMP_DBG_PRINTF(D_INFO, ("%-45.45s\t%s\n", entry->option, entry->value));
	strcpy(id, rindex((char *)entry->option, '.') + 1);  
	strcpy(encodedPasswd, entry->value);
	if (index(encodedPasswd, '\n'))
    {
      	*index(encodedPasswd, '\n') = 0;
    }

	/* get clear text password */
	crypt_decode(encodedPasswd, pswd);

	switch (reqAuthTyp)
	{
		case NDMP_AUTHNONE:
			break;

		case NDMP_AUTHTXT:
		{
			rsp.srvr_result.ndmp_AuthData_u.authTxt.authId = id;
			rsp.srvr_result.ndmp_AuthData_u.authTxt.authPswd = pswd;
			break;
		}
		
		case NDMP_AUTHMD5:
		{
			char8 		*challenge;
            char        msg[128];
            crypt_md_t  digest;

			/* Using client's challenge string, we go fetch digest string
			 * using our password.
			 */
			challenge = req->client_attr.ndmp_AuthAttr_u.challenge;

            bzero((void *) msg, sizeof(msg));
            memcpy((void *) msg, pswd, strlen(pswd));
            memcpy((void *) &msg[128 - strlen(pswd)],
                   pswd, strlen(pswd));
            memcpy((void *) &msg[128 - strlen(pswd) - 64],
                   challenge, 64);

            if (crypt_digest("md5", msg, sizeof(msg), &digest) != NFX_OK)
			{	
				NDMP_DBG_PRINTF(D_INFO, ("ndmp_procConnSrvrAuthRsp:  Failed to get server's id & digest."));
				E_LOG(class_1, err_s, eee_getApplicationId("ssc_ndmp"), 0, 0, 0,
						("Failed to get server's id & digest."));
				rsp.error = NDMP_ILLEGAL_ARGS_ERR;
				break;
			}
			rsp.srvr_result.ndmp_AuthData_u.authMd5.authId = id;
			memcpy(rsp.srvr_result.ndmp_AuthData_u.authMd5.authDigest, digest.value, digest.len);
			break;
		}
		
		default:
		{
			E_LOG(class_1, err_s, eee_getApplicationId("ssc_ndmp"), 0, 0, 0,
					("Invalid auth type received %d.", reqAuthTyp));
			rsp.error = NDMP_ILLEGAL_ARGS_ERR;
		}
	}
	
    /* Send Client a respond */
    return (ndmp_rspClient(sess, &rsp));

} /* ndmp_procConnSrvrAuthRsp */


/*-----------------------------------------------------------------
 * Name:  ndmp_procConnCloseReq
 *
 * Description:	  Close the control connection on behalf of the client.
 * 	
 * Created by:  Anh Vu
 *
 * Date Created:        8/20/01
 *
 *-----------------------------------------------------------------
 */
int
ndmp_procConnCloseReq(ndmpSess_t *sess)
{
	NDMP_DBG_PRINTF(D_INFO, ("ndmp_procConnCloseReq: begin.\n"));

	(void)ndmp_CloseSess(sess);
	return NFX_OK;
}


