/* 
 *-----------------------------------------------------------------
 *
 * Name:        ndmp-data-v2-func.c
 *
 * RCS:  $Id: ndmp-data-v2-func.c,v 1.8 2002/10/26 00:54:04 anhv Exp $
 *
 * Copyright (C) 2001, Agile Storage Inc.
 *
 * Description: 
 *
 * Created by: Anh Vu
 *
 * Date Created:        9/06/01
 *
 *-----------------------------------------------------------------
 */
#include <nfx-incl.h>
#include <exports/eee-api.h>
#include <exports/malloc-api.h>
#include <exports/cm-api.h>
#include "ndmp.h"
#include "ndmp-data-sm.h"
#include "ndmp-data-v2-sm.h"
#include "ndmp-mover-sm.h"
#include <exports/fs-ndmp-api.h>
#include "ndmp-fs-api.h"
#include "ndmp-debug.h"


extern int ndmp_bkup(NdmpNIMService_t *servp, ndmp_NFXReq_t *req);
extern int ndmp_recov(NdmpNIMService_t *servp, ndmp_NFXReq_t *req);
extern int ndmp_saveDumpDate(NdmpNIMService_t *servp, ndmp_set_dumpdate_req_msg_t *fs_req);

extern ndmp_NNIMCtrl_t ndmp_NNIMCtrl;

int fill_Data_get_state_info (Data_t *data, ndmp_data_get_state_rsp *gs_rsp, 
			esm_handle_t handle);

/*
 *------------------------------------------------------------------------------
 * Function Name:   ndmp_Data_v2_create
 *
 * Description:     Device init state machine initialization function.
 *
 * Created by:      Anh Vu
 *
 * Data Created:    09/17/01 
 *
 * Returns:
 *------------------------------------------------------------------------------
 */
int32
ndmp_Data_v2_create(esm_handle_t   handle,     /* handle for this sm inst    */
                 int32          nextState,  /* next state from table      */
                 esm_event_t    thisEvent,  /* event driving this action  */
                 void           *userInfo)  /* ctrl blk linked to sm      */
{
    NdmpNIMService_t *servp = (NdmpNIMService_t *) userInfo;
    Data_t                 *data = &servp->Data;

    NDMP_DBG_PRINTF(D_INFO,("ndmp_Data_v2_create:   Begin, sessid %d, esm_hdl %d\n", servp->sessId, handle));

	ndmp_data_init(data);

    return(nextState);

} /* ndmp_Data_create */
/*
 *------------------------------------------------------------------------------
 * Function Name:   ndmp_Data_v2_opsRsp
 *
 * Description:
 *
 * Created by:      Anh Vu
 *
 * Data Created:    9/17/01
 *
 * Returns:
 *------------------------------------------------------------------------------
 */
int
ndmp_Data_v2_opsRsp(esm_handle_t handle,
                    int32 nextState, /* next state from table       */
                    esm_event_t thisEvent, /* event driving this action   */
                    void  *userInfo) /* ctrl blk linked to sm       */
{
    NdmpNIMService_t      *servp = (NdmpNIMService_t *) userInfo;

    NDMP_DBG_PRINTF(D_INFO, ("ndmp_Data_v2_opsRsp:  Begin, sessid %d, esm_hdl %d\n", servp->sessId, handle));

	return ndmp_Data_opsRsp (handle, nextState, thisEvent, userInfo);

} /* ndmp_Data_v2_opsRsp */

/*
 *------------------------------------------------------------------------------
 * Function Name:   ndmp_Data_v2_sndNameLst
 *
 * Description:
 *
 * Created by:      Anh Vu
 *
 * Data Created:    1/9/03
 *
 * Returns:
 *------------------------------------------------------------------------------
 */
int
ndmp_Data_v2_sndNameLst(esm_handle_t handle,
                    int32 nextState, /* next state from table       */
                    esm_event_t thisEvent, /* event driving this action   */
                    void  *userInfo) /* ctrl blk linked to sm       */
{
    NdmpNIMService_t      *servp = (NdmpNIMService_t *) userInfo;

    NDMP_DBG_PRINTF(D_INFO, ("ndmp_Data_v2_sndNameLst:  Begin, sessid %d, esm_hdl %d\n", servp->sessId, handle));

	return ndmp_Data_sndNameLst (handle, nextState, thisEvent, userInfo);

} /* ndmp_Data_v2_sndNameLst */
/*
 *------------------------------------------------------------------------------
 * Function Name:   ndmp_Data_v2_bkup_rcov
 *
 * Description:
 *
 * Created by:      Anh Vu
 *
 * Data Created:    9/17/01
 *
 * Returns:
 *------------------------------------------------------------------------------
 */
int
ndmp_Data_v2_bkup_rcov(esm_handle_t handle,
                    int32       nextState, /* next state from table       */
                    esm_event_t thisEvent, /* event driving this action   */
                    void        *userInfo)  /* ctrl blk linked to sm          */
{
    ndmp_NFXReq_t   			*req = (ndmp_NFXReq_t *) thisEvent.p1.u.p;
    NdmpNIMService_t   			*servp = (NdmpNIMService_t *) userInfo;
	ndmp_NFXBkupReq_t           *bkup_req =
                (ndmp_NFXBkupReq_t *) (req->content + req->len1);
	ndmp_NFXRecovReq_t          *recov_req =  (ndmp_NFXRecovReq_t *) req->content;
	esm_event_t     			esmEvent;
	int32						rc, addr_typ;

    NDMP_DBG_PRINTF(D_INFO, ("ndmp_Data_bkup_rcov:  Begin, sessid %d, esm_hdl %d\n", servp->sessId, handle));

	addr_typ = (req->hdr.mid == NDMP_MID_DATA_START_BACKUP)? 
				bkup_req->addr_typ: recov_req->addr_typ;

	if (addr_typ == NDMP_ADDR_LOCAL)
	{
		/* send event to signal Mover to be in CONNECTED mode as well */
        bzero(&esmEvent, sizeof(esm_event_t));
		esmEvent.event = NDMP_MOVER_EVENT_CONNECTED;
		rc = esm_deliverEvent(servp->Mover.my_smHdl, &esmEvent);
		if (rc != NFX_OK)
		{
			NDMP_DBG_PRINTF(D_INFO,("ndmp_Data_bkup_rcov: ERROR, cannot post event %d to SM hdl %d",
				NDMP_MOVER_EVENT_CONNECTED, servp->Mover.my_smHdl));
		}

		/* We are now connected (locally) of course */
	}

	return ndmp_Data_bkup_rcov(handle, nextState, thisEvent, userInfo);
	
} /* ndmp_Data_v2_bkup_rcov */

/*
 *------------------------------------------------------------------------------
 * Function Name:   ndmp_Data_v2_FsIOReq
 *
 * Description:		Process FS request to read/write data from/to Mover server
 *					(for local) or data connection (for remote).
 *
 * Created by:      Anh Vu
 *
 * Data Created:    11/17/01
 *
 * Returns:
 *------------------------------------------------------------------------------
 */
int
ndmp_Data_v2_FsIOReq(esm_handle_t handle,
                    int32       nextState, /* next state from table       */
                    esm_event_t thisEvent, /* event driving this action   */
                    void        *userInfo)  /* ctrl blk linked to sm          */
{
    NdmpNIMService_t            *servp = (NdmpNIMService_t *) userInfo;

    NDMP_DBG_PRINTF(D_INFO, ("ndmp_Data_v2_procFsReq:  Begin, sessid %d, esm_hdl %d\n", servp->sessId, handle));

	return ndmp_Data_FsIOReq(handle, nextState, thisEvent, userInfo); 

} /* ndmp_Data_v2_FsIOReq */

/*
 *------------------------------------------------------------------------------
 * Function Name:   ndmp_Data_v2_FsIORsp
 *
 * Description:		Send FS responses to its read/write request to 
 *					data from/to Mover server (for local) or data connection 
 *					(for remote).  We do this when the Mover Server tells
 *					us he's done writing/reading (for local backup/restore) 
 *					or when TPLxmtPkt or TPLrcvPkt signals us the data 
 *					has been xmitted or received over the data connection.
 *
 * Created by:      Anh Vu
 *
 * Data Created:    11/17/01
 *
 * Returns:
 *------------------------------------------------------------------------------
 */
int
ndmp_Data_v2_FsIORsp(esm_handle_t handle,
                    int32       nextState, /* next state from table       */
                    esm_event_t thisEvent, /* event driving this action   */
                    void        *userInfo)  /* ctrl blk linked to sm          */
{
    NdmpNIMService_t            *servp = (NdmpNIMService_t *) userInfo;

    NDMP_DBG_PRINTF(D_INFO, ("ndmp_Data_v2_FsIORsp:  Begin, sessid %d, esm_hdl %d\n", servp->sessId, handle));

	return ndmp_Data_FsIORsp(handle, nextState, thisEvent, userInfo);

} /* ndmp_Data_v2_FsIORsp */

/*
 *------------------------------------------------------------------------------
 * Function Name:   ndmp_Data_v2_snd_info
 *
 * Description:		It does the following:
 *					- Sends Logs 
 *						1) Messages (ERROR, WARNING, LOG, DBG, INFO) 
 *						2) File Log for recovery 
 *					- send File History during backup.
 *					- send Set-Dump-Date request issued by FS.
 *
 * Created by:      Anh Vu
 *
 * Data Created:    9/17/01
 *
 * Returns:
 *------------------------------------------------------------------------------
 */
int
ndmp_Data_v2_snd_info(esm_handle_t handle,
                    int32       nextState, /* next state from table       */
                    esm_event_t thisEvent, /* event driving this action   */
                    void        *userInfo)  /* ctrl blk linked to sm          */
{
    NdmpNIMService_t            *servp = (NdmpNIMService_t *) userInfo;

    NDMP_DBG_PRINTF(D_INFO, ("ndmp_Data_snd_info:  Begin, sessid %d, esm_hdl %d\n", servp->sessId, handle));
	
	return ndmp_Data_snd_info(handle, nextState, thisEvent, userInfo);  
	
} /* ndmp_Data_v2_snd_info */

/*
 *------------------------------------------------------------------------------
 * Function Name:   ndmp_Data_v2_getenv
 *
 * Description:		We return to client the ENV variables that he sent us
 *					at time of BKUP or RECOVER begins.  We can also send back to him
 *					additioanal ENV vars or modified but must return all that
 *					he sent us, even the ones we don't support.
 *
 * Created by:      Anh Vu
 *
 * Data Created:    9/17/01
 *
 * Returns:
 *------------------------------------------------------------------------------
 */
int
ndmp_Data_v2_getenv(esm_handle_t handle,
                    int32       nextState, /* next state from table       */
                    esm_event_t thisEvent, /* event driving this action   */
                    void        *userInfo)  /* ctrl blk linked to sm          */
{

	NdmpNIMService_t	*servp = (NdmpNIMService_t *) userInfo;


    NDMP_DBG_PRINTF(D_INFO, ("ndmp_Data_v2_getenv:  Begin, sessid %d, esm_hdl %d\n", servp->sessId, handle));

	return ndmp_Data_getenv(handle, nextState, thisEvent, userInfo);

} /* ndmp_Data_v2_getenv */
/*
 *------------------------------------------------------------------------------
 * Function Name:   fill_Data_v2_get_state_info
 *
 * Description:		diff from v3:  no invalid & data_conn_addr fields, diff
 *					ndmpv2_data_get_state_rsp structure.
 *
 * Created by:      Anh Vu
 *
 * Data Created:    9/17/01
 *
 * Returns:
 *------------------------------------------------------------------------------
 */
int
fill_Data_v2_get_state_info (Data_t *data, ndmpv2_data_get_state_rsp *gs_rsp, 
							esm_handle_t handle)	
{
	int32	cur_st;	


	NDMP_DBG_PRINTF(D_INFO, ("fill_Data_v2_get_state_info:\n"));

	gs_rsp->error   = NDMP_NO_ERR;
    gs_rsp->op = data->oper;  /* current operation */
	cur_st = esm_getCurrState(handle);
    gs_rsp->state = (cur_st == NDMP_DATA_V2_STATE_IDLE)? NDMP_DATA_ST_IDLE:
		(cur_st == NDMP_DATA_V2_STATE_ACTIVE)? NDMP_DATA_ST_ACTIVE: NDMP_DATA_ST_HALTED;  
    gs_rsp->halt_reason = data->haltReas;/* current reason */
    gs_rsp->bytes_processed.low = data->bytesProcessed;        /* total bytes processed so far */
    gs_rsp->bytes_processed.high = data->bytesProcessed >> 32;        /* total bytes processed so far */
	gs_rsp->est_bytes_remain.high = data->bytesLeftToRead >> 32;

#if 0	//We don't support this for now.
	gs_rsp->est_time_remain = data->estTimeRemain;
	gs_rsp->est_bytes_remain.low = data->bytesLeftToRead;
#endif

    gs_rsp->read_offset.low = data->readOffset;      /* data read seek offset */
    gs_rsp->read_offset.high = data->readOffset >> 32;  
			/* data read seek offset */
    gs_rsp->read_length.low = data->readLen;         /* data read length */
    gs_rsp->read_length.high = data->readLen >> 32;  /* data read length */

	NDMP_DBG_PRINTF(D_INFO, ("Data Get State:\n" "---------------\n" "State            = %s\n" 
				"Halt Reason  = %s\n" "Bytes_processed  = %lld\n" 
				"Est_bytes_remain = %lld\n" "Est_time_remain  = %lld\n" 
				"Read_offset      = %lld\n" "Read_len         = %lld\n" 
				"Addr type        = %s\n", 
				(cur_st == NDMP_DATA_V2_STATE_IDLE)? "Idle":
				(cur_st == NDMP_DATA_V2_STATE_ACTIVE)? "Active": "Halted", 
				decode_data_halt_reas(data->haltReas), data->bytesProcessed, 
				data->bytesLeftToRead, data->estTimeRemain, data->readOffset, data->readLen, 
				(data->addr_type == NDMP_ADDR_LOCAL)? "local": "tcp"));

	return NFX_OK;
} /* fill_Data_v2_get_state_info */

/*
 *------------------------------------------------------------------------------
 * Function Name:   ndmp_Data_v2_get_state
 *
 * Description:
 *
 * Created by:      Anh Vu
 *
 * Data Created:    9/17/01
 *
 * Returns:
 *------------------------------------------------------------------------------
 */
int
ndmp_Data_v2_get_state(esm_handle_t handle,
                    int32       nextState, /* next state from table       */
                    esm_event_t thisEvent, /* event driving this action   */
                    void        *userInfo)  /* ctrl blk linked to sm          */
{
	ndmp_NFXReq_t   			*req = (ndmp_NFXReq_t *) thisEvent.p1.u.p;
 	ndmp_NFXRsp_t               *rsp;
	ndmpv2_data_get_state_rsp	*gs_rsp;
    NdmpNIMService_t 			*servp = (NdmpNIMService_t *) userInfo;
	Data_t                      *data = &servp->Data;
    int32       				dst_port = servp->ssc_src_port, msglen;

    NDMP_DBG_PRINTF(D_INFO, ("ndmp_Data_get_state:  Begin, sessid %d, esm_hdl %d\n", servp->sessId, handle));

	msglen = sizeof(ndmp_NFXRsp_t) + sizeof(ndmpv2_data_get_state_rsp) - 1;
	NDMP_FP_ALLOC(rsp, ndmp_NFXRsp_t, msglen, "nfxrsp DATA_GS");
	if (rsp == NULL)
	{
        ndmp_sendCmdRsp(req, dst_port, NDMP_UNDEFINED_ERR, 0);
        return esm_getCurrState(handle);
	}
	gs_rsp = (ndmpv2_data_get_state_rsp *) rsp->content;

    rsp->hdr.sessId  = req->hdr.sessId;
    rsp->hdr.mid     = req->hdr.mid;
    rsp->ndmp_mid    = req->ndmp_mid; 
	memcpy(&rsp->cl_hdr, &req->cl_hdr, sizeof(ndmp_Hdr));

	fill_Data_v2_get_state_info (data, gs_rsp, handle);

	eee_sendMessage((void *) rsp, msglen, dst_port, ndmp_NNIMCtrl.src_port);
	NDMP_FP_FREE(rsp, "nfxrsp DATA_GS");
	NDMP_FP_FREE(req, "msgp DATA_GS");		/* it was originally msgp in ndmp_NNIMReceiveMsg() */
    return nextState;

} /* ndmp_Data_v2_get_state */
/*
 *------------------------------------------------------------------------------
 * Function Name:   ndmp_Data_v2_op_comp
 *
 * Description:		Signalled by FS backup/recovery operation completes.
 *					So Data server will send a HALT NOTIFICATION to NDMP
 *					client.
 *
 * Created by:      Anh Vu
 *
 * Data Created:    10/29/01
 *
 * Returns:
 *------------------------------------------------------------------------------
 */
int
ndmp_Data_v2_op_comp(esm_handle_t handle,
                    int32       nextState, /* next state from table       */
                    esm_event_t thisEvent, /* event driving this action   */
                    void        *userInfo)  /* ctrl blk linked to sm          */
{
	NdmpNIMService_t 			*servp = (NdmpNIMService_t *) userInfo;

    NDMP_DBG_PRINTF(D_INFO, ("ndmp_Data_op_comp:  Begin, sessid %d, esm_hdl %d\n", servp->sessId, handle));
	
	return ndmp_Data_op_comp(handle, nextState, thisEvent, userInfo);  

} /* ndmp_Data_v2_op_comp */

/* *------------------------------------------------------------------------------
 * Function Name:   ndmp_Data_v2_abort
 *
 * Description:
 *
 * Created by:      Anh Vu
 *
 * Data Created:    9/17/01
 *
 * Returns:
 *------------------------------------------------------------------------------
 */
int
ndmp_Data_v2_abort(esm_handle_t handle,
                    int32       nextState, /* next state from table       */
                    esm_event_t thisEvent, /* event driving this action   */
                    void        *userInfo)  /* ctrl blk linked to sm          */
{
    NdmpNIMService_t            *servp = (NdmpNIMService_t *) userInfo;

    NDMP_DBG_PRINTF(D_INFO, ("ndmp_Data_v2_abort:  Begin, sessid %d, esm_hdl %d\n", servp->sessId, handle));

	return ndmp_Data_abort(handle, nextState, thisEvent, userInfo);

} /* ndmp_Data_v2_abort */

/*
 *------------------------------------------------------------------------------
 * Function Name:   ndmp_Data_v2_stop
 *
 * Description:
 *
 * Created by:      Anh Vu
 *
 * Data Created:    9/17/01
 *
 * Returns:
 *------------------------------------------------------------------------------
 */
int
ndmp_Data_v2_stop(esm_handle_t handle,
                    int32       nextState, /* next state from table       */
                    esm_event_t thisEvent, /* event driving this action   */
                    void        *userInfo)  /* ctrl blk linked to sm          */
{
    NdmpNIMService_t            *servp = (NdmpNIMService_t *) userInfo;


    NDMP_DBG_PRINTF(D_INFO, ("ndmp_Data_stop:  Begin, sessid %d, esm_hdl %d\n", servp->sessId, handle));

	return ndmp_Data_stop(handle, nextState, thisEvent, userInfo); 

} /* ndmp_Data_v2_stop */
/*
 *------------------------------------------------------------------------------
 * Function Name:   ndmp_Data_v2_noop
 *
 * Description:
 *
 * Created by:      Anh Vu
 *
 * Data Created:    9/17/01
 *
 * Returns:
 *------------------------------------------------------------------------------
 */
int
ndmp_Data_v2_noop(esm_handle_t handle,
                    int32       nextState, /* next state from table       */
                    esm_event_t thisEvent, /* event driving this action   */
                    void        *userInfo)  /* ctrl blk linked to sm          */
{
    NDMP_DBG_PRINTF(D_INFO, ("ndmp_Data_v2_noop\n"));
    return nextState;

} /* ndmp_Data_v2_noop */

/*
 *------------------------------------------------------------------------------
 * Function Name:   ndmp_Data_err
 *
 * Description:
 *
 * Created by:      Anh Vu
 *
 * Data Created:    9/17/01
 *
 * Returns:
 *------------------------------------------------------------------------------
 */
int
ndmp_Data_v2_err(esm_handle_t handle,
                    int32       nextState, /* next state from table       */
                    esm_event_t thisEvent, /* event driving this action   */
                    void        *userInfo)  /* ctrl blk linked to sm          */
{
	ndmp_NFXReq_t *msg = (ndmp_NFXReq_t *) thisEvent.p1.u.p;
	NdmpNIMService_t *servp = (NdmpNIMService_t *) userInfo;
    int32       dst_port = servp->ssc_src_port;
    int32       errCode, currState;

    NDMP_DBG_PRINTF(D_INFO, ("ndmp_Data_v2_err:  Begin, sessid %d, esm_hdl %d\n", servp->sessId, handle));

    currState = esm_getCurrState(handle);

    NDMP_DBG_PRINTF(D_INFO, ("ndmp_Data_v2_err:  event %s, cur_state %s\n", 
		(thisEvent.event ==NDMP_DATA_V2_EVENT_START_OPS)? "START_OPS":
		(thisEvent.event ==NDMP_DATA_V2_EVENT_FS_OPS_RSP)? "FS_OPSRSP":
		(thisEvent.event ==NDMP_DATA_V2_EVENT_IO_REQ)? "IO_REQ":
		(thisEvent.event ==NDMP_DATA_V2_EVENT_IO_DONE)? "IO_DONE":
		(thisEvent.event ==NDMP_DATA_V2_EVENT_GET_ENV)? "GET_ENV":
		(thisEvent.event ==NDMP_DATA_V2_EVENT_GET_STATE)? "GET_STATE":
		(thisEvent.event ==NDMP_DATA_V2_EVENT_SND_INFO)? "SND_INFO":
		(thisEvent.event ==NDMP_DATA_V2_EVENT_ABORT)? "ABORT":
		(thisEvent.event ==NDMP_DATA_V2_EVENT_STOP)? "STOP":
		(thisEvent.event ==NDMP_DATA_V2_EVENT_INTERNAL_ERR)? "INT_ERR": 
		(thisEvent.event ==NDMP_DATA_V2_EVENT_CONN_ERR)? "CONN_ERR": "UNKNOWN",

		(currState==NDMP_DATA_V2_STATE_IDLE)? "IDLE":
		(currState==NDMP_DATA_V2_STATE_ACTIVE)? "ACTIVE":
		(currState==NDMP_DATA_V2_STATE_HALTED)? "HALTED": "UNKNOWN"));
    
    errCode = NDMP_ILLEGAL_STATE_ERR;
    
    switch (thisEvent.event)
    {
        case NDMP_DATA_V2_EVENT_START_OPS:
            if (currState != NDMP_DATA_V2_STATE_IDLE) 
                ndmp_sendCmdRsp(msg, dst_port, errCode, 0);
            break;
        case NDMP_DATA_V2_EVENT_GET_ENV:
            if (currState == NDMP_DATA_V2_STATE_IDLE) 
                ndmp_sendCmdRsp(msg, dst_port, errCode, 0);
            break;
        case NDMP_DATA_V2_EVENT_ABORT:
            if (currState == NDMP_DATA_V2_STATE_IDLE)        
                ndmp_sendCmdRsp(msg, dst_port, errCode, 0);
            break;
        case NDMP_DATA_V2_EVENT_STOP:
            if (currState != NDMP_DATA_V2_STATE_HALTED)      
                ndmp_sendCmdRsp(msg, dst_port, errCode, 0);
            break;
		default:
			NDMP_DBG_PRINTF(D_INFO, ("ndmp_Data_v2_err:  Invalid event %d\n", thisEvent.event));
    }
	NDMP_FP_FREE(msg, "msgp");   /* it was originally msgp in ndmp_NNIMReceiveMsg() */
    return nextState;

} /* ndmp_Data_v2_err */

/*-----------------------------------------------------------------
 * Name:        ndmp_Data_v2_bad_state
 *
 * Description: sm action function to handle bad state transitions
 *
 * created by : Anh Vu
 *
 * Date Created: 10/28/01
 *-----------------------------------------------------------------
 */

int32
ndmp_Data_v2_bad_state(esm_handle_t handle,
                    int32       nextState,
                    esm_event_t thisEvent,
                    void        * userInfo)
{

    NDMP_DBG_PRINTF(D_INFO, ("NDMP Data SM[%d]: BAD STATE transition event %d nextState %d",
              handle, thisEvent.event, nextState));

    return nextState;

} /* ndmp_Data_v2_bad_state */


