/* vi:ts=4
 *-----------------------------------------------------------------
 *
 * Name:        ndmp-msg-tbl.c
 *
 * RCS:  $Id: 
 *
 * Copyright (C) 2001, Agile Storage Inc.
 *
 * Description:	Contains table of messages received on the N-NIM
 *			Table entries points to the correct event for the
 *			correct state machine according to current version of NDMP
 *			protocol.
 *
 * Created by: Anh Vu
 *
 * Date Created:        04/08/02
 *
 *-----------------------------------------------------------------
 */

#include <nfx-incl.h>
#include <exports/eee-api.h>
#include <exports/malloc-api.h>
#include <exports/cm-api.h>
#include "ndmp.h"
#include "ndmp-fs-api.h"
#include "ndmp-mover-sm.h"
#include "ndmp-data-sm.h"
#include "ndmp-data-v2-sm.h"
#include <exports/fs-ndmp-api.h>
#include "ndmp-debug.h"


/* Possible msgs received on NNIM side */
/* Each message received is associated w/ an event and a state
 * machine to be queued to.
 *
 * Entries:
 * -------
  	MsgId, 
 	v2-Event, 
 	v3-Event, 
 	which State Machine, 
 	Message Id Name for debug,
	enum in stat tbl to update.
	orig (SSC/FP)
 */


ndmpNNIMMsgEnt_t ndmpNNIMMsgTbl[] =
{
    { NDMP_MID_NEW_SESSION,     0, 0, 							 0, "NDMP_MID_NEW_SESSION", 
			SSC_NEW_SESSION_REQ_RCVD, -1, SSC_CARD },
    { NDMP_MID_CLOSE_SESSION,   0, 0, 							 0, "NDMP_MID_CLOSE_SESSION", 
			SSC_CLOSE_SESSION_REQ_RCVD, -1, SSC_CARD },
    { NDMP_MID_UI_DM_GET_STATE, 0, 0,       					 0, "NDMP_MID_UI_DM_GET_STATE", 
			SSC_UI_DM_GETSTATE_REQ_RCVD, -1, SSC_CARD },
    { NDMP_MID_REL_RSRC,        0, 0,       					 0, "NDMP_MID_REL_RSRC", 
			REL_RSRC_REQ_RCVD, -1, FP_CARD },

    // From FS responding to a NDMP_START_bkup/restore req from NDMP client
    { FS_REQ_DUMP_SETUP,    	NDMP_DATA_V2_EVENT_FS_OPS_RSP,	NDMP_DATA_EVENT_FS_OPS_RSP,    
			DATA_SM, "FS_REQ_DUMP_SETUP", FS_DMP_SETUP_RSP_RCVD, -1, FP_CARD },
    { FS_REQ_DUMP_NAMES,    	NDMP_DATA_V2_EVENT_FS_OPS_RSP,	NDMP_DATA_EVENT_FS_OPS_RSP,    
			DATA_SM, "FS_REQ_DUMP_NAMES", FS_DMP_NAMES_RSP_RCVD, -1, FP_CARD },
    { FS_REQ_DUMP_START,    	NDMP_DATA_V2_EVENT_FS_OPS_RSP,	NDMP_DATA_EVENT_FS_OPS_RSP,    
			DATA_SM, "FS_REQ_DUMP_START", FS_DMP_START_RSP_RCVD, -1, FP_CARD },
    { FS_REQ_REST_SETUP,    	NDMP_DATA_V2_EVENT_FS_OPS_RSP,	NDMP_DATA_EVENT_FS_OPS_RSP, 
			DATA_SM, "FS_REQ_REST_SETUP", FS_RST_SETUP_RSP_RCVD, -1, FP_CARD },
    { FS_REQ_REST_NAMES,    	NDMP_DATA_V2_EVENT_FS_OPS_RSP,	NDMP_DATA_EVENT_FS_OPS_RSP,	
			DATA_SM, "FS_REQ_REST_NAMES", FS_RST_NAMES_RSP_RCVD, -1, FP_CARD },
    { FS_REQ_REST_START,    	NDMP_DATA_V2_EVENT_FS_OPS_RSP,	NDMP_DATA_EVENT_FS_OPS_RSP,    
			DATA_SM, "FS_REQ_REST_START", FS_RST_START_RSP_RCVD, -1, FP_CARD },

	// FS tells us he's got our abort msg, we do nothing for this reply
    { FS_REQ_DUMP_ABORT,    	NDMP_DATA_V2_EVENT_FS_ABORT_RSP,	NDMP_DATA_EVENT_FS_ABORT_RSP,  	
			DATA_SM, "FS_REQ_DUMP_ABORT", FS_DMP_ABORT_RSP_RCVD, -1, FP_CARD },
    { FS_REQ_REST_ABORT,    	NDMP_DATA_V2_EVENT_FS_ABORT_RSP,	NDMP_DATA_EVENT_FS_ABORT_RSP,  
			DATA_SM, "FS_REQ_REST_ABORT", FS_RST_ABORT_RSP_RCVD, -1, FP_CARD },

    // From FS dump/restore
    { NDMP_MID_SEND_LOG_REQ,    NDMP_DATA_V2_EVENT_SND_INFO,	NDMP_DATA_EVENT_SND_INFO,    
			DATA_SM, "NDMP_MID_SEND_LOG_REQ", FS_SND_LOG_REQ_RCVD, -1, FP_CARD },

    // From FS dump program
    { NDMP_MID_SEND_FH_DIR_REQ,     NDMP_DATA_V2_EVENT_SND_INFO,	NDMP_DATA_EVENT_SND_INFO, 
			DATA_SM, "NDMP_MID_SEND_FH_DIR_REQ", FS_SND_FH_DIR_REQ_RCVD, -1, FP_CARD },
    { NDMP_MID_SEND_FH_NODE_REQ,    NDMP_DATA_V2_EVENT_SND_INFO,	NDMP_DATA_EVENT_SND_INFO, 
			DATA_SM, "NDMP_MID_SEND_FH_NODE_REQ", FS_SND_FH_NODE_REQ_RCVD, -1, FP_CARD },
    { NDMP_MID_SET_DUMPDATE_REQ,NDMP_DATA_V2_EVENT_SND_INFO,	NDMP_DATA_EVENT_SND_INFO,    
			DATA_SM, "NDMP_MID_SET_DUMPDATE_REQ", FS_SET_DMPDATE_REQ_RCVD, -1, FP_CARD },

    // FS request to read/write data to/from tape (LR) or Mover server over 
	// the data connection (RR)
    { NDMP_MID_READ_DATA_REQ,   NDMP_DATA_V2_EVENT_IO_REQ,		NDMP_DATA_EVENT_IO_REQ,  	 
			DATA_SM, "NDMP_MID_READ_DATA_REQ", FS_RD_DATA_REQ_RCVD, -1, FP_CARD },
    { NDMP_MID_WRITE_DATA_REQ,  NDMP_DATA_V2_EVENT_IO_REQ,		NDMP_DATA_EVENT_IO_REQ,  	 
			DATA_SM, "NDMP_MID_WRITE_DATA_REQ", FS_WR_DATA_REQ_RCVD, -1, FP_CARD },

    // FS tells us that bkup/restore ops is done
    { NDMP_MID_OPS_COMPLETE_REQ,NDMP_DATA_V2_EVENT_OPS_COMP,	NDMP_DATA_EVENT_OPS_COMP,    
			DATA_SM, "NDMP_MID_OPS_COMPLETE_REQ", FS_OPS_CMPL_REQ_RCVD, -1, FP_CARD },

    // DATA service commands from client relayed by SSC
    { NDMP_MID_DATA_LISTEN,         0,	NDMP_DATA_EVENT_LISTEN,      
			DATA_SM, "NDMP_MID_DATA_LISTEN", SSC_DATA_LISTEN_REQ_RCVD, SSC_DATA_LISTEN_RSP_SNT, SSC_CARD },
    { NDMP_MID_DATA_CONNECT,        0,	NDMP_DATA_EVENT_CONNECT,     
			DATA_SM, "NDMP_MID_DATA_CONNECT", SSC_DATA_CONN_REQ_RCVD, SSC_DATA_CONN_RSP_SNT, SSC_CARD },
    { NDMP_MID_DATA_START_BACKUP,   NDMP_DATA_V2_EVENT_START_OPS,	NDMP_DATA_EVENT_START_OPS,   
			DATA_SM, "NDMP_MID_DATA_START_BACKUP", SSC_DATA_START_BKUP_REQ_RCVD, SSC_DATA_START_BKUP_RSP_SNT, SSC_CARD},
    { NDMP_MID_DATA_START_RECOVER,  NDMP_DATA_V2_EVENT_START_OPS,	NDMP_DATA_EVENT_START_OPS,   
			DATA_SM, "NDMP_MID_DATA_START_RECOVER", SSC_DATA_START_RST_REQ_RCVD, SSC_DATA_START_RST_RSP_SNT, SSC_CARD},
    { NDMP_MID_DATA_SEND_NAME_REQ,  NDMP_DATA_V2_EVENT_RST_NAMES,	NDMP_DATA_EVENT_RST_NAMES,   
			DATA_SM, "NDMP_MID_DATA_SEND_NAME_REQ", SSC_DATA_RST_NAMES_REQ_RCVD, -1, SSC_CARD},

    { NDMP_MID_DATA_GET_ENV,        NDMP_DATA_V2_EVENT_GET_ENV,		NDMP_DATA_EVENT_GET_ENV,     
			DATA_SM, "NDMP_MID_DATA_GET_ENV", SSC_DATA_GETENV_REQ_RCVD, SSC_DATA_GETENV_RSP_SNT, SSC_CARD},
    { NDMP_MID_DATA_GET_STATE,      NDMP_DATA_V2_EVENT_GET_STATE,	NDMP_DATA_EVENT_GET_STATE,   
			DATA_SM, "NDMP_MID_DATA_GET_STATE", SSC_DATA_GETSTATE_REQ_RCVD, SSC_DATA_GETSTATE_RSP_SNT, SSC_CARD},
    { NDMP_MID_DATA_STOP,           NDMP_DATA_V2_EVENT_STOP,	NDMP_DATA_EVENT_STOP,        
			DATA_SM, "NDMP_MID_DATA_STOP", SSC_DATA_STOP_REQ_RCVD, SSC_DATA_STOP_RSP_SNT, SSC_CARD},
    { NDMP_MID_DATA_ABORT,          NDMP_DATA_V2_EVENT_ABORT,	NDMP_DATA_EVENT_ABORT,       
			DATA_SM, "NDMP_MID_DATA_ABORT", SSC_DATA_ABORT_REQ_RCVD, SSC_DATA_ABORT_RSP_SNT, SSC_CARD},

    // Mover service commands from client relayed by SSC, 
	// V2 does not have MOVER_CONNECT cmd.

    { NDMP_MID_MOVER_LISTEN,        NDMP_MOVER_EVENT_LISTEN,	NDMP_MOVER_EVENT_LISTEN,     
			MOVER_SM, "NDMP_MID_MOVER_LISTEN", SSC_MOVER_LISTEN_REQ_RCVD, SSC_MOVER_LISTEN_RSP_SNT, SSC_CARD},
    { NDMP_MID_MOVER_CONNECT,       0,	NDMP_MOVER_EVENT_CONNECT,    
			MOVER_SM, "NDMP_MID_MOVER_CONNECT", SSC_MOVER_CONN_REQ_RCVD, SSC_MOVER_CONN_RSP_SNT, SSC_CARD},
    { NDMP_MID_MOVER_CONTINUE,      NDMP_MOVER_EVENT_CONTINUE,	NDMP_MOVER_EVENT_CONTINUE,   
			MOVER_SM, "NDMP_MID_MOVER_CONTINUE", SSC_MOVER_CONT_REQ_RCVD, SSC_MOVER_CONT_RSP_SNT, SSC_CARD},
    { NDMP_MID_MOVER_READ,          NDMP_MOVER_EVENT_READ_DATA,		NDMP_MOVER_EVENT_READ_DATA,  
			MOVER_SM, "NDMP_MID_MOVER_READ", SSC_MOVER_RD_REQ_RCVD, SSC_MOVER_RD_RSP_SNT, SSC_CARD},
    { NDMP_MID_MOVER_SET_RECORD_SIZE, NDMP_MOVER_EVENT_SET_INFO,	NDMP_MOVER_EVENT_SET_INFO, 
			MOVER_SM, "NDMP_MID_MOVER_SET_RECORD_SIZE", SSC_MOVER_SET_RECSIZ_REQ_RCVD, SSC_MOVER_SET_RECSIZ_RSP_SNT, SSC_CARD},
    { NDMP_MID_MOVER_SET_WINDOW,    NDMP_MOVER_EVENT_SET_INFO,	NDMP_MOVER_EVENT_SET_INFO,   
			MOVER_SM, "NDMP_MID_MOVER_SET_WINDOW", SSC_MOVER_SET_WDW_REQ_RCVD, SSC_MOVER_SET_WDW_RSP_SNT, SSC_CARD},
    { NDMP_MID_MOVER_GET_STATE,     NDMP_MOVER_EVENT_GET_STATE,	NDMP_MOVER_EVENT_GET_STATE,  
			MOVER_SM, "NDMP_MID_MOVER_GET_STATE", SSC_MOVER_GET_STATE_REQ_RCVD, SSC_MOVER_GET_STATE_RSP_SNT, SSC_CARD},
    { NDMP_MID_MOVER_ABORT,         NDMP_MOVER_EVENT_ABORT,	NDMP_MOVER_EVENT_ABORT,      
			MOVER_SM, "NDMP_MID_MOVER_ABORT", SSC_MOVER_ABORT_REQ_RCVD, SSC_MOVER_ABORT_RSP_SNT, SSC_CARD},
    { NDMP_MID_MOVER_STOP,          NDMP_MOVER_EVENT_STOP,	NDMP_MOVER_EVENT_STOP,       
			MOVER_SM, "NDMP_MID_MOVER_STOP", SSC_MOVER_STOP_REQ_RCVD, SSC_MOVER_STOP_RSP_SNT, SSC_CARD},
    { NDMP_MID_MOVER_CLOSE,         NDMP_MOVER_EVENT_CLOSE,	NDMP_MOVER_EVENT_CLOSE,      
			MOVER_SM, "NDMP_MID_MOVER_CLOSE", SSC_MOVER_CLOSE_REQ_RCVD, SSC_MOVER_CLOSE_RSP_SNT, SSC_CARD},
    { NDMP_MID_TAPE_SPACE_RSP,  	NDMP_MOVER_EVENT_TAPE_OPS_DONE,	NDMP_MOVER_EVENT_TAPE_OPS_DONE, 
			MOVER_SM, "NDMP_MID_TAPE_SPACE_RSP", -1, -1, SSC_CARD},
    { 0, 0, 0, 0, "", 0 }
};

