/* 
 *-----------------------------------------------------------------
 *
 * Name:        ndmp-msg.c
 *
 * RCS:  $Id: ndmp-msg.c,v 1.49 2002/11/21 18:10:39 nitinj Exp $
 *
 * Copyright (C) 2001, Agile Storage Inc.
 *
 * Description: Process all messages received from the SSC to N-NIM>
 *
 * Created by: Anh Vu
 *
 * Date Created:        9/06/01
 *
 *-----------------------------------------------------------------
 */
#include <nfx-incl.h>
#include <exports/eee-api.h>
#include <exports/malloc-api.h>
#include <exports/timer-api.h>
#include <exports/cm-api.h>
#include "ndmp.h"
#include "ndmp-fs-api.h"
#include "ndmp-mover-sm.h"
#include "ndmp-data-sm.h"
#include "ndmp-data-v2-sm.h"
#include <exports/fs-ndmp-api.h>
#include "ndmp-debug.h"

ndmp_NNIMCtrl_t ndmp_NNIMCtrl;

/* Performance measure */
eeeRTC_t time_wr_req;
eeeRTC_t time_wr_rsp;
eeeRTC_t time_rd_req;
eeeRTC_t time_rd_rsp;
uint64   pm_ioCookie;		

int num_time_wr_req = 0;
int num_time_rd_req = 0;
int avg_time_wr_req = 0;
int avg_time_rd_req = 0;

int ndmp_fp_malloc = 0;
int ndmp_fp_free = 0;
int ndmp_fp_msgStats[MAX_FP_STAT];

int ndmp_closeSessOnNIM (int idx, sess_node_t *node, char *msgp);

#ifdef NFX_SMP
nfx_spinlock_t          fpstat_spin;
msgCntxt_t * saveMsgConTxt(uint32 msgtyp, int32 sessId, ndmpNNIMMsgEnt_t *msgent, 
				char *msgp, uint32 ssc_src_port);
int32 chkNProcMsgIn_SSCq(msg_q_t *rcv_q, nfx_spinlock_t *msgq_lock);
int32 chkNProcMsgIn_FPq(msg_q_t *rcv_q, nfx_spinlock_t *msgq_lock);
int32 ndmp_procMsg(msgCntxt_t *msgI);
void ndmp_procMsgThread(esm_handle_t handle, void *arg);

#define NUM_MSG_TO_PROC 5
#endif

/*------------------------------------------------------------------------------
 * Function Name:   ndmp_initModule
 *
 * Description:     Initializes the NDMP services on the N-NIM, ie. Data and Mover
 *
 * Created by:      Anh Vu
 *
 * Data Created:    9/17/01
 *
 * Returns:
 *
 *------------------------------------------------------------------------------
 */
int32
ndmp_initModule()
{
#ifdef NFX_SMP
    int32       i; 
#endif
    int32       rc;

    /*
     *--------------------------------------
     * Register with EEE and get app id
     *--------------------------------------
     */
	printf("ndmp_initModule\n");
	memset ( &ndmp_NNIMCtrl, 0, sizeof (ndmp_NNIMCtrl_t));

#ifdef NFX_SMP
	NFX_SPIN_INIT(&fpstat_spin, SPINLOCK_LEVEL_UNKNOWN);
	NFX_SPIN_INIT(&(ndmp_NNIMCtrl.spin), SPINLOCK_LEVEL_UNKNOWN);
	for (i =0; i < NDMP_MAX_NUM_SESS; i++)
	{
		ndmp_NNIMCtrl.sessLst[i].sessid = -1;
		NFX_SPIN_INIT(&(ndmp_NNIMCtrl.sessLst[i].fp_msgq_lock), SPINLOCK_LEVEL_UNKNOWN);
		NFX_SPIN_INIT(&(ndmp_NNIMCtrl.sessLst[i].ssc_msgq_lock), SPINLOCK_LEVEL_UNKNOWN);
		NFX_SPIN_INIT(&(ndmp_NNIMCtrl.sessLst[i].servp_lock), SPINLOCK_LEVEL_UNKNOWN);
	}
#endif

	if (ndmp_perf_meas)
	{
	    time_wr_req = *(eee_rtcZero());
	    time_wr_rsp = *(eee_rtcZero());
	    time_rd_req = *(eee_rtcZero());
	    time_rd_rsp = *(eee_rtcZero());
	}

    if ((rc = eee_allocateApplicationId("ndmp")) == NFX_ERR)
    {
        NDMP_DBG_PRINTF(D_INFO, ("ndmp_NNIMInit:  eeeAllocate app id failed.\n"));
        E_LOG(class_1, err_s, ndmp_NNIMCtrl.appId, 0, 0, 0,
						("ndmp_NNIMInit:  eeeAllocate app id failed.\n"));
        return NFX_ERR;
    }
	NDMP_DBG_PRINTF(D_INFO, ("ndmp_initModule:  ndmp_appId = %d\n", rc));	
	ndmp_NNIMCtrl.appId = rc;
#ifdef NFX_SMP
	ndmp_NNIMCtrl.src_port = PACK_UNICAST_PORT_ADDRESS(
											 nfxMySlotId,
											 NFX_NFP_FP_ANY,
                                             ndmp_NNIMCtrl.appId,
                                             EEE_CONTROL,
                                             NFX_COS_HIGH,
                                             PORT_BEST_FIT);
#else
	ndmp_NNIMCtrl.src_port = PACK_UNICAST_PORT_ADDRESS(
											 nfxMySlotId,
											 nfxMyCPUId,	
                                             ndmp_NNIMCtrl.appId,
                                             EEE_CONTROL,
                                             NFX_COS_HIGH,
                                             PORT_BEST_FIT);
#endif
    if (eee_registerReceiveFunc( ndmp_NNIMCtrl.appId,	/* App Id        */
                                  ndmp_NNIMReceiveMsg, 	/* Ucast Rx Func */
                                  NULL) != NFX_OK)     /* Mcast Rx Func */
    {
        NDMP_DBG_PRINTF(D_INFO, ("ndmp_initModule:  Register eee Rx func failed."));
		E_LOG(class_1, err_s, ndmp_NNIMCtrl.appId, 0, 0, 0,
        		("ndmp_initModule:  Register eee Rx func failed."));
        return NFX_ERR;
    }

    event_registerClient(NFX_EVENT_CPU, ndmp_NNIMProcessEvents);

#ifdef NFX_SMP
	/* Create and start our processing_msg_thread */
	ndmp_NNIMCtrl.msgThreadHdl = thread_create(&ndmp_NNIMCtrl);
	printf("threadHdl = %d\n", ndmp_NNIMCtrl.msgThreadHdl);
	NFX_ASSERT(ndmp_NNIMCtrl.msgThreadHdl != 0);
	NFX_ASSERT(thread_start(&(ndmp_NNIMCtrl.msgThreadHdl), ndmp_procMsgThread) == NFX_OK);
#endif

	return NFX_OK;

}  /* ndmp_initModule */

/*
 *------------------------------------------------------------------------------
 * Function Name:   ndmp_newSess
 *
 * Description:     This function is called when a new NDMP session is 
 *                  established on the SSC. This is where the Data and Mover 
 *					services control structures and state machine instances  
 *					are created.
 *
 * Created by:      Anh Vu 
 *
 * Data Created:    9/17/01
 *
 * Returns:         NFX_OK | NFX_ERR
 *
 *------------------------------------------------------------------------------
 */
int
ndmp_newSess (int idx, int32 sessId, uint32 ssc_src_port, int32 version) 
             
{
    NdmpNIMService_t *servp;
	sess_node_t	*node;

	/* In here we init sessid in our node w/ 1st msg from SSC 
	 *  because sessions are always first established by the SSC
	 */
	node = ndmp_add_sessid_entry(idx, sessId);

#ifndef NFX_SMP
	/* If node cannot be allocated, return err */
	if (node == NULL)
	{
		NDMP_DBG_PRINTF(D_INFO, ("ndmp_newSess:  Error add session node for sessid %d.\n", sessId));
		E_LOG(class_1, err_s, ndmp_NNIMCtrl.appId, 0, 0, 0,
					            ("ndmp_newSess:  Error add session node for sessid %d.\n", sessId));
		return(NFX_ERR);
	}
#endif
    /*
     * Allocate a new device control block
     */
    NDMP_FP_ALLOC(servp, NdmpNIMService_t, sizeof(NdmpNIMService_t), "servp");
    if( !servp )
	{
        NDMP_DBG_PRINTF(D_INFO, ("ndmp_newSess : Error eee_ramAlloc NdmpNIMService_t.\n"));
		E_LOG(class_1, err_s, ndmp_NNIMCtrl.appId, 0, 0, 0,
					            ("ndmp_newSess:  Error eee_ramAlloc NdmpNIMService_t.\n"));
        return(NFX_ERR);
	}
	NDMP_DBG_PRINTF(D_INFO, ("ndmp_newSess :  servp %p, sessId = %d\n", servp, sessId));
	servp->sessId = sessId;
	servp->sess_idx = idx;
	servp->version = version;

	servp->ssc_src_port = ssc_src_port;		// so we can send log, fh, or pause indication
#ifdef NFX_SMP
    servp->fp_src_port = PACK_UNICAST_PORT_ADDRESS(
                                            nfxMySlotId,
											NFX_NFP_FP_ANY,
                                            eee_getApplicationId("efs"), 
                                            EEE_CONTROL,
                                            NFX_COS_HIGH,
                                            PORT_BEST_FIT);
#else
    servp->fp_src_port = PACK_UNICAST_PORT_ADDRESS(
                                            nfxMySlotId,
                                            nfxMyCPUId,
                                            eee_getApplicationId("efs"), 
                                            EEE_CONTROL,
                                            NFX_COS_HIGH,
                                            PORT_BEST_FIT);
#endif
	NDMP_DBG_PRINTF(D_INFO, ("ndmp_newSess:  ssc_src_port = %d, fp src port = %d\n", 
									servp->ssc_src_port, servp->fp_src_port));

    /*
     * Create Mover state machine instance (used for 
     */
    if (ndmp_Mover_sm_create( &servp->Mover.my_smHdl, servp ) != NFX_OK)
	{
        /*
         * State machine instance not created so.....
         */
        NDMP_FP_FREE(servp, "servp");
        NDMP_DBG_PRINTF(D_INFO, (": Error creating Mover state machine.\n"));
		E_LOG(class_1, err_s, ndmp_NNIMCtrl.appId, 0, 0, 0,
					            ("ndmp_newSess:  Error creating Mover state machinen"));
        return NFX_ERR;
    }

	if (version == NDMP_VER_2)
	{
    	/* Create V2 Data state machine instance */
    	if (ndmp_Data_v2_sm_create( &servp->Data.my_smHdl, servp ) != NFX_OK)
		{
        	/* State machine instance not created so.....  */
        	NDMP_FP_FREE(servp, "servp");        /* free services data block memory */
        	NDMP_DBG_PRINTF(D_INFO, (": Error creating Data state machine.\n"));
			E_LOG(class_1, err_s, ndmp_NNIMCtrl.appId, 0, 0, 0,
					            ("ndmp_newSess:  Error creating Data state machinen"));
        	return NFX_ERR;
    	}
	} else { 		

    	/* Create V3 Data state machine instance */
    	if (ndmp_Data_sm_create( &servp->Data.my_smHdl, servp ) != NFX_OK)
		{
        	/* State machine instance not created so.....  */
        	NDMP_FP_FREE(servp, "servp");        /* free services data block memory */
        	NDMP_DBG_PRINTF(D_INFO, (": Error creating v2 Data state machine.\n"));
			E_LOG(class_1, err_s, ndmp_NNIMCtrl.appId, 0, 0, 0,
					            ("ndmp_newSess:  Error creating v2 Data state machinen"));
        	return NFX_ERR;
    	}
	}	

	/* Data and Mover keep the other's state machine hdl case needed */
	servp->Data.mover_smHdl = servp->Mover.my_smHdl;	
	servp->Mover.data_smHdl = servp->Data.my_smHdl;	
	node->servp = servp;

#ifdef NFX_SMP
	NFX_SPIN_INIT(&(servp->Data.spin), SPINLOCK_LEVEL_UNKNOWN);
	NFX_SPIN_INIT(&(servp->Mover.spin), SPINLOCK_LEVEL_UNKNOWN);
#endif
	NDMP_DBG_PRINTF(D_INFO, ("ndmp_newSess: Mover hdl = %d, Data hdl %d, "
							"node->servp = %p, node->sessId = %d\n", 
							servp->Mover.my_smHdl, servp->Data.my_smHdl, node->servp, node->sessid));
#if 0
	/* go thru and remove other msgs w/ diff sessid in our SSC rcv_msg q,
	* now that we got a brand new session, this is part of clean up
	* any previous lingering old msgs, if any.
	*/
	NFX_SPIN_LOCK(&node->ssc_msgq_lock);
	q_headNodePtr_t msg_q = node->ssc_rcv_q->msg_q;

	if (q_resetReadPointer(msg_q) != NFX_OK)
	{
		NFX_SPIN_UNLOCK(msgq_lock);
		return NFX_ERR;
	}   
	if ((msg_q->size > 0) && msg_q->next)
	{
		seq_id = msg_q->next->item1;
		msgI = (msgCntxt_t *) msg_q->next->item2;
		if ((msgI->msgtyp == ) && (msgI->sessid == sessId))
		{
			/* ok, proceed from here */
			q_readNextQueueItem();
		}
		req = (ndmp_NFXReq_t *) msgI->msgp;
	while ((rc = q_deleteQueueItem(*msg_q, &seq_id, &msgI)) != Q_ERR_EMPTY_QUEUE)
	{   
		//printf("freeing msgI %p, seqId %lld \n", msgI, seq_id);
		//            NDMP_FP_FREE(msgI, "msgI del_sessid_entry");
	}
#endif
    return( NFX_OK );

} /* ndmp_newSess */
/*
 *------------------------------------------------------------------------------
 * Function Name:   ndmp_relSessRsrc
 *
 * Description:     This function release session resources (for both Data & Mover)
 * 
 * Created by:      Anh Vu
 *
 * Data Created:    11/30/01
 *
 * Returns:         NFX_OK | NFX_ERR
 *
 *------------------------------------------------------------------------------
 */
#ifdef NFX_SMP
int 
ndmp_relSessRsrc(NdmpNIMService_t *servp)
{
	 sess_node_t *node;
	 int idx, msglen, rc;
	 ndmp_NFXRsp_t          *rsp;
#else
int 
ndmp_relSessRsrc(int idx, sess_node_t *node)
{
    NdmpNIMService_t *servp;
   
	if ( (!node) || !(node->servp))
	{
		NDMP_DBG_PRINTF(D_INFO, ("ndmp_relSessRsrc:  invalid session, idx %d, ignore.\n", idx)); 
		E_LOG(class_1, err_s, ndmp_NNIMCtrl.appId, 0, 0, 0,
					            ("ndmp_relSessRsrc:  invalid session, idx %d, ignore.", idx));
		return NFX_ERR;
	} else
		servp = node->servp;
#endif

	NDMP_DBG_PRINTF(D_INFO, ("ndmp_relSessRsrc:  begin\n"));
	if (servp->Data.my_smHdl)
	{
		ndmp_data_rel_resource(servp);
		if (servp->version == NDMP_VER_2)
		{
			ndmp_Data_v2_sm_destroy(servp->Data.my_smHdl);
		} else 
			ndmp_Data_sm_destroy(servp->Data.my_smHdl);
	}

	if (servp->Mover.my_smHdl)
	{
		ndmp_mover_release_resource(&servp->Mover);
		ndmp_Mover_sm_destroy(servp->Mover.my_smHdl);
	}

#ifdef NFX_SMP
	node = ndmp_find_sessid_entry(servp->sessId, &idx);

	/* Clean out the SSC msg rcvd Q */
	cleanup_msg_q(&node->ssc_msgq_lock, &node->ssc_rcv_q);
	//printf("after cleanup:  msgcnt %d, nxtseqExp %lld\n", 
	//					node->ssc_rcv_q.msgq_cnt, node->ssc_rcv_q.nxt_seq_expected);

	/* Clean out the FP msg rcvd Q */
	cleanup_msg_q(&node->fp_msgq_lock, &node->fp_rcv_q);
	//printf("after cleanup:  msgcnt %d, nxtseqExp %lld\n", node->fp_rcv_q.msgq_cnt, node->fp_rcv_q.nxt_seq_expected);
	
	ndmp_del_sessid_entry(idx, servp->sessId);

	/* Send a DELETE response msg to SSC */

	msglen = sizeof(ndmp_NFXRsp_t);
	NDMP_FP_ALLOC(rsp, ndmp_NFXRsp_t, msglen, "nfxrsp relSessRsrc");
    if (rsp == NULL)
    {
		NDMP_DBG_PRINTF(D_INFO,("ndmp_relSessRsrc:  malloc err for nfxrsp, sessid %d.\n", 
								servp->sessId));  
		E_LOG(class_1, err_s, ndmp_NNIMCtrl.appId, 0, 0, 0,
				("ndmp_relSessRsrc:  malloc err for nfxrsp, sessid %d.\n", servp->sessId)); 			
        return NFX_ERR;
    }
    rsp->hdr.sessId  = servp->sessId;
    rsp->hdr.mid     = NDMP_MID_CLOSE_SESSION_RSP; 
	
	rc = eee_sendMessage((void *) rsp, msglen, servp->ssc_src_port, ndmp_NNIMCtrl.src_port);
    if (rc != NFX_OK) 
    {
        NDMP_DBG_PRINTF(D_INFO, ("ndmp_relSessRsr: eee_sendMessage failed! sessid %d\n", servp->sessId));
        E_LOG(class_1, err_s, ndmp_NNIMCtrl.appId, 0, 0, 0,
               ("ndmp_relSessRsrc: eee_sendMessage failed! sessid %d\n", servp->sessId));
    } else {
		//printf("ndmp_relSessRsrc:  Close_Rsp sent fine to ssc port %d\n", servp->ssc_src_port);
		ATOMIC_INC_INT(&ndmp_fp_msgStats[SSC_CLOSE_SESSION_RSP_SNT], 1);
	}

    NDMP_FP_FREE(servp, "servp");
#else
	ndmp_del_sessid_entry(idx, node->sessid);
#endif
	return NFX_OK;
} /* ndmp_relSessRsrc */

/*
 *------------------------------------------------------------------------------
 * Function Name:   ndmp_closeSessOnNIM
 *
 * Description:     This function is called when the NDMP control session is
 *                  closed on the SSC. This is where the Data and Mover
 *                  services control structures and state machine instances
 *                  are deleted.
 * 
 * Created by:      Anh Vu
 *
 * Data Created:    11/30/01
 *
 * Returns:         NFX_OK | NFX_ERR
 *
 *------------------------------------------------------------------------------
 */
int
ndmp_closeSessOnNIM (int idx, sess_node_t *node, char *msgp)

{
    NdmpNIMService_t *servp;
	Data_t			 *data;
    esm_event_t   	 esmEvent;
	int32			 rc, cur_st, sessId;

	if ((node == NULL) || (idx == -1))
	{
    	NDMP_DBG_PRINTF(D_INFO, ("ndmp_closeSessOnNIM:  Invalid session to close, NULL node and idx."));
    	E_LOG(class_1, err_s, ndmp_NNIMCtrl.appId, 0, 0, 0,
						("ndmp_closeSessOnNIM:  Invalid session to close, NULL node and idx."));
		return NFX_ERR;
	}
	sessId =  node->sessid;
	servp = node->servp;

    NDMP_DBG_PRINTF(D_INFO, ("ndmp_closeSessOnNIM:  Begin.  Req Idx = %d, SessId = %d\n", idx, sessId));

	/* If session does not exist on NNIM yet, return */
	if (servp == NULL)
	{
		NDMP_DBG_PRINTF(D_INFO, ("Session %d does not exist, no close done.\n", sessId));
		E_LOG(class_1, err_s, ndmp_NNIMCtrl.appId, 0, 0, 0,
						("Session %d does not exist, no close done.\n", sessId));
		NDMP_FP_FREE(msgp, "msgp CLOSESESS");
		return NFX_OK;	
	}

	data = &servp->Data;

	if (data && (data->addr_type == NDMP_ADDR_LOCAL))
	{ 
		/* For local backup, Data Server initiates connection
	 	 * on Mover Server.  So if we're in idle state, releasing resources
	 	 * for the session is sufficient.  If we're not in idle state,
	 	 * send an abort to Data Server and it'll start the abort
	 	 * with client who will eventually abort Mover Server for us.
		 * After the abort is done, Data server will notify
		 * us when it's done so we can release resources for 
		 * this session.
		 */	
		cur_st = esm_getCurrState(servp->Data.my_smHdl);	
		if (cur_st == NDMP_DATA_STATE_IDLE) 
		{
#ifdef NFX_SMP
			ndmp_relSessRsrc (servp);
#else
			ndmp_relSessRsrc (idx, node);
#endif

			NDMP_DBG_PRINTF(D_INFO, ("ndmp_closeSessOnNIM:  Data Server is in IDLE state, session %d deleted on NNIM.\n", sessId));
            NDMP_FP_FREE(msgp, "msgp CLOSESESS");
			return NFX_OK;	

		} else {
       		NDMP_DBG_PRINTF(D_INFO, 
							("ndmp_closeSessOnNIM:  Deleting SessId %d -- firt post ABORT event %d to SM hdl %d",
                      sessId, esmEvent.event, servp->Data.my_smHdl));
       		E_LOG(class_1, err_s, ndmp_NNIMCtrl.appId, 0, 0, 0,
							("ndmp_closeSessOnNIM:  Deleting SessId %d -- firt post ABORT event %d to SM hdl %d",
                      sessId, esmEvent.event, servp->Data.my_smHdl));
            bzero(&esmEvent, sizeof(esm_event_t));
    		esmEvent.event = (servp->version == NDMP_VER_2) ? 
						NDMP_DATA_V2_EVENT_ABORT : NDMP_DATA_EVENT_ABORT;
    		esmEvent.p1.u.p = msgp;
    		rc = esm_deliverEvent(servp->Data.my_smHdl, &esmEvent);
    		if (rc != NFX_OK)
    		{
         		NDMP_DBG_PRINTF(D_INFO, ("ndmp_closeSessOnNIM:  SessId %d -- ERROR, cannot post event %d to SM hdl %d",
                      sessId, esmEvent.event, servp->Data.my_smHdl));
         		E_LOG(class_1, err_s, ndmp_NNIMCtrl.appId, 0, 0, 0,
					  ("ndmp_closeSessOnNIM:  SessId %d -- ERROR, cannot post event %d to SM hdl %d",
                      sessId, esmEvent.event, servp->Data.my_smHdl));
    		}
		}
	}
	return NFX_OK;
}
/*
 *------------------------------------------------------------------------------
 * Function Name:   ndmp_UIGetDMStateOnNIM
 *
 * Description:     This function is called to get Data and Mover states
 *					quickly without queueing a separate DATA_GET_STATE
 *					and MOVER_GET_STATE message to each of these state machines.
 *                  We need to reply to user quickly just in case he decides
 *					to repeatedly asking for these states. 
 * 
 * Created by:      Anh Vu
 *
 * Data Created:    11/30/01
 *
 * Returns:         NFX_OK | NFX_ERR
 *
 *------------------------------------------------------------------------------
 */
int
ndmp_UIGetDMStateOnNIM (sess_node_t *node, 
						char *msgp, 
						uint32 ssc_src_port) 
{
    NdmpNIMService_t *servp;
	int32			 rc = NFX_OK;
	uint32			 src_port;
	ndmp_NFXReq_t 	 *req = (ndmp_NFXReq_t *)msgp;
	Data_t			 *data; 
	Mover_t			 *mover;
	ndmp_NFXUI_GetState_Rsp_t rsp;
	int32			 sessId = req->hdr.sessId;

    if (!req)
    {
        NDMP_DBG_PRINTF(D_INFO, ("ndmp_UIGetDMStateOnNIM: req NULL\n"));
        E_LOG(class_1, err_s, ndmp_NNIMCtrl.appId, 0, 0, 0,
						("ndmp_UIGetDMStateOnNIM: req NULL\n"));
        return NFX_ERR;
    }

    NDMP_DBG_PRINTF(D_INFO, ("ndmp_UIGetDMStateOnNIM: Begin. SessId = %d\n", req->hdr.sessId));

	rsp.hdr.sessId 	= req->hdr.sessId;
	rsp.hdr.mid     = req->hdr.mid;
	rsp.hdr.clientHdl = req->hdr.clientHdl;	/* returns cookie so requester will 
											 * know which response this is 
											 */

	if ( (!node) || !(node->servp) )
	{
		sessId = *((int32 *)(msgp + sizeof(uint32)));
		NDMP_DBG_PRINTF(D_INFO, ("ndmp_UIGetDMStateOnNIM: Session %d does not exist, noop .\n", sessId));
		E_LOG(class_1, err_s, ndmp_NNIMCtrl.appId, 0, 0, 0,
					   	("ndmp_UIGetDMStateOnNIM: Session %d does not exist, noop .\n", sessId));
		/* Send reply back to this request */
		rsp.error = NDMP_UNDEFINED_ERR;
		src_port = ssc_src_port;
		rc = NFX_ERR;
	} else {
		servp = node->servp;
		sessId = node->sessid;
		data = &servp->Data;
		mover = &servp->Mover;

		rsp.error = NDMP_NO_ERR;
		
		fill_Data_get_state_info (data, &rsp.data_state, data->my_smHdl);
		fill_Mover_get_state_info (mover, &rsp.mover_state, mover->my_smHdl);
		rsp.mover_mode = mover->mode;	/* extra info for Mover UI */
		src_port = servp->ssc_src_port;
	}
	eee_sendMessage((void *) &rsp, sizeof(rsp), src_port, ndmp_NNIMCtrl.src_port);	

	return rc;
}

#if 0 
/*
 *------------------------------------------------------------------------------
 * Function Name:   ndmp_send_fs_names_list
 *
 * Description:     
 *
 * Created by:      Nitin Jain
 *
 * Data Created:    
 *
 *------------------------------------------------------------------------------
 */
int32
ndmp_send_fs_names_list(sess_node_t *node, char *msgp, uint32 ssc_src_port) 
{
    NdmpNIMService_t *servp = node->servp;
	int32			 rc = NFX_OK;
	ndmp_NFXReq_t 	 *req = (ndmp_NFXReq_t *)msgp;
	Data_t			 *data = &servp->Data;
	ndmp_NFXSendNamesReq_t *nameReq = (ndmp_NFXSendNamesReq_t *)req->content;;
	int32			 sessId = req->hdr.sessId;

    if (!req)
    {
        NDMP_DBG_PRINTF(D_INFO, ("ndmp_send_fs_names_list: req NULL\n"));
        E_LOG(class_1, err_s, ndmp_NNIMCtrl.appId, 0, 0, 0,
						("ndmp_send_fs_names_list: req NULL\n"));
        return NFX_ERR;
    }

    NDMP_DBG_PRINTF(D_INFO, ("ndmp_send_fs_names_list: Begin. SessId = %d\n", req->hdr.sessId));

	if ( (!node) || !(node->servp) )
	{
		sessId = *((int32 *)(msgp + sizeof(uint32)));
		NDMP_DBG_PRINTF(D_INFO, ("ndmp_send_fs_names_list: Session %d does not exist, noop .\n", sessId));
		E_LOG(class_1, err_s, ndmp_NNIMCtrl.appId, 0, 0, 0,
					   	("ndmp_send_fs_names_list: Session %d does not exist, noop .\n", sessId));
		/* Send reply back to this request */
        ndmp_doBkupRecovErr(servp, NDMP_UNDEFINED_ERR);
        return NFX_OK;
	} else {
		servp = node->servp;
		sessId = node->sessid;
        servp->moreNames = req->userInfo;
        if (ndmp_save_flist(&nameReq->data[0], nameReq->num_files, data) == NFX_ERR)
        {
            NDMP_DBG_PRINTF(D_INFO, ("ndmp_send_fs_names_list: save_flist failed, sessId %d.\n", servp->sessId));
            E_LOG(class_1, err_s, ndmp_NNIMCtrl.appId, 0, 0, 0, 
                        ("ndmp_send_fs_names_list: save_flist failed, sessId %d.\n", servp->
sessId));
            return ndmp_doBkupRecovErr(servp, NDMP_UNDEFINED_ERR);
        }
        data->flist_cur_indx = 0;

        ndmp_sndNamesReq(servp, data->flist_cur_indx);
        data->start_ops_state = NDMP_OPS_STATE_REST_NAMES_SENT;
	}

	return rc;
}
#endif
/*
 *------------------------------------------------------------------------------
 * Function Name:   ndmp_NNIMProcessEvents
 *
 * Description:     
 *
 * Created by:      Nitin Jain
 *
 * Data Created:    
 *
 *------------------------------------------------------------------------------
 */
void ndmp_NNIMProcessEvents(nfx_eventMsg_t *evt)
{
    NDMP_DBG_PRINTF(D_INFO, ("ndmp_NNIMProcessEvents: Begin\n"));

    if (evt->param.cpuEvent.state == EVENT_STATE_DOWN)
    {
        switch(evt->param.cpuEvent.slot)
        {
            case 1:
                /*
                 * This should never happen on FP
                 */
                NDMP_DBG_PRINTF(D_INFO, ("ndmp_NNIMProcessEvents: FP-down event\n"));
                E_LOG(class_1, notice_s, ndmp_NNIMCtrl.appId, 0, 0, 0,
								("ndmp_NNIMProcessEvents: FP-down event\n"));
                break;

            case 2:
                /*
                 * FC down. We don't do anything, FS will detect the failure
                 * and get start the abort procedure.
                 */
                NDMP_DBG_PRINTF(D_INFO, ("ndmp_NNIMProcessEvents: SP-down event\n"));
                E_LOG(class_1, notice_s, ndmp_NNIMCtrl.appId, 0, 0, 0,
								("ndmp_NNIMProcessEvents: SP-down event\n"));
                break;

            case 0:
            {
#ifndef NFX_SMP
                ndmp_NFXReq_t *req = NULL;
                int32 i;
                sess_node_t *np = NULL;
#endif
                /*
                 * SSC down. We have to treat it as kill session. Since we have
                 * lost connection to client, just clean FS and NDMP.
                 */
#ifdef NFX_SMP
				NFX_SPIN_LOCK(&ndmp_NNIMCtrl.spin);
				// ... some stuff here to be done
				NFX_SPIN_UNLOCK(&ndmp_NNIMCtrl.spin);
#else
                for (i = 0; i < NDMP_MAX_SESS; i++)
                {
                    for (np = ndmp_NNIMCtrl.sessLst[i]; np != NULL; np=np->nxt)
                    {
                        NDMP_FP_ALLOC(req, ndmp_NFXReq_t, sizeof(ndmp_NFXReq_t), "ndmp_NFXReq_t");
                        req->hdr.mid    = NDMP_MID_CLOSE_SESSION;
                        req->hdr.sessId = np->sessid;
                        req->content[0] = i;
                        if (ndmp_closeSessOnNIM(i, np, (char *)req) == NFX_ERR)
                            NDMP_FP_FREE(req, "ndmp_NFXReq_t");
                        /* NDMP close session will free the req */
                    }
                }
#endif
                break;
             }
        }
    }

    NDMP_DBG_PRINTF(D_INFO, ("ndmp_NNIMProcessEvents: End\n"));
    return;
}
#ifdef NFX_SMP
/*
 *------------------------------------------------------------------------------
 * Function Name:   ndmp_NNIMReceiveMsg
 *
 * Description:     Sends cmd response event to the Data and Mover 
 *                  state machines.
 *
 * Created by:      Anh Vu
 *
 * Data Created:    9/17/01
 *
 *------------------------------------------------------------------------------
 */
void
ndmp_NNIMReceiveMsg(eee_desc_t *eDesc)
{
	uint32        buffSize, *msgtyp, ssc_src_port;
	int32		  rc, sessId, idx;
	char		  		*msgp; 
	ndmpNNIMMsgEnt_t   	*msgent;
	sess_node_t			*node = NULL;
	msgCntxt_t			*msgI;
	uint64				seq_id;
	int32				ssc_msgcnt, fp_msgcnt;


    if( !eDesc ) {
        NDMP_DBG_PRINTF(D_INFO, ("ndmp_NNIMReceiveMsg: Null eDesc\n"));
        E_LOG(class_1, err_s, ndmp_NNIMCtrl.appId, 0, 0, 0,
						("ndmp_NNIMReceiveMsg: Null eDesc\n"));
        return;
    }

	buffSize = eDesc->hdr.control & EDESC_TOTAL_LEN;
    /*
     * convert EEE packet to flat buffer
     */
    NDMP_FP_ALLOC(msgp, char, buffSize, "msgp");
    NFX_ASSERT(msgp != NULL);

	if ((rc = eee_copyPacketToLocalBuffer(msgp, eDesc, buffSize, 0)) != buffSize)
    {
        NDMP_DBG_PRINTF(D_INFO, ("ndmp_NNIMReceiveMsg: error flattening EEE packet (expected %d got %d)",
                  buffSize, rc));
        E_LOG(class_1, err_s, ndmp_NNIMCtrl.appId, 0, 0, 0,
				("ndmp_NNIMReceiveMsg: error flattening EEE packet (expected %d got %d)",
                 buffSize, rc));
        eee_freePacket(eDesc);
		NDMP_FP_FREE(msgp, "msgp")
        return;
    }
	ssc_src_port = eDesc->hdr.src_port;
	eee_freePacket(eDesc);

    /*
     * Point to request message portion of data buffer.
     */
    msgtyp = (uint32 *) msgp;					// 1st field is always msgtype.
	sessId = *((int32 *)(msgp + sizeof(uint32)));	// 2nd field is always sessid or cHandle (which is also sessid)
	
    /* Find message entry in table  */
    if ((msgent = ndmpGetNNIMMsgEnt(*msgtyp)) == 0)
    {
        NDMP_DBG_PRINTF (D_INFO, ("ndmp_NNIMReceiveMsg:   message %d: not supported.\n", *msgtyp));
        E_LOG(class_1, err_s, ndmp_NNIMCtrl.appId, 0, 0, 0,
						("ndmp_NNIMReceiveMsg:   message %d: not supported.\n", *msgtyp));
		NDMP_FP_FREE(msgp, "msgp")
		ATOMIC_INC_INT(&ndmp_fp_msgStats[UNSUPPORTED_MSG_RCVD], 1);
        return;
    }                    

	/* Update stats of msg received */
	ATOMIC_INC_INT(&ndmp_fp_msgStats[msgent->stat_idx_rcvd], 1);

	/* Queue up all received messages from SSC as well as from FP (in separate
	 * queues) so we can process them in order, as they may arrive out of order
	 */ 
	if (msgent->orig == SSC_CARD)
	{
		ndmp_NFXReq_t          *req = (ndmp_NFXReq_t *) msgp;

		seq_id = req->hdr.seqId;
		idx = req->hdr.idx;
		node = &ndmp_NNIMCtrl.sessLst[idx];
		//printf("msgtyp = %d, sessId = %d, msgent %p, msgp %p, ssc_src_port %d, idx %d\n", 
		//				*msgtyp, sessId, msgent, msgp, ssc_src_port, idx);
		msgI = saveMsgConTxt(*msgtyp, sessId, msgent, msgp, ssc_src_port);
		//printf("msgI ptr = %p\n", msgI);
		NFX_SPIN_LOCK(&node->ssc_msgq_lock);
		ssc_msgcnt = ndmp_InsertMsgInOrder(&seq_id, &msgI, node, SSC_CARD);
		NFX_SPIN_UNLOCK(&node->ssc_msgq_lock);
	} else {
		//seq_id = 1;  //????
		node = ndmp_find_sessid_entry(sessId, &idx);
		if (node == NULL)
		{
			NDMP_DBG_PRINTF(D_INFO, ("ndmp_NNIMReceiveMsg:  Received msg %s" 
						" for invalid session %d, ignore.\n", msgent->msg_name, sessId));
			NDMP_FP_FREE(msgp, "msgp")
			return;
		}
		msgI = saveMsgConTxt(*msgtyp, sessId, msgent, msgp, 0);
		NFX_SPIN_LOCK(&node->fp_msgq_lock);
		fp_msgcnt = ndmp_InsertMsgInOrder(&seq_id, &msgI, node, FP_CARD);
		NFX_SPIN_UNLOCK(&node->fp_msgq_lock);
	}
	return;
} /* ndmp_NNIMReceiveMsg */

/*
 *------------------------------------------------------------------------------
 * Function Name:   saveMsgConTxt
 *
 * Description:     Save the context of the msg received to be used when
 *					the msg is actually processed.
 *
 * Created by:      Anh Vu
 *
 * Data Created:    01/06/03
 *
 *------------------------------------------------------------------------------
 */
msgCntxt_t *
saveMsgConTxt(uint32 msgtyp, int32 sessId, 
				ndmpNNIMMsgEnt_t *msgent, char *msgp, uint32 ssc_src_port)
{
	msgCntxt_t *msgI = NULL;

	NDMP_FP_ALLOC(msgI, msgCntxt_t, sizeof(msgCntxt_t), "msgCntxt_t");
	NFX_ASSERT(msgI);
	if (msgI)
	{
		msgI->msgtyp = msgtyp;
		msgI->sessid = sessId;
		msgI->msgEnt = msgent;
		msgI->msgp = msgp;
		msgI->ssc_src_port = ssc_src_port;
	} 
	return msgI;
}
/*
 *------------------------------------------------------------------------------
 * Function Name:   ndmp_procMsgThread
 *
 * Description:     Save the context of the msg received to be used when
 *					the msg is actually processed.
 *
 * Created by:      Anh Vu
 *
 * Data Created:    01/06/03
 *
 *------------------------------------------------------------------------------
 */
void ndmp_procMsgThread(esm_handle_t    handle,
				    	void *arg)
{
	esm_event_t wakeEvent;
	int32 i, ssc_msgcnt, fp_msgcnt;
	sess_node_t *node;

	while (1)
	{
		thread_yield(&wakeEvent, 0);
		if (wakeEvent.event != NDMP_THREAD_EVENT_GOT_MSG) 
		{
	            //NDMP_DBG_PRINTF(D_INFO, ("ndmp thread: wrong event"));
	            printf(" procMsgthread: wrong event %d", wakeEvent.event);
				NFX_ASSERT(1);				
		}
		node = (sess_node_t *) wakeEvent.p1.u.p;
		NFX_ASSERT(node != NULL);
		i = ssc_msgcnt = fp_msgcnt = 0;
		
		while (i < NUM_MSG_TO_PROC)
		{
			//printf("ndmp_procMsgThread: chkNProcMsgInQ for SSC, i = %d\n", i);
			ssc_msgcnt = chkNProcMsgIn_SSCq(&node->ssc_rcv_q, &node->ssc_msgq_lock);
			//printf("ndmp_procMsgThread: chkNProcMsgInQ for FP, i = %d\n", i);
			fp_msgcnt = chkNProcMsgIn_FPq(&node->fp_rcv_q, &node->fp_msgq_lock);
			i++;
			/* if we have no more msgs in q, get out */
			if (!ssc_msgcnt && !fp_msgcnt)
			{
				//printf("ndmp_procMsgThread:  no more msgs to process, i = %d\n", i);
				break;
			}
		}
		/* If there are more msgs in either Qs and we have processed up
		* to NUM_MSG_TO_PROC already, post an event to wake  ourselves 
		* up to process them, only after we return here to yield processor
		* to others, that is.
		*/
		if (ssc_msgcnt || fp_msgcnt)
			ndmp_send_evnt(node);
	} 
}

/*
 *------------------------------------------------------------------------------
 * Function Name:   chkNProcMsgIn_SSCq
 *
 * Description:     Check and process any messages in either the SSC rcv
 *					Q or the FP rcv Q.
 *
 * Created by:      Anh Vu
 *
 * Data Created:    01/06/03
 *
 *------------------------------------------------------------------------------
 */
int32
chkNProcMsgIn_SSCq(msg_q_t *rcv_q, nfx_spinlock_t *msgq_lock)
{
	msgCntxt_t *msgI = NULL;
	uint64 seq_id = 0;
	int		rc, msgcnt = 0, msgtyp_processed;
	ndmp_NFXReq_t          *req; 

	//printf(" chkNProcMsgIn_SSCq:  begin\n");
	if (rcv_q->msg_q == NULL)
	{
		//printf("Rcv Q is NULL\n");
		return 0;
	}
	/* Check SSC receive Q & process msgs there */
	NFX_SPIN_LOCK(msgq_lock);
	//printf(" chkNProcMsgIn_SSCq:  got the lock\n");
	msgcnt = rcv_q->msg_q->size;
	
	/* Preview the head of msg rcv_q */
	if ((msgcnt > 0) && rcv_q->msg_q->next)
	{
		seq_id = rcv_q->msg_q->next->item1;
		msgI = (msgCntxt_t *) rcv_q->msg_q->next->item2;
		req = (ndmp_NFXReq_t *) msgI->msgp;
		//printf("chkNProcMsgIn_SSCq:  msgcnt = %d, seq_id %lld, sessid %d, nxtseqExp %lld, idx %d\n", 
		//					msgcnt, seq_id, msgI->sessid, rcv_q->nxt_seq_expected, req->hdr.idx);
		if (seq_id == rcv_q->nxt_seq_expected)
		{
			NFX_SPIN_UNLOCK(msgq_lock);
			msgtyp_processed = msgI->msgtyp;
			rc = ndmp_procMsg(msgI);
			if (rc == NO_LOCK)
			{
				printf("Can't get lock, exit\n");
			} else {
				if (rcv_q->msg_q != NULL)
				{
					//printf("chkNProcMsgIn_SSCq:  msgtyp = %d\n", msgI->msgtyp);
					/* Find item in Q to remove */
					NFX_SPIN_LOCK(msgq_lock);
					if ((rc = q_findQueueItem(rcv_q->msg_q, &seq_id, &msgI, Q_FIND_ONE)) == NFX_OK)
					{
						/* Free msg context before removing the msg node in q */
						NDMP_FP_FREE(msgI, "msgI chkNProcMsgInQ");
						if (q_deleteQueueItem(rcv_q->msg_q, &seq_id, &msgI) != NFX_OK)
						{
							//printf("Alert!  Q's empty, can't delete, msgI %p\n", msgI);
						}
					} else {
						//printf("Alert!  Can't find item to delete\n");
					}
					(rcv_q->nxt_seq_expected) ++;
					NFX_SPIN_UNLOCK(msgq_lock);
				} 
			}
		} else {
			NFX_SPIN_UNLOCK(msgq_lock);
		}
	} else {
		NFX_SPIN_UNLOCK(msgq_lock);
	}
	if (rcv_q->msg_q == NULL)
	{
		//printf("Receive Q is NULL\n");
		return 0;
	} else
		return rcv_q->msg_q->size;
}
/*
 *------------------------------------------------------------------------------
 * Function Name:   chkNProcMsgIn_FPq
 *
 * Description:     Check and process any messages in either the SSC rcv
 *					Q or the FP rcv Q.
 *
 * Created by:      Anh Vu
 *
 * Data Created:    01/06/03
 *
 *------------------------------------------------------------------------------
 */
int32
chkNProcMsgIn_FPq(msg_q_t *rcv_q, nfx_spinlock_t *msgq_lock)
{
	msgCntxt_t *msgI = NULL;
	int		rc, msgcnt = 0;

	//printf(" chkNProcMsgIn_FPq:  begin\n");
	NFX_SPIN_LOCK(msgq_lock);
	if (rcv_q->msg_q == NULL)
	{
		//printf("FP Rcv Q is NULL\n");
		NFX_SPIN_UNLOCK(msgq_lock);
		return 0;
	}
	/* Check FP receive Q & process msgs there */
	//printf(" chkNProcMsgIn_FPq:  got the lock\n");
	msgcnt = rcv_q->msg_q->size;
	
	/* Preview the head of msg rcv_q */
	if ((msgcnt > 0) && rcv_q->msg_q->next)
	{
		msgI = (msgCntxt_t *) rcv_q->msg_q->next->item1;
		//printf("chkNProcMsgIn_FPq:  msgcnt = %d, sessid %d, nxtseqExp %lld\n", 
		//				msgcnt, msgI->sessid, rcv_q->nxt_seq_expected);
		NFX_SPIN_UNLOCK(msgq_lock);
		rc = ndmp_procMsg(msgI);
		if (rc == NO_LOCK)
		{
			//printf("Can't get lock, exit\n");
		} else {
			/* Remove first entry in Q */
			NFX_SPIN_LOCK(msgq_lock);
			if ((rc = q_deQueue(rcv_q->msg_q, &msgI, &msgI)) == NFX_OK)
			{
				NDMP_FP_FREE(msgI, "msgI chkNProcMsgIn_FPq");
			} else {
				//printf("No deletion.  Q's empty\n");
			}
			NFX_SPIN_UNLOCK(msgq_lock);
		}
	} else {
		NFX_SPIN_UNLOCK(msgq_lock);
	}
	return rcv_q->msg_q->size;
}
/*
 *------------------------------------------------------------------------------
 * Function Name:   ndmp_procMsg
 *
 * Description:     Sends cmd response event to the Data and Mover 
 *                  state machines.  This code is SMP compatible.
 *
 * Created by:      Anh Vu
 *
 * Data Created:    01/02/03
 *
 *------------------------------------------------------------------------------
 */
int32
ndmp_procMsg(msgCntxt_t *msgI)
{
	esm_handle_t  		Data_smHdl, Mover_smHdl;
	ndmpNNIMMsgEnt_t   	*msgent;
    esm_event_t   		esmEvent;
	ndmp_NFXReq_t		*newsess_req;
    NdmpNIMService_t 	*servp = NULL;
	sess_node_t         *node = NULL;
	int					idx;
	int32				rc, event, sessId, msgtyp, ver, ssc_src_port;
	char				*msgp;
	esm_handle_t  		hdl;

	//printf("ndmp_procMsg: begin\n");
	if (msgI == NULL)
	{
    	NDMP_DBG_PRINTF (D_INFO, ("ndmp_procMsg:   sessId %d (0x%x)-- msgI is NULL\n", sessId, sessId));
		return NFX_ERR;

	}	
	sessId = msgI->sessid;
	msgp = msgI->msgp;
	msgent = msgI->msgEnt;
	msgtyp = msgI->msgtyp;
	ssc_src_port = msgI->ssc_src_port;

	node = ndmp_find_sessid_entry(sessId, &idx);

	if (idx != -1)
		servp = node->servp;

    NDMP_DBG_PRINTF (D_INFO, ("ndmp_procMsg:   sessId %d (0x%x)-- received %s message --, idx %d\n", 
							sessId, sessId, msgent->msg_name, idx));

	/* Handle special messages */

	if ((msgtyp != NDMP_MID_NEW_SESSION) && (node))
		if (!NFX_SPIN_TRYLOCK(&node->servp_lock))
			return NO_LOCK;

	switch (msgtyp)
	{
		case NDMP_MID_NEW_SESSION:
		{
			int32 rc = NFX_OK; 
			int32 unlock = FALSE;

			/* Create Data and Mover state machines */
			if ((idx != -1) && (node))
			{
				NDMP_DBG_PRINTF (D_INFO, ("ndmp_NNIMReceiveMsg:  Warning, sessid %d entry existed, return.\n", 
						sessId));
				E_LOG(class_1, err_s, ndmp_NNIMCtrl.appId, 0, 0, 0, 
						("ndmp_NNIMReceiveMsg:  Warning, sessid %d entry existed, return.\n", 
						sessId));
				return NFX_ERR;
			}
			newsess_req = (ndmp_NFXReq_t *)msgp;
			idx = newsess_req->content[0];
			ver = newsess_req->userInfo;
			if ((ver != NDMP_VER_2) && (ver != NDMP_VER_3))
			{
				NDMP_DBG_PRINTF (D_INFO, ("ndmp_NNIMReceiveMsg:  Invalid NDMP version received %d, ignored.\n", ver));
				E_LOG(class_1, err_s, ndmp_NNIMCtrl.appId, 0, 0, 0,
						("ndmp_NNIMReceiveMsg:  Invalid NDMP version received %d, ignored.\n", ver));
			} else {
				node = &ndmp_NNIMCtrl.sessLst[idx];	
				if (!NFX_SPIN_TRYLOCK(&node->servp_lock))
					rc = NO_LOCK;
				else {
					ndmp_newSess(idx, sessId, ssc_src_port, ver);
					unlock = TRUE;
				}
			}	
			NDMP_FP_FREE(msgp, "msgp NEWSESS")
			if (unlock == TRUE)
				NFX_SPIN_UNLOCK(&node->servp_lock);
			return rc;	
		}
		case NDMP_MID_CLOSE_SESSION:
			/* Send an abort to Data server a bkup is in progress */
            if (sessId == NDMP_SESSID_ALL)
            {
                ndmp_NFXReq_t *req = NULL;
                int32 i;
                sess_node_t *np = NULL;

                /*
                 * SSC down. We have to treat it as kill session. Since we have
                 * lost connection to client, just clean FS and NDMP.
                 */
                for (i = 0; i < NDMP_MAX_SESS; i++)
                {
                    np = &ndmp_NNIMCtrl.sessLst[i];
                    NDMP_FP_ALLOC(req, ndmp_NFXReq_t, sizeof(ndmp_NFXReq_t), "ndmp_NFXReq_t");
                    req->hdr.mid    = NDMP_MID_CLOSE_SESSION;
                    req->hdr.sessId = np->sessid;
                    req->content[0] = i;
                    if (ndmp_closeSessOnNIM(i, np, (char *)req) == NFX_ERR)
                        NDMP_FP_FREE(req, "ndmp_NFXReq_t");
                    /* NDMP close session will free the req */
                }
            }
            else if (node && (idx != -1) && (sessId == node->sessid))
            {
        	    if (ndmp_closeSessOnNIM(idx, node, msgp) == NFX_ERR)
				{
                    NDMP_FP_FREE(msgp, "msgp CLOSE_SESS");
					NFX_SPIN_UNLOCK(&node->servp_lock);
					return NFX_ERR;
				}
            }
            else
                NDMP_FP_FREE(msgp, "msgp CLOSE_SESS");
			
			NFX_SPIN_UNLOCK(&node->servp_lock);
			return NFX_OK;	

		case NDMP_MID_UI_DM_GET_STATE:
			/* Get states of Data and Mover servers */
			if (sessId == node->sessid)
			{
        		ndmp_UIGetDMStateOnNIM(node, msgp, ssc_src_port);
            	NDMP_FP_FREE(msgp, "msgp UI_GS");
			}
			NFX_SPIN_UNLOCK(&node->servp_lock);
			return NFX_OK;	
		default:
			;
			//Proceed to process normal msg below.
	}

	if ( (idx == -1) || (!node) || (node && (node->servp == NULL)) || (node->sessid != sessId))
	{
		NDMP_DBG_PRINTF(D_INFO, ("ndmp_NNIMReceiveMsg:  Received msg %s for invalid session %d, ignore.\n", msgent->msg_name, sessId));
		NDMP_FP_FREE(msgp, "msgp")
		if (!node || (idx == -1))
		{
			NFX_SPIN_UNLOCK(&node->servp_lock);
		}
		return NFX_ERR;
	}
	servp = node->servp;
	Data_smHdl = servp->Data.my_smHdl;
	Mover_smHdl = servp->Mover.my_smHdl;

	event = (servp->version == NDMP_VER_2)? msgent->v2_event: msgent->v3_event;
	hdl = (msgent->sm == MOVER_SM)? Mover_smHdl: Data_smHdl;

	bzero(&esmEvent, sizeof(esm_event_t));
    esmEvent.event = event;
    esmEvent.p1.u.p = msgp;
    rc = esm_deliverEvent(hdl, &esmEvent);
    if (rc != NFX_OK)
    {
		NDMP_DBG_PRINTF(D_INFO, ("ndmp_NNIMReceiveMsg:  SessId %d -- ERROR, cannot post event %d to SM hdl %d",
                      sessId, event, hdl));
        E_LOG(class_1, err_s, ndmp_NNIMCtrl.appId, 0, 0, 0,
					("ndmp_NNIMReceiveMsg:  SessId %d -- ERROR, cannot post event %d to SM hdl %d",
                      sessId, event, hdl));
		NDMP_FP_FREE(msgp, "msgp")
    }

	NFX_SPIN_UNLOCK(&node->servp_lock);
    return NFX_OK; 

} /*  ndmp_procMsg */

#else

/*
 *------------------------------------------------------------------------------
 * Function Name:   ndmp_NNIMReceiveMsg
 *
 * Description:     Sends cmd response event to the Data and Mover 
 *                  state machines.
 *
 * Created by:      Anh Vu
 *
 * Data Created:    9/17/01
 *
 *------------------------------------------------------------------------------
 */
void
ndmp_NNIMReceiveMsg(eee_desc_t *eDesc)
{
	uint32        buffSize, *msgtyp, ssc_src_port;
	int32		  ret, rc, sessId, event, ver;
	esm_handle_t  hdl;
    NdmpNIMService_t 	*servp;
	char		  		*msgp; 
	esm_handle_t  		Data_smHdl, Mover_smHdl;
	ndmpNNIMMsgEnt_t   	*msgent;
    esm_event_t   		esmEvent;
	ndmp_NFXReq_t		*newsess_req;
	int					idx;
	sess_node_t			*node = NULL;


    if( !eDesc ) {
        NDMP_DBG_PRINTF(D_INFO, ("ndmp_NNIMReceiveMsg: Null eDesc\n"));
        E_LOG(class_1, err_s, ndmp_NNIMCtrl.appId, 0, 0, 0,
						("ndmp_NNIMReceiveMsg: Null eDesc\n"));
        return;
    }

	buffSize = eDesc->hdr.control & EDESC_TOTAL_LEN;
    /*
     * convert EEE packet to flat buffer
     */
    NDMP_FP_ALLOC(msgp, char, buffSize, "msgp");
    NFX_ASSERT(msgp != NULL);

	if ((rc = eee_copyPacketToLocalBuffer(msgp, eDesc, buffSize, 0)) != buffSize)
    {
        NDMP_DBG_PRINTF(D_INFO, ("ndmp_NNIMReceiveMsg: error flattening EEE packet (expected %d got %d)",
                  buffSize, rc));
        E_LOG(class_1, err_s, ndmp_NNIMCtrl.appId, 0, 0, 0,
				("ndmp_NNIMReceiveMsg: error flattening EEE packet (expected %d got %d)",
                 buffSize, rc));
        eee_freePacket(eDesc);
		NDMP_FP_FREE(msgp, "msgp")
        return;
    }
	ssc_src_port = eDesc->hdr.src_port;
	eee_freePacket(eDesc);

    /*
     * Point to request message portion of data buffer.
     */
    msgtyp = (uint32 *) msgp;					// 1st field is always msgtype.
	sessId = *((int32 *)(msgp + sizeof(uint32)));	// 2nd field is always sessid or cHandle (which is also sessid)
	
    /* Find message entry in table  */
    if ((msgent = ndmpGetNNIMMsgEnt(*msgtyp)) == 0)
    {
        NDMP_DBG_PRINTF (D_INFO, ("ndmp_NNIMReceiveMsg:   message %d: not supported.\n", *msgtyp));
        E_LOG(class_1, err_s, ndmp_NNIMCtrl.appId, 0, 0, 0,
						("ndmp_NNIMReceiveMsg:   message %d: not supported.\n", *msgtyp));
		NDMP_FP_FREE(msgp, "msgp")
		ndmp_fp_msgStats[UNSUPPORTED_MSG_RCVD]++;
        return;
    }                    

	/* Update stats of msg received */
	ndmp_fp_msgStats[msgent->stat_idx_rcvd]++;

	/* If session does not exist on NNIM yet, return */
	node = ndmp_find_sessid_entry(sessId, &idx);

    NDMP_DBG_PRINTF (D_INFO, ("ndmp_NNIMReceiveMsg:   sessId %d (0x%x)-- received %s message --, idx %d, node %p\n", sessId, sessId, msgent->msg_name, idx, node));

	/* Handle special messages */
	switch (*msgtyp)
	{
		case NDMP_MID_NEW_SESSION:
			/* Create Data and Mover state machines */
			if ((idx != -1) && (node))
			{
				NDMP_DBG_PRINTF (D_INFO, ("ndmp_NNIMReceiveMsg:  Warning, sessid %d entry existed, return.\n", 
							node->sessid));
				E_LOG(class_1, err_s, ndmp_NNIMCtrl.appId, 0, 0, 0, 
							("ndmp_NNIMReceiveMsg:  Warning, sessid %d entry existed, return.\n", 
							node->sessid));
				return;
			}
			newsess_req = (ndmp_NFXReq_t *)msgp;
			idx = newsess_req->content[0];
			ver = newsess_req->userInfo;
			if ((ver != NDMP_VER_2) && (ver != NDMP_VER_3))
			{
				NDMP_DBG_PRINTF (D_INFO, ("ndmp_NNIMReceiveMsg:  Invalid NDMP version received %d, ignored.\n", ver));
				E_LOG(class_1, err_s, ndmp_NNIMCtrl.appId, 0, 0, 0,
								("ndmp_NNIMReceiveMsg:  Invalid NDMP version received %d, ignored.\n", ver));
			} else 
       			ndmp_newSess(idx, sessId, ssc_src_port, ver);
			
            NDMP_FP_FREE(msgp, "msgp NEWSESS")
			return;	
		case NDMP_MID_CLOSE_SESSION:
			/* Send an abort to Data server a bkup is in progress */
            if (sessId == NDMP_SESSID_ALL)
            {
                ndmp_NFXReq_t *req = NULL;
                int32 i;
                sess_node_t *np = NULL;

                /*
                 * SSC down. We have to treat it as kill session. Since we have
                 * lost connection to client, just clean FS and NDMP.
                 */
                for (i = 0; i < NDMP_MAX_SESS; i++)
                {
                    for (np = ndmp_NNIMCtrl.sessLst[i]; np != NULL; np=np->nxt)
                    {
                        NDMP_FP_ALLOC(req, ndmp_NFXReq_t, sizeof(ndmp_NFXReq_t), "ndmp_NFXReq_t");
                        req->hdr.mid    = NDMP_MID_CLOSE_SESSION;
                        req->hdr.sessId = np->sessid;
                        req->content[0] = i;
                        if (ndmp_closeSessOnNIM(i, np, (char *)req) == NFX_ERR)
                            NDMP_FP_FREE(req, "ndmp_NFXReq_t");
                        /* NDMP close session will free the req */
                    }
                }
            }
            else if (node && (idx != -1))
            {
        	    if (ndmp_closeSessOnNIM(idx, node, msgp) == NFX_ERR)
                    NDMP_FP_FREE(msgp, "msgp CLOSE_SESS");
            }
            else
                NDMP_FP_FREE(msgp, "msgp CLOSE_SESS");
			return;	
		case NDMP_MID_UI_DM_GET_STATE:
			/* Get states of Data and Mover servers */
        	ndmp_UIGetDMStateOnNIM(node, msgp, ssc_src_port);
            NDMP_FP_FREE(msgp, "msgp UI_GS");
			return;	
		case NDMP_MID_REL_RSRC:
			/* Destroy Data and Mover state machines */
        	ret = ndmp_relSessRsrc(idx, node);
            NDMP_FP_FREE(msgp, "msgp REL_RSRC");
			return;	
		default:
			;
			//Proceed to process normal msg below.
	}

	if ( (idx == -1) || (!node) || (node && (node->servp == NULL)))
	{
		NDMP_DBG_PRINTF(D_INFO, ("ndmp_NNIMReceiveMsg:  Received msg %s for invalid session %d, ignore.\n", msgent->msg_name, sessId));
		NDMP_FP_FREE(msgp, "msgp")
		return;
	}
	servp = node->servp;
	Data_smHdl = servp->Data.my_smHdl;
	Mover_smHdl = servp->Mover.my_smHdl;

	event = (servp->version == NDMP_VER_2)? msgent->v2_event: msgent->v3_event;
	hdl = (msgent->sm == MOVER_SM)? Mover_smHdl: Data_smHdl;

    bzero(&esmEvent, sizeof(esm_event_t));
    esmEvent.event = event;
    esmEvent.p1.u.p = msgp;
    rc = esm_deliverEvent(hdl, &esmEvent);
    if (rc != NFX_OK)
    {
		NDMP_DBG_PRINTF(D_INFO, ("ndmp_NNIMReceiveMsg:  SessId %d -- ERROR, cannot post event %d to SM hdl %d",
                      sessId, event, hdl));
        E_LOG(class_1, err_s, ndmp_NNIMCtrl.appId, 0, 0, 0,
					("ndmp_NNIMReceiveMsg:  SessId %d -- ERROR, cannot post event %d to SM hdl %d",
                      sessId, event, hdl));
		NDMP_FP_FREE(msgp, "msgp")
    }


    return; 
} /*  ndmp_NNIMReceiveMsg */
#endif
