/* vi:ts=4
 *-----------------------------------------------------------------
 *
 * Name:        ndmp-nim-cmd.c
 *                              
 * RCS:  $Id: ndmp-nim-cmd.c,v 1.12 2002/10/18 02:52:43 johnh Exp $
 *
 * Copyright (C) 2001, Agile Storage Inc.
 *
 * Description:
 *  	Contain ndmp UI command functions on N-NIM only.
 *
 * Created by: Anh Vu
 *
 * Date Created:        02/18/02
 *
 *-----------------------------------------------------------------
 */
#ifdef	NFP_FP
#include <nfx-incl.h>

#ifdef	LINUX_TEST	
#include <string.h>
#include <stdio.h>
#endif	//LINUX_TEST
#include <exports/eee-api.h>
#include "ndmp.h"


extern int ndmp_dbg_level;
extern int ndmp_perf_meas;
extern int ndmp_loop_bk;
extern ndmp_NNIMCtrl_t ndmp_NNIMCtrl;
extern int ndmp_fp_msgStats[MAX_FP_STAT];

int32 ndmp_cmdDebug ( int32 argc, char *argv[] );
int32 ndmp_cmdPerfMeas ( int32 argc, char *argv[] );
int32 ndmp_cmdLoopBk ( int32 argc, char *argv[] );
int32 ndmp_cmdKillSess ( int32 argc, char *argv[] );
int32 ndmp_cmdMemStat ( int32 argc, char *argv[] );
int32 ndmp_cmdClearMemStat(int32 argc, char *argv[]);
int32 ndmp_cmdMsgRcvdStat(int32 argc, char *argv[]);
int32 ndmp_cmdClearMsgRcvdStat(int32 argc, char *argv[]);
int32 ndmp_cmdPrintSIdTbl ( int32 argc, char *argv[] );

typedef struct
{
	int32   (*cmdFunc)( int32 argc, char *argv[] );
	char    *cmd;
	char    *helpTxt;
} ndmp_cmdTbl_t;

ndmp_cmdTbl_t    ndmp_cmdTable[ ] =
{
	{ ndmp_cmdDebug, "dbg",    "ndmp dbg [0-3]" },
	{ ndmp_cmdPerfMeas, "pm",    "ndmp pm [0-1]" },
	{ ndmp_cmdLoopBk, "lpbk",    "ndmp lpbk [0-1]" },
	{ ndmp_cmdMemStat, "memstat",    "ndmp memstat" },
	{ ndmp_cmdClearMemStat, "clear_memstat",    "ndmp clear_memstat" },
	{ ndmp_cmdMsgRcvdStat, "msgstat",    "ndmp msgstat" },
	{ ndmp_cmdClearMsgRcvdStat, "clear_msgstat",    "ndmp clear_msgstat" },
	{ ndmp_cmdKillSess, "kill",    "ndmp kill <sessid>" },
	{ ndmp_cmdPrintSIdTbl, "sessid",    "ndmp sessid" }
};

#define NDMP_MAX_CMDS	sizeof(ndmp_cmdTable)/sizeof(ndmp_cmdTbl_t)


/*-----------------------------------------------------------------
 * Name: ndmp_cmdHelpAll
 *
 * Description: Display help-texts for all ndmp commands.
 *
 * Created by : Anh Vu 
 *
 * Date Created: 02/18/02
 *-----------------------------------------------------------------
 */
int32 ndmp_cmdHelpAll(void)
{
	int32	i;

	printf("Usage:");
	for (i=0; i < NDMP_MAX_CMDS; i++)
	{
		printf("\t%s\n", ndmp_cmdTable[i].helpTxt);
	}
	return (0);
}

/*-----------------------------------------------------------------
 * Name: ndmp_cmdHelpCmd
 *
 * Description: Display help-texts for an individual ndmp command.
 *
 * Created by : Anh Vu 
 * 
 * Date Created: 02/18/02
 *-----------------------------------------------------------------
 */
int32 ndmp_cmdHelpCmd (char *pCmd)
{
	int32	i;

	for (i=0; i < NDMP_MAX_CMDS; i++)
	{
		if (strequ(pCmd, ndmp_cmdTable[i].cmd))
		{
			printf("Usage:");
			printf("\t%s\n", ndmp_cmdTable[i].helpTxt);
			return (0);
		}
	}
	return (ndmp_cmdHelpAll( ));

}

/*-----------------------------------------------------------------
 * Name: ndmp_cmdHelp
 *
 * Description: Display ndmp help-texts.
 *
 *              ndmp help [show | debug]
 *
 * Created by : Anh Vu 
 *
 * Date Created: 02/18/02
 *-----------------------------------------------------------------
 */
int32
ndmp_cmdHelp ( int32 argc, char *argv[] )
{
	if (argc < 3)
		return (ndmp_cmdHelpAll());
	else
		return (ndmp_cmdHelpCmd( argv[2] ));
}

/*-----------------------------------------------------------------
 * Name: ndmp_cmdDebug
 *
 * Description: Enables or disables ndmp debug trace.
 *
 *              ndmp dbg       - shows the current dbg level
 *              ndmp dbg [0-3] - sets the dbg level
 * 
 * Created by : Anh Vu 
 * 
 * Date Created: 02/18/02
 *-----------------------------------------------------------------
 */
int32 ndmp_cmdDebug ( int32 argc, char *argv[] )
{
	if (argc == 2)
	{
		printf("Current NDMP debug level = %d.\n", ndmp_dbg_level);
		return 0;
	}
	ndmp_dbg_level = atoi(argv[2]);
	if (ndmp_dbg_level > 0)
	{
		printf("NDMP debug is set to level %d.\n", ndmp_dbg_level);
	}
	else if (ndmp_dbg_level == 0)
	{
		printf("NDMP debug is off.\n");
	}
	else
		ndmp_cmdHelpCmd("dbg");

	return 0;
}
/*-----------------------------------------------------------------
 * Name: ndmp_cmdLoopBk
 *
 * Description: Enables or disables ndmp loopback response to FS
 * 				to measure disk i/o, ie. don't forward the ios to scsi.
 *
 *              ndmp lpbk <1/0> - enable/disable loopback code
 * 
 * Created by : Anh Vu 
 * 
 * Date Created: 08/15/02
 *-----------------------------------------------------------------
 */
int32 ndmp_cmdLoopBk ( int32 argc, char *argv[] )
{
	if (argc == 2)
	{
		printf("NDMP Loopback Code is %s.\n", (ndmp_loop_bk == 1)? "ON": "OFF");
		return 0;
	}
	if (!strcmp(argv[2], "1"))
	{
		ndmp_loop_bk = 1;
	} else if (!strcmp(argv[2], "0"))
	{
		ndmp_loop_bk = 0;
	} else {
		ndmp_cmdHelpCmd("lpbk");
		return 0;
	}

	printf("NDMP loopback is now %s.\n", ndmp_loop_bk? "ON": "OFF");

	return 0;
}
/*-----------------------------------------------------------------
 * Name: ndmp_cmdPerfMeas
 *
 * Description: Enables or disables ndmp performance measure.
 *
 *              ndmp pm <1/0> - enable/disable perf meas code
 * 
 * Created by : Anh Vu 
 * 
 * Date Created: 08/15/02
 *-----------------------------------------------------------------
 */
int32 ndmp_cmdPerfMeas ( int32 argc, char *argv[] )
{
	if (argc == 2)
	{
		printf("NDMP Perf Meas Code is %s.\n", (ndmp_perf_meas == 1)? "ON": "OFF");
		return 0;
	}
	if (!strcmp(argv[2], "1"))
	{
		ndmp_perf_meas = 1;
	} else if (!strcmp(argv[2], "0"))
	{
		ndmp_perf_meas = 0;
	} else {
		ndmp_cmdHelpCmd("pm");
		return 0;
	}

	printf("NDMP perf meas is now %s.\n", ndmp_perf_meas? "ON": "OFF");

	return 0;
}
/*-----------------------------------------------------------------
 * Name: ndmp_cmdKillSess
 *
 * Description: Kill a lingering session on FP
 *
 *              ndmp kill <sessid> 
 *				(shown by using cmd 'ndmp sessid')
 * 
 * Created by : Anh Vu 
 * 
 * Date Created: 06/25/02
 *-----------------------------------------------------------------
 */
int32 ndmp_cmdKillSess ( int32 argc, char *argv[] )
{
	ndmp_NFXReq_t       ndmp_req;
	sess_node_t			*node;
	int					sessId, idx, rc;


	if (argc < 3)
	{
		ndmp_cmdHelpCmd("kill");
		return 0;
	}
	sessId = atoi(argv[2]);
	node = ndmp_find_sessid_entry(sessId, &idx);

	if ((idx == -1) || (node == NULL))
	{
		printf("Invalid sessId %d\n", sessId);
		return 0;
	}
	/* Send message to NNIM to remove Mover and Data
	 * state machines there
	 */
	memset((void*)&ndmp_req, 0, sizeof(ndmp_req));
	ndmp_req.hdr.mid        = NDMP_MID_CLOSE_SESSION;
	ndmp_req.hdr.sessId     = sessId;
	ndmp_req.content[0]     = idx;

	printf("Send Close msg for sessId %d\n", sessId);

	rc = eee_sendMessage((void *) &ndmp_req, sizeof(ndmp_req),
                ndmp_NNIMCtrl.src_port, ndmp_NNIMCtrl.src_port);
	if (rc != NFX_OK)
		printf("Send Close msg for sessId %d failed.\n", sessId);
	return 0;
}

/*-----------------------------------------------------------------
 * Name: ndmp_cmdMemStat
 *
 * Description: Prints memory stats
 *
 *              ndmp memstat
 * 
 * Created by : Nitin Jain
 * 
 * Date Created: 
 *-----------------------------------------------------------------
 */
int32 ndmp_cmdMemStat(int32 argc, char *argv[])
{
    printf ("FP MALLOCS : %d\n", ndmp_fp_malloc);
    printf ("FP FREES   : %d\n", ndmp_fp_free);

    return 0;
}
/*-----------------------------------------------------------------
 * Name: ndmp_cmdClearMemStat
 *
 * Description: Clear memory stats
 *
 *              ndmp clear_memstat
 * 
 * Created by : Anh Vu
 * 
 * Date Created:  7/24/02
 *-----------------------------------------------------------------
 */
int32 ndmp_cmdClearMemStat(int32 argc, char *argv[])
{
	printf("Memory stats cleared.\n");
    ndmp_fp_malloc = 0;
    ndmp_fp_free = 0;

    return 0;
}

/*-----------------------------------------------------------------
 * Name: 		ndmp_cmdClearMsgRcvdStat
 *
 * Description: Prints message received stats
 *
 *              ndmp msgstat - show for all current sessions
 * 
 * Created by : Anh Vu
 * 
 * Date Created: 07/24/02 
 *-----------------------------------------------------------------
 */
int32 ndmp_cmdClearMsgRcvdStat(int32 argc, char *argv[])
{
	if (argc == 2)
	{
		printf("Msg received stats cleared.\n");
		memset ( ndmp_fp_msgStats, 0, sizeof(int32) * MAX_FP_STAT );	
    } else
		ndmp_cmdHelpCmd("clear_msgstat");

    return 0;
}
/*-----------------------------------------------------------------
 * Name:		print_stats_rcvd 
 *
 * Description: Prints message received stats
 *
 * Created by : Anh Vu
 * 
 * Date Created:  07/24/02
 *-----------------------------------------------------------------
 */
void print_stats_rcvd (void)
{
	printf("New_Session_Req_Rcvd       : %d\n", ndmp_fp_msgStats[SSC_NEW_SESSION_REQ_RCVD]);
	printf("Close_Session_Req_Rcvd     : %d\n", ndmp_fp_msgStats[SSC_CLOSE_SESSION_REQ_RCVD]);
	printf("UI_DM_GetState_Req_Rcvd    : %d\n", ndmp_fp_msgStats[SSC_UI_DM_GETSTATE_REQ_RCVD]);
	printf("Rel_Rsrc_Req_Rcvd          : %d\n\n", ndmp_fp_msgStats[REL_RSRC_REQ_RCVD]);
#ifdef NO_SCSIIO
	printf("SCSI_Wr_Rsp_Rcvd           : %d\n", ndmp_fp_msgStats[SCSI_WR_RSP_RCVD]);
	printf("SCSI_Rd_Rsp_Rcvd           : %d\n\n", ndmp_fp_msgStats[SCSI_RD_RSP_RCVD]);
#endif
	printf("FS_Dmp_SetUp_Rsp_Rcvd      : %-5d\tFS_Dmp_SetUp_Req_Snt      : %d\n", 
				ndmp_fp_msgStats[FS_DMP_SETUP_RSP_RCVD], ndmp_fp_msgStats[FS_DMP_SETUP_REQ_SNT]);
	printf("FS_Dmp_Names_Rsp_Rcvd      : %-5d\tFS_Dmp_Names_Req_Snt      : %d\n", 
				ndmp_fp_msgStats[FS_DMP_NAMES_RSP_RCVD], ndmp_fp_msgStats[FS_DMP_NAMES_REQ_SNT]);
	printf("FS_Dmp_Start_Rsp_Rcvd      : %-5d\tFS_Dmp_Start_Req_Snt      : %d\n", 
				ndmp_fp_msgStats[FS_DMP_START_RSP_RCVD], ndmp_fp_msgStats[FS_DMP_START_REQ_SNT]);
	printf("FS_Dmp_Abort_Rsp_Rcvd      : %-5d\tFS_Dmp_Abort_Req_Snt      : %d\n", 
				ndmp_fp_msgStats[FS_DMP_ABORT_RSP_RCVD], ndmp_fp_msgStats[FS_DMP_ABORT_REQ_SNT]);
	printf("FS_Rst_SetUp_Rsp_Rcvd      : %-5d\tFS_Rst_SetUp_Req_Snt      : %d\n", 
				ndmp_fp_msgStats[FS_RST_SETUP_RSP_RCVD], ndmp_fp_msgStats[FS_RST_SETUP_REQ_SNT]);
	printf("FS_Rst_Names_Rsp_Rcvd      : %-5d\tFS_Rst_Names_Req_Snt      : %d\n", 
				ndmp_fp_msgStats[FS_RST_NAMES_RSP_RCVD], ndmp_fp_msgStats[FS_RST_NAMES_REQ_SNT]);
	printf("FS_Rst_Start_Rsp_Rcvd      : %-5d\tFS_Rst_Start_Req_Snt      : %d\n", 
				ndmp_fp_msgStats[FS_RST_START_RSP_RCVD], ndmp_fp_msgStats[FS_RST_START_REQ_SNT]);
	printf("FS_Rst_Abort_Rsp_Rcvd      : %-5d\tFS_Rst_Abort_Req_Snt      : %d\n", 
				ndmp_fp_msgStats[FS_RST_ABORT_RSP_RCVD], ndmp_fp_msgStats[FS_RST_ABORT_REQ_SNT]);
	printf("FS_Snd_Log_Req_Rcvd        : %-5d\tSSC_Log_File_Req_Snt      : %d\n", 
				ndmp_fp_msgStats[FS_SND_LOG_REQ_RCVD], ndmp_fp_msgStats[SSC_LOG_FILE_REQ_SNT]);
	printf("                                \tSSC_Log_Msg_Req_Snt       : %d\n", 
				ndmp_fp_msgStats[SSC_LOG_MSG_REQ_SNT]);
	printf("FS_Snd_FH_Dir_Req_Rcvd     : %-5d\tSSC_Snd_FH_Dir_Req_Snt    : %d\n", 
				ndmp_fp_msgStats[FS_SND_FH_DIR_REQ_RCVD], ndmp_fp_msgStats[SSC_SND_FH_DIR_REQ_SNT]);
	printf("FS_Snd_FH_Node_Req_Rcvd    : %-5d\tSSC_Snd_FH_Node_Req_Snt   : %d\n", 
				ndmp_fp_msgStats[FS_SND_FH_NODE_REQ_RCVD], ndmp_fp_msgStats[SSC_SND_FH_NODE_REQ_SNT]);
	printf("FS_Set_DmpDate_Req_Rcvd    : %-5d\tSSC_Set_DmpDate_Req_Snt   : %d\n", 
				ndmp_fp_msgStats[FS_SET_DMPDATE_REQ_RCVD], ndmp_fp_msgStats[SSC_SET_DMPDATE_REQ_SNT]);
	printf("FS_Rd_Data_Req_Rcvd        : %-5d\tFS_Rd_Data_Rsp_Snt        : %d\n",
				ndmp_fp_msgStats[FS_RD_DATA_REQ_RCVD], ndmp_fp_msgStats[FS_RD_DATA_RSP_SNT]);
	printf("FS_Wr_Data_Req_Rcvd        : %-5d\tFS_Wr_Data_Rsp_Snt        : %d\n",
				ndmp_fp_msgStats[FS_WR_DATA_REQ_RCVD], ndmp_fp_msgStats[FS_WR_DATA_RSP_SNT]);
	printf("SCSI_Rd_Data_Rsp_Rcvd      : %-5d\tSCSI_Rd_Data_Req_Snt      : %d\n", 
				ndmp_fp_msgStats[SCSI_RD_DATA_RSP_RCVD], ndmp_fp_msgStats[SCSI_RD_DATA_REQ_SNT]);
	printf("SCSI_Wr_Data_Rsp_Rcvd      : %-5d\tSCSI_Wr_Data_Req_Snt      : %d\n", 
				ndmp_fp_msgStats[SCSI_WR_DATA_RSP_RCVD], ndmp_fp_msgStats[SCSI_WR_DATA_REQ_SNT]);
	printf("FS_Ops_Cmpl_Req_Rcvd       : %-5d\tUpdate_DmpDate_Env_Req_Snt: %d\n",
				ndmp_fp_msgStats[FS_OPS_CMPL_REQ_RCVD], ndmp_fp_msgStats[SSC_UPD_ENV_REQ_SNT]);

	printf("CL_Data_Listen_Req_Rcvd    : %-5d\tCL_Data_Listen_Rsp_Snt    : %d\n", 
					ndmp_fp_msgStats[SSC_DATA_LISTEN_REQ_RCVD], ndmp_fp_msgStats[SSC_DATA_LISTEN_RSP_SNT]);
	printf("Cl_Data_Conn_Req_Rcvd      : %-5d\tCL_Data_Conn_Rsp_Snt      : %d\n", 
					ndmp_fp_msgStats[SSC_DATA_CONN_REQ_RCVD], ndmp_fp_msgStats[SSC_DATA_CONN_RSP_SNT]);
	printf("Cl_Data_Start_Bkup_Req_Rcvd: %-5d\tCl_Data_Start_Bkup_Rsp_Snt: %d\n", 
					ndmp_fp_msgStats[SSC_DATA_START_BKUP_REQ_RCVD], ndmp_fp_msgStats[SSC_DATA_START_BKUP_RSP_SNT]);
	printf("Cl_Data_Start_Rst_Req_Rcvd : %-5d\tCl_Data_Start_Rst_Rsp_Snt : %d\n", 
					ndmp_fp_msgStats[SSC_DATA_START_RST_REQ_RCVD], ndmp_fp_msgStats[SSC_DATA_START_RST_RSP_SNT]);
	printf("Cl_Data_GetEnv_Req_Rcvd    : %-5d\tCl_Data_GetEnv_Rsp_Snt    : %d\n", 
					ndmp_fp_msgStats[SSC_DATA_GETENV_REQ_RCVD], ndmp_fp_msgStats[SSC_DATA_GETENV_RSP_SNT]);
	printf("Cl_Data_GetState_Req_Rcvd  : %-5d\tCl_Data_GetState_Rsp_Snt  : %d\n", 
					ndmp_fp_msgStats[SSC_DATA_GETSTATE_REQ_RCVD], ndmp_fp_msgStats[SSC_DATA_GETSTATE_RSP_SNT]);
	printf("Cl_Data_Stop_Req_Rcvd      : %-5d\tCl_Data_Stop_Rsp_Snt      : %d\n", 
					ndmp_fp_msgStats[SSC_DATA_STOP_REQ_RCVD], ndmp_fp_msgStats[SSC_DATA_STOP_RSP_SNT]);
	printf("Cl_Data_Abort_Req_Rcvd     : %-5d\tCl_Data_Abort_Rsp_Snt     : %d\n\n", 
					ndmp_fp_msgStats[SSC_DATA_ABORT_REQ_RCVD], ndmp_fp_msgStats[SSC_DATA_ABORT_RSP_SNT]);

	printf("Cl_Mover_Listen_Req_Rcvd   : %-5d\tCl_Mover_Listen_Rsp_Snt   : %d\n", 
					ndmp_fp_msgStats[SSC_MOVER_LISTEN_REQ_RCVD], ndmp_fp_msgStats[SSC_MOVER_LISTEN_RSP_SNT]);
	printf("Cl_Mover_Conn_Req_Rcvd     : %-5d\tCl_Mover_Conn_Rsp_Snt     : %d\n", 
					ndmp_fp_msgStats[SSC_MOVER_CONN_REQ_RCVD], ndmp_fp_msgStats[SSC_MOVER_CONN_RSP_SNT]);
	printf("Cl_Mover_Contin_Req_Rcvd   : %-5d\tCl_Mover_Contin_Rsp_Snt   : %d\n", 
					ndmp_fp_msgStats[SSC_MOVER_CONT_REQ_RCVD], ndmp_fp_msgStats[SSC_MOVER_CONT_RSP_SNT]);
	printf("Cl_Mover_Rd_Req_Rcvd       : %-5d\tCl_Mover_Rd_Rsp_Snt       : %d\n", 
					ndmp_fp_msgStats[SSC_MOVER_RD_REQ_RCVD], ndmp_fp_msgStats[SSC_MOVER_RD_RSP_SNT]);
	printf("Cl_Mover_SetRecSiz_Req_Rcvd: %-5d\tCl_Mover_SetRecSiz_Rsp_Snt: %d\n", 
					ndmp_fp_msgStats[SSC_MOVER_SET_RECSIZ_REQ_RCVD], ndmp_fp_msgStats[SSC_MOVER_SET_RECSIZ_RSP_SNT]);
	printf("Cl_Mover_Set_Wdw_Req_Rcvd  : %-5d\tCl_Mover_Set_Wdw_Rsp_Snt  : %d\n", 
					ndmp_fp_msgStats[SSC_MOVER_SET_WDW_REQ_RCVD], ndmp_fp_msgStats[SSC_MOVER_SET_WDW_RSP_SNT]);
	printf("Cl_Mover_GetState_Req_Rcvd : %-5d\tCl_Mover_GetState_Rsp_Snt : %d\n", 
					ndmp_fp_msgStats[SSC_MOVER_GET_STATE_REQ_RCVD], ndmp_fp_msgStats[SSC_MOVER_GET_STATE_RSP_SNT]);
	printf("Cl_Mover_Abort_Req_Rcvd    : %-5d\tCl_Mover_Abort_Rsp_Snt    : %d\n", 
					ndmp_fp_msgStats[SSC_MOVER_ABORT_REQ_RCVD], ndmp_fp_msgStats[SSC_MOVER_ABORT_RSP_SNT]);
	printf("Cl_Mover_Stop_Req_Rcvd     : %-5d\tCl_Mover_Stop_Rsp_Snt     : %d\n", 
					ndmp_fp_msgStats[SSC_MOVER_STOP_REQ_RCVD], ndmp_fp_msgStats[SSC_MOVER_STOP_RSP_SNT]);
	printf("Cl_Mover_Close_Req_Rcvd    : %-5d\tCl_Mover_Close_Rsp_Snt    : %d\n\n", 
					ndmp_fp_msgStats[SSC_MOVER_CLOSE_REQ_RCVD], ndmp_fp_msgStats[SSC_MOVER_CLOSE_RSP_SNT]);

	printf("Unsupported_Msg_Rcvd       : %-5d\tUnsupported_Msg_Snt       : %d\n", 
					ndmp_fp_msgStats[UNSUPPORTED_MSG_RCVD], ndmp_fp_msgStats[UNSUPPORTED_MSG_SNT]);
	printf("-----------------------------------------------------------------\n");
}
/*-----------------------------------------------------------------
 * Name: 		ndmp_cmdMsgRcvdStat
 *
 * Description: Prints message received stats
 *
 *              ndmp msgstat - show for all current sessions
 * 
 * Created by : Anh Vu
 * 
 * Date Created: 07/24/02 
 *-----------------------------------------------------------------
 */
int32 ndmp_cmdMsgRcvdStat(int32 argc, char *argv[])
{

	if (argc == 2)
	{
		printf("Messages received stats:\n");
		printf("-----------------------\n");
		print_stats_rcvd();
    } else
		ndmp_cmdHelpCmd("msgstat");

    return 0;
}

/*-----------------------------------------------------------------
 * Name: ndmp_cmdPrintSIdTbl
 *
 * Description: Prints SessId table
 *
 * Created by : Anh Vu 
 * 
 * Date Created: 05/23/02
 *-----------------------------------------------------------------
 */
int32 ndmp_cmdPrintSIdTbl(int32 argc, char *argv[])
{
	print_sessid_tbl();
	return 0;
}

/*-----------------------------------------------------------------
 * Name: ndmp_cmd
 *
 * Description: Main dispatch for ndmp commands.  Called from
 *              command table.
 *
 *				ndmp show statistics
 *				ndmp debug level (0-2)
 * Created by : Anh Vu
 *
 * Date Created: 02/18/02
 *-----------------------------------------------------------------
 */
int32 ndmp_cmd ( int32 argc, char *argv[] )
{
	int32	i;

	if (argc < 2)
	{
		return (ndmp_cmdHelp(argc, argv));
	}

	for (i=0; i < NDMP_MAX_CMDS; i++)
	{
		if (strequ(argv[1], ndmp_cmdTable[i].cmd))
		{
			return ((*ndmp_cmdTable[i].cmdFunc)(argc, argv));
		}
	}
	return (ndmp_cmdHelp(argc, argv));
}

#endif	//NFP_FP
