/* vi:ts=4
 *-----------------------------------------------------------------
 *
 * Name:        ndmp-tpl.c
 *
 * RCS:  $Id: ndmp-tpl.c,v 1.1.1.1 2001/11/10 02:26:51 anhv Exp $
 *
 * Copyright (C) 2001, Agile Storage Inc.
 *
 * Description:
 *	Contain ndmp Transport Layer Protocol (TPL) interface functions.
 *
 * Created by: Anh Vu
 *
 * Date Created:        9/17/01
 *
 *-----------------------------------------------------------------
 *
#include <stdio.h>			/*REMOVE*/
#include <nfx-incl.h>
#include <exports/tpl-api.h>
#include <exports/evm_com_defs.h>



/*-----------------------------------------------------------------
 * Name:      	ndmp_tplAddConnInd	
 *
 * Description:
 *	Add connection indication from the transport layer to
 *	adds a new connection. 
 *
 * Created by:  Anh Vu
 *
 * Date Created:        9/17/01
 *
 *-----------------------------------------------------------------
 */
int32
ndmp_tplAddConnInd(void *connIdp, 	/*Pointer to newly added connection*/
		void **appHdlpp, 			/*Pointer to application handle 
					 				 *correspond to this connection
					 				 */
		uint32 raddr,				/*remote IP address*/
		ushort16 rport				/*remote port*/
)
{
	uint32			connId;

	NDMP_DBG_PRINTF("ndmp_tplAddConnInd 1\n");
	connId = (uint32)connIdp;

	/*
	 *Check for duplicate connection
	 */
	
	/*Filled in handle upon return to TPL
	 */

	NDMP_DBG_PRINTF("ndmp_tplAddConnInd \n");
	eee_sendMessage();

	return (NFX_OK);	
}

/*-----------------------------------------------------------------
 * Name:	ndmp_tplDelConnInd	
 *
 * Description: 
 * 	Delete connection indication from the transport layer to delete
 *	an existing connection.
 *
 * Created by:  Anh Vu
 *
 * Date Created:        9/17/01
 *
 *-----------------------------------------------------------------
 */
int32
ndmp_tplDelConnInd (void *appHdlp)
{
	return;
}

/*-----------------------------------------------------------------
 * Name:       	ndmp_tplRcvProcessPkt 
 *
 * Description:
 *	Process packet received from the transport layer.
 * Created by:  Anh Vu
 *
 * Date Create: 5/28/01
 *
 *-----------------------------------------------------------------
 */
int32	/*
	 	 *NFX_OK or NFX_ERROR
	 	 */
ndmp_tplRcvProcessPkt(
		nfp_clientCBPtr_t       clientCBp,
		rpcCallHdr_t			*pktRpcCallHdrp,
		pkt_desc_t *descp		/*start of message header*/
)
{
	int32                   rpcHdrSize;
	int32                   error;
	int32                   fieldLen;
	char                    *currp;
	nfp_requestCBPtr_t      reqCBp;
	ndmp_request_t           *ndmpReqp;
	ndmp_clientInfo_t        *ndmp_clientInfop;
	nfp_nodePtr_t           tailp;
	rpcCallHdr_t        	*newRpcCallp;

	NDMP_DBG_PRINTF("ndmp_tplRcvProcessPkt 1\n");

}

/*-----------------------------------------------------------------
 * Name:	ndmp_tplRcvPktInd 
 *
 * Description: 
 *	Receive packet indication from the transport layer for processing.
 *
 * Created by:  Anh Vu
 *
 * Date Create:	5/28/01
 *
 *-----------------------------------------------------------------
 */
int32
ndmp_tplRcvPktInd(void *appHdlp, pkt_desc_t *descp)
{
}

/*-----------------------------------------------------------------
 * Name: ndmp_tplXmtCmpInd       
 *
 * Description: 
 *	Transmit packet complete indication from the transport layer.
 *	 
 *
 * Created by:  Anh Vu
 *
 * Date Created:        5/28/01
 *
 *-----------------------------------------------------------------
 */
int32
ndmp_tplXmtCmpInd (void *appHdlp, pkt_desc_t *descp)
{

	NDMP_DBG_PRINTF("ndmp_tplXmtCmpInd 1: appHdlp: %08x descp: %08x\n", appHdlp,descp);
	return (TPL_FREE_PKT);
	
}


/*-----------------------------------------------------------------
 * Name: ndmp_tplBindReq
 *
 * Description:
 * Binds ndmp to Transport Layer (TCP).  
 *
 * Created by:  Anh Vu
 *
 * Date Created:        9/17/01
 *
 *-----------------------------------------------------------------
 */
int32 ndmp_tplBindReq(int32 address, ushort16 port, int bind_who)
{
	int32			ret = 0;
	uint32			laddr;
	uint32			tplProt;
	tpl_bindCb_t	bindCB;
	tpl_bindCb_t	*bindCBp;
	tpl_bindCb_t	*tplRspBindp;


	/*
	 *fill in all fields in tpl_bindCB_t
	 */
	bindCBp = &bindCB;
	memset(bindCBp, 0, sizeof(tpl_bindCb_t));
	bindCBp->laddr = 0;	
	bindCBp->lport = port;
	bindCBp->raddr = 0;	
	bindCBp->rport = port;
	bindCBp->tplProt = TPL_PROT_TCP;	
	bindCBp->ul_addConnInd = ndmp_tplAddConnInd;
	bindCBp->ul_delConnInd = ndmp_tplDelConnInd;
	bindCBp->ul_rcvPktInd = ndmp_tplRcvPktInd;
	bindCBp->ul_xmtCmpInd = ndmp_tplXmtCmpInd;
	bindCBp->userInfo = ;		// my Hdl
	
	return (tpl_bindReq(bindCBp, &tplRspBindp, bind_who));			
}

