/* vi:ts=4
 *-----------------------------------------------------------------
 *
 * Name:        ndmp-ui-api.h
 *
 * RCS:  $Id: ndmp-ui-api.h,v 1.15 2002/12/04 22:40:36 nitinj Exp $
 *
 * Copyright (C) 2001, Agile Storage Inc.
 *
 * Description: NDMP UI API definitions
 *
 * Created by:  Anh Vu
 *
 * Date Created:        04/05/02
 *
 *-----------------------------------------------------------------
 */
#if !defined(NDMP_UI_H)
#define NDMP_UI_H           

#define NDMP_ON		1
#define NDMP_OFF 	2
#define NDMP_MAX_SESS       64          /* max number of NDMP sessions allowed */
#define MAX_TAPE_ALIAS	10
#define MAX_TAPE_NAME	56

typedef enum {
	NDMP_MID_UI_ENABLE_NDMP = NDMP_MSG_BASE+60,	/* 9664 */
	NDMP_MID_UI_DISABLE_NDMP,
	NDMP_MID_UI_SHOW_STATE,
	NDMP_MID_UI_SHOW_STATISTICS,
	NDMP_MID_UI_SHOW_STATUS,
	NDMP_MID_UI_SHOW_DETAIL, 
	NDMP_MID_UI_SET_VERSION,
	NDMP_MID_UI_DEL_SESS,
	NDMP_MID_UI_GET_SESS_TBL,	
	NDMP_MID_UI_SHOW_PAR_STATS,
	NDMP_MID_UI_CLEAR_PAR_STATS	
} ndmp_uiMsgType_t;

typedef enum {
	UI_SHOW_STATISTICS_REQ_RCVD,					// UI received by Child process
	UI_SHOW_STATUS_REQ_RCVD,					
	UI_SHOW_DETAIL_REQ_RCVD,
	UI_DEL_SESS_REQ_RCVD,
	UI_CHLD_SET_DBG_LEVEL_REQ_RCVD,
	UI_CHLD_SET_TRACE_FLAG_REQ_RCVD,
	UI_DM_GET_STATE_REQ_RCVD,

	FP_CLOSE_SESS_RSP_RCVD,						// Responses/requests received by FP
	FP_MOVER_LISTEN_RSP_RCVD,					
	FP_MOVER_CONN_RSP_RCVD,						//9
	FP_MOVER_GET_STATE_RSP_RCVD,
	FP_MOVER_CONT_RSP_RCVD,
	FP_MOVER_ABORT_RSP_RCVD,
	FP_MOVER_CLOSE_RSP_RCVD,
	FP_MOVER_STOP_RSP_RCVD,
	FP_MOVER_SET_WDW_RSP_RCVD,
	FP_MOVER_READ_RSP_RCVD,
	FP_MOVER_SET_RECSIZ_RSP_RCVD,

	FP_DATA_LISTEN_RSP_RCVD,
	FP_DATA_CONN_RSP_RCVD,						
	FP_DATA_START_BKUP_RSP_RCVD,
	FP_DATA_START_RST_RSP_RCVD,
	FP_DATA_GET_STATE_RSP_RCVD,
	FP_DATA_SEND_NAME_REQ_SNT,
	FP_DATA_SEND_NAME_RSP_RCVD,					
	FP_DATA_GET_ENV_RSP_RCVD,
	FP_DATA_ABORT_RSP_RCVD,
	FP_DATA_STOP_RSP_RCVD,

	FP_MOVER_LISTEN_REQ_SNT,					// Requests sent to FP
	FP_MOVER_CONN_REQ_SNT,
	FP_MOVER_GET_STATE_REQ_SNT,					//30
	FP_MOVER_CONT_REQ_SNT,						
	FP_MOVER_ABORT_REQ_SNT,
	FP_MOVER_CLOSE_REQ_SNT,
	FP_MOVER_STOP_REQ_SNT,						
	FP_MOVER_SET_WDW_REQ_SNT,
	FP_MOVER_READ_REQ_SNT,
	FP_MOVER_SET_RECSIZ_REQ_SNT,

	FP_DATA_LISTEN_REQ_SNT,
	FP_DATA_CONN_REQ_SNT,
	FP_DATA_START_BKUP_REQ_SNT,
	FP_DATA_START_RST_REQ_SNT,					//41
	FP_DATA_GET_STATE_REQ_SNT,
	FP_DATA_GET_ENV_REQ_SNT,
	FP_DATA_ABORT_REQ_SNT,						
	FP_DATA_STOP_REQ_SNT,

	FP_NOTIFY_DATA_HALTED_REQ_RCVD,
	FP_NOTIFY_MOVER_HALTED_REQ_RCVD,
	FP_NOTIFY_DATA_RD_REQ_RCVD,
	FP_NOTIFY_MOVER_PAUSED_REQ_RCVD,
	FP_LOG_FILE_REQ_RCVD,
	FP_LOG_MSG_REQ_RCVD,						//51
	FP_FH_ADD_DIR_REQ_RCVD,
	FP_FH_ADD_NODE_REQ_RCVD,
	FP_SET_DMPDATE_REQ_RCVD,
	FP_UPD_ENV_REQ_RCVD,						
	FP_TAPE_SPACE_REQ_RCVD,

	RM_OPT_GET_OPTION_RSP_RCVD,					// Response received by Registry Mgr
	RM_OPT_GET_OPTION_REQ_SNT,					// Request sent to Registry Mgr
	FS_REQ_FSSTAT_VOL_RSP_RCVD,
	FS_REQ_FSSTAT_VOL_REQ_SNT,
	EVM_VOL_GET_NAME_LST_RSP_RCVD,
	EVM_VOL_GET_NAME_LST_REQ_SNT,
	EVM_VOL_GET_ID_RSP_RCVD,
	EVM_VOL_GET_ID_REQ_SNT,

	TAPE_OPEN_RSP_RCVD,							// Response received by Tape-Driver
	TAPE_CLOSE_RSP_RCVD,
	TAPE_SPACE_RSP_RCVD,
	TAPE_WR_FM_RSP_RCVD,
	TAPE_GET_STATE_RSP_RCVD,
	TAPE_LOAD_RSP_RCVD,
	TAPE_RD_RSP_RCVD,
	TAPE_WR_RSP_RCVD,
	TAPE_PASSTHRU_RSP_RCVD,
	TAPE_GET_DEVLST_RSP_RCVD,

	TAPE_OPEN_REQ_SNT,							// 71 Requests sent to Tape-Driver
	TAPE_CLOSE_REQ_SNT,
	TAPE_SPACE_REQ_SNT,
	TAPE_WR_FM_REQ_SNT,
	TAPE_GET_STATE_REQ_SNT,
	TAPE_LOAD_REQ_SNT,
	TAPE_RD_REQ_SNT,
	TAPE_WR_REQ_SNT,
	TAPE_PASSTHRU_REQ_SNT,
	TAPE_GET_DEVLST_REQ_SNT,

	MC_GET_DEVLST_RSP_RCVD,
	MC_OPEN_RSP_RCVD,
	MC_CLOSE_RSP_RCVD,
	MC_PASSTHRU_RSP_RCVD,

	MC_GET_DEVLST_REQ_SNT,
	MC_OPEN_REQ_SNT,
	MC_CLOSE_REQ_SNT,
	MC_PASSTHRU_REQ_SNT,

	MAX_SSC_CHLD_STAT
} ssc_chld_stat_t;

typedef enum {
	UI_ENABLE_REQ_RCVD,							// UI received by Parent process
	UI_DISABLE_REQ_RCVD,
	UI_CLEAR_PAR_STATISTICS_REQ_RCVD,
	UI_SHOW_PAR_STATISTICS_REQ_RCVD,
	UI_SHOW_STATE_REQ_RCVD,
	UI_GET_SESSTBL_REQ_RCVD,
	UI_SET_VER_REQ_RCVD,
	UI_PAR_SET_DBG_LEVEL_REQ_RCVD,
	CHILD_RPT_REQ_RCVD,
	CHILD_SND_STATS_REQ_RCVD,
	EVENT_POST_REQ_RCVD,
	UI_SET_TRACE_FLAG_REQ_RCVD,
	UI_SET_TRACE_FILES_REQ_RCVD,
	UI_SET_RESTORE_MODE_REQ_RCVD,

	MAX_SSC_PAR_STAT

} ssc_par_stat_t;

#define MAX_SSC_TOT_STAT	(MAX_SSC_PAR_STAT + MAX_SSC_CHLD_STAT)

// Command table entry
typedef struct {
    char *cmd_name;
    int  arg_cnt;   /* arg count not counting 1st cmd in cmd_name */
    int  (*proc_func)(int, char **);
    char *usage;
} ndmp_UIEnt_t;    

// Session Id Table Entry 
typedef struct
{
    int32   in_use;             // TRUE/FALSE
    int32   pid;                // which client process is using this sessId
	int32	sessId;
	int32	ndmp_trace_msg;
    int32   sessDel;
    struct sockaddr_in  from;   // address of NDMP client
    char    app_name[80];
} sessId_entry_t;


/*------------- Scsi ---------------*/
typedef struct scsi_ui
{
	int32		hdl;            /* tape device handle */
    char		devname[MAX_TAPE_ALIAS];    /* alias */
    char		phys_devname[MAX_TAPE_NAME];

} Scsi_ui_t;
/*------------- Tape ---------------*/
typedef struct tape_ui
{
	int32		hdl;            /* tape device handle */
	int32		mode;           /* tape device open mode */
#define TP_MODE_NONE          0
#define TP_MODE_RD_ONLY       1
#define TP_MODE_RD_WR         2

    int32		status;
    char		devname[MAX_TAPE_ALIAS];    /* alias */
    char		phys_devname[MAX_TAPE_NAME];
						/* wwn (16 bits LUN + 48 bits WWN = 16chars) +
						 *  3 chars for Rewind +
						 *  3 chars for Unload +
						 * 32 chars for density
						 */
} Tape_ui_t;

#define ADDR_TYP_LOCAL	0
#define ADDR_TYP_TCP	1

/*------------- Mover ---------------*/

typedef struct mover_ui
{
	int32	state;
#define MOVER_ST_IDLE	0
#define MOVER_ST_LISTEN	1
#define MOVER_ST_ACTIVE	2
#define MOVER_ST_PAUSED	3
#define MOVER_ST_HALTED	4

	int32	pause_reason;
#define MOVER_PAUSE_NA		0
#define MOVER_PAUSE_EOM		1
#define MOVER_PAUSE_EOF		2
#define MOVER_PAUSE_SEEK	3
#define MOVER_PAUSE_MEDIA_ERROR  4
#define MOVER_PAUSE_EOW  	5

	int32	halt_reason;
#define MOVER_HALT_NA  0
#define MOVER_HALT_CONNECT_CLOSED  1
#define MOVER_HALT_ABORTED  2
#define MOVER_HALT_INTERNAL_ERROR  3
#define MOVER_HALT_CONNECT_ERROR  4

	int32	recsiz;
	int32	recnum;
	uint64	data_written;
	uint64	seek_pos;
	uint64	bytes_left_to_read;
	uint64	window_offset;
	uint64	window_len;
	int32	addr_type;
	int32	mode;	/* (NOACTION, READ, WRITE) */
#define MVR_MODE_READ 0
#define MVR_MODE_WRITE 1

} Mover_ui_t;


/*------------- Data ---------------*/

typedef struct data_ui
{
	int32	state;
#define DATA_ST_IDLE	0
#define DATA_ST_ACTIVE	1
#define DATA_ST_HALTED	2
#define DATA_ST_LISTEN	3
#define DATA_ST_CONNECTED	4

	int32	op;
#define DATA_OP_NOACTION	0
#define DATA_OP_BACKUP		1
#define DATA_OP_RESTORE		2

	int32	halt_reason;
#define DATA_HALT_NA  			0
#define DATA_HALT_SUCCESSFUL 	1
#define DATA_HALT_ABORTED  		2
#define DATA_HALT_MEDIA_ERROR  	3
#define DATA_HALT_INTERNAL_ERROR  4

	uint64	bytes_processed;
	uint64	read_offset;
	uint64	read_length;
	int32	addr_type;
} Data_ui_t;

/*------------- NDMP Session Data Structure ---------------*/
typedef struct ndmp_Sexxion
{
    boolean             authed;         /* Is this connection authorized yet? */
    boolean             auth_typ;       /* auth type requested */
#define AUTHNONE	0
#define AUTHTXT		1
#define AUTHMD5		2
    char8               protoVer;
	boolean				trace;
	int32				trace_files;
    int32               sessId;
	struct sockaddr_in  host_ipaddr;
	char                start_time[20];
	Tape_ui_t			Tape;	
	Scsi_ui_t			SCSI;	
	Mover_ui_t			Mover;	
	Data_ui_t			Data;	
} ndmp_Session_t;

typedef struct ndmp_NFXGenUIReq
{
	uint32		mid;	/* NDMP_MID_UI_ENABLE_NDMP,
						 * NDMP_MID_UI_DISABLE_NDMP,
						 * NDMP_MID_UI_SHOW_STATE,
						 * NDMP_MID_UI_GET_SESS_TBL
						 * NDMP_MID_UI_SHOW_PAR_STATS
						 */
} ndmp_NFXGenericUIReq_t; 

typedef struct ndmp_NFXGen2UIReq
{
	uint32		mid;	
	int32		sessId;
} ndmp_NFXGen2UIReq_t; 

typedef struct ndmp_NFXGen2UIRsp
{
	uint32		mid;	
	int32		stat;
} ndmp_NFXGen2UIRsp_t; 

typedef struct ndmp_NFXGetTblRsp
{
	uint32		mid;	/* NDMP_MID_UI_GET_SESS_TBL */
	sessId_entry_t  tbl[NDMP_MAX_SESS];

} ndmp_NFXGetTblRsp_t;

typedef struct ndmp_NFXDERsp
{
	uint32		mid;	/* NDMP_MID_ENABLE_NDMP or NDMP_MID_DISABLE_NDMP */
	int32		state;	/* 1 for enable, 2 for disabled */
	int32		running_sess;
	int32		already_so;
} ndmp_NFXDisEnaRsp_t; 

typedef struct ndmp_NFXShStRsp
{
	uint32		mid;	/* NDMP_MID_UI_SHOW_STATE */
	int32		state;	/* 1 for enable, 2 for disabled */
	int32		version;/* Current highest version we support */
	int32		trace;	/* trace msgs between client/server */
	int32		tracefile;/* # trace files we keep */
    int32       restoreMode; /* remove most common path or not during restore */
} ndmp_NFXShowStateRsp_t; 

typedef struct ndmp_NFXDSReq
{
	uint32		mid;	/* NDMP_MID_DEL_SESS */
	int32		sessId;	/* If -1, means all sessions */
    char        reason[64]; /* Add reason text */
} ndmp_NFXDelSessReq_t;

typedef struct ndmp_NFXDSRsp
{
	uint32		mid;	/* NDMP_MID_DEL_SESS */
	int32		stat;	/* NFX_ERR or NFX_OK, 
						 * leave 'stat' as 2nd var in struct */
} ndmp_NFXDelSessRsp_t;


typedef struct ndmp_NFXShowReq
{
	uint32		mid;	/* NDMP_MID_UI_SHOW_STAT or
						 * SHOW_DETAIL 
						 */
	int32		sessId;	/* If -1, means all sessions */
} ndmp_NFXShowReq_t;


typedef struct ndmp_NFXShowChildRsp
{
	uint32		mid;	/* NDMP_MID_UI_SHOW_STATUS or 
						 * NDMP_MID_UI_SHOW_DETAIL 
						 */
	int32		stat;	/* NFX_ERR or NFX_OK */
	ndmp_Session_t  sess;
} ndmp_NFXShowChildRsp_t;

typedef struct ndmp_NFXShowCStatsRsp
{
	uint32		mid;	/* NDMP_MID_UI_SHOW_STATISTIC or */
	int32		stat;	/* NFX_ERR or NFX_OK */
	int32		sessId;
	int32		statistic[MAX_SSC_CHLD_STAT];
} ndmp_NFXShowChildStatsRsp_t;

typedef struct ndmp_NFXShowPStatsRsp
{
	uint32		mid;	/* NDMP_MID_UI_SHOW_PAR_STATS */
	int32		stat;	/* NFX_ERR or NFX_OK */
	int32		statistic[MAX_SSC_PAR_STAT+MAX_SSC_CHLD_STAT];
} ndmp_NFXShowParStatsRsp_t;

typedef struct ndmp_NFXSVerReq
{
	uint32		mid;	/* NDMP_MID_UI_SET_VERSION */
	int32		version;/* highest version NDMP server should support */
} ndmp_NFXSetVerReq_t;

typedef struct ndmp_NFXSVerRsp
{
	uint32		mid;	/* NDMP_MID_UI_SET_VERSION */
	int32		stat;	/* NFX_ERR or NFX_OK */
	int32		already_so; /* TRUE if version is already set as in specified in request */
} ndmp_NFXSetVerRsp_t;

typedef struct ndmp_NFXSDebugReq
{
	uint32		mid;	/* NDMP_MID_UI_SET_DBG_LEVEL */
	int32		sessId;	/* If -1, means all sessions */
	int32		dbg_level;	/* debug level */
} ndmp_NFXSetDebugReq_t;

typedef struct ndmp_NFXSRestoreModeReq
{
	uint32		mid;	/* NDMP_MID_UI_SET_RESTORE_MODE */
	int32		flag;	/* 0 - Default mode, 1 - eliminate common path */
} ndmp_NFXSetRestoreModeReq_t;

typedef struct ndmp_NFXSRestoreModeRsp
{
	uint32		mid;	/* NDMP_MID_UI_SET_RESTORE_MODE */
	int32		stat;
} ndmp_NFXSetRestoreModeRsp_t;

typedef struct ndmp_NFXSDebugRsp
{
	uint32		mid;	/* NDMP_MID_UI_SET_DBG_LEVEL */
	int32		stat;	/* NFX_ERR or NFX_OK, 
						   leave 'stat' as 2nd var in struct */
	int32		sessId;	/* If -1, means all sessions */
} ndmp_NFXSetDebugRsp_t;

typedef struct ndmp_NFXSMsgTraceReq
{
    uint32      mid;   /* NDMP_MID_UI_SET_TRACE_ON_OFF */
	int32		sessId;	/* If -1, means all sessions */
    int32       flag;  /* 0 - OFF; 1 - ON */
    int32       num_files;  /* number of trace log files to retain */
} ndmp_NFXSMsgTraceReq_t;

typedef struct ndmp_NFXSMsgTraceRsp
{
    uint32      mid;   /* NDMP_MID_UI_SET_TRACE_ON_OFF */
    int32       stat;  /* NFX_ERR or NFX_OK 
						  leave 'stat' as 2nd var in struct */
	int32		sessId;	/* If -1, means all sessions */
} ndmp_NFXSMsgTraceRsp_t;


/* UI process functions */

int ndmp_UI_EnaDis(int argc, char **agrv) ;
int ndmp_UI_ShowActive(int argc, char **agrv);
int ndmp_UI_ShowState(int argc, char **agrv);
int ndmp_UI_ShowStats(int argc, char **agrv);
int ndmp_UI_ShowParStats(int argc, char **agrv);
int ndmp_UI_ClearParStats(int argc, char **agrv);
int ndmp_UI_ShowStatus(int argc, char **agrv);
int ndmp_UI_ShowDetail(int argc, char **agrv);
int ndmp_UI_ShowAuthClient(int argc, char **agrv);
int ndmp_UI_ShowAuthSrvr(int argc, char **agrv);
int ndmp_UI_SetVer(int argc, char **argv);
int ndmp_UI_DelSess(int argc, char **agrv);
int ndmp_UI_SetDbgLevel(int argc, char **argv);
int ndmp_UI_SetMsgTrace(int argc, char **argv);
int ndmp_UI_SetMsgTraceFile(int argc, char **argv);
int ndmp_schedule_clean_up(int argc, char **argv);
int ndmp_schedule_clean_up_show(int argc, char **argv);
int ndmp_schedule_clean_up_del(int argc, char **argv);
int ndmp_restore_path(int argc, char **argv);

#endif /* NDMP_UI_API_H */     

