/* vi:ts=4
 *-----------------------------------------------------------------
 *
 * Name:        ndmp.x
 *
 * RCS:  $Id: ndmp_xdr.x,v 1.7 2002/11/01 19:45:39 anhv Exp $
 *
 * Copyright (C) 2001, Agile Storage Inc.
 *
 * Description:
 *
 * Created by: Anh Vu
 *
 * Date Created:        8/20/01
 *
 *-----------------------------------------------------------------
 */

const NDMP_VER_2 = 2;
const NDMP_VER_3 = 3;
const NDMP_SVR_PORT = 10000;

struct ndmp_u_quad
{
	u_long	high;
	u_long	low;
};


enum ndmp_HdrMsgTyp
{
	NDMP_MSG_REQ,
	NDMP_MSG_RSP
};

enum ndmp_Err
{
    NDMP_NO_ERR,
    NDMP_NOT_SUPPORTED_ERR,		/* Msg not supported */
    NDMP_DEVICE_BUSY_ERR,		/* Specified device is in use */
    NDMP_DEVICE_OPENED_ERR,		/* Device is already opened */
    NDMP_NOT_AUTHORIZED_ERR,	/* Control connection is not yet authenticated */
    NDMP_PERMISSION_ERR,		/* Username used to authenticate the connection doesn't have
								 * access to execute this message.
								 */	
    NDMP_DEV_NOT_OPEN_ERR,		/* Error trying to access a device that wasn't opened */
    NDMP_IO_ERR,				/* Device I/O error */
    NDMP_TIMEOUT_ERR,			/* Command timeout error */
    NDMP_ILLEGAL_ARGS_ERR,		/* Msg received contained invalid arguments */
    NDMP_NO_TAPE_LOADED_ERR,	/* Tape device can't be opened cause no tape's loaded */
    NDMP_WRITE_PROTECT_ERR,		/* Tape can't be opened in write mode cause tape is write 
								 * protected. 
								 */
    NDMP_EOF_ERR,				/* Tape command failed due to end of Filemark found */
    NDMP_EOM_ERR,				/* Tape command failed due to End of Media was encountered */
    NDMP_FILE_NOT_FOUND_ERR,	/* File not found during a recovery operation */
    NDMP_BAD_FILE_ERR,			/* Invalid file descriptor */
    NDMP_NO_DEVICE_ERR,			/* Device does not exist */
    NDMP_NO_BUS_ERR,			/* Specified SCSI controller does not exist */
    NDMP_XDR_DECODE_ERR,		/* Error decoding the XDR message */
    NDMP_ILLEGAL_STATE_ERR,		/* Message cannot be processed in current state */
    NDMP_UNDEFINED_ERR,			/* Undefined error */
    NDMP_XDR_ENCODE_ERR,		/* Error encode the message */
    NDMP_NO_MEM_ERR,			/* Memory allocation error */
    NDMP_CONNECT_ERR			/* Error connecting to another NDMP server */

};

enum ndmp_Msg
{

	/***** Management service *****/
	/* Connect interface commands */
    NDMP_CONNECT_OPEN 				= 0x900,
    NDMP_CONNECT_CLIENT_AUTH 		= 0x901,
    NDMP_CONNECT_CLOSE 				= 0x902,
    NDMP_CONNECT_SERVER_AUTH 		= 0x903,

	/* Configuration interface commands */
    NDMP_CONFIG_GET_HOST_INFO 		= 0x100,
	NDMP_CONFIG_GET_BUTYPE_ATTR     = 0x101,		/* v2 only */
    NDMP_CONFIG_GET_CONNECTION_TYPE = 0x102,		/* same as NDMP_CONFIG_GET_MOVER_TYPE for v2 */
    NDMP_CONFIG_GET_AUTH_ATTR 		= 0x103,		
    NDMP_CONFIG_GET_BUTYPE_INFO 	= 0x104,		/* v3, v4 only */
    NDMP_CONFIG_GET_FS_INFO 		= 0x105,		/* v3, v4 only */
    NDMP_CONFIG_GET_TAPE_INFO 		= 0x106,		/* v3, v4 only */
    NDMP_CONFIG_GET_SCSI_INFO 		= 0x107,		/* v3, v4 only */
    NDMP_CONFIG_GET_SERVER_INFO 	= 0x108,		/* v3, v4 only */
	NDMP_CONFIG_SET_EXT_LIST		= 0x109, 		/* v4 only */
	NDMP_CONFIG_GET_EXT_LIST		= 0x10a, 		/* v4 only */

	/* Notify interface commands */
	NDMP_NOTIFY_RESVD1              = 0x500,		/* v2 only */
    NDMP_NOTIFY_DATA_HALTED 		= 0x501,
    NDMP_NOTIFY_CONNECTED 			= 0x502,
    NDMP_NOTIFY_MOVER_HALTED 		= 0x503,
    NDMP_NOTIFY_MOVER_PAUSED 		= 0x504,
    NDMP_NOTIFY_DATA_READ 			= 0x505,

	/* Log interface commands */
	NDMP_LOG_LOG					= 0x600,		/* v2 only */
	NDMP_LOG_DBG					= 0x601,		/* v2 only */
    NDMP_LOG_FILE 					= 0x602,		/* v2, v3, v4 */
    NDMP_LOG_MESSAGE 				= 0x603,		/* v3, v4 */

	/* File History interface commands */
	NDMP_FH_ADD_UNIX_PATH           = 0x700,		/* v2 only */
	NDMP_FH_ADD_UNIX_DIR            = 0x701,		/* v2 only */
	NDMP_FH_ADD_UNIX_NODE           = 0x702,		/* v2 only */
    NDMP_FH_ADD_FILE 				= 0x703,
    NDMP_FH_ADD_DIR 				= 0x704,
    NDMP_FH_ADD_NODE 				= 0x705,

	/***** SCSI service commands *****/
    NDMP_SCSI_OPEN 					= 0x200,
    NDMP_SCSI_CLOSE 				= 0x201,
    NDMP_SCSI_GET_STATE 			= 0x202,		/* not supported */
    NDMP_SCSI_SET_TARGET 			= 0x203,		/* not supported, not in v4 */
    NDMP_SCSI_RESET_DEVICE 			= 0x204,		/* not supported */
    NDMP_SCSI_RESET_BUS 			= 0x205,		/* not supported, not in v4 */
    NDMP_SCSI_EXECUTE_CDB 			= 0x206,

	/***** Tape service commands *****/
    NDMP_TAPE_OPEN 					= 0x300,
    NDMP_TAPE_CLOSE 				= 0x301,
    NDMP_TAPE_GET_STATE 			= 0x302,
    NDMP_TAPE_MTIO 					= 0x303,
    NDMP_TAPE_WRITE 				= 0x304,
    NDMP_TAPE_READ 					= 0x305,
	NDMP_TAPE_RESVD1				= 0x306,		/* v2 only */
    NDMP_TAPE_EXECUTE_CDB 			= 0x307,

	/***** Data service commands *****/
    NDMP_DATA_GET_STATE 			= 0x400,
    NDMP_DATA_START_BACKUP 			= 0x401,
    NDMP_DATA_START_RECOVER 		= 0x402,
    NDMP_DATA_ABORT 				= 0x403,
    NDMP_DATA_GET_ENV 				= 0x404,
	NDMP_DATA_RESVD1				= 0x405,		/* v2 only */
	NDMP_DATA_RESVD2				= 0x406,		/* v2 only */
    NDMP_DATA_STOP 					= 0x407,
    NDMP_DATA_LISTEN 				= 0x409,		/* v3, v4 */
    NDMP_DATA_CONNECT 				= 0x40a,		/* v3, v4 */
    NDMP_DATA_START_RECOVER_FILEHIST= 0x40b,		/* v4 only */
	

	/***** Mover service commands *****/
    NDMP_MOVER_GET_STATE 			= 0xa00,
    NDMP_MOVER_LISTEN 				= 0xa01,
    NDMP_MOVER_CONTINUE 			= 0xa02,
    NDMP_MOVER_ABORT 				= 0xa03,
    NDMP_MOVER_STOP 				= 0xa04,
    NDMP_MOVER_SET_WINDOW 			= 0xa05,
    NDMP_MOVER_READ 				= 0xa06,
    NDMP_MOVER_CLOSE 				= 0xa07,
    NDMP_MOVER_SET_RECORD_SIZE 		= 0xa08,
    NDMP_MOVER_CONNECT 				= 0xa09,		/* v3, v4 only */

	/* Reserved for vendor specific usage commands.  From 0xf000 - 0xfeff */
    NDMP_VENDORS_BASE 				= 0xf000,

	/* Reserved for prototyping.  From 0xff00 - 0xffff */
    NDMP_RESERVED_BASE 				= 0xff00,

	/* EFX proprietary commands */
	NDMP_NFX_NEW_CLIENT				= 0x001,		/* Notify N-NIM of new session to create
												     * Data and Mover state machines 
													 */
	NDMP_NFX_EFS_DONE				= 0x002,		/* EFS is done w/ backup or recovery */
	NDMP_NFX_TAPE_OPS				= 0x003,		/* Request for Tape service last ops on
													 * Tape by Mover.
													 */
	NDMP_NFX_DATA_READY				= 0x004,		/* Data or Mover notify one another data is
													 * ready in the SAC.
													 */
	NDMP_NFX_DEV_OPENED				= 0x005			/* Tape drive notify Mover that a tape device
													 * is opened.
													 */
};

struct ndmp_Hdr
{
	u_long				xmtSeq;			/* Sequence number of msgs sent for this connection.
										 * Starts from 1 up
										 */
	u_long				time_stamp;		/* Time stamp for time in seconds since 00:00:00 GMT
										 * Jan 1, 1970, that msg was sent.
										 */
	ndmp_HdrMsgTyp		msgtyp;			/* what type of message (request/response) */
	enum ndmp_Msg		msgId;			/* message id */
	u_long				rspSeq;			/* In request msg = 0.
										 * In reply msg = sequence of msg we're replying to.
										 */
	ndmp_Err			error;			/* communications errors */
};

/**********************************************************************************************/
/******************************* MANAGEMENT Services structures *******************************/

/*********** CONNECT interface messages structures ***********/

/* --NDMP_CONNECT_OPEN-- */
struct ndmp_conn_open_req
{
	u_short	protoVer;					/* Protocol version for negotiation, sent by client */
};

struct ndmp_conn_open_rsp
{
	ndmp_Err	error;
};

/* --NDMP_CONNECT_CLIENT_AUTH-- */
/* Authentication types allowed */
enum ndmp_AuthTyp
{
	NDMP_AUTHNONE,		/* no password is required */
	NDMP_AUTHTXT,		/* the clear text password */
	NDMP_AUTHMD5		/* md5 */
};

struct ndmp_AuthTxt
{
	string	authId<>;			/* Username */
	string	authPswd<>;			/* 1-32 bytes */

};

struct ndmp_AuthMd5
{
	string	authId<>;			/* Username */
	opaque	authDigest[16];		/* Digest calculated from username & challenge string */
};

union ndmp_AuthData switch (enum ndmp_AuthTyp authTyp)
{
	case NDMP_AUTHNONE:
		void;
	case NDMP_AUTHTXT:
		struct ndmp_AuthTxt	authTxt;
	case NDMP_AUTHMD5:
		struct ndmp_AuthMd5	authMd5;
};

struct ndmp_conn_client_auth_req
{
	ndmp_AuthData	auth_data;
};

struct ndmp_conn_client_auth_rsp
{
	ndmp_Err	error;
};


/* --NDMP_CONNECT_CLOSE-- */
/* no arguments for request or response */


/* --NDMP_CONNECT_SERVER_AUTH-- */
union ndmp_AuthAttr switch (enum ndmp_AuthTyp authTyp)
{
	case NDMP_AUTHNONE:
		void;
	case NDMP_AUTHTXT:
		void;
	case NDMP_AUTHMD5:
		opaque	challenge[64];
};

struct ndmp_conn_srvr_auth_req
{
	ndmp_AuthAttr	client_attr;
};

struct ndmp_conn_srvr_auth_rsp
{
	ndmp_Err		error;
	ndmp_AuthData	srvr_result;
};


/********* CONFIGURATION interface messages structures ********/

/* --NDMP_CONFIG_GET_HOST_INFO-- */
/* no arguments for request */

struct ndmp_cfg_get_host_info_rsp
{
	ndmp_Err		error;
	string			hostname<>;	/* host name of NDMP server */
	string			os_type<>;	/* Name of NDMP server operating system (ex: SOLARIS) */
	string			os_vers<>;	/* Version of NDMP server operation system (ex: 2.5) */
	string			hostid<>;	/* NDMP server host identifier */
};

/* --NDMP_CONFIG_GET_SERVER_INFO-- */
/* no argument for request */

struct ndmp_cfg_get_srvr_info_rsp
{
	ndmp_Err		error;
	string			vendorName<>; /* Name of vendor that implements the NDMP server */
	string			prodName<>;	  /* Product name of NDMP server provided by the vendor */
	string			revNum<>;	  /* Current revision # of NDMP server provided by vendor */
	ndmp_AuthTyp	authTyp<>;	  /* Connection authentication types supported by server */
};

/* --NDMP_CONFIG_GET_CONNECTION_TYPE-- */
enum ndmp_AddrTyp
{
	NDMP_ADDR_LOCAL,			/* Tape and Data service are on same host */
	NDMP_ADDR_TCP				/* Tape and Data service are on separate hosts  */
};

/* no argument for request */

struct ndmp_cfg_get_conn_type_rsp
{
	ndmp_Err		error;
	ndmp_AddrTyp	addrTypes<>;
};

/* --NDMP_CONFIG_GET_AUTH_ATTR-- */
struct ndmp_cfg_get_auth_attr_req
{
	ndmp_AuthTyp	authTyp;
};

struct ndmp_cfg_get_auth_attr_rsp
{
	ndmp_Err		error;
	ndmp_AuthAttr	srvrAttr;	/* Attributes required for specific auth scheme.  For ex:
								 * for MD5, server returns a per session challenge.
								 */
};


/* --NDMP_CONFIG_GET_BUTYPE_INFO-- */

/* Bkup type attributes - 
 * these bit masks indicates the bkup type capabilities 
 */
const NDMP_BUTYPE_BACKUP_FILE_HISTORY	= 0x0001;	/* file history is supported */
const NDMP_BUTYPE_BACKUP_FILELIST		= 0x0002;	/* archiving of selective files as specified 
													 * by file list */
const NDMP_BUTYPE_RECOVER_FILELIST		= 0x0004;	/* restoration of individual files supported */
const NDMP_BUTYPE_BACKUP_DIRECT			= 0x0008;	/* provides file history info for direct access restore *
const NDMP_BUTYPE_RECOVER_DIRECT		= 0x0010;	/* supporst direct access restore (positioning to the
													 * offset of bkup image and restoring specified file
													 */ 
const NDMP_BUTYPE_BACKUP_INCREMENTAL	= 0x0020;	/* incremental bkup is supported */
const NDMP_BUTYPE_RECOVER_INCREMENTAL	= 0x0040;	/* supports incremental only restoration 
													 * a full restore must be done prior to incremental
													 * restore.
													 */
const NDMP_BUTYPE_BACKUP_UTF8			= 0x0080;	/* supports UTF8 fmt in file history. */
const NDMP_BUTYPE_RECOVER_UTF8			= 0x0100;	/* supports UTF8 fmt in restored file list. */
 
struct ndmp_pval
{
	string	name<>;
	string	value<>;
};

struct ndmp_butype_info
{
	string		butype_name<>;		/* Name of backup (dump, tar, cpio) */
	ndmp_pval	defEnv<>;			
	/* Default environment vars such as Prefix, data type, userid, */
	/* direct access flag, file history flag,  fs name, dump level, */
	/* -x or -y extract, update dumpdates file flag. */
	u_long		attrs;				/* Bkup attributes bit mask, described above */
};

/* no argument for request */

struct ndmp_cfg_get_butype_info_rsp 
{
	ndmp_Err			error;
	ndmp_butype_info	butype_info<>;
};

/* --NDMP_CONFIG_GET_FS_INFO-- */

/* invalid bit */
const	NDMP_FS_INFO_TOTAL_SIZE_INVALID 	= 0x00000001;
const	NDMP_FS_INFO_USED_SIZE_INVALID		= 0x00000002;
const	NDMP_FS_INFO_AVAIL_SIZE_INVALID		= 0x00000004;
const	NDMP_FS_INFO_TOTAL_INODES_INVALID	= 0x00000008;
const	NDMP_FS_INFO_USED_INODES_INVALID	= 0x00000010;

struct ndmp_fs_info
{
	u_long		invalid;			/* identify unsupported args in msg */
	string		fs_type<>;			/* type of fs */
	string		fs_logical_device<>;/* mount point or share name of fs */
	string		fs_physical_device<>;/* physical dev name of fs, e.g. /dev/rsd0c */
	ndmp_u_quad	total_size;			/* fs total siz in bytes, */
									/* DMP_FS_INFO_TOTAL_SIZE_INVALID */
									/* is set if this argument is not supported */
	ndmp_u_quad	used_size;			/* in bytes, NDMP_FS_INFO_USED_SIZE_INVALID 		
									 * is set if this argument is not supported */
	ndmp_u_quad	avail_size;			/* in bytes, NDMP_FS_INFO_AVAIL_SIZE_INVALID
									 * is set if this argument is not supported */
	ndmp_u_quad	total_inodes;		/* in bytes, NDMP_FS_INFO_TOTAL_INODES_INVALID
                                     * is set if this argument is not supported */
	ndmp_u_quad	used_inodes;		/* in bytes, NDMP_FS_INFO_USED_INODES_INVALID
                                     * is set if this argument is not supported */
	ndmp_pval	fs_env<>;			/* environment vars defined for fs */
	string		fs_status<>;		/* curr status of fs */
};

/* no argument for request */

struct ndmp_cfg_get_fs_info_rsp
{
	ndmp_Err		error;
	ndmp_fs_info	fs_info<>;
};

/* --NDMP_CONFIG_GET_TAPE_INFO-- */
/* no argument for request */

/* tape attributes */
const NDMP_TAPE_ATTR_REWIND		= 0x00000001;
const NDMP_TAPE_ATTR_UNLOAD	 	= 0x00000002;	

struct ndmp_dev_capab
{
	string		devname<>;	/* device name of tape drive, WWN */
	u_long		attr;		/* bitmask described above */
	ndmp_pval	capab<>;	/* what tape drive can do, compression 1, 2, etc. */
};

struct ndmp_dev_info
{
	string					model<>;  /* manufacture name of tape device, for ex: DLT4000. */
	ndmp_dev_capab			caplist<>;/* list of capabilities */ 
};

struct ndmp_cfg_get_tape_info_rsp 
{
	ndmp_Err			error;
	ndmp_dev_info		tape_info<>;	/* list of tape devices and its capabilities */

};

/* --NDMP_CONFIG_GET_SCSI_INFO-- */
/* No SCSI attributes defined yet */

/* no argument for request */

struct ndmp_cfg_get_scsi_info_rsp
{
	ndmp_Err			error;
	ndmp_dev_info		scsi_info<>;	/* list of SCSI dev and their capabilities */
										/* attributes for SCSI is not defined yet  */
};

/********* NOTIFY Interface messages structures *********/

/* --NDMP_NOTIFY_DATA_HALTED-- */

enum ndmp_data_halt_reason
{
	NDMP_DATA_HALT_NA,				/* data op not in progress or not in halt state */
	NDMP_DATA_HALT_SUCCESSFUL,		/* data op completed */
	NDMP_DATA_HALT_ABORTED,			/* data op aborted by client */
	NDMP_DATA_HALT_INTERNAL_ERROR,	/* data op halted due to unrecov. err incurred by NDMP server*/
	NDMP_DATA_HALT_CONNECT_ERROR	/* data op has err w/ connection to tape server */
};

struct ndmp_notify_data_halted_req
{
	ndmp_data_halt_reason	reason;	/* reason why halted */
	string					text_reason<>;	/* diagnostic err msg, srvr indep. implementation. */
};

/* --NDMP_NOTIFY_CONNECTED-- */
enum ndmp_conn_reason
{
	NDMP_CONNECTED,		/* Connect sucessfully */
	NDMP_SHUTDOWN,		/* Connection shutdown */
	NDMP_REFUSED		/* reach the maximum number of connections */
};

struct ndmp_notify_conn_req
{
	ndmp_conn_reason	reason;		/* reason for curr conn. state */
	u_short				protoVer;	/* protocol version used */
	string				text_reason<>; /* srvr implementation dependent 
										* msg saying why conn is shutdown 
										* or refused */
};

enum ndmp_mover_pause_reason
{
	NDMP_MOVER_PAUSE_NA,
	NDMP_MOVER_PAUSE_EOM,
	NDMP_MOVER_PAUSE_EOF,
	NDMP_MOVER_PAUSE_SEEK,
	NDMP_MOVER_PAUSE_MEDIA_ERROR,
	NDMP_MOVER_PAUSE_EOW				/* this one is not in v2 */
};

enum ndmp_mover_halt_reason
{
	NDMP_MOVER_HALT_NA,
	NDMP_MOVER_HALT_CONNECT_CLOSED,
	NDMP_MOVER_HALT_ABORTED,
	NDMP_MOVER_HALT_INTERNAL_ERROR,
	NDMP_MOVER_HALT_CONNECT_ERROR
};


/* NDMP_NOTIFY_MOVER_PAUSED */
struct ndmp_notify_mover_paused_req
{
	ndmp_mover_pause_reason	reason;			/* reason why paused */
	ndmp_u_quad				seek_position;	/* if reason = PAUSE_SEEK, means desired 
											 * data stream seek position.  Client should 
											 * load tape containing requested seek_position, 
											 * position tape appropriately, set new data 
											 * window, and then continue tape server. 
											 */
};

/* No response */

/* --NDMP_NOTIFY_MOVER_HALTED-- */
struct ndmp_notify_mover_halted_req
{
	ndmp_mover_halt_reason	reason;			/* reason why halted */
	string					text_reason<>;  /* srvr independent diagnostic msg */
};
/* No response */

/* --NDMP_NOTIFY_DATA_READ-- */
struct ndmp_notify_data_read_req
{
	ndmp_u_quad	offset;						/* offset of 1st byte that should be sent to 
											 * data conn */
	ndmp_u_quad	length;						/* bytes tape server should read from tape 
											 * & sent to data conn */
};
/* No response */

/********* LOG Interface messages structures *********/

/* --NDMP_LOG_MESSAGE-- */
enum ndmp_log_type
{
	NDMP_LOG_NORMAL,
	NDMP_LOG_DEBUG,
	NDMP_LOG_ERROR,
	NDMP_LOG_WARNING
};

struct ndmp_log_msg_req
{
	ndmp_log_type	log_type;	/* desc. above */
	u_long			message_id; /* server dependent */
	string			entry<>;	/* txt msg */
};
/* No response */

/* --NDMP_LOG_FILE-- */
struct ndmp_log_file_req		/* used during recovery to notify client a file from recovery list
								 * sent in DATA_START_RECOVER req has/has not been recovered.
								 * this msg should be sent for files having a name in the recovery
								 * list.
								 */
{
	string		name<>;			/* file names */
	ndmp_Err	error;
};
/* No response */


/********** FILE HISTORY Interface messages structures *********/

/* --NDMP_FH_ADD_FILE-- */
enum ndmp_fs_type
{
	NDMP_FS_UNIX,	/* UNIX */
	NDMP_FS_NT,		/* NT */
	NDMP_FS_OTHER
};

typedef string ndmp_path<>;
struct ndmp_nt_path
{
	ndmp_path	nt_path;
	ndmp_path	dos_path;
};
 
union ndmp_file_name switch (ndmp_fs_type fs_type)
{
	case NDMP_FS_UNIX:
		ndmp_path		unix_name;
	case NDMP_FS_NT:
		ndmp_nt_path	nt_name;
	default:
		ndmp_path		other_name;
};

enum ndmp_file_type
{
	NDMP_FILE_DIR,
	NDMP_FILE_FIFO,
	NDMP_FILE_CSPEC,
	NDMP_FILE_BSPEC,
	NDMP_FILE_REG,
	NDMP_FILE_SLINK,
	NDMP_FILE_SOCK,
	NDMP_FILE_REGISTRY,			/* not in v2 */
	NDMP_FILE_OTHER				/* not in v2 */
};

/* invalid bit */
const NDMP_FILE_STAT_ATIME_INVALID	= 0x00000001;
const NDMP_FILE_STAT_CTIME_INVALID	= 0x00000002;
const NDMP_FILE_STAT_GROUP_INVALID	= 0x00000004;

struct ndmp_file_stat
{
	u_long			invalid;
	ndmp_fs_type	fs_type;
	ndmp_file_type	ftype;
	u_long			mtime;
	u_long			atime;
	u_long			ctime;
	u_long			owner; /* uid for UNIX, owner for NT */
	u_long			group; /* gid for UNIX, NA for NT */
	u_long			fattr; /* mode for UNIX, fattr for NT */
	ndmp_u_quad		size;
	u_long			links;
};


/* one file could have both UNIX and NT name and attributes */
struct ndmp_file
{
	ndmp_file_name	names<>;
	ndmp_file_stat	stats<>;
	ndmp_u_quad		node;		/* used for the direct access */
	ndmp_u_quad		fh_info;	/* used for the direct access */
};

/* This is for tar/cpio, not supported in 1st release */
struct ndmp_fh_add_file_req
{
	ndmp_file		files<>;
};

/* No response */

/* --NDMP_FH_ADD_DIR-- */

struct ndmp_dir
{
	ndmp_file_name	names<>;	/* base name relative to node's parent dir */
	ndmp_u_quad		node;		/* node id matches a node in corresponding add node msg */
	ndmp_u_quad		parent;		/* node id of node's parent dir. */
};
 
struct ndmp_fh_add_dir_req
{
	ndmp_dir	dirs<>;			/* list of dir entries */
};
/* No response */
 
/* --NDMP_FH_ADD_NODE-- */
struct ndmp_node
{
	ndmp_file_stat	stats<>;	/* file attrib data for single file */
	ndmp_u_quad		node;		/* node id matches a node in corresponding add dir msg */
	ndmp_u_quad		fh_info;	/* file hist tape positioning data representing tape 
								 * position at time file was written to tape.  Data may 
								 * be used by restore to perform tape positioning for DAR.  
								 * Ndmp server dependent, usually is byte ore rec offset 
								 * from BOT of file to be recovered. */
};
 
struct ndmp_fh_add_node_req
{
	ndmp_node	nodes<>;		/* list of file history node entries */
};
/* No response */
/**************************** SCSI service messages structures *****************************/

/* --NDMP_SCSI_OPEN-- */
struct ndmp_scsi_open_req
{
	string	device<>;		/* name of SCSI device to open, WWN */
};

struct ndmp_scsi_open_rsp
{
	ndmp_Err	error;
};

/* --NDMP_SCSI_CLOSE-- */
/* Close currently open SCSI i/f device.  No further requests can be made til another 
 * open request is executed.
 */
/* no arguments for request */

struct ndmp_scsi_close_rsp
{
	ndmp_Err	error;
};

/* --NDMP_SCSI_GET_STATE-- */
/* no arguments for request */

struct ndmp_scsi_get_state_rsp
{
	ndmp_Err	error;
	short		targ_crtller;	/* Id of SCSI controller, could be CPU, Slot # in our context */
	short		targ_id;		/* SCSI target id, could be device_handle in our context */
	short		targ_lun;		/* LUN number, same in our context, part of WWN */
};

/* --NDMP_SCSI_RESET_DEVICE-- */
/* no arguments for request */

struct ndmp_scsi_reset_dev_rsp	/* a H/W reset */
{
	ndmp_Err	error;
};

/* --NDMP_SCSI_RESET_BUS-- */
/* no arguments for request */

struct ndmp_scsi_reset_bus_rsp  /* a H/W reset */
{
    ndmp_Err    error;
};

/* --NDMP_SCSI_SET_TARGET-- */
struct ndmp_scsi_set_target_req
{
	string		device<>;
	u_short		target_controller;
	u_short		target_id;
	u_short		target_lun;
};

struct ndmp_scsi_set_target_rsp
{
    ndmp_Err    error;
};

/* --NDMP_SCSI_EXECUTE_CDB-- */
const NDMP_SCSI_DATA_IN		= 0x00000001;	/* Expect data from SCSI device */
const NDMP_SCSI_DATA_OUT	= 0x00000002;	/* Transfer data to SCSI device */

struct ndmp_exec_cdb_req
{
	u_long	flags;			/* data xfer dir DATA_IN/DATA_OUT reference to srvr */
	u_long	timeout;		/* No. millisecs to wait for cmd completion to occur */
	u_long	datain_len;		/* No. data bytes to read */
	opaque	cdb<>;			/* list of CDBs */
	opaque	dataout<>;		/* Data to send to SCSI device */
};


struct ndmp_exec_cdb_rsp
{
	ndmp_Err	error;
	u_char		status;			/* SCSI status bytes */
	u_long		dataout_len;	/* No. data bytes to xfer to SCSI device */
	opaque		datain<>;		/* SCSI data to xfer from SCSI device */
	opaque		ext_sense<>;	/* Extended sense data, explain why cmd was not completed */
};

/* --NDMP_SCSI_EXECUTE_CDB-- */
typedef ndmp_exec_cdb_req		ndmp_scsi_exec_cdb_req;
typedef ndmp_exec_cdb_rsp		ndmp_scsi_exec_cdb_rsp;

/************************** TAPE Service messages structures ****************************/

/* --NDMP_TAPE_OPEN-- */
enum ndmp_tape_open_mode
{
	NDMP_TAPE_READ_MODE,
	NDMP_TAPE_RDWR_MODE
};

struct ndmp_tape_open_req
{
	string	devname<>;			/* tape device to open */
	ndmp_tape_open_mode	mode;
};

struct ndmp_tape_open_rsp
{
	ndmp_Err	error;
};

/* NDMP_TAPE_CLOSE */
/* no arguments for request */

struct ndmp_tape_close_rsp
{
	ndmp_Err	error;
};

/*NDMP_TAPE_GET_STATE */
/* no arguments for request */

/* flags */
const NDMP_TAPE_STATE_NOREWIND	= 0x0008;	/* if close, tape will not be rewind */
const NDMP_TAPE_STATE_WR_PROT	= 0x0010;	/* write-protected */
const NDMP_TAPE_STATE_ERROR		= 0x0020;	/* media error detected w/ previous tape ops */
const NDMP_TAPE_STATE_UNLOAD	= 0x0040;	/* tape will be unloaded when
											 * the device is closed */

/* invalid bit */
const NDMP_TAPE_STATE_FILE_NUM_INVALID		= 0x00000001;
const NDMP_TAPE_STATE_SOFT_ERRORS_INVALID	= 0x00000002;
const NDMP_TAPE_STATE_BLOCK_SIZE_INVALID	= 0x00000004;
const NDMP_TAPE_STATE_BLOCKNO_INVALID		= 0x00000008;
const NDMP_TAPE_STATE_TOTAL_SPACE_INVALID	= 0x00000010;
const NDMP_TAPE_STATE_SPACE_REMAIN_INVALID	= 0x00000020;
const NDMP_TAPE_STATE_PARTITION_INVALID		= 0x00000040;


struct ndmp_tape_get_state_rsp
{
	u_long		invalid;	/* used to id unsupported args in msg */
	ndmp_Err	error;		
	u_long		flags;		/* flags specified above */
	u_long		file_num;	/* curr file position, first file on tape is no. 0 */
	u_long		soft_errors;/* no. of soft media errs detected since device opened */
	u_long		block_size;	/* tape blk size in bytes, 0 if dev is in variable blk siz mode. */
	u_long		blockno;	/* curr tape blk no.  1st tape blk is 0 */
	ndmp_u_quad	total_space;/* tape capacity in bytes */
	ndmp_u_quad	space_remain;/* remaining tape capacity in bytes */
	u_long		partition;	/* tape partition no. 1st partition on tape is 0 */
};

/* --NDMP_TAPE_MTIO-- */
enum ndmp_tape_mtio_op
{
	NDMP_MTIO_FSF,		/* Fwd space over file marks */ 
	NDMP_MTIO_BSF,		/* Bwd space over file marks */ 
	NDMP_MTIO_FSR,		/* Fwd space over tape record */	
	NDMP_MTIO_BSR,		/* Bwd space over tape record */
	NDMP_MTIO_REW,		/* Rewind tape */
	NDMP_MTIO_EOF,		/* Write end of file marks */
	NDMP_MTIO_OFF		/* Eject tape from device */
};

struct ndmp_tape_mtio_req
{
	ndmp_tape_mtio_op	tape_op;	/* specified above */
	u_long				count;		/* how many ops to do */
};

struct ndmp_tape_mtio_rsp
{
	ndmp_Err	error;
	u_long		resid_count;	/* no. of ops couldn't do due to BOT, EOT, 
								 * end of written media or tape err */
};

/* --NDMP_TAPE_WRITE-- */
struct ndmp_tape_write_req
{
	opaque	data_out<>;			/* Data to write to tape device */
};

struct ndmp_tape_write_rsp
{
	ndmp_Err	error;
	u_long		count;			/* Bytes written */
};

/* --NDMP_TAPE_READ-- */
struct ndmp_tape_read_req
{
	u_long	count;				/* Bytes to read */
};

struct ndmp_tape_read_rsp
{
	ndmp_Err	error;
	opaque		data_in<>;		/* Data read from tape */
};


/* --NDMP_TAPE_EXECUTE_CDB-- */
typedef ndmp_exec_cdb_req		ndmp_tape_exec_cdb_req;
typedef ndmp_exec_cdb_rsp		ndmp_tape_exec_cdb_rsp;

/************************** DATA Service messages structures ************************/

/* --NDMP_DATA_GET_STATE-- */
/* no arguments for request */
enum ndmp_data_ops
{
	NDMP_DATA_OP_NOACTION,		/* No data ops in progress */
	NDMP_DATA_OP_BACKUP,		/* bkup in progress */
	NDMP_DATA_OP_RESTORE		/* restore in progress */
};

enum ndmp_data_state
{
	NDMP_DATA_ST_IDLE,			/* no active data op */
	NDMP_DATA_ST_ACTIVE,		/* data op in progress */
	NDMP_DATA_ST_HALTED,		/* data op completed */
	NDMP_DATA_ST_LISTEN,		/* wait for data conn to be established */
	NDMP_DATA_ST_CONNECTED		/* data conn connected */
};

/* invalid bit */
const NDMP_DATA_STATE_EST_BYTES_REMAIN_INVALID	= 0x00000001;
const NDMP_DATA_STATE_EST_TIME_REMAIN_INVALID	= 0x00000002;


struct ndmp_tcp_addr
{
	u_long	ip_addr;
	u_short	port;
};

union ndmp_addr switch (ndmp_AddrTyp addr_type)
{
	case NDMP_ADDR_LOCAL:
		void;
	case NDMP_ADDR_TCP:
		ndmp_tcp_addr	tcp_addr;
};
struct ndmp_data_get_state_rsp
{
	u_long					invalid;	/* specify unsupported args, listed above */
	ndmp_Err				error;
	ndmp_data_ops			op;			/* data op in progress */
	ndmp_data_state			state;		/* current state of server */
	ndmp_data_halt_reason	halt_reason;	/* reason data op is halted */
	ndmp_u_quad				bytes_processed;	
	ndmp_u_quad				est_bytes_remain;	/* set flag above if not supported */ 
	u_long					est_time_remain;	/* set flag above if not supported */ 
	ndmp_addr				data_conn_addr;		/* As a listener:
												 * When in LISTEN, is addr offer to remote server 
												 * when in CONNECTED or ACTIVE, is addr of clien
												 * when in IDLE, field is ignored. 
												 * As a connector:
												 * When in ACTIVE state, is addr of remote server 
												 * when in IDLE, field is ignored. 
												 */
	ndmp_u_quad				read_offset;	/* offset val specified in last NOTIFY_DATA_READ req */	
	ndmp_u_quad				read_length;	/* len val specified in last NOTIFY_DATA_READ req */
};

/* --NDMP_DATA_LISTEN-- */
struct ndmp_data_listen_req
{
	ndmp_AddrTyp	addr_type;		/* Local, TCP, etc... */
};

struct ndmp_data_listen_rsp
{
	ndmp_Err	error;
	ndmp_addr	data_conn_addr;		/* The addr I'm advertising to remote server */
};

/* --NDMP_DATA_CONNECT-- */
struct ndmp_data_conn_req
{
	ndmp_addr	addr;				/* Addr i am to connect to */
};
struct ndmp_data_conn_rsp
{
	ndmp_Err	error;
};

/* --NDMP_DATA_START_BACKUP-- */
struct ndmp_data_start_bkup_req
{
	string		bu_type<>;				/* backup method to use */
	ndmp_pval	env<>;					/* list of params names & vals for configuring bkup type */
};

struct ndmp_data_start_bkup_rsp
{
	ndmp_Err	error;
};

/* --NDMP_DATA_START_RECOVER-- */
struct ndmp_name
{
	string		orig_path<>;			/* path of original file/dir to be recovered, 
										 * relative to bkup root dir.  If not specified, 
										 * entire bkup image will be restored 
										 */
	string		dest_dir<>;				/* if not specified, files'll be restored to the orig dir. */
	string		new_name<>;				/* For direct access restore only, restored file is 
										 * renamed to this name.
										 */
	string		other_name<>;			/* For direct access restore only, when multiple file 
										 * names for a single file is supported. 
										 */
	ndmp_u_quad	node;					/* For direct access restore only.  Used to verify 
										 * retrieval if bkup method support inode type of bkup, 
										 * e.g. dump.  Set to 0 if not supported. 
										 */
	ndmp_u_quad	fh_info;				/* For direct access restore only. Is file history 
										 * tape positioning data recorded when file is bkup.  
										 * May be used by restore method to position tape for DAR.  
										 * Usually is byte or rec offset from BOT of file to be 
										 * recovered. 
										 */
};

struct ndmp_data_start_recover_req
{
	ndmp_pval	env<>;					/* Bkup env. that was returned from a data get env. req made
										 * prior to notifying NDMP sever that bkup was complete thru a
									     * data stop msg.
										 */
	ndmp_name	nlist<>;				/* list of files to be recovered & locaiton to which each file
										 * is to be recovered. 
										 */
	string		bu_type<>;				/* Name of recover method. */
};

struct ndmp_data_start_recover_rsp
{
	ndmp_Err	error;
};

/* --NDMP_DATA_ABORT-- */
/* no arguments for request */

struct ndmp_data_abort_rsp
{
	ndmp_Err	error;
};

/* --NDMP_DATA_STOP-- */
/* no arguments for request */

struct ndmp_data_stop_rsp
{
	ndmp_Err	error;
};

/* --NDMP_DATA_GET_ENV-- */
/* no arguments for request */

struct ndmp_data_get_env_rsp
{
	ndmp_Err	error;
	ndmp_pval	env<>;				/* Bkup env params & values */
};



/********************** MOVER service messages structures ***************************/

/* --NDMP_MOVER_GET_STATE-- */
enum ndmp_mover_state
{
	NDMP_MOVER_ST_IDLE,
	NDMP_MOVER_ST_LISTEN,
	NDMP_MOVER_ST_ACTIVE,
	NDMP_MOVER_ST_PAUSED,
	NDMP_MOVER_ST_HALTED
};

/* mover address */
enum ndmp_mover_mode
{
	NDMP_MOVER_MODE_READ,	/* read from data connection; write to tape */
	NDMP_MOVER_MODE_WRITE	/* write to data connection; read from tape */
};

/* no arguments for request */

struct ndmp_mover_get_state_rsp
{
	ndmp_Err				error;
	ndmp_mover_state		state;
	ndmp_mover_pause_reason	pause_reason;
	ndmp_mover_halt_reason	halt_reason;
	u_long					recsiz;				/* size of tape data record */
	u_long					recnum;				/* curr tape rec # */
	ndmp_u_quad				data_written; 		/* total no. of bytes written to tape */
	ndmp_u_quad				seek_pos;			/* offset val from last NDMP_MOVER_READ msg */ 
	ndmp_u_quad				bytes_left_to_read;	/* no. of bytes remaining to be read to 
												 * satisfy last NDMP_MOVER_READ msg */ 
	ndmp_u_quad				window_offset;		/* offset val from last NDMP_MOVER_SET_WINDOW req */	
	ndmp_u_quad				window_len;			/* len val from last NDMP_MOVER_SET_WINDOW req */
	ndmp_addr				data_conn_addr;		/* When tape server listens a data conn:
												 *  In LISTEN state, is the addr to listen the data connn.
												 *  While state xsition to CONNECTED or ACTIVE, is addr
												 *  of client.  In IDLE state, field's ignored.
												 * When tape server connects to other NDMP server:
												 *  In ACTIVE state, is addr of remote server listening
												 *  to data conn.  In IDLE, field's ignored.
												 */
};

/* --NDMP_MOVER_LISTEN-- */
struct ndmp_mover_listen_req
{
	ndmp_mover_mode		mode;				/* READ from data conn & write to tape (in bkup) or 
											 * read data from tape and WRITE to data conn (in restored) */
	ndmp_AddrTyp	addr_type;				/* Local or TCP */
};

struct ndmp_mover_listen_rsp
{
	ndmp_Err	error;
	ndmp_addr	data_conn_addr;
};

/* --NDMP_MOVER_CONNECT-- */
struct ndmp_mover_conn_req
{
	ndmp_mover_mode		mode;				/* Dir of data flow */
	ndmp_addr		addr;					/* addr where remote server is listening for data conn */
};

struct ndmp_mover_conn_rsp
{
	ndmp_Err	error;
};
/* --NDMP_MOVER_SET_RECORD_SIZE-- */
struct ndmp_mover_set_record_size_req
{
	u_long	len;							/* rec size in bytes */
};

struct ndmp_mover_set_record_size_rsp
{
	ndmp_Err	error;
};

/* --NDMP_MOVER_SET_WINDOW-- */
struct ndmp_mover_set_window_req
{
	ndmp_u_quad	offset;						/* data stream byte offset of 1st byte in window.  Field
											 * is ignored if tape server is writing data to bkup dev.
											 */
	ndmp_u_quad	length;						/* no. of bytes in window during DAR.  Max no. of bytes of data
											 * could be written to each tape file if tape server is
											 * writing data to bkup device. 
											 */
};

struct ndmp_mover_set_window_rsp
{
	ndmp_Err	error;
};

/* --NDMP_MOVER_CONTINUE-- */
/* no arguments for request */

struct ndmp_mover_cont_rsp
{
	ndmp_Err	error;
};


/* --NDMP_MOVER_ABORT-- */
/* no arguments for request */

struct ndmp_mover_abort_rsp
{
	ndmp_Err	error;
};

/* --NDMP_MOVER_STOP-- */
/* no arguments for request */

struct ndmp_mover_stop_rsp
{
	ndmp_Err	error;
};

/* --NDMP_MOVER_READ-- */
struct ndmp_mover_read_req
{
	ndmp_u_quad	offset;			/* offset w/in data stream of 1st byte to be sent 
								 * to data conn.  Tape server should seek tape to 
								 * the rec containing the requested offset and then 
								 * read and discard data til the offeset has been 
								 * reached.   If offset is outside curr set data 
								 * window, tape server should notify client that a 
								 * seek is needed. */
	ndmp_u_quad	length;			/* bytes to read & send to data conn. */
};

struct ndmp_mover_read_rsp
{
	ndmp_Err	error;
};

/* --NDMP_MOVER_CLOSE-- */
/* no arguments for request */

struct ndmp_mover_close_rsp
{
	ndmp_Err	error;
};

/* ======================================================================== 
/* 			V2 specifics
 * ========================================================================
 */

/********* CONFIGURATION interface messages structures ********/

/* --NDMP_CONFIG_GET_HOST_INFO-- */
/* no arguments for request */

struct ndmpv2_cfg_get_host_info_rsp
{
	ndmp_Err		error;
	string			hostname<>;	/* host name of NDMP server */
	string			os_type<>;	/* Name of NDMP server operating system (ex: SOLARIS) */
	string			os_vers<>;	/* Version of NDMP server operation system (ex: 2.5) */
	string			hostid<>;	/* NDMP server host identifier */
	ndmp_AuthTyp	authTyp<>;	  /* Connection authentication types supported by server */
};

/* --NDMP_CONFIG_GET_BUTYPE_ATTR-- */

/* Bkup type attributes - 
 * these bit masks indicates the bkup type capabilities 
 */
const NDMP_BUTYPE_NO_BACKUP_FILELIST	= 0x0001;	/* archiving of selective files as specified */
													/* by file list is supported */
const NDMP_BUTYPE_NO_BACKUP_FHINFO		= 0x0002;	/* no file history is supported */
const NDMP_BUTYPE_NO_RECOVER_FILELIST	= 0x0004;	/* no restoration of individual files supported */
const NDMP_BUTYPE_NO_RECOVER_FHINFO		= 0x0008;	/* no direct access restore is supported (positioning to the
                                                     * offset of bkup image and restoring specified file) */
const NDMP_BUTYPE_NO_RECOVER_RESVD		= 0x0010;	/* reserved */
const NDMP_BUTYPE_NO_RECOVER_INC_ONLY	= 0x0020;	/* no incremental-only restore (a full restore must be done 
													 * before an incremental restore).
													 */
 
struct ndmpv2_cfg_get_butype_attr_req 
{
	string		butype_name<>;		/* Name of backup (dump, tar, cpio) */
};

struct ndmpv2_cfg_get_butype_attr_rsp 
{
	ndmp_Err	error;
	u_long		attrs;				/* Bkup attributes bit mask, described above */
};

/********* LOG Interface messages structures *********/

/* --NDMP_LOG_LOG -- */

struct ndmpv2_log_log_req
{
	string			entry<>;	/* txt msg */
};
/* No response */

enum ndmpv2_dbg_level
{
	NDMP_DBG_USER_INFO,
	NDMP_DBG_USER_SUMMARY,
	NDMP_DBG_USER_DETAIL,
	NDMP_DBG_DIAG_INFO,
	NDMP_DBG_DIAG_SUMMARY,
	NDMP_DBG_DIAG_DETAIL,
	NDMP_DBG_PROG_INFO,
	NDMP_DBG_PROG_SUMMARY,
	NDMP_DBG_PROG_DETAIL
};

/* --NDMP_LOG_DBG -- */

struct ndmpv2_log_dbg_req
{
	ndmpv2_dbg_level 	level;
	string			msg<>;	/* txt msg */
};

/* --NDMP_LOG_FILE-- */

struct ndmpv2_log_file_req		/* used during recovery to notify client 
								 * a file from recovery list sent in 
								 * DATA_START_RECOVER req has/has not been 
								 * recovered.  This msg should be sent for 
								 * files having a name in the recovery
								 * list.
								 */
{
	string		name<>;			/* file names */
	u_short		reserved;
	ndmp_Err	error;
};
/* No response */


/********** FILE HISTORY Interface messages structures *********/

/* --NDMP_FH_ADD_FILE-- */

struct ndmpv2_file_stat
{
	ndmp_file_type	ftype;
	u_long			mtime;
	u_long			atime;
	u_long			ctime;
	u_long			owner; /* uid for UNIX, owner for NT */
	u_long			group; /* gid for UNIX, NA for NT */
	u_long			fattr; /* mode for UNIX, fattr for NT */
	ndmp_u_quad		size;
	ndmp_u_quad		fh_info; /* used for direct access */
};

typedef string ndmpv2_unix_path<>;

/* --NDMP_FH_ADD_DIR-- */

/* one file could have both UNIX and NT name and attributes */
struct ndmpv2_file
{
	ndmpv2_unix_path	name;
	ndmpv2_file_stat	stat;
};

/* This is for tar/cpio, but we use for dump */
struct ndmpv2_fh_add_file_req
{
	ndmpv2_file	files<>;
};

/* --NDMP_FH_ADD_DIR-- */

struct ndmpv2_dir
{
	ndmpv2_unix_path name;		/* base name relative to node's parent dir */
	u_long			node;		/* node id matches a node in corresponding add node msg */
	u_long			parent;		/* node id of node's parent dir. */
};
 
struct ndmpv2_fh_add_dir_req
{
	ndmpv2_dir	dirs<>;			/* list of dir entries */
};
/* No response */
 
/* --NDMP_FH_ADD_NODE-- */
struct ndmpv2_node
{
	ndmpv2_file_stat	stat;		/* file attrib data for single file */
	u_long			node;		/* node id matches a node in corresponding add dir msg */
};
 
struct ndmpv2_fh_add_node_req
{
	ndmpv2_node		nodes<>;		/* list of file history node entries */
};
/* No response */

/************************** TAPE Service messages structures ****************************/

/*NDMP_TAPE_GET_STATE */
/* no arguments for request */

struct ndmpv2_tape_get_state_rsp
{
	ndmp_Err	error;		
	u_long		flags;		/* flags specified above */
	u_long		file_num;	/* curr file position, first file on tape is no. 0 */
	u_long		soft_errors;/* no. of soft media errs detected since device opened */
	u_long		block_size;	/* tape blk size in bytes, 0 if dev is in variable blk siz mode. */
	u_long		blockno;	/* curr tape blk no.  1st tape blk is 0 */
	ndmp_u_quad	total_space;/* tape capacity in bytes */
	ndmp_u_quad	space_remain;/* remaining tape capacity in bytes */
};

/************************** DATA Service messages structures ************************/

/* --NDMP_DATA_GET_STATE-- */
/* no arguments for request */

struct ndmpv2_data_get_state_rsp
{
	ndmp_Err				error;
	ndmp_data_ops			op;			/* data op in progress */
	ndmp_data_state			state;		/* current state of server */
	ndmp_data_halt_reason	halt_reason;	/* reason data op is halted */
	ndmp_u_quad				bytes_processed;	
	ndmp_u_quad				est_bytes_remain;	/* set flag above if not supported */ 
	u_long					est_time_remain;	/* set flag above if not supported */ 
	ndmp_addr				data_conn_addr;		/* As a listener:
												 * When in LISTEN, is addr offer to remote server 
												 * when in CONNECTED or ACTIVE, is addr of clien
												 * when in IDLE, field is ignored. 
												 * As a connector:
												 * When in ACTIVE state, is addr of remote server 
												 * when in IDLE, field is ignored. 
												 */
	ndmp_u_quad				read_offset;	/* offset val specified in last NOTIFY_DATA_READ req */	
	ndmp_u_quad				read_length;	/* len val specified in last NOTIFY_DATA_READ req */
};

/* --NDMP_DATA_START_BACKUP-- */
struct ndmpv2_data_start_bkup_req
{
	ndmp_addr	addr;					/* mover to receive data */
	string		bu_type<>;				/* backup method to use */
	ndmp_pval	env<>;					/* list of params names & vals for configuring bkup type */
};

/* --NDMP_DATA_START_RECOVER-- */
struct ndmpv2_name
{
	string		orig_path<>;			/* path of original file/dir to be recovered, 
										 * relative to bkup root dir.  If not specified, 
										 * entire bkup image will be restored 
										 */
	string		dest_dir<>;				/* if not specified, files'll be restored to the orig dir. */
	u_short		rsvd;					/* reserved*/
	ndmp_u_quad	fh_info;				/* For direct access restore only. Is file history 
										 * tape positioning data recorded when file is bkup.  
										 * May be used by restore method to position tape for DAR.  
										 * Usually is byte or rec offset from BOT of file to be 
										 * recovered. 
										 */
};

struct ndmpv2_data_start_recover_req
{
	ndmp_addr	addr;					/* mover from which to receive data */
	ndmp_pval	env<>;					/* Bkup env. that was returned from a data get env. req made
										 * prior to notifying NDMP sever that bkup was complete thru a
									     * data stop msg.
										 */
	ndmpv2_name	nlist<>;				/* list of files to be recovered & locaiton to which each file
										 * is to be recovered. 
										 */
	string		bu_type<>;				/* Name of recover method. */
};


/********************** MOVER service messages structures ***************************/

/* --NDMP_MOVER_GET_STATE-- */
/* no arguments for request */

struct ndmpv2_mover_get_state_rsp
{
	ndmp_Err				error;
	ndmp_mover_state		state;
	ndmp_mover_pause_reason	pause_reason;
	ndmp_mover_halt_reason	halt_reason;
	u_long					recsiz;				/* size of tape data record */
	u_long					recnum;				/* curr tape rec # */
	ndmp_u_quad				data_written; 		/* total no. of bytes written to tape */
	ndmp_u_quad				seek_pos;			/* offset val from last NDMP_MOVER_READ msg */ 
	ndmp_u_quad				bytes_left_to_read;	/* no. of bytes remaining to be read to 
												 * satisfy last NDMP_MOVER_READ msg */ 
	ndmp_u_quad				window_offset;		/* offset val from last NDMP_MOVER_SET_WINDOW req */	
	ndmp_u_quad				window_len;			/* len val from last NDMP_MOVER_SET_WINDOW req */
};

