/* vi:ts=4
 *-----------------------------------------------------------------
 *
 * Name:        nscd.c
 *
 * RCS:  
 *
 * Copyright (C) 2002, Agile Storage Inc.
 *
 * Description:		NDMP snap shot clean deamon.
 *
 * Created by: Nitin Jain
 *
 * Date Created: 10/10/02
 *
 *-----------------------------------------------------------------
 */
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <nfx-types.h>
#include <exports/eee-api.h>
#include <exports/elog-api.h>
#include <exports/anpssc-api.h>
#include "ndmp.h"

int32 nscAppId;
int32 nscDesc;
struct timeval waitEfsTo = {60, 0};

void nsc_removeSnap(evm_volId_t volId, char *snapName);
void nsc_cleanSnapShots(fs_MsgSnapShowResp *fsSnapRsp, char *snapList,
                        int32 snapCnt);
int32 nsc_getNdmpSnapsot(evm_volId_t volId, char *snapList);
fs_MsgSnapShowResp *nsc_getSnapVolSnapshots(evm_volId_t volId);

/*
 *-----------------------------------------------------------------
 * Name: nsc_removeSnap
 *
 * Description: Send message to FS to remove the snapshot.
 *
 * Created by:  Nitin Jain
 *
 * Date Created: 10/10/02
 *-----------------------------------------------------------------
 */

void nsc_removeSnap(evm_volId_t volId, char *snapName)
{
    fs_MsgSnapRemoveReq *req;
    int32  len;

    len = sizeof(fs_MsgSnapRemoveReq) + strlen(snapName)+1;

    req = (fs_MsgSnapRemoveReq *)malloc(len);
    if (!req)
        return;
    req->hdr.requestType = FS_REQ_SNAP_REMOVE;
    req->hdr.len = len;
    req->volId.whole_thing = volId.whole_thing;
    req->nameLen = strlen(snapName);
    strcpy(req->snapDirName, snapName);

    sendAgileMsg(-1, req, len, "efs", NFX_NFP_FP2, 1, MGMTPLANE, NULL, NULL, 0);

    free(req);
    // printf("Remove snap : %lld %s\n", volId.whole_thing, snapName);
    return;
}

/*
 *-----------------------------------------------------------------
 * Name: ndmp_cleanSnapShot
 *
 * Description: Get the list of mounted volumes from the file system
 *
 * Created by:  Nitin Jain
 *
 * Date Created: 10/10/02
 *-----------------------------------------------------------------
 */

void nsc_cleanSnapShots(fs_MsgSnapShowResp *fsSnapRsp, char *snapList,
                        int32 snapCnt)
{
    int32 i, j, found;

    // printf ("FS snapshots...\n");
    for (i = 0; i < fsSnapRsp->nUsers; i++)
    {
        // printf ("snapshot[%d] = %s ", i, fsSnapRsp->user[i].name);
        found = 0;
        if (strncasecmp("dump_snap", fsSnapRsp->user[i].name, 8))
        {
            // printf ("\n");
            continue;
        }
        for (j = 0; j < snapCnt; j++)
        {
            if (!strcasecmp(fsSnapRsp->user[i].name,
                            &snapList[j*NDMP_MAX_SNAPSHOT_NAME_LEN]))
            {
                found = 1;
                // printf("found");
            }
        }
        if (!found)
             nsc_removeSnap(fsSnapRsp->volId, fsSnapRsp->user[i].name);
        // printf("\n");
    }

    return;
}

/*
 *-----------------------------------------------------------------
 * Name: nsc_getNdmpSnapsot
 *
 * Description: Read ndmp dump date file to get list of snapshots
 *
 * Created by:  Nitin Jain
 *
 * Date Created: 10/10/02
 *-----------------------------------------------------------------
 */

int32 nsc_getNdmpSnapsot(evm_volId_t volId, char *snapList)
{
    int32 cnt = 0;
    FILE *fp;
    char fileName[1024], linkName[1024];
    char blob[256], *name = NULL;

    sprintf(linkName, "%s%lld.dmp", NFX_DUMPDATE_DIR, volId.whole_thing);
    // printf( "File name %s\n", linkName);

    if (readlink(linkName, fileName, 1024) == -1)
        return cnt;

    fp = fopen(fileName, "r");
    if (fp == NULL)
        return cnt;

    // printf ("NDMP snapshots...\n");
    bzero(blob, sizeof(blob));
    while (fgets(blob, sizeof(char)*256, fp) != NULL)
    {
        name = strrchr(blob, ' ');
        name++;
        name[strlen(name) - 1] = '\0';
        strcpy(&snapList[cnt*NDMP_MAX_SNAPSHOT_NAME_LEN], name);
        cnt++;
        // printf ("snapshot[%d] = %s\n", cnt, name);
    }
    fclose(fp);

    return cnt;
}

/*
 *-----------------------------------------------------------------
 * Name: nsc_getSnapVolSnapshots
 *
 * Description: Ask file system for list of snapshots for the volume.
 *
 * Created by:  Nitin Jain
 *
 * Date Created: 10/10/02
 *-----------------------------------------------------------------
 */

fs_MsgSnapShowResp *nsc_getSnapVolSnapshots(evm_volId_t volId)
{
    fs_MsgSnapShowReq req;
    char *msg;
    int32 rc;

    /*
     * Get list of snapshots
     */
    msg = (char *)malloc(sizeof(char)*8192);
    if (!msg)
        return NULL;
    bzero(msg, sizeof(char)*8192);
    bzero(&req, sizeof(req));
    req.hdr.requestType = FS_REQ_SNAP_SHOW;
    req.hdr.len = sizeof(req);
    req.type = FS_SNAP_SHOW_USAGE;
    req.volId = volId;
    rc = sendAgileMsg(-1, &req, sizeof(req), "efs",
                      NFX_NFP_FP2, 1, MGMTPLANE, &waitEfsTo, msg,
                      sizeof(char)*8192);

    if (rc < sizeof(req))
        return NULL;

    return ((fs_MsgSnapShowResp *)msg);
}

/*
 *-----------------------------------------------------------------
 * Name: main
 *
 * Description: Entry point for the daemon.
 *
 * Created by:  Nitin Jain
 *
 * Date Created: 10/10/02
 *-----------------------------------------------------------------
 */

int32 main(int32 argc, char **argv)
{
    int32 rc, snapCnt = 0, i, len;
    int32 numFS = 0, totalFS = 0;
    char  msg[8192];
    fs_MsgViewFsReq fsViewMsg;
    fs_MsgViewFsResp *fsViewRsp;
    fs_MsgSnapShowResp *fsSnapRsp = NULL;
    char *snapList;

    bzero(&fsViewMsg, sizeof(fs_MsgViewFsReq));

    fsViewMsg.hdr.requestType = FS_REQ_VIEWFS;
    fsViewMsg.hdr.len = sizeof(fs_MsgViewFsReq);
    fsViewMsg.type = FS_VIEW_LIST_OF_VOLUMES;
    fsViewMsg.clientprivate = FALSE;

    len = sizeof(char)*NDMP_MAX_SNAPSHOT_NAME_LEN*10;
    snapList = (char *)malloc(len);
    if (!snapList)
        return 1;
    // while (numFS < totalFS)
    {
        fsViewMsg.volId.whole_thing = 0;     // not used

        bzero(msg, sizeof(char)*8192);
        rc = sendAgileMsg(-1, &fsViewMsg, sizeof(fsViewMsg), "efs", NFX_NFP_FP2,
                          1, MGMTPLANE, &waitEfsTo, msg, sizeof(char)*8192);

        if (rc < sizeof(fs_MsgViewFsResp))
            return 1;

        fsViewRsp = (fs_MsgViewFsResp *)msg;
        // printf("Number of FS : %d\n", fsViewRsp->nbrFs);
        numFS += fsViewRsp->nbrFs;
        totalFS = fsViewRsp->totalNbrFs;
        for (i = 0; i < fsViewRsp->nbrFs; i++)
        {
            // printf("FS[%d] name: %lld\n", i, fsViewRsp->u.volId[i].whole_thing);
            fsSnapRsp = nsc_getSnapVolSnapshots(fsViewRsp->u.volId[i]);
            if (fsSnapRsp && fsSnapRsp->nUsers)
            {
                // printf ("Got snapshots from FS\n");
                bzero(snapList, len);
                snapCnt = nsc_getNdmpSnapsot(fsViewRsp->u.volId[i],
                                                      &snapList[0]);
                // printf ("Ndmp snapshots %lld : %d\n", fsViewRsp->u.volId[i].whole_thing, snapCnt);
                nsc_cleanSnapShots(fsSnapRsp, &snapList[0], snapCnt);
            }
            if (fsSnapRsp)
                free(fsSnapRsp);
            fsSnapRsp = NULL;
        }
    }
    free(snapList);

    return 1;
}
