/* vi:ts=4
 *-----------------------------------------------------------------
 *
 * Copyright (C) 2001, Agile Storage Inc.
 *
 * RCS:  $Id: nb-api.h,v 1.1.1.1 2002/04/23 17:52:14 laurentb Exp $
 *  
 * Name			: nb-api.h
 *
 * Description	: NetBIOS module inteface related definitions.
 *
 * Created by	: Xiaobing Zhang
 *
 * Date Created	: 11/07/2001
 *
 *-----------------------------------------------------------------
 */
#ifndef _NB_API_H
#define _NB_API_H

#define NBTEST

#ifdef LINUX_TEST
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <netinet/in.h>
#endif


#ifdef FS_STANDALONE

#include "../sm-fs/fs-includes.h"
#include "../sm-tpl/tpl-api.h"
#include "../sm-pkt/pkt-api.h"
//#include "../sm-bsd-api/bsdsock-api.h"
#include "../sm-cifs/nfp-objects-api.h"
#include "../sm-cifs/cifs.x"
#include "../sm-netbios/nb.h"

#else   //FS_STANDALONE

#include <nfx-incl.h>
#include <exports/cm-api.h>
#include <exports/malloc-api.h>
#include <exports/eee-api.h>
#include <exports/elog-api.h>
#include <exports/tpl-api.h>
#include <exports/pkt-api.h>
#include <exports/bsdsock-api.h>
#include <exports/nfp-objects-api.h>
#include <exports/timer-api.h>
#include <exports/cifs.x>
#include <exports/nb.h>

#endif  //FS_STANDALONE

/*-----------------------
 * Session control block
 *-----------------------
 */
// Control block for fragmented user data
typedef struct nb_ssnFragUdCB
{
	pkt_desc_t	*firstP;			// First packet descriptor
	pkt_desc_t	*currP;				// Current packet descriptor
	uint32		totalLen;			// Total bytes of user data
	uint32		currLen;			// # of bytes of user data received so far
}  nb_ssnFragUdCB_t;

typedef struct nb_ssnCB
{
	struct nb_ssnCB		*prevP;		// Pointer to previous entry
	struct nb_ssnCB		*nextP;		// Pointer to next entry
	struct nb_srvCB		*nbSrvCbP;	// Prt to the server the session belongs to
	nb_serviceCB_t		sessionService;
	nb_ssnFragUdCB_t	fragUd;		// Fragmented user data CB
	void				*cifsSsnCbP;// Prt to CIFS session CB
	uint16				state;		// Init, establishing, or established
	uint32				rIpAddr;	// CIFS client IP address
	uint16				rPort;		// CIFS client port
	nb_name_t			rName;		// CIFS client NetBIOS name
	nb_name_t			lName;		// Local NetBIOS name. It should be CIFS 
									// server's NetBIOS name.
	int32				kaTimerHdl;	// Keep-alive timer handle
} nb_ssnCB_t;

/*--------------------------------------------------------------------------
 * Register control block. Every CIFS server needs to register with NetBIOS.
 *--------------------------------------------------------------------------
 */
typedef struct nb_srvRegCB
{
	uint32		cifsSrvIpAddr;	// CIFS server IP address 
	uint32		subnetMask;		// Subnet mask
	uint32		winsSrvIpAddr;	// WINS server IP address. 
								// 0 if not configured 
	void		*cifsSrvCbP;	// CIFS server CB ptr 
	nb_name_t	cifsSrvName;	// CIFS server NetBIOS name
	nb_name_t	cifsDomName;	// CIFS server's domain name

	/*
	 * Name registration confirmation upcall
	 */
	void	(*ul_nameRegCfmInd) (void		*cifsSrvCbP,	// CFIS srv CB ptr
								 int32		status);		// NFX_OK or NFX_ERR

	/*
	 * Session establishment indication upcall, indicating a session
	 * establishment request is received. Return code NFX_OK means
	 * CIFS accepts the session; return code NFX_ERR means CIFS rejects
	 * the session. 
	 */
	int32	(*ul_ssnInd) (void			*cifsSrvCbP,// CFIS srv CB ptr
						  nb_ssnCB_t 	*nbSsnCbP,	// NetBIOS session CB ptr
						  void			**cifsSsnCbP);
											// CIFS session CB ptr upon return

	/*
	 * Session message reception indication upcall
	 */
	void	(*ul_rcvPktInd) (void		*cifsSsnCbP,	// CIFS session CB ptr
							 pkt_desc_t	*msgBufferP,	// Received msg buffer
							 void		*smbMsgHdrP,	// SMB header ptr
							 uint32		smbMsgLen);		// SMB msg length

	/*
	 * Session message transmission completion indication upcall
	 */
	void	(*ul_ssnXmtCmpInd) (void		*cifsSsnCbP, // CIFS session CB ptr
								void		*cifsXmtCbP, // CIFS xmt CB ptr
								pkt_desc_t	*xmtPktDescP);

	/* 
	 * Session termination indication upcall
	 */
	void	(*ul_ssnEndInd) (void	*cifsSsnCbP);	// CIFS session CB ptr

} nb_srvRegCB_t;

/*-----------------------
 * Server control block
 *-----------------------
 */
typedef struct nb_srvCB
{
	struct nb_srvCB		*prevP;			// Pointer to previous entry
	struct nb_srvCB		*nextP;			// Pointer to next entry
	nb_srvRegCB_t		regCb;			// CIFS server register CB
	uint16				currNodeType;	// Current node type:
										//     B-node or H-node
	uint64				srvNameTTL;		// Time to live of the CIFS server name
	uint32				srvNameState;	// CIFS server name registration state
	uint32				domNameState;	// Domain name registration state
	nb_serviceCB_t		nameService;
	nb_serviceCB_t		sessionService;
	nb_serviceCB_t		datagramService;
	nb_announceCB_t		annceCb;		// Server announcement CB
	nb_ssnCB_t			*ssnCbP;		// Point to a list of session CB 
	nb_reqCB_t			*reqCbP;		// Point to a list of request CB
	nb_queryCB_t		*queryCbP;		// Query control block list. 
										//  - Most likely used for quering DCs.
										//  - A CIFS server might request 
										//    several queries.
										//  - The list serves as cache for names
	int32				nameListTimerHdl;
} nb_srvCB_t;


/*-----------------------------------------------------------------------------
 * Name			: nb_srvRegisterReq
 *
 * Description	: A CIFS server registers with NetBIOS module. Called by CIFS.
 *
 * Created by	: Xiaobing Zhang
 *
 * Date created	: 11/07/2001
 *-----------------------------------------------------------------------------
 */
int32 			// NFX_OK or NFX_ERR
nb_srvRegisterReq(nb_srvRegCB_t	*regCbP,	// Register request CB prt
				  nb_srvCB_t	**nbSrvCbP);// NetBIOS srv CB ptr upon return

/*-----------------------------------------------------------------------------
 * Name			: nb_srvReleaseReq
 *
 * Description	: A CIFS server unregister with NetBIOS module.
 * 				  Release name and disconnect all running sessions.
 *
 * Created by	: Xiaobing Zhang
 *
 * Date created	: 11/07/2001
 *-----------------------------------------------------------------------------
 */
int32			// NFX_OK or NFX_ERR 
nb_srvReleaseReq (nb_srvCB_t  *nbSrvCbP,		// NetBIOS srv CB ptr
				  int32 (*ul_releaseCfmInd) ()	// Callback function
				 );

#ifdef NOTUSED
/*-----------------------------------------------------------------------------
 * Name			: nb_nameQueryReq
 *
 * Description	: Process a NetBIOS name query request. Called by CIFS.
 *
 * Created by	: Xiaobing Zhang
 *
 * Date created	: 11/07/2001
 *-----------------------------------------------------------------------------
 */
int32			// NFX_OK or NFX_ERR 
nb_nameQueryReq (nb_srvCB_t	*nbSrvCbP,		// NetBIOS server CB ptr
				 nb_name_t	name,			// Name to query
				 void		*cifsQueryCbP,	// CIFS query CB ptr. Given back
											// to caller in callback func.
				 void (*ul_nameQueryInd) (		// Callback function 
							uint32 ipAddr, 		// IP address related to name
							void *cifsQueryCbP,	// CIFS query CB ptr
							int32 status)		// NFX_OK or NFX_ERR
				 );
#endif /* NOTUSED */

/*-----------------------------------------------------------------------------
 * Name			: nb_ssnSetupReq
 *
 * Description	: Process a NetBIOS session establishment request. 
 * 				  Called by CIFS.
 *
 * Created by	: Xiaobing Zhang
 *
 * Date created	: 11/07/2001
 *-----------------------------------------------------------------------------
 */
int32			// NFX_OK or NFX_ERR
nb_ssnSetupReq (nb_srvCB_t	*nbSrvCbP,		// NetBIOS server CB ptr
				uint32		raddr,			// Destination IP address
				uint16		rport,			// Destination port
				void		*cifsSsnCbP,	// CIFS session CB ptr. Given
											// back to caller in callback.
				nb_ssnCB_t	**nbSsnCbP,		// NetBIOS session CB ptr
												// upon return
				void (*ul_ssnSetupCfmInd) (		// Callback function
							void *cifsSrvCbP,	// CIFS server CB ptr
							void *cifsSsnCbP,	// CIFS session CB prt
							int32 status)		// NFX_OK or NFX_ERR

			   );

/*-----------------------------------------------------------------------------
 * Name			: nb_ssnXmtPktReq
 *
 * Description	: Process a NetBIOS session send message request. 
 * 				  Called by CIFS.
 *
 * Created by	: Xiaobing Zhang
 *
 * Date created	: 11/07/2001
 *-----------------------------------------------------------------------------
 */
int32			// NFX_OK or NFX_ERR
nb_ssnXmtPktReq (nb_ssnCB_t	*nbSsnCbP,		// NetBIOS session CB ptr
		  		 void		*cifsXmtCbP,	// CIFS xmt CB ptr
		 		 pkt_desc_t	*msgBufferP,	// Msg to be sent buffer ptr
				 uint32		smbMsgLen);		// SMB message length

/*-----------------------------------------------------------------------------
 * Name			: nb_ssnEndReq
 *
 * Description	: Process a NetBIOS session termination request. Called by CIFS.
 *
 * Created by	:Xiaobing Zhang
 *
 * Date created	: 11/07/2001
 *-----------------------------------------------------------------------------
 */
void
nb_ssnEndReq (nb_ssnCB_t	*nbSsnCbP);		// NetBIOS session CB ptr

/*-----------------------------------------------------------------------------
 * Name			: nb_ssnGetRemoteIpAddr
 *
 * Description	: Get IP address of remote side of a session. Called by CIFS.
 *
 * Created by	:Xiaobing Zhang
 *
 * Date created	: 11/12/2001
 *-----------------------------------------------------------------------------
 */
uint32			// Remote IP address
nb_ssnGetRemoteIpAddr(nb_ssnCB_t	*nbSsnCbP);	// NetBIOS session CB ptr

/*-----------------------------------------------------------------------------
 * Name			: nb_ssnGetRemotePort
 *
 * Description	: Get port of remote side of a session. Called by CIFS.
 *
 * Created by	:Xiaobing Zhang
 *
 * Date created	: 11/12/2001
 *-----------------------------------------------------------------------------
 */
uint16			// Remote port
nb_ssnGetRemotePort(nb_ssnCB_t	*nbSsnCbP);	// NetBIOS session CB ptr

/*-----------------------------------------------------------------------------
 * Name			: nb_ssnGetRemoteName
 *
 * Description	: Get NetBIOS name of remote side of a session. Called by CIFS.
 *
 * Created by	:Xiaobing Zhang
 *
 * Date created	: 11/12/2001
 *-----------------------------------------------------------------------------
 */
nb_name_t		// Remote name
nb_ssnGetRemoteName(nb_ssnCB_t	*nbSsnCbP);	// NetBIOS session CB ptr

/*-----------------------------------------------------------------------------
 * Name			: nb_ssnGetLocalName
 *
 * Description	: Get local NetBIOS name in a session request. Called by CIFS.
 *
 * Created by	:Xiaobing Zhang
 *
 * Date created	: 11/12/2001
 *-----------------------------------------------------------------------------
 */
nb_name_t		// Local name 
nb_ssnGetLocalName(nb_ssnCB_t	*nbSsnCbP);	// NetBIOS session CB ptr

/*-----------------------------------------------------------------------------
 * Name			: nb_allocOutgoingPkt
 *
 * Description	: Calls TPL packet allocation function to allocate a packet 
 * 				  buffer. Called by CIFS.
 *
 * Created by	: Xiaobing Zhang
 *
 * Date created	: 11/07/2001
 *-----------------------------------------------------------------------------
 */
pkt_desc_t *	// PKT descriptor or NULL
nb_allocOutgoingPkt (nb_ssnCB_t	*nbSsnCbP,		// NetBIOS session CB ptr
			 									// For datagram, it is NULL.
					 void		**smbMsgHdrP);	// Ptr to location where
												// the SMB message should start.

/*-----------------------------------------------------------------------------
 * Name			: nb_freeOutgoingPkt
 *
 * Description	: Calls TPL packet free function to free a packet buffer.
 * 				  Called by CIFS.
 *
 * Created by	: Xiaobing Zhang
 *
 * Date created	: 11/07/2001
 *-----------------------------------------------------------------------------
 */
int32			// NFX_OK or TPL error
nb_freeOutgoingPkt (nb_ssnCB_t	*nbSsnCbP,	// NetBIOS session CB ptr
			 								// For datagram, it is NULL.
					pkt_desc_t	*pktDescP);	// Packet descriptor to be freed

/*-----------------------------------------------------------------------------
 * Name			: nb_freeIncomingPkt
 *
 * Description	: Calls PKT or TPL packet free function to free a 
 * 				  packet buffer that stores incoming packet. Called by CIFS.
 *
 * Created by	: Xiaobing Zhang
 *
 * Date created	: 11/07/2001
 *-----------------------------------------------------------------------------
 */
int32			// NFX_OK or PKT error
nb_freeIncomingPkt (nb_ssnCB_t	*nbSsnCbP,	// NetBIOS session CB ptr
			 								// For datagram, it is NULL.
					pkt_desc_t	*pktDescP);	// Packet descriptor to be freed

/*-----------------------------------------------------------------------------
 * Name			: nb_enableBrowserAnnouncement
 *
 * Description	: Called by CIFS to send server announcement
 * 				  to the master browser.
 *
 * Created by	: Xiaobing Zhang
 *
 * Date created	: 11/07/2001
 *-----------------------------------------------------------------------------
 */
int32			// NFX_OK or NFX_ERR
nb_enableBrowserAnnouncement(
					nb_srvCB_t	*nbSrvCbP,		// Prt to NetBIOS server CB
					char		*destName,		// Destination name
					pkt_desc_t	*msgBufferP,	// Ptr to msg to be sent buffer
					uint32		smbMsgLen);		// SMB message length

/*-----------------------------------------------------------------------------
 * Name			: nb_init
 *
 * Description	: NetBIOS initialization function. Called from CIFS.
 *
 * Created by	: Xiaobing Zhang
 *
 * Date created	: 11/07/2001
 *-----------------------------------------------------------------------------
 */
void
nb_init(uint32	maxSrvCnt,		// Max server count
		uint32	maxSsnCnt,		// Max session count
		int32	cifsAppId);		// CIFS application ID registered with EEE


#endif /* _NB_API_H */
