/* vi:ts=4
 *-----------------------------------------------------------------
 *
 * Copyright (C) 2001, Agile Storage Inc.
 *
 * RCS:  $Id: nb-ss.h,v 1.1.1.1 2002/04/23 17:52:14 laurentb Exp $
 *
 * Name			: nb-ss.h
 *
 * Description	: NetBIOS session service definitions.
 *
 * Created by	: Xiaobing Zhang
 *
 * Date Created	: 12/01/2001
 *
 *-----------------------------------------------------------------
 */
#ifndef _NB_SS_H
#define _NB_SS_H

/*----------------------------
 * Session service definitions
 *----------------------------
 */
#define NB_SS_HEADER_LEN			4		// NetBIOS session header length

#define NB_SS_TCP_PORT				139
#define NB_SS_N_RETRY				4		// Maximum number TCP connection 
											// attempts allowable a single 
											// NetBIOS session request
#define NB_SS_CLOSE_TIMEOUT			30000	// 30 seconds
#ifndef NBTEST
#define NB_SS_KEEP_ALIVE_TIMEOUT	60000	// 60 seconds
#else
#define NB_SS_KEEP_ALIVE_TIMEOUT	6000000	// 6000 seconds. for testing
#endif /* NBTEST */

/* Session packet type */
#define NB_SS_TYPE_MSG				0x00	// Session message for user data
#define NB_SS_TYPE_REQ				0x81	// Session establishment request
#define NB_SS_TYPE_POSRSP			0x82	// Positive session response
#define NB_SS_TYPE_NEGRSP			0x83	// Negative session response
#define NB_SS_TYPE_RETRSP			0x84	// Retarget session response
#define NB_SS_TYPE_KEEPALIVE		0x85	// Session keep alive

/* Session error code */
#define NB_SS_ERR_NOT_LISTEN_CALLEDNAME		0x80
#define NB_SS_ERR_NOT_LISTEN_CALLINGNAME	0x81
#define NB_SS_ERR_CALLEDNAME_NOT_PRESENT	0x82
#define NB_SS_ERR_INSUFFICIENT_RSRC			0x83
#define NB_SS_ERR_UNSPECIFIED				0x8F

/* Session state */
#define NB_SS_INIT					1	// Initial state
#define NB_SS_IN_ESTABLISHING		2	// An incoming session is establishing 
#define NB_SS_OUT_ESTABLISHING		3	// An outgoing session is establishing
#define NB_SS_ESTABLISHED			4	// A session is established 


#define NB_SS_GETLEN(buf) ((((buf)[1] & 0x01) << 16) | GETBE2(buf,2))
#define NB_SS_SETLEN(buf, val) { (buf)[1] = ((val) >> 16) & 0x01; \
								 PUTBE2((buf), 2, ((val) & 0xFFFF)); \
							   }

/*--------------------
 * Function prototypes
 *--------------------
 */
void nb_ssKeepAlive(nb_ssnCB_t *ssnCbP);

int32 nb_ssKaTimeout(int32 timerHdl, void* param1,
					 void* param2, void* param3, void* param4);

int32 nb_ssXmtMsg(nb_ssnCB_t *ssnCbP,
				  pkt_desc_t *xmtPktDescP,
				  uint32 msgType,
				  int32  errCode);

int32 nb_ssProcUserData(nb_ssnCB_t *ssnCbP, pkt_desc_t *rcvPktDescP);

int32 nb_ssProcEstablishReq(nb_ssnCB_t *ssnCbP, pkt_desc_t *rcvPktDescP);

int32 nb_ssRcvMsg(nb_ssnCB_t *ssnCbP, pkt_desc_t *rcvPktDescP);

#endif /* _NB_SS_H */
