/* vi:ts=4
 *-----------------------------------------------------------------
 *
 * Copyright (C) 2001, Agile Storage Inc.
 *
 * RCS:  $Id: nb.h,v 1.1.1.1 2002/04/23 17:52:14 laurentb Exp $
 *
 * Name			: nb.h
 *
 * Description	: NetBIOS module definitions.
 *
 * Created by	: Xiaobing Zhang
 *
 * Date Created	: 11/07/2001
 *
 *-----------------------------------------------------------------
 */
#ifndef _NB_H
#define _NB_H

/*------------------------------
 * General constants definitions
 *------------------------------
 */

/*
#define NB_N_MAX_SERVER			255			// Max number of CIFS servers 
#define NB_N_MAX_SESSION		4096		// Max number of CIFS sessions
*/
#define NB_N_MAX_SERVER			4			// Max number of CIFS servers 
#define NB_N_MAX_SESSION		16			// Max number of CIFS sessions

#define NB_N_MAX_REQUEST		(NB_N_MAX_SERVER * 5)
											// Each server may have up to 5
											// name/datagram requests pending
#define NB_N_MAX_QUERY			(NB_N_MAX_SERVER * 3)
											// Each server may have up to 
											// 3 name queries pending
#define NB_N_MAX_INMSG			(NB_N_MAX_SERVER * 10)
										// Each server may have up to 10 name or
										// datagram incoming messages pending

#define NB_N_MAX_IPADDR			25			// Max number of IP addresses
											// that each group name may have

#define NB_NS	1
#define NB_SS	2
#define NB_DS	3

#define NB_BCAST_REQ_RETRY_TIMEOUT		250		// Default is 250 ms
#define NB_BCAST_REQ_RETRY_TIMEOUT_MS	750		// Microsoft uses 750 ms
#define NB_N_BCAST_REQ_RETRY			3

#define NB_UCAST_REQ_RETRY_TIMEOUT		5000	// Default is 5 seconds
#define NB_N_UCAST_REQ_RETRY			3

#define NB_S_MAX_IP_DATAGRAM		576		// Default

#define NB_S_NAME					15
/*
#define NB_S_SCOPE					63
*/
#define NB_S_COMPRESSED_NAME		(NB_S_NAME + 2) * 2

#define NB_NAME_LIST_TIMEOUT		1*60*1000		// 1 minute

#define NB_MIN_HOST_ANNCE_INTERVAL	1*60*1000		// 1 minute
#define NB_MAX_HOST_ANNCE_INTERVAL	12*60*1000		// 12 minutes

/*-----------
 * Error code
 * TODO: add in file nfx-error.h error base for Netbios and CIFS 
 *-----------
 */

/*-------------------
 * Name serivce state
 *-------------------
 */
#define NB_NAME_STATE_UNREGISTERED	0x00  /* Initial state.
										   * Name is not registered */
#define NB_NAME_STATE_REGISTERING	0x01  /* Name is registering */
#define NB_NAME_STATE_REGISTERED	0x02  /* Name is registered */
#define NB_NAME_STATE_DEREGISTERING	0x04  /* Name is deregistering */
#define NB_NAME_STATE_REFRESHING	0x08  /* Name is refreshing */
#define NB_NAME_STATE_CONFLICT		0x10  /* Name is conflict */
#define NB_NAME_STATE_QUERING		0x20  /* name is being queried */
#define NB_NAME_STATE_VALID			0x40  /* After quering, the name 
										   * becomes valid */
/*-------------
 * NetBIOS name
 *-------------
 */
typedef struct nb_name
{
	uint32	type;
	char	name[NB_S_NAME + 1];
	/* TODO: The scope is not considered yet.
	char	scope[NB_S_SCOPE + 1];
	*/
} nb_name_t;

/*----------------------------------------------------
 * Name query control block. Name to be queried can be 
 * server, workstation, pdc, etc.
 *----------------------------------------------------
 */
typedef struct nb_queryCB
{
	struct nb_queryCB	*prevP;
	struct nb_queryCB	*nextP;
	nb_name_t		name;			// NetBIOS name
	uint16			state;			// State of the name: quering, valid
	uint16			ipAddrCnt;		// Number of IP addresses of the name
	uint32			ipAddr[NB_N_MAX_IPADDR];	// Name's IP address(es),
												// obtained from query response
	uint32			ttl;			// Time to live of the name. The value is 
									// obtained from query response.
	// The following 5 variables are for name confilct check
	uint32			raddr;			// IP address of node that sent 
									// authoritative response to this query.
	uint16			rport;			// Port of node that sent authoritative response
	boolean			group;			// TRUE if the authoritative response 
									// is for group name.
	uint16			rspMsgCnt;		// Number of response message received
	int32			conflTimerHdl;	// Conflict timer handle

	struct nb_reqCB	*nbReqCbP;		// When state is quering, this points to
									// the request CB; otherwise, it is NULL.
	struct nb_srvCB *nbSrvCbP;
	void			*userInfo;		// Ptr to user specific info
	void (*nb_nsNameQueryInd) (		// Callback function 
				uint16 ipAddrCnt,	// Number of IP addresses of the name
				uint32 *ipAddr,		// Ptr to IP address related to name
				void *userInfo,		// Ptr to user specific info
				int32 status);		// NFX_OK or NFX_ERR
} nb_queryCB_t;

/*-------------------------------
 * Outgoing request control block
 *-------------------------------
 */
typedef struct nb_reqCB
{
	struct nb_reqCB	*prevP;
	struct nb_reqCB	*nextP;
	struct nb_srvCB	*nbSrvCbP;
	struct nb_reqCB	*bcastReqCbP;	// These two are for name registration or
	struct nb_reqCB	*ucastReqCbP;	// release request when WINS server is 
									// configured. If unicast request fails, 
									// a corresponding broadcast request is 
									// sent. These point to the corresponding
									// broadcast and unicast request CBs.
	uint16			reqType;
#define NB_REQ_TO_WINS				0x80
#define NB_REQ_TYPE_NAME_REG		0x01
#define NB_REQ_TYPE_NAME_QUERY		0x02
#define NB_REQ_TYPE_NAME_RELEASE	0x03
#define NB_REQ_TYPE_NAME_REFRESH	0x04
#define NB_REQ_TYPE_SESSION			0x05
#define NB_REQ_TYPE_DATAGRAM		0x07
	uint16			reqID;			// ID used to identify request
									// For name service, it is transaction ID
	void			*reqPktP;		// Point to different data structure
									// depending on request type.
	pkt_desc_t		*reqXmtPktP;
	uint32			raddr;
	uint16			rport;
	uint16			retryCnt;		// Broadcast retry counter
	int32			reqTimerHdl;
	uint32			retryInterval;	// Retry interval.
	nb_queryCB_t	*queryCbP;		// For name query request, this points to
									// query CB.
} nb_reqCB_t;

/*---------------------------
 * Service control block
 *---------------------------
 */
typedef struct nb_serviceCB
{
	void	*tplBindCbP;	// TPL bind CB ptr
	void	*tplConnCbP;	// TPL connection CB ptr
	void	*bsdSockP;		// BSD socket ptr
	uint32	mtu;			// MTU for the connection.
							// TODO type of mtu should be consistent with TPL
} nb_serviceCB_t;

/*-------------------------------
 * Incoming message control block
 *-------------------------------
 */
typedef struct nb_incomingMsgCB
{
	struct nb_srvCB	*nbSrvCbP;
	uint32			raddr;
	uint16			rport;
} nb_incomingMsgCB_t;

/*----------------------------------
 * Server announcement control block
 *----------------------------------
 */
typedef struct nb_announceCB
{
	uint32				interval;		// Announce interval.
   										// Minimum value is 1 minute.
										// Maximum value is 12 minutes.
	uint32				count;			// Resent count
	struct nb_dsPacket	*anncePktP;
	pkt_desc_t			*xmtPktP;	// Packet containing announcement message.
	int32				annceTimerHdl;	// Announcement timer handle
} nb_announceCB_t;

/*-----------------------------
 * NetBIOS global configuration
 *-----------------------------
 */
typedef struct nb_globalCfg
{
	nfp_objList		srvCbList;		// Server CB list. Contains used and unused
	nfp_objList		ssnCbList;		// Session CB list, Contains used and unused
	nfp_objList		reqCbList;		// Request CB list, Contains used and unused
	nfp_objList		queryCbList;	// Query CB list, Contains used and unused
	nfp_objList		inMsgCbList;	// Incoming msg CB list, used and unused
	uint32			serverCnt;		// # of servers in the list
	struct nb_srvCB	*servers;		// A list of CIFS servers	

	/*
	nb_statCounters_t	statCounters;	// NetBIOS status counters
	nb_nsCounters_t		nsCounter;		// NetBIOS name service counters
	nb_ssCounters_t		ssCounter;		// NetBIOS session service counters
	nb_dsCounters_t		dsCounter;		// NetBIOS datagram service counters
	*/
} nb_globalCfg_t;

#define nb_allocSrvCb() \
		(nb_srvCB_t*)nfp_allocObject(&nb_globalCfgP->srvCbList, TRUE)
#define nb_allocSsnCb() \
		(nb_ssnCB_t*)nfp_allocObject(&nb_globalCfgP->ssnCbList, TRUE)
#define nb_allocReqCb() \
		(nb_reqCB_t*)nfp_allocObject(&nb_globalCfgP->reqCbList, TRUE)
#define nb_allocQueryCb() \
		(nb_queryCB_t*)nfp_allocObject(&nb_globalCfgP->queryCbList, TRUE)
#define nb_allocInMsgCb() \
		(nb_incomingMsgCB_t*)nfp_allocObject(&nb_globalCfgP->inMsgCbList, TRUE)

#define nb_freeSrvCb(srv) \
		nfp_deallocObject(&nb_globalCfgP->srvCbList, srv);
#define nb_freeSsnCb(ssn) \
		nfp_deallocObject(&nb_globalCfgP->ssnCbList, ssn);
#define nb_freeReqCb(req) \
		nfp_deallocObject(&nb_globalCfgP->reqCbList, req);
#define nb_freeQueryCb(query) \
		nfp_deallocObject(&nb_globalCfgP->queryCbList, query);
#define nb_freeInMsgCb(inMsg) \
		nfp_deallocObject(&nb_globalCfgP->inMsgCbList, inMsg);

/*--------
 * Externs
 *--------
 */
extern uint32 nfxMyCPUId;
extern uint32 nfxMySlotId;

extern nb_globalCfg_t	*nb_globalCfgP;
extern uchar8			nb_debugLevel;
extern int32			nb_cifsAppId;
extern uint32			nb_elog_map[6];

/*--------------------
 * Debug traces levels
 *--------------------
 */
#define NB_DBGLV_ERR		1
#define NB_DBGLV_WARN		2
#define NB_DBGLV_INFO		3
#define NB_DBGLV_DBG		4
#define NB_DBGLV_DBG2		5

#define NB_PRINT(_lvl_, _par_)                                                \
    if (_lvl_ <= nb_debugLevel)                                               \
        E_LOG(class_1, nb_elog_map[_lvl_], nb_cifsAppId,                      \
              0, 0, 0, _par_)

/*------------------
 * Memory allocation
 *------------------
 */
#define NB_RAM_ALLOC(_p_, _type_, _size_)                                     \
    _p_ = (_type_ *) eee_ramAlloc(_size_);                                    \
    if (_p_ == NULL)                                                          \
    {                                                                         \
        NB_PRINT(NB_DBGLV_ERR, ("eee_ramAlloc() failed"));                    \
        NFX_ASSERT(TRUE);                                                     \
    }

#define NB_RAM_ALLOC_ZERO(_p_, _type_, _size_)                                \
    _p_ = (_type_ *) eee_ramAllocZero(_size_);                                \
    if (_p_ == NULL)                                                          \
    {                                                                         \
        NB_PRINT(NB_DBGLV_ERR, ("eee_ramAllocZero() failed"));                \
        NFX_ASSERT(TRUE);                                                     \
    }

#define NB_RAM_ALLOC_ALIGNED(_p_, _type_, _size_)                             \
    _p_ = (_type_ *) eee_ramAllocAligned(_size_);                             \
    if (_p_ == NULL)                                                          \
    {                                                                         \
        NB_PRINT(NB_DBGLV_ERR, ("eee_ramAllocAligned() failed"));             \
        NFX_ASSERT(TRUE);                                                     \
    }

#define NB_RAM_ALLOC_ALIGNED_ZERO(_p_, _type_, _size_)                        \
    _p_ = (_type_ *) eee_ramAllocAlignedZero(_size_);                         \
    if (_p_ == NULL)                                                          \
    {                                                                         \
        NB_PRINT(NB_DBGLV_ERR, ("eee_ramAllocAlignedZero() failed"));         \
        NFX_ASSERT(TRUE);                                                     \
    }

#define NB_RAM_DEALLOC(_p_)                                                   \
    if (_p_ != NULL)                                                          \
    {                                                                         \
        eee_ramDealloc(_p_);                                                  \
        _p_ = NULL;                                                           \
    }

#endif /* _NB_H */
