/* vi:ts=4
 *-----------------------------------------------------------------
 *
 * Name:        nfp-api.c
 *
 * RCS:  $Id: nfp-api.c,v 1.37 2002/11/06 03:35:43 joseph Exp $
 *
 * Copyright (C) 2001, Agile Storage Inc.
 *
 * Description: 
 *
 * Created by: Joseph Vu 
 *
 * Date Created:        5/18/01
 *
 *-----------------------------------------------------------------
 */
#include <nfx-incl.h>
#include <exports/nfs-api.h>
#include <exports/queue-api.h>
#include <sys/machine-dep.h>
#include <exports/hash-api.h>
#include <exports/lock-api.h>
#include <exports/vs-mem-api.h>

#define		NFP_DBG_PRINTF		

//#define		NFP_USE_SHMEM	1
#ifdef	NFP_USE_SHMEM
#define NFP_SHMEM_ALLOC		eee_allocateBlock
#else
#define	NFP_SHMEM_ALLOC		eee_ramAlloc
#endif	/*NFP_USE_SHMEM*/

//static q_headNodePtr_t		nfp_fileCbQueue;


nfp_globalCfg_t				*nfp_globalCfgp;




/*-----------------------------------------------------------------
 * Name:       	nfp_allocNode 
 *
 * Description: Allocate a generic node for maintaining list for
 * active clients or list of filesystem or list of open files.
 *
 * Created by:  Joseph Vu
 *
 * Date Created:        5/18/01
 *
 *-----------------------------------------------------------------
 */
nfp_nodePtr_t		/*Return pointer to node entry or NULL if entry
			 *can not be allocated
			 */
nfp_allocNode(void)
{
	nfp_nodePtr_t       availEntryp;
	int32               index;
	int32               count;

	/*
	 *Check for any free entries
	 */
	if (nfp_globalCfgp->nodeCbCfgList.freeEntriesCnt <= 0)
	{
		//printf("nfp_allocNode: no free NFP node!\n");
		return (NULL);
	}
	/*
	 *Get current index
	 */
	availEntryp = (nfp_nodePtr_t)nfp_globalCfgp->nodeCbCfgList.listBasep;
	index = nfp_globalCfgp->nodeCbCfgList.listIndex;
	availEntryp += index;
	for (count = 0; ((availEntryp != NULL) && (count <  NFP_MAX_NODE)); count++)
	{
		if (!(availEntryp->nfp_flag & NFP_ENTRY_INUSED))
			break;
		if (index == (NFP_MAX_NODE - 1))
		{
			/*
			 *We've reach end of the list, go back to the top
			 *of the list
			 */
			availEntryp = (nfp_nodePtr_t)nfp_globalCfgp->nodeCbCfgList.listBasep;
			index = 0;
			continue;
		}
		index++;
		availEntryp++;
	}
	if (index == (NFP_MAX_NODE-1))
		nfp_globalCfgp->nodeCbCfgList.listIndex = 0;
	else
		nfp_globalCfgp->nodeCbCfgList.listIndex = index + 1;
#ifdef	NFP_DEBUG
	NFP_DBG_PRINTF("nfp_allocNode: availEntryp: %p avail index: %d\n",
					availEntryp, index);
	NFP_DBG_PRINTF("nfp_allocNode: next index: %d\n",
					nfp_globalCfgp->nodeCbCfgList.listIndex);
#endif
	nfp_globalCfgp->debugStat.numOfAllocNode++;
	nfp_globalCfgp->nodeCbCfgList.freeEntriesCnt -= 1;
	memset(availEntryp, 0, sizeof(nfp_node_t));
	availEntryp->nfp_flag |= NFP_ENTRY_INUSED;
	if (nfp_globalCfgp->nodeCbCfgList.freeEntriesCnt < 
		nfp_globalCfgp->debugStat.nodeCbLwm)
		nfp_globalCfgp->debugStat.nodeCbLwm = 
				nfp_globalCfgp->nodeCbCfgList.freeEntriesCnt;

	return (availEntryp);

}
/*-----------------------------------------------------------------
 * Name:        nfp_deallocNode
 *
 * Description: De-allocate a node and return it back to
 * the available buffer list.
 *
 * Created by:  Joseph Vu
 *
 * Date Created:        5/18/01
 *
 *-----------------------------------------------------------------
 */
int32
nfp_deallocNode (nfp_nodePtr_t deallocEntryp	/*Pointer to de-allocate entry*/
)
{
	//NFP_DBG_PRINTF("nfp_deallocNode: deallocEntryp: %p\n", deallocEntryp);
	if (deallocEntryp == NULL)
		return (NFX_ERR);
	nfp_globalCfgp->debugStat.numOfFreeNode++;
	deallocEntryp->nfp_flag &= ~NFP_ENTRY_INUSED;
	nfp_globalCfgp->nodeCbCfgList.freeEntriesCnt += 1;
	return (NFX_OK);
}

/*-----------------------------------------------------------------
 * Name:        nfp_validateNode
 *
 * Description: Validates a NFP generic node buffer address.
 *
 * Created by:  Joseph Vu
 *
 * Date Created:        5/18/01
 *
 *-----------------------------------------------------------------
 */
int32
nfp_validateNode(nfp_nodePtr_t nodep)
{
	nfp_nodePtr_t		lastNodep;

	lastNodep = nfp_globalCfgp->nodeCbCfgList.listBasep;
	lastNodep += (NFP_MAX_NODE - 1);
	if ((nodep < (nfp_nodePtr_t)nfp_globalCfgp->nodeCbCfgList.listBasep) ||
		(nodep > lastNodep) )
	{
		nfp_globalCfgp->debugStat.numOfBadNode++;
		return (NFX_ERR);
	}
	return (NFX_OK);
}

/*-----------------------------------------------------------------
 * Name:        nfp_allocClientCB
 *
 * Description: Allocate a Client Control Block (CCB) from the available
 * buffer list.
 *
 * Created by:  Joseph Vu
 *
 * Date Created:        5/18/01
 *
 *-----------------------------------------------------------------
 */
nfp_clientCBPtr_t               /*Return pointer to clientCB entry of NULL
                                 *if entry cannot be allocated
                                 */
nfp_allocClientCB (void)
{
	nfp_clientCBPtr_t   availEntryp;
	int32               index;
	int32               count;

	/*
	 *Check for any free entries
	 */
	if (nfp_globalCfgp->clientCbCfgList.freeEntriesCnt <= 0)
	{
		//printf(" nfp_allocClientCB: no free Client Control Block!\n");
		return (NULL);
	}
	/*
	 *Get the next available entry
	 */
	availEntryp = (nfp_clientCBPtr_t)nfp_globalCfgp->clientCbCfgList.listBasep;
	index = nfp_globalCfgp->clientCbCfgList.listIndex;
	availEntryp += index;
	for (count = 0; ( (availEntryp != NULL) && (count <  NFP_MAX_CLIENT)); count++)
	{
		if (!(availEntryp->nfp_flag & NFP_ENTRY_INUSED))
		{
			/*Found free entry*/
			break;
		}
		if (index == (NFP_MAX_CLIENT-1))
		{
			/*
			 *We 've reach end of the list, go back to
			 *the top of the list
			 */
			availEntryp = (nfp_clientCBPtr_t)nfp_globalCfgp->clientCbCfgList.listBasep;
			index = 0;
			continue;
		}
		index++;
		availEntryp++;
	}

	if (index == (NFP_MAX_CLIENT-1))
	{
		nfp_globalCfgp->clientCbCfgList.listIndex = 0;
	}
	else
		nfp_globalCfgp->clientCbCfgList.listIndex = index + 1;
#ifdef	NFP_DEBUG
	NFP_DBG_PRINTF("nfp_allocClientCB: availEntryp: %p avail index: %d\n",
														availEntryp, index);
	NFP_DBG_PRINTF("nfp_allocClientCB: next index: %d \n",
								nfp_globalCfgp->clientCbCfgList.listIndex );
#endif

	nfp_globalCfgp->debugStat.numOfAllocClientCB++;
	nfp_globalCfgp->clientCbCfgList.freeEntriesCnt -= 1;
	memset(availEntryp, 0, sizeof(nfp_clientCB_t));
	availEntryp->nfp_flag |= NFP_ENTRY_INUSED;
	if (nfp_globalCfgp->clientCbCfgList.freeEntriesCnt <
		nfp_globalCfgp->debugStat.clientCbLwm)
		nfp_globalCfgp->debugStat.clientCbLwm = nfp_globalCfgp->clientCbCfgList.freeEntriesCnt;

	return (availEntryp);
}

/*-----------------------------------------------------------------
 * Name:        nfp_deallocClientCB
 *
 * Description: De-allocate a Client Control Block (CCB) and returns a CCB
 * back to the available buffer list.
 *
 * Created by:  Joseph Vu
 *
 * Date Created:        5/18/01
 *
 *-----------------------------------------------------------------
 */
int32           /*NFX_OK:  Entry has been de-allocate*/
                /*NFX_ERR: Failure to de-allocate entry*/
nfp_deallocClientCB (
        nfp_clientCBPtr_t	deallocEntryp /*Pointer to ClientCB to de-allocate*/
)
{
	if (deallocEntryp == NULL)
		return (NFX_ERR);
	nfp_globalCfgp->debugStat.numOfFreeClientCB++;
	deallocEntryp->nfp_flag &= ~NFP_ENTRY_INUSED;
	nfp_globalCfgp->clientCbCfgList.freeEntriesCnt += 1;

	return (NFX_OK);
}

/*-----------------------------------------------------------------
 * Name:        nfp_addClientCB
 *
 * Description: Add a Client Control Block (CCB) into the  CCB table.
 *
 * Created by:  Joseph Vu
 *
 * Date Created:        5/18/01
 *
 *-----------------------------------------------------------------
 */
int32	/*NFX_OK: Entry has been added*/
		/*NFX_ERR: Failure to add entry*/
nfp_addClientCB (
	uint32				nfp_connId,     /*TCP/UDP connection ID*/
	nfp_clientCBPtr_t	nfp_addEntryp 	/*ClientCB entry to be added*/
)
{
	if (nfp_addEntryp == NULL)
		return (NFX_ERR);

	/*
	 *check for duplicate entry
	 */
	if ((nfp_connId != 0) && nfp_getClientCB (nfp_connId) != NULL)
	{

		return (NFX_ERR);
	}
	nfp_addEntryp->nfp_connId = nfp_connId;


	return (NFX_OK);
}


/*-----------------------------------------------------------------
 * Name:        nfp_delClientCB
 *
 * Description: delete a Client Control Block (CCB) from the  CCB table.
 *
 * Created by:  Joseph Vu
 *
 * Date Created:        5/18/01
 *
 *-----------------------------------------------------------------
 */
int32		/*NFX_OK:  Entry has been deleted*/
			/*NFX_ERR:  Failure to delete entry*/
nfp_delClientCB (
		nfp_clientCBPtr_t	nfp_delEntryp /*ClientCB entry to be deleted*/
)
{
	if (nfp_delEntryp == NULL)
		return (NFX_ERR);
	/*
	 *Check if entry exist
	 */
	if ((nfp_delEntryp->nfp_connId != 0) &&  
		nfp_getClientCB (nfp_delEntryp->nfp_connId) == NULL)
	{
		return (NFX_ERR);
	}
	/*
	 *call de-allocate entry
	 */
	return (nfp_deallocClientCB(nfp_delEntryp));
}

/*-----------------------------------------------------------------
 * Name:       	nfp_getClientCB  
 *
 * Description: Retrieves a Client Control Block (CCB) from the  CCB list 
 *
 * Created by:  Joseph Vu
 *
 * Date Created:        5/18/01
 *
 *-----------------------------------------------------------------
 */
nfp_clientCBPtr_t	/*Pointer to NFP ClientCb or NULL 
					 */
nfp_getClientCB (
		uint32 connId		/*TCP/UDP connection ID*/
)
{
	nfp_clientCBPtr_t   currp;
	uint32              count;
   
	count = 0;
	for (currp = (nfp_clientCBPtr_t)nfp_globalCfgp->clientCbCfgList.listBasep;
		((count < NFP_MAX_CLIENT) && (currp != NULL)); count++, currp++)
	{
		if (!(currp->nfp_flag & NFP_ENTRY_INUSED))
			continue;
		if (connId == currp->nfp_connId)
			return (currp);
	}

	return (NULL);
}

/*-----------------------------------------------------------------
 * Name:       	nfp_getClientCBbyAddr 
 *
 * Description: Retrieves a Client Control Block (CCB) from the  CCB list 
 * by ip address.
 *
 * Created by:  Joseph Vu
 *
 * Date Created:        5/18/01
 *
 *-----------------------------------------------------------------
 */
nfp_clientCBPtr_t	/*Pointer to NFP ClientCb or NULL 
					 */
nfp_getClientCBbyAddr (
		uint32 clientAddr	/*client IP address*/
)
{
	nfp_clientCBPtr_t   currp;
	uint32              count;
   
	count = 0;
	for (currp = (nfp_clientCBPtr_t)nfp_globalCfgp->clientCbCfgList.listBasep;
		((count < NFP_MAX_CLIENT) && (currp != NULL)); count++, currp++)
	{
		if (!(currp->nfp_flag & NFP_ENTRY_INUSED))
			continue;
		if ( clientAddr == currp->nfp_ipAddr )
			return (currp);
	}
	return (NULL);
}


/*-----------------------------------------------------------------
 * Name:       	nfp_validateClientCB 
 *
 * Description: Validates a Client Control Block (CCB) buffer address.
 *
 * Created by:  Joseph Vu
 *
 * Date Created:        5/18/01
 *
 *-----------------------------------------------------------------
 */
int32 nfp_validateClientCB(nfp_clientCBPtr_t clientCBp)
{
	nfp_clientCBPtr_t 	lastClientCBp;

	lastClientCBp = nfp_globalCfgp->clientCbCfgList.listBasep;
	lastClientCBp += (NFP_MAX_CLIENT -1);

	if ((clientCBp < (nfp_clientCBPtr_t)nfp_globalCfgp->clientCbCfgList.listBasep) ||
		(clientCBp > lastClientCBp) )
	{
		nfp_globalCfgp->debugStat.numOfBadClientCB++;
		return (NFX_ERR);
	}
	return (NFX_OK);
}

/*-----------------------------------------------------------------
 * Name:        nfp_allocFileCB
 *
 * Description: Allocate a File Control Block (FCB) from the available
 * buffer list.
 *
 * Created by:  Joseph Vu
 *
 * Date Created:        5/18/01
 *
 *-----------------------------------------------------------------
 */
nfp_fileCBPtr_t         /*Return pointer to fileCB entry or NULL if
                         *entry cannot be allocated
                         */
nfp_allocFileCB (void)
{
	nfp_fileCBPtr_t     availEntryp;
	int32               index;
	int32               count;
    int32               rc;

	/*
	 *Check for any free entries
	 */
	if (nfp_globalCfgp->fileCbCfgList.freeEntriesCnt <= 0)
	{
		//printf("nfp_allocFileCB: no free File Control Block!\n");
		return (NULL);
	}

	/*
	 *Get next available entry by index
	 */
	availEntryp = (nfp_fileCBPtr_t)nfp_globalCfgp->fileCbCfgList.listBasep;
	index = nfp_globalCfgp->fileCbCfgList.listIndex;
	availEntryp += index;
	for (count = 0; ((availEntryp != NULL) && (count <  NFP_MAX_FILE)); count++)
	{
		if (!(availEntryp->nfp_flag & NFP_ENTRY_INUSED))
			break;
		if (index == (NFP_MAX_FILE - 1))
		{
			/*
			 *We 've reach end of the list, go back to the top
			 *of the list
			 */
			availEntryp = (nfp_fileCBPtr_t)nfp_globalCfgp->fileCbCfgList.listBasep;
			index = 0;
			continue;
		}
		index++;
		availEntryp++;
	}
	/*
	 *Set index for next available entry
	 */
	if (index == (NFP_MAX_FILE-1))
		nfp_globalCfgp->fileCbCfgList.listIndex = 0;
	else
		nfp_globalCfgp->fileCbCfgList.listIndex = index + 1;
#ifdef	NFP_DEBUG
	NFP_DBG_PRINTF("nfp_allocFileCB: avail index: %d availEntryp: %p\n", index, availEntryp);
	NFP_DBG_PRINTF("nfp_allocFileCB: next index: %d\n",
							nfp_globalCfgp->fileCbCfgList.listIndex);
#endif

	memset(availEntryp, 0, sizeof(nfp_fileCB_t));
    rc = lock_initFileLockReq(&availEntryp->fileLockIdx);
    if (rc != NFX_OK)
        return (NULL);
	nfp_globalCfgp->debugStat.numOfAllocFileCB++;
	nfp_globalCfgp->fileCbCfgList.freeEntriesCnt -= 1;
	availEntryp->nfp_flag |= NFP_ENTRY_INUSED;
	if (nfp_globalCfgp->fileCbCfgList.freeEntriesCnt <
		nfp_globalCfgp->debugStat.fileCbLwm )
		nfp_globalCfgp->debugStat.fileCbLwm = nfp_globalCfgp->fileCbCfgList.freeEntriesCnt;

	return (availEntryp);
}

/*-----------------------------------------------------------------
 * Name:        nfp_deallocFileCB
 *
 * Description: de-allocate a File Control Block (FCB) and return entry
 * back to  available buffer list.
 *
 * Created by:  Joseph Vu
 *
 * Date Created:        5/18/01
 *
 *-----------------------------------------------------------------
 */
int32           /*NFX_OK:  Entry has been de-allocate*/
                /*NFX_ERR: Failure to de-allocate entry*/
nfp_deallocFileCB (nfp_fileCBPtr_t deallocEntryp  /*de-allocate entry*/ 
                )
{

	if (deallocEntryp == NULL)
		return (NFX_ERR);
	nfp_globalCfgp->debugStat.numOfFreeFileCB++;
	deallocEntryp->nfp_flag &= ~NFP_ENTRY_INUSED;
	nfp_globalCfgp->fileCbCfgList.freeEntriesCnt += 1;
    lock_uninitFileLockReq(&deallocEntryp->fileLockIdx);
	return (NFX_OK);
}

/*-----------------------------------------------------------------
 * Name:        nfp_addFileCB
 *
 * Description: Add a File Control Block (FCB) into the hash table.
 * 
 *
 * Created by:  Joseph Vu
 *
 * Date Created:        5/18/01
 *
 *-----------------------------------------------------------------
 */
int32           /*NFX_OK:  Entry has been added*/
                /*NFX_ERR: Failure to add entry*/
nfp_addFileCB (
	nfp_fileCBPtr_t	nfp_fileCBp,   	/*Entry to be added*/
	fs_Handle		*nfp_fhandlep	/*filehandle*/
)
{
//	nfp_fileCBPtr_t	fileCBp;	
	
	if ( (nfp_fhandlep == NULL) || (nfp_fileCBp == NULL) )
		return (NFX_ERR);

	memcpy(&nfp_fileCBp->nfp_fileHandle, 
			nfp_fhandlep, sizeof(fs_Handle));
	
	if (hash_insert(nfp_globalCfgp->fileCbHash, 
				(void *)nfp_fhandlep, nfp_fileCBp) != NFX_OK)
		return (NFX_ERR);


	nfp_fileCBp->nfp_flag |= NFP_ENTRY_ADDED;
#ifdef	NFP_DEBUG
	NFP_DBG_PRINTF("nfp_addFileCB: fileCBp %p volId: %lld inum: %lld\n", nfp_fileCBp,
				nfp_fileCBp->nfp_fileHandle.volId.whole_thing, nfp_fileCBp->nfp_fileHandle.inum);
#endif

	return (NFX_OK);	

}


/*-----------------------------------------------------------------
 * Name:        nfp_delFileCB
 *
 * Description: Delete a File Control Block (FCB) from the hash table.
 *
 * Created by:  Joseph Vu
 *
 * Date Created:        5/18/01
 *
 *-----------------------------------------------------------------
 */
int32           /*NFX_OK:  Entry has been deleted*/
                /*NFX_ERR: Failure to delete entry*/
nfp_delFileCB ( fs_Handle	*nfp_fhandlep     /*filehandle*/
                )
{
	nfp_fileCBPtr_t	fileCBp;
	nfp_fileCB_t	search_fileCB;

	if (nfp_fhandlep == NULL)
		return (NFX_ERR);

	memcpy(&search_fileCB.nfp_fileHandle, nfp_fhandlep, 
			sizeof(fs_Handle));

	fileCBp = (nfp_fileCBPtr_t)hash_delete(nfp_globalCfgp->fileCbHash, 
						(void *)nfp_fhandlep,  
						(void *)&search_fileCB);

	if (fileCBp == NULL)
	{
		return (NFX_ERR);
	}


	if (nfp_deallocFileCB (fileCBp) != NFX_OK)
		return (NFX_ERR); 
#ifdef	NFP_DEBUG
	NFP_DBG_PRINTF("nfp_delFileCB: fileCBp %p volId: %lld inum: %lld\n", fileCBp,
				nfp_fhandlep->volId.whole_thing, nfp_fhandlep->inum);
#endif

	return (NFX_OK);
		
}

/*-----------------------------------------------------------------
 * Name:        nfp_getFileCB
 *
 * Description: Get a File Control Block (FCB).
 *
 * Created by:  Joseph Vu
 *
 * Date Created:        5/18/01
 *
 *-----------------------------------------------------------------
 */
nfp_fileCBPtr_t		
nfp_getFileCB ( fs_Handle	*nfp_fhandlep    /*filehandle*/
)
{
	nfp_fileCBPtr_t	currp;	
	nfp_fileCB_t	fileCB;
	
	if (nfp_fhandlep == NULL)
		return (NULL);

	memcpy(&fileCB.nfp_fileHandle, nfp_fhandlep, sizeof(fs_Handle));
	currp = (nfp_fileCBPtr_t )hash_lookup(nfp_globalCfgp->fileCbHash,
					(void *)nfp_fhandlep, (void *)&fileCB);

	return (currp);	
}

/*-----------------------------------------------------------------
 * Name:       	nfp_validateFileCB 
 *
 * Description: Validate File Control Block (FCB) buffer address.
 *
 * Created by:  Joseph Vu
 *
 * Date Created:        5/18/01
 *
 *-----------------------------------------------------------------
 */
int32
nfp_validateFileCB(nfp_fileCBPtr_t fileCBp)
{
	nfp_fileCBPtr_t		lastFileCBp;

	lastFileCBp = nfp_globalCfgp->fileCbCfgList.listBasep;
	lastFileCBp += (NFP_MAX_FILE -1);
	if ((fileCBp < (nfp_fileCBPtr_t)nfp_globalCfgp->fileCbCfgList.listBasep) ||
		(fileCBp > lastFileCBp) )
	{
		nfp_globalCfgp->debugStat.numOfBadFileCB++;
		return (NFX_ERR);
	}
	return (NFX_OK);

}
/*-----------------------------------------------------------------
 * Name:        nfp_allocFileSysCB
 *
 * Description: Allocate a File System Control Block (FSCB) from the available
 * buffer list.
 *
 * Created by:  Joseph Vu
 *
 * Date Created:        5/18/01
 *
 *-----------------------------------------------------------------
 */

nfp_fileSysCBPtr_t      /*Return pointer to fileSysCB entry or NULL if
                         *entry cannot be allocated
                         */
nfp_allocFileSysCB (void)
{
	nfp_fileSysCBPtr_t      availEntryp;
	int32                   index;
	int32                   count;

	/*
	 *Check for any free entries
	 */
	if (nfp_globalCfgp->fileSysCbCfgList.freeEntriesCnt <= 0)
	{
		//printf("nfp_allocFileSysCB: no free File System Control Block!\n");
		return (NULL);
	}
	/*
	 *Get current index
	 */
	availEntryp = (nfp_fileSysCBPtr_t)nfp_globalCfgp->fileSysCbCfgList.listBasep;
	index = nfp_globalCfgp->fileSysCbCfgList.listIndex;
	availEntryp += index;
	for (count = 0; ((availEntryp != NULL) && (count <  NFP_MAX_FILESYS)); count++)
	{
		if (!(availEntryp->nfp_flag & NFP_ENTRY_INUSED))
			break;
		if (index == (NFP_MAX_FILESYS - 1))
		{
			/*
			 *We 've reach end of the list, go back to the top
			 *of the list
			 */
			availEntryp = (nfp_fileSysCBPtr_t)nfp_globalCfgp->fileSysCbCfgList.listBasep;
			index = 0;
			continue;
		}
		index++;
		availEntryp++;
	}
	if (index == (NFP_MAX_FILESYS-1))
		nfp_globalCfgp->fileSysCbCfgList.listIndex = 0;
	else
		nfp_globalCfgp->fileSysCbCfgList.listIndex = index + 1;
#ifdef	NFP_DEBUG
	NFP_DBG_PRINTF("nfp_allocFileSysCB: availEntryp: %p avail index: %d\n",
					availEntryp, index);
	NFP_DBG_PRINTF("nfp_allocFileSysCB: next index: %d \n",
					nfp_globalCfgp->fileSysCbCfgList.listIndex);
#endif

	nfp_globalCfgp->debugStat.numOfAllocFileSysCB++;
	nfp_globalCfgp->fileSysCbCfgList.freeEntriesCnt -= 1;
	memset(availEntryp, 0, sizeof(nfp_fileSysCB_t) );
	availEntryp->nfp_flag |= NFP_ENTRY_INUSED;
	if (nfp_globalCfgp->fileSysCbCfgList.freeEntriesCnt <
		nfp_globalCfgp->debugStat.fileSysCbLwm)
		nfp_globalCfgp->debugStat.fileSysCbLwm =
				nfp_globalCfgp->fileSysCbCfgList.freeEntriesCnt;

	return (availEntryp);
}

/*-----------------------------------------------------------------
 * Name:        nfp_deallocFileSysCB
 *
 * Description: De-allocate a File System Control Block (FSCB) and return entry
 * back to the  available buffer list.
 *
 * Created by:  Joseph Vu
 *
 * Date Created:        5/18/01
 *
 *-----------------------------------------------------------------
 */
int32           /*NFX_OK:  Entry has been de-allocate*/
                /*NFX_ERR:  Failure to de-allocate entry*/
nfp_deallocFileSysCB (
        nfp_fileSysCBPtr_t deallocEntryp   /*Entry to de-allocate*/
)
{
	
	if (deallocEntryp == NULL)
		return (NFX_ERR);
	nfp_globalCfgp->debugStat.numOfFreeFileSysCB++;
	//deallocEntryp->nfp_flag &= ~NFP_ENTRY_INUSED;
	deallocEntryp->nfp_flag = 0;
	memset(&deallocEntryp->nfp_fileHandle, 0, sizeof(deallocEntryp->nfp_fileHandle));
	nfp_globalCfgp->fileSysCbCfgList.freeEntriesCnt += 1;
	return (NFX_OK);
}

/*-----------------------------------------------------------------
 * Name:        nfp_addFileSysCB
 *
 * Description: Add a File System Control Block (FSCB) entry into
 * the FCSB list.
 *
 * Created by:  Joseph Vu
 *
 * Date Created:        5/18/01
 *
 *-----------------------------------------------------------------
 */
int32   /*NFX_OK: Entry has been added*/
		/*NFX_ERR:  Failure to add entry*/
nfp_addFileSysCB ( 
	nfp_fileSysCBPtr_t 	nfp_addEntryp, 	/*entry to be added*/
	fs_Handle		*nfp_fhandlep	/*filehandle*/

)
{
	nfp_fileSysCBPtr_t	fileSysCBp;

	if ((nfp_addEntryp == NULL) || (nfp_fhandlep == NULL))
		return (NFX_ERR);
	/*
	 *Check for duplicate entry
	 */
	fileSysCBp = nfp_getFileSysCB(nfp_fhandlep);
	if (nfp_fhandlep->volId.whole_thing != 0 && fileSysCBp != NULL)
	{

		return (NFX_ERR);	
	}
	memcpy (&(nfp_addEntryp->nfp_fileHandle), nfp_fhandlep, sizeof(fs_Handle));

	return (NFX_OK);
}

/*-----------------------------------------------------------------
 * Name:        nfp_delFileSysCB
 *
 * Description: Delete a File System Control Block (FSCB) entry from 
 * the FCSB list.
 *
 * Created by:  Joseph Vu
 *
 * Date Created:        5/18/01
 *
 *-----------------------------------------------------------------
 */
int32	
nfp_delFileSysCB (nfp_fileSysCBPtr_t  nfp_delEntryp	 /*entry to be delete*/
)
{
	nfp_fileSysCBPtr_t	fileSysCBp;

	if (nfp_delEntryp == NULL)
		return (NFX_ERR);
	/*
	 *Check if delete entry exist
	 */
	fileSysCBp = nfp_getFileSysCB(&nfp_delEntryp->nfp_fileHandle);	
	if (nfp_delEntryp->nfp_fileHandle.volId.whole_thing != 0 && fileSysCBp == NULL)
		return (NFX_ERR);
	/*
	 *For now just call de-allocate entry
	 */
	return (nfp_deallocFileSysCB(nfp_delEntryp));

}
/*-----------------------------------------------------------------
 * Name:       	nfp_getFileSysCB 
 *
 * Description: Retrieve a File System Control Block (FSCB) entry 
 * by the input filehandle.
 *
 * Created by:  Joseph Vu
 *
 * Date Created:        5/18/01
 *
 *-----------------------------------------------------------------
 */
nfp_fileSysCBPtr_t 	/*
					 *Return pointer to FSCB entry of NULL if entry
					 *not found
					 */
nfp_getFileSysCB ( fs_Handle	*nfp_fhandle        /*filehandle*/
)
{
	nfp_fileSysCBPtr_t  currp;
	uint32              count;
	uint64              searchId;
	uint64				searchInum;


	count = 0;
	searchId = nfp_fhandle->volId.whole_thing;
	searchInum = nfp_fhandle->inum; 
	for (currp = (nfp_fileSysCBPtr_t)nfp_globalCfgp->fileSysCbCfgList.listBasep;
		((count < NFP_MAX_FILESYS) && (currp != NULL)); count++, currp++)
	{
		if (!(currp->nfp_flag & NFP_ENTRY_INUSED))
			continue;
		if ( (currp->nfp_fileHandle.volId.whole_thing == searchId) &&
//			(currp->nfp_fileHandle.inum == searchInum) &&
			(currp->nfp_fileHandle.mntPnt == nfp_fhandle->mntPnt) )
			return (currp);
	}
	return (NULL);

}
/*-----------------------------------------------------------------
 * Name:       	nfp_getFileSysByName 
 *
 * Description:
 *      Retrieves a File System Control Block from the buffer list based
 *	on the input directory pathname.
 *
 * Created by:  Joseph Vu
 *
 * Date Created:        5/18/01
 *
 *-----------------------------------------------------------------
 */
nfp_fileSysCBPtr_t      /*
                         *Return pointer to FSCB entry of NULL if entry
                         *not found
                         */
nfp_getFileSysByName (
			uchar8	*pathNamep,	/*pathname of the directory*/
			uint32 pathLen		/*pathname Len*/
			
)
{
	int32                   count;
	nfp_fileSysCBPtr_t      currp;

	if (pathNamep == NULL)
		return (NULL);
	count = 0;
	for (currp = (nfp_fileSysCBPtr_t)nfp_globalCfgp->fileSysCbCfgList.listBasep;
		((count < NFP_MAX_FILESYS) && (currp != NULL)); count++, currp++)
	{
		if (!(currp->nfp_flag & NFP_ENTRY_INUSED))
			continue;
		if ( (pathLen == currp->nfp_nameLen) &&
			(!(strncmp(currp->nfp_name, pathNamep, currp->nfp_nameLen))) )
			return (currp);
	}
	return (NULL);
	
}

/*-----------------------------------------------------------------
 * Name:       	nfp_validateFileSysCB 
 *
 * Description:
 *      Validates a File System Control Block buffer address.
 *
 * Created by:  Joseph Vu
 *
 * Date Created:        5/18/01
 *
 *-----------------------------------------------------------------
 */
int32 
nfp_validateFileSysCB(nfp_fileSysCBPtr_t fileSysCBp)
{
	nfp_fileSysCBPtr_t		lastFileSysCBp;
    void                    *ra;

    ra = (void *)get_ra();

	lastFileSysCBp = nfp_globalCfgp->fileSysCbCfgList.listBasep;
	lastFileSysCBp += (NFP_MAX_FILESYS -1);
	if ((fileSysCBp < (nfp_fileSysCBPtr_t)nfp_globalCfgp->fileSysCbCfgList.listBasep) ||
		(fileSysCBp > lastFileSysCBp) )
	{
		nfp_globalCfgp->debugStat.numOfBadFileSysCB++;
		return (NFX_ERR);
	}
	return (NFX_OK);
}

/*-----------------------------------------------------------------
 * Name:        nfp_allocReqCB
 *
 * Description: Allocate a Request Control Block (RCB) from the available
 * buffer list.
 *
 * Created by:  Joseph Vu
 *
 * Date Created:        5/18/01
 *
 *-----------------------------------------------------------------
 */
nfp_requestCBPtr_t	/*Return pointer to reqCB entry or NULL 
					 *if entry cannot be allocated
					 */
nfp_allocReqCB (void)
{
	nfp_requestCBPtr_t      availEntryp;
	int32                   index;
	int32                   count;

	/*
	 *Check for any free entries
	 */
	if (nfp_globalCfgp->reqCbCfgList.freeEntriesCnt <= 0)
	{
		//printf("nfp_allocReqCB: no free Request Control Block!\n");
		return (NULL);
	}
	/*
	 *Get current index
	 */
	availEntryp = (nfp_requestCBPtr_t)nfp_globalCfgp->reqCbCfgList.listBasep;
	index = nfp_globalCfgp->reqCbCfgList.listIndex;
	availEntryp += index;
	for (count = 0; ((availEntryp != NULL) && (count <  NFP_MAX_REQUEST)); count++)
	{
		if (!(availEntryp->nfp_flag & NFP_ENTRY_INUSED))
			break;
		if (index == (NFP_MAX_REQUEST - 1))
		{
			/*
			 *We 've reach end of the list, go back to the top
			 *of the list
			 */
			availEntryp = (nfp_requestCBPtr_t)nfp_globalCfgp->reqCbCfgList.listBasep;
			index = 0;
			continue;
		}
		index++;
		availEntryp++;
	}
	if (index == (NFP_MAX_REQUEST-1))
		nfp_globalCfgp->reqCbCfgList.listIndex = 0;
	else
		nfp_globalCfgp->reqCbCfgList.listIndex = index + 1;
#ifdef	NFP_DEBUG
	NFP_DBG_PRINTF("nfp_allocReqCB: availEntryp: %p avail index: %d\n",
						availEntryp, index);
	NFP_DBG_PRINTF("nfp_allocReqCB: next index: %d \n",
						nfp_globalCfgp->reqCbCfgList.listIndex);
#endif

	nfp_globalCfgp->debugStat.numOfAllocReqCB++;
	nfp_globalCfgp->reqCbCfgList.freeEntriesCnt -= 1;
	memset(availEntryp, 0, sizeof(nfp_requestCB_t) );
	availEntryp->nfp_flag |= NFP_ENTRY_INUSED;
    vs_mem_alloc(0, vs_mem_dt_nlm_request_t, &availEntryp->protocol.vs_nlm_idx);
	if (nfp_globalCfgp->reqCbCfgList.freeEntriesCnt <
		nfp_globalCfgp->debugStat.reqCbLwm)
		nfp_globalCfgp->debugStat.reqCbLwm = nfp_globalCfgp->reqCbCfgList.freeEntriesCnt; 
	return (availEntryp);

}
/*-----------------------------------------------------------------
 * Name:       	nfp_deallocReqCB 
 *
 * Description: De-allocate a Request Control Block (RCB) 
 * and returns a RCB 
 * back to the available buffer list.
 *
 * Created by:  Joseph Vu
 *
 * Date Created:        5/18/01
 *
 *-----------------------------------------------------------------
 */
int32           /*NFX_OK:  Entry has been de-allocate*/
                /*NFX_ERR: Failure to de-allocate entry*/
nfp_deallocReqCB( nfp_requestCBPtr_t	deallocEntryp /*Pointer to ClientCB to de-allocate*/
)
{

	if (deallocEntryp == NULL)
		return (NFX_ERR);
	nfp_globalCfgp->debugStat.numOfFreeReqCB++;
	deallocEntryp->nfp_flag &= ~NFP_ENTRY_INUSED;
	nfp_globalCfgp->reqCbCfgList.freeEntriesCnt += 1;
	return (NFX_OK);   
}

/*-----------------------------------------------------------------
 * Name:        nfp_allocReqCB
 *
 * Description: Allocate a Reply Control Block (RCB) 
 * from the available  buffer list.
 *
 * Created by:  Joseph Vu
 *
 * Date Created:        5/18/01
 *
 *-----------------------------------------------------------------
 */
nfp_replyCBPtr_t
nfp_allocReplyCB (void)
{
	nfp_replyCBPtr_t		availEntryp;
	int32                   index;
	int32                   count;

	/*
	 *Check for any free entries
	 */
	if (nfp_globalCfgp->replyCbCfgList.freeEntriesCnt <= 0)
	{
		//printf("nfp_allocReplyCB: no free Reply Control Block!\n");
		return (NULL);
	}
	/*
	 *Get current index
	 */
	availEntryp = (nfp_replyCBPtr_t)nfp_globalCfgp->replyCbCfgList.listBasep;
	index = nfp_globalCfgp->replyCbCfgList.listIndex;
	availEntryp += index;
	for (count = 0; 
		((availEntryp != NULL) && (count <  NFP_MAX_REPLY)); count++)
	{
		if (!(availEntryp->nfp_flag & NFP_ENTRY_INUSED))
			break;
		if (index == (NFP_MAX_REPLY - 1))
		{
			/*
			 *We 've reach end of the list, go back to the top
			 *of the list
			 */
			availEntryp = (nfp_replyCBPtr_t)nfp_globalCfgp->replyCbCfgList.listBasep;
			index = 0;
			continue;
		}
		index++;
		availEntryp++;
	}
	if (index == (NFP_MAX_REPLY-1))
		nfp_globalCfgp->replyCbCfgList.listIndex = 0;
	else
		nfp_globalCfgp->replyCbCfgList.listIndex = index + 1;
#ifdef	NFP_DEBUG
	NFP_DBG_PRINTF("nfp_allocReplyCB: availEntryp: %p avail index: %d\n",
						availEntryp, index);
	NFP_DBG_PRINTF("nfp_allocReplyCB: next index: %d \n",
						nfp_globalCfgp->replyCbCfgList.listIndex);
#endif

	nfp_globalCfgp->debugStat.numOfAllocReplyCB++;
	nfp_globalCfgp->replyCbCfgList.freeEntriesCnt -= 1;
	memset(availEntryp, 0, sizeof(nfp_replyCB_t) );
	availEntryp->nfp_flag |= NFP_ENTRY_INUSED;
	if (nfp_globalCfgp->replyCbCfgList.freeEntriesCnt <
		nfp_globalCfgp->debugStat.replyCbLwm)
		nfp_globalCfgp->debugStat.replyCbLwm = nfp_globalCfgp->replyCbCfgList.freeEntriesCnt;

	return (availEntryp);
}

/*-----------------------------------------------------------------
 * Name:       	nfp_deallocReplyCB 
 *
 * Description: De-allocate a Reply Control Block (RCB) 
 * and returns a RCB 
 * back to the available buffer list.
 *
 * Created by:  Joseph Vu
 *
 * Date Created:        5/18/01
 *
 *-----------------------------------------------------------------
 */
int32
nfp_deallocReplyCB(nfp_replyCBPtr_t deallocEntryp)
{
	if (deallocEntryp == NULL)
		return (NFX_ERR);
	nfp_globalCfgp->debugStat.numOfFreeReplyCB++;
	deallocEntryp->nfp_flag &= ~NFP_ENTRY_INUSED;
	nfp_globalCfgp->replyCbCfgList.freeEntriesCnt += 1;
	return (NFX_OK);
}

/*-----------------------------------------------------------------
 * Name:       	nfp_validateReplyCB 
 *
 * Description: Validates a Reply Control Block (RCB) buffer address.
 *
 * Created by:  Joseph Vu
 *
 * Date Created:        5/18/01
 *
 *-----------------------------------------------------------------
 */
int32
nfp_validateReplyCB (nfp_replyCBPtr_t replyCBp)
{
	nfp_replyCBPtr_t		lastReplyCBp;

	lastReplyCBp = nfp_globalCfgp->replyCbCfgList.listBasep;
	lastReplyCBp += (NFP_MAX_REPLY -1);
	if ((replyCBp < 
		(nfp_replyCBPtr_t)nfp_globalCfgp->replyCbCfgList.listBasep) ||
		(replyCBp > lastReplyCBp) )
	{
		nfp_globalCfgp->debugStat.numOfBadReplyCB++;
		return (NFX_ERR);
	}
	return (NFX_OK);
}


/*-----------------------------------------------------------------
 * Name:       	nfp_validateReqCB 
 *
 * Description: Validates a Request Control Block (RCB) buffer address. 
 *
 * Created by:  Joseph Vu
 *
 * Date Created:        5/18/01
 *
 *-----------------------------------------------------------------
 */
int32
nfp_validateReqCB(nfp_requestCBPtr_t reqCBp)
{
	nfp_requestCBPtr_t		lastReqCBp;

	lastReqCBp = nfp_globalCfgp->reqCbCfgList.listBasep;
	lastReqCBp += (NFP_MAX_REQUEST - 1);
	if ((reqCBp < (nfp_requestCBPtr_t)nfp_globalCfgp->reqCbCfgList.listBasep) ||
		(reqCBp > lastReqCBp) )
	{
		nfp_globalCfgp->debugStat.numOfBadReqCB++;
		return (NFX_ERR);
	}
	if (!(reqCBp->nfp_flag & NFP_ENTRY_INUSED))
	{
//		nfp_globalCfgp->debugStat.numOfBadReqCB++;
		return (NFX_ERR);
	}

	return (NFX_OK);
}

/*-----------------------------------------------------------------
 * Name:        nfp_validateNfsReqCB
 *
 * Description: Validates the NFS Request Control Block (RCB) buffer.
 *
 * Created by:  Joseph Vu
 *
 * Date Created:        5/18/01
 *
 *-----------------------------------------------------------------
 */
int32
nfp_validateNfsReqCB(nfs_request_t *nfsReqp)
{
	nfp_requestCBPtr_t      lastReqCBp;
	nfp_requestCBPtr_t      firstReqCBp;
	nfs_request_t           *firstNfsReqp;
	nfs_request_t           *lastNfsReqp;
	nfp_requestCBPtr_t 		reqCBp;
	void            		*ra;

	ra = (void *)get_ra();
	firstReqCBp = (nfp_requestCBPtr_t)nfp_globalCfgp->reqCbCfgList.listBasep;
	lastReqCBp = (nfp_requestCBPtr_t)nfp_globalCfgp->reqCbCfgList.listBasep;
	lastReqCBp += (NFP_MAX_REQUEST - 1);
	firstNfsReqp = &firstReqCBp->protocol.nfs_req;
	lastNfsReqp = &lastReqCBp->protocol.nfs_req;

	if ((nfsReqp < firstNfsReqp) ||(nfsReqp > lastNfsReqp) )
	{
		nfp_globalCfgp->debugStat.numOfBadReqCB++;
		return (NFX_ERR);
	}

	reqCBp = (nfp_requestCBPtr_t)nfsReqp->reqCBp;
	if (nfp_validateReqCB(reqCBp) != NFX_OK)
		return (NFX_ERR);
	return (NFX_OK);
}

/*-----------------------------------------------------------------
 * Name:        nfp_validateMntReqCB
 *
 * Description: Validates the MNT Request Control Block (RCB) buffer.
 *
 * Created by:  Joseph Vu
 *
 * Date Created:        5/18/01
 *
 *-----------------------------------------------------------------
 */
int32
nfp_validateMntReqCB(mnt_request_t *mntReqp)
{
	nfp_requestCBPtr_t      lastReqCBp;
	nfp_requestCBPtr_t      firstReqCBp;
	mnt_request_t           *firstMntReqp;
	mnt_request_t           *lastMntReqp;

	firstReqCBp = (nfp_requestCBPtr_t)nfp_globalCfgp->reqCbCfgList.listBasep;
	lastReqCBp = nfp_globalCfgp->reqCbCfgList.listBasep;
	lastReqCBp += (NFP_MAX_REQUEST - 1);
	firstMntReqp = &firstReqCBp->protocol.mnt_req;
	lastMntReqp = &lastReqCBp->protocol.mnt_req;

	if ((mntReqp < firstMntReqp) ||(mntReqp > lastMntReqp) )
		return (NFX_ERR);

	return (NFX_OK);
}

/*-----------------------------------------------------------------
 * Name:        nfp_validateNlmReqCB
 *
 * Description: Validates the NLM Request Control Block (RCB) buffer.
 *
 * Created by:  Xiaobing Zhang
 *
 * Date Created:        7/15/02
 *
 *-----------------------------------------------------------------
 */
int32
nfp_validateNlmReqCB(nlm_request_t *nlmReqp)
{
#ifdef NOT_USED
	nfp_requestCBPtr_t      lastReqCBp;
	nfp_requestCBPtr_t      firstReqCBp;
	nlm_request_t           *firstNlmReqp;
	nlm_request_t           *lastNlmReqp;

	firstReqCBp = (nfp_requestCBPtr_t)nfp_globalCfgp->reqCbCfgList.listBasep;
	lastReqCBp = nfp_globalCfgp->reqCbCfgList.listBasep;
	lastReqCBp += (NFP_MAX_REQUEST - 1);
	firstNlmReqp = &firstReqCBp->protocol.nlm_req;
	lastNlmReqp = &lastReqCBp->protocol.nlm_req;

	if ((nlmReqp < firstNlmReqp) ||(nlmReqp > lastNlmReqp) )
		return (NFX_ERR);
#endif

	return (NFX_OK);
}

/*-----------------------------------------------------------------
 * Name:        nfp_allocLockCB
 *
 * Description: Allocate a Lock Control Block (LCB) from the available
 * buffer list.
 *
 * Created by:  Xiaobing Zhang
 *
 * Date Created:        7/15/02
 *
 *-----------------------------------------------------------------
 */
nfp_lockCBPtr_t
nfp_allocLockCB(void)
{
	nfp_lockCBPtr_t	availEntryp;
	int32			index;
	int32			count;

	/*
	 *Check for any free entries
	 */
	if (nfp_globalCfgp->lockCbCfgList.freeEntriesCnt <= 0)
	{
		return (NULL);
	}

	/*
	 *Get current index
	 */
	availEntryp = (nfp_lockCBPtr_t)nfp_globalCfgp->lockCbCfgList.listBasep;
	index = nfp_globalCfgp->lockCbCfgList.listIndex;
	availEntryp += index;
	for (count = 0; 
		((availEntryp != NULL) && (count <  NFP_MAX_LOCK)); count++)
	{
		if (!(availEntryp->nfp_flag & NFP_ENTRY_INUSED))
			break;
		if (index == (NFP_MAX_LOCK - 1))
		{
			/*
			 *We 've reach end of the list, go back to the top
			 *of the list
			 */
			availEntryp = (nfp_lockCBPtr_t)nfp_globalCfgp->lockCbCfgList.listBasep;
			index = 0;
			continue;
		}
		index++;
		availEntryp++;
	}
	/*
	 *Set index for next available entry
	*/
	if (index == (NFP_MAX_LOCK - 1))
		nfp_globalCfgp->lockCbCfgList.listIndex = 0;
	else
		nfp_globalCfgp->lockCbCfgList.listIndex = index + 1;
#ifdef	NFP_DEBUG
	NFP_DBG_PRINTF("nfp_allocLockCB: availEntryp: %p avail index: %d\n",
						availEntryp, index);
	NFP_DBG_PRINTF("nfp_allocLockCB: next index: %d \n",
						nfp_globalCfgp->replyCbCfgList.listIndex);
#endif

	nfp_globalCfgp->debugStat.numOfAllocLockCB++;
	nfp_globalCfgp->lockCbCfgList.freeEntriesCnt -= 1;
//	memset(availEntryp, 0, sizeof(nfp_lockCB_t) );
	availEntryp->nfp_flag |= NFP_ENTRY_INUSED;
	if (nfp_globalCfgp->lockCbCfgList.freeEntriesCnt <
		nfp_globalCfgp->debugStat.lockCbLwm)
		nfp_globalCfgp->debugStat.lockCbLwm = nfp_globalCfgp->lockCbCfgList.freeEntriesCnt;

#ifdef NOT_USED
	availEntryp->lockOwnerList.numOfEntries = 0;
	availEntryp->lockOwnerPendingList.numOfEntries = 0;
#endif

	return (availEntryp);
}

/*-----------------------------------------------------------------
 * Name:        nfp_deallocLockCB
 *
 * Description: de-allocate a Lock Control Block (LCB) and return entry
 * back to available buffer list.
 *
 * Created by:  Xiaobing Zhang
 *
 * Date Created:        7/15/02
 *
 *-----------------------------------------------------------------
 */
int32           /*NFX_OK:  Entry has been de-allocate*/
                /*NFX_ERR: Failure to de-allocate entry*/
nfp_deallocLockCB(nfp_lockCBPtr_t	deallocEntryp	/*de-allocate entry*/)
{

	if (deallocEntryp == NULL)
		return (NFX_ERR);
	nfp_globalCfgp->debugStat.numOfFreeLockCB++;
	deallocEntryp->nfp_flag &= ~NFP_ENTRY_INUSED;
	nfp_globalCfgp->lockCbCfgList.freeEntriesCnt += 1;
	return (NFX_OK);   
}

/*-----------------------------------------------------------------
 * Name:       	nfp_validateLockCB 
 *
 * Description: Validate Lock Control Block (LCB) buffer address.
 *
 * Created by:  Xiaobing Zhang
 *
 * Date Created:        7/15/02
 *
 *-----------------------------------------------------------------
 */
int32
nfp_validateLockCB(nfp_lockCBPtr_t lockCBp)
{
	nfp_lockCBPtr_t		lastLockCBp;

	lastLockCBp = nfp_globalCfgp->lockCbCfgList.listBasep;
	lastLockCBp += (NFP_MAX_LOCK - 1);
	if ((lockCBp < (nfp_lockCBPtr_t)nfp_globalCfgp->lockCbCfgList.listBasep) ||
		(lockCBp > lastLockCBp) ||
	   (!(lockCBp->nfp_flag & NFP_ENTRY_INUSED)) )
	{
		nfp_globalCfgp->debugStat.numOfBadLockCB++;
		return (NFX_ERR);
	}
	return (NFX_OK);
}

/*-----------------------------------------------------------------
 * Name:   nfp_initLockCbDll    		
 *
 * Description:
 *		Initializes lock owner list and pending list for each
 * Lock Control Block  (LCB).
 *
 * Created by:  Joseph Vu 
 *
 * Date Created:        10/28/02
 *
 *-----------------------------------------------------------------
 */
void
nfp_initLockCbDll(void)
{
	nfp_lockCBPtr_t	lockCBp;
	int32			i;


	lockCBp = (nfp_lockCBPtr_t)nfp_globalCfgp->lockCbCfgList.listBasep;
	for (i = 0; (i < NFP_MAX_LOCK); i++)
	{
		bzero(lockCBp, sizeof(nfp_lockCB_t) );
#ifdef NOT_USED
		DLL_INIT(&lockCBp->lockOwnerList.list);
		DLL_INIT(&lockCBp->lockOwnerPendingList.list);
#endif
		lockCBp++;
	}


}

/*-----------------------------------------------------------------
 * Name:       	nfp_fileCbHashFn 
 *
 * Description:
 * Generic hashing function for hash table of the
 * File Control Blocks.
 * Created by:  Joseph Vu
 *
 * Date Created:        5/18/01
 *
 *-----------------------------------------------------------------
 */
uint32 nfp_fileCbHashFn(uint64 key)
{
	fs_Handle		*fhandlep;
	uint32			volId;
	uint32			inum;		
	uint32			hashVal;

	fhandlep = (fs_Handle    *) key;

	volId = (uint32)fhandlep->volId.whole_thing;
	inum = (uint32)fhandlep->inum;
	
	hashVal = ((((volId)>>24) ^ ((volId)>>16) ^ ((volId)>>8) ^(volId)) & 0x7f) |
				( (((inum)>>24) ^ ((inum)>>16) ^ ((inum )>>8) ^(inum)) & 0x7f);

	return (hashVal);


}
/*-----------------------------------------------------------------
 * Name:       	nfp_fileCbHashCompare	
 *
 * Description:
 *	Compare function for hash table of the 
 *	File Control Blocks.
 *
 * Created by:  Joseph Vu
 *
 * Date Created:        5/18/01
 *
 *-----------------------------------------------------------------
 */
int32
nfp_fileCbHashCompare(nfp_fileCBPtr_t fileCBp,	//data stored in hash
			nfp_fileCBPtr_t compare_fileCBp		//data to compare
)
{
	if  ( (fileCBp->nfp_fileHandle.volId.whole_thing ==
                compare_fileCBp->nfp_fileHandle.volId.whole_thing ) &&
            (fileCBp->nfp_fileHandle.inum == 
				compare_fileCBp->nfp_fileHandle.inum) &&
			(fileCBp->nfp_fileHandle.gen ==
			compare_fileCBp->nfp_fileHandle.gen) )
		//match
		return 0;
	else
		return 1;

}
/*-----------------------------------------------------------------
 * Name:       	nfp_fileCbHashRelease	
 *
 * Description:
 *	Release function for hash table of the 
 *	File Control Blocks.
 *
 * Created by:  Joseph Vu
 *
 * Date Created:        5/18/01
 *
 *-----------------------------------------------------------------
 */
void
nfp_fileCbHashRelease(nfp_fileCBPtr_t fileCBp)
{
	nfp_deallocFileCB (fileCBp);
}

/*-----------------------------------------------------------------
 * Name:      	nfp_initCb 
 * 
 * Description:
 *  NFP initializes control block type based on the input type .
 *
 * Created by:  Joseph Vu
 *
 * Date Created:        5/18/01
 *
 *-----------------------------------------------------------------
 */
int32   nfp_initCb(nfp_cfgList_t *nfp_cfgListp, uint32 listType)
{
	uint32      allocSize;
	uint32      freeEntries = 0;

	switch (listType)
	{
		case NFP_GENERIC_NODE:
			allocSize = sizeof (nfp_node_t) * NFP_MAX_NODE;
			freeEntries = NFP_MAX_NODE;
#ifdef	NFP_DEBUG
			NFP_DBG_PRINTF("nfp_initCb: node entry Size: %d Alloc Size: %d\n",
					sizeof(nfp_node_t), allocSize);
#endif
			break;
		case NFP_CLIENTCB_NODE:
			allocSize =  sizeof(nfp_clientCB_t) * NFP_MAX_CLIENT;
			freeEntries = NFP_MAX_CLIENT;
#ifdef	NFP_DEBUG
			NFP_DBG_PRINTF("nfp_initCb: clientCB entry Size: %d Alloc Size: %d\n",
				sizeof(nfp_clientCB_t), allocSize);
#endif
			break;
		case NFP_FILESYSCB_NODE:
			allocSize = sizeof(nfp_fileSysCB_t) * NFP_MAX_FILESYS;
			freeEntries = NFP_MAX_FILESYS;
#ifdef	NFP_DEBUG
			NFP_DBG_PRINTF("nfp_initCb: fileSys entry Size: %d Alloc Size: %d\n",
					sizeof(nfp_fileSysCB_t), allocSize);
#endif
			break;
		case NFP_FILECB_NODE:
			allocSize =  sizeof(nfp_fileCB_t) * NFP_MAX_FILE;
			freeEntries = NFP_MAX_FILE;
#ifdef	NFP_DEBUG
			NFP_DBG_PRINTF("nfp_initCb: fileCB entry Size: %d Alloc Size: %d\n",
					sizeof(nfp_fileCB_t), allocSize);
#endif
			break;
		case NFP_REQCB_NODE:
			allocSize  = (sizeof (nfp_requestCB_t) * NFP_MAX_REQUEST);
			freeEntries = NFP_MAX_REQUEST;
#ifdef	NFP_DEBUG
			NFP_DBG_PRINTF("nfp_initCb: reqCB entry Size: %d Alloc Size: %d\n",
						sizeof(nfp_requestCB_t), allocSize);
#endif
			break;
		case NFP_REPLY_NODE:
			allocSize  = (sizeof (nfp_replyCB_t) * NFP_MAX_REPLY);
			freeEntries = NFP_MAX_REPLY;
#ifdef	NFP_DEBUG
			printf("nfp_initCb: replyCB entry Size: %d Alloc Size: %d\n",
					sizeof (nfp_replyCB_t), allocSize);
#endif
			break; 
		case NFP_LOCKCB_NODE:
			allocSize  = (sizeof (nfp_lockCB_t) * NFP_MAX_LOCK);
			freeEntries = NFP_MAX_LOCK;
#ifdef	NFP_DEBUG
			printf("nfp_initCb: lockCB entry Size: %d Alloc Size: %d\n",
					sizeof (nfp_lockCB_t), allocSize);
#endif
			break; 


		default:
			return (NFX_ERR);
	}
	nfp_cfgListp->listBasep = (void *)NFP_SHMEM_ALLOC(allocSize);
	if (nfp_cfgListp->listBasep == NULL)
	{
		panic("nfp_initCb: Failure to alloc type: %x list\n",listType);
	}
	/*
	 *Initialize index & number of free entries
	 */
	memset(nfp_cfgListp->listBasep, 0, allocSize);
	nfp_cfgListp->listIndex = 0;
	nfp_cfgListp->freeEntriesCnt = freeEntries;

    return (NFX_OK);
}

/*-----------------------------------------------------------------
 * Name:       nfp_init 
 *
 * Description:
 *	NFP initialize function.
 *
 * Created by:  Joseph Vu
 *
 * Date Created:        5/18/01
 *
 *-----------------------------------------------------------------
 */
void nfp_init(void)	
{

	
	/*
	 *TODO: Initialize NFP on NFP RX CPU 
	 */
	/*
	if (nfxMyCPUId != NFX_NFP_RX)
		return;
	*/
	if ( !( (nfxMySlotId == 1) && (nfxMyCPUId == 0)) )
		return;
	/*
	 *-Initializes Client Control Block (clientCB) data
	 *-Initializes File System Control Block (fileSysCB) data
	 *-Initializes File Control Block (fileCB) data
	 *-Initializes Node List
	 *-Initializes Request Control Block (reqCB) data
	 *-Initializes Lock Control Block (lockCB) data
	 */
	/*
	 *TODO: Set nfxMyCPUId to RX CPU for now.  
	 */
	
	printf("**********nfp_init*******\n");	
	nfp_globalCfgp = (nfp_globalCfg_t *)NFP_SHMEM_ALLOC(sizeof(nfp_globalCfg_t));
	if (nfp_globalCfgp == NULL)
		printf("nfp_init: Failure to alloc memory for NFP global Config!\n");
	memset (nfp_globalCfgp, 0, sizeof(nfp_globalCfg_t));

	nfp_initCb(&nfp_globalCfgp->clientCbCfgList, NFP_CLIENTCB_NODE);
	nfp_initCb(&nfp_globalCfgp->fileCbCfgList, NFP_FILECB_NODE);
	nfp_initCb(&nfp_globalCfgp->nodeCbCfgList, NFP_GENERIC_NODE);
	nfp_initCb(&nfp_globalCfgp->fileSysCbCfgList, NFP_FILESYSCB_NODE);
	nfp_initCb(&nfp_globalCfgp->reqCbCfgList, NFP_REQCB_NODE);
	nfp_initCb(&nfp_globalCfgp->replyCbCfgList, NFP_REPLY_NODE);
	nfp_initCb(&nfp_globalCfgp->lockCbCfgList, NFP_LOCKCB_NODE);

	//Init file control block hash table
	//nfp_globalCfgp->fileCbHash
	nfp_globalCfgp->fileCbHash = hash_create (NFP_MAX_FILE,
							nfp_fileCbHashFn,
							nfp_fileCbHashCompare,
							nfp_fileCbHashRelease);
	if (nfp_globalCfgp->fileCbHash == NULL)
		panic ("nfp_init: failure to allocate File Control Block hash table!\n");


	nfp_globalCfgp->debugStat.clientCbLwm = NFP_MAX_CLIENT;
	nfp_globalCfgp->debugStat.fileSysCbLwm = NFP_MAX_FILESYS;
	nfp_globalCfgp->debugStat.fileCbLwm = NFP_MAX_FILE;
	nfp_globalCfgp->debugStat.reqCbLwm = NFP_MAX_REQUEST;
	nfp_globalCfgp->debugStat.nodeCbLwm = NFP_MAX_NODE;
	nfp_globalCfgp->debugStat.replyCbLwm = NFP_MAX_REPLY;
	nfp_globalCfgp->debugStat.lockCbLwm = NFP_MAX_LOCK;

	/*
	 *Init DLL for LockCB
	 */
#ifdef NOT_USED
	nfp_initLockCbDll();	
#endif
}

int32   /*NFX_OK or NFX_ERR*/
nfp_initFileData(void)
{
#ifdef  NOTUSED
    /*TODO
     *Initialize skip list info.
     */
    nfp_fileCBSkListp = &nfp_fileCBSkList;
    NFP_FILE_CB_SLIST_HDR =
        NFP_SHMEM_ALLOC(sizeof(nfp_fileCBNode_t) +
            NFP_FCB_SLIST_MAX_LEVEL*sizeof(nfp_fileCBNode_t *));
    if (NFP_FILE_CB_SLIST_HDR == NULL)
    {
        panic("nfp_initFileData:  Failure to alloc memmory!\n");
        exit(1);
    }
    for (i = 0; i <= NFP_FCB_SLIST_MAX_LEVEL; i++)
        NFP_FILE_CB_SLIST_HDR->forward[i] = NIL;
    NFP_FILE_CB_SLIST_LEVEL = 0;


    NFP_DBG_PRINTF("nfp_initFileData: listhdr:%08x listLevel:%d\n",
            NFP_FILE_CB_SLIST_HDR ,
            NFP_FILE_CB_SLIST_LEVEL);

#endif      /*NOTUSED*/
    return (NFX_OK);
}

