/* vi:ts=4
 *-----------------------------------------------------------------
 *
 * Name:        nfp-api.h
 *
 * RCS:  $Id: nfp-api.h,v 1.38 2002/11/06 03:35:43 joseph Exp $
 *
 * Copyright (C) 2001, Agile Storage Inc.
 *
 * Description: Network File Protocol (NFP) definitions
 *
 * Created by: 	Joseph Vu 
 *
 * Date Created:        5/18/01
 *
 *-----------------------------------------------------------------
 */
#ifndef _NFP_API_H
#define _NFP_API_H

#include <exports/vs-mem-api.h>

#ifdef	NOTUSED
#define		NFP_MAX_CLIENT	4096	/*Set to max 4096*/
#define		NFP_MAX_FILESYS	255	/*set to max 255*/	
#define		NFP_MAX_FILE	50000	/*Set to max 50,000*/ 
#define		NFP_MAX_NODE	100000	/*Set to max 100,000*/
#define		NFP_MAX_REQUEST	65536	
//#define		NFP_MAX_NAME	1024

#endif	/*NOTUSED*/

#ifdef	LINUX_TEST
#define		NFP_MAX_CLIENT	16		/*Set to max 4096*/
#define		NFP_MAX_FILESYS	32		/*set to max 255*/	
#define		NFP_MAX_FILE	5000	/*Set to max 50,000*/ 
#define		NFP_MAX_NODE	30000	/*Set to max 100,000*/
#define		NFP_MAX_REQUEST	4096	/*set to 32768*/
#define		NFP_MAX_REPLY	4096	
#define		NFP_MAX_NAME	1024
#else
#define		NFP_MAX_CLIENT	512		
#define		NFP_MAX_FILESYS	255		
#define		NFP_MAX_FILE	2048	/* make sure power of 2 */
#define		NFP_MAX_NODE	65536	
#define		NFP_MAX_REQUEST	10000	
#define		NFP_MAX_REPLY	8192	
#define		NFP_MAX_LOCK	(1024 * 4)	
#define		NFP_MAX_NAME	1024

#endif

typedef struct nfp_node
{
        uint32                  nfp_flag;
#define NFP_ENTRY_INUSED        0x00000001	/*Entry is in-used*/
	void			*nfp_entryCBp;	/*Pointer to Control Block entry*/
	uint32			nfp_nodeType;	/*Node Type*/
#define	NFP_GENERIC_NODE	0x00000000
#define	NFP_CLIENTCB_NODE	0x00000001	/*ClientCB entry*/
#define	NFP_FILESYSCB_NODE	0x00000002	/*FileSysCB entry*/
#define	NFP_FILECB_NODE		0x00000004	/*FileCB entry*/
#define	NFP_REQCB_NODE		0x00000008	/*RequestCB entry*/
#define	NFP_DIRCB_NODE		0x00000010	/*Directory entry*/
#define	NFP_TPL_INFO_NODE	0x00000020
#define	NFP_REPLY_NODE		0x00000040	/*ReplyCB entry*/
#define	NFP_LOCKCB_NODE		0x00000080	/*LockCB entry*/
#define	NFP_NLMREQCB_NODE	0x00000100	/*NLM RequestCB entry*/
	struct nfp_node		*nfp_prevp;	/*Pointer to previous entry*/
	struct nfp_node		*nfp_nextp;	/*Pointer to next entry*/
} nfp_node_t, *nfp_nodePtr_t;

typedef struct nfp_list
{
	nfp_nodePtr_t           nfp_headp;              /*list head pointer*/
	nfp_nodePtr_t           nfp_tailp;              /*list tail pointer*/
	uint32                  nfp_reqCnt;             /*list entries count*/
} nfp_list_t;

typedef struct nfp_requestCB 
{
        uint32                  nfp_flag;
#define NFP_ENTRY_INUSED        0x00000001              /*Entry is in-used*/
	union
	{
		nfs_request_t	nfs_req;		/*NFS requests */
		mnt_request_t	mnt_req;		/*Mount requests*/
#ifdef NOT_USED
		nlm_request_t	nlm_req;		/*NLM requests*/
#else
        vs_mem_index_t  vs_nlm_idx;
#endif
	} protocol;
	struct nfp_requestCB	*nfp_prevp;		/*Pointer to previous entry*/
	struct nfp_requestCB	*nfp_nextp;		/*Pointer to next entry*/ 
} nfp_requestCB_t, *nfp_requestCBPtr_t;

typedef struct nfp_replyCB 
{
	uint32                  nfp_flag;
#define NFP_ENTRY_INUSED	0x00000001		/*Entry is in-used*/
	union
	{
		nfs_reply_t			nfs_reply;		/*NFS reply */
		nlm_reply_t			nlm_reply;		/*NLM reply */
	} protocol;
	struct nfp_replyCB		*nfp_prevp;		/*Pointer to previous entry*/
	struct nfp_replyCB		*nfp_nextp;		/*Pointer to next entry*/ 
} nfp_replyCB_t, *nfp_replyCBPtr_t;


/*
 *File Control Block (FCB) maintains a list of current clients that are using
 *this file, the parent filesystem, any locks on the file and the file info 
 *associated with this file.  The FCB saved look up results received from
 *the File System (attributes, symbolic links and Access info), so it does 
 *not consult the File System on every NFS lookup type requests.
 */
typedef struct nfp_fileCB
{
	uint32			nfp_flag;
#define	NFP_ENTRY_INUSED	0x00000001		/*Entry is in-used*/
#define	NFP_ENTRY_ADDED 	0x00000002		/*Entry is has been added*/

	nfp_list_t		nfp_clientList;			/*clients list*/
	void			*nfp_fileSysp;			/*Parent filesystem*/
	nfp_list_t		nfp_dirCacheFileList;	//Directory cache file list used for
											//lookup
	nfp_list_t		nfp_dirFileList;		/*Directory file list*/
#ifdef NOT_USED
	nfp_list_t		nfp_lockList;			/*Lock list*/
#else
    vs_mem_index_t  fileLockIdx;            /* index to file in Byte-lock mod */
#endif
	fs_Handle		nfp_fileHandle; 		/*file handle*/
	void			*nfp_parentDirFileCBp;	/*parent directory file control block*/
	union
	{
		nfs_fileInfo_t	nfp_nfsFileInfo;	/*NFS file information*/
											/*TBD:  CIFS information*/
	} protocol;
	nlm_fileInfo_t		nfp_nlmFileInfo;	/*NLM specific information*/
	struct nfp_fileCB	*nfp_prevp;			/*Pointer to previous fileCB entry*/
	struct nfp_fileCB	*nfp_nextp;			/*Pointer to next fileCB entry*/
} nfp_fileCB_t, *nfp_fileCBPtr_t;

#ifdef	NOTUSED
/*
 *File Control Block (FCB) skip lists information used for lookup FCB based
 *on i-node number (part of the filehandle).  
 */
#define	NFP_FCB_SLIST_MAX_LEVEL	16	/*skip list max level*/
typedef struct nfp_fileCBNode
{
	fs_ExtInode	nfp_inode;	/*64-bit inode number key use for searching*/
	nfp_fileCB_t	*nfp_fileCBp;	/*File Control Block data*/
	struct nfp_fileCBNode	*forward[1];	/*skip list forward pointer*/
} nfp_fileCBNode_t;

typedef struct nfp_fileCBSkList
{
	nfp_fileCBNode_t	*nfp_hdr;		/*list header*/
	int32			nfp_currSlistLevel;	/*current level of the skip lists*/
} nfp_fileCBSkList_t, *nfp_fileCBSkListPtr_t;

#endif	/*NOTUSED*/

typedef struct nfsTplConn
{
	address_t		nfsConnId;
	uint32			nfsPrevPktCnt;
	uint32			nfsCurrPktCnt;
	uint32			delFlag;	
} nfsTplConn_t;

typedef struct nfp_nfsExport
{
	uint32			clientAddr;
	uint32			exportOptionMask;	
} nfp_nfsExport_t;

/*
 *File System Control Block (FSCB) maintain list of current clients that are using
 *the filesystem, the list of files that currently open and the information 
 *(attributes, filesystem information) associated with this filesystem.
 */
typedef struct nfp_fileSysCB
{
	uint32			nfp_flag;
#define	NFP_ENTRY_INUSED			0x00000001		/*Entry is in-used*/
#define	NFP_ENTRY_CACHE_FHANDLE		0x00000002
#define NFP_ENTRY_MARK_FOR_DEL      0x00000004      /*mark for delete*/
#define NFP_SHARE_ENABLED			0x00000008		/*filesystem is enabled*/
#define NFP_SHARE_DELETED			0x00000010		/*filesystem is deleted*/
#define	NFP_SHARE_INIT_STATE		0x00000020		/*share is in INIT state*/
	nfp_list_t		nfp_clientList;		/*clients list*/
	nfp_list_t		nfp_fileList;		/*files list*/
	nfp_list_t		nfp_tplInfoList;
	fs_Handle		nfp_fileHandle;		/*filehandle*/
	char			nfp_name[VOL_MAX_NAME];	/*Filesystem volume name*/		
	uint32			nfp_nameLen;		/*Name length*/
	union
	{
		nfs_fileSysInfo_t	nfp_nfsFileSysInfo;	/*NFS Info*/
								/*TBD:  CIFS info*/
	} protocol;
	int32			laddr;					
	uint32			atime;					/*access time*/
	uint32			defaultExpMask;			/*client default export mask*/
#define	NFP_NFS_MAX_EXP_MASK_ENTRIES	32	//client specific export mask
	nfp_nfsExport_t	clientSpecificExpMask[NFP_NFS_MAX_EXP_MASK_ENTRIES]; 	
	uint32			clientExpMaskIdx;		//File System CB Client's
											//Specific Export Mask
	int32			numOfClientSpecificMasks;
	struct nfp_fileSysCB	*nfp_nextp;		/*Pointer to next entry*/
	struct nfp_fileSysCB	*nfp_prevp;		/*Pointer to previous entry*/
} nfp_fileSysCB_t, *nfp_fileSysCBPtr_t;


/*
 *The Client Control Block (CCB) maintain all active file systems and open
 *files by a client
 */
typedef struct nfp_clientCB
{
	uint32			nfp_flag;
#define	NFP_ENTRY_INUSED		0x00000001		/*Entry is in-used*/
#define	NFP_ENTRY_DEL_PENDING	0x00000004		/*Entry is pending delete */
	nfp_list_t		nfp_fileSysList;		/*file Systems list*/
	nfp_list_t		nfp_fileList;			/*files list*/
	uint32			nfp_connCnt;           	/* # of connection in client */
	uint32			nfp_connId;           	/*TCP/UDP connection ID*/
	uint32			nfp_connId2;           	/*TCP/UDP connection ID 2*/
	uint32			nfp_ipAddr;				/*Client IP address*/
	uint32			nfp_localIpAddr;		/*local IP addres*/
	uchar8          *nfp_hostnamep;         /*Pointer to hostname*/
	uint32			hostNameLen;
	uint32			proto;					/*protocol field*/
#define	NFP_PROTO_NFS	1
#define	NFP_PROTO_MNT	2
#define	NFP_PROTO_CIFS	3
	union
	{
		nfs_clientInfo_t        nfp_nfsClientInfo;      /*NFS specific info*/
		mnt_clientInfo_t		nfp_mntClientInfo;		/*MOUNT specific info*/
														/*TBD: CIF specific info*/
	} protocol;
	nlm_clientInfo_t	nfp_nlmClientInfo;	/*NLM specific info*/
	nfp_list_t		nfp_reqList;		/*requests list*/
	nfp_list_t		nfp_replyList;		/*replies list*/
	nfp_list_t		nfp_nlmReqList;		/*NLM requests list*/
	nfp_list_t		nfp_nlmReplyList;	/*NLM replies list*/
	struct nfp_clientCB	*nfp_prevp;		/*Pointer to previous entry*/
	struct nfp_clientCB	*nfp_nextp;		/*Pointer to next clientCB*/
} nfp_clientCB_t, *nfp_clientCBPtr_t;

typedef struct nfp_lockCB
{
	uint32			nfp_flag;
#define	NFP_ENTRY_INUSED	0x00000001			/*Entry is in-used*/
#define NFP_ENTRY_ADD_TO_PENDING_LIST   0x00000002
	nfp_clientCBPtr_t	nfp_clientCbp;	
	nfp_fileCBPtr_t		nfp_fileCbp;
	uint32				proto;
	union
	{
		nlm_lockInfo_t	nfp_nlmLock; 			/*NLM specific info*/
		
	} protocol;
#ifdef NOT_USED
	nlm_lockRecOwnerList_t lockOwnerList;		/*active lock list*/
	nlm_lockRecOwnerList_t lockOwnerPendingList;/*Blocked  lock list*/
#endif

	struct nfp_lockCB	*nfp_prevp;		/*Pointer to previous lockCB entry*/
	struct nfp_lockCB	*nfp_nextp;		/*Pointer to next lockCB entry*/
} nfp_lockCB_t, *nfp_lockCBPtr_t;


typedef struct nfp_stat
{
	uint32	numOfAllocClientCB;
	uint32	numOfFreeClientCB;
	uint32	clientCbLwm;
	uint32	numOfAllocFileSysCB;
	uint32	numOfFreeFileSysCB;
	uint32	fileSysCbLwm;
	uint32	numOfAllocFileCB;
	uint32	numOfFreeFileCB;
	uint32	fileCbLwm;
	uint32	numOfAllocReqCB;
	uint32	numOfFreeReqCB;
	uint32	reqCbLwm;
	uint32	numOfAllocNode;
	uint32	numOfFreeNode;	
	uint32	nodeCbLwm;
	uint32	numOfAllocReplyCB;
	uint32	numOfFreeReplyCB;
	uint32	replyCbLwm;
	uint32	numOfAllocLockCB;
	uint32	numOfFreeLockCB;
	uint32	lockCbLwm;
	uint32	numOfBadClientCB;
	uint32	numOfBadFileSysCB;
	uint32	numOfBadFileCB;
	uint32	numOfBadReqCB;
	uint32	numOfBadNode;	
	uint32	numOfBadReplyCB;
	uint32	numOfBadLockCB;
} nfp_stat_t;

typedef struct nfp_cfgList
{
	void        *listBasep;
	uint32      listIndex;
	uint32      freeEntriesCnt;
} nfp_cfgList_t;

typedef struct nfp_globalCfg
{
	nfp_cfgList_t       nodeCbCfgList;
	nfp_cfgList_t       clientCbCfgList;
	nfp_cfgList_t       fileCbCfgList;
	nfp_cfgList_t       fileSysCbCfgList;
	nfp_cfgList_t       reqCbCfgList;
	nfp_cfgList_t		replyCbCfgList;
	nfp_cfgList_t		lockCbCfgList;
	nfp_stat_t			debugStat;	
	address_t			fileCbHash;
	void                *pFileCbHashTbl;	
} nfp_globalCfg_t;

extern nfp_globalCfg_t *nfp_globalCfgp;

#ifdef	NOTUSED
extern nfp_fileCBSkList_t                      	nfp_fileCBSkList;
extern nfp_fileCBSkListPtr_t                   	nfp_fileCBSkListp;
#define NIL                             	nfp_fileCBSkList.nfp_hdr
#define NFP_FILE_CB_SLIST_HDR           	(nfp_fileCBSkListp->nfp_hdr)
#define NFP_FILE_CB_SLIST_LEVEL         	(nfp_fileCBSkListp->nfp_currSlistLevel)
#endif	/*NOTUSED*/


/*-----------------------------------------------------------------
 * Name:        nfp_allocClientCB
 *
 * Description: Allocate a Client Control Block (CCB) from the available
 * buffer list.
 *
 * Created by:  Joseph Vu
 *
 * Date Created:        5/18/01
 *
 *-----------------------------------------------------------------
 */
nfp_clientCBPtr_t		/*Return pointer to clientCB entry or NULL
                                 *if entry cannot be allocated
                                 */
nfp_allocClientCB (void);

/*-----------------------------------------------------------------
 * Name:        nfp_deallocClientCB
 *
 * Description: De-allocate a Client Control Block (CCB) and returns a CCB
 * back to the available buffer list.
 *
 * Created by:  Joseph Vu
 *
 * Date Created:        5/18/01
 *
 *-----------------------------------------------------------------
 */
int32		/*NFX_OK:  Entry has been de-allocate*/
		/*NFX_ERR: Failure to de-allocate entry*/
nfp_deallocClientCB (
	nfp_clientCBPtr_t	nfp_clientCBp	/*Pointer to ClientCB to de-allocate*/
);

/*-----------------------------------------------------------------
 * Name:        nfp_addClientCB
 * 
 * Description: Add a Client Control Block (CCB) into the  CCB table.
 *
 * Created by:  Joseph Vu
 *
 * Date Created:        5/18/01
 *
 *-----------------------------------------------------------------
 */
int32		/*NFX_OK: Entry has been added*/
		/*NFX_ERR: Failure to add entry*/
nfp_addClientCB (
	uint32			nfp_connId,	/*TCP/UDP connection ID*/
	nfp_clientCBPtr_t	nfp_clientCBp	/*ClientCB entry to be added*/
);

/*-----------------------------------------------------------------
 * Name:        nfp_delClientCB
 * 
 * Description: delete a Client Control Block (CCB) from the  CCB table.
 *
 * Created by:  Joseph Vu
 *
 * Date Created:        5/18/01
 *
 *-----------------------------------------------------------------
 */
int32		/*NFX_OK:  Entry has been deleted*/
		/*NFX_ERR:  Failure to delete entry*/
nfp_delClientCB (
	nfp_clientCBPtr_t	nfp_clientCBp	/*ClientCB entry to be deleted*/
);
/*-----------------------------------------------------------------
 * Name:        nfp_getClientCB
 *
 * Description: Retrieve a Client Control Block (CCB) from the  CCB list
 *
 * Created by:  Joseph Vu
 *
 * Date Created:        5/18/01 
 *
 *-----------------------------------------------------------------
 */
nfp_clientCBPtr_t       /*      
                         *Return pointer to the NFP Client Control Block or
                         *NULL if entry is not found
                         */     
nfp_getClientCB (
                uint32 connId           /*TCP/UDP connection ID*/
);

/*-----------------------------------------------------------------
 * Name:       	nfp_allocFileSysCB 
 *
 * Description: Allocate a File System Control Block (FSCB) from the available
 * buffer list.
 *
 * Created by:  Joseph Vu
 *
 * Date Created:        5/18/01
 *
 *-----------------------------------------------------------------
 */
nfp_fileSysCBPtr_t	/*Return pointer to fileSysCB entry or NULL if
			 *entry cannot be allocated
			 */
nfp_allocFileSysCB (void);

/*-----------------------------------------------------------------
 * Name:        nfp_deallocFileSysCB
 *
 * Description: De-allocate a File System Control Block (FSCB) and return entry
 * back to the  available buffer list.
 *
 * Created by:  Joseph Vu
 *
 * Date Created:        5/18/01
 *
 *-----------------------------------------------------------------
 */
int32		/*NFX_OK:  Entry has been de-allocate*/
		/*NFX_ERR:  Failure to de-allocate entry*/
nfp_deallocFileSysCB (
        nfp_fileSysCBPtr_t deallocEntryp   /*Entry to de-allocate*/
);

/*-----------------------------------------------------------------
 * Name:        nfp_addFileSysCB
 *
 * Description: Add a File System Control Block (FSCB) entry into 
 * the FSCB list. 
 *
 * Created by:  Joseph Vu
 *
 * Date Created:        5/18/01
 *
 *-----------------------------------------------------------------
 */
int32	/*NFX_OK: Entry has been added*/
	/*NFX_ERR:  Failure to add entry*/
nfp_addFileSysCB (
        nfp_fileSysCBPtr_t      nfp_addEntryp,  /*entry to be added*/
        fs_Handle               *nfp_fhandlep   /*filehandle*/
);

/*-----------------------------------------------------------------
 * Name:        nfp_delFileSysCB
 *
 * Description: Delete a File System Control Block (FSCB) entry from 
 * the FSCB list. 
 *
 * Created by:  Joseph Vu
 *
 * Date Created:        5/18/01
 *
 *-----------------------------------------------------------------
 */
int32	/*NFX_OK:  Entry has been deleted*/
	/*NFX_ERR: Failure to delete entry*/
nfp_delFileSysCB (nfp_fileSysCBPtr_t  nfp_delEntryp      /*entry to be delete*/
);

/*-----------------------------------------------------------------
 * Name:        nfp_getFileSysCB
 *
 * Description: Retrieves a File System Control Block (FSCB) entry from
 * the FCSB list.
 *
 * Created by:  Joseph Vu
 *
 * Date Created:        5/18/01
 *
 *-----------------------------------------------------------------
 */
nfp_fileSysCBPtr_t      /*
                         *Return pointer to FSCB entry of NULL if entry
                         *not found
                         */
nfp_getFileSysCB ( fs_Handle    *nfp_fhandle        /*filehandle*/
);

/*----------------------------------------------------------------- 
 * Name:        nfp_getFileSysByName
 *
 * Description:
 *      Retrieves a File System Control Block from the buffer list based
 *      on the input directory pathname.
 * 
 * Created by:  Joseph Vu
 *
 * Date Created:        5/18/01
 *
 *-----------------------------------------------------------------
 */
nfp_fileSysCBPtr_t      /*
                         *Return pointer to FSCB entry of NULL if entry
                         *not found
                         */
nfp_getFileSysByName (
                        uchar8  *pathNamep,     /*pathname of the directory*/
                        uint32 pathLen			/*pathname Len*/
);


/*-----------------------------------------------------------------
 * Name:        nfp_allocFileCB
 *
 * Description: Allocate a File Control Block (FCB) from the available
 * buffer list.
 *
 * Created by:  Joseph Vu
 *
 * Date Created:        5/18/01
 *
 *-----------------------------------------------------------------
 */
nfp_fileCBPtr_t      	/*Return pointer to fileCB entry or NULL if
                         *entry cannot be allocated
                         */
nfp_allocFileCB (void);

/*-----------------------------------------------------------------
 * Name:        nfp_deallocFileCB
 *
 * Description: de-allocate a File Control Block (FCB) and return entry 
 * back to  available buffer list.
 *
 * Created by:  Joseph Vu
 *
 * Date Created:        5/18/01
 *
 *-----------------------------------------------------------------
 */
int32		/*NFX_OK:  Entry has been de-allocate*/
		/*NFX_ERR: Failure to de-allocate entry*/
nfp_deallocFileCB (nfp_fileCBPtr_t nfp_fileCBp	/*de-allocate entry*/
		);

/*-----------------------------------------------------------------
 * Name:        nfp_addFileCB
 *
 * Description: Add a File Control Block (FCB) into the FCB skip lists.  
 *
 * Created by:  Joseph Vu
 *
 * Date Created:        5/18/01
 *
 *-----------------------------------------------------------------
 */
int32		/*NFX_OK:  Entry has been added*/
		/*NFX_ERR: Failure to add entry*/
nfp_addFileCB (
		nfp_fileCBPtr_t		nfp_fileCBp,	/*Entry to be added*/
		fs_Handle		*nfp_fhandlep	/*filehandle*/
		);

/*-----------------------------------------------------------------
 * Name:        nfp_delFileCB
 *
 * Description: Delete a File Control Block (FCB) from the FCB skip lists.  
 *
 * Created by:  Joseph Vu
 *
 * Date Created:        5/18/01
 *
 *-----------------------------------------------------------------
 */
int32		/*NFX_OK:  Entry has been deleted*/
		/*NFX_ERR: Failure to delete entry*/
nfp_delFileCB (
		fs_Handle	*nfp_fhandlep	/*filehandle*/
		);

/*-----------------------------------------------------------------
 * Name:        nfp_getFileCB
 *
 * Description: Get a File Control Block (FCB) in the FCB skip lists.
 *
 * Created by:  Joseph Vu
 *
 * Date Created:        5/18/01
 *
 *-----------------------------------------------------------------
 */
nfp_fileCBPtr_t      /*Return pointer to fileSysCB entry or NULL if
                         *entry cannot be allocated
                         */
nfp_getFileCB (
		fs_Handle	*nfp_fhandlep	/*filehandle*/
		);

/*-----------------------------------------------------------------
 * Name:        nfp_allocReqCB
 *
 * Description: Allocate a Request Control Block (RCB) from the available
 * buffer list.
 *
 * Created by:  Joseph Vu
 *
 * Date Created:        5/18/01
 *
 *-----------------------------------------------------------------
 */
nfp_requestCBPtr_t              /*Return pointer to reqCB entry or NULL
                                 *if entry cannot be allocated
                                 */
nfp_allocReqCB (void);

/*-----------------------------------------------------------------
 * Name:        nfp_deallocReqCB
 *
 * Description: De-allocate a Request Control Block (RCB) and returns a RCB
 * back to the available buffer list.
 *
 * Created by:  Joseph Vu
 *
 * Date Created:        5/18/01
 *
 *-----------------------------------------------------------------
 */
int32           /*NFX_OK:  Entry has been de-allocate*/
                /*NFX_ERR: Failure to de-allocate entry*/
nfp_deallocReqCB( nfp_requestCBPtr_t    deallocEntryp );

/*-----------------------------------------------------------------
 * Name:        nfp_allocNode
 *
 * Description: Allocate a generic node for maintaining list for
 * active clients or list of filesystem or list of open files.
 *
 * Created by:  Joseph Vu
 *
 * Date Created:        5/18/01
 *
 *-----------------------------------------------------------------
 */
nfp_nodePtr_t           /*Return pointer to node entry or NULL if entry
                         *can not be allocated
                         */
nfp_allocNode(void);

/*-----------------------------------------------------------------
 * Name:        nfp_deallocNode
 *
 * Description: De-allocate a node and return it back to
 * the available buffer list.
 *
 * Created by:  Joseph Vu
 *
 * Date Created:        5/18/01
 *
 *-----------------------------------------------------------------
 */
int32
nfp_deallocNode (nfp_nodePtr_t deallocEntryp    /*Pointer to de-allocate entry*/
);

/*-----------------------------------------------------------------
 * Name:        nfp_getClientCBbyAddr
 *
 * Description: Retrieves a Client Control Block (CCB) from the  CCB list
 * by ip address.
 *
 * Created by:  Joseph Vu
 *
 * Date Created:        5/18/01
 *
 *-----------------------------------------------------------------
 */
nfp_clientCBPtr_t   /*Pointer to NFP ClientCb or NULL
                     */
nfp_getClientCBbyAddr (
        uint32 clientAddr   /*client IP address*/
);

/*-----------------------------------------------------------------
 * Name:        nfp_validateNode
 *
 * Description: Validates a NFP generic node buffer address.
 *
 * Created by:  Joseph Vu
 *
 * Date Created:        5/18/01
 *
 *-----------------------------------------------------------------
 */
int32
nfp_validateNode(nfp_nodePtr_t nodep);

/*-----------------------------------------------------------------
 * Name:        nfp_validateClientCB
 *
 * Description: Validates a Client Control Block (CCB) buffer address.
 *
 * Created by:  Joseph Vu
 *
 * Date Created:        5/18/01
 *
 *-----------------------------------------------------------------
 */
int32 nfp_validateClientCB(nfp_clientCBPtr_t clientCBp);

/*-----------------------------------------------------------------
 * Name:        nfp_validateFileCB
 *
 * Description: Validate File Control Block (FCB) buffer address.
 *
 * Created by:  Joseph Vu
 *
 * Date Created:        5/18/01
 *
 *-----------------------------------------------------------------
 */
int32
nfp_validateFileCB(nfp_fileCBPtr_t fileCBp);

/*-----------------------------------------------------------------
 * Name:        nfp_validateFileSysCB
 *
 * Description:
 *      Validates a File System Control Block buffer address.
 *
 * Created by:  Joseph Vu
 *
 * Date Created:        5/18/01
 *
 *-----------------------------------------------------------------
 */
int32
nfp_validateFileSysCB(nfp_fileSysCBPtr_t fileSysCBp);
/*-----------------------------------------------------------------
 * Name:        nfp_validateReplyCB
 *
 * Description: Validates a Reply Control Block (RCB) buffer address.
 *
 * Created by:  Joseph Vu
 *
 * Date Created:        5/18/01
 *
 *-----------------------------------------------------------------
 */
int32
nfp_validateReplyCB (nfp_replyCBPtr_t replyCBp);

/*-----------------------------------------------------------------
 * Name:        nfp_validateNfsReqCB
 *
 * Description: Validates the NFS Request Control Block (RCB) buffer.
 *
 * Created by:  Joseph Vu
 *
 * Date Created:        5/18/01
 *
 *-----------------------------------------------------------------
 */
int32
nfp_validateNfsReqCB(nfs_request_t *nfsReqp);

/*-----------------------------------------------------------------
 * Name:        nfp_validateMntReqCB
 *
 * Description: Validates the MNT Request Control Block (RCB) buffer.
 *
 * Created by:  Joseph Vu
 *
 * Date Created:        5/18/01
 *
 *-----------------------------------------------------------------
 */
int32
nfp_validateMntReqCB(mnt_request_t *mntReqp);

/*-----------------------------------------------------------------
 * Name:        nfp_validateNlmReqCB
 *
 * Description: Validates the NLM Request Control Block (RCB) buffer.
 *
 * Created by:  Xiaobing Zhang
 *
 * Date Created:        7/15/02
 *
 *-----------------------------------------------------------------
 */
int32
nfp_validateNlmReqCB(nlm_request_t *nlmReqp);

/*-----------------------------------------------------------------
 * Name:        nfp_allocLockCB
 *
 * Description: Allocate a Lock Control Block (LCB) from the available
 * buffer list.
 * Created by:  Xiaobing Zhang
 *
 * Date Created:        7/15/02
 *
 *-----------------------------------------------------------------
 */
nfp_lockCBPtr_t nfp_allocLockCB(void);

/*-----------------------------------------------------------------
 * Name:        nfp_deallocLockCB
 *
 * Description: de-allocate a Lock Control Block (LCB) and return entry
 * back to available buffer list.
 *
 * Created by:  Xiaobing Zhang
 *
 * Date Created:        7/15/02
 *
 *-----------------------------------------------------------------
 */
int32
nfp_deallocLockCB(nfp_lockCBPtr_t deallocEntryp);

/*-----------------------------------------------------------------
 * Name:        nfp_validateLockCB 
 *
 * Description: Validate Lock Control Block (LCB) buffer address.
 *
 * Created by:  Xiaobing Zhang
 *
 * Date Created:        7/15/02
 *
 *-----------------------------------------------------------------
 */
int32
nfp_validateLockCB(nfp_lockCBPtr_t lockCBp);


#endif  /* _NFP_API_H */


