/* vi:ts=4 
 *-------------------------------------------------------
 *
 * Name:        nfs-ssc-cmd.c
 *                              
 * RCS:  $Id: nfs-ssc-cmd.c,v 1.23 2002/11/07 18:46:16 janv Exp $
 *
 * Copyright (C) 2001, Agile Storage Inc.
 *
 * Description:
 *  Contain NFS command functions on SSC
 *
 * Created by: Joseph Vu
 *
 * Date Created:        10/28/01
 *
 *-------------------------------------------------------
 */
#include <nfx-incl.h>
//#include <exports/nfs-api.h>
#include <exports/timer-api.h>
#include <exports/evm-api.h>
#include <exports/anpssc-api.h>
#include <exports/rpc.h>
#include <exports/fs-api.h> 
#include <exports/mount.h>

#ifdef	NOTUSED
#include "exports/elog-api.h"
#endif	//NOTUSED

#include <exports/nfs.h>
#include "nfs-ssc-cmd.h"
#ifdef  LINUX_TEST
#include <string.h>
#include <stdio.h>

#define		NFS_APP_NAME		"nfs_srvr"
#define		NFS_CMD_APP_NAME	"nfs_cmd"

#define		NFS_RESP_BUFLEN		4096

#endif	//LINUX_TEST
int  nfs_sscCmdExport ( int argc, char *argv[] );
int  nfs_sscCmdReadVolIds ( int argc, char *argv[] );
int  nfs_sscCmdPortRegPorts ( int argc, char *argv[] );
int  nfs_sscCmdShowStat ( int argc, char *argv[] );
int	 nfs_sscCmdHelpAll (void);
int	nfs_sscCmdHelpCmd (char *pCmd);
int	nfs_sscCmdHelp ( int argc, char *argv[] );
int	nfs_sscCmdExportDel ( int argc, char *argv[] );
int	nfs_sscCmdExportDelAll ( int argc, char *argv[] );
int	nfs_sscCmdExportUpdate(int argc, char *argv[] );
int	nfs_sscCmdExportAddAll ( int argc, char *argv[] );
int nfs_sscCmdExportShow ( int argc, char *argv[] );
int nfs_sscCmdIsVolExported ( int argc, char *argv[] );
int nfs_sscCmdTrace ( int argc, char *argv[] );
int nfs_sscCmdExpRule ( int argc, char *argv[] );
int32 getIpAddr( char *ipAddrStr, uint32 *addr);
void nfs_sscCmdDisplayStat(nfs_infoResp_t   *nfs_infoRespMsgp, uint32 clientAddr);
void nfs_sscCmdDisplayEntries(mnt_infoResp_t      *mntInfoResp);
typedef struct
{       
	int   (*cmdFunc)( int argc, char *argv[] );
	char    *cmd;
	char    *helpTxt;
} nfs_sscCmdTbl_t;

nfs_sscCmdTbl_t    nfs_sscCmdTable[ ] =
{
//{ nfs_sscCmdExport,  "exportfs", "nfscmd exportfs [-auiv] [-o options] [client:/path]" },
//{ nfs_sscCmdReadVolIds,  "readvolids", "nfscmd readvolids" },
//{ nfs_sscCmdPortRegPorts, "portmapreg", "nfscmd portmapreg" },
{ nfs_sscCmdShowStat, "nfsstat", "nfscmd nfsstat [clientIp]" },
{ nfs_sscCmdTrace, "trace", "nfscmd trace on|off" },
{ nfs_sscCmdExpRule, "exprule", "nfscmd exprule" }
};

#define NFS_SSC_CMD_MAX sizeof(nfs_sscCmdTable)/sizeof(nfs_sscCmdTbl_t)

#define	NFS_STR				"NFS"
#define	NFS_ERR_STR			"NFS-ERR"
#define	NFS_PROCESS_NFS_CMD_STR	"Process command!\n"

#define	NFS_CMD_FMT_ERR		"Format error!\n"
#define NFS_SSC_ALLOC        malloc
#define NFS_SSC_FREE         free

#define	NFS_MSG_BUF_LEN		2048
#define		NFS_DBG_PRINTF	

/* Declare globals used by eee_rtc calls.  The actual values will be returned
 * as part of the responses.
 */
uint64 count_per_usec = 1;

/*-----------------------------------------------------------------
 * Name: nfs_cmdHelpAll
 *
 * Description: Display help-texts for all NFS commands.
 *
 * Created by : Joseph Vu
 *
 * Date Created: 09/13/2001
 *-----------------------------------------------------------------
 */
int
nfs_sscCmdHelpAll (void)
{
	int		i;
	printf("Usage:");

	for (i=0; i < NFS_SSC_CMD_MAX; i++)
	{
		printf("\t%s\n", nfs_sscCmdTable[i].helpTxt);
	}
	return (0);
}

/*-----------------------------------------------------------------
 * Name: nfs_cmdHelpCmd
 *
 * Description: Display help-texts for an individual NFS command.
 *
 * Created by : Joseph Vu
 *
 * Date Created: 09/13/2001
 *-----------------------------------------------------------------
 */
int
nfs_sscCmdHelpCmd (char *pCmd)
{
	int		i;
	for (i=0; i < NFS_SSC_CMD_MAX; i++)
	{
		if (strcmp(pCmd, nfs_sscCmdTable[i].cmd) == 0)
		{
			printf("Usage:");
			printf("\t%s\n", nfs_sscCmdTable[i].helpTxt);
			return 0;
		}
	}
	return (nfs_sscCmdHelpAll( ));

}

/*-----------------------------------------------------------------
 * Name:	nfs_sscCmdHelp 
 *
 * Description: Display NFS commands help-text.
 *
 *
 * Created by : Joseph Vu
 *
 * Date Created: 09/13/2001
 *-----------------------------------------------------------------
 */
int
nfs_sscCmdHelp ( int argc, char *argv[] )
{
	if (argc < 3)
		return (nfs_sscCmdHelpAll());
	else
		return (nfs_sscCmdHelpCmd(( argv[2] )));


}

/*-----------------------------------------------------------------
 * Name:   	nfs_sscCmdExportDel 
 *
 * Description:
 *      Process an unexport command.
 *      nfscmd exportfs -u
 *
 * Created by : Joseph Vu
 *
 * Date Created: 09/13/2001
 *-----------------------------------------------------------------
 */
int
nfs_sscCmdExportDel ( int argc, char *argv[] )
{
	//char				*namep;
	int					nameLen;
	int                 msgLen;
	mnt_infoReq_t		*mntInfoReqp;
	int                 stat;
	struct timeval      timeout = { 10, 0};
	mnt_infoResp_t      *mntInfoResp;
	char                respbuff[NFS_MSG_BUF_LEN];
	char				reqMsgBuff[NFS_MSG_BUF_LEN];


	NFS_DBG_PRINTF("nfs_sscCmdExportDel: Enter\n");
#ifdef	NOTUSED
	if (argc != 4)
	{
		printf("Format error!\n");
		return (NFX_ERR);
	}
#endif

#ifdef	NOTUSED	
	{
		int			i;

		for (i=0; i < argc; i++)
		{
			printf("nfs_sscCmdExportDel: argv[%d]: %s\n", i, argv[i]);
		}
	}

#endif

	//argv[0]: nfscmd
	//argv[1]: exportfs
	//argv[2]: -u
	//argv[3]: /pathname
	//argv[4]: localIpAddr

	//argv[3]: host:/pathname
	nameLen = strlen (argv[3]);
	if (nameLen <= 0)
		return (NFX_ERR);
	msgLen = sizeof(mnt_infoReq_t) + nameLen + 1;
	memset(reqMsgBuff, 0, sizeof(reqMsgBuff));
	mntInfoReqp = (mnt_infoReq_t   *)reqMsgBuff;
#ifdef	NOTUSED	
	//mntInfoReqp = (mnt_infoReq_t *)NFS_SSC_ALLOC(msgLen);
	if (mntInfoReqp == NULL)
	{
		printf("nfs_sscCmdExportDel: Failure to alloc memory!\n");
		return (NFX_ERR);		
	}
#endif	//NOTUSED

	memset(respbuff, 0, sizeof(respbuff));
	mntInfoResp = (mnt_infoResp_t      *)respbuff;
	memset(mntInfoReqp, 0, msgLen);
	mntInfoReqp->msgHdr.private = 0;
	mntInfoReqp->msgHdr.msgType = MNT_UNEXPORT_REQ;
	mntInfoReqp->nameLen = nameLen;
	memcpy(&mntInfoReqp->pathName[0], argv[3], mntInfoReqp->nameLen);

    getIpAddr(argv[4], &mntInfoReqp->localIpAddr);
	
	NFS_DBG_PRINTF("nfs_sscCmdExportDel: pathName: %s len: %d localIp: %08x\n",
					mntInfoReqp->pathName, mntInfoReqp->nameLen, mntInfoReqp->localIpAddr);
	stat = sendAgileMsg (-1, mntInfoReqp, msgLen,
				MOUNTD_APP_NAME, NFX_SSC_UNIX, SSC_SLOT, MGMTPLANE,
				&timeout,
				respbuff, sizeof(respbuff));
	if ( (stat < 0) || (mntInfoResp->msgHdr.status != NFX_OK) )
	{
		printf("nfs_sscCmdExportDel: sendAgileMsg failed!\n");
	}



	//NFS_SSC_FREE(mntInfoReqp);
	return (NFX_OK);

}

/*-----------------------------------------------------------------
 * Name:   	nfs_sscCmdExportDelAll 
 *
 * Description:
 *      Process an unexport all command.
 *      nfscmd exportfs -ua or
 * 		nfscmd exportfs -au
 *
 * Created by : Joseph Vu
 *
 * Date Created: 09/13/2001
 *-----------------------------------------------------------------
 */
int
nfs_sscCmdExportDelAll ( int argc, char *argv[] )
{
	int                 msgLen;
	mnt_infoReq_t       mntInfoReq;
	int                 stat;
	struct timeval      timeout = { 10, 0};
	mnt_infoResp_t      *mntInfoResp;
	char                respbuff[NFS_MSG_BUF_LEN];

	NFS_DBG_PRINTF("nfs_sscCmdExportDelAll Enter\n");
	memset(respbuff, 0, sizeof(respbuff));
	mntInfoResp = (mnt_infoResp_t      *)respbuff;
	memset(&mntInfoReq, 0, sizeof(mntInfoReq));
	mntInfoReq.msgHdr.private = 0;
	mntInfoReq.msgHdr.msgType = MNT_UNEXPORT_REQ;
	msgLen = sizeof(mntInfoReq);

	stat = sendAgileMsg (-1, &mntInfoReq, msgLen,
				MOUNTD_APP_NAME, NFX_SSC_UNIX, SSC_SLOT, MGMTPLANE,
				&timeout,
				respbuff, sizeof(respbuff));
	if ( (stat < 0) || (mntInfoResp->msgHdr.status != NFX_OK) )
	{
		printf("nfs_sscCmdExportDel: sendAgileMsg failed!\n");
		return (NFX_ERR);
	}

	if (mntInfoResp->msgHdr.status != NFX_OK)
	{
		printf("ERROR: %s\n",NFS_PROCESS_NFS_CMD_STR);	
		return (NFX_ERR);
	}
	return (NFX_OK);


}

/*-----------------------------------------------------------------
 * Name:   	nfs_sscCmdExportUpdate 
 *
 * Description:
 *      Update an export entry.  Ignores current export options
 * in exports file. 
 *      nfscmd exportfs -i <client/vol> or
 *		nfscmd exportfs -i -o <options> <client/vol>
 *
 *
 * Created by : Joseph Vu
 *
 * Date Created: 09/13/2001
 *-----------------------------------------------------------------
 */
int
nfs_sscCmdExportUpdate(int argc, char *argv[] )
{
	int					retStat;
	int					optionLen;
	int					nameLen;
	int                 msgLen;
	char                respbuff[NFS_MSG_BUF_LEN];
	char                reqMsgBuff[NFS_MSG_BUF_LEN];
	mnt_infoReq_t       *mntInfoReqp;
	int                 stat;
	struct timeval      timeout = { 10, 0};
	mnt_infoResp_t      *mntInfoResp;
	char                *optionp;

	if ( (argc != 6) &&  (argc != 4))
	{
		printf("[%s]:  %s", NFS_ERR_STR, NFS_CMD_FMT_ERR);
		return (NFX_ERR);
	}
	retStat = NFX_OK;
	if (argc == 6)
	{
		optionLen = strlen(argv[4]);
		nameLen = strlen(argv[5]);
	}
	else
	{
		optionLen = 0;
		nameLen = strlen(argv[3]);
	}
	msgLen = sizeof(mnt_infoReq_t) + optionLen + 1 + nameLen + 1;
    memset(reqMsgBuff, 0, sizeof(reqMsgBuff));
    mntInfoReqp = (mnt_infoReq_t   *)reqMsgBuff;
#ifdef	NOTUSED
	mntInfoReqp	= (mnt_infoReq_t *)NFS_SSC_ALLOC(msgLen); 
	if (mntInfoReqp == NULL)
	{
		printf("nfs_sscCmdExportUpdate: Failure to alloc memory!\n");
		return (NFX_ERR);
	}
#endif	//NOTUSED
	memset(respbuff, 0, sizeof(respbuff));
	mntInfoResp = (mnt_infoResp_t      *)respbuff;
	memset(mntInfoReqp, 0, msgLen);
	mntInfoReqp->msgHdr.private = 0;
	mntInfoReqp->msgHdr.msgType = MNT_EXPORT_UPDATE;

	if (strncmp(argv[3], "-o", 2) == 0)
	{
		if (argc != 6)
		{
			retStat = NFX_ERR;
			goto nfs_sscCmdExportUpdateErr;
		}
		//argv[4]: options
		//argv[5]: contains client/volume1
		mntInfoReqp->optionsLen = optionLen;
		mntInfoReqp->nameLen = nameLen;
		mntInfoReqp->msgHdr.private = 0;
		mntInfoReqp->msgHdr.msgType = MNT_EXPORT_UPDATE;
		if ( (mntInfoReqp->nameLen <= 0) ||
				(mntInfoReqp->optionsLen <= 0) )
		{
			retStat = NFX_ERR;
			goto nfs_sscCmdExportUpdateErr;
		}
		memcpy(&mntInfoReqp->pathName[0], 
				argv[5], mntInfoReqp->nameLen);
		optionp = &mntInfoReqp->pathName[0]+ mntInfoReqp->nameLen;
		memcpy(optionp, argv[4], mntInfoReqp->optionsLen);

	}
	else
	{
		//exportfs -i client/vol1
		//argv[3]: contains client/volume1
		if (argc < 4)
		{
			retStat = NFX_ERR;
			goto nfs_sscCmdExportUpdateErr;
		}

		mntInfoReqp->optionsLen = 0;
		mntInfoReqp->nameLen = nameLen;
		mntInfoReqp->msgHdr.private = 0;
		mntInfoReqp->msgHdr.msgType = MNT_EXPORT_UPDATE;
		if (mntInfoReqp->nameLen <= 0)
		{
			retStat = NFX_ERR;
			goto nfs_sscCmdExportUpdateErr;
		}
		memcpy(&mntInfoReqp->pathName[0], argv[3], 
					mntInfoReqp->nameLen);

	}

	stat = sendAgileMsg (-1, mntInfoReqp, msgLen,
				MOUNTD_APP_NAME, NFX_SSC_UNIX, 
				SSC_SLOT, MGMTPLANE,
				&timeout,
				respbuff, sizeof(respbuff));
	if ( (stat < 0) || (mntInfoResp->msgHdr.status != NFX_OK) )
	{
		printf("nfs_sscCmdExportAdd: sendAgileMsg failed!\n");
		retStat = NFX_ERR;
		goto nfs_sscCmdExportUpdateErr;
	}

	if (mntInfoResp->msgHdr.status != NFX_OK)
	{
		printf("ERROR: %s\n",NFS_PROCESS_NFS_CMD_STR);
	}

nfs_sscCmdExportUpdateErr:
//	NFS_SSC_FREE(mntInfoReqp);
	return (retStat);

}

/*-----------------------------------------------------------------
 * Name:   	nfs_sscCmdExportAdd 
 *
 * Description:
 *     	Process an export command. 
 *      nfscmd exportfs -o
 * 
 *
 * Created by : Joseph Vu
 *
 * Date Created: 09/13/2001
 *-----------------------------------------------------------------
 */
int 
nfs_sscCmdExportAdd ( int argc, char *argv[] )
{
	int					retStat;
	int					msgLen;
	char                reqMsgBuff[NFS_MSG_BUF_LEN];	
	char                respbuff[NFS_MSG_BUF_LEN];
	mnt_infoReq_t       *mntInfoReqp;
	int                 stat;
	struct timeval      timeout = { 10, 0};
	mnt_infoResp_t      *mntInfoResp;
	char				*optionp;

	NFS_DBG_PRINTF("nfs_sscCmdExportAdd:  Enter\n");
	if (argc != 6)
	{
		return (NFX_ERR);
	}


	retStat = NFX_OK;
	//printf("nfs_sscCmdExportAdd 2: argv[3]: %s argv[4]: %s\n", argv[3], argv[4]);

	//User specified options
	//exportfs -o options host:/vol1
	//argv[0]:nfscmd
	//argv[1]:exportfs
	//argv[2]: -o
	//argv[3]: localIpaddr
	//argv[4]: options
	//argv[5]: contains host1/volume1
#ifdef    NOTUSED
    {
        //For debug only
        int     i;
        printf("nfs_sscCmdExportAddshare: argc: %d\n", argc);
        for (i=0; i < argc; i++)
        {
            printf("argv[%d]: %s\n", i, argv[i]);
        }
    }
#endif    //NOTUSED

	if ( (strlen(argv[4]) <= 0) ||
		(strlen(argv[5]) <= 0) )
		return (NFX_ERR);

	msgLen = sizeof(mnt_infoReq_t) + 
				strlen(argv[4]) + strlen(argv[5]) + 2;
	memset(reqMsgBuff, 0, sizeof(reqMsgBuff));
	mntInfoReqp = (mnt_infoReq_t   *)reqMsgBuff;

	
	
	memset(mntInfoReqp, 0, msgLen);
	memset(respbuff, 0, sizeof(respbuff));
	mntInfoResp = (mnt_infoResp_t      *)respbuff;
	mntInfoReqp->msgHdr.private = 0;
	mntInfoReqp->msgHdr.msgType = MNT_EXPORT_REQ;

	mntInfoReqp->optionsLen = strlen(argv[4]);
	mntInfoReqp->nameLen = strlen(argv[5]);
	mntInfoReqp->msgHdr.private = 0;
	mntInfoReqp->msgHdr.msgType = MNT_EXPORT_REQ;

	memcpy(&mntInfoReqp->pathName[0], argv[5], mntInfoReqp->nameLen);
	optionp = &mntInfoReqp->pathName[0]+ mntInfoReqp->nameLen;
	memcpy(optionp, argv[4], mntInfoReqp->optionsLen);

	//mntInfoReqp->localIpAddr
	getIpAddr(argv[3], &mntInfoReqp->localIpAddr);
	/*
	printf("nfs_sscCmdExportAdd: optionp: %s len: %d laddr: %08x\n", 
			optionp, mntInfoReqp->optionsLen, mntInfoReqp->localIpAddr);	
	*/
	stat = sendAgileMsg (-1, mntInfoReqp, msgLen,
				MOUNTD_APP_NAME, NFX_SSC_UNIX, SSC_SLOT, MGMTPLANE,
				&timeout, 
				respbuff, sizeof(respbuff));
	if ( (stat < 0) || (mntInfoResp->msgHdr.status != NFX_OK) )
	{
		printf("CMD ERR: check syntax (duplicate entry?)\n");	
		retStat = NFX_ERR;
		goto nfs_sscCmdExportAddErr;
	}

	if (mntInfoResp->msgHdr.status != NFX_OK)
	{
		printf("ERROR: %s\n",NFS_PROCESS_NFS_CMD_STR);	
		retStat = NFX_ERR;
	}

nfs_sscCmdExportAddErr:

	return (retStat);
}

/*-----------------------------------------------------------------
 * Name:   	nfs_sscCmdExportAddAll 
 *
 * Description:
 *      Process an export  all command.
 *      nfscmd exportfs -a
 *
 *
 * Created by : Joseph Vu
 *
 * Date Created: 09/13/2001
 *-----------------------------------------------------------------
 */
int
nfs_sscCmdExportAddAll ( int argc, char *argv[] )
{
	int					msgLen;
	char                respbuff[NFS_MSG_BUF_LEN];
	mnt_infoReq_t       mntInfoReq;
	int                 stat;
	struct timeval      timeout = { 10, 0};
	mnt_infoResp_t      *mntInfoResp;

	memset(respbuff, 0, sizeof(respbuff));
	mntInfoResp = (mnt_infoResp_t      *)respbuff;
	memset(&mntInfoReq, 0, sizeof(mntInfoReq));
	mntInfoReq.msgHdr.private = 0;
	mntInfoReq.msgHdr.msgType = MNT_EXPORT_REQ;
	msgLen = sizeof(mntInfoReq);

	
	//exportfs -a
	//	export all entries in exports file
	mntInfoReq.nameLen = 0;
	mntInfoReq.optionsLen = 0;
	

	stat = sendAgileMsg (-1, &mntInfoReq, msgLen,
				MOUNTD_APP_NAME, NFX_SSC_UNIX, SSC_SLOT, MGMTPLANE,
				&timeout, 
				respbuff, sizeof(respbuff));
	if ( (stat < 0) || (mntInfoResp->msgHdr.status != NFX_OK) )
	{
		printf("nfs_sscCmdExportAdd: sendAgileMsg failed!\n");	
		return (NFX_ERR);
	}

	if (mntInfoResp->msgHdr.status != NFX_OK)
	{
		printf("ERROR: %s\n",NFS_PROCESS_NFS_CMD_STR);	
		return (NFX_ERR);
	}

	return (NFX_OK);
}

/*-----------------------------------------------------------------
 * Name:   	nfs_sscCmdExportShow 
 *
 * Description:
 *     	Display current export entries. 
 *		nfscmd exportfs or
 *      nfscmd exportfs -v 
 *  
 *
 * Created by : Joseph Vu
 *
 * Date Created: 09/13/2001
 *-----------------------------------------------------------------
 */
int nfs_sscCmdExportShow ( int argc, char *argv[] )
{
	char 				respbuff[NFS_MSG_BUF_LEN];
	mnt_infoReq_t   	mntInfoReq;
	int					stat;
	struct timeval		timeout = { 10, 0};
	mnt_infoResp_t		*mntInfoResp;

	memset(respbuff, 0, sizeof(respbuff));
	mntInfoResp = (mnt_infoResp_t      *)respbuff;
	memset(&mntInfoReq, 0, sizeof(mntInfoReq));
	mntInfoReq.msgHdr.private = 0;

	//exportfs
	if (argc == 2)
	{
		mntInfoReq.msgHdr.msgType = MNT_EXPORT_SHOW;
	}
	else
	{
		//exportfs -v
		mntInfoReq.msgHdr.msgType = MNT_EXPORT_SHOWALL;
	}
	stat = sendAgileMsg (-1, &mntInfoReq, sizeof(mntInfoReq),
				MOUNTD_APP_NAME, NFX_SSC_UNIX, SSC_SLOT, MGMTPLANE,
				&timeout, 
				respbuff, sizeof(respbuff));
	if (stat < 0)
	{
		printf("sendAgileMsg failed!\n");
		return (NFX_ERR);
	}

	if (mntInfoResp->msgHdr.status != NFX_OK)
	{
		printf("ERROR: %s\n",NFS_PROCESS_NFS_CMD_STR);	
		return (NFX_ERR);
	}

	nfs_sscCmdDisplayEntries(mntInfoResp);		
	return (NFX_OK);
}

/*-----------------------------------------------------------------
 * Name:	nfs_sscCmdExport 
 *
 * Description:
 *		NFS exportfs command. 
 *		nfscmd exportfs [-auiv] [-o options] [client/path]
 *             
 *
 * Created by : Joseph Vu
 *
 * Date Created: 09/13/2001
 *-----------------------------------------------------------------
 */
int  nfs_sscCmdExport ( int argc, char *argv[] ) 
{

	//printf("nfs_sscCmdExport: argc: %d\n", argc);
	if (argc < 2)
	{
		nfs_sscCmdHelpCmd("exportfs");
		return (NFX_OK);
	}

	if (argc == 2)
	{
		//show current export entries.
		nfs_sscCmdExportShow (argc, argv);
		return (NFX_OK);	
	}

	if ((strncmp(argv[2], "-au", 3) == 0) ||
			(strncmp(argv[2], "-ua", 3) == 0) )	
	{
		//unexport all entries
		nfs_sscCmdExportDelAll (argc, argv);
		return (NFX_OK);
	}
	else if (strncmp(argv[2], "-a", 2) == 0)
	{
		//add export option
		if (nfs_sscCmdExportAddAll (argc, argv) == NFX_ERR)
		{
			nfs_sscCmdHelpCmd("exportfs");
		}
		return (NFX_OK);
	}
	else if (strncmp(argv[2], "-u", 2) == 0)
	{
		//unexport option
		nfs_sscCmdExportDel (argc, argv);
		return (NFX_OK);
	}
	else if (strncmp(argv[2], "-o", 2) == 0)
	{
		//To export the /vol1 directory to host djando,  allowing
		//read/write, one would do this:
		//exportfs -o rw django/vol1
		//add export option
		if (nfs_sscCmdExportAdd (argc, argv) == NFX_ERR)
		{
			nfs_sscCmdHelpCmd("exportfs");
		}
		return (NFX_OK);
	}
	else if (strncmp(argv[2], "-v", 2) == 0)
	{
		//show (details) current export entries.
		nfs_sscCmdExportShow (argc, argv);
		return (NFX_OK);
	}
	else if (strncmp(argv[2], "-i", 2) == 0)
	{
		//export entry listed in the exports file, but
		//ingores the options specified for the entry
		if (argc < 3)
		{
			nfs_sscCmdHelpCmd("exportfs");
			return (NFX_OK);
		}
		nfs_sscCmdExportUpdate(argc, argv);
		return (NFX_OK);
	}
	else
	{	
		nfs_sscCmdHelpCmd("exportfs");
		return (NFX_OK);

	}
	return (NFX_OK);
}

#define		NFX_TXRX_SLOT	1
int  nfs_sscCmdShowStat ( int argc, char *argv[] ) 
{
	uint32				clientAddr;
//	int					i;
	int					stat;
	int					msgLen;
	nfs_infoReq_t		nfs_infoReqMsg;
	char				respbuff[NFS_RESP_BUFLEN];
	nfs_infoResp_t		*nfs_infoRespMsgp;
	struct timeval      timeout = { 20, 0};

	/*	
	for (i=0; i < argc; i++)
		printf("argv[%d]: %s\n", i, argv[i]);
	*/

	//argv[0]: nfscmd
	//argv[1]: nfsstat
	//argv[2]: 10.2.2.100
	
		
	clientAddr = 0;

	if ( (argc != 2) && (argc !=3 ) )
	{
		nfs_sscCmdHelpCmd("nfsstat");
		return (NFX_OK);
	}

	if (argc == 3)
	{
		if (getIpAddr ( argv[2], &clientAddr) != 0)
		{
			nfs_sscCmdHelpCmd("nfsstat");
			return (NFX_OK);
		}
		printf("nfs_sscCmdShowStat: clientAddr: %08x\n", clientAddr);
	}
	memset (respbuff, 0, sizeof(respbuff));
	nfs_infoRespMsgp = (nfs_infoResp_t *)respbuff;
	memset (&nfs_infoReqMsg, 0, sizeof(nfs_infoReqMsg));

	nfs_infoReqMsg.msgHdr.private = 0;
	nfs_infoReqMsg.msgHdr.msgType = NFS_REQ_STAT_MSG;
	nfs_infoReqMsg.clientAddr = clientAddr;
	msgLen = sizeof(nfs_infoReq_t);

	//TXRX CPU: NFX_NFP_RX
	//TXRX Slot: 1

	stat = sendAgileMsg (-1, &nfs_infoReqMsg, msgLen,
                NFS_CMD_APP_NAME, NFX_NFP_TXRX1, NFX_TXRX_SLOT, MGMTPLANE,
                &timeout,
                respbuff, sizeof(respbuff));

	if  (stat <= 0)
	{
		printf("nfs_sscCmdShowStat 1: sendAgileMsg failed!\n");
		return (NFX_ERR);
	}	
	/*
   	printf("nfs_sscCmdShowStat: sendAgileMsg stat: %d msg ret status: %d\n",
				stat, nfs_infoRespMsgp->msgHdr.status);
	*/
	nfs_infoRespMsgp = (nfs_infoResp_t *)respbuff;	
	if (nfs_infoRespMsgp->msgHdr.status != NFX_OK)
	{
		printf("nfs_sscCmdShowStat 2: Bad response message!\n");
		return (NFX_ERR);
	}
	
	nfs_sscCmdDisplayStat(nfs_infoRespMsgp, clientAddr);	
	//process response	
	return (NFX_OK);
}	//nfs_sscCmdShowStat

/*-----------------------------------------------------------------
 * Name:   	nfs_sscCmdReadVolIds 
 *
 * Description:
 *      NFS read volume Ids command.
 *      nfscmd readvolids 
 *
 *
 * Created by : Joseph Vu
 *
 * Date Created: 09/13/2001
 *-----------------------------------------------------------------
 */
int  nfs_sscCmdReadVolIds ( int argc, char *argv[] )
{
	char                respbuff[NFS_MSG_BUF_LEN];
	mnt_infoReq_t       mntInfoReq;
	int                 stat;
	struct timeval      timeout = { 10, 0};
	mnt_infoResp_t      *mntInfoResp;

	if (argc < 2)
	{
		nfs_sscCmdHelpCmd("readvolids");
		return (NFX_OK);
	}

	memset(respbuff, 0, sizeof(respbuff));
	mntInfoResp = (mnt_infoResp_t      *)respbuff;
	memset(&mntInfoReq, 0, sizeof(mntInfoReq));
	mntInfoReq.msgHdr.private = 0;
	mntInfoReq.msgHdr.msgType = MNT_GET_VOLIDS;

	stat = sendAgileMsg (-1, &mntInfoReq, sizeof(mntInfoReq),
				MOUNTD_APP_NAME, NFX_SSC_UNIX, SSC_SLOT, MGMTPLANE,
				&timeout, 
				respbuff, sizeof(respbuff));
	if (stat < 0)
	{
		printf("sendAgileMsg failed!\n");
		return (NFX_ERR);
	}

	if (mntInfoResp->msgHdr.status != NFX_OK)
	{
		printf("ERROR: %s\n",NFS_PROCESS_NFS_CMD_STR);	
		return (NFX_ERR);
	}

	return (NFX_OK);
}

/*-----------------------------------------------------------------
 * Name:   	nfs_sscCmdPortRegPorts 
 *
 * Description:
 *      Register MOUNT/NFS ports with portmapper.
 *      nfscmd portmap  
 *
 *
 * Created by : Joseph Vu
 *
 * Date Created: 09/13/2001
 *-----------------------------------------------------------------
 */
int  nfs_sscCmdPortRegPorts ( int argc, char *argv[] )
{
	char                respbuff[1024];
	mnt_infoReq_t       mntInfoReq;
	int                 stat;
	struct timeval      timeout = { 10, 0};
	mnt_infoResp_t      *mntInfoResp;

	if (argc < 2)
	{
		nfs_sscCmdHelpCmd("readvolids");
		return (NFX_OK);
	}

	memset(respbuff, 0, sizeof(respbuff));
	mntInfoResp = (mnt_infoResp_t      *)respbuff;
	memset(&mntInfoReq, 0, sizeof(mntInfoReq));
	mntInfoReq.msgHdr.private = 0;
	mntInfoReq.msgHdr.msgType = MNT_PORTMAP_REG_PORTS;

	stat = sendAgileMsg (-1, &mntInfoReq, sizeof(mntInfoReq),
				MOUNTD_APP_NAME, NFX_SSC_UNIX, SSC_SLOT, MGMTPLANE,
				&timeout, 
				respbuff, sizeof(respbuff));
	if (stat < 0)
	{
		printf("sendAgileMsg failed!\n");
		return (NFX_ERR);
	}

	if (mntInfoResp->msgHdr.status != NFX_OK)
	{
		printf("ERROR: %s\n",NFS_PROCESS_NFS_CMD_STR);	
		return (NFX_ERR);
	}

	return (NFX_OK);
}

int nfs_sscCmdIsVolExported ( int argc, char *argv[] )
{
	char                reqMsgBuff[NFS_MSG_BUF_LEN];	
    char                respbuff[NFS_MSG_BUF_LEN];
    mnt_infoReq_t       *mntInfoReqp;
    int                 stat;
    struct timeval      timeout = { 10, 0};
    mnt_infoResp_t      *mntInfoResp;
	evm_volId_t			reqVolId;

    if (argc < 2)
    {
        nfs_sscCmdHelpCmd("readvolids");
        return (NFX_OK);
    }
	reqVolId.whole_thing = 0;
	memset(reqMsgBuff, 0, sizeof(reqMsgBuff));
	
    memset(respbuff, 0, sizeof(respbuff));
	mntInfoReqp = (mnt_infoReq_t       *)reqMsgBuff;
    mntInfoResp = (mnt_infoResp_t      *)respbuff;
    mntInfoReqp->msgHdr.private = 0;
	mntInfoReqp->volId = reqVolId;
    mntInfoReqp->msgHdr.msgType = MNT_IS_VOL_EXPORTED;

    stat = sendAgileMsg (-1, mntInfoReqp, sizeof(reqMsgBuff),
                MOUNTD_APP_NAME, NFX_SSC_UNIX, SSC_SLOT, MGMTPLANE,
                &timeout,
                respbuff, sizeof(respbuff));
    if (stat < 0)
    {
        printf("sendAgileMsg failed!\n");
        return (NFX_ERR);
    }

    if (mntInfoResp->msgHdr.status != NFX_OK)
    {
        printf("VolId : %lld is not currently exported\n");
    }
	else
	{
        printf("VolId : %lld is currently exported\n");
	}

    return (NFX_OK);
}

/*-----------------------------------------------------------------
 * Name:   	nfs_sscCmdTrace 
 *
 * Description:
 * 		Enables or disables mountd tracing on SSC.
 *
 * Created by : Xiaobing Zhang
 *
 * Date Created: 05/16/2002
 *-----------------------------------------------------------------
 */
int  nfs_sscCmdTrace ( int argc, char *argv[] )
{
	mnt_infoReq_t		mntInfoReq;
	mnt_msgType_t		msgType;
	ssize_t				msgLen;
	struct timeval		timeout = { 10, 0};
	int					stat;

	if (argc < 3)
	{
		nfs_sscCmdHelpCmd("trace");
		return (NFX_OK);
	}

	if (!strncmp(argv[2], "on", 2))
	{
		msgType = MNT_TRACE_ENABLE;
	}
	else if (!strncmp(argv[2], "off", 3))
	{
		msgType = MNT_TRACE_DISABLE;
	}
	else
	{
		nfs_sscCmdHelpCmd("trace");
		return (NFX_OK);
	}

	memset(&mntInfoReq, 0, sizeof(mntInfoReq));
	mntInfoReq.msgHdr.msgType = msgType;
	msgLen = sizeof(mntInfoReq);

	stat = sendAgileMsg (-1, &mntInfoReq, msgLen,
				MOUNTD_APP_NAME, NFX_SSC_UNIX, SSC_SLOT, MGMTPLANE,
				&timeout, NULL, 0);
	if (stat < 0)
	{
		printf("nfs_sscCmdTrace: sendAgileMsg failed!\n");
		return (NFX_ERR);
	}

	return (NFX_OK);
}

/*-----------------------------------------------------------------
 * Name:   	nfs_sscCmdExpRule 
 *
 * Description:
 * 		Display mountd rules and statistics for adding and deleting a share.
 *
 * Created by : Xiaobing Zhang
 *
 * Date Created: 05/23/2002
 *-----------------------------------------------------------------
 */
int  nfs_sscCmdExpRule ( int argc, char *argv[] )
{
	mnt_infoReq_t		mntInfoReq;
	mnt_msgType_t		msgType;
	ssize_t				msgLen;
	struct timeval		timeout = { 10, 0};
	int					stat;

	if (argc < 2)
	{
		nfs_sscCmdHelpCmd("exprule");
		return (NFX_OK);
	}

	msgType = MNT_EXPRULE_DISPLAY;

	memset(&mntInfoReq, 0, sizeof(mntInfoReq));
	mntInfoReq.msgHdr.msgType = msgType;
	msgLen = sizeof(mntInfoReq);

	stat = sendAgileMsg (-1, &mntInfoReq, msgLen,
				MOUNTD_APP_NAME, NFX_SSC_UNIX, SSC_SLOT, MGMTPLANE,
				&timeout, NULL, 0);
	if (stat < 0)
	{
		printf("nfs_sscCmdExpRule: sendAgileMsg failed!\n");
		return (NFX_ERR);
	}

	return (NFX_OK);
}

/*-----------------------------------------------------------------
 * Name:    main
 *
 * Description:
 *      NFS commands main function.
 * Created by:      Joseph Vu
 *
 * Date Created:    08/9/01
 *
 *-----------------------------------------------------------------
 */
int main(int argc, char *argv[])
{
	int	i;

	if (argc < 2)
	{
		nfs_sscCmdHelp(argc, argv);
		exit(0);
	}

	for (i=0; i < NFS_SSC_CMD_MAX; i++)
	{
		if (strcmp(argv[1], nfs_sscCmdTable[i].cmd) == 0)
		{
			(*nfs_sscCmdTable[i].cmdFunc)(argc, argv);
		}
	}

	exit(0);
}

int32 getIpAddr( char *ipAddrStr, uint32 *addr)
{
    byte    ipAddr[4];
    char    *ip;
    char    *p;
    char    c;
    int32   i;

    ip = ipAddrStr;
    p = ip;
    c = *p;
    i = 0;

    while ( i < 4)
    {
        if ((c == '.') || (c == '\0'))
        {
            *p = '\0';
            ipAddr[i] = atoi(ip);
            ip = p+1;
            i++;

            if (i > 3)
                break;
        }
        p++;
        c = *p;
    }

    if (i < 4)
        return -1;

    *addr = (ipAddr[0] << 24) | (ipAddr[1] << 16) |
            (ipAddr[2] <<  8) | (ipAddr[3] );

        return 0;
}

/*-----------------------------------------------------------------
 * Name:   	nfs_sscTotalPktCn 
 *
 * Description:
 *     Return total packet count. 
 *
 * Created by:      Joseph Vu
 *
 * Date Created:    03/9/02
 *
 *-----------------------------------------------------------------
 */
uint64 nfs_sscTotalPktCnt(nfs_pktCounters_t   *pktStatp, uint32 nfs_version)
{
   int32           totalPktCnt = 0;

    if (nfs_version == NFS_VER2)
    {
        totalPktCnt =
            pktStatp->null + pktStatp->getAttr +  pktStatp->setAttr + pktStatp->lookUp +
            pktStatp->readLink + pktStatp->read +  pktStatp->write + pktStatp->create +
            pktStatp->mkdir + pktStatp->symlink + pktStatp->remove + pktStatp->rmdir +
            pktStatp->rename + pktStatp->link + pktStatp->readdir + pktStatp->statfs;
    }
    else
    {
        //NFS V3
        totalPktCnt =
            pktStatp->null + pktStatp->getAttr +  pktStatp->setAttr + pktStatp->lookUp +
            pktStatp->readLink + pktStatp->read +  pktStatp->write + pktStatp->create +
            pktStatp->mkdir + pktStatp->symlink + pktStatp->remove + pktStatp->rmdir +
            pktStatp->rename + pktStatp->link + pktStatp->readdir +
            pktStatp->access +  pktStatp->mknod + pktStatp->readdirplus + pktStatp->fsstat +
            pktStatp->fsinfo + pktStatp->pathconf + pktStatp->commit;
    }

    return (totalPktCnt);
}

#ifdef NFS_USE_TIME_STAMP	
static char *nfsProcName[] = {
                        "NULL",
                        "GETATTR",
                        "SETATTR",
                        "LOOKUP",
                        "ACCESS",
                        "READLINK",
                        "READ",
                        "WRITE",
                        "CREATE",
                        "MKDIR",
                        "SYMLINK",  
                        "MKNOD",
                        "REMOVE",
                        "RMDIR",
                        "RENAME",
                        "LINK",
                        "READDIR",
                        "READDIR+",
                        "STATFS",
                        "FSSTAT",
                        "FSINFO",
                        "PATHCONF",
                        "COMMIT" };

/*-----------------------------------------------------------------
 * Name:   		nfs_sscCmdShowNfsProcTime
 *
 * Description: Display various processing time stamp for NFS Procedures.
 *
 * Created by:  Shaun
 *
 * Date Created: 09/25/02
 *
 *-----------------------------------------------------------------
 */
void nfs_sscCmdShowNfsProcTime( nfs_procTime_t *nfsProcTime )
{
    int32   proc;
    uint64  ave;
    int32   idx;
    int64   totalReqs = 0, totalrxBytes = 0, totaltxBytes = 0 ;
    uint64  ht_bytes_tx;
    
    printf("\n");

    printf("NFS PROC      NFS REQ  LUC-RX  TPL-RX  NFS-RX  FS-RSP  ");
    printf("NFS-TX  TPL-TX  LUC-TX  LONG-RSP  RSP-HWM ");
    printf("      RX BYTES         TX BYTES\n");
    printf("-------------------------------------------------------");
    printf("------------------------------------------");
    printf("-------------------------------\n");

    ht_bytes_tx = nfsProcTime->ht_bytes_tx;

    for (proc =0; proc <= NFS_COMMIT; proc++)
    {
        printf("%9s:%10llu  ", nfsProcName[proc], nfsProcTime->ops);

        for (idx = NFS_TIME_IDX_LUC_RX; idx < NFS_TIME_IDX_LUC_TX_CMP; idx++)
        {
        	ave = (nfsProcTime->ops) ?
            	(eee_rtcToUsec(&(nfsProcTime->duration[idx])) /
				 nfsProcTime->ops): 0;

			printf(" %6llu ", ave);
        }
        printf(" %6u  ", nfsProcTime->rspTooLong);
		printf(" %6llu ", eee_rtcToUsec(&(nfsProcTime->rspHWM)));
        printf(" %15lld ", nfsProcTime->rxBytes);
        printf(" %15lld ", nfsProcTime->txBytes);
        printf("\n");

        totalReqs += nfsProcTime->ops;
        totalrxBytes += nfsProcTime->rxBytes;
        totaltxBytes += nfsProcTime->txBytes;

		nfsProcTime++;
    }

    printf("%9s:%10lld   %88lld  %15lld\n", "Totals", totalReqs, totalrxBytes, totaltxBytes);
    printf("Total bytes Tx over hyper channel to FP: %lld\n", ht_bytes_tx);

}
#endif


/*-----------------------------------------------------------------
 * Name:   	nfs_sscCmdDisplayStat 
 *
 * Description:
 *      Display NFS statistics on the SSC.
 *
 * Created by:      Joseph Vu
 *
 * Date Created:    03/9/02
 *
 *-----------------------------------------------------------------
 */
void nfs_sscCmdDisplayStat(nfs_infoResp_t	*nfs_infoRespMsgp, uint32 clientAddr)
{
	nfs_pktCounters_t		*rxPktStatp;
	nfs_pktCounters_t       *txPktStatp;
	int32					numOfFileSysResVolErrs = 0;

	if (clientAddr)
	{
        printf("\nClient IP: %d.%d.%d.%d\n",
                ((clientAddr >> 24) & 0xff),
                ((clientAddr >> 16) & 0xff),
                ((clientAddr >> 8) & 0xff),
                (clientAddr & 0xff) );
	}
	rxPktStatp = &nfs_infoRespMsgp->msg.stat.nfs_v2RxPktCounters;
	txPktStatp = &nfs_infoRespMsgp->msg.stat.nfs_v2XmtPktCounters;
		
	printf("NFS V2 call counters\n");
	printf("----------------------------\n");
	printf("Null: %lld Getattr: %lld SetAttr: %lld Lookup: %lld\n",
		rxPktStatp->null, rxPktStatp->getAttr, rxPktStatp->setAttr, rxPktStatp->lookUp);
	printf("Readlink: %lld Read: %lld Write: %lld Create: %lld\n",
                rxPktStatp->readLink, rxPktStatp->read, rxPktStatp->write, rxPktStatp->create);
	printf("Mkdir: %lld Symlink: %lld Remove: %lld Rmdir: %lld\n",
                rxPktStatp->mkdir, rxPktStatp->symlink, rxPktStatp->remove, rxPktStatp->rmdir);
	printf("Rename: %lld Link: %lld Readdir: %lld Statfs: %lld\n",
                rxPktStatp->rename, rxPktStatp->link, rxPktStatp->readdir, rxPktStatp->statfs);
	if (clientAddr)
		printf("Total client NFS V2 calls: %lld\n\n", nfs_sscTotalPktCnt(rxPktStatp, NFS_VER2));
	else
		printf("Total NFS V2 calls: %lld\n\n", nfs_sscTotalPktCnt(rxPktStatp, NFS_VER2));

	printf("Client NFS V2 Replies Counters\n");
	printf("----------------------------\n");
	printf("Null: %lld Getattr: %lld SetAttr: %lld Lookup: %lld\n",
            txPktStatp->null, txPktStatp->getAttr, txPktStatp->setAttr, txPktStatp->lookUp);
	printf("Readlink: %lld Read: %lld Write: %lld Create: %lld\n",
            txPktStatp->readLink, txPktStatp->read, txPktStatp->write, txPktStatp->create);
	printf("Mkdir: %lld Symlink: %lld Remove: %lld Rmdir: %lld\n",
            txPktStatp->mkdir, txPktStatp->symlink, txPktStatp->remove, txPktStatp->rmdir);
	printf("Rename: %lld Link: %lld Readdir: %lld Statfs: %lld\n",
            txPktStatp->rename, txPktStatp->link, txPktStatp->readdir, txPktStatp->statfs);
	if (clientAddr)
		printf("Total client NFS V2 replies: %lld\n\n",nfs_sscTotalPktCnt(txPktStatp, NFS_VER2));
	else
		printf("Total NFS V2 replies: %lld\n\n",nfs_sscTotalPktCnt(txPktStatp, NFS_VER2));


	//NFS V3 client
	rxPktStatp = &nfs_infoRespMsgp->msg.stat.nfs_v3RxPktCounters;
	txPktStatp = &nfs_infoRespMsgp->msg.stat.nfs_v3XmtPktCounters;

	printf("Client NFS V3 call counters\n");
	printf("----------------------------\n");
	printf("Null: %lld Getattr: %lld SetAttr: %lld Lookup: %lld\n",
            rxPktStatp->null, rxPktStatp->getAttr, rxPktStatp->setAttr, rxPktStatp->lookUp);
	printf("Access: %lld Readlink: %lld Read: %lld Write: %lld\n",
            rxPktStatp->access, rxPktStatp->readLink, rxPktStatp->read, rxPktStatp->write);
	printf("Create: %lld Mkdir: %lld Symlink: %lld Mknod: %lld\n",
            rxPktStatp->create, rxPktStatp->mkdir, rxPktStatp->symlink, rxPktStatp->mknod);
	printf("Remove: %lld Rmdir: %lld Rename: %lld Link: %lld\n",
            rxPktStatp->remove, rxPktStatp->rmdir, rxPktStatp->rename, rxPktStatp->link);
	printf("Readdir: %lld Readdirplus: %lld Fsstat: %lld Fsinfo: %lld\n",
            rxPktStatp->readdir, rxPktStatp->readdirplus, rxPktStatp->fsstat, rxPktStatp->fsinfo);
	printf("Pathconf: %lld Commit: %lld\n",
            rxPktStatp->pathconf, rxPktStatp->commit);
	if (clientAddr)
		printf("Total client NFS V3 calls: %lld\n\n", nfs_sscTotalPktCnt(rxPktStatp, NFS_VER3));
	else
		printf("Total NFS V3 calls: %lld\n\n", nfs_sscTotalPktCnt(rxPktStatp, NFS_VER3));


	printf("Client NFS V3 Replies Counters\n");
	printf("----------------------------\n");
	printf("Null: %lld Getattr: %lld SetAttr: %lld Lookup: %lld\n",
            txPktStatp->null, txPktStatp->getAttr, txPktStatp->setAttr, txPktStatp->lookUp);
	printf("Access: %lld Readlink: %lld Read: %lld Write: %lld\n",
            txPktStatp->access, txPktStatp->readLink, txPktStatp->read, txPktStatp->write);
	printf("Create: %lld Mkdir: %lld Symlink: %lld Mknod: %lld\n",
            txPktStatp->create, txPktStatp->mkdir, txPktStatp->symlink, txPktStatp->mknod);
	printf("Remove: %lld Rmdir: %lld Rename: %lld Link: %lld\n",
            txPktStatp->remove, txPktStatp->rmdir, txPktStatp->rename, txPktStatp->link);
	printf("Readdir: %lld Readdirplus: %lld Fsstat: %lld Fsinfo: %lld\n",
            txPktStatp->readdir, txPktStatp->readdirplus, txPktStatp->fsstat, txPktStatp->fsinfo);
	printf("Pathconf: %lld Commit: %lld\n",
            txPktStatp->pathconf, txPktStatp->commit);
	if (clientAddr)
		printf("Total client NFS V3 replies: %lld\n\n",nfs_sscTotalPktCnt(txPktStatp, NFS_VER3));
	else
		printf("Total NFS V3 replies: %lld\n\n",nfs_sscTotalPktCnt(txPktStatp, NFS_VER3));

	numOfFileSysResVolErrs = 
	nfs_infoRespMsgp->msg.stat.nfs_v2XmtPktCounters.nfs_fsResourceErr + 
	nfs_infoRespMsgp->msg.stat.nfs_v3XmtPktCounters.nfs_fsResourceErr;

	printf("Total num of file system resource/volume errors: %d\n",
		numOfFileSysResVolErrs);

	printf("Total NFS UDP calls:                             %lld\n", 
		nfs_infoRespMsgp->msg.stat.nfs_v3RxPktCounters.totalUdpCalls);
	printf("Total NFS UDP bad calls:                         %lld\n", 
		nfs_infoRespMsgp->msg.stat.nfs_v3RxPktCounters.totalBadUdpCalls);
	printf("Total NFS TCP calls:                             %lld\n", 
		nfs_infoRespMsgp->msg.stat.nfs_v3RxPktCounters.totalTcpCalls);
	printf("Total NFS TCP bad calls:                         %lld\n", 
		nfs_infoRespMsgp->msg.stat.nfs_v3RxPktCounters.totalBadTcpCalls);


#ifdef NFS_USE_TIME_STAMP	
	/* Set the value of the rtc to usec conversion factor as
	 * returned by the server.
	 */
	count_per_usec = nfs_infoRespMsgp->count_per_usec;
	nfs_sscCmdShowNfsProcTime( &(nfs_infoRespMsgp->procTime[0]) );
#endif

}


/*-----------------------------------------------------------------
 * Name:   	nfs_sscCmdDisplayEntries 
 *
 * Description:
 *      NFS display export entries.
 * 
 * Created by:      Joseph Vu
 *
 * Date Created:    03/9/02
 *
 *-----------------------------------------------------------------
 */
void nfs_sscCmdDisplayEntries(mnt_infoResp_t      *mntInfoResp)
{
	int				numOfEntries;
	int				i;
	mnt_infoEntry_t *mtnInfoEntryp;
    char            *pathNamep;
    char            *exportInfop;
	char            pathName[MAX_NAME_LEN];
	char            exportInfo[MAX_NAME_LEN];


	numOfEntries = mntInfoResp->numOfEntries;
	mtnInfoEntryp = &mntInfoResp->mtnInfoEntry[0];		
	for (i=0; i < numOfEntries; i++)
	{
		memset(&pathName[0], 0, sizeof(pathName));
        memset(&exportInfo[0], 0, sizeof(exportInfo));
		pathNamep = &mtnInfoEntryp->pathName[0];
		exportInfop = pathNamep + mtnInfoEntryp->pathNameLen;
		//printf("nfs_sscCmdDisplayEntries: localAddr: %08x\n", mtnInfoEntryp->localIpAddr);
		//printf("nfs_sscCmdDisplayEntries: hostAddr: %08x\n", mtnInfoEntryp->hostAddr);
		if ((mtnInfoEntryp->pathNameLen > 0) && (mtnInfoEntryp->pathNameLen < MAX_NAME_LEN))
		{
			memcpy(&pathName[0], pathNamep, mtnInfoEntryp->pathNameLen);
			printf("%s\n", &pathName[0]);
		}
		if ((mtnInfoEntryp->exportInfoLen > 0) && (mtnInfoEntryp->exportInfoLen < MAX_NAME_LEN))
		{
        	memcpy(&exportInfo[0],
                exportInfop, mtnInfoEntryp->exportInfoLen);
			printf("\t%s\n", &exportInfo[0]);
		}
		printf("\tState: %s\n\n",
			(mtnInfoEntryp->state == MNT_SHARE_EXPORT_ENABLED)? 
			"Enabled" : "Disabled");
		//Advance to next entry
		MNT_INFO_RESP_ADV_ENTRY_PTR(mtnInfoEntryp);
	}	
	printf("Number of current export entries: %d\n", numOfEntries);
	return;
}
