/* vi:ts=4
 *-----------------------------------------------------------------
 *
 * Name:        nfs.h
 *
 * RCS:  $Id: nfs.h,v 1.79 2002/11/21 00:37:40 rick Exp $
 *
 * Copyright (C) 2001, Agile Storage Inc.
 *
 * Description: Network File System (NFS) definitions
 *
 * Created by:  Joseph Vu
 *
 * Date Created:        5/18/01
 *
 *-----------------------------------------------------------------
 */
#ifndef _NFS_CLIENT_H
#define _NFS_CLIENT_H

#include <exports/eee-api.h>
#include <exports/timer-api.h>
#ifdef NFP_TXRX
#include <exports/pkt-api.h>
#endif

#define NFS_ATTR_IDX_PRE_PARENT         0
#define NFS_ATTR_IDX_POST_PARENT        (NFS_ATTR_IDX_PRE_PARENT+1)
#define NFS_ATTR_IDX_PRE                (NFS_ATTR_IDX_POST_PARENT+1)
#define NFS_ATTR_IDX_POST               (NFS_ATTR_IDX_PRE+1)
#define NFS_ATTR_IDX_PRE_FROM_PARENT    (NFS_ATTR_IDX_POST+1)
#define NFS_ATTR_IDX_POST_FROM_PARENT   (NFS_ATTR_IDX_PRE_FROM_PARENT+1)
#define NFS_ATTR_IDX_PRE_TO_PARENT      (NFS_ATTR_IDX_POST_FROM_PARENT+1)
#define NFS_ATTR_IDX_POST_TO_PARENT     (NFS_ATTR_IDX_PRE_TO_PARENT+1)

#define NFS_MAX_ATTR_IDX        (NFS_ATTR_IDX_POST_TO_PARENT+1)

extern int32   nfs_attrIdxArray[];

/*
 *Constants as defined in the Sun NFS Version 2 and 3 specs.
 */
#define	NFS_PORT	2049		/*NFS default port*/
#define	NFS_PROG	100003
#define	NFS_VER2	2
#define	NFS_VER3	3

#define	TRUE		1
#define	FALSE		0

#define	NFS_NO_ATTR_FLAG_NEEDED		0
#define	NFS_ATTR_FLAG_NEEDED		1
#define	NFS_ATTR_NOT_AVAIL			0
#define	NFS_ATTR_AVAIL				1

#define	NFS_V2_COOKIE_SIZE	4	/*Size of NFS V2 Readdir cookie in bytes*/
#define	NFS_V3_COOKIE_SIZE	8	/*Size of NFS V3 Readdir cookie in bytes*/
#define	NFS_V3_COOKIE_VERF	8	/*Size of NFS V3 cookie verifier in bytes*/

#define	NFS_MIN_PROC	0	
#define	NFS_MAX_V2PROC	18
#define	NFS_MAX_V3PROC	22		/*NFS MAX PROCEDURE NUMBERS*/

#define NFS_MAX_HASH_REQCB		(4*1024)	/* reqCB hash table size */

#define	RPC_VER2	2			/*RPC Version number*/
/*
 *RPC Reply Status
 */
#define	NFS_RPC_MISMATCH		0	/*RPC version mismatch*/
#define	NFS_RPC_AUTH_ERROR		1	/*authentication errror*/

/*
 *RPC Accept status
 */
#define	NFS_RPC_OK				0	/*Remote procdedure invoked successfully*/
#define	NFS_RPC_PROG_UNAVAIL	1	/*server does not recognize program number*/
#define NFS_RPC_PROG_MISMATCH	2	/*Bad program number*/
#define	NFS_RPC_PROC_UNAVAIL	3	/*Bad procedure number*/
#define	NFS_RPC_PROG_BAD_ARGS	4	/*argument cannot be XDR *decode.  */
#define	RPC_PROG_SYS_ERR		5	/*Unknown error on server*/	

#define	NFS_COOKIE_VERF_LEN		8	/*Cookie verifier length in bytes*/

#define	MNT_EXPORT_INFO_MAX		255

/*
 *Our internal assigned error numbers
 */
									//RPC version mismatch
#define NFS_INT_RPC_MISMATCH    	NFS_ERROR_BASE
									//authentication errror
#define NFS_INT_RPC_AUTH_ERROR  	NFS_ERROR_BASE - 1
									//The server does not
									//recognize program number
#define NFS_INT_RPC_PROG_UNAVAIL 	NFS_ERROR_BASE - 2 
									//Bad program number
#define NFS_INT_RPC_PROG_MISMATCH	NFS_ERROR_BASE - 3
								 	//Bad procedure number
#define NFS_INT_RPC_PROC_UNAVAIL    NFS_ERROR_BASE - 4
									//The argument to the remote
									//procedure cannot be XDR decode
#define NFS_INT_RPC_PROG_BAD_ARGS   NFS_ERROR_BASE - 5 
									 //Unknown error on server
#define RPC_INT_PROG_SYS_ERR        NFS_ERROR_BASE - 6
    
#define	NFS_MAX_NUM_OF_ATTR			10

typedef enum nfs_status
{
	NFS_OK			= 0,	/*No Error*/
	NFSERR_PERM		= 1,	/*Not owner.  Caller does not have
					 		 *correct permission
					 		 */
	NFSERR_NOENT            = 2,	/*No such file directory*/	
	NFSERR_IO               = 5,	/*hard error occured-like disk error*/ 
	NFSERR_NXIO             = 6,	/*No such device or address*/
	NFSERR_ACCES            = 13,	/*Permission denied*/
	NFSERR_EXIST            = 17,	/*File already exist*/
	NFSERR_XDEV             = 18,	/*Version 3 only. Attempt to do cross-device
					 				 *hard link */
	NFSERR_NODEV            = 19,	/*No such device*/	
	NFSERR_NOTDIR           = 20,	/*Is not directory*/
	NFSERR_ISDIR            = 21,	/*is a directory*/
	NFSERR_INVAL            = 22,	/*Version 3 only.  Invalid argument*/
	NFSERR_FBIG             = 27,	/*File too large*/
	NFSERR_NOSPC            = 28,	/*No space left*/
	NFSERR_ROFS             = 30,	/*Read-only file system*/
	NFSERR_MLINK            = 31,	/*Version 3 only.  Too many hard links */
	NFSERR_NAMETOL          = 63,	/*Filename was too long*/
	NFSERR_NOTEMPTY         = 66,	/*Remove a dir that was not empty*/
	NFSERR_DQUOT            = 69,	/*Resource (quota) hard limit exceeded*/
	NFSERR_STALE            = 70,	/*Invalid filehandle*/
	NFSERR_REMOTE           = 71,	/* Version 3 only.  Too many levels of remote
					 				 * in path. */
	NFSERR_WFLUSH           = 99,	/* Version 2 only */

										/*The rest are version 3 only*/
	NFSERR_BADHANDLE        = 10001,	/*Illegal NFS filehandle*/
	NFSERR_NOT_SYNC         = 10002,	/*Update synch mismatch was detected 
						 				*during a SETATTR*/
	NFSERR_BAD_COOKIE       = 10003,	/*READDIR/READDIRPLUS cookie is stale*/
	NFSERR_NOTSUPP          = 10004,	/*Operation is not supported*/
	NFSERR_TOOSMALL         = 10005,	/*Buffer or request is too small*/
	NFSERR_SERVERFAULT      = 10006,	/*A generic error-default error*/
	NFSERR_BADTYPE          = 10007,	/*Object type not supported*/	
	NFSERR_JUKEBOX          = 10008,	/*Server was not able to complete the
						 				 *the request in a timely fashion.  
						 				 *The client should wait and then try
						 				 *the request with a new RPC XID*/
} nfs_status_t; 

/*
 *NFS creates a file
 */
#define	NFS_CREATED_UNCHECKED		0	/*No error if file is already exist*/
#define	NFS_CREATED_GUARDED			1	/*Error if file is already exist*/
#define	NFS_CREATED_EXCLUSIVE		2	/*No error if file already exist and verfier
						 				 *equals existing cookie for the file.  
						 				 *No attributes is provided in this case
						 				 */
/*
 *Our own errors
 */
#define	NFSERR_ADD_FILECB		(NFS_ERROR_BASE+0)	
#define	NFSERR_FILENAME_TLONG	(NFS_ERROR_BASE+1)	
#define	NFSERR_MAP_FS_MSG		(NFS_ERROR_BASE+2)	
#define	NFSERR_SENDMSG_TO_FS	(NFS_ERROR_BASE+3)	
#define	NFSERR_SETATTR			(NFS_ERROR_BASE+4)	
#define	NFSERR_CREATE			(NFS_ERROR_BASE+5)	
#define	NFS_FILESYS_NOT_READY	(NFS_ERROR_BASE+6)	

/*
 *File Types
 *Look fs_FileType in fs-api.h
 */
typedef enum 
{
	NFS_FTYPE_NFNON = 0,		/*Non-file*/
	NFS_FTYPE_NFREG = 1,		/*Regular file*/
	NFS_FTYPE_NFDIR = 2,		/*Directory*/
	NFS_FTYPE_NFBLK = 3,		/*Block-special file*/
	NFS_FTYPE_NFCHR = 4,		/*Character-special file*/
	NFS_FTYPE_NFLNK = 5,		/*Symbolic link*/
	NFS_FTYPE_NFSOCK = 6,		/*Socket*/
	NFS_FTYPE_NFFIFO = 7		/*Named pipe*/
} nfs_ftypes;

/*
 *Specific to NFS Verison 2
 */
#define	NFS_V2_FHANDLE_LEN	32
#define	NFS_V2_COOKIE		4
/*
 *Specific to NFS Version 3
 */
#define	NFS_V3_COOKIE		8
#define	NFS_V3_COOKIE_VERF	8

/*
 * Maxmium number of retransmitted requests accepted
 */
#define NFS_MAX_REXMT_REQ	5

typedef enum nfs_stableHow
{
	NFS_WRITE_UNSTABLE = 0,
	NFS_WRITE_DATASYNC = 1,
	NFS_WRITE_FILESYNC =2
} nfs_stableHow_t;

/* 
 *NFS procedure numbers 
 */
typedef enum nfs_proc
{
	NFS_V3_PROC_NULL            = 0,	/*Do nothing*/
	NFS_V3_PROC_GETATTR         = 1,	/*Get attributes for file/dir*/
	NFS_V3_PROC_SETATTR         = 2,	/*Set attributes for file/dir*/
	NFS_V3_PROC_LOOKUP          = 3,	/*lookup attributes of a file*/
	NFS_V3_PROC_ACCESS          = 4,	/*check access permission on object*/
	NFS_V3_PROC_READLINK        = 5,	/*read a soft link*/
	NFS_V3_PROC_READ            = 6,	/*read data from a file*/
	NFS_V3_PROC_WRITE           = 7,	/*write data to a file*/
	NFS_V3_PROC_CREATE          = 8,	/*create a new file*/
	NFS_V3_PROC_MKDIR           = 9,	/*create a new directory*/
	NFS_V3_PROC_SYMLINK         = 10,	/*create a soft link*/
	NFS_V3_PROC_MKNOD           = 11,	/*create a device file*/
	NFS_V3_PROC_REMOVE          = 12,	/*remove a file*/
	NFS_V3_PROC_RMDIR           = 13,	/*remove a directory*/
	NFS_V3_PROC_RENAME          = 14,	/*rename a file/directory*/
	NFS_V3_PROC_LINK            = 15,	/*create a hard link to a file*/
	NFS_V3_PROC_READDIR         = 16,	/*read a directory contents*/
	NFS_V3_PROC_READDIRPLUS     = 17,	/*read a directory's contents & attributes*/
	NFS_V3_PROC_FSSTAT          = 18, 	/*get filesystem statistic*/
	NFS_V3_PROC_FSINFO          = 19,	/*get statistic filesystem information*/
	NFS_V3_PROC_PATHCONF        = 20,	/*get POSIX information*/
	NFS_V3_PROC_COMMIT          = 21	/*commit write data to stable storage*/
} nfs_proc_t;


/* Actual Version 2 procedure numbers */
typedef enum nfs_v2Proc
{
	NFS_V2_PROC_NULL        = 0,	/*Do nothing*/
	NFS_V2_PROC_GETATTR     = 1,	/*Get attributes for file/dir*/
	NFS_V2_PROC_SETATTR     = 2,	/*Set attributes for file/dir*/
	NFS_V2_PROC_ROOT        = 3,	/* Obsolete */
	NFS_V2P_PROC_LOOKUP     = 4,	/*lookup attributes of a file*/
	NFS_V2_PROC_READLINK    = 5,	/*read a soft link*/
	NFS_V2_PROC_READ        = 6,	/*read data from a file*/
	NFS_V2_PROC_WRITECACHE  = 7,	/* Obsolete */
	NFS_V2_PROC_WRITE	= 8,	/*write data to a file*/
	NFS_V2_PROC_CREATE      = 9,	/*create a new file*/
	NFS_V2_PROC_REMOVE      = 10,	/*remove a file*/
	NFS_V2_PROC_RENAME      = 11,	/*rename a file/directory*/
	NFS_V2_PROC_LINK        = 12,	/*create a hard link to a file*/
	NFS_V2_PROC_SYMLINK     = 13,	/*create a soft link*/
	NFS_V2_PROC_MKDIR       = 14,	/*create a new directory*/
	NFS_V2_PROC_RMDIR       = 15,	/*remove a directory*/
	NFS_V2_PROC_READDIR     = 16,	/*read a directory contents*/
	NFS_V2_PROC_STATFS      = 17	/*get filesystem statistic*/
} nfs_v2Proc_t;

/*
 *define for NFS decoding functions
 */
#define NFS_DECODE_FILENAME     1
#define NFS_DECODE_PATHNAME     2
#define NFS_DECODE_EXPORTINFO   3
#define NFS_DECODE_HOSTNAME     4

#define NFS_DECODE_RX       1
#define NFS_DECODE_XMT      2

#define NFS_DECODE_DIRCOUNT 1
#define NFS_DECODE_MAXCOUNT 2

/*
 *NFS Version 2 STATFS: filsystem attributes
 */
typedef struct nfs_v2Statfs
{
	uint32	nfs_tsize;	/*Optimum transfer size of the server in bytes*/
	uint32	nfs_bsize;	/*Block size in bytes of the filesystem*/
	uint32	nfs_blocks;	/*Total number of bsize blocks on the filesystem*/
	uint32	nfs_bfree;	/*Total number of free bsize blocks on the filesystem*/
	uint32	nfs_bavail;	/*Numberof bsize blocks available to nonprivillege users*/
} nfs_v2Statfs_t;

/*
 *NFS Version 3 FSSTAT: filesystem attributes
 */
typedef struct fs_v3Fsstat 
{
	/*
	 *TBD:  obj_attr
	 */	
	uint64	nfs_tbytes;	/*The total size, in bytes, of the filesystem*/
	uint64	nfs_fbytes;	/*The amount of free space, in bytes, in the filesystem*/
	uint64	nfs_abytes;	/*The amount of free space, in bytes, available to the user
				 		 *identified by the authentication info in the RPC.
				 		 */
	uint64	nfs_tfiles;	/*The total number of file slots in the filesystem*/
	uint64	nfs_ffiles;	/*The number of free file slots in the filesystem*/
	uint64	nfs_afiles;	/*The number of free file slots that available to the user 
				 		 *corresponding to the authentication info in the RPC.
				 		 */
	uint32	nfs_invarsec;	/*A measure of filesystem volatility:  This is the number of 
				 			 *seconds for which the filesystem not supposed to change.
				 			 */	

} nfs_v3Fsstat_t;
/*
 *STATFS/FSSTATFS: File System Attributes
 */
typedef struct nfs_statInfo
{
        union {
		nfs_v2Statfs_t	nfs_v2Statfs;	/*NFS Version 2 filesystem attributes*/
		nfs_v3Fsstat_t	nfs_v3Statfs;	/*NFS Version 3 filesystem attributes*/
        } version;
} nfs_statInfo_t;

/*
 *FSINFO-Static Filesystem Information
 */
typedef struct nfs_fsInfo
{
	/*TBD: obj_attr:  The attributes of the filesystem*/
	uint32	nfs_rtmax;	/*The maximum size in bytes of a READ supported by server*/
	uint32	nfs_rtpref;	/*The preferred size in bytes of a READ request*/
	uint32	nfs_rtmult;	/*The suggested multiple for the size of READ request*/
	uint32	nfs_wtmax;	/*The max size in bytes of a WRITE request supported by
				 *the server.
				 */
	uint32	nfs_wtpref;	/*The preferred size in bhytes of a WRITE request*/
	uint32	nfs_wtmult;	/*The suggested multiple for the size of Write request*/
	uint32	nfs_dtpref;	/*The preffered size in bytes of a READDIR request*/
	uint64	nfs_maxFileSize; /*The maximum size in bytes of a file on the filesystem*/
	fs_Time	nfs_timeDelta;	/*The server time granularity.  When setting a file time using
				 *SETATTR, the server guarantee only to preserve times to 
				 *this accuracy.   If this {0,1}, the server can support
				 *nanosecond times; {0, 1000000} denotes millisecond precision,
				 *and [1,0] indicates the times are accurate only to 
				 *nearest second.
				 */
	uint32	properties;
} nfs_fsInfo_t;


typedef struct nfs_pathConf
{
	/*TBD:  Object Post OP attributes*/
	uint32		nfs_linkMax;	//Max number of hard links to an object
	uint32		nfs_nameMax;	//The max length of a component of a filename
	uchar8		nfs_noTrunc;	//if TRUE, the server will reject any request
						 		// that includes a name longer than nfs_nameMax
				 				//with error NFS_ERR_NAMETOOLONG.  if FALSE,
						 		// any length name nfs_nameMax bytes will 
						 		//silently truncated to nfs_nameMax
						 			 
	uchar8		nfs_chownRestricted; //if TRUE, the server will reject any request
						 			 //to change either the owner or the group 
						 			 //associated with a file if the caller is
						 			 //not the privillege user (uid 0)	
	uchar8		nfs_caseInsensitive; 	//if TRUE, the server filesystem does not 
						 				//distinguish case when interpreting filenames
						 				//	
	uchar8		nfs_casePreserving;
} nfs_pathConf_t;

typedef struct nfs_statV3Counters_t 
{
	uint32	nfs_errRemote;		/*Too many levels of remote path*/
	uint32	nfs_errBadFhandle;	/*Illegal NFS filehandle*/
	uint32	nfs_errNotSync;		/*Update synchronization mismatch was detected
					 			 *during the SETATTR operation
					 			 */
	uint32	nfs_errBadCookie;	/*READDIR or READDIRPLUS cookie is stale*/
	uint32	nfs_errNotSupp;		/*Operation is not supported*/
	uint32	nfs_errTooSmall;	/*Buffer or request is too small*/
	uint32	nfs_errServerFault;	/*An error occurred on the server that does not
					 			 *map to any legal NFS version 3 protocol error.
					 			 */
	uint32	nfs_errBadType;		/*An attemp was made to create an object 
					 			 *or type not supported by the server.
					 			 */
	uint32	nfs_errJukeBox;		/*Server initiated the request was not able to
					 			 *complete it in a timely fashion.  Client
					 			 *should wait and then try the request a new 
					 			 *RPC XID
					 			 */
	uint32	nfs_errXdev;		/*Attempt to do a cross-device hard link*/
	uint32	nfs_errInvalid;		/*Invalid or unsupported argument*/
	uint32	nfs_errMlink;		/*Too many hard links*/
} nfs_statV3Counters_t;

typedef struct nfs_statV4Counters 
{
	uint32	nfs_errSame;			/*nverify say attrs same*/
	uint32	nfs_errDenied;			/*lock unavailable*/
	uint32	nfs_errExpired;			/*lock lease expired*/
	uint32	nfs_errLocked;			/*I/O failed due to lock*/
	uint32	nfs_errGrace;			/*In Grace period*/
	uint32	nfs_errFHExpired;		/*Filehandle expired*/
	uint32	nfs_errShareDenied;		/*Share reserve denied*/
	uint32	nfs_errWrongSec;		/*Wrong security flavor*/
	uint32	nfs_errClidInused;		/*client ID in-used*/
	uint32	nfs_errResource;		/*Resource exhaustion*/
	uint32	nfs_errMoved;			/*Filesystem relocated*/
	uint32	nfs_errNoFhandle;		/*Current FH is not set*/
	uint32	nfs_errVersMismatch;	/*minor vers not supp*/
	uint32	nfs_errStateClid;		/*Stale Client ID*/
	uint32	nfs_errStaleStateId;	/*Stale State ID*/
	uint32	nfs_errOldStateId;		/*Old State ID*/
	uint32	nfs_errBadStateId;		/*Bad State ID*/
	uint32	nfs_errBadSeqId;		/*Bad Sequence ID*/
	uint32	nfs_errNotSame;			/*Verify - attrs not same*/
	uint32	nfs_errLockRange;		/*Invalid lock range*/
	uint32	nfs_errSymlink;			/*Invalid symbolic link*/
	uint32	nfs_errReaddirNospc;
	uint32	nfs_errLeaseMoved;
} nfs_statV4Counters_t;


typedef struct nfs_statCounters
{
	uint64	nfs_ok;			/*No error*/
	uint64	nfs_errPerm;	/*Permission error*/
	uint64	nfs_errNoEnt;	/*No such file or directory*/
	uint64	nfs_errIO;		/*Some of hard error occured when the operation
					 		 *was in progress (ex:  a disk error)
					 		 */
	uint64	nfs_errNxio;		/*No such device or address*/
	uint64	nfs_errAccess;		/*Permission denied*/
	uint64	nfs_errFileExist;	/*File Exists*/
	uint64	nfs_errNoDev;		/*No such device*/
	uint64	nfs_errNotDir;		/*Is not directory.  The caller specified a non-directory
					 			 *in a directory operation
					 			 */
	uint64	nfs_errIsDir;		/*Is a directory.  The caller specified a non-directory
					 			 *in a directory operation.
					 			 */
	uint64	nfs_errInVal;		/*Invalid argument or unsupported argument for an op*/
	uint64	nfs_errFileBig;		/*File too large*/
	uint64	nfs_errNoSpc;		/*No space left on device*/
	uint64	nfs_errRofs;		/*Read-only filesystem*/
	uint64	nfs_errMlink;		/*Too many hard links*/
	uint64	nfs_errNameTooLong;	/*Filename too long*/
	uint64	nfs_errNotEmpty;	/*Directory not empty.  Attempted to remove a dir.
					 			 *that was not empty.
					 			 */
	uint64	nfs_errDquota;			/*Disk quota exceed*/
	uint64	nfs_errStaleFhandle;	/*Error or invalid filehandle*/
	uint64	nfs_errRemote;			/*Too many levels in remote in path*/
	uint64	nfs_errFlush;
	uint64	nfs_errBadHandle;
	uint64 	nfs_errNotSync;
	uint64 	nfs_errBadCookie;
	uint64	nfs_errNotSupp;
	uint64	nfs_errGeneric;	
	union
	{
		nfs_statV3Counters_t	nfs_v3Counters;	/*NFS Version 3 stat. counters*/
		nfs_statV4Counters_t	nfs_v4Counters;	/*NFS Version 4 stat. counters*/	
	} version;		
} nfs_statCounters_t; 


typedef struct nfs_pktCounters
{
		uint64	null;
        uint64  getAttr;     	
        uint64  setAttr;     
        uint64  lookUp;      
        uint64  access;     
        uint64  readLink;    
        uint64  read;        
        uint64  write;      
        uint64  create;    
        uint64  mkdir;    
        uint64  symlink; 
        uint64  mknod;  
        uint64  remove;
        uint64  rmdir;   
        uint64  rename; 
        uint64  link;  
        uint64  readdir;
        uint64  readdirplus;
        uint64  statfs;   
        uint64  fsstat; 
        uint64  fsinfo;
        uint64  pathconf;
        uint64  commit; 
		uint64	totalUdpCalls;
		uint64	totalBadUdpCalls;
		uint64	totalTcpCalls;
		uint64	totalBadTcpCalls;
		uint64	nfs_fsResourceErr;		/*when receive a file system volume/resouce error */
} nfs_pktCounters_t;

typedef struct nfs_debugCounters 
{
	uint64				tpl_xmtPktAlloc;
	uint64				tpl_xmtPktFree;
	uint64				tpl_noPktRes;
	uint64				tpl_xmtPktSend;
	uint64				tpl_xmtPktCmpInd;
	uint64				nfs_readXmtReq;
	uint64				nfs_readXmtCmpInd;
	uint64				nfs_readXmtCmpInd2;
	uint64				nfs_dropFsResp;
	uint64				nfs_fileSysNotMnt;
	uint64				tpl_badDesc;
	uint64				nfs_eeeRamAlloc1;
	uint64				nfs_eeeRamDeAlloc1;
	uint64				nfs_eeeRamAlloc2;
	uint64				nfs_eeeRamDeAlloc2;
	uint64				nfs_eeeRamAlloc3;
	uint64				nfs_eeeRamDeAlloc3;
	uint64				nfs_eeeRamAlloc4;
	uint64				nfs_eeeRamDeAlloc4;
	uint64				nfs_eeeRamAlloc5;
	uint64				nfs_eeeRamDeAlloc5;
	uint64				nfs_fsResourceErr;
	uint64				nfs_invalidTplPktRcvd;
    uint64              nfs_allocReqCB;
    uint64              nfs_freeReqCB;
	uint64				nfs_fsNoRespRcvdErr;
	uint64				nfs_rxInvalidFSResp;		
	uint64				nfs_rxInvalidResp;
	uint64              nfs_maxReqsDropPktErr;
	uint64				nfs_getFileSysByIdErr;
	uint64				nfs_rxPktIfErr;
	uint64				nfs_rxPktNfsShareErr;
	uint64				nfs_tplRxPktHold;
	uint64				nfs_tplRxPktFree;
	uint64				nfs_tplUDPConnActTimerErr;
	uint64				nfs_tplNoRespRcvdErr;
    nfs_pktCounters_t   nfs_PktFsXmtCounters;
    nfs_pktCounters_t   nfs_PktFsRxCounters;
    nfs_pktCounters_t   nfs_PktRxRetriesCounters;
} nfs_debugCounters_t;


typedef struct nfs_clientInfo
{
	uint32					nfs_version;			/*NFS version number*/
	nfs_statCounters_t		nfs_statCounters;		/*NFS Procedure status counters*/
#ifdef	NOTUSED
	nfs_pktCounters_t		nfs_xmtPktCounters;		/*NFS transmit pkt counters*/
	nfs_pktCounters_t		nfs_rxPktCounters;		/*NFS receive rx counters*/
#endif	//NOTUSED
	nfs_pktCounters_t		nfs_v2XmtPktCounters;	//NFS V2 transmit pkt counters
	nfs_pktCounters_t		nfs_v2RxPktCounters;	//NFS V2 receive rx counters
	nfs_pktCounters_t		nfs_v3XmtPktCounters;	//NFS V2 transmit pkt counters
	nfs_pktCounters_t		nfs_v3RxPktCounters;	//NFS V2 receive rx counters
} nfs_clientInfo_t;

typedef struct nfs_globalCfg
{
	int32				reqTimeoutHandle;
//#define	NFS_REQ_TIMEOUT_PERIOD	5000			//5 seconds
#define		NFS_REQ_TIMEOUT_PERIOD	100			
	int32				txReqRetryHandle;		
#define	NFS_REQ_RETRY_PERIOD	200				
	uint32				eee_efsPortId;			//file system EFS Port ID
	uint32   			eee_nfsPortId;			//NFS Svr EEE Port Id
	int32				eee_efsAppId;			//file system EFS app ID
	int32   			eee_nfsAppId;			//NFS Svr EEE app Id
	int32				eee_nfsCmdAppId;
	nfs_statCounters_t	nfs_statCounters;		//NFS Procedure status counters
	nfs_pktCounters_t	nfs_xmtPktCounters;		//NFS transmit pkt counters
	nfs_pktCounters_t	nfs_rxPktCounters;		//NFS receive rx counters
	nfs_pktCounters_t	nfs_v2XmtPktCounters;	//NFS V2 transmit pkt counters
	nfs_pktCounters_t	nfs_v2RxPktCounters;	//NFS V2 receive rx counters
	nfs_pktCounters_t	nfs_v3XmtPktCounters;	//NFS V2 transmit pkt counters
	nfs_pktCounters_t	nfs_v3RxPktCounters;	//NFS V2 receive rx counters
	int32				nfs_cacheTimeCnt;
	nfs_debugCounters_t	nfs_debugPktCounters;		
#define	NFS_CACHE_TIME_MAX_CNT		200	
#define	NFS_MNT_PNT_ACT_TIME_CNT	600			//60 Secs
#define	NFS_MNT_PNT_INACT_PERIOD	(SEC_PER_MIN * 20)		//20 minutes inactivity time
						//Adjust this if NFS_REQ_TIMEOUT_PERIOD is changed
	uint32				nfs_activityTimeCnt;
	uint32				nfs_BootUpTimeCnt;
#define NFS_BOOTUP_TIME_CNT      1800			//180 secs or 3 mins
												//Adjust this if NFS_REQ_TIMEOUT_PERIOD 
												//is changed
	int32				nfs_maxReqsThreshold;
#define	NFS_MIN_REQ_THRESHOLD		1000
#define	NFS_MAX_REQ_THRESHOLD		(NFP_MAX_REQUEST - NFS_MIN_REQ_THRESHOLD)

	uint64              nfs_totalNumOfReqXmtToFs;
    uint64              nfs_totalNumOfRespRcvdFromFs;
	uint32				nfs_tplUdpActTimeOutHandle;		
#define NFS_TPL_UDP_ACT_TIMOUT_PERIOD	30000				//30 seconds  
	void				*pReqCbHashTbl;			// Req CB hash table ptr
	void				*pReqCbList;			// list of Req CB in use
}  nfs_globalCfg_t;

extern nfs_globalCfg_t  *nfs_globalCfgp;
extern uint32  trace_enable;
/*
 * Major/minor number pair.
 */
typedef struct nfs_devNum
{
        uint32          major;
        uint32          minor;

} nfs_devNum_t;

typedef struct nfsV2_Time
{
    uint32      secs;
    uint32      usecs;
} nfsV2_Time;

typedef struct nfsV3_Time
{
    uint32      secs;
    uint32      nsecs;
} nfsV3_Time;

typedef struct nfs_v2SpecificFattr
{
	uint32		ftype;
	uint32		mode;
	uint32		nlink;
	uint32		uid;
	uint32		gid;
	uint32  	faSize;			/*File size in bytes*/
	uint32  	faBlocksize;    /*Size of disk block in bytes*/
	uint32  	faRdev;         /*Device no. of a file*/
	uint32  	faBlocks;       /*No. of disk blocks used*/
	uint32  	faFsid;         /*Identifier of filesystem*/
	uint32  	faFileid;       /*Identifier of file in filesystem*/
	nfsV2_Time 	faAtime;        /*Last access time*/
	nfsV2_Time 	faMtime;        /*Last modification time*/
	nfsV2_Time 	faCtime;        /*Attribute modification time*/
} nfs_v2SpecificFattr_t;

typedef struct nfs_v3SpecificFattr
{
	uint32		ftype;
	uint32		mode;
	uint32		nlink;
	uint32		uid;
	uint32		gid;
	uint32  	faSize[2];      /*file size in bytes*/
	uint32  	faUsed[2];      /*Disk space used*/
	nfs_devNum_t	faRdev;     /*File device info*/
	uint32  	faFsid[2];      /*Filesystem ID*/
	uint32  	faFileid[2];    /*File no. within filesystem*/
	nfsV3_Time 	faAtime;        /*Last access time*/
	nfsV3_Time 	faMtime;        /*Last modify time*/
	nfsV3_Time 	faCtime;        /*Last attribute change time*/
} nfs_v3SpecificFattr_t;

/*
 * File attributes and setable attributes. These structures cover both
 * NFS version 2 and the version 3 protocol. 
 */
typedef struct nfs_fattr
{
	uint32		faType;		/*Type of a file*/
	uint32		faMode;		/*Protection mode bits*/
#define	NFS_FA_MODE_NFDIR 	0x0040000	/*Type field should be NFDIR*/
#define	NFS_FA_MODE_NFCHR 	0x0020000	/*Type field should be NFCHR*/
#define	NFS_FA_MODE_NFBLK 	0x0060000	/*Type field should be NFBLK*/
#define	NFS_FA_MODE_NFREG 	0x0100000	/*Type field should be NFREG*/
#define	NFS_FA_MODE_NFLNK 	0x0120000	/*Type field should be NFLNK*/
#define	NFS_FA_MODE_NFNON 	0x0140000	/*Type filed should be NFNON*/
#define	NFS_FA_MODE_SETUID 	0x0004000	/*Set user ID on execution*/
#define	NFS_FA_MODE_SETGID 	0x0002000	/*Set group ID on execution*/
#define	NFS_FA_MODE_DONOTCACHE	0x0001000	/*On directories, restricted deletion
						 					 *flag.  On regular files, do-not-cache
						 					 *flag
						 					 */
#define	NFS_FA_MODE_OWNER_READ 	0x0000400	/*Read permision for owner*/
#define	NFS_FA_MODE_OWNER_WRITE	0x0000200	/*Write permssion for owner*/
#define NFS_FA_MODE_OWNER_EXEC	0x0000100	/*Execute permssion for owner*/
#define	NFS_FA_MODE_GRP_READ	0x0000040	/*Read permission for group*/
#define	NFS_FA_MODE_GRP_WRITE	0x0000020	/*Write permission for group*/
#define	NFS_FA_MODE_GRP_EXEC	0x0000010	/*Execute permission for group*/
#define	NFS_FA_MODE_OTHER_READ	0x0000004	/*Read permission for others*/
#define	NFS_FA_MODE_OTHER_WRITE	0x0000002	/*Write permission for others*/
#define	NFS_FA_MODE_OTHER_EXEC	0x0000001	/*Execute permission for others*/
	uint32		faLink;						/*Number of hard links to the file.
						 					 *No. of different names for same file.
						 					 */
	uint32		faUid;						/*User ID of the owner of the file*/
	uint32		faGid;						/*Group ID of the group of the file*/
	uint32		reserved;
	union
	{
		nfs_v2SpecificFattr_t	v2Fattr;	
		nfs_v3SpecificFattr_t	v3Fattr;	
	} version;
} nfs_fattr_t;

/*
 *NFS V2 Setattr (Set attributes)
 *A value of -1 indicates field should be ignored.
 */
typedef struct nfs_v2Sattr
{
	uint32	mode;		/*Permission bits*/
	uint32	uid;		/*User ID*/
	uint32	gid;		/*Group ID*/
	uint32	size;		/*Size of file in bytes*/
	fs_Time	atime;		/*Last access time*/
	fs_Time	mtime;		/*Last modification time*/
} nfs_v2Sattr_t;

/*
 *NFS V3 Setattr (Set attributes)
 */
typedef struct nfs_v3Sattr
{
	uint32	modeTrue;	/*TRUE = 1, FALSE = 0*/
	uint32	mode;		/*Permission bits*/
	uint32	uidTrue;
	uint32	uid;		/*User ID*/
	uint32	gidTrue;
	uint32	gid;		/*Group ID*/
	uint32	sizeTrue;
	uint32	size;		/*Size of file in bytes*/
	uint32	atimeType;
#define	NFS_SATTR_DONT_CHANGE	0
#define	NFS_SATTR_SERVER_TIME	1
#define	NFS_SATTR_CLIENT_TIME	2
	fs_Time	atime;		/*Last access time*/
	uint32	mtimeType;
	fs_Time	mtime;		/*Last modification time*/
}  nfs_v3Sattr_t;

/*
 *NFS Version 3: Pre-operation attributes
 */
typedef struct nfs_preOpAttr
{
	uint32  	faSize[2];      /*file size in bytes*/
	fs_Time 	faMtime;        /*Last modify time*/
	fs_Time 	faCtime;        /*Last attribute change time*/
} nfs_preOpAttr_t;

typedef struct nfs_hardlink
{
	/*TBD*/

} nfs_hardlink_t;

typedef struct nfs_fileInfo
{
	fs_Attr		fs_attr;				/*cache file attributes*/
	uint32		nfs_flag;
#define NFS_CACHE_FATTR		0x00000001 	/*cache file attributes */
#define	NFS_CACHE_SYMLINK	0x00000002	/*cache symbolic link*/
#define	NFS_CACHE_ACCESS_PERM	0x00000004	/*cache file access permission*/
#define	NFS_CACHE_LOOKUP	0x00000008	/*cache lookup filename*/
	uint32		nfs_version;			/*NFS Version number*/
	ushort16	reserved;
	ushort16	nfs_nameLen;			//cache filename length in bytes, 
										//Initialize (clear) periodically
	ushort16	nfs_filenameLen;		//filename entry, not cache entry.
										//Will be valid until file/dir is removed
	uint32		nfs_accessPerm;			/*NFS Permission*/
	nfs_hardlink_t	*nfs_hardlinkp;		/*Hard link entries*/
	uchar8		*nfs_namep;				//cache pointer to filename, dir name,
										//initialize (clear) periodically
	uchar8		*nfs_fileNamep;			//Pointer to filename, not cache entry.
										//It will be valid until file/dir is removed 
	char		nfs_writeVerf[NFS_COOKIE_VERF_LEN];
	fs_Time		cacheTime;
} nfs_fileInfo_t;


typedef struct nfs_fileSysInfo
{
        uint32                  nfs_flag;
#define NFS_CACHE_FATTR         0x00000001              /*cache attributes */
#define NFS_CACHE_STATFS        0x00000002              /*cache statfs info*/
#define	NFS_CACHE_EXPORT		0x00000004
        uint32                  nfs_searchNameKey;      /*Convert name key for directory
                                                         *based on dir pathname
                                                         */
		uint32                  exportOptionMask;
        nfs_fattr_t             attr;                   /*attributes of object*/
		fs_Attr					fs_attr;
        nfs_statInfo_t          nfs_statInfo;           /*stats info*/
        nfs_fsInfo_t            nfs_fsInfo;             /*FSINFO: Static filesystem info*/
        nfs_pathConf_t          nfs_pathConf;           /*POSIX information*/
        uint32                  exportInfoLen;
        char                    exportInfoStr[MNT_EXPORT_INFO_MAX];  /*FIX THIS LATER, USING VAR LEN ARRAY*/
} nfs_fileSysInfo_t;

/*--------------------------------------------------------------------------*/


typedef struct nfs_rpcAuthInfo
{
        uint32          authtype;       /* auth type */
#define NFS_RPC_AUTH_NONE       0       /*NULL authentication*/
#define NFS_PRC_AUTH_SYS        1       /*Same as AUTH_UNIX: UID, GID, groups.*/
#define NFS_RPC_AUTH_SHORT      2       /*Token authenticator for AUTH_SYS*/
#define NFS_RPC_AUTH_DH         3       /*Deffie-Hellman exchange of DES key*/
#define NFS_RPC_AUTH_KERB4      4       /*Kerberos ver 4 exchange of DES key*/
#define NFS_RPC_RSA             5
#define NFS_RPCSEC_GSS          6       /*Generic sercurity services API*/
        uint32          authlen;        /* auth length */
	/*
	 *AUTH_SYS credential
	 */
	uint32		userId;
	uint32		groupId;
	/*
	 *TODO:  Look at Gid Group
	 */
	uint32		auxGidCount;
#define	NFS_AUTH_SYS_GID_COUNT	16
	uint32		auxGid[NFS_AUTH_SYS_GID_COUNT];	
} nfs_rpcAuthInfo_t;

/*
 *reqType field for request block (nfs_request_t)
 */
#define	NFS_NULL	0
#define	NFS_GETATTR	1
#define	NFS_SETATTR	2
#define	NFS_LOOKUP	3
#define	NFS_ACCESS	4
#define	NFS_READLINK	5
#define	NFS_READ	6
#define	NFS_WRITE	7
#define	NFS_CREATE	8
#define	NFS_MKDIR	9
#define	NFS_SYMLINK	10
#define	NFS_MKNOD	11
#define	NFS_REMOVE	12
#define	NFS_RMDIR	13
#define	NFS_RENAME	14
#define	NFS_LINK	15
#define	NFS_READDIR	16
#define	NFS_READDIRPLUS	17
#define	NFS_STATFS	18
#define	NFS_FSSTAT	19
#define	NFS_FSINFO	20
#define	NFS_PATHCONF	21
#define	NFS_COMMIT	22

/*
 *Attributes type field
 */
#define	NFS_ATTR_TYPE		1
#define	NFS_ATTR_MODE		2
#define	NFS_ATTR_NLINK		3
#define	NFS_ATTR_UID		4
#define	NFS_ATTR_GID		5
#define	NFS_ATTR_SIZE		6
#define	NFS_ATTR_BLOCKSIZE	7
#define	NFS_ATTR_USED		8	
#define	NFS_ATTR_RDEV		9	
#define	NFS_ATTR_BLOCKS		10	
#define	NFS_ATTR_FSID		11	
#define	NFS_ATTR_FILEID		12
#define	NFS_ATTR_ATIME		13
#define	NFS_ATTR_CTIME		14
#define	NFS_ATTR_MTIME		15

#define	RX_STAT		1		/*RX Statistic*/
#define	TX_STAT		2		/*TX Statistic*/
#define	RESULT_STAT	3		/*Procedure result statistic*/

typedef struct nfs_readXmitCompInd
{
	fs_Offset           fileOffset;     /* Byte offset to start read action */
	uint32              bytesRead;      /* Number of bytes actually read */
	uint64				fsPrivate;		/* Private */
} nfs_readXmitCompInd_t;

#define	NFS_MAX_CACHE_REQ		6	//Duplicate requests cache 
									//max entries-per client
#define NFS_MAX_FREE_ENTRIES   32 

#define	NFS_TX_REG_MSG_TYPE		1
#define	NFS_TX_CACHE_MSG_TYPE	2
#define	NFS_TX_ERR_MSG_TYPE		3
#define	NFS_TX_SUNACL_MSG_TYPE	4

#define	NFS_TPL_CONN_ACT_TIMEOUT	(60 * 10) //(60 secs*10): 10 mins

/*
 *Hold info for each NFS request
 */
typedef struct nfs_request
{
	rpcCallHdr_t *rpcCallHdrp;		/*Pointer RPC Call hdr*/
	uint32  reqState;               /*State of the request to handle duplicate request cache*/
#define NFS_REQ_INPROG  	1     	/*Req. is in progress and has been replied to.  Ignore the request*/ 
#define NFS_REQ_SEND_TO_FS  2       //Req has been sent to file system and awaiting
                                    //for response
#define	NFS_REQ_REPLIED		3		/*Req. has been replied to the client recently, */ 
									/*it is likely that the retransmitted request and its reply crossed*/
									/*in transit.  Ignore the request*/
#define NFS_REQ_DONE    	4       /*Reply was sent, but not recently, returns cached reply to user*/
#define NFS_REQ_REXMIT		5		/*re-transmit request*/	
	uint32		reqType;
	void    	*clientCBp;			/*Pointer to clientCB*/
	void		*fileSysCBp;		/*Pointer to file system control block*/
	void		*fileCBp;			/*Pointer to fileCB*/
	void		*toFileCBp;			/*Hold to FileCBp (link, rename)*/
	void		*reqCBp;			/*Pointer to reqCB*/
	void		*pktDescp;			/*TPL Tx pkt descriptor*/
	void		*rxPktDesc;			/*Rx pkt descriptor*/
	char    	*procStart;			/*Start of procedure*/
	int32   	procLen;			/*procedure length*/
	uchar8		*namep;				/*Pointer to filename, dir name or symlink*/
	uchar8		fname[32];
	ushort16	nameLen;			/*filename length in bytes*/
	ushort16    dataSize;       	/*Size of user data (0 if no user data)*/
	//nfx_cod_t   cod;				/* COD */
	//int32       offset;				/* COD Offset */
	uint32		nbrUsedCod;
	nfx_cod_entry_t	cod[NFX_MAX_COD_MSG];	//array of COD entries
	fs_Handle	fileHandle;
	fs_Handle	toFileHandle;
	uchar8		*fileMsgp;		/*file system request msg*/
	int32		fileMsgLen;		/*file system message len*/
//	uint64  	reqTimeStamp;   	/*Timestamp of the request*/
    uint32      timeSendToFsInSecs; //Time in seconds since req
                                    //has sent to file system
#define NFS_MAX_WAIT_FS_RESP_TIME   (SEC_PER_MIN * 1)   
									//Wait for 1 min. for resp
                                    //from the file system
    uint32      pad;

	void		*tplConnp;			/*TPL conn ID*/
	void		*pTime;				/* ptr to Time Stamp */
	void		*pNode;				/* ptr to nfp node */
	void		*pNext;				/* ptr to next Req */
	uint32		reqFlag;			
#define NFS_REQ_FILESYS_NOT_READY   0x00000001      /*File System is not ready yet*/
	void        *edescp;			/*hold EEE desc rcv'd from FS*/
	ushort16	reXmtCounter;	/*Number of times the request is retransmitted*/
	ushort16	keepAliveCounter;	/*Used for retransmission of reply*/
	rpcCallHdr_t	rpcHdr;
	nfs_readXmitCompInd_t	readXmitCmpInd;
} nfs_request_t;

typedef struct nfs_reply
{
	uint32      tx_msgType;
#define NFS_TX_REG_MSG_TYPE     1
#define NFS_TX_CACHE_MSG_TYPE   2
#define NFS_TX_ERR_MSG_TYPE     3
	void        *info1p;
	void        *info2p;
	void        *info3p;
} nfs_reply_t;


/*------------------------------------
 * NFS Time Stamp related defines
 *------------------------------------
 */
#define NFS_USE_TIME_STAMP				1

#define NFS_TIME_IDX_LUC_RX             0
#define NFS_TIME_IDX_TPL_RX             1
#define NFS_TIME_IDX_NFS_RX             2
#define NFS_TIME_IDX_FS_REQ             3
#define NFS_TIME_IDX_FS_RSP             4
#define NFS_TIME_IDX_TPL_TX             5
#define NFS_TIME_IDX_LUC_TX             6
#define NFS_TIME_IDX_LUC_TX_CMP         7

#define NFS_MAX_TIME_IDX                (NFS_TIME_IDX_LUC_TX_CMP+1)


typedef struct
{
    uint64  ops;
    uint64  rxBytes;
    uint64  txBytes;
    eeeRTC_t rspHWM;
    uint32  rspTooLong;
    eeeRTC_t duration[NFS_MAX_TIME_IDX];
    uint64  ht_bytes_tx;
} nfs_procTime_t;

/*
 *Externs
 */
extern uint32 nfxMyCPUId;
extern uint32 nfxMySlotId;
extern int32   cacheEnable;

/*---------------------------------------------------------------
 *Prototypes
 *--------------------------------------------------------------
 */
#ifdef  NFP_TXRX

/*
 * NFS procedure prototypes from /sm-nfs/nfs-acl.c
 */
int32 nfs_procRxNfsAcl(nfs_request_t  *nfsReqp);

/*
 *NFS message prototypes from /sm-nfs/nfs-msg.c
 */
void
nfs_msgFreeAll(nfs_request_t *nfsReqp,  int32 freePktDesc);
void
nfs_msgBuildFSHdr(fs_MsgReqHdr *fsMsgReqHdrp,           /*File System message req hdr*/
                uint32 reqType,                         /*Req message type*/
                uint32 attrFlag,                        /*request attributes flag*/
                nfs_request_t *nfsReqp                  /*reqCB*/
		);

fs_Attr *nfs_msgFindAttrType(fs_Attr *attrp, int32 numAttrs,
                        int32 searchAttrType);

/*-------------------------------------------------
 *NFS message prototypes from /sm-nfs/nfs.c
 *------------------------------------------------
 */

void nfs_errMsg(uint32 xid, uint32 procNum,  int32 error);

int32
nfs_storeFieldPresentFLag(char *msgp, uint32 presentFlag);

int32
nfs_storeString(char *DestBufp,
	char *string,  
	uint32 strLen,
	uint32 maxLen);

void nfs_displayStat();
void nfs_displayMntInfo(void);
void nfs_clearCache(void);

int32
nfs_checkClientPermission(nfs_request_t *nfsReqp);

void
nfs_updateXmtStatCounter(nfs_request_t *nfsReqp);

/*-------------------------------------------------
 *NFS message prototypes from /sm-nfs/nfs-util.c
 *------------------------------------------------
 */
int32
nfs_reqTimeoutHandler(int32 timer_handle, void* param1, void* param2,
                void* param3, void* param4);

/*-----------------------------------------------------------------
 * Name:	nfs_utilGetTimeOfDayInSecs 
 *
 * Description: 
 *	Return time of day in seconds.
 *
 * Created by:  Joseph Vu
 *
 * Date Created:        4/28/02
 *
 *-----------------------------------------------------------------
 */
static inline uint32
nfs_utilGetTimeOfDayInSecs(void)
{
	eeeGMT_t now;

	eee_TimeGMT (&now);

	return now.sec;
}

#ifdef	NOTUSED
pkt_desc_t *nfs_tplAllocPkt(nfp_clientCBPtr_t	clientCBp, 
	nfs_request_t       *nfsReqp,	
	int32				usrDataLen);
#endif

/*-------------------------------------------------
 *NFS message prototypes from /sm-nfs/nfs-msg.c
 *------------------------------------------------
 */
int32   /*Return length in bytes of RPC reply header*/
nfs_msgRpcReply( char *msgp,
        uint32  reqXid,
        int32 replyStatus,
        int32 acceptStatus
);

int32 nfs_msgTxTimerHandler(int32 tmr,
	void *info1, void *info2,
	void *info3, void *info4);

/*-----------------------------------------------------------------
 * Name:		
 *
 * Description: 
 *	Check for activities on NFS/TPL UDP connections.	
 *
 * Created by:  Joseph Vu
 *
 * Date Created:        4/28/02
 *
 *-----------------------------------------------------------------
 */
static inline void
nfs_utilCheckUdpConn(int32 timer_handle, void* param1, void* param2, 
				void* param3, void* param4)
{
	//printf("nfs_utilCheckUdpConn enter\n");
	tpl_checkConnActivity(NFS_TPL_CONN_ACT_TIMEOUT);
	return;
}

int32 nfs_msgXmtPkt(pkt_desc_t *tplXmitPktDescp, nfs_request_t *nfsReqp);

int32	/*
	 	 *NFX_OK or
	 	 *NFX_ERR
	 	 */
nfs_msgSendRespFromCache(nfs_request_t *nfsReqp);

int32	/*
	 	 *NFX_OK: No error 
	 	 *NFX_ERR: error occurred
	 	 */
nfs_msgSendToFs(nfs_request_t *nfsReqp	/*current reqCB*/
);

int32   /*
         *NFX_OK or
         *NFX_ERR
         */
nfs_msgSendErrResp (nfs_request_t *nfsReqp, uint32 reqXid, int32 status);

int32
nfs_msgXmtComplete(nfs_request_t *nfsReqp, pkt_desc_t *descp);


/*-------------------------------------------------
 *NFS message prototypes from /sm-nfs/nfs-decode.c
 *------------------------------------------------
 */
int32 
nfs_decodeName(char *nfs_msgp, uint32 type, int32 direction);
char *
nfs_decodeGetProgNumStr(uint32 rpcProgNum );
int32
nfs_rxDecodeAuthSysHostName(rpcCallHdr_t        *rpcCallp);
int32 
nfs_decodeFhandle(char *nfs_msgp, 
		int32 nfs_version);
int32 
nfs_decodeAuthFlavor (char *destMsgp);
int32
nfs_decodePresentFlag (char *nfs_msgp, uint32 *presentFlag);
char *
nfs_getStatus(uint32 replyStatus);

int32
nfs_rxDecodePkt(pkt_desc_t *descp);


//Prototypes for /sm-nfs/nfs-nfp.c

//Prototypes for sm-nfs/nfs-cmd.c
void nfs_cmdRcv(eee_desc_t *eDescp );


int32
nfs_dbgCod( nfs_request_t   *nfsReqp,
            uint32          numOfBytes);


#endif	//NFP_TXRX

#endif	/*_NFS_CLIENT_H*/
