/* vi:ts=4
 *-----------------------------------------------------------------
 *
 * Name:        nfsv2-msg.c
 *
 * RCS:  $Id: nfsv2-msg.c,v 1.30 2002/10/04 01:29:25 shaun Exp $
 *
 * Copyright (C) 2001, Agile Storage Inc.
 *
 * Description:
 *      Contain NFS V2 and File System interface functions.
 *
 * Created by: Joseph Vu
 *
 * Date Created:        5/28/01
 *
 *-----------------------------------------------------------------
 */
#ifdef  NFP_TXRX

#include <nfx-incl.h>
#include <exports/nfs-api.h>
#include "nfsv2-msg.h"

#define	NFS_DBG_COD
/*
#define	NFS_DBG_ON
*/

/*
#define	NFS_USE_SHMEM
*/


extern uint32  trace_enable;

extern nfp_fileCBPtr_t nfs_cacheFindFileName(
        nfp_list_t *nodeListp,  /*list*/
        uchar8 *namep,          /*filename*/
        ushort16 nameLen);      /*filename length in bytes*/

extern nfp_fileCBPtr_t nfs_findFileName(
        nfp_list_t *nodeListp,  /*list*/
        uchar8 *namep,          /*filename*/
        ushort16 nameLen);      /*filename length in bytes*/


/*-----------------------------------------------------------------
 * Name: nfsv2_msgGetAttrResp 
 *
 * Description:
 * Map file system Getattr response message to a NFS V2 
 * reply procedure.
 * Created by:  Joseph Vu
 *
 * Date Created:        5/28/01
 *
 *-----------------------------------------------------------------
 */
int32   /*
         *NFX_OK or NFX_ERR
         */
nfsv2_msgGetAttrResp(int32 status, 	/*operation result status*/
	eee_descBuf_t	*bd,			/*File system GetAttr response msg*/
	char *txMsgp,           		/*Pointer to transmit message buffer*/
	int32 *txMsgLen,        		/*current length of the message*/
	nfs_request_t *nfsReqp  		/*reqCB*/
)
{
	nfs_clientInfo_t    	*clientInfop;
	char					*msgp;
	fs_Attr					*fsAttrp;
	nfp_fileCBPtr_t			fileCBp;
	int32					length;
	nfp_clientCBPtr_t		clientCBp;
	fs_MsgGetAttrResp 		*fsGetAttrMsgp;


	fsGetAttrMsgp = (fs_MsgGetAttrResp *)bd->buf;
	nfs_globalCfgp->nfs_v2XmtPktCounters.getAttr++;
	clientCBp = nfsReqp->clientCBp;
	clientInfop = &clientCBp->protocol.nfp_nfsClientInfo;
	clientInfop->nfs_v2XmtPktCounters.getAttr++;
	nfs_globalCfgp->nfs_xmtPktCounters.getAttr++;
	nfs_globalCfgp->nfs_debugPktCounters.nfs_PktFsRxCounters.getAttr++;

	length = 0;
	msgp = txMsgp;
	/*
	 *Set status
	 */
	*(uint32 *)msgp = status;

	msgp += 4;
	*txMsgLen += 4;

	if (status != NFS_OK)
		return (NFX_OK);


	fsAttrp = &fsGetAttrMsgp->attr;
	length += nfs_storeV2Attributes(nfsReqp->reqType, NFS_ATTR_AVAIL,
					fsAttrp, msgp);
	*txMsgLen += length;

	if (!cacheEnable)
		return (NFX_OK);
	
	/*
	 *Save attributes return from the file system to
	 *the  File Conrol Block (fileCB) for future attributes lookup
	 */
	fileCBp = nfp_getFileCB(&nfsReqp->fileHandle);
	if (fileCBp == NULL)
	{
		fileCBp = nfp_allocFileCB();
		if (fileCBp)
			nfs_addFileCBToList(fileCBp, &nfsReqp->fileHandle, nfsReqp->clientCBp);
		
	}

	if (fileCBp)
	{
		nfsReqp->fileCBp = fileCBp;
		nfs_updateAttr(fileCBp, fsAttrp, NFS_VER2);
	}
	return (NFX_OK);
}

/*-----------------------------------------------------------------
 * Name:       	nfsv2_msgLookupResp 
 *
 * Description:
 *      Map File System Lookup response to a NFS V2 reply procedure. 
 *
 * Created by:  Joseph Vu
 *
 * Date Created:        5/28/01
 *
 *-----------------------------------------------------------------
 */
int32
nfsv2_msgLookupResp(int32 status, 		/*operation result status*/
	eee_descBuf_t *bd,		/*File System response message*/	
	char *txMsgp, 			/*Pointer to transmit message buffer*/
	int32 *txMsgLen, 		/*current length of the message*/
	nfs_request_t *nfsReqp 	/*reqCB*/ 
)
{
	fs_MsgLookupResp 		*fsLookupMsgp;
	nfs_clientInfo_t    	*clientInfop;
//	fs_MsgLookupReq			*fsLookupReqp;
	char	                *msgp;
	fs_Handle 				*fhandlep;
	fs_Attr					*filenameAttrp;	
//	fs_Attr					*parentAttrp;
	fs_Attr					*attrp;
	nfs_fileInfo_t  		*nfs_fileInfop;
	fs_MsgRespHdr			*lookupRespHdrp;
	nfp_fileCBPtr_t			fileCBp;
	nfp_fileCBPtr_t			parentDirFileCBp;
	nfp_clientCBPtr_t		clientCBp;
	int32					length;
//	uint32					cpuId;
	int32					error;

	fsLookupMsgp = (fs_MsgLookupResp *)bd->buf;
	nfs_globalCfgp->nfs_xmtPktCounters.lookUp++;	
	nfs_globalCfgp->nfs_debugPktCounters.nfs_PktFsRxCounters.lookUp++;

	nfs_globalCfgp->nfs_v2XmtPktCounters.lookUp++;

	clientCBp = (nfp_clientCBPtr_t)nfsReqp->clientCBp;
	clientInfop = &clientCBp->protocol.nfp_nfsClientInfo;
	clientInfop->nfs_v2XmtPktCounters.lookUp++;

	error = NFX_OK;
	length = 0;	
	/*
	 *Set operation status result
	 */
	msgp = txMsgp;
	/*
	 *Set operation status
	 */
	*(uint32 *)msgp = status;

	msgp += 4;
	*txMsgLen += 4;
	/*
	 *Error occured, return
	 */
	if (status != NFS_OK)
	{
		error = NFX_ERR;
		goto nfs_msgLookupRespEnd;	
	}
	
	fhandlep = &fsLookupMsgp->fileHandle;

	/*
 	 *Store return filehandle from the file system for search name object
 	 *to destination response msg buffer
 	 */
	length = nfs_storeV2Fhandle(fhandlep, msgp);

	/*
 	 *Update current destination  buffer pointer & length
 	 */
	*txMsgLen += length;
	msgp += length;
	clientCBp = (nfp_clientCBPtr_t)nfsReqp->clientCBp;

	/*
 	 *Store post-operation file attributes in to destination response msg buffer 
 	 */
	lookupRespHdrp = &fsLookupMsgp->hdr;

    /*
     *Get object attributes
     */
	fileCBp = NULL;
	parentDirFileCBp = NULL;

	attrp = &(fsLookupMsgp->attr[0]);
	filenameAttrp = nfs_msgFindAttrType(attrp,
			fsLookupMsgp->numAttrs, FS_POST_ATTRS);
	if (filenameAttrp && 
		(lookupRespHdrp->attrType & FS_POST_ATTRS))
	{
		length = nfs_storeV2Attributes(nfsReqp->reqType, NFS_ATTR_AVAIL,
							filenameAttrp, msgp);
		if (cacheEnable)
		{
			/*
	 	 	*Save filename attributes return from the file system
	 	 	*/
			fileCBp = nfp_getFileCB(fhandlep);
			if (fileCBp == NULL)
			{
				fileCBp = nfp_allocFileCB();
				if (fileCBp)
					nfs_addFileCBToList(fileCBp, fhandlep, clientCBp);
			}
			if (fileCBp)
			{		
				nfsReqp->toFileCBp = fileCBp;
				nfs_updateAttr(fileCBp, filenameAttrp, NFS_VER2);
			}
		}
	}

	*txMsgLen += length;
	msgp += length;

	if (!cacheEnable)
		goto nfs_msgLookupRespEnd; 
		
	//Get parent directory FileCB
	parentDirFileCBp = nfp_getFileCB(&nfsReqp->fileHandle);
	if (parentDirFileCBp == NULL)
	{
		parentDirFileCBp = nfp_allocFileCB();
		if (parentDirFileCBp)
			nfs_addFileCBToList(parentDirFileCBp, &nfsReqp->fileHandle, clientCBp);
	}

	/*
	 *Save filename under parent dir for future lookup
	 */
	if (fileCBp)
	{
		if (parentDirFileCBp)
		{
			fileCBp->nfp_parentDirFileCBp = parentDirFileCBp;
			nfsReqp->fileCBp = parentDirFileCBp;
		}
		nfs_fileInfop = &fileCBp->protocol.nfp_nfsFileInfo;
		if (nfs_fileInfop->nfs_namep)
			goto nfs_msgLookupRespEnd;

		if (parentDirFileCBp && nfsReqp->nameLen && nfsReqp->namep)
		{
			if (nfs_cacheFindFileName(&parentDirFileCBp->nfp_dirCacheFileList,
                   nfsReqp->namep, nfsReqp->nameLen)) 
       		{
				/*
		 	 	 *return if entry is already in cache
		 	 	 */
				goto nfs_msgLookupRespEnd;
			}
			else
			{
				/*
		 	 	 *Save entry in cache
		 	 	 */
       			nfs_fileInfop->nfs_namep = nfsReqp->namep;
       			nfs_fileInfop->nfs_nameLen =  nfsReqp->nameLen;
				nfs_addCBToList(&parentDirFileCBp->nfp_dirCacheFileList, 
								(void *)fileCBp, NFP_FILECB_NODE);
			}
		}
	}
	return (NFX_OK);		
nfs_msgLookupRespEnd:
	if ( (nfsReqp->namep) && (nfsReqp->nameLen) )
	{
#ifdef	NFS_USE_SHMEM
		eee_deallocateBuffer((void *)nfsReqp->namep);
#else	
		eee_ramDealloc((uchar8 *)nfsReqp->namep);		
#endif
		nfsReqp->nameLen = 0;
		nfsReqp->namep = NULL;
	}

	
	return (error);

}

/*-----------------------------------------------------------------
 * Name: nfsv2_msgSetAttrResp 
 *
 * Description:
 * Map File System SetAttr response to a NFS V2 reply procedure.
 *
 * Created by:  Joseph Vu
 *
 * Date Created:        5/28/01
 *
 *-----------------------------------------------------------------
 */
int32
nfsv2_msgSetAttrResp(int32 status, 		/*operation result status*/
	eee_descBuf_t *bd,			/*file system SetAttr resp. msg*/
	char *txMsgp, 				/*Pointer to transmit message buffer*/
	int32 *txMsgLen,			/*current length of the message*/
	nfs_request_t *nfsReqp		/*reqCB*/
)
{
	fs_MsgSetAttrResp 		*fsSetAttrMsgp;
	nfp_clientCBPtr_t   	clientCBp;
	nfs_clientInfo_t    	*clientInfop;
	char                  	*msgp;
	fs_Attr                 *attrp;
//	fs_Attr					*preOpAttrp;
	fs_Attr					*postOpAttrp;
//	nfs_fileInfo_t          *nfs_fileInfop;
	nfp_fileCBPtr_t         fileCBp;
	fs_MsgRespHdr           *respHdrp;
	int32					length;

	fsSetAttrMsgp = (fs_MsgSetAttrResp *)bd->buf;
	clientCBp = (nfp_clientCBPtr_t)nfsReqp->clientCBp;
	clientInfop = &clientCBp->protocol.nfp_nfsClientInfo;
	clientInfop->nfs_v2XmtPktCounters.setAttr++;
	nfs_globalCfgp->nfs_v2XmtPktCounters.setAttr++;
	nfs_globalCfgp->nfs_xmtPktCounters.setAttr++;
	nfs_globalCfgp->nfs_debugPktCounters.nfs_PktFsRxCounters.setAttr++;

	length = 0;
	/*
	 *Set operation status result
	 */
	msgp = txMsgp;
	/*
	 *Set operation status
	 */
	*(uint32 *)msgp = status;

	msgp += 4;
	*txMsgLen += 4;
	
	/*
	 *Retrieve attributes information from the file system
	 */
	
	if (status != NFS_OK)
		return (NFX_OK);

	respHdrp = &fsSetAttrMsgp->hdr;

	/*
	 *Store post-operation attributes if available
	 */
	attrp = &(fsSetAttrMsgp->attr[0]);
	postOpAttrp = nfs_msgFindAttrType(attrp,
			fsSetAttrMsgp->numAttrs, FS_POST_ATTRS);
	if (postOpAttrp && 
		(respHdrp->attrType & FS_POST_ATTRS))
	{
		length = nfs_storeV2Attributes(nfsReqp->reqType, NFS_ATTR_AVAIL,
					postOpAttrp, msgp);
		/*
	 	 *Save SetAttr file attributes info return from file system to our 
	 	 *File Control Block (fileCB) for future attributes lookup
	 	 */
		if (cacheEnable)
		{
			fileCBp = nfp_getFileCB(&nfsReqp->fileHandle);
			if (fileCBp == NULL)
			{
				fileCBp = nfp_allocFileCB();
				if (fileCBp)
					nfs_addFileCBToList(fileCBp, &nfsReqp->fileHandle, nfsReqp->clientCBp);
			}

			if (fileCBp)
			{
				nfsReqp->fileCBp = fileCBp;
				nfs_updateAttr(nfsReqp->fileCBp, postOpAttrp,  NFS_VER2);
			}
		}
	}
	else
		length = nfs_storeV2Attributes(nfsReqp->reqType, NFS_ATTR_NOT_AVAIL,
					postOpAttrp, msgp);
	*txMsgLen += length;
	msgp += length;

	return (NFX_OK);
}


/*-----------------------------------------------------------------
 * Name:       nfsv2_msgSymlinkResp 
 *
 * Description:
 * Map File System Symlink response to a NFS V2 reply procedure.
 *
 * Created by:  Joseph Vu
 *
 * Date Created:        5/28/01
 *
 *-----------------------------------------------------------------
 */
int32
nfsv2_msgSymlinkResp(int32 status, 		/*operation result status*/
	eee_descBuf_t *bd,      /*file system resp. msg*/
	char *txMsgp, 			/*pointer to transmit message buffer*/
	int32 *txMsgLen, 		/*current length of the message*/
	nfs_request_t *nfsReqp	/*reqCB*/
)
{
	fs_MsgSymLinkResp 		*fsSymlinkMsgp;
	char					*msgp;
	nfs_clientInfo_t    	*clientInfop;
	nfp_clientCBPtr_t   	clientCBp;


	fsSymlinkMsgp = (fs_MsgSymLinkResp *)bd->buf;
	clientCBp = nfsReqp->clientCBp;
	clientInfop = &clientCBp->protocol.nfp_nfsClientInfo;
	clientInfop->nfs_v2XmtPktCounters.symlink++;
	nfs_globalCfgp->nfs_v2XmtPktCounters.symlink++;
	nfs_globalCfgp->nfs_xmtPktCounters.symlink++;
	nfs_globalCfgp->nfs_debugPktCounters.nfs_PktFsRxCounters.symlink++;
	nfs_globalCfgp->nfs_xmtPktCounters.symlink++;
	msgp = txMsgp;
	/*
	 *Set operation status
	 */
	*(uint32 *)msgp = status;

	msgp += 4;
	*txMsgLen += 4;


	return (NFX_OK);
}

/*-----------------------------------------------------------------
 * Name:      	nfsv2_msgHardLinkResp 
 *
 * Description:
 * Map File System Link response to a NFS V2 reply procedure.
 *
 * Created by:  Joseph Vu
 *
 * Date Created:        5/28/01
 *
 *-----------------------------------------------------------------
 */
int32
nfsv2_msgHardLinkResp(
	int32	status, 				/*operation result status*/
	eee_descBuf_t *bd,      /*file system resp. msg*/
	char *txMsgp, 			/*pointer to transmit message buffer*/
	int32 *txMsgLen, 		/*current length of the message*/
	nfs_request_t *nfsReqp	/*reqCB*/
)
{
	char                  	*msgp;
	nfs_clientInfo_t    	*clientInfop;
	nfp_clientCBPtr_t   	clientCBp;	
	fs_MsgHardLinkResp		*fsLinkRespMsgp;

	fsLinkRespMsgp = (fs_MsgHardLinkResp *)bd->buf;
	clientCBp = nfsReqp->clientCBp;
	clientInfop = &clientCBp->protocol.nfp_nfsClientInfo;
	clientInfop->nfs_v2XmtPktCounters.link++;
	nfs_globalCfgp->nfs_v2XmtPktCounters.link++;
	nfs_globalCfgp->nfs_debugPktCounters.nfs_PktFsRxCounters.link++;
	nfs_globalCfgp->nfs_xmtPktCounters.link++;

	msgp = txMsgp;
	/*
	 *Set operation status
	 */
	*(uint32 *)msgp = status;

	msgp += 4;
	*txMsgLen += 4;

	return (NFX_OK);
}


/*-----------------------------------------------------------------
 * Name: nfsv2_msgReadlinkResp 
 *
 * Description:
 *Map File System Readlink response message to a NFS V2 reply procedure.
 *
 * Created by:  Joseph Vu
 * 
 * Date Created:        5/28/01
 *
 *-----------------------------------------------------------------
 */
int32	/*
	 	 *NFX_OK or NFX_ERR
	 	 */
nfsv2_msgReadlinkResp(
	int32 status, 			/*operation result status*/
	eee_descBuf_t *bd,		/*file system resp. msg*/
	char *txMsgp,			/*Pointer to transmit msg buffer*/
	int32 *txMsgLen,		/*current length of msg buffer*/
	nfs_request_t *nfsReqp	/*req control block*/
)
{
	fs_MsgReadLinkResp 		*fsReadLinkMsgp;
	nfp_clientCBPtr_t   	clientCBp;
	nfs_clientInfo_t    	*clientInfop;
	nfp_fileCBPtr_t         fileCBp;
	char					*msgp;
	nfs_fileInfo_t          *nfs_fileInfop;
	fs_MsgRespHdr           *respHdrp;
	int32					length;
	ushort16				linkNameLen;


	fsReadLinkMsgp = (fs_MsgReadLinkResp *)bd->buf;
	clientCBp = nfsReqp->clientCBp;
	clientInfop = &clientCBp->protocol.nfp_nfsClientInfo;
	clientInfop->nfs_v2XmtPktCounters.readLink++;
	nfs_globalCfgp->nfs_v2XmtPktCounters.readLink++;
	nfs_globalCfgp->nfs_xmtPktCounters.readLink++;	
	nfs_globalCfgp->nfs_debugPktCounters.nfs_PktFsRxCounters.readLink++;

	length = 0;

	msgp = txMsgp;
	/*
	 *Set operation status
	 */
	*(uint32 *)msgp = status;

	msgp += 4;
	*txMsgLen += 4;

	if (status != NFS_OK)
		return (NFX_ERR);

	respHdrp = &fsReadLinkMsgp->hdr;

	if (cacheEnable)
	{
		fileCBp = nfp_getFileCB(&nfsReqp->fileHandle);
		if (fileCBp == NULL)
		{
			fileCBp = nfp_allocFileCB();
			if (fileCBp)
				nfs_addFileCBToList(fileCBp, &nfsReqp->fileHandle, nfsReqp->clientCBp);
		} 
	}	
	linkNameLen = fsReadLinkMsgp->nameLen;	
	if (linkNameLen <= 0)
		return (NFX_ERR);

	/*
	 *Get data associated with the symbolic link
	 */
	length = nfs_storeString(msgp, fsReadLinkMsgp->linkName, linkNameLen, FS_PATH_MAX);
	*txMsgLen += length;

	/*
	 *save symbolic link data for future lookup
	 */
	if (cacheEnable && fileCBp)
	{
		nfsReqp->fileCBp = fileCBp;
		nfs_fileInfop = &fileCBp->protocol.nfp_nfsFileInfo;
		nfs_fileInfop->nfs_namep = (char *)eee_ramAlloc(linkNameLen);
		if (nfs_fileInfop->nfs_namep != NULL)
		{
			memcpy (nfs_fileInfop->nfs_namep, 
				fsReadLinkMsgp->linkName, linkNameLen);
			nfs_fileInfop->nfs_nameLen = linkNameLen;
			nfs_fileInfop->nfs_flag |= NFS_CACHE_SYMLINK;
		}
	}
	return (NFX_OK);
}

/*-----------------------------------------------------------------
 * Name:        nfsv2_msgCreateResp
 *
 * Description:
 *	 Map File System Create response to a NFS reply procedure.
 *
 * Created by:  Joseph Vu
 *
 * Date Created:        5/28/01
 *
 *-----------------------------------------------------------------
 */
int32   /*
         *NFS_OK or NFX_ERR
         */
nfsv2_msgCreateResp(int32 status, 			/*Operation result status*/
	eee_descBuf_t *bd,		/*file system resp. msg*/
	char *txMsgp, 			/*Poiner to transmit message buffer*/
	int32 *txMsgLen, 		/*current length of the message*/
	nfs_request_t *nfsReqp	/*reqCB*/
)
{
	fs_MsgCreateResp 		*fsCreateRespMsgp;
	nfs_clientInfo_t		*clientInfop;
	char					*msgp;
	fs_Handle               *fhandlep;
	fs_Attr                 *attrp;
	fs_Attr                 *filenameAttrp;
	nfs_fileInfo_t          *nfs_fileInfop;
	fs_MsgRespHdr			*respHdrp;
	nfp_fileCBPtr_t         fileCBp;
	nfp_fileCBPtr_t         parentDirFileCBp;
	nfp_clientCBPtr_t       clientCBp;
	int32                   length;
	int32					error;

	fsCreateRespMsgp = (fs_MsgCreateResp *)bd->buf;
	clientCBp = nfsReqp->clientCBp;
	clientInfop = &clientCBp->protocol.nfp_nfsClientInfo;
	clientInfop->nfs_v2XmtPktCounters.create++;
	nfs_globalCfgp->nfs_v2XmtPktCounters.create++;
	nfs_globalCfgp->nfs_xmtPktCounters.create++;
    nfs_globalCfgp->nfs_debugPktCounters.nfs_PktFsRxCounters.create++;

	error = NFX_OK;
	length = 0;

	msgp = txMsgp;
	/*
	 *Set operation status
	 */
	*(uint32 *)msgp = status;

	msgp += 4;
	*txMsgLen += 4;

	/*
	 *Error occured, return
	 */
	if (status != NFS_OK)
	{
		error = NFX_ERR;
		goto nfs_msgCreateRespEnd;
	}

	/*
	 *Store return filehandle from file system to
	 *the destination response msg buffer
	 */
	fhandlep = &fsCreateRespMsgp->fileHandle;
	length = nfs_storeV2Fhandle(fhandlep, msgp);

	/*
	 *Update current destination  buffer pointer & length
	 */
	*txMsgLen += length;
	msgp += length;

	/*
	 *Store attributes of the regular file just created
	 */
	respHdrp = &fsCreateRespMsgp->hdr;

	attrp = &(fsCreateRespMsgp->attr[0]);
	filenameAttrp = nfs_msgFindAttrType(attrp,
			fsCreateRespMsgp->numAttrs, FS_POST_ATTRS);

	if (filenameAttrp && 
		(respHdrp->attrType & FS_POST_ATTRS) )
	{
		length = nfs_storeV2Attributes(nfsReqp->reqType, 
						NFS_ATTR_AVAIL, filenameAttrp, 
						msgp);		

		/*
	 	 *Save filename attributes return from the 
		 *file system to its File Control
	 	 *Block (fileCB) for future lookup
	 	 */
		if (cacheEnable)
		{
			fileCBp = nfp_getFileCB(fhandlep);
			if (fileCBp == NULL)
			{
				fileCBp = nfp_allocFileCB();
				if (fileCBp)
					nfs_addFileCBToList(fileCBp, fhandlep, nfsReqp->clientCBp);
			}
			if (fileCBp)	
				nfs_updateAttr(fileCBp, filenameAttrp, NFS_VER2);
		}	//cacheEnable
	}
	else
	{
		length = nfs_storeV2Attributes(nfsReqp->reqType, 
						NFS_ATTR_NOT_AVAIL, filenameAttrp, 
						msgp);		
	}
	*txMsgLen += length;
	msgp += length;

	if (!cacheEnable)
		goto nfs_msgCreateRespEnd;
		
	parentDirFileCBp = nfp_getFileCB(&nfsReqp->fileHandle);
	if (parentDirFileCBp == NULL)
	{
		parentDirFileCBp = nfp_allocFileCB();
		if (parentDirFileCBp)
			nfs_addFileCBToList(parentDirFileCBp, &nfsReqp->fileHandle, 
							nfsReqp->clientCBp);
	}
	if (parentDirFileCBp)
		nfsReqp->fileCBp = parentDirFileCBp;		

	if (fileCBp)
	{
		/*
	 	*Save filename under parent dir for future lookup
	 	*/
		nfs_fileInfop = &fileCBp->protocol.nfp_nfsFileInfo;
		if (nfsReqp->nameLen && nfsReqp->namep)
		{
			//Add entry to cache list.  Cache list is being update periodically
			nfs_fileInfop->nfs_namep = nfsReqp->namep;
			nfs_fileInfop->nfs_nameLen =  nfsReqp->nameLen;
			nfs_addCBToList(&parentDirFileCBp->nfp_dirCacheFileList, 
						(void *)fileCBp, NFP_FILECB_NODE);
#ifdef  NFS_USE_SHMEM
			cpuId = EEE_TXRX_POOL;
			nfs_fileInfop->nfs_fileNamep = 
				eee_allocateBuffer(cpuId, EEE_BUF_SMALL, NFX_COS_HIGH);
#else
			nfs_fileInfop->nfs_fileNamep =
				(uchar8 *)eee_ramAlloc(nfsReqp->nameLen+1);
#endif
			if (nfs_fileInfop->nfs_fileNamep == NULL)
				panic("nfsv2_msgCreateResp: Failure to alloc memory!\n");


			memset(nfs_fileInfop->nfs_fileNamep, 0, nfsReqp->nameLen+1);
			memcpy(nfs_fileInfop->nfs_fileNamep, nfsReqp->namep, nfsReqp->nameLen);
			nfs_fileInfop->nfs_filenameLen = nfsReqp->nameLen;
			if (parentDirFileCBp)
			{
				fileCBp->nfp_parentDirFileCBp = parentDirFileCBp;
				nfs_addCBToList(&parentDirFileCBp->nfp_dirFileList,
					(void *)fileCBp, NFP_FILECB_NODE);
			}
		}

	}	

	return (NFX_OK);

nfs_msgCreateRespEnd:
	if ( (nfsReqp->namep) && (nfsReqp->nameLen) )
	{
#ifdef	NFS_USE_SHMEM
		eee_deallocateBuffer((void *)nfsReqp->namep);
#else
		eee_ramDealloc((uchar8 *)nfsReqp->namep);
#endif
		nfsReqp->nameLen = 0;
		nfsReqp->namep = NULL;
	}

	return (error);
}

/*-----------------------------------------------------------------
 * Name:      	nfsv2_msgRmdirResp 
 *                      
 * Description:         
 * Map File System RMDIR response to a NFS V2 reply procedure.
 *             
 * Created by:  Joseph Vu
 *
 * Date Created:        5/28/01
 *
 *-----------------------------------------------------------------
 */
int32
nfsv2_msgRmdirResp(int32 status,		/*Operation result status*/ 
	eee_descBuf_t *bd,	/*file system resp. msg*/
	char *txMsgp,		/*pointer to transmit message buffer*/ 
	int32 *txMsgLen, 	/*current length of the message*/
	nfs_request_t *nfsReqp	
)
{
	fs_MsgRmdirResp 		*fsRmdirRespMsgp;
	nfp_clientCBPtr_t   	clientCBp;	
	nfs_clientInfo_t    	*clientInfop;
	fs_Attr                 *attrp;
	fs_Attr					*postOpAttrp;
	fs_MsgRespHdr           *respHdrp;
	char					*msgp;
	nfp_fileCBPtr_t			parentDirFileCBp;
	nfp_fileCBPtr_t			fileCBp;
	int32					length;
	
	fsRmdirRespMsgp = (fs_MsgRmdirResp *)bd->buf;
	clientCBp = nfsReqp->clientCBp;
	clientInfop = &clientCBp->protocol.nfp_nfsClientInfo;
	clientInfop->nfs_v2XmtPktCounters.rmdir++;
	nfs_globalCfgp->nfs_v2XmtPktCounters.rmdir++;
	nfs_globalCfgp->nfs_debugPktCounters.nfs_PktFsRxCounters.rmdir++;
	nfs_globalCfgp->nfs_xmtPktCounters.rmdir++;

	length = 0;

	msgp = txMsgp;
	/*
	 *Set operation status
	 */
	*(uint32 *)msgp = status;

	msgp += 4;
	*txMsgLen += 4;

	if (status != NFX_OK)
		goto nfs_msgRmdirRespEnd;

	respHdrp = &fsRmdirRespMsgp->hdr;
	parentDirFileCBp = nfp_getFileCB (&nfsReqp->fileHandle);
	if (parentDirFileCBp == NULL)
		goto nfs_msgRmdirRespEnd;
	/*
	 *Remove entry
	 */
	fileCBp = nfs_findFileName(&parentDirFileCBp->nfp_dirFileList,
						nfsReqp->namep, nfsReqp->nameLen);
	if (fileCBp)
		nfs_delFileCBFromList(parentDirFileCBp, fileCBp, 
		(nfp_clientCBPtr_t)nfsReqp->clientCBp);		

	//Check if the directory still has pending
	//cache entries
	if (parentDirFileCBp->nfp_dirCacheFileList.nfp_reqCnt != 0)
		nfs_removeAllCacheEntries(&parentDirFileCBp->nfp_dirCacheFileList);
	

	/*
	 *Get update attributes if link count is non-zero
	 */
	attrp = &(fsRmdirRespMsgp->attr[0]);
	postOpAttrp = nfs_msgFindAttrType(attrp,
						fsRmdirRespMsgp->numAttrs,
						FS_POST_PAR_ATTRS);
	if (cacheEnable && postOpAttrp &&
		(postOpAttrp->numLinks))
	{
		nfs_updateAttr(parentDirFileCBp, postOpAttrp, NFS_VER2);
	}

nfs_msgRmdirRespEnd: 
	if ( (nfsReqp->namep) && (nfsReqp->nameLen) )
	{
#ifdef	NFS_USE_SHMEM
		eee_deallocateBuffer((void *)nfsReqp->namep);
#else
		eee_ramDealloc((uchar8 *)nfsReqp->namep);
#endif
		nfsReqp->nameLen = 0;
		nfsReqp->namep = NULL;
	}

	return (NFX_OK);	
}

/*-----------------------------------------------------------------
 * Name:       nfsv2_msgRemoveResp 
 *
 * Description:
 * Map File System Remove/RMDIR response to a NFS V2 reply procedure.
 *
 * Created by:  Joseph Vu
 *
 * Date Created:        5/28/01
 *
 *-----------------------------------------------------------------
 */
int32
nfsv2_msgRemoveResp(int32 status,			/*Operation result status*/
	eee_descBuf_t *bd,		/*file system resp. msg*/
	char *txMsgp,			/*pointer to transmit message buffer*/
	int32 *txMsgLen,		/*current length of the message*/
	nfs_request_t *nfsReqp	/*reqCB*/
)
{
	fs_MsgRemoveResp 		*fsRemoveRespMsgp;
	nfp_clientCBPtr_t       clientCBp;
	nfs_clientInfo_t    	*clientInfop;
	fs_Attr                 *attrp;
	fs_Attr					*postOpAttrp;
	fs_MsgRespHdr           *respHdrp;
	char					*msgp;
	nfp_fileCBPtr_t			parentDirFileCBp;
	nfp_fileCBPtr_t			fileCBp;
	int32					length;
	
	fsRemoveRespMsgp = (fs_MsgRemoveResp *)bd->buf;
	clientCBp = nfsReqp->clientCBp;
	clientInfop = &clientCBp->protocol.nfp_nfsClientInfo;
	clientInfop->nfs_v2XmtPktCounters.remove++;
	nfs_globalCfgp->nfs_v2XmtPktCounters.remove++;
	nfs_globalCfgp->nfs_debugPktCounters.nfs_PktFsRxCounters.remove++;
	nfs_globalCfgp->nfs_xmtPktCounters.remove++;
	length = 0;

	msgp = txMsgp;
	/*
	 *Set operation status
	 */
	*(uint32 *)msgp = status;

	msgp += 4;
	*txMsgLen += 4;

	if (status != NFX_OK)
	{
		goto nfs_msgRemoveRespEnd;
	}

	respHdrp = &fsRemoveRespMsgp->hdr;
	parentDirFileCBp = nfp_getFileCB (&nfsReqp->fileHandle);
	if (parentDirFileCBp == NULL)
		goto nfs_msgRemoveRespEnd;
	/*
	 *remove delete file
	 */
	fileCBp = nfs_findFileName(&parentDirFileCBp->nfp_dirFileList,
			nfsReqp->namep, nfsReqp->nameLen);
	if (fileCBp == NULL)
		fileCBp =  nfp_getFileCB (&fsRemoveRespMsgp->fileHandle);	
	
	if (fileCBp)	
		nfs_delFileCBFromList(parentDirFileCBp, fileCBp, 
				(nfp_clientCBPtr_t)nfsReqp->clientCBp);		

	//Check if remove entry is still in cache
	fileCBp = nfs_cacheFindFileName(&parentDirFileCBp->nfp_dirCacheFileList,
								nfsReqp->namep, nfsReqp->nameLen);
	if (fileCBp != NULL)
		nfs_removeCacheEntry(fileCBp);

	if (!cacheEnable)
		goto nfs_msgRemoveRespEnd;

	/*
	 *Get update attributes if link count is non-zero
	 */
	attrp = &(fsRemoveRespMsgp->attr[0]);
	postOpAttrp = nfs_msgFindAttrType(attrp,
						fsRemoveRespMsgp->numAttrs,
						FS_POST_ATTRS);
	if (postOpAttrp &&
		(postOpAttrp->numLinks))
	{

		fileCBp =  nfp_getFileCB (&fsRemoveRespMsgp->fileHandle);		
		if (fileCBp != NULL)
		{
			nfs_updateAttr(fileCBp, postOpAttrp, NFS_VER2);
		}
	}
nfs_msgRemoveRespEnd: 
	if ( (nfsReqp->namep) && (nfsReqp->nameLen) )
	{
#ifdef	NFS_USE_SHMEM
		eee_deallocateBuffer((void *)nfsReqp->namep);
#else
		eee_ramDealloc((uchar8 *)nfsReqp->namep);
#endif
		nfsReqp->nameLen = 0;
		nfsReqp->namep = NULL;
	}

	return (NFX_OK);	
}       

/*-----------------------------------------------------------------
 * Name:       	nfsv2_msgMkdirResp 
 *
 * Description:
 * Map File System Mkdir (make directory) response to a NFS V2 
 * reply procedure.
 *
 * Created by:  Joseph Vu
 *
 * Date Created:        5/28/01
 *
 *-----------------------------------------------------------------
 */
int32
nfsv2_msgMkdirResp(int32 status,		/*operation result status*/
	eee_descBuf_t *bd,      /*file system resp. msg*/
	char *txMsgp,			/*pointer to transmit message buffer*/
	int32 *txMsgLen,		/*current length of the message*/
	nfs_request_t *nfsReqp	/*reqCB*/
)
{
	fs_MsgMkdirResp 		*fsMkdirRespMsgp;
	nfs_clientInfo_t    	*clientInfop;
	nfp_clientCBPtr_t		clientCBp;
	nfp_fileCBPtr_t			fileCBp;
	nfp_fileCBPtr_t			parentDirFileCBp;
	char                  	*msgp;
	fs_Handle               *fhandlep;
	fs_Attr					*attrp;
	fs_Attr					*subdirAttrp;
	nfs_fileInfo_t          *nfs_fileInfop;
	fs_MsgRespHdr           *respHdrp;
	int32					length;


	fsMkdirRespMsgp = (fs_MsgMkdirResp *)bd->buf;
	nfs_globalCfgp->nfs_v2XmtPktCounters.mkdir++;
	clientCBp = nfsReqp->clientCBp;
	clientInfop = &clientCBp->protocol.nfp_nfsClientInfo;
	clientInfop->nfs_v2XmtPktCounters.mkdir++;
	nfs_globalCfgp->nfs_xmtPktCounters.mkdir++;
	nfs_globalCfgp->nfs_debugPktCounters.nfs_PktFsRxCounters.mkdir++;

	length = 0;    

	msgp = txMsgp;
	/*
	 *Set operation status
	 */
	*(uint32 *)msgp = status;

	msgp += 4;
	*txMsgLen += 4;


	if  (status != NFS_OK)
	{
		goto nfs_msgMkdirRespEnd;
	}
	/*
	 *Store the return filehandle from the file system into the 
	 *destination transmit msg buffer
	 */
	fhandlep = &fsMkdirRespMsgp->dirHandle;
	length = nfs_storeV2Fhandle(fhandlep, msgp);
	/*
	 *Update current destination buffer pointer & length
	 */
	*txMsgLen += length;
	msgp += length;

	/*
	 *Store the post-op attributes for newly created sub-dir
	 */
	respHdrp = &fsMkdirRespMsgp->hdr;

	attrp = &(fsMkdirRespMsgp->attr[0]);
	subdirAttrp = nfs_msgFindAttrType(attrp,
						fsMkdirRespMsgp->numAttrs, FS_POST_ATTRS); 
	if (subdirAttrp && (respHdrp->attrType & FS_POST_ATTRS))
	{
		length = nfs_storeV2Attributes(nfsReqp->reqType, 
						NFS_ATTR_AVAIL, subdirAttrp, 
						msgp);
		/*
	 	 *Save the return file attributes for newly created sub-dir	
	 	 */
		if (cacheEnable)
		{
			fileCBp = nfp_getFileCB(fhandlep);
			if (fileCBp == NULL)
			{
				fileCBp = nfp_allocFileCB();
				if (fileCBp)
					nfs_addFileCBToList(fileCBp, fhandlep, 
								nfsReqp->clientCBp);
			}
			if (fileCBp)
				nfs_updateAttr(fileCBp, subdirAttrp, NFS_VER2);
		}	//cacheEnable
	}
	else
	{
		length = nfs_storeV2Attributes(nfsReqp->reqType, 
						NFS_ATTR_NOT_AVAIL, subdirAttrp, 
						msgp);
	}
	*txMsgLen += length;
	msgp += length;

	if (!cacheEnable)
		goto nfs_msgMkdirRespEnd;


	/*
	 *Save filename under parent dir for future lookup
	 */	
	parentDirFileCBp = nfp_getFileCB(&nfsReqp->fileHandle);
	if (parentDirFileCBp == NULL)
	{
		parentDirFileCBp = nfp_allocFileCB();
		if (parentDirFileCBp)
			nfs_addFileCBToList(parentDirFileCBp, &nfsReqp->fileHandle, 
					nfsReqp->clientCBp);
	}
	if (parentDirFileCBp)	
		nfsReqp->fileCBp = parentDirFileCBp;
	/*
	 *Save filename under parent dir for future lookup
	 */
	if (fileCBp)
	{
		nfs_fileInfop = &fileCBp->protocol.nfp_nfsFileInfo;
		if (nfsReqp->nameLen && nfsReqp->namep)
		{
			//Add entry to cache list.  Cache list is being update
			//periodically
			nfs_fileInfop->nfs_namep = nfsReqp->namep;
			nfs_fileInfop->nfs_nameLen =  nfsReqp->nameLen;
			nfs_addCBToList(&parentDirFileCBp->nfp_dirCacheFileList, 
								(void *)fileCBp, NFP_FILECB_NODE);
			//Add entry to directory file list.  Entry will be there
			//until file is being removed
#ifdef  NFS_USE_SHMEM
			cpuId = EEE_TXRX_POOL;
			nfs_fileInfop->nfs_fileNamep =
				eee_allocateBuffer(cpuId, EEE_BUF_SMALL, NFX_COS_HIGH);
#else
			nfs_fileInfop->nfs_fileNamep = 
				(uchar8 *)eee_ramAlloc(nfsReqp->nameLen+1);
#endif
			
			if (nfs_fileInfop->nfs_fileNamep == NULL)
			{
				panic("nfsv2_msgMkdirResp: Failure to alloc memory!\n");
			}

			memset(nfs_fileInfop->nfs_fileNamep, 0, nfsReqp->nameLen+1);
			memcpy(nfs_fileInfop->nfs_fileNamep, nfsReqp->namep, nfsReqp->nameLen);
			nfs_fileInfop->nfs_filenameLen = nfsReqp->nameLen;
			if (parentDirFileCBp)
			{
		   		fileCBp->nfp_parentDirFileCBp = parentDirFileCBp;
				nfs_addCBToList(&parentDirFileCBp->nfp_dirFileList,
					(void *)fileCBp, NFP_FILECB_NODE);
			}
		}

	}

	return (NFX_OK);

nfs_msgMkdirRespEnd:
	if ( (nfsReqp->namep) && (nfsReqp->nameLen) )
	{
#ifdef	NFS_USE_SHMEM
		eee_deallocateBuffer((void *)nfsReqp->namep);
#else
		eee_ramDealloc((uchar8 *)nfsReqp->namep);
#endif
		nfsReqp->nameLen = 0;
		nfsReqp->namep = NULL;
	}

	return (NFX_OK);
}


/*-----------------------------------------------------------------
 * Name:        nfsv2_msgReaddirResp
 *
 * Description:
 * Map File System Readdir response to a NFS V2 reply procedure.
 *
 * Created by:  Joseph Vu
 *
 * Date Created:        5/28/01
 *
 *-----------------------------------------------------------------
 */
int32	/*
		 *NFX_OK  or NFX_ERR
	 	 */
nfsv2_msgReaddirResp(int32 status, 			/*Operation result status*/
	eee_descBuf_t *bd,      /*file system resp. msg*/
	char *txMsgp, 			/*Poiner to transmit message buffer*/
	int32 *txMsgLen, 		/*current length of the message*/
	nfs_request_t *nfsReqp	/*reqCB*/
)
{
	int32				rc;
	int32				msgLen;
	int32				length;
	int32				i;
	ushort16			numOfEntries;
	char				*msgp;
	nfp_fileCBPtr_t		fileCBp;
	fs_DirEntry			*dirent;
	nfs_clientInfo_t    *clientInfop;
	nfp_clientCBPtr_t   clientCBp;
	fs_MsgReadDirResp 	*fsReaddirRespMsgp;
	char8				*msgBufp;
	eee_desc_t 			*eDescp;
	static char			msgBuf[8192];

	/*
	 *Retrieve EEE message into local buffer
	 */	
	eDescp = nfsReqp->edescp;
	msgLen =  (eDescp->hdr.control & EDESC_TOTAL_LEN);
	/*
	 *Check if message length fits in single descriptor
	 */
	if (msgLen < EEE_BUFSIZE_LARGE)
	{
		fsReaddirRespMsgp = (fs_MsgReadDirResp *)bd->buf;
	}
	else
	{

		memset(msgBuf, 0, sizeof(msgBuf));
		msgBufp = &msgBuf[0];

		rc = eee_copyPacketToLocalBuffer(msgBufp, eDescp, msgLen, 0);
		if (rc != msgLen)
		{
			/*copied failed*/	
			panic("%s: eee_copyPacketToLocalBuffer failed!\n",
						__FUNCTION__);
		}
		fsReaddirRespMsgp = (fs_MsgReadDirResp *)msgBufp;
	}
	
	nfs_globalCfgp->nfs_v2XmtPktCounters.readdir++;
	clientCBp = nfsReqp->clientCBp;
	clientInfop = &clientCBp->protocol.nfp_nfsClientInfo;
	clientInfop->nfs_v2XmtPktCounters.readdir++;
	nfs_globalCfgp->nfs_debugPktCounters.nfs_PktFsRxCounters.readdir++;
	nfs_globalCfgp->nfs_xmtPktCounters.readdir++;

	length = 0;

	msgp = txMsgp;
	/*
	 *Set operation status
	 */
	*(uint32 *)msgp = status;

	msgp += 4;
	*txMsgLen += 4;

	if (status != NFS_OK) 
		return (NFX_OK);

	/*
	 *Get number of directory entries
	 */	
	numOfEntries = fsReaddirRespMsgp->numEntriesThis;
	dirent = &fsReaddirRespMsgp->dirEntry[0];
	for (i = 0; i < numOfEntries; i++)
	{
		/*
		 *Retrieve fileid, name, cookie
		 */
		length = nfs_storeDirEntry(NFS_VER2, dirent, msgp);
		FS_READDIR_ADV_ENTRY_PTR(dirent);
		msgp += length;
		*txMsgLen += length;
	}
	length = nfs_storeFieldPresentFLag(msgp, FALSE);
	*txMsgLen += length;
	msgp += length;

	/*
	 *Store eof flag from file system into 
	 *destination message buffer
	 */
	length = nfs_storeLongValToBuf(msgp, fsReaddirRespMsgp->eof);
	*txMsgLen += length;

	if (!cacheEnable)
		return (NFX_OK);	

	/*
	 *Save directory post-op attributes for future lookup
	 */
	fileCBp = nfp_getFileCB (&nfsReqp->fileHandle);
	if (fileCBp == NULL)
	{
		/*
	 	 *Alloc fileCB 
	 	 */
		fileCBp = nfp_allocFileCB();
		if (fileCBp)
			nfs_addFileCBToList(fileCBp, &nfsReqp->fileHandle, 
					nfsReqp->clientCBp);
	}
	if (fileCBp)
	{
		nfsReqp->fileCBp = fileCBp;
		nfs_updateAttr(fileCBp, &fsReaddirRespMsgp->dirAttr, NFS_VER2);
	}

	return (NFX_OK);	

}

/*-----------------------------------------------------------------
 * Name:      	nfsv2_msgRenameResp	
 *
 * Description:
 * Map File System Rename response to a NFS V2 reply procedure.
 *
 * Created by:  Joseph Vu
 *
 * Date Created:        5/28/01
 *
 *-----------------------------------------------------------------
 */
int32
nfsv2_msgRenameResp(int32 status,      	/*operation result status*/
	eee_descBuf_t *bd,      /*file system resp. msg*/
	char *txMsgp,			/*pointer to transmit message buffer*/
	int32 *txMsgLen,		/*current length of the message*/
	nfs_request_t *nfsReqp	/*reqCB*/
)
{
	fs_MsgRenameResp 		*fsRenameRespMsgp;
	nfp_clientCBPtr_t   	clientCBp;
	nfs_clientInfo_t    	*clientInfop;
	uchar8					*freeNamep;
	char 					*msgp;
	int32					length;

	fsRenameRespMsgp = (fs_MsgRenameResp *)bd->buf;
	clientCBp = nfsReqp->clientCBp;
	clientInfop = &clientCBp->protocol.nfp_nfsClientInfo;
	clientInfop->nfs_v2XmtPktCounters.rename++;
	nfs_globalCfgp->nfs_v2XmtPktCounters.rename++;
	nfs_globalCfgp->nfs_debugPktCounters.nfs_PktFsRxCounters.rename++;
	nfs_globalCfgp->nfs_xmtPktCounters.rename++;

	length = 0;

	msgp = txMsgp;
	/*
	 *Set operation status
	 */
	*(uint32 *)msgp = status;

	msgp += 4;
	*txMsgLen += 4;

	freeNamep = nfsReqp->namep;

	if (status != NFS_OK)
	{
		return (NFX_OK);	
	}

	return (NFX_OK);
}

/*-----------------------------------------------------------------
 * Name:       	nfsv2_msgStatfsResp 
 *
 * Description:
 *  Map the file system control message to
 *	NFS V2 STATFS procedure
 *
 * Created by:  Joseph Vu
 *
 * Date Created:        5/28/01
 *
 *-----------------------------------------------------------------
 */
int32
nfsv2_msgStatfsResp (int32 status,		/*operation result status*/
	eee_descBuf_t *bd,      /*file system resp. msg*/
	char *txMsgp,			/*Ptr. to transmit message buffer*/
	int32 *txMsgLen,		/*current length of the message*/
	nfs_request_t *nfsReqp	/*request control block*/
)
{
	fs_MsgStatResp 			*fsstatRespMsgp;
	nfp_clientCBPtr_t   	clientCBp;
	nfs_clientInfo_t    	*clientInfop;
	nfp_fileSysCBPtr_t		fileSysCBp;
	nfs_statInfo_t			*statInfop;
	nfs_fileSysInfo_t		*fileSysInfop;
	fs_MsgRespHdr           *respHdrp;
	char                  	*msgp;
	int32					length;

	fsstatRespMsgp = (fs_MsgStatResp *)bd->buf;
	clientCBp = nfsReqp->clientCBp;
	clientInfop = &clientCBp->protocol.nfp_nfsClientInfo;
	clientInfop->nfs_v2XmtPktCounters.statfs++;
	nfs_globalCfgp->nfs_v2XmtPktCounters.statfs++;
	nfs_globalCfgp->nfs_debugPktCounters.nfs_PktFsRxCounters.statfs++;
	nfs_globalCfgp->nfs_xmtPktCounters.statfs++;

	length = 0;
	msgp = txMsgp;
	/*
	 *Set operation status
	 */
	*(uint32 *)msgp = status;

	msgp += 4;
	*txMsgLen += 4;

	respHdrp = &fsstatRespMsgp->hdr;

	*txMsgLen += length;
	msgp += length;

	length = nfs_storeStatInfo(msgp, 
						&fsstatRespMsgp->stat, NFS_VER2);
	*txMsgLen += length;

	/*
	 *Save attributes & Statfs info
	 */

	fileSysCBp = nfp_getFileSysCB(&nfsReqp->fileHandle);
	if (fileSysCBp != NULL)
	{
#ifdef	NOTUSED
		fileSysInfop = &fileSysCBp->protocol.nfp_nfsFileSysInfo;
		if (respHdrp->attrType & FS_PRE_ATTRS)
		{
			nfs_updatefileSysAttr(fileSysCBp, 
						&fileSysInfop->attr, NFS_VER2);
		}
#endif	//NOTUSED
		fileSysInfop = &fileSysCBp->protocol.nfp_nfsFileSysInfo;
		statInfop = &fileSysInfop->nfs_statInfo;

		
		length = 
		nfs_storeStatInfo ((char *)&statInfop->version.nfs_v2Statfs,
						&fsstatRespMsgp->stat, NFS_VER2);
		
	}
	return (NFX_OK);
}


/*-----------------------------------------------------------------
 * Name:       	nfsv2_msgWriteResp 
 *
 * Description:
 * Map File System Write response to a NFS V2 Write procedure.
 *
 * Created by:  Joseph Vu
 *
 * Date Created:        5/28/01
 *
 *-----------------------------------------------------------------
 */
int32
nfsv2_msgWriteResp (int32 status,		/*operation result status*/
	eee_descBuf_t *bd,      			/*file system resp. msg*/
	char *txMsgp,			/*Pointer to transmit message buffer*/
	int32 *txMsgLen,		/*current length of the message*/
	nfs_request_t *nfsReqp	/*request control block*/
)
{
	int32                   length;
	char                  	*msgp;
	fs_MsgRespHdr           *respHdrp;
	fs_Attr                 *postOpAttrp;
	fs_Attr					*attrp;
	nfp_fileCBPtr_t			fileCBp;
	nfs_clientInfo_t    	*clientInfop;
	nfp_clientCBPtr_t   	clientCBp;
	fs_MsgWriteResp 		*fsWriteRespMsgp;

	fsWriteRespMsgp = (fs_MsgWriteResp *)bd->buf;
	nfs_globalCfgp->nfs_xmtPktCounters.write++;	
	nfs_globalCfgp->nfs_debugPktCounters.nfs_PktFsRxCounters.write++;
	nfs_globalCfgp->nfs_v2XmtPktCounters.write++;
	clientCBp = nfsReqp->clientCBp;
	clientInfop = &clientCBp->protocol.nfp_nfsClientInfo;
	clientInfop->nfs_v2XmtPktCounters.write++;

	length = 0;
	msgp = txMsgp;
	/*
	 *Set operation status
	 */
	*(uint32 *)msgp = status;

	msgp += 4;
	*txMsgLen += 4;

	respHdrp = &fsWriteRespMsgp->hdr;

	/*
	 *Store post-op attributes
	 */
	attrp = &(fsWriteRespMsgp->attr[0]);
	postOpAttrp = nfs_msgFindAttrType(attrp,
					fsWriteRespMsgp->numAttrs, FS_POST_ATTRS);
	if (postOpAttrp && (respHdrp->attrType & FS_POST_ATTRS))
	{
		length = nfs_storeV2Attributes(nfsReqp->reqType, 
					NFS_ATTR_AVAIL,
					postOpAttrp, msgp);
		if (cacheEnable)
		{
			/*
		 	 *Save updated attributes in FileCB
		 	 */
			fileCBp = nfp_getFileCB (&nfsReqp->fileHandle);
			if (fileCBp != NULL)
			{
				nfs_updateAttr(fileCBp, postOpAttrp, NFS_VER2);
			}
		}
	}
	else
		length = nfs_storeV2Attributes(nfsReqp->reqType, 
					NFS_ATTR_NOT_AVAIL,
					postOpAttrp, msgp);

	*txMsgLen += length;
	msgp += length;


	return (NFX_OK);
}

/*-----------------------------------------------------------------
 * Name:	nfsv2_msgReadResp 
 *  
 * Description:
 * Map File System Read response to a NFS V2 Read procedure.
 *
 * Created by:  Joseph Vu
 *
 * Date Created:        5/28/01
 *
 *-----------------------------------------------------------------
 */
int32
nfsv2_msgReadResp( int32 status,	/*operation result status*/
	eee_descBuf_t *bd,      /*file system resp. msg*/
	char *txMsgp,			/*Pointer to transmit message buffer*/
	int32 *txMsgLen,		/*current length of the message*/
	nfs_request_t *nfsReqp	/*request control block*/
)
{
	int32					codIdx;
	int32					pad;
	int32					length;
	char					*msgp;
	fs_MsgRespHdr           *respHdrp;
	nfp_fileCBPtr_t			fileCBp;
	fs_Attr					*attrp;
	fs_Attr					*fileAttrp;
	nfs_clientInfo_t    	*clientInfop;	
	nfp_clientCBPtr_t   	clientCBp;
	fs_MsgReadResp 			*fsReadRespMsgp;
	pkt_desc_t				*tplXmitPktDescp;

	fsReadRespMsgp = (fs_MsgReadResp *)bd->buf;
	nfs_globalCfgp->nfs_v2XmtPktCounters.read++;
	clientCBp = nfsReqp->clientCBp;
	clientInfop = &clientCBp->protocol.nfp_nfsClientInfo;
	clientInfop->nfs_v2XmtPktCounters.read++;
	nfs_globalCfgp->nfs_xmtPktCounters.read++;	
	nfs_globalCfgp->nfs_debugPktCounters.nfs_PktFsRxCounters.read++;


	length = 0;

	msgp = txMsgp;
	/*
	 *Set operation status
	 */
	*(uint32 *)msgp = status;

	length = 4;
	msgp += 4;
	*txMsgLen += 4;

	//Check for error status from the file system
	if (status != NFS_OK)
		return (NFX_ERR);

	respHdrp = &fsReadRespMsgp->hdr;
	attrp = &(fsReadRespMsgp->attr[0]);
	fileAttrp = nfs_msgFindAttrType(attrp,
					fsReadRespMsgp->numAttrs, FS_POST_ATTRS);
	if (fileAttrp && (respHdrp->attrType & FS_POST_ATTRS))
	{
		length = nfs_storeV2Attributes(nfsReqp->reqType, 
				NFS_ATTR_AVAIL, 
				fileAttrp, msgp);
		/*
		 *Save updated attributes in FileCB
		 */
		if (cacheEnable)
		{
			fileCBp = nfp_getFileCB (&nfsReqp->fileHandle);
			if (fileCBp != NULL)
				nfs_updateAttr(fileCBp, fileAttrp, NFS_VER2);
		}

	}
	else
	{
		length = nfs_storeV2Attributes(nfsReqp->reqType, 
				NFS_ATTR_NOT_AVAIL, 
				fileAttrp, msgp);
	}

	*txMsgLen += length;
	msgp += length;

	
    /* Account for the length in opaque < > data */

    *(uint32 *)msgp = fsReadRespMsgp->bytesRead;
	*txMsgLen += sizeof(uint32);

	/*
	 *save info fs Read Complete Indication
	 */	
	nfsReqp->readXmitCmpInd.fileOffset = fsReadRespMsgp->fileOffset;
	nfsReqp->readXmitCmpInd.bytesRead = fsReadRespMsgp->bytesRead;
	nfsReqp->readXmitCmpInd.fsPrivate = fsReadRespMsgp->fsPrivate;

	nfs_globalCfgp->nfs_debugPktCounters.nfs_eeeRamAlloc2++;
	tplXmitPktDescp = nfsReqp->pktDescp;
	/*
	 * RPC requires data to be padded to 4 byte.
	 */
	pad = (fsReadRespMsgp->bytesRead & 0x03);
	if (pad)
		pad = 4 - pad;

	/*
	 *Fill in data part
	 */
	tplXmitPktDescp->padSize  = pad;
	tplXmitPktDescp->dataSize = pad +
				fsReadRespMsgp->bytesRead;	//Total number of bytes

	if (  ((fsReadRespMsgp->bytesRead <= 0) && 
		   (fsReadRespMsgp->nbrUsedCod != 0))   ||
			((fsReadRespMsgp->bytesRead > 0) &&
			(fsReadRespMsgp->nbrUsedCod > NFX_MAX_COD_MSG)) )
	{
		panic ("Incorrect # of used CODs (%d) rcv'd from the file system!\n", 
		fsReadRespMsgp->nbrUsedCod);
	}
	tplXmitPktDescp->numOfCod = fsReadRespMsgp->nbrUsedCod;
	for (codIdx=0; codIdx < tplXmitPktDescp->numOfCod; codIdx++)
	{
		tplXmitPktDescp->codEntry[codIdx] = fsReadRespMsgp->cod[codIdx];
	}

#ifdef  NFS_DBG_COD
    {  
        nfx_cod_entry_t *tempCod;
        int32           cod_totalNumOfBytes;
        int32           codIdx;
        int32           dataSize;
        int32           numOfCod;

        numOfCod = fsReadRespMsgp->nbrUsedCod;
        dataSize = fsReadRespMsgp->bytesRead;
        tempCod = &fsReadRespMsgp->cod[0];;
        cod_totalNumOfBytes = 0;
        for (codIdx =0; codIdx < numOfCod; codIdx++)
        {
            cod_totalNumOfBytes +=  tempCod->length;
            tempCod++;
        }
        if (cod_totalNumOfBytes != dataSize)
        {
            tempCod = &fsReadRespMsgp->cod[0];;
            for (codIdx =0; codIdx < numOfCod; codIdx++, tempCod++)
            {
                printf("cod: %llx offset: %d length: %d\n",
                    tempCod->cod, tempCod->offset, tempCod->length);
            }
            panic("%s: dataSize: %d numOfCod:%d cod_totalNumOfBytes: %d\n",
                    __FUNCTION__, dataSize, numOfCod, cod_totalNumOfBytes);
        }
    }
#endif  //NFS_DBG_COD
	
	return (NFX_OK);

}

/*-----------------------------------------------------------------
 * Name:nfsv2_msgErrResp		
 *  
 * Description:
 *	Received an invalid message type received from the file system, 
 *  Set request state done to clean up request and return. 
 *
 * Created by:  Joseph Vu
 *
 * Date Created:        5/28/01
 *
 *-----------------------------------------------------------------
 */

int32
nfsv2_msgErrResp( int32 status,	/*operation result status*/
	eee_descBuf_t *bd,      /*file system resp. msg*/
	char *txMsgp,			/*Pointer to transmit message buffer*/
	int32 *txMsgLen,		/*current length of the message*/
	nfs_request_t *nfsReqp	/*request control block*/
)
{
	nfs_globalCfgp->nfs_debugPktCounters.nfs_rxInvalidFSResp++;
 	nfsReqp->reqState = NFS_REQ_DONE;
	return (NFX_OK);
}
#endif  //NFP_TXRX
