/* vi:ts=4
 *-----------------------------------------------------------------
 *
 * Name:		nfsv2-proc.h
 *
 * RCS:  $Id: nfsv2-proc.h,v 1.2 2002/10/04 01:29:25 shaun Exp $
 *
 * Copyright (C) 2002, Agile Storage Inc.
 *
 * Description: Contains NFS V2 Procedures function prototypes.
 *
 * Created by: 	Shaun Hui
 *
 * Date Created: 10/01/02
 *
 * History: 8/27/02 - Shaun splitted nfs-proc.c into nfsv3-proc.c
 *                    nfsv2-proc.c & nfs-acl.c
 *
 *-----------------------------------------------------------------
 */
#ifndef _NFSV2_PROC_H
#define _NFSV2_PROC_H

/*-----------------------------------------------------------------
 * Name:    nfsv2_procRxNullReq
 *
 * Description: Handles NFS V2 NULL Requests.
 *
 *-----------------------------------------------------------------
 */
int32   nfsv2_procRxNullReq(        nfs_request_t  *nfsReqp);

/*-----------------------------------------------------------------
 * Name: nfsv2_procRxGetAttrReq
 *
 * Description: Handle NFS V2 Get Attribute Requests.
 *
 *-----------------------------------------------------------------
 */
int32   nfsv2_procRxGetAttrReq(     nfs_request_t  *nfsReqp);

/*-----------------------------------------------------------------
 * Name:        nfsv2_procRxSetAttrReq
 *
 * Description: Handle NFS V2 Set Attribute Requests.
 *
 *-----------------------------------------------------------------
 */
int32   nfsv2_procRxSetAttrReq(     nfs_request_t  *nfsReqp);

/*-----------------------------------------------------------------
 * Name: nfsv2_procRxLookupReq
 *
 * Description: Handle NFS V2 Lookup Requests.
 *
 *-----------------------------------------------------------------
 */
int32   nfsv2_procRxLookupReq(      nfs_request_t  *nfsReqp);

/*-----------------------------------------------------------------
 * Name:        nfsv2_procRxReadLinkReq
 *
 * Description: Handle NFS V2 Readlink requests.
 *
 *-----------------------------------------------------------------
 */
int32   nfsv2_procRxReadLinkReq(    nfs_request_t  *nfsReqp);

/*-----------------------------------------------------------------
 * Name:        nfsv2_procRxReadReq
 *
 * Description: Handle NFS V2 Read Requests.
 *
 *-----------------------------------------------------------------
 */
int32   nfsv2_procRxReadReq(        nfs_request_t  *nfsReqp);

/*-----------------------------------------------------------------
 * Name:        nfs_procRxWriteReq
 *
 * Description: Handle NFS V2 Write requests.
 *
 *-----------------------------------------------------------------
 */
int32   nfsv2_procRxWriteReq(       nfs_request_t  *nfsReqp);

/*-----------------------------------------------------------------
 * Name:        nfsv2_procRxCreateReq
 *
 * Description: Handle NFS V2 Create Requests.
 *
 *-----------------------------------------------------------------
 */
int32   nfsv2_procRxCreateReq(      nfs_request_t  *nfsReqp);

/*-----------------------------------------------------------------
 * Name: nfsv2_procRxMkdirReq
 *
 * Description: Handle NFS V2 Mkdir Requests.
 *
 *-----------------------------------------------------------------
 */
int32   nfsv2_procRxMkdirReq(       nfs_request_t  *nfsReqp);

/*-----------------------------------------------------------------
 * Name:        nfsv2_procRxSymlinkReq
 *
 * Description: Handle NFS V2 Symlink Requests.
 *
 *-----------------------------------------------------------------
 */
int32   nfsv2_procRxSymlinkReq(     nfs_request_t  *nfsReqp);

/*-----------------------------------------------------------------
 * Name: nfsv2_procRxRemoveReq
 *
 * Description: Handle NFS V2 Remove Requests.
 *
 *-----------------------------------------------------------------
 */
int32   nfsv2_procRxRemoveReq(      nfs_request_t  *nfsReqp);

/*-----------------------------------------------------------------
 * Name: nfsv2_procRxRmdirReq
 *
 * Description: Handle NFS V2 Rmdir Requests.
 *
 *-----------------------------------------------------------------
 */
int32   nfsv2_procRxRmdirReq(       nfs_request_t  *nfsReqp);

/*-----------------------------------------------------------------
 * Name:        nfsv2_procRxRenameReq
 *
 * Description: Handle NFS V2 Rename Requests.
 *
 *-----------------------------------------------------------------
 */
int32   nfsv2_procRxRenameReq(      nfs_request_t  *nfsReqp);

/*-----------------------------------------------------------------
 * Name:        nfsv2_procRxLinkReq
 *
 * Description: Handle NFS V2 Link Requests.
 *
 *-----------------------------------------------------------------
 */
int32   nfsv2_procRxLinkReq(        nfs_request_t  *nfsReqp);

/*-----------------------------------------------------------------
 * Name:        nfsv2_procRxReaddirReq
 *
 * Description: Handle NFS V2 Readdir Requests.
 *
 *-----------------------------------------------------------------
 */
int32   nfsv2_procRxReaddirReq(     nfs_request_t  *nfsReqp);

/*-----------------------------------------------------------------
 * Name:        nfsv2_procRxStatfsReq
 *
 * Description: Handle NFS V2 StatFS Requests.
 *
 *-----------------------------------------------------------------
 */
int32   nfsv2_procRxStatfsReq(      nfs_request_t  *nfsReqp);

/*-----------------------------------------------------------------
 * Name:        nfsv2_procRxNoOpReq
 *
 * Description: No Op routine for obsolete procedures.
 *
 *-----------------------------------------------------------------
 */
int32   nfsv2_procRxNoOpReq(        nfs_request_t  *nfsReqp);

#endif	/* _NFSV2_PROC_H */
