/* vi:ts=4
 *-----------------------------------------------------------------
 *
 * Name:	  nfsv3-msg.h
 *
 * RCS:  $Id: nfsv3-msg.h,v 1.1 2002/09/23 23:30:07 shaun Exp $
 *
 * Copyright (C) 2001, Agile Storage Inc.
 *
 * Description: Contains function prototypes for routines that
 *				handle NFS V3 response from File System. 
 *
 *-----------------------------------------------------------------
 */
#ifndef _NFSV3_MSG_H
#define _NFSV3_MSG_H

/*-----------------------------------------------------------------
 * Name:		nfsv3_msgGetAttrResp 
 *
 * Description: Map file system Getattr response message to a NFS 
 *				reply procedure.
 *
 *-----------------------------------------------------------------
 */
int32   /*
         *NFX_OK or NFX_ERR
         */
nfsv3_msgGetAttrResp( int32 		status, 	/*operation result status*/
					  eee_descBuf_t *bd,		/*file system resp. msg*/
					  char 			*txMsgp,	/*Pointer to tx msg buffer*/
					  int32 		*txMsgLen,  /*current length of message*/
					  nfs_request_t *nfsReqp );	/*reqCB*/

/*-----------------------------------------------------------------
 * Name:       	nfsv3_msgLookupResp	
 *
 * Description: Map File System Lookup response to a NFS V3 reply. 
 *
 *-----------------------------------------------------------------
 */
int32
nfsv3_msgLookupResp(int32 			status,		/*operation result status*/
					eee_descBuf_t 	*bd,		/*file system resp. msg*/
					char 			*txMsgp,	/*Pointer to tx msg buffer*/
					int32 			*txMsgLen, 	/*current length of message*/
					nfs_request_t 	*nfsReqp );	/*reqCB*/ 

/*-----------------------------------------------------------------
 * Name: 		nfsv3_msgSetAttrResp 
 *
 * Description: Map File System SetAttr response to a NFS reply procedure.
 *
 *-----------------------------------------------------------------
 */
int32
nfsv3_msgSetAttrResp(int32 			status,		/*operation result status*/
					 eee_descBuf_t	*bd,		/*file system resp. msg*/
					 char 			*txMsgp,	/*Pointer to tx msg buffer*/
					 int32 			*txMsgLen,	/*current length of message*/
					 nfs_request_t 	*nfsReqp );	/*reqCB*/

/*-----------------------------------------------------------------
 * Name: 		nfsv3_msgAccessResp 
 *
 * Description: Map File System Access response to a NFS V3 reply procedure.
 *
 *-----------------------------------------------------------------
 */
int32
nfsv3_msgAccessResp(int32 			status,		/*operation result status*/
					eee_descBuf_t	*bd,		/*file system resp. msg*/	
					char 			*txMsgp,	/*Pointer to tx msg buffer*/
					int32 			*txMsgLen, 	/*current length of message*/
					nfs_request_t 	*nfsReqp );	/*reqCB*/

/*-----------------------------------------------------------------
 * Name:		nfsv3_msgSymlinkResp 
 *
 * Description: Map File System Symlink response to a NFS reply procedure.
 *
 *-----------------------------------------------------------------
 */
int32
nfsv3_msgSymlinkResp(int32 			status,		/*operation result status*/
					 eee_descBuf_t	*bd,		/*file system resp. msg*/
					 char 			*txMsgp, 	/*pointer to tx Msg buffer*/
					 int32 			*txMsgLen, 	/*current length of message*/
					 nfs_request_t 	*nfsReqp );	/*reqCB*/

/*-----------------------------------------------------------------
 * Name:      	nfsv3_msgHardLinkResp 
 *
 * Description: Map File System Link response to a NFS V3 reply procedure.
 *
 *-----------------------------------------------------------------
 */
int32
nfsv3_msgHardLinkResp(int32			status,		/*operation result status*/
					  eee_descBuf_t *bd,      	/*file system resp. msg*/
					  char 			*txMsgp,	/*pointer to tx msg buffer*/
					  int32 		*txMsgLen,	/*current length of message*/
					  nfs_request_t *nfsReqp );	/*reqCB*/

/*-----------------------------------------------------------------
 * Name: 		nfsv3_msgReadlinkResp 
 *
 * Description: Map File System Readlink response to a NFS V3 reply. 
 *
 *-----------------------------------------------------------------
 */
int32	/*
	 	 *NFX_OK or NFX_ERR
	 	 */
nfsv3_msgReadlinkResp(int32 		status,		/*operation result status*/
					  eee_descBuf_t *bd,		/*file system resp. msg*/
					  char 			*txMsgp, 	/*Pointer to tx msg buffer*/
					  int32 		*txMsgLen, 	/*length of msg buffer*/
					  nfs_request_t *nfsReqp );	/*req control block*/

/*-----------------------------------------------------------------
 * Name:        nfsv3_msgCreateResp
 *
 * Description: Map File System Create response to a NFS V3 reply procedure.
 *
 *-----------------------------------------------------------------
 */
int32   /*
         *NFS_OK or NFX_ERR
         */
nfsv3_msgCreateResp(int32 			status,		/*Operation result status*/
					eee_descBuf_t	*bd,		/*file system resp. msg*/
					char 			*txMsgp,	/*Poiner to tx msg buffer*/
					int32 			*txMsgLen,	/*length of the message*/
					nfs_request_t 	*nfsReqp );	/*reqCB*/

/*-----------------------------------------------------------------
 * Name:		nfsv3_msgRemoveResp 
 *
 * Description: Map File System Remove response to a NFS reply procedure.
 *
 *-----------------------------------------------------------------
 */
int32
nfsv3_msgRemoveResp(int32 			status,		/*Operation result status*/
					eee_descBuf_t	*bd,		/*file system resp. msg*/
					char 			*txMsgp,	/*pointer to tx msg buffer*/
					int32 			*txMsgLen,	/*current length of message*/
					nfs_request_t	*nfsReqp );	/*reqCB*/

/*-----------------------------------------------------------------
 * Name:      	nfsv3_msgRmdirResp 
 *
 * Description: Map File System RMDIR response to a NFS reply procedure.
 *
 *-----------------------------------------------------------------
 */
int32
nfsv3_msgRmdirResp( int32 			status,		/*Operation result status*/
					eee_descBuf_t	*bd,		/*file system resp. msg*/
					char 			*txMsgp,	/*pointer to tx msg buffer*/
					int32 			*txMsgLen,	/*current length of message*/
					nfs_request_t	*nfsReqp );	/*reqCB*/
       

/*-----------------------------------------------------------------
 * Name:       	nfsv3_msgMkdirResp 
 *
 * Description: Map File System Mkdir response to a NFS reply procedure.
 *
 *-----------------------------------------------------------------
 */
int32
nfsv3_msgMkdirResp(int32 			status,		/*operation result status*/
					eee_descBuf_t	*bd,		/*file system resp. msg*/
					char 			*txMsgp,	/*pointer to tx msg buffer*/
					int32 			*txMsgLen, 	/*current length of message*/
					nfs_request_t 	*nfsReqp );	/*reqCB*/

/*-----------------------------------------------------------------
 * Name:        nfsv3_msgReaddirResp
 *
 * Description: Map File System Readdir response to a NFS V3 reply procedure.
 *
 *-----------------------------------------------------------------
 */
int32	/*
		 *NFX_OK  or NFX_ERR
	 	 */
nfsv3_msgReaddirResp(int32 			status,		/*Operation result status*/
					 eee_descBuf_t 	*bd,		/*file system resp. msg*/
					 char 			*txMsgp,	/*Pointer to tx msg buffer*/
					 int32 			*txMsgLen,	/*current length of message*/
					 nfs_request_t 	*nfsReqp );	/*reqCB*/

/*-----------------------------------------------------------------
 * Name:        nfsv3_msgReaddirplusResp
 *
 * Description: Map File System Readdirplus response to a NFS V3 reply.
 *
 *-----------------------------------------------------------------
 */
int32	/*
		 *NFX_OK  or NFX_ERR
         */
nfsv3_msgReaddirplusResp(int32 			status,		/*Operation result status*/
						 eee_descBuf_t	*bd,		/*file system resp. msg*/
						 char 			*txMsgp, 	/*Pointer to tx msg buf*/
						 int32 			*txMsgLen,  /*current length of msg*/
						 nfs_request_t 	*nfsReqp );	/*reqCB*/

/*-----------------------------------------------------------------
 * Name: 		nfsv3_msgMknodResp 
 *
 * Description: Map File System MKNOD response to a NFS V3 reply procedure.
 *
 *-----------------------------------------------------------------
 */
int32
nfsv3_msgMknodResp( int32 			status,		/*operation result status*/
					eee_descBuf_t 	*bd,		/*file system resp. msg*/
					char 			*txMsgp,	/*Pointer to tx msg buffer*/
					int32 			*txMsgLen,	/*current length of message*/
					nfs_request_t 	*nfsReqp );	/*reqCB*/

/*-----------------------------------------------------------------
 * Name:      	nfsv3_msgRenameResp	
 *
 * Description: Map File System Rename response to a NFS V3 reply procedure.
 *
 *-----------------------------------------------------------------
 */
int32
nfsv3_msgRenameResp(int32 			status,		/*operation result status*/
					eee_descBuf_t 	*bd,		/*file system resp. msg*/
					char 			*txMsgp,	/*pointer to tx msg buffer*/
					int32 			*txMsgLen,	/*current length of message*/
					nfs_request_t 	*nfsReqp );	/*reqCB*/


/*-----------------------------------------------------------------
 * Name:       	nfsv3_msgStatfsResp 
 *
 * Description: Map the file system control message to NFS V3 STATFS 
 *
 * Created by:  Joseph Vu
 *
 * Date Created: 5/28/01
 *
 *-----------------------------------------------------------------
 */
int32
nfsv3_msgStatfsResp (int32 			status,		/*operation result status*/
					 eee_descBuf_t 	*bd,		/*file system resp. msg*/
					 char 			*txMsgp,	/*Ptr. to tx msg buffer*/
					 int32 			*txMsgLen,	/*current length of message*/
					 nfs_request_t 	*nfsReqp );	/*request control block*/

/*-----------------------------------------------------------------
 * Name:       	nfsv3_msgPathconfResp 
 *
 * Description: Map the file system control message to NFS V3 PATHCONF 
 *
 *-----------------------------------------------------------------
 */
int32
nfsv3_msgPathconfResp ( int32 			status,		/*operation result status*/
						eee_descBuf_t 	*bd,		/*file system resp. msg*/
						char 			*txMsgp,	/*Pointer to tx msg buf*/
						int32 			*txMsgLen,	/*current length of msg*/
						nfs_request_t 	*nfsReqp );	/*request control block*/
	 
/*-----------------------------------------------------------------
 * Name:       	nfsv3_msgFSInfoResp 
 *
 * Description: Map the file system control message to NFS V3 FSINFO 
 *
 *-----------------------------------------------------------------
 */
int32
nfsv3_msgFSInfoResp(int32 			status,		/*operation result status*/
					eee_descBuf_t 	*bd,		/*file system resp. msg*/
					char 			*txMsgp,	/*Pointer to tx msg buffer*/
					int32 			*txMsgLen,	/*current length of message*/
					nfs_request_t 	*nfsReqp );	/*request control block*/

/*-----------------------------------------------------------------
 * Name:       	nfsv3_msgWriteResp 
 *
 * Description: Map File System Write response to a NFS V3 Write .
 *
 *-----------------------------------------------------------------
 */
int32
nfsv3_msgWriteResp (int32 			status,		/*operation result status*/
					eee_descBuf_t 	*bd,		/*file system resp. msg*/
					char 			*txMsgp,	/*Pointer to tx msg buffer*/
					int32 			*txMsgLen,	/*current length of message*/
					nfs_request_t 	*nfsReqp );	/*request control block*/

/*-----------------------------------------------------------------
 * Name:       nfsv3_msgCommitResp 
 *
 * Description: Map File System COMMIT response to a NFS V3 COMMIT procedure.
 *
 *-----------------------------------------------------------------
 */
int32
nfsv3_msgCommitResp (int32 			status,		/*operation result status*/
					 eee_descBuf_t	*bd,		/*file system resp. msg*/
					 char 			*txMsgp,	/*Pointer to tx msg buffer*/
					 int32 			*txMsgLen,	/*current length of message*/
					 nfs_request_t	*nfsReqp );	/*request control block*/

/*-----------------------------------------------------------------
 * Name:	nfsv3_msgReadResp 
 *  
 * Description: Map File System Read response to a NFS V3 Read procedure.
 *
 *-----------------------------------------------------------------
 */
int32
nfsv3_msgReadResp(int32 			status,		/*operation result status*/
				  eee_descBuf_t 	*bd,		/*file system resp. msg*/
				  char 				*txMsgp,	/*Pointer to tx msg buffer*/
				  int32 			*txMsgLen,	/*current length of message*/
				  nfs_request_t 	*nfsReqp );	/*request control block*/

/*-----------------------------------------------------------------
 * Name:nfsv3_msgErrResp
 *
 * Description:
 *  Received an invalid message type received from the file system,
 *  Set request state done to clean up request and return.
 *
 *-----------------------------------------------------------------
 */

int32
nfsv3_msgErrResp( int32 		status,		/*operation result status*/
				  eee_descBuf_t *bd,		/*file system resp. msg*/
				  char 			*txMsgp,	/*Pointer to tx msg buffer*/
				  int32 		*txMsgLen,	/*current length of message*/
				  nfs_request_t *nfsReqp );	/*request control block*/
#endif /* _NFSV3_MSG_H */
