/* vi:ts=4
 *-----------------------------------------------------------------
 *
 * Name:        nlm-init.c
 *
 * RCS:  $Id: nlm-init.c,v 1.3 2002/11/06 03:35:39 joseph Exp $
 *
 * Copyright (C) 2001, Agile Storage Inc.
 *
 * Description:
 *	Contain Network Lock Manager (NLM) initialize functions.
 *
 * Created by: Joseph Vu
 *
 * Date Created:        6/28/02
 *
 *-----------------------------------------------------------------
 */
#ifdef NFP_TXRX

#include <nfx-incl.h>
#include <exports/pkt-api.h>
#include <exports/vs-mem-api.h>
#include <exports/lock-api.h>
#include <exports/nfs-api.h>
#include "nlm.h"
#include <exports/nsm-api.h>
#include "nlm-prototypes.h"

#define NLM_DBG_PRINTF  if (trace_enable) printf	

nlm_globalCfg_t *nlm_globalCfgp;		//NLM global cfg
//extern nlm_portMapCb_t	nlm_portMapCB;

/*-----------------------------------------------------------------
 * Name:        nlm_init
 *
 * Description:
 *		NLM initialize function on the TX/RX CPU.
 *
 * Created by:  Joseph Vu
 *
 * Date Created:        6/28/02
 *
 *-----------------------------------------------------------------
 */
int32
nlm_init(void)
{
	int32		rc;
    nsm_nlmCallbackCB_t     callbackCB;
#ifdef NOT_USED
	nlm_lockRec_t			*lockRecp;
	int32					i;
#endif

	//Alloc & Initialize NLM global Config
	nlm_globalCfgp = 
			(nlm_globalCfg_t *)eee_ramAlloc(sizeof(nlm_globalCfg_t) );

	if (nlm_globalCfgp == NULL)
		panic ("%s: Failure to alloc NLM global Cfg!\n",__FUNCTION__);


	bzero(nlm_globalCfgp, sizeof(nlm_globalCfg_t));

	nlm_globalCfgp->reqTimeoutHandle = 
			eee_timerCreate(NLM_REQ_TIMEOUT_PERIOD,
							EEE_TIMER_REPEAT);

	if (nlm_globalCfgp->reqTimeoutHandle == TIMER_ERR_NO_TIMER_AVAILABLE)
		panic ("%s: Failure to create NLM request timer!\n",
						__FUNCTION__);

	rc = eee_timerStart(nlm_globalCfgp->reqTimeoutHandle,
					(void *)nlm_reqTimeoutHandler, 0, 0, 0, 0);
	if (rc != NFX_OK)
		panic("%s: Failure to start requests timer!\n", __FUNCTION__);


	//Create & start grace period timer
	//
	nlm_globalCfgp->gracePeriodTimeoutHandle = 
			eee_timerCreate(NLM_GRACE_TIMEOUT_PERIOD,
							EEE_TIMER_ONCE);

	if (nlm_globalCfgp->gracePeriodTimeoutHandle == 
								TIMER_ERR_NO_TIMER_AVAILABLE)
		panic ("%s: Failure to create Grace Period timer!\n",
						__FUNCTION__);

	rc = eee_timerStart(nlm_globalCfgp->gracePeriodTimeoutHandle,
					(void *)nlm_gracePeriodTimeoutHandler, 
					0, 0, 0, 0);
	if (rc != NFX_OK)
		panic("%s: Failure to start Grace Period timer!\n",
					   	__FUNCTION__);
	nlm_globalCfgp->gracePeriod = 1;

	//Bind NLM server with TPL
	
	if (nlm_tplBindReq (0) != NFX_OK)
	{
		panic ("%s: TPL Bind failed!\n", __FUNCTION__);
	}


	/*
	 *Register with EEE and app Id
	 */
	nlm_globalCfgp->eee_nlmAppId = 
		eee_allocateApplicationId("nlm_srvr");
	if (nlm_globalCfgp->eee_nlmAppId == EEE_ERR_INVALID_APP_NAME)
	{
		/*
		 *TODO:  Handle error
		 */		
		panic("%s: Failure to alloc NLM Application ID!\n",
			__FUNCTION__);
	}
	/*
	 *Save appId
	 */
	if (eee_registerReceiveFunc(nlm_globalCfgp->eee_nlmAppId, 
				nlm_msgRcv, NULL) != NFX_OK)
	{
		panic("%s: Register nlm_msgRcv() failed!\n",
				__FUNCTION__);
	}

	/*
	 *Get file system App Id
	 */
	nlm_globalCfgp->eee_efsAppId = eee_getApplicationId("efs");
	if (nlm_globalCfgp->eee_efsAppId == EEE_ERR_INVALID_APP_NAME)
		panic("%s: Failure to get EFS Application ID!\n",
				__FUNCTION__);
	nlm_globalCfgp->reqTimeoutHandle = 
		eee_timerCreate(NLM_REQ_TIMEOUT_PERIOD,
						EEE_TIMER_REPEAT);

	//Set source & destinattion port for NLM & file system
	nlm_globalCfgp->srcPort = 
		PACK_UNICAST_PORT_ADDRESS(
						nfxMySlotId,
						nfxMyCPUId,
						nlm_globalCfgp->eee_nlmAppId,
						EEE_CONTROL,
						NFX_COS_HIGH,
						0);

	nlm_globalCfgp->efsDstPort = 
		PACK_UNICAST_PORT_ADDRESS(
							nfxMySlotId,
							NFX_NFP_FP2,
							nlm_globalCfgp->eee_efsAppId,
							EEE_CONTROL,
							NFX_COS_HIGH,
							0);
	


	if (nlm_portMapBindReq (0) != NFX_OK)
		panic ("%s: nlm_portMapBindReq() failed\n",
					   	__FUNCTION__);
#ifdef	NOTUSED
	nlm_globalCfgp->pFileCbHashTbl = nlm_hashCreate (NFP_MAX_LOCK);
	if (nlm_globalCfgp->pFileCbHashTbl == NULL)
		panic("%s: Failure to alloc FileCB hash table!\n",
						__FUNCTION__);
#endif	//NOTUSED
	
	//Init XID generate a random number
	nlm_globalCfgp->cmdXid = 0x10000;	
	nlm_globalCfgp->cookie = 0;


    //------------------
    // Register callback
    //------------------
    callbackCB.monCmpInd = nlm_nsmMonCmpInd;
    callbackCB.unmonCmpInd = nlm_nsmUnmonCmpInd;
    callbackCB.clientDownInd = nlm_nsmClientDownInd;
    nsm_registerCallback(&callbackCB);

#ifdef NOT_USED
	/*-------------------------------------------
	 *Allocate lock owner list table
	 *--------------------------------------------
	 */
	
	nlm_globalCfgp->lockTblSize = sizeof(nlm_lockRec_t) *
								NLM_MAX_LOCK_REC;

	nlm_globalCfgp->lockAllocTbl = eee_ramAlloc(nlm_globalCfgp->lockTblSize);
	if (nlm_globalCfgp->lockAllocTbl == NULL)
		panic("%s: Failure to Allocate lock Table!\n"
						, __FUNCTION__);
#ifdef	NLM_DEBUG
	printf("%s: lockTblSize: %d  lockAllocTbl: %p\n", 
					__FUNCTION__,
					nlm_globalCfgp->lockTblSize,
	 			 	nlm_globalCfgp->lockAllocTbl);
#endif
	/*------------------------
	 *Init lock record lists
	 *-------------------------
	 */
	DLL_INIT(&nlm_globalCfgp->lockRecAllocList.list);	

	lockRecp = 	nlm_globalCfgp->lockAllocTbl;
	for (i = 0; i < NLM_MAX_LOCK_REC ; i++)
	{
		bzero(lockRecp, sizeof(nlm_lockRec_t));
		DLL_INSERT_TAIL(&nlm_globalCfgp->lockRecAllocList.list, 
						lockRecp, lockList);
		nlm_globalCfgp->lockRecAllocList.numOfEntries++;
		lockRecp++;
	}
	nlm_globalCfgp->lockRecFreeCnt = NLM_MAX_LOCK_REC;
	nlm_globalCfgp->lockRecLwm = NLM_MAX_LOCK_REC;

#else
    vs_memSetDataSize(vs_mem_dt_nlm_request_t, sizeof(nlm_request_t));
#endif // NOT_USED

	return (NFX_OK);
}

#endif	//NFP_TXRX
