/* vi:ts=4
 *------------------------------------------------------------
 *
 * Name:        nlm-pmapTpl.c
 *
 * RCS:  $Id: nlm-pmapTpl.c,v 1.5 2002/10/04 02:33:40 joseph Exp $
 *
 * Copyright (C) 2001, Agile Storage Inc.
 *
 * Description:
 *
 * Created by: Joseph Vu
 *
 * Date Created:        7/28/02
 *
 *--------------------------------------------------------------
 */
#include <nfx-incl.h>
#include <exports/pkt-api.h>
#include <exports/nfs-api.h>
#include "nlm-prototypes.h"
#include "../sm-tpl/tpl.h"
#define NLM_DBG_FUNC_PRINTF if (nlm_traceDebugEnable) printf

//#define		NLM_DEBUG

/*-----------------------------------------------------------------
 * Name:   nlm_pmapTplAddConnInd    	
 *
 * Description:
 *	Add connection indication from the transport layer to
 *	adds a new connection. 
 *
 * Created by:  Joseph Vu
 *
 * Date Created:        7/30/02
 *
 *-----------------------------------------------------------------
 */
int32
nlm_pmapTplAddConnInd (
	tpl_bindCb_t	*bindp,		
	void 			*connHdlp, 
	void 			**pAppHdl, 
	uint32 			raddr, 
	ushort16 		rport 
)
{
	nfp_clientCBPtr_t	clientCBp;
//	tpl_connCb_t		*pConn;

#ifdef	NLM_DEBUG
	NLM_DBG_FUNC_PRINTF("%s: pConnHdl: %p raddr: %08x rport: %d\n",
		__FUNCTION__, connHdlp, htonl(raddr), htons(rport));
#endif

	clientCBp = nfp_getClientCBbyAddr(raddr);
	if (clientCBp == NULL)
	{   
		/*
	 	 *Allocate NFP Client Control Block (CCB)
	 	 */
		clientCBp = nfp_allocClientCB();
		if (clientCBp == NULL)
			return (TPL_ERR_NO_RSRC);
    }

#ifdef  NOTUSED
	pConn = (tpl_connCb_t        *)connHdlp;
	//Save update socket psock & 
	//return appHdl to Transport Layer (TPL)

	clientCBp->nfp_nlmClientInfo.pmap_pBsdSockUdp = pConn->pSock;
#endif  //NOTUSED

	*pAppHdl = clientCBp;
	clientCBp->nfp_ipAddr = raddr;

	/* 
	 * incr conn count in client 
	 */
	clientCBp->nfp_connCnt++;		
	return (NFX_OK);
}

/*-----------------------------------------------------------------
 * Name:nlm_pmapTplDelConnInd 		
 *
 * Description: 
 * 	Delete connection indication from the transport layer to delete
 *	an existing connection.
 *
 * Created by:  Joseph Vu
 *
 * Date Created:        7/30/02
 *
 *-----------------------------------------------------------------
 */

int32
nlm_pmapTplDelConnInd (void *hdlp, void *connHdlp)
{
	nfp_clientCBPtr_t	clientCBp;	

	NLM_DBG_FUNC_PRINTF("%s: connection %p closed\n",
		__FUNCTION__, connHdlp);

	clientCBp = (nfp_clientCBPtr_t)hdlp;
	if ( (clientCBp == NULL) || (connHdlp == NULL) )
		return (TPL_ERR_BAD_HDL);

	/*
	 * decr. conn count in client
	 */
	clientCBp->nfp_connCnt--;

	/* Shaun 09/27/02   
	 * Set delete pending flag if all connections
	 * assoc. with clients are deleted.
	 * 
	 * Actual removal will be done in
	 * nfs_reqTimeoutHandler() when all nfsReqCBs
	 * are freed.
	 * 
     */
	if (clientCBp->nfp_connCnt <= 0)
		clientCBp->nfp_flag |= NFP_ENTRY_DEL_PENDING;

	return (NFX_OK);

}


/*-----------------------------------------------------------------
 * Name:	nlm_pmapTplrcvInd	
 *
 * Description: 
 *	Receive packet indication from the transport 
 *	layer for processing client portmapper Getport
 *	reply.
 *
 * Created by:  Joseph Vu
 *
 * Date Create:	7/30/02
 *
 *-----------------------------------------------------------------
 */

int32
nlm_pmapTplrcvInd (void *hdlp, pkt_desc_t *descp)
{
	int32				rc;
	int32				rpcReplyHdrLen;
	int32				error;
	nfp_clientCBPtr_t	clientCBp;
	char				*portmapperMsgp;
	nlm_request_t		*nlmReqp;
    nlm_lockInfo_t      *lockInfop;
	rpcReplyHdr_t		*rpcReplyp;

	if ( ( hdlp == NULL) || (descp == NULL) ||
		(descp->pHdr == NULL ) )
		return (NULL);


	if (!(descp->flags & PKT_DESC_INUSE))
	{
		panic("%s: bad pkt=%x flags=%x\n",
			 __FUNCTION__,descp, descp->flags);
    }

	nlmReqp = NULL;
	error = NFX_OK;
	clientCBp = (nfp_clientCBPtr_t)hdlp;

	
	/*----------------------------------------
	 * Perform Net to Host conversion
	 *----------------------------------------
	 */	
	xdr_swap(descp->pHdr,descp->hdrSize, 0);


	if (nlm_traceEnable != 0)
		nlm_rxDecodePkt(descp);	

	/*----------------------------------------
	 *Find nlmReq correspond to the reply by 
	 *the Rpc Xid
	 *----------------------------------------
	 */	
	
	rpcReplyp = (rpcReplyHdr_t *)descp->pHdr;

	nlmReqp = nlm_findNlmReply(clientCBp, 
					rpcReplyp->rpc_xid);	
	if (nlmReqp == NULL)
	{
		nlm_globalCfgp->nlm_debugCounters.rxGetPortLocateReqErr++;
		goto nlm_pmapTplrcvIndExit;
	}
    lockInfop = nlmReqp->lockInfop;

	portmapperMsgp = descp->pHdr;
	rpcReplyHdrLen = rpc_processReplyHdr(descp->pHdr);
    // printf ("%s : Lock nlm_callerName : %d\n", __FUNCTION__, lockInfop->nlm_callerNameLen);
#ifdef	NLM_DEBUG
	NLM_DBG_FUNC_PRINTF("%s: rpcReplyHdrLen: %d nlmReqp: %p replyXid: %d\n",
			__FUNCTION__, rpcReplyHdrLen, nlmReqp, rpcReplyp->rpc_xid);
#endif

	if (rpcReplyHdrLen == NFX_ERR)
	{
		nlmReqp->nlm_txCmdInfo.getportRetry = 0;
		nlmReqp->nlm_txCmdInfo.getportMaxTimeWaitCnt = 0;
		nlmReqp->reqState = NLM_REQ_DONE;
		nlm_freeAll(nlmReqp, TRUE);
		goto nlm_pmapTplrcvIndExit;
	}

	/*----------------------------------------
	 *skip pass RPC header to 
	 *portmapper of the message
	 *----------------------------------------
	 */	
	
	portmapperMsgp += rpcReplyHdrLen;

	nlmReqp->reqState = NLM_GETPORT_REPLY_RCVD;
	nlm_globalCfgp->nlm_debugCounters.rxGetPortReplyCnt++;

	/*----------------------------------------
	 *Update client NLM port num
	 *----------------------------------------
	 */	
	clientCBp->nfp_nlmClientInfo.nlmPort = *(uint32 *)portmapperMsgp;

#ifdef	NLM_DEBUG
	NLM_DBG_FUNC_PRINTF("%s: nlmPort: %d\n",
				__FUNCTION__, clientCBp->nfp_nlmClientInfo.nlmPort);
#endif


	/*----------------------------------------
	 *Check for Asychnonous request/responses
	 *procecedure
	 *----------------------------------------
	 */	
	if ( (nlmReqp->procNum >=NLM_TEST_MSG) &&
		 (nlmReqp->procNum <= NLM_GRANTED_RES) )
	{

		nlm_delReqCBFromList(nlmReqp);

		if (nlmReqp->procNum == NLM_GRANTED_MSG)
		{
			nlm_procTxAsyncGrantedMsg(nlmReqp);
		}
		else
		{
			nlm_procTxSendASyncMsgResp(	nlmReqp,
										nlmReqp->replyStatus,
										nlmReqp->cookie.netobj_p,
										nlmReqp->cookie.netobjLen);
		}
		return TPL_FREE_PKT;
	}


	/*----------------------------------------
	 *Send NLM_GRANTED message to callback blocked
	 *lock request
	 *----------------------------------------
	 */	
	rc = nlm_procTxGranted(nlmReqp);
#ifdef	NOTUSED
	if (rc != NFX_OK)
	{
		nlmReqp->nlm_txCmdInfo.grantedRetry = 0;
		nlmReqp->nlm_txCmdInfo.grantedMaxTimeWaitCnt = 0;
		nlmReqp->reqState = NLM_REQ_DONE;
		nlm_delReqCBFromList(nlmReqp);
		nlm_freeAll(nlmReqp, TRUE);
	}
#endif	//NOTUSED

nlm_pmapTplrcvIndExit:
	return TPL_FREE_PKT;
}


/*-----------------------------------------------------------------
 * Name:	nlm_pmapTplXmtCmpInd 
 *
 * Description: 
 *	Transmit packet complete indication from the transport layer.
 *	 
 *
 * Created by:  Joseph Vu
 *
 * Date Created:        7/30/02
 *
 *-----------------------------------------------------------------
 */
int32
nlm_pmapTplXmtCmpInd ( void *appHdlp, pkt_desc_t *descp)
{

	//Increment Counters & Free TPL PKT
	//Free it back to TPL
    nlm_globalCfgp->nlm_debugCounters.nlm_tplXmtPktCmpInd++;
	nlm_globalCfgp->nlm_debugCounters.nlm_tplXmtPktFree++;

	return TPL_FREE_PKT;
}

#if defined(NFX_SMP)
/*-----------------------------------------------------------------
 * Name: nlm_portMapBindReqAck
 *
 * Description:	bind request ack
 *	
 *
 * Created by:  Su Wang
 *
 * Date Created:        12/04/02
 *
 *-----------------------------------------------------------------
 */
int32 nlm_portMapBindReqAck(void* pHandle, int32 rc, void *in_pBind)
{
	tpl_bindCb_t	*pBind = (tpl_bindCb_t*)in_pBind;

	if (rc != NFX_OK)
		panic("%s: tpl bind failed\n", __FUNCTION__);
	
	if (pBind->tplProt == TPL_PROT_UDP) {
		nlm_globalCfgp->pmap_pBsdSockUdp = pBind->pBsdSock;
		nlm_globalCfgp->pmap_pTplBindUdp = pBind;
	}
	else {
		nlm_globalCfgp->pmap_pBsdSockTcp = pBind->pBsdSock;
		nlm_globalCfgp->pmap_pTplBindTcp = pBind;
	}
	
	return NFX_OK;
}
#endif /* NFX_SMP */

/*-----------------------------------------------------------------
 * Name:nlm_portMapBindReq	 
 *
 * Description:
 * Binds NLM Portmapper Getport to Transport Layer (UDP).  
 *
 * Created by:  Joseph Vu
 *
 * Date Created:        5/28/01
 *
 *-----------------------------------------------------------------
 */
int32
nlm_portMapBindReq (uint32 ipAddr)
{
	int32			rc;
	int32			bindOpt;
	ushort16		protocol;
	tpl_bindCb_t	bindReq;
//	tpl_bindCb_t	*bindCBp;
#if !defined(NFX_SMP)
	tpl_bindCb_t	*tplRspBindUdp;
	tpl_bindCb_t	*tplRspBindTcp;
#endif /* NFX_SMP */

#ifdef	NLM_DEBUG
	NLM_DBG_FUNC_PRINTF("%s: enter ipAddr: %08x\n", 
				__FUNCTION__, ipAddr);
#endif

	protocol = TPL_PROT_UDP;
	while (1)
	{
		memset( &bindReq, 0, sizeof(tpl_bindCb_t));

		bindOpt = TPL_BIND_USE_RPC | TPL_BIND_SERVER;
		bindReq.laddr   	  = ipAddr;
		bindReq.lport   	  = ntohs(NLM_PORT + 1);	
		bindReq.tplProt 	  = protocol;
		bindReq.ul_addConnInd = nlm_pmapTplAddConnInd;
		bindReq.ul_delConnInd = nlm_pmapTplDelConnInd;
		bindReq.ul_rcvPktInd  = nlm_pmapTplrcvInd;
		bindReq.ul_xmtCmpInd  = nlm_pmapTplXmtCmpInd;

#if defined(NFX_SMP)
		(&bindReq)->ul_bindReqAck = nlm_portMapBindReqAck;
		rc = tpl_bindReq(&bindReq, NULL, bindOpt);

		if (rc != NFX_OK)
				panic ("%s: bind failured\n", __FUNCTION__);
		
		if (protocol == TPL_PROT_TCP)
			break;
#else
		if (protocol == TPL_PROT_UDP)
		{
			rc = tpl_bindReq( &bindReq, &tplRspBindUdp, bindOpt);
			if (rc == NFX_ERR)
				panic ("nlm_portMapBindReq: bind failure (UDP)!\n");

			nlm_globalCfgp->pmap_pTplBindUdp = tplRspBindUdp;
			nlm_globalCfgp->pmap_pBsdSockUdp = tplRspBindUdp->pBsdSock;

		}
		else
		{
			rc = tpl_bindReq( &bindReq, &tplRspBindTcp, bindOpt);
			if (rc == NFX_ERR)
				panic ("nlm_portMapBindReq: bind failure(TCP)!\n");
			nlm_globalCfgp->pmap_pTplBindTcp = tplRspBindTcp;
			nlm_globalCfgp->pmap_pBsdSockTcp = tplRspBindTcp->pBsdSock;

			break;
		}
#endif /* NFX_SMP */

		protocol = TPL_PROT_TCP;
	}
	return (NFX_OK);

}
