/* vi:ts=4
 *-----------------------------------------------------------------
 *
 * Name:        nlm-procRx.c
 *
 * RCS:  $Id: nlm-procRx.c,v 1.8 2002/11/06 03:35:39 joseph Exp $
 *
 * Copyright (C) 2001, Agile Storage Inc.
 *
 * Description: 
 *	Contains received NLM Procedures.
 * Created by: Joseph Vu 
 *
 * Date Created:        7/28/02
 *
 *-----------------------------------------------------------------
 */
#ifdef NFP_TXRX

#include <nfx-incl.h>
#include <exports/vs-mem-api.h>
#include <exports/lock-api.h>
#include <exports/nfs-api.h>
#include "nlm-prototypes.h"

#include <sys/machine-dep.h>

#define	NLM_DBG_PRINTF	if (nlm_traceEnable) printf

//#define	NLM_USE_SHMEM

//Externs
#ifdef	NOTUSED
extern pkt_desc_t *nlm_tplAllocPkt(
	nfp_clientCBPtr_t	clientCBp, 
	nlm_request_t 		*nlmReqp,	
	int32				usrDataLen);
#endif	//NOTUSED

extern int32
nfs_addFileCBToList(nfp_fileCBPtr_t fileCBp, fs_Handle *fhandlep,
            nfp_clientCBPtr_t clientCBp);

extern int32 nfs_delCBFromList(
	nfp_list_t *nodeListp,
	void *CtrlBlkp,
	int32 nodeType
);

extern
int32 nfs_delNode(nfp_list_t *nodeListp, nfp_nodePtr_t delNodep);

//#define		NLM_DEBUG
#define NLM_DBG_FUNC_PRINTF if (nlm_traceDebugEnable) printf


/*-----------------------------------------------------------------
 * Name:	nfs_procRxNullReq 
 *
 * Description:
 * Process received NLM_NULL request from client.
 *
 * Created by:  Joseph Vu
 *
 * Date Created:        6/28/02
 *
 *---------------------------------------------------------------
 */
int32
nlm_procRxNullReq(nlm_request_t    *nlmReqp)
{
	int32				error;
	uint32				txMsgLen;
	char				*txMsgp;
	pkt_desc_t 			*tplXmitPktDescp;


	/*-----------------------------
	 *Set default error
	 *-----------------------------
	 */
	error = NLM_FAILED;

	/*-----------------------------
	 *Alloc TPL packet to send
	 *-----------------------------
	 */

	tplXmitPktDescp = TPL_ALLOC_PACKET(nlmReqp->tplConnp, 0);
	if (tplXmitPktDescp == NULL)
	{
		nlm_globalCfgp->nlm_debugCounters.nlm_tplXmtNoPktResource++;
		return (NFX_ERR);	
	}
	nlm_globalCfgp->nlm_debugCounters.nlm_tplXmtPktAlloc++;

	txMsgp = (char *)tplXmitPktDescp->pHdr;
	txMsgLen = 0;

	/*-----------------------------
	 *Start with RPC reply header
	 *-----------------------------
	 */

	txMsgLen += nfs_msgRpcReply(txMsgp, 
								nlmReqp->rpcCallHdrp->rpc_xid,
								RPC_REPLY_MSG_ACCEPTED, 
								RPC_REPLY_SUCCESS);
	txMsgp += txMsgLen;

	/*-----------------------------
	 *Set packet desc hdr size.
	 *Send packet
	 *-----------------------------
	 */
		
	tplXmitPktDescp->hdrSize = txMsgLen;

	if (nlm_procTxPkt(tplXmitPktDescp, nlmReqp) != NFX_OK)
	{
		nlmReqp->txPktDescp = (void *)tplXmitPktDescp;
		nlmReqp->reqState = NLM_REQ_DONE;
		return (NFX_ERR);
	}

	return (NFX_OK);
}

/*-----------------------------------------------------------------
 * Name:nlm_procRxTestReq		
 *
 * Description:
 * Process received NLM_TEST request from client.
 * This procedure checks whether a lock is available
 * to the client.
 *
 * Created by:  Joseph Vu
 *
 * Date Created:        7/18/02
 *
 *--------------------------------------------------------------
 */

int32
nlm_procRxTestReq(nlm_request_t    *nlmReqp)
{
	int32				rc;			
	int32				error;
	int32				length;
	ushort16			cookieLen;
	uint32				exclusiveFlag;
	char				*nlm_msgp;
	nfp_fileCBPtr_t		fileCBp;	
	char				cookie[NLM_MAX_NAME];
	static char			callerName[NLM_MAX_NAME];
	static char			oh[NLM_MAX_NAME];		
	static char 		buf[sizeof(nlm_lockInfo_t)];
#ifdef NOT_USED
	uint32				listType;
	int32				searchMask;
	nlm_lockRec_t 		*lockRecp;
	nlm_lockRec_t		*nextLockRecp;
#else
	nlm_lockInfo_t		*nlm_lockInfop;
    int32               newFile = 0;
#endif
	fs_Handle       	fileHandle;	
	
	error = NLM_FAILED;
#ifdef NOT_USED
	nlmReqp->lockRecp = NULL;
#endif

	/*-----------------------------
	 *Get start of NLM_TEST procedure 
	 *-----------------------------
	 */
	
	nlm_msgp = nlmReqp->procStart;

	/*-----------------------------
	 * Get client's cookie 
	 *-----------------------------
	 */

	length = nfs_extractString(	nlm_msgp, 
								&cookie[0],
								&cookieLen, 
								NLM_MAX_NAME);		
	if (length == NFX_ERR)
	{
		nlm_globalCfgp->nlm_debugCounters.numOfRxBadCookies++;
		return (NFX_ERR);
	}
	cookie[cookieLen]  = '\0';	


	/*-----------------------------
	 * update msg field
	 *-----------------------------
	 */
	
	nlm_msgp += length;

	/*-----------------------------
	 * Get exclusive flag
	 * and alock info
	 *-----------------------------
	 */
	
	exclusiveFlag = 0;
	length = nfs_extractLongVal(nlm_msgp, &exclusiveFlag);	
	nlm_msgp += length;

	nlm_lockInfop = (nlm_lockInfo_t *)buf;
	nlm_lockInfop->nlm_callerName = &callerName[0];
	nlm_lockInfop->nlm_oh = &oh[0];
	length = nlm_extractAlockInfo(	nlm_msgp, 		
									nlm_lockInfop,
									nlmReqp->rpcCallHdrp->progVer,
									&fileHandle); 
	if (length == NFX_ERR)
	{
		nlm_globalCfgp->nlm_debugCounters.numOfRxBadAlockObj++;
		rc = nlm_procTxSendResp(nlmReqp, error, cookie, cookieLen);
		return (rc);
	}

	/*-----------------------------
	 *Set exclusive flag
	 *-----------------------------
	 */
#ifdef NOT_USED
	nlm_lockInfop->lockRec.nlm_exclusive =
		(exclusiveFlag != 0)? TRUE : FALSE;
#else
	nlm_lockInfop->nlm_exclusive = (exclusiveFlag != 0)? TRUE : FALSE;
#endif

#ifdef	NLM_DEBUG
#ifdef NOT_USED
	printf("%s: svid: %d l_offset: %lld l_len: %lld exl: %d state: %d\n",
		__FUNCTION__, nlm_lockInfop->nlm_svid, 
		nlm_lockInfop->lockRec.nlm_l_offset,
		nlm_lockInfop->lockRec.nlm_l_len,
		nlm_lockInfop->lockRec.nlm_exclusive, 
		nlm_lockInfop->lockRec.nlm_state);	
#else
	printf("%s: svid: %d l_offset: %lld l_len: %lld exl: %d state: %d\n",
		__FUNCTION__, nlm_lockInfop->nlm_svid, 
		nlm_lockInfop->nlm_l_offset,
		nlm_lockInfop->nlm_l_len,
		nlm_lockInfop->nlm_exclusive, 
		nlm_lockInfop->nlm_state);	
#endif
	printf("%s: callerNameLen: %d CallerName: %s\n",
		__FUNCTION__,nlm_lockInfop->nlm_callerNameLen, 
		nlm_lockInfop->nlm_callerName);	
	printf("%s: ohLen: %d oh: %s\n",
		__FUNCTION__,nlm_lockInfop->nlm_ohLen, 
		nlm_lockInfop->nlm_oh);	
#endif	

#ifdef NOT_USED
	E_LOG (class_1, LOG_DEBUG, 
			nlm_globalCfgp->eee_nlmAppId, 0, 0, 0,
		("%s: svid:%d offset:%lld len:%lld exl:%d state:%d\n",
		__FUNCTION__,
		nlm_lockInfop->nlm_svid, 
		nlm_lockInfop->lockRec.nlm_l_offset, 
		nlm_lockInfop->lockRec.nlm_l_len,
		nlm_lockInfop->lockRec.nlm_exclusive, 
		nlm_lockInfop->lockRec.nlm_state));
#else
	E_LOG (class_1, LOG_DEBUG, 
			nlm_globalCfgp->eee_nlmAppId, 0, 0, 0,
		("%s: svid:%d offset:%lld len:%lld exl:%d state:%d\n",
		__FUNCTION__,
		nlm_lockInfop->nlm_svid, 
		nlm_lockInfop->nlm_l_offset, 
		nlm_lockInfop->nlm_l_len,
		nlm_lockInfop->nlm_exclusive, 
		nlm_lockInfop->nlm_state));
#endif

	/*-------------------------------------------
	 *Check if there is any locks on this file
	 *-------------------------------------------
	 */
	
	fileCBp = nfp_getFileCB (&fileHandle);
	if (fileCBp == NULL)
    {
		/*-------------------------------------------------
		 *No lock on this file, return LCK_GRANTED status 
		 *to client
		 *------------------------------------------------
		 */
#ifdef NOT_USED
		goto nlm_procRxTestReqOK;
#else
        fileCBp = nfp_allocFileCB();
        if (fileCBp == NULL)
            goto nlm_procRxTestReqOK;
        newFile = 1;
        nfp_addFileCB (fileCBp, &fileHandle);
#endif
    }
	nlmReqp->fileCBp = fileCBp;

#ifdef NOT_USED
	/*-----------------------------
	 *Get end of lock region	
	 *-----------------------------
	 */
	
	NLM_GET_L_REGION_END(nlm_lockInfop->lockRec.nlm_l_end, 
						nlm_lockInfop->lockRec.nlm_l_offset, 
						nlm_lockInfop->lockRec.nlm_l_len,
						nlm_lockInfop->lockRec.nlm_l_eof);			
	

	/*-----------------------------
	 *There're locks on this file
	 *Check if request lock are conflict 
	 *with existing locks or pending
	 *locks
	 *-----------------------------
	 */
		
	searchMask = 0;
	if (!exclusiveFlag)
		searchMask = NLM_SEARCH_EXCLUSIVE;
	lockRecp = 	nlm_getConflictinglock(	fileCBp,
										nlm_lockInfop, 
										searchMask, 
										&nextLockRecp,
										&listType);

    error = NLM_STAT_GRANTED;
	if (lockRecp != NULL)
	{
		/*-----------------------------
	 	 *conflicting lock, return DENIED 
	 	 *-----------------------------
	 	 */
		nlmReqp->lockRecp = lockRecp;
#ifdef	NLM_DEBUG
		printf("%s: DENIED: lockRecp:%p oh:%s off:%lld len:%lld\n", 
		__FUNCTION__, lockRecp,
		((nfp_lockCBPtr_t)lockRecp->lockCBp)->protocol.nfp_nlmLock.nlm_oh,
		lockRecp->nlm_l_offset,
		lockRecp->nlm_l_len);
#endif
		rc = nlm_procTxSendResp(nlmReqp, 
								NLM_ERR_DENIED,
								cookie, 
								cookieLen);
        if (rc == NFX_OK)
        {
            /* We should be freeing up the memory */
            // nlmReqp->rxPktDescp = NULL;
            // nlm_freeAll(nlmReqp, TRUE);
        }
		return (rc);

	}
#else  // NOT_USED
 	/*
	 *Get end of lock region	
	 */
	
	NLM_GET_L_REGION_END(nlm_lockInfop->nlm_l_end, 
						 nlm_lockInfop->nlm_l_offset, 
						 nlm_lockInfop->nlm_l_len,
						 nlm_lockInfop->nlm_l_eof);			
   error = nlm_procTestReq(fileCBp, nlm_lockInfop, nlmReqp);
#endif // NOT_USED

	/*-------------------------------------------
 	 *No conflicting lock, return GRANTED status 
 	 *--------------------------------------------
 	 */
nlm_procRxTestReqOK:
	rc = nlm_procTxSendResp(nlmReqp, 
							error,
							cookie, 
							cookieLen);
    if (newFile)
    {
        nfp_deallocFileCB(fileCBp);
        nfp_delFileCB(&fileHandle);
    }
	return (rc);
}


/*---------------------------------------------------------
 * Name:	nlm_procRxLockReq 	
 *
 * Description:
 * Process received NLM_LOCK request from client.
 * This procedure creates a locked byte range on a file. 
 *
 * Created by:  Joseph Vu
 *
 * Date Created:        7/18/02
 *
 *---------------------------------------------------------
 */

int32
nlm_procRxLockReq (nlm_request_t    *nlmReqp)
{
	nfp_fileCBPtr_t		fileCBp;	
	nlm_lockInfo_t		*nlm_lockInfop;
	nfp_clientCBPtr_t	clientCBp;
	int32				rc;
	uint32				block;
	uint32				exclusive;
	uint32				reclaim;
	int32				error;
	int32				length;
	ushort16			cookieLen;
	char				*nlm_msgp;
	char				cookie[NLM_MAX_NAME];
	static char			callerName[NLM_MAX_NAME];
	static char			oh[NLM_MAX_NAME];
	static char    		buf[sizeof(nlm_lockInfo_t)];
	fs_Handle       	fileHandle;	
#ifdef NOT_USED
	nlm_lockRec_t 		*lockRecp;
#else
    int32               newFile = 0, ret;
#endif

	error = NLM_FAILED;
	rc = NFX_OK;

	clientCBp = nlmReqp->clientCBp;

	/*-----------------------------
 	 * Get start of message
 	 *-----------------------------
 	 */
	nlm_msgp = nlmReqp->procStart;

	/*-----------------------------
 	 * Get client's cookie 
 	 *-----------------------------
 	 */

	length = nfs_extractString(	nlm_msgp, 
								&cookie[0],
								&cookieLen, 
								NLM_MAX_NAME);		
	if (length == NFX_ERR)
	{
		nlm_globalCfgp->nlm_debugCounters.numOfRxBadCookies++;
		return (NFX_ERR);
	}
	cookie[cookieLen]  = '\0';	
	nlm_msgp += length;

	/*-----------------------------
 	 *Get block field, exclusive field
	 *and alock info 
 	 *-----------------------------
 	 */
	length = nfs_extractLongVal(nlm_msgp, &block);
	nlm_msgp += length;

	length = nfs_extractLongVal(nlm_msgp, &exclusive);
	nlm_msgp += length;

	nlm_lockInfop = (nlm_lockInfo_t *)buf;
    nlm_lockInfop->nlm_callerName = &callerName[0];
    nlm_lockInfop->nlm_oh = &oh[0];
	length = nlm_extractAlockInfo(	nlm_msgp, 
									nlm_lockInfop,
									nlmReqp->rpcCallHdrp->progVer,
									&fileHandle);
	if (length == NFX_ERR)
	{
		nlm_globalCfgp->nlm_debugCounters.numOfRxBadAlockObj++;
		goto nlm_procRxLockReqErr;
	}
	nlm_msgp += length;

	/*-----------------------------
 	 *Get reclaim field 
 	 *-----------------------------
 	 */
	length = nfs_extractLongVal(nlm_msgp, &reclaim);
	nlm_msgp += length;

	/*----------------------------------------------
	 *Check if state is in GRACE PERIOD.
	 *If so, only allow clients with reclaim locks
 	 * 
 	 *-----------------------------------------------
 	 */

	if (nlm_globalCfgp->gracePeriod && !reclaim)
	{
		/* Don't accept new lock requests during grace period */
		error = NLM_ERR_DENIED_GRACE_PERIOD;
		nlm_globalCfgp->nlm_debugCounters.numOfRxGracePeriodErr++;
		goto nlm_procRxLockReqErr;
	}

	/*-----------------------------
 	 *Get current state of NSM,
	 *save block field,
	 *and set exlusive flag
 	 *-----------------------------
 	 */
#ifdef NOT_USED
	length = nfs_extractLongVal(nlm_msgp, 
								&nlm_lockInfop->lockRec.nlm_state);
#else
	length = nfs_extractLongVal(nlm_msgp, &nlm_lockInfop->nlm_state);
#endif
	nlm_msgp += length;

	nlmReqp->block = block;

#ifdef NOT_USED
	NLM_GET_L_REGION_END(nlm_lockInfop->lockReq.nlm_l_end, 
						 nlm_lockInfop->lockReq.nlm_l_offset, 
						 nlm_lockInfop->lockReq.nlm_l_len,
						 nlm_lockInfop->lockReq.nlm_l_eof);	
	nlm_lockInfop->lockRec.nlm_exclusive = 
		(exclusive != 0)? TRUE : FALSE;
#else
	NLM_GET_L_REGION_END(nlm_lockInfop->nlm_l_end, 
						 nlm_lockInfop->nlm_l_offset, 
						 nlm_lockInfop->nlm_l_len,
						 nlm_lockInfop->nlm_l_eof);	
	nlm_lockInfop->nlm_exclusive = (exclusive != 0)? TRUE : FALSE;
#endif // NOT_USED

#ifdef	NLM_DEBUG
#ifdef NOT_USED
	printf("%s: svid: %d l_offset: %lld l_len: %lld exl: %d state: %d\n",
		__FUNCTION__, nlm_lockInfop->nlm_svid, 
		nlm_lockInfop->lockRec.nlm_l_offset, 
		nlm_lockInfop->lockRec.nlm_l_len,
		nlm_lockInfop->lockRec.nlm_exclusive, 
		nlm_lockInfop->lockRec.nlm_state);	
#else // NOT_USED
	printf("%s: svid: %d l_offset: %lld l_len: %lld exl: %d state: %d\n",
		__FUNCTION__, nlm_lockInfop->nlm_svid, 
		nlm_lockInfop->nlm_l_offset, 
		nlm_lockInfop->nlm_l_len,
		nlm_lockInfop->nlm_exclusive, 
		nlm_lockInfop->nlm_state);	
#endif // NOT_USED
	printf("%s: callerNameLen: %d CallerName: %s\n",
		__FUNCTION__,nlm_lockInfop->nlm_callerNameLen, 
		nlm_lockInfop->nlm_callerName);	
	printf("%s: ohLen: %d oh: %s\n",
		__FUNCTION__,nlm_lockInfop->nlm_ohLen, 
		nlm_lockInfop->nlm_oh);	
#endif	

	/*---------------------------------------------
	 *Check if there is any locks on this file
 	 *---------------------------------------------
 	 */
	
	fileCBp = nfp_getFileCB (&fileHandle);
#ifdef NOT_USED

	if (fileCBp == NULL)
	{
		/*-----------------------------
		 *No lock on this file
		 *Alloc FileCB and add lock Rec 
 	 	 *-----------------------------
 	 	 */

		fileCBp = nfp_allocFileCB();
		if (fileCBp == NULL)
			goto nlm_procRxLockReqErr;
		lockRecp = nlm_addLockRecToCurrList(clientCBp,
											fileCBp, 
											nlm_lockInfop,
											NULL,
											1,
											&fileHandle);
		if (lockRecp == NULL)
		{
			error = NLM_ERR_DENIED_NOLOCKS;
			nfp_delFileCB (&fileHandle);
			goto nlm_procRxLockReqErr;
		}

		rc = nlm_procTxSendResp(nlmReqp, 
								NLM_STAT_GRANTED,
								cookie, 
								cookieLen);
		if (rc != NFX_OK)
		{
			nfp_lockCB_t 		*lockCBp;

			lockCBp = (nfp_lockCB_t	*)lockRecp->lockCBp;
			DLL_REMOVE(&lockCBp->lockOwnerList.list,
						lockRecp, lockList);
			nlm_deallocLockRec(lockRecp);
			lockCBp->lockOwnerList.numOfEntries--;
			nfp_deallocLockCB(lockCBp);
//			nfs_delCBFromList(&clientCBp->nfp_fileList,
//				(void *)fileCBp, NFP_FILECB_NODE);
			nfs_delCBFromList(&fileCBp->nfp_lockList, 
				(void *)lockCBp, NFP_LOCKCB_NODE);
			nfp_delFileCB (&fileHandle);
			return (NFX_ERR);
		}


		/*---------------------------------------------
		 *register client with NSM (Network Status Mon)
 	 	 *---------------------------------------------
 	 	 */
#ifdef	NLM_DEBUG
		NLM_DBG_FUNC_PRINTF("%s: Register client %s addr: %08x volId: %llx clientCBp: %p\n",
				__FUNCTION__,
				nlm_lockInfop->nlm_callerName,
				clientCBp->nfp_ipAddr,
				fileHandle.volId.whole_thing,
				clientCBp);
#endif

		nsm_procMonReq(	nlm_lockInfop->nlm_callerName,
						clientCBp->nfp_ipAddr,
						fileHandle.volId,
						clientCBp);

		return (NFX_OK);
	}

#ifdef	NLM_DEBUG
	NLM_DBG_FUNC_PRINTF("%s: Existing fileCBp: %p volId:%llx inum: %llx mntPnt:%llx \n",
		   	__FUNCTION__, fileCBp,
			fileCBp->nfp_fileHandle.volId.whole_thing,
			fileCBp->nfp_fileHandle.inum,
			fileCBp->nfp_fileHandle.mntPnt);
#endif

	/*-------------------------------------
	 *There're active locks on this file.
	 *Checks for conflict or overlap region
  	 *--------------------------------------
  	 */
	nlmReqp->fileCBp = fileCBp;

	error = nlm_checkforConflictLock(nlmReqp,
									fileCBp, 
									nlm_lockInfop, 
									0,
									cookie, 
									cookieLen);

	if (error != NFX_OK)
	{
		goto nlm_procRxLockReqErr;
	}
	return (NFX_OK);
#else // NOT_USED
	if (fileCBp == NULL)
	{
        /*
         * No locks on this file. Allocate FileCB add lock
         */
        fileCBp = nfp_allocFileCB();
        if (fileCBp == NULL)
            goto nlm_procRxLockReqErr;
        nfp_addFileCB(fileCBp, &fileHandle);
        newFile = 1;
    }
    nlmReqp->fileCBp = fileCBp;
    nlmReqp->lockInfop = nlm_lockInfop;
    ret = nlm_procLockReq(nlm_lockInfop, nlmReqp);
    if (ret == LOCK_STATE_GRANTED)
    {
        /* The lock has been granted */
        error = NLM_STAT_GRANTED;
    }
    else if (ret == LOCK_STATE_DENIED)
    {
        /* The lock has been denied. This could happen for any # of reasons */
        error = (block)?NLM_ERR_DEADLCK:NLM_ERR_DENIED;
        // goto nlm_procRxLockReqErr;
    }
    else
    {
        /*
         * The lock has been put in the pending Q. We should get a call back.
         */
        error = NLM_ERR_BLOCKED;
    }
    rc = nlm_procTxSendResp(nlmReqp, error, cookie, cookieLen);
    if (rc != NFX_OK)
    {
        if (ret == LOCK_STATE_GRANTED)
            nlm_procUnlockReq(nlm_lockInfop, nlmReqp);
        else if (ret == LOCK_STATE_BLOCKED)
            nlm_procCancellockReq(nlm_lockInfop, nlmReqp);
        if (lock_areThereCurrentFileLocksReq(fileCBp->fileLockIdx) == FALSE)
        {
            nfp_deallocFileCB(fileCBp);
            nfp_delFileCB(&fileHandle);
        }

        return (NFX_ERR);
    }
    if (newFile && ret != LOCK_STATE_DENIED)
    {
        /*
         * Register client with NSM (Network Status Mon)
         */
#ifdef	NLM_DEBUG
		NLM_DBG_FUNC_PRINTF("%s: Register client %s addr: %08x volId: %llx clientCBp: %p\n",
				__FUNCTION__,
				nlm_lockInfop->nlm_callerName,
				clientCBp->nfp_ipAddr,
				fileHandle.volId.whole_thing,
				clientCBp);
#endif
		nsm_procMonReq(	nlm_lockInfop->nlm_callerName,
						clientCBp->nfp_ipAddr,
						fileHandle.volId,
						clientCBp);
	}

#ifdef	NLM_DEBUG
	NLM_DBG_FUNC_PRINTF("%s: FileCBp: %p volId:%llx inum: %llx mntPnt:%llx \n",
		   	__FUNCTION__, fileCBp,
			fileCBp->nfp_fileHandle.volId.whole_thing,
			fileCBp->nfp_fileHandle.inum,
			fileCBp->nfp_fileHandle.mntPnt);
#endif
    return (NFX_OK);
#endif // NOT_USED

nlm_procRxLockReqErr:
	if (error == NFX_ERR)
		return (NFX_ERR);
	
	/*-----------------------------------
	 *Send error response back to client
  	 *-----------------------------------
  	 */
	rc = nlm_procTxSendResp(nlmReqp, 
							error,
							cookie, 
							cookieLen);
	return (rc);
}

/*-----------------------------------------------------------------
 * Name: nlm_procRxCancelReq
 *
 * Description:
 * Process received NLM_CANCEL request from client.
 * This procedure cancels a blocked LOCK request. 
 *
 * Created by:  Joseph Vu
 *
 * Date Created:        7/18/02
 *
 *-----------------------------------------------------------------
 */

int32
nlm_procRxCancelReq (nlm_request_t    *nlmReqp)
{
	fs_Handle       	fileHandle;	
	int32				rc;			
	int32				error;
	int32				length;
	uint32				block;
	ushort16			cookieLen;
	uint32				exclusiveFlag;
	char				*nlm_msgp;
	nfp_fileCBPtr_t		fileCBp;
	char				cookie[NLM_MAX_NAME];	
	static char			callerName[NLM_MAX_NAME];	
	static char			oh[NLM_MAX_NAME];	
	static char			buf[sizeof(nlm_lockInfo_t)];
	nlm_lockInfo_t		*nlm_lockInfop;
#ifdef NOT_USED
	short16				searchMask;
	nlm_lockRec_t		*next_lockRecp;
	nfp_lockCBPtr_t		lockCBp;
	nlm_lockRec_t		*lockRecp;
#else
    int32               ret;
#if 0
    nlm_pendingLockNode_t *pendNode;
#endif
#endif


	/*-------------------------------------
	 *Set default error.
	 *Get start of NLM_CANCEL procedure
  	 *--------------------------------------
  	 */
	error = NLM_STAT_GRANTED;
	nlm_msgp = nlmReqp->procStart;

	/*-------------------------------------
	 *Get client's cookie, block field,
	 *exlusive flag and alock info. 
  	 *--------------------------------------
  	 */
	length = nfs_extractString(	nlm_msgp, 
								&cookie[0],
								&cookieLen, 
								NLM_MAX_NAME);		
	if (length == NFX_ERR)
	{
		nlm_globalCfgp->nlm_debugCounters.numOfRxBadCookies++;
		return (NFX_ERR);
	}
	cookie[cookieLen]  = '\0';
	nlm_msgp += length;

	/*Get block field*/
	length = nfs_extractLongVal(nlm_msgp, &block);
	nlm_msgp += length;

	/*Get exclusive flag*/
	exclusiveFlag = 0;
	length = nfs_extractLongVal(nlm_msgp, &exclusiveFlag);	
	nlm_msgp += length;

	/*Get alock info*/
	nlm_lockInfop = (nlm_lockInfo_t *)buf;
    nlm_lockInfop->nlm_callerName = &callerName[0];
    nlm_lockInfop->nlm_oh = &oh[0];

	length = nlm_extractAlockInfo(	nlm_msgp, 
									nlm_lockInfop,
									nlmReqp->rpcCallHdrp->progVer,
									&fileHandle); 
	if (length != NFX_ERR)
	{
		nlm_globalCfgp->nlm_debugCounters.numOfRxBadAlockObj++;
		goto nlm_procRxCancelReqErr;	
	}
#ifdef NOT_USED
	NLM_GET_L_REGION_END(nlm_lockInfop->lockRec.nlm_l_end, 
						 nlm_lockInfop->lockRec.nlm_l_offset, 
						 nlm_lockInfop->lockRec.nlm_l_len,
						 nlm_lockInfop->lockRec.nlm_l_eof);	
    nlm_lockInfop->lockRec.nlm_exclusive =
        (exclusiveFlag != 0)? TRUE : FALSE;
#else // NOT_USED
	NLM_GET_L_REGION_END(nlm_lockInfop->nlm_l_end, 
						 nlm_lockInfop->nlm_l_offset, 
						 nlm_lockInfop->nlm_l_len,
						 nlm_lockInfop->nlm_l_eof);	
    nlm_lockInfop->nlm_exclusive = (exclusiveFlag != 0)? TRUE : FALSE;
#endif // NOT_USED

#ifdef	NLM_DEBUG
#ifdef NOT_USED
	NLM_DBG_FUNC_PRINTF("%s: svid: %d l_offset: %lld l_len: %lld exl: %d state: %d\n",
		__FUNCTION__, 
		nlm_lockInfop->nlm_svid, 
		nlm_lockInfop->lockRec.nlm_l_offset, 
		nlm_lockInfop->lockRec.nlm_l_len,
		nlm_lockInfop->lockRec.nlm_exclusive, 
		nlm_lockInfop->lockRec.nlm_state);	
#else // NOT_USED
	NLM_DBG_FUNC_PRINTF("%s: svid: %d l_offset: %lld l_len: %lld exl: %d state: %d\n",
		__FUNCTION__, 
		nlm_lockInfop->nlm_svid, 
		nlm_lockInfop->nlm_l_offset, 
		nlm_lockInfop->nlm_l_len,
		nlm_lockInfop->nlm_exclusive, 
		nlm_lockInfop->nlm_state);	
#endif // NOT_USED
	NLM_DBG_FUNC_PRINTF("%s: callerNameLen: %d CallerName: %s\n",
		__FUNCTION__,nlm_lockInfop->nlm_callerNameLen, 
		nlm_lockInfop->nlm_callerName);	
	NLM_DBG_FUNC_PRINTF("%s: ohLen: %d oh: %s\n",
		__FUNCTION__,nlm_lockInfop->nlm_ohLen, 
		nlm_lockInfop->nlm_oh);	
#endif

	/*--------------------------------------------
	 *Check if there is a matching pending request 
  	 *--------------------------------------------
  	 */
	
	fileCBp = nfp_getFileCB (&fileHandle);
	if (fileCBp == NULL)
	{
		error = NLM_ERR_DENIED;
		goto nlm_procRxCancelReqErr;	
	}
    nlmReqp->fileCBp = fileCBp;
    nlmReqp->lockInfop = nlm_lockInfop;
#ifdef NOT_USED
	searchMask = (NLM_SEARCH_OH | NLM_SEARCH_SVID | 
				NLM_SEARCH_REGION_MATCH | NLM_SEARCH_EXCLUSIVE);
	lockRecp = nlm_getLockRecInPendingList(	fileCBp,
											nlm_lockInfop,
											searchMask,
											&next_lockRecp);	

	if (lockRecp == NULL)
	{
		error = NLM_ERR_DENIED;
		goto nlm_procRxCancelReqErr;
	}

	/*------------------------------------------
	 *Remove pending lock record from pending list
	 *------------------------------------------
	 */
	lockCBp = (nfp_lockCBPtr_t )lockRecp->lockCBp;

	if (nfp_validateLockCB(lockCBp) != NFX_OK)
	{
		panic("%s: Bad lockCBp: %p\n", 
						__FUNCTION__, lockCBp);		
	}


	DLL_REMOVE(&lockCBp->lockOwnerPendingList.list,		
				lockRecp, lockList);
	nlm_deallocLockRec(lockRecp);
	lockCBp->lockOwnerPendingList.numOfEntries--;

	rc = nlm_procTxSendResp(nlmReqp, 
							NLM_STAT_GRANTED,
							cookie, 
							cookieLen);

	return (rc);
#else
    ret = nlm_procCancellockReq(nlm_lockInfop, nlmReqp);
    if (ret == NFX_ERR)
        error = NLM_ERR_DENIED;
    else
        error = NLM_STAT_GRANTED; 
    if (lock_areThereCurrentFileLocksReq(fileCBp->fileLockIdx) == FALSE)
    {
        nfp_deallocFileCB(fileCBp);
        nfp_delFileCB(&fileHandle);
    }
#endif // NOT_USED


nlm_procRxCancelReqErr:
	/*--------------------------------------------
	 *Return LCK_DENIED
	 *Error: no matching lock to cancel
	 *Send error response back to client
  	 *--------------------------------------------
  	 */
#ifdef NOT_USED
	nlmReqp->lockRecp = NULL;
#endif
	nlm_globalCfgp->nlm_debugCounters.nlm_rcvCancelErr++;
	
	rc = nlm_procTxSendResp(nlmReqp, 
							error,
							cookie, 
							cookieLen);
	return (rc);

}


/*-----------------------------------------------------------------
 * Name:	nlm_procRxUnlockReq 	 
 *
 * Description:
 * Process received NLM_UNLOCK request from client.
 * This procedure unlocks a locked region of a file. 
 *
 *
 * Created by:  Joseph Vu
 *
 * Date Created:        7/18/02
 *
 *-----------------------------------------------------------------
 */
int32
nlm_procRxUnlockReq (nlm_request_t    *nlmReqp)
{
	fs_Handle       	fileHandle;	
	uint32				sendResp;
	int32				rc;
	int32				error;
	int32				length;
	ushort16			cookieLen;
	char				*nlm_msgp;
	nfp_fileCBPtr_t		fileCBp;	
	char				cookie[NLM_MAX_NAME];
	static char			callerName[NLM_MAX_NAME];	
	static char			oh[NLM_MAX_NAME];	
	static char			buf[sizeof(nlm_lockInfo_t)];
	nlm_lockInfo_t		*nlm_lockInfop;
	nfp_lockCB_t		*unlockCBp;
#ifdef NOT_USED
	int32				overlap;
	int32				unlockType;
	short16				searchMask;
	nfp_lockCB_t		*pendingLockCBp;
	nlm_lockInfo_t		*pendingLockInfop;
	nlm_lockRec_t		*unlockRecp, *conflictLockRecp;
	nlm_lockRec_t		*pendingLockRecp, *nextLockRecp;
	nlm_lockRec_t		*unlockRecNextp;
	nlm_lockInfo_t 		*unlockAlockp;
	nfp_requestCBPtr_t	new_reqCBp;
	nlm_request_t		*new_nlmReqp;
#else
    uint32              ret;
#endif // NOT_USED
	nfp_clientCBPtr_t	clientCBp;

	/*-----------------------
	 *Initializes values
  	 *-----------------------
  	 */
	rc = NFX_OK;
	error = NLM_FAILED;
	sendResp = 0;	
	clientCBp = nlmReqp->clientCBp;
	nlm_msgp = nlmReqp->procStart;
	unlockCBp = NULL;
#ifdef NOT_USED
#if 0
	pendingLockRecp = NULL;
#endif
#endif  // NOT_USED

	/*-----------------------
	 *Get client's cookie and
	 *alock info
  	 *-----------------------
  	 */
	length = nfs_extractString(	nlm_msgp, 
								&cookie[0],
								&cookieLen, 
								NLM_MAX_NAME);		
	if (length == NFX_ERR)
	{
		nlm_globalCfgp->nlm_debugCounters.numOfRxBadCookies++;
		return (NFX_ERR);
	}
	cookie[cookieLen]  = '\0';
	nlm_msgp += length;

	/*Get alock info*/
	nlm_lockInfop = (nlm_lockInfo_t *)buf;
	nlm_lockInfop->nlm_callerName = &callerName[0];
	nlm_lockInfop->nlm_oh = &oh[0];
	length = nlm_extractAlockInfo(	nlm_msgp, 
									nlm_lockInfop,
									nlmReqp->rpcCallHdrp->progVer,
									&fileHandle); 
	if (length == NFX_ERR)
	{
		nlm_globalCfgp->nlm_debugCounters.numOfRxBadAlockObj++;
		goto nlm_procRxUnlockReqExit;
	}
		
	fileCBp = nfp_getFileCB (&fileHandle);
	if (fileCBp == NULL)
	{
		/*No lock on this file, return response to client*/
		error = NLM_STAT_GRANTED;
		goto nlm_procRxUnlockReqExit;
	}
    nlmReqp->fileCBp = fileCBp;
    nlmReqp->lockInfop = nlm_lockInfop;

#ifdef NOT_USED
	/*----------------------------
	 *Check our current lock list
	 *for this unlock request
	 *on this file
  	 *----------------------------
  	 */
	NLM_GET_L_REGION_END(nlm_lockInfop->lockRec.nlm_l_end, 
						nlm_lockInfop->lockRec.nlm_l_offset, 
						nlm_lockInfop->lockRec.nlm_l_len,
						nlm_lockInfop->lockRec.nlm_l_eof);	

#ifdef  NLM_DEBUG
	NLM_DBG_FUNC_PRINTF("%s: svid: %d l_offset: %lld l_len: %lld exl: %d state: %d\n",
		__FUNCTION__, 
		nlm_lockInfop->nlm_svid, 
		nlm_lockInfop->lockRec.nlm_l_offset, 
		nlm_lockInfop->lockRec.nlm_l_len,
		nlm_lockInfop->lockRec.nlm_exclusive, 
		nlm_lockInfop->lockRec.nlm_state);	
	NLM_DBG_FUNC_PRINTF("%s: callerNameLen: %d CallerName: %s\n",
		__FUNCTION__,nlm_lockInfop->nlm_callerNameLen, 
		nlm_lockInfop->nlm_callerName);	
	NLM_DBG_FUNC_PRINTF("%s: ohLen: %d oh: %s\n",
		__FUNCTION__,nlm_lockInfop->nlm_ohLen, 
		nlm_lockInfop->nlm_oh);	
#endif

	searchMask = NLM_SEARCH_OH | NLM_SEARCH_SVID;

	unlockRecp = nlm_getLockRecInCurrList(	fileCBp,
											nlm_lockInfop,
											searchMask,
											&nextLockRecp);
	if (unlockRecp == NULL)
	{
		/*----------------------------
		 *No matching lock found, 
		 *return response to client
		 *----------------------------
		 */
		error = NLM_STAT_GRANTED;
		goto nlm_procRxUnlockReqExit;
	}
	unlockCBp = (nfp_lockCB_t	*)unlockRecp->lockCBp;

	if (nfp_validateLockCB(unlockCBp) != NFX_OK)
	{
#ifdef	NLM_VALIDATE_LOCK
		panic("%s: Bad unlockCBp: %p\n", 
						__FUNCTION__, unlockCBp);		
#endif
		error = NLM_STAT_GRANTED;
		goto nlm_procRxUnlockReqExit;
	}


	unlockAlockp = &unlockCBp->protocol.nfp_nlmLock;

#ifdef	NLM_DEBUG	
	printf("%s: unlockCBp: %p svid: %d l_offset: %lld l_len: %lld exl: %d state: %d\n",
		__FUNCTION__, 
		unlockCBp,
		nlm_lockInfop->nlm_svid, 
		nlm_lockInfop->lockRec.nlm_l_offset, 
		nlm_lockInfop->lockRec.nlm_l_len,
		nlm_lockInfop->lockRec.nlm_exclusive, 
		nlm_lockInfop->lockRec.nlm_state);	
	printf("%s: callerNameLen: %d CallerName: %s\n",
		__FUNCTION__,nlm_lockInfop->nlm_callerNameLen, 
		nlm_lockInfop->nlm_callerName);	
#endif

	
	DLL_ITERATE_NEXT(unlockRecp, unlockRecNextp,
					&unlockCBp->lockOwnerList.list,
					lockList)
	{
		if ((unlockRecp == NULL) || (unlockRecp == DLL_NIL) )
			break;

		if (nlm_validateLockRec(unlockRecp) != NFX_OK)	
		{
			DLL_REMOVE(&unlockCBp->lockOwnerList.list,
						unlockRecp, lockList);
			unlockCBp->lockOwnerList.numOfEntries--;
			nlm_deallocLockRec(unlockRecp);
            continue;
		}

		if (nlm_regionCmp(unlockRecp, &nlm_lockInfop->lockRec))
			/*skip if region not overlap*/
			continue;	
		/*----------------------------
		 *Found overlap region to unlock
  	 	 *----------------------------
  	 	 */
		rc = nlm_checkUnlockedRegions(	unlockRecp,
										nlm_lockInfop,
										nlmReqp,
										&overlap,
										&unlockType);

		if (rc != NFX_OK)
			goto nlm_procRxUnlockReqExit;

		/*-------------------------------------------
		 *Check if locked region has been splitted
		 *by this unlock
		 *--------------------------------------------
		 */
		pendingLockRecp = unlockRecp->pendingLockRecp;
		if (unlockType != NLM_UNLOCK_WHOLE_REGION)
		{
			if (!overlap)
			{
				/*still conflict, return*/
				error = NLM_STAT_GRANTED;
				goto nlm_procRxUnlockReqExit;
			}
			continue;
		}
		
		/*---------------------------------------
		 *Remove Unlock Record from current list	
	 	 *---------------------------------------
	 	 */

		DLL_REMOVE(&unlockCBp->lockOwnerList.list,
					unlockRecp, lockList);
		nlm_deallocLockRec(unlockRecp);
		unlockCBp->lockOwnerList.numOfEntries--;
		if ( ((nlm_lockInfop->lockRec.nlm_l_offset == 0) && 
			(nlm_lockInfop->lockRec.nlm_l_len == 0) ) )
		{
			/*------------------------------------
			 *unlock the whole file.  
			 *See if any more any more overlapped
			 *lock regions
	 	 	 *-----------------------------------
	 	 	 */
			continue;
		}

		break;
	}

#ifdef	NLM_DEBUG
	printf("%s 3: \n", __FUNCTION__);
	nlm_showFileLocks(fileCBp);
#endif

	/*----------------------------------------------
	 *Lock found.
	 *Return LCK_GRANTED resp to the Unlock request
	 *----------------------------------------------
	 */

	if (!sendResp)
	{
		rc = nlm_procTxSendResp(nlmReqp, 
								NLM_STAT_GRANTED,
								cookie, 
								cookieLen);
		if (rc != NFX_OK)
		{
			/*----------------------------
			 *Cannot send response,
			 *Just drop Rx pkt and let client rexmit
	 		 *----------------------------
	 		 */
			nlm_globalCfgp->nlm_debugCounters.nlm_sendUnlockReplyErr++;
			return (NFX_ERR);
		}
		sendResp = 1;
	}

	/*-------------------------------------------------
	 *Check if previous unlock Control block 
	 *ownerlist and pending list to see if they're empty.
	 *------------------------------------------------
	 */
	if ( unlockCBp &&
		(unlockCBp->lockOwnerList.numOfEntries <= 0) &&
		(unlockCBp->lockOwnerPendingList.numOfEntries <= 0 ) )
	{
		nlm_rmLockCBFromList(unlockCBp,
							TRUE);
		if (fileCBp && (fileCBp->nfp_lockList.nfp_reqCnt <= 0))
		{
			nfp_delFileCB (&fileCBp->nfp_fileHandle);
		}	 
	}
	
	/*--------------------------------------------
	 *Check for any pending or block request
	 *that matches the lock that we just freed.
	 *If there's pending lock, we need to notify
	 *the the block client
 	 *--------------------------------------------
 	 */
	if (pendingLockRecp == NULL)
	{
		/*------------------------------------
		 *Return if no pending lock 
 	 	 *-----------------------------------
 	 	 */
		goto nlm_procRxUnlockReqExit;
	}

	if (nlm_validateLockRec(pendingLockRecp) != NFX_OK)
	{
		panic ("%s 2: Invalid pendingLockRecp %p\n",
					   	__FUNCTION__,pendingLockRecp);
	}


	if (pendingLockRecp->waitForLockRecp != unlockRecp)
	{
		nlm_globalCfgp->nlm_debugCounters.numOfCallBackErr++;
		return (NFX_ERR);
	}

	/*------------------------------------------------------
	 *Check if pending (block) lock is still in conflict 
	 *with any current active lock(s) on this file
  	 *------------------------------------------------------
  	 */

	searchMask = 0;
	if (!pendingLockRecp->nlm_exclusive)	
		searchMask = NLM_SEARCH_EXCLUSIVE;

	pendingLockCBp = (nfp_lockCBPtr_t)pendingLockRecp->lockCBp;

	if (nfp_validateLockCB(pendingLockCBp) != NFX_OK)
	{
		panic("%s 3: Bad pendingLockCBp: %p\n", 
						__FUNCTION__, pendingLockCBp);		
	}

	pendingLockInfop = &pendingLockCBp->protocol.nfp_nlmLock;
	pendingLockInfop->lockRec.nlm_l_end = pendingLockRecp->nlm_l_end;
	pendingLockInfop->lockRec.nlm_l_offset = pendingLockRecp->nlm_l_offset;
	pendingLockInfop->lockRec.nlm_l_len = pendingLockRecp->nlm_l_len;
	pendingLockInfop->lockRec.nlm_l_eof = pendingLockRecp->nlm_l_eof;

	conflictLockRecp = nlm_getLockRecInCurrList(fileCBp,
												pendingLockInfop,
												searchMask,
												&nextLockRecp);

	if (conflictLockRecp == NULL)
	{
		/*------------------------------------------
		 *Remove pending lock record from pending list	
	 	 *------------------------------------------
	 	 */

		DLL_REMOVE(&pendingLockCBp->lockOwnerPendingList.list,
					pendingLockRecp, lockList);
		pendingLockCBp->lockOwnerPendingList.numOfEntries--;


		/*--------------------------------------------
		 *Add the pending lock to the active lock list	
  	 	 *---------------------------------------------
  	 	 */
		if (nextLockRecp ==  NULL)
		{
			DLL_INSERT_TAIL(&pendingLockCBp->lockOwnerList.list,
							pendingLockRecp, lockList);
		}
		else
		{
			DLL_INSERT_BEFORE(&pendingLockCBp->lockOwnerList.list,
							nextLockRecp, pendingLockRecp, lockList);
		}

		pendingLockCBp->lockOwnerList.numOfEntries++;
		

		/*------------------------------------
		 *call back to block client
		 *Send a Portmapper GetPort to the client to
		 *find the client NLM port 
  	 	 *-----------------------------------
  	 	 */
		new_reqCBp = nfp_allocReqCB ();
		if (new_reqCBp == NULL)
			return (NFX_ERR);
		nlm_globalCfgp->nlm_debugCounters.nlm_allocReqCB++;
		new_nlmReqp = &new_reqCBp->protocol.nlm_req;
		new_nlmReqp->reqCBp = new_reqCBp;
		new_nlmReqp->tplConnp = NULL;
		new_nlmReqp->rxPktDescp = NULL;
		new_nlmReqp->txPktDescp = NULL;
		new_nlmReqp->clientCBp = clientCBp;
		new_nlmReqp->fileCBp =fileCBp;
		new_nlmReqp->lockRecp = pendingLockRecp;
		memcpy(&new_nlmReqp->rpcHdr, 
				nlmReqp->rpcCallHdrp, 
				sizeof(rpcCallHdr_t));
		new_nlmReqp->rpcCallHdrp = &new_nlmReqp->rpcHdr;
		new_nlmReqp->nlm_txCmdInfo.laddr = 
			((pkt_desc_t *)nlmReqp->rxPktDescp)->laddr;

		rc = nlm_getportNlm(new_nlmReqp, pendingLockRecp);
		if (rc != NFX_OK)
		{
			return (NFX_ERR);
		}
		return (NFX_OK);
	}
	else
	{
		rc = nlm_setPendingLockEntry(	conflictLockRecp,
										pendingLockRecp);
		return (rc);
	}

	return (NFX_OK);
#else // NOT_USED
 	/*
	 * Check our current lock list for this unlock request on this file
  	 */
	NLM_GET_L_REGION_END(nlm_lockInfop->nlm_l_end, 
						 nlm_lockInfop->nlm_l_offset, 
						 nlm_lockInfop->nlm_l_len,
						 nlm_lockInfop->nlm_l_eof);	
#ifdef	NLM_DEBUG
	NLM_DBG_FUNC_PRINTF("%s: svid: %d l_offset: %lld l_len: %lld exl: %d state: %d\n",
		__FUNCTION__, 
		nlm_lockInfop->nlm_svid, 
		nlm_lockInfop->nlm_l_offset, 
		nlm_lockInfop->nlm_l_len,
		nlm_lockInfop->nlm_exclusive, 
		nlm_lockInfop->nlm_state);	
	NLM_DBG_FUNC_PRINTF("%s: callerNameLen: %d CallerName: %s\n",
		__FUNCTION__,nlm_lockInfop->nlm_callerNameLen, 
		nlm_lockInfop->nlm_callerName);	
	NLM_DBG_FUNC_PRINTF("%s: ohLen: %d oh: %s\n",
		__FUNCTION__,nlm_lockInfop->nlm_ohLen, 
		nlm_lockInfop->nlm_oh);	
#endif
    /*
     * Send unlock req, to the lock module.
     * If there are pending locks that now become granted, we will get a
     * callback.
     */
    ret = nlm_procUnlockReq(nlm_lockInfop, nlmReqp);
    if (ret == NFX_ERR)
    {
        /*
         * Some internal error.
         */
        error = NLM_FAILED;
    }
    else
    {
        /*
         * Successfully removed the lock
         * If not matching lock was found, it would be treated as a duplicate
         * message and returned OK.
         */
        error = NLM_STAT_GRANTED;
    }
    if (lock_areThereCurrentFileLocksReq(fileCBp->fileLockIdx) == FALSE)
    {
        nfp_deallocFileCB(fileCBp);
        nfp_delFileCB(&fileHandle);
    }
#endif // NOT_USED


nlm_procRxUnlockReqExit:

	if (!sendResp)
	{
//		nlm_globalCfgp->nlm_debugCounters.nlm_rcvUnlockErr++;
		rc = nlm_procTxSendResp(nlmReqp, 
								error,
								cookie, 
								cookieLen);
	}

#ifdef NOT_USED
	if ( unlockCBp && 
		(unlockCBp->lockOwnerList.numOfEntries <= 0) &&
		(unlockCBp->lockOwnerPendingList.numOfEntries <= 0 ) )
	{
		nlm_rmLockCBFromList(unlockCBp,
							TRUE);
		if (fileCBp && (fileCBp->nfp_lockList.nfp_reqCnt <= 0))
		{
			nfp_delFileCB (&fileCBp->nfp_fileHandle);
		}	 

	}
#endif
    if (rc == NFX_OK)
    {
        /* We should be freeing up the memory */
        // nlmReqp->rxPktDescp = NULL;
        // nlm_freeAll(nlmReqp, TRUE);
    }
	return (rc);

}


/*-----------------------------------------------------------------
 * Name:	nlm_procRxGrantedReply	
 *
 * Description:
 * Process received NLM_GRANTED reply from the client.
 *
 *
 * Created by:  Joseph Vu
 *
 * Date Created:        7/18/02
 *
 *-----------------------------------------------------------------
 */

#ifdef NOT_USED
int32 
nlm_procRxGrantedReply (nlm_request_t    *nlmReqp)
#else // NOT_USED
int32 
nlm_procRxGrantedReply (nlm_request_t *nlmReqp, lock_request_t *lockReq)
#endif // NOT_USED
{
	int32				length;
	int32				status;
	ushort16			cookieLen;
	char				*nlm_msgp;
	char				cookie[NLM_MAX_NAME];
	nfp_clientCBPtr_t	clientCBp;
#ifndef NOT_USED
    nfp_fileCB_t        *fileCBp = (nfp_fileCB_t *)nlmReqp->fileCBp;
#endif

	clientCBp = nlmReqp->clientCBp;

	/*------------------------------------
	 *Get start of NLM_Granted reply
	 *Get client's cookie 
   	 *-----------------------------------
   	 */
	nlm_msgp = nlmReqp->procStart;

	length = nfs_extractString(nlm_msgp, 
								&cookie[0],
								&cookieLen, 
								NLM_MAX_NAME);		
	if (length == NFX_ERR)
	{
		nlm_globalCfgp->nlm_debugCounters.numOfRxBadCookies++;
		return (NFX_ERR);
	}
	cookie[cookieLen]  = '\0';		//NULL terminated
	nlm_msgp += length;


	status = 0;
	length = nfs_extractLongVal(nlm_msgp, &status);	

	if (status != NLM_STAT_GRANTED)
	{
		/*------------------------------------
		 *Client denied lock callback
		 *Check for next pending lock
   	 	 *-----------------------------------
   	 	 */
#ifdef NOT_USED
		//nlm_callbackNextPendingLock(nlmReqp->lockRecp,
	//								nlmReqp, NLM_CURRENT_LIST);
#else
        lock_unlockSingleReq(fileCBp->fileLockIdx, lockReq);
        /* Cannot do any thing about the return code here */
        if (lock_areThereCurrentFileLocksReq(fileCBp->fileLockIdx) == FALSE)
        {
            nfp_delFileCB(&fileCBp->nfp_fileHandle);
            nfp_deallocFileCB(fileCBp);
        }
#endif
		nlm_globalCfgp->nlm_debugCounters.nlm_grantedReplyErr++;
//		return (NFX_ERR);
	}

	/*------------------------------------
	 *Delete NLM request control block from  client list
   	 *-----------------------------------
 	 */
	nlmReqp->nlm_txCmdInfo.grantedRetry = 0;
	nlmReqp->nlm_txCmdInfo.grantedMaxTimeWaitCnt = 0;
	nlmReqp->reqState = NLM_REQ_DONE;
	nlmReqp->rxPktDescp = NULL;
	nlm_delReqCBFromList(nlmReqp);
	nlm_freeAll(nlmReqp, TRUE);

	return (NFX_OK);
}


int32
nlm_procRxInvalidReq (nlm_request_t    *nlmReqp)
{
	//Should not get here

	return (NLM_FAILED);
}

int32 
nlm_procRxShareReq (nlm_request_t    *nlmReqp)
{



	return (NLM_STAT_GRANTED);
}

int32
nlm_procRxUnsharetReq (nlm_request_t    *nlmReqp)
{



	return (NLM_STAT_GRANTED);

}

int32
nlm_procRxNmLockReq (nlm_request_t    *nlmReqp)
{


	return (NLM_STAT_GRANTED);

}


/*-----------------------------------------------------------------
 * Name:nlm_procRxNlmFreeAllReq		
 *
 * Description:
 *		Free all locks.
 *  
 * Created by:  Joseph Vu
 *
 * Date Created:        7/18/02
 *
 *-----------------------------------------------------------------
 */

int32
nlm_procRxNlmFreeAllReq (nlm_request_t    *nlmReqp)
{
   int32               error;
    uint32              txMsgLen;
	char                *txMsgp;
    nfp_clientCBPtr_t   clientCBp;
    pkt_desc_t          *tplXmitPktDescp;


    error = NLM_FAILED;
    clientCBp = nlmReqp->clientCBp;

    /*------------------------------------
     *Alloc TPL packet to send
	 *------------------------------------
	 */

    tplXmitPktDescp = TPL_ALLOC_PACKET(nlmReqp->tplConnp, 0);
    if (tplXmitPktDescp == NULL)
    {
        nlm_globalCfgp->nlm_debugCounters.nlm_tplXmtNoPktResource++;
        return (NFX_ERR);
    }
    nlm_globalCfgp->nlm_debugCounters.nlm_tplXmtPktAlloc++;

    txMsgp = (char *)tplXmitPktDescp->pHdr;
    txMsgLen = 0;

    /*------------------------------------
     *Set up RPC reply header &
	 *Set Pkt Desc Hdr size
	 *------------------------------------
	 */

    txMsgLen += nfs_msgRpcReply(txMsgp, 
								nlmReqp->rpcCallHdrp->rpc_xid,
                				RPC_REPLY_MSG_ACCEPTED, 
								RPC_REPLY_SUCCESS);
    txMsgp += txMsgLen;

    /*Set packet desc hdr size*/
    tplXmitPktDescp->hdrSize = txMsgLen;

    if (nlm_procTxPkt(tplXmitPktDescp, nlmReqp) != NFX_OK)
    {
        nlmReqp->txPktDescp = (void *)tplXmitPktDescp;
        nlmReqp->reqState = NLM_REQ_DONE;
        return (NFX_ERR);
    }

    /*-----------------------------------------------------
	 *CLients has restarted, need to clean up all locks
	 *held by client
	 *----------------------------------------------------
	 */

#ifdef NOT_USED
	nlm_clearLock( ((pkt_desc_t *)nlmReqp->rxPktDescp)->raddr);
#else
    nlm_nsmClientDownInd(clientCBp->nfp_ipAddr);
#endif

    return (NFX_OK);

}

/*-----------------------------------------------------------------
 * Name:	nlm_procRxGrantedReq	
 *
 * Description:
 *	We should not has received a Granted callback since we 
 *	would never asked for any file lock request, but
 *	in case we do received this callback, just return
 *	reply back to the client.
 *  
 * Created by:  Joseph Vu
 *
 * Date Created:        7/18/02
 *
 *-----------------------------------------------------------------
 */
int32
nlm_procRxGrantedReq(nlm_request_t    *nlmReqp)
{
	int32				rc;			
	int32				error;
	int32				length;
	ushort16			cookieLen;
	char				*nlm_msgp;
	char         		cookie[NLM_MAX_NAME];	

	
	/*------------------------------------
	 *Set default error
	 *Get start of message
   	 *-----------------------------------
   	 */

	error = NLM_FAILED;
	nlm_msgp = nlmReqp->procStart;

	/*------------------------------------
	 *Get client's cookie &
	 *Send a reply to client
   	 *-----------------------------------
   	 */
	
	length = nfs_extractString(	nlm_msgp, 
								&cookie[0],
								&cookieLen, 
								NLM_MAX_NAME);		
	if (length == NFX_ERR)
	{
		nlm_globalCfgp->nlm_debugCounters.numOfRxBadCookies++;
		return (NFX_ERR);
	}
	cookie[cookieLen]  = '\0';		//NULL terminated

	rc = nlm_procTxSendResp(nlmReqp, 
							NLM_STAT_GRANTED,
							cookie, 
							cookieLen);

	return (rc);
}


/*-----------------------------------------------------------------
 * Name:		nlm_procRxTestMsg	
 *
 * Description:
 * Process received Async NLM_TEST_MSG request from client.
 * This procedure checks whether a lock is available
 * to the client.
 *
 * Created by:  Joseph Vu
 *
 * Date Created:        7/18/02
 *
 *-----------------------------------------------------------------
 */
int32
nlm_procRxTestMsg(nlm_request_t    *nlmReqp)
{
	int32				rc;			
	int32				error;
	int32				length;
	ushort16			cookieLen;
	uint32				exclusiveFlag;
	char				*nlm_msgp;
	nfp_fileCBPtr_t		fileCBp;	
	char				cookie[NLM_MAX_NAME];
	static char			callerName[NLM_MAX_NAME];
	static char			oh[NLM_MAX_NAME];		
	static char 		buf[sizeof(nlm_lockInfo_t)];
#ifdef NOT_USED
	uint32				listType;
	int32				searchMask;
	nlm_lockRec_t 		*lockRecp;
	nlm_lockRec_t		*nextLockRecp;
#else
	nlm_lockInfo_t		*nlm_lockInfop;
#endif
	fs_Handle       	fileHandle;	

	
	error = NLM_FAILED;
#ifdef NOT_USED
	nlmReqp->lockRecp = NULL;
#endif

	/*-----------------------------
	 *Get start of NLM_TEST procedure 
	 *-----------------------------
	 */
	
	nlm_msgp = nlmReqp->procStart;

	/*-----------------------------
	 * Get client's cookie &
	 * update msg ptr. 
	 *-----------------------------
	 */
	
	length = nfs_extractString(	nlm_msgp, 
								&cookie[0],
								&cookieLen, 
								NLM_MAX_NAME);		
	if (length == NFX_ERR)
	{
		nlm_globalCfgp->nlm_debugCounters.numOfRxBadCookies++;
		return (NFX_ERR);
	}

	cookie[cookieLen]  = '\0';
	nlm_msgp += length;

	/*-----------------------------
	 * Get exclusive flag
	 * and alock info
	 *-----------------------------
	 */

	exclusiveFlag = 0;
	length = nfs_extractLongVal(nlm_msgp, &exclusiveFlag);	
	nlm_msgp += length;

	nlm_lockInfop = (nlm_lockInfo_t *)buf;
	nlm_lockInfop->nlm_callerName = &callerName[0];
	nlm_lockInfop->nlm_oh = &oh[0];
	length = nlm_extractAlockInfo(	nlm_msgp, 
									nlm_lockInfop,
									nlmReqp->rpcCallHdrp->progVer,
									&fileHandle); 
	if (length == NFX_ERR)
	{
		nlm_globalCfgp->nlm_debugCounters.numOfRxBadAlockObj++;
		rc = nlm_procTxSendASyncMsgResp(nlmReqp, 
										error,
										cookie, 
										cookieLen);
		return (rc);
	}

	/*-----------------------------
	 *Set exclusive flag
	 *-----------------------------
	 */
#ifdef NOT_USED
	nlm_lockInfop->lockRec.nlm_exclusive = 
		(exclusiveFlag != 0)? TRUE : FALSE;
#else
    nlm_lockInfop->nlm_exclusive = (exclusiveFlag != 0)? TRUE : FALSE;
#endif

#ifdef	NLM_DEBUG
#ifdef NOT_USED
	printf("%s: svid: %d l_offset: %lld l_len: %lld exl: %d state: %d\n",
		__FUNCTION__, nlm_lockInfop->nlm_svid, 
		nlm_lockInfop->lockRec.nlm_l_offset, 
		nlm_lockInfop->lockRec.nlm_l_len,
		nlm_lockInfop->lockRec.nlm_exclusive, 
		nlm_lockInfop->lockRec.nlm_state);
#else // NOT_USED
    printf("%s: svid: %d l_offset: %lld l_len: %lld exl: %d state: %d\n",
        __FUNCTION__, nlm_lockInfop->nlm_svid,
        nlm_lockInfop->nlm_l_offset,
        nlm_lockInfop->nlm_l_len,
        nlm_lockInfop->nlm_exclusive,
        nlm_lockInfop->nlm_state);
#endif // NOT_USED
	printf("%s: callerNameLen: %d CallerName: %s\n",
		__FUNCTION__,nlm_lockInfop->nlm_callerNameLen, 
		nlm_lockInfop->nlm_callerName);	
	printf("%s: ohLen: %d oh: %s\n",
		__FUNCTION__,nlm_lockInfop->nlm_ohLen, 
		nlm_lockInfop->nlm_oh);	
#endif	

#ifdef NOT_USED
	E_LOG (class_1, LOG_DEBUG, 
			nlm_globalCfgp->eee_nlmAppId, 0, 0, 0,
		("%s: svid:%d offset:%lld len:%lld exl:%d state:%d\n",
		__FUNCTION__,
		nlm_lockInfop->nlm_svid, 
		nlm_lockInfop->lockRec.nlm_l_offset, 
		nlm_lockInfop->lockRec.nlm_l_len,
		nlm_lockInfop->lockRec.nlm_exclusive, 
		nlm_lockInfop->lockRec.nlm_state));
#else
    E_LOG (class_1, LOG_DEBUG,
            nlm_globalCfgp->eee_nlmAppId, 0, 0, 0,
        ("%s: svid:%d offset:%lld len:%lld exl:%d state:%d\n",
        __FUNCTION__,
        nlm_lockInfop->nlm_svid,
        nlm_lockInfop->nlm_l_offset,
        nlm_lockInfop->nlm_l_len,
        nlm_lockInfop->nlm_exclusive,
        nlm_lockInfop->nlm_state));
#endif

	/*-------------------------------------------
	 *Check if there is any locks on this file
	 *-------------------------------------------
	 */
	
	fileCBp = nfp_getFileCB (&fileHandle);
	if (fileCBp == NULL)
		/*-------------------------------------------------
		 *No lock on this file, return LCK_GRANTED status 
		 *to client
		 *------------------------------------------------
		 */
		goto nlm_procRxTestMsgOK;	

	nlmReqp->fileCBp = fileCBp;

#ifdef NOT_USED
	/*-----------------------------
	 *Get end of lock region	
	 *-----------------------------
	 */
	NLM_GET_L_REGION_END(nlm_lockInfop->lockRec.nlm_l_end, 
						nlm_lockInfop->lockRec.nlm_l_offset, 
						nlm_lockInfop->lockRec.nlm_l_len,
						nlm_lockInfop->lockRec.nlm_l_eof);			
	
	/*-------------------------------------
	 *There're locks on this file
	 *Check if request lock are conflict 
	 *with existing locks or pending locks
	 *-------------------------------------
	 */
		
	searchMask = 0;
	if (!exclusiveFlag)
		searchMask = NLM_SEARCH_EXCLUSIVE;
	lockRecp =   nlm_getConflictinglock(fileCBp,
										nlm_lockInfop, 
										searchMask, 
										&nextLockRecp,
										&listType);
	if (lockRecp != NULL)
	{
		/*-----------------------------
	 	 *conflicting lock, return DENIED 
	 	 *-----------------------------
	 	 */
		nlmReqp->lockRecp = lockRecp;

#ifdef	NLM_DEBUG
		printf("%s: DENIED: lockRecp:%p oh:%s off:%lld len:%lld\n", 
		__FUNCTION__, lockRecp,
		((nfp_lockCBPtr_t)lockRecp->lockCBp)->protocol.nfp_nlmLock.nlm_oh,
		lockRecp->nlm_l_offset,
		lockRecp->nlm_l_len);
#endif
		rc = nlm_procTxSendASyncMsgResp(nlmReqp, 
										NLM_ERR_DENIED,
										cookie, 
										cookieLen);
		return (rc);

	}
#else
    /*
     *Get end of lock region    
     */

    NLM_GET_L_REGION_END(nlm_lockInfop->nlm_l_end,
                         nlm_lockInfop->nlm_l_offset,
                         nlm_lockInfop->nlm_l_len,
                         nlm_lockInfop->nlm_l_eof);
   error = nlm_procTestReq(fileCBp, nlm_lockInfop, nlmReqp);
#endif // NOT_USED

	/*-------------------------------------------
 	 *No conflicting lock, return GRANTED status 
 	 *--------------------------------------------
 	 */
nlm_procRxTestMsgOK:
	rc = nlm_procTxSendASyncMsgResp(nlmReqp, 
									NLM_STAT_GRANTED,
									cookie, 
									cookieLen);
    if (rc == NFX_OK)
    {
        /* We should be freeing up the memory */
        // nlmReqp->rxPktDescp = NULL;
        // nlm_freeAll(nlmReqp, TRUE);
    }

	return (rc);
}

/*-----------------------------------------------------------------
 * Name:		nlm_procRxLockMsg	
 *
 * Description:
 *	Process received Asynchronous NLM_LOCK_MSG request from client.
 *	This procedure creates a locked byte range on a file. 
 * Created by:  Joseph Vu
 *
 * Date Created:        7/18/02
 *
 *-----------------------------------------------------------------
 */

int32
nlm_procRxLockMsg(nlm_request_t    *nlmReqp)
{
	nfp_fileCBPtr_t		fileCBp;	
	nlm_lockInfo_t		*nlm_lockInfop;
	nfp_clientCBPtr_t	clientCBp;
	int32				rc;
	uint32				block;
	uint32				exclusive;
	uint32				reclaim;
	int32				error;
	int32				length;
	ushort16			cookieLen;
	char				*nlm_msgp;
	char				cookie[NLM_MAX_NAME];
	static char			callerName[NLM_MAX_NAME];
	static char			oh[NLM_MAX_NAME];
	static char    		buf[sizeof(nlm_lockInfo_t)];
	fs_Handle       	fileHandle;	
#ifdef NOT_USED
    nlm_lockRec_t       *lockRecp;
#else
    int32               newFile = 0, ret;
#endif


	error = NLM_FAILED;
	rc = NFX_OK;

	clientCBp = nlmReqp->clientCBp;

	/*-----------------------------
	 * Get start of message &
	 * client cookie field
	 *-----------------------------
	 */
	
	nlm_msgp = nlmReqp->procStart;

	length = nfs_extractString(	nlm_msgp, 
								&cookie[0],
								&cookieLen, 
								NLM_MAX_NAME);		
	if (length == NFX_ERR)
	{
		nlm_globalCfgp->nlm_debugCounters.numOfRxBadCookies++;
		return (NFX_ERR);
	}
	cookie[cookieLen]  = '\0';		
	nlm_msgp += length;

	/*-----------------------------
 	 *Get block field, exclusive field
	 *and alock info 
 	 *-----------------------------
 	 */
	length = nfs_extractLongVal(nlm_msgp, &block);
	nlm_msgp += length;

	length = nfs_extractLongVal(nlm_msgp, &exclusive);
	nlm_msgp += length;

	nlm_lockInfop = (nlm_lockInfo_t *)buf;
    nlm_lockInfop->nlm_callerName = &callerName[0];
    nlm_lockInfop->nlm_oh = &oh[0];
	length = nlm_extractAlockInfo(	nlm_msgp, 
									nlm_lockInfop,
									nlmReqp->rpcCallHdrp->progVer,
									&fileHandle);
	if (length == NFX_ERR)
	{
		nlm_globalCfgp->nlm_debugCounters.numOfRxBadAlockObj++;
		goto nlm_procRxLockMsgErr;
	}
	nlm_msgp += length;

	/*-----------------------------
 	 *Get reclaim field 
 	 *-----------------------------
 	 */
	length = nfs_extractLongVal(nlm_msgp, &reclaim);
	nlm_msgp += length;

	/*----------------------------------------------
	 *Check if state is in GRACE PERIOD.
	 *If so, only allow clients with reclaim locks
 	 *-----------------------------------------------
 	 */

	if (nlm_globalCfgp->gracePeriod && !reclaim)
	{
		/* Don't accept new lock requests during grace period */
		error = NLM_ERR_DENIED_GRACE_PERIOD;
		nlm_globalCfgp->nlm_debugCounters.numOfRxGracePeriodErr++;
		goto nlm_procRxLockMsgErr;
	}

	/*-----------------------------
 	 *Get current state of NSM,
	 *save block field,
	 *and set exlusive flag
 	 *-----------------------------
 	 */
#ifdef NOT_USED
	length = nfs_extractLongVal(nlm_msgp, 
								&nlm_lockInfop->lockRec.nlm_state);
#else
    length = nfs_extractLongVal(nlm_msgp, &nlm_lockInfop->nlm_state);
#endif
	nlm_msgp += length;

	nlmReqp->block = block;

#ifdef NOT_USED
	nlm_lockInfop->lockRec.nlm_exclusive = 
		(exclusive != 0)? TRUE : FALSE;
#else
    nlm_lockInfop->nlm_exclusive = (exclusive != 0)? TRUE : FALSE;
#endif

#ifdef	NLM_DEBUG
#ifdef NOT_USED
	NLM_DBG_FUNC_PRINTF("%s: svid: %d l_offset: %lld l_len: %lld exl: %d state: %d\n",
		__FUNCTION__, nlm_lockInfop->nlm_svid, 
		nlm_lockInfop->lockRec.nlm_l_offset, 
		nlm_lockInfop->lockRec.nlm_l_len,
		nlm_lockInfop->lockRec.nlm_exclusive, 
		nlm_lockInfop->lockRec.nlm_state);	
#else
    printf("%s: svid: %d l_offset: %lld l_len: %lld exl: %d state: %d\n",
        __FUNCTION__, nlm_lockInfop->nlm_svid,
        nlm_lockInfop->nlm_l_offset,
        nlm_lockInfop->nlm_l_len,
        nlm_lockInfop->nlm_exclusive,
        nlm_lockInfop->nlm_state);
#endif
	NLM_DBG_FUNC_PRINTF("%s: callerNameLen: %d CallerName: %s\n",
		__FUNCTION__,nlm_lockInfop->nlm_callerNameLen, 
		nlm_lockInfop->nlm_callerName);	
	NLM_DBG_FUNC_PRINTF("%s: ohLen: %d oh: %s\n",
		__FUNCTION__,nlm_lockInfop->nlm_ohLen, 
		nlm_lockInfop->nlm_oh);	
#endif	

	/*---------------------------------------------
	 *Check if there is any locks on this file
 	 *---------------------------------------------
 	 */
	
	fileCBp = nfp_getFileCB (&fileHandle);
#ifdef NOT_USED
	if (fileCBp == NULL)
	{
		/*-----------------------------
		 *No lock on this file
		 *Alloc FileCB and add Lock Rec 
 	 	 *-----------------------------
 	 	 */

		fileCBp = nfp_allocFileCB();
		if (fileCBp == NULL)
			goto nlm_procRxLockMsgErr;

		lockRecp = nlm_addLockRecToCurrList(clientCBp,
											fileCBp, 
											nlm_lockInfop,
											NULL,
											1,
											&fileHandle);
		if (lockRecp == NULL)
		{
			error = NLM_ERR_DENIED_NOLOCKS;
			nfp_delFileCB (&fileHandle);
			goto nlm_procRxLockMsgErr;
		}


		rc = nlm_procTxSendASyncMsgResp(nlmReqp, 
										NLM_STAT_GRANTED,
										cookie, 
										cookieLen);
		if (rc != NFX_OK)
		{
			nfp_lockCB_t 		*lockCBp;

			lockCBp = (nfp_lockCB_t	*)lockRecp->lockCBp;
			DLL_REMOVE(&lockCBp->lockOwnerList.list,
						lockRecp, lockList);
			nlm_deallocLockRec(lockRecp);
			lockCBp->lockOwnerList.numOfEntries--;
			nfp_deallocLockCB(lockCBp);
//			nfs_delCBFromList(&clientCBp->nfp_fileList,
//				(void *)fileCBp, NFP_FILECB_NODE);
			nfs_delCBFromList(&fileCBp->nfp_lockList, 
				(void *)lockCBp, NFP_LOCKCB_NODE);
			nfp_delFileCB (&fileHandle);
			return (NFX_ERR);
		}

#ifdef	NLM_DEBUG
		NLM_DBG_FUNC_PRINTF("%s: Register client %s addr: %08x volId: %llx clientCBp: %p\n",
				__FUNCTION__,
				nlm_lockInfop->nlm_callerName,
				clientCBp->nfp_ipAddr,
				fileHandle.volId.whole_thing,
				clientCBp);
#endif
		/*---------------------------------------------
		 *register client with NSM (Network Status Mon)
 	 	 *---------------------------------------------
 	 	 */
		nsm_procMonReq(	nlm_lockInfop->nlm_callerName,
						clientCBp->nfp_ipAddr,
						fileHandle.volId,
						clientCBp);


		return (NFX_OK);
	}

#ifdef	NLM_DEBUG
	NLM_DBG_FUNC_PRINTF("%s: Existing fileCBp: %p volId:%llx inum: %llx mntPnt:%llx \n",
		   	__FUNCTION__, fileCBp,
			fileCBp->nfp_fileHandle.volId.whole_thing,
			fileCBp->nfp_fileHandle.inum,
			fileCBp->nfp_fileHandle.mntPnt);
#endif

	/*-------------------------------------
	 *There're active locks on this file.
	 *Checks for conflict or overlap region
  	 *--------------------------------------
  	 */

	nlmReqp->fileCBp = fileCBp;
	error = nlm_checkforConflictLock(nlmReqp,
									fileCBp, 
									nlm_lockInfop, 
									0,
									cookie, 
									cookieLen);

	if (error != NFX_OK)
	{
		goto nlm_procRxLockMsgErr;
	}
	return (NFX_OK);

#else // NOT_USED

    if (fileCBp == NULL)
    {
        /*
         * No locks on this file. Allocate FileCB add lock
         */
        fileCBp = nfp_allocFileCB();
        if (fileCBp == NULL)
            goto nlm_procRxLockMsgErr;
        nfp_addFileCB(fileCBp, &fileHandle);
        newFile = 1;
    }
    nlmReqp->fileCBp = fileCBp;
    nlmReqp->lockInfop = nlm_lockInfop;
    ret = nlm_procLockReq(nlm_lockInfop, nlmReqp);
    if (ret == LOCK_STATE_GRANTED)
    {
        /* The lock has been granted */
        error = NLM_STAT_GRANTED;
    }
    else if (ret == LOCK_STATE_DENIED)
    {
        /* The lock has been denied. This could happen for any # of reasons */
        error = NLM_ERR_DENIED;
        goto nlm_procRxLockMsgErr;
    }
    else
    {
        /*
         * The lock has been put in the pending Q. We should get a call back.
         */
        error = NLM_ERR_BLOCKED;
    }
    rc = nlm_procTxSendASyncMsgResp(nlmReqp, error, cookie, cookieLen);
    if (rc != NFX_OK)
    {
        if (ret == LOCK_STATE_GRANTED)
            nlm_procUnlockReq(nlm_lockInfop, nlmReqp);
        else if (ret == LOCK_STATE_BLOCKED)
            nlm_procCancellockReq(nlm_lockInfop, nlmReqp);
        if (lock_areThereCurrentFileLocksReq(fileCBp->fileLockIdx) == FALSE)
        {
            nfp_deallocFileCB(fileCBp);
            nfp_delFileCB(&fileHandle);
        }

        return (NFX_ERR);
    }
    if (newFile && ret != LOCK_STATE_DENIED)
    {
        /*
         * Register client with NSM (Network Status Mon)
         */
#ifdef  NLM_DEBUG
        NLM_DBG_FUNC_PRINTF("%s: Register client %s addr: %08x volId: %llx clientCBp: %p\n",
                __FUNCTION__,
                nlm_lockInfop->nlm_callerName,
                clientCBp->nfp_ipAddr,
                fileHandle.volId.whole_thing,
                clientCBp);
#endif
        nsm_procMonReq( nlm_lockInfop->nlm_callerName,
                        clientCBp->nfp_ipAddr,
                        fileHandle.volId,
                        clientCBp);
    }

#ifdef  NLM_DEBUG
    NLM_DBG_FUNC_PRINTF("%s: FileCBp: %p volId:%llx inum: %llx mntPnt:%llx \n",
            __FUNCTION__, fileCBp,
            fileCBp->nfp_fileHandle.volId.whole_thing,
            fileCBp->nfp_fileHandle.inum,
            fileCBp->nfp_fileHandle.mntPnt);
#endif
    return (NFX_OK);

#endif // NOT_USED

nlm_procRxLockMsgErr:
	if (error == NFX_ERR)
		return (NFX_ERR);
	
	//Send error response back to client
	rc = nlm_procTxSendASyncMsgResp(nlmReqp, 
									error,
									cookie, 
									cookieLen);
    if (rc == NFX_OK)
    {
        /* We should be freeing up the memory */
        // nlmReqp->rxPktDescp = NULL;
        // nlm_freeAll(nlmReqp, TRUE);
    }
	return (rc);

}

/*-----------------------------------------------------------------
 * Name:		nlm_procRxCancelMsg	
 *
 * Description:
 * Process received Asynchronous NLM_CANCEL_MSG request from client.
 * This procedure cancels a blocked LOCK request. 
 *
 * Created by:  Joseph Vu
 *
 * Date Created:        7/18/02
 *
 *-----------------------------------------------------------------
 */
int32
nlm_procRxCancelMsg(nlm_request_t    *nlmReqp)
{
	fs_Handle       	fileHandle;	
	int32				rc;			
	int32				error;
	int32				length;
	uint32				block;
	ushort16			cookieLen;
	uint32				exclusiveFlag;
	char				*nlm_msgp;
	nfp_fileCBPtr_t		fileCBp;
	char				cookie[NLM_MAX_NAME];	
	static char			callerName[NLM_MAX_NAME];	
	static char			oh[NLM_MAX_NAME];	
	static char			buf[sizeof(nlm_lockInfo_t)];
	nlm_lockInfo_t		*nlm_lockInfop;
#ifdef NOT_USED
	short16				searchMask;
	nlm_lockRec_t       *next_lockRecp;
	nfp_lockCBPtr_t		lockCBp;
	nlm_lockRec_t		*lockRecp;
#else
    int32               ret;
#if 0
    nlm_pendingLockNode_t *pendNode;
#endif
#endif

	/*-------------------------------------
	 *Set default error.
	 *Get start of NLM_CANCEL_MSG procedure
  	 *--------------------------------------
  	 */
	error = NLM_STAT_GRANTED;
	nlm_msgp = nlmReqp->procStart;

	/*-------------------------------------
	 *Get client's cookie, block field,
	 *exlusive flag and alock info. 
  	 *--------------------------------------
  	 */
	length = nfs_extractString(	nlm_msgp, 
								&cookie[0],
								&cookieLen, 
								NLM_MAX_NAME);		
	if (length == NFX_ERR)
	{
		nlm_globalCfgp->nlm_debugCounters.numOfRxBadCookies++;
		return (NFX_ERR);
	}
	cookie[cookieLen]  = '\0';
	nlm_msgp += length;

	/*Get block field*/
	length = nfs_extractLongVal(nlm_msgp, &block);
	nlm_msgp += length;

	/*Get exclusive flag*/
	exclusiveFlag = 0;
	length = nfs_extractLongVal(nlm_msgp, &exclusiveFlag);	
	nlm_msgp += length;

	/*Get alock info*/
	nlm_lockInfop = (nlm_lockInfo_t *)buf;
    nlm_lockInfop->nlm_callerName = &callerName[0];
    nlm_lockInfop->nlm_oh = &oh[0];

	length = nlm_extractAlockInfo(	nlm_msgp, 
									nlm_lockInfop,
									nlmReqp->rpcCallHdrp->progVer,
									&fileHandle); 
	if (length != NFX_ERR)
	{
		nlm_globalCfgp->nlm_debugCounters.numOfRxBadAlockObj++;
		goto nlm_procRxCancelMsgErr;	
	}
#ifdef NOT_USED
    nlm_lockInfop->lockRec.nlm_exclusive =
        (exclusiveFlag != 0)? TRUE : FALSE;
#else
    nlm_lockInfop->nlm_exclusive = (exclusiveFlag != 0)? TRUE : FALSE;
#endif

#ifdef	NLM_DEBUG
#ifdef NOT_USED
	NLM_DBG_FUNC_PRINTF("%s: svid: %d l_offset: %lld l_len: %lld exl: %d state: %d\n",
		__FUNCTION__, 
		nlm_lockInfop->nlm_svid, 
		nlm_lockInfop->lockRec.nlm_l_offset, 
		nlm_lockInfop->lockRec.nlm_l_len,
		nlm_lockInfop->lockRec.nlm_exclusive, 
		nlm_lockInfop->lockRec.nlm_state);	
#else // NOT_USED
    NLM_DBG_FUNC_PRINTF("%s: svid: %d l_offset: %lld l_len: %lld exl: %d state: %d\n",
        __FUNCTION__,
        nlm_lockInfop->nlm_svid,
        nlm_lockInfop->nlm_l_offset,
        nlm_lockInfop->nlm_l_len,
        nlm_lockInfop->nlm_exclusive,
        nlm_lockInfop->nlm_state);
#endif
	NLM_DBG_FUNC_PRINTF("%s: callerNameLen: %d CallerName: %s\n",
		__FUNCTION__,nlm_lockInfop->nlm_callerNameLen, 
		nlm_lockInfop->nlm_callerName);	
	NLM_DBG_FUNC_PRINTF("%s: ohLen: %d oh: %s\n",
		__FUNCTION__,nlm_lockInfop->nlm_ohLen, 
		nlm_lockInfop->nlm_oh);	
#endif

	/*--------------------------------------------
	 *Check if there is a matching pending request 
  	 *--------------------------------------------
  	 */
	
	fileCBp = nfp_getFileCB (&fileHandle);
	if (fileCBp == NULL)
	{
		error = NLM_ERR_DENIED;
		goto nlm_procRxCancelMsgErr;	
	}
    nlmReqp->fileCBp = fileCBp;
    nlmReqp->lockInfop = nlm_lockInfop;
#ifdef NOT_USED
	searchMask = (NLM_SEARCH_OH | NLM_SEARCH_SVID | 
				NLM_SEARCH_REGION_MATCH | NLM_SEARCH_EXCLUSIVE);
	lockRecp = nlm_getLockRecInPendingList(	fileCBp,
											nlm_lockInfop,
											searchMask,
											&next_lockRecp);	

	if (lockRecp == NULL)
	{
		error = NLM_ERR_DENIED;
		goto nlm_procRxCancelMsgErr;
	}

	/*------------------------------------------
	 *Remove pending lock record from pending list
	 *------------------------------------------
	 */
	lockCBp = (nfp_lockCBPtr_t )lockRecp->lockCBp;
	DLL_REMOVE(&lockCBp->lockOwnerPendingList.list,
				lockRecp, lockList);
	nlm_deallocLockRec(lockRecp);
	lockCBp->lockOwnerPendingList.numOfEntries--;

	rc = nlm_procTxSendASyncMsgResp(nlmReqp, 
									NLM_STAT_GRANTED,
									cookie, 
									cookieLen);

	return (rc);
#else // NOT_USED
    ret = nlm_procCancellockReq(nlm_lockInfop, nlmReqp);
    if (ret == NFX_ERR)
        error = NLM_ERR_DENIED;
    else
        error = NLM_STAT_GRANTED;
    if (lock_areThereCurrentFileLocksReq(fileCBp->fileLockIdx) == FALSE)
    {
        nfp_deallocFileCB(fileCBp);
        nfp_delFileCB(&fileHandle);
    }
#endif // NOT_USED


nlm_procRxCancelMsgErr:
	/*--------------------------------------------
	 *Return LCK_DENIED
	 *Error: no matching lock to cancel
	 *Send error response back to client
  	 *--------------------------------------------
  	 */
#ifdef NOT_USED
    nlmReqp->lockRecp = NULL;
#endif
	nlm_globalCfgp->nlm_debugCounters.nlm_rcvCancelErr++;

	rc = nlm_procTxSendASyncMsgResp(nlmReqp, 
									error,
									cookie, 
									cookieLen);
	return (rc);

}

/*-----------------------------------------------------------------
 * Name:		nlm_procRxUnlockMsg	
 *
 * Description:
 * Process received NLM_UNLOCK_MSG request from client.
 * This procedure unlocks a locked region of a file. 
 *
 * Created by:  Joseph Vu
 *
 * Date Created:        7/18/02
 *
 *-----------------------------------------------------------------
 */
int32
nlm_procRxUnlockMsg(nlm_request_t    *nlmReqp)
{
	fs_Handle       	fileHandle;	
	uint32				sendResp;
	int32				rc;
	int32				error;
	int32				length;
	ushort16			cookieLen;
	char				*nlm_msgp;
	nfp_fileCBPtr_t		fileCBp;	
	char				cookie[NLM_MAX_NAME];
	static char			callerName[NLM_MAX_NAME];	
	static char			oh[NLM_MAX_NAME];	
	static char			buf[sizeof(nlm_lockInfo_t)];
	nlm_lockInfo_t		*nlm_lockInfop;
	nfp_lockCB_t		*unlockCBp;
#ifdef NOT_USED
	int32				overlap;
	int32				unlockType;
	short16				searchMask;
	nlm_lockInfo_t		*pendingLockInfop;
	nfp_lockCB_t		*pendingLockCBp;
	nlm_lockRec_t		*unlockRecp, *conflictLockRecp;
	nlm_lockRec_t		*pendingLockRecp, *nextLockRecp;
	nlm_lockRec_t		*unlockRecNextp;
	nlm_lockInfo_t 		*unlockAlockp;
	nfp_requestCBPtr_t	new_reqCBp;
	nlm_request_t		*new_nlmReqp;
#else
    uint32              ret;
#endif
	nfp_clientCBPtr_t	clientCBp;


	/*-----------------------
	 *Initializes values
  	 *-----------------------
  	 */
	rc = NFX_OK;
	error = NLM_FAILED;
	sendResp = 0;	
	clientCBp = nlmReqp->clientCBp;
	nlm_msgp = nlmReqp->procStart;
	unlockCBp = NULL;
#ifdef NOT_USED
    pendingLockRecp = NULL;
#endif

	/*-----------------------
	 *Get client's cookie and
	 *alock info
  	 *-----------------------
  	 */

	length = nfs_extractString(	nlm_msgp, 
								&cookie[0],
								&cookieLen, 
								NLM_MAX_NAME);		
	if (length == NFX_ERR)
	{
		nlm_globalCfgp->nlm_debugCounters.numOfRxBadCookies++;
		return (NFX_ERR);
	}
	cookie[cookieLen]  = '\0';
	nlm_msgp += length;

	/*Get alock info*/
	nlm_lockInfop = (nlm_lockInfo_t *)buf;
	nlm_lockInfop->nlm_callerName = &callerName[0];
	nlm_lockInfop->nlm_oh = &oh[0];
	length = nlm_extractAlockInfo(	nlm_msgp, 
									nlm_lockInfop,
									nlmReqp->rpcCallHdrp->progVer,
									&fileHandle); 
	if (length == NFX_ERR)
	{
		nlm_globalCfgp->nlm_debugCounters.numOfRxBadAlockObj++;
		goto nlm_procRxUnlockMsgExit;
	}

	fileCBp = nfp_getFileCB (&fileHandle);
	if (fileCBp == NULL)
	{
		/*No lock on this file, return response to client*/
		error = NLM_STAT_GRANTED;
		goto nlm_procRxUnlockMsgExit;
	}
    nlmReqp->fileCBp = fileCBp;
    nlmReqp->lockInfop = nlm_lockInfop;

#ifdef NOT_USED
	/*----------------------------
	 *Check our current lock list
	 *for this unlock request
	 *on this file
  	 *----------------------------
  	 */

	NLM_GET_L_REGION_END(nlm_lockInfop->lockRec.nlm_l_end, 
						nlm_lockInfop->lockRec.nlm_l_offset, 
						nlm_lockInfop->lockRec.nlm_l_len,
						nlm_lockInfop->lockRec.nlm_l_eof);			


	searchMask = NLM_SEARCH_OH | NLM_SEARCH_SVID;

	unlockRecp = nlm_getLockRecInCurrList(	fileCBp,
											nlm_lockInfop,
											searchMask,
											&nextLockRecp);
	if (unlockRecp == NULL)
	{
		/*----------------------------
		 *No matching lock found, 
		 *return response to client
		 *----------------------------
		 */
		error = NLM_STAT_GRANTED;
		goto nlm_procRxUnlockMsgExit;
	}
	unlockCBp = (nfp_lockCB_t	*)unlockRecp->lockCBp;

	if (nfp_validateLockCB(unlockCBp) != NFX_OK)
	{
#ifdef	NLM_VALIDATE_LOCK
		panic("%s: Bad unlockCBp: %p\n", 
						__FUNCTION__, unlockCBp);		
#endif
		error = NLM_STAT_GRANTED;
		goto nlm_procRxUnlockMsgExit;
	}


	unlockAlockp = &unlockCBp->protocol.nfp_nlmLock;

#ifdef	NLM_DEBUG	
	printf("%s: unlockCBp: %p svid: %d l_offset: %lld l_len: %lld exl: %d state: %d\n",
		__FUNCTION__, 
		unlockCBp,
		nlm_lockInfop->nlm_svid, 
		nlm_lockInfop->lockRec.nlm_l_offset, 
		nlm_lockInfop->lockRec.nlm_l_len,
		nlm_lockInfop->lockRec.nlm_exclusive, 
		nlm_lockInfop->lockRec.nlm_state);	
	printf("%s: callerNameLen: %d CallerName: %s\n",
		__FUNCTION__,nlm_lockInfop->nlm_callerNameLen, 
		nlm_lockInfop->nlm_callerName);	
#endif


	DLL_ITERATE_NEXT(unlockRecp, unlockRecNextp,
					&unlockCBp->lockOwnerList.list,
					lockList)
	{
		if ((unlockRecp == NULL) || (unlockRecp == DLL_NIL) )
			break;

		if (nlm_validateLockRec(unlockRecp) != NFX_OK)
		{
			DLL_REMOVE(&unlockCBp->lockOwnerList.list,
						unlockRecp, lockList);
			unlockCBp->lockOwnerList.numOfEntries--;
			nlm_deallocLockRec(unlockRecp);
		}


		if (nlm_regionCmp(unlockRecp, &nlm_lockInfop->lockRec))
			/*skip if region not overlap*/
			continue;	

		/*----------------------------
		 *Found overlap region to unlock
  	 	 *----------------------------
  	 	 */
		rc = nlm_checkUnlockedRegions(	unlockRecp,
										nlm_lockInfop,
										nlmReqp,
										&overlap,
										&unlockType);
		if (rc != NFX_OK)
			goto nlm_procRxUnlockMsgExit;
		
		/*-------------------------------------------
		 *Check if locked region has been splitted
		 *by this unlock
		 *--------------------------------------------
		 */
		pendingLockRecp = unlockRecp->pendingLockRecp;
		if (unlockType != NLM_UNLOCK_WHOLE_REGION)
		{
			if (!overlap)
			{
				/*still conflict, return*/
				error = NLM_STAT_GRANTED;
				goto nlm_procRxUnlockMsgExit;
			}
			continue;
		}
		
		/*---------------------------------------
		 *Remove Unlock Record from current list	
	 	 *---------------------------------------
	 	 */
		DLL_REMOVE(&unlockCBp->lockOwnerList.list,
					unlockRecp, lockList);
		nlm_deallocLockRec(unlockRecp);
		unlockCBp->lockOwnerList.numOfEntries--;

		if (((nlm_lockInfop->lockRec.nlm_l_offset == 0) && 
			(nlm_lockInfop->lockRec.nlm_l_len == 0) ) )
		{
			/*------------------------------------
			 *unlock the whole file.  
			 *See if any more any more overlapped
			 *lock regions
	 	 	 *-----------------------------------
	 	 	 */
			continue;
		}

		break;
	}	

	/*----------------------------------------------
	 *Lock found.
	 *Return LCK_GRANTED resp to the Unlock request
	 *----------------------------------------------
	 */
	if (!sendResp)
	{
		rc = nlm_procTxSendASyncMsgResp(nlmReqp, 
										NLM_STAT_GRANTED,
										cookie, 
										cookieLen);
		if (rc != NFX_OK)
		{
			/*----------------------------
			 *Cannot send response,
			 *Just drop Rx pkt and let client rexmit
	 		 *----------------------------
	 		 */
			nlm_globalCfgp->nlm_debugCounters.nlm_sendUnlockReplyErr++;
			return (NFX_ERR);
		}
		sendResp = 1;
	}

	/*-------------------------------------------------
	 *Check if previous unlock Control block 
	 *ownerlist and pending list to see if they're empty.
	 *------------------------------------------------
	 */
	if ( unlockCBp &&
		(unlockCBp->lockOwnerList.numOfEntries <= 0) &&
		(unlockCBp->lockOwnerPendingList.numOfEntries <= 0 ) )
	{
		nlm_rmLockCBFromList(unlockCBp,
							TRUE);
		if (fileCBp && (fileCBp->nfp_lockList.nfp_reqCnt <= 0))
		{
			nfp_delFileCB (&fileCBp->nfp_fileHandle);
		}	 
	}

	
	/*--------------------------------------------
	 *Check for any pending or block request
	 *that matches the lock that we just freed.
	 *If there's pending lock, we need to notify
	 *the the block client
 	 *--------------------------------------------
 	 */
	if (pendingLockRecp == NULL)
	{
		/*------------------------------------
		 *Return if no pending lock 
 	 	 *-----------------------------------
 	 	 */
		goto nlm_procRxUnlockMsgExit;
	}

	if (nlm_validateLockRec(pendingLockRecp) != NFX_OK)
		panic ("%s 2: Invalid pendingLockRecp %p\n",
				   	__FUNCTION__,pendingLockRecp);


	if (pendingLockRecp->waitForLockRecp != unlockRecp)
	{
		nlm_globalCfgp->nlm_debugCounters.numOfCallBackErr++;
		return (NFX_ERR);
	}

	/*------------------------------------------------------
	 *Check if pending (block) lock is still in conflict 
	 *with any current active lock(s) on this file
  	 *-----------------------------------------------------
  	 */

	searchMask = 0;
	if (!pendingLockRecp->nlm_exclusive)	
		searchMask = NLM_SEARCH_EXCLUSIVE;

	pendingLockCBp = (nfp_lockCBPtr_t)pendingLockRecp->lockCBp;

	if (nfp_validateLockCB(pendingLockCBp) != NFX_OK)
	{
		panic("%s 3: Bad pendingLockCBp: %p\n", 
						__FUNCTION__, pendingLockCBp);		
	}


	pendingLockInfop = &pendingLockCBp->protocol.nfp_nlmLock;
	pendingLockInfop->lockRec.nlm_l_end = pendingLockRecp->nlm_l_end;
	pendingLockInfop->lockRec.nlm_l_offset = pendingLockRecp->nlm_l_offset;
	pendingLockInfop->lockRec.nlm_l_len = pendingLockRecp->nlm_l_len;
	pendingLockInfop->lockRec.nlm_l_eof = pendingLockRecp->nlm_l_eof;

	conflictLockRecp = nlm_getLockRecInCurrList(fileCBp,
												pendingLockInfop,
												searchMask,
												&nextLockRecp);

	if (conflictLockRecp == NULL)
	{
		/*------------------------------------------
		 *Remove pending lock record from pending list	
	 	 *------------------------------------------
	 	 */
		DLL_REMOVE(&pendingLockCBp->lockOwnerPendingList.list,
					pendingLockRecp, lockList);
		pendingLockCBp->lockOwnerPendingList.numOfEntries--;

		/*---------------------------------------------
		 *Add the pending lock to the active lock list	
  	 	 *---------------------------------------------
  	 	 */
		if (nextLockRecp ==  NULL)
		{
			DLL_INSERT_TAIL(&pendingLockCBp->lockOwnerList.list,
							pendingLockRecp, lockList);
		}
		else
		{
			DLL_INSERT_BEFORE(&pendingLockCBp->lockOwnerList.list,
							nextLockRecp, pendingLockRecp, lockList);
		}


		pendingLockCBp->lockOwnerList.numOfEntries++;

	
		/*------------------------------------
		 *call back to block client
		 *Send a Portmapper GetPort to the client to
		 *find the client NLM port 
  	 	 *-----------------------------------
  	 	 */

		new_reqCBp = nfp_allocReqCB ();
		if (new_reqCBp == NULL)
			return (NFX_ERR);
		nlm_globalCfgp->nlm_debugCounters.nlm_allocReqCB++;
		new_nlmReqp = &new_reqCBp->protocol.nlm_req;
		new_nlmReqp->reqCBp = new_reqCBp;
		new_nlmReqp->tplConnp = NULL;
		new_nlmReqp->rxPktDescp = NULL;
		new_nlmReqp->txPktDescp = NULL;
		new_nlmReqp->clientCBp = clientCBp;
		new_nlmReqp->fileCBp =fileCBp;
		new_nlmReqp->lockRecp = pendingLockRecp;
		memcpy(&new_nlmReqp->rpcHdr, 
				nlmReqp->rpcCallHdrp, 
				sizeof(rpcCallHdr_t));
		new_nlmReqp->rpcCallHdrp = &new_nlmReqp->rpcHdr;
		new_nlmReqp->nlm_txCmdInfo.laddr = 
			((pkt_desc_t *)nlmReqp->rxPktDescp)->laddr;
		new_nlmReqp->nlm_txCmdInfo.raddr = 
			((nfp_clientCBPtr_t) pendingLockCBp->nfp_clientCbp)->nfp_ipAddr;
		new_nlmReqp->procNum = NLM_GRANTED_MSG;

		if (pendingLockCBp->nfp_clientCbp &&
			!(pendingLockCBp->nfp_clientCbp->nfp_nlmClientInfo.nlmPort))
		{
			rc = nlm_getportNlm(new_nlmReqp, pendingLockRecp);
			if (rc != NFX_OK)
			{
				return (NFX_ERR);
			}

		}
		else
		{
			//Send NLM_GRANTED_MSG
			rc = nlm_procTxAsyncGrantedMsg(	new_nlmReqp);
		}
		return (NFX_OK);
	}
	else
	{
		rc = nlm_setPendingLockEntry(	conflictLockRecp,
										pendingLockRecp);
		 
		return (rc);	
	
	}

	return (NFX_OK);
#else // NOT_USED
    /*
     * Check our current lock list for this unlock request on this file
     */
    NLM_GET_L_REGION_END(nlm_lockInfop->nlm_l_end,
                         nlm_lockInfop->nlm_l_offset,
                         nlm_lockInfop->nlm_l_len,
                         nlm_lockInfop->nlm_l_eof);
    /*
     * Send unlock req, to the lock module.
     * If there are pending locks that now become granted, we will get a
     * callback.
     */
    ret = nlm_procUnlockReq(nlm_lockInfop, nlmReqp);
    if (ret == NFX_ERR)
    {
        /*
         * Some internal error.
         */
        error = NLM_FAILED;
    }
    else
    {
        /*
         * Successfully removed the lock
         * If not matching lock was found, it would be treated as a duplicate
         * message and returned OK.
         */
        error = NLM_STAT_GRANTED;
    }
    if (lock_areThereCurrentFileLocksReq(fileCBp->fileLockIdx) == FALSE)
    {
        nfp_deallocFileCB(fileCBp);
        nfp_delFileCB(&fileHandle);
    }
#endif // NOT_USED


nlm_procRxUnlockMsgExit:
	if (!sendResp)
	{
//		nlm_globalCfgp->nlm_debugCounters.nlm_rcvUnlockErr++;
		rc = nlm_procTxSendASyncMsgResp(nlmReqp, 
										error,
										cookie, 
										cookieLen);
	}

#ifdef NOT_USED
	if ( unlockCBp &&
		(unlockCBp->lockOwnerList.numOfEntries <= 0) &&
		(unlockCBp->lockOwnerPendingList.numOfEntries <= 0 ) )
	{
		nlm_rmLockCBFromList(unlockCBp,
							TRUE);
		if (fileCBp && (fileCBp->nfp_lockList.nfp_reqCnt <= 0))
		{
			nfp_delFileCB (&fileCBp->nfp_fileHandle);
		}	 

	}
#endif
	return (rc);
}

/*-----------------------------------------------------------------
 * Name:		nlm_procRxGrantedMsg	
 *
 * Description:
 *	 Process received NLM_GRANTED_MSG request from client.
 *
 * Created by:  Joseph Vu
 *
 * Date Created:        7/18/02
 *
 *-----------------------------------------------------------------
 */
int32
nlm_procRxGrantedMsg(nlm_request_t    *nlmReqp)
{


	return (NFX_OK);

}

/*-----------------------------------------------------------------
 * Name:		nlm_procRxTestRes	
 *
 * Description:
 *	 Process received NLM_TEST_RES request from client.
 *
 * Created by:  Joseph Vu
 *
 * Date Created:        7/18/02
 *
 *-----------------------------------------------------------------
 */
int32
nlm_procRxTestRes(nlm_request_t    *nlmReqp)
{


	return (NFX_OK);

}

/*-----------------------------------------------------------------
 * Name:		nlm_procRxLockRes	
 *
 * Description:
 *	 Process received NLM_LOCK_RES request from client.
 *
 * Created by:  Joseph Vu
 *
 * Date Created:        7/18/02
 *
 *-----------------------------------------------------------------
 */
int32
nlm_procRxLockRes(nlm_request_t    *nlmReqp)
{


	return (NFX_OK);
}

/*-----------------------------------------------------------------
 * Name:	nlm_procRxCancelRes	
 *
 * Description:
 *	 Process received NLM_CANCEL_RES request from client.
 *
 * Created by:  Joseph Vu
 *
 * Date Created:        7/18/02
 *
 *-----------------------------------------------------------------
 */
int32
nlm_procRxCancelRes(nlm_request_t    *nlmReqp)
{


	return (NFX_OK);
}

/*-----------------------------------------------------------------
 * Name:	nlm_procRxUnlockRes	
 *
 * Description:
 *	 Process received NLM_UNLOCK_RES request from client.
 *
 * Created by:  Joseph Vu
 *
 * Date Created:        7/18/02
 *
 *-----------------------------------------------------------------
 */
int32
nlm_procRxUnlockRes(nlm_request_t    *nlmReqp)
{


	return (NFX_OK);
}

/*-----------------------------------------------------------------
 * Name:	nlm_procRxGrantedRes	
 *
 * Description:
 *	 Process received NLM_GRANTED_RES request from client.
 *
 * Created by:  Joseph Vu
 *
 * Date Created:        7/18/02
 *
 *-----------------------------------------------------------------
 */
int32
nlm_procRxGrantedRes(nlm_request_t    *nlmReqp)
{
	pkt_desc_t *descp;

	descp = nlmReqp->rxPktDescp;
	if ( descp && (descp->tplProt == TPL_PROT_UDP) &&
			descp->pTplConn)
	{
		tpl_delConnReq(descp->pTplConn);
	}
	nlmReqp->reqState = NLM_REQ_DONE;
	nlm_freeAll(nlmReqp, TRUE);

	return (NFX_OK);
}


#endif	//NFP_TXRX
