/* vi:ts=4
 *-----------------------------------------------------------------
 *
 * Name:        nfs-procTx.c
 *
 * RCS:  $Id: nlm-procTx.c,v 1.8 2002/11/06 03:35:39 joseph Exp $
 *
 * Copyright (C) 2001, Agile Storage Inc.
 *
 * Description:
 *      Contain NLM transmit (XMT) functions.
 *
 * Created by: Joseph Vu
 *
 * Date Created:        7/05/02
 *
 *---------------------------------------------------------------
 */

#include <nfx-incl.h>
#include <exports/pkt-api.h>
#include <exports/vs-mem-api.h>
#include <exports/lock-api.h>
#include <exports/nfs-api.h>
#include "nlm-prototypes.h"
#include    <exports/bsdsock-api.h>
#include "../sm-tpl/tpl.h"

#include <sys/machine-dep.h>

//#define	NLM_DEBUG
#define	NLM_DBG_PRINTF		if (nlm_traceEnable) printf
#define NLM_DBG_FUNC_PRINTF if (nlm_traceDebugEnable) printf

/*
 *Externs
 */
#ifdef	NOTUSED
pkt_desc_t *nlm_tplAllocPkt(
	nfp_clientCBPtr_t	clientCBp, 
	nlm_request_t 		*nlmReqp,	
	int32				usrDataLen);
#endif	//NOTUSED

extern int32 luc_simCamFind(int32	proto,
						uint32      laddr,
						uint32      raddr,
						ushort16    lport,
						ushort16    rport );

extern void *luc_camGetUsrInfo( int32 idx);

#define		NLM_ASYNC_RESP_PROC_MIN		0
#define		NLM_ASYNC_RESP_PROC_MAX		4

static uint32 nlm_asyncProcResType[5] =
{
        NLM_TEST_RES,				/*0: NLM_TEST_RES*/
        NLM_LOCK_RES,				/*1: NLM_LOCK_RES*/
        NLM_CANCEL_RES,				/*2: NLM_CANCEL_RES*/
        NLM_UNLOCK_RES,				/*3: NLM_UNLOCK_RES*/
        NLM_GRANTED_RES,			/*4: NLM_GRANTED_RES*/
};


/*-----------------------------------------------------------------
 * Name:	nlm_procTxPkt  
 *
 * Description:
 *      Transmit a NLM reply pkt to the client.
 *
 * Created by:  Joseph Vu
 *
 * Date Created:        7/05/02
 *
 *-----------------------------------------------------------------
 */

int32
nlm_procTxPkt (
	pkt_desc_t *tplXmitPktDescp, 
	nlm_request_t *nlmReqp
)
{
	int32 				ret;

	if ( (tplXmitPktDescp == NULL) || (nlmReqp == NULL ) )
	{
		return (NFX_ERR);
	}

	//Save NLM req in appInfo for callback complete Ind.
	
	tplXmitPktDescp->appInfo = nlmReqp;
	nlmReqp->txPktDescp = (void *)tplXmitPktDescp;

	nlmReqp->reqState = NLM_REQ_REPLIED;

	if (nlm_traceEnable != 0)
		nlm_xmtDecodePkt (nlmReqp);

	//Increment Tx stat counter
	
	nlm_globalCfgp->nlm_txPktCounters[nlmReqp->rpcCallHdrp->rpc_proc] += 1;
	if (nlmReqp->reqState != NLM_REQ_DONE)
		xdr_swap(tplXmitPktDescp->pHdr, tplXmitPktDescp->hdrSize, 1);
	
	nlm_globalCfgp->nlm_debugCounters.nlm_tplXmtPktSend++;

	ret = tpl_xmtPkt (nlmReqp->tplConnp, tplXmitPktDescp);

	return (ret);

}

/*
 *-------------------------------------------------------------------
 * Function Name:  nlm_procTxSendResp    	
 *
 * Description:
 *	This function is call to send a response to a client's request. 
 *
 * Created by:          Joseph Vu
 *
 * Created:             7/15/2002
 *
 *-------------------------------------------------------------------
 */
int32
nlm_procTxSendResp(
	nlm_request_t *nlmReqp,
	int32 status,
	char *nlm_cookiep,
	short16 nlm_cookieLen
)
{
	rpcCallHdr_t		*rpcCallp;
	pkt_desc_t 			*tplXmitPktDescp;
	nfp_clientCBPtr_t	clientCBp;
	char				*txMsgp;
	int32				txMsgLen;
	int32				length;

	if ( (nlmReqp == NULL) ||
		(nlmReqp->rpcCallHdrp == NULL ) )
		return (NFX_ERR);

	/*-------------------------------------------
	 *validate input procedure and input status
 	 *--------------------------------------------
 	 */
	rpcCallp = nlmReqp->rpcCallHdrp;
	if ( (rpcCallp->rpc_proc < NLM_NULL ) || 
		(rpcCallp->rpc_proc > NLM_FREE_ALL )  || 
		(status < NLM_STAT_GRANTED) ||
		(status > NLM_FAILED ) )
		return (NFX_ERR);

	clientCBp = nlmReqp->clientCBp;
	if (nfp_validateClientCB(clientCBp) != NFX_OK)
	{
		return (NFX_ERR);
	}

	/*-------------------------------------------
	 *Alloc TPL packet to send
	 *Increment stat.
 	 *--------------------------------------------
 	 */
	
	tplXmitPktDescp = TPL_ALLOC_PACKET(nlmReqp->tplConnp, 0);
	if (tplXmitPktDescp == NULL)
	{
		nlm_globalCfgp->nlm_debugCounters.nlm_tplXmtNoPktResource++;
		return (NFX_ERR);	
	}
	nlm_globalCfgp->nlm_debugCounters.nlm_tplXmtPktAlloc++;
	nlm_globalCfgp->nlm_statCounters[status] += 1;

	nlmReqp->txPktDescp = (void *)tplXmitPktDescp;
	txMsgp = (char *)tplXmitPktDescp->pHdr;
	txMsgLen = 0;

	/*-------------------------------------------
	 *Start with RPC reply header
	 *cookie opague value &
	 *update msgp & msg length
 	 *--------------------------------------------
 	 */
	txMsgLen += nfs_msgRpcReply(txMsgp, rpcCallp->rpc_xid,
			RPC_REPLY_MSG_ACCEPTED, RPC_REPLY_SUCCESS);
	txMsgp += txMsgLen;

	length = nfs_storeString(txMsgp, nlm_cookiep,
							nlm_cookieLen, NLM_MAX_NAME);
	
	txMsgp += length;
	txMsgLen += length;

	/*-------------------------------------------
	 *Set status & update msg ptr.
 	 *--------------------------------------------
 	 */
	*(uint32 *)txMsgp = status;
	txMsgp += 4;
	txMsgLen += 4;

	if ( (status == NLM_ERR_DENIED) &&
		(rpcCallp->rpc_proc == NLM_TEST) &&
#ifdef NOT_USED
	    (nlmReqp->lockRecp)
#else
	    (nlmReqp->lockInfop)
#endif
       )
	{
		/*-------------------------------------------
		 *If Test procedure & status is DENIED, 
		 *return current lock holder
		 *--------------------------------------------
		 */
		length = nlm_storeAlockInfo(txMsgp, 
#ifdef NOT_USED
									(nlm_lockRec_t *)nlmReqp->lockRecp,
#else
									(nlm_lockInfo_t *)nlmReqp->lockInfop,
#endif
			   						rpcCallp->progVer);

		txMsgp += length;
		txMsgLen += length;

	}

	/*-------------------------
	 *Set packet desc hdr size,
	 *and send pkt
	 *---------------------------
	 */
	tplXmitPktDescp->hdrSize = txMsgLen;

	if (nlm_procTxPkt(tplXmitPktDescp, nlmReqp) != NFX_OK)
	{
		return (NFX_ERR);
	}

	return (NFX_OK);
}

/*
 *-------------------------------------------------------------------
 * Function Name: nlm_procTxSendErrResp	 
 *
 * Description:
 *	This function is call to send an error response 
 *  to a client's request. 
 *
 * Created by:          Joseph Vu
 *
 * Created:             7/15/2002
 *
 *-------------------------------------------------------------------
 */
int32
nlm_procTxSendErrResp(nlm_request_t *nlmReqp,	
					int32 status,		
					char *procStartp		
)
{
	char		*nlm_msgp;
	char		nlm_cookie[NLM_MAX_NAME];
	ushort16	nlm_cookieLen;
	int32		length;

	nlm_msgp = procStartp;

	//Get client's cookie 
	
	length = nfs_extractString(nlm_msgp, &nlm_cookie[0],
			&nlm_cookieLen, NLM_MAX_NAME);		
	if (length == NFX_ERR)
	{
		printf("nlm_procRxTestReq: Failure to decode cookie Obj!\n");
		return (NFX_ERR);
	}
	nlm_cookie[nlm_cookieLen]  = '\0';		//NULL terminated

	return (nlm_procTxSendResp(
			nlmReqp, status, nlm_cookie, nlm_cookieLen));

}

/*
 *-----------------------------------------------------------------
 * Function Name:	 nlm_procTxGetportReq
 *
 * Description:
 *	Sends a Getport command to client to
 *  get the NLM port.
 * 
 * Created by:          Joseph Vu
 *
 * Created:             7/30/2002
 *
 *
 *-----------------------------------------------------------------
 */
int32
nlm_procTxGetportReq(nlm_request_t *nlmReqp)
{
	int32					rc;
	nfp_clientCBPtr_t		clientCBp;
	pkt_desc_t				*descp;
	char					*txMsgp;
	int32					txMsgLen;
	int32					camIdx;
	tpl_connCb_t			*pConn;
	boolean					fastPath;

	clientCBp = nlmReqp->clientCBp;
	if (clientCBp == NULL)
	{
		return (NFX_ERR);
	}
	txMsgLen = 0;
	fastPath = FALSE;
	pConn = NULL;

	/*------------------------------
	 *Check if this is a new socket
	 *------------------------------
	 */	
	camIdx = luc_simCamFind(TPL_PROT_UDP,	
			nlmReqp->nlm_txCmdInfo.laddr, 			
			nlmReqp->nlm_txCmdInfo.raddr,
			htons(NLM_PORT + 1),			
			htons(nlmReqp->nlm_txCmdInfo.rport));

	if (camIdx >= 0)
	{
		pConn = (tpl_connCb_t *) luc_camGetUsrInfo( camIdx );
		if (pConn)
		{
			/*------------------------------
			 *if socket exist,
			 *	Send packet via fastpath
			 *-------------------------------
			 */
			fastPath = TRUE;
		}
	}

	if (fastPath == FALSE)
	{
		descp = PKT_ALLOC_PACKET(1);
	}
	else
	{
		descp = TPL_ALLOC_PACKET(pConn, 0);
		nlm_globalCfgp->nlm_debugCounters.nlm_tplXmtPktAlloc++;
	}

	if (!descp)
	{
		nlm_globalCfgp->nlm_debugCounters.nlm_xmtNoRsrcErr++;
		return (NFX_ERR);
	}

	nlm_globalCfgp->nlm_debugCounters.xmtGetPortReqCnt++;
	txMsgp = (char *)descp->pHdr;

	/*-----------------------------------
	 *Set up portmapper RPC header 
	 *-----------------------------------
	 */
	if (!nlmReqp->nlm_txCmdInfo.cmdXid)	
		nlmReqp->nlm_txCmdInfo.cmdXid = nlm_getNextCmdXid();

	txMsgLen +=  nlm_msgRpcCallHdr(	txMsgp,
									nlmReqp->nlm_txCmdInfo.cmdXid,
									RPC_PORTMAP_PROG_NUM,
									RPC_PORTMAPPER_VER2,
									RPC_PORTMAPPER_GETPORT);	
	
	txMsgp += txMsgLen;

	/*-----------------------------------
	 *RPC NLM Program Num 
	 *-----------------------------------
	 */
	
	*(uint32 *)txMsgp = RPC_NLM_PROG_NUM;
	txMsgp += 4;
	txMsgLen += 4;

	/*-----------------------------------
	 *RPC NLM Program Ver,
	 *Transport protocol 
	 *-----------------------------------
	 */
	
	*(uint32 *)txMsgp = NLM_VER4;
	txMsgp += 4;
	txMsgLen += 4;

	*(uint32 *)txMsgp = TPL_PROT_UDP;
	txMsgp += 4;
	txMsgLen += 4;

	*(uint32 *)txMsgp = 0;
	txMsgp += 4;
	txMsgLen += 4;

	/*-----------------------------------
	 *Increment retry count,
	 *Initializes max time to wait count,
	 *Set up Pkt Decp header size
	 *and send packet
	 *-----------------------------------
	 */
	
	nlmReqp->nlm_txCmdInfo.getportRetry++;
	nlmReqp->nlm_txCmdInfo.getportMaxTimeWaitCnt = 0;
	nlmReqp->reqState = NLM_GETPORT_CALL_SND;

	descp->hdrSize = txMsgLen;
	xdr_swap(descp->pHdr, descp->hdrSize, 1);


	if (fastPath == FALSE)
	{
#if defined(NFX_SMP)
		rc = tpl_xmtPktSlow(nlmReqp->nlm_txCmdInfo.raddr,
							htons(nlmReqp->nlm_txCmdInfo.rport),
							descp,
							nlm_globalCfgp->pmap_pBsdSockUdp,
							0);
		
		/* pkt is freed on NCPU */
#else
		rc = bsd_SendSock(
				nlm_globalCfgp->pmap_pBsdSockUdp,
				nlmReqp->nlm_txCmdInfo.raddr,
				htons(nlmReqp->nlm_txCmdInfo.rport),
				descp->pHdr, 
				BSD_BUF_FLAT, 
				descp->hdrSize, 
				0);
		PKT_FREE_PACKET(descp);
#endif /* NFX_SMP */

	}
	else
	{
		void					**pAppHdl;

		/*Send fastpath packet*/
		descp->appInfo = nlmReqp;
		nlmReqp->txPktDescp = (void *)descp;

		pAppHdl = &pConn->pAppHdl;			
		if (*pAppHdl != clientCBp)
		{
			/*-------------------------------------------
			 *update TPL Handle since our handle
			 *has changed for callback via nlm_pmapTplrcvInd()
			 *-------------------------------------------------
			 */
			*pAppHdl = clientCBp;
		}

        rc = tpl_xmtPkt (pConn, descp);
		nlm_globalCfgp->nlm_debugCounters.nlm_tplXmtPktSend++;
	}


	if (rc != NFX_OK)
	{
		nlmReqp->nlm_txCmdInfo.getportRetry = 0;
		nlmReqp->nlm_txCmdInfo.getportMaxTimeWaitCnt = 0;
		nlmReqp->reqState = NLM_REQ_DONE;
		nlm_delReqCBFromList(nlmReqp);
		nlm_freeAll(nlmReqp, TRUE);
		nlm_globalCfgp->nlm_debugCounters.txGetportErr++;
	}

	return (NFX_OK);
}

/*
 *-----------------------------------------------------------------
 * Function Name:	 nlm_procTxGranted
 *
 * Description:
 *	Sends a NLM_GRANTED to the client to indicates
 *  that a blocked LOCK request has been granted.
 * 
 * Created by:          Joseph Vu
 *
 * Created:             7/30/2002
 *
 *
 *-----------------------------------------------------------------
 */
int32
nlm_procTxGranted(nlm_request_t *nlmReqp)
{
	nfp_fileCBPtr_t		fileCBp;
	nfp_clientCBPtr_t	clientCBp;
	nlm_lockInfo_t		*nlm_lockInfop;
#ifdef NOT_USED
	nfp_lockCB_t        *pending_lockCBp;
	nlm_lockRec_t		*pending_lockRecp;
#else
	nlm_lockInfo_t		*pend_lockInfop;
#endif
	pkt_desc_t			*descp;
	char				*txMsgp;
	int32				txMsgLen;
	int32				nfs_version;
	uint32				cookieLen;
	int32				length;
	int32				rc;
	int32				camIdx;
	tpl_connCb_t		*pConn;
	boolean				fastPath;
	
	rc = NFX_OK;

#ifdef NOT_USED
	pending_lockRecp = (nlm_lockRec_t *)nlmReqp->lockRecp;

	if (nlm_validateLockRec(pending_lockRecp) != NFX_OK)
	{
#ifdef	NLM_VALIDATE_LOCK
		panic ("%s: Invalid pending_lockRecp %p\n",
						   	__FUNCTION__,pending_lockRecp);
#endif
		rc = NFX_ERR;
		goto nlm_procTxGrantedErr;
	}
	pending_lockCBp = (nfp_lockCB_t  *)pending_lockRecp->lockCBp;

	if (nfp_validateLockCB(pending_lockCBp) != NFX_OK)
	{
#ifdef	NLM_VALIDATE_LOCK
		panic("%s: Bad pending_lockCBp: %p\n",
					   	__FUNCTION__, pending_lockCBp);	
#endif
		rc = NFX_ERR;
		goto nlm_procTxGrantedErr;
	}
#else // NOT_USED
	pend_lockInfop = (nlm_lockInfo_t *)nlmReqp->lockInfop;
#if 0
	if (nlm_validateLockRec(pend_lockInfop) != NFX_OK)
	{
#ifdef	NLM_VALIDATE_LOCK
		panic ("%s: Invalid pending_lockRecp %p\n",
						   	__FUNCTION__,pend_lockInfop);
#endif
		rc = NFX_ERR;
		goto nlm_procTxGrantedErr;
	}
#endif
#endif // NOT_USED

	txMsgLen = 0;
    fastPath = FALSE;
    pConn = NULL;
	fileCBp = nlmReqp->fileCBp;

	/*----------------------------------------
	 *Retrieve next command XID & Cookie
	 *----------------------------------------
	 */	
	nlmReqp->nlm_txCmdInfo.cmdXid = nlm_getNextCmdXid();	
	nlmReqp->nlm_txCmdInfo.cmdCookie = nlm_getNextCmdCookie();
	
	clientCBp = nlmReqp->clientCBp;

	//Map NLM ver to comparable
	//NFS version
	
	NLM_MAP_TO_COMP_NFS_VER(nfs_version, 
					nlmReqp->rpcCallHdrp->progVer);

	//Check if this is a new socket
	//
	camIdx = luc_simCamFind(TPL_PROT_UDP,		
		nlmReqp->nlm_txCmdInfo.laddr, 
		nlmReqp->nlm_txCmdInfo.raddr,
		htons(NLM_PORT),
		htons(clientCBp->nfp_nlmClientInfo.nlmPort));

	if (camIdx >= 0)
	{
		pConn = (tpl_connCb_t *) luc_camGetUsrInfo( camIdx );
		if (pConn)
		{
			//If socket exist, send packet via fastpath
			fastPath = TRUE;
		}
	}

	if (fastPath == FALSE)
	{
		descp = PKT_ALLOC_PACKET(1);
	}
	else
	{
		descp = TPL_ALLOC_PACKET(pConn, 0);
		nlm_globalCfgp->nlm_debugCounters.nlm_tplXmtPktAlloc++;
	}
	if (!descp)
	{
		nlm_globalCfgp->nlm_debugCounters.nlm_xmtNoRsrcErr++;
		return NFX_ERR;
	}
	nlm_globalCfgp->nlm_debugCounters.xmtGrantedReqCnt++;
	nlm_globalCfgp->nlm_txPktCounters[NLM_GRANTED] += 1;
	txMsgp = (char *)descp->pHdr;

	//save command command XID for matching 
	//client return reply 

	//Set up portmapper RPC header 
	txMsgLen +=  nlm_msgRpcCallHdr(txMsgp,
									nlmReqp->nlm_txCmdInfo.cmdXid,
									RPC_NLM_PROG_NUM,
									nlmReqp->rpcCallHdrp->progVer,
									NLM_GRANTED);		
	txMsgp += txMsgLen;

#ifdef NOT_USED
	nlm_lockInfop = &pending_lockCBp->protocol.nfp_nlmLock;
#else
	nlm_lockInfop = pend_lockInfop;
#endif

	//store NLM part
	//stores cookie	

	cookieLen = 4;
	length = nfs_storeLongValToBuf(txMsgp,
					cookieLen );
	txMsgp += length;
	txMsgLen += length;

	length = nfs_storeLongValToBuf(txMsgp,
					nlmReqp->nlm_txCmdInfo.cmdCookie);
	txMsgp += length;
	txMsgLen += length;

	//store exclusive field
#ifdef NOT_USED
	length = nfs_storeLongValToBuf(txMsgp,
				pending_lockRecp->nlm_exclusive );
#else
	length = nfs_storeLongValToBuf(txMsgp, nlm_lockInfop->nlm_exclusive );
#endif
	txMsgp += length;
	txMsgLen += length;

	//alock info, starts with callerName
	length = nfs_storeString(txMsgp, 
					nlm_lockInfop->nlm_callerName,
					nlm_lockInfop->nlm_callerNameLen, 
					NLM_MAX_NAME);

	//update msgp & msg length 
	txMsgp += length;
	txMsgLen += length;

	//stores filehandle
    length = nfs_storeV3Fhandle(&fileCBp->nfp_fileHandle, 
								txMsgp);

	//update msgp & msg length 
	txMsgp += length;
	txMsgLen += length;
	

	//stores oh

	length = nfs_storeString(txMsgp, 
							nlm_lockInfop->nlm_oh,
							nlm_lockInfop->nlm_ohLen, 
							NLM_MAX_NAME);

	//update msgp & msg length 
	txMsgp += length;
	txMsgLen += length;
	

	//stores svid
	
	length = nfs_storeLongValToBuf(	txMsgp,
									nlm_lockInfop->nlm_svid );
	//update msgp & msg length 
	txMsgp += length;
	txMsgLen += length;
	
	//stores l_offset and l_len
	
	if (nfs_version == NFS_VER2)
	{
		//32 bits l_offset
#ifdef NOT_USED
		length = nfs_storeLongValToBuf(txMsgp, 
									(uint32)pending_lockRecp->nlm_l_offset );
#else
		length = nfs_storeLongValToBuf(txMsgp, 
									(uint32)nlm_lockInfop->nlm_l_offset );
#endif
		txMsgp += length;
		txMsgLen += length;

		//32 bits l_len
#ifdef NOT_USED
		length = nfs_storeLongValToBuf(txMsgp, 
										(uint32 )pending_lockRecp->nlm_l_len );
#else
		length = nfs_storeLongValToBuf(txMsgp, 
										(uint32 )nlm_lockInfop->nlm_l_len );
#endif
		txMsgp += length;
		txMsgLen += length;
	}
	else
	{
		//NFS V3
		//64 bits l_offset
#ifdef NOT_USED
		length = nfs_storeLongLongValToBuf(	txMsgp,
                							pending_lockRecp->nlm_l_offset );
#else
		length = nfs_storeLongLongValToBuf(	txMsgp,
                							nlm_lockInfop->nlm_l_offset );
#endif
		txMsgp += length;
		txMsgLen += length;

		//64 bits l_len
#ifdef NOT_USED
		length = nfs_storeLongLongValToBuf(	txMsgp,
                							pending_lockRecp->nlm_l_len);
#else
		length = nfs_storeLongLongValToBuf(	txMsgp,
                							nlm_lockInfop->nlm_l_len);
#endif
		txMsgp += length;
		txMsgLen += length;
	}

	//Increment retry count
	//Initializes max time to wait count
	
	nlmReqp->nlm_txCmdInfo.grantedRetry++;	
	nlmReqp->nlm_txCmdInfo.grantedMaxTimeWaitCnt = 0;
	nlmReqp->reqState = NLM_GRANTED_CALL_SND;

	//Send NLM_GRANTED message.
	//Set up pktDesc header size
	descp->hdrSize = txMsgLen;

	xdr_swap(descp->pHdr, descp->hdrSize, 1);

	if (fastPath == FALSE)
	{
#if defined(NFX_SMP)
		rc = tpl_xmtPktSlow(clientCBp->nfp_ipAddr,
							htons(clientCBp->nfp_nlmClientInfo.nlmPort),
							descp, 
							nlm_globalCfgp->nlm_pBsdSockUdp,
							0);
		
		/* pkt is freed on NCPU */
#else
		rc = bsd_SendSock(
				nlm_globalCfgp->nlm_pBsdSockUdp,
				clientCBp->nfp_ipAddr,
				htons(clientCBp->nfp_nlmClientInfo.nlmPort),
				descp->pHdr, 
				BSD_BUF_FLAT, 
				descp->hdrSize, 
				0);

		PKT_FREE_PACKET(descp);
#endif /* NFX_SMP */
	}
	else
	{
		void					**pAppHdl;

		//Send fastpath packet
		descp->appInfo = nlmReqp;
		nlmReqp->txPktDescp = (void *)descp;

		pAppHdl = &pConn->pAppHdl;			
		if (*pAppHdl != clientCBp)
		{
			//update TPL Handle since our handle has changed
			*pAppHdl = clientCBp;
		}
        rc = tpl_xmtPkt (pConn, descp);
		nlm_globalCfgp->nlm_debugCounters.nlm_tplXmtPktSend++;
	}
	NLM_DBG_FUNC_PRINTF("%s: exit\n", __FUNCTION__);
#ifdef NOT_USED
nlm_procTxGrantedErr:
#endif

	if (rc != NFX_OK)
	{
		nlmReqp->nlm_txCmdInfo.grantedRetry = 0;
		nlmReqp->nlm_txCmdInfo.grantedMaxTimeWaitCnt = 0;
		nlmReqp->reqState = NLM_REQ_DONE;
		nlm_delReqCBFromList(nlmReqp);
		nlm_freeAll(nlmReqp, TRUE);
		nlm_globalCfgp->nlm_debugCounters.txGrantedErr++;
	}
	return (NFX_OK);

}

/*
 *-----------------------------------------------------------------
 * Function Name:	nlm_procTxSendASyncMsgResp 
 *
 * Description:
 *	Sends a NLM Asynchronous response message to client.
 * 
 * Created by:          Joseph Vu
 *
 * Created:             7/30/2002
 *
 *
 *-----------------------------------------------------------------
 */
int32
nlm_procTxSendASyncMsgResp(
	nlm_request_t *nlmReqp,
	int32 status,
	char *nlm_cookiep,
	short16 nlm_cookieLen
)
{
	tpl_connCb_t		*pConn;
	pkt_desc_t			*descp;
	char				*txMsgp;
	nfp_clientCBPtr_t	clientCBp;
	uint32				procIdx;
	uint32				resProcNum;
	int32				rc;
	int32				txMsgLen;
	int32				nfs_version;
	boolean				fastPath;
	int32				camIdx;
	int32				length;
	
	clientCBp = nlmReqp->clientCBp;
	if (!clientCBp->nfp_nlmClientInfo.nlmPort)
	{
		uchar8 			*allocBufp;

		/*--------------------------------------
		 *go get NLM client port
	 	 *--------------------------------------
	 	 */
		allocBufp = (uchar8 *)eee_ramAlloc(nlm_cookieLen + 1);
		if (allocBufp == NULL)
		{
			//Increment counter
			return (NFX_ERR);
		}
		memcpy(allocBufp, nlm_cookiep,
				nlm_cookieLen);
		nlmReqp->cookie.netobjLen = nlm_cookieLen;
		nlmReqp->cookie.netobj_p = allocBufp;
		nlm_globalCfgp->nlm_debugCounters.nlm_allocCookie++;
		nlmReqp->replyStatus = status;

		rc = nlm_getportNlm (nlmReqp, NULL);
		if (rc != NFX_OK)
		{
		
			return (NFX_ERR);
		}
		return (NFX_OK);
	}


	/*--------------------------------------
	 *Get response procedure number
	 *--------------------------------------
	 */
	procIdx = nlmReqp->procNum - NLM_TEST_MSG;
	if ( (procIdx < NLM_ASYNC_RESP_PROC_MIN ) || 
		(procIdx > NLM_ASYNC_RESP_PROC_MAX) )
	{
		//increment counter
		return NFX_ERR;
	}	
	resProcNum = 
		nlm_asyncProcResType[nlmReqp->procNum - NLM_TEST_MSG];



	txMsgLen = 0;
    fastPath = FALSE;
    pConn = NULL;
	

	/*--------------------------------------
	 *Map NLM ver to comparable
	 *NFS version
	 *--------------------------------------
	 */
	NLM_MAP_TO_COMP_NFS_VER(nfs_version, 
							nlmReqp->rpcCallHdrp->progVer);

	/*--------------------------------------
	 *Check if this is a new socket
	 *--------------------------------------
	 */
	camIdx = luc_simCamFind(TPL_PROT_UDP,		
		nlmReqp->nlm_txCmdInfo.laddr, 
		nlmReqp->nlm_txCmdInfo.raddr,
		htons(NLM_PORT),
		htons(clientCBp->nfp_nlmClientInfo.nlmPort));

	if (camIdx >= 0)
	{
		pConn = (tpl_connCb_t *) luc_camGetUsrInfo( camIdx );
		if (pConn)
		{
			fastPath = TRUE;
		}
	}

	if (fastPath == FALSE)
	{
		descp = PKT_ALLOC_PACKET(1);
	}
	else
	{
		descp = TPL_ALLOC_PACKET(pConn, 0);
		nlm_globalCfgp->nlm_debugCounters.nlm_tplXmtPktAlloc++;
	}
	if (!descp)
	{
		nlm_globalCfgp->nlm_debugCounters.nlm_xmtNoRsrcErr++;
		return NFX_ERR;
	}

	nlm_globalCfgp->nlm_txPktCounters[resProcNum] += 1;
	txMsgp = (char *)descp->pHdr;

	if (!nlmReqp->nlm_txCmdInfo.cmdXid)		
		nlmReqp->nlm_txCmdInfo.cmdXid =  nlm_getNextCmdXid();

	/*--------------------------------------
	 *Set up portmapper RPC header,
	 *stores response cookie,
	 *and nlm_stat status
	 *--------------------------------------
	 */
	txMsgLen +=  nlm_msgRpcCallHdr(	txMsgp,
									nlmReqp->nlm_txCmdInfo.cmdXid,
									RPC_NLM_PROG_NUM,
									nlmReqp->rpcCallHdrp->progVer,
									resProcNum);		
	txMsgp += txMsgLen;


	length = nfs_storeString(txMsgp, 
							nlm_cookiep,
							nlm_cookieLen,
							NLM_MAX_NAME);

	txMsgp += length;
	txMsgLen += length;

   
	*(uint32 *)txMsgp = status;

	/*--------------------------------------
	 *Update message ptr. & length
	 *--------------------------------------
	 */

	txMsgp += 4;
	txMsgLen += 4;

	if ( (status == NLM_ERR_DENIED) &&
		(nlmReqp->procNum == NLM_TEST_MSG) &&
#ifdef NOT_USED
	  	(nlmReqp->lockRecp)
#else
	  	(nlmReqp->lockInfop)
#endif
       )
	{
		/*--------------------------------------
	 	 *If TEST_MSG procedure & status is DENIED
	 	 *--------------------------------------
	 	 */
        length = nlm_storeAlockInfo(txMsgp,
#ifdef NOT_USED
									(nlm_lockRec_t *)nlmReqp->lockRecp,
#else
									(nlm_lockInfo_t *)nlmReqp->lockInfop,
#endif
                					nlmReqp->rpcCallHdrp->progVer);
        
        txMsgp += length;
        txMsgLen += length;
	}
	
	/*--------------------------------------
	 *Set up PktDescp hdr size
	 *and sent pkt 
	 *--------------------------------------
	 */
	descp->hdrSize = txMsgLen;

	xdr_swap(descp->pHdr, descp->hdrSize, 1);

	if (fastPath == FALSE)
	{
#if defined(NFX_SMP)
		rc = tpl_xmtPktSlow(clientCBp->nfp_ipAddr,
							htons(clientCBp->nfp_nlmClientInfo.nlmPort),
							descp,
							nlm_globalCfgp->nlm_pBsdSockUdp,
							0);
		
		/* pkt is freed on NCPU */
#else
		rc = bsd_SendSock(
				nlm_globalCfgp->nlm_pBsdSockUdp,
				clientCBp->nfp_ipAddr,
				htons(clientCBp->nfp_nlmClientInfo.nlmPort),
				descp->pHdr, 
				BSD_BUF_FLAT, 
				descp->hdrSize, 
				0);

		PKT_FREE_PACKET(descp);
#endif /* NFX_SMP */

		nlmReqp->reqState = NLM_REQ_DONE;
		nlm_freeAll(nlmReqp, TRUE);

	}
	else
	{
		void					**pAppHdl;

		descp->appInfo = nlmReqp;
		nlmReqp->txPktDescp = (void *)descp;

		pAppHdl = &pConn->pAppHdl;			
		if (*pAppHdl != clientCBp)
		{
			/*--------------------------------------
			 *update TPL Handle since our handle has changed
	 		 *--------------------------------------
	 		 */
			*pAppHdl = clientCBp;
		}
        rc = tpl_xmtPkt (pConn, descp);
		nlm_globalCfgp->nlm_debugCounters.nlm_tplXmtPktSend++;

	}
	NLM_DBG_FUNC_PRINTF("%s: exit\n", __FUNCTION__);
	if (rc != NFX_OK)
	{
		//nlmReqp->nlm_txCmdInfo.grantedRetry = 0;
		//nlmReqp->nlm_txCmdInfo.grantedMaxTimeWaitCnt = 0;
		nlmReqp->reqState = NLM_REQ_DONE;
		nlm_delReqCBFromList(nlmReqp);
		nlm_freeAll(nlmReqp, TRUE);
	}


	return (NFX_OK);

}

/*
 *-----------------------------------------------------------------
 * Function Name:nlm_procTxAsyncGrantedMsg		
 *
 * Description:
 *	Sends a NLM Asynchronous GRANTED_MSG to client to indicates
 *	that a blocked LOCK request has been granted.
 * 
 * Created by:          Joseph Vu
 *
 * Created:             7/30/2002
 *
 *
 *-----------------------------------------------------------------
 */

int32
nlm_procTxAsyncGrantedMsg(nlm_request_t *nlmReqp)
{
	nfp_fileCBPtr_t		fileCBp;
	nfp_clientCBPtr_t	clientCBp;
#ifdef NOT_USED
    nfp_lockCB_t        *pending_lockCBp;
    nlm_lockRec_t       *pending_lockRecp;
#else
    nlm_lockInfo_t      *pend_lockInfop;
#endif
	nlm_lockInfo_t		*nlm_lockInfop;
	pkt_desc_t			*descp;
	char				*txMsgp;
	int32				txMsgLen;
	int32				nfs_version;
	uint32				cookieLen;
	int32				length;
	int32				rc;
	int32				camIdx;
	tpl_connCb_t		*pConn;
	boolean				fastPath;

	rc = NFX_OK;
#ifdef NOT_USED
	pending_lockRecp = nlmReqp->lockRecp;
	if (nlm_validateLockRec(pending_lockRecp) != NFX_OK)
	{
#ifdef	NLM_VALIDATE_LOCK
		panic ("%s: Invalid pending_lockRecp %p\n",
						   	__FUNCTION__,pending_lockRecp);
#endif
		rc = NFX_ERR;
		goto nlm_procTxAsyncGrantedMsgErr;
	}
	
	pending_lockCBp = (nfp_lockCB_t  *)pending_lockRecp->lockCBp;
	if (nfp_validateLockCB(pending_lockCBp) != NFX_OK)
	{
#ifdef	NLM_VALIDATE_LOCK
		panic("%s: Bad pending_lockCBp: %p\n",
					   	__FUNCTION__, pending_lockCBp);	
#endif
		rc = NFX_ERR;
		goto nlm_procTxAsyncGrantedMsgErr;
	}
#else // NOT_USED
    pend_lockInfop = nlmReqp->lockInfop;
#endif // NOT_USED

	/*----------------------------------------
	 *Retrieve next command XID & Cookie
	 *----------------------------------------
	 */
	nlmReqp->nlm_txCmdInfo.cmdXid = nlm_getNextCmdXid();
	nlmReqp->nlm_txCmdInfo.cmdCookie = nlm_getNextCmdCookie();

	clientCBp = nlmReqp->clientCBp;
	fileCBp = nlmReqp->fileCBp;

	txMsgLen = 0;
    fastPath = FALSE;
    pConn = NULL;
	

	//Map NLM ver to comparable
	//NFS version
	
	NLM_MAP_TO_COMP_NFS_VER(nfs_version, 
					nlmReqp->rpcCallHdrp->progVer);

	//Check if this is a new socket
	//
	camIdx = luc_simCamFind(TPL_PROT_UDP,		
		nlmReqp->nlm_txCmdInfo.laddr, 
		nlmReqp->nlm_txCmdInfo.raddr,
		htons(NLM_PORT),
		htons(clientCBp->nfp_nlmClientInfo.nlmPort));

	if (camIdx >= 0)
	{
		pConn = (tpl_connCb_t *) luc_camGetUsrInfo( camIdx );
		if (pConn)
		{
			//If socket exist, send packet via fastpath
			fastPath = TRUE;
		}
	}

	if (fastPath == FALSE)
	{
		descp = PKT_ALLOC_PACKET(1);
	}
	else
	{
		descp = TPL_ALLOC_PACKET(pConn, 0);
		nlm_globalCfgp->nlm_debugCounters.nlm_tplXmtPktAlloc++;
	}
	if (!descp)
	{
		nlm_globalCfgp->nlm_debugCounters.nlm_xmtNoRsrcErr++;
		return NFX_ERR;
	}

	nlm_globalCfgp->nlm_txPktCounters[NLM_GRANTED_MSG] += 1;
	txMsgp = (char *)descp->pHdr;

	//save command command XID for matching 
	//client return reply 

	//Set up portmapper RPC header 
	txMsgLen +=  nlm_msgRpcCallHdr(	txMsgp,
									nlmReqp->nlm_txCmdInfo.cmdXid,
									RPC_NLM_PROG_NUM,
									nlmReqp->rpcCallHdrp->progVer,
									NLM_GRANTED_MSG);		
	txMsgp += txMsgLen;

#ifdef NOT_USED
	nlm_lockInfop = &pending_lockCBp->protocol.nfp_nlmLock;
#else
	nlm_lockInfop = pend_lockInfop;
#endif

	//store NLM part
	//stores cookie	

	cookieLen = 4;
	length = nfs_storeLongValToBuf(	txMsgp,
									cookieLen );
	txMsgp += length;
	txMsgLen += length;

	length = nfs_storeLongValToBuf(	txMsgp,
									nlmReqp->nlm_txCmdInfo.cmdCookie);
	txMsgp += length;
	txMsgLen += length;

	//store exclusive field
#ifdef NOT_USED
	length = nfs_storeLongValToBuf(	txMsgp,
									pending_lockRecp->nlm_exclusive );
#else // NOT_USED
	length = nfs_storeLongValToBuf(	txMsgp,
									nlm_lockInfop->nlm_exclusive );
#endif // NOT_USED
	txMsgp += length;
	txMsgLen += length;

	//alock info, starts with callerName
	length = nfs_storeString(txMsgp, 
					nlm_lockInfop->nlm_callerName,
					nlm_lockInfop->nlm_callerNameLen, 
					NLM_MAX_NAME);

	//update msgp & msg length 
	txMsgp += length;
	txMsgLen += length;

	//stores filehandle
    length = nfs_storeV3Fhandle(&fileCBp->nfp_fileHandle, 
								txMsgp);

	//update msgp & msg length 
	txMsgp += length;
	txMsgLen += length;
	

	//stores oh

	length = nfs_storeString(txMsgp, 
							nlm_lockInfop->nlm_oh,
							nlm_lockInfop->nlm_ohLen, 
							NLM_MAX_NAME);

	//update msgp & msg length 
	txMsgp += length;
	txMsgLen += length;
	

	//stores svid
	
	length = nfs_storeLongValToBuf(	txMsgp,
									nlm_lockInfop->nlm_svid );
	//update msgp & msg length 
	txMsgp += length;
	txMsgLen += length;
	
	//stores l_offset and l_len
	
	if (nfs_version == NFS_VER2)
	{
		//32 bits l_offset
#ifdef NOT_USED
		length = nfs_storeLongValToBuf(	txMsgp, 
									(uint32)pending_lockRecp->nlm_l_offset );
#else // NOT_USED
		length = nfs_storeLongValToBuf(	txMsgp, 
									(uint32)nlm_lockInfop->nlm_l_offset );
#endif // NOT_USED
		txMsgp += length;
		txMsgLen += length;

		//32 bits l_len
#ifdef NOT_USED
		length = nfs_storeLongValToBuf(	txMsgp, 
										(uint32 )pending_lockRecp->nlm_l_len );
#else // NOT_USED
		length = nfs_storeLongValToBuf(	txMsgp, 
										(uint32)nlm_lockInfop->nlm_l_len );
#endif // NOT_USED
		txMsgp += length;
		txMsgLen += length;
	}
	else
	{
		//NFS V3
		//64 bits l_offset
#ifdef NOT_USED
		length = nfs_storeLongLongValToBuf(	txMsgp,
                							pending_lockRecp->nlm_l_offset );
#else // NOT_USED
		length = nfs_storeLongLongValToBuf(	txMsgp,
                							nlm_lockInfop->nlm_l_offset );
#endif // NOT_USED
		txMsgp += length;
		txMsgLen += length;

		//64 bits l_len
#ifdef NOT_USED
		length = nfs_storeLongLongValToBuf(	txMsgp,
                							pending_lockRecp->nlm_l_len );
#else // NOT_USED
		length = nfs_storeLongLongValToBuf(	txMsgp,
                							nlm_lockInfop->nlm_l_len );
#endif // NOT_USED
		txMsgp += length;
		txMsgLen += length;
	}

	//Increment retry count
	//Initializes max time to wait count
	
	nlmReqp->nlm_txCmdInfo.grantedRetry++;	
	nlmReqp->nlm_txCmdInfo.grantedMaxTimeWaitCnt = 0;
	nlmReqp->reqState = NLM_GRANTED_CALL_SND;

	//Send NLM_GRANTED message.
	//Set up pktDesc header size
	descp->hdrSize = txMsgLen;

	xdr_swap(descp->pHdr, descp->hdrSize, 1);

	if (fastPath == FALSE)
	{
#if defined(NFX_SMP)
		rc = tpl_xmtPktSlow(clientCBp->nfp_ipAddr,
							htons(clientCBp->nfp_nlmClientInfo.nlmPort),
							descp,
							nlm_globalCfgp->nlm_pBsdSockUdp,
							0);
		
		/* pkt is freed on NCPU */
#else
		rc = bsd_SendSock(nlm_globalCfgp->nlm_pBsdSockUdp,
							clientCBp->nfp_ipAddr,
							htons(clientCBp->nfp_nlmClientInfo.nlmPort),
							descp->pHdr, 
							BSD_BUF_FLAT, 
							descp->hdrSize, 
							0);

		PKT_FREE_PACKET(descp);
#endif /* NFX_SMP */

		nlmReqp->reqState = NLM_REQ_DONE;
		nlm_freeAll(nlmReqp, TRUE);

	}
	else
	{
		void					**pAppHdl;

		//Send fastpath packet
		descp->appInfo = nlmReqp;
		nlmReqp->txPktDescp = (void *)descp;

		pAppHdl = &pConn->pAppHdl;			
		if (*pAppHdl != clientCBp)
		{
			//update TPL Handle since our handle has changed
			*pAppHdl = clientCBp;
		}
        rc = tpl_xmtPkt (pConn, descp);
		nlm_globalCfgp->nlm_debugCounters.nlm_tplXmtPktSend++;

	}
#ifdef NOT_USED
nlm_procTxAsyncGrantedMsgErr:
#endif
	NLM_DBG_FUNC_PRINTF("%s: exit\n", __FUNCTION__);
	if (rc != NFX_OK)
	{
		nlmReqp->nlm_txCmdInfo.grantedRetry = 0;
		nlmReqp->nlm_txCmdInfo.grantedMaxTimeWaitCnt = 0;
		nlmReqp->reqState = NLM_REQ_DONE;
		nlm_delReqCBFromList(nlmReqp);
		nlm_freeAll(nlmReqp, TRUE);
	}
	return (NFX_OK);
}
