/* vi:ts=4
 *
 *-----------------------------------------------------------------
 *
 * Name:			nlm-prototype.h
 *
 * RCS:  $Id: nlm-prototypes.h,v 1.7 2002/11/06 03:35:39 joseph Exp $
 *
 * Copyright (C) 2002, Agile Storage Inc.
 *
 * Description: NLM module functions prototype.
 *
 * Created by:		Xiaobing Zhang
 *
 * Date Created:	07/19/2002
 *
 *-----------------------------------------------------------------
 */
#ifndef _NLM_PROTOTYPE_H
#define _NLM_PROTOTYPE_H

#include <exports/lock-api.h>

/*
 * Prototype for sm-nlm/nlm-utils.c
 */
extern nfp_fileCB_t *
nlm_getFileCB(fs_Handle *fileHandlep, nfp_clientCB_t *clientCBp);

extern int32
nlm_addLockCBToCurrList(nfp_fileCB_t	*fileCBp,
						nfp_node_t		*currLockNodep,
						nfp_lockCB_t	*addLockCBp);

extern int32
nlm_addLockCBToPendList(nfp_lockCB_t	*currLockCBp,
						nfp_lockCB_t	*addLockCBp);

extern int32
nlm_delLockCB(nfp_lockCB_t	*lockCBp);

extern int32
nlm_rmLockCBFromList(nfp_lockCB_t	*rmLockCBp,
					 boolean		deleteIt);

extern int32
nlm_delLockCBList(nfp_list_t	*lockListp);


#ifdef NOT_USED
extern nlm_lockRec_t *
nlm_getLockRecInCurrList(nfp_fileCB_t	*fileCBp,
						nlm_lockInfo_t	*alockp, 
						short16			searchMask,
						nlm_lockRec_t	**next_lockRecp);

extern nlm_lockRec_t *
nlm_getLockRecInPendingList(nfp_fileCB_t	*fileCBp,
							nlm_lockInfo_t	*alockp, 
							short16			searchMask,
							nlm_lockRec_t	**next_lockRecp);
#endif


extern int32
nlm_delReqCBFromList(nlm_request_t *nlmReqp);

extern int32
nlm_delReqCB(nfp_list_t *nlmReqListp);


extern int32 
nlm_utilMapToCompNFSVer(int32 nlm_ver);

extern nlm_request_t 
*nlm_findNlmReply (
	nfp_clientCBPtr_t clientCBp,    //Client control block
	uint32 replyXid					//transaction ID
);

extern nfp_lockCB_t *	
nlm_getLockCBOwner(	nfp_fileCB_t	*fileCBp,
					nlm_lockInfo_t	*alockp);

#ifdef NOT_USED
extern int32
nlm_regionCmp(nlm_lockRec_t *lock1p,
			  nlm_lockRec_t *lock2p);


extern nlm_lockRec_t *
nlm_getDeadlockEntry(nlm_lockInfo_t	*alockp);

extern nlm_lockRec_t *
nlm_addLockRecToPendingList(nfp_clientCBPtr_t	clientCBp,
							nfp_fileCBPtr_t		fileCb, 
							nlm_lockInfo_t		*nlm_lockInfop,
							nlm_lockRec_t 		*next_lockRecp);

extern nlm_lockRec_t *
nlm_addLockRecToCurrList(nfp_clientCBPtr_t	clientCBp,
						nfp_fileCBPtr_t		fileCb, 
						nlm_lockInfo_t		*nlm_lockInfop,
						nlm_lockRec_t 		*next_lockRecp,
						int32				newLockOwner,
						fs_Handle 			*fhandlep);
#endif

#ifndef NOT_USED
extern int32
nlm_convertToLockReq(nlm_lockInfo_t *lockInfo, nlm_request_t *nlmReqp,
                     lock_request_t *lockReq);

extern int32
nlm_procTestReq(nfp_fileCB_t *fileCBp, nlm_lockInfo_t *lockInfo,
                nlm_request_t *nlmReqp);

extern int32
nlm_procLockReq(nlm_lockInfo_t *lockInfo, nlm_request_t *nlmReqp);

extern int32
nlm_procCancellockReq(nlm_lockInfo_t *lockInfo, nlm_request_t *nlmReqp);

extern int32
nlm_procUnlockReq(nlm_lockInfo_t *lockInfo, nlm_request_t *nlmReqp);

extern void
nlm_procGrantedCallback(vs_mem_index_t private);
#endif

/*
 * Prototype for sm-nlm/nlm-msg.c
 */
extern int32
nlm_msgFsGetAttrReq(nlm_request_t *nlmReqp);

extern void
nlm_msgRcv( eee_desc_t *eDescp );

extern int32	
nlm_msgRpcCallHdr(
	char *msgp,
	uint32 cmdXid,
	uint32 rpcProgNum,
	uint32 rpcProgVer,
	uint32 rpcProcNum
);


/*
 * Prototype for sm-nlm/nlm_debug.c
 */
extern void
nlm_showLock(nfp_lockCB_t *lockCBp, char *indentation);

#ifdef NOT_USED
extern void
nlm_showLockList(nfp_list_t *lockListp, char *indentation1, char *indentation2);
#endif

extern void
nlm_showFileLocks(nfp_fileCB_t  *fileCBp);

extern void
nlm_showStat();


/*
 * Prototype for sm-nlm/nlm.c
 */
int32
nlm_reqTimeoutHandler(int32 timer_handle, void* param1,
                void* param2,
                void* param3, void* param4);

int32
nlm_gracePeriodTimeoutHandler(	int32 timer_handle, 
								void* param1, void* param2, 
								void* param3, void* param4);

void
nlm_freeAll(nlm_request_t *nlmReqp, int32 freeTxPktDescFlag);

int32		//len=bytes of alock info
			//or NFX_ERR 
nlm_extractAlockInfo(char *nlm_msgp, 
	nlm_lockInfo_t *alockp,
	int32 nlm_version,
	fs_Handle *fileHandlep);

#ifdef NOT_USED
int32
nlm_saveLockInfo(nfp_lockCBPtr_t dest_lockCBp, 
				nlm_lockRec_t *lockRecp,
				nlm_lockInfo_t *src_lockInfop);

int32
nlm_storeAlockInfo(	char *nlm_txMsgp,
					nlm_lockRec_t *lockRecp,
					int32 nlm_version);

int32
nlm_checkforConflictLock(
	nlm_request_t 		*nlmReqp,
	nfp_fileCBPtr_t 	fileCBp,
	nlm_lockInfo_t		*nlm_lockInfop,
	uint64				fsize,
	char 				*cookiep,
	ushort16 			cookieLen);

int32
nlm_addNewLock(nfp_fileCBPtr_t fileCBp, 
	nfp_lockCBPtr_t	lockCBp,				
	nfp_nodePtr_t	nextLockNodep,
	nfp_clientCBPtr_t clientCBp,
	fs_Handle *fhandlep);
#else // NOT_USED
extern int32
nlm_storeAlockInfo(	char *nlm_txMsgp,
					nlm_lockInfo_t *lockRecp,
					int32 nlm_version);

extern int32
nlm_saveLockInfo(nlm_lockInfo_t *srcLock, nlm_request_t *origNlmReqp,
                 nfp_requestCB_t **dstReqCB);

#endif // NOT_USED

int32
nlm_reqTimeoutHandler(int32 timer_handle, 
	void* param1, void* param2, 
	void* param3, void* param4);


#ifdef NOT_USED
int32 
nlm_getportNlm (	nlm_request_t *nlmReqp, 
						nlm_lockRec_t *pendingLockRecp);
#else
int32 nlm_getportNlm (nlm_request_t *newNlmReqp, nlm_request_t *nlmReqp);
#endif

int32
nlm_nsmMonCmpInd(evm_volId_t        volId,
                  nfp_clientCB_t     *nlm_clientCBp,
                  int32              status);

int32
nlm_nsmUnmonCmpInd(evm_volId_t        volId,
                    nfp_clientCB_t     *nlm_clientCBp,
                    int32              status);

int32
nlm_nsmClientDownInd(uint32    hostIpAddr);

int32
nlm_procTxGetportReq(nlm_request_t *nlmReqp);
#ifdef	NOTUSED
nlm_lockRec_t *nlm_checkUnlockedRegions(
	nfp_fileCBPtr_t fileCBp,		
	nlm_lockInfo_t *unlockAlockp,  
	int32 *unlockType
);
#endif
#ifdef NOT_USED
int32
nlm_checkUnlockedRegions(nlm_lockRec_t *unlockRecp,
            			nlm_lockInfo_t *unlockAlockp,
            			nlm_request_t *nlmReqp,
            			int32 *overlap_p,
            			int32 *unlockType);
#endif


int32  
nlm_extractFhandle(
    char *nfsMsgp,  
    fs_Handle *fhandle
);


#ifdef NOT_USED
int32 nlm_callbackNextPendingLock(	nlm_lockRec_t	*lockRecp,
									nlm_request_t 	*nlmReqp,
									uint32 			rmListType);


void nlm_clearLock(uint32 clientAddr);

nlm_lockRec_t *
nlm_unlockSplittedTwoRegions(nlm_lockRec_t	*unlockRecp,
							nlm_lockInfo_t	*nlm_lockInfop,
							nfp_fileCBPtr_t		fileCBp,
							nlm_request_t *nlmReqp);
nlm_lockRec_t *
nlm_getConflictinglock(	nfp_fileCB_t	*fileCBp,
						nlm_lockInfo_t	*alockp, 
						short16			searchMask,
						nlm_lockRec_t	**nextLockRecpp,
						uint32			*listType);

int32	/*1: deadlock or 0: no deadlock*/
nlm_isLockReqCauseDeadlock(	nlm_lockRec_t 	*currHolderLockRecp,
							nlm_lockInfo_t	*nlm_lockInfop);


int32
nlm_setPendingLockEntry(	nlm_lockRec_t	*lockRecp,
							nlm_lockRec_t	*pendingLockRecp);
#endif // NOT_USED
#ifdef	NOTUSED
int32
nlm_notifyRemoveStaleLock(nfp_lockCBPtr_t	unlockCBp);
#endif

/*
 *sm-nlm/nlm-tpl.c
 */
int32 nlm_tplRcvCheckPkt(rpcCallHdr_t *rpcCallp);

int32
nlm_tplAddConnInd( tpl_bindCb_t	*pBind, 	/*ptr to Bind CB */
	void 		*connIdp, 	/*ptr to newly added connection*/
	void 		**appHdlpp,	/*ptr to correspond app. handle*/
	uint32 		raddr,		/*remote IP address*/
	ushort16 	rport		/*remote port*/
);

int32
nlm_tplDelConnInd (void *appHdlp, void *connIdp);


int32
nlm_tplRcvPktInd(void *appHdlp, pkt_desc_t *descp);

int32
nlm_tplXmtCmpInd (void *appHdlp, pkt_desc_t *descp);

int32 
nlm_tplBindReq(uint32 ipaddr);

/*sm-nlm/nlm-procRx.c*/
int32
nlm_procRxNullReq(nlm_request_t    *nlmReqp);

int32
nlm_procRxTestReq(nlm_request_t    *nlmReqp);

int32
nlm_procRxLockReq (nlm_request_t    *nlmReqp);

int32
nlm_procRxCancelReq (nlm_request_t    *nlmReqp);

int32
nlm_procRxUnlockReq (nlm_request_t    *nlmReqp);

int32
nlm_procRxInvalidReq (nlm_request_t    *nlmReqp);

int32 
nlm_procRxShareReq (nlm_request_t    *nlmReqp);

int32
nlm_procRxUnsharetReq (nlm_request_t    *nlmReqp);

int32
nlm_procRxNmLockReq (nlm_request_t    *nlmReqp);

int32
nlm_procRxNlmFreeAllReq (nlm_request_t    *nlmReqp);

#ifdef NOT_USED
int32
nlm_procRxGrantedReply (nlm_request_t    *nlmReqp);
#else
int32
nlm_procRxGrantedReply (nlm_request_t *nlmReqp, lock_request_t *lockReq);
#endif

int32
nlm_procRxGrantedReq(nlm_request_t    *nlmReqp);

int32
nlm_procRxTestMsg(nlm_request_t    *nlmReqp);

int32
nlm_procRxLockMsg(nlm_request_t    *nlmReqp);

int32
nlm_procRxCancelMsg(nlm_request_t    *nlmReqp);

int32
nlm_procRxUnlockMsg(nlm_request_t    *nlmReqp);

int32
nlm_procRxGrantedMsg(nlm_request_t    *nlmReqp);

int32
nlm_procRxTestRes(nlm_request_t    *nlmReqp);

int32
nlm_procRxLockRes(nlm_request_t    *nlmReqp);

int32
nlm_procRxCancelRes(nlm_request_t    *nlmReqp);

int32
nlm_procRxUnlockRes(nlm_request_t    *nlmReqp);

int32
nlm_procRxGrantedRes(nlm_request_t    *nlmReqp);















/*
 *sm-nlm/nlm-procTx.c
 */
int32
nlm_procTxPkt(
	pkt_desc_t *tplXmitPktDescp, 
	nlm_request_t *nlmReqp
);

int32
nlm_procTxSendResp(
	nlm_request_t *nlmReqp,
	int32 status,
	char *nlm_cookiep,
	short16 nlm_cookieLen
);

int32
nlm_procTxSendErrResp(
	nlm_request_t *nlmReqp,	//Ptr to nlm request
	int32 status,			//error status
	char *procStartp		//Ptr to where the NLM procedure starts
);

int32
nlm_procTxGranted(nlm_request_t *nlmReqp);

int32
nlm_procTxSendASyncMsgResp(
	nlm_request_t *nlmReqp,
	int32 status,
	char *nlm_cookiep,
	short16 nlm_cookieLen
);

int32
nlm_procTxAsyncGrantedMsg(nlm_request_t *nlmReqp);

/*
 *sm-nlm/nlm-trace.c
 */
void nlm_decodeRpcHdr(rpcCallHdr_t *rpcCallp);
void nlm_decodeRpcHdrErr(int32 error);
int32 nlm_decodeCookieObj(char *msgp, int32 direction);
int32 nlm_decodeExclusiveObj(char *msgp, int32 direction);
int32 nlm_decodeReclaimObj(char *msgp);
int32 nlm_decodeStateObj(char *msgp);
int32 nlm_decodeBlockObj(char *msgp);
int32 nlm_decodeAlockObj(char *msgp,
					int32 nlm_version);
int32 nlm_rxDecodeTestProc(rpcCallHdr_t *rpcCallp);
int32 nlm_rxDecodeLockProc(rpcCallHdr_t *rpcCallp);
int32 nlm_rxDecodeCancelProc(rpcCallHdr_t *rpcCallp);
int32 nlm_rxDecodeUnlockProc(rpcCallHdr_t *rpcCallp);
int32 nlm_rxDecodePkt(pkt_desc_t *descp);
void nlm_xmtDecodePkt(nlm_request_t *nlmReqp);

/*
 *sm-nlm/nlm-debug.c
 */
void
nlm_clearStat(void);

/*
 *sm-nlm/nlm-pmapTpl.c
 */
int32
nlm_portMapBindReq (uint32 ipAddr);


#endif  /*_NLM_PROTOTYPE_H*/

