/* vi:ts=4
 *-----------------------------------------------------------------
 *
 * Name:        nlm-tpl.c
 *
 * RCS:  $Id: nlm-tpl.c,v 1.7 2002/11/06 03:35:39 joseph Exp $
 *
 * Copyright (C) 2001, Agile Storage Inc.
 *
 * Description:
 *	Contain Network Lock Manager (NLM)/Transport Layer Protocol (TPL)
 *	interface functions.
 *
 * Created by: Joseph Vu
 *
 * Date Created:        6/28/02
 *
 *-----------------------------------------------------------------
 */
#ifdef NFP_TXRX

#include <nfx-incl.h>
#include <exports/pkt-api.h>
#include <exports/vs-mem-api.h>
#include <exports/lock-api.h>
#include <exports/nfs-api.h>
#include "nlm-prototypes.h"
#include "../sm-tpl/tpl.h"

//#define	NLM_DEBUG
#define NLM_DBG_PRINTF  if (nlm_traceEnable) printf	
#define NLM_DBG_FUNC_PRINTF if (nlm_traceDebugEnable) printf

/*
 *NLM Rx procedures
 */
int32 (*nlm_procs[NLM_MAX_PROC]) (nlm_request_t    *nlmReqp) = 
{
        nlm_procRxNullReq,			/*0: NLM_NULL*/
        nlm_procRxTestReq,			/*1: NLM_TEST*/
        nlm_procRxLockReq,			/*2: NLM_LOCK*/
        nlm_procRxCancelReq,		/*3: NLM_CANCEL*/
        nlm_procRxUnlockReq,		/*4: NLM_UNLOCK*/
        nlm_procRxGrantedReq,		/*5: NLM_GRANTED*/
        nlm_procRxTestMsg,			/*6: NLM_TEST_MSG*/
        nlm_procRxLockMsg,			/*7: NLM_LOCK_MSG*/
        nlm_procRxCancelMsg,		/*8: NLM_CANCEL_MSG*/
        nlm_procRxUnlockMsg,		/*9: NLM_UNLOCK*/
        nlm_procRxGrantedMsg,		/*10:NLM_GRANTED_MSG*/
        nlm_procRxTestRes,			/*11:  NLM_TEST_RES*/
        nlm_procRxLockRes,			/*12:  NLM_LOCK_RES*/
        nlm_procRxCancelRes,		/*13:  NLM_CANCEL_RES*/
        nlm_procRxUnlockRes,		/*14:  NLM_UNLOCK_RES*/
        nlm_procRxGrantedRes,		/*15:  NLM_GRANTED_RES*/
        nlm_procRxInvalidReq,		/*16:  NOT ASSIGNED*/
        nlm_procRxInvalidReq,		/*17:  NOT ASSIGNED*/
        nlm_procRxInvalidReq,		/*18:  NOT ASSIGNED*/ 
        nlm_procRxInvalidReq,		/*19:  NOT ASSIGNED*/
        nlm_procRxShareReq,			/*20:  NLM_SHARE*/
        nlm_procRxUnsharetReq,		/*21:  NLM_UNSHARE*/
        nlm_procRxNmLockReq,		/*22:  NLM_NM_LOCK*/
        nlm_procRxNlmFreeAllReq,	/*23:  NLM_FREE_ALL*/
};

/*
 *True if the received NLM procedure is
 *one the valid procedures handle
 *by NLM server 
 */
boolean nlm_isProcValid[NLM_MAX_PROC] =
{
        TRUE,						/*0: NLM_NULL*/
        TRUE,						/*1: NLM_TEST*/
        TRUE,						/*2: NLM_LOCK*/
        TRUE,						/*3: NLM_CANCEL*/
        TRUE,						/*4: NLM_UNLOCK*/
        TRUE,						/*5: NLM_GRANTED*/
        TRUE,						/*6: NLM_TEST_MSG*/
        TRUE,						/*7: NLM_LOCK_MSG*/
        TRUE,						/*8: NLM_CANCEL_MSG*/
        TRUE,						/*9: NLM_UNLOCK*/
        FALSE,						/*10:NLM_GRANTED_MSG*/
        FALSE,						/*11:  NLM_TEST_RES*/
        FALSE,						/*12:  NLM_LOCK_RES*/
        FALSE,						/*13:  NLM_CANCEL_RES*/
        FALSE,						/*14:  NLM_UNLOCK_RES*/
        FALSE,						/*15:  NLM_GRANTED_RES*/
        FALSE,						/*16:  NOT ASSIGNED*/
        FALSE,						/*17:  NOT ASSIGNED*/
        FALSE,						/*18:  NOT ASSIGNED*/ 
        FALSE,						/*19:  NOT ASSIGNED*/
        TRUE,						/*20:  NLM_SHARE*/
        TRUE,						/*21:  NLM_UNSHARE*/
        TRUE,						/*22:  NLM_NM_LOCK*/
        TRUE,						/*23:  NLM_FREE_ALL*/
};

//Externs

nlm_request_t *nlm_findReqCB(
	nfp_clientCBPtr_t clientCBp,    //Client control block
	uint32 xid,						//transaction ID
	uint32 procNum					//procedure number
);

void
tpl_connUpdateActivityTime(void *pTplConn,
    uint32 activityTime         //Activity time in seconds
);

extern uint32 tpl_delConnReqCnt;

/*-----------------------------------------------------------------
 * Name:	nlm_tplRcvCheckPkt 
 *
 * Description:
 *		Check received NLM packet. 	
 * Created by:  Joseph Vu
 *
 * Date Created:        6/28/02
 *
 *-----------------------------------------------------------------
 */
int32 nlm_tplRcvCheckPkt(rpcCallHdr_t *rpcCallp)
{
	/*
	 *Check RPC version number
	 */
	if (rpcCallp->rpc_rpcVer != RPC_VER2)
	{
		return (NFS_INT_RPC_MISMATCH);
	}

	/*
	 *Check program version number
	 */
	if ((rpcCallp->progVer !=  NLM_VER1) && 
		(rpcCallp->progVer != NLM_VER3) &&
	    (rpcCallp->progVer != NLM_VER4) )
	{
		return (NFS_INT_RPC_PROG_MISMATCH);
	}

	/*
	 *Check procedure number
	 */
	if ( (rpcCallp->rpc_proc < NLM_NULL) ||
		(rpcCallp->rpc_proc >= NLM_MAX_PROC)  ||
		 (nlm_isProcValid[rpcCallp->rpc_proc] != TRUE) )
	{
		return (NFS_INT_RPC_PROC_UNAVAIL);
	}
	return (NFX_OK);


}

/*-----------------------------------------------------------------
 * Name:      	nlm_tplAddConnInd	
 *
 * Description:
 *	Add connection indication from the transport layer to
 *	adds a new connection. 
 *
 * Created by:  Joseph Vu
 *
 * Date Created:        6/28/02
 *
 *-----------------------------------------------------------------
 */
int32
nlm_tplAddConnInd( 
	tpl_bindCb_t	*pBind,	/*ptr to Bind CB */
	void *connIdp, 			/*ptr to newly added connection*/
	void 		**appHdlpp,	/*return ptr app. handle to TPL*/
	uint32 		raddr,		/*remote IP address*/
	ushort16 	rport		/*remote port*/
)
{
	nfp_clientCBPtr_t	clientCBp;	


	/*
	 *Check if client control block exist 
	 */
	clientCBp = nfp_getClientCBbyAddr(raddr);
	if (clientCBp != NULL)
	{
		/*
		 *Use same client control block
		 */
		*appHdlpp = clientCBp;
		return (NFX_OK);
	}

	/*
	 *Allocate NLM Client Control  Block
	 */
	clientCBp = nfp_allocClientCB();
	if (clientCBp == NULL)
		return (TPL_ERR_NO_RSRC);


	/*
	 *Fill in handle upon return to TPL
	 */
	*appHdlpp = clientCBp;
	clientCBp->nfp_connId = connIdp;
	clientCBp->nfp_ipAddr = raddr;

	/* 
	 * incr conn count in client 
	 */
	clientCBp->nfp_connCnt++;			
#ifdef	NLM_DEBUG
	NLM_DBG_FUNC_PRINTF("%s: Exit-clientCBp: %p connId: %p raddr: %08x\n", 
		__FUNCTION__, clientCBp, clientCBp->nfp_connId, 
		clientCBp->nfp_ipAddr);
#endif	//NLM_DEBUG


	return (NFX_OK);
}

/*-----------------------------------------------------------------
 * Name:	nlm_tplDelConnInd	
 *
 * Description: 
 * 	Delete connection indication from the transport layer to delete
 *	an existing connection.
 *
 * Created by:  Joseph Vu
 *
 * Date Created:        6/28/02
 *
 *-----------------------------------------------------------------
 */
int32
nlm_tplDelConnInd (void *appHdlp, void *connIdp)
{
	nfp_clientCBPtr_t   clientCBp;


	clientCBp= (nfp_clientCBPtr_t)appHdlp;
	if (nfp_validateClientCB(clientCBp) != NFX_OK)
	{
		panic("%s:  bad client control block!\n",__FUNCTION__);
	}
	/*
	 * decr. conn count in client
	 */
	clientCBp->nfp_connCnt--;

    /* 
	 * Delete client if all connections
     * assoc. with clients are deleted.
     */
    if (clientCBp->nfp_connCnt <= 0)
		nfp_delClientCB (clientCBp);


	return (NFX_OK);
}

/*-----------------------------------------------------------------
 * Name:	nlm_tplRcvProcessPkt 
 *
 * Description: 
 *	Process NLM packet received from the Transport Layer 
 *	Protocol (TPL).
 *
 * Created by:  Joseph Vu
 *
 * Date Create:	6/28/02
 *
 *-----------------------------------------------------------------
 */
int32		//NLM status 
nlm_tplRcvProcessPkt(
	nfp_clientCBPtr_t clientCBp,
	rpcCallHdr_t	*rpcCallp,
	pkt_desc_t *descp
)				
{
	int32					rpcHdrSize;
	int32					error;
	nfp_requestCBPtr_t      reqCBp;
	nlm_request_t			*nlmReqp;
		
	
	error = NFX_OK;

	/*
	 *Allocate request control block
	 */
	reqCBp = nfp_allocReqCB ();
	if (reqCBp == NULL)
		return (NFX_ERR);
	nlm_globalCfgp->nlm_debugCounters.nlm_allocReqCB++;
#ifdef NOT_USED
	nlmReqp = &reqCBp->protocol.nlm_req;	
#else
	nlmReqp = (nlm_request_t *)vs_mem_idxToPtr(reqCBp->protocol.vs_nlm_idx);	
#endif
	rpcHdrSize =  nfs_processRpcAuth(rpcCallp, 
					&nlmReqp->rpcHdr);
	if (rpcHdrSize <= 0 )	
	{
		nfp_deallocReqCB( reqCBp );
		return (NFX_ERR);
	}

	nlmReqp->reqCBp = reqCBp;

	nlmReqp->txPktDescp = NULL;
	nlmReqp->fileCBp = NULL;
#ifdef NOT_USED
	nlmReqp->lockRecp = NULL;
#else
	nlmReqp->lockInfop = NULL;
#endif
	nlmReqp->rpcCallHdrp = &nlmReqp->rpcHdr;
	nlmReqp->clientCBp = clientCBp;
	nlmReqp->reqState = NLM_REQ_INPROG;
	nlmReqp->tplConnp = descp->pTplConn;
	nlmReqp->rxPktDescp = descp;
	clientCBp->nfp_nlmClientInfo.nlm_version = 	rpcCallp->progVer;	
	nlmReqp->procNum = rpcCallp->rpc_proc;
	nlmReqp->nlm_txCmdInfo.laddr = descp->laddr;
	nlmReqp->nlm_txCmdInfo.raddr = descp->raddr;
	clientCBp->nfp_localIpAddr = descp->laddr;
	/*
	 *Save procedure start of message ptr & message length
	 */
	nlmReqp->procStart = descp->pHdr + rpcHdrSize;
	nlmReqp->procLen = descp->hdrSize - rpcHdrSize;
#ifdef	NOTUSED
	/*
	 *Add nlmReq to client control block
	 */	
    if (nfs_addCBToList(&clientCBp->nfp_nlmReqList, 
				(void *)nlmReqp, NFP_REQCB_NODE)  != NFX_OK)
    {
        NLM_DBG_FUNC_PRINTF("%s: Failure to Add ReqCb to client control blk!\n",
						__FUNCTION__);
        return (NFX_ERR);
    }
#endif	//NOTUSED

	/*
	 *Increment Rx stat pkt counter.
	 *Call NLM protocol procedure
	 */
	nlm_globalCfgp->nlm_rxPktCounters[rpcCallp->rpc_proc] += 1;

	error = (*(nlm_procs[rpcCallp->rpc_proc]))(nlmReqp);
	if (error != NFX_OK)
		goto nlm_tplRcvProcessPktErr;

	return (NFX_OK);



nlm_tplRcvProcessPktErr:
	if (error == NFX_ERR)
	{
		nlmReqp->reqState = NLM_REQ_DONE;
		//Set Rx Pkt Descp to NULL and
		//let TPL free the PKT
		nlmReqp->rxPktDescp = NULL;
		nlm_freeAll(nlmReqp, TRUE);

		return (error);
	}
#ifdef	NOTUSED	
	if (nlm_procTxSendErrResp(nlmReqp, error, 
						nlmReqp->procStart) != NFX_OK)
   	{
		nlmReqp->rxPktDescp = NULL;
		nlm_freeAll(nlmReqp, TRUE);

		return (NFX_ERR);	
   	}
	//Increment procedure stat counter
	nlm_globalCfgp->nlm_txPktCounters[rpcCallp->rpc_proc] += 1;
#endif	//NOTUSED

	return (error);	
}

/*-----------------------------------------------------------------
 * Name:	nlm_tplRcvPktInd 
 *
 * Description: 
 *	Receive packet indication from the transport 
 *	layer for processing.
 *
 * Created by:  Joseph Vu
 *
 * Date Create:	6/28/02
 *
 *-----------------------------------------------------------------
 */
int32
nlm_tplRcvPktInd(void *appHdlp, pkt_desc_t *descp)
{
	int32				error;
	int32				rc;
	rpcCallHdr_t		*rpcCallp;
	nfp_clientCBPtr_t	clientCBp;	
	nlm_request_t		*nlmReqp;
	uint32				callDir;
#ifndef NOT_USED
    lock_request_t      *lockReq;
#endif

	if ((appHdlp == NULL) || (descp == NULL) ||
		(descp->pHdr == NULL) )
		return (TPL_FREE_PKT);


	if (!(descp->flags & PKT_DESC_INUSE))
	{
		panic("%s: bad pkt=%x flags=%x\n",
			__FUNCTION__,descp, descp->flags);
	}

    /*-----------------------------------------
	 *Update access time for UDP connection
	 *-----------------------------------------
	 */	

	if (descp->tplProt == TPL_PROT_UDP)
	{
		int32               aTime;

		aTime = nfs_utilGetTimeOfDayInSecs();
		tpl_connUpdateActivityTime(descp->pTplConn, aTime);
	}


	clientCBp = (nfp_clientCBPtr_t)appHdlp;

	if (nfp_validateClientCB(clientCBp) != NFX_OK)
	{
		panic("%s:  bad client control block!\n",
						__FUNCTION__);
		return (TPL_FREE_PKT);
	} 

	rpcCallp = (rpcCallHdr_t *)descp->pHdr;
	callDir = ntohl(rpcCallp->rpc_direction);

	error = NFX_OK;

	if (callDir == RPC_CALL)
	{
    	/*-----------------------------------------
	 	 * Perform Net to Host conversion.
	 	 *-----------------------------------------
	 	 */	
		xdr_swap(descp->pHdr, 
				(sizeof(rpcCallHdr_t)-sizeof(rpc_authSysInfo_t)), 0);

		if (nlm_traceEnable != 0)
			nlm_rxDecodePkt(descp);	

		rc = nlm_tplRcvCheckPkt(rpcCallp);
		if (rc != NFX_OK)
		{
			nlm_globalCfgp->nlm_debugCounters.nlm_tplDropRcvPkt++;
			return (TPL_FREE_PKT);
		}
#ifdef	NOTUSED
		/*
	 	 *Check if request is in 
	 	 *duplicate request cache 
	 	 */
		nlmReqp = nlm_findReqCB(clientCBp, rpcCallp->rpc_xid, 
					rpcCallp->rpc_proc);
		if (nlmReqp != NULL)
		{
			//TODO: Found request in duplicate request cache
			if (nlmReqp->reqState == NLM_REQ_DONE)
			{
				//Handle retransmission
				nlm_procTxPkt( (pkt_desc_t *)nlmReqp->txPktDescp,
							nlmReqp);
				nlmReqp->reqState = NLM_REQ_REXMIT;
			}
			nlm_globalCfgp->nlm_txPktCounters[rpcCallp->rpc_proc] += 1;
		}
#endif	//NOTUSED

		error = nlm_tplRcvProcessPkt(clientCBp, rpcCallp, descp);
		if (error != NFX_OK)
		{

			nlm_globalCfgp->nlm_debugCounters.nlm_rcvPktIndErr++;
			if ((descp->tplProt == TPL_PROT_UDP) &&
				descp->pTplConn)
			{
				tpl_delConnReq(descp->pTplConn);
			}
		}	
		return (TPL_FREE_PKT);
	}
	else
	{
		int32			rpcReplyHdrLen;
		rpcReplyHdr_t	*rpcReplyp;

    	/*-----------------------------------------
	 	 * Perform Net to Host conversion.
	 	 *-----------------------------------------
	 	 */	

		xdr_swap(descp->pHdr,sizeof(rpcReplyHdr_t), 0);
		if (nlm_traceEnable != 0)
			nlm_rxDecodePkt(descp);	

		rpcReplyp = (rpcReplyHdr_t *)descp->pHdr;

		nlmReqp =  nlm_findNlmReply(clientCBp, rpcReplyp->rpc_xid); 
		if (nlmReqp == NULL)
		{
			nlm_globalCfgp->nlm_debugCounters.rxGrantedLocateReqErr++;
			goto nlm_rcvPktIndExit;
		}
#ifndef NOT_USED
        lockReq = (lock_request_t *)nlmReqp->lockInfop;
#endif

#ifdef	NLM_DEBUG
		NLM_DBG_FUNC_PRINTF("%s: replyXid: %d nlmReqp: %p\n",
				 __FUNCTION__, rpcReplyp->rpc_xid, nlmReqp);
#endif

		if (nlmReqp->reqState == NLM_REQ_DONE)
			/*rexmit:  ignore*/
			goto nlm_rcvPktIndExit;

    	/*-----------------------------------------
		 *Increment reply cnt
		 *process RPC reply hdr 
	 	 *-----------------------------------------
	 	 */	
		nlm_globalCfgp->nlm_debugCounters.rxGrantedReplyCnt++;

		rpcReplyHdrLen = rpc_processReplyHdr(descp->pHdr);
		if (rpcReplyHdrLen == NFX_ERR)
		{
			nlmReqp->nlm_txCmdInfo.grantedRetry = 0;
			nlmReqp->nlm_txCmdInfo.grantedMaxTimeWaitCnt = 0;
			/*----------------------------------
			 *Set Rx Pkt Descp to NULL and
			 *let TPL free the PKT
			 *---------------------------------
			 */
			nlmReqp->rxPktDescp = NULL;
			nlmReqp->reqState = NLM_REQ_DONE;
			nlm_freeAll(nlmReqp, TRUE);

			goto nlm_rcvPktIndExit;
		}

		nlm_globalCfgp->nlm_rxPktCounters[NLM_GRANTED] += 1;
		
		nlmReqp->procStart = descp->pHdr + rpcReplyHdrLen;
		nlmReqp->procLen = descp->hdrSize - rpcReplyHdrLen;

#ifdef NOT_USED
		error = nlm_procRxGrantedReply(nlmReqp);
#else
		error = nlm_procRxGrantedReply(nlmReqp, lockReq);
#endif
		if (error != NFX_OK)
		{
			nlm_globalCfgp->nlm_debugCounters.nlm_rcvPktIndErr++;
		}

#ifdef	NLM_DEBUG
		NLM_DBG_FUNC_PRINTF("%s: rport: %d client NLM port: %d\n",
				__FUNCTION__,
				htons(descp->rport), 
				clientCBp->nfp_nlmClientInfo.nlmPort);
#endif

#ifdef	NOTUSED
		if (htons(descp->rport) == 
						clientCBp->nfp_nlmClientInfo.nlmPort)
		{
			tpl_connCb_t		*pConn;

			pConn = (tpl_connCb_t        *)descp->pTplConn;

			clientCBp->nfp_nlmClientInfo.nlm_pBsdSockUdp = 
												pConn->pSock;
			NLM_DBG_FUNC_PRINTF("%s: nlm sock: %p\n",
					__FUNCTION__,
					clientCBp->nfp_nlmClientInfo.nlm_pBsdSockUdp );
		}
#endif	//NOTUSED

		return (TPL_FREE_PKT);
	}

nlm_rcvPktIndExit:
	return (TPL_FREE_PKT);
}


/*-----------------------------------------------------------------
 * Name: nlm_tplXmtCmpInd       
 *
 * Description: 
 *	Transmit packet complete indication from the transport layer.
 *	 
 *
 * Created by:  Joseph Vu
 *
 * Date Created:        6/28/02
 *
 *-----------------------------------------------------------------
 */
int32
nlm_tplXmtCmpInd (void *appHdlp, pkt_desc_t *descp)
{
	nlm_request_t			*nlmReqp;

	if ( (descp == NULL) || (appHdlp == NULL ) )
	{
		panic ("%s: bad descp or app handle!\n",
						__FUNCTION__); 
	}
	
	nlm_globalCfgp->nlm_debugCounters.nlm_tplXmtPktCmpInd++;

	//Retrieve our NLM request from desc appInfo
	
	nlmReqp = descp->appInfo;
	if (nfp_validateNlmReqCB (nlmReqp) != NFX_OK)
	{
		panic("%s: Bad nlmReqp %p\n", 
						__FUNCTION__, nlmReqp);		
	}

	if (nlmReqp->reqState == NLM_GRANTED_CALL_SND)
	{
		nlm_globalCfgp->nlm_debugCounters.nlm_tplXmtPktFree++;
		return (TPL_FREE_PKT);
	}

	//Check for NULL procedure
	//or Granted procedure

	if (nlmReqp->rpcCallHdrp)  
	{
		if ( (nlmReqp->rpcCallHdrp->rpc_proc == NLM_NULL) ||
			(nlmReqp->rpcCallHdrp->rpc_proc == NLM_GRANTED) )
			if (descp->tplProt == TPL_PROT_UDP)
			{
				tpl_delConnReq(descp->pTplConn);
			}
	}
	nlmReqp->reqState = NLM_REQ_DONE;

	nlm_freeAll(nlmReqp, TRUE);
	nlm_globalCfgp->nlm_debugCounters.nlm_tplXmtPktFree++;

	return (TPL_FREE_PKT);
}

#if defined(NFX_SMP)
/*-----------------------------------------------------------------
 * Name: nlm_tplBindReqAck
 *
 * Description:	bind request ack
 *	
 *
 * Created by:  Su Wang
 *
 * Date Created:        12/04/02
 *
 *-----------------------------------------------------------------
 */
int32 nlm_tplBindReqAck(void* pHandle, int32 rc, void *in_pBind)
{
	tpl_bindCb_t	*pBind = (tpl_bindCb_t*)in_pBind;
	
	if (rc != NFX_OK)
		panic("%s: tpl bind failed\n", __FUNCTION__);
	
	if (pBind->tplProt == TPL_PROT_UDP)
		nlm_globalCfgp->nlm_pTplBindUdp = pBind;
	else
		nlm_globalCfgp->nlm_pTplBindTcp = pBind;
	
	return NFX_OK;
}
#endif /* NFX_SMP */

/*-----------------------------------------------------------------
 * Name: nlm_tplBindReq
 *
 * Description:
 * Binds NLM to Transport Layer (UDP/TCP).  
 *
 * Created by:  Joseph Vu
 *
 * Date Created:        5/28/01
 *
 *-----------------------------------------------------------------
 */
int32 nlm_tplBindReq(uint32 ipaddr)
{
	int32			bindOpt;
	ushort16        protocol;
	tpl_bindCb_t    bindCB;
	tpl_bindCb_t    *bindCBp;
#if !defined(NFX_SMP)
	tpl_bindCb_t    *tplRspBindUdp;
	tpl_bindCb_t    *tplRspBindTcp;
#endif /* NFX_SMP */

	protocol = TPL_PROT_UDP;
	while (1)
	{
		/*
	 	 * Bind NLM server on UDP or TCP
	 	 */
    	memset(&bindCB, 0, sizeof(tpl_bindCb_t));

		bindOpt = TPL_BIND_SERVER | TPL_BIND_USE_RPC;
		bindCBp = &bindCB;
		bindCBp->laddr         = ipaddr;
//		bindCBp->lport         = htons(NLM_PORT);	
		bindCBp->lport         = ntohs(NLM_PORT);	
		bindCBp->tplProt       = protocol;
		bindCBp->ul_addConnInd = nlm_tplAddConnInd;
		bindCBp->ul_delConnInd = nlm_tplDelConnInd;
		bindCBp->ul_rcvPktInd  = nlm_tplRcvPktInd;
		bindCBp->ul_xmtCmpInd  = nlm_tplXmtCmpInd;

#if defined(NFX_SMP)
		bindCBp->ul_bindReqAck = nlm_tplBindReqAck;
		tpl_bindReq(bindCBp, NULL, bindOpt);
		
		if (protocol == TPL_PROT_TCP)
			break;
#else
		if (protocol == TPL_PROT_UDP)
		{
			if (tpl_bindReq(bindCBp, 
							&tplRspBindUdp, 
							bindOpt) != NFX_OK)
				panic("nlm_tplBindReq: bind failure (UDP)!\n");
			nlm_globalCfgp->nlm_pTplBindUdp = tplRspBindUdp;
			nlm_globalCfgp->nlm_pBsdSockUdp = tplRspBindUdp->pBsdSock;

#ifdef	NLM_DEBUG
			NLM_DBG_FUNC_PRINTF("%s: nlm_pBsdSockUdp: %p\n", 
					__FUNCTION__, 
					nlm_globalCfgp->nlm_pBsdSockUdp);
#endif

		}
		else
		{
			if (tpl_bindReq(bindCBp, &tplRspBindTcp, bindOpt) != NFX_OK)
				panic("nlm_tplBindReq: bind failure (TCP)!\n");
			nlm_globalCfgp->nlm_pTplBindTcp = tplRspBindTcp;
			nlm_globalCfgp->nlm_pBsdSockTcp = tplRspBindTcp->pBsdSock;
#ifdef	NLM_DEBUG
			NLM_DBG_FUNC_PRINTF("%s: nlm_pBsdSockTdp: %p\n", 
						__FUNCTION__, 
						nlm_globalCfgp->nlm_pBsdSockTcp);
#endif
			break;

		}
#endif /* NFX_SMP */

		protocol = TPL_PROT_TCP;
	}

	return (NFX_OK);
}
#endif  //NFP_TXRX
