/* vi:ts=4
 *-----------------------------------------------------------------
 *
 * Name:        nlm.c
 *
 * RCS:  $Id: nlm.c,v 1.9 2002/11/27 21:27:46 joseph Exp $
 *
 * Copyright (C) 2001, Agile Storage Inc.
 *
 * Description:
 *	Contain Network Lock Manager (NLM)/Transport Layer Protocol (TPL)
 *	interface functions.
 *
 * Created by: Joseph Vu
 *
 * Date Created:        6/28/02
 *
 *-----------------------------------------------------------------
 */
#ifdef NFP_TXRX

#include <nfx-incl.h>
#include <exports/pkt-api.h>
#include <exports/vs-mem-api.h>
#include <exports/lock-api.h>
#include <exports/nfs-api.h>
#include "nlm-prototypes.h"

#include <sys/machine-dep.h>

#define NLM_DBG_PRINTF  if (nlm_traceEnable) printf	
//#define	NLM_DEBUG

#define NLM_DBG_FUNC_PRINTF if (nlm_traceDebugEnable) printf

//nlm_portMapCb_t	nlm_portMapCB;


extern int32 nfs_addCBToList(nfp_list_t *nodeListp,  
    void *CtrlBlkp, uint32 nodeType);
extern int32 nfs_delNode(nfp_list_t *nodeListp, nfp_nodePtr_t delNodep);


/*
 *---------------------------------------------------------------
 * Function Name:    nlm_tplAllocPkt  		
 *
 * Description:
 * This function is called by NLM transmit functions to 
 * Allocate a packet from the Transport Layer (TPL). 
 * This function will try to delete
 * an oldest request entry from the duplicate 
 * request cache to service
 * new request if it cannot allocate packet from TPL.
 *
 * Created by:          Joseph Vu
 *
 * Created:             7/6/2002
 *
 *
 *------------------------------------------------------------------
 */
#ifdef	NOTUSED
pkt_desc_t *nlm_tplAllocPkt(
	nfp_clientCBPtr_t	clientCBp, 
	nlm_request_t 		*nlmReqp,	
	int32				usrDataLen)
{

	pkt_desc_t          *tplXmitPktDescp;
	void				*tplConnp;
	nfp_nodePtr_t		tailp;
	int32				done;
	int32				count;

	if ( (clientCBp == NULL) || (nlmReqp == NULL) )
		return (NULL);
	tplConnp = nlmReqp->tplConnp;
	done = 0;
	count = clientCBp->nfp_nlmReqList.nfp_reqCnt;
	tplXmitPktDescp = TPL_ALLOC_PACKET(tplConnp, usrDataLen);	
	if (tplXmitPktDescp == NULL)
	{
		/*
	 	 *Removes oldest entry in the duplicate reqs cache 
		 *to service new request.
	 	 */
		for (tailp = clientCBp->nfp_nlmReqList.nfp_tailp; 
			(tailp != NULL && count > 0);
			tailp = tailp->nfp_prevp, count--)
		{
			nlmReqp = (nlm_request_t *)tailp->nfp_entryCBp;
			if (nlmReqp->reqState == NFS_REQ_DONE)
			{
				nlm_freeAll(nlmReqp, TRUE);
				break;
			}
		}		
	}
	else
	{	
		tplXmitPktDescp->dataSize = 0;
		nlm_globalCfgp->nlm_debugCounters.nlm_tplXmtPktAlloc++;
		return (tplXmitPktDescp);
	}

	/*
	 *Try again
	 */ 
	tplXmitPktDescp = TPL_ALLOC_PACKET(tplConnp, usrDataLen);	
	if (tplXmitPktDescp != NULL)
		tplXmitPktDescp->dataSize = 0;
	nlm_globalCfgp->nlm_debugCounters.nlm_tplXmtPktAlloc++;
	return (tplXmitPktDescp);
}

#endif	//NOTUSED

/*
 *-----------------------------------------------------------------------
 * Function Name:   nlm_freeAll  		
 *
 * Description:        
 * This function called after received a transmit 
 * complete indicdation from 
 * Transport Layer (TPL) to de-allocate all previous 
 * allocated memory for this request. 
 *
 * Created by:          Joseph Vu
 *
 * Created:             6/28/2002
 *
 *
 *----------------------------------------------------------------------
 */
void
nlm_freeAll(nlm_request_t *nlmReqp, int32 freeTxPktDescFlag)
{
//	int32			rc;
	void            *ra;

	ra = (void *)get_ra();
	if (nlmReqp->reqState != NLM_REQ_DONE)
	{
		panic ("%s: Caller (%p) not in valid state!\n", 
				__FUNCTION__, ra);
	}


	if ((nlmReqp->nlm_lockInformation.nlm_callerNameLen != 0) &&
		(nlmReqp->nlm_lockInformation.nlm_callerName) )
	{

		eee_ramDealloc(nlmReqp->nlm_lockInformation.nlm_callerName);	
		nlmReqp->nlm_lockInformation.nlm_callerName = NULL;
		nlmReqp->nlm_lockInformation.nlm_callerNameLen = 0;
		nlm_globalCfgp->nlm_debugCounters.nlm_freeCallerName++;
	}

	if ((nlmReqp->nlm_lockInformation.nlm_ohLen != 0) &&
		(nlmReqp->nlm_lockInformation.nlm_oh) )
	{
		eee_ramDealloc(nlmReqp->nlm_lockInformation.nlm_oh);	
		nlmReqp->nlm_lockInformation.nlm_ohLen = 0;
		nlmReqp->nlm_lockInformation.nlm_oh = NULL;
		nlm_globalCfgp->nlm_debugCounters.nlm_freeOh++;
	}

	if ( (nlmReqp->cookie.netobjLen != 0)  &&
		 (nlmReqp->cookie.netobj_p) )
	{
		eee_ramDealloc( nlmReqp->cookie.netobj_p);
		nlmReqp->cookie.netobjLen = 0;
		nlmReqp->cookie.netobj_p = NULL;
		nlm_globalCfgp->nlm_debugCounters.nlm_freeCookie++;
	}
#ifdef	NOTUSED

	/*
	 *Free pkt at nlm_tplXmtCmpInd() instead.
	 *Free TPL Tx request packet
	 */	
	if ((nlmReqp->txPktDescp != NULL) && freeTxPktDescFlag)
	{

		rc = TPL_FREE_PACKET(nlmReqp->tplConnp, nlmReqp->txPktDescp);
		if (rc != NFX_OK)
		{
			NLM_DBG_FUNC_PRINTF("nlm_freeAll: Free TPL Tx req Pkt failed!\n");
		}

		nlm_globalCfgp->nlm_debugCounters.nlm_tplXmtPktFree++;
		nlmReqp->txPktDescp = NULL;
	}


#endif	//NOTUSED

#ifdef	NOTUSED
	/*
	 *Delete NLM request control block from  client list
	 */
	if (nlm_delReqCBFromList(nlmReqp) != NFX_OK)
	{
		/*
		 *TODO:  Handle error
		 */
		NLM_DBG_FUNC_PRINTF("nlm_freeAll: nlm_delReqCBFromList failed!\n");
	}

#endif	//NOTUSED


	/*
	 *De-alloc rquest control block
	 */
	if (nfp_deallocReqCB((nfp_requestCBPtr_t)nlmReqp->reqCBp) == NFX_OK)
	{
		nlm_globalCfgp->nlm_debugCounters.nlm_freeReqCB++;
	}


	return;
}

/*
 *------------------------------------------------------------------
 * Function Name:      	nlm_reqTimeoutHandler 
 *
 * Description:
 *	Periodic timeout function to delete oldest complete 
 *  request entry from the Duplicate request cache.
 *
 * Created by:          Joseph Vu
 *
 * Created:             7/14/2002
 *
 *
 *------------------------------------------------------------------
 */

int32
nlm_reqTimeoutHandler(int32 timer_handle, 
				void* param1, void* param2, 
				void* param3, void* param4)
{
	nlm_request_t		*nlmReqp;
	nfp_nodePtr_t		tailp;
	nfp_clientCBPtr_t   clientCBp;
	uint32              count;
	int32				iterations;
	int32				rc;	

	count = 0;
    for (clientCBp= 
			(nfp_clientCBPtr_t)nfp_globalCfgp->clientCbCfgList.listBasep;
        ((count < NFP_MAX_CLIENT) && 
		 (clientCBp != NULL)); count++, clientCBp++)
	{
		if (!(clientCBp->nfp_flag & NFP_ENTRY_INUSED))
			continue;
	
		iterations = clientCBp->nfp_nlmReqList.nfp_reqCnt;
		for (tailp = clientCBp->nfp_nlmReqList.nfp_tailp;
			(tailp != NULL && iterations > 0);
			tailp = tailp->nfp_prevp, iterations--)
		{
			if (nfp_validateNode(tailp) != NFX_OK)
				continue;
			nlmReqp = (nlm_request_t  *)tailp->nfp_entryCBp;
#ifdef	NOTUSED
			if (nlmReqp->reqState == NLM_REQ_DONE)
			{
				nlm_freeAll(nlmReqp, TRUE);
			}
#endif
			if (nlmReqp->reqState == NLM_GETPORT_CALL_SND) 
			{
				if ( nlmReqp->nlm_txCmdInfo.getportRetry < NLM_MAX_RETRY_CNT )
				{
					rc = nlm_procTxGetportReq(nlmReqp);
					nlm_globalCfgp->nlm_debugCounters.numTxGetportRetries++;
					continue;
				}
				//We've reach max retry for this request

				nlmReqp->nlm_txCmdInfo.getportRetry = 0;
				nlmReqp->nlm_txCmdInfo.getportMaxTimeWaitCnt = 0;
				nlmReqp->reqState = NLM_REQ_DONE;
#ifdef NOT_USED
				nlm_delReqCBFromList(nlmReqp);
#endif
				nlm_freeAll(nlmReqp, TRUE);
			}
			else if ( nlmReqp->reqState == NLM_GRANTED_CALL_SND) 
			{
				if ( nlmReqp->nlm_txCmdInfo.grantedRetry < NLM_MAX_RETRY_CNT )
				{
					rc = nlm_procTxGranted(nlmReqp);
					nlm_globalCfgp->nlm_debugCounters.numTxGrantedRetries++;
					continue;
				}

				//We've reach max retry for this request

				nlmReqp->nlm_txCmdInfo.grantedRetry = 0;
				nlmReqp->nlm_txCmdInfo.grantedMaxTimeWaitCnt = 0;
				nlmReqp->reqState = NLM_REQ_DONE;
				nlm_delReqCBFromList(nlmReqp);
				nlm_freeAll(nlmReqp, TRUE);
			}
		}
    }


	return (NFX_OK);
}

/*
 *------------------------------------------------------------------
 * Function Name:      nlm_extractFhandle 
 *
 * Description:
 *	NLM extract filehandle from client message buffer.
 *
 * Created by:          Joseph Vu
 *
 * Created:             7/14/2002
 *
 *
 *------------------------------------------------------------------
 */

int32	/*
	 	 *NFX_ERR: Failure to extract file handle 
	 	 *length:  Filehandle length in bytes
	 	 */
nlm_extractFhandle(
	char *nfsMsgp,		
	fs_Handle *fhandlep	
)
{
	int32		ourFhandleLen;
	uint32		*msgp;
	uint32		fhandleLen;
	uint32		fieldLen;

	ourFhandleLen = sizeof(fs_Handle);

		
	msgp = (int32 *)nfsMsgp;
	fhandleLen = ntohl(*msgp++);
	fieldLen = sizeof(uint32);
	if (fhandleLen != ourFhandleLen)
		return (NFX_ERR);

	memcpy(fhandlep, (char *)msgp, fhandleLen);

	xdr_swap((byte *)fhandlep, fhandleLen, 0);
	fieldLen += fhandleLen;
	return (fieldLen);
		
}

/*
 *---------------------------------------------------------------
 * Function Name:   nlm_extractAlockInfo   		
 *
 * Description:
 *	Extract nlm_lock struct fields:
 *		string<1024> caller_name: name of client machine
 *		fileHandle:  identity of the file
 *		lock identity:  identity of the lock owner
 *		int32 svid:  unique process ID
 *		uint32/64 l_offset:  offset of locked region
 *		uint32/64 l_len:  extent of locked region
 *	
 * Created by:          Joseph Vu
 *
 * Created:             7/14/2002
 *
 *
 *-----------------------------------------------------------------
 */
int32		//len=bytes of alock info
			//or NFX_ERR 
nlm_extractAlockInfo(char *nlm_msgp, 
					nlm_lockInfo_t *alockp,
					int32 nlm_version,
					fs_Handle *fileHandlep)
{
	int32		totalLen;
	int32		nfs_version;
	int32		length;

	totalLen = 0;

    //Map NLM version to comparable
    //NFS version

	NLM_MAP_TO_COMP_NFS_VER(nfs_version, nlm_version);

	//Get caller Name

	length =  nfs_extractString(nlm_msgp,
				alockp->nlm_callerName, 
				(ushort16 *)&alockp->nlm_callerNameLen, NLM_MAX_NAME);
	if (length < 0)
		return (NFX_ERR);

	//update message ptr
	
	nlm_msgp += length;
	totalLen += length;

	//Get fileHandle
	
	length = nlm_extractFhandle( nlm_msgp, fileHandlep);
	if (length == NFX_ERR)
		return (NFX_ERR);
	
	//update message ptr.
	
	nlm_msgp += length;
	totalLen += length;

	//Get owner Id data 
	
	length =  nfs_extractString(nlm_msgp,
				alockp->nlm_oh, 
				(ushort16 *)&alockp->nlm_ohLen, NLM_MAX_NAME);
	if (length < 0)
		return (NFX_ERR);
	
	//update message ptr
	
	nlm_msgp += length;
	totalLen += length;

	//Get SVID
	
	length = nfs_extractLongVal(nlm_msgp, &alockp->nlm_svid);	

	//update msg field
	
	nlm_msgp += length;
	totalLen += length;

	//Get file offset
	
	if (nfs_version == NFS_VER2)
	{
		uint32		l_offset32, l_len32;

		length = nfs_extractLongVal(nlm_msgp, &l_offset32);
#ifdef NOT_USED
		alockp->lockRec.nlm_l_offset = l_offset32;	
#else
		alockp->nlm_l_offset = l_offset32;	
#endif

		//update msg field
		nlm_msgp += length;
		totalLen += length;

		//Get length
		length = nfs_extractLongVal(nlm_msgp, &l_len32);
		totalLen += length;
#ifdef NOT_USED
		alockp->lockRec.nlm_l_len = l_len32;
#else
		alockp->nlm_l_len = l_len32;	
#endif
	}
	else
	{
#ifdef NOT_USED
		length = nfs_extractLongLongVal(nlm_msgp, 
						&alockp->lockRec.nlm_l_offset);
#else
		length = nfs_extractLongLongVal(nlm_msgp, &alockp->nlm_l_offset);
#endif
		//update msg field
		nlm_msgp += length;
		totalLen += length;

		//Get length
#ifdef NOT_USED
		length = nfs_extractLongLongVal(nlm_msgp, 
						&alockp->lockRec.nlm_l_len);
#else
		length = nfs_extractLongLongVal(nlm_msgp, &alockp->nlm_l_len);
#endif
		totalLen += length;
	}

	return (totalLen);
}

/*
 *---------------------------------------------------------------
 * Function Name:   nlm_storeAlockInfo   		
 *
 * Description:
 *	Stores nlm_lock information fields into response message to
 *	be returned to the client.
 *		bool exlusive
 *		int32 svid:  unique process ID
 *		lock identity:  identity of the lock owner (oh)
 *		uint32/64 l_offset:  offset of locked region
 *		uint32/64 l_len:  extent of locked region
 *	
 * Created by:          Joseph Vu
 *
 * Created:             7/14/2002
 *
 *
 *-----------------------------------------------------------------
 */
#ifdef NOT_USED
int32
nlm_storeAlockInfo( char *nlm_txMsgp,
                    nlm_lockRec_t *lockRecp,
                    int32 nlm_version)
{   
    nfp_lockCBPtr_t lockCBp;
    nlm_lockInfo_t  *nlm_lockInfop;
    char            *msgp;
    int32           totalLen;
    int32           length;
    int32           nfs_version;

#ifdef  NLM_VALIDATE_LOCK   
    if (nlm_validateLockRec(lockRecp) != NFX_OK)
        panic ("%s: Invalid lockRecp %p\n",
                            __FUNCTION__,lockRecp);
#endif

    lockCBp = (nfp_lockCBPtr_t)lockRecp->lockCBp;
#ifdef  NLM_VALIDATE_LOCK
    if (nfp_validateLockCB(lockCBp) != NFX_OK)
    {
        panic("%s: Bad lockCBp: %p\n", __FUNCTION__, lockCBp);
    }
#endif
    nlm_lockInfop = &lockCBp->protocol.nfp_nlmLock;

    totalLen = 0;
    msgp = nlm_txMsgp;

    //Map NLM version to comparable 
    //NFS version
    //nfs_version = nlm_utilMapToCompNFSVer(nlm_version);

    NLM_MAP_TO_COMP_NFS_VER(nfs_version, nlm_version);

    //exclusive
    length = nfs_storeLongValToBuf( msgp,
                                    lockRecp->nlm_exclusive );
    msgp += length;
    totalLen += length;

    //svid 
    length = nfs_storeLongValToBuf( msgp,
                                    nlm_lockInfop->nlm_svid );
    msgp += length;
    totalLen += length;

    length = nfs_storeString(msgp,
                            nlm_lockInfop->nlm_oh, 
                            nlm_lockInfop->nlm_ohLen,
                            NLM_MAX_NAME);
    msgp += length;
    totalLen += length;

    if (nfs_version == NFS_VER2)
    {
        //32 bits l_offset
        length = nfs_storeLongValToBuf( msgp,
                                        (uint32)lockRecp->nlm_l_offset );
        msgp += length;
        totalLen += length;

        //32 bits l_len
        length = nfs_storeLongValToBuf( msgp,
                                        (uint32 )lockRecp->nlm_l_len );
        msgp += length;
        totalLen += length;
    }
    else
    {
        //NFS V3
        //64 bits l_offset
        length = nfs_storeLongLongValToBuf( msgp,
                                            lockRecp->nlm_l_offset );
        msgp += length;
        totalLen += length;

        //64 bits l_len
        length = nfs_storeLongLongValToBuf( msgp,
                                            lockRecp->nlm_l_len );
        msgp += length;
        totalLen += length;

    }
    return (totalLen);
}

/*
 *---------------------------------------------------------------
 * Function Name:  nlm_saveLockInfo 
 *
 * Description:
 *	Save lock information from source to destination structure.
 *	
 * Created by:          Joseph Vu
 *
 * Created:             7/14/2002
 *
 *
 *-----------------------------------------------------------------
 */

int32
nlm_saveLockInfo(	nfp_lockCB_t	*dest_lockCBp,
					nlm_lockRec_t	*dest_lockRecp,	
	   				nlm_lockInfo_t *src_lockInfop)
	   
{
	nlm_lockInfo_t *dest_lockInfop;
	uchar8 			*allocBufp;
	int32			allocSize;

#ifdef	NLM_VALIDATE_LOCK
	if (nfp_validateLockCB(dest_lockCBp) != NFX_OK)
	{
		panic("%s: Bad lockCBp: %p\n", __FUNCTION__, dest_lockCBp);	
	}

	if (nlm_validateLockRec(dest_lockRecp) != NFX_OK)
		panic ("%s: Invalid dest_lockRecp %p\n",
						   	__FUNCTION__,dest_lockRecp);


#endif
	dest_lockInfop = &dest_lockCBp->protocol.nfp_nlmLock;
	dest_lockRecp->lockCBp = dest_lockCBp; 
	/*----------------------------------------
	 *save specific info into new lock record
	 *----------------------------------------
	 */	
	dest_lockRecp->nlm_l_offset = 
			src_lockInfop->lockRec.nlm_l_offset;
	dest_lockRecp->nlm_l_len = 
			src_lockInfop->lockRec.nlm_l_len;

	NLM_GET_L_REGION_END(dest_lockRecp->nlm_l_end,
						dest_lockRecp->nlm_l_offset,
						dest_lockRecp->nlm_l_len,
						dest_lockRecp->nlm_l_eof);   
	
	dest_lockRecp->nlm_exclusive = 
			src_lockInfop->lockRec.nlm_exclusive;
	dest_lockRecp->nlm_state = src_lockInfop->lockRec.nlm_state;
 
	/*--------------------------------
	 *save common lock owner info.
	 *svid: process Id,
	 *Oh: lock owner &
	 *callerName
	 *--------------------------------
	 */	
	dest_lockInfop->nlm_svid = src_lockInfop->nlm_svid;	
	dest_lockInfop->nlm_callerNameLen = src_lockInfop->nlm_callerNameLen;
	dest_lockInfop->nlm_ohLen = src_lockInfop->nlm_ohLen;

	allocSize = src_lockInfop->nlm_callerNameLen + 1;
	allocBufp = (uchar8 *)eee_ramAlloc(allocSize);
	if (allocBufp == NULL)
		return (NFX_ERR);
	bzero(allocBufp, allocSize);
	memcpy(allocBufp, src_lockInfop->nlm_callerName,
			src_lockInfop->nlm_callerNameLen);
	nlm_globalCfgp->nlm_debugCounters.nlm_allocCallerName++;
	dest_lockInfop->nlm_callerName = allocBufp;

	allocSize = src_lockInfop->nlm_ohLen + 1;
	allocBufp = (uchar8 *)eee_ramAlloc(allocSize);
	if (allocBufp == NULL)
	{
		eee_ramDealloc(dest_lockInfop->nlm_callerName);
		return (NFX_ERR);
	}

	nlm_globalCfgp->nlm_debugCounters.nlm_allocOh++;
	bzero(allocBufp, allocSize);
	memcpy (allocBufp, src_lockInfop->nlm_oh,
			src_lockInfop->nlm_ohLen);
	dest_lockInfop->nlm_oh = allocBufp;


	return (NFX_OK);
}

/*
 *---------------------------------------------------------------
 * Function Name: nlm_getConflictinglock 
 *
 * Description:
 *	Check incoming alock request for conflict by checking
 *	both the current active locks list and the pending
 *	locks list.
 * 
 * Created by:          Joseph Vu
 *
 * Created:             7/14/2002
 *
 *
 *-----------------------------------------------------------------
 */
nlm_lockRec_t *	/*Pointer to found LockRec or NULL*/
nlm_getConflictinglock(	nfp_fileCB_t	*fileCBp,
						nlm_lockInfo_t	*nlm_lockInfop, 
						short16			searchMask,
						nlm_lockRec_t	**nextLockRecpp,
						uint32			*listType)
{
	nlm_lockRec_t 		*lockRecp;	
	nlm_lockRec_t 		*ret_nextLockRecp;

	lockRecp = NULL;
	*nextLockRecpp = NULL;
	/*-------------------------
	 *First check current lock list
	 *for conflict
	 *-------------------------
	 */
	lockRecp =   nlm_getLockRecInCurrList(fileCBp,
										nlm_lockInfop, 
										searchMask, 
										&ret_nextLockRecp);  

	*nextLockRecpp = ret_nextLockRecp;
	*listType = NLM_CURRENT_LIST;
	if (lockRecp != NULL)
	{
#ifdef	NLM_DEBUG
		
		printf("%s 1: conlict with active lockRecp: %p offset: %lld len: %lld\n", 
			__FUNCTION__,
			lockRecp,
			lockRecp->nlm_l_offset,
			lockRecp->nlm_l_len);
#endif

		return (lockRecp);
	}
#ifdef	NLM_DEBUG
		printf("%s: 2:*nextLockRecpp: %p \n",
					   	__FUNCTION__, *nextLockRecpp);
#endif

	/*-------------------------
	 *check pending lock list
	 *for conflict
	 *-------------------------
	 */
	lockRecp =  nlm_getLockRecInPendingList(fileCBp,
											nlm_lockInfop, 
											searchMask, 
											&ret_nextLockRecp);  
#ifdef	NLM_DEBUG	
	if (lockRecp)
		printf("%s 3: conlict with pending lockRecp: %p offset: %lld len: %lld\n", 
			__FUNCTION__,
			lockRecp,
			lockRecp->nlm_l_offset,
			lockRecp->nlm_l_len);

#endif

	if (lockRecp && ret_nextLockRecp)
	{
		/*------------------------------
	 	 *Update next node if conflict
	 	 *with a pending lock entry
	 	 *-------------------------
	 	 */
		*listType = NLM_PENDING_LIST;
		*nextLockRecpp = ret_nextLockRecp;
#ifdef	NLM_DEBUG
		printf("%s: 4:*nextLockRecpp: %p \n",
					   	__FUNCTION__, *nextLockRecpp);
#endif

	}

#ifdef	NLM_DEBUG
		printf("%s: exit:lockRecp: %p *nextLockRecpp: %p\n",
					   	__FUNCTION__, 
						lockRecp,
						*nextLockRecpp);
#endif

	return (lockRecp);
}

/*
 *---------------------------------------------------------------
 * Function Name:  nlm_isLockReqCauseDeadlock
 *
 * Description:
 *	Check for deadlock before allowing 
 *	the incoming lock request to be blocked. 
 * 
 * Created by:          Joseph Vu
 *
 * Created:             7/14/2002
 *
 *
 *-----------------------------------------------------------------
 */
int32	/*1: deadlock or 0: no deadlock*/
nlm_isLockReqCauseDeadlock(	nlm_lockRec_t 	*currHolderLockRecp,
							nlm_lockInfo_t	*nlm_lockInfop)
{
	nfp_lockCBPtr_t		currHolderlockCBp;
	nlm_lockRec_t		*nextLockRecp;
	nlm_lockRec_t		*currPendingLockRecp;
	nlm_lockRec_t		*waitForLockRecp;
	nfp_lockCB_t 		*reqLockCBp;
	nlm_lockInfo_t		*reqLockp;
//	nfp_lockCB_t 		*currLockCBp;
	nlm_lockInfo_t		*currHolderalockp;
	int32				lockCount;

	if ( (currHolderLockRecp == NULL) || 
		 (nlm_lockInfop == NULL) ) 
		return 0;

	currHolderlockCBp = (nfp_lockCBPtr_t)currHolderLockRecp->lockCBp;

	/*----------------------------------------
	 *Check if incoming lock request own
	 *any locks.
	 *----------------------------------------
	 */
	reqLockCBp = nlm_getLockCBOwner(currHolderlockCBp->nfp_fileCbp,
									nlm_lockInfop);	
	if  (reqLockCBp == NULL)
		return (0);
	reqLockp = &reqLockCBp->protocol.nfp_nlmLock;

	lockCount = 0;
	currHolderalockp = &currHolderlockCBp->protocol.nfp_nlmLock;
	currPendingLockRecp = 
			nlm_getLockRecInPendingList(currHolderlockCBp->nfp_fileCbp,
								currHolderalockp,
								(NLM_SEARCH_OH | NLM_SEARCH_SVID),
								&nextLockRecp);

	
	while ( (currPendingLockRecp &&
			currPendingLockRecp->waitForLockRecp) &&
			(lockCount++ < NLM_MAX_LOCKS_PER_FILE))
	{
		waitForLockRecp = (nlm_lockRec_t *)currPendingLockRecp->waitForLockRecp;
		currHolderlockCBp = (nfp_lockCBPtr_t)waitForLockRecp->lockCBp;
		if (currHolderlockCBp == NULL)
			break;
		currHolderalockp = &currHolderlockCBp->protocol.nfp_nlmLock;

		if ((currHolderalockp->nlm_ohLen == reqLockp->nlm_ohLen) && 
			(memcmp(currHolderalockp->nlm_oh, reqLockp->nlm_oh, 
					reqLockp->nlm_ohLen) == 0) && 
				(currHolderalockp->nlm_svid == reqLockp->nlm_svid) )
			return (1);

		currPendingLockRecp = 
			nlm_getLockRecInPendingList(currHolderlockCBp->nfp_fileCbp,
								currHolderalockp,
								(NLM_SEARCH_OH | NLM_SEARCH_SVID),
								&nextLockRecp);


	}


	return (0);
}

/*
 *---------------------------------------------------------------
 * Function Name: nlm_checkforConflictLock	 
 *
 * Description:
 * Check for conflicting lock on a file
 * based on byte offset lock region.
 * 
 * Created by:          Joseph Vu
 *
 * Created:             7/14/2002
 *
 *
 *-----------------------------------------------------------------
 */
int32
nlm_checkforConflictLock(	nlm_request_t 		*nlmReqp,
							nfp_fileCBPtr_t 	fileCBp,
							nlm_lockInfo_t		*nlm_lockInfop,
							uint64				fsize,
							char 				*cookiep,
							ushort16 			cookieLen)
{
	nlm_lockRec_t 		*conflictLockRecp;
	nlm_lockRec_t 		*lockRecp;
	nlm_lockRec_t 		*nextLockRecp;
	nfp_lockCB_t 		*lockCBp;
	short16				searchMask;
	int32				error;
	int32				rc;
	uint32				listType;

	error = NLM_FAILED;
	nextLockRecp = NULL;

	/*--------------------------------------
	 *There're locks on this file
	 *Check if request lock is conflict
	 *with current existing locks 
	 *or pending ( wait) locks
 	 *--------------------------------------
 	 */

	searchMask = 0;
	if (nlm_lockInfop->lockRec.nlm_exclusive == FALSE) 
		searchMask = NLM_SEARCH_EXCLUSIVE;

	conflictLockRecp = nlm_getConflictinglock(	fileCBp,
												nlm_lockInfop, 
												searchMask,
												&nextLockRecp,
												&listType);

	if (conflictLockRecp != NULL)
	{
		nlm_lockInfo_t		*lockInfop;

		nlmReqp->lockRecp = conflictLockRecp;
		lockCBp = (nfp_lockCBPtr_t)conflictLockRecp->lockCBp;

		if (nfp_validateLockCB(lockCBp) != NFX_OK)
		{
#ifdef	NLM_VALIDATE_LOCK
			panic("%s: invalid lockCBp %p\n", __FUNCTION__,lockCBp);
#endif	
			return (NFX_ERR);
		}
		lockInfop = &lockCBp->protocol.nfp_nlmLock;
		if ( ((lockInfop->nlm_ohLen == nlm_lockInfop->nlm_ohLen) &&
			(!memcmp(lockInfop->nlm_oh, 
					 nlm_lockInfop->nlm_oh, 
					 nlm_lockInfop->nlm_ohLen)) ) &&
			(conflictLockRecp->nlm_l_offset == 
			 nlm_lockInfop->lockRec.nlm_l_offset) &&
		    (conflictLockRecp->nlm_l_end == 
			 		nlm_lockInfop->lockRec.nlm_l_end) )
		{
			/*-----------------------------------------------
			 *Entry  is already on our list. 
			 *Cannot wait for itself.
			 *return NLM_GRANTED if entry on active list
			 *or NLM_ERR_BLOCKED if entry is on pending list
	 	 	 *-----------------------------------------------
	 	 	 */
#ifdef	NLM_DEBUG
			printf("%s: lock request is already on the list.  svid:%d offset:%lld len: %lld\n",
				__FUNCTION__,
				lockInfop->nlm_svid, 
				conflictLockRecp->nlm_l_offset,
				conflictLockRecp->nlm_l_end);
#endif	


			if (listType == NLM_PENDING_LIST)
			{
				return (NLM_ERR_BLOCKED);
			}
				
			/*--------------------------------------
			 *Lock request has already been granted.
			 *return NLM_GRANTED
	 	 	 *--------------------------------------
	 	 	 */
			if (nlmReqp->procNum == NLM_LOCK)
				//Response to synchronous request
				rc = nlm_procTxSendResp(nlmReqp, 
										NLM_STAT_GRANTED,
										cookiep, 
										cookieLen);
			else
				//response to async. request
				rc = nlm_procTxSendASyncMsgResp(nlmReqp, 
												NLM_STAT_GRANTED,
												cookiep, 
												cookieLen);
			
				
			return (rc);

		}

		/*-----------------------------------------
		 *Conflicting lock.
		 *if Block is requested, add to pending list
		 *otherwise, return NLM_DENIED status
 	 	 *------------------------------------------
 	 	 */
#ifdef	NLM_DEBUG	
		printf("%s 1:conflicting with lockrecp:%p. Oh:%s offset:%lld len:%lld\n",
				__FUNCTION__, 
			conflictLockRecp,
			lockInfop->nlm_oh,
			conflictLockRecp->nlm_l_offset,
			conflictLockRecp->nlm_l_len);
#endif


		if (nlmReqp->block)
		{
			/*-------------------------------
			 *Before we can block, we
			 *need to check for deadlock 
			 *--------------------------------
			 */

			if (nlm_isLockReqCauseDeadlock(conflictLockRecp, 
											nlm_lockInfop) )
			{
				nlm_globalCfgp->nlm_debugCounters.numOfDeadlockErr++;
				return (NLM_ERR_DEADLCK);
			}

			/*------------?------------------------------
			 *Add lock request entry to the pending list
			 *-------------------------------------------
			 */
#ifdef	NLM_DEBUG
			printf("%s: conflictLockRecp: %p listType: %d nextLockRecp: %p\n",
					   	__FUNCTION__,
						conflictLockRecp,
						listType,
						nextLockRecp);


#endif
			if (listType != NLM_PENDING_LIST)
				nextLockRecp = NULL;
			lockRecp = nlm_addLockRecToPendingList(	nlmReqp->clientCBp,
						   							fileCBp,
													nlm_lockInfop,
													nextLockRecp);
			if (lockRecp == NULL)
				return (NFX_ERR);

			rc = nlm_setPendingLockEntry(conflictLockRecp, lockRecp);
			if (rc != NFX_OK)
				return (NFX_ERR);


			return (NLM_ERR_BLOCKED);
		}
		else
		{

			/*Not blocked, return NLM_DENIED status*/
			return (NLM_ERR_DENIED);
		}

	}
	else
	{

		/*------------------------------------------
		 *No conflicting lock
		 *Create new lock and add
		 *to current lock list of
		 *File Control Block
		 *-------------------------------------------
		 */
		lockRecp = nlm_addLockRecToCurrList(nlmReqp->clientCBp,
					   						fileCBp,
											nlm_lockInfop,
											nextLockRecp,
											0,
											NULL);
		if (lockRecp)
			goto nlm_checkforConflictLockExit;
	}
	return (error);

nlm_checkforConflictLockExit:
	if (nlmReqp->procNum == NLM_LOCK)
		/*Response to synchronous request*/
		rc = nlm_procTxSendResp(nlmReqp, 
								NLM_STAT_GRANTED,
								cookiep, 
								cookieLen);
	else
		/*Response to Asynchronous request*/
		rc = nlm_procTxSendASyncMsgResp(nlmReqp, 
										NLM_STAT_GRANTED,
										cookiep, 
										cookieLen);

	return (rc);

}

/*
 *---------------------------------------------------------------
 * Function Name:	nlm_setPendingLockEntry		
 *
 * Description:
 *	Find the current lock holder to insert 
 *	the waiting pending lock.
 *				 
 * Created by:          Joseph Vu
 *
 * Created:             7/14/2002
 *
 *
 *-----------------------------------------------------------------
 */

int32
nlm_setPendingLockEntry(nlm_lockRec_t	*lockRecp,
						nlm_lockRec_t	*pendingLockRecp)
{
	int32			lockCount;
	nlm_lockRec_t	*currLockRecp;
	nlm_lockRec_t	*nextLockRecp;

	lockCount = 0;
#ifdef	NLM_DEBUG
	printf("%s 1: lockRecp: %p lockRecp->pendingLockRecp: %p\n",
				   	__FUNCTION__,
					lockRecp, 
					lockRecp->pendingLockRecp);
#endif


	currLockRecp = nextLockRecp = lockRecp;
	while ( (nextLockRecp != NULL) &&
			(lockCount++ < NLM_MAX_LOCKS_PER_FILE ) )
	{
		currLockRecp = nextLockRecp;
		nextLockRecp = nextLockRecp->pendingLockRecp;
	}
	if ( (nextLockRecp != NULL) ||
		(lockCount >= NLM_MAX_LOCKS_PER_FILE ) )
	{
		nlm_globalCfgp->nlm_debugCounters.numOfAddPendingLckErr++;
		return (NFX_ERR);
	}

	currLockRecp->pendingLockRecp = pendingLockRecp;
	pendingLockRecp->waitForLockRecp = currLockRecp;



#ifdef	NLM_DEBUG
	printf("%s exit: lockRecp: %p lockrecp->pendingLockRecp :%p lockCount:%d\n",
				   	__FUNCTION__,
					currLockRecp, 
					currLockRecp->pendingLockRecp,
					lockCount);
#endif


	return (NFX_OK);
}

/*
 *------------------------------------------------------------------
 * Function Name:  	nlm_addNewLock	
 *
 * Description:
 * Handle processing for adding a new lock owner.
 *
 *  
 * Created by:          Joseph Vu
 *
 * Created:             7/30/2002
 *
 *
 *------------------------------------------------------------------
 */
int32
nlm_addNewLock(	nfp_fileCBPtr_t fileCBp, 
				nfp_lockCBPtr_t	lockCBp,
				nfp_nodePtr_t nextLockNodep,
				nfp_clientCBPtr_t clientCBp,
				fs_Handle *fhandlep)
{
	int32			rc;

	
	if  ((fileCBp == NULL) || (clientCBp == NULL) || 
	    (lockCBp == NULL)	)
		return (NFX_ERR);
#ifdef	NLM_VALIDATE_LOCK	
	if (nfp_validateLockCB(lockCBp) != NFX_OK)
	{
		panic("%s: Bad lockCBp: %p\n", 
						__FUNCTION__, lockCBp);		
	}
#endif

	lockCBp->nfp_fileCbp = fileCBp;	
	lockCBp->nfp_clientCbp = clientCBp;
	if (!(fileCBp->nfp_flag & NFP_ENTRY_ADDED))
	{	
		rc = nfp_addFileCB(fileCBp, fhandlep);
		if (rc != NFX_OK)
		{
			
			return (NFX_ERR);
		}

		E_LOG (class_1, LOG_DEBUG, 
			nlm_globalCfgp->eee_nlmAppId, 0, 0, 0,
         ("%s:add fileCB %p VolId: %llx inum: %llx mntPnt: %llx\n",
		  	__FUNCTION__,
			fileCBp,
			fileCBp->nfp_fileHandle.volId.whole_thing,
			fileCBp->nfp_fileHandle.inum,
			fileCBp->nfp_fileHandle.mntPnt  ));
	}

#ifdef	NOTUSED	
	/*
	 *Add fileCB into file list by a client (ClientCB)
	 */
	rc = nfs_addCBToList(&clientCBp->nfp_fileList, 
		(void *)fileCBp, NFP_FILECB_NODE);
	if (rc != NFX_OK)
	{
		NLM_DBG_FUNC_PRINTF("nlm_addNewLock 3: failure to add fileCB to client file list!\n");
	return (NFX_ERR);
	}
#endif	//NOTUSED	

	/*-------------------------------------------------
	 *Add new lockCb into File Control Block lock list
	 *-------------------------------------------------
	 */
	rc = nlm_addLockCBToCurrList(fileCBp, nextLockNodep,lockCBp);
	if (rc != NFX_OK)
	{
//		nfs_delCBFromList(&clientCBp->nfp_fileList,
//				(void *)fileCBp, NFP_FILECB_NODE);

		return (NFX_ERR);
	}

#ifdef	NLM_DEBUG
	nlm_showFileLocks(fileCBp);
#endif


    return (NFX_OK);
}

#else  // NOT_USED
/*
 *---------------------------------------------------------------
 * Function Name:   nlm_storeAlockInfo   		
 *
 * Description:
 *	Stores nlm_lock information fields into response message to
 *	be returned to the client.
 *		bool exlusive
 *		int32 svid:  unique process ID
 *		lock identity:  identity of the lock owner (oh)
 *		uint32/64 l_offset:  offset of locked region
 *		uint32/64 l_len:  extent of locked region
 *	
 * Created by:          Joseph Vu
 * Modified by:         Nitin Jain
 *
 * Created:             7/14/2002
 * Modified:            1/24/2003
 *
 *
 *-----------------------------------------------------------------
 */

int32
nlm_storeAlockInfo(	char *nlm_txMsgp,
					nlm_lockInfo_t *lockInfop,
					int32 nlm_version)
{
#ifdef NOT_USED
	nfp_lockCBPtr_t	lockCBp;
#endif
	char			*msgp;
	int32			totalLen;
	int32			length;
	int32			nfs_version;

#ifdef	NLM_VALIDATE_LOCK	
	if (nlm_validateLockRec(lockInfop) != NFX_OK)
		panic ("%s: Invalid lockRecp %p\n",
						   	__FUNCTION__,lockInfop);
#endif

#ifdef	NLM_VALIDATE_LOCK
	if (nfp_validateLockCB(lockCBp) != NFX_OK)
	{
		panic("%s: Bad lockCBp: %p\n", __FUNCTION__, lockCBp);	
	}
#endif
	totalLen = 0;
	msgp = nlm_txMsgp;

	//Map NLM version to comparable 
	//NFS version
	//nfs_version = nlm_utilMapToCompNFSVer(nlm_version);
	
	NLM_MAP_TO_COMP_NFS_VER(nfs_version, nlm_version);

	//exclusive
	length = nfs_storeLongValToBuf(	msgp,
									lockInfop->nlm_exclusive );	
	msgp += length;
	totalLen += length;
	
	//svid
	length = nfs_storeLongValToBuf(	msgp,
									lockInfop->nlm_svid );
	msgp += length;
	totalLen += length;
	
	length = nfs_storeString(msgp,
							lockInfop->nlm_oh, 
							lockInfop->nlm_ohLen,
							NLM_MAX_NAME);	
	msgp += length;
	totalLen += length;

	if (nfs_version == NFS_VER2)
	{
		//32 bits l_offset
		length = nfs_storeLongValToBuf(	msgp, 
										(uint32)lockInfop->nlm_l_offset );
		msgp += length;
		totalLen += length;

		//32 bits l_len
		length = nfs_storeLongValToBuf(	msgp, 
										(uint32 )lockInfop->nlm_l_len );
		msgp += length;
		totalLen += length;
	}
	else
	{
		//NFS V3
		//64 bits l_offset
		length = nfs_storeLongLongValToBuf(	msgp,
                							lockInfop->nlm_l_offset );
		msgp += length;
		totalLen += length;

		//64 bits l_len
		length = nfs_storeLongLongValToBuf(	msgp,
                							lockInfop->nlm_l_len );
		msgp += length;
		totalLen += length;

	}
	return (totalLen);
}

/*
 *---------------------------------------------------------------
 * Function Name:  nlm_saveLockInfo 
 *
 * Description:
 *	Save lock information from source to destination structure.
 *	
 * Created by:          Joseph Vu
 *
 * Created:             7/14/2002
 *
 *
 *-----------------------------------------------------------------
 */

int32
nlm_saveLockInfo(nlm_lockInfo_t *srcLock, nlm_request_t *origNlmReqp,
                 nfp_requestCB_t **dstReqCB)
{
    char *caller, *owner;
    nfp_requestCB_t *reqCBp;
    nlm_request_t *nlmReqp;

    caller = eee_ramAlloc(srcLock->nlm_callerNameLen);
    if (caller == NULL)
        return NFX_ERR;

    owner = eee_ramAlloc(srcLock->nlm_ohLen);
    if (owner == NULL)
        return NFX_ERR;

    *dstReqCB = NULL;
    reqCBp = nfp_allocReqCB ();
    if (reqCBp == NULL)
    {
        eee_ramDealloc(owner);
        eee_ramDealloc(caller);
        return NFX_ERR;
    }
    *dstReqCB = reqCBp;

    nlm_globalCfgp->nlm_debugCounters.nlm_allocReqCB++;
    nlmReqp = (nlm_request_t *)vs_mem_idxToPtr(reqCBp->protocol.vs_nlm_idx);
    nlmReqp->reqCBp      = reqCBp;
    nlmReqp->tplConnp    = NULL;
    nlmReqp->rxPktDescp  = NULL;
    nlmReqp->txPktDescp  = NULL;
    nlmReqp->clientCBp   = (void *)origNlmReqp->clientCBp;
    nlmReqp->fileCBp     = (void *)origNlmReqp->fileCBp;
    nlmReqp->lockInfop   = (void *)&nlmReqp->nlm_lockInformation;
    nlmReqp->rpcCallHdrp = &nlmReqp->rpcHdr;
    memcpy(&nlmReqp->rpcHdr, origNlmReqp->rpcCallHdrp, sizeof(rpcCallHdr_t));
    nlmReqp->nlm_txCmdInfo.laddr=((pkt_desc_t *)origNlmReqp->rxPktDescp)->laddr;

    memcpy(&nlmReqp->nlm_lockInformation, srcLock, sizeof(nlm_lockInfo_t));
    nlmReqp->nlm_lockInformation.nlm_callerName = caller;
    nlmReqp->nlm_lockInformation.nlm_oh = owner;
	return (NFX_OK);
}


#endif // NOT_USED

int32 nlm_dataInd ( char *pData, int32 size )
{
	printf("nlm_dataInd: pdata: %p size: %d\n", pData, size);

	//luc_trace("nlm_dataInd", LUC_TRACE_PKT, 0, pData, size);

	return 0;
}


int32 nlm_eventInd (void *nlmReqp, int32 event)
{
	
	printf("nlm_eventInd: nlmReqp: %p  event: %d\n", 
					nlmReqp, event);


	return 0;
}

/*
 *-----------------------------------------------------------------
 * Function Name:  nlm_getportNlm 		
 *
 * Description:
 *	Sends portmapper Getport command to get client
 *  Network Lock Manager (NLM) port. 
 *  
 * Created by:          Joseph Vu
 *
 * Created:             7/30/2002
 *
 *
 *-----------------------------------------------------------------
 */
#ifdef NOT_USED
int32 nlm_getportNlm (	nlm_request_t *nlmReqp, 
						nlm_request_t *pendingLockRecp)
{
	nfp_lockCB_t 		*pending_lockCBp;
	nfp_clientCBPtr_t	clientCBp;
	int32				rc;


	if ( (nlmReqp == NULL)  || 
	   (nlmReqp->clientCBp == NULL ) )
		return (NFX_ERR);

		
	/*-------------------------------------
	 *Add nlmReq to client control block
	 *Assign Xid for getport RPC call
	 *-------------------------------------
	 */	
	clientCBp = nlmReqp->clientCBp;

	if (nfs_addCBToList(&clientCBp->nfp_nlmReqList, 
				(void *)nlmReqp, NFP_REQCB_NODE)  != NFX_OK)
    {
        return (NFX_ERR);
    }
	nlm_globalCfgp->nlm_debugCounters.numOfAddNlmReqCbToList++;

	if (pendingLockRecp)
	{
		pending_lockCBp = (nfp_lockCB_t *)pendingLockRecp->lockCBp;
		nlmReqp->lockRecp = pendingLockRecp;
		nlmReqp->nlm_txCmdInfo.raddr = 
				pending_lockCBp->nfp_clientCbp->nfp_ipAddr;
	}
	else
	{
		nlmReqp->nlm_txCmdInfo.raddr = clientCBp->nfp_ipAddr;
	}

	nlmReqp->nlm_txCmdInfo.rport = RPC_PORTMAPPER_WELLKNOWN_PORT;
	nlmReqp->nlm_txCmdInfo.dataInd = nlm_dataInd;
	nlmReqp->nlm_txCmdInfo.eventInd = nlm_eventInd;
	
	rc = nlm_procTxGetportReq(nlmReqp);

	return (rc);
}
#else // NOT_USED
int32 nlm_getportNlm (nlm_request_t *newNlmReqp, nlm_request_t *nlmReqp)
{
	nfp_clientCBPtr_t	clientCBp;
	int32				rc;
    void                *req = nlmReqp;

	if ((newNlmReqp == NULL)  || (newNlmReqp->clientCBp == NULL ))
		return (NFX_ERR);

	/*-------------------------------------
	 *Add nlmReq to client control block
	 *Assign Xid for getport RPC call
	 *-------------------------------------
	 */	
	clientCBp = newNlmReqp->clientCBp;

    if (nlmReqp == NULL)
        req = newNlmReqp;

	if (nfs_addCBToList(&clientCBp->nfp_nlmReqList, 
				        req, NFP_REQCB_NODE)  != NFX_OK)
    {
        return (NFX_ERR);
    }
	nlm_globalCfgp->nlm_debugCounters.numOfAddNlmReqCbToList++;

	if (nlmReqp)
		newNlmReqp->nlm_txCmdInfo.raddr = nlmReqp->nlm_txCmdInfo.raddr;
	else
		newNlmReqp->nlm_txCmdInfo.raddr = clientCBp->nfp_ipAddr;

	newNlmReqp->nlm_txCmdInfo.rport = RPC_PORTMAPPER_WELLKNOWN_PORT;
	newNlmReqp->nlm_txCmdInfo.dataInd = nlm_dataInd;
	newNlmReqp->nlm_txCmdInfo.eventInd = nlm_eventInd;
	
	rc = nlm_procTxGetportReq(newNlmReqp);

	return (rc);
}
#endif // NOT_USED

/*
 *-------------------------------------------------------------
 * Function Name: 	nlm_callbackNextPendingLock 
 *
 * Description:
 *	This function callback the next pending lock
 *  on the list.
 *
 * Created by:          Joseph Vu
 *
 * Created:             7/30/2002
 *
 *
 *-------------------------------------------------------------
 */
#ifdef NOT_USED
int32 nlm_callbackNextPendingLock(	nlm_lockRec_t	*lockRecp,
									nlm_request_t 	*nlmReqp,
									uint32 			rmListType)
{
	nlm_lockRec_t		*nextLockRecp;
	nlm_lockRec_t 		*conflictLockRecp;
	nlm_lockRec_t		*pendingLockRecp;
	nfp_requestCBPtr_t	reqCBp;
	nfp_lockCB_t 		*lockCBp;
	nfp_lockCB_t 		*pendingLockCBp;
	nlm_lockInfo_t		*pendingLockInfop;
	int32				searchMask;
	int32				rc;

	rc = NFX_OK;

	if (nlm_validateLockRec(lockRecp) != NFX_OK)
	{
#ifdef	NLM_VALIDATE_LOCK
		panic ("%s: Invalid lock record %p\n",
					   	__FUNCTION__,lockRecp);
#endif
		goto nlm_callbackNextPendingLockErr;
	}

	lockCBp = (nfp_lockCB_t *)lockRecp->lockCBp;	
	if (nfp_validateLockCB(lockCBp) != NFX_OK)
	{
#ifdef	NLM_VALIDATE_LOCK
		panic ("%s: Invalid lock record %p\n",
					   	__FUNCTION__,lockCBp);
#endif
		goto nlm_callbackNextPendingLockErr;
	}


	/*-------------------------------------------
	 *Remove current lock from the current list
	 *and check for any waiting pending lock request
	 *--------------------------------------------
	 */

//	if (rmListType == NLM_CURRENT_LIST)
	if (lockRecp->flags & NLM_LOCKREC_CURR_LIST)
	{
		DLL_REMOVE(&lockCBp->lockOwnerList.list,
				 lockRecp, lockList);
		lockCBp->lockOwnerList.numOfEntries--;
	}
	else
	{
		DLL_REMOVE(&lockCBp->lockOwnerPendingList.list,
				 lockRecp, lockList);
		lockCBp->lockOwnerPendingList.numOfEntries--;
	}
	pendingLockRecp = lockRecp->pendingLockRecp;

	nlm_deallocLockRec(lockRecp);	


	if (pendingLockRecp == NULL)
	{
		if  (nlmReqp == NULL )
			return (NFX_OK);	
		else
			goto nlm_callbackNextPendingLockErr;
	}

	if (nlm_validateLockRec(pendingLockRecp) != NFX_OK)
	{
		panic ("%s: Invalid pendinglock record %p\n",
					   	__FUNCTION__,pendingLockRecp);
		goto nlm_callbackNextPendingLockErr;
	}

	pendingLockCBp = (nfp_lockCB_t	*)pendingLockRecp->lockCBp;

	if (nfp_validateLockCB(pendingLockCBp) != NFX_OK)
	{
		panic ("%s: Invalid pending pendingLockCBp  %p\n",
					   	__FUNCTION__,pendingLockCBp);
		goto nlm_callbackNextPendingLockErr;
	}

	if (nlmReqp == NULL)
	{
		
		reqCBp = nfp_allocReqCB ();
		if (reqCBp == NULL)
			return (NFX_ERR);
		nlm_globalCfgp->nlm_debugCounters.nlm_allocReqCB++;
		nlmReqp = &reqCBp->protocol.nlm_req;	
		nlmReqp->reqCBp = reqCBp;
		nlmReqp->tplConnp = NULL;
		nlmReqp->rxPktDescp = NULL;
		nlmReqp->clientCBp = pendingLockCBp->nfp_clientCbp;
		nlmReqp->fileCBp = pendingLockCBp->nfp_fileCbp;
		nlmReqp->lockRecp = pendingLockRecp;		
		nlmReqp->nlm_txCmdInfo.laddr = 
				pendingLockCBp->nfp_clientCbp->nfp_localIpAddr;
		nlmReqp->rpcHdr.progVer = 
				pendingLockCBp->nfp_clientCbp->nfp_nlmClientInfo.nlm_version;
		nlmReqp->rpcCallHdrp = &nlmReqp->rpcHdr;
	}


	if (pendingLockRecp->waitForLockRecp != lockRecp)
	{
		nlm_globalCfgp->nlm_debugCounters.numOfCallBackErr++;
		goto nlm_callbackNextPendingLockErr;
	}

	/*---------------------------------------------------
	 *Check if pending (block) lock is still in conflict 
	 *with any current active lock(s) on this file
	 *---------------------------------------------------
	 */

	searchMask = 0;
	if (!pendingLockRecp->nlm_exclusive)
		searchMask = NLM_SEARCH_EXCLUSIVE;

	pendingLockInfop = &pendingLockCBp->protocol.nfp_nlmLock;
	pendingLockInfop->lockRec.nlm_l_end = pendingLockRecp->nlm_l_end;
	pendingLockInfop->lockRec.nlm_l_offset = pendingLockRecp->nlm_l_offset;
	pendingLockInfop->lockRec.nlm_l_len = pendingLockRecp->nlm_l_len;
	pendingLockInfop->lockRec.nlm_l_eof = pendingLockRecp->nlm_l_eof;

	conflictLockRecp = nlm_getLockRecInCurrList(pendingLockCBp->nfp_fileCbp,
												pendingLockInfop,
												searchMask,
												&nextLockRecp);

	if (conflictLockRecp == NULL)
	{


		/*----------------------------------------------
		 *The pending lock does not conflict
		 *with other active locks. 
		 *Add the pending lock to the active lock list	
  	 	 *----------------------------------------------
  	 	 */
		DLL_INSERT_TAIL(&pendingLockCBp->lockOwnerList.list,
						pendingLockRecp, lockList);

		pendingLockCBp->lockOwnerList.numOfEntries++;


		/*------------------------------------------
		 *Remove pending lock record from pending list	
	 	 *------------------------------------------
	 	 */
		DLL_REMOVE(&pendingLockCBp->lockOwnerPendingList.list,
					pendingLockRecp, lockList);
		pendingLockCBp->lockOwnerPendingList.numOfEntries--;

		/*------------------------------------
		 *call back to block client
		 *Send a Portmapper GetPort to the client to
		 *find the client NLM port 
  	 	 *-----------------------------------
  	 	 */
		nlmReqp->lockRecp = pendingLockRecp;

		rc = nlm_getportNlm(nlmReqp, pendingLockRecp);
		if (rc != NFX_OK)
		{
			printf("%s: Failure to send getport message!\n",
						__FUNCTION__);
			goto nlm_callbackNextPendingLockErr;
		}
		return (NFX_OK);
	}
	else
	{
		rc = nlm_setPendingLockEntry(conflictLockRecp,
									pendingLockRecp);
		if (rc != NFX_OK)
			goto nlm_callbackNextPendingLockErr;	
	}

	return (NFX_OK);

nlm_callbackNextPendingLockErr:
	if (nlmReqp)
	{
		nlmReqp->nlm_txCmdInfo.grantedRetry = 0;
		nlmReqp->nlm_txCmdInfo.grantedMaxTimeWaitCnt = 0;
		nlmReqp->reqState = NLM_REQ_DONE;
		nlmReqp->rxPktDescp = NULL;
		nlm_delReqCBFromList(nlmReqp);
		nlm_freeAll(nlmReqp, TRUE);
	}
	return (NFX_ERR);
}


/*
 *-------------------------------------------------------------
 * Function Name: nlm_checkUnlockedRegions		
 *
 * Description:
 *	Check the unlocked byte offset regions.  
 *  
 * Created by:          Joseph Vu
 *
 * Created:             7/30/2002
 *
 *
 *-------------------------------------------------------------
 */
#ifdef	NOTUSED
nlm_lockRec_t *
nlm_checkUnlockedRegions(nfp_fileCBPtr_t fileCBp,
						nlm_lockInfo_t *unlockAlockp,
						int32 *unlockType
)
{
	uint32				searchMask;
	nlm_lockRec_t		*lockRecp;
	nfp_lockCB_t 		*lockCBp;	
	nlm_lockInfo_t 		*lockAlockp;
	nlm_lockRec_t		*nextLockRecp;


	*unlockType = 0;

	/*--------------------------------------
	 *Search by OH & process ID 
 	 *--------------------------------------
 	 */
	searchMask = NLM_SEARCH_OH | NLM_SEARCH_SVID;		
	lockRecp =   nlm_getLockRecInCurrList(fileCBp,
                                        unlockAlockp,
                                        searchMask,
                                        &nextLockRecp);

	if (lockRecp == NULL)
		return (NULL);

	lockCBp = (nfp_lockCB_t 	*)lockRecp->lockCBp;
	lockAlockp = &lockCBp->protocol.nfp_nlmLock;

#ifdef	NLM_DEBUG
	printf("%s: input unlock svid: %d offset:%lld  len: %lld end: %lld\n",
		__FUNCTION__, 
		unlockAlockp->nlm_svid,
		unlockAlockp->lockRec.nlm_l_offset,
		unlockAlockp->lockRec.nlm_l_len,
		unlockAlockp->lockRec.nlm_l_end); 
	printf("%s: Found overlap lock oh:%s offset:%lld len:%lld end: %lld\n",
		__FUNCTION__, 
		lockAlockp->nlm_oh,
		lockRecp->nlm_l_offset,
		lockRecp->nlm_l_len,
		lockRecp->nlm_l_end); 
#endif

	E_LOG (class_1, LOG_DEBUG, 
			nlm_globalCfgp->eee_nlmAppId, 0, 0, 0,
         ("%s: Found overlap lock oh: %s offset: %lld len: %lld\n",
		  	__FUNCTION__, 
		  	lockAlockp->nlm_oh,
   			lockRecp->nlm_l_offset,
			lockRecp->nlm_l_len ));

	/*--------------------------------------
	 * Check for unlocked the whole region cases
 	 *--------------------------------------
	 */

	if ((unlockAlockp->lockRec.nlm_l_offset == 
							lockRecp->nlm_l_offset)  &&
		(unlockAlockp->lockRec.nlm_l_end  == 
		 			lockRecp->nlm_l_end) )
	{
		*unlockType = NLM_UNLOCK_WHOLE_REGION;
		return (lockRecp);
	}

	if ((unlockAlockp->lockRec.nlm_l_offset <= 
							lockRecp->nlm_l_offset) &&
	 	( (unlockAlockp->lockRec.nlm_l_eof  == TRUE) ||
		  ((lockRecp->nlm_l_eof != TRUE) &&
	 	    (unlockAlockp->lockRec.nlm_l_end  > 
			 				lockRecp->nlm_l_end))) )
	{
		*unlockType = NLM_UNLOCK_WHOLE_REGION;
		return (lockRecp);
	}

	/*--------------------------------------
	 * Check for splitted lock region cases
 	 *--------------------------------------
	 */
	if ( (unlockAlockp->lockRec.nlm_l_offset > 
							lockRecp->lockRec.nlm_l_offset) &&
		( (unlockAlockp->lockRec.nlm_l_eof != TRUE) &&
		 (lockRecp->nlm_l_eof != TRUE ) && 
		(unlockAlockp->lockRec.nlm_l_end < lockRecp->nlm_l_end) ) )
	{
		/*--------------------------------------
	 	 * split lock into 2 regions
 	 	 *--------------------------------------
	 	 */
		*unlockType = NLM_UNLOCK_SPLITTED_TWO_REGIONS; 
		return (lockRecp);
	}

	if ( (unlockAlockp->lockRec.nlm_l_offset > 
							lockRecp->nlm_l_offset) &&
		 ( (unlockAlockp->lockRec.nlm_l_eof  == TRUE) ||
		   ((lockRecp->nlm_l_eof != TRUE ) &&
			(unlockAlockp->lockRec.nlm_l_end >= 
			 			lockRecp->nlm_l_end)) ) ) 
	{
		*unlockType = NLM_UNLOCK_SPLITTED_REGION_BEFORE; 
		return (lockRecp);
	}

	if ( (unlockAlockp->lockRec.nlm_l_offset <= 
							lockRecp->nlm_l_offset) &&
		 (unlockAlockp->lockRec.nlm_l_eof != TRUE) &&
		 ( (lockRecp->nlm_l_eof == TRUE) || 
		   ((lockAlockp->lockRec.nlm_l_eof != TRUE) &&
		    (unlockAlockp->lockRec.nlm_l_end < lockRecp->nlm_l_end)) ) )
	{
		*unlockType = NLM_UNLOCK_SPLITTED_REGION_AFT; 
		return (lockRecp);
	}

	/*--------------------------------------
 	 * Unlocked the whole region
 	 *--------------------------------------
 	 */
	*unlockType = NLM_UNLOCK_WHOLE_REGION;
	return (lockRecp);

}

#endif	//NOTUSED
int32
nlm_checkUnlockedRegions(nlm_lockRec_t *unlockRecp,
						nlm_lockInfo_t *unlockAlockp,
						nlm_request_t *nlmReqp,
						int32 *overlap_p,
						int32 *unlockType)
{
	nlm_lockRec_t		*pendingLockRecp;
	nlm_lockInfo_t 		*lockAlockp;
	nfp_fileCBPtr_t		fileCBp;
	nfp_lockCBPtr_t		unlockCBp;

	*overlap_p = 0;
	*unlockType = 0;
	pendingLockRecp = unlockRecp->pendingLockRecp;
	unlockCBp = (nfp_lockCBPtr_t )unlockRecp->lockCBp;
	fileCBp = unlockCBp->nfp_fileCbp;

	if ((unlockAlockp->lockRec.nlm_l_offset == 
							unlockRecp->nlm_l_offset)  &&
		(unlockAlockp->lockRec.nlm_l_end  == 
		 			unlockRecp->nlm_l_end) )
	{
		*unlockType = NLM_UNLOCK_WHOLE_REGION;
		return (NFX_OK);
	}

	if ((unlockAlockp->lockRec.nlm_l_offset <= 
							unlockRecp->nlm_l_offset) &&
	 	( (unlockAlockp->lockRec.nlm_l_eof  == TRUE) ||
		  ((unlockRecp->nlm_l_eof != TRUE) &&
	 	    (unlockAlockp->lockRec.nlm_l_end  > 
			 				unlockRecp->nlm_l_end))) )
	{
		*unlockType = NLM_UNLOCK_WHOLE_REGION;
		return (NFX_OK);
	}

	/*--------------------------------------
	 * Check for splitted lock region cases
 	 *--------------------------------------
	 */
	if ( (unlockAlockp->lockRec.nlm_l_offset > 
							unlockRecp->nlm_l_offset) &&
		( (unlockAlockp->lockRec.nlm_l_eof != TRUE) &&
		 (unlockRecp->nlm_l_eof != TRUE ) && 
		(unlockAlockp->lockRec.nlm_l_end < unlockRecp->nlm_l_end) ) )
	{
		nlm_lockRec_t		*lockRec2p = NULL;

		/*--------------------------------------
	 	 * split lock into 2 regions
 	 	 *--------------------------------------
	 	 */
		*unlockType = NLM_UNLOCK_SPLITTED_TWO_REGIONS; 

		/*Lock regions will be splitted by this unlock*/
		lockRec2p = nlm_unlockSplittedTwoRegions(unlockRecp,
												unlockAlockp,
												fileCBp,
												nlmReqp);
		if (lockRec2p == NULL)
			return (NFX_ERR);

		if (pendingLockRecp == NULL)
			/*no pending lock waiting, return*/
			return (NFX_OK);

		/*Check for conflict against new lock region 1*/
		*overlap_p = nlm_regionCmp(unlockRecp, 
								pendingLockRecp);
		if (!*overlap_p)
			/*still conflict with region 1, return*/
			return (NFX_OK);

		/*Check for conflict against new lock region 2*/
		*overlap_p = nlm_regionCmp(lockRec2p,
								pendingLockRecp);
		if (!*overlap_p)
		{
			/*-----------------------------------------
			 *Conflicting with new lock region 2
			 *adjust pending lock ptrs
	 	 	 *------------------------------------------
	 	 	 */
			lockRec2p->pendingLockRecp = pendingLockRecp;
			pendingLockRecp->waitForLockRecp = lockRec2p;
			unlockRecp->waitForLockRecp = NULL;
			return (NFX_OK);
		}
		return (NFX_OK);
	}

	if ( (unlockAlockp->lockRec.nlm_l_offset > 
							unlockRecp->nlm_l_offset) &&
		 ( (unlockAlockp->lockRec.nlm_l_eof  == TRUE) ||
		   ((unlockRecp->nlm_l_eof != TRUE ) &&
			(unlockAlockp->lockRec.nlm_l_end >= 
			 			unlockRecp->nlm_l_end)) ) ) 
	{
		*unlockType = NLM_UNLOCK_SPLITTED_REGION_BEFORE; 

		/*--------------------------------------------
		 *re-calculate splitted region. 
	 	 *---------------------------------------------
	 	 */
		unlockRecp->nlm_l_len = 
	   		(unlockAlockp->lockRec.nlm_l_offset - 1 - 
				unlockRecp->nlm_l_offset ); 

		NLM_GET_L_REGION_END(unlockRecp->nlm_l_end, 
							unlockRecp->nlm_l_offset, 
							unlockRecp->nlm_l_len,
							unlockRecp->nlm_l_eof);

#ifdef	NLM_DEBUG
		printf("%s: split region-before unlockRecp:%p. svid:%d offset:%lld len:%lld end: %lld\n",
   					__FUNCTION__, 
					unlockRecp,
					unlockAlockp->nlm_svid,
					unlockRecp->nlm_l_offset,
					unlockRecp->nlm_l_len,
					unlockRecp->nlm_l_end);
#endif	
		if (pendingLockRecp == NULL)
			/*no pending lock waiting, return*/
			return (NFX_OK);

		/*Check for conflict against new lock region*/
		*overlap_p = nlm_regionCmp(unlockRecp, 
								pendingLockRecp);
		return (NFX_OK);
	}

	if ( (unlockAlockp->lockRec.nlm_l_offset <= 
							unlockRecp->nlm_l_offset) &&
		 (unlockAlockp->lockRec.nlm_l_eof != TRUE) &&
		 ( (unlockRecp->nlm_l_eof == TRUE) || 
		   ((lockAlockp->lockRec.nlm_l_eof != TRUE) &&
		    (unlockAlockp->lockRec.nlm_l_end < unlockRecp->nlm_l_end)) ) )
	{
		uint32			saveOffset;

		*unlockType = NLM_UNLOCK_SPLITTED_REGION_AFT; 

		saveOffset = unlockRecp->nlm_l_offset;
		unlockRecp->nlm_l_offset = 
			unlockAlockp->lockRec.nlm_l_offset + 
				unlockAlockp->lockRec.nlm_l_len + 1;
		if (saveOffset)
			unlockRecp->nlm_l_len = 
				(unlockAlockp->lockRec.nlm_l_offset + 
				unlockRecp->nlm_l_len - 
				unlockRecp->nlm_l_offset );
		else
			unlockRecp->nlm_l_len = 
				(unlockRecp->nlm_l_len - 
				unlockRecp->nlm_l_offset );

		NLM_GET_L_REGION_END(unlockRecp->nlm_l_end, 
							unlockRecp->nlm_l_offset, 
							unlockRecp->nlm_l_len,
							unlockRecp->nlm_l_eof);
#ifdef	NLM_DEBUG
		printf("%s: split region-aft unlockRecp:%p. svid:%d offset:%lld len:%lld end: %lld\n",
   				__FUNCTION__, 
				unlockRecp,
				unlockAlockp->nlm_svid,
				unlockRecp->nlm_l_offset,
				unlockRecp->nlm_l_len,
				unlockRecp->nlm_l_end);
#endif

		if (pendingLockRecp == NULL)
			/*no pending lock waiting, return*/
			return (NFX_OK);

		/*Check for conflict against new lock region*/
		*overlap_p = nlm_regionCmp(unlockRecp, 
									pendingLockRecp);
		return (NFX_OK) ;
	}

	/*--------------------------------------
 	 * Unlocked the whole region
 	 *--------------------------------------
 	 */
	*unlockType = NLM_UNLOCK_WHOLE_REGION;
	return (NFX_OK);

}
#endif // NOT_USED

/*
 *-------------------------------------------------------------
 * Function Name:nlm_clearLock	 
 *
 * Description:
 * 	Clear or remove client NLM locks 
 *  
 * Created by:          Joseph Vu
 *
 * Created:             7/30/2002
 *
 *
 *-------------------------------------------------------------
 */
#ifdef NOT_USED
void nlm_clearLock(uint32 clientAddr)
#else
void nlm_clearLock(vs_mem_index_t fIdx, uint32 clientAddr)
#endif
{
#ifdef NOT_USED
	nfp_fileCBPtr_t		fileCBp;
	int32				count, nodeCount;
	nfp_list_t			*nodeListp;
	nfp_nodePtr_t		nodep;
	nfp_lockCBPtr_t		lockCBp;
	nlm_lockRec_t		*lockRecp, *lockRecNextp;
	int32				rc;

	//NLM_DBG_FUNC_PRINTF("%s: enter\n", __FUNCTION__);	
	count = 0;
	for (fileCBp = (nfp_fileCBPtr_t)nfp_globalCfgp->fileCbCfgList.listBasep;
		((count < NFP_MAX_FILE) && 
		(fileCBp != NULL)); count++, fileCBp++)
	{
		if (!(fileCBp->nfp_flag & NFP_ENTRY_INUSED))
			continue;

		/*--------------------------------
		 *Check current active lock list
		 *--------------------------------
		 */
		nodeListp = &fileCBp->nfp_lockList;
		nodeCount = nodeListp->nfp_reqCnt;
		for (nodep = nodeListp->nfp_headp;
			((nodep != NULL) && (nodeCount > 0));
			nodep = nodep->nfp_nextp, nodeCount--)
		{
			lockCBp = (nfp_lockCBPtr_t)nodep->nfp_entryCBp;
			if ( lockCBp && clientAddr && lockCBp->nfp_clientCbp &&
				(((nfp_clientCBPtr_t)lockCBp->nfp_clientCbp)->nfp_ipAddr
							 != clientAddr) )
				/*skip if not client addr does not match*/
				continue;

			/*-----------------------------
			 *Check current locks ower list
			 *-----------------------------
			 */
			DLL_ITERATE_NEXT(lockRecp, lockRecNextp,
						&lockCBp->lockOwnerList.list,
				   		lockList)
			{
				if ((lockRecp == NULL) || (lockRecp == DLL_NIL) )
					break;
#ifdef	NLM_VALIDATE_LOCK
				if (nlm_validateLockRec(lockRecp) != NFX_OK)
				{
					panic ("%s 1: Invalid unlockRecp %p\n",
						   	__FUNCTION__,lockRecp);
				}
#endif

				rc = nlm_callbackNextPendingLock(lockRecp, 
												NULL, 
												NLM_CURRENT_LIST);
				if (rc != NFX_OK)
				{
					DLL_REMOVE(&lockCBp->lockOwnerList.list,
								lockRecp, lockList);
					lockCBp->lockOwnerList.numOfEntries--;
				}

			}

			/*-----------------------------
			 *Check pending locks ower list
			 *-----------------------------
			 */
			DLL_ITERATE_NEXT(lockRecp, lockRecNextp,
						&lockCBp->lockOwnerPendingList.list,
				   		lockList)
			{
				if ((lockRecp == NULL) || (lockRecp == DLL_NIL) )
					break;
#ifdef	NLM_VALIDATE_LOCK

				if (nlm_validateLockRec(lockRecp) != NFX_OK)
				{
					panic ("%s 2: Invalid unlockRecp %p\n",
						   	__FUNCTION__,lockRecp);
				}
#endif

				rc = nlm_callbackNextPendingLock(lockRecp, 
												NULL, 
												NLM_PENDING_LIST);
				if (rc != NFX_OK)
				{
					DLL_REMOVE(&lockCBp->lockOwnerPendingList.list,
								lockRecp, lockList);
					lockCBp->lockOwnerPendingList.numOfEntries--;

				}
			}

			if ((lockCBp->lockOwnerList.numOfEntries <= 0) &&
				(lockCBp->lockOwnerPendingList.numOfEntries <= 0 ) )
			{
				nlm_rmLockCBFromList(lockCBp,
									TRUE);
			}


		}
		if (!fileCBp->nfp_lockList.nfp_reqCnt) 
		{
#ifdef	NLM_DEBUG
			printf("Delete FileCB: %p volId: %llx inum: %llx mntPnt: %llx\n",
				fileCBp, 
				fileCBp->nfp_fileHandle.volId.whole_thing,
				fileCBp->nfp_fileHandle.inum,
				fileCBp->nfp_fileHandle.mntPnt);
#endif
			nfp_delFileCB(&fileCBp->nfp_fileHandle);
		}
	}
#else
    lock_unlockAllByIpAddrReq(fIdx, clientAddr);
#endif // NOT_USED
    return;
}

/*
 *-------------------------------------------------------------
 * Function Name:  nlm_nsmMonCmpInd
 *
 * Description:
 *	This is callback function from Network Status Monitor (NSM) 
 *  program  for Monitor host complete indication event.  
 *  
 * Created by:          Joseph Vu
 *
 * Created:             7/30/2002
 *
 *
 *-------------------------------------------------------------
 */

int32
nlm_nsmMonCmpInd(evm_volId_t        volId,
                  nfp_clientCB_t     *nlm_clientCBp,
                  int32              status)
{
#ifdef	NLM_DEBUG
    printf("%s: Finished processing monitor request\n",__FUNCTION__);
    printf("%s: volId.whole_thing = 0x%llx, nlm_clientCBp = %p, status = %d\n",
	   	__FUNCTION__,volId.whole_thing, nlm_clientCBp, status);
#endif

    return NFX_OK;
}

/*
 *-------------------------------------------------------------
 * Function Name: nlm_nsmUnmonCmpInd 
 *
 * Description:
 *	This is callback function from Network Status Monitor (NSM) 
 *  program  for Unmonitor host complete indication event.  
 *  
 * Created by:          Joseph Vu
 *
 * Created:             7/30/2002
 *
 *
 *-------------------------------------------------------------
 */

int32
nlm_nsmUnmonCmpInd(evm_volId_t        volId,
                    nfp_clientCB_t     *nlm_clientCBp,
                    int32              status)
{

#ifdef	NLM_DEBUG
    printf("%s: Finished processing unmonitor request\n", __FUNCTION__);
    printf("%s: volId.whole_thing = 0x%llx, nlm_clientCBp = %p, status = %d\n", 
			__FUNCTION__, volId.whole_thing, nlm_clientCBp, status);
#endif
    return NFX_OK;
}

#ifdef	NOTUSED
/*
 *-----------------------------------------------------------------
 * Function Name:nlm_notifyRemoveStaleLock	 
 *
 * Description:
 *
 * Created by:          Joseph Vu
 *
 * Created:             7/30/2002
 *
 *
 *-----------------------------------------------------------------
 */

int32
nlm_notifyRemoveStaleLock(nfp_lockCBPtr_t			unlockCBp
)
{
	nfp_fileCBPtr_t			fileCBp;

	nlm_callbackNextPendingLock(unlockCBp, NULL, NLM_CURRENT_LIST);
	fileCBp = unlockCBp->nfp_fileCbp;
	if (fileCBp && fileCBp->nfp_lockList.nfp_reqCnt == 0 &&
		(fileCBp->nfp_lockPendingList.nfp_reqCnt == 0))
	{
		nfp_delFileCB (&fileCBp->nfp_fileHandle);
	}
#ifdef	NLM_DEBUG
		nlm_showFileLocks(fileCBp);
#endif	//NLM_DEBUG


	return (NFX_OK);
}

#endif	//NOTUSED

/*
 *-----------------------------------------------------------------
 * Function Name: nlm_nsmClientDownInd	 
 *
 * Description:
 *		This callback function from NSM (Network Status Monitor)
 * program when its received SM_NOTIFY from a client for us,  
 * the server to remove all stale locks that were hold 
 * by the restarted client.
 *
 * Created by:          Joseph Vu
 *
 * Created:             7/30/2002
 *
 *
 *-----------------------------------------------------------------
 */

int32
nlm_nsmClientDownInd(uint32    hostIpAddr)
{
//	int32 					count;
//	nfp_lockCBPtr_t			lockCBp;
//	nfp_fileCBPtr_t			fileCBp;
//	nfp_clientCBPtr_t		clientCBp;

#ifdef	NLM_DEBUG
	printf("%s: client Notify request\n",
				   	__FUNCTION__);
    printf("%s: hostIpAddr = 0x%x\n", 
					__FUNCTION__, hostIpAddr);
#endif

#ifdef	NOTUSED
	count = 0;	
	for (lockCBp= 
		(nfp_lockCBPtr_t)nfp_globalCfgp->lockCbCfgList.listBasep;
        ((count < NFP_MAX_LOCK) && (lockCBp != NULL)); 
		count++, lockCBp++)
    {
		if (!(lockCBp->nfp_flag & NFP_ENTRY_INUSED))
			continue;

		clientCBp = lockCBp->nfp_clientCbp;
		if ((clientCBp == NULL) ||
			(clientCBp->nfp_ipAddr != hostIpAddr) )
			continue;
#ifdef	NLM_DEBUG
		printf("%s: found client lock for remove. oh: %s offset:%lld len:%lld clientIpAddr: %08x\n", 
			__FUNCTION__, 
			lockCBp->protocol.nfp_nlmLock.nlm_oh,
			lockCBp->protocol.nfp_nlmLock.lockRec.nlm_l_offset,
			lockCBp->protocol.nfp_nlmLock.lockRec.nlm_l_len,
			ntohl(clientCBp->nfp_ipAddr));
#endif

		if (lockCBp->nfp_fileCbp == NULL)
			continue;
#ifndef NOT_USED
	    nlm_clearLock(lockCBp->nfp_fileCbp->fileLockIdx, hostIpAddr);
#endif
		nlm_notifyRemoveStaleLock(lockCBp);
	}

#endif	//NOTUSED
#ifdef NOT_USED
	nlm_clearLock(hostIpAddr);
#endif

    return NFX_OK;
}

/*
 *-----------------------------------------------------------------
 * Function Name: nlm_unlockSplittedTwoRegions	 
 *
 * Description:
 * This function is called when a client unlock request
 * caused the current lock region to be splitted.
 *
 * Created by:          Joseph Vu
 *
 * Created:             7/30/2002
 *
 *
 *-----------------------------------------------------------------
 */
#ifdef NOT_USED
nlm_lockRec_t *
nlm_unlockSplittedTwoRegions(nlm_lockRec_t	*unlockRecp,
							nlm_lockInfo_t	*nlm_lockInfop,
							nfp_fileCBPtr_t	fileCBp,
							nlm_request_t 	*nlmReqp)
{
	nlm_lockRec_t 		*lockRecp;
	nfp_lockCBPtr_t		unlockCBp;
	nlm_lockInfo_t 		*unlockAlockp;
	int32				save_nlmStartRegion;
	int32				save_nlmLenRegion;

	unlockCBp = (nfp_lockCBPtr_t)unlockRecp->lockCBp;
	unlockAlockp = &unlockCBp->protocol.nfp_nlmLock;

	save_nlmStartRegion = unlockRecp->nlm_l_offset;
	save_nlmLenRegion = unlockRecp->nlm_l_len;

	/*--------------------------------------
	 *locked region has been splitted
	 *from this unlock request
	 *we need recalculate splitted regions.
	 *Calculate splitted region 1
	 *--------------------------------------
	 */

	unlockRecp->nlm_l_len = 
		nlm_lockInfop->lockRec.nlm_l_offset - unlockRecp->nlm_l_offset; 

	NLM_GET_L_REGION_END(unlockRecp->nlm_l_end, 
						unlockRecp->nlm_l_offset, 
						unlockRecp->nlm_l_len,
						unlockRecp->nlm_l_eof);
#ifdef	NLM_DEBUG
	NLM_DBG_FUNC_PRINTF("%s: 1st region unlockRecp:%p. svid:%d offset:%lld len:%lld end: %lld\n",
   		__FUNCTION__, 
		unlockRecp,
		unlockAlockp->nlm_svid,
		unlockRecp->nlm_l_offset,
		unlockRecp->nlm_l_len,
		unlockRecp->nlm_l_end);
#endif

	/*--------------------------------------
	 *Calculate slitted region 2
	 *--------------------------------------
	 */

	lockRecp = 	nlm_AllocLockRec(NLM_LOCKREC_CURR_LIST);
	if (lockRecp == NULL)
	{
		return (NULL);						
	}

	lockRecp->nlm_l_offset =  
			nlm_lockInfop->lockRec.nlm_l_offset + 
			nlm_lockInfop->lockRec.nlm_l_len;

	lockRecp->nlm_l_len = save_nlmStartRegion + 
				save_nlmLenRegion - lockRecp->nlm_l_offset;

	NLM_GET_L_REGION_END(lockRecp->nlm_l_end, 
						lockRecp->nlm_l_offset,
						lockRecp->nlm_l_len, 
						lockRecp->nlm_l_eof);

	DLL_INSERT_TAIL(&unlockCBp->lockOwnerList.list,
					lockRecp, lockList);
	unlockCBp->lockOwnerList.numOfEntries++;
	lockRecp->lockCBp = unlockCBp;
	lockRecp->waitForLockRecp = unlockRecp->waitForLockRecp;
	lockRecp->pendingLockRecp = unlockRecp->pendingLockRecp;
#ifdef	NLM_DEBUG
		printf("%s: added 2nd splitted lock region. lockRecp: %p svid: %d offset:%lld len: %lld end: %lld\n",
					__FUNCTION__, 
					lockRecp,
					unlockAlockp->nlm_svid,
					lockRecp->nlm_l_offset,
					lockRecp->nlm_l_len,
					lockRecp->nlm_l_end);
#endif
		return (lockRecp);
}
#endif

/*
 *-----------------------------------------------------------------
 * Function Name:  nlm_gracePeriodTimeoutHandler
 *
 * Description:
 *	Callback function to reset Grace period flag.
 *
 * Created by:          Joseph Vu
 *
 * Created:             7/30/2002
 *
 *-----------------------------------------------------------------
 */

int32
nlm_gracePeriodTimeoutHandler(	int32 timer_handle, 
								void* param1, void* param2, 
								void* param3, void* param4)
{
	printf("%s: End of NLM Grace Period.  \n", __FUNCTION__);
	//reset grace period flag
	nlm_globalCfgp->gracePeriod = 0;
	return (NFX_OK);
}


#endif	//NFP_TXRX
