/* vi:ts=4
 *
 *-----------------------------------------------------------------
 *
 * Name:			nlm.h
 *
 * RCS:  $Id: nlm.h,v 1.8 2002/11/06 03:35:39 joseph Exp $
 *
 * Copyright (C) 2002, Agile Storage Inc.
 *
 * Description: NLM protocol definitions.
 *
 * Created by:		Xiaobing Zhang
 *
 * Date Created:	07/15/2002
 *
 *-----------------------------------------------------------------
 */
#ifndef _NLM_H
#define _NLM_H

#include <exports/malloc-api.h>
#include <exports/queue-api.h>

#define     NLM_PROG    		100021      
#define     NLM_PORT    		2090
#define		NLM_INT_PMAP_PORT   2091 
#define     NLM_MIN_PROC    	0      
#define     NLM_MAX_PROC    	24      

#define		NLM_MAX_LOCK_REC	(1024 * 32)

#define NLM_MAX_NAME		1024

/*NLM versions*/
#define NLM_VER1			1
#define NLM_VER3			3
#define NLM_VER4			4

/*LockCB list type */
#define NLM_CURRENT_LIST	0
#define NLM_PENDING_LIST	1

#define	NLM_MAX_LOCKS_PER_FILE	(1024 *4)	

/*Criteria for searching a lockCB in list*/
#define NLM_SEARCH_OH			0x0001		/*Search by lock owner*/
#define NLM_SEARCH_SVID			0x0002		/*Search by process ID*/
#define NLM_SEARCH_REGION_MATCH	0x0004		/*Locked region length must exactly match*/
#define NLM_SEARCH_EXCLUSIVE	0x0008		/*Search exclusive locked region*/

#define	NLM_UNLOCK_WHOLE_REGION				1
#define	NLM_UNLOCK_SPLITTED_TWO_REGIONS		2
#define	NLM_UNLOCK_SPLITTED_REGION_AFT 		3	
#define	NLM_UNLOCK_SPLITTED_REGION_BEFORE	4	

typedef enum nlm_stats
{
	NLM_STAT_GRANTED			= 0, /*Lock was granted*/
	NLM_ERR_DENIED				= 1, /*Lock was not granted due to conflict*/
	NLM_ERR_DENIED_NOLOCKS		= 2, /*No resource available*/
	NLM_ERR_BLOCKED				= 3, /*Lock cannot be granted immediately*/
	NLM_ERR_DENIED_GRACE_PERIOD	= 4, /*Server is in grace period after reboot*/
	NLM_ERR_DEADLCK				= 5, /*Blocking would cause a deadlock*/
	NLM_ERR_ROFS				= 6, /*Remote file system is read-only*/
	NLM_ERR_STALE_FH			= 7, /*Filehandle is invalid*/
	NLM_ERR_FBIG				= 8, /*Offset/length exceeds range supported by server*/
	NLM_FAILED					= 9, /*Failure not listed as above*/
	NLM_ERR_MAX					
} nlm_stats_t;

typedef enum nlm_proc_num
{
	NLM_NULL					= 0,	/*Do nothing*/
	NLM_TEST					= 1,	/*Test for a lock*/
	NLM_LOCK					= 2,	/*Create a lock*/
	NLM_CANCEL					= 3,	/*Cancel a lock*/
	NLM_UNLOCK					= 4,	/*Remove a lock*/
	NLM_GRANTED					= 5,	/*Lock granted*/
	NLM_TEST_MSG				= 6,	/*Test lock message*/
	NLM_LOCK_MSG				= 7,	/*Create a lock message*/
	NLM_CANCEL_MSG				= 8,	/*Cancel a lock message*/
	NLM_UNLOCK_MSG				= 9,	/*Unlock message*/
	NLM_GRANTED_MSG				= 10,	/*Lock granted message*/
	NLM_TEST_RES				= 11,	/*Test lock result*/
	NLM_LOCK_RES				= 12,	/*Create a lock result*/
	NLM_CANCEL_RES				= 13,	/*Cancel a lock result*/
	NLM_UNLOCK_RES				= 14,	/*Unlock result*/
	NLM_GRANTED_RES				= 15,	/*Lock granted result*/
	NLM_SHARE					= 20,	/*Share a file*/
	NLM_UNSHARE					= 21,	/*Unshare a file*/
	NLM_NM_LOCK					= 22,	/*Nonmonitored lock*/
	NLM_FREE_ALL				= 23	/*Free all locks*/
} nlm_proc_num_t;

typedef struct nlm_statCounters
{
	uint64  nlm_granted;			/*Lock was granted*/
	uint64	nlm_errDenied;			/*Lock was not granted due to conflict*/
	uint64	nlm_errDeniedNolocks;	/*No resource available*/
	uint64	nlm_errBlocked;			/*Lock cannot be granted immediately*/
	uint64	nlm_errDeniedGracePer;  /*Server is in grace period after reboot*/
	uint64 	nlm_errDeadLck;			/*Blocking would cause a deadlock*/
	uint64	nlm_errRofs;			/*Remote file system is read-only*/
	uint64	nlm_errStaleFh;			/*Filehandle is invalid*/
	uint64	nlm_errFbig;			/*Offset/length exceeds range supported by server*/
	uint64	nlm_errFailed;			/*Generic error*/  
} nlm_statCounters_t;


typedef struct nlm_lockCounters
{
	uint64				nlm_locks_req;		/*Number of lock requests*/
	uint64				nlm_locks_granted;	/*Number of locks granted*/
} nlm_lockCounters_t;

typedef struct nlm_debugCounters
{
    uint64              nlm_tplXmtPktAlloc;
    uint64              nlm_tplXmtPktFree;
    uint64              nlm_tplXmtPktSend;
    uint64              nlm_tplXmtPktCmpInd;
	uint64				nlm_tplRxPktHold;
	uint64				nlm_tplRxPktFree;
    uint64              nlm_tplDropRcvPkt;
	uint64				nlm_eeeAlloc;
	uint64				nlm_eeeDealloc;
    uint64              nlm_allocReqCB;
    uint64              nlm_freeReqCB;
    uint64              nlm_tplXmtNoPktResource;
	uint64				nlm_rcvInvalidFSRespErr;
	uint64				nlm_fsResourceErr;
	uint64				nlm_fsErr;
	uint64				nlm_rcvPktIndErr;
    uint64              nlm_sendUnlockReplyErr;
    uint64              nlm_rcvCancelErr;
    uint64              nlm_rcvUnlockErr;
	uint64				nlm_maxReqsDropPktErr;
	uint64				nlm_xmtNoRsrcErr;
	uint64				nlm_allocCallerName;
	uint64				nlm_freeCallerName;
	uint64				nlm_allocOh;
	uint64				nlm_freeOh;
	uint64				nlm_reqsXmtToFs; 
	uint64				nlm_rspsRcvFmFs;
    uint64      		xmtGetPortReqCnt;
    uint64      		rxGetPortReplyCnt;
    uint64      		xmtGrantedReqCnt;
    uint64      		rxGrantedReplyCnt;
    uint64      		rxGetPortLocateReqErr;
    uint64      		rxGrantedLocateReqErr;
    uint64      		rxGrantedErr;
    uint64      		txGetportErr;
    uint64      		txGrantedErr;
    uint64      		numTxGetportRetries;
    uint64      		numTxGrantedRetries;
	uint64				numOfRxBadCookies;
	uint64				numOfRxBadAlockObj;
	uint64				numOfRxGracePeriodErr;
	uint64				numOfAddNlmReqCbToList;
	uint64				numOfDelNlmReqCbFromList;
	uint64				nlm_allocCookie;
	uint64				nlm_freeCookie;
	uint64				nlm_grantedReplyErr;
	uint64				numOfAddPendingLckErr;
	uint64				numOfDeadlockErr;
	uint64				numOfCallBackErr;
	uint64				numOfAllocLockRec;
	uint64				numOfFreeLockRec;
	uint64				numOfAllocLockRecErr;
	uint64				numOfValidateLockRecRangeErr;
	uint64				numOfValidateLockRecInusedEerr;
	uint64				nlm_pktRxRetries[NLM_MAX_PROC];
} nlm_debugCounters_t;

typedef struct nlm_clientInfo
{
	uint32				nlm_version;	 	/*NLM version number*/
    nlm_statCounters_t  nlm_statCounters;    /*NLM procedure stat. counters*/
	uint32				nlmPort;			/*NLM port of last Getport*/
	void				*nlm_pBsdSockUdp;	/* BSD socket ptr */
	void				*pmap_pBsdSockUdp;	/* BSD socket ptr */
	void				*nlm_pBsdSockTcp;	/* BSD socket ptr */
	void				*pmap_pBsdSockTcp;	/* BSD socket ptr */
    uint64				nlm_rxPktCounters[NLM_MAX_PROC];
    uint64				nlm_txPktCounters[NLM_MAX_PROC];
} nlm_clientInfo_t;

typedef struct nlm_fileInfo
{
	uint64				nlm_locks_current;	/*Number of current locks*/
	uint64				nlm_locks_pending;	/*Number of locks pending*/
	void				*pNext;				/* ptr to next entry */
	nlm_lockCounters_t	nlm_counters;
} nlm_fileInfo_t;

#ifdef NOT_USED
typedef struct _nlm_lockRec_t 	nlm_lockRec_t;	 
struct _nlm_lockRec_t
{
	ushort16		flags;
#define		NLM_LOCKREC_INUSED			0x0001
#define		NLM_LOCKREC_CURR_LIST		0x0002
#define		NLM_LOCKRED_PENDING_LIST	0x0004
	uint64			nlm_l_offset;			/*Start of locked region*/
	uint64			nlm_l_len;				/*Length of locked region*/
	uint64			nlm_l_end;				/*End of locked region*/
	boolean			nlm_l_eof;				/*Lock to EOF or not*/
	boolean			nlm_exclusive;			/*Exclusive or not*/
	int32			nlm_state;				/*Client's SM state number*/
	void			*lockCBp;				/*ptr to lock owner LockCB*/
	void			*pendingLockRecp;		/*Ptr. to pending block rec*/
	void			*waitForLockRecp;		/*Ptr. to curr lock rec*/	
	DLL_ENTRY(nlm_lockRec_t)	lockList;
};
#endif

typedef struct nlm_lockInfo
{
#ifdef NOT_USED
	nlm_lockRec_t	lockRec;				/*lock record*/
#else
    uint64			nlm_l_offset;			/*Start of locked region*/
	uint64			nlm_l_len;				/*Length of locked region*/
	uint64			nlm_l_end;				/*End of locked region*/
	boolean			nlm_l_eof;				/*Lock to EOF or not*/
	boolean			nlm_exclusive;			/*Exclusive or not*/
	int32			nlm_state;				/*Client's SM state number*/
#endif
	int32			nlm_svid;				/*Unique process id*/
	uint32			nlm_callerNameLen;
	uint32			nlm_ohLen;
	char			*nlm_callerName;		/*Name of client machine*/
	char			*nlm_oh;				/*lock owner*/
} nlm_lockInfo_t;

typedef struct _nlm_lockRecAllocList_t
{
#ifdef NOT_USED
	DLL_HDR(nlm_lockRec_t)		list;
#endif
	int32						numOfEntries;
} nlm_lockRecAllocList_t;

typedef struct _nlm_lockRecOwnerList_t
{
#ifdef NOT_USED
	DLL_HDR(nlm_lockRec_t)		list;
#endif
	int32						numOfEntries;
} nlm_lockRecOwnerList_t; 

typedef struct nlm_txCmdInfo
{
	void			*pTplBind;	/* TPL Bind CB ptr */
	void			*pTplConn;	/* TPL Conn CB ptr */
	void			*pBsdSock;	/* BSD socket ptr */
	int32			cmdXid;		/* matching command/reply XID*/
	int32			cmdCookie;		/*cookie for outgoing req*/
	uint32			laddr;		/* Local IP */
	uint32			raddr;		/* Remote IP */
	ushort16		lport;		/* Local port */
	ushort16		rport;		/* Remote port */
#define	NLM_MAX_RETRY_CNT		5
#define	NLM_MAX_TIME_WAIT_CNT	3
    short16			getportRetry;			/* Tx retry count */
	int32			getportMaxTimeWaitCnt;	//Count wait for response
									//restransmit request
    short16			grantedRetry;			/* Tx retry count */
	int32			grantedMaxTimeWaitCnt;	//Count wait for response
	int32			(*dataInd)( char *pData, int32 size);/* App data upcall*/
	int32			(*eventInd)(void *nlmPortmapCb,int32 event ); /* event upcall*/
} nlm_txCmdInfo_t;

typedef struct nlm_netobj
{
#define	NLM_MAX_NETOBJ_LEN		NLM_MAX_NAME	
//	char		netobj[NLM_MAX_NETOBJ_LEN];
	char		*netobj_p;			
	ushort16	netobjLen;	
} nlm_netobj_t;

typedef struct nlm_request
{
	rpcCallHdr_t	*rpcCallHdrp;			/*Pointer RPC Call hdr*/
	uint32			reqState;
#define NLM_REQ_INPROG			1			//NLM req is in progress
#define NLM_REQ_SEND_TO_FS		2			//NLM req has sent to FS
											//Currently, the only req
											//send to FS is the 
											//Getattr to get filesize
#define NLM_REQ_REPLIED			3			//NLM req has been replied
#define NLM_REQ_DONE			4			//NLM req has been completed
#define NLM_REQ_REXMIT			5			//NLM reply has been rexmit
#define	NLM_GETPORT_CALL_SND	6			//Portmap Getport has been
											//send to retrieve NLM por//t
#define	NLM_GETPORT_REPLY_RCVD	7			//Portmap reply has been received
#define	NLM_GRANTED_CALL_SND	8			//NLM GRANTED has been sent
#define	NLM_GRANTED_REPLY_RCVD	9			//NLM GRANTED reply has been received
	char			*procStart;				/*Start of rx NLM procedure*/
	int32			procLen;				/*NLM procedure length*/
	void			*rxPktDescp;			/*Rx pkt descriptor*/
	void			*txPktDescp;			/*Tx pkt descriptor*/
	void			*clientCBp;				/*Pointer to clientCB*/
	void        	*fileCBp;				/*Pointer to fileCB*/
//	void			*lockCBp;	
#ifdef NOT_USED
	void			*lockRecp;				/*Pointer to lockRec*/
#else
	void			*lockInfop;				/*Pointer to lock_info_t*/
#endif
	void			*reqCBp;				/*Pointer to reqCB*/
	void			*tplConnp;				/*TPL conn ID*/
	void			*pTime;					/*Pointer to Time Stamp*/
	ushort16		reXmtCounter;			/*Number of times the request 
											  is retransmitted*/
	uint32			timeSendToFsInSecs;		/*Time in seconds since req
											  has sent to file system*/
	int32			fileMsgLen;				/*File system message len*/
	uchar8			*fileMsgp;				/*File system request msg*/
    uint32          block;
    nlm_lockInfo_t 	nlm_lockInformation;
	nlm_txCmdInfo_t	nlm_txCmdInfo;		
	rpcCallHdr_t	rpcHdr;
	uint32			procNum;
	nlm_netobj_t	cookie;					/*client Cookie value*/
	int32			replyStatus;			/*reply status to client*/	
} nlm_request_t;

typedef struct nlm_reply
{   
	uint32	tx_msgType;
#define NLM_TX_REG_MSG_TYPE     1
#define NLM_TX_CACHE_MSG_TYPE   2
#define NLM_TX_ERR_MSG_TYPE     3
	void	*info1p; 
	void	*info2p; 
	void	*info3p; 
} nlm_reply_t;


typedef struct nlm_globalCfg
{
	int32				eee_nlmAppId;
	int32				eee_efsAppId;
	uint32				srcPort;			 /*Control source port*/
	uint32				efsDstPort;			 /*FS destination port*/
    int32               reqTimeoutHandle;
#define     NLM_REQ_TIMEOUT_PERIOD  400
	int32				cmdXid;		/*xid in outgoing req*/
	int32				cookie;		/*cookie value for outgoing req*/	
	uint64				nlm_statCounters[NLM_ERR_MAX];
    uint64              nlm_rxPktCounters[NLM_MAX_PROC];
    uint64              nlm_txPktCounters[NLM_MAX_PROC];
	void				*pmap_pTplBindUdp;		/* TPL Bind CB ptr */
	void				*pmap_pBsdSockUdp;		/* BSD socket ptr */
	void				*pmap_pTplBindTcp;		/* TPL Bind CB ptr */
	void				*pmap_pBsdSockTcp;		/* BSD socket ptr */
	void				*nlm_pTplBindUdp;		/* UDP TPL Bind CB ptr */
	void				*nlm_pBsdSockUdp;		/* UDP BSD socket ptr */
	void				*nlm_pTplBindTcp;		/* TCP TPL Bind CB ptr */
	void				*nlm_pBsdSockTcp;		/* TCPP BSD socket ptr */
    nlm_debugCounters_t nlm_debugCounters;
	void				*pFileCbHashTbl;	
    int32               gracePeriodTimeoutHandle;
#define     NLM_GRACE_TIMEOUT_PERIOD  	60000 	/*grace period timer*/
	uint32				gracePeriod;			/*grace period flag*/
	int32				lockTblSize;
	int32				lockRecFreeCnt;			/*lockRec Free cnt*/
	int32				lockRecLwm;				/*lockRec LWM*/
#ifdef NOT_USED
	nlm_lockRec_t		*lockAllocTbl;			/*lock alloc table*/
	nlm_lockRecAllocList_t lockRecAllocList;	/*DDL of lock rec. Alloc list*/
#endif
} nlm_globalCfg_t;


/*Global variables*/
extern nlm_globalCfg_t	*nlm_globalCfgp;
extern uint32 nlm_traceEnable;
extern uint32 nlm_traceDebugEnable;
int32  rpc_processReplyHdr (char *rpcHdrp);

void xdr_swap(byte *pkt, int32 hdrSize, int32 dir);

int32
nfs_processRpcAuth(rpcCallHdr_t *pktRpcCallHdrp, 
                   rpcCallHdr_t *newRpcCallp);

/*
 *Macros
 */
#ifdef  NLM_USE_SHMEM
#define NLM_SHMEM_ALLOC     eee_allocateBlock
#define NLM_SHMEM_DEALLOC
#else
#define NLM_SHMEM_ALLOC     eee_ramAlloc
#define NLM_SHMEM_DEALLOC	eee_ramDealloc
#endif  /*NLM_USE_SHMEM*/

/*Get start and end addresses of locked region given offset and length*/
/*Also, set eof flag*/
#define NLM_GET_L_REGION_END(l_end, l_offset, l_len, l_eof)					\
	{																		\
		if ((l_len) == 0)													\
		{																	\
			l_end = 0;														\
			l_eof = TRUE;													\
		}																	\
		else																\
		{																	\
			l_end = ((l_offset) + (l_len) - 1);								\
			l_eof = FALSE;													\
		}																	\
	}


#define NLM_MAP_TO_COMP_NFS_VER(nfs_ver, nlm_ver)		\
    {                                                   \
        if ((nlm_ver) == NLM_VER4)                      \
            nfs_ver = (NFS_VER3);                       \
        else                                            \
            nfs_ver = (NFS_VER2);                       \
    }

/*----------------------
 *Return next Cmd Xid
 *----------------------
 */
static inline int32 
nlm_getNextCmdXid(void)
{
	nlm_globalCfgp->cmdXid++;
	if (nlm_globalCfgp->cmdXid < 0)
	{
		nlm_globalCfgp->cmdXid = 1;
	}

	return (nlm_globalCfgp->cmdXid);

}

/*-------------------------------
 *Return next cmd Cookie value
 *-------------------------------
 */
static inline int32 
nlm_getNextCmdCookie(void)
{
	nlm_globalCfgp->cookie++;
	if (nlm_globalCfgp->cookie < 0)
	{
		nlm_globalCfgp->cookie = 1;
	}

	return (nlm_globalCfgp->cmdXid);
}

#ifdef NOT_USED
static inline nlm_lockRec_t	*
nlm_AllocLockRec(ushort16 RecType)
{
	nlm_lockRec_t			*lockRecp;

	if (nlm_globalCfgp->lockRecFreeCnt <= 0)
	{
		printf("%s: ERR: No free Lock Records!\n",
					   	__FUNCTION__);
		return (NULL);
	}

	DLL_REMOVE_HEAD(&nlm_globalCfgp->lockRecAllocList.list, 
					lockRecp,
					lockList);
	if (lockRecp)
	{
		if (lockRecp->flags & NLM_LOCKREC_INUSED)
		{
			printf("%s: ERR: Lock Record in used!\n",
					   	__FUNCTION__);
			nlm_globalCfgp->nlm_debugCounters.numOfAllocLockRecErr++;
		}
		lockRecp->flags = (RecType | NLM_LOCKREC_INUSED);
		nlm_globalCfgp->lockRecFreeCnt--;

		if (nlm_globalCfgp->lockRecFreeCnt <
			nlm_globalCfgp->lockRecLwm)
			nlm_globalCfgp->lockRecLwm = nlm_globalCfgp->lockRecFreeCnt;
		nlm_globalCfgp->nlm_debugCounters.numOfAllocLockRec++;
		return (lockRecp);
	}

	return (NULL);
}

static inline void 
nlm_deallocLockRec(nlm_lockRec_t *lockRecp)
{
	DLL_INSERT_TAIL(&nlm_globalCfgp->lockRecAllocList.list, 
					lockRecp, lockList);

	lockRecp->flags &= ~ (NLM_LOCKREC_INUSED | 
						NLM_LOCKREC_CURR_LIST | 
						NLM_LOCKRED_PENDING_LIST);
	lockRecp->lockCBp = NULL;
	lockRecp->pendingLockRecp = NULL;
	lockRecp->waitForLockRecp = NULL;
	nlm_globalCfgp->lockRecFreeCnt++;
	nlm_globalCfgp->nlm_debugCounters.numOfFreeLockRec++;
	return;
}


static inline int32 
nlm_validateLockRec(nlm_lockRec_t *lockRecp)
{
	nlm_lockRec_t *lastLockRecp;

	lastLockRecp = nlm_globalCfgp->lockAllocTbl;
	lastLockRecp += (NLM_MAX_LOCK_REC - 1);
	if ( (lockRecp < (nlm_lockRec_t *)nlm_globalCfgp->lockAllocTbl)	||
		(lockRecp > lastLockRecp) )
	{
		nlm_globalCfgp->nlm_debugCounters.numOfValidateLockRecRangeErr++;
		return (NFX_ERR);
	}

	if (!(lockRecp->flags & NLM_LOCKREC_INUSED))
	{
		nlm_globalCfgp->nlm_debugCounters.numOfValidateLockRecInusedEerr++;
		return (NFX_ERR);
	}
	return (NFX_OK);
}
#endif

#endif  /*_NLM_H*/

