/* vi:ts=4
 *
 *-----------------------------------------------------------------
 *
 * Name:			nsm-api.h
 *
 * RCS:  $Id: nsm-api.h,v 1.1.1.1 2002/09/13 01:05:53 joseph Exp $
 *
 * Copyright (C) 2002, Agile Storage Inc.
 *
 * Description: NSM API definitions.
 *
 * Created by:		Xiaobing Zhang
 *
 * Date Created:	09/03/2002
 *
 *-----------------------------------------------------------------
 */
#ifndef _NSM_API_H
#define _NSM_API_H

#include <exports/nfp-api.h>
#include <exports/evm-api.h>

typedef struct nsm_nlmCallbackCB
{
	// Monitor complete indication
	int32   (*monCmpInd)(evm_volId_t		volId,
						 nfp_clientCB_t		*nlm_clientCBp,
						 int32				status		//NFX_OK or NFX_ERR
						 );

	// Unmonitor complete indication
	int32   (*unmonCmpInd)(evm_volId_t		volId,
						   nfp_clientCB_t	*nlm_clientCBp,
						   int32			status		//NFX_OK or NFX_ERR
						   );

	// Client reboot indication
	int32	(*clientDownInd)(uint32			hostIpAddr);
} nsm_nlmCallbackCB_t;

/*-----------------------------------------------------------------
 * Name:        nsm_registerCallback
 *
 * Description: NLM registers callback functions for mon and unmom
 *              requests.
 *
 * Created by:  Xiaobing Zhang
 *
 * Date Created:    09/03/2002
 *
 *-----------------------------------------------------------------
 */
void
nsm_registerCallback(nsm_nlmCallbackCB_t *callbackCBp);


/*-----------------------------------------------------------------
 * Name:        nsm_procMonReq
 *
 * Description: Establish monitoring of a given host.
 *              Called by NLM module.
 *
 * Created by:  Xiaobing Zhang
 *
 * Date Created:    08/01/2002
 *
 *-----------------------------------------------------------------
 */
int32       //NFX_OK or NFX_ERR
nsm_procMonReq(char				*hostName,
			   uint32			hostIpAddr,
			   evm_volId_t		volId,
			   nfp_clientCB_t	*nlm_clientCBp);    //NLM clientCB

/*-----------------------------------------------------------------
 * Name:        nsm_procUnmonReq
 *
 * Description: Stop monitoring a given host.
 *              Called by NLM module.
 *
 * Created by:  Xiaobing Zhang
 *
 * Date Created:    08/16/2002
 *
 *-----------------------------------------------------------------
 */
int32       //NFX_OK or NFX_ERR
nsm_procUnmonReq(char			*hostName,
				 uint32			hostIpAddr,
				 evm_volId_t	volId,
				 nfp_clientCB_t	*nlm_clientCBp);    //NLM clientCB


#endif  /*_NSM_API_H*/

