/* vi:ts=4
 *------------------------------------------------------------
 *
 * Name:        nsm-pmapTpl.c
 *
 * RCS:  $Id: nsm-pmapTpl.c,v 1.2 2002/10/17 01:38:56 shaun Exp $
 *
 * Copyright (C) 2001, Agile Storage Inc.
 *
 * Description:
 *
 * Created by: Xiaobing Zhang
 *
 * Date Created:        8/21/2002
 *
 *--------------------------------------------------------------
 */

#include "nsm.h"

/*-----------------------------------------------------------------
 * Name:		nsm_pmapTplAddConnInd    	
 *
 * Description: Add connection indication from the transport layer
 *	            to add a new connection. 
 *
 * Created by:	Xiaobing Zhang
 *
 * Date Created:	08/21/2002
 *
 *-----------------------------------------------------------------
 */
int32		// NFX_OK or NFX_ERR
nsm_pmapTplAddConnInd(tpl_bindCb_t	*bindp,		
					  void 			*connHdlp, 
					  void 			**pAppHdl, 
					  uint32 		raddr, 
					  ushort16 		rport)
{
	nsm_clientCB_t	*clientCBp;
	tpl_connCb_t	*pConn;

	NSM_PRINTF(LOG_DEBUG, ("nsm_pmapTplAddConnInd: "
							"pConnHdl: %p raddr: %08x rport: %d",
							connHdlp, raddr, rport));
	clientCBp = nsm_getClientCB(raddr);
	if (clientCBp == NULL)
	{   
		// We didn't request NSM port of client with raddr IP address
		NSM_PRINTF(LOG_ERR, ("nsm_pmapTplAddConnInd: "
						"Didn't ask for NSM port from host (%x)."
						"Drop the connection.", raddr));
		return (NFX_ERR);
    }

	pConn = (tpl_connCb_t *)connHdlp;
	clientCBp->pmap_pBsdSock = pConn->pSock;
	*pAppHdl = clientCBp;
	return (NFX_OK);
} // nsm_pmapTplAddConnInd()

/*-----------------------------------------------------------------
 * Name:		nsm_pmapTplDelConnInd 		
 *
 * Description: Delete connection indication from the transport layer
 *              to delete an existing connection.
 *
 * Created by:	Xiaobing Zhang
 *
 * Date Created:	08/21/2002
 *
 *-----------------------------------------------------------------
 */
int32
nsm_pmapTplDelConnInd(void *hdlp, void *connHdlp)
{

	NSM_PRINTF(LOG_INFO, ("nsm_pmapTplDelConnInd: connection %p closed",
							connHdlp));

	return (NFX_OK);
} // nsm_pmapTplDelConnInd()

/*-----------------------------------------------------------------
 * Name:		nlm_tplRcvPktInd 
 *
 * Description: Receive packet indication from the transport layer
 *	            for processing.
 *
 * Created by:  Xiaobing Zhang
 *
 * Date Create:		08/21/2002
 *
 *-----------------------------------------------------------------
 */
int32
nsm_pmapTplRcvInd(void *hdlp, pkt_desc_t *descp)
{
	int32				rc;
	nsm_clientCB_t		*clientCBp;
	nsm_requestCB_t		*reqCBp;
	rpcReplyHdr_t		*rpcReplyHdrp;
	int32				rpcReplyHdrLen;
	char				*portmapperMsgp;
	int32				clientPort;
	esm_event_t			evt;

	if ( ( hdlp == NULL) || (descp == NULL) || (descp->pHdr == NULL ) )
		return (TPL_FREE_PKT);

	if (!(descp->flags & PKT_DESC_INUSE))
	{
		panic("%s: bad pkt=%x flags=%x\n",
			 __FUNCTION__,descp, descp->flags);
    }

	clientCBp = (nsm_clientCB_t *)hdlp;

	//-------------------------------
	// Perform Net to Host conversion
	//-------------------------------
	//xdr_swap(descp->pHdr,(sizeof(rpcCallHdr_t)-sizeof(rpc_authSysInfo_t)), 0);

	NSM_PRINTF(LOG_DEBUG, ("nsm_pmapTplRcvInd: hdrSize: %d", descp->hdrSize));
	xdr_swap(descp->pHdr, descp->hdrSize, 0);

	//TODO:  trace
	//
	//-------------------
	// Process RPC header
	//-------------------
	portmapperMsgp = descp->pHdr;
	rpcReplyHdrp = (rpcReplyHdr_t *)descp->pHdr;
	rpcReplyHdrLen = rpc_processReplyHdr(descp->pHdr);
	NSM_PRINTF(LOG_DEBUG, ("nsm_pmapTplRcvInd: "
							"rpcReplyHdrLen: %d", rpcReplyHdrLen));

	if (rpcReplyHdrLen == NFX_ERR)
	{
		goto nsm_pmapTplRcvIndExit;
	}

	//----------------------------------
	// Validate the reply by the RPC Xid
	//----------------------------------
	reqCBp = clientCBp->notifyReqp;
	if (reqCBp == NULL)
	{
		// This may happen when the last retry of getPortReq timeout
		// before this reply is received.
		NSM_PRINTF(LOG_ERR, ("nsm_pmapTplRcvInd: "
								"clientCBp->notifyReqp is NULL"));
		nsm_globalCfgp->nsm_debugCounters.nsm_rcvGetPortLocateReqErr++;
		goto nsm_pmapTplRcvIndExit;
	}
	NSM_PRINTF(LOG_DEBUG, ("nsm_pmapTplRcvInd: reqCBp: %p, replyXid: %d", 
								reqCBp, rpcReplyHdrp->rpc_xid));
	if (reqCBp->txCmdInfo.cmdXid != rpcReplyHdrp->rpc_xid)
	{
		// Reply Xid doesn't match the request Xid
		nsm_globalCfgp->nsm_debugCounters.nsm_rcvGetPortLocateReqErr++;
		goto nsm_pmapTplRcvIndExit;
	}

	//--------------------------------------------------
	// Skip pass RPC header to portmapper of the message
	//--------------------------------------------------
	portmapperMsgp += rpcReplyHdrLen;

	nsm_globalCfgp->nsm_debugCounters.nsm_rxGetPortRspPktCnt++;

	clientPort = *(uint32 *)portmapperMsgp;

	NSM_PRINTF(LOG_DEBUG, ("nsm_pmapTplRcvInd: client: %s nlmPort: %d",
							clientCBp->hostName, clientPort));

	//-------------------------------------------------------
	// Send event to the thread that is waiting for the reply
	//-------------------------------------------------------
	evt.event = NSM_MSG_PMAP_RESP;
	evt.p1.u.w = clientPort;
	evt.p2.u.p = NULL;

	rc = thread_event(clientCBp->notifyThreadId, &evt);

nsm_pmapTplRcvIndExit:
	return TPL_FREE_PKT;
}

/*-----------------------------------------------------------------
 * Name:		nsm_pmapTplXmtCmpInd 
 *
 * Description: Transmit packet complete indication from the 
 *              transport layer.
 *
 * Created by:  Xiaobing Zhang
 *
 * Date Created:	08/21/2002
 *
 *-----------------------------------------------------------------
 */
int32
nsm_pmapTplXmtCmpInd( void *pHdl, pkt_desc_t *pDesc)
{
	//Free it back to TPL
	return TPL_FREE_PKT;
}

#if defined(NFX_SMP)
/*-----------------------------------------------------------------
 * Name: nsm_portMapBindReqAck
 *
 * Description:	bind request ack
 *	
 *
 * Created by:  Su Wang
 *
 * Date Created:        12/04/02
 *
 *-----------------------------------------------------------------
 */
int32 nsm_portMapBindReqAck(void* pHandle, int32 rc, void *in_pBind)
{

	tpl_bindCb_t	*pBind = (tpl_bindCb_t*)in_pBind;
	
	if (rc != NFX_OK)
		panic("%s: tpl bind failed\n", __FUNCTION__);
	
	if (pBind->tplProt == TPL_PROT_UDP) {
		nsm_globalCfgp->pmap_pBsdSock = pBind->pBsdSock;
		nsm_globalCfgp->pmap_pTplBind = pBind;
	}
//	else
//		nsm_globalCfgp->pMap_pTplBindTcp = pBind;
	
	return NFX_OK;
}
#endif /* NFX_SMP */

/*-----------------------------------------------------------------
 * Name:		nsm_portMapBindReq	 
 *
 * Description: Binds NSM Portmapper Getport to Transport Layer (UDP).  
 *
 * Created by:	Xiaobing Zhang
 *
 * Date Created:	08/21/2002
 *
 *-----------------------------------------------------------------
 */
int32
nsm_portMapBindReq (uint32 ipAddr)
{
	ushort16		protocol;
	int32			rc;
	int32			bindOpt;
	tpl_bindCb_t	bindReq;
#if !defined(NFX_SMP)
	tpl_bindCb_t	*tplRspBindUdp;
	tpl_bindCb_t	*tplRspBindTcp;
#endif /* NFX_SMP */

	protocol = TPL_PROT_UDP;
    while (1)
	{
		memset( &bindReq, 0, sizeof(tpl_bindCb_t));

		bindOpt = TPL_BIND_SERVER | TPL_BIND_USE_RPC;
		bindReq.laddr   	  = ipAddr;
		bindReq.lport   	  = ntohs(NSM_PMAP_PORT);	
		bindReq.tplProt 	  = protocol;
		bindReq.ul_addConnInd = nsm_pmapTplAddConnInd;
		bindReq.ul_delConnInd = nsm_pmapTplDelConnInd;
		bindReq.ul_rcvPktInd  = nsm_pmapTplRcvInd;
		bindReq.ul_xmtCmpInd  = nsm_pmapTplXmtCmpInd;

#if defined(NFX_SMP)
		bindReq.ul_bindReqAck = nsm_portMapBindReqAck;
		rc = tpl_bindReq(&bindReq, NULL, bindOpt);
		
		if (protocol == TPL_PROT_TCP)
			break;
#else
		if (protocol == TPL_PROT_UDP)
		{
			rc = tpl_bindReq( &bindReq, &tplRspBindUdp, bindOpt);
			if (rc != NFX_OK)
				panic("nsm_portMapBindReq: bind failure (UDP)!\n");
			nsm_globalCfgp->pmap_pTplBind = tplRspBindUdp;
			nsm_globalCfgp->pmap_pBsdSock = tplRspBindUdp->pBsdSock;
		}
		else
		{

			rc = tpl_bindReq( &bindReq, &tplRspBindTcp, bindOpt);
			if (rc != NFX_OK)
				panic("nsm_portMapBindReq: bind failure (TCP)!\n");
			//nsm_globalCfgp->pmap_pTplBindTcp = tplRspBindTcp;
			//nsm_globalCfgp->pmap_pBsdSockTcp = tplRspBindTcp->pBsdSock;
			break;
		}
#endif /* NFX_SMP */

		protocol = TPL_PROT_TCP;

	}
	return (NFX_OK);
}
