/* vi:ts=4
 *
 *-----------------------------------------------------------------
 *
 * Name:			nsm.h
 *
 * RCS:  $Id: nsm.h,v 1.5 2002/10/17 01:38:56 shaun Exp $
 *
 * Copyright (C) 2002, Agile Storage Inc.
 *
 * Description: NSM protocol definitions.
 *
 * Created by:		Xiaobing Zhang
 *
 * Date Created:	07/31/2002
 *
 *-----------------------------------------------------------------
 */
#ifndef _NSM_H
#define _NSM_H

#include <nfx-incl.h>
#include <exports/nfs-api.h>
#include <exports/pkt-api.h>
#include <exports/malloc-api.h>
#include <exports/cm-api.h>
#include <exports/hash-api.h>
#include <exports/thread-api.h>
#include <exports/file-api.h>
#include <exports/bsdsock-api.h>
#include "../sm-tpl/tpl.h"
#include "nsm-api.h"

//#define NSM_UNIT_TEST

#define NSM_VER1			1		//NSM versions

#define	NSM_PROG			100024
#define	NSM_PORT			2092
#define	NSM_PMAP_PORT		2093
#define	NSM_MIN_PROC		0
#define	NSM_MAX_PROC		7

#define NSM_STAT_SUCC		0		//Status monitor agrees to monitor
#define NSM_STAT_FAIL		1		//Status monitor cannot monitor

#define NSM_MAX_NAME		1024

#define NSM_LOCK_FILE_NAME		".nsm_lock_hosts"
#define NSM_LOCK_FILE_NAME_TMP	".tmp_nsm_lock_hosts"

#ifdef NSM_UNIT_TEST
#define NSM_FILEBUF_SIZE	72
#else
#define NSM_FILEBUF_SIZE	(4 * 1024)
#endif //NSM_UNIT_TEST

/* File open mode */
#define NSM_FILE_OPENMODE_READ		0x01
#define NSM_FILE_OPENMODE_WRITE		0x02

/* Timeout */
#define NSM_TIMEOUT_WAIT_VOLS_UP	(30 * 1000)	//In milliseconds.
												//Wait for volumes up
#define NSM_TIMEOUT_WAIT_FS_UP		30			//In seconds. Wait for fs up
#define NSM_TIMEOUT_EVM				30			//In seconds. Thread timeout
												//while waiting for EFS resp.
#define NSM_TIMEOUT_EFS				30			//In seconds. Thread timeout
												//while waiting for EFS resp.
#define NSM_TIMEOUT_PMAP			30 	//TODO	//In seconds. Thread timeout
												//while waiting for pmap resp.
#define NSM_TIMEOUT_NOTIFY			30	//TODO 	//In seconds. Thread timeout
												//while waiting for notify resp.

#define NSM_MAX_RETRIES_FS_UP	3		//Number of retries of sending message
										//to fs to see if a file system is up

#define NSM_MAX_RETRIES_REQ_MSG	3		//Number of retries of sending a request
										//message to client

#define NSM_DECODE_RX	1
#define NSM_DECODE_TX	2

/* NSM procedure numbers */
typedef enum nsm_proc_num
{
	NSM_NULL					= 0,	//Do nothing
	NSM_STAT					= 1,	//Check status
	NSM_MON						= 2,	//Monitor a host
	NSM_UNMON					= 3,	//Unmonitor a host
	NSM_UNMON_ALL				= 4,	//Unmonitor all hosts
	NSM_SIMU_CRASH				= 5,	//Simulate a crash
	NSM_NOTIFY					= 6,	//Notify a host
} nsm_proc_num_t;

/* NSM thread events */
/* (The event number starts from an offset of NFS_MSG_BASE) */
typedef enum nsm_msg
{
	NSM_MSG_MON = NFS_MSG_BASE + 200,	//TODO: use other message base?
	NSM_MSG_MON_RESP,
	NSM_MSG_UNMON,
	NSM_MSG_UNMON_RESP,
	NSM_MSG_NOTIFY,
	NSM_MSG_PMAP_RESP,
	NSM_MSG_NOTIFY_RESP,

	NSM_MSG_MAX
} nsm_msg_t;

/* Statistics */
typedef struct nsm_statCounters
{
    uint64  nsm_succ;                //SM agrees to monitor
    uint64  nsm_fail;                //SM cannot monitor
    uint64  nsm_errNotSupp;         //Operation not supported
    uint64  nsm_errSrvFault;        //A failure on the server
} nsm_statCounters_t;

/* Debug counters */
typedef struct nsm_debugCounters
{
    uint64              nsm_tplXmtPktAlloc;
    uint64              nsm_tplXmtPktFree;
    uint64              nsm_tplXmtPktSend;
    uint64              nsm_tplXmtPktCmpInd;
	uint64				nsm_tplRxPktHold;
	uint64				nsm_tplRxPktFree;
    uint64              nsm_tplDropRcvPkt;
	uint64				nsm_eeeAlloc;
	uint64				nsm_eeeDealloc;
    uint64              nsm_allocReqCB;
    uint64              nsm_freeReqCB;
	uint64				nsm_reqsXmtToFs;	 //Number of req sent to FS
	uint64				nsm_rspsRcvFmFs;	 //Number of resp rcvd from FS
	uint64				nsm_reqsXmtToEvm;	 //Number of req sent to EVM
	uint64				nsm_rspsRcvFmEvm;	 //Number of resp rcvd from EVM
	uint64				nsm_evtToThreadSend;
	uint64				nsm_evtToThreadRcvd;
	uint64				nsm_evtToThreadSendErr;
	uint64				nsm_msgFromThreadSend;
	uint64				nsm_msgFromThreadRcvd;

    uint64              nsm_tplXmtNoPktRsrcErr;
    uint64              nsm_tplXmtPktSendErr;
	uint64				nsm_tplBadDescErr;
	uint64				nsm_tplInvalidPktRcvdErr;
	uint64				nsm_rcvPktIndErr;
	uint64				nsm_rcvRpcHdrErr;
	uint64				nsm_txGetPortReqPktErr;
	uint64				nsm_txNotifyReqPktErr;
	uint64				nsm_rcvGetPortLocateReqErr;
	uint64				nsm_rcvNotifyLocateReqErr;

    uint64              nsm_rxReqPktCnt[NSM_MAX_PROC];
    uint64              nsm_txRspPktCnt[NSM_MAX_PROC];
	uint64				nsm_txNotifyReqPktCnt;
	uint64				nsm_rxNotifyRspPktCnt;
	uint64				nsm_pktNotifyRetries;
	uint64				nsm_txGetPortReqPktCnt;
	uint64				nsm_rxGetPortRspPktCnt;
	uint64				nsm_pktGetPortRetries;
    uint64              allocMsgToThread;
    uint64              freeMsgToThread;
	/*
	uint64				totalUdpCalls;
	uint64				totalBadUdpCalls;
	uint64				totalTcpCalls;
	uint64				totalBadTcpCalls;
	*/
} nsm_debugCounters_t;

typedef struct nsm_volumeCB
{
	evm_volId_t		volId;
	uint32			nameLen;
	char			volName[0];			//Max length is (VOL_MAX_NAME + 1)
} nsm_volumeCB_t;

/* Monitored client */
typedef struct nsm_clientCB
{
	uint32				status;
#define NSM_CLIENT_STATUS_RECOV	0x01	//Client is in recovery stage
#define NSM_CLIENT_STATUS_MON	0x02	//Client is monitored
#define NSM_CLIENT_STATUS_UNMON	0x03	//Client is unmonitored

	nfp_clientCB_t	*nlm_clientCBp;		//Pointer to NLM clientCB
	nfp_list_t		volumeList;
	nfp_list_t		clientReqList;		//List of requests received from client
	//nfp_list_t		srvrReplyList;		//List of replies sent to client
	esm_handle_t	notifyThreadId;		//Thread used to send notify request
	struct nsm_requestCB	*notifyReqp;	//SM_NOTIFY request sent to client 
	uint32			notifyReXmtCnt;		//Number of retransmition of SM_NOTIFY
	void			*nsm_pBsdSock;		// BSD socket ptr
	void			*pmap_pBsdSock;		// BSD socket ptr
	uint32			hostIpAddr;			//Client's IP address
	uint32			hostPort;			//NSM port of last Getport
	uint32			nameLen;			//Length of client's host name
	char			hostName[0];		//NULL terminated name of client's host
										//Max length is (NSM_MAX_NAME + 1)
} nsm_clientCB_t;

/* NSM request CB */
typedef struct nsm_txCmdInfo
{
	void			*pTplBind;		//TPL Bind CB ptr
	void			*pTplConn;		//TPL Conn CB ptr
	void			*pBsdSock;		//BSD socket ptr
	int32			cmdXid;			//matching command/reply XID
	uint32			laddr;			//Local IP
	uint32			raddr;			//Remote IP
	ushort16		lport;			//Local port
	ushort16		rport;			//Remote port
	//short16		retry;			//Tx retry count
	//int32			tmr_id;			//Timer ID
	//int32			maxTimeWaitCnt; //Count wait for response
									//restransmit request
	//int32			(*dataInd)( char *pData, int32 size);	//App data upcall
	//int32			(*eventInd)(void *nlmPortmapCb,int32 event );
															//event upcall
} nsm_txCmdInfo_t;

typedef struct nsm_requestCB
{
	rpcCallHdr_t	*rpcCallHdrp;			//Pointer RPC Call hdr
	uint32			reqState;
#define NSM_REQ_INPROG			1			//NSM req is in progress
#define NSM_REQ_REPLIED			2			//NSM req has been replied
#define NSM_REQ_DONE			3			//NSM req has been completed
#define NSM_REQ_REXMIT			4			//NSM reply has been rexmit

	nsm_clientCB_t	*clientCBp;				//Pointer to clientCB
	char			*procStart;				//Start of rx NSM procedure
	int32			procLen;				//NSM procedure length
	void			*rxPktDescp;			//Rx pkt descriptor
	void			*txPktDescp;			//Tx pkt descriptor
	void			*tplConnp;				//TPL conn ID
	void			*pTime;					//Pointer to Time Stamp
	ushort16		reXmtCounter;			//Number of times the request 
											//is retransmitted
	nsm_txCmdInfo_t	txCmdInfo;
	struct nsm_requestCB	*prevp;
	struct nsm_requestCB	*nextp;
} nsm_requestCB_t;

/* Requeset event sent to thread */
typedef struct nsm_event
{
	evm_volId_t		volId;				//VolId
	nsm_clientCB_t	*nsm_clientCBp;		//Pointer to nsm clientCB
	nfp_clientCB_t	*nlm_clientCBp;		//Pointer to nlm clientCB
	nsm_requestCB_t	*nsm_requestCBp;	//Pointer to nsm requestCB
	uint32			ipAddr;				//IP address of host
	uint32			nameLen;			//Length of host name
	char			hostName[0];		//NULL terminated host name
										//Max length is (NSM_MAX_NAME + 1)
} nsm_event_t;

/* Response message sent from thread */
typedef struct nsm_message
{
	uint32			msgType;
	uint32			status;
	nfp_clientCB_t	*nlm_clientCBp;
	nsm_requestCB_t	*nsm_requestCBp;	//Pointer to nsm requestCB
	evm_volId_t		volId;
	uint32			state;
} nsm_message_t;

/* NSM reply CB */
typedef struct nsm_replyCB
{   
	uint32	tx_msgType;
#define NSM_TX_REG_MSG_TYPE     1
#define NSM_TX_CACHE_MSG_TYPE   2
#define NSM_TX_ERR_MSG_TYPE     3
	void	*info1p; 
	void	*info2p; 
	void	*info3p; 
	struct nsm_replyCB		*prevp;
	struct nsm_replyCB		*nextp;
} nsm_replyCB_t;

/* Record stored in lock hosts file */
typedef struct nsm_fileRecord
{
	uint32		ipAddr;					//IP address of host
	uint32		nameLen;				//Length of host name
	char		hostName[NSM_MAX_NAME + 1];	//Host name
} nsm_fileRecord_t;

/* Buffer containing partial content of lock host file */
typedef struct nsm_fileBuffer
{
	uint32				position;		//Current position in the buffer
	uint32				size;			//Number of types read from file
	char				*buf;			//Buffer that stores file content
} nsm_fileBuffer_t;

/* Message to be sent to thread */
typedef struct nsm_msgToThreadCB
{
	struct nsm_msgToThreadCB	*prevp;
	struct nsm_msgToThreadCB	*nextp;
	int32				msgType;
	nsm_clientCB_t		*nsm_clientCBp;
	nfp_clientCB_t		*nlm_clientCBp; //NLM clientCB
	nsm_requestCB_t		*nsm_requestCBp;
	evm_volId_t			volId;
	uint32				hostIpAddr;
	char				hostName[0];	//NULL terminated host name
										//Max length is (NSM_MAX_NAME + 1)
} nsm_msgToThreadCB_t;

/* NSM global config */
typedef struct nsm_globalCfg
{
	int32				eee_nsmAppId;
	uint32				ctrlSrcPort;		//Control source port
	uint32				mgmtSrcPort;		//Control source port
	uint32				efsDstPort;			//FS destination port
	uint32				evmDstPort;			//FS destination port
	void				*nsm_pTplBind;		// TPL Bind CB ptr
	void				*nsm_pBsdSock;		// BSD socket ptr
	void				*pmap_pTplBind;		// TPL Bind CB ptr
	void				*pmap_pBsdSock;		// BSD socket ptr
	nsm_nlmCallbackCB_t	nlmCallbackFunc;
	uint32				state;				//Server state number
	uint32				clientCnt;			//Number of clients in the list
	address_t			nsm_clientHash;		//Hash table of monitored hosts
#define NSM_HASH_CLI_BUCKETS	(NFP_MAX_CLIENT / 2)
	address_t			nsm_volumeHash;		//Hash table of all volumes
#define NSM_HASH_VOL_BUCKETS	(MAX_IDS / 2)
	esm_handle_t		threadId;			//Id of thread to do recovery proc
#ifdef NOTUSED
    int32               reqTimeoutHandle;
#define     NSM_REQ_TIMEOUT_PERIOD  200
#endif //NOTUSED
	nsm_msgToThreadCB_t	*msgToThreadListHead;	//Head of waiting list of 
												//messages to be sent to thread
	nsm_msgToThreadCB_t	*msgToThreadListTail;	//Tail of waiting list of
												//messages to be sent to thread
	nsm_fileBuffer_t 	readBuf;			//Buffer used while reading file
	nsm_fileBuffer_t 	writeBuf;			//Buffer used while writing file
	nsm_statCounters_t  nsm_statCounters;	//NSM procedure statistic
    nsm_debugCounters_t nsm_debugCounters;
} nsm_globalCfg_t;


/*
 * Global variables
 */
extern nsm_globalCfg_t	*nsm_globalCfgp;
extern uint32			nsm_traceEnable;
extern boolean			nsm_firstMsgToThread;

/*
 * Macros
 */

/*
#define NSM_PRINTF(_lvl_, _par_)											\
	E_LOG(class_1, _lvl_, nsm_globalCfgp->eee_nsmAppId, 0, 0, 0, _par_);	\
	if (nsm_traceEnable) {printf _par_; printf("\n");}
*/
#define NSM_PRINTF(_lvl_, _par_)											\
	if (nsm_traceEnable) {printf _par_; printf("\n");}

#ifdef  NSM_USE_SHMEM
#define NSM_SHMEM_ALLOC     eee_allocateBlock
#define NSM_SHMEM_DEALLOC
#else
#define NSM_SHMEM_ALLOC(_p_, _type_, _size_)								\
	{																		\
		_p_ = (_type_ *) eee_ramAlloc(_size_);								\
		if (_p_ == NULL)													\
		{																	\
			NSM_PRINTF(LOG_ERR, ("%s: eee_ramAlloc() failed", __FUNCTION__)) \
		}																	\
		else																\
		{																	\
			nsm_globalCfgp->nsm_debugCounters.nsm_eeeAlloc++;				\
		}																	\
	}
#define NSM_SHMEM_DEALLOC(_p_)												\
	{																		\
		if (_p_ != NULL)													\
		{																	\
			eee_ramDealloc(_p_);											\
			_p_ = NULL;														\
			nsm_globalCfgp->nsm_debugCounters.nsm_eeeDealloc++;				\
		}																	\
	}
#endif  /*NSM_USE_SHMEM*/


/*
 * Prototypes
 */

// nsm-init.c
int32
nsm_init(void);

// nsm-tpl.c
int32
nsm_tplBindReq(uint32 ipaddr);

// nsm-pmapTpl.c
int32
nsm_portMapBindReq (uint32 ipAddr);

// nsm-file.c
int32
nsm_fileOpen(char *fullPathName, int32 mode, uint32 *count);

void
nsm_fileClose(int32 fd, uint32 count);

int32
nsm_fileGetHostCount(int32 fd, uint32 *countp);

int32
nsm_fileGetNextRecord(int32 fd, nsm_fileRecord_t *recordp);

int32
nsm_addHostToFile(char *volName, uint32 ipAddr, char *hostName);

int32
nsm_removeHostFromFile(char *volName, uint32 ipAddr, char *hostName);

// nsm-thread.c
void
nsm_process(esm_handle_t handle, void *userInfo);

// nsm-proc.c
int32
(*nsm_procs[NSM_MAX_PROC]) (nsm_requestCB_t *nsmReqp);

int32
nsm_procRxMonReq(nsm_requestCB_t *reqCBp);

int32
nsm_procRxUnmonReq(nsm_requestCB_t *reqCBp);

int32
nsm_procRxUnmonAllReq(nsm_requestCB_t *reqCBp);

int32
nsm_procRxSimuCrashReq(nsm_requestCB_t *reqCBp);

int32
nsm_procRxNullReq(nsm_requestCB_t *reqCBp);

int32
nsm_procRxStatReq(nsm_requestCB_t *reqCBp);

int32
nsm_procRxNotifyReq(nsm_requestCB_t *reqCBp);

int32
nsm_procTxGetPortReq(nsm_requestCB_t *reqCBp);

int32
nsm_procTxNotifyReq(nsm_requestCB_t *reqCBp);

int32
nsm_procTxNotifyResp(nsm_requestCB_t *reqCBp);

int32
nsm_procTxPkt(pkt_desc_t *tplXmitPktDescp, nsm_requestCB_t *reqCBp);

int32
nsm_procTxSendResp(nsm_requestCB_t *reqCBp, int32 status);

int32
nsm_procTxSendErrResp(nsm_requestCB_t	*reqCBp,
					  int32				status);

// nsm-msg.c
int32
nsm_msgGetVolCfgReq(evm_volId_t id);

int32
nsm_msgGetVolNameListReq();

int32
nsm_msgViewFsReq(evm_volId_t volId);

int32
nsm_msgToThread();

int32
nsm_msgFromThread(evm_volId_t		volId,
				  nfp_clientCB_t	*nlm_clientCBp,
				  nsm_requestCB_t	*nsm_requestCBp,
				  uint32			status,
				  int32				msgType);

void
nsm_msgRcv(eee_desc_t *eDescp);

// nsm.c

int32
nsm_reqTimeoutHandler(	int32 timer_handle,
						void* param1, void* param2,
						void* param3, void* param4);

uint32
nsm_clientHashFunc(void *key);

int32
nsm_clientHashCompare(nsm_clientCB_t *stored, nsm_clientCB_t *compare);

void
nsm_clientHashRelease(nsm_clientCB_t *clientCBp);

uint32
nsm_volumeHashFunc(void *key);

int32
nsm_volumeHashCompare(nsm_volumeCB_t *stored, nsm_volumeCB_t *compare);

void
nsm_volumeHashRelease(nsm_volumeCB_t *volumeCBp);

nsm_clientCB_t *
nsm_addClientCB(uint32 ipAddr, uint32 nameLen, char *hostName);

void
nsm_delClientCB(nsm_clientCB_t *clientCBp);

nsm_clientCB_t *
nsm_getClientCB(uint32 clientIpAddr);

nsm_volumeCB_t *
nsm_addVolumeCB(evm_volId_t volId, char *volName);

nsm_volumeCB_t *
nsm_getVolumeCB(evm_volId_t volId);

nfp_node_t *
nsm_findVolumeCBNode(nfp_list_t *nodeListp, evm_volId_t volId);

nsm_requestCB_t *
nsm_allocReqCB(nsm_clientCB_t *clientCBp);

nsm_requestCB_t * 
nsm_findReqCB(nsm_clientCB_t	*clientCBp,
			  uint32			xid,
			  uint32			procNum);

void
nsm_delReqCB(nsm_requestCB_t *reqCBp, int32 rmFromList);

int32
nsm_addMsgToThreadCB(char				*hostName,
					 uint32				hostIpAddr,
					 evm_volId_t		volId,
					 nsm_clientCB_t		*nsm_clientCBp,
					 nfp_clientCB_t		*nlm_clientCBp,
					 nsm_requestCB_t	*nsm_requestCBp,
					 int32				msgType);

nsm_msgToThreadCB_t * nsm_getMsgToThreadCB();

void
nsm_decodeReqPkt(pkt_desc_t *descp, int32 direction);

void
nsm_decodeRspPkt(pkt_desc_t *tplXmitPktDescp, nsm_requestCB_t *reqCBp);

void
nsm_showStats();

// nsm-cmd.c
int32
nsm_cmd(int32 argc, char *argv[]);


//-----------------
// extern functions
//-----------------
extern void
xdr_swap(byte *pkt, int32 hdrSize, int32 dir);

extern int32
pkt_validateDesc( pkt_desc_t *pDesc, int32 chkDup);

extern uint32
rpc_getHdrSize(rpcCallHdr_t *pktRpcCallHdrp);

extern int32
rpc_processReplyHdr (char *rpcHdrp);

extern int32
nfs_addCBToList(nfp_list_t *nodeListp, void *CtrlBlkp, uint32 nodeType);

extern int32
nfs_delNode(nfp_list_t *nodeListp, nfp_nodePtr_t delNodep);

extern int32
nfs_txDecodeString (char		*sourceBufp,
					char		*destNamep,
					ushort16	*nameLen);

extern int32
nfs_txExtractLongVal(char *nfsMsgp, uint32 *value);

extern int32
nlm_getNextCmdXid(void);

extern int32
nlm_msgRpcCallHdr(char *msgp, uint32 cmdXid, uint32 rpcProgNum,
					uint32 rpcProgVer, uint32 rpcProcNum);
extern void
nsm_clearStats();

extern void 
xdr_swap(byte *pkt, int32 hdrSize, int32 dir);

#endif  /*_NSM_H*/

