/*
 * Name:  nvram.h
 *
 * RCS:  $Id: nvram.h,v 1.1 2001/09/13 04:26:52 janv Exp $
 *
 * Copyright (C) 2001, Agile Storage Inc.
 *
 * Description:  Internal NVRAM type definitions.
 * 
 * Created By:  Jan Vanhoutte
 *
 * Date Created:  8/20/2001
 */

#ifndef _NVRAM_H_
#define _NVRAM_H_

/*
 * Header for a nvram, located at offset 0
 */

typedef struct nvram_header
{
    uint32      magic;
#define NVRAM_HDR_MAGIC        0xd00fbead
    uint32      version;
#define NVRAM_VERSION_ONE  0x00000001

    uint32      mode;           // see nvram-api.h
    uint32      nvramSize;      // see nvram-api.h
    uint32      segments;       // see nvram-api.h
    uint32      blockSize;      // see nvram-api.h

    uint32      blocks;         // Total number of blocks
                                // ((NVRAM_NVRAM_SIZE - Overhead ) / NVRAM_BLOCK_SIZE)

    uint32      timeStamp;
    uint32      cksum;          // Checksum over the Hdr.

    uint32      spare[1];       // Pad to 64-bit (if needed ??)
} nvram_header_t ;



/*
 * The segment structure in the nvram
 * These are located in the NVRAM, just after the nvram_Hdr 
 * The number of structures is defined by nvram_Hdr.segments
 * The first one is used internally for keeping all free blocks
 *  and related statistics
 */

typedef struct nvram_segment
{
    uint32             magic;
#define NVRAM_SEGMENT_MAGIC        0x5e6ebead

    nvram_clientId_t   clientId;       // See nvram-api.h
    nvram_cookie_t     clientCookie;   // See nvram-api.h (uint64 !!)
  
    uint32      startBlk;              // Index of starting block for this segment
    uint32      numBlks;               // Number of blocks in this segment

    uint32      Handle;                // Handle AND Generation number (??)
#define NVRAM_SEGMENT_HANDLE(_handle_)  ((_handle_) & 0x0000FFFF)

    uint32      timeStamp;
    uint32      cksum;                 // Checksum over the segment structure.

    uint32      spare[1];              // Pad to 64-bit (if needed ??)
} nvram_segment_t ;



/*
 * The block structure in the nvram
 * These are located in the NVRAM, just after all segments 
 * The max number of structures is defined by nvram_Hdr.blocks
 */

typedef struct nvram_block
{
    uint32             magic;
#define NVRAM_BLOCK_MAGIC        0xb10cbead

    uint32             next_block;
#define NVRAM_END_OF_BLOCK       0xffffeeee

    uint32             cksum;                 // Checksum over the block structure.

    char               data[0];
} nvram_block_t ;


typedef struct nvram_modification
{
    uint32             magic;
#define NVRAM_MODIFICATION_MAGIC        0xcad1bead

    int                state;
#define NVRAM_MODIFICATION_NONE       0
#define NVRAM_MODIFICATION_PROGRESS   1

    uint32             block;                 // The block which will te changed
    uint32             next_block;            // The new next_block indicator for changed block

    uint32             free_sgm_new_numBlks;  // the new values for the free segment list
    uint32             free_sgm_new_startBlk;
  
    uint32             segment;               // The segment which will be changed
    nvram_clientId_t   sgm_clientId;          // the new values for the related segment list
    nvram_cookie_t     sgm_clientCookie;
    uint32             sgm_startBlk;
    uint32             sgm_numBlks;
    uint32             sgm_Handle;
    uint32             sgm_state;

    uint32             cksum;                 // Checksum over the block structure.
    uint32             spare[0];
} nvram_modification_t;





#endif /* _NVRAM_H_ */



