/*
 * Name:  fs-opt-api.c
 *
 * RCS:  $Id: fs-opt-api.c,v 1.1 2002/09/15 00:53:44 nagarajs Exp $
 *
 * Description:  utility cluster api routines for fs administration.
 * 
 * Created By:  Nagaraj Shyam
 *
 * Date Created:  9/14/2002
 */
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <exports/evm-api.h>
#include <exports/ncm-api.h>
#include <exports/lnm-api.h>
#include <exports/anpssc-api.h>
#include <exports/fs-api.h>
#include <exports/fs-opt-api.h>

struct timeval  def_clus_timeout = {300,0};   /* 5 minutes max for command to execute over
                                                         cluster */

#define FS_SHELL_MEM_FREE(_p_)    free((_p_))

#define FS_SHELL_RESOURCE_ERROR      (NFX_ERR + 1)
#define FS_SHELL_SEND_ERROR          (NFX_ERR + 2)
#define FS_SHELL_SEND_TIMEOUT        (NFX_ERR + 3)

#define FS_SHELL_MEM_ALLOC(_p_, _type_, _size_)  {                              \
        (_p_) = (_type_*) malloc((_size_))                                     ;\
        if ((_p_) != NULL)                                                      \
        memset ((_p_), 0, (_size_))                                            ;\
        }

#define FS_SET_SEND_ERROR(err) \
        (((err) <= 0) ? (((err) < 0) ? FS_SHELL_SEND_ERROR : FS_SHELL_SEND_TIMEOUT) : \
                                                                                NFX_OK)

/*
 *   Convert signed 64 bit integer to string in hexadecimal form with
 *   16 nibbles.  It is the caller's responsibility to provide enough 
 *   memory in str.  This conversion is done in a format understood by 
 *   the conf file.
*/
int32
fs_opt_lltostr (char *str, int64 ll)
{

    int  pad;
    char tmpStr[256];

    memset (str, 0x30, 16);

    if (str != NULL) {   

        sprintf (tmpStr, "%llx", ll);
        pad = 16 - strlen (tmpStr);
        sprintf ((str + pad), tmpStr);
        return 0;

    }
    return -1;
}

/*
*  - Obtain the volume id from the agile.conf file.
*  - Copy into volId and string into volName if successful.
*  - return NFX_OK if able to find the volume.
*/
int32
fs_opt_get_clus_vol_id (char *volName, evm_volId_t *volId)
{

    char  **lVid = NULL;
    char  **lVidtmp;
    char  cvName[LNM_MAX_NODE_NAME];
    int32 rc = NFX_ERR;

    if (!node_is_PCC()) {

        fprintf (stderr, "Unable to obtain volume id for volume %s\n", volName);
        fprintf (stderr, "Please run command on the cluster PCC\n");
        return rc;

    }

    /*
     *    format of entry in the agile.conf file is:
     *              lvname.volName=volId 
     *    where volId is a hex string.
    */
    strcpy (cvName, "lvname.");
    strcat (cvName, volName);

    lVid = get_cfg_val (cvName);

    if (lVid != NULL) {

        for (lVidtmp = lVid; lVidtmp[0] != NULL; lVidtmp = lVidtmp + 2) {

            if (strcmp (lVidtmp[0], cvName) == 0) {

                /*
                 * we found a match, the volId string is in lVidtmp[1]
                 * convert to the format we want.  Assume, that conf files
                 * store volId in hex format.
                */
                *volId = (evm_volId_t) strtoll (lVidtmp[1], NULL, 16);

                rc = NFX_OK;
                break;

            }

        }

        free_cfg_val (lVid);

    } 

    return rc;

}

/*
 * This routine returns the node on which the logical volume is online.
 *     - Query the pcc to obtain a list of nodes in the cluster.
 *     - Query the evm on each node in the cluster to enquire if the lvol is
 *             online.
 *     - reutrn NFX_OK with the nodeName iff the lvol is online on one of the nodes
 *             in the cluster.
*/
int32
fs_opt_get_clus_lvol_online_node (evm_volId_t volId, char *nodeName)
{

    evm_MsgVolGetAttrib_t   *evmReq;
    evm_MsgVolCreate_t      *evmResp;
    ncmRqNodeList_t         ncmRqNL;
    ncmNodeList_t           *ncmList = NULL;
    ncmMsg_t                *ncmReq = NULL;
    ncmMsg_t                *ncmResp = NULL;
    uint32                  ncmReqSize, ncmRespSize;
    uint32                  i;
    uint32                  retLen;
    int32                   rc = NFX_ERR;

    ncmReqSize = sizeof (ncmMsg_type_t) + sizeof (int32) + 
                 MAX_NODE_PER_CLUSTER * LNM_MAX_NODE_NAME;
    
    FS_SHELL_MEM_ALLOC(ncmList, ncmNodeList_t, ncmReqSize);

    ncmRqNL.msgType = NCM_RQ_NODE_LIST;

    memset (ncmList, 0, ncmReqSize);


    retLen = sendAgileMsg (-1, &ncmRqNL, sizeof (ncmRqNL), "ncm", 
                       NFX_SSC_UNIX, SSC_SLOT, MGMTPLANE, &def_clus_timeout,  
                       ncmList, ncmReqSize);


    if (retLen >= sizeof (ncmRqNodeList_t) && (ncmList->msgType == NCM_NODE_LIST) &&
                                           ncmList->numConnNodes >= 1) {

        /*
         * We need to send a message to the evm on each node in the cluster in
         * turn, in order to find where the lvol is actually online.
        */

        ncmReqSize = sizeof (ncmMsg_t) + sizeof (evm_MsgVolGetAttrib_t);
        FS_SHELL_MEM_ALLOC(ncmReq, ncmMsg_t, ncmReqSize);

        /*
         * Initialize the request message.
        */
        ncmReq->msgType = NCM_SEND;
        ncmReq->rqResp = 1;
        strcpy (ncmReq->appName, "evm");

        evmReq = (evm_MsgVolGetAttrib_t *)ncmReq->appMsg; 
        evmReq->hdr.msgType = EVM_VOL_GET_ATTRIB; 
        evmReq->id = volId;

        /*
         * and the response message.
        */

        ncmRespSize = sizeof (ncmMsg_t) + sizeof (evm_MsgVolCreate_t) - sizeof (uint64);
        ncmResp = (ncmMsg_t *) malloc (ncmRespSize);

        FS_SHELL_MEM_ALLOC(ncmResp, ncmMsg_t, ncmRespSize);

        for (i = 0; i < ncmList->numConnNodes; i++) {
 
            strcpy (ncmReq->nodeName, ncmList->nodeNames[i]);

            retLen = sendAgileMsg (-1, ncmReq, ncmReqSize, "ncm", NFX_SSC_UNIX, SSC_SLOT,
                               MGMTPLANE, &def_clus_timeout, ncmResp, ncmRespSize);

            if (retLen == ncmRespSize && ncmResp->msgType == NCM_RESP) {

                evmResp = (evm_MsgVolCreate_t *)ncmResp->appMsg;
                if (evmResp->hdr.msgType == EVM_VOL_CONFIG && evmResp->config.onLine) {

                    if (nodeName != NULL) {

                        strcpy (nodeName, ncmReq->nodeName);

                    }
                    rc = NFX_OK;
                    break;

                }

            } 

        }

    } else {

        rc = FS_SET_SEND_ERROR(rc);

    }
  
    if (ncmList != NULL) {

        FS_SHELL_MEM_FREE(ncmList);

    }

    if (ncmReq != NULL) {

        FS_SHELL_MEM_FREE(ncmReq);

    }

    if (ncmResp != NULL) {

        FS_SHELL_MEM_FREE(ncmResp);

    }

    return rc;

}

/*
 * This routine returns the number of blocks in the logical volume.
 *     - Query the pcc to obtain a list of nodes in the cluster.
 *     - Query the evm on each node in the cluster to obtain the lvol attribute. 
 *     - reutrn NFX_OK with the numblocks iff the lvol is online on one of the nodes
 *             in the cluster.
*/
int32
fs_opt_get_clus_vol_mkfs_numblks (evm_volId_t volId, uint64 *numBlks)
{

    evm_MsgVolGetAttrib_t   *evmReq;
    evm_MsgVolCreate_t      *evmResp;
    ncmRqNodeList_t         ncmRqNL;
    ncmNodeList_t           *ncmList = NULL;
    ncmMsg_t                *ncmReq = NULL;
    ncmMsg_t                *ncmResp = NULL;
    uint32                  ncmReqSize, ncmRespSize;
    uint32                  i;
    uint32                  retLen;
    int32                   rc = NFX_ERR;

    ncmReqSize = sizeof (ncmMsg_type_t) + sizeof (int32) + 
                 MAX_NODE_PER_CLUSTER * LNM_MAX_NODE_NAME;
    
    FS_SHELL_MEM_ALLOC(ncmList, ncmNodeList_t, ncmReqSize);

    ncmRqNL.msgType = NCM_RQ_NODE_LIST;

    retLen = sendAgileMsg (-1, &ncmRqNL, sizeof (ncmRqNL), "ncm", 
                       NFX_SSC_UNIX, SSC_SLOT, MGMTPLANE, &def_clus_timeout,  
                       ncmList, ncmReqSize);

    if (retLen >= sizeof (ncmRqNodeList_t) && (ncmList->msgType == NCM_NODE_LIST) &&
                                           ncmList->numConnNodes >= 1) {

        /*
         * We need to send a message to the evm on each node in the cluster in
         * turn, in order to find where the lvol is actually online.
        */

        ncmReqSize = sizeof (ncmMsg_t) + sizeof (evm_MsgVolGetAttrib_t);
        FS_SHELL_MEM_ALLOC(ncmReq, ncmMsg_t, ncmReqSize);

        /*
         * Initialize the request message.
        */
        ncmReq->msgType = NCM_SEND;
        ncmReq->rqResp = 1;
        strcpy (ncmReq->appName, "evm");

        evmReq = (evm_MsgVolGetAttrib_t *)ncmReq->appMsg; 
        evmReq->hdr.msgType = EVM_VOL_GET_ATTRIB; 
        evmReq->id = volId;

        /*
         * and the response message.
        */

        ncmRespSize = sizeof (ncmMsg_t) + sizeof (evm_MsgVolCreate_t) - sizeof (uint64);
        FS_SHELL_MEM_ALLOC(ncmResp, ncmMsg_t, ncmRespSize);

        for (i = 0; i < ncmList->numConnNodes; i++) {
 
            strcpy (ncmReq->nodeName, ncmList->nodeNames[i]);

            retLen = sendAgileMsg (-1, ncmReq, ncmReqSize, "ncm", NFX_SSC_UNIX, SSC_SLOT,
                               MGMTPLANE, &def_clus_timeout, ncmResp, ncmRespSize);

            if (retLen == ncmRespSize && ncmResp->msgType == NCM_RESP) {

                evmResp = (evm_MsgVolCreate_t *)ncmResp->appMsg;

                if (evmResp->hdr.msgType == EVM_VOL_CONFIG) {

                    *numBlks = evmResp->config.currSize;
                    *numBlks = ((*numBlks) * EVM_LBLK_SIZE) / EFS_BLOCK_SIZE;
                    rc = NFX_OK;
                    break;

                }

            } 

        }

    } else {

        rc = FS_SET_SEND_ERROR(rc);

    }
  
    if (ncmList != NULL) {

        FS_SHELL_MEM_FREE(ncmList);

    }

    if (ncmReq != NULL) {

        FS_SHELL_MEM_FREE(ncmReq);

    }

    if (ncmResp != NULL) {

        FS_SHELL_MEM_FREE(ncmResp);

    }

    return rc;

}

/*
 * Given the volume id, figure out the node on which the volume is online.  
 * If able to determine that volume is online on any one node in the cluster
 * return NFX_OK, else return appropriate error code.  
 * It is the caller's responsibility to allocate enough memory for nodeName 
 * (LNM_MAX_NODE_NAME bytes). 
*/
int32
fs_opt_get_clus_vol_online_node (evm_volId_t volId, char *nodeName)
{

    char   **cNode = NULL;
    char   **cNodetmp = NULL;
    char   cvName[LNM_MAX_NODE_NAME];
    char   volIdStr[LNM_MAX_NODE_NAME];
    int32  rc = NFX_ERR;

    if (!node_is_PCC()) {

        fprintf (stderr, "Unable to obtain online node for volume, id = %lld\n", 
                                                             volId.whole_thing);
        fprintf (stderr, "Please run command on the cluster PCC\n");
        return rc;

    }

    /*
     * The entries in the config files are in the format:
     *      cluster.lv.volId = online:nodeName0
     *      cluster.lv.volId = offline:nodeName1
     *  etc. 
    */

    strcpy (cvName, "cluster.lv.");

    if (fs_opt_lltostr (volIdStr, volId.whole_thing) == -1) {

        fprintf (stderr, "internal error, unable to convert vol id to string form\n");    

    }

    strcat (cvName, volIdStr);

    cNode = get_cfg_val (cvName);

    if (cNode != NULL) {

        for (cNodetmp = cNode; cNodetmp[0] != NULL; cNodetmp = cNodetmp + 2) {

            if (strcmp (cNodetmp[0], cvName) == 0) {

                if (strncmp (cNodetmp[1], "online:", strlen("online:")) == 0) {

                    if (nodeName != NULL) {

                        strcpy (nodeName, cNodetmp[1] + strlen("online:"));

                    }
                    rc = NFX_OK;
                    break;

                }
            }

        }

        free_cfg_val (cNode);

    } 
    return rc;

}

