/*
 *-----------------------------------------------------------------
 *
 * Name:	pci-api.c
 *
 * RCS:  $Id: pci-api.c,v 1.6 2001/12/02 02:39:23 rick Exp $
 *
 * Copyright (C) 2001, Agile Storage, Inc.
 *
 * Description:	API functions for PCI operations
 *
 * Created by:	Rick Lund
 *
 * Date Created:	6/22/01
 *
 *-----------------------------------------------------------------
 */

#include <nfx-incl.h>
#include <exports/pci-api.h>
#include "pci.h"

extern pci_api_func_t	pci_func;

/*-----------------------------------------------------------------
 * Name:	pci_cfg_read()
 *
 * Description:	Generate a PCI read configuration cycle
 *
 * Created by:	Rick Lund
 *
 * Date Created:	6/22/01
 *
 *-----------------------------------------------------------------
 */
int32						/* NFX_OK if data was read successfully
							 * PCI_ERR_BAD_PARAM
							 * PCI_ERR_MASTER_ABORT
							 * PCI_ERR_TARGET_ABORT
							 * PCI_ERR_PARITY_ERROR
							 */
pci_cfg_read(
	uint32	bus,
	uint32	dev,
	uint32	func,
	uint32	reg,
	uint32	* data
)
{

/*
	printf("[pci_cfg_read], bus %d, dev %d, func %d, reg 0x%x\n",
														bus, dev, func, reg);
*/

	return ((*pci_func.cfg_read)(bus, dev, func, reg, data));

}	/* pci_cfg_read() */

/*-----------------------------------------------------------------
 * Name:	pci_cfg_write()
 *
 * Description:	Generate a PCI write configuration cycle
 *
 * Created by:	Rick Lund
 *
 * Date Created:	6/22/01
 *
 *-----------------------------------------------------------------
 */
int32						/* NFX_OK if data was written successfully
							 * PCI_ERR_BAD_PARAM
							 * PCI_ERR_MASTER_ABORT
							 * PCI_ERR_TARGET_ABORT
							 * PCI_ERR_PARITY_ERROR
							 */
pci_cfg_write(
	uint32	bus,
	uint32	dev,
	uint32	func,
	uint32	reg,
	uint32	data
)
{

/*
	printf("[pci_cfg_write], bus %d, dev %d, func %d, reg 0x%x, data 0x%x\n",
													bus, dev, func, reg, data);
*/

	return ((*pci_func.cfg_write)(bus, dev, func, reg, data));

}	/* pci_cfg_write() */

/*-----------------------------------------------------------------
 * Name:	pci_cfg_check_status()
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	6/22/01
 *
 *-----------------------------------------------------------------
 */
int32							/* NFX_OK
								 * PCI_ERR_MASTER_ABORT
								 * PCI_ERR_TARGET_ABORT
								 * PCI_ERR_PARITY_ERROR
								 */
pci_cfg_check_status(
	uint32	bus,
	uint32	dev
)
{

/*
	return (pci_gt_cfg_check_status(bus, dev));
*/

	return ((*pci_func.check_status)(bus, dev));

}	/* pci_cfg_check_status() */

/*-----------------------------------------------------------------
 * Name:	pci_cfg_clear_status()
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	6/22/01
 *
 *-----------------------------------------------------------------
 */
void
pci_cfg_clear_status(
	uint32	bus,
	uint32	dev
)
{

/*
	return (pci_gt_cfg_clear_status(bus, dev));
*/

	(*pci_func.clear_status)(bus, dev);

}	/* pci_cfg_clear_status() */

