/*
 *-----------------------------------------------------------------
 *
 * Name:	pci-api.h
 *
 * RCS:  $Id: pci-api.h,v 1.4 2001/12/02 02:39:23 rick Exp $
 *
 * Copyright (C) 2001, Agile Storage, Inc.
 *
 * Description:	API defitions for PCI operations
 *
 * Created by:	Rick Lund
 *
 * Date Created:	6/22/01
 *
 *-----------------------------------------------------------------
 */

#ifndef _PCI_API_H
#define _PCI_API_H

#include <exports/pci-error.h>

typedef struct pci_api_func {
	int32	(*cfg_read)(uint32, uint32, uint32, uint32, uint32 *);
	int32	(*cfg_write)(uint32, uint32, uint32, uint32, uint32);
	int32	(*check_status)(uint32, uint32);
	void	(*clear_status)(uint32, uint32);
} pci_api_func_t;

/*-----------------------------------------------------------------
 * Name:	pci_cfg_read()
 *
 * Description:	Generate a PCI read configuration cycle
 *
 * Created by:	Rick Lund
 *
 * Date Created:	6/22/01
 *
 *-----------------------------------------------------------------
 */
int32						/* NFX_OK if data was read successfully
							 * PCI_ERR_BAD_PARAM
							 * PCI_ERR_MASTER_ABORT
							 */
pci_cfg_read(
	uint32	bus,
	uint32	dev,
	uint32	func,
	uint32	reg,
	uint32	* data
);

/*-----------------------------------------------------------------
 * Name:	pci_cfg_write()
 *
 * Description:	Generate a PCI write configuration cycle
 *
 * Created by:	Rick Lund
 *
 * Date Created:	6/22/01
 *
 *-----------------------------------------------------------------
 */
int32						/* NFX_OK if data was written successfully
							 * PCI_ERR_BAD_PARAM
							 * PCI_ERR_MASTER_ABORT
							 */
pci_cfg_write(
	uint32	bus,
	uint32	dev,
	uint32	func,
	uint32	reg,
	uint32	data
);

/*-----------------------------------------------------------------
 * Name:	pci_cfg_check_status()
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	6/22/01
 *
 *-----------------------------------------------------------------
 */
int32							/* NFX_OK
								 * PCI_ERR_MASTER_ABORT
								 * PCI_ERR_TARGET_ABORT
								 * PCI_ERR_PARITY_ERROR
								 */
pci_cfg_check_status(
	uint32	bus,
	uint32	dev
);

/*-----------------------------------------------------------------
 * Name:	pci_cfg_clear_status()
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	6/22/01
 *
 *-----------------------------------------------------------------
 */
void
pci_cfg_clear_status(
	uint32	bus,
	uint32	dev
);

#endif	/* _PCI_API_H */

