/*
 *-----------------------------------------------------------------
 *
 * Name:	pci-nfp.c
 *
 * RCS:  $Id: pci-nfp.c,v 1.13 2002/12/05 23:47:36 rick Exp $
 *
 * Copyright (C) 2001, Agile Storage, Inc.
 *
 * Description:	PCI/LDT initialization for NFP board
 *				(mostly adapted from broadcom's CFE code)
 *
 * Created by:	Rick Lund
 *
 * Date Created:	1/03/02
 *
 *-----------------------------------------------------------------
 */
#include <nfx-incl.h>
#include <exports/pci-api.h>
#include "pci.h"

#ifdef SIBYTE

uint32	pci_host_mode;
uint32	pci_device_mode;
uint32	ldt_slave_mode;
uint32	ldt_master_mode;

uint64	ht_bytes_tx = 0;
uint64	ht_bytes_rx = 0;

#ifdef PROM_SIBYTE_BC	
#if defined (FEATURE_AUTOLOAD_RUNTIME)
uint32 autoload_txrxfp_runtime=1; //Always do this by default
#else
uint32 autoload_txrxfp_runtime=0; 
#endif
#endif

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	1/03/02
 *
 *-----------------------------------------------------------------
 */
void
phb_init()
{
	uint32	cmdstat;
#ifdef PROM_SIBYTE_BC	
	uint32	t;
        unsigned long long memorysize;
	unsigned long long	memsize;
	extern uint64 get_memorysize();

    /* On boot up open the 83000000 window for SSC to copy the 
     * runtime images over.
     * Once that is done, this routine is called again, with the
     * autoload_txrxfp_runtime flag set to 0.
     */ 	
    if(autoload_txrxfp_runtime) {
	pci_cfg_write(0, 0, 0, 0x44, 0x00030001);
	pci_cfg_write(0, 0, 0, 0x48, 0x00031001);
	pci_cfg_write(0, 0, 0, 0x4c, 0x00032001);
	pci_cfg_write(0, 0, 0, 0x50, 0x00033001);
	pci_cfg_write(0, 0, 0, 0x54, 0x00034001);
	pci_cfg_write(0, 0, 0, 0x58, 0x00035001);
	pci_cfg_write(0, 0, 0, 0x5c, 0x00036001);
	pci_cfg_write(0, 0, 0, 0x60, 0x00037001);
	pci_cfg_write(0, 0, 0, 0x64, 0x00038001);
	pci_cfg_write(0, 0, 0, 0x68, 0x00039001);
	pci_cfg_write(0, 0, 0, 0x6c, 0x0003a001);
	pci_cfg_write(0, 0, 0, 0x70, 0x0003b001);
	pci_cfg_write(0, 0, 0, 0x74, 0x0003c001);
	pci_cfg_write(0, 0, 0, 0x78, 0x0003d001);
	pci_cfg_write(0, 0, 0, 0x7c, 0x0003e001);
	pci_cfg_write(0, 0, 0, 0x80, 0x0003f001);
    }
    else {
#if defined (FEATURE_AUTOLOAD_RUNTIME)
	/* Open up the txrx/fp memory to the SSC pci for
	 * normal operation.
	 */     
	printf("Reinit'ing phb_init.\n");    

	// write 1 to register 0x94 to enable writes.
    	pci_cfg_write(0, 0, 0, 0x94, 0x01); 
#endif	
	memorysize = get_memorysize();
#define GIG	(1024 * 1024 * 1024)
        /* Please ensure that if this is changed, you also have to modify the
         * prom-pmon/more-commands.c/do_fpBc_copy and do_txrxBc_copy - the two functions
         * which copy the runtime code from the shared memory to the 83000000 location.
         */	
        memsize = memorysize/GIG;
        if(memsize == 2) {
       	    pci_cfg_write(0, 0, 0, 0x44, 0x00c00001);
    	    pci_cfg_write(0, 0, 0, 0x48, 0x00c01001);
	    pci_cfg_write(0, 0, 0, 0x4c, 0x00c02001);
	    pci_cfg_write(0, 0, 0, 0x50, 0x00c03001);
	    pci_cfg_write(0, 0, 0, 0x54, 0x00c04001);
	    pci_cfg_write(0, 0, 0, 0x58, 0x00c05001);
	    pci_cfg_write(0, 0, 0, 0x5c, 0x00c06001);
	    pci_cfg_write(0, 0, 0, 0x60, 0x00c07001);
    	    pci_cfg_write(0, 0, 0, 0x64, 0x00c08001);
	    pci_cfg_write(0, 0, 0, 0x68, 0x00c09001);
	    pci_cfg_write(0, 0, 0, 0x6c, 0x00c0a001);
	    pci_cfg_write(0, 0, 0, 0x70, 0x00c0b001);
	    pci_cfg_write(0, 0, 0, 0x74, 0x00c0c001);
	    pci_cfg_write(0, 0, 0, 0x78, 0x00c0d001);
	    pci_cfg_write(0, 0, 0, 0x7c, 0x00c0e001);
	    pci_cfg_write(0, 0, 0, 0x80, 0x00c0f001);
        }	
        else {
	    pci_cfg_write(0, 0, 0, 0x44, 0x00c80001);
	    pci_cfg_write(0, 0, 0, 0x48, 0x00c81001);
	    pci_cfg_write(0, 0, 0, 0x4c, 0x00c82001);
	    pci_cfg_write(0, 0, 0, 0x50, 0x00c83001);
	    pci_cfg_write(0, 0, 0, 0x54, 0x00c84001);
	    pci_cfg_write(0, 0, 0, 0x58, 0x00c85001);
	    pci_cfg_write(0, 0, 0, 0x5c, 0x00c86001);
	    pci_cfg_write(0, 0, 0, 0x60, 0x00c87001);
	    pci_cfg_write(0, 0, 0, 0x64, 0x00c88001);
	    pci_cfg_write(0, 0, 0, 0x68, 0x00c89001);
	    pci_cfg_write(0, 0, 0, 0x6c, 0x00c8a001);
	    pci_cfg_write(0, 0, 0, 0x70, 0x00c8b001);
	    pci_cfg_write(0, 0, 0, 0x74, 0x00c8c001);
	    pci_cfg_write(0, 0, 0, 0x78, 0x00c8d001);
	    pci_cfg_write(0, 0, 0, 0x7c, 0x00c8e001);
	    pci_cfg_write(0, 0, 0, 0x80, 0x00c8f001);
        }	
    }	
	pci_cfg_write(0, 0, 0, 0x94, 0);
	pci_cfg_read(0, 0, 0, 0x94, &t);

	pci_cfg_write(0, 0, 0, 0x40, 0x7db08080);
	pci_cfg_write(0, 0, 0, 0x4, 0xf9000152); /* disable the master bit */
#else
	pci_cfg_write(0, 0, 0, 0x44, 0);
	pci_cfg_write(0, 0, 0, 0x48, 0);
	pci_cfg_write(0, 0, 0, 0x4c, 0);
	pci_cfg_write(0, 0, 0, 0x50, 0);
	pci_cfg_write(0, 0, 0, 0x54, 0);
	pci_cfg_write(0, 0, 0, 0x58, 0);
	pci_cfg_write(0, 0, 0, 0x5c, 0);
	pci_cfg_write(0, 0, 0, 0x60, 0);
	pci_cfg_write(0, 0, 0, 0x64, 0);
	pci_cfg_write(0, 0, 0, 0x68, 0);
	pci_cfg_write(0, 0, 0, 0x6c, 0);
	pci_cfg_write(0, 0, 0, 0x70, 0);
	pci_cfg_write(0, 0, 0, 0x74, 0);
	pci_cfg_write(0, 0, 0, 0x78, 0);
	pci_cfg_write(0, 0, 0, 0x7c, 0);
	pci_cfg_write(0, 0, 0, 0x80, 0);

	pci_cfg_write(0, 0, 0, 0x40, 0x7db38080);
	pci_cfg_write(0, 0, 0, 0x4, 0xf9000156);
#endif	

	pci_cfg_read(0, 0, 0, 0x4, &cmdstat);
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	1/03/02
 *
 *-----------------------------------------------------------------
 */
void
lhb_link_reset()
{
	uint32	t;

	pci_cfg_read(0, 1, 0, 0x40, &t);
	t |= 0x00010000;		/* WARM RESET */
	pci_cfg_write(0, 1, 0, 0x40, t);

	delay_us(1000000);

	t &= ~0x00010000;		/* WARM RESET */
	pci_cfg_write(0, 1, 0, 0x40, t);
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	1/03/02
 *
 *-----------------------------------------------------------------
 */
int
lhb_link_ready(
	uint32	* ctrl_ret
)
{
	int	maxpoll = 0x1000000;
	int	ctrl = 0;
	int	count = 0;
	int	linkerr = 0;

	while ((ctrl & (0x10 | 0x20)) == 0 && count < maxpoll) {
		pci_cfg_read(0, 1, 0, 0x44, &ctrl);
		count++;
		if ((ctrl & 0x190) != 0 && !linkerr) {
			printf("LDT Err 0x%x\n", ctrl);
			linkerr = 1;
		}
	}

	*ctrl_ret = ctrl;

	//printf("lhb link ready: count %d\n", count);
	return linkerr;
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	1/03/02
 *
 *-----------------------------------------------------------------
 */
void
lhb_null_config()
{
	uint32	t;

	pci_cfg_write(0, 1, 0, 0x18, 0);
	pci_cfg_read(0, 1, 0, 0x1c, &t);
	t &= ~(0x000000ff | 0x0000ff00);
	t |= (1 << 4) | (0 << (8+4));
	pci_cfg_write(0, 1, 0, 0x1c, t);
	pci_cfg_write(0, 1, 0, 0x30, 0x0000f200);
	pci_cfg_read(0, 1, 0, 0x20, &t);
	t &= ~(0x0000ffff | 0xffff0000);
	t |= (1 << 4) | (0 << (16+4));
	pci_cfg_write(0, 1, 0, 0x20, t);
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description: ldt init
 *
 * Created by:	Rick Lund
 *
 * Date Created:	1/03/02
 *
 *-----------------------------------------------------------------
 */
int32
lhb_init()
{
	uint32	t, i;
	uint32	sri_cmd, freq;
	int32	linkerr = 0;
	int32	retry;
	uint32	ctrl;

	pci_cfg_read(0, 1, 0, 0x68, &t);
	pci_cfg_write(0, 1, 0, 0x68, t | 0xff000000);

	pci_cfg_read(0, 1, 0, 0x50, &t);
	if (t & 0x00010000) {		/* SIPREADY */
		printf("SipReady already set\n");
		lhb_link_reset();
		goto finish;
	}

	sri_cmd = 0x52281010;
	pci_cfg_write(0, 1, 0, 0x50, sri_cmd);

	pci_cfg_write(0, 1, 0, 0x54, 0x0000ffff);
	pci_cfg_write(0, 1, 0, 0x58, 0x0000ffff);
	pci_cfg_read(0, 1, 0, 0x58, &t);

	for (i=0; i<10; i++)
		pci_cfg_read(0, 1, 0, 0, &t);

	pci_cfg_read(0, 1, 0, 0x48, &freq);
	freq &= 0xFFFF00FF;
	if (get_busclock() >= 600)
		freq |= 0x00000400;		// 300 = 500 Mhz , 400 = 600 Mhz, 500 = 800 Mhz
	else
		freq |= 0x00000200;		// 300 = 500 Mhz , 400 = 600 Mhz, 500 = 800 Mhz
	pci_cfg_write(0, 1, 0, 0x48, freq);

	pci_cfg_write(0, 1, 0, 0x68, 0x00001209);
	pci_cfg_write(0, 1, 0, 0x6c, 0x00042525);
	pci_cfg_write(0, 1, 0, 0xc8, 0x00ffffff);
	pci_cfg_read(0, 1, 0, 0xc8, &t);

	sri_cmd |= 0x00010000;		/* SIPREADY */
	pci_cfg_write(0, 1, 0, 0x50, sri_cmd);
	pci_cfg_read(0, 1, 0, 0x50, &t);

	retry = 0;
	for (;;) {
		linkerr = lhb_link_ready(&ctrl);
//printf("ctrl = 0x%x\n", ctrl);
		pci_cfg_read(0, 1, 0, 0x68, &t);
		if ((t & 0xff000000) != 0) {
			linkerr = 1;
			printf("lhb_init, ErrStat = 0x%x\n", t);
			pci_cfg_write(0, 1, 0, 0x68, t | 0xff000000);
		}
		pci_cfg_read(0, 1, 0, 0x44, &t);
		if ((t & 0x190) != 0) {
			linkerr = 1;
			printf("lhb_init, CrcErr = 0x%x\n", t);
			t |= 0x0f00;
			pci_cfg_write(0, 1, 0, 0x44, t);
		}
		if ((ctrl & 0x20) == 0) {
			printf("LDT not init done\n");
if (isTXRX()) {
	*(volatile uint32 *)(int)0xbf4000fc = 1;
}
			retry++;
			lhb_link_reset();
			continue;
		}

		if (!linkerr || retry > 0)
			break;

		retry++;
		lhb_link_reset();
	}

finish:
	pci_cfg_read(0, 1, 0, 0x44, &t);

	if ((t & 0x20) == 0) {
		uint32	cmd, ctrl;
		pci_cfg_read(0, 1, 0, 0x40, &cmd);
		pci_cfg_read(0, 1, 0, 0x44, &ctrl);
		printf("lhb_init: LDT init not done, cmd = 0x%x, ctrl = 0x%x\n",
																	cmd, ctrl);
		return 1;
	}
	else if ((t & 0x10) != 0) {
		printf("lhb_init: LDT link fail set, t = 0x%x\n", t);
		return 1;
	}

	pci_cfg_read(0, 1, 0, 0x68, &t);
	pci_cfg_write(0, 1, 0, 0x68, t & 0xff000000);

	if (ldt_slave_mode) {
		lhb_null_config();
		pci_cfg_read(0, 1, 0, 4, &t);
		pci_cfg_write(0, 1, 0, 4, t | 0x4);		/* MASTER ENABLE */
	}

	return 0;
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	12/27/01
 *
 *-----------------------------------------------------------------
 */
void
pci_nfp_init()
{
	uint32	cmdstat;
	uint32	t;

#ifdef SB1250_SIMULATOR
	return;
#endif

	pci_host_mode = isTXRX();
	pci_device_mode = isFP();
	ldt_slave_mode = isFP();
	ldt_master_mode = isTXRX();

	//printf("PCIH\n");
	pci_cfg_read(0, 0, 0, 4, &cmdstat);
	cmdstat &= 0xFFFF0000;
	pci_cfg_write(0, 0, 0, 4, cmdstat);
	pci_cfg_read(0, 1, 0, 4, &cmdstat);
	cmdstat &= 0xFFFF0000;
	pci_cfg_write(0, 1, 0, 4, cmdstat);

	phb_init();

//WTG need to set up the Hyper Transport Bridge Controll reg
//   for both Master and Slave to forward Bus Errors
	pci_cfg_write(0, 1, 0, 0x3c, 0x220000);
	pci_cfg_read(0, 1, 0, 0x3c, &t);

	if (lhb_init()) // ldt init OK 
		return;

#ifdef PROM_SIBYTE
	watchdog_kick();
#endif	
	delay_us(1000000);

	if (pci_device_mode) {
		pci_cfg_write(0, 0, 0, 0x94, 0);
		pci_cfg_read(0, 0, 0, 0x94, &t);
		return;
	}

	//printf("PCIB\n");
	pci_cfg_write(0, 1, 0, 0x18, 0x00ff0100);
#ifndef PROM_SIBYTE_BC	
	pci_cfg_write(1, 0, 0, 0x40, 0x20030008);
#endif	
	pci_cfg_write(0, 1, 0, 0x40, 0x20030008);
	pci_cfg_write(0, 1, 0, 0x18, 0x00010100);

	//printf("PCIS\n");
	pci_cfg_write(0, 0, 0, 0x3c, 0x138);

	pci_cfg_write(0, 1, 0, 0x3c, 0xff);
	pci_cfg_read(0, 1, 0, 4, &cmdstat);
	cmdstat &= 0xffff0000;
	pci_cfg_write(0, 1, 0, 4, cmdstat);
	pci_cfg_read(0, 1, 0, 4, &cmdstat);
	pci_cfg_write(0, 1, 0, 0x10, 0);
	pci_cfg_write(0, 1, 0, 0x14, 0);
	pci_cfg_write(0, 1, 0, 0x38, 0);

//WTG need to set up the Hyper Transport Bridge Controll reg
//   for both Master and Slave to forward Bus Errors
	pci_cfg_write(0, 1, 0, 0x3c, 0x220000);
	pci_cfg_read(0, 1, 0, 0x3c, &t);

	cmdstat |= 0x13;
	pci_cfg_write(0, 1, 0, 4, cmdstat);
	pci_cfg_read(0, 1, 0, 4, &cmdstat);

#ifndef PROM_SIBYTE_BC
	pci_cfg_write(0, 2, 0, 0x3c, 0x1ff);
	pci_cfg_read(0, 2, 0, 4, &cmdstat);
	cmdstat &= 0xffff0000;
	pci_cfg_write(0, 2, 0, 4, cmdstat);
	pci_cfg_read(0, 2, 0, 4, &cmdstat);
	pci_cfg_write(0, 2, 0, 0x10, 0x60000008);
	pci_cfg_write(0, 2, 0, 0x14, 0);
	pci_cfg_write(0, 2, 0, 0x18, 0x70000008);
	pci_cfg_write(0, 2, 0, 0x1c, 0x71000008);
	pci_cfg_write(0, 2, 0, 0x20, 0);
	pci_cfg_write(0, 2, 0, 0x24, 0);
	pci_cfg_write(0, 2, 0, 0x30, 0x73000000);
	cmdstat |= 0x12;
	pci_cfg_write(0, 2, 0, 4, cmdstat);
	pci_cfg_read(0, 2, 0, 4, &cmdstat);

	pci_cfg_write(1, 0, 0, 0x3c, 0xff);
	pci_cfg_read(1, 0, 0, 4, &cmdstat);
	cmdstat &= 0xffff0000;
	pci_cfg_write(1, 0, 0, 4, cmdstat | 4);
	pci_cfg_read(1, 0, 0, 4, &cmdstat);
	pci_cfg_write(1, 0, 0, 0x10, 0);
	pci_cfg_write(1, 0, 0, 0x14, 0);
	pci_cfg_write(1, 0, 0, 0x38, 0);
	pci_cfg_write(1, 0, 0, 4, cmdstat | 0x17);
	pci_cfg_read(1, 0, 0, 4, &cmdstat);
	
	//printf("PCIW\n");
	pci_cfg_write(0, 2, 0, 0x10, 0x41000008);
	pci_cfg_write(0, 2, 0, 0x1c, 0x42021008);
	pci_cfg_write(0, 2, 0, 0x18, 0x42022008);
	pci_cfg_write(0, 2, 0, 0x30, 0x42000000);
	pci_cfg_read(0, 2, 0, 0x30, &t);
#endif	
	pci_cfg_write(0, 1, 0, 0x30, 0);
	pci_cfg_write(0, 1, 0, 0x1c, 0x2000a090);
	pci_cfg_write(0, 1, 0, 0x20, 0x42204210);
	pci_cfg_write(0, 1, 0, 0x24, 0x10);

// WTG  moved up to top so both SiBytes get the right Bridge controll bits set
//	pci_cfg_write(0, 1, 0, 0x3c, 0x220000);
//	pci_cfg_read(0, 1, 0, 0x3c, &t);

	pci_cfg_write(0, 1, 0, 4, 0x100007);
	pci_cfg_read(0, 1, 0, 4, &t);

#ifdef PROM_SIBYTE_BC
	pci_cfg_write(0, 0, 0, 4, 0x2a00152);
#else	
	pci_cfg_write(0, 0, 0, 4, 0x2a00157);
#endif	
	pci_cfg_write(0, 0, 0, 0xc, 0xff08);
	pci_cfg_read(0, 0, 0, 4, &t);
	pci_cfg_write(0, 1, 0, 4, 0x100147);
	pci_cfg_write(0, 1, 0, 0xc, 0x1ff08);
	pci_cfg_read(0, 1, 0, 4, &t);
#ifndef PROM_SIBYTE_BC
	pci_cfg_write(0, 2, 0, 4, 0x2a00157);
	pci_cfg_write(0, 2, 0, 0xc, 0xff08);
	pci_cfg_read(0, 2, 0, 4, &t);
	pci_cfg_write(1, 0, 0, 4, 0x100147);
	pci_cfg_write(1, 0, 0, 0xc, 0x1ff08);
	pci_cfg_read(1, 0, 0, 4, &t);
#endif	
	
}	/* pci_nfp_init() */

#endif	/* SIBYTE */
