/*
 *-----------------------------------------------------------------
 *
 * Name:	pci.h
 *
 * RCS:  $Id: pci.h,v 1.6 2002/01/02 22:32:36 rick Exp $
 *
 * Copyright (C) 2001, Agile Storage, Inc.
 *
 * Description:	Internal PCI definitions
 *
 * Created by:	Rick Lund
 *
 * Date Created:	6/22/01
 *
 *-----------------------------------------------------------------
 */

#ifndef _PCI_H
#define _PCI_H

#define PCI_NUM_BUSSES	16
#define PCI_NUM_DEVS	32
#define PCI_NUM_FUNCS	8
#define PCI_NUM_REGS	256

#define PCI_CFG_REG_SHIFT	2
#define PCI_CFG_FUNC_SHIFT	8
#define PCI_CFG_DEV_SHIFT	11
#define PCI_CFG_BUS_SHIFT	16
#define PCI_CFG_ENABLE		0x80000000

#define PCI_CFG_STAT_PERR		0x80000000
#define PCI_CFG_STAT_SERR		0x40000000
#define PCI_CFG_STAT_MABORT		0x20000000
#define PCI_CFG_STAT_TABORT		0x10000000
#define PCI_CFG_STAT_SIG_TABORT	0x08000000
#define PCI_CFG_STAT_FB2B		0x00800000
#define PCI_CFG_CMD_SERR_ENA	0x00000100
#define PCI_CFG_CMD_PERR_ENA	0x00000040
#define PCI_CFG_CMD_MASTER_ENA	0x00000004
#define PCI_CFG_CMD_MEM_ENA		0x00000002
#define PCI_CFG_CMD_IO_ENA		0x00000001

#if #endian(little)
#define WORD_SWAP(x)	(x)
#else
#define WORD_SWAP(x)	(((x) & 0xFF) << 24) +		\
						(((x) & 0xFF00) << 8) +		\
						(((x) & 0xFF0000) >> 8) +	\
						(((x) & 0xFF000000) >> 24)
#endif

/*
 * Standard register definitions for first 16 bytes of config space
 */
#define PCI_CFG_ID			0x0
#define PCI_CFG_CMDSTAT		0x4
#define PCI_CFG_CLASS		0x8
#define PCI_CFG_HDR_LAT		0xC

extern int32	pci_gt_cfg_read(uint32, uint32, uint32, uint32, uint32 *);
extern int32	pci_gt_cfg_write(uint32, uint32, uint32, uint32, uint32);
extern int32	pci_gt_cfg_check_status(uint32, uint32);
extern void		pci_gt_cfg_clear_status(uint32, uint32);
extern int32	pci_sb1250_cfg_read(uint32, uint32, uint32, uint32, uint32 *);
extern int32	pci_sb1250_cfg_write(uint32, uint32, uint32, uint32, uint32);
extern int32	pci_sb1250_cfg_check_status(uint32, uint32);
extern void		pci_sb1250_cfg_clear_status(uint32, uint32);

#endif	/* _PCI_H */

