/* vi:ts=4
 *-----------------------------------------------------------------
 *
 * Name:	pkt-api.h
 *
 * RCS:  $Id: pkt-api.h,v 1.10 2002/10/22 01:30:40 jong Exp $
 *
 * Copyright (C) 2001, Agile Storage, Inc.
 *
 * Description: PKT related data structure definitions & function 
 *				prototypes.
 *
 * Created by:	Shaun Hui
 *
 * Date Created: 6/01/01
 *
 *-----------------------------------------------------------------
 */

#ifndef _PKT_API_H
#define _PKT_API_H

#include <exports/pkt.h>
#include <exports/eee-api.h>

int32 pkt_freePkt( pkt_desc_t *pDesc, char *filename, int32 linenum );
int32 pkt_freeDesc(pkt_desc_t *pDesc, char *filename, int32 linenum );

pkt_desc_t *pkt_allocPkt(ushort16 pktUsr, int type, char *filename, int32 linenum );
pkt_desc_t *pkt_allocChainPkt(ushort16 pktUsr, int32 cnt, pkt_desc_t **ppLast);
pkt_desc_t *pkt_allocDesc(  char *filename, int32 linenum );
pkt_desc_t *pkt_allocMultiDesc( int32 count );
char *pkt_allocBuff( void );

void pkt_showCfg( void );
void pkt_showStats( void );
void pkt_showDescRing( pkt_descRing_t *pRing );
void pkt_showBuffRing( pkt_buffRing_t *pRing );
void pkt_showDesc( pkt_desc_t *pDesc );

void pkt_logErr(char *file, int32 line, int32 errCode, pkt_desc_t *pDesc );

int32 pkt_validateDesc( pkt_desc_t *pDesc, int32 chkDup);
int32 pkt_AdjPkt(pkt_desc_t *pdesc, int offset );

#define PKT_ALLOC_PACKET( _usr )	\
		pkt_allocPkt( _usr, EEE_BUF_LARGE, __FILE__, __LINE__)
#define PKT_ALLOC_DESC( )   		pkt_allocDesc(  __FILE__, __LINE__)

/* su, 06/17/02. 
 * type indicates EEE_BUF_ULTRA, 
 * EEE_BUF_SMALL, or EEE_BUF_LARGE
 */
#define PKT_ALLOC_PKT_BYTYPE(_usr, _type)		\
		pkt_allocPkt( _usr, _type, __FILE__, __LINE__)

#define PKT_FREE_PACKET( _pdesc )	pkt_freePkt( _pdesc, __FILE__, __LINE__)
#define PKT_FREE_DESC( _pdesc )		pkt_freeDesc( _pdesc, __FILE__, __LINE__)

/* get num. of available pkt desc */
#define PKT_GET_NUM_AVAIL()	\
		(pPktCfg->freeDescRing.avail)


#ifdef PKT_USE_TIME_STAMP
#include <exports/timer-api.h>

typedef struct
{
    eeeRTC_t    timeVal[PKT_MAX_TIME_IDX];
} pkt_timeStamp_t;

void *pkt_allocTimeStamp( void );
void pkt_freeTimeStamp( void  *pTime );
void pkt_setTimeStamp( pkt_timeStamp_t *pTime, int32 timeIdx );


#endif

#endif
