/* vi:ts=4
 *-----------------------------------------------------------------
 *
 * Name: pkt.c
 *
 * RCS:  $Id: pkt-init.c,v 1.8 2002/09/17 22:21:37 shaun Exp $
 *
 * Copyright (C) 2001, Agile Storage, Inc.
 *
 * Description: Contains Packet Processing layer related routines
 *
 * Created by:  Shaun Hui
 *
 * Date Created: 6/01/2001
 *
 *-----------------------------------------------------------------
 */
#include <nfx-incl.h>
#include <exports/eee-api.h>
#include <exports/pkt-api.h>
#include <exports/malloc-api.h>

pkt_cfg_t	*pPktCfg;

void pkt_init( void )
{
	pkt_descRing_t	*pDescRing;
	pkt_desc_t		*pDesc;
	int32			i;

	/*-------------------------------
	 * Allocate & clr PKT CFG block.
	 *-------------------------------
	 */
    pPktCfg = (pkt_cfg_t *)eee_ramAlloc( sizeof(pkt_cfg_t ));
	if (!pPktCfg)
	{
		printf("pkt init failed to allocate config.\n");
		PKT_PANIC()
	}
	memset(pPktCfg, 0, sizeof(pkt_cfg_t));

	/*-------------------------------
	 * Initialize free packet desc ring. 
	 *-------------------------------
	 */

	/*-------------------------------
	 * Allocate free packet desc. ring.
	 *-------------------------------
	 */
    pDescRing = &pPktCfg->freeDescRing;
    pDescRing->startAddr = (pkt_desc_t **)eee_ramAlloc(PKT_MAX_DESC_CNT
												* sizeof(pkt_desc_t *));
	if (!pDescRing->startAddr)
	{
		printf("pkt init failed to allocate desc. ring.\n");
		PKT_PANIC()
	}
	pDescRing->endAddr = pDescRing->startAddr+PKT_MAX_DESC_CNT-1;
	pDescRing->rdAddr  = pDescRing->startAddr;
	pDescRing->wrAddr  = pDescRing->startAddr;
    pDescRing->avail   = 0;
    pDescRing->lowWaterMark  = 0;

	/*-------------------------------
	 * Allocate Free descriptors & put 
	 * them in PKT Free desc. ring.
	 *-------------------------------
	 */
    pDesc = (pkt_desc_t *) eee_ramAlloc(sizeof(pkt_desc_t ) * 
										 (PKT_MAX_DESC_CNT-1));
	if (!pDesc)
	{
		printf("pkt init failed to allocate desc. block.\n");
		PKT_PANIC()
	}
    for (i=0; i < PKT_MAX_DESC_CNT-1; i++)
    {
		(*pDescRing->wrAddr++) = pDesc;
    	pDescRing->avail++; 
    	pDescRing->lowWaterMark++; 

		memset(pDesc, 0, sizeof(pkt_desc_t));
		pDesc++;
	}
	pPktCfg->descStart = *(pDescRing->startAddr);
	pPktCfg->descEnd   = *(pDescRing->wrAddr - 1);

}

