#ifndef __DISK_H
#define __DISK_H

/*
 * Copyright (c) 1998 Manuel Bouyer.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *	This product includes software developed by the University of
 *	California, Berkeley and its contributors.
 * 4. Neither the name of the University nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */


/* XXX */
#define BYTEORDER LITTLE_ENDIAN

#define DEV_BSIZE 512

#define ATAPARAMS_SIZE 512

/*
 * Drive parameter structure for ATA/ATAPI.
 * Bit fields: WDC_* : common to ATA/ATAPI
 *             ATA_* : ATA only
 *             ATAPI_* : ATAPI only.
 */
struct ataparams {
    /* drive info */
    ushort16	atap_config;		/* 0: general configuration */
#define WDC_CFG_ATAPI_MASK    		0xc000
#define WDC_CFG_ATAPI    		0x8000
#define	ATA_CFG_REMOVABLE		0x0080
#define	ATA_CFG_FIXED			0x0040
#define ATAPI_CFG_TYPE_MASK		0x1f00
#define ATAPI_CFG_TYPE(x)		(((x) & ATAPI_CFG_TYPE_MASK) >> 8)
#define ATAPI_CFG_TYPE_DIRECT		0x00
#define ATAPI_CFG_TYPE_SEQUENTIAL	0x01
#define ATAPI_CFG_TYPE_CDROM		0x05
#define ATAPI_CFG_TYPE_OPTICAL		0x07
#define ATAPI_CFG_TYPE_NODEVICE		0x1F
#define	ATAPI_CFG_REMOV			0x0080
#define ATAPI_CFG_DRQ_MASK		0x0060
#define ATAPI_CFG_STD_DRQ		0x0000
#define ATAPI_CFG_IRQ_DRQ		0x0020
#define ATAPI_CFG_ACCEL_DRQ		0x0040
#define ATAPI_CFG_CMD_MASK		0x0003
#define ATAPI_CFG_CMD_12		0x0000
#define ATAPI_CFG_CMD_16		0x0001
/* words 1-9 are ATA only */
    ushort16	atap_cylinders;		/* 1: # of non-removable cylinders */
    ushort16	__reserved1;
    ushort16	atap_heads;		/* 3: # of heads */
    ushort16	__retired1[2];		/* 4-5: # of unform. bytes/track */
    ushort16	atap_sectors;		/* 6: # of sectors */
    ushort16	__retired2[3];

    uchar8	atap_serial[20];	/* 10-19: serial number */
    ushort16	__retired3[2];
    ushort16	__obsolete1;
    uchar8	atap_revision[8];	/* 23-26: firmware revision */
    uchar8	atap_model[40];		/* 27-46: model number */
    ushort16	atap_multi;		/* 47: maximum sectors per irq (ATA) */
    ushort16	__reserved2;
    ushort16	atap_capabilities1;	/* 49: capability flags */
#define WDC_CAP_IORDY	0x0800
#define WDC_CAP_IORDY_DSBL 0x0400
#define	WDC_CAP_LBA	0x0200
#define	WDC_CAP_DMA	0x0100
#define ATA_CAP_STBY	0x2000
#define ATAPI_CAP_INTERL_DMA	0x8000
#define ATAPI_CAP_CMD_QUEUE	0x4000
#define	ATAPI_CAP_OVERLP	0x2000
#define ATAPI_CAP_ATA_RST	0x1000
    ushort16	atap_capabilities2;	/* 50: capability flags (ATA) */
#if BYTE_ORDER == LITTLE_ENDIAN
    uchar8	__junk2;
    uchar8	atap_oldpiotiming;	/* 51: old PIO timing mode */
    uchar8	__junk3;
    uchar8	atap_olddmatiming;	/* 52: old DMA timing mode (ATA) */
#else
    uchar8	atap_oldpiotiming;	/* 51: old PIO timing mode */
    uchar8	__junk2;
    uchar8	atap_olddmatiming;	/* 52: old DMA timing mode (ATA) */
    uchar8	__junk3;
#endif
    ushort16	atap_extensions;	/* 53: extentions supported */
#define WDC_EXT_UDMA_MODES	0x0004
#define WDC_EXT_MODES		0x0002
#define WDC_EXT_GEOM		0x0001
/* words 54-62 are ATA only */
    ushort16	atap_curcylinders;	/* 54: current logical cyliners */
    ushort16	atap_curheads;		/* 55: current logical heads */
    ushort16	atap_cursectors;	/* 56: current logical sectors/tracks */
    ushort16	atap_curcapacity[2];	/* 57-58: current capacity */
    ushort16	atap_curmulti;		/* 59: current multi-sector setting */
#define WDC_MULTI_VALID 0x0100
#define WDC_MULTI_MASK  0x00ff
    ushort16	atap_capacity[2];  	/* 60-61: total capacity (LBA only) */
    ushort16	__retired4;
#if BYTE_ORDER == LITTLE_ENDIAN
    uchar8	atap_dmamode_supp; 	/* 63: multiword DMA mode supported */
    uchar8	atap_dmamode_act; 	/*     multiword DMA mode active */
    uchar8	atap_piomode_supp;       /* 64: PIO mode supported */
    uchar8	__junk4;
#else
    uchar8	atap_dmamode_act; 	/*     multiword DMA mode active */
    uchar8	atap_dmamode_supp; 	/* 63: multiword DMA mode supported */
    uchar8	__junk4;
    uchar8	atap_piomode_supp;       /* 64: PIO mode supported */
#endif
    ushort16	atap_dmatiming_mimi;	/* 65: minimum DMA cycle time */
    ushort16	atap_dmatiming_recom;	/* 66: recomended DMA cycle time */
    ushort16	atap_piotiming;    	/* 67: mini PIO cycle time without FC */
    ushort16	atap_piotiming_iordy;	/* 68: mini PIO cycle time with IORDY FC */
    ushort16	__reserved3[2];
/* words 71-72 are ATAPI only */
    ushort16	atap_pkt_br;		/* 71: time (ns) to bus release */
    ushort16	atap_pkt_bsyclr;	/* 72: tme to clear BSY after service */
    ushort16	__reserved4[2];	
    ushort16	atap_queuedepth;   	/* 75: */
#define WDC_QUEUE_DEPTH_MASK 0x0F
    ushort16	__reserved5[4];   	
    ushort16	atap_ata_major;  	/* 80: Major version number */
#define	WDC_VER_ATA1	0x0002
#define	WDC_VER_ATA2	0x0004
#define	WDC_VER_ATA3	0x0008
#define	WDC_VER_ATA4	0x0010
#define	WDC_VER_ATA5	0x0020
    ushort16   atap_ata_minor;  	/* 81: Minor version number */
    ushort16	atap_cmd_set1;    	/* 82: command set suported */
#define WDC_CMD1_NOP	0x4000
#define WDC_CMD1_RB	0x2000
#define WDC_CMD1_WB	0x1000
#define WDC_CMD1_HPA	0x0400
#define WDC_CMD1_DVRST	0x0200
#define WDC_CMD1_SRV	0x0100
#define WDC_CMD1_RLSE	0x0080
#define WDC_CMD1_AHEAD	0x0040
#define WDC_CMD1_CACHE	0x0020
#define WDC_CMD1_PKT	0x0010
#define WDC_CMD1_PM	0x0008
#define WDC_CMD1_REMOV	0x0004
#define WDC_CMD1_SEC	0x0002
#define WDC_CMD1_SMART	0x0001
    ushort16	atap_cmd_set2;    	/* 83: command set suported */
#define WDC_CMD2_RMSN	0x0010
#define WDC_CMD2_DM	0x0001
#define ATA_CMD2_APM	0x0008
#define ATA_CMD2_CFA	0x0004
#define ATA_CMD2_RWQ	0x0002
    ushort16	atap_cmd_ext;		/* 84: command/features supp. ext. */
    ushort16	atap_cmd1_en;		/* 85: cmd/features enabled */
/* bits are the same as atap_cmd_set1 */
    ushort16	atap_cmd2_en;		/* 86: cmd/features enabled */
/* bits are the same as atap_cmd_set2 */
    ushort16	atap_cmd_def;		/* 87: cmd/features default */
#if BYTE_ORDER == LITTLE_ENDIAN
    uchar8	atap_udmamode_supp; 	/* 88: Ultra-DMA mode supported */
    uchar8	atap_udmamode_act; 	/*     Ultra-DMA mode active */
#else
    uchar8	atap_udmamode_act; 	/*     Ultra-DMA mode active */
    uchar8	atap_udmamode_supp; 	/* 88: Ultra-DMA mode supported */
#endif
/* 89-92 are ATA-only */
    ushort16	atap_seu_time;		/* 89: Sec. Erase Unit compl. time */
    ushort16	atap_eseu_time;		/* 90: Enhanced SEU compl. time */
    ushort16	atap_apm_val;		/* 91: current APM value */
    ushort16	__reserved6[35];	/* 92-126: reserved */
    ushort16	atap_rmsn_supp;		/* 127: remov. media status notif. */
#define WDC_RMSN_SUPP_MASK 0x0003
#define WDC_RMSN_SUPP 0x0001
    ushort16	atap_sec_st;		/* 128: security status */
#define WDC_SEC_LEV_MAX	0x0100
#define WDC_SEC_ESE_SUPP 0x0020
#define WDC_SEC_EXP	0x0010
#define WDC_SEC_FROZEN	0x0008
#define WDC_SEC_LOCKED	0x0004
#define WDC_SEC_EN	0x0002
#define WDC_SEC_SUPP	0x0001
};



struct disksc
{
	uint32 cmdreg;
	uint32 ctlreg;
	int channel;
	uint32 drive_flags;
#define DRIVE_ATA	0x0001
#define DRIVE_ATAPI	0x0002
#define DRIVE_OLD	0x0004 
#define DRIVE (DRIVE_ATA|DRIVE_ATAPI|DRIVE_OLD)
#define DRIVE_CAP32	0x0008
#define DRIVE_DMA	0x0010 
#define DRIVE_UDMA	0x0020
#define DRIVE_MODE	0x0040 /* the drive reported its mode */
#define DRIVE_RESET	0x0080 /* reset the drive state at next xfer */
#define DRIVE_DMAERR	0x0100 /* Udma transfer had crc error, don't try DMA */
#define DRIVE_DSCBA	0x0200 /* DSC in buffer availability mode */
#define DRIVE_DSCWAIT	0x0400 /* In wait for DSC to be asserted */
	uint32 state;
	uint8 status;				/* copy of the status register */
	uint8 error;				/* copy of the error register */

	struct disklabel label;

	int opened;
	unsigned int position;
};


/*
 * ata/ATAPI commands description 
 *
 * This structure defines the interface between the ATA/ATAPI device driver
 * and the controller for short commands. It contains the command's parameter,
 * the len of data's to read/write (if any), and a function to call upon
 * completion.
 * If no sleep is allowed, the driver can poll for command completion.
 * Once the command completed, if the error registed is valid, the flag
 * AT_ERROR is set and the error register value is copied to r_error .
 * A separate interface is needed for read/write or ATAPI packet commands
 * (which need multiple interrupts per commands).
 */
struct wdc_command {
    uchar8 r_command;  /* Parameters to upload to registers */
    uchar8 r_head;
    ushort16 r_cyl;
    uchar8 r_sector;
    uchar8 r_count;
    uchar8 r_precomp;
    uchar8 r_st_bmask; /* status register mask to wait for before command */
    uchar8 r_st_pmask; /* status register mask to wait for after command */
    uchar8 r_error;    /* error register after command done */
    volatile ushort16 flags;
#define AT_READ     0x0001 /* There is data to read */
#define AT_WRITE    0x0002 /* There is data to write (excl. with AT_READ) */
#define AT_WAIT     0x0008 /* wait in controller code for command completion */
#define AT_POLL     0x0010 /* poll for command completion (no interrupts) */
#define AT_DONE     0x0020 /* command is done */
#define AT_ERROR    0x0040 /* command is done with error */
#define AT_TIMEOU   0x0080 /* command timed out */
#define AT_DF       0x0100 /* Drive fault */
#define AT_READREG  0x0200 /* Read registers on completion */
    int timeout;	 /* timeout (in ms) */
    void *data;          /* Data buffer address */
    int bcount;           /* number of bytes to transfer */
    void (*callback) (void*); /* command to call once command completed */
    void *callback_arg;  /* argument passed to *callback() */
};

int wdc_pcmcia_init(struct disksc *sc, struct pcmcia_socket *psock);
struct disksc *wdc_alloc_sc(int disk_no);
int wdc_probe(struct disksc *sc);
uint8 wdc_read_cmdreg(struct disksc *sc, int reg);
void wdc_write_cmdreg(struct disksc *sc, int reg, uint8 val);
void do_disk_init(void);
int wdc_exec_command (struct disksc *, struct wdc_command*);
int ata_get_params(struct disksc *sc, int flags, struct ataparams *prms);
int ata_set_mode(struct disksc *sc, uchar8 mode, uchar8 flags);
int loadfile(char *filename, uint32 address);
int loadelffile(char *filename);
int oopen(const char *filename, int mode);
int olseek(int fd, int offset, int whence);
int oread(int fd, void *buf, unsigned int cnt);
void twiddle(void);
int disk_read_block(struct disksc *sc, int blkno, char *buf, int bcount);
void do_disk_toshiba_init(void);
void do_disk_load(int argc, char **argv);
int disk_load_file(char *filename, uint32 load_address);

#define WDC_COMPLETE 0x01
#define WDC_QUEUED   0x02
#define WDC_TRY_AGAIN 0x03

#endif

