/*	$OpenBSD$ */

/*
 * Copyright (c) 2000 Opsycon AB  (www.opsycon.se)
 * Copyright (c) 2000 Rtmx, Inc   (www.rtmx.com)
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *	This product includes software developed for Rtmx, Inc by
 *	Opsycon Open System Consulting AB, Sweden.
 * 4. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS
 * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

#include <nfx-types.h>
#include <nfx-defs.h>
#include <exports/libc.h>

#include "queue.h"
#include "pcmciavar.h"
#include "disklabel.h"
#include "disk.h"
#include "diskio.h"

#if defined(DEBUG)
#define debug(x) { if (disk_debug_param & DEBUG_LOAD) { printf x; }}
#else
#define debug(x)
#endif


extern int disk_debug_param;

void *eee_ramAlloc(unsigned int);

struct	dio_softc {
	int	sc_fd;			/* PROM file id */
	int	sc_part;		/* disk partition number */
	struct	disklabel sc_label;	/* disk label for this disk */
};

int
diostrategy(devdata, rw, bn, reqcnt, addr, cnt)
	void *devdata;
	int rw;
	daddr_t bn;
	u_int reqcnt;
	char *addr;
	u_int *cnt;	/* out: number of bytes transfered */
{
	struct dio_softc *sc = (struct dio_softc *)devdata;
	struct partition *pp = &sc->sc_label.d_partitions[sc->sc_part];
	int s;
	long offset;

	offset = (pp->p_offset + bn) * DEV_BSIZE;

	(void)pmon_lseek(sc->sc_fd, offset, 0);
	s = pmon_read(sc->sc_fd, addr, reqcnt);
	if (s < 0)
		return (-s);
	*cnt = s;
	return (0);
}

int
dioopen(struct open_file *f, ...)
{
	char *ctlr;
	int unit;

	struct dio_softc *sc;
	struct disklabel *lp;
	int i, fd;
	char *msg = "rd err";
	char buf[DEV_BSIZE];
	int cnt;
	daddr_t labelsector;
	static char device[8];
	va_list ap;

	va_start(ap, f);

	ctlr = va_arg(ap, char *);
	unit = va_arg(ap, int);
	if (unit >= 10)
		return (ENXIO);

	debug(("dioopen: %s(%d)\n", ctlr, unit));

	bcopy(ctlr, device, sizeof(device));
	device[2] = unit + '0';
	device[3] = '\0';
	fd = pmon_open(device, 0);
	if (fd < 0) {
		return (ENXIO);
	}

	sc = eee_ramAlloc(sizeof(struct dio_softc));
	bzero(sc, sizeof(struct dio_softc));
	f->f_devdata = (void *)sc;

	sc->sc_fd = fd;
	sc->sc_part = 0;

	lp = &sc->sc_label;
	lp->d_secsize = DEV_BSIZE;
	lp->d_secpercyl = 1;
	lp->d_npartitions = MAXPARTITIONS;
	lp->d_partitions[0].p_offset = 0;
	lp->d_partitions[0].p_size = 0x7fff0000;

	labelsector = LABELSECTOR;

#ifdef USE_DOSBBSECTOR
	/* First check for any DOS partition table */
	i = diostrategy(sc, F_READ, (daddr_t)DOSBBSECTOR, DEV_BSIZE, buf, &cnt);
	if (!(i || cnt != DEV_BSIZE)) {
		struct dos_partition dp, *dp2;
		bcopy(buf + DOSPARTOFF, &dp, NDOSPART * sizeof(dp));
		for (dp2=&dp, i=0; i < NDOSPART; i++, dp2++) {
			if (dp2->dp_size) {
				if((dp2->dp_typ == DOSPTYP_OPENBSD) ||
				   (dp2->dp_typ == DOSPTYP_FREEBSD)) {
					labelsector += dp2->dp_start;
					break;
				}
			}
		}
	}
	else {
		goto bad;
	}
#endif


	/* try to read disk label and partition table information */
	i = diostrategy(sc, F_READ, (daddr_t)labelsector, DEV_BSIZE, buf, &cnt);
	if (i || cnt != DEV_BSIZE) {
		goto bad;
	}
	msg = getdisklabel(buf, lp);
	if (msg) {
		goto bad;
	}

	if (lp->d_partitions[0].p_size == 0) {
		msg = "no part";
bad:
		printf("%s: %s\n", device, msg);
		return (ENXIO);
	}
	return (0);
}

int
dioclose(f)
	struct open_file *f;
{
	return (0);
}
