
#ifndef __PCMCIAVAR_H
#define __PCMCIAVAR_H

#define DEBUG_CMD 0x1
#define DEBUG_READ 0x2
#define DEBUG_PCMCIA 0x4
#define DEBUG_LOAD 0x8

extern int disk_debug_param;

struct pcmcia_window
{
	uint32 addr;
	uint32 size;
	int width;
	int kind;
	uint32 offset;
};
	

struct pcmcia_socket;


struct pcmcia
{
	uint32 regs;				/* The device registers */
	struct pcmcia_socket *sockets[2];
	uint32 bus;					/* PCI bus */
	uint32 dev;					/* PCI device */
	uint32 fun;					/* PCI function */
};


#define PCMCIA_SOCKID_A 0
#define PCMCIA_SOCKID_B 1

#define PCMCIA_SOCK_UNITIALIZED 0x0
#define PCMCIA_SOCK_PRESENT 0x1
#define PCMCIA_SOCK_NOTPRESENT 0x2
#define	PCMCIA_SOCK_ENABLED 0x4

#define	PCMCIA_MEM_ATTR		1
#define	PCMCIA_MEM_COMMON	2

#define	PCMCIA_WIDTH_MEM8	8
#define	PCMCIA_WIDTH_MEM16	16

#define	PCMCIA_WIDTH_MEM_MASK	24

#define	PCMCIA_WIDTH_AUTO	0
#define	PCMCIA_WIDTH_IO8	1
#define	PCMCIA_WIDTH_IO16	2

#define PCMCIA_IOWINDOW 0x400	/* Offset from PCI I/O base */

#define PCIC_MEM_ALIGN PCIC_MEM_PAGESIZE
#define	PCMCIA_CIS_SIZE		1024

struct pcmcia_function;

struct pcmcia_socket
{
	int id;
	struct pcmcia *pcmcia;
	struct pcmcia_window iomap[2];
	struct pcmcia_window mem[4];
	int niowindows;
	int nmemwindows;
	int state;

	char cis1_info_buf[256];
	char *cis1_info[4];

	int cis1_major;
	int cis1_minor;

	uint16 manufacturer;
	uint16 product;

	int number_of_functions;

	int memalloc;

	SIMPLEQ_HEAD(, pcmcia_function) pf_head;
};


struct pcmcia_config_entry {
	int		number;
	uint32	flags;
	int		iftype;
	int		num_iospace;

	/*
	 * The card will only decode this mask in any case, so we can
	 * do dynamic allocation with this in mind, in case the suggestions
	 * below are no good.
	 */
	uint32		iomask;
	struct {
		uint32	length;
		uint32	start;
	} iospace[4];		/* XXX this could be as high as 16 */
	uint16	irqmask;
	int		num_memspace;
	struct {
		uint32	length;
		uint32	cardaddr;
		uint32	hostaddr;
	} memspace[2];		/* XXX this could be as high as 8 */
	int		maxtwins;
	SIMPLEQ_ENTRY(pcmcia_config_entry) cfe_list;
};

struct pcmcia_function {
	/* read off the card */
	int		number;
	int		function;
	int		last_config_index;
	uint32		ccr_base;
	uint32		ccr_mask;
	SIMPLEQ_HEAD(, pcmcia_config_entry) cfe_head;
	SIMPLEQ_ENTRY(pcmcia_function) pf_list;
	struct pcmcia_config_entry *cfe;
	struct pcmcia_socket *socket;
	uint32 pf_ccrh;
#define	pf_ccr_mhandle	pf_pcmh.mhandle
#define	pf_ccr_realsize	pf_pcmh.realsize

	uint32	pf_ccr_offset;
	int		pf_ccr_window;
	long		pf_mfc_iobase;
	long		pf_mfc_iomax;
	struct pcmcia_config_entry *default_cfe;
	struct pcmcia_config_entry temp_cfe;
#if 0
	int		(*ih_fct) __P((void *));
	void		*ih_arg;
	int		ih_ipl;
	int		pf_flags;
#endif	
};



struct pcmcia_tuple {
	unsigned int	code;
	unsigned int	length;
	uint32		mult;
	uint32		ptr;
	uint32		memh;
};

// RoHS compliant part
#define PCMCIA_PCI_VENDOR_2  0x104C
#define PCMCIA_PCI_PRODUCT_2 0xAC55

#define PCMCIA_PCI_VENDOR  0x1013
#define PCMCIA_PCI_PRODUCT 0x1100

#define PCMCIA_PCI_DEVICE 8
#define PCMCIA_PCI_BUS 0

#define PCMCIA_CFE_MWAIT_REQUIRED	0x0001
#define PCMCIA_CFE_RDYBSY_ACTIVE	0x0002
#define PCMCIA_CFE_WP_ACTIVE		0x0004
#define PCMCIA_CFE_BVD_ACTIVE		0x0008
#define PCMCIA_CFE_IO8			0x0010
#define PCMCIA_CFE_IO16			0x0020
#define PCMCIA_CFE_IRQSHARE		0x0040
#define PCMCIA_CFE_IRQPULSE		0x0080
#define PCMCIA_CFE_IRQLEVEL		0x0100
#define PCMCIA_CFE_POWERDOWN		0x0200
#define PCMCIA_CFE_READONLY		0x0400
#define PCMCIA_CFE_AUDIO		0x0800


void delay_us(uint32 us);
int do_pcmcia_init(void);
uint8 pcmcia_read_ext_reg(struct pcmcia_socket *sc, uint8 reg);
void pcmcia_write_ext_reg(struct pcmcia_socket *sc, uint8 reg, uint8 data);
struct pcmcia *pcmcia_alloc_sc(void);
struct pcmcia_socket *pcmcia_socket(struct pcmcia *sc, int sockid);
char *pcmcia_sockname(struct pcmcia_socket *sc);
int pcmcia_device_present(struct pcmcia_socket *sc);
void pcmcia_function_init(struct pcmcia_function *pf,
						 struct pcmcia_config_entry *cfe);
int pcmcia_function_enable(struct pcmcia_function *pf);
struct pcmcia *pcmcia_sc(void);
void pcmcia_map_io(struct pcmcia_socket *sc);
void pcmcia_enable_socket(struct pcmcia_socket *sc);


#define	pcmcia_cis_read_1(tuple, idx0)					\
     (*(uint8 *)((tuple)->memh + (tuple)->mult*(idx0)))

#define	pcmcia_tuple_read_1(tuple, idx1)				\
	(pcmcia_cis_read_1((tuple), ((tuple)->ptr+(2+(idx1)))))

#define	pcmcia_tuple_read_2(tuple, idx2)				\
	(pcmcia_tuple_read_1((tuple), (idx2)) | 			\
	 (pcmcia_tuple_read_1((tuple), (idx2)+1)<<8))

#define	pcmcia_tuple_read_3(tuple, idx3)				\
	(pcmcia_tuple_read_1((tuple), (idx3)) |				\
	 (pcmcia_tuple_read_1((tuple), (idx3)+1)<<8) |			\
	 (pcmcia_tuple_read_1((tuple), (idx3)+2)<<16))

#define	pcmcia_tuple_read_4(tuple, idx4)				\
	(pcmcia_tuple_read_1((tuple), (idx4)) |				\
	 (pcmcia_tuple_read_1((tuple), (idx4)+1)<<8) |			\
	 (pcmcia_tuple_read_1((tuple), (idx4)+2)<<16) |			\
	 (pcmcia_tuple_read_1((tuple), (idx4)+3)<<24))

#define	pcmcia_tuple_read_n(tuple, n, idxn)				\
	(((n)==1)?pcmcia_tuple_read_1((tuple), (idxn)) :		\
	 (((n)==2)?pcmcia_tuple_read_2((tuple), (idxn)) :		\
	  (((n)==3)?pcmcia_tuple_read_3((tuple), (idxn)) :		\
	   /* n == 4 */ pcmcia_tuple_read_4((tuple), (idxn)))))


#endif

