
#include <nfx-types.h>
#include <nfx-defs.h>
#include <exports/libc.h>

#include "queue.h"
#include "pcmciavar.h"
#include "disklabel.h"
#include "disk.h"
#include "diskio.h"

#define assert NFX_ASSERT

#undef DEBUG
#if defined(DEBUG)
#define debug(x) printf x
#else
#define debug(x)
#endif

extern int strcmp(const char *, const char *);

#if defined(DEBUG)
static char *
whence_str(int whence)
{
	switch (whence) {
	case SEEK_SET:
		return "SEEK_SET";
	case SEEK_CUR:
		return "SEEK_CUR";
	case SEEK_END:
		return "SEEK_END";
	default:
		assert(0);
		return "";
	}
}
#endif

int pmon_lseek(int fd, off_t offset, int whence)
{
	struct disksc *dsc = wdc_alloc_sc(fd);

	debug(("pmon_lseek: fd=%d off=%d whence=%s ", fd, offset,
		   whence_str(whence)));
	assert(dsc->opened);
	switch (whence) {
	case SEEK_SET :
		dsc->position = offset;
		break;
	case SEEK_CUR:
		dsc->position += offset;
		break;
	case SEEK_END:
		assert(0);
		break;
	default:
		return -1;
	}

	debug(("new pos=%d\n", dsc->position));

	
	return dsc->position;
}


int pmon_read(int fd, char *buf, int count)
{
	struct disksc *dsc = wdc_alloc_sc(fd);
	char *readptr;
	int readbytes;
	int error = 0;
	
	debug(("pmon_read:fd=%d cnt=%d\n", fd, count));
	assert(dsc->opened);
	for (readptr = buf, readbytes = 0; readbytes < count && !error;
		 readbytes += 512, readptr += 512) {
		error = !disk_read_block(dsc, dsc->position / 512, readptr, 512);
		dsc->position += 512;
	}

	return error ? -1 : count;
}


/*
 * do_pmon_open - open the disk with the given id in the pcmcia socket
 * sockid.
 */
static int
do_pmon_open(int fd, int sockid)
{
	struct pcmcia_socket *sock = pcmcia_socket(pcmcia_sc(), sockid);
	int rc = -1;
	struct disksc *dsc;
	if (pcmcia_device_present(sock)
		&& wdc_pcmcia_init((dsc = wdc_alloc_sc(fd)), sock)
		&& wdc_probe(dsc)) {
		rc = fd;
	}
	return rc;
}


int pmon_open(char *file, int mode, ...)
{
	int fd;
	int sockid;
	
	if (!strcmp(file, "wd0")) {
		sockid = PCMCIA_SOCKID_A;
		fd = 0;
	}
	else if (!strcmp(file, "wd1")) {
		sockid = PCMCIA_SOCKID_B;
		fd = 1;
	}
	else {
		printf("pmon_open: invalid file %s\n", file);
		return -1;
	}

	return do_pmon_open(fd, sockid);
}
