/* vi:ts=4
 *-----------------------------------------------------------------
 *
 * Name: ppmd.c
 *
 * RCS:  $Id: ppmd.c,v 1.8 2002/01/30 01:17:38 swang Exp $
 *
 * Copyright (C) 2001, Agile Storage, Inc.
 *
 * Description: this module take care of control plane messaging
 *				on ssc
 *
 * Created by:	Su Wang
 *
 * Date Created: 10/16/2001
 *
 *-----------------------------------------------------------------
 */
#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include <nfx-error.h>
#include <nfx-types.h>
#include <nfx-defs.h>
#include <exports/eee-api.h>
#include <exports/anpssc-api.h>
#include <exports/ppm-api.h>

#define PPM_PRINTF		printf

extern void ipmd_Init();

ppmc_ctrl_t		ppm_cb[PPM_CLIENT_MAX];
int				ppm_appid;


/* 
 *-----------------------------------------------------------------
 *
 * Name: ppmd_Init
 *
 * Description: Initiate ppm on txrx.
 *
 * Created by:	Su Wang
 *
 * Date Created: 10/16/2001
 *
 *-----------------------------------------------------------------
 */
static void
ppmd_Init()
{
  /* intialize control block */
  memset(ppm_cb, 0, sizeof(ppm_cb));

  /* register with Kernel */
  ppm_appid = regAgileApp("ppmsg");
  
  if (ppm_appid == -1)
	{
	  printf("ppm_Init: invalid app name \n");
	  exit(0);
	}

  /* ppm clients can be initialized here */
  ipmd_Init();

  return;
}

#if defined(xxx)
bsd_ShowPkt(char* pkt, int len)
{
  int new_line = 1;
  int count = 0;

  if (!pkt || !len)
    return;

  printf("\n");
  printf("Print packet size %d\n", len);
  while (len > count)
    {
      if (new_line)
	{
	  printf("0X%08lX:      ", count);
	  new_line = 0;
	}
      
      printf("%02X ", (*pkt++) & 0xff);
      count++;
      
      if (count % 16 == 0)
	{
	  printf("\n");
	  new_line = 1;
	}
    }

  printf("\n\n");
  return;

}
#endif /* xxx */

/* 
 *-----------------------------------------------------------------
 *
 * Name: main
 *
 * Description: ppm main function.
 *
 * Created by:	Su Wang
 *
 * Date Created: 10/16/2001
 *
 *-----------------------------------------------------------------
 */
int
main()
{
  ppm_mhdr_t	*ppm_msgbuf;
  char			app[16];
  uint32		cpu;
  uint32		slot;
  uint32		plane;
  int			rcvbyte;
  int			gid;
  int			action;

  /* ensure byte alignment */
  ppm_msgbuf =(ppm_mhdr_t*)malloc(PPM_MSG_MAX_SIZE);
  
  ppmd_Init();

  while (1)
	{
	  rcvbyte = recvAgileMsg(ppm_appid,
							 ppm_msgbuf,
							 PPM_MSG_MAX_SIZE,
							 0,
							 app,
							 &cpu, 
							 &slot,
							 &plane);


	  if (rcvbyte == -1)
		{
		  printf(" error receiving\n");
		  continue;
		}

	  PPM_PRINTF("msg rcv from app %s, slot %d, cpu %d, plane %d\n",
				 app, slot, cpu, plane);


	  if ((rcvbyte > PPM_MSG_MAX_SIZE) ||
		  rcvbyte < ppm_msgbuf->ppmh_size)
		{
		  printf("inconsistent rcv size rcv=%d, size=%d\n",
				 rcvbyte, ppm_msgbuf->ppmh_size);
		  continue;
		}

	  /* rcv byte does not always reflect the corret ppmsg size */
	  rcvbyte = ppm_msgbuf->ppmh_size;
//	  bsd_ShowPkt(ppm_msgbuf, rcvbyte);
	  rcvbyte -= sizeof(ppm_mhdr_t);
//	  bsd_ShowPkt(ppm_msgbuf+1, rcvbyte);

	  /* msg received from one of ppm client on txrx */
	  if (app &&
		  strcmp(app, "ppmsg") == 0)
		{
		  /* give to client */
		  slot = ppm_msgbuf->ppmh_slot;
		  cpu = ppm_msgbuf->ppmh_cpu;
		  gid = ppm_msgbuf->ppmh_gid;
		  action = ppm_msgbuf->ppmh_action;

		  /* if msg from the same cpu, something messed up */
		  if (slot == SSC_SLOT && cpu == NFX_SSC_UNIX)
			{
			  PPM_PRINTF("msg rcved at its origination\n");
			  continue;
			}

		  if ((gid < PPM_CLIENT_MAX) && ppm_cb[gid].ppmc_call)
			(*ppm_cb[gid].ppmc_call)(gid, 
									 action, 
									 (ppm_msgbuf+1),
									 rcvbyte,
									 cpu,
									 slot,
									 ppm_cb[gid].ppmc_data);
		}
	  else /* msg from app,  peer to peer */
		{
		  char			*gotapp;

		  gid = ppm_msgbuf->ppmh_gid;
		  action = ppm_msgbuf->ppmh_action;
		  
		  /* don't bother with app name for now */
		  //gotapp = eee_getAppNameFromId(ppm_msgbuf->ppmh_client);
		  // strcpy(app, gotapp);

		  if ((gid < PPM_CLIENT_MAX) && ppm_cb[gid].ppmc_scall)
			(*ppm_cb[gid].ppmc_scall)(ppm_msgbuf->ppmh_mtype,
									  app, 
									  gid, 
									  action,
									  (void*)(ppm_msgbuf+1),
									  rcvbyte,
									  ppm_cb[gid].ppmc_data);
		  
		}
	}
  
  free(ppm_msgbuf);
  return 1;
}


/* 
 *-----------------------------------------------------------------
 *
 * Name: ppmd_RegRcv
 *
 * Description: register with txrx control. 
 *
 * Created by:	Su Wang
 *
 * Date Created: 10/16/2001
 *
 *-----------------------------------------------------------------
 */
int32							/* NFX_OK, NFX_ERR */
ppmd_RegRcv(int			gid,	/* group id */
		   void*		usrdata,
		   ppm_func		func,	/* callbck when msg received */
		   ppm_sscfunc	sfunc	/* callbck to handle ui etc. on ssc */
		   )
{
  if ((gid >= PPM_CLIENT_MAX)	||
	  (func == NULL)			||
	  ppm_cb[gid].ppmc_gid		||
	  ppm_cb[gid].ppmc_data		||
	  ppm_cb[gid].ppmc_call     ||
	  ppm_cb[gid].ppmc_scall )
	return NFX_ERR;

  ppm_cb[gid].ppmc_gid = gid;
  ppm_cb[gid].ppmc_data = usrdata;
  ppm_cb[gid].ppmc_call = func;
  ppm_cb[gid].ppmc_scall = sfunc;

  return NFX_OK;
}


/* 
 *-----------------------------------------------------------------
 *
 * Name: ppmd_UnregRcv
 *
 * Description: unregister with txrx control
 *
 * Created by:	Su Wang
 *
 * Date Created: 10/16/2001
 *
 *-----------------------------------------------------------------
 */
int32							/* NFX_OK, NFX_ERR */
ppmd_UnregRcv(int gid)		/* group id */
{
  if ((gid >= PPM_CLIENT_MAX))
	 return NFX_ERR;
  
  bzero(&ppm_cb[gid], sizeof(ppmc_ctrl_t));
  return NFX_OK;
}

/* 
 *-----------------------------------------------------------------
 *
 * Name: ppmd_SendMsg
 *
 * Description: send contrl msg to another card, client has msg
 *				ownership, should free msg buffer after send.
 *
 * Created by:	Su Wang
 *
 * Date Created: 10/16/2001
 *
 *-----------------------------------------------------------------
 */
int32							/* NFX_OK, NFX_ERR */
ppmd_SendMsg(int   gid,			/* group id */
			int	   action,		/* action on msg */
			int	   slot,
			int    cpu,
			void   *msg,		/* ptr to real msg */
			uint32 size,		/* size of msg */
			int    bcast
			)
{
  ppm_mhdr_t	*ppmh;

  /* msg must be allocated with PPM_MSG_ALLOC, */
  /* there are some space at begining of buff for ppm hdr */
  ppmh = (ppm_mhdr_t*)(((char*)msg) - sizeof(ppm_mhdr_t));
  
  ppmh->ppmh_cpu = NFX_SSC_UNIX;
  ppmh->ppmh_slot= SSC_SLOT;
  ppmh->ppmh_gid = gid;
  ppmh->ppmh_action = action;
  ppmh->ppmh_size = size + sizeof(ppm_mhdr_t);


  if (ppmh->ppmh_size > PPM_MSG_MAX_SIZE)
	return NFX_ERR;

  /* check for broadcast here */
  if (bcast)
	{
	  /* no bcast yet, send to TXRX */
	  slot = 1;
	  cpu  = 0; /* NFX_NFP_TX */
	}
  

  // bsd_ShowPkt(ppmh, size+sizeof(*ppmh));	
  if (sendAgileMsg(ppm_appid, ppmh, size+sizeof(*ppmh), 
				  "ppmsg", cpu, slot, MGMTPLANE,
				   NULL, NULL, 0) < 0)
	{
	  printf("ppmd_SendMsg on ssc: send fail\n");
	  return NFX_ERR;
	}

  return NFX_OK;
}

/* 
 *-----------------------------------------------------------------
 *
 * Name: ppmd_SendMsgSSC
 *
 * Description: send contrl msg to another module on SSC,
 *				client has ownership, should free msg buffer after send.
 *
 * Created by:	Su Wang
 *
 * Date Created: 11/12/2001
 *
 *-----------------------------------------------------------------
 */
int32 
ppmd_SendMsgSSC(char		*app,
			   int			gid,
			   int          action,
			   void		    *msg,
			   uint32	    size
			   )
{
  ppm_mhdr_t	*ppmh;

  ppmh->ppmh_gid = gid;
  ppmh->ppmh_action = action;
  ppmh->ppmh_size = size + sizeof(ppm_mhdr_t);

  if (ppmh->ppmh_size > PPM_MSG_MAX_SIZE)
	return NFX_ERR;

  if (sendAgileMsg(ppm_appid, ppmh, size+sizeof(*ppmh), 
				   app, 0, 0, MGMTPLANE,
				   NULL, NULL, 0) < 0)
	{
	  printf("ppmd_SendMsgSSC on ssc: send fail\n");
	  return NFX_ERR;
	}

  return NFX_OK;
}
