/* vi:ts=4
 *-----------------------------------------------------------------
 *
 * Name: ppm-api.h
 *
 * RCS:  $Id: ppm-api.h,v 1.3 2001/12/18 20:43:38 swang Exp $
 *
 * Copyright (C) 2001, Agile Storage, Inc.
 *
 * Description: this module take care of control plane messaging
 *				between txrx and ssc
 *
 * Created by:	Su Wang
 *
 * Date Created: 10/16/2001
 *
 *-----------------------------------------------------------------
 */
#ifndef _PPM_API_H
#define _PPM_API_H

#include "exports/ppm-hdr.h"

/* define groud ids */
#define PPM_IPM			0
#define PPM_CLIENT_MAX	1		/* depend on num of client registered */

/* define action */
#define PPM_ACT_NO		0
#define PPM_ACT_UI		1

/* max ppm msg size, including ppm hdr, client can change */
/* the value  */
#define PPM_MSG_MAX_SIZE		512		

/* call bck function definitions */
typedef void	(*ppm_func)(int			gid,/* client id of ppm */
							int			action,	/* action needed */
							void		*msg, /* ptr beyond ppm header */
							int			size, /* msg size without ppm header */
							uint32		scpu, /* src cpu */
							uint32		sslot, /* src slot */
							void		*usrdata); 
								/* *msg is ptr to real msg, */
								/* I have ownership for msg buf, */
								/* client shouldnot free it */

#if defined(SSC)
/* call bck when receive msg from other module on ssc */
typedef void	(*ppm_sscfunc)(int		mtype,/* msg types: range defined in nfx-def.h */
							   char		*app, /* app name of sender */
							   int		gid, /* client id dest */
							   int		action,
							   void		*msg, /* ptr beyond ppm header */
							   int		size, /* msg size with ppm header */
							   void		*usrdata
							   );	
#endif /* SSC */

/* control blocks */
typedef struct ppm_ctrl {
  int		    ppmc_gid;
  void			*ppmc_data;		/* user defined data */
  ppm_func		ppmc_call;		/* callbck to handle msg from another card */
#if defined(SSC)
  ppm_sscfunc	ppmc_scall;		/* callbck to handle msg from an application on ssc */
#endif /* SSC */
} ppmc_ctrl_t;
  

/* message allocation taken care of msg hdr, and returns */
/* ptr to the real msg */
#if defined(NFP_TXRX)

#define PPM_MSG_ALLOC(_ptr, _psize)								\
{																\
	(_ptr) = eee_ramAlloc(_psize+sizeof(ppm_mhdr_t));			\
	(_ptr) = (_ptr)? ((char*)_ptr)+(sizeof(ppm_mhdr_t)):0;		\
}

/* message free expect ptr to real msg */
#define PPM_MSG_FREE(_ptr)										\
		eee_ramDealloc((char*)_ptr - sizeof(ppm_mhdr_t))

#else  /* NFP_SSC */

#define PPM_MSG_ALLOC(_ptr, _psize)								\
{																\
	(_ptr) = malloc(_psize+sizeof(ppm_mhdr_t));					\
	(_ptr) = (_ptr)? ((char*)_ptr)+(sizeof(ppm_mhdr_t)):0;		\
}

/* message free expect ptr to real msg */
#define PPM_MSG_FREE(_ptr)										\
		free((char*)_ptr - sizeof(ppm_mhdr_t))

#endif /* NFP_TXRX */

#if defined(NFP_TXRX) 
/* 
 *-----------------------------------------------------------------
 *
 * Name: ppm_RegRcv
 *
 * Description: register with txrx control. 
 *
 * Created by:	Su Wang
 *
 * Date Created: 10/16/2001
 *
 *-----------------------------------------------------------------
 */
int32							/* NFX_OK, NFX_ERR */
ppm_RegRcv(int			gid,	/* group id */
		   void*		usrdata,
		   ppm_func		func	/* callbck when msg received */
		   );

/* 
 *-----------------------------------------------------------------
 *
 * Name: ppm_UnregRcv
 *
 * Description: unregister with txrx control
 *
 * Created by:	Su Wang
 *
 * Date Created: 10/16/2001
 *
 *-----------------------------------------------------------------
 */
int32							/* NFX_OK, NFX_ERR */
ppm_UnregRcv(int gid			/* group id */
			 );

/* 
 *-----------------------------------------------------------------
 *
 * Name: ppm_SendMsg
 *
 * Description: send contrl msg to another card, client has msg
 *				ownership, should free msg buffer after send.
 *
 * Created by:	Su Wang
 *
 * Date Created: 10/16/2001
 *
 *-----------------------------------------------------------------
 */
int32							/* NFX_OK, NFX_ERR */
ppm_SendMsg(int	   gid,			/* group id */
			int    action,		/* action on msg */
			int	   slot,		/* dest slot */
			int	   cpu,			/* dest cpu */
								/* msg send to ssc */
			void	*msg,		/* ptr to real msg */
			uint32	size,		/* size of msg */
			int	    bcast		/* bcast msg ? */
			);

#else  /* SSC  */
/* 
 *-----------------------------------------------------------------
 *
 * Name: ppmd_RegRcv
 *
 * Description: register with txrx control. 
 *
 * Created by:	Su Wang
 *
 * Date Created: 10/16/2001
 *
 *-----------------------------------------------------------------
 */
int32							/* NFX_OK, NFX_ERR */
ppmd_RegRcv(int		gid,	/* group id */
		   void*		usrdata,
		   ppm_func		func,	/* callbck when msg received */
		   ppm_sscfunc	sfunc	/* callbck to handle ui etc. on ssc */
			);

/* 
 *-----------------------------------------------------------------
 *
 * Name: ppmd_UnregRcv
 *
 * Description: unregister with txrx control
 *
 * Created by:	Su Wang
 *
 * Date Created: 10/16/2001
 *
 *-----------------------------------------------------------------
 */
int32							/* NFX_OK, NFX_ERR */
ppmd_UnregRcv(int gid			/* group id */
			 );

/* 
 *-----------------------------------------------------------------
 *
 * Name: ppm_SendMsg
 *
 * Description: send contrl msg to another card, client has msg
 *				ownership, should free msg buffer after send.
 *
 * Created by:	Su Wang
 *
 * Date Created: 10/16/2001
 *
 *-----------------------------------------------------------------
 */
int32							/* NFX_OK, NFX_ERR */
ppmd_SendMsg(int gid,			/* group id */
			int action,			/* action on msg */
			int	   slot,		/* dest slot */
			int	   cpu,			/* dest cpu */
								/* msg send to ssc */
			void	*msg,		/* ptr to real msg */
			uint32	size,		/* size of msg */
			int	    bcast		/* bcast msg ? */
			);

/* 
 *-----------------------------------------------------------------
 *
 * Name: ppm_SendMsgSSC
 *
 * Description: send contrl msg to another module on SSC,
 *				client has ownership, should free msg buffer after send.
 *
 * Created by:	Su Wang
 * 
* Date Created: 11/12/2001
 *
 *-----------------------------------------------------------------
 */
int32 
ppmd_SendMsgSSC(char			*app,
			   int				gid,
			   int				action,
			   void				*msg,
			   uint32			size
			   );
#endif /* NFP_TXRX  */

#endif /* _PPM_API_H */
