/* vi:ts=4
 *-----------------------------------------------------------------
 *
 * Name: ppm.c
 *
 * RCS:  $Id: ppm.c,v 1.5 2002/01/19 02:18:51 swang Exp $
 *
 * Copyright (C) 2001, Agile Storage, Inc.
 *
 * Description: this module take care of control plane messaging
 *				between txrx and ssc
 *
 * Created by:	Su Wang
 *
 * Date Created: 10/16/2001
 *
 *-----------------------------------------------------------------
 */
#include <nfx-incl.h>
#include "exports/eee-api.h"
#include "exports/cm-api.h"
#include "exports/ppm-api.h"

ppmc_ctrl_t		ppm_cb[PPM_CLIENT_MAX];
int				ppm_appid;
uint32			ppmeee_src;		/* eee src address */

static void		ppm_ProcRcvMsg(eee_descPtr_t);

/* 
 *-----------------------------------------------------------------
 *
 * Name: ppm_Init
 *
 * Description: Initiate ppm on txrx.
 *
 * Created by:	Su Wang
 *
 * Date Created: 10/16/2001
 *
 *-----------------------------------------------------------------
 */
void 
ppm_Init()
{
  /* intialize control block */
  memset(ppm_cb, 0, sizeof(ppm_cb));

  /* register with EEE */
  ppm_appid = eee_allocateApplicationId("ppmsg");

  if (ppm_appid == EEE_ERR_INVALID_APP_NAME)
	panic("ppm_Init: invalid app name \n");

  if (eee_registerReceiveFunc(ppm_appid, ppm_ProcRcvMsg, ppm_ProcRcvMsg)
	  != NFX_OK)
	panic("ppm_Init: duplicated registeration with EEE\n");
  
  /* initiaze eee src address */
  ppmeee_src = PACK_UNICAST_PORT_ADDRESS(
										nfxMySlotId,
										nfxMyCPUId,
										ppm_appid,
										EEE_MGMT,
										NFX_COS_HIGH,
										0);

  if (ppm_appid == EEE_ERR_INVALID_APP_NAME)
	panic("ppm_Init: invalid app name \n");

  /* ppm clients can be initialized here */
  ipm_Init();

  return;
}

/* 
 *-----------------------------------------------------------------
 *
 * Name: ppm_ProcRcvMsg
 *
 * Description: process msg rcved on eee layer.
 *
 * Created by:	Su Wang
 *
 * Date Created: 10/16/2001
 *
 *-----------------------------------------------------------------
 */
static void 
ppm_ProcRcvMsg(eee_descPtr_t epdesc)
{
  int			size;
  int		    gid;
  int   		action;
  ppm_mhdr_t	*ppm_msgbuf = 0;
  uint32		slot;
  uint32		cpu;
  

  size = EEE_GET_PKT_LEN(epdesc);

  if (size > PPM_MSG_MAX_SIZE)
	goto out;

  ppm_msgbuf = (ppm_mhdr_t*)eee_ramAlloc(size);
  if (!ppm_msgbuf)
	goto out;

  size = eee_copyPacketToLocalBuffer((void*)ppm_msgbuf,
									epdesc,
									size,
									0); 

  /* free edesc because local buff is used*/
  eee_freePacket(epdesc);

  if (size != EEE_ERR_PACKET_COPY_FAILED ||
	  size < ppm_msgbuf->ppmh_size) 
	{

	  /* size does not always reflect the correct ppmsg size */
	  size = ppm_msgbuf->ppmh_size;

	  /* get rid of ppm hdr */
	  size -= sizeof(ppm_mhdr_t);
	  
	  slot = ppm_msgbuf->ppmh_slot;
	  cpu = ppm_msgbuf->ppmh_cpu;
	  gid = ppm_msgbuf->ppmh_gid;
	  action = ppm_msgbuf->ppmh_action;
	  

	  /* gid is within range and registered */
	  if ((gid < PPM_CLIENT_MAX) && ppm_cb[gid].ppmc_call)
		(*ppm_cb[gid].ppmc_call)(gid, 
								 action, 
								 (ppm_msgbuf+1),
								 size,
								 cpu,
								 slot,
								 ppm_cb[gid].ppmc_data);
	}

out:
  if (ppm_msgbuf)
	eee_ramDealloc(ppm_msgbuf);
 
  return;
}
