/*
 *-----------------------------------------------------------------
 *
 * Name:	from-api.h
 *
 * RCS:  $Id: from-api.h,v 1.10 2002/11/06 22:46:06 maximk Exp $
 *
 * Copyright (C) 2001, Agile Storage, Inc.
 *
 * Description:	Flash ROM API functions
 *
 * Created by:	Rick Lund
 *
 * Date Created:	10/8/01
 *
 *-----------------------------------------------------------------
 */

#if defined(__OpenBSD__) && defined(_KERNEL)
#define R9000 1
#define R7000 1
#define isOldFC() 0
#endif

#include <sys/amd29f040.h>

#ifdef SSC_MGMT
#define TYPE_LEN    ushort16
#else
#define TYPE_LEN    uchar8
#endif

#define FROM_CBX_PRIMARY   0
#define FROM_CBX_ALTERNATE 1


void    from_init(void);
int     from_cfg_proms(void);
int32	from_erasesector(int32 sec_num);
int32	from_erasedevice(void);
int32	from_programbytes(int32 offset, TYPE_LEN * src_buf, uint32 len);
int32	from_readbytes(int32 offset, uchar8 * dest_buf, uint32 len);
int32   from_readId(TYPE_LEN * man, TYPE_LEN * dev);
void    from_print_info(void);

/* status returned by above functions */
#define FROM_ERR_FAIL		(FROM_UPGRADE_ERROR_BASE+1)	/* parameter error */
#define FROM_ERR_PROTECTED	(FROM_UPGRADE_ERROR_BASE+2)
											/* can't program protected sector */
#define FROM_ERR_PARTIAL	(FROM_UPGRADE_ERROR_BASE+3)
												/* can't erase partial sector */
#define FROM_ERR_NOMEM		(FROM_UPGRADE_ERROR_BASE+4)
													/* no memory for buffer */
#define FROM_ERR_FATAL		(FROM_UPGRADE_ERROR_BASE+5)
												/* fatal programming error */

#define FROM_SEL_INVALID    (-1)

struct prom_part_cb;
struct prom_part_desc;

address_t   from_get_base(void);
uint32      from_get_size(void);
uint32      from_get_sectors(void);
void        from_set_primary(void);
void        from_set_alternate(void);
void        from_set_current(int select);
int         from_swap_base(void);
int         from_select_recov(void);
int         from_select_std(void);
int32       from_upgrade(uint32	ip, uchar8	* fname, uint32	fetch_fp);

extern void from_old_prom_print(uchar8 *buf);
extern int from_old_prom_type(uchar8 *buf, int32 prom_len);
extern int from_check_old_prom_type(uchar8 *buf, int len);
extern int from_get_prom_info(char *buf, int len, struct prom_part_cb **,
							  struct prom_part_desc **);
extern int from_device_present(int id);

extern int from_current;
extern int primary_is_rprom;
extern int from_probed_proms;

extern nfx_spinlock_t	prom_lock;

#define FROM_BASE                   (from_get_base())
#define FROM_SIZE                   (from_get_size())
#define FROM_SECS                   (from_get_size()/FROM_SECTORSIZE)
#define FROM_ADDR(_tcast, _offs)    ((_tcast)(FROM_BASE +  (_offs)))

#if   defined(SIBYTE) || defined (R9000)
#define FROM_MAX_SIZE               AMD_29LV033_SIZE
#elif defined(R7000)
#define FROM_MAX_SIZE               AMD_29LV017_SIZE
#else
#define FROM_MAX_SIZE               AMD_29LV040_SIZE
#endif

#define FROM_CRASHDUMP_OFFSET       (FROM_SIZE - (16*1024))        


#if defined(PROM_SIBYTE) || defined(PROM_R7000) || defined (PROM_R9000)
/*
 * size the prom_upgrade area to be the large flash size plus extra 2MB
 * we're using load area in memory so there's plenty of space
 */
#define PROM_UPGRADE_MAX_SIZE       (FROM_MAX_SIZE + (2*1024*1024))
extern uchar8 * prom_upgrade_area;
#else
/*
 * size the prom_upgrade area to be the large flash size plus extra 4K
 */
#define PROM_UPGRADE_MAX_SIZE       (FROM_MAX_SIZE + 4096)
extern uchar8 prom_upgrade_area[PROM_UPGRADE_MAX_SIZE];
#endif

#ifdef FAKE_SEEP_ON_FLASH_WAR
#define FROM_FAKE_SEEP_OFFSET       (FROM_CRASHDUMP_OFFSET - SEEP_TOTAL_LEN)       
#endif
