/*
 *------------------------------------------------------------------------------
 *
 * Name:	raid-main.c
 *
 * RCS:		$Id: raid-main.c,v 1.1 2002/04/26 03:39:15 raster Exp $
 *
 * Copyright (C) 2001, 2002 Agile Storage Inc.
 *
 * Description:	RAID config manager code.
 *
 * Created by:	Radek Aster
 *
 * Date Created: 4/16/02
 *
 *------------------------------------------------------------------------------
 */

#include "raid-common.h"
#include "raid-msg.h"
#include <signal.h>

void raid_terminate_handler(int);

int terminate = 0;

int
main(int argc, char **argv)
{
	/* Register SIGTERM handler */
	signal(SIGKILL, raid_terminate_handler);
	signal(SIGINT, raid_terminate_handler);
	signal(SIGTERM, raid_terminate_handler);

	raid_device_management_init();
	raid_msg_init();

	kill (getppid(), SIGUSR1);
	E_LOG (class_1, notice_s, eee_getApplicationId("raid-config"), 0, 0, 0,
	       ("RCM: RAID configuration Manager (c)2002: Started"));

	while (!terminate) {
		raid_receive_poll();
		raid_device_management_poll();
	}

	raid_cleanup();

	E_LOG (class_1, notice_s, eee_getApplicationId("raid-config"), 0, 0, 0,
	       ("RCM: RAID configuration Manager (c)2002: Terminated"));

	exit(0);
}

void
panic(char * fmt, ...)
{
	va_list	ap;
	int32	len;
	char	panic_buf[256];

	va_start(ap, fmt);
	len = vsprintf(panic_buf, fmt, ap);
	va_end(ap);

	printf("PANIC: %s\n", panic_buf);
	abort();
}

void 
raid_terminate_handler(int sig)
{
	RAID_DBG(2, ("raid_terminate_handler: signal %d\n", sig));
	terminate = 1;
}

void
raid_terminate(void)
{
	RAID_DBG(2, ("raid_terminate: \n"));
	terminate = 1;
}
