/*
 *------------------------------------------------------------------------------
 *
 * Name:	raid-sm.h
 *
 * RCS:		$Id: raid-sm.h,v 1.5 2002/09/28 19:51:14 nitinb Exp $
 *
 * Copyright (C) 2001, 2002 Agile Storage Inc.
 *
 * Description:	RAID config manager device state machines.
 *
 * Created by:	Radek Aster
 *
 * Date Created: 4/18/02
 *
 *------------------------------------------------------------------------------
 */

#ifndef __RAID_SM_H__
#define __RAID_SM_H__

typedef enum {
	RAID_EVENT_OPEN_REQ = 0,
	RAID_EVENT_SDM_OPEN_DONE,
	RAID_EVENT_SDM_OPEN_FAILED,
	RAID_EVENT_SDM_OPEN_ALREADY_OPEN,
	RAID_EVENT_SDM_INFO_DONE,
	RAID_EVENT_SDM_INFO_FAILED,
	RAID_EVENT_CLOSE_REQ,
	RAID_EVENT_SDM_CLOSE_DONE,
	RAID_EVENT_SDM_CLOSE_FAILED,
	RAID_EVENT_MESSAGE_REQ,
	RAID_EVENT_RESPONSE_REQ,
	RAID_EVENT_SDM_RELAY_DONE,
	RAID_EVENT_SDM_RELAY_FAILED,
	RAID_EVENT_SDM_RELAY_SCSI_ERROR,
	RAID_EVENT_DEVICE_REMOVED,
	
	RAID_EVENT_COUNT
} raid_dev_event_t;

typedef enum {
	RAID_STATE_CLOSED = 0,
	RAID_STATE_SDM_OPEN_PENDING,
	RAID_STATE_OPEN,
	RAID_STATE_OPEN_ACTIVE,
	RAID_STATE_HOLDING_CLOSE,
	RAID_STATE_SDM_CLOSE_PENDING,
	RAID_STATE_DESTROY_PENDING,
	RAID_STATE_DESTROY_PENDING_AWAITING_CLOSE,

	RAID_STATE_COUNT,

	RAID_STATE_TERMINAL = -2,
	RAID_STATE_INVALID = -1,
	RAID_STATE_INITIAL = RAID_STATE_CLOSED
} raid_dev_state_t;

typedef struct _raid_msg_context {
	raid_msgHdr_t * req_msg;
	char 		req_src_app[32];
	uint32		req_src_cpu;
	uint32		req_src_slot;
	raid_msgHdr_t *	rsp_msg;
	uint32		rsp_msg_len;
	uint32		private;
} raid_msg_context_t;

#define RAID_VENDOR_NAME_LEN	(SDM_VENDOR_NAME_LEN+1)
#define RAID_MODEL_NAME_LEN	(SDM_MODEL_NAME_LEN+1)
#define RAID_DEV_NAME_LEN	(SDM_DEV_NAME_LEN+1)

typedef struct _raid_dev_ {
	uint32		poll_ref_number;
	scsi_id_t	id;
	uint64		wwn;
	uint32		lun;
	char		vendor_name[RAID_VENDOR_NAME_LEN];
	char		model_name[RAID_MODEL_NAME_LEN];
	char		device_name[RAID_DEV_NAME_LEN];

	/* State machine state */
	struct _raid_cmd_transformer_ * dev_cmd_transformer;
	struct _raid_sm_ *		dev_sm;
	raid_dev_state_t		state;
	uint32				open_handle;
	uint32				sdm_close_on_close;
	raid_msg_context_t *		open_req_context;
	raid_msg_context_t *		close_req_context;
	raid_msg_context_t *		message_req_context;
	void *				pending_req_context;

	void *				private_context;

	uint32				rediscOnClose;

	struct _raid_dev_ *		next;
	struct _raid_dev_ *		prev;
} raid_dev_t;

typedef struct _raid_cmd_transformer_ {
	raid_msgTypes_t		req_mid;
	raid_msgTypes_t		rsp_mid;
	int32 (*constructor)(raid_dev_t *, raid_msgHdr_t *, sdm_relayCdbReqMsg_t **, int32 *);
	int32 (*deconstructor)(raid_dev_t *, sdm_relayCdbRspMsg_t *, sdm_relayCdbReqMsg_t **, int32 *, raid_msgHdr_t **, int32 *);
	int32 (*aborter)(raid_dev_t *, sdm_relayCdbRspMsg_t *);
} raid_cmd_transformer_t;

typedef struct _raid_sm_ {
	raid_dev_state_t		current_state;
	struct {
		raid_dev_event_t	event;
		raid_dev_state_t	next_state;
		void (*callout)(raid_dev_t *, void *);
	} events[RAID_EVENT_COUNT];
} raid_sm_t;

void raid_sm_create(raid_dev_t *);
void raid_sm_destroy(raid_dev_t *);
void raid_sm_run(raid_dev_t *, void *, raid_dev_event_t);

raid_msg_context_t *raid_save_context(raid_msgHdr_t *, char *, uint32, uint32);
void raid_free_context(raid_msg_context_t *);
void raid_close_all_devices(void);

#endif /* __RAID_SM_H__ */
