/* vi:ts=4
 *-----------------------------------------------------------------
 *
 * Name:	rcon-api.c
 *
 * RCS:  $Id: rcon-api.c,v 1.7 2002/12/05 06:45:31 tonyw Exp $
 *
 * Copyright (C) 2001, Agile Storage, Inc.
 *
 * Description:	functions for the rcon module.
 *
 * Created by:	eric choi
 *
 * Date Created:	12/20/01
 *
 *-----------------------------------------------------------------
 */

#include <nfx-incl.h>
#include "rcon-api.h"
#include <exports/eee-api.h>
#include <exports/cm-api.h>
#include <exports/malloc-api.h>

#ifdef NFX_SMP
typedef struct _rcon_q {
	ushort16		cnt;
	ushort16		index;
	ushort16		outdex;
	nfx_spinlock_t	lock;
	eee_descPtr_t	msg[RCON_QUEUE_MAX];
} rcon_queue_t;

rcon_queue_t	rcon_queue[RCON_NUM_CPUS];
#endif

uint32	rcon									CPU_PRIVATE;
uchar8	rcon_user_buf[RCON_MAX_USER_LEN]		CPU_PRIVATE;
uchar8	*rcon_user_buf_head						CPU_PRIVATE;
uchar8	*rcon_user_buf_tail						CPU_PRIVATE;
uint32	incoming_appid							CPU_PRIVATE;
uint32	rcon_user_buf_len						CPU_PRIVATE;

/*-----------------------------------------------------------------
 * Name:	rconTx()
 *
 * Description:
 *
 * Created by:	eric choi
 *
 * Date Created:	12/20/01
 *
 *-----------------------------------------------------------------
 */
uint32
rconTx(
	uint32	slot,
	uint32	cpu,
	void	* send_ptr,
	uint32	size
)
{
	uint32 destport;
	uint32 srcport;

	destport = PACK_UNICAST_PORT_ADDRESS(slot, cpu,
                incoming_appid,
                EEE_MGMT, NFX_COS_HIGH, 0);
	srcport = PACK_UNICAST_PORT_ADDRESS(nfxMySlotId, nfxMyCPUId,
                eee_allocateApplicationId("rcon"),
                EEE_MGMT, NFX_COS_HIGH, 0);

	return eee_sendMessage(send_ptr, size, destport, srcport);
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:
 *
 * Date Created:	7/18/02
 *
 *-----------------------------------------------------------------
 */
uint32
rconACK(
	uint32	slot,
	uint32	cpu,
	uint32	appid
)
{
	uint32			srcport, destport;
    rconStatMsg_t   stat_snd;

	stat_snd.hdr.msgType = RCON_STAT;
	stat_snd.hdr.clientPrivate = rcon;
	stat_snd.stat = NFX_OK;
	stat_snd.errMsg[0] = 0xDD;

	destport = PACK_UNICAST_PORT_ADDRESS(slot, cpu,
                appid,
                EEE_MGMT, NFX_COS_HIGH, 0);
	srcport = PACK_UNICAST_PORT_ADDRESS(nfxMySlotId, nfxMyCPUId,
                eee_allocateApplicationId("rcon"),
                EEE_MGMT, NFX_COS_HIGH, 0);

	return eee_sendMessage(&stat_snd, sizeof(rconStatMsg_t), destport, srcport);
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	7/18/02
 *
 *-----------------------------------------------------------------
 */
uint32
rconNACK(
	uint32	slot,
	uint32	cpu,
	uint32	appid
)
{
	uint32			srcport, destport;
    rconStatMsg_t   * stat_snd;
	uint32			msg_len = sizeof(rconStatMsg_t) + 30;
	int32			rc;

	stat_snd = (rconStatMsg_t *)eee_ramAlloc(msg_len);
	bzero(stat_snd, msg_len);

	stat_snd->hdr.msgType = RCON_STAT;
	stat_snd->hdr.clientPrivate = rcon;
	stat_snd->stat = NFX_ERR;
	strcpy(stat_snd->errMsg, "rcon session already in use");

	destport = PACK_UNICAST_PORT_ADDRESS(slot, cpu,
                appid,
                EEE_MGMT, NFX_COS_HIGH, 0);
	srcport = PACK_UNICAST_PORT_ADDRESS(nfxMySlotId, nfxMyCPUId,
                eee_allocateApplicationId("rcon"),
                EEE_MGMT, NFX_COS_HIGH, 0);

	rc = eee_sendMessage(stat_snd, msg_len, destport, srcport);

	eee_ramDealloc(stat_snd);

	return rc;
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:
 *
 * Date Created:	7/18/02
 *
 *-----------------------------------------------------------------
 */
void
rcon_send_start_connection(
	uint32	slot,
	uint32	cpu
)
{
    rconCntlMsg_t   cntl_snd;

	cntl_snd.hdr.msgType = RCON_CNTL;
	cntl_snd.action = INIT_CONNECTION;

	rconTx(slot, cpu, &cntl_snd, sizeof(rconCntlMsg_t));
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:
 *
 * Date Created:	7/18/02
 *
 *-----------------------------------------------------------------
 */
void
rcon_send_end_connection(
	uint32	slot,
	uint32	cpu
)
{
    rconCntlMsg_t   cntl_snd;

	cntl_snd.hdr.msgType = RCON_CNTL;
	cntl_snd.action = DESTROY_CONNECTION;

	rconTx(slot, cpu, &cntl_snd, sizeof(rconCntlMsg_t));
}

/*-----------------------------------------------------------------
 * Name:	rconUserBufService()
 *
 * Description:
 *
 * Created by:	eric choi
 *
 * Date Created:	02/07/02
 *
 *-----------------------------------------------------------------
 */
uint32
rconUserBufService(
	uchar8	* srcp,
	uint32	len
)
{
	uint32	i;

	for (i=0; i<len; i++) {
		*rcon_user_buf_head++ = *srcp++;
		if (rcon_user_buf_head == &rcon_user_buf[RCON_MAX_USER_LEN])
				rcon_user_buf_head = rcon_user_buf;

		rcon_user_buf_len++;
	}

	return 0;
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	1/09/03
 *
 *-----------------------------------------------------------------
 */
int32
rcon_getchar(void)
{
	int32	ch;

	ch = *rcon_user_buf_tail++;
	if (rcon_user_buf_tail == &rcon_user_buf[RCON_MAX_USER_LEN])
		rcon_user_buf_tail = rcon_user_buf;

	rcon_user_buf_len--;

	return ch;
}

/*-----------------------------------------------------------------
 * Name:	rconRcv()
 *
 * Description:
 *
 * Created by:	eric choi
 *
 * Date Created:	12/20/01
 *
 *-----------------------------------------------------------------
 */
void
rconRcv(
	eee_descPtr_t	edesc
)
{
	uchar8 *msg;
	uint32  srcslot;
	uint32  cpuid, appid;

	rconCntlMsg_t *c_msg;
	rconDataMsg_t *d_msg;
	rconStatMsg_t *s_msg;
	
#ifdef NFX_SMP
	cpuid = EEE_GET_DEST_CPU(edesc->hdr.dest_port);	

	if (cpuid != nfxMyCPUId) {
#ifdef NFP_FP
		rcon_queue_t * qptr = &rcon_queue[cpuid - NFX_NFP_FP1];
#else
		rcon_queue_t * qptr = &rcon_queue[cpuid - NFX_NFP_TXRX1];
#endif

		NFX_SPIN_LOCK(&qptr->lock);
		qptr->msg[qptr->index++] = edesc;
		if (qptr->index >= RCON_QUEUE_MAX)
			qptr->index = 0;
		qptr->cnt++;
		NFX_SPIN_UNLOCK(&qptr->lock);

		return;
	}
#endif

	srcslot = EEE_GET_SLOT_NUM(edesc->hdr.src_port);
	cpuid = EEE_GET_DEST_CPU(edesc->hdr.src_port);
	appid = EEE_GET_APP_ID(edesc->hdr.src_port);
	if (!rcon)
		incoming_appid = appid;

	msg = edesc->bd[1].buf;
	c_msg = (rconCntlMsg_t *)msg;
	d_msg = (rconDataMsg_t *)msg;
	s_msg = (rconStatMsg_t *)msg;

	if (c_msg->hdr.msgType == RCONN_MSG_BASE) {
		switch(c_msg->action)
		{
			case(INIT_CONNECTION):
						if (!rcon) {
							/*add to find potential errant multiple sessions*/
							rcon = c_msg->hdr.clientPrivate;
							/* now send back ACK*/
							rconACK(srcslot, cpuid, incoming_appid);
							rcon_user_buf_head = rcon_user_buf;
							rcon_user_buf_tail = rcon_user_buf;
							bzero(rcon_user_buf, RCON_MAX_USER_LEN);
							rcon_user_buf_len = 0;
						}
						else {
							rconNACK(srcslot, cpuid, appid);
						}

						break;

			case(DESTROY_CONNECTION):
						/*cut off all connections*/
						rcon = 0;
						rcon_user_buf_head = rcon_user_buf;
						rcon_user_buf_tail = rcon_user_buf;
						bzero(rcon_user_buf, RCON_MAX_USER_LEN);
						rcon_user_buf_len = 0;
						break;
		}
	}
	else if (c_msg->hdr.msgType == RCON_DATA) {
		/* store in user buffer so scanf can pick it up */
		rconUserBufService(d_msg->buf, d_msg->len);
	}
	else
		printf("\nrconRcv: message type OTHER: %x",*msg);

	eee_freePacket(edesc);
}


/*-----------------------------------------------------------------
 * Name:	rconInit()
 *
 * Description:
 *
 * Created by:	eric choi
 *
 * Date Created:	12/20/01
 *
 *-----------------------------------------------------------------
 */
void
rconInit()
{
#if defined(NFP_TXRX) || defined(FCNIM)
	uint32	* rcon_addr, * incoming_appid_addr;
	extern uint32 * get_rcon_flag_addr();
	extern uint32 * get_incoming_appid_addr();

	rcon_addr = get_rcon_flag_addr();
	if (rcon_addr) {
		rcon = *rcon_addr;
		if (rcon) {
			incoming_appid_addr = get_incoming_appid_addr();
			if (incoming_appid_addr)
				incoming_appid = *incoming_appid_addr;
			else
				rcon = 0;
		}
	}
	else
		rcon = 0;
#else
	rcon = 0;
#endif

#ifdef NFX_SMP
	{
	int i;
	for (i=0; i<RCON_NUM_CPUS; i++) {
		rcon_queue[i].cnt = 0;
		rcon_queue[i].index = 0;
		rcon_queue[i].outdex = 0;
		NFX_SPIN_INIT(&rcon_queue[i].lock, SPINLOCK_LEVEL_UNKNOWN);
	}
	}
#endif

	eee_registerReceiveFunc(eee_allocateApplicationId("rcon"), rconRcv, rconRcv);
}

#ifdef NFX_SMP
/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *		Polled by eee_poll()
 *
 * Created by:
 *
 * Date Created:	1/09/03
 *
 *-----------------------------------------------------------------
 */
void
rconPoll()
{
	rcon_queue_t	* qptr = &rcon_queue[nfxMyCoreId];

	if (qptr->cnt > 0) {
		NFX_SPIN_LOCK(&qptr->lock);
		rconRcv(qptr->msg[qptr->outdex++]);
		if (qptr->outdex >= RCON_QUEUE_MAX)
			qptr->outdex = 0;
		qptr->cnt--;
		NFX_SPIN_UNLOCK(&qptr->lock);
	}
}
#endif	/* NFX_SMP */

