/*
 *-----------------------------------------------------------------
 *
 * Name:	rtc-api.h
 *
 * RCS:  $Id: rtc-api.h,v 1.3 2001/11/20 03:50:58 rick Exp $
 *
 * Copyright (C) 2001, Agile Storage, Inc.
 *
 * Description:	exported API for the Chassis Manager
 *
 * Created by:	ericc
 *
 * Date Created:	09/17/01
 *
 *-----------------------------------------------------------------
 */

/* mapped out registers */

/*
 * registers are accessed as a byte on doubleword boundaries
 */
#if #endian(little)
#define rtcreg(x)	unsigned x:8; unsigned :24; unsigned :32
#else
#define rtcreg(x)	unsigned :32; unsigned :24; unsigned x:8
#endif

typedef struct {
	rtcreg(seconds);
	rtcreg(minutes);
	rtcreg(hours);
	rtcreg(day);
	rtcreg(date);
	rtcreg(month);
	rtcreg(year);
	rtcreg(century);
	rtcreg(alarm_seconds);
	rtcreg(alarm_minutes);
	rtcreg(alarm_hours);
	rtcreg(alarm_day_date);
	rtcreg(watchdog1);
	rtcreg(watchdog2);
	rtcreg(control_a);
	rtcreg(control_b);
	rtcreg(extended_ram_addr);
	rtcreg(dummy0);
	rtcreg(dummy1);
	rtcreg(extended_ram_data);
} ds1501_t;

#define NO_RTC_DATA	-1
#define RTC_BATTERY_LOW	-1

#define RTC_CONTROL_B_OFFSET  0xf

#define RTC_TE_DISABLE	~0x8
#define RTC_TE_ENABLE	0x8

#define RTC_PER_SEC			0
#define	RTC_MATCH_SEC			1
#define	RTC_MATCH_MIN_SEC		2
#define	RTC_MATCH_HOUR_MIN_SEC		3
#define	RTC_MATCH_DATE_HOUR_MIN_SEC	4
#define	RTC_MATCH_DAY_HOUR_MIN_SEC	5

#define	RTC_SECONDS	(0x0*8)
#define	RTC_MINUTES	(0x1*8)
#define	RTC_HOURS	(0x2*8)
#define RTC_DAY		(0x3*8)
#define	RTC_DATE	(0x4*8)
#define	RTC_MONTH	(0x5*8)
#define	RTC_YEAR	(0x6*8)
#define	RTC_CENTURY	(0x7*8)

#define RTC_ALARM_SECONDS	(RTC_SECONDS + (0x8*8))
#define RTC_ALARM_MINUTES	(RTC_MINUTES + (0x8*8))
#define RTC_ALARM_HOURS		(RTC_HOURS   + (0x8*8))
#define RTC_ALARM_DAY		(RTC_DAY     + (0x8*8))
#define RTC_ALARM_DATE		(RTC_DATE    + (0x8*8))
#define RTC_ALARM_MONTHS	(RTC_MONTHS  + (0x8*8))
#define RTC_ALARM_DAY_DATE	()

#define	RTC_SEC_MASK	0x70
#define	RTC_MIN_MASK	0x70
#define RTC_HOUR_MASK	0x30

/* rtc prototypes */
uint32 	rtcGetTime();
void 	rtcDisplayTime();
void 	rtcDisplayDateTime();
uint32 	rtcGetDate();
void	rtcUIGetDateTime();
void	rtcSetTime(uint32 value);
void	rtcSetAlarmBits(uint32 condition);
void	rtcSetTimeItem(uint32 item, uchar8 item_value);
void	rtcRamByteWrite(uchar8 address, uchar8 value);
void	rtcRamBurstWrite(uchar8 address, uchar8 *src, uchar8 burstsize);
void 	rtcRamBurstRead(uchar8 address, uchar8 *dest, uchar8 burstsize);
uchar8	rtcRamByteRead(uchar8 address);
int32	rtcCheckBattery();
void 	rtcSetWatchDog(uchar8 num_seconds, uchar8 num_milli_seconds);
void	rtcDisableWatchDog();
