/*
 *=============================================================================
 * File Name  :	sb1250dm-ui.c
 *
 * Description:    
 *
 * Created    :	07 February 2002
 *
 * Author     :	Frank Salzmann
 *
 * Copyright(c)  Agile Storage, Inc. 2002
 *=============================================================================
 */
#include "sb1250-dm.h"

//
//-- Function Prototypes.
//
int sb1250dm_diagCallBack(char *, int, sb1250dmCfg_t *cfg);

/*
 *------------------------------------------------------------------------------
 * Function Name:	sb1250dm_uiDiag
 *
 * Description: 	SB1250DM UI entry function.
 *
 * Created by:		Frank Salzmann
 *
 * Data Created:	February 07, 2002
 * 					
 * Returns: NFX_OK
 *------------------------------------------------------------------------------
 */
int
sb1250dm_uiDiag()
{
	uint32 chnl;
	chnl = sb1250dm_uiGetInput("Which Channel:[1|2|3|4]: ");

	if( (chnl < 1) || (chnl > 4) )
		return 0;

	sb1250dm_uiDiagMenu( sb1250dmCfg[chnl-1] ); 
	return(NFX_OK);
} /* bmc12500Eth_uiDiag */

/*
 *------------------------------------------------------------------------------
 * Function Name:	sb1250dm_uiShowDiagMenu
 *
 * Description: 	Prints sb1250dm diagnostic menu items.
 *
 * Created by:		Frank Salzmann
 *
 * Data Created:	February 07, 2002
 * 					
 * Returns: NFX_OK
 *
 *------------------------------------------------------------------------------
 */
int
sb1250dm_uiShowDiagMenu()
{
	int sel;
	char cmd[80];

	sel = 0;
	printf("1  : Show Mover Configuration\n");
	printf("2  : Show Mover Regs\n");
	printf("3  : Show Mover DMA Ring\n");
	printf("4  : Open Channel\n");
	printf("5  : Close Channel\n");
	printf("6  : Copy Buffer Test.\n");
	printf("7  : Enable/Disable Tracing\n");
	printf("999: Quit\n");
	printf("Enter Selection: ");
#ifdef LINUX_TEST
	fflush(0);
#endif
	memset(cmd, 0, 80);
	get_cmd(cmd);
	sscanf(cmd, "%d", &sel);
	printf("\n");
	return sel;
} // sb1250dm_uiShowDiagMenu 

/*
 *------------------------------------------------------------------------------
 * Function Name:	sb1250dm_uiGetInput	
 *
 * Description: 	Gets console input.
 *
 * Created by:		Frank Salzmann
 *
 * Data Created:	May 09, 2001
 * 					
 * Returns: NFX_OK
 *
 *------------------------------------------------------------------------------
 */
int
sb1250dm_uiGetInput( char *msg )
{
	int sel;
	char cmd[80];

	printf("%s: ", msg);
#ifdef LINUX_TEST
	fflush(0);
#else
	memset(cmd, 0, 80);
	get_cmd(cmd);
	sscanf(cmd, "%d", &sel);
#endif
	return sel; 
} // sb1250dm_uiGetInput 

/*
 *------------------------------------------------------------------------------
 * Function Name:	sb1250dm_uiDiagMenu	
 *
 * Description: 	Runs the SB1250DM diagnostic menu.
 *
 * Created by:		Frank Salzmann
 *
 * Data Created:	February 07, 2002
 * 					
 * Returns: NFX_OK
 *------------------------------------------------------------------------------
 */
int
sb1250dm_uiDiagMenu(sb1250dmCfg_t *cfg)
{
	int sel ;
	for(;;) {
		printf("SB1250DM: Channel: %d\n", cfg->chnlId+1);
		printf("=====================\n");
		sel = sb1250dm_uiShowDiagMenu();
		switch(sel)
		{
			case 1:
				sb1250dm_uiViewCfg(cfg);
				break;
			case 2:
				sb1250dm_uiShowRegs(cfg);
				break;
			case 3:
				sb1250dm_uiShowQ(cfg);
				break;
			case 4:
				if( cfg->inUse == 0 ) {
					sb1250dm_openChnl( cfg->chnlId+1, sb1250dm_diagCallBack );	
					printf("SB1250DM: Channel %d opened.\n\n", cfg->chnlId+1);
				}
				else {
					printf("Channel %d already open.\n\n", cfg->chnlId+1);
				}
				break;
			case 5:
				sb1250dm_closeChnl(cfg->chnlId+1);
				printf("SB1250DM: Channel %d closed.\n\n", cfg->chnlId+1);
				break;
			case 6:
				sb1250dm_uiCopyBuf(cfg);
				break;
			case 7:
				sb1250dm_uiSetTrace(cfg);
				break;
			case 999:
				return 0;
		}
	} 
	return(NFX_OK);
} // sb1250dm_uiDiagMenu 

/*
 *------------------------------------------------------------------------------
 * Function Name:	sb1250dm_uiShowAllRegs
 *
 * Description: 	Prints all of the chip registers.
 *
 * Created by:		Frank Salzmann
 *
 * Data Created:	February 07, 2002
 * 					
 * Returns: NFX_OK
 *
 *------------------------------------------------------------------------------
 */
int
sb1250dm_uiShowRegs( sb1250dmCfg_t *cfg )
{
	int				i;
	sb1250dm_regs_t *r;

	r = &cfg->regs;
	printf("SB1250DM REGISTER DUMP: Channel: %d\n", cfg->chnlId+1);
	printf("========================================================== \n\n");
	printf("DM DESC BASE     :%016llx\n", sb1250dm_readReg(r->dm_desc_base));
	printf("DM DESC COUNT    :%016llx\n", sb1250dm_readReg(r->dm_desc_count));
	printf("DM CUR DESC ADDR :%016llx\n", sb1250dm_readReg(r->dm_cur_desc_addr));
	printf("DM DBG DESC BASE :%016llx\n", sb1250dm_readReg(r->dm_dbg_desc_base));

	return(NFX_OK);
} // sb1250dm_uiShowAllRegs

/*
 *------------------------------------------------------------------------------
 * Function Name:	sb1250dm_uiViewCfg
 *
 * Description: 	Display device/driver configuration.
 *
 * Created by:		Frank Salzmann
 *
 * Data Created:	February 07, 2002
 * 					
 * Returns: NFX_OK
 *------------------------------------------------------------------------------
 */
int
sb1250dm_uiViewCfg(sb1250dmCfg_t *cfg)
{
	uint32 i;

	printf("SB1250 DATA MOVER CONFIGURATION: \n");
	printf("=============== General Config ============== \n\n");
	printf("Channel Id           : %08d\n", cfg->chnlId+1);
	printf("Channel In Use       : %08d\n", cfg->inUse);
	printf("CallBack Ptr         : %p\n", cfg->callback);
	printf("\n=============== Queue Control ============== \n\n");
	printf("Start Addr           : %p\n", cfg->dmqc.qStart);
	printf("End Addr             : %p\n", cfg->dmqc.qEnd);
	printf("Current Addr         : %p\n", cfg->dmqc.qtxCur);
	printf("Cleanup Addr         : %p\n", cfg->dmqc.qclnCur);
	printf("\n=============== Statistics ============== \n\n");
	printf("Buffer Copy Cnt      : %08d\n", cfg->dmqc.bufCnt);
	printf("Byte   Copy Cnt      : %08d\n", cfg->dmqc.byteCnt);
	printf("Q Full Cnt           : %08d\n", cfg->dmqc.qFullCnt);
	printf("Desc Q'd             : %08d\n", cfg->dmqc.qd);
	printf("\n");
	return(NFX_OK);
} // sb1250dm_uiViewCfg 


/*
 *------------------------------------------------------------------------------
 * Function Name:	sb1250dm_uiShowQ
 *
 * Description: 	Dump the chip transmit descriptor queue.
 *
 * Created by:		Frank Salzmann
 *
 * Data Created:	February 07, 2002
 * 					
 * Returns: NFX_OK
 *------------------------------------------------------------------------------
 */
int
sb1250dm_uiShowQ( sb1250dmCfg_t *dp)
{
	int i, quit;
	volatile sb1250dm_desc_t *descp;

	descp = dp->dmqc.qStart;
	if( !descp ) {
		printf("Channel not open. DMA Ring not present.\n");
		return 0;
	}

	printf("SB1250 Data Mover Descriptor Queue \n");
	printf("===================================\n");
	for(i=0; i < SB1250_DM_Q_LEN; i++) {
		printf("%p (%03d)   Word_1: %016llx  Word_2: %016llx\n", descp, i, descp->dw1, descp->dw2 );
		descp++;
		if( !(i % 25) && i != 0 ) {
			quit = sb1250dm_uiGetInput("Quit [99]: ");
			if( quit == 99 )
				return 0;
		}
	}
	printf("\n");
	return 0;
} // sb1250dm_uiShowQ 

/*
 *-----------------------------------------------------------------------------
 * Function Name:   sb1250dm_uiSetTrace
 *
 * Description:     Enables/Disables packet tracing in sb1250dm driver.
 *
 * Created by:      Frank Salzmann
 *
 * Data Created:    February 08, 2002
 *
 * Returns: NFX_OK
 *-----------------------------------------------------------------------------
 */
int
sb1250dm_uiSetTrace(sb1250dmCfg_t *dp)
{
	uint32        sel;
	extern uint32 sb1250dm_trace;

	printf("DM Trace: %s\n", (sb1250dm_trace == 0) ? "DISABLED" : "ENABLED");
    sel = sb1250dm_uiGetInput("Trace Dis/Ena [0|1]: ");
	switch(sel) {
	  
		case 0:
			sb1250dm_trace = 0;
			break;
		case 1:
			sb1250dm_trace = 1;
			break;
		default: 
			printf("Invalid Entry: %d\n", sel);
	}
	printf("DM Trace: %s\n", (sb1250dm_trace == 0) ? "DISABLED" : "ENABLED");
    printf("\n");
    return 0;
} // sb1250dm_uiSetTrace

#ifndef PROM
#define SB_DIAG_BUF_LEN		256
#else
#define SB_DIAG_BUF_LEN		0x40000
#endif
/*
 *-----------------------------------------------------------------------------
 * Function Name:   sb1250dm_uiCopyBuf
 *
 * Description:     Enables/Disables packet tracing in sb1250dm driver.
 *
 * Created by:      Frank Salzmann
 *
 * Data Created:    February 08, 2002
 *
 * Returns: NFX_OK
 *-----------------------------------------------------------------------------
 */
int
sb1250dm_uiCopyBuf( sb1250dmCfg_t *cfg )
{
	uint32 rc;
	uint32 i;
	uint32 chnlId  = cfg->chnlId+1;
	sb1250dm_test_t	* tparam = &(cfg->test);
	uint32	sel;


	if( cfg->inUse == 0 ) {
		printf("SB1250DM: Channel %d not open.\n", chnlId);
		return SB1250DM_OK;
	}

	if (tparam->running == 0) {

	    sel = sb1250dm_uiGetInput("Num Loops: ");

		tparam->byteCnt = SB_DIAG_BUF_LEN;
		tparam->numLoopsDone = 0;
		tparam->numLoopsToDo = sel;
		tparam->totalTime = *(eee_rtcZero());
	}

	tparam->srcAddr = eee_ramAlloc(tparam->byteCnt);
#ifndef PROM
	tparam->destAddr = eee_allocateBuffer(EEE_SHARED_POOL,
												EEE_BUF_LARGE, NFX_COS_HIGH);
#else
	if (isTXRX())
		tparam->destAddr = (uchar8 *)(0x40000000+((chnlId-1)*SB_DIAG_BUF_LEN));
	else
		tparam->destAddr = (uchar8 *)(0x20000000+((chnlId-1)*SB_DIAG_BUF_LEN));
#endif

	if (tparam->running == 0) {
		printf("Src: %p, Dst: %p, Len: %d\n",	tparam->srcAddr,
												tparam->destAddr,
												tparam->byteCnt);

		//-- Initailize source buffer.
		for(i=0; i < tparam->byteCnt; i++)
			tparam->srcAddr[i] = i & 0xff;

		//-- Initialize destination buffer.
		for(i=0; i < tparam->byteCnt; i++)
			tparam->destAddr[i] = 1;

		tparam->running = 1;
	}

	eee_rtc(&(tparam->startTime));

	//-- Call data mover.
	rc = sb1250dm_movData_buf(	chnlId,
								tparam->byteCnt,
								tparam->srcAddr,
								tparam->destAddr,
								cfg);

	if( rc == SB1250DM_ERR ) {
		if( tparam->srcAddr ) 
			eee_ramDealloc(tparam->srcAddr);
#ifndef PROM
		if( tparam->destAddr ) {
			eee_deallocateBuffer(tparam->destAddr);
		}
#endif
	}

	return(NFX_OK);
} // sb1250dm_uiCopyBuf

static void
dm_printResult(
    char    *callerStr,
    uint64  numBytes,
    uint64  numIter,
    eeeRTC_t  time)
{
	uint64 usecs = eee_rtcToUsec(&time);

	printf("\n%s: %lld bytes transferred in %lld usecs\n",
           callerStr, numBytes * numIter, usecs);

    printf("    %lld MegaBytes/sec throughput ",
            numBytes * numIter / usecs);

    if (usecs < 1000000)
    {
        printf("(elapsed time < 1 sec, result unreliable)");
    }
    printf("\n");
}

int
sb1250dm_diagCallBack( char *buf, int32 len, sb1250dmCfg_t * cfg )
{

	sb1250dm_test_t *	tparam;
	eeeRTC_t			delta;

	tparam = &(cfg->test);

	eee_rtcDuration(&(tparam->startTime), &delta);
	eee_rtcIncr(&(tparam->totalTime), &delta);

//	printf("sb1250dm_diagCallBack: Buf: %p, len = 0x%x\n", buf, len);

	if( tparam->srcAddr) {
/*
		bmc12500Eth_printBuf( tparam->srcAddr, tparam->byteCnt, 0 );
*/
		eee_ramDealloc(tparam->srcAddr);
	}

	if( tparam->destAddr ) {
/*
		bmc12500Eth_printBuf( tparam->srcAddr, tparam->byteCnt, 0 );
*/
#ifndef PROM
		eee_deallocateBuffer(tparam->destAddr);
#endif
	}

	tparam->numLoopsDone++;

	/*
	 * Are we done?  If so, calculate throughput and display
	 */
	if (tparam->numLoopsDone == tparam->numLoopsToDo) {
		if (cfg->chnlId == 0)
			dm_printResult("DataMover 1",
							tparam->byteCnt,
							tparam->numLoopsDone,
							tparam->totalTime);
		else if (cfg->chnlId == 1)
			dm_printResult("DataMover 2",
							tparam->byteCnt,
							tparam->numLoopsDone,
							tparam->totalTime);
		else if (cfg->chnlId == 2)
			dm_printResult("DataMover 3",
							tparam->byteCnt,
							tparam->numLoopsDone,
							tparam->totalTime);
		else if (cfg->chnlId == 3)
			dm_printResult("DataMover 4",
							tparam->byteCnt,
							tparam->numLoopsDone,
							tparam->totalTime);
		tparam->running = 0;
	}
	else
		sb1250dm_uiCopyBuf(cfg);

	return NFX_OK;
} //sb1250dm_diagCallBack
