/*  *********************************************************************
    *  SB1250 Board Support Package
    *  
    *  Pass2 Enhancements			File: sb1250_pass2.h
    *  
    *  this file contains the enhancements made to the SB1250 in
    *  pass2 of the silicon.  It is intended to be included
    *  *after* other sb1250-related include files.  In some cases
    *  it may undefine and replace certain symbols.
    *
    *  someday, this functionality will be folded into the standard
    *  include files and this file will be eliminated or reduced to
    *  a dummy.
    *
    *  Author:  Mitch Lichtenberg (mitch@sibyte.com)
    *  
    *********************************************************************  
    *
    *  Copyright 2000,2001
    *  Broadcom Corporation. All rights reserved.
    *  
    *  This software is furnished under license and may be used and 
    *  copied only in accordance with the following terms and 
    *  conditions.  Subject to these conditions, you may download, 
    *  copy, install, use, modify and distribute modified or unmodified 
    *  copies of this software in source and/or binary form.  No title 
    *  or ownership is transferred hereby.
    *  
    *  1) Any source code used, modified or distributed must reproduce 
    *     and retain this copyright notice and list of conditions as 
    *     they appear in the source file.
    *  
    *  2) No right is granted to use any trade name, trademark, or 
    *     logo of Broadcom Corporation. Neither the "Broadcom 
    *     Corporation" name nor any trademark or logo of Broadcom 
    *     Corporation may be used to endorse or promote products 
    *     derived from this software without the prior written 
    *     permission of Broadcom Corporation.
    *  
    *  3) THIS SOFTWARE IS PROVIDED "AS-IS" AND ANY EXPRESS OR
    *     IMPLIED WARRANTIES, INCLUDING BUT NOT LIMITED TO, ANY IMPLIED 
    *     WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR 
    *     PURPOSE, OR NON-INFRINGEMENT ARE DISCLAIMED. IN NO EVENT 
    *     SHALL BROADCOM BE LIABLE FOR ANY DAMAGES WHATSOEVER, AND IN 
    *     PARTICULAR, BROADCOM SHALL NOT BE LIABLE FOR DIRECT, INDIRECT, 
    *     INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
    *     (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE 
    *     GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
    *     BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY 
    *     OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR 
    *     TORT (INCLUDING NEGLIGENCE OR OTHERWISE), EVEN IF ADVISED OF 
    *     THE POSSIBILITY OF SUCH DAMAGE.
    ********************************************************************* */



#ifndef _SB1250_PASS2_H
#define _SB1250_PASS2_H


/*  *********************************************************************
    *  System Control and Debug Unit (sb1250_scd.h)
    ********************************************************************* */

/* 
 * SYSTEM_CONFIG register
 */

#define M_SYS_SW_FLAG		  _SB_MAKEMASK1(63)


/*  *********************************************************************
    *  Interrupt Sources (sb1250_int.h)
    ********************************************************************* */

#undef K_INT_SPARE_0
#undef K_INT_SPARE_1
#undef K_INT_SPARE_3
#undef K_INT_SPARE_4
#undef K_INT_SPARE_5

#define K_INT_CYCLE_CP0_INT	    30
#define K_INT_CYCLE_CP1_INT	    31
#define K_INT_MAC_0_CH1		    61
#define K_INT_MAC_1_CH1		    62
#define K_INT_MAC_2_CH1		    63

#undef M_INT_SPARE_0
#undef M_INT_SPARE_1
#undef M_INT_SPARE_3
#undef M_INT_SPARE_4
#undef M_INT_SPARE_5

#define M_INT_CYCLE_CP0_INT	_SB_MAKEMASK1(K_INT_CYCLE_CP0_INT)
#define M_INT_CYCLE_CP1_INT	_SB_MAKEMASK1(K_INT_CYCLE_CP1_INT)
#define M_INT_MAC_0_CH1		_SB_MAKEMASK1(K_INT_MAC_0_CH1)
#define M_INT_MAC_1_CH1		_SB_MAKEMASK1(K_INT_MAC_1_CH1)
#define M_INT_MAC_2_CH1		_SB_MAKEMASK1(K_INT_MAC_2_CH1)



/*  *********************************************************************
    *  DMA (sb1250_dma.h)
    ********************************************************************* */

#define M_DMA_ETHRX_BADTCPCS	_SB_MAKEMASK1(0)

#define M_DM_DSCRA_RD_BKOFF	_SB_MAKEMASK1(52)
#define M_DM_DSCRA_WR_BKOFF	_SB_MAKEMASK1(53)

#undef M_DM_DSCRA_THROTTLE		/* removed in pass 2 */
#undef M_DM_DSCRA_RESERVED2
#define M_DM_DSCRA_RESERVED2        _SB_MAKEMASK(10,54)


/*  *********************************************************************
    *  MAC (sb1250_mac.h)
    ********************************************************************* */

/*
 * MAC Config Register
 */

#define M_MAC_RX_CH_SEL		    _SB_MAKEMASK1(44)

/*
 * MAC Threshhold register
 */

#undef M_MAC_TX_WR_THRSH
#define M_MAC_TX_WR_THRSH           _SB_MAKEMASK(7,S_MAC_TX_WR_THRSH)

#undef M_MAC_TX_RD_THRSH
#define M_MAC_TX_RD_THRSH           _SB_MAKEMASK(7,S_MAC_TX_RD_THRSH)

#define S_MAC_ENC_FC_THRSH           _SB_MAKE64(56)
#define M_MAC_ENC_FC_THRSH           _SB_MAKEMASK(6,S_MAC_ENC_FC_THRSH)
#define V_MAC_ENC_FC_THRSH(x)        _SB_MAKEVALUE(x,S_MAC_ENC_FC_THRSH)
#define G_MAC_ENC_FC_THRSH(x)        _SB_GETVALUE(x,S_MAC_ENC_FC_THRSH,M_MAC_ENC_FC_THRSH)

/*
 * MAC INterrupt mask register 
 */

#define M_MAC_SPLIT_EN		     _SB_MAKEMASK1(47) 	/* interrupt mask only */

/*
 * MAC Receive Address Filter 
 */

#define M_MAC_ALLMCAST_EN	     _SB_MAKEMASK1(7)




/*  *********************************************************************
    *  PCI (sb1250_pci.h)
    ********************************************************************* */

#define R_PCI_TYPE0_READHOST	0x0094	/* Read Host register */
#define R_PCI_TYPE0_ADXTEND	0x0098	/* Adaptive Extend register */

#define M_PCI_READHOST_RDHOST	_SB_MAKEMASK1_32(0)

#define S_PCI_ADXTEND_NOM_TAR_RETRY	1
#define M_PCI_ADXTEND_NOM_TAR_RETRY	_SB_MAKEMASK_32(3,S_PCI_ADXTEND_NOM_TAR_RETRY)
#define V_PCI_ADXTEND_NOM_TAR_RETRY(x)	_SB_MAKEVALUE_32(x,S_PCI_ADXTEND_NOM_TAR_RETRY)
#define G_PCI_ADXTEND_NOM_TAR_RETRY(x)	_SB_GETVALUE_32(x,S_PCI_ADXTEND_NOM_TAR_RETRY,M_PCI_ADXTEND_NOM_TAR_RETRY)

#define S_PCI_ADXTEND_MAX_TAR_RETRY	4
#define M_PCI_ADXTEND_MAX_TAR_RETRY	_SB_MAKEMASK_32(2,S_PCI_ADXTEND_MAX_TAR_RETRY)
#define V_PCI_ADXTEND_MAX_TAR_RETRY(x)	_SB_MAKEVALUE_32(x,S_PCI_ADXTEND_MAX_TAR_RETRY)
#define G_PCI_ADXTEND_MAX_TAR_RETRY(x)	_SB_GETVALUE_32(x,S_PCI_ADXTEND_MAX_TAR_RETRY,M_PCI_ADXTEND_MAX_TAR_RETRY)

#define M_PCI_ADXTEND_DIS_DMAR_IOW_DEP  _SB_MAKEMASK1_32(6)
#define M_PCI_ADXTEND_DIS_MEMRD_BE      _SB_MAKEMASK1_32(6)


/*  *********************************************************************
    *  LDT (sb1250_ldt.h)
    ********************************************************************* */

#define R_LDT_TYPE1_ADDSTATUS	0x0070

#define M_LDT_STATUS_VGAEN		_SB_MAKEMASK1_32(3)

#undef M_LDT_SRICMD_DISSTARVATIONCNT
#define M_LDT_SRICMD_DISMULTTXVLD	_SB_MAKEMASK1_32(19)
#define M_LDT_SRICMD_EXPENDIAN		_SB_MAKEMASK1_32(26)

#define S_LDT_ADDSTATUS_TGTDONE		0
#define M_LDT_ADDSTATUS_TGTDONE		_SB_MAKEMASK_32(8,S_LDT_ADDSTATUS_TGTDONE)
#define V_LDT_ADDSTATUS_TGTDONE(x)	_SB_MAKEVALUE_32(x,S_LDT_ADDSTATUS_TGTDONE)
#define G_LDT_ADDSTATUS_TGTDONE(x)	_SB_GETVALUE_32(x,S_LDT_ADDSTATUS_TGTDONE,M_LDT_ADDSTATUS_TGTDONE)

/*  *********************************************************************
    *  DUART (sb1250_uart.h)
    ********************************************************************* */

/* 
 * Full Interrupt Control Register 
 */

#define S_DUART_SIG_FULL           _SB_MAKE64(0)
#define M_DUART_SIG_FULL           _SB_MAKEMASK(4,S_DUART_SIG_FULL)
#define V_DUART_SIG_FULL(x)        _SB_MAKEVALUE(x,S_DUART_SIG_FULL)
#define G_DUART_SIG_FULL(x)        _SB_GETVALUE(x,S_DUART_SIG_FULL,M_DUART_SIG_FULL)

#define S_DUART_INT_TIME           _SB_MAKE64(4)
#define M_DUART_INT_TIME           _SB_MAKEMASK(4,S_DUART_INT_TIME)
#define V_DUART_INT_TIME(x)        _SB_MAKEVALUE(x,S_DUART_INT_TIME)
#define G_DUART_INT_TIME(x)        _SB_GETVALUE(x,S_DUART_INT_TIME,M_DUART_INT_TIME)

/*  *********************************************************************
    *  Generic Bus (sb1250_genbus.h)
    ********************************************************************* */

/* 
 * IO_EXT_CFG 
 */

#define K_IO_WIDTH_SEL_1L	   2
#define M_IO_BURST_EN		   _SB_MAKEMASK1(5)

/* 
 * IO_EXT_TIME_CFG0
 */

#define M_IO_EARLY_CS	           _SB_MAKEMASK1(3)

#define S_IO_BURST_WIDTH           _SB_MAKE64(6)
#define M_IO_BURST_WIDTH           _SB_MAKEMASK(2,S_IO_BURST_WIDTH)
#define V_IO_BURST_WIDTH(x)        _SB_MAKEVALUE(x,S_IO_BURST_WIDTH)
#define G_IO_BURST_WIDTH(x)        _SB_GETVALUE(x,S_IO_BURST_WIDTH,M_IO_BURST_WIDTH)

/* 
 * IO_EXT_TIME_CFG1
 */

#define M_IO_RDY_SYNC		   _SB_MAKEMASK1(3)


/* 
 * IO_INTERRUPT_STATUS 
 */

#define M_IO_COH_ERR		   _SB_MAKEMASK1(14)

/*  *********************************************************************
    *  SMBus (sb1250_smbus.h)
    ********************************************************************* */

#define S_SMB_CMDH                  8
#define M_SMB_CMDH                  _SB_MAKEMASK(8,S_SMBH_CMD)
#define V_SMB_CMDH(x)               _SB_MAKEVALUE(x,S_SMBH_CMD)

#define M_SMB_EXTEND		    _SB_MAKEMASK1(14)

#define M_SMB_DIR		    _SB_MAKEMASK1(13)

#define S_SMB_DFMT                    8
#define M_SMB_DFMT                    _SB_MAKEMASK(3,S_SMB_DFMT)
#define V_SMB_DFMT(x)                 _SB_MAKEVALUE(x,S_SMB_DFMT)
#define G_SMB_DFMT(x)                 _SB_GETVALUE(x,S_SMB_DFMT,M_SMB_DFMT)

#define K_SMB_DFMT_1BYTE            0
#define K_SMB_DFMT_2BYTE            1
#define K_SMB_DFMT_3BYTE            2
#define K_SMB_DFMT_4BYTE            3
#define K_SMB_DFMT_NODATA           4
#define K_SMB_DFMT_CMD4BYTE         5
#define K_SMB_DFMT_CMD5BYTE         6
#define K_SMB_DFMT_RESERVED         7

#define V_SMB_DFMT_1BYTE	    V_SMB_DFMT(K_SMB_DFMT_1BYTE)
#define V_SMB_DFMT_2BYTE	    V_SMB_DFMT(K_SMB_DFMT_2BYTE)
#define V_SMB_DFMT_3BYTE	    V_SMB_DFMT(K_SMB_DFMT_3BYTE)
#define V_SMB_DFMT_4BYTE	    V_SMB_DFMT(K_SMB_DFMT_4BYTE)
#define V_SMB_DFMT_NODATA	    V_SMB_DFMT(K_SMB_DFMT_NODATA)
#define V_SMB_DFMT_CMD4BYTE	    V_SMB_DFMT(K_SMB_DFMT_CMD4BYTE)
#define V_SMB_DFMT_CMD5BYTE	    V_SMB_DFMT(K_SMB_DFMT_CMD5BYTE)
#define V_SMB_DFMT_RESERVED	    V_SMB_DFMT(K_SMB_DFMT_RESERVED)


/*  *********************************************************************
    *  Register Definitions (sb1250_regs.h)
    ********************************************************************* */

/*
 * System Control and Debug Unit (sb1250_scd.h)
 */

#define A_SCD_SCRATCH		   0x0010020C10

#define A_SCD_ZBBUS_CYCLE_COUNT	   0x0010030000
#define A_SCD_ZBBUS_CYCLE_CP0	   0x0010020C00
#define A_SCD_ZBBUS_CYCLE_CP1	   0x0010020C08

/*
 * System Bus Watcher (sb1250_scd.h)
 */

#define A_SCD_BUS_ERR_STATUS_DEBUG 0x00100208D0

/*
 * Address Trap (sb1250_scd.h)
 */

#define A_ADDR_TRAP_REG_DEBUG	   0x0010020460

/*
 * Trace unit (sb1250_scd.h)
 */

#define M_SCD_TRACE_CFG_FORCECNT   _SB_MAKEMASK1(8)

/*
 * L2 Cache Control Registers (sb1250_l2c.h)
 */

#define A_L2_CACHE_DISABLE	   0x0010042000
#define A_L2_MAKECACHEDISABLE(x)   (A_L2_CACHE_DISABLE | (((x)&0x0F) << 8))
#define A_L2_MISC_CONFIG	   0x0010043000

/*
 * MAC (sb1250_mac.h)
 */

#define R_MAC_STATUS1		   0x00000430

/*
 * DUART (sb1250_uart.h)
 */

#define A_DUART_FULL_CTL_A	   0x0010060140
#define A_DUART_FULL_CTL_B	   0x0010060240

#define A_DUART_OPCR_A	  	   0x0010060180
#define A_DUART_OPCR_B	  	   0x0010060280

#define A_DUART_INPORT_CHNG_DEBUG  0x00100603F0

#define R_DUART_FULL_CTL	   0x140
#define R_DUART_OPCR		   0x180
#define R_DUART_AUXCTL		   0x190

#define M_DUART_CTS_CHNG_ENA        _SB_MAKEMASK1(0)
#define M_DUART_CIN_CHNG_ENA        _SB_MAKEMASK1(2)


#endif
