/*
 *-----------------------------------------------------------------
 *
 * Name:
 *
 * RCS:  $Id: smp.c,v 1.9 2002/12/04 01:49:41 rick Exp $
 *
 * Copyright (C) 2002, Agile Storage, Inc.
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	11/07/02
 *
 *-----------------------------------------------------------------
 */

#include <nfx-incl.h>
#include <sys/mips.h>
#include <exports/cm-api.h>
#include <exports/smp-api.h>
#include <exports/except-api.h>
#include <exports/bsp-api.h>

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	11/07/02
 *
 *-----------------------------------------------------------------
 */
int32
sb1250_getCoreId()
{
	return ((_mips_mfc0(C0_PRID) >> 25) & 0x7);
}

#ifdef NFX_SMP

smp_rfunc_t smp_remote_func_struct[SMP_NUM_CPUS];

extern void prof_init(void);
extern void nfs_init();
extern void mnt_init();
extern void nsm_init();
extern void nlm_init();
/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	11/07/02
 *
 *-----------------------------------------------------------------
 */
void
smp_entry(
	uint64	a0,
	uint64	cpu_num,
	uint64	a2,
	uint64	a3
)
{
	uint32	sr;

	extern void mem_map_tlb_init();
	extern void panic_prompt();
	extern void runtime_prompt();
	extern void crashdump_init();
	extern void histinit();
	extern address_t	stack_addr;

	sr = 0x20000fa1;

	_mips_mtc0(C0_SR, sr);
	_mips_mtc0(C0_CAUSE, CAUSE_IV);
	mem_map_tlb_init(cpu_num);

	stack_addr = ((address_t)&smp_stack[cpu_num][0]);

	nfxMyCoreId = cpu_num;

	histinit();

#ifdef NFP_FP
	nfxMyCPUId = cpu_num + NFX_NFP_FP1;
#elif NFP_TXRX
	nfxMyCPUId = cpu_num + NFX_NFP_TXRX1;

	/* initialize application */
		nfs_init();
		mnt_init();
        nsm_init();
        nlm_init();
#endif

	{
        uint32  seed = _mips_mfc0(C0_COUNT);
        printf("random seed = 0x%x\n", seed);
        srandom(seed);
    }

#ifdef NFX_PROFILING
	prof_init();
	*(volatile uint64 *)PHYS_TO_K1(0x10022028) &= ~0x8;
	*(volatile uint64 *)PHYS_TO_K1(0x10022218) = 3;
	sr = _mips_mfc0(C0_SR);
	sr |= SR_IM5;
	_mips_mtc0(C0_SR,sr);
#endif
		
	crashdump_init();

	cmSetCPUState(CM_SLICE_STATE_RUNTIME);
	cmSendVersionResp(0, 0);

#if 0
	panic_prompt();
#else	
	watchdog_init();
	runtime_prompt();
#endif	
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	11/12/02
 *
 *-----------------------------------------------------------------
 */
void
smp_remote_function_call(
	uint32	which_core,
	uint32	num_args,
	char8	* func,
	...
)
{
	uint32	i;
	va_list	ap;
	int32	arg_array[4];
	smp_rfunc_t	* smp_mbox_msg_ptr;

	if (num_args > 4)
		panic("Invalid number of args to remote function call, %d\n", num_args);

	va_start(ap, func);

	printf("remote_function_call(%d, %d, %p", which_core, num_args, func);
	for (i=0; i<num_args; i++) {
		arg_array[i] = va_arg(ap, int);
		printf(" 0x%x", arg_array[i]);
	}
	printf("\n");

	va_end(ap);

	smp_mbox_msg_ptr = &smp_remote_func_struct[which_core];

	smp_mbox_msg_ptr->func = (address_t)func;
	smp_mbox_msg_ptr->arg0 = arg_array[0];
	smp_mbox_msg_ptr->arg1 = arg_array[1];
	smp_mbox_msg_ptr->arg2 = arg_array[2];
	smp_mbox_msg_ptr->arg3 = arg_array[3];

	sb1250_mbox_poke(which_core, (void *)smp_mbox_msg_ptr);
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	11/12/02
 *
 *-----------------------------------------------------------------
 */
smp_rfunc_t	*
smp_remote_function_poll(
	uint32	which_core
)
{
	smp_rfunc_t	* msgp;
	uint32		i;

	for (i=0; i<0x10000000; i++) {
		if ((msgp = sb1250_mbox_peek(which_core))) {
			if (which_core == nfxMyCoreId)
				sb1250_mbox_clear(which_core);
			break;
		}
	}

	if (!msgp)
		printf("Failure in smp_remote_function_poll(%d)\n", which_core);

	return msgp;
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	11/12/02
 *
 *-----------------------------------------------------------------
 */
int32
smp_remote_function_wait_to_clear(
	uint32	which_core
)
{
	uint32	i;

	for (i=0; i<0x10000000; i++) {
		if (!sb1250_mbox_peek(which_core))
			return 0;
	}
	return 1;

}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	11/12/02
 *
 *-----------------------------------------------------------------
 */
void
smp_remote_function_clear(
	uint32	which_core
)
{
	sb1250_mbox_clear(which_core);
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	11/12/02
 *
 *-----------------------------------------------------------------
 */
void
counttest_int()
{
	uint32	count = _mips_mfc0(C0_COUNT);
	printf("count = %u\n", count);
	timer_int_clear(2);
//	mips_wbflush();
}

int32
counttest(
	int32	argc,
	uchar8	* argv[]
)
{
	uint32	sr;

	if (nfxMyCPUId == NFX_NFP_FP1) {
		timer_start_us(2, 2000000, counttest_int);
	}
	else if (nfxMyCPUId == NFX_NFP_FP2) {
		*(volatile uint64 *)PHYS_TO_K1(0x10022028) &= ~0x10;
		*(volatile uint64 *)PHYS_TO_K1(0x10022220) = 3;
		sr = _mips_mfc0(C0_SR);
		sr |= SR_IM5;
		_mips_mtc0(C0_SR,sr);
	}

	return 0;
}

#endif	/* NFX_SMP */

