/*
 *-----------------------------------------------------------------
 *
 * Name:
 *
 * RCS:  $Id: timer.c,v 1.9 2002/11/26 22:23:10 rick Exp $
 *
 * Copyright (C) 2002, Agile Storage, Inc.
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	2/25/02
 *
 *-----------------------------------------------------------------
 */

#include <nfx-incl.h>
#include <sys/mips.h>
#include <sys/sb1250/sb1250_regs.h>
#include <sys/sb1250/sb1250_int.h>
#include <exports/except-api.h>

volatile uint64	timer_refSecond;

/*-----------------------------------------------------------------
 * Name:	timer_us
 *
 * Description:	
 *
 *				Parameter is in microseconds.
 *
 *				Uses GP timer 0.
 *
 * Created by:	Rick Lund
 *
 * Date Created:	10/30/01
 *
 *-----------------------------------------------------------------
 */
void
timer_start_us(
	uint32	timer_num,
	uint32	us,
	void	(* int_func)(uint32, uint32, address_t)
)
{
	volatile sb1250_gptimer_t	* tptr;
	volatile uint64	* imask = (volatile uint64 *)
							PHYS_TO_K1(A_IMR_CPU0_BASE+ R_IMR_INTERRUPT_MASK);
	volatile uint64	* imap = (volatile uint64 *)
						PHYS_TO_K1(A_IMR_CPU0_BASE+ R_IMR_INTERRUPT_MAP_BASE);
	uint32	use_int = 0;

	if (us > 8 * 1000 * 1000) {
		panic("Invalid delay setting, %d microseconds\n", us);
		return;
	}

	if (int_func)
		use_int = 1;

	if (timer_num == 1) {
		tptr = (volatile sb1250_gptimer_t *)PHYS_TO_K1(A_SCD_TIMER_1);
		if (use_int) {
			imap += K_INT_TIMER_1;
			*imask &= ~0x8;
		}
	}
	else if (timer_num == 2) {
		tptr = (volatile sb1250_gptimer_t *)PHYS_TO_K1(A_SCD_TIMER_2);
		if (use_int) {
			imap += K_INT_TIMER_2;
			*imask &= ~0x10;
		}
	}
	else if (timer_num == 3) {
		tptr = (volatile sb1250_gptimer_t *)PHYS_TO_K1(A_SCD_TIMER_3);
		if (use_int) {
			imap += K_INT_TIMER_3;
			*imask &= ~0x20;
		}
	}
	else {
		panic("cannot allocate timer, timer_num = %d\n", timer_num);
		return;
	}

	tptr->init_cnt = us;

	if (use_int) {
		*imap = 3;							/* interrupt 3 */

		intr_registerHandler(5, int_func);	/* int3 maps to IP5 */
	}

	tptr->config = GPTIMER_CFG_MODE_CONTINUOUS | GPTIMER_CFG_ENABLE;
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	2/21/02
 *
 *-----------------------------------------------------------------
 */
void
timer_stop(
	uint32	timer_num
)
{
	volatile sb1250_gptimer_t	* tptr;

	if (timer_num == 1)
		tptr = (volatile sb1250_gptimer_t *)PHYS_TO_K1(A_SCD_TIMER_1);
	else if (timer_num == 2)
		tptr = (volatile sb1250_gptimer_t *)PHYS_TO_K1(A_SCD_TIMER_2);
	else if (timer_num == 3)
		tptr = (volatile sb1250_gptimer_t *)PHYS_TO_K1(A_SCD_TIMER_3);
	else {
		panic("cannot allocate timer, timer_num = %d\n", timer_num);
		return;
	}

	intr_unregisterHandler(5);

	tptr->config = GPTIMER_CFG_DISABLE;
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	9/24/02
 *
 *-----------------------------------------------------------------
 */
void
timer_int_clear(
	uint32	timer_num
)
{
	volatile sb1250_gptimer_t	* tptr;

	if (timer_num == 1)
		tptr = (volatile sb1250_gptimer_t *)PHYS_TO_K1(A_SCD_TIMER_1);
	else if (timer_num == 2)
		tptr = (volatile sb1250_gptimer_t *)PHYS_TO_K1(A_SCD_TIMER_2);
	else if (timer_num == 3)
		tptr = (volatile sb1250_gptimer_t *)PHYS_TO_K1(A_SCD_TIMER_3);
	else
		return;

	tptr->config = GPTIMER_CFG_MODE_CONTINUOUS | GPTIMER_CFG_ENABLE;
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	11/12/02
 *
 *-----------------------------------------------------------------
 */
uint64
timer_get_count(
	uint32	timer_num
)
{
	volatile sb1250_gptimer_t	* tptr;

	if (timer_num == 3)
		tptr = (volatile sb1250_gptimer_t *)PHYS_TO_K1(A_SCD_TIMER_3);
	else if (timer_num == 1)
		tptr = (volatile sb1250_gptimer_t *)PHYS_TO_K1(A_SCD_TIMER_1);
	else if (timer_num == 2)
		tptr = (volatile sb1250_gptimer_t *)PHYS_TO_K1(A_SCD_TIMER_2);
	else
		return -1;

	return (tptr->timer_cnt);
}

