/*
 *-----------------------------------------------------------------
 *
 * Name:	tlb.c
 *
 * RCS:  $Id: tlb.c,v 1.12 2002/11/26 22:23:10 rick Exp $
 *
 * Copyright (C) 2001, Agile Storage, Inc.
 *
 * Description:	TLB read/write commands
 *
 * Created by:	Rick Lund
 *
 * Date Created:	12/01/01
 *
 *-----------------------------------------------------------------
 */

#include <nfx-incl.h>
#include <sys/mips.h>
#include <exports/pte-api.h>
#include <exports/smp-api.h>

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	12/01/01
 *
 *-----------------------------------------------------------------
 */
void
tlb_display()
{
	uint32	pgmask, mask, pgsize;
	int32	idx, valid;
	uint64	valuehi, valuelo0, valuelo1, pa0, pa1;
	int32	low = 0, high = 64;

	for (idx = low ; idx < high; idx++) {

		__asm__ __volatile__ (
		"     mtc0 %4, $0 \n"
		"     tlbr \n"
		"     dmfc0 %0,$10 \n"
		"     dmfc0 %1,$2  \n"
		"     dmfc0 %2,$3  \n"
		"     mfc0 %3,$5  \n"
		: "=r" (valuehi) , "=r" (valuelo0) , "=r" (valuelo1), "=r" (pgmask)
		: "r" (idx)
		);

		mask = pgmask | 0x1fff;
		pa0 = (valuelo0 & 0xFFFFFFFFC0) << (12-6);
		pa1 = (valuelo1 & 0xFFFFFFFFC0) << (12-6);
		valid = (valuelo0 & TLBLO_V) || (valuelo1 & TLBLO_V);

		printf("%c%2d: hi=0x%016llx", valid ? '*' : ' ', idx, valuehi);

		pgsize = (mask + 1)/2;
		if (pgsize >= 1024*1024)
			printf(" sz=%dM", pgsize/(1024*1024));
		else
			printf(" sz=%dK", pgsize/1024);

		printf(" 0x%016llx %c%c%c%c", pa0,
				valuelo0 & TLBLO_V ? 'V':'v',
				valuelo0 & TLBLO_D ? 'D':'d',
				valuelo0 & TLBLO_G ? 'G':'g',
				((valuelo0 & TLBLO_CALGMASK) >> 3) == CFG_C_UNCACHED ? 'U':'c');
		printf(" 0x%016llx %c%c%c%c", pa1,
				valuelo1 & TLBLO_V ? 'V':'v',
				valuelo1 & TLBLO_D ? 'D':'d',
				valuelo1 & TLBLO_G ? 'G':'g',
				((valuelo1 & TLBLO_CALGMASK) >> 3) == CFG_C_UNCACHED ? 'U':'c');
		printf("\n");
	}
}	/* tlb_display() */

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	12/01/01
 *
 *-----------------------------------------------------------------
 */
int32
tlb_set(
	int32	index,
	uint64	va,
	uint64	pa,
	uint32	size,
	uint32	calg
)
{
	uint32	asm_index, mask;
	uint64	asm_src, asm_dest0, asm_dest1, asm_size;

	if (size == 0x100000)
		asm_size = TLBMASK_MASK1Mb;
	else if (size == 0x400000)
		asm_size = TLBMASK_MASK4Mb;
	else if (size == 0x1000000)
		asm_size = TLBMASK_MASK16Mb;
	else if (size == 0x1000)
		asm_size = TLBMASK_MASK4Kb;
	else if (size == 0x10000)
		asm_size = TLBMASK_MASK64kb;
	else {
		printf("Invalid TLB size, 0x%x\n", size);
		return 0;
	}
	mask = size-1;
	if (va & mask) {
		printf("Invalid Virtual address for requested entry size, %p\n", (void *)va);
		return -1;
	}
	if (pa & mask) {
		printf("Invalid Physical address for requested entry size, %p\n", (void *)pa);
		return -1;
	}

#define TLB_ENTHI_ADDR_MASK		0x00000FFFFFFFE000
#define TLB_ENTLO_ADDR_MASK		0xFFFFFFF000
#define TLB_ENTLO_ADDR_SHIFT    6

	asm_index = index;
	asm_src = (va & TLB_ENTHI_ADDR_MASK);
	asm_dest0 = ((pa & TLB_ENTLO_ADDR_MASK) >> TLB_ENTLO_ADDR_SHIFT) |
							(TLBLO_G | TLBLO_V | TLBLO_D) | (calg << 3);
	asm_dest1 = (((pa+size) & TLB_ENTLO_ADDR_MASK) >> TLB_ENTLO_ADDR_SHIFT)|
							(TLBLO_G | TLBLO_V | TLBLO_D) | (calg << 3);

	__asm__ __volatile__ (
	"     mtc0 %0,$0 \n"
	"     dmtc0 %4,$5 \n"
	"     dmtc0 %1,$10 \n"
	"     dmtc0 %2,$2  \n"
	"     dmtc0 %3,$3  \n"
	"     nop \n"
	"     nop \n"
	"     nop \n"
	"     nop \n"
	"     tlbwi \n"
	"     nop \n"
	"     nop \n"
	"     nop \n"
	"     nop \n"
	:
	: "r" (asm_index), "r" (asm_src), "r" (asm_dest0), "r" (asm_dest1), "r" (asm_size)
	);

	return 0;

}	/* tlb_set() */

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	5/12/02
 *
 *-----------------------------------------------------------------
 */
void
tlb_init()
{

	__asm__ __volatile__ (
	"     dmtc0 $0,$2 \n"
	"     dmtc0 $0,$3 \n"
	"     dmtc0 $0,$5 \n"
	"     lui   $8,0xA000 \n"
	"     li   $9,63 \n"
	"     .set noreorder \n"
	"     nop \n"
	"     1:dmtc0  $8,$10 \n"
	"     mtc0 $9,$0 \n"
	"     addu $8,0x2000\n"
	"     tlbwi \n"
	"     bnez $9,1b \n"
	"     subu $9,1 \n"
	"     .set reorder \n"
	);

}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	12/01/01
 *
 *-----------------------------------------------------------------
 */
void
mem_map_tlb_init(uint32 cpu_num)
{

	int32	index = 0;

#ifdef PROM
	/*
	 *
	 *
	 *
	 * Assumes 256 MB of memory installed, and both images are running this
	 * same memory map
	 *
	 *
	 *
	 */

/* TXRX */
	if (isTXRX()) {
		/*
		 * Map DRAM (image and heap) at virtual 0x10000000 to physical 0, 128 MB
		 */
		tlb_set(index++, 0x10000000, 0x00000000, 0x1000000, CFG_C_COHERENTXCLW);
		tlb_set(index++, 0x12000000, 0x02000000, 0x1000000, CFG_C_COHERENTXCLW);
		tlb_set(index++, 0x14000000, 0x04000000, 0x1000000, CFG_C_COHERENTXCLW);
		tlb_set(index++, 0x16000000, 0x06000000, 0x1000000, CFG_C_COHERENTXCLW);

		/*
		 * Map Shared Memory at virtual 0x20000000 to physical 0x08000000, 64 MB
		 */
		tlb_set(index++, 0x20000000, 0x08000000, 0x1000000, CFG_C_COHERENTXCLW);
		tlb_set(index++, 0x22000000, 0x0A000000, 0x1000000, CFG_C_COHERENTXCLW);

#if 0
		/*
		 * Map SAC at virtual 0x70000000 to physical 0x0C000000, 48 MB
		 */
		tlb_set(index++, 0x70000000, 0x0C000000, 0x1000000, CFG_C_COHERENTXCLW);
		tlb_set(index++, 0x72000000, 0x0E000000, 0x100000, CFG_C_COHERENTXCLW);
		tlb_set(index++, 0x72200000, 0x0E200000, 0x100000, CFG_C_COHERENTXCLW);
		tlb_set(index++, 0x72400000, 0x0E400000, 0x100000, CFG_C_COHERENTXCLW);
		tlb_set(index++, 0x72600000, 0x0E600000, 0x100000, CFG_C_COHERENTXCLW);
		tlb_set(index++, 0x72800000, 0x0E800000, 0x100000, CFG_C_COHERENTXCLW);
		tlb_set(index++, 0x72A00000, 0x0EA00000, 0x100000, CFG_C_COHERENTXCLW);
		tlb_set(index++, 0x72C00000, 0x0EC00000, 0x100000, CFG_C_COHERENTXCLW);
		tlb_set(index++, 0x72E00000, 0x0EE00000, 0x100000, CFG_C_COHERENTXCLW);
#endif

		/*
		 * Last 16 MB used by CFE
		 */

		/*
		 * Map FP's shared memory
		 * Use virtual 0x40000000 to physical 0xE0_0800_0000 (112 MB)
		 */
		tlb_set(index++, 0x40000000, 0xE008000000, 0x1000000, CFG_C_UNCACHED);
		tlb_set(index++, 0x42000000, 0xE00A000000, 0x1000000, CFG_C_UNCACHED);
		tlb_set(index++, 0x44000000, 0xE00C000000, 0x1000000, CFG_C_UNCACHED);
		tlb_set(index++, 0x46000000, 0xE00E000000, 0x100000, CFG_C_UNCACHED);
		tlb_set(index++, 0x46200000, 0xE00E200000, 0x100000, CFG_C_UNCACHED);
		tlb_set(index++, 0x46400000, 0xE00E400000, 0x100000, CFG_C_UNCACHED);
		tlb_set(index++, 0x46600000, 0xE00E600000, 0x100000, CFG_C_UNCACHED);
		tlb_set(index++, 0x46800000, 0xE00E800000, 0x100000, CFG_C_UNCACHED);
		tlb_set(index++, 0x46A00000, 0xE00EA00000, 0x100000, CFG_C_UNCACHED);
		tlb_set(index++, 0x46C00000, 0xE00EC00000, 0x100000, CFG_C_UNCACHED);
		tlb_set(index++, 0x46E00000, 0xE00EE00000, 0x100000, CFG_C_UNCACHED);

		/* FP shmem cached , 32 MB */
		tlb_set(index++, 0x70000000, 0xE008000000, 0x1000000, CFG_C_WBACK);

		/*
		 * Map EEE's WKA in TXRX memory
		 */
		tlb_set(index++, 0x73000000, 0x0F000000, 0x100000, CFG_C_COHERENTXCLW);
	}

/* FP */
	if (isFP()) {

		/*
		 * Map DRAM (image and heap) at virtual 0x10000000 to physical 0, 128 MB
		 */
		tlb_set(index++, 0x10000000, 0x00000000, 0x1000000, CFG_C_COHERENTXCLW);
		tlb_set(index++, 0x12000000, 0x02000000, 0x1000000, CFG_C_COHERENTXCLW);
		tlb_set(index++, 0x14000000, 0x04000000, 0x1000000, CFG_C_COHERENTXCLW);
		tlb_set(index++, 0x16000000, 0x06000000, 0x1000000, CFG_C_COHERENTXCLW);

		/*
		 * Map Shared Memory at virtual 0x40000000 to physical 0x08000000, 112 MB
		 */
		tlb_set(index++, 0x40000000, 0x08000000, 0x1000000, CFG_C_COHERENTXCLW);
		tlb_set(index++, 0x42000000, 0x0A000000, 0x1000000, CFG_C_COHERENTXCLW);
		tlb_set(index++, 0x44000000, 0x0C000000, 0x1000000, CFG_C_COHERENTXCLW);
		tlb_set(index++, 0x46000000, 0x0E000000, 0x100000, CFG_C_COHERENTXCLW);
		tlb_set(index++, 0x46200000, 0x0E200000, 0x100000, CFG_C_COHERENTXCLW);
		tlb_set(index++, 0x46400000, 0x0E400000, 0x100000, CFG_C_COHERENTXCLW);
		tlb_set(index++, 0x46600000, 0x0E600000, 0x100000, CFG_C_COHERENTXCLW);
		tlb_set(index++, 0x46800000, 0x0E800000, 0x100000, CFG_C_COHERENTXCLW);
		tlb_set(index++, 0x46A00000, 0x0EA00000, 0x100000, CFG_C_COHERENTXCLW);
		tlb_set(index++, 0x46C00000, 0x0EC00000, 0x100000, CFG_C_COHERENTXCLW);
		tlb_set(index++, 0x46E00000, 0x0EE00000, 0x100000, CFG_C_COHERENTXCLW);

		/*
		 * Last 16 MB used by CFE
		 */

		/*
		 * Map TXRX's shared memory
		 * Use virtual 0x20000000 to physical 0xE0_0800_0000 (112 MB)
		 */
		tlb_set(index++, 0x20000000, 0xE008000000, 0x1000000, CFG_C_UNCACHED);
		tlb_set(index++, 0x22000000, 0xE00A000000, 0x1000000, CFG_C_UNCACHED);

		/* TXRX shmem cached (32 MB) */
		tlb_set(index++, 0x60000000, 0xE008000000, 0x1000000, CFG_C_WBACK);

#if 0
		/*
		 * Map SAC in TXRX's memory
		 */
		tlb_set(index++, 0x70000000, 0xE00C000000, 0x1000000, CFG_C_UNCACHED);
		tlb_set(index++, 0x72000000, 0xE00E000000, 0x100000, CFG_C_UNCACHED);
		tlb_set(index++, 0x72200000, 0xE00E200000, 0x100000, CFG_C_UNCACHED);
		tlb_set(index++, 0x72400000, 0xE00E400000, 0x100000, CFG_C_UNCACHED);
		tlb_set(index++, 0x72600000, 0xE00E600000, 0x100000, CFG_C_UNCACHED);
		tlb_set(index++, 0x72800000, 0xE00E800000, 0x100000, CFG_C_UNCACHED);
		tlb_set(index++, 0x72A00000, 0xE00EA00000, 0x100000, CFG_C_UNCACHED);
		tlb_set(index++, 0x72C00000, 0xE00EC00000, 0x100000, CFG_C_UNCACHED);
		tlb_set(index++, 0x72E00000, 0xE00EE00000, 0x100000, CFG_C_UNCACHED);
#endif

		/*
		 * Map EEE's WKA in TXRX memory
		 */
		tlb_set(index++, 0x73000000, 0xE00F000000, 0x100000, CFG_C_UNCACHED);
	}

	/*
	 * PCI space
	 */
	tlb_set(index++, 0x50000000, 0xF800000000, 0x1000000, CFG_C_UNCACHED);

#else

#ifdef LARGE_MEMORY
	tlb_init();

#ifdef NFP_TXRX
	/*
	 * Map EEE's WKA in TXRX memory
	 */
	tlb_set(index++, 0x73000000, 0x0F000000, 0x100000, CFG_C_COHERENTXCLW);

	/*
	 * PCI space
	 */
	tlb_set(index++, 0x50000000, 0xF800000000, 0x1000000, CFG_C_UNCACHED);

	tlb_set(index++, 0x60000000,
		K0_TO_PHYS(&(smp_ClientPrivatePhysSpace[cpu_num][0])),
								SMP_CLIENTPRIVATE_TLB_SIZE, CFG_C_COHERENTXCLW);

#endif

#ifdef NFP_FP
	/*
	 * Map EEE's WKA in TXRX memory
	 */
	tlb_set(index++, 0x73000000, 0xE00F000000, 0x100000, CFG_C_UNCACHED);

	/*
	 * PCI space
	 */
	tlb_set(index++, 0x50000000, 0xF800000000, 0x1000000, CFG_C_UNCACHED);

	tlb_set(index++, 0x60000000,
		K0_TO_PHYS(&(smp_ClientPrivatePhysSpace[cpu_num][0])),
								SMP_CLIENTPRIVATE_TLB_SIZE, CFG_C_COHERENTXCLW);

#endif

	_mips_mtc0(C0_WIRED, 3);
#endif

#endif

}

