/*
 *-----------------------------------------------------------------
 *
 * Name:	sbm-api.h
 *
 * RCS:  $Id: sbm-api.h,v 1.10 2002/11/07 01:16:54 rick Exp $
 *
 * Copyright (C) 2002, Agile Storage Inc.
 *
 * Description:	API for interface into SAC Buffer Management.
 *
 * Created by:	Rick Lund
 *
 * Date Created:	8/7/02
 *
 *-----------------------------------------------------------------
 */

#ifndef _SBM_API_H
#define _SBM_API_H

#ifdef NFP_TXRX
/*------------------------------------------------------------------
 *
 * These are command functions, called by LUC functions on the TXRX
 *
 *------------------------------------------------------------------*/
extern int32 sbm_sendDataToLuc(	nfx_cod_t	cod,
								int32		cod_len,
								int32		cod_offset,
								int32		pad_size,
								char		* newbuf,
                                void        * cb_cookie);

extern void sbm_sendCODToSac(	nfx_cod_t	cod);

extern int32
sbm_verifyPktInTXRXPool(
    eee_descPtr_t   edesc
);

extern int32
sbm_rx_copy_to_txrx(
    eee_descPtr_t   edesc,
    eee_descPtr_t   * new_edesc,
    uint32          nbufs
);

#endif


#include <exports/eee-api.h>
extern nfx_cod_t sbm_getCODFromDesc(eee_descPtr_t	edesc,
									uint32			slot_id,
									uint32			cpu_id);

extern int32
sbm_freeCOD(
    nfx_cod_t   cod,
    uint32      update_stats
);

/*------------------------------------------------------------------
 *
 * These are command messages sent from the FP to the SBM
 *
 *------------------------------------------------------------------*/
typedef enum {
	CMD_TO_MCPU_SEND_CONTROL_AND_DATA_TO_SNIM	= SBM_MSG_BASE,
	CMD_TO_MCPU_SEND_CONTROL_AND_DATA_TO_SRC,
	CMD_TO_MCPU_MANIPULATE_FREE,
	CMD_TO_MCPU_MANIPULATE_MERGE,
	CMD_TO_MCPU_MANIPULATE_CREATE,
	CMD_TO_MCPU_MANIPULATE_SPLIT,
	CMD_TO_MCPU_MANIPULATE_CONCAT,
	CMD_TO_MCPU_ADDRESS_OF_SAC_INFO_STRUCT
} sbm_msgType_t;

/*-----------------------------------------------------------------*/
/*
 * This command is sent to the MCPU by the FP or TX, when it wants to send
 * SAC data, prepended with a control packet (for data writes to disk) to
 * either FCNIM or LUC.
 */
typedef struct send_control_and_sac_data {
	uint32		cmd;
	uint32		dest_port;
	uint32		source_port;
	uint32		msg_len;
	nfx_cod_t	COD;
	uint32		length;
	uint32		offset;
	uchar8		* msg;
}  sbm_sendControlAndSACData_t;

/*-----------------------------------------------------------------*/
/*
 * This command is sent to the MCPU to free a COD handle and associated
 * SAC blocks.
 */
typedef struct free_cod {
	uint32		cmd;
	uint32		resv;
	nfx_cod_t	COD;
} sbm_freeCOD_t;

/*-----------------------------------------------------------------*/
/*
 * This command is sent to the MCPU to merge the contents of one COD with
 * the contents of another COD.
 *
 * NOTE:  AFTER THE MERGE, THE SOURCE COD IS INVALID!!!!!!
 */
typedef struct merge_cod {
	uint32		cmd;
	uint32		src_len;
	nfx_cod_t	src_COD;
	uint32		src_offset;
	uint32		dest_offset;
	nfx_cod_t	dest_COD;
}  sbm_mergeCOD_t;

/*-----------------------------------------------------------------*/
/*
 * This command is sent to the MCPU to split a COD.
 */
typedef struct split_cod {
	uint32		cmd;
	uint32		src_offset;
	nfx_cod_t	src_COD;
	nfx_cod_t	dest_COD;
}  sbm_splitCOD_t;

/*-----------------------------------------------------------------*/
/*
 * This command is sent to the MCPU to concatenate 2 cods.
 */
typedef struct concat_cod {
	uint32		cmd;
	nfx_cod_t	dest_COD;
	nfx_cod_t	src_COD;
}  sbm_concatCOD_t;

/*-----------------------------------------------------------------*/
/*
 * This command is sent to the MCPU to create a COD and zero it.
 */
typedef struct create_cod {
	uint32		cmd;
	uint32		len;
	nfx_cod_t	COD;
	uint32		fill;
}  sbm_createCOD_t;


/*
 * sbm_sac_info_t is structure in FP memory which is written by TXRX and
 * read by FP, containing real time info about sac block utilization.
 */
typedef struct sbm_sac_i {
	uint64		num_desc_avail;
	uint64		num_buf_small_avail;
	uint64		num_buf_large_avail;
	uint64		num_buf_ultra_avail;
	uint64		num_buf_LWM_small;
	uint64		num_buf_LWM_large;
	uint64		num_buf_LWM_ultra;
} sbm_sac_info_t;

typedef struct sbm_addr_msg {
	uint32			cmd;
	uint32			reserved;
	sbm_sac_info_t	* addr;
} sbm_infoAddrMsg_t;

extern sbm_sac_info_t	* sbm_sac_info;

extern uint64	sbm_get_sac_size();

#endif	/* _SBM_API_H */

