/*
 *-----------------------------------------------------------------
 *
 * Name:	sbm-init.c
 *
 * RCS:  $Id: sbm-init.c,v 1.15 2002/11/01 23:25:14 janv Exp $
 *
 * Copyright (C) 2002, Agile Storage Inc.
 *
 * Description:	Initialization functions for SAC Buffer Management.
 *				Should allocate memory structures, etc.
 *
 * Created by:	Rick Lund
 *
 * Date Created:	8/7/02
 *
 *-----------------------------------------------------------------
 */

#include <nfx-incl.h>

#ifndef SBM_REV2
#error Compiled without SBM_REV2
#endif

#include <exports/eee-api.h>
#include <exports/sbm-api.h>
#include <exports/cm-api.h>
#include <exports/malloc-api.h>
#ifndef LINUX_TEST
#ifdef NFP_TXRX
#include <exports/sb1250-dm-api.h>
#include <exports/cpu-api.h>
#endif
#endif
#include "sbm.h"

#ifdef NFP_TXRX
sbm_stats_t			sbm_stats;
#endif

sbm_sac_info_t		* sbm_sac_info;

#ifdef NFP_TXRX

extern uint32 sbm_evm_dest_port;
extern uint32 sbm_scsit_dest_port;
extern int32 sbm_tx_to_luc_use_datamover;
void sbm_sac_scsit_rx();
/*-----------------------------------------------------------------
 * Name:	sbm_init()
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	8/7/02
 *
 *-----------------------------------------------------------------
 */
int32			/* returns	NFX_OK,
				 *			SBM_ERR_INIT_NO_MEM,
				 */
sbm_init(
	uint32	pass
)
{
	int32	sbm_app_id, sac_app_id, test_app_id, sac_scsit_app_id;

	if (pass == 1) {
		SBM_INIT_DEBUG(("[sbm_init]:entry1\n"));

		bzero(&sbm_stats, sizeof(sbm_stats_t));
	}
	else if (pass == 2) {
		SBM_INIT_DEBUG(("[sbm_init]:entry2\n"));

		/*------------------------------------------------------------------*/
		/*						Application Interface						*/
		/*------------------------------------------------------------------*/
		sbm_app_id = eee_allocateApplicationId("sbm");
		eee_registerReceiveFunc(sbm_app_id,
								sbm_processReceivePackets,
								sbm_processMcastReceivePackets);

		sac_app_id = eee_allocateApplicationId("sac");
        eee_registerReceiveFunc(sac_app_id, sbm_sac_rx, sbm_sac_rx);

		sbm_evm_dest_port = PACK_UNICAST_PORT_ADDRESS(nfxMySlotId,
													NFX_NFP_FP_ANY,
													eee_getApplicationId("evm"),
													EEE_USER_DATA,
													NFX_COS_HIGH,
													PORT_BEST_FIT);

		sac_scsit_app_id = eee_allocateApplicationId("sac_scsit");
        eee_registerReceiveFunc(sac_scsit_app_id, sbm_sac_scsit_rx, sbm_sac_scsit_rx);

		sbm_scsit_dest_port = PACK_UNICAST_PORT_ADDRESS(nfxMySlotId,
														NFX_NFP_FP_ANY,
														eee_getApplicationId("scsitest"),
														EEE_USER_DATA,
														NFX_COS_HIGH,
														PORT_BEST_FIT);

		/*------------------------------------------------------------------*/
		/*						Test Function Registration					*/
		/*------------------------------------------------------------------*/
		test_app_id = eee_allocateApplicationId("gmc_test_rcv");
		eee_registerReceiveFunc(test_app_id,
								sbm_test_rx,
								sbm_test_rx);

#ifndef LINUX_TEST
		if (sb1250_getRev() > SB1250_PASS1)
#endif
			sbm_tx_to_luc_use_datamover = 1;

		SBM_INIT_DEBUG(("[sbm_init]:leaving\n"));
	}
	else if (pass == 3) {
		SBM_INIT_DEBUG(("[sbm_init]:entry3\n"));

#ifndef LINUX_TEST
		if (sb1250_getRev() > SB1250_PASS1) {
			sb1250dm_openChnl(SB1250DM_CHNL_SAC_TO_SNIM,
											sbm_txCodToSnimWithDataMoverCB);
		}
#endif

		eee_registerPollingFunc(EEE_HIGH_PRIORITY, sbm_mgmt_update_sac_info_struct, NULL);

		SBM_INIT_DEBUG(("[sbm_init]:leaving\n"));

	}

	return (NFX_OK);

} /* sbm_init() */

#else

int32
sbm_init(
	uint32	pass
)
{
	int32	test_app_id;

#ifdef NFP_FP
	if (pass == 1) {
		sbm_sac_info = (sbm_sac_info_t *)eee_allocateBlock(
													sizeof(sbm_sac_info_t));
		printf("sbm_sac_info at %p\n", sbm_sac_info);
		bzero(sbm_sac_info, sizeof(sbm_sac_info_t));
	}
#endif

	if (pass == 2) {
		test_app_id = eee_allocateApplicationId("gmc_test_rcv");
		eee_registerReceiveFunc(test_app_id,
								sbm_test_rx,
								sbm_test_rx);
#ifdef NFP_FP
		{
			sbm_infoAddrMsg_t	* msg;
			uint32				dp, sp;
			int32				sbm_app_id;

			sbm_app_id = eee_allocateApplicationId("sbm");

			dp = PACK_UNICAST_PORT_ADDRESS(	nfxMySlotId,
											NFX_NFP_TXRX_ANY,
											sbm_app_id,
											EEE_MGMT,
											NFX_COS_HIGH,
											PORT_BEST_FIT);

			sp = PACK_UNICAST_PORT_ADDRESS(	nfxMySlotId,
											NFX_NFP_FP_ANY,
											sbm_app_id,
											EEE_MGMT,
											NFX_COS_HIGH,
											PORT_BEST_FIT);

			msg = (sbm_infoAddrMsg_t *)eee_ramAlloc(sizeof(sbm_infoAddrMsg_t));
			msg->cmd = CMD_TO_MCPU_ADDRESS_OF_SAC_INFO_STRUCT;
			msg->addr = sbm_sac_info;

			eee_sendMessage(msg, sizeof(sbm_infoAddrMsg_t), dp, sp);
		}
#endif
	}

	return NFX_OK;

} /* sbm_init() */

#endif	/* NFP_TXRX */

