/*
 *-----------------------------------------------------------------
 *
 * Name:	sbm-test.c
 *
 * RCS:  $Id: sbm-test.c,v 1.17 2002/11/07 01:16:54 rick Exp $
 *
 * Copyright (C) 2002, Agile Storage, Inc.
 *
 * Description:	Contains routines for displaying and debugging the
 *				SAC buffer management structures.
 *
 * Created by:	Rick Lund
 *
 * Date Created:	8/9/02
 *
 *-----------------------------------------------------------------
 */

#include <nfx-incl.h>

#ifndef SBM_REV2
#error Compiled without SBM_REV2
#endif

#include <exports/eee-api.h>
#include <exports/cm-api.h>
#include <exports/malloc-api.h>
#include <exports/sbm-api.h>
#include <exports/scsi-api.h>
#include "sbm.h"
#include "sbm-test.h"

#ifdef LINUX_TEST
#include <fcntl.h>
#endif


#ifdef NFP_TXRX

/*-----------------------------------------------------------------
 * Name:	sbm_testDisplayCod()
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	8/9/02
 *
 *-----------------------------------------------------------------
 */
void
sbm_testDisplayCod(
	nfx_cod_t	cod
)
{
	eee_descPtr_t	edesc;
	eee_descBuf_t	* bd;
	byte			attr0 = 0, attr1 = 0, attr2 = 0, attr3 = 0;
	int32			num_bufs, i;

	if (sbm_verifyIsCODInSac(cod, 1)) {
		edesc = sbm_getDescFromCOD(cod);

		printf("cod = 0x%016llx\n", cod);
		while (edesc) {
			printf("    edesc = %p, len = 0x%x, nb = 0x%x, last_attr = %d\n",
											edesc, EEE_GET_PKT_LEN(edesc),
											EEE_GET_NUM_BUFS(edesc),
											eee_attrGetDescLastAttr(edesc));
			eee_attrGetDescValidFreeAttr(edesc, &attr0, &attr1, &attr2, &attr3);
			printf("        valid free = (%d, %d, %d, %d)\n",
												attr0, attr1, attr2, attr3);
			num_bufs = EEE_GET_NUM_BUFS(edesc);

			if (edesc->hdr.control & EDESC_COMMAND_USE_FIRST_BUFFER)
				bd = &(edesc->bd[0]);
			else
				bd = &(edesc->bd[1]);

			for (i=0; i<num_bufs; i++) {
				printf("        buf = %p, len = 0x%x, last_attr = %d\n",
							bd->buf, bd->len, eee_attrGetBufLastAttr(bd->buf));
				eee_attrGetBufValidFreeAttr(bd->buf, &attr0, &attr1,
																&attr2, &attr3);
				printf("        valid free = (%d, %d, %d, %d)\n",
													attr0, attr1, attr2, attr3);
				bd++;
			}
			edesc = edesc->hdr.next;
		}
	}
	else
		printf("cod, 0x%llx, not in SAC\n", cod);
}

/*-----------------------------------------------------------------
 * Name:	sbm_testDisplayCod()
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	8/9/02
 *
 *-----------------------------------------------------------------
 */
void
sbm_testDisplayPkt(
    eee_descPtr_t   edesc
)
{
	eee_descBuf_t	* bd;
	byte			attr0 = 0, attr1 = 0, attr2 = 0, attr3 = 0;
	int32			num_bufs, i;

		while (edesc) {
			printf("    edesc = %p, len = 0x%x, nb = 0x%x, last_attr = %d\n",
											edesc, EEE_GET_PKT_LEN(edesc),
											EEE_GET_NUM_BUFS(edesc),
											eee_attrGetDescLastAttr(edesc));
			eee_attrGetDescValidFreeAttr(edesc, &attr0, &attr1, &attr2, &attr3);
			printf("        valid free = (%d, %d, %d, %d)\n",
												attr0, attr1, attr2, attr3);
			num_bufs = EEE_GET_NUM_BUFS(edesc);

			if (edesc->hdr.control & EDESC_COMMAND_USE_FIRST_BUFFER)
				bd = &(edesc->bd[0]);
			else
				bd = &(edesc->bd[1]);

			for (i=0; i<num_bufs; i++) {
				printf("        buf = %p, len = 0x%x, last_attr = %d\n",
							bd->buf, bd->len, eee_attrGetBufLastAttr(bd->buf));
				eee_attrGetBufValidFreeAttr(bd->buf, &attr0, &attr1,
																&attr2, &attr3);
				printf("        valid free = (%d, %d, %d, %d)\n",
													attr0, attr1, attr2, attr3);
				bd++;
			}
			edesc = edesc->hdr.next;
		}
}

/*-----------------------------------------------------------------
 * Name:	sbm_testFindAllCods()
 *
 * Description: searches TXRX descriptors for an EEE attribute that means the
 *				descriptor is in the SAC.
 *
 * Created by:	Rick Lund
 *
 * Date Created:	8/9/02
 *
 *-----------------------------------------------------------------
 */
void
sbm_testFindAllCods()
{
#ifdef EEE_ATTR
#include "../sm-eee/eee.h"
	eee_descPtr_t	edesc;

	for (edesc = eee.desc_base[EEE_TXRX_POOL - SHARED_POOL_OFFSET];
			edesc < eee.desc_end[EEE_TXRX_POOL - SHARED_POOL_OFFSET]; edesc++) {
		if (sbm_verifyIsEdescInSac(edesc, 1)) {
			if (sbm_verifyIsPktInSac(edesc, 1))
				sbm_testDisplayCod((nfx_cod_t)edesc);
/*
			else
				printf("Error in sac packet, edesc = %p\n", edesc);
*/
		}
	}
#endif
}

/*-----------------------------------------------------------------
 * Name:	sbm_testDisplayStats()
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	8/9/02
 *
 *-----------------------------------------------------------------
 */
void
sbm_testDisplayStats()
{
	printf("COD:  present    %16lld\n", sbm_stats.cods_present);
	printf("      tot_bytes  %16lld\n", sbm_stats.tot_bytes);
	printf("      num_bufs   %16lld\n", sbm_stats.num_bufs);
	printf("      bytes/buf  %16lld\n", (sbm_stats.num_bufs== 0) ? 0 : sbm_stats.tot_bytes / sbm_stats.num_bufs);
	printf("\n");
	printf("      rcvd     %16lld   (%lld)\n",
										sbm_stats.cods_rx,
										sbm_stats.cods_rx_total);
	printf("      created  %16lld   (%lld)\n",
										sbm_stats.cods_created,
										sbm_stats.cods_created_total);
	printf("      freed    %16lld   (%lld)\n",
										sbm_stats.cods_freed,
										sbm_stats.cods_freed_total);
	printf("      merged   %16lld   (%lld)\n",
										sbm_stats.cods_merged,
										sbm_stats.cods_merged_total);
	printf("      split    %16lld   (%lld)\n",
										sbm_stats.cods_split,
										sbm_stats.cods_split_total);
	printf("      concat   %16lld   (%lld)\n",
										sbm_stats.cods_concat,
										sbm_stats.cods_concat_total);

	printf("RX:   rx_luc       %16lld   (%lld)\n",
										sbm_stats.cods_rx_luc,
										sbm_stats.cods_rx_luc_total);
	printf("      rx_snim      %16lld   (%lld)\n",
										sbm_stats.cods_rx_snim,
										sbm_stats.cods_rx_snim_total);

	printf("TX:   tx_luc       %16lld   (%lld)\n",
										sbm_stats.cods_tx_luc,
										sbm_stats.cods_tx_luc_total);
	printf("      tx_snim      %16lld   (%lld)\n",
										sbm_stats.cods_tx_snim,
										sbm_stats.cods_tx_snim_total);
	printf("      tx_src       %16lld   (%lld)\n",
										sbm_stats.cods_tx_src,
										sbm_stats.cods_tx_src_total);

	printf("COPY: bytes copied mgmt     %16lld   (%lld)\n",
										sbm_stats.bytes_copied_mgmt,
										sbm_stats.bytes_copied_mgmt_total);
	printf("      bytes copied tx       %16lld   (%lld)\n",
										sbm_stats.bytes_copied_tx,
										sbm_stats.bytes_copied_tx_total);
	printf("      bytes copied rx       %16lld   (%lld)\n",
										sbm_stats.bytes_copied_rx,
										sbm_stats.bytes_copied_rx_total);
	printf("      bytes copied collapse %16lld   (%lld)\n",
										sbm_stats.bytes_copied_collapse,
										sbm_stats.bytes_copied_collapse_total);
	printf("      desc collapse         %16lld   (%lld)\n",
										sbm_stats.desc_collapse,
										sbm_stats.desc_collapse_total);
	printf("\n");
	printf("DESC: avail        %16d\n", eee_getAvailableNbrOfDesc());
	printf("      error        %16lld   (%lld)\n",
										sbm_stats.no_desc_error,
										sbm_stats.no_desc_error_total);
	printf("BUF:  small avail  %16d\n",eee_getAvailableNbrOfBufs(EEE_BUF_SMALL));
	printf("      large avail  %16d\n",eee_getAvailableNbrOfBufs(EEE_BUF_LARGE));
	printf("      ultra avail  %16d\n",eee_getAvailableNbrOfBufs(EEE_BUF_ULTRA));
	printf("      error        %16lld   (%lld)\n",
										sbm_stats.no_buf_error,
										sbm_stats.no_buf_error_total);


	printf("RX VERIFY: len verify ok      %16lld   (%lld)\n",
										sbm_stats.len_verify_ok,
										sbm_stats.len_verify_ok_total);
	printf("           len verify error   %16lld   (%lld)\n",
										sbm_stats.len_verify_error,
										sbm_stats.len_verify_error_total);
	printf("           len verify skip    %16lld   (%lld)\n",
										sbm_stats.len_verify_skip,
										sbm_stats.len_verify_skip_total);
#ifdef USE_SAC_CHECKSUM
	printf("           crc verify ok      %16lld   (%lld)\n",
										sbm_stats.chksum_verify_ok,
										sbm_stats.chksum_verify_ok_total);
	printf("           crc verify error   %16lld   (%lld)\n",
										sbm_stats.chksum_verify_error,
										sbm_stats.chksum_verify_error_total);
	printf("           crc verify skip    %16lld   (%lld)\n",
										sbm_stats.chksum_verify_skip,
										sbm_stats.chksum_verify_skip_total);
#endif

}

/*-----------------------------------------------------------------
 * Name:	sbm_testClearStats()
 *
 * Description: doesn't zero out all fields.  leaves some, such as # of cods
 *				present in sac.
 *
 * Created by:	Rick Lund
 *
 * Date Created:	8/9/02
 *
 *-----------------------------------------------------------------
 */
void
sbm_testClearStats()
{
	sbm_stats.cods_rx = 0;
	sbm_stats.cods_created = 0;
	sbm_stats.cods_freed = 0;
	sbm_stats.cods_merged = 0;
	sbm_stats.cods_split = 0;
	sbm_stats.cods_concat = 0;

	sbm_stats.cods_rx_luc = 0;
	sbm_stats.cods_rx_snim = 0;

	sbm_stats.cods_tx_luc = 0;
	sbm_stats.cods_tx_snim = 0;
	sbm_stats.cods_tx_src = 0;

	sbm_stats.bytes_copied_mgmt = 0;
	sbm_stats.bytes_copied_tx = 0;
	sbm_stats.bytes_copied_rx = 0;
	sbm_stats.bytes_copied_collapse = 0;
	sbm_stats.desc_collapse = 0;

	sbm_stats.no_desc_error = 0;
	sbm_stats.no_buf_error = 0;

	sbm_stats.empty_rq_desc = 0;
	sbm_stats.empty_rq_buf_ultra = 0;

	sbm_stats.len_verify_ok = 0;
	sbm_stats.len_verify_skip = 0;
	sbm_stats.len_verify_error = 0;
#ifdef USE_SAC_CHECKSUM
	sbm_stats.chksum_verify_ok = 0;
	sbm_stats.chksum_verify_skip = 0;
	sbm_stats.chksum_verify_error = 0;
#endif

}

#endif	/* NFP_TXRX */

/*-----------------------------------------------------------------
 * Name:	sbm_test_rx()
 *
 * Description: registered receive application for SBM test packets
 *
 *		on TXRX, packets will be integrated into the SAC
 *		on FP, scsi_readRspMsg will be delivered here from test packets
 *			sent to the SAC from the SNIM.
 *		on FCNIM, user data test packet will be sent here.  The function should
 *			save the data to a file (LINUX_TEST).
 *
 *
 *
 *
 * Created by:	Rick Lund
 *
 * Date Created:	8/9/02
 *
 *-----------------------------------------------------------------
 */
void
sbm_test_rx(
	eee_descPtr_t	edesc
)
{
#ifdef NFP_TXRX
	uint32	src_slot, ack_port;

	src_slot = EEE_GET_SLOT_NUM(edesc->hdr.src_port);

	if (src_slot == 2) {
		printf("sbm_test_rx, edesc = %p\n", edesc);
		ack_port = PACK_UNICAST_PORT_ADDRESS(nfxMySlotId,
											NFX_NFP_FP_ANY,
											eee_getApplicationId("gmc_test_rcv"),
											EEE_USER_DATA,
											NFX_COS_HIGH,
											PORT_BEST_FIT);

		sbm_rx_add_to_sac(edesc, ack_port);
	}
	else
		panic("sbm_test_rx, bad src_slot = %d\n", src_slot);
#endif

#ifdef NFP_FP
#include <exports/scsi-api.h>
	eee_descBuf_t		* bd;
	scsi_readRspMsg_t	* scsi_msg;

	printf("sbm_test_rx, edesc = %p\n", edesc);

	if (edesc->hdr.control & EDESC_COMMAND_USE_FIRST_BUFFER)
		bd = &(edesc->bd[0]);
	else
		bd = &(edesc->bd[1]);

	scsi_msg = (scsi_readRspMsg_t *)(bd->buf);
	printf("scsi_readRspMsg, new cod = 0x%llx, len = 0x%x\n",
										scsi_msg->codHandle, scsi_msg->dataLen);

	eee_freePacket(edesc);

#endif

#ifdef FCNIM
#include <exports/scsi-api.h>
	uint32	len;
	uchar8	fname[8];
#ifdef LINUX_TEST
	uchar8	* rambuf;
#endif

	strcpy(fname, "abcd");

	len = EEE_GET_PKT_LEN(edesc);

	printf("sbm_test_rx, edesc = %p, len = 0x%x\n", edesc, len);

#ifdef LINUX_TEST
	rambuf = eee_ramAlloc(len);

	eee_copyPacketToLocalBuffer(rambuf, edesc, len, 0);

	rambuf += sizeof(scsi_writeReqMsg_t);
	len -= sizeof(scsi_writeReqMsg_t);

	printf("save to file, rambuf = %p, fname = %s\n", rambuf, fname);
	sbm_testSaveToFile(rambuf, len, fname);

	eee_ramDealloc(rambuf);
#endif

	eee_freePacket(edesc);

#endif

}

#if defined(NFP_FP)
/*-----------------------------------------------------------------
 * Name:	sbm_testTransmitFromSacToSnim()
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	8/7/02
 *
 *-----------------------------------------------------------------
 */
void
sbm_testTransmitFromSacToSnim(
	nfx_cod_t	cod,
	uint32		len,
	uint32		off,
	uint32		slot,
	uint32		cpu
)
{
	uchar8	buf[sizeof(sbm_sendControlAndSACData_t) + sizeof(scsi_writeReqMsg_t)];
	sbm_sendControlAndSACData_t	* msg;
	uint32	app_id, test_app_id;
	scsi_writeReqMsg_t	scsi_msg;
	uint32	dp;

	printf("transmit sac data to SNIM, COD 0x%llx, length %d, offset %d\n",
								                                cod, len, off);

	app_id = eee_getApplicationId("sbm");
	test_app_id = eee_getApplicationId("gmc_test_rcv");

	msg = (sbm_sendControlAndSACData_t *)buf;

	msg->cmd = CMD_TO_MCPU_SEND_CONTROL_AND_DATA_TO_SNIM;

	msg->dest_port = PACK_UNICAST_PORT_ADDRESS(
												slot,
												cpu,
												test_app_id,
												EEE_USER_DATA,
												NFX_COS_HIGH,
												0);

	msg->source_port = PACK_UNICAST_PORT_ADDRESS(
												nfxMySlotId,
												nfxMyCPUId,
												test_app_id,
												EEE_USER_DATA,
												NFX_COS_HIGH,
												0);
	msg->msg_len = sizeof(scsi_writeReqMsg_t);
	msg->COD = cod;
	msg->length = len;
	msg->offset = off;

	scsi_msg.hdr.mid = SCSI_MID_WRITE_REQ;
	scsi_msg.hdr.cHandle = 0xabcd0123;
	scsi_msg.blkOffset = off;
	scsi_msg.blkCnt = len;

	memcpy(&(msg->msg), &scsi_msg, sizeof(scsi_writeReqMsg_t));

	dp = PACK_UNICAST_PORT_ADDRESS(	1,              /* GbE NIM */
									NFX_NFP_TXRX_ANY,
									app_id,
									EEE_CONTROL,
									NFX_COS_HIGH,
									PORT_BEST_FIT);

	eee_sendMessage(	buf,
						sizeof(sbm_sendControlAndSACData_t) -
						sizeof(uchar8 *)+sizeof(scsi_writeReqMsg_t),
						dp,
						msg->source_port);
}
#endif	/* NFP_FP */

#ifdef NFP_TXRX
/*-----------------------------------------------------------------
 * Name:	sbm_testTransmitFromSacToLuc()
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	8/7/02
 *
 *-----------------------------------------------------------------
 */
void
sbm_testTransmitFromSacToLuc(
	nfx_cod_t	cod,
	uint32		len,
	uint32		off
)
{
	uchar8	* rambuf;
#ifdef LINUX_TEST
	uchar8	fname[] = "ofile";
#endif

	printf("transmit sac data to LUC, COD 0x%llx, length %d, offset %d\n",
								                                cod, len, off);

	rambuf = eee_ramAlloc(len+0x40);

	sbm_sendDataToLuc(cod, len, off, 0, rambuf+0x40, 0);

#ifdef LINUX_TEST
	sbm_testSaveToFile(rambuf+0x40, len, fname);
#endif

	eee_ramDealloc(rambuf);
}
#endif	/* NFP_TXRX */

#ifdef LINUX_TEST
#if defined(NFP_TXRX) | defined(FCNIM)
/*-----------------------------------------------------------------
 * Name:	sbm_testSaveToFile()
 *
 * Description:	create a file with the user data (strip the write request
 *				control message)
 *
 * Created by:	Rick Lund
 *
 * Date Created:	8/9/02
 *
 *-----------------------------------------------------------------
 */
void
sbm_testSaveToFile(
	uchar8	* buf,
	uint32	len,
	uchar8	* fname
)
{
	int32	fd;
	int32	n;

	/*
	 * send sac data
	 */
	if ((fd = open(fname, O_WRONLY | O_CREAT, 0666)) < 0) {
		printf("Unable to open file %s for writing\n", fname);
		return;
	}

	n = write(fd, buf, len);
	if (n != len)
		printf("error writing file %s\n", fname);

	close(fd);
}

#ifdef NFP_TXRX

//#define BUF_LEN		EEE_BUFSIZE_LARGE
#define BUF_LEN		800

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	8/7/02
 *
 *-----------------------------------------------------------------
 */
nfx_cod_t
sbm_testCreateCodFromBuffer(
	uchar8	* buf,
	uint32	len
)
{
	eee_descPtr_t	edesc;
	eee_descPtr_t	first_edesc;
	eee_descBuf_t	* bd;
	uint32	save_len = len;
	uint32	off = 0, num_bufs;

#define TEST_BUF_MAX_PER_DESC	1

	first_edesc = edesc = eee_allocateDesc(EEE_TXRX_POOL, NFX_COS_HIGH);

	while (edesc) {
		num_bufs = 0;
		bd = &(edesc->bd[1]);
		while (len) {
			bd->buf = eee_allocateBuffer(EEE_TXRX_POOL,
												EEE_BUF_LARGE, NFX_COS_HIGH);
			if (!bd->buf) {
				sbm_stats.no_buf_error++;
				sbm_stats.no_buf_error_total++;
			}
			bd->control = EDESC_BD_CMD_IN_USE | EDESC_BD_LARGE;
			if (len > BUF_LEN) {
				bd->len = BUF_LEN;
				len -= BUF_LEN;
			}
			else {
				bd->len = len;
				len = 0;
			}
			memcpy(bd->buf, buf+off, bd->len);
			off += bd->len;

			num_bufs++;
			if (num_bufs == TEST_BUF_MAX_PER_DESC)
				break;
			else
				bd++;
		}
		edesc->hdr.control = (num_bufs << EDESC_BUFCNT_SHIFT);

		if (len)
			edesc->hdr.next = eee_allocateDesc(EEE_TXRX_POOL, NFX_COS_HIGH);
		else
			edesc->hdr.next = 0;
		edesc = edesc->hdr.next;
	}

	first_edesc->hdr.control |= save_len;

	return (sbm_getCODFromDesc(first_edesc, nfxMySlotId, nfxMyCPUId));
}
#endif

/*-----------------------------------------------------------------
 * Name:	sbm_testSacify()
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	7/30/01
 *
 *-----------------------------------------------------------------
 */
void
sbm_testSacify(
	uchar8	* fname,
	int32	gran
)
{
	uchar8	* buf;
	uint32	dp, sp;
	int32	len;
	int32	sacify_fd;
	uint32	sac_app_id, test_app_id;
	uint32			hdr_offset;
	int32			rc;
	nfx_cod_t		cod;
	scsi_readRspMsg_t	* scsi_msg;

#ifdef NFP_TXRX
	hdr_offset = 0;
#endif
#ifdef FCNIM
	hdr_offset = sizeof(scsi_readRspMsg_t);
#endif

	if ((buf = eee_ramAlloc(gran + hdr_offset)) == 0) {
		printf("sacify, unable to malloc buffer, %d\n", gran);
		return;
	}

	if ((sacify_fd = open(fname, O_RDONLY)) < 0) {
		printf("Error opening file %s\n", fname);
		return;
	}

	sac_app_id = eee_getApplicationId("gmc_test_rcv");
	test_app_id = eee_getApplicationId("gmc_test_rcv");

	dp = PACK_UNICAST_PORT_ADDRESS(
							1,				/* GbE NIM */
							NFX_NFP_TXRX_ANY,
							sac_app_id,
							EEE_USER_DATA,
							NFX_COS_HIGH,
							0);

	sp = PACK_UNICAST_PORT_ADDRESS(
							nfxMySlotId,
							nfxMyCPUId,
							test_app_id,
							EEE_USER_DATA,
							NFX_COS_HIGH,
							0);



	

	while ((len = read(sacify_fd, buf+hdr_offset, gran)) > 0) {

#ifdef NFP_TXRX
		cod = sbm_testCreateCodFromBuffer(buf, len+hdr_offset);
		sbm_sendCODToSac(cod);
		printf("LUC store cod 0x%llx\n", cod);
#endif
#ifdef FCNIM
		scsi_msg = (scsi_readRspMsg_t *)buf;
		scsi_msg->hdr.mid = SCSI_MID_READ_RSP,
		scsi_msg->hdr.cHandle = 0x11223344;
		scsi_msg->dataLen = len;
#ifdef USE_SAC_CHECKSUM
		scsi_msg->chksum = uint32_checksum(buf+hdr_offset, len, 0);
#else
		scsi_msg->chksum = 0;
#endif
		scsi_msg->codHandle = 0;

		rc = eee_sendMessage(buf, len+hdr_offset, dp, sp);
		if (rc != NFX_OK)
			printf("sbm_debugSacify error, rc = 0x%x\n", rc);
#endif
	}

	eee_ramDealloc(buf);

	close(sacify_fd);

}
#endif	/* NFP_TXRX | FCNIM */

#endif	/* LINUX_TEST */

