/*
 *-----------------------------------------------------------------
 *
 *
 * Name:	sbm-api.h
 *
 * RCS:  $Id: sbm-api.h,v 1.16 2002/05/07 03:01:42 rick Exp $
 *
 * Copyright (C) 2001, Agile Storage Inc.
 *
 * Description:	API for interface into SAC Buffer Management.
 *
 * Created by:	Rick Lund
 *
 * Date Created:	3/29/01
 *
 *-----------------------------------------------------------------
 */

#ifndef _SBM_API_H
#define _SBM_API_H

#define CMD_TO_MCPU_SEND_CONTROL_READ_REQ			0x0
#define CMD_TO_MCPU_SEND_DATA_TO_LUC				0x10
#define CMD_TO_MCPU_SEND_CONTROL_AND_DATA_TO_SNIM	0x20
#define CMD_TO_MCPU_SEND_CONTROL_AND_DATA_TO_SRC 	0x21
#define CMD_TO_MCPU_RETRIEVE_SAC_DATA				0x30
#define CMD_TO_MCPU_MANIPULATE_FREE					0x100
#define CMD_TO_MCPU_MANIPULATE_MERGE				0x101
#define CMD_TO_MCPU_MANIPULATE_SPLIT				0x102
#define CMD_TO_MCPU_MANIPULATE_CONCAT				0x103
#define CMD_TO_MCPU_MANIPULATE_COPY					0x104
#define CMD_TO_MCPU_MANIPULATE_CREATE				0x105
#define CMD_TO_MCPU_MANIPULATE_XOR					0x106
#define CMD_TO_MCPU_ADDRESS_OF_SAC_INFO_STRUCT		0x200

/*-----------------------------------------------------------------*/
/*
 * This command is sent to the MCPU when the FP wants to send a
 * Read Request to the SNIM.  The entire structure should fit into a single
 * system buffer.
 */
typedef struct send_control_pkt {
	uint32		cmd;
	uint32		dest_port;
	uint32		source_port;
	uint32		TID;
	uint32		msg_len;
	uchar8		* msg;
} sbm_sendControlPkt_t;

/*-----------------------------------------------------------------*/
/*
 * This command is sent to the MCPU when SAC data needs to be sent out
 * to either the data plane connection (sent from FP) or to the local
 * LUC (sent by the NIM_TX).
 */
typedef struct send_sac_data {
	uint32		cmd;
	uint32		dest_port;
	uint32		source_port;
	uint32		TID;
	nfx_cod_t	COD;
	uint32		length;
	uint32		offset;
} sbm_sendSACData_t;

/*-----------------------------------------------------------------*/
/*
 * This command is sent to the MCPU by the FP or TX, when it wants to send
 * SAC data, prepended with a control packet (for data writes to disk) to
 * either FCNIM or LUC.
 */
typedef struct send_control_and_sac_data {
	uint32		cmd;
	uint32		dest_port;
	uint32		source_port;
	uint32		TID;
	uint32		msg_len;
	uint32		length;
	nfx_cod_t	COD;
	uint32		offset;
	uint32		luc_port_id;	/* for tx to LUC. unused for tx to FCNIM */
	uchar8		* msg;
}  sbm_sendControlAndSACData_t;

/*-----------------------------------------------------------------*/
/*
 * This command is sent to the MCPU when data needs to be read from SAC
 * blocks by the GMC and written to an address on the PCI bus (metadata
 * retrieve).
 */
typedef struct retrieve_sac_data {
	uint32		cmd;
	uint32		length;
	uint32		offset;
	uint32		TID;
	nfx_cod_t	COD;
	uchar8		* dest_addr;
} sbm_retrieveSACData_t;

/*-----------------------------------------------------------------*/
/*
 * This command is sent to the MCPU to free a COD handle and associated
 * SAC blocks.
 */
typedef struct free_cod {
	uint32		cmd;
	uint32		resv;
	nfx_cod_t	COD;
} sbm_freeCOD_t;

/*-----------------------------------------------------------------*/
/*
 * This command is sent to the MCPU to merge the contents of one COD with
 * the contents of another COD.
 *
 * NOTE:  AFTER THE MERGE, THE SOURCE COD IS INVALID!!!!!!
 */
typedef struct merge_cod {
	uint32		cmd;
	uint32		src_len;
	nfx_cod_t	src_COD;
	uint32		src_offset;
	uint32		dest_offset;
	nfx_cod_t	dest_COD;
	uint32		TID;
}  sbm_mergeCOD_t;

/*-----------------------------------------------------------------*/
/*
 * This command is sent to the MCPU to split a COD.
 */
typedef struct split_cod {
	uint32		cmd;
	uint32		src_offset;
	nfx_cod_t	src_COD;
	nfx_cod_t	dest_COD;
	uint32		TID;
}  sbm_splitCOD_t;

/*-----------------------------------------------------------------*/
/*
 * This command is sent to the MCPU to concatenante two CODs into a single
 * COD.
 *
 * NOTE:  AFTER THE CONCATENATE, THE SOURCE CODs ARE INVALID!!!!!!
 */
typedef struct concat_cod {
	uint32		cmd;
	uint32		num_src_CODs;
	nfx_cod_t	dest_COD;
	nfx_cod_t	src_COD;		/* a list of these */
	uint32		TID;
}  sbm_concatCOD_t;

/*-----------------------------------------------------------------*/
/*
 * This command is sent to the MCPU to copy a COD.  Mainly for debugging.
 */
typedef struct copy_cod {
	uint32		cmd;
	uint32		src_len;
	nfx_cod_t	src_COD;
	uint32		src_offset;
	uint32		dest_offset;
	nfx_cod_t	dest_COD;
	uint32		TID;
}  sbm_copyCOD_t;

/*-----------------------------------------------------------------*/
/*
 * This command is sent to the MCPU to create a COD and zero it.
 */
typedef struct create_cod {
	uint32		cmd;
	uint32		len;
	nfx_cod_t	COD;
	uint32		fill;
	uint32		TID;
}  sbm_createCOD_t;

/*-----------------------------------------------------------------*/
/*
 * This command is sent to the MCPU to XOR the contents of two CODs into
 * a third COD.
 */
typedef struct xor_cod {
	uint32		cmd;
	uint32		num_src_CODs;
	uint32		num_bytes;
	uint32		TID;
	nfx_cod_t	dest_COD;
	uint32		dest_COD_offset;
	uint32		src_COD_offset;
	nfx_cod_t	src_COD;		/* a list of these */
}  sbm_xorCOD_t;

/*-----------------------------------------------------------------
 * Name:	sbm_freeCOD(nfx_cod_t COD)
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	4/9/01
 *
 *-----------------------------------------------------------------
 */
int32
sbm_freeCOD(
	nfx_cod_t	COD
);


/*-----------------------------------------------------------------
 * Name:	sbm_get_sac_size
 *
 * Description:
 *
 * Created by:	Jan Vanhoutte
 *
 * Date Created:	5/3/02
 *
 *-----------------------------------------------------------------
 */
uint64
sbm_get_sac_size(
    void
);




/*
 *=================================================================
 * SBM Message Types
 *=================================================================
 */
typedef enum {
	SBM_MSG_COD_RCV = SBM_MSG_BASE,
	SBM_MSG_MERGE_ACK,
	SBM_MSG_SPLIT_ACK,
	SBM_MSG_CONCAT_ACK,
	SBM_MSG_XOR_ACK
} sbm_msgType_t;

typedef struct sbm_cod_rcv {
	nfx_cod_t		cod;	/* COD							*/
	uint32			len;	/* length in bytes				*/
} sbm_cod_rcv_msg_t;

typedef struct sbm_mgmt_ack {
	uint32			TID;
	uint32			mgmt_cmd;
} sbm_mgmt_ack_msg_t;

typedef struct sbm_ack_msg
{
	sbm_msgType_t	mid;		/* Message Id					*/
	uint32			err;		/* error bits, 0 == success		*/
	union {
		sbm_cod_rcv_msg_t	cod_rcv;
		sbm_mgmt_ack_msg_t	mgmt_ack;
	} msg;
} sbm_ack_msg_t;

/*
 * sbm_sac_info_t is structure in FP memory which is written by TXRX and
 * read by FP, containing real time info about sac block utilization.
 */
typedef struct sbm_sac_i {
	uint32		num_sac_blocks_avail;
	uint32		total_sac_blocks;
	uint32		total_memory_size;
} sbm_sac_info_t;

typedef struct sbm_addr_msg {
	uint32			cmd;
	uint32			reserved;
	sbm_sac_info_t	* addr;
} sbm_infoAddrMsg_t;

extern sbm_sac_info_t	* sbm_sac_info;

#endif	/* _SBM_API_H */


