/*
 *-----------------------------------------------------------------
 *
 * Name:	sbm-init.c
 *
 * RCS:  $Id: sbm-init.c,v 1.15 2002/08/22 18:23:29 bob Exp $
 *
 * Copyright (C) 2001, Agile Storage Inc.
 *
 * Description:	Initialization functions for SAC Buffer Management.
 *				Should allocate memory structures, etc.
 *
 * Created by:	Rick Lund
 *
 * Date Created:	3/30/01
 *
 *-----------------------------------------------------------------
 */

#include <nfx-incl.h>


#ifdef SBM_REV2
#error Compiled with SBM_REV2
#endif


#include <exports/cm-api.h>
#ifdef LARGE_MEMORY
#include <exports/pte-api.h>
#endif
#include <exports/sbm-api.h>
#include "sbm.h"

sbm_config_t		sbm;
sbm_config_t		* sbm_ptr;
sbm_stats_t			* sbm_stats;
sbm_hmtEntry_t		* sbm_hmt;
sbm_bdlEntry_t		* sbm_bdl;
sbm_ifq_t			* sbm_ifq;
sbm_fbq_t			* sbm_fbq;
#ifdef LINUX_TEST
uchar8				sbm_sac[SBM_SAC_SIZE];
#else
uchar8				* sbm_sac;
#endif

sbm_sac_info_t		* sbm_sac_info;

#ifdef NFP_TXRX

/*-----------------------------------------------------------------
 * Name:	sbm_init()
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	3/30/01
 *
 *-----------------------------------------------------------------
 */
int32			/* returns	NFX_OK,
				 *			SBM_ERR_INIT_NO_MEM,
				 *			SBM_ERR_MISALIGNED_STRUCT
				 */
sbm_init(
	uint32	pass
)
{
	uint32	i;

	if (pass == 1) {
		SBM_DEBUG(("[sbm_init]:entry1\n"));

		sbm_ptr = &sbm;
		sbm_stats = &(sbm.stats);
		sbm_ifq = &(sbm.ifq);
		sbm_fbq = &(sbm.fbq);

		bzero(sbm_ptr, sizeof(sbm));

#ifdef LARGE_MEMORY
		sbm.sac_size = get_memory_region_size(MMU_SAC_REGION);
#else
		sbm.sac_size = SBM_SAC_SIZE;
#endif
		sbm.sac_block_size = SBM_SAC_BLOCK_SIZE;
		sbm.num_sac_blocks = sbm.sac_size / sbm.sac_block_size;

		/*------------------------------------------------------------------*/
		/*						HMT Initialization							*/
		/*------------------------------------------------------------------*/
		SBM_DEBUG(("[sbm_init]:hmt\n"));

		sbm.hmt = (sbm_hmtEntry_t *)eee_allocateBlock(sbm.num_sac_blocks *
													sizeof(sbm_hmtEntry_t));

		if (sbm.hmt == NULL)
			return (SBM_ERR_INIT_NO_MEM);

		sbm_hmt = sbm.hmt;
		/*------------------------------------------------------------------*/
		/*						IFQ Initialization							*/
		/*------------------------------------------------------------------*/
		SBM_DEBUG(("[sbm_init]:ifq\n"));
		sbm.ifq.start = (sbm_ifqEntry_t *)
						eee_ramAllocAlignedZero(sbm.num_sac_blocks *
													sizeof(sbm_ifqEntry_t));
		sbm.ifq.head = sbm.ifq.end = sbm.ifq.start + sbm.num_sac_blocks;
		if (sbm.ifq.start == NULL)
			return (SBM_ERR_INIT_NO_MEM);

		sbm_stats->num_free_hmt_indicies = 0;

		for (i=0; i< sbm.num_sac_blocks; i++) {
			sbm.hmt[i].bdl_index = HMT_ENTRY_IN_USE;
			sbm_freeHMTIndex(i);
		}

		/*------------------------------------------------------------------*/
		/*						BDL Initialization							*/
		/*------------------------------------------------------------------*/
		SBM_DEBUG(("[sbm_init]:bdl\n"));
		sbm.bdl = (sbm_bdlEntry_t *)eee_allocateBlock(sbm.num_sac_blocks *
													sizeof(sbm_bdlEntry_t));
		if (sbm.bdl == NULL)
			return (SBM_ERR_INIT_NO_MEM);

		sbm_bdl = sbm.bdl;
		/*------------------------------------------------------------------*/
		/*						FBQ Initialization							*/
		/*------------------------------------------------------------------*/
		SBM_DEBUG(("[sbm_init]:fbq\n"));
		sbm.fbq.start = (sbm_fbqEntry_t *)
								eee_ramAllocAlignedZero(sbm.num_sac_blocks *
													sizeof(sbm_fbqEntry_t));
		sbm.fbq.head = sbm.fbq.end = sbm.fbq.start + sbm.num_sac_blocks;
		if (sbm.fbq.start == NULL)
			return (SBM_ERR_INIT_NO_MEM);

		sbm_stats->num_sac_blocks_in_use = sbm.num_sac_blocks;
		sbm_stats->num_sac_blocks_avail = 0;

		for (i=0; i< sbm.num_sac_blocks; i++) {
			sbm.bdl[i].next_bdl_index = BDL_ENTRY_IN_USE;
			sbm_freeBDLIndex(i);
		}
		SBM_DEBUG(("[sbm_init]:leaving\n"));

		/*------------------------------------------------------------------*/
		/*						SAC Initialization							*/
		/*------------------------------------------------------------------*/
#ifdef LINUX_TEST
/*
		sbm_sac = (uchar8 *)malloc(SBM_SAC_SIZE * SBM_SAC_BLOCK_SIZE);
		if (sbm_sac == NULL) {
			printf("SBM ERROR, could not allocate SAC memory\n");
			return (SBM_ERR_INIT_NO_MEM);
		}
*/
#else
#ifdef LARGE_MEMORY
		sbm_sac = (uchar8 *)get_memory_virt_base(MMU_SAC_REGION);
#else
		sbm_sac = (uchar8 *)0x70000000;
#endif
#endif
	}
	else if (pass == 2) {
		SBM_DEBUG(("[sbm_init]:entry2\n"));
		/*------------------------------------------------------------------*/
		/*							Driver Interface						*/
		/*------------------------------------------------------------------*/

		/*------------------------------------------------------------------*/
		/*						Application Interface						*/
		/*------------------------------------------------------------------*/
		sbm.app_id = eee_allocateApplicationId("sbm");
		eee_registerReceiveFunc(sbm.app_id,
								sbm_processReceivePackets,
								sbm_processMcastReceivePackets);

		sbm.sac_app_id = eee_allocateApplicationId("sac");
        eee_registerReceiveFunc(sbm.sac_app_id, sbm_sac_rcv, sbm_sac_rcv);

		/*------------------------------------------------------------------*/
		/*						Hash Table Creation							*/
		/*------------------------------------------------------------------*/
		SBM_DEBUG(("[sbm_init]:hash\n"));
		sbm_hashInit();

		/*------------------------------------------------------------------*/
		/*						Test Function Registration					*/
		/*------------------------------------------------------------------*/
#if 0
		sbm.test_app_id = eee_allocateApplicationId("sbm_test");
		eee_registerReceiveFunc(sbm.test_app_id, sbm_test_rcv, sbm_test_rcv);
#endif

		sbm.sacify_ack_id = eee_allocateApplicationId("sacify_ack");
		eee_registerReceiveFunc(sbm.sacify_ack_id,
								sacify_ack_rcv,
								sacify_ack_rcv);

		sbm.gmc_test_rcv_id = eee_allocateApplicationId("gmc_test_rcv");
		eee_registerReceiveFunc(sbm.gmc_test_rcv_id,
								gmc_test_rcv,
								gmc_test_rcv);

		SBM_DEBUG(("[sbm_init]:leaving\n"));
	}

	return (NFX_OK);

} /* sbm_init() */

#else

int32
sbm_init(
	uint32	pass
)
{
#ifdef NFP_FP
	if (pass == 1) {
		sbm_sac_info = (sbm_sac_info_t *)eee_allocateBlock(
													sizeof(sbm_sac_info_t));
		printf("sbm_sac_info at %p\n", sbm_sac_info);
		bzero(sbm_sac_info, sizeof(sbm_sac_info_t));
	}
#endif
	if (pass == 2) {
#if 0
		sbm.test_app_id = eee_allocateApplicationId("sbm_test");
		eee_registerReceiveFunc(sbm.test_app_id, sbm_test_rcv, sbm_test_rcv);
#endif

		sbm.sacify_ack_id = eee_allocateApplicationId("sacify_ack");
		eee_registerReceiveFunc(sbm.sacify_ack_id,
								sacify_ack_rcv,
								sacify_ack_rcv);

		sbm.gmc_test_rcv_id = eee_allocateApplicationId("gmc_test_rcv");
		eee_registerReceiveFunc(sbm.gmc_test_rcv_id,
								gmc_test_rcv,
								gmc_test_rcv);
#ifdef NFP_FP
		{
			sbm_infoAddrMsg_t	* msg;
			uint32				dp, sp;

			sbm.app_id = eee_allocateApplicationId("sbm");

			dp = PACK_UNICAST_PORT_ADDRESS(	nfxMySlotId,
											NFX_NFP_TXRX_ANY,
											sbm.app_id,
											EEE_MGMT,
											NFX_COS_HIGH,
											PORT_BEST_FIT);

			sp = PACK_UNICAST_PORT_ADDRESS(	nfxMySlotId,
											NFX_NFP_TXRX_ANY,
											sbm.app_id,
											EEE_MGMT,
											NFX_COS_HIGH,
											PORT_BEST_FIT);

			msg = (sbm_infoAddrMsg_t *)eee_ramAlloc(sizeof(sbm_infoAddrMsg_t));
			msg->cmd = CMD_TO_MCPU_ADDRESS_OF_SAC_INFO_STRUCT;
			msg->addr = sbm_sac_info;

			eee_sendMessage(msg, sizeof(sbm_infoAddrMsg_t), dp, sp);
		}
#endif
	}

	return NFX_OK;

} /* sbm_init() */

#endif	/* NFP_TXRX */

