/*
 *-----------------------------------------------------------------
 *
 * Name:	gmc-rx.c
 *
 * RCS:  $Id: sbm-rx.c,v 1.15 2002/08/20 20:23:22 rick Exp $
 *
 * Copyright (C) 2001, Agile Storage Inc.
 *
 * Description:	"rx driver" for packets received by GMC
 *
 * Created by:	Rick Lund
 *
 * Date Created:	4/1/01
 *
 *-----------------------------------------------------------------
 */

#include <nfx-incl.h>

#ifdef SBM_REV2
#error Compiled with SBM_REV2
#endif

#ifdef NFP_TXRX

#include <exports/eee-api.h>
#include <exports/cm-api.h>
#include <exports/hash-api.h>
#include <exports/sbm-api.h>
#include "sbm.h"

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	2/4/02
 *
 *-----------------------------------------------------------------
 */
void
sbm_sac_rcv(
	eee_descPtr_t	edesc
)
{
	eee_descPtr_t	fp_edesc;
	uint32			hdr_len, tot_len, cod_len, num_bufs, bd_len, fp_pool, i;
	eee_descBuf_t	* bd;
	uint32			index;
	sbm_hmtEntry_t	* hmt;
	sbm_bdlEntry_t	* bdl;
	uchar8			* sac_block_addr;
	nfx_cod_t		COD;
	gmc_app_hdr_t	* gmc_app_hdr;
	uchar8			* src;
	uint32			dest_app;
	uint32			fb[EEE_MAX_BUFS];
	uchar8			* buf;
	sbm_ack_msg_t	* ack_msg;
	uint64			chksum;

	SBM_RX_DEBUG(("gmc_sacify_rcv, edesc = 0x%x\n", edesc));

#ifdef FIX_HDR_STUFF
	hdr_len = sizeof(gmc_app_hdr_t);
#else
	hdr_len = sizeof(enet_hdr_t) + sizeof(agile_hdr_t) + sizeof(gmc_app_hdr_t);
#endif

	tot_len = edesc->hdr.control & EDESC_TOTAL_LEN;
	cod_len = tot_len - hdr_len;

	SBM_RX_DEBUG(("    length = %d\n", tot_len));
	num_bufs = EEE_GET_NUM_BUFS(edesc);
	if (edesc->hdr.control & EDESC_COMMAND_USE_FIRST_BUFFER)
		bd = &(edesc->bd[0]);
	else
		bd = &(edesc->bd[1]);

	if (sbm_stats->num_sac_blocks_avail >= num_bufs) {
		index = sbm_getHMTIndex();

		hmt = (sbm_hmtEntry_t *)&(sbm_hmt[index]);

		for (i=0; i<num_bufs; i++)
			fb[i] = sbm_getBDLIndex();
	}
	else {
		sbm_stats->no_free_fbq++;
		eee_freePacket(edesc);
		return;
	}

	for (i=0; i<num_bufs; i++) {
		bdl = (sbm_bdlEntry_t *)&(sbm_bdl[fb[i]]);
		bdl->offset = 0;

		if (i==0) {
#ifdef FIX_HDR_STUFF
			gmc_app_hdr = (gmc_app_hdr_t *)(bd->buf);
#else
			gmc_app_hdr = (gmc_app_hdr_t *)(bd->buf +
									sizeof(enet_hdr_t) + sizeof(agile_hdr_t));
#endif
			bd_len = bd->len - hdr_len;
			src = bd->buf + hdr_len;
		}
		else {
			bd_len = bd->len;
			src = bd->buf;
		}

		SBM_RX_DEBUG(("    buffer at 0x%x, length %d\n", src, bd_len));

		if (bd_len > sbm.sac_block_size) {
			panic("buffer length (0x%x)> sac block size\n", bd_len);
		}
		sac_block_addr = sbm_sac + (sbm.sac_block_size * fb[i]);
		SBM_RX_DEBUG(("bdl index = %d, sac_block at 0x%x\n",
													fb[i], sac_block_addr));
		memcpy(sac_block_addr, src, bd_len);
		bdl->len = bd_len;
		bdl->next_bdl_index = fb[i+1] | BDL_ENTRY_IN_USE;

		bd++;
	}
    bdl->next_bdl_index = BDL_EOC;

	hmt->COD_len = cod_len;
	hmt->bdl_index = fb[0] | HMT_ENTRY_IN_USE;

	SBM_RX_DEBUG(("gmc_app_hdr at 0x%x\n", gmc_app_hdr));
	SBM_RX_DEBUG(("cod = 0x%llx\n", gmc_app_hdr->id.cod));
	SBM_RX_DEBUG(("control = 0x%x\n", gmc_app_hdr->control));

	/*
	 * Add new entry into hash table (COD handle <--> hmt index)
	 */
	COD = gmc_app_hdr->id.cod;
	if (sbm_insertCOD(COD, index) != NFX_OK) {
		printf("Error inserting COD 0x%llx into hash\n", COD);
	}

	/*
	 * Verify COD contents
	 */
	if (gmc_app_hdr->cod_len) {
		if (cod_len != gmc_app_hdr->cod_len) {
			printf("COD (%llx) receive len error, cod_len = 0x%x, hdr->cod_len = 0x%x\n",
											COD, cod_len, gmc_app_hdr->cod_len);

			sbm_stats->num_cods_received++;
			sbm_stats->num_cods_in_sac++;
			sbm_stats->len_verify_error++;

			sbm_freeCOD(COD);

			eee_freePacket(edesc);
			return;
		}
		sbm_stats->len_verify_ok++;
	}
	else
		sbm_stats->len_verify_skip++;

#ifdef USE_SAC_CHECKSUM
	if (gmc_app_hdr->chksum) {
		if ((chksum = (uint64)sbm_checksum_cod(COD)) != gmc_app_hdr->chksum) {
			printf("COD (%llx) chksum error, len = 0x%x, chksum = %llx, hdr->chksum = %llx\n",
									COD, cod_len, chksum, gmc_app_hdr->chksum);

			sbm_stats->num_cods_received++;
			sbm_stats->num_cods_in_sac++;
			sbm_stats->chksum_verify_error++;

			sbm_freeCOD(COD);

			eee_freePacket(edesc);
			return;
		}
		sbm_stats->chksum_verify_ok++;
	}
	else {
		sbm_stats->chksum_verify_skip++;
		chksum = (uint64)sbm_checksum_cod(COD);
	}

	hmt->chksum = chksum;
#endif


	/*
	 * Send Rcv ACK Message to dest_app on local FP
	 */
	dest_app = gmc_app_hdr->control & 0xFF;
	fp_pool = EEE_FP_POOL;
	buf = eee_allocateBuffer(fp_pool, EEE_BUF_SMALL, NFX_COS_HIGH);
	if (!buf)
		panic("couldn't allocate FP buffer\n");
	fp_edesc = eee_allocateDesc(fp_pool, NFX_COS_HIGH);
	if (!fp_edesc)
		panic("couldn't allocate FP desc\n");

	ack_msg = (sbm_ack_msg_t *)buf;
	ack_msg->mid = SBM_MSG_COD_RCV;
	ack_msg->err = 0;
	ack_msg->msg.cod_rcv.cod = COD;
	ack_msg->msg.cod_rcv.len = hmt->COD_len;

	fp_edesc->hdr.control = (1 << EDESC_BUFCNT_SHIFT) | sizeof(sbm_ack_msg_t);
	fp_edesc->hdr.dest_port = PACK_UNICAST_PORT_ADDRESS(
												nfxMySlotId,
												NFX_NFP_FP2,
												dest_app,
												EEE_CONTROL,
												NFX_COS_HIGH,
												0);
	fp_edesc->hdr.src_port = PACK_UNICAST_PORT_ADDRESS(
												nfxMySlotId,
												nfxMyCPUId,
												sbm.sac_app_id,
												EEE_CONTROL,
												NFX_COS_HIGH,
												0);
	bd = &(fp_edesc->bd[1]);
	bd->control = EDESC_BD_CMD_IN_USE | EEE_BUF_SMALL;
	bd->len = sizeof(sbm_ack_msg_t);
	bd->buf = buf;

	sbm_stats->num_cods_received++;
	sbm_stats->num_cods_in_sac++;

	eee_forwardPacket(fp_edesc);

	eee_freePacket(edesc);

}	/* sbm_sac_rcv */

#endif	/* NFP_TXRX */

