/*
 *-----------------------------------------------------------------
 *
 * Name:	sbm-util.c
 *
 * RCS:  $Id: sbm-util.c,v 1.13 2002/08/20 20:23:22 rick Exp $
 *
 * Copyright (C) 2001, Agile Storage, Inc.
 *
 * Description:	support functions
 *
 * Created by:	Rick Lund
 *
 * Date Created:	7/24/01
 *
 *-----------------------------------------------------------------
 */


#include <nfx-incl.h>

#ifdef SBM_REV2
#error Compiled with SBM_REV2
#endif

#include <exports/sbm-api.h>
#ifdef LARGE_MEMORY
#include <exports/pte-api.h>
#endif
#include "sbm-hash.h"
#include "sbm.h"

#ifdef NFP_TXRX

/*-----------------------------------------------------------------
 * Name:	sbm_GetNumBD()
 *
 * Description:	returns # of BD entries associated with the given COD,
 *				length, and offset
 *
 * Created by:	Rick Lund
 *
 * Date Created:	7/24/01
 *
 *-----------------------------------------------------------------
 */
int32
sbm_GetNumBD(
	nfx_cod_t		cod,
	uint32			len,
	uint32			off,
	uint32			* first_bdl_index,
	uint32			*first_bdl_off
)
{
	sbm_hmtEntry_t	* hmt;
	sbm_bdlEntry_t	* bdl;
	uint32			cnt = 0;
	uint32			index;
	uint32			offset, length;
	uint32			COD_low = cod;
	sbm_hash_node_t	* bucket_entry_ptr, * bptr;
	sbm_hash_node_t	node;

	SBM_TRACE(("sbm_GetNumBD(0x%llx, 0x%x, 0x%x)\n", cod, len, off));

//	bucket_entry_ptr = (sbm_hash_node_t *)eee_ramAlloc(sizeof(sbm_hash_node_t));

	bucket_entry_ptr = &node;
	bucket_entry_ptr->COD = cod;
	bptr = (sbm_hash_node_t *)
					hash_lookup(sbm.sbm_hash, COD_low, bucket_entry_ptr);
//	eee_ramDealloc(bucket_entry_ptr);
	if (!bptr) {
		printf("COD not in hash table, 0x%llx\n", cod);
		return -1;
	}

	index = bptr->hmt_index;

	hmt = &(sbm_hmt[index]);
	if (!(hmt->bdl_index & HMT_ENTRY_IN_USE))
		panic("COD not in use, 0x%llx\n", cod);
	length = len + off;
	if (length > hmt->COD_len)
		panic("COD 0x%llx size, 0x%x, too big (COD size = 0x%x)\n",
												cod, length, hmt->COD_len);
	bdl = sbm_bdl + (hmt->bdl_index & ~HMT_ENTRY_IN_USE);

	*first_bdl_index = (hmt->bdl_index & ~HMT_ENTRY_IN_USE);
	/*
	 * Count how many are needed for desired length
	 */
	while (off >= bdl->len) {
		off -= bdl->len;
		length -= bdl->len;
		*first_bdl_index = (bdl->next_bdl_index & ~BDL_ENTRY_IN_USE);
		bdl = sbm.bdl + (bdl->next_bdl_index & ~BDL_ENTRY_IN_USE);
	}
	*first_bdl_off = off;

	while (length) {
		cnt++;
		if (length > bdl->len)
			length -= bdl->len;
		else
			length = 0;
		bdl = sbm.bdl + (bdl->next_bdl_index & ~BDL_ENTRY_IN_USE);
	}

	SBM_TRACE(("sbm_GetNumBD, index = 0x%x, cnt = 0x%x\n", index, cnt));
	SBM_TRACE(("sbm_GetNumBD, first_bdl_index = 0x%x, first_bdl_offset = 0x%x\n",
										*first_bdl_index, *first_bdl_off));

	return (cnt);

}	/* sbm_GetNumBD() */

#if 0
/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:
 *
 *-----------------------------------------------------------------
 */
void
sbm_CopyBD(
	uchar8		* data_ptr,
	nfx_cod_t	cod,
	uint32		len,
	uint32		off,
	void		* end,
	void		* start
)
{
	sbm_hmtEntry_t	* hmt;
	sbm_bdlEntry_t	* bdl;
	sbm_txq_bd_t	* bd = (sbm_txq_bd_t *)data_ptr;
	uint32			index;
	uint32			length;
	uint32			COD_low = cod;
	sbm_hash_node_t	* bucket_entry_ptr, * bptr;

	bucket_entry_ptr = (sbm_hash_node_t *)eee_ramAlloc(sizeof(sbm_hash_node_t));
	bucket_entry_ptr->COD = cod;
	bptr = (sbm_hash_node_t *)
					hash_lookup(sbm.sbm_hash, COD_low, bucket_entry_ptr);
	eee_ramDealloc(bucket_entry_ptr);
	index = bptr->hmt_index;

	hmt = sbm.hmt + index;
	if (!(hmt->bdl_index & HMT_ENTRY_IN_USE))
		panic("COD not in use, 0x%llx\n", cod);
	length = len + off;
	if (length > hmt->COD_len)
		panic("COD 0x%llx size, 0x%x, too big (COD size = 0x%x)\n",
												cod, length, hmt->COD_len);
	bdl = sbm.sac_bdl + (hmt->bdl_index & ~HMT_ENTRY_IN_USE);

	while (off >= bdl->len) {
		off -= bdl->len;
		bdl = sbm.sac_bdl + (bdl->next_bdl_index & ~BDL_ENTRY_IN_USE);
	}
		
	while (len) {
		if (off == 0) {
			if (len > bdl->len) {
				bd->length = bdl->len;
				len -= bdl->len;
			}
			else {
				bd->length = len;
				len = 0;
			}
			bd->offset = bdl->offset;
		}
		else {
			if (len > bdl->len - off) {
				bd->length = bdl->len - off;
				len -= (bdl->len - off);
			}
			else {
				bd->length = len;
				len = 0;
			};
			bd->offset = bdl->offset + off;
			off = 0;
		}
		bd->sac_index = bdl - sbm.sac_bdl;

		bdl = sbm.sac_bdl + (bdl->next_bdl_index & ~BDL_ENTRY_IN_USE);

		if (++bd == end)
			bd = start;
	}

}	/* sbm_copyBD() */
#endif

/*-----------------------------------------------------------------
 * Name:	sbm_doesCODExist()
 *
 * Description:	Does a COD already exist in the hash table? If it does,
 *				return the address of the node.
 *
 * Created by:	Rick Lund
 *
 * Date Created:	7/26/01
 *
 *-----------------------------------------------------------------
 */
sbm_hash_node_t	*
sbm_doesCODExist(
	nfx_cod_t	cod
)
{
	sbm_hash_node_t     data, * bptr;
	uint32				COD_low;

	data.COD = cod;
	COD_low = cod;
	if ((bptr = (sbm_hash_node_t *)hash_lookup(sbm.sbm_hash, COD_low, &data))
																		!= NULL)
		return bptr;
	else
		return 0;
}

/*-----------------------------------------------------------------
 * Name:	sbm_insertCOD()
 *
 * Description:	Insert a new COD/hmt_index into the hash table
 *
 * Created by:	Rick Lund
 *
 * Date Created:	7/26/01
 *
 *-----------------------------------------------------------------
 */
int32
sbm_insertCOD(
	nfx_cod_t	cod,
	uint32		hmt_index
)
{
	sbm_hash_node_t	* bptr;
	uint32			COD_low;

	bptr = sbm_hashNodeStackPop();
	bptr->hmt_index = hmt_index;
	bptr->COD = cod;

	COD_low = cod;

	if (hash_insert(sbm.sbm_hash, COD_low, bptr) != NFX_OK) {
		printf("Error in Hash Insert\n");
		sbm_hashNodeStackPush(bptr);
		return -1;
	}

	SBM_UPDATE_NUM_BLOCKS;

	return NFX_OK;

}	/* sbm_insertCOD() */

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	2/4/02
 *
 *-----------------------------------------------------------------
 */
uchar8 *
sbm_getSACBlockAddr(
	uint32	index
)
{
	return &(sbm_sac[index * sbm.sac_block_size]);
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	2/4/02
 *
 *-----------------------------------------------------------------
 */
void
sbm_fill_sac_block(
	uint32	index,
	uint32	fill
)
{
	uint32	* addr;
	uint32	i;

	addr = (uint32 *)&(sbm_sac[index * sbm.sac_block_size]);

	SBM_DEBUG(("sbm_fill_sac_block, sac at %p\n", sbm_sac));
	SBM_DEBUG(("                    filling index %d at %p with 0x%x\n",
															index, addr, fill));

	for (i=0; i<sbm.sac_block_size/4; i++)
		addr[i] = fill;
}


#endif	/* NFP_TXRX */

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Jan Vanhoutte
 *
 * Date Created:	5/3/02
 *
 *-----------------------------------------------------------------
 */
uint64
sbm_get_sac_size(
    void
    )
{
#ifdef LARGE_MEMORY
	return get_memory_region_size(MMU_SAC_REGION);
#else
    return (SBM_SAC_SIZE);
#endif
};

#ifdef USE_SAC_CHECKSUM
/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	7/1/02
 *
 *-----------------------------------------------------------------
 */
uint32
sbm_checksum_cod(
	nfx_cod_t	COD
)
{
	sbm_hmtEntry_t	* hmt;
	sbm_bdlEntry_t	* bdl;
	uint32			cnt = 0;
	uint32			index, bdl_index;
	uint32			offset, length;
	sbm_hash_node_t	* bucket_entry_ptr, * bptr;
	uint32			COD_low = COD;
	uint32			chksum = 0;
	uint32			* addr;

	bucket_entry_ptr = (sbm_hash_node_t *)eee_ramAlloc(sizeof(sbm_hash_node_t));
	bucket_entry_ptr->COD = COD;
	bptr = (sbm_hash_node_t *)
					hash_lookup(sbm.sbm_hash, COD_low, bucket_entry_ptr);
	eee_ramDealloc(bucket_entry_ptr);
	if (!bptr) {
		printf("COD not in hash table, 0x%llx\n", COD);
		return -1;
	}

	index = bptr->hmt_index;

	hmt = &(sbm_hmt[index]);
	if (!(hmt->bdl_index & HMT_ENTRY_IN_USE))
		panic("COD not in use, 0x%llx\n", COD);
	length = hmt->COD_len;

	bdl_index = hmt->bdl_index & ~HMT_ENTRY_IN_USE;

	while (length) {
	    bdl = sbm_bdl + bdl_index;

		addr = &(sbm_sac[bdl_index*sbm.sac_block_size]);
		chksum = uint32_checksum(addr , bdl->len, chksum);

		if (length > bdl->len)
			length -= bdl->len;
		else
			length = 0;

	    bdl_index = bdl->next_bdl_index & ~BDL_ENTRY_IN_USE;
	}

	return chksum;
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	7/1/02
 *
 *-----------------------------------------------------------------
 */
uint32
sbm_checksum_msg(
	uchar8	* ptr,
	uint32	len
)
{
	return (uint32_checksum(ptr, len, 0));
}

#endif

