/*
 *-----------------------------------------------------------------
 *
 * Name:	sbm.h
 *
 * RCS:  $Id: sbm.h,v 1.12 2002/07/12 20:54:51 rick Exp $
 *
 * Copyright (C) 2001, Agile Storage Inc.
 *
 * Description:	local include file for SAC Buffer Management
 *
 * Created by:	Rick Lund
 *
 * Date Created:	3/30/01
 *
 *-----------------------------------------------------------------
 */

#ifndef _SBM_H
#define _SBM_H

#include <exports/eee-api.h>
#include <exports/elog-api.h>
#include "sbm-sac.h"
#include "sbm-hash.h"
#include "sbm-error.h"

#define SBM_VALIDATE		1			/* comment out for no validation */

// hardcoded applid for gmc 
// Todo: clean this up
#define SBM_TRACE(_par_) E_LOG(class_1, LOG_DEBUG, 4, 0, 0, 0, _par_)

/*
 * RAL - temporarily, just copy data into destination SAC.  The scheme
 *      below works, but if leaves some 0 length sac blocks in the ptr
 *      manipulation, and we run out of free BDL's, so a createCOD from
 *      the file system fails.  We're assuming a createCOD can NEVER fail.
 */
#define SBM_USE_TEMPORARY_COPY

/*
 * RAL - there is no GMC and no dedicated SAC memory.  SAC blocks are now
 *		just shared memory buffers.
 *		This should remove most of the memcpy's, since the "SAC" is just
 *		a mapping of COD handles to shared memory buffers.
 */
/* #define SBM_USE_SM_BUFFERS */
 
#if 0
#define SBM_DEBUG(x)			printf x
#define SBM_RX_DEBUG(x)			printf x
#define SBM_TX_DEBUG(x)			printf x
#define SBM_RESOURCE_DEBUG(x)	printf x
#else
#define SBM_DEBUG
#define SBM_RX_DEBUG
#define SBM_TX_DEBUG
#define SBM_RESOURCE_DEBUG
#endif


/*-----------------------------------------------------------------*/
/*
 * The HMT is the 'Handle Mapping Table'.  Each entry in the HMT is
 * a list of buffer descriptors, making up a single COD.
 * The overall COD length, as well as the first BD index is stored in the HMT
 * entry.
 */
typedef struct hmt_entry {
	uint32	bdl_index;
#define HMT_ENTRY_IN_USE		0x80000000
#define HMT_ENTRY_FREE			0x00000000
	uint32	COD_len;
#ifdef USE_SAC_CHECKSUM
	uint32	chksum;
#endif
} sbm_hmtEntry_t;

/*-----------------------------------------------------------------*/
/*
 * Each BDL entry maps to a SAC block.  The data inside that block can be
 * at an offset, and can have arbitrary length.  The BDL is an array with
 * next pointers that operate as a linked list.  This way, the BDL
 * entries are the RxDesc as well as the block descriptors associated with
 * the HMT.
 *
 * There is one BDL entry for each SAC block, and it is a one-to-one
 * mapping between them (i.e. index 0 in the BDL describes SAC block #0).
 */
typedef struct bdl_entry {
	ushort16	len;
	ushort16	offset;
	uint32		next_bdl_index;
#define BDL_EOC				0xFFFFFFFF
#define BDL_ENTRY_IN_USE	0x80000000
#define BDL_ENTRY_FREE		0x00000000
} sbm_bdlEntry_t;

/*-----------------------------------------------------------------*/
/*
 * Each FBQ entry is an available index into the BDL.  It is therefore also
 * an available SAC block.
 */
typedef uint32		sbm_fbqEntry_t;
#define	FBQ_GMC_OWN		0x80000000
#define FBQ_CPU_OWN		0x00000000

typedef struct fbq {
	sbm_fbqEntry_t	*start;
	sbm_fbqEntry_t	*end;
	sbm_fbqEntry_t	*head;
} sbm_fbq_t;

/*-----------------------------------------------------------------*/
/*
 * Each IFQ entry is an available HMT index.
 */
typedef uint32		sbm_ifqEntry_t;
#define	IFQ_GMC_OWN		0x80000000
#define IFQ_CPU_OWN		0x00000000

typedef struct ifq {
	sbm_ifqEntry_t	*start;
	sbm_ifqEntry_t	*end;
	sbm_ifqEntry_t	*head;
} sbm_ifq_t;


/*------------------------------------------------------------------*/
/*
 * Statistics and Debugging counters
 */
typedef struct sbm_stats {

	uint32		num_cods_in_sac;
	uint32		num_cods_received;
	uint32		num_cods_created;
	uint32		num_cods_copied;
	uint32		num_cods_freed;

	uint32		num_sac_blocks_in_use;
	uint32		num_sac_blocks_avail;

	uint32		num_free_hmt_indicies;

	uint32		no_free_ifq;
	uint32		no_free_fbq;

	uint32		len_verify_ok;
	uint32		len_verify_skip;
	uint32		len_verify_error;
#ifdef USE_SAC_CHECKSUM
	uint32		chksum_verify_ok;
	uint32		chksum_verify_skip;
	uint32		chksum_verify_error;
#endif
} sbm_stats_t;

/*------------------------------------------------------------------*/
typedef struct sbm {

	uint32				sac_size;
	uint32				sac_block_size;
	uint32				num_sac_blocks;
	
	sbm_hmtEntry_t		* hmt;

	sbm_ifq_t			ifq;

	sbm_fbq_t			fbq;

	sbm_bdlEntry_t		* bdl;

	uchar8				mac_addr[6];

	uint32				sbm_dest_port;
	uint32				sbm_source_port;
	int32				app_id;
	int32				sac_app_id;

	address_t				sbm_hash;			/* handle to hash table */
	sbm_hash_node_t			* hash_node;
	sbm_hash_node_stack_t	hash_node_stack;

	sbm_stats_t				stats;

	uint32					test_app_id;
	uint32					sacify_ack_id;
	uint32					gmc_test_rcv_id;

} sbm_config_t, *sbm_configPtr_t;

extern sbm_config_t		sbm;
extern sbm_config_t		* sbm_ptr;
extern sbm_stats_t		* sbm_stats;
extern sbm_hmtEntry_t	* sbm_hmt;
extern sbm_bdlEntry_t	* sbm_bdl;
extern sbm_ifq_t		* sbm_ifq;
extern sbm_fbq_t		* sbm_fbq;

/*
 * THE SAC
 */
#ifdef LINUX_TEST
extern uchar8			sbm_sac[];
#else
extern uchar8			* sbm_sac;
#endif

/*------------------------------------------------------------------*/

extern int32 sbm_init();

extern void sbm_freeIFQEntry(uint32 index);
extern void sbm_freeFBQEntry(uint32 index);

extern void sbm_sac_rcv(eee_descPtr_t edesc);
extern void gmc_test_rcv(eee_descPtr_t edesc);
extern void sacify_ack_rcv(eee_descPtr_t edesc);

extern void sbm_processReceivePackets(eee_descPtr_t edesc);
extern void sbm_processMcastReceivePackets(eee_descPtr_t edesc);

#define SBM_UPDATE_NUM_BLOCKS	sbm_sac_info->num_sac_blocks_avail = \
												sbm_stats->num_sac_blocks_avail;
#ifdef USE_SAC_CHECKSUM
extern uint32 sbm_checksum_cod(nfx_cod_t COD);
extern uint32 sbm_checksum_msg(uchar8 * ptr, uint32 len);
#endif

#endif	/* _SBM_H */

