/*  File Name  :	scsi-err.h
 *
 *  Description:	Contains defines, macros and structures for handling 
 *					SCSI module error and debug messages. 
 *
 *  Created    :    22 February 2001
 *
 *  Author	   :    Frank Salzmann
 *
 *  Copyright(c)  Agile Storage, Inc. 2001
 *
 */


#ifndef SCSI_ERR_H_INCLUDED

#define SCSI_ERR_H_INCLUDED

/*
 *-----------------------------
 * SCSI ERROR MESSAGE/LEVELS
 *-----------------------------
 */
#define SCSI_ERR_LEVEL_ENABLED	1
#define SCSI_ERR_LEVEL_DISABLED	0

/*
 * Number of SCSI error levels.
 */
#define SCSI_MAX_ERR_LEVEL	4

#define SCSI_ERR_LEVEL_0			0x00010000	  /* Critical Error */
#define SCSI_ERR_LEVEL_1			0x00020000
#define SCSI_ERR_LEVEL_2			0x00030000
#define SCSI_ERR_LEVEL_3			0x00040000	  /* Debug 	Error */

#define SCSI_OK						0
#define SCSI_ERR_GENERIC			0x00000001	  /* Generic Error			 */
#define SCSI_ERR_MALLOC				0x00000002	  /* Mem alloc error		 */
#define SCSI_ERR_DEVNUM_RANGE		0x00000003	  /* Dev num out of range 	 */
#define SCSI_ERR_SM_CREATE			0x00000004    /* SM create error		 */
#define SCSI_ERR_SD_ALLOC			0x00000005    /* Out of scsi descriptors */

/*
 *-----------------------------
 * SCSI DEBUG MESSAGE LEVELS
 *-----------------------------
 */
#define SCSI_MAX_DEBUG_LEVEL	4

#define SCSI_DBL_0			0	  
#define SCSI_DBL_1			1
#define SCSI_DBL_2			2
#define SCSI_DBL_3			3	 

//-- Function Prototype
int scsi_err( int errCode, char *msg ); 
int scsi_debug( char *msg, uint32 level );

#endif  /* SCSI_ERR_H_INCLUDED */
