/*  
 *------------------------------------------------------------------------------
 *  FileName   : scsi-ui.c
 *
 *  Description: SCSI application embedded user-interface functions.
 *
 *	Created    : 14 December 2001
 *
 *  Author     : Frank Salzmann
 *
 *  RCS        : $Id: scsi-ui.c,v 1.36 2002/11/23 00:30:31 nitinb Exp $
 *
 *  Copyright (c)  Agile Storage, Inc. 2001
 *------------------------------------------------------------------------------
 */			 

#include "scsi.h"

char *scsi_devType_str[] = {"DISK",      "TAPE",
							"PRINTER",   "PROCESSOR",
							"WORM",      "CDROM",
							"SCANNER",   "OPTICAL_DISK",
							"CHANGER",   "COMM_DEV",
							"PRINTER1",  "PRINTER2",
							"RAID",      "SES",
							"UNKNOWN",   "UNKNOWN1"
};

char *scsi_ui_state_str[] = {"DISABLED", "ENABLED"};

#define LEAF	0
#define BRANCH	1

#define EightMB	(8*1024*1024)

typedef struct _scsi_ui_node {
	char *name;
	int   type;				// UI_BRANCH_NODE | UI_LEAF_NODE
	int   arg_cnt;			// Min argument count for leaf node
	struct _scsi_ui_node *branch;
	int (*cmd_func)(int argc, char *argv[]);
	char *usage;
} scsi_ui_node_t;


// User-Interface Command Function Prototypes
int scsi_ui_trace(int argc, char *argv[]);
int scsi_ui_dbg_level(int argc, char *argv[]);
int scsi_ui_viewList(int argc, char *argv[]);
int scsi_ui_viewInfo(int argc, char *argv[]);
int scsi_ui_viewStatus(int argc, char *argv[]);
int scsi_ui_viewStats(int argc, char *argv[]);
int scsi_ui_clearStats(int argc, char *argv[]);
int scsi_ui_delete(int argc, char *argv[]);
int scsi_ui_viewDevStats(int argc, char *argv[]);
int scsi_ui_clearDevStats(int argc, char *argv[]);
int scsi_ui_viewHistory(int argc, char *argv[]);
int scsi_ui_debugcrash(int argc, char *argv[]);
int scsi_ui_history(int argc, char *argv[]);
int scsi_ui_usage(char *prg_name, scsi_ui_node_t *nd);
int scsi_ui_parse(char *prg_name, scsi_ui_node_t *nd, int argc, char *argv[]);
int32 scsi_ui_viewDevIoStats();
int32 scsi_ui_viewDevByteStats();
int dp83820_uiGetInput(char *);
uint64 strtoq(const char *nptr, char **endptr, int base);

int scsi_ui_write(int argc, char *argv[]);
int scsi_ui_read(int argc, char *argv[]);
int scsi_ui_stop(int argc, char *argv[]);
int scsi_ui_viewBgJobs(int argc, char *argv[]);
int scsi_ui_wrv(int argc, char *argv[]);

//-- View Stats Commands
scsi_ui_node_t scsi_ui_view_stats[] = {
  {"dev"  , LEAF,  0,  0, scsi_ui_viewDevStats, "view stats dev"},
  {"scsi" , LEAF,  0,  0, scsi_ui_viewStats, "view stats scsi"},
  {0, 0, 0, 0, 0, 0}
};

//-- View Commands
scsi_ui_node_t scsi_ui_view[] = {
  {"list",   LEAF,  1,  0, scsi_ui_viewList,  "view list <disk|switch|tape|all>"},
  {"history",LEAF,  0,  0, scsi_ui_viewHistory,"view history"},
  {"info",   LEAF,  2,  0, scsi_ui_viewInfo,  "view info <wwn> <lun>"},
  {"stats",  BRANCH,1,  scsi_ui_view_stats, 0, 0},
  {"status", LEAF,  2,  0, scsi_ui_viewStatus,"view status <wwn> <lun>"},
  {"bg_jobs",LEAF,  0,  0, scsi_ui_viewBgJobs, "view bg_jobs"},
  {0, 0, 0, 0, 0, 0}
};

//-- Clear Stats Commands
scsi_ui_node_t scsi_ui_clear_stats[] = {
  {"dev"  , LEAF,  0,  0, scsi_ui_clearDevStats, "clear stats dev"},
  {"scsi" , LEAF,  0,  0, scsi_ui_clearStats   , "clear stats scsi"},
  {0, 0, 0, 0, 0, 0}
};

//-- Clear Commands
scsi_ui_node_t scsi_ui_clear[] = {
  {"stats",   BRANCH,2,  scsi_ui_clear_stats, 0, 0},
  {0, 0, 0, 0, 0, 0}
};

//-- Top Level Command Table
scsi_ui_node_t scsi_ui_root[] = {
  {"clear",   	  BRANCH,2,  scsi_ui_clear, 0, 0},
  {"debug_crash", LEAF,  1,  0, scsi_ui_debugcrash, "debug_crash <ena|dis>"},
  {"delete",      LEAF,  2,  0, scsi_ui_delete, "delete <wwn> <lun>"},
  {"history",     LEAF,  1,  0, scsi_ui_history, "history <ena|dis>"},
  {"trace",       LEAF,  2,  0, scsi_ui_trace, "trace <ena|dis> <pkt|func>"},
  {"dbg_level",   LEAF,	 1,	 0, scsi_ui_dbg_level, "dbg_level <value>" },
  {"view",   	  BRANCH,2,  scsi_ui_view, 0, 0},
  {"read",		  LEAF,  5,  0, scsi_ui_read, "read <wwn> <lun> <lba> <size-in-KB> <number_of_reps>"},
  {"write",		  LEAF,  5,  0, scsi_ui_write, "write <wwn> <lun> <lba> <size-in-KB> <number_of_reps>"},
  {"stop",		  LEAF,  2,  0, scsi_ui_stop, "stop <wwn> <lun>"},
  {"wrv",		  LEAF,  2,  0, scsi_ui_wrv, "wrv <wwn> <lun> <lba> <size-in-KB>"},
  {0, 0, 0, 0, 0, 0}
};

int
scsi_uiAdmin( int32 argc, char **argv )
{
	if( argc == 1 ) {
		printf("\n");
		scsi_ui_usage(argv[0], &scsi_ui_root[0]);
		printf("\n");
		return(0);
	}
	scsi_ui_parse(argv[0], &scsi_ui_root[0], --argc, &argv[1]);
	return 0;

} // scsi_uiAdmin

int
scsi_ui_usage( char *prg_name, scsi_ui_node_t *nd )
{
	scsi_ui_node_t *node;

	node = nd;
	while(node->name) {
		if( node->type == LEAF )
			printf("Usage: %s %s\n", prg_name, node->usage);
		else
			scsi_ui_usage(prg_name, node->branch);
		node++;
	} // while
	return 0;
} // scsi_ui_usage

int
scsi_ui_parse(char *prg_name, scsi_ui_node_t *nd, int argc, char *argv[] )
{
	int  rc;
	char *tok;
	scsi_ui_node_t *node;
	
	node = nd;
	tok = argv[0];

	if( tok ) {
		while( node->name ) {
			if( !(strcmp(tok, node->name)) ) {
				if( node->type == LEAF ) {
					if( --argc >= node->arg_cnt ) {
						rc = node->cmd_func(argc, &argv[1]);
						if( rc < 0 )
							printf("Usage: %s %s\n", prg_name, node->usage);
					}
					else
						printf("Usage: %s %s\n", prg_name, node->usage);
				}
				else {
					if( --argc > 0 )
						scsi_ui_parse(prg_name, node->branch, argc, &argv[1]);
					else
						scsi_ui_usage(prg_name, node->branch);
				}
				return 0;
			}
			else {
				node++;
			}
		} // while
	}
	printf("Command Not Found.\n");
	return 0;
} // scsi_ui_parse

/*-----------------------------------------------------------------
 * Name: scsi_ui_trace
 *
 * Description: Allow scsi tracing to be enabled/disabled.
 *
 * Created by:  Frank Salzmann
 *
 * Date Created: December 14, 2001
 *
 *-----------------------------------------------------------------
 */
int
scsi_ui_trace(int argc, char *argv[])
{
	int	state;

	//i = strtoul(argv[0],NULL,0);
	//id = strtoq(argv[0],NULL,0); 
	
	if( argc != 2 )
		return -1;

	//-- Determine action.
	if( !strcmp("ena", argv[0]) ) 
		state = 1;
	else
	if( !strcmp("dis", argv[0]) )
		state = 0;
	else
		return -1;

	if( !strcmp("pkt", argv[1]) ) 
		scsi_pkt_trace = state;
	else
	if( !strcmp("func", argv[1]) ) 
		scsi_func_trace = state;
	else
		return -1;

	printf("SCSI: Pkt Trace: %s, Func Trace: %s\n",
		scsi_ui_state_str[scsi_pkt_trace],
		scsi_ui_state_str[scsi_func_trace]);

	return 0;
} // scsi_ui_trace

/*-----------------------------------------------------------------
 * Name: scsi_ui_dbg_level
 *
 * Description: Allow scsi debugging to be set to specified level
 *
 * Created by:  Radek Aster
 *
 * Date Created: October 23, 2002
 *
 *-----------------------------------------------------------------
 */
int 
scsi_ui_dbg_level(int argc, char *argv[])
{
	if (argc != 1)
		return -1;

	scsi_dbg_level = atol(argv[0]);

	return 0;
}

/*-----------------------------------------------------------------
 * Name: scsi_ui_viewList
 *
 * Description: Display SCSI device list.
 *
 * Created by:  Frank Salzmann
 *
 * Date Created: December 14, 2001
 *
 *-----------------------------------------------------------------
 */
int
scsi_ui_viewList(int argc, char *argv[])
{
	int	i;
	scsi_dev_t *pdev;

	printf("\n");
    printf("DEV  SCSI ID             WWN                 LUN  NAME                                           STATE\n"); 
	printf("---  ------------------  ------------------  ---  ---------------------------------------------  -----\n");

	for(i=0; i < SCSI_MAX_DEV_PER_PORT; i++) {
		if( NULL != scsiDevList[i] ) {
			pdev = scsiDevList[i];
			printf("%03d  0x%016llx  0x%016llx  %03d  %-45s  %-5s\n",
				   i,
				   pdev->id,
				   pdev->wwn,
				   pdev->lun,
				   pdev->name,
				   (( pdev->flags.fInitMode )  ? "INIT" 	:
					( pdev->flags.fOpenMode)  ? "OPEN" 	:
					( pdev->flags.fCloseMode )  ? "CLOSE" 	:
					"UNKNOWN" ));
		}
	}
	return 0;
} // scsi_ui_viewList

/*-----------------------------------------------------------------
 * Name: scsi_ui_viewInfo
 *
 * Description: Display detailed SCSI device information.
 *
 * Created by:  Frank Salzmann
 *
 * Date Created: December 14, 2001
 *
 *-----------------------------------------------------------------
 */
int
scsi_ui_viewInfo(int argc, char *argv[])
{
	int    i;
	uint64 wwn;
	uint32 lun;
	scsi_dev_t *pdev;

	wwn = strtoq(argv[0],NULL,0); 
	lun = atol(argv[1]); 

	for(i=0; i < SCSI_MAX_DEV_PER_PORT; i++) {
		if( NULL != scsiDevList[i] ) {
			pdev = scsiDevList[i];
			if( pdev->wwn == wwn && pdev->lun == lun) {
				printf("\n");
				scsi_showDevice(pdev);
				printf("\n");
				return 0;
			}
		}
	}
	printf("Device Not Found: 0x%llx:%d\n", wwn, lun);
	return 0;
} // scsi_ui_viewInfo

/*-----------------------------------------------------------------
 * Name: scsi_ui_viewStatus
 *
 * Description: Display SCSI device status information.
 *
 * Created by:  Frank Salzmann
 *
 * Date Created: December 14, 2001
 *
 *-----------------------------------------------------------------
 */
int
scsi_ui_viewStatus(int argc, char *argv[])
{
	printf("Not implemented\n");

	return 0;
} // scsi_ui_viewStatus


//-----------------------------------------------------------------
// Name: scsi_ui_viewStats
// 
// Description: Display SCSI message statistics.
//
// Created by:  Frank Salzmann
//
// Date Created: April 01, 2002
//-----------------------------------------------------------------
int
scsi_ui_viewStats(int argc, char *argv[])
{
	uint32       tmp;
	scsi_stats_t *stats;

	stats = &scsiCtrl->stats;

	printf("\n\n");
	printf("                                   SCSI Statistics: CPU: %d\n", nfxMyCPUId);
	printf("                             ======================================\n");
	printf("                  Request   Response   TotalTime   Avg TTime    FC Time    Avg FC Time              \n");
	printf("                  Count     Count        (usec)     (usec)       (usec)       (usec)        Bytes   \n");
	printf("                  -------   --------   ---------   ---------   ---------   -----------   -----------\n");
	tmp = stats->readUserRspCnt_ssc + stats->readUserRspCnt_fp;
	printf("user read(fp) ..  %7d   %8d   %9llu   %9llu   %9llu   %11llu   %11llu\n", 
	  stats->readUserReqCnt_fp, stats->readUserRspCnt_fp, 
	  eee_rtcToUsec(&stats->readUserTotalTime), 
	  tmp ? eee_rtcToUsec(&stats->readUserTotalTime)/tmp : 0,
	  eee_rtcToUsec(&stats->readUserFCTime),
	  tmp ? eee_rtcToUsec(&stats->readUserFCTime)/tmp : 0,
	  stats->readUserBytes_fp);
	printf("user read(ssc)..  %7d   %8d                                                     %11llu\n", 
		   stats->readUserReqCnt_ssc, 
		   stats->readUserRspCnt_ssc,
		   stats->readUserBytes_ssc);
	tmp =stats->readMetaRspCnt_fp + stats->readMetaRspCnt_ssc; 
	printf("meta read(fp)...  %7d   %8d   %9llu   %9llu   %9llu   %11llu   %11llu\n", 
	  stats->readMetaReqCnt_fp, stats->readMetaRspCnt_fp, 
	  eee_rtcToUsec(&stats->readMetaTotalTime), 
	  tmp ? eee_rtcToUsec(&stats->readMetaTotalTime)/tmp : 0,
	  eee_rtcToUsec(&stats->readMetaFCTime),
	  tmp ? eee_rtcToUsec(&stats->readMetaFCTime)/tmp : 0,
	  stats->readMetaBytes_fp);
	printf("meta read(ssc)..  %7d   %8d                                                     %11llu\n", 
		   stats->readMetaReqCnt_ssc, 
		   stats->readMetaRspCnt_ssc,
		   stats->readMetaBytes_ssc);
	tmp = stats->writeUserRspCnt_fp + stats->writeUserRspCnt_ssc ;
	printf("write(fp).......  %7d   %8d   %9llu   %9llu   %9llu   %11llu   %11llu\n", 
	 stats->writeUserReqCnt_fp, stats->writeUserRspCnt_fp, 
	 eee_rtcToUsec(&stats->writeUserTotalTime), 
	 tmp ? eee_rtcToUsec(&stats->writeUserTotalTime)/tmp : 0,
	 eee_rtcToUsec(&stats->writeUserFCTime),
	 tmp ? eee_rtcToUsec(&stats->writeUserFCTime)/tmp : 0,
	 stats->writeUserBytes_fp);
	printf("write(ssc)......  %7d   %8d                                                     %11llu\n", 
		   stats->writeUserReqCnt_ssc, 
		   stats->writeUserRspCnt_ssc,
		   stats->writeUserBytes_ssc);
#if 0
	printf("meta write......  %7d   %8d   %9llu   %9llu   %9llu   %11llu\n", 
	 stats->writeMetaReqCnt, stats->writeMetaRspCnt, 
	 eee_rtcToUsec(&stats->writeMetaTotalTime), 
	 stats->writeMetaRspCnt? eee_rtcToUsec(&stats->writeMetaTotalTime)/stats->writeMetaRspCnt:0,
	 eee_rtcToUsec(&stats->writeMetaFCTime),
	 stats->writeMetaRspCnt ? eee_rtcToUsec(&stats->writeMetaFCTime)/stats->writeMetaRspCnt:0);
#endif
	printf("cmd relay.......  %7d   %8d   %9llu   %9llu   %9llu   %11llu\n", 
	 stats->relayCnt, stats->relayRspCnt, 
	 eee_rtcToUsec(&stats->relayTotalTime), 
	 stats->relayRspCnt?eee_rtcToUsec(&stats->relayTotalTime)/stats->relayRspCnt:0,
	 eee_rtcToUsec(&stats->relayFCTime),
	 stats->relayRspCnt ? eee_rtcToUsec(&stats->relayFCTime)/stats->relayRspCnt:0);
	printf("open............  %7d   %8d   %9d   %9d   %9d   %11d\n", 
	 stats->openReqCnt, 0, 0, 0, 0, 0);
	printf("close...........  %7d   %8d   %9d   %9d   %9d   %11d\n", 
	 stats->closeReqCnt, 0, 0, 0, 0, 0);
	printf("sync............  %7d   %8d   %9d   %9d   %9d   %11d\n", 
	 stats->syncReqCnt, 0, 0, 0, 0, 0);
	printf("status..........  %7d   %8d   %9d   %9d   %9d   %11d\n", 
	 stats->statusReqCnt, 0, 0, 0, 0, 0);
	printf("stats...........  %7d   %8d   %9d   %9d   %9d   %11d\n", 
	 stats->statsReqCnt, 0, 0, 0, 0, 0);
	printf("iotl............  %7d   %8d   %9d   %9d   %9d   %11d\n", 
	 stats->ioctlReqCnt, 0, 0, 0, 0, 0);
	printf("delete..........  %7d   %8d   %9d   %9d   %9d   %11d\n", 
	 stats->deleteReqCnt, 0, 0, 0, 0, 0);
	printf("add.............  %7d   %8d   %9d   %9d   %9d   %11d\n", 
	 stats->addReqCnt, 0, 0, 0, 0, 0);
	printf("invalid.........  %7d   %8d   %9d   %9d   %9d   %11d\n", 
	 stats->invalidMsgCnt, 0, 0, 0, 0, 0);
	printf("\n");
#if 0
	printf("Total Message Count .......... %d\n", 
		   (stats->readUserReqCnt   + stats->readMetaReqCnt   +
			stats->writeUserReqCnt  + stats->writeMetaReqCnt  +
			stats->relayCnt         + stats->openReqCnt       +
			stats->closeReqCnt      + stats->syncReqCnt       +
			stats->statusReqCnt     + stats->statsReqCnt      +
			stats->ioctlReqCnt      + stats->deleteReqCnt     +
			stats->addReqCnt        + stats->invalidMsgCnt));
#endif
	printf("Num Messages Sent to SSC: %d\n", stats->sscMsgSentCnt);
	printf("SSC Messages Queue Size : %d\n", stats->sscMsgQueueSize);

	return 0;
} // scsi_ui_viewStats

/*-----------------------------------------------------------------
 * Name: scsi_ui_viewStatus
 *
 * Description: Display SCSI device status information.
 *
 * Created by:  Frank Salzmann
 *
 * Date Created: December 14, 2001
 *
 *-----------------------------------------------------------------
 */
int
scsi_ui_clearStats(int argc, char *argv[])
{
	scsi_stats_t *stats;

	stats = &scsiCtrl->stats;

	memset(stats, 0, sizeof(scsi_stats_t));

	printf("SCSI Statistics Cleared !\n");

	return 0;
} // scsi_ui_viewStatus

//-----------------------------------------------------------------
// Name: scsi_ui_delete
//
// Description: Delete SCSI device.
//
// Created by:  Frank Salzmann
//
// Date Created: April 11, 2002
//
//-----------------------------------------------------------------
int
scsi_ui_delete(int argc, char *argv[])
{
	int    i;
	scsi_dev_t *pdev;
    esm_event_t esmEvent;
	uint64 wwn;
	uint32 lun;

	wwn = strtoq(argv[0],NULL,0); 
	lun = atol(argv[1]); 

	for(i=0; i < SCSI_MAX_DEV_PER_PORT; i++) {
		if( NULL != scsiDevList[i] ) {
			pdev = scsiDevList[i];
			if( pdev->wwn == wwn && pdev->lun == lun) {
				esmEvent.event  = SCSI_DEVO_EVENT_DELETE;
				esmEvent.p1.u.w = 0;
				esmEvent.p2.u.w = 0;
				esm_deliverEvent(pdev->smHandle, &esmEvent);
				return 0;
			}
		}
	}
	printf("Device Not Found: 0x%llx:%d\n", wwn, lun);
	return 0;
} // scsi_ui_delete

/*
 *------------------------------------------------------------------------------
 * Function Name:	scsi_ui_viewDevStats
 *
 * Description: 	
 *
 * Created by:		Frank Salzmann
 *
 * Data Created:	April 17, 2002
 *
 * Returns:         NFX_OK | NFX_ERR 
 *
 *------------------------------------------------------------------------------
 */
int32
scsi_ui_viewDevStats(int argc, char *argv[])
{
	scsi_ui_viewDevIoStats();
	scsi_ui_viewDevByteStats();
	return NFX_OK;
} // scsi_ui_viewDevStats

/*
 *------------------------------------------------------------------------------
 * Function Name:	scsi_ui_viewDevIoStats
 *
 * Description: 	
 *
 * Created by:		Frank Salzmann
 *
 * Data Created:	April 17, 2002
 *
 * Returns:         NFX_OK | NFX_ERR 
 *
 *------------------------------------------------------------------------------
 */
int32
scsi_ui_viewDevIoStats()
{
	int32 i;
	scsi_dev_t *pdev;

	printf("\n");
	printf("                        Read    Cur Rd  Max Rd  Write   Cur Wr  Max Wr  Gen IO Q  HiPriO Q   Other Q\n"); 
	printf("WWN/LUN                 Ops     Op/Sec  Op/Sec  Ops     Op/Sec  Op/Sec   Depth     Depth      Depth \n");
	printf("----------------------  ------  ------  ------  ------  ------  ------  --------  --------  --------\n");
	for(i=0; i < SCSI_MAX_DEV_PER_PORT; i++) {
		if( NULL != scsiDevList[i] ) {
			pdev = scsiDevList[i];
			if( !pdev->flags.fInitMode )
				printf("0x%016llx:%03d  %6u  %6u  %6u  %6u  %6u  %6u  %8u  %8u  %8u\n", 
					   pdev->wwn,
					   pdev->lun,
					   pdev->readRspCnt,
					   pdev->readIoPerSecCur,
					   pdev->readIoPerSecMax,
					   pdev->writeRspCnt,
					   pdev->writeIoPerSecCur,
					   pdev->writeIoPerSecMax,
					   scsi_queue_len(pdev->genio_q),
					   scsi_queue_len(pdev->hiprio_q),
					   scsi_queue_len(pdev->other_q));
		}
	}
	printf("\n");
	return NFX_OK;
} // scsi_ui_viewDevIoStats

/*
 *------------------------------------------------------------------------------
 * Function Name:	scsi_ui_viewDevByteStats
 *
 * Description: 	
 *
 * Created by:		Frank Salzmann
 *
 * Data Created:	April 17, 2002
 *
 * Returns:         NFX_OK | NFX_ERR 
 *
 *------------------------------------------------------------------------------
 */
int32
scsi_ui_viewDevByteStats()
{
	int32 i;
	scsi_dev_t *pdev;

	printf("\n");
	printf("                        Read              Cur Rd    Max Rd    Write             Cur Wr    Max Wr  \n");
	printf("WWN/LUN                 Bytes             Byte/Sec  Byte/Sec  Bytes             Byte/Sec  Byte/Sec\n");
	printf("----------------------  ----------------  --------  --------  ----------------  --------  --------\n");
	for(i=0; i < SCSI_MAX_DEV_PER_PORT; i++) {
		if( NULL != scsiDevList[i] ) {
			pdev = scsiDevList[i];
			if( !pdev->flags.fInitMode )
				printf("0x%016llx:%03d  %16llu  %8u  %8u  %16llu  %8u  %8u\n", 
					   pdev->wwn,
					   pdev->lun,
					   pdev->readByteCnt,
					   pdev->readBytePerSecCur,
					   pdev->readBytePerSecMax,
					   pdev->writeByteCnt,
					   pdev->writeBytePerSecCur,
					   pdev->writeBytePerSecMax);
		}
	}
	printf("\n");
	return NFX_OK;
} // scsi_ui_viewDevByteStats

/*
 *------------------------------------------------------------------------------
 * Function Name:	scsi_ui_viewDevQueueStats
 *
 * Description: 	
 *
 * Created by:		Frank Salzmann
 *
 * Data Created:	June 1, 2002
 *
 * Returns:         NFX_OK | NFX_ERR 
 *
 *------------------------------------------------------------------------------
 */
int32
scsi_ui_viewDevQueueStats()
{
	int32 i;
	scsi_dev_t *pdev;

	printf("\n");
	printf("                        Gen IO Q  HiPriO Q   Other Q\n");
	printf("WWN/LUN                  Depth     Depth      Depth \n");
	printf("----------------------  --------  --------  --------\n");
	for(i=0; i < SCSI_MAX_DEV_PER_PORT; i++) {
		if( NULL != scsiDevList[i] ) {
			pdev = scsiDevList[i];
			if( !pdev->flags.fInitMode )
				printf("0x%016llx:%03d  %8u  %8u  %8u\n", 
					   pdev->wwn,
					   pdev->lun,
					   scsi_queue_len(pdev->genio_q),
					   scsi_queue_len(pdev->hiprio_q),
					   scsi_queue_len(pdev->other_q));
		}
	}
	printf("\n");
	return NFX_OK;
} // scsi_ui_viewDevQueueStats

/*
 *------------------------------------------------------------------------------
 * Function Name:	scsi_ui_clearDevStats
 *
 * Description: 	
 *
 * Created by:		Frank Salzmann
 *
 * Data Created:	April 17, 2002
 *
 * Returns:         NFX_OK | NFX_ERR 
 *
 *------------------------------------------------------------------------------
 */
int32
scsi_ui_clearDevStats(int argc, char *argv[])
{
	int32 i;
	scsi_dev_t *pdev;

	for(i=0; i < SCSI_MAX_DEV_PER_PORT; i++) {
		if( NULL != scsiDevList[i] ) {
			pdev = scsiDevList[i];
			pdev->readRspCnt         = 0;
			pdev->readRspLast        = 0;
			pdev->readIoPerSecCur    = 0;
			pdev->readIoPerSecMax    = 0;
			pdev->writeRspCnt        = 0;
			pdev->writeRspLast       = 0;
			pdev->writeIoPerSecCur   = 0;
			pdev->writeIoPerSecMax   = 0;
			pdev->readByteCnt        = 0;
			pdev->readByteLast       = 0;
			pdev->readBytePerSecCur  = 0;
			pdev->readBytePerSecMax  = 0;
			pdev->writeByteCnt       = 0;
			pdev->writeByteLast      = 0;
			pdev->writeBytePerSecCur = 0;
			pdev->writeBytePerSecMax = 0;
		}
	}
	printf("SCSI Device Statistics Cleared!\n");
	return NFX_OK;
} // scsi_ui_clearDevStats

//-----------------------------------------------------------------
// Name: scsi_ui_viewHistory
//
//Description: Display SCSI device list.
//
// Created by:  Frank Salzmann
//
// Date Created: May 23, 2002
//-----------------------------------------------------------------
int
scsi_ui_viewHistory(int argc, char *argv[])
{
	int	i, quit, count=0;
	char *op;
	scsi_history_t *sh;

	printf("\n");
    printf("ENTRY  SCSI ID           OPERATION  CHANDLE   LBA       LB CNT  \n"); 
	printf("-----  ----------------  ---------  --------  --------  --------\n");

	for(i=0; i < SCSI_MAX_HISTORY; i++) {
		sh = &scsiHistory[i];
		switch( sh->op ) {
			case SCSI_MID_READ_REQ:
				op = "READ REQ";
				break;
			case SCSI_MID_READ_RSP:
				op = "READ RSP";
				break;
			case SCSI_MID_WRITE_REQ:
				op = "WRITE REQ";
				break;
			case SCSI_MID_WRITE_RSP:
				op = "WRITE RSP";
				break;
			default:
				op = NULL;
		}
		if(op) {
			count++;
			printf("%05d  %16llx  %9s  %8x  %8d  %8d\n",
					i,
					sh->id,
			   		op,
			   		sh->cHandle,
			   		sh->lba,
			   		sh->lbCnt);
		}

		if( !(count % 64) && count != 0 ) {
			quit = dp83820_uiGetInput("Quit [99]: ");
		    if( quit == 99 )
				return 0;
		}
	}
	return 0;
} // scsi_ui_viewHistory

/*-----------------------------------------------------------------
 * Name: scsi_ui_debugcrash
 *
 * Description:
 *
 * Created by:  Frank Salzmann
 *
 * Date Created: December 14, 2001
 *
 *-----------------------------------------------------------------
 */
int
scsi_ui_debugcrash(int argc, char *argv[])
{
	int	state;

	if( argc != 1 )
		return -1;

	//-- Determine action.
	if( !strcmp("ena", argv[0]) ) 
		state = 1;
	else
	if( !strcmp("dis", argv[0]) )
		state = 0;
	else
		return -1;

	scsi_debug_drop = state;
	printf("SCSI: debug_crash: %s\n", argv[0]);

	return 0;
} // scsi_ui_debugcrash

//-----------------------------------------------------------------
// Name: scsi_ui_history
//
// Description: Allow scsi cmd history to be enabled/disabled.
//
// Created by:  Frank Salzmann
//
// Date Created: July 18, 2002
//-----------------------------------------------------------------
int
scsi_ui_history(int argc, char *argv[])
{
	if( argc != 1 )
		return -1;

	//-- Determine action.
	if( !strcmp("ena", argv[0]) ) 
		scsi_history_enable = 1;
	else
	if( !strcmp("dis", argv[0]) )
		scsi_history_enable = 0;
	else
		return -1;

	printf("SCSI: Cmd History: %s\n", (scsi_history_enable ? "ENABLED":"DISABLED"));
	return 0;
} // scsi_ui_history


//-----------------------------------------------------------------
// Name: scsi_ui_findDevice
//
// Description: Given a wwn:lun, returns its device info structure
//
// Created by:  Nitin Bahadur
//
// Date Created: Oct 14, 2002
//-----------------------------------------------------------------
scsi_dev_t*
scsi_ui_findDevice(uint64 wwn, uint32 lun)
{
	int 		i;

	for(i=0; i < SCSI_MAX_DEV_PER_PORT; i++) {
		if(scsiDevList[i] && scsiDevList[i]->wwn == wwn && scsiDevList[i]->lun == lun)
			return scsiDevList[i];
	}

	return NULL;
}

//-----------------------------------------------------------------
// Name: scsi_ui_read
//
// Description: Reads data directly from scsi
//
// Created by:  Nitin Bahadur
//
// Date Created: Oct 14, 2002
//-----------------------------------------------------------------
int
scsi_ui_read(int argc, char *argv[])
{
	scsi_dev_t		*pdev;
	uint64			wwn;
	uint32			lun;
	uint32 			lba;
	uint32			size;
	uint32 			count;

	if(argc != 5)
		return 1;

	// get parameters
	wwn = strtoq(argv[0], NULL, 0);
	lun = atol(argv[1]);
	lba = atol(argv[2]);
	size = atol(argv[3]) * 1024;
	count = atol(argv[4]);

	// get device structure
	pdev = scsi_ui_findDevice(wwn, lun);
	if(!pdev) {
		printf("Device 0x%llx:%d not found\n", wwn, lun);
		return 1;
	}

	if(size > EightMB) {
		printf("Max allowed size = %d KB\n", EightMB/1024);
		return 1;
	}

	// check if an op. is already going on
	if(pdev->hugeio_state) {
		printf("Operation in progress, device busy\n");
		return 1;
	}

	if(count <= 0) {
		printf("Rep. count must be greater than 0\n");
		return 1;
	}

	// start the read state machine now
	scsi_start_huge_io(pdev, lba, size, count, SD_OPCODE_READ, scsi_huge_io_done, NULL);
	
	printf("Read operation started\n");

	return 0;
} // scsi_ui_read


//-----------------------------------------------------------------
// Name: scsi_ui_write
//
// Description: Writes data directly to scsi
//
// Created by:  Nitin Bahadur
//
// Date Created: Oct 14, 2002
//-----------------------------------------------------------------
int
scsi_ui_write(int argc, char *argv[])
{
	scsi_dev_t		*pdev;
	uint64			wwn;
	uint32			lun;
	uint32 			lba;
	uint32			size;
	uint32 			count;

	if(argc != 5)
		return 1;

	// get parameters
	wwn = strtoq(argv[0], NULL, 0);
	lun = atol(argv[1]);
	lba = atol(argv[2]);
	size = atol(argv[3]) * 1024;
	count = atol(argv[4]);

	// get device structure
	pdev = scsi_ui_findDevice(wwn, lun);
	if(!pdev) {
		printf("Device 0x%llx:%d not found\n", wwn, lun);
		return 1;
	}

	if(size > EightMB) {
		printf("Max allowed size = %d KB\n", EightMB/1024);
		return 1;
	}

	// check if an op. is already going on
	if(pdev->hugeio_state) {
		printf("Operation in progress, device busy\n");
		return 1;
	}

	if(count <= 0) {
		printf("Rep. count must be greater than 0\n");
		return 1;
	}

	// start the write state machine now
	scsi_start_huge_io(pdev, lba, size, count, SD_OPCODE_WRITE, scsi_huge_io_done, NULL);
	
	printf("Write operation started\n");
	
	return 0;
} // scsi_ui_write


//-----------------------------------------------------------------
// Name: scsi_ui_write
//
// Description: Writes data directly to scsi
//
// Created by:  Nitin Bahadur
//
// Date Created: Oct 14, 2002
//-----------------------------------------------------------------
int
scsi_ui_wrv(int argc, char *argv[])
{
	scsi_dev_t		*pdev;
	uint64			wwn;
	uint32			lun;
	uint32 			lba;
	uint32			size;

	if(argc != 4)
		return 1;

	// get parameters
	wwn = strtoq(argv[0], NULL, 0);
	lun = atol(argv[1]);
	lba = atol(argv[2]);
	size = atol(argv[3]) * 1024;

	// get device structure
	pdev = scsi_ui_findDevice(wwn, lun);
	if(!pdev) {
		printf("Device 0x%llx:%d not found\n", wwn, lun);
		return 1;
	}

	if(size > EightMB) {
		printf("Max allowed size = %d KB\n", EightMB/1024);
		return 1;
	}

	// check if an op. is already going on
	if(pdev->hugeio_state) {
		printf("Operation in progress, device busy\n");
		return 1;
	}

	// start the write-verify state machine now
	scsi_do_write_verify(pdev, lba, size);

	return 0;
}

//-----------------------------------------------------------------
// Name: scsi_ui_stop
//
// Description: Stops an running read/write operation
//
// Created by:  Nitin Bahadur
//
// Date Created: Oct 14, 2002
//-----------------------------------------------------------------
int
scsi_ui_stop(int argc, char *argv[])
{
	scsi_dev_t	*pdev;
	uint64		wwn;
	uint32		lun;

	if(argc != 2)
		return 1;

	// get parameters
	wwn = strtoq(argv[0], NULL, 0);
	lun = atol(argv[1]);

	// get device structure
	pdev = scsi_ui_findDevice(wwn, lun);
	if(!pdev) {
		printf("Device 0x%llx:%d not found\n", wwn, lun);
		return 1;
	}

	// mark op as stopped	
	scsi_stop_huge_io(pdev);

	return 0;
} // scsi_ui_stop


//-----------------------------------------------------------------
// Name: scsi_ui_viewBgJobs
//
// Description: Displays list of running back-ground jobs
//
// Created by:  Nitin Bahadur
//
// Date Created: Oct 14, 2002
//-----------------------------------------------------------------
int 
scsi_ui_viewBgJobs(int argc, char *argv[])
{
	int			i,j;
	scsi_dev_t	*pdev;
	static char opn_string[][7] = { "READ", "WRITE", "N/A" };
	char 		*operation;

	if(argc != 0) return 1;

	printf("No    WWN                 LUN   Operation  Size    Repetitions Left\n");
	printf("----  ------------------  ----  ---------  ------  ----------------\n");
	for(i=0, j=0; i < SCSI_MAX_DEV_PER_PORT; i++) {
		pdev = scsiDevList[i];
		if(pdev && pdev->hugeio_state) {
			switch(pdev->hugeio_state->op & SD_OPCODE_MASK) {
				case SD_OPCODE_READ:
					operation = opn_string[0];
					break;
				case SD_OPCODE_WRITE:
					operation = opn_string[1];
					break;
				default:
					operation = opn_string[2];
					break;
			}
			printf("%-4d  0x%-16llx  %4d  %9s  %4dKB  %d\n",
					++j, pdev->wwn, pdev->lun, operation, 
					pdev->hugeio_state->byte_count/1024,
					pdev->hugeio_state->rep_count);
		}
	}

	return 0;
}
