/*
 *------------------------------------------------------------------------------
 *
 * Name:	scsiio-api.h
 *
 * RCS:		$Id: scsiio-api.h,v 1.5 2002/12/03 01:03:47 raster Exp $
 *
 * Copyright (C) 2001, 2002, Agile Storage Inc.
 *
 * Description:	API for code handling large SCSI IOs originated by FP processes.
 *
 * Created by:	Radek Aster
 *
 * Date Created: 3/20/02
 *
 *------------------------------------------------------------------------------
 */
#ifndef __SCSIIO_API_H__
#define __SCSIIO_API_H__

void scsiio_init(void);

/*
 ******************************************************************************
 * SCSI IO completion callback
 * Arguments: 1 - int status    = SCSI status or NFX_ERR
 *            2 - void *buf     = on WRITES, pointer to transfer data buffer, 
 *                                on READS, pointer to transfer data buffer if 
 *                                caller provided it, else points to the first 
 *                                of a chain of eDesc which span the read data.
 *            3 - uint32 len    = length of data actually written/read.
 *            4 - void * arg1   = argument passed in.
 *            5 - void * arg2   = argument passed in.
 ******************************************************************************
 */
typedef void (*scsiio_callback_t)(int, void *, uint32, void *, void *);

/*
 ******************************************************************************
 * SCSI IO tape I/O initiation functions.
 * Arguments: 1 - uint32        = SCSI handle for open device.
 *            2 - char * buf    = pointer to buffer to write or read into.
 *            3 - uint32 buflen = length of data to write/read.
 *            4 - callback      = pointer to callback function.
 *            5 - void *arg1    = argument to return to caller.
 *            6 - void *arg2    = argument to return to caller.
 *            7 - int free      = Free buffer when done (write only).
 ******************************************************************************
 */
int scsiio_read(uint32, char *, uint32, scsiio_callback_t callback, void *, void *);
int scsiio_write(uint32, char *, uint32, scsiio_callback_t callback, void *, void *, int);

/*
 ******************************************************************************
 * SCSI IO disk I/O initiation functions.
 * Arguments: 1 - uint32        = SCSI handle for open device.
 *            2 - char *buf     = on WRITES, pointer to transfer data buffer, 
 *                                on READS, pointer to transfer data buffer if 
 *                                caller provided it; if NULL a chain of e-descs
 *                                will be returned.
 *            3 - uint32 lba    = disk LBA where to transfer from/to.
 *            4 - uint32 buflen = length of data to write/read.
 *            5 - callback      = pointer to callback function.
 *            6 - void *arg1    = argument to return to caller.
 *            7 - void *arg2    = argument to return to caller.
 *            8 - int free      = Free buffer when done (write only).
 ******************************************************************************
 */
int scsiio_disk_read(uint32, char *, uint32, uint32, scsiio_callback_t callback, void *, void *);
int scsiio_disk_write(uint32, char *, uint32, uint32, scsiio_callback_t callback, void *, void *, int);

#endif /* __SCSIIO_API_H__ */









