/*
 *------------------------------------------------------------------------------
 *
 * Name:	scsiio.h
 *
 * RCS:		$Id: scsiio.h,v 1.9 2002/12/03 01:03:47 raster Exp $
 *
 * Copyright (C) 2001, 2002, Agile Storage Inc.
 *
 * Description:	Definitions for code handling large SCSI IOs originated by FP 
 *              processes.
 *
 * Created by:	Radek Aster
 *
 * Date Created: 3/20/02
 *
 *------------------------------------------------------------------------------
 */

#ifndef __SCSIIO_H__
#define __SCSIIO_H__

typedef struct {
	scsiio_callback_t	callback;
	void *			arg1;
	void *			arg2;
	int			imm_free;
	uint32			scsi_handle;
	char *			buf;
	uint32			buflen;
	uint32			lba;
	scsi_path_t 		scsi_path;
	uint32			scsi_port;
	uint32			context_id;
	uint32			transaction_id;
	uint32			max_sequence_id;
	uint32			current_sequence_id;
	uint32			current_buffer_offset;
	char *			current_buffer_pointer;
	eee_descBuf_t *		wrt_buflist;
	uint32			wrt_buflist_len;
	eee_desc_t *		rd_edesc_first;
	eee_desc_t *		rd_edesc_last;
	uint32			write;
	uint32			disk_io;
	uint32			state;
	int32			rsp_timer_handle;		/* RSP timer handle */
	int32			ack_timer_handle;		/* ACK timer handle */
	int32			exch_timer_handle;		/* EXCH timer handle */
	int32			nack_received;			/* Schedule write retransmission */
	nfx_spinlock_t		context_lock;
	int32			msg_q_size;			/* number of probable seq. in a transaction */
	eee_desc_t**		msg_q;				/* pointer to each each seq. in transaction */
} scsiio_context_t;

#define SCSIIO_CONTEXT_STATE_AWAITING_ACK	1
#define SCSIIO_CONTEXT_STATE_AWAITING_RSP	2	

/* 64 concurrent backups with 64 concurrent I/Os */
#define SCSIIO_MAX_CONTEXTS	(64 * 64)

#define SCSIIO_MAX_PACKET_DATA	8192

#define SCSIIO_RSP_TIMEOUT	(60 * 60 * 1000)
#define SCSIIO_DISK_RSP_TIMEOUT	(61 * 1000)
#define SCSIIO_ACK_TIMEOUT	(60 * 1000)
#define SCSIIO_EXCH_TIMEOUT	(30 * 1000) /* Must be smaller than ACK_TIMEOUT */

#define SCSIIO_DISK_BLOCKSIZE	512

typedef struct 
{
	uint32			scsiio_data_addr;
	int32			scsiio_app_id;
	scsiio_context_t	scsiio_contexts[SCSIIO_MAX_CONTEXTS];
	nfx_spinlock_t		scsiio_contextq_lock;
	uint32			num_reads[2];
	uint32			num_writes[2];
	uint32			next_context_id;
} scsiio_cb_t;

void  scsiio_receive_msg(eee_desc_t *);
void  scsiio_free_read_chain(scsiio_context_t *);
scsiio_context_t *scsiio_allocate_context(void);
void              scsiio_deallocate_context(scsiio_context_t *);
scsiio_context_t *scsiio_lookup_context(uint32);

int32 scsiio_transmit_read_req(scsiio_context_t *);
int32 scsiio_transmit_write_seq(scsiio_context_t *);
int32 scsiio_transmit_ack(scsiio_context_t *, int);

void  scsiio_reset_read_context(scsiio_context_t *);
void  scsiio_reset_write_context(scsiio_context_t *);

int32 scsiio_rsp_timeout_handler(int32, scsiio_context_t *);
int32 scsiio_exch_timeout_handler(int32, scsiio_context_t *);
int32 scsiio_ack_timeout_handler(int32, scsiio_context_t *);

void  scsiio_start_rsp_timer(scsiio_context_t *);
void  scsiio_stop_rsp_timer(scsiio_context_t *);
void  scsiio_start_ack_timer(scsiio_context_t *);
void  scsiio_stop_ack_timer(scsiio_context_t *);
void  scsiio_start_exch_timer(scsiio_context_t *);
void  scsiio_stop_exch_timer(scsiio_context_t *);

#endif /* __SCSIIO_H__ */
