#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/ipc.h>
#include <sys/msg.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <string.h>

#undef NULL
#include <nfx-incl.h>
#include <exports/eee-api.h>
#include <exports/scsi-api.h>
#include "../sm-scsi-test/scsi-test.h"

#include "sdm-api.h"
#include "sdm-db.h"
#include "sdm.h"
#include "sdm-conf.h"

//
//-- Global Variables
//
sdm_conf_t *sdm_conf[SDM_MAX_CFG_ENTRIES];
char confFileName[256];
boolean sdm_conf_port_filter[NFX_NUM_SLOTS_PER_CHASSIS][NFX_FCNIM_NUM_CPUS];


int sdm_parse_cnt = 3;
sdm_conf_parse_t sdm_parse_db[] = {
	{"undefined", SDM_CONF_ATTR_UNDEF,	NULL			  },
	{"mask"     , SDM_CONF_ATTR_MASK,	sdm_confParseMask },
	{"allow"    , SDM_CONF_ATTR_ALLOW,	sdm_confParseMask }
};

static int	sdm_confVerifyAllow( uint64, uint32, uint32, uint32, char * );
static int	sdm_confVerifyMask( uint64, uint32, uint32, uint32, char * );

#if 0
main() {
	sdm_confRead("sdm.conf");
	sdm_confViewRecs();
}
#endif

int
sdm_confRead( char *fileName )
{
	FILE *fd;

	strcpy(confFileName, fileName);

	fd = fopen(confFileName, "r");
	if( fd == (FILE *)NULL ) {
		//printf("SDM conf file not found: %s\n", confFileName );
		return 0;
	}
	else {
		//printf("Reading SDM conf file....\n");
		memset(sdm_conf_port_filter, 0, sizeof(sdm_conf_port_filter));
		memset(sdm_conf, 0, sizeof(sdm_conf));
		sdm_confParse(fd);
	}

	sdm_confClose(fd);
	return 0;
} // sdm_confRead

int
sdm_confWrite( char *fileName )
{
	FILE		*fp;
	sdm_conf_t*	rec;
	int 		i, j;
	char		confString[256];

	// open conf. file
	//fp = fopen(fileName, "w");
	if(!fileName)
		fp = fopen("/tmp/sdm.conf", "w");
	else
		fp = fopen(confFileName, "w");
	if( fp == (FILE *)NULL ) {
		return SDM_FAIL;
	}
	
	// write entries into it
	for(i=0; i < SDM_MAX_CFG_ENTRIES; i++) {
		if((rec = sdm_conf[i])) {
			snprintf(confString, 255, "%s\t0x%llx\t%d\tport=",
						sdm_parse_db[rec->attr.action].keyword,
						rec->wwn, rec->lun);

			// set the port mask
			if(rec->attr.port == SDM_CONF_ATTR_MAX_PORT_BIT)
				strncat(confString, "*,", 255);
			else {
				for(j=0; j < SDM_CONF_ATTR_MAX_PORT; j++)
					if(rec->attr.port & (1 << j))
						snprintf(confString, 255, "%s%d,", confString, j);
			}

			// set the slot mask
			strncat(confString, " slot=", 255);
			if(rec->attr.slot == SDM_CONF_ATTR_MAX_SLOT_BIT)
				strncat(confString, "*", 255);
			else {
				for(j=0; j < SDM_CONF_ATTR_MAX_SLOT; j++)
					if(rec->attr.slot & (1 << j))
						snprintf(confString, 255, "%s%d,", confString, j);
			}
			strncat(confString, "\n", 255);

			// write out the mask string
			fprintf(fp, "%s", confString); 
		}
	}

	fclose(fp);
	return SDM_OK;
} // sdm_confWrite

int
sdm_confClose(FILE * fd)
{
	fclose(fd);
	return(0);
} // sdm_dbClose 

#define LINE_SIZE 256

int
sdm_confParse( FILE *fd )
{
	int        		i;
	char            parse_line[LINE_SIZE];
	char			*tok;
	sdm_conf_t      *confRec;
	int             action_key;

	memset(parse_line, 0, sizeof(parse_line));
	while( fgets(parse_line, LINE_SIZE, fd) != (char *)NULL ) {

		//-- Look for action keyword.
		tok = strtok(parse_line, " ,\t");
		if( *tok == '#' )
			goto END_OF_LINE;

		for(i=0; i < sdm_parse_cnt; i++) {
			if( !strcmp(sdm_parse_db[i].keyword, tok) ) {
				action_key = sdm_parse_db[i].action_key;
				confRec = sdm_parse_db[i].parse_func(action_key);
				//-- Save config record.
				if( confRec ) 
					sdm_confSaveRec(confRec);
				break;
			}
		}
	END_OF_LINE:
		memset(parse_line, 0, sizeof(parse_line));
	} 
	return 0;
} /* sdm_confParse */

sdm_conf_t *
sdm_confParseMask( int action_key )
{
	uint64 wwn;
	uint32 lun;
	uint32 val;
	char*  tok;
	sdm_conf_t *confRec;

	//-- First argument needs to be a WWN/scsi_id_t
	tok = strtok(NULL," ,\t");

	if(*tok == '#' )
		return( (sdm_conf_t *)NULL );

	//-- Get WWN
	wwn = strtoq(tok, NULL, 16);

	//-- Get LUN
	tok = strtok(NULL," ,\t=");
	lun = strtoq(tok, NULL, 10);

	confRec = sdm_confFindRec(wwn,lun);
	if( !confRec ) {
		confRec = (sdm_conf_t *)malloc(sizeof(sdm_conf_t));
		if( confRec == NULL ) {
			printf("Error: Cannot allocate SDM configuration record.\n");
			return((sdm_conf_t *)NULL);
		}
		memset((void *)confRec, 0, sizeof(sdm_conf_t));
		confRec->wwn = wwn;
		confRec->lun = lun;
		confRec->devname[0] = '\0';
	}


	//-- Set Action Key
	confRec->attr.action = action_key;

	//-- Start off assuming that the action should be
	//-- turned off globally.
	confRec->attr.slot = 0;
	confRec->attr.port = 0;

	//-- Now look for port/slot restrictors.
	while( (tok = strtok(NULL," ,\t=")) ) {
		if( !strncmp(tok, "port", 4) ) {
			tok = strtok(NULL," ,\t=");
			if( *tok == '#' )
				goto PARSE_MASK_DONE;

			if( *tok == '*' ) {
				confRec->attr.port = SDM_CONF_ATTR_MAX_PORT_BIT;
				if( action_key == SDM_CONF_ATTR_ALLOW ) {
					sdm_conf_port_filter[2][0] = action_key; 	//slot 2 for now 
					sdm_conf_port_filter[2][1] = action_key; 	//slot 2 for now
					sdm_conf_port_filter[2][2] = action_key; 	//slot 2 for now
					sdm_conf_port_filter[2][3] = action_key; 	//slot 2 for now
				}
			}

			val = strtol(tok, NULL, 0);
			if( val < SDM_CONF_ATTR_MAX_PORT ) {
				confRec->attr.port |= (1 << val);
				if( action_key == SDM_CONF_ATTR_ALLOW ) 
					sdm_conf_port_filter[2][val] = action_key; //slot 2 for now
			}
		}
		else
		if( !strncmp(tok, "slot", 4) ) {
			tok = strtok(NULL," ,\t=");
			if( *tok == '*' )
				confRec->attr.slot = SDM_CONF_ATTR_MAX_SLOT_BIT;

			val = strtol(tok, NULL, 0);
			if( val < SDM_CONF_ATTR_MAX_SLOT )
				confRec->attr.slot |= (1 << val);
		}
	}

	/*while( (tok = strtok(NULL," ,\t=")) ) {
		if(!strncmp(tok, "port", 4)) {
			state = SDM_CONF_GET_PORT;
			continue;
		}
		else if(!strncmp(tok, "slot", 4)) {
			state = SDM_CONF_GET_SLOT;
			continue;
		}

		if( *tok == '#' )
			goto PARSE_MASK_DONE;

		if( *tok == '*' ) {
			switch(state) {
				case SDM_CONF_GET_PORT:
					confRec->attr.port = SDM_CONF_ATTR_MAX_PORT_BIT;
					if( action_key == SDM_CONF_ATTR_ALLOW ) {
						sdm_conf_port_filter[2][0] = action_key; 	//slot 2 for now 
						sdm_conf_port_filter[2][1] = action_key; 	//slot 2 for now
						sdm_conf_port_filter[2][2] = action_key; 	//slot 2 for now
						sdm_conf_port_filter[2][3] = action_key; 	//slot 2 for now
					}
					state = 0;
					break;

				case SDM_CONF_GET_SLOT:
					confRec->attr.slot = SDM_CONF_ATTR_MAX_SLOT_BIT;
					state = 0;
					break;
			}
		}
		else {
			val = strtol(tok, NULL, 10);
			switch(state) {
				case SDM_CONF_GET_PORT:
					if( val < SDM_CONF_ATTR_MAX_PORT ) {
						confRec->attr.port |= (1 << val);
						if( action_key == SDM_CONF_ATTR_ALLOW ) 
							sdm_conf_port_filter[2][val] = action_key; //slot 2 for now
					}
					break;
				case SDM_CONF_GET_SLOT:
					if( val < SDM_CONF_ATTR_MAX_SLOT )
						confRec->attr.slot |= (1 << val);
					break;
			}
		} // end of else
	} // end of while
	*/

	//printf("wwn: 0x%llx ", confRec->wwn);
	//printf("action:%d ", confRec->attr.action);
	//printf("port:%x ", confRec->attr.port);
	//printf("slot:%x", confRec->attr.slot);
	//printf("\n");
 PARSE_MASK_DONE:
	return confRec;
} // sdm_confParseMask

sdm_conf_t *
sdm_confFindRec( uint64 wwn, uint32 lun ) 
{
	int i;
	sdm_conf_t *rec;

	for(i=0; i < SDM_MAX_CFG_ENTRIES; i++) {
		rec = sdm_conf[i];
		if( rec && (rec->wwn == wwn)  && (rec->lun == lun) ) {
			//printf("Conf rec found: www: %llx, lun: %d\n", rec->wwn, rec->lun);
			return rec;
		}
	}
	return (sdm_conf_t *)NULL;
}

int
sdm_confRemRec( sdm_conf_t *rec )
{
	int i;

	for(i=0; i < SDM_MAX_CFG_ENTRIES; i++) 
		if ( sdm_conf[i] == rec) {
			sdm_conf[i] = 0;
			break;
		}

	free(rec);
	return 0;
	
} // sdm_confRemRec

int 
sdm_confSaveRec( sdm_conf_t *rec )
{
	int i;

	//-- See if an entry already exists.
	if( sdm_confFindRec(rec->wwn, rec->lun) )
		return 0;

	//-- Add to confRec list.
	for(i=0; i < SDM_MAX_CFG_ENTRIES; i++) {
		if( sdm_conf[i] == 0 ) {
			sdm_conf[i] = rec;
			break;
		}	
	}
	return 0;
}

int
sdm_confViewRecs() 
{
	int i,j;
	sdm_conf_t *rec;

	for(i=0; i < SDM_MAX_CFG_ENTRIES; i++) {
		if( (rec = sdm_conf[i]) ) {
			printf("%s: ", rec->attr.action == SDM_CONF_ATTR_MASK ? "MASK":"ALLOW");
			printf("0x%016llx/%d: ", rec->wwn, rec->lun);
			printf("Port: ");
			for(j=0; j < SDM_CONF_ATTR_MAX_PORT; j++)
				if( (1 << j)  & rec->attr.port ) {
					printf("%d ", j);
				}
			printf(" Slot: ");
			for(j=0; j < SDM_CONF_ATTR_MAX_SLOT; j++)
				if( (1 << j)  & rec->attr.slot ) {
					printf("%d ", j);
				}
			printf("\n");
		}
	}

	return 0;
} // sdm_confViewRecs


int
sdm_confVerify( uint64 wwn, uint32 lun, scsi_path_t path, char *devname )
{
	uint32 rc, slot, port;

	slot = SCSI_PATH_GET_SLOT(path);
	port = SCSI_PATH_GET_PORT(path);

	//printf("Conf Verify: id: %llx:%d, %d:%d\n", wwn, lun, slot, port);

	//-- The "allow" keyword take precidence over the "mask" keywork. 
	//-- See if an "allow" has been setup on the link. If so then
	//-- Only allow the wwn/lun matching this filter to be initialized
	//-- on this link.
	if( sdm_conf_port_filter[slot][port] == SDM_CONF_ATTR_ALLOW )
		rc = sdm_confVerifyAllow( wwn, lun, slot, port, devname );
	else
		rc = sdm_confVerifyMask( wwn, lun, slot, port, devname );

	return rc;
} // sdm_confVerify

int
sdm_confVerifyAllow( uint64 wwn, uint32 lun, uint32 slot, uint32 port, char *devname )
{
	sdm_conf_t *rec;

	//printf("Conf Verify Allow: id: %llx:%d", wwn, lun); 
	//printf("slot: %d, port: %d\n", slot, port);
	//printf("attr.slot: %x, attr.port: %x, action:%x \n", 
	//			rec->attr.slot, rec->attr.port, rec->attr.action);

	//-- See if a config entry exists for this device id.
	rec = sdm_confFindRec(wwn, lun);
	if( rec ) {

		if(strlen(rec->devname) == 0)
			strncpy(rec->devname, devname, sizeof(rec->devname));

		//printf("attr.slot: %x, attr.port: %x, action:%x \n", 
		//	   rec->attr.slot, rec->attr.port, rec->attr.action);
		if( (1 << (slot)) & rec->attr.slot ) 
			if( (1 << (port)) & rec->attr.port ) 
				if( rec->attr.action == SDM_CONF_ATTR_ALLOW ) {
					//printf("Allowed: %llx:%d, %d:%d\n", wwn,lun,slot,port);
					return 0;
				}
	}
	//printf("Not Allowed: %llx:%d, %d:%d\n", wwn,lun,slot,port);
	return -1;
} // sdm_confVerifyAllow

int
sdm_confVerifyMask( uint64 wwn, uint32 lun, uint32 slot, uint32 port, char *devname )
{
	sdm_conf_t *rec;

	//printf("Conf Verify Mask: id: %llx:%d", wwn, lun);

	//-- See if a config entry exists for this device id.
	rec = sdm_confFindRec(wwn, lun);
	if( !rec )
		return 0;

	if(strlen(rec->devname) == 0)
		strncpy(rec->devname, devname, sizeof(rec->devname));

	//printf("slot: %d, port: %d\n", slot, port);
	//printf("attr.slot: %x, attr.port: %x, action:%x \n", 
    //			rec->attr.slot, rec->attr.port, rec->attr.action);

	if( (1 << (slot)) & rec->attr.slot ) 
		if( (1 << (port)) & rec->attr.port ) 
			if( rec->attr.action == SDM_CONF_ATTR_MASK ) 
				return -1;

	//printf("Not Masked: %llx:%d, %d:%d\n", wwn,lun,slot,port);
	return 0;
} // sdm_confVerifyMask

// gets number of valid mask entries
int
sdm_confGetMaskCount()
{
	int i, count = 0;

	for(i=0; i < SDM_MAX_CFG_ENTRIES; i++) 
		if( sdm_conf[i]) count++;

	return count;
} // sdm_confGetMaskCount

// returns list of valid mask entries
int
sdm_confGetMasks(sdm_conf_t *mask_list)
{
	int i, count = 0;

	for(i=0; i < SDM_MAX_CFG_ENTRIES; i++) {
		if(sdm_conf[i]) 
			memcpy(&mask_list[count++], sdm_conf[i], sizeof(sdm_conf_t));
	}

	return 0;
} // sdm_confGetMasks
