/*  File Name  :    sdm-db.h
 *
 *  RCS        :    $Id: sdm-db.h,v 1.13 2002/11/09 00:37:35 nitinb Exp $
 *
 *  Description:    
 *
 *  Created    :    13 June 2001
 *
 *  Author     :    Frank Salzmann
 *
 *  Copyright(c)  Agile Storage, Inc. 2001
 *
 */
#ifndef __SDM_DB_H_
#define __SDM_DB_H_

typedef struct _sdm_respInfo {
	uint32 chandle;
	uint32 srcSlot;
	uint32 srcCpu;
	char   srcApp[32];
} sdm_rspInfo_t;

typedef struct _sdm_dev_db_rec
{
	scsi_id_t 	id;		 	// 64 bit record search key: scsi_id_t id  
	uint64      wwn;
	uint32      lun;
	uint32		devType;				        // SCSI device type			     
	sdm_dev_strings_t	st;		// Device strings
	uint32		interfaceType;
	uint32		lbCnt;
	uint32		lbSize;
	uint32		raidLevel;
	uint32		pathCnt;
	sdm_path_t  pathList[SDM_MAX_PATHS];
	uint32		state;
	uint32		handleCnt;
	uint32		openHandles[SDM_MAX_HANDLES_PER_DEV];	
	sdm_rspInfo_t  rspInfo;		// Used to keep track of app waiting 
								// for a response from SDM.
} sdm_devDBRec_t;

typedef struct _eagle_db_master_record
{
	char dbName[64];
	int recCount;
} eDB_mrec_t;

typedef struct _eagle_db_data_record
{
	uint64	key;
	void	*dobject;
	uint32  dobject_size;
	struct  _eagle_db_data_record *prev;
	struct  _eagle_db_data_record *next;
} eDB_drec_t;

typedef struct _eagle_db
{
	int	 		fdData;
	int	 		fdIndex;
	eDB_mrec_t 	*mrec;					/* Master Record	*/
	eDB_drec_t 	*drec_head;				/* Data record head */
	eDB_drec_t	*drec_tail;				/* Data record tail */
} eDB_t;


/* 
 *-----------------------
 * External Declarations.
 *-----------------------
 */
extern eDB_t *sdmDevDB;

eDB_t * sdm_dbInit(char *, char *);
int sdm_dbImport(eDB_t *);
int sdm_dbClose(eDB_t *);
int sdm_dbOpen(eDB_t *);
int sdm_dbWriteRec(eDB_t *, eDB_drec_t *);
int sdm_dbAddRec(eDB_t *, uint64, void *, uint32, int32);
int sdm_dbRemRec(eDB_t *, uint64);
void * sdm_dbFindRec(eDB_t *, uint64);
int sdm_dbGetDevList(eDB_t *, scsi_id_t *, uint32, uint32, char *);
int sdm_dbGetNumEntries(eDB_t *, uint32, uint32, char *);
void * sdm_dbGetNextRec(eDB_t *, uint64);
sdm_devDBRec_t * sdm_dbFindWwnLun(eDB_t *, uint64, uint32 );
sdm_devDBRec_t * sdm_dbFindDevname(eDB_t *, char * );
sdm_devDBRec_t * sdm_dbFindId(eDB_t *, scsi_id_t );

#endif
