//------------------------------------------------------------------------------
//  FileName   : sdm-init-tape.c
//
//  Description: Contains state tables and initialization functions for
//               tape devices. 
//
//	Created    : 28 March 2002
//
//  Author     : Frank Salzmann
//
//  RCS        : $Id: sdm-init-tape.c,v 1.2 2002/11/09 00:37:35 nitinb Exp $
//
//  Copyright (c)  Agile Storage, Inc. 2002
//------------------------------------------------------------------------------

#include <stdio.h>
#include <sys/types.h>
#include <sys/ipc.h>
#include <sys/msg.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <signal.h>

#undef NULL

#include <nfx-incl.h>
#include <nfx-types.h>
#include <exports/eee-api.h>  
#include <exports/scsi-api.h>
#include <exports/elog-api.h>

#include "sdm-api.h"
#include "sdm-db.h"
#include "sdm.h"
#include "sdm-init.h"


//-----------------------------------------------------------------------
//-- TAPES
//-----------------------------------------------------------------------
int sdm_initTapeGeneric(sdm_dir_t *, char *);

sdm_dist_t sdm_itGenericST[] = {
	{SDM_DIS_INIT, SDM_DIE_INIT_DEV, SDM_DIE_SN_REQ  , sdm_initTapeGeneric},
	{SDM_DIS_INIT, SDM_DIE_INIT_LUN, SDM_DIE_SN_REQ  , sdm_initTapeGeneric}, 
	{SDM_DIS_INIT, SDM_DIE_SN_REQ  , SDM_DIE_SN_RSP  , sdm_sendSnReq      },
	{SDM_DIS_INIT, SDM_DIE_SN_RSP  , SDM_DIE_FIN     , sdm_procSnRsp      },
	{SDM_DIS_INIT, SDM_DIE_FIN     , NULL            , sdm_initFinish     },
	{SDM_DIS_INIT, SDM_DIE_TO      , NULL            , sdm_initTimeout    },
	{0,0,0}
};

//-- List of supported tape devices.
sdm_devSupport_t sdm_tapeList[] = {
	{"QUANTUM" , "X231_ST336704FC", "*", sdm_itGenericST },
	{"SEGATE"  , "*"              , "*", sdm_itGenericST },
	{"*"       , "*"              , "*", sdm_itGenericST },  // Tape Catch All
	{0,0,"",0}
};

//------------------------------------------------------------------------------
// Function Name:   sdm_initTapeGeneric
// 
// Description:     
//
// Data Created:    March 16, 2002
//
// Returns:         SDM_OK
//------------------------------------------------------------------------------
int
sdm_initTapeGeneric(sdm_dir_t *dir, char *inqRsp)
{
	//printf("sdm_initTapeGeneric\n");

	//-- Create scsi id
	dir->id = sdm_initGetScsiId(dir->wwn, dir->lun, dir->st.vendor);

	//-- Perform next initialization step.
	sdm_initDevSM(dir->nextEvt, dir, NULL);

	return SDM_OK;
} // sdm_initTapeGeneric
