//------------------------------------------------------------------------------
//  FileName   : sdm-init.h
//
//  Description: Structures and definitions to support SCSI device 
//               initialization.
//
//	Created    : 28 March 2002
//
//  Author     : Frank Salzmann
//
//  RCS        : $Id: sdm-init.h,v 1.14 2002/11/09 00:37:35 nitinb Exp $
//
//  Copyright (c)  Agile Storage, Inc. 2002
//------------------------------------------------------------------------------
#ifndef __SDM_INIT_H_
#define __SDM_INIT_H_


#define SDM_DIE_MAX_LUN_PROBE  256

//
//-- Macros to create temporary id during the init process.
//
#define SDM_INIT_ID_WWN_MASK	0x0000FFFFFFFFFFFFll
#define SDM_INIT_ID_LUN_MASK	0xFFFF000000000000ll

#define SDM_INIT_ID_LUN_SHIFT	48

#define SDM_INIT_ID_SET_WWN(_id_, _x_) { \
	(_id_) &= ~SDM_INIT_ID_WWN_MASK;			  \
	(_id_) |= ((_x_) & SDM_INIT_ID_WWN_MASK); \
}

#define SDM_INIT_ID_SET_LUN(_id_, _x_)	  { \
    uint64 _xx_;							\
    _xx_ = _x_;								\
	(_id_) &= ~SDM_INIT_ID_LUN_MASK;			\
    (_id_) |= (((_xx_) << SDM_INIT_ID_LUN_SHIFT) & SDM_INIT_ID_LUN_MASK); \
}

//
//-- Device init state table structure definition.
//
typedef struct _sdm_devInit_statetable {
	int state;
	int event;
	int nextEvt;
	int (*func)();
} sdm_dist_t;

//
//-- Supported device list structure definition.
//
typedef struct _sdm_supported_dev {
    char *vendor;
    char *productId;
	char revision[5];
    sdm_dist_t  *dist;
} sdm_devSupport_t;

//
//-- Device init record structure definition.
//
typedef struct _sdm_devInitRec {
	scsi_id_t   id;
	uint64      wwn;
	uint64		chn_wwn;
	scsi_path_t path;
	uint32      lun;
	uint32      devType;
	uint32      raidLevel;
	uint32      llHandle;
	sdm_dev_strings_t st;
	sdm_dist_t *dist;

	int         state;
#define SDM_DIS_INIT		1			//-- When lun is discovered.
#define SDM_DIS_VERIFY		2			//-- Make sure lun is still there.

	int         nextEvt;
#define SDM_DIE_INIT_DEV	1			//-- Initial device evt
#define SDM_DIE_INIT_LUN	2			//-- Initial lun evt
#define SDM_DIE_RCAP_REQ	3			//-- Report Capacity Rsq evt
#define SDM_DIE_RCAP_RSP	4			//-- Report Capacity Rsp evt
#define SDM_DIE_RLUN_REQ	5			//-- Report Lun Rsq evt
#define SDM_DIE_RLUN_RSP	6			//-- Report Lun Rsp evt
#define SDM_DIE_SN_REQ		7			//-- Inquiry SN Req evt
#define SDM_DIE_SN_RSP		8			//-- Inquiry SN Rsp evt
#define SDM_DIE_TO			9			//-- Time Out Resp evt
#define SDM_DIE_FIN			10			//-- Final processing evt
#define SDM_DIE_INQ_REQ		11			//-- Send Inquiry Cmd evt
#define SDM_DIE_INQ_RSP		12			//-- Inquiry Response evt
#define SDM_DIE_RAID_REQ	13			//-- Raid Req evt
#define SDM_DIE_RAID_RSP	14			//-- Raid Rsp evt
#define SDM_DIE_TUNIT_REQ	15			//-- Test Unit Ready Req evt
#define SDM_DIE_TUNIT_RSP	16			//-- Test Unit Ready Rsp evt
#define SDM_DIE_LUNID_REQ	17			//-- Get unique lun id : LSI
#define SDM_DIE_LUNID_RSP	18			
#define SDM_DIE_ACCESS_REQ  19			//-- Get access control page : LSI
#define SDM_DIE_ACCESS_RSP  20			
#define SDM_DIE_ISSNAP_REQ  21			//-- Get snapshot base id page : LSI
#define SDM_DIE_ISSNAP_RSP  22			
#define SDM_DIE_RHW_REQ		23			//-- Hardware Version Page Req evt
#define SDM_DIE_RHW_RSP		24			//-- Hardware Version Page Rsp evt

	//-- Disk Specific Stuff
	uint32	    physicalBlockSize;
	uint32	    logicalBlockSize;
	uint32		physicalBlockCnt;
	uint32		logicalBlockCnt;

	//-- Initialization control information
	uint32		control;
#define SDM_DIE_CTRL_EVM_NO_NOTIFY	0x1
#define SDM_DIE_CTRL_INIT_ERROR		0x2

#define SDM_DIE_USER_LABEL_LEN  0x3C
#define SDM_DIE_ID_LEN          16
	//-- Vendor Specific stuff 
	union {
		struct {
			uint32 ldevId;
		} hitachi;

		struct {
			char lunid[SDM_DIE_ID_LEN];
			char lunid_str[(SDM_DIE_ID_LEN*2)+1];
			char volUsrLabel[SDM_DIE_USER_LABEL_LEN+1];
			char arrayId[SDM_DIE_ID_LEN];
			char arrayId_str[(SDM_DIE_ID_LEN*2)+1];
			char arrayUsrLabel[SDM_DIE_USER_LABEL_LEN+1];
		}lsi;

		struct {
			uint32 lunid;
		} xyratek;

	} v;
} sdm_dir_t;

//
//-- LSI Device Id Page
//
typedef struct sdminit_lsi_devIdPage {
	char devType;
	char pgCode;
	char rsv;
	char pgLen;
	char codeSet;
	char idType;
	char rsv1;
	char idLen;
	char id[16];
} sdm_lsiDevIdPg_t;

typedef struct sdminit_lsi_extDevIdPage {
	char devType;
	char pgCode;
	char rsv0;
	char pgLen;
	char pgId[4];
	char rsv1;
	char rsv2;
	char rsv3;
	char volIdLen;
	char volId[16];
	char volUsrLabelLen;
	char volUsrLabel[0x3C];
	char arrayIdLen;
	char arrayId[16];
	char arrayUsrLabelLen;
	char arrayUsrLabel[0x3C];
	char lunId;
} sdm_lsiExtDevIdPg_t;
//
//-- Device initialization function protypes.
//
int             sdm_initDevSM(int, sdm_dir_t *, void *);
int             sdm_initDevProcMsg(sdm_dir_t *, void *);
int             sdm_procNewDevMsg(sdm_msgDesc_t *);
int             sdm_procScsiAddRsp(sdm_msgDesc_t *);
int             sdm_procInqRsp(sdm_dir_t *, char *);
int             sdm_parseInqRsp(uchar8 *, uchar8 *, uchar8 *, uchar8 *, uint32 *, char *);
int             sdm_sendInqCmd(sdm_dir_t *); 
int             sdm_sendRLunCmd(sdm_dir_t *, void *);
int             sdm_sendRCapCmd(sdm_dir_t *, void *); 
int             sdm_sendSnReq(sdm_dir_t *, void *); 
int             sdm_procSnRsp(sdm_dir_t *, scsi_relayRspMsg_t *);
int             sdm_procRCap(sdm_dir_t *, scsi_relayRspMsg_t *);
int             sdm_procRLun(sdm_dir_t *, scsi_relayRspMsg_t *);
int             sdm_initFinish(sdm_dir_t *, char *);
int             sdm_initTimeout(sdm_dir_t *);
int             sdm_verifyInqRsp(sdm_dir_t *, scsi_relayRspMsg_t *);
scsi_id_t       sdm_initGetScsiId(uint64 , uint32 , char *); 
sdm_dir_t      *sdm_initNewLun(uint64, uint32, uint32, uint32);
sdm_dist_t     *sdm_getInitST(sdm_devSupport_t *, char *, char *, char *);
sdm_devDBRec_t *sdm_initCreateDevrec(sdm_dir_t *);  
int				sdm_setInitST( sdm_dir_t * );
int				sdm_createDevName(char *, char *, char *, uint32 ); 

#endif
