/*  File Name  :    sdm.h
 *
 *  RCS        :    $Id: sdm.h,v 1.23 2002/11/09 00:37:35 nitinb Exp $
 *
 *  Description:    
 *
 *  Created    :    19 June 2001
 *
 *  Author     :    Frank Salzmann
 *
 *  Copyright(c)  Agile Storage, Inc. 2001
 *
 */
#ifndef __SDM_H_
#define __SDM_H_

#define SDM_DEV_INIT			1

#define SDM_DEBUG_RX_MSG 		0
#define SDM_DEBUG_TX_MSG 		0


#define SDM_TRANSACTION_MAX				2048		// Maximum client handles.

#define SDM_SDMA_HANDLE_TIMEOUT_EVENT	9998
#define SDM_TIMEOUT_EVENT				9999		// Timeout Event

#define SDM_MAX_MPS			128

typedef struct {
	uint32	    transactionId;    	// Not related to SDM_TRANSACTION_MAX
	uint32		maxSequenceId;
	uint32		lastSequenceId;
	uint64		firstCODHandle;
	uint32		firstBufferOffset;
	byte *		firstBufP;
	byte *		lastBufP;
	uint32		totalDataLen;
} sdm_mp_desc_t;

typedef struct _sdm_transaction_desc {
	uint32     inuse;
	void      *msg;
	
	uint32     len;
	uint32	   fsendRsp;
    char       srcApp[32];
	uint32     srcCpu;
	uint32     srcSlot;	
	uint32     srcHandle;
	struct _sdm_timer      *tmr;
	struct _sdm_dev_db_rec *dev;
	int    (*callBackFunc)();
	int      callBackData;
} sdm_msgDesc_t;

typedef struct _sdm_handle {
	scsi_path_t path;
	struct _sdm_dev_db_rec *dev;
} sdm_handle_t;

typedef union _sdm_64 {
	uint64 w;
	char   b[8];	
} sdm_64_t;

//
// SDM Timer Structure.
//
typedef struct _sdm_timer {
	// uint32 time_len;				Len in seconds of the timer.
	// uint32 time_remain;			How much time is left before expiry.
    uint64 time_expire;             // When does the timer expire
	int (*callback)();				// Pointer to expiry callback function.
	int param1;						// Parameter 1 passed to callback func.
	int param2;						// Parameter 2 passed to callback func.
	struct _sdm_timer *next;		// Pointer to next timer in chain.
} sdm_tmr_t;

typedef struct _sdm_timer_head {
	int active_timers;
	sdm_tmr_t *list_start;	
	sdm_tmr_t *list_end;	
} sdm_tmr_head_t;


//
//-- SCSI Sense Data
//
typedef struct _sdm_scsi_sense {
	uchar8	rspCode:7;
	uchar8  valid  :1;
	uchar8  segNum;
	uchar8  senseKey:4;
	uchar8  resv    :1;
	uchar8  ili     :1;
	uchar8  eom     :1;
	uchar8  filemark:1;
	uchar8  info[4];
	uchar8  addSenseLen;
	uchar8  cmdInfo[4];
	uchar8  addSenseCode;
	uchar8  addSenseCodeQual;
	uchar8  unitCode;
	uchar8  bitPtr:3;
	uchar8  bpv   :1;
	uchar8  resv1 :2;
	uchar8  cd    :1;
	uchar8  sksv  :1;
	uchar8  fieldPtrMsb;
	uchar8  fieldPtrLsb;
	uchar8  addSenseBytes[1];
} sdm_scsi_sense_t;

//-- SDM Message Table
typedef struct _sdm_msg_table{
    uint32 mid;
    int    (*procFunc)();
} sdm_msgTab_t;

/*
 * External Declarations.
*/
extern sdm_msgDesc_t sdmDispatchedMsgList[SDM_TRANSACTION_MAX];
extern sdm_handle_t  devHandles[SDM_SLOT_MAX][SDM_SLOT_MAX][SDM_PORT_MAX][SDM_OPEN_HANDLE_MAX];
extern sdm_mp_desc_t smps[SDM_MAX_MPS];

// Function Prototypes
int				sdm_openTimeout( int, int );
int				sdm_sdmaHandleTimeout( int, int );
sdm_tmr_t*		sdm_timerStart( int, int (*callBack)(), int, int );
scsi_id_t		sdm_wwnToID( uint64, uint32 );
scsi_id_t		sdm_nameToID( char * );
sdm_handle_t*	sdm_getHandleEntry( uint32 );
scsi_path_t		sdm_devHandle2Path( uint32 );
int				sdm_saveHandleEntry( sdm_devDBRec_t *, scsi_path_t, uint32 );
int				sdm_releaseDevHandle( uint32 );
uint32			sdm_wwnToHandle( uint64 , uint32 );
uint32			sdm_idToHandle( scsi_id_t );
scsi_id_t		sdm_wwnToID( uint64, uint32 );
scsi_id_t		sdm_nameToID( char * );
uint32			sdm_nameToHandle( char * );
scsi_path_t		sdm_openPath( sdm_devDBRec_t *, uint32, uint32, uint32, uint32 );
int				sdm_removePath( sdm_devDBRec_t *, scsi_path_t );
scsi_path_t		sdm_getActivePath( sdm_devDBRec_t * );
sdm_path_t*		sdm_getActivePathEntry( sdm_devDBRec_t * );
int				sdm_setPathState( sdm_devDBRec_t *, scsi_path_t, uint32 );
int				sdm_devOpenSM( sdm_devDBRec_t *, uint32, void * );
void			sdm_timerProc(void);
int				sdm_timerStop( sdm_tmr_t * );
int				sdm_testMenu(void);
int				sdm_printDevAddMsg( sdm_addDevMsg_t * );
int				sdm_printDevOpenRsp( scsi_openRspMsg_t * );
int				sdm_printDevCloseRsp( scsi_closeRspMsg_t * );
int				sdm_printBuf( char *, int );

#endif
