/*
 *-----------------------------------------------------------------
 *
 * Name: wrv.c
 *
 * RCS:  
 *
 * Copyright (C) 2001, Agile Storage, Inc.
 *
 * Description: Write/Read/Verify disk test.
 *
 * Created by:	Radek Aster
 *
 * Date Created: August 8, 2001
 *
 *-----------------------------------------------------------------
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "rw_api.h"

static void print_usage(char *command)
{
	fprintf(stderr, "Usage: %s <wwn> <lun> <block_count> <dev_capacity>\n", command);
}

int main(int argc, char *argv[])
{
	int	rc = 0;
	uint64	wwn;
	uint32	lun;
	uint32	sdmHandle;
	uint	block_count;
	uint	capacity;

	if (argc < 5) {
		print_usage(argv[0]);
		exit(1);
	}
	else {
		wwn = strtoq(argv[1], NULL, 0);
		lun = strtol(argv[2], NULL, 0);
		block_count = strtoul(argv[3], 0, 10);
		capacity = strtoul(argv[4], 0, 10);
		if (block_count % 2) {
			fprintf(stderr, "%s: block_count must be a multiple of 2\n", argv[0]);
			exit(1);
		}
		sdmHandle = sdm_ui_getSdmHandle(wwn, lun);
		if (sdmHandle == 0) {
			fprintf(stderr, "%s: cannot find open handle for wwn/lun 0x%llx/%d\n", argv[0], wwn, lun);
			exit(1);
		}
		rc = disk_wrv(sdmHandle, block_count, capacity, 512);
	}
	exit (rc);
}
