/*
 *-----------------------------------------------------------------
 *
 * Name:	seep-api.c
 *
 * RCS:  $Id: seep-api.c,v 1.29 2002/12/10 23:08:08 rick Exp $
 *
 * Copyright (C) 2001, Agile Storage, Inc.
 *
 * Description:
 *
 * Created by:	ericc
 *
 * Date Created:	9/17/01
 *
 *-----------------------------------------------------------------
 */

#include <nfx-incl.h>
#include <exports/cm-api.h>
#if !defined(LINUX_TEST)
#if !defined(SSC_MGMT)
#include <exports/i2c-api.h>
#endif
#endif
#include <exports/ibutton-api.h>
#include <exports/env-api.h>
#include <exports/seep-api.h>
#include "seep.h"
#include <exports/from-api.h>
#include <exports/timer-api.h>

#ifdef PROM_R7000
#include <exports/prom-part-api.h>
#endif

#if defined (SIBYTE_BC)
//#include <sys/gt64440r.h>
#include <sys/sb1250/sb1250_defs.h>
#endif

#ifdef SSC_MGMT
#include <sys/gt64240r.h>
int32 i2c(
    uint32  op,
    uchar8  i2c_addr,
    uchar8  byte_addr,
    uint32  len,
    uchar8  * buf
);
#endif

#ifdef R9000
/* bobcat model number for 2220 in seep
 */
#define MODEL_NUM_2220 "ONS-SYS-2220"
/* Set to 1 if model number is 2220
 */
int is_BC2220 = 0;
#endif

uchar8	seep_struct[NUM_SEEPS][SEEP_TOTAL_LEN];
#ifdef NFX_SMP
nfx_spinlock_t wait_seep_lock;
#endif

#ifndef AGILE_HARDWARE
uchar8	linux_seep[SEEP_TOTAL_LEN];
#endif

int32 seep_board_timer;
void seepUptimeTimer_init();
void seep_startTimer(int32 timer);
void seep_timeout(int32 tmr_handle, void * usr_param0, void * usr_param1, void * usr_param2, void * usr_param3);

extern int32 system_uptime_timer;
void chassis_timeout(int32 tmr_handle, void * usr_param0, void * usr_param1, void * usr_param2, void * usr_param3);

#if defined (PROM_R7000) || defined (PROM_R9000)
extern uchar8   ImageVerString[];
#endif

#ifdef PROM_R7000
extern uchar8   ImageVerString[];
#define CRASHDUMP_MAX_SIZE      (16 * 1024 - PROM_PART_CB_SIZE)
#endif

#if !defined (SSC_MGMT) && !defined(SIBYTE_BC)
#if defined (FAKE_SEEP_ON_FLASH_WAR) && defined(AGILE_HARDWARE)
/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:   Rick Lund
 *
 * Date Created:   2/21/02
 *
 *-----------------------------------------------------------------
 */
static int32
fake_i2c(
    uint32  op,
    uchar8  i2c_addr,
    uchar8  byte_addr,
    uint32  len,
    uchar8  * buf
)
{
    switch (op) {
        case I2C_READ:
			NFX_SPIN_LOCK(&prom_lock);
            from_readbytes(FROM_FAKE_SEEP_OFFSET+byte_addr, buf, len);
			NFX_SPIN_UNLOCK(&prom_lock);
            break;

        case I2C_WRITE:
			NFX_SPIN_LOCK(&prom_lock);
            from_programbytes(FROM_FAKE_SEEP_OFFSET+byte_addr, buf, len);
			NFX_SPIN_UNLOCK(&prom_lock);
            break;

        default:
            printf("bad fake_i2c op, %d\n", op);
    }

    return 0;
}
#endif  // FAKE_SEEP_ON_FLASH_WAR
#endif


/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	3/2/02
 *
 *-----------------------------------------------------------------
 */
void
seepWrite(
	int32	which_seep,
	uchar8	* seep,
	uint32	offset,
	uint32	len
)
{
    if (which_seep == SEEP_PCB) {
#ifndef SIBYTE
        if (isFC())
#if defined(R7000) && defined(FAKE_SEEP_ON_FLASH_WAR)
            if (isOldFC())
                fake_i2c(I2C_WRITE, FC_I2C_ADDR, offset, len, seep);
            else
#endif  // FAKE_SEEP_ON_FLASH_WAR
            i2c(I2C_WRITE, 0, FC_I2C_ADDR, offset, len, seep);
        else if (isSSC())
            i2c(I2C_WRITE, 0, SSC_I2C_ADDR, offset, len, seep);
#else
        i2c(I2C_WRITE, 0, FP_I2C_ADDR, offset, len, seep);
#endif
	}
#if !defined (SIBYTE) && !defined (R9000)	
	else if (which_seep == SEEP_CHASSIS) {
             if (isSSC()) {
                 if (ibutton(IBUTTON_DETECT, IBUTTON_BACKPLANE, 0, 0, 0)) {
                     ibutton(IBUTTON_WRITE, IBUTTON_BACKPLANE, offset, len, seep);
                 }	
                 else {
                     i2c(I2C_WRITE, 0, SSC_I2C_ADDR, SEEP_CHASSIS_START + offset, len, seep);
                 }	
             }
        }
#endif	
#ifdef SIBYTE_BC	
        else if(which_seep == SEEP_PCB_LOCAL) {
            i2c(I2C_WRITE, 0, FP_I2C_ADDR, offset, len, seep);
        }	
#endif	
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	3/2/02
 *
 *-----------------------------------------------------------------
 */
void
seepRead(
	int32	which_seep,
	uchar8	* seep,
	uint32	offset,
	uint32	len
)
{
	if (which_seep == SEEP_PCB) {
#ifndef SIBYTE
        if (isFC()) {
#if defined(R7000) && defined(FAKE_SEEP_ON_FLASH_WAR)
            if (isOldFC())
                fake_i2c(I2C_READ, FC_I2C_ADDR, offset, len, seep);
            else
#endif  // FAKE_SEEP_ON_FLASH_WAR
            i2c(I2C_READ, 0, FC_I2C_ADDR, offset, len, seep);
        }
        else if (isSSC())
            i2c(I2C_READ, 0, SSC_I2C_ADDR, offset, len, seep);
#else

#ifdef SIBYTE_BC		
	/* We access the board seep via memory mapped registers of the Galileo */	
	memcpy((void *)seep, (void *)HW_OC_SRAM_BASE_FROM_TXRX+offset, len);
#else	
        i2c(I2C_READ, 0, FP_I2C_ADDR, offset, len, seep);
#endif	

#endif
	}

#if !defined (SIBYTE) && !defined (R9000)
	else if (which_seep == SEEP_CHASSIS) {
            if (isSSC()) {
                if (ibutton(IBUTTON_DETECT, IBUTTON_BACKPLANE, 0, 0, 0))
                    ibutton(IBUTTON_READ, IBUTTON_BACKPLANE, offset, len, seep);
                else
                    i2c(I2C_READ, 0, SSC_I2C_ADDR, SEEP_CHASSIS_START + offset, len, seep);
            }
        }
	else if (which_seep == SEEP_PS0) {
            if (isSSC()) {
                if (ibutton(IBUTTON_DETECT, IBUTTON_PS0, 0, 0, 0))
                    ibutton(IBUTTON_READ, IBUTTON_PS0, offset, len, seep);
            }
        }
	else if (which_seep == SEEP_PS1) {
            if (isSSC()) {
                if (ibutton(IBUTTON_DETECT, IBUTTON_PS1, 0, 0, 0))
                    ibutton(IBUTTON_READ, IBUTTON_PS1, offset, len, seep);
            }
        }
#endif
#ifdef SIBYTE_BC	
        else if(which_seep == SEEP_PCB_LOCAL) {
           i2c(I2C_READ, 0, FP_I2C_ADDR, offset, len, seep);
        }	
#endif	
}

/*-----------------------------------------------------------------
 * Name:	seepInit()
 *
 * Description: pull out seep contents and store locally.
 *
 * Created by:	ericc
 *
 * Date Created:	9/24/01
 *
 *-----------------------------------------------------------------
 */
void
seepInit()
{
    seepPCB_t	* seep;

#if defined (SIBYTE_BC) || defined (R9000)
    seepInit_BC();
    return;
#endif    

	NFX_SPIN_INIT(&wait_seep_lock, SPINLOCK_LEVEL_UNKNOWN);

    /*read in seep info struct from seep*/
	if (isFC() || isTXRX() || isFP() || isSSC()) 
	{
            seep = (seepPCB_t *)seep_struct[SEEP_PCB];
            NFX_SPIN_LOCK(&wait_seep_lock);
            seepRead(SEEP_PCB, (void *)seep, 0, sizeof(seepPCB_t));
            NFX_SPIN_UNLOCK(&wait_seep_lock);

            if(isSSC())
            {		
                uint32  noOfReboots = 0;
                uint32 i=0;
                /* Start the timers that keep track of board life */
	        seepUptimeTimer_init();

	        /* Update the No of reboots field */
	        for(i=0;i<SEEP_PCB_REBOOTNUM_LEN;i++)
	            noOfReboots |= seep->no_of_reboots[i] & 0xff << 8*i;

	        noOfReboots ++;

       	        NFX_SPIN_LOCK(&wait_seep_lock);
                seepWrite(SEEP_PCB, (uchar8 *)&noOfReboots, 0x79, SEEP_PCB_REBOOTNUM_LEN-1);
       	        NFX_SPIN_UNLOCK(&wait_seep_lock);

            }	
            if (strcmp(seep->signature, SEEP_SIG_PCB)) {
                memset((void *)seep, 0, sizeof(seepPCB_t));
                strcpy(seep->signature, SEEP_SIG_PCB);

                NFX_SPIN_LOCK(&wait_seep_lock);
                seepWrite(SEEP_PCB, (void *)seep, 0, sizeof(seepPCB_t));
                NFX_SPIN_UNLOCK(&wait_seep_lock);

                seepenv_reset();
            }
	}
	if (isSSC()) {
#if !defined (PROM_R9000) && !defined (PROM_SIBYTE_BC)	
		if (ibutton(IBUTTON_DETECT, IBUTTON_BACKPLANE, 0, 0, 0)) {
	        seep = (seepPCB_t *)seep_struct[SEEP_CHASSIS];
			seepRead(SEEP_CHASSIS, (void *)seep, 0, sizeof(seepChassis_t));
		}
		else {
	        seep = (seepPCB_t *)seep_struct[SEEP_CHASSIS];
	        seepRead(SEEP_CHASSIS, (void *)seep, SEEP_CHASSIS_START,
									sizeof(seepOldChassis_t));
		}

		if (ibutton(IBUTTON_DETECT, IBUTTON_PS0, 0, 0, 0)) {
	        seep = (seepPCB_t *)seep_struct[SEEP_PS0];
			seepRead(SEEP_PS0, (void *)seep, 0, sizeof(seepPowerSupply_t));
		}
		else
			printf("PS0 ibutton not present\n");
		if (ibutton(IBUTTON_DETECT, IBUTTON_PS1, 0, 0, 0)) {
	        seep = (seepPCB_t *)seep_struct[SEEP_PS1];
			seepRead(SEEP_PS1, (void *)seep, 0, sizeof(seepPowerSupply_t));
		}
		else
			printf("PS1 ibutton not present\n");
#endif

#ifdef PROM_R7000
		/*
		 * Copy Prom Version string to the big flash
		 */  
		from_programbytes(FROM_CRASHDUMP_OFFSET+CRASHDUMP_MAX_SIZE, ImageVerString, CM_VER_STRING_LEN);
#endif		
	}

#ifdef LINUX_TEST
	seepenv_reset();
#endif
}

#if defined (SIBYTE_BC) || defined (R9000)
/*-----------------------------------------------------------------
 * Name:	seepInit_BC()
 *
 * Description: pull out seep contents and store locally for Bobcat.
 *
 * Created by:	Lipika Bhattacharya
 *
 * Date Created: 7/01/04
 *
 *-----------------------------------------------------------------
 */
void
seepInit_BC()
{
    seepPCB_t	* seep;

    NFX_SPIN_INIT(&wait_seep_lock, SPINLOCK_LEVEL_UNKNOWN);

    /*read in seep info struct from seep*/
    if (isFC() || isSSC() || isTXRX() || isFP()) 
    {
        seep = (seepPCB_t *)seep_struct[SEEP_PCB];
        NFX_SPIN_LOCK(&wait_seep_lock);
        seepRead(SEEP_PCB, (void *)seep, 0, sizeof(seepPCB_t));
        NFX_SPIN_UNLOCK(&wait_seep_lock);

#if defined(R9000)
        /* If true mark this system as Bobcat 2220, which
         * only has 2 FP ports.
         */
        if (strcmp(seep->model_num, MODEL_NUM_2220) == 0) {
            is_BC2220 = 1;
        }
#endif

#ifndef PROM	
	// We chose the FC to be in control of the board uptime feature for convenience.
	if(isFC()) {
            /* Only in runtime: Start the timers that keep track of board life */
	    seepUptimeTimer_init();
	}
#endif    	

#ifdef R7000	
        if(isSSC())  // Only SSC controls the no of reboots while in PROM mode.
        {		
#ifdef PROM	
            uint32 i=0;
            uint32  noOfReboots = 0;

            /* Update the No of reboots field */
	    for(i=0;i<SEEP_PCB_REBOOTNUM_LEN;i++)
	        noOfReboots |= seep->no_of_reboots[i] & 0xff << 8*i;

	    noOfReboots ++;
	    memcpy(seep->no_of_reboots, &noOfReboots, sizeof(uchar8)*SEEP_PCB_REBOOTNUM_LEN);
       	    NFX_SPIN_LOCK(&wait_seep_lock);
            seepWrite(SEEP_PCB, (void *)seep, 0, sizeof(seepPCB_t));
       	    NFX_SPIN_UNLOCK(&wait_seep_lock);

		// Copy SEEP info into GT SRAM for TXRX/FP to pickup
		memcpy((void *)OC_SRAM_BASE, seep, sizeof(seepPCB_t));
#endif	    
            if (strcmp(seep->signature, SEEP_SIG_PCB)) {
                memset((void *)seep, 0, sizeof(seepPCB_t));
                strcpy(seep->signature, SEEP_SIG_PCB);

                NFX_SPIN_LOCK(&wait_seep_lock);
                seepWrite(SEEP_PCB, (void *)seep, 0, sizeof(seepPCB_t));
                NFX_SPIN_UNLOCK(&wait_seep_lock);

                seepenv_reset();
            }
	}
#endif	
    }	
#ifdef SIBYTE_BC    
    if (isTXRX() || isFP())
    {
	// The LOCAL_SEEP is the processor seep at address 0x50 on the TXRX/FP.    
	    
        seepPCB_t	* seep_local = (seepPCB_t *)seep_struct[SEEP_PCB_LOCAL];

        NFX_SPIN_LOCK(&wait_seep_lock);
        seepRead(SEEP_PCB_LOCAL, (void *)seep_local, 0, sizeof(seepPCB_t));
        NFX_SPIN_UNLOCK(&wait_seep_lock);

        if (strcmp(seep_local->signature, SEEP_SIG_PCB)) {
            memset((void *)seep_local, 0, sizeof(seepPCB_t));
            strcpy(seep_local->signature, SEEP_SIG_PCB);

            NFX_SPIN_LOCK(&wait_seep_lock);
            seepWrite(SEEP_PCB_LOCAL, (void *)seep_local, 0, sizeof(seepPCB_t));
            NFX_SPIN_UNLOCK(&wait_seep_lock);

            seepenv_reset();
        }
    }	
#endif    
	if (isSSC()) {
#if defined (PROM_R7000) && !defined (PROM_R9000)		
		/*
		 * Copy Prom Version string to the big flash
		 */  
		from_programbytes(FROM_CRASHDUMP_OFFSET+CRASHDUMP_MAX_SIZE, ImageVerString, CM_VER_STRING_LEN);
#elif defined (PROM_R9000)		
		/*
		 * Copy Prom Version string to the board seep.
		 */  
                seepWrite(SEEP_PCB, ImageVerString, SEEP_NVRAM_START, SEEP_VER_STRING_LEN_SIZE);
#endif		
	}
}
#endif

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	1/19/02
 *
 *-----------------------------------------------------------------
 */
void
seepGetIpAddr(
	uchar8	* ipaddr
)
{
	seepPCB_t	* seep;
	uchar8		* seep_str;

	seep = (seepPCB_t *)seep_struct[SEEP_PCB];
	seep_str = seep->ip_addr;

#ifdef PROM_R7000
	if(isSSC()) {
	    if (strlen(seep_str) == 0)
		memcpy(seep_str, "1.2.3.4\0", 15);
	}	
#endif	
	
//	printf("debug IP addr = %s\n", seep_str);

	memcpy(ipaddr, seep_str, 15);
}

void
seepGetTFTPServer(uint8 *ipaddr)
{
    seepPCB_t	* seep;
    uchar8		* seep_str;

    seep = (seepPCB_t *)seep_struct[SEEP_PCB];
    seep_str = seep->uptime_since_reboot;

#ifdef PROM_R7000
    if(isSSC()) {
        if (strlen(seep_str) == 0)
        memcpy(seep_str, "1.2.3.4\0", 15);
    }	
#endif	

//	printf("debug IP addr = %s\n", seep_str);

    sprintf(ipaddr, "%d.%d.%d.%d", seep_str[0], seep_str[1], seep_str[2],
            seep_str[3]);
}


/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	4/15/02
 *
 *-----------------------------------------------------------------
 */
void
seepGetIpMask(
	uchar8	* ipmask
)
{
	seepPCB_t	* seep;
	uchar8		* seep_str;

	seep = (seepPCB_t *)seep_struct[SEEP_PCB];
	seep_str = seep->ip_mask;

//	printf("debug IP mask = %s\n", seep_str);

	if (strlen(seep_str) == 0)
		memcpy(ipmask, "255.255.255.0\0", 15);
	else
		memcpy(ipmask, seep_str, 15);
}

uchar8	seepMyExtMacAddr[4][6];


#if defined(PROM)
uint32
gethostid()
{
	return 0;
}


static int
char_to_int(char c)
{
	if (isdigit(c))
		return c - '0';
	else if (isalpha(c))
		return isupper(c) ? (c - 'A') + 10 : (c - 'a') + 10;
	else
		return -1;
}
#if defined (PROM_R7000) || defined (PROM_R9000)
static uchar8 bin_mac_addr[6];

/*-----------------------------------------------------------------
 * Name :	seepGetBSDMacAddr()
 *
 * Description: Return mac address from chassis seep for passing to BSD
 * in binary form
 *
 * Created by:	Maxim Kozlovsky
 *
 * Date Created:	09/04/02
 *
 *-----------------------------------------------------------------
 */
uchar8 *
seepGetBSDMacAddr(uint32 port)
{
#if defined (R9000) || defined (SIBYTE_BC)	
	seepPCB_t *chassis_seep = (seepPCB_t *)seep_struct[SEEP_PCB];
#else	
	seepChassis_t *chassis_seep = (seepChassis_t *)seep_struct[SEEP_CHASSIS];
#endif	
	int i;

	for (i = 0; i < 6; ++i) {
		bin_mac_addr[i] = char_to_int(chassis_seep->mac_addr[i * 2]) * 16 +
			char_to_int(chassis_seep->mac_addr[i * 2 + 1]);
	}

	if (bin_mac_addr[0] == 0x00
		&& bin_mac_addr[1] == 0x07
		&& bin_mac_addr[2] == 0x34
		&& bin_mac_addr[5] == 0x00) {
		/* Got valid mac addr in seep */
		bin_mac_addr[5] = port;
		return bin_mac_addr;
	}
	else {
		printf("MAC address in chassis seep is invalid\n");
		return seepGetMyExtMacAddr(port);
	}
}
#endif
#endif

#ifndef SSC_MGMT
/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	1/14/02
 *
 *-----------------------------------------------------------------
 */
void
seepInitExtMacAddr(
	uint32	ipaddr
	)
{
        static uchar8 t_mac_addr[6];
	uint32	c,d;
	uint32	i;
	extern uint32 gethostid();
	uint32 hostid = gethostid();
#if defined (R9000) || defined (SIBYTE_BC)
        seepPCB_t *chassis_seep = (seepPCB_t *)seep_struct[SEEP_PCB];
#else
        seepChassis_t *chassis_seep = (seepChassis_t *)seep_struct[SEEP_CHASSIS];
#endif

       /* If we don't have a MAC Address in Seep, try IP Address
        * If we don't have a MAC Address, and don't have an IP Address 
        *  we get a default   MAC of 00 07 34 00 00 00 ....
        */
        c = 0;  /* set to default value */
        d = 0;
        for (i = 0; i < 6; ++i) {
            t_mac_addr[i] = char_to_int(chassis_seep->mac_addr[i * 2]) * 16 +
                char_to_int(chassis_seep->mac_addr[i * 2 + 1]);
        }

	if (hostid != 0) {
		c = (hostid & 0x0000FF00) >> 8;
		d = (hostid & 0x000000FF);
	}
	else if ( t_mac_addr[0] == 0x00
                && t_mac_addr[1] == 0x07
                && t_mac_addr[2] == 0x34) {
                /* we have a valid mac in seep */

                c = t_mac_addr[3];
                d = t_mac_addr[4];
        }
        else { 
            if (ipaddr != 0) {
		c = (ipaddr & 0x0000FF00) >> 8;
		d = (ipaddr & 0x000000FF);
            }
	}

	for (i=0; i<4; i++) {
		seepMyExtMacAddr[i][0] = 0x00;
		seepMyExtMacAddr[i][1] = 0x07;
		seepMyExtMacAddr[i][2] = 0x34;
		seepMyExtMacAddr[i][3] = c;
		seepMyExtMacAddr[i][4] = d;
		seepMyExtMacAddr[i][5] = (nfxMySlotId << 4) | i;
#if 0
  printf("Ext Mac Addr (port %d) = %02x:%02x:%02x:%02x:%02x:%02x\n",
  i,
  seepMyExtMacAddr[i][0],
  seepMyExtMacAddr[i][1],
  seepMyExtMacAddr[i][2],
  seepMyExtMacAddr[i][3],
  seepMyExtMacAddr[i][4],
  seepMyExtMacAddr[i][5]);
#endif  
	}
}
#endif

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	1/14/02
 *
 *-----------------------------------------------------------------
 */
uchar8 *
seepGetMyExtMacAddr(
	uint32	port			/* TXRX = 0,1,2,3    SSC = 1    FC = 0 */
)
{
#if 0
	printf("seepGetMyExtMacAddr, port = %d\n", port);
	printf("returning macaddr %02x:%02x:%02x:%02x:%02x:%02x\n",
		seepMyExtMacAddr[port][0],
		seepMyExtMacAddr[port][1],
		seepMyExtMacAddr[port][2],
		seepMyExtMacAddr[port][3],
		seepMyExtMacAddr[port][4],
		seepMyExtMacAddr[port][5]);
#endif
	return &(seepMyExtMacAddr[port][0]);
}

#if 0
void
seepGetMiscInfo(uchar8	*info)
{
	seepPCB_t	* seep;
	uchar8		* seep_str;

	seep = (seepPCB_t *)seep_struct[SEEP_PCB];
	seep_str = seep->misc_info;

	memcpy(info, seep_str, 8);
}
#endif

void
seepGetHwRevision(uchar8 *rev)
{
	seepPCB_t	* seep;
	uchar8		* seep_str;

	seep = (seepPCB_t *)seep_struct[SEEP_PCB];
	seep_str = seep->revision;

	sprintf(rev, "HW revision: %s", seep_str);
}

/* Code below is for i2c on Toshiba only */
/* Needed to access SEEP from Toshiba */
#ifdef SSC_MGMT
static void
i2c_reg_write(
    uint32  reg,
    uchar8  data
)
{
    volatile uint32 * ptr = (volatile uint32 *)(HW_GALILEO_BASE + reg);
/*
#ifdef SIBYTE_BC   
    printf("%s <== 0x%x\n", i2c_sb1250_reg_string[(reg - 0xc000)/4], data);
#else    
    printf("%s <== 0x%x\n", i2c_reg_string[(reg - 0xc000)/4], data);
#endif 
*/
    *ptr = data;
}

static uchar8
i2c_reg_read(
    uint32  reg
)
{
    volatile uint32 * ptr = (volatile uint32 *)(HW_GALILEO_BASE + reg);
    uchar8  data;

    data = *ptr;
/*
#ifdef SIBYTE_BC   
    printf("%s <== 0x%x\n", i2c_sb1250_reg_string[(reg - 0xc000)/4], data);
#else    
    printf("%s ==> 0x%x\n", i2c_reg_string[(reg - 0xC000)/4], data);
#endif
*/
    return data;
}
int32
i2c(
    uint32  op,
    uchar8  i2c_addr,
    uchar8  byte_addr,
    uint32  len,
    uchar8  * buf
)
{
    uchar8  data;
    uint32  i;
    extern void delay_us(uint32 us);

    switch (op) {
        case I2C_READ:
            for (i=0; i<len; i++) {
                /*
                 * transmit start + device address
                 */
                i2c_reg_write(I2C_CONTROL, 0x60);

                while (!(i2c_reg_read(I2C_CONTROL) & 0x8));

                i2c_reg_write(I2C_DATA, i2c_addr | 0);
                i2c_reg_write(I2C_CONTROL, 0x40);

                while (!(i2c_reg_read(I2C_CONTROL) & 0x8));

                while ((data = i2c_reg_read(I2C_STATUS_BAUDE_RATE)) == 0xF8);
                if (data != 0x18) 
                    return -1;

                /*
                 * transmit word address
                 */
                i2c_reg_write(I2C_DATA, (byte_addr+i) | 0);
                i2c_reg_write(I2C_CONTROL, 0x40);

                while (!(i2c_reg_read(I2C_CONTROL) & 0x8));

                while ((data = i2c_reg_read(I2C_STATUS_BAUDE_RATE)) == 0xF8);
                if (data != 0x28) 
                    return -1;
                /*
                 * transmit start + device address
                 */
                i2c_reg_write(I2C_CONTROL, 0x60);

                while (!(i2c_reg_read(I2C_CONTROL) & 0x8));

                i2c_reg_write(I2C_DATA, i2c_addr | 1);
                i2c_reg_write(I2C_CONTROL, 0x40);

                while (!(i2c_reg_read(I2C_CONTROL) & 0x8));

                while ((data = i2c_reg_read(I2C_STATUS_BAUDE_RATE)) == 0xF8);
                if (data != 0x40) 
                    return -1;
                /*
                 * read data byte
                 */
                i2c_reg_write(I2C_CONTROL, 0x40);

                while (!(i2c_reg_read(I2C_CONTROL) & 0x8));
                while ((data = i2c_reg_read(I2C_STATUS_BAUDE_RATE)) == 0xF8);
                if ((data != 0x50) && (data != 0x58)) 
                    return -1;

                buf[i] = i2c_reg_read(I2C_DATA);

                /*
                 * transmit stop
                 */
                i2c_reg_write(I2C_CONTROL, 0x50);

                while ((data = i2c_reg_read(I2C_CONTROL)) & 0x10);
//printf("I2C_READ, offset = %d, byte = 0x%x\n", byte_addr+i, buf[i]);
            }

            break;

        case I2C_WRITE:
            for (i=0; i<len; i++) {
//printf("I2C_WRITE, offset = %d, byte = 0x%x\n", byte_addr+i, buf[i]);
                /*
                 * transmit start + device address
                 */
                i2c_reg_write(I2C_CONTROL, 0x60);

                while (!(i2c_reg_read(I2C_CONTROL) & 0x8));

                i2c_reg_write(I2C_DATA, i2c_addr | 0);
                i2c_reg_write(I2C_CONTROL, 0x40);

                while (!(i2c_reg_read(I2C_CONTROL) & 0x8));

                while ((data = i2c_reg_read(I2C_STATUS_BAUDE_RATE)) == 0xF8);
                if (data != 0x18) 
                    return -1;
                /*
                 * transmit word address
                 */
                i2c_reg_write(I2C_DATA, (byte_addr+i) | 0);
                i2c_reg_write(I2C_CONTROL, 0x40);

                while (!(i2c_reg_read(I2C_CONTROL) & 0x8));

                while ((data = i2c_reg_read(I2C_STATUS_BAUDE_RATE)) == 0xF8);
                if (data != 0x28) 
	                return -1;
                /*
                 * write data byte
                 */
                i2c_reg_write(I2C_DATA, buf[i]);
                i2c_reg_write(I2C_CONTROL, 0x40);

                while (!(i2c_reg_read(I2C_CONTROL) & 0x8));

                while ((data = i2c_reg_read(I2C_STATUS_BAUDE_RATE)) == 0xF8);
                if (data != 0x28) 
                    return -1;
                /*
                 * transmit stop
                 */
                i2c_reg_write(I2C_CONTROL, 0x50);

                while ((data = i2c_reg_read(I2C_CONTROL)) & 0x10);

                delay_us(11 * 1000);
            }

            break;

        default:
            printf("bad i2c op, %d\n", op);
    }

    return 0;

}   /* i2c() */
#endif

#ifndef PROM
/*-----------------------------------------------------------------
 * Name:	seepCumUptimeTimer()
 *
 * Description: Creates the cumulative uptime timer. 
 *              Cpu's affected: FC, TXRX, FP, SSC.
 *
 * Created by:	lipika
 *
 * Date Created:	4/24/03
 *
 *-----------------------------------------------------------------
 */
void
seepUptimeTimer_init()
{
	seepPCB_t	* seep;

	seep = (seepPCB_t *)seep_struct[SEEP_PCB];
	seep->uptime_since_reboot = 0;

#if defined (R9000) || defined (SIBYTE_BC)	
	//FC is responsible for starting the timers on behalf of the Bobcat Board.
	if(isFC())
#else		
	if((isSSC()) || (isTXRX()) || (isFP()) || (isFC())) 
#endif		
	{
	/* Create the timer for the first time that board is powered up */
	seep_board_timer = eee_timerCreate(5*60*MSEC_PER_SEC, EEE_TIMER_ONCE); //5 min timer
	if(seep_board_timer == TIMER_ERR_NO_TIMER_AVAILABLE)
			printf("ERROR: Could not create board uptime timer for this card\n");

	/* Board uptime should be cleared to 0 on bootup */
        NFX_SPIN_LOCK(&wait_seep_lock);
	seepWrite(SEEP_PCB, seep, 0, sizeof(seepPCB_t));
        NFX_SPIN_UNLOCK(&wait_seep_lock);

	seep_startTimer(seep_board_timer);
	}
}


/*-----------------------------------------------------------------
 * Name:	seep_timeout()
 *
 * Description: generic timeout function
 *              Cpu's affected: FC, TXRX, FP, SSC.
 *
 * Created by:	lipika
 *
 * Date Created:	4/24/03
 *
 *-----------------------------------------------------------------
 */
void
seep_timeout(int32 tmr_handle,
         void * usr_param0,
         void * usr_param1,
         void * usr_param2,
         void * usr_param3)
{
	uint32 cumUptime=0;
	uint32 boardUptime=0;

	NFX_SPIN_INIT(&wait_seep_lock, SPINLOCK_LEVEL_UNKNOWN);

    	/*read in seep info struct from seep*/
#if defined (R9000) || defined (SIBYTE_BC)	
	//FC is responsible for starting the timers on behalf of the Bobcat Board.
	if (isFC())
#else	
	if (isFC() || isTXRX() || isFP() || isSSC()) 
#endif		
	{
       	    NFX_SPIN_LOCK(&wait_seep_lock);
            seepRead(SEEP_PCB, (uchar8 *)&cumUptime, SEEP_PCB_CUM_UPTIME, SEEP_PCB_UPTIME_LEN-1); //cumulative uptime
            seepRead(SEEP_PCB, (uchar8 *)&boardUptime, SEEP_PCB_UPTIME_SINCE_REBOOT, SEEP_PCB_UPTIME_LEN-1); // board uptime
            NFX_SPIN_UNLOCK(&wait_seep_lock);

	    /* Increment the timer field and restart timer */	
	    if(tmr_handle == seep_board_timer){
		cumUptime += 5;  // 5 minute timer
		boardUptime += 5; // 5 min timer
            	NFX_SPIN_LOCK(&wait_seep_lock);
		seepWrite(SEEP_PCB, (uchar8 *)&cumUptime, SEEP_PCB_CUM_UPTIME, SEEP_PCB_UPTIME_LEN-1);
		seepWrite(SEEP_PCB, (uchar8 *)&boardUptime, SEEP_PCB_UPTIME_SINCE_REBOOT, SEEP_PCB_UPTIME_LEN-1);
            	NFX_SPIN_UNLOCK(&wait_seep_lock);
	    }				

	    seep_startTimer(tmr_handle);
	}
}

/*-----------------------------------------------------------------
 * Name:	seep_startTimer(uint32 timer)
 *
 * Description: Generic function to start timers
 *              Cpu's affected: FC, TXRX, FP, SSC, SSC-MGMT
 *
 * Created by:	lipika
 *
 * Date Created:	4/24/03
 *
 *-----------------------------------------------------------------
 */
void
seep_startTimer(int32 timer)
{
	uint32 rct;
#ifdef SSC_MGMT
        if(timer == system_uptime_timer)
		rct = eee_timerStart(timer, (void *)chassis_timeout,0,0,0,0);
	else
#endif		
		rct = eee_timerStart(timer, (void *)seep_timeout,0,0,0,0);

	if(rct != NFX_OK)
		printf("Seep:Could not start timer %d. Error %08x \n",timer, rct);
}
#endif
