#ifndef __GT64240_UART_REGS_H
#define __GT64240_UART_REGS_H

/* Global Register offsets */
#define GT_MPSC_MRR 0xb400		/* Main Routing Register */
#define GT_MPSC_RCRR 0xb404		/* Rx Clock Routing Register */
#define GT_MPSC_TCRR 0xb408		/* Tx Clock Routing Register */

/* Interrupt register offsets */
#define GT_MPSC_CAUSE 0xb804
#define GT_MPSC_MASK 0xb884
#define GT_MPSC_INTOFFSET 0x8

/* MPSCs base offsets */
#define GT_MPSC_0 0x8000
#define GT_MPSC_1 0x9000

/* MPSC register numbers, each is 32 bits, offset can be calculated
   as base+4*number */
enum gtuart_reg {
	GT_MPSC_MMCRL = 0,			/* Main configuration register low */
	GT_MPSC_MMCRH,				/* Main configuration register high */
	GT_MPSC_PCR,				/* Protocol configuration register */
	GT_MPSC_UBSR,				/* Uart Break/Shift register */
	GT_MPSC_CR,					/* Command register */
	GT_MPSC_MIR,				/* Max Idle Register */
	GT_MPSC_CFR,				/* Control Filtering Register */
	GT_MPSC_CHR5,				/* Uart Control Registers 5-8 */
	GT_MPSC_CHR6,
	GT_MPSC_CHR7,
	GT_MPSC_CHR8,
	GT_MPSC_ADR,				/* Address Register */
	GT_MPSC_ESR,				/* Uart Event Status Register */
};

/* Main Routing Register bits */
#define GT_MPSC_MRR_MR0_OFFSET 0 /* MPSC0 Routing bits offset */
#define GT_MPSC_MRR_MR1_OFFSET 6 /* MPSC1 Routing bits offset */
#define GT_MPSC_MRR_MR_SERx 0x0	/* Route to serial port */
#define GT_MPSC_MRR_MR_UNCONNECTED 0x7 /* Leave unconnected */
#define GT_MPSC_MRR_Dont_Stop_Clock 0x80000000 /*  */

/* Rx Clock Routing Register bits */
#define GT_MPSC_RCRR_CRR0_OFFSET 0/* MPSC0 routing bits offset */
#define GT_MPSC_RCRR_CRR1_OFFSET 8 /* MPSC1 routing bits offset */
#define GT_MPSC_RCRR_BRG0 0	
#define GT_MPSC_RCRR_BRG1 1		
#define GT_MPSC_RCRR_BRG2 2		
#define GT_MPSC_RCRR_SCLK 0x8

/* Tx Clock Routing Register bits */
#define GT_MPSC_TCRR_CRT0_OFFSET 0/* MPSC0 routing bits offset */
#define GT_MPSC_TCRR_CRT1_OFFSET 8 /* MPSC1 routing bits offset */
#define GT_MPSC_TCRR_BRG0 0x0
#define GT_MPSC_TCRR_BRG1 0x1
#define GT_MPSC_TCRR_BRG2 0x2
#define GT_MPSC_TCRR_SCLK 0x8
#define GT_MPSC_TCRR_TSCLK 0x9

/* Main Configuration register low bits */
#define GT_MPSC_MMCRL_MODE_HDLC 0x0/* Protocol Modes, bits 0:2 */
#define GT_MPSC_MMCRL_MODE_UART 0x4
#define GT_MPSC_MMCRL_MODE_BISYNC 0x5
#define GT_MPSC_MMCRL_MODE_MASK 0x7
#define GT_MPSC_MMCRL_TTX (1<<3)		/* Transparent transmitter */
#define GT_MPSC_MMCRL_TRX (1<<4)	/* Transparent receiver */
#define GT_MPSC_MMCRL_ET (1<<6)		/* Enable transmitter */
#define GT_MPSC_MMCRL_ER (1<<7)	/* Enable receiver */
#define GT_MPSC_MMCRL_LPBK (1<<8)	/* Enable loopback */
#define GT_MPSC_MMCRL_ECHO (1<<9)	/* Enable echo */
#define GT_MPSC_MMCRL_NLM (1<<10)	/* Null Modem */
#define GT_MPSC_MMCRL_TSYN (1<<12)	/* Transmitter Synch to Receive */
#define GT_MPSC_MMCRL_TIDL (1<<16) /* Transmit idles */
#define GT_MPSC_MMCRL_RTSM (1<<17) /* RTS Mode */
#define GT_MPSC_MMCRL_CTSS (1<<19) /* CTS Sampling Mode */
#define GT_MPSC_MMCRL_CDS (1<<20) /* CD Sampling Mode */
#define GT_MPSC_MMCRL_CTSM (1<<21) /* CTS Operating Mode */
#define GT_MPSC_MMCRL_CDM (1<<22) /* CD* Operating Mode */
#define GT_MPSC_MMCRL_CRCM_OFFSET 23
#define GT_MPSC_MMCRL_TRVD (1<<28) /* Transmit reverse data */
#define GT_MPSC_MMCRL_RRVD (1<<29)	/* Receive reverse data */
#define GT_MPSC_MMCRL_GDE (1<<31) /* Glitch detect enable */

/* Main configuration register high bits */
#define GT_MPSC_MMCRH_TCI (1<<0)	/* Transmit clock invert */
#define GT_MPSC_MMCRH_TINV (1<<1)	/* Transmit bit stream inversion */
#define GT_MPSC_MMCRH_TPL_OFFSET 2 /* Transmit preamble length */
#define GT_MPSC_MMCRH_TPPT_OFFSET 5	/* Transmit preamble pattern */
#define GT_MPSC_MMCRH_TCDV_OFFSET 9	/* Transmit clock divider */
#define GT_MPSC_MMCRH_TDEC_OFFSET 11 /* Transmit encoder */
#define GT_MPSC_MMCRH_RINV (1<<16) /* Receive bit stream inversion */
#define GT_MPSC_MMCRH_GDW_OFFSET 17	/* Clock Glitch Width */
#define GT_MPSC_MMCRH_RDW (1<<22) /* Receive data width */
#define GT_MPSC_MMCRH_RSYL_OFFSET 23 /* Receive Sync Length */
#define GT_MPSC_MMCRH_RCDV_OFFSET 25 /* Receive Clock Divider */
#define GT_MPSC_MMCRH_RENC_OFFSET 27 /* Receive Encoder */
#define GT_MPSC_MMCRH_SEDG_OFFSET 30 /* Synchronization Clock Edge */

/* UART Protocol Configuration Register */
#define GT_MPSC_PCR_DRT (1<<6)
#define GT_MPSC_PCR_ISO (1<<7)
#define GT_MPSC_PCR_RZS (1<<8)
#define GT_MPSC_PCR_FRZ (1<<9)
#define GT_MPSC_PCR_UM_MULTIDROP (1<<10)
#define GT_MPSC_PCR_CL_OFFSET 12
#define GT_MPSC_PCR_CL_5 (0x0 << GT_MPSC_PCR_CL_OFFSET)
#define GT_MPSC_PCR_CL_6 (0x1 << GT_MPSC_PCR_CL_OFFSET)
#define GT_MPSC_PCR_CL_7 (0x2 << GT_MPSC_PCR_CL_OFFSET)
#define GT_MPSC_PCR_CL_8 (0x3 << GT_MPSC_PCR_CL_OFFSET)
#define GT_MPSC_PCR_CL_MASK (0x3 << GT_MPSC_PCR_CL_OFFSET)
#define GT_MPSC_PCR_SBL (1<<14)
#define GT_MPSC_PCR_FLC (1<<15)

/* Break register */
#define GT_MPSC_UBSR_BRK_OFFSET 16/* Break count offset */
#define GT_MPSC_UBSR_CHR_OFFSET 0 /* Break char offset */

/* Command register */
#define GT_MPSC_CR_TEV (1<<1)	/* Tx Enable Vertical Redundancy Check */
#define GT_MPSC_CR_TPM_OFFSET 2	/* Tx Parity Mode Offset */
#define GT_MPSC_CR_PAR_ODD 0x0	/* Odd parity */
#define GT_MPSC_CR_PAR_LOW 0x1	/* Low */
#define GT_MPSC_CR_PAR_EVEN 0x2	/* Even */
#define GT_MPSC_CR_PAR_HIGH 0x3	/* High */
#define GT_MPSC_CR_TPM_ODD (GT_MPSC_CR_PAR_ODD<<GT_MPSC_CR_TPM_OFFSET)
#define GT_MPSC_CR_TPM_LOW (GT_MPSC_CR_PAR_LOW<<GT_MPSC_CR_TPM_OFFSET)
#define GT_MPSC_CR_TPM_EVEN (GT_MPSC_CR_PAR_EVEN<<GT_MPSC_CR_TPM_OFFSET)
#define GT_MPSC_CR_TPM_HIGH (GT_MPSC_CR_PAR_HIGH<<GT_MPSC_CR_TPM_OFFSET)
#define GT_MPSC_CR_TA (1<<7)	/* Transmit abort */
#define GT_MPSC_CR_TCS (1<<9)	/* Transmit TCS Character */
#define GT_MPSC_CR_REV (1<<17)	/* Rx Enable Vertical Redundancy Check */
#define GT_MPSC_CR_RPM_OFFSET 18 /* Rx Parity Mode Offset */
#define GT_MPSC_CR_RPM_ODD (GT_MPSC_CR_PAR_ODD<<GT_MPSC_CR_TPM_OFFSET)
#define GT_MPSC_CR_RPM_LOW (GT_MPSC_CR_PAR_LOW<<GT_MPSC_CR_TPM_OFFSET)
#define GT_MPSC_CR_RPM_EVEN (GT_MPSC_CR_PAR_EVEN<<GT_MPSC_CR_TPM_OFFSET)
#define GT_MPSC_CR_RPM_HIGH (GT_MPSC_CR_PAR_HIGH<<GT_MPSC_CR_TPM_OFFSET)
#define GT_MPSC_CR_RA (1<<23)	/* Receive Abort */
#define GT_MPSC_CR_CRD (1<<25)	/* Close Rx Descriptor */
#define GT_MPSC_CR_EH (1<<31)	/* Enter Hunt */

/* Uart Control Character Register */
#define GT_MPSC_CCR_INT (1<<12)	/* Generate interrupt on receiving char */
#define GT_MPSC_CCR_CO (1<<13)	/* Control octet */
#define GT_MPSC_CCR_R (1<<14)	/* Reject */
#define GT_MPSC_CCR_V (1<<16)	/* Valid */

/* Address register */

/* UART Event Status Register */
#define GT_MPSC_ESR_CTS (1<<0)	/* Clear to Send Signal */
#define GT_MPSC_ESR_CD (1<<1)	/* Carrier Detect Signal */
#define GT_MPSC_ESR_TIDLE (1<<3) /* Tx in Idle State */
#define GT_MPSC_ESR_RHS (1<<5)	/* Rx in HUNT State */
#define GT_MPSC_ESR_RLS (1<<7)	/* Rx Line Status */
#define GT_MPSC_ESR_RLIDL (1<<11) /* Rx IDLE */
#define GT_MPSC_ESR_RCR_OFFSET 16 /* Received Control Char */

/* SDMAx Command/Status Field for UART mode */
#define GT_MPSC_UARTSDMA_RX_PE (1<<0) /* Parity Error */
#define GT_MPSC_UARTSDMA_RX_CDL (1<<1) /* CD loss */
#define GT_MPSC_UARTSDMA_TX_CTSL (1<<1)	/* CTS loss */
#define GT_MPSC_UARTSDMA_RX_FR (1<<3) /* Framing Error */
#define GT_MPSC_UARTSDMA_RX_OR (1<<6) /* Data overrun */
#define GT_MPSC_UARTSDMA_RX_BR (1<<9) /* Break Received */
#define GT_MPSC_UARTSDMA_RX_MI (1<<10) /* Max Idle */
#define GT_MPSC_UARTSDMA_RX_A (1<<11) /* Address */
#define GT_MPSC_UARTSDMA_RX_AM (1<<12) /* Address match */
#define GT_MPSC_UARTSDMA_RX_CT (1<<13) /* Control octet */
#define GT_MPSC_UARTSDMA_RX_C (1<<14) /* Control Character */
#define GT_MPSC_UARTSDMA_ES (1<<15)	/* Error Summary */
#define GT_MPSC_UARTSDMA_TX_P (1<<18) /* Preamble */
#define GT_MPSC_UARTSDMA_TX_A (1<<19) /* Address */
#define GT_MPSC_UARTSDMA_TX_NS (1<<20) /* No stop bit */

/* Interrupt cause/mask register bits */
#define GT_MPSC_INT_RX (1<<0)
#define GT_MPSC_INT_RXERR (1<<1)
#define GT_MPSC_INT_TXERR (1<<3)
#define GT_MPSC_INT_RLSC (1<<4)
#define GT_MPSC_INT_RHNT (1<<5)
#define GT_MPSC_INT_RCC (1<<6)
#define GT_MPSC_INT_RCSC (1<<7)
#define GT_MPSC_INT_ROVR (1<<8)
#define GT_MPSC_INT_RCDL (1<<9)
#define GT_MPSC_INT_RCKG (1<<10)
#define GT_MPSC_INT_BPER (1<<11)
#define GT_MPSC_INT_TEIDLE (1<<12)
#define GT_MPSC_INT_TUDR (1<<13)
#define GT_MPSC_INT_TCTSL (1<<14)
#define GT_MPSC_INT_TCKG (1<<15)

#endif
