
#if !defined(LINUX_TEST) && !defined(EVAL_BOARD) && !defined(SSC_MGMT) && !defined(SIBYTE)

#if defined(__OpenBSD__)
#include <sys/types.h>
#include <sys/param.h>
#include <sys/mbuf.h>
#include <sys/protosw.h>
#include <sys/socket.h>
#include <sys/ioctl.h>
#include <sys/errno.h>
#include <sys/kernel.h>
#include <sys/systm.h>
#include <sys/device.h>
#endif

#include <nfx-incl.h>
#include <nfx-defs.h>
#include <exports/hwMap.h>

#ifdef R9000
#include <exports/eee-api.h>
#include <exports/mem-api.h>
#include <exports/serio-api.h>
#include <sys/gt64440r.h>
#endif

#include "gt64240-sdma.h"
#include "gt64240-uart.h"


#if defined(DEBUG_UART)
#define debug(x) printf x
#else
#define debug(x)
#endif

#define GTSDMA_NRX 2000
#define GTSDMA_NTX 2000

#define GTSDMA_DESC_SIZE 	(((GTSDMA_NRX+GTSDMA_NTX)*GTSDMA_UNITS + 4) * \
							 sizeof(struct gtsdma_desc))

#define GTSDMA_BUFF_SIZE	(((GTSDMA_NTX+GTSDMA_NRX)*GTSDMA_UNITS + 64) * \
							sizeof(char))

#ifndef R9000
/* Preallocated descriptors array, extra for alignment */
static struct gtsdma_desc descriptors[GTSDMA_NRX * GTSDMA_UNITS +
								GTSDMA_NTX * GTSDMA_UNITS + 4];

/* Preallocate memory for rx/tx buffers, extra for alignment */
static char bufmemory[GTSDMA_NTX * GTSDMA_UNITS +
					 GTSDMA_NRX * GTSDMA_UNITS + 64];
#else
static struct gtsdma_desc *descriptors = 0;
static char *bufmemory = 0;
#endif

static struct gtsdma_desc *last_alloc = 0;

static char *mem_last_alloc = 0;

static struct gtsdma *scs[2] = {0, 0};


static void *align(void *p, int align)
{
	if ((unsigned)p % align)
		p = (char *)p + align - (unsigned)p % align;
	assert((unsigned)p % align == 0);
	return p;
}

#define GTSDMA_DESC_ALIGN 32


static int
chain_nextidx(struct gtsdma_chain *chain, int idx)
{
	if (++idx == chain->length)
		idx = 0;

	return idx;
}


static int
alloc_chain(struct gtsdma_chain *chain, unsigned length)
{
	if (!last_alloc)
	{
#ifdef R9000
		// allocate SDMA descriptors from GT memory (uncached).
		descriptors = eee_allocateBlock( GTSDMA_DESC_SIZE );
		descriptors = (struct gtsdma_desc *)cached_to_uncached( (address_t)descriptors );
#endif
		last_alloc = align(descriptors, GTSDMA_DESC_ALIGN);
	}

	chain->length = length;
	chain->free = length;
	chain->first = 0;
	chain->last = 0;

	/* Use uncached memory to write into descriptors */
	chain->desc = last_alloc;
#ifndef R9000
	chain->desc = (struct gtsdma_desc *)K0_TO_K1(last_alloc);

	CACHE_FLUSH_STRUCT(((uint32)last_alloc & ~0x1F), GTSDMA_DESC_SIZE);
#endif

	last_alloc += length;

	assert(last_alloc <=
		   (struct gtsdma_desc *)(char *)descriptors + GTSDMA_DESC_SIZE);

	return NFX_OK;
}


static int
alloc_memory(char **ptr, int length)
{
	if (!mem_last_alloc)
	{
#ifdef R9000
		// allocate SDMA buffers from GT memory (uncached).
		bufmemory = eee_allocateBlock( GTSDMA_BUFF_SIZE );
		bufmemory = (char *)cached_to_uncached( (address_t)bufmemory );
#endif
		mem_last_alloc = align(bufmemory, 32);
	}

#ifndef R9000
	*ptr = (char *)K0_TO_K1(mem_last_alloc);

	/* Use uncached memory for further accesses to buffer memory */
	CACHE_FLUSH_STRUCT(((uint32)mem_last_alloc & ~0x1F), GTSDMA_BUFF_SIZE);
#else
	*ptr = (char *)(mem_last_alloc);
#endif

	mem_last_alloc += length;

	assert(mem_last_alloc <= bufmemory + GTSDMA_BUFF_SIZE);

	return NFX_OK;
}


static void
init_txdescriptor(struct gtsdma_desc *d, char *txbuf,
				  struct gtsdma_desc *next)
{
	d->cmd_status = 0;
	d->bufsize_cnt = (1 << 16) | 1;
#ifndef R9000
	d->bufptr = K1_TO_PHYS(txbuf);
	d->next = K1_TO_PHYS(next);
#else
	d->bufptr = (uint32) txbuf;
	d->next = (uint32) next;
#endif
}


static void
init_txchain_descriptors(struct gtsdma_chain *chain)
{
	int i;
	int last;

	for (i = 0, last = chain->length - 1; i < last; ++i)
		init_txdescriptor(&chain->desc[i], &chain->mem[i],
						  &chain->desc[i + 1]);
	init_txdescriptor(&chain->desc[i], &chain->mem[i], &chain->desc[0]);
}


static int
init_txchain(struct gtsdma_chain *chain, unsigned length)
{
	int rc;
	
	if ((rc = alloc_chain(chain, length)) == NFX_OK &&
		(rc = alloc_memory(&chain->mem, length)) == NFX_OK) {
		init_txchain_descriptors(chain);
		chain->txcmd = GTSDMA_FIRST | GTSDMA_OWN | GTSDMA_EI | GTSDMA_LAST |
			GT_MPSC_UARTSDMA_TX_P;
	}

	return rc;
}


static void
init_rxdescriptor(struct gtsdma_desc *d, char *rxbuf,
				  struct gtsdma_desc *next)
{
	d->cmd_status = GTSDMA_OWN | GTSDMA_EI;
	d->bufsize_cnt = 1 << 16;
#ifndef R9000
	d->bufptr = K1_TO_PHYS(rxbuf);
	d->next = K1_TO_PHYS(next);
#else
	d->bufptr = (uint32)(rxbuf);
	d->next = (uint32)(next);
#endif
}


static void
init_rxchain_descriptors(struct gtsdma_chain *chain)
{
	int i;
	int last;

	for (i = 0, last = chain->length - 1; i < last; ++i)
		init_rxdescriptor(&chain->desc[i], &chain->mem[i],
						  &chain->desc[i + 1]);
	init_rxdescriptor(&chain->desc[i], &chain->mem[i], &chain->desc[0]);
}


static int
init_rxchain(struct gtsdma_chain *chain, unsigned length)
{
	int rc;

	if ((rc = alloc_chain(chain, length)) == NFX_OK &&
		(rc = alloc_memory(&chain->mem, length)) == NFX_OK)
		init_rxchain_descriptors(chain);

	return rc;
}


static void
sdma_free_txchain(struct gtsdma_chain *chain)
{
	/* Should not be called for now */
	assert(0);
}


static void
sdma_free_rxchain(struct gtsdma_chain *chain)
{
	/* Should not be called for now */
	assert(0);
}


void
gtsdma_txabort(struct gtsdma *sc)
{
	/* write abort transmit bit and wait for it to clear */
	gtsdma_write_reg(sc->unit, GTSDMA_SDCM, GTSDMA_SDCM_AT);
	while (gtsdma_read_reg(sc->unit, GTSDMA_SDCM) & GTSDMA_SDCM_AT);
}


void
gtsdma_rxabort(struct gtsdma *sc)
{
	/* write abort receive bit and wait for it to clear */
	gtsdma_write_reg(sc->unit, GTSDMA_SDCM, GTSDMA_SDCM_AR);
	while (gtsdma_read_reg(sc->unit, GTSDMA_SDCM) & GTSDMA_SDCM_AR);
}


int
gtsdma_init(struct gtsdma *sc, int unit)
{
	int rc;

	debug(("gtsdma_init:%d\n", unit));
	
	sc->unit = unit;
	scs[unit] = sc;

	gtsdma_rxabort(sc);
	gtsdma_txabort(sc);

	if ((rc = init_txchain(&sc->tx, GTSDMA_NTX)) == NFX_OK
		&& (rc = init_rxchain(&sc->rx, GTSDMA_NRX)) == NFX_OK) {

#ifdef R9000
		// configure base address if it has not been setup
		if (GT_READ(COMM_UNIT_BASE_ADDRESS_0) == 0)
		{
			// Mask of lower 24 bits of base address 
			// Upper 16 bits specify base, lower 16 specify CS 0.
			GT_WRITE(COMM_UNIT_BASE_ADDRESS_0, 
				(((uint32)(&sc->rx.desc[0]) & 0xFF000000) | 0x00000e00));

			// Size is 16 MB.
			GT_WRITE(COMM_UNIT_SIZE_REGISTER_0, 0x00ff0000);

			// Enable CS 0 by clearing bit 0.
			GT_WRITE(COMM_UNIT_BASE_ADDR_ENABLE, 0x0000000e);
		}
#endif

		gtsdma_write_reg(unit, GTSDMA_SDC, GTSDMA_SDC_DEFAULT);


		/* Inilize the hardware descriptor pointers to point to the
		   first descriptors in chains */
#ifndef R9000
		gtsdma_write_reg(unit, GTSDMA_SCRDP, K1_TO_PHYS(&sc->rx.desc[0]));
		gtsdma_write_reg(unit, GTSDMA_SCTDP, K1_TO_PHYS(&sc->tx.desc[0]));
		gtsdma_write_reg(unit, GTSDMA_SFTDP, K1_TO_PHYS(&sc->tx.desc[0]));
#else
		gtsdma_write_reg(unit, GTSDMA_SCRDP, (uint32)(&sc->rx.desc[0]));
		gtsdma_write_reg(unit, GTSDMA_SCTDP, (uint32)(&sc->tx.desc[0]));
		gtsdma_write_reg(unit, GTSDMA_SFTDP, (uint32)(&sc->tx.desc[0]));
#endif
		debug(("gtsdma_init: init chains done\n"));

		/* Enable receiver */
		gtsdma_write_reg(unit, GTSDMA_SDCM, GTSDMA_SDCM_ERD);

		/* Enable interrupts */
		gtsdma_write_intreg(unit, GTSDMA_MASK, 0xffffffff);
	}

	if (rc != NFX_OK) {
		debug(("gtsdma: free chains\n"));
		sdma_free_txchain(&sc->tx);
		sdma_free_rxchain(&sc->rx);
	}
	
	return rc;
}


void
gtsdma_rxstart(struct gtsdma *sc)
{
	gtsdma_write_reg(sc->unit, GTSDMA_SDCM, GTSDMA_SDCM_ERD);
}


void
gtsdma_txstart(struct gtsdma *sc)
{
	gtsdma_write_reg(sc->unit, GTSDMA_SDCM, GTSDMA_SDCM_TXD);

}

/*
 * gtsdma_desc - get the last descriptor in the chain
 */
static struct gtsdma_desc *
chain_getdesc(struct gtsdma_chain *ch)
{
	struct gtsdma_desc *d = 0;

	if (ch->free) {
		d = &ch->desc[ch->last];
		ch->last = chain_nextidx(ch, ch->last);
		--ch->free;
	}

	return d;
}


/*
 * gtsdma_tx - transmit some characters
 */
int
gtsdma_tx(struct gtsdma *sc, char *buf, int length)
{
	int rc = NFX_OK;

	debug(("gtsdma_tx: u=%d len=%d\n", sc->unit, length));
	
	if (length <= sc->tx.free) {
/*
		while (length) {
*/
			volatile struct gtsdma_desc *d = chain_getdesc(&sc->tx);

			/* Should succeed since just checked for free descriptors */
			assert(d);

#ifndef R9000
			*(char *)PHYS_TO_K1(d->bufptr) = *buf;
#else
			*(char *)(d->bufptr) = *buf;
#endif
			d->cmd_status = sc->tx.txcmd;
			--length;
			++buf;

			debug(("send desc %x: %x %d %d %x %x\n",
				   d, d->cmd_status, d->bufsize_cnt & 0xffff,
				   d->bufsize_cnt << 16, d->bufptr, d->next));
/*
		}
*/

		gtsdma_txstart(sc);
		while (d->cmd_status & 0x80000000);
	}
	else
		rc = -1;

	return rc;
}


static void
do_error(struct gtsdma *sc)
{
	panic("");
}


static struct gtsdma_desc *
next_tx(struct gtsdma_chain *c)
{
	struct gtsdma_desc *d = &c->desc[c->first];
	if (c->first == c->last || (d->cmd_status & GTSDMA_OWN))
		d = 0;

	return d;
}


static void
tx_cleanup(struct gtsdma *sc, struct gtsdma_desc *d)
{
	struct gtsdma_chain *c = &sc->tx;

	debug(("tx_cleanup:\n"));
	/* Reinitialize cnt to 1 because hardware clears it */
	c->desc[c->first].bufsize_cnt = (1 << 16) | 1;
	c->first = chain_nextidx(c, c->first);
	++c->free;
}


static void
do_txeof(struct gtsdma *sc)
{
	struct gtsdma_desc *d;

	debug(("do_txeof: %d\n", sc->unit));
	
	while ((d = next_tx(&sc->tx)))
		tx_cleanup(sc, d);
}


static struct gtsdma_desc *
next_rx(struct gtsdma_chain *c)
{
	struct gtsdma_desc *d = &c->desc[c->last];
	if (d->cmd_status & GTSDMA_OWN)
		d = 0;

	return d;
}


static void
rx_cleanup(struct gtsdma *sc, struct gtsdma_desc *d)
{
	struct gtsdma_chain *c = &sc->rx;

	debug(("rx_cleanup:\n"));
	/* Reinitialize count to 0 and return the descriptor to the device */
	d->bufsize_cnt = (1 << 16);
	d->cmd_status = GTSDMA_OWN | GTSDMA_EI;
	c->last = chain_nextidx(c, c->last);
}


static void
do_rxeof(struct gtsdma *sc)
{
	struct gtsdma_desc *d;

	debug(("do_rxeof: %d\n", sc->unit));
	
	while ((d = next_rx(&sc->rx)))
		rx_cleanup(sc, d);
}


static int
intr_rxdone(uint32 status)
{
#if defined(GT_MPSC_GOOD_REGS)
	return status & GTSDMA_INT_RXBUF;
#else
	return 1;
#endif
}


static int
intr_txdone(uint32 status)
{
#if defined(GT_MPSC_GOOD_REGS)
	return status & GTSDMA_INT_TXBUF;
#else
	return 1;
#endif
}


static int
intr_error(uint32 status)
{
#if defined(GT_MPSC_GOOD_REGS)
	return status & (GTSDMA_INT_RXERR | GTSDMA_INT_TXERR);
#else
	return 0;
#endif
}


static void
do_intr(struct gtsdma *sc, uint32 status)
{
	debug(("do_intr: u=%d st=%x\n", sc->unit, status));
	
	if (intr_rxdone(status))
		do_rxeof(sc);
	if (intr_txdone(status))
		do_txeof(sc);
	if (intr_error(status))
		do_error(sc);
}

/*
 * gtsdma_intr - process interrupt
 */
void
gtsdma_intr(void *arg)
{
	uint32 status;

	status = gt_read_reg(GTSDMA_CAUSE);
	/* Clear the interrupts we are going to process */
	gt_write_reg(GTSDMA_CAUSE, ~status);
	if (scs[0]) do_intr(scs[0], status);
	if (scs[1]) do_intr(scs[1], status >> GTSDMA_INT_UNITOFFSET);
}


/*
 * gtsdma_rx - read some characters from rxmemory. Return the number
 * of characters read
 */
int
gtsdma_rx(struct gtsdma *sc, char *buf, int length)
{
	struct gtsdma_chain *c = &sc->rx;
	int cnt = 0;

	while ((c->first != c->last) && length--) {
		*buf = c->mem[c->first];
		c->first = chain_nextidx(c, c->first);
		++cnt;
		++buf;
	}

	return cnt;
}

#endif	/* !LINUX_TEST && !EVAL_BOARD && !SSC_MGMT && !SIBYTE */
