#ifndef __GT64240_H
#define __GT64240_H

#define GT64240_BASE	PHYS_TO_K1(HW_GALILEO_BASE)

static char *const _gt64p = (char *)GT64240_BASE;

#if #endian(little)
#define GT64W(x)		*(volatile unsigned long *)(_gt64p + (x))
#define GT64H(x)		*(volatile unsigned short *)(_gt64p + (x))
#define GT64B(x)		*(volatile unsigned char *)(_gt64p + (x))

#define GT_SWAP(x) (x)

#define WORD_SWAP(x) (x)
#else
#define GT64W(x)		*(volatile unsigned long *)(_gt64p + (x))
#define GT64H(x)		*(volatile unsigned short *)(_gt64p + ((x)^2))
#define GT64B(x)		*(volatile unsigned char *)(_gt64p + ((x)^3))

#define GT_SWAP(x) (						\
	{										\
	unsigned int v, z = (unsigned int)(x);	\
	v = z >> 24;							\
	v |= (z >> 8) & 0xff00;					\
	v |= (z & 0xff00) << 8;					\
	v |= z << 24;							\
	v;										\
	})
#endif


static inline uint32
gt_read_reg(uint32 reg)
{
	return GT_SWAP(GT64W(reg));
}


static inline void
gt_write_reg(uint32 reg, uint32 val)
{
	GT64W(reg) = GT_SWAP(val);
}


#if defined(__OpenBSD__)
#define K0_TO_PHYS(a) CACHED_TO_PHYS(a)
#define K1_TO_PHYS(a) UNCACHED_TO_PHYS(a)
#define PHYS_TO_K1(a) PHYS_TO_UNCACHED(a)
#define K0_TO_K1(a) CACHED_TO_UNCACHED(a)
#define VIRTUAL_TO_PHY(a) UNCACHED_TO_PHYS(a)
#define CACHE_FLUSH(sa, ea) Mips_SyncDCache((vm_offset_t)sa, (vm_offset_t)ea - (vm_offset_t)sa)
void *galileo_establish_intr __P((void *, u_long, int, int,
								  int (*) __P((void *)), void *, char *));
vm_offset_t galileo_vtop __P((vm_offset_t));
intrmask_t galileointr(intrmask_t, struct trap_frame *);
#else
#include <sys/cache.h>
#define assert NFX_ASSERT
#endif


#define GT_REG_WRITE(r, v) (GT64W(r) = GT_SWAP(v))
#define GT_REG_READ(r, v) (*v = GT_SWAP(GT64W(r)))

#endif

