/*
 *-----------------------------------------------------------------
 *
 * Name:	gt64440-uart.c
 *
 * RCS:  $Id: $
 *
 * Copyright (C) 2004, OnStor Inc.
 *
 * Description: Contains code for gt64440 debug mode uart.
 *
 * Created by:	Shaun Hui
 *
 * Date Created: 04/05/04
 *
 *-----------------------------------------------------------------
 */

#if defined(R9000)

#include <nfx-incl.h>
#include <sys/mips.h>
#include <exports/serio-api.h>
#include <exports/hwMap.h>
#include <sys/gt64440r.h> 
#include "gt64440-uart.h"

void
gt64440_uart_init( volatile gt64440_uart_t *uart_p )
{

	/*
	 * Set MCR-LO to uart mode, enable rx & tx.
	 */
	uart_p->mcr_lo = GT_MPSC_MCRL_MODE_UART | 
			 GT_MPSC_MCRL_ER | 
			 GT_MPSC_MCRL_ET | 
			 GT_MPSC_MCRL_NLM;

	/*
	 * Set CFR bit 29 to enable uart in dbg mode.
	 */
	uart_p->cfr = GT_MPSC_CFR_DBG_MODE;

	/*
	 * Set PCR to support 8 bit data, 1 stop bit
	 */
	uart_p->pcr = GT_MPSC_PCR_CL_8;

	/*
	 * Set CR to enter hunt mode.
	 */
	uart_p->cr = GT_MPSC_CR_EH;

	/*
	 * Set CHR5 to valid & enable interrupt
	 * so that MPSCx Cause can be polled to 
	 * detect character received.
	 */
	uart_p->chr5 = GT_MPSC_CHR5_V | GT_MPSC_CHR5_INT;
}


int
gt64440_uart (int op, int chan, int data)
{
	volatile gt64440_uart_t *uart_p;
	int32    mpsc_cause;

	if (chan > 1)
		return 1;

	mpsc_cause = (chan == 0) ? MPSC0_CAUSE : MPSC1_CAUSE;
                                                                                
	uart_p = (volatile gt64440_uart_t *)GT_UART_BASE( chan );

	switch (op) {
		case OP_INIT:
			gt64440_uart_init ( uart_p );
			return 0;
                                                                                
		case OP_BAUD:
			return 0; /*gt64440_uart_baud (dp, chan, data); */

		case OP_TXRDY:
			return (!(uart_p->cr & GT_MPSC_CR_TCS));

		case OP_TX:
			uart_p->ubsr = (data & 0xff);
			uart_p->cr  |= GT_MPSC_CR_TCS;
			break;

		case OP_RXRDY:
			return (GT_READ(mpsc_cause) & 0x00000040);

		case OP_RX:
		{
			char c;
			GT_WRITE(mpsc_cause, 0);
			c = ((uart_p->esr & 0x00ff0000) >> 16);
			uart_p->esr |= 0x00ff0000;
			return (c);
		}

		case OP_RXSTOP:
			if (data)
				uart_p->mcr_lo &= ~GT_MPSC_MCRL_ER; /* Dis Rx */
			else
				uart_p->mcr_lo |= GT_MPSC_MCRL_ER; /* Ena Rx */
			break;
	}
	return 0;
}

#endif	/* R9000 */
