/* vi:ts=4
 *-----------------------------------------------------------------
 *
 * Name:	sb1250_uart.c
 *
 * RCS:  $Id: sb1250_uart.c,v 1.13 2002/11/05 00:25:12 rick Exp $
 *
 * Copyright (C) 2001, Agile Storage, Inc.
 *
 * Description:	 Serial driver for sb1250
 *
 * Created by:	Rick Lund
 *
 * Date Created:	6/14/01
 *
 *-----------------------------------------------------------------
 */

#ifdef SIBYTE

#include <nfx-incl.h>
#include <sys/sb1250/sb1250_uart.h>
#include <sys/mips.h>
#include <exports/serio-api.h>
#include <exports/cm-api.h>

extern uint32 sb1250_cable_installed[2];
/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:
 *
 *-----------------------------------------------------------------
 */
int32
sb1250_uart(
	uint32	op,
	uint32	chan,
	int32	data
)
{
	volatile sb1250_uart_t	* dp;
	volatile sb1250_uart_t	* uartA;
	volatile sb1250_uart_t	* uartB;
	uint64					garbage;

	if (chan > 1)
		return SERIO_ERR_BAD_PARAM;

	uartA = (volatile sb1250_uart_t *)
								PHYS_TO_K1(A_DUART + (DUART_CHANREG_SPACING*1));
	uartB = (volatile sb1250_uart_t *)
								PHYS_TO_K1(A_DUART + (DUART_CHANREG_SPACING*2));
	if (chan == 0)
		dp = (volatile sb1250_uart_t *)
								PHYS_TO_K1(A_DUART + (DUART_CHANREG_SPACING*1));
	else
		dp = (volatile sb1250_uart_t *)
								PHYS_TO_K1(A_DUART + (DUART_CHANREG_SPACING*2));

	switch (op) {
		case OP_INIT:
			while (!(dp->status & 0x8));
			dp->cmd = 0x20;
			garbage = uartA->mode;
			garbage = uartB->mode;
			delay_us(100);
			dp->cmd = 0x30;
			garbage = uartA->mode;
			garbage = uartB->mode;
			delay_us(100);
			dp->cmd = 0;
			garbage = uartA->mode;
			garbage = uartB->mode;
			delay_us(100);
#ifdef PROM
			dp->mode = 0x13;	/* 8 bits, no parity */
			dp->mode1 = 0;		/* normal operation, 1 stop bit */
			garbage = uartA->mode;
			garbage = uartB->mode;
#else
            /*
			 * Enable RTS/CTS flow control
             */
		    dp->mode = 0x13 | M_DUART_RX_RTS_ENA;
           	dp->mode1 = M_DUART_TX_CTS_ENA;
			garbage = uartA->mode;
			garbage = uartB->mode;
#endif
			dp->brg = V_DUART_BAUD_RATE(data);		/* set baud rate */
			garbage = uartA->mode;
			garbage = uartB->mode;
			dp->cmd = 0x5;		/* TX enable + RX enable */
			garbage = uartA->mode;
			garbage = uartB->mode;
			break;

		case OP_BAUD:
			dp->cmd = 0;		/* TX disable + RX disable */
			garbage = uartA->mode;
			garbage = uartB->mode;
			dp->brg = V_DUART_BAUD_RATE(data);		/* set baud rate */
			garbage = uartA->mode;
			garbage = uartB->mode;
			dp->cmd = 0x5;		/* TX enable + RX enable */
			garbage = uartA->mode;
			garbage = uartB->mode;
			break;

		case OP_TXRDY:
			if (!sb1250_cable_installed[nfxMyCoreId])
				return 1;
			return (dp->status & M_DUART_TX_RDY);

		case OP_RXRDY:
			return (dp->status & M_DUART_RX_RDY);

		case OP_TX:
			dp->tx_hold = data;
			garbage = uartA->mode;
			garbage = uartB->mode;
			break;

		case OP_RX:
			return (dp->rx_hold & 0xFF);
	}		

	return NFX_OK;

}	/* sb1250() */

#endif	/* SIBYTE */
