/*
 *-----------------------------------------------------------------
 *
 * Name:
 *
 * RCS:  $Id: spinlock-ui.c,v 1.8 2002/12/09 23:54:18 rick Exp $
 *
 * Copyright (C) 2002, Agile Storage, Inc.
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	11/12/02
 *
 *-----------------------------------------------------------------
 */

#include <nfx-incl.h>
#include <exports/cm-api.h>
#include "spinlock.h"

#define ATOMIC_TEST_INC	0x1000000

uint32	atomic_counter_int;
uint64	atomic_counter_dword;

#if defined(NFX_SMP)
#if defined(NFX_SPINLOCK_DEBUG)

uint64	trylock_failed CPU_PRIVATE;
uint64	locks_acquired_count CPU_PRIVATE;
uint32	locks_currently_held CPU_PRIVATE;

nfx_spinlock_t	spinlock_debug[SPINLOCK_DEBUG_NUM];
uint32	spinlock_debug_inuse[SPINLOCK_DEBUG_NUM] CPU_PRIVATE;

extern void signextend(address_t * addr);

extern uint32			spinlock_static_num;
extern nfx_spinlock_t	* spinlock_static_lock[];

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	12/3/02
 *
 *-----------------------------------------------------------------
 */
void
spinlock_view_lock(
	nfx_spinlock_t	* lock
)
{
	extern uint32	busclock;

	printf("Lock Status for %p\n", lock);
	printf("   lock->lock = 0x%x\n", lock->lock);
	printf("   lock->cpu_num = 0x%x\n", lock->cpu_num);
	printf("   lock->lock_level = 0x%x\n", lock->lock_level);
	printf("   lock->owner = 0x%lx\n", lock->owner);
	printf("   lock->lock_start_time = 0x%x\n",lock->lock_start_time);
	printf("   lock->accumulated_wait_time = 0x%llx (%d us)\n",
									lock->accumulated_wait_time,
								(uint32)(lock->accumulated_wait_time/busclock));
	printf("   lock->num_got = 0x%x\n",lock->num_got);
	printf("   lock->num_waited_for = 0x%x\n",lock->num_waited_for);
	printf("   lock->magic = 0x%x\n", lock->sl_magic);
	printf("\n");
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	11/12/02
 *
 *-----------------------------------------------------------------
 */
void
spinlock_ui_view()
{
	uint32	i;

	printf("\n");
	printf("Num Locks Currently Held = %d\n", locks_currently_held);
	printf("Num Total Locks Acquired = %lld\n", locks_acquired_count);
	printf("Num TryLock Failed       = %lld\n", trylock_failed);
	printf("\n");

	for (i=0; i<spinlock_static_num; i++)
		spinlock_view_lock(spinlock_static_lock[i]);
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	11/12/02
 *
 *-----------------------------------------------------------------
 */
int32
spinlock_ui(
	int32	argc,
	uchar8	* argv[]
)
{
	uint32	num_locks, i, lnum;

	if (strcmp(argv[1], "view") == 0) {
		if (argc == 3) {
			nfx_spinlock_t	* lock;

			atollx((uint64 *)&lock, argv[2]);
			signextend((address_t *)&lock);
			spinlock_view_lock(lock);
		}
		else {
			uint32	first = 1;
			spinlock_ui_view();
			for (i=0; i<SPINLOCK_DEBUG_NUM; i++) {
				if (spinlock_debug_inuse[i]) {
					if (first) {
						printf("debug spinlocks held :\n   ");
						first = 0;
					}
					printf("%d ", i);
				}
			}
			printf("\n");
		}
	}
	else if (strcmp(argv[1], "get") == 0) {
		if (argc == 3)
			num_locks = atoi(argv[2]);
		else
			num_locks = 1;
		for (i=0; i<num_locks; i++) {
			do {
				lnum = random() % SPINLOCK_DEBUG_NUM;
			} while (spinlock_debug_inuse[lnum]);
			printf("   get lock %d\n", lnum);
			NFX_SPIN_LOCK(&spinlock_debug[lnum]);
			spinlock_debug_inuse[lnum] = 1;
		}
	}
	else if (strcmp(argv[1], "release") == 0) {
		if (argc == 3) {
			lnum = atoi(argv[2]);
			NFX_SPIN_UNLOCK(&spinlock_debug[lnum]);
			spinlock_debug_inuse[lnum] = 0;
		}
		else {
			for (i=0; i<SPINLOCK_DEBUG_NUM; i++)
				if (spinlock_debug_inuse[i]) {
					NFX_SPIN_UNLOCK(&spinlock_debug[i]);
					spinlock_debug_inuse[i] = 0;
				}
		}
	}
	else if (strcmp(argv[1], "test") == 0) {
		spinlock_test(nfxMyCoreId);
	}
	else
		return -1;

	return 0;
}
#else

int32
spinlock_ui(int32 argc,	uchar8 *argvp[])
{
	printf("must be compiled with -DNFX_SPINLOCK_DEBUG to use spinlock ui");
	return NFX_OK;
}

#endif  /* NFX_SPINLOCK_DEBUG */

#endif	/* NFX_SMP */

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	11/12/02
 *
 *-----------------------------------------------------------------
 */
int32
atomic_ui(
	int32	argc,
	uchar8	* argv[]
)
{
	uint32	i, step, val;
	uint32	prev_32;
	uint64	prev_64;

	if (strcmp(argv[1], "inc") == 0) {
		if (argc == 3)
			step = atoi(argv[2]);
		else
			step = 1;
		for (i=0; i<ATOMIC_TEST_INC; i++)
			atomic_increment(&atomic_counter_int, 4, step);
		for (i=0; i<ATOMIC_TEST_INC; i++)
			atomic_increment(&atomic_counter_dword, 8, step);
		printf("atomic_counter_int = 0x%x\n", atomic_counter_int);
		printf("atomic_counter_dword = 0x%llx\n", atomic_counter_dword);
	}
	else if (strcmp(argv[1], "dec") == 0) {
		if (argc == 3)
			step = atoi(argv[2]);
		else
			step = 1;
		for (i=0; i<ATOMIC_TEST_INC; i++)
			atomic_decrement(&atomic_counter_int, 4, step);
		for (i=0; i<ATOMIC_TEST_INC; i++)
			atomic_decrement(&atomic_counter_dword, 8, step);
		printf("atomic_counter_int = 0x%x\n", atomic_counter_int);
		printf("atomic_counter_dword = 0x%llx\n", atomic_counter_dword);
	}
	else if (strcmp(argv[1], "or") == 0) {
		if (argc == 3)
			val = atox(argv[2]);
		else
			return -1;
		prev_32 = ATOMIC_OR_INT(&atomic_counter_int, val);
		prev_64 = ATOMIC_OR_DWORD(&atomic_counter_dword, (uint64)val);
		printf("atomic_counter_int = 0x%x (prev = 0x%x)\n",
												atomic_counter_int, prev_32);
		printf("atomic_counter_dword = 0x%llx (prev = 0x%llx)\n",
												atomic_counter_dword, prev_64);
	}
	else if (strcmp(argv[1], "and") == 0) {
		if (argc == 3)
			val = atox(argv[2]);
		else
			return -1;
		prev_32 = ATOMIC_AND_INT(&atomic_counter_int, val);
		prev_64 = ATOMIC_AND_DWORD(&atomic_counter_dword, (uint64)val);
		printf("atomic_counter_int = 0x%x (prev = 0x%x)\n",
												atomic_counter_int, prev_32);
		printf("atomic_counter_dword = 0x%llx (prev = 0x%llx)\n",
												atomic_counter_dword, prev_64);
	}
	else
		return -1;

	return 0;
}

