/*
 *------------------------------------------------------------------------------
 * RCS:  $Id: spm-api.h,v 1.39 2002/11/07 01:15:25 rick Exp $
 * Copyright (C) 2001, Agile Storage, Inc.
 *
 * Name         : spm-api.h
 *
 * Description  : Defines the interfaces of the Storage Pool Manger (SPM).
 *
 * Created by   : Isabelle Cnudde
 *
 * Date Created : 15-FEB-2002
 *
 *------------------------------------------------------------------------------
 */

#ifndef _SPM_API_H_
#define _SPM_API_H_

/*
 * Include files
 */

#include "nfx-incl.h"
#include "exports/anpssc-api.h"
#include "exports/eee-api.h"
#include "exports/scsi-api.h"
#include "exports/sdm-api.h"
#include <exports/evm-api.h>

#define SPM_MAX_POOL_NAME_LEN      64
#define SPM_MAX_CLUSTER_NAME      256
#define SPM_MAX_NODE_NAME_LEN     256

#define SPM_DBG_LEVEL_NONE     0
#define SPM_DBG_LEVEL_ELOG     1
#define SPM_DBG_LEVEL_ERR      2
#define SPM_DBG_LEVEL_FCT      3
#define SPM_DBG_LEVEL_DBG      4
#define SPM_DBG_LEVEL_MEM      5

typedef enum {
    SPM_LUN_ALL_LIST = 0,       
    SPM_LUN_FREE_LIST = 1,
    SPM_LUN_OUTCLUSTER_LIST = 2,
    SPM_LUN_FOREIGN_LIST = 4,
    SPM_LUN_USED_LIST = 8
} spm_lunType_t;

#define SPM_OK         1
#define SPM_NOK        2
#define SPM_NOT_FND    3
#define SPM_NOT_FOR    4
#define SPM_NOT_FREE   5
#define SPM_NOT_DISK   6
typedef int32 spm_ret_status_t;

#define SPM_ALLOC_OK        0   /* NFX_OK is also '0' */
#define SPM_ALLOC_PENDING   1
#define SPM_ALLOC_FAILED    2
typedef int32 spm_allocStatus_t;

typedef enum
{
    SPM_LUN_FREE = 1,
    SPM_LUN_USED = 2,
    SPM_LUN_FOREIGN = 3,
    SPM_LUN_OC = 4
} spm_lunState_t;

typedef enum {
    SPM_DEV_DISK,
    SPM_DEV_TAPE,
    SPM_DEV_CHANGER,
    SPM_DEV_STACKER,
    SPM_DEV_MISC
} spm_dev_t;

/*
 * Header of the message from/to AV.
 */

/*
 * Message type
 */
typedef enum
{
    // Interface with CLI/AV
    SPM_GET_TOTAL_POOLS_REQ = SPM_MSG_BASE,   // Get list of pools in SPM 2560
    SPM_GET_TOTAL_POOLS_RSP,
    SPM_GET_POOL_LIST_REQ,
    SPM_GET_POOL_LIST_RSP,
    SPM_GET_TOTAL_LUNS_REQ,                 // Get number of Lun's in a type
    SPM_GET_TOTAL_LUNS_RSP,
    SPM_GET_LUN_LIST_REQ,                   // Get list of luns in a pool
    SPM_GET_LUN_LIST_RSP,
    SPM_GET_TOTAL_CTRLS_REQ,
    SPM_GET_TOTAL_CTRLS_RSP,
    SPM_GET_CTRL_LIST_REQ,
    SPM_GET_CTRL_LIST_RSP,
    SPM_GET_CTRL_NODE_LIST_REQ,
    SPM_GET_CTRL_NODE_LIST_RSP,
    SPM_GET_DEV_DETAIL_REQ,
    SPM_GET_DEV_DETAIL_RSP,
    SPM_GET_DEV_PORT_REQ,
    SPM_GET_DEV_PORT_RSP,

    SPM_MARK_LUN_AS_AGILE_REQ,
    SPM_MARK_LUN_AS_AGILE_RSP,
    SPM_MARK_LUN_AS_OC_REQ,
    SPM_MARK_LUN_AS_OC_RSP,
    SPM_UNMARK_LUN_AS_AGILE_REQ,
    SPM_UNMARK_LUN_AS_AGILE_RSP,

    SPM_RESCAN_LUN_REQ,
    SPM_RESCAN_LUN_RSP,

    // Interface with CLI for debug only
    SPM_GET_STORAGE_POOL_LIST_REQ,
    SPM_GET_STORAGE_POOL_LIST_RSP,
    SPM_GET_STORAGE_LUN_LIST_REQ,
    SPM_GET_STORAGE_LUN_LIST_RSP,

    // Interface with EVM
    SPM_ADD_LUN_REQ,
    SPM_ADD_LUN_RSP,

    SPM_DEL_LUN_REQ,
    SPM_DEL_LUN_RSP,

    SPM_ALLOC_LUN_REQ,
    SPM_ALLOC_LUN_RSP,

    SPM_FREE_LUN_REQ,
    SPM_FREE_LUN_RSP,

    SPM_UPDATE_LUN_LABEL_IND, // EVM sends this indication when label is changed

    // Interface with SRM
    SPM_SRM_INIT_IND,
    SPM_SRM_NEW_LUN_IND,
    SPM_SRM_DEL_LUN_IND,
    SPM_SRM_LUN_STATE_CHANGE_IND,

    // Debug level
    SPM_DEBUG_LEVEL,

    // SPM update cluster
    SPM_UPDATE_CLUSTER_NAME_REQ,
    SPM_UPDATE_CLUSTER_NAME_RSP,

    SPM_MSG_BASE_END      // This the last message
} spm_msgType_t;

/*
 * Header of the message.
 */
typedef struct
{
    spm_msgType_t   msgType;    // the message type SHALL always be first
    uint32          spare;
    uint64          clientPrivate;
    char            node[EVM_MAX_NODE_NAME];  // Sender node name for messages
} spm_msgHdr_t;

typedef struct
{
    scsi_id_t       scsiId;
    uint64          wwn;                            // WWN
    evm_raid_t      raidLevel;
    uint16          lun;                            // LUN
    char            name[SDM_DEV_NAME_LEN+6];       // ASCII name set by user
    char            vendor[SDM_VENDOR_NAME_LEN+8];  // Device vendor string
    char            model[SDM_MODEL_NAME_LEN+8];    // Device model string
    uint64          size;                           // LUN size (blocks)
    evm_lunState_t  state;                          // LUN state
    evm_volId_t     volId;
    char            cluster[SPM_MAX_CLUSTER_NAME];
    int32           devType;
} spm_lunAttr_t;

typedef struct
{
    scsi_id_t       scsiId;
    uint64          wwn;                            // WWN
    evm_raid_t      raidLevel;
    spm_lunState_t  state;                          // LUN state
    uint64          size;                           // LUN size (blocks)
    uint16          lun;                            // LUN
    char            name[SDM_DEV_NAME_LEN+6];       // ASCII name set by user
    char            vendor[SDM_VENDOR_NAME_LEN+8];  // Device vendor string
    char            model[SDM_MODEL_NAME_LEN+8];    // Device model string
} spm_lunDetail_t;

typedef struct
{
    evm_raid_t  rL;
    uint32      usedSize;      // Used Size
    uint32      freeSize;      // Free Size
    uint32      ocSize;        // OutCluster Size
    uint32      foreignSize;   // Foreign Size
    uint32      numLuns;       // Total number of LUNs
    spm_dev_t   type;          // device type
    char        ctrl[SDM_DEV_NAME_LEN+4];
    char        make[SDM_VENDOR_NAME_LEN+8];
    char        model[SDM_MODEL_NAME_LEN+8];
} spm_poolNode_t;

typedef struct
{
    uint64           size;       // Blocks
    scsi_id_t        sid;
    evm_lunState_t   state;
    evm_raid_t       rL;
    spm_dev_t        type;
    uint16           lun;
    char             dev[SDM_DEV_NAME_LEN+2];
    char             model[SDM_MODEL_NAME_LEN+8];
    char             lunCluster[SPM_MAX_CLUSTER_NAME+8];
} spm_poolLunList_t;

typedef struct {
    scsi_id_t       scsiId;     // physical device
    uint32          nfxRefCnt;  // # of NFX that can see the LUN
    int32           lunState;   // current state of this LUN
    uint64          lunSize;    // size of LUN
} spm_lunsInPool_t;

typedef struct
{ 
    int32        poolId;
    evm_raid_t   raidLevel;
    uint64       usedSize;
    uint64       freeSize;
    uint64       foreSize;
    uint64       outcSize;
    uint32       usedLun;
    uint32       freeLun;
    uint32       foreLun;
    uint32       outcLun;
    char         ctrl[SDM_DEV_NAME_LEN+8];
    char         name[SDM_VENDOR_NAME_LEN+8];
    char         model[SDM_MODEL_NAME_LEN+8];
} spm_storageAttrib_t;

typedef struct
{
    int32        cnt;
    char         node[1][SPM_MAX_NODE_NAME_LEN];
} spm_nodeList_t;

typedef spm_nodeList_t spm_ctrlList_t;

/*
 * Requests
 */

/*
 * Interface with CLI/AV
 */

/*
 * Get the list of pools with their attributes
 */
typedef struct
{
    spm_msgHdr_t    hdr;
} spm_justHdr_t;

typedef spm_justHdr_t spm_MsgTotalPoolReq_t; // SPM_GET_TOTAL_POOLS_REQ

typedef struct
{
    spm_msgHdr_t    hdr;
    uint32          num;
} spm_MsgTotalPoolRsp_t;                    // SPM_GET_TOTAL_POOLS_RSP

typedef spm_justHdr_t spm_MsgPoolListReq_t; // SPM_GET_POOL_LIST_REQ

typedef struct
{
    spm_msgHdr_t    hdr;
    uint32          numEntries;  // number of pool list entries
    char            cluster[SPM_MAX_CLUSTER_NAME+4];
    spm_poolNode_t  poolList[1];
} spm_MsgPoolListRsp_t;                     // SPM_GET_POOL_LIST_RSP

/*
 * Get total number of LUN's for a given type of LUN in a pool
 */
typedef struct
{
    spm_msgHdr_t    hdr;
    evm_raid_t      rL;
    spm_lunType_t   lunType;
    uint32          diskOtherAll; // 1 - All, 0 - Disk, 2 - Other's
    char            ctrl[SDM_DEV_NAME_LEN+4];
} spm_MsgTotalLunReq_t;                     // SPM_GET_TOTAL_LUNS_REQ

typedef struct
{
    spm_msgHdr_t    hdr;
    evm_raid_t      rL;
    spm_lunType_t   lunType;
    uint32          numLun;
    char            cluster[SPM_MAX_CLUSTER_NAME+4];
    char            ctrl[SDM_DEV_NAME_LEN+8];
} spm_MsgTotalLunRsp_t;                     // SPM_GET_TOTAL_LUNS_RSP

typedef spm_justHdr_t spm_MsgTotalCtrlsReq_t; // SPM_GET_TOTAL_CTRLS_REQ

typedef struct
{
    spm_msgHdr_t    hdr;
    uint32          num;
} spm_MsgTotalCtrlsRsp_t;         // SPM_GET_TOTAL_CTRLS_RSP

typedef struct
{
    spm_msgHdr_t    hdr;
    uint32          numReq;     // 0 all CTRL's in one response
    char            lastCtrl[SDM_DEV_NAME_LEN+4];
} spm_MsgCtrlListReq_t;     // SPM_GET_CTRL_LIST_REQ

typedef struct
{
    spm_msgHdr_t    hdr;
    uint32          numCtrls;
    uint32          spare;
    char            ctrl[1][SDM_DEV_NAME_LEN];
} spm_MsgCtrlListRsp_t;     // SPM_GET_CTRL_LIST_RSP

typedef struct
{
    spm_msgHdr_t    hdr;
    char            ctrl[SDM_DEV_NAME_LEN];
} spm_MsgCtrlNodeListReq_t;     // SPM_GET_CTRL_NODE_LIST_REQ

typedef struct
{
    spm_msgHdr_t    hdr;
    uint32          numNode;
    spm_dev_t       type;
    char            ctrl[SDM_DEV_NAME_LEN+8];
    char            nodes[1][SPM_MAX_NODE_NAME_LEN];
} spm_MsgCtrlNodeListRsp_t;     // SPM_GET_CTRL_NODE_LIST_RSP

typedef struct
{
    spm_msgHdr_t    hdr;
    char            devName[SDM_DEV_NAME_LEN+8];
    char            tNode[SPM_MAX_NODE_NAME_LEN+8];
} spm_MsgDevDetailReq_t; // SPM_GET_DEV_DETAIL_REQ

typedef struct
{
    uint64          wwn;
    uint32          lun;
    uint32          path;
} spm_devPath_t;

typedef struct
{
    spm_msgHdr_t    hdr;
    uint64          size;
    spm_lunState_t  state;
    spm_dev_t       devType;
    uint32          activeFlag; // 1 mean active path set
    uint32          active;
    uint32          pathCnt;
    uint32          nodeCnt;
    spm_devPath_t   path[4];
    evm_volId_t     volid;
    evm_raid_t      rL;
    uint32          result;
    char            devName[SDM_DEV_NAME_LEN+8];
    char            model[SDM_MODEL_NAME_LEN+8];
    char            nodes[1][SPM_MAX_NODE_NAME_LEN];
} spm_MsgDevDetailRsp_t; // SPM_GET_DEV_DETAIL_RSP

typedef struct
{
    spm_msgHdr_t    hdr;
    uint32          port;
    uint32          numReq; // 0 - send all in one req
    uint64          lastSID; // 0 - start
    char            tNode[SPM_MAX_NODE_NAME_LEN+8];
} spm_MsgDevPortReq_t;  // SPM_GET_DEV_PORT_REQ

/*
 * Get the list of LUN's in the POOL
 */
typedef struct
{
    spm_msgHdr_t    hdr;
    evm_raid_t      rL;
    spm_lunType_t   lunType;
    uint64          lastSID;    // 0 start from begin
    uint32          numReq;     // 0 all LUN's in one response
    uint32          diskOtherAll; // 1 - All, 0 - Disk, 2 - Other's
    char            ctrl[SDM_DEV_NAME_LEN+8];
    char            tNode[SPM_MAX_NODE_NAME_LEN+8];
} spm_MsgLunListReq_t;          // SPM_GET_LUN_LIST_REQ

typedef struct
{
    spm_msgHdr_t        hdr;
    evm_raid_t          rL;
    spm_lunType_t       lunType;
    int32               numLuns;
    char                cluster[SPM_MAX_CLUSTER_NAME+4];
    char                ctrl[SDM_DEV_NAME_LEN+8];
    char                make[SDM_VENDOR_NAME_LEN+8];
    spm_poolLunList_t   lunList[1];
} spm_MsgLunListRsp_t;        // SPM_GET_LUN_LIST_RSP

typedef spm_MsgLunListRsp_t spm_MsgDevPortRsp_t;  // SPM_GET_DEV_PORT_RSP

typedef struct
{
    spm_msgHdr_t        hdr;
    uint32              devCnt;
    char                clNode[EVM_MAX_NODE_NAME+4]; // Target Node
    char                dev[1][SDM_DEV_NAME_LEN+8];
} spm_MsgLunRescanReq_t;   // SPM_RESCAN_LUN_REQ

typedef struct
{
    spm_msgHdr_t        hdr;
    int32               result;
    char                cluster[SPM_MAX_CLUSTER_NAME+4];
} spm_MsgLunRescanRsp_t;   // SPM_RESCAN_LUN_RSP

/*
 * Debug messages only
 */

typedef spm_justHdr_t spm_MsgStoragePoolListReq_t; // SPM_GET_STORAGE_POOL_LIST_REQ

typedef struct
{
    spm_msgHdr_t        hdr;
    int32               num;
    char                cluster[SPM_MAX_CLUSTER_NAME+4];
    spm_storageAttrib_t strg[1];
} spm_MsgStoragePoolListRsp_t; // SPM_GET_STORAGE_POOL_LIST_RSP

typedef struct
{
    spm_msgHdr_t        hdr;
    int32               poolId;
} spm_MsgStorageLunReq_t;     // SPM_GET_STORAGE_LUN_LIST_REQ

typedef struct
{
    spm_msgHdr_t        hdr;
    uint64              num;
    char                cluster[SPM_MAX_CLUSTER_NAME+8];
    spm_lunsInPool_t    luns[1];
} spm_MsgStorageLunRsp_t;     // SPM_GET_STORAGE_LUN_LIST_RSP

/*
 * General response structure
 */
typedef struct
{
    spm_msgHdr_t        hdr;        // SPM_MSG_STAT
    uint32              stat;       // NFX_* codes
    uint32              spare;
    union {
        evm_volId_t     lVolId;     // This LV
        int32           sPoolId;    // or Storage Pool
        scsi_id_t       sId;        // or this physical volume
    } payload;
} spm_MsgStat_t;

/*
 * Interface with EVM
 */

/*
 * New LUN is seen by the node.
 */
typedef struct
{
    spm_msgHdr_t        hdr;
    spm_lunAttr_t       lunAttr;
    char                clNode[EVM_MAX_NODE_NAME]; // Node in context
} spm_MsgAddLunReq_t;                       // SPM_ADD_LUN_REQ


typedef struct
{
    spm_msgHdr_t        hdr;
    int32               result;             // NFX_OK / NFX_ERR
} spm_MsgAddLunRsp_t;                       // SPM_ADD_LUN_RSP

/*
 * A LUN has disappear from the node's view.
 */
typedef struct
{
    spm_msgHdr_t        hdr;
    scsi_id_t           id;                 // SCSI id
} spm_MsgDelLunReq_t;                       // SPM_DEL_LUN_REQ

typedef struct
{
    spm_msgHdr_t        hdr;
    int32               result;             // NFX_OK / NFX_ERR
} spm_MsgDelLunRsp_t;                       // SPM_DEL_LUN_RSP

/*
 * A node needs more LUNs.
 */
typedef struct
{
    spm_msgHdr_t            hdr;
    uint64                  minSize;        // min. size needed (blocks)
    evm_strg_attributes_t   attr;           // LUN's attributes
    evm_volId_t             volId;
    uint32                  nbLun;
    char                    lunList[1][SDM_DEV_NAME_LEN+8];
} spm_MsgAllocLunReq_t;                     // SPM_ALLOC_LUN_REQ

typedef struct
{
    spm_msgHdr_t        hdr;
    spm_allocStatus_t   result;
    evm_volId_t         volId;
    int32               nbLun;              // number of allocated LUNs
    scsi_id_t           id[1];              // list of allocated LUNs
} spm_MsgAllocLunRsp_t;                     // SPM_ALLOC_LUN_RSP

/*
 * Node doesn't need the LUN anymore
 */
typedef struct
{
    spm_msgHdr_t        hdr;
    int32               nbLun;              // number of allocated LUNs
    scsi_id_t           id[1];              // list of allocated LUNs
} spm_MsgFreeLunReq_t;                      // SPM_FREE_LUN_REQ

typedef struct
{
    spm_msgHdr_t        hdr;
    int32               result;             // NFX_OK / NFX_ERR
} spm_MsgFreeLunRsp_t;                      // SPM_FREE_LUN_RSP

typedef struct
{
    spm_msgHdr_t        hdr;
    uint32              devCnt;
    uint32              force; // 1 - just do it!!!
    uint64              vgReqIndex; // Match this to the Vol Grow req.
    char                cluster[SPM_MAX_CLUSTER_NAME]; // -c option
    char                dev[1][SDM_DEV_NAME_LEN+8];
} spm_MsgMarkAgileLunReq_t;   // SPM_MARK_LUN_AS_AGILE_REQ
                              // SPM_MARK_LUN_AS_OC_REQ
                              // SPM_UNMARK_LUN_AS_AGILE_REQ
                              // SPM_UPDATE_CLUSTER_NAME_REQ

typedef struct
{
    spm_msgHdr_t       hdr;
    spm_ret_status_t   status;
} spm_MsgMarkAgileLunRsp_t;   // SPM_MARK_LUN_AS_AGILE_RSP
                              // SPM_MARK_LUN_AS_OC_RSP
                              // SPM_UNMARK_LUN_AS_AGILE_RSP
                              // SPM_UPDATE_CLUSTER_NAME_RSP

typedef struct
{
    spm_msgHdr_t       hdr;
    spm_lunAttr_t      attr;
} spm_MsgUpdateLunLabelInd_t; // SPM_UPDATE_LUN_LABEL_IND
    


/*
 * Interface with SRM
 */
typedef struct
{
    spm_msgHdr_t        hdr;
    char                cluster[SPM_MAX_CLUSTER_NAME];
} spmSrm_MsgSpmInitInd_t; // SPM_SRM_INIT_IND

typedef struct
{
    spm_msgHdr_t    hdr;
    spm_lunDetail_t attr;
    char            node[EVM_MAX_NODE_NAME];      // Node that discovers
    char            cluster[SPM_MAX_CLUSTER_NAME]; // Cluster that discovers
} spmSrm_MsgNewLunInd_t; // SPM_SRM_NEW_LUN_IND

typedef struct
{
    spm_msgHdr_t    hdr;
    char            node[EVM_MAX_NODE_NAME];      // Node that delete
    char            cluster[SPM_MAX_CLUSTER_NAME]; // Cluster that delete
    char            devName[SDM_DEV_NAME_LEN+8];
} spmSrm_MsgDelLunInd_t; // SPM_SRM_DEL_LUN_IND

typedef struct
{
    spm_lunState_t frmState;
    spm_lunState_t toState;
    char           name[SDM_DEV_NAME_LEN+8];  // Device name
} spm_devStateTrans_t;

typedef struct
{
    spm_msgHdr_t        hdr;
    int32               devCnt;
    char                cluster[SPM_MAX_CLUSTER_NAME+4];
    spm_devStateTrans_t dev[1]; // List of dev's
} spm_MsgLunChangeStateInd_t;   // SPM_SRM_LUN_STATE_CHANGE_IND


/*
 * Debug Level
 */
typedef struct
{
    spm_msgHdr_t    hdr;
    uint32          dbg_level;
} spm_setDebugLevel_t;

/*----------------------------------------------------------------------------*/

#endif // _SPM_API_H_
