/*------------------------------------------------------------------------------
 * RCS:  $Id: spm-reg.c,v 1.13 2002/10/17 18:53:58 nitinj Exp $
 * Copyright (C) 2001, Agile Storage, Inc.
 *
 * Name         : spm_reg.c
 *
 * Description  : This file contains the functions to manipulate
 *                the EVM parameters in the registry
 *
 * Created by   : Isabelle Cnudde
 *
 * Date Created : 17-FEB-2002
 *
 *------------------------------------------------------------------------------
 */

#include "spm.h"

/*------------------------------------------------------------------------------
 * 
 *------------------------------------------------------------------------------
 */

typedef struct
{
    char    buf[4096];
    int32   len;
} spm_msg_buf_t;

static struct timeval waitRegm = {10, 0};

char currCluster[SPM_MAX_CLUSTER_NAME];

/*------------------------------------------------------------------------------
 * Local function prototypes.
 *------------------------------------------------------------------------------
 */

static int32
spm_regGetOption(char *option, int32 optionLen, spm_msg_buf_t *rsp_buf);


/*------------------------------------------------------------------------------
 * Name         : spm_regGetOption
 *
 * Description  : called to get an EVM option from the registry manager
 *
 * Created by   : Lan Chin
 *
 * Date Created : 05 Nov 2001
 *
 *------------------------------------------------------------------------------
 */

static int32
spm_regGetOption(char *option, int32 optionLen, spm_msg_buf_t *rsp_buf)
{
    opt_msgHdr_t        *hdr;
    opt_getOptionReq_t  *req;
    opt_getOptionRsp_t  *rsp;
    spm_msg_buf_t       msg;

    SPM_DBG_PRINT(("spm_regGetOption : Get option %s", option));

    memset((void *) msg.buf, 0, sizeof(msg.buf));
    hdr = (opt_msgHdr_t *) msg.buf;
    hdr->msgType = OPT_GET_OPTION_REQ;
    hdr->userInfo = 0;  // TBD
    req = (opt_getOptionReq_t *) (msg.buf + sizeof(opt_msgHdr_t));
    memcpy((void *) &req->option, (void *) option, optionLen);
    msg.len = sizeof(opt_msgHdr_t) + optionLen + 1;    
    rsp_buf->len = sendAgileMsg(-1, (void *) &msg.buf, msg.len, "regm",
                            NFX_SSC_UNIX, SSC_SLOT, MGMTPLANE,
                            &waitRegm, (void *) rsp_buf->buf, sizeof(rsp_buf->buf));
    if (rsp_buf->len <= 0)
    {
        SPM_DBG_PRINT(("Cannot send get option message to registry manager"));
        return NFX_ERR;
    }
    hdr = (opt_msgHdr_t *) rsp_buf->buf;
    if (hdr->msgType != OPT_GET_OPTION_RSP)
    {
        SPM_DBG_PRINT(("Invalid message type %d from registry manager",
                       hdr->msgType));
        return NFX_ERR;
    }
    rsp = (opt_getOptionRsp_t *) (rsp_buf->buf + sizeof(opt_msgHdr_t));
    if (rsp->nbEntry == 0)
        SPM_DBG_PRINT(("No registry entry found"));

    return NFX_OK;

} /* spm_regGetOption */


/*
 *------------------------------------------------------------------------------
 * Name         : checkNodeIsPCC
 *
 * Description  : Get the cluster type from registry
 *
 * Created by   : Nitin Jain
 *
 * Date Created : 27-APR-2002
 *
 *------------------------------------------------------------------------------
 */

int32 checkNodeIsPCC()
{
    int32               isPcc = 1;
    int32               rc, n;
    spm_msg_buf_t       msg;
    opt_getOptionRsp_t  *rsp;

    n = strlen(OPTIONS_CLUSTER_NODE_TYPE);
    rc = spm_regGetOption(OPTIONS_CLUSTER_NODE_TYPE, n, &msg);
    if (rc == NFX_OK)
    {
        rsp = (opt_getOptionRsp_t *) (msg.buf + sizeof(opt_msgHdr_t));
        SPM_DBG_PRINT(("checkNodeIsPCC : cluster.nodetype entries %d",
                                                             rsp->nbEntry));
        if (rsp->nbEntry)
        {
            n = strlen(OPTIONS_NODE_TYPE);
            bzero(&msg, sizeof(spm_msg_buf_t));
            rc = spm_regGetOption(OPTIONS_NODE_TYPE, n, &msg);
            if (rc == NFX_OK)
            {
                rsp = (opt_getOptionRsp_t *) (msg.buf + sizeof(opt_msgHdr_t));
                /*
                 * There is no node Type, it is stand alone.
                 * The node type is PCC.
                 */
                SPM_DBG_PRINT(("checkNodeIsPCC : node.type entries %d",
                                                             rsp->nbEntry));
                isPcc=(rsp->nbEntry && !strcasecmp(rsp->entry[0].value,"pcc"));
            }
        }
        else
            SPM_DBG_PRINT(("checkNodeIsPCC : node is Standalone"));
    }
    else
        SPM_DBG_PRINT(("checkNodeIsPCC : node is Standalone"));

    return isPcc;
}

/*------------------------------------------------------------------------------
 * Name         : spm_regGetClusterName
 *
 * Description  : Get the cluster name from registry
 *
 * Created by   : Nitin Jain
 *
 * Date Created : 04-APR-2002
 *
 *------------------------------------------------------------------------------
 */

int32 spm_regGetClusterName()
{
    int32               rc, n;
    spm_msg_buf_t       msg;
    opt_getOptionRsp_t  *rsp;

    n = strlen("cluster.name");
    rc = spm_regGetOption("cluster.name", n, &msg);
    strcpy(currCluster, "");
    if (rc != NFX_ERR)
    {
        rsp = (opt_getOptionRsp_t *) (msg.buf + sizeof(opt_msgHdr_t));
        SPM_DBG_PRINT(("Number of entries: %d", rsp->nbEntry));
        if (rsp->nbEntry)
            strcpy(currCluster, rsp->entry[0].value);
    }

    return rc;

} /* spm_regGetClusterName */

/*----------------------------------------------------------------------------*/
