/*------------------------------------------------------------------------------
 * RCS:  $Id: stats-api.h,v 1.3 2002/11/06 19:06:59 syen Exp $
 * Copyright (C) 2002, Agile Storage, Inc.
 *
 * Name         : stats-api.h
 *
 * Description  : API for communicating with Stats Agent
 *
 * Created by   : Shyh-Pei Yen
 *
 * Date Created : 10/23/02
 *
 *------------------------------------------------------------------------------
 */

#ifndef _STATS_API_H_
#define _STATS_API_H_


/*------------------------------------------------------------------------------
 * Common stats message header.
 *------------------------------------------------------------------------------
 */

typedef	enum
{
	// The External Messages
	STATS_GETSTATS = STATS_MSG_BASE,
	STATS_RESETCOUNTERS,

	STATS_DUMMY
} stats_msgType_t;

typedef	struct
{
	stats_msgType_t	msgType;		// What this message is
	uint32			len;			// length of the total message
	uint64			clientPrivate;	// Opaque to stats agent...
} stats_msgHdr_t;


//------------------------------------------------------------------------------
// Stats Agent message structures - TXRX
//------------------------------------------------------------------------------

#if defined(NFP_TXRX) || defined(SSC)

typedef struct _stats_txrxBmc12500Stats_t
{
	uint64	rxBadPkts;
	uint64	rxNoBufErrs;
	uint64	rxNoSysDescErrs;
	uint64	txQFullDrops;
} stats_txrxBmc12500Stats_t;

typedef struct _stats_txrxLsiStats_t
{
	uint64	rxBytes;
	uint64	rxPkts;
	uint64	rxBcastPkts;
	uint64	rxMcastPkts;
	uint64	rxDrops;
	uint64	rxCrcErrs;
	uint64	rxUndersize;
	uint64	rxPausePkts;
	uint64	rxSymbolErrs;

	uint64	txBytes;
	uint64	txPkts;
	uint64	txBcastPkts;
	uint64	txMcastPkts;
	uint64	txUnderflows;
	uint64	txErrors;
	uint64	txPausePkts;
} stats_txrxLsiStats_t;

typedef struct _stats_txrxLsiPktDist_t
{
	uint64	rx_64;
	uint64	rx65_127;
	uint64	rx128_255;
	uint64	rx256_511;
	uint64	rx512_1023;
	uint64	rx1024_max;

	uint64	tx_64;
	uint64	tx65_127;
	uint64	tx128_255;
	uint64	tx256_511;
	uint64	tx512_1023;
	uint64	tx1024_max;
} stats_txrxLsiPktDist_t;

typedef struct _stats_txrxDp83820Stats_t
{
	uint64	rxBytes;
	uint64	rxPkts;
	uint64	rxErrors;
	uint64	rxFcsErrs;
	uint64	rxOverruns;
	uint64	rxAlignErrs;
	uint64	rxSymbolErrs;
	uint64	rxLenErrs;
	uint64	rxAborts;
	uint64	rxNoBufErrs;
	uint64	rxNoSysDescs;
	uint64	rxRuntPkts;
	uint64	rxCrcErrs;

	uint64	txBytes;
	uint64	txPkts;
	uint64	txQFullDrops;
	uint64	txAborts;
	uint64	txUnderruns;
} stats_txrxDp83820Stats_t;

typedef	struct _stats_GetTxRxStatsReq_t
{
	stats_msgHdr_t		hdr;		// STATS_GETSTATS
#define STATS_TXRX_BMC12500		0
#define STATS_TXRX_LSI			1
#define STATS_TXRX_LSI_DIST		2
#define STATS_TXRX_DP83820		3
	uint32				type;
	uint32				port;		// for GbE MAC stats only
} stats_GetTxRxStatsReq_t;

typedef	struct _stats_GetTxRxStatsRsp_t
{
	stats_msgHdr_t	hdr;		// STATS_GETSTATS
	int32			status;
	uint32			type;		// same as type in request
	uint32			port;		// for GbE MAC stats only
	char			data[0];	// above defined data appended at this end
} stats_GetTxRxStatsRsp_t;

#endif


//------------------------------------------------------------------------------
// Stats Agent message structures - FP
//------------------------------------------------------------------------------

#if defined(NFP_FP) || defined(SSC)

typedef struct _stats_fpBmc12500Stats_t
{
	uint64	rxBadPkts;
	uint64	rxNoBufErrs;
	uint64	rxNoSysDescErrs;
	uint64	txQFullDrops;
} stats_fpBmc12500Stats_t;

typedef struct _stats_fpLsiStats_t
{
	uint64	rxBytes;
	uint64	rxPkts;
	uint64	rxDrops;
	uint64	rxCrcErrs;
	uint64	rxUndersize;
	uint64	rxSymbolErrs;
	uint64	txBytes;
	uint64	txPkts;
	uint64	txUnderflows;
	uint64	txErrors;
} stats_fpLsiStats_t;

typedef	struct _stats_GetFPStatsReq_t
{
	stats_msgHdr_t		hdr;		// STATS_GETSTATS
#define STATS_FP_BMC12500		0
#define STATS_FP_LSI			1
	uint32				type;
	uint32				port;		// for lsi stats only
} stats_GetFPStatsReq_t;

typedef	struct _stats_GetFPStatsRsp_t
{
	stats_msgHdr_t	hdr;		// STATS_GETSTATS
	int32			status;
	uint32			type;		// same as type in request
	uint32			port;		// for lsi stats only
	char			data[0];	// above defined data appended at this end
} stats_GetFPStatsRsp_t;

#endif


//------------------------------------------------------------------------------
// Stats Agent message structures - FC
//------------------------------------------------------------------------------

#if defined(FCNIM) || defined(SSC)

typedef struct _stats_fcDp83820Stats_t
{
	uint64	rxBytes;
	uint64	rxPkts;
	uint64	rxErrors;
	uint64	rxFcsErrs;
	uint64	rxOverruns;
	uint64	rxAlignErrs;
	uint64	rxSymbolErrs;
	uint64	rxLenErrs;
	uint64	rxAborts;
	uint64	rxNoBufErrs;
	uint64	rxNoSysDescs;
	uint64	rxRuntPkts;
	uint64	rxCrcErrs;

	uint64	txBytes;
	uint64	txPkts;
	uint64	txQFullDrops;
	uint64	txAborts;
	uint64	txUnderruns;
} stats_fcDp83820Stats_t;

typedef struct _stats_fcGt64240Stats_t
{
	uint64	rxBytes;
	uint64	rxPkts;
	uint64	rxCrcErrs;
	uint64	rxFragments;
	uint64	rxJabbers;
	uint64	rxCollisions;
	uint64	rxLateCollis;
	uint64	rxErrors;
	uint64	rxUndersized;
	uint64	rxNoSysDescs;
	uint64	rxNoBufErrs;
	uint64	txBytes;
	uint64	txPkts;
	uint64	txQFullDrops;
	uint64	txUnderruns;
} stats_fcGt64240Stats_t;

typedef	struct _stats_GetFCStatsReq_t
{
	stats_msgHdr_t		hdr;		// STATS_GETSTATS
#define STATS_FC_DP83820		0
#define STATS_FC_GT64240		1
	uint32				type;
} stats_GetFCStatsReq_t;

typedef	struct _stats_GetFCStatsRsp_t
{
	stats_msgHdr_t	hdr;		// STATS_GETSTATS
	int32			status;
	uint32			type;		// same as type in request
	char			data[0];	// above defined data appended at this end
} stats_GetFCStatsRsp_t;

#endif


//------------------------------------------------------------------------------
// Stats Agent message structures - SSC_MGMT
//------------------------------------------------------------------------------

#if defined(SSC_MGMT) || defined(SSC)

typedef struct _stats_sscmgmtBcm5600Stats_t
{
	uint64	rxBytes;
	uint64	rxPkts;
	uint64	rxFcsErrs;
	uint64	rxAlignErrs;
	uint64	rxSymbolErrs;
	uint64	rxFalseCarrs;
	uint64	rxUndersized;
	uint64	rxFragments;
	uint64	rxJabbers;
	uint64	rxDiscards;
	uint64	txBytes;
	uint64	txPkts;
	uint64	txFcsErrs;
	uint64	txAborts;
	uint64	txFragments;
	uint64	txJabbers;
} stats_sscmgmtBcm5600Stats_t;

typedef	struct _stats_GetSscMgmtStatsReq_t
{
	stats_msgHdr_t		hdr;		// STATS_GETSTATS
#define STATS_SSCMGMT_BCM5600	0
	uint32				type;
#define STATS_SSCMGMT_PORT_FC0	0
#define STATS_SSCMGMT_PORT_FC1	1
#define STATS_SSCMGMT_PORT_FC2	2
#define STATS_SSCMGMT_PORT_FC3	3
#define STATS_SSCMGMT_PORT_TXRX	4
#define STATS_SSCMGMT_PORT_SSC	23
#define STATS_SSCMGMT_TOTAL_PORTS	6
	uint32				port;
} stats_GetSscMgmtStatsReq_t;

typedef	struct _stats_GetSscMgmtStatsRsp_t
{
	stats_msgHdr_t	hdr;		// STATS_GETSTATS
	int32			status;
	uint32			type;		// same as type in request
	uint32			port;
	char			data[0];	// above defined data appended at this end
} stats_GetSscMgmtStatsRsp_t;

#endif


//------------------------------------------------------------------------------
// Common request/response data structures
//------------------------------------------------------------------------------

typedef struct _stats_resetCountersReq_t
{
	stats_msgHdr_t	hdr;		// STATS_RESETCOUNTERS
	uint32			type;		// same as type in stats_GetTxRxStatsReq_t
	uint32			port;		// for GbE MAC stats only
} stats_resetCountersReq_t;

typedef struct _stats_resetCountersRsp_t
{
	stats_msgHdr_t	hdr;		// STATS_RESETCOUNTERS
	uint32			type;		// same as type in request
	uint32			port;		// for GbE MAC stats only
	int32			status;
} stats_resetCountersRsp_t;


#endif		// _STATS_API_H_
