//------------------------------------------------------------------------------
// RCS:  $Id: stats-fp.c,v 1.5 2002/11/15 22:16:06 syen Exp $
// Copyright (C) 2002, Agile Storage, Inc.
//
// Name         : stats-fp.c
//
// Description  : 
//
// Created by   : Shyh-Pei Yen
//
// Date Created : 10/29/2002
//
//------------------------------------------------------------------------------


#if defined(NFP_FP)

#include "stats.h"
#include "stats-dbg.h"

//------------------------------------------------------------------------------
// Global variables
//------------------------------------------------------------------------------

stats_fpBmc12500Stats_t	fpSysStats;
stats_fpLsiStats_t		fpLsiStats[4];	// 4 ports

typedef struct _bmc12500Stats_t
{
	uint32 rxBadPkts;
	uint32 rxNoBufErrs;
	uint32 rxNoSysDesc;
	uint32 txQueueFull;
} bmc12500Stats_t;

typedef struct _lsiStats_t
{
	uint32 rxDrops;
	uint32 rxBytes;
	uint32 rxPkts;
	uint32 rxCrcErrs;
	uint32 rxUndersize;
	uint32 txBytes;
	uint32 txPkts;
	uint32 txUnderflow;
	uint32 txErrs;
	uint32 rxSymErrs;
} lsiStats_t;

static int statsStep = 0;


// Local functions prototype.
static void stats_GetBmc12500Stats(int interface);
static void stats_GetLsiStats(int port);

// external functions prototype
void bmc12500Eth_get_stats(int interface, uint32 *bufptr);
void lsi_get_stats(int port, uint32 *bufptr);


//------------------------------------------------------------------------------
// Name         : init_statsCounters()
// 
// Description  : 
//
// Created by   : Shyh-Pei Yen
// 
// Date Created : 10/29/02
//
//------------------------------------------------------------------------------

void
init_statsCounters()
{
	memset(&fpSysStats,  0, sizeof(stats_fpBmc12500Stats_t));
	memset(fpLsiStats,   0, sizeof(stats_fpLsiStats_t)*4);
}


//------------------------------------------------------------------------------
// Name         : stats_GetBmc12500Stats
// 
// Description  : 
//
// Created by   : Shyh-Pei Yen
// 
// Date Created : 10/29/02
//
//------------------------------------------------------------------------------

static void
stats_GetBmc12500Stats(int interface)
{
bmc12500Stats_t stats;

	stats_trace(LOG_DEBUG, "> stats_GetBmc12500Stats()\n");

	memset(&stats, 0, sizeof(bmc12500Stats_t));

	bmc12500Eth_get_stats(interface, (uint32 *) &stats);

	fpSysStats.rxBadPkts		+= (uint64) stats.rxBadPkts;
	fpSysStats.rxNoBufErrs		+= (uint64) stats.rxNoBufErrs;
	fpSysStats.rxNoSysDescErrs	+= (uint64) stats.rxNoSysDesc;
	fpSysStats.txQFullDrops		+= (uint64) stats.txQueueFull;

	stats_trace(LOG_DEBUG, "< stats_GetBmc12500Stats()\n");
}


//------------------------------------------------------------------------------
// Name         : stats_GetLsiStats
// 
// Description  : 
//
// Created by   : Shyh-Pei Yen
// 
// Date Created : 10/29/02
//
//------------------------------------------------------------------------------

static void
stats_GetLsiStats(int port)
{
lsiStats_t stats;

	stats_trace(LOG_DEBUG, "> stats_GetLsiStats()\n");

	memset(&stats, 0, sizeof(lsiStats_t));
	lsi_get_stats(port, (uint32 *) &stats);

	fpLsiStats[port].rxBytes		+= (uint64) stats.rxBytes;
	fpLsiStats[port].rxPkts			+= (uint64) stats.rxPkts;
	fpLsiStats[port].rxDrops		+= (uint64) stats.rxDrops;
	fpLsiStats[port].rxCrcErrs		+= (uint64) stats.rxCrcErrs;
	fpLsiStats[port].rxUndersize	+= (uint64) stats.rxUndersize;
	fpLsiStats[port].rxSymbolErrs	+= (uint64) stats.rxSymErrs;

	fpLsiStats[port].txBytes		+= (uint64) stats.txBytes;
	fpLsiStats[port].txPkts			+= (uint64) stats.txPkts;
	fpLsiStats[port].txUnderflows	+= (uint64) stats.txUnderflow;
	fpLsiStats[port].txErrors		+= (uint64) stats.txErrs;

	stats_trace(LOG_DEBUG, "< stats_GetLsiStats()\n");
	return;
}


//------------------------------------------------------------------------------
// Name         : statsTimerProc
// 
// Description  : Callback function when timer expires.
//                Timer wake up every 5 seconds for FP stats collector.
//                This is defined in stats-fp.h with STATS_TICKS.
//                For each wake up, get the specified stats below. After
//                all 4 stats are finished, we cycle it again...
//                0. lsi port 0
//                1. lsi port 1, bmc12500 0
//                2. lsi port 2
//                3. lsi port 3, bmc12500 1
//
// Created by   : Shyh-Pei Yen
// 
// Date Created : 10/29/02
//
//------------------------------------------------------------------------------

int32
statsTimerProc(int32 tmr_handle,
	void *usr_param1, void *usr_param2, void *usr_param3, void *usr_param4)
{
	stats_trace(LOG_DEBUG, "> statsTimerProc() - step %d\n", statsStep);

#if 1
// this code should be removed for formal release
	if (!collect_data)
		return NFX_OK;
#endif

	stats_GetLsiStats(statsStep);
	// Only interface 0 and 2 for Cheetah BMC12500
	// For Garfield, it should be 0 and 1
	if (statsStep % 2)
	{
		if (isCheetah())
			stats_GetBmc12500Stats(statsStep-1);
		else
			stats_GetBmc12500Stats(statsStep/2);
	}

	statsStep++;
	if (statsStep > 3)
		statsStep = 0;

	stats_trace(LOG_DEBUG, "< statsTimerProc()\n");

	return NFX_OK;
}


//------------------------------------------------------------------------------
// Name         : stats_processGetStats
//
// Description  : 
//
// Created by   : Shyh-Pei Yen
//
// Date Created : 10/29/02
//------------------------------------------------------------------------------

int32
stats_getStats(char *msgBuf, uint32 msgLen, uint32 srcPort, uint32 dstPort)
{
	stats_GetFPStatsReq_t	*req = (stats_GetFPStatsReq_t *) msgBuf;
	stats_GetFPStatsRsp_t	*rsp;
	uint32					rspSize, dataSize = 0;
	char					*ptr = NULL;

	stats_trace(LOG_DEBUG, "> stats_processGetStats()");

	NFX_ASSERT(msgLen >= sizeof(stats_GetFPStatsReq_t));

	switch (req->type)
	{
	case STATS_FP_BMC12500:
		dataSize = sizeof(stats_fpBmc12500Stats_t);
		ptr = (char *) &fpSysStats;
		break;
	case STATS_FP_LSI:
		if (req->port < 4)
		{
			dataSize = sizeof(stats_fpLsiStats_t);
			ptr = (char *) &fpLsiStats[req->port];
		}
		break;
	}

	rspSize = sizeof(stats_GetFPStatsRsp_t) + dataSize;
	rsp = (stats_GetFPStatsRsp_t *) eee_ramAllocZero(rspSize);
	NFX_ASSERT(rsp != NULL);

	rsp->hdr.msgType = req->hdr.msgType;
	rsp->hdr.clientPrivate = req->hdr.clientPrivate;
	rsp->type = req->type;
	rsp->port = req->port;

	if (ptr)
	{
		rsp->status = NFX_OK;
		NFX_SPIN_LOCK(&stats_lock);
		memcpy(rsp->data, ptr, dataSize);
		NFX_SPIN_UNLOCK(&stats_lock);
	}
	else
		rsp->status = NFX_ERR;

	eee_sendMessage(rsp, rspSize, srcPort, dstPort);

	stats_trace(LOG_DEBUG, "> stats_processGetStats()");
	return NFX_OK;
}

//------------------------------------------------------------------------------
// Name         : stats_resetCounters
//
// Description  : 
//
// Created by   : Shyh-Pei Yen
//
// Date Created : 10/29/02
//------------------------------------------------------------------------------

void
stats_resetCounters(char *msgBuf, uint32 msgLen, uint32 srcPort, uint32 dstPort)
{
	stats_resetCountersReq_t	*req = (stats_resetCountersReq_t *) msgBuf;
	stats_resetCountersRsp_t	rsp;

	stats_trace(LOG_DEBUG, "> stats_resetCounters()");

	NFX_ASSERT(msgLen >= sizeof(stats_resetCountersReq_t));

	rsp.hdr.msgType = req->hdr.msgType;
	rsp.hdr.clientPrivate = req->hdr.clientPrivate;
	rsp.status = NFX_OK;
	rsp.type = req->type;

	NFX_SPIN_LOCK(&stats_lock);
	switch (req->type)
	{
	case STATS_FP_BMC12500:
		memset(&fpSysStats, 0, sizeof(stats_fpBmc12500Stats_t));
		break;
	case STATS_FP_LSI:
		if (req->port < 4)
			memset(&fpLsiStats[req->port], 0, sizeof(stats_fpLsiStats_t));
		else
			rsp.status = NFX_ERR;
		break;
	default:
		rsp.status = NFX_ERR;
		break;
	}
	NFX_SPIN_UNLOCK(&stats_lock);

	eee_sendMessage(&rsp, sizeof(stats_resetCountersRsp_t), srcPort, dstPort);

	stats_trace(LOG_DEBUG, "> stats_resetCounters()");
	return;
}

#endif		// defined(NFP_FP)
