//------------------------------------------------------------------------------
// RCS:  $Id: stats-txrx.c,v 1.7 2002/11/15 22:16:06 syen Exp $
// Copyright (C) 2002, Agile Storage, Inc.
//
// Name         : stats-txrx.c
//
// Description  : 
//
// Created by   : Shyh-Pei Yen
//
// Date Created : 10/23/2002
//
//------------------------------------------------------------------------------


#if defined(NFP_TXRX)

#include "stats.h"
#include "stats-dbg.h"

//------------------------------------------------------------------------------
// Global variables
//------------------------------------------------------------------------------

stats_txrxBmc12500Stats_t	txrxBmc12500Stats;
stats_txrxLsiStats_t		txrxLsiStats[4];	// 4 GbE ports
stats_txrxLsiPktDist_t		txrxLsiPktDist[4];
stats_txrxDp83820Stats_t	txrxDp83820Stats;

typedef struct _bmc12500Stats_t
{
	uint32 rxBadPkts;
	uint32 rxNoBufErrs;
	uint32 rxNoSysDesc;
	uint32 txQueueFull;
} bmc12500Stats_t;

typedef struct _lsiStats_t
{
	uint32 rxDrops;
	uint32 rxBytes;
	uint32 rxPkts;
	uint32 rxBcastPkts;
	uint32 rxMcastPkts;
	uint32 rxCrcErrs;
	uint32 rxUndersize;
	uint32 rx64Pkts;
	uint32 rx65_127Pkts;
	uint32 rx128_255Pkts;
	uint32 rx256_511Pkts;
	uint32 rx512_1kPkts;
	uint32 rx1kupPkts;
	uint32 txBytes;
	uint32 txPkts;
	uint32 txBcastPkts;
	uint32 txMcastPkts;
	uint32 tx64Pkts;
	uint32 tx65_127Pkts;
	uint32 tx128_255Pkts;
	uint32 tx256_511Pkts;
	uint32 tx512_1kPkts;
	uint32 tx1kupPkts;
	uint32 txUnderflow;
	uint32 txErrs;
	uint32 rxSymErrs;
	uint32 txPausePkts;
	uint32 rxPausePkts;
} lsiStats_t;

typedef struct _dp83820Stats_t
{
	uint32 rxErrs;
	uint32 rxFcsErrs;
	uint32 rxOverruns;
	uint32 rxFrmAlignErrs;
	uint32 rxSymErrs;
	uint32 rxLenErrs;
	uint32 rxPkts;
	uint32 rxBytes;
	uint32 rxAborts;
	uint32 rxNoBufErrs;
	uint32 rxNoSysDesc;
	uint32 rxRuntPkts;
	uint32 rxCrcErrs;
	uint32 txPkts;
	uint32 txBytes;
	uint32 txQueueFull;
	uint32 txAborts;
	uint32 txUnderrun;
} dp83820Stats_t;


static int statsStep = 1;


// Local functions prototype.
static void stats_GetBmc12500Stats(void);
static void stats_GetLSIStats(int port);
static void stats_GetDp83820Stats(void);

// external functions prototype
void bmc12500Eth_get_stats(int interface, uint32 *bufptr);
void lsi_get_stats(int port, uint32 *bufptr);
void dp83820_get_stats(uint32 *bufptr);



//------------------------------------------------------------------------------
// Name         : init_statsCounters()
// 
// Description  : 
//
// Created by   : Shyh-Pei Yen
// 
// Date Created : 10/24/02
//
//------------------------------------------------------------------------------

void
init_statsCounters()
{
	memset(&txrxBmc12500Stats, 0, sizeof(stats_txrxBmc12500Stats_t));
	memset(txrxLsiStats,       0, sizeof(stats_txrxLsiStats_t)*4);
	memset(txrxLsiPktDist,     0, sizeof(stats_txrxLsiPktDist_t)*4);
	memset(&txrxDp83820Stats,  0, sizeof(stats_txrxDp83820Stats_t));
}


//------------------------------------------------------------------------------
// Name         : stats_GetBmc12500Stats
// 
// Description  : 
//
// Created by   : Shyh-Pei Yen
// 
// Date Created : 10/24/02
//
//------------------------------------------------------------------------------

static void
stats_GetBmc12500Stats()
{
int interface;
bmc12500Stats_t stats;

	stats_trace(LOG_DEBUG, "> stats_GetBmc12500Stats()\n");

	memset(&stats, 0, sizeof(bmc12500Stats_t));
	// interface 0 and 2 are valid for Cheetah
	// for Garfield it should be 0 and 1
	for (interface = 0; interface < 2; interface++)
	{
		if (isCheetah())
			bmc12500Eth_get_stats(interface*2, (uint32 *) &stats);
		else
			bmc12500Eth_get_stats(interface, (uint32 *) &stats);

		txrxBmc12500Stats.rxBadPkts			+= (uint64) stats.rxBadPkts;
		txrxBmc12500Stats.rxNoBufErrs		+= (uint64) stats.rxNoBufErrs;
		txrxBmc12500Stats.rxNoSysDescErrs	+= (uint64) stats.rxNoSysDesc;
		txrxBmc12500Stats.txQFullDrops		+= (uint64) stats.txQueueFull;
	}

	stats_trace(LOG_DEBUG, "< stats_GetBmc12500Stats()\n");
}


//------------------------------------------------------------------------------
// Name         : stats_GetLsiPortStats
// 
// Description  : 
//
// Created by   : Shyh-Pei Yen
// 
// Date Created : 10/24/02
//
//------------------------------------------------------------------------------

static void
stats_GetLsiPortStats(int port)
{
lsiStats_t stats;

	stats_trace(LOG_DEBUG, "> stats_GetLsiPortStats()\n");

	memset(&stats, 0, sizeof(lsiStats_t));
	lsi_get_stats(port, (uint32 *) &stats);

	txrxLsiStats[port].rxBytes		+= (uint64) stats.rxBytes;
	txrxLsiStats[port].rxPkts		+= (uint64) stats.rxPkts;
	txrxLsiStats[port].rxBcastPkts	+= (uint64) stats.rxBcastPkts;
	txrxLsiStats[port].rxMcastPkts	+= (uint64) stats.rxMcastPkts;
	txrxLsiStats[port].rxDrops		+= (uint64) stats.rxDrops;
	txrxLsiStats[port].rxCrcErrs	+= (uint64) stats.rxCrcErrs;
	txrxLsiStats[port].rxUndersize	+= (uint64) stats.rxUndersize;
	txrxLsiStats[port].rxPausePkts	+= (uint64) stats.rxPausePkts;
	txrxLsiStats[port].rxSymbolErrs	+= (uint64) stats.rxSymErrs;

	txrxLsiStats[port].txBytes		+= (uint64) stats.txBytes;
	txrxLsiStats[port].txPkts		+= (uint64) stats.txPkts;
	txrxLsiStats[port].txBcastPkts	+= (uint64) stats.txBcastPkts;
	txrxLsiStats[port].txMcastPkts	+= (uint64) stats.txMcastPkts;
	txrxLsiStats[port].txUnderflows	+= (uint64) stats.txUnderflow;
	txrxLsiStats[port].txErrors		+= (uint64) stats.txErrs;
	txrxLsiStats[port].txPausePkts	+= (uint64) stats.txPausePkts;

	txrxLsiPktDist[port].rx_64			+= (uint64) stats.rx64Pkts;
	txrxLsiPktDist[port].rx65_127		+= (uint64) stats.rx65_127Pkts;
	txrxLsiPktDist[port].rx128_255		+= (uint64) stats.rx128_255Pkts;
	txrxLsiPktDist[port].rx256_511		+= (uint64) stats.rx256_511Pkts;
	txrxLsiPktDist[port].rx512_1023	+= (uint64) stats.rx512_1kPkts;
	txrxLsiPktDist[port].rx1024_max	+= (uint64) stats.rx1kupPkts;

	txrxLsiPktDist[port].tx_64			+= (uint64) stats.tx64Pkts;
	txrxLsiPktDist[port].tx65_127		+= (uint64) stats.tx65_127Pkts;
	txrxLsiPktDist[port].tx128_255		+= (uint64) stats.tx128_255Pkts;
	txrxLsiPktDist[port].tx256_511		+= (uint64) stats.tx256_511Pkts;
	txrxLsiPktDist[port].tx512_1023	+= (uint64) stats.tx512_1kPkts;
	txrxLsiPktDist[port].tx1024_max	+= (uint64) stats.tx1kupPkts;

	stats_trace(LOG_DEBUG, "< stats_GetLsiPortStats()\n");
	return;
}


//------------------------------------------------------------------------------
// Name         : stats_GetDp83820Stats
// 
// Description  : 
//
// Created by   : Shyh-Pei Yen
// 
// Date Created : 10/24/02
//
//------------------------------------------------------------------------------

static void
stats_GetDp83820Stats()
{
dp83820Stats_t stats;

	stats_trace(LOG_DEBUG, "> stats_GetDp83820Stats()\n");

	memset(&stats, 0, sizeof(dp83820Stats_t));
	dp83820_get_stats((uint32 *) &stats);

	txrxDp83820Stats.rxBytes		+= stats.rxBytes;
	txrxDp83820Stats.rxPkts			+= stats.rxPkts;
	txrxDp83820Stats.rxErrors		+= stats.rxErrs;
	txrxDp83820Stats.rxFcsErrs		+= stats.rxFcsErrs;
	txrxDp83820Stats.rxOverruns		+= stats.rxOverruns;
	txrxDp83820Stats.rxAlignErrs	+= stats.rxFrmAlignErrs;
	txrxDp83820Stats.rxSymbolErrs	+= stats.rxSymErrs;
	txrxDp83820Stats.rxAborts		+= stats.rxAborts;
	txrxDp83820Stats.rxNoBufErrs	+= stats.rxNoBufErrs;
	txrxDp83820Stats.rxNoSysDescs	+= stats.rxNoSysDesc;
	txrxDp83820Stats.rxRuntPkts		+= stats.rxRuntPkts;
	txrxDp83820Stats.rxCrcErrs		+= stats.rxCrcErrs;
	txrxDp83820Stats.txBytes		+= stats.txBytes;
	txrxDp83820Stats.txPkts			+= stats.txPkts;
	txrxDp83820Stats.txQFullDrops	+= stats.txQueueFull;
	txrxDp83820Stats.txAborts		+= stats.txAborts;
	txrxDp83820Stats.txUnderruns	+= stats.txUnderrun;

	stats_trace(LOG_DEBUG, "< stats_GetDp83820Stats()\n");
	return;
}


//------------------------------------------------------------------------------
// Name         : statsTimerProc
// 
// Description  : Callback function when timer expires.
//                Timer wake up every 2.5 seconds for TXRX stats collector.
//                This is defined in stats-txrx.h with STATS_TICKS.
//                For each wake up, get the specified stats below. After
//                all 8 stats are finished, we cycle it again...
//                1. GbE port 0
//                2. system
//                3. GbE port 1
//                4. backplane
//                5. GbE port 2
//                6. system
//                7. GbE port 3
//                8. system
//
// Created by   : Shyh-Pei Yen
// 
// Date Created : 10/23/02
//
//------------------------------------------------------------------------------

int32
statsTimerProc(int32 tmr_handle,
	void *usr_param1, void *usr_param2, void *usr_param3, void *usr_param4)
{
	stats_trace(LOG_DEBUG, "> statsTimerProc() - step %d\n", statsStep);

#if 1
// this code should be removed for formal release
	if (!collect_data)
		return NFX_OK;
#endif

	if (statsStep % 2)
		stats_GetLsiPortStats(statsStep/2);
	else if (statsStep == 4)
		stats_GetDp83820Stats();
	else
		stats_GetBmc12500Stats();

	statsStep++;
	if (statsStep > 8)
		statsStep = 1;

	stats_trace(LOG_DEBUG, "< statsTimerProc()\n");

	return NFX_OK;
}


//------------------------------------------------------------------------------
// Name         : stats_processGetStats
//
// Description  : 
//
// Created by   : Shyh-Pei Yen
//
// Date Created : 10/23/02
//------------------------------------------------------------------------------

int32
stats_getStats(char *msgBuf, uint32 msgLen, uint32 srcPort, uint32 dstPort)
{
	stats_GetTxRxStatsReq_t	*req = (stats_GetTxRxStatsReq_t *) msgBuf;
	stats_GetTxRxStatsRsp_t	*rsp;
	uint32					rspSize, dataSize = 0;
	char					*ptr = NULL;

	stats_trace(LOG_DEBUG, "> stats_processGetStats()");

	NFX_ASSERT(msgLen >= sizeof(stats_GetTxRxStatsReq_t));

	switch (req->type)
	{
	case STATS_TXRX_BMC12500:
		dataSize = sizeof(stats_txrxBmc12500Stats_t);
		ptr = (char *) &txrxBmc12500Stats;
		break;
	case STATS_TXRX_LSI:
		if (req->port < 4)
		{
			dataSize = sizeof(stats_txrxLsiStats_t);
			ptr = (char *) &txrxLsiStats[req->port];
		}
		break;
	case STATS_TXRX_LSI_DIST:
		if (req->port < 4)
		{
			dataSize = sizeof(stats_txrxLsiPktDist_t);
			ptr = (char *) &txrxLsiPktDist[req->port];
		}
		break;
	case STATS_TXRX_DP83820:
		dataSize = sizeof(stats_txrxDp83820Stats_t);
		ptr = (char *) &txrxDp83820Stats;
		break;
	}

	rspSize = sizeof(stats_GetTxRxStatsRsp_t) + dataSize;
	rsp = (stats_GetTxRxStatsRsp_t	*) eee_ramAllocZero(rspSize);
	NFX_ASSERT(rsp != NULL);

	rsp->hdr.msgType = req->hdr.msgType;
	rsp->hdr.clientPrivate = req->hdr.clientPrivate;
	rsp->type = req->type;
	rsp->port = req->port;

	if (ptr)
	{
		rsp->status = NFX_OK;

		NFX_SPIN_LOCK(&stats_lock);
		memcpy(rsp->data, ptr, dataSize);
		NFX_SPIN_UNLOCK(&stats_lock);
	}
	else
		rsp->status = NFX_ERR;

	eee_sendMessage(rsp, rspSize, srcPort, dstPort);

	stats_trace(LOG_DEBUG, "> stats_processGetStats()");
	return NFX_OK;
}

//------------------------------------------------------------------------------
// Name         : stats_resetCounters
//
// Description  : 
//
// Created by   : Shyh-Pei Yen
//
// Date Created : 10/28/02
//------------------------------------------------------------------------------

void
stats_resetCounters(char *msgBuf, uint32 msgLen, uint32 srcPort, uint32 dstPort)
{
	stats_resetCountersReq_t	*req = (stats_resetCountersReq_t *) msgBuf;
	stats_resetCountersRsp_t	rsp;

	stats_trace(LOG_DEBUG, "> stats_resetCounters()");

	NFX_ASSERT(msgLen >= sizeof(stats_resetCountersReq_t));

	rsp.hdr.msgType = req->hdr.msgType;
	rsp.hdr.clientPrivate = req->hdr.clientPrivate;
	rsp.status = NFX_OK;
	rsp.type = req->type;

	NFX_SPIN_LOCK(&stats_lock);
	switch (req->type)
	{
	case STATS_TXRX_BMC12500:
		memset(&txrxBmc12500Stats, 0, sizeof(stats_txrxBmc12500Stats_t));
		break;
	case STATS_TXRX_LSI:
		if (req->port < 4)
			memset(&txrxLsiStats[req->port], 0, sizeof(stats_txrxLsiStats_t));
		else
			rsp.status = NFX_ERR;
		break;
	case STATS_TXRX_LSI_DIST:
		if (req->port < 4)
			memset(&txrxLsiPktDist[req->port], 0, sizeof(stats_txrxLsiPktDist_t));
		else
			rsp.status = NFX_ERR;
		break;
	case STATS_TXRX_DP83820:
		memset(&txrxDp83820Stats, 0, sizeof(stats_txrxDp83820Stats_t));
		break;
	default:
		rsp.status = NFX_ERR;
		break;
	}
	NFX_SPIN_UNLOCK(&stats_lock);

	eee_sendMessage(&rsp, sizeof(stats_resetCountersRsp_t), srcPort, dstPort);

	stats_trace(LOG_DEBUG, "> stats_resetCounters()");
	return;
}

#endif		// defined(NFP_TXRX)
