/*
 *-----------------------------------------------------------------
 *
 * Name:	tlbinit.S
 *
 * RCS:  $Id: tlbinit.S,v 1.1.1.1 2001/10/31 01:27:27 rick Exp $
 *
 * Copyright (C) 2001, Agile Storage, Inc.
 *
 * Description:	Initialize the TLB entries
 *
 * Created by:	Rick Lund
 *
 * Date Created:	6/18/01
 *
 *-----------------------------------------------------------------
 */

#include <nfx-incl.h>
#include <exports/hwMap.h>
#include <sys/mips.h>

	.text

	.globl	init_tlb
	.globl	tlb_inc
	.globl	tlb_error

#define INDEX		t8
#define SIZE		t9
#define PHYSICAL	a0
#define VIRTUAL		a1
#define TABLE		a2

/*-----------------------------------------------------------------
 * Name:	init_tlb()
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	6/18/01
 *
 *-----------------------------------------------------------------
 */
	.ent	init_tlb
init_tlb:
	move	v0,ra

	/*
	 * initialise tlb
	 */
	mtc0	zero,C0_TLBLO0		/* tlblo0 = invalid */
	mtc0	zero,C0_TLBLO1		/* tlblo1 = invalid */
	mtc0	zero,C0_PGMASK
	li		t8,K1BASE			/* tlbhi  = impossible vpn */
	li		t9,(NTLBENTRIES-1)	/* index */

	.set noreorder
	nop
1:
	mtc0	t8,C0_TLBHI
	mtc0	t9,C0_INX
	addu	t8,0x2000		/* inc vpn */
	tlbwi
	bnez	t9,1b
	subu	t9,1			# BDSLOT

	/*
	 * Set up mappings for our board
	 */
	li		PHYSICAL,HW_EVAL_BOARD_RAM_BASE
	li		VIRTUAL,HW_LOCAL_MEMORY_BASE
	la		TABLE,tlb_mem_32

	move	INDEX,zero
tlb_install_entries:
	lw		SIZE,0(TABLE)
	beq		SIZE,-1,tlb_done
	nop
	mtc0	INDEX,C0_INX
	mtc0	SIZE,C0_PGMASK
	move	t1,VIRTUAL
	and		t1,TLB_ENTHI_ADDR_MASK
	mtc0	t1,C0_TLBHI
	move	t1,PHYSICAL
	and		t1,TLB_ENTLO_ADDR_MASK
	srl		t1,TLB_ENTLO_ADDR_SHIFT
	or		t1,TLB_ENTLO_GLOBAL
	or		t1,TLB_ENTLO_VALID
	or		t1,TLB_ENTLO_DIRTY
	or		t1,TLB_ENTLO_CACHED
	mtc0	t1,C0_TLBLO0

	jal		tlb_inc
	nop

	move	t1,PHYSICAL
	and		t1,TLB_ENTLO_ADDR_MASK
	srl		t1,TLB_ENTLO_ADDR_SHIFT
	or		t1,TLB_ENTLO_GLOBAL
	or		t1,TLB_ENTLO_VALID
	or		t1,TLB_ENTLO_DIRTY
	or		t1,TLB_ENTLO_CACHED
	mtc0	t1,C0_TLBLO1

	jal		tlb_inc
	nop

	tlbwi
	nop; nop; nop

	add		INDEX,1
	add		TABLE,4
	b		tlb_install_entries

	.set	reorder

tlb_done:
	move	ra,v0
	j		ra

	.end	init_tlb

/*-----------------------------------------------------------------
 * Name:	tlb_inc()
 *
 * Description:	Increment the physical and virtual addresses by the
 *				page size
 *
 * Created by:	Rick Lund
 *
 * Date Created:	6/18/01
 *
 *-----------------------------------------------------------------
 */
	.ent	tlb_inc
tlb_inc:
	beq		SIZE,TLB_PGMASK_256K,tlb_add_256k
	beq		SIZE,TLB_PGMASK_1M,tlb_add_1
	beq		SIZE,TLB_PGMASK_4M,tlb_add_4
	beq		SIZE,TLB_PGMASK_16M,tlb_add_16
	j		tlb_error

tlb_add_256K:
	add		PHYSICAL,0x40000
	add		VIRTUAL,0x40000
	j		ra
tlb_add_1:
	add		PHYSICAL,0x100000
	add		VIRTUAL,0x100000
	j		ra
tlb_add_4:
	add		PHYSICAL,0x400000
	add		VIRTUAL,0x400000
	j		ra
tlb_add_16:
	add		PHYSICAL,0x1000000
	add		VIRTUAL,0x1000000
	j		ra

	.end	tlb_inc

/*-----------------------------------------------------------------
 * Name:	tlb_error()
 *
 * Description:	There was an error during TLB initialization.  Do something.
 *
 * Created by:	Rick Lund
 *
 * Date Created:	6/18/01
 *
 *-----------------------------------------------------------------
 */
	.ent	tlb_error
tlb_error:

1:	b		1b

	.end	tlb_error

/*------------------------------------------------------------------*/
/*
 * Tables for memory sizes
 */

tlb_mem_1024:
	.word	TLB_PGMASK_16M
	.word	TLB_PGMASK_16M
	.word	TLB_PGMASK_16M
	.word	TLB_PGMASK_16M
tlb_mem_896:
	.word	TLB_PGMASK_16M
	.word	TLB_PGMASK_16M
	.word	TLB_PGMASK_16M
	.word	TLB_PGMASK_16M
tlb_mem_768:
	.word	TLB_PGMASK_16M
	.word	TLB_PGMASK_16M
	.word	TLB_PGMASK_16M
	.word	TLB_PGMASK_16M
tlb_mem_640:
	.word	TLB_PGMASK_16M
	.word	TLB_PGMASK_16M
	.word	TLB_PGMASK_16M
	.word	TLB_PGMASK_16M
tlb_mem_512:
	.word	TLB_PGMASK_16M
	.word	TLB_PGMASK_16M
	.word	TLB_PGMASK_16M
	.word	TLB_PGMASK_16M
tlb_mem_384:
	.word	TLB_PGMASK_16M
	.word	TLB_PGMASK_16M
	.word	TLB_PGMASK_16M
	.word	TLB_PGMASK_16M
tlb_mem_256:
	.word	TLB_PGMASK_16M
	.word	TLB_PGMASK_16M
	.word	TLB_PGMASK_16M
	.word	TLB_PGMASK_16M
tlb_mem_128:
	.word	TLB_PGMASK_16M
tlb_mem_96:
	.word	TLB_PGMASK_16M
tlb_mem_64:
	.word	TLB_PGMASK_16M
tlb_mem_32:
	.word	TLB_PGMASK_16M
	.word	-1

tlb_mem_16:
	.word	TLB_PGMASK_4M
tlb_mem_8:
	.word	TLB_PGMASK_4M
	.word	-1

tlb_mem_4:
	.word	TLB_PGMASK_1M
	.word	TLB_PGMASK_1M
	.word	-1

tlb_mem_1:
	.word	TLB_PGMASK_256K
	.word	TLB_PGMASK_256K
	.word	-1

