/* vi:ts=4
 *-----------------------------------------------------------------
 *
 * Name:	dvt-api.c
 *
 * RCS:  $Id: dvt-api.c,v 1.8 2002/09/27 22:30:56 rick Exp $
 *
 * Copyright (C) 2002, Agile Storage, Inc.
 *
 * Description:	api functions for the system dvt test.
 *
 * Created by:	eric choi
 *
 * Date Created:	3/4/02
 *
 *-----------------------------------------------------------------
 */
#ifndef PROM 
/*#if 0 */
#define DVT_USE_SSC_IN_TEST
/* #endif */

#include <nfx-incl.h>
#include "dvt-api.h"
#include <exports/eee-api.h>
#include <exports/eee-desc.h>
#include <exports/eee-drv.h>
#include <exports/eee-api.h>
#include "../sm-eee/eee.h"
#if defined (NFP_FP)  || defined (NFP_TXRX)
#include "../sm-dp83820/dp83820.h"
#endif
#ifdef FCNIM
#include "../sm-dp83820/dp83820.h"
#include "../sm-gt64240-mii/gt64240mii-regs.h"
#endif

/* Extern Variables */

#if defined (NFP_TXRX) || defined (FCNIM)
extern dp83820_cfg_t *dp83820;
#endif

uint32			sysDVTrunning; /* used to add eee Hdr for front plane pkts fr txrx */
uint32			txrxdebug; /* This flag is used for txrx pkt dump.*/
uint32			dvtFirstPkt; /* only add eee Hdr for first x pkts. not needed for spinning pkts*/
uint32			dvt_test_status = DVT_HALT;
uint32			tests_running = 0;
dvt_test_stats	dvtstats;
dvt_local_stats_t	myStats;
uint32			expected_seq[SYS_DVT_CPU_MAX][SYS_DVT_TESTTYPE_MAX];
uint32			glo_num_bytes;
uint32			glo_buf_size;
uint32			glob_seq[SYS_DVT_CPU_MAX][SYS_DVT_TESTTYPE_MAX];
uint32			dvt_front_seq[3][DVT_NFP_PORT_TXRX_MAX] = {{0,0},
												  {0,0},
												  {0,0}};
uint32			exp_front_seq[3][DVT_NFP_PORT_TXRX_MAX] = {{0,0},
												  {0,0},
												  {0,0}};
uint32			dvt_num_pkts = DVT_TEST_NUM_PKT_SPEC;

uint32			txrx_mgmt_seq;
uint32			exp_txrx_mgmt_seq;
uint32	incoming_dvt_appid;
volatile uint32	dvt_all_packets_sent = 0;
uint32			data[2];

extern eee_config_t		eee;
extern uint32 nfxMySlotId, nfxMyCPUId;
eee_descPtr_t customsystemDVTPacketGenerator( uint32	buf_size, uint32	num_bytes,
		uint32	mode, uint32	port);
void systemDVTSendCommand( uint32 slot, uint32 cpu, uint32 type_of_test, uint32 cmd);
eee_descPtr_t systemDVTPacketGenerator( uint32	buf_size, uint32 num_bytes, uint32 mode, uint32 testPort);
void sys_dvt_do_frontplane() ;

uint32 txrxFrontnumPkts = DVT_TEST_NUM_PKT_SPEC;
uint32 DVTdebug;
uchar8 *dvt_dtype[] = {"DATA", "CTRL", "MGMT", "", "FRONT"};



/*-----------------------------------------------------------------
 * Name:	()
 *
 * Description:
 *
 * Created by:	eric choi
 *
 * Date Created:	3/4/02
 *
 *-----------------------------------------------------------------
 */
void 
systemDVTSendCommand( uint32 slot, uint32 cpu, uint32 type_of_test, uint32 cmd)
{
	sys_dvt_control	msg;
	uint32	destport, srcport, send_msg_mode = EEE_MGMT;

	msg.command   = cmd;
	msg.test_type = type_of_test;
	msg.num_bytes = glo_num_bytes;
	msg.buf_size  = glo_buf_size;

#if 0
	if (type_of_test == EEE_MGMT)
		send_msg_mode = EEE_USER_DATA;
#endif

	destport =	PACK_UNICAST_PORT_ADDRESS(slot, cpu,
				eee_allocateApplicationId("dvt"),
				send_msg_mode, NFX_COS_HIGH, PORT_BEST_FIT);

	srcport  =	PACK_UNICAST_PORT_ADDRESS(nfxMySlotId, nfxMyCPUId,
				eee_allocateApplicationId("dvt"),
				send_msg_mode, NFX_COS_HIGH, PORT_BEST_FIT);

	eee_sendMessage(&msg, sizeof(sys_dvt_control), destport, srcport);
}

eee_descPtr_t 
systemDVTPacketGenerator(
		uint32	buf_size,
		uint32	num_bytes,
		uint32	mode,
		uint32  testPort
)
{
	eee_descPtr_t   edesc;
	uint32                  remain;
	eee_descBuf_t   * bd;
	uchar8                  * buf[EEE_MAX_BUFS];
	uint32                  bd_size[EEE_MAX_BUFS];
	uint32                  num_bufs;
	uint32                  buf_bytes;
	uint32                  i, port, use_app_id;
	sys_dvt_control  *test_hdr;
	uchar8 * tmp;
	uint32 storeMode;
	uchar8 *bufPtr;
	uint32 mark;
	/*
	 * Allocate descriptor from my pool
	 */
	edesc = eee_allocateDesc(eee.my_pool_id, NFX_COS_HIGH);
	if (!edesc) {
		return NULL;
	}

	/*
	 * Allocate buffers from my pool
	 */
	num_bufs = 0;
	buf_bytes = eee.buf_size[buf_size] - eee.buf_offset;  /* buf_size = 1 from ssc */
	remain = num_bytes; /* whatever is passed from ssc:64 */
	do {
		buf[num_bufs] = eee_allocateBuffer(eee.my_pool_id, buf_size, NFX_COS_HIGH);
		if (!buf[num_bufs]) {
			for (i=0; i<num_bufs; i++)
				eee_deallocateBuffer(buf[i]);
			eee_deallocateDesc(edesc);
			return NULL;
		}
		if (remain > buf_bytes) {
			bd_size[num_bufs] = buf_bytes;
			remain -= buf_bytes;
		}
		else {
			bd_size[num_bufs] = remain;
			remain = 0;
		}
		num_bufs++;
	} while (remain);

	/*
	 * Create packet
	 */
	bd = &(edesc->bd[0]);
	edesc->hdr.control = (num_bufs << EDESC_BUFCNT_SHIFT) | num_bytes;
	if (num_bufs == EEE_MAX_BUFS) {
		edesc->hdr.control |= EDESC_COMMAND_USE_FIRST_BUFFER;
	}
	else
		bd++;

	for (i=0; i<num_bufs; i++) {
		bd->control = EDESC_BD_CMD_IN_USE | buf_size;
		bd->len = bd_size[i];
		bd->buf = buf[i];
		bd++;
	}

	/*set port to BEST_FIT just for starters*/
	port = PORT_BEST_FIT;
	use_app_id = eee_allocateApplicationId("dvt"); 
	storeMode = mode;
	
	if (isSSC()) {
		if (mode == EEE_USER_DATA)
			port = SSC_PORT_BP_DATA;
		else if (mode == EEE_MGMT)
			port = SSC_PORT_BP_MGMT;
	}
	else if (isFC()) {
		if (mode == EEE_USER_DATA)
			port = FCNIM_PORT_BP_DATA;
		else if (mode == EEE_MGMT) {
			port = FCNIM_PORT_BP_MGMT;
		}
		else if (mode == EEE_CONTROL) {
			port = FCNIM_PORT_BP_CONTROL;
		}
		else if (mode == SYS_DVT_FRONT) {
			mode = EEE_USER_DATA;
			port = FCNIM_PORT_FC;
		}
	}
	else if (isFP()) {
		if (mode == EEE_CONTROL)
			port = NFP_PORT_FP_ETH1;
		else if (mode == EEE_USER_DATA)
			port = NFP_PORT_FP_ETH0;
	}
	else if (isTXRX()) {
		if (mode == SYS_DVT_FRONT) {
			mode = EEE_USER_DATA;
			if(testPort == 0)	
				port = NFP_PORT_TXRX_ETH0;
			else if(testPort == 1)
				port = NFP_PORT_TXRX_ETH1;
			else if(testPort == 2)
				port = NFP_PORT_TXRX_ETH2;
		}	
		else if (mode == EEE_MGMT)
			port = NFP_PORT_MGMT;
	}
#ifdef DVT_USE_SSC_IN_TEST
	edesc->hdr.dest_port = PACK_UNICAST_PORT_ADDRESS(NFX_CARDTYPE_SSC,
									NFX_SSC_UNIX,
									use_app_id,
									mode,
									NFX_COS_HIGH,
									port);

#else
	edesc->hdr.dest_port = PACK_UNICAST_PORT_ADDRESS(nfxMySlotId,
									nfxMyCPUId,
									use_app_id,
									mode,
									NFX_COS_HIGH,
									port);
#endif

	edesc->hdr.src_port = PACK_UNICAST_PORT_ADDRESS(nfxMySlotId,
									nfxMyCPUId,
									use_app_id,
									mode,
									NFX_COS_HIGH,
									port);

	if(isTXRX())
			edesc->hdr.dest_port = edesc->hdr.src_port;
	if(isFC()) 
			edesc->hdr.dest_port = edesc->hdr.src_port;
	if(isFP()) 
			edesc->hdr.dest_port = edesc->hdr.src_port;

	edesc->hdr.attr = 0;
	test_hdr = (sys_dvt_control *)(buf[0]);
	test_hdr->command = SYS_DVT_DATA;
	if(isTXRX() && (storeMode == EEE_MGMT))
		test_hdr->seq = txrx_mgmt_seq;
	if(storeMode == SYS_DVT_FRONT)
		test_hdr->seq = dvt_front_seq[nfxMySlotId][testPort];
	else
		test_hdr->seq = glob_seq[nfxMyCPUId][storeMode];
	test_hdr->port = testPort;
	test_hdr->buf_size = buf_size;
    test_hdr->num_bytes = num_bytes;
    test_hdr->orig_slot = nfxMySlotId;
    test_hdr->orig_cpu = nfxMyCPUId;
	test_hdr->test_type = storeMode;

	if(isTXRX()  && (storeMode == EEE_MGMT))
			return(edesc);

	if(storeMode == SYS_DVT_FRONT)
    	dvt_front_seq[nfxMySlotId][testPort]  = ((dvt_front_seq[nfxMySlotId][testPort]) + 1) % txrxFrontnumPkts;
	else 
    	glob_seq[nfxMyCPUId][storeMode] = ((glob_seq[nfxMyCPUId][storeMode]) + 1) % DVT_TEST_NUM_PKT_SPEC;

	return (edesc);
}

void
systemDVTSend(uint32 slot, uint32 cpu, uchar8 * bfr, uint32 size)
{
	uint32                  dp;
	uint32                  sp;
	uint32 rc;
	uchar8 * tmp;
	uint32 j;
	
	dp = PACK_UNICAST_PORT_ADDRESS(slot,
						cpu,
						incoming_dvt_appid,
						EEE_MGMT,
						NFX_COS_HIGH,
						PORT_BEST_FIT);

	sp = PACK_UNICAST_PORT_ADDRESS(nfxMySlotId,
						nfxMyCPUId,
						eee_allocateApplicationId("dvt"),
						EEE_MGMT,
						NFX_COS_HIGH,
						PORT_BEST_FIT);

	/*
      tmp = bfr;
      for (j=0; j<64; j++)
         printf("%0x ", *tmp++);
	*/	 
	rc = eee_sendMessage(bfr, size, dp, sp);
	if (rc == EEE_FORWARDED_PACKET) {
			if(DVTdebug)
					printf("systemDVTSend dp %08x sp %08x, msg %d\n",dp, sp,bfr[0]);
	}
	else {
			if(DVTdebug)
				printf("PKT NOT fwd!!\n");					
	}
}

void
dvtSendPkt(sys_dvt_control *msg, uint32 port, uint32 srcslot, uint32 srccpu)
{
		uint32 i;
		eee_descPtr_t   edesc_created;
		volatile uint32 seq, tempport;
		uint32 numPkts = DVT_TEST_NUM_PKT_SPEC;
		uint32 num_bytes = msg->num_bytes;

		if((isFP()) && (msg->test_type == EEE_MGMT))
				return; 
		if((isTXRX()) && ((msg->test_type == EEE_CONTROL) || 
						(msg->test_type == EEE_USER_DATA)))
				return;

		if(port == 0xFF)
				port = 0; /* External port is invalid */

		if (msg->test_type == SYS_DVT_FRONT) {
				numPkts = txrxFrontnumPkts;
				num_bytes = 66;
		}		

		if( isTXRX() && (msg->test_type == EEE_MGMT))
				numPkts = 1;

		for(i = 0; i < numPkts; i++) 
		{
				dvtFirstPkt = 1;
				edesc_created = systemDVTPacketGenerator(
								msg->buf_size, num_bytes, msg->test_type, port);

				tempport = eee_forwardPacket(edesc_created);
				if (tempport != EEE_FORWARDED_PACKET) {
					myStats.packet_send_errors[srccpu][msg->test_type]++;
				}
				else {
					myStats.packets_sent[srccpu][msg->test_type]++;
				}
		}
		dvtFirstPkt = 0;
}		



void
systemDVTRcv(eee_descPtr_t  edesc)
{
	sys_dvt_control	*msg;
	uint32  srcslot,slot;
	volatile  i, diff;
	uint32  srccpu, cpu, num_cpus;
	eee_descBuf_t   * bd;
	eee_descPtr_t   edesc_created;
	eee_desc_t   edesc_next;
	volatile uint32 seq, tempport;
	uint32 port;
	uint32 tmp;
	uint32 j;
	uchar8 *tmp1;
	static uint32 cnt=0;
	uint32 pktLen = 66;
	static uint32 n[SYS_DVT_TESTTYPE_MAX] = {0,0,0,0,0};
	volatile uint32 crc;
	uint32 loopback_set = 0;
	static uint32 sbMacSet = 0;
/*	
take_an_edump(edesc);
*/
	incoming_dvt_appid = EEE_GET_APP_ID(edesc->hdr.src_port);
	bd 	= &(edesc->bd[0]);

	if (!(edesc->hdr.control & EDESC_COMMAND_USE_FIRST_BUFFER))
		bd 	= &(edesc->bd[1]);

	msg = (sys_dvt_control *) bd->buf;
	srcslot  = EEE_GET_SLOT_NUM(edesc->hdr.src_port);
	srccpu 	 = EEE_GET_DEST_CPU(edesc->hdr.src_port);

	switch(msg->command) {

		case SYS_DVT_START:

#ifdef SSC_MGMT
			if(!loopback_set) {
			/* Set ctrl and data loopback*/
			diag5632_enableLoopbacks(0);
			/* Set mgmt loopback*/
			bcm_init_5600(0);
			diag5600_sys_dvt_mgmt_port_mac_loop_set();
			loopback_set = 1;
			}
			eee_freePacket(edesc);
			return;
#endif
			if (isFP()) {
				if(msg->test_type == SYS_DVT_FRONT) {
						eee_freePacket(edesc);
						return;
				}
				if (tests_running & (1<<msg->test_type)) {
					eee_freePacket(edesc);
					return;
				}	
				else
					tests_running |= (1<<msg->test_type);
			}
			if (isTXRX()) {
				if((msg->test_type == EEE_CONTROL) || (msg->test_type == EEE_USER_DATA))
				{
					eee_freePacket(edesc);
					return;	
				}
				if (tests_running & (1<<msg->test_type)) {
					eee_freePacket(edesc);
					return;
				}
				else
					tests_running |= (1<<msg->test_type);
			}
#ifdef FCNIM			
			if (isFC()) {
				if(msg->test_type == SYS_DVT_FRONT) {
					eee_freePacket(edesc);
					return;
				}
				if(msg->test_type == EEE_CONTROL) {
					if((cnt) && (nfxMyCPUId == 0)) 
							bcm_init_5600(0);
					cnt++;
				}
			}
#endif			
			if(msg->test_type == SYS_DVT_FRONT)
				sysDVTrunning = 0;
			if(isTXRX() && (msg->test_type == EEE_MGMT)) {
					txrx_mgmt_seq = 0;
					exp_txrx_mgmt_seq = 0;
			}
			if(dvt_test_status == DVT_HALT)
				dvt_test_status = DVT_RUN;
			txrxdebug = 1;
			glo_buf_size = 1;
			glo_num_bytes = 64;
			glob_seq[srccpu][msg->test_type] = 0;
			expected_seq[srccpu][msg->test_type] = 0;

			/*displays running test on console port for each card*/
			for(i=0;i<SYS_DVT_TESTTYPE_MAX;i++) {
				if(!n[i])
				 	n[i] = 0;
			}

			if (msg->test_type == SYS_DVT_FRONT) {
					if(msg->port) {
							dvt_front_seq[srcslot][(msg->port)-1] = 0;
							exp_front_seq[srcslot][(msg->port)-1] = 0;
					}
					else {		
							for(j=0; j<DVT_NFP_PORT_TXRX_MAX; j++) {
									dvt_front_seq[srcslot][j] = 0;
									exp_front_seq[srcslot][j] = 0;
							}
					}		
			}		
#if defined (NFP_FP) || defined (NFP_TXRX)
			if ((msg->test_type == SYS_DVT_FRONT) && (isTXRX()))
			{
				sys_dvt_do_frontplane();
			}
			if(isFP() && (!sbMacSet)) {
				dvt_set_mac_promisc_mode();
				sbMacSet = 1;
			}
#endif

			/*loopback and update stats*/
			if(msg->test_type == SYS_DVT_FRONT) {
					/* run loopback only on port specified */
					if(msg->port) {
							port = (msg->port) - 1; /* from SSC we map port0->1 */	
							dvtSendPkt(msg, port, srcslot, srccpu);
					}
					else {	
							/* run loopback only on all ports */
							for(j=0; j<DVT_NFP_PORT_TXRX_MAX; j++) {
									port = j ; 
									dvtSendPkt(msg, port, srcslot, srccpu);
							}	
					}	
			}
			else {
					port = 0xFF ; 
					dvtSendPkt(msg, port, srcslot, srccpu);
			}
			eee_freePacket(edesc);
		break;

		case SYS_DVT_STOP:
			dvt_test_status = DVT_HALT;
			tests_running = 0;

			/* display counters reset to 0 */
			for(i=0;i<SYS_DVT_TESTTYPE_MAX;i++) {
				 	n[i] = 0;
			}

			if(isFP())
				sbMacSet = 0; /* keeps track of sibyte mac set to promiscous mode */

#if defined (NFP_TXRX) || defined (NFP_FP)
			if (isTXRX() || isFP()) {
				dvt_Reset_mac_promisc_mode();
				sysDVTrunning = 0;
			}	
#endif				
#ifdef SSC_MGMT
			/* Unset ctrl and data loopback*/
			diag5632_disableLoopbacks(0);
			/* Unset mgmt loopback*/
			diag5600_sys_dvt_mgmt_port_mac_loop_unset();
			loopback_set = 0;
#endif
			eee_freePacket(edesc);
		break;

		case SYS_DVT_DATA:
/*enable this if things runaway*/
#if 0
			if (tempcount < 10)
				tempcount++;
			else {
				eee_freePacket(edesc);
				return;
			}
#endif

			if (dvt_test_status != DVT_RUN) {
				eee_freePacket(edesc);
				return;
			}

#ifdef XXX
			while (!(dvt_all_packets_sent))
			{
				i = (volatile)dvt_all_packets_sent;
				sleep(20);
			}
#else
#endif

			/*
			  printf("RcvData: dp: %08x sp:%08x testtype %d\n", 
					edesc->hdr.dest_port, edesc->hdr.src_port,msg->test_type);
			 */			

			/* Check to see if packet rcvd is really for us */
			if((msg->orig_slot != nfxMySlotId) && (msg->orig_cpu != nfxMyCPUId))
			{
				printf("wrong dest: pkt for s:%d c:%d, rcvd at s:%d c:%d\n",msg->orig_slot, msg->orig_cpu, nfxMySlotId, nfxMyCPUId);
				eee_freePacket(edesc);
				return;
			}
			else 
			{
				if (msg->test_type == SYS_DVT_FRONT) 
				{
						port = msg->port;
						if (msg->seq != exp_front_seq[srcslot][port]) {
							myStats.seq_errors[srccpu][msg->test_type]++;
							exp_front_seq[srcslot][port] = msg->seq;
						}
						exp_front_seq[srcslot][port] = ((exp_front_seq[srcslot][port]) +1) % txrxFrontnumPkts;
						/* Move the bd to point to start of pkt- which has the MAC/EEE header */
						bd->buf-=0x30;
						bd->len+=0x30;
						pktLen += 0x30; 
						edesc->hdr.control |= (pktLen & EDESC_TOTAL_LEN);
				}
				else 
				{
					if( isTXRX() && (msg->test_type == EEE_MGMT)) {
							if (msg->seq != exp_txrx_mgmt_seq) {
									myStats.seq_errors[srccpu][msg->test_type]++;
									exp_txrx_mgmt_seq = msg->seq;
							}
							exp_txrx_mgmt_seq ++;
							msg->seq += 1;
					}
					else {
							if (msg->seq != expected_seq[srccpu][msg->test_type]) {
									myStats.seq_errors[srccpu][msg->test_type]++;
									expected_seq[srccpu][msg->test_type] = msg->seq;
							}
							expected_seq[srccpu][msg->test_type] = ((expected_seq[srccpu][msg->test_type]) +1) % DVT_TEST_NUM_PKT_SPEC;
					}
				}

			tempport = edesc->hdr.dest_port;
			edesc->hdr.dest_port = edesc->hdr.src_port;
			edesc->hdr.src_port = tempport;

			/*this is so we don't Rx faster than we can Tx...*/	
			if (msg->test_type != SYS_DVT_FRONT) {
				eee_poll(1);
				eee_poll(1);
				eee_poll(1);
				eee_poll(1);
			}
		
			/* if(isFC() || isFP()) */
			tmp = eee_forwardPacket(edesc);
			if (tmp != EEE_FORWARDED_PACKET)
				myStats.packet_send_errors[srccpu][msg->test_type]++;
			else
				myStats.packets_sent[srccpu][msg->test_type]++;
		
			myStats.packets_received[srccpu][msg->test_type]++;
			if(myStats.packets_sent[srccpu][msg->test_type] > n[msg->test_type]*3000) {
				printf("PKTS SENT [%s]: %08d\n",dvt_dtype[msg->test_type],myStats.packets_sent[srccpu][msg->test_type]);
				printf("PKTS RCVD [%s]: %08d\n",dvt_dtype[msg->test_type], myStats.packets_received[srccpu][msg->test_type]); 
				n[msg->test_type]++;
			}
		}
		break;

		case SYS_DVT_STATS_REPORT:
			dvt_stats_display(&dvtstats);
		break;

		case SYS_DVT_STATS:
#ifdef SSC_MGMT
			/* Do not return stats for SSC_MGMT */
			eee_freePacket(edesc);
			return;
#endif

#if defined(NFP_TXRX) || defined (NFP_FP)			
			if(isTXRX()) 
			{
				for(j=0;j<DP83820_RX_Q_CNT;j++) {
							myStats.crc_errors[nfxMyCPUId][EEE_MGMT] += dp83820->rxc[j].crcErr;
				}			
				myStats.crc_errors[nfxMyCPUId][SYS_DVT_FRONT] = 0; /* not valid */ 
				myStats.crc_errors[nfxMyCPUId][EEE_CONTROL] = 0; /* not valid */ 
				myStats.crc_errors[nfxMyCPUId][EEE_USER_DATA] = 0; /* not valid */ 
			}		
#elif defined (FCNIM)			
			myStats.crc_errors[nfxMyCPUId][EEE_MGMT] = gt_mii_readPortReg(1, GT_MII_CRC_ERRS); 
			myStats.crc_errors[nfxMyCPUId][EEE_CONTROL] = gt_mii_readPortReg(2, GT_MII_CRC_ERRS); 
			for(j=0;j<DP83820_RX_Q_CNT;j++) {
					myStats.crc_errors[nfxMyCPUId][EEE_USER_DATA] += dp83820->rxc[j].crcErr;
			}		
			myStats.crc_errors[nfxMyCPUId][SYS_DVT_FRONT] = 0; /* not valid */ 
#endif			
			dvtstats.command = SYS_DVT_STATS_REPORT;
			dvtstats.slot = nfxMySlotId;
			dvtstats.cpu = nfxMyCPUId;
			for(i=0;i<SYS_DVT_TESTTYPE_MAX;i++) {
				dvtstats.packets_sent[i] = myStats.packets_sent[nfxMyCPUId][i];
				dvtstats.packets_received[i] = myStats.packets_received[nfxMyCPUId][i];
				dvtstats.packet_errors[i] = myStats.packet_errors[nfxMyCPUId][i];
				dvtstats.packet_send_errors[i] = myStats.packet_send_errors[nfxMyCPUId][i];
				dvtstats.seq_errors[i] = myStats.seq_errors[nfxMyCPUId][i];
				dvtstats.crc_errors[i] = myStats.crc_errors[nfxMyCPUId][i];
			}
			dvt_stats_display(&dvtstats);
			systemDVTSend(srcslot, srccpu, (uchar8 *)&dvtstats, sizeof(dvt_test_stats));
			eee_freePacket(edesc);
		break;

		case SYS_DVT_CLEAR_STATS:
			slot = myStats.slot;
			cpu = myStats.cpu;
			memset(&dvtstats,0,sizeof(dvt_test_stats));
			memset(&myStats,0,sizeof(dvt_local_stats_t));
			myStats.slot = dvtstats.slot = slot;
			myStats.cpu = dvtstats.cpu = cpu;
			eee_freePacket(edesc);
			break;				
		default:
			printf("dvt Rcv: unrecognized command %d",msg->command);
		break;
	}
	
}

/*-----------------------------------------------------------------
 * Name:	dvt_stats_display()
 *
 * Description:
 *
 * Created by:	eric choi
 *
 * Date Created:	3/11/02
 *
 *-----------------------------------------------------------------
 */
void
dvt_stats_display(dvt_test_stats  * d_stats)
{
	uint32 slot = d_stats->slot;
	uint32 cpu  = d_stats->cpu;
	uint32	i;

	printf("\n========= System DVT Statistics Slot %d CPU %d ==========\n",slot, cpu);
	printf("\t\t\tDATA\tCONTROL  \tMGMT\tFRONT\n");
	printf("PKT SEND ERR:	\t%d   \t%d \t\t%d \t%d\n",
			d_stats->packet_send_errors[0],
			d_stats->packet_send_errors[1],
			d_stats->packet_send_errors[2],
			d_stats->packet_send_errors[4]);
	printf("SEQ ERR:		%d   \t%d \t\t%d \t%d\n",
			d_stats->seq_errors[0],
			d_stats->seq_errors[1],
			d_stats->seq_errors[2],
			d_stats->seq_errors[4]);
	printf("PKT SENT:		%d   \t%d \t\t%d \t%d\n",
			d_stats->packets_sent[0],
			d_stats->packets_sent[1],
			d_stats->packets_sent[2],
			d_stats->packets_sent[4]);
	printf("PKT RCVD:	 	%d   \t%d \t\t%d \t%d\n",
			d_stats->packets_received[0],
			d_stats->packets_received[1],
			d_stats->packets_received[2],
			d_stats->packets_received[4]);
	printf("CRC ERR:		%d   \t%d \t\t%d \t%d\n",
			d_stats->crc_errors[0],
			d_stats->crc_errors[1],
			d_stats->crc_errors[2],
			d_stats->crc_errors[4]);
}

/*-----------------------------------------------------------------
 * Name:	dvt_mgmt_start()
 *
 * Description:
 *
 * Created by:	eric choi
 *
 * Date Created:	7/16/01
 *
 *-----------------------------------------------------------------
 */
void
dvt_msg_test( uint32 mode, uint32 msg)
{
	uint32 slot, cpu;

	glo_buf_size = 1;
	glo_num_bytes = 64;
	
	/*
		systemDVTSendCommand( 0, 1, mode,  msg);
		return;
	*/
#if 0
	for (slot=0; slot<NFX_NUM_SLOTS_PER_CHASSIS; slot++) {
		for (cpu=0; cpu<NFX_MAX_CPUS_PER_SLOT; cpu++) {
#endif
	slot = 0;
	cpu = 0;	
	systemDVTSendCommand( slot, cpu, mode,  msg);
#if 0	
	for (slot=0; slot<3; slot++) {
		for (cpu=0; cpu<5; cpu++) {
			/*test not applicable for these CPUs*/
			if ((slot == nfxMySlotId) && (cpu == nfxMyCPUId))
				continue;
			else if (slot == NFX_CARDTYPE_NFP)
				if (mode == EEE_MGMT) {
					if ((cpu == NFX_NFP_TXRX2) || (cpu == NFX_NFP_FP1))
						continue;
				}
				else if (mode == EEE_USER_DATA) {
					if ((cpu == NFX_NFP_TXRX1) || (cpu == NFX_NFP_TXRX2))
						continue;
				}

			systemDVTSendCommand( slot, cpu, mode,  msg);
		}
	}
#endif		
}

void do_sysdvt(uint32 argc, uchar8* argv[])
{
	dvt_test_status = DVT_RUN;

	if (argc == 1) {
		dvt_msg_test( EEE_MGMT, SYS_DVT_START);
		dvt_msg_test( EEE_USER_DATA, SYS_DVT_START);
		dvt_msg_test( EEE_CONTROL, SYS_DVT_START);
	}

	if (argc == 2) {
		if (!strcmp(argv[1],"mgmt"))
			dvt_msg_test( EEE_MGMT, SYS_DVT_START);
		else if (!strcmp(argv[1],"data"))
			dvt_msg_test( EEE_USER_DATA, SYS_DVT_START);
		else if (!strcmp(argv[1],"ctrl"))
			dvt_msg_test( EEE_CONTROL, SYS_DVT_START);
		else if (!strcmp(argv[1],"halt"))
			dvt_msg_test( EEE_MGMT, SYS_DVT_STOP);
		else if (!strcmp(argv[1],"stats"))
			dvt_msg_test( EEE_MGMT, SYS_DVT_STATS_REPORT);
		else if (!strcmp(argv[1],"front"))
			dvt_msg_test( SYS_DVT_FRONT, SYS_DVT_START);
	}

}

void do_dvtstats(uint32 argc, uchar8 *argv[])
{
		dvt_stats_display(&dvtstats);
}

void
sysDVTInit()
{
	eee_unregisterReceiveFunc(eee_allocateApplicationId("dvt"));
	eee_registerReceiveFunc(eee_allocateApplicationId("dvt"), systemDVTRcv, systemDVTRcv);

	dvtstats.slot = nfxMySlotId;
	dvtstats.cpu  = nfxMyCPUId;
	memset(&expected_seq,0,sizeof(expected_seq));
	memset(&glob_seq,0,sizeof(glob_seq));
}

void
do_dvt(uint32 argc, uchar8* argv[])
{

	volatile eee_descPtr_t  eptr;
	int32 mode = atoi(argv[1]);
	int32 port = atoi(argv[2]);
	volatile uint32 i , tempport;

	dvt_test_status = DVT_RUN;
	glob_seq[nfxMyCPUId][mode] = 0;
	expected_seq[port][mode] = 0;

	for (i = 0; i < 1 ; i++) {
		eptr = customsystemDVTPacketGenerator(1,64,mode, port);
		tempport = eee_forwardPacket(eptr);

		if (tempport != EEE_FORWARDED_PACKET) {
			printf("not fwd'd %d, %d\n", mode,port);
		}
		else
			printf("itz fwd'd %d, %d\n", mode,port);
	}

	/*flag rcv func for processing*/
	dvt_all_packets_sent = 1;  

	return;
}

eee_descPtr_t 
customsystemDVTPacketGenerator(
		uint32	buf_size,
		uint32	num_bytes,
		uint32	mode,
		uint32	port
)
{
	eee_descPtr_t   edesc;
	uint32                  remain;
	eee_descBuf_t   * bd;
	uchar8                  * buf[EEE_MAX_BUFS];
	uint32                  bd_size[EEE_MAX_BUFS];
	uint32                  num_bufs;
	uint32                  buf_bytes;
	uint32                  i, use_app_id;
	sys_dvt_control  *test_hdr;

	/*
	 * Allocate descriptor from my pool
	 */
	edesc = eee_allocateDesc(eee.my_pool_id, NFX_COS_HIGH);
	if (!edesc) {
		return NULL;
	}

	/*
	 * Allocate buffers from my pool
	 */
	num_bufs = 0;
	buf_bytes = eee.buf_size[buf_size] - eee.buf_offset;
	remain = num_bytes;
	do {
		buf[num_bufs] = eee_allocateBuffer(eee.my_pool_id, buf_size, NFX_COS_HIGH);
		if (!buf[num_bufs]) {
			for (i=0; i<num_bufs; i++)
				eee_deallocateBuffer(buf[i]);
			eee_deallocateDesc(edesc);
			return NULL;
		}
		if (remain > buf_bytes) {
			bd_size[num_bufs] = buf_bytes;
			remain -= buf_bytes;
		}
		else {
			bd_size[num_bufs] = remain;
			remain = 0;
		}
memset(buf[num_bufs], 0xab, buf_bytes);
		num_bufs++;
	} while (remain);

	/*
	 * Create packet
	 */
	bd = &(edesc->bd[0]);
	edesc->hdr.control = (num_bufs << EDESC_BUFCNT_SHIFT) | num_bytes;
	if (num_bufs == EEE_MAX_BUFS) {
		edesc->hdr.control |= EDESC_COMMAND_USE_FIRST_BUFFER;
	}
	else
		bd++;

	for (i=0; i<num_bufs; i++) {
		bd->control = EDESC_BD_CMD_IN_USE | buf_size;
		bd->len = bd_size[i];
		bd->buf = buf[i];
		bd++;
	}

	use_app_id = eee_allocateApplicationId("dvt"); 

	edesc->hdr.dest_port = PACK_UNICAST_PORT_ADDRESS(nfxMySlotId,
									nfxMyCPUId,
									use_app_id,
									mode,
									NFX_COS_HIGH,
									port);

	edesc->hdr.src_port = PACK_UNICAST_PORT_ADDRESS(nfxMySlotId,
									nfxMyCPUId,
									use_app_id,
									mode,
									NFX_COS_HIGH,
									port);

	edesc->hdr.attr = 0;
	test_hdr = (sys_dvt_control *)(buf[0]);
	test_hdr->command = SYS_DVT_DATA;
	test_hdr->seq = glob_seq[nfxMyCPUId][mode];
	test_hdr->buf_size = buf_size;
    test_hdr->num_bytes = num_bytes;
    test_hdr->orig_slot = nfxMySlotId;
    test_hdr->orig_cpu = nfxMyCPUId;
	test_hdr->test_type = mode;

    glob_seq[nfxMyCPUId][mode] = (glob_seq[nfxMyCPUId][mode] + 1) % DVT_TEST_NUM_PKT_SPEC;

	return (edesc);
}
#endif
