/*
 *-----------------------------------------------------------------
 *
 * Name:	membench.c
 *
 * RCS:  $Id: membench.c,v 1.4 2002/10/22 01:30:42 jong Exp $
 *
 * Copyright (C) 2001, Agile Storage, Inc.
 *
 * Description:	Memory benchmarking tests
 *
 * Created by:	Bob Fozard
 *
 * Date Created:	8/14/2002
 *
 *-----------------------------------------------------------------
 */

#include <nfx-incl.h>

#ifdef PROM

#include "exports/malloc-api.h"
#include "exports/timer-api.h"
#include "exports/eee-desc.h"
#include "sys/mips.h"


#ifdef SIBYTE
address_t
uncached_to_cached(
	address_t	uncached
)
{
	address_t	addr;

	if (isTXRX()) {
		addr = uncached & ~0xF0000000;
		addr |= 0x70000000;
	}
	else if (isFP()) {
		addr = uncached & ~0xF0000000;
		addr |= 0x60000000;
	}

	return addr;
}

address_t
cached_to_uncached(
	address_t	cached
)
{
	address_t	addr;

	if (isTXRX()) {
		addr = cached & ~0xF0000000;
		addr |= 0x40000000;
	}
	else if (isFP()) {
		addr = cached & ~0xF0000000;
		addr |= 0x20000000;
	}

	return addr;
}
#endif /* SIBYTE */

//
// TODO: - disable the watchdog?
//       - disable timer interrupts?
//       - run them from the prom?
//       - provide support for lcl/rmt shm?
//


/**
 * Memory region types that can
 * be tested.
 */
enum
{
    MB_MEM_TYPE_HEAP,
    MB_MEM_TYPE_LOCAL_SHM,
    MB_MEM_TYPE_REMOTE_SHM,
    MB_MEM_TYPE_REMOTE_SHM_CACHED,
};


/**
 * Options to the tests.
 */
enum
{
    MB_OPT_DO_UNROLLED  = 0x0001,
    MB_OPT_INIT_CACHE   = 0x0002,
    MB_OPT_CACHE_LINE   = 0x0004,
};


/**
 * Do this for the fp to provide an analyzer
 * trigger.
 */
#ifdef PROM_SIBYTE
#define TRIGGER_START    *(uint32 *)0xffffffffbf400088ll = 1;
#define TRIGGER_END      *(uint32 *)0xffffffffbf400088ll = 1;
#else
#define TRIGGER_START
#define TRIGGER_END
#endif


/*------------------------------------------------------------------*/
#define mb_cacheop(_op, _adr) {             \
	address_t   _r;                         \
	__asm__ __volatile__ (                  \
	"	.set noreorder   \n"                \
	"	and %0,%2,~0x1F  \n"                \
	"	and %0,%0,%0     \n"                \
	"	cache   %1,0(%0) \n"                \
	"	.set reorder     \n"                \
	: "=r" (_r) : "i" (_op), "r" (_adr));   \
}

#define mb_cacheopStruct(_op, _adr, _lines) {             \
	address_t   _r;                         \
	__asm__ __volatile__ (                  \
	"	.set noreorder   \n"                \
	"L1:	cache   %1,0x0(%2) \n"                \
	"	cache   %1,0x20(%2) \n"                \
	"	cache   %1,0x40(%2) \n"                \
	"	cache   %1,0x60(%2) \n"                \
	"	cache   %1,0x80(%2) \n"                \
	"	cache   %1,0xA0(%2) \n"                \
	"	cache   %1,0xC0(%2) \n"                \
	"	cache   %1,0xE0(%2) \n"                \
	"	cache   %1,0x100(%2) \n"                \
	"	cache   %1,0x120(%2) \n"                \
	"	cache   %1,0x140(%2) \n"                \
	"	cache   %1,0x160(%2) \n"                \
	"	cache   %1,0x180(%2) \n"                \
	"	cache   %1,0x1A0(%2) \n"                \
	"	cache   %1,0x1C0(%2) \n"                \
	"	cache   %1,0x1E0(%2) \n"                \
	"	cache   %1,0x200(%2) \n"                \
	"	cache   %1,0x220(%2) \n"                \
	"	cache   %1,0x240(%2) \n"                \
	"	cache   %1,0x260(%2) \n"                \
	"	cache   %1,0x280(%2) \n"                \
	"	cache   %1,0x2A0(%2) \n"                \
	"	cache   %1,0x2C0(%2) \n"                \
	"	cache   %1,0x2E0(%2) \n"                \
	"	cache   %1,0x300(%2) \n"                \
	"	cache   %1,0x320(%2) \n"                \
	"	cache   %1,0x340(%2) \n"                \
	"	cache   %1,0x360(%2) \n"                \
	"	cache   %1,0x380(%2) \n"                \
	"	cache   %1,0x3A0(%2) \n"                \
	"	cache   %1,0x3C0(%2) \n"                \
	"	cache   %1,0x3E0(%2) \n"                \
	"	cache   %1,0x400(%2) \n"                \
	"	cache   %1,0x420(%2) \n"                \
	"	cache   %1,0x440(%2) \n"                \
	"	cache   %1,0x460(%2) \n"                \
	"	cache   %1,0x480(%2) \n"                \
	"	cache   %1,0x4A0(%2) \n"                \
	"	cache   %1,0x4C0(%2) \n"                \
	"	cache   %1,0x4E0(%2) \n"                \
	"	cache   %1,0x500(%2) \n"                \
	"	cache   %1,0x520(%2) \n"                \
	"	cache   %1,0x540(%2) \n"                \
	"	cache   %1,0x560(%2) \n"                \
	"	cache   %1,0x580(%2) \n"                \
	"	cache   %1,0x5A0(%2) \n"                \
	"	cache   %1,0x5C0(%2) \n"                \
	"	cache   %1,0x5E0(%2) \n"                \
	"	cache   %1,0x600(%2) \n"                \
	"	cache   %1,0x620(%2) \n"                \
	"	cache   %1,0x640(%2) \n"                \
	"	cache   %1,0x660(%2) \n"                \
	"	cache   %1,0x680(%2) \n"                \
	"	cache   %1,0x6A0(%2) \n"                \
	"	cache   %1,0x6C0(%2) \n"                \
	"	cache   %1,0x6E0(%2) \n"                \
	"	cache   %1,0x700(%2) \n"                \
	"	cache   %1,0x720(%2) \n"                \
	"	cache   %1,0x740(%2) \n"                \
	"	cache   %1,0x760(%2) \n"                \
	"	cache   %1,0x780(%2) \n"                \
	"	cache   %1,0x7A0(%2) \n"                \
	"	cache   %1,0x7C0(%2) \n"                \
	"	cache   %1,0x7E0(%2) \n"                \
	"	addiu   %3,%3,-64\n"					\
	"	bne		%3,$0,L1\n"					\
	"	daddiu  %2,%2,0x800\n"					\
	"	.set reorder     \n"                \
	: "=r" (_r) : "i" (_op), "r" (_adr), "r" (_lines));   \
}

static void
mb_cacheInvalidate(
	address_t   _sa,
	uint32      _nbytes
)
{
	uint32  _clines, _i;
	address_t   _addr;

	if ((_sa) & CACHELINE_MASK)
		panic("unaligned invalidate command, 0x%x\n", (_sa));

	_clines = ((_nbytes) + (CACHELINE_SIZE - 1)) >> CACHELINE_SHIFT;

	for (_i = 0, _addr = (_sa); _i < _clines; _i++) {
		mb_cacheop(Hit_Invalidate_D, _addr);
		_addr += CACHELINE_SIZE;
	}
}

static void
mb_cacheWritebackInv(
	address_t   _sa,
	uint32      _nbytes
)
{
	uint32  _clines, _i;
	address_t   _addr;

	if ((_sa) & CACHELINE_MASK)
		panic("unaligned invalidate command, 0x%x\n", (_sa));

	_clines = ((_nbytes) + (CACHELINE_SIZE - 1)) >> CACHELINE_SHIFT;

	_addr = _sa;

	mb_cacheopStruct(Hit_Writeback_Inv_D, _addr, _clines);
}

static void
mb_cacheCreateDirty(
	address_t   _sa,
	uint32      _nbytes
)
{
//  uint32  _clines, _i;
//  address_t   _addr;
//
//  if ((_sa) & CACHELINE_MASK)
//      panic("unaligned invalidate command, 0x%x\n", (_sa));
//
//  _clines = ((_nbytes) + (CACHELINE_SIZE - 1)) >> CACHELINE_SHIFT;
//
//  for (_i = 0, _addr = (_sa); _i < _clines; _i++) {
//      mb_cacheop(Index_Store_Tag_D, _addr);
//      _addr += CACHELINE_SIZE;
//  }
}

/*------------------------------------------------------------------*/
/**
 * Malloc a buffer of the specified type,
 * truncating numBytes if appropriate.
 */
static void *
mb_malloc(
    int32   type,
    int32   *numBytes)
{
    void    *retPtr = NULL;

    switch(type)
    {
    case MB_MEM_TYPE_HEAP:
        retPtr = eee_ramAlloc(*numBytes);
        break;
    case MB_MEM_TYPE_LOCAL_SHM:
        retPtr = eee_allocateBuffer(EEE_SHARED_POOL, EEE_BUF_LARGE,
                                    NFX_COS_HIGH);
        if (*numBytes > EEE_BUFSIZE_LARGE)
        {
            *numBytes = EEE_BUFSIZE_LARGE;
            printf("truncating numBytes to %d (max shm buffer size)\n",
                EEE_BUFSIZE_LARGE);
        }
        break;
    case MB_MEM_TYPE_REMOTE_SHM:
    case MB_MEM_TYPE_REMOTE_SHM_CACHED:
		if (isFP())
   	     retPtr = (void *)0x20000000;
		else if (isTXRX())
	        retPtr = (void *)0x40000000;
		else
			retPtr = (void *)0;
        if (*numBytes > 0x100000)
        {
            *numBytes = 0x100000;
            printf("truncating numBytes to %d (max shm buffer size)\n",
												                0x100000);
        }
		if (type == MB_MEM_TYPE_REMOTE_SHM_CACHED)
			retPtr = (void *)uncached_to_cached((address_t)retPtr);
        break;
    default:
        printf("mb_malloc bad memtype %d\n", type);
    }

    return retPtr;
}


/**
 * Free a buffer of the specified type.
 */
static void
mb_free(
    int32  type,
    void   *ptr)
{
    switch(type)
    {
    case MB_MEM_TYPE_HEAP:
        eee_ramDealloc(ptr);
        break;
    case MB_MEM_TYPE_LOCAL_SHM:
    case MB_MEM_TYPE_REMOTE_SHM:
//        eee_deallocateBuffer(ptr);
        break;
	case MB_MEM_TYPE_REMOTE_SHM_CACHED:
//		eee_deallocateBuffer(cached_to_uncached(ptr));
		break;
    default:
        printf("mb_free bad memtype %d\n", type);
    }
}


/**
 * Present the results of a test.
 */
static void
mb_printResult(
    char    *callerStr,
    uint64  numBytes,
    uint64  numIter,
    uint64  usecs)
{
    printf("%s: %lld bytes transferred in %lld usecs\n",
           callerStr, numBytes * numIter, usecs);

    printf("    %lld MegaBytes/sec throughput ",
            numBytes * numIter / usecs);

    if (usecs < 1000000)
    {
        printf("(elapsed time < 1 sec, result unreliable)");
    }
    printf("\n");
}


/**
 * Do bzero test.
 */
int32
mb_bzero(
    int32   numBytes,
    int32   numIter,
    int32   memType,
    uint32  options)
{
    char        *datap;
    eeeRTC_t    startTime, delta;
    char        *testName = "mb_bzero";
    int32       iter;

    if (numBytes == 0 ||
        numIter == 0)
    {
        printf("%s: bad params (numBytes: %d, numIter: %d)\n",
            testName, numBytes, numIter);
        return NFX_ERR;
    }

    datap = mb_malloc(memType, &numBytes);
    if (!datap)
    {
        printf("%s: malloc failed\n", testName);
        return NFX_ERR;
    }

    if (options & MB_OPT_INIT_CACHE)
    {
        bzero(datap, numBytes);
    }

    eee_rtc(&startTime);
    TRIGGER_START
    for (iter = 0; iter < numIter; iter++)
    {
        bzero(datap, numBytes);
    }
    TRIGGER_END
	eee_rtcDuration(&startTime, &delta);

    mb_printResult(testName, numBytes, numIter,
				   eee_rtcToUsec(&delta));

    mb_free(memType, datap);

    return NFX_OK;
}


/**
 * Perform unrolled U64 write.
 */
static void
mb_write64unrolled(
    int32           numBytes,
    volatile uint64 *to)
{

#define WRITE_128_BYTES \
        to[0] = 1;      \
        to[1] = 1;      \
        to[2] = 1;      \
        to[3] = 1;      \
        to[4] = 1;      \
        to[5] = 1;      \
        to[6] = 1;      \
        to[7] = 1;      \
        to[8] = 1;      \
        to[9] = 1;      \
        to[10] = 1;     \
        to[11] = 1;     \
        to[12] = 1;     \
        to[13] = 1;     \
        to[14] = 1;     \
        to[15] = 1;     \
        to += 16;       \
        numBytes -= 128;

#define WRITE_UNROLLED_BYTES   512

    while (numBytes > WRITE_UNROLLED_BYTES)
    {
        WRITE_128_BYTES
        WRITE_128_BYTES
        WRITE_128_BYTES
        WRITE_128_BYTES
    }
#undef WRITE_128_BYTES
}


/**
 * Perform unrolled U32 write.
 */
static void
mb_write32unrolled(
    int32           numBytes,
    volatile uint32 *to)
{
#define WRITE_64_BYTES  \
        to[0] = 1;      \
        to[1] = 1;      \
        to[2] = 1;      \
        to[3] = 1;      \
        to[4] = 1;      \
        to[5] = 1;      \
        to[6] = 1;      \
        to[7] = 1;      \
        to[8] = 1;      \
        to[9] = 1;      \
        to[10] = 1;     \
        to[11] = 1;     \
        to[12] = 1;     \
        to[13] = 1;     \
        to[14] = 1;     \
        to[15] = 1;     \
        to += 16;       \
        numBytes -= 64;

    while (numBytes > WRITE_UNROLLED_BYTES)
    {
        WRITE_64_BYTES
        WRITE_64_BYTES
        WRITE_64_BYTES
        WRITE_64_BYTES
        WRITE_64_BYTES
        WRITE_64_BYTES
        WRITE_64_BYTES
        WRITE_64_BYTES
    }
#undef WRITE_64_BYTES
}


/**
 * Do write test.
 */
int32
mb_write(
    int32   numBytes,
    int32   numIter,
    int32   memType,
    uint32  options,
    int32   dataSize,
    char    *testName)
{
    char        *datap;
    eeeRTC_t    startTime, delta, sum;
    int32       i;
    int32       iter;

    if (numBytes == 0 ||
        numIter == 0 ||
        numBytes % dataSize)
    {
        printf("%s: bad params (numBytes: %d, numIter: %d)\n",
            testName, numBytes, numIter);
        return NFX_ERR;
    }

    datap = mb_malloc(memType, &numBytes);
    if (!datap)
    {
        printf("%s: malloc failed\n", testName);
        return NFX_ERR;
    }

if (memType == MB_MEM_TYPE_REMOTE_SHM_CACHED) {
	if (numBytes > 8192) {
		printf("numBytes truncated to 8192, to fit in 1 way of dcache\n");
		numBytes = 8192;
	}

	sum = *(eee_rtcZero());
	for (iter = 0; iter < numIter; iter++) {
		memset(datap, iter, numBytes);
		eee_rtc(&startTime);
		mb_cacheWritebackInv((address_t)datap, numBytes);
		eee_rtcDuration(&startTime, &delta);
		eee_rtcIncr(&sum, &delta);
	}
}
else
{

    if (options & MB_OPT_INIT_CACHE)
    {
        bzero(datap, numBytes);
    }

    if (options & MB_OPT_DO_UNROLLED)
    {
        if (numBytes % WRITE_UNROLLED_BYTES)
        {
            numBytes &= ~(WRITE_UNROLLED_BYTES - 1);
            printf("%s: unrolled test truncating to %d bytes\n",
                testName, numBytes);
        }

        if (dataSize == sizeof(uint32))
        {
            eee_rtc(&startTime);
            TRIGGER_START
            for (iter = 0; iter < numIter; iter++)
            {
                mb_write32unrolled(numBytes, (volatile uint32 *) datap);
            }
        }
        else
        {
            eee_rtc(&startTime);
            TRIGGER_START
            for (iter = 0; iter < numIter; iter++)
            {
                mb_write64unrolled(numBytes, (volatile uint64 *) datap);
            }
        }
    }
    else
    {
        if (dataSize == sizeof(uint32))
        {
            eee_rtc(&startTime);
            TRIGGER_START
            for (iter = 0; iter < numIter; iter++)
            {
                for (i = 0; i < numBytes; i += sizeof(uint32))
                {
                    *((volatile uint32 *)(datap + i)) = i;
                }
            }
        }
        else
        {
            eee_rtc(&startTime);
            TRIGGER_START
            for (iter = 0; iter < numIter; iter++)
            {
                for (i = 0; i < numBytes; i += sizeof(uint64))
                {
                    *((volatile uint64 *)(datap + i)) = i;
                }
            }
        }
    }
    TRIGGER_END
	eee_rtcDuration(&startTime, &delta);
	eee_rtcIncr(&sum, &delta);

if (memType == MB_MEM_TYPE_REMOTE_SHM_CACHED) {
mb_cacheWritebackInv((address_t)datap, numBytes);
}

}
	mb_printResult(testName, numBytes, numIter,
					   eee_rtcToUsec(&sum));

    mb_free(memType, datap);

    return NFX_OK;
}


/**
 * Do U64 write test.
 */
int32
mb_write64(
    int32   numBytes,
    int32   numIter,
    int32   memType,
    uint32  options)
{
    return mb_write(numBytes, numIter, memType,
                    options, sizeof(uint64), "mb_write64");
}


/**
 * Do U32 write test.
 */
int32
mb_write32(
    int32   numBytes,
    int32   numIter,
    int32   memType,
    uint32  options)
{
    return mb_write(numBytes, numIter, memType,
                    options, sizeof(uint32), "mb_write32");
}


/**
 * Perform unrolled read by cache-line.
 */
static void
mb_readCacheLineUnrolled(
    int32           numBytes,
    volatile uint64 *from)
{

// hard-coded for 32-byte cache line size
#define READ_2048_BYTES                  \
        __asm__ __volatile__ (          \
        "     .set noreorder  \n"       \
        "     ld $8,     0($5) \n"       \
        "     ld $9,    32($5) \n"       \
        "     ld $10,   64($5) \n"       \
        "     ld $11,   96($5) \n"       \
        "     ld $12,  128($5) \n"       \
        "     ld $13,  160($5) \n"       \
        "     ld $14,  192($5) \n"       \
        "     ld $15,  224($5) \n"       \
        "     ld $24,  256($5) \n"       \
        "     ld $25,  288($5) \n"       \
        "     ld $26,  320($5) \n"       \
        "     ld $27,  352($5) \n"       \
        "     ld $28,  384($5) \n"       \
        "     ld $2,   416($5) \n"       \
        "     ld $3,   448($5) \n"       \
        "     ld $0,   480($5) \n"       \
        "     ld $8,   0x200($5) \n"       \
        "     ld $9,   0x220($5) \n"       \
        "     ld $10,  0x240($5) \n"       \
        "     ld $11,  0x260($5) \n"       \
        "     ld $12,  0x280($5) \n"       \
        "     ld $13,  0x2A0($5) \n"       \
        "     ld $14,  0x2C0($5) \n"       \
        "     ld $15,  0x2E0($5) \n"       \
        "     ld $24,  0x300($5) \n"       \
        "     ld $25,  0x320($5) \n"       \
        "     ld $26,  0x340($5) \n"       \
        "     ld $27,  0x360($5) \n"       \
        "     ld $28,  0x380($5) \n"       \
        "     ld $2,   0x3A0($5) \n"       \
        "     ld $3,   0x3C0($5) \n"       \
        "     ld $0,   0x3E0($5) \n"       \
        "     ld $8,   0x400($5) \n"       \
        "     ld $9,   0x420($5) \n"       \
        "     ld $10,  0x440($5) \n"       \
        "     ld $11,  0x460($5) \n"       \
        "     ld $12,  0x480($5) \n"       \
        "     ld $13,  0x4A0($5) \n"       \
        "     ld $14,  0x4C0($5) \n"       \
        "     ld $15,  0x4E0($5) \n"       \
        "     ld $24,  0x500($5) \n"       \
        "     ld $25,  0x520($5) \n"       \
        "     ld $26,  0x540($5) \n"       \
        "     ld $27,  0x560($5) \n"       \
        "     ld $28,  0x580($5) \n"       \
        "     ld $2,   0x5A0($5) \n"       \
        "     ld $3,   0x5C0($5) \n"       \
        "     ld $0,   0x5E0($5) \n"       \
        "     ld $8,   0x600($5) \n"       \
        "     ld $9,   0x620($5) \n"       \
        "     ld $10,  0x640($5) \n"       \
        "     ld $11,  0x660($5) \n"       \
        "     ld $12,  0x680($5) \n"       \
        "     ld $13,  0x6A0($5) \n"       \
        "     ld $14,  0x6C0($5) \n"       \
        "     ld $15,  0x6E0($5) \n"       \
        "     ld $24,  0x700($5) \n"       \
        "     ld $25,  0x720($5) \n"       \
        "     ld $26,  0x740($5) \n"       \
        "     ld $27,  0x760($5) \n"       \
        "     ld $28,  0x780($5) \n"       \
        "     ld $2,   0x7A0($5) \n"       \
        "     ld $3,   0x7C0($5) \n"       \
        "     ld $0,   0x7E0($5) \n"       \
        "     daddiu   $5,$5,2048 \n"     \
        "     addiu   %0,%0,-2048 \n"    \
        "     .set reorder \n"          \
        : "=r" (numBytes)               \
        );

#define READ_CACHE_LINE_UNROLLED_BYTES     2048

    while (numBytes > READ_CACHE_LINE_UNROLLED_BYTES)
    {
        READ_2048_BYTES
    }
#undef READ_2048_BYTES
}


/**
 * Perform unrolled U64 read.
 */
static void
mb_read64unrolled(
    int32           numBytes,
    volatile uint64 *from)
{
#define READ_128_BYTES      \
        (void) *(from+0);   \
        (void) *(from+1);   \
        (void) *(from+2);   \
        (void) *(from+3);   \
        (void) *(from+4);   \
        (void) *(from+5);   \
        (void) *(from+6);   \
        (void) *(from+7);   \
        (void) *(from+8);   \
        (void) *(from+9);   \
        (void) *(from+10);  \
        (void) *(from+11);  \
        (void) *(from+12);  \
        (void) *(from+13);  \
        (void) *(from+14);  \
        (void) *(from+15);  \
        from += 16;         \
        numBytes -= 128;

#define READ_UNROLLED_BYTES     512

    while (numBytes > READ_UNROLLED_BYTES)
    {
        READ_128_BYTES
        READ_128_BYTES
        READ_128_BYTES
        READ_128_BYTES
    }
#undef READ_128_BYTES
}


/**
 * Perform unrolled U32 read.
 */
static void
mb_read32unrolled(
    int32           numBytes,
    volatile uint32 *from)
{
#define READ_64_BYTES       \
        (void) *(from+0);   \
        (void) *(from+1);   \
        (void) *(from+2);   \
        (void) *(from+3);   \
        (void) *(from+4);   \
        (void) *(from+5);   \
        (void) *(from+6);   \
        (void) *(from+7);   \
        (void) *(from+8);   \
        (void) *(from+9);   \
        (void) *(from+10);  \
        (void) *(from+11);  \
        (void) *(from+12);  \
        (void) *(from+13);  \
        (void) *(from+14);  \
        (void) *(from+15);  \
        from += 16;         \
        numBytes -= 64;

    while (numBytes > READ_UNROLLED_BYTES)
    {
        READ_64_BYTES
        READ_64_BYTES
        READ_64_BYTES
        READ_64_BYTES
        READ_64_BYTES
        READ_64_BYTES
        READ_64_BYTES
        READ_64_BYTES
    }
#undef READ_64_BYTES
}


/**
 * Do U32/U64 read test.
 */
int32
mb_read(
    int32   numBytes,
    int32   numIter,
    int32   memType,
    uint32  options,
    int32   dataSize,
    char    *testName)
{
    char        *datap;
    eeeRTC_t    startTime, delta;
    int32       i;
    uint32      summ;
    int32       iter;

    if (numBytes == 0 ||
        numIter == 0 ||
        numBytes % dataSize)
    {
        printf("%s: bad params (numBytes: %d, numIter: %d)\n",
            testName, numBytes, numIter);
        return NFX_ERR;
    }

    datap = mb_malloc(memType, &numBytes);
    if (!datap)
    {
        printf("%s: malloc failed\n", testName);
        return NFX_ERR;
    }

    if (options & MB_OPT_INIT_CACHE)
    {
        bzero(datap, numBytes);
    }

    if (options & MB_OPT_DO_UNROLLED)
    {
        if (numBytes % READ_UNROLLED_BYTES)
        {
            numBytes &= ~(READ_UNROLLED_BYTES - 1);
            printf("%s: unrolled test truncating to %d bytes\n",
                testName, numBytes);
        }

        if (options & MB_OPT_CACHE_LINE)
        {
            if (numBytes % READ_CACHE_LINE_UNROLLED_BYTES)
            {
                numBytes &= ~(READ_CACHE_LINE_UNROLLED_BYTES - 1);
                printf("%s: unrolled test truncating to %d bytes\n",
                    testName, numBytes);
            }

printf("read cacheline unrolled\n");
            eee_rtc(&startTime);
            TRIGGER_START
            for (iter = 0; iter < numIter; iter++)
            {
                mb_readCacheLineUnrolled(numBytes, (volatile uint64 *) datap);
            }
        }
        else if (dataSize == sizeof(uint32))
        {
            eee_rtc(&startTime);
            TRIGGER_START
            for (iter = 0; iter < numIter; iter++)
            {
                mb_read32unrolled(numBytes, (volatile uint32 *) datap);
            }
        }
        else
        {
            eee_rtc(&startTime);
            TRIGGER_START
            for (iter = 0; iter < numIter; iter++)
            {
                mb_read64unrolled(numBytes, (volatile uint64 *) datap);
            }
        }
    }
    else
    {
        if (dataSize == sizeof(uint32))
        {
            eee_rtc(&startTime);
            TRIGGER_START
            for (iter = 0; iter < numIter; iter++)
            {
                for (i = 0; i < numBytes; i += sizeof(uint32))
                {
                    summ += *((volatile uint32 *)(datap + i));
                }
            }
        }
        else
        {
            eee_rtc(&startTime);
            TRIGGER_START
            for (iter = 0; iter < numIter; iter++)
            {
                for (i = 0; i < numBytes; i += sizeof(uint64))
                {
                    summ += *((volatile uint64 *)(datap + i));
                }
            }
        }
    }
    TRIGGER_END

		eee_rtcDuration(&startTime, &delta);

if (memType == MB_MEM_TYPE_REMOTE_SHM_CACHED) {
mb_cacheInvalidate((address_t)datap, numBytes);
}

    mb_printResult(testName, numBytes, numIter,
				   eee_rtcToUsec(&delta));

    mb_free(memType, datap);

    return NFX_OK;
}


/**
 * Do U32 read test.
 */
int32
mb_read32(
    int32   numBytes,
    int32   numIter,
    int32   memType,
    uint32  options)
{
    return mb_read(numBytes, numIter, memType,
                   options, sizeof(uint32), "mb_read32");
}


/**
 * Do U64 read test.
 */
int32
mb_read64(
    int32   numBytes,
    int32   numIter,
    int32   memType,
    uint32  options)
{
    return mb_read(numBytes, numIter, memType,
                   options, sizeof(uint64), "mb_read64");
}



/**
 * Perform unrolled U64 copy.
 */
static void
mb_copy64unrolled(
    int32           numBytes,
    volatile uint64 *from,
    volatile uint64 *to)
{
#define COPY_128_BYTES      \
        to[0] = from[0];    \
        to[1] = from[1];    \
        to[2] = from[2];    \
        to[3] = from[3];    \
        to[4] = from[4];    \
        to[5] = from[5];    \
        to[6] = from[6];    \
        to[7] = from[7];    \
        to[8] = from[8];    \
        to[9] = from[9];    \
        to[10] = from[10];  \
        to[11] = from[11];  \
        to[12] = from[12];  \
        to[13] = from[13];  \
        to[14] = from[14];  \
        to[15] = from[15];  \
        to += 16;           \
        from += 16;         \
        numBytes -= 128;

#define COPY_UNROLLED_BYTES     512

    while (numBytes > COPY_UNROLLED_BYTES)
    {
        COPY_128_BYTES
        COPY_128_BYTES
        COPY_128_BYTES
        COPY_128_BYTES
    }
#undef COPY_128_BYTES
}


/**
 * Perform unrolled U32 copy.
 */
static void
mb_copy32unrolled(
    int32           numBytes,
    volatile uint32 *from,
    volatile uint32 *to)
{
#define COPY_64_BYTES       \
        to[0] = from[0];    \
        to[1] = from[1];    \
        to[2] = from[2];    \
        to[3] = from[3];    \
        to[4] = from[4];    \
        to[5] = from[5];    \
        to[6] = from[6];    \
        to[7] = from[7];    \
        to[8] = from[8];    \
        to[9] = from[9];    \
        to[10] = from[10];  \
        to[11] = from[11];  \
        to[12] = from[12];  \
        to[13] = from[13];  \
        to[14] = from[14];  \
        to[15] = from[15];  \
        to += 16;           \
        from += 16;         \
        numBytes -= 64;

    while (numBytes > COPY_UNROLLED_BYTES)
    {
        COPY_64_BYTES
        COPY_64_BYTES
        COPY_64_BYTES
        COPY_64_BYTES
        COPY_64_BYTES
        COPY_64_BYTES
        COPY_64_BYTES
        COPY_64_BYTES
    }
#undef COPY_64_BYTES
}


/**
 * Do U32/U64 copy test.
 */
int32
mb_copy(
    int32   numBytes,
    int32   numIter,
    int32   fromMemType,
    int32   toMemType,
    uint32  options,
    int32   dataSize,
    char    *testName)
{
    char        *from, *to;
    eeeRTC_t    startTime, delta;
    int32       iter;

    if (numBytes == 0 ||
        numIter == 0 ||
        numBytes % dataSize)
    {
        printf("%s: bad params (numBytes: %d, numIter: %d)\n",
            testName, numBytes, numIter);
        return NFX_ERR;
    }

    from = mb_malloc(fromMemType, &numBytes);
    if (!from)
    {
        printf("%s: malloc failed\n", testName);
        return NFX_ERR;
    }
    to = mb_malloc(toMemType, &numBytes);
    if (!to)
    {
        printf("%s: malloc failed\n", testName);
        mb_free(fromMemType, from);
        return NFX_ERR;
    }

    if (options & MB_OPT_INIT_CACHE)
    {
        bzero(from, numBytes);
        bzero(to, numBytes);
    }

    if (options & MB_OPT_DO_UNROLLED)
    {
        if (numBytes % COPY_UNROLLED_BYTES)
        {
            numBytes &= ~(COPY_UNROLLED_BYTES - 1);
            printf("%s: unrolled test truncating to %d bytes\n",
                testName, numBytes);
        }

        if (dataSize == sizeof(uint32))
        {
            eee_rtc(&startTime);
            TRIGGER_START
            for (iter = 0; iter < numIter; iter++)
            {
if (toMemType == MB_MEM_TYPE_REMOTE_SHM_CACHED) {
mb_cacheCreateDirty((address_t)to, numBytes);
}
                mb_copy32unrolled(numBytes, (volatile uint32 *) from,
                                            (volatile uint32 *) to);
if (fromMemType == MB_MEM_TYPE_REMOTE_SHM_CACHED) {
mb_cacheInvalidate((address_t)from, numBytes);
}
if (toMemType == MB_MEM_TYPE_REMOTE_SHM_CACHED) {
mb_cacheWritebackInv((address_t)to, numBytes);
}
            }
        }
        else
        {
            eee_rtc(&startTime);
            TRIGGER_START
            for (iter = 0; iter < numIter; iter++)
            {
if (toMemType == MB_MEM_TYPE_REMOTE_SHM_CACHED) {
mb_cacheCreateDirty((address_t)to, numBytes);
}
                mb_copy64unrolled(numBytes, (volatile uint64 *) from,
                                            (volatile uint64 *) to);
if (fromMemType == MB_MEM_TYPE_REMOTE_SHM_CACHED) {
mb_cacheInvalidate((address_t)from, numBytes);
}
if (toMemType == MB_MEM_TYPE_REMOTE_SHM_CACHED) {
mb_cacheWritebackInv((address_t)to, numBytes);
}
            }
        }
    }
    else
    {
        printf("FYI, copy test optimized for platform (32/64 bit)\n");
        eee_rtc(&startTime);
        TRIGGER_START
        for (iter = 0; iter < numIter; iter++)
        {
if (toMemType == MB_MEM_TYPE_REMOTE_SHM_CACHED) {
mb_cacheCreateDirty((address_t)to, numBytes);
}
            memcpy(to, from, numBytes);
if (fromMemType == MB_MEM_TYPE_REMOTE_SHM_CACHED) {
mb_cacheInvalidate((address_t)from, numBytes);
}
if (toMemType == MB_MEM_TYPE_REMOTE_SHM_CACHED) {
mb_cacheWritebackInv((address_t)to, numBytes);
}
        }
    }
    TRIGGER_END
	eee_rtcDuration(&startTime, &delta);

    mb_printResult(testName, numBytes, numIter,
				   eee_rtcToUsec(&delta));

    mb_free(fromMemType, from);
    mb_free(toMemType, to);

    return NFX_OK;
}


/**
 * Do U32 copy test.
 */
int32
mb_copy32(
    int32   numBytes,
    int32   numIter,
    int32   fromMemType,
    int32   toMemType,
    uint32  options)
{
    return mb_copy(numBytes, numIter, fromMemType, toMemType,
                    options, sizeof(uint32), "mb_copy32");
}   

/**
 * Do U64 copy test.
 */
int32
mb_copy64(
    int32   numBytes,
    int32   numIter,
    int32   fromMemType,
    int32   toMemType,
    uint32  options)
{
    return mb_copy(numBytes, numIter, fromMemType, toMemType,
                   options, sizeof(uint64), "mb_copy64");
}   

typedef struct mb_CommonArgs
{
    int32   numBytes;
    int32   numIter;
    int32   memType;
    int32   options;
} mb_CommonArgs;


/**
 * Common parser for most tests.
 */
static int32
mb_parseCommon(int argc, char **argv, mb_CommonArgs *commonArgs)
{
    int32   i;

    if (argc < 4)
    {
        goto usage;
    }

    commonArgs->numBytes = atoi(argv[1]);
    commonArgs->numIter = atoi(argv[2]);
    if (!strcmp(argv[3], "heap"))
        commonArgs->memType = MB_MEM_TYPE_HEAP;
    else if (!strcmp(argv[3], "lcl"))
        commonArgs->memType = MB_MEM_TYPE_LOCAL_SHM;
    else if (!strcmp(argv[3], "rmt_c"))
        commonArgs->memType = MB_MEM_TYPE_REMOTE_SHM_CACHED;
    else if (!strcmp(argv[3], "rmt"))
        commonArgs->memType = MB_MEM_TYPE_REMOTE_SHM;
    else
        goto usage;

    commonArgs->options = 0;
    if (argc >= 5)
    {
        for (i = 0; i < argc - 4; i++)
        {
            if (!strcmp(argv[4 + i], "-u"))
                commonArgs->options |= MB_OPT_DO_UNROLLED;
            else if (!strcmp(argv[4 + i], "-c"))
                commonArgs->options |= MB_OPT_INIT_CACHE;
            else if (!strcmp(argv[4 + i], "-l"))
                commonArgs->options |= MB_OPT_CACHE_LINE;
            else {
                printf("unknown argument %s\n", argv[4 + i]);
                goto usage;
            }
        }
    }

    return NFX_OK;

usage:
    printf("Usage: %s <numBytes> <numIter> <memType> [-u] [-c] [-l]\n",
        argv[0]);
    printf("   use <memType>  heap   for malloc space\n");
    printf("   use <memType>  lcl    for local shm buffers\n");
    printf("   use <memType>  rmt    for rmt shm buffers\n");
    printf("   use <memType>  rmt_c  for cached rmt shm buffers\n");
    printf("   use -u to perform unrolled loops\n");
    printf("   use -c to init cache\n");
    printf("   use -l to touch by cache-line (for read tests only)\n");

    return NFX_ERR;
}


int32
mb_memBenchWrite32(int argc, char **argv)
{
    mb_CommonArgs   commonArgs;

    if (mb_parseCommon(argc, argv, &commonArgs) == NFX_OK)
    {
        return mb_write32(commonArgs.numBytes, commonArgs.numIter,
                          commonArgs.memType, commonArgs.options);
    }

    return -1;
}

int32
mb_memBenchWrite64(int argc, char **argv)
{
    mb_CommonArgs   commonArgs;

    if (mb_parseCommon(argc, argv, &commonArgs) == NFX_OK)
    {
        return mb_write64(commonArgs.numBytes, commonArgs.numIter,
                          commonArgs.memType, commonArgs.options);
    }

    return -1;
}

int32
mb_memBenchRead32(int argc, char **argv)
{
    mb_CommonArgs   commonArgs;

    if (mb_parseCommon(argc, argv, &commonArgs) == NFX_OK)
    {
        return mb_read32(commonArgs.numBytes, commonArgs.numIter,
                          commonArgs.memType, commonArgs.options);
    }

    return -1;
}

int32
mb_memBenchRead64(int argc, char **argv)
{
    mb_CommonArgs   commonArgs;

    if (mb_parseCommon(argc, argv, &commonArgs) == NFX_OK)
    {
        return mb_read64(commonArgs.numBytes, commonArgs.numIter,
                          commonArgs.memType, commonArgs.options);
    }

    return -1;
}

int32
mb_memBenchBzero(int argc, char **argv)
{
    mb_CommonArgs   commonArgs;

    if (mb_parseCommon(argc, argv, &commonArgs) == NFX_OK)
    {
        return mb_bzero(commonArgs.numBytes, commonArgs.numIter,
                        commonArgs.memType, commonArgs.options);
    }

    return -1;
}


/**
 * Structure for args to copy test.
 */
typedef struct mb_CopyArgs
{
    int32   numBytes;
    int32   numIter;
    int32   fromMemType;
    int32   toMemType;
    int32   options;
} mb_CopyArgs;


/**
 * Parser for copy tests.
 */
static int32
mb_parseCopy(int argc, char **argv, mb_CopyArgs *copyArgs)
{
    int32   i;

    if (argc < 5)
    {
        goto usage;
    }

    copyArgs->numBytes = atoi(argv[1]);
    copyArgs->numIter = atoi(argv[2]);
    if (!strcmp(argv[3], "heap"))
        copyArgs->fromMemType = MB_MEM_TYPE_HEAP;
    else if (!strcmp(argv[3], "lcl"))
        copyArgs->fromMemType = MB_MEM_TYPE_LOCAL_SHM;
    else if (!strcmp(argv[3], "rmt_c"))
        copyArgs->fromMemType = MB_MEM_TYPE_REMOTE_SHM_CACHED;
    else if (!strcmp(argv[3], "rmt"))
        copyArgs->fromMemType = MB_MEM_TYPE_REMOTE_SHM;
    else
        goto usage;
    if (!strcmp(argv[4], "heap"))
        copyArgs->toMemType = MB_MEM_TYPE_HEAP;
    else if (!strcmp(argv[4], "lcl"))
        copyArgs->toMemType = MB_MEM_TYPE_LOCAL_SHM;
    else if (!strcmp(argv[4], "rmt_c"))
        copyArgs->toMemType = MB_MEM_TYPE_REMOTE_SHM_CACHED;
    else if (!strcmp(argv[4], "rmt"))
        copyArgs->toMemType = MB_MEM_TYPE_REMOTE_SHM;
    else
        goto usage;

    copyArgs->options = 0;
    if (argc >= 6)
    {
        for (i = 0; i < argc - 5; i++)
        {
            if (!strcmp(argv[5 + i], "-u"))
                copyArgs->options |= MB_OPT_DO_UNROLLED;
            else if (!strcmp(argv[5 + i], "-c"))
                copyArgs->options |= MB_OPT_INIT_CACHE;
            else {
                printf("unknown argument %s\n", argv[4 + i]);
                goto usage;
            }
        }
    }

    return NFX_OK;

usage:
    printf(
     "Usage: %s <numBytes> <numIter> <fromMemType> <toMemType> [-u] [-c]\n",
        argv[0]);
    printf("   use <memType>  heap   for malloc space\n");
    printf("   use <memType>  lcl    for local shm buffers\n");
    printf("   use <memType>  rmt    for rmt shm buffers\n");
    printf("   use <memType>  rmt_c  for cached rmt shm buffers\n");
    printf("   use -u to perform unrolled loops\n");
    printf("   use -c to init cache\n");

    return NFX_ERR;
}

int32
mb_memBenchCopy32(int argc, char **argv)
{
    mb_CopyArgs   copyArgs;

    if (mb_parseCopy(argc, argv, &copyArgs) == NFX_OK)
    {
        return mb_copy32(copyArgs.numBytes, copyArgs.numIter,
                         copyArgs.fromMemType, copyArgs.toMemType,
                         copyArgs.options);
    }

    return -1;
}

int32
mb_memBenchCopy64(int argc, char **argv)
{
    mb_CopyArgs   copyArgs;

    if (mb_parseCopy(argc, argv, &copyArgs) == NFX_OK)
    {
        return mb_copy64(copyArgs.numBytes, copyArgs.numIter,
                         copyArgs.fromMemType, copyArgs.toMemType,
                         copyArgs.options);
    }

    return -1;
}

#endif	/* PROM */

