/*
 *------------------------------------------------------------------------------
 *
 * Name:	tape-api.c
 *
 * RCS:		$Id: tape-api.h,v 1.14 2002/07/24 04:11:01 raster Exp $
 *
 * Copyright (C) 2001, Agile Storage Inc.
 *
 * Description:	Tape driver API definitions.
 *
 * Created by:	Radek Aster
 *
 * Date Created: 9/29/01
 *
 *------------------------------------------------------------------------------
 */

#ifndef __TAPE_API_H__
#define __TAPE_API_H__

/*
 *------------------------------------
 * Tape Message Types
 *------------------------------------
 */
typedef enum {
	TAPE_MID_GET_DEVLIST_REQ = TAPE_MSG_BASE,
	TAPE_MID_GET_DEVLIST_RSP,
	MC_MID_GET_DEVLIST_REQ,
	MC_MID_GET_DEVLIST_RSP,

	TAPE_MID_OPEN_REQ,
	TAPE_MID_OPEN_RSP,
	TAPE_MID_GET_HANDLE_REQ,
	TAPE_MID_GET_HANDLE_RSP,
	TAPE_MID_CLOSE_REQ,
	TAPE_MID_CLOSE_RSP,
	MC_MID_OPEN_REQ,
	MC_MID_OPEN_RSP,
	MC_MID_GET_HANDLE_REQ,
	MC_MID_GET_HANDLE_RSP,
	MC_MID_CLOSE_REQ,
	MC_MID_CLOSE_RSP,

	TAPE_MID_GET_STATE_REQ,
	TAPE_MID_GET_STATE_RSP,
	TAPE_MID_SPACE_REQ,
	TAPE_MID_SPACE_RSP,
	TAPE_MID_WRITE_FM_REQ,
	TAPE_MID_WRITE_FM_RSP,
	TAPE_MID_LOAD_REQ,
	TAPE_MID_LOAD_RSP,
	TAPE_MID_READ_REQ,
	TAPE_MID_READ_RSP,
	TAPE_MID_WRITE_REQ,
	TAPE_MID_WRITE_RSP,
	TAPE_MID_PASSTHRU_REQ,
	TAPE_MID_PASSTHRU_RSP,
	MC_MID_PASSTHRU_REQ,
	MC_MID_PASSTHRU_RSP,

	TAPE_MID_ERROR_RSP,	/* Wildcard error */

	TAPE_MID_SET_DEBUG_LEVEL_REQ,
	TAPE_MID_SET_DEBUG_LEVEL_RSP,
    TAPE_MID_ID_CLOSE_REQ /* Close tape/mc based on client tag */
} tape_msg_type_t;

/* Devicename length: WWN (16) + 1 + rewind attribute (2) + 1 + unload attribute (2) + 1 + density name (32) + 1 */
#define TAPE_MAX_DEVICENAME	56
#define MC_MAX_DEVICENAME	18

typedef struct {
	tape_msg_type_t	mid;		/* Message Id */
	uint64		cookie;		/* Client transaction handle/cookie */
} tape_msg_hdr_t;

/*
 *-----------------------------------------------
 * DEVICE DISCOVERY GROUP
 *-----------------------------------------------
 */
#define TAPE_ATTRIB_REWIND_ON_CLOSE	0x00000001
#define TAPE_ATTRIB_UNLOAD_ON_CLOSE	0x00000002
#define TAPE_ATTRIB_DENSITY_MASK	0x00000030
#define TAPE_ATTRIB_DENSITY_SHIFT	4
#define TAPE_ATTRIB_DENSITY_0		0x00000000
#define TAPE_ATTRIB_DENSITY_1		0x00000010
#define TAPE_ATTRIB_DENSITY_2		0x00000020
#define TAPE_ATTRIB_DENSITY_3		0x00000030

typedef struct {
	char		devicename[TAPE_MAX_DEVICENAME];
	char		physical_devicename[TAPE_MAX_DEVICENAME];
	char		scsi_vendor_id[10];
	char		scsi_product_id[18];
	scsi_id_t	scsi_id;
	uint32		attributes;
} tape_dev_info_t;

typedef struct {
	tape_msg_hdr_t  hdr;			/* TAPE_MID_GET_DEVLIST_REQ */
} tape_get_devlist_req_msg_t;

typedef struct {
	tape_msg_hdr_t  hdr;			/* TAPE_MID_GET_DEVLIST_RSP */
	uint32		ecode;			/* return code */
	uint32		dev_count;		/* Number of devices available */
	tape_dev_info_t	info[1];		/* List of available devices */
} tape_get_devlist_rsp_msg_t;

typedef struct {
	char		devicename[MC_MAX_DEVICENAME];
	char		physical_devicename[MC_MAX_DEVICENAME];
	char		scsi_vendor_id[10];
	char		scsi_product_id[18];
	scsi_id_t	scsi_id;
} mc_dev_info_t;

typedef struct {
	tape_msg_hdr_t  hdr;			/* MC_MID_GET_DEVLIST_REQ */
} mc_get_devlist_req_msg_t;

typedef struct {
	tape_msg_hdr_t  hdr;			/* MC_MID_GET_DEVLIST_RSP */
	uint32		ecode;			/* return code */
	uint32		dev_count;		/* Number of devices available */
	mc_dev_info_t	info[1];		/* List of available devices */
} mc_get_devlist_rsp_msg_t;

/*
 *-----------------------------------------------
 * CONNECTION ESTABLISHMENT/TEARDOWN GROUP
 *-----------------------------------------------
 */
typedef struct {
	tape_msg_hdr_t  hdr;			/* TAPE_MID_OPEN_REQ */
	char		devicename[TAPE_MAX_DEVICENAME]; /* Logical device name */
	uint32		client_tag;		/* Client/owner tag */
} tape_open_req_msg_t;

typedef struct {
	tape_msg_hdr_t  hdr;			/* TAPE_MID_OPEN_RSP */
	uint32		ecode;			/* return code */
	uint32		handle;			/* SCSI device handle */
	scsi_id_t	scsi_id;		/* 64-bit SCSI ID of open device */
	uint64		wwn;			/* WWN of open device */
	uint32		lun;			/* LUN of open device */ 
	char		devicename[TAPE_MAX_DEVICENAME];
	char		physical_devicename[TAPE_MAX_DEVICENAME];
	uint32		client_tag;		/* Client/owner tag */
} tape_open_rsp_msg_t;

typedef struct {
	tape_msg_hdr_t  hdr;			/* TAPE_MID_GET_HANDLE_REQ */
	char		devicename[TAPE_MAX_DEVICENAME]; /* Logical device name */
} tape_get_handle_req_msg_t;

typedef struct {
	tape_msg_hdr_t  hdr;			/* TAPE_MID_GET_HANDLE_RSP */
	uint32		ecode;			/* return code */
	uint32		handle;			/* SCSI device handle */
	scsi_id_t	scsi_id;		/* 64-bit SCSI ID of open device */
	uint64		wwn;			/* WWN of open device */
	uint32		lun;			/* LUN of open device */ 
	char		devicename[TAPE_MAX_DEVICENAME];
	char		physical_devicename[TAPE_MAX_DEVICENAME];
} tape_get_handle_rsp_msg_t;

typedef struct {
	tape_msg_hdr_t  hdr;			/* TAPE_MID_CLOSE_REQ */
	char		devicename[TAPE_MAX_DEVICENAME]; /* Logical device name */
	uint32		client_tag;		/* Client/owner tag */
} tape_close_req_msg_t;

typedef struct {
	tape_msg_hdr_t  hdr;			/* TAPE_MID_CLOSE_RSP */
	uint32		ecode;			/* return code */
} tape_close_rsp_msg_t;

typedef struct {
	tape_msg_hdr_t  hdr;			/* MC_MID_OPEN_REQ */
	char		devicename[MC_MAX_DEVICENAME]; /* Logical device name */
	uint32		client_tag;		/* Client/owner tag */
} mc_open_req_msg_t;

typedef struct {
	tape_msg_hdr_t  hdr;			/* MC_MID_OPEN_RSP */
	uint32		ecode;			/* return code */
	uint32		handle;			/* SCSI device handle */
	scsi_id_t	scsi_id;		/* 64-bit SCSI ID of open device */
	uint64		wwn;			/* WWN of open device */
	uint32		lun;			/* LUN of open device */ 
	char		devicename[MC_MAX_DEVICENAME];
	char		physical_devicename[MC_MAX_DEVICENAME];
	uint32		client_tag;		/* Client/owner tag */
} mc_open_rsp_msg_t;

typedef struct {
	tape_msg_hdr_t  hdr;			/* MC_MID_GET_HANDLE_REQ */
	char		devicename[MC_MAX_DEVICENAME]; /* Logical device name */
} mc_get_handle_req_msg_t;

typedef struct {
	tape_msg_hdr_t  hdr;			/* MC_MID_GET_HANDLE_RSP */
	uint32		ecode;			/* return code */
	uint32		handle;			/* SCSI device handle */
	scsi_id_t	scsi_id;		/* 64-bit SCSI ID of open device */
	uint64		wwn;			/* WWN of open device */
	uint32		lun;			/* LUN of open device */ 
	char		devicename[MC_MAX_DEVICENAME];
	char		physical_devicename[MC_MAX_DEVICENAME];
} mc_get_handle_rsp_msg_t;

typedef struct {
	tape_msg_hdr_t  hdr;			/* MC_MID_CLOSE_REQ */
	char		devicename[MC_MAX_DEVICENAME]; /* Logical device name */
	uint32		client_tag;		/* Client/owner tag */
} mc_close_req_msg_t;

typedef struct {
	tape_msg_hdr_t  hdr;			/* MC_MID_CLOSE_RSP */
	uint32		ecode;			/* return code */
} mc_close_rsp_msg_t;

/*
 *-----------------------------------------------
 * CONTROL GROUP
 *-----------------------------------------------
 */
typedef struct {
	tape_msg_hdr_t  hdr;			/* TAPE_MID_GET_STATE_REQ */
	uint32		handle;			/* SCSI device handle */
} tape_get_state_req_msg_t; 

#define TAPE_STATE_FLAGS_REWIND_ON_CLOSE	0x00000001
#define TAPE_STATE_FLAGS_UNLOAD_ON_CLOSE	0x00000002
#define TAPE_STATE_FLAGS_DENSITY_MASK		0x00000030
#define TAPE_STATE_FLAGS_DENSITY_SHIFT		4
#define TAPE_STATE_FLAGS_DENSITY_0		0x00000000
#define TAPE_STATE_FLAGS_DENSITY_1		0x00000010
#define TAPE_STATE_FLAGS_DENSITY_2		0x00000020
#define TAPE_STATE_FLAGS_DENSITY_3		0x00000030
#define TAPE_STATE_FLAGS_WRT_PROTECT		0x00000100
#define TAPE_STATE_FLAGS_BOM			0x00000200
#define TAPE_STATE_FLAGS_EOM			0x00000400
#define TAPE_STATE_FLAGS_FILENO_VALID		0x00010000
#define TAPE_STATE_FLAGS_BLOCKNO_VALID		0x00020000
#define TAPE_STATE_FLAGS_BLOCKSIZE_VALID	0x00040000
#define TAPE_STATE_FLAGS_TOTAL_SPACE_VALID	0x00080000
#define TAPE_STATE_FLAGS_SPACE_REMAINING_VALID	0x00100000
#define TAPE_STATE_FLAGS_SOFT_ERRORS_VALID	0x00200000

typedef struct {
	tape_msg_hdr_t  hdr;			/* TAPE_MID_GET_STATE_RSP */
	uint32		ecode;			/* return code */
	uint32		flags;
	uint32		fileno;
	uint32		blockno;
	uint32		blocksize;
	uint64		total_space;
	uint64		space_remaining;
	uint32		soft_errors;
} tape_get_state_rsp_msg_t; 

#define TAPE_SPACE_OPCODE_BLOCKS	0
#define TAPE_SPACE_OPCODE_FILEMARKS	1
#define TAPE_SPACE_OPCODE_END_OF_DATA	2
#define TAPE_SPACE_OPCODE_REWIND	3

typedef struct {
	tape_msg_hdr_t  hdr;			/* TAPE_MID_SPACE_REQ */
	uint32		handle;			/* SCSI device handle */
	uint32		opcode;			
	int32		count;			/* negative means backward */
} tape_space_req_msg_t; 

typedef struct {
	tape_msg_hdr_t  hdr;			/* TAPE_MID_SPACE_RSP */
	uint32		ecode;			/* return code */
	int32		residue;		/* residual count */
} tape_space_rsp_msg_t; 

typedef struct {
	tape_msg_hdr_t  hdr;			/* TAPE_MID_WRITE_FM_REQ */
	uint32		handle;			/* SCSI device handle */
	int32		count;			/* number of consecutive FMs */
} tape_write_fm_req_msg_t; 

typedef struct {
	tape_msg_hdr_t  hdr;			/* TAPE_MID_WRITE_FM_RSP */
	uint32		ecode;			/* return code */
	int32		residue;		/* residual count */
} tape_write_fm_rsp_msg_t; 

typedef struct {
	tape_msg_hdr_t  hdr;			/* TAPE_MID_LOAD_REQ */
	uint32		handle;			/* SCSI device handle */
	uint32		load_or_unload;		/* 0 == UNLOAD, 1 == LOAD */
} tape_load_req_msg_t; 

typedef struct {
	tape_msg_hdr_t  hdr;			/* TAPE_MID_GET_POSITION_RSP */
	uint32		ecode;			/* return code */
} tape_load_rsp_msg_t; 

typedef struct {
	tape_msg_hdr_t hdr;			/* TAPE_MID_READ_REQ */
	uint32		handle;			/* SCSI device handle */
	uint32 		bytecount;		/* bytes to read 
						   must be multiple of blocksize, if fixed block mode */
} tape_read_req_msg_t;

typedef struct {
	tape_msg_hdr_t hdr;			/* TAPE_MID_READ_RSP */
	uint32		ecode;			/* return code */
	uint32          bytecount;              /* Number of bytes read */
	int32		residue;		/* Valid when ILI and non-zero; requested length - actual block length.
						 * Negative means overlength condition, i.e. block > requested length */
	uchar8		data[1];		/* read data to follow */
} tape_read_rsp_msg_t;

typedef struct {
	tape_msg_hdr_t hdr;			/* TAPE_MID_WRITE_REQ */
	uint32		handle;			/* SCSI device handle */
	uint32          bytecount;              /* Number of bytes */
	uchar8		data[1];		/* write data to follow 
						   must be multiple of blocksize, if fixed block mode */
} tape_write_req_msg_t;

typedef struct {
	tape_msg_hdr_t hdr;			/* TAPE_MID_WRITE_RSP */
	uint32		ecode;			/* return code */
} tape_write_rsp_msg_t;

#define TAPE_PASSTHRU_FLAGS_DATA_IN	0x00000001
#define TAPE_PASSTHRU_FLAGS_DATA_OUT	0x00000002

typedef struct {
	tape_msg_hdr_t hdr;			/* TAPE_MID_PASSTHRU_REQ */
	uint32          handle;                 /* SCSI device handle */
	uchar8		cdb[16];
	uint32		flags;
	uint32		timeout;
	int32  		data_len;		/* Expected length of data --
						 * if TAPE_PASSTHRU_FLAGS_DATA_IN, then the expected length of data in
						 * if TAPE_PASSTHRU_FLAGS_DATA_OUT, then length of data which follows
						 */
	uchar8		data[1];		/* Variable data */
} tape_passthru_req_msg_t;

typedef struct {
	tape_msg_hdr_t hdr;			/* TAPE_MID_PASSTHRU_RSP */
	uint32		ecode;			/* return code */
	uchar8		scsi_status;		/* SCSI status byte */
	uchar8		sense_data[32];
	int32		data_len;		/* Actual length of data --
						 * if TAPE_PASSTHRU_FLAGS_DATA_IN, then length of data which follows
						 * if TAPE_PASSTHRU_FLAGS_DATA_OUT, then length of data actually transmitted
						 */
	uchar8		data[1];		/* Variable data */
} tape_passthru_rsp_msg_t;

#define MC_PASSTHRU_FLAGS_DATA_IN	0x00000001
#define MC_PASSTHRU_FLAGS_DATA_OUT	0x00000002

typedef struct {
	tape_msg_hdr_t hdr;			/* MC_MID_PASSTHRU_REQ */
	uint32          handle;                 /* SCSI device handle */
	uchar8		cdb[16];
	uint32		flags;
	uint32		timeout;
	int32  		data_len;		/* Expected length of data --
						 * if MC_PASSTHRU_FLAGS_DATA_IN, then the expected length of data in
						 * if MC_PASSTHRU_FLAGS_DATA_OUT, then length of data which follows
						 */
	uchar8		data[1];		/* Variable data */
} mc_passthru_req_msg_t;

typedef struct {
	tape_msg_hdr_t hdr;			/* MC_MID_PASSTHRU_RSP */
	uint32		ecode;			/* return code */
	uchar8		scsi_status;		/* SCSI status byte */
	uchar8		sense_data[32];
	int32		data_len;		/* Actual length of data --
						 * if MC_PASSTHRU_FLAGS_DATA_IN, then length of data which follows
						 * if MC_PASSTHRU_FLAGS_DATA_OUT, then length of data actually transmitted
						 */
	uchar8		data[1];		/* Variable data */
} mc_passthru_rsp_msg_t;

/*
 *-----------------------------------------------
 * DRIVER ADMIN GROUP
 *-----------------------------------------------
 */
typedef struct {
	tape_msg_hdr_t hdr;			/* TAPE_MID_SET_DEBUG_LEVEL_REQ */
	uint32	       debug_level;
} tape_set_debug_level_req_msg_t;

typedef struct {
	tape_msg_hdr_t hdr;			/* TAPE_MID_SET_DEBUG_LEVEL_RSP */
} tape_set_debug_level_rsp_msg_t;

/*
 *-----------------------------------------------
 * ERROR CODES 
 *-----------------------------------------------
 */
#define TAPE_ECODE_SUCCESS			0
#define TAPE_ECODE_DEVICE_BUSY			1	/* Open failed -- device busy */
#define TAPE_ECODE_NO_DEVICE			2	/* Open failed -- no physical device */
#define TAPE_ECODE_NO_TAPE_LOADED		3	/* Open failed -- no tape loaded */
#define TAPE_ECODE_ILI				4	/* Incorrect-length-indicator */
#define TAPE_ECODE_EOM_ENCOUNTERED		5	/* (Logical) End-of-Media encountered */
#define TAPE_ECODE_FM_ENCOUNTERED		6	/* File-mark encountered */
#define TAPE_ECODE_DATA_SIZE_MISMATCH		7	/* Data size must be multiple of fixed block size */
#define TAPE_ECODE_MEDIUM_ERR			8	/* Medium fault */
#define TAPE_ECODE_EOD_ENCOUNTERED		9	/* End-of-data encountered */
#define TAPE_ECODE_IO_ERROR			10	
#define TAPE_ECODE_DEVICE_NOT_OPEN		11
#define TAPE_ECODE_DEVICE_NOT_OPEN_BY_CLIENT	12

/*
*************************************************************************
##                                                         M
##                                                         E
##                                                   D     D
##                                                   E     I
##                                                   V   D U
##                                                   I   A M
##                                                   C   T _ 
##                                                 D E   A E E E
##                                             N   E _   _ R O O F
##                                             O   V N   S R M D M
##                                             _   I O   I _ _ _ _
##                                         D   T   C T   Z E E E E
##                                         E   A   E _   E N N N N
##                                         V N P   _ O   _ C C C C
##                                         I O E I N P   M O O O O
##                                         C _ _ O O E   I U U U U
##                                         E D L _ T N   S N N N N
##                                         _ E O E _ _   M T T T T
##                                         B V A R O B   A E E E E
##                                         U I D R P Y I T R R R R
##                                         S C E O E _ L C E E E E
##                                         Y E D R N C I H D D D D
##                                         - - - - - - - - - - - -
## TAPE_MID_GET_DEVLIST_REQ                
## TAPE_MID_OPEN_REQ                       X X X X
## TAPE_MID_CLOSE_REQ                            X X X
## TAPE_MID_SPACE_REQ                            X X       X X X X
## TAPE_MID_WRITE_FM_REQ                         X X       X X 
## TAPE_MID_GET_STATE_REQ                        X X 
## TAPE_MID_LOAD_REQ                             X X
## TAPE_MID_READ_REQ                             X X   X   X X X X
## TAPE_MID_WRITE_REQ                            X X     X X X
## TAPE_MID_PASSTHRU_REQ                         X X
*************************************************************************
*/

#endif /* __TAPE_API_H__ */
