/*
 *------------------------------------------------------------------------------
 *
 * Name:	tape-cap.c
 *
 * RCS:		$Id: tape-cap.c,v 1.8 2002/11/05 00:37:27 nitinb Exp $
 *
 * Copyright (C) 2001, Agile Storage Inc.
 *
 * Description:	Tape driver device capabilities code.
 *
 * Created by:	Radek Aster
 *
 * Date Created: 10/16/01
 *
 *------------------------------------------------------------------------------
 */

#include "tape-common.h"

tape_cap_t tape_caps[] = {
	{
		"UNKNOWN", " ", 0,
		{
			{ DEFAULT_DENSITY, 0x00, UNKNOWN_CAPACITY,	"default-density",			"l" },
			{ DEFAULT_DENSITY, 0x00, UNKNOWN_CAPACITY,	"default-density",			"l" },
			{ DEFAULT_DENSITY, 0x00, UNKNOWN_CAPACITY,	"default-density",			"l" },
			{ DEFAULT_DENSITY, 0x01, UNKNOWN_CAPACITY,	"default-density-compressed",		"lc" }
		}
	},
	{
		"AGILESTO", "LINUX TAPE 2001", 0,
		{
			{ DEFAULT_DENSITY, 0x00, 100 * ONE_MEG,		"density-1",				"l" },
			{ 0x01,		   0x01, 100 * ONE_MEG,		"density-1-compressed",			"lc" },
			{ 0x02,		   0x00, 100 * ONE_MEG,		"density-2",				"h" },
			{ 0x03,		   0x01, 100 * ONE_MEG,		"density-2-compressed",			"hc" }
		}
	},
	{
		"QUANTUM", "DLT8000", 0,
		{
			{ 0x84,		   0x00, 35 * ONE_GIG,		"DLTtape-IV-85937-bpi",			"l" },
			{ 0x85,		   0x10, 70 * ONE_GIG,		"DLTtape-IV-85937-bpi-compressed",	"lc" },
			{ 0x88,		   0x00, 40 * ONE_GIG,		"DLTtape-IV-98250-bpi",			"h" },
			{ 0x89,		   0x10, 80 * ONE_GIG,		"DLTtape-IV-98250-bpi-compressed",	"hc" }
		}
	},
	{
		"QUANTUM", "SuperDLT1", 0,
		{
			{ 0x88,		   0x00, 40 * ONE_GIG,		"DLTtape-IV-40GB",			"l" },
			{ 0x89,		   0x10, 80 * ONE_GIG,		"DLTtape-IV-40GB-compressed",		"lc" },
			{ 0x90,		   0x00, 100 * ONE_GIG,		"SuperDLTtape-I-100GB",			"h" },
			{ 0x91,		   0x10, 200 * ONE_GIG,		"SuperDLTtape-I-100GB-compressed",	"hc" }
		}
	},
	{
		"QUANTUM", "SDLT320", 0,
		{
			{ 0x90,		   0x00, 110 * ONE_GIG,		"SuperDLTtape-I-110GB",			"l" },
			{ 0x91,		   0x10, 220 * ONE_GIG,		"SuperDLTtape-I-220GB-compressed",	"lc" },
			{ 0x92,		   0x00, 160 * ONE_GIG,		"SuperDLTtape-I-160GB",			"h" },
			{ 0x93,		   0x10, 320 * ONE_GIG,		"SuperDLTtape-I-320GB-compressed",	"hc" }
		}
	},
	{
		"SEAGATE", "ULTRIUM06242", 0,
		{
			{ 0x40,		   0x00, 100 * ONE_GIG,		"Ultrium-1",				"l" },
			{ 0x40,		   0x00, 100 * ONE_GIG,		"Ultrium-1",				"l" },
			{ 0x40,		   0x00, 100 * ONE_GIG,		"Ultrium-1",				"l" },
			{ 0x40,		   0x01, 100 * ONE_GIG,		"Ultrium-1-compressed",			"lc" },
		}
	}
};
#define	NUM_CAP_ENTRIES	(sizeof(tape_caps)/sizeof(tape_caps[0]))
#define DEFAULT_CAP	(&tape_caps[0])

tape_cap_t *
tape_find_cap_entry(char *vendor_name, char *model_name)
{
	int		i;

	for (i = 0; i < NUM_CAP_ENTRIES; ++i) {
		if (bcmp(vendor_name, tape_caps[i].vendor_id, strlen(tape_caps[i].vendor_id)) == 0 &&
		    bcmp(model_name, tape_caps[i].product_id, strlen(tape_caps[i].product_id)) == 0) {
			return(&tape_caps[i]);
		}
	}
	return(DEFAULT_CAP);
}

int 
tape_density_code_match(tape_cap_t *cap, char *density_str)
{
	int i;
	int density = NFX_ERR;

	if (cap) {
		for (i = 0; i < TAPE_DENSITY_COUNT; ++i) {
			if (strcasecmp(density_str, cap->densities[i].dens_name) == 0 ||
			    strcasecmp(density_str, cap->densities[i].alias_dens_name) == 0) {
				density = i;
				break;
			}
		}
	}
	return(density);
}

int tape_is_DLT(tape_cap_t *cap)
{
	return(strstr(cap->product_id, "DLT") != NULL);
}
