/*
 *------------------------------------------------------------------------------
 *
 * Name:	tape-main.c
 *
 * RCS:		$Id: tape-main.c,v 1.5 2002/04/16 22:04:51 raster Exp $
 *
 * Copyright (C) 2001, Agile Storage Inc.
 *
 * Description:	Tape driver code.
 *
 * Created by:	Radek Aster
 *
 * Date Created: 9/28/01
 *
 *------------------------------------------------------------------------------
 */

#include "tape-common.h"
#include "tape-msg.h"
#include "tape-discovery.h"
#include <signal.h>

void tape_terminate_handler(int);

int terminate = 0;

int
main(int argc, char **argv)
{
	/* Register SIGTERM handler */
	signal(SIGKILL, tape_terminate_handler);
	signal(SIGINT, tape_terminate_handler);
	signal(SIGTERM, tape_terminate_handler);

	tape_device_management_init();
	tape_msg_init();

	kill (getppid(), SIGUSR1);

	while (!terminate) {
		tape_receive_poll();
		tape_device_management_poll();
	}
	tape_close_all_devices();
	exit(0);
}

void
panic(char * fmt, ...)
{
	va_list	ap;
	int32	len;
	char	panic_buf[256];

	va_start(ap, fmt);
	len = vsprintf(panic_buf, fmt, ap);
	va_end(ap);

	/* 	tape_close_all_devices(); */

	printf("PANIC: %s\n", panic_buf);
	abort();
}

void 
tape_terminate_handler(int sig)
{
	TP_DBG(2, ("tape_terminate_handler: signal %d\n", sig));
	terminate = 1;
}
