/*
 *------------------------------------------------------------------------------
 *
 * Name:	tape-sm.h
 *
 * RCS:		$Id: tape-sm.h,v 1.17 2002/11/01 19:29:15 raster Exp $
 *
 * Copyright (C) 2001, Agile Storage Inc.
 *
 * Description:	Tape driver device state machines.
 *
 * Created by:	Radek Aster
 *
 * Date Created: 10/1/01
 *
 *------------------------------------------------------------------------------
 */

#ifndef __TAPE_SM_H__
#define __TAPE_SM_H__

typedef enum {
	TAPE_EVENT_OPEN_REQ = 0,
	TAPE_EVENT_SDM_OPEN_DONE,
	TAPE_EVENT_SDM_OPEN_FAILED,
	TAPE_EVENT_SDM_OPEN_ALREADY_OPEN,
	TAPE_EVENT_SDM_INFO_DONE,
	TAPE_EVENT_SDM_INFO_FAILED,
	TAPE_EVENT_SDM_CLOSE_DONE,
	TAPE_EVENT_SDM_CLOSE_FAILED,
	TAPE_EVENT_SDM_RELAY_DONE,
	TAPE_EVENT_SDM_RELAY_FAILED,
	TAPE_EVENT_SDM_RELAY_SCSI_ERROR, // 10
	TAPE_EVENT_SDM_READ_DONE,
	TAPE_EVENT_SDM_READ_FAILED,
	TAPE_EVENT_SDM_READ_SCSI_ERROR,
	TAPE_EVENT_SDM_WRITE_DONE,
	TAPE_EVENT_SDM_WRITE_FAILED,
	TAPE_EVENT_SDM_WRITE_SCSI_ERROR,
	TAPE_EVENT_SDM_DEV_STATUS_DONE,
	TAPE_EVENT_SDM_DEV_STATUS_FAILED,
	TAPE_EVENT_CLOSE_REQ,
	TAPE_EVENT_SERF_CLOSE_REQ, // 20
	TAPE_EVENT_MESSAGE_REQ,
	TAPE_EVENT_DEVICE_REMOVED,
	TAPE_EVENT_GET_STATE_REQ,

	MC_EVENT_CLOSE_REQ,
	MC_EVENT_SERF_CLOSE_REQ,
	MC_EVENT_SDM_OPEN_DONE,
	MC_EVENT_SDM_OPEN_FAILED,
	MC_EVENT_SDM_OPEN_ALREADY_OPEN,
	MC_EVENT_SDM_INFO_DONE,
	MC_EVENT_SDM_INFO_FAILED, // 30
	MC_EVENT_SDM_CLOSE_DONE,
	MC_EVENT_SDM_CLOSE_FAILED,
	
	TAPE_EVENT_COUNT
} tape_dev_event_t;

typedef enum {
	TAPE_STATE_CLOSED = 0,
	TAPE_STATE_SDM_OPEN_PENDING,
	TAPE_STATE_TUR_PENDING,
	TAPE_STATE_LOAD_PENDING,
	TAPE_STATE_MODE_SENSE_COMPRESSION_PENDING,
	TAPE_STATE_MODE_SELECT_COMPRESSION_PENDING,
	TAPE_STATE_MODE_SELECT_DLTEEPROM_PENDING,
	TAPE_STATE_OPEN_ERROR_SDM_CLOSE_PENDING,
	TAPE_STATE_OPEN_ABORTED_CLOSE_PENDING,
	TAPE_STATE_OPEN,
	TAPE_STATE_OPEN_ACTIVE, // 10
	TAPE_STATE_HOLDING_CLOSE,
	TAPE_STATE_READ_POSITION_PENDING,
	TAPE_STATE_REWIND_PENDING,
	TAPE_STATE_UNLOAD_PENDING,
	TAPE_STATE_SDM_CLOSE_PENDING,
	TAPE_STATE_DESTROY_PENDING,
	TAPE_STATE_DESTROY_PENDING_AWAITING_CLOSE,
	TAPE_STATE_SDM_CLOSE_PENDING_DESTROY,
	TAPE_STATE_DESTROY_PENDING_HOLDING_CLOSE,
	TAPE_STATE_DESTROY_OPEN_PENDING, // 20
	TAPE_STATE_DESTROY_OPEN_ABORT_PENDING,
	TAPE_STATE_OPEN_ERROR_SDM_CLOSE_PENDING_DESTROY,
	TAPE_STATE_OPEN_ABORTED_CLOSE_PENDING_DESTROY,
	TAPE_STATE_READ_POSITION_PENDING_HOLDING_CLOSE,
	TAPE_STATE_DESTROY_READ_PENDING_HOLDING_CLOSE,
	TAPE_STATE_DESTROY_CLOSE_PENDING, // 26

	MC_STATE_HOLDING_CLOSE,

	TAPE_STATE_COUNT,

	TAPE_STATE_TERMINAL = -2,
	TAPE_STATE_INVALID = -1,
	TAPE_STATE_INITIAL = TAPE_STATE_CLOSED
} tape_dev_state_t;

typedef struct _tape_msg_context {
	tape_msg_hdr_t *req_msg;
	char 		req_src_app[32];
	uint32		req_src_cpu;
	uint32		req_src_slot;
	uint32		private;
	void (*callback_success)(struct _tape_msg_context *, void *);
	void (*callback_failed)(struct _tape_msg_context *, uint32);
} tape_msg_context_t;

typedef struct _tape_dev_ {
	uint32			poll_ref_number;
	scsi_id_t		id;
	uint64			wwn;
	uint32			lun;
	ushort16		index;
	int32			alias;
	char			vendor_name[TAPE_VENDOR_NAME_LEN];
        char			model_name[TAPE_MODEL_NAME_LEN];
	tape_cap_t *		cap;
	uint32			fixed_blocksize;
	uint32			mc:1,
				write_protected:1,
				bom:1,
				eom:1;

	/* State machine state */
	struct _tape_sm_ *	dev_sm;
	tape_dev_state_t	state;
	uint32			open_handle;
	uint32			open_attributes;
	uint32			open_client_tag;
	tape_msg_context_t *	open_req_context;
	tape_msg_context_t *	close_req_context;
	tape_msg_context_t *	get_state_req_context;
	tape_msg_context_t *	message_req_context;
	void *			pending_req_context;

	/* Tape state from last executed command */
	int32			fileno;
	int32			blockno;

	struct _tape_dev_ *	next;
	struct _tape_dev_ *	prev;
} tape_dev_t;

typedef struct _tape_sm_ {
	tape_dev_state_t		current_state;
	struct {
		tape_dev_event_t	event;
		tape_dev_state_t	next_state;
		void (*callout)(tape_dev_t *, void *);
	} events[TAPE_EVENT_COUNT];
} tape_sm_t;

void tape_sm_create(tape_dev_t *);
void tape_sm_destroy(tape_dev_t *);
void tape_sm_run(tape_dev_t *, void *, tape_dev_event_t);

tape_msg_context_t *tape_save_context(tape_msg_hdr_t *, char *, uint32, uint32);
void tape_free_context(tape_msg_context_t *);

void tape_close_all_devices(void);

#endif /* __TAPE_SM_H__ */
