/*
 *-----------------------------------------------------------------
 *
 * Name:	myesm.c
 *
 * RCS:  $Id: myesm.c,v 1.5 2002/10/22 01:30:42 jong Exp $
 *
 * Copyright (C) 2001, Agile Storage, Inc.
 *
 * Description:	functions for mytest state machine
 *
 * Created by:	Rick Lund
 *
 * Date Created:	5/30/01
 *
 *-----------------------------------------------------------------
 */

#include <nfx-incl.h>
#include <exports/esm-api.h>
#include <exports/queue-api.h>
#include <exports/timer-api.h>
#include "mytest.h"
#include "myesm.h"

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:
 *
 *-----------------------------------------------------------------
 */
int32
mytest_inc_counter(
	uint32	tmr_handle,
	void	* p1,
	void	* p2,
	void	* p3,
	void	* p4
)
{
	mytest_cb_t	* cb = (mytest_cb_t *)p1;

	cb->counter++;
	cb->timeout_remain = 6000 - cb->counter;

	return NFX_OK;

}	/* mytest_inc_counter() */

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:
 *
 *-----------------------------------------------------------------
 */
int32
mytest_tmr_timeout(
	uint32	tmr_handle,
	void	* p1,
	void	* p2,
	void	* p3,
	void	* p4
)
{
	esm_handle_t	handle = (esm_handle_t)(address_t)p2;
    esm_event_t     esmEvent;

    esmEvent.event = MYTEST_EVENT_TIMEOUT;
    esmEvent.p1.u.w = 0;
    esmEvent.p2.u.w = 0;
    if (esm_deliverEvent(handle, &esmEvent) != NFX_OK)
		printf("Error in esm_deliverEvent\n");

	return NFX_OK;

}	/* mytest_tmr_timeout() */

/*-----------------------------------------------------------------
 * Name:	mytest_start_test()
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	5/30/01
 *
 *-----------------------------------------------------------------
 */
int32
mytest_start_test(
	esm_handle_t	handle,
	int32			nextState,
	esm_event_t	    thisEvent,
	void			* cb
)
{
	mytest_cb_t	* mycb = (mytest_cb_t *)cb;

	printf("mytest_start_test\n");

	if ((mycb->tmr_counter_handle = eee_timerCreate(100, EEE_TIMER_REPEAT)) < 0) {
		printf("Error creating counter timer\n");
		return MYTEST_STATE_INIT;
	}

	if ((mycb->tmr_timeout_handle = eee_timerCreate(6000, EEE_TIMER_ONCE)) < 0) {
		eee_timerDestroy(mycb->tmr_counter_handle);
		printf("Error creating timeout timer\n");
		return MYTEST_STATE_INIT;
	}

	if (eee_timerStart(mycb->tmr_counter_handle, mytest_inc_counter, mycb,
					   (void *)(address_t)handle, 0, 0) != NFX_OK) {
		printf("Error starting counter timer\n");
		eee_timerDestroy(mycb->tmr_counter_handle);
		eee_timerDestroy(mycb->tmr_timeout_handle);
		return MYTEST_STATE_INIT;
	}

	if (eee_timerStart(mycb->tmr_timeout_handle, mytest_tmr_timeout, mycb,
					   (void *)(address_t)handle, 0, 0) != NFX_OK) {
		printf("Error starting timeout timer\n");
		eee_timerStop(mycb->tmr_counter_handle);
		eee_timerDestroy(mycb->tmr_counter_handle);
		eee_timerDestroy(mycb->tmr_timeout_handle);
		return MYTEST_STATE_INIT;
	}

	return nextState;

}	/* mytest_start_test() */

/*-----------------------------------------------------------------
 * Name:	mytest_print_stats()
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	5/30/01
 *
 *-----------------------------------------------------------------
 */
int32
mytest_print_stats(
	esm_handle_t	handle,
	int32			nextState,
	esm_event_t     thisEvent,
	void			* cb
)
{
	mytest_cb_t	* mycb = (mytest_cb_t *)cb;

	printf("mytest_print_stats()\n");

	printf("    esm handle = 0x%llx\n", (uint64)((uintptr_t)handle));
	printf("    tmr counter handle = 0x%x\n", mycb->tmr_counter_handle);
	printf("    tmr timeout handle = 0x%x\n", mycb->tmr_timeout_handle);
	printf("    counter = %d\n", mycb->counter);
	printf("    timeout remaining = %d\n", mycb->timeout_remain);

	return nextState;

}	/* mytest_print_stats() */

/*-----------------------------------------------------------------
 * Name:	mytest_clear_stats()
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	5/30/01
 *
 *-----------------------------------------------------------------
 */
int32
mytest_clear_stats(
	esm_handle_t	handle,
	int32			nextState,
	esm_event_t     thisEvent,
	void			* cb
)
{
	mytest_cb_t	* mycb = (mytest_cb_t *)cb;

	printf("mytest_clear_stats()\n");

	printf("    esm handle = 0x%llx\n", (uint64)((uintptr_t)handle));
	mycb->counter = 0;

	return nextState;

}	/* mytest_clear_stats() */

/*-----------------------------------------------------------------
 * Name:	mytest_stop_test()
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	5/30/01
 *
 *-----------------------------------------------------------------
 */
int32
mytest_stop_test(
	esm_handle_t	handle,
	int32			nextState,
	esm_event_t		thisEvent,
	void			* cb
)
{
	mytest_cb_t	* mycb = (mytest_cb_t *)cb;

	printf("mytest_stop_test()\n");

	if (eee_timerStop(mycb->tmr_counter_handle) != NFX_OK)
		printf("Error in stopping counter timer\n");

	if (eee_timerStop(mycb->tmr_timeout_handle) != NFX_OK)
		printf("Error in stopping timeout timer\n");

	eee_timerDestroy(mycb->tmr_counter_handle);
	eee_timerDestroy(mycb->tmr_timeout_handle);

	return nextState;

}	/* mytest_stop_test() */

/*-----------------------------------------------------------------
 * Name:	mytest_timeout()
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	5/30/01
 *
 *-----------------------------------------------------------------
 */
int32
mytest_timeout(
	esm_handle_t	handle,
	int32			nextState,
	esm_event_t		thisEvent,
	void			* cb
)
{
	mytest_cb_t	* mycb = (mytest_cb_t *)cb;

	printf("mytest_timeout()\n");

	if (eee_timerStop(mycb->tmr_counter_handle) != NFX_OK)
		printf("Error in stopping counter timer\n");

	eee_timerDestroy(mycb->tmr_counter_handle);
	eee_timerDestroy(mycb->tmr_timeout_handle);

	return nextState;

}	/* mytest_timeout() */

/*-----------------------------------------------------------------
 * Name:	mytest_error()
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	5/30/01
 *
 *-----------------------------------------------------------------
 */
int32
mytest_error(
	esm_handle_t	handle,
	int32			nextState,
	esm_event_t		thisEvent,
	void			* cb
)
{
	mytest_cb_t	* mycb = (mytest_cb_t *)cb;

	printf("mytest_error()\n");

	printf("    handle = 0x%llx\n", (uint64)((uintptr_t)handle));
	printf("    nextState = %d\n", nextState);
	printf("    thisEvent = %d\n", thisEvent.event);
	printf("    cb = %p\n", mycb);

	return nextState;

}	/* mytest_error() */

/*-----------------------------------------------------------------
 * Name:	mytest_create()
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	5/30/01
 *
 *-----------------------------------------------------------------
 */
int32
mytest_create(
	esm_handle_t	handle,
	int32			nextState,
	esm_event_t		thisEvent,
	void			* cb
)
{
	//mytest_cb_t	* mycb = (mytest_cb_t *)cb;

	printf("mytest_create()\n");

	return nextState;

}	/* mytest_create() */

/*-----------------------------------------------------------------
 * Name:	mytest_bad_state()
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	5/30/01
 *
 *-----------------------------------------------------------------
 */
int32
mytest_bad_state(
	esm_handle_t	handle,
	int32			nextState,
	esm_event_t		thisEvent,
	void			* cb
)
{
	//mytest_cb_t	* mycb = (mytest_cb_t *)cb;

	printf("mytest_bad_state()\n");

	return NFX_ERR;
}	/* mytest_bad_state() */

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:
 *
 *-----------------------------------------------------------------
 */
int32
myesm_deliverStartEvent(
	uint32	handle,
	void	* p1,
	void	* p2
)
{
    esm_event_t esmEvent;

    esmEvent.event = MYTEST_EVENT_START;
    esmEvent.p1.u.p = p1;
    esmEvent.p2.u.p = p2;
    return esm_deliverEvent(handle, &esmEvent);

}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:
 *
 *-----------------------------------------------------------------
 */
int32
myesm_deliverStopEvent(
	uint32	handle,
	void	* p1,
	void	* p2
)
{
    esm_event_t esmEvent;

    esmEvent.event = MYTEST_EVENT_STOP;
    esmEvent.p1.u.p = p1;
    esmEvent.p2.u.p = p2;
    return esm_deliverEvent(handle, &esmEvent);

}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:
 *
 *-----------------------------------------------------------------
 */
int32
myesm_deliverStatsEvent(
	uint32	handle,
	void	* p1,
	void	* p2
)
{
    esm_event_t esmEvent;

    esmEvent.event = MYTEST_EVENT_STATS;
    esmEvent.p1.u.p = p1;
    esmEvent.p2.u.p = p2;
    return esm_deliverEvent(handle, &esmEvent);

}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:
 *
 *-----------------------------------------------------------------
 */
int32
myesm_deliverClearEvent(
	uint32	handle,
	void	* p1,
	void	* p2
)
{
    esm_event_t esmEvent;

    esmEvent.event = MYTEST_EVENT_CLEAR;
    esmEvent.p1.u.p = p1;
    esmEvent.p2.u.p = p2;
    return esm_deliverEvent(handle, &esmEvent);

}

