
#include <nfx-incl.h>
#include <exports/hash-api.h>

#define NUM_BUCKETS	10
int32	ht;

typedef struct {
	uint32	a;
	uint32	b;
	uint32	c;
} test_struct;

/*------------------------------------------------------------------*/
/*------------------------------------------------------------------*/
int32	Alloc(uint32 num_bytes)
{
	int32	t;

	t = eee_ramAlloc(num_bytes);
	printf("MALLOC: 0x%x\n", t);
	return t;
}


void	MyFree(void * t)
{
	printf("FREE:  0x%x\n", t);
	eee_ramDealloc(t);
}

/*------------------------------------------------------------------*/
/*------------------------------------------------------------------*/

uint32 test_hash(uint32 key)
{
	return (key % NUM_BUCKETS);
}

int32 test_compare(test_struct * d1, test_struct * d2)
{
	if ((d1->a == d2->a) &&
		(d1->b == d2->b) &&
		(d1->c == d2->c))
		return 0;
	else
		return 1;
}

void test_release( test_struct * t)
{
	eee_ramDealloc(t);
}

main()
{
	test_struct	* t, * rt;

	ht = hash_create(NUM_BUCKETS, test_hash, test_compare, test_release);

	printf("------------ CREATE -----------\n");
	t = (test_struct *)Alloc(sizeof(test_struct));
	t->a = 4;
	t->b = 5;
	t->c = 6;
	hash_insert(ht, (void *)(t->a), t);

	t = (test_struct *)Alloc(sizeof(test_struct));
	t->a = 7;
	t->b = 8;
	t->c = 9;
	hash_insert(ht, (void *)(t->a), t);

	t = (test_struct *)Alloc(sizeof(test_struct));
	t->a = 10;
	t->b = 11;
	t->c = 12;
	hash_insert(ht, (void *)(t->a), t);

	t = (test_struct *)Alloc(sizeof(test_struct));
	t->a = 13;
	t->b = 14;
	t->c = 15;
	hash_insert(ht, (void *)(t->a), t);

/*------------------------------------------------------------------*/

/*------------------------------------------------------------------*/

	printf("------------ LOOKUP -----------\n");
	t = (test_struct *)Alloc(sizeof(test_struct));
	t->a = 4;
	t->b = 5;
	t->c = 6;
	rt = hash_lookup(ht, (void *)(t->a), t);
	if (rt) {
		printf("found in hash table, rt = 0x%x\n", rt);
	}
	else {
		printf("not found\n");
	}
	MyFree(t);

	t = (test_struct *)Alloc(sizeof(test_struct));
	t->a = 7;
	t->b = 8;
	t->c = 9;
	rt = hash_lookup(ht, (void *)(t->a), t);
	if (rt) {
		printf("found in hash table, rt = 0x%x\n", rt);
	}
	else {
		printf("not found\n");
	}
	MyFree(t);

	t = (test_struct *)Alloc(sizeof(test_struct));
	t->a = 10;
	t->b = 11;
	t->c = 12;
	rt = hash_lookup(ht, (void *)(t->a), t);
	if (rt) {
		printf("found in hash table, rt = 0x%x\n", rt);
	}
	else {
		printf("not found\n");
	}
	MyFree(t);

	t = (test_struct *)Alloc(sizeof(test_struct));
	t->a = 13;
	t->b = 14;
	t->c = 15;
	rt = hash_lookup(ht, (void *)(t->a), t);
	if (rt) {
		printf("found in hash table, rt = 0x%x\n", rt);
	}
	else {
		printf("not found\n");
	}
	MyFree(t);

/*------------------------------------------------------------------*/

	hash_display(ht);

/*------------------------------------------------------------------*/

	printf("------------ DELETE -----------\n");
	t = (test_struct *)Alloc(sizeof(test_struct));
	t->a = 4;
	t->b = 5;
	t->c = 6;
	rt = hash_delete(ht, (void *)(t->a), t);
	if (rt) {
		printf("found in hash table, freeing, 0x%x\n",rt);
		MyFree(rt);
	}
	else {
		printf("not found\n");
	}
	MyFree(t);
	
	printf("------------ LOOKUP -----------\n");
	t = (test_struct *)Alloc(sizeof(test_struct));
	t->a = 4;
	t->b = 5;
	t->c = 6;
	rt = hash_lookup(ht, (void *)(t->a), t);
	if (rt) {
		printf("found in hash table, freeing, 0x%x\n",rt);
	}
	else {
		printf("not found\n");
	}
	MyFree(t);
	
/*------------------------------------------------------------------*/

	hash_display(ht);

/*------------------------------------------------------------------*/

	printf("------------ DESTROY -----------\n");

	hash_destroy(ht);

}

