/*
 *-----------------------------------------------------------------
 *
 * Name:	test.c
 *
 * RCS:  $Id: test.c,v 1.131 2002/12/09 23:55:34 rick Exp $
 *
 * Copyright (C) 2001, Agile Storage, Inc.
 *
 * Description:	main() for code testing
 *
 * Created by:	Rick Lund
 *
 * Date Created:	4/14/01
 *
 *-----------------------------------------------------------------
 */

#include <nfx-incl.h>
#include <exports/eee-api.h>
#ifndef SSC_MGMT
#include <exports/sbm-api.h>
#endif
#include <exports/cm-api.h>
#include <exports/queue-api.h>
#include <exports/timer-api.h>
#include <exports/mem-api.h>
#include <exports/esm-api.h>
#include <exports/crashdump-api.h>
#if defined(EVAL_BOARD) || defined(AGILE_HARDWARE)
#include <exports/hwMap.h>
#include <sys/gt64240r.h>
#endif	/* EVAL_BOARD || AGILE_HARDWARE */
#include <exports/pte-api.h>
#include <exports/except-api.h>
#include "../sm-sbm/sbm.h"
#include "myesm.h"
#include "mytest.h"
#include "../sm-dp-proxy/dp-proxy.h"
#include <exports/seep-api.h>
#include <exports/env-api.h>
#if defined(FCNIM)
#include "../sm-ispfc/ispfc.h"
#endif

extern void do_cmd(uchar8 *);

extern void mallocInit(uchar8 *base, uint32 num_bytes);

extern void do_global_ctors(void);

extern void tsc_init(void);
extern void upgrade_init(void);

extern void serio_init(void);
extern void pte_init(void);
extern void mips_size_cache(void);
extern void except_init(void);
extern void from_init(void);
extern void crashdump_init(void);
extern void spinlock_init_spinlocks(void);
extern void cod_init(void);
extern void pci_init(void);
extern void event_init(void);
extern void file_initModule(void);
extern void q_initQueue(void);
extern void eee_timerInit(void);
extern void timer_ticInit(void);
extern void sysDVTInit(void);
extern void bp_init(void);
extern void rconInit(void);
extern void nvram_init(void);
extern void sdma_init(void);
extern void scsitest_init(void);
extern void evm_initModule(void);
extern void fs_initModule(void);
extern void scsiio_init(void);
extern void ndmp_initModule(void);
extern void am_ag_init(void);
extern void prof_init_common(void);
extern void prof_init(void);
extern void crash_init(void);
extern void histinit(void);
extern void eee_timerInitGMTTime(void);
extern void ssc_stdio_init(void);
extern void stats_initModule(void);
extern void runtime_prompt(void);
extern void bmfpga_nmi_clear(uint32 val);
extern void e_log_get_config(void);

#if defined(FCNIM)
extern void scsi_initModule(void);
#endif


#ifdef LINUX_TEST
#include <sys/time.h>
#endif

extern sbm_config_t     sbm;

extern uchar8			ImageVerString[];

q_headNodePtr_t		q[Q_MAX_QUEUES];

uint32	main_loop_quit = 0;
uchar8	* malloc_base;

uchar8	prompt_string[80] CPU_PRIVATE;
extern int32	histno;


/*------------------------------------------------------------------*/
void
#ifdef LINUX_TEST
main()
#else
_main()
#endif
{
#if defined (SSC)
	uint32	fd;
#endif
	int32	rc;
	uchar8	ipaddr[16];
	uint32	ipaddr_uint;
#ifdef LINUX_TEST
    extern int eee_linuxPollFdMax;
    extern fd_set eee_linuxPollFdset;
    extern void eee_timerLinuxAdjustCurrTicCount();
#endif
	void rconRcv(void);
	uint32	malloc_size;

	serio_init();

#ifdef LARGE_MEMORY
	pte_init();
#endif

#ifndef LINUX_TEST
	mips_size_cache();
#endif

#ifndef SSC
#ifdef EXCEPTIONS_IN_KSEG0
	except_init();
#endif
    from_init();
	crashdump_init();
#ifdef USE_WATCHDOG
	watchdog_init();
#endif
#endif

#ifdef LARGE_MEMORY
    pte_zero_regions();
#endif

#if defined(FCNIM) || defined(NFP_TXRX) || defined(NFP_FP)
	ecc_exception_init();
#endif

	printf("\n\n%s\n\n", ImageVerString);

	cm_init(0);
	
#ifdef NFX_SMP
	spinlock_init_spinlocks();
#endif

#ifdef LINUX_TEST
#define MALLOC_SIZE		(25 * 1024 * 1024)
#else
#ifdef SIBYTE
/*
 * don't fucking touch this!  See Rick if you have a problem with that.
 */
#define MALLOC_SIZE		(64 * 1024 * 1024)
#else
#define MALLOC_SIZE		(10 * 1024 * 1024)
#endif
#endif

	malloc_size = MALLOC_SIZE;

#ifdef LINUX_TEST

#define JUSTIFY			0x100000

	malloc_base = (uchar8 *)malloc(malloc_size+JUSTIFY);
	if (!malloc_base)
		panic("Unable to malloc memory\n");
	malloc_base += JUSTIFY;
	malloc_base = (uchar8 *)((address_t)malloc_base & ~(JUSTIFY-1));
	printf("malloc base at %p\n", malloc_base);

#endif	/* LINUX_TEST */

#if defined(EVAL_BOARD) || defined(AGILE_HARDWARE)
	{
#ifdef LARGE_MEMORY
	malloc_base = (uchar8 *)get_memory_virt_base(MMU_MALLOC_REGION);
	malloc_size = get_memory_region_size(MMU_MALLOC_REGION);
#else
	extern uchar8	_end[];

#define JUSTIFY			0x1000
	malloc_base = _end;
	malloc_base += JUSTIFY-1;
	malloc_base = (uchar8 *)((address_t)malloc_base & ~(JUSTIFY-1));

#ifdef FC_USE_NEW_MEM_MAP
	malloc_base = (uchar8 *)(SW_RAM_BASE_CACHED + KVA_TO_PA(malloc_base));
    printf("memorysize = 0x%llx\n", memorysize);
	if (memorysize_MB == 512)
		malloc_size += 0x1000000;
#endif

#endif
	printf("malloc base at %p\n", malloc_base);
	}
#endif	/* EVAL_BOARD || AGILE_HARDWARE */

	mallocInit(malloc_base, malloc_size);

#if !defined(PROM) && !defined(SSC_MGMT) && !defined(FCNIM_TM)
	do_global_ctors();
#endif

	cm_init(1);

	{
		uint32	seed = _mips_mfc0(C0_COUNT);
		printf("random seed = 0x%x\n", seed);
	    srandom(seed);
	}
//    srandom(1); // TODO - get seed from real-time clock

#ifndef SSC_MGMT
	cod_init();

	pci_init();
#endif

	eee_init(1);

#ifndef FCNIM_TM
	event_init();
#endif

#ifndef SBM_REV2
#ifdef NFX_MOD_NFP
	{
	extern uchar8	* metadata_buffer;

	metadata_buffer = (uchar8 *)eee_allocateBlock(0x2000);
	if (metadata_buffer == NULL)
		printf("ERROR : metadata buffer = 0\n");
	}
#endif
#endif

#if defined(FCNIM)
	ispfc_early_init();
#endif

#ifdef NFP_TXRX
        mbinit();
        pkt_init();
        luc_init();
        tpl_init();
        nfp_init();
#endif

#if defined(NFP_TXRX) || defined(NFP_FP)
	dp_proxy_init(1);
#endif

#ifdef NFP_TXRX
#ifndef SBM_REMOVE_GMC
	/*
	 * Initialize GMC emulation
	 */
	gmc_init(1);
#endif
#endif	/* NFP_TXRX */

#if !defined(SSC_MGMT) && !defined(FCNIM_TM)
	if ((rc = sbm_init(1)) != NFX_OK)
		printf("sbm init error 0x%x\n", rc);
#endif

	eee_init(2);

#if !defined(SSC_MGMT) && !defined(FCNIM_TM)
	if ((rc = sbm_init(2)) != NFX_OK)
		printf("sbm init error 0x%x\n", rc);
#endif

#ifdef NFX_PROFILING
    file_initModule();
#endif

#if defined(NFP_TXRX) || defined(NFP_FP) || defined(FCNIM)
	e_log_init();
#endif

	eee_ipcInit();

	q_initQueue();
	eee_timerInit();
	timer_ticInit();

	eee_initFragmentTimeout();


#if defined(NFP_TXRX) || defined(NFP_FP)
	dp_proxy_init(2);
#endif

#ifndef FCNIM_TM
	sysDVTInit();
#endif

#ifdef USE_TEMP_EXT_MAC_ADDR_SCHEME
#ifndef SSC_MGMT
	if (isFC() || isTXRX() || isFP()) {
		seepInit();
		seepGetIpAddr(ipaddr);
		getIpAddr(ipaddr, &ipaddr_uint);
		seepInitExtMacAddr(ipaddr_uint);
	}
#endif
#endif

#ifdef NFP_TXRX
	sb1250dm_initModule();
    sbm_init(3);
#endif

	esm_initModule();
	
#if defined(EVAL_BOARD) && !defined(SIBYTE)
	// Setup size of galileo slave window.
	// i.e. lets dp83820 get to memory.
	*(uint32 *)PHYS_TO_K1(HW_GALILEO_BASE + PCI_1SCS_0_BANK_SIZE) = 0x01FFF000;
	*(uint32 *)PHYS_TO_K1(HW_GALILEO_BASE + PCI_0SCS_0_BANK_SIZE) = 0x01FFF000;
#endif // EVAL_BOARD && !SIBYTE

#if defined(AGILE_HARDWARE)
	bp_init();
#endif

	rconInit();

#if defined(NFP_TXRX)
	/* luc init need to be moved after bp init
	 * because ipm msg need to be sent to ssc via bp
	 */
	domaininit();
	luc_postInit();
#if defined(NFX_SMP)
	 tpl_ipcInit(); 
#endif /* NFX_SMP */  
#endif /* NFP_TXRX */

#if defined(FCNIM) || defined(FCNIM_TM)
    vs_mem_init(); // vs-mem
    scsi_initModule();
#endif

#if defined(FCNIM) || defined(FCNIM_TM)
    ispfc_pci_init(-1, -1);
    ispfc_init();
#endif

#ifdef NFP_TXRX
#ifndef SBM_REMOVE_GMC
	gmc_init(2);
#endif
#endif	/* NFP_TXRX */

#ifdef NFP_FP
    nvram_init();
#endif

#ifdef NFP_FP
    vs_mem_init(); // vs-mem
	sdma_init();
	scsitest_init();    
	evm_initModule();
    fs_initModule();
	scsiio_init();
	ndmp_initModule();
#endif /* NFP_FP */

#if defined(NFP_TXRX) && !defined(NFX_SMP) 
/* for smp txrx, initilize these on acpu */
//	nfp_initQueues();
    vs_mem_init(); // vs-mem
    lock_init();   // sm-lock
	nfs_init();
	mnt_init();
        nsm_init();
        nlm_init();
#endif
#ifdef	NOTUSED
        mountd_init();
#endif

#ifdef SSC
    vs_mem_init(); // vs-mem
	anp_sscInit();
#endif	/* SSC */

#ifdef LINUX_TEST
#if NFX_SOCKET_BACKPLANE
    eee_linuxSockDrvInit();     // Allow communications over linux eth port.
#endif /* NFX_SOCKET_BACKPLANE */
#endif // LINUX_TEST

#ifdef LINUX_TEST
	eee_registerPollingFunc(EEE_HIGH_PRIORITY,
							eee_timerLinuxAdjustCurrTicCount, NULL);

#ifdef LINUX_POLL_NONBLOCKING_FD
    /* Add stdin to poll indication fd set, so cli will work. */
    eee_addPollIndicationFd(0);
#endif
#endif

#ifdef NFP_FP
	am_ag_init();	// init Agile Mirror agent
#endif
#ifdef NFP_TXRX
    dm_init();      // init datamover
#endif

#ifdef NFX_PROFILING
    /* initialize the system profiler */
	prof_init_common();
    prof_init();
#endif


#if defined(SSC_MGMT) && defined(AGILE_HARDWARE)
	/* Initialize messaging with SSC */
    vs_mem_init(); // vs-mem
	tsc_init();
#endif

	/* Initialize prom upgrade application */
	upgrade_init();

	/* Get current e_log configuration from SSC */
	e_log_get_config();

    /* Initialize crash dump application (fp only) */
#if defined(NFP_FP)
    crash_init();
#endif

	cm_init(2);

	exc_crashdump_store(EXC_CD_SAVED);

	// Initialize command history list.
	histinit();

#if defined(NFP_FP)
	eee_timerInitGMTTime();
#endif

#if !defined(PROM) && !defined(SSC_MGMT) && !defined(FCNIM_TM)
	ssc_stdio_init();
#endif	

#if defined(NFP_TXRX) || defined(NFP_FP) || defined(FCNIM) || defined(SSC_MGMT)
	stats_initModule();
#endif

	cmSetCPUState(CM_SLICE_STATE_RUNTIME);

#ifdef NFX_SMP
	{
		extern void smp_entry();
//		if ((nfxMyCPUId == NFX_NFP_TXRX1) || (nfxMyCPUId == NFX_NFP_FP1))
#if defined(NFP_TXRX)
		if (nfxMyCPUId == NFX_NFP_TXRX1)
			altcpu_start(1, (void *)smp_entry);
#endif
#if defined(NFP_FP)
		if (nfxMyCPUId == NFX_NFP_FP1)
			altcpu_start(1, (void *)smp_entry);
#endif
	}
#endif

	runtime_prompt();

}

/*------------------------------------------------------------------
 *                        Runtime prompt
 *------------------------------------------------------------------*/
void
runtime_prompt(void)
{
	uchar8	cmd[80];
	extern uint32	rcon[];

	sprintf(prompt_string, "%s%s:%d", (rcon[nfxMyCPUId] ? "RCON-" : ""),
							cmGetCpuString(nfxMySlotId, nfxMyCPUId), histno);
	printf("%s > ", prompt_string);

	while (!main_loop_quit) {
		get_cmd(cmd);
		do_cmd(cmd);
		sprintf(prompt_string, "%s%s:%d", (rcon[nfxMyCPUId] ? "RCON-" : ""),
							cmGetCpuString(nfxMySlotId, nfxMyCPUId), histno);
		printf("%s > ", prompt_string);

	}

}

/*------------------------------------------------------------------
 *                        Panic prompt
 *------------------------------------------------------------------*/
void
panic_prompt()
{
	uchar8	cmd[80];
	extern int32	dont_poll;

	dont_poll = 1;

#ifndef SSC_MGMT
    watchdog_disable(); 
    bmfpga_nmi_clear(1);
	if (check_env_autoreboot_flag() == 0) {
		printf("Autoreboot \"off\", stopping in debug mode\n");
	}
    else {
		printf("Autoreboot \"on\", waiting to be reset from ssc-mgmt\n");
        delay_us(1000);
        cmSetCPUState(CM_SLICE_STATE_RESET_WAIT);
    }
#endif

	while (1) {
		sprintf(prompt_string, "DEBUG-%s:%d",
							cmGetCpuString(nfxMySlotId, nfxMyCPUId), histno);
		printf("%s > ", prompt_string);
		get_cmd(cmd);
		do_cmd(cmd);
	}
}

/*------------------------------------------------------------------
 *                        Other stuff
 *------------------------------------------------------------------*/
#ifdef LINUX_TEST

#if !defined(__OpenBSD__)
#include <execinfo.h>

uint32
get_ra()
{
	uint32	ra_list[32], n, i;

	n = backtrace((void *)&ra_list, 3);
/*
	printf("get_ra, n = %d\n", n);
	for (i=0; i<n; i++)
		printf("    0x%x\n", ra_list[i]);
*/

	return (ra_list[2]);

}
#endif
#endif /* LINUX_TEST */

