/*
 *-----------------------------------------------------------------
 *
 * Name:	thread-api.h
 *
 * RCS:  $Id: thread-api.h,v 1.5 2002/10/24 21:12:49 isabellec Exp $
 *
 * Copyright (C) 2001,2002 Agile Storage, Inc.
 *
 * Description:  API for thread library.
 *
 *-----------------------------------------------------------------
 */

#ifndef _THREAD_API_H
#define _THREAD_API_H

#include <exports/esm-api.h>


/**
 * Maximum event queue size for a thread.
 */
enum {
    THREAD_QUEUE_MAX        = 32,
};


/**
 * Library defined thread events.
 */
enum {
    THREAD_EVENT_USER_MAX = NEXT_MSG_BASE,              // User-defined event
                                                        // numbers must be
                                                        // below this value.
    THREAD_EVENT_TIMEOUT = THREAD_EVENT_USER_MAX,       // Timeout expired
                                                        // while in 
                                                        // thread_yield().
    THREAD_EVENT_WAKEUP,                                // Timer expired in
                                                        // thread_sleep().
    THREAD_EVENT_MAX,
};

NFX_ASSERT_STATIC(THREAD_EVENT_MAX < ESM_THREAD_EVENT_USER_MAX);

/**
 * Create a thread and run the specified function
 * in it with the specified argument.  Returns a
 * non-NULL esm_handle_t for the thread on success,
 * NULL otherwise.  A thread remains in existence
 * until it returns or thread_exit() is used.
 */
extern esm_handle_t
thread_create(
    // Opaque userInfo to deliver to the thread.
    void            *userInfo);

/**
 * Start a thread with the function to run in the thread,
 * once a thread is created.
 * Destroy the thread if it failed.
 */ 
int32
thread_start(
    // The handle to the thread
    esm_handle_t * handle,
    // The function to run in the thread.
    void    (*func)(esm_handle_t    handle,     // The thread's esm_handle_t.
                    void            *userInfo));// Opaque userInfo.

/**
 * Yield the cpu back to the executive.
 */
extern void
thread_yield(
	esm_event_t			*eventp,		// Return my waking event info here,
                                        // NULL if I don't care.
	int32				waitSecs);		// Deliver TIMEOUT event if I sleep
                                        // longer than this, 0 to let me sleep
                                        // forever.

/**
 * Sleep the thread for a while, waking it with
 * a WAKEUP event (unless another event is delivered
 * first) when the specified period has elapsed.
 * Returns NFX_ERR if unable to schedule a timer,
 * NFX_OK otherwise.
 */
extern int32
thread_sleep(
    esm_event_t         *eventp,        // Return my waking event info here,
                                        // NULL if I don't care.
    int32               waitMsecs);     // Wake me up after this many
                                        // milliseconds (10 msec granularity).

/**
 * Enqueue an event to a thread.  A thread can have up to
 * THREAD_EVENT_QUEUE_MAX threads awaiting delivery at any
 * given time.  (While multiple enqueued events are supported,
 * generally, a thread will be sleeping waiting for a specific
 * event, thus only a single event would normally be enqueued
 * to a thread at any given time).  Returns NFX_ERR if unable
 * to enqueue the event, otherwise NFX_OK.
 *
 * The value of event.event is user-defined, however, event
 * numbers greater than or equal to THREAD_EVENT_USER_MAX are
 * reserved.
 */
extern int32
thread_event(
    esm_handle_t    handle,
    esm_event_t     *eventp);

extern int32
thread_highPriorityEvent(
    esm_handle_t    handle,
    esm_event_t     *eventp);


/**
 * Perform a long jump of the thread back
 * to the executive.  Use this anywhere in a
 * thread to return to the executive and destroy
 * the thread.  This function does not return.
 */
extern void
thread_exit();


#endif /* _ESM_API_H */
