/* vi:ts=4
 *-----------------------------------------------------------------
 *
 * Name:        tpl-api.h
 *
 * RCS:  $Id: tpl-api.h,v 1.17 2002/08/22 16:58:05 swang Exp $
 *
 * Copyright (C) 2001, Agile Storage Inc.
 *
 * Description: Transport Layer inteface related definitions.
 *
 * Created by:	Shaun Hui
 *
 * Date Created: 07/01/01
 *
 *-----------------------------------------------------------------
 */

#ifndef _TPL_API_H
#define _TPL_API_H

#include <exports/pkt-api.h>
#include <exports/tpl-ipc.h>


#define TPL_PROT_UDP        	17
#define TPL_PROT_TCP        	6

#define	TPL_FREE_PKT			PKT_FREE_PKT
#define	TPL_HOLD_PKT			PKT_HOLD_PKT

#define	TPL_ERR_NO_SHMEM_RSRC	(TPL_ERROR_BASE)
#define	TPL_ERR_NO_BIND_RSRC	(TPL_ERR_NO_SHMEM_RSRC-1)
#define	TPL_ERR_NO_CAM_RSRC		(TPL_ERR_NO_BIND_RSRC-1)
#define	TPL_ERR_NO_PKT_RSRC		(TPL_ERR_NO_CAM_RSRC-1)
#define	TPL_ERR_NO_RSRC			(TPL_ERR_NO_PKT_RSRC-1)
#define	TPL_ERR_DUP_CONN		(TPL_ERR_NO_RSRC-1)
#define	TPL_ERR_BAD_HDL			(TPL_ERR_DUP_CONN-1)
#define	TPL_ERR_BAD_PROT		(TPL_ERR_BAD_HDL-1)
#define	TPL_ERR_BAD_FUNC		(TPL_ERR_BAD_PROT-1)
#define	TPL_ERR_BAD_BIND		(TPL_ERR_BAD_FUNC-1)
#define	TPL_ERR_BAD_CONN		(TPL_ERR_BAD_BIND-1)
#define	TPL_ERR_BAD_EVT			(TPL_ERR_BAD_CONN-1)
#define	TPL_ERR_BAD_PKT			(TPL_ERR_BAD_EVT-1)
#define	TPL_ERR_DUP_BIND		(TPL_ERR_BAD_PKT-1)
#define	TPL_ERR_NOT_BIND		(TPL_ERR_DUP_BIND-1)
#define	TPL_ERR_UL_ADD_FAIL		(TPL_ERR_NOT_BIND-1)
#define	TPL_ERR_UL_DEL_FAIL		(TPL_ERR_UL_ADD_FAIL-1)
#define	TPL_ERR_RING_FULL		(TPL_ERR_UL_DEL_FAIL-1)
#define	TPL_ERR_XMT_FAIL		(TPL_ERR_RING_FULL-1)
#define TPL_ERR_NO_LADDR        (TPL_ERR_XMT_FAIL-1)
#define TPL_ERR_NO_RADDR        (TPL_ERR_NO_LADDR-1)
#define TPL_ERR_NO_LPORT        (TPL_ERR_NO_RADDR-1)
#define TPL_ERR_NO_RPORT        (TPL_ERR_NO_LPORT-1)
#define TPL_ERR_RADDR_IN_SRVR   (TPL_ERR_NO_RPORT-1)
#define TPL_ERR_RPORT_IN_SRVR   (TPL_ERR_RADDR_IN_SRVR-1)
#define TPL_ERR_BAD_MODE        (TPL_ERR_RPORT_IN_SRVR-1)
#define TPL_ERR_BAD_LINK        (TPL_ERR_BAD_MODE-1)
#define TPL_ERR_MAX        		(TPL_ERR_BAD_LINK)



#define TPL_ANY_PORT			0
#define TPL_BIND_SERVER			0x0000
#define TPL_BIND_CLIENT			0x0001		/* Bind client option   */
#define TPL_BIND_PKTCNT_SM		0x0002		/* alloc. small # of pkts */
#define TPL_BIND_USE_RPC		0x0010		/* App. uses RPC header */

#define TPL_FLAG_CLIENT			TPL_BIND_CLIENT
#define TPL_FLAG_USE_RPC		TPL_BIND_USE_RPC
#define TPL_FLAG_PKTCNT_SM		TPL_BIND_PKTCNT_SM

/*------------------------------------
 * TPL Bind Control Block.
 *------------------------------------
 */

typedef struct _tpl_bindCb
{
	uint32		laddr;			/* Local  IP IF address */
	uint32		raddr;			/* Remote IP IF address */
	ushort16	lport;			/* Local  TCP/UDP port */
	ushort16	rport;			/* Remote TCP/UDP port */
	ushort16	tplProt;		/* Protocol UDP/TCP */
	ushort16	bindFlags;		/* Binding flags */
	void		*usrInfo;		/* Usr cookie */
	void		*pBsdSock;		/* ptr to Bsd Sock */
	void		*pConnList;		/* ptr to ConnCb List */
	int32		numConnCb;		/* num Conn Cb assoc. with bindCb */

	/*------------------------------------
	 * Add Connection Upcall.
	 *------------------------------------
	 */
	int32	(*ul_addConnInd) (struct _tpl_bindCb	*pBind,	/* Bind Cb ptr */
							  void 			*pConnId,  	/* Conn ID */	
							  void 			**ppAppHdl, /* App Hdl ptr */
							  uint32 		raddr, 		/* Remote IP */
							  ushort16 		rport);		/* Remote port */
	/*------------------------------------
	 * Del Connection Upcall.
	 *------------------------------------
	 */
	int32		(*ul_delConnInd) (void *pAppHdl, void *pConnId);

	/*-----------------------------------
	 * Bind Request acknowleding upcall
	 *-----------------------------------
	 */
	int32		(*ul_bindReqAck) (void	*pAppHdl, int32 rc,  void *pBind);

	/*------------------------------------
	 * Receive Packet Upcall.
	 *------------------------------------
	 */
	int32		(*ul_rcvPktInd)  (void *pAppHdl, pkt_desc_t *pDesc);

	/*------------------------------------
	 * Transmit Complete Upcall
	 *------------------------------------
	 */
	int32		(*ul_xmtCmpInd)  (void *pAppHdl, pkt_desc_t *pDesc);	

} tpl_bindCb_t;

/*-----------------------------------------------------------------
 * Name:        tpl_init
 *
 * Description: Perform Transport Layer initialization.
 *
 * Created by : Shaun Hui
 *
 * Date Created: 7/01/01
 *
 *-----------------------------------------------------------------
 */
void tpl_init( void );

/*-----------------------------------------------------------------
 * Name:        tpl_bindReq
 *
 * Description: Binds upper layer to Transport layer. Also binds
 *              TPL to BSD layer for slow-path.
 *
 *              The following table shows the fields required  
 *              in the tpl_bindCb for Server or Client bind request.
 *
 *                          SERVER          CLIENT
 *              ------------------------------------
 *              laddr       Required        Required
 *              lport       Required        Required
 *              raddr       No              Required
 *              rport       No              Required
 *              addConnInd  Required        Required
 *              delConnInd  Required        Required
 *              rcvPktInd   Required        Required
 *              xmtCmpInd   Required        Required
 * 
 *              NOTE: laddr, lport, raddr, rport are expected to
 *                    be in NETWORK ORDER (Big Endian)
 *
 * Created by : Shaun Hui
 * 
 * Date Created: 7/01/01
 *-----------------------------------------------------------------
 */
#if defined(NFX_SMP)
int32
tpl_bindReq(tpl_bindCb_t *bindReq, void* handle, ushort16 bindOpt);
#else
int32
tpl_bindReq(tpl_bindCb_t *pReq ,    /* ptr to Bind Req Cb */
            tpl_bindCb_t **pRsp,    /* ptr to Bind Response Cb ptr */
            int32        bindMode); /* TPL_BIND_SERVER or TPL_BIND_CLIENT */
#endif /* NFX_SMP */

/*-----------------------------------------------------------------
 * Name:        tpl_unbindReq
 *
 * Description: Unbinds upper layer from Transport layer.
 *              Deletes all connections associated with bindCB.
 *              Also removes TPL from BSD layer for slow-path.
 *
 * Created by : Shaun Hui
 *
 * Date Created: 12/12/01
 *-----------------------------------------------------------------
 */
int32 /* NFX_OK or TPL error */
tpl_unbindReq( tpl_bindCb_t *pBind /* ptr to CB returned at tpl_bindReq */);

/*-----------------------------------------------------------------
 * Name:        tpl_delConnReq
 *
 * Description: Called by Upper layers (e.g. NFS/Mount/CIFS) to 
 *				delete a connection.
 *
 * Created by : Shaun Hui
 *
 * Date Created: 9/21/01
 *-----------------------------------------------------------------
 */
int32 /* NFX_OK or TPL error */
tpl_delConnReq( void * pConn);

/*-----------------------------------------------------------------
 * Name:        tpl_allocPkt
 *  
 * Description: Allocates pkt from free list associated with connection.
 *  
 * Created by : Shaun Hui
 *
 * Date Created: 7/01/01
 *-----------------------------------------------------------------
 */
#define TPL_PKT_USR		1
#define TPL_PKT_META	2
pkt_desc_t *	/* PKT descriptor or NULL */
tpl_allocPkt(void *pConnId, int32 buffCnt, int type, char *file, int32 linenum);

/*-----------------------------------------------------------------
 * Name:        tpl_freePkt
 *  
 * Description: Free pkt allocated using tpl_allocPkt() back to
 *              free pkt list associated with connection.
 * 
 * Created by : Shaun Hui
 * 
 * Date Created: 7/01/01
 *-----------------------------------------------------------------
 */
int32 			/* NFX_OK or TPL Error */
tpl_freePkt(void *pConnId, pkt_desc_t *pDesc, char *file, int32 linenum);

/*-----------------------------------------------------------------
 * Name:  		tpl_freePktByApp
 *
 * Description: consult application before call tpl_freePkt
 *
 * Created by : Su Wang
 * 
 * Date Created: 1/04/02
 *-----------------------------------------------------------------
 */
int32 		/* NFX_OK or error */
tpl_freePktByApp( void *pConnId, pkt_desc_t *pDesc );

/*-----------------------------------------------------------------
 * Name:        tpl_xmtPkt
 * 
 * Description: "Fast-path" transmit function to be called by
 *              upper layer for sending a packet.
 * 
 * Created by : Shaun Hui
 *
 * Date Created: 7/15/01 
 *-----------------------------------------------------------------
 */
int32 			/* NFX_OK or TPL Error */
tpl_xmtPkt(void *pConnId, pkt_desc_t *pDesc);
#if defined(NFX_SMP)
int32
tpl_xmtPktSlow(uint32 raddr, ushort16 rport, pkt_desc_t	*pDesc, 
			   void	*sock, int owner);
int32 			/* NFX_OK or TPL Error */
tpl_xmtPktCore(void *pConnId, pkt_desc_t *pDesc);
#endif /* NFX_SMP */

/*-----------------------------------------------------------------
 * Name:  		tpl_xmtCmp
 *
 * Description: Transmit complete function called by lower layer (LUC)
 *				after sending a packet.
 *
 * Created by : Shaun Hui
 *
 * Date Created: 7/15/01
 *-----------------------------------------------------------------
 */
int32 		/* NFX_OK or error */
tpl_xmtCmp( void 		*pConnId, pkt_desc_t	*pDesc);


/*-----------------------------------------------------------------
 * Name:        tpl_rcvPkt
 *
 * Description: Receive function to be called by lower layer (LUC)
 *              for receiving a "fast-path" packet.
 *              pHdr should point beyound UDP/TCP header to RPC header.
 * 
 * Created by : Shaun Hui
 * 
 * Date Created: 7/15/01
 *-----------------------------------------------------------------
 */
int32       	/* TPL_FREE_PKT or TPL_HOLD_PKT */
tpl_rcvPkt(void *pConnId, pkt_desc_t *pDesc);


/*-----------------------------------------------------------------
 * Name:        TPL_ALLOC_PACKET
 * 
 * Description: Macro to allocate a packet from the Transport Layer
 * 
 * Created by : Shaun Hui
 *
 * Date Created: 7/15/01 
 *-----------------------------------------------------------------
 */
#define TPL_ALLOC_PACKET(_conn,_usrDataSize ) 		\
						tpl_allocPkt(_conn,_usrDataSize, TPL_PKT_USR, __FILE__,__LINE__)

#define TPL_ALLOC_PACKET_META(_conn,_usrDataSize ) 		\
						tpl_allocPkt(_conn,_usrDataSize, TPL_PKT_META, __FILE__,__LINE__)
/*-----------------------------------------------------------------
 * Name:        TPL_FREE_PACKET
 * 
 * Description: Macro to free a packet that was allocated from the 
 *				Transport Layer
 * 
 * Created by : Shaun Hui
 *
 * Date Created: 7/15/01 
 *-----------------------------------------------------------------
 */
#define TPL_FREE_PACKET(_conn, _pdesc) 	\
						tpl_freePkt(_conn,_pdesc,__FILE__,__LINE__)

#endif
