/* vi:ts=4
 *-----------------------------------------------------------------
 *
 * Name: tpl-init.c
 *
 * RCS:  $Id: tpl-init.c,v 1.7 2002/10/17 01:38:56 shaun Exp $
 *
 * Copyright (C) 2001, Agile Storage, Inc.
 *
 * Description: Transport Layer interface initialization routines.
 *
 * Created by:	Shaun Hui
 *
 * Date Created: 07/01/2001
 *
 *-----------------------------------------------------------------
 */
#include <nfx-incl.h>
#include <exports/malloc-api.h>
#include <exports/pkt-api.h>
#include <exports/tpl-api.h>
#include "tpl.h"

tpl_cfg_t	*pTplCfg;

#ifdef TPL_USE_SHMEM
#define TPL_SHMEM_ALLOC		eee_allocateBlock
#else
#define TPL_SHMEM_ALLOC		eee_ramAlloc
#endif

/*-----------------------------------------------------------------
 * Name: 		tpl_init
 *
 * Description: Perform Transport Layer initialization.
 *
 * Created by : Shaun Hui
 *  
 * Date Created: 7/01/01
 *
 *-----------------------------------------------------------------
 */
void 
tpl_init( void )
{
	tpl_cfg_t	*pCfg;
	int32		allocSize;

	pCfg = (tpl_cfg_t *) TPL_SHMEM_ALLOC( sizeof(tpl_cfg_t) );
	if (pCfg == NULL)
	{
		TPL_LOGERR( TPL_ERR_NO_SHMEM_RSRC, pCfg );
		return;
	}
	memset(pCfg, 0, sizeof (tpl_cfg_t));

	/*-------------------------------
	 * Allocates connection CB table
	 * out of shared mem.
	 *-------------------------------
	 */
	allocSize		  = sizeof( tpl_connCb_t ) * TPL_MAX_CONN_CB;
	pCfg->maxConnCb   = TPL_MAX_CONN_CB;
	pCfg->connTblBase = (tpl_connCb_t *) TPL_SHMEM_ALLOC( allocSize );
	pCfg->connTblEnd  = pCfg->connTblBase + allocSize;
	if (pCfg->connTblBase == NULL)
	{
		TPL_LOGERR( TPL_ERR_NO_SHMEM_RSRC, pCfg );
		return;
	}

	/*-------------------------------
	 * Clear Conn CB table.
	 *-------------------------------
	 */
	memset(pCfg->connTblBase, 0, allocSize);

	/*-------------------------------
	 * Allocates bind CB table
	 * out of shared mem.
	 *-------------------------------
	 */
	allocSize 		  = sizeof( tpl_bindCb_t ) * TPL_MAX_BIND_CB;
	pCfg->maxBindCb   = TPL_MAX_BIND_CB;
	pCfg->bindTblBase = (tpl_bindCb_t *) TPL_SHMEM_ALLOC( allocSize );
	pCfg->bindTblEnd  = pCfg->bindTblBase + allocSize;
	if (pCfg->bindTblBase == NULL) 
	{
		TPL_LOGERR( TPL_ERR_NO_SHMEM_RSRC, pCfg );
		return;
	}

	memset(pCfg->bindTblBase, 0, allocSize);

	pCfg->maxPkt = TPL_MAX_CONN_PKT;

	pTplCfg = pCfg;
}


