/* vi:ts=4
 *-----------------------------------------------------------------
 *
 * Name:        tpl.h
 *
 * RCS:  $Id: tpl.h,v 1.35 2002/10/29 01:21:34 swang Exp $
 *
 * Copyright (C) 2001, Agile Storage Inc.
 *
 * Description: Transport Layer internal data definitions.
 *
 * Created by:	Shaun Hui
 *
 * Date Created: 07/01/01
 *
 *-----------------------------------------------------------------
 */

#ifndef _TPL_H
#define _TPL_H

#define TPL_SIM				1	/* turn on TPL Simulation */
#define TPL_DEBUG			1

#define TPL_LOGERR( code, info )	tpl_logErr( __FILE__, __LINE__, code, info)
#define TPL_PANIC( )				{ *(char *)0 = 1; }

/*------------------------------------
 * Some TPL Max. Needs to be increased
 * in  'real' system.
 *------------------------------------
 */
#define TPL_MAX_CONN_CB		 NFX_MAX_CONN
#define TPL_MAX_BIND_CB		 32	/*  32 Binding CB max. */
#define TPL_MAX_CONN_PKT	 64 /*  64 Packet per TPL Conn (default)*/
#define TPL_MAX_CONN_PKT_SM	  8 /*   8 Packet per TPL Conn (small)*/



/*------------------------------------
 * TPL Connection Control Block.
 *------------------------------------
 */
typedef struct _tpl_connCb
{
	int32			flags;		/* See below */
#define		TPL_CONN_INUSE		0x80000000
#define		TPL_CONN_USE_RPC	0x00000001
#define		TPL_CONN_RPC_REASM	0x00000002
#define		TPL_CONN_DEL_PENDING	0x00000004
#define		TPL_CONN_NOMAC			0x00000008
#define		TPL_CONN_VIF_LA			0x00000010 /* conn has link that is part of vif-la */

#define		TPL_CONN_IPCSTATE_MASK		0x000000ff
#define		TPL_CONN_IPCSTATE_MSHIFT	8
#define		TPL_CONN_IPCSTATE_NO		0 /* no connection */
#define		TPL_CONN_IPCSTATE_ADDING	1 /* connection add message send to ACPU */
#define		TPL_CONN_IPCSTATE_ADDED     2 /* connection add ack rcved from ACPU */
#define		TPL_CONN_IPCSTATE_DELING	3 /* connection delete message send to ACPU */
#define     TPL_CONN_IPCSTATE_DELED		4 /* connection delete ack rcved from ACPU */

	uint32			connId;		/* connection id */
	short16			linkNum;	/* Link # (1 - 4) */
	short16			linkType;	/* Link # (1 - 4) */
	ushort16		srcMac[3];	/* our MAC address */
	ushort16		dstMac[3];	/* dst MAC address */
	ushort16		vlanId;		/* VLAN ID (for vlan I/F) */
	ushort16		prot;		/* UDP/TCP */
	ushort16		camIdx;		/* CAM entry index */
	uint32			laddr;		/* local  IP address */
	uint32			raddr;		/* remote IP address */
	ushort16		lport;		/* local  UDP/TCP port */
	ushort16		rport;		/* remote UDP/TCP port */
	void			*pSock;		/* Bsd socket */
	void			*pInpCb;	/* FreeBSD in_pcb ptr */
	void			*pTcpCb;	/* FreeBSD tcp_cb ptr */
	tpl_bindCb_t	*pBindCb;	/* binding Ctrl Blk ptr */
	pkt_desc_t		*txPktEntry[TPL_MAX_CONN_PKT];
	pkt_descRing_t	txPktRing;
	void			*pAppHdl;	/* application handle */
	pkt_desc_t		*rpcRasmList;	/* RPC Reassembly List */
	pkt_desc_t		*rpcRasmCurr;	/* Curr. pkt in RPC Reassembly List */
	int32			rpcRasmLen;		/* total RPC Reassembly Len */
	int32			rpcRasmCnt;		/* Count # pkt queued up */
	int32			rpcPullUp;
	int32			last_aTime;		/*last access time*/
	pkt_desc_t		*arpTx;			/* su, 05/13/02, arp queue because no mac */
	pkt_desc_t		*ipcRxPkt;	/* to prevent pkt arrives to appl. before add conn msg */
	uint32			cntOutCons;
	struct _tpl_connCb *pNext;	/* next ConnCb of the same "bind" */
} tpl_connCb_t;

#define TPL_IPC_GET_CONNSTATE(_connCb)	\
	(((_connCb)->flags >> TPL_CONN_IPCSTATE_MSHIFT) & TPL_CONN_IPCSTATE_MASK)

#define TPL_IPC_SET_CONNSTATE(_connCb, state)	\
{				\
	(_connCb)->flags &= ~(TPL_CONN_IPCSTATE_MASK << TPL_CONN_IPCSTATE_MSHIFT);	\
	(_connCb)->flags |= (state & TPL_CONN_IPCSTATE_MASK) << TPL_CONN_IPCSTATE_MSHIFT;\
}


/*------------------------------------
 * TPL Statitics counters.
 *------------------------------------
 */
typedef struct
{
	uint32			cntBadBindErr;	/* count invalid Bind CB error */
	uint32			cntBadConnErr;	/* count invalid Conn CB error */
	uint32			cntBadPktErr;	/* count invalid Pkt Desc error */
	uint32			cntBadLinkErr;	/* count invalid Link Id error */
	uint32			cntNoCamRsrc;	/* count no more CAM entry */
	uint32			cntNoPktRsrc;	/* count no more free PKT entry */
	uint32			cntUlAddFail;	/* count Upper layer add fail */
	uint32			cntUlDelFail;	/* count Upper layer delete fail */
	uint32			cntBindReq;		/* count bind req. rcvd */
	uint32			cntUnbindReq;	/* count unbind req. rcvd */
	uint32			cntAddConnInd;	/* count add Conn Ind rcvd */
	uint32			cntDelConnInd;	/* count del Conn Ind rcvd */
	uint32			cntDelConnReq;	/* count del Conn Req rcvd */
	uint32			cntAllocPkt;	/* count tpl pkt alloc */
	uint32			cntFreePkt;		/* count tpl pkt free */
	uint32			cntOutConnAlloc;/* count out of conn. alloc */
	uint32			cntOutHWater;
	uint32			cntRcvPkt;		/* count tpl pkt rcv */
	uint32			cntIpCsumErr;	/* count IP checksum error rcvd*/
	uint32			cntUdpCsumErr;	/* count UDP checksum error rcvd*/
	uint32			cntTcpCsumErr;	/* count TCP checksum error rcvd*/
	uint32			cntXmtPkt;		/* count tpl pkt xmt */
	uint32			cntXmtCmp;		/* count tpl xmt complete*/
	uint32			cntXmtUpCall;	/* count tpl xmt complete Up Calls*/
	uint32			cntXmtDownCall;	/* count tpl pkt send from application */
	uint32			cntXmtFail;		/* count tpl xmt failure*/
	uint32			cntInTcpXmt;	/* count desc with PKT_USR_TCP_TRANSIT set*/
	uint32			cntInTcpTxQ;	/* count desc with PKT_USR_TCP_TX_Q set */
	uint32			cntRefCnt;		/* count desc with non-zero refcnt */
	uint32			cntRpcLenErr;	/* count length error in RPC Marker */
	uint32			cntRpcRasmErr;	/* count length error in RPC Marker */
	uint32			cntRpcSplit1;	
	uint32			cntRpcSplit2;	
	uint32			cntRpcPullup;	
	uint32			cntRpcUpcall1;	
	uint32			cntRpcUpcall2;	
	uint32			cntRpcUpcall3;	
	uint32			cntRpcUpcall4;	
	uint32			cntRasmDepth;
	uint32			cntMaxRpcDepth;
	uint32			cntMaxRasmDepth;
	uint32			cntXmtNoMacErr;	/* application pkt dropped because no mac */
	uint32			cntXmtMacQueue;	/* num. of pkt waiting in mac queue */
	uint32			cntXmtMacUncall;/* mac is available, yet there was no upcall */
	uint32			cntXmtDup;	    /* count dup. pkt send by application */
	uint32			cntXmtNoTplUsr; /* count pkt with PKT_USR_TPL_TX not set */
	uint32			cntXmtTcpDup;   /* tcp attempt to send pkts still in luc */
	uint32			cntXmtTcpErr;   /* fast path tcp send error */
	uint32			cntXmtTcpQueue;	/* num. of pkt in tcp send queue */
	uint32			cntConnLinkChge;/* conn. update due to link change */
	uint32			cntPktLinkChge;	/* pkt update due to link change */
} tpl_stats_t;


/* su, 05/03/02, pending queue when no mac avail */
typedef struct tpl_pend 
{
	struct tpl_pend	*next;
	struct tpl_pend *prev;
	tpl_connCb_t	*pConn;
	int             expire;     /* counter to expire pConn */
} tpl_pend_t;	


/*------------------------------------
 * TPL Configuration Control Block
 *------------------------------------
 */
typedef struct
{
    tpl_bindCb_t    *bindTblBase;	/* bind CB table base */
    tpl_bindCb_t    *bindTblEnd;	/* bind CB table end */
    tpl_connCb_t    *connTblBase;	/* connection CB table base */
    tpl_connCb_t    *connTblEnd;	/* connection CB table end */
    uint32          numConnCb;		/* num of conn. cb in use*/
    uint32          numBindCb;		/* num of bind. cb in use*/
    uint32          maxConnCb;		/* max. num of conn. cb */
    uint32          maxBindCb;		/* max. num of bind. cb */
    uint32          maxPkt;			/* max. num of pkts */
	tpl_stats_t		tplStats;		/* TPL I/F stats */
	int				numPend;	    /* number of current pend pConns */
	tpl_pend_t		*tplPend;	    /* queue for pConn waiting for valid mac */
	int32			tplTimer;	    /* timer handle for purging pending pConn */
	uint32			glConnId;		/* global ConnId */
#if defined(NFX_SMP)
	tpl_ipcq_t	tplIpcQ[TPL_IPC_Q_CNT];	
	uint32		NtoAMsgCnt;		/* cnt of NtoA evnt msgs currently allocated */
	uint32		AtoNMsgCnt;		/* cnt of AtoN evnt msgs currently allocated */
#endif /* NFX_SMP */	
} tpl_cfg_t;


extern tpl_cfg_t   *pTplCfg; 

int tpl_bsdConnInd(void *sock, void *pcb, int evt, void **pHdl);

void tpl_bsdRcvInd( void        *sock, 
                    void        *pDesc,
                    uint32      saddr,
                    ushort16    sport,
                    uint32      len, 
                    void        *hdl );

void tpl_logErr( char *fileName, int32 lineNum, int32 errCode, void *info );
void tpl_showCfg( void );
void tpl_showStats( void );
void tpl_showBind( ushort16 lport );
void tpl_showConn( ushort16 lport );
void tpl_showPkt(  ushort16 lport );

tpl_bindCb_t * tpl_findBindCb( ushort16 prot, ushort16 lport );

int32 tpl_validateConnCb( void *pConn );
int32 tpl_validateBindCb( tpl_bindCb_t *pBind );
ushort16 tpl_cksumIp( tpl_connCb_t *pConn, pkt_desc_t *pDesc);
ushort16 tpl_cksumTpl(tpl_connCb_t *pConn, pkt_desc_t *pDesc);

void tpl_setIpHdr( tpl_connCb_t *pConn, pkt_desc_t *pDesc);
void tpl_setUdpHdr(tpl_connCb_t *pConn, pkt_desc_t *pDesc);
void tpl_setTcpHdr(tpl_connCb_t *pConn, pkt_desc_t *pDesc);
void tpl_connSetupPktHdrs( tpl_connCb_t *pConn, pkt_desc_t *pDesc);

/* su, 05/03, declaration */
int32 tpl_purgeConn(uint32, void*, void*, void*, void*);

#if defined(NFX_SMP)
int32 tpl_delConnReqCore( void *connPtr );
int32 tpl_bindReqCore(tpl_bindCb_t *pReq , 
					  tpl_bindCb_t **pRsp,	
					  int32		 bindOpt);
int32 tpl_unbindReqCore( tpl_bindCb_t *pBind );
#endif /* NFX_SMP */

#define TPL_PRING_CHGE_SRCMAC 0x1
#define TPL_PRING_CHGE_DSTMAC 0x2
void  tpl_UpdatePkt(pkt_desc_t *pDesc, tpl_connCb_t  *pConn, uint32 flags);
#endif
