/*
 *-----------------------------------------------------------------
 *
 * Name:	tlb.c
 *
 * RCS:  $Id: tlb.c,v 1.6 2002/04/29 17:31:30 maximk Exp $
 *
 * Copyright (C) 2001, Agile Storage, Inc.
 *
 * Description:	TLB read/write commands
 *
 * Created by:	Rick Lund
 *
 * Date Created:	12/05/01
 *
 *-----------------------------------------------------------------
 */

#include <nfx-incl.h>
#include <sys/mips.h>

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	12/01/01
 *
 *-----------------------------------------------------------------
 */
void
tlb_display()
{
	uint32	pgmask, mask, pgsize;
	int32	idx, valid, index;
	uint32	valuehi, valuelo, pa;
	int32	low = 0, high = 32;

	for (idx = low ; idx < high; idx++) {

		index = idx << 8;

		__asm__ __volatile__ (
		"     mtc0 %3, $0 \n"
		"     nop \n"
		"     nop \n"
		"     nop \n"
		"     nop \n"
		"     tlbr \n"
		"     nop \n"
		"     nop \n"
		"     nop \n"
		"     nop \n"
		"     nop \n"
		"     mfc0 %0,$10 \n"
		"     mfc0 %1,$2  \n"
		"     mfc0 %2,$5  \n"
		"     nop \n"
		"     nop \n"
		"     nop \n"
		"     nop \n"
		: "=r" (valuehi) , "=r" (valuelo) , "=r" (pgmask)
		: "r" (index)
		);

		mask = pgmask | 0x0fff;
		pa = (valuelo & 0xFFFFF000) << (0);
		valid = (valuelo & TLBLO_V);

		printf("%c%2d: hi=0x%08x", valid ? '*' : ' ', idx, valuehi);

		pgsize = (mask + 1);
		if (pgsize >= 1024*1024)
			printf(" sz=%dM", pgsize/(1024*1024));
		else
			printf(" sz=%dK", pgsize/1024);

		printf(" 0x%08x %c%c%c%c", valuelo,
				valuelo & TLBLO_V ? 'V':'v',
				valuelo & TLBLO_D ? 'D':'d',
				valuelo & TLBLO_G ? 'G':'g',
				valuelo & TLBLO_N ? 'U':'c');
		printf("\n");
	}
}	/* tlb_display() */

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	12/01/01
 *
 *-----------------------------------------------------------------
 */
int32
tlb_set(
	int32	index,
	uint32	va,
	uint32	pa,
	uint32	size,
	uint32	calg
)
{
	uint32	asm_index, mask;
	uint32	asm_src, asm_dest, asm_size;

/*
printf("tlb_set, index = %d, va = %x, pa = %x, size = %x, calg = %x\n",
index, va, pa, size, calg);
*/

	if (size == 0x100000)
		asm_size = TLBMASK_MASK1Mb;
	else if (size == 0x400000)
		asm_size = TLBMASK_MASK4Mb;
	else {
/*
		printf("Invalid TLB size, 0x%x\n", size);
*/
		return 0;
	}
	mask = size-1;
	if (va & mask) {
/*
		printf("Invalid Virtual address for requested entry size, 0x%x\n", va);
*/
		return -1;
	}
	if (pa & mask) {
/*
		printf("Invalid Physical address for requested entry size, 0x%x\n", pa);
*/
		return -1;
	}

	asm_index = index << 8;
	asm_src = (va & TLB_ENTHI_ADDR_MASK);
	asm_dest = ((pa & TLB_ENTLO_ADDR_MASK) >> TLB_ENTLO_ADDR_SHIFT) |
							(TLBLO_G | TLBLO_V | TLBLO_D) | calg;

	__asm__ __volatile__ (
	"     mtc0 %0,$0 \n"
	"     mtc0 %3,$5 \n"
	"     mtc0 %1,$10 \n"
	"     mtc0 %2,$2  \n"
	"     nop \n"
	"     nop \n"
	"     nop \n"
	"     nop \n"
	"     tlbwi \n"
	"     nop \n"
	"     nop \n"
	"     nop \n"
	"     nop \n"
	:
	: "r" (asm_index), "r" (asm_src), "r" (asm_dest), "r" (asm_size)
	);

	return 0;

}	/* tlb_set() */

#if defined(EVAL_BOARD) || defined(AGILE_HARDWARE)
/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	12/01/01
 *
 *-----------------------------------------------------------------
 */
uint32 ssc_memory_base()
{
	return 0x8f000000;
}


uint32 ssc_memory_map()
{
	return 0x40000000;
}


void
mem_map_tlb_init()
{
	int32	index = 0;

	/*
	 * BMFPGA
	 */
	tlb_set(index++, 0x20000000, 0xA0000000, 0x100000, TLBLO_N);
	tlb_set(index++, 0x20100000, 0xA0100000, 0x100000, TLBLO_N);

	/*
	 * 5600s
	 */
	tlb_set(index++, 0x26000000, 0xA6000000, 0x400000, TLBLO_N);
	tlb_set(index++, 0x26400000, 0xA6400000, 0x400000, TLBLO_N);
	tlb_set(index++, 0x26800000, 0xA6800000, 0x400000, TLBLO_N);
	tlb_set(index++, 0x26C00000, 0xA6C00000, 0x400000, TLBLO_N);

	/*
	 * 5632s
	 */
	tlb_set(index++, 0x38000000, 0xB0000000, 0x400000, TLBLO_N);
	tlb_set(index++, 0x38400000, 0xB0400000, 0x400000, TLBLO_N);
	tlb_set(index++, 0x38800000, 0xB0800000, 0x400000, TLBLO_N);
	tlb_set(index++, 0x38C00000, 0xB0C00000, 0x400000, TLBLO_N);
	tlb_set(index++, 0x39000000, 0xB1000000, 0x400000, TLBLO_N);
	tlb_set(index++, 0x39400000, 0xB1400000, 0x400000, TLBLO_N);
	tlb_set(index++, 0x39800000, 0xB1800000, 0x400000, TLBLO_N);
	tlb_set(index++, 0x39C00000, 0xB1C00000, 0x400000, TLBLO_N);
	tlb_set(index++, 0x3A000000, 0xB2000000, 0x400000, TLBLO_N);
	tlb_set(index++, 0x3A400000, 0xB2400000, 0x400000, TLBLO_N);
	tlb_set(index++, 0x3A800000, 0xB2800000, 0x400000, TLBLO_N);
	tlb_set(index++, 0x3AC00000, 0xB2C00000, 0x400000, TLBLO_N);
	tlb_set(index++, 0x3B000000, 0xB3000000, 0x400000, TLBLO_N);
	tlb_set(index++, 0x3B400000, 0xB3400000, 0x400000, TLBLO_N);
	tlb_set(index++, 0x3B800000, 0xB3800000, 0x400000, TLBLO_N);
	tlb_set(index++, 0x3BC00000, 0xB3C00000, 0x400000, TLBLO_N);

	/*
	 * GT registers
	 */
	tlb_set(index++, 0x24000000, 0xa4000000, 0x100000, TLBLO_N);

	/*
	 * GT memory
	 */
	{
		int i;
		for (i = 0; i < 8; ++i) {
			tlb_set(index++, ssc_memory_map() + i * 0x400000,
					ssc_memory_base() + i * 0x400000, 0x400000, TLBLO_N);
		}
	}
}

#endif	/* EVAL_BOARD || AGILE_HARDWARE */

