/* vi:ts=4
 *-----------------------------------------------------------------
 *
 * Name:	commands.c
 *
 * RCS:  $Id: commands.c,v 1.78 2002/12/10 00:05:02 rick Exp $
 *
 * Copyright (C) 2001, Agile Storage, Inc.
 *
 * Description:	UI commands
 *
 * Created by:	Rick Lund
 *
 * Date Created:	6/3/01
 *
 *-----------------------------------------------------------------
 */

#include <nfx-incl.h>
#include <sys/mips.h>
#include <sys/bmfpga.h>
#include <exports/eee-api.h>
#include <exports/cm-api.h>
#include <exports/queue-api.h>
#include <exports/timer-api.h>
#include <exports/esm-api.h>
#include <exports/serio-api.h>
#include <exports/malloc-api.h>
#include <exports/mem-api.h>
#include <exports/pte-api.h>
#include <exports/except-api.h>
#ifndef SSC_MGMT
#include <exports/pci-api.h>
#endif
#ifdef NFX_MOD_NFP
#include <exports/sbm-api.h>
#endif
#ifdef NFP_TXRX
#include "../sm-sbm/sbm.h"
#endif
#include "../sm-test/myesm.h"

#include <exports/eee-debug.h>
#include "../sm-esm/esm.h"
#include "../sm-test/myesm.h"
#if !defined(NFX_MOD_NFP)
#include "../sm-malloc/malloc.h"
#else
#include "../sm-malloc-slab/malloc.h"
#endif

extern int32 mytest_sm_create(esm_handle_t	*handle, void *cb_ptr);
extern int32 mytest_sm_destroy(esm_handle_t	handle);

extern uchar8		ImageVerString[];
q_headNodePtr_t		q[Q_MAX_QUEUES];

extern uint32	main_loop_quit;

#ifdef NFP_TXRX
extern sbm_config_t		sbm;
#endif

#ifdef AGILE_HARDWARE
/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	ericc
 *
 * Date Created:
 *
 *-----------------------------------------------------------------
 */
int32
do_reboot(
	int32	argc,
	char8	* argv[]
)
{
	printf("Rebooting...\n");
    delay_us(1000);

	if (isSSC()) {
		*(volatile char *)0xbf000068 = 0x1;   /*write to RTC (1 second)*/
		*(volatile char *)0xbf000078 = 0xb3;  /*write to RTC*/
        while(1);  /*spin*/
	}
	else if (isFC()) {
        if (nfxMyCPUId == NFX_FCNIM_CTRL_CPU) {     // control cpuid is 3(fc slice 0)
            /* FC0 */
            bmfpgaSetCPUState(	(CM_SLICE_STATE_RUNTIME_REBOOT << 0) |
                                (CM_SLICE_STATE_RUNTIME_REBOOT << 4) |
                                (CM_SLICE_STATE_RUNTIME_REBOOT << 8) |
                                (CM_SLICE_STATE_RUNTIME_REBOOT << 12));
            delay_us(1000);
            *(volatile char *)0xbf4001f4 = 0;   /*Brian's hack*/
        }
        else {
            /* FC1,2,3 */
            printf(" wait for reset from SSC Mgmt ....\n");
            cmSetCPUState(CM_SLICE_STATE_RESET_WAIT);
        }
	}
	else if (isTXRX()) {
		bmfpgaSetCPUState(	(CM_SLICE_STATE_RUNTIME_REBOOT << 0) |
							(CM_SLICE_STATE_RUNTIME_REBOOT << 4) |
							(CM_SLICE_STATE_RUNTIME_REBOOT << 8) |
							(CM_SLICE_STATE_RUNTIME_REBOOT << 12));
		delay_us(1000);
		*(volatile char *)(int)0xbf4000fc = 0;	/* Mehdi's hack*/
	}
    else if (isFP()) {
        printf(" wait for reset from SSC Mgmt ....\n");
        cmSetCPUState(CM_SLICE_STATE_RESET_WAIT);
    }
	else {
		printf("Unsupported on this CPU\n");
        return 0;
    }

	return 0;
}

#endif

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:
 *
 *-----------------------------------------------------------------
 */
int32
do_break(
	int32	argc,
	char8	* argv[]
)
{
	dummy();

	return 0;
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:
 *
 *-----------------------------------------------------------------
 */
int32
do_ver(
	int32	argc,
	char8	* argv[]
)
{
	printf("\n%s\n", ImageVerString);
	printf("%s\n", getPromImageVerString());

	return 0;
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:
 *
 *-----------------------------------------------------------------
 */
int32
do_rq(
	int32	argc,
	char8	* argv[]
)
{
	eee_displayRQ();

	return 0;
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:
 *
 *-----------------------------------------------------------------
 */
int32
do_poll(
	int32	argc,
	char8	* argv[]
)
{
	eee_poll(1);

	return 0;
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:
 *
 *-----------------------------------------------------------------
 */
int32
do_showEEEconfig(
	int32	argc,
	char8	* argv[]
)
{
	if ((argc == 2) && (strcmp(argv[1], "clear") == 0))
		eee_clearFwdRcvStats();
	else
		eee_displayEEEconfig();

	return 0;
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:
 *
 *-----------------------------------------------------------------
 */
int32
do_showEEEError(
	int32	argc,
	char8	* argv[]
)
{
	eee_displayErrorStats();

	return 0;
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:
 *
 *-----------------------------------------------------------------
 */
int32
do_allocateDesc(
	int32	argc,
	char8	* argv[]
)
{
	uint32	pool_id;
	eee_descPtr_t	desc;

	if (argc != 2)
		return -1;

	pool_id = atoi(argv[1]);

	desc = eee_allocateDesc(pool_id, NFX_COS_HIGH);

	printf("alloc descriptor, %p, from pool_id %d\n", desc, pool_id);

	return 0;
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:
 *
 *-----------------------------------------------------------------
 */
int32
do_freeDesc(
	int32	argc,
	char8	* argv[]
)
{
	eee_descPtr_t	desc;

	if (argc != 2)
		return -1;

	desc = (eee_descPtr_t)atolx(argv[1]);

	printf("    freeing descriptor, %p\n", desc);

	eee_deallocateDesc(desc);

	return 0;
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:
 *
 *-----------------------------------------------------------------
 */
int32
do_allocateBuf(
	int32	argc,
	char8	* argv[]
)
{
	uint32	pool_id, size;
	uchar8	* buf;
	uchar8	* phys;

	if (argc != 3)
		return -1;

	pool_id = atoi(argv[1]);
	size = atoi(argv[2]);

	buf = eee_allocateBuffer(pool_id, size, NFX_COS_HIGH);

	printf("alloc %s buffer, %p, from pool_id %d\n",
									((size == 0) ? "Small" :
									((size == 1) ? "Large" :
									((size == 2) ? "Ultra" : "Mgmt"))),
									buf, pool_id);

	printf("EEE_GET_BUF_SIZE = %d\n", eee_getBufSize(eee_getBufType(buf)));
#ifdef LARGE_MEMORY
	printf("virt_to_phys = %p\n",
		   phys = (uchar8 *)virt_to_phys((address_t)buf));
	printf("phys_to_virt = %p\n", (uchar8 *)phys_to_virt((address_t)phys));
#endif
	return 0;
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:
 *
 *-----------------------------------------------------------------
 */
int32
do_freeBuf(
	int32	argc,
	char8	* argv[]
)
{
	uchar8	* buf;

	if (argc != 2)
		return -1;

	buf = (uchar8 *)atolx(argv[1]);

	printf("    freeing buffer, %p\n", eee_getBufBase(buf));

	printf("EEE_GET_BUF_SIZE = %d\n", eee_getBufSize(eee_getBufType(buf)));

	eee_deallocateBuffer(buf);

	return 0;
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:
 *
 *-----------------------------------------------------------------
 */
int32	
do_simple_EEE_test(
	int32	argc,
	char8	* argv[]
)
{
	uchar8	cmd[80];
	uint32	cpu, bufsize, len, slot, plane;
	int32	rc;
	uint32	num_pkts, i;

	if (argc != 1)
		return -1;

	printf("   Enter destination slot : ");
	get_input(cmd);
	slot = atoi(cmd);

	printf("   Enter destination cpu : ");
	get_input(cmd);
	cpu = atoi(cmd);

	printf("   Enter buffer size : ");
	get_input(cmd);
	bufsize = atoi(cmd);

	printf("   Enter number of bytes to send : ");
	get_input(cmd);
	len = atoi(cmd);

	printf("   Enter number of packets to send : ");
	get_input(cmd);
	num_pkts = atoi(cmd);

	printf("   Enter data plane (0 = data, 1 = mgmt, 2 = control, 3 = sac) : ");
	get_input(cmd);
	plane = atoi(cmd);

	printf("Sending %d test packet(s) to slot %d, cpu %d, len %d, bufsize %d, plane %d\n",
									 num_pkts, slot, cpu, len, bufsize, plane);

	for (i=0; i<num_pkts; i++) {
		if ((rc = eee_testCreateSimplePacket(slot, cpu, len, bufsize, plane, i))
														!= NFX_OK) {
			printf("Error returned 0x%x\n", rc);
			break;
		}
	}

	printf("\nNumber packets sent: %d\n\n", i);

	return 0;
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:
 *
 *-----------------------------------------------------------------
 */
int32
do_mgmt_test_stats_print(
	int32	argc,
	char8	* argv[]
)
{
	eee_testDVTPrintStats(EEE_MGMT);

	return 0;
}

int32
do_data_test_stats_print(
	int32	argc,
	char8	* argv[]
)
{
	eee_testDVTPrintStats(EEE_USER_DATA);

	return 0;
}

int32
do_ctrl_test_stats_print(
	int32	argc,
	char8	* argv[]
)
{
	eee_testDVTPrintStats(EEE_CONTROL);

	return 0;
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	12/05/01
 *
 *-----------------------------------------------------------------
 */
int32
do_mgmt_test_stats_clear(
	int32	argc,
	char8	* argv[]
)
{
	eee_testDVTClearStats(EEE_MGMT);
	eee_testDVTClear(EEE_MGMT);

	return 0;
}

int32
do_data_test_stats_clear(
	int32	argc,
	char8	* argv[]
)
{
	eee_testDVTClearStats(EEE_USER_DATA);
	eee_testDVTClear(EEE_USER_DATA);

	return 0;
}

int32
do_ctrl_test_stats_clear(
	int32	argc,
	char8	* argv[]
)
{
	eee_testDVTClearStats(EEE_CONTROL);
	eee_testDVTClear(EEE_CONTROL);

	return 0;
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:
 *
 *-----------------------------------------------------------------
 */
int32
do_mgmt_test_start(
	int32	argc,
	char8	* argv[]
)
{
	uchar8	cmd[80];
	uint32	num_pkts = 8;
	int32	rc;
	int32	spin;

	if (argc == 2)
		spin = atoi(argv[1]);
	else
		spin = 1;

	printf("Enter number of packets [8]: ");
	get_input(cmd);
	if (strlen(cmd)) {
		num_pkts = atoi(cmd);
		num_pkts += 7;
		num_pkts &= ~7;
	}
	
	printf("Starting test with %d packets\n", num_pkts);

	if ((rc = eee_testDVTStart(num_pkts, EEE_MGMT, spin)) != NFX_OK)
		printf("Error returned 0x%x\n", rc);

	return 0;
}

int32
do_data_test_start(
	int32	argc,
	char8	* argv[]
)
{
	uchar8	cmd[80];
	uint32	num_pkts = 8;
	int32	rc;
	int32	spin;

	if (argc == 2)
		spin = atoi(argv[1]);
	else
		spin = 1;

	printf("Enter number of packets [8]: ");
	get_input(cmd);
	if (strlen(cmd)) {
		num_pkts = atoi(cmd);
		num_pkts += 7;
		num_pkts &= ~7;
	}
	
	printf("Starting test with %d packets\n", num_pkts);

	if ((rc = eee_testDVTStart(num_pkts, EEE_USER_DATA, spin)) != NFX_OK)
		printf("Error returned 0x%x\n", rc);

	return 0;
}

int32
do_ctrl_test_start(
	int32	argc,
	char8	* argv[]
)
{
	uchar8	cmd[80];
	uint32	num_pkts = 8;
	int32	rc;
	int32	spin;

	if (argc == 2)
		spin = atoi(argv[1]);
	else
		spin = 1;

	printf("Enter number of packets [8]: ");
	get_input(cmd);
	if (strlen(cmd)) {
		num_pkts = atoi(cmd);
		num_pkts += 7;
		num_pkts &= ~7;
	}
	
	printf("Starting test with %d packets\n", num_pkts);

	if ((rc = eee_testDVTStart(num_pkts, EEE_CONTROL, spin)) != NFX_OK)
		printf("Error returned 0x%x\n", rc);

	return 0;
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:
 *
 *-----------------------------------------------------------------
 */
int32
do_mgmt_test_end(
	int32	argc,
	char8	* argv[]
)
{
	eee_testDVTEnd(EEE_MGMT);

	return 0;
}

int32
do_data_test_end(
	int32	argc,
	char8	* argv[]
)
{
	eee_testDVTEnd(EEE_USER_DATA);

	return 0;
}

int32
do_ctrl_test_end(
	int32	argc,
	char8	* argv[]
)
{
	eee_testDVTEnd(EEE_CONTROL);

	return 0;
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:
 *
 *-----------------------------------------------------------------
 */
int32
do_ipc(
	int32	argc,
	char8	* argv[]
)
{
#if (EEE_NUM_IPC_QUEUES > 0)
	uchar8	cmd[80];
	uint32	dest;
	int32	rc;
	uint32	resp_cmd, resp_msg_size, *resp_msg;
	uint32	msg[80];

	if (argc != 2)
		return -1;

	dest = atoi(argv[1]);

	printf("    Enter message to send : ");
	get_input(cmd);

	if ((rc = eee_sendIPC(dest, IPC_CLIENT_EEE, IPC_ACK_REQ,
							EEE_IPC_CMD_SYNC, strlen(msg), msg, &resp_cmd,
							&resp_msg_size, &resp_msg)) != NFX_OK)
		printf("Error in sendIPC, rc = 0x%x\n", rc);
	else {
		printf("response command = 0x%x, resp_msg_size = 0x%x\n",
												resp_cmd, resp_msg_size);
		printf("response_msg at %p\n", resp_msg);
	}

#endif	/* EEE_NUM_IPC_QUEUES > 0 */
	return 0;
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:
 *
 *-----------------------------------------------------------------
 */
int32
do_ipc_stats(
	int32	argc,
	char8	* argv[]
)
{
#if (EEE_NUM_IPC_QUEUES > 0)
	eee_showIPCStats();

#endif	/* EEE_NUM_IPC_QUEUES > 0 */
	return 0;
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:
 *
 *-----------------------------------------------------------------
 */
int32
do_qstats(
	int32	argc,
	char8	* argv[]
)
{
	q_displayStats();

	return 0;
}

/*-----------------------------------------------------------------
 * Name: do_qcompare
 *
 * Description: This is the queue comparision routine
 *				We enqueued long values and this routine is given
 *				pointers to the type we enqueued.
 *
 * Created by:	Rick Lund
 *
 * Date Created:
 *
 *-----------------------------------------------------------------
 */
static int32
do_qcompare(
	const void *i1,
	const void *i2
)
{
	long item1 = *((long *)i1);
	long item2 = *((long *)i2);

	return item1 - item2;
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:
 *
 *-----------------------------------------------------------------
 */
int32
do_qcreate(
	int32	argc,
	char8	* argv[]
)
{
	uint32	i;

	if (argc != 2)
		return -1;

	i = atoi(argv[1]);

	if ((i < Q_MAX_QUEUES) && (q[i] == 0)) {
		q[i] = q_createQueue(sizeof(long), sizeof(long),
							 do_qcompare, do_qcompare);
		if (q[i] == NULL)
			printf("returned NULL\n");
		else
			printf("q[%d] = %p\n", i, q[i]);
	}
	else
		printf("error\n");

	return 0;
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:
 *
 *-----------------------------------------------------------------
 */
int32
do_qdestroy(
	int32	argc,
	char8	* argv[]
)
{
	uint32	i;

	if (argc != 2)
		return -1;

	i = atoi(argv[1]);

	if (i < Q_MAX_QUEUES) {
		q_destroyQueue(q[i]);
		q[i] = 0;
	}

	return 0;
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:
 *
 *-----------------------------------------------------------------
 */
int32
do_enqueue(
	int32	argc,
	char8	* argv[]
)
{
	uint32	i;
	long	item1, item2;
	int32	rc;

	if (argc != 4)
		return -1;

	i = atoi(argv[1]);
	item1 = atol(argv[2]);
	item2 = atol(argv[3]);

	if ((i < Q_MAX_QUEUES) && (q[i] != 0)) {
		rc = q_enQueue(q[i], (void *)&item1, (void *)&item2);
		if (rc != NFX_OK)
			printf("returned 0x%x\n", rc);
	}
	else
		printf("error\n");

	return 0;
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:
 *
 *-----------------------------------------------------------------
 */
int32
do_dequeue(
	int32	argc,
	char8	* argv[]
)
{
	uint32	i;
	long	item1, item2;
	int32	rc;

	if (argc != 2)
		return -1;

	i = atoi(argv[1]);

	if ((i < Q_MAX_QUEUES) && (q[i] != 0)) {
		rc = q_deQueue(q[i], (void *)&item1, (void *)&item2);
		if (rc != NFX_OK)
			printf("returned 0x%x\n", rc);
		else
			printf("item1 = 0x%lx, item2 = 0x%lx\n", item1, item2);
	}
	else
		printf("error\n");
	
	return 0;
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:
 *
 *-----------------------------------------------------------------
 */
int32
do_qreset(
	int32	argc,
	char8	* argv[]
)
{
	uint32	i;
	int32	rc;

	if (argc != 2)
		return -1;

	i = atoi(argv[1]);

	if ((i < Q_MAX_QUEUES) && (q[i] != 0)) {
		rc = q_resetReadPointer(q[i]);
		if (rc != NFX_OK)
			printf("returned 0x%x\n", rc);
	}
	else
		printf("error\n");

	return 0;
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:
 *
 *-----------------------------------------------------------------
 */
int32
do_qreadadvance(
	int32	argc,
	char8	* argv[]
)
{
	uint32	i;
	long	item1, item2;
	int32	rc;

	if (argc != 2)
		return -1;

	i = atoi(argv[1]);

	if ((i < Q_MAX_QUEUES) && (q[i] != 0)) {
		rc = q_readNextQueueItem(q[i], (void *)&item1, (void *)&item2);
		if (rc != NFX_OK)
			printf("returned 0x%x\n", rc);
		else
			printf("item1 = %lx, item2 = %lx\n", item1, item2);
	}
	else
		printf("error\n");

	return 0;
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:
 *
 *-----------------------------------------------------------------
 */
int32
do_qread(
	int32	argc,
	char8	* argv[]
)
{
	uint32	i;
	long	item1, item2;
	int32	rc;

	if (argc != 2)
		return -1;

	i = atoi(argv[1]);

	if ((i < Q_MAX_QUEUES) && (q[i] != 0)) {
		rc = q_readQueueItem(q[i], (void *)&item1, (void *)&item2);
		if (rc != NFX_OK)
			printf("returned 0x%x\n", rc);
		else
			printf("item1 = %lx, item2 = %lx\n", item1, item2);
	}
	else
		printf("error\n");

	return 0;
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:
 *
 *-----------------------------------------------------------------
 */
int32
do_qdelete(
	int32	argc,
	char8	* argv[]
)
{
	uint32	i;
	long	item1, item2;
	int32	rc;

	if (argc != 2)
		return -1;

	i = atoi(argv[1]);

	if ((i < Q_MAX_QUEUES) && (q[i] != 0)) {
		rc = q_deleteQueueItem(q[i], (void *)&item1, (void *)&item2);
		if (rc != NFX_OK)
			printf("returned 0x%x\n", rc);
		else
			printf("item1 = %lx, item2 = %lx\n", item1, item2);
	}
	else
		printf("error\n");

	return 0;
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:
 *
 *-----------------------------------------------------------------
 */
int32
do_qchange(
	int32	argc,
	char8	* argv[]
)
{
	uint32	i;
	long	item1, item2;
	int32	rc;

	if (argc != 4)
		return -1;

	i = atoi(argv[1]);
	item1 = atol(argv[2]);
	item2 = atol(argv[3]);

	if ((i < Q_MAX_QUEUES) && (q[i] != 0)) {
		rc = q_changeQueueItem(q[i], (void *)&item1, (void *)&item2);
		if (rc != NFX_OK)
			printf("returned 0x%x\n", rc);
	}
	else
		printf("error\n");

	return 0;
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:
 *
 *-----------------------------------------------------------------
 */
int32
do_qinsertbefore(
	int32	argc,
	char8	* argv[]
)
{
	uint32	i;
	long	item1, item2;
	int32	rc;

	if (argc != 4)
		return -1;

	i = atoi(argv[1]);
	item1 = atol(argv[2]);
	item2 = atol(argv[3]);

	if ((i < Q_MAX_QUEUES) && (q[i] != 0)) {
		rc = q_insertBeforeItem(q[i], (void *)&item1, (void *)&item2);
		if (rc != NFX_OK)
			printf("returned 0x%x\n", rc);
	}
	else
		printf("error\n");

	return 0;
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:
 *
 *-----------------------------------------------------------------
 */
int32
do_qinsertafter(
	int32	argc,
	char8	* argv[]
)
{
	uint32	i;
	long	item1, item2;
	int32	rc;

	if (argc != 4)
		return -1;

	i = atoi(argv[1]);
	item1 = atol(argv[2]);
	item2 = atol(argv[3]);

	if ((i < Q_MAX_QUEUES) && (q[i] != 0)) {
		rc = q_insertAfterItem(q[i], (void *)&item1, (void *)&item2);
		if (rc != NFX_OK)
			printf("returned 0x%x\n", rc);
	}
	else
		printf("error\n");

	return 0;
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:
 *
 *-----------------------------------------------------------------
 */
int32
do_qinsertorder(
	int32	argc,
	char8	* argv[]
)
{
	uint32	i;
	long	item1, item2;
	int32	rc;

	if (argc != 4)
		return -1;

	i = atoi(argv[1]);
	item1 = atol(argv[2]);
	item2 = atol(argv[3]);

	if ((i < Q_MAX_QUEUES) && (q[i] != 0)) {
		rc = q_insertItemInOrder(q[i], (void *)&item1, (void *)&item2,
							Q_FIND_ONE | Q_INSERT_ORDER_ASCENDING);
		if (rc != NFX_OK)
			printf("returned 0x%x\n", rc);
	}
	else
		printf("error\n");

	return 0;
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:
 *
 *-----------------------------------------------------------------
 */
int32
do_qfind(
	int32	argc,
	char8	* argv[]
)
{
	uint32	i;
	long	item1, item2;
	int32	rc;

	if (argc != 3)
		return -1;

	i = atoi(argv[1]);
	item1 = atol(argv[2]);

	if ((i < Q_MAX_QUEUES) && (q[i] != 0)) {
		rc = q_findQueueItem(q[i], (void *)&item1, (void *)&item2, Q_FIND_ONE);
		if (rc != NFX_OK)
			printf("returned 0x%x\n", rc);
		printf("item2 = 0x%lx\n", item2);
	}
	else
		printf("error\n");

	return 0;
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:
 *
 *-----------------------------------------------------------------
 */
int32
timer_test_callback(uint32 handle, void * p1, void * p2, void * p3, void * p4)
{
	eeeRTC_t	now;
	eee_rtc(&now);

	printf("\ntimer callback function, handle = %d, time = %lld", handle, now.val);
/*
	printf("                p1 = %d\n", (uint32)p1);
	printf("                p2 = %d\n", (uint32)p2);
	printf("                p3 = %d\n", (uint32)p3);
	printf("                p4 = %d\n", (uint32)p4);
	printf("\n");
*/
	return NFX_OK;
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:
 *
 *-----------------------------------------------------------------
 */
int32
do_timercreate(
	int32	argc,
	char8	* argv[]
)
{
	uint32	duration, type;
	int32	rc;
	uchar8	cmd[80];

	if (argc != 1)
		return -1;

	printf("Enter duration (in ms) : ");
	get_input(cmd);
	duration = atoi(cmd);

	printf("Enter type (1 = ONCE, 2 = REPEAT) : ");
	get_input(cmd);
	type = atoi(cmd);

	rc = eee_timerCreate(duration, type);
	if (rc == TIMER_ERR_NO_TIMER_AVAILABLE)
		printf("error\n");
	else
		printf("new timer handle = %d\n", rc);

	return 0;
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:
 *
 *-----------------------------------------------------------------
 */
int32
do_timerstart(
	int32	argc,
	char8	* argv[]
)
{
	uint32	handle;
	address_t p1, p2, p3, p4;
	int32	rc;
	uchar8	cmd[80];

	if (argc != 2)
		return -1;

	handle = atoi(argv[1]);

	printf("Enter param 1 : ");
	get_input(cmd);
	p1 = atoi(cmd);
	printf("Enter param 2 : ");
	get_input(cmd);
	p2 = atoi(cmd);
	printf("Enter param 3 : ");
	get_input(cmd);
	p3 = atoi(cmd);
	printf("Enter param 4 : ");
	get_input(cmd);
	p4 = atoi(cmd);

	rc = eee_timerStart(handle, timer_test_callback, (void *)p1, (void *)p2, (void *)p3, (void *)p4);
	if (rc != NFX_OK)
		printf("error, rc = 0x%x\n", rc);

	return 0;
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:
 *
 *-----------------------------------------------------------------
 */
int32
do_timerstop(
	int32	argc,
	char8	* argv[]
)
{
	uint32	handle;
	int32	rc;

	if (argc != 2)
		return -1;

	handle = atoi(argv[1]);

	rc = eee_timerStop(handle);
	if (rc != NFX_OK)
		printf("error, rc = 0x%x\n", rc);

	return 0;
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:
 *
 *-----------------------------------------------------------------
 */
int32
do_timerdestroy(
	int32	argc,
	char8	* argv[]
)
{
	uint32	handle;
	int32	rc;

	if (argc != 2)
		return -1;

	handle = atoi(argv[1]);

	rc = eee_timerDestroy(handle);
	if (rc != NFX_OK)
		printf("error, rc = 0x%x\n", rc);

	return 0;
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:
 *
 *-----------------------------------------------------------------
 */
int32
do_timerstats(
	int32	argc,
	char8	* argv[]
)
{
	timer_showStats();

	return 0;
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:
 *
 *-----------------------------------------------------------------
 */
int32
do_showtic(
	int32	argc,
	char8	* argv[]
)
{
	eee_timerShowTic();

	return 0;
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Jan Vanhoutte
 *
 * Date Created:
 *
 *-----------------------------------------------------------------
 */

int32
do_showperf(
	int32	argc,
	char8	* argv[]
    )
{
    int32 clear = FALSE;
    
    if (argc > 1)
    {
        if (strncmp(argv[1], "-c", 2) == 0)
        {
            clear = TRUE;
        }
    }
    
	eee_showPerf(clear);

	return 0;
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:
 *
 *-----------------------------------------------------------------
 */
int32
do_esmcreate(
	int32	argc,
	char8	* argv[]
)
{
	esm_handle_t	handle;
	int32			rc;
	mytest_cb_t		* cb;

	cb = eee_ramAlloc(sizeof(mytest_cb_t));

	rc = mytest_sm_create(&handle, cb);

	printf("returned %d, cb at %p, handle = 0x%x\n", rc, cb, handle);

	if (rc < 0)
		eee_ramDealloc(cb);

	return 0;
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:
 *
 *-----------------------------------------------------------------
 */
int32
do_esmdestroy(
	int32	argc,
	char8	* argv[]
)
{
	uint32	handle;
	int32	rc;

	if (argc != 2)
		return -1;

	handle = atox(argv[1]);

	rc = mytest_sm_destroy(handle);
	if (rc != NFX_OK)
		printf("error in sm_destroy, rc = 0x%x\n", rc);

	return 0;
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:
 *
 *-----------------------------------------------------------------
 */
int32
do_esmstart(
	int32	argc,
	char8	* argv[]
)
{
	uint32	handle;

	if (argc != 2)
		return -1;

	handle = atox(argv[1]);

	if (myesm_deliverStartEvent(handle, 0, 0) != NFX_OK)
		printf("Error in event delivery\n");

	return 0;
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:
 *
 *-----------------------------------------------------------------
 */
int32
do_esmstop(
	int32	argc,
	char8	* argv[]
)
{
	uint32	handle;

	if (argc != 2)
		return -1;

	handle = atox(argv[1]);

	if (myesm_deliverStopEvent(handle, 0, 0) != NFX_OK)
		printf("Error in event delivery\n");

	return 0;
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:
 *
 *-----------------------------------------------------------------
 */
int32
do_esmstats(
	int32	argc,
	char8	* argv[]
)
{
	uint32	handle;

	if (argc != 2)
		return -1;

	handle = atox(argv[1]);

	if (myesm_deliverStatsEvent(handle, 0, 0) != NFX_OK)
		printf("Error in event delivery\n");

	return 0;
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:
 *
 *-----------------------------------------------------------------
 */
int32
do_esmclear(
	int32	argc,
	char8	* argv[]
)
{
	uint32	handle;

	if (argc != 2)
		return -1;

	handle = atox(argv[1]);

	if (myesm_deliverClearEvent(handle, 0, 0) != NFX_OK)
		printf("Error in event delivery\n");

	return 0;
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:
 *
 *-----------------------------------------------------------------
 */
int32
do_mmstats(
	int32	argc,
	char8	* argv[]
)
{
	malloc_showStats();

	return 0;
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:
 *
 *-----------------------------------------------------------------
 */
int32
do_malloc(
	int32	argc,
	char8	* argv[]
)
{
	uint32	num_bytes;
	uchar8	* ptr;

	if (argc != 2)
		return -1;

	num_bytes = atoi(argv[1]);
	ptr = eee_ramAlloc(num_bytes);

	printf("MALLOC : ptr = %p\n", ptr);

	return 0;
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:
 *
 *-----------------------------------------------------------------
 */
int32
do_free(
	int32	argc,
	char8	* argv[]
)
{
	uchar8	* ptr;

	if (argc != 2)
		return -1;

	ptr = (uchar8 *)atolx(argv[1]);

	printf("freeing buffer %p\n", ptr);

	eee_ramDealloc(ptr);

	return 0;
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:
 *
 *-----------------------------------------------------------------
 */
int32
do_mmfreelist(
	int32	argc,
	char8	* argv[]
)
{
	malloc_showFreeList();

	return 0;
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:
 *
 *-----------------------------------------------------------------
 */
int32
do_mmalloclist(
	int32	argc,
	char8	* argv[]
)
{
	malloc_showAllocList();

	return 0;
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:
 *
 *-----------------------------------------------------------------
 */
int32
do_mmfreestack(
	int32	argc,
	char8	* argv[]
)
{
	malloc_showFreeNodeStack();

	return 0;
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:
 *
 *-----------------------------------------------------------------
 */
int32
do_mmallocstack(
	int32	argc,
	char8	* argv[]
)
{
	malloc_showAllocNodeStack();

	return 0;
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:
 *
 *-----------------------------------------------------------------
 */
int32
do_mmallocrange(
	int32	argc,
	char8	* argv[]
)
{
	malloc_showAllocRanges();

	return 0;
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Jan Vanhoutte
 *
 * Date Created:
 *
 *-----------------------------------------------------------------
 */

int32
do_showHistogram(
	int32	argc,
	char8	* argv[]
    )
{
    int mode = 0, output = 0, type = 0, nbr = 0;
	uchar8	cmd[80];

    if (argc > 1)
    {
        if (strncmp(argv[1], "s", 1) == 0)
        {
            mode = 0;
            type = 0;
        }
        else
        if (strncmp(argv[1], "m", 1) == 0)
        {
            mode = 1;
            type = 0;
        }
        else
        if (strncmp(argv[1], "c", 1) == 0)
        {
            mode = 1;
            type = 1;
        }
        else
        if (strncmp(argv[1], "t", 1) == 0)
        {
            mode = 1;
            type = 2;
        }
        else
        {
            printf("Usage: mallochist [s|m|c|t|help] [nbr]\n");
            printf("With: h[elp] : help\n");
            printf("      s|m|c|t : sort by size | max | current | total\n");
            printf("      nbr: the max number to display\n");
            return 0;
        }
    }
    
    if (argc > 2)
    {
        nbr = atoi(argv[2]);
    }
    

	// printf("Full description <enter>, or short notation (s) : ");
	// get_input(cmd);
    if (cmd[0] == 's')
    {
        output = 1;
    }
    output = 0;
    
    if (mode == 1)
    {
        // printf("Show only top N lines {default= 100) : ");
        if (nbr <= 5)
            nbr = 5;
        if (nbr > 100)
            nbr = 100;
    } 
    else
    {
        // printf("Show only if counter if greater than (default 0) : ");
        if (nbr < 0)
            nbr = 0;
    }

/*
    if (mode)
    {
        if (type == 1)
            printf ("Sort by current usage\n");
        else
            if (type == 2)
                printf ("Sort by total usage\n");
            else
                printf ("Sort by max usage\n");
    }
    else
    {
        printf ("Sort by malloc size\n");
    }
*/

    malloc_showHistogram (mode, output, type, nbr);

	return 0;
}

#ifdef EVAL_BOARD
/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:
 *
 *-----------------------------------------------------------------
 */
int32
do_led(
	int32	argc,
	char8	* argv[]
)
{
	uint32	num;
	uint32	state;
	uchar8	* led_ptr;

	if (argc != 3)
		return -1;

	if (strcmp(argv[2], "on") == 0)
		led_ptr = (uchar8 *)0xBC808000;
	else if (strcmp(argv[2], "off") == 0)
		led_ptr = (uchar8 *)0xBC814000;
	else {
		printf("Invalid state, %s\n", argv[2]);
		return -1;
	}

	num = atoi(argv[1]);
	if (num > 2) {
		printf("Invalid led, %d\n", num);
		return -1;
	}

	led_ptr += (num * 0x4000);

	printf("writing 0 to 0x%x\n", led_ptr);

	*led_ptr = 0;

	return 0;
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:
 *
 *-----------------------------------------------------------------
 */
int32
do_baud(
	int32	argc,
	char8	* argv[]
)
{
	uint32	baud;

	if (argc != 2)
		return -1;

	baud = atoi(argv[1]);

#ifdef SIBYTE
	sb1250_uart(OP_BAUD, 0, baud);
	sb1250_uart(OP_BAUD, 1, baud);
#else
	st16552(OP_BAUD, 0, baud);
	st16552(OP_BAUD, 1, baud);
#endif
/*
	else {
		printf("Unsupported baud rate, %d\n", baud);
		return -1;
	}
*/


	return 0;
}
#endif	/* EVAL_BOARD */

#if defined(FCNIM) || defined(NFP_TXRX)
/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:
 *
 *-----------------------------------------------------------------
 */
int32
do_pci_read(
	int32	argc,
	char8	* argv[]
)
{
	uint32	bus, dev, func, off;
	uint32	i, data;
	int32	rc;

	if ((argc != 4) && (argc != 5))
		return -1;

	bus = atoi(argv[1]);
	dev = atoi(argv[2]);
	func = atoi(argv[3]);
	if (argc == 5)
		off = atox(argv[4]);
	else
		off = 0xFF;

	if (off == 0xFF) {
		for (i=0; i<0x40; i+=4) {
			if ((rc = pci_cfg_read(bus, dev, func, i, &data)) != NFX_OK)
				printf("pci_cfg_read returned error, 0x%x\n", rc);
			else
				printf("pci reg 0x%02x = 0x%x\n", i, data);
		}
	}
	else {
		if ((rc = pci_cfg_read(bus, dev, func, off, &data)) != NFX_OK)
			printf("pci_cfg_read returned error, 0x%x\n", rc);
		else
			printf("pci reg = 0x%x\n", data);
	}

	return 0;
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:
 *
 *-----------------------------------------------------------------
 */
int32
do_pci_write(
	int32	argc,
	char8	* argv[]
)
{
	uint32	bus, dev, func, reg, data;
	int32	rc;

	if (argc != 6)
		return -1;

	bus = atoi(argv[1]);
	dev = atoi(argv[2]);
	func = atoi(argv[3]);
	reg = atoi(argv[4]);
	data = atox(argv[5]);

	if ((rc = pci_cfg_write(bus, dev, func, reg, data)) != NFX_OK)
		printf("pci_cfg_write returned error, 0x%x\n", rc);

	return 0;
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:
 *
 *-----------------------------------------------------------------
 */
int32
do_pci_clear(
	int32	argc,
	char8	* argv[]
)
{
	uint32	bus, dev;

	if (argc != 3)
		return -1;

	bus = atoi(argv[1]);
	dev = atoi(argv[2]);

	pci_cfg_clear_status(bus, dev);

	return 0;
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:
 *
 *-----------------------------------------------------------------
 */
int32
do_pci_stat(
	int32	argc,
	char8	* argv[]
)
{
	uint32	bus, dev;
	int32	rc;

	if (argc != 3)
		return -1;

	bus = atoi(argv[1]);
	dev = atoi(argv[2]);

	if ((rc = pci_cfg_check_status(bus, dev)) != NFX_OK)
		printf("ERROR : rc = 0x%x\n", rc);
	else
		printf("no error status bits\n");

	return 0;
}

#endif	/* EVAL_BOARD */

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:
 *
 *-----------------------------------------------------------------
 */
int32
do_send_message(
	int32	argc,
	char8	* argv[]
)
{
	uint32	slot, cpu, num_bytes;
	int32	rc;

	if (argc != 4)
		return -1;

	slot = atoi(argv[1]);
	cpu = atoi(argv[2]);
	num_bytes = atoi(argv[3]);

	rc = eee_sendMessageTest(slot, cpu, num_bytes);
	if (rc != EEE_FORWARDED_PACKET) {
		printf("Error sending message, slot %d, cpu %d, num_bytes = %d\n",
														slot, cpu, num_bytes);
	}
	else
		printf("sent %d bytes to slot %d, cpu %d\n", num_bytes, slot, cpu);

	return 0;
}

#ifndef SBM_REV2
/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:
 *
 *-----------------------------------------------------------------
 */
nfx_cod_t
nfx_cod_t get_cod(uchar8 *l)
{
	nfx_cod_t	cod;

	get_input(l);
/*
	printf("line = %s\n", l);
*/
	atollx(&cod, l);
/*
	printf("get_cmd, cod = %llx\n", cod);
*/

	return cod;
}

#ifdef NFX_MOD_NFP
/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:
 *
 *-----------------------------------------------------------------
 */
test_transmit_sac_data(snim)
{
	uchar8						line[80];
	uchar8						buf[128];
	nfx_cod_t					cod;
	uint32						length, offset;
	uint32						slot, cpu, app_id, test_app_id;
	sbm_sendControlAndSACData_t	* msg;
	int32						rc;
	uint32						num_bd;
	uint32						dp;

	printf("Enter COD handle : ");
	cod = get_cod(line);

	printf("Enter length : ");
	get_input(line);
	length = atoi(line);

	printf("Enter offset : ");
	get_input(line);
	offset = atoi(line);

	if (snim) {
		printf("Enter destination slot : ");
		get_input(line);
		slot = atoi(line);

		printf("Enter destination CPU : ");
		get_input(line);
		cpu = atoi(line);
	}
	else {
		slot = nfxMySlotId;
		cpu = NFX_NFP_TXRX1;
	}

	printf("Enter control message : ");
	get_input(line);

	printf("transmit sac data to %s, COD 0x%llx, length %d, offset %d\n",
								(snim ? "SNIM" : "LUC"), cod, length, offset);

	app_id = eee_getApplicationId("sbm");
	test_app_id = eee_getApplicationId("gmc_test_rcv");

	msg = (sbm_sendControlAndSACData_t *)buf;

	if (snim)
		msg->cmd = CMD_TO_MCPU_SEND_CONTROL_AND_DATA_TO_SNIM;
	else
		msg->cmd = CMD_TO_MCPU_SEND_DATA_TO_LUC;

	msg->dest_port = PACK_UNICAST_PORT_ADDRESS(
										slot,
										cpu,
										test_app_id,
										EEE_USER_DATA,
										NFX_COS_HIGH,
										0);

	msg->source_port = PACK_UNICAST_PORT_ADDRESS(
										nfxMySlotId,
										nfxMyCPUId,
										test_app_id,
										EEE_USER_DATA,
										NFX_COS_HIGH,
										0);
	msg->TID = 0x65656565;
	msg->msg_len = strlen(line);
	msg->COD = cod;
	msg->length = length;
	msg->offset = offset;
	memcpy(&(msg->msg), line, msg->msg_len);

	dp = PACK_UNICAST_PORT_ADDRESS(
							1,				/* GbE NIM */
							NFX_NFP_TXRX_ANY,
							app_id,
							EEE_CONTROL,
							NFX_COS_HIGH,
							0);

	rc = eee_sendMessage(	buf,
							sizeof(sbm_sendControlAndSACData_t) -
												sizeof(uchar8 *)+strlen(line),
							dp,
							msg->source_port);

	if (rc != NFX_OK)
		printf("eee_sendMessage error, 0x%x\n", rc);
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:
 *
 *-----------------------------------------------------------------
 */
test_transmit_read_req()
{
	uchar8					line[80];
	uchar8					buf[128];
	sbm_sendControlPkt_t	* msg;
	uint32					slot, cpu, app_id, test_app_id;
	int32					rc;
	uint32					dp;

	printf("transmit read request control message\n");
	printf("Enter destination slot : ");
	get_input(line);
	slot = atoi(line);
	printf("Enter destination CPU : ");
	get_input(line);
	cpu = atoi(line);
	printf("Enter control message : ");
	get_input(line);

	app_id = eee_getApplicationId("sbm");
	test_app_id = eee_getApplicationId("gmc_test_rcv");

	msg = (sbm_sendControlPkt_t *)buf;
	msg->cmd = CMD_TO_MCPU_SEND_CONTROL_READ_REQ;
	msg->dest_port = PACK_UNICAST_PORT_ADDRESS(
										slot,
										cpu,
										test_app_id,
										EEE_USER_DATA,
										NFX_COS_HIGH,
										0);
	msg->source_port = PACK_UNICAST_PORT_ADDRESS(
											nfxMySlotId,
											nfxMyCPUId,
											test_app_id,
											EEE_USER_DATA,
											NFX_COS_HIGH,
											0);
	dp = PACK_UNICAST_PORT_ADDRESS(
							1,				/* GbE NIM */
							NFX_NFP_TXRX_ANY,
							app_id,
							EEE_CONTROL,
							NFX_COS_HIGH,
							0);

	msg->TID = 0xA5A5A5A5;
	msg->msg_len = strlen(line);
	memcpy(&(msg->msg), line, msg->msg_len);

	rc = eee_sendMessage(buf,
					sizeof(sbm_sendControlPkt_t)-sizeof(uchar8 *)+strlen(line),
					dp,
					msg->source_port);
	if (rc != NFX_OK)
		printf("eee_sendMessage error, 0x%x\n", rc);

}

uchar8	* metadata_buffer;		/* will be allocated from shared memory */

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:
 *
 *-----------------------------------------------------------------
 */
void
test_transmit_metadata()
{
	uchar8						line[80];
	uchar8						buf[128];
	sbm_retrieveSACData_t		* msg;
	nfx_cod_t					cod;
	uint32						length, offset;
	uint32						slot, cpu, app_id, test_app_id;
	uint32						dp, sp;
	int32						rc;

	printf("Enter COD handle : ");
	cod = get_cod(line);

	printf("Enter length : ");
	get_input(line);
	length = atoi(line);

	printf("Enter offset : ");
	get_input(line);
	offset = atoi(line);

	app_id = eee_getApplicationId("sbm");
	test_app_id = eee_getApplicationId("gmc_test_rcv");

	msg = (sbm_retrieveSACData_t *)buf;

	msg->cmd = CMD_TO_MCPU_RETRIEVE_SAC_DATA;
	msg->COD = cod;
	msg->length = length;
	msg->offset = offset;
	msg->TID = 0x12345678;
	msg->dest_addr = metadata_buffer;

	printf("retrieve %d bytes from cod, 0x%llx, to 0x%x\n",
										length, cod, msg->dest_addr);

	dp = PACK_UNICAST_PORT_ADDRESS(
							1,				/* GbE NIM */
							NFX_NFP_TXRX_ANY,
							app_id,
							EEE_CONTROL,
							NFX_COS_HIGH,
							0);

	sp = PACK_UNICAST_PORT_ADDRESS(
							nfxMySlotId,
							nfxMyCPUId,
							test_app_id,
							EEE_CONTROL,
							NFX_COS_HIGH,
							0);

	rc = eee_sendMessage(	buf,
							sizeof(sbm_retrieveSACData_t),
							dp,
							sp);
	if (rc != NFX_OK)
		printf("eee_sendMessage error, 0x%x\n", rc);

}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:
 *
 *-----------------------------------------------------------------
 */
test_transmit()
{
	uchar8	line[80];

	printf("Send read request (r), SAC data (l or s), or Metadata (m) : ");
	get_input(line);
	if (line[0] == 'r')
		test_transmit_read_req();
	else if (line[0] == 'l')
		test_transmit_sac_data(0);
	else if (line[0] == 's')
		test_transmit_sac_data(1);
	else if (line[0] == 'm')
		test_transmit_metadata();
	else
		printf("Invalid selection\n");
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:
 *
 *-----------------------------------------------------------------
 */
test_free_cod()
{
	uchar8			line[80];
	uchar8			buf[128];
	sbm_freeCOD_t	* msg;
	nfx_cod_t		cod;
	uint32			app_id, test_app_id;
	uint32			dp, sp;
	int32			rc;

	printf("Enter COD handle : ");
	cod = get_cod(line);

	app_id = eee_getApplicationId("sbm");
	test_app_id = eee_getApplicationId("sbm_test");

	msg = (sbm_freeCOD_t *)buf;
	msg->cmd = CMD_TO_MCPU_MANIPULATE_FREE;
	msg->COD = cod;

	dp = PACK_UNICAST_PORT_ADDRESS(
							1,				/* GbE NIM */
							NFX_NFP_TXRX_ANY,
							app_id,
							EEE_CONTROL,
							NFX_COS_HIGH,
							0);

	sp = PACK_UNICAST_PORT_ADDRESS(
							nfxMySlotId,
							nfxMyCPUId,
							test_app_id,
							EEE_CONTROL,
							NFX_COS_HIGH,
							0);

	rc = eee_sendMessage(	buf,
							sizeof(sbm_freeCOD_t),
							dp,
							sp);
	if (rc != NFX_OK)
		printf("eee_sendMessage error, 0x%x\n", rc);
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:
 *
 *-----------------------------------------------------------------
 */
test_merge_cod()
{
	uchar8			line[80];
	uchar8			buf[128];
	sbm_mergeCOD_t	* msg;
	nfx_cod_t		src_cod, dest_cod;
	uint32			src_length, src_offset, dest_offset;
	uint32			app_id, test_app_id;
	uint32			dp, sp;
	uint32			TID;
	int32			rc;

	printf("Enter src COD handle : ");
	src_cod = get_cod(line);

	printf("Enter src length : ");
	get_input(line);
	src_length = atoi(line);

	printf("Enter src offset : ");
	get_input(line);
	src_offset = atoi(line);

	printf("Enter dest COD handle : ");
	dest_cod = get_cod(line);

	printf("Enter dest offset : ");
	get_input(line);
	dest_offset = atoi(line);

	printf("Enter TID : ");
	get_input(line);
	TID = atoi(line);

	app_id = eee_getApplicationId("sbm");
	test_app_id = eee_getApplicationId("sbm_test");

	msg = (sbm_mergeCOD_t *)buf;
	msg->cmd = CMD_TO_MCPU_MANIPULATE_MERGE;
	msg->src_COD = src_cod;
	msg->src_len = src_length;
	msg->src_offset = src_offset;
	msg->dest_COD = dest_cod;
	msg->dest_offset = dest_offset;
	msg->TID = TID;

	dp = PACK_UNICAST_PORT_ADDRESS(
							1,				/* GbE NIM */
							NFX_NFP_TXRX_ANY,
							app_id,
							EEE_CONTROL,
							NFX_COS_HIGH,
							0);

	sp = PACK_UNICAST_PORT_ADDRESS(
							nfxMySlotId,
							nfxMyCPUId,
							test_app_id,
							EEE_CONTROL,
							NFX_COS_HIGH,
							0);

	rc = eee_sendMessage(	buf,
							sizeof(sbm_mergeCOD_t),
							dp,
							sp);
	if (rc != NFX_OK)
		printf("eee_sendMessage error, 0x%x\n", rc);
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:
 *
 *-----------------------------------------------------------------
 */
test_split_cod()
{
	uchar8			line[80];
	uchar8			buf[128];
	sbm_splitCOD_t	* msg;
	nfx_cod_t		src_cod, dest_cod;
	uint32			src_offset;
	uint32			app_id, test_app_id;
	uint32			dp, sp;
	int32			rc;

	printf("Enter src COD handle : ");
	src_cod = get_cod(line);

	printf("Enter src offset : ");
	get_input(line);
	src_offset = atoi(line);

	printf("Enter dest COD handle : ");
	dest_cod = get_cod(line);

	app_id = eee_getApplicationId("sbm");
	test_app_id = eee_getApplicationId("sbm_test");

	msg = (sbm_splitCOD_t *)buf;
	msg->cmd = CMD_TO_MCPU_MANIPULATE_SPLIT;
	msg->src_COD = src_cod;
	msg->src_offset = src_offset;
	msg->dest_COD = dest_cod;
	msg->TID = 0x22222222;

	dp = PACK_UNICAST_PORT_ADDRESS(
							1,				/* GbE NIM */
							NFX_NFP_TXRX_ANY,
							app_id,
							EEE_CONTROL,
							NFX_COS_HIGH,
							0);

	sp = PACK_UNICAST_PORT_ADDRESS(
							nfxMySlotId,
							nfxMyCPUId,
							test_app_id,
							EEE_CONTROL,
							NFX_COS_HIGH,
							0);

	rc = eee_sendMessage(	buf,
							sizeof(sbm_splitCOD_t),
							dp,
							sp);
	if (rc != NFX_OK)
		printf("eee_sendMessage error, 0x%x\n", rc);

}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:
 *
 *-----------------------------------------------------------------
 */
test_concat_cod()
{
	uchar8			line[80];
	uchar8			buf[128];
	sbm_concatCOD_t	* msg;
	nfx_cod_t		src_cod[10], dest_cod;
	uint32			app_id, test_app_id;
	uint32			dp, sp;
	int32			rc;
	uint32			num = 0, i;
	nfx_cod_t		* p;

	while (num < 10) {
		printf("Enter src COD handle (return for none): ");
		src_cod[num] = get_cod(line);
		if (strlen(line))
			num++;
		else
			break;
	}

	printf("Enter dest COD handle : ");
	dest_cod = get_cod(line);

	app_id = eee_getApplicationId("sbm");
	test_app_id = eee_getApplicationId("sbm_test");

	msg = (sbm_concatCOD_t *)buf;
	msg->cmd = CMD_TO_MCPU_MANIPULATE_CONCAT;
	msg->num_src_CODs = num;
	msg->TID = 0x33333333;
	msg->dest_COD = dest_cod;
	p = &(msg->src_COD);
	for (i=0; i<num; i++)
		*p++ = src_cod[i];

	dp = PACK_UNICAST_PORT_ADDRESS(
							1,				/* GbE NIM */
							NFX_NFP_TXRX_ANY,
							app_id,
							EEE_CONTROL,
							NFX_COS_HIGH,
							0);

	sp = PACK_UNICAST_PORT_ADDRESS(
							nfxMySlotId,
							nfxMyCPUId,
							test_app_id,
							EEE_CONTROL,
							NFX_COS_HIGH,
							0);

	rc = eee_sendMessage(	buf,
							sizeof(sbm_concatCOD_t)+(sizeof(nfx_cod_t)*(num-1)),
							dp,
							sp);

	if (rc != NFX_OK)
		printf("eee_sendMessage error, 0x%x\n", rc);
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:
 *
 *-----------------------------------------------------------------
 */
test_copy_cod()
{
	uchar8			line[80];
	uchar8			buf[128];
	sbm_copyCOD_t	* msg;
	nfx_cod_t		src_cod, dest_cod;
	uint32			src_off, src_len, dest_off;
	uint32			app_id, test_app_id;
	uint32			dp, sp;
	int32			rc;

	printf("Enter src COD handle : ");
	src_cod = get_cod(line);

	printf("Enter src length : ");
	get_input(line);
	src_len = atoi(line);

	printf("Enter src offset : ");
	get_input(line);
	src_off = atoi(line);

	printf("Enter dest COD handle : ");
	dest_cod = get_cod(line);

	printf("Enter dest offset : ");
	get_input(line);
	dest_off = atoi(line);

	app_id = eee_getApplicationId("sbm");
	test_app_id = eee_getApplicationId("sbm_test");

	msg = (sbm_copyCOD_t *)buf;
	msg->cmd = CMD_TO_MCPU_MANIPULATE_COPY;
	msg->src_COD = src_cod;
	msg->src_len = src_len;
	msg->src_offset = src_off;
	msg->dest_COD = dest_cod;
	msg->dest_offset = dest_off;
	msg->TID = 0x44444444;
	
	dp = PACK_UNICAST_PORT_ADDRESS(
							1,				/* GbE NIM */
							NFX_NFP_TXRX_ANY,
							app_id,
							EEE_CONTROL,
							NFX_COS_HIGH,
							0);

	sp = PACK_UNICAST_PORT_ADDRESS(
							nfxMySlotId,
							nfxMyCPUId,
							test_app_id,
							EEE_CONTROL,
							NFX_COS_HIGH,
							0);

	rc = eee_sendMessage(	buf,
							sizeof(sbm_copyCOD_t),
							dp,
							sp);

	if (rc != NFX_OK)
		printf("eee_sendMessage error, 0x%x\n", rc);
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:
 *
 *-----------------------------------------------------------------
 */
test_create_cod()
{
	uchar8			line[80];
	uchar8			buf[128];
	sbm_createCOD_t	* msg;
	nfx_cod_t		cod;
	uint32			len, fill;
	uint32			app_id, test_app_id;
	uint32			dp, sp;
	int32			rc;

	printf("Enter new COD handle : ");
	cod = get_cod(line);

	printf("Enter length : ");
	get_input(line);
	len = atoi(line);

	printf("Enter fill value : ");
	get_input(line);
	fill = atox(line);

	app_id = eee_getApplicationId("sbm");
	test_app_id = eee_getApplicationId("sbm_test");

	msg = (sbm_createCOD_t *)buf;
	msg->cmd = CMD_TO_MCPU_MANIPULATE_CREATE;
	msg->COD = cod;
	msg->len = len;
	msg->fill = fill;
	msg->TID = 0x55555555;

	dp = PACK_UNICAST_PORT_ADDRESS(
							1,				/* GbE NIM */
							NFX_NFP_TXRX_ANY,
							app_id,
							EEE_CONTROL,
							NFX_COS_HIGH,
							0);

	sp = PACK_UNICAST_PORT_ADDRESS(
							nfxMySlotId,
							nfxMyCPUId,
							test_app_id,
							EEE_CONTROL,
							NFX_COS_HIGH,
							0);

	rc = eee_sendMessage(	buf,
							sizeof(sbm_createCOD_t),
							dp,
							sp);

	if (rc != NFX_OK)
		printf("eee_sendMessage error, 0x%x\n", rc);
}

#ifdef NFP_TXRX
/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:
 *
 *-----------------------------------------------------------------
 */
test_view_sbm(int32 t)
{
	uchar8		line[80];
	uint32		index;
	nfx_cod_t	cod;

	switch (t) {
		case 0:			/* hmt */
			printf("Enter hmt index (or 'all') : ");
            get_input(line);
            if (isdigit(line[0]))
                index = atoi(line);
			else
				index = 0xFFFFFFFF;
            sbm_debugHMTDisplay(index);
			break;

		case 1:			/* bdl */
			printf("Enter bdl index (or 'all') : ");
			get_input(line);
			if (isdigit(line[0]))
				index = atoi(line);
			else
				index = 0xFFFFFFFF;
			sbm_debugBDLDisplay(index);
			break;

		case 2:			/* hash */
			sbm_debugHashTableDisplay();
			break;

#ifndef SBM_REMOVE_GMC
		case 3:			/* LUC */
			printf("View MCPU or GMC TxQ : ");
			get_input(line);
			if (line[0] == 'm')
				sbm_debugLUCTxQDisplay();
			else if (line[0] == 'g')
				gmc_debugLUCTxQDisplay();
			break;

		case 4:			/* SNIM */
			printf("View MCPU or GMC TxQ : ");
			get_input(line);
			if (line[0] == 'm')
				sbm_debugSNIMTxQDisplay();
			else if (line[0] == 'g')
				gmc_debugSNIMTxQDisplay();
			break;

		case 5:			/* CopyQ */
			printf("View MCPU or GMC CopyQ : ");
			get_input(line);
			if (line[0] == 'm')
				sbm_debugCopyQDisplay();
			else if (line[0] == 'g')
				gmc_debugCopyQDisplay();
			break;

		case 6:			/* Metadata */
			printf("View MCPU or GMC MetaQ : ");
			get_input(line);
			if (line[0] == 'm')
				sbm_debugMetaQDisplay();
			else if (line[0] == 'g')
				gmc_debugMetaQDisplay();
			break;
#endif

		case 7:			/* IFQ */
#ifndef SBM_REMOVE_GMC
			printf("View MCPU or GMC IFQ : ");
			get_input(line);
			if (line[0] == 'm')
#endif
				sbm_debugIFQDisplay();
#ifndef SBM_REMOVE_GMC
			else
				gmc_debugIFQDisplay();
#endif
			break;

		case 8:			/* FBQ */
#ifndef SBM_REMOVE_GMC
			printf("View MCPU or GMC IFQ : ");
			get_input(line);
			if (line[0] == 'm')
#endif
				sbm_debugFBQDisplay();
#ifndef SBM_REMOVE_GMC
			else
				gmc_debugFBQDisplay();
#endif
			break;

		case 9:
			printf("Enter COD handle : ");
			cod = get_cod(line);
			sbm_debugCODDisplay(cod);
			 break;

		case 10:
			sbm_debugSummaryDisplay();
			break;

		default:
			return -1;
	}

	return 0;
}

#ifndef SBM_REMOVE_GMC
/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:
 *
 *-----------------------------------------------------------------
 */
test_receive_sac_data(nfx_cod_t cod, int32 len)
{
	uchar8	line[80];

	if (cod == -1) {
		printf("Enter COD handle : ");
		cod = get_cod(line);
	}
	if (len == -1) {
		printf("Enter COD length : ");
		get_input(line);
		len = atoi(line);
	}
	printf("receive SAC data, len = %d, COD = 0x%llx\n", len, cod);
	gmc_receive(cod, len);

}
#endif
#endif	/* NFP_TXRX */

/*-----------------------------------------------------------------
 * Name:	do_sac()
 *
 * Description:	 for MCPU Sac Buffer Management debugging
 *
 * Created by:	Rick Lund
 *
 * Date Created:	7/8/01
 *
 *-----------------------------------------------------------------
 */
int32
do_sac(
	int32	argc,
	char8	* argv[]
)
{
	if (strncmp(argv[1], "tr", 2) == 0)
		test_transmit();
	else if (strncmp(argv[1], "free", 4) == 0)
		test_free_cod();
	else if (strncmp(argv[1], "mer", 3) == 0)
		test_merge_cod();
	else if (strncmp(argv[1], "con", 3) == 0)
		test_concat_cod();
	else if (strncmp(argv[1], "spl", 3) == 0)
		test_split_cod();
	else if (strncmp(argv[1], "copy", 4) == 0)
		test_copy_cod();
	else if (strncmp(argv[1], "create", 5) == 0)
		test_create_cod();
#ifdef NFP_TXRX
#ifndef SBM_REMOVE_GMC
	else if (strncmp(argv[1], "r", 1) == 0) {
		if (argc == 3) {
			uint64	cod;
			atollx(&cod, argv[2]);
			test_receive_sac_data(cod, -1);
		}
		else if (argc == 4) {
			uint64	cod;
			atollx(&cod, argv[2]);
			test_receive_sac_data(cod, atoi(argv[3]));
		}
		else if (argc == 2)
			test_receive_sac_data(-1, -1);
		else
			return -1;
	}
#endif
	else if (strncmp(argv[1], "view", 4) == 0) {
		if (argc != 3)
			return -1;
		if (strncmp(argv[2], "hmt", 3) == 0)
			test_view_sbm(0);
		else if (strncmp(argv[2], "bdl", 3) == 0)
			test_view_sbm(1);
		else if (strncmp(argv[2], "hash", 4) == 0)
			test_view_sbm(2);
#ifndef SBM_REMOVE_GMC
		else if (strncmp(argv[2], "luc", 3) == 0)
			test_view_sbm(3);
		else if (strncmp(argv[2], "snim", 4) == 0)
			test_view_sbm(4);
		else if (strncmp(argv[2], "copyq", 4) == 0)
			test_view_sbm(5);
		else if (strncmp(argv[2], "meta", 4) == 0)
			test_view_sbm(6);
#endif
		else if (strncmp(argv[2], "ifq", 3) == 0)
			test_view_sbm(7);
		else if (strncmp(argv[2], "fbq", 3) == 0)
			test_view_sbm(8);
		else if (strncmp(argv[2], "cod", 3) == 0)
			test_view_sbm(9);
		else if (strncmp(argv[2], "sum", 3) == 0)
			test_view_sbm(10);
	}
#endif

	return 0;
}

#endif	/* NFX_MOD_NFP */

#ifdef LINUX_TEST
#ifndef SSC_MGMT
/*-----------------------------------------------------------------
 * Name:	test_sacify()
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:
 *
 *-----------------------------------------------------------------
 */
void
test_sacify(
	uchar8	* file,
	int32	len
)
{
	uchar8	line[80], fname[80];
	uint32	gran;

	if (!file) {
		printf("Enter filename : ");
		get_input(line);
		strcpy(fname, line);
	}
	else
		strcpy(fname, file);

	if (len == -1) {
		printf("Enter packet length : ");
		get_input(line);
		gran = atoi(line);
	}
	else
		gran = len;

	printf("test_sacify, fname = %s, gran = %d\n", fname, gran);

	sbm_debugSacify(fname, gran);

}

/*-----------------------------------------------------------------
 * Name:	do_sacify_file()
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:
 *
 *-----------------------------------------------------------------
 */
int32
do_sacify_file(
	int32	argc,
	char8	* argv[]
)
{
	int32	gran;

	if (argc == 3) {
		gran = atoi(argv[2]);
		test_sacify(argv[1], gran);
	}
	else if (argc == 2) {
		test_sacify(argv[1], -1);
	}
	else if (argc == 1)
		test_sacify(0, -1);

	return 0;
}
#endif
#endif
#endif	/* !SBM_REV2 */

/*-----------------------------------------------------------------
 * Name:	do_ssc_msg()
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	7/17/01
 *
 *-----------------------------------------------------------------
 */
int32
do_ssc_msg(
	int32	argc,
	char8	* argv[]
)
{
	int32	rc;
	uchar8	msg[80];
	uchar8	* app;
	uint32	dp, sp;
	uint32	app_id;

	strcpy(msg, "test message");

	if (argc != 2)
		return -1;

	app = argv[1];

	app_id = eee_getApplicationId("ssc_evm");

	printf("app_id = %d\n", app_id);
	dp = PACK_UNICAST_PORT_ADDRESS(
									0,
									0,
									app_id,
									EEE_CONTROL,
									NFX_COS_HIGH,
									0);
	sp = PACK_UNICAST_PORT_ADDRESS(
									nfxMySlotId,
									nfxMyCPUId,
									app_id,
									EEE_CONTROL,
									NFX_COS_HIGH,
									0);
	printf("sending message, \"%s\", to SSC app %d\n", msg, app_id);
	rc = eee_sendMessage(msg, strlen(msg)+1, dp, sp);
	if (rc != NFX_OK)
		printf("eee_sendMessage returned %x\n", rc);

	return NFX_OK;
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:
 *
 *-----------------------------------------------------------------
 */
int32
do_copytest(
	int32	argc,
	char8	* argv[]
)
{
	uchar8	not_malloc_buf[100];
	uchar8	* malloc_buf;
	eee_descPtr_t	edesc;
	eee_descBuf_t	* bd;
	uchar8	* buf;
	uint32	control;

	edesc = eee_allocateDesc(EEE_SHARED_POOL, NFX_COS_HIGH);
	if (!edesc) {
		printf("failed to allocate descriptor\n");
		return -1;
	}
	buf = eee_allocateBuffer(EEE_SHARED_POOL, EEE_BUF_LARGE, NFX_COS_HIGH);
	if (!buf) {
		printf("failed to allocate buffer\n");
		return -1;
	}

	control = 1 << EDESC_BUFCNT_SHIFT;
	bd = &(edesc->bd[1]);
	bd->control = EDESC_BD_CMD_IN_USE | EDESC_BD_LARGE;
	bd->buf = buf;

	/*
	 * test malloc'd buffer - size ok
	 */
	malloc_buf = eee_ramAlloc(512);
	edesc->hdr.control = control | 2048;
	bd->len = 2048;
	if (eee_copyPacketToLocalBuffer(malloc_buf, edesc, 512, 1) < 0)
		printf("copyPkt returned error\n");
	else
		printf("copyPkt OK\n");

	edesc->hdr.control = control | 100;
	bd->len = 100;
	if (eee_copyPacketToLocalBuffer(malloc_buf, edesc, 512, 0) < 0)
		printf("copyPkt returned error\n");
	else
		printf("copyPkt OK\n");
	eee_ramDealloc(malloc_buf);

	/*
	 * test not a malloc'd buffer
	 */
	edesc->hdr.control = control | 100;
	bd->len = 100;
	if (eee_copyPacketToLocalBuffer(not_malloc_buf, edesc, 100, 1) < 0)
		printf("copyPkt returned error\n");
	else
		printf("copyPkt OK\n");

	/*
	 * test error case - (copy len < pkt_len, truncate = 0)
	 */
	malloc_buf = eee_ramAlloc(512);
	edesc->hdr.control = control | 2048;
	bd->len = 2048;
	if (eee_copyPacketToLocalBuffer(malloc_buf, edesc, 512, 0) < 0)
		printf("copyPkt returned error\n");
	else
		printf("copyPkt OK\n");
	eee_ramDealloc(malloc_buf);

	/*
	 * test error case - (copy len > pkt_len)
	 */
	malloc_buf = eee_ramAlloc(3000);
	edesc->hdr.control = control | 2048;
	bd->len = 2048;
	if (eee_copyPacketToLocalBuffer(malloc_buf, edesc, 3000, 1) < 0)
		printf("copyPkt returned error\n");
	else
		printf("copyPkt OK\n");
	eee_ramDealloc(malloc_buf);

	return NFX_OK;
}

#if 0
/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:
 *
 *-----------------------------------------------------------------
 */
int32
do_test_promio(
	int32	argc,
	char8	* argv[]
)
{
	char8	* str;

	str = argv[1];
	printf("trying to print string: %s\n", str);

	promio_prs(str);
	promio_prs("\r\n");

	printf("reinit uart and try same test: \n");
	promio_init();
	promio_prs(str);
	promio_prs("\r\n");

	return 0;
}
#endif


extern int threadtest_func(void);

/*-----------------------------------------------------------------
 * Name:	do_threadtest()
 *
 * Description:
 *
 * Created by:	bob
 *
 * Date Created:	11/6/01
 *
 *-----------------------------------------------------------------
 */
int32
do_threadtest(
    int32   argc,
    char8   * argv[]
)
{
    int32     rc;

    rc = threadtest_func();
    printf("threadtest_func returns %d\n", rc);
	return 0;
}

#if defined(NFP_TXRX) || defined(NFP_FP)
extern void dp_proxy_ui(void);
int32
do_dpp(
	int32	argc,
	char8	* argv[]
)
{
	dp_proxy_ui();
	return 0;
}

#endif

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	1/30/02
 *
 *-----------------------------------------------------------------
 */
int32
do_edebug(
	int32	argc,
	char8	* argv[]
)
{
	eee_debug_ui();

	return 0;
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	2/21/02
 *
 *-----------------------------------------------------------------
 */
int32
do_mallocOwner(
	int32	argc,
	char8	* argv[]
)
{
	malloc_showOwner();

	return 0;
}

/*-----------------------------------------------------------------
 * Display ESM timing statistics.
 *-----------------------------------------------------------------
 */
int32
do_esmTimeStats(
	int32	argc,
	char8	* argv[]
)
{
    extern int esm_showTimeStats(int argc, char **argv);

    esm_showTimeStats(argc, argv);

	return 0;
}

#ifdef SIBYTE
int32
do_cerr(
	int32	argc,
	uchar8	* argv[]
)
{

	printf("Current Cache Error count = %d\n", *(volatile uint32 *)PHYS_TO_K0(0x00700000));
	return 0;
}

int32
ht_traffic_ui(
	int32	argc,
	uchar8	* argv[]
)
{
	extern uint64	ht_bytes_tx;
	extern uint64	ht_bytes_rx;

	if (argc == 2) {
		if (strcmp(argv[1], "clear") == 0) {
			ht_bytes_tx = 0;
			ht_bytes_rx = 0;
		}
		else
			return -1;
	}
	else {
		printf("HT bytes TX = %lld\n", ht_bytes_tx);
		printf("HT bytes RX = %lld\n", ht_bytes_rx);
	}

	return 0;
}

#endif

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	4/8/02
 *
 *-----------------------------------------------------------------
 */
extern int xtlb_refill_count;
extern int xtlb_refill_time;

int32
do_cp0_regs(
	int32	argc,
	uchar8	* argv[]
)
{
	printf("sr    = 0x%08x\n", (uint32) _mips_mfc0(C0_SR));
	printf("cfg   = 0x%08x\n", (uint32) _mips_mfc0(C0_CONFIG));
	printf("cause = 0x%08x\n", (uint32) _mips_mfc0(C0_CAUSE));
	printf("sp    = %p\n", (void *)_mips_get_address_reg(R_SP));

	printf("\nxtlb refill count = %d\n", xtlb_refill_count);
	printf("xtlb refill time = %d\n", xtlb_refill_time);
	if (xtlb_refill_count)
	printf("average refill cycles = %d\n", xtlb_refill_time/xtlb_refill_count);
	xtlb_refill_count = 0;
	xtlb_refill_time = 0;

	return 0;
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	5/20/02
 *
 *-----------------------------------------------------------------
 */
int32
show_config(
	int32	argc,
	uchar8	* argv[]
)
{
	extern uchar8	_ftext[];

#define MB	(1024*1024)

#ifdef NFX_MOD_NFP
#ifdef LARGE_MEMORY
	printf("\n");
	printf("memory size    = %llu MB\n", memorysize / MB);
	printf("\n");
	printf("runtime offset = %lu MB\n",
								((address_t)_ftext - (address_t)K0BASE) / MB);
	printf("malloc size    = %llu MB\n",
								get_memory_region_size(MMU_MALLOC_REGION) / MB);
	printf("sm size (txrx) = %llu MB\n",
								get_memory_region_size(MMU_SM_TXRX_REGION) / MB);
	printf("sac size       = %llu MB\n",
								get_memory_region_size(MMU_SAC_REGION) / MB);
	printf("sm size (fp)   = %llu MB\n",
								get_memory_region_size(MMU_SM_FP_REGION) / MB);
	printf("\n");
	printf("malloc virtual base    = %p\n",
								(void *)get_memory_virt_base(MMU_MALLOC_REGION));
	printf("sm (txrx) virtual base = %p\n",
								(void *)get_memory_virt_base(MMU_SM_TXRX_REGION));
	printf("sac virtual base       = %p\n",
								(void *)get_memory_virt_base(MMU_SAC_REGION));
	printf("sm (fp) virtual base   = %p\n",
								(void *)get_memory_virt_base(MMU_SM_FP_REGION));

	printf("\n");
	printf("malloc physical base    = %p\n",
								(void *)get_memory_phys_base(MMU_MALLOC_REGION));
	printf("sm (txrx) physical base = %p\n",
								(void *)get_memory_phys_base(MMU_SM_TXRX_REGION));
	printf("sac physical base       = %p\n",
								(void *)get_memory_phys_base(MMU_SAC_REGION));
	printf("sm (fp) physical base   = %p\n",
								(void *)get_memory_phys_base(MMU_SM_FP_REGION));

	printf("\n");
#endif
#endif

	return 0;
}

#if defined(USE_THREAD_STACKS) || defined(USE_BSS_THREAD_STACKS)
/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	6/5/02
 *
 *-----------------------------------------------------------------
 */
int32
ts_cmd(
	int32	argc,
	uchar8	* argv[]
)
{
	if (!(strncmp(argv[1], "alloc", 3))) {
		address_t	ptr;
		ptr = esm_stackAlloc();
		printf("allocated %lx\n", ptr);
	}
	else if (!(strncmp(argv[1], "free", 2))) {
		address_t	ptr;
		ptr = atolx(argv[2]);
		printf("dealloc %lx\n", ptr);
		esm_stackDealloc(ptr);
	}
	else if (!(strncmp(argv[1], "stats", 2))) {
		esm_stackViewStats();
	}
	else
		return -1;

	return 0;
}
#endif

#if defined(NFP_TXRX) || defined(NFP_FP)
extern int32 bmc_rxMax;
extern int32 bmc_rxHWM;
extern int32 bmc_txMax;
extern int32 bmc_txHWM;

#if defined(NFP_FP)
#define ESM_SET_POLL_VIEW  0
#define ESM_SET_POLL_CLEAR 1
#define ESM_SET_POLL_HI    2
#define ESM_SET_POLL_LO    3
extern int esm_poll_config (int action, int value);
#endif



/*-----------------------------------------------------------------
 * Name: poll_cmd
 *
 * Description: UI command to configure polling loop max. count for 
 *				performance tuning.
 *				
 * Created by:	Shaun Hui
 *
 * Date Created:	7/11/02
 *
 *-----------------------------------------------------------------
 */
int32
poll_cmd( int32 argc, char *argv[] )
{

	if (argc < 2)
		goto poll_usage;

	
	if (strequ(argv[1], "show"))
	{
        printf("bmc rx HWM = %4d MAX = %4d\n", bmc_rxHWM, bmc_rxMax);
        printf("bmc tx HWM = %4d MAX = %4d\n", bmc_txHWM, bmc_txMax);
        printf("eee rx HWM = %4d MAX = %4d\n",
			   eee_my_receive_queue()->rcv_hwm,
			   eee_my_receive_queue()->rcv_max);
#if defined(NFP_FP)
        esm_poll_config (ESM_SET_POLL_VIEW, 0);
#endif
		return(0);
	}
	else if (strequ(argv[1], "set"))
	{
        int value;

        if (argc < 4)
            goto poll_usage;

        if ((value = atoi(argv[3])) < 1)
            goto poll_usage;

		if (strequ(argv[2], "eee"))
		{
			/*---------------------------------
			 * Set eee_processReceiveQueue() 
			 * max. loop count.
			 *---------------------------------
			 */
			eee_my_receive_queue()->rcv_max = value;
		}
		else if (strequ(argv[2], "rx"))
		{
			/*---------------------------------
			 * Set bmc12500Eth_rxPktPoll() 
			 * max. loop count.
			 *---------------------------------
			 */
			bmc_rxMax = value;
		}
		else if (strequ(argv[2], "tx"))
		{
			/*---------------------------------
			 * Set bmc12500Eth_txCleanupQ()
			 * max. loop count.
			 *---------------------------------
			 */
			bmc_txMax = value;
		}
#if defined(NFP_FP)
		else if (strncmp(argv[2], "esmHI", 4) == 0)
		{
			/*---------------------------------
			 * Set esm HI prio max polling count ()
			 *---------------------------------
			 */
            esm_poll_config (ESM_SET_POLL_HI, value);
		}
		else if (strncmp(argv[2], "esmLO", 4) == 0)
		{
			/*---------------------------------
			 * Set esm LO prio max polling count ()
			 *---------------------------------
			 */	
            esm_poll_config (ESM_SET_POLL_LO, value);
        }
#endif
		else
			goto poll_usage;

		return (0);
	}
	else if (strequ(argv[1], "clear"))
	{
		/*---------------------------------
		 * Clear High Water Marks.
		 *---------------------------------
		 */
		eee_my_receive_queue()->rcv_hwm = 0;
		bmc_rxHWM = 0;
		bmc_txHWM = 0;
#if defined(NFP_FP)
        esm_poll_config (ESM_SET_POLL_CLEAR, 0);
#endif
		return (0);
    }
	

poll_usage:
	printf("usage: pollcfg show\n");
	printf("       pollcfg clear\n");
#if defined(NFP_FP)
	printf("       pollcfg set eee|rx|tx|esmHI|esmLO <count>\n");
#else
	printf("       pollcfg set eee|rx|tx <count>\n");
#endif
	return(0);

}
#endif



