/*
 *-----------------------------------------------------------------
 *
 * Name:	help.c
 *
 * RCS:  $Id: help.c,v 1.2 2001/10/08 19:03:51 joseph Exp $
 *
 * Copyright (C) 2001, Agile Storage, Inc.
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	5/31/01
 *
 *-----------------------------------------------------------------
 */

#include <nfx-incl.h>
#include "pmon-ui.h"

/*-----------------------------------------------------------------
 * Name:	do_help()
 *
 * Description:	prints commands from table
 *
 * Created by:	Rick Lund
 *
 * Date Created:	5/31/01
 *
 *-----------------------------------------------------------------
 */
int32
do_help (
	int32	ac,
	char8	*av[]
)
{
	int32	i, j, namemax, optsmax, descmax, len;
	int32	ln, siz;

	namemax = optsmax = descmax = 0;
	for (i = 0; CmdTable[i].name != 0; i++) {
		len = strlen (CmdTable[i].name);
		if (len > namemax)
			namemax = len;
		if (CmdTable[i].opts)
		{
			len = strlen (CmdTable[i].opts);
			if (len > optsmax)
				optsmax = len;
		}
		if (CmdTable[i].desc)
		{
			len = strlen (CmdTable[i].desc);
			if (len > descmax)
				descmax = len;
		}
	}

	siz = moresz;
#if 0
	ioctl (STDIN, CBREAK, NULL);
#endif

	ln = siz;
	if (ac >= 2) {		/* extended help */
		if (strequ (av[1], "*")) {	/* all commands */
			for (i = 0; CmdTable[i].name != 0; i++) {
				if (prhelp (i, &ln, siz, namemax, optsmax))
					break;
			}
		} else {		/* specific commands */
			for (j = 1; j < ac; j++) {
				for (i = 0; CmdTable[i].name != 0; i++) {
					if (strequ (CmdTable[i].name, av[j]))
						break;
				}
				if (CmdTable[i].name) {
					if (prhelp (i, &ln, siz, namemax, optsmax))
						break;
				} else
					printf ("%s: command not found\n", av[j]);
			}
		}
	} else {			/* general help only */
		for (i = 0; CmdTable[i].name != 0; i++) {
			printf ("%*s  %-*s", namemax, CmdTable[i].name, descmax,
														CmdTable[i].desc);
			if (i % 2 != 0)
				printf ("\n");
			else
				printf ("   ");
		}
		if (i % 2 != 0)
			printf ("\n");
	}

	return (0);

}	/* do_help() */

/*-----------------------------------------------------------------
 * Name:	prhelp()
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	5/31/01
 *
 *-----------------------------------------------------------------
 */
int32
prhelp (
	int32	n,
	int32	* lnp,
	int32	siz,
	int32	namemax,
	int32	optsmax
)
{
	Optdesc		* p;
	int32		i;

	sprintf (prnbuf, "%*s  %-*s    %s", namemax, CmdTable[n].name, optsmax,
										CmdTable[n].opts, CmdTable[n].desc);
	if (more (prnbuf, lnp, siz))
		return (1);

	p = CmdTable[n].optdesc;
	if (p != 0) {
		for (i = 0; p[i].name; i++) {
			sprintf (prnbuf, "%*s  %15s    %s", namemax, "",
													p[i].name, p[i].desc);
			if (more (prnbuf, lnp, siz))
				return (1);
		}
	}

	return (0);

}	/* prhelp() */

