/*
 *-----------------------------------------------------------------
 *
 * Name:	mem-commands.c
 *
 * RCS:  $Id: mem-commands.c,v 1.9 2002/09/07 08:04:02 johnh Exp $
 *
 * Copyright (C) 2001, Agile Storage, Inc.
 *
 * Description:	memory debug utilities
 *
 * Created by:	Rick Lund
 *
 * Date Created:	11/10/01
 *
 *-----------------------------------------------------------------
 */

#include <nfx-incl.h>
#include <exports/mem-api.h>
#include <exports/pte-api.h>
#include "pmon-ui.h"

#define MB	(1024*1024)
#ifdef USE_WATCHDOG
#define WDOG	    watchdog_kick()
#else
#define WDOG
#endif


#ifndef LINUX_TEST

#if __long64
/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	5/12/02
 *
 *-----------------------------------------------------------------
 */
void
signextend(
	address_t	* addr
)
{
	if ((*addr & 0xFFFFFFFF80000000) == 0x0000000080000000)
		*addr = *addr | 0xFFFFFFFF00000000;
}
#endif

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	11/10/01
 *
 *-----------------------------------------------------------------
 */
int32
do_mem_modify(
	int32	argc,
	char8	* argv[]
)
{
	uint32			datasz = 4;
	address_t		addr;
	unsigned long	data;

	if (argc > 3) {
		if (argv[1][0] == '-') {
			if (argv[1][1] == 'b')
				datasz = 1;
			else if (argv[1][1] == 'h')
				datasz = 2;
			else if (argv[1][1] == 'w')
				datasz = 4;
#if __mips >= 3
			else if (argv[1][1] == 'd')
				datasz = 8;
#endif
			else
				return -1;
			addr = atolx(argv[2]);
			data = atolx(argv[3]);
		}
		else
			return -1;
	}
	else {
		addr = atolx(argv[1]);
		data = atolx(argv[2]);
	}

#if __long64
    signextend(&addr);
#endif

	if (!is_valid_addr(addr)) {
		printf("Invalid address, %p\n", (void *)addr);
		return 0;
	}

	switch(datasz) {
		case 1:
			*(uchar8 *)addr = (uchar8)data;
			break;
		case 2:
			if (addr & 1)
				return 1;
			*(ushort16 *)addr = (ushort16)data;
			break;
		case 4:
			if (addr & 3)
				return 1;
			*(uint32 *)addr = (uint32)data;
			break;
#if __mips >= 3
		case 8:
			if (addr & 7)
				return 1;
			*(uint64 *)addr = (uint64)data;
			break;
#endif
	}

	return 0;

}	/* do_mem_modify() */

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	11/10/01
 *
 *-----------------------------------------------------------------
 */
int32
do_mem_dump(
	int32	argc,
	char8	* argv[]
)
{
	uint32		datasz = 4;
	uint32		len = 256;
	address_t	addr;
	address_t	raddr;
	uint32		i, j;

	if (argc > 2) {
		if (argv[1][0] == '-') {
			if (argv[1][1] == 'b')
				datasz = 1;
			else if (argv[1][1] == 'h')
				datasz = 2;
			else if (argv[1][1] == 'w')
				datasz = 4;
			else if (argv[1][1] == 'd')
				datasz = 8;
			else
				return -1;
			addr = atolx(argv[2]);
			if (argc > 3)
				len = atox(argv[3]);
		}
		else {
			addr = atolx(argv[1]);
			if (argc > 2)
				len = atox(argv[2]);
		}
	}
	else
		addr = atolx(argv[1]);

#if __long64
signextend(&addr);
#endif

	if (!is_valid_addr(addr)) {
		printf("Invalid address, %p\n", (void *)addr);
		return 0;
	}

	raddr = addr;

	printf("\n");

	for (i=0; i<len; i+=16) {
		printf("%p:  ", (void *)raddr);
		for (j=0; j<16/datasz;j++) {
			switch (datasz) {
				case 1:
					printf("%02x ", *(uchar8 *)raddr);
					break;
				case 2:
					if (addr & 1)
						return 1;
					printf("%04x ", *(ushort16 *)raddr);
					break;
				case 4:
					if (addr & 3)
						return 1;
					printf("%08x ", *(uint32 *)raddr);
					break;
				case 8:
					if (addr & 7)
						return 1;
					printf("%016llx ", *(uint64 *)raddr);
					break;
			}
			raddr += datasz;
		}
		printf("\n");
	}

	return 0;

}	/* do_mem_dump() */

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	12/01/01
 *
 *-----------------------------------------------------------------
 */
int32
domt(
	unsigned long	* adr,
	unsigned long	size
)
{
	int32			i, err = 0;
	unsigned long	*p, r;

#if 0
	int32 j;
	unsigned long w;
/* walking ones test */
	for (p = adr, i = 0; i < size; i += sizeof(long), p++) {
		w = 1;
		for (j = 0; j < sizeof(long)*8; j++) {
			*p = w;
			r = *p;
			if (r != w) {
				err++;
				printf ("\b\nerror: addr=%p read=%p expected=%p  ", p, r, w);
			}
			w <<= 1;
		}
		dotik (1, 0);
	}
#endif

/* store the address in each address */
	for (p = adr, i = 0; i < size; i += sizeof(long), p++) {
		*p = (unsigned long)p;
		dotik (1, 0);
	}

/* check that each address contains its address */
	for (p = adr, i = 0; i < size; i += sizeof(long), p++) {
		r = *p;
		if (r != (unsigned long)p) {
			err++;
			printf ("\b\nerror: adr=%p read=%p expected=%p  ", p,
					(void *)r, p);
		}
		dotik (1, 0);
	}

	return (err);

}	/* domt() */

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	12/01/01
 *
 *-----------------------------------------------------------------
 */
int32
do_mem_test(
	int32	argc,
	char8	* argv[]
)
{
	unsigned long	addr, size;
	int32			err;

	if (argc != 3)
		return -1;

	addr = atolx(argv[1]);
	size = atolx(argv[2]);

	if (addr & (sizeof(long)-1)) {
		printf("Misaligned start address, %p\n", (void *)addr);
		return 0;
	}
#if __long64
    signextend(&addr);
#endif

	printf("Testing %p to %p\n", (void *)addr, (void *)(addr + size));
	printf("Memory test running..  ");
	
	err = domt((unsigned long *)addr, size);

	if (err)
		printf("\b\nThere were %d errors.\n", err);
	else
		printf("\b\nTest passed with no errors.\n");

	return 0;

}	/* do_mem_test() */


int32
domr(
	unsigned long	* adr,
	unsigned long	size
)
{
	int32			i;
	volatile unsigned long	*p, r;

    if (adr && size) {
        goto user_defined;
    }
#if 0
    else {
        adr = (unsigned long *) 0x80000000;
#if __long64
        signextend(&adr);
#endif
#ifdef LARGE_MEMORY
        /* below malloc is system, prom & misc */
        size = get_memory_phys_base(MMU_MALLOC_REGION);
#else
        size = 0x3000000;  /* lo 48 MB */
#endif
    }
    printf("\nbase lomem       base=%p size=%ld MB\n", adr, size/MB);
    p = (volatile unsigned long *) adr;
	for (i = 0; i < size; i += sizeof(unsigned long), p++) {
		r = *p;
        if ((i % 0x10000) == 0) { dotik (1, 0); WDOG; }
	}
#endif

#ifdef LARGE_MEMORY
    adr = (void *)get_memory_virt_base(MMU_MALLOC_REGION);
    size = get_memory_region_size(MMU_MALLOC_REGION);
    printf("\nmalloc    region base=%p size=%ld MB\n", adr, size/MB);
    p = (volatile unsigned long *) adr;
	for (i = 0; i < size; i += sizeof(unsigned long), p++) {
		r = *p;
        if ((i % 0x10000) == 0) { dotik (1, 0); WDOG; }
	}

    adr = (void *)get_memory_virt_base(MMU_SM_TXRX_REGION);
    size = get_memory_region_size(MMU_SM_TXRX_REGION);
    printf("\nsm (txrc) region base=%p size=%ld MB\n", adr, size/MB);
    p = (volatile unsigned long *) adr;
	for (i = 0; i < size; i += sizeof(unsigned long), p++) {
		r = *p;
        if ((i % 0x10000) == 0) { dotik (1, 0); WDOG; }
	}

    adr = (void *)get_memory_virt_base(MMU_SAC_REGION);
    size = get_memory_region_size(MMU_SAC_REGION);
    printf("\nsac       region base=%p size=%ld MB\n", adr, size/MB);
    p = (volatile unsigned long *) adr;
	for (i = 0; i < size; i += sizeof(unsigned long), p++) {
		r = *p;
        if ((i % 0x10000) == 0) { dotik (1, 0); WDOG; }
	}

    adr = (void *)get_memory_virt_base(MMU_SM_FP_REGION);
    size = get_memory_region_size(MMU_SM_FP_REGION);
    printf("\nsm (fp)   region base=%p size=%ld MB\n", adr, size/MB);
#if 0
    p = (volatile unsigned long *) adr;
	for (i = 0; i < size; i += sizeof(unsigned long), p++) {
		r = *p;
        if ((i % 0x10000) == 0) { dotik (1, 0); WDOG; }
	}

    adr = (void *)get_memory_virt_base(MMU_THREADSTACK_REGION);
    size = get_memory_region_size(MMU_THREADSTACK_REGION);
    printf("\nthreadstk region base=%p size=%ld MB\n", adr, size/MB);
    if (size == 0)
        return 0;
#endif

#else
#endif
user_defined:
    p = (volatile unsigned long *) adr;
	for (i = 0; i < size; i += sizeof(unsigned long), p++) {
		r = *p;
        if ((i % 0x10000) == 0) { dotik (1, 0); WDOG; }
	}

	return 0;

}	/* domr() */

int32
do_mem_read(
	int32	argc,
	char8	* argv[]
)
{
	unsigned long	addr, size;
	int32			err;
    int             all;

    printf("argc %d \n", argc);
	if (argc < 3) {
		all = 1;
    }
    else
        all = 0;


    if (all) {
        printf("default setting, reading all memory\n");
        addr = 0;
        size = 0;
    }
    else {
        addr = atolx(argv[1]);
        size = atolx(argv[2]);
        if (addr & (sizeof(long)-1)) {
            printf("Misaligned start address, %p\n", (void *)addr);
            return 0;
        }
#if __long64
        signextend(&addr);
#endif
        printf("reading %p to %p\n", (void *)addr, (void *)(addr + size));
    }

	err = domr((void *)addr, size);

	return 0;

}	/* do_mem_read() */


#endif	/* !LINUX_TEST */
