/*
 *-----------------------------------------------------------------
 *
 * Name:	sbm-commands.c
 *
 * RCS:  $Id: sbm-commands.c,v 1.5 2002/09/27 19:17:29 rick Exp $
 *
 * Copyright (C) 2002, Agile Storage, Inc.
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	8/7/02
 *
 *-----------------------------------------------------------------
 */

#include <nfx-incl.h>

#ifdef SBM_REV2

#include <exports/eee-api.h>
#include <exports/sbm-api.h>
#include <exports/cm-api.h>

#include "../sm-sbm-rev2/sbm-test.h"

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:
 *
 *-----------------------------------------------------------------
 */
nfx_cod_t get_cod(uchar8 *l)
{
	nfx_cod_t	cod;

	get_input(l);

	atollx(&cod, l);

	return cod;
}

#if defined(NFP_TXRX) | defined(NFP_FP)
/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:
 *
 *-----------------------------------------------------------------
 */
void
test_transmit_from_sac()
{
	uchar8						line[80];
	nfx_cod_t					cod;
	uint32						length, offset;
#ifdef NFP_FP
	uint32						slot, cpu;
#endif

	printf("Enter COD handle : ");
	cod = get_cod(line);

	printf("Enter length : ");
	get_input(line);
	length = atoi(line);

	printf("Enter offset : ");
	get_input(line);
	offset = atoi(line);

#ifdef NFP_FP
	printf("Enter destination slot : ");
	get_input(line);
	slot = atoi(line);

	printf("Enter destination CPU : ");
	get_input(line);
	cpu = atoi(line);

	sbm_testTransmitFromSacToSnim(cod, length, offset, slot, cpu);
#endif	/* NFP_FP */

#ifdef NFP_TXRX
	sbm_testTransmitFromSacToLuc(cod, length, offset);
#endif

}
#endif	/* NFP_TXRX | FP */

#ifdef NFP_FP
/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:
 *
 *-----------------------------------------------------------------
 */
void
test_free_cod()
{
	uchar8			line[80];
	uchar8			buf[128];
	sbm_freeCOD_t	* msg;
	nfx_cod_t		cod;
	uint32			app_id;
	uint32			dp, sp;
	int32			rc;

	printf("Enter COD handle : ");
	cod = get_cod(line);

	app_id = eee_getApplicationId("sbm");

	msg = (sbm_freeCOD_t *)buf;
	msg->cmd = CMD_TO_MCPU_MANIPULATE_FREE;
	msg->COD = cod;

	dp = PACK_UNICAST_PORT_ADDRESS(
							1,				/* GbE NIM */
							NFX_NFP_TXRX_ANY,
							app_id,
							EEE_CONTROL,
							NFX_COS_HIGH,
							0);

	sp = PACK_UNICAST_PORT_ADDRESS(
							nfxMySlotId,
							nfxMyCPUId,
							app_id,
							EEE_CONTROL,
							NFX_COS_HIGH,
							0);

	rc = eee_sendMessage(	buf,
							sizeof(sbm_freeCOD_t),
							dp,
							sp);
	if (rc != NFX_OK)
		printf("eee_sendMessage error, 0x%x\n", rc);
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:
 *
 *-----------------------------------------------------------------
 */
void
test_merge_cod()
{
	uchar8			line[80];
	uchar8			buf[128];
	sbm_mergeCOD_t	* msg;
	nfx_cod_t		src_cod, dest_cod;
	uint32			src_length, src_offset, dest_offset;
	uint32			app_id;
	uint32			dp, sp;
	int32			rc;

	printf("Enter src COD handle : ");
	src_cod = get_cod(line);

	printf("Enter src length : ");
	get_input(line);
	src_length = atoi(line);

	printf("Enter src offset : ");
	get_input(line);
	src_offset = atoi(line);

	printf("Enter dest COD handle : ");
	dest_cod = get_cod(line);

	printf("Enter dest offset : ");
	get_input(line);
	dest_offset = atoi(line);

	app_id = eee_getApplicationId("sbm");

	msg = (sbm_mergeCOD_t *)buf;
	msg->cmd = CMD_TO_MCPU_MANIPULATE_MERGE;
	msg->src_COD = src_cod;
	msg->src_len = src_length;
	msg->src_offset = src_offset;
	msg->dest_COD = dest_cod;
	msg->dest_offset = dest_offset;

	dp = PACK_UNICAST_PORT_ADDRESS(
							1,				/* GbE NIM */
							NFX_NFP_TXRX_ANY,
							app_id,
							EEE_CONTROL,
							NFX_COS_HIGH,
							0);

	sp = PACK_UNICAST_PORT_ADDRESS(
							nfxMySlotId,
							nfxMyCPUId,
							app_id,
							EEE_CONTROL,
							NFX_COS_HIGH,
							0);

	rc = eee_sendMessage(	buf,
							sizeof(sbm_mergeCOD_t),
							dp,
							sp);
	if (rc != NFX_OK)
		printf("eee_sendMessage error, 0x%x\n", rc);
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:
 *
 *-----------------------------------------------------------------
 */
void
test_create_cod()
{
	uchar8			line[80];
	uchar8			buf[128];
	sbm_createCOD_t	* msg;
	eee_descPtr_t	edesc;
	nfx_cod_t		cod;
	uint32			len, fill;
	uint32			app_id;
	uint32			dp, sp;
	int32			rc;

	edesc = eee_allocateDesc(EEE_TXRX_POOL, NFX_COS_HIGH);
	if (!edesc) {
		printf("Error allocating descriptor\n");
		return;
	}
	cod = sbm_getCODFromDesc(edesc, nfxMySlotId, nfxMyCPUId);

	printf("Enter length : ");
	get_input(line);
	len = atoi(line);

	printf("Enter fill value : ");
	get_input(line);
	fill = atox(line);

	app_id = eee_getApplicationId("sbm");

	msg = (sbm_createCOD_t *)buf;
	msg->cmd = CMD_TO_MCPU_MANIPULATE_CREATE;
	msg->COD = cod;
	msg->len = len;
	msg->fill = fill;

	dp = PACK_UNICAST_PORT_ADDRESS(
							1,				/* GbE NIM */
							NFX_NFP_TXRX_ANY,
							app_id,
							EEE_CONTROL,
							NFX_COS_HIGH,
							0);

	sp = PACK_UNICAST_PORT_ADDRESS(
							nfxMySlotId,
							nfxMyCPUId,
							app_id,
							EEE_CONTROL,
							NFX_COS_HIGH,
							0);

	rc = eee_sendMessage(	buf,
							sizeof(sbm_createCOD_t),
							dp,
							sp);

	if (rc != NFX_OK)
		printf("eee_sendMessage error, 0x%x\n", rc);
	else
		printf("Created cod, %p, len = %d\n", edesc, len);
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	8/7/02
 *
 *-----------------------------------------------------------------
 */
void
test_split_cod()
{
	uchar8			line[80];
	uchar8			buf[128];
	sbm_splitCOD_t	* msg;
	nfx_cod_t		src_cod, dest_cod;
	uint32			src_offset;
	uint32			app_id;
	uint32			dp, sp;
	int32			rc;
	eee_descPtr_t	edesc;
	
	printf("Enter src COD handle : ");
	src_cod = get_cod(line);
	
	printf("Enter src offset : ");
	get_input(line);
	src_offset = atoi(line);
	
	edesc = eee_allocateDesc(EEE_TXRX_POOL, NFX_COS_HIGH);
	if (!edesc) {
		printf("Error allocating descriptor\n");
		return;
	}
	dest_cod = sbm_getCODFromDesc(edesc, nfxMySlotId, nfxMyCPUId);

	app_id = eee_getApplicationId("sbm");
	
	msg = (sbm_splitCOD_t *)buf;
	msg->cmd = CMD_TO_MCPU_MANIPULATE_SPLIT;
	msg->src_COD = src_cod;
	msg->src_offset = src_offset;
	msg->dest_COD = dest_cod;
	
	dp = PACK_UNICAST_PORT_ADDRESS(
									1,              /* GbE NIM */
									NFX_NFP_TXRX_ANY,
									app_id,
									EEE_CONTROL,
									NFX_COS_HIGH,
									0);
	
	sp = PACK_UNICAST_PORT_ADDRESS(
									nfxMySlotId,
									nfxMyCPUId,
									app_id,
									EEE_CONTROL,
									NFX_COS_HIGH,
									0);
	
	rc = eee_sendMessage(   buf,
							sizeof(sbm_splitCOD_t),
							dp,
							sp);
	if (rc != NFX_OK)
		printf("eee_sendMessage error, 0x%x\n", rc);
	
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	8/7/02
 *
 *-----------------------------------------------------------------
 */
void
test_concat_cod()
{
	uchar8			line[80];
	uchar8			buf[128];
	sbm_concatCOD_t	* msg;
	nfx_cod_t		src_cod, dest_cod;
	uint32			app_id;
	uint32			dp, sp;
	int32			rc;

	printf("Enter src COD handle : ");
	src_cod = get_cod(line);

	printf("Enter dest COD handle : ");
	dest_cod = get_cod(line);

	app_id = eee_getApplicationId("sbm");

	msg = (sbm_concatCOD_t *)buf;
	msg->cmd = CMD_TO_MCPU_MANIPULATE_CONCAT;
	msg->src_COD = src_cod;
	msg->dest_COD = dest_cod;

	dp = PACK_UNICAST_PORT_ADDRESS(
							1,				/* GbE NIM */
							NFX_NFP_TXRX_ANY,
							app_id,
							EEE_CONTROL,
							NFX_COS_HIGH,
							0);

	sp = PACK_UNICAST_PORT_ADDRESS(
							nfxMySlotId,
							nfxMyCPUId,
							app_id,
							EEE_CONTROL,
							NFX_COS_HIGH,
							0);

	rc = eee_sendMessage(	buf,
							sizeof(sbm_concatCOD_t),
							dp,
							sp);
	if (rc != NFX_OK)
		printf("eee_sendMessage error, 0x%x\n", rc);
}
#endif	/* NFP_FP */

#ifdef NFP_TXRX
/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:
 *
 *-----------------------------------------------------------------
 */
int
test_view_sbm(int32 t)
{
	uchar8		line[80];
	nfx_cod_t	cod;

	switch (t) {
		case 0:			/* view stats */
			sbm_testDisplayStats();
			break;

		case 1:			/* view cod */
			printf("Enter COD handle : ");
			cod = get_cod(line);
			sbm_testDisplayCod(cod);
			break;

		case 2:			/* view cod all */
			sbm_testFindAllCods();
			break;

		default:
			return -1;
	}

	return 0;
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	8/7/02
 *
 *-----------------------------------------------------------------
 */
void
test_clear_sbm_stats()
{
	sbm_testClearStats();
}

#endif	/* NFP_TXRX */

/*-----------------------------------------------------------------
 * Name:	do_sac()
 *
 * Description:	 for MCPU Sac Buffer Management debugging
 *
 * Created by:	Rick Lund
 *
 * Date Created:	7/8/01
 *
 *-----------------------------------------------------------------
 */
int32
do_sac(
	int32	argc,
	char8	* argv[]
)
{
#if defined(NFP_TXRX) || defined(NFP_FP)
	if (strncmp(argv[1], "tx", 2) == 0)
		test_transmit_from_sac();
#endif

#ifdef NFP_TXRX
	else if (strncmp(argv[1], "view", 2) == 0) {
		if (argc >= 3) {
			if (strncmp(argv[2], "stats", 2) == 0)
				test_view_sbm(0);
			else if (strncmp(argv[2], "cod", 2) == 0) {
				if (argc >= 4) {
					if (strncmp(argv[3], "all", 2) == 0)
						test_view_sbm(2);
					else
						goto sac_usage;
				}
				else
					test_view_sbm(1);
			}
			else
				goto sac_usage;
		}
		else
			goto sac_usage;
	}
	else if (strncmp(argv[1], "clear", 2) == 0)
		test_clear_sbm_stats();
#endif
#ifdef NFP_FP
	else if (strncmp(argv[1], "free", 2) == 0)
		test_free_cod();
	else if (strncmp(argv[1], "mer", 3) == 0)
		test_merge_cod();
	else if (strncmp(argv[1], "create", 2) == 0)
		test_create_cod();
	else if (strncmp(argv[1], "split", 2) == 0)
		test_split_cod();
	else if (strncmp(argv[1], "concat", 2) == 0)
		test_concat_cod();
#endif
	
	return 0;

#ifdef NFP_TXRX
sac_usage:
#endif	
	printf("sac tx\n");
	printf("sac view stats\n");
	printf("sac view cod (cod # | all)\n");
	printf("sac clear stats\n");
	printf("sac create\n");
	printf("sac merge\n");
	printf("sac free\n");

	return 0;
}

#ifdef LINUX_TEST
#if defined(NFP_TXRX) | defined(FCNIM)
/*-----------------------------------------------------------------
 * Name:	test_sacify()
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:
 *
 *-----------------------------------------------------------------
 */
void
test_sacify(
	uchar8	* file,
	int32	len
)
{
	uchar8	line[80], fname[80];
	uint32	gran;

	if (!file) {
		printf("Enter filename : ");
		get_input(line);
		strcpy(fname, line);
	}
	else
		strcpy(fname, file);

	if (len == -1) {
		printf("Enter packet length : ");
		get_input(line);
		gran = atoi(line);
	}
	else
		gran = len;

	printf("test_sacify, fname = %s, gran = %d\n", fname, gran);

	sbm_testSacify(fname, gran);

}

/*-----------------------------------------------------------------
 * Name:	do_sacify_file()
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:
 *
 *-----------------------------------------------------------------
 */
int32
do_sacify_file(
	int32	argc,
	char8	* argv[]
)
{
	int32	gran;

	if (argc == 3) {
		gran = atoi(argv[2]);
		test_sacify(argv[1], gran);
	}
	else if (argc == 2) {
		test_sacify(argv[1], -1);
	}
	else if (argc == 1)
		test_sacify(0, -1);

	return 0;
}
#endif	/* NFP_TXRX | FCNIM */
#endif	/* LINUX_TEST */

#endif	/* SBM_REV2 */
