/* vi:ts=4
 *-----------------------------------------------------------------
 *
 * Name: ssc-reset
 *
 * RCS:  $Id: ssc-reset.c,v 1.1 2002/03/26 23:04:01 ericc Exp $
 *
 * Copyright (C) 2002, Agile Storage, Inc.
 *
 * Description: This is the "remote console" application butchered into
 *				a view chassis application where a BSD_VIEW_CHASSIS message
 *				is delivered to the MGMT cpu and the chassis mapping
 *				is returned and displayed.  While the code has been butchered,
 *				to bend to my will, I have left the comments vintage.
 *
 * Created by:	eric choi
 *
 * Date Created: 19 Feb 2002
 *
 *-----------------------------------------------------------------
 */
#include <sys/types.h>
#include <sys/time.h>
#include <sys/socket.h>
#include <neteee/eee.h>
#include <pthread.h>
#include <stdio.h>
#include <stdlib.h>
#include <limits.h>
#include <unistd.h>
#include <signal.h>

#include "nfx-incl.h"
#include "cm-helper.h"
#include "exports/anpssc-api.h"		// For SSC_SLOT define
#include "exports/eee-error.h"		// Error code definitions

#define BSD_RESET_SLOT		20
#define BSD_RESET_SLICE		21

static struct sockaddr_eee ourPeer;		// To whom we are speaking
static int Sock;						// Use this to send.
static int myPid;						// Who we are

/*-----------------------------------------------------------------
 * Name: main
 *
 * Description: It all starts here
 *
 * Created by:	eric choi
 *
 * Date Created: 19 Feb 2002
 *
 *-----------------------------------------------------------------
 */
extern int
main (int argc, char *argv[])
{
	struct sockaddr_eee ourAddr;
	char msgBuff[8192];									// Max possible MTU
	sigset_t sigs;
	int i = 0, rc;
	fd_set selset;
	bsd_cm_msg_t sendmsg;
	struct timeval selTO = {5, 0};	// 5 seconds should be enough

#define BUFSIZE (sizeof (msgBuff) - sizeof (rconDataMsg_t))

	map_of_chassis = (cmCard_t *)msgBuff;
/*
	First things first.
	Let's make sure we have rightous arguments.
 */
	myPid = getpid();
	if ((argc != 3) || (argc != 4)) {
		fprintf (stderr, "Usage: reset <slot|slice> [slot] [slice] \n");
		return 1;
	}
	else {
		sendmsg.slot = atoi(argv[2]); 
	
		if ((argc == 3) && (!strcmp(argv[1],"slot")))
			sendmsg.msg = BSD_RESET_SLOT;
		else if ((argc == 4) && (!strcmp(argv[1],"slice"))) {
			sendmsg.slice = atoi(argv[3]); 
			sendmsg.msg = BSD_RESET_SLICE;
		}
	}
	

/*
	Set up our peer's address
 */
	memset (&ourPeer, 0, sizeof(ourPeer));
	ourPeer.seee_family = AF_EEE;
	ourPeer.seee_bplane = 1;
	ourPeer.seee_app = eee_getApplicationId("cm");
	if (ourPeer.seee_app == EEE_ERR_INVALID_APP_NAME) {
		fprintf (stderr, "Unknown application id cm\n");
		exit (1);
	}
	ourPeer.seee_slot = SSC_SLOT;
	ourPeer.seee_cpu = NFX_SSC_MGMT;
/*
	Let's see if we can establish communications with our peer.
	To do that, we'll create two persistent sockets of the EEE kind.
	"Why not use receiveAgileMsg()?", you ask. Well, to do that we would
	need a unique app id for each instance of rcon on the SSC. Could also
	hack "special behavior" into regAgileApp too, but it's really not worth
	the overhead. This is pretty much a special case.
 */
	memset (&ourAddr, 0, sizeof (ourAddr));
	ourAddr.seee_family = AF_EEE;
	ourAddr.seee_slot = SSC_SLOT;
	ourAddr.seee_cpu = NFX_SSC_UNIX;
	ourAddr.seee_bplane = 1;
	
	if ((Sock = socket(PF_EEE, SOCK_DGRAM, 0)) >= 0) {
		if (bind(Sock, (struct sockaddr *)&ourAddr, sizeof(ourAddr)) < 0) {
			perror ("Error binding Rx socket");
			return 1;
		}
	}
	else {
		perror ("Error creating Rx socket");
		return 1;
	}

	if (sendto (Sock, &sendmsg  , sizeof(bsd_cm_msg_t) , 0, (struct sockaddr *)&ourPeer,
		sizeof(struct sockaddr_eee)) < 0) {
			perror("sendto msg failed for view chassis.");
			return 1;
	}
	
/*
	We have sockets! Let's use the Rx one so our peer can record its address
	for later consumption (that's the app id it'll send its stdout data to).
	if (init_viewchassis (Sock, &ourPeer)) {
		fprintf (stderr, "Couldn't initialize view chassis session\n");
		return 1;
	}
 */
/*
	Set up signal handler(s) to terminate the connection upon exit.
	We'll use SIGHUP and/or SIGINT as "normal" termination. We'll catch
	SIGBUS and SIGSEGV and note it as abnormal.
 */
	sigfillset (&sigs);
	sigdelset (&sigs, SIGHUP);
	sigdelset (&sigs, SIGINT);
	sigdelset (&sigs, SIGBUS);
	sigdelset (&sigs, SIGSEGV);
	sigprocmask (SIG_SETMASK, &sigs, NULL);

/*
	signal (SIGHUP, termHand);
	signal (SIGINT, termHand);
	signal (SIGBUS, termHand);
	signal (SIGSEGV, termHand);
	And cried the Raven, evermore. (Sorry Edgar)
	Display the strings we get from our peer and send the stuff we get from
	stdin to them.
 */
#if 0
	do {
#endif
		FD_ZERO (&selset);
		FD_SET (Sock, &selset);

		rc = select (Sock + 1, &selset, NULL, NULL, &selTO);

		if (FD_ISSET (Sock, &selset)) {
			rc = recv (Sock, msgBuff, sizeof (msgBuff), 0);
			cmViewChassis();
		}
		else


#if 0
		i++;
	} while ( i < 0x10000);

	if (i >= 0x10000)
		perror("Timeout on response");
#endif

	if (rc < 0) {
		perror ("RX recv error");
		return 1;
	}
	else {
		return 0;
	}
}
