/* vi:ts=4
 *-----------------------------------------------------------------
 *
 * Name: ccm-loc.h
 *
 * RCS:  $Id: ccm-loc.h,v 1.20 2002/10/04 02:17:05 gregs Exp $
 *
 * Copyright (C) 2001, Agile Storage, Inc.
 *
 * Description: Cluster Config Manager local defs
 *
 * Created by:	Greg Seibert
 *
 * Date Created: 14 Feb 2002
 *
 *-----------------------------------------------------------------
 */
#if !defined (__ccm_loc_h_)
#define __ccm_loc_h_

#include "exports/event-api.h"		// Passing event messages around

// Node shutdown handlers' prototypes

extern void shutdown_node (int sock, char *app, int CPU, int Slot, int Plane, 
                           ccmShutdown_Msg_t *msg);

extern int shut_reply (ncmMsg_t *buf); 

extern int is_shutting_down (char *node);

extern void node_up (int sock, ccmLclCfg_t *lcfg);

// Take over stuff

extern void complete_takeover (ccmShutdown_Msg_t *msg);

extern void takeover_node_services (int sock, char *app, int CPU, int Slot, 
                                    int Plane, ccmTakeOverRq_t *msg);

extern void fake_vis_on_new_node (char *new, evm_volId_t lv);

// Route stuff

extern void rte_chg (nfx_eventMsg_t *msg);
extern void restore_rte (uint32 if_addr, uint32 mask, char *node);

// Volume failure methods

typedef enum {
	not_failed,
	recoverable_failure,
	permanent_failure
} ccm_fail_t;

typedef struct fail_lst {
	evm_volId_t		lv;
	int				evm_OK;	// EVM still believes the volume intact
	time_t			first_fail;
	time_t			can_continue;
	int				failure_count;
	int				got_fsysabort;
	ccm_fail_t		fail_type;
	struct fail_lst	*prev;
	struct fail_lst	*next;
} fail_list_t;

extern int lv_has_failed (evm_volId_t vol);
extern void lv_just_failed (nfx_eventMsg_t *evtMsg);
extern int have_failed_lvs (void);
extern void check_failed_lvs (void);
extern int lv_can_be_online (evm_volId_t vol, char *node);
extern void lv_going_online (evm_volId_t vol);
extern void remove_failed_lv (evm_volId_t vol);
extern void lv_failed_movin (evm_volId_t vol);

// Node cfg sync stuff

extern int we_have_a_scc;
extern void ask_node_for_update(char *nname);
extern void lvVis (int appsock, char *node, evm_volId_t lv);
extern void sync_up_SCC (char *sccName);
extern void sync_SCC (char **cfg);
extern void update_pcc_cfg (char *obj, char *val, char *node);

// Node Failure stuff

extern void node_failed (char *node);
extern void force_takeover (char *newnode, char *lvstr, char *nfs_opts);
extern void record_ip_taken (uint32 addr, char *old, char *new, char *portmsk);
extern void record_lv_taken (evm_volId_t id, char *old_node, char *new_node);

// Share this with our friends

extern uint32 myappid;

#endif
