/* vi:ts=4
 *-----------------------------------------------------------------
 *
 * Name: ncm-loc.h
 *
 * RCS:  $Id: ncm-loc.h,v 1.5 2002/08/30 23:54:30 gregs Exp $
 *
 * Copyright (C) 2001, Agile Storage, Inc.
 *
 * Description: Node Connection Manager. Maintains intra-cluster node
 *              connections via TCP.
 *
 * Created by:	Greg Seibert
 *
 * Date Created: 5 Feb 2002
 *
 *-----------------------------------------------------------------
 */
#if !defined(__ncm_loc_h)
#define __ncm_loc_h

#include <sys/types.h>				// We use the BSD types

#define RESPONSE_WAIT	3600		// After 3600 secs, we'll stop waiting
#define PING_FREQ		30			// Keep-alive the connections

// Each active link has one of these.

typedef struct ncmLink {
	char			*nodeName;		// The node's name
	int 			s;				// TCP Socket fd
	in_addr_t		lnkAddr[2];		// Node's Mgmt port addresses (net order)
	int				usingIF;		// Which are we using (0 = prim, 1 = sec)
	time_t			lastTime;		// when a msg was last recvd on this link
	int				pingSent;		// waiting for ping response
	int 			ackSeq;			// Msg seq number for ack
	struct ncmAck	*needsAck;		// Msgs sent but not ack'ed
	struct ncmRsp	*outstanding;	// list of outstanding responses
	struct ncmLink	*next;			// PCC will have a list. Others == NULL.
} ncmLink_t;

struct ncmAck {
	int				msgSeq;
	ncmMsg_t		*msg;
	int				msgSize;
	struct ncmAck	*next;
};

// This is how we track where response messages are to be routed. These get
// created by the receiving NCM when the message indicate the other end
// expects a response.

typedef struct ncmRsp {
	int				rspSock;		// Who's expecting a response
	char			*remApp;		// App name on remote node
	char			*locApp;		// App name on this node
	ncmLink_t		*link;			// at the other end of this link
	time_t			sent;			// When I relayed the message
	struct ncmRsp	*next;			// Could be we have a bunch
} ncmRsp_t;

// A few Magic definitions. We overload the fields defined of the message
// defined in ncm-api.h since we want to use the same buffer that goes over
// the link to/from send/recvAgileMsg() stuff. They're not big but why copy
// if you really don't have to.

//   msgType gets overloaded so we can ensure we don't get out of sync with
//   the other end (It's a stream socket after all) and we know the
//   whole message size ahead of time. 1/64k we'll get a false frame marker.
//   As long as we stay in sync, (it's a TCP socket. How to lose data?) we're
//   cool. I'm just paranoid.
//   Top 16 bits gets to be a marker while the bottom 16 bits is a byte count.
//   Since these messages pass through UDP while in ANPSSC, they cannot exceed
//   64k anyways.

#define FRAMETAG  0xbabe0000
#define TAGMASK   0xffff0000
#define COUNTMASK ~TAGMASK
#define PINGMAGIC 0xffff		// We'll hide the link status message types
#define RESPMAGIC 0xfffe		// as fake message counts
#define ACKMAGIC  0xfffd

//  I'll mention it here that we overload the nodeName buffer with the source
//  application name when we send it over the TCP connection too.

#define SOURCEAPP nodeName

// In the message over the inter-node pipe, we use rqMsg for extra info.
// Additionally, we use some of those bits for message sequence number.

#define RESPONSE_MASK	0xf0000000
#define NO_RESPONSE		0x00000000
#define EXPECT_REPONSE	0x10000000
#define IS_RESPONSE		0x20000000
#define SEQ_MASK		~RESPONSE_MASK

#endif
