// vi:ts=4
#include <stdio.h>
#include <stdlib.h>

#include "exports/anpssc-api.h"
#include "exports/evm-api.h"
#include "exports/fs-api.h"
#include "exports/rpc.h"
#include "exports/mount.h"

extern int
main(int argc, char *argv[])
{
	int sock;
	long long buf[1024];
	int rc;
	int cpu, slot, plane;
	char app[80];
	mnt_infoReq_t *req = (mnt_infoReq_t *)buf;
	mnt_infoResp_t *rsp = (mnt_infoResp_t *)buf;
	char *path, *opts;
	int msgtype;

	sock = regAgileApp("mountd");
	if (sock < 0) {
		perror ("Unable to register mountd");
		exit (1);
	}

	while (1) {
		rc = recvAgileMsg (sock, buf, sizeof(buf), NULL, 
		                   app, &cpu, &slot, &plane);

		msgtype = req->msgHdr.msgType;
		switch (msgtype) {
			case MNT_EXPORT_REQ:

				printf ("%s: export request for lv %llx\n", argv[0], 
				        req->volId.whole_thing);

				path = malloc (req->nameLen + 1);
				opts = malloc (req->optionsLen + 1);

				memcpy (path, req->pathName, req->nameLen);
				path[req->nameLen] = '\0';
				memcpy (opts, &req->pathName[req->nameLen], req->optionsLen);
				opts[req->optionsLen] = '\0';

				printf ("Path: %s\n", path);
				printf ("Opts: %s\n", opts);

				free(path); free(opts);

			case MNT_UNEXPORT_REQ:

				rsp->msgHdr.msgType = MNT_INFO_RESP;
				rsp->msgHdr.status = NFX_OK;

				path = malloc (req->nameLen + 1);
				memcpy (path, req->pathName, req->nameLen);
				path[req->nameLen] = '\0';

				if (msgtype == MNT_UNEXPORT_REQ) {
					printf ("%s unexport req %s\n", argv[0], path);
				}
				free (path);

				sendAgileMsg (-1, buf, sizeof(mnt_infoResp_t), 
				              app, cpu, slot, plane,
		                      NULL, NULL, 0);
				break;

				default:
					printf ("%s unknown message type: %d\n", 
					         argv[0], req->msgHdr.msgType);
		}

		printf ("%s: got %d(len %d), from %s:%d:%d\n", argv[0], *(int *)buf,
		        rc, app, slot, cpu);
	}
}
