/* vi:ts=4
 *-----------------------------------------------------------------
 *
 * Name: genlic
 *
 * RCS:  $Id: genlic.c,v 1.3 2002/08/01 17:56:53 sandrineb Exp $
 *
 * Copyright (C) 2001, Agile Storage, Inc.
 *
 * Description: Generate a vaild license key
 *
 * Created by:	Greg Seibert
 *
 * Date Created: 04 Feb 2002
 *
 *-----------------------------------------------------------------
 */
#include <sys/time.h>
#include <sys/types.h>
#include <stdlib.h>
#include <limits.h>
#include <stdio.h>
#include <ctype.h>

/*
	Layout of software key (nibbles)

	 11 22 33 44444 55666

	 1 - inverted XOR byte-wise sum over other 7 bytes
	 2 - Hardware revision
	 3 - Software product number
	 4 - Obfuscated Host id
	 5 - Software maximum major revision valid
	 6 - expiration date

	Obfuscation:

	44444 = 20 LSBs of chassis id ^ ~(55666) ^ 2233
 */
extern int main (int argc, char *argv[])
{
	u_int64_t key;
	u_int8_t *keyptr = (u_int8_t *)&key;
	u_int32_t prod_id;
	u_int32_t hw_prod_id;
	u_int32_t sw_prod_id;
	u_int32_t prod_ser_num;
	u_int32_t chassis_id;
	u_int32_t max_sw_rev;
	char inp_buf[100];
	u_int8_t sum;
	int i;
	u_int32_t top_part;
	u_int32_t mid_part;
	u_int64_t bot_part;
	u_int32_t exp_date;
	struct timeval ti;

	fputs ("Enter Software Product Id: ", stdout);
	fgets (inp_buf, sizeof(inp_buf), stdin);

	sw_prod_id = strtoul (inp_buf, NULL, 0);

	fputs ("Enter Software Revision number: ", stdout);
	fgets (inp_buf, sizeof(inp_buf), stdin);

	max_sw_rev = strtoul (inp_buf, NULL, 0);

	fputs ("Enter Hardware Product Id: ", stdout);
	fgets (inp_buf, sizeof(inp_buf), stdin);

	hw_prod_id = strtoul (inp_buf, NULL, 0);

	prod_id = sw_prod_id | (hw_prod_id << 8);

	fputs ("Enter Chassis Id: ", stdout);
	fgets (inp_buf, sizeof(inp_buf), stdin);

	chassis_id = strtoul (inp_buf, NULL, 0) & 0xfffff;

	do {
		fputs ("[T]rial or [P]erpetual key? ", stdout);
		fgets (inp_buf, sizeof(inp_buf), stdin);
		inp_buf[0] = toupper (inp_buf[0]);
	} while ((inp_buf[0] != 'T') && (inp_buf[0] != 'P'));

	if (inp_buf[0] == 'T') {
		gettimeofday (&ti, NULL);
		exp_date = ti.tv_sec + (9 * 0x00100000);	// 9 * 12days3hours16:16
		exp_date >>= 20;
	}
	else {
		exp_date = 0x00000fff;
	}

	prod_ser_num = (exp_date & 0xfff) | ((max_sw_rev & 0xff) << 12);
/*
	Security through obscurity
 */
	key = (u_int64_t)prod_ser_num;
	key |= ((u_int64_t)
		          (chassis_id ^ ((~prod_ser_num) & 0xfffff) ^ prod_id) << 20);
	key |= ((u_int64_t)prod_id << 40);

	for (sum = 0, i = 0; i < sizeof (key); i++) {
		sum = sum ^ keyptr[i];
	}

	keyptr[sizeof (key) - 1] = ~sum;

	top_part = (u_int32_t)(key >> 56) & 0xff;
	mid_part = (u_int32_t)(key >> 40) & 0xffff;
	bot_part = key & 0xffffffffff;

	fprintf (stdout, "Key is: %02x-%04x-%010llx\n", top_part, mid_part, 
	                 bot_part);

	return 0;
}
