/* vi:ts=4
 *-----------------------------------------------------------------
 *
 * Name:	lnm-ui.c
 *
 * RCS:  $Id: lnm-ui.c,v 1.3 2002/05/10 23:42:41 gregs Exp $
 *
 * Copyright (C) 2001, Agile Storage, Inc.
 *
 * Description:
 *
 * Created by:	Pankaj Gupta 
 *
 * Date Created:	11/05/01
 *
 *-----------------------------------------------------------------
 */

#include <nfx-incl.h>

#include <stdio.h>      // fprintf(), stderr
#include <stdlib.h>     // exit()
#include <string.h>     // memset()
#include <unistd.h>     // close()


#include <fcntl.h>
#include <netinet/in.h>

#include <sys/time.h>
#include <sys/socket.h>
#include <signal.h>
#include <errno.h>

#include <exports/anpssc-api.h>
#include <exports/lnm-api.h>
#include <exports/ncm-api.h>
#include <exports/opt-api.h>
#include "lnm-local.h"


/*------------------------------------------------------------------------------
 * Prototypes of local functions.
 *------------------------------------------------------------------------------
 */


/*------------------------------------------------------------------------------
 * Global variables.
 *------------------------------------------------------------------------------
 */

extern  char                myClusterName[LNM_MAX_CLUSTER_NAME];
extern  cluster_nodeInfo_t 	*myClusterNode;
extern  cluster_nodeInfo_t  *myClusterPCC;
extern  cluster_nodeInfo_t  *myClusterSCC;
extern  cluster_nodeInfo_t  clusterNodeInfoArray[MAX_NODE_PER_CLUSTER];
extern	int					clusterDebugLevel;

char *cnStateStr [] = 	{
							"INIT",
							"LISTEN",
							"FIND PCC",
							"CC",
							"NON CC",
							"PCC",
							"SCC",
							"DOWN"
						};

char *ccMsgTypeStr [] = 	{ 	/* if using this array make sure to subtract
 							 	offset from msg type */
							"Invalid",
							"Init Hello",
							"Node Hello",
							"Assign SCC",
							"Find Host Req",
							"Find Host Rsp",
							"Mount Point Update",
							"InterNode CLI Msg",
							"Find PCC Req",
							"Command Req",
							"Command Rsp",
							"Report Resource Req",
							"Sync Config Req"
						};

/*------------------------------------------------------------------------------
 * Name         : cluster_printUiUsage()
 *
 * Description  : Shows the correct usage of this program.
 *
 * Created by:	Pankaj Gupta 
 *
 * Date Created:	
 *
 *------------------------------------------------------------------------------
 */
void cluster_printUiUsage()
{
	fprintf(stderr,"Usage:\n");
	fprintf(stderr," cluster view status\n");
}

/*------------------------------------------------------------------------------
 * Name         : cluster_displayStatus()
 *
 * Description  :
 *
 * Created by:	Pankaj Gupta 
 *
 * Date Created:	
 *
 *------------------------------------------------------------------------------
 */
void cluster_displayStatus()
{
	int idx;
	cluster_nodeInfo_t   *nodeEntry;

	fprintf(stdout, "\n\nCluster Name: %s\n", myClusterName);

	if (myClusterPCC)
	{
		fprintf(stdout, "Primary Cluster Controller: %s Ch: %d\n", 
					printIpa(myClusterPCC->nodeIpa[0]), 
						myClusterPCC->nodeChassisId);
	}

	if (myClusterSCC)
	{
		fprintf(stdout, "Secondary Cluster Controller: %s Ch: %d\n", 
			printIpa(myClusterSCC->nodeIpa[0]), myClusterSCC->nodeChassisId);
	}

	fprintf(stdout, "Cluster State: %s\n", 
		myClusterNode->clusterState ? "Enabled":"Disabled");

	fprintf(stdout, "Unmanaged Orphan Volume Adopt: %s\n", 
		myClusterNode->unmanagedOrphanAdoptMode ? "Enabled":"Disabled"); 

	fprintf(stdout, "Node Timeout Forced Takeover Mode: %s\n", 
		myClusterNode->nodeTimeoutForcedTakeoverMode ? "Enabled":"Disabled"); 

	fprintf(stdout, "Cluster Node Table\n");
	fprintf(stdout, "----------------------------------------------\n");
	fprintf(stdout, "Node IP            ChId   State       CCMode\n");
	fprintf(stdout, "----------------------------------------------\n");


	for (idx = 0; idx < MAX_NODE_PER_CLUSTER; idx++)
	{
		nodeEntry = &clusterNodeInfoArray[idx];

		if (!(nodeEntry->useFlag & NODE_ENTRY_ADDED))
			continue;

		fprintf(stdout, "%-15s  %-5d  %-10s  %s/%s\n", 
				printIpa(nodeEntry->nodeIpa[0]), nodeEntry->nodeChassisId, 
				cnStateStr[nodeEntry->nodeState], 
					nodeEntry->CCMode ? "Enabled":"Disabled",
					nodeEntry->preferredPCCMode ? "(P)":" ");
	}

	fprintf(stdout, "------------------------------------------------------\n");
	fprintf(stdout, "\n");

}

#if THIS_IS_STANDALONE
/* The name of this program. */
static char * myName;

/*-----------------------------------------------------------------
 * Name:	cluster_processCli()
 *
 * Description:
 *
 * Created by:	Pankaj Gupta 
 *
 * Date Created:	
 *
 *-----------------------------------------------------------------
 */
void cluster_processCli(int argc, char	*argv[])
{
	int i;
	char *actionStr;
	char *objectStr;

	if (clusterDebugLevel == 0)
	{
		printf("Input rcvd with %d args: \n", argc);
		for (i = 0; i < argc; i++)
			printf("%s ", argv[i]);
		printf("\n");
	}

    /* 
	 * Initialize my own name.
     * Discard path if any.
     */
	myName = argv[0];
	if (rindex(myName, '/'))
    {
		strcpy(myName, rindex(myName, '/') + 1);
	}
    
    /* 
	 * Make sure there's at least the action and the object strings.
     * Get the command and the object on which to execute the command.
     */
	if (argc < 3)
		return;

	actionStr = argv[1];
	objectStr = argv[2];
    
    /* 
	 * Each "command" has it's own action function.
     */
	if (!strcasecmp(actionStr, "view"))
    {
        if (!strcasecmp(objectStr, "status"))
        {
            /* 
			 * Create  message as would come from EA and
			 * send it to our registered application where
			 * we process it and execute the command
             */
			 cluster_displayStatus();
        }
		else
        {
			cluster_printUiUsage();
			return;
		}            
	}
	else if(!strcmp(actionStr, "del"))
    {
		fprintf(stderr, "Invalid command: %s\n",actionStr);
		cluster_printUiUsage();
		return;
	}
	else if(!strcmp(actionStr, "mod"))
    {
		fprintf(stderr, "Invalid command: %s\n",actionStr);
		cluster_printUiUsage();
		return;
	}
	else
    {
		fprintf(stderr, "Invalid command: %s\n",actionStr);
		cluster_printUiUsage();
		return;
	}

    /* 
	 * Send message to EA.
    EL_ASSERT(tcpFd >= 0);
    printf("Sending a message to EA...(%d)\n", tcpFd);
    rc = sendto(tcpFd, (void *) &msgBuf, msgLen, 0, NULL, 0);
    EL_ASSERT(rc == msgLen);
     */
		
	return;
}
#endif
