/* vi:ts=4
 *-----------------------------------------------------------------
 *
 * Name: nfxdns.c
 *
 * RCS:  $Id: nfxdns.c,v 1.3 2002/06/27 22:20:44 gregs Exp $
 *
 * Copyright (C) 2002, Agile Storage, Inc.
 *
 * Description: Resolve host names via DNS config in agile.conf.
 *
 * Created by:	Greg Seibert
 *
 * Date Created: 2 Apr 2002
 *
 *-----------------------------------------------------------------
 */
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <arpa/nameser.h>
#include <resolv.h>
#include <netdb.h>
#include <stdlib.h>
#include <string.h>

#include "nfxnis-loc.h"

static struct __res_state *save = NULL;

static int
setup_resolver (in_addr_t exp_addr)
{
	char *cfg;
	char *domstr;
	char *srvstr;
	int i;
/*
	First time through, we snarf a legit resolver state so we can later
	change it to suit us.
 */
	if (!save) {
		res_init();	// make _res a valid thing
		save = malloc (sizeof(struct __res_state));
		*save = _res;
	}
/*
	We're going to meddle with the search domain list and server list
 */
	memset (save->dnsrch, 0, sizeof (save->dnsrch));
	memset (save->nsaddr_list, 0 , sizeof (save->nsaddr_list));
	memset (save->defdname, 0, sizeof (save->defdname));
/*
	Look through the config file for a match
 */
	cfg = nfxnis_get_dns_srvr (exp_addr);
	if (cfg) {
		srvstr = cfg;
		domstr = strsep (&srvstr, ":");	// Whack off "dns:"
		domstr = strsep (&srvstr, ":");	// Now separate domain from server list

		save->dnsrch[0] = save->defdname;	// resolv.h say deprecated. Liar!
		strcpy (save->defdname, domstr);

		save->nscount = 0;
		for (i = 0; srvstr && (i < MAXNS); i++) {
			save->nsaddr_list[i].sin_len = sizeof (struct sockaddr_in);
			save->nsaddr_list[i].sin_family = AF_INET;
			save->nsaddr_list[i].sin_port = htons (53);		// DNS is port 53
			save->nsaddr_list[i].sin_addr.s_addr = 
			                        strtol (strsep (&srvstr, ":"), NULL, 16);
			save->nscount++;
		}

		free (cfg);
/*
	Set the lookup order to use file, bind
 */
		save->lookups[0] = 'f';
		save->lookups[1] = 'b';
		save->lookups[2] = '\0';
		save->lookups[3] = '\0';

		_res = *save;	// Ready it with our data
		return 0;
	}
	return 1;
}

extern int
nfxnis_resolve_via_DNS (char *exportAddr, char *hostName, in_addr_t *rtn_data)
{
	in_addr_t exp_add;
	struct hostent *host_info;
/*
	Is this a good ip address?
 */
	exp_add = inet_addr (exportAddr);
	if (exp_add == INADDR_NONE) {
		return 1;
	}
/*
	Is this address something we have DNS configuration for?
 */
	if (setup_resolver (exp_add)) {
		return 1;
	}
/*
	Does DNS know about him?
 */
	host_info = gethostbyname (hostName);
	if (!host_info) {
		return 1;
	}

	memcpy (rtn_data, host_info->h_addr_list[0], sizeof (in_addr_t));
	return 0;
}
