/* vi:ts=4
 *-----------------------------------------------------------------
 *
 * Name:
 *
 * RCS:  $Id: cmd_av.c,v 1.5 2002/10/02 21:32:04 gregs Exp $
 *
 * Copyright (C) 2001, Agile Storage, Inc.
 *
 * Description:
 *
 * Created by:    Pankaj Gupta
 *
 * Date Created:
 *
 *-----------------------------------------------------------------
 */
#include <sys/types.h>
#include <pwd.h>
#include <readline/history.h>
#include <grp.h>
#include <unistd.h>

#include "zebra.h"
#include "command.h"
#include "log.h"
#include "vtysh.h"
#include "memory.h"

#include "nfxsh.h"
#include "nfx-incl.h"
#include "exports/anpssc-api.h"
#include "exports/opt-api.h"
#include "exports/lnm-api.h"
#include "exports/el-api.h"

// Wait for EL to respond (10s)
static struct timeval waitElTo = {10, 0};

/*
 * Extern functions.
 */

#if HERE_BUT_NOT_USED
/*
 *------------------------------------------------------------------------------
 * Name         : av_userid
 *
 * Description  : Define userid for EL to connect to AV
 *
 * Created by   : Pankaj Gupta
 *
 * Date Created : 06-NOV-2001
 *------------------------------------------------------------------------------
 */

DEFUN(av_userid,
      av_userid_cmd,
      "av userid USERID",
      "AV config command\n"
	  "define AV userid\n"
	  "AV userid\n")
{
	int rc;
	el_msg_t   elMsg;

    bzero((void *) &elMsg, sizeof(el_msg_t));

	elMsg.msgType = EL_CONFIG_SET_REQ;
	strcpy(elMsg.entry.option, OPTIONS_AV_LOGIN);
	strncpy(elMsg.entry.value, argv[0], MAX_VALUE_LEN);

	/*
	 * Send the request to EL daemon on SSC
	 */
	rc = sendAgileMsg(-1, &elMsg, sizeof(el_msg_t),
					"ev", NFX_SSC_UNIX, SSC_SLOT, MGMTPLANE,
					&waitElTo, NULL, 0);

	if (rc != NFX_OK)
	{
		printf("No response from AV link manager\n");
		return CMD_ERR_FAILED;
	}

	return CMD_SUCCESS;
}
#endif

/*
 *------------------------------------------------------------------------------
 * Name         : av_password
 *
 * Description  : Define password for EL to connect to AV
 *
 * Created by   : Pankaj Gupta
 *
 * Date Created : 06-NOV-2001
 *------------------------------------------------------------------------------
 */

DEFUN(av_password,
      av_password_cmd,
      "av password PASSWORD",
      "AV config command\n"
	  "define AV password\n"
	  "AV password\n")
{
	int rc;
	el_msg_t   elMsg;

    bzero((void *) &elMsg, sizeof(el_msg_t));

	elMsg.msgType = EL_CONFIG_SET_REQ;
	strcpy(elMsg.entry.option, OPTIONS_AV_PASSWORD);
	strncpy(elMsg.entry.value, argv[0], MAX_VALUE_LEN);

	/*
	 * Send the request to EL daemon on SSC
	 */
	rc = sendAgileMsg(-1, &elMsg, sizeof(el_msg_t),
					"ev", NFX_SSC_UNIX, SSC_SLOT, MGMTPLANE,
					&waitElTo, NULL, 0);

	if (rc != NFX_OK)
	{
		printf("No response from AV link manager\n");
		return CMD_ERR_FAILED;
	}

	return CMD_SUCCESS;
}

/*
 *------------------------------------------------------------------------------
 * Name         : av_ip
 *
 * Description  : Define IP addr for EL to connect to AV
 *
 * Created by   : Pankaj Gupta
 *
 * Date Created : 06-NOV-2001
 *------------------------------------------------------------------------------
 */

DEFUN(av_ip,
      av_ip_cmd,
      "av host IPADDR",
      "AV config command\n"
	  "define AV host IP address\n"
	  "AV host IP address\n")
{
	int rc;
	el_msg_t   elMsg;
 	struct in_addr  ip_addr;

    bzero((void *) &elMsg, sizeof(el_msg_t));
	memset((char *)&ip_addr, 0, sizeof(struct in_addr));

	elMsg.msgType = EL_CONFIG_SET_REQ;
	strcpy(elMsg.entry.option, OPTIONS_AV_IP);

	if (inet_aton(argv[0], (struct in_addr*)&ip_addr) == 0)
	{
		printf("Invalid AV IP addr specified.\n");
		return CMD_ERR_FAILED;
	}
	else
	{
		strncpy(elMsg.entry.value, argv[0], MAX_VALUE_LEN);
	}

	/*
	 * Send the request to EL daemon on SSC
	 */
	rc = sendAgileMsg(-1, &elMsg, sizeof(el_msg_t),
					"ev", NFX_SSC_UNIX, SSC_SLOT, MGMTPLANE,
					&waitElTo, NULL, 0);

	if (rc != NFX_OK)
	{
		printf("No response from AV link manager\n");
		return CMD_ERR_FAILED;
	}

	return CMD_SUCCESS;
}

/*
 *------------------------------------------------------------------------------
 * Name         : av_show_config
 *
 * Description  : Show current EL configuration
 *
 * Created by   : Pankaj Gupta
 *
 * Date Created : 21-NOV-2001
 *------------------------------------------------------------------------------
 */

DEFUN(av_show_config,
      av_show_config_cmd,
      "av show config",
      "av\n"
      "show AV related info\n"
      "show AV configuration for this node\n")
{
	int 			rsplen;
	el_config_t		elCfg;
	el_msg_t   		elMsg;
 	struct in_addr  ip_addr;

    bzero((void *) &elMsg, sizeof(el_msg_t));

	elMsg.msgType = EL_CONFIG_VIEW_REQ;

	/*
	 * Send the request to EL daemon on SSC
	 */
	rsplen = sendAgileMsg(-1, &elMsg, sizeof(el_msg_t),
						"ev", NFX_SSC_UNIX, SSC_SLOT, MGMTPLANE,
						&waitElTo, &elCfg, sizeof(el_config_t));

	if (rsplen != sizeof(el_config_t))
	{
		printf("No response from AV link manager\n");
		return CMD_ERR_FAILED;
	}

	printf("AV configuration\n");
	printf("----------------\n");

	printf("AV Userid : %s\n", elCfg.avUserid);
	memset((char *)&ip_addr, 0, sizeof(struct in_addr));

	ip_addr.s_addr = elCfg.avIp;
	printf("AV IP Addr: %s\n\n", inet_ntoa(ip_addr));

	return CMD_SUCCESS;
}

void
install_av (priv_t myPriv, enum node_type node)
{
    switch (myPriv)
    {
        case diagPriv:
        case fullPriv:
        case addPriv:
        case changePriv:
             // install_element(node, &av_userid_cmd);
             install_element(node, &av_password_cmd);
             install_element(node, &av_ip_cmd);

        case viewPriv:
             install_element(node, &av_show_config_cmd);
    }
    return;
}
