/* vi:ts=4
 *-----------------------------------------------------------------
 *
 * Name:
 *
 * RCS:  $Id: cmd_elog.c,v 1.8 2002/09/05 23:52:34 gregs Exp $
 *
 * Copyright (C) 2001, Agile Storage, Inc.
 *
 * Description:
 *
 * Created by:    Pankaj Gupta
 *
 * Date Created:
 *
 *-----------------------------------------------------------------
 */
#include <sys/types.h>
#include <pwd.h>
#include <readline/history.h>
#include <grp.h>
#include <unistd.h>

#include "zebra.h"
#include "command.h"
#include "log.h"
#include "vtysh.h"
#include "memory.h"

#include "nfxsh.h"
#include "nfx-incl.h"
#include "exports/anpssc-api.h"
#include "exports/elog-api.h"
#include "../sm-elog/elog-def.h"
#include "exports/opt-api.h"

#define ELOG_HELP_STR "Event Logging Facility\n"

// Wait for Elog to respond (60s)
static struct timeval waitElogTo = {60, 0};

char *elogLevelStr[] = {
						"Emergency",
						"Alert",
						"Critical",
						"Error",
						"Warning",
						"Notice",
						"Info",
						"Debug"
						};

char *elogFacilityStr[] = {
						"local0",
						"local1",
						"local2",
						"local3",
						"local4",
						"local5",
						"local6",
						"local7"
						};

/*
 * Extern functions.
 */

extern void install_lvol (priv_t myPriv, enum node_type node);

/*
 *------------------------------------------------------------------------------ * Name         : has_digits
 *
 * Description  :
 *
 * Created by   : Pankaj Gupta
 *
 * Date Created : 06-NOV-2001
 *------------------------------------------------------------------------------ */

static int32 has_digits(char *str)
{
	int32  cnt;

	for (cnt=0; cnt < strlen(str); cnt++)
		if (!isdigit(str[cnt]))
			return 0;

	return 1;
}

/*
 *------------------------------------------------------------------------------
 * Name         : elog_display
 *
 * Description  : Modify elog display mode for the NFX
 *
 * Created by   : Pankaj Gupta
 *
 * Date Created : 06-NOV-2001
 *------------------------------------------------------------------------------
 */

DEFUN(elog_display,
      elog_display_cmd,
      "elog display MODE",
      ELOG_HELP_STR
	  "modify elog display mode\n"
	  "enable or disable\n")
{
	int rc;
	elog_configNtfy_t   elogCfgReq;

    bzero((void *) &elogCfgReq, sizeof(elog_configNtfy_t));

	elogCfgReq.msgType = ELOG_CONFIG_SET_REQ;
	strcpy(elogCfgReq.entry.option, OPTIONS_ELOG_DISPLAY);
	if (strncmp ("enable", argv[0], strlen(argv[0])) == 0)
	{
		strcpy(elogCfgReq.entry.value, "on");
	}
	else if (strncmp ("disable", argv[0], strlen(argv[0])) == 0)
	{
		strcpy(elogCfgReq.entry.value, "off");
	}
	else
	{
		printf("Invalid elog display mode specified. Choose {enable|disable}\n");
		return CMD_ERR_FAILED;
	}

	/*
	 * Send the request to ELOG daemon on SSC
	 */
	rc = sendAgileMsg(-1, &elogCfgReq, sizeof(elog_configNtfy_t),
						"elog", NFX_SSC_UNIX, SSC_SLOT, MGMTPLANE,
						&waitElogTo, NULL, 0);

	if (rc != NFX_OK)
		return CMD_ERR_FAILED;

	return CMD_SUCCESS;
}

/*
 *------------------------------------------------------------------------------
 * Name         : elog_state
 *
 * Description  : Modify elog state for the NFX
 *
 * Created by   : Pankaj Gupta
 *
 * Date Created : 06-NOV-2001
 *------------------------------------------------------------------------------
 */

DEFUN(elog_state,
      elog_state_cmd,
      "elog state MODE",
      ELOG_HELP_STR
	  "modify elog state\n"
	  "enable or disable\n")
{
	int rc;
	elog_configNtfy_t   elogCfgReq;

    bzero((void *) &elogCfgReq, sizeof(elog_configNtfy_t));

	elogCfgReq.msgType = ELOG_CONFIG_SET_REQ;
	strcpy(elogCfgReq.entry.option, OPTIONS_ELOG_ENABLE);
	if (strncmp ("enable", argv[0], strlen(argv[0])) == 0)
	{
		strcpy(elogCfgReq.entry.value, "on");
	}
	else if (strncmp ("disable", argv[0], strlen(argv[0])) == 0)
	{
		strcpy(elogCfgReq.entry.value, "off");
	}
	else
	{
		printf("Invalid elog state specified. Choose {enable|disable}\n");
		return CMD_ERR_FAILED;
	}

	/*
	 * Send the request to ELOG daemon on SSC
	 */
	rc = sendAgileMsg(-1, &elogCfgReq, sizeof(elog_configNtfy_t),
						"elog", NFX_SSC_UNIX, SSC_SLOT, MGMTPLANE,
						&waitElogTo, NULL, 0);

	if (rc != NFX_OK)
		return CMD_ERR_FAILED;

	return CMD_SUCCESS;
}

/*
 *------------------------------------------------------------------------------
 * Name         : elog_level
 *
 * Description  : Modify elog state for the NFX
 *
 * Created by   : Pankaj Gupta
 *
 * Date Created : 06-NOV-2001
 *------------------------------------------------------------------------------
 */

DEFUN(elog_level,
      elog_level_cmd,
      "elog level LEVEL",
      ELOG_HELP_STR
	  "modify elog default level\n"
	  "debug, info, notice, warning, error, critical, alert, emergency\n")
{
	int rc;
	elog_configNtfy_t   elogCfgReq;

    bzero((void *) &elogCfgReq, sizeof(elog_configNtfy_t));

	elogCfgReq.msgType = ELOG_CONFIG_SET_REQ;
	strcpy(elogCfgReq.entry.option, OPTIONS_ELOG_LEVEL);

	if (strncmp ("debug", argv[0], strlen(argv[0])) == 0)
	{
		strcpy(elogCfgReq.entry.value, "debug");
	}
	else if (strncmp ("info", argv[0], strlen(argv[0])) == 0)
	{
		strcpy(elogCfgReq.entry.value, "info");
	}
	else if (strncmp ("notice", argv[0], strlen(argv[0])) == 0)
	{
		strcpy(elogCfgReq.entry.value, "notice");
	}
	else if (strncmp ("warning", argv[0], strlen(argv[0])) == 0)
	{
		strcpy(elogCfgReq.entry.value, "warning");
	}
	else if ((strncmp ("error", argv[0], strlen(argv[0])) == 0) &&
				(strlen(argv[0]) > 1))
	{
		strcpy(elogCfgReq.entry.value, "error");
	}
	else if (strncmp ("critical", argv[0], strlen(argv[0])) == 0)
	{
		strcpy(elogCfgReq.entry.value, "critical");
	}
	else if (strncmp ("alert", argv[0], strlen(argv[0])) == 0)
	{
		strcpy(elogCfgReq.entry.value, "alert");
	}
	else if ((strncmp ("emergency", argv[0], strlen(argv[0])) == 0) &&
				(strlen(argv[0]) > 1))
	{
		strcpy(elogCfgReq.entry.value, "emergency");
	}
	else
	{
		printf("Invalid elog level specified.\n");
		return CMD_ERR_FAILED;
	}

	/*
	 * Send the request to ELOG daemon on SSC
	 */
	rc = sendAgileMsg(-1, &elogCfgReq, sizeof(elog_configNtfy_t),
						"elog", NFX_SSC_UNIX, SSC_SLOT, MGMTPLANE,
						&waitElogTo, NULL, 0);

	if (rc != NFX_OK)
		return CMD_ERR_FAILED;

	return CMD_SUCCESS;
}

/*
 *------------------------------------------------------------------------------
 * Name         : elog_facility
 *
 * Description  : Modify elog facility to be used
 *
 * Created by   : Pankaj Gupta
 *
 * Date Created : 06-NOV-2001
 *------------------------------------------------------------------------------
 */

DEFUN(elog_facility,
      elog_facility_cmd,
      "elog facility FACILITY",
      ELOG_HELP_STR
	  "modify elog facility name\n"
	  "local0, local1, local2, local3, local4, local5, local6, local7\n")
{
	int rc;
	elog_configNtfy_t   elogCfgReq;

    bzero((void *) &elogCfgReq, sizeof(elog_configNtfy_t));

	elogCfgReq.msgType = ELOG_CONFIG_SET_REQ;
	strcpy(elogCfgReq.entry.option, OPTIONS_ELOG_FACILITY);

	if ((strcmp ("local0", argv[0]) == 0) ||
		(strcmp ("local1", argv[0]) == 0) ||
		(strcmp ("local2", argv[0]) == 0) ||
		(strcmp ("local3", argv[0]) == 0) ||
		(strcmp ("local4", argv[0]) == 0) ||
		(strcmp ("local5", argv[0]) == 0) ||
		(strcmp ("local6", argv[0]) == 0) ||
		(strcmp ("local7", argv[0]) == 0))
	{
		strcpy(elogCfgReq.entry.value, argv[0]);
	}
	else
	{
		printf("Invalid elog facility specified.\n");
		return CMD_ERR_FAILED;
	}

	/*
	 * Send the request to ELOG daemon on SSC
	 */
	rc = sendAgileMsg(-1, &elogCfgReq, sizeof(elog_configNtfy_t),
						"elog", NFX_SSC_UNIX, SSC_SLOT, MGMTPLANE,
						&waitElogTo, NULL, 0);

	if (rc != NFX_OK)
		return CMD_ERR_FAILED;

	return CMD_SUCCESS;
}

/*
 *------------------------------------------------------------------------------
 * Name         : elog_host
 *
 * Description  : Modify elog host for the NFX
 *
 * Created by   : Pankaj Gupta
 *
 * Date Created : 06-NOV-2001
 *------------------------------------------------------------------------------
 */

DEFUN(elog_host,
      elog_host_cmd,
      "elog host A.B.C.D",
      ELOG_HELP_STR
	  "modify elog host\n"
	  "Syslog host IP addr (0.0.0.0 for local)\n")
{
	int rc;
	elog_configNtfy_t   elogCfgReq;
 	struct in_addr  ip_addr;
	char *dec_addr;
	char *there, *here;
	int i;

    bzero((void *) &elogCfgReq, sizeof(elog_configNtfy_t));

	elogCfgReq.msgType = ELOG_CONFIG_SET_REQ;
	strcpy(elogCfgReq.entry.option, OPTIONS_ELOG_HOST);

	memset((char *)&ip_addr, 0, sizeof(struct in_addr));

	// Since the IP address string conversion services assume the "standard"
	// numbering convention (leading '0' means octal), we need to compensate
	// for that here. We will force decimal. The parser will ensure no more
	// than 3 characters per octet member.

	dec_addr = strdup (argv[0]);
	for (here = dec_addr, i = 0; here && i < 4; i++, here++) {
		there = here;
		while ((here[0] == '0') && here[1] && (here[1] != '.')) {
			strcpy (there, here + 1);
		}

		if (!(here = strchr (there, '.'))) {
			break;
		}
	}

	if ((i < 3) || !inet_aton(dec_addr, &ip_addr))
	{
		printf("Invalid elog host IP addr specified.\n");
		free (dec_addr);
		return CMD_ERR_FAILED;
	}
	else
	{
		strcpy(elogCfgReq.entry.value, dec_addr);
	}
	free (dec_addr);

	/*
	 * Send the request to ELOG daemon on SSC
	 */
	rc = sendAgileMsg(-1, &elogCfgReq, sizeof(elog_configNtfy_t),
						"elog", NFX_SSC_UNIX, SSC_SLOT, MGMTPLANE,
						&waitElogTo, NULL, 0);

	if (rc != NFX_OK)
		return CMD_ERR_FAILED;

	return CMD_SUCCESS;
}

/*
 *------------------------------------------------------------------------------
 * Name         : elog_show_config
 *
 * Description  : Show current elog configuration
 *
 * Created by   : Pankaj Gupta
 *
 * Date Created : 21-NOV-2001
 *------------------------------------------------------------------------------
 */

DEFUN(elog_show_config,
      elog_show_config_cmd,
      "elog show config",
      ELOG_HELP_STR
      "show elog related info\n"
      "show elog configuration for this node\n")
{
	int rsplen;
	elog_config_t		elogCfg;
	elog_configNtfy_t   elogCfgReq;
 	struct in_addr  	ip_addr;
	uint32				facilityIdx = 0;

    bzero((void *) &elogCfgReq, sizeof(elog_configNtfy_t));

	elogCfgReq.msgType = ELOG_CONFIG_VIEW_REQ;

	/*
	 * Send the request to ELOG daemon on SSC
	 */
	rsplen = sendAgileMsg(-1, &elogCfgReq, sizeof(elog_configNtfy_t),
						"elog", NFX_SSC_UNIX, SSC_SLOT, MGMTPLANE,
						&waitElogTo, &elogCfg, sizeof(elog_config_t));

	if (rsplen != sizeof(elog_config_t))
		return CMD_ERR_FAILED;

	printf("ELOG configuration\n");
	printf("------------------\n");
	printf("ELOG State: %s\n", elogCfg.enable ? "Enabled" : "Disabled");
	printf("ELOG Display: %s\n", elogCfg.display ? "Enabled" : "Disabled");
	printf("ELOG Level: %s\n", elogLevelStr[elogCfg.levelDefault]);

	facilityIdx = LOG_FAC(elogCfg.facility) - LOG_FAC(LOG_LOCAL0);
	printf("ELOG Facility: %s\n", elogFacilityStr[facilityIdx]);

	memset((char *)&ip_addr, 0, sizeof(struct in_addr));
	ip_addr.s_addr = elogCfg.hostIpa;
	printf("ELOG Host Addr: %s\n\n", inet_ntoa(ip_addr));
	return CMD_SUCCESS;
}

/*
 *------------------------------------------------------------------------------
 * Name         : elog_show_log
 *
 * Description  : Show elog messages stored locally
 *
 * Created by   : Pankaj Gupta
 *
 * Date Created : 21-NOV-2001
 *------------------------------------------------------------------------------
 */

DEFUN(elog_show_log,
      elog_show_log_cmd,
      "elog show log [NUMLINES]",
      ELOG_HELP_STR
      "show elog related info\n"
      "show elog messages stored locally\n"
	  "Number of lines from the end to be displayed\n")
{
	char cmdBuf[256];
	int32 numLines = 0;

	if (argc == 1)
	{
		if (!has_digits(argv[0]))
		{
			printf ("Number of lines not a number\n");
			return CMD_ERR_FAILED;
		}
		numLines = strtol(argv[0], NULL, 0);
		if ((numLines == LONG_MIN) || (numLines == LONG_MAX) ||
						(numLines < 0))
		{
			printf ("Invalid number range\n");
			return CMD_ERR_FAILED;
		}
	}

	if (numLines == 0)
	{
		sprintf(cmdBuf, "more %s", SYSLOG_MSG_FILE);
	}
	else
	{
		sprintf(cmdBuf, "tail -%d %s", numLines, SYSLOG_MSG_FILE);
	}

	system(cmdBuf);
	return CMD_SUCCESS;
}

/*
 *------------------------------------------------------------------------------
 * Name         : elog_clear_log
 *
 * Description  : Clear all locally stored elog messages
 *
 * Created by   : Pankaj Gupta
 *
 * Date Created : 21-NOV-2001
 *------------------------------------------------------------------------------
 */

DEFUN(elog_clear_log,
      elog_clear_log_cmd,
      "elog clear log",
      ELOG_HELP_STR
      "clear elog messages\n"
      "clear elog messages stored locally\n")
{
	char cmdBuf[256];

	/*
	 * Remove rotated old files
	 */
	become_root();
	sprintf(cmdBuf, "rm -rf %s.*", SYSLOG_MSG_FILE);
	system(cmdBuf);

	/*
	 * Truncate main file to size 0
	 */
	truncate(SYSLOG_MSG_FILE, 0);
	unbecome_root();

	return CMD_SUCCESS;
}

/*
 *------------------------------------------------------------------------------
 * Name         : elog_find
 *
 * Description  : Search for specified pattern in locally stored elog messages
 *
 * Created by   : Pankaj Gupta
 *
 * Date Created : 21-NOV-2001
 *------------------------------------------------------------------------------
 */

DEFUN(elog_find,
      elog_find_cmd,
      "elog find KEYWORD",
      ELOG_HELP_STR
      "find elog related info\n"
	  "keyword to be searched for\n")
{
	char cmdBuf[256];

	sprintf(cmdBuf, "zgrep %s %s", argv[0], SYSLOG_MSG_FILE_ALL);

	system(cmdBuf);
	return CMD_SUCCESS;
}

void
install_elog (priv_t myPriv, enum node_type node)
{
    switch (myPriv)
    {
        case diagPriv:
        case fullPriv:
        case addPriv:
        case changePriv:
             install_element(node, &elog_display_cmd);
             install_element(node, &elog_state_cmd);
             install_element(node, &elog_level_cmd);
             install_element(node, &elog_facility_cmd);
             install_element(node, &elog_host_cmd);
             install_element(node, &elog_clear_log_cmd);

        case viewPriv:
             install_element(node, &elog_find_cmd);
             install_element(node, &elog_show_log_cmd);
             install_element(node, &elog_show_config_cmd);
    }
    return;
}
