/* vi:ts=4
 *-----------------------------------------------------------------
 *
 * Name:
 *
 * RCS:  $Id: cmd_fileset.c,v 1.17 2002/10/02 00:52:45 laurentb Exp $
 *
 * Copyright (C) 2001, Agile Storage, Inc.
 *
 * Description:
 *
 * Created by:	Greg Seibert
 *
 * Date Created:
 *
 *-----------------------------------------------------------------
 */
#include <sys/types.h>
#include <nfx-types.h>
#include <pwd.h>
#include <readline/history.h>
#include <grp.h>
#include <unistd.h>

#include "zebra.h"
#include "command.h"
#include "log.h"
#include "vtysh.h"
#include "memory.h"

#include "nfxsh.h"
#include <time.h>
#include "exports/anpssc-api.h"
#include "exports/evm-api.h"
#include "exports/fs-api.h"

#include "cmd_fsutil.h"

extern int	optind;

#define FILESET_STR "Fileset Administration Commands\n"

#define FSCMD           "fscmd"
#define MKFSET          "mkfset"
#define RMFSET          "rmfset"
#define LISTFSET        "list_fsets"
#define GETFSETUSAGE    "get_fset_usage"
#define REPORTFSETUSAGE "report_fset_usage"
#define GETUGUSAGE      "get_ug_usage"
#define REPORTUGUSAGE   "report_ug_usage"

#define USER            "user"
#define GROUP           "group"

#define CREATE_FILESET_CMD "fileset create VOL FILESET_PATH"

#define CREATE_FILESET_HELP                                 \
"create a fileset\n"\
"Specify the volume/filesystem in which fileset is to be created\n"\
"Specify the path in the filesystem where fileset is to be created\n"

#define REMOVE_FILESET_CMD "fileset remove VOL FILESET_PATH"

#define REMOVE_FILESET_HELP                                 \
"remove a fileset\n"\
"Specify the volume/filesystem in which fileset is to be removed\n"\
"Specify the path in the filesystem of the fileset to be removed\n"

#define LIST_FILESET_CMD "fileset show list VOL"

#define LIST_FILESET_HELP                                   \
"show fileset details for a given volume\n"\
"show list of filesets for a given volume\n"\
"Specify the volume name\n"


#define SHOW_FILESET_USER_USAGE_CMD_UID \
"fileset show usage user VOL FILESET_PATH -u UID"

#define SHOW_FILESET_USER_USAGE_CMD_ALL \
"fileset show usage user VOL FILESET_PATH -A"

#define SHOW_FILESET_USER_USAGE_HELP    \
"show fileset details for a given volume\n"\
"show usage of resources for the filesets in the specified volume\n"\
"show usage for users of the specified fileset\n"\
"Specify the volume/filesystem in fileset is present\n"\
"Specify the path of the fileset in the filesystem\n"

#define SHOW_FILESET_USER_USAGE_HELP_UID	\
SHOW_FILESET_USER_USAGE_HELP	\
"to show usage of a particular user\n"	\
"user id for which usage is to be provided"

#define SHOW_FILESET_USER_USAGE_HELP_ALL	\
SHOW_FILESET_USER_USAGE_HELP	\
"to show usage for all users\n"


#define SHOW_FILESET_GROUP_USAGE_CMD_GID \
"fileset show usage group VOL FILESET_PATH -g GID"

#define SHOW_FILESET_GROUP_USAGE_CMD_ALL \
"fileset show usage group VOL FILESET_PATH -A"

#define SHOW_FILESET_GROUP_USAGE_HELP    \
"show fileset details for a given volume\n"\
"show usage of resources for the filesets in the specified volume\n"\
"show usage for groups of the specified fileset\n"\
"Specify the volume/filesystem in fileset is present\n"\
"Specify the path of the fileset in the filesystem\n"\

#define SHOW_FILESET_GROUP_USAGE_HELP_GID    \
SHOW_FILESET_GROUP_USAGE_HELP   \
"to show fileset usage of a particular group\n"	\
"group id for which fileset usage is to be provided"

#define SHOW_FILESET_GROUP_USAGE_HELP_ALL    \
SHOW_FILESET_GROUP_USAGE_HELP   \
"to show usage for all groups\n"


#define SHOW_FILESET_USAGE_CMD \
"fileset show usage fileset VOL [FILESET_PATH]"

#define SHOW_FILESET_USAGE_HELP    \
"show fileset details for a given volume\n"\
"show usage of resources for the filesets in the specified volume\n"\
"show usage of resources for the specified fileset\n"\
"Specify the volume/filesystem in fileset is present\n"\
"Specify the path of the fileset in the filesystem\n"

/*
 * create a fileset
 */

#if 0
DEFUN(create_fileset, create_fileset_cmd, CREATE_FILESET_CMD, 
	  FILESET_STR
      CREATE_FILESET_HELP)
{

    int rc;
    /*
     * Invoke fscmd
     */

    rc = execute_command(FSCMD,
                         4,
                         MKFSET,
                         argv[0],           // volume id
                         argv[1],            // fileset path
                         NULL
                        );
    if (rc) {
            return CMD_ERR_FAILED;
    }
    return CMD_SUCCESS;
}

/*
 * remove a fileset
 */

DEFUN(remove_fileset, remove_fileset_cmd, REMOVE_FILESET_CMD, 
	  FILESET_STR
      REMOVE_FILESET_HELP)
{

    int rc;
    /*
     * Invoke fscmd
     */

    rc = execute_command(FSCMD,
                         4,
                         RMFSET,
                         argv[0],           // volume id
                         argv[1],            // fileset path
                         NULL
                        );
    if (rc) {
            return CMD_ERR_FAILED;
    }
    return CMD_SUCCESS;
}

/*
 * List filesets
 */
DEFUN(list_filesets, list_fileset_cmd, LIST_FILESET_CMD,
	  FILESET_STR
	  LIST_FILESET_HELP)
{
    int rc;
    /*
     * Invoke fscmd
     */

    rc = execute_command(FSCMD,
                         3,
                         LISTFSET,
                         argv[0],           // volume id
                         NULL
                        );
    if (rc) {
            return CMD_ERR_FAILED;
    }
    return CMD_SUCCESS;
}

/*
 * Show the consumption by users
 */

DEFUN(	show_user_usage,
		show_user_usage_cmd_uid,
		SHOW_FILESET_USER_USAGE_CMD_UID,
	  	FILESET_STR
      	SHOW_FILESET_USER_USAGE_HELP_UID)
{
    int     rc;
   
    if (argc == 2) {
        rc = execute_command(FSCMD,
                             5,
                             REPORTUGUSAGE,
                             argv[0],           // volume
                             argv[1],           // fileset path
                             USER,
                             NULL);
    } else {
        rc = execute_command(FSCMD,
                             6,
                             GETUGUSAGE,
                             argv[0],           // volume
                             argv[1],           // fileset path
                             USER,
                             argv[2],           // uid
                             NULL);
    }
    return rc;
}

ALIAS(	show_user_usage,
		show_user_usage_cmd_all,
		SHOW_FILESET_USER_USAGE_CMD_ALL,
	  	FILESET_STR
      	SHOW_FILESET_USER_USAGE_HELP_ALL)

/*
 * Show the consumption by groups
 */

DEFUN(  show_group_usage,
        show_group_usage_cmd_gid,
        SHOW_FILESET_GROUP_USAGE_CMD_GID,
        FILESET_STR
        SHOW_FILESET_GROUP_USAGE_HELP_GID)
{
    int     rc;
   
    if (argc == 2) {
        rc = execute_command(FSCMD,
                             5,
                             REPORTUGUSAGE,
                             argv[0],           // volume
                             argv[1],           // fileset path
                             GROUP,
                             NULL);
    } else {
        rc = execute_command(FSCMD,
                             6,
                             GETUGUSAGE,
                             argv[0],           // volume
                             argv[1],           // fileset path
                             GROUP,
                             argv[2],           // gid
                             NULL);
    }
    return rc;
}

ALIAS(  show_group_usage,
        show_group_usage_cmd_all,
        SHOW_FILESET_GROUP_USAGE_CMD_ALL,
        FILESET_STR
        SHOW_FILESET_GROUP_USAGE_HELP_ALL)


/*
 * Show the consumption for the fileset
 */

DEFUN(show_fileset_usage, show_fileset_usage_cmd, SHOW_FILESET_USAGE_CMD,
	  FILESET_STR
      SHOW_FILESET_USAGE_HELP)
{
    
    int rc;
    char                    *fileset = argv[1];
    
    if (fileset && argc == 2) {
        rc = execute_command(FSCMD,
                             4,
                             GETFSETUSAGE,
                             argv[0],           // volume
                             argv[1],           // fileset path
                             NULL
                            );
        if (rc) {
            return CMD_ERR_FAILED;
        }
    } else {
        rc = execute_command(FSCMD,
                             3,
                             REPORTFSETUSAGE,
                             argv[0],           // volume
                             NULL
                            );
        if (rc) {
            return CMD_ERR_FAILED;
        }
    }
}
#endif
void
install_fileset (priv_t myPriv, enum node_type node)
{
    // put the higher privilage commands first and fall through
    // to pick up the lower priority ones 

#if 0
    switch (myPriv) {
        case diagPriv:
        case fullPriv:
            install_element(node, &remove_fileset_cmd);
        case changePriv:
            // nothing
        case addPriv:
            install_element(node, &create_fileset_cmd);
        case viewPriv:
            install_element(node, &show_user_usage_cmd_uid);
            install_element(node, &show_user_usage_cmd_all);
            install_element(node, &show_group_usage_cmd_gid);
            install_element(node, &show_group_usage_cmd_all);
            install_element(node, &show_fileset_usage_cmd);
            install_element(node, &list_fileset_cmd);
            break;
    }
#endif
}
