/* vi:ts=4
 *-----------------------------------------------------------------
 *
 * Name:
 *
 * RCS:  $Id: cmd_rcon.c,v 1.4 2002/08/21 21:40:04 gregs Exp $
 *
 * Copyright (C) 2001, Agile Storage, Inc.
 *
 * Description:
 *
 * Created by:	Greg Seibert
 *
 * Date Created:
 *
 *-----------------------------------------------------------------
 */
#include <sys/types.h>
#include <pwd.h>
#include <readline/history.h>
#include <grp.h>
#include <unistd.h>

#include "zebra.h"
#include "command.h"
#include "log.h"
#include "vtysh.h"
#include "memory.h"

#include "nfxsh.h"

DEFUN	(rcont1,
		rcont1_cmd,
		"rcon txrx1",
		"Remote Console\n"
		"To the TxRx #1 (slot 1 CPU 0)\n")
{
	execute_command ("/usr/local/agile/bin/rcon", 2, "1", "0", NULL);

	return CMD_SUCCESS;
}

ALIAS	(rcont1,
		rcont_cmd,
		"rcon txrx",
		"Remote Console\n"
		"To the TxRx (slot 1 CPU 0)\n")

DEFUN	(rcont2,
		rcont2_cmd,
		"rcon txrx2",
		"Remote Console\n"
		"To the TxRx #2 (slot 1 CPU 1)\n")
{
	execute_command ("/usr/local/agile/bin/rcon", 2, "1", "1", NULL);

	return CMD_SUCCESS;
}

DEFUN	(rconf1,
		rconf1_cmd,
		"rcon fp1",
		"Remote Console\n"
		"To the FP #1 (slot 1 CPU 2)\n")
{
	execute_command ("/usr/local/agile/bin/rcon", 2, "1", "2", NULL);

	return CMD_SUCCESS;
}

DEFUN	(rconf2,
		rconf2_cmd,
		"rcon fp2",
		"Remote Console\n"
		"To the FP #2 (slot 1 CPU 3)\n")
{
	execute_command ("/usr/local/agile/bin/rcon", 2, "1", "3", NULL);

	return CMD_SUCCESS;
}

DEFUN	(rconsp0,
		rconsp0_cmd,
		"rcon sp0",
		"Remote Console\n"
		"To the SP #0 (slot 2 CPU 0)\n")
{
	execute_command ("/usr/local/agile/bin/rcon", 2, "2", "0", NULL);

	return CMD_SUCCESS;
}

DEFUN	(rconsp1,
		rconsp1_cmd,
		"rcon sp1",
		"Remote Console\n"
		"To the SP #1 (slot 2 CPU 1)\n")
{
	execute_command ("/usr/local/agile/bin/rcon", 2, "2", "1", NULL);

	return CMD_SUCCESS;
}

DEFUN	(rconsp2,
		rconsp2_cmd,
		"rcon sp2",
		"Remote Console\n"
		"To the SP #2 (slot 2 CPU 2)\n")
{
	execute_command ("/usr/local/agile/bin/rcon", 2, "2", "2", NULL);

	return CMD_SUCCESS;
}

DEFUN	(rconsp3,
		rconsp3_cmd,
		"rcon sp3",
		"Remote Console\n"
		"To the SP #3 (slot 2 CPU 3)\n")
{
	execute_command ("/usr/local/agile/bin/rcon", 2, "2", "3", NULL);

	return CMD_SUCCESS;
}

ALIAS	(rconf2,
		rconfp_cmd,
		"rcon fp",
		"Remote Console\n"
		"To the FP (slot 1 CPU 3)\n")

DEFUN	(rcon,
		rcon_cmd,
		"rcon <0-11> <0-15>",
		"Remote Console\n"
		"In slot\n"
		"On CPU\n")
{
	execute_command ("/usr/local/agile/bin/rcon", 2, argv[0], argv[1], NULL);

	return CMD_SUCCESS;
}
void
install_rcon (priv_t myPriv, enum node_type node)
{
/*
	Organizing this switch statment such that the higher priority commands
	are first allows us to fall through and pick up the lower priority
	ones at no extra cost.
 */
	switch (myPriv) {
		case diagPriv:
			install_element (node, &rcon_cmd);
			install_element (node, &rcont1_cmd);
			install_element (node, &rcont2_cmd);
			install_element (node, &rcont_cmd);
			install_element (node, &rconf1_cmd);
			install_element (node, &rconf2_cmd);
			install_element (node, &rconfp_cmd);
			install_element (node, &rconsp0_cmd);
			install_element (node, &rconsp1_cmd);
			install_element (node, &rconsp2_cmd);
			install_element (node, &rconsp3_cmd);
		case fullPriv:
		case changePriv:
		case addPriv:
		case viewPriv:
/*
	These commands are available in all modes
 */
	}
}
