/* vi:ts=4
 *-----------------------------------------------------------------
 *
 * Name:
 *
 * RCS:  $Id: cmd_scsi.c,v 1.7 2002/11/06 18:59:16 nitinb Exp $
 *
 * Copyright (C) 2001, Agile Storage, Inc.
 *
 * Description:
 *
 * Created by:	Frank Salzmann 
 *
 * Date Created:
 *
 *-----------------------------------------------------------------
 */
#include <sys/types.h>
#include <pwd.h>
#include <readline/history.h>
#include <grp.h>
#include <unistd.h>

#include "zebra.h"
#include "command.h"
#include "log.h"
#include "vtysh.h"
#include "memory.h"

#include "nfxsh.h"

#define STORAGE_CMD "/usr/local/agile/bin/storage"

/*-----------------------------------------------------------------
 * Name: scsi_test
 *
 * Description: Send Test Unit command to scsi storage device.
 *
 * Created by:  Frank Salzmann
 *
 * Date Created: 14 March 2002
 *
 *-----------------------------------------------------------------
 */
DEFUN   (scsi_test,
         scsi_test_cmd,
         "scsi test DEVNAME",
         "SCSI Command\n"
         "Issue TEST UNIT READY command.\n"
         "device name\n")
{
    execute_command (STORAGE_CMD, argc+2, 
					 "scsi", "test", argv[0], NULL);
    return CMD_SUCCESS;
} // scsi_test cmd

/*-----------------------------------------------------------------
 * Name: scsi_inquiry
 *
 * Description: Send inquiry command to scsi storage device.
 *
 * Created by:  Frank Salzmann
 *
 * Date Created: 14 March 2002
 *
 *-----------------------------------------------------------------
 */
DEFUN   (scsi_inquiry,
         scsi_inquiry_cmd,
         "scsi inquiry DEVNAME PCODE",
         "SCSI Command\n"
         "Issue INQUIRY command.\n"
         "device name\n"
		 "page code\n")
{
    execute_command (STORAGE_CMD, argc+2, 
					 "scsi", "inquiry", argv[0], argv[1], NULL);
    return CMD_SUCCESS;
} // scsi_inquiry cmd

/*-----------------------------------------------------------------
 * Name: scsi_modesense
 *
 * Description: Send mode sense command to scsi storage device.
 *
 * Created by:  Frank Salzmann
 *
 * Date Created: 14 March 2002
 *
 *-----------------------------------------------------------------
 */
DEFUN   (scsi_modesense,
         scsi_modesense_cmd,
         "scsi modesense DEVNAME PCODE",
         "SCSI Command\n"
         "Issue MODE SENSE command.\n"
         "device name\n"
		 "page code\n")
{
    execute_command (STORAGE_CMD, argc+2, 
					 "scsi", "modesense", argv[0], argv[1], NULL);
    return CMD_SUCCESS;
} // scsi_modesense cmd

/*-----------------------------------------------------------------
 * Name: scsi_modeselect
 *
 * Description: Send mode select command to scsi storage device.
 *
 * Created by:  Frank Salzmann
 *
 * Date Created: 14 March 2002
 *
 *-----------------------------------------------------------------
 */
DEFUN   (scsi_modeselect,
         scsi_modeselect_cmd,
         "scsi modeselect DEVNAME PCODE BOFFSET VALUE",
         "SCSI Command\n"
         "Issue MODE SELECT command.\n"
         "device name\n"
		 "page code\n"
		 "byte offset\n"
		 "value\n")
{
    execute_command (STORAGE_CMD, argc+2, 
					 "scsi", "modeselect", 
	                 argv[0], argv[1], argv[2], argv[3], NULL);
    return CMD_SUCCESS;
} // scsi_modeselect cmd

/*-----------------------------------------------------------------
 * Name: scsi_format
 *
 * Description: Format disk device.
 *
 * Created by:  Frank Salzmann
 *
 * Date Created: 02 Jan 2002
 *
 *-----------------------------------------------------------------
 */
DEFUN   (scsi_format,
         scsi_format_cmd,
         "scsi format DEVNAME SECTORSIZE",
         "SCSI Command\n"
         "Format disk device with new sector size.\n"
         "device name\n")
{
    execute_command (STORAGE_CMD, argc+2,
					 "scsi","format", argv[0], argv[1], NULL);
    return CMD_SUCCESS;
 
} // scsi_format cmd


/*-----------------------------------------------------------------
 * Name: scsi_show_list
 *
 * Description: Show storage device list.
 *
 * Created by:  Frank Salzmann
 *
 * Date Created: 14 Nov 2001
 *
 *-----------------------------------------------------------------
 */
DEFUN   (scsi_show,
         scsi_show_cmd,
         "scsi show (ALL|DISK|TAPE)",
         "SCSI Command\n"
         "Display information about attached storage devices.\n")
{
	execute_command (STORAGE_CMD, argc+2, 
					 "view", "list", argv[0], NULL);
    return CMD_SUCCESS;
}
 
/*-----------------------------------------------------------------
 * Name: scsi_show_detail
 *
 * Description: Show storage device detail.
 *
 * Created by:  Frank Salzmann
 *
 * Date Created: 14 Nov 2001
 *
 *-----------------------------------------------------------------
 */
DEFUN  (scsi_show_detail,
        scsi_show_detail_cmd,
        "scsi show detail NAME",
        "SCSI Command\n"
        "Display information about attached storage devices.\n"
        "Display detailed information about attached storage device.\n"
        "Device name string.\n")
{
    execute_command (STORAGE_CMD,argc+3,
					 "view","dev","detail",argv[0],NULL);
    return CMD_SUCCESS;
}

/*-----------------------------------------------------------------
 * Name: scsi_show_stats
 *
 * Description: Show storage device statistics.
 *
 * Created by:  Frank Salzmann
 *
 * Date Created: 06 Feb 2002
 *
 *-----------------------------------------------------------------
 */
DEFUN  (scsi_show_stats,
        scsi_show_stats_cmd,
        "scsi show stats DEVNAME",
        "SCSI Command\n"
        "Display information about attached storage devices.\n"
        "Display storage device statistics.\n"
        "Device name\n")
{
    execute_command (STORAGE_CMD, argc+3, 
					 "view", "dev", "stats", argv[0], NULL);
    return CMD_SUCCESS;
}

/*-----------------------------------------------------------------
 * Name: scsi_open
 *
 * Description: Send open device request.
 *
 * Created by:  Frank Salzmann
 *
 * Date Created: February 1, 2002
 *
 *-----------------------------------------------------------------
 */
DEFUN  (scsi_open,
        scsi_open_cmd,
        "scsi open DEVNAME [OPTIONS .]",
		"SCSI Command\n"
        "Open storage device.\n"
        "Device name\n"
		"[-p Port Number ] [ -i Path ]\n")
{
	execute_vararg_command(STORAGE_CMD, argc, "open", argv);

    return CMD_SUCCESS;
}

/*-----------------------------------------------------------------
 * Name: scsi_close
 *
 * Description: Send close device request.
 *
 * Created by:  Frank Salzmann
 *
 * Date Created: February 1, 2002
 *
 *-----------------------------------------------------------------
 */
DEFUN  (scsi_close,
        scsi_close_cmd,
        "scsi close DEVNAME",
		"SCSI Command\n"
        "close storage device.\n"
        "device name\n")
{
    execute_command (STORAGE_CMD, argc+1, "close", argv[0], NULL);
    return CMD_SUCCESS;
}

/*-----------------------------------------------------------------
 * Name: scsi discover
 *
 * Description: Performs re-discovery on an existing device
 *
 * Created by:  Nitin Bahadur
 *
 * Date Created: Oct 31, 2002
 *
 *-----------------------------------------------------------------
 */
DEFUN  (scsi_discover,
		scsi_discover_cmd,
		"scsi discover ( WWN | sp.0 | sp.1 | sp.2 | sp.3 )",
		"SCSI Command\n"
		"storage device discovery.\n"
		"device world-wide name\n")
{
	if(!strcmp(argv[0], "sp.0") || !strcmp(argv[0], "sp.1") ||
		!strcmp(argv[0], "sp.2") || !strcmp(argv[0], "sp.3"))
		execute_command(STORAGE_CMD, argc+3, "control", "port", "rescan", argv[0]);
	else
		execute_command (STORAGE_CMD, argc+1, "discover", argv[0], NULL);

	return CMD_SUCCESS;
}


/*-----------------------------------------------------------------
 * Name: scsi mask show
 *
 * Description: Shows currently masked luns
 *
 * Created by:  Nitin Bahadur
 *
 * Date Created: Sept 24, 2002
 *
 *-----------------------------------------------------------------
 */
DEFUN	(scsi_mask_show,
		 scsi_mask_show_cmd,
		 "scsi mask show",
		 "SCSI Command\n"
		 "LUN masking command\n"
		 "show lun masks\n")
{
	execute_command(STORAGE_CMD, 3, "lun", "mask", "show");
	return CMD_SUCCESS;
}

void
install_scsi (priv_t myPriv, enum node_type node)
{
	switch (myPriv)
    {
        case diagPriv:
			install_element (node, &scsi_test_cmd);	
			install_element (node, &scsi_inquiry_cmd);	
			install_element (node, &scsi_modesense_cmd);	
			install_element (node, &scsi_modeselect_cmd);	
			install_element (node, &scsi_format_cmd);
			install_element (node, &scsi_open_cmd);	
			install_element (node, &scsi_close_cmd);	
			install_element (node, &scsi_show_detail_cmd);	
        case fullPriv:
			install_element (node, &scsi_discover_cmd);
        case changePriv:
        case addPriv:
        case viewPriv:
			install_element (node, &scsi_show_cmd);	
			install_element (node, &scsi_show_stats_cmd);	
			install_element (node, &scsi_mask_show_cmd);	
    }
    return;
}
