/* vi:ts=4
 *-----------------------------------------------------------------
 *
 * Name:
 *
 * RCS:  $Id: cmd_stats.c,v 1.6 2002/11/21 00:56:02 syen Exp $
 *
 * Copyright (C) 2002, Agile Storage, Inc.
 *
 * Description:
 *
 * Created by:    Shyh-Pei Yen
 *
 * Date Created:  10/24/02
 *
 *-----------------------------------------------------------------
 */
#include <sys/types.h>
#include <pwd.h>
#include <readline/history.h>
#include <grp.h>
#include <unistd.h>

#include "zebra.h"
#include "command.h"
#include "log.h"
#include "vtysh.h"
#include "memory.h"

#include "nfxsh.h"
#include "nfx-incl.h"
#include "exports/anpssc-api.h"
#include "exports/stats-api.h"

#define STATS_COMMAND_STR  "Components statistics commands\n"

// Wait for stats to respond ()
static struct timeval waitStatsTo = {30, 0};

typedef struct _sscmgmt_port_t
{
	char	name[8];
	int		num;
} sscmgmt_port_t;

static sscmgmt_port_t sscmgmtPort[STATS_SSCMGMT_TOTAL_PORTS] =
{
	{"fc0",		STATS_SSCMGMT_PORT_FC0},
	{"fc1",		STATS_SSCMGMT_PORT_FC1},
	{"fc2",		STATS_SSCMGMT_PORT_FC2},
	{"fc3",		STATS_SSCMGMT_PORT_FC3},
	{"txrx",	STATS_SSCMGMT_PORT_TXRX},
	{"ssc",		STATS_SSCMGMT_PORT_SSC}
};


//------------------------------------------------------------------------------
// Name         : stats_txrx_bmc12500
//
// Description  : 
//
// Created by   : Shyh-Pei Yen
//
// Date Created : 10/24/02
//------------------------------------------------------------------------------

DEFUN(stats_txrx_bmc12500,
      stats_txrx_bmc12500_cmd,
      "stats txrx bmc12500",
      STATS_COMMAND_STR
      "TXRX statistics.\n"
      "TXRX BMC12500 statistics.\n")
{
	int							rc;
	char						tmpbuf[512];
	stats_GetTxRxStatsReq_t		req;
	stats_GetTxRxStatsRsp_t		*rsp = (stats_GetTxRxStatsRsp_t *) tmpbuf;
	stats_txrxBmc12500Stats_t	*pStats;

	// Get TXRX stats first

	memset((void *) &req,   0, sizeof(stats_GetTxRxStatsReq_t));
	memset((void *) tmpbuf, 0, sizeof(tmpbuf));

	req.hdr.msgType = STATS_GETSTATS;
	req.hdr.clientPrivate = getpid();
	req.type = STATS_TXRX_BMC12500;

	rc = sendAgileMsg(-1, &req, sizeof(stats_GetTxRxStatsReq_t),
		"stats", NFX_NFP_TXRX1, 1, MGMTPLANE,
		&waitStatsTo, tmpbuf, sizeof(tmpbuf));

	if ((rc == -1) ||
		(rc < sizeof(stats_GetTxRxStatsRsp_t)) ||
		(rsp->status != NFX_OK))
	{
		printf("Stats request failed\n");
		return CMD_ERR_FAILED;
	}

	pStats = (stats_txrxBmc12500Stats_t *) rsp->data;
	printf("  rx bad packets:     %lld\n", pStats->rxBadPkts);
	printf("  rx no buffer errs:  %lld\n", pStats->rxNoBufErrs);
	printf("  rx no sysDesc errs: %lld\n", pStats->rxNoSysDescErrs);
	printf("  tx queue full drops:%lld\n", pStats->txQFullDrops);
	printf("\n");

	return CMD_SUCCESS;
}


//------------------------------------------------------------------------------
// Name         : stats_txrx_bmc12500_reset
//
// Description  : 
//
// Created by   : Shyh-Pei Yen
//
// Date Created : 10/24/02
//------------------------------------------------------------------------------

DEFUN(stats_txrx_bmc12500_reset,
      stats_txrx_bmc12500_reset_cmd,
      "stats txrx bmc12500 reset",
      STATS_COMMAND_STR
      "TXRX statistics.\n"
      "TXRX BMC12500 statistics.\n"
	  "Reset TXRX BMC12500 statistics counters.\n")
{
	int							rc;
	stats_resetCountersReq_t	req;
	stats_resetCountersRsp_t	rsp;

	memset((void *) &req, 0, sizeof(stats_resetCountersReq_t));
	memset((void *) &rsp, 0, sizeof(stats_resetCountersRsp_t));

	req.hdr.msgType = STATS_RESETCOUNTERS;
	req.hdr.clientPrivate = getpid();
	req.type = STATS_TXRX_BMC12500;

	rc = sendAgileMsg(-1, &req, sizeof(stats_resetCountersReq_t),
		"stats", NFX_NFP_TXRX1, 1, MGMTPLANE,
		&waitStatsTo, &rsp, sizeof(stats_resetCountersRsp_t));

	if ((rc == -1) ||
		(rc < sizeof(stats_resetCountersRsp_t)) ||
		(rsp.status != NFX_OK))
	{
		printf("Stats request failed\n");
		return CMD_ERR_FAILED;
	}

	printf("Counters reset done.\n");
	return CMD_SUCCESS;
}


//------------------------------------------------------------------------------
// Name         : stats_txrx_lsi
//
// Description  : 
//
// Created by   : Shyh-Pei Yen
//
// Date Created : 10/24/02
//------------------------------------------------------------------------------

DEFUN(stats_txrx_lsi,
      stats_txrx_lsi_cmd,
      "stats txrx lsi (0|1|2|3)",
      STATS_COMMAND_STR
      "TXRX statistics.\n"
      "TXRX LSI statistics.\n"
      "Port number (from 0 to 3).\n")
{
	int						rc;
	char					tmpbuf[512];
	stats_GetTxRxStatsReq_t	req;
	stats_GetTxRxStatsRsp_t	*rsp = (stats_GetTxRxStatsRsp_t *) tmpbuf;
	stats_txrxLsiStats_t	*pStats;
	stats_txrxLsiPktDist_t	*pDist;

	// Get TXRX stats first

	memset((void *) &req,   0, sizeof(stats_GetTxRxStatsReq_t));
	memset((void *) tmpbuf, 0, sizeof(tmpbuf));

	req.hdr.msgType = STATS_GETSTATS;
	req.hdr.clientPrivate = getpid();
	req.type = STATS_TXRX_LSI;
	req.port = *argv[0] - '0';

	rc = sendAgileMsg(-1, &req, sizeof(stats_GetTxRxStatsReq_t),
		"stats", NFX_NFP_TXRX1, 1, MGMTPLANE,
		&waitStatsTo, tmpbuf, sizeof(tmpbuf));

	if ((rc == -1) ||
		(rc < sizeof(stats_GetTxRxStatsRsp_t)) ||
		(rsp->status != NFX_OK))
	{
		printf("Stats request failed\n");
		return CMD_ERR_FAILED;
	}

	pStats = (stats_txrxLsiStats_t *) rsp->data;
	printf("Performance:\n");
	printf("  rx bytes:           %lld\n", pStats->rxBytes);
	printf("  rx packets:         %lld\n", pStats->rxPkts);
	printf("  rx broadcast pkts:  %lld\n", pStats->rxBcastPkts);
	printf("  rx multicast pkts:  %lld\n", pStats->rxMcastPkts);
	printf("  rx symbol errors:   %lld\n", pStats->rxSymbolErrs);
	printf("  rx pause pkts:      %lld\n", pStats->rxPausePkts);
	printf("  tx bytes:           %lld\n", pStats->txBytes);
	printf("  tx packets:         %lld\n", pStats->txPkts);
	printf("  tx broadcast pkts:  %lld\n", pStats->txBcastPkts);
	printf("  tx multicast pkts:  %lld\n", pStats->txMcastPkts);
	printf("  tx pause pkts:      %lld\n", pStats->txPausePkts);
	printf("\n");
	printf("Faults:\n");
	printf("  rx drops:           %lld\n", pStats->rxDrops);
	printf("  rx crc errors:      %lld\n", pStats->rxCrcErrs);
	printf("  rx undersized pkts: %lld\n", pStats->rxUndersize);
	printf("  tx underflows:      %lld\n", pStats->txUnderflows);
	printf("  tx errors:          %lld\n", pStats->txErrors);
	printf("\n");

	// Get TXRX packet distribution now

	memset((void *) &req,   0, sizeof(stats_GetTxRxStatsReq_t));
	memset((void *) tmpbuf, 0, sizeof(tmpbuf));

	req.hdr.msgType = STATS_GETSTATS;
	req.hdr.clientPrivate = getpid();
	req.type = STATS_TXRX_LSI_DIST;
	req.port = *argv[0] - '0';

	rc = sendAgileMsg(-1, &req, sizeof(stats_GetTxRxStatsReq_t),
		"stats", NFX_NFP_TXRX1, 1, MGMTPLANE,
		&waitStatsTo, tmpbuf, sizeof(tmpbuf));

	if ((rc == -1) ||
		(rc < sizeof(stats_GetTxRxStatsRsp_t)) ||
		(rsp->status != NFX_OK))
	{
		printf("Stats request failed\n");
		return CMD_ERR_FAILED;
	}

	pDist = (stats_txrxLsiPktDist_t *) rsp->data;
	printf("Packet distribution:\n");
	printf("  rx 64 bytes         %lld\n", pDist->rx_64);
	printf("  rx 65-127 bytes     %lld\n", pDist->rx65_127);
	printf("  rx 128-255 bytes    %lld\n", pDist->rx128_255);
	printf("  rx 255-511 bytes    %lld\n", pDist->rx256_511);
	printf("  rx 511-1023 bytes   %lld\n", pDist->rx512_1023);
	printf("  rx 1023-max bytes   %lld\n", pDist->rx1024_max);
	printf("  tx 64 bytes         %lld\n", pDist->tx_64);
	printf("  tx 65-127 bytes     %lld\n", pDist->tx65_127);
	printf("  tx 128-255 bytes    %lld\n", pDist->tx128_255);
	printf("  tx 255-511 bytes    %lld\n", pDist->tx256_511);
	printf("  tx 511-1023 bytes   %lld\n", pDist->tx512_1023);
	printf("  tx 1023-max bytes   %lld\n", pDist->tx1024_max);
	printf("\n");

	return CMD_SUCCESS;
}


//------------------------------------------------------------------------------
// Name         : stats_txrx_lsi_reset
//
// Description  : 
//
// Created by   : Shyh-Pei Yen
//
// Date Created : 10/24/02
//------------------------------------------------------------------------------

DEFUN(stats_txrx_lsi_reset,
      stats_txrx_lsi_reset_cmd,
      "stats txrx lsi (0|1|2|3) reset",
      STATS_COMMAND_STR
      "TXRX statistics.\n"
      "TXRX LSI statistics.\n"
      "Port number (from 0 to 3).\n"
	  "Reset TXRX LSI port statistics counters\n")
{
	int							rc;
	stats_resetCountersReq_t	req;
	stats_resetCountersRsp_t	rsp;

	memset((void *) &req, 0, sizeof(stats_resetCountersReq_t));
	memset((void *) &rsp, 0, sizeof(stats_resetCountersRsp_t));

	req.hdr.msgType = STATS_RESETCOUNTERS;
	req.hdr.clientPrivate = getpid();
	req.type = STATS_TXRX_LSI;
	req.port = *argv[0] - '0';

	rc = sendAgileMsg(-1, &req, sizeof(stats_resetCountersReq_t),
		"stats", NFX_NFP_TXRX1, 1, MGMTPLANE,
		&waitStatsTo, &rsp, sizeof(stats_resetCountersRsp_t));

	if ((rc == -1) ||
		(rc < sizeof(stats_GetTxRxStatsRsp_t)) ||
		(rsp.status != NFX_OK))
	{
		printf("Stats request failed\n");
		return CMD_ERR_FAILED;
	}

	memset((void *) &req, 0, sizeof(stats_resetCountersReq_t));
	memset((void *) &rsp, 0, sizeof(stats_resetCountersRsp_t));

	req.hdr.msgType = STATS_RESETCOUNTERS;
	req.hdr.clientPrivate = getpid();
	req.type = STATS_TXRX_LSI_DIST;
	req.port = *argv[0] - '0';

	rc = sendAgileMsg(-1, &req, sizeof(stats_resetCountersReq_t),
		"stats", NFX_NFP_TXRX1, 1, MGMTPLANE,
		&waitStatsTo, &rsp, sizeof(stats_resetCountersRsp_t));

	if ((rc == -1) ||
		(rc < sizeof(stats_GetTxRxStatsRsp_t)) ||
		(rsp.status != NFX_OK))
	{
		printf("Stats request failed\n");
		return CMD_ERR_FAILED;
	}

	printf("Counters reset done.\n");
	return CMD_SUCCESS;
}


//------------------------------------------------------------------------------
// Name         : stats_txrx_dp83820
//
// Description  : 
//
// Created by   : Shyh-Pei Yen
//
// Date Created : 10/24/02
//------------------------------------------------------------------------------

DEFUN(stats_txrx_dp83820,
      stats_txrx_dp83820_cmd,
      "stats txrx dp83820",
      STATS_COMMAND_STR
      "TXRX statistics.\n"
      "TXRX DP83820 statistics.\n")
{
	int							rc;
	char						tmpbuf[512];
	stats_GetTxRxStatsReq_t		req;
	stats_GetTxRxStatsRsp_t		*rsp = (stats_GetTxRxStatsRsp_t *) tmpbuf;
	stats_txrxDp83820Stats_t	*pStats;

	// Get TXRX stats first

	memset((void *) &req,   0, sizeof(stats_GetTxRxStatsReq_t));
	memset((void *) tmpbuf, 0, sizeof(tmpbuf));

	req.hdr.msgType = STATS_GETSTATS;
	req.hdr.clientPrivate = getpid();
	req.type = STATS_TXRX_DP83820;

	rc = sendAgileMsg(-1, &req, sizeof(stats_GetTxRxStatsReq_t),
		"stats", NFX_NFP_TXRX1, 1, MGMTPLANE,
		&waitStatsTo, tmpbuf, sizeof(tmpbuf));

	if ((rc == -1) ||
		(rc < sizeof(stats_GetTxRxStatsRsp_t)) ||
		(rsp->status != NFX_OK))
	{
		printf("Stats request failed\n");
		return CMD_ERR_FAILED;
	}

	pStats = (stats_txrxDp83820Stats_t *) rsp->data;
	printf("Performance:\n");
	printf("  rx bytes:           %lld\n", pStats->rxBytes);
	printf("  rx packets:         %lld\n", pStats->rxPkts);
	printf("  tx bytes:           %lld\n", pStats->txBytes);
	printf("  tx packets:         %lld\n", pStats->txPkts);
	printf("\n");
	printf("Faults:\n");
	printf("  rx errors:          %lld\n", pStats->rxErrors);
	printf("  rx fcs errors:      %lld\n", pStats->rxFcsErrs);
	printf("  rx overruns:        %lld\n", pStats->rxOverruns);
	printf("  rx align errors:    %lld\n", pStats->rxAlignErrs);
	printf("  rx symbol errors:   %lld\n", pStats->rxSymbolErrs);
	printf("  rx inrange len errs:%lld\n", pStats->rxLenErrs);
	printf("  rx aborts:          %lld\n", pStats->rxAborts);
	printf("  rx rxNoBufErrs:     %lld\n", pStats->rxNoBufErrs);
	printf("  rx rxNoSysDescs:    %lld\n", pStats->rxNoSysDescs);
	printf("  rx runt pkts:       %lld\n", pStats->rxRuntPkts);
	printf("  rx crc errors:      %lld\n", pStats->rxCrcErrs);

	printf("  tx queue full drops:%lld\n", pStats->txQFullDrops);
	printf("  tx aborts:          %lld\n", pStats->txAborts);
	printf("  tx underruns:       %lld\n", pStats->txUnderruns);
	printf("\n");

	return CMD_SUCCESS;
}


//------------------------------------------------------------------------------
// Name         : stats_txrx_dp83820_reset
//
// Description  : 
//
// Created by   : Shyh-Pei Yen
//
// Date Created : 10/24/02
//------------------------------------------------------------------------------

DEFUN(stats_txrx_dp83820_reset,
      stats_txrx_dp83820_reset_cmd,
      "stats txrx dp83820 reset",
      STATS_COMMAND_STR
      "TXRX statistics.\n"
      "TXRX DP83820 statistics.\n"
	  "Reset TXRX DP83820 statistics counters.\n")
{
	int							rc;
	stats_resetCountersReq_t	req;
	stats_resetCountersRsp_t	rsp;

	memset((void *) &req, 0, sizeof(stats_resetCountersReq_t));
	memset((void *) &rsp, 0, sizeof(stats_resetCountersRsp_t));

	req.hdr.msgType = STATS_RESETCOUNTERS;
	req.hdr.clientPrivate = getpid();
	req.type = STATS_TXRX_DP83820;

	rc = sendAgileMsg(-1, &req, sizeof(stats_resetCountersReq_t),
		"stats", NFX_NFP_TXRX1, 1, MGMTPLANE,
		&waitStatsTo, &rsp, sizeof(stats_resetCountersRsp_t));

	if ((rc == -1) ||
		(rc < sizeof(stats_resetCountersRsp_t)) ||
		(rsp.status != NFX_OK))
	{
		printf("Stats request failed\n");
		return CMD_ERR_FAILED;
	}

	printf("Counters reset done.\n");
	return CMD_SUCCESS;
}


//------------------------------------------------------------------------------
// Name         : stats_fp_bmc12500
//
// Description  : 
//
// Created by   : Shyh-Pei Yen
//
// Date Created : 10/24/02
//------------------------------------------------------------------------------

DEFUN(stats_fp_bmc12500,
      stats_fp_bmc12500_cmd,
      "stats fp bmc12500",
      STATS_COMMAND_STR
      "FP statistics.\n"
      "FP BMC12500 statistics.\n")
{
	int						rc;
	char					tmpbuf[512];
	stats_GetFPStatsReq_t	req;
	stats_GetFPStatsRsp_t	*rsp = (stats_GetFPStatsRsp_t *) tmpbuf;
	stats_fpBmc12500Stats_t	*pStats;

	// Get FP stats first

	memset((void *) &req,   0, sizeof(stats_GetFPStatsReq_t));
	memset((void *) tmpbuf, 0, sizeof(tmpbuf));

	req.hdr.msgType = STATS_GETSTATS;
	req.hdr.clientPrivate = getpid();
	req.type = STATS_FP_BMC12500;

	rc = sendAgileMsg(-1, &req, sizeof(stats_GetFPStatsReq_t),
		"stats", NFX_NFP_FP2, 1, MGMTPLANE,
		&waitStatsTo, tmpbuf, sizeof(tmpbuf));

	if ((rc == -1) ||
		(rc < sizeof(stats_GetFPStatsRsp_t)) ||
		(rsp->status != NFX_OK))
	{
		printf("Stats request failed\n");
		return CMD_ERR_FAILED;
	}

	pStats = (stats_fpBmc12500Stats_t *) rsp->data;
	printf("  rx bad packets:     %lld\n", pStats->rxBadPkts);
	printf("  rx no buffer errs:  %lld\n", pStats->rxNoBufErrs);
	printf("  rx no sysDesc errs: %lld\n", pStats->rxNoSysDescErrs);
	printf("  tx queue full drops:%lld\n", pStats->txQFullDrops);
	printf("\n");

	return CMD_SUCCESS;
}


//------------------------------------------------------------------------------
// Name         : stats_fp_bmc12500_reset
//
// Description  : 
//
// Created by   : Shyh-Pei Yen
//
// Date Created : 10/24/02
//------------------------------------------------------------------------------

DEFUN(stats_fp_bmc12500_reset,
      stats_fp_bmc12500_reset_cmd,
      "stats fp bmc12500 reset",
      STATS_COMMAND_STR
      "FP statistics.\n"
      "FP BMC12500 statistics.\n"
	  "Reset FP BMC12500 statistics counters.\n")
{
	int							rc;
	stats_resetCountersReq_t	req;
	stats_resetCountersRsp_t	rsp;

	memset((void *) &req, 0, sizeof(stats_resetCountersReq_t));
	memset((void *) &rsp, 0, sizeof(stats_resetCountersRsp_t));

	req.hdr.msgType = STATS_RESETCOUNTERS;
	req.hdr.clientPrivate = getpid();
	req.type = STATS_FP_BMC12500;

	rc = sendAgileMsg(-1, &req, sizeof(stats_resetCountersReq_t),
		"stats", NFX_NFP_FP2, 1, MGMTPLANE,
		&waitStatsTo, &rsp, sizeof(stats_resetCountersRsp_t));

	if ((rc == -1) ||
		(rc < sizeof(stats_resetCountersRsp_t)) ||
		(rsp.status != NFX_OK))
	{
		printf("Stats request failed\n");
		return CMD_ERR_FAILED;
	}

	printf("Counters reset done.\n");
	return CMD_SUCCESS;
}


//------------------------------------------------------------------------------
// Name         : stats_fp_lsi
//
// Description  : 
//
// Created by   : Shyh-Pei Yen
//
// Date Created : 10/29/02
//------------------------------------------------------------------------------

DEFUN(stats_fp_lsi,
      stats_fp_lsi_cmd,
      "stats fp lsi (0|1|2|3)",
      STATS_COMMAND_STR
      "FP statistics.\n"
      "FP LSI statistics.\n"
      "Port number (from 0 to 3).\n")
{
	int						rc;
	char					tmpbuf[512];
	stats_GetFPStatsReq_t	req;
	stats_GetFPStatsRsp_t	*rsp = (stats_GetFPStatsRsp_t *) tmpbuf;
	stats_fpLsiStats_t		*pStats;

	// Get FP stats first

	memset((void *) &req,   0, sizeof(stats_GetFPStatsReq_t));
	memset((void *) tmpbuf, 0, sizeof(tmpbuf));

	req.hdr.msgType = STATS_GETSTATS;
	req.hdr.clientPrivate = getpid();
	req.type = STATS_FP_LSI;
	req.port = *argv[0] - '0';

	rc = sendAgileMsg(-1, &req, sizeof(stats_GetFPStatsReq_t),
		"stats", NFX_NFP_FP2, 1, MGMTPLANE,
		&waitStatsTo, tmpbuf, sizeof(tmpbuf));

	if ((rc == -1) ||
		(rc < sizeof(stats_GetFPStatsRsp_t)) ||
		(rsp->status != NFX_OK))
	{
		printf("Stats request failed\n");
		return CMD_ERR_FAILED;
	}

	pStats = (stats_fpLsiStats_t *) rsp->data;
	printf("Performance:\n");
	printf("  rx bytes:           %lld\n", pStats->rxBytes);
	printf("  rx packets:         %lld\n", pStats->rxPkts);
	printf("  tx bytes:           %lld\n", pStats->txBytes);
	printf("  tx packets:         %lld\n", pStats->txPkts);
	printf("\n");
	printf("Faults:\n");
	printf("  rx drops:           %lld\n", pStats->rxDrops);
	printf("  rx crc errors:      %lld\n", pStats->rxCrcErrs);
	printf("  rx undersized pkts: %lld\n", pStats->rxUndersize);
	printf("  rx symbol errors:   %lld\n", pStats->rxSymbolErrs);
	printf("  tx underflows:      %lld\n", pStats->txUnderflows);
	printf("  tx errors:          %lld\n", pStats->txErrors);
	printf("\n");

	return CMD_SUCCESS;
}


//------------------------------------------------------------------------------
// Name         : stats_fp_lsi_reset
//
// Description  : 
//
// Created by   : Shyh-Pei Yen
//
// Date Created : 10/29/02
//------------------------------------------------------------------------------

DEFUN(stats_fp_lsi_reset,
      stats_fp_lsi_reset_cmd,
      "stats fp lsi (0|1|2|3) reset",
      STATS_COMMAND_STR
      "FP statistics.\n"
      "FP LSI statistics.\n"
      "Port number (from 0 to 3).\n"
	  "Reset FP LSI port statistics counters\n")
{
	int							rc;
	stats_resetCountersReq_t	req;
	stats_resetCountersRsp_t	rsp;

	memset((void *) &req, 0, sizeof(stats_resetCountersReq_t));
	memset((void *) &rsp, 0, sizeof(stats_resetCountersRsp_t));

	req.hdr.msgType = STATS_RESETCOUNTERS;
	req.hdr.clientPrivate = getpid();
	req.type = STATS_FP_LSI;
	req.port = *argv[0] - '0';

	rc = sendAgileMsg(-1, &req, sizeof(stats_resetCountersReq_t),
		"stats", NFX_NFP_FP2, 1, MGMTPLANE,
		&waitStatsTo, &rsp, sizeof(stats_resetCountersRsp_t));

	if ((rc == -1) ||
		(rc < sizeof(stats_resetCountersRsp_t)) ||
		(rsp.status != NFX_OK))
	{
		printf("Stats request failed\n");
		return CMD_ERR_FAILED;
	}

	printf("Counters reset done.\n");
	return CMD_SUCCESS;
}


//------------------------------------------------------------------------------
// Name         : stats_fc_dp83820
//
// Description  : 
//
// Created by   : Shyh-Pei Yen
//
// Date Created : 10/29/02
//------------------------------------------------------------------------------

DEFUN(stats_fc_dp83820,
      stats_fc_dp83820_cmd,
      "stats fc dp83820 (0|1|2|3)",
      STATS_COMMAND_STR
      "FC statistics.\n"
      "FC DP83820 statistics.\n"
	  "FC slice 0\n"
	  "FC slice 1\n"
	  "FC slice 2\n"
	  "FC slice 3\n")
{
	int						rc, slice;
	char					tmpbuf[512];
	stats_GetFCStatsReq_t	req;
	stats_GetFCStatsRsp_t	*rsp = (stats_GetFCStatsRsp_t *) tmpbuf;
	stats_fcDp83820Stats_t	*pStats;

	// Get FC stats first

	memset((void *) &req,   0, sizeof(stats_GetFCStatsReq_t));
	memset((void *) tmpbuf, 0, sizeof(tmpbuf));

	req.hdr.msgType = STATS_GETSTATS;
	req.hdr.clientPrivate = getpid();
	req.type = STATS_FC_DP83820;

	slice = *argv[0] - '0';
	if ((slice < 0) || (slice > 3))
	{
		printf("Invalid slice number!\n");
		return CMD_ERR_FAILED;
	}

	rc = sendAgileMsg(-1, &req, sizeof(stats_GetFCStatsReq_t),
		"stats", slice, NFX_SLOTTYPE_FC, MGMTPLANE,
		&waitStatsTo, tmpbuf, sizeof(tmpbuf));

	if ((rc == -1) ||
		(rc < sizeof(stats_GetFCStatsRsp_t)) ||
		(rsp->status != NFX_OK))
	{
		printf("Stats request failed\n");
		return CMD_ERR_FAILED;
	}

	pStats = (stats_fcDp83820Stats_t *) rsp->data;
	printf("Performance:\n");
	printf("  rx bytes:           %lld\n", pStats->rxBytes);
	printf("  rx packets:         %lld\n", pStats->rxPkts);
	printf("  tx bytes:           %lld\n", pStats->txBytes);
	printf("  tx packets:         %lld\n", pStats->txPkts);
	printf("\n");
	printf("Faults:\n");
	printf("  rx errors:          %lld\n", pStats->rxErrors);
	printf("  rx fcs errors:      %lld\n", pStats->rxFcsErrs);
	printf("  rx overruns:        %lld\n", pStats->rxOverruns);
	printf("  rx align errors:    %lld\n", pStats->rxAlignErrs);
	printf("  rx symbol errors:   %lld\n", pStats->rxSymbolErrs);
	printf("  rx inrange len errs:%lld\n", pStats->rxLenErrs);
	printf("  rx aborts:          %lld\n", pStats->rxAborts);
	printf("  rx no buffer errs:  %lld\n", pStats->rxNoBufErrs);
	printf("  rx no sys desc:     %lld\n", pStats->rxNoSysDescs);
	printf("  rx runt pkts:       %lld\n", pStats->rxRuntPkts);
	printf("  rx crc errors:      %lld\n", pStats->rxCrcErrs);
	printf("  tx queue full drops:%lld\n", pStats->txQFullDrops);
	printf("  tx aborts:          %lld\n", pStats->txAborts);
	printf("  tx underruns:       %lld\n", pStats->txUnderruns);
	printf("\n");

	return CMD_SUCCESS;
}


//------------------------------------------------------------------------------
// Name         : stats_fc_dp83820_reset
//
// Description  : 
//
// Created by   : Shyh-Pei Yen
//
// Date Created : 10/29/02
//------------------------------------------------------------------------------

DEFUN(stats_fc_dp83820_reset,
      stats_fc_dp83820_reset_cmd,
      "stats fc dp83820 (0|1|2|3) reset",
      STATS_COMMAND_STR
      "FC statistics.\n"
      "FC DP83820 statistics.\n"
	  "FC slice 0\n"
	  "FC slice 1\n"
	  "FC slice 2\n"
	  "FC slice 3\n"
	  "Reset dp83820 statistics counters of specified fc slice\n")
{
	int							rc, slice;
	stats_resetCountersReq_t	req;
	stats_resetCountersRsp_t	rsp;

	memset((void *) &req, 0, sizeof(stats_resetCountersReq_t));
	memset((void *) &rsp, 0, sizeof(stats_resetCountersRsp_t));

	req.hdr.msgType = STATS_RESETCOUNTERS;
	req.hdr.clientPrivate = getpid();
	req.type = STATS_FC_DP83820;

	slice = *argv[0] - '0';
	if ((slice < 0) || (slice > 3))
	{
		printf("Invalid slice number!\n");
		return CMD_ERR_FAILED;
	}

	rc = sendAgileMsg(-1, &req, sizeof(stats_resetCountersReq_t),
		"stats", slice, NFX_SLOTTYPE_FC, MGMTPLANE,
		&waitStatsTo, &rsp, sizeof(stats_resetCountersRsp_t));

	if ((rc == -1) ||
		(rc < sizeof(stats_resetCountersRsp_t)) ||
		(rsp.status != NFX_OK))
	{
		printf("Stats request failed\n");
		return CMD_ERR_FAILED;
	}

	printf("Counters reset done.\n");
	return CMD_SUCCESS;
}


//------------------------------------------------------------------------------
// Name         : stats_fc_gt64240mii
//
// Description  : 
//
// Created by   : Shyh-Pei Yen
//
// Date Created : 10/29/02
//------------------------------------------------------------------------------

DEFUN(stats_fc_gt64240mii,
      stats_fc_gt64240mii_cmd,
      "stats fc gt64240mii (0|1|2|3)",
      STATS_COMMAND_STR
      "FC statistics.\n"
      "FC GT64240mii statistics.\n"
	  "FC slice 0\n"
	  "FC slice 1\n"
	  "FC slice 2\n"
	  "FC slice 3\n")
{
	int						rc, slice;
	char					tmpbuf[512];
	stats_GetFCStatsReq_t	req;
	stats_GetFCStatsRsp_t	*rsp = (stats_GetFCStatsRsp_t *) tmpbuf;
	stats_fcGt64240Stats_t	*pStats;

	// Get FP stats first

	memset((void *) &req,   0, sizeof(stats_GetFCStatsReq_t));
	memset((void *) tmpbuf, 0, sizeof(tmpbuf));

	req.hdr.msgType = STATS_GETSTATS;
	req.hdr.clientPrivate = getpid();
	req.type = STATS_FC_GT64240;

	slice = *argv[0] - '0';
	if ((slice < 0) || (slice > 3))
	{
		printf("Invalid slice number!\n");
		return CMD_ERR_FAILED;
	}

	rc = sendAgileMsg(-1, &req, sizeof(stats_GetFCStatsReq_t),
		"stats", slice, NFX_SLOTTYPE_FC, MGMTPLANE,
		&waitStatsTo, tmpbuf, sizeof(tmpbuf));

	if ((rc == -1) ||
		(rc < sizeof(stats_GetFCStatsRsp_t)) ||
		(rsp->status != NFX_OK))
	{
		printf("Stats request failed\n");
		return CMD_ERR_FAILED;
	}

	pStats = (stats_fcGt64240Stats_t *) rsp->data;
	printf("Performance:\n");
	printf("  rx bytes:           %lld\n", pStats->rxBytes);
	printf("  rx packets:         %lld\n", pStats->rxPkts);
	printf("  tx bytes:           %lld\n", pStats->txBytes);
	printf("  tx packets:         %lld\n", pStats->txPkts);
	printf("\n");
	printf("Faults:\n");
	printf("  rx crc errors:      %lld\n", pStats->rxCrcErrs);
	printf("  rx fragments:       %lld\n", pStats->rxFragments);
	printf("  rx jabbers:         %lld\n", pStats->rxJabbers);
	printf("  rx collisions:      %lld\n", pStats->rxCollisions);
	printf("  rx late collisions: %lld\n", pStats->rxLateCollis);
	printf("  rx errors:          %lld\n", pStats->rxErrors);
	printf("  rx undersized pkts: %lld\n", pStats->rxUndersized);
	printf("  rx no sys desc:     %lld\n", pStats->rxNoSysDescs);
	printf("  rx no buffer errs:  %lld\n", pStats->rxNoBufErrs);
	printf("  tx queue full drops:%lld\n", pStats->txQFullDrops);
	printf("  tx underruns:       %lld\n", pStats->txUnderruns);
	printf("\n");

	return CMD_SUCCESS;
}


//------------------------------------------------------------------------------
// Name         : stats_fc_gt64240mii_reset
//
// Description  : 
//
// Created by   : Shyh-Pei Yen
//
// Date Created : 10/29/02
//------------------------------------------------------------------------------

DEFUN(stats_fc_gt64240mii_reset,
      stats_fc_gt64240mii_reset_cmd,
      "stats fc gt64240mii (0|1|2|3) reset",
      STATS_COMMAND_STR
      "FC statistics.\n"
      "FC GT64240mii statistics.\n"
	  "FC slice 0\n"
	  "FC slice 1\n"
	  "FC slice 2\n"
	  "FC slice 3\n"
	  "Reset gt64240mii statistics counters of specified fc slice\n")
{
	int							rc, slice;
	stats_resetCountersReq_t	req;
	stats_resetCountersRsp_t	rsp;

	memset((void *) &req, 0, sizeof(stats_resetCountersReq_t));
	memset((void *) &rsp, 0, sizeof(stats_resetCountersRsp_t));

	req.hdr.msgType = STATS_RESETCOUNTERS;
	req.hdr.clientPrivate = getpid();
	req.type = STATS_FC_GT64240;

	slice = *argv[0] - '0';
	if ((slice < 0) || (slice > 3))
	{
		printf("Invalid slice number!\n");
		return CMD_ERR_FAILED;
	}

	rc = sendAgileMsg(-1, &req, sizeof(stats_resetCountersReq_t),
		"stats", slice, NFX_SLOTTYPE_FC, MGMTPLANE,
		&waitStatsTo, &rsp, sizeof(stats_resetCountersRsp_t));

	if ((rc == -1) ||
		(rc < sizeof(stats_resetCountersRsp_t)) ||
		(rsp.status != NFX_OK))
	{
		printf("Stats request failed\n");
		return CMD_ERR_FAILED;
	}

	printf("Counters reset done.\n");
	return CMD_SUCCESS;
}


//------------------------------------------------------------------------------
// Name         : stats_sscmgmt_bcm5600
//
// Description  : 
//
// Created by   : Shyh-Pei Yen
//
// Date Created : 10/29/02
//------------------------------------------------------------------------------

DEFUN(stats_sscmgmt_bcm5600,
      stats_sscmgmt_bcm5600_cmd,
      "stats sscmgmt bcm5600 (fc0|fc1|fc2|fc3|txrx|ssc)",
      STATS_COMMAND_STR
      "SSC(Toshiba) statistics.\n"
      "SSC(Toshiba) BCM5600 statistics.\n"
	  "FC slice 0.\n"
	  "FC slice 1.\n"
	  "FC slice 2.\n"
	  "FC slice 3.\n"
	  "TXRX.\n"
	  "SSC.\n")
{
	int						i, rc;
	char					tmpbuf[512];
	stats_GetSscMgmtStatsReq_t	req;
	stats_GetSscMgmtStatsRsp_t	*rsp = (stats_GetSscMgmtStatsRsp_t *) tmpbuf;
	stats_sscmgmtBcm5600Stats_t	*pStats;

	// Get FP stats first

	memset((void *) &req,   0, sizeof(stats_GetSscMgmtStatsReq_t));
	memset((void *) tmpbuf, 0, sizeof(tmpbuf));

	req.hdr.msgType = STATS_GETSTATS;
	req.hdr.clientPrivate = getpid();
	req.type = STATS_SSCMGMT_BCM5600;

	for (i = 0; i < STATS_SSCMGMT_TOTAL_PORTS; i++)
		if (strcmp(argv[0], sscmgmtPort[i].name) == 0)
			break;

	if (i >= STATS_SSCMGMT_TOTAL_PORTS)
	{
		printf("Invalid port specified.\n");
		return CMD_ERR_FAILED;
	}
	else
		req.port = sscmgmtPort[i].num;

	rc = sendAgileMsg(-1, &req, sizeof(stats_GetSscMgmtStatsReq_t),
		"stats", NFX_SSC_MGMT, NFX_SLOTTYPE_SSC, MGMTPLANE,
		&waitStatsTo, tmpbuf, sizeof(tmpbuf));

	if ((rc == -1) ||
		(rc < sizeof(stats_GetSscMgmtStatsRsp_t)) ||
		(rsp->status != NFX_OK))
	{
		printf("Stats request failed\n");
		return CMD_ERR_FAILED;
	}

	pStats = (stats_sscmgmtBcm5600Stats_t *) rsp->data;
	printf("Performance:\n");
	printf("  rx bytes:           %lld\n", pStats->rxBytes);
	printf("  rx packets:         %lld\n", pStats->rxPkts);
	printf("  tx bytes:           %lld\n", pStats->txBytes);
	printf("  tx packets:         %lld\n", pStats->txPkts);
	printf("\n");
	printf("Faults:\n");
	printf("  rx fcs errors:      %lld\n", pStats->rxFcsErrs);
	printf("  rx alignments:      %lld\n", pStats->rxAlignErrs);
	printf("  rx symbol errors:   %lld\n", pStats->rxSymbolErrs);
	printf("  rx false carriers:  %lld\n", pStats->rxFalseCarrs);
	printf("  rx undersized pkts: %lld\n", pStats->rxUndersized);
	printf("  rx fragments:       %lld\n", pStats->rxFragments);
	printf("  rx jabbers:         %lld\n", pStats->rxJabbers);
	printf("  rx discards:        %lld\n", pStats->rxDiscards);
	printf("  tx fcs errors:      %lld\n", pStats->txFcsErrs);
	printf("  tx aborts:          %lld\n", pStats->txAborts);
	printf("  tx fragments:       %lld\n", pStats->txFragments);
	printf("  tx jabbers:         %lld\n", pStats->txJabbers);
	printf("\n");

	return CMD_SUCCESS;
}


//------------------------------------------------------------------------------
// Name         : stats_sscmgmt_bcm5600_reset
//
// Description  : 
//
// Created by   : Shyh-Pei Yen
//
// Date Created : 10/29/02
//------------------------------------------------------------------------------

DEFUN(stats_sscmgmt_bcm5600_reset,
      stats_sscmgmt_bcm5600_reset_cmd,
      "stats sscmgmt bcm5600 (fc0|fc1|fc2|fc3|txrx|ssc) reset",
      STATS_COMMAND_STR
      "SSC(Toshiba) statistics.\n"
      "SSC(Toshiba) BCM5600 statistics.\n"
	  "FC slice 0.\n"
	  "FC slice 1.\n"
	  "FC slice 2.\n"
	  "FC slice 3.\n"
	  "TXRX.\n"
	  "SSC.\n"
	  "Reset SSC(Toshiba) BCM5600 statistics counters for the specified port.\n")
{
	int							i, rc;
	stats_resetCountersReq_t	req;
	stats_resetCountersRsp_t	rsp;

	memset((void *) &req, 0, sizeof(stats_resetCountersReq_t));
	memset((void *) &rsp, 0, sizeof(stats_resetCountersRsp_t));

	req.hdr.msgType = STATS_RESETCOUNTERS;
	req.hdr.clientPrivate = getpid();
	req.type = STATS_SSCMGMT_BCM5600;

	for (i = 0; i < STATS_SSCMGMT_TOTAL_PORTS; i++)
		if (strcmp(argv[0], sscmgmtPort[i].name) == 0)
			break;

	if (i >= STATS_SSCMGMT_TOTAL_PORTS)
	{
		printf("Invalid port specified.\n");
		return CMD_ERR_FAILED;
	}
	else
		req.port = sscmgmtPort[i].num;

	rc = sendAgileMsg(-1, &req, sizeof(stats_resetCountersReq_t),
		"stats", NFX_SSC_MGMT, NFX_SLOTTYPE_SSC, MGMTPLANE,
		&waitStatsTo, &rsp, sizeof(stats_resetCountersRsp_t));

	if ((rc == -1) ||
		(rc < sizeof(stats_resetCountersRsp_t)) ||
		(rsp.status != NFX_OK))
	{
		printf("Stats request failed\n");
		return CMD_ERR_FAILED;
	}

	printf("Counters reset done.\n");
	return CMD_SUCCESS;
}


void
install_stats(priv_t myPriv, enum node_type node)
{
    switch (myPriv)
    {
        case diagPriv:
             install_element(node, &stats_txrx_bmc12500_cmd);
             install_element(node, &stats_txrx_bmc12500_reset_cmd);
             install_element(node, &stats_txrx_lsi_cmd);
             install_element(node, &stats_txrx_lsi_reset_cmd);
             install_element(node, &stats_txrx_dp83820_cmd);
             install_element(node, &stats_txrx_dp83820_reset_cmd);
             install_element(node, &stats_fp_bmc12500_cmd);
             install_element(node, &stats_fp_bmc12500_reset_cmd);
             install_element(node, &stats_fp_lsi_cmd);
             install_element(node, &stats_fp_lsi_reset_cmd);
             install_element(node, &stats_fc_dp83820_cmd);
             install_element(node, &stats_fc_dp83820_reset_cmd);
             install_element(node, &stats_fc_gt64240mii_cmd);
             install_element(node, &stats_fc_gt64240mii_reset_cmd);
             install_element(node, &stats_sscmgmt_bcm5600_cmd);
             install_element(node, &stats_sscmgmt_bcm5600_reset_cmd);
        case fullPriv:
        case changePriv:
        case addPriv:
        case viewPriv:
    }
    return;
}
