/*
 *-----------------------------------------------------------------
 *
 * Name: FP Hardware System Health Monitor 
 *
 * RCS:  $Id:
 *
 * Copyright (C) 2002, Agile Storage, Inc.
 *
 * Description: Connects to FP card using rcon and collects hardware
 * 							statistics.
 *
 * Created by:	Christa Porter
 *
 * Date Created: 20 Sep 2002
 *
 *-----------------------------------------------------------------
 */

#include "sysHealth.h"

#define NUMRXTXCMDS 13
#define NUMFPCMDS 15


int sysHealthFP(void) {

	FILE *fpout;
	char inStr[MAXLINE];
	char parseStr[MAXLINE];
	char *charP;
	char *endP;
	int done, first, TxRx, FP;

	/* FP register variables */
	unsigned int bus_l2_errors;
	unsigned char l2_cor_d_ecc;
	unsigned char l2_bad_d_ecc;
	unsigned char l2_cor_t_ecc;
	unsigned char l2_bad_t_ecc;
	unsigned int bus_mem_io_errors;
	unsigned char mem_cor_d_ecc;
	unsigned char mem_bad_d_ecc;
	unsigned char bus_error;
	unsigned int bus_err_status;
	unsigned int tid;
	unsigned char rid;
	unsigned char dcode;
	unsigned char mult_errors;

	/* FP PCI variables */
	unsigned int PCICommStatusReg;
	unsigned char MstrDParErr;
	unsigned char SigdTgtAbort;
	unsigned char RcvdTgtAbort;
	unsigned char RcvdMstrAbort;
	unsigned char SigdSerr;
	unsigned char DetParErr;
	unsigned int PCIAddStatCommReg;
	unsigned char serr_det;
	unsigned char TrdyErr;
	unsigned char RetryErr;
	unsigned int LDTErrStatCntlReg;
	unsigned char ProtoErr;
	unsigned char OvfErr;
	unsigned char EocNxaErr;
	unsigned char SrcTagErr;
	unsigned char MapNxaError;

	int i, slot, port, rc;
	
	int byteMask[] = {
		0x000000ff,
		0x0000ff00,
		0x00ff0000,
		0xff000000
	};
	
	enum fpCmds_e {
		bus_err_status_c,				/* bus watcher error status register */
		bus_l2_errors_c,				/* bus watcher L2 ECC counter register*/
		bus_mem_io_errors_c,		/* bus watcher memory & I/O error counter register */ 
		pci_conf_status_c,			/* PCI status in configuration header */
		pci_add_status_c,				/* PCI additional status in configuration header */
		ldt_err_status_c,				/* LDT error status register */
		bmc12500_0_c,						/* SiByte diags on FIFO port 0 */
		bmc12500_2_c,						/* SiByte diags on FIFO port 2 */
		dp83820_c,							/* National MAC diags */
		lsi_0_c,								/* LSI MAC diags port 0 */
		lsi_1_c,								/* LSI MAC diags port 1 */
		lsi_2_c,								/* LSI MAC diags port 2 */
		lsi_3_c,								/* LSI MAC diags port 3 */
		lsi_4_c,								/* LSI MAC diags port 4 FP SiByte only */
		lsi_5_c									/* LSI MAC diags port 5 FP SiByte only */
	};

	char *fpCmds[] = {
		"dm 0xb0020880 1\n",			/* bus watcher error status register */
		"dm 0xb00208c0 1\n",			/* bus watcher L2 ECC counter register*/
		"dm 0xb00208c8 1\n",			/* bus watcher memory & I/O error counter register */ 
		"pciread 0 0 0 4\n",			/* PCI status in configuration header */
		"pciread 0 0 0 88\n",			/* PCI additional status in configuration header */
		"pciread 0 1 0 68\n",			/* LDT error status register */
		"bmc12500\n0\n2\n999\n",		/* SiByte diags on FIFO port 0 */
		"bmc12500\n2\n2\n999\n",		/* SiByte diags on FIFO port 0 */
		"dp83820\n2\n99\n",				/* National MAC diags */
		"lsi\n0\n4\n99\n",				/* LSI MAC diags port 0 */
		"lsi\n1\n4\n99\n",				/* LSI MAC diags port 1 */
		"lsi\n2\n4\n99\n",				/* LSI MAC diags port 2 */
		"lsi\n3\n4\n99\n",				/* LSI MAC diags port 3 */
		"lsi\n4\n4\n99\n",				/* LSI MAC diags port 4 FP SiByte only */
		"lsi\n5\n4\n99\n"					/* LSI MAC diags port 5 FP SiByte only */
	};

	if((fpout = fopen(FPFILE, "w")) == NULL) {
		printf("Cannot open %s\n", FPFILE);
		exit(1);
	}

	/* FP statics collection */
	fprintf(fpout, "****************************************************************\n");
	fprintf(fpout, "*************** NFX System Harware Health Report ***************\n");
	fprintf(fpout, "****************************************************************\n");
	fprintf(fpout, "*************************** FP Log  ****************************\n");
	fprintf(fpout, "****************************************************************\n");
	/* The FP SiBytes are slice 1, ports 0 & 3 */
	slot = 1;
	for(port=0; port<4; port+=3) {
		if(port)
			if(establish_rcon_session (slot, port)) {
				fprintf(fpout, "Connection to FP SiByte on FP card, slot %d, port %d failed\n", slot, port);
				fprintf(stderr, "Connection to FP SiByte on FP card, slot %d, port %d failed\n", slot, port);
				exit (1);
			} else {
				fprintf(fpout, "\nConnecting to FP SiByte on FP card, slot %d, port %d\n\n", slot, port);
				fprintf(stdout, "\nConnecting to FP SiByte on FP card, slot %d, port %d\n\n", slot, port);
				FP = 1;
				TxRx = 0;
			}
		else
			if(establish_rcon_session (slot, port)) {
				fprintf(stderr, "Connection to Rx/Tx SiByte on FP card, slot %d, port %d failed\n", slot, port);
				exit (1);
			} else {
				fprintf(fpout, "\nConnecting to Rx/Tx SiByte on FP card, slot %d, port %d\n\n", slot, port);
				fprintf(stdout, "\nConnecting to Rx/Tx SiByte on FP card, slot %d, port %d\n\n", slot, port);
				FP = 0;
				TxRx = 1;
			}

		/* process FP command file, more LSI MACs on FP SiByte */
		for(i=0; i<(port ? NUMFPCMDS : NUMRXTXCMDS); i++) {
			/* check for PCI commands on FP SiByte and skip */
			if(!strncmp(fpCmds[i], "pciread", 7) && FP)
				continue;
			/* check for National MAC commands on FP SiByte and skip */
			if(!strncmp(fpCmds[i], "dp83820", 7) && FP)
				continue;
			/* output command using rcon function */
			fprintf(fpout, "Sending FP Command: %s", fpCmds[i]);
			fprintf(stdout, "Sending FP Command: %s", fpCmds[i]);
			if(send_to_rcon (fpCmds[i])) {
				fprintf (fpout, "RCON send to SiByte on FP card, slot %d, port %d failed\n", slot, port);
				fprintf (stderr, "RCON send to SiByte on FP card, slot %d, port %d failed\n", slot, port);
				shutdown_rcon_session();
				exit (1);
			}

			done = 0;
			first = 1;
			/* Get RCON command response and process */
			while((rc = get_from_rcon (inStr, MAXLINE, 5)) > 0) {
#ifdef DEBUG
				fprintf(stdout, "Got response: %s\n", inStr);
#endif

				/* make copy for parsing */
				strcpy(parseStr, inStr);
				charP = strtok(parseStr, " \t\n");
#ifdef DEBUG
				fprintf(stdout, "charP = %s\n", charP);
#endif

				if(!done) {
					/* main case for parsing each FP command */
					switch(i) {
					case bus_err_status_c:
						/* throw out null strings & rcon prompts */
						if(!charP || !strncmp(charP, "RCON", 4)) break;
						/* throw away echos & addresses from dm command */
						if(!strncmp(charP, "dm", 2) || !strncmp(charP, "0x", 2)) break;
						fprintf(fpout, "FP Bus Watcher Error Status Register value: %s\n", charP);
						fprintf(stdout, "FP Bus Watcher Error Status Register value: %s\n", charP);
						if(sizeof(charP) == 4) {
							bus_err_status = (unsigned int)strtoul(charP, &endP, 16);
						} else {
							fprintf(fpout, "Error - Bad FP Bus Watcher Error Status Register value returned\n");
							fprintf(stderr, "Error - Bad FP Bus Watcher Error Status Register value returned\n");
						}
						tid =  (bus_err_status & 0x3FF00) >> 8;
						fprintf(fpout, "Transfer ID that had error code: %d\n", tid);
						rid =  (bus_err_status & 0x3C0000) >> 18;
						fprintf(fpout, "Responder ID that reported the error: %d\n", rid);
						dcode =  (bus_err_status & 0x1C00000) >> 22;
						fprintf(fpout, "Data Transfer error code: %d\n", dcode);
						mult_errors =  (bus_err_status & 0x40000000) >> 30;
						fprintf(fpout, "Multiple errors: %d\n", mult_errors);
						fprintf(fpout, "\n");
						done = 1;
						break;
					case bus_l2_errors_c:
						/* throw out null strings & rcon prompts */
						if(!charP || !strncmp(charP, "RCON", 4)) break;
						/* throw away echos & addresses from dm command */
						if(!strncmp(charP, "dm", 2) || !strncmp(charP, "0x", 2)) break;
						fprintf(fpout, "FP Bus Watcher L2 ECC Counter Register value: %s\n", charP);
						fprintf(stdout, "FP Bus Watcher L2 ECC Counter Register value: %s\n", charP);
						if(sizeof(charP) == 4) {
							bus_l2_errors = (unsigned int)strtoul(charP, &endP, 16);
						} else {
							fprintf(fpout, "Error - Bad FP Bus Watcher L2 ECC Counter Register value returned\n");
							fprintf(stderr, "Error - Bad FP Bus Watcher L2 ECC Counter Register value returned\n");
						}
						l2_cor_d_ecc =  (bus_l2_errors & byteMask[0]);
						fprintf(fpout, "Correctable L2 cache data errors: %d\n", l2_cor_d_ecc);
						l2_bad_d_ecc =  (bus_l2_errors & byteMask[1]) >> 8;
						fprintf(fpout, "Uncorrectable L2 cache data errors: %d\n", l2_bad_d_ecc);
						l2_cor_t_ecc =  (bus_l2_errors & byteMask[2]) >> 16;
						fprintf(fpout, "Correctable L2 cache tag errors: %d\n", l2_cor_t_ecc);
						l2_bad_t_ecc =  (bus_l2_errors & byteMask[3]) >> 24;
						fprintf(fpout, "Uncorrectable L2 cache tag errors: %d\n", l2_bad_t_ecc);
						fprintf(fpout, "\n");
						done = 1;
						break;
					case bus_mem_io_errors_c:
						/* throw out null strings & rcon prompts */
						if(!charP || !strncmp(charP, "RCON", 4)) break;
						/* throw away echos & addresses from dm command */
						if(!strncmp(charP, "dm", 2) || !strncmp(charP, "0x", 2)) break;
						fprintf(fpout, "FP Bus Watcher Memory & I/O Error Counter Register value: %s\n", charP);
						fprintf(stdout, "FP Bus Watcher Memory & I/O Error Counter Register value: %s\n", charP);
						if(sizeof(charP) == 4) {
							bus_mem_io_errors = (unsigned int)strtoul(charP, &endP, 16);
						} else {
							fprintf(fpout, "Error - Bad FP Bus Watcher Memory & I/O Error Counter Register value returned\n");
							fprintf(stderr, "Error - Bad FP Bus Watcher Memory & I/O Error Counter Register value returned\n");
						}
						mem_cor_d_ecc =  (bus_mem_io_errors & byteMask[0]);
						fprintf(fpout, "Correctable memory data errors: %d\n", mem_cor_d_ecc);
						mem_bad_d_ecc =  (bus_mem_io_errors & byteMask[1]) >> 8;
						fprintf(fpout, "Uncorrectable memory data errors: %d\n", mem_bad_d_ecc);
						bus_error =  (bus_mem_io_errors & byteMask[2]) >> 16;
						fprintf(fpout, "Bus errors & fatal bus errors: %d\n", bus_error);
						fprintf(fpout, "\n");
						done = 1;
						break;
					case pci_conf_status_c:
						/* throw out null strings, rcon prompts & echos */
						if(!charP || !strncmp(charP, "RCON", 4) || !strncmp(charP, "pciread", 7)) break;
						charP = strtok(NULL, " \t\n"); /* get the "reg" */
						charP = strtok(NULL, " \t\n"); /* get the "=" */
						charP = strtok(NULL, "x"); /* get the "0x" */
						charP = strtok(NULL, " \t\n"); /* get the register value */
						fprintf(fpout, "FP PCI configuration header status/command register value: %s\n", charP);
						fprintf(stdout, "FP PCI configuration header status/command register value: %s\n", charP);
						if(sizeof(charP) == 4) {
							PCICommStatusReg = (unsigned int)strtoul(charP, &endP, 16);
						} else {
							fprintf(fpout, "Error - Bad FP PCI configuration header status/command register value returned\n");
							fprintf(stderr, "Error - Bad FP PCI configuration header status/command register value returned\n");
						}
						MstrDParErr =  (PCICommStatusReg & 0x01000000) >> 24;
						fprintf(fpout, "Data parity error as master: %d\n", MstrDParErr);
						SigdTgtAbort =  (PCICommStatusReg & 0x08000000) >> 27;
						fprintf(fpout, "Signaled target abort as target: %d\n", SigdTgtAbort);
						RcvdTgtAbort =  (PCICommStatusReg & 0x10000000) >> 28;
						fprintf(fpout, "Received target abort as master: %d\n", RcvdTgtAbort);
						RcvdMstrAbort =  (PCICommStatusReg & 0x20000000) >> 29;
						fprintf(fpout, "Signaled master abort as master: %d\n", RcvdMstrAbort);
						SigdSerr =  (PCICommStatusReg & 0x40000000) >> 30;
						fprintf(fpout, "System error (parity error on address): %d\n", SigdSerr);
						DetParErr =  (PCICommStatusReg & 0x80000000) >> 31;
						fprintf(fpout, "Detected parity error: %d\n", DetParErr);
						fprintf(fpout, "\n");
						done = 1;
						break;
					case pci_add_status_c:
						/* throw out null strings, rcon prompts & echos */
						if(!charP || !strncmp(charP, "RCON", 4) || !strncmp(charP, "pciread", 7)) break;
						charP = strtok(NULL, " \t\n"); /* get the "reg" */
						charP = strtok(NULL, " \t\n"); /* get the "=" */
						charP = strtok(NULL, "x"); /* get the "0x" */
						charP = strtok(NULL, " \t\n"); /* get the register value */
						fprintf(fpout, "FP PCI configuration header additional status/command register value: %s\n", charP);
						fprintf(stdout, "FP PCI configuration header additional status/command register value: %s\n", charP);
						if(sizeof(charP) == 4) {
							PCIAddStatCommReg = (unsigned int)strtoul(charP, &endP, 16);
						} else {
							fprintf(fpout, "Error - Bad FP PCI configuration header additional status/command register value returned\n");
							fprintf(stderr, "Error - Bad FP PCI configuration header additional status/command register value returned\n");
						}
						serr_det =  (PCIAddStatCommReg & 0x00000002) >> 1;
						fprintf(fpout, "SERR from other device: %d\n", serr_det);
						TrdyErr =  (PCIAddStatCommReg & 0x00000004) >> 2;
						fprintf(fpout, "Aborted transfer due to TRDY timeout: %d\n", TrdyErr);
						RetryErr =  (PCIAddStatCommReg & 0x00000008) >> 3;
						fprintf(fpout, "Aborted transfer due to read retry timeout: %d\n", RetryErr);
						fprintf(fpout, "\n");
						done = 1;
						break;
					case ldt_err_status_c:
						/* throw out null strings, rcon prompts & echos */
						if(!charP || !strncmp(charP, "RCON", 4) || !strncmp(charP, "pciread", 7)) break;
						charP = strtok(NULL, " \t\n"); /* get the "reg" */
						charP = strtok(NULL, " \t\n"); /* get the "=" */
						charP = strtok(NULL, "x"); /* get the "0x" */
						charP = strtok(NULL, " \t\n"); /* get the register value */
						fprintf(fpout, "FP LDT configuration header status/command register value: %s\n", charP);
						fprintf(stdout, "FP LDT configuration header status/command register value: %s\n", charP);
						if(sizeof(charP) == 4) {
							LDTErrStatCntlReg = (unsigned int)strtoul(charP, &endP, 16);
						} else {
							fprintf(fpout, "Error - Bad FP LDT configuration header status/command register value returned\n");
							fprintf(stderr, "Error - Bad FP LDT configuration header status/command register value returned\n");
						}
						ProtoErr =  (LDTErrStatCntlReg & 0x00000001);
						fprintf(fpout, "Protocol error on incoming link: %d\n", ProtoErr);
						OvfErr =  (LDTErrStatCntlReg & 0x00000002) >> 1;
						fprintf(fpout, "Receive FIFO overflow: %d\n", OvfErr);
						EocNxaErr =  (LDTErrStatCntlReg & 0x00000004) >> 2;
						fprintf(fpout, "Received non existent address error(End Of Chain): %d\n", EocNxaErr);
						SrcTagErr =  (LDTErrStatCntlReg & 0x00000008) >> 3;
						fprintf(fpout, "Response source ID does not match outstanding request: %d\n", SrcTagErr);
						MapNxaError =  (LDTErrStatCntlReg & 0x00000010) >> 4;
						fprintf(fpout, "Received non existent address error(mapping problem): %d\n", MapNxaError);
						fprintf(fpout, "\n");
						done = 1;
						break;
					case bmc12500_0_c:
						if(first) {
							fprintf(fpout, "Collecting bmc12500 port 0 diag output\n");
							fprintf(stdout, "Collecting bmc12500 port 0 diag output\n");
							first = 0;
						}
						/* throw out null strings & rcon prompts */
						if(!charP || !strncmp(charP, "RCON", 4)) break;
						fprintf(fpout, "%s", inStr);
						fprintf(stdout, "%s", inStr);
						break;
					case bmc12500_2_c:
						if(first) {
							fprintf(fpout, "Collecting bmc12500 port 2 diag output\n");
							fprintf(stdout, "Collecting bmc12500 port 2 diag output\n");
							first = 0;
						}
						/* throw out null strings & rcon prompts */
						if(!charP || !strncmp(charP, "RCON", 4)) break;
						fprintf(fpout, "%s", inStr);
						fprintf(stdout, "%s", inStr);
						break;
					case dp83820_c:
						if(first) {
							fprintf(fpout, "Collecting National MAC diag output\n");
							fprintf(stdout, "Collecting National MAC diag output\n");
							first = 0;
						}
						/* throw out null strings & rcon prompts */
						if(!charP || !strncmp(charP, "RCON", 4)) break;
						fprintf(fpout, "%s", inStr);
						fprintf(stdout, "%s", inStr);
						break;
					case lsi_0_c:
						if(first) {
							fprintf(fpout, "Collecting LSI MAC port 0 diag output\n");
							fprintf(stdout, "Collecting LSI MAC port 0 diag output\n");
							first = 0;
						}
						/* throw out null strings & rcon prompts */
						if(!charP || !strncmp(charP, "RCON", 4)) break;
						fprintf(fpout, "%s", inStr);
						fprintf(stdout, "%s", inStr);
						break;
					case lsi_1_c:
						if(first) {
							fprintf(fpout, "Collecting LSI MAC port 1 diag output\n");
							fprintf(stdout, "Collecting LSI MAC port 1 diag output\n");
							first = 0;
						}
						/* throw out null strings & rcon prompts */
						if(!charP || !strncmp(charP, "RCON", 4)) break;
						fprintf(fpout, "%s", inStr);
						fprintf(stdout, "%s", inStr);
						break;
					case lsi_2_c:
						if(first) {
							fprintf(fpout, "Collecting LSI MAC port 2 diag output\n");
							fprintf(stdout, "Collecting LSI MAC port 2 diag output\n");
							first = 0;
						}
						/* throw out null strings & rcon prompts */
						if(!charP || !strncmp(charP, "RCON", 4)) break;
						fprintf(fpout, "%s", inStr);
						fprintf(stdout, "%s", inStr);
						break;
					case lsi_3_c:
						if(first) {
							fprintf(fpout, "Collecting LSI MAC port 3 diag output\n");
							fprintf(stdout, "Collecting LSI MAC port 3 diag output\n");
							first = 0;
						}
						/* throw out null strings & rcon prompts */
						if(!charP || !strncmp(charP, "RCON", 4)) break;
						fprintf(fpout, "%s", inStr);
						fprintf(stdout, "%s", inStr);
						break;
					case lsi_4_c:
						if(first) {
							fprintf(fpout, "Collecting LSI MAC port 4 diag output\n");
							fprintf(stdout, "Collecting LSI MAC port 4 diag output\n");
							first = 0;
						}
						/* throw out null strings & rcon prompts */
						if(!charP || !strncmp(charP, "RCON", 4)) break;
						fprintf(fpout, "%s", inStr);
						fprintf(stdout, "%s", inStr);
						break;
					case lsi_5_c:
						if(first) {
							fprintf(fpout, "Collecting LSI MAC port 5 diag output\n");
							fprintf(stdout, "Collecting LSI MAC port 5 diag output\n");
							first = 0;
						}
						/* throw out null strings & rcon prompts */
						if(!charP || !strncmp(charP, "RCON", 4)) break;
						fprintf(fpout, "%s", inStr);
						fprintf(stdout, "%s", inStr);
						break;
					default:
						printf("Error - command index out of bounds\n");
						break;
					}
				}
			}
			
		}
		shutdown_rcon_session();
	}

	fclose(fpout);
	return(0);
}
