/*
 *-----------------------------------------------------------------
 *
 * Name: SP Hardware System Health Monitor 
 *
 * RCS:  $Id:
 *
 * Copyright (C) 2002, Agile Storage, Inc.
 *
 * Description: Connects to SP card using rcon and collects hardware
 * 							statistics.
 *
 * Created by:	Christa Porter
 *
 * Date Created: 20 Sep 2002
 *
 *-----------------------------------------------------------------
 */

#include "sysHealth.h"

#define NUMSPCMDS 23

int sysHealthSP(void) {

	FILE *spout;
	char inStr[MAXLINE];
	char parseStr[MAXLINE];
	char *charP;
	char *endP;
	int done, first;

	/* Galileo register variables */
	//unsigned int errAddrLow;
	//unsigned int errAddrHigh;
	//unsigned int errDataLow;
	//unsigned int errDataHigh;
	//unsigned int errParity;

	unsigned int errCause;
	unsigned char addrOut;
	unsigned char addrPErr;
	unsigned char TTErr;
	unsigned char accErr;
	unsigned char wrErr;
	unsigned char cacheErr;
	unsigned char wrDataPErr;
	unsigned char rdDataPErr;


  /*  SDRAM register variables */
	//unsigned int eccCounter;

	/* PCI register variables */
	unsigned int pci0Intr;
	unsigned int pci1Intr;

	//unsigned int pci0ErrAddrLow;
	//unsigned int pci0ErrAddrHigh;
	//unsigned int pci0ErrDataLow;
	//unsigned int pci0ErrDataHigh;
	//unsigned int pci0ErrDataComm;

	//unsigned int pci1ErrAddrLow;
	//unsigned int pci1ErrAddrHigh;
	//unsigned int pci1ErrDataLow;
	//unsigned int pci1ErrDataHigh;
	//unsigned int pci1ErrDataComm;


	int i, slot, slice, rc;
	
	//int byteMask[] = {
		//0x000000ff,
		//0x0000ff00,
		//0x00ff0000,
		//0xff000000
	//};
	
	enum SpCmds_e {
		cpu_err_addr_low_c = 0,	
		cpu_err_addr_high_c = 1,
		cpu_err_data_low_c = 2,
		cpu_err_data_high_c = 3,
		cpu_err_parity_c = 4,
		cpu_err_cause_c = 5,

		sdram_ecc_counter_c = 6,

		pci0_intr_cause_c = 7,
		pci0_err_addr_low_c = 8,
		pci0_err_addr_high_c = 9,
		pci0_err_data_low_c = 10,
		pci0_err_data_high_c = 11,
		pci0_err_comm_c = 12,

		pci1_intr_cause_c = 13,
		pci1_err_addr_low_c = 14,
		pci1_err_addr_high_c = 15,
		pci1_err_data_low_c = 16,
		pci1_err_data_high_c = 17,
		pci1_err_comm_c = 18,

		dp83820_c = 19,
		gtdiag_1_c = 20,
		gtdiag_2_c = 21,
		gtdiag_3_c = 22

	};

	char *spCmds[] = {
		"dm 0xb4000070 1\n",			/* CPU error address LOW */
		"dm 0xb4000078 1\n",			/* CPU error address HIGH */
		"dm 0xb4000128 1\n",			/* CPU error data LOW */
		"dm 0xb4000130 1\n",			/* CPU error data HIGH */
		"dm 0xb4000138 1\n",			/* CPU error parity */
		"dm 0xb4000140 1\n",			/* CPU error cause */

		"dm 0xb4000498 1\n",			/* SDRAM ECC Counter */

		"dm 0xb4001d58 1\n",      /* PCI0 Interrupt Cause */
		"dm 0xb4001d40 1\n",      /* PCI0 Error Address LOW */
		"dm 0xb4001d44 1\n",      /* PCI0 Error Address HIGH */
		"dm 0xb4001d48 1\n",      /* PCI0 Error Data LOW */
		"dm 0xb4001d4c 1\n",      /* PCI0 Error Data HIGH */
		"dm 0xb4001d50 1\n",      /* PCI0 Error Command */

		"dm 0xb4001dd8 1\n",      /* PCI1 Interrupt Cause */
		"dm 0xb4001dc0 1\n",      /* PCI1 Error Address LOW */
		"dm 0xb4001dc4 1\n",      /* PCI1 Error Address HIGH */
		"dm 0xb4001dc8 1\n",      /* PCI1 Error Data LOW */
		"dm 0xb4001dcc 1\n",      /* PCI1 Error Data HIGH */
		"dm 0xb4001dd0 1\n",      /* PCI1 Error Command */

		"dp83820\n2\n99\n",				/* National MAC diags */
		"gtdiag\n1\n2\n999\n",    /* Galileo Ethernet Stats. Mac 1 */ 
		"gtdiag\n2\n2\n999\n",    /* Galileo Ethernet Stats. Mac 2 */ 
		"gtdiag\n3\n2\n999\n",    /* Galileo Ethernet Stats. Mac 3 */ 
	};

	if((spout = fopen(SPFILE, "w")) == NULL) {
		printf("Cannot open %s\n", SPFILE);
		exit(1);
	}

	/* SP statics collection */
	fprintf(spout, "****************************************************************\n");
	fprintf(spout, "*************** NFX System Harware Health Report ***************\n");
	fprintf(spout, "****************************************************************\n");
	fprintf(spout, "*************************** SP Log  ****************************\n");
	fprintf(spout, "****************************************************************\n");

	/* There are 4 slices, 0 to 3 and and slot number is 2 */
	slot = 2;
	/* connect to individual slices and issue commands*/
	for(slice=0; slice<4; slice++) {
		if(establish_rcon_session (slot, slice)) {
			fprintf(spout, "Connection to SP card, slot %d, slice %d failed\n", slot, slice);
			fprintf(stderr, "Connection to SP card, slot %d, slice %d failed\n", slot, slice);
			exit (1);
		} else {
			fprintf(spout, "\n\nConnecting to SP card, slot %d, slice %d\n\n", slot, slice);
			fprintf(stdout, "\n\nConnecting to SP card, slot %d, slice %d\n\n", slot, slice);
		}

		/* issue commands specific to a slice */
		/* process SP commands common to all the slices */
		for(i=0; i< NUMSPCMDS; i++) {
			/* output command using rcon function */
			fprintf(spout, "Sending SP Command: %s", spCmds[i]);
			fprintf(stdout, "Sending SP Command: %s", spCmds[i]);
			if(send_to_rcon (spCmds[i])) {
				fprintf (spout, "RCON send to SP card, slot %d, slice %d failed\n", slot, slice);
				fprintf (stderr, "RCON send to SP card, slot %d, slice %d failed\n", slot, slice);
				shutdown_rcon_session();
				exit (1);
			}

			done = 0;
			first = 1;
			/* Get RCON command response and process */
			while((rc = get_from_rcon (inStr, MAXLINE, 5)) > 0) {
#ifdef DEBUG
				fprintf(stdout, "Got response: %s\n", inStr);
#endif

				/* make copy for parsing */
				strcpy(parseStr, inStr);
				charP = strtok(parseStr, " \t\n");
#ifdef DEBUG
				fprintf(stdout, "charP = %s\n", charP);
#endif

				if(!done) {
					/* main case for parsing each FP command */
					switch(i) {
					case cpu_err_addr_low_c:
						/* throw out null strings & rcon prompts */
						if(!charP || !strncmp(charP, "RCON", 4)) break;
						/* throw away echos & addresses from dm command */
						if(!strncmp(charP, "dm", 2) || !strncmp(charP, "0x", 2)) break;
						fprintf(spout, "SP CPU Error Address (low) Register value: %s\n", charP);
						fprintf(stdout, "SP CPU Error Address (low) Register value: %s\n", charP);
						fprintf(spout, "\n");
						done = 1;
						break;

					case cpu_err_addr_high_c:
						/* throw out null strings & rcon prompts */
						if(!charP || !strncmp(charP, "RCON", 4)) break;
						/* throw away echos & addresses from dm command */
						if(!strncmp(charP, "dm", 2) || !strncmp(charP, "0x", 2)) break;
						fprintf(spout, "SP CPU Error Address (high) Register value: %s\n", charP);
						fprintf(stdout, "SP CPU Error Address (high) Register value: %s\n", charP);
						fprintf(spout, "\n");
						done = 1;
						break;

					case cpu_err_data_low_c:
						/* throw out null strings & rcon prompts */
						if(!charP || !strncmp(charP, "RCON", 4)) break;
						/* throw away echos & addresses from dm command */
						if(!strncmp(charP, "dm", 2) || !strncmp(charP, "0x", 2)) break;
						fprintf(spout, "SP CPU Error Data (low) Register value: %s\n", charP);
						fprintf(stdout, "SP CPU Error Data (low) Register value: %s\n", charP);
						fprintf(spout, "\n");
						done = 1;
						break;

					case cpu_err_data_high_c:
						/* throw out null strings & rcon prompts */
						if(!charP || !strncmp(charP, "RCON", 4)) break;
						/* throw away echos & addresses from dm command */
						if(!strncmp(charP, "dm", 2) || !strncmp(charP, "0x", 2)) break;
						fprintf(spout, "SP CPU Error Data (high) Register value: %s\n", charP);
						fprintf(stdout, "SP CPU Error Data (high) Register value: %s\n", charP);
						fprintf(spout, "\n");
						done = 1;
						break;

					case cpu_err_parity_c:
						/* throw out null strings & rcon prompts */
						if(!charP || !strncmp(charP, "RCON", 4)) break;
						/* throw away echos & addresses from dm command */
						if(!strncmp(charP, "dm", 2) || !strncmp(charP, "0x", 2)) break;
						fprintf(spout, "SP CPU Error Parity  Register value: %s\n", charP);
						fprintf(stdout, "SP CPU Error Parity Register value: %s\n", charP);
						fprintf(spout, "\n");
						done = 1;
						break;

					case cpu_err_cause_c:
						/* throw out null strings & rcon prompts */
						if(!charP || !strncmp(charP, "RCON", 4)) break;
						/* throw away echos & addresses from dm command */
						if(!strncmp(charP, "dm", 2) || !strncmp(charP, "0x", 2)) break;
						fprintf(spout, "SP CPU Error Cause Register value: %s\n", charP);
						fprintf(stdout, "SP CPU Error Cause Register value: %s\n", charP);
						if(sizeof(charP) == 4) {
							errCause = (unsigned int)strtoul(charP, &endP, 16);
						} else {
							fprintf(spout, "Error - Bad SP CPU Error Cause Register Value returned\n");
							fprintf(stderr, "Error - Bad SP CPU Error Cause Register Value returned\n");
						}
						addrOut = (errCause & 0x01);
						fprintf(spout, "\t\t Addr Out of range: %d\n",addrOut);
						addrPErr = (errCause & 0x02) >> 1;
						fprintf(spout, "\t\t Bad Addr Parity: %d\n",addrPErr);
						TTErr = (errCause & 0x04) >> 2;
						fprintf(spout, "\t\t Transfer type Violation: %d\n",TTErr);
						accErr = (errCause & 0x08) >> 3;
						fprintf(spout, "\t\t Access To protected region: %d\n",accErr);
						wrErr = (errCause & 0x10) >> 4;
						fprintf(spout, "\t\t Write to write protected region: %d\n",wrErr);
						cacheErr = (errCause & 0x20) >> 5;
						fprintf(spout, "\t\t Read from caching protected region: %d\n",cacheErr);
						wrDataPErr = (errCause & 0x40) >> 6;
						fprintf(spout, "\t\t Bad write data parity detected: %d\n",wrDataPErr);
						rdDataPErr = (errCause & 0x80) >> 7;
						fprintf(spout, "\t\t Bad read data parity detected: %d\n",rdDataPErr);

						fprintf(spout, "\n");
						done = 1;
						break;

					case sdram_ecc_counter_c:
						/* throw out null strings & rcon prompts */
						if(!charP || !strncmp(charP, "RCON", 4)) break;
						/* throw away echos & addresses from dm command */
						if(!strncmp(charP, "dm", 2) || !strncmp(charP, "0x", 2)) break;
						fprintf(spout, "SP SDRAM ECC Counter Register value: %s\n", charP);
						fprintf(stdout, "SP SDRAM ECC Counter Register value: %s\n", charP);
						fprintf(spout, "\n");
						done = 1;
						break;

					case pci0_intr_cause_c:
						/* throw out null strings & rcon prompts */
						if(!charP || !strncmp(charP, "RCON", 4)) break;
						/* throw away echos & addresses from dm command */
						if(!strncmp(charP, "dm", 2) || !strncmp(charP, "0x", 2)) break;
						fprintf(spout, "SP PCI0 Interrupt Cause Register value: %s\n", charP);
						fprintf(stdout, "SP PCI0 Interrupt Cause Register value: %s\n", charP);
						fprintf(spout, "\n");
						done = 1;
						pci0Intr = (unsigned int)strtoul(charP, &endP, 16);
						/* if the value of pci interrupt cause is 0, we don't need to read other pci 0 regs */
						if((pci0Intr == 0x100) || (pci0Intr == 0x00)) {
							fprintf(spout, "End PCI0 \n");
							i = (int)pci0_err_comm_c;
						}
						break;

					case pci1_intr_cause_c:
						/* throw out null strings & rcon prompts */
						if(!charP || !strncmp(charP, "RCON", 4)) break;
						/* throw away echos & addresses from dm command */
						if(!strncmp(charP, "dm", 2) || !strncmp(charP, "0x", 2)) break;
						fprintf(spout, "SP PCI1 Interrupt Cause Register value: %s\n", charP);
						fprintf(stdout, "SP PCI1 Interrupt Cause Register value: %s\n", charP);
						fprintf(spout, "\n");
						done = 1;
						/* if the value of pci interrupt cause is 0, we don't need to read other pci 1 regs */
						if((pci1Intr == 0x100) || (pci1Intr == 0x00)) {
							fprintf(spout, "End PCI1 \n");
							i = (int)pci1_err_comm_c;
						}
						break;

					case pci0_err_addr_low_c:
						/* throw out null strings & rcon prompts */
						if(!charP || !strncmp(charP, "RCON", 4)) break;
						/* throw away echos & addresses from dm command */
						if(!strncmp(charP, "dm", 2) || !strncmp(charP, "0x", 2)) break;
						fprintf(spout, "SP PCI0 Error Addr low Register value: %s\n", charP);
						fprintf(stdout, "SP PCI0 Error Addr low Register value: %s\n", charP);
						fprintf(spout, "\n");
						done = 1;
						break;

					case pci0_err_addr_high_c:
						/* throw out null strings & rcon prompts */
						if(!charP || !strncmp(charP, "RCON", 4)) break;
						/* throw away echos & addresses from dm command */
						if(!strncmp(charP, "dm", 2) || !strncmp(charP, "0x", 2)) break;
						fprintf(spout, "SP PCI0 Error Addr high Register value: %s\n", charP);
						fprintf(stdout, "SP PCI0 Error Addr high Register value: %s\n", charP);
						fprintf(spout, "\n");
						done = 1;
						break;

					case pci0_err_data_low_c:
						/* throw out null strings & rcon prompts */
						if(!charP || !strncmp(charP, "RCON", 4)) break;
						/* throw away echos & addresses from dm command */
						if(!strncmp(charP, "dm", 2) || !strncmp(charP, "0x", 2)) break;
						fprintf(spout, "SP PCI0 Error Data low Register value: %s\n", charP);
						fprintf(stdout, "SP PCI0 Error Data low Register value: %s\n", charP);
						fprintf(spout, "\n");
						done = 1;
						break;

					case pci0_err_data_high_c:
						/* throw out null strings & rcon prompts */
						if(!charP || !strncmp(charP, "RCON", 4)) break;
						/* throw away echos & addresses from dm command */
						if(!strncmp(charP, "dm", 2) || !strncmp(charP, "0x", 2)) break;
						fprintf(spout, "SP PCI0 Error Data high Register value: %s\n", charP);
						fprintf(stdout, "SP PCI0 Error Data high Register value: %s\n", charP);
						fprintf(spout, "\n");
						done = 1;
						break;

					case pci0_err_comm_c:
						/* throw out null strings & rcon prompts */
						if(!charP || !strncmp(charP, "RCON", 4)) break;
						/* throw away echos & addresses from dm command */
						if(!strncmp(charP, "dm", 2) || !strncmp(charP, "0x", 2)) break;
						fprintf(spout, "SP PCI0 Error Command Register value: %s\n", charP);
						fprintf(stdout, "SP PCI0 Error Command Register value: %s\n", charP);
						fprintf(spout, "End PCI0 \n");
						fprintf(spout, "\n");
						done = 1;
						break;

					case pci1_err_addr_low_c:
						/* throw out null strings & rcon prompts */
						if(!charP || !strncmp(charP, "RCON", 4)) break;
						/* throw away echos & addresses from dm command */
						if(!strncmp(charP, "dm", 2) || !strncmp(charP, "0x", 2)) break;
						fprintf(spout, "SP PCI1 Error Addr low Register value: %s\n", charP);
						fprintf(stdout, "SP PCI1 Error Addr low Register value: %s\n", charP);
						fprintf(spout, "\n");
						done = 1;
						break;

					case pci1_err_addr_high_c:
						/* throw out null strings & rcon prompts */
						if(!charP || !strncmp(charP, "RCON", 4)) break;
						/* throw away echos & addresses from dm command */
						if(!strncmp(charP, "dm", 2) || !strncmp(charP, "0x", 2)) break;
						fprintf(spout, "SP PCI1 Error Addr high Register value: %s\n", charP);
						fprintf(stdout, "SP PCI1 Error Addr high Register value: %s\n", charP);
						fprintf(spout, "\n");
						done = 1;
						break;

					case pci1_err_data_low_c:
						/* throw out null strings & rcon prompts */
						if(!charP || !strncmp(charP, "RCON", 4)) break;
						/* throw away echos & addresses from dm command */
						if(!strncmp(charP, "dm", 2) || !strncmp(charP, "0x", 2)) break;
						fprintf(spout, "SP PCI1 Error Data low Register value: %s\n", charP);
						fprintf(stdout, "SP PCI1 Error Data low Register value: %s\n", charP);
						fprintf(spout, "\n");
						done = 1;
						break;

					case pci1_err_data_high_c:
						/* throw out null strings & rcon prompts */
						if(!charP || !strncmp(charP, "RCON", 4)) break;
						/* throw away echos & addresses from dm command */
						if(!strncmp(charP, "dm", 2) || !strncmp(charP, "0x", 2)) break;
						fprintf(spout, "SP PCI1 Error Data high Register value: %s\n", charP);
						fprintf(stdout, "SP PCI1 Error Data high Register value: %s\n", charP);
						fprintf(spout, "\n");
						done = 1;
						break;

					case pci1_err_comm_c:
						/* throw out null strings & rcon prompts */
						if(!charP || !strncmp(charP, "RCON", 4)) break;
						/* throw away echos & addresses from dm command */
						if(!strncmp(charP, "dm", 2) || !strncmp(charP, "0x", 2)) break;
						fprintf(spout, "SP PCI1 Error Command Register value: %s\n", charP);
						fprintf(stdout, "SP PCI1 Error Command Register value: %s\n", charP);
						fprintf(spout, "End PCI1 \n");
						fprintf(spout, "\n");
						done = 1;
						break;

					case dp83820_c:
						if(first) {
						  fprintf(spout, "Collecting National MAC diag output\n");
						  fprintf(stdout, "Collecting National MAC diag output\n");
							first = 0;
						}
						/* throw out null strings & rcon prompts */
						if(!charP || !strncmp(charP, "RCON", 4)) break;
						fprintf(spout, "%s", inStr);
						fprintf(stdout, "%s", inStr);
						break;

					case gtdiag_1_c:
						if(first) {
						  fprintf(spout, "Collecting Galileo Ethernet port 1 diag output\n");
						  fprintf(stdout, "Collecting Galileo Ethernet port 1 diag output\n");
							first = 0;
						}
						/* throw out null strings & rcon prompts */
						if(!charP || !strncmp(charP, "RCON", 4)) break;
						fprintf(spout, "%s", inStr);
						fprintf(stdout, "%s", inStr);
						break;

					case gtdiag_2_c:
					  if(first) {
						  fprintf(spout, "Collecting Galileo Ethernet port 2 diag output\n");
						  fprintf(stdout, "Collecting Galileo Ethernet port 2 diag output\n");
							first = 0;
						}
						/* throw out null strings & rcon prompts */
						if(!charP || !strncmp(charP, "RCON", 4)) break;
						fprintf(spout, "%s", inStr);
						fprintf(stdout, "%s", inStr);
						break;

					case gtdiag_3_c:
						if(first) {
						  fprintf(spout, "Collecting Galileo Ethernet port 3 diag output\n");
						  fprintf(stdout, "Collecting Galileo Ethernet port 3 diag output\n");
						  first = 0;
						}
						/* throw out null strings & rcon prompts */
						if(!charP || !strncmp(charP, "RCON", 4)) break;
						fprintf(spout, "%s", inStr);
						fprintf(stdout, "%s", inStr);
						break;

					default:
						printf("Error - command index out of bounds\n");
						break;
					}
				}
			}
			
		}
		shutdown_rcon_session();
	}

	fclose(spout);
	return(0);
}
