/*
 *-----------------------------------------------------------------
 *
 * Name: SSC Hardware System Health Monitor 
 *
 * RCS:  $Id:
 *
 * Copyright (C) 2002, Agile Storage, Inc.
 *
 * Description: Connects to SSC card using rcon and collects hardware
 * 							statistics.
 *
 * Created by:	Christa Porter
 *
 * Date Created: 20 Sep 2002
 *
 *-----------------------------------------------------------------
 */

#include "sysHealth.h"

#define NUMSSCCMDS 39 


int sysHealthSSC(void) {

	FILE *sscout;
	char inStr[MAXLINE];
	char parseStr[MAXLINE];
	char *charP;
	char *endP;
	int done, first;

	/* SSC register variables */
	unsigned int cpu_err_add_l;
	unsigned int cpu_err_add_h;
	unsigned int cpu_err_data_l;
	unsigned int cpu_err_data_h;
	unsigned int cpu_err_parity;
	unsigned int cpu_err_cause;
	unsigned int sdram_ecc_count;

	/* SSC PCI variables */
	unsigned int pci0_int_cause;
	unsigned int pci0_err_add_l;
	unsigned int pci0_err_add_h;
	unsigned int pci0_err_data_l;
	unsigned int pci0_err_data_h;
	unsigned int pci0_err_cmd;
	unsigned int pci1_int_cause;
	unsigned int pci1_err_add_l;
	unsigned int pci1_err_add_h;
	unsigned int pci1_err_data_l;
	unsigned int pci1_err_data_h;
	unsigned int pci1_err_cmd;

	int i, slot, port, rc;
	
	enum fpCmds_e {
		cpu_err_add_l_c,				/* CPU error address (low) */
		cpu_err_add_h_c,				/* CPU error address (high) */
		cpu_err_data_l_c,				/* CPU error data (low) */
		cpu_err_data_h_c,				/* CPU error data (high) */
		cpu_err_parity_c,				/* CPU error parity */
		cpu_err_cause_c,				/* CPU error cause */
		sdram_ecc_count_c,			/* SDRAM ECC counter */
		pci0_int_cause_c,				/* PCI 0 interupt cause */
		pci0_err_add_l_c,				/* PCI 0 error address (low) */
		pci0_err_add_h_c,				/* PCI 0 error address (high) */
		pci0_err_data_l_c,			/* PCI 0 error data (low) */
		pci0_err_data_h_c,			/* PCI 0 error data (high) */
		pci0_err_cmd_c,					/* PCI 0 error parity */
		pci1_int_cause_c,				/* PCI 1 interupt cause */
		pci1_err_add_l_c,				/* PCI 1 error address (low) */
		pci1_err_add_h_c,				/* PCI 1 error address (high) */
		pci1_err_data_l_c,			/* PCI 1 error data (low) */
		pci1_err_data_h_c,			/* PCI 1 error data (high) */
		pci1_err_cmd_c,					/* PCI 1 error parity */
#ifdef GTDIAG
		gtdiag_1_c,							/* Galileo diags on RMII port 1 */
		gtdiag_2_c,							/* Galileo diags on RMII port 2 */
		gtdiag_3_c,							/* Galileo diags on RMII port 3 */
#endif
		bcm5632_0_c,						/* 5632 diags on GMII port 0 */
		bcm5632_1_c,						/* 5632 diags on GMII port 1 */
		bcm5632_2_c,						/* 5632 diags on GMII port 2 */
		bcm5632_3_c,						/* 5632 diags on GMII port 3 */
		bcm5632_4_c,						/* 5632 diags on GMII port 4 */
		bcm5632_5_c,						/* 5632 diags on GMII port 5 */
		bcm5632_6_c,						/* 5632 diags on GMII port 6 */
		bcm5632_7_c,						/* 5632 diags on GMII port 7 */
		bcm5632_8_c,						/* 5632 diags on GMII port 8 */
		bcm5632_9_c,						/* 5632 diags on GMII port 9 */
		bcm5632_10_c,						/* 5632 diags on GMII port 10 */
		bcm5632_11_c,						/* 5632 diags on GMII port 11 */
		bcm5600_0_c,						/* 5600 diags on GMII port 0 */
		bcm5600_1_c,						/* 5600 diags on GMII port 1 */
		bcm5600_2_c,						/* 5600 diags on GMII port 2 */
		bcm5600_3_c,						/* 5600 diags on GMII port 3 */
		bcm5600_4_c,						/* 5600 diags on GMII port 4 */
		bcm5600_5_c,						/* 5600 diags on GMII port 5 */
		bcm5600_6_c,						/* 5600 diags on GMII port 6 */
		bcm5600_7_c							/* 5600 diags on GMII port 7 */
	};

	char *fpCmds[] = {
		"dm 0x24000070 1\n",				/* CPU error address (low) */
		"dm 0x24000078 1\n",				/* CPU error address (high) */
		"dm 0x24000128 1\n",				/* CPU error data (low) */ 
		"dm 0x24000130 1\n",				/* CPU error data (high) */ 
		"dm 0x24000138 1\n",				/* CPU error parity */   
		"dm 0x24000140 1\n",				/* CPU error cause */    
		"dm 0x24000498 1\n",				/* SDRAM ECC counter */
		"dm 0x24001d58 1\n",				/* PCI 0 interupt cause */
		"dm 0x24001d40 1\n",				/* PCI 0 error address (low) */
		"dm 0x24001d44 1\n",				/* PCI 0 error address (high) */
		"dm 0x24001d48 1\n",				/* PCI 0 error data (low) */
		"dm 0x24001d4c 1\n",				/* PCI 0 error data (high) */
		"dm 0x24001d50 1\n",				/* PCI 0 error parity */  
		"dm 0x24001dd8 1\n",				/* PCI 1 interupt cause */
		"dm 0x24001dc0 1\n",				/* PCI 1 error address (low) */
		"dm 0x24001dc4 1\n",				/* PCI 1 error address (high) */
		"dm 0x24001dc8 1\n",				/* PCI 1 error data (low) */
		"dm 0x24001dcc 1\n",				/* PCI 1 error data (high) */
		"dm 0x24001dd0 1\n",				/* PCI 1 error parity */  
#ifdef GTDIAG
		"gtdiag\n1\n2\n999\n",			/* Galileo diags on RMII port 1 */
		"gtdiag\n2\n2\n999\n",			/* Galileo diags on RMII port 2 */
		"gtdiag\n3\n2\n999\n",			/* Galileo diags on RMII port 3 */
#endif
		"bcm5632\n0\n6\n0\n99\n",		/* 5632 diags on GMII port 0 */
		"bcm5632\n0\n6\n1\n99\n",		/* 5632 diags on GMII port 1 */
		"bcm5632\n0\n6\n2\n99\n",		/* 5632 diags on GMII port 2 */
		"bcm5632\n0\n6\n3\n99\n",		/* 5632 diags on GMII port 3 */
		"bcm5632\n0\n6\n4\n99\n",		/* 5632 diags on GMII port 4 */
		"bcm5632\n0\n6\n5\n99\n",		/* 5632 diags on GMII port 5 */
		"bcm5632\n0\n6\n6\n99\n",		/* 5632 diags on GMII port 6 */
		"bcm5632\n0\n6\n7\n99\n",		/* 5632 diags on GMII port 7 */
		"bcm5632\n0\n6\n8\n99\n",		/* 5632 diags on GMII port 8 */
		"bcm5632\n0\n6\n9\n99\n",		/* 5632 diags on GMII port 9 */
		"bcm5632\n0\n6\n10\n99\n",	/* 5632 diags on GMII port 10 */
		"bcm5632\n0\n6\n11\n99\n",	/* 5632 diags on GMII port 11 */
		"bcm5600\n8\n0\n99\n",			/* 5600 diags on RMII port 0 */
		"bcm5600\n8\n1\n99\n",			/* 5600 diags on RMII port 1 */
		"bcm5600\n8\n2\n99\n",			/* 5600 diags on RMII port 2 */
		"bcm5600\n8\n3\n99\n",			/* 5600 diags on RMII port 3 */
		"bcm5600\n8\n4\n99\n",			/* 5600 diags on RMII port 4 */
		"bcm5600\n8\n5\n99\n",			/* 5600 diags on RMII port 5 */
		"bcm5600\n8\n6\n99\n",			/* 5600 diags on RMII port 6 */
		"bcm5600\n8\n7\n99\n"				/* 5600 diags on RMII port 7 */
	};

	if((sscout = fopen(SSCFILE, "w")) == NULL) {
		printf("Cannot open %s\n", SSCFILE);
		exit(1);
	}

	/* FP statics collection */
	fprintf(sscout, "****************************************************************\n");
	fprintf(sscout, "*************** NFX System Harware Health Report ***************\n");
	fprintf(sscout, "****************************************************************\n");
	fprintf(sscout, "*************************** SSC Log ****************************\n");
	fprintf(sscout, "****************************************************************\n");
	/* The SSC is slice 0, the Toshiba is port 1 */
	slot = 0;
	port = 1;
	if(establish_rcon_session (slot, port)) {
		fprintf(sscout, "Connection to Toshiba on SSC card, slot %d, port %d failed\n", slot, port);
		fprintf(stderr, "Connection to Toshiba on SSC card, slot %d, port %d failed\n", slot, port);
		exit (1);
	} else {
		fprintf(sscout, "\n\nConnecting to Toshiba on SSC card, slot %d, port %d\n\n", slot, port);
		fprintf(stdout, "\n\nConnecting to Toshiba on SSC card, slot %d, port %d\n\n", slot, port);
	}

	/* process FP command file, more LSI MACs on FP SiByte */
	for(i=0; i<NUMSSCCMDS; i++) {
		/* output command using rcon function */
		fprintf(sscout, "Sending SSC Command: %s", fpCmds[i]);
		fprintf(stdout, "Sending SSC Command: %s", fpCmds[i]);
		if(send_to_rcon (fpCmds[i])) {
			fprintf (sscout, "RCON send to SiByte on FP card, slot %d, port %d failed\n", slot, port);
			fprintf (stderr, "RCON send to SiByte on FP card, slot %d, port %d failed\n", slot, port);
			shutdown_rcon_session();
			exit (1);
		}

		done = 0;
		first = 1;
		/* Get RCON command response and process */
		while((rc = get_from_rcon (inStr, MAXLINE, 5)) > 0) {
#ifdef DEBUG
			fprintf(stdout, "Got response: %s\n", inStr);
#endif

			/* make copy for parsing */
			strcpy(parseStr, inStr);
			charP = strtok(parseStr, " \t\n");
#ifdef DEBUG
			fprintf(stdout, "charP = %s\n", charP);
#endif

			if(!done) {
				/* main case for parsing each FP command */
				switch(i) {
				case cpu_err_add_l_c:
					/* throw out null strings & rcon prompts */
					if(!charP || !strncmp(charP, "RCON", 4)) break;
					/* throw away echos & addresses from dm command */
					if(!strncmp(charP, "dm", 2) || !strncmp(charP, "0x", 2)) break;
					fprintf(sscout, "CPU Error Address Register value(low): %s\n", charP);
					fprintf(stdout, "CPU Error Address Register value(low): %s\n", charP);
					if(sizeof(charP) == 4) {
						cpu_err_add_l = (unsigned int)strtoul(charP, &endP, 16);
					} else {
						fprintf(sscout, "Error - Bad CPU Error Address Register value returned\n");
						fprintf(stderr, "Error - Bad CPU Error Address Register value returned\n");
					}
					done = 1;
					break;
				case cpu_err_add_h_c:
					/* throw out null strings & rcon prompts */
					if(!charP || !strncmp(charP, "RCON", 4)) break;
					/* throw away echos & addresses from dm command */
					if(!strncmp(charP, "dm", 2) || !strncmp(charP, "0x", 2)) break;
					fprintf(sscout, "CPU Error Address Register value(high): %s\n", charP);
					fprintf(stdout, "CPU Error Address Register value(high): %s\n", charP);
					if(sizeof(charP) == 4) {
						cpu_err_add_h = (unsigned int)strtoul(charP, &endP, 16);
					} else {
						fprintf(sscout, "Error - Bad CPU Error Address Register value returned\n");
						fprintf(stderr, "Error - Bad CPU Error Address Register value returned\n");
					}
					done = 1;
					break;
				case cpu_err_data_l_c:
					/* throw out null strings & rcon prompts */
					if(!charP || !strncmp(charP, "RCON", 4)) break;
					/* throw away echos & addresses from dm command */
					if(!strncmp(charP, "dm", 2) || !strncmp(charP, "0x", 2)) break;
					fprintf(sscout, "CPU Error Data Register value(low): %s\n", charP);
					fprintf(stdout, "CPU Error Data Register value(low): %s\n", charP);
					if(sizeof(charP) == 4) {
						cpu_err_data_l = (unsigned int)strtoul(charP, &endP, 16);
					} else {
						fprintf(sscout, "Error - Bad CPU Error Data Register value returned\n");
						fprintf(stderr, "Error - Bad CPU Error Data Register value returned\n");
					}
					done = 1;
					break;
				case cpu_err_data_h_c:
					/* throw out null strings & rcon prompts */
					if(!charP || !strncmp(charP, "RCON", 4)) break;
					/* throw away echos & addresses from dm command */
					if(!strncmp(charP, "dm", 2) || !strncmp(charP, "0x", 2)) break;
					fprintf(sscout, "CPU Error Data Register value(high): %s\n", charP);
					fprintf(stdout, "CPU Error Data Register value(high): %s\n", charP);
					if(sizeof(charP) == 4) {
						cpu_err_data_h = (unsigned int)strtoul(charP, &endP, 16);
					} else {
						fprintf(sscout, "Error - Bad CPU Error Data Register value returned\n");
						fprintf(stderr, "Error - Bad CPU Error Data Register value returned\n");
					}
					done = 1;
					break;
				case cpu_err_parity_c:
					/* throw out null strings & rcon prompts */
					if(!charP || !strncmp(charP, "RCON", 4)) break;
					/* throw away echos & addresses from dm command */
					if(!strncmp(charP, "dm", 2) || !strncmp(charP, "0x", 2)) break;
					fprintf(sscout, "CPU Error Parity Register value: %s\n", charP);
					fprintf(stdout, "CPU Error Parity Register value: %s\n", charP);
					if(sizeof(charP) == 4) {
						cpu_err_parity = (unsigned int)strtoul(charP, &endP, 16);
					} else {
						fprintf(sscout, "Error - Bad CPU Error Parity Register value returned\n");
						fprintf(stderr, "Error - Bad CPU Error Parity Register value returned\n");
					}
					done = 1;
					break;
				case cpu_err_cause_c:
					/* throw out null strings & rcon prompts */
					if(!charP || !strncmp(charP, "RCON", 4)) break;
					/* throw away echos & addresses from dm command */
					if(!strncmp(charP, "dm", 2) || !strncmp(charP, "0x", 2)) break;
					fprintf(sscout, "CPU Error Cause Register value: %s\n", charP);
					fprintf(stdout, "CPU Error Cause Register value: %s\n", charP);
					if(sizeof(charP) == 4) {
						cpu_err_cause = (unsigned int)strtoul(charP, &endP, 16);
					} else {
						fprintf(sscout, "Error - Bad CPU Error Cause Register value returned\n");
						fprintf(stderr, "Error - Bad CPU Error Cause Register value returned\n");
					}
					done = 1;
					break;
				case sdram_ecc_count_c:
					/* throw out null strings & rcon prompts */
					if(!charP || !strncmp(charP, "RCON", 4)) break;
					/* throw away echos & addresses from dm command */
					if(!strncmp(charP, "dm", 2) || !strncmp(charP, "0x", 2)) break;
					fprintf(sscout, "SDRAM ECC Counter Register value: %s\n", charP);
					fprintf(stdout, "SDRAM ECC Counter Register value: %s\n", charP);
					if(sizeof(charP) == 4) {
						sdram_ecc_count = (unsigned int)strtoul(charP, &endP, 16);
					} else {
						fprintf(sscout, "Error - Bad SDRAM ECC Counter Register value returned\n");
						fprintf(stderr, "Error - Bad SDRAM ECC Counter Register value returned\n");
					}
					done = 1;
					break;
				case pci0_int_cause_c:
					/* throw out null strings & rcon prompts */
					if(!charP || !strncmp(charP, "RCON", 4)) break;
					/* throw away echos & addresses from dm command */
					if(!strncmp(charP, "dm", 2) || !strncmp(charP, "0x", 2)) break;
					fprintf(sscout, "SSC PCI 0 Interupt Cause register value: %s\n", charP);
					fprintf(stdout, "SSC PCI 0 Interupt Cause register value: %s\n", charP);
					if(sizeof(charP) == 4) {
						pci0_int_cause = (unsigned int)strtoul(charP, &endP, 16);
					} else {
						fprintf(sscout, "Error - Bad SSC PCI 0 Interupt Cause register value returned\n");
						fprintf(stderr, "Error - Bad SSC PCI 0 Interupt Cause register value returned\n");
					}
					done = 1;
					break;
				case pci0_err_add_l_c:
					/* throw out null strings & rcon prompts */
					if(!charP || !strncmp(charP, "RCON", 4)) break;
					/* throw away echos & addresses from dm command */
					if(!strncmp(charP, "dm", 2) || !strncmp(charP, "0x", 2)) break;
					fprintf(sscout, "SSC PCI 0 Error Address register value(low): %s\n", charP);
					fprintf(stdout, "SSC PCI 0 Error Address register value(low): %s\n", charP);
					if(sizeof(charP) == 4) {
						pci0_err_add_l = (unsigned int)strtoul(charP, &endP, 16);
					} else {
						fprintf(sscout, "Error - Bad SSC PCI 0 Error Address register value returned\n");
						fprintf(stderr, "Error - Bad SSC PCI 0 Error Address register value returned\n");
					}
					done = 1;
					break;
				case pci0_err_add_h_c:
					/* throw out null strings & rcon prompts */
					if(!charP || !strncmp(charP, "RCON", 4)) break;
					/* throw away echos & addresses from dm command */
					if(!strncmp(charP, "dm", 2) || !strncmp(charP, "0x", 2)) break;
					fprintf(sscout, "SSC PCI 0 Error Address register value(high): %s\n", charP);
					fprintf(stdout, "SSC PCI 0 Error Address register value(high): %s\n", charP);
					if(sizeof(charP) == 4) {
						pci0_err_add_h = (unsigned int)strtoul(charP, &endP, 16);
					} else {
						fprintf(sscout, "Error - Bad SSC PCI 0 Error Address register value returned\n");
						fprintf(stderr, "Error - Bad SSC PCI 0 Error Address register value returned\n");
					}
					done = 1;
					break;
				case pci0_err_data_l_c:
					/* throw out null strings & rcon prompts */
					if(!charP || !strncmp(charP, "RCON", 4)) break;
					/* throw away echos & addresses from dm command */
					if(!strncmp(charP, "dm", 2) || !strncmp(charP, "0x", 2)) break;
					fprintf(sscout, "SSC PCI 0 Error Data register value(low): %s\n", charP);
					fprintf(stdout, "SSC PCI 0 Error Data register value(low): %s\n", charP);
					if(sizeof(charP) == 4) {
						pci0_err_data_l = (unsigned int)strtoul(charP, &endP, 16);
					} else {
						fprintf(sscout, "Error - Bad SSC PCI 0 Error Data register value returned\n");
						fprintf(stderr, "Error - Bad SSC PCI 0 Error Data register value returned\n");
					}
					done = 1;
					break;
				case pci0_err_data_h_c:
					/* throw out null strings & rcon prompts */
					if(!charP || !strncmp(charP, "RCON", 4)) break;
					/* throw away echos & addresses from dm command */
					if(!strncmp(charP, "dm", 2) || !strncmp(charP, "0x", 2)) break;
					fprintf(sscout, "SSC PCI 0 Error Data register value(high): %s\n", charP);
					fprintf(stdout, "SSC PCI 0 Error Data register value(high): %s\n", charP);
					if(sizeof(charP) == 4) {
						pci0_err_data_h = (unsigned int)strtoul(charP, &endP, 16);
					} else {
						fprintf(sscout, "Error - Bad SSC PCI 0 Error Data register value returned\n");
						fprintf(stderr, "Error - Bad SSC PCI 0 Error Data register value returned\n");
					}
					done = 1;
					break;
				case pci0_err_cmd_c:
					/* throw out null strings & rcon prompts */
					if(!charP || !strncmp(charP, "RCON", 4)) break;
					/* throw away echos & addresses from dm command */
					if(!strncmp(charP, "dm", 2) || !strncmp(charP, "0x", 2)) break;
					fprintf(sscout, "SSC PCI 0 Error Command register value: %s\n", charP);
					fprintf(stdout, "SSC PCI 0 Error Command register value: %s\n", charP);
					if(sizeof(charP) == 4) {
						pci0_err_cmd = (unsigned int)strtoul(charP, &endP, 16);
					} else {
						fprintf(sscout, "Error - Bad SSC PCI 0 Error Command register value returned\n");
						fprintf(stderr, "Error - Bad SSC PCI 0 Error Command register value returned\n");
					}
					done = 1;
					break;
				case pci1_int_cause_c:
					/* throw out null strings & rcon prompts */
					if(!charP || !strncmp(charP, "RCON", 4)) break;
					/* throw away echos & addresses from dm command */
					if(!strncmp(charP, "dm", 2) || !strncmp(charP, "0x", 2)) break;
					fprintf(sscout, "SSC PCI 1 Interupt Cause register value: %s\n", charP);
					fprintf(stdout, "SSC PCI 1 Interupt Cause register value: %s\n", charP);
					if(sizeof(charP) == 4) {
						pci1_int_cause = (unsigned int)strtoul(charP, &endP, 16);
					} else {
						fprintf(sscout, "Error - Bad SSC PCI 1 Interupt Cause register value returned\n");
						fprintf(stderr, "Error - Bad SSC PCI 1 Interupt Cause register value returned\n");
					}
					done = 1;
					break;
				case pci1_err_add_l_c:
					/* throw out null strings & rcon prompts */
					if(!charP || !strncmp(charP, "RCON", 4)) break;
					/* throw away echos & addresses from dm command */
					if(!strncmp(charP, "dm", 2) || !strncmp(charP, "0x", 2)) break;
					fprintf(sscout, "SSC PCI 1 Error Address register value(low): %s\n", charP);
					fprintf(stdout, "SSC PCI 1 Error Address register value(low): %s\n", charP);
					if(sizeof(charP) == 4) {
						pci1_err_add_l = (unsigned int)strtoul(charP, &endP, 16);
					} else {
						fprintf(sscout, "Error - Bad SSC PCI 1 Error Address register value returned\n");
						fprintf(stderr, "Error - Bad SSC PCI 1 Error Address register value returned\n");
					}
					done = 1;
					break;
				case pci1_err_add_h_c:
					/* throw out null strings & rcon prompts */
					if(!charP || !strncmp(charP, "RCON", 4)) break;
					/* throw away echos & addresses from dm command */
					if(!strncmp(charP, "dm", 2) || !strncmp(charP, "0x", 2)) break;
					fprintf(sscout, "SSC PCI 1 Error Address register value(high): %s\n", charP);
					fprintf(stdout, "SSC PCI 1 Error Address register value(high): %s\n", charP);
					if(sizeof(charP) == 4) {
						pci1_err_add_h = (unsigned int)strtoul(charP, &endP, 16);
					} else {
						fprintf(sscout, "Error - Bad SSC PCI 1 Error Address register value returned\n");
						fprintf(stderr, "Error - Bad SSC PCI 1 Error Address register value returned\n");
					}
					done = 1;
					break;
				case pci1_err_data_l_c:
					/* throw out null strings & rcon prompts */
					if(!charP || !strncmp(charP, "RCON", 4)) break;
					/* throw away echos & addresses from dm command */
					if(!strncmp(charP, "dm", 2) || !strncmp(charP, "0x", 2)) break;
					fprintf(sscout, "SSC PCI 1 Error Data register value(low): %s\n", charP);
					fprintf(stdout, "SSC PCI 1 Error Data register value(low): %s\n", charP);
					if(sizeof(charP) == 4) {
						pci1_err_data_l = (unsigned int)strtoul(charP, &endP, 16);
					} else {
						fprintf(sscout, "Error - Bad SSC PCI 1 Error Data register value returned\n");
						fprintf(stderr, "Error - Bad SSC PCI 1 Error Data register value returned\n");
					}
					done = 1;
					break;
				case pci1_err_data_h_c:
					/* throw out null strings & rcon prompts */
					if(!charP || !strncmp(charP, "RCON", 4)) break;
					/* throw away echos & addresses from dm command */
					if(!strncmp(charP, "dm", 2) || !strncmp(charP, "0x", 2)) break;
					fprintf(sscout, "SSC PCI 1 Error Data register value(high): %s\n", charP);
					fprintf(stdout, "SSC PCI 1 Error Data register value(high): %s\n", charP);
					if(sizeof(charP) == 4) {
						pci1_err_data_h = (unsigned int)strtoul(charP, &endP, 16);
					} else {
						fprintf(sscout, "Error - Bad SSC PCI 1 Error Data register value returned\n");
						fprintf(stderr, "Error - Bad SSC PCI 1 Error Data register value returned\n");
					}
					done = 1;
					break;
				case pci1_err_cmd_c:
					/* throw out null strings & rcon prompts */
					if(!charP || !strncmp(charP, "RCON", 4)) break;
					/* throw away echos & addresses from dm command */
					if(!strncmp(charP, "dm", 2) || !strncmp(charP, "0x", 2)) break;
					fprintf(sscout, "SSC PCI 1 Error Command register value: %s\n", charP);
					fprintf(stdout, "SSC PCI 1 Error Command register value: %s\n", charP);
					if(sizeof(charP) == 4) {
						pci1_err_cmd = (unsigned int)strtoul(charP, &endP, 16);
					} else {
						fprintf(sscout, "Error - Bad SSC PCI 1 Error Command register value returned\n");
						fprintf(stderr, "Error - Bad SSC PCI 1 Error Command register value returned\n");
					}
					done = 1;
					break;
#ifdef GTDIAG
				case gtdiag_1_c:
					if(first) {
						fprintf(sscout, "Collecting Galileo, port 1 diag output\n");
						fprintf(stdout, "Collecting Galileo, port 1 diag output\n");
						first = 0;
					}
					/* throw out null strings & rcon prompts */
					if(!charP || !strncmp(charP, "RCON", 4)) break;
					fprintf(sscout, "%s", inStr);
					fprintf(stdout, "%s", inStr);
					break;
				case gtdiag_2_c:
					if(first) {
						fprintf(sscout, "Collecting Galileo, port 2 diag output\n");
						fprintf(stdout, "Collecting Galileo, port 2 diag output\n");
						first = 0;
					}
					/* throw out null strings & rcon prompts */
					if(!charP || !strncmp(charP, "RCON", 4)) break;
					fprintf(sscout, "%s", inStr);
					fprintf(stdout, "%s", inStr);
					break;
				case gtdiag_3_c:
					if(first) {
						fprintf(sscout, "Collecting Galileo, port 3 diag output\n");
						fprintf(stdout, "Collecting Galileo, port 3 diag output\n");
						first = 0;
					}
					/* throw out null strings & rcon prompts */
					if(!charP || !strncmp(charP, "RCON", 4)) break;
					fprintf(sscout, "%s", inStr);
					fprintf(stdout, "%s", inStr);
					break;
#endif
				case bcm5632_0_c:
					if(first) {
						fprintf(sscout, "Collecting 5632, port 0 diag output\n");
						fprintf(stdout, "Collecting 5632, port 0 diag output\n");
						first = 0;
					}
					/* throw out null strings & rcon prompts */
					if(!charP || !strncmp(charP, "RCON", 4)) break;
					fprintf(sscout, "%s", inStr);
					fprintf(stdout, "%s", inStr);
					break;
				case bcm5632_1_c:
					if(first) {
						fprintf(sscout, "Collecting 5632, port 1 diag output\n");
						fprintf(stdout, "Collecting 5632, port 1 diag output\n");
						first = 0;
					}
					/* throw out null strings & rcon prompts */
					if(!charP || !strncmp(charP, "RCON", 4)) break;
					fprintf(sscout, "%s", inStr);
					fprintf(stdout, "%s", inStr);
					break;
				case bcm5632_2_c:
					if(first) {
						fprintf(sscout, "Collecting 5632, port 2 diag output\n");
						fprintf(stdout, "Collecting 5632, port 2 diag output\n");
						first = 0;
					}
					/* throw out null strings & rcon prompts */
					if(!charP || !strncmp(charP, "RCON", 4)) break;
					fprintf(sscout, "%s", inStr);
					fprintf(stdout, "%s", inStr);
					break;
				case bcm5632_3_c:
					if(first) {
						fprintf(sscout, "Collecting 5632, port 3 diag output\n");
						fprintf(stdout, "Collecting 5632, port 3 diag output\n");
						first = 0;
					}
					/* throw out null strings & rcon prompts */
					if(!charP || !strncmp(charP, "RCON", 4)) break;
					fprintf(sscout, "%s", inStr);
					fprintf(stdout, "%s", inStr);
					break;
				case bcm5632_4_c:
					if(first) {
						fprintf(sscout, "Collecting 5632, port 4 diag output\n");
						fprintf(stdout, "Collecting 5632, port 4 diag output\n");
						first = 0;
					}
					/* throw out null strings & rcon prompts */
					if(!charP || !strncmp(charP, "RCON", 4)) break;
					fprintf(sscout, "%s", inStr);
					fprintf(stdout, "%s", inStr);
					break;
				case bcm5632_5_c:
					if(first) {
						fprintf(sscout, "Collecting 5632, port 5 diag output\n");
						fprintf(stdout, "Collecting 5632, port 5 diag output\n");
						first = 0;
					}
					/* throw out null strings & rcon prompts */
					if(!charP || !strncmp(charP, "RCON", 4)) break;
					fprintf(sscout, "%s", inStr);
					fprintf(stdout, "%s", inStr);
					break;
				case bcm5632_6_c:
					if(first) {
						fprintf(sscout, "Collecting 5632, port 6 diag output\n");
						fprintf(stdout, "Collecting 5632, port 6 diag output\n");
						first = 0;
					}
					/* throw out null strings & rcon prompts */
					if(!charP || !strncmp(charP, "RCON", 4)) break;
					fprintf(sscout, "%s", inStr);
					fprintf(stdout, "%s", inStr);
					break;
				case bcm5632_7_c:
					if(first) {
						fprintf(sscout, "Collecting 5632, port 7 diag output\n");
						fprintf(stdout, "Collecting 5632, port 7 diag output\n");
						first = 0;
					}
					/* throw out null strings & rcon prompts */
					if(!charP || !strncmp(charP, "RCON", 4)) break;
					fprintf(sscout, "%s", inStr);
					fprintf(stdout, "%s", inStr);
					break;
				case bcm5632_8_c:
					if(first) {
						fprintf(sscout, "Collecting 5632, port 8 diag output\n");
						fprintf(stdout, "Collecting 5632, port 8 diag output\n");
						first = 0;
					}
					/* throw out null strings & rcon prompts */
					if(!charP || !strncmp(charP, "RCON", 4)) break;
					fprintf(sscout, "%s", inStr);
					fprintf(stdout, "%s", inStr);
					break;
				case bcm5632_9_c:
					if(first) {
						fprintf(sscout, "Collecting 5632, port 9 diag output\n");
						fprintf(stdout, "Collecting 5632, port 9 diag output\n");
						first = 0;
					}
					/* throw out null strings & rcon prompts */
					if(!charP || !strncmp(charP, "RCON", 4)) break;
					fprintf(sscout, "%s", inStr);
					fprintf(stdout, "%s", inStr);
					break;
				case bcm5632_10_c:
					if(first) {
						fprintf(sscout, "Collecting 5632, port 10 diag output\n");
						fprintf(stdout, "Collecting 5632, port 10 diag output\n");
						first = 0;
					}
					/* throw out null strings & rcon prompts */
					if(!charP || !strncmp(charP, "RCON", 4)) break;
					fprintf(sscout, "%s", inStr);
					fprintf(stdout, "%s", inStr);
					break;
				case bcm5632_11_c:
					if(first) {
						fprintf(sscout, "Collecting 5632, port 11 diag output\n");
						fprintf(stdout, "Collecting 5632, port 11 diag output\n");
						first = 0;
					}
					/* throw out null strings & rcon prompts */
					if(!charP || !strncmp(charP, "RCON", 4)) break;
					fprintf(sscout, "%s", inStr);
					fprintf(stdout, "%s", inStr);
					break;
				case bcm5600_0_c:
					if(first) {
						fprintf(sscout, "Collecting 5600, port 0 diag output\n");
						fprintf(stdout, "Collecting 5600, port 0 diag output\n");
						first = 0;
					}
					/* throw out null strings & rcon prompts */
					if(!charP || !strncmp(charP, "RCON", 4)) break;
					fprintf(sscout, "%s", inStr);
					fprintf(stdout, "%s", inStr);
					break;
				case bcm5600_1_c:
					if(first) {
						fprintf(sscout, "Collecting 5600, port 1 diag output\n");
						fprintf(stdout, "Collecting 5600, port 1 diag output\n");
						first = 0;
					}
					/* throw out null strings & rcon prompts */
					if(!charP || !strncmp(charP, "RCON", 4)) break;
					fprintf(sscout, "%s", inStr);
					fprintf(stdout, "%s", inStr);
					break;
				case bcm5600_2_c:
					if(first) {
						fprintf(sscout, "Collecting 5600, port 2 diag output\n");
						fprintf(stdout, "Collecting 5600, port 2 diag output\n");
						first = 0;
					}
					/* throw out null strings & rcon prompts */
					if(!charP || !strncmp(charP, "RCON", 4)) break;
					fprintf(sscout, "%s", inStr);
					fprintf(stdout, "%s", inStr);
					break;
				case bcm5600_3_c:
					if(first) {
						fprintf(sscout, "Collecting 5600, port 3 diag output\n");
						fprintf(stdout, "Collecting 5600, port 3 diag output\n");
						first = 0;
					}
					/* throw out null strings & rcon prompts */
					if(!charP || !strncmp(charP, "RCON", 4)) break;
					fprintf(sscout, "%s", inStr);
					fprintf(stdout, "%s", inStr);
					break;
				case bcm5600_4_c:
					if(first) {
						fprintf(sscout, "Collecting 5600, port 4 diag output\n");
						fprintf(stdout, "Collecting 5600, port 4 diag output\n");
						first = 0;
					}
					/* throw out null strings & rcon prompts */
					if(!charP || !strncmp(charP, "RCON", 4)) break;
					fprintf(sscout, "%s", inStr);
					fprintf(stdout, "%s", inStr);
					break;
				case bcm5600_5_c:
					if(first) {
						fprintf(sscout, "Collecting 5600, port 5 diag output\n");
						fprintf(stdout, "Collecting 5600, port 5 diag output\n");
						first = 0;
					}
					/* throw out null strings & rcon prompts */
					if(!charP || !strncmp(charP, "RCON", 4)) break;
					fprintf(sscout, "%s", inStr);
					fprintf(stdout, "%s", inStr);
					break;
				case bcm5600_6_c:
					if(first) {
						fprintf(sscout, "Collecting 5600, port 6 diag output\n");
						fprintf(stdout, "Collecting 5600, port 6 diag output\n");
						first = 0;
					}
					/* throw out null strings & rcon prompts */
					if(!charP || !strncmp(charP, "RCON", 4)) break;
					fprintf(sscout, "%s", inStr);
					fprintf(stdout, "%s", inStr);
					break;
				case bcm5600_7_c:
					if(first) {
						fprintf(sscout, "Collecting 5600, port 7 diag output\n");
						fprintf(stdout, "Collecting 5600, port 7 diag output\n");
						first = 0;
					}
					/* throw out null strings & rcon prompts */
					if(!charP || !strncmp(charP, "RCON", 4)) break;
					fprintf(sscout, "%s", inStr);
					fprintf(stdout, "%s", inStr);
					break;
				default:
					printf("Error - command index out of bounds\n");
					break;
				}
			}
		}
		
	}
	shutdown_rcon_session();

	fclose(sscout);
	return(0);
}
