/*
 *-----------------------------------------------------------------
 *
 * Name: Hardware System Health Monitor 
 *
 * RCS:  $Id:
 *
 * Copyright (C) 2002, Agile Storage, Inc.
 *
 * Description: Parses through the three sysHealth log files
 * 							(sysHealthFP.log, sysHealthSSC.log, & sysHealthSP.log)
 * 							and produces summary file (sysHealthSum.log) in /tmp.
 *
 * Created by:	Christa Porter
 *
 * Date Created: 2 Oct 2002
 *
 *-----------------------------------------------------------------
 */

#include "sysHealth.h"

#define NUMFPREG 8
#define NUMFPPCI 14
#define NUMFP12500 15
#define NUMFPLSI 9
#define NUMFPNAT 28
#define NUMSPREG 8
#define NUMSPGALILEO 21
#define NUMSSCREG 8
#define NUMSSCPCIREG 5

char *get_regerror(int, regex_t*);

void parseFields(FILE*, FILE*, char*, char [][80], int, char*);

void parseMultFields(FILE*, FILE*, char*, char [][80], int, char*);

void parseSpFields(FILE*, FILE*, char*, char*, char [][80], int, char*);

void parseLsiFields(FILE*, FILE*, char*, char [][80], int, char*);

int sysHealthSum(int fp, int ssc, int sp) {

	FILE *fpin;
	FILE *sscin;
	FILE *spin;
	FILE *sumout;
	char fpFileEx[][80] = {	"Connecting to Rx/Tx SiByte",
													"Connecting to FP SiByte",
													".*BMC12500 ETH.*0",
													".*BMC12500 ETH.*2",
													"Collecting LSI MAC port 0.*",
													"Collecting LSI MAC port 1.*",
													"Collecting LSI MAC port 2.*",
													"Collecting LSI MAC port 3.*",
													"Collecting LSI MAC port 4.*",
													"Collecting LSI MAC port 5.*",
													"Collecting National MAC diag output"
												};
	char fpRegEx[][80] = {	"Transfer ID that had error code:",
													"Responder ID that reported the error:",
													"Data Transfer error code:",
													"Multiple errors:",
													"Correctable L2 cache data errors:",
													"Uncorrectable L2 cache data errors:",
													"Correctable L2 cache tag errors:",
													"Uncorrectable L2 cache tag errors:"
												};
	char fpPciEx[][80] = {	"Data parity error as master:",
													"Signaled target abort as target:",
													"Received target abort as master:",
													"Signaled master abort as master:",
													"System error (parity error on address):",
													"Detected parity error:",
													"SERR from other device:",
													"Aborted transfer due to TRDY timeout:",
													"Aborted transfer due to read retry timeout:",
													"Protocol error on incoming link:",
													"Receive FIFO overflow:",
													"Received non existent address error(End Of Chain):",
													"Response source ID does not match outstanding request:",
													"Received non existent address error(mapping problem):"
												};
	char fp12500Ex[][80] = {"Rx Bad Packets",
													"Rx Runt Errors",
													"Rx Over Sized Errors",
													"Rx FCS Errors",
													"Rx Length Errors",
													"Rx Code Errors",
													"Rx Dribble Packets",
													"Tx Bad Packets",
													"Tx Runt Errors",
													"Tx Over Sized Errors",
													"Tx FCS Errors",
													"Tx Collisions",
													"Tx Late Collision",
													"Tx Excess Collision",
													"Tx Abort"
													};
	char fpLsiEx[][80] = {"CRC align errors",
												"Undersized errors",
												"Oversized errors",
												"Fragment errors",
												"Jabber errors",
												"Tx underflows",
												"Rx symbol errors",
												"Tx errors",
												"Rx overflows"
												};
	char fpNatEx[][80] = {"RxQ 0 rxo err",
												"RxQ 0 rxa err",
												"RxQ 0 no buf err",
												"RxQ 0 no desc err",
												"RxQ 0 too long err",
												"RxQ 0 runt err",
												"RxQ 0 is err",
												"RxQ 0 crc err",
												"RxQ 0 fa err",
												"RxQ 0 lbp err",
												"RxQ 0 irl err",
												"TxQ 0 qFull err",
												"TxQ 0 txa err",
												"TxQ 0 tfu err",
												"TxQ 0 crs err",
												"TxQ 0 td err",
												"TxQ 0 ed err",
												"TxQ 0 owc err",
												"TxQ 0 ec err",
												"MIB0 RXErroredPkts",
												"MIB1 RXFCSErrors",
												"MIB2 RxFifoOverrun",
												"MIB3 RXFAErrors",
												"MIB4 RXSymbolErrs",
												"MIB5 RXBig",
												"MIB6 RXIRLErrors",
												"MIB7 RXBadOpcodes",
												"MIB10 TXSQEErrors"
												};

	char sscFileEx[][80] = { "Connecting to SSC card, slot 0",
													"Sending SSC Command: dm 0x24000498 1",
													"Sending SSC Command: dm 0x24001d58 1",
													"Sending SSC Command: dm 0x24001dd8 1"
												};
	char sscRegEx[][80] = {  "Addr Out of range:",
													"Bad Addr Parity:",
                          "Transfer type Violation:",
                          "Access To protected region:",
                          "Write to write protected region:",
                          "Read from caching protected region:",
                          "Bad write data parity detected:",
                          "Bad read data parity detected:"
                       };
	char sscPCIEndEx[][80] = { "End PCI0",
														"End PCI1"
													};

	char spFileEx[][80] = { "Connecting to SP card, slot 2, slice 0",
													"Sending SP Command: dm 0xb4000498 1",
													"Sending SP Command: dm 0xb4001d58 1",
													"Sending SP Command: dm 0xb4001dd8 1",
													"Collecting Galileo Ethernet port 1 ",
													"Collecting Galileo Ethernet port 2 ",
													"Collecting Galileo Ethernet port 3 ",
													"Connecting to SP card, slot 2, slice 1",
													"Sending SP Command: dm 0xb4000498 1",
													"Sending SP Command: dm 0xb4001d58 1",
													"Sending SP Command: dm 0xb4001dd8 1",
													"Collecting Galileo Ethernet port 1 ",
													"Collecting Galileo Ethernet port 2 ",
													"Collecting Galileo Ethernet port 3 ",
													"Connecting to SP card, slot 2, slice 2",
													"Sending SP Command: dm 0xb4000498 1",
													"Sending SP Command: dm 0xb4001d58 1",
													"Sending SP Command: dm 0xb4001dd8 1",
													"Collecting Galileo Ethernet port 1 ",
													"Collecting Galileo Ethernet port 2 ",
													"Collecting Galileo Ethernet port 3 ",
													"Connecting to SP card, slot 2, slice 3",
													"Sending SP Command: dm 0xb4000498 1",
													"Sending SP Command: dm 0xb4001d58 1",
													"Sending SP Command: dm 0xb4001dd8 1",
													"Collecting Galileo Ethernet port 1 ",
													"Collecting Galileo Ethernet port 2 ",
													"Collecting Galileo Ethernet port 3 "
												};
	char spRegEx[][80] = {  "Addr Out of range:",
													"Bad Addr Parity:",
                          "Transfer type Violation:",
                          "Access To protected region:",
                          "Write to write protected region:",
                          "Read from caching protected region:",
                          "Bad write data parity detected:",
                          "Bad read data parity detected:"
                       };
	char spPCIEndEx[][80] = { "End PCI0",
														"End PCI1"
													};
	char sp64240Ex[][80] = { "CRC errors",
													 "Oversize frames",
                           "Fragments",
                           "Jabber",
                           "Collisions",
                           "Late collisions",
                           "Rx errors",
                           "Undersize frames",
                           "Rx No Buff Errs",
                           "RxQ 0 too short",
                           "RxQ 0 too long",
                           "RxQ 0 overrun",
                           "RxQ 0 collision",
                           "RxQ 0 crc err",
                           "RxQ 0 no buffer",
                           "RxQ 0 no desc",
                           "TxQ 0 qFull err",
                           "TxQ 0 collision",
                           "TxQ 0 late collision",
                           "TxQ 0 underrun",
                           "TxQ 0 retransmit limit"
                         };

	char banner[80];

	if(!fp && !ssc && !sp) {
		fprintf(stderr, "Error - no cards specified for summary\n");
		exit(1);
	}

	if(fp)
		if((fpin = fopen(FPFILE, "r")) == NULL) {
			fprintf(stderr, "Cannot open %s\n", FPFILE);
			exit(1);
		}
	if(ssc)
		if((sscin = fopen(SSCFILE, "r")) == NULL) {
			fprintf(stderr, "Cannot open %s\n", SSCFILE);
			exit(1);
		}

	if(sp)
		if((spin = fopen(SPFILE, "r")) == NULL) {
			fprintf(stderr, "Cannot open %s\n", SPFILE);
			exit(1);
		}

	if((sumout = fopen(SUMFILE, "w")) == NULL) {
		fprintf(stderr, "Cannot open %s\n", SUMFILE);
		exit(1);
	}

 	/* FP statics collection */
	fprintf(sumout, "****************************************************************\n");
	fprintf(sumout, "*************** NFX System Harware Health Report ***************\n");
	fprintf(sumout, "****************************************************************\n");
	fprintf(sumout, "*************************** Summary ****************************\n");
	fprintf(sumout, "****************************************************************\n\n");

	fprintf(sumout, "****************************************************************\n");
	fprintf(sumout, "********************** Processor Errors ************************\n");
	fprintf(sumout, "****************************************************************\n");

	if(fp) {
		strcpy(banner, "******************** FP Tx/Rx Proc Errors **********************");
		parseFields(fpin, sumout, fpFileEx[0], fpRegEx, NUMFPREG, banner);

		strcpy(banner, "********************** FP FP Proc Errors ***********************");
		parseFields(fpin, sumout, fpFileEx[1], fpRegEx, NUMFPREG, banner);

		strcpy(banner, "**************** FP Tx/Rx FIFO Port 0 Errors *******************");
		parseFields(fpin, sumout, fpFileEx[2], fp12500Ex, NUMFP12500, banner);

		strcpy(banner, "**************** FP Tx/Rx FIFO Port 2 Errors *******************");
		parseFields(fpin, sumout, fpFileEx[3], fp12500Ex, NUMFP12500, banner);

		strcpy(banner, "***************** FP FP FIFO Port 0 Errors *********************");
		parseFields(fpin, sumout, fpFileEx[2], fp12500Ex, NUMFP12500, banner);

		strcpy(banner, "***************** FP FP FIFO Port 2 Errors *********************");
		parseFields(fpin, sumout, fpFileEx[3], fp12500Ex, NUMFP12500, banner);

		rewind(fpin);
	}

	if(ssc) {
		strcpy(banner,"******************* SSC card CPU bus Errors ********************");
		parseSpFields(sscin, sumout, sscFileEx[0], NULL,  sscRegEx, NUMSSCREG, banner);
		strcpy(banner,"******************* SSC card SDRAM ECC Errors ******************");
		parseSpFields(sscin, sumout, sscFileEx[1], NULL,  sscRegEx, 0, banner);
	}

	if(sp) {
		strcpy(banner,"**************** SP card, slice 0 CPU bus Errors ***************");
		parseSpFields(spin, sumout, spFileEx[0], NULL,  spRegEx, NUMSPREG, banner);
		strcpy(banner,"************** SP card, slice 0 SDRAM ECC Errors ***************");
		parseSpFields(spin, sumout, spFileEx[1], NULL,  spRegEx, 0, banner);

		strcpy(banner,"**************** SP card, slice 1 CPU bus Errors ***************");
		parseSpFields(spin, sumout, spFileEx[7], NULL, spRegEx, NUMSPREG, banner);
		strcpy(banner,"************** SP card, slice 1 SDRAM ECC Errors ***************");
		parseSpFields(spin, sumout, spFileEx[8], NULL, spRegEx, 0, banner);

		strcpy(banner,"**************** SP card, slice 2 CPU bus Errors ***************");
		parseSpFields(spin, sumout, spFileEx[14], NULL, spRegEx, NUMSPREG, banner);
		strcpy(banner,"************** SP card, slice 2 SDRAM ECC Errors ***************");
		parseSpFields(spin, sumout, spFileEx[15], NULL, spRegEx, 0, banner);

		strcpy(banner,"**************** SP card, slice 3 CPU bus Errors ***************");
		parseSpFields(spin, sumout, spFileEx[21], NULL, spRegEx, NUMSPREG, banner);
		strcpy(banner,"************** SP card, slice 3 SDRAM ECC Errors ***************");
		parseSpFields(spin, sumout, spFileEx[22], NULL, spRegEx, 0, banner);

		rewind(spin);
	
	}

	fprintf(sumout, "\n****************************************************************\n");
	fprintf(sumout, "************************* PCI Errors ***************************\n");
	fprintf(sumout, "****************************************************************\n");

	if(fp) {
		strcpy(banner, "******************** FP Tx/Rx PCI Errors ***********************");
		parseFields(fpin, sumout, fpFileEx[0], fpPciEx, NUMFPPCI, banner);

		rewind(fpin);
	}

	if(ssc) {
		strcpy(banner,"********************* SSC card PCI0 Errors *********************");
		parseSpFields(sscin, sumout, sscFileEx[2], sscPCIEndEx[0],  sscRegEx, NUMSSCPCIREG, banner);
		strcpy(banner,"********************* SSC card PCI1 Errors *********************");
		parseSpFields(sscin, sumout, sscFileEx[3], sscPCIEndEx[1], sscRegEx, NUMSSCPCIREG, banner);
	}

	if(sp) {
		strcpy(banner,"***************** SP card, slice 0 PCI0 Errors *****************");
		parseSpFields(spin, sumout, spFileEx[2], spPCIEndEx[0],  spRegEx, 5, banner);
		strcpy(banner,"***************** SP card, slice 0 PCI1 Errors *****************");
		parseSpFields(spin, sumout, spFileEx[3], spPCIEndEx[1], spRegEx, 5, banner);

		strcpy(banner,"***************** SP card, slice 1 PCI0 Errors *****************");
		parseSpFields(spin, sumout, spFileEx[9], spPCIEndEx[0], spRegEx, 5, banner);
		strcpy(banner,"***************** SP card, slice 1 PCI1 Errors *****************");
		parseSpFields(spin, sumout, spFileEx[10], spPCIEndEx[1], spRegEx, 5, banner);

		strcpy(banner,"***************** SP card, slice 2 PCI0 Errors *****************");
		parseSpFields(spin, sumout, spFileEx[16], spPCIEndEx[0], spRegEx, 5, banner);
		strcpy(banner,"***************** SP card, slice 2 PCI1 Errors *****************");
		parseSpFields(spin, sumout, spFileEx[17], spPCIEndEx[1], spRegEx, 5, banner);

		strcpy(banner,"***************** SP card, slice 3 PCI0 Errors *****************");
		parseSpFields(spin, sumout, spFileEx[23], spPCIEndEx[0], spRegEx, 5, banner);
		strcpy(banner,"***************** SP card, slice 3 PCI1 Errors *****************");
		parseSpFields(spin, sumout, spFileEx[24], spPCIEndEx[1], spRegEx, 5, banner);

		rewind(spin);
	}

	fprintf(sumout, "\n****************************************************************\n");
	fprintf(sumout, "************************* MAC Errors ***************************\n");
	fprintf(sumout, "****************************************************************\n");

	if(fp) {
		strcpy(banner, "***************** FP Tx/Rx National MAC Errors *****************");
		parseMultFields(fpin, sumout, fpFileEx[10], fpNatEx, NUMFPNAT, banner);

		strcpy(banner, "**************** FP Tx/Rx LSI Port 0 MAC Errors ****************");
		parseLsiFields(fpin, sumout, fpFileEx[4], fpLsiEx, NUMFPLSI, banner);

		strcpy(banner, "**************** FP Tx/Rx LSI Port 1 MAC Errors ****************");
		parseLsiFields(fpin, sumout, fpFileEx[5], fpLsiEx, NUMFPLSI, banner);

		strcpy(banner, "**************** FP Tx/Rx LSI Port 2 MAC Errors ****************");
		parseLsiFields(fpin, sumout, fpFileEx[6], fpLsiEx, NUMFPLSI, banner);

		strcpy(banner, "**************** FP Tx/Rx LSI Port 3 MAC Errors ****************");
		parseLsiFields(fpin, sumout, fpFileEx[7], fpLsiEx, NUMFPLSI, banner);

		strcpy(banner, "***************** FP FP LSI Port 0 MAC Errors ******************");
		parseLsiFields(fpin, sumout, fpFileEx[4], fpLsiEx, NUMFPLSI, banner);

		strcpy(banner, "***************** FP FP LSI Port 1 MAC Errors ******************");
		parseLsiFields(fpin, sumout, fpFileEx[5], fpLsiEx, NUMFPLSI, banner);

		strcpy(banner, "***************** FP FP LSI Port 2 MAC Errors ******************");
		parseLsiFields(fpin, sumout, fpFileEx[6], fpLsiEx, NUMFPLSI, banner);

		strcpy(banner, "***************** FP FP LSI Port 3 MAC Errors ******************");
		parseLsiFields(fpin, sumout, fpFileEx[7], fpLsiEx, NUMFPLSI, banner);

		strcpy(banner, "***************** FP FP LSI Port 4 MAC Errors ******************");
		parseLsiFields(fpin, sumout, fpFileEx[8], fpLsiEx, NUMFPLSI, banner);

		strcpy(banner, "***************** FP FP LSI Port 5 MAC Errors ******************");
		parseLsiFields(fpin, sumout, fpFileEx[9], fpLsiEx, NUMFPLSI, banner);
	}

	if(sp) {
		strcpy(banner,"*************** SP card, slice 0 Galileo Port 1 ****************");
		parseSpFields(spin, sumout, spFileEx[4], NULL, sp64240Ex, NUMSPGALILEO, banner);
		strcpy(banner,"*************** SP card, slice 0 Galileo Port 2 ****************");
		parseSpFields(spin, sumout, spFileEx[5], NULL, sp64240Ex, NUMSPGALILEO, banner);
		strcpy(banner,"*************** SP card, slice 0 Galileo Port 3 ****************");
		parseSpFields(spin, sumout, spFileEx[6], NULL, sp64240Ex, NUMSPGALILEO, banner);

		strcpy(banner,"*************** SP card, slice 1 Galileo Port 1 ****************");
		parseSpFields(spin, sumout, spFileEx[11], NULL, sp64240Ex, NUMSPGALILEO, banner);
		strcpy(banner,"*************** SP card, slice 1 Galileo Port 2 ****************");
		parseSpFields(spin, sumout, spFileEx[12], NULL, sp64240Ex, NUMSPGALILEO, banner);
		strcpy(banner,"*************** SP card, slice 1 Galileo Port 3 ****************");
		parseSpFields(spin, sumout, spFileEx[13], NULL, sp64240Ex, NUMSPGALILEO, banner);

		strcpy(banner,"*************** SP card, slice 2 Galileo Port 1 ****************");
		parseSpFields(spin, sumout, spFileEx[18], NULL, sp64240Ex, NUMSPGALILEO, banner);
		strcpy(banner,"*************** SP card, slice 2 Galileo Port 2 ****************");
		parseSpFields(spin, sumout, spFileEx[19], NULL, sp64240Ex, NUMSPGALILEO, banner);
		strcpy(banner,"*************** SP card, slice 2 Galileo Port 3 ****************");
		parseSpFields(spin, sumout, spFileEx[20], NULL, sp64240Ex, NUMSPGALILEO, banner);
		
		strcpy(banner,"*************** SP card, slice 3 Galileo Port 1 ****************");
		parseSpFields(spin, sumout, spFileEx[25], NULL, sp64240Ex, NUMSPGALILEO, banner);
		strcpy(banner,"*************** SP card, slice 3 Galileo Port 2 ****************");
		parseSpFields(spin, sumout, spFileEx[26], NULL, sp64240Ex, NUMSPGALILEO, banner);
		strcpy(banner,"*************** SP card, slice 3 Galileo Port 3 ****************");
		parseSpFields(spin, sumout, spFileEx[27], NULL, sp64240Ex, NUMSPGALILEO, banner);
	}

	if(fp) fclose(fpin);
	if(ssc) fclose(sscin);
	if(sp) fclose(spin);
	fclose(sumout);
	return(0);
}

void parseFields(FILE *in, FILE *out, char *startLabel, char parseLabels[][80], int numLabels, char *sumBanner) {
	regex_t compEx;
	char inStr[128];
	char parseStr[128];
	char tmpStr[128];
	int errCode;
	char *errP;
	char *endP;
	char *charP;
	unsigned int value;
	int i;
	char valueLabel[] = "value:";
	int group;

	/* check FP log file for FP proc errors */
	while(!feof(in)) {
		fgets(inStr, MAXLINE, in);
		if(ferror(in)) {
			printf("Error in fgets of parse input file \n");
			exit(1);
		}
#ifdef DEBUG
		fprintf(stdout, "%s", inStr);
#endif

		/* compile regex for start label*/
		if((errCode = regcomp(&compEx, startLabel,REG_NEWLINE))) {
			errP = get_regerror(errCode, &compEx);
			fprintf(stderr, "regcomp ERROR - %s\n", errP);
			free(errP);
			exit(1);
		}
		/* check for start label match */
		if((errCode = regexec(&compEx, inStr, (size_t)0, NULL, 0))) {
			errP = get_regerror(errCode, &compEx);
#ifdef DEBUG
			fprintf(out, "regexec ERROR - %s\n", errP);
#endif
			free(errP);
			continue;
		} else {
#ifdef DEBUG
			fprintf(out, "Matched start label: %s", inStr);
#endif

			/* print banner */
			fprintf(out, "\n%s\n", sumBanner);

			i = 0;
			group = 0;
			if((errCode = regcomp(&compEx, parseLabels[i],REG_NEWLINE))) {
				errP = get_regerror(errCode, &compEx);
				fprintf(stderr, "regcomp ERROR - %s\n", errP);
				free(errP);
				exit(1);
			}

			/* parse for specific labels in FP log file */
			while(!feof(in)) {
				fgets(inStr, MAXLINE, in);
				if(ferror(in)) {
					printf("Error in fgets of file %s\n", FPFILE);
					exit(1);
				}
				/* output file input */
				fprintf(stdout, "%s", inStr);

				strcpy(parseStr, inStr);

				/* compile regex for register value label*/
				if((errCode = regcomp(&compEx, valueLabel,REG_NEWLINE))) {
					errP = get_regerror(errCode, &compEx);
					fprintf(stderr, "regcomp ERROR - %s\n", errP);
					free(errP);
					exit(1);
				}
				/* check for register value match */
				if((errCode = regexec(&compEx, parseStr, (size_t)0, NULL, 0))) {
					errP = get_regerror(errCode, &compEx);
#ifdef DEBUG
					fprintf(out, "regexec ERROR - %s\n", errP);
#endif
					free(errP);
				} else {
#ifdef DEBUG
					fprintf(out, "Matched register value label: %s", parseStr);
#endif
					strcpy(tmpStr, parseStr);
					group = 1;
				}
				/* recompile back to label we are looking for */
				if((errCode = regcomp(&compEx, parseLabels[i],REG_NEWLINE))) {
					errP = get_regerror(errCode, &compEx);
					fprintf(stderr, "regcomp ERROR - %s\n", errP);
					free(errP);
					exit(1);
				}

				/* check for specific label */
				if((errCode = regexec(&compEx, parseStr, (size_t)0, NULL, 0))) {
					errP = get_regerror(errCode, &compEx);
#ifdef DEBUG
					fprintf(out, "regexec ERROR - %s\n", errP);
#endif
					free(errP);
				} else {	/* match */
#ifdef DEBUG
					fprintf(out, "\nMatched: %s\n", parseStr);
#endif
					charP = strtok(parseStr, ":"); /* remove label */
					charP = strtok(NULL, " \t\n"); /* grab value */
					if(charP) {
#ifdef DEBUG
						fprintf(out, "charP = %s\n", charP);
						fprintf(stdout, "charP = %s\n", charP);
#endif
						if((value = (unsigned int)strtoul(charP, &endP, 16))) {
							if(group) {
								fprintf(out, "******** Error found *********\n");
								fprintf(stdout, "******** Error found *********\n");
								fprintf(out, "%s", tmpStr);
								fprintf(stdout, "%s", tmpStr);
								group = 0;
							}
							fprintf(out, "%s", inStr);
							fprintf(stdout, "%s", inStr);
						}
					} else {
						fprintf(out, "Error - no value for matched label\n");
						fprintf(stderr, "Error - no value for matched label\n");
					}
					/* check for end of current parsing */
					if(++i >= numLabels) break;
				}
			}
		}
		break;
	}
	regfree(&compEx);
}

void parseMultFields(FILE *in, FILE *out, char *startLabel, char parseLabels[][80], int numLabels, char *sumBanner) {
	regex_t compEx;
	char inStr[128];
	char parseStr[128];
	char *tmpLbl;
	int errCode;
	char *errP;
	char *endP;
	char *charP;
	char *tmpP;
	unsigned int value;
	int i, group;

	/* check FP log file for FP proc errors */
	while(!feof(in)) {
		fgets(inStr, MAXLINE, in);
		if(ferror(in)) {
			printf("Error in fgets of parse input file \n");
			exit(1);
		}
#ifdef DEBUG
		fprintf(stdout, "%s", inStr);
#endif

		/* compile regex for start label*/
		if((errCode = regcomp(&compEx, startLabel,REG_NEWLINE))) {
			errP = get_regerror(errCode, &compEx);
			fprintf(stderr, "regcomp ERROR - %s\n", errP);
			free(errP);
			exit(1);
		}
		/* check for start label match */
		if((errCode = regexec(&compEx, inStr, (size_t)0, NULL, 0))) {
			errP = get_regerror(errCode, &compEx);
#ifdef DEBUG
			fprintf(out, "regexec ERROR - %s\n", errP);
#endif
			free(errP);
			continue;
		} else {
#ifdef DEBUG
			fprintf(out, "Matched start label: %s", inStr);
#endif

			/* print banner */
			fprintf(out, "\n%s\n", sumBanner);

			i = 0;
			group = 1;
			/* parse for specific labels in FP log file */
			while(!feof(in)) {
				fgets(inStr, MAXLINE, in);
				if(ferror(in)) {
					printf("Error in fgets of file %s\n", FPFILE);
					exit(1);
				}
#ifdef DEBUG
				fprintf(out, "Input string = %s", inStr);
#endif

				strcpy(parseStr, inStr);

				/* compile for first possible label in line */
				if((errCode = regcomp(&compEx, parseLabels[i],REG_NEWLINE))) {
					errP = get_regerror(errCode, &compEx);
					fprintf(stderr, "regcomp ERROR - %s\n", errP);
					free(errP);
					exit(1);
				}
				/* check for first possible label in line */
				if((errCode = regexec(&compEx, parseStr, (size_t)0, NULL, 0))) {
					errP = get_regerror(errCode, &compEx);
#ifdef DEBUG
					fprintf(out, "regexec ERROR - %s\n", errP);
#endif
					free(errP);
				} else { /* matched first label */
#ifdef DEBUG
					fprintf(out, "Matched: %s", parseStr);
#endif
					tmpLbl = strtok(parseStr, ":"); /* remove label */
					charP = strtok(NULL, " \t\n"); /* grab value */
					/* check if label is in first column */
					if((errCode = regexec(&compEx, tmpLbl, (size_t)0, NULL, 0))) {
						errP = get_regerror(errCode, &compEx);
#ifdef DEBUG
						fprintf(out, "regexec ERROR - %s\n", errP);
#endif
						free(errP);
					} else { /* matched first label */
						if(charP) {
#ifdef DEBUG
							fprintf(out, "charP = %s\n", charP);
#endif
							if((value = (unsigned int)strtoul(charP, &endP, 16))) {
								if(group) {
									fprintf(out, "******** Error found *********\n");
									fprintf(stdout, "******** Error found *********\n");
									group = 0;
								}
								strcat(tmpLbl, charP);
								fprintf(out, "%s\n", tmpLbl);
								fprintf(stdout, "%s\n", tmpLbl);
							}
						} else {
							fprintf(out, "Error - no value for matched label\n");
							fprintf(stderr, "Error - no value for matched label\n");
						}
						/* check for end of current parsing */
						if(++i >= numLabels) break;
					}
				}
				/* compile for second possible label in line or use same label to match with second column */
				if((errCode = regcomp(&compEx, parseLabels[i],REG_NEWLINE))) {
					errP = get_regerror(errCode, &compEx);
					fprintf(out, "regcomp ERROR - %s\n", errP);
					fprintf(stderr, "regcomp ERROR - %s\n", errP);
					free(errP);
					exit(1);
				}

				/* check for second possible label in line */
				if((errCode = regexec(&compEx, inStr, (size_t)0, NULL, 0))) {
					errP = get_regerror(errCode, &compEx);
#ifdef DEBUG
					fprintf(out, "regexec ERROR - %s\n", errP);
#endif
					free(errP);
				} else {	/* match */
#ifdef DEBUG
					fprintf(out, "\nMatched: %s\n", inStr);
#endif
					tmpLbl = strtok(NULL, ":"); /* grab label */
					tmpP = strstr(tmpLbl, parseLabels[i]); /* remove leading spaces */
					charP = strtok(NULL, " \t\n"); /* grab value */
					if(charP) {
#ifdef DEBUG
						fprintf(out, "charP = %s\n", charP);
#endif
						if((value = (unsigned int)strtoul(charP, &endP, 16))) {
							if(group) {
								fprintf(out, "******** Error found *********\n");
								fprintf(stdout, "******** Error found *********\n");
								group = 0;
							}
							strcat(tmpP, "\t\t");
							strcat(tmpP, charP);
							fprintf(out, "%s\n", tmpP);
							fprintf(stdout, "%s\n", tmpP);
						}
					} else {
						fprintf(out, "Error - no value for matched label\n");
						fprintf(stderr, "Error - no value for matched label\n");
					}
					/* check for end of current parsing */
					if(++i >= numLabels) break;
				}
			}
		}
		break;
	}
	regfree(&compEx);
}

void parseSpFields(FILE *in,
										FILE *out,
										char *startLabel,
										char* endLabel,
										char parseLabels[][80],
										int numLabels,
										char *sumBanner) {
	regex_t compEx;
	char inStr[128];
	char parseStr[128];
	char tmpStr[128];
	char prevStr[128];
	int errCode;
	char *errP;
	char *endP;
	char *charP;
	unsigned int value;
	int i;
	char valueLabel[] = "value:";
	int group;

	/* check FP log file for FP proc errors */
	while(!feof(in)) {
		fgets(inStr, MAXLINE, in);
		if(ferror(in)) {
			printf("Error in fgets of parse input file \n");
			exit(1);
		}
#ifdef DEBUG
		fprintf(stdout, "%s", inStr);
#endif

		/* compile regex for FP label*/
		if((errCode = regcomp(&compEx, startLabel,REG_NEWLINE))) {
			errP = get_regerror(errCode, &compEx);
			fprintf(stderr, "regcomp ERROR - %s\n", errP);
			free(errP);
			exit(1);
		}
		/* check for match */
		if((errCode = regexec(&compEx, inStr, (size_t)0, NULL, 0))) {
			errP = get_regerror(errCode, &compEx);
#ifdef DEBUG
			fprintf(out, "regexec ERROR - %s\n", errP);
#endif
			free(errP);
			continue;
		} else {
#ifdef DEBUG
			fprintf(out, "Matched start label: %s", inStr);
#endif

			/* print banner */
			fprintf(out, "\n%s\n", sumBanner);

			i = 0;
			if((errCode = regcomp(&compEx, parseLabels[i],REG_NEWLINE))) {
				errP = get_regerror(errCode, &compEx);
				fprintf(stderr, "regcomp ERROR - %s\n", errP);
				free(errP);
				exit(1);
			}

			/* parse for specific labels in FP log file */
			while(!feof(in)) {
				fgets(inStr, MAXLINE, in);
				if(ferror(in)) {
					printf("Error in fgets of file %s\n", FPFILE);
					exit(1);
				}
#ifdef DEBUG
				fprintf(out, "Input string = %s", inStr);
#endif

				strcpy(parseStr, inStr);

				/* check for the endLabel. If found return from the function */
				if(endLabel != NULL) {
				  if((errCode = regcomp(&compEx, endLabel,REG_NEWLINE))) {
					  errP = get_regerror(errCode, &compEx);
					  fprintf(stderr, "regcomp ERROR - %s\n", errP);
					  free(errP);
					  exit(1);
				  }
				  if((errCode = regexec(&compEx, parseStr, (size_t)0, NULL, 0))) {
					  errP = get_regerror(errCode, &compEx);
#ifdef DEBUG
					  fprintf(out, "regexec ERROR - %s\n", errP);
#endif
					  free(errP);
				  } else {
					  return ;
				  }
				}
				/* compile regex for register value label*/
				if((errCode = regcomp(&compEx, valueLabel,REG_NEWLINE))) {
					errP = get_regerror(errCode, &compEx);
					fprintf(stderr, "regcomp ERROR - %s\n", errP);
					free(errP);
					exit(1);
				}
				/* check for register value match */
				if((errCode = regexec(&compEx, parseStr, (size_t)0, NULL, 0))) {
					errP = get_regerror(errCode, &compEx);
#ifdef DEBUG
					fprintf(out, "regexec ERROR - %s\n", errP);
#endif
					free(errP);
				} else {
#ifdef DEBUG
					fprintf(out, "Matched register value label: %s", parseStr);
#endif
					group = 1;

					/* Check if the parsed register without subfields has any errors */
					/* registers with subfields is checked through the label parsing  */
					strcpy(tmpStr, parseStr);
					strcpy(prevStr, parseStr);
					charP = strtok(prevStr, ":"); /* remove label */
					charP = strtok(NULL, " \t\n"); /* grab value */
					if(charP) {
#ifdef DEBUG
						fprintf(out, "charP = %s\n", charP);
#endif
						/* skip slice 1-3 address out of range errors in CPU error cause register */
						if((!strcmp(startLabel, "Connecting to SP card, slot 2, slice 1") ||
								!strcmp(startLabel, "Connecting to SP card, slot 2, slice 2") || 
								!strcmp(startLabel, "Connecting to SP card, slot 2, slice 3")) && 
								!strncmp(parseStr, "SP CPU Error Cause Register value:", 34)) {
							value = (unsigned int)strtoul(charP, &endP, 16) & 0xFFFFFFFE;
							i++;
						} else {
							value = (unsigned int)strtoul(charP, &endP, 16);
						}
						if(value) {
							if(group) {
								fprintf(out, "******** Error found *********\n");
								fprintf(stdout, "******** Error found *********\n");
								group = 0;
							}
							fprintf(out, "%s", tmpStr);
							fprintf(stdout, "%s", tmpStr);
						}
					}
				}
				/* recompile back to label we are looking for */
				if((errCode = regcomp(&compEx, parseLabels[i],REG_NEWLINE))) {
					errP = get_regerror(errCode, &compEx);
					fprintf(stderr, "regcomp ERROR - %s\n", errP);
					free(errP);
					exit(1);
				}

				/* check for specific label */
				if((errCode = regexec(&compEx, parseStr, (size_t)0, NULL, 0))) {
					errP = get_regerror(errCode, &compEx);
#ifdef DEBUG
					fprintf(out, "regexec ERROR - %s\n", errP);
#endif
					free(errP);
				} else {	/* match */
#ifdef DEBUG
					fprintf(out, "\nMatched: %s\n", parseStr);
#endif
					strcpy(tmpStr, parseStr);
					charP = strtok(parseStr, ":"); /* remove label */
					charP = strtok(NULL, " \t\n"); /* grab value */
					if(charP) {
#ifdef DEBUG
						fprintf(out, "charP = %s\n", charP);
#endif
						if((value = (unsigned int)strtoul(charP, &endP, 16))) {
							  fprintf(out, "%s", inStr);
							  fprintf(stdout, "%s", inStr);
						}
					} else {
						fprintf(out, "Error - no value for matched label\n");
						fprintf(stderr, "Error - no value for matched label\n");
					}
					/* check for end of current parsing */
					if(++i >= numLabels) break;
				}
				if((numLabels == 0) && (group == 1))
					break;
			}
		}
		break;
	}
	regfree(&compEx);
}

void parseLsiFields(FILE *in, FILE *out, char *startLabel, char parseLabels[][80], int numLabels, char *sumBanner) {
	regex_t compEx;
	char inStr[128];
	char parseStr[128];
	int errCode;
	char *errP;
	char *endP;
	char *charP;
	unsigned int value;
	int i;
	int group;

	/* check FP log file for FP proc errors */
	while(!feof(in)) {
		fgets(inStr, MAXLINE, in);
		if(ferror(in)) {
			printf("Error in fgets of parse input file \n");
			exit(1);
		}
#ifdef DEBUG
		fprintf(stdout, "%s", inStr);
#endif

		/* compile regex for start label*/
		if((errCode = regcomp(&compEx, startLabel,REG_NEWLINE))) {
			errP = get_regerror(errCode, &compEx);
			fprintf(stderr, "regcomp ERROR - %s\n", errP);
			free(errP);
			exit(1);
		}
		/* check for start label match */
		if((errCode = regexec(&compEx, inStr, (size_t)0, NULL, 0))) {
			errP = get_regerror(errCode, &compEx);
#ifdef DEBUG
			fprintf(out, "regexec ERROR - %s\n", errP);
#endif
			free(errP);
			continue;
		} else {
#ifdef DEBUG
			fprintf(out, "Matched start label: %s", inStr);
#endif

			/* print banner */
			fprintf(out, "\n%s\n", sumBanner);

			i = 0;
			group = 1;
			if((errCode = regcomp(&compEx, parseLabels[i],REG_NEWLINE))) {
				errP = get_regerror(errCode, &compEx);
				fprintf(stderr, "regcomp ERROR - %s\n", errP);
				free(errP);
				exit(1);
			}

			/* parse for specific labels in log file */
			while(!feof(in)) {
				fgets(inStr, MAXLINE, in);
				if(ferror(in)) {
					printf("Error in fgets of file %s\n", FPFILE);
					exit(1);
				}
				/* output file input */
#ifdef DEBUG
				fprintf(out, "%s", inStr);
#endif

				strcpy(parseStr, inStr);

				/* compile to label we are looking for */
				if((errCode = regcomp(&compEx, parseLabels[i],REG_NEWLINE))) {
					errP = get_regerror(errCode, &compEx);
					fprintf(stderr, "regcomp ERROR - %s\n", errP);
					free(errP);
					exit(1);
				}

				/* check for specific label */
				if((errCode = regexec(&compEx, parseStr, (size_t)0, NULL, 0))) {
					errP = get_regerror(errCode, &compEx);
#ifdef DEBUG
					fprintf(out, "regexec ERROR - %s\n", errP);
#endif
					free(errP);
				} else {	/* match */
#ifdef DEBUG
					fprintf(out, "\nMatched: %s\n", parseStr);
#endif
					charP = strtok(parseStr, ":"); /* remove label */
					charP = strtok(NULL, " \t\n"); /* grab value */
					if(charP) {
#ifdef DEBUG
						fprintf(out, "charP = %s\n", charP);
#endif
						if((value = (unsigned int)strtoul(charP, &endP, 16))) {
							if(group) {
								fprintf(out, "******** Error found *********\n");
								fprintf(stdout, "******** Error found *********\n");
								group = 0;
							}
							fprintf(out, "%s", inStr);
							fprintf(stdout, "%s", inStr);
						}
					} else {
						fprintf(out, "Error - no value for matched label\n");
						fprintf(stderr, "Error - no value for matched label\n");
					}
					/* check for end of current parsing */
					if(++i >= numLabels) break;
				}
			}
		}
		break;
	}
	regfree(&compEx);
}

char *get_regerror(int errcode, regex_t *compiled) {
	size_t length = regerror(errcode, compiled, NULL, 0);
	char *buffer = (char *)malloc(length);
	if(!buffer) {
		fprintf(stderr, "ERROR - Can't allocate buffer memory\n");
		exit(1);
	}
	(void) regerror(errcode, compiled, buffer, length);
	return buffer;
}
