/* vi:ts=4
 *-----------------------------------------------------------------
 *
 * Name: ssc-chassis
 *
 * RCS:  $Id: ssc-sys-dvt.c,v 1.6 2002/10/08 15:18:35 lipikaathome Exp $
 *
 * Copyright (C) 2002, Agile Storage, Inc.
 *
 * Description: This is the "remote console" application butchered into
 *				a view chassis application where a BSD_VIEW_CHASSIS message
 *				is delivered to the MGMT cpu and the chassis mapping
 *				is returned and displayed.  While the code has been butchered,
 *				to bend to my will, I have left the comments vintage.
 *
 * Created by:	eric choi
 *
 * Date Created: 19 Feb 2002
 *
 *-----------------------------------------------------------------
 */
#define SEND_SYS_DVT_UDP	1

#include <sys/types.h>
#include <sys/time.h>
#include <sys/socket.h>
#include <neteee/eee.h>
#include <pthread.h>
#include <stdio.h>
#include <stdlib.h>
#include <limits.h>
#include <unistd.h>
#include <signal.h>
#ifdef DVT_REV2		
#include "../sm-eee/eee.h"
#include "../sm-eee/eee-test.h"
#endif
#include "nfx-incl.h"
#include "dvt-helper.h"
#include "exports/anpssc-api.h"		// For SSC_SLOT define
#include "exports/eee-error.h"		// Error code definitions

static struct sockaddr_eee ourPeer;		// To whom we are speaking
static int Sock;						// Use this to send.
static int myPid;						// Who we are

int	glob_num_bytes	= 64;
int glob_buf_size	= 1;

void sysdvt_stats_display(dvt_test_stats * d_stats);
int send_dvt_msg(int slot, int cpu, char *test, sys_dvt_control dvtmsg);

/*-----------------------------------------------------------------
 * Name: main
 *
 * Description: sends out message to all cards
 *
 * Created by:	eric choi
 *
 * Date Created: 4 Mar 2002
 *
 *-----------------------------------------------------------------
 */
void 
sysdvt_stats_display(dvt_test_stats * d_stats)
{
	int slot; 
	int cpu; 

	slot = d_stats->slot;
	cpu = d_stats->cpu; 
	printf("\n=================== System DVT Statistics Slot %d CPU %d ====================\n",slot, cpu);
	printf("                       DATA                 MGMT               QLOGIC\n");
	printf("FATAL ERR:             N/A                  N/A           %016d\n",
			d_stats->fatal_errors);
	printf("PKT SEND ERR:     %016d    %016d    %016d\n",
			d_stats->packet_send_errors[0],
			d_stats->packet_send_errors[2],
			d_stats->packet_send_errors[3]);
	printf("SEQ ERR:          %016d    %016d    %016d\n",
			d_stats->seq_errors[0],
			d_stats->seq_errors[2],
			d_stats->seq_errors[3]);
	printf("CRC ERR:          %016d    %016d    %016d\n",
			d_stats->crc_errors[0],
			d_stats->crc_errors[2],
			d_stats->crc_errors[3]);
	printf("PKT CONTENT ERR:  %016d    %016d    %016d\n",
			d_stats->pktComp_errors[0],
			d_stats->pktComp_errors[2],
			d_stats->pktComp_errors[3]);
	printf("PKT SENT:         %016d    %016d    %016d\n",
			d_stats->packets_sent[0],
			d_stats->packets_sent[2],
			d_stats->packets_sent[3]);
	printf("BYTES SENT:       %016lld    %016lld    %016lld\n",
			d_stats->bytes_sent[0],
			d_stats->bytes_sent[2],
			d_stats->bytes_sent[3]);
	printf("PKT RCVD:         %016d    %016d    %016d\n",
			d_stats->packets_received[0],
			d_stats->packets_received[2],
			d_stats->packets_received[3]);
	printf("BYTES RCVD:       %016lld    %016lld    %016lld\n",
			d_stats->bytes_received[0],
			d_stats->bytes_received[2],
			d_stats->bytes_received[3]);
	printf("============================================================================\n");
}


/*-----------------------------------------------------------------
 * Name: main
 *
 * Description: sends out message to all cards
 *
 * Created by:	eric choi
 *
 * Date Created: 4 Mar 2002
 *
 *-----------------------------------------------------------------
 */
#if 0
void
sysdvtMsg_Broadcast(int mode, sys_dvt_control *msg)
{
	int slot, cpu;

	for (slot = 0 ; slot < 10 ; slot++) {
		for (cpu = 0; cpu < 7; cpu++) {
			if ((slot == NFX_SLOT_SSC) && (cpu == NFX_SSC_MGMT))
				continue;

			else if (slot == NFX_CARDTYPE_NFP)
				if (mode == EEE_MGMT) {
					if ((cpu == NFX_NFP_TXRX2) || (cpu == NFX_NFP_FP1))
						continue;
				}
			else if (mode == EEE_USER_DATA) {
					if ((cpu == NFX_NFP_TXRX1) || (cpu == NFX_NFP_TXRX2))
						continue;
			}
			destport = PACK_UNICAST_PORT_ADDRESS(slot, cpu, 
				eee_allocateApplicationId("dvt"), EEE_MGMT,NFX_COS_HIGH, 
				PORT_BEST_FIT);
			srcport = PACK_UNICAST_PORT_ADDRESS(NFX_SLOT_SSC, NFX_SSC_MGMT, 
				eee_allocateApplicationId("dvt"), EEE_MGMT,NFX_COS_HIGH, 
				PORT_BEST_FIT);

			eee_sendMessage(&msg, sizeof(sys_dvt_control), destport, srcport); 
		}
	}
}
#endif

/*-----------------------------------------------------------------
 * Name: main
 *
 * Description: It all starts here
 *
 * Created by:	eric choi
 *
 * Date Created: 19 Feb 2002
 *
 *-----------------------------------------------------------------
 */
int
main (int argc, char *argv[])
{
	sys_dvt_control	dvtmsg;

	int rc;
	int test_type, size;
	int slot, cpu;
#ifndef DVT_REV2		
	int port=0;
	int numPkts=0;
#endif	
	char *test;
	int send_only = 0;
	/* int ssc_mgmt_later = 0; */
/*
	First things first.
	Let's make sure we have rightous arguments.
 */
	myPid = getpid();
#ifdef DVT_REV2		
	dvtmsg.spin = 0;
	/* If spin is not specified, set it to 1 by default */
	if((argc < 2) || (argc > 5)) {
		fprintf (stderr, "Usage %s [halt|stats|mgmt|data|ctrl|bp|fp|all|clear] <spin> \n", argv[0]);
		return 1;
	}	
	if(argc == 2)
		dvtmsg.spin = 1;
	else if(argc == 3)
		dvtmsg.spin = atoi(argv[2]);
	else if(argc == 5)
	{		
		switch (*argv[3]) {
				case 'n':	
					size = atoi(argv[4]);
					break;
		}
	}	
#else		
	if((argc < 2) || (argc > 5)) {
		fprintf (stderr, "Usage %s [halt|stats|mgmt|data|ctrl|all|clear] <p:port, n:number of pkts, d:send direct to slot cpu> <port/pkts/slot cpu>\n", argv[0]);
		return 1;
	}	
	else if(argc > 2) {
			switch (*argv[2]) {
				case 'd':	
					slot = atoi(argv[3]);
					cpu = atoi(argv[4]);
					send_only = 1;
					break;
				case 'n':
					numPkts = atoi(argv[3]);
					break;
				case 'p':	
					port = atoi(argv[3]);
					break;
			}
	}	
#endif	
	/*command preset to START msg*/
#ifdef DVT_REV2		
	dvtmsg.cmd 	= EEE_MGMT_TEST_CMD_START;
#else		
	dvtmsg.command 	= SYS_DVT_START;
#endif	
	test_type = EEE_MGMT;
	
	if (!strcmp(argv[1],"mgmt"))
		test_type = EEE_MGMT;
	else if (!strcmp(argv[1],"data"))
		test_type = EEE_USER_DATA;
	else if (!strcmp(argv[1],"ctrl"))
		test_type = EEE_CONTROL;
	else if (!strcmp(argv[1],"fp"))
		test_type = SYS_DVT_FRONT;
	else if (!strcmp(argv[1],"halt")) {
#ifdef DVT_REV2		
		dvtmsg.cmd 	= EEE_MGMT_TEST_CMD_STOP;
#else		
		dvtmsg.command = SYS_DVT_STOP;
#endif	
	}	
	else if (!strcmp(argv[1],"stats")) {
#ifdef DVT_REV2		
		dvtmsg.cmd = EEE_MGMT_STATS_REQUEST;
#else		
		dvtmsg.command = SYS_DVT_STATS;
#endif	
	}	
	else if (!strcmp(argv[1],"all")) {
#ifdef DVT_REV2		
		dvtmsg.cmd 	= EEE_MGMT_TEST_CMD_START;
#else		
		dvtmsg.command = SYS_DVT_START;
#endif	
	}	
	else if (!strcmp(argv[1],"clear")) {
#ifdef DVT_REV2		
		dvtmsg.cmd = EEE_MGMT_STATS_CLEAR;
#else		
		dvtmsg.command = SYS_DVT_CLEAR_STATS;
#endif	
	}	
#ifdef DVT_REV2		
	else if (!strcmp(argv[1],"bp")) {
		dvtmsg.cmd 	= EEE_MGMT_TEST_CMD_START;
	}		
#endif	
	else {
		printf("Unrecognized argument: %s\n", argv[1]);
		return(1);
	}

	test = argv[1];
#ifdef DVT_REV2		
	dvtmsg.plane = test_type;
	dvtmsg.size = size;
#else		
	dvtmsg.rsvd = 0xaabb; //marker for our msg
	dvtmsg.test_type = test_type;
	dvtmsg.port = port;
	dvtmsg.numPkts = numPkts;
	dvtmsg.num_bytes = glob_num_bytes;
	dvtmsg.buf_size = glob_buf_size;
#endif	

	if(send_only) {
		if((slot == 0) && (cpu == 0))
			return 1;
		rc = send_dvt_msg(0, 1, test, dvtmsg);
		rc = send_dvt_msg(slot, cpu, test, dvtmsg);
	}
	else {
#ifdef DVT_REV2		
		if((dvtmsg.cmd == EEE_MGMT_STATS_REQUEST) || (dvtmsg.cmd == EEE_MGMT_STATS_CLEAR) ||
						(dvtmsg.cmd == EEE_MGMT_TEST_CMD_STOP)) {
#endif				
			for (slot = 0; slot < SYS_DVT_SLOT_MAX; slot++) {
				for (cpu = 0; cpu < SYS_DVT_CPU_MAX; cpu++) {
					if ((!slot) && (!cpu))
						continue;
					if ((slot == SSC_SLOT) && (cpu == NFX_SSC_UNIX))
						continue;
					rc = send_dvt_msg(slot, cpu, test, dvtmsg);
			}/*for cpu*/
			}/* for slot*/
		}
		else if(dvtmsg.cmd == EEE_MGMT_TEST_CMD_START) {
			for (slot = SYS_DVT_SLOT_MAX-1; slot >= 0 ; slot--) {
			for (cpu = 0; cpu < SYS_DVT_CPU_MAX; cpu++) {
				if ((!slot) && (!cpu))
					continue;
				if ((slot == SSC_SLOT) && (cpu == NFX_SSC_UNIX))
					continue;
				rc = send_dvt_msg(slot, cpu, test, dvtmsg);
			}/*for cpu*/
			}/* for slot*/
		}
#ifndef DVT_REV2		
		else {
					/* Send START & STOP only to txrx and fp */
					rc = send_dvt_msg(1, 0, test, dvtmsg); /* txrx */
					rc = send_dvt_msg(1, 3, test, dvtmsg); /* fp */
		}		
#endif		
				
	}

	if( rc == 1)
		return 1;
	else if (rc < 0) {
		perror ("RX recv error");
		return 1;
	}
	else {
		return 0;
	}

}

int
send_dvt_msg(int slot, int cpu, char *test,sys_dvt_control dvtmsg)
{
	struct sockaddr_eee ourAddr;
	char msgBuff[8192];	// Max possible MTU
	sigset_t sigs;
	int rc;
	fd_set selset;
	struct timeval selTO = {1, 0};	// 1 seconds should be enough
	int i;
	dvt_test_stats *dvtstats = (dvt_test_stats *)msgBuff;
	int noOfTests;

	if(dvtmsg.cmd == EEE_MGMT_STATS_REQUEST) {
		if((slot == NFX_SLOTTYPE_NFP) && ((cpu == NFX_NFP_TXRX2) || (cpu == NFX_NFP_FP2)))
		return 1;
	}

	dvtmsg.orig_slot = slot;
	dvtmsg.orig_cpu = cpu;
#ifndef SEND_SYS_DVT_UDP
#ifdef DVT_REV2		
		Sock = regAgileApp("eee_test_complex");
#else	
		Sock = regAgileApp("dvt");
#endif		

#if 0
		kill(getppid(), SIGUSR1);
#endif

#else
/*
	Set up our peer's address
 */
	memset (&ourPeer, 0, sizeof(ourPeer));
	ourPeer.seee_family = AF_EEE;
	if ((slot == 0) && (cpu == 1))
		ourPeer.seee_bplane = 1;
	else
		ourPeer.seee_bplane = EEE_CONTROLBPLANE;
#ifdef DVT_REV2		
	ourPeer.seee_app = eee_getApplicationId("eee_test_complex");
#else	
	ourPeer.seee_app = eee_getApplicationId("dvt");
#endif	
	if (ourPeer.seee_app == EEE_ERR_INVALID_APP_NAME) {
		fprintf (stderr, "Unknown application id dvt\n");
		exit (1);
	}
	ourPeer.seee_slot = slot;
	ourPeer.seee_cpu = cpu;
/*
	Let's see if we can establish communications with our peer.
	To do that, we'll create two persistent sockets of the EEE kind.
 */
	memset (&ourAddr, 0, sizeof (ourAddr));
	ourAddr.seee_family = AF_EEE;
	ourAddr.seee_slot = SSC_SLOT;
	ourAddr.seee_cpu = NFX_SSC_UNIX;
	if ((slot == 0) && (cpu == 1))
		ourAddr.seee_bplane = 1;
	else 
		ourAddr.seee_bplane = EEE_CONTROLBPLANE;
#ifdef DVT_REV2		
	ourPeer.seee_app = eee_getApplicationId("eee_test_complex");
#else	
	ourAddr.seee_app = eee_getApplicationId("dvt");
#endif	

	if ((Sock = socket(PF_EEE, SOCK_DGRAM, 0)) >= 0) {
		if (bind(Sock, (struct sockaddr *)&ourAddr, sizeof(ourAddr)) < 0) {
			perror ("Error binding Rx socket");
			return 1;
		}
	}
	else {
		perror ("Error creating Rx socket");
		return 1;
	}
#endif

	if ((strcmp(test,"all") == 0) ) 
		noOfTests = 4; /* DATA(0), CTRL(1), MGMT(2), FRONT (4) */
	else if ((strcmp(test,"bp") == 0) ) 
		noOfTests = 2;

	if ((noOfTests ) && (slot)) {
		for (i = EEE_USER_DATA; i <= noOfTests; i++) {
			if(i == EEE_CONTROL)
				continue;	
#ifdef DVT_REV2		
			dvtmsg.plane = i;
#else		
			dvtmsg.test_type = i;
#endif	
#ifndef SEND_SYS_DVT_UDP
#ifdef DVT_REV2		
			sendAgileMsg(Sock, &dvtmsg, sizeof(sys_dvt_control),"eee_test_complex", 
					cpu, slot,MGMTPLANE, NULL, NULL, 0);
#else	
			sendAgileMsg(Sock, &dvtmsg, sizeof(sys_dvt_control),"dvt", 
					cpu, slot,MGMTPLANE, NULL, NULL, 0);
#endif			
#else
			/*
			{
					uchar8 * tmp;
					uint32 j;
					tmp = (uchar8 *)&dvtmsg;
					for(j=0;j<sizeof(sys_dvt_control);j++)
					{		
					printf("%02x ",*tmp);
					tmp++;
					}
			}*/
			if (sendto (Sock, &dvtmsg  , sizeof(sys_dvt_control) , 0, 
				(struct sockaddr *)&ourPeer, sizeof(struct sockaddr_eee)) < 0) {
					perror("sendto msg failed for system dvt.");
					return 1;
			}
#endif
		}
#if 0
		/* Also send to front plane */
		dvtmsg.test_type = SYS_DVT_FRONT;
		if (sendto (Sock, &dvtmsg  , sizeof(sys_dvt_control) , 0, 
		(struct sockaddr *)&ourPeer, sizeof(struct sockaddr_eee)) < 0) {
			perror("sendto msg failed for system dvt.");
			return 1;
		}
#endif
	}
	else {  /* else just send once */
#ifndef SEND_SYS_DVT_UDP
#ifdef DVT_REV2		
			rc = sendAgileMsg(Sock, &dvtmsg, sizeof(sys_dvt_control),"eee_test_complex", 
					cpu, slot,MGMTPLANE, NULL, NULL, 0);
#else			
			rc = sendAgileMsg(Sock, &dvtmsg, sizeof(sys_dvt_control),"dvt", 
					cpu,slot,MGMTPLANE, NULL, NULL, 0);
#endif			
			if(rc < 0)
					printf("sendAgileMsg error.\n");
#else
	/*
	{
			uchar8 * tmp;
			uint32 j;
			tmp = (uchar8 *)&dvtmsg;
			for(j=0;j<sizeof(sys_dvt_control);j++)
			{		
			printf("%02x ",*tmp);
			tmp++;
			}
	}*/

			
	if (sendto (Sock, &dvtmsg  , sizeof(sys_dvt_control) , 0, (struct sockaddr *)&ourPeer,
		sizeof(struct sockaddr_eee)) < 0) {
			perror("sendto msg failed for system dvt.");
			return 1;
		}
#endif
	}
		printf("Sent to slot %d cpu %d\n",slot,cpu);

#ifdef SEND_SYS_DVT_UDP
	sigfillset (&sigs);
	sigdelset (&sigs, SIGHUP);
	sigdelset (&sigs, SIGINT);
	sigdelset (&sigs, SIGBUS);
	sigdelset (&sigs, SIGSEGV);
	sigprocmask (SIG_SETMASK, &sigs, NULL);

		/*
		if	(dvtmsg.cmd == EEE_MGMT_TEST_CMD_START) {
			selTO.tv_sec = 0;
			selTO.tv_usec = 10000;
		}*/
#ifdef DVT_REV2		
		if	(dvtmsg.cmd == EEE_MGMT_STATS_REQUEST)
#else		
		if	(dvtmsg.command == SYS_DVT_STATS)
#endif	
			selTO.tv_sec = 3;

		FD_ZERO (&selset);
		FD_SET (Sock, &selset);

		rc = select (Sock + 1, &selset, NULL, NULL, &selTO);

		if (FD_ISSET (Sock, &selset)) {
			rc = recv (Sock, msgBuff, sizeof (msgBuff), 0);
#ifdef DVT_REV2		
			if	(dvtstats->command == EEE_MGMT_STATS_REPORT) {
				sysdvt_stats_display((dvt_test_stats *)msgBuff);
			}	
#else		
			if	(dvtstats->command == SYS_DVT_STATS_REPORT) 
				sysdvt_stats_display((dvt_test_stats *)msgBuff);
#endif					
		}

	close(Sock);
#endif
	
	/*
		if((dvtmsg.command == SYS_DVT_START) && (dvtmsg.test_type != SYS_DVT_FRONT))
		{
				if((slot == 0) && (cpu == 1))
					sleep(5);
		} */		
	return(rc);
}
