/* vi:ts=4
 *-----------------------------------------------------------------
 *
 * Name: timekeep-api.h
 *
 * RCS:  $Id: timekeep-api.h,v 1.4 2002/06/06 17:59:38 gregs Exp $
 *
 * Copyright (C) 2002, Agile Storage, Inc.
 *
 * Description: How one is to configure NTP
 *
 * Created by:	Greg Seibert
 *
 * Date Created: 5 Apr 2002
 *
 *-----------------------------------------------------------------
 */

#if !defined (__TIMEKEEP_API__)
#define __TIMEKEEP_API__

#define NTP_MODE "ntp.mode"
#define NTPCONF "/etc/ntp.conf"
#define NTPTICKERS "/etc/ntp/step-tickers"
#define NTPCONFTEMPLATE "/usr/local/agile/etc/ntp.conf.template"
#define NTPDPROG "/usr/local/sbin/ntpd"
#define NTPDPID "/var/run/ntpd.pid"
#define SERVERSTR "servers:"

// How to manipulate NTP configuration

// Use NTP broadcast messages

extern int tk_set_ntp_broadcast (void);

// You can add or delete servers with this based upon the value of 'del'
// Will return 0 if OK or server already configured.
// !0 if error with errno set:
//   ENOENT - when deleting a server that's not on the list
//   EIO - Failed to update configuration database

extern int tk_mod_ntp_servers (char *serv_addr, int del);

// This returns the ntp configuration string in the form of:
//  "broadcast" or
//  "servers:addr1[|addr2...]"
// Returns NULL if ntp not configured.
// Address is that of heap storage and should be free()ed when done with it.

extern char *tk_get_ntp_cfg (void);

// Completely disable NTP

extern void tk_disable_ntp (void);

// Do this after commiting all configuration changes
// You must be UID 0 (root) for this to work.

extern void tk_update_ntp_conf (void);

#endif
