/*
 *=============================================================================
 * File Name   :    bcm5600-socregs.h
 *
 *  RCS: $Id: bcm5600-socregs.h,v 1.1.1.1 2001/12/20 04:53:01 tonyw Exp $
 *
 *  Description:    Copied and modified from Broadcom
 *
 *  Created    :    09-04-2001
 *
 *  Author     :    Tony Wong
 *
 *  Copyright(c)  Agile Storage, Inc. 2001
 *=============================================================================
 */

#ifndef BCM5600_SOCREGS_H
#define BCM5600_SOCREGS_H

/* This remaps the names of FE macros according to whether
 * we're in GSL mode or not.  In that mode, the macros have
 * _GTH_ inserted in their names.
 */
#define       FE_SET(_p, _t, _v, _val)        SET_ ## _t(_v, _val)
#define       FE_GET(_p, _t, _v)              GET_ ## _t(_v)
#define       FE_READ(_t, _u, _p, _v)         (READ_ ## _t(_u, _p, _v))
#define       FE_WRITE(_t, _u, _p, _v)        (WRITE_ ## _t(_u, _p, _v))

#define _u32 unsigned int
#define BCM5600
#define BCM5600_LE_HOST
#define NO_FLD_NAMES
#define NO_DESCRIPTIONS
#define NO_REG_NAMES
#define NO_MEM_NAMES
#define NO_RESET_INFO
#define NDEBUG

#define NUM_EPIC_5600	3
#define NUM_GPIC_5600	2
#define NUM_PIC_5600	(NUM_EPIC_5600+NUM_GPIC_5600)

#define NUM_COS_5600	4

#define MIN_FE_PORT_5600	0
#define MAX_FE_PORT_5600	23
#define NUM_FE_PORT_5600	24
#define MIN_GE_PORT_5600	24
#define MAX_GE_PORT_5600	25
#define NUM_GE_PORT_5600	2
#define MIN_E_PORT_5600		MIN_FE_PORT_5600
#define MAX_E_PORT_5600		MAX_GE_PORT_5600
#define NUM_E_PORT_5600		(NUM_FE_PORT_5600+NUM_GE_PORT_5600)

#define CMIC_PORT_5600	0x1b

#define BYTES2WORDS(x)	(((x)+3)>>2)

#define MAX_MEM_BYTES	72
#define MAX_MEM_WORDS	BYTES2WORDS(MAX_MEM_BYTES)

#define EPIC0_SCH_BLK_NUM       0
#define EPIC1_SCH_BLK_NUM       1
#define EPIC2_SCH_BLK_NUM       2
#define GPIC0_SCH_BLK_NUM       3
#define GPIC1_SCH_BLK_NUM       4
#define CMIC_SCH_BLK_NUM        5
#define MMU_SCH_BLK_NUM         6


/*****************************************************************************
 **	ACCESS MACROS
 **
 ** EXAMPLES:
 **
 **	// Global (generic non-PIC) Register
 **	uint32 fooval=0;		// Zero all register fields
 **	SET_FOOr_F1(fooval,42);		// Set field F1 of FOO register to 42
 **	SET_FOOr_F2(fooval,1);		// Set field F1 of FOO register to 1
 **	WRITE_FOOr(unit, fooval);	// Update SOC
 **
 **	READ_FOOr(unit, &fooval);		// Read FOO register from SOC
 **	SET_FOOr_F2(fooval,0);		// Set field F1 of FOO register to 0
 **	WRITE_FOOr(unit, fooval);	// Update SOC
 **
 **	// Monolithic Memory
 **	uint32 barval[BARm_WORDS];	// Buffer for BAR memory datum
 **	uint32 bar_f1val=0x20;		// Mem field F1 (less than 33 bits)
 **	uint32 bar_f2val[BARm_F4_WORDS]={0x00000001, 0x0b}; // 1 + 11*2^32
 **	assert(BAR_F1_WORDS==1);
 **	ZERO_BARm(barval);		// Zero all fields
 **	SET_BARm_F1(barval,&bar_f1val);	// Set field F1
 **	SET_BARm_F2(barval,&bar_f2val);	// Set field F2
 **	WRITE_BARm(unit,7,barval);	// Write to BAR memory location 7
 **
 *****************************************************************************/


#ifdef BCM5600

#define AGE_TIMERr_ADDR(pic)            reg_addr(AGE_TIMERr,pic)
#define READ_AGE_TIMERr(unit,pic,rvp) \
	cmic_read_reg(unit,AGE_TIMERr_ADDR(pic),rvp)
#define WRITE_AGE_TIMERr(unit,pic,rv) \
	cmic_write_reg(unit,AGE_TIMERr_ADDR(pic),rv)
#define GET_AGE_TIMERr_AGE_ENA(rv)      ((rv)>>20&0x1)
#define SET_AGE_TIMERr_AGE_ENA(rv,fv)   (rv=(rv&~(0x1<<20))|((fv)&0x1)<<20)
#define GET_AGE_TIMERr_AGE_VAL(rv)      ((rv)>>0&0xfffff)
#define SET_AGE_TIMERr_AGE_VAL(rv,fv)   (rv=(rv&~(0xfffff<<0))|((fv)&0xfffff)<<0)

#define ANCTLr_ADDR(port)               reg_addr(ANCTLr,port)
#define READ_ANCTLr(unit,port,rvp) \
	cmic_read_reg(unit,ANCTLr_ADDR(port),rvp)
#define WRITE_ANCTLr(unit,port,rv) \
	cmic_write_reg(unit,ANCTLr_ADDR(port),rv)
#define GET_ANCTLr_ANEN(rv)             ((rv)>>1&0x1)
#define SET_ANCTLr_ANEN(rv,fv)          (rv=(rv&~(0x1<<1))|((fv)&0x1)<<1)
#define GET_ANCTLr_RSTAN(rv)            ((rv)>>0&0x1)
#define SET_ANCTLr_RSTAN(rv,fv)         (rv=(rv&~(0x1<<0))|((fv)&0x1)<<0)

#define ANLPAr_ADDR(port)               reg_addr(ANLPAr,port)
#define READ_ANLPAr(unit,port,rvp) \
	cmic_read_reg(unit,ANLPAr_ADDR(port),rvp)
#define WRITE_ANLPAr(unit,port,rv) \
	cmic_write_reg(unit,ANLPAr_ADDR(port),rv)
#define GET_ANLPAr_LPACK2(rv)           ((rv)>>28&0x1)
#define SET_ANLPAr_LPACK2(rv,fv)        (rv=(rv&~(0x1<<28))|((fv)&0x1)<<28)
#define GET_ANLPAr_LPANERR(rv)          ((rv)>>12&0x3)
#define SET_ANLPAr_LPANERR(rv,fv)       (rv=(rv&~(0x3<<12))|((fv)&0x3)<<12)
#define GET_ANLPAr_LPASMDR(rv)          ((rv)>>8&0x1)
#define SET_ANLPAr_LPASMDR(rv,fv)       (rv=(rv&~(0x1<<8))|((fv)&0x1)<<8)
#define GET_ANLPAr_LPCF(rv)             ((rv)>>16&0x7ff)
#define SET_ANLPAr_LPCF(rv,fv)          (rv=(rv&~(0x7ff<<16))|((fv)&0x7ff)<<16)
#define GET_ANLPAr_LPFD(rv)             ((rv)>>5&0x1)
#define SET_ANLPAr_LPFD(rv,fv)          (rv=(rv&~(0x1<<5))|((fv)&0x1)<<5)
#define GET_ANLPAr_LPMSG(rv)            ((rv)>>29&0x1)
#define SET_ANLPAr_LPMSG(rv,fv)         (rv=(rv&~(0x1<<29))|((fv)&0x1)<<29)
#define GET_ANLPAr_LPNEXTP(rv)          ((rv)>>15&0x1)
#define SET_ANLPAr_LPNEXTP(rv,fv)       (rv=(rv&~(0x1<<15))|((fv)&0x1)<<15)
#define GET_ANLPAr_LPNP(rv)             ((rv)>>31&0x1)
#define SET_ANLPAr_LPNP(rv,fv)          (rv=(rv&~(0x1<<31))|((fv)&0x1)<<31)
#define GET_ANLPAr_LPPAUSE(rv)          ((rv)>>7&0x1)
#define SET_ANLPAr_LPPAUSE(rv,fv)       (rv=(rv&~(0x1<<7))|((fv)&0x1)<<7)
#define GET_ANLPAr_LPTOG(rv)            ((rv)>>27&0x1)
#define SET_ANLPAr_LPTOG(rv,fv)         (rv=(rv&~(0x1<<27))|((fv)&0x1)<<27)

#define ANNPGr_ADDR(port)               reg_addr(ANNPGr,port)
#define READ_ANNPGr(unit,port,rvp) \
	cmic_read_reg(unit,ANNPGr_ADDR(port),rvp)
#define WRITE_ANNPGr(unit,port,rv) \
	cmic_write_reg(unit,ANNPGr_ADDR(port),rv)
#define GET_ANNPGr_ANACK2(rv)           ((rv)>>28&0x1)
#define SET_ANNPGr_ANACK2(rv,fv)        (rv=(rv&~(0x1<<28))|((fv)&0x1)<<28)
#define GET_ANNPGr_ANERR(rv)            ((rv)>>12&0x3)
#define SET_ANNPGr_ANERR(rv,fv)         (rv=(rv&~(0x3<<12))|((fv)&0x3)<<12)
#define GET_ANNPGr_ANMSG(rv)            ((rv)>>29&0x1)
#define SET_ANNPGr_ANMSG(rv,fv)         (rv=(rv&~(0x1<<29))|((fv)&0x1)<<29)
#define GET_ANNPGr_ANNP(rv)             ((rv)>>31&0x1)
#define SET_ANNPGr_ANNP(rv,fv)          (rv=(rv&~(0x1<<31))|((fv)&0x1)<<31)
#define GET_ANNPGr_ANTOG(rv)            ((rv)>>27&0x1)
#define SET_ANNPGr_ANTOG(rv,fv)         (rv=(rv&~(0x1<<27))|((fv)&0x1)<<27)
#define GET_ANNPGr_ASMDR(rv)            ((rv)>>8&0x1)
#define SET_ANNPGr_ASMDR(rv,fv)         (rv=(rv&~(0x1<<8))|((fv)&0x1)<<8)
#define GET_ANNPGr_FD(rv)               ((rv)>>5&0x1)
#define SET_ANNPGr_FD(rv,fv)            (rv=(rv&~(0x1<<5))|((fv)&0x1)<<5)
#define GET_ANNPGr_NEXTP(rv)            ((rv)>>15&0x1)
#define SET_ANNPGr_NEXTP(rv,fv)         (rv=(rv&~(0x1<<15))|((fv)&0x1)<<15)
#define GET_ANNPGr_NPLPCF(rv)           ((rv)>>16&0x7ff)
#define SET_ANNPGr_NPLPCF(rv,fv)        (rv=(rv&~(0x7ff<<16))|((fv)&0x7ff)<<16)
#define GET_ANNPGr_PAUSE(rv)            ((rv)>>7&0x1)
#define SET_ANNPGr_PAUSE(rv,fv)         (rv=(rv&~(0x1<<7))|((fv)&0x1)<<7)

#define ANSTTr_ADDR(port)               reg_addr(ANSTTr,port)
#define READ_ANSTTr(unit,port,rvp) \
	cmic_read_reg(unit,ANSTTr_ADDR(port),rvp)
#define WRITE_ANSTTr(unit,port,rv) \
	cmic_write_reg(unit,ANSTTr_ADDR(port),rv)
#define GET_ANSTTr_ANCPLT(rv)           ((rv)>>3&0x1)
#define SET_ANSTTr_ANCPLT(rv,fv)        (rv=(rv&~(0x1<<3))|((fv)&0x1)<<3)
#define GET_ANSTTr_LINKOK(rv)           ((rv)>>1&0x1)
#define SET_ANSTTr_LINKOK(rv,fv)        (rv=(rv&~(0x1<<1))|((fv)&0x1)<<1)
#define GET_ANSTTr_PGRX(rv)             ((rv)>>2&0x1)
#define SET_ANSTTr_PGRX(rv,fv)          (rv=(rv&~(0x1<<2))|((fv)&0x1)<<2)
#define GET_ANSTTr_RFIND(rv)            ((rv)>>4&0x1)
#define SET_ANSTTr_RFIND(rv,fv)         (rv=(rv&~(0x1<<4))|((fv)&0x1)<<4)
#define GET_ANSTTr_SYNCOK(rv)           ((rv)>>0&0x1)
#define SET_ANSTTr_SYNCOK(rv,fv)        (rv=(rv&~(0x1<<0))|((fv)&0x1)<<0)

#define BISTMEMFAILUREr_ADDR()          reg_addr(BISTMEMFAILUREr)
#define READ_BISTMEMFAILUREr(unit,rvp) \
	cmic_read_reg(unit,BISTMEMFAILUREr_ADDR(),rvp)
#define WRITE_BISTMEMFAILUREr(unit,rv) \
	cmic_write_reg(unit,BISTMEMFAILUREr_ADDR(),rv)
#define GET_BISTMEMFAILUREr_DEADADDRESS1(rv) ((rv)>>0&0x3fff)
#define SET_BISTMEMFAILUREr_DEADADDRESS1(rv,fv) (rv=(rv&~(0x3fff<<0))|((fv)&0x3fff)<<0)
#define GET_BISTMEMFAILUREr_DEADADDRESS2(rv) ((rv)>>14&0x3fff)
#define SET_BISTMEMFAILUREr_DEADADDRESS2(rv,fv) (rv=(rv&~(0x3fff<<14))|((fv)&0x3fff)<<14)
#define GET_BISTMEMFAILUREr_DEADADDRESSCOUNT(rv) ((rv)>>28&0xf)
#define SET_BISTMEMFAILUREr_DEADADDRESSCOUNT(rv,fv) (rv=(rv&~(0xf<<28))|((fv)&0xf)<<28)
#define GET_BISTMEMFAILUREr_GETDEADADDRESS(rv) ((rv)>>0&0xffffffff)
#define SET_BISTMEMFAILUREr_GETDEADADDRESS(rv,fv) (rv=(rv&~(0xffffffff<<0))|((fv)&0xffffffff)<<0)

#define CABCONTROLr_ADDR()              reg_addr(CABCONTROLr)
#define READ_CABCONTROLr(unit,rvp) \
	cmic_read_reg(unit,CABCONTROLr_ADDR(),rvp)
#define WRITE_CABCONTROLr(unit,rv) \
	cmic_write_reg(unit,CABCONTROLr_ADDR(),rv)
#define GET_CABCONTROLr_CABBOOTOUTTIMER(rv) ((rv)>>0&0xff)
#define SET_CABCONTROLr_CABBOOTOUTTIMER(rv,fv) (rv=(rv&~(0xff<<0))|((fv)&0xff)<<0)

#define CAR1r_ADDR(port)                reg_addr(CAR1r,port)
#define READ_CAR1r(unit,port,rvp) \
	cmic_read_reg(unit,CAR1r_ADDR(port),rvp)
#define WRITE_CAR1r(unit,port,rv) \
	cmic_write_reg(unit,CAR1r_ADDR(port),rv)
#define GET_CAR1r_RALN(rv)              ((rv)>>8&0x1)
#define SET_CAR1r_RALN(rv,fv)           (rv=(rv&~(0x1<<8))|((fv)&0x1)<<8)
#define GET_CAR1r_RBCA(rv)              ((rv)>>12&0x1)
#define SET_CAR1r_RBCA(rv,fv)           (rv=(rv&~(0x1<<12))|((fv)&0x1)<<12)
#define GET_CAR1r_RBYT(rv)              ((rv)>>16&0x1)
#define SET_CAR1r_RBYT(rv,fv)           (rv=(rv&~(0x1<<16))|((fv)&0x1)<<16)
#define GET_CAR1r_RCDE(rv)              ((rv)>>6&0x1)
#define SET_CAR1r_RCDE(rv,fv)           (rv=(rv&~(0x1<<6))|((fv)&0x1)<<6)
#define GET_CAR1r_RCSE(rv)              ((rv)>>5&0x1)
#define SET_CAR1r_RCSE(rv,fv)           (rv=(rv&~(0x1<<5))|((fv)&0x1)<<5)
#define GET_CAR1r_RFCS(rv)              ((rv)>>14&0x1)
#define SET_CAR1r_RFCS(rv,fv)           (rv=(rv&~(0x1<<14))|((fv)&0x1)<<14)
#define GET_CAR1r_RFLR(rv)              ((rv)>>7&0x1)
#define SET_CAR1r_RFLR(rv,fv)           (rv=(rv&~(0x1<<7))|((fv)&0x1)<<7)
#define GET_CAR1r_RFRG(rv)              ((rv)>>2&0x1)
#define SET_CAR1r_RFRG(rv,fv)           (rv=(rv&~(0x1<<2))|((fv)&0x1)<<2)
#define GET_CAR1r_RJBR(rv)              ((rv)>>1&0x1)
#define SET_CAR1r_RJBR(rv,fv)           (rv=(rv&~(0x1<<1))|((fv)&0x1)<<1)
#define GET_CAR1r_RMCA(rv)              ((rv)>>13&0x1)
#define SET_CAR1r_RMCA(rv,fv)           (rv=(rv&~(0x1<<13))|((fv)&0x1)<<13)
#define GET_CAR1r_ROVR(rv)              ((rv)>>3&0x1)
#define SET_CAR1r_ROVR(rv,fv)           (rv=(rv&~(0x1<<3))|((fv)&0x1)<<3)
#define GET_CAR1r_RPKT(rv)              ((rv)>>15&0x1)
#define SET_CAR1r_RPKT(rv,fv)           (rv=(rv&~(0x1<<15))|((fv)&0x1)<<15)
#define GET_CAR1r_RUND(rv)              ((rv)>>4&0x1)
#define SET_CAR1r_RUND(rv,fv)           (rv=(rv&~(0x1<<4))|((fv)&0x1)<<4)
#define GET_CAR1r_RUXO(rv)              ((rv)>>9&0x1)
#define SET_CAR1r_RUXO(rv,fv)           (rv=(rv&~(0x1<<9))|((fv)&0x1)<<9)
#define GET_CAR1r_RXCF(rv)              ((rv)>>11&0x1)
#define SET_CAR1r_RXCF(rv,fv)           (rv=(rv&~(0x1<<11))|((fv)&0x1)<<11)
#define GET_CAR1r_RXPF(rv)              ((rv)>>10&0x1)
#define SET_CAR1r_RXPF(rv,fv)           (rv=(rv&~(0x1<<10))|((fv)&0x1)<<10)
#define GET_CAR1r_TR1023(rv)            ((rv)>>27&0x1)
#define SET_CAR1r_TR1023(rv,fv)         (rv=(rv&~(0x1<<27))|((fv)&0x1)<<27)
#define GET_CAR1r_TR127(rv)             ((rv)>>30&0x1)
#define SET_CAR1r_TR127(rv,fv)          (rv=(rv&~(0x1<<30))|((fv)&0x1)<<30)
#define GET_CAR1r_TR255(rv)             ((rv)>>29&0x1)
#define SET_CAR1r_TR255(rv,fv)          (rv=(rv&~(0x1<<29))|((fv)&0x1)<<29)
#define GET_CAR1r_TR511(rv)             ((rv)>>28&0x1)
#define SET_CAR1r_TR511(rv,fv)          (rv=(rv&~(0x1<<28))|((fv)&0x1)<<28)
#define GET_CAR1r_TR64(rv)              ((rv)>>31&0x1)
#define SET_CAR1r_TR64(rv,fv)           (rv=(rv&~(0x1<<31))|((fv)&0x1)<<31)
#define GET_CAR1r_TRMAX(rv)             ((rv)>>26&0x1)
#define SET_CAR1r_TRMAX(rv,fv)          (rv=(rv&~(0x1<<26))|((fv)&0x1)<<26)
#define GET_CAR1r_TRMGV(rv)             ((rv)>>25&0x1)
#define SET_CAR1r_TRMGV(rv,fv)          (rv=(rv&~(0x1<<25))|((fv)&0x1)<<25)

#define CAR2r_ADDR(port)                reg_addr(CAR2r,port)
#define READ_CAR2r(unit,port,rvp) \
	cmic_read_reg(unit,CAR2r_ADDR(port),rvp)
#define WRITE_CAR2r(unit,port,rv) \
	cmic_write_reg(unit,CAR2r_ADDR(port),rv)
#define GET_CAR2r_RDISC(rv)             ((rv)>>25&0x1)
#define SET_CAR2r_RDISC(rv,fv)          (rv=(rv&~(0x1<<25))|((fv)&0x1)<<25)
#define GET_CAR2r_RIPC(rv)              ((rv)>>20&0x1)
#define SET_CAR2r_RIPC(rv,fv)           (rv=(rv&~(0x1<<20))|((fv)&0x1)<<20)
#define GET_CAR2r_RIPCHK(rv)            ((rv)>>22&0x1)
#define SET_CAR2r_RIPCHK(rv,fv)         (rv=(rv&~(0x1<<22))|((fv)&0x1)<<22)
#define GET_CAR2r_RIPD(rv)              ((rv)>>21&0x1)
#define SET_CAR2r_RIPD(rv,fv)           (rv=(rv&~(0x1<<21))|((fv)&0x1)<<21)
#define GET_CAR2r_RIPX(rv)              ((rv)>>23&0x1)
#define SET_CAR2r_RIPX(rv,fv)           (rv=(rv&~(0x1<<23))|((fv)&0x1)<<23)
#define GET_CAR2r_RIPXD(rv)             ((rv)>>24&0x1)
#define SET_CAR2r_RIPXD(rv,fv)          (rv=(rv&~(0x1<<24))|((fv)&0x1)<<24)
#define GET_CAR2r_TABRT(rv)             ((rv)>>31&0x1)
#define SET_CAR2r_TABRT(rv,fv)          (rv=(rv&~(0x1<<31))|((fv)&0x1)<<31)
#define GET_CAR2r_TAGE(rv)              ((rv)>>26&0x1)
#define SET_CAR2r_TAGE(rv,fv)           (rv=(rv&~(0x1<<26))|((fv)&0x1)<<26)
#define GET_CAR2r_TBCA(rv)              ((rv)>>10&0x1)
#define SET_CAR2r_TBCA(rv,fv)           (rv=(rv&~(0x1<<10))|((fv)&0x1)<<10)
#define GET_CAR2r_TBYT(rv)              ((rv)>>13&0x1)
#define SET_CAR2r_TBYT(rv,fv)           (rv=(rv&~(0x1<<13))|((fv)&0x1)<<13)
#define GET_CAR2r_TDFR(rv)              ((rv)>>8&0x1)
#define SET_CAR2r_TDFR(rv,fv)           (rv=(rv&~(0x1<<8))|((fv)&0x1)<<8)
#define GET_CAR2r_TEDF(rv)              ((rv)>>7&0x1)
#define SET_CAR2r_TEDF(rv,fv)           (rv=(rv&~(0x1<<7))|((fv)&0x1)<<7)
#define GET_CAR2r_TFCS(rv)              ((rv)>>18&0x1)
#define SET_CAR2r_TFCS(rv,fv)           (rv=(rv&~(0x1<<18))|((fv)&0x1)<<18)
#define GET_CAR2r_TFRG(rv)              ((rv)>>14&0x1)
#define SET_CAR2r_TFRG(rv,fv)           (rv=(rv&~(0x1<<14))|((fv)&0x1)<<14)
#define GET_CAR2r_TIP(rv)               ((rv)>>27&0x1)
#define SET_CAR2r_TIP(rv,fv)            (rv=(rv&~(0x1<<27))|((fv)&0x1)<<27)
#define GET_CAR2r_TIPD(rv)              ((rv)>>28&0x1)
#define SET_CAR2r_TIPD(rv,fv)           (rv=(rv&~(0x1<<28))|((fv)&0x1)<<28)
#define GET_CAR2r_TIPX(rv)              ((rv)>>30&0x1)
#define SET_CAR2r_TIPX(rv,fv)           (rv=(rv&~(0x1<<30))|((fv)&0x1)<<30)
#define GET_CAR2r_TJBR(rv)              ((rv)>>19&0x1)
#define SET_CAR2r_TJBR(rv,fv)           (rv=(rv&~(0x1<<19))|((fv)&0x1)<<19)
#define GET_CAR2r_TLCL(rv)              ((rv)>>4&0x1)
#define SET_CAR2r_TLCL(rv,fv)           (rv=(rv&~(0x1<<4))|((fv)&0x1)<<4)
#define GET_CAR2r_TMCA(rv)              ((rv)>>11&0x1)
#define SET_CAR2r_TMCA(rv,fv)           (rv=(rv&~(0x1<<11))|((fv)&0x1)<<11)
#define GET_CAR2r_TMCL(rv)              ((rv)>>5&0x1)
#define SET_CAR2r_TMCL(rv,fv)           (rv=(rv&~(0x1<<5))|((fv)&0x1)<<5)
#define GET_CAR2r_TNCL(rv)              ((rv)>>2&0x1)
#define SET_CAR2r_TNCL(rv,fv)           (rv=(rv&~(0x1<<2))|((fv)&0x1)<<2)
#define GET_CAR2r_TOVR(rv)              ((rv)>>16&0x1)
#define SET_CAR2r_TOVR(rv,fv)           (rv=(rv&~(0x1<<16))|((fv)&0x1)<<16)
#define GET_CAR2r_TPFH(rv)              ((rv)>>1&0x1)
#define SET_CAR2r_TPFH(rv,fv)           (rv=(rv&~(0x1<<1))|((fv)&0x1)<<1)
#define GET_CAR2r_TPKT(rv)              ((rv)>>12&0x1)
#define SET_CAR2r_TPKT(rv,fv)           (rv=(rv&~(0x1<<12))|((fv)&0x1)<<12)
#define GET_CAR2r_TSCL(rv)              ((rv)>>6&0x1)
#define SET_CAR2r_TSCL(rv,fv)           (rv=(rv&~(0x1<<6))|((fv)&0x1)<<6)
#define GET_CAR2r_TVLAN(rv)             ((rv)>>29&0x1)
#define SET_CAR2r_TVLAN(rv,fv)          (rv=(rv&~(0x1<<29))|((fv)&0x1)<<29)
#define GET_CAR2r_TXCF(rv)              ((rv)>>17&0x1)
#define SET_CAR2r_TXCF(rv,fv)           (rv=(rv&~(0x1<<17))|((fv)&0x1)<<17)
#define GET_CAR2r_TXCL(rv)              ((rv)>>3&0x1)
#define SET_CAR2r_TXCL(rv,fv)           (rv=(rv&~(0x1<<3))|((fv)&0x1)<<3)
#define GET_CAR2r_TXPF(rv)              ((rv)>>9&0x1)
#define SET_CAR2r_TXPF(rv,fv)           (rv=(rv&~(0x1<<9))|((fv)&0x1)<<9)

#define CBPCONTROLr_ADDR()              reg_addr(CBPCONTROLr)
#define READ_CBPCONTROLr(unit,rvp) \
	cmic_read_reg(unit,CBPCONTROLr_ADDR(),rvp)
#define WRITE_CBPCONTROLr(unit,rv) \
	cmic_write_reg(unit,CBPCONTROLr_ADDR(),rv)
#define GET_CBPCONTROLr_SAFETYMARGIN(rv) ((rv)>>0&0xffff)
#define SET_CBPCONTROLr_SAFETYMARGIN(rv,fv) (rv=(rv&~(0xffff<<0))|((fv)&0xffff)<<0)

#define CFAPPOOLSIZEr_ADDR()            reg_addr(CFAPPOOLSIZEr)
#define READ_CFAPPOOLSIZEr(unit,rvp) \
	cmic_read_reg(unit,CFAPPOOLSIZEr_ADDR(),rvp)
#define WRITE_CFAPPOOLSIZEr(unit,rv) \
	cmic_write_reg(unit,CFAPPOOLSIZEr_ADDR(),rv)
#define GET_CFAPPOOLSIZEr_CFAPPOOLSIZE(rv) ((rv)>>0&0xffff)
#define SET_CFAPPOOLSIZEr_CFAPPOOLSIZE(rv,fv) (rv=(rv&~(0xffff<<0))|((fv)&0xffff)<<0)

#define CFAPREADPOINTERr_ADDR()         reg_addr(CFAPREADPOINTERr)
#define READ_CFAPREADPOINTERr(unit,rvp) \
	cmic_read_reg(unit,CFAPREADPOINTERr_ADDR(),rvp)
#define WRITE_CFAPREADPOINTERr(unit,rv) \
	cmic_write_reg(unit,CFAPREADPOINTERr_ADDR(),rv)
#define GET_CFAPREADPOINTERr_CRP(rv)    ((rv)>>0&0xffff)
#define SET_CFAPREADPOINTERr_CRP(rv,fv) (rv=(rv&~(0xffff<<0))|((fv)&0xffff)<<0)

#define CONFIGr_ADDR(pic)               reg_addr(CONFIGr,pic)
#define READ_CONFIGr(unit,pic,rvp) \
	cmic_read_reg(unit,CONFIGr_ADDR(pic),rvp)
#define WRITE_CONFIGr(unit,pic,rv) \
	cmic_write_reg(unit,CONFIGr_ADDR(pic),rv)
#define GET_CONFIGr_BIST_DONE(rv)       ((rv)>>11&0x3)
#define SET_CONFIGr_BIST_DONE(rv,fv)    (rv=(rv&~(0x3<<11))|((fv)&0x3)<<11)
#define GET_CONFIGr_CLK_GRAN(rv)        ((rv)>>13&0x7)
#define SET_CONFIGr_CLK_GRAN(rv,fv)     (rv=(rv&~(0x7<<13))|((fv)&0x7)<<13)
#define GET_CONFIGr_CPU_PRI(rv)         ((rv)>>6&0x7)
#define SET_CONFIGr_CPU_PRI(rv,fv)      (rv=(rv&~(0x7<<6))|((fv)&0x7)<<6)
#define GET_CONFIGr_DST_HIT(rv)         ((rv)>>5&0x1)
#define SET_CONFIGr_DST_HIT(rv,fv)      (rv=(rv&~(0x1<<5))|((fv)&0x1)<<5)
#define GET_CONFIGr_FIL_EN(rv)          ((rv)>>3&0x1)
#define SET_CONFIGr_FIL_EN(rv,fv)       (rv=(rv&~(0x1<<3))|((fv)&0x1)<<3)
#define GET_CONFIGr_FX100(rv)           ((rv)>>31&0x1)
#define SET_CONFIGr_FX100(rv,fv)        (rv=(rv&~(0x1<<31))|((fv)&0x1)<<31)
#define GET_CONFIGr_IN_BIST(rv)         ((rv)>>9&0x3)
#define SET_CONFIGr_IN_BIST(rv,fv)      (rv=(rv&~(0x3<<9))|((fv)&0x3)<<9)
#define GET_CONFIGr_IP_CFG(rv)          ((rv)>>0&0x3)
#define SET_CONFIGr_IP_CFG(rv,fv)       (rv=(rv&~(0x3<<0))|((fv)&0x3)<<0)
#define GET_CONFIGr_JAM_EN(rv)          ((rv)>>4&0x1)
#define SET_CONFIGr_JAM_EN(rv,fv)       (rv=(rv&~(0x1<<4))|((fv)&0x1)<<4)
#define GET_CONFIGr_L3_ENA(rv)          ((rv)>>2&0x1)
#define SET_CONFIGr_L3_ENA(rv,fv)       (rv=(rv&~(0x1<<2))|((fv)&0x1)<<2)
#define GET_CONFIGr_ST_IS_MIRR(rv)      ((rv)>>25&0x1)
#define SET_CONFIGr_ST_IS_MIRR(rv,fv)   (rv=(rv&~(0x1<<25))|((fv)&0x1)<<25)
#define GET_CONFIGr_ST_LINK(rv)         ((rv)>>17&0x1)
#define SET_CONFIGr_ST_LINK(rv,fv)      (rv=(rv&~(0x1<<17))|((fv)&0x1)<<17)
#define GET_CONFIGr_ST_MCNT(rv)         ((rv)>>20&0x1f)
#define SET_CONFIGr_ST_MCNT(rv,fv)      (rv=(rv&~(0x1f<<20))|((fv)&0x1f)<<20)
#define GET_CONFIGr_ST_MODULE(rv)       ((rv)>>18&0x1)
#define SET_CONFIGr_ST_MODULE(rv,fv)    (rv=(rv&~(0x1<<18))|((fv)&0x1)<<18)
#define GET_CONFIGr_ST_SIMPLEX(rv)      ((rv)>>19&0x1)
#define SET_CONFIGr_ST_SIMPLEX(rv,fv)   (rv=(rv&~(0x1<<19))|((fv)&0x1)<<19)
#define GET_CONFIGr_TAG_EN(rv)          ((rv)>>16&0x1)
#define SET_CONFIGr_TAG_EN(rv,fv)       (rv=(rv&~(0x1<<16))|((fv)&0x1)<<16)
#define GET_CONFIGr_TIMER_EN(rv)        ((rv)>>26&0x1)
#define SET_CONFIGr_TIMER_EN(rv,fv)     (rv=(rv&~(0x1<<26))|((fv)&0x1)<<26)

#define COS_ENABLEr_ADDR(cos,pic)       reg_addr(COS_ENABLEr,cos,pic)
#define READ_COS_ENABLEr(unit,cos,pic,rvp) \
	cmic_read_reg(unit,COS_ENABLEr_ADDR(cos,pic),rvp)
#define WRITE_COS_ENABLEr(unit,cos,pic,rv) \
	cmic_write_reg(unit,COS_ENABLEr_ADDR(cos,pic),rv)
#define GET_COS_ENABLEr_PORT_BITMAP(rv) ((rv)>>0&0xfffffff)
#define SET_COS_ENABLEr_PORT_BITMAP(rv,fv) (rv=(rv&~(0xfffffff<<0))|((fv)&0xfffffff)<<0)

#define COS_SELr_ADDR(pic)              reg_addr(COS_SELr,pic)
#define READ_COS_SELr(unit,pic,rvp) \
	cmic_read_reg(unit,COS_SELr_ADDR(pic),rvp)
#define WRITE_COS_SELr(unit,pic,rv) \
	cmic_write_reg(unit,COS_SELr_ADDR(pic),rv)
#define GET_COS_SELr_COS0(rv)           ((rv)>>0&0x7)
#define SET_COS_SELr_COS0(rv,fv)        (rv=(rv&~(0x7<<0))|((fv)&0x7)<<0)
#define GET_COS_SELr_COS1(rv)           ((rv)>>3&0x7)
#define SET_COS_SELr_COS1(rv,fv)        (rv=(rv&~(0x7<<3))|((fv)&0x7)<<3)
#define GET_COS_SELr_COS2(rv)           ((rv)>>6&0x7)
#define SET_COS_SELr_COS2(rv,fv)        (rv=(rv&~(0x7<<6))|((fv)&0x7)<<6)
#define GET_COS_SELr_COS3(rv)           ((rv)>>9&0x7)
#define SET_COS_SELr_COS3(rv,fv)        (rv=(rv&~(0x7<<9))|((fv)&0x7)<<9)
#define GET_COS_SELr_COS4(rv)           ((rv)>>12&0x7)
#define SET_COS_SELr_COS4(rv,fv)        (rv=(rv&~(0x7<<12))|((fv)&0x7)<<12)
#define GET_COS_SELr_COS5(rv)           ((rv)>>15&0x7)
#define SET_COS_SELr_COS5(rv,fv)        (rv=(rv&~(0x7<<15))|((fv)&0x7)<<15)
#define GET_COS_SELr_COS6(rv)           ((rv)>>18&0x7)
#define SET_COS_SELr_COS6(rv,fv)        (rv=(rv&~(0x7<<18))|((fv)&0x7)<<18)
#define GET_COS_SELr_COS7(rv)           ((rv)>>21&0x7)
#define SET_COS_SELr_COS7(rv,fv)        (rv=(rv&~(0x7<<21))|((fv)&0x7)<<21)

#define DEST_MIRRr_ADDR(pic)            reg_addr(DEST_MIRRr,pic)
#define READ_DEST_MIRRr(unit,pic,rvp) \
	cmic_read_reg(unit,DEST_MIRRr_ADDR(pic),rvp)
#define WRITE_DEST_MIRRr(unit,pic,rv) \
	cmic_write_reg(unit,DEST_MIRRr_ADDR(pic),rv)
#define GET_DEST_MIRRr_PORT_BITMAP(rv)  ((rv)>>0&0xfffffff)
#define SET_DEST_MIRRr_PORT_BITMAP(rv,fv) (rv=(rv&~(0xfffffff<<0))|((fv)&0xfffffff)<<0)

#define DROPPEDCELLCOUNTr_ADDR()        reg_addr(DROPPEDCELLCOUNTr)
#define READ_DROPPEDCELLCOUNTr(unit,rvp) \
	cmic_read_reg(unit,DROPPEDCELLCOUNTr_ADDR(),rvp)
#define WRITE_DROPPEDCELLCOUNTr(unit,rv) \
	cmic_write_reg(unit,DROPPEDCELLCOUNTr_ADDR(),rv)
#define GET_DROPPEDCELLCOUNTr_DCC(rv)   ((rv)>>0&0xffff)
#define SET_DROPPEDCELLCOUNTr_DCC(rv,fv) (rv=(rv&~(0xffff<<0))|((fv)&0xffff)<<0)

#define DROPPEDPACKETCOUNTr_ADDR()      reg_addr(DROPPEDPACKETCOUNTr)
#define READ_DROPPEDPACKETCOUNTr(unit,rvp) \
	cmic_read_reg(unit,DROPPEDPACKETCOUNTr_ADDR(),rvp)
#define WRITE_DROPPEDPACKETCOUNTr(unit,rv) \
	cmic_write_reg(unit,DROPPEDPACKETCOUNTr_ADDR(),rv)
#define GET_DROPPEDPACKETCOUNTr_DPC(rv) ((rv)>>0&0xffff)
#define SET_DROPPEDPACKETCOUNTr_DPC(rv,fv) (rv=(rv&~(0xffff<<0))|((fv)&0xffff)<<0)

#define EGR_BKP_DISr_ADDR(pic)          reg_addr(EGR_BKP_DISr,pic)
#define READ_EGR_BKP_DISr(unit,pic,rvp) \
	cmic_read_reg(unit,EGR_BKP_DISr_ADDR(pic),rvp)
#define WRITE_EGR_BKP_DISr(unit,pic,rv) \
	cmic_write_reg(unit,EGR_BKP_DISr_ADDR(pic),rv)
#define GET_EGR_BKP_DISr_PORT_BITMAP(rv) ((rv)>>0&0xfffffff)
#define SET_EGR_BKP_DISr_PORT_BITMAP(rv,fv) (rv=(rv&~(0xfffffff<<0))|((fv)&0xfffffff)<<0)

#define EGR_ENABLEr_ADDR(pic)           reg_addr(EGR_ENABLEr,pic)
#define READ_EGR_ENABLEr(unit,pic,rvp) \
	cmic_read_reg(unit,EGR_ENABLEr_ADDR(pic),rvp)
#define WRITE_EGR_ENABLEr(unit,pic,rv) \
	cmic_write_reg(unit,EGR_ENABLEr_ADDR(pic),rv)
#define GET_EGR_ENABLEr_PRT_ENABLE(rv)  ((rv)>>0&0xff)
#define SET_EGR_ENABLEr_PRT_ENABLE(rv,fv) (rv=(rv&~(0xff<<0))|((fv)&0xff)<<0)

#define EGR_MAXTIMEr_ADDR(pic)          reg_addr(EGR_MAXTIMEr,pic)
#define READ_EGR_MAXTIMEr(unit,pic,rvp) \
	cmic_read_reg(unit,EGR_MAXTIMEr_ADDR(pic),rvp)
#define WRITE_EGR_MAXTIMEr(unit,pic,rv) \
	cmic_write_reg(unit,EGR_MAXTIMEr_ADDR(pic),rv)
#define GET_EGR_MAXTIMEr_MAXTIME(rv)    ((rv)>>0&0x3fff)
#define SET_EGR_MAXTIMEr_MAXTIME(rv,fv) (rv=(rv&~(0x3fff<<0))|((fv)&0x3fff)<<0)

#define EN_IFILTERr_ADDR(pic)           reg_addr(EN_IFILTERr,pic)
#define READ_EN_IFILTERr(unit,pic,rvp) \
	cmic_read_reg(unit,EN_IFILTERr_ADDR(pic),rvp)
#define WRITE_EN_IFILTERr(unit,pic,rv) \
	cmic_write_reg(unit,EN_IFILTERr_ADDR(pic),rv)
#define GET_EN_IFILTERr_PORT_BITMAP(rv) ((rv)>>0&0xfffffff)
#define SET_EN_IFILTERr_PORT_BITMAP(rv,fv) (rv=(rv&~(0xfffffff<<0))|((fv)&0xfffffff)<<0)

#define EPC_BIST1r_ADDR(pic)            reg_addr(EPC_BIST1r,pic)
#define READ_EPC_BIST1r(unit,pic,rvp) \
	cmic_read_reg(unit,EPC_BIST1r_ADDR(pic),rvp)
#define WRITE_EPC_BIST1r(unit,pic,rv) \
	cmic_write_reg(unit,EPC_BIST1r_ADDR(pic),rv)
#define GET_EPC_BIST1r_ADDR1(rv)        ((rv)>>0&0x3fff)
#define SET_EPC_BIST1r_ADDR1(rv,fv)     (rv=(rv&~(0x3fff<<0))|((fv)&0x3fff)<<0)
#define GET_EPC_BIST1r_ADDR2(rv)        ((rv)>>14&0x3fff)
#define SET_EPC_BIST1r_ADDR2(rv,fv)     (rv=(rv&~(0x3fff<<14))|((fv)&0x3fff)<<14)
#define GET_EPC_BIST1r_FCNT(rv)         ((rv)>>28&0xf)
#define SET_EPC_BIST1r_FCNT(rv,fv)      (rv=(rv&~(0xf<<28))|((fv)&0xf)<<28)

#define EPC_BIST2r_ADDR(pic)            reg_addr(EPC_BIST2r,pic)
#define READ_EPC_BIST2r(unit,pic,rvp) \
	cmic_read_reg(unit,EPC_BIST2r_ADDR(pic),rvp)
#define WRITE_EPC_BIST2r(unit,pic,rv) \
	cmic_write_reg(unit,EPC_BIST2r_ADDR(pic),rv)
#define GET_EPC_BIST2r_ADDR1(rv)        ((rv)>>0&0x3fff)
#define SET_EPC_BIST2r_ADDR1(rv,fv)     (rv=(rv&~(0x3fff<<0))|((fv)&0x3fff)<<0)
#define GET_EPC_BIST2r_ADDR2(rv)        ((rv)>>14&0x3fff)
#define SET_EPC_BIST2r_ADDR2(rv,fv)     (rv=(rv&~(0x3fff<<14))|((fv)&0x3fff)<<14)
#define GET_EPC_BIST2r_FCNT(rv)         ((rv)>>28&0xf)
#define SET_EPC_BIST2r_FCNT(rv,fv)      (rv=(rv&~(0xf<<28))|((fv)&0xf)<<28)

#define EPC_BPDU1r_ADDR(pic)            reg_addr(EPC_BPDU1r,pic)
#define READ_EPC_BPDU1r(unit,pic,rvp) \
	cmic_read_reg(unit,EPC_BPDU1r_ADDR(pic),rvp)
#define WRITE_EPC_BPDU1r(unit,pic,rv) \
	cmic_write_reg(unit,EPC_BPDU1r_ADDR(pic),rv)
#define GET_EPC_BPDU1r_BPDU1(rv)        ((rv)>>0&0xffffffff)
#define SET_EPC_BPDU1r_BPDU1(rv,fv)     (rv=(rv&~(0xffffffff<<0))|((fv)&0xffffffff)<<0)

#define EPC_BPDU2r_ADDR(pic)            reg_addr(EPC_BPDU2r,pic)
#define READ_EPC_BPDU2r(unit,pic,rvp) \
	cmic_read_reg(unit,EPC_BPDU2r_ADDR(pic),rvp)
#define WRITE_EPC_BPDU2r(unit,pic,rv) \
	cmic_write_reg(unit,EPC_BPDU2r_ADDR(pic),rv)
#define GET_EPC_BPDU2r_BPDU2(rv)        ((rv)>>0&0xffff)
#define SET_EPC_BPDU2r_BPDU2(rv,fv)     (rv=(rv&~(0xffff<<0))|((fv)&0xffff)<<0)
#define GET_EPC_BPDU2r_BPDU3(rv)        ((rv)>>16&0xffff)
#define SET_EPC_BPDU2r_BPDU3(rv,fv)     (rv=(rv&~(0xffff<<16))|((fv)&0xffff)<<16)

#define EPC_BPDU4r_ADDR(pic)            reg_addr(EPC_BPDU4r,pic)
#define READ_EPC_BPDU4r(unit,pic,rvp) \
	cmic_read_reg(unit,EPC_BPDU4r_ADDR(pic),rvp)
#define WRITE_EPC_BPDU4r(unit,pic,rv) \
	cmic_write_reg(unit,EPC_BPDU4r_ADDR(pic),rv)
#define GET_EPC_BPDU4r_BPDU4(rv)        ((rv)>>0&0xffffffff)
#define SET_EPC_BPDU4r_BPDU4(rv,fv)     (rv=(rv&~(0xffffffff<<0))|((fv)&0xffffffff)<<0)

#define EPC_EGR_DBGr_ADDR(pic)          reg_addr(EPC_EGR_DBGr,pic)
#define READ_EPC_EGR_DBGr(unit,pic,rvp) \
	cmic_read_reg(unit,EPC_EGR_DBGr_ADDR(pic),rvp)
#define WRITE_EPC_EGR_DBGr(unit,pic,rv) \
	cmic_write_reg(unit,EPC_EGR_DBGr_ADDR(pic),rv)
#define GET_EPC_EGR_DBGr_DEBUG_ON(rv)   ((rv)>>0&0xff)
#define SET_EPC_EGR_DBGr_DEBUG_ON(rv,fv) (rv=(rv&~(0xff<<0))|((fv)&0xff)<<0)

#define EPC_EGR_SNGL_OUTr_ADDR(pic)     reg_addr(EPC_EGR_SNGL_OUTr,pic)
#define READ_EPC_EGR_SNGL_OUTr(unit,pic,rvp) \
	cmic_read_reg(unit,EPC_EGR_SNGL_OUTr_ADDR(pic),rvp)
#define WRITE_EPC_EGR_SNGL_OUTr(unit,pic,rv) \
	cmic_write_reg(unit,EPC_EGR_SNGL_OUTr_ADDR(pic),rv)
#define GET_EPC_EGR_SNGL_OUTr_PRT_ENABLE(rv) ((rv)>>0&0xff)
#define SET_EPC_EGR_SNGL_OUTr_PRT_ENABLE(rv,fv) (rv=(rv&~(0xff<<0))|((fv)&0xff)<<0)

#define EPC_EGR_SNGL_PKTr_ADDR(pic)     reg_addr(EPC_EGR_SNGL_PKTr,pic)
#define READ_EPC_EGR_SNGL_PKTr(unit,pic,rvp) \
	cmic_read_reg(unit,EPC_EGR_SNGL_PKTr_ADDR(pic),rvp)
#define WRITE_EPC_EGR_SNGL_PKTr(unit,pic,rv) \
	cmic_write_reg(unit,EPC_EGR_SNGL_PKTr_ADDR(pic),rv)
#define GET_EPC_EGR_SNGL_PKTr_PRT_ENABLE(rv) ((rv)>>0&0xff)
#define SET_EPC_EGR_SNGL_PKTr_PRT_ENABLE(rv,fv) (rv=(rv&~(0xff<<0))|((fv)&0xff)<<0)

#define EPC_HOLr_ADDR(pic)              reg_addr(EPC_HOLr,pic)
#define READ_EPC_HOLr(unit,pic,rvp) \
	cmic_read_reg(unit,EPC_HOLr_ADDR(pic),rvp)
#define WRITE_EPC_HOLr(unit,pic,rv) \
	cmic_write_reg(unit,EPC_HOLr_ADDR(pic),rv)
#define GET_EPC_HOLr_PORT_BITMAP(rv)    ((rv)>>0&0xfffffff)
#define SET_EPC_HOLr_PORT_BITMAP(rv,fv) (rv=(rv&~(0xfffffff<<0))|((fv)&0xfffffff)<<0)

#define EPC_LINKr_ADDR(pic)             reg_addr(EPC_LINKr,pic)
#define READ_EPC_LINKr(unit,pic,rvp) \
	cmic_read_reg(unit,EPC_LINKr_ADDR(pic),rvp)
#define WRITE_EPC_LINKr(unit,pic,rv) \
	cmic_write_reg(unit,EPC_LINKr_ADDR(pic),rv)
#define GET_EPC_LINKr_PORT_BITMAP(rv)   ((rv)>>0&0xfffffff)
#define SET_EPC_LINKr_PORT_BITMAP(rv,fv) (rv=(rv&~(0xfffffff<<0))|((fv)&0xfffffff)<<0)

#define EPC_PFM0r_ADDR(pic)             reg_addr(EPC_PFM0r,pic)
#define READ_EPC_PFM0r(unit,pic,rvp) \
	cmic_read_reg(unit,EPC_PFM0r_ADDR(pic),rvp)
#define WRITE_EPC_PFM0r(unit,pic,rv) \
	cmic_write_reg(unit,EPC_PFM0r_ADDR(pic),rv)
#define GET_EPC_PFM0r_FE0(rv)           ((rv)>>0&0x3)
#define SET_EPC_PFM0r_FE0(rv,fv)        (rv=(rv&~(0x3<<0))|((fv)&0x3)<<0)
#define GET_EPC_PFM0r_FE1(rv)           ((rv)>>2&0x3)
#define SET_EPC_PFM0r_FE1(rv,fv)        (rv=(rv&~(0x3<<2))|((fv)&0x3)<<2)
#define GET_EPC_PFM0r_FE10(rv)          ((rv)>>20&0x3)
#define SET_EPC_PFM0r_FE10(rv,fv)       (rv=(rv&~(0x3<<20))|((fv)&0x3)<<20)
#define GET_EPC_PFM0r_FE11(rv)          ((rv)>>22&0x3)
#define SET_EPC_PFM0r_FE11(rv,fv)       (rv=(rv&~(0x3<<22))|((fv)&0x3)<<22)
#define GET_EPC_PFM0r_FE12(rv)          ((rv)>>24&0x3)
#define SET_EPC_PFM0r_FE12(rv,fv)       (rv=(rv&~(0x3<<24))|((fv)&0x3)<<24)
#define GET_EPC_PFM0r_FE13(rv)          ((rv)>>26&0x3)
#define SET_EPC_PFM0r_FE13(rv,fv)       (rv=(rv&~(0x3<<26))|((fv)&0x3)<<26)
#define GET_EPC_PFM0r_FE14(rv)          ((rv)>>28&0x3)
#define SET_EPC_PFM0r_FE14(rv,fv)       (rv=(rv&~(0x3<<28))|((fv)&0x3)<<28)
#define GET_EPC_PFM0r_FE15(rv)          ((rv)>>30&0x3)
#define SET_EPC_PFM0r_FE15(rv,fv)       (rv=(rv&~(0x3<<30))|((fv)&0x3)<<30)
#define GET_EPC_PFM0r_FE2(rv)           ((rv)>>4&0x3)
#define SET_EPC_PFM0r_FE2(rv,fv)        (rv=(rv&~(0x3<<4))|((fv)&0x3)<<4)
#define GET_EPC_PFM0r_FE3(rv)           ((rv)>>6&0x3)
#define SET_EPC_PFM0r_FE3(rv,fv)        (rv=(rv&~(0x3<<6))|((fv)&0x3)<<6)
#define GET_EPC_PFM0r_FE4(rv)           ((rv)>>8&0x3)
#define SET_EPC_PFM0r_FE4(rv,fv)        (rv=(rv&~(0x3<<8))|((fv)&0x3)<<8)
#define GET_EPC_PFM0r_FE5(rv)           ((rv)>>10&0x3)
#define SET_EPC_PFM0r_FE5(rv,fv)        (rv=(rv&~(0x3<<10))|((fv)&0x3)<<10)
#define GET_EPC_PFM0r_FE6(rv)           ((rv)>>12&0x3)
#define SET_EPC_PFM0r_FE6(rv,fv)        (rv=(rv&~(0x3<<12))|((fv)&0x3)<<12)
#define GET_EPC_PFM0r_FE7(rv)           ((rv)>>14&0x3)
#define SET_EPC_PFM0r_FE7(rv,fv)        (rv=(rv&~(0x3<<14))|((fv)&0x3)<<14)
#define GET_EPC_PFM0r_FE8(rv)           ((rv)>>16&0x3)
#define SET_EPC_PFM0r_FE8(rv,fv)        (rv=(rv&~(0x3<<16))|((fv)&0x3)<<16)
#define GET_EPC_PFM0r_FE9(rv)           ((rv)>>18&0x3)
#define SET_EPC_PFM0r_FE9(rv,fv)        (rv=(rv&~(0x3<<18))|((fv)&0x3)<<18)

#define EPC_PFM1r_ADDR(pic)             reg_addr(EPC_PFM1r,pic)
#define READ_EPC_PFM1r(unit,pic,rvp) \
	cmic_read_reg(unit,EPC_PFM1r_ADDR(pic),rvp)
#define WRITE_EPC_PFM1r(unit,pic,rv) \
	cmic_write_reg(unit,EPC_PFM1r_ADDR(pic),rv)
#define GET_EPC_PFM1r_FE16(rv)          ((rv)>>0&0x3)
#define SET_EPC_PFM1r_FE16(rv,fv)       (rv=(rv&~(0x3<<0))|((fv)&0x3)<<0)
#define GET_EPC_PFM1r_FE17(rv)          ((rv)>>2&0x3)
#define SET_EPC_PFM1r_FE17(rv,fv)       (rv=(rv&~(0x3<<2))|((fv)&0x3)<<2)
#define GET_EPC_PFM1r_FE18(rv)          ((rv)>>4&0x3)
#define SET_EPC_PFM1r_FE18(rv,fv)       (rv=(rv&~(0x3<<4))|((fv)&0x3)<<4)
#define GET_EPC_PFM1r_FE19(rv)          ((rv)>>6&0x3)
#define SET_EPC_PFM1r_FE19(rv,fv)       (rv=(rv&~(0x3<<6))|((fv)&0x3)<<6)
#define GET_EPC_PFM1r_FE20(rv)          ((rv)>>8&0x3)
#define SET_EPC_PFM1r_FE20(rv,fv)       (rv=(rv&~(0x3<<8))|((fv)&0x3)<<8)
#define GET_EPC_PFM1r_FE21(rv)          ((rv)>>10&0x3)
#define SET_EPC_PFM1r_FE21(rv,fv)       (rv=(rv&~(0x3<<10))|((fv)&0x3)<<10)
#define GET_EPC_PFM1r_FE22(rv)          ((rv)>>12&0x3)
#define SET_EPC_PFM1r_FE22(rv,fv)       (rv=(rv&~(0x3<<12))|((fv)&0x3)<<12)
#define GET_EPC_PFM1r_FE23(rv)          ((rv)>>14&0x3)
#define SET_EPC_PFM1r_FE23(rv,fv)       (rv=(rv&~(0x3<<14))|((fv)&0x3)<<14)
#define GET_EPC_PFM1r_GE0(rv)           ((rv)>>16&0x3)
#define SET_EPC_PFM1r_GE0(rv,fv)        (rv=(rv&~(0x3<<16))|((fv)&0x3)<<16)
#define GET_EPC_PFM1r_GE1(rv)           ((rv)>>18&0x3)
#define SET_EPC_PFM1r_GE1(rv,fv)        (rv=(rv&~(0x3<<18))|((fv)&0x3)<<18)

#define EPC_RATE_N_IPr_ADDR(pic)        reg_addr(EPC_RATE_N_IPr,pic)
#define READ_EPC_RATE_N_IPr(unit,pic,rvp) \
	cmic_read_reg(unit,EPC_RATE_N_IPr_ADDR(pic),rvp)
#define WRITE_EPC_RATE_N_IPr(unit,pic,rv) \
	cmic_write_reg(unit,EPC_RATE_N_IPr_ADDR(pic),rv)
#define GET_EPC_RATE_N_IPr_BCAST_EN(rv) ((rv)>>20&0x1)
#define SET_EPC_RATE_N_IPr_BCAST_EN(rv,fv) (rv=(rv&~(0x1<<20))|((fv)&0x1)<<20)
#define GET_EPC_RATE_N_IPr_DLFBC_EN(rv) ((rv)>>18&0x1)
#define SET_EPC_RATE_N_IPr_DLFBC_EN(rv,fv) (rv=(rv&~(0x1<<18))|((fv)&0x1)<<18)
#define GET_EPC_RATE_N_IPr_IPX_SIZE(rv) ((rv)>>26&0x1f)
#define SET_EPC_RATE_N_IPr_IPX_SIZE(rv,fv) (rv=(rv&~(0x1f<<26))|((fv)&0x1f)<<26)
#define GET_EPC_RATE_N_IPr_IP_SIZE(rv)  ((rv)>>21&0x1f)
#define SET_EPC_RATE_N_IPr_IP_SIZE(rv,fv) (rv=(rv&~(0x1f<<21))|((fv)&0x1f)<<21)
#define GET_EPC_RATE_N_IPr_LIMIT(rv)    ((rv)>>0&0x3ffff)
#define SET_EPC_RATE_N_IPr_LIMIT(rv,fv) (rv=(rv&~(0x3ffff<<0))|((fv)&0x3ffff)<<0)
#define GET_EPC_RATE_N_IPr_MCAST_EN(rv) ((rv)>>19&0x1)
#define SET_EPC_RATE_N_IPr_MCAST_EN(rv,fv) (rv=(rv&~(0x1<<19))|((fv)&0x1)<<19)

#define EPC_STAD1r_ADDR(pic)            reg_addr(EPC_STAD1r,pic)
#define READ_EPC_STAD1r(unit,pic,rvp) \
	cmic_read_reg(unit,EPC_STAD1r_ADDR(pic),rvp)
#define WRITE_EPC_STAD1r(unit,pic,rv) \
	cmic_write_reg(unit,EPC_STAD1r_ADDR(pic),rv)
#define GET_EPC_STAD1r_DUMMY(rv)        ((rv)>>0&0x1)
#define SET_EPC_STAD1r_DUMMY(rv,fv)     (rv=(rv&~(0x1<<0))|((fv)&0x1)<<0)

#define ESA0r_ADDR(port)                reg_addr(ESA0r,port)
#define READ_ESA0r(unit,port,rvp) \
	cmic_read_reg(unit,ESA0r_ADDR(port),rvp)
#define WRITE_ESA0r(unit,port,rv) \
	cmic_write_reg(unit,ESA0r_ADDR(port),rv)
#define GET_ESA0r_STAD0(rv)             ((rv)>>0&0xffff)
#define SET_ESA0r_STAD0(rv,fv)          (rv=(rv&~(0xffff<<0))|((fv)&0xffff)<<0)

#define ESA1r_ADDR(port)                reg_addr(ESA1r,port)
#define READ_ESA1r(unit,port,rvp) \
	cmic_read_reg(unit,ESA1r_ADDR(port),rvp)
#define WRITE_ESA1r(unit,port,rv) \
	cmic_write_reg(unit,ESA1r_ADDR(port),rv)
#define GET_ESA1r_STAD0(rv)             ((rv)>>0&0xffff)
#define SET_ESA1r_STAD0(rv,fv)          (rv=(rv&~(0xffff<<0))|((fv)&0xffff)<<0)

#define ESA2r_ADDR(port)                reg_addr(ESA2r,port)
#define READ_ESA2r(unit,port,rvp) \
	cmic_read_reg(unit,ESA2r_ADDR(port),rvp)
#define WRITE_ESA2r(unit,port,rv) \
	cmic_write_reg(unit,ESA2r_ADDR(port),rv)
#define GET_ESA2r_STAD0(rv)             ((rv)>>0&0xffff)
#define SET_ESA2r_STAD0(rv,fv)          (rv=(rv&~(0xffff<<0))|((fv)&0xffff)<<0)

#define FE_CLRTr_ADDR(port)             reg_addr(FE_CLRTr,port)
#define READ_FE_CLRTr(unit,port,rvp) \
	cmic_read_reg(unit,FE_CLRTr_ADDR(port),rvp)
#define WRITE_FE_CLRTr(unit,port,rv) \
	cmic_write_reg(unit,FE_CLRTr_ADDR(port),rv)
#define GET_FE_CLRTr_COL_WIN(rv)        ((rv)>>8&0x3f)
#define SET_FE_CLRTr_COL_WIN(rv,fv)     (rv=(rv&~(0x3f<<8))|((fv)&0x3f)<<8)
#define GET_FE_CLRTr_RETRY(rv)          ((rv)>>0&0xf)
#define SET_FE_CLRTr_RETRY(rv,fv)       (rv=(rv&~(0xf<<0))|((fv)&0xf)<<0)

#define FE_IPGRr_ADDR(port)             reg_addr(FE_IPGRr,port)
#define READ_FE_IPGRr(unit,port,rvp) \
	cmic_read_reg(unit,FE_IPGRr_ADDR(port),rvp)
#define WRITE_FE_IPGRr(unit,port,rv) \
	cmic_write_reg(unit,FE_IPGRr_ADDR(port),rv)
#define GET_FE_IPGRr_IPGR1(rv)          ((rv)>>8&0xff)
#define SET_FE_IPGRr_IPGR1(rv,fv)       (rv=(rv&~(0xff<<8))|((fv)&0xff)<<8)
#define GET_FE_IPGRr_IPGR2(rv)          ((rv)>>0&0xff)
#define SET_FE_IPGRr_IPGR2(rv,fv)       (rv=(rv&~(0xff<<0))|((fv)&0xff)<<0)

#define FE_IPGTr_ADDR(port)             reg_addr(FE_IPGTr,port)
#define READ_FE_IPGTr(unit,port,rvp) \
	cmic_read_reg(unit,FE_IPGTr_ADDR(port),rvp)
#define WRITE_FE_IPGTr(unit,port,rv) \
	cmic_write_reg(unit,FE_IPGTr_ADDR(port),rv)
#define GET_FE_IPGTr_IPGT(rv)           ((rv)>>0&0xff)
#define SET_FE_IPGTr_IPGT(rv,fv)        (rv=(rv&~(0xff<<0))|((fv)&0xff)<<0)

#define FE_MAC1r_ADDR(port)             reg_addr(FE_MAC1r,port)
#define READ_FE_MAC1r(unit,port,rvp) \
	cmic_read_reg(unit,FE_MAC1r_ADDR(port),rvp)
#define WRITE_FE_MAC1r(unit,port,rv) \
	cmic_write_reg(unit,FE_MAC1r_ADDR(port),rv)
#define GET_FE_MAC1r_LBACK(rv)          ((rv)>>4&0x1)
#define SET_FE_MAC1r_LBACK(rv,fv)       (rv=(rv&~(0x1<<4))|((fv)&0x1)<<4)
#define GET_FE_MAC1r_PASSALL(rv)        ((rv)>>1&0x1)
#define SET_FE_MAC1r_PASSALL(rv,fv)     (rv=(rv&~(0x1<<1))|((fv)&0x1)<<1)
#define GET_FE_MAC1r_RST_RFUN(rv)       ((rv)>>10&0x1)
#define SET_FE_MAC1r_RST_RFUN(rv,fv)    (rv=(rv&~(0x1<<10))|((fv)&0x1)<<10)
#define GET_FE_MAC1r_RST_RMCS(rv)       ((rv)>>11&0x1)
#define SET_FE_MAC1r_RST_RMCS(rv,fv)    (rv=(rv&~(0x1<<11))|((fv)&0x1)<<11)
#define GET_FE_MAC1r_RST_SIM(rv)        ((rv)>>14&0x1)
#define SET_FE_MAC1r_RST_SIM(rv,fv)     (rv=(rv&~(0x1<<14))|((fv)&0x1)<<14)
#define GET_FE_MAC1r_RST_TFUN(rv)       ((rv)>>8&0x1)
#define SET_FE_MAC1r_RST_TFUN(rv,fv)    (rv=(rv&~(0x1<<8))|((fv)&0x1)<<8)
#define GET_FE_MAC1r_RST_TMCS(rv)       ((rv)>>9&0x1)
#define SET_FE_MAC1r_RST_TMCS(rv,fv)    (rv=(rv&~(0x1<<9))|((fv)&0x1)<<9)
#define GET_FE_MAC1r_RX_EN(rv)          ((rv)>>0&0x1)
#define SET_FE_MAC1r_RX_EN(rv,fv)       (rv=(rv&~(0x1<<0))|((fv)&0x1)<<0)
#define GET_FE_MAC1r_RX_PAU(rv)         ((rv)>>2&0x1)
#define SET_FE_MAC1r_RX_PAU(rv,fv)      (rv=(rv&~(0x1<<2))|((fv)&0x1)<<2)
#define GET_FE_MAC1r_SOFT_RST(rv)       ((rv)>>15&0x1)
#define SET_FE_MAC1r_SOFT_RST(rv,fv)    (rv=(rv&~(0x1<<15))|((fv)&0x1)<<15)
#define GET_FE_MAC1r_TX_PAU(rv)         ((rv)>>3&0x1)
#define SET_FE_MAC1r_TX_PAU(rv,fv)      (rv=(rv&~(0x1<<3))|((fv)&0x1)<<3)

#define FE_MAC2r_ADDR(port)             reg_addr(FE_MAC2r,port)
#define READ_FE_MAC2r(unit,port,rvp) \
	cmic_read_reg(unit,FE_MAC2r_ADDR(port),rvp)
#define WRITE_FE_MAC2r(unit,port,rv) \
	cmic_write_reg(unit,FE_MAC2r_ADDR(port),rv)
#define GET_FE_MAC2r_AUTOZ(rv)          ((rv)>>15&0x1)
#define SET_FE_MAC2r_AUTOZ(rv,fv)       (rv=(rv&~(0x1<<15))|((fv)&0x1)<<15)
#define GET_FE_MAC2r_AUTO_PAD(rv)       ((rv)>>7&0x1)
#define SET_FE_MAC2r_AUTO_PAD(rv,fv)    (rv=(rv&~(0x1<<7))|((fv)&0x1)<<7)
#define GET_FE_MAC2r_BP_NO_BOFF(rv)     ((rv)>>13&0x1)
#define SET_FE_MAC2r_BP_NO_BOFF(rv,fv)  (rv=(rv&~(0x1<<13))|((fv)&0x1)<<13)
#define GET_FE_MAC2r_CLRCNT(rv)         ((rv)>>11&0x1)
#define SET_FE_MAC2r_CLRCNT(rv,fv)      (rv=(rv&~(0x1<<11))|((fv)&0x1)<<11)
#define GET_FE_MAC2r_CRC_EN(rv)         ((rv)>>4&0x1)
#define SET_FE_MAC2r_CRC_EN(rv,fv)      (rv=(rv&~(0x1<<4))|((fv)&0x1)<<4)
#define GET_FE_MAC2r_DEL_CRC(rv)        ((rv)>>3&0x1)
#define SET_FE_MAC2r_DEL_CRC(rv,fv)     (rv=(rv&~(0x1<<3))|((fv)&0x1)<<3)
#define GET_FE_MAC2r_EXC_DEF(rv)        ((rv)>>14&0x1)
#define SET_FE_MAC2r_EXC_DEF(rv,fv)     (rv=(rv&~(0x1<<14))|((fv)&0x1)<<14)
#define GET_FE_MAC2r_FULL_DUP(rv)       ((rv)>>0&0x1)
#define SET_FE_MAC2r_FULL_DUP(rv,fv)    (rv=(rv&~(0x1<<0))|((fv)&0x1)<<0)
#define GET_FE_MAC2r_HUGE_FR(rv)        ((rv)>>2&0x1)
#define SET_FE_MAC2r_HUGE_FR(rv,fv)     (rv=(rv&~(0x1<<2))|((fv)&0x1)<<2)
#define GET_FE_MAC2r_LG_CHK(rv)         ((rv)>>1&0x1)
#define SET_FE_MAC2r_LG_CHK(rv,fv)      (rv=(rv&~(0x1<<1))|((fv)&0x1)<<1)
#define GET_FE_MAC2r_LONG_PRE(rv)       ((rv)>>9&0x1)
#define SET_FE_MAC2r_LONG_PRE(rv,fv)    (rv=(rv&~(0x1<<9))|((fv)&0x1)<<9)
#define GET_FE_MAC2r_NO_BOFF(rv)        ((rv)>>12&0x1)
#define SET_FE_MAC2r_NO_BOFF(rv,fv)     (rv=(rv&~(0x1<<12))|((fv)&0x1)<<12)
#define GET_FE_MAC2r_PAD_EN(rv)         ((rv)>>5&0x1)
#define SET_FE_MAC2r_PAD_EN(rv,fv)      (rv=(rv&~(0x1<<5))|((fv)&0x1)<<5)
#define GET_FE_MAC2r_PURE_PAD(rv)       ((rv)>>8&0x1)
#define SET_FE_MAC2r_PURE_PAD(rv,fv)    (rv=(rv&~(0x1<<8))|((fv)&0x1)<<8)
#define GET_FE_MAC2r_VLAN_PAD(rv)       ((rv)>>6&0x1)
#define SET_FE_MAC2r_VLAN_PAD(rv,fv)    (rv=(rv&~(0x1<<6))|((fv)&0x1)<<6)

#define FE_MAXFr_ADDR(port)             reg_addr(FE_MAXFr,port)
#define READ_FE_MAXFr(unit,port,rvp) \
	cmic_read_reg(unit,FE_MAXFr_ADDR(port),rvp)
#define WRITE_FE_MAXFr(unit,port,rv) \
	cmic_write_reg(unit,FE_MAXFr_ADDR(port),rv)
#define GET_FE_MAXFr_MAXFR(rv)          ((rv)>>0&0xffff)
#define SET_FE_MAXFr_MAXFR(rv,fv)       (rv=(rv&~(0xffff<<0))|((fv)&0xffff)<<0)

#define FE_SUPPr_ADDR(port)             reg_addr(FE_SUPPr,port)
#define READ_FE_SUPPr(unit,port,rvp) \
	cmic_read_reg(unit,FE_SUPPr_ADDR(port),rvp)
#define WRITE_FE_SUPPr(unit,port,rv) \
	cmic_write_reg(unit,FE_SUPPr_ADDR(port),rv)
#define GET_FE_SUPPr_BIT_MODE(rv)       ((rv)>>0&0x1)
#define SET_FE_SUPPr_BIT_MODE(rv,fv)    (rv=(rv&~(0x1<<0))|((fv)&0x1)<<0)
#define GET_FE_SUPPr_ENBL_JABBR(rv)     ((rv)>>1&0x1)
#define SET_FE_SUPPr_ENBL_JABBR(rv,fv)  (rv=(rv&~(0x1<<1))|((fv)&0x1)<<1)
#define GET_FE_SUPPr_FORCE_QUIET(rv)    ((rv)>>6&0x1)
#define SET_FE_SUPPr_FORCE_QUIET(rv,fv) (rv=(rv&~(0x1<<6))|((fv)&0x1)<<6)
#define GET_FE_SUPPr_LNK_FAIL(rv)       ((rv)>>4&0x1)
#define SET_FE_SUPPr_LNK_FAIL(rv,fv)    (rv=(rv&~(0x1<<4))|((fv)&0x1)<<4)
#define GET_FE_SUPPr_NO_CIPHER(rv)      ((rv)>>5&0x1)
#define SET_FE_SUPPr_NO_CIPHER(rv,fv)   (rv=(rv&~(0x1<<5))|((fv)&0x1)<<5)
#define GET_FE_SUPPr_RESET_100X(rv)     ((rv)>>7&0x1)
#define SET_FE_SUPPr_RESET_100X(rv,fv)  (rv=(rv&~(0x1<<7))|((fv)&0x1)<<7)
#define GET_FE_SUPPr_RESET_10T(rv)      ((rv)>>2&0x1)
#define SET_FE_SUPPr_RESET_10T(rv,fv)   (rv=(rv&~(0x1<<2))|((fv)&0x1)<<2)
#define GET_FE_SUPPr_RESET_RMII(rv)     ((rv)>>11&0x1)
#define SET_FE_SUPPr_RESET_RMII(rv,fv)  (rv=(rv&~(0x1<<11))|((fv)&0x1)<<11)
#define GET_FE_SUPPr_SPEED(rv)          ((rv)>>8&0x1)
#define SET_FE_SUPPr_SPEED(rv,fv)       (rv=(rv&~(0x1<<8))|((fv)&0x1)<<8)

#define FE_TESTr_ADDR(port)             reg_addr(FE_TESTr,port)
#define READ_FE_TESTr(unit,port,rvp) \
	cmic_read_reg(unit,FE_TESTr_ADDR(port),rvp)
#define WRITE_FE_TESTr(unit,port,rv) \
	cmic_write_reg(unit,FE_TESTr_ADDR(port),rv)
#define GET_FE_TESTr_SHORT_QNTA(rv)     ((rv)>>0&0x1)
#define SET_FE_TESTr_SHORT_QNTA(rv,fv)  (rv=(rv&~(0x1<<0))|((fv)&0x1)<<0)
#define GET_FE_TESTr_TEST_BACK(rv)      ((rv)>>2&0x1)
#define SET_FE_TESTr_TEST_BACK(rv,fv)   (rv=(rv&~(0x1<<2))|((fv)&0x1)<<2)
#define GET_FE_TESTr_TEST_PAUSE(rv)     ((rv)>>1&0x1)
#define SET_FE_TESTr_TEST_PAUSE(rv,fv)  (rv=(rv&~(0x1<<1))|((fv)&0x1)<<1)

#define FLOWCONTROLr_ADDR(cos)          reg_addr(FLOWCONTROLr,cos)
#define READ_FLOWCONTROLr(unit,cos,rvp) \
	cmic_read_reg(unit,FLOWCONTROLr_ADDR(cos),rvp)
#define WRITE_FLOWCONTROLr(unit,cos,rv) \
	cmic_write_reg(unit,FLOWCONTROLr_ADDR(cos),rv)
#define GET_FLOWCONTROLr_MAXLATENCY(rv) ((rv)>>0&0xff)
#define SET_FLOWCONTROLr_MAXLATENCY(rv,fv) (rv=(rv&~(0xff<<0))|((fv)&0xff)<<0)
#define GET_FLOWCONTROLr_MAXPACKETS(rv) ((rv)>>16&0xff)
#define SET_FLOWCONTROLr_MAXPACKETS(rv,fv) (rv=(rv&~(0xff<<16))|((fv)&0xff)<<16)

#define GCAR1r_ADDR(port)               reg_addr(GCAR1r,port)
#define READ_GCAR1r(unit,port,rvp) \
	cmic_read_reg(unit,GCAR1r_ADDR(port),rvp)
#define WRITE_GCAR1r(unit,port,rv) \
	cmic_write_reg(unit,GCAR1r_ADDR(port),rv)
#define GET_GCAR1r_GRALN(rv)            ((rv)>>8&0x1)
#define SET_GCAR1r_GRALN(rv,fv)         (rv=(rv&~(0x1<<8))|((fv)&0x1)<<8)
#define GET_GCAR1r_GRBCA(rv)            ((rv)>>12&0x1)
#define SET_GCAR1r_GRBCA(rv,fv)         (rv=(rv&~(0x1<<12))|((fv)&0x1)<<12)
#define GET_GCAR1r_GRBYT(rv)            ((rv)>>16&0x1)
#define SET_GCAR1r_GRBYT(rv,fv)         (rv=(rv&~(0x1<<16))|((fv)&0x1)<<16)
#define GET_GCAR1r_GRCDE(rv)            ((rv)>>6&0x1)
#define SET_GCAR1r_GRCDE(rv,fv)         (rv=(rv&~(0x1<<6))|((fv)&0x1)<<6)
#define GET_GCAR1r_GRCSE(rv)            ((rv)>>5&0x1)
#define SET_GCAR1r_GRCSE(rv,fv)         (rv=(rv&~(0x1<<5))|((fv)&0x1)<<5)
#define GET_GCAR1r_GRFCS(rv)            ((rv)>>14&0x1)
#define SET_GCAR1r_GRFCS(rv,fv)         (rv=(rv&~(0x1<<14))|((fv)&0x1)<<14)
#define GET_GCAR1r_GRFLR(rv)            ((rv)>>7&0x1)
#define SET_GCAR1r_GRFLR(rv,fv)         (rv=(rv&~(0x1<<7))|((fv)&0x1)<<7)
#define GET_GCAR1r_GRJBR(rv)            ((rv)>>1&0x1)
#define SET_GCAR1r_GRJBR(rv,fv)         (rv=(rv&~(0x1<<1))|((fv)&0x1)<<1)
#define GET_GCAR1r_GRMCA(rv)            ((rv)>>13&0x1)
#define SET_GCAR1r_GRMCA(rv,fv)         (rv=(rv&~(0x1<<13))|((fv)&0x1)<<13)
#define GET_GCAR1r_GROVR(rv)            ((rv)>>3&0x1)
#define SET_GCAR1r_GROVR(rv,fv)         (rv=(rv&~(0x1<<3))|((fv)&0x1)<<3)
#define GET_GCAR1r_GRPKT(rv)            ((rv)>>15&0x1)
#define SET_GCAR1r_GRPKT(rv,fv)         (rv=(rv&~(0x1<<15))|((fv)&0x1)<<15)
#define GET_GCAR1r_GRUND(rv)            ((rv)>>4&0x1)
#define SET_GCAR1r_GRUND(rv,fv)         (rv=(rv&~(0x1<<4))|((fv)&0x1)<<4)
#define GET_GCAR1r_GRUXO(rv)            ((rv)>>9&0x1)
#define SET_GCAR1r_GRUXO(rv,fv)         (rv=(rv&~(0x1<<9))|((fv)&0x1)<<9)
#define GET_GCAR1r_GRXCF(rv)            ((rv)>>11&0x1)
#define SET_GCAR1r_GRXCF(rv,fv)         (rv=(rv&~(0x1<<11))|((fv)&0x1)<<11)
#define GET_GCAR1r_GRXPF(rv)            ((rv)>>10&0x1)
#define SET_GCAR1r_GRXPF(rv,fv)         (rv=(rv&~(0x1<<10))|((fv)&0x1)<<10)
#define GET_GCAR1r_GTR1023(rv)          ((rv)>>27&0x1)
#define SET_GCAR1r_GTR1023(rv,fv)       (rv=(rv&~(0x1<<27))|((fv)&0x1)<<27)
#define GET_GCAR1r_GTR127(rv)           ((rv)>>30&0x1)
#define SET_GCAR1r_GTR127(rv,fv)        (rv=(rv&~(0x1<<30))|((fv)&0x1)<<30)
#define GET_GCAR1r_GTR255(rv)           ((rv)>>29&0x1)
#define SET_GCAR1r_GTR255(rv,fv)        (rv=(rv&~(0x1<<29))|((fv)&0x1)<<29)
#define GET_GCAR1r_GTR511(rv)           ((rv)>>28&0x1)
#define SET_GCAR1r_GTR511(rv,fv)        (rv=(rv&~(0x1<<28))|((fv)&0x1)<<28)
#define GET_GCAR1r_GTR64(rv)            ((rv)>>31&0x1)
#define SET_GCAR1r_GTR64(rv,fv)         (rv=(rv&~(0x1<<31))|((fv)&0x1)<<31)
#define GET_GCAR1r_GTRMAX(rv)           ((rv)>>26&0x1)
#define SET_GCAR1r_GTRMAX(rv,fv)        (rv=(rv&~(0x1<<26))|((fv)&0x1)<<26)
#define GET_GCAR1r_GTRMGV(rv)           ((rv)>>25&0x1)
#define SET_GCAR1r_GTRMGV(rv,fv)        (rv=(rv&~(0x1<<25))|((fv)&0x1)<<25)

#define GCAR2r_ADDR(port)               reg_addr(GCAR2r,port)
#define READ_GCAR2r(unit,port,rvp) \
	cmic_read_reg(unit,GCAR2r_ADDR(port),rvp)
#define WRITE_GCAR2r(unit,port,rv) \
	cmic_write_reg(unit,GCAR2r_ADDR(port),rv)
#define GET_GCAR2r_GRDISC(rv)           ((rv)>>25&0x1)
#define SET_GCAR2r_GRDISC(rv,fv)        (rv=(rv&~(0x1<<25))|((fv)&0x1)<<25)
#define GET_GCAR2r_GRIPC(rv)            ((rv)>>20&0x1)
#define SET_GCAR2r_GRIPC(rv,fv)         (rv=(rv&~(0x1<<20))|((fv)&0x1)<<20)
#define GET_GCAR2r_GRIPCHK(rv)          ((rv)>>22&0x1)
#define SET_GCAR2r_GRIPCHK(rv,fv)       (rv=(rv&~(0x1<<22))|((fv)&0x1)<<22)
#define GET_GCAR2r_GRIPD(rv)            ((rv)>>21&0x1)
#define SET_GCAR2r_GRIPD(rv,fv)         (rv=(rv&~(0x1<<21))|((fv)&0x1)<<21)
#define GET_GCAR2r_GRIPX(rv)            ((rv)>>23&0x1)
#define SET_GCAR2r_GRIPX(rv,fv)         (rv=(rv&~(0x1<<23))|((fv)&0x1)<<23)
#define GET_GCAR2r_GRIPXD(rv)           ((rv)>>24&0x1)
#define SET_GCAR2r_GRIPXD(rv,fv)        (rv=(rv&~(0x1<<24))|((fv)&0x1)<<24)
#define GET_GCAR2r_GTABRT(rv)           ((rv)>>31&0x1)
#define SET_GCAR2r_GTABRT(rv,fv)        (rv=(rv&~(0x1<<31))|((fv)&0x1)<<31)
#define GET_GCAR2r_GTAGE(rv)            ((rv)>>26&0x1)
#define SET_GCAR2r_GTAGE(rv,fv)         (rv=(rv&~(0x1<<26))|((fv)&0x1)<<26)
#define GET_GCAR2r_GTBCA(rv)            ((rv)>>10&0x1)
#define SET_GCAR2r_GTBCA(rv,fv)         (rv=(rv&~(0x1<<10))|((fv)&0x1)<<10)
#define GET_GCAR2r_GTBYT(rv)            ((rv)>>13&0x1)
#define SET_GCAR2r_GTBYT(rv,fv)         (rv=(rv&~(0x1<<13))|((fv)&0x1)<<13)
#define GET_GCAR2r_GTDFR(rv)            ((rv)>>8&0x1)
#define SET_GCAR2r_GTDFR(rv,fv)         (rv=(rv&~(0x1<<8))|((fv)&0x1)<<8)
#define GET_GCAR2r_GTFCS(rv)            ((rv)>>18&0x1)
#define SET_GCAR2r_GTFCS(rv,fv)         (rv=(rv&~(0x1<<18))|((fv)&0x1)<<18)
#define GET_GCAR2r_GTIP(rv)             ((rv)>>27&0x1)
#define SET_GCAR2r_GTIP(rv,fv)          (rv=(rv&~(0x1<<27))|((fv)&0x1)<<27)
#define GET_GCAR2r_GTIPD(rv)            ((rv)>>28&0x1)
#define SET_GCAR2r_GTIPD(rv,fv)         (rv=(rv&~(0x1<<28))|((fv)&0x1)<<28)
#define GET_GCAR2r_GTIPX(rv)            ((rv)>>30&0x1)
#define SET_GCAR2r_GTIPX(rv,fv)         (rv=(rv&~(0x1<<30))|((fv)&0x1)<<30)
#define GET_GCAR2r_GTJBR(rv)            ((rv)>>19&0x1)
#define SET_GCAR2r_GTJBR(rv,fv)         (rv=(rv&~(0x1<<19))|((fv)&0x1)<<19)
#define GET_GCAR2r_GTMCA(rv)            ((rv)>>11&0x1)
#define SET_GCAR2r_GTMCA(rv,fv)         (rv=(rv&~(0x1<<11))|((fv)&0x1)<<11)
#define GET_GCAR2r_GTOVR(rv)            ((rv)>>16&0x1)
#define SET_GCAR2r_GTOVR(rv,fv)         (rv=(rv&~(0x1<<16))|((fv)&0x1)<<16)
#define GET_GCAR2r_GTPFH(rv)            ((rv)>>1&0x1)
#define SET_GCAR2r_GTPFH(rv,fv)         (rv=(rv&~(0x1<<1))|((fv)&0x1)<<1)
#define GET_GCAR2r_GTPKT(rv)            ((rv)>>12&0x1)
#define SET_GCAR2r_GTPKT(rv,fv)         (rv=(rv&~(0x1<<12))|((fv)&0x1)<<12)
#define GET_GCAR2r_GTVLAN(rv)           ((rv)>>29&0x1)
#define SET_GCAR2r_GTVLAN(rv,fv)        (rv=(rv&~(0x1<<29))|((fv)&0x1)<<29)
#define GET_GCAR2r_GTXCF(rv)            ((rv)>>17&0x1)
#define SET_GCAR2r_GTXCF(rv,fv)         (rv=(rv&~(0x1<<17))|((fv)&0x1)<<17)
#define GET_GCAR2r_GTXPF(rv)            ((rv)>>9&0x1)
#define SET_GCAR2r_GTXPF(rv,fv)         (rv=(rv&~(0x1<<9))|((fv)&0x1)<<9)

#define GCCCOUNTr_ADDR(port)            reg_addr(GCCCOUNTr,port)
#define READ_GCCCOUNTr(unit,port,rvp) \
	cmic_read_reg(unit,GCCCOUNTr_ADDR(port),rvp)
#define WRITE_GCCCOUNTr(unit,port,rv) \
	cmic_write_reg(unit,GCCCOUNTr_ADDR(port),rv)
#define GET_GCCCOUNTr_GCC(rv)           ((rv)>>0&0xffff)
#define SET_GCCCOUNTr_GCC(rv,fv)        (rv=(rv&~(0xffff<<0))|((fv)&0xffff)<<0)

#define GEGR_ENABLEr_ADDR(pic)          reg_addr(GEGR_ENABLEr,pic)
#define READ_GEGR_ENABLEr(unit,pic,rvp) \
	cmic_read_reg(unit,GEGR_ENABLEr_ADDR(pic),rvp)
#define WRITE_GEGR_ENABLEr(unit,pic,rv) \
	cmic_write_reg(unit,GEGR_ENABLEr_ADDR(pic),rv)
#define GET_GEGR_ENABLEr_PRT_ENABLE(rv) ((rv)>>0&0x1)
#define SET_GEGR_ENABLEr_PRT_ENABLE(rv,fv) (rv=(rv&~(0x1<<0))|((fv)&0x1)<<0)

#define GLOBALRETURNMARKERr_ADDR(port)  reg_addr(GLOBALRETURNMARKERr,port)
#define READ_GLOBALRETURNMARKERr(unit,port,rvp) \
	cmic_read_reg(unit,GLOBALRETURNMARKERr_ADDR(port),rvp)
#define WRITE_GLOBALRETURNMARKERr(unit,port,rv) \
	cmic_write_reg(unit,GLOBALRETURNMARKERr_ADDR(port),rv)
#define GET_GLOBALRETURNMARKERr_GRM(rv) ((rv)>>0&0xf)
#define SET_GLOBALRETURNMARKERr_GRM(rv,fv) (rv=(rv&~(0xf<<0))|((fv)&0xf)<<0)

#define GMACC0r_ADDR(port)              reg_addr(GMACC0r,port)
#define READ_GMACC0r(unit,port,rvp) \
	cmic_read_reg(unit,GMACC0r_ADDR(port),rvp)
#define WRITE_GMACC0r(unit,port,rv) \
	cmic_write_reg(unit,GMACC0r_ADDR(port),rv)
#define GET_GMACC0r_L10B(rv)            ((rv)>>9&0x1)
#define SET_GMACC0r_L10B(rv,fv)         (rv=(rv&~(0x1<<9))|((fv)&0x1)<<9)
#define GET_GMACC0r_L32B(rv)            ((rv)>>8&0x1)
#define SET_GMACC0r_L32B(rv,fv)         (rv=(rv&~(0x1<<8))|((fv)&0x1)<<8)
#define GET_GMACC0r_RANX(rv)            ((rv)>>24&0x1)
#define SET_GMACC0r_RANX(rv,fv)         (rv=(rv&~(0x1<<24))|((fv)&0x1)<<24)
#define GET_GMACC0r_RMGT(rv)            ((rv)>>25&0x1)
#define SET_GMACC0r_RMGT(rv,fv)         (rv=(rv&~(0x1<<25))|((fv)&0x1)<<25)
#define GET_GMACC0r_RREX(rv)            ((rv)>>20&0x1)
#define SET_GMACC0r_RREX(rv,fv)         (rv=(rv&~(0x1<<20))|((fv)&0x1)<<20)
#define GET_GMACC0r_RRFN(rv)            ((rv)>>21&0x1)
#define SET_GMACC0r_RRFN(rv,fv)         (rv=(rv&~(0x1<<21))|((fv)&0x1)<<21)
#define GET_GMACC0r_RRMC(rv)            ((rv)>>22&0x1)
#define SET_GMACC0r_RRMC(rv,fv)         (rv=(rv&~(0x1<<22))|((fv)&0x1)<<22)
#define GET_GMACC0r_RTEX(rv)            ((rv)>>16&0x1)
#define SET_GMACC0r_RTEX(rv,fv)         (rv=(rv&~(0x1<<16))|((fv)&0x1)<<16)
#define GET_GMACC0r_RTFN(rv)            ((rv)>>17&0x1)
#define SET_GMACC0r_RTFN(rv,fv)         (rv=(rv&~(0x1<<17))|((fv)&0x1)<<17)
#define GET_GMACC0r_RTMC(rv)            ((rv)>>18&0x1)
#define SET_GMACC0r_RTMC(rv,fv)         (rv=(rv&~(0x1<<18))|((fv)&0x1)<<18)
#define GET_GMACC0r_SRST(rv)            ((rv)>>31&0x1)
#define SET_GMACC0r_SRST(rv,fv)         (rv=(rv&~(0x1<<31))|((fv)&0x1)<<31)
#define GET_GMACC0r_TMDS(rv)            ((rv)>>0&0xff)
#define SET_GMACC0r_TMDS(rv,fv)         (rv=(rv&~(0xff<<0))|((fv)&0xff)<<0)

#define GMACC1r_ADDR(port)              reg_addr(GMACC1r,port)
#define READ_GMACC1r(unit,port,rvp) \
	cmic_read_reg(unit,GMACC1r_ADDR(port),rvp)
#define WRITE_GMACC1r(unit,port,rv) \
	cmic_write_reg(unit,GMACC1r_ADDR(port),rv)
#define GET_GMACC1r_ADPAD(rv)           ((rv)>>3&0x1)
#define SET_GMACC1r_ADPAD(rv,fv)        (rv=(rv&~(0x1<<3))|((fv)&0x1)<<3)
#define GET_GMACC1r_AUTOZ(rv)           ((rv)>>31&0x1)
#define SET_GMACC1r_AUTOZ(rv,fv)        (rv=(rv&~(0x1<<31))|((fv)&0x1)<<31)
#define GET_GMACC1r_CLRCNT(rv)          ((rv)>>27&0x1)
#define SET_GMACC1r_CLRCNT(rv,fv)       (rv=(rv&~(0x1<<27))|((fv)&0x1)<<27)
#define GET_GMACC1r_CRCEN(rv)           ((rv)>>4&0x1)
#define SET_GMACC1r_CRCEN(rv,fv)        (rv=(rv&~(0x1<<4))|((fv)&0x1)<<4)
#define GET_GMACC1r_DCRC12(rv)          ((rv)>>22&0x3)
#define SET_GMACC1r_DCRC12(rv,fv)       (rv=(rv&~(0x3<<22))|((fv)&0x3)<<22)
#define GET_GMACC1r_FCRX(rv)            ((rv)>>10&0x1)
#define SET_GMACC1r_FCRX(rv,fv)         (rv=(rv&~(0x1<<10))|((fv)&0x1)<<10)
#define GET_GMACC1r_FCTX(rv)            ((rv)>>9&0x1)
#define SET_GMACC1r_FCTX(rv,fv)         (rv=(rv&~(0x1<<9))|((fv)&0x1)<<9)
#define GET_GMACC1r_FLCHK(rv)           ((rv)>>5&0x1)
#define SET_GMACC1r_FLCHK(rv,fv)        (rv=(rv&~(0x1<<5))|((fv)&0x1)<<5)
#define GET_GMACC1r_FULLD(rv)           ((rv)>>0&0x1)
#define SET_GMACC1r_FULLD(rv,fv)        (rv=(rv&~(0x1<<0))|((fv)&0x1)<<0)
#define GET_GMACC1r_GLVR(rv)            ((rv)>>19&0x1)
#define SET_GMACC1r_GLVR(rv,fv)         (rv=(rv&~(0x1<<19))|((fv)&0x1)<<19)
#define GET_GMACC1r_HUGEN(rv)           ((rv)>>6&0x1)
#define SET_GMACC1r_HUGEN(rv,fv)        (rv=(rv&~(0x1<<6))|((fv)&0x1)<<6)
#define GET_GMACC1r_JUMBO(rv)           ((rv)>>7&0x1)
#define SET_GMACC1r_JUMBO(rv,fv)        (rv=(rv&~(0x1<<7))|((fv)&0x1)<<7)
#define GET_GMACC1r_LONGP(rv)           ((rv)>>12&0x1)
#define SET_GMACC1r_LONGP(rv,fv)        (rv=(rv&~(0x1<<12))|((fv)&0x1)<<12)
#define GET_GMACC1r_MIFG(rv)            ((rv)>>16&0x7)
#define SET_GMACC1r_MIFG(rv,fv)         (rv=(rv&~(0x7<<16))|((fv)&0x7)<<16)
#define GET_GMACC1r_PADEN(rv)           ((rv)>>1&0x1)
#define SET_GMACC1r_PADEN(rv,fv)        (rv=(rv&~(0x1<<1))|((fv)&0x1)<<1)
#define GET_GMACC1r_PARF(rv)            ((rv)>>11&0x1)
#define SET_GMACC1r_PARF(rv,fv)         (rv=(rv&~(0x1<<11))|((fv)&0x1)<<11)
#define GET_GMACC1r_PUREP(rv)           ((rv)>>8&0x1)
#define SET_GMACC1r_PUREP(rv,fv)        (rv=(rv&~(0x1<<8))|((fv)&0x1)<<8)
#define GET_GMACC1r_RXEN0(rv)           ((rv)>>28&0x1)
#define SET_GMACC1r_RXEN0(rv,fv)        (rv=(rv&~(0x1<<28))|((fv)&0x1)<<28)
#define GET_GMACC1r_TXEN0(rv)           ((rv)>>30&0x1)
#define SET_GMACC1r_TXEN0(rv,fv)        (rv=(rv&~(0x1<<30))|((fv)&0x1)<<30)
#define GET_GMACC1r_VLPAD(rv)           ((rv)>>2&0x1)
#define SET_GMACC1r_VLPAD(rv,fv)        (rv=(rv&~(0x1<<2))|((fv)&0x1)<<2)

#define GMACC2r_ADDR(port)              reg_addr(GMACC2r,port)
#define READ_GMACC2r(unit,port,rvp) \
	cmic_read_reg(unit,GMACC2r_ADDR(port),rvp)
#define WRITE_GMACC2r(unit,port,rv) \
	cmic_write_reg(unit,GMACC2r_ADDR(port),rv)
#define GET_GMACC2r_IPGT(rv)            ((rv)>>0&0x3f)
#define SET_GMACC2r_IPGT(rv,fv)         (rv=(rv&~(0x3f<<0))|((fv)&0x3f)<<0)
#define GET_GMACC2r_MCLKFQ(rv)          ((rv)>>16&0x7)
#define SET_GMACC2r_MCLKFQ(rv,fv)       (rv=(rv&~(0x7<<16))|((fv)&0x7)<<16)

#define GPCSCr_ADDR(port)               reg_addr(GPCSCr,port)
#define READ_GPCSCr(unit,port,rvp) \
	cmic_read_reg(unit,GPCSCr_ADDR(port),rvp)
#define WRITE_GPCSCr(unit,port,rv) \
	cmic_write_reg(unit,GPCSCr_ADDR(port),rv)
#define GET_GPCSCr_AUTOS(rv)            ((rv)>>8&0x1)
#define SET_GPCSCr_AUTOS(rv,fv)         (rv=(rv&~(0x1<<8))|((fv)&0x1)<<8)
#define GET_GPCSCr_DRRD(rv)             ((rv)>>13&0x1)
#define SET_GPCSCr_DRRD(rv,fv)          (rv=(rv&~(0x1<<13))|((fv)&0x1)<<13)
#define GET_GPCSCr_DTRD(rv)             ((rv)>>12&0x1)
#define SET_GPCSCr_DTRD(rv,fv)          (rv=(rv&~(0x1<<12))|((fv)&0x1)<<12)
#define GET_GPCSCr_ECMDT(rv)            ((rv)>>0&0x1)
#define SET_GPCSCr_ECMDT(rv,fv)         (rv=(rv&~(0x1<<0))|((fv)&0x1)<<0)
#define GET_GPCSCr_EWRAP(rv)            ((rv)>>1&0x1)
#define SET_GPCSCr_EWRAP(rv,fv)         (rv=(rv&~(0x1<<1))|((fv)&0x1)<<1)
#define GET_GPCSCr_JTRPS(rv)            ((rv)>>28&0x7)
#define SET_GPCSCr_JTRPS(rv,fv)         (rv=(rv&~(0x7<<28))|((fv)&0x7)<<28)
#define GET_GPCSCr_JTRPT(rv)            ((rv)>>16&0x3ff)
#define SET_GPCSCr_JTRPT(rv,fv)         (rv=(rv&~(0x3ff<<16))|((fv)&0x3ff)<<16)
#define GET_GPCSCr_RCSEL(rv)            ((rv)>>2&0x1)
#define SET_GPCSCr_RCSEL(rv,fv)         (rv=(rv&~(0x1<<2))|((fv)&0x1)<<2)
#define GET_GPCSCr_SCLT(rv)             ((rv)>>14&0x1)
#define SET_GPCSCr_SCLT(rv,fv)          (rv=(rv&~(0x1<<14))|((fv)&0x1)<<14)

#define GPC_EGR_DBGr_ADDR(pic)          reg_addr(GPC_EGR_DBGr,pic)
#define READ_GPC_EGR_DBGr(unit,pic,rvp) \
	cmic_read_reg(unit,GPC_EGR_DBGr_ADDR(pic),rvp)
#define WRITE_GPC_EGR_DBGr(unit,pic,rv) \
	cmic_write_reg(unit,GPC_EGR_DBGr_ADDR(pic),rv)
#define GET_GPC_EGR_DBGr_DEBUG_ON(rv)   ((rv)>>0&0x1)
#define SET_GPC_EGR_DBGr_DEBUG_ON(rv,fv) (rv=(rv&~(0x1<<0))|((fv)&0x1)<<0)

#define GPC_EGR_SNGL_OUTr_ADDR(pic)     reg_addr(GPC_EGR_SNGL_OUTr,pic)
#define READ_GPC_EGR_SNGL_OUTr(unit,pic,rvp) \
	cmic_read_reg(unit,GPC_EGR_SNGL_OUTr_ADDR(pic),rvp)
#define WRITE_GPC_EGR_SNGL_OUTr(unit,pic,rv) \
	cmic_write_reg(unit,GPC_EGR_SNGL_OUTr_ADDR(pic),rv)
#define GET_GPC_EGR_SNGL_OUTr_PRT_ENABLE(rv) ((rv)>>0&0x1)
#define SET_GPC_EGR_SNGL_OUTr_PRT_ENABLE(rv,fv) (rv=(rv&~(0x1<<0))|((fv)&0x1)<<0)

#define GPC_EGR_SNGL_PKTr_ADDR(pic)     reg_addr(GPC_EGR_SNGL_PKTr,pic)
#define READ_GPC_EGR_SNGL_PKTr(unit,pic,rvp) \
	cmic_read_reg(unit,GPC_EGR_SNGL_PKTr_ADDR(pic),rvp)
#define WRITE_GPC_EGR_SNGL_PKTr(unit,pic,rv) \
	cmic_write_reg(unit,GPC_EGR_SNGL_PKTr_ADDR(pic),rv)
#define GET_GPC_EGR_SNGL_PKTr_PRT_ENABLE(rv) ((rv)>>0&0x1)
#define SET_GPC_EGR_SNGL_PKTr_PRT_ENABLE(rv,fv) (rv=(rv&~(0x1<<0))|((fv)&0x1)<<0)

#define GRALNr_ADDR(port)               reg_addr(GRALNr,port)
#define READ_GRALNr(unit,port,rvp) \
	cmic_read_reg(unit,GRALNr_ADDR(port),rvp)
#define WRITE_GRALNr(unit,port,rv) \
	cmic_write_reg(unit,GRALNr_ADDR(port),rv)
#define GET_GRALNr_COUNT(rv)            ((rv)>>0&0xffff)
#define SET_GRALNr_COUNT(rv,fv)         (rv=(rv&~(0xffff<<0))|((fv)&0xffff)<<0)

#define GRBCAr_ADDR(port)               reg_addr(GRBCAr,port)
#define READ_GRBCAr(unit,port,rvp) \
	cmic_read_reg(unit,GRBCAr_ADDR(port),rvp)
#define WRITE_GRBCAr(unit,port,rv) \
	cmic_write_reg(unit,GRBCAr_ADDR(port),rv)
#define GET_GRBCAr_COUNT(rv)            ((rv)>>0&0xffffff)
#define SET_GRBCAr_COUNT(rv,fv)         (rv=(rv&~(0xffffff<<0))|((fv)&0xffffff)<<0)

#define GRBYTr_ADDR(port)               reg_addr(GRBYTr,port)
#define READ_GRBYTr(unit,port,rvp) \
	cmic_read_reg(unit,GRBYTr_ADDR(port),rvp)
#define WRITE_GRBYTr(unit,port,rv) \
	cmic_write_reg(unit,GRBYTr_ADDR(port),rv)
#define GET_GRBYTr_COUNT(rv)            ((rv)>>0&0xffffffff)
#define SET_GRBYTr_COUNT(rv,fv)         (rv=(rv&~(0xffffffff<<0))|((fv)&0xffffffff)<<0)

#define GRCDEr_ADDR(port)               reg_addr(GRCDEr,port)
#define READ_GRCDEr(unit,port,rvp) \
	cmic_read_reg(unit,GRCDEr_ADDR(port),rvp)
#define WRITE_GRCDEr(unit,port,rv) \
	cmic_write_reg(unit,GRCDEr_ADDR(port),rv)
#define GET_GRCDEr_COUNT(rv)            ((rv)>>0&0xffff)
#define SET_GRCDEr_COUNT(rv,fv)         (rv=(rv&~(0xffff<<0))|((fv)&0xffff)<<0)

#define GRDISCr_ADDR(port)              reg_addr(GRDISCr,port)
#define READ_GRDISCr(unit,port,rvp) \
	cmic_read_reg(unit,GRDISCr_ADDR(port),rvp)
#define WRITE_GRDISCr(unit,port,rv) \
	cmic_write_reg(unit,GRDISCr_ADDR(port),rv)
#define GET_GRDISCr_COUNT(rv)           ((rv)>>0&0xffff)
#define SET_GRDISCr_COUNT(rv,fv)        (rv=(rv&~(0xffff<<0))|((fv)&0xffff)<<0)

#define GRFCRr_ADDR(port)               reg_addr(GRFCRr,port)
#define READ_GRFCRr(unit,port,rvp) \
	cmic_read_reg(unit,GRFCRr_ADDR(port),rvp)
#define WRITE_GRFCRr(unit,port,rv) \
	cmic_write_reg(unit,GRFCRr_ADDR(port),rv)
#define GET_GRFCRr_COUNT(rv)            ((rv)>>0&0xffff)
#define SET_GRFCRr_COUNT(rv,fv)         (rv=(rv&~(0xffff<<0))|((fv)&0xffff)<<0)

#define GRFCSr_ADDR(port)               reg_addr(GRFCSr,port)
#define READ_GRFCSr(unit,port,rvp) \
	cmic_read_reg(unit,GRFCSr_ADDR(port),rvp)
#define WRITE_GRFCSr(unit,port,rv) \
	cmic_write_reg(unit,GRFCSr_ADDR(port),rv)
#define GET_GRFCSr_COUNT(rv)            ((rv)>>0&0xffff)
#define SET_GRFCSr_COUNT(rv,fv)         (rv=(rv&~(0xffff<<0))|((fv)&0xffff)<<0)

#define GRFLRr_ADDR(port)               reg_addr(GRFLRr,port)
#define READ_GRFLRr(unit,port,rvp) \
	cmic_read_reg(unit,GRFLRr_ADDR(port),rvp)
#define WRITE_GRFLRr(unit,port,rv) \
	cmic_write_reg(unit,GRFLRr_ADDR(port),rv)
#define GET_GRFLRr_COUNT(rv)            ((rv)>>0&0xffff)
#define SET_GRFLRr_COUNT(rv,fv)         (rv=(rv&~(0xffff<<0))|((fv)&0xffff)<<0)

#define GRIPCr_ADDR(port)               reg_addr(GRIPCr,port)
#define READ_GRIPCr(unit,port,rvp) \
	cmic_read_reg(unit,GRIPCr_ADDR(port),rvp)
#define WRITE_GRIPCr(unit,port,rv) \
	cmic_write_reg(unit,GRIPCr_ADDR(port),rv)
#define GET_GRIPCr_COUNT(rv)            ((rv)>>0&0xffffff)
#define SET_GRIPCr_COUNT(rv,fv)         (rv=(rv&~(0xffffff<<0))|((fv)&0xffffff)<<0)

#define GRIPCHKr_ADDR(port)             reg_addr(GRIPCHKr,port)
#define READ_GRIPCHKr(unit,port,rvp) \
	cmic_read_reg(unit,GRIPCHKr_ADDR(port),rvp)
#define WRITE_GRIPCHKr(unit,port,rv) \
	cmic_write_reg(unit,GRIPCHKr_ADDR(port),rv)
#define GET_GRIPCHKr_COUNT(rv)          ((rv)>>0&0xffff)
#define SET_GRIPCHKr_COUNT(rv,fv)       (rv=(rv&~(0xffff<<0))|((fv)&0xffff)<<0)

#define GRIPDr_ADDR(port)               reg_addr(GRIPDr,port)
#define READ_GRIPDr(unit,port,rvp) \
	cmic_read_reg(unit,GRIPDr_ADDR(port),rvp)
#define WRITE_GRIPDr(unit,port,rv) \
	cmic_write_reg(unit,GRIPDr_ADDR(port),rv)
#define GET_GRIPDr_COUNT(rv)            ((rv)>>0&0xffff)
#define SET_GRIPDr_COUNT(rv,fv)         (rv=(rv&~(0xffff<<0))|((fv)&0xffff)<<0)

#define GRIPXr_ADDR(port)               reg_addr(GRIPXr,port)
#define READ_GRIPXr(unit,port,rvp) \
	cmic_read_reg(unit,GRIPXr_ADDR(port),rvp)
#define WRITE_GRIPXr(unit,port,rv) \
	cmic_write_reg(unit,GRIPXr_ADDR(port),rv)
#define GET_GRIPXr_COUNT(rv)            ((rv)>>0&0x1)
#define SET_GRIPXr_COUNT(rv,fv)         (rv=(rv&~(0x1<<0))|((fv)&0x1)<<0)

#define GRIPXDr_ADDR(port)              reg_addr(GRIPXDr,port)
#define READ_GRIPXDr(unit,port,rvp) \
	cmic_read_reg(unit,GRIPXDr_ADDR(port),rvp)
#define WRITE_GRIPXDr(unit,port,rv) \
	cmic_write_reg(unit,GRIPXDr_ADDR(port),rv)
#define GET_GRIPXDr_COUNT(rv)           ((rv)>>0&0x1)
#define SET_GRIPXDr_COUNT(rv,fv)        (rv=(rv&~(0x1<<0))|((fv)&0x1)<<0)

#define GRJBRr_ADDR(port)               reg_addr(GRJBRr,port)
#define READ_GRJBRr(unit,port,rvp) \
	cmic_read_reg(unit,GRJBRr_ADDR(port),rvp)
#define WRITE_GRJBRr(unit,port,rv) \
	cmic_write_reg(unit,GRJBRr_ADDR(port),rv)
#define GET_GRJBRr_COUNT(rv)            ((rv)>>0&0xffff)
#define SET_GRJBRr_COUNT(rv,fv)         (rv=(rv&~(0xffff<<0))|((fv)&0xffff)<<0)

#define GRMCAr_ADDR(port)               reg_addr(GRMCAr,port)
#define READ_GRMCAr(unit,port,rvp) \
	cmic_read_reg(unit,GRMCAr_ADDR(port),rvp)
#define WRITE_GRMCAr(unit,port,rv) \
	cmic_write_reg(unit,GRMCAr_ADDR(port),rv)
#define GET_GRMCAr_COUNT(rv)            ((rv)>>0&0xffffff)
#define SET_GRMCAr_COUNT(rv,fv)         (rv=(rv&~(0xffffff<<0))|((fv)&0xffffff)<<0)

#define GROVRr_ADDR(port)               reg_addr(GROVRr,port)
#define READ_GROVRr(unit,port,rvp) \
	cmic_read_reg(unit,GROVRr_ADDR(port),rvp)
#define WRITE_GROVRr(unit,port,rv) \
	cmic_write_reg(unit,GROVRr_ADDR(port),rv)
#define GET_GROVRr_COUNT(rv)            ((rv)>>0&0xffff)
#define SET_GROVRr_COUNT(rv,fv)         (rv=(rv&~(0xffff<<0))|((fv)&0xffff)<<0)

#define GRPKTr_ADDR(port)               reg_addr(GRPKTr,port)
#define READ_GRPKTr(unit,port,rvp) \
	cmic_read_reg(unit,GRPKTr_ADDR(port),rvp)
#define WRITE_GRPKTr(unit,port,rv) \
	cmic_write_reg(unit,GRPKTr_ADDR(port),rv)
#define GET_GRPKTr_COUNT(rv)            ((rv)>>0&0xffffff)
#define SET_GRPKTr_COUNT(rv,fv)         (rv=(rv&~(0xffffff<<0))|((fv)&0xffffff)<<0)

#define GRUNDr_ADDR(port)               reg_addr(GRUNDr,port)
#define READ_GRUNDr(unit,port,rvp) \
	cmic_read_reg(unit,GRUNDr_ADDR(port),rvp)
#define WRITE_GRUNDr(unit,port,rv) \
	cmic_write_reg(unit,GRUNDr_ADDR(port),rv)
#define GET_GRUNDr_COUNT(rv)            ((rv)>>0&0xffff)
#define SET_GRUNDr_COUNT(rv,fv)         (rv=(rv&~(0xffff<<0))|((fv)&0xffff)<<0)

#define GRXCFr_ADDR(port)               reg_addr(GRXCFr,port)
#define READ_GRXCFr(unit,port,rvp) \
	cmic_read_reg(unit,GRXCFr_ADDR(port),rvp)
#define WRITE_GRXCFr(unit,port,rv) \
	cmic_write_reg(unit,GRXCFr_ADDR(port),rv)
#define GET_GRXCFr_COUNT(rv)            ((rv)>>0&0xffff)
#define SET_GRXCFr_COUNT(rv,fv)         (rv=(rv&~(0xffff<<0))|((fv)&0xffff)<<0)

#define GRXPFr_ADDR(port)               reg_addr(GRXPFr,port)
#define READ_GRXPFr(unit,port,rvp) \
	cmic_read_reg(unit,GRXPFr_ADDR(port),rvp)
#define WRITE_GRXPFr(unit,port,rv) \
	cmic_write_reg(unit,GRXPFr_ADDR(port),rv)
#define GET_GRXPFr_COUNT(rv)            ((rv)>>0&0xffff)
#define SET_GRXPFr_COUNT(rv,fv)         (rv=(rv&~(0xffff<<0))|((fv)&0xffff)<<0)

#define GRXUOr_ADDR(port)               reg_addr(GRXUOr,port)
#define READ_GRXUOr(unit,port,rvp) \
	cmic_read_reg(unit,GRXUOr_ADDR(port),rvp)
#define WRITE_GRXUOr(unit,port,rv) \
	cmic_write_reg(unit,GRXUOr_ADDR(port),rv)
#define GET_GRXUOr_COUNT(rv)            ((rv)>>0&0xffff)
#define SET_GRXUOr_COUNT(rv,fv)         (rv=(rv&~(0xffff<<0))|((fv)&0xffff)<<0)

#define GSA0r_ADDR(port)                reg_addr(GSA0r,port)
#define READ_GSA0r(unit,port,rvp) \
	cmic_read_reg(unit,GSA0r_ADDR(port),rvp)
#define WRITE_GSA0r(unit,port,rv) \
	cmic_write_reg(unit,GSA0r_ADDR(port),rv)
#define GET_GSA0r_STAD1(rv)             ((rv)>>0&0xffffffff)
#define SET_GSA0r_STAD1(rv,fv)          (rv=(rv&~(0xffffffff<<0))|((fv)&0xffffffff)<<0)

#define GSA1r_ADDR(port)                reg_addr(GSA1r,port)
#define READ_GSA1r(unit,port,rvp) \
	cmic_read_reg(unit,GSA1r_ADDR(port),rvp)
#define WRITE_GSA1r(unit,port,rv) \
	cmic_write_reg(unit,GSA1r_ADDR(port),rv)
#define GET_GSA1r_STAD2(rv)             ((rv)>>16&0xffff)
#define SET_GSA1r_STAD2(rv,fv)          (rv=(rv&~(0xffff<<16))|((fv)&0xffff)<<16)

#define GTABRTr_ADDR(port)              reg_addr(GTABRTr,port)
#define READ_GTABRTr(unit,port,rvp) \
	cmic_read_reg(unit,GTABRTr_ADDR(port),rvp)
#define WRITE_GTABRTr(unit,port,rv) \
	cmic_write_reg(unit,GTABRTr_ADDR(port),rv)
#define GET_GTABRTr_COUNT(rv)           ((rv)>>0&0xffff)
#define SET_GTABRTr_COUNT(rv,fv)        (rv=(rv&~(0xffff<<0))|((fv)&0xffff)<<0)

#define GTAGEr_ADDR(port)               reg_addr(GTAGEr,port)
#define READ_GTAGEr(unit,port,rvp) \
	cmic_read_reg(unit,GTAGEr_ADDR(port),rvp)
#define WRITE_GTAGEr(unit,port,rv) \
	cmic_write_reg(unit,GTAGEr_ADDR(port),rv)
#define GET_GTAGEr_COUNT(rv)            ((rv)>>0&0xffff)
#define SET_GTAGEr_COUNT(rv,fv)         (rv=(rv&~(0xffff<<0))|((fv)&0xffff)<<0)

#define GTBCAr_ADDR(port)               reg_addr(GTBCAr,port)
#define READ_GTBCAr(unit,port,rvp) \
	cmic_read_reg(unit,GTBCAr_ADDR(port),rvp)
#define WRITE_GTBCAr(unit,port,rv) \
	cmic_write_reg(unit,GTBCAr_ADDR(port),rv)
#define GET_GTBCAr_COUNT(rv)            ((rv)>>0&0xffffff)
#define SET_GTBCAr_COUNT(rv,fv)         (rv=(rv&~(0xffffff<<0))|((fv)&0xffffff)<<0)

#define GTBYTr_ADDR(port)               reg_addr(GTBYTr,port)
#define READ_GTBYTr(unit,port,rvp) \
	cmic_read_reg(unit,GTBYTr_ADDR(port),rvp)
#define WRITE_GTBYTr(unit,port,rv) \
	cmic_write_reg(unit,GTBYTr_ADDR(port),rv)
#define GET_GTBYTr_COUNT(rv)            ((rv)>>0&0xffffffff)
#define SET_GTBYTr_COUNT(rv,fv)         (rv=(rv&~(0xffffffff<<0))|((fv)&0xffffffff)<<0)

#define GTFCSr_ADDR(port)               reg_addr(GTFCSr,port)
#define READ_GTFCSr(unit,port,rvp) \
	cmic_read_reg(unit,GTFCSr_ADDR(port),rvp)
#define WRITE_GTFCSr(unit,port,rv) \
	cmic_write_reg(unit,GTFCSr_ADDR(port),rv)
#define GET_GTFCSr_COUNT(rv)            ((rv)>>0&0xffff)
#define SET_GTFCSr_COUNT(rv,fv)         (rv=(rv&~(0xffff<<0))|((fv)&0xffff)<<0)

#define GTIPr_ADDR(port)                reg_addr(GTIPr,port)
#define READ_GTIPr(unit,port,rvp) \
	cmic_read_reg(unit,GTIPr_ADDR(port),rvp)
#define WRITE_GTIPr(unit,port,rv) \
	cmic_write_reg(unit,GTIPr_ADDR(port),rv)
#define GET_GTIPr_COUNT(rv)             ((rv)>>0&0xffffff)
#define SET_GTIPr_COUNT(rv,fv)          (rv=(rv&~(0xffffff<<0))|((fv)&0xffffff)<<0)

#define GTIPDr_ADDR(port)               reg_addr(GTIPDr,port)
#define READ_GTIPDr(unit,port,rvp) \
	cmic_read_reg(unit,GTIPDr_ADDR(port),rvp)
#define WRITE_GTIPDr(unit,port,rv) \
	cmic_write_reg(unit,GTIPDr_ADDR(port),rv)
#define GET_GTIPDr_COUNT(rv)            ((rv)>>0&0xffff)
#define SET_GTIPDr_COUNT(rv,fv)         (rv=(rv&~(0xffff<<0))|((fv)&0xffff)<<0)

#define GTIPXr_ADDR(port)               reg_addr(GTIPXr,port)
#define READ_GTIPXr(unit,port,rvp) \
	cmic_read_reg(unit,GTIPXr_ADDR(port),rvp)
#define WRITE_GTIPXr(unit,port,rv) \
	cmic_write_reg(unit,GTIPXr_ADDR(port),rv)
#define GET_GTIPXr_COUNT(rv)            ((rv)>>0&0x1)
#define SET_GTIPXr_COUNT(rv,fv)         (rv=(rv&~(0x1<<0))|((fv)&0x1)<<0)

#define GTJBRr_ADDR(port)               reg_addr(GTJBRr,port)
#define READ_GTJBRr(unit,port,rvp) \
	cmic_read_reg(unit,GTJBRr_ADDR(port),rvp)
#define WRITE_GTJBRr(unit,port,rv) \
	cmic_write_reg(unit,GTJBRr_ADDR(port),rv)
#define GET_GTJBRr_COUNT(rv)            ((rv)>>0&0xffff)
#define SET_GTJBRr_COUNT(rv,fv)         (rv=(rv&~(0xffff<<0))|((fv)&0xffff)<<0)

#define GTMCAr_ADDR(port)               reg_addr(GTMCAr,port)
#define READ_GTMCAr(unit,port,rvp) \
	cmic_read_reg(unit,GTMCAr_ADDR(port),rvp)
#define WRITE_GTMCAr(unit,port,rv) \
	cmic_write_reg(unit,GTMCAr_ADDR(port),rv)
#define GET_GTMCAr_COUNT(rv)            ((rv)>>0&0xffffff)
#define SET_GTMCAr_COUNT(rv,fv)         (rv=(rv&~(0xffffff<<0))|((fv)&0xffffff)<<0)

#define GTOVRr_ADDR(port)               reg_addr(GTOVRr,port)
#define READ_GTOVRr(unit,port,rvp) \
	cmic_read_reg(unit,GTOVRr_ADDR(port),rvp)
#define WRITE_GTOVRr(unit,port,rv) \
	cmic_write_reg(unit,GTOVRr_ADDR(port),rv)
#define GET_GTOVRr_COUNT(rv)            ((rv)>>0&0xffff)
#define SET_GTOVRr_COUNT(rv,fv)         (rv=(rv&~(0xffff<<0))|((fv)&0xffff)<<0)

#define GTPKTr_ADDR(port)               reg_addr(GTPKTr,port)
#define READ_GTPKTr(unit,port,rvp) \
	cmic_read_reg(unit,GTPKTr_ADDR(port),rvp)
#define WRITE_GTPKTr(unit,port,rv) \
	cmic_write_reg(unit,GTPKTr_ADDR(port),rv)
#define GET_GTPKTr_COUNT(rv)            ((rv)>>0&0xffffff)
#define SET_GTPKTr_COUNT(rv,fv)         (rv=(rv&~(0xffffff<<0))|((fv)&0xffffff)<<0)

#define GTR1023r_ADDR(port)             reg_addr(GTR1023r,port)
#define READ_GTR1023r(unit,port,rvp) \
	cmic_read_reg(unit,GTR1023r_ADDR(port),rvp)
#define WRITE_GTR1023r(unit,port,rv) \
	cmic_write_reg(unit,GTR1023r_ADDR(port),rv)
#define GET_GTR1023r_COUNT(rv)          ((rv)>>0&0xffffff)
#define SET_GTR1023r_COUNT(rv,fv)       (rv=(rv&~(0xffffff<<0))|((fv)&0xffffff)<<0)

#define GTR127r_ADDR(port)              reg_addr(GTR127r,port)
#define READ_GTR127r(unit,port,rvp) \
	cmic_read_reg(unit,GTR127r_ADDR(port),rvp)
#define WRITE_GTR127r(unit,port,rv) \
	cmic_write_reg(unit,GTR127r_ADDR(port),rv)
#define GET_GTR127r_COUNT(rv)           ((rv)>>0&0xffffff)
#define SET_GTR127r_COUNT(rv,fv)        (rv=(rv&~(0xffffff<<0))|((fv)&0xffffff)<<0)

#define GTR255r_ADDR(port)              reg_addr(GTR255r,port)
#define READ_GTR255r(unit,port,rvp) \
	cmic_read_reg(unit,GTR255r_ADDR(port),rvp)
#define WRITE_GTR255r(unit,port,rv) \
	cmic_write_reg(unit,GTR255r_ADDR(port),rv)
#define GET_GTR255r_COUNT(rv)           ((rv)>>0&0xffffff)
#define SET_GTR255r_COUNT(rv,fv)        (rv=(rv&~(0xffffff<<0))|((fv)&0xffffff)<<0)

#define GTR511r_ADDR(port)              reg_addr(GTR511r,port)
#define READ_GTR511r(unit,port,rvp) \
	cmic_read_reg(unit,GTR511r_ADDR(port),rvp)
#define WRITE_GTR511r(unit,port,rv) \
	cmic_write_reg(unit,GTR511r_ADDR(port),rv)
#define GET_GTR511r_COUNT(rv)           ((rv)>>0&0xffffff)
#define SET_GTR511r_COUNT(rv,fv)        (rv=(rv&~(0xffffff<<0))|((fv)&0xffffff)<<0)

#define GTR64r_ADDR(port)               reg_addr(GTR64r,port)
#define READ_GTR64r(unit,port,rvp) \
	cmic_read_reg(unit,GTR64r_ADDR(port),rvp)
#define WRITE_GTR64r(unit,port,rv) \
	cmic_write_reg(unit,GTR64r_ADDR(port),rv)
#define GET_GTR64r_COUNT(rv)            ((rv)>>0&0xffffff)
#define SET_GTR64r_COUNT(rv,fv)         (rv=(rv&~(0xffffff<<0))|((fv)&0xffffff)<<0)

#define GTRMAXr_ADDR(port)              reg_addr(GTRMAXr,port)
#define READ_GTRMAXr(unit,port,rvp) \
	cmic_read_reg(unit,GTRMAXr_ADDR(port),rvp)
#define WRITE_GTRMAXr(unit,port,rv) \
	cmic_write_reg(unit,GTRMAXr_ADDR(port),rv)
#define GET_GTRMAXr_COUNT(rv)           ((rv)>>0&0xffffff)
#define SET_GTRMAXr_COUNT(rv,fv)        (rv=(rv&~(0xffffff<<0))|((fv)&0xffffff)<<0)

#define GTRMGVr_ADDR(port)              reg_addr(GTRMGVr,port)
#define READ_GTRMGVr(unit,port,rvp) \
	cmic_read_reg(unit,GTRMGVr_ADDR(port),rvp)
#define WRITE_GTRMGVr(unit,port,rv) \
	cmic_write_reg(unit,GTRMGVr_ADDR(port),rv)
#define GET_GTRMGVr_COUNT(rv)           ((rv)>>0&0xffffff)
#define SET_GTRMGVr_COUNT(rv,fv)        (rv=(rv&~(0xffffff<<0))|((fv)&0xffffff)<<0)

#define GTVLANr_ADDR(port)              reg_addr(GTVLANr,port)
#define READ_GTVLANr(unit,port,rvp) \
	cmic_read_reg(unit,GTVLANr_ADDR(port),rvp)
#define WRITE_GTVLANr(unit,port,rv) \
	cmic_write_reg(unit,GTVLANr_ADDR(port),rv)
#define GET_GTVLANr_COUNT(rv)           ((rv)>>0&0xffffff)
#define SET_GTVLANr_COUNT(rv,fv)        (rv=(rv&~(0xffffff<<0))|((fv)&0xffffff)<<0)

#define GTXCFr_ADDR(port)               reg_addr(GTXCFr,port)
#define READ_GTXCFr(unit,port,rvp) \
	cmic_read_reg(unit,GTXCFr_ADDR(port),rvp)
#define WRITE_GTXCFr(unit,port,rv) \
	cmic_write_reg(unit,GTXCFr_ADDR(port),rv)
#define GET_GTXCFr_COUNT(rv)            ((rv)>>0&0xffff)
#define SET_GTXCFr_COUNT(rv,fv)         (rv=(rv&~(0xffff<<0))|((fv)&0xffff)<<0)

#define GTXPFr_ADDR(port)               reg_addr(GTXPFr,port)
#define READ_GTXPFr(unit,port,rvp) \
	cmic_read_reg(unit,GTXPFr_ADDR(port),rvp)
#define WRITE_GTXPFr(unit,port,rv) \
	cmic_write_reg(unit,GTXPFr_ADDR(port),rv)
#define GET_GTXPFr_COUNT(rv)            ((rv)>>0&0xffff)
#define SET_GTXPFr_COUNT(rv,fv)         (rv=(rv&~(0xffff<<0))|((fv)&0xffff)<<0)

#define HIGHWATERMARKERr_ADDR(port)     reg_addr(HIGHWATERMARKERr,port)
#define READ_HIGHWATERMARKERr(unit,port,rvp) \
	cmic_read_reg(unit,HIGHWATERMARKERr_ADDR(port),rvp)
#define WRITE_HIGHWATERMARKERr(unit,port,rv) \
	cmic_write_reg(unit,HIGHWATERMARKERr_ADDR(port),rv)
#define GET_HIGHWATERMARKERr_HWM(rv)    ((rv)>>0&0xfff)
#define SET_HIGHWATERMARKERr_HWM(rv,fv) (rv=(rv&~(0xfff<<0))|((fv)&0xfff)<<0)

#define HOLBLOCKINGMARKERr_ADDR(port)   reg_addr(HOLBLOCKINGMARKERr,port)
#define READ_HOLBLOCKINGMARKERr(unit,port,rvp) \
	cmic_read_reg(unit,HOLBLOCKINGMARKERr_ADDR(port),rvp)
#define WRITE_HOLBLOCKINGMARKERr(unit,port,rv) \
	cmic_write_reg(unit,HOLBLOCKINGMARKERr_ADDR(port),rv)
#define GET_HOLBLOCKINGMARKERr_HOL(rv)  ((rv)>>0&0xffff)
#define SET_HOLBLOCKINGMARKERr_HOL(rv,fv) (rv=(rv&~(0xffff<<0))|((fv)&0xffff)<<0)

#define IBPCOUNTr_ADDR(port)            reg_addr(IBPCOUNTr,port)
#define READ_IBPCOUNTr(unit,port,rvp) \
	cmic_read_reg(unit,IBPCOUNTr_ADDR(port),rvp)
#define WRITE_IBPCOUNTr(unit,port,rv) \
	cmic_write_reg(unit,IBPCOUNTr_ADDR(port),rv)
#define GET_IBPCOUNTr_ICC(rv)           ((rv)>>0&0xffff)
#define SET_IBPCOUNTr_ICC(rv,fv)        (rv=(rv&~(0xffff<<0))|((fv)&0xffff)<<0)

#define INGRESSBACKPRESSUREr_ADDR(port) reg_addr(INGRESSBACKPRESSUREr,port)
#define READ_INGRESSBACKPRESSUREr(unit,port,rvp) \
	cmic_read_reg(unit,INGRESSBACKPRESSUREr_ADDR(port),rvp)
#define WRITE_INGRESSBACKPRESSUREr(unit,port,rv) \
	cmic_write_reg(unit,INGRESSBACKPRESSUREr_ADDR(port),rv)
#define GET_INGRESSBACKPRESSUREr_IBP(rv) ((rv)>>0&0xffff)
#define SET_INGRESSBACKPRESSUREr_IBP(rv,fv) (rv=(rv&~(0xffff<<0))|((fv)&0xffff)<<0)

#define INITIATEBIST1r_ADDR()           reg_addr(INITIATEBIST1r)
#define READ_INITIATEBIST1r(unit,rvp) \
	cmic_read_reg(unit,INITIATEBIST1r_ADDR(),rvp)
#define WRITE_INITIATEBIST1r(unit,rv) \
	cmic_write_reg(unit,INITIATEBIST1r_ADDR(),rv)
#define GET_INITIATEBIST1r_CAB0_DONE(rv) ((rv)>>0&0x1)
#define SET_INITIATEBIST1r_CAB0_DONE(rv,fv) (rv=(rv&~(0x1<<0))|((fv)&0x1)<<0)
#define GET_INITIATEBIST1r_CAB1_DONE(rv) ((rv)>>1&0x1)
#define SET_INITIATEBIST1r_CAB1_DONE(rv,fv) (rv=(rv&~(0x1<<1))|((fv)&0x1)<<1)
#define GET_INITIATEBIST1r_CAB2_DONE(rv) ((rv)>>2&0x1)
#define SET_INITIATEBIST1r_CAB2_DONE(rv,fv) (rv=(rv&~(0x1<<2))|((fv)&0x1)<<2)
#define GET_INITIATEBIST1r_CAB3_DONE(rv) ((rv)>>3&0x1)
#define SET_INITIATEBIST1r_CAB3_DONE(rv,fv) (rv=(rv&~(0x1<<3))|((fv)&0x1)<<3)
#define GET_INITIATEBIST1r_CBP0_DONE(rv) ((rv)>>10&0x1)
#define SET_INITIATEBIST1r_CBP0_DONE(rv,fv) (rv=(rv&~(0x1<<10))|((fv)&0x1)<<10)
#define GET_INITIATEBIST1r_CBP1_DONE(rv) ((rv)>>11&0x1)
#define SET_INITIATEBIST1r_CBP1_DONE(rv,fv) (rv=(rv&~(0x1<<11))|((fv)&0x1)<<11)
#define GET_INITIATEBIST1r_CBP2_DONE(rv) ((rv)>>12&0x1)
#define SET_INITIATEBIST1r_CBP2_DONE(rv,fv) (rv=(rv&~(0x1<<12))|((fv)&0x1)<<12)
#define GET_INITIATEBIST1r_CBP3_DONE(rv) ((rv)>>13&0x1)
#define SET_INITIATEBIST1r_CBP3_DONE(rv,fv) (rv=(rv&~(0x1<<13))|((fv)&0x1)<<13)
#define GET_INITIATEBIST1r_CBPH_DONE(rv) ((rv)>>4&0x1)
#define SET_INITIATEBIST1r_CBPH_DONE(rv,fv) (rv=(rv&~(0x1<<4))|((fv)&0x1)<<4)
#define GET_INITIATEBIST1r_CCP_DONE(rv) ((rv)>>5&0x1)
#define SET_INITIATEBIST1r_CCP_DONE(rv,fv) (rv=(rv&~(0x1<<5))|((fv)&0x1)<<5)
#define GET_INITIATEBIST1r_CFAP_DONE(rv) ((rv)>>7&0x1)
#define SET_INITIATEBIST1r_CFAP_DONE(rv,fv) (rv=(rv&~(0x1<<7))|((fv)&0x1)<<7)
#define GET_INITIATEBIST1r_PID_DONE(rv) ((rv)>>9&0x1)
#define SET_INITIATEBIST1r_PID_DONE(rv,fv) (rv=(rv&~(0x1<<9))|((fv)&0x1)<<9)
#define GET_INITIATEBIST1r_PPP_DONE(rv) ((rv)>>6&0x1)
#define SET_INITIATEBIST1r_PPP_DONE(rv,fv) (rv=(rv&~(0x1<<6))|((fv)&0x1)<<6)
#define GET_INITIATEBIST1r_RUNBIST_CAB0(rv) ((rv)>>0&0x1)
#define SET_INITIATEBIST1r_RUNBIST_CAB0(rv,fv) (rv=(rv&~(0x1<<0))|((fv)&0x1)<<0)
#define GET_INITIATEBIST1r_RUNBIST_CAB1(rv) ((rv)>>1&0x1)
#define SET_INITIATEBIST1r_RUNBIST_CAB1(rv,fv) (rv=(rv&~(0x1<<1))|((fv)&0x1)<<1)
#define GET_INITIATEBIST1r_RUNBIST_CAB2(rv) ((rv)>>2&0x1)
#define SET_INITIATEBIST1r_RUNBIST_CAB2(rv,fv) (rv=(rv&~(0x1<<2))|((fv)&0x1)<<2)
#define GET_INITIATEBIST1r_RUNBIST_CAB3(rv) ((rv)>>3&0x1)
#define SET_INITIATEBIST1r_RUNBIST_CAB3(rv,fv) (rv=(rv&~(0x1<<3))|((fv)&0x1)<<3)
#define GET_INITIATEBIST1r_RUNBIST_CBP0(rv) ((rv)>>10&0x1)
#define SET_INITIATEBIST1r_RUNBIST_CBP0(rv,fv) (rv=(rv&~(0x1<<10))|((fv)&0x1)<<10)
#define GET_INITIATEBIST1r_RUNBIST_CBP1(rv) ((rv)>>11&0x1)
#define SET_INITIATEBIST1r_RUNBIST_CBP1(rv,fv) (rv=(rv&~(0x1<<11))|((fv)&0x1)<<11)
#define GET_INITIATEBIST1r_RUNBIST_CBP2(rv) ((rv)>>12&0x1)
#define SET_INITIATEBIST1r_RUNBIST_CBP2(rv,fv) (rv=(rv&~(0x1<<12))|((fv)&0x1)<<12)
#define GET_INITIATEBIST1r_RUNBIST_CBP3(rv) ((rv)>>13&0x1)
#define SET_INITIATEBIST1r_RUNBIST_CBP3(rv,fv) (rv=(rv&~(0x1<<13))|((fv)&0x1)<<13)
#define GET_INITIATEBIST1r_RUNBIST_CBPH(rv) ((rv)>>4&0x1)
#define SET_INITIATEBIST1r_RUNBIST_CBPH(rv,fv) (rv=(rv&~(0x1<<4))|((fv)&0x1)<<4)
#define GET_INITIATEBIST1r_RUNBIST_CCP(rv) ((rv)>>5&0x1)
#define SET_INITIATEBIST1r_RUNBIST_CCP(rv,fv) (rv=(rv&~(0x1<<5))|((fv)&0x1)<<5)
#define GET_INITIATEBIST1r_RUNBIST_CFAP(rv) ((rv)>>7&0x1)
#define SET_INITIATEBIST1r_RUNBIST_CFAP(rv,fv) (rv=(rv&~(0x1<<7))|((fv)&0x1)<<7)
#define GET_INITIATEBIST1r_RUNBIST_PID(rv) ((rv)>>9&0x1)
#define SET_INITIATEBIST1r_RUNBIST_PID(rv,fv) (rv=(rv&~(0x1<<9))|((fv)&0x1)<<9)
#define GET_INITIATEBIST1r_RUNBIST_PPP(rv) ((rv)>>6&0x1)
#define SET_INITIATEBIST1r_RUNBIST_PPP(rv,fv) (rv=(rv&~(0x1<<6))|((fv)&0x1)<<6)
#define GET_INITIATEBIST1r_RUNBIST_SFAP(rv) ((rv)>>8&0x1)
#define SET_INITIATEBIST1r_RUNBIST_SFAP(rv,fv) (rv=(rv&~(0x1<<8))|((fv)&0x1)<<8)
#define GET_INITIATEBIST1r_SFAP_DONE(rv) ((rv)>>8&0x1)
#define SET_INITIATEBIST1r_SFAP_DONE(rv,fv) (rv=(rv&~(0x1<<8))|((fv)&0x1)<<8)

#define INITIATEBIST2r_ADDR()           reg_addr(INITIATEBIST2r)
#define READ_INITIATEBIST2r(unit,rvp) \
	cmic_read_reg(unit,INITIATEBIST2r_ADDR(),rvp)
#define WRITE_INITIATEBIST2r(unit,rv) \
	cmic_write_reg(unit,INITIATEBIST2r_ADDR(),rv)
#define GET_INITIATEBIST2r_RUNBIST_XQ(rv) ((rv)>>0&0xffffffff)
#define SET_INITIATEBIST2r_RUNBIST_XQ(rv,fv) (rv=(rv&~(0xffffffff<<0))|((fv)&0xffffffff)<<0)
#define GET_INITIATEBIST2r_XQ_DONE(rv)  ((rv)>>0&0xffffffff)
#define SET_INITIATEBIST2r_XQ_DONE(rv,fv) (rv=(rv&~(0xffffffff<<0))|((fv)&0xffffffff)<<0)

#define LCCCOUNTr_ADDR(port)            reg_addr(LCCCOUNTr,port)
#define READ_LCCCOUNTr(unit,port,rvp) \
	cmic_read_reg(unit,LCCCOUNTr_ADDR(port),rvp)
#define WRITE_LCCCOUNTr(unit,port,rv) \
	cmic_write_reg(unit,LCCCOUNTr_ADDR(port),rv)
#define GET_LCCCOUNTr_LCC(rv)           ((rv)>>0&0xffff)
#define SET_LCCCOUNTr_LCC(rv,fv)        (rv=(rv&~(0xffff<<0))|((fv)&0xffff)<<0)

#define LOWWATERMARKERr_ADDR(port)      reg_addr(LOWWATERMARKERr,port)
#define READ_LOWWATERMARKERr(unit,port,rvp) \
	cmic_read_reg(unit,LOWWATERMARKERr_ADDR(port),rvp)
#define WRITE_LOWWATERMARKERr(unit,port,rv) \
	cmic_write_reg(unit,LOWWATERMARKERr_ADDR(port),rv)
#define GET_LOWWATERMARKERr_LWM(rv)     ((rv)>>0&0xfff)
#define SET_LOWWATERMARKERr_LWM(rv,fv)  (rv=(rv&~(0xfff<<0))|((fv)&0xfff)<<0)

#define MAXFRr_ADDR(port)               reg_addr(MAXFRr,port)
#define READ_MAXFRr(unit,port,rvp) \
	cmic_read_reg(unit,MAXFRr_ADDR(port),rvp)
#define WRITE_MAXFRr(unit,port,rv) \
	cmic_write_reg(unit,MAXFRr_ADDR(port),rv)
#define GET_MAXFRr_MAXFR(rv)            ((rv)>>0&0xffff)
#define SET_MAXFRr_MAXFR(rv,fv)         (rv=(rv&~(0xffff<<0))|((fv)&0xffff)<<0)

#define MIRRORr_ADDR(pic)               reg_addr(MIRRORr,pic)
#define READ_MIRRORr(unit,pic,rvp) \
	cmic_read_reg(unit,MIRRORr_ADDR(pic),rvp)
#define WRITE_MIRRORr(unit,pic,rv) \
	cmic_write_reg(unit,MIRRORr_ADDR(pic),rv)
#define GET_MIRRORr_MIRR_EN(rv)         ((rv)>>6&0x1)
#define SET_MIRRORr_MIRR_EN(rv,fv)      (rv=(rv&~(0x1<<6))|((fv)&0x1)<<6)
#define GET_MIRRORr_MIRR_L3(rv)         ((rv)>>7&0x1)
#define SET_MIRRORr_MIRR_L3(rv,fv)      (rv=(rv&~(0x1<<7))|((fv)&0x1)<<7)
#define GET_MIRRORr_MPORT(rv)           ((rv)>>0&0x3f)
#define SET_MIRRORr_MPORT(rv,fv)        (rv=(rv&~(0x3f<<0))|((fv)&0x3f)<<0)

#define MISCCONFIGr_ADDR()              reg_addr(MISCCONFIGr)
#define READ_MISCCONFIGr(unit,rvp) \
	cmic_read_reg(unit,MISCCONFIGr_ADDR(),rvp)
#define WRITE_MISCCONFIGr(unit,rv) \
	cmic_write_reg(unit,MISCCONFIGr_ADDR(),rv)
#define GET_MISCCONFIGr_CMICARBITERDELAY(rv) ((rv)>>8&0xf)
#define SET_MISCCONFIGr_CMICARBITERDELAY(rv,fv) (rv=(rv&~(0xf<<8))|((fv)&0xf)<<8)
#define GET_MISCCONFIGr_MAXOUTSTANDINGREAD(rv) ((rv)>>4&0xf)
#define SET_MISCCONFIGr_MAXOUTSTANDINGREAD(rv,fv) (rv=(rv&~(0xf<<4))|((fv)&0xf)<<4)
#define GET_MISCCONFIGr_POACHINGTIMER(rv) ((rv)>>16&0xffff)
#define SET_MISCCONFIGr_POACHINGTIMER(rv,fv) (rv=(rv&~(0xffff<<16))|((fv)&0xffff)<<16)
#define GET_MISCCONFIGr_RESERVED0(rv)   ((rv)>>0&0x1)
#define SET_MISCCONFIGr_RESERVED0(rv,fv) (rv=(rv&~(0x1<<0))|((fv)&0x1)<<0)
#define GET_MISCCONFIGr_RESERVED1(rv)   ((rv)>>2&0x1)
#define SET_MISCCONFIGr_RESERVED1(rv,fv) (rv=(rv&~(0x1<<2))|((fv)&0x1)<<2)
#define GET_MISCCONFIGr_RESERVED2(rv)   ((rv)>>3&0x1)
#define SET_MISCCONFIGr_RESERVED2(rv,fv) (rv=(rv&~(0x1<<3))|((fv)&0x1)<<3)
#define GET_MISCCONFIGr_SHORTOUTUNUSEDBITS(rv) ((rv)>>1&0x1)
#define SET_MISCCONFIGr_SHORTOUTUNUSEDBITS(rv,fv) (rv=(rv&~(0x1<<1))|((fv)&0x1)<<1)
#define GET_MISCCONFIGr_SKIDMARK(rv)    ((rv)>>12&0xf)
#define SET_MISCCONFIGr_SKIDMARK(rv,fv) (rv=(rv&~(0xf<<12))|((fv)&0xf)<<12)

#define MMUPORTENABLEr_ADDR()           reg_addr(MMUPORTENABLEr)
#define READ_MMUPORTENABLEr(unit,rvp) \
	cmic_read_reg(unit,MMUPORTENABLEr_ADDR(),rvp)
#define WRITE_MMUPORTENABLEr(unit,rv) \
	cmic_write_reg(unit,MMUPORTENABLEr_ADDR(),rv)
#define GET_MMUPORTENABLEr_PORT_BITMAP(rv) ((rv)>>0&0xffffffff)
#define SET_MMUPORTENABLEr_PORT_BITMAP(rv,fv) (rv=(rv&~(0xffffffff<<0))|((fv)&0xffffffff)<<0)

#define PPPEMPTYr_ADDR()                reg_addr(PPPEMPTYr)
#define READ_PPPEMPTYr(unit,rvp) \
	cmic_read_reg(unit,PPPEMPTYr_ADDR(),rvp)
#define WRITE_PPPEMPTYr(unit,rv) \
	cmic_write_reg(unit,PPPEMPTYr_ADDR(),rv)
#define GET_PPPEMPTYr_PORT_BITMAP(rv)   ((rv)>>0&0xffffffff)
#define SET_PPPEMPTYr_PORT_BITMAP(rv,fv) (rv=(rv&~(0xffffffff<<0))|((fv)&0xffffffff)<<0)

#define PPPFULLr_ADDR()                 reg_addr(PPPFULLr)
#define READ_PPPFULLr(unit,rvp) \
	cmic_read_reg(unit,PPPFULLr_ADDR(),rvp)
#define WRITE_PPPFULLr(unit,rv) \
	cmic_write_reg(unit,PPPFULLr_ADDR(),rv)
#define GET_PPPFULLr_PORT_BITMAP(rv)    ((rv)>>0&0xffffffff)
#define SET_PPPFULLr_PORT_BITMAP(rv,fv) (rv=(rv&~(0xffffffff<<0))|((fv)&0xffffffff)<<0)

#define RALNr_ADDR(port)                reg_addr(RALNr,port)
#define READ_RALNr(unit,port,rvp) \
	cmic_read_reg(unit,RALNr_ADDR(port),rvp)
#define WRITE_RALNr(unit,port,rv) \
	cmic_write_reg(unit,RALNr_ADDR(port),rv)
#define GET_RALNr_COUNT(rv)             ((rv)>>0&0xfff)
#define SET_RALNr_COUNT(rv,fv)          (rv=(rv&~(0xfff<<0))|((fv)&0xfff)<<0)

#define RBCAr_ADDR(port)                reg_addr(RBCAr,port)
#define READ_RBCAr(unit,port,rvp) \
	cmic_read_reg(unit,RBCAr_ADDR(port),rvp)
#define WRITE_RBCAr(unit,port,rv) \
	cmic_write_reg(unit,RBCAr_ADDR(port),rv)
#define GET_RBCAr_COUNT(rv)             ((rv)>>0&0xfffff)
#define SET_RBCAr_COUNT(rv,fv)          (rv=(rv&~(0xfffff<<0))|((fv)&0xfffff)<<0)

#define RBYTr_ADDR(port)                reg_addr(RBYTr,port)
#define READ_RBYTr(unit,port,rvp) \
	cmic_read_reg(unit,RBYTr_ADDR(port),rvp)
#define WRITE_RBYTr(unit,port,rv) \
	cmic_write_reg(unit,RBYTr_ADDR(port),rv)
#define GET_RBYTr_COUNT(rv)             ((rv)>>0&0xffffffff)
#define SET_RBYTr_COUNT(rv,fv)          (rv=(rv&~(0xffffffff<<0))|((fv)&0xffffffff)<<0)

#define RCDEr_ADDR(port)                reg_addr(RCDEr,port)
#define READ_RCDEr(unit,port,rvp) \
	cmic_read_reg(unit,RCDEr_ADDR(port),rvp)
#define WRITE_RCDEr(unit,port,rv) \
	cmic_write_reg(unit,RCDEr_ADDR(port),rv)
#define GET_RCDEr_COUNT(rv)             ((rv)>>0&0xfff)
#define SET_RCDEr_COUNT(rv,fv)          (rv=(rv&~(0xfff<<0))|((fv)&0xfff)<<0)

#define RDISCr_ADDR(port)               reg_addr(RDISCr,port)
#define READ_RDISCr(unit,port,rvp) \
	cmic_read_reg(unit,RDISCr_ADDR(port),rvp)
#define WRITE_RDISCr(unit,port,rv) \
	cmic_write_reg(unit,RDISCr_ADDR(port),rv)
#define GET_RDISCr_COUNT(rv)            ((rv)>>0&0xfff)
#define SET_RDISCr_COUNT(rv,fv)         (rv=(rv&~(0xfff<<0))|((fv)&0xfff)<<0)

#define REROUTINGr_ADDR()               reg_addr(REROUTINGr)
#define READ_REROUTINGr(unit,rvp) \
	cmic_read_reg(unit,REROUTINGr_ADDR(),rvp)
#define WRITE_REROUTINGr(unit,rv) \
	cmic_write_reg(unit,REROUTINGr_ADDR(),rv)
#define GET_REROUTINGr_FORCE_PORT_BITMAP(rv) ((rv)>>0&0xffffffff)
#define SET_REROUTINGr_FORCE_PORT_BITMAP(rv,fv) (rv=(rv&~(0xffffffff<<0))|((fv)&0xffffffff)<<0)
#define GET_REROUTINGr_STATUS_PORT_BITMAP(rv) ((rv)>>0&0xffffffff)
#define SET_REROUTINGr_STATUS_PORT_BITMAP(rv,fv) (rv=(rv&~(0xffffffff<<0))|((fv)&0xffffffff)<<0)

#define RESURRECTr_ADDR()               reg_addr(RESURRECTr)
#define READ_RESURRECTr(unit,rvp) \
	cmic_read_reg(unit,RESURRECTr_ADDR(),rvp)
#define WRITE_RESURRECTr(unit,rv) \
	cmic_write_reg(unit,RESURRECTr_ADDR(),rv)
#define GET_RESURRECTr_PORT_BITMAP(rv)  ((rv)>>0&0xffffffff)
#define SET_RESURRECTr_PORT_BITMAP(rv,fv) (rv=(rv&~(0xffffffff<<0))|((fv)&0xffffffff)<<0)

#define REVCDr_ADDR(port)               reg_addr(REVCDr,port)
#define READ_REVCDr(unit,port,rvp) \
	cmic_read_reg(unit,REVCDr_ADDR(port),rvp)
#define WRITE_REVCDr(unit,port,rv) \
	cmic_write_reg(unit,REVCDr_ADDR(port),rv)
#define GET_REVCDr_MONTH(rv)            ((rv)>>4&0xf)
#define SET_REVCDr_MONTH(rv,fv)         (rv=(rv&~(0xf<<4))|((fv)&0xf)<<4)
#define GET_REVCDr_REV(rv)              ((rv)>>0&0xf)
#define SET_REVCDr_REV(rv,fv)           (rv=(rv&~(0xf<<0))|((fv)&0xf)<<0)
#define GET_REVCDr_YEAR(rv)             ((rv)>>8&0xff)
#define SET_REVCDr_YEAR(rv,fv)          (rv=(rv&~(0xff<<8))|((fv)&0xff)<<8)

#define RFCRr_ADDR(port)                reg_addr(RFCRr,port)
#define READ_RFCRr(unit,port,rvp) \
	cmic_read_reg(unit,RFCRr_ADDR(port),rvp)
#define WRITE_RFCRr(unit,port,rv) \
	cmic_write_reg(unit,RFCRr_ADDR(port),rv)
#define GET_RFCRr_COUNT(rv)             ((rv)>>0&0xfff)
#define SET_RFCRr_COUNT(rv,fv)          (rv=(rv&~(0xfff<<0))|((fv)&0xfff)<<0)

#define RFCSr_ADDR(port)                reg_addr(RFCSr,port)
#define READ_RFCSr(unit,port,rvp) \
	cmic_read_reg(unit,RFCSr_ADDR(port),rvp)
#define WRITE_RFCSr(unit,port,rv) \
	cmic_write_reg(unit,RFCSr_ADDR(port),rv)
#define GET_RFCSr_COUNT(rv)             ((rv)>>0&0xfff)
#define SET_RFCSr_COUNT(rv,fv)          (rv=(rv&~(0xfff<<0))|((fv)&0xfff)<<0)

#define RFLRr_ADDR(port)                reg_addr(RFLRr,port)
#define READ_RFLRr(unit,port,rvp) \
	cmic_read_reg(unit,RFLRr_ADDR(port),rvp)
#define WRITE_RFLRr(unit,port,rv) \
	cmic_write_reg(unit,RFLRr_ADDR(port),rv)
#define GET_RFLRr_COUNT(rv)             ((rv)>>0&0xfff)
#define SET_RFLRr_COUNT(rv,fv)          (rv=(rv&~(0xfff<<0))|((fv)&0xfff)<<0)

#define RFRGr_ADDR(port)                reg_addr(RFRGr,port)
#define READ_RFRGr(unit,port,rvp) \
	cmic_read_reg(unit,RFRGr_ADDR(port),rvp)
#define WRITE_RFRGr(unit,port,rv) \
	cmic_write_reg(unit,RFRGr_ADDR(port),rv)
#define GET_RFRGr_COUNT(rv)             ((rv)>>0&0xfff)
#define SET_RFRGr_COUNT(rv,fv)          (rv=(rv&~(0xfff<<0))|((fv)&0xfff)<<0)

#define RIPCr_ADDR(port)                reg_addr(RIPCr,port)
#define READ_RIPCr(unit,port,rvp) \
	cmic_read_reg(unit,RIPCr_ADDR(port),rvp)
#define WRITE_RIPCr(unit,port,rv) \
	cmic_write_reg(unit,RIPCr_ADDR(port),rv)
#define GET_RIPCr_COUNT(rv)             ((rv)>>0&0xfffff)
#define SET_RIPCr_COUNT(rv,fv)          (rv=(rv&~(0xfffff<<0))|((fv)&0xfffff)<<0)

#define RIPCHKr_ADDR(port)              reg_addr(RIPCHKr,port)
#define READ_RIPCHKr(unit,port,rvp) \
	cmic_read_reg(unit,RIPCHKr_ADDR(port),rvp)
#define WRITE_RIPCHKr(unit,port,rv) \
	cmic_write_reg(unit,RIPCHKr_ADDR(port),rv)
#define GET_RIPCHKr_COUNT(rv)           ((rv)>>0&0xfff)
#define SET_RIPCHKr_COUNT(rv,fv)        (rv=(rv&~(0xfff<<0))|((fv)&0xfff)<<0)

#define RIPDr_ADDR(port)                reg_addr(RIPDr,port)
#define READ_RIPDr(unit,port,rvp) \
	cmic_read_reg(unit,RIPDr_ADDR(port),rvp)
#define WRITE_RIPDr(unit,port,rv) \
	cmic_write_reg(unit,RIPDr_ADDR(port),rv)
#define GET_RIPDr_COUNT(rv)             ((rv)>>0&0xfff)
#define SET_RIPDr_COUNT(rv,fv)          (rv=(rv&~(0xfff<<0))|((fv)&0xfff)<<0)

#define RIPXr_ADDR(port)                reg_addr(RIPXr,port)
#define READ_RIPXr(unit,port,rvp) \
	cmic_read_reg(unit,RIPXr_ADDR(port),rvp)
#define WRITE_RIPXr(unit,port,rv) \
	cmic_write_reg(unit,RIPXr_ADDR(port),rv)
#define GET_RIPXr_COUNT(rv)             ((rv)>>0&0x1)
#define SET_RIPXr_COUNT(rv,fv)          (rv=(rv&~(0x1<<0))|((fv)&0x1)<<0)

#define RIPXDr_ADDR(port)               reg_addr(RIPXDr,port)
#define READ_RIPXDr(unit,port,rvp) \
	cmic_read_reg(unit,RIPXDr_ADDR(port),rvp)
#define WRITE_RIPXDr(unit,port,rv) \
	cmic_write_reg(unit,RIPXDr_ADDR(port),rv)
#define GET_RIPXDr_COUNT(rv)            ((rv)>>0&0x1)
#define SET_RIPXDr_COUNT(rv,fv)         (rv=(rv&~(0x1<<0))|((fv)&0x1)<<0)

#define RJBRr_ADDR(port)                reg_addr(RJBRr,port)
#define READ_RJBRr(unit,port,rvp) \
	cmic_read_reg(unit,RJBRr_ADDR(port),rvp)
#define WRITE_RJBRr(unit,port,rv) \
	cmic_write_reg(unit,RJBRr_ADDR(port),rv)
#define GET_RJBRr_COUNT(rv)             ((rv)>>0&0xfff)
#define SET_RJBRr_COUNT(rv,fv)          (rv=(rv&~(0xfff<<0))|((fv)&0xfff)<<0)

#define RMCAr_ADDR(port)                reg_addr(RMCAr,port)
#define READ_RMCAr(unit,port,rvp) \
	cmic_read_reg(unit,RMCAr_ADDR(port),rvp)
#define WRITE_RMCAr(unit,port,rv) \
	cmic_write_reg(unit,RMCAr_ADDR(port),rv)
#define GET_RMCAr_COUNT(rv)             ((rv)>>0&0xfffff)
#define SET_RMCAr_COUNT(rv,fv)          (rv=(rv&~(0xfffff<<0))|((fv)&0xfffff)<<0)

#define ROVRr_ADDR(port)                reg_addr(ROVRr,port)
#define READ_ROVRr(unit,port,rvp) \
	cmic_read_reg(unit,ROVRr_ADDR(port),rvp)
#define WRITE_ROVRr(unit,port,rv) \
	cmic_write_reg(unit,ROVRr_ADDR(port),rv)
#define GET_ROVRr_COUNT(rv)             ((rv)>>0&0xfff)
#define SET_ROVRr_COUNT(rv,fv)          (rv=(rv&~(0xfff<<0))|((fv)&0xfff)<<0)

#define RPKTr_ADDR(port)                reg_addr(RPKTr,port)
#define READ_RPKTr(unit,port,rvp) \
	cmic_read_reg(unit,RPKTr_ADDR(port),rvp)
#define WRITE_RPKTr(unit,port,rv) \
	cmic_write_reg(unit,RPKTr_ADDR(port),rv)
#define GET_RPKTr_COUNT(rv)             ((rv)>>0&0xfffff)
#define SET_RPKTr_COUNT(rv,fv)          (rv=(rv&~(0xfffff<<0))|((fv)&0xfffff)<<0)

#define RUNDr_ADDR(port)                reg_addr(RUNDr,port)
#define READ_RUNDr(unit,port,rvp) \
	cmic_read_reg(unit,RUNDr_ADDR(port),rvp)
#define WRITE_RUNDr(unit,port,rv) \
	cmic_write_reg(unit,RUNDr_ADDR(port),rv)
#define GET_RUNDr_COUNT(rv)             ((rv)>>0&0xfff)
#define SET_RUNDr_COUNT(rv,fv)          (rv=(rv&~(0xfff<<0))|((fv)&0xfff)<<0)

#define RXCFr_ADDR(port)                reg_addr(RXCFr,port)
#define READ_RXCFr(unit,port,rvp) \
	cmic_read_reg(unit,RXCFr_ADDR(port),rvp)
#define WRITE_RXCFr(unit,port,rv) \
	cmic_write_reg(unit,RXCFr_ADDR(port),rv)
#define GET_RXCFr_COUNT(rv)             ((rv)>>0&0xfff)
#define SET_RXCFr_COUNT(rv,fv)          (rv=(rv&~(0xfff<<0))|((fv)&0xfff)<<0)

#define RXPFr_ADDR(port)                reg_addr(RXPFr,port)
#define READ_RXPFr(unit,port,rvp) \
	cmic_read_reg(unit,RXPFr_ADDR(port),rvp)
#define WRITE_RXPFr(unit,port,rv) \
	cmic_write_reg(unit,RXPFr_ADDR(port),rv)
#define GET_RXPFr_COUNT(rv)             ((rv)>>0&0xfff)
#define SET_RXPFr_COUNT(rv,fv)          (rv=(rv&~(0xfff<<0))|((fv)&0xfff)<<0)

#define RXUOr_ADDR(port)                reg_addr(RXUOr,port)
#define READ_RXUOr(unit,port,rvp) \
	cmic_read_reg(unit,RXUOr_ADDR(port),rvp)
#define WRITE_RXUOr(unit,port,rv) \
	cmic_write_reg(unit,RXUOr_ADDR(port),rv)
#define GET_RXUOr_COUNT(rv)             ((rv)>>0&0xfff)
#define SET_RXUOr_COUNT(rv,fv)          (rv=(rv&~(0xfff<<0))|((fv)&0xfff)<<0)

#define SCHEDULERCONTROLr_ADDR(port)    reg_addr(SCHEDULERCONTROLr,port)
#define READ_SCHEDULERCONTROLr(unit,port,rvp) \
	cmic_read_reg(unit,SCHEDULERCONTROLr_ADDR(port),rvp)
#define WRITE_SCHEDULERCONTROLr(unit,port,rv) \
	cmic_write_reg(unit,SCHEDULERCONTROLr_ADDR(port),rv)
#define GET_SCHEDULERCONTROLr_MAXNUMBEROFCOS(rv) ((rv)>>0&0x3)
#define SET_SCHEDULERCONTROLr_MAXNUMBEROFCOS(rv,fv) (rv=(rv&~(0x3<<0))|((fv)&0x3)<<0)
#define GET_SCHEDULERCONTROLr_PACKETPOINTERPOOLSIZE(rv) ((rv)>>16&0x1)
#define SET_SCHEDULERCONTROLr_PACKETPOINTERPOOLSIZE(rv,fv) (rv=(rv&~(0x1<<16))|((fv)&0x1)<<16)

#define SDRAMCONTROLr_ADDR()            reg_addr(SDRAMCONTROLr)
#define READ_SDRAMCONTROLr(unit,rvp) \
	cmic_read_reg(unit,SDRAMCONTROLr_ADDR(),rvp)
#define WRITE_SDRAMCONTROLr(unit,rv) \
	cmic_write_reg(unit,SDRAMCONTROLr_ADDR(),rv)
#define GET_SDRAMCONTROLr_DDR_SDRAMEN(rv) ((rv)>>24&0x1)
#define SET_SDRAMCONTROLr_DDR_SDRAMEN(rv,fv) (rv=(rv&~(0x1<<24))|((fv)&0x1)<<24)
#define GET_SDRAMCONTROLr_DDR_SDRAM_BSEL(rv) ((rv)>>27&0x3)
#define SET_SDRAMCONTROLr_DDR_SDRAM_BSEL(rv,fv) (rv=(rv&~(0x3<<27))|((fv)&0x3)<<27)
#define GET_SDRAMCONTROLr_DDR_SDRAM_CLSEL(rv) ((rv)>>25&0x3)
#define SET_SDRAMCONTROLr_DDR_SDRAM_CLSEL(rv,fv) (rv=(rv&~(0x3<<25))|((fv)&0x3)<<25)
#define GET_SDRAMCONTROLr_DDR_SDRAM_TRP(rv) ((rv)>>29&0x1)
#define SET_SDRAMCONTROLr_DDR_SDRAM_TRP(rv,fv) (rv=(rv&~(0x1<<29))|((fv)&0x1)<<29)
#define GET_SDRAMCONTROLr_MAXCELLSPERSLOT(rv) ((rv)>>0&0xf)
#define SET_SDRAMCONTROLr_MAXCELLSPERSLOT(rv,fv) (rv=(rv&~(0xf<<0))|((fv)&0xf)<<0)
#define GET_SDRAMCONTROLr_MAXSLOTSPERCHAIN(rv) ((rv)>>4&0xf)
#define SET_SDRAMCONTROLr_MAXSLOTSPERCHAIN(rv,fv) (rv=(rv&~(0xf<<4))|((fv)&0xf)<<4)
#define GET_SDRAMCONTROLr_SDRAMCLK0DLL(rv) ((rv)>>8&0xf)
#define SET_SDRAMCONTROLr_SDRAMCLK0DLL(rv,fv) (rv=(rv&~(0xf<<8))|((fv)&0xf)<<8)
#define GET_SDRAMCONTROLr_SDRAMCLK1DLL(rv) ((rv)>>12&0xf)
#define SET_SDRAMCONTROLr_SDRAMCLK1DLL(rv,fv) (rv=(rv&~(0xf<<12))|((fv)&0xf)<<12)
#define GET_SDRAMCONTROLr_SDRAMCLK2DLL(rv) ((rv)>>16&0xf)
#define SET_SDRAMCONTROLr_SDRAMCLK2DLL(rv,fv) (rv=(rv&~(0xf<<16))|((fv)&0xf)<<16)
#define GET_SDRAMCONTROLr_SDRAMCLK3DLL(rv) ((rv)>>20&0xf)
#define SET_SDRAMCONTROLr_SDRAMCLK3DLL(rv,fv) (rv=(rv&~(0xf<<20))|((fv)&0xf)<<20)

#define SDRAMCONTROL2r_ADDR()           reg_addr(SDRAMCONTROL2r)
#define READ_SDRAMCONTROL2r(unit,rvp) \
	cmic_read_reg(unit,SDRAMCONTROL2r_ADDR(),rvp)
#define WRITE_SDRAMCONTROL2r(unit,rv) \
	cmic_write_reg(unit,SDRAMCONTROL2r_ADDR(),rv)
#define GET_SDRAMCONTROL2r_NATPRIENB(rv) ((rv)>>16&0x1)
#define SET_SDRAMCONTROL2r_NATPRIENB(rv,fv) (rv=(rv&~(0x1<<16))|((fv)&0x1)<<16)
#define GET_SDRAMCONTROL2r_OVHEADTIMEPRIENB(rv) ((rv)>>17&0x1)
#define SET_SDRAMCONTROL2r_OVHEADTIMEPRIENB(rv,fv) (rv=(rv&~(0x1<<17))|((fv)&0x1)<<17)
#define GET_SDRAMCONTROL2r_REF_CNT(rv)  ((rv)>>0&0xfff)
#define SET_SDRAMCONTROL2r_REF_CNT(rv,fv) (rv=(rv&~(0xfff<<0))|((fv)&0xfff)<<0)
#define GET_SDRAMCONTROL2r_REQTIMERPRIENB(rv) ((rv)>>18&0x1)
#define SET_SDRAMCONTROL2r_REQTIMERPRIENB(rv,fv) (rv=(rv&~(0x1<<18))|((fv)&0x1)<<18)
#define GET_SDRAMCONTROL2r_ROUNDROBINPRIENB(rv) ((rv)>>15&0x1)
#define SET_SDRAMCONTROL2r_ROUNDROBINPRIENB(rv,fv) (rv=(rv&~(0x1<<15))|((fv)&0x1)<<15)
#define GET_SDRAMCONTROL2r_SAUFULLPRIENB(rv) ((rv)>>19&0x1)
#define SET_SDRAMCONTROL2r_SAUFULLPRIENB(rv,fv) (rv=(rv&~(0x1<<19))|((fv)&0x1)<<19)
#define GET_SDRAMCONTROL2r_SAUFULLVALUE(rv) ((rv)>>12&0x7)
#define SET_SDRAMCONTROL2r_SAUFULLVALUE(rv,fv) (rv=(rv&~(0x7<<12))|((fv)&0x7)<<12)
#define GET_SDRAMCONTROL2r_SAUTIMERVALUE(rv) ((rv)>>23&0x7)
#define SET_SDRAMCONTROL2r_SAUTIMERVALUE(rv,fv) (rv=(rv&~(0x7<<23))|((fv)&0x7)<<23)
#define GET_SDRAMCONTROL2r_SDUTIMERVALUE(rv) ((rv)>>20&0x7)
#define SET_SDRAMCONTROL2r_SDUTIMERVALUE(rv,fv) (rv=(rv&~(0x7<<20))|((fv)&0x7)<<20)
#define GET_SDRAMCONTROL2r_SFAPRDTIMERVALUE(rv) ((rv)>>29&0x7)
#define SET_SDRAMCONTROL2r_SFAPRDTIMERVALUE(rv,fv) (rv=(rv&~(0x7<<29))|((fv)&0x7)<<29)
#define GET_SDRAMCONTROL2r_SFAPWRTIMERVALUE(rv) ((rv)>>26&0x7)
#define SET_SDRAMCONTROL2r_SFAPWRTIMERVALUE(rv,fv) (rv=(rv&~(0x7<<26))|((fv)&0x7)<<26)

#define SFAPPOOLSIZEr_ADDR()            reg_addr(SFAPPOOLSIZEr)
#define READ_SFAPPOOLSIZEr(unit,rvp) \
	cmic_read_reg(unit,SFAPPOOLSIZEr_ADDR(),rvp)
#define WRITE_SFAPPOOLSIZEr(unit,rv) \
	cmic_write_reg(unit,SFAPPOOLSIZEr_ADDR(),rv)
#define GET_SFAPPOOLSIZEr_GBPFULLCLEARPOINT(rv) ((rv)>>22&0x1f)
#define SET_SFAPPOOLSIZEr_GBPFULLCLEARPOINT(rv,fv) (rv=(rv&~(0x1f<<22))|((fv)&0x1f)<<22)
#define GET_SFAPPOOLSIZEr_GBPFULLSETPOINT(rv) ((rv)>>27&0x1f)
#define SET_SFAPPOOLSIZEr_GBPFULLSETPOINT(rv,fv) (rv=(rv&~(0x1f<<27))|((fv)&0x1f)<<27)
#define GET_SFAPPOOLSIZEr_SDRAMSIZE(rv) ((rv)>>10&0x3)
#define SET_SFAPPOOLSIZEr_SDRAMSIZE(rv,fv) (rv=(rv&~(0x3<<10))|((fv)&0x3)<<10)
#define GET_SFAPPOOLSIZEr_SFAPPOOLSIZE(rv) ((rv)>>0&0x3ff)
#define SET_SFAPPOOLSIZEr_SFAPPOOLSIZE(rv,fv) (rv=(rv&~(0x3ff<<0))|((fv)&0x3ff)<<0)
#define GET_SFAPPOOLSIZEr_SLOTEJECTGAP(rv) ((rv)>>17&0x1f)
#define SET_SFAPPOOLSIZEr_SLOTEJECTGAP(rv,fv) (rv=(rv&~(0x1f<<17))|((fv)&0x1f)<<17)
#define GET_SFAPPOOLSIZEr_SLOTFETCHGAP(rv) ((rv)>>12&0x1f)
#define SET_SFAPPOOLSIZEr_SLOTFETCHGAP(rv,fv) (rv=(rv&~(0x1f<<12))|((fv)&0x1f)<<12)

#define SFAPREADPOINTERr_ADDR()         reg_addr(SFAPREADPOINTERr)
#define READ_SFAPREADPOINTERr(unit,rvp) \
	cmic_read_reg(unit,SFAPREADPOINTERr_ADDR(),rvp)
#define WRITE_SFAPREADPOINTERr(unit,rv) \
	cmic_write_reg(unit,SFAPREADPOINTERr_ADDR(),rv)
#define GET_SFAPREADPOINTERr_SRP(rv)    ((rv)>>0&0xffff)
#define SET_SFAPREADPOINTERr_SRP(rv,fv) (rv=(rv&~(0xffff<<0))|((fv)&0xffff)<<0)

#define SUM_LWMSr_ADDR()                reg_addr(SUM_LWMSr)
#define READ_SUM_LWMSr(unit,rvp) \
	cmic_read_reg(unit,SUM_LWMSr_ADDR(),rvp)
#define WRITE_SUM_LWMSr(unit,rv) \
	cmic_write_reg(unit,SUM_LWMSr_ADDR(),rv)
#define GET_SUM_LWMSr_SUM(rv)           ((rv)>>0&0xffff)
#define SET_SUM_LWMSr_SUM(rv,fv)        (rv=(rv&~(0xffff<<0))|((fv)&0xffff)<<0)

#define TABRTr_ADDR(port)               reg_addr(TABRTr,port)
#define READ_TABRTr(unit,port,rvp) \
	cmic_read_reg(unit,TABRTr_ADDR(port),rvp)
#define WRITE_TABRTr(unit,port,rv) \
	cmic_write_reg(unit,TABRTr_ADDR(port),rv)
#define GET_TABRTr_COUNT(rv)            ((rv)>>0&0xfff)
#define SET_TABRTr_COUNT(rv,fv)         (rv=(rv&~(0xfff<<0))|((fv)&0xfff)<<0)

#define TAGEr_ADDR(port)                reg_addr(TAGEr,port)
#define READ_TAGEr(unit,port,rvp) \
	cmic_read_reg(unit,TAGEr_ADDR(port),rvp)
#define WRITE_TAGEr(unit,port,rv) \
	cmic_write_reg(unit,TAGEr_ADDR(port),rv)
#define GET_TAGEr_COUNT(rv)             ((rv)>>0&0xfff)
#define SET_TAGEr_COUNT(rv,fv)          (rv=(rv&~(0xfff<<0))|((fv)&0xfff)<<0)

#define TBCAr_ADDR(port)                reg_addr(TBCAr,port)
#define READ_TBCAr(unit,port,rvp) \
	cmic_read_reg(unit,TBCAr_ADDR(port),rvp)
#define WRITE_TBCAr(unit,port,rv) \
	cmic_write_reg(unit,TBCAr_ADDR(port),rv)
#define GET_TBCAr_COUNT(rv)             ((rv)>>0&0xfffff)
#define SET_TBCAr_COUNT(rv,fv)          (rv=(rv&~(0xfffff<<0))|((fv)&0xfffff)<<0)

#define TBYTr_ADDR(port)                reg_addr(TBYTr,port)
#define READ_TBYTr(unit,port,rvp) \
	cmic_read_reg(unit,TBYTr_ADDR(port),rvp)
#define WRITE_TBYTr(unit,port,rv) \
	cmic_write_reg(unit,TBYTr_ADDR(port),rv)
#define GET_TBYTr_COUNT(rv)             ((rv)>>0&0xffffffff)
#define SET_TBYTr_COUNT(rv,fv)          (rv=(rv&~(0xffffffff<<0))|((fv)&0xffffffff)<<0)

#define TDFRr_ADDR(port)                reg_addr(TDFRr,port)
#define READ_TDFRr(unit,port,rvp) \
	cmic_read_reg(unit,TDFRr_ADDR(port),rvp)
#define WRITE_TDFRr(unit,port,rv) \
	cmic_write_reg(unit,TDFRr_ADDR(port),rv)
#define GET_TDFRr_COUNT(rv)             ((rv)>>0&0xfff)
#define SET_TDFRr_COUNT(rv,fv)          (rv=(rv&~(0xfff<<0))|((fv)&0xfff)<<0)

#define TEDFr_ADDR(port)                reg_addr(TEDFr,port)
#define READ_TEDFr(unit,port,rvp) \
	cmic_read_reg(unit,TEDFr_ADDR(port),rvp)
#define WRITE_TEDFr(unit,port,rv) \
	cmic_write_reg(unit,TEDFr_ADDR(port),rv)
#define GET_TEDFr_COUNT(rv)             ((rv)>>0&0xfff)
#define SET_TEDFr_COUNT(rv,fv)          (rv=(rv&~(0xfff<<0))|((fv)&0xfff)<<0)

#define TEST2r_ADDR(port)               reg_addr(TEST2r,port)
#define READ_TEST2r(unit,port,rvp) \
	cmic_read_reg(unit,TEST2r_ADDR(port),rvp)
#define WRITE_TEST2r(unit,port,rv) \
	cmic_write_reg(unit,TEST2r_ADDR(port),rv)
#define GET_TEST2r_SPQCT(rv)            ((rv)>>0&0x1)
#define SET_TEST2r_SPQCT(rv,fv)         (rv=(rv&~(0x1<<0))|((fv)&0x1)<<0)
#define GET_TEST2r_TPAUSE(rv)           ((rv)>>1&0x1)
#define SET_TEST2r_TPAUSE(rv,fv)        (rv=(rv&~(0x1<<1))|((fv)&0x1)<<1)

#define TFCSr_ADDR(port)                reg_addr(TFCSr,port)
#define READ_TFCSr(unit,port,rvp) \
	cmic_read_reg(unit,TFCSr_ADDR(port),rvp)
#define WRITE_TFCSr(unit,port,rv) \
	cmic_write_reg(unit,TFCSr_ADDR(port),rv)
#define GET_TFCSr_COUNT(rv)             ((rv)>>0&0xfff)
#define SET_TFCSr_COUNT(rv,fv)          (rv=(rv&~(0xfff<<0))|((fv)&0xfff)<<0)

#define TFRGr_ADDR(port)                reg_addr(TFRGr,port)
#define READ_TFRGr(unit,port,rvp) \
	cmic_read_reg(unit,TFRGr_ADDR(port),rvp)
#define WRITE_TFRGr(unit,port,rv) \
	cmic_write_reg(unit,TFRGr_ADDR(port),rv)
#define GET_TFRGr_COUNT(rv)             ((rv)>>0&0xfff)
#define SET_TFRGr_COUNT(rv,fv)          (rv=(rv&~(0xfff<<0))|((fv)&0xfff)<<0)

#define TIPr_ADDR(port)                 reg_addr(TIPr,port)
#define READ_TIPr(unit,port,rvp) \
	cmic_read_reg(unit,TIPr_ADDR(port),rvp)
#define WRITE_TIPr(unit,port,rv) \
	cmic_write_reg(unit,TIPr_ADDR(port),rv)
#define GET_TIPr_COUNT(rv)              ((rv)>>0&0xfffff)
#define SET_TIPr_COUNT(rv,fv)           (rv=(rv&~(0xfffff<<0))|((fv)&0xfffff)<<0)

#define TIPDr_ADDR(port)                reg_addr(TIPDr,port)
#define READ_TIPDr(unit,port,rvp) \
	cmic_read_reg(unit,TIPDr_ADDR(port),rvp)
#define WRITE_TIPDr(unit,port,rv) \
	cmic_write_reg(unit,TIPDr_ADDR(port),rv)
#define GET_TIPDr_COUNT(rv)             ((rv)>>0&0xfff)
#define SET_TIPDr_COUNT(rv,fv)          (rv=(rv&~(0xfff<<0))|((fv)&0xfff)<<0)

#define TIPXr_ADDR(port)                reg_addr(TIPXr,port)
#define READ_TIPXr(unit,port,rvp) \
	cmic_read_reg(unit,TIPXr_ADDR(port),rvp)
#define WRITE_TIPXr(unit,port,rv) \
	cmic_write_reg(unit,TIPXr_ADDR(port),rv)
#define GET_TIPXr_COUNT(rv)             ((rv)>>0&0x1)
#define SET_TIPXr_COUNT(rv,fv)          (rv=(rv&~(0x1<<0))|((fv)&0x1)<<0)

#define TJBRr_ADDR(port)                reg_addr(TJBRr,port)
#define READ_TJBRr(unit,port,rvp) \
	cmic_read_reg(unit,TJBRr_ADDR(port),rvp)
#define WRITE_TJBRr(unit,port,rv) \
	cmic_write_reg(unit,TJBRr_ADDR(port),rv)
#define GET_TJBRr_COUNT(rv)             ((rv)>>0&0xfff)
#define SET_TJBRr_COUNT(rv,fv)          (rv=(rv&~(0xfff<<0))|((fv)&0xfff)<<0)

#define TLCLr_ADDR(port)                reg_addr(TLCLr,port)
#define READ_TLCLr(unit,port,rvp) \
	cmic_read_reg(unit,TLCLr_ADDR(port),rvp)
#define WRITE_TLCLr(unit,port,rv) \
	cmic_write_reg(unit,TLCLr_ADDR(port),rv)
#define GET_TLCLr_COUNT(rv)             ((rv)>>0&0xfff)
#define SET_TLCLr_COUNT(rv,fv)          (rv=(rv&~(0xfff<<0))|((fv)&0xfff)<<0)

#define TMCAr_ADDR(port)                reg_addr(TMCAr,port)
#define READ_TMCAr(unit,port,rvp) \
	cmic_read_reg(unit,TMCAr_ADDR(port),rvp)
#define WRITE_TMCAr(unit,port,rv) \
	cmic_write_reg(unit,TMCAr_ADDR(port),rv)
#define GET_TMCAr_COUNT(rv)             ((rv)>>0&0xfffff)
#define SET_TMCAr_COUNT(rv,fv)          (rv=(rv&~(0xfffff<<0))|((fv)&0xfffff)<<0)

#define TMCLr_ADDR(port)                reg_addr(TMCLr,port)
#define READ_TMCLr(unit,port,rvp) \
	cmic_read_reg(unit,TMCLr_ADDR(port),rvp)
#define WRITE_TMCLr(unit,port,rv) \
	cmic_write_reg(unit,TMCLr_ADDR(port),rv)
#define GET_TMCLr_COUNT(rv)             ((rv)>>0&0xfff)
#define SET_TMCLr_COUNT(rv,fv)          (rv=(rv&~(0xfff<<0))|((fv)&0xfff)<<0)

#define TNCLr_ADDR(port)                reg_addr(TNCLr,port)
#define READ_TNCLr(unit,port,rvp) \
	cmic_read_reg(unit,TNCLr_ADDR(port),rvp)
#define WRITE_TNCLr(unit,port,rv) \
	cmic_write_reg(unit,TNCLr_ADDR(port),rv)
#define GET_TNCLr_COUNT(rv)             ((rv)>>0&0xfff)
#define SET_TNCLr_COUNT(rv,fv)          (rv=(rv&~(0xfff<<0))|((fv)&0xfff)<<0)

#define TOVRr_ADDR(port)                reg_addr(TOVRr,port)
#define READ_TOVRr(unit,port,rvp) \
	cmic_read_reg(unit,TOVRr_ADDR(port),rvp)
#define WRITE_TOVRr(unit,port,rv) \
	cmic_write_reg(unit,TOVRr_ADDR(port),rv)
#define GET_TOVRr_COUNT(rv)             ((rv)>>0&0xfff)
#define SET_TOVRr_COUNT(rv,fv)          (rv=(rv&~(0xfff<<0))|((fv)&0xfff)<<0)

#define TPKTr_ADDR(port)                reg_addr(TPKTr,port)
#define READ_TPKTr(unit,port,rvp) \
	cmic_read_reg(unit,TPKTr_ADDR(port),rvp)
#define WRITE_TPKTr(unit,port,rv) \
	cmic_write_reg(unit,TPKTr_ADDR(port),rv)
#define GET_TPKTr_COUNT(rv)             ((rv)>>0&0xfffff)
#define SET_TPKTr_COUNT(rv,fv)          (rv=(rv&~(0xfffff<<0))|((fv)&0xfffff)<<0)

#define TR1023r_ADDR(port)              reg_addr(TR1023r,port)
#define READ_TR1023r(unit,port,rvp) \
	cmic_read_reg(unit,TR1023r_ADDR(port),rvp)
#define WRITE_TR1023r(unit,port,rv) \
	cmic_write_reg(unit,TR1023r_ADDR(port),rv)
#define GET_TR1023r_COUNT(rv)           ((rv)>>0&0xfffff)
#define SET_TR1023r_COUNT(rv,fv)        (rv=(rv&~(0xfffff<<0))|((fv)&0xfffff)<<0)

#define TR127r_ADDR(port)               reg_addr(TR127r,port)
#define READ_TR127r(unit,port,rvp) \
	cmic_read_reg(unit,TR127r_ADDR(port),rvp)
#define WRITE_TR127r(unit,port,rv) \
	cmic_write_reg(unit,TR127r_ADDR(port),rv)
#define GET_TR127r_COUNT(rv)            ((rv)>>0&0xfffff)
#define SET_TR127r_COUNT(rv,fv)         (rv=(rv&~(0xfffff<<0))|((fv)&0xfffff)<<0)

#define TR255r_ADDR(port)               reg_addr(TR255r,port)
#define READ_TR255r(unit,port,rvp) \
	cmic_read_reg(unit,TR255r_ADDR(port),rvp)
#define WRITE_TR255r(unit,port,rv) \
	cmic_write_reg(unit,TR255r_ADDR(port),rv)
#define GET_TR255r_COUNT(rv)            ((rv)>>0&0xfffff)
#define SET_TR255r_COUNT(rv,fv)         (rv=(rv&~(0xfffff<<0))|((fv)&0xfffff)<<0)

#define TR511r_ADDR(port)               reg_addr(TR511r,port)
#define READ_TR511r(unit,port,rvp) \
	cmic_read_reg(unit,TR511r_ADDR(port),rvp)
#define WRITE_TR511r(unit,port,rv) \
	cmic_write_reg(unit,TR511r_ADDR(port),rv)
#define GET_TR511r_COUNT(rv)            ((rv)>>0&0xfffff)
#define SET_TR511r_COUNT(rv,fv)         (rv=(rv&~(0xfffff<<0))|((fv)&0xfffff)<<0)

#define TR64r_ADDR(port)                reg_addr(TR64r,port)
#define READ_TR64r(unit,port,rvp) \
	cmic_read_reg(unit,TR64r_ADDR(port),rvp)
#define WRITE_TR64r(unit,port,rv) \
	cmic_write_reg(unit,TR64r_ADDR(port),rv)
#define GET_TR64r_COUNT(rv)             ((rv)>>0&0xfffff)
#define SET_TR64r_COUNT(rv,fv)          (rv=(rv&~(0xfffff<<0))|((fv)&0xfffff)<<0)

#define TRMAXr_ADDR(port)               reg_addr(TRMAXr,port)
#define READ_TRMAXr(unit,port,rvp) \
	cmic_read_reg(unit,TRMAXr_ADDR(port),rvp)
#define WRITE_TRMAXr(unit,port,rv) \
	cmic_write_reg(unit,TRMAXr_ADDR(port),rv)
#define GET_TRMAXr_COUNT(rv)            ((rv)>>0&0xfffff)
#define SET_TRMAXr_COUNT(rv,fv)         (rv=(rv&~(0xfffff<<0))|((fv)&0xfffff)<<0)

#define TRMGVr_ADDR(port)               reg_addr(TRMGVr,port)
#define READ_TRMGVr(unit,port,rvp) \
	cmic_read_reg(unit,TRMGVr_ADDR(port),rvp)
#define WRITE_TRMGVr(unit,port,rv) \
	cmic_write_reg(unit,TRMGVr_ADDR(port),rv)
#define GET_TRMGVr_COUNT(rv)            ((rv)>>0&0xfffff)
#define SET_TRMGVr_COUNT(rv,fv)         (rv=(rv&~(0xfffff<<0))|((fv)&0xfffff)<<0)

#define TSCLr_ADDR(port)                reg_addr(TSCLr,port)
#define READ_TSCLr(unit,port,rvp) \
	cmic_read_reg(unit,TSCLr_ADDR(port),rvp)
#define WRITE_TSCLr(unit,port,rv) \
	cmic_write_reg(unit,TSCLr_ADDR(port),rv)
#define GET_TSCLr_COUNT(rv)             ((rv)>>0&0xfff)
#define SET_TSCLr_COUNT(rv,fv)          (rv=(rv&~(0xfff<<0))|((fv)&0xfff)<<0)

#define TVLANr_ADDR(port)               reg_addr(TVLANr,port)
#define READ_TVLANr(unit,port,rvp) \
	cmic_read_reg(unit,TVLANr_ADDR(port),rvp)
#define WRITE_TVLANr(unit,port,rv) \
	cmic_write_reg(unit,TVLANr_ADDR(port),rv)
#define GET_TVLANr_COUNT(rv)            ((rv)>>0&0xfffff)
#define SET_TVLANr_COUNT(rv,fv)         (rv=(rv&~(0xfffff<<0))|((fv)&0xfffff)<<0)

#define TXCFr_ADDR(port)                reg_addr(TXCFr,port)
#define READ_TXCFr(unit,port,rvp) \
	cmic_read_reg(unit,TXCFr_ADDR(port),rvp)
#define WRITE_TXCFr(unit,port,rv) \
	cmic_write_reg(unit,TXCFr_ADDR(port),rv)
#define GET_TXCFr_COUNT(rv)             ((rv)>>0&0xfff)
#define SET_TXCFr_COUNT(rv,fv)          (rv=(rv&~(0xfff<<0))|((fv)&0xfff)<<0)

#define TXCLr_ADDR(port)                reg_addr(TXCLr,port)
#define READ_TXCLr(unit,port,rvp) \
	cmic_read_reg(unit,TXCLr_ADDR(port),rvp)
#define WRITE_TXCLr(unit,port,rv) \
	cmic_write_reg(unit,TXCLr_ADDR(port),rv)
#define GET_TXCLr_COUNT(rv)             ((rv)>>0&0xfff)
#define SET_TXCLr_COUNT(rv,fv)          (rv=(rv&~(0xfff<<0))|((fv)&0xfff)<<0)

#define TXPFr_ADDR(port)                reg_addr(TXPFr,port)
#define READ_TXPFr(unit,port,rvp) \
	cmic_read_reg(unit,TXPFr_ADDR(port),rvp)
#define WRITE_TXPFr(unit,port,rv) \
	cmic_write_reg(unit,TXPFr_ADDR(port),rv)
#define GET_TXPFr_COUNT(rv)             ((rv)>>0&0xfff)
#define SET_TXPFr_COUNT(rv,fv)          (rv=(rv&~(0xfff<<0))|((fv)&0xfff)<<0)

#define XQEMPTYr_ADDR(port)             reg_addr(XQEMPTYr,port)
#define READ_XQEMPTYr(unit,port,rvp) \
	cmic_read_reg(unit,XQEMPTYr_ADDR(port),rvp)
#define WRITE_XQEMPTYr(unit,port,rv) \
	cmic_write_reg(unit,XQEMPTYr_ADDR(port),rv)
#define GET_XQEMPTYr_COS0(rv)           ((rv)>>0&0x1)
#define SET_XQEMPTYr_COS0(rv,fv)        (rv=(rv&~(0x1<<0))|((fv)&0x1)<<0)
#define GET_XQEMPTYr_COS1(rv)           ((rv)>>1&0x1)
#define SET_XQEMPTYr_COS1(rv,fv)        (rv=(rv&~(0x1<<1))|((fv)&0x1)<<1)
#define GET_XQEMPTYr_COS2(rv)           ((rv)>>2&0x1)
#define SET_XQEMPTYr_COS2(rv,fv)        (rv=(rv&~(0x1<<2))|((fv)&0x1)<<2)
#define GET_XQEMPTYr_COS3(rv)           ((rv)>>3&0x1)
#define SET_XQEMPTYr_COS3(rv,fv)        (rv=(rv&~(0x1<<3))|((fv)&0x1)<<3)


#define ARL_m_ADDR(blkidx,index)        mem_addr(ARL_m,blkidx,index)
#define ARL_m_BYTES                     11
#define ARL_m_WORDS                     BYTES2WORDS(ARL_m_BYTES)
#define ARL_m_COPIES                    3
#define READ_ARL_m(unit,blkidx,index,data) \
	cmic_read_memory(unit,ARL_m,blkidx,index,data)
#define WRITE_ARL_m(unit,blkidx,index,data) \
	cmic_write_memory(unit,ARL_m,blkidx,index,data)
#define ZERO_ARL_m(buf) \
	zero_mem_value(ARL_m,buf)
#define GET_ARL_m_COS_DST(entbuf,fldbuf) \
	get_mem_field(ARL_m,entbuf,COS_DSTf,fldbuf)
#define SET_ARL_m_COS_DST(entbuf,fldbuf) \
	set_mem_field(ARL_m,entbuf,COS_DSTf,fldbuf)
#define ARL_m_COS_DST_WORDS             1
#define GET_ARL_m_COS_SRC(entbuf,fldbuf) \
	get_mem_field(ARL_m,entbuf,COS_SRCf,fldbuf)
#define SET_ARL_m_COS_SRC(entbuf,fldbuf) \
	set_mem_field(ARL_m,entbuf,COS_SRCf,fldbuf)
#define ARL_m_COS_SRC_WORDS             1
#define GET_ARL_m_CPU(entbuf,fldbuf) \
	get_mem_field(ARL_m,entbuf,CPUf,fldbuf)
#define SET_ARL_m_CPU(entbuf,fldbuf) \
	set_mem_field(ARL_m,entbuf,CPUf,fldbuf)
#define ARL_m_CPU_WORDS                 1
#define GET_ARL_m_HIT(entbuf,fldbuf) \
	get_mem_field(ARL_m,entbuf,HITf,fldbuf)
#define SET_ARL_m_HIT(entbuf,fldbuf) \
	set_mem_field(ARL_m,entbuf,HITf,fldbuf)
#define ARL_m_HIT_WORDS                 1
#define GET_ARL_m_L3(entbuf,fldbuf) \
	get_mem_field(ARL_m,entbuf,L3f,fldbuf)
#define SET_ARL_m_L3(entbuf,fldbuf) \
	set_mem_field(ARL_m,entbuf,L3f,fldbuf)
#define ARL_m_L3_WORDS                  1
#define GET_ARL_m_MAC_ADDR(entbuf,fldbuf) \
	get_mem_field(ARL_m,entbuf,MAC_ADDRf,fldbuf)
#define SET_ARL_m_MAC_ADDR(entbuf,fldbuf) \
	set_mem_field(ARL_m,entbuf,MAC_ADDRf,fldbuf)
#define ARL_m_MAC_ADDR_WORDS            2
#define GET_ARL_m_PORT(entbuf,fldbuf) \
	get_mem_field(ARL_m,entbuf,PORTf,fldbuf)
#define SET_ARL_m_PORT(entbuf,fldbuf) \
	set_mem_field(ARL_m,entbuf,PORTf,fldbuf)
#define ARL_m_PORT_WORDS                1
#define GET_ARL_m_RTAG(entbuf,fldbuf) \
	get_mem_field(ARL_m,entbuf,RTAGf,fldbuf)
#define SET_ARL_m_RTAG(entbuf,fldbuf) \
	set_mem_field(ARL_m,entbuf,RTAGf,fldbuf)
#define ARL_m_RTAG_WORDS                1
#define GET_ARL_m_SCP(entbuf,fldbuf) \
	get_mem_field(ARL_m,entbuf,SCPf,fldbuf)
#define SET_ARL_m_SCP(entbuf,fldbuf) \
	set_mem_field(ARL_m,entbuf,SCPf,fldbuf)
#define ARL_m_SCP_WORDS                 1
#define GET_ARL_m_SD_DIS(entbuf,fldbuf) \
	get_mem_field(ARL_m,entbuf,SD_DISf,fldbuf)
#define SET_ARL_m_SD_DIS(entbuf,fldbuf) \
	set_mem_field(ARL_m,entbuf,SD_DISf,fldbuf)
#define ARL_m_SD_DIS_WORDS              1
#define GET_ARL_m_ST(entbuf,fldbuf) \
	get_mem_field(ARL_m,entbuf,STf,fldbuf)
#define SET_ARL_m_ST(entbuf,fldbuf) \
	set_mem_field(ARL_m,entbuf,STf,fldbuf)
#define ARL_m_ST_WORDS                  1
#define GET_ARL_m_TGID(entbuf,fldbuf) \
	get_mem_field(ARL_m,entbuf,TGIDf,fldbuf)
#define SET_ARL_m_TGID(entbuf,fldbuf) \
	set_mem_field(ARL_m,entbuf,TGIDf,fldbuf)
#define ARL_m_TGID_WORDS                1
#define GET_ARL_m_TRUNK(entbuf,fldbuf) \
	get_mem_field(ARL_m,entbuf,TRUNKf,fldbuf)
#define SET_ARL_m_TRUNK(entbuf,fldbuf) \
	set_mem_field(ARL_m,entbuf,TRUNKf,fldbuf)
#define ARL_m_TRUNK_WORDS               1
#define GET_ARL_m_VLAN_TAG(entbuf,fldbuf) \
	get_mem_field(ARL_m,entbuf,VLAN_TAGf,fldbuf)
#define SET_ARL_m_VLAN_TAG(entbuf,fldbuf) \
	set_mem_field(ARL_m,entbuf,VLAN_TAGf,fldbuf)
#define ARL_m_VLAN_TAG_WORDS            1

#define ARL_CNT_m_ADDR(blkidx,index)    mem_addr(ARL_CNT_m,blkidx,index)
#define ARL_CNT_m_BYTES                 2
#define ARL_CNT_m_WORDS                 BYTES2WORDS(ARL_CNT_m_BYTES)
#define ARL_CNT_m_COPIES                3
#define READ_ARL_CNT_m(unit,blkidx,index,data) \
	cmic_read_memory(unit,ARL_CNT_m,blkidx,index,data)
#define WRITE_ARL_CNT_m(unit,blkidx,index,data) \
	cmic_write_memory(unit,ARL_CNT_m,blkidx,index,data)
#define ZERO_ARL_CNT_m(buf) \
	zero_mem_value(ARL_CNT_m,buf)
#define GET_ARL_CNT_m_LEARN_CNT(entbuf,fldbuf) \
	get_mem_field(ARL_CNT_m,entbuf,LEARN_CNTf,fldbuf)
#define SET_ARL_CNT_m_LEARN_CNT(entbuf,fldbuf) \
	set_mem_field(ARL_CNT_m,entbuf,LEARN_CNTf,fldbuf)
#define ARL_CNT_m_LEARN_CNT_WORDS       1

#define GBPm_ADDR(index)                mem_addr(GBPm,index)
#define GBPm_BYTES                      16
#define GBPm_WORDS                      BYTES2WORDS(GBPm_BYTES)
#define READ_GBPm(unit,index,data) \
	cmic_read_memory(unit,GBPm,0,index,data)
#define WRITE_GBPm(unit,index,data) \
	cmic_write_memory(unit,GBPm,0,index,data)
#define ZERO_GBPm(buf) \
	zero_mem_value(GBPm,buf)
#define GET_GBPm_FIELD(entbuf,fldbuf) \
	get_mem_field(GBPm,entbuf,FIELDf,fldbuf)
#define SET_GBPm_FIELD(entbuf,fldbuf) \
	set_mem_field(GBPm,entbuf,FIELDf,fldbuf)
#define GBPm_FIELD_WORDS                4

#define MARL_m_ADDR(blkidx,index)       mem_addr(MARL_m,blkidx,index)
#define MARL_m_BYTES                    16
#define MARL_m_WORDS                    BYTES2WORDS(MARL_m_BYTES)
#define MARL_m_COPIES                   3
#define READ_MARL_m(unit,blkidx,index,data) \
	cmic_read_memory(unit,MARL_m,blkidx,index,data)
#define WRITE_MARL_m(unit,blkidx,index,data) \
	cmic_write_memory(unit,MARL_m,blkidx,index,data)
#define ZERO_MARL_m(buf) \
	zero_mem_value(MARL_m,buf)
#define GET_MARL_m_COS_DST(entbuf,fldbuf) \
	get_mem_field(MARL_m,entbuf,COS_DSTf,fldbuf)
#define SET_MARL_m_COS_DST(entbuf,fldbuf) \
	set_mem_field(MARL_m,entbuf,COS_DSTf,fldbuf)
#define MARL_m_COS_DST_WORDS            1
#define GET_MARL_m_MAC_ADDR(entbuf,fldbuf) \
	get_mem_field(MARL_m,entbuf,MAC_ADDRf,fldbuf)
#define SET_MARL_m_MAC_ADDR(entbuf,fldbuf) \
	set_mem_field(MARL_m,entbuf,MAC_ADDRf,fldbuf)
#define MARL_m_MAC_ADDR_WORDS           2
#define GET_MARL_m_PORT_BITMAP(entbuf,fldbuf) \
	get_mem_field(MARL_m,entbuf,PORT_BITMAPf,fldbuf)
#define SET_MARL_m_PORT_BITMAP(entbuf,fldbuf) \
	set_mem_field(MARL_m,entbuf,PORT_BITMAPf,fldbuf)
#define MARL_m_PORT_BITMAP_WORDS        1
#define GET_MARL_m_UT_PORT_BITMAP(entbuf,fldbuf) \
	get_mem_field(MARL_m,entbuf,UT_PORT_BITMAPf,fldbuf)
#define SET_MARL_m_UT_PORT_BITMAP(entbuf,fldbuf) \
	set_mem_field(MARL_m,entbuf,UT_PORT_BITMAPf,fldbuf)
#define MARL_m_UT_PORT_BITMAP_WORDS     1
#define GET_MARL_m_VLAN_TAG(entbuf,fldbuf) \
	get_mem_field(MARL_m,entbuf,VLAN_TAGf,fldbuf)
#define SET_MARL_m_VLAN_TAG(entbuf,fldbuf) \
	set_mem_field(MARL_m,entbuf,VLAN_TAGf,fldbuf)
#define MARL_m_VLAN_TAG_WORDS           1

#define PTABLE_m_ADDR(blkidx,index)     mem_addr(PTABLE_m,blkidx,index)
#define PTABLE_m_BYTES                  12
#define PTABLE_m_WORDS                  BYTES2WORDS(PTABLE_m_BYTES)
#define PTABLE_m_COPIES                 3
#define READ_PTABLE_m(unit,blkidx,index,data) \
	cmic_read_memory(unit,PTABLE_m,blkidx,index,data)
#define WRITE_PTABLE_m(unit,blkidx,index,data) \
	cmic_write_memory(unit,PTABLE_m,blkidx,index,data)
#define ZERO_PTABLE_m(buf) \
	zero_mem_value(PTABLE_m,buf)
#define GET_PTABLE_m_BPDU(entbuf,fldbuf) \
	get_mem_field(PTABLE_m,entbuf,BPDUf,fldbuf)
#define SET_PTABLE_m_BPDU(entbuf,fldbuf) \
	set_mem_field(PTABLE_m,entbuf,BPDUf,fldbuf)
#define PTABLE_m_BPDU_WORDS             1
#define GET_PTABLE_m_CML(entbuf,fldbuf) \
	get_mem_field(PTABLE_m,entbuf,CMLf,fldbuf)
#define SET_PTABLE_m_CML(entbuf,fldbuf) \
	set_mem_field(PTABLE_m,entbuf,CMLf,fldbuf)
#define PTABLE_m_CML_WORDS              1
#define GET_PTABLE_m_CPU(entbuf,fldbuf) \
	get_mem_field(PTABLE_m,entbuf,CPUf,fldbuf)
#define SET_PTABLE_m_CPU(entbuf,fldbuf) \
	set_mem_field(PTABLE_m,entbuf,CPUf,fldbuf)
#define PTABLE_m_CPU_WORDS              1
#define GET_PTABLE_m_JUMBO(entbuf,fldbuf) \
	get_mem_field(PTABLE_m,entbuf,JUMBOf,fldbuf)
#define SET_PTABLE_m_JUMBO(entbuf,fldbuf) \
	set_mem_field(PTABLE_m,entbuf,JUMBOf,fldbuf)
#define PTABLE_m_JUMBO_WORDS            1
#define GET_PTABLE_m_MI(entbuf,fldbuf) \
	get_mem_field(PTABLE_m,entbuf,MIf,fldbuf)
#define SET_PTABLE_m_MI(entbuf,fldbuf) \
	set_mem_field(PTABLE_m,entbuf,MIf,fldbuf)
#define PTABLE_m_MI_WORDS               1
#define GET_PTABLE_m_PORT_BITMAP(entbuf,fldbuf) \
	get_mem_field(PTABLE_m,entbuf,PORT_BITMAPf,fldbuf)
#define SET_PTABLE_m_PORT_BITMAP(entbuf,fldbuf) \
	set_mem_field(PTABLE_m,entbuf,PORT_BITMAPf,fldbuf)
#define PTABLE_m_PORT_BITMAP_WORDS      1
#define GET_PTABLE_m_PRT_DIS(entbuf,fldbuf) \
	get_mem_field(PTABLE_m,entbuf,PRT_DISf,fldbuf)
#define SET_PTABLE_m_PRT_DIS(entbuf,fldbuf) \
	set_mem_field(PTABLE_m,entbuf,PRT_DISf,fldbuf)
#define PTABLE_m_PRT_DIS_WORDS          1
#define GET_PTABLE_m_PTYPE(entbuf,fldbuf) \
	get_mem_field(PTABLE_m,entbuf,PTYPEf,fldbuf)
#define SET_PTABLE_m_PTYPE(entbuf,fldbuf) \
	set_mem_field(PTABLE_m,entbuf,PTYPEf,fldbuf)
#define PTABLE_m_PTYPE_WORDS            1
#define GET_PTABLE_m_RTAG(entbuf,fldbuf) \
	get_mem_field(PTABLE_m,entbuf,RTAGf,fldbuf)
#define SET_PTABLE_m_RTAG(entbuf,fldbuf) \
	set_mem_field(PTABLE_m,entbuf,RTAGf,fldbuf)
#define PTABLE_m_RTAG_WORDS             1
#define GET_PTABLE_m_SP_ST(entbuf,fldbuf) \
	get_mem_field(PTABLE_m,entbuf,SP_STf,fldbuf)
#define SET_PTABLE_m_SP_ST(entbuf,fldbuf) \
	set_mem_field(PTABLE_m,entbuf,SP_STf,fldbuf)
#define PTABLE_m_SP_ST_WORDS            1
#define GET_PTABLE_m_TGID(entbuf,fldbuf) \
	get_mem_field(PTABLE_m,entbuf,TGIDf,fldbuf)
#define SET_PTABLE_m_TGID(entbuf,fldbuf) \
	set_mem_field(PTABLE_m,entbuf,TGIDf,fldbuf)
#define PTABLE_m_TGID_WORDS             1
#define GET_PTABLE_m_TRUNK(entbuf,fldbuf) \
	get_mem_field(PTABLE_m,entbuf,TRUNKf,fldbuf)
#define SET_PTABLE_m_TRUNK(entbuf,fldbuf) \
	set_mem_field(PTABLE_m,entbuf,TRUNKf,fldbuf)
#define PTABLE_m_TRUNK_WORDS            1
#define GET_PTABLE_m_UT_PORT_BITMAP(entbuf,fldbuf) \
	get_mem_field(PTABLE_m,entbuf,UT_PORT_BITMAPf,fldbuf)
#define SET_PTABLE_m_UT_PORT_BITMAP(entbuf,fldbuf) \
	set_mem_field(PTABLE_m,entbuf,UT_PORT_BITMAPf,fldbuf)
#define PTABLE_m_UT_PORT_BITMAP_WORDS   1
#define GET_PTABLE_m_VLAN_TAG(entbuf,fldbuf) \
	get_mem_field(PTABLE_m,entbuf,VLAN_TAGf,fldbuf)
#define SET_PTABLE_m_VLAN_TAG(entbuf,fldbuf) \
	set_mem_field(PTABLE_m,entbuf,VLAN_TAGf,fldbuf)
#define PTABLE_m_VLAN_TAG_WORDS         1

#define TTABLE_TBMAPm_ADDR(blkidx,index) mem_addr(TTABLE_TBMAPm,blkidx,index)
#define TTABLE_TBMAPm_BYTES             4
#define TTABLE_TBMAPm_WORDS             BYTES2WORDS(TTABLE_TBMAPm_BYTES)
#define TTABLE_TBMAPm_COPIES            3
#define READ_TTABLE_TBMAPm(unit,blkidx,index,data) \
	cmic_read_memory(unit,TTABLE_TBMAPm,blkidx,index,data)
#define WRITE_TTABLE_TBMAPm(unit,blkidx,index,data) \
	cmic_write_memory(unit,TTABLE_TBMAPm,blkidx,index,data)
#define ZERO_TTABLE_TBMAPm(buf) \
	zero_mem_value(TTABLE_TBMAPm,buf)
#define GET_TTABLE_TBMAPm_TBMAP(entbuf,fldbuf) \
	get_mem_field(TTABLE_TBMAPm,entbuf,TBMAPf,fldbuf)
#define SET_TTABLE_TBMAPm_TBMAP(entbuf,fldbuf) \
	set_mem_field(TTABLE_TBMAPm,entbuf,TBMAPf,fldbuf)
#define TTABLE_TBMAPm_TBMAP_WORDS       1

#define TTABLE_TTRm_ADDR(blkidx,index)  mem_addr(TTABLE_TTRm,blkidx,index)
#define TTABLE_TTRm_BYTES               7
#define TTABLE_TTRm_WORDS               BYTES2WORDS(TTABLE_TTRm_BYTES)
#define TTABLE_TTRm_COPIES              3
#define READ_TTABLE_TTRm(unit,blkidx,index,data) \
	cmic_read_memory(unit,TTABLE_TTRm,blkidx,index,data)
#define WRITE_TTABLE_TTRm(unit,blkidx,index,data) \
	cmic_write_memory(unit,TTABLE_TTRm,blkidx,index,data)
#define ZERO_TTABLE_TTRm(buf) \
	zero_mem_value(TTABLE_TTRm,buf)
#define GET_TTABLE_TTRm_TG_SIZE(entbuf,fldbuf) \
	get_mem_field(TTABLE_TTRm,entbuf,TG_SIZEf,fldbuf)
#define SET_TTABLE_TTRm_TG_SIZE(entbuf,fldbuf) \
	set_mem_field(TTABLE_TTRm,entbuf,TG_SIZEf,fldbuf)
#define TTABLE_TTRm_TG_SIZE_WORDS       1
#define GET_TTABLE_TTRm_TP0(entbuf,fldbuf) \
	get_mem_field(TTABLE_TTRm,entbuf,TP0f,fldbuf)
#define SET_TTABLE_TTRm_TP0(entbuf,fldbuf) \
	set_mem_field(TTABLE_TTRm,entbuf,TP0f,fldbuf)
#define TTABLE_TTRm_TP0_WORDS           1
#define GET_TTABLE_TTRm_TP1(entbuf,fldbuf) \
	get_mem_field(TTABLE_TTRm,entbuf,TP1f,fldbuf)
#define SET_TTABLE_TTRm_TP1(entbuf,fldbuf) \
	set_mem_field(TTABLE_TTRm,entbuf,TP1f,fldbuf)
#define TTABLE_TTRm_TP1_WORDS           1
#define GET_TTABLE_TTRm_TP2(entbuf,fldbuf) \
	get_mem_field(TTABLE_TTRm,entbuf,TP2f,fldbuf)
#define SET_TTABLE_TTRm_TP2(entbuf,fldbuf) \
	set_mem_field(TTABLE_TTRm,entbuf,TP2f,fldbuf)
#define TTABLE_TTRm_TP2_WORDS           1
#define GET_TTABLE_TTRm_TP3(entbuf,fldbuf) \
	get_mem_field(TTABLE_TTRm,entbuf,TP3f,fldbuf)
#define SET_TTABLE_TTRm_TP3(entbuf,fldbuf) \
	set_mem_field(TTABLE_TTRm,entbuf,TP3f,fldbuf)
#define TTABLE_TTRm_TP3_WORDS           1
#define GET_TTABLE_TTRm_TP4(entbuf,fldbuf) \
	get_mem_field(TTABLE_TTRm,entbuf,TP4f,fldbuf)
#define SET_TTABLE_TTRm_TP4(entbuf,fldbuf) \
	set_mem_field(TTABLE_TTRm,entbuf,TP4f,fldbuf)
#define TTABLE_TTRm_TP4_WORDS           1
#define GET_TTABLE_TTRm_TP5(entbuf,fldbuf) \
	get_mem_field(TTABLE_TTRm,entbuf,TP5f,fldbuf)
#define SET_TTABLE_TTRm_TP5(entbuf,fldbuf) \
	set_mem_field(TTABLE_TTRm,entbuf,TP5f,fldbuf)
#define TTABLE_TTRm_TP5_WORDS           1
#define GET_TTABLE_TTRm_TP6(entbuf,fldbuf) \
	get_mem_field(TTABLE_TTRm,entbuf,TP6f,fldbuf)
#define SET_TTABLE_TTRm_TP6(entbuf,fldbuf) \
	set_mem_field(TTABLE_TTRm,entbuf,TP6f,fldbuf)
#define TTABLE_TTRm_TP6_WORDS           1
#define GET_TTABLE_TTRm_TP7(entbuf,fldbuf) \
	get_mem_field(TTABLE_TTRm,entbuf,TP7f,fldbuf)
#define SET_TTABLE_TTRm_TP7(entbuf,fldbuf) \
	set_mem_field(TTABLE_TTRm,entbuf,TP7f,fldbuf)
#define TTABLE_TTRm_TP7_WORDS           1

#define VTABLE_m_ADDR(blkidx,index)     mem_addr(VTABLE_m,blkidx,index)
#define VTABLE_m_BYTES                  12
#define VTABLE_m_WORDS                  BYTES2WORDS(VTABLE_m_BYTES)
#define VTABLE_m_COPIES                 3
#define READ_VTABLE_m(unit,blkidx,index,data) \
	cmic_read_memory(unit,VTABLE_m,blkidx,index,data)
#define WRITE_VTABLE_m(unit,blkidx,index,data) \
	cmic_write_memory(unit,VTABLE_m,blkidx,index,data)
#define ZERO_VTABLE_m(buf) \
	zero_mem_value(VTABLE_m,buf)
#define GET_VTABLE_m_PORT_BITMAP(entbuf,fldbuf) \
	get_mem_field(VTABLE_m,entbuf,PORT_BITMAPf,fldbuf)
#define SET_VTABLE_m_PORT_BITMAP(entbuf,fldbuf) \
	set_mem_field(VTABLE_m,entbuf,PORT_BITMAPf,fldbuf)
#define VTABLE_m_PORT_BITMAP_WORDS      1
#define GET_VTABLE_m_UT_PORT_BITMAP(entbuf,fldbuf) \
	get_mem_field(VTABLE_m,entbuf,UT_PORT_BITMAPf,fldbuf)
#define SET_VTABLE_m_UT_PORT_BITMAP(entbuf,fldbuf) \
	set_mem_field(VTABLE_m,entbuf,UT_PORT_BITMAPf,fldbuf)
#define VTABLE_m_UT_PORT_BITMAP_WORDS   1
#define GET_VTABLE_m_VLAN_TAG(entbuf,fldbuf) \
	get_mem_field(VTABLE_m,entbuf,VLAN_TAGf,fldbuf)
#define SET_VTABLE_m_VLAN_TAG(entbuf,fldbuf) \
	set_mem_field(VTABLE_m,entbuf,VLAN_TAGf,fldbuf)
#define VTABLE_m_VLAN_TAG_WORDS         1

#endif /* BCM5600 */

/*****************************************************************************
 **	TABLE ENTRY STRUCTURES
 ** Bitfield structures are an alternative to the SET_FOOm_BAR macros.
 ** Two versions are defined to support big- and little-endian CPUs;
 ** the user must define one of LE_HOST or BE_HOST.
 *****************************************************************************/

#ifdef BCM5600 
#ifndef _u32
#define _u32 unsigned int
#define _u32_local_def
#endif

typedef struct arl_entry_s {
#if defined(BCM5600_LE_HOST)
    _u32 mac_addr_b0:8;         /* MAC ADDR */
    _u32 mac_addr_b1:8;         /* MAC ADDR */
    _u32 mac_addr_b2:8;         /* MAC ADDR */
    _u32 mac_addr_b3:8;         /* MAC ADDR */
    _u32 mac_addr_b4:8;         /* MAC ADDR */
    _u32 mac_addr_b5:8;         /* MAC ADDR */
    _u32 vlan_tag:12;           /* VLAN Tag */
    _u32 cos_dst:3;             /* Destination COS */
    _u32 cpu:1;                 /* CPU bit */
    _u32 l3:1;                  /* L3 bit */
    _u32 sd_dis:2;              /* Source_Dest Discard */
    _u32 st:1;                  /* Static */
    _u32 hit:1;                 /* Hit bit */
    _u32 cos_src:3;             /* Source COS */
    _u32 trunk:1;               /* Trunk Bit */
    _u32 tgid:3;                /* Trunk group ID */
    _u32 rtag:3;                /* Trunk Select */
    _u32 port:6;                /* Port Number */
    _u32 scp:1;                 /* Source COS Priority */
    _u32 _pad0:10;              /* Reserved */
#else /* BCM5600_BE_HOST */
    _u32 mac_addr_b3:8;         /* MAC ADDR */
    _u32 mac_addr_b2:8;         /* MAC ADDR */
    _u32 mac_addr_b1:8;         /* MAC ADDR */
    _u32 mac_addr_b0:8;         /* MAC ADDR */
    _u32 cpu:1;                 /* CPU bit */
    _u32 cos_dst:3;             /* Destination COS */
    _u32 vlan_tag:12;           /* VLAN Tag */
    _u32 mac_addr_b5:8;         /* MAC ADDR */
    _u32 mac_addr_b4:8;         /* MAC ADDR */
    _u32 _pad0:10;              /* Reserved */
    _u32 scp:1;                 /* Source COS Priority */
    _u32 port:6;                /* Port Number */
    _u32 rtag:3;                /* Trunk Select */
    _u32 tgid:3;                /* Trunk group ID */
    _u32 trunk:1;               /* Trunk Bit */
    _u32 cos_src:3;             /* Source COS */
    _u32 hit:1;                 /* Hit bit */
    _u32 st:1;                  /* Static */
    _u32 sd_dis:2;              /* Source_Dest Discard */
    _u32 l3:1;                  /* L3 bit */
#endif /* BE_HOST */
} arl_entry_t;

typedef struct arl_cnt_entry_s {
#if defined(BCM5600_LE_HOST)
    _u32 learn_cnt:13;          /* LEARN CNT */
    _u32 _pad0:19;              /* Reserved */
#else /* BCM5600_BE_HOST */
    _u32 _pad0:19;              /* Reserved */
    _u32 learn_cnt:13;          /* LEARN CNT */
#endif /* BCM5600_BE_HOST */
} arl_cnt_entry_t;

/* NOTE: FILTER_IMASK has a word-reversed format */
typedef struct imask_entry_s {
#if defined(BCM5600_LE_HOST)
    _u32 reserved_word;         /* Reserved */
    _u32 egrmask:6;             /* Egress Mask */
    _u32 ingmask:6;             /* Ingress Mask */
    _u32 _pad0:20;              /* Reserved */
    _u32 fmask[16];
#else /* BCM5600_BE_HOST */
    _u32 reserved_word;         /* Reserved */
    _u32 _pad0:20;              /* Reserved */
    _u32 ingmask:6;             /* Ingress Mask */
    _u32 egrmask:6;             /* Egress Mask */
    _u32 fmask[16];
#endif /* BCM5600_BE_HOST */
} imask_entry_t;

/* NOTE: FILTER_IRULE has a word-reversed format */
typedef struct irule_entry_s {
#if defined(BCM5600_LE_HOST)
    _u32 pri:3;                 /* 802.1P PRI */
    _u32 _pad0:29;              /* Reserved */
    _u32 eport:6;               /* Egress Port */
    _u32 iport:6;               /* Ingress Port */
    _u32 fsel:3;                /* Filter Select */
    _u32 action:7;              /* Action */
    _u32 tos_p:4;               /* TOS */
    _u32 port:6;                /* Port Number */
    _u32 filter[16];
#else /* BCM5600_BE_HOST */
    _u32 _pad0:29;              /* Reserved */
    _u32 pri:3;                 /* 802.1P PRI */
    _u32 port:6;                /* Port Number */
    _u32 tos_p:4;               /* TOS */
    _u32 action:7;              /* Action */
    _u32 fsel:3;                /* Filter Select */
    _u32 iport:6;               /* Ingress Port */
    _u32 eport:6;               /* Egress Port */
    _u32 filter[16];
#endif /* BCM5600_BE_HOST */
} irule_entry_t;

typedef struct gbp_entry_s {
#if defined(BCM5600_LE_HOST)
    _u32 field[4];
#else /* BCM5600_BE_HOST */
    _u32 field[4];
#endif /* BCM5600_BE_HOST */
} gbp_entry_t;

typedef struct def_ip_entry_s {
#if defined(BCM5600_LE_HOST)
    _u32 sub_addr;              /* Subnet IP address */
    _u32 mac_addr_b0:8;         /* MAC Address */
    _u32 mac_addr_b1:8;         /* MAC Address */
    _u32 mac_addr_b2:8;         /* MAC Address */
    _u32 mac_addr_b3:8;         /* MAC Address */
    _u32 mac_addr_b4:8;         /* MAC Address */
    _u32 mac_addr_b5:8;         /* MAC Address */
    _u32 port:6;                /* Port Number */
    _u32 intf:5;                /* L3 Interface Number */
    _u32 subnet_len:5;          /* Subnet bit length */
    _u32 cpu:1;                 /* CPU Bit */
    _u32 _pad0:31;              /* Reserved */
#else /* BCM5600_BE_HOST */
    _u32 sub_addr;              /* Subnet IP address */
    _u32 mac_addr_b3:8;         /* MAC Address */
    _u32 mac_addr_b2:8;         /* MAC Address */
    _u32 mac_addr_b1:8;         /* MAC Address */
    _u32 mac_addr_b0:8;         /* MAC Address */
    _u32 subnet_len:5;          /* Subnet bit length */
    _u32 intf:5;                /* L3 Interface Number */
    _u32 port:6;                /* Port Number */
    _u32 mac_addr_b5:8;         /* MAC Address */
    _u32 mac_addr_b4:8;         /* MAC Address */
    _u32 _pad0:31;              /* Reserved */
    _u32 cpu:1;                 /* CPU Bit */
#endif /* BCM5600_BE_HOST */
} def_ip_entry_t;

typedef struct def_ipx_entry_s {
#if defined(BCM5600_LE_HOST)
    _u32 sub_addr;              /* Subnet IP address */
    _u32 mac_addr_b0:8;         /* MAC Address */
    _u32 mac_addr_b1:8;         /* MAC Address */
    _u32 mac_addr_b2:8;         /* MAC Address */
    _u32 mac_addr_b3:8;         /* MAC Address */
    _u32 mac_addr_b4:8;         /* MAC Address */
    _u32 mac_addr_b5:8;         /* MAC Address */
    _u32 port:6;                /* Port Number */
    _u32 intf:5;                /* L3 Interface Number */
    _u32 subnet_len:5;          /* Subnet bit length */
    _u32 cpu:1;                 /* CPU Bit */
    _u32 _pad0:31;              /* Reserved */
#else /* BCM5600_BE_HOST */
    _u32 sub_addr;              /* Subnet IP address */
    _u32 mac_addr_b3:8;         /* MAC Address */
    _u32 mac_addr_b2:8;         /* MAC Address */
    _u32 mac_addr_b1:8;         /* MAC Address */
    _u32 mac_addr_b0:8;         /* MAC Address */
    _u32 subnet_len:5;          /* Subnet bit length */
    _u32 intf:5;                /* L3 Interface Number */
    _u32 port:6;                /* Port Number */
    _u32 mac_addr_b5:8;         /* MAC Address */
    _u32 mac_addr_b4:8;         /* MAC Address */
    _u32 _pad0:31;              /* Reserved */
    _u32 cpu:1;                 /* CPU Bit */
#endif /* BCM5600_BE_HOST */
} def_ipx_entry_t;

typedef struct l3_entry_s {
#if defined(BCM5600_LE_HOST)
    _u32 ip_addr;               /* IP/IPX Address */
    _u32 mac_addr_b0:8;         /* MAC Address */
    _u32 mac_addr_b1:8;         /* MAC Address */
    _u32 mac_addr_b2:8;         /* MAC Address */
    _u32 mac_addr_b3:8;         /* MAC Address */
    _u32 mac_addr_b4:8;         /* MAC Address */
    _u32 mac_addr_b5:8;         /* MAC Address */
    _u32 port:6;                /* Port Number */
    _u32 intf:5;                /* L3 Interface Number */
    _u32 hit:1;                 /* L3 Hit Bit */
    _u32 ftype:2;               /* IPX Frame Type */
    _u32 _pad0:2;               /* Reserved */
#else /* BCM5600_BE_HOST */
    _u32 ip_addr;               /* IP/IPX Address */
    _u32 mac_addr_b3:8;         /* MAC Address */
    _u32 mac_addr_b2:8;         /* MAC Address */
    _u32 mac_addr_b1:8;         /* MAC Address */
    _u32 mac_addr_b0:8;         /* MAC Address */
    _u32 _pad0:2;               /* Reserved */
    _u32 ftype:2;               /* IPX Frame Type */
    _u32 hit:1;                 /* L3 Hit Bit */
    _u32 intf:5;                /* L3 Interface Number */
    _u32 port:6;                /* Port Number */
    _u32 mac_addr_b5:8;         /* MAC Address */
    _u32 mac_addr_b4:8;         /* MAC Address */
#endif /* BCM5600_BE_HOST */
} l3_entry_t;

typedef struct l3intf_entry_s {
#if defined(BCM5600_LE_HOST)
    _u32 rout_addr_b0:8;        /* Router MAC Address */
    _u32 rout_addr_b1:8;        /* Router MAC Address */
    _u32 rout_addr_b2:8;        /* Router MAC Address */
    _u32 rout_addr_b3:8;        /* Router MAC Address */
    _u32 rout_addr_b4:8;        /* Router MAC Address */
    _u32 rout_addr_b5:8;        /* Router MAC Address */
    _u32 vlan_tag:12;           /* VLAN Tag */
    _u32 _pad0:4;               /* Reserved */
#else /* BCM5600_BE_HOST */
    _u32 rout_addr_b3:8;        /* Router MAC Address */
    _u32 rout_addr_b2:8;        /* Router MAC Address */
    _u32 rout_addr_b1:8;        /* Router MAC Address */
    _u32 rout_addr_b0:8;        /* Router MAC Address */
    _u32 _pad0:4;               /* Reserved */
    _u32 vlan_tag:12;           /* VLAN Tag */
    _u32 rout_addr_b5:8;        /* Router MAC Address */
    _u32 rout_addr_b4:8;        /* Router MAC Address */
#endif /* BCM5600_BE_HOST */
} l3intf_entry_t;

typedef struct marl_entry_s {
#if defined(BCM5600_LE_HOST)
    _u32 mac_addr_b0:8;         /* Mcast Address */
    _u32 mac_addr_b1:8;         /* Mcast Address */
    _u32 mac_addr_b2:8;         /* Mcast Address */
    _u32 mac_addr_b3:8;         /* Mcast Address */
    _u32 mac_addr_b4:8;         /* Mcast Address */
    _u32 mac_addr_b5:8;         /* Mcast Address */
    _u32 vlan_tag:12;           /* VLAN Tag */
    _u32 cos_dst:3;             /* Destination COS */
    _u32 _pad0:1;               /* Reserved */
    _u32 port_bitmap:28;        /* Port Bitmap */
    _u32 _pad1:4;               /* Reserved */
    _u32 ut_port_bitmap:28;     /* Untag Bitmap */
    _u32 _pad2:4;               /* Reserved */
#else /* BCM5600_BE_HOST */
    _u32 mac_addr_b3:8;         /* Mcast Address */
    _u32 mac_addr_b2:8;         /* Mcast Address */
    _u32 mac_addr_b1:8;         /* Mcast Address */
    _u32 mac_addr_b0:8;         /* Mcast Address */
    _u32 _pad0:1;               /* Reserved */
    _u32 cos_dst:3;             /* Destination COS */
    _u32 vlan_tag:12;           /* VLAN Tag */
    _u32 mac_addr_b5:8;         /* Mcast Address */
    _u32 mac_addr_b4:8;         /* Mcast Address */
    _u32 _pad1:4;               /* Reserved */
    _u32 port_bitmap:28;        /* Port Bitmap */
    _u32 _pad2:4;               /* Reserved */
    _u32 ut_port_bitmap:28;     /* Untag Bitmap */
#endif /* BCM5600_BE_HOST */
} marl_entry_t;

typedef struct ptable_entry_s {
#if defined(BCM5600_LE_HOST)
    _u32 vlan_tag:12;           /* VLAN Tag */
    _u32 sp_st:2;               /* Spanning Tree State */
    _u32 prt_dis:6;             /* Port Discard */
    _u32 jumbo:1;               /* Jumbo */
    _u32 rtag:3;                /* Trunk Selection */
    _u32 tgid:3;                /* Trunk Group ID */
    _u32 trunk:1;               /* Trunk Bit */
    _u32 cpu:1;                 /* CPU Bit */
    _u32 ptype:2;               /* Port type */
    _u32 bpdu:1;                /* BPDU Bit */
    _u32 port_bitmap:28;        /* Port Bitmap */
    _u32 mi:1;                  /* Mirror on Ingress */
    _u32 cml:2;                 /* CPU Managed Learning */
    _u32 _pad0:1;               /* Reserved */
    _u32 ut_port_bitmap:28;     /* Untag Bitmap */
    _u32 _pad1:4;               /* Reserved */
#else /* BCM5600_BE_HOST */
    _u32 bpdu:1;                /* BPDU Bit */
    _u32 ptype:2;               /* Port type */
    _u32 cpu:1;                 /* CPU Bit */
    _u32 trunk:1;               /* Trunk Bit */
    _u32 tgid:3;                /* Trunk Group ID */
    _u32 rtag:3;                /* Trunk Selection */
    _u32 jumbo:1;               /* Jumbo */
    _u32 prt_dis:6;             /* Port Discard */
    _u32 sp_st:2;               /* Spanning Tree State */
    _u32 vlan_tag:12;           /* VLAN Tag */
    _u32 _pad0:1;               /* Reserved */
    _u32 cml:2;                 /* CPU Managed Learning */
    _u32 mi:1;                  /* Mirror on Ingress */
    _u32 port_bitmap:28;        /* Port Bitmap */
    _u32 _pad1:4;               /* Reserved */
    _u32 ut_port_bitmap:28;     /* Untag Bitmap */
#endif /* BCM5600_BE_HOST */
} ptable_entry_t;

typedef struct tbmap_entry_s {
#if defined(BCM5600_LE_HOST)
    _u32 tbmap:28;              /* Trunk Bitmap */
    _u32 _pad0:4;               /* Reserved */
#else /* BCM5600_BE_HOST */
    _u32 _pad0:4;               /* Reserved */
    _u32 tbmap:28;              /* Trunk Bitmap */
#endif /* BCM5600_BE_HOST */
} tbmap_entry_t;

typedef struct ttr_entry_s {
#if defined(BCM5600_LE_HOST)
    _u32 tp0:6;                 /* Trunk Port 0 */
    _u32 tp1:6;                 /* Trunk Port 1 */
    _u32 tp2:6;                 /* Trunk Port 2 */
    _u32 tp3:6;                 /* Trunk Port 3 */
    _u32 tp4:6;                 /* Trunk Port 4 */
    _u32 _pad0:2;               /* Reserved */
    _u32 tp5:6;                 /* Trunk Port 5 */
    _u32 tp6:6;                 /* Trunk Port 6 */
    _u32 tp7:6;                 /* Trunk Port 7 */
    _u32 tg_size:4;             /* Trunk Size */
    _u32 _pad1:10;              /* Reserved */
#else /* BCM5600_BE_HOST */
    _u32 _pad0:2;               /* Reserved */
    _u32 tp4:6;                 /* Trunk Port 4 */
    _u32 tp3:6;                 /* Trunk Port 3 */
    _u32 tp2:6;                 /* Trunk Port 2 */
    _u32 tp1:6;                 /* Trunk Port 1 */
    _u32 tp0:6;                 /* Trunk Port 0 */
    _u32 _pad1:10;              /* Reserved */
    _u32 tg_size:4;             /* Trunk Size */
    _u32 tp7:6;                 /* Trunk Port 7 */
    _u32 tp6:6;                 /* Trunk Port 6 */
    _u32 tp5:6;                 /* Trunk Port 5 */
#endif /* BCM5600_BE_HOST */
} ttr_entry_t;

typedef struct vtable_entry_s {
#if defined(BCM5600_LE_HOST)
    _u32 vlan_tag:12;           /* VLAN Tag */
    _u32 _pad0:20;              /* Reserved */
    _u32 port_bitmap:28;        /* Port Bitmap */
    _u32 _pad1:4;               /* Reserved */
    _u32 ut_port_bitmap:28;     /* Untag Bitmap */
    _u32 _pad2:4;               /* Reserved */
#else /* BCM5600_BE_HOST */
    _u32 _pad0:20;              /* Reserved */
    _u32 vlan_tag:12;           /* VLAN Tag */
    _u32 _pad1:4;               /* Reserved */
    _u32 port_bitmap:28;        /* Port Bitmap */
    _u32 _pad2:4;               /* Reserved */
    _u32 ut_port_bitmap:28;     /* Untag Bitmap */
#endif /* BCM5600_BE_HOST */
} vtable_entry_t;


#ifdef _u32_local_def
#undef _u32
#undef _u32_local_def
#endif

#endif  /* BCM5600 */

#define MIN_EPIC_5600	0x0
#define MAX_EPIC_5600	(MIN_EPIC_5600+NUM_EPIC_5600-1)
#define MIN_GPIC_5600	0x3
#define MAX_GPIC_5600	(MIN_GPIC_5600+NUM_GPIC_5600-1)
#define MIN_PIC_5600	0x0
#define MAX_PIC_5600	MAX_GPIC_5600

#define MIN_PORT_5600	0x00
#define MAX_PORT_5600	0x1b
#define NUM_PORT_5600	(MAX_PORT_5600 - MIN_PORT_5600 + 1)

#define SR_BLOCK_BP	20
#define SR_REGOFS_BP	0
#define SR_MEMOFS_BP	16
#define SR_REGIDX_BP	12
#define SR_MEMIDX_BP	0

typedef enum soc_reg_t {
	INVALIDr = -1,
	AGE_TIMERr = 0,
	ANCTLr = 1,
	ANLPAr = 2,
	ANNPGr = 3,
	ANSTTr = 4,
	BISTMEMFAILUREr = 5,
	CABCONTROLr = 6,
	CAR1r = 7,
	CAR2r = 8,
	CBPCONTROLr = 9,
	CFAPPOOLSIZEr = 10,
	CFAPREADPOINTERr = 11,
	CONFIGr = 12,
	COS_ENABLEr = 13,
	COS_SELr = 14,
	DEST_MIRRr = 15,
	DROPPEDCELLCOUNTr = 16,
	DROPPEDPACKETCOUNTr = 17,
	EGR_BKP_DISr = 18,
	EGR_ENABLEr = 19,
	EGR_MAXTIMEr = 20,
	EN_IFILTERr = 21,
	EPC_BIST1r = 22,
	EPC_BIST2r = 23,
	EPC_BPDU1r = 24,
	EPC_BPDU2r = 25,
	EPC_BPDU4r = 26,
	EPC_EGR_DBGr = 27,
	EPC_EGR_SNGL_OUTr = 28,
	EPC_EGR_SNGL_PKTr = 29,
	EPC_HOLr = 30,
	EPC_LINKr = 31,
	EPC_PFM0r = 32,
	EPC_PFM1r = 33,
	EPC_RATE_N_IPr = 34,
	EPC_STAD1r = 35,
	ESA0r = 36,
	ESA1r = 37,
	ESA2r = 38,
	FE_CLRTr = 39,
	FE_IPGRr = 40,
	FE_IPGTr = 41,
	FE_MAC1r = 42,
	FE_MAC2r = 43,
	FE_MAXFr = 44,
	FE_SUPPr = 45,
	FE_TESTr = 46,
	FLOWCONTROLr = 47,
	GCAR1r = 48,
	GCAR2r = 49,
	GCCCOUNTr = 50,
	GEGR_ENABLEr = 51,
	GLOBALRETURNMARKERr = 52,
	GMACC0r = 53,
	GMACC1r = 54,
	GMACC2r = 55,
	GPCSCr = 56,
	GPC_EGR_DBGr = 57,
	GPC_EGR_SNGL_OUTr = 58,
	GPC_EGR_SNGL_PKTr = 59,
	GRALNr = 60,
	GRBCAr = 61,
	GRBYTr = 62,
	GRCDEr = 63,
	GRDISCr = 64,
	GRFCRr = 65,
	GRFCSr = 66,
	GRFLRr = 67,
	GRIPCr = 68,
	GRIPCHKr = 69,
	GRIPDr = 70,
	GRIPXr = 71,
	GRIPXDr = 72,
	GRJBRr = 73,
	GRMCAr = 74,
	GROVRr = 75,
	GRPKTr = 76,
	GRUNDr = 77,
	GRXCFr = 78,
	GRXPFr = 79,
	GRXUOr = 80,
	GSA0r = 81,
	GSA1r = 82,
	GTABRTr = 83,
	GTAGEr = 84,
	GTBCAr = 85,
	GTBYTr = 86,
	GTFCSr = 87,
	GTIPr = 88,
	GTIPDr = 89,
	GTIPXr = 90,
	GTJBRr = 91,
	GTMCAr = 92,
	GTOVRr = 93,
	GTPKTr = 94,
	GTR1023r = 95,
	GTR127r = 96,
	GTR255r = 97,
	GTR511r = 98,
	GTR64r = 99,
	GTRMAXr = 100,
	GTRMGVr = 101,
	GTVLANr = 102,
	GTXCFr = 103,
	GTXPFr = 104,
	HIGHWATERMARKERr = 105,
	HOLBLOCKINGMARKERr = 106,
	IBPCOUNTr = 107,
	INGRESSBACKPRESSUREr = 108,
	INITIATEBIST1r = 109,
	INITIATEBIST2r = 110,
	LCCCOUNTr = 111,
	LOWWATERMARKERr = 112,
	MAXFRr = 113,
	MIRRORr = 114,
	MISCCONFIGr = 115,
	MMUPORTENABLEr = 116,
	PPPEMPTYr = 117,
	PPPFULLr = 118,
	RALNr = 119,
	RBCAr = 120,
	RBYTr = 121,
	RCDEr = 122,
	RDISCr = 123,
	REROUTINGr = 124,
	RESURRECTr = 125,
	REVCDr = 126,
	RFCRr = 127,
	RFCSr = 128,
	RFLRr = 129,
	RFRGr = 130,
	RIPCr = 131,
	RIPCHKr = 132,
	RIPDr = 133,
	RIPXr = 134,
	RIPXDr = 135,
	RJBRr = 136,
	RMCAr = 137,
	ROVRr = 138,
	RPKTr = 139,
	RUNDr = 140,
	RXCFr = 141,
	RXPFr = 142,
	RXUOr = 143,
	SCHEDULERCONTROLr = 144,
	SDRAMCONTROLr = 145,
	SDRAMCONTROL2r = 146,
	SFAPPOOLSIZEr = 147,
	SFAPREADPOINTERr = 148,
	SUM_LWMSr = 149,
	TABRTr = 150,
	TAGEr = 151,
	TBCAr = 152,
	TBYTr = 153,
	TDFRr = 154,
	TEDFr = 155,
	TEST2r = 156,
	TFCSr = 157,
	TFRGr = 158,
	TIPr = 159,
	TIPDr = 160,
	TIPXr = 161,
	TJBRr = 162,
	TLCLr = 163,
	TMCAr = 164,
	TMCLr = 165,
	TNCLr = 166,
	TOVRr = 167,
	TPKTr = 168,
	TR1023r = 169,
	TR127r = 170,
	TR255r = 171,
	TR511r = 172,
	TR64r = 173,
	TRMAXr = 174,
	TRMGVr = 175,
	TSCLr = 176,
	TVLANr = 177,
	TXCFr = 178,
	TXCLr = 179,
	TXPFr = 180,
	XQEMPTYr = 181,
	NUM_SOC_REG = 182
} soc_reg_t;

/* NOTE: 'FIELDf' is the zero value */
typedef enum soc_field_t {
	INVALIDf = -1,
	FIELDf = 0,
	ACTIONf = 1,
	ADDR1f = 2,
	ADDR2f = 3,
	ADPADf = 4,
	AGE_ENAf = 5,
	AGE_VALf = 6,
	ANACK2f = 7,
	ANCPLTf = 8,
	ANENf = 9,
	ANERRf = 10,
	ANMSGf = 11,
	ANNPf = 12,
	ANTOGf = 13,
	ASMDRf = 14,
	AUTOSf = 15,
	AUTOZf = 16,
	AUTO_PADf = 17,
	BCAST_ENf = 18,
	BIST_DONEf = 19,
	BIT_MODEf = 20,
	BPDUf = 21,
	BPDU1f = 22,
	BPDU2f = 23,
	BPDU3f = 24,
	BPDU4f = 25,
	BP_NO_BOFFf = 26,
	CAB0f = 27,
	CAB0_DONEf = 28,
	CAB1f = 29,
	CAB1_DONEf = 30,
	CAB2f = 31,
	CAB2_DONEf = 32,
	CAB3f = 33,
	CAB3_DONEf = 34,
	CABBOOTOUTTIMERf = 35,
	CBD0f = 36,
	CBD1f = 37,
	CBD2f = 38,
	CBD3f = 39,
	CBHf = 40,
	CBP0_DONEf = 41,
	CBP1_DONEf = 42,
	CBP2_DONEf = 43,
	CBP3_DONEf = 44,
	CBPH_DONEf = 45,
	CCP_DONEf = 46,
	CFAPf = 47,
	CFAPPOOLSIZEf = 48,
	CFAP_DONEf = 49,
	CLK_GRANf = 50,
	CLRCNTf = 51,
	CMICARBITERDELAYf = 52,
	CMLf = 53,
	COL_WINf = 54,
	COS0f = 55,
	COS1f = 56,
	COS2f = 57,
	COS3f = 58,
	COS4f = 59,
	COS5f = 60,
	COS6f = 61,
	COS7f = 62,
	COS_DSTf = 63,
	COS_SRCf = 64,
	COUNTf = 65,
	CPUf = 66,
	CPU_PRIf = 67,
	CRCENf = 68,
	CRC_ENf = 69,
	CRPf = 70,
	DCCf = 71,
	DCRC12f = 72,
	DDR_SDRAMENf = 73,
	DDR_SDRAM_BSELf = 74,
	DDR_SDRAM_CLSELf = 75,
	DDR_SDRAM_TRPf = 76,
	DEADADDRESS1f = 77,
	DEADADDRESS2f = 78,
	DEADADDRESSCOUNTf = 79,
	DEBUG_ONf = 80,
	DEL_CRCf = 81,
	DLFBC_ENf = 82,
	DPCf = 83,
	DRRDf = 84,
	DST_HITf = 85,
	DTRDf = 86,
	DUMMYf = 87,
	ECMDTf = 88,
	EGRMASKf = 89,
	ENBL_JABBRf = 90,
	EPORTf = 91,
	EWRAPf = 92,
	EXC_DEFf = 93,
	FCNTf = 94,
	FCRXf = 95,
	FCTXf = 96,
	FDf = 97,
	FE0f = 98,
	FE1f = 99,
	FE10f = 100,
	FE11f = 101,
	FE12f = 102,
	FE13f = 103,
	FE14f = 104,
	FE15f = 105,
	FE16f = 106,
	FE17f = 107,
	FE18f = 108,
	FE19f = 109,
	FE2f = 110,
	FE20f = 111,
	FE21f = 112,
	FE22f = 113,
	FE23f = 114,
	FE3f = 115,
	FE4f = 116,
	FE5f = 117,
	FE6f = 118,
	FE7f = 119,
	FE8f = 120,
	FE9f = 121,
	FILTERf = 122,
	FIL_ENf = 123,
	FLCHKf = 124,
	FMASKf = 125,
	FORCE_PORT_BITMAPf = 126,
	FORCE_QUIETf = 127,
	FSELf = 128,
	FTYPEf = 129,
	FULLDf = 130,
	FULL_DUPf = 131,
	FX100f = 132,
	GBPFULLCLEARPOINTf = 133,
	GBPFULLSETPOINTf = 134,
	GCCf = 135,
	GE0f = 136,
	GE1f = 137,
	GETDEADADDRESSf = 138,
	GLVRf = 139,
	GRALNf = 140,
	GRBCAf = 141,
	GRBYTf = 142,
	GRCDEf = 143,
	GRCSEf = 144,
	GRDISCf = 145,
	GRFCSf = 146,
	GRFLRf = 147,
	GRIPCf = 148,
	GRIPCHKf = 149,
	GRIPDf = 150,
	GRIPXf = 151,
	GRIPXDf = 152,
	GRJBRf = 153,
	GRMf = 154,
	GRMCAf = 155,
	GROVRf = 156,
	GRPKTf = 157,
	GRUNDf = 158,
	GRUXOf = 159,
	GRXCFf = 160,
	GRXPFf = 161,
	GTABRTf = 162,
	GTAGEf = 163,
	GTBCAf = 164,
	GTBYTf = 165,
	GTDFRf = 166,
	GTFCSf = 167,
	GTIPf = 168,
	GTIPDf = 169,
	GTIPXf = 170,
	GTJBRf = 171,
	GTMCAf = 172,
	GTOVRf = 173,
	GTPFHf = 174,
	GTPKTf = 175,
	GTR1023f = 176,
	GTR127f = 177,
	GTR255f = 178,
	GTR511f = 179,
	GTR64f = 180,
	GTRMAXf = 181,
	GTRMGVf = 182,
	GTVLANf = 183,
	GTXCFf = 184,
	GTXPFf = 185,
	HITf = 186,
	HOLf = 187,
	HUGENf = 188,
	HUGE_FRf = 189,
	HWMf = 190,
	IBPf = 191,
	ICCf = 192,
	INGMASKf = 193,
	INTFf = 194,
	IN_BISTf = 195,
	IPGR1f = 196,
	IPGR2f = 197,
	IPGTf = 198,
	IPORTf = 199,
	IPX_SIZEf = 200,
	IP_ADDRf = 201,
	IP_CFGf = 202,
	IP_SIZEf = 203,
	JAM_ENf = 204,
	JTRPSf = 205,
	JTRPTf = 206,
	JUMBOf = 207,
	L10Bf = 208,
	L3f = 209,
	L32Bf = 210,
	L3_ENAf = 211,
	LBACKf = 212,
	LCCf = 213,
	LEARN_CNTf = 214,
	LG_CHKf = 215,
	LIMITf = 216,
	LINKOKf = 217,
	LNK_FAILf = 218,
	LONGPf = 219,
	LONG_PREf = 220,
	LPACK2f = 221,
	LPANERRf = 222,
	LPASMDRf = 223,
	LPCFf = 224,
	LPFDf = 225,
	LPMSGf = 226,
	LPNEXTPf = 227,
	LPNPf = 228,
	LPPAUSEf = 229,
	LPTOGf = 230,
	LWMf = 231,
	MAC_ADDRf = 232,
	MAXCELLSPERSLOTf = 233,
	MAXFRf = 234,
	MAXLATENCYf = 235,
	MAXNUMBEROFCOSf = 236,
	MAXOUTSTANDINGREADf = 237,
	MAXPACKETSf = 238,
	MAXSLOTSPERCHAINf = 239,
	MAXTIMEf = 240,
	MCAST_ENf = 241,
	MCLKFQf = 242,
	MIf = 243,
	MIFGf = 244,
	MIRR_ENf = 245,
	MIRR_L3f = 246,
	MONTHf = 247,
	MPORTf = 248,
	NATPRIENBf = 249,
	NEXTPf = 250,
	NO_BOFFf = 251,
	NO_CIPHERf = 252,
	NPLPCFf = 253,
	OVHEADTIMEPRIENBf = 254,
	PACKETPOINTERPOOLSIZEf = 255,
	PADENf = 256,
	PAD_ENf = 257,
	PARFf = 258,
	PASSALLf = 259,
	PAUSEf = 260,
	PGRXf = 261,
	PIDf = 262,
	PID_DONEf = 263,
	POACHINGTIMERf = 264,
	PORTf = 265,
	PORT_BITMAPf = 266,
	PPPf = 267,
	PPP_DONEf = 268,
	PRIf = 269,
	PRT_DISf = 270,
	PRT_ENABLEf = 271,
	PTYPEf = 272,
	PUREPf = 273,
	PURE_PADf = 274,
	RALNf = 275,
	RANXf = 276,
	RBCAf = 277,
	RBYTf = 278,
	RCDEf = 279,
	RCSEf = 280,
	RCSELf = 281,
	RDISCf = 282,
	REF_CNTf = 283,
	REQTIMERPRIENBf = 284,
	RESERVED0f = 285,
	RESERVED1f = 286,
	RESERVED2f = 287,
	RESERVED_WORDf = 288,
	RESET_100Xf = 289,
	RESET_10Tf = 290,
	RESET_RMIIf = 291,
	RETRYf = 292,
	REVf = 293,
	RFCSf = 294,
	RFINDf = 295,
	RFLRf = 296,
	RFRGf = 297,
	RIPCf = 298,
	RIPCHKf = 299,
	RIPDf = 300,
	RIPXf = 301,
	RIPXDf = 302,
	RJBRf = 303,
	RMCAf = 304,
	RMGTf = 305,
	ROUNDROBINPRIENBf = 306,
	ROUT_ADDRf = 307,
	ROVRf = 308,
	RPKTf = 309,
	RREXf = 310,
	RRFNf = 311,
	RRMCf = 312,
	RSTANf = 313,
	RST_RFUNf = 314,
	RST_RMCSf = 315,
	RST_SIMf = 316,
	RST_TFUNf = 317,
	RST_TMCSf = 318,
	RTAGf = 319,
	RTEXf = 320,
	RTFNf = 321,
	RTMCf = 322,
	RUNBIST_CAB0f = 323,
	RUNBIST_CAB1f = 324,
	RUNBIST_CAB2f = 325,
	RUNBIST_CAB3f = 326,
	RUNBIST_CBP0f = 327,
	RUNBIST_CBP1f = 328,
	RUNBIST_CBP2f = 329,
	RUNBIST_CBP3f = 330,
	RUNBIST_CBPHf = 331,
	RUNBIST_CCPf = 332,
	RUNBIST_CFAPf = 333,
	RUNBIST_PIDf = 334,
	RUNBIST_PPPf = 335,
	RUNBIST_SFAPf = 336,
	RUNBIST_XQf = 337,
	RUNDf = 338,
	RUXOf = 339,
	RXCFf = 340,
	RXEN0f = 341,
	RXPFf = 342,
	RX_ENf = 343,
	RX_PAUf = 344,
	SAFETYMARGINf = 345,
	SAUFULLPRIENBf = 346,
	SAUFULLVALUEf = 347,
	SAUTIMERVALUEf = 348,
	SCLTf = 349,
	SCPf = 350,
	SDRAMCLK0DLLf = 351,
	SDRAMCLK1DLLf = 352,
	SDRAMCLK2DLLf = 353,
	SDRAMCLK3DLLf = 354,
	SDRAMSIZEf = 355,
	SDUTIMERVALUEf = 356,
	SD_DISf = 357,
	SFAPf = 358,
	SFAPPOOLSIZEf = 359,
	SFAPRDTIMERVALUEf = 360,
	SFAPWRTIMERVALUEf = 361,
	SFAP_DONEf = 362,
	SHORTOUTUNUSEDBITSf = 363,
	SHORT_QNTAf = 364,
	SKIDMARKf = 365,
	SLOTEJECTGAPf = 366,
	SLOTFETCHGAPf = 367,
	SOFT_RSTf = 368,
	SPEEDf = 369,
	SPQCTf = 370,
	SP_STf = 371,
	SRPf = 372,
	SRSTf = 373,
	STf = 374,
	STAD0f = 375,
	STAD1f = 376,
	STAD2f = 377,
	STATUS_PORT_BITMAPf = 378,
	ST_IS_MIRRf = 379,
	ST_LINKf = 380,
	ST_MCNTf = 381,
	ST_MODULEf = 382,
	ST_SIMPLEXf = 383,
	SUBNET_LENf = 384,
	SUB_ADDRf = 385,
	SUMf = 386,
	SYNCOKf = 387,
	TABRTf = 388,
	TAGEf = 389,
	TAG_ENf = 390,
	TBCAf = 391,
	TBMAPf = 392,
	TBYTf = 393,
	TDFRf = 394,
	TEDFf = 395,
	TEST_BACKf = 396,
	TEST_PAUSEf = 397,
	TFCSf = 398,
	TFRGf = 399,
	TGIDf = 400,
	TG_SIZEf = 401,
	TIMER_ENf = 402,
	TIPf = 403,
	TIPDf = 404,
	TIPXf = 405,
	TJBRf = 406,
	TLCLf = 407,
	TMCAf = 408,
	TMCLf = 409,
	TMDSf = 410,
	TNCLf = 411,
	TOS_Pf = 412,
	TOVRf = 413,
	TP0f = 414,
	TP1f = 415,
	TP2f = 416,
	TP3f = 417,
	TP4f = 418,
	TP5f = 419,
	TP6f = 420,
	TP7f = 421,
	TPAUSEf = 422,
	TPFHf = 423,
	TPKTf = 424,
	TR1023f = 425,
	TR127f = 426,
	TR255f = 427,
	TR511f = 428,
	TR64f = 429,
	TRMAXf = 430,
	TRMGVf = 431,
	TRUNKf = 432,
	TSCLf = 433,
	TVLANf = 434,
	TXCFf = 435,
	TXCLf = 436,
	TXEN0f = 437,
	TXPFf = 438,
	TX_PAUf = 439,
	UT_PORT_BITMAPf = 440,
	VLAN_PADf = 441,
	VLAN_TAGf = 442,
	VLPADf = 443,
	XQ0f = 444,
	XQ1f = 445,
	XQ10f = 446,
	XQ11f = 447,
	XQ12f = 448,
	XQ13f = 449,
	XQ14f = 450,
	XQ15f = 451,
	XQ16f = 452,
	XQ17f = 453,
	XQ18f = 454,
	XQ19f = 455,
	XQ2f = 456,
	XQ20f = 457,
	XQ21f = 458,
	XQ22f = 459,
	XQ23f = 460,
	XQ24f = 461,
	XQ25f = 462,
	XQ26f = 463,
	XQ27f = 464,
	XQ28f = 465,
	XQ29f = 466,
	XQ3f = 467,
	XQ30f = 468,
	XQ31f = 469,
	XQ4f = 470,
	XQ5f = 471,
	XQ6f = 472,
	XQ7f = 473,
	XQ8f = 474,
	XQ9f = 475,
	XQ_DONEf = 476,
	YEARf = 477,
	NUM_SOC_FIELD = 478
} soc_field_t;

typedef enum soc_mem_t {
	INVALIDm = -1,
	ARL_m = 0,
	ARL_CNT_m = 1,
	FILTER_IMASKm = 2,
	FILTER_IRULEm = 3,
	GBPm = 4,
	GFILTER_IMASKm = 5,
	GFILTER_IRULEm = 6,
	L3_DEF_IPm = 7,
	L3_DEF_IPXm = 8,
	L3_L3m = 9,
	L3_L3INTFm = 10,
	MARL_m = 11,
	MMU_MEMORIES1_CAB0m = 12,
	MMU_MEMORIES1_CAB1m = 13,
	MMU_MEMORIES1_CAB2m = 14,
	MMU_MEMORIES1_CAB3m = 15,
	MMU_MEMORIES1_CBPDATA0m = 16,
	MMU_MEMORIES1_CBPDATA1m = 17,
	MMU_MEMORIES1_CBPDATA2m = 18,
	MMU_MEMORIES1_CBPDATA3m = 19,
	MMU_MEMORIES1_CBPHEADERm = 20,
	MMU_MEMORIES1_CCPm = 21,
	MMU_MEMORIES1_CFAPm = 22,
	MMU_MEMORIES1_PIDm = 23,
	MMU_MEMORIES1_PPPm = 24,
	MMU_MEMORIES1_SFAPm = 25,
	MMU_MEMORIES2_XQ0m = 26,
	MMU_MEMORIES2_XQ1m = 27,
	MMU_MEMORIES2_XQ10m = 28,
	MMU_MEMORIES2_XQ11m = 29,
	MMU_MEMORIES2_XQ12m = 30,
	MMU_MEMORIES2_XQ13m = 31,
	MMU_MEMORIES2_XQ14m = 32,
	MMU_MEMORIES2_XQ15m = 33,
	MMU_MEMORIES2_XQ2m = 34,
	MMU_MEMORIES2_XQ3m = 35,
	MMU_MEMORIES2_XQ4m = 36,
	MMU_MEMORIES2_XQ5m = 37,
	MMU_MEMORIES2_XQ6m = 38,
	MMU_MEMORIES2_XQ7m = 39,
	MMU_MEMORIES2_XQ8m = 40,
	MMU_MEMORIES2_XQ9m = 41,
	MMU_MEMORIES3_XQ16m = 42,
	MMU_MEMORIES3_XQ17m = 43,
	MMU_MEMORIES3_XQ18m = 44,
	MMU_MEMORIES3_XQ19m = 45,
	MMU_MEMORIES3_XQ20m = 46,
	MMU_MEMORIES3_XQ21m = 47,
	MMU_MEMORIES3_XQ22m = 48,
	MMU_MEMORIES3_XQ23m = 49,
	MMU_MEMORIES3_XQ24m = 50,
	MMU_MEMORIES3_XQ25m = 51,
	MMU_MEMORIES3_XQ26m = 52,
	MMU_MEMORIES3_XQ27m = 53,
	MMU_MEMORIES3_XQ28m = 54,
	MMU_MEMORIES3_XQ29m = 55,
	MMU_MEMORIES3_XQ30m = 56,
	MMU_MEMORIES3_XQ31m = 57,
	PTABLE_m = 58,
	TTABLE_TBMAPm = 59,
	TTABLE_TTRm = 60,
	VTABLE_m = 61,
	NUM_SOC_MEM = 62
} soc_mem_t;

typedef _u32 soc_addr_t;

typedef enum soc_block_t {
	INVALIDb = -1,
	EPIC0b = 0,
	EPIC1b = 1,
	EPIC2b = 2,
	GPIC0b = 3,
	GPIC1b = 4,
	CMICb = 5,
	MMUb = 6,
	ANY_EPIC,
	ANY_GPIC,
	ANY_PIC
} soc_block_t;

typedef int soc_cos_t;
typedef int soc_port_t;

#define SOCF_LE		0x01	/* Flag: little endian */
#define SOCF_RO		0x02	/* Flag: read only */
#define SOCF_WO		0x04	/* Flag: write only */
#define SOCF_SC		0x08
#define SOCF_RES        0x10    /* Flag: reserved (do not test, etc.) */

typedef struct soc_fieldinfo_t {
	soc_field_t	field;
	uint16		len;	/* Bits in field */
	uint16		bp;	/* Least bit position of the field */
	unsigned char	flags;	/* Logical OR of SOC_FIELD_XXX */
} soc_fieldinfo_t;

typedef enum soc_regtype_t {
	soc_genreg, soc_portreg, soc_cosreg
} soc_regtype_t;

extern char *soc_regtypenames[];

typedef struct soc_reginfo_t {
#ifndef NDEBUG
	soc_reg_t		reg;		/* Redundant */
#endif /* !NDEBUG */
#ifndef	NO_REG_NAMES
	char			*name;
#endif /* !NO_REG_NAMES */
	soc_block_t		block;
	soc_regtype_t		regtype;
	uint32			offset;		/* Includes 2-bit form field */
	unsigned		nFields;
	soc_fieldinfo_t		*fields;
#ifndef	NO_RESET_INFO
	uint32			resetVal;
	uint32			resetMask; 	/* 1 where resetVal is valid */
#endif /* !NO_RESET_INFO */
#ifndef	NO_DESCRIPTIONS
	char			*desc;
#endif /* !NO_DESCRIPTIONS */
} soc_reginfo_t;

typedef struct soc_meminfo_t {
#ifndef NDEBUG
	soc_mem_t		mem;		/* Redundant */
#endif /* !NDEBUG */
#ifndef	NO_MEM_NAMES
	char			*name;
#endif /* !NO_MEM_NAMES */
	uint16			be;		/* nonzero for BE word order */
	uint16			nBlocks;	/* 0 if monolithic */
	soc_block_t		*blocks;
	uint32			offset;		/* Includes region */
	uint16			bytes;
	uint16			nFields;
	soc_fieldinfo_t		*fields;
} soc_meminfo_t;

typedef struct soc_portinfo_t {
#ifndef NDEBUG
	soc_port_t		portnum;	/* Redundant */
#endif /* !NDEBUG */
	soc_block_t		block;
	int			index;
} soc_portinfo_t;

typedef struct soc_regaddrinfo_t {
	uint32			addr;
	int			valid;
	soc_reg_t		reg;		/* INVALIDr if not used */
	soc_block_t		block;		/* INVALIDb if not used */
	soc_port_t		port;		/* -1 if not used */
	soc_cos_t		cos;		/* -1 if not used */
	soc_field_t		field;		/* INVALIDf for entire reg. */
} soc_regaddrinfo_t;

#define __NCREG			(NUM_PIC_5600 * NUM_COS_5600)
#define __NPREG			(NUM_PORT_5600)

#define REGADDRLIST_MAX		(__NCREG > __NPREG ? __NCREG : __NPREG)

typedef struct soc_regaddrlist_t {
	int			count;
	soc_regaddrinfo_t	ainfo[REGADDRLIST_MAX];
} soc_regaddrlist_t;

typedef struct soc_regaddrfields_t {
	soc_regtype_t	type;
	uint32		offset;
	soc_cos_t	minCOS;
	soc_cos_t	maxCOS;
	int		nBPs;
	struct {
		soc_block_t	block;
		soc_port_t	port;
	} bps[MAX_PORT_5600+1];
} soc_regaddrfields_t;

#ifndef NO_FLD_NAMES
extern char *soc_fieldnames[];
#endif
extern soc_reginfo_t soc_registers[];
extern soc_meminfo_t soc_memories[];
extern soc_portinfo_t soc_ports[];
extern char *soc_blocknames[];
extern char *soc_regtypenames[];

#define port2block(port)	(soc_ports[port].block)

/* General:	 reg_addr(soc_reg_t reg);
 * General(PIC): reg_addr(soc_reg_t reg, unsigned pic);
 * Per-port:	 reg_addr(soc_reg_t reg, unsigned port);
 * Per-COS:	 reg_addr(soc_reg_t reg, unsigned cos);
 * Per-COS(PIC): reg_addr(soc_reg_t reg, unsigned cos, unsigned pic);
 *
 * PIC forms are for EPIC/GPIC registers. PIC's 0 thru NUM_EPIC_5600-1 are the
 * EPIC's. PIC's NUM_EPIC_5600 thru NUM_PIC_5600-1 are the GPIC's.
 */
extern uint32 reg_addr(soc_reg_t, ...);

/* Regular:	mem_addr(soc_mem_t mem, unsigned blkidx, uint32 index);
 * Monolithic:	mem_addr(soc_mem_t mem, uint32 index);
 *
 * 'blkidx' indicates which version of the table to access, and must be in
 * the range 0 thru get_mem_numcopies(mem)-1.
 */
extern uint32 mem_addr(soc_mem_t mem, ...);

/* Returns the number of copies of a given memory. 0 is returned if the
 * memory uses monolithic addresses, in which case there is exactly 1 copy.
 */
extern unsigned get_mem_numcopies(soc_mem_t);

/* Get 'field' of datum 'current'.
 */
extern uint32 get_reg_field(
 soc_reg_t reg, uint32 current, soc_field_t field
);
extern uint32 *get_mem_field(
 soc_mem_t mem, uint32 *entbuf, soc_field_t field, uint32 *fldbuf
);
extern unsigned get_mem_field_numwords(soc_mem_t mem, soc_field_t field);
extern unsigned get_mem_numbytes(soc_mem_t mem);

/* Set 'field' of datum 'current' to 'value'.
 */
extern uint32 set_reg_field(
 soc_reg_t reg, uint32 current, soc_field_t field, uint32 value
);
extern uint32 *set_mem_field(
 soc_mem_t mem, uint32 *entbuf, soc_field_t field, uint32 *fldbuf
);
extern uint32 *zero_mem_value(soc_mem_t mem, uint32 *buf);

/* Set all fields to -1.
 */
#if 0
extern uint32 get_reg_datamask(soc_reg_t reg, int flags);
extern uint32 *get_mem_datamask(soc_mem_t mem, uint32 *buf);
#endif

/* Print fields
 */
#if 0
extern void sprint_reg_data(char *bp, char *infix,
			    soc_reg_t reg, uint32 value);
extern void sprint_reg_addr(char *bp, soc_regaddrinfo_t *ainfo);
#endif

/* Build a register address based on the address fields
 */
extern uint32 make_reg_addr(
 soc_block_t block, soc_regtype_t type,
 uint32 offset, soc_port_t port, soc_cos_t cos
);

/* Find the valid address fields for a given register
 */
extern soc_regaddrfields_t get_regaddrfields(soc_reg_t reg);

extern char *socreg_info(soc_reg_t);

/* Miscellaneous
 */
extern int isPIC(soc_block_t eblock);
extern int match_block(soc_block_t block, soc_block_t eblock, int *picp);
extern int get_port_index(
 soc_port_t port, soc_block_t eblock, soc_block_t *blockp
);
extern soc_block_t pic2block(unsigned pic);
#if 0
extern int block2schblknum(soc_block_t b);
#endif
extern soc_fieldinfo_t *find_field(
 soc_field_t field, unsigned n, soc_fieldinfo_t *fields
);
extern soc_block_t minblock(soc_block_t block);
extern soc_block_t maxblock(soc_block_t block);
extern soc_port_t minport(soc_block_t block);
extern soc_port_t maxport(soc_block_t block);

#endif /* BCM5600_SOCREGS_H */
