/*
 * Name         : fs-quota.c
 * RCS          : $Id: fs-quota.c,v 1.99 2002/12/05 21:08:57 bob Exp $
 * Description	: Quota implementation data structures
 * Created by	: Rahul Khona
 * Date Created	: 8/10/2001
 */

#if !defined(SSC)

#include "fs-includes.h"
PRIVATE void notifyAdmin(fs_Context *context,
                         fs_QuotaContextList_t *quotaContextList);
PRIVATE void
fs_zapConsumptionInfo(fs_Context *context, fs_FileSetIncoreRecord_t *fset,
                      int type);
PRIVATE void
fs_invalidateIncoreQuota(fs_Context *context, fs_FileSetIncoreRecord_t *fset,
                         int type);
PRIVATE void
fs_zapFilesetConsumptionInfo(fs_Context *context,
                             fs_FileSetIncoreRecord_t *fset);

/*
 * Fileset quota debug UI
 */

//#define N_QUOTA_DBG_COUNTERS    (sizeof(quotaDbgCounterTypes)/sizeof(uint64))
typedef enum {
    quotaHoldCnt,
    quotaReleaseCnt,
    N_QUOTA_DBG_COUNTERS
}quotaDbgCounters;

char *qDbgCntLabel[N_QUOTA_DBG_COUNTERS] = {
    "quotaHold",
    "quotaRelease",
};

uint64  qDbgCnt[N_QUOTA_DBG_COUNTERS] = { 0 };   //zeroes the whole thing

int32 fsetQuotaDbg_statCmd(int32 argc, char8 *argv[])
{
    int i;
    
    if (argc == 2 && strcmp(argv[1], "reset") == 0)   {
        bzero(qDbgCnt, sizeof(qDbgCnt));
    } else {
        for (i = 0; i < N_QUOTA_DBG_COUNTERS; i++)
        {
             printf("%-20s:%llu\n", qDbgCntLabel[i], qDbgCnt[i]);
        }
    }    
    return (NFX_OK);
}

int32 fs_laudbg(int32 argc, char8 *argv[])
{
    int32   rc = NFX_ERR;
    char8   *cmd;

    if (argc < 2)
        goto printUsage;

    cmd = argv[1];
    if (strcmp(cmd, "stat") == 0)
        rc = fsetQuotaDbg_statCmd(--argc, ++argv);
//    if (strcmp(cmd, "") == 0) {
//    }            
    
    return(rc);
    
printUsage:
    printf("USAGE: laudbg <stat [reset] | >\n");
    return(NFX_ERR);
}

    
        
                
/*
 * This file contains quota implementation and support routines.
 */


/*
 * Report quotas for a fileset, user or group
 */

int32
fs_reportQuotaThread( fs_Context *context)
{
	uint32	requests;
	uint64	start;
	fs_MsgReportQuotaReq_t	*req;
	fs_MsgReportQuotaResp_t	*resp;

	resp = (fs_MsgReportQuotaResp_t *) context->respHdr;
	req = (fs_MsgReportQuotaReq_t *) context->reqHdr;

	requests = req->numberEntries;
	start = req->startIndex;

    FS_ASSERT(context->respSize >= sizeof (fs_MsgReportQuotaResp_t) +
                                    (requests * sizeof (fs_QuotaBlk_t)));

	context->respSize = sizeof (fs_MsgReportQuotaResp_t) +
        				(requests * sizeof (fs_QuotaBlk_t));
    resp->msgHdr.len = context->respSize;
#if 0                                    
    WE CAN NO LONGER MALLOC RESPONSE. SET TO MAXIMUM IN REQUEST.TAB, THEN
    JUST ADJUST THE SIZE HERE
	/*
	 * Create our response space as we would not have allocated
	 * space for this request at context creation time because
	 * of variable nature of the request
	 */

	context->respHdr = fs_ramAlloc(FS_RAM_RESP_MSG, context->respSize);
	resp = (fs_MsgReportQuotaResp_t *)context->respHdr;
	bzero(context->respHdr, context->respSize);
	context->respHdr->requestType = req->msgHdr.requestType;
	context->respHdr->clientPrivate = req->msgHdr.clientPrivate;
	context->respHdr->clientId = req->msgHdr.clientId;
	context->initRespSize = context->respSize;
	context->initRespSize = context->respSize;
#endif
    

	/*
	 * Now call the appropriate quota reporting routine
	 */

	FS_ASSERT(((req->flag & (FS_QUOTA_AGGREGATE | FS_QUOTA_USER |
						   FS_QUOTA_GROUP)) == FS_QUOTA_AGGREGATE) ||
			  ((req->flag & (FS_QUOTA_AGGREGATE | FS_QUOTA_USER |
						   FS_QUOTA_GROUP)) == FS_QUOTA_GROUP) ||
			  ((req->flag & (FS_QUOTA_AGGREGATE | FS_QUOTA_USER |
						   FS_QUOTA_USER)) == FS_QUOTA_USER));

	FS_ASSERT((req->flag & ~(FS_QUOTA_DEFAULT | FS_QUOTA_AGGREGATE |
							FS_QUOTA_USER | FS_QUOTA_GROUP)) == 0);


	
	if ((req->flag & (FS_QUOTA_AGGREGATE | FS_QUOTA_DEFAULT)) ==
		FS_QUOTA_AGGREGATE) {

			/*
		 	 * Report aggregate quota for this fileset
		 	 */

			return fs_reportFileSetQuota(context, context->fileSystem,
										 req, resp);

	} else if ((req->flag & (FS_QUOTA_USER | FS_QUOTA_GROUP |
                             FS_QUOTA_DEFAULT)) == (FS_QUOTA_USER) ||
                (req->flag & (FS_QUOTA_USER | FS_QUOTA_GROUP |
                             FS_QUOTA_DEFAULT)) == (FS_QUOTA_GROUP)) {

			/*
		 	 * Report user/group quota for this fileset
		 	 */
			return fs_reportUGQuota(context, context->fileSystem,
									 req, resp);
	} else {
			resp->numberResponses = 0;
			return FS_ERR_INVAL;
	}

	/* NOTREACHED */
}

/*
 * Set user/group or fileset quota or default quotas
 */


int32
fs_setQuotaThread(fs_Context	*context)
{
	fs_MsgSetQuotaReq_t			*req;
	fs_MsgSetQuotaResp_t		*resp;
	fs_QuotaBlk_t				*qblk;
	fs_FileSetIncoreRecord_t	*fset;
	fs_Status					status;
	fs_Inode					*root;
	int32						rc;

	resp = (fs_MsgSetQuotaResp_t *) context->respHdr;
	req = (fs_MsgSetQuotaReq_t *) context->reqHdr;


	/*
	 * For now we don't support multiple set quota requests
	 */

	FS_ASSERT(req->numberLimits < 2);
	if (req->numberLimits < 1) {
		return FS_ERR_INVAL;
	}

	/*
	 * findFset returns a locked root inode, so that the fileset
	 * cannot be removed while we are operating on it
	 */

	status = fs_findFset(context, context->fileSystem, req->path, &root, &fset); 
	if (status != FS_ERR_NONE) {
		return status;
	}

	qblk = &req->limits[0];

	FS_ASSERT(((qblk->flag & (FS_QUOTA_AGGREGATE | FS_QUOTA_USER |
							 FS_QUOTA_GROUP)) ==
							FS_QUOTA_AGGREGATE) ||
			   ((qblk->flag & (FS_QUOTA_AGGREGATE | FS_QUOTA_USER |
							   FS_QUOTA_GROUP)) ==
							   FS_QUOTA_GROUP) ||
			   ((qblk->flag & (FS_QUOTA_AGGREGATE | FS_QUOTA_USER |
							   FS_QUOTA_GROUP)) ==
							   FS_QUOTA_USER));

	if (qblk->flag & (FS_QUOTA_AGGREGATE | FS_QUOTA_DEFAULT)) {

		/*
		 * Set aggregate or default quotas
		 */

		status = fs_setFileSetQuota(context, context->fileSystem, fset, qblk);

	} else {

		/*
		 * Set user / group quotas
		 */
		fs_lockShared(context, &fset->lock, NULL);
		fs_setUGQuota(context, context->fileSystem, fset, qblk);
		fs_unlockShared(context, &fset->lock);

	}

	fs_inodeRelease(context, root);

    if (status != FS_ERR_NONE)  {
         return status;
    }
          
	/*
	 * Commit the transaction and unlock structures
	 */

    rc = fs_logCommitEntry(context);
    if (rc != NFX_OK)
    {
        fs_error(context, FS_INTERNAL_ERR, "commit");
        return FS_ERR_IO;
    }
	return FS_ERR_NONE;
	
}


/*
 * Get quota for a user/group/aggregate or default user/group/aggregate
 * quotas
 */

int32
fs_getQuotaThread(fs_Context	*context)
{
	fs_MsgGetQuotaReq_t			*req;
	fs_MsgGetQuotaResp_t		*resp;
	fs_QuotaBlk_t				*qblk;
	fs_FileSetIncoreRecord_t	*fset;
	fs_Status					status;
	uint32						id;
	int							flag, i;
	fs_QuotaIncoreRecord_t		*quota;
	fs_Inode					*root;

	resp = (fs_MsgGetQuotaResp_t *) context->respHdr;
	req = (fs_MsgGetQuotaReq_t *) context->reqHdr;

	resp->numberResponse = 0;
	/*
	 * For now we don't support multiple set quota requests
	 */

	FS_ASSERT(req->numberEntities < 5);
	if (req->numberEntities < 1) {
		return FS_ERR_INVAL;
	}

	status = fs_findFset(context, context->fileSystem, req->path, &root, &fset); 
	if (status != FS_ERR_NONE) {
		return status;
	}

    for (i = 0; i < req->numberEntities; i++)   {
    	id = req->entities[i].id;
    	flag = req->entities[i].flag;
    	qblk = &resp->responses[i];

    	FS_ASSERT(((flag & (FS_QUOTA_AGGREGATE | FS_QUOTA_USER |
    							 FS_QUOTA_GROUP)) ==
    							FS_QUOTA_AGGREGATE) ||
    			   ((flag & (FS_QUOTA_AGGREGATE | FS_QUOTA_USER |
    							   FS_QUOTA_GROUP)) ==
    							   FS_QUOTA_GROUP) ||
    			   ((flag & (FS_QUOTA_AGGREGATE | FS_QUOTA_USER |
	    						   FS_QUOTA_GROUP)) ==
    							   FS_QUOTA_USER));
    
        //Overload flag with QUOTA_ON / QUOTA_TURN_ON flags
    	NFX_SPIN_LOCK(&fset->spinlock);
    	if (fset->incoreFlag & FS_FILESET_QUOTA_ON) {
    		flag |= FS_QUOTA_ON;
    	}
        if (fset->incoreFlag & FS_FILESET_QUOTA_ON_THREAD)  {
    		flag |= FS_QUOTA_TURN_ON;
    	}
    	NFX_SPIN_UNLOCK(&fset->spinlock);
            
    	if (flag & (FS_QUOTA_AGGREGATE | FS_QUOTA_DEFAULT)) {

    		if ((flag & (FS_QUOTA_DEFAULT | FS_QUOTA_AGGREGATE)) ==
    			FS_QUOTA_AGGREGATE) {
                    
    			/*
    			 * Retrieve aggregate quota
    			 */
   
    			qblk->flag = flag;
    			NFX_SPIN_LOCK(&fset->spinlock);
                if (fset->cachedRecord.fileSetFlags & FS_FILESET_USE_QDEFAULTS)
                {
                    qblk->flag |= FS_QUOTA_USE_DEFAULTS;
                }
    			qblk->id = fset->cachedRecord.rootInode;
    			qblk->inodeSoftLimit =
    				fset->cachedRecord.fileSetQuota.inodeSoftLimit;
    			qblk->inodeHardLimit =
    				fset->cachedRecord.fileSetQuota.inodeHardLimit;
    			qblk->inodeTimeLimit =
    				fset->cachedRecord.fileSetQuota.inodeTimeLimit;
    			qblk->blockSoftLimit =
    				fset->cachedRecord.fileSetQuota.blockSoftLimit;
    			qblk->blockHardLimit =
    				fset->cachedRecord.fileSetQuota.blockHardLimit;
    			qblk->blockTimeLimit =
    				fset->cachedRecord.fileSetQuota.blockTimeLimit;

    			if (flag & FS_QUOTA_ON)	{
    				qblk->inodesConsumed = 
								fset->cachedRecord.fileSetConsumption.inodes;
    				qblk->blocksConsumed =
					   			fset->cachedRecord.fileSetConsumption.blocks;

    				if (fset->cachedRecord.fileSetConsumption.timeExtraInodes
                                                                         == 0)
    					qblk->inodeSoftLimitTime = 0;
    				else	
    					qblk->inodeSoftLimitTime = fs_getTimeOfDaySecs()
    					- fset->cachedRecord.fileSetConsumption.timeExtraInodes;
				
	    			if (fset->cachedRecord.fileSetConsumption.timeExtraBlocks
                                                                         == 0)
    					qblk->blockSoftLimitTime = 0;
    				else	
    					qblk->blockSoftLimitTime = fs_getTimeOfDaySecs()
						- fset->cachedRecord.fileSetConsumption.timeExtraBlocks;
    			}
    			NFX_SPIN_UNLOCK(&fset->spinlock);
#if 0
TOBEUPDATED
    		} else if (flag & (FS_QUOTA_USER)) {

    			/*
    			 * Retrieve Default user quota
    			 */

	    		qblk->flag = flag & ~(FS_QUOTA_ON | FS_QUOTA_TURN_ON);
    			NFX_SPIN_LOCK(&fset->spinlock);
    			qblk->id = fset->cachedRecord.rootInode;
    			qblk->inodeSoftLimit =
    				fset->cachedRecord.defaultUserQuota.inodeSoftLimit;
    			qblk->inodeHardLimit =
    				fset->cachedRecord.defaultUserQuota.inodeHardLimit;
    			qblk->inodeTimeLimit =
    				fset->cachedRecord.defaultUserQuota.inodeTimeLimit;
    			qblk->blockSoftLimit =
    				fset->cachedRecord.defaultUserQuota.blockSoftLimit;
    			qblk->blockHardLimit =
    				fset->cachedRecord.defaultUserQuota.blockHardLimit;
    			qblk->blockTimeLimit =
    				fset->cachedRecord.defaultUserQuota.blockTimeLimit;
    			NFX_SPIN_UNLOCK(&fset->spinlock);
    		} else if (flag & (FS_QUOTA_GROUP)) {

    			/*
    			 * Retrieve Default group quota
    			 */
	    		qblk->flag = flag & ~(FS_QUOTA_ON | FS_QUOTA_TURN_ON);
    			NFX_SPIN_LOCK(&fset->spinlock);
    			qblk->id = fset->cachedRecord.rootInode;
    			qblk->inodeSoftLimit =
    				fset->cachedRecord.defaultGroupQuota.inodeSoftLimit;
    			qblk->inodeHardLimit =
    				fset->cachedRecord.defaultGroupQuota.inodeHardLimit;
    			qblk->inodeTimeLimit =
    				fset->cachedRecord.defaultGroupQuota.inodeTimeLimit;
    			qblk->blockSoftLimit =
    				fset->cachedRecord.defaultGroupQuota.blockSoftLimit;
    			qblk->blockHardLimit =
    				fset->cachedRecord.defaultGroupQuota.blockHardLimit;
    			qblk->blockTimeLimit =
    				fset->cachedRecord.defaultGroupQuota.blockTimeLimit;
			    NFX_SPIN_UNLOCK(&fset->spinlock);
#endif
    		} else {

    			/*
    			 * Retrieve Default aggregate quota
    			 */
    			qblk->flag = flag & ~(FS_QUOTA_ON | FS_QUOTA_TURN_ON);
    			NFX_SPIN_LOCK(&fset->spinlock);
    			qblk->id = fset->cachedRecord.rootInode;
    			qblk->inodeSoftLimit =
    				fset->cachedRecord.defaultAggregateQuota.inodeSoftLimit;
    			qblk->inodeHardLimit =
    				fset->cachedRecord.defaultAggregateQuota.inodeHardLimit;
    			qblk->inodeTimeLimit =
    				fset->cachedRecord.defaultAggregateQuota.inodeTimeLimit;
    			qblk->blockSoftLimit =
    				fset->cachedRecord.defaultAggregateQuota.blockSoftLimit;
    			qblk->blockHardLimit =
    				fset->cachedRecord.defaultAggregateQuota.blockHardLimit;
    			qblk->blockTimeLimit =
    				fset->cachedRecord.defaultAggregateQuota.blockTimeLimit;
			    NFX_SPIN_UNLOCK(&fset->spinlock);
    		}
    	} else {
#if 0
TOBEUPDATED
    		quota = fs_getUGQuota(context, context->fileSystem, fset, flag,
    							  id, FALSE);
    		FS_ASSERT(quota != NULL);
    
    		qblk->flag = flag;
    		qblk->id = id;
    		NFX_SPIN_LOCK(&quota->spinlock);
    		qblk->inodeSoftLimit = quota->cachedRecords.quota.inodeSoftLimit;
    		qblk->inodeHardLimit = quota->cachedRecords.quota.inodeHardLimit;
    		qblk->inodeTimeLimit = quota->cachedRecords.quota.inodeTimeLimit;
    		qblk->blockSoftLimit = quota->cachedRecords.quota.blockSoftLimit;
    		qblk->blockHardLimit = quota->cachedRecords.quota.blockHardLimit;
    		qblk->blockTimeLimit = quota->cachedRecords.quota.blockTimeLimit;
    		if (flag & FS_QUOTA_ON)	{
    			qblk->inodesConsumed = quota->cachedRecords.consumption.inodes;
    			qblk->blocksConsumed = quota->cachedRecords.consumption.blocks;
    			if (quota->cachedRecords.consumption.timeExtraInodes == 0)
    				qblk->inodeSoftLimitTime = 0;
    			else		
    				qblk->inodeSoftLimitTime = fs_getTimeOfDaySecs()
							- quota->cachedRecords.consumption.timeExtraInodes;
    			if (quota->cachedRecords.consumption.timeExtraBlocks == 0)
    				qblk->blockSoftLimitTime = 0;
    			else
    				qblk->blockSoftLimitTime = fs_getTimeOfDaySecs()
							- quota->cachedRecords.consumption.timeExtraBlocks;
       		}
            if (quota->cachedRecords.flags & FS_UG_DEFAULT_QUOTALIMITS) {
                qblk->flag |= FS_QUOTA_USE_DEFAULTS;
            }
    		NFX_SPIN_UNLOCK(&quota->spinlock);
    		fs_releQuota(quota);
#endif
    	}
    }
    
	resp->numberResponse = req->numberEntities;
    
	fs_inodeRelease(context, root);
	return FS_ERR_NONE;

}

/*
 * Do quota on job
 */

PRIVATE void
fs_quotaOnThread2(esm_handle_t    handle,
                  void            *arg)
{
    fs_Context               *context = (fs_Context *)arg;
	fs_FileSetIncoreRecord_t *fset = 
                                (fs_FileSetIncoreRecord_t*)context->extraArg;
	int32						rc;

    // Lock the global access lock
    fs_lockShared(context, &context->fileSystem->accessLock, NULL);
    context->flags |= FS_CONTEXT_ACCESS_LOCK_ACQUIRED;
    // Lock the global modify lock
    fs_lockShared(context, &context->fileSystem->modifyLock, NULL);        
    context->flags |= FS_CONTEXT_MODIFY_LOCK_ACQUIRED;

    fs_logBeginTransaction(context);
    
    FS_TRACE(context, FS_TRACE_REQ, ("Quota On - Zapping fsets\n"));
    fs_zapFilesetConsumptionInfo(context, fset);
    FS_TRACE(context, FS_TRACE_REQ, ("Quota On - Zapping users\n"));
    fs_zapConsumptionInfo(context, fset, FS_QUOTA_USER);
    FS_TRACE(context, FS_TRACE_REQ, ("Quota On - Zapping groups\n"));
    fs_zapConsumptionInfo(context, fset, FS_QUOTA_GROUP);
    FS_TRACE(context, FS_TRACE_REQ, ("Quota On - Invalidating users\n"));
    fs_invalidateIncoreQuota(context, fset, FS_QUOTA_USER);
    FS_TRACE(context, FS_TRACE_REQ, ("Quota On - Invalidating groups\n"));
    fs_invalidateIncoreQuota(context, fset, FS_QUOTA_GROUP);

	fs_lockExclusive(context, &fset->lock, NULL);
	NFX_SPIN_LOCK(&fset->spinlock);
    fset->incoreFlag |= FS_FILESET_QUOTA_TURN_ON;
    fset->inodeSeen = 0;
	NFX_SPIN_UNLOCK(&fset->spinlock);
	fs_unlockExclusive(context, &fset->lock);
    
    FS_TRACE(context, FS_TRACE_REQ, ("Quota On - Counting...\n"));
	fs_quotaOn(context, context->fileSystem, fset);
    FS_TRACE(context, FS_TRACE_REQ, ("Quota On - Counted !\n"));

    rc = fs_logCommitEntry(context);
    if (rc != NFX_OK)
    {
        fs_error(context, FS_INTERNAL_ERR, "commit");
    }

    //release global modify lock
    fs_unlockShared(context, &context->fileSystem->modifyLock);
    context->flags &= ~FS_CONTEXT_MODIFY_LOCK_ACQUIRED;
    //release global access lock
    fs_unlockShared(context, &context->fileSystem->accessLock);
    context->flags &= ~FS_CONTEXT_ACCESS_LOCK_ACQUIRED;

    fs_threadTerminate(context);
}

/*
 * Turn quota on on a fileset
 */

int32
fs_quotaOnThread(fs_Context	*context)
{
	fs_MsgQuotaOnReq_t			*req;
	fs_MsgQuotaOnResp_t			*resp;
	fs_FileSetIncoreRecord_t	*fset;
	fs_Status					status;
	fs_Inode					*root;

	resp = (fs_MsgQuotaOnResp_t *) context->respHdr;
	req = (fs_MsgQuotaOnReq_t *) context->reqHdr;

	status = fs_findFset(context, context->fileSystem, req->path,
			     &root, &fset);
	if (status != FS_ERR_NONE) {
		return status;
	}

	fs_lockExclusive(context, &fset->lock, NULL);
	NFX_SPIN_LOCK(&fset->spinlock);
	if (fset->incoreFlag & FS_FILESET_QUOTA_ON) {
		// Quota is already on, return success, quota on
		NFX_SPIN_UNLOCK(&fset->spinlock);
		fs_unlockExclusive(context, &fset->lock);
		fs_inodeRelease(context, root);
        resp->status = FS_QUOTA_ON;
		return FS_ERR_NONE;
	} else if (fset->incoreFlag & FS_FILESET_QUOTA_ON_THREAD) {
			// Quota is being turned on. return success, quota being turned on.
			NFX_SPIN_UNLOCK(&fset->spinlock);
			fs_unlockExclusive(context, &fset->lock);
            fs_inodeRelease(context, root);
            resp->status = FS_QUOTA_TURN_ON;
            return FS_ERR_NONE;
	}

	/*
	 * Quota is not on and is not being turned on.
	 * do the work
	 */

	FS_ASSERT(fs_lockOwnedExclusive(context, &fset->lock));

	fset->incoreFlag |= FS_FILESET_QUOTA_ON_THREAD;
	NFX_SPIN_UNLOCK(&fset->spinlock);
	fs_unlockExclusive(context, &fset->lock);
    
    //Release the root earlier to prevent lock timeout if quota on is too long
    //09/04/2002
	fs_inodeRelease(context, root);

    fs_threadFork(context, fs_quotaOnThread2, TRUE, FALSE, 
                  FS_DAEMON_TYPE_QUOTA_ON, fset);
    
    resp->status = FS_QUOTA_TURN_ON;
	return FS_ERR_NONE;
}

/*
 * Turn quota off on a fileset
 */

int32
fs_quotaOffThread(fs_Context	*context)
{
	fs_MsgQuotaOffReq_t			*req;
	fs_MsgQuotaOffResp_t		*resp;
	fs_FileSetIncoreRecord_t	*fset;
	fs_Status					status;
	fs_Inode					*root;
	int32						rc;

	resp = (fs_MsgQuotaOffResp_t *) context->respHdr;
	req = (fs_MsgQuotaOffReq_t *) context->reqHdr;

	status = fs_findFset(context, context->fileSystem, req->path, &root, &fset);
	if (status != FS_ERR_NONE) {
		return status;
	}
    resp->status = 0;

    //We try without the traverse lock first to give a better answer to the
    //operator (if get the traverseLock fails we will only return BUSY)
	NFX_SPIN_LOCK(&fset->spinlock);
	if (fset->incoreFlag & FS_FILESET_QUOTA_ON_THREAD) {
        //Quota is being turned on, do nothing
        NFX_SPIN_UNLOCK(&fset->spinlock);
		fs_inodeRelease(context, root);
        resp->status = FS_QUOTA_TURN_ON;
        return FS_ERR_NONE;        
	}
	NFX_SPIN_UNLOCK(&fset->spinlock);
    
    if (!fs_trylockExclusive(context, &fset->traverseLock)) {
        return FS_ERR_BUSY;
    }
	fs_lockExclusive(context, &fset->lock, NULL);
	NFX_SPIN_LOCK(&fset->spinlock);
	if (fset->incoreFlag & FS_FILESET_QUOTA_ON) {
            
		//Quota on, turn it off
        
		NFX_SPIN_UNLOCK(&fset->spinlock);
		fs_unlockExclusive(context, &fset->lock);

		fs_quotaOff(context, context->fileSystem, fset);
		fs_unlockExclusive(context, &fset->traverseLock);
		fs_inodeRelease(context, root);
    	rc = fs_logCommitEntry(context);
    	if (rc != NFX_OK)
    	{
        	fs_error(context, FS_INTERNAL_ERR, "commit");
        	return FS_ERR_IO;
    	}
		return FS_ERR_NONE;
	} else if (fset->incoreFlag & FS_FILESET_QUOTA_ON_THREAD) {
            
        //Quota is being turned on, do nothing
        
        resp->status = FS_QUOTA_TURN_ON;
	}

	NFX_SPIN_UNLOCK(&fset->spinlock);
	fs_unlockExclusive(context, &fset->lock);
	fs_unlockExclusive(context, &fset->traverseLock);
	fs_inodeRelease(context, root);
	return FS_ERR_NONE;
}

/*
 * Hold the quota structure
 */

EXPORTED void
fs_holdQuota(
	fs_QuotaIncoreRecord_t	*quota)
{
	FS_ASSERT(quota != NULL);
	NFX_SPIN_LOCK(&(quota->spinlock));
	quota->count++;
    qDbgCnt[quotaHoldCnt]++;

	/*
	 * If quota count is 1, then take it off the freelist
	 */

	if (quota->count == 1) {
		DLL_REMOVE(&fs_quotaCache.quotaFreeList, quota, freeList);
	}
	NFX_SPIN_UNLOCK(&(quota->spinlock));

}

/*
 * Release the quota structure
 */

EXPORTED void
fs_releQuota(
	fs_QuotaIncoreRecord_t	*quota)
{
	FS_ASSERT(quota != NULL);
    FS_ASSERT(quota->count != 0);
	NFX_SPIN_LOCK(&(quota->spinlock));
	quota->count--;
    qDbgCnt[quotaReleaseCnt]++;
    
	/*
	 * If quota count is 0, then put it in the freelist
	 */

	if (quota->count == 0) {
		DLL_INSERT_TAIL(&fs_quotaCache.quotaFreeList, quota, freeList);
	}
	NFX_SPIN_UNLOCK(&(quota->spinlock));
    
}

/*
 * Initialize the quota structure
 */

PRIVATE	void
fs_initializeQuota(
	fs_FileSystem           *core,
	fs_QuotaIncoreRecord_t  *quota,
    int     type,
	uint32	id,
	uint64	fileSetId)
{

	fs_QuotaDiskRecord_t	*incore = &quota->cachedRecords;
	/*
	 * XXX - Assert that the appropriate lock is held
	 */

	quota->freeList.prev = quota->freeList.next = NULL;
	quota->hashList.prev = quota->hashList.next = NULL;
    quota->volId.whole_thing = core->volId.whole_thing;
	quota->id = id;
	quota->fileSetId = fileSetId;
	quota->flag = FS_QUOTA_INCORE_NEEDS_INIT;
	if (type & FS_QUOTA_USER) {
        quota->flag |= FS_QUOTA_INCORE_USER;
    } else {
        quota->flag |= FS_QUOTA_INCORE_GROUP;
    }
	quota->count = 0;
	bzero(incore, sizeof (fs_QuotaDiskRecord_t));

}

/*
 * Put the quota structure on the hash queues
 */

PRIVATE void
fs_hashInQuota(
	fs_QuotaIncoreRecord_t	*quota,
	int						type,
	uint32					id,
	int						hash
)
{
	FS_ASSERT(hash == FS_QCACHE_HASH(id));

	FS_ASSERT(quota->count == 1);

	if (type & FS_QUOTA_USER) {
		DLL_INSERT_HEAD(&fs_quotaCache.userQuotaHashList[hash],	quota, 
                        hashList);
	} else {
		DLL_INSERT_HEAD(&fs_quotaCache.groupQuotaHashList[hash], quota, 
                        hashList);
	}
}

/*
 * Remove the quota structure from the hash queue
 */

PRIVATE void
fs_unHashQuota(
	fs_QuotaIncoreRecord_t	*quota
)
{
	int hash = FS_QCACHE_HASH(quota->id);

	FS_ASSERT(quota->count == 0);

	if (quota->flag & FS_QUOTA_INCORE_USER) {
		DLL_REMOVE(&fs_quotaCache.userQuotaHashList[hash], quota, hashList);
	} else {
		DLL_REMOVE(&fs_quotaCache.groupQuotaHashList[hash], quota, 
                   hashList);
	}
}


/*
 * Allocate an incore quota structure if not already available in core
 */

PRIVATE fs_QuotaIncoreRecord_t *
fs_allocateIncoreQuotaRecord(
	fs_Context					*context,
	fs_FileSystem				*core,
	fs_FileSetIncoreRecord_t	*fset,
	int							type,
	uint32						id)
{
	int	steal = 0;
	int	hash = FS_QCACHE_HASH(id);
	fs_QuotaIncoreRecord_t	*quota, *nquota = NULL;
	nfx_spinlock_t		*listlock;

	listlock = &fs_quotaCache.spin;

	/*
	 * XXX - Assert that the list lock is not held
	 */

retry:

	NFX_SPIN_LOCK(listlock);
	if (type & FS_QUOTA_USER) {
			DLL_ITERATE(quota, &fs_quotaCache.userQuotaHashList[hash], hashList)
            {
				if (quota == NULL || quota == DLL_NIL) {
					break;
				}
				if (quota->id == id
                 && quota->fileSetId == fset->fileSetId
                 && quota->volId.whole_thing == core->volId.whole_thing)    {
					break;
				}
			}
	} else {
			DLL_ITERATE(quota, &fs_quotaCache.groupQuotaHashList[hash], 
                        hashList)
            {
				if (quota == NULL || quota == DLL_NIL) {
					break;
				}
				if (quota->id == id
                 && quota->fileSetId == fset->fileSetId
                 && quota->volId.whole_thing == core->volId.whole_thing)    {
					break;
				}
			}
	}

	/*
	 * Quota structure was found. Return it
	 */

	if (quota != NULL && quota != DLL_NIL) {
		fs_holdQuota(quota);

		/*
		 * If we allocated memory for nquota, free it
		 */

		if (nquota != NULL) {
			nquota->count = 0;
            qDbgCnt[quotaReleaseCnt]++;
			DLL_INSERT_HEAD(&fs_quotaCache.quotaFreeList, nquota, freeList);
		}
		NFX_SPIN_UNLOCK(listlock);
        fs_quotaCache.hitCount++;
		return quota;
	}

	if (nquota == NULL &&
	    (fs_quotaCache.quotasCached >= fs_quotaCache.quotaCacheLimit || steal))
    {

		DLL_REMOVE_HEAD(&fs_quotaCache.quotaFreeList, quota, freeList);

		if (quota == NULL) {
			FS_ASSERT(steal > 0);
			NFX_SPIN_UNLOCK(listlock);

			if (steal > 10) {

				/*
				 * We retried many a times.
				 * must be a bug panic
				 */

				fs_panic("%s:%s:%d",
					 "Cannot allocate quota structures",
					 __FILE__, __LINE__);
			}

			/*
			 * There isn't memory available and there aren't
			 * any free structures available. Sleep for a second
			 */

			FS_ASSERT(FALSE); //sleep(1);
			steal++;
			goto retry;
		}
        
        fs_quotaCache.missCount++;

        /*
         * Remove the quota structure from the hash table
         */ 
        fs_unHashQuota(quota);

		fs_initializeQuota(core, quota, type, id, fset->fileSetId);

		/*
	 	 * We are still holding the list lock and have just
		 * removed this entry from the freelist. We can
		 * arbitrarily set the id to 1
		 */
		quota->count = 1;
        qDbgCnt[quotaHoldCnt]++;
		/*
		 * We did not drop the list lock at any time, so no new
		 * entry would be created. We can safely assume that
		 * the quota structure of identity we need is not created
		 */

		fs_hashInQuota(quota, type, id, hash);

		/*
		 * Assert that we state that disk quota needs to be read
		 * in from the disk
		 */

		FS_ASSERT((quota->flag & FS_QUOTA_INCORE_NEEDS_INIT) != 0);

		NFX_SPIN_UNLOCK(listlock);
		return quota;
	}


	/*
	 * We need to allocate more memory. malloc can never fail and
	 * will panic if it cannot allocate memory. Make sure that
	 * we don't allocate more than our partitioned limit.
 	 * The side effect is that we can hold spin locks while allocating
	 * memory. 
	 */

	if (fs_quotaCache.quotasCached < fs_quotaCache.quotaCacheLimit) {
		nquota = fs_ramAlloc(FS_RAM_QUOTA,
				     sizeof (fs_QuotaIncoreRecord_t));
		bzero(nquota, sizeof (fs_QuotaIncoreRecord_t));

		/*
		 * Initialize the spinlock as soon as we allocated memory, so
		 * that we don't have to bother about reused or new structure
		 * in init routines
		 */

		NFX_SPIN_INIT(&nquota->spinlock,SPINLOCK_LEVEL_UNKNOWN);
		fs_lockInit(&nquota->lock, FS_LOCK_TYPE_QUOTA);

		/*
		 * Increment the count of cached quotas and cache miss
		 */

		fs_quotaCache.quotasCached++;
        fs_quotaCache.missCount++;

		/*
		 * We had previously allocated a structure and a new one
		 * with identify of our desire is not created, assign
		 * our identity to it, hash it in and return it
		 */

		quota = nquota;
		fs_initializeQuota(core, quota, type, id, fset->fileSetId);
		quota->count = 1;
        qDbgCnt[quotaHoldCnt]++;

		/*
		 * Assert that we state that disk quota needs to be read
		 * in from the disk
		 */

		FS_ASSERT((quota->flag & FS_QUOTA_INCORE_NEEDS_INIT) != 0);
		fs_hashInQuota(quota, type, id, hash);
		NFX_SPIN_UNLOCK(listlock);
		return quota;

	} else {

		NFX_SPIN_UNLOCK(listlock);

		/*
		 * We are out of memory, try stealing some from the freelist
		 */

		steal = 1;
		FS_ASSERT(FALSE); // sleep(1);
	}

	goto retry;

	/* NOTREACHED */
}

/*
 * Do a transaction to write the new quota information. It can
 * perform allocation to the quota file
 */

PRIVATE void
fs_initializeUGQuotaRecord(
	fs_Context					*context,
	fs_FileSystem				*core,
	fs_FileSetIncoreRecord_t	*fset,
	fs_Inode					*inode,
	fs_QuotaBlk_t				*qblk,
	int32						id,
	int							type)
{
#if 0
TOBEUPDATED
	uint64					    offset = id * sizeof (fs_QuotaDiskRecordPadded);
	fs_Buf					    *buf;
	fs_QuotaDiskRecordPadded	*dquota;
    fs_QuotaDiskRecord_t        *dqr;
	fs_QuotaRecord_t		    *defRecord;
	uint32					    boff = (uint32)(offset & (FS_BLOCK_SIZE - 1));
	int						    i;
	uint64					    wr_off = boff;
	int32					    wr_len = sizeof (fs_QuotaDiskRecord_t);
    uint32                      now = fs_getTimeOfDaySecs();

	/*
	 * Assert that the inode is held exclusively
	 */

	FS_ASSERT(fs_lockOwnedExclusive(context, &inode->lock) != 0);

	FS_ASSERT(((type & (FS_QUOTA_USER | FS_QUOTA_DEFAULT | FS_QUOTA_GROUP |
			  FS_QUOTA_AGGREGATE)) ==
			  FS_QUOTA_USER) ||
			  ((type & (FS_QUOTA_USER | FS_QUOTA_DEFAULT | FS_QUOTA_GROUP |
			  FS_QUOTA_AGGREGATE)) ==
			  FS_QUOTA_GROUP));

	if (type & FS_QUOTA_USER) {
		defRecord = &fset->cachedRecord.defaultUserQuota;
	} else {
		defRecord = &fset->cachedRecord.defaultGroupQuota;
	}


	/*
	 * We rely on the fact that for a hole, we would be returned
	 * a null buffer rather than a zeroed buffer
	 */

	buf = O_GET_EXCL(inode, offset >> FS_BLOCK_SHIFT);

	if (buf == NULL) {

		
		buf = O_CREATE(inode, offset >> FS_BLOCK_SHIFT);
        
		/*
		 * Initialize the buffer.data 
         * (we only set the quotaRecord field in the following loop)
		 */
        bzero(buf->u.data, FS_BLOCK_SIZE);

        FS_BUF_ZERO_ALL(buf);

		/*
		 * Copy default quotas from the fileset in to
		 * the new buffer
		 */

		dquota = buf->u.quota->recs;
		for (i = 0; i < FS_QUOTA_RECS_PER_PAGE; i++) {
            dqr = (fs_QuotaDiskRecord_t *)dquota;
			bcopy(defRecord, &dqr->quota, sizeof (fs_QuotaRecord_t));
            //remember that this quota is using the default limits
            dqr->flags = FS_UG_DEFAULT_QUOTALIMITS;
			dquota++;
		}

		wr_off = 0;
		wr_len = FS_BLOCK_SIZE;

		/*
		 * We also allocated an extent in the process. Add it
		 * to the inode.
		 */

        if ((offset + FS_BLOCK_SIZE) > inode->dinode.size) {
            inode->dinode.size = (offset + FS_BLOCK_SIZE);
            fs_inodeWrite(context, inode, INO_WRITE_LOG);
        }
	}

/*printf("initialize %cid %u quota rec: block %llu boff %u recoff %u\n", 
       (type & FS_QUOTA_USER) ? 'u' : 'g', id, (offset >> FS_BLOCK_SHIFT),
       boff, (boff >> FS_QUOTA_REC_SHIFT));
*/
	dquota = &buf->u.quota->recs[boff >> FS_QUOTA_REC_SHIFT];

	/*
	 * Now copy in the new quota
	 */

    dqr = (fs_QuotaDiskRecord_t *)dquota;
	dqr->quota.inodeHardLimit = qblk->inodeHardLimit;
	dqr->quota.inodeTimeLimit = qblk->inodeTimeLimit;
	dqr->quota.blockSoftLimit = qblk->blockSoftLimit;
	dqr->quota.blockHardLimit = qblk->blockHardLimit;
	dqr->quota.blockTimeLimit = qblk->blockTimeLimit;
    dqr->lastChanged = now;
    if (!(type & FS_QUOTA_USE_DEFAULTS))   {
        //This quota is not using the defaults limits
        dqr->flags &= ~FS_UG_DEFAULT_QUOTALIMITS;
    }

	/*
	 * Now write the buffer. The transaction commit would release
	 * buffer lock as well as inode lock
	 */

	FS_BUF_WRITE(buf, wr_off, wr_len);
#endif
}

/*
 * Get a quota structure for the user or group. Return a held
 * quota structure. The structure is not locked. In order to
 * do a transaction, the structure should be locked appropriately
 *
 * In order to look at the incore values, caller should always
 * acquire the quota spin lock before looking and using the values.
 * When we return here, we would return a held quota structure which
 * can be held by many contexts.
 */


EXPORTED fs_QuotaIncoreRecord_t *
fs_getUGQuota(
	fs_Context					*context,
	fs_FileSystem				*core,
	fs_FileSetIncoreRecord_t	*fset,	// fileset we are working on
	int							type,	// whether user or group quota
	uint32						id, 	// uid or gid
    boolean                     failIfDoesNotExist
)
{

	fs_QuotaIncoreRecord_t	*quota;
#if 0
TOBEUPDATED
	fs_QuotaDiskRecord_t	*dquot = NULL;
	uint64					quotaInum = 0;
	fs_Inode				*quotaInode = NULL;
	uint64					recordOffset;
	fs_Buf					*buf;
	int						broff;		//record offset in buffer
	int						get_fsetlock = 0;

	/*
	 * Assert that we have correct arguments. 
	 */

	FS_ASSERT(context != NULL && core != NULL && fset != NULL);

	/*
	 * XXX - We are currently not validating id. Should do when we
	 * have arrived at maximum id implementation
	 */

	FS_ASSERT(core != NULL && fset != NULL &&
		  (((type &
		    (FS_QUOTA_USER | FS_QUOTA_GROUP)) == FS_QUOTA_USER) ||
		  ((type &
		    (FS_QUOTA_USER | FS_QUOTA_GROUP)) == FS_QUOTA_GROUP)));

	quota = fs_allocateIncoreQuotaRecord(context, core, fset, type, id);
        
	/*
	 * XXX - Assert that quota is not locked and its spin lock is not held
	 */

	FS_ASSERT(fs_lockOwnedExclusive(context, &quota->lock) == 0 &&
			  fs_lockOwnedExclusive(context, &quota->lock) == 0);

	/*
	 * Check if we need to read the quota off the disk or allocate
	 * the quota structure to the quota file
	 */

	NFX_SPIN_LOCK(&quota->spinlock);
	if (!(quota->flag & FS_QUOTA_INCORE_NEEDS_INIT)) {
		NFX_SPIN_UNLOCK(&quota->spinlock);
		return quota;
	}
	NFX_SPIN_UNLOCK(&quota->spinlock);

	/*
	 * Read the quota information from the quota file. If
	 * quota does not exist, then we will have to allocate one
	 */

	if (type & FS_QUOTA_USER) {
		quotaInum = fset->cachedRecord.userQuotaInode;
	} else {
		quotaInum = fset->cachedRecord.groupQuotaInode;
	}

	recordOffset = id << FS_QUOTA_REC_SHIFT;
	broff = (recordOffset & (FS_BLOCK_SIZE-1)) >> FS_QUOTA_REC_SHIFT;

/*printf("get %cid %u quota rec: block %llu boff %llu recoff %u\n", 
       (type & FS_QUOTA_USER) ? 'u' : 'g', id, (recordOffset >> FS_BLOCK_SHIFT),
       (recordOffset & (FS_BLOCK_SIZE-1)), broff);
*/
       
retry:
	if (get_fsetlock) {
		fs_lockShared(context, &fset->lock, NULL);
	}

	/*
	 * Quotas are applicable only to live filesystem. Snapnum is
	 * always 0
	 */

	quotaInode = I_GET_SHARED(0, quotaInum);
	fs_lockExclusive(context, &quota->lock, NULL);

	FS_ASSERT(quotaInode != NULL);

	/*
	 * assert that quotaInode is held
	 */

	FS_ASSERT(fs_lockHeldShare(context, &quotaInode->lock) != 0);
	
	/*
	 * Get the buffer for the quota record
	 */

	FS_ASSERT(quotaInode->inum == quotaInum);
	buf = O_GET_SHARED(quotaInode, recordOffset >> FS_BLOCK_SHIFT);

	/*
	 * Locking hierarchy says that quota file inode lock is
	 * below quota block lock
	 */

	/*
	 * If there is a hole in the quotas file, then return default
	 * user quotas if allocation for the same is not requested
	 */

	if (buf != NULL) {

		/*
	 	 * Copy the data from the buffer into the quota structure
		 * and return
		 */

		NFX_SPIN_LOCK(&quota->spinlock);
		if (quota->flag & FS_QUOTA_INCORE_NEEDS_INIT) {
			dquot = (fs_QuotaDiskRecord_t *)&buf->u.quota->recs[broff];

			eee_ramCopy((char *)&quota->cachedRecords,
    				    (char *)dquot,
	    			    sizeof (fs_QuotaDiskRecord_t));

            if (quota->cachedRecords.flags & FS_UG_DEFAULT_QUOTALIMITS) {
                    
        		/*
        		 * We are copying default values from the
        		 * fileset. Ensure that the fileset structure is
        		 * stable. In most cases fset lock should be available
        		 * so we will try locking it so that we can violate
        		 * the locking hierarchy is possible
        		 */
        		if (get_fsetlock == 0 
                 && !fs_trylockShared(context, &fset->lock)) {
        			/*
        		 	 * drop every thing and try again
        			 */
        			get_fsetlock = 1;
            		NFX_SPIN_UNLOCK(&quota->spinlock);
            		fs_bufRelease(context, buf);
        			fs_unlockExclusive(context, &quota->lock);
        			fs_inodeRelease(context, quotaInode);
        			goto retry;
        		}
                if (type & FS_QUOTA_USER) {
                    bcopy(  &fset->cachedRecord.defaultUserQuota,
                            &quota->cachedRecords.quota,
                            sizeof (fs_QuotaRecord_t));
                } else {
                    bcopy(  &fset->cachedRecord.defaultGroupQuota,
                            &quota->cachedRecords.quota,
                            sizeof (fs_QuotaRecord_t));
                }
                //if we are here we will need to release the lock
       			get_fsetlock = 1;
            }
            
			quota->flag &= ~FS_QUOTA_INCORE_NEEDS_INIT;
		}
		NFX_SPIN_UNLOCK(&quota->spinlock);
		fs_bufRelease(context, buf);
		fs_unlockExclusive(context, &quota->lock);
		fs_inodeRelease(context, quotaInode);		
    	if (get_fsetlock) {
    		fs_unlockShared(context, &fset->lock);
    	}

	} else  {

        if (failIfDoesNotExist) {
			fs_unlockExclusive(context, &quota->lock);
			fs_inodeRelease(context, quotaInode);
    		fs_releQuota(quota);
            return NULL;
        }

		/*
		 * We are copying default values from the
		 * fileset. Ensure that the fileset structure is
		 * stable. In most cases fset lock should be available
		 * so we will try locking it so that we can violate
		 * the locking hierarchy is possible
		 */

		if (get_fsetlock == 0 &&
		    !fs_trylockShared(context, &fset->lock)) {
			/*
		 	 * drop every thing and try again
			 */

			get_fsetlock = 1;
			fs_unlockExclusive(context, &quota->lock);
			fs_inodeRelease(context, quotaInode);
			goto retry;
		}
			
		NFX_SPIN_LOCK(&quota->spinlock);
		if (quota->flag & FS_QUOTA_INCORE_NEEDS_INIT) {
            if (type & FS_QUOTA_USER) {
                bcopy(  &fset->cachedRecord.defaultUserQuota,
                        &quota->cachedRecords.quota,
                        sizeof (fs_QuotaRecord_t));
            } else {
                bcopy(  &fset->cachedRecord.defaultGroupQuota,
                        &quota->cachedRecords.quota,
                        sizeof (fs_QuotaRecord_t));
            }
            //remember that this quota is using the default limits
            quota->cachedRecords.flags |= FS_UG_DEFAULT_QUOTALIMITS;
		}
		NFX_SPIN_UNLOCK(&quota->spinlock);
		fs_unlockExclusive(context, &quota->lock);
		fs_inodeRelease(context, quotaInode);		
		fs_unlockShared(context, &fset->lock);
	}
#endif
	return quota;
}


/*
 * Account for the blocks and return how many blocks are allocable
 * in this particular context. Set the notify flag if required
 */

PRIVATE int
accountUserGroupBlocks(
    fs_Context          *context,
	fs_QuotaContext_t	*quotaContext,	// quota context
	uint64				*blocks,		// number of blocks
    uint32              *now,
    boolean             alwaysSucceed
)
{
	fs_QuotaIncoreRecord_t	*quota;
	fs_ConsumptionRecord_t	*qUsage;
	int			notify = 0;
    uint64      slimit, hlimit, tlimit, blks = *blocks;

	quota = quotaContext->quota;
	qUsage = &quota->cachedRecords.consumption;
    slimit = quota->cachedRecords.quota.blockSoftLimit;
    slimit = (slimit == 0) ? ~slimit : slimit;
    hlimit = quota->cachedRecords.quota.blockHardLimit;
    hlimit = (hlimit == 0) ? ~hlimit : hlimit;
    tlimit = quota->cachedRecords.quota.blockTimeLimit;
	tlimit = (tlimit == 0) ? ~tlimit : tlimit;

	if (qUsage->blocks + *blocks > slimit) {
        if (*now == 0)  {
            *now = fs_getTimeOfDaySecs();
        }
		if (qUsage->timeExtraBlocks == 0) {
			//soft quota just exceeded, store time and notify
			qUsage->timeExtraBlocks = *now;
			quotaContext->notify |= FS_EXCEED_SOFTLIMIT;
			notify++;
		} else {
			//soft quota was already exceeded
			if (*now - qUsage->timeExtraBlocks  > tlimit) {
				//time limit exceeded, soft quota is now "hard quota"
				*blocks = 0;
				if (!(quota->cachedRecords.flags & FS_UG_QNOT_BLK_TIMELIMIT)) {
					quota->cachedRecords.flags |= FS_UG_QNOT_BLK_TIMELIMIT;
					quotaContext->flagsWr++;
					quotaContext->notify |= FS_EXCEED_TIMELIMIT;
					notify++;
				}
			}
		}
	} else {
		if (qUsage->timeExtraBlocks != 0)	{
			//We are below softlimit and timeExtraBlocks has not been reset yet
			if (qUsage->blocks + *blocks < slimit) {
				quota->cachedRecords.flags &= ~FS_UG_QNOT_BLK_TIMELIMIT;
				quotaContext->flagsWr++;
				qUsage->timeExtraBlocks = 0;
			}
		}
	}
	
	if (*blocks != 0 && qUsage->blocks + *blocks > hlimit) {
		if (hlimit > qUsage->blocks)
			*blocks = hlimit - qUsage->blocks;
		else 
			*blocks = 0;
		if (*blocks == 0 
		 && !(quota->cachedRecords.flags & FS_UG_QNOT_BLK_HARDLIMIT)) {
			//hard quota exceeded but not notified yet
			quota->cachedRecords.flags |= FS_UG_QNOT_BLK_HARDLIMIT;
			quotaContext->flagsWr++;
			quotaContext->notify |= FS_EXCEED_HARDLIMIT;
			notify++;
		}
	} else {
		if (quota->cachedRecords.flags & FS_UG_QNOT_BLK_HARDLIMIT)	{
			//We are below hardlimit and hard quota notified flag is still set
			if (qUsage->blocks + *blocks < hlimit)	{
				quota->cachedRecords.flags &= ~FS_UG_QNOT_BLK_HARDLIMIT;
				quotaContext->flagsWr++;
			}
		}			
	}
	if (alwaysSucceed) {
        *blocks = blks;
    }
	if (*blocks != 0) {
		quota->cachedRecords.consumption.blocks += *blocks;
        quotaContext->count = *blocks;
	}
	if (notify) {
		quotaContext->notify |= FS_EXCEED_BLOCK;
	}
	return notify;
	
}

/*
 * Account for the inodes in the context structures and return how
 * many inodes are allocable. Set the notify flag if required
 */

PRIVATE int
accountUserGroupInodes(
    fs_Context          *context,
	fs_QuotaContext_t	*quotaContext,	// quota context
	uint64				*inodes,		// number of inodes 
    uint32              *now,
    boolean             alwaysSucceed
)
{
	fs_QuotaIncoreRecord_t	*quota;
	fs_ConsumptionRecord_t	*qUsage;
	int			notify = 0;
    uint64      slimit, hlimit, tlimit, inos = *inodes;

	quota  = quotaContext->quota;
	qUsage = &quota->cachedRecords.consumption;
    slimit = quota->cachedRecords.quota.inodeSoftLimit;
    slimit = (slimit == 0) ? ~slimit : slimit;
    hlimit = quota->cachedRecords.quota.inodeHardLimit;
    hlimit = (hlimit == 0) ? ~hlimit : hlimit;
    tlimit = quota->cachedRecords.quota.inodeTimeLimit;
	tlimit = (tlimit == 0) ? ~tlimit : tlimit;

	if (qUsage->inodes + *inodes > slimit) {
        if (*now == 0)  {
            *now = fs_getTimeOfDaySecs();
        }
		if (qUsage->timeExtraInodes == 0) {
			//soft quota just exceeded, store time and notify
			qUsage->timeExtraInodes = *now;
			quotaContext->notify |= FS_EXCEED_SOFTLIMIT;
			notify++;
		} else {
			//soft quota was already exceeded
			if (*now - qUsage->timeExtraInodes  > tlimit) {
				//time limit exceeded, soft quota is now "hard quota"
				*inodes = 0;
				if (!(quota->cachedRecords.flags & FS_UG_QNOT_INO_TIMELIMIT)) {
					quota->cachedRecords.flags |= FS_UG_QNOT_INO_TIMELIMIT;
					quotaContext->flagsWr++;
					quotaContext->notify |= FS_EXCEED_TIMELIMIT;
					notify++;
				}
			}
		}
	} else {
		if (qUsage->timeExtraInodes != 0)	{
			//We are below softlimit and timeExtraInodes has not been reset yet
			if (qUsage->inodes + *inodes < slimit) {
				quota->cachedRecords.flags &= ~FS_UG_QNOT_INO_TIMELIMIT;
				quotaContext->flagsWr++;
				qUsage->timeExtraInodes = 0;
			}
		}
	}
	
	if (*inodes != 0 && qUsage->inodes + *inodes > hlimit) {
		if (hlimit > qUsage->inodes)
			*inodes = hlimit - qUsage->inodes;
		else 
			*inodes = 0;
		if (*inodes == 0 
		 && !(quota->cachedRecords.flags & FS_UG_QNOT_INO_HARDLIMIT)) {
			//hard quota exceeded but not notified yet
			quota->cachedRecords.flags |= FS_UG_QNOT_INO_HARDLIMIT;
			quotaContext->flagsWr++;
			quotaContext->notify |= FS_EXCEED_HARDLIMIT;
			notify++;
		}
	} else {
		if (quota->cachedRecords.flags & FS_UG_QNOT_INO_HARDLIMIT)	{
			//We are below hardlimit and hard quota notified flag is still set
			if (qUsage->inodes + *inodes < hlimit)	{
				quota->cachedRecords.flags &= ~FS_UG_QNOT_INO_HARDLIMIT;
				quotaContext->flagsWr++;
			}
		}			
	}
	
	if (alwaysSucceed) {
        *inodes = inos;
    }
	if (*inodes != 0) {
		quota->cachedRecords.consumption.inodes += *inodes;
        quotaContext->count = *inodes;
	}
	if (notify) {
		quotaContext->notify |= FS_EXCEED_INODE;
	}
	return notify;
}

/*
 * Account for the blocks and return how many blocks are allocable
 * in this particular context. Set the notify flag if required
 */

PRIVATE int
accountFileSetBlocks(
    fs_Context          *context,
	fs_FsetContext_t	*fsetContext,	// quota context
	uint64				*blocks,		// number of blocks
    uint32              *now,
    boolean             alwaysSucceed
)
{
	fs_FileSetIncoreRecord_t	*fset;
	fs_ConsumptionRecord_t		*fsetUsage;
	int							notify = 0;
    uint64      slimit, hlimit, tlimit, blks = *blocks;

	fset = fsetContext->fset;
	fsetUsage = &fset->cachedRecord.fileSetConsumption;
    slimit = fset->cachedRecord.fileSetQuota.blockSoftLimit;
    slimit = (slimit == 0) ? ~slimit : slimit;
    hlimit = fset->cachedRecord.fileSetQuota.blockHardLimit;
    hlimit = (hlimit == 0) ? ~hlimit : hlimit;
	tlimit = fset->cachedRecord.fileSetQuota.blockTimeLimit;
	tlimit = (tlimit == 0) ? ~tlimit : tlimit;

	if ((fsetUsage->blocks + *blocks) > slimit) {
        if (*now == 0)  {
            *now = fs_getTimeOfDaySecs();
        }
		if (fsetUsage->timeExtraBlocks == 0) {
			//soft quota just exceeded, store time and notify
			fsetUsage->timeExtraBlocks = *now;
			fsetContext->notify |= FS_EXCEED_SOFTLIMIT;
			notify++;
		} else {
			//soft quota was already exceeded
			if (*now - fsetUsage->timeExtraBlocks > tlimit) {
				//time limit exceeded, soft quota is now "hard quota"
				*blocks = 0;
				if (!(fset->incoreFlag & FS_FSET_QNOT_BLK_TIMELIMIT)) {
					fset->incoreFlag |= FS_FSET_QNOT_BLK_TIMELIMIT;
					fsetContext->notify |= FS_EXCEED_TIMELIMIT;
					notify++;
				}
			}
		}
	} else {
		if (fsetUsage->timeExtraBlocks != 0) {
			//We are below softlimit and timeExtraBlocks has not been reset yet
			if (fsetUsage->blocks + *blocks < slimit) {
				fset->incoreFlag &= ~FS_FSET_QNOT_BLK_TIMELIMIT;
				fsetUsage->timeExtraBlocks = 0;
			}
		}
	}
	
	if (*blocks != 0 && (fsetUsage->blocks + *blocks) > hlimit) {
		if (hlimit > fsetUsage->blocks)
			*blocks = hlimit - fsetUsage->blocks;
		else
			*blocks = 0;
		if (*blocks == 0 && !(fset->incoreFlag & FS_FSET_QNOT_BLK_HARDLIMIT)) {
			//hard quota exceeded but not notified yet
			fset->incoreFlag |= FS_FSET_QNOT_BLK_HARDLIMIT;
			fsetContext->notify |= FS_EXCEED_HARDLIMIT;
			notify++;
		}
	} else {
		if (fset->incoreFlag & FS_FSET_QNOT_BLK_HARDLIMIT)	{
			//We are below hardlimit and hard quota notified flag is still set
			if (fsetUsage->blocks + *blocks < hlimit)	{
				fset->incoreFlag &= ~FS_FSET_QNOT_BLK_HARDLIMIT;
			}
		}
	}
	
	if (alwaysSucceed) {
        *blocks = blks;
    }
	if (*blocks != 0) {
		fset->cachedRecord.fileSetConsumption.blocks += *blocks;
        fsetContext->count = *blocks;
	}
	if (notify) {
		fsetContext->notify |= FS_EXCEED_BLOCK;
	}
	return notify;
}

/*
 * Account for the inodes in the context structures and return how
 * many inodes are allocable. Set the notify flag if required
 */

PRIVATE int
accountFileSetInodes(
    fs_Context          *context,
	fs_FsetContext_t	*fsetContext,	// quota context
	uint64				*inodes,		// number of inodes
    uint32              *now,
    boolean             alwaysSucceed
)
{
	fs_FileSetIncoreRecord_t	*fset;
	fs_ConsumptionRecord_t		*fsetUsage;
	int							notify = 0;
    uint64      slimit, hlimit, tlimit, inos = *inodes;

	fset = fsetContext->fset;
	fsetUsage = &fset->cachedRecord.fileSetConsumption;
    slimit = fset->cachedRecord.fileSetQuota.inodeSoftLimit;
    slimit = (slimit == 0) ? ~slimit : slimit;
    hlimit = fset->cachedRecord.fileSetQuota.inodeHardLimit;
    hlimit = (hlimit == 0) ? ~hlimit : hlimit;
	tlimit = fset->cachedRecord.fileSetQuota.inodeTimeLimit;
	tlimit = (tlimit == 0) ? ~tlimit : tlimit;

	if ((fsetUsage->inodes + *inodes) > slimit) {
        if (*now == 0)  {
            *now = fs_getTimeOfDaySecs();
        }
		if (fsetUsage->timeExtraInodes == 0) {
			//soft quota just exceeded, store time and notify
			fsetUsage->timeExtraInodes = *now;
			fsetContext->notify |= FS_EXCEED_SOFTLIMIT;
			notify++;
		} else {
			//soft quota was already exceeded
			if (*now - fsetUsage->timeExtraInodes > tlimit) {
				//time limit exceeded, soft quota is now "hard quota"
				*inodes = 0;
				if (!(fset->incoreFlag & FS_FSET_QNOT_INO_TIMELIMIT)) {
					fset->incoreFlag |= FS_FSET_QNOT_INO_TIMELIMIT;
					fsetContext->notify |= FS_EXCEED_TIMELIMIT;
					notify++;
				}
			}
		}
	} else {
		if (fsetUsage->timeExtraInodes != 0) {
			//We are below softlimit and timeExtraInodes has not been reset yet
			if (fsetUsage->inodes + *inodes < slimit) {
				fset->incoreFlag &= ~FS_FSET_QNOT_INO_TIMELIMIT;
				fsetUsage->timeExtraInodes = 0;
			}
		}
	}
	
	if (*inodes != 0 && (fsetUsage->inodes + *inodes) > hlimit) {
		if (hlimit > fsetUsage->inodes)
			*inodes = hlimit - fsetUsage->inodes;
		else
			*inodes = 0;
		if (*inodes == 0 && !(fset->incoreFlag & FS_FSET_QNOT_INO_HARDLIMIT)) {
			//hard quota exceeded but not notified yet
			fset->incoreFlag |= FS_FSET_QNOT_INO_HARDLIMIT;
			fsetContext->notify |= FS_EXCEED_HARDLIMIT;
			notify++;
		}
		
	} else {
		if (fset->incoreFlag & FS_FSET_QNOT_INO_HARDLIMIT)	{
			//We are below hardlimit and hard quota notified flag is still set
			if (fsetUsage->inodes + *inodes < hlimit)	{
				fset->incoreFlag &= ~FS_FSET_QNOT_INO_HARDLIMIT;
			}
		}
	}
	
	if (alwaysSucceed) {
        *inodes = inos;
    }
	if (*inodes != 0) {
		fset->cachedRecord.fileSetConsumption.inodes += *inodes;
        fsetContext->count = *inodes;
	}
	if (notify) {
		fsetContext->notify |= FS_EXCEED_INODE;
	}
	return notify;
}


/*
 * Write the on disk user/group quota notification state
 */
PRIVATE int
writeUGQuotaFlags(
    fs_Context          *context,
	fs_QuotaContext_t	*quotaContext,	// quota context
	uint16				type,			// user or group
	uint64				id,				// user/group id
	int					qtype			// blocks or inodes 
	)	
{
#if 0
TOBEUPDATED
	fs_QuotaIncoreRecord_t		*quota;
	fs_FileSetIncoreRecord_t	*fset;
	fs_Inode					*quotaInode;
	fs_Buf						*buf;
	fs_QuotaDiskRecord_t		*dquot;
	uint64						inum, offset, boff;
	
	quota = quotaContext->quota;
	fset = quotaContext->fset;

	fs_lockShared(context, &fset->lock, NULL);
	fs_lockExclusive(context, &quota->lock, NULL);
	
	if (type & FS_QUOTA_USER) {
		inum = fset->cachedRecord.userQuotaInode;
	} else {
		inum = fset->cachedRecord.groupQuotaInode;
	}

	quotaInode = I_GET_EXCL(0, inum);
	offset = id * sizeof (fs_QuotaDiskRecord_t);
	boff = (uint32)(offset & (FS_BLOCK_SIZE - 1));
	
	buf = O_GET_EXCL(quotaInode, offset >> FS_BLOCK_SHIFT);
	//buffer should always exist since we are changing flags on existing quotas
	FS_ASSERT(buf != NULL);

	dquot = (fs_QuotaDiskRecord_t *)(buf->u.data + boff);
	dquot->flags = quota->cachedRecords.flags;
	FS_BUF_WRITE(buf, ((char *)&dquot->flags - buf->u.data),
					sizeof (dquot->flags));
	
	if (qtype & FS_COUNT_BLOCKS)	{
		dquot->consumption.timeExtraBlocks = 
							quota->cachedRecords.consumption.timeExtraBlocks;
		FS_BUF_WRITE(buf, 
					((char *)&dquot->consumption.timeExtraBlocks - buf->u.data),
					sizeof (dquot->consumption.timeExtraBlocks));
	} else {
		dquot->consumption.timeExtraInodes = 
							quota->cachedRecords.consumption.timeExtraInodes;
		FS_BUF_WRITE(buf, 
					((char *)&dquot->consumption.timeExtraInodes - buf->u.data),
					sizeof (dquot->consumption.timeExtraInodes));
	}	
	
    if (!fs_inodeWasModified(context, quotaInode)) {
		fs_inodeReleaseExclusive(context, quotaInode);
	}
	fs_unlockExclusive(context, &quota->lock);
	fs_unlockShared(context, &fset->lock);

#endif
	return 0;
}


/*
 * commit to disk quota notification state changes for the whole quota context
 */
PRIVATE void
fs_commitQuotaContextNotifChanges(
	fs_Context				*context,	
	fs_QuotaContextList_t	*quotaContextList,
	boolean					releaseSharedLocksBefore,
	boolean					releaseQuotaHoldAfter)
{
	fs_QuotaContext_t			*quotaContext;
	fs_FsetContext_t			*fsetContext;
	fs_LogSummary				*logSummary;

	if (releaseSharedLocksBefore)	{
		DLL_ITERATE(quotaContext, &quotaContextList->uQuotaList, list) {
			fs_unlockShared(context, &quotaContext->quota->lock);
		}
		DLL_ITERATE(quotaContext, &quotaContextList->gQuotaList, list) {
			fs_unlockShared(context, &quotaContext->quota->lock);
		}
		DLL_ITERATE(fsetContext, &quotaContextList->fsetList, list) {
			fs_unlockShared(context, &fsetContext->fset->lock);
		}
	}

	//Create a nested transaction
	fs_logNestTransaction (context);
	logSummary = context->logSummary;
	
	DLL_ITERATE(quotaContext, &quotaContextList->uQuotaList, list) {
		if (quotaContext->flagsWr)	{
			writeUGQuotaFlags(context, quotaContext, FS_QUOTA_USER, 
							quotaContextList->uid, quotaContextList->flag);
		}
	}
	DLL_ITERATE(quotaContext, &quotaContextList->gQuotaList, list) {
		if (quotaContext->flagsWr)	{
			writeUGQuotaFlags(context, quotaContext, FS_QUOTA_GROUP, 
							quotaContextList->gid, quotaContextList->flag);
		}
	}
	
	//Commit nested transaction
	if (fs_logCommitEntry(context) != NFX_OK)	{
		fs_error(context, FS_INTERNAL_ERR, "nested commit");
    }
	
	if (releaseQuotaHoldAfter)	{
		DLL_ITERATE(quotaContext, &quotaContextList->uQuotaList, list) {
			fs_releQuota(quotaContext->quota);
		}
		DLL_ITERATE(quotaContext, &quotaContextList->gQuotaList, list) {
			fs_releQuota(quotaContext->quota);
			
		}
	}
}

		
/*
 * This routine reserves both user/group and aggregate quotas for an
 * operation against all the filesets. It returns the number of blocks/inodes
 * that are reserved and also returns information in the context
 * structure regarding quota and fset structures against which quota
 * needs to be accounted for when write completes.
 *
 * If the caller is requesting quota for inodes, make sure that the
 * number of resources requested is 1 and the inode is a directory.
 *
 * On return we return number of resources that are allowed. If the return
 * value is non zero, then a quotaContextList is pointed to by fs_FileSystem
 * structure which will be used to apply the changes to the on disk
 * structure when transaction is commited. Look at fs-quota.h and 
 * fs-fileset.h for detailed description of why we need to defer updating
 * on disk copy of quotas till the transaction is committed.
 *
 * quota structures pointed to by quotaContextList are held and when
 * the transaction is completed, they should be released using
 * fs_releQuota. File set structures are not held as the assumption
 * is that they will always be incore. If that assumption changes, we
 * will also need to hold fsets. (Wait till impact is visible on performance)
 *
 * Speaking of performance, fs_getQuotaReservation can be a costly routine
 * especially if it has to get user / group quota structures from disk.
 * The assumption is that administrator knows what penalty he/she is
 * paying by enabling quotas.
 *
 * We will be locking multiple filesets in this routine. The locking
 * hierarchy for filesets is from child to parent to ancestors. We will
 * also be holding multiple quota structures. The locking hierarchy is
 * that quota objects for uids of the lowest node are locked first followed
 * by parent, followed by ancestors. The same is true for group quotas.
 * For each level user quotas are locked before group quotas.
 */

EXPORTED uint64
fs_getQuotaReservation(
	fs_Context	*context,	
	fs_FileSystem	*core,		
	fs_Inode	*inode,
	uint64		requested,	// count of resources requested
	int			flag,   	// count inodes or blocks
	uint32		uid,
	uint32		gid,
	int			alwaysSucceedUnlessNoSpace,
	fs_QuotaContextList_t	**list
)
{

	fs_QuotaContextList_t		*quotaContextList = NULL;
	fs_FileSetIncoreRecord_t	*tfset;
	fs_FsetContext_t			*fsetContext;
	fs_QuotaContext_t			*quotaContext;
	fs_QuotaIncoreRecord_t		*quota = NULL;
	int							notify = 0;
	int							quotaFlagsWrite = 0;
	uint64						granted = requested;
	fs_Inode					*fsetInfoInode;
    uint32                      now = 0;

	FS_ASSERT(requested != 0 && context != NULL && core != NULL 
           && inode != NULL);
	FS_ASSERT(flag == FS_COUNT_INODES || flag == FS_COUNT_BLOCKS);

	FS_ASSERT(inode == NULL || fs_lockOwnedExclusive(context, &inode->lock));
	
	/*
	 * Build up the list of fileset structures that we want to
	 * operate on
	 */

	fsetInfoInode = I_GET_SHARED(0, FS_FILESET_INFO_INODE);

	/*
	 * Now that we have blocked out creation of any new filesets
	 * the parent chains are stable
	 */

	tfset = inode->fset = fs_getFset(context, core, inode);
    FS_ASSERT(tfset != NULL);

	while (tfset != NULL) {

		/*
		 * If the fset does not have quotas turned on or
		 * the allocation version is older than quota turn
		 * on allocation version, skip this fset
		 */

		fs_lockShared(context, &tfset->lock, NULL);
		NFX_SPIN_LOCK(&tfset->spinlock);
		if ((tfset->incoreFlag & (FS_FILESET_QUOTA_ON |
					  FS_FILESET_QUOTA_TURN_ON)) == 0) {
			NFX_SPIN_UNLOCK(&tfset->spinlock);
			fs_unlockShared(context, &tfset->lock);
			tfset = tfset->parent;
			continue;
		}


		/*
		 * If the quota is being turned on, quota on has not yet
		 * processed this inode, don't count it yet. Quota on will
		 * account for it when it gets here. This means that
		 * somebody would be able to sneak in allocation over the
		 * quota limit while quota is being turned on but not fully on
		 */

		if ((tfset->incoreFlag & FS_FILESET_QUOTA_TURN_ON) &&
			(inode->inum > tfset->inodeSeen) && (flag & FS_COUNT_BLOCKS)) {
			NFX_SPIN_UNLOCK(&tfset->spinlock);
			fs_unlockShared(context, &tfset->lock);
			tfset = tfset->parent;
			continue;
		}
		NFX_SPIN_UNLOCK(&tfset->spinlock);
			
        if (quotaContextList == NULL)   {
            //There is at least one fileset with quota on in the path, allocate
            //and initialize the quota context list
        	quotaContextList = (fs_QuotaContextList_t *)
                                fs_ramAlloc(FS_RAM_QUOTA_CONTEXT_LIST,
                                			sizeof (fs_QuotaContextList_t));
            quotaContextList->newInode = NULL;

            /*
             * We are overloading newInode flag to also contain inode for which
             * blocks are allocated. We can do this since a quotaContextList can
             * only support either block allocation or inode allocation
             */

            if (flag == FS_COUNT_BLOCKS) {
                quotaContextList->newInode = inode;
            }

        	DLL_INIT(&quotaContextList->fsetList);
        	DLL_INIT(&quotaContextList->uQuotaList);
        	DLL_INIT(&quotaContextList->gQuotaList);
        	quotaContextList->uid = uid;
        	quotaContextList->gid = gid;
        	quotaContextList->flag = flag;
        	/*
        	 * Point to this list from the context structure, so that
        	 * we can get back our hands on this list during write completion
        	 * to perform appropriate logging and transactions
        	 */
        	*list = quotaContextList;
        }
        
		fsetContext = (fs_FsetContext_t *)fs_ramAlloc(
				FS_RAM_FSET_CONTEXT, sizeof (fs_FsetContext_t));
		bzero(fsetContext, sizeof (fs_FsetContext_t));
		fsetContext->fset = tfset;

		/*
		 * Note that we don't hold the fset structure. The
		 * reason is that hold on the inode would not allow
		 * the fset structure to be removed.
		 */

		fs_holdFileSet(context, core, fsetContext->fset);
		DLL_INSERT_HEAD(&quotaContextList->fsetList, fsetContext,
				   list);
		tfset = tfset->parent;
	}

	fs_inodeRelease(context, fsetInfoInode);

    if (quotaContextList == NULL)   {
        //No fileset with quota on
        return granted;
    }
    
#if 0
TOBEUPDATED
	/*
	 * Now walk each of the filesets and get a user and group quota
	 * structure in to the context. Each of the user and group
	 * quota structures are held, so the completion routines would
	 * have to appropriately release them
	 */

	DLL_ITERATE(fsetContext, &quotaContextList->fsetList, list) {
		
		/*
		 * Get the user quota context
		 */

		tfset = fsetContext->fset;

		quota = fs_getUGQuota(context, core, tfset, FS_QUOTA_USER, uid, FALSE);
		fs_lockShared(context, &quota->lock, NULL);
        
		NFX_SPIN_LOCK(&quota->spinlock);
		if (quota->flag & FS_QUOTA_INCORE_NEEDS_INIT
         && !fs_haveMinfreeAvail(core)) {
            //We need to allocate a quota block and there is no space left 
			NFX_SPIN_UNLOCK(&quota->spinlock);
			fs_unlockShared(context, &quota->lock);
           	fs_releQuota(quota);
			granted = 0;
            break;
		}
		NFX_SPIN_UNLOCK(&quota->spinlock);
        
		quotaContext = (fs_QuotaContext_t *)fs_ramAlloc(
			FS_RAM_QUOTA_CONTEXT, sizeof (fs_QuotaContext_t));
		bzero(quotaContext, sizeof (fs_QuotaContext_t));
		quotaContext->quota = quota;
		quotaContext->fset = tfset;
		DLL_INSERT_TAIL(&quotaContextList->uQuotaList, quotaContext,
				   list);

		/*
		 * Get the group quota context
		 */

		quota = fs_getUGQuota(context, core, tfset, FS_QUOTA_GROUP, gid, FALSE);
        FS_ASSERT(quota != NULL);
		fs_lockShared(context, &quota->lock, NULL);
		
		NFX_SPIN_LOCK(&quota->spinlock);
		if (quota->flag & FS_QUOTA_INCORE_NEEDS_INIT
         && !fs_haveMinfreeAvail(core)) {
            //We need to allocate a quota block and there is no space left 
			NFX_SPIN_UNLOCK(&quota->spinlock);
			fs_unlockShared(context, &quota->lock);
           	fs_releQuota(quota);
			granted = 0;
            break;
		}
		NFX_SPIN_UNLOCK(&quota->spinlock);
        
		quotaContext = (fs_QuotaContext_t *)fs_ramAlloc(
			FS_RAM_QUOTA_CONTEXT, sizeof (fs_QuotaContext_t));
		bzero(quotaContext, sizeof (fs_QuotaContext_t));
		quotaContext->quota = quota;
		quotaContext->fset = tfset;

		DLL_INSERT_TAIL(&quotaContextList->gQuotaList, quotaContext, list);
	}

	/*
	 * If no space, then let us unlock the structures that we have locked
     * and return an indication that no blocks are allocable, so transaction
     * cannot not proceed.
	 */

	if (granted == 0) {
		DLL_ITERATE(quotaContext, &quotaContextList->uQuotaList, list) {
			fs_unlockShared(context, &quotaContext->quota->lock);
		}
		DLL_ITERATE(quotaContext, &quotaContextList->gQuotaList, list) {
			fs_unlockShared(context, &quotaContext->quota->lock);
		}
		DLL_ITERATE(fsetContext, &quotaContextList->fsetList, list) {
			fs_unlockShared(context, &fsetContext->fset->lock);
		}
    	return granted;
	}

	/*
	 * Now that we have all fsets and quotas held. Walk through all
	 * of them and figure out how many blocks/inode we can afford to provide
	 */

	/*
	 * Start with user quota
	 */

	DLL_ITERATE(quotaContext, &quotaContextList->uQuotaList, list) {
        FS_ASSERT(quotaContext != NULL);
        FS_ASSERT(quotaContext != DLL_NIL);
		quota = quotaContext->quota;
		NFX_SPIN_LOCK(&quota->spinlock);
		// Based on the context account for either blocks or inodes
		if (flag == FS_COUNT_BLOCKS) {
			notify += accountUserGroupBlocks(context, quotaContext, &granted, 
                                             &now, alwaysSucceedUnlessNoSpace);
		} else {
			notify += accountUserGroupInodes(context, quotaContext, &granted,
                                             &now, alwaysSucceedUnlessNoSpace);
		}
		NFX_SPIN_UNLOCK(&quota->spinlock);
		quotaFlagsWrite += quotaContext->flagsWr;
	}

	/*
	 * If blocks that can be provided are zero, then let us unlock
	 * all the structures that we have locked and return an indication
	 * that no blocks are allocable, so transaction cannot not proceed.
	 */

	if (granted == 0) {
		DLL_ITERATE(quotaContext, &quotaContextList->uQuotaList, list) {
       		quota = quotaContext->quota;
            if (quotaContext->count)    {
           		NFX_SPIN_LOCK(&quota->spinlock);
           		if (flag == FS_COUNT_BLOCKS) {
           			quota->cachedRecords.consumption.blocks -= 
                                                        quotaContext->count;
           		} else {
           			quota->cachedRecords.consumption.inodes -= 
                                                        quotaContext->count;
           		}
           		NFX_SPIN_UNLOCK(&quota->spinlock);
            }
			fs_unlockShared(context, &quota->lock);
		}
		DLL_ITERATE(quotaContext, &quotaContextList->gQuotaList, list) {
			fs_unlockShared(context, &quotaContext->quota->lock);
		}
		DLL_ITERATE(fsetContext, &quotaContextList->fsetList, list) {
			fs_unlockShared(context, &fsetContext->fset->lock);
		}
		if (quotaFlagsWrite)	{
			fs_commitQuotaContextNotifChanges(context, quotaContextList, TRUE, 
												FALSE);
        }
		if (notify) {
			notifyAdmin(context, quotaContextList);
		}
		return granted;
	}

	DLL_ITERATE(quotaContext, &quotaContextList->gQuotaList, list) {
        FS_ASSERT(quotaContext != NULL);
        FS_ASSERT(quotaContext != DLL_NIL);
		quota = quotaContext->quota;
        FS_ASSERT(quota != NULL);
		NFX_SPIN_LOCK(&quota->spinlock);
		// Based on the context account for either blocks or inodes
		if (flag == FS_COUNT_BLOCKS) {
			notify += accountUserGroupBlocks(context, quotaContext, &granted,
                                             &now, alwaysSucceedUnlessNoSpace);
		} else {
			notify += accountUserGroupInodes(context, quotaContext, &granted,
                                             &now, alwaysSucceedUnlessNoSpace);
		}
		NFX_SPIN_UNLOCK(&quota->spinlock);
		quotaFlagsWrite += quotaContext->flagsWr;
	}

	/*
	 * If blocks that can be provided are zero, then let us unlock
	 * all the structures that we have locked and return an indication
	 * that no blocks are allocable, so transaction cannot not proceed.
	 */

	if (granted == 0) {
		DLL_ITERATE(quotaContext, &quotaContextList->uQuotaList, list) {
      		quota = quotaContext->quota;
       		NFX_SPIN_LOCK(&quota->spinlock);
       		if (flag == FS_COUNT_BLOCKS) {
       			quota->cachedRecords.consumption.blocks -= quotaContext->count;
       		} else {
       			quota->cachedRecords.consumption.inodes -= quotaContext->count;
       		}
       		NFX_SPIN_UNLOCK(&quota->spinlock);
    		fs_unlockShared(context, &quota->lock);
		}
		DLL_ITERATE(quotaContext, &quotaContextList->gQuotaList, list) {
       		quota = quotaContext->quota;
            if (quotaContext->count)    {
           		NFX_SPIN_LOCK(&quota->spinlock);
           		if (flag == FS_COUNT_BLOCKS) {
           			quota->cachedRecords.consumption.blocks -= 
                                                        quotaContext->count;
           		} else {
           			quota->cachedRecords.consumption.inodes -= 
                                                        quotaContext->count;
           		}
           		NFX_SPIN_UNLOCK(&quota->spinlock);
            }
    		fs_unlockShared(context, &quota->lock);
		}
		DLL_ITERATE(fsetContext, &quotaContextList->fsetList, list) {
			fs_unlockShared(context, &fsetContext->fset->lock);
		}
		if (quotaFlagsWrite)	{
			fs_commitQuotaContextNotifChanges(context, quotaContextList, FALSE, 
												FALSE);
        }
		if (notify) {
			notifyAdmin(context, quotaContextList);
		}
		return granted;
	}

#endif
	/*
	 * Get the reservation against fileset quotas
	 */

	DLL_ITERATE(fsetContext, &quotaContextList->fsetList, list) {
        FS_ASSERT(fsetContext != NULL);
        FS_ASSERT(fsetContext != DLL_NIL);
		tfset = fsetContext->fset;
		NFX_SPIN_LOCK(&tfset->spinlock);
		// Based on the context account for either blocks or inodes
		if (flag == FS_COUNT_BLOCKS) {
			notify += accountFileSetBlocks(context, fsetContext, &granted,
                                           &now, alwaysSucceedUnlessNoSpace);
		} else {
			notify += accountFileSetInodes(context, fsetContext, &granted,
                                           &now, alwaysSucceedUnlessNoSpace);
		}
		NFX_SPIN_UNLOCK(&tfset->spinlock);
	}

	if (granted == 0) {
		DLL_ITERATE(quotaContext, &quotaContextList->uQuotaList, list) {
      		quota = quotaContext->quota;
       		NFX_SPIN_LOCK(&quota->spinlock);
       		if (flag == FS_COUNT_BLOCKS) {
       			quota->cachedRecords.consumption.blocks -= quotaContext->count;
       		} else {
       			quota->cachedRecords.consumption.inodes -= quotaContext->count;
       		}
       		NFX_SPIN_UNLOCK(&quota->spinlock);
    		fs_unlockShared(context, &quota->lock);
		}
		DLL_ITERATE(quotaContext, &quotaContextList->gQuotaList, list) {
      		quota = quotaContext->quota;
       		NFX_SPIN_LOCK(&quota->spinlock);
       		if (flag == FS_COUNT_BLOCKS) {
       			quota->cachedRecords.consumption.blocks -= quotaContext->count;
       		} else {
       			quota->cachedRecords.consumption.inodes -= quotaContext->count;
       		}
       		NFX_SPIN_UNLOCK(&quota->spinlock);
    		fs_unlockShared(context, &quota->lock);
		}
		DLL_ITERATE(fsetContext, &quotaContextList->fsetList, list) {
    		tfset = fsetContext->fset;
            if (fsetContext->count) {
        		NFX_SPIN_LOCK(&tfset->spinlock);
        		if (flag == FS_COUNT_BLOCKS) {
        			tfset->cachedRecord.fileSetConsumption.blocks -= 
                                                            fsetContext->count;
        		} else {
        			tfset->cachedRecord.fileSetConsumption.inodes -= 
                                                            fsetContext->count;
        		}
        		NFX_SPIN_UNLOCK(&tfset->spinlock);
            }
			fs_unlockShared(context, &tfset->lock);
		}
		if (quotaFlagsWrite)	{
			fs_commitQuotaContextNotifChanges(context, quotaContextList, FALSE, 
												FALSE);
        }
		if (notify) {
			notifyAdmin(context, quotaContextList);
		}
		return granted;
	}

  	quotaContextList->availCount = granted;
    if (granted != requested)    {
        //We didn't get as many blocks/inodes as requested, trim.
    	DLL_ITERATE(quotaContext, &quotaContextList->uQuotaList, list) {
    		quota = quotaContext->quota;
    		NFX_SPIN_LOCK(&quota->spinlock);
    		if (flag == FS_COUNT_BLOCKS) {
    			quota->cachedRecords.consumption.blocks -= quotaContext->count
                                                            - granted;
    		} else {
    			quota->cachedRecords.consumption.inodes -= quotaContext->count 
                                                            - granted;
    		}
    		NFX_SPIN_UNLOCK(&quota->spinlock);
    		fs_unlockShared(context, &quota->lock);
    	}
    	DLL_ITERATE(quotaContext, &quotaContextList->gQuotaList, list) {
    		quota = quotaContext->quota;
    		NFX_SPIN_LOCK(&quota->spinlock);
    		if (flag == FS_COUNT_BLOCKS) {
    			quota->cachedRecords.consumption.blocks -= quotaContext->count
                                                            - granted;
    		} else {
    			quota->cachedRecords.consumption.inodes -= quotaContext->count 
                                                            - granted;
    		}
    		NFX_SPIN_UNLOCK(&quota->spinlock);
    		fs_unlockShared(context, &quota->lock);
    	}
    	DLL_ITERATE(fsetContext, &quotaContextList->fsetList, list) {
    		tfset = fsetContext->fset;
    		NFX_SPIN_LOCK(&tfset->spinlock);
    		if (flag == FS_COUNT_BLOCKS) {
    			tfset->cachedRecord.fileSetConsumption.blocks -= 
                                                   fsetContext->count - granted;
    		} else {
    			tfset->cachedRecord.fileSetConsumption.inodes -= 
                                                   fsetContext->count - granted;
    		}
    		NFX_SPIN_UNLOCK(&tfset->spinlock);
		    fs_unlockShared(context, &tfset->lock);
        }
	} else {
    	DLL_ITERATE(quotaContext, &quotaContextList->uQuotaList, list) {
    		fs_unlockShared(context, &quotaContext->quota->lock);
    	}
    	DLL_ITERATE(quotaContext, &quotaContextList->gQuotaList, list) {
    		fs_unlockShared(context, &quotaContext->quota->lock);
    	}
    	DLL_ITERATE(fsetContext, &quotaContextList->fsetList, list) {
		    fs_unlockShared(context, &fsetContext->fset->lock);
        }
    }

	if (quotaFlagsWrite)	{
		fs_commitQuotaContextNotifChanges(context, quotaContextList, FALSE, 
											FALSE);
	}
	if (notify) {
		notifyAdmin(context, quotaContextList);
	}

	return granted;
}

/*
 * Set user or a group quota for a given user and a given file set.
 * Set quota operation can fail for lock of space if blocks need to
 * be allocated. Quota accounting in fileset for blocks to quota file
 * will never be accounted for.
 *
 * It is assumed that the caller has sanity checked the quotablock.
 */

EXPORTED void
fs_setUGQuota(
	fs_Context					*context,
	fs_FileSystem				*core,
	fs_FileSetIncoreRecord_t	*fset,	// fileset we are working on
	fs_QuotaBlk_t				*qblk	// new quota information
)
{
#if 0
TOBEUPDATED
	fs_QuotaIncoreRecord_t	*quota = NULL;
	fs_QuotaRecord_t		*quotaRecord = NULL;
	fs_QuotaRecord_t		*defQuotaRecord;
	uint64					inum;
	fs_Inode				*quotaInode = NULL;
	uint32					id;
	int						type;
    uint32                  now = fs_getTimeOfDaySecs();

	/*
	 * Assert that arguments are correct
	 */

	FS_ASSERT(context != NULL && core != NULL && fset != NULL &&
		  	  qblk != NULL);
	id = (uint32)qblk->id;
	type = qblk->flag;

	/*
	 * Assert that this is either user quota or group quota and
	 * not filset quota and also it is not a default quota
	 */

	FS_ASSERT((qblk->flag & (FS_QUOTA_AGGREGATE | FS_QUOTA_DEFAULT)) == 0);
	FS_ASSERT(((qblk->flag & (FS_QUOTA_USER | FS_QUOTA_GROUP)) ==
		    FS_QUOTA_USER) ||
		  ((qblk->flag & (FS_QUOTA_USER | FS_QUOTA_GROUP)) ==
		    FS_QUOTA_GROUP));

	/*
	 * XXX - Assert that we are not holding any locks at this point,
	 * except fileset lock in shared mode
	 */
	
	FS_ASSERT(fs_lockHeldShare(context, &fset->lock));

	/*
	 * Fetch the incore record quota record for the id and fileset
	 */

	quota = fs_getUGQuota(context, core, fset, type, (uint32)qblk->id, FALSE);
	quotaRecord = &quota->cachedRecords.quota;
    quota->cachedRecords.lastChanged = now;

	if (type & FS_QUOTA_USER) {
		inum = fset->cachedRecord.userQuotaInode;
        defQuotaRecord = &fset->cachedRecord.defaultUserQuota;
	} else {
		inum = fset->cachedRecord.groupQuotaInode;
        defQuotaRecord = &fset->cachedRecord.defaultGroupQuota;
	}

    //If we are requested to use the default quota, get it from the fileset 
    if (type & FS_QUOTA_USE_DEFAULTS) {
      	NFX_SPIN_LOCK(&fset->spinlock);
        qblk->inodeSoftLimit = defQuotaRecord->inodeSoftLimit;
        qblk->inodeHardLimit = defQuotaRecord->inodeHardLimit;
        qblk->inodeTimeLimit = defQuotaRecord->inodeTimeLimit;
        qblk->blockSoftLimit = defQuotaRecord->blockSoftLimit;
        qblk->blockHardLimit = defQuotaRecord->blockHardLimit;
        qblk->blockTimeLimit = defQuotaRecord->blockTimeLimit;
      	NFX_SPIN_UNLOCK(&fset->spinlock);
    }

	/*
	 * Lock the quota inode in exclusive mode. Since we are
	 * going to be writing to this file
	 */

	quotaInode = I_GET_EXCL(0, inum);

	/*
	 * Get an exclusive lock on the quota structure
	 */

	fs_lockExclusive(context, &quota->lock, NULL);

	/*
	 * Allocate and write new quota information
	 */

	fs_initializeUGQuotaRecord(context, core, fset, quotaInode, qblk, id, type);

	/*
	 * Mark the incore structure as not needing an
	 * initialization
	 */

	NFX_SPIN_LOCK(&quota->spinlock);
	quota->flag &= ~FS_QUOTA_INCORE_NEEDS_INIT;
	NFX_SPIN_UNLOCK(&quota->spinlock);

	/*
	 * Update incore values now as we have successfully written the
	 * transaction
	 */

	NFX_SPIN_LOCK(&quota->spinlock);
	quotaRecord->inodeSoftLimit = qblk->inodeSoftLimit;
	quotaRecord->inodeHardLimit = qblk->inodeHardLimit;
	quotaRecord->inodeTimeLimit = qblk->inodeTimeLimit;
	quotaRecord->blockSoftLimit = qblk->blockSoftLimit;
	quotaRecord->blockHardLimit = qblk->blockHardLimit;
	quotaRecord->blockTimeLimit = qblk->blockTimeLimit;
	NFX_SPIN_UNLOCK(&quota->spinlock);

	fs_unlockExclusive(context, &quota->lock);

	/*
	 * XXX - Inode should be released by the write transaction. How
	 * do we assert that? We need to indicate that the lock is
	 * transfered to another context
	 */
    if (!fs_inodeWasModified(context, quotaInode)) {
			fs_inodeReleaseExclusive(context, quotaInode);
	}

	/*
	 * Now release our hold on the quota structure
	 */

	fs_releQuota(quota);
	return;
#endif
}


/* 
 * Update incore quotas with new default values
 */

PRIVATE void
fs_updateIncoreUGQuotaWithDefaults(
    fs_Context                  *context,
    fs_FileSetIncoreRecord_t    *fset,
    int                         type,
    fs_QuotaRecord_t            *rec)	// new quota information
{
    fs_FileSystem           *fs = context->fileSystem;
	fs_QuotaIncoreRecord_t	*quota;
	nfx_spinlock_t		    *listlock;
    int                     hash;
    int                     count = 0;
    

    FS_TRACE(context, FS_TRACE_REQ, ("in updateIncoreUGQuotaWithDefaults"));
	listlock = &fs_quotaCache.spin;

	if (type & FS_QUOTA_USER) {
        NFX_SPIN_LOCK(listlock);
        for (hash = 0; hash <  FS_QCACHE_HASH_MAX; hash++) {
            DLL_ITERATE(quota, &fs_quotaCache.userQuotaHashList[hash], 
                        hashList) {
                count++;
                if (quota == NULL || quota == DLL_NIL) {
                    break;
                }
                if (quota->fileSetId != fset->fileSetId
                 || quota->volId.whole_thing != fs->volId.whole_thing    
                 || !(quota->cachedRecords.flags & FS_UG_DEFAULT_QUOTALIMITS))
                {
                    continue;
                }
                fs_holdQuota(quota);
            	NFX_SPIN_LOCK(&quota->spinlock);
                bcopy(rec, &quota->cachedRecords.quota,
                       sizeof(fs_QuotaRecord_t));
            	NFX_SPIN_UNLOCK(&quota->spinlock);
                fs_releQuota(quota);
            }
#ifndef FS_STANDALONE
#ifdef USE_WATCHDOG
            if (count < fs_quotaCache.quotaCacheLimit)   {
                watchdog_kick();
            }
#endif // USE_WATCHDOG
#endif // FS_STANDALONE
        }
        NFX_SPIN_UNLOCK(listlock);
	} else {
        NFX_SPIN_LOCK(listlock);
        for (hash = 0; hash <  FS_QCACHE_HASH_MAX; hash++) {
			DLL_ITERATE(quota, &fs_quotaCache.groupQuotaHashList[hash],
                        hashList) {
                count++;
                if (quota == NULL || quota == DLL_NIL) {
    			    break;
    			}
                if (quota->fileSetId != fset->fileSetId
                 || quota->volId.whole_thing != fs->volId.whole_thing    
                 || !(quota->cachedRecords.flags & FS_UG_DEFAULT_QUOTALIMITS))
                {
                    continue;
                }
                fs_holdQuota(quota);
            	NFX_SPIN_LOCK(&quota->spinlock);
                bcopy(rec, &quota->cachedRecords.quota,
                       sizeof(fs_QuotaRecord_t));
            	NFX_SPIN_UNLOCK(&quota->spinlock);
                fs_releQuota(quota);
    	    }
#ifndef FS_STANDALONE
#ifdef USE_WATCHDOG
            if (count < fs_quotaCache.quotaCacheLimit)   {
                watchdog_kick();
            }
#endif // USE_WATCHDOG
#endif // FS_STANDALONE
        }
        NFX_SPIN_UNLOCK(listlock);
	}
    FS_TRACE(context, FS_TRACE_REQ, ("out updateIncoreUGQuotaWithDefaults"));
}


PRIVATE void
fs_updateIncoreFilesetQuotaWithDefaults(
    fs_Context                  *context,
    fs_FileSetIncoreRecord_t    *fset,
    fs_QuotaRecord_t            *rec)   // new quota information
{
    fs_FileSystem               *fs = context->fileSystem;
    fs_FileSetIncoreRecord_t    *tfset;
    

    NFX_SPIN_LOCK(&fs->filesetListLock);
    DLL_ITERATE(tfset, &fs->filesetList, fsetList) {
        if (tfset == NULL || tfset == DLL_NIL) {
             break;
        }
        if (tfset->cachedRecord.parentId != fset->fileSetId
         || !(tfset->cachedRecord.fileSetFlags & FS_FILESET_USE_QDEFAULTS))
        {
             continue;
        }
       	NFX_SPIN_LOCK(&tfset->spinlock);
       	NFX_SPIN_LOCK(&fset->spinlock);
        bcopy(rec, &tfset->cachedRecord.fileSetQuota, sizeof(fs_QuotaRecord_t));
      	NFX_SPIN_UNLOCK(&fset->spinlock);
      	NFX_SPIN_UNLOCK(&tfset->spinlock);
    }
    NFX_SPIN_UNLOCK(&fs->filesetListLock);
}


/*
 * Set quotas for a given file set. This includes fileset quota,
 * default user and group quota and default file set quotas
 * Quota accounting in fileset for blocks to quota file
 * will never be accounted for.
 *
 * It is assumed that the caller has sanity checked the quotablock.
 */

EXPORTED fs_Status
fs_setFileSetQuota(
	fs_Context					*context,
	fs_FileSystem				*core,
	fs_FileSetIncoreRecord_t	*fset,	// fileset we are working on
	fs_QuotaBlk_t				*qblk	// new quota information
)
{
	fs_Buf					*buf;
	int						boff;
	fs_FileSetDiskRecord_t	*dfset;
	fs_QuotaRecord_t		*qrec, *cqrec, *pcqrec;
	fs_Inode				*fsetInfoInode;
    uint32                  now = fs_getTimeOfDaySecs();

	/*
	 * Assert that arguments are correct
	 */

	FS_ASSERT(context != NULL && core != NULL && fset != NULL &&
		  	  qblk != NULL);

	FS_ASSERT(qblk->flag & FS_QUOTA_AGGREGATE ||
		  	  (qblk->flag & FS_QUOTA_DEFAULT) != 0);

	FS_ASSERT(((qblk->flag & (FS_QUOTA_AGGREGATE | FS_QUOTA_USER |
							 FS_QUOTA_GROUP)) == FS_QUOTA_AGGREGATE) ||
		  		((qblk->flag & (FS_QUOTA_AGGREGATE | FS_QUOTA_USER |
							   FS_QUOTA_GROUP)) == FS_QUOTA_USER) ||
		  		((qblk->flag & (FS_QUOTA_AGGREGATE | FS_QUOTA_USER |
								FS_QUOTA_GROUP)) == FS_QUOTA_GROUP));
	
    if ((qblk->flag & FS_QUOTA_USE_DEFAULTS) && fset->parent == NULL) {
        //Request to use default on root fileset
        return FS_ERR_INVAL;
    }

	boff = (uint32)((fset->fileSetId * sizeof(fs_FileSetDiskRecord_t)) &
					 (FS_BLOCK_SIZE - 1));

	/*
	 * Get the fileset info inode and the associated buffer
	 */

	fsetInfoInode = I_GET_EXCL(0, FS_FILESET_INFO_INODE);
	fs_lockExclusive(context, &fset->lock, NULL);

	FS_ASSERT(fsetInfoInode != NULL);

	/*
	 * Assert that fset Info inode is held
	 */

	FS_ASSERT(fs_lockOwnedExclusive(context, &fsetInfoInode->lock));

	/*
	 * Get the associated buffer for write
	 */

	buf = O_GET_EXCL(fsetInfoInode, (fset->fileSetId *
			  sizeof (fs_FileSetDiskRecord_t)) >> FS_BLOCK_SHIFT);

	FS_ASSERT(buf != NULL);

	dfset = (fs_FileSetDiskRecord_t *)(buf->u.data + boff);

	if (qblk->flag & FS_QUOTA_DEFAULT) {
#if 0
		if (qblk->flag & FS_QUOTA_USER) {
			qrec = &dfset->defaultUserQuota;
			cqrec = &fset->cachedRecord.defaultUserQuota;
		} else if (qblk->flag & FS_QUOTA_GROUP) {
			qrec = &dfset->defaultGroupQuota;
			cqrec = &fset->cachedRecord.defaultGroupQuota;
		} else {
			qrec = &dfset->defaultAggregateQuota;
			cqrec = &fset->cachedRecord.defaultAggregateQuota;
		}
#endif
        qrec = &dfset->defaultAggregateQuota;
        cqrec = &fset->cachedRecord.defaultAggregateQuota;
	} else {
		qrec = &dfset->fileSetQuota;
		cqrec = &fset->cachedRecord.fileSetQuota;
        //If we are requested to use the default quota, get it from the parent
        if (qblk->flag & FS_QUOTA_USE_DEFAULTS)    {
            pcqrec = &fset->parent->cachedRecord.defaultAggregateQuota;
        	NFX_SPIN_LOCK(&fset->parent->spinlock);
            qblk->inodeSoftLimit = pcqrec->inodeSoftLimit;
            qblk->inodeHardLimit = pcqrec->inodeHardLimit;
            qblk->inodeTimeLimit = pcqrec->inodeTimeLimit;
            qblk->blockSoftLimit = pcqrec->blockSoftLimit;
            qblk->blockHardLimit = pcqrec->blockHardLimit;
            qblk->blockTimeLimit = pcqrec->blockTimeLimit;
        	NFX_SPIN_UNLOCK(&fset->parent->spinlock);
        }
	}

	/*
	 * Copy the quota into the disk buffer and release it for
	 * transactional update.
	 */

	qrec->inodeSoftLimit = qblk->inodeSoftLimit;
	qrec->inodeHardLimit = qblk->inodeHardLimit;
	qrec->inodeTimeLimit = qblk->inodeTimeLimit;
	qrec->blockSoftLimit = qblk->blockSoftLimit;
	qrec->blockHardLimit = qblk->blockHardLimit;
	qrec->blockTimeLimit = qblk->blockTimeLimit;
	if (!(qblk->flag & FS_QUOTA_DEFAULT)) {
        //store whether fileset uses parent's default quota or not
        if (qblk->flag & FS_QUOTA_USE_DEFAULTS) {
            dfset->fileSetFlags |= FS_FILESET_USE_QDEFAULTS;
        } else {
            dfset->fileSetFlags &= ~FS_FILESET_USE_QDEFAULTS;
        }
    }
    dfset->lastChanged = now;

	/*
	 * Release the buffer. Do transaction logging for this buffer
	 */

	FS_BUF_WRITE(buf, ((char*)dfset - buf->u.data), 
                        sizeof(fs_FileSetDiskRecord_t));

	/*
	 * Update incore values now as we have successfully written the
	 * transaction
	 */

	NFX_SPIN_LOCK(&fset->spinlock);
	cqrec->inodeSoftLimit = qblk->inodeSoftLimit;
	cqrec->inodeHardLimit = qblk->inodeHardLimit;
	cqrec->inodeTimeLimit = qblk->inodeTimeLimit;
	cqrec->blockSoftLimit = qblk->blockSoftLimit;
	cqrec->blockHardLimit = qblk->blockHardLimit;
	cqrec->blockTimeLimit = qblk->blockTimeLimit;
    fset->cachedRecord.lastChanged = now;
	if (!(qblk->flag & FS_QUOTA_DEFAULT)) {
        //store whether fileset uses parent's default quotas or not
        if (qblk->flag & FS_QUOTA_USE_DEFAULTS) {
            fset->cachedRecord.fileSetFlags |= FS_FILESET_USE_QDEFAULTS;
        } else {
            fset->cachedRecord.fileSetFlags &= ~FS_FILESET_USE_QDEFAULTS;
        }
    }
	NFX_SPIN_UNLOCK(&fset->spinlock);

    //Update all incore quotas using the default limits just changed
	if (qblk->flag & FS_QUOTA_DEFAULT) {
		if (qblk->flag & FS_QUOTA_USER) {
            fs_updateIncoreUGQuotaWithDefaults(context, fset, FS_QUOTA_USER,
                                                cqrec);      
		} else if (qblk->flag & FS_QUOTA_GROUP) {
            fs_updateIncoreUGQuotaWithDefaults(context, fset, FS_QUOTA_GROUP,
                                                cqrec);
		} else {
            fs_updateIncoreFilesetQuotaWithDefaults(context, fset, cqrec);
		}
    }
       
	fs_unlockExclusive(context, &fset->lock);

    if (!fs_inodeWasModified(context, fsetInfoInode)) {
		fs_inodeReleaseExclusive(context, fsetInfoInode);
	}

	return FS_ERR_NONE;
}

/*
 * Hold all the relevant quota and fset structures for frees and
 * return them back via context structure
 */

EXPORTED void
fs_freeQuotaResources(
	fs_Context	*context,	
	fs_FileSystem	*core,		
	fs_Inode	*inode,
	uint64		count,	// count of resources required
	int			flag,	// count inodes or blocks
	uint32		uid,
	uint32		gid,
	fs_QuotaContextList_t	**list
)
{

	fs_QuotaContextList_t		*quotaContextList = NULL;
	fs_FileSetIncoreRecord_t	*tfset;
	fs_FsetContext_t			*fsetContext;
	fs_QuotaContext_t			*quotaContext;
	fs_QuotaIncoreRecord_t		*quota;
	fs_Inode					*fsetInfoInode;


	FS_ASSERT(count != 0 && context != NULL && core != NULL &&
		  inode != NULL);
	FS_ASSERT(flag == FS_COUNT_INODES || flag == FS_COUNT_BLOCKS);

	/*
	 * Build up the list of fileset structures that we want to
	 * operate on
	 */

	fsetInfoInode = I_GET_SHARED(0, FS_FILESET_INFO_INODE);

	/*
	 * Now that we have blocked out creation of any new filesets
	 * the parent chains are stable
	 */

	tfset = inode->fset = fs_getFset(context, core, inode);

	while (tfset != NULL) {

		/*
		 * If the fset does not have quotas turned on or
		 * the allocation version is older than quota turn
		 * on allocation version, skip this fset
		 */

		NFX_SPIN_LOCK(&tfset->spinlock);
		if ((tfset->incoreFlag & (FS_FILESET_QUOTA_ON |
					  FS_FILESET_QUOTA_TURN_ON)) == 0) {
			NFX_SPIN_UNLOCK(&tfset->spinlock);
			tfset = tfset->parent;
			continue;
		}


		/*
		 * Quota on has not yet seen this inode. Don't bother accounting
		 * for its consumption as quota on will process it when it gets
		 * there
	 	 */

		if ((tfset->incoreFlag & FS_FILESET_QUOTA_TURN_ON) &&
			inode->inum > tfset->inodeSeen) {
			NFX_SPIN_UNLOCK(&tfset->spinlock);
			tfset = tfset->parent;
			continue;
		}
		NFX_SPIN_UNLOCK(&tfset->spinlock);
			
        if (quotaContextList == NULL)   {
            //There is at least one fileset with quota on in the path, allocate
            //and initialize the quota context list
        	quotaContextList = (fs_QuotaContextList_t *)
                                fs_ramAlloc(FS_RAM_QUOTA_CONTEXT_LIST,
                                			sizeof (fs_QuotaContextList_t));
            quotaContextList->newInode = NULL;

            /*
             * We are overloading newInode flag to also contain inode for which
             * blocks are allocated. We can do this since a quotaContextList can
             * only support either block allocation or inode allocation
             */

            if (flag == FS_COUNT_BLOCKS) {
                quotaContextList->newInode = inode;
            }

        	DLL_INIT(&quotaContextList->fsetList);
        	DLL_INIT(&quotaContextList->uQuotaList);
        	DLL_INIT(&quotaContextList->gQuotaList);
        	quotaContextList->uid = uid;
        	quotaContextList->gid = gid;
        	quotaContextList->flag = flag | FS_FREE_RESOURCES;
        }
        
		fsetContext = (fs_FsetContext_t *)fs_ramAlloc(
				FS_RAM_FSET_CONTEXT, sizeof (fs_FsetContext_t));
		bzero(fsetContext, sizeof (fs_FsetContext_t));
		fsetContext->fset = tfset;

		/*
		 * Note that we don't hold the fset structure. The
		 * reason is that hold on the inode would not allow
		 * the fset structure to be removed.
		 */

		DLL_INSERT_TAIL(&quotaContextList->fsetList, fsetContext,
				   list);
		tfset = tfset->parent;
	}

	fs_inodeRelease(context, fsetInfoInode);

    if (quotaContextList == NULL)   {
        //No fileset with quota on
        return;
    }
    
	/*
	 * Now walk each of the filesets and get a user and group quota
	 * structure in to the context. Each of the user and group
	 * quota structures are held, so the completion routines would
	 * have to appropriately release them
	 */

	DLL_ITERATE(fsetContext, &quotaContextList->fsetList, list) {
		
		/*
		 * Get the user quota context
		 */

		tfset = fsetContext->fset;

		quota = fs_getUGQuota(context, core, tfset, FS_QUOTA_USER, uid, FALSE);
				
		quotaContext = (fs_QuotaContext_t *)fs_ramAlloc(
			FS_RAM_QUOTA_CONTEXT, sizeof (fs_QuotaContext_t));
		bzero(quotaContext, sizeof (fs_QuotaContext_t));
		quotaContext->quota = quota;
		quotaContext->fset = tfset;
		DLL_INSERT_TAIL(&quotaContextList->uQuotaList, quotaContext,
				   list);

		/*
		 * Get the group quota context
		 */

		quota = fs_getUGQuota(context, core, tfset, FS_QUOTA_GROUP, gid, FALSE);
				
		quotaContext = (fs_QuotaContext_t *)fs_ramAlloc(
			FS_RAM_QUOTA_CONTEXT, sizeof (fs_QuotaContext_t));
		bzero(quotaContext, sizeof (fs_QuotaContext_t));
		quotaContext->quota = quota;
		quotaContext->fset = tfset;
		DLL_INSERT_TAIL(&quotaContextList->gQuotaList, quotaContext,
				   list);
	}


	quotaContextList->availCount = count;
	DLL_ITERATE(quotaContext, &quotaContextList->uQuotaList, list) {
		quota = quotaContext->quota;
		fs_lockExclusive(context, &quota->lock, NULL);
		NFX_SPIN_LOCK(&quota->spinlock);
		if (flag == FS_COUNT_BLOCKS) {
			quota->cachedRecords.consumption.blocks -= count;
		} else {
			quota->cachedRecords.consumption.inodes -= count;
		}
		NFX_SPIN_UNLOCK(&quota->spinlock);
		fs_unlockExclusive(context, &quota->lock);
	}
	DLL_ITERATE(quotaContext, &quotaContextList->gQuotaList, list) {
		quota = quotaContext->quota;
		fs_lockExclusive(context, &quota->lock, NULL);
		NFX_SPIN_LOCK(&quota->spinlock);
		if (flag == FS_COUNT_BLOCKS) {
			quota->cachedRecords.consumption.blocks -= count;
		} else {
			quota->cachedRecords.consumption.inodes -= count;
		}
		NFX_SPIN_UNLOCK(&quota->spinlock);
		fs_unlockExclusive(context, &quota->lock);
	}
	DLL_ITERATE(fsetContext, &quotaContextList->fsetList, list) {
		tfset = fsetContext->fset;
		fs_lockExclusive(context, &tfset->lock, NULL);
		NFX_SPIN_LOCK(&tfset->spinlock);
		if (flag == FS_COUNT_BLOCKS) {
			tfset->cachedRecord.fileSetConsumption.blocks -= count;
		} else {
			tfset->cachedRecord.fileSetConsumption.inodes -= count;
		}
		NFX_SPIN_UNLOCK(&tfset->spinlock);

		fs_unlockExclusive(context, &tfset->lock);
	}

	/*
	 * Point to this list from the context structure, so that
	 * we can get back our hands on this list during write completion
	 * to perform appropriate logging and transactions
	 */

	*list = quotaContextList;

	return;
}

/*
 * Iterate through all the objects in the quota context list and
 * free them. In the process release quotas and fsets as well
 */

EXPORTED void
fs_freeQuotaContextList(
	fs_Context		*context,
	fs_FileSystem		*core,	// core
	fs_QuotaContextList_t	*q	// Free this quota context list
)
{
	fs_FsetContext_t	*fsetContext;
	fs_QuotaContext_t	*quotaContext;

	FS_ASSERT(q != NULL && core != NULL);


	if (q->uQuotaList.head != NULL) {
		DLL_REMOVE_HEAD(&q->uQuotaList, quotaContext, list);
		while (quotaContext != NULL && quotaContext != DLL_NIL) {
			fs_releQuota(quotaContext->quota);
			fs_ramDealloc(FS_RAM_QUOTA_CONTEXT, quotaContext);
			DLL_REMOVE_HEAD(&q->uQuotaList, quotaContext, list);
		}
	}

	if (q->gQuotaList.head != NULL) {
		DLL_REMOVE_HEAD(&q->gQuotaList, quotaContext, list);
		while (quotaContext != NULL && quotaContext != DLL_NIL) {
			fs_releQuota(quotaContext->quota);
			fs_ramDealloc(FS_RAM_QUOTA_CONTEXT, quotaContext);
			DLL_REMOVE_HEAD(&q->gQuotaList, quotaContext, list);
		}
	}

	DLL_REMOVE_HEAD(&q->fsetList, fsetContext, list);
	while (fsetContext != NULL && fsetContext != DLL_NIL) {
		fs_releFileSet(context, core, fsetContext->fset);
		fs_ramDealloc(FS_RAM_FSET_CONTEXT, fsetContext);
		DLL_REMOVE_HEAD(&q->fsetList, fsetContext, list);
	}

	fs_ramDealloc(FS_RAM_QUOTA_CONTEXT_LIST, q);
}


/*
 * Flush the quota inode to disk asynchronously. This is called with
 * the inode whose quota is being updated held in shared mode, so that
 * its block count cannot change nor can its identity change. 
 * Apply the changes being made to the incore quota to on disk quota, 
 * so the transaction commit operations would update the data correctly
 */
PRIVATE void
fs_quotaOnFlushQuota(
	fs_Context					*context,
	fs_FileSystem					*core,
	fs_FileSetIncoreRecord_t	*fset,
	fs_QuotaIncoreRecord_t		*quota,
	int							type,
	fs_Inode					*inode
)
{
#if 0
TOBEUPDATED
	int32					id = quota->id;
	uint64					offset = id * sizeof (fs_QuotaDiskRecordPadded);
	fs_Inode				*quotaInode;
	uint32					boff;
	fs_Buf					*buf;
	fs_QuotaDiskRecord_t	*rec;

	FS_ASSERT(context != NULL && core != NULL && fset != NULL &&
			  quota != NULL && inode != NULL);

	FS_ASSERT(fs_lockHeldShare(context, &fset->lock));
	FS_ASSERT(fs_lockHeldShare(context, &quota->lock));
	FS_ASSERT(type == FS_QUOTA_USER || type == FS_QUOTA_GROUP);

	if (type == FS_QUOTA_USER) {
		quotaInode = I_GET_EXCL(0, fset->cachedRecord.userQuotaInode);
	} else {
		quotaInode = I_GET_EXCL(0, fset->cachedRecord.groupQuotaInode);
	}

	boff = (uint32)(offset & (FS_BLOCK_SIZE - 1));
	buf = O_GET_EXCL(quotaInode, offset >> FS_BLOCK_SHIFT);
	FS_ASSERT(buf != NULL);
	rec = (fs_QuotaDiskRecord_t *)&buf->u.quota->recs[boff>>FS_QUOTA_REC_SHIFT];
	rec->consumption.inodes++;
	rec->consumption.blocks += inode->dinode.numblocks;
	FS_BUF_WRITE_NO_LOG(buf, 0, FS_BLOCK_SIZE);
	fs_bufReleaseExclusive(context, buf);
    if (!fs_inodeWasModified(context, quotaInode)) {
		fs_inodeReleaseExclusive(context, quotaInode);
	}
#endif
}

/*
 * Flush the fset quota to disk asynchronously. This is called with
 * the inode whose quota is being updated held in shared mode, so that
 * its block count cannot change nor can its identity change. fset
 * structure is also exclusively locked, so its contents cannot
 * change. Apply the changes being made to the incore quota to
 * on disk quota, so the transaction commit operations would update
 * the data correctly
 */

PRIVATE void
fs_quotaOnFlushFset(
	fs_Context					*context,
	fs_FileSystem					*core,
	fs_FileSetIncoreRecord_t	*fset,
	fs_Inode					*inode,
	fs_Inode					*fsetInfoInode
)
{
	int32					id = fset->fileSetId;
	uint64					offset = id * sizeof (fs_FileSetDiskRecord_t);
	uint32					boff;
	fs_Buf					*buf;
	fs_FileSetDiskRecord_t	*rec;

	FS_ASSERT(context != NULL && core != NULL && fset != NULL && inode != NULL);

	FS_ASSERT(fs_lockHeldShare(context, &fsetInfoInode->lock));
	FS_ASSERT(fs_lockHeldShare(context, &fset->lock));

	boff = (uint32)(offset & (FS_BLOCK_SIZE - 1));
	buf = O_GET_EXCL(fsetInfoInode, offset >> FS_BLOCK_SHIFT);
	FS_ASSERT(buf != NULL);
	rec = (fs_FileSetDiskRecord_t *)(buf->u.data + boff);
	rec->fileSetConsumption.inodes++;
	rec->fileSetConsumption.blocks += inode->dinode.numblocks;
	FS_BUF_WRITE_NO_LOG(buf, 0, FS_BLOCK_SIZE);
	fs_bufReleaseExclusive(context, buf);
}

#if 0

/*
 * Quota On
 *
 * This routine is called from the quota on thread and it is assumed
 * that the caller has already marked the fileset as quota being
 * turned on.
 */

EXPORTED void
fs_quotaOn(
	fs_Context					*context,
	fs_FileSystem					*core,
	fs_FileSetIncoreRecord_t	*fset
)
{
	fs_FileSetIncoreRecord_t	*tfset = NULL;
	fs_QuotaContextList_t		*list;
	fs_FsetContext_t			*fsetContext, *tfsetContext;
	fs_Inode					*inode;
	uint64						inum;
	int32						uid, gid;
	fs_QuotaIncoreRecord_t		*quota;
	fs_QuotaBlk_t				qblk;	// on stack allocation
	fs_Inode					*quotaInode;
	fs_Inode					*fsetInfoInode = NULL;
	uint64						ilimit;
	fs_Buf						*buf;
	uint32						boff;
	uint64						offset;
	fs_FileSetDiskRecord_t		*drec;
	int							restart = 0;
	uint32						tuid, tgid;

	FS_ASSERT(context != NULL && core != NULL);
	
	/*
	 * Build a list of filesets whose inodes should be accounted for
	 * in this fileset.
	 */

	/*
     * TODO check this. I think holding exclusif traverseLock takes care of it
     * 
	 * XXX - To do. Mark the fset so that it cannot be removed while
	 * we are working on this. We protect the fset in question by
	 * caller holding the root of the fset, but the descendents
	 * are not protected.
	 */

	list = fs_ramAlloc(FS_RAM_QUOTA_CONTEXT_LIST,
					   sizeof (fs_QuotaContextList_t));
	DLL_INIT(&list->fsetList);
	DLL_INIT(&list->uQuotaList);
	DLL_INIT(&list->gQuotaList);
	fsetContext = fs_ramAlloc(FS_RAM_FSET_CONTEXT,
							  sizeof (fs_FsetContext_t));
	fsetContext->fset = fset;
	fs_holdFileSet(context, core, fset);
	fs_lockExclusive(context, &fset->traverseLock, NULL);
	DLL_ISOLATE_ELEM(fsetContext, list);
	DLL_INSERT_TAIL(&list->fsetList, fsetContext, list);
	
	DLL_ITERATE(tfsetContext, &list->fsetList, list) {

		DLL_ITERATE(tfset, &core->filesetList, fsetList) {

			if (tfset->parent == tfsetContext->fset) {
				fsetContext = fs_ramAlloc(FS_RAM_FSET_CONTEXT,
										  sizeof (fs_FsetContext_t));
				fsetContext->fset = tfset;
				fs_holdFileSet(context, core, tfset);
				fs_lockExclusive(context, &tfset->traverseLock, NULL);

				DLL_ISOLATE_ELEM(fsetContext, list);
				DLL_INSERT_TAIL(&list->fsetList, fsetContext, list);
			}
		}
	}

	ilimit = context->fileSystem->params.maxInodeNumber;

	/*
	 * Scan through all the inodes in this filesystem and
	 * for every inode that belongs to the fileset or any of
	 * its descendants, account for it. We are relying on the fact
	 * that lock implementation supports recursive locks, so that
	 * we don't special case locking of the fileset on whom quota
	 * is being turned on
	 */

	inum = 0;
	while (inum < ilimit) {

		if (inum < 2) {
			inum++;
			continue;
		}

        if (inum < FS_MAX_RESERVED_INODE && inum != FS_ROOT_INODE) {
            inum++;
            continue;
        }

		/*
		 * Be a good citizen and yield the processor every 1000
		 * inodes we have processed
		 */
		
		if (inum % 1000 == 0) {				
			//fs_yieldPeriodic(context);
			fs_sleep(context, 10);
		}
		
	    inode = I_GET_SHARED(0, inum);

        if (inode == NULL) {
            inum++;
            continue;
        }

		uid = inode->sd->u.nfs.uid;
		gid = inode->sd->u.nfs.gid;

		/*
		 * If the inode is not a directory, file, symlink, device, fifo
		 * or socket or it is free don't bother
		 * If it is a trashcan file (already deleted), don't bother either
		 */

		if ((!FS_I_IS_DIR(inode->dinode.attrs) &&
		    !FS_I_IS_REG(inode->dinode.attrs) &&
		    !FS_I_IS_SYML(inode->dinode.attrs) &&
			!FS_I_IS_CHR(inode->dinode.attrs) &&
			!FS_I_IS_BLK(inode->dinode.attrs) &&
			!FS_I_IS_FIFO(inode->dinode.attrs) &&
			!FS_I_IS_SOCK(inode->dinode.attrs)) ||
			inode->dinode.attrs == 0 ||
			inode->dinode.piu.parentInum == FS_TRASHCAN_INODE)
        {
			fs_inodeRelease(context, inode);
			inum++;
			continue;
		}

		inode->fset = fs_getFset(context, core, inode);

#if 0
TOBEUPDATED
		/*
		 * First allocate any space that needs to be allocated
		 * to quota inodes, once we are done for all the filesets
		 * only then we will count this inode for quota
		 */

		DLL_ITERATE(tfsetContext, &list->fsetList, list) {

			tfset = tfsetContext->fset;
			/*
			 * If the inode does not belong to this fileset don't
			 * bother. If it belongs to any of the filesets that
			 * are decedents, then we will do the necessary quota
			 * initialization
			 */
			if (tfset->fileSetId != inode->dinode.fsetNum) {
				continue;
			}
			quota = fs_getUGQuota(context, core, fset, FS_QUOTA_USER, uid, 
                                    FALSE);
			fs_lockShared(context, &tfset->lock, NULL);
			fs_lockShared(context, &quota->lock, NULL);
			NFX_SPIN_LOCK(&quota->spinlock);
			if (!(quota->flag & FS_QUOTA_INCORE_NEEDS_INIT)) {
				NFX_SPIN_UNLOCK(&quota->spinlock);
				fs_unlockShared(context, &quota->lock);
				fs_unlockShared(context, &tfset->lock);
            	fs_releQuota(quota);
				continue;
			}

			NFX_SPIN_UNLOCK(&quota->spinlock);
			fs_unlockShared(context, &quota->lock);
			fs_lockExclusive(context, &quota->lock, NULL);
			NFX_SPIN_LOCK(&quota->spinlock);
            
			/*
			 * Reverify that we don't need to initialize
			 * quota for this id
			 */

			if (!(quota->flag & FS_QUOTA_INCORE_NEEDS_INIT)) {
				NFX_SPIN_UNLOCK(&quota->spinlock);
				fs_unlockExclusive(context, &quota->lock);
				fs_unlockShared(context, &tfset->lock);
            	fs_releQuota(quota);
				continue;
			}
			NFX_SPIN_UNLOCK(&quota->spinlock);

		   /*
			* Initialize quota with default quotas.
			* Duplicate some of the code from setUGQuota
			* here as we cannot drop fset lock from this
			* point onwards
			*/

			bzero (&qblk, sizeof (fs_QuotaBlk_t));
			qblk.id = uid;
			qblk.flag = FS_QUOTA_USER;
			qblk.inodeSoftLimit = quota->cachedRecords.quota.inodeSoftLimit;
			qblk.inodeHardLimit = quota->cachedRecords.quota.inodeHardLimit;
			qblk.inodeTimeLimit = quota->cachedRecords.quota.inodeTimeLimit;
			qblk.blockSoftLimit = quota->cachedRecords.quota.blockSoftLimit;
			qblk.blockHardLimit = quota->cachedRecords.quota.blockHardLimit;
			qblk.blockTimeLimit = quota->cachedRecords.quota.blockTimeLimit;

			quotaInode = I_GET_EXCL(0, fset->cachedRecord.userQuotaInode);
			fs_initializeUGQuotaRecord(context, core, fset, quotaInode, 
                                        &qblk, uid,	
                                        (FS_QUOTA_USER|FS_QUOTA_USE_DEFAULTS));

            if (!fs_inodeWasModified(context, quotaInode)) {
				fs_inodeReleaseExclusive(context, quotaInode);
			}

			/*
			 * Commit the transaction
			 */

			NFX_SPIN_LOCK(&quota->spinlock);
			quota->flag &= ~FS_QUOTA_INCORE_NEEDS_INIT;
			NFX_SPIN_UNLOCK(&quota->spinlock);
			fs_unlockExclusive(context, &quota->lock);
			fs_unlockShared(context, &tfset->lock);
           	fs_releQuota(quota);
			tuid = uid;
			tgid = gid;
			fs_inodeRelease(context, inode);

            fs_logCommitEntry(context);
			fs_logBeginTransaction(context);

			inode = I_GET_SHARED(0, inum);
	    	inum = inode->inum;
			uid = inode->sd->u.nfs.uid;
			gid = inode->sd->u.nfs.gid;

			/*
		 	 * If the inode is free, don't bother counting
		 	 */

			if (inode->dinode.attrs == 0) {
				fs_inodeRelease(context, inode);
				restart = 1;
				break;
			}
			if (uid != tuid || gid != tgid) {
				fs_inodeRelease(context, inode);
				restart = 1;
				break;
			}

        }

		if (restart) {
			restart = 0;
			continue;
		}

		FS_ASSERT(inode->inum == fset->cachedRecord.rootInode ||
				  fs_lockHeldShare(context, &inode->lock));
		DLL_ITERATE(tfsetContext, &list->fsetList, list) {
			tfset = tfsetContext->fset;

			/*
			 * If the inode does not belong to this fileset don't
			 * bother. If it belongs to any of the filesets that
			 * are descendents, then we will do the necessary quota
			 * initialization
			 */

			if (tfset->fileSetId != inode->dinode.fsetNum) {
				continue;
			}
			quota = fs_getUGQuota(context, core, fset, FS_QUOTA_GROUP, gid,
                                    FALSE);
			fs_lockShared(context, &tfset->lock, NULL);
			fs_lockShared(context, &quota->lock, NULL);
			NFX_SPIN_LOCK(&quota->spinlock);
			if (!(quota->flag & FS_QUOTA_INCORE_NEEDS_INIT)) {
				NFX_SPIN_UNLOCK(&quota->spinlock);
				fs_unlockShared(context, &quota->lock);
				fs_unlockShared(context, &tfset->lock);
               	fs_releQuota(quota);
				continue;
			}

			NFX_SPIN_UNLOCK(&quota->spinlock);
			fs_unlockShared(context, &quota->lock);
			fs_lockExclusive(context, &quota->lock, NULL);
			NFX_SPIN_LOCK(&quota->spinlock);

			/*
			 * Reverify that we don't need to initialize
			 * quota for this id
			 */

			if (!(quota->flag & FS_QUOTA_INCORE_NEEDS_INIT)) {
				NFX_SPIN_UNLOCK(&quota->spinlock);
				fs_unlockExclusive(context, &quota->lock);
				fs_unlockShared(context, &tfset->lock);
               	fs_releQuota(quota);
				continue;
			}
			NFX_SPIN_UNLOCK(&quota->spinlock);
		   /*
			* Initialize quota with default quotas.
			* Duplicate some of the code from setUGQuota
			* here as we cannot drop fset lock from this
			* point onwards
			*/

			bzero (&qblk, sizeof (fs_QuotaBlk_t));
			qblk.id = uid;
			qblk.flag = FS_QUOTA_USER;
			qblk.inodeSoftLimit =
			quota->cachedRecords.quota.inodeSoftLimit;
			qblk.inodeHardLimit =
			quota->cachedRecords.quota.inodeHardLimit;
			qblk.inodeTimeLimit =
			quota->cachedRecords.quota.inodeTimeLimit;
			qblk.blockSoftLimit =
			quota->cachedRecords.quota.blockSoftLimit;
			qblk.blockHardLimit =
			quota->cachedRecords.quota.blockHardLimit;
			qblk.blockTimeLimit =
			quota->cachedRecords.quota.blockTimeLimit;

            quotaInode = I_GET_EXCL(0, fset->cachedRecord.groupQuotaInode);
			fs_initializeUGQuotaRecord(context, core, fset, quotaInode, 
                                        &qblk, gid, 
                                        (FS_QUOTA_GROUP|FS_QUOTA_USE_DEFAULTS));

            if (!fs_inodeWasModified(context, quotaInode)) {
				fs_inodeReleaseExclusive(context, quotaInode);
			}

			/*
			 * Commit the transaction
			 */

			NFX_SPIN_LOCK(&quota->spinlock);
			quota->flag &= ~FS_QUOTA_INCORE_NEEDS_INIT;
			NFX_SPIN_UNLOCK(&quota->spinlock);
			fs_unlockExclusive(context, &quota->lock);
			fs_unlockShared(context, &tfset->lock);
           	fs_releQuota(quota);
			tuid = uid;
			tgid = gid;
			fs_inodeRelease(context, inode);

            fs_logCommitEntry(context);
			fs_logBeginTransaction(context);

			inode = I_GET_SHARED(0, inum);
	    	inum = inode->inum;
			uid = inode->sd->u.nfs.uid;
			gid = inode->sd->u.nfs.gid;

			/*
		 	 * If the inode is free, don't bother counting
		 	 */

			if (inode->dinode.attrs == 0) {
				fs_inodeRelease(context, inode);
				restart = 1;
				break;
			}
			if (uid != tuid || gid != tgid) {
				fs_inodeRelease(context, inode);
				restart = 1;
				break;
			}

		}
		if (restart) {
			restart = 0;
			continue;
		}

		/*
		 * Count this inode's usage. We don't need to
		 * perform any transactions here because in case
		 * of a crash, quota on will be aborted unless
		 * completely turned on. Before setting the
		 * quota on flag on the fileset, we will freeze
		 * the filesystem, which will write all the quota
		 * information on disk before setting the flag while
		 * the filesystem is frozen.
		 *
		 * If blocks need to be allocated in quota file
		 * then we will perform a transaction, so that
		 * we don't waste the work we have done here.
		 */ 

		/*
		 * First get the user quota for each of
		 * filesets in the list
		 */

//		FS_ASSERT(fs_lockHeldShare(context, &inode->lock));
		FS_ASSERT(inode->inum == fset->cachedRecord.rootInode ||
				  fs_lockHeldShare(context, &inode->lock));
		DLL_ITERATE(tfsetContext, &list->fsetList, list) {
			tfset = tfsetContext->fset;

			/*
			 * If the inode does not belong to this fileset don't
			 * bother. If it belongs to any of the filesets that
			 * are decedents, then we will do the necessary quota
			 * initialization
			 */

			if (tfset->fileSetId != inode->dinode.fsetNum) {
				continue;
			}
			quota = fs_getUGQuota(context, core, fset, FS_QUOTA_USER, uid,
                                    FALSE);
			fs_lockShared(context, &fset->lock, NULL);
			fs_lockExclusive(context, &quota->lock, NULL);
			NFX_SPIN_LOCK(&quota->spinlock);
			if (!(quota->flag & FS_QUOTA_INCORE_NEEDS_INIT)) {
				quota->cachedRecords.consumption.inodes++;
				quota->cachedRecords.consumption.blocks +=
													inode->dinode.numblocks;

				/*
				 * XXX - Do we want notify users of excess at quota
				 * on time?
				 */

				NFX_SPIN_UNLOCK(&quota->spinlock);

				/*
				 * Copy the data into disk buffers and
				 * mark the buffers for delayed flush
				 */

				fs_quotaOnFlushQuota(context, core, fset, quota,
    								  FS_QUOTA_USER, inode);
				fs_unlockExclusive(context, &quota->lock);
				fs_unlockShared(context, &fset->lock);
               	fs_releQuota(quota);

			} else {
					FS_ASSERT(0);
			}

		}

        
		/*
		 * Do the same for group quota
		 */
//		FS_ASSERT(fs_lockHeldShare(context, &inode->lock));
		FS_ASSERT(inode->inum == fset->cachedRecord.rootInode ||
				  fs_lockHeldShare(context, &inode->lock));
			DLL_ITERATE(tfsetContext, &list->fsetList, list) {
			tfset = tfsetContext->fset;

            /*
			 * If the inode does not belong to this fileset don't
			 * bother. If it belongs to any of the filesets that
			 * are decedents, then we will do the necessary quota
			 * initialization
			 */

			if (tfset->fileSetId != inode->dinode.fsetNum) {
				continue;
			}
			quota = fs_getUGQuota(context, core, fset, FS_QUOTA_GROUP, gid,
                                    FALSE);
			fs_lockShared(context, &fset->lock, NULL);
			fs_lockExclusive(context, &quota->lock, NULL);
			NFX_SPIN_LOCK(&quota->spinlock);
			if (!(quota->flag & FS_QUOTA_INCORE_NEEDS_INIT)) {
				quota->cachedRecords.consumption.inodes++;
				quota->cachedRecords.consumption.blocks +=
													inode->dinode.numblocks;

				/*
				 * XXX - Do we want notify users of excess at quota
				 * on time?
				 */

				NFX_SPIN_UNLOCK(&quota->spinlock);

				/*
				 * Copy the data into disk buffers and
				 * mark the buffers for delayed flush
				 */

				fs_quotaOnFlushQuota(context, core, fset, quota,
	 								  	 FS_QUOTA_GROUP, inode);
				fs_unlockExclusive(context, &quota->lock);
				fs_unlockShared(context, &fset->lock);
               	fs_releQuota(quota);

			} else {
				FS_ASSERT(0);
			}
		}

#endif
		/*
		 * Now update each of the filesets
		 */

			FS_ASSERT(inode->inum == fset->cachedRecord.rootInode ||
					  fs_lockHeldShare(context, &inode->lock));
			DLL_ITERATE(tfsetContext, &list->fsetList, list) {

			tfset = tfsetContext->fset;

            /*
			 * If the inode does not belong to this fileset don't
			 * bother. If it belongs to any of the filesets that
			 * are decedents, then we will do the necessary quota
			 * initialization
			 */

			if (tfset->fileSetId != inode->dinode.fsetNum) {
				continue;
			}

			fsetInfoInode = I_GET_SHARED(0, FS_FILESET_INFO_INODE);
			fs_lockExclusive(context, &fset->lock, NULL);
			NFX_SPIN_LOCK(&tfset->spinlock);
			fset->cachedRecord.fileSetConsumption.inodes++;
			fset->cachedRecord.fileSetConsumption.blocks +=
													inode->dinode.numblocks;
			NFX_SPIN_UNLOCK(&tfset->spinlock);
			/*
			 * Copy the data into disk buffers and
			 * mark the buffers for delayed flush
			 */
			fs_quotaOnFlushFset(context, core, fset, inode,	fsetInfoInode);
			fs_unlockExclusive(context, &fset->lock);
			fs_inodeRelease(context, fsetInfoInode);
		}

		/*
		 * Now record the inode in the fileset as been processed, so
		 * any new reference to it would do the quota comsumption update
		 */

//		FS_ASSERT(fs_lockOwnedShared(context, &inode->lock));
		fset->inodeSeen = inode->inum;
		fs_inodeRelease(context, inode);
		inum++;

		/*
		 * Revalidate the size of the inode inode file, and
		 * make sure we are not reading beyond
		 */

#if 0
		ilist = I_GET_SHARED(0, FS_INODE_INODE);
		ilimit = ilist->dinode.size / sizeof (fs_Dinode);
		fs_inodeRelease(context, ilist);
#endif
	}
    
	DLL_ITERATE(tfsetContext, &list->fsetList, list) {
		tfset = tfsetContext->fset;
		fs_unlockExclusive(context, &tfset->traverseLock);
	}
	/*
	 * Free the quota context structure
	 */

	fs_freeQuotaContextList(context, core, list);

#if 0
TOBEUPDATED
    //Make sure all buffers are written out to disk
  	quotaInode = I_GET_EXCL(0, fset->cachedRecord.userQuotaInode);
    fs_bufSyncInodeBufs(context, quotaInode);
	if (!fs_inodeWasModified(context, quotaInode)) {
	    fs_inodeReleaseExclusive(context, quotaInode);
    }
    
  	quotaInode = I_GET_EXCL(0, fset->cachedRecord.groupQuotaInode);
    fs_bufSyncInodeBufs(context, quotaInode);
	if (!fs_inodeWasModified(context, quotaInode)) {
	    fs_inodeReleaseExclusive(context, quotaInode);
    }
#endif
    
	fsetInfoInode = I_GET_EXCL(0, FS_FILESET_INFO_INODE);
    fs_bufSyncInodeBufs(context, fsetInfoInode);

	/*
	 * Now write quota on flag to disk
	 */
	
	fs_lockExclusive(context, &fset->lock, NULL);
	NFX_SPIN_LOCK(&fset->spinlock);
	fset->incoreFlag &= ~(FS_FILESET_QUOTA_TURN_ON|FS_FILESET_QUOTA_ON_THREAD);
	fset->incoreFlag |= FS_FILESET_QUOTA_ON;
	fset->cachedRecord.fileSetFlags |= FS_FILESET_QUOTA_ON;
	NFX_SPIN_UNLOCK(&fset->spinlock);

	/*
	 * Mark it on disk as well
	 */

	offset = fset->fileSetId * sizeof (fs_FileSetDiskRecord_t);
	boff = (uint32)(offset & (FS_BLOCK_SIZE - 1));
	buf = O_GET_EXCL(fsetInfoInode, offset >> FS_BLOCK_SHIFT);

	drec = (fs_FileSetDiskRecord_t *)(buf->u.data + boff);

	drec->fileSetFlags |= FS_FILESET_QUOTA_ON;

	FS_BUF_WRITE(buf, (uint32)((char *)&drec->fileSetFlags -
							    buf->u.data), sizeof (drec->fileSetFlags));
	fs_unlockExclusive(context, &fset->lock);

	if (!fs_inodeWasModified(context, fsetInfoInode)) {
	    fs_inodeReleaseExclusive(context, fsetInfoInode);
    }
}

#endif


/*
 * Quota On
 *
 * This routine is called from the quota on thread and it is assumed
 * that the caller has already marked the fileset as quota being
 * turned on.
 */

EXPORTED void
fs_quotaOn(
	fs_Context					*context,
	fs_FileSystem				*core,
	fs_FileSetIncoreRecord_t	*fset
)
{
	fs_FileSetIncoreRecord_t	*tfset = NULL;
	fs_QuotaContextList_t		*list;
	fs_FsetContext_t			*fsetContext, *tfsetContext;
	fs_Inode					*inode;
	uint64						inum;
	int32						uid, gid;
	fs_QuotaIncoreRecord_t		*quota;
	fs_QuotaBlk_t				qblk;	// on stack allocation
	fs_Inode					*quotaInode;
	fs_Inode					*fsetInfoInode = NULL;
	uint64						ilimit;
	fs_Buf						*buf;
	uint32						boff;
	uint64						offset;
	fs_FileSetDiskRecord_t		*drec;
	int							restart = 0;
	uint32						tuid, tgid;

	FS_ASSERT(context != NULL && core != NULL);
	
	/*
	 * Build a list of filesets whose inodes should be accounted for
	 * in this fileset.
	 */

	/*
     * TODO check this. I think holding exclusif traverseLock takes care of it
     * 
	 * XXX - To do. Mark the fset so that it cannot be removed while
	 * we are working on this. We protect the fset in question by
	 * caller holding the root of the fset, but the descendents
	 * are not protected.
	 */

	list = fs_ramAlloc(FS_RAM_QUOTA_CONTEXT_LIST,
					   sizeof (fs_QuotaContextList_t));
	DLL_INIT(&list->fsetList);
	DLL_INIT(&list->uQuotaList);
	DLL_INIT(&list->gQuotaList);
	fsetContext = fs_ramAlloc(FS_RAM_FSET_CONTEXT,
							  sizeof (fs_FsetContext_t));
	fsetContext->fset = fset;
	fs_holdFileSet(context, core, fset);
	fs_lockExclusive(context, &fset->traverseLock, NULL);
	DLL_ISOLATE_ELEM(fsetContext, list);
	DLL_INSERT_TAIL(&list->fsetList, fsetContext, list);
	
	DLL_ITERATE(tfsetContext, &list->fsetList, list) {

		DLL_ITERATE(tfset, &core->filesetList, fsetList) {

			if (tfset->parent == tfsetContext->fset) {
				fsetContext = fs_ramAlloc(FS_RAM_FSET_CONTEXT,
										  sizeof (fs_FsetContext_t));
				fsetContext->fset = tfset;
				fs_holdFileSet(context, core, tfset);
				fs_lockExclusive(context, &tfset->traverseLock, NULL);

				DLL_ISOLATE_ELEM(fsetContext, list);
				DLL_INSERT_TAIL(&list->fsetList, fsetContext, list);
			}
		}
	}

	ilimit = context->fileSystem->params.maxInodeNumber;

	/*
	 * Scan through all the inodes in this filesystem and
	 * for every inode that belongs to the fileset or any of
	 * its descendants, account for it. We are relying on the fact
	 * that lock implementation supports recursive locks, so that
	 * we don't special case locking of the fileset on whom quota
	 * is being turned on
	 */

	inum = 0;
	while (inum < ilimit) {

		if (inum < 2) {
			inum++;
			continue;
		}

        if (inum < FS_MAX_RESERVED_INODE && inum != FS_ROOT_INODE) {
            inum++;
            continue;
        }

		/*
		 * Be a good citizen and yield the processor every 1000
		 * inodes we have processed
		 */
		
		if (inum % 1000 == 0) {				
			//fs_yieldPeriodic(context);
			fs_sleep(context, 10);
		}
		
	    inode = I_GET_SHARED(0, inum);

        if (inode == NULL) {
            inum++;
            continue;
        }

		uid = inode->sd->u.nfs.uid;
		gid = inode->sd->u.nfs.gid;

		/*
		 * If the inode is not a directory, file, symlink, device, fifo
		 * or socket or it is free don't bother
		 * If it is a trashcan file (already deleted), don't bother either
		 */

		if ((!FS_I_IS_DIR(inode->dinode.attrs) &&
		    !FS_I_IS_REG(inode->dinode.attrs) &&
		    !FS_I_IS_SYML(inode->dinode.attrs) &&
			!FS_I_IS_CHR(inode->dinode.attrs) &&
			!FS_I_IS_BLK(inode->dinode.attrs) &&
			!FS_I_IS_FIFO(inode->dinode.attrs) &&
			!FS_I_IS_SOCK(inode->dinode.attrs)) ||
			inode->dinode.attrs == 0 ||
			inode->dinode.piu.parentInum == FS_TRASHCAN_INODE)
        {
			fs_inodeRelease(context, inode);
			inum++;
			continue;
		}

        //TODO check why we need to do that ???
		inode->fset = fs_getFset(context, core, inode);

#if 0
TOBEUPDATED
		/*
		 * First allocate any space that needs to be allocated
		 * to quota inodes, once we are done for all the filesets
		 * only then we will count this inode for quota
		 */

		DLL_ITERATE(tfsetContext, &list->fsetList, list) {

			tfset = tfsetContext->fset;
			/*
			 * If the inode does not belong to this fileset don't
			 * bother. If it belongs to any of the filesets that
			 * are decedents, then we will do the necessary quota
			 * initialization
			 */
			if (tfset->fileSetId != inode->dinode.fsetNum) {
				continue;
			}
			quota = fs_getUGQuota(context, core, fset, FS_QUOTA_USER, uid, 
                                    FALSE);
			fs_lockShared(context, &tfset->lock, NULL);
			fs_lockShared(context, &quota->lock, NULL);
			NFX_SPIN_LOCK(&quota->spinlock);
			if (!(quota->flag & FS_QUOTA_INCORE_NEEDS_INIT)) {
				NFX_SPIN_UNLOCK(&quota->spinlock);
				fs_unlockShared(context, &quota->lock);
				fs_unlockShared(context, &tfset->lock);
            	fs_releQuota(quota);
				continue;
			}
			NFX_SPIN_UNLOCK(&quota->spinlock);

            if (!fs_haveMinfreeAvail(core))   {
                //We need to allocate a quota block and there is no space left
                goto errNoSpace;
            }

		   /*
			* Initialize quota with default quotas.
			* Duplicate some of the code from setUGQuota
			* here as we cannot drop fset lock from this
			* point onwards
			*/

			bzero (&qblk, sizeof (fs_QuotaBlk_t));
			qblk.id = uid;
			qblk.flag = FS_QUOTA_USER;
			qblk.inodeSoftLimit = quota->cachedRecords.quota.inodeSoftLimit;
			qblk.inodeHardLimit = quota->cachedRecords.quota.inodeHardLimit;
			qblk.inodeTimeLimit = quota->cachedRecords.quota.inodeTimeLimit;
			qblk.blockSoftLimit = quota->cachedRecords.quota.blockSoftLimit;
			qblk.blockHardLimit = quota->cachedRecords.quota.blockHardLimit;
			qblk.blockTimeLimit = quota->cachedRecords.quota.blockTimeLimit;

			quotaInode = I_GET_EXCL(0, fset->cachedRecord.userQuotaInode);
			fs_initializeUGQuotaRecord(context, core, fset, quotaInode, 
                                        &qblk, uid,	
                                        (FS_QUOTA_USER|FS_QUOTA_USE_DEFAULTS));

            if (!fs_inodeWasModified(context, quotaInode)) {
				fs_inodeReleaseExclusive(context, quotaInode);
			}

			/*
			 * Commit the transaction
			 */

			NFX_SPIN_LOCK(&quota->spinlock);
			quota->flag &= ~FS_QUOTA_INCORE_NEEDS_INIT;
			NFX_SPIN_UNLOCK(&quota->spinlock);
			fs_unlockShared(context, &quota->lock);
			fs_unlockShared(context, &tfset->lock);
           	fs_releQuota(quota);
			tuid = uid;
			tgid = gid;
			fs_inodeRelease(context, inode);

            fs_logCommitEntry(context);
			fs_logBeginTransaction(context);

			inode = I_GET_SHARED(0, inum);
	    	inum = inode->inum;
			uid = inode->sd->u.nfs.uid;
			gid = inode->sd->u.nfs.gid;

			/*
		 	 * If the inode is free, don't bother counting
		 	 */

			if (inode->dinode.attrs == 0) {
				fs_inodeRelease(context, inode);
				restart = 1;
				break;
			}
			if (uid != tuid || gid != tgid) {
				fs_inodeRelease(context, inode);
				restart = 1;
				break;
			}

        }

		if (restart) {
			restart = 0;
			continue;
		}

		FS_ASSERT(inode->inum == fset->cachedRecord.rootInode ||
				  fs_lockHeldShare(context, &inode->lock));
		DLL_ITERATE(tfsetContext, &list->fsetList, list) {
			tfset = tfsetContext->fset;

			/*
			 * If the inode does not belong to this fileset don't
			 * bother. If it belongs to any of the filesets that
			 * are descendents, then we will do the necessary quota
			 * initialization
			 */

			if (tfset->fileSetId != inode->dinode.fsetNum) {
				continue;
			}
			quota = fs_getUGQuota(context, core, fset, FS_QUOTA_GROUP, gid,
                                    FALSE);
			fs_lockShared(context, &tfset->lock, NULL);
			fs_lockShared(context, &quota->lock, NULL);
			NFX_SPIN_LOCK(&quota->spinlock);
			if (!(quota->flag & FS_QUOTA_INCORE_NEEDS_INIT)) {
				NFX_SPIN_UNLOCK(&quota->spinlock);
				fs_unlockShared(context, &quota->lock);
				fs_unlockShared(context, &tfset->lock);
               	fs_releQuota(quota);
				continue;
			}
			NFX_SPIN_UNLOCK(&quota->spinlock);
            
            if (!fs_haveMinfreeAvail(core))   {
                //We need to allocate a quota block and there is no space left
                goto errNoSpace;
            }

		   /*
			* Initialize quota with default quotas.
			* Duplicate some of the code from setUGQuota
			* here as we cannot drop fset lock from this
			* point onwards
			*/

			bzero (&qblk, sizeof (fs_QuotaBlk_t));
			qblk.id = uid;
			qblk.flag = FS_QUOTA_USER;
			qblk.inodeSoftLimit =
			quota->cachedRecords.quota.inodeSoftLimit;
			qblk.inodeHardLimit =
			quota->cachedRecords.quota.inodeHardLimit;
			qblk.inodeTimeLimit =
			quota->cachedRecords.quota.inodeTimeLimit;
			qblk.blockSoftLimit =
			quota->cachedRecords.quota.blockSoftLimit;
			qblk.blockHardLimit =
			quota->cachedRecords.quota.blockHardLimit;
			qblk.blockTimeLimit =
			quota->cachedRecords.quota.blockTimeLimit;

            quotaInode = I_GET_EXCL(0, fset->cachedRecord.groupQuotaInode);
			fs_initializeUGQuotaRecord(context, core, fset, quotaInode, 
                                        &qblk, gid, 
                                        (FS_QUOTA_GROUP|FS_QUOTA_USE_DEFAULTS));

            if (!fs_inodeWasModified(context, quotaInode)) {
				fs_inodeReleaseExclusive(context, quotaInode);
			}

			/*
			 * Commit the transaction
			 */

			NFX_SPIN_LOCK(&quota->spinlock);
			quota->flag &= ~FS_QUOTA_INCORE_NEEDS_INIT;
			NFX_SPIN_UNLOCK(&quota->spinlock);
			fs_unlockShared(context, &quota->lock);
			fs_unlockShared(context, &tfset->lock);
           	fs_releQuota(quota);
			tuid = uid;
			tgid = gid;
			fs_inodeRelease(context, inode);

            fs_logCommitEntry(context);
			fs_logBeginTransaction(context);

			inode = I_GET_SHARED(0, inum);
	    	inum = inode->inum;
			uid = inode->sd->u.nfs.uid;
			gid = inode->sd->u.nfs.gid;

			/*
		 	 * If the inode is free, don't bother counting
		 	 */

			if (inode->dinode.attrs == 0) {
				fs_inodeRelease(context, inode);
				restart = 1;
				break;
			}
			if (uid != tuid || gid != tgid) {
				fs_inodeRelease(context, inode);
				restart = 1;
				break;
			}

		}
		if (restart) {
			restart = 0;
			continue;
		}
#endif
        
		/*
		 * Count this inode's usage. We don't need to
		 * perform any transactions here because in case
		 * of a crash, quota on will be aborted unless
		 * completely turned on. Before setting the
		 * quota on flag on the fileset, we will freeze
		 * the filesystem, which will write all the quota
		 * information on disk before setting the flag while
		 * the filesystem is frozen.
		 *
		 * If blocks need to be allocated in quota file
		 * then we will perform a transaction, so that
		 * we don't waste the work we have done here.
		 */ 

		/*
		 * First get the user quota for each of
		 * filesets in the list
		 */

#if 0
TOBEUPDATED
		FS_ASSERT(inode->inum == fset->cachedRecord.rootInode ||
				  fs_lockHeldShare(context, &inode->lock));
		DLL_ITERATE(tfsetContext, &list->fsetList, list) {
			tfset = tfsetContext->fset;

			/*
			 * If the inode does not belong to this fileset don't
			 * bother. If it belongs to any of the filesets that
			 * are decedents, then we will do the necessary quota
			 * initialization
			 */

			if (tfset->fileSetId != inode->dinode.fsetNum) {
				continue;
			}

			quota = fs_getUGQuota(context, core, fset, FS_QUOTA_USER, uid,
                                    FALSE);
			fs_lockShared(context, &fset->lock, NULL);
			fs_lockShared(context, &quota->lock, NULL);
			NFX_SPIN_LOCK(&quota->spinlock);
			if (!(quota->flag & FS_QUOTA_INCORE_NEEDS_INIT)) {
				quota->cachedRecords.consumption.inodes++;
				quota->cachedRecords.consumption.blocks +=
													inode->dinode.numblocks;

				/*
				 * XXX - Do we want notify users of excess at quota
				 * on time?
				 */

				NFX_SPIN_UNLOCK(&quota->spinlock);

				/*
				 * Copy the data into disk buffers and
				 * mark the buffers for delayed flush
				 */

				fs_quotaOnFlushQuota(context, core, fset, quota,
    								  FS_QUOTA_USER, inode);
				fs_unlockShared(context, &quota->lock);
				fs_unlockShared(context, &fset->lock);
               	fs_releQuota(quota);

			} else {
					FS_ASSERT(0);
			}

		}

        
		/*
		 * Do the same for group quota
		 */
		FS_ASSERT(inode->inum == fset->cachedRecord.rootInode ||
				  fs_lockHeldShare(context, &inode->lock));
			DLL_ITERATE(tfsetContext, &list->fsetList, list) {
			tfset = tfsetContext->fset;

            /*
			 * If the inode does not belong to this fileset don't
			 * bother. If it belongs to any of the filesets that
			 * are decedents, then we will do the necessary quota
			 * initialization
			 */

			if (tfset->fileSetId != inode->dinode.fsetNum) {
				continue;
			}
			quota = fs_getUGQuota(context, core, fset, FS_QUOTA_GROUP, gid,
                                    FALSE);
			fs_lockShared(context, &fset->lock, NULL);
			fs_lockShared(context, &quota->lock, NULL);
			NFX_SPIN_LOCK(&quota->spinlock);
			if (!(quota->flag & FS_QUOTA_INCORE_NEEDS_INIT)) {
				quota->cachedRecords.consumption.inodes++;
				quota->cachedRecords.consumption.blocks +=
													inode->dinode.numblocks;

				/*
				 * XXX - Do we want notify users of excess at quota
				 * on time?
				 */

				NFX_SPIN_UNLOCK(&quota->spinlock);

				/*
				 * Copy the data into disk buffers and
				 * mark the buffers for delayed flush
				 */

				fs_quotaOnFlushQuota(context, core, fset, quota,
	 								  	 FS_QUOTA_GROUP, inode);
				fs_unlockShared(context, &quota->lock);
				fs_unlockShared(context, &fset->lock);
               	fs_releQuota(quota);

			} else {
				FS_ASSERT(0);
			}
		}
#endif
		/*
		 * Now update each of the filesets
		 */

			FS_ASSERT(inode->inum == fset->cachedRecord.rootInode ||
					  fs_lockHeldShare(context, &inode->lock));
			DLL_ITERATE(tfsetContext, &list->fsetList, list) {

			tfset = tfsetContext->fset;

            /*
			 * If the inode does not belong to this fileset don't
			 * bother. If it belongs to any of the filesets that
			 * are decedents, then we will do the necessary quota
			 * initialization
			 */

			if (tfset->fileSetId != inode->dinode.fsetNum) {
				continue;
			}

			fsetInfoInode = I_GET_SHARED(0, FS_FILESET_INFO_INODE);
			fs_lockShared(context, &fset->lock, NULL);
			NFX_SPIN_LOCK(&tfset->spinlock);
			fset->cachedRecord.fileSetConsumption.inodes++;
			fset->cachedRecord.fileSetConsumption.blocks +=
													inode->dinode.numblocks;
			NFX_SPIN_UNLOCK(&tfset->spinlock);
			/*
			 * Copy the data into disk buffers and
			 * mark the buffers for delayed flush
			 */
			fs_quotaOnFlushFset(context, core, fset, inode,	fsetInfoInode);
			fs_unlockShared(context, &fset->lock);
			fs_inodeRelease(context, fsetInfoInode);
		}

		/*
		 * Now record the inode in the fileset as been processed, so
		 * any new reference to it would do the quota comsumption update
		 */

        NFX_SPIN_LOCK(&fset->spinlock);
		fset->inodeSeen = inode->inum;
        NFX_SPIN_UNLOCK(&fset->spinlock);
		fs_inodeRelease(context, inode);
		inum++;

        //Check log state
        fs_makeSureLogIsNotFull(context);

        //Drop the modify lock for any thread wishing to get it exclusive
        fs_unlockShared(context, &context->fileSystem->modifyLock);
        if (fs_growIfNeeded(context)
         || !fs_haveMininodesAvail(context->fileSystem))
        {
            fs_sleep (context, 0);
        } else {
            fs_yieldPeriodic(context);
        }
        //Get it again
        fs_lockShared(context, &context->fileSystem->modifyLock, NULL);        
       
		//Update the size of the inode inode file
        ilimit = context->fileSystem->params.maxInodeNumber;
	}
    
	DLL_ITERATE(tfsetContext, &list->fsetList, list) {
		tfset = tfsetContext->fset;
		fs_unlockExclusive(context, &tfset->traverseLock);
	}
	/*
	 * Free the quota context structure
	 */

	fs_freeQuotaContextList(context, core, list);
    
#if 0
TOBEUPDATED
    //Make sure all buffers are written out to disk
  	quotaInode = I_GET_EXCL(0, fset->cachedRecord.userQuotaInode);
    fs_bufSyncInodeBufs(context, quotaInode);
	if (!fs_inodeWasModified(context, quotaInode)) {
	    fs_inodeReleaseExclusive(context, quotaInode);
    }
    
  	quotaInode = I_GET_EXCL(0, fset->cachedRecord.groupQuotaInode);
    fs_bufSyncInodeBufs(context, quotaInode);
	if (!fs_inodeWasModified(context, quotaInode)) {
	    fs_inodeReleaseExclusive(context, quotaInode);
    }
#endif
    
	fsetInfoInode = I_GET_EXCL(0, FS_FILESET_INFO_INODE);
    fs_bufSyncInodeBufs(context, fsetInfoInode);

    /*
     * Make sure that the volume log is empty, so that log replay does not
     * overwrite our not logged writes 
     */
    
    //Drop the modify lock
    fs_unlockShared(context, &core->modifyLock);
       
    fs_freeze(context);
    FS_ASSERT(core->logRoot->currLen == 0);
    fs_thaw(context);
    
    //Get it again
    fs_lockShared(context, &core->modifyLock, NULL);
    
	/*
	 * Now write quota on flag to disk
	 */
	
	fs_lockExclusive(context, &fset->lock, NULL);
	NFX_SPIN_LOCK(&fset->spinlock);
	fset->incoreFlag &= ~(FS_FILESET_QUOTA_TURN_ON|FS_FILESET_QUOTA_ON_THREAD);
	fset->incoreFlag |= FS_FILESET_QUOTA_ON;
	fset->cachedRecord.fileSetFlags |= FS_FILESET_QUOTA_ON;
	NFX_SPIN_UNLOCK(&fset->spinlock);

	/*
	 * Mark it on disk as well
	 */

	offset = fset->fileSetId * sizeof (fs_FileSetDiskRecord_t);
	boff = (uint32)(offset & (FS_BLOCK_SIZE - 1));
	buf = O_GET_EXCL(fsetInfoInode, offset >> FS_BLOCK_SHIFT);

	drec = (fs_FileSetDiskRecord_t *)(buf->u.data + boff);

	drec->fileSetFlags |= FS_FILESET_QUOTA_ON;

	FS_BUF_WRITE(buf, (uint32)((char *)&drec->fileSetFlags -
							    buf->u.data), sizeof (drec->fileSetFlags));
	fs_unlockExclusive(context, &fset->lock);

	if (!fs_inodeWasModified(context, fsetInfoInode)) {
	    fs_inodeReleaseExclusive(context, fsetInfoInode);
    }
    return;


errNoSpace:    
    fs_unlockShared(context, &quota->lock);
    fs_unlockShared(context, &tfset->lock);
    fs_releQuota(quota);
    
    //Release all fsets traverseLock
   	DLL_ITERATE(tfsetContext, &list->fsetList, list) {
        tfset = tfsetContext->fset;
        fs_unlockExclusive(context, &tfset->traverseLock);
   	}
    
	//Reset quota on flags
	fs_lockExclusive(context, &fset->lock, NULL);
	NFX_SPIN_LOCK(&fset->spinlock);
	fset->incoreFlag &= ~(FS_FILESET_QUOTA_TURN_ON|FS_FILESET_QUOTA_ON_THREAD);
	NFX_SPIN_UNLOCK(&fset->spinlock);
	fs_unlockExclusive(context, &fset->lock);
    
   	//Free the quota context structure
    fs_freeQuotaContextList(context, core, list);

    FS_ELOG(context, LOG_ERR, FS_TRACE_REQ,
            ("Quota on failed for fileset %u, no space left",
             fset->fileSetId));
}
EXPORTED void
fs_quotaOff(
	fs_Context					*context,
	fs_FileSystem					*core,
	fs_FileSetIncoreRecord_t	*fset
)
{
	fs_Inode					*fsetInfoInode = NULL;
	fs_Buf						*buf;
	uint32						boff;
	uint64						offset;
	fs_FileSetDiskRecord_t		*drec;

	fsetInfoInode = I_GET_EXCL(0, FS_FILESET_INFO_INODE);

	/*
	 * Clear quota on flag on disk outside of a transaction as
	 * we don't care whether it makes it to disk or not on a crash
	 */

	fs_lockExclusive(context, &fset->lock, NULL);
	NFX_SPIN_LOCK(&fset->spinlock);
	fset->incoreFlag &= ~(FS_FILESET_QUOTA_TURN_ON|FS_FILESET_QUOTA_ON_THREAD);
	fset->incoreFlag &= ~FS_FILESET_QUOTA_ON;
	NFX_SPIN_UNLOCK(&fset->spinlock);


	/*
	 * Mark it on disk as well
	 */

	offset = fset->fileSetId * sizeof (fs_FileSetDiskRecord_t);
	boff = (uint32)(offset & (FS_BLOCK_SIZE - 1));
	
	buf = O_GET_EXCL(fsetInfoInode, offset >> FS_BLOCK_SHIFT);

	drec = (fs_FileSetDiskRecord_t *)(buf->u.data + boff);

	drec->fileSetFlags &= ~FS_FILESET_QUOTA_ON;

	FS_BUF_WRITE(buf, (uint32)((char *)&drec->fileSetFlags -
							    buf->u.data), sizeof (drec->fileSetFlags));
	fs_unlockExclusive(context, &fset->lock);

	if (!fs_inodeWasModified(context, fsetInfoInode)) {
		fs_inodeReleaseExclusive(context, fsetInfoInode);
	}
}

/*
 * Report the fileset quota and consumption records. This is a simple
 * routine which will walk through the incore lists of filesets and
 * return values. Our assumption that all the fileset records are always
 * incore is the basis for this implementation. If we decide to not keep
 * all the filesets incore, then we will have to revisit this routine
 *
 * The caller is expected to provide the starting position in
 * fileset file and number of entries to return.
 */

EXPORTED int32
fs_reportFileSetQuota(
	fs_Context				*context,
	fs_FileSystem				*core,
	fs_MsgReportQuotaReq_t	*req,
	fs_MsgReportQuotaResp_t	*resp
)
{

	/*
	 * It is assumed that the caller has already allocated memory for
	 * the required amount of entries in response structure
	 */

	fs_Inode					*fsetInfoInode;
	int							id = req->startIndex;
	fs_FileSetIncoreRecord_t	*fset;
	fs_QuotaBlk_t				*qblk;
	fs_FileSetDiskRecord_t		*dreq;

	/*
	 * Assert that we are not holding any locks
	 */
	FS_ASSERT(context != NULL && core != NULL && req != NULL && resp != NULL);

	/*
	 * Lock the fileset info inode in shared mode, so that we cannot
	 * create or remove filesets while this operation is in progress,
	 */

	fsetInfoInode = I_GET_SHARED(0, FS_FILESET_INFO_INODE);

	/*
	 * Also hold the file system's filesetlistLock.
	 */

	NFX_SPIN_LOCK(&core->filesetListLock);
	
	/*
	 * Iterate through all the filesets and copy quota and consumption
	 * data into the response
	 */

	qblk = resp->responses;
	qblk->flag = 0;
	resp->numberResponses = 0;
	DLL_ITERATE(fset, &core->filesetList, fsetList) {
		if (fset == NULL || fset == DLL_NIL) {
				break;
		}
		if (fset->fileSetId >= id) {

			if (fset->cachedRecord.rootInode < FS_ROOT_INODE) {
					continue;
			}

			fs_holdFileSet(context, core, fset);

			/*
		 	 * Copy the record
		 	 */
			
			qblk->id = fset->fileSetId;
            qblk->rootInode = fset->cachedRecord.rootInode;
			
			/*
			 * Protect against quota reservations
		 	 */

			dreq = &fset->cachedRecord;
			NFX_SPIN_LOCK(&fset->spinlock);
			if (fset->incoreFlag & FS_FILESET_QUOTA_ON) {
				qblk->flag |= FS_QUOTA_ON;
			}
            if (fset->incoreFlag & FS_FILESET_QUOTA_ON_THREAD)  {
				qblk->flag |= FS_QUOTA_TURN_ON;
			}
            if (dreq->fileSetFlags & FS_FILESET_USE_QDEFAULTS)  {
                qblk->flag |= FS_QUOTA_USE_DEFAULTS;
            }
			qblk->inodeSoftLimit = dreq->fileSetQuota.inodeSoftLimit;
			qblk->inodeHardLimit = dreq->fileSetQuota.inodeHardLimit;
			qblk->inodeTimeLimit = dreq->fileSetQuota.inodeTimeLimit;
			qblk->blockSoftLimit = dreq->fileSetQuota.blockSoftLimit;
			qblk->blockHardLimit = dreq->fileSetQuota.blockHardLimit;
			qblk->blockTimeLimit = dreq->fileSetQuota.blockTimeLimit;

			/*
			 * Now copy the consumption records
			 */
			if (fset->incoreFlag & FS_FILESET_QUOTA_ON) {
				qblk->inodesConsumed = dreq->fileSetConsumption.inodes;
				qblk->blocksConsumed = dreq->fileSetConsumption.blocks;
				if (dreq->fileSetConsumption.timeExtraInodes == 0)
					qblk->inodeSoftLimitTime = 0;
				else
					qblk->inodeSoftLimitTime = fs_getTimeOfDaySecs()
									- dreq->fileSetConsumption.timeExtraInodes;
				if (dreq->fileSetConsumption.timeExtraBlocks == 0)
					qblk->blockSoftLimitTime = 0;
				else	
					qblk->blockSoftLimitTime = fs_getTimeOfDaySecs()
		   						- dreq->fileSetConsumption.timeExtraBlocks;
			} else {
				qblk->inodesConsumed = ~0;
				qblk->blocksConsumed = ~0;
				qblk->inodeSoftLimitTime = ~0;
				qblk->blockSoftLimitTime = ~0;
			}
			NFX_SPIN_UNLOCK(&fset->spinlock);
			fs_releFileSet(context, core, fset);

			resp->numberResponses++;
			qblk++;
			qblk->flag = 0;
		}
		if (resp->numberResponses >= req->numberEntries) {
				break;
		}
	}

	context->respSize = sizeof (fs_MsgReportQuotaResp_t) +
        				(sizeof (fs_QuotaBlk_t) * (resp->numberResponses - 1));
    resp->msgHdr.len = context->respSize;

	NFX_SPIN_UNLOCK(&core->filesetListLock);
	fs_inodeRelease(context, fsetInfoInode);
	return FS_ERR_NONE;
}

/*
 * Report user or group quota for a given fileset
 */

EXPORTED int32
fs_reportUGQuota(
	fs_Context		*context,
	fs_FileSystem		*core,
	fs_MsgReportQuotaReq_t	*req,
	fs_MsgReportQuotaResp_t	*resp
)
{
#if 0
TOBEUPDATED
	fs_FileSetIncoreRecord_t		*fset;
	fs_Inode						*quotaInode;
	uint64							inum;
	uint64							quotaFileSize;
	uint64							nQuota;
	uint64							i = 0;
	uint32							count = 0;
	fs_QuotaIncoreRecord_t			*quota;
	fs_QuotaDiskRecord_t			*rec = NULL;
	fs_QuotaBlk_t					*qblk;
	fs_Status						status;
	fs_Inode						*root;
	int								flag;

	FS_ASSERT(context != NULL && core != NULL && req != NULL && resp != NULL);
	status = fs_findFset(context, core, req->path, &root, &fset);
	if (status != FS_ERR_NONE) {
		return status;
	}

	/*
	 * We are already holding the root inode, so this fileset cannot
	 * be removed. We don't want to protect against, set quota and quota on
	 * as report quota is advisory and as soon as we report it, the values
	 * can change, so we will not lock fset for the operation
	 */

	flag = req->flag & (FS_QUOTA_USER | FS_QUOTA_GROUP);
	NFX_SPIN_LOCK(&fset->spinlock);
	if (fset->incoreFlag & FS_FILESET_QUOTA_ON) {
		flag |= FS_QUOTA_ON;
       	NFX_SPIN_UNLOCK(&fset->spinlock);
	} else {
       	NFX_SPIN_UNLOCK(&fset->spinlock);
        goto quotaNotOn;
    }
	if (req->flag & FS_QUOTA_USER) {
		inum = fset->cachedRecord.userQuotaInode;
	} else {
		inum = fset->cachedRecord.groupQuotaInode;
	}

	quotaInode = I_GET_SHARED(0, inum);
	quotaFileSize = quotaInode->dinode.size;
	fs_inodeRelease(context, quotaInode);
	nQuota = quotaFileSize / sizeof (fs_QuotaDiskRecordPadded);    

	/*
	 * Fetch each individual quota and return its current state. We
	 * could have gone through the disk and read in values, that would
	 * have required us to freeze the filesystem and prevent new
	 * operations while quotas are being reported.
	 */
	i = req->startIndex;
	qblk = &resp->responses[0];
	while (i < nQuota && count < req->numberEntries) {
			
		fs_yieldPeriodic(context);
    	quota = fs_getUGQuota(  context,
                                core,
                                fset,
    						    req->flag & (FS_QUOTA_USER | FS_QUOTA_GROUP),
                                i,
                                TRUE);
        if (quota == NULL)  {
             //quota block does not exist, jump to next one
             i += FS_QUOTA_RECS_PER_PAGE;
             goto skipBlock;
        }
		rec = &quota->cachedRecords;
		NFX_SPIN_LOCK(&quota->spinlock);
        if (rec->consumption.inodes == 0 && rec->consumption.blocks == 0) {
             NFX_SPIN_UNLOCK(&quota->spinlock);
             goto skipEntry;
        }
                
    	qblk->inodeSoftLimit = rec->quota.inodeSoftLimit;
		qblk->inodeHardLimit = rec->quota.inodeHardLimit;
		qblk->inodeTimeLimit = rec->quota.inodeTimeLimit;
		qblk->blockSoftLimit = rec->quota.blockSoftLimit;
		qblk->blockHardLimit = rec->quota.blockHardLimit;
		qblk->blockTimeLimit = rec->quota.blockTimeLimit;

		/*
		 * Now copy the consumption records
		 */
		if (flag & FS_QUOTA_ON) {
			qblk->inodesConsumed = rec->consumption.inodes;
			qblk->blocksConsumed = rec->consumption.blocks;
			if (rec->consumption.timeExtraInodes == 0)			
				qblk->inodeSoftLimitTime = 0;
			else		
				qblk->inodeSoftLimitTime = fs_getTimeOfDaySecs()
										- rec->consumption.timeExtraInodes;
			if (rec->consumption.timeExtraBlocks == 0)
				qblk->blockSoftLimitTime = 0;
			else 
				qblk->blockSoftLimitTime = fs_getTimeOfDaySecs()
										- rec->consumption.timeExtraBlocks;
		}
    	qblk->flag = flag;
        if (quota->cachedRecords.flags & FS_UG_DEFAULT_QUOTALIMITS) {
            qblk->flag |= FS_QUOTA_USE_DEFAULTS;
        }
		NFX_SPIN_UNLOCK(&quota->spinlock);
	    qblk->id = i;
		qblk++;

		count++;
skipEntry:            
        i++;
		fs_releQuota(quota);
skipBlock:            

		/*
		 * Revalidate the size of the quota file
		 */

		quotaInode = I_GET_SHARED(0, inum);
		quotaFileSize = quotaInode->dinode.size;
		fs_inodeRelease(context, quotaInode);
		nQuota = quotaFileSize / sizeof (fs_QuotaDiskRecordPadded);
    }
    
quotaNotOn:
	resp->numberResponses = count;
    if (resp->numberResponses == 0) {
        context->respSize = sizeof (fs_MsgReportQuotaResp_t) 
                            - sizeof (fs_QuotaBlk_t);
    } else {
    	context->respSize = sizeof (fs_MsgReportQuotaResp_t) +
           				(sizeof (fs_QuotaBlk_t) * (resp->numberResponses - 1));
    }
    resp->msgHdr.len = context->respSize;

	fs_inodeRelease(context, root);
#endif
	return FS_ERR_NONE;
}

/*
 * This routine goes through each and every item on the context list
 * and reduces quota reservation by the specified amount. The routine
 * assumes that the trimming value is less than the actual reservation
 * It is caller's responsibility to ensure that.
 */

EXPORTED void
fs_trimQuotaReservation(
	fs_Context				*context,
	fs_FileSystem			*core,
	fs_QuotaContextList_t	*contextList,
	uint64					amount)
{
	fs_FsetContext_t			*fsetContext;
	fs_QuotaContext_t			*quotaContext;
	fs_FileSetIncoreRecord_t	*fset;
	fs_QuotaIncoreRecord_t		*quota;

	FS_ASSERT(context != NULL && core != NULL && contextList != NULL);
	FS_ASSERT(amount <= contextList->availCount);

	contextList->availCount -= amount;

	/*
	 * Iterate through all the lists
	 */

	DLL_ITERATE(fsetContext, &contextList->fsetList, list) {
		if (fsetContext == NULL || fsetContext == DLL_NIL) {
			break;
		}
		fset = fsetContext->fset;
		fs_lockShared(context, &fset->lock, NULL);
		NFX_SPIN_LOCK(&fset->spinlock);
		if (contextList->flag & FS_COUNT_BLOCKS) {
				fset->cachedRecord.fileSetConsumption.blocks -= amount;
		} else {
				fset->cachedRecord.fileSetConsumption.inodes -= amount;
		}
		NFX_SPIN_UNLOCK(&fset->spinlock);
		fs_unlockShared(context, &fset->lock);
		
	}

	DLL_ITERATE(quotaContext, &contextList->uQuotaList, list) {
		if (quotaContext == NULL || quotaContext == DLL_NIL) {
			break;
		}
		quota = quotaContext->quota;
		fs_lockShared(context, &quota->lock, NULL);
		NFX_SPIN_LOCK(&quota->spinlock);
		if (contextList->flag & FS_COUNT_BLOCKS) {
				quota->cachedRecords.consumption.blocks -= amount;
		} else {
				quota->cachedRecords.consumption.inodes -= amount;
		}
		NFX_SPIN_UNLOCK(&quota->spinlock);
		fs_unlockShared(context, &quota->lock);
		
	}
	DLL_ITERATE(quotaContext, &contextList->gQuotaList, list) {
		if (quotaContext == NULL || quotaContext == DLL_NIL) {
			break;
		}
		quota = quotaContext->quota;
		fs_lockShared(context, &quota->lock, NULL);
		NFX_SPIN_LOCK(&quota->spinlock);
		if (contextList->flag & FS_COUNT_BLOCKS) {
				quota->cachedRecords.consumption.blocks -= amount;
		} else {
				quota->cachedRecords.consumption.inodes -= amount;
		}
		NFX_SPIN_UNLOCK(&quota->spinlock);
		fs_unlockShared(context, &quota->lock);
		
	}
}

/*
 * Allocate space for quota record and initialize it if required.
 * This is called out of commitContext only
 */

PRIVATE fs_Buf *
fs_allocateQuotaBlock(
    fs_Context                  *context,
    fs_FileSystem               *fs,
    fs_FileSetIncoreRecord_t    *fset,
    fs_Inode                    *inode,
    uint64                      offset,
    int                         type
)
{
    fs_Buf                      *buf;
#if 0
TOBEUPDATED
	fs_QuotaRecord_t		    *defRecord;
    fs_QuotaDiskRecordPadded    *dquota;
    fs_QuotaDiskRecord_t        *dqr;
    int                         i;
    fs_Block                    block;
    fs_Extent                   extent;

	if (type & FS_QUOTA_USER) {
		defRecord = &fset->cachedRecord.defaultUserQuota;
	} else {
		defRecord = &fset->cachedRecord.defaultGroupQuota;
	}

	/*
	 * We rely on the fact that for a hole, we would be returned
	 * a null buffer rather than a zeroed buffer
	 */

	buf = O_GET_EXCL(inode, offset >> FS_BLOCK_SHIFT);

	if (buf == NULL) {

		
		buf = O_CREATE(inode, offset >> FS_BLOCK_SHIFT);
		/*
		 * Initialize the buffer.data
		 */

		bzero(buf->u.data, FS_BLOCK_SIZE);

        FS_BUF_ZERO_ALL(buf);

		/*
		 * Copy default quotas from the fileset in to
		 * the new buffer
		 */

		dquota = buf->u.quota->recs;
		for (i = 0; i < FS_QUOTA_RECS_PER_PAGE; i++) {
            dqr = (fs_QuotaDiskRecord_t *)dquota;
			bcopy(defRecord, &dqr->quota, sizeof (fs_QuotaRecord_t));
            //remember that this quota is using the default limits
            dqr->flags = FS_UG_DEFAULT_QUOTALIMITS;
			dquota++;
		}

        FS_BUF_WRITE(buf, 0, FS_BLOCK_SIZE);

		/*
		 * We also allocated an extent in the process. Add it
		 * to the inode.
		 */

		block = buf->block;
		extent = fs_makeExtent(1, block);
        if ((offset + FS_BLOCK_SIZE) > inode->dinode.size) {
            inode->dinode.size = (offset + FS_BLOCK_SIZE);
            fs_inodeWrite(context, inode, INO_WRITE_LOG);
        }
	}
#endif
    return buf;
}

/*
 * Apply the context list to the transaction once we are all done and
 * ready to commit the transaction. We are currently assuming that
 * transaction log is big enough to handle logging of all the fsets
 * and quotas from the given fset to its ancestors. In the long run
 * though we want to log the intent of applying quotas to all the
 * relevant fsets and quotas.
 *
 * We can support upto 4096 fsets and in the worst case this would
 * be 4096 * 24 bytes worth of log. 
 *
 * This dictates that the locking order of buffers is
 * fset buffer --> user quota buffer --> group quota --> buffer
 *
 * If other commitContextList follow, we need to keep all the inodes that have
 * dirty buffers. As those buffers are not released, getting their inodes a
 * second time in this thread would break the lock order.
 */

EXPORTED void
fs_commitContextList(
	fs_Context				*context,
	fs_FileSystem			*core,
	fs_QuotaContextList_t	*contextList,
	boolean					otherCommitFollow)
{
	fs_FsetContext_t			*fsetContext;
	fs_QuotaContext_t			*quotaContext;
	fs_FileSetIncoreRecord_t	*fset;
	fs_QuotaIncoreRecord_t		*quota;
	fs_Inode					*inode;
	fs_Inode					*fsetInfoInode;
	fs_Buf						*buf;
	uint64						offset;
	uint32						boff;
	fs_FileSetDiskRecord_t		*dfset;
	fs_QuotaDiskRecord_t		*dquot;
	fs_ConsumptionRecord_t		*creq;
    fs_Inode                    *newInode = contextList->newInode;

	FS_ASSERT(context != NULL && core != NULL && contextList != NULL);
    FS_ASSERT(newInode == NULL || fs_lockOwnedExclusive(context,
                                                        &newInode->lock));

	/*
	 * Walk through each of the quota / fset objects and perform
	 * transaction against it
	 */

	fsetInfoInode = I_GET_EXCL(0, FS_FILESET_INFO_INODE);
	DLL_ITERATE(fsetContext, &contextList->fsetList, list) {
		if (fsetContext == NULL || fsetContext == DLL_NIL) {
			break;
		}
		fset = fsetContext->fset;
		offset = fset->fileSetId * sizeof (fs_FileSetDiskRecord_t);
		boff = (uint32)(offset & (FS_BLOCK_SIZE - 1));

        //Deadlock with GetQuotaReservation 09/04/2002
		//fs_lockExclusive(context, &fset->lock, NULL);
		fs_lockShared(context, &fset->lock, NULL);
		buf = O_GET_EXCL(fsetInfoInode, offset >> FS_BLOCK_SHIFT);
		dfset = (fs_FileSetDiskRecord_t *)(buf->u.data + boff);
		creq = &dfset->fileSetConsumption;

        /*
         * If the quota is already on, account for the resources. If
         * quota is being turned on, then if the inode is already accounted
         * for by quota on, then count the resources else let quota on
         * handle it
         */

        if (contextList->flag & FS_COUNT_BLOCKS) {
            FS_ASSERT(newInode != NULL);
    		NFX_SPIN_LOCK(&fset->spinlock);
            if (fset->incoreFlag & FS_FILESET_QUOTA_ON ||
                (fset->incoreFlag & FS_FILESET_QUOTA_TURN_ON &&
                fset->inodeSeen >= newInode->inum)) {
                NFX_SPIN_UNLOCK(&fset->spinlock);
                creq->blocks += (contextList->availCount * 
                                 ((contextList->flag & FS_FREE_RESOURCES) ?
                                    -1 : 1));
                FS_BUF_WRITE(buf, ((char *)&creq->blocks
								- buf->u.data), sizeof (creq->blocks));
            } else {
                NFX_SPIN_UNLOCK(&fset->spinlock);
            }
        } else {

            /*
             * This either inode allocation or free. Note that new inode
             * is overloaded and free routines should mark which inode
             * is being free by pointing new inode to it
             */

            FS_ASSERT(newInode != NULL);
    		NFX_SPIN_LOCK(&fset->spinlock);
            if (fset->incoreFlag & FS_FILESET_QUOTA_ON ||
                (fset->incoreFlag & FS_FILESET_QUOTA_TURN_ON &&
                fset->inodeSeen >= newInode->inum)) {
                NFX_SPIN_UNLOCK(&fset->spinlock);
                creq->inodes += (contextList->availCount * 
                                 ((contextList->flag & FS_FREE_RESOURCES) ?
                                    -1 : 1));
                FS_BUF_WRITE(buf, ((char *)&creq->inodes
								- buf->u.data), sizeof (creq->inodes));
            } else {
                NFX_SPIN_UNLOCK(&fset->spinlock);
            }
        }
		if (fsetContext->notify & FS_EXCEED_SOFTLIMIT &&
             (!(contextList->flag & FS_FREE_RESOURCES))) {
			//Just exceeded the soft limit, need to set timeExtraBlocks
			if (contextList->flag & FS_COUNT_BLOCKS) {
        		NFX_SPIN_LOCK(&fset->spinlock);
				creq->timeExtraBlocks =
					fset->cachedRecord.fileSetConsumption.timeExtraBlocks;
                NFX_SPIN_UNLOCK(&fset->spinlock);
				FS_BUF_WRITE(buf, ((char *)&creq->timeExtraBlocks
								- buf->u.data),
							sizeof (creq->timeExtraBlocks));
			} else {
        		NFX_SPIN_LOCK(&fset->spinlock);
				creq->timeExtraInodes =
					fset->cachedRecord.fileSetConsumption.timeExtraInodes;
                NFX_SPIN_UNLOCK(&fset->spinlock);
				FS_BUF_WRITE(buf, ((char *)&creq->timeExtraInodes
								- buf->u.data),
							sizeof (creq->timeExtraInodes));
			}
		}
		if (!fs_bufWasModified(context, buf))
			fs_bufReleaseExclusive(context, buf);
		else {
			if (otherCommitFollow)	{
				//We need to dirty the inode so that it is not released
				fsetInfoInode->dinode.mtime = fs_getTimeOfDay();
				fs_inodeWrite(context, fsetInfoInode, INO_WRITE_LOG);
			}
		}
        //Deadlock with GetQuotaReservation 09/04/2002
		//fs_unlockExclusive(context, &fset->lock);
		fs_unlockShared(context, &fset->lock);
	}

#if 0
TOBEUPDATED
	/*
	 * Process user quotas
	 */ 
	DLL_ITERATE(quotaContext, &contextList->uQuotaList, list) {
		if (quotaContext == NULL || quotaContext == DLL_NIL) {
			break;
		}
		quota = quotaContext->quota;
		fset = quotaContext->fset;
		fs_lockShared(context, &fset->lock, NULL);
        //Deadlock with GetQuotaReservation 09/16/2002
		//fs_lockExclusive(context, &quota->lock, NULL);
		fs_lockShared(context, &quota->lock, NULL);

		inode = I_GET_EXCL(0, fset->cachedRecord.userQuotaInode);
		offset = contextList->uid * sizeof (fs_QuotaDiskRecordPadded);
		boff = (uint32)(offset & (FS_BLOCK_SIZE - 1));

        /*
         * If there is a hole in the quota file, allocate blocks
         * to it and initialize with default value for the fset
         */

        buf = fs_allocateQuotaBlock(context, core, fset, inode, offset,
									FS_QUOTA_USER);
		dquot = (fs_QuotaDiskRecord_t *)
                &buf->u.quota->recs[boff>>FS_QUOTA_REC_SHIFT];
		creq = &dquot->consumption;
        /*
         * If the quota is already on, account for the resources. If
         * quota is being turned on, then if the inode is already accounted
         * for by quota on, then count the resources else let quota on
         * handle it
         */

        if (contextList->flag & FS_COUNT_BLOCKS) {
            FS_ASSERT(newInode != NULL);
        	NFX_SPIN_LOCK(&fset->spinlock);
            if (fset->incoreFlag & FS_FILESET_QUOTA_ON ||
                (fset->incoreFlag & FS_FILESET_QUOTA_TURN_ON &&
                fset->inodeSeen >= newInode->inum)) {
                NFX_SPIN_UNLOCK(&fset->spinlock);
                creq->blocks += (contextList->availCount * 
                                 ((contextList->flag & FS_FREE_RESOURCES) ?
                                    -1 : 1));
                FS_BUF_WRITE(buf, ((char *)&creq->blocks
								- buf->u.data), sizeof (creq->blocks));
            } else {
                NFX_SPIN_UNLOCK(&fset->spinlock);
            }
        } else {

            /*
             * This either inode allocation or free. Note that new inode
             * is overloaded and free routines should mark which inode
             * is being free by pointing new inode to it
             */

            FS_ASSERT(newInode != NULL);
        	NFX_SPIN_LOCK(&fset->spinlock);
            if (fset->incoreFlag & FS_FILESET_QUOTA_ON ||
                (fset->incoreFlag & FS_FILESET_QUOTA_TURN_ON &&
                fset->inodeSeen >= newInode->inum)) {
                NFX_SPIN_UNLOCK(&fset->spinlock);
                creq->inodes += (contextList->availCount * 
                                 ((contextList->flag & FS_FREE_RESOURCES) ?
                                    -1 : 1));
                FS_BUF_WRITE(buf, ((char *)&creq->inodes
								- buf->u.data), sizeof (creq->inodes));
            } else {
                NFX_SPIN_UNLOCK(&fset->spinlock);
            }
        }
		if (quotaContext->notify & FS_EXCEED_SOFTLIMIT &&
            (!(contextList->flag & FS_FREE_RESOURCES))) {
			//Just exceeded the soft limit, need to set timeExtraBlocks
			if (contextList->flag & FS_COUNT_BLOCKS) {
        		NFX_SPIN_LOCK(&quota->spinlock);
				creq->timeExtraBlocks =
					quota->cachedRecords.consumption.timeExtraBlocks;
                NFX_SPIN_UNLOCK(&quota->spinlock);
				FS_BUF_WRITE(buf, ((char *)&creq->timeExtraBlocks
								- buf->u.data),
							sizeof (creq->timeExtraBlocks));
			} else {
        		NFX_SPIN_LOCK(&quota->spinlock);
				creq->timeExtraInodes =
					quota->cachedRecords.consumption.timeExtraInodes;
                NFX_SPIN_UNLOCK(&quota->spinlock);
				FS_BUF_WRITE(buf, ((char *)&creq->timeExtraInodes
								- buf->u.data),
							sizeof (creq->timeExtraInodes));
			}
		}
		if (!fs_bufWasModified(context, buf))
			fs_bufReleaseExclusive(context, buf);
		else {
			if (otherCommitFollow)	{
				//We need to dirty the inode so that it is not released
				inode->dinode.mtime = fs_getTimeOfDay();
				fs_inodeWrite(context, inode, INO_WRITE_LOG);
			}
		}
        if (!fs_inodeWasModified(context, inode)) {
            fs_inodeReleaseExclusive(context, inode);
        }
        NFX_SPIN_LOCK(&quota->spinlock);
        quota->flag &= ~FS_QUOTA_INCORE_NEEDS_INIT;
        NFX_SPIN_UNLOCK(&quota->spinlock);
        //Deadlock with GetQuotaReservation 09/16/2002
		//fs_unlockExclusive(context, &quota->lock);
		fs_unlockShared(context, &quota->lock);
		fs_unlockShared(context, &fset->lock);
	}

	
	/*
	 * Process group quotas
	 */ 
	DLL_ITERATE(quotaContext, &contextList->gQuotaList, list) {
		if (quotaContext == NULL || quotaContext == DLL_NIL) {
			break;
		}
		quota = quotaContext->quota;
		fset = quotaContext->fset;

        FS_ASSERT(fset != NULL);

		fs_lockShared(context, &fset->lock, NULL);
        //Deadlock with GetQuotaReservation 09/16/2002
		//fs_lockExclusive(context, &quota->lock, NULL);
		fs_lockShared(context, &quota->lock, NULL);

		inode = I_GET_EXCL(0, fset->cachedRecord.groupQuotaInode);
		offset = contextList->gid * sizeof (fs_QuotaDiskRecordPadded);
		boff = (uint32)(offset & (FS_BLOCK_SIZE - 1));

        /*
         * If there is a hole in the quota file, allocate blocks
         * to it and initialize with default value for the fset
         */

        buf = fs_allocateQuotaBlock(context, core, fset, inode, offset,
                                    FS_QUOTA_GROUP);
		dquot = (fs_QuotaDiskRecord_t *)
                &buf->u.quota->recs[boff >> FS_QUOTA_REC_SHIFT];
		creq = &dquot->consumption;
        /*
         * If the quota is already on, account for the resources. If
         * quota is being turned on, then if the inode is already accounted
         * for by quota on, then count the resources else let quota on
         * handle it
         */

        if (contextList->flag & FS_COUNT_BLOCKS) {
            FS_ASSERT(newInode != NULL);
    		NFX_SPIN_LOCK(&fset->spinlock);
            if (fset->incoreFlag & FS_FILESET_QUOTA_ON ||
                (fset->incoreFlag & FS_FILESET_QUOTA_TURN_ON &&
                fset->inodeSeen >= newInode->inum)) {
                NFX_SPIN_UNLOCK(&fset->spinlock);
                creq->blocks += (contextList->availCount * 
                                 ((contextList->flag & FS_FREE_RESOURCES) ?
                                    -1 : 1));
                FS_BUF_WRITE(buf, ((char *)&creq->blocks
								- buf->u.data), sizeof (creq->blocks));
            } else {
                NFX_SPIN_UNLOCK(&fset->spinlock);
            }
        } else {

            /*
             * This either inode allocation or free. Note that new inode
             * is overloaded and free routines should mark which inode
             * is being free by pointing new inode to it
             */

            FS_ASSERT(newInode != NULL);
    		NFX_SPIN_LOCK(&fset->spinlock);
            if (fset->incoreFlag & FS_FILESET_QUOTA_ON ||
                (fset->incoreFlag & FS_FILESET_QUOTA_TURN_ON &&
                fset->inodeSeen >= newInode->inum)) {
                NFX_SPIN_UNLOCK(&fset->spinlock);
                creq->inodes += (contextList->availCount * 
                                 ((contextList->flag & FS_FREE_RESOURCES) ?
                                    -1 : 1));
                FS_BUF_WRITE(buf, ((char *)&creq->inodes
								- buf->u.data), sizeof (creq->inodes));
            } else {
                NFX_SPIN_UNLOCK(&fset->spinlock);
            }
        }
		if (quotaContext->notify & FS_EXCEED_SOFTLIMIT &&
            (!(contextList->flag & FS_FREE_RESOURCES))) {
			//Just exceeded the soft limit, need to set timeExtraBlocks
			if (contextList->flag & FS_COUNT_BLOCKS) {
        		NFX_SPIN_LOCK(&quota->spinlock);
				creq->timeExtraBlocks =
					quota->cachedRecords.consumption.timeExtraBlocks;
                NFX_SPIN_UNLOCK(&quota->spinlock);
				FS_BUF_WRITE(buf, ((char *)&creq->timeExtraBlocks
								- buf->u.data),
							sizeof (creq->timeExtraBlocks));
			} else {
        		NFX_SPIN_LOCK(&quota->spinlock);
				creq->timeExtraInodes =
					quota->cachedRecords.consumption.timeExtraInodes;
                NFX_SPIN_UNLOCK(&quota->spinlock);
				FS_BUF_WRITE(buf, ((char *)&creq->timeExtraInodes
								- buf->u.data),
							sizeof (creq->timeExtraInodes));
			}
		}
		if (!fs_bufWasModified(context, buf))
			fs_bufReleaseExclusive(context, buf);
		else {
			if (otherCommitFollow)	{
				//We need to dirty the inode so that it is not released
				inode->dinode.mtime = fs_getTimeOfDay();
				fs_inodeWrite(context, inode, INO_WRITE_LOG);
			}
		}
        if (!fs_inodeWasModified(context, inode)) {
            fs_inodeReleaseExclusive(context, inode);
        }
        NFX_SPIN_LOCK(&quota->spinlock);
        quota->flag &= ~FS_QUOTA_INCORE_NEEDS_INIT;
        NFX_SPIN_UNLOCK(&quota->spinlock);
        //Deadlock with GetQuotaReservation 09/16/2002
		//fs_unlockExclusive(context, &quota->lock);
		fs_unlockShared(context, &quota->lock);
		fs_unlockShared(context, &fset->lock);
	}
#endif
		
	if (!fs_inodeWasModified(context, fsetInfoInode)) {
		fs_inodeReleaseExclusive(context, fsetInfoInode);
	}
}

EXPORTED fs_QuotaDiskRecord_t *
fs_getUGSnappedQuota(
	fs_Context					*context,
	fs_FileSystem			    *core,
    uint64                      snapnum,
	fs_FileSetDiskRecord_t	    *dfset,	// fileset we are working on
	int							type,	// whether user or group quota
	uint32						id		// uid or gid
)
{

	fs_QuotaDiskRecord_t	*quot = NULL, *dquot = NULL;
#if 0
TOBEUPDATED
	uint64					quotaInum = 0;
	fs_Inode				*quotaInode = NULL;
	uint64					recordOffset;
	fs_Buf					*buf;
	int						broff;		//record offset in buffer

	/*
	 * Assert that we have correct arguments. 
	 */

	FS_ASSERT(context != NULL && core != NULL && dfset != NULL);

	/*
	 * XXX - We are currently not validating id. Should do when we
	 * have arrived at maximum id implementation
	 */

	FS_ASSERT(core != NULL && dfset != NULL &&
		  (((type &
		    (FS_QUOTA_USER | FS_QUOTA_GROUP)) == FS_QUOTA_USER) ||
		  ((type &
		    (FS_QUOTA_USER | FS_QUOTA_GROUP)) == FS_QUOTA_GROUP)));

	quot = (fs_QuotaDiskRecord_t *)fs_ramAlloc(FS_RAM_QUOTA,
                                        sizeof (fs_QuotaDiskRecord_t));
    FS_ASSERT(quot != NULL);

	/*
	 * Read the quota information from the quota file. If
	 * quota does not exist, then we will have to allocate one
	 */

	if (type & FS_QUOTA_USER) {
		quotaInum = dfset->userQuotaInode;
	} else {
		quotaInum = dfset->groupQuotaInode;
	}

	recordOffset = id * sizeof (fs_QuotaDiskRecordPadded);
	broff = (recordOffset & (FS_BLOCK_SIZE-1)) >> FS_QUOTA_REC_SHIFT;

	quotaInode = I_GET_SHARED(snapnum, quotaInum);

	FS_ASSERT(quotaInode != NULL);

	/*
	 * assert that quotaInode is held
	 */

	FS_ASSERT(fs_lockHeldShare(context, &quotaInode->lock) != 0);
	
	/*
	 * Get the buffer for the quota record
	 */

	FS_ASSERT(quotaInode->inum == quotaInum);
	buf = O_GET_SHARED(quotaInode, recordOffset >> FS_BLOCK_SHIFT);

	/*
	 * Locking hierarchy says that quota file inode lock is
	 * below quota block lock
	 */

	/*
	 * If there is a hole in the quotas file, then return default
	 * user quotas if allocation for the same is not requested
	 */

	if (buf != NULL) {

		/*
	 	 * Copy the data from the buffer into the quota structure
		 * and return
		 */

        dquot = (fs_QuotaDiskRecord_t *)&buf->u.quota->recs[broff];

        bcopy((char *)dquot, quot, sizeof (fs_QuotaDiskRecord_t));

		fs_bufRelease(context, buf);

	} else  {
        bzero(quot, sizeof (fs_QuotaDiskRecord_t));
	}
    fs_inodeRelease(context, quotaInode);		
#endif
	return quot;
}

#ifndef FS_STANDALONE
#include "exports/event-api.h"


PRIVATE void
sendEventdMsg(
    fs_Context  *context,
    char        *reason,
	uint32		fileSetId,
    uint32      quotaType,
    uint32      id,
	int			notify)
{
    nfx_eventMsg_t msg;
    char    *volName = context->fileSystem->super->mountName;

    bzero(&msg, sizeof (nfx_eventMsg_t));
    msg.msgType     = NFX_EVENT_POST;
    msg.needAck     = FALSE;
    msg.eventScope  = NFX_EVENT_LOCAL;
    msg.eventType   = NFX_EVENT_QUOTA;
    strcpy(msg.reason, reason);
    msg.param.quotaEvent.volId     = context->volId;
    msg.param.quotaEvent.fileSetId = fileSetId;
    msg.param.quotaEvent.quotaType = quotaType;
    msg.param.quotaEvent.id        = id;
    msg.param.quotaEvent.notify    = notify;
    strcpy(msg.param.quotaEvent.volName, volName);
    event_postEvent(&msg);
}

#define VOLNAMESHORT_SIZE   20

PRIVATE void
notifyAdmin(fs_Context				*context,
            fs_QuotaContextList_t   *quotaContextList)
{
	fs_QuotaContext_t		*quotaContext;
	fs_FsetContext_t		*fsetContext;
    char                    elogstr[256];

	DLL_ITERATE(quotaContext, &quotaContextList->uQuotaList, list) {
		if (quotaContext->notify) {
                
            sendEventdMsg(context,
                          "User Quota Expiry",
                          quotaContext->fset->fileSetId, 
                          FS_QUOTA_USER,
                          quotaContextList->uid, 
                          quotaContext->notify);
            
            sprintf(elogstr, 
                   "User Quota Expiry volId %llx fileset %u uid %u: ",
                   context->volId.whole_thing, quotaContext->fset->fileSetId, 
                   quotaContextList->uid);
            if (quotaContext->notify & FS_EXCEED_BLOCK) {
                strcat(elogstr, "size exceeded ");
            } else {
                strcat(elogstr, "files exceeded ");
            }

            if (quotaContext->notify & FS_EXCEED_HARDLIMIT) {
                strcat(elogstr, "Hardlimit:");
            }
            if (quotaContext->notify & FS_EXCEED_SOFTLIMIT) {
                strcat(elogstr, "Softlimit:");
            }
            if (quotaContext->notify & FS_EXCEED_TIMELIMIT) {
                strcat(elogstr, "Timelimit:");
            }
            
            FS_ELOG(context, LOG_WARNING, FS_TRACE_REQ, ("%s", elogstr));
		}
	}
	DLL_ITERATE(quotaContext, &quotaContextList->gQuotaList, list) {
		if (quotaContext->notify) {
                
            sendEventdMsg(context,
                          "Group Quota Expiry",
                          quotaContext->fset->fileSetId, 
                          FS_QUOTA_GROUP,
                          quotaContextList->gid, 
                          quotaContext->notify);
            
            sprintf(elogstr, 
                   "Group Quota Expiry volId %llx fileset %u gid %u: ",
                   context->volId.whole_thing, quotaContext->fset->fileSetId,
                   quotaContextList->gid);
            if (quotaContext->notify & FS_EXCEED_BLOCK) {
                strcat(elogstr, "size exceeded ");
            } else {
                strcat(elogstr, "files exceeded ");
            }

            if (quotaContext->notify & FS_EXCEED_HARDLIMIT) {
                strcat(elogstr, "Hardlimit:");
            }
            if (quotaContext->notify & FS_EXCEED_SOFTLIMIT) {
                strcat(elogstr, "Softlimit:");
            }
            if (quotaContext->notify & FS_EXCEED_TIMELIMIT) {
                strcat(elogstr, "Timelimit:");
            }

            FS_ELOG(context, LOG_WARNING, FS_TRACE_REQ, ("%s", elogstr));
		}
	}
	DLL_ITERATE(fsetContext, &quotaContextList->fsetList, list) {
		if (fsetContext->notify) {

            sendEventdMsg(context, 
                          "Fileset Quota Expiry",
                          fsetContext->fset->fileSetId, 
                          FS_QUOTA_AGGREGATE,
                          0,
                          fsetContext->notify);
            
            sprintf(elogstr, "Fileset Quota Expiry volId %llx fileset %u: ",
                    context->volId.whole_thing , fsetContext->fset->fileSetId);
            if (fsetContext->notify & FS_EXCEED_BLOCK) {
                strcat(elogstr, "size exceeded ");
            } else {
                strcat(elogstr, "files exceeded ");
            }

            if (fsetContext->notify & FS_EXCEED_HARDLIMIT) {
                strcat(elogstr, "Hardlimit:");
            }
            if (fsetContext->notify & FS_EXCEED_SOFTLIMIT) {
                strcat(elogstr, "Softlimit:");
            }
            if (fsetContext->notify & FS_EXCEED_TIMELIMIT) {
                strcat(elogstr, "Timelimit:");
            }

            FS_ELOG(context, LOG_WARNING, FS_TRACE_REQ, ("%s", elogstr));
		}
	}
}

#else
PRIVATE void
notifyAdmin(fs_Context				*context,
            fs_QuotaContextList_t   *quotaContextList)
{
	fs_QuotaContext_t		*quotaContext;
	fs_FsetContext_t		*fsetContext;
    char                    *volName = context->fileSystem->super->mountName;

    char    buf[512];
	DLL_ITERATE(quotaContext, &quotaContextList->uQuotaList, list) {
		if (quotaContext->notify) {

            bzero(buf, sizeof (buf));
            sprintf(buf, 
                   "User Quota Expiry: vol %s : fileset %d : uid %d :",
                   volName, quotaContext->fset->fileSetId,
                   quotaContextList->uid);
            if (quotaContext->notify & FS_EXCEED_BLOCK) {
                strcat(buf, "size ");
            } else {
                strcat(buf, "number of files ");
            }

            if (quotaContext->notify & FS_EXCEED_HARDLIMIT) {
                strcat(buf, "exceeded Hardlimit :");
            }
            if (quotaContext->notify & FS_EXCEED_SOFTLIMIT) {
                strcat(buf, "exceeded Softlimit :");
            }
            if (quotaContext->notify & FS_EXCEED_TIMELIMIT) {
                strcat(buf, "exceeded Timelimit :");
            }
            FS_ELOG(context, LOG_WARNING, FS_TRACE_REQ, ("%s", buf));
		}
	}
	DLL_ITERATE(quotaContext, &quotaContextList->gQuotaList, list) {
		if (quotaContext->notify) {
            bzero(buf, sizeof (buf));
            sprintf(buf, 
                   "Group Quota Expiry: vol %s : fileset %d : gid %d :",
                   volName, quotaContext->fset->fileSetId,
                   quotaContextList->gid);
            if (quotaContext->notify & FS_EXCEED_BLOCK) {
                strcat(buf, "size ");
            } else {
                strcat(buf, "number of files ");
            }

            if (quotaContext->notify & FS_EXCEED_HARDLIMIT) {
                strcat(buf, "exceeded Hardlimit :");
            }
            if (quotaContext->notify & FS_EXCEED_SOFTLIMIT) {
                strcat(buf, "exceeded Softlimit :");
            }
            if (quotaContext->notify & FS_EXCEED_TIMELIMIT) {
                strcat(buf, "exceeded Timelimit :");
            }
            FS_ELOG(context, LOG_WARNING, FS_TRACE_REQ, ("%s", buf));
		}
	}
	DLL_ITERATE(fsetContext, &quotaContextList->fsetList, list) {
		if (fsetContext->notify) {
            bzero(buf, sizeof (buf));
            sprintf(buf, "Fileset Quota Expiry: vol %s : fileset %d : ",
                    volName, fsetContext->fset->fileSetId);
            if (fsetContext->notify & FS_EXCEED_BLOCK) {
                strcat(buf, "size ");
            } else {
                strcat(buf, "number of files ");
            }

            if (fsetContext->notify & FS_EXCEED_HARDLIMIT) {
                strcat(buf, "exceeded Hardlimit :");
            }
            if (fsetContext->notify & FS_EXCEED_SOFTLIMIT) {
                strcat(buf, "exceeded Softlimit :");
            }
            if (fsetContext->notify & FS_EXCEED_TIMELIMIT) {
                strcat(buf, "exceeded Timelimit :");
            }
            FS_ELOG(context, LOG_WARNING, FS_TRACE_REQ, ("%s", buf));
		}
	}
}
#endif

/*
 * Zap on disk consumption records for a given quota inode
 */

PRIVATE void
fs_zapConsumptionInfo(
    fs_Context                  *context,
    fs_FileSetIncoreRecord_t    *fset,
    int                         type
)
{
#if 0
TOBEUPDATED
    fs_Inode        *quotaInode;
    fs_Inumber      quotaInum;
    uint64          curOffset = 0;
    uint64          nbufs = 0;
    fs_Buf          *buf;
	fs_QuotaDiskRecordPadded	*dquota;
    int             i;

	if (type & FS_QUOTA_USER) {
		quotaInum = fset->cachedRecord.userQuotaInode;
	} else {
		quotaInum = fset->cachedRecord.groupQuotaInode;
	}
    quotaInode = I_GET_EXCL(0, quotaInum);
    FS_ASSERT(FS_I_IS_QUOT(quotaInode->dinode.attrs));
    FS_ASSERT(fs_lockOwnedExclusive(context, &quotaInode->lock));

    FS_TRACE(context, FS_TRACE_REQ, 
             ("Quota On - Zapping quota inode (s=%llu)\n", 
              quotaInode->dinode.size));
        
    while (curOffset < quotaInode->dinode.size) {
		fs_yieldPeriodic(context);
        buf = O_GET_EXCL(quotaInode, curOffset >> FS_BLOCK_SHIFT);
        if (buf == NULL) {
            curOffset += FS_BLOCK_SIZE;
            continue;
        }
        dquota = buf->u.quota->recs;
        for (i = 0; i < FS_QUOTA_RECS_PER_PAGE; i++) {
            bzero((char *)(((fs_QuotaDiskRecord_t *)&dquota)->consumption),
                  sizeof (fs_ConsumptionRecord_t));
            dquota++;
        }
        FS_BUF_WRITE_NO_LOG(buf, 0, FS_BLOCK_SIZE);
        fs_bufReleaseExclusive(context, buf);
        curOffset += FS_BLOCK_SIZE;
        nbufs++;
        
        if (!(nbufs % 1024))    {
            FS_TRACE(context, FS_TRACE_REQ, 
             ("Quota On - nbufs=%llu (offset=%llu)\n", nbufs, curOffset));
            //Make sure buffers are written out to disk
            fs_bufSyncInodeBufs(context, quotaInode);
        
            FS_ASSERT(!fs_inodeWasModified(context, quotaInode));
            fs_inodeReleaseExclusive(context, quotaInode);    
            //Allow processes scanning the inodes (like other quota on threads)
            //to have a look to this one

            //Drop the modify lock for any thread wishing to get it exclusive
            fs_unlockShared(context, &context->fileSystem->modifyLock);
            if (fs_growIfNeeded(context)
             || !fs_haveMininodesAvail(context->fileSystem))
            {
                fs_sleep (context, 0);
            } else {
                fs_yieldPeriodic(context);
            }
            //Get it again
            fs_lockShared(context, &context->fileSystem->modifyLock, NULL);        
            quotaInode = I_GET_EXCL(0, quotaInum);
        }
        
    }

    //Make sure buffers are written out to disk
    fs_bufSyncInodeBufs(context, quotaInode);
                    
    FS_ASSERT(!fs_inodeWasModified(context, quotaInode));
    fs_inodeReleaseExclusive(context, quotaInode);
    
#endif
    return;
}

/*
 * invalidate all the incore quota records for a given quota type and a given
 * fileset
 */

PRIVATE void
fs_invalidateIncoreQuota(
    fs_Context              *context,
    fs_FileSetIncoreRecord_t *fset,
    int                   type
)
{
    fs_FileSystem           *fs = context->fileSystem;
	fs_QuotaIncoreRecord_t	*quota;
	nfx_spinlock_t		*listlock;
    int                 hash;
    int                 count = 0;

    FS_TRACE(context, FS_TRACE_REQ, ("in invalidateIncoreQuota"));
	listlock = &fs_quotaCache.spin;

	if (type & FS_QUOTA_USER) {
        NFX_SPIN_LOCK(listlock);
        for (hash = 0; hash <  FS_QCACHE_HASH_MAX; hash++) {
            DLL_ITERATE(quota, &fs_quotaCache.userQuotaHashList[hash], 
                        hashList) {
                count++;
                if (quota == NULL || quota == DLL_NIL) {
                    break;
                }
                if (quota->fileSetId != fset->fileSetId
                 || quota->volId.whole_thing != fs->volId.whole_thing)  { 
                    continue;
                }
                fs_holdQuota(quota);
                NFX_SPIN_LOCK(&quota->spinlock);
                quota->flag |= FS_QUOTA_INCORE_NEEDS_INIT;
                NFX_SPIN_UNLOCK(&quota->spinlock);
                fs_releQuota(quota);
            }
#ifndef FS_STANDALONE
#ifdef USE_WATCHDOG
            if (count < fs_quotaCache.quotaCacheLimit)   {
                watchdog_kick();
            }
#endif // USE_WATCHDOG
#endif // FS_STANDALONE
        }
        NFX_SPIN_UNLOCK(listlock);
	} else {
        NFX_SPIN_LOCK(listlock);
        for (hash = 0; hash <  FS_QCACHE_HASH_MAX; hash++) {
			DLL_ITERATE(quota, &fs_quotaCache.groupQuotaHashList[hash],
                        hashList) {
                count++;
				if (quota == NULL || quota == DLL_NIL) {
					break;
				}
                if (quota->fileSetId != fset->fileSetId
                 || quota->volId.whole_thing != fs->volId.whole_thing)  { 
                    continue;
                }
                fs_holdQuota(quota);
                NFX_SPIN_LOCK(&quota->spinlock);
                quota->flag |= FS_QUOTA_INCORE_NEEDS_INIT;
                NFX_SPIN_UNLOCK(&quota->spinlock);
                fs_releQuota(quota);
			}
#ifndef FS_STANDALONE
#ifdef USE_WATCHDOG
            if (count < fs_quotaCache.quotaCacheLimit)   {
                watchdog_kick();
            }
#endif // USE_WATCHDOG
#endif // FS_STANDALONE
        }
        NFX_SPIN_UNLOCK(listlock);
	}
    FS_TRACE(context, FS_TRACE_REQ, ("out invalidateIncoreQuota"));
}

/*
 * Zap fileset consumption information
 */

PRIVATE void
fs_zapFilesetConsumptionInfo(
    fs_Context                  *context,
    fs_FileSetIncoreRecord_t    *fset
)
{
	/*
	 * local variables
	 */

	fs_Buf					*buf;
	int						boff;
	fs_FileSetDiskRecord_t	*dfset;
	fs_QuotaRecord_t		*qrec;
	fs_Inode				*fsetInfoInode;

    FS_TRACE(context, FS_TRACE_REQ, ("in zapFilesetConsumptionInfo"));
	/*
	 * Assert that arguments are correct
	 */

	boff = (uint32)((fset->fileSetId * sizeof(fs_FileSetDiskRecord_t)) &
					 (FS_BLOCK_SIZE - 1));

	/*
	 * Get the fileset info inode and the associated buffer
	 */

	fsetInfoInode = I_GET_EXCL(0, FS_FILESET_INFO_INODE);
	fs_lockExclusive(context, &fset->lock, NULL);

	FS_ASSERT(fsetInfoInode != NULL);

	/*
	 * Assert that fset Info inode is held
	 */

	FS_ASSERT(fs_lockOwnedExclusive(context, &fsetInfoInode->lock));

	/*
	 * Get the associated buffer for write
	 */

	buf = O_GET_EXCL(fsetInfoInode, (fset->fileSetId *
			  sizeof (fs_FileSetDiskRecord_t)) >> FS_BLOCK_SHIFT);

	FS_ASSERT(buf != NULL);

	dfset = (fs_FileSetDiskRecord_t *)(buf->u.data + boff);

    qrec = &dfset->fileSetQuota;
    bzero(&dfset->fileSetConsumption, sizeof (fs_ConsumptionRecord_t));

    FS_BUF_WRITE_NO_LOG(buf, 0, FS_BLOCK_SIZE);
    fs_bufReleaseExclusive(context, buf);

    /*
     *Zero the incore consumption records as well
     */
    bzero(&fset->cachedRecord.fileSetConsumption,
          sizeof (fs_ConsumptionRecord_t));
	fs_unlockExclusive(context, &fset->lock);
    
    //Make sure the buffer is written out to disk
    fs_bufSyncInodeBufs(context, fsetInfoInode);
                    
    FS_ASSERT(!fs_inodeWasModified(context, fsetInfoInode));
    fs_inodeReleaseExclusive(context, fsetInfoInode);
#if 0    
    NOW THAT COW IS A NESTED TRANSACTION, THIS IS NOT NECESSARY
    REMOVE WHEN TESTED
	if (!fs_inodeWasModified(context, fsetInfoInode)) {
		fs_inodeReleaseExclusive(context, fsetInfoInode);
	}
#endif    
    FS_TRACE(context, FS_TRACE_REQ, ("out zapFilesetConsumptionInfo"));
	return;
}




EXPORTED void
fs_QuotaAdjustmentNoCheck(
	fs_Context	*context,	
	fs_FileSystem	*core,		
	fs_Inode	*inode,		// Inode changed
	fs_Inode	*newInode,	// New Inode created, if count inodes
	int64		count,	// resources (if + : required, if - : to be released)
	int			flag,	// count inodes or blocks
	uint32		uid,
	uint32		gid,
	fs_QuotaContextList_t   **list	
)
{
	fs_QuotaContextList_t		*quotaContextList = NULL;
	fs_FileSetIncoreRecord_t	*tfset;
	fs_FsetContext_t			*fsetContext;
	fs_QuotaContext_t			*quotaContext;
	fs_QuotaIncoreRecord_t		*quota;
	fs_Inode					*fsetInfoInode;


	FS_ASSERT(count != 0 && context != NULL && core != NULL &&
		  inode != NULL);
	FS_ASSERT(flag == FS_COUNT_INODES || flag == FS_COUNT_BLOCKS);

	// Build up the list of fileset structures that we want to operate on

	fsetInfoInode = I_GET_SHARED(0, FS_FILESET_INFO_INODE);

	/*
	 * Now that we have blocked out creation of any new filesets
	 * the parent chains are stable
	 */

	tfset = inode->fset = fs_getFset(context, core, inode);

	while (tfset != NULL) {

		/*
		 * If the fset does not have quotas turned on or
		 * the allocation version is older than quota turn
		 * on allocation version, skip this fset
		 */

		NFX_SPIN_LOCK(&tfset->spinlock);
		if ((tfset->incoreFlag & (FS_FILESET_QUOTA_ON |
					  FS_FILESET_QUOTA_TURN_ON)) == 0) {
			NFX_SPIN_UNLOCK(&tfset->spinlock);
			tfset = tfset->parent;
			continue;
		}


		/*
		 * Quota on has not yet seen this inode. Don't bother accounting
		 * for its consumption as quota on will process it when it gets
		 * there
	 	 */

		if ((tfset->incoreFlag & FS_FILESET_QUOTA_TURN_ON) &&
			inode->inum > tfset->inodeSeen) {
			NFX_SPIN_UNLOCK(&tfset->spinlock);
			tfset = tfset->parent;
			continue;
		}
		NFX_SPIN_UNLOCK(&tfset->spinlock);
			
        if (quotaContextList == NULL)   {
            //There is at least one fileset with quota on in the path, allocate
            //and initialize the quota context list
        	quotaContextList = (fs_QuotaContextList_t *)
                                fs_ramAlloc(FS_RAM_QUOTA_CONTEXT_LIST,
                                			sizeof (fs_QuotaContextList_t));
            quotaContextList->newInode = NULL;

            /*
             * We are overloading newInode flag to also contain inode for which
             * blocks are allocated. We can do this since a quotaContextList can
             * only support either block allocation or inode allocation
             */

            if (flag == FS_COUNT_BLOCKS) {
                quotaContextList->newInode = inode;
            }

        	DLL_INIT(&quotaContextList->fsetList);
        	DLL_INIT(&quotaContextList->uQuotaList);
        	DLL_INIT(&quotaContextList->gQuotaList);
        	quotaContextList->uid = uid;
        	quotaContextList->gid = gid;
        	quotaContextList->flag = flag;
        	/*
        	 * Point to this list from the context structure, so that
        	 * we can get back our hands on this list during write completion
        	 * to perform appropriate logging and transactions
        	 */
        	*list = quotaContextList;
        }

		fsetContext = (fs_FsetContext_t *)fs_ramAlloc(
				FS_RAM_FSET_CONTEXT, sizeof (fs_FsetContext_t));
		bzero(fsetContext, sizeof (fs_FsetContext_t));
		fsetContext->fset = tfset;

		/*
		 * Note that we don't hold the fset structure. The
		 * reason is that hold on the inode would not allow
		 * the fset structure to be removed.
		 */

		DLL_INSERT_TAIL(&quotaContextList->fsetList, fsetContext,
				   list);
		tfset = tfset->parent;
	}

	fs_inodeRelease(context, fsetInfoInode);

    if (quotaContextList == NULL)   {
        //No fileset with quota on
        return;
    }
    
	/*
	 * Now walk each of the filesets and get a user and group quota
	 * structure in to the context. Each of the user and group
	 * quota structures are held, so the completion routines would
	 * have to appropriately release them
	 */

	DLL_ITERATE(fsetContext, &quotaContextList->fsetList, list) {
		
		// Get the user quota context
		tfset = fsetContext->fset;

		quota = fs_getUGQuota(context, core, tfset, FS_QUOTA_USER, uid, FALSE);
				
		quotaContext = (fs_QuotaContext_t *)fs_ramAlloc(
			FS_RAM_QUOTA_CONTEXT, sizeof (fs_QuotaContext_t));
		bzero(quotaContext, sizeof (fs_QuotaContext_t));
		quotaContext->quota = quota;
		quotaContext->fset = tfset;
		DLL_INSERT_TAIL(&quotaContextList->uQuotaList, quotaContext,
				   list);

		// Get the group quota context
		quota = fs_getUGQuota(context, core, tfset, FS_QUOTA_GROUP, gid, FALSE);
				
		quotaContext = (fs_QuotaContext_t *)fs_ramAlloc(
			FS_RAM_QUOTA_CONTEXT, sizeof (fs_QuotaContext_t));
		bzero(quotaContext, sizeof (fs_QuotaContext_t));
		quotaContext->quota = quota;
		quotaContext->fset = tfset;
		DLL_INSERT_TAIL(&quotaContextList->gQuotaList, quotaContext,
				   list);
	}

	// At this point we have all the fsets and quotas held but not locked
	
	DLL_ITERATE(quotaContext, &quotaContextList->uQuotaList, list) {
		quota = quotaContext->quota;
		fs_lockExclusive(context, &quota->lock, NULL);
		NFX_SPIN_LOCK(&quota->spinlock);
		if (flag == FS_COUNT_BLOCKS) {
			quota->cachedRecords.consumption.blocks += count;
		} else {
			quota->cachedRecords.consumption.inodes += count;
		}
		NFX_SPIN_UNLOCK(&quota->spinlock);
		fs_unlockExclusive(context, &quota->lock);
	}
	DLL_ITERATE(quotaContext, &quotaContextList->gQuotaList, list) {
		quota = quotaContext->quota;
		fs_lockExclusive(context, &quota->lock, NULL);
		NFX_SPIN_LOCK(&quota->spinlock);
		if (flag == FS_COUNT_BLOCKS) {
			quota->cachedRecords.consumption.blocks += count;
		} else {
			quota->cachedRecords.consumption.inodes += count;
		}
		NFX_SPIN_UNLOCK(&quota->spinlock);
		fs_unlockExclusive(context, &quota->lock);
	}
	DLL_ITERATE(fsetContext, &quotaContextList->fsetList, list) {
		tfset = fsetContext->fset;
		fs_lockExclusive(context, &tfset->lock, NULL);
		NFX_SPIN_LOCK(&tfset->spinlock);
		if (flag == FS_COUNT_BLOCKS) {
			tfset->cachedRecord.fileSetConsumption.blocks += count;
		} else {
			tfset->cachedRecord.fileSetConsumption.inodes += count;
		}
		NFX_SPIN_UNLOCK(&tfset->spinlock);

		fs_unlockExclusive(context, &tfset->lock);
	}
	
	if (count < 0)	{
		quotaContextList->availCount = -count;
		quotaContextList->flag = flag | FS_FREE_RESOURCES;
	} else {
		quotaContextList->availCount = count;
		quotaContextList->flag = flag;
	}
	
	return;
}

EXPORTED void
fs_quotaCacheInit(void)
{
	int32	quotasLimit;
    int i;

    // TODO - include this in incoretune
    // Set very low for stress testing
	quotasLimit = 200;


	fs_quotaCache.quotaCacheLimit = quotasLimit;
	NFX_SPIN_INIT(&fs_quotaCache.spin,SPINLOCK_LEVEL_UNKNOWN);

	/*
	 * Now initialize the quota lists
	 */

	for (i = 0; i < FS_QCACHE_HASH_MAX; i++) {
		DLL_INIT(&fs_quotaCache.userQuotaHashList[i]);
		DLL_INIT(&fs_quotaCache.groupQuotaHashList[i]);
	}

	DLL_INIT(&fs_quotaCache.quotaFreeList);
}

#endif // SSC
